/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 * 
 */
package mot.client.actions;

import mot.client.deligates.NamedObjectDelegate;
import mot.client.swing.common.Observer;
import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

/**
 * This represents an action that is to be taken when a user 
 * clicks or double clicks on an object. The object is define
 * to be within the bounds of this rectangle.
 * 
 * When a panel is being animated it is redrawn many times each 
 * second. It would be inefficient to re-build all the active objects
 * on a panel every time it is redrawn. Therefore the observer pattern 
 * is supported - An active rectangle is an observer, and should receive
 * updates when the observable changes - so it does not need to be rebuilt!
 *
 */
public abstract class ActiveRectangle extends Rectangle implements MotAction, Observer {

	/**
	 * Construct an active rectangle
	 * @param x The centre of the active rectangle
	 * @param y The centre of the active rectangle
	 * @param width The width of the active rectangle
	 * @param height The height of the active rectangle
	 */
    public ActiveRectangle(int x, int y, int dimention) {
    	super(x-dimention/2, y-dimention/2, dimention, dimention);
    }
	
    public ActiveRectangle(int x, int y, int width, int height) {
    	super(x-width/2, y-height/2, width, height);
    }
	
    public ActiveRectangle(int x, int y, int width, int height, boolean useTopLeftIsTrue) {
    	super(x, y, width, height);
    }
	
    public ActiveRectangle(Location loc, int size) {
    	super(loc, size);
    }
    
    public void setPosition(int x, int y) {
    	setLeft(x-getWidth()/2);
    	setTop(y-getHeight()/2);
    }
	
    public void setPosition(Location l) {
    	setLeft(l.getX()-getWidth()/2);
    	setTop(l.getY()-getHeight()/2);
    }
	
	/**
	 * @param o The object that may be represented by this rectangle
	 * @return True if this rectangle represents the object o and false otherwise.
	 */
	public abstract boolean contains(NamedObjectDelegate o);

	/**
	 * @param o The name of an object that may be represented by this rectangle
	 * @return True if this rectangle represents an object with name o and false otherwise.
	 */
	public abstract boolean contains(String name);

	/**
	 * @return The object contains in this rectangle or null.
	 */
	public abstract NamedObjectDelegate getContents();
	
	/**
	 * If multiple objects share the same location then which one would be displayed
	 * can be determined by specifying one as a default.
	 * @return True if this object is the default one.
	 */
	public boolean isDefault() { return false; }
	
	/**
	 *  
	 * @param dx change in x
	 * @param dy change in y
	 * @throws ApplicationError 
	 */
	public void mouseDragged(int dx, int dy) throws ApplicationError { }
	public boolean isDraggable() { return false; }

	/**
	 *  
	 * @param dx change in x
	 * @param dy change in y
	 * @throws ApplicationError 
	 * @throws ValidationError 
	 */
	public void mouseClicked(int x, int y) throws ApplicationError, ValidationError { }


}
