/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.actions;

import java.awt.Container;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.window.galaxy.GalaxyMapDestination;
import mot.client.swing.window.galaxy.GalaxyMapMoo;
import mot.client.swing.window.galaxy.GalaxyMapMooList;
import mot.client.swing.window.galaxy.GalaxyMapMooListItem;
import mot.client.swing.window.galaxy.GalaxyMapMooScrollBar;
import mot.client.swing.window.galaxy.GalaxyMapPlanet;
import mot.client.swing.window.galaxy.GalaxyMapStellarObject;
import mot.client.swing.window.galaxy.GalaxyWindowListener;
import mot.client.swing.window.galaxy.actions.GalaxyMapButton;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftTransportColonists;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftTransportTroops;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCreateTradeRoute;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonRemoveWaypoint;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonSetWaypoint;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.common.vo.OrbitVO;
import mot.common.vo.TransportVO;

/**
 * This represents a list actions that should be searched when a user clicks
 * a mouse button. Each action is defined by a rectangle and implements a task 
 * to be performed when the user clicks within the rectangle.
 *  
 * You must supply an implementation of a selection policy so than that when
 * multiple objects are located at the point where the user clicked a single
 * item may still be selected.
 * 
 * This class initialises the selection policy manager to select the first one
 * in the list again when the user clicks on a new location to select a different 
 * object. This class assumes the user has selected a new location if the last item 
 * selected is not in the list of objects at this location!
 *  
 */
public class ActiveRectangleList {
	
	private MultipleMatchSelectionPolicy selectionPolicy;
	
	private ArrayList arList = new ArrayList();
	
    /**
     * This is a list of MOOs in orbit around a planet displayed when a user clicks on 
     * a GalaxyMapMoo. The user can then select one of the MOOs at this locations and issue commands.
     */
    private GalaxyMapMooList mooList;
	
	/**
	 * The list of buttons to be rendered beside the selected item in the list.
	 */
	private GalaxyMapButton [] buttons;

	public ActiveRectangleList(MultipleMatchSelectionPolicy selectionPolicy) {
		this.selectionPolicy = selectionPolicy;
	}
	
	public void removeAll() { arList.clear(); }
	public int getSize() { return arList.size(); }

	public void add(ActiveRectangle ar) {
		if (ar.isDefault()) {
			arList.add(0, ar); 
		} else {
			arList.add(ar); 
		}
	}

	/**
	 * @param o The object that may be represented by one of these rectangles on the screen
	 * @return True if this list contains object o and false otherwise.
	 */
	public boolean contains(NamedObjectDelegate o) {
		return _contains(arList, o);
	}
	
	private boolean _contains(List list, NamedObjectDelegate o) {
		Iterator i = list.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar.contains(o)) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @param o The object that may be represented by a rectangle in this list
	 * @return The ActiveRectangle that represents the space on the screen
	 * occupied by object o, if one exists. 
	 */
	public ActiveRectangle get(NamedObjectDelegate o) {
		Iterator i = arList.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar.contains(o)) {
				return ar;
			}
		}
		return null;
	}
	
	/**
	 * @param o The name of an object that may be represented by a rectangle in this list
	 * @return The ActiveRectangle that represents the space on the screen
	 * occupied by object with name, if one exists. 
	 */
	public ActiveRectangle get(String name) {
		Iterator i = arList.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar.contains(name)) {
				return ar;
			}
		}
		return null;
	}
	
	/**
	 * @param o The object that may be represented by a rectangle in this list
	 * @return All the ActiveRectangle objects in this list. 
	 */
	public Iterator getAll() {
		return arList.iterator();
	}
	
	/**
	 * This is the last rectangle that was selected from this list.
	 * This is used to determine if the selection policy should be
	 * reinitialised from the start of the list of candidates or
	 * the next one should be selected.
	 */
	private List lastCandidates;
	
	/**
	 * Used to determine if the user has clicked at the same point as last time.
	 * If this returns false then the selection policy should be initialised to 
	 * the start of the list again.
	 * @param candidates The list of objects located at the point where the user clicked.
	 * @return True if and only if the user has clicked at the same point as last 
	 * time.
	 */
	private boolean clickedAtSamePoint(List candidates) {
		if (lastCandidates == null || lastCandidates.size() <= 1) {
			return false;
		}
		int qtyMissed = 0;
		Iterator i = candidates.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (!_contains(lastCandidates, ar.getContents())) {
				qtyMissed++;
			}
		}
		return (qtyMissed <= 1);
	}
	
	public ActiveRectangle getItemAt(int x, int y) throws ApplicationError {
		if (mooList != null) {
			GalaxyMapMooListItem[] mooListContent = mooList.getContent();
			if (mooListContent != null) {
				for (int i = 0; i < mooListContent.length; i++) {
					if (mooListContent[i].contains(x,y)) {
						return mooListContent[i];
					}
				}
			}
			GalaxyMapMooScrollBar sb = mooList.getScrollBar();
			if (sb != null && sb.contains(x, y)) {
				return sb;
			}
		}
		GalaxyMapButton ar = getButtonAt(x, y);
		if (ar != null) {
			return ar;
		}
		return null;
	}
	
	public ActiveRectangle getObjectItemAt(int x, int y) throws ApplicationError, ValidationError {

		if (arList == null || arList.size() == 0) {
			return null;
		}
		
		Iterator i = arList.iterator();
		ArrayList candidates = new ArrayList();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar.contains(x,y)) {
				candidates.add(ar);
			}
		}
		
		if (candidates.size() > 1) {
			if (!clickedAtSamePoint(candidates)) {
				selectionPolicy.moveToStart();
			}
			ActiveRectangle ar = (ActiveRectangle) selectionPolicy.selectFrom(candidates);
			lastCandidates = candidates;
			return ar;
		} else if (candidates.size() == 1) {
			ActiveRectangle ar = (ActiveRectangle) candidates.get(0);
			lastCandidates = candidates;
			return ar;
		}
		
		return null;
	}

	/**
	 * Perform a click operation on the object located at x,y. If multiple objects exist at 
	 * this location use the selection policy to determine which one to use.
	 * @see ActiveRectangle for details.
	 * @param x The location on the map
	 * @param y The location on the map
	 * @throws ApplicationError
	 * @throws ValidationError
	 */
	public ActiveRectangle click(int x, int y) throws ApplicationError, ValidationError {
		ActiveRectangle ar = getItemAt(x,y);
		if (ar != null) {
			ar.mouseClicked(x,y);
			return ar;
		}
		//mooList = null;
		ar = getObjectItemAt(x, y);
		if (ar != null) {
			ar.click();
		}
		return ar;
	}

	/**
	 * Perform a double click operation on the object located at x,y. If multiple objects exist at 
	 * this location use the selection policy to determine which one to use.
	 * @see ActiveRectangle for details.
	 * @param x The location on the map
	 * @param y The location on the map
	 * @throws ApplicationError
	 * @throws ValidationError
	 */
	public ActiveRectangle doubleClick(int x, int y) throws ApplicationError, ValidationError {
		ActiveRectangle ar = getItemAt(x,y);
		if (ar != null) {
			ar.doubleClick();
			return ar;
		}
		ar = getObjectItemAt(x, y);
		if (ar != null) {
			ar.doubleClick();
		}
		return ar;
	}

	/**
	 * @param x The location on the map in screen coordinates
	 * @param y The location on the map in screen coordinates
	 * @return The stellar object at location x,y in the galaxy map or null if there is not one.
	 */
	public GalaxyMapStellarObject getStellarObjectAt(int x, int y) {
		if (arList == null || arList.size() == 0) {
			return null;
		}
		Iterator i = arList.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar instanceof GalaxyMapStellarObject && ar.contains(x,y)) {
				return (GalaxyMapStellarObject) ar;
			}
		}
		return null;
	}
	
	/**
	 * @param x The location on the map in screen coordinates
	 * @param y The location on the map in screen coordinates
	 * @return The stellar object or planet at location x,y in the galaxy map or null if there is not one.
	 */
	public GalaxyMapDestination getPlanetOrStellarObjectAt(int x, int y) {
		if (arList == null || arList.size() == 0) {
			return null;
		}
		Iterator i = arList.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar instanceof GalaxyMapDestination && ar.contains(x,y)) {
				return (GalaxyMapDestination) ar;
			}
		}
		return null;
	}

	public GalaxyMapMoo getMoo(MoveableOwnedObjectDelegate selection) {
		if (selection == null || arList == null || arList.size() == 0) {
			return null;
		}
		Iterator i = arList.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar instanceof GalaxyMapMoo && ar.contains(selection.getName())) {
				return (GalaxyMapMoo) ar;
			}
		}
		return null;
	}

	public GalaxyMapPlanet getPlanet(PlanetDelegate selection) {
		if (selection == null || arList == null || arList.size() == 0) {
			return null;
		}
		Iterator i = arList.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar instanceof GalaxyMapPlanet && ar.contains(selection.getName())) {
				return (GalaxyMapPlanet) ar;
			}
		}
		return null;
	}

	public GalaxyMapStellarObject getStellarObject(StellarObjectDelegate selection) {
		if (selection == null || arList == null || arList.size() == 0) {
			return null;
		}
		Iterator i = arList.iterator();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar instanceof GalaxyMapStellarObject && ar.contains(selection.getName())) {
				return (GalaxyMapStellarObject) ar;
			}
		}
		return null;
	}

	public GalaxyMapMoo [] getMoosAt(int x, int y) {
		if (arList == null || arList.size() == 0) {
			return null;
		}
		Iterator i = arList.iterator();
		ArrayList<GalaxyMapMoo> r = null;
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar instanceof GalaxyMapMoo && ar.contains(x,y)) {
				if (r == null) r = new ArrayList<GalaxyMapMoo>();
				r.add((GalaxyMapMoo) ar);
			}
		}
		if (r == null) {
			return null;
		} else {
			return (GalaxyMapMoo[]) r.toArray(new GalaxyMapMoo[r.size()]);
		}
	}

	public GalaxyMapMoo [] getAllMoos() {
		if (arList == null || arList.size() == 0) {
			return null;
		} else {
			int size = arList.size();
			ArrayList candidates = null;
			for (int i = 0; i < size; i++) {
				ActiveRectangle ar = (ActiveRectangle) arList.get(i);
				if (ar instanceof GalaxyMapMoo) {
					if (candidates == null) candidates = new ArrayList();
					candidates.add(ar);
				}
			}
			if (candidates == null || candidates.size() == 0) {
				return null;
			} else {
				return (GalaxyMapMoo[]) candidates.toArray(new GalaxyMapMoo[candidates.size()]);
			}
		}
	}

	public GalaxyMapMoo [] getAllMoos(PlayerDelegate owner, boolean mustBeInOrbit) throws ApplicationError {
		if (arList == null || arList.size() == 0) {
			return null;
		}
		Iterator i = arList.iterator();
		ArrayList candidates = null;
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar instanceof GalaxyMapMoo && ((GalaxyMapMoo)ar).getMostVisibleMoo().isOwnedBy(owner)) {
				if (!mustBeInOrbit || ((GalaxyMapMoo)ar).getMostVisibleMoo().getOrbit() != null) {					
					if (candidates == null) candidates = new ArrayList();
					candidates.add(ar);
				}
			}
		}
		if (candidates == null || candidates.size() == 0) {
			return null;
		} else {
			return (GalaxyMapMoo[]) candidates.toArray(new GalaxyMapMoo [candidates.size()]);
		}
	}

	public GalaxyMapMoo getNextMoo(MoveableOwnedObjectDelegate selection) throws ApplicationError {
		if (selection == null || arList == null || arList.size() == 0) {
			return null;
		}
		PlayerDelegate selectionOwner = DeligateCacheManager.getPlayer(selection.getOwner());
		GalaxyMapMoo [] moos = getAllMoos(selectionOwner, true);
		for (int i = 0; i < moos.length; i++) {
			if (moos[i].contains(selection.getName())) {
				if (i == moos.length-1) {
					return moos[0];
				} else {
					return moos[i+1];
				}
			}
		}
		return null;
	}
	
	public GalaxyMapMoo getPreviousMoo(MoveableOwnedObjectDelegate selection) throws ApplicationError {
		if (selection == null || arList == null || arList.size() == 0) {
			return null;
		}
		GalaxyMapMoo previous = null;
		PlayerDelegate selectionOwner = DeligateCacheManager.getPlayer(selection.getOwner());
		GalaxyMapMoo [] moos = getAllMoos(selectionOwner, true);
		for (int i = 0; i < moos.length; i++) {
			if (moos[i].contains(selection.getName())) {
				if (previous == null) {
					return moos[moos.length-1];
				} else {
					return previous;
				}
			}
			previous = moos[i];
		}
		return null;
	}

	public GalaxyMapMooList getMooList() {
		return mooList;
	}

//	public void setMooList(GalaxyMapMooList mooList) {
//		this.mooList = mooList;
//	}
	
	public void createMooList(MoveableOwnedObjectDelegate [] moos, MoveableOwnedObjectDelegate selected, ActiveRectangle selectedAR, int left, int top, int panelWidth, int panelHeight, boolean originIsTop) throws ApplicationError {
		mooList = new GalaxyMapMooList(this, moos, selected, selectedAR, left, top, panelWidth, panelHeight, originIsTop);
	}

	public void createPlanetButtons(PlanetDelegate planet, GalaxyMapPlanet ar, int left, int top) throws ApplicationError {
		mooList = null;

		UserPlayerDelegate owner = MainWindow.getOwner();
		GalaxyWindowListener listener = MainWindow.getInstance().getGalaxyMapListener();
		int qtyFreeTransport = owner.getQtyFreeTransport();
		int qtyColonistsTransportsInOrbit = DeligateCacheManager.getQtyTransportsWithEmptySpaceAt(owner, planet, TransportVO.CONTENT_TYPE_COLONISTS);
		int qtyTroopsTransportsInOrbit = DeligateCacheManager.getQtyTransportsWithEmptySpaceAt(owner, planet, TransportVO.CONTENT_TYPE_TROOPS);
		boolean loadTroops = (planet.isOwnedBy(owner) && !planet.isMiningColony() && planet.getQtyGroundDefense() > 6 && (qtyTroopsTransportsInOrbit > 0 || qtyFreeTransport > 0));
		boolean loadColonists = (planet.isOwnedBy(owner) && !planet.isMiningColony() && planet.getPopulation() > 6 && (qtyColonistsTransportsInOrbit > 0 || qtyFreeTransport > 0)) ;
		
		int direction = ar.getRight() < left ? 1 : -1;
		ArrayList<GalaxyMapButton> list = new ArrayList();
		
		if (!planet.isWayPoint()) {
			GalaxyMapButton button = new GalaxyMapButtonSetWaypoint(listener, left, top);
			list.add(button);
			left += (button.getWidth() + GalaxyMapMooList.BUTTON_GAP) * direction; 
		} else {
			GalaxyMapButton button = new GalaxyMapButtonRemoveWaypoint(listener, left, top);
			list.add(button);
			left += (button.getWidth() + GalaxyMapMooList.BUTTON_GAP) * direction; 
		}
		if (loadTroops) {
			GalaxyMapButton button = new GalaxyMapButtonCraftTransportTroops(planet, listener, left, top);
			list.add(button);
			left += (button.getWidth() + GalaxyMapMooList.BUTTON_GAP) * direction; 
		}
		if (loadColonists) {
			GalaxyMapButton button = new GalaxyMapButtonCraftTransportColonists(planet, listener, left, top);
			list.add(button);
			left += (button.getWidth() + GalaxyMapMooList.BUTTON_GAP) * direction; 
		}

		buttons = list.toArray(new GalaxyMapButton[list.size()]);

	}

	public void createStellarObjectButtons(StellarObjectDelegate so, GalaxyMapStellarObject ar, int left, int top) throws ApplicationError {
		mooList = null;

		UserPlayerDelegate owner = MainWindow.getOwner();
		GalaxyWindowListener listener = MainWindow.getInstance().getGalaxyMapListener();
		boolean createTradeRoute = (so.isOwnedBy(owner) && so instanceof SolarSystemDelegate && ((SolarSystemDelegate)so).canCreateTradeRoute());
		
		int direction = ar.getRight() < left ? 1 : -1;
		ArrayList<GalaxyMapButton> list = new ArrayList();
		
		if (!so.isWayPoint()) {
			GalaxyMapButton button = new GalaxyMapButtonSetWaypoint(listener, left, top);
			list.add(button);
			left += (button.getWidth() + GalaxyMapMooList.BUTTON_GAP) * direction; 
		} else {
			GalaxyMapButton button = new GalaxyMapButtonRemoveWaypoint(listener, left, top);
			list.add(button);
			left += (button.getWidth() + GalaxyMapMooList.BUTTON_GAP) * direction; 
		}
		if (createTradeRoute) {
			GalaxyMapButton button = new GalaxyMapButtonCreateTradeRoute(so, listener, left, top);
			list.add(button);
			left += (button.getWidth() + GalaxyMapMooList.BUTTON_GAP) * direction; 
		}

		if (list.size() > 0) {
			buttons = list.toArray(new GalaxyMapButton[list.size()]);
		} else {
			buttons = null;
		}

	}

	public void clearSelection() {
		this.mooList = null;
		this.buttons = null;
	}

	public void updateMooList(MoveableOwnedObjectDelegate newSelection) throws ApplicationError {
		if (mooList == null) return;
    	GalaxyMapMooListItem[] moos = mooList.getContent();
    	MoveableOwnedObjectDelegate selection = mooList.getSelection();
		if (selection != null) {
			updateMooListAt(newSelection, selection.getLocation(), selection.hasNavigator(), selection.getOrbit());
		} else if (moos != null && moos.length > 0) {
			MoveableOwnedObjectDelegate moo = moos[0].getMoo();
			updateMooListAt(newSelection, moo.getLocation(), moo.hasNavigator(), moo.getOrbit());
		}
	}
	
    private void updateMooListAt(MoveableOwnedObjectDelegate newSelection, Location loc, boolean hasNavigator, OrbitVO orbitDeligate) throws ApplicationError {
    	Rectangle rect = loc.createRectangleAround(2);
    	MoveableOwnedObjectDelegate[] currentMoos =  DeligateCacheManager.getMOOsAt(rect, hasNavigator, orbitDeligate);
		GalaxyWindowListener listener = MainWindow.getExestingInstance().getGalaxyMapListener();
    	if (currentMoos == null || currentMoos.length == 0) {
    		mooList = null;
    		listener.getView().getContextPanel().showEmpireMenu();
    		return;
    	}
    	
    	if (newSelection == null || !Comp.in(newSelection, currentMoos)) {    		
    		MoveableOwnedObjectDelegate oldSelection = mooList.getSelection();
    		if (Comp.find(oldSelection, currentMoos) == null) {
    			newSelection = mooList.getNextSelectionInList(oldSelection);
    		} else {
    			newSelection = oldSelection;
    		}
    	}
		if (newSelection == null) newSelection = currentMoos[0];
		
		mooList.setContent(currentMoos, newSelection);
		listener.getView().getContextPanel().showMoo(newSelection);
    }

	public boolean keyPressed(char key, int modifiers) throws ApplicationError, ValidationError {
		if (buttons != null) {
			for (int i = 0; i < buttons.length; i++) {
				if (buttons[i].keyPressed(key, modifiers)) {
					return true;
				}
			}
		}
		return false;
	}
	
	public GalaxyMapButton [] getButtons() {
		return buttons;
	}
	
	public void setButtons(GalaxyMapButton []  v) {
		buttons = v;
	}
	
	public GalaxyMapButton getButtonAt(int x, int y) {
		if (buttons == null) return null;
		for (int i = 0; i < buttons.length; i++) {
			if (buttons[i].contains(x, y)) {
				return buttons[i];
			}
		}
		return null;
	}

	public void render(Container container, Graphics2D graphics, Rectangle visible) throws ApplicationError {
		if (buttons != null) {
			for (int i = 0; i < buttons.length; i++) {					
				int left = buttons[i].getLeft();
				int top = buttons[i].getTop(); 
				int size = buttons[i].getWidth();
				ImageUtils.drawImage(container, graphics, buttons[i].getPicture(), left, top, size, size);					
			}
		}
	}
	
}
