/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */package mot.client.alarms;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import mot.client.actions.ActiveRectangleList;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.event.EventMailBox;
import mot.client.sounds.Sounds;
import mot.client.swing.window.galaxy.GalaxyMapMoo;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.MessageType;
import mot.common.bean.Settings;
import mot.common.event.MessageEvent;
import mot.common.event.NewBattleEvent;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.vo.NavigatorVO;

public class AlarmSystem {

	private Map<String, AlarmItem> map = new HashMap();
	private boolean endTurnPressed;
	private Long contentRebuiltTime;
	
	private AlarmSystem() {
	}
	
	private static AlarmSystem alarmSystem = null;	
	public static AlarmSystem getInstance() {
		if (alarmSystem== null) {
			alarmSystem = new AlarmSystem();
		}
		return alarmSystem;
	}
	
	public void clear() {
		map.clear();
	}

	public void notifySetupWindowOpened() {
		try {
			String latestVersion = Settings.getInstance().getLatestVersion();
			if (latestVersion != null) {
				checkLatestVersion(latestVersion);
			} else {
				long buildTime = 0;
				File file = new File("mot.jar");
				if (file.exists()) {
					buildTime = file.lastModified();
				} else {
					file = new File("mot.exe");
					if (file.exists()) {
						buildTime = file.lastModified();
					} else {
						buildTime = 1;
					}
				}
				if (buildTime > 0) {
					long timeNow = System.currentTimeMillis();
					long DAY = ((long)1000) * 60 * 60 * 24; 
					if (timeNow > buildTime + 60*DAY) {
						long timeVersionChecked = Settings.getInstance().getTimeVersionChecked();
						if (timeNow > timeVersionChecked + 7*DAY) {
							Logger.logDebug("AlarmSystem.checkForUpdate()");
							new LatestVersionFetcher().start();
						}
					}
				} else {
					Logger.logError("No mot executable found", new ApplicationError());
				}
			}
		} catch (Exception e) {
			Logger.logError("Unable to check version", e);
		}
	}

	public void checkLatestVersion(String latestVersion) throws ApplicationError {
		if (LatestVersionFetcher.isNewerVersion(latestVersion)) {
			MainWindow.getExestingInstance().showMessageActive(L.UPGRADE_NOW, true);
		}
	}
	
	public void notifyEndTurnButtonPressed() throws ApplicationError, ValidationError {
		endTurnPressed = true;
	}

	public void notifyContentRebuilt() throws ApplicationError {
		contentRebuiltTime = System.currentTimeMillis();
	}
	
	public void tick() throws ApplicationError {
		MainWindow mainWin = MainWindow.getInstance();
		if (endTurnPressed && contentRebuiltTime != null && mainWin !=  null && mainWin.isGalaxyMapConfigurationActive()) {
			if (!mainWin.getEventMailBox().hasEventTypeInQueue(NewBattleEvent.class)) {
				ActiveRectangleList arl = mainWin.getGalaxyMapConfiguration().getGalaxyMapPanel().getObjects();
				long timeSinceContentRebuilt = System.currentTimeMillis() - contentRebuiltTime;
				if (arl != null && timeSinceContentRebuilt > 500) {
					contentRebuiltTime = null;
					endTurnPressed = false;
					removeObsoleteAlarms(arl);
					createNewAlarms(arl);
				}
			}
		}
	}
	
	private void createNewAlarms(ActiveRectangleList arl) throws ApplicationError {
		GalaxyMapMoo[] moos = arl.getAllMoos();
		UserPlayerDelegate player = MainWindow.getOwner();
		if (moos != null && player != null) {
			for (int i = 0; i < moos.length; i++) {
				MoveableOwnedObjectDelegate moo = moos[i].getMostVisibleMoo();
				NavigatorVO navigator = moo.getNavigator();
				if (navigator != null && player.isEnemy(moo.getOwner())) {
					String soName = navigator.getDestination().getStellarObject();
					StellarObjectDelegate destination = DeligateCacheManager.getStellarObject(soName);
					if (destination != null && destination.isOwnedBy(player)) {
						if (!hasAlarm(moo)) {
							createAlarm(moo, destination);
						}
					}
				}
			}
		}
	}

	private boolean hasAlarm(MoveableOwnedObjectDelegate moo) {
		return map.containsKey(moo.getName());
	}

	private void createAlarm(MoveableOwnedObjectDelegate moo, StellarObjectDelegate destination) throws ApplicationError {
		EventMailBox eventMailBox = MainWindow.getInstance().getEventMailBox();
		if (eventMailBox != null) {
			MessageEvent event = new MessageEvent("Enemy heading to "+destination.getName(), MessageType.alarm, Sounds.ALARM_SOUND);
			eventMailBox.addEventToQueue(event);
			map.put(moo.getName(), new AlarmItemMoo(moo));
		}
	}

	private void removeObsoleteAlarms(ActiveRectangleList arl) {
		AlarmItem[] aa = getAttackAlarms();
		if (aa != null && arl != null) {
			for (int i = 0; i < aa.length; i++) {	
				NamedObjectDelegate item = aa[i].getItem();
				if (item != null && !arl.contains(item)) {
					map.remove(aa[i].getName());
				}
			}
		}
	}

	private AlarmItem [] getAttackAlarms() {
		Collection<AlarmItem> values = map.values();
		if (values != null && values.size() > 0) {
			return values.toArray(new AlarmItem[values.size()]);
		} else {
			return null;
		}
	}

}
