/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.animation;

import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.sounds.Sound;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.Area;
import mot.common.bean.AreaSrc;
import mot.common.util.ApplicationError;

/**
 * An animation is an event that has occurred in the game engine and have some display 
 * manifestations. For example a weapon fire event, will cause a weapon fire animation.
 * 
 * How animations are displayed is dependent of the animation type. This is the base 
 * class for all animations.
 */
public abstract class BattleAnimation implements AreaSrc {

    /**
     * This is the current place in this animation life.
     * For example an explosion may display a number of bitmaps until it is complete.
     * This is an index from 0 to life-1 of the current bitmap that is being used.
     */
    private int stage;
    
    /**
     * This is the quantity of turns (stages) this animation should remain alive. When 
     * stage reaches life the animation is complete.
     */
    private int life;
    
    /**
     * True if an only if this animation is to be drawn on the screen before the crafts
     * are painted.
     * This is a bit of a hack because it is GUI implementation dependent. But what the heck!
     */
    private boolean after;
    
    /**
     * This is the interval (quantity of ticks) between each stage.
     */
    private int interval;
    
    /**
     * The quantity of ticks remaining to start the next stage.
     * When this is 0 the next tick will start the next stage.
     */
    private int intervalCount;

    /**
     * These are the sounds to be played while this animation is active.
     * This can be used to make sure a sound is only played once. When a
     * sound is played it is moved from the waiting list to the playing 
     * list - so that it is not played again.
     */
    private BattleAnimationSoundList soundsPlaying;
    
    /**
     * A list of sounds waiting to play during this animation. Use
     * if (!anim.hasSoundToPlay()) anim.playSound()
     */
    private BattleAnimationSoundList soundsWaiting;
    
    public static final int INFINITE_LIFE = 99999;

    public BattleAnimation(int life, boolean after, int interval, Sound sound) {
        this.life = (life < 1) ? 1 : life;
        this.stage = 0;
        this.after = after;
        this.interval = interval;
        if (sound != null) {
        	addSoundWaiting(sound, 0);
        }
        
        intervalCount = interval;
    }

    public int getStage() {
        return stage;
    }

    public void setStage(int stage) {
        this.stage = stage;
    }

    public int getLife() {
        return life;
    }

    public void setLife(int life) {
        this.life = life;
    }
    
    public void setInitialDelay(int delay) {
    	intervalCount = interval * delay;
    	life += delay;
    }
    
    
    /**
     * Moves this animation to the next stage.
     * @throws mot.common.util.ApplicationError
     */
    public void tick() throws ApplicationError {
    	//Logger.logDebug("tick "+stage+" on "+toString());
        if (intervalCount == 0) {
            if (stage < life) {
                stage++;
                stageChange();
            }
            intervalCount = interval;
        } else {
            intervalCount--;
        }
    }
    
    /**
     * Called when the stage is changed.
     * This can be over-ridden to carry out some action when the stage changes.
     * @throws ApplicationError 
     */
    protected void stageChange() throws ApplicationError {
	}

	/**
     * @return True if and only if this animation has completed. i.e. stage is equal to life.
     */
    public boolean isComplete() { return stage >= life; }

    /**
     * Usually a sound is complete when the animation is complete.
     * But if this animation has a linked animation then the actual animation may be complete BUT the isComplete()
     * method returns false because it is waiting for the linked animation to complete.
     * @return True iff the sound on this animation must be stopped.
     */
    public boolean soundIsComplete() { return isComplete(); }
    
    public boolean isAfter() {
        return after;
    }

    public void setAfter(boolean after) {
        this.after = after;
    }
    
    /**
     * This is called to paint the animation on the screen when the animation is active.
     * 
     * @param painter The parameter is of type object because this animation class is independent 
     * of any GUI implementation. However, a swing interface would probably be passing a Graphics2D
     * object.
     */
    public abstract void paintActive(Object painter) throws ApplicationError;
    
    /**
     * This is called to paint the animation on the screen when the animation is not active.
     * 
     * An animation is not active when there is an exclusive animation in front of it or when
     * it is an exclusive animation not at the front. Most non active animations do not need to be 
     * drawn, but Missiles need drawing because the missile is deleted before the animations are 
     * executed. Also the state of objects may have changed, so painting the animation while it 
     * is being queued draws the correct state, for this point in time (from the users perspective).
     * 
     * @param painter The parameter is of type object because this animation class is independent 
     * of any GUI implementation. However, a swing interface would probably be passing a Graphics2D
     * object.
     * @param list The current list of animations scheduled for display. If an animation is not active
     * it must make sure that there are no other animation BEFORE this animation in the list that has 
     * vetoed the draw object. If this object has been vetoed by an animation before this one in the list
     * it means the state of the object this animation contains has not yet come into reality.
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
    }
    
    /**
     * This is called to paint any thing to the screen after all animations, crafts, planets
     * and automatons have been pained.
     * 
     * Usually this can be left but missiles are painted as a post process as they need to appear 
     * on top of any craft that may occupy the same location - animations may paint the crafts so
     * they must be painted after the animations.
     * 
     * NOTE - Automatons are painted after all the animations so any painting that MUSt be done
     * after the automatons this may be used.
     * 
     * @param painter The parameter is of type object because this animation class is independent 
     * of any GUI implementation. However, a swing interface would probably be passing a Graphics2D
     * object.
     * @param list The current list of animations scheduled for display. If an animation is not active
     * it must make sure that there are no other animation BEFORE this animation in the list that has 
     * vetoed the draw object. If this object has been vetoed by an animation before this one in the list
     * it means the state of the object this animation contains has not yet come into reality.
     */
    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
    }
    
    
    /**
     * This may be used after tick to check if this animation requires repainting.
     * @return True if an only if the animations needs repainting.
     */
    public boolean needsPainting() {
        return (intervalCount == interval);
    }

    /**
     * If this method returns true then this animation is only executed when it is the first
     * in the animations list AND no other animation is displayed while it is running.
     * @return True if and only if this animation should only be displayed when no 
     * other animations are being displayed.
     */
    public boolean isExclusive() {
        return false;
    }

    /**
     * Animations like move missile do not want the missile to be painted when the
     * animations is in progress. However animations like damage, weapon fire want
     * the both crafts rendered because they do not render replacements.
     * @param bo The battle object that is should not be draw on the screen if this method 
     * returns true.
     * @return True if an only if the object passed in is painted by this animation 
     * when it is active.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return false;
    }

    /**
     * Animations like fire weapons for SpacialDistortion displays the creation of a spatial
     * distortion on the battle field. It is important that this spatial distortion is
     * not displayed before its creation has been animated. Therefore the anything that
     * animated the creation of the spatial distortion should veto its display.
     * @param add A spatial distortion that is waiting to be displayed on the battle map.
     * @return True if the spatial distortion passed in should NOT be drawn.
     */
    public boolean vetoDrawAnimatedArea(AnimatedBattleAreaDetails sdd) {
        return false;
    }
    
    public int getInterval() {
        return interval;
    }

    public void setInterval(int interval) {
        this.interval = interval;
    }

    /**
     * Append a simple description of this animation to the buffer being - class name(target)
     */
	public abstract void toString(StringBuffer sb);

	/**
	 * Sounds that have not already been played should be in the waiting list!
	 * @return True if this animation has a sound that has not already been played.
	 */
	public boolean hasSoundToPlay() {
		return (soundsWaiting != null) && (soundsWaiting.size() > 0);
	}
	
	public boolean hasSoundPlaying() {
		return (soundsPlaying != null) && (soundsPlaying.size() > 0);
	}
	
	public void playSound() throws ApplicationError {
		if (soundsWaiting != null && soundsWaiting.getSounds() != null) {
			for (BattleAnimationSound sound : soundsWaiting.getSounds()) {
				if (sound.getStage() <= stage) {
					playSound(sound);
				}
			}
		}
	}

	public void playSound(BattleAnimationSound sound) throws ApplicationError {
		if (soundsWaiting != null) {
			soundsWaiting.remove(sound);
		}
		addSoundPlaying(sound);
		//Logger.logDebug("Start");
		sound.getSound().play();
	}
	
	public void stopSound() {
		if (soundsPlaying != null && soundsPlaying.getSounds() != null) {
			for (BattleAnimationSound sound : soundsPlaying.getSounds()) {
				sound.getSound().stop();
				//Logger.logDebug("Stop ");
			}
		}
	}
	
	public void addSound(Sound sound, int stage) {
		addSoundWaiting(sound,stage); 
	}

	public void addSound(Sound sound, double percentOfLife) {
		addSoundWaiting(sound, (int)(getLife()*percentOfLife)); 
	}

	private void addSoundWaiting(Sound sound, int stage) {
		if (soundsWaiting == null) {
			soundsWaiting = new BattleAnimationSoundList();
		}
		soundsWaiting.add(new BattleAnimationSound(sound, stage));
	}

	private void addSoundPlaying(BattleAnimationSound sound) {
		if (soundsPlaying == null) {
			soundsPlaying = new BattleAnimationSoundList();
		}
		soundsPlaying.add(sound);
	}

	/**
	 * This is used to determine the area on the screen this animations effects.
	 * This is important because the GUI may want to move the view area of the screen when to make sure this animation is visible.
	 * @return The area on the battle this animations uses.
	 * @throws ApplicationError
	 */
	public Area getArea() throws ApplicationError {
		return null;
	}

	/**
	 * @return True if this animation operates on the object bo.
	 */
	public boolean isAnimated(BattleObjectDelegate bo) {
		return false;
	}

	public PlayerDelegate getAnimatedObjectOwner() throws ApplicationError {
		return null;
	}

    
}