/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.animation;

import java.util.ArrayList;
import java.util.Iterator;

import mot.client.deligates.BattleAutomatonDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.swing.animation.CraftShiftAnimation;
import mot.client.swing.animation.ExplodeAnimation;
import mot.client.swing.animation.LinkedAnimation;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;

/**
 * This is a list of animations waiting to be displayed. Once an animation has been 
 * displayed it should be removed from the list.
 */
public class BattleAnimationList  {

    /**
     * The list of animation currently being displayed on the screen.
     */
    private ArrayList list = new ArrayList();

    public BattleAnimationList() {
    }

	public ArrayList getList() {
        return list;
    }

    public void setList(ArrayList list) {
        this.list = list;
    }
    
    public void add(BattleAnimation a) {
        list.add(a);
//        if (Logger.isDebug()) {
//            Logger.logDebug("AnimationList.add() animations="+toString());
//        }
    }
    
    public void add(BattleAnimation [] a) {
        if (a == null) return;
        for (int i=0; i<a.length; i++) {
        	add(a[i]);
        }
    }
    
    public void remove(BattleAnimation a) {
        list.remove(a);
//        if (Logger.isDebug()) {
//            Logger.logDebug("AnimationList.remove() animations="+toString());
//        }
        if (a.hasSoundPlaying()) {
        	a.stopSound();
        }
    }
    
    public int getQuantity() {
        return list.size();
    }
    
    /**
     * An animation is not active when there is an exclusive animation in front of it or when
     * it is an exclusive animation not at the front.
     * @return A list containing all active animations.
     */
    public BattleAnimation [] getAllAnimations() {
        Iterator i = list.iterator();
        if (i == null) return null;
        ArrayList r = null;
        while (i.hasNext()) {
            BattleAnimation a = (BattleAnimation) i.next();
            if (r == null) r = new ArrayList();
            r.add(a);
        }
        if (r == null) {
            return null;
        } else {
            return (BattleAnimation[]) r.toArray(new BattleAnimation[r.size()]);
        }
    }
    
    /**
     * An animation is not active when there is an exclusive animation in front of it or when
     * it is an exclusive animation not at the front.
     * @return A list containing all active animations.
     */
    public BattleAnimation [] getActiveAnimations(Boolean after) {
        Iterator i = list.iterator();
        if (i == null) return null;
        boolean first = true;
        ArrayList r = null;
        while (i.hasNext()) {
            BattleAnimation a = (BattleAnimation) i.next();
            if (a.isExclusive() && !first) {
                break;
            }
            if (after  == null || a.isAfter() == after.booleanValue()) {
                if (r == null) r = new ArrayList();
                r.add(a);
            }
            first = false;
            if (a.isExclusive()) {
                break;
            }
        }
        if (r == null) {
            return null;
        } else {
            //if (r.size() > 3) {
            //    Logger.logDebug("XX");
            //}
            return (BattleAnimation[]) r.toArray(new BattleAnimation[r.size()]);
        }
    }

    public Area getActiveAnimationArea(Boolean after) throws ApplicationError {
    	BattleAnimation[] activeAnimations = getActiveAnimations(after);
    	if (activeAnimations == null || activeAnimations.length == 0) {
    		return null;
    	}
    	if (activeAnimations.length == 1) {
    		return activeAnimations[0].getArea();
    	}
    	Area r = Area.createArea(activeAnimations);
    	return r;
    }
    
    /**
     * @return True iff the animation is currently active.
     */
    public boolean isActiveAnimation(BattleAnimation animation) {
        Iterator i = list.iterator();
        if (i == null) return false;
        while (i.hasNext()) {
            BattleAnimation a = (BattleAnimation) i.next();
            if (a == animation) {
            	return true;
            }
            if (a.isExclusive()) {
                return false;
            }
        }
        return false;
    }
    
    /**
     * An animation is not active when there is an exclusive animation in front of it or when
     * it is an exclusive animation not at the front.
     * @return A list containing all non active animations.
     */
    public BattleAnimation [] getNonActiveAnimations(Boolean after) {
        Iterator i = list.iterator();
        if (i == null) return null;
        boolean first = true;
        boolean foundExclusive = false;
        ArrayList r = null;
        while (i.hasNext()) {
            BattleAnimation a = (BattleAnimation) i.next();
            if (a.isExclusive()) {
                foundExclusive = true;
            }
            if (foundExclusive && !first) {
                if (after == null || a.isAfter() == after.booleanValue()) {
                    if (r == null) r = new ArrayList();
                    r.add(a);
                }
            }
            first = false;
        }
        if (r == null) {
            return null;
        } else {
            return (BattleAnimation[]) r.toArray(new BattleAnimation[r.size()]);
        }
    }
 
	public boolean containsActiveAnimationFor(BattleObjectDelegate bo) {
        Iterator i = list.iterator();
        if (i == null) return false;
        while (i.hasNext()) {
            BattleAnimation a = (BattleAnimation) i.next();
            if (a.isExclusive()) {
            	return false;
            }
            if (a.isAnimated(bo)) {
            	return true;
            }
        }
        return false;
	}

	public BattleAnimation getFirstActiveAnimation() {
        if (list.size() > 0) {
            BattleAnimation a = (BattleAnimation) list.get(0);
           	return a;
        }
        return null;
	}

	public BattleAnimation getActiveAnimationOfClass(Class clazz) {
        Iterator i = list.iterator();
        if (i == null) return null;
        while (i.hasNext()) {
            BattleAnimation a = (BattleAnimation) i.next();
            if (a.getClass() == clazz) {
            	return a;
            }
            if (a.isExclusive()) {
            	return null;
            }
        }
        return null;
	}

	public boolean containsActiveAnimationForFromBack(BattleObjectDelegate bo) {
		int qty = list.size();
		if (qty == 0) return false;
        for (int i = qty-1; i >= 0; i--) {
        	BattleAnimation ba = (BattleAnimation) list.get(i);
            if (ba.isAnimated(bo)) {
            	return true;
            }
            if (ba.isExclusive()) {
            	return false;
            }
        }
        return false;
	}

	public BattleAnimation getActiveAnimationFromBackOfClass(Class clazz) {
		int qty = list.size();
		if (qty == 0) return null;
        for (int i = qty-1; i >= 0; i--) {
        	BattleAnimation ba = (BattleAnimation) list.get(i);
            if (ba.getClass() == clazz) {
            	return ba;
            }
            if (ba.isExclusive()) {
            	return null;
            }
        }
        return null;
	}
	
	public BattleAnimation getActiveAnimationFromBackOfClass(Class [] clazz) {
		int qty = list.size();
		if (qty == 0) return null;
        for (int i = qty-1; i >= 0; i--) {
        	BattleAnimation ba = (BattleAnimation) list.get(i);
            if (Comp.in(ba.getClass(), clazz)) {
            	return ba;
            }
            if (ba.isExclusive()) {
            	return null;
            }
        }
        return null;
	}
	
	public BattleAnimation getActiveAnimationFromBackOfClass(Class clazz, BattleObjectDelegate bo) {
		int qty = list.size();
		if (qty == 0) return null;
        for (int i = qty-1; i >= 0; i--) {
        	BattleAnimation ba = (BattleAnimation) list.get(i);
            if (ba.getClass() == clazz && ba.isAnimated(bo)) {
               	return ba;
            }
            if (ba.isExclusive()) {
            	return null;
            }
        }
        return null;
	}
	
	public BattleAnimation getAnimationOfClassFromBackOrLastActive(Class clazz) {
		int qty = list.size();
		if (qty == 0) return null;
        for (int i = qty-1; i >= 0; i--) {
        	BattleAnimation ba = (BattleAnimation) list.get(i);
            if (ba.getClass() == clazz) {
            	return ba;
            }
            if (ba.isExclusive()) {
            	return ba;
            }
        }
        return null;
	}
	
	public BattleAnimation getLastAnimation(int index) {
		int qty = list.size();
		if (qty == 0) return null;
		if (index > qty-1) return null;
        return (BattleAnimation) list.get(qty-(index+1));
	}
	
	public boolean lastAnimationIsExplodeFor(BattleObjectDelegate bo, int index) {
		BattleAnimation lastAnimation = getLastAnimation(index);
 		if (lastAnimation != null && lastAnimation instanceof ExplodeAnimation) {
 			ExplodeAnimation explode = (ExplodeAnimation)lastAnimation;
 			if (explode.getTarget() instanceof BattleAutomatonDelegate) {
 				return lastAnimationIsExplodeFor(bo, ++index);
 			} else {
 				return (Comp.same(explode.getTarget(), bo));
 			}
 		}
 		return false;
	}
	
	public boolean lastAnimationsInList(Class class1, Class class2) {
		int qty = list.size();
		if (qty < 2) return false;
        Object l1 = list.get(qty-1);
        Object l2 = list.get(qty-2);
		return l2.getClass() == class1 && l1.getClass() == class2;
	}

	
    public boolean tick() throws ApplicationError {
    	return tick(1);
    }
    
    /**
     * Moves all the animation to the next stage of the animation Any animations 
     * that reach the end of their life will be removed.
     * @return True if an only if the animations need repainting.
     * @throws mot.common.util.ApplicationError
     */
    public boolean tick(int qty) throws ApplicationError {
        return tick(getActiveAnimations(null), qty);
    }
    
    private boolean tick(BattleAnimation [] al, int qty) throws ApplicationError {
		boolean repaint = tick(al);
    	while (qty > 1) {
    		qty--;
    		boolean r = tick(al);
    		repaint = repaint || r;
    	}
    	return repaint;
    }
    
    private boolean tick(BattleAnimation [] al) throws ApplicationError {
        //Logger.logDebug("ALL ANIMATIONS : "+toString(list.iterator()));
        //Logger.logDebug("tick : "+toString(aa));
        boolean repaint = false;
        boolean removedHead = false;
        if (al != null) {
            for (int i=0; i<al.length; i++) {
                BattleAnimation a = al[i];
                a.tick();
                if (a.isComplete()) {
                    remove(a);
                    repaint = true;
                    removedHead = true;
                } else {
                	if (a.soundIsComplete() && a.hasSoundPlaying()) {
                    	a.stopSound();
                	}
                	if (a.needsPainting()) {
                		repaint = true;
                	}
                }
            }
            if (removedHead && list.size() > 0) {
                // We must remove any animations at the front of the list that have expired
                // The list al may only contain active animations (the front of the list) and
                // not it has been removed it may reveal an animation that is already complete
                // e.g. A delay of zero animation (these are used as separators).
                BattleAnimation head = (BattleAnimation)list.get(0);
                while (head != null && head.isComplete()) {
                    list.remove(0);
                    if (list.size() > 0) {
                    	head = (BattleAnimation)list.get(0);
                    } else {
                    	head = null;
                    }
                }
            }
        }
        return repaint;
    }
 
 
     /**
     * Returns Animation that require displaying before craft a painted.
     * @return A Iterator of Animation objects that are active and need displaying.
     */
    public Iterator getAnimations() {
        return list.iterator();
    }
   
    /**
     * @param bo The battle object that may be drawn by an animation and thus have its 
     * painting else where vetoed.
     * @param before If this is not null any animation up to but not including before may veto
     * the bo draw and all animations after and including before are not given the opportunity
     * to veto. If before is null, then all animations are given the chance to veto.
     * @return  true if this list of animations contains one or more animations 
     * that do not want this objected painted because they them selves draw it. See 
     * Animation, drawObject(BattleObject) for details.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo, BattleAnimation before) {
        BattleAnimation [] aa = getAllAnimations();
        if (aa != null) {
            for (int i=0; i<aa.length; i++) {
                BattleAnimation a = aa[i];
                if (a == before) {
                    return false;
                }
                if (before != null && a instanceof EventAnimation && before instanceof LinkedAnimation) {
                    EventAnimation ea = (EventAnimation) a;
                    LinkedAnimation la = (LinkedAnimation) before;
                    if (ea.containsLinkedAnimation(la)) {
                        return false;
                    }
                }
                if (a.vetoDrawObject(bo)) {
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     * When a move animation is in the queue, the delegate will return a location newer then the animation's old location.
     * @param target A battle object on the battle map
     * @return The location of this object at this time if it contains an animation in the queue. If no animations exist it will return null.
     */
	public Location getOldestLocationFor(String target) {
		if (target == null) return null;
		int qty = list.size();
		for (int i = 0; i < qty; i++) {
			BattleAnimation ba = (BattleAnimation) list.get(i);
			if (ba instanceof CraftShiftAnimation) {
				CraftShiftAnimation ma = (CraftShiftAnimation) ba;
				if (Comp.same(ma.getCraft(), target)) {
					return ma.getLocation();
				}
			}
		}
		return null;
	}

    
    /**
     * Animations like fire weapons for SpacialDistortion displays the creation of a spatial
     * distortion on the battle field. It is important that this spatial distortion is
     * not displayed before its creation has been animated. Therefore the anything that
     * animated the creation of the spatial distortion should veto its display.
     * @param sdd A spatial distortion that is waiting to be displayed on the battle map.
     * @return True if the spatial distortion passed in should NOT be drawn.
     */
    public boolean vetoDrawAnimatedArea(AnimatedBattleAreaDetails abad) {
        BattleAnimation [] aa = getAllAnimations();
        if (aa != null) {
            for (int i=0; i<aa.length; i++) {
                BattleAnimation a = aa[i];
                if (a.vetoDrawAnimatedArea(abad)) {
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     * Play any sounds of active animations that have not already been played.
     * NB This ensure each sound is only played once.
     * @throws ApplicationError 
     */
	public void playSounds() throws ApplicationError {
		BattleAnimation[] al = getActiveAnimations(null);
		if (al != null) {
			for (int i = 0; i < al.length; i++) {
				if (al[i].hasSoundToPlay()) { 
					al[i].playSound();
				}
			}
		}
	}
    
    public String toString() {
        Iterator i = list.iterator();
        if (i == null) return null;
		StringBuffer sb = new StringBuffer();
        while (i.hasNext()) {
            BattleAnimation a = (BattleAnimation) i.next();
            a.toString(sb);
            sb.append(",");
        }
		return sb.toString();
	}
    
    public static String toString(BattleAnimation [] l) {
        if (l == null) return null;
		StringBuffer sb = new StringBuffer();
        for (int i=0; i<l.length; i++) {
            BattleAnimation a = l[i];
            a.toString(sb);
			sb.append(":"+a.getStage());
			sb.append(", ");
        }
		return sb.toString();
	}

	public void stopAll() {
        BattleAnimation [] al = getActiveAnimations(null);
        for (int i = 0; al != null && i < al.length; i++) {
        	al[i].stopSound();
		}
        list.clear();
	}

	/**
	 * @return The owner (player who owned the object being animated) of the active animation in the window.
	 * Returns null if there are is nothing being animated;
	 * @throws ApplicationError 
	 */
	public PlayerDelegate getOwnerOfActiveAnimation(boolean ignoreDelayAnimationAtStart) throws ApplicationError {
		int size = list.size();
		for (int i=0; i < size; i++) {
            BattleAnimation a = (BattleAnimation) list.get(i);
// Damage animations last longer than weapon fire and do not have an owner - SO this causes blinking on troop transport
//            if (i == 0 && ignoreDelayAnimationAtStart && a instanceof DelayAnimation) {
//            	continue;
//            }
            PlayerDelegate owner = a.getAnimatedObjectOwner();
            if (owner != null) {
            	return owner; 
            }
//            if (a.isExclusive()) {
//            	return null;
//            }
        }
        return null;
	}

}