/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.animation;

import java.util.HashMap;
import mot.client.deligates.BattleObjectDelegate;

/**
 * Maintains a list of BattleObjects that have been painted on the screen.
 * At the start of every paint() method reset the contents of this list, then
 * every time a battle object is painted add it to the register. Before painting
 * a battle object that may already have been painted, check if it is already in
 * this register.
 * 
 * Note - This is a singleton class.
 * 
 * This is important for two reasons;
 *  1 - Effeciency
 *  2 - The correct image needs to be on the screen.
 *      An animation may draw a battle object at health X but the actual object
 *      is drawn with health Y - becuase damage has been subtracted from the objects
 *      health.
 *      E.g. MissileMove - Damage displayed must be damage from before the missile
 *      strike (not after).
 *      
 * NOTE - This is no longer used! Animations veto can veto the painting on the 
 *        objects they animate - this is a more flexible approach!
 */
public class BattleObjectPaintRegister  {

    /**
     * The list of battle object that have been painted.
     */
    private HashMap list = new HashMap();

    private BattleObjectPaintRegister() {
    }
    
    private static BattleObjectPaintRegister instance = new BattleObjectPaintRegister();
    public static BattleObjectPaintRegister getInstance() { return instance; }
    
    /**
     * Remove all entries from the paint register.
     */
    public void clear() {
        list.clear();
    }

    /**
     * Adds a new battle object to the register.
     * @param bo The battle object to add
     */
    public void add(BattleObjectDelegate bo) {
        list.put(bo.getName(), bo);
    }

    /**
     * @return true if this register contains the battle object passes in.
     * @param bo The battle object that may have been added.
     */
    public boolean contains(BattleObjectDelegate bo) {
        return list.containsKey(bo.getName());
    }
}