/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.animation;

import java.util.ArrayList;

import mot.client.deligates.BattleObjectDelegate;
import mot.client.sounds.Sound;
import mot.client.swing.animation.LinkedAnimation;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.event.LinkedEventBase;
import mot.common.event.MotEvent;
import mot.common.util.ApplicationError;

/** 
 * This is an animation that was caused by an event.
 * e.g. A weapon fire event causes a animation of a weapons fire.
 */
public abstract class EventAnimation extends BattleAnimation {

    /**
     * This is the list of linked animations to this animation.
     * 
     * Some events contained linked events and as such some animations contain linked animations.
     * When an animation is linked to another animation they contain the same life and stage values
     * and are animated together as a single animation.
     */
    private ArrayList linkedAnimations;
    private transient LinkedAnimation [] linkedAnimationsArray;
    
    /**
     * This is the event that has caused this animation.
     * e.g. A weapon fire event causes a animation of a weapons fire.
     */
    private MotEvent event;

    public EventAnimation(MotEvent event, int life, boolean after, int interval, Sound sound) throws ApplicationError {
        super(life, after, interval, sound);
        this.event = event;
    }
    
    public EventAnimation(MotEvent event, int life, boolean after, int interval, Object container, Sound sound) throws ApplicationError {
        super(life, after, interval, sound);
        this.event = event;

        if (event instanceof LinkedEventBase) {
            LinkedEventBase e = (LinkedEventBase)event;
            MotEvent[] linkedEvents = e.getLinkedEvents();
            if (linkedEvents != null) {
                for (int i=0; i<linkedEvents.length; i++) {
                    addLinkedEvent(linkedEvents[i], container);
                }
            }
        }
    }

    public MotEvent getEvent() {
        return event;
    }

    public void setEvent(MotEvent event) {
        this.event = event;
    }
    
    public void addLinkedAnimation(LinkedAnimation anim) {
        if (linkedAnimations == null) linkedAnimations = new ArrayList();
        linkedAnimations.add(anim);
        linkedAnimationsArray = null;
    }
    
    public void addLinkedAnimation(LinkedAnimation [] anims) {
        if (anims == null) return;
        for (int i=0; i<anims.length; i++) {
        	addLinkedAnimation(anims[i]);
        }
    }
    
    public LinkedAnimation [] getLinkedAnimations() {
        if (linkedAnimations == null) {
            return null;
        } else {
        	if (linkedAnimationsArray == null) {
        		linkedAnimationsArray = (LinkedAnimation[]) linkedAnimations.toArray(new LinkedAnimation[linkedAnimations.size()]);
        	}
            return linkedAnimationsArray;
        }
    }

    public boolean containsLinkedAnimation(LinkedAnimation anim) {
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                if (la[i] == anim) {
                    return true;
                }
            }
        }
        return false;
    }
    
    public void tick() throws ApplicationError {
        super.tick();
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                la[i].tick();
            }
        }
    }

    public int getTotalLife() {
        int r = getLife();
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                int l = la[i].getLife();
                if (l > r) r = l;
            }
        }
        return r;
    }
    
    public boolean isComplete() { return super.isComplete() && allLinkedAnimationsComplete(); }

    public boolean soundIsComplete() { return super.isComplete(); }

	/**
     * Some linked animations must be complete before the owning animation is considered 
     * complete. This method checks that there are no linked animations that this animation
     * @return True if all the linked animations are complete 
     * must wait for that have not completed.
     */
    protected boolean allLinkedAnimationsComplete() {
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                if (!la[i].canComplete(this)) {
               		return false;
                }
            }
        }
        return true;
    }

    /**
     * Override this method to handle linked events. Create the correct animation and call
     * addLinkedAnimation() for add the new animation to the list of animations.
     * This does not handle linked animations when they have more than one owner. This creates
     * a LinkedAnimation for the event passed in with a single owner (this event).
     * @param event The event for which to create a new animation.
     * @param container This is the window to contain this animation.
     */
    public void addLinkedEvent(MotEvent event, Object container) throws ApplicationError {
    }
    
    public void linkedAnimationsToString(StringBuffer sb) {
        if (linkedAnimations != null) {
        	sb.append("{");
        	LinkedAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
            	la[i].toString(sb);
            }
            sb.append("}");
        }
    }

    /**
     * @return True if any of the linked animations veto the drawing of this battle object or
     * false otherwise.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                if (la[i].vetoDrawObject(bo)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * @return True if any of the linked animations veto the drawing of this spacial 
     * distortion or false otherwise.
     */
    public boolean vetoDrawAnimatedArea(AnimatedBattleAreaDetails sdd) {
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                if (la[i].vetoDrawAnimatedArea(sdd)) {
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     * Paint all of the linked animations that are active.
     */
    public void paintActive(Object painter) throws ApplicationError {
        if (linkedAnimations != null) {
            BattleAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                la[i].paintActive(painter);
            }
        }
    }

    /**
     * Paint all of the linked animations that are active.
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        if (linkedAnimations != null) {
            BattleAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                la[i].paintNonActive(painter, list);
            }
        }
    }

    /**
     * Paint all of the linked animations that require post process painting.
     */
    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
        if (linkedAnimations != null) {
            BattleAnimation [] la = getLinkedAnimations();
            for (int i=0; i<la.length; i++) {
                la[i].paintPostProcess(painter, active, list);
            }
        }
    }

	public boolean hasSoundToPlay() {
        if (super.hasSoundToPlay()){
    		return true;
        }
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
        	for (int i = 0; i < la.length; i++) {
        		if (la[i].hasSoundToPlay()) {
        			return true;
        		}
			}
        }
        return false;
	}
	
	public boolean hasSoundPlaying() {
        if (super.hasSoundPlaying()){
    		return true;
        }
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
        	for (int i = 0; i < la.length; i++) {
        		if (la[i].hasSoundPlaying()) {
        			return true;
        		}
			}
        }
        return false;
	}
	
	public void playSound() throws ApplicationError {
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
        	for (int i = 0; i < la.length; i++) {
        		la[i].playSound();
			}
        }
    	super.playSound();
	}

	public void stopSound() {
        if (linkedAnimations != null) {
        	LinkedAnimation [] la = getLinkedAnimations();
        	for (int i = 0; i < la.length; i++) {
        		la[i].stopSound();
			}
        }
    	super.stopSound();
	}

    
}