/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.animation;

import java.awt.Graphics2D;
import java.util.HashMap;

import mot.common.util.ApplicationError;

/**
 * An animation is an event that has occurred in the game engine and have some display 
 * manifestations. For example a animated line across the galaxy representing a warp
 * trail.
 * 
 * How animations are displayed is dependent of the animation type. This is the base 
 * class for all animations.
 */
public abstract class GalaxyAnimation {

    /**
     * This is the current place in this animation life.
     * For example an explosion may display a number of bitmaps until it is complete.
     * This is an index from 0 to life-1 of the current bitmap that is being used.
     */
    private int stage;
    
    /**
     * This is the quantity of turns (stages) this animation should remain alive. When 
     * stage reaches life the animation is complete.
     */
    private int life;
    
    /**
     * This is the interval (quantity of ticks) between each stage.
     */
    private int interval;
    
    /**
     * The quantity of ticks remaining to start the next stage.
     * When this is 0 the next tick will start the next stage.
     */
    private int intervalCount;
    
    /**
     * Some animations must maintain their stage between rebuilt events on the galaxy map.
     * This is a store of an animations that will ensure the animation object being 
     */
    class StageStoreItem {
    	public StageStoreItem(String name, int stage) { this.name = name; this.stage = stage; }
    	public String name;
    	public int stage;
    }
    
    private static HashMap<String, StageStoreItem> stageStore = new HashMap();
    private StageStoreItem stageStoreItem = null;

    public GalaxyAnimation(int life, int interval) {
        this.life = life;
        this.stage = 0;
        this.interval = interval;
        
        intervalCount = interval;
    }

    public GalaxyAnimation(int life, int interval, String stageStoreName) {
        this.life = life;
        this.stage = 0;
        this.interval = interval;
        
        intervalCount = interval;
        
        if (stageStoreName != null) {
        	stageStoreItem = stageStore.get(stageStoreName);
        	if (stageStoreItem != null) {
        		if (life == 0 || stageStoreItem.stage < life) {
        			stage = stageStoreItem.stage;
       	    		//Logger.logDebug("Restore Stage "+stageStoreName+" stage="+stage);
                }
        	} else {
        		stageStoreItem = new StageStoreItem(stageStoreName, stage);
        		stageStore.put(stageStoreName, stageStoreItem);
        	}
        }
    }

    public int getStage() {
        return stage;
    }

    public void setStage(int stage) {
    	//Logger.logDebug(""+getClass().getName()+" "+stage+" life "+life);
    	if (life == 0) {
    		this.stage = stage;
    	} else {
    		this.stage = stage % life;
    	}
    	if (stageStoreItem != null) {
    		stageStoreItem.stage = this.stage;
    	}
    }

    public int getLife() {
        return life;
    }

    public void setLife(int life) {
        this.life = life;
    }
    
    /**
     * Moves this animation to the next stage.
     * @throws mot.common.util.ApplicationError
     */
    public void tick() throws ApplicationError {
        if (intervalCount <= 0) {
            if (life == 0 || stage < life) {
                stage++;
            }
            intervalCount = interval;
        } else {
            intervalCount--;
        }
    	if (stageStoreItem != null) {
    		stageStoreItem.stage = stage;
    	}
    }
    
    /**
     * @return True if and only if this animation has completed. i.e. stage is equal to life.
     */
    public boolean isComplete() {
    	if (life == 0) {
    		return false;
    	} else {
        	return stage >= life; 
    	}
    }

    /**
     * This is called to paint the animation on the screen when the animation is active.
     */
    public abstract void paint(Graphics2D graphics) throws ApplicationError;
    
    public int getInterval() {
        return interval;
    }

    public void setInterval(int interval) {
        this.interval = interval;
    }
    
}