/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.animation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import mot.client.swing.window.galaxy.GalaxyMapPanel;
import mot.client.swing.window.main.MainWindow;

/**
 * Each planet orbits its sun. To describe an orbit we need
 *   - A starting angle (a value between 0 and 360).
 *   - The distance from the sun (radius)
 * A starting angle is created using a random value. The radius is 
 * calculated from the zoom and planet index (in the solar system).
 * To rotate around the sun we also need to have a speed of rotation.
 * This is the amount the planet moves around its orbit every animation
 * tick.
 * 
 * Each planet has two values that are generated each time the game 
 * is started
 *    - The orbit (angle and radius)
 *    - The speed of rotation
 *
 * These values are fixed for the duration of a game but are not saved
 * and are purely a UI concern.
 */
public class GalaxyAnimationRegister {

	/**
	 * Map from planet name to GalaxyOrbitDescription
	 */
	private static HashMap planetMap = new HashMap();

	/**
	 * Map from solar system name to GalaxyOrbitCollection
	 */
	private static HashMap solarSystemMap = new HashMap();
	
	/**
	 * This is the quantity of orbits being used by planets around this 
	 * solar system. @see getQtyOrbits() for details. 
	 */
	private static HashMap qtyOrbitIndexMap = new HashMap();
	
	public static GalaxyOrbitDescription getOrbitFor(String planetName, String solarSystemName) {
		GalaxyOrbitDescription od = (GalaxyOrbitDescription) planetMap.get(planetName);
		if (od == null) {
			GalaxyOrbitCollection oc = (GalaxyOrbitCollection) solarSystemMap.get(solarSystemName);
			if (oc == null) {
				oc = new GalaxyOrbitCollection();
				solarSystemMap.put(solarSystemName, oc);
			}
			od = new GalaxyOrbitDescription(planetName, oc);
			planetMap.put(planetName, od);
			oc.add(od);
			qtyOrbitIndexMap.remove(solarSystemName);
		}
		return od;
	}
		
	/**
	 * Note a solar system may have three planets orbiting it, but only two orbit 
	 * as planets may share the same orbit! @see GalaxyOrbitDescription for details 
	 * of orbit indexes.
	 * @param solarSystemName The name of the solar system we want to measure the qty orbits for
	 * @return The quantity of orbits being used around this solar system. 
	 */
	public static int getQtyOrbitIndexes(String solarSystemName) {
		Integer qtyOrbitIndexes = (Integer) qtyOrbitIndexMap.get(solarSystemName);
		if (qtyOrbitIndexes == null) {
			GalaxyOrbitCollection goc = (GalaxyOrbitCollection) solarSystemMap.get(solarSystemName);
			int r = 0;
			if (goc != null) {
				int qty = goc.getQtyOrbits();
				ArrayList indexes = new ArrayList();
				for (int i = 0; i < qty; i++) {
					GalaxyOrbitDescription go = goc.getOrbitWithIndex(i);
					Integer index = new Integer(go.getOrbitIndex());
					if ( !indexes.contains(index) ) {
						r++;
						indexes.add(index);
					}
				}
			}
			qtyOrbitIndexes = new Integer(r);
			qtyOrbitIndexMap.put(solarSystemName, qtyOrbitIndexes);
		}
		return qtyOrbitIndexes.intValue();
	}
	
	public static void tick() {
		MainWindow mainWin = MainWindow.getExestingInstance();
		if (mainWin != null && mainWin.isGalaxyMapConfigurationActive()) {
			GalaxyMapPanel galaxyMapPanel = mainWin.getGalaxyMapConfiguration().getGalaxyMapPanel();
			if (!galaxyMapPanel.isShowingMooListOrButtons()) {				
				if (planetMap.size() == 0) return;
				Iterator i = planetMap.values().iterator();
				while (i.hasNext()) {
					GalaxyOrbitDescription od = (GalaxyOrbitDescription) i.next();
					od.tick();
				}
			}
		}
	}

	public static void clean() {
		planetMap = new HashMap();
		solarSystemMap = new HashMap();
		qtyOrbitIndexMap = new HashMap();
	}
}
