/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.animation;

import mot.common.util.Utils;

/**
 * Describes the orbits of an object such as several planets around a sun.
 * 
 * Note that when 
 *   - Two things are in orbit around a star one is in orbit 
 *     index 1 the other in orbit index 2;
 *   - Three things are in orbit around a star two are in orbit 
 *     index 1 the other in orbit index 2;
 *   - Four+ things are in orbit around a star two are in orbit 
 *     index 1 and the rest are in orbit index 2;
 * 
 * This is purly a UI concern - The engine is not aware about the 
 * details of a planets orbit around a star.
 * 
 * @see GalaxyAnimationRegister for more details
 */
public class GalaxyOrbitDescription {
	
    public static final int MAX_PLANET_ORBITS = 2;
	
	/**
	 * The name of the object whose orbit this description is describing
	 */
	private String name;
	
	/**
	 * Current rotation of the planet that this description belongs to
	 * This is a value between 0 and 360.
	 */
	private double rotation;
	
	/**
	 * The speed at which this planet rotates
	 */
	private double speedOfRotation;
	
	/**
	 * This is the index of the orbit of this object around the star.
	 * A value of 0 or 1 - There are only two orbits around a star!
	 */
	private int orbitIndex;
	
	public GalaxyOrbitDescription(String name, GalaxyOrbitCollection oc) {
		this.name = name;
		
		int qtyOrbits = oc.getQtyOrbits();
		if (qtyOrbits < 2) {
			orbitIndex = qtyOrbits+1;
			rotation = Utils.random(360);
			speedOfRotation = ((1.0+Utils.random(2))/3.0 + orbitIndex/3.0) / 8.0;
		} else if (qtyOrbits == 2) {
			orbitIndex = 1;
			rotation = (oc.getOrbitWithIndex(0).rotation + Utils.random(100) + 135) % 360;
			speedOfRotation = oc.getOrbitWithIndex(0).speedOfRotation;
		} else if (qtyOrbits == 3) {
			orbitIndex = 2;
			rotation = (oc.getOrbitWithIndex(1).rotation + Utils.random(25) + 180) % 360;
			speedOfRotation = oc.getOrbitWithIndex(1).speedOfRotation;
		} else if (qtyOrbits == 4) {
			orbitIndex = 2;
			rotation = (oc.getOrbitWithIndex(1).rotation + Utils.random(25) + 90) % 360;
			speedOfRotation = oc.getOrbitWithIndex(1).speedOfRotation;
		} else if (qtyOrbits == 5) {
			orbitIndex = 2;
			rotation = (oc.getOrbitWithIndex(1).rotation + Utils.random(25) + 270) % 360;
			speedOfRotation = oc.getOrbitWithIndex(1).speedOfRotation;
		} else {
			orbitIndex = 2;
			rotation = (oc.getOrbitWithIndex(1).rotation + 45+90*(qtyOrbits-6)) % 360;
			speedOfRotation = oc.getOrbitWithIndex(1).speedOfRotation;
		}
	}
	
	public void tick() {
		rotation += speedOfRotation;
		if (rotation >= 360) rotation = 0;
	}

	public double getRotation() {
		return rotation;
	}

	public void setRotation(double rotation) {
		this.rotation = rotation;
	}

	public double getSpeedOfRotation() {
		return speedOfRotation;
	}

	public void setSpeedOfRotation(double speedOfRotation) {
		this.speedOfRotation = speedOfRotation;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getOrbitIndex() {
		return orbitIndex;
	}

	public void setOrbitIndex(int orbitIndex) {
		this.orbitIndex = orbitIndex;
	}

}
