/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.animation;
import java.awt.Color;

import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageAnimationConfig;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;

/**
 * The main battle window must call tick on this class every interval.
 * This feeds timer information to the different objects that are animated on the 
 * battle window.
 * 
 * Do not confuse this with normal animation such as craft destroyed, craft 
 * move.
 * 
 * This governs things like shields and decorators that are not part of a 
 * special event but still are animated just to provide a better feel to the 
 * GUI.
 */
public class MovementRegister {

    private static final int QTY_TICKS_TO_SHIELD_CHANGE = 2;
    private static final int QTY_STAGES_IN_SHIELD_ANIMATION = 5;

    private static int tickNo = 0;

    /**
     * Must be called every tick by the battle window.
     */
    public static void tick() {
        tickNo++;
    }

    private static final Color blue1 = new Color(50, 200, 200);
    private static final Color blue2 = new Color(55, 180, 180);
    private static final Color blue3 = new Color(60, 160, 160);
    private static final Color blue4 = new Color(65, 150, 150);
    private static final Color blue5 = new Color(70, 140, 140);

    /**
     * Shields are animated through MAX_SHIELD_STATES. This is a number between 0 and
     * MAX_SHIELD_STATES -1 that is the current stage of the shield animation.
     * @return The state of the animation for the shields.
     */
    public static Color getShieldColor() {
        int stage = tickNo%50;
        if (stage > 25) stage = 25 - (stage - 25);
        if (stage < 5) {
            return blue1;
        } else if (stage < 10) {
            return blue2;
        } else if (stage < 15) {
            return blue3;
        } else if (stage < 20) {
            return blue4;
        } else if (stage <= 25) {
            return blue5;
        } else {
            return blue1;
        }
    }

    public static String getShieldImage(double shieldPercent) throws ApplicationError {
    	ImageAnimationConfig config = AnimationConfigFactory.getInstance().getShieldsAnimationConfig();
    	String[] images = config.getActualImages();
    	double qtyImages = config.getQtyImages();
    	double imageStep = qtyImages / 10;
		int startImage = 0;
		if (shieldPercent > 0.9) startImage = 0;
		else if (shieldPercent > 0.8) startImage = (int) imageStep;
		else if (shieldPercent > 0.7) startImage = (int) (imageStep * 2);
		else if (shieldPercent > 0.6) startImage = (int) (imageStep * 3);
		else if (shieldPercent > 0.5) startImage = (int) (imageStep * 4);
		else if (shieldPercent > 0.4) startImage = (int) (imageStep * 5);
		else if (shieldPercent > 0.3) startImage = (int) (imageStep * 6);
		else if (shieldPercent > 0.2) startImage = (int) (imageStep * 7);
		else if (shieldPercent > 0.1) startImage = (int) (imageStep * 8);
		else if (shieldPercent > 0.0) startImage = (int) (imageStep * 9);
		int indexOffset = tickNo / QTY_TICKS_TO_SHIELD_CHANGE;
		int index = indexOffset % (2*QTY_STAGES_IN_SHIELD_ANIMATION-1);
		if (index >= QTY_STAGES_IN_SHIELD_ANIMATION) index = (2*QTY_STAGES_IN_SHIELD_ANIMATION-1) - index;
		String image = images[Comp.min(startImage + index, images.length-1)];
    	return image;
    }

    /**
     * @return A value between 0 and 360 that is the amount a rotation animation 
     * should be rotated at this point in time. 
     */
    public static int getRotation() {
        return tickNo%360;
    }
    
	public static int getTickNo() {
		return tickNo;
	}
}