/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.bean;

import java.util.ArrayList;
import java.util.Iterator;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.vo.NavigatorVO;
import mot.common.vo.OrbitVO;

/**
 * This is a stack of moveable owned objects located in a single orbit.
 * In a game multiple craft / fleets can be located together (around a planet, 
 * star or whatever). The most visible one appear on top (unless the selected item
 * is in the stack).
 * 
 * The class hold the list of moos at a single location.
 *
 */
public class MoveableObjectLocation {

	public Location location;
	
    public ArrayList<MoveableOwnedObjectDelegate> playerList = new ArrayList();
    public int playerMostVisibleScore;
    public MoveableOwnedObjectDelegate playerMostVisible;
    
    public ArrayList<MoveableOwnedObjectDelegate> otherList = new ArrayList();
    public int otherMostVisibleScore;
    public MoveableOwnedObjectDelegate otherMostVisible;

    //public ArrayList<MoveableOwnedObjectDelegate> allList = new ArrayList();

    public MoveableObjectLocation(MoveableOwnedObjectDelegate object) throws ApplicationError { 
        this.location = object.getLocation(); 
        add(object); 
    }
    
    public void add(MoveableOwnedObjectDelegate object) throws ApplicationError { 
        int score = getVisibilityScore(object);
        if (object.isOwnedBy(MainWindow.getOwner())) {
        	playerList.add(object); 
            if (playerMostVisible == null || score > playerMostVisibleScore) { 
            	playerMostVisible = object; 
            	playerMostVisibleScore = score; 
            }
        } else {
        	otherList.add(object); 
            if (otherMostVisible == null || score > otherMostVisibleScore) { 
            	otherMostVisible = object; 
            	otherMostVisibleScore = score; 
            }
        }
        //allList.add(object);
    }

	private static int getVisibilityScore(MoveableOwnedObjectDelegate object) throws ApplicationError {
       	return object.getVisibility();
	}
    
    public MoveableOwnedObjectDelegate [] getPlayerMoos() {
        if (playerList == null || playerList.size() == 0) {
            return null;
        } else {
            return (MoveableOwnedObjectDelegate []) playerList.toArray(new MoveableOwnedObjectDelegate[playerList.size()]);
        }
    }
    
    public MoveableOwnedObjectDelegate getPlayerMooMostVisible() {
    	return playerMostVisible;
    }
    
    public MoveableOwnedObjectDelegate [] getOtherMoos() {
        if (otherList == null || otherList.size() == 0) {
            return null;
        } else {
            return (MoveableOwnedObjectDelegate []) otherList.toArray(new MoveableOwnedObjectDelegate[otherList.size()]);
        }
    }
    
    public MoveableOwnedObjectDelegate getOtherMooMostVisible() {
    	return otherMostVisible;
    }

    
    
	public OrbitVO getOrbit() throws ApplicationError {
		if (playerMostVisible != null) {
			return playerMostVisible.getOrbit();
		} else {
			return otherMostVisible.getOrbit();
		}
	}

	public Location getLocation() {
		return location;
	}

	public NavigatorVO getNavigator() throws ApplicationError {
		if (playerMostVisible != null) {
			return playerMostVisible.getNavigator();
		} else {
			return otherMostVisible.getNavigator();
		}
	}

	public boolean playerListContains(MoveableOwnedObjectDelegate object) {
		if (playerList != null && playerList.size() > 0) {
				Iterator i = playerList.iterator();
				while (i.hasNext()) {
					MoveableOwnedObjectDelegate moo = (MoveableOwnedObjectDelegate) i.next();
					if (moo.equals(object)) return true;
				}
		  }
		return false;
	}

	public boolean otherListContains(MoveableOwnedObjectDelegate object) {
		if (otherList != null && otherList.size() > 0) {
			Iterator i = otherList.iterator();
			while (i.hasNext()) {
				MoveableOwnedObjectDelegate moo = (MoveableOwnedObjectDelegate) i.next();
				if (moo.equals(object)) return true;
			}
		}
		return false;
	}

	public boolean contains(MoveableOwnedObjectDelegate object) {
		return playerListContains(object) || otherListContains(object);
	}

 
	public MoveableOwnedObjectDelegate getMostVisible() {
		if (playerMostVisible == null) {
			return otherMostVisible;
		} else {
			return playerMostVisible;
		}
	}
  
    public MoveableOwnedObjectDelegate [] getMoos() {
    	ArrayList r = new ArrayList();
		if (playerList != null && playerList.size() > 0) {
			Iterator i = playerList.iterator();
			while (i.hasNext()) {
				r.add(i.next());
			}
	    }
		if (otherList != null && otherList.size() > 0) {
			Iterator i = otherList.iterator();
			while (i.hasNext()) {
				r.add(i.next());
			}
		}
        return (MoveableOwnedObjectDelegate []) r.toArray(new MoveableOwnedObjectDelegate[r.size()]);
    }
    

}