/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.bean;

import java.util.HashMap;
import java.util.Iterator;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.common.util.ApplicationError;
import mot.common.vo.OrbitVO;

/**
 * Constructs MOO (Moveable Owned Object) locations. @see MoveableObjectLocation
 * for details.
 */
public class MoveableObjectLocationFactory {
	
    /**
     * @return All crafts visible by this player on the map. If the craft is a member of a fleet, then
     * the single fleet object is returned for all crafts in the fleet. Each instance of MoveableObjectLocation
     * contains all craft at that location.
     * This also includes Transports that are visible on the galaxy map. 
     * @see mot.engine.transports.Transport.displayOnGalaxyMap() for details of when a transport is displayed.
     */
    public static MoveableObjectLocation [] sortMoosByLocation(MoveableOwnedObjectDelegate [] moos, boolean objectsWithOrdersInSeperateList, boolean usePlanetaryOrbit) throws ApplicationError {
        HashMap r = null;
        
        if (moos != null) {
            for (int i=0; i<moos.length; i++) {
                MoveableOwnedObjectDelegate moo = moos[i];
                r = _addMooToTable(r, moo, objectsWithOrdersInSeperateList, usePlanetaryOrbit);
            }
        }
        
        if (r == null) {
            return null;
        } else {
            MoveableObjectLocation [] results = new MoveableObjectLocation[r.size()];
            Iterator i = r.values().iterator();
            int index = 0;
            while (i.hasNext()) {
                MoveableObjectLocation mol = (MoveableObjectLocation) i.next();
                results[index++] = mol;
            }
            return results;
        }
    }
    
    private static HashMap _addMooToTable(HashMap r, MoveableOwnedObjectDelegate moo, boolean objectsWithOrdersInSeperateList, boolean usePlanetaryOrbit) throws ApplicationError {
        String hashKey = getLocationHashKey(moo, objectsWithOrdersInSeperateList, usePlanetaryOrbit);
        if (r  == null) {
            r = new HashMap();
            r.put(hashKey, new MoveableObjectLocation(moo));
        } else {
            MoveableObjectLocation mol = (MoveableObjectLocation) r.get(hashKey);
            if (mol == null) {
                r.put(hashKey, new MoveableObjectLocation(moo));
            } else {
                if (!mol.contains(moo)) {
                    mol.add(moo);
                }
            }
        }
        return r;
    }

    private static String getLocationHashKey(MoveableOwnedObjectDelegate moo, boolean placeObjectWithOrdersInSeperateList, boolean usePlanetaryOrbit) throws ApplicationError {
    	String key = null;
    	OrbitVO orbit = moo.getOrbit();
    	if (usePlanetaryOrbit && orbit != null && orbit.getPlanet() != null) {
    		key = orbit.getPlanet();
    	} else if (orbit != null) {
    		key = orbit.getStellarObject();
    	} else {
    		key = moo.getLocation().getHashKey();
    	}
    	if (placeObjectWithOrdersInSeperateList && moo.getNavigator() != null) {
            return key+"withOrders";
        } else {
            return key;
        }
    }
    
}
