/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.coms;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import mot.client.event.MotClientEventProcessor;
import mot.client.proxy.CommandResponseProcessor;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.RemoteConnectionServerProxy;
import mot.common.command_delegates.CommandDeligate;
import mot.common.command_delegates.StartNextBattleInQueueCmdDeligate;
import mot.common.command_results.CommandResult;
import mot.common.coms.protocols.ClientEventProcessor;
import mot.common.coms.protocols.ServerConnection;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.RequestHandler;
import mot.engine.ConnectionList;

/**
 * This is the thread that is run on the server.
 * This thread is used to run long commands like endTurn.
 * This thread is not used for small commands or delegates which are run in the client thread (for performance reasons).
 * Because of this hybrid threading - we must only call engine code inside a synchronised block.
 * The Engine thread its self must only enter start working inside the same critical section.
 */
public class LocalServerConnection extends Thread implements ServerConnection {

	private static final String MOT_MP_SERVER_SOCKET_THREAD = "mot.mp.ServerSocketThread";

	/**
	 * List of commands / requests queued up for the engine to run.
	 */
	private BlockingQueue<CommandDeligate> inQueue;
	
	/**
	 * Engine object that processes the commands in the engine.
	 */
	private RequestHandler requestHandler;

	/**
	 * The client object that will process each command response coming back from the server.
	 */
	private CommandResponseProcessor responseProcessor;

	/**
	 * The client object that will process events send by the server to the client that owns this connection.
	 */
	private ClientEventProcessor clientEventProcessor;
	

	/**
	 * Thread that accepts remote connections to allow multi-player.
	 */
	private RemoteConnectionServerProxy remoteConnectionServerProxy; 


	/**
	 * USED ONLE BY TEST HARNESS
	 */
	private DebugCommandResponseHook debugHook;

	public LocalServerConnection(CommandResponseProcessor responseProcessor, MotClientEventProcessor eventProcessor) throws ApplicationError {
		super("EngineThread");
		this.responseProcessor = responseProcessor;
		this.inQueue = new LinkedBlockingQueue<CommandDeligate>();
		this.requestHandler = new RequestHandler();
		this.clientEventProcessor = eventProcessor;
		ConnectionList.setLocalServerConnection(this);
		start();
	}

	public void connect(String ip) throws ApplicationError, ValidationError {
	}

	public synchronized void disconnect() {
		ConnectionList.setLocalServerConnection(null);
		if (!started) return;
		interrupt();
	}

	@Override
	public boolean isConnected() {
		return true;
	}

//	public void register(MotClientEventProcessor eventProcessor) {
//		ConnectionList.add(this);
//		clientEventProcessor = eventProcessor;
//	}


	
	/*
	 * 
	 * ONLY CALLED IN CLIENT GUI THREAD
	 * 
	 */

	@Override
	public void send(CommandDeligate command) throws ApplicationError, ValidationError {
		inQueue.add(command);
		yield();
	}
	
	public LocalServerProxy getLocalServerProxy() {
		return new LocalServerProxy();
	}
	
	public RemoteConnectionServerProxy getRemoteConnectionServerProxy() throws ApplicationError {
		if (remoteConnectionServerProxy == null) {
			try {
				Class configClass = ProxyStore.getServerSocketClass();
				remoteConnectionServerProxy = (RemoteConnectionServerProxy) configClass.newInstance();
				remoteConnectionServerProxy.setRequestHandler(requestHandler);
			} catch (Exception e) {
				Logger.logError("", e);
				throw new ApplicationError(e.getMessage());
			}
		}
		return remoteConnectionServerProxy;
	}
	
	public boolean supportsRemoteConnection() {
		return remoteConnectionServerProxy != null || ProxyStore.getServerSocketClass() != null;
	}

	/*
	 * 
	 * METHODS BELOW THIS POINT ARE CALLED BY THE ENGINE USING THE ENGINES THREAD
	 * 
	 */
	

	public void processQueuedCommands() throws ApplicationError, InterruptedException {
		CommandDeligate command = inQueue.take();
		CommandResult cr = requestHandler.process(command, clientEventProcessor);
		if (cr != null) {
			sendResponseToGuiThread(cr);
		}
	}
	
	private void sendResponseToGuiThread(CommandResult cr) {
		if (debugHook != null) {
			debugHook.setCommandResponse(cr);
		} else {
			SendResponseToGuiThread.send(responseProcessor, cr);
		}
	}

	/**
	 * Engine thread starts here.
	 */
	public void run() {
		
		Logger.logInfo("LocalServerConnection.run() Start");
			
		while (true) {
			try {
				processQueuedCommands();
//			} catch (ClosedByInterruptException e2) {
//				Logger.logDebug("Server Thread - SHUTTING DOWN");
			} catch (InterruptedException e1) {
				Logger.logDebug("Server Thread - Interupted");
			} catch (Throwable t) {
				Logger.logError("EngineThread.run() "+t.getMessage(), t);
			}
		}
	}

	private boolean started;

	public synchronized void start() {
		if (!started) super.start();
		started = true;
	}

    public void addStartNextBattleToQueue() throws ApplicationError {
    	//Logger.logError("LocalServerConnection.addStartNextBattleToQueue() ", new Exception());
    	inQueue.add(new StartNextBattleInQueueCmdDeligate(-1));
    }

	public DebugCommandResponseHook getDebugHook() {
		return debugHook;
	}

	public void setDebugHook(DebugCommandResponseHook debugHook) {
		this.debugHook = debugHook;
	}

	public CommandResponseProcessor getResponseProcessor() {
		return responseProcessor;
	}

	public void setResponseProcessor(CommandResponseProcessor responseProcessor) {
		this.responseProcessor = responseProcessor;
	}

	public ClientEventProcessor getClientEventProcessor() {
		return clientEventProcessor;
	}

}
