/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates;

import java.awt.Color;

import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.vo.NavigatorVO;
import mot.common.vo.OrbitVO;

public interface MoveableOwnedObjectDelegate extends NamedObjectDelegate, Comparable<MoveableOwnedObjectDelegate> {
    public String getName();
    
    /**
     * The location can be anywhere in the galaxy map. However, if this object
     * is in orbit around a stellar object then this location should be the same
     * location as the stellar object's and getOrbit() will not return null.
     * @return The location of this object.
     * @throws ApplicationError
     */
	public Location getLocation() throws ApplicationError;
	
    /**
     * Note that the name of the image may be different for users and different for the
     * same user over time. This is due to cloaked ships etc. 
     * @return the name of the bitmap used to display this craft for this player on the screen.
     */
    public String getImage() throws ApplicationError;

    /**
     * @return the size of the bitmap used to display this craft for this player on the screen.
     */
    public Size getImageSize() throws ApplicationError;

    /**
     * @return an image of this craft in 3D form with no colour indicating the crafts owner.
     */
	public String get3DImage() throws ApplicationError;


    /**
     * @return True if this object is owned by the player passed in 
     * @throws ApplicationError
     */
    public boolean isOwnedBy(PlayerDelegate player) throws ApplicationError;

    /**
     * @return The name of the owner of this object 
     * @throws ApplicationError
     */
    public String getOwner() throws ApplicationError;
	public PlayerDelegate getOwnerDelegate() throws ApplicationError;
	
	/**
	 * @return the colour of this MOOs owner.
	 */
	public Color getOwnerColor() throws ApplicationError;
    
    /**
     * @return True if getOrbit() will not return null.
     * @throws ApplicationError 
     */
	public boolean hasOrbit() throws ApplicationError;

	/**
     * Generally - A movable object will either have an orbit or a navigator, but
     * if a craft has been assigned an order to leave a solar system but not yet
     * left (end turn has not been pressed) then it may have an orbit and a navigator!
     * @return The orbit of this object or null if it is not in orbit
     * @throws ApplicationError
     */
    public OrbitVO getOrbit() throws ApplicationError;

    /**
     * @return True if getNavigator() will not return null.
     */
	public boolean hasNavigator() throws ApplicationError;

	/**
     * Generally - A movable object will either have an orbit or a navigator, but
     * if a craft has been assigned an order to leave a solar system but not yet
     * left (end turn has not been pressed) then it may have an orbit and a navigator!
     * @return The navigator that controls the movement of a craft between one stellar 
     * object and another - or null if this craft is in orbit and not assigned an order
     * to travel to another stellar object
     * @throws ApplicationError
	 */
    public NavigatorVO getNavigator() throws ApplicationError;
    
	/**
	 * @return the visibility of this object by player at this point in time (on the galaxy map).
	 * The visibility of a craft is determined by the closest scanner strength (on a planet or a craft) and the 
	 * shields cloak value of this craft. The player cloak bonus also effects a crafts visibility.
	 */
    public int getVisibility() throws ApplicationError;

	/**
	 * @return The engine that governs this object.
	 */
	public int getRange() throws ApplicationError;
    
	/**
	 * NOTE - this state is store in the engine but is a client side concern. The engine
	 * does not take any action based upon the sleep state. The GUI on the other hand focuses 
	 * on any object not sleeping when the user presses end turn prompting the user to give 
	 * an order or sleep the craft.
	 * @return Has this object been commanded to - not request new orders each turn.
	 */
	public boolean isSleeping() throws ApplicationError;

  	public boolean equals(MoveableOwnedObjectDelegate other);
  	
  	/**
  	 * @return A description of this MOO and its state suitable to display to the user.
  	 * @throws ApplicationError
  	 */
  	public String getDescription(int maxLines, boolean withName, boolean withEta) throws ApplicationError;
	public String getEnrouteDescription() throws ApplicationError;

	/**
	 * Has this objected moved or has some orders for this turn
	 * @return Will return false if this object is idle and may want to do something this turn.
	 */
	public boolean hasMoved();


	public void flushImageCache();

	/**
	 * @return Can the owner issue an attack command to this MOO at the moment?
	 */
	public boolean galaxyMenuCanAttackActive() throws ApplicationError;
	
	/**
	 * @return Can the owner issue a bombard command to this MOO at the moment?
	 */
	public boolean galaxyMenuCanBombardActive() throws ApplicationError;
	
	/**
	 * @return Can the owner issue an invade command to this MOO at the moment?
	 */
	public boolean galaxyMenuCanInvadeActive() throws ApplicationError;
	
	/**
	 * @return Can the owner issue a colonise command to this MOO at the moment?
	 */
	public boolean galaxyMenuCanColoniseActive() throws ApplicationError;
	
	/**
	 * @return Can the owner issue a deploy command to this MOO at the moment?
	 */
	public boolean galaxyMenuCanDeployActive() throws ApplicationError;
	
	/**
	 * @return Can the owner issue a join fleet command to this MOO at the moment?
	 */
	public boolean galaxyMenuCanJoinFleetActive() throws ApplicationError;
	
	/**
	 * @return Can the owner issue an upgrade command to this MOO at the moment?
	 */
	public boolean galaxyMenuCanUpgradeActive() throws ApplicationError;
	
	/**
	 * @return Can the owner issue a delete command to this MOO at the moment?
	 */
	public boolean galaxyMenuCanDeleteActive() throws ApplicationError;


}