/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates;

import java.awt.Color;

import mot.common.designs.DesignBuilder;
import mot.common.designs.Orbital;
import mot.common.designs.ProjectDesign;
import mot.common.resources.Resource;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.vo.OrbitVO;
import mot.common.vo.PlanetCrisisVO;

public interface PlanetDelegate extends LabeledObjectDelegate, DesignBuilder {
    public String getName();
    public String getOwnerName() throws ApplicationError;
    public String getOriginName() throws ApplicationError;

    /**
     * @return The owner of this planet 
     */
	public PlayerDelegate getOwner() throws ApplicationError;
	
    /**
     * This is the first owner that this object had (will be different from the owner
     * id this object has changed hands during the game).
     * @return The origin player of this object 
     */
	public PlayerDelegate getOrigin() throws ApplicationError;
	
	/**
	 * @return The solar system this planet is part of
	 */
    public SolarSystemDelegate getSolarSystem();

    /**
     * @return The name of the solar system this planet is part of.
     */
    public String getSolarSystemName();

    /**
     * Used by user interfaces to avoid prompting a user to select a project every turn when
     * no project has been selected.
     * @return true if the owner of this planet does not want this planet to have a project.
     */
    public boolean getOwnerWantsNoProject();

    /**
     * @return The project this planet is currently working on or null if this 
     * planet is not building anything
     */
    public ProjectDesign getProjectDesign();
    
    /**
     * @return The quantity of turns to build the planet's current project.
     * @throws ApplicationError
     */
    public int getTurnsToCompleteProject(ProjectDesign project) throws ApplicationError;
    public int getTurnsToCompleteProject(int productionRequiredToFinish) throws ApplicationError;
    public int getTurnsToCompleteProject() throws ApplicationError;
	public int getCostToBuild() throws ApplicationError;

    /**
     * This description contains the name of the object being built by this project
     * followed by the quantity of turns until completion in square brackets.
     * @return A description of the current project.
     */
 	public String getProjectDescription() throws ApplicationError;
 	
 	/**
 	 * @return The quantity of times the current project is to be repeated.
 	 */
    public int getRepeatProject();
    
    /**
     * @return The cost to purchase this project (money required to build this project NOW).
     * @throws ApplicationError
     */
    public long getCostToPurchaseProject() throws ApplicationError;
    
    
    /**
     * @return If enemy military crafts are in orbit at this solar system.
     * @throws ApplicationError
     */
    public boolean isBlockaded() throws ApplicationError;
    public boolean isGuarded() throws ApplicationError;

    /**
     * A Planets view ability should not be confused with a solar systems visibility. 
     * If a solar system is visible then the planets may be viewed, but not the 
     * population/production/defence details. If a planet is viewable then all details of 
     * the planet may be viewed.
     * 
     * @return true if this delegate's owner can view the details of this planet.
     * If true the player can use the SolarSystemWindow to examine the planets details.
     */
    public boolean isViewable() throws ApplicationError;

    /**
     * @return true if player passed in is the owner of this object.
     */
    public boolean isOwnedBy(PlayerDelegate p) throws ApplicationError;

    /**
     * @return work done on this planets Project each turn
     */
	public int getProductionOutput() throws ApplicationError;
	public int getLostProductionOutput() throws ApplicationError;
	public int getResearchOutput() throws ApplicationError;
	public int getLostResearchOutput() throws ApplicationError;
	public double getEcologyOutput() throws ApplicationError;
	public double getDevelopmentOutput() throws ApplicationError;
    public int getMoneyOutput() throws ApplicationError;
    public int getMoneyWasted() throws ApplicationError;
	public double getMinimumEcologyOutput() throws ApplicationError;
	public double getMinimumEcologyPercent() throws ApplicationError;
	
	/**
	 * @return Production output + research output + ecology output + development output + money output
	 */
	public int getTotalOutput() throws ApplicationError;

	public double getTeraformingGrowth() throws ApplicationError;
	public double getPopulationGrowth() throws ApplicationError;
	public double getFactoriesGrowth() throws ApplicationError;
    
    public int getQtyColonistsOnrouteTo() throws ApplicationError;
    public int getColonistsEtaOnrouteTo() throws ApplicationError;

	/**
	 * @return The quantity of production accumulated so far producing the project.
	 */
	public int getProduction();
	
	public Weapons getWeapons() throws ApplicationError;
	public Shields getShields() throws ApplicationError;
	public Scanners getScanners() throws ApplicationError;
	
	/**
	 * The maximum quantity of people this planet can sustain. This is calculated from
	 * the size and terraform of the planet (see terraform below). However, player bonuses,
	 * and space/battle stations can also effect a planet's maximum population. 
	 */
	public double getMaxPopulation() throws ApplicationError;
	
	/**
	 * Qty People on this planet at this point in time. This would normaly be a value between
	 * zero and the the maximum population. However, this may be higher than the maximum 
	 * population - for example when a planet's terraform has been reduced due to bio weapons
	 * or industrial polution.
	 */
	public double getPopulation();
	
	/**
	 * The quality of the environment this planet provised. It is a fraction applied
	 * to the size of a planet that is used to calculate the maximum population this
	 * planet can support. For example if a planet was size 100 and has a terraform of
	 * 0.8 then it could support a population of 80. 
	 * Terraform quality is associated with a name e.g. Toxic is 0.2 or less, Utopia is
	 * 1.8 or higher, Terran is 1.0 to 1.1.
	 */
	public double getTeraforming();
	
	/**
	 * The planet size - its mass. How big the planet is. This is used to calculate the
	 * maximum population this planet can support (see terraform for details).
	 */
	public int getSize();
	
	public double getQtyFactories();
	public double getPercentEcology();
	public double getPercentDevelopment();
	public double getPercentResearch();
	public double getPercentProduction();
	public double getPercentMoney();
	public boolean getNotifiedTerraformComplete();
	public boolean getNotifiedFactoriesComplete();
	
    /**
     * The production bonus this planet enjoys. A value of -0.1 would imply a bonus of -10%
     * production on this planet while a value of 0.1 would imply a bonus of +10% production
     * on this planet. A value of 0.0 implies no bonus.
     */
	public double getProductionBonus();
	
    /**
     * The research bonus this planet enjoys. A value of -0.1 would imply a bonus of -10%
     * research on this planet while a value of 0.1 would imply a bonus of +10% research
     * on this planet. A value of 0.0 implies no bonus.
     */
	public double getResearchBonus();
	
    /**
     * The reproduction bonus this planet enjoys. A value of -0.1 would imply a bonus of -10%
     * reproduction on this planet while a value of 0.1 would imply a bonus of +10% reproduction
     * on this planet. A value of 0.0 implies no bonus.
     */
	public double getReproductionBonus();
	
    /**
     * The money bonus this planet enjoys. This is the quantity of extra money generated each turn
     * on this planet - due to valuable mineral resources like gold or diamonds.
     */
	public int getMoneyBonus();
	
	public boolean getAutoEcology();
	public boolean getAutoDevelopment();
	public boolean getVaryProduction();
	public boolean getVaryResearch();
	public int getInfluence();
	public int getQtyGroundDefense();
    public Orbital getOrbital() throws ApplicationError;
	public String getMiningColony();
    public boolean hasMiningColony();
    public Resource getResource();
    
    public int getMiningPopulation() throws ApplicationError;
    public int getMiningProduction() throws ApplicationError;
    public int getMiningResearch() throws ApplicationError;
    public int getMiningMoney() throws ApplicationError;

    public int getLostMiningProduction() throws ApplicationError;
    public int getLostMiningResearch() throws ApplicationError;
    public int getLostMiningMoney() throws ApplicationError;

  	/**
  	 * A planet is dominated if it is in a sector of space owned by another player
  	 * Some production is lost when a planet is dominated!
  	 */
  	public boolean isDominated();

    public boolean equals(PlanetDelegate other);  
    
    //
    // Functionality from Helper
    //
    
    public boolean hasOwner() throws ApplicationError;
    public Color getTextColor() throws ApplicationError;
    public int getImageSize() throws ApplicationError;
    public String getImageForPlanet() throws ApplicationError;
    
    /**
     * @return True iff this planet has an asteroid field around it. 
     */
	public boolean hasAsteroids();

    /**
     * The same as Planet.hasDefences()
     * @return True iff this planet has defences. 
     * @throws ApplicationError 
     */
	public boolean hasDefences() throws ApplicationError;

	
    /**
     * The same as Planet.hasNatives()
     * @return True iff this planet had natives at the start of the game (Natives are used when planet is colonised). 
     */
	public boolean hasNatives();

	public boolean isMiningColony() throws ApplicationError;
    public PlanetDelegate getMiningColonyDelegate() throws ApplicationError;
	public PlanetDelegate getOwnerOfMiningStation() throws ApplicationError;
	
	public void updateImages() throws ApplicationError;
	public String getImageForPlanet(int animationStage) throws ApplicationError;
	public String getPlanetTypeName();
	public String getMoneyBonusName();
	public String getProductionBonusName();
	public String getResearchBonusName();
	public String getFertilityBonusName();
	public OrbitVO getOrbit();
	public PlanetCrisisVO getCrisis();
	public boolean isWayPoint() throws ApplicationError;

    public int getMaintenanceCost() throws ApplicationError;
    public int getWeaponMaintenanceCost() throws ApplicationError;
    public int getShieldMaintenanceCost() throws ApplicationError;
    public int getScannerMaintenanceCost() throws ApplicationError;
    public int getOrbitalMaintenanceCost() throws ApplicationError;
    
    public String getWeaponIcon() throws ApplicationError;
    public String getShieldIcon() throws ApplicationError;
    public String getScannerIcon() throws ApplicationError;
    public String getOrbitalIcon() throws ApplicationError;

}