/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates;

import mot.common.bean.PlayerCharacterBasic;
import mot.common.bean.PlayerColor;
import mot.common.bean.SalvageList;
import mot.common.morals.MoralChoiceList;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.vo.StrategistEventVO;

public interface PlayerDelegate extends NamedObjectDelegate {
    public String getAlignment();
    public int getFunds();
	public int getBoralium();

    /**
     * @return True iff this player is a computer player or a monster player
     */
    public boolean isAutomated();
    
    /**
     * @return The color of this player
     */
    public PlayerColor getColor() throws ApplicationError;
    public String getColorName() throws ApplicationError;

    public boolean equals(PlayerDelegate other);
    
    /**
     * Salvage is collected from battles. It can be used to install on crafts a player is building
     * even if the player has not researched the salvaged technology. It reduces the time to build
     * a craft because less stuff has to be built. Alternatively, it can be converted to cash.
     * 
     * Will ALWAYS be null for other players
     * 
     * @return All the salvage this player has at this point in time.
     * @throws ApplicationError 
     */
    public SalvageList getSalvage() throws ApplicationError;
	
    /**
     * @return The character information for this player. 
     */
	public PlayerCharacterBasic getPlayerCharacter() throws ApplicationError;


	
	//
	// THESE VALUES NOT ALWAYS UP TO DATE BUT ALWAYS UP TO DATE FOR USER PLAYERS
	//
	

    /**
     * NOT kept up to date - use fetchEmpireDetails()
     * @return The total research work performed each turn by this player on research.
     * NOTE - This may not be the actual work performed at the end of the turn as settings change 
     * and work produced depends on many settings including the percentage output designated to 
     * research on each planet.
     */
    public int getTotalResearch() throws ApplicationError;


    /**
     * @return The bonus provided by planets between solar system in this empire
     * @throws ApplicationError
     */
    public double getTradeRouteBonus() throws ApplicationError;

    /**
     * @return Get the relationship definition between this player and the player other.
     * @throws ApplicationError
     */
    public RelationDelegate getRelation(PlayerDelegate other) throws ApplicationError;
    
	public boolean hasDiplomaticRelationsWith(PlayerDelegate other) throws ApplicationError;

    public String [] getPlayersWithTradeAgreements() throws ApplicationError;
    public String [] getPlayersWithResearchAgreements() throws ApplicationError;
    public String [] getPlayersAllianceTreaty() throws ApplicationError;

	//
	// THESE VALUES NOT ALWAYS UP TO DATE
	//


    /**
     * NOT kept up to date - use fetchDiplomacyDetails()
     */
    public int getQtySpiesEspionage(PlayerDelegate other) throws ApplicationError;
    public int getQtySpiesSabotage(PlayerDelegate other) throws ApplicationError;

    /**
     * NOT kept up to date - use fetchDiplomacyDetails()
     * @return Descriptions of all the events/actions this player has done that player 
     * other has remembered (and effect the relations).
     */
    public StrategistEventVO [] getStrategistEvents(PlayerDelegate other) throws ApplicationError;
	
    /**
     * NOT kept up to date - use fetchDiplomacyDetails()
     * @return Descriptions of all the events/actions this player has done that player 
     * other has remembered (and effect the relations).
     */
    //public DiplomaticHistoryVO getDiplomaticHistory(PlayerDelegate other) throws ApplicationError;
	
    /**
     * NOT kept up to date - use fetchDiplomacyDetails()
     */
	public int getSpyRecruitCosts();
	public double getSpyRecruitChance();
	public int getSpyRecruitTotal();
	public int getEspionageSuccessChance(String player) throws ApplicationError;
	public int getSabotageSuccessChance(String player) throws ApplicationError;
	public Technology getEspionageTarget(String player) throws ApplicationError;

    /**
     * NOT kept up to date - use fetchEmpireDetails()
     * @return The list of moral choices this player has adopted
     */
    public MoralChoiceList getMoralChoices() throws ApplicationError;
    
    /**
     * NOT kept up to date - use fetchEmpireDetails()
     */
    public int getTotalPopulation() throws ApplicationError;
    public int getTotalProduction() throws ApplicationError;

    /**
     * NOT kept up to date - use fetchEmpireDetails()
     */
    public int getTotalIncome(boolean includeTrade) throws ApplicationError;
    public int getTradeIncome() throws ApplicationError;
    
    /**
     * NOT kept up to date - use fetchEmpireDetails()
     */
	public int getTotalMaintenanceCost() throws ApplicationError;
	public int getSpiesMaintenanceCost() throws ApplicationError;
    public boolean isMonsterPlayer();
    
	public boolean isEnemy(String otherPlayerName) throws ApplicationError;
	public boolean isEnemy(String [] otherPlayerNames) throws ApplicationError;
	
	public boolean canAttack(PlanetDelegate planetDeligate, boolean mustHaveDefences) throws ApplicationError;
	public boolean canColonise(PlanetDelegate planetDeligate) throws ApplicationError;
	
	public PlanetDelegate[] getColonisablePlanets(PlanetDelegate[] planets) throws ApplicationError;
	public PlanetDelegate[] getAttackablePlanets(PlanetDelegate[] planets, boolean includeMiningColonies, boolean mustHaveDefences) throws ApplicationError;
	public PlanetDelegate[] getNonMiningColonies() throws ApplicationError;
	public PlanetDelegate[] getVisibleHabitablePlanets() throws ApplicationError;
	public PlanetDelegate[] getVisiblePlanets() throws ApplicationError;
	
	public double getPlanetMaintenanceFactor();
	
	public String[] getEnemyPlayers() throws ApplicationError;
	
}
