/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates;

import mot.common.bean.Location;
import mot.common.bean.StellarObjectGraphics;
import mot.common.util.ApplicationError;
import mot.common.vo.OrbitVO;
import mot.common.vo.StellarObjectVO;

public interface StellarObjectDelegate extends LabeledObjectDelegate {
	
    /**
     * Names are guaranteed to be unique.
     * @return The name of this stellar object.
     */
    public String getName();
    
    /**
     * @return The name of the bitmap used to display this stellar object on the galaxy map.
     */
    public StellarObjectGraphics getGraphics(boolean zoomedIn) throws ApplicationError;
    public String getGraphicsName() throws ApplicationError;
    
    /**
     * Note this is the location in the game and does not include any scaling (zooming)
     * that is a client side concern!
     * @return The location of this stellar object.
     */
	public Location getLocation();
	
	/**
	 * @return the date of the time the player visited this object. If the
     * player has not visited this object it will return -1.
	 */
	public int getLastVisitedDate() throws ApplicationError ;

    /**
     * @return true if player is the owner of this stellar object.
     */
	public boolean isOwnedBy(PlayerDelegate player) throws ApplicationError;

	/**
	 * @return true if this stellar object can be seen by this player at this point in time.
     * When a stellar object can be seen by a player then that player can double click on the
     * stellar object to view its details.
     * 
     * A player can view a stellar object if a) Its the owner b) Its within scanning range or
     * c) Has visited the object.
     * 
     * This should not be confused with if a planet is viewable. If a solar system is visible then
     * the planets may be viewed, but not the population/production/defence details. If a planet is 
     * viewable then all details of the planet may be viewed.
	 */
    public boolean isVisible() throws ApplicationError;

    /**
     * A stellar object may have more than one owner because planets with more than
     * one colony may have a single planet invaded and thus change owner! Currently
     * a player is not allowed to colonise a planet in a solar system owned by another
     * player, but this could change!  
     * @return The list of owners of this stellar object.
     */
    public String [] getOwners() throws ApplicationError;
	public PlayerDelegate[] getOwnerDelegates() throws ApplicationError;

    /**
     * @return The space station of outpost orbiting the sun or null if one does not exist.
     */
    public StellarObjectOrbitalDelegate getStellarObjectOrbital() throws ApplicationError;
    
    /**
     * @return TRUE if this stellar object contains a monster craft OR an ancient craft.
     * @throws ApplicationError 
     */
    public boolean isGuardedByMonster() throws ApplicationError;
    
    public boolean equals(StellarObjectDelegate other);

	public OrbitVO getOrbit();

	public void update(StellarObjectVO vo);

	/**
	 * This distance from your closest solar system to this solar system
	 */
	public int getRange(PlayerDelegate owner) throws ApplicationError;

	/**
	 * @return The name of background image for this object
	 * @throws ApplicationError 
	 */
	public String getImage() throws ApplicationError;

	public boolean isWayPoint() throws ApplicationError;

    
}