/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.cache;

import java.util.ArrayList;

import mot.client.deligates.BattleAutomatonDelegate;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.BattlePlanetDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.mot.MotBattleCraftDelegate;
import mot.client.deligates.mot.MotBattleDelegate;
import mot.client.deligates.mot.MotBattleDroidMineDelegate;
import mot.client.deligates.mot.MotBattleMineDelegate;
import mot.client.deligates.mot.MotBattleMissilesDelegate;
import mot.client.deligates.mot.MotBattleObjectDelegate;
import mot.client.deligates.mot.MotBattlePlanetDelegate;
import mot.client.deligates.mot.MotCraftDelegate;
import mot.client.sandbox.battle.BattleSandbox;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.bean.NameCraftStatePair;
import mot.common.bean.WreckageDetails;
import mot.common.event.BattleCraftLeaveEvent;
import mot.common.event.BattleCraftNewEvent;
import mot.common.event.BattleDamageRingEvent;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.BattleStateUpdateEventAdapter;
import mot.common.event.BattleStateUpdateMultiEventAdapter;
import mot.common.event.BattleStateUpdateSingleEventAdapter;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.LinkedEventBase;
import mot.common.event.MotEvent;
import mot.common.event.NewBattleEvent;
import mot.common.event.StateContainer;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.vo.BattleCraftSpeedUpdateVO;
import mot.common.vo.BattleCraftVO;
import mot.common.vo.BattleMineVO;
import mot.common.vo.BattleNextObjectToMoveVO;
import mot.common.vo.BattleObjectStateUpdateVO;
import mot.common.vo.BattleObjectWeaponStateUpdateVO;
import mot.common.vo.BattlePlanetVO;
import mot.common.vo.BattleStateVO;
import mot.common.vo.BattleStationVO;
import mot.common.vo.CraftVO;
import mot.common.vo.DroidMineVO;
import mot.common.vo.MissilesVO;
import mot.common.vo.ValueObject;

public class BattleDelegateCache {
	
	private MotBattleDelegate battle;
	
	private MotBattlePlanetDelegate _planet;
	private MotBattleCraftDelegate [] _crafts;
	private BattleAutomatonDelegate [] _automatons;
	
	private ArrayList<MotBattleObjectDelegate> content = new ArrayList<MotBattleObjectDelegate>();
	private ArrayList<MotBattleObjectDelegate> removedContent = new ArrayList<MotBattleObjectDelegate>();
	
	private ArrayList<AnimatedBattleAreaDetails> animatedBattleAreas;
	private AnimatedBattleAreaDetails[] _animatedBattleAreas;
	
	private BattleSandbox battleSandbox;

	public BattleDelegateCache() {
	}
	
	public void setup(NewBattleEvent event) {
		this.battle = new MotBattleDelegate(event, this);
		ArrayList<ValueObject> state = event.getState();
		if (state != null) {
			for (ValueObject v : state) {
				addOrUpdate(event, v);
			}
		}
		AnimatedBattleAreaDetails[] abad = event.getAnnimatedBattleAreas();
		if (abad != null) {
			animatedBattleAreas = new ArrayList<AnimatedBattleAreaDetails>();
			for (int i = 0; i < abad.length; i++) {
				animatedBattleAreas.add(abad[i]);
			}
		}
	}
	
	public void addOrUpdate(StateContainer container, ValueObject v) {
		_planet = null;
		_crafts = null;
		_automatons = null;
		_animatedBattleAreas = null;
		battleSandbox = null;
		if (v instanceof BattleCraftVO) {
			BattleCraftVO vo = (BattleCraftVO) v;
			CraftDelegate craft = DeligateCacheManager.getCraft(vo.getName());
			if (craft == null && container != null) {
				ValueObject nvo = container.getState(vo.getName());
				if (nvo != null && nvo instanceof CraftVO) {
					CraftVO craftVO = (CraftVO) nvo;
					craft = new MotCraftDelegate(craftVO);
					DeligateCacheManager.addMoo(craft);
					// things like sentinels and battle stations do not have existing delegates
					Logger.logDebug("Create Craft Delegate for BattleCraft "+vo.getName());
				}
			}
			if (vo instanceof BattleStationVO) {
				MotBattleCraftDelegate delegate = (MotBattleCraftDelegate) getBattleCraft(vo.getName());
				if (delegate == null) {
					delegate = new MotBattleCraftDelegate((BattleStationVO)vo, craft);
					content.add(delegate);
				} else {
					delegate.setBattleObjectValue(vo);
				}
			} else {
				MotBattleCraftDelegate delegate = (MotBattleCraftDelegate) getBattleCraft(vo.getName());
				if (delegate == null) {
					delegate = new MotBattleCraftDelegate(vo, craft);
					content.add(delegate);
				} else {
					delegate.setBattleObjectValue(vo);
				}
			}
		} else if (v instanceof BattleObjectStateUpdateVO) {
			BattleObjectStateUpdateVO vo = (BattleObjectStateUpdateVO) v;
			MotBattleCraftDelegate delegate = (MotBattleCraftDelegate) getBattleCraft(vo.getName());
			if (delegate != null) {
				delegate.setBattleObjectValue(vo);
			} else {
				Logger.logWarning("MISSING BattleCraft Delegate for "+vo.getName());
			}
		} else if (v instanceof MissilesVO) {
			MissilesVO vo = (MissilesVO) v;
			MotBattleMissilesDelegate delegate = (MotBattleMissilesDelegate) getBattleAutomaton(vo.getName());
			if (delegate == null) {
				delegate = new MotBattleMissilesDelegate(vo);
				content.add(delegate);
			} else {
				delegate.setBattleObjectValue(vo);
			}
		} else if (v instanceof DroidMineVO) {
			DroidMineVO vo = (DroidMineVO) v;
			MotBattleDroidMineDelegate delegate = (MotBattleDroidMineDelegate) getBattleAutomaton(vo.getName());
			if (delegate == null) {
				delegate = new MotBattleDroidMineDelegate(vo);
				content.add(delegate);
			} else {
				delegate.setBattleObjectValue(vo);
			}
		} else if (v instanceof BattleMineVO) {
			BattleMineVO vo = (BattleMineVO) v;
			MotBattleMineDelegate delegate = (MotBattleMineDelegate) getBattleAutomaton(vo.getName());
			if (delegate == null) {
				delegate = new MotBattleMineDelegate(vo);
				content.add(delegate);
			} else {
				delegate.setBattleObjectValue(vo);
			}
		} else if (v instanceof BattlePlanetVO) {
			BattlePlanetVO vo = (BattlePlanetVO) v;
			MotBattlePlanetDelegate delegate = (MotBattlePlanetDelegate) getPlanet();
			if (delegate == null) {
				delegate = new MotBattlePlanetDelegate(vo);
				content.add(delegate);
			} else {
				delegate.setBattleObjectValue(vo);
			}
		} else if (v instanceof BattleStateVO) {
			BattleStateVO bs = (BattleStateVO) v;
			battle.setBattleState(bs);
		} else if (v instanceof BattleObjectWeaponStateUpdateVO) {
			BattleObjectWeaponStateUpdateVO ws = (BattleObjectWeaponStateUpdateVO) v;
			BattleObjectDelegate bo = getBattleObject(ws.getSource());
			if (bo != null && bo instanceof MotBattleObjectDelegate) {
				((MotBattleObjectDelegate)bo).update(ws.getState());
			}
		}  else if (v instanceof BattleNextObjectToMoveVO) {
			BattleNextObjectToMoveVO nvo = (BattleNextObjectToMoveVO) v;
			battle.setNextObject(nvo);
		}  else if (v instanceof BattleCraftSpeedUpdateVO) {
			BattleCraftSpeedUpdateVO vo = (BattleCraftSpeedUpdateVO) v;
			MotBattleCraftDelegate bc = (MotBattleCraftDelegate) getBattleCraft(vo.getCraft());
			if (bc != null && bc.getBattleCraftValue() != null) {
				bc.getBattleCraftValue().setSpeed(vo.getSpeed());
			}
		}  else if (v instanceof AnimatedBattleAreaDetails) {
			AnimatedBattleAreaDetails abad = (AnimatedBattleAreaDetails) v;
			AnimatedBattleAreaDetails r = findAnimatedBattleArea(abad);
			if (r != null) {
				r.update(abad);
			} else {
				if (animatedBattleAreas == null) animatedBattleAreas = new ArrayList<AnimatedBattleAreaDetails>();
				animatedBattleAreas.add(abad);
			}
		}
	}

	private AnimatedBattleAreaDetails findAnimatedBattleArea(AnimatedBattleAreaDetails abad) {
		if (animatedBattleAreas == null) {
			return null;
		}
		for (AnimatedBattleAreaDetails v : animatedBattleAreas) {
			if (v.getClass() == abad.getClass() && v.equals(abad)) {
				return v;
			}
		}
		return null;
	}
	
	public boolean isDeadBattleObject(String name) {
		return find(name, false, true) != null;
	}

	private MotBattleObjectDelegate find(String name) {
		return find(name, true, true);
	}
	
	private MotBattleObjectDelegate find(String name, boolean includeAlive, boolean includeDead) {
		if (includeAlive && content != null) {
			for (MotBattleObjectDelegate c : content) {
				if (Comp.same(name, c.getName())) {
					return c;
				}
			}
		}
		if (includeDead && removedContent != null) {
			for (MotBattleObjectDelegate c : removedContent) {
				if (Comp.same(name, c.getName())) {
					return c;
				}
			}
		}
		return null;
	}

	public BattlePlanetDelegate getPlanet() {
		if (_planet == null && _crafts == null && content != null) {
			for (MotBattleObjectDelegate c : content) {
				if (c instanceof MotBattlePlanetDelegate) _planet = (MotBattlePlanetDelegate) c;
			}
		}
		return _planet;
	}

	public BattleCraftDelegate[] getCrafts() {
		if (_crafts == null && content != null) {
			ArrayList<MotBattleCraftDelegate> r = null;
			for (MotBattleObjectDelegate c : content) {
				if (c instanceof MotBattleCraftDelegate) {
					if (r == null) r = new ArrayList<MotBattleCraftDelegate>();
					r.add((MotBattleCraftDelegate)c);
				} else if (c instanceof MotBattlePlanetDelegate) {
					_planet = (MotBattlePlanetDelegate) c;
				}
			}
			if (r == null) {
				return null;
			} else {
				_crafts = (MotBattleCraftDelegate[]) r.toArray(new MotBattleCraftDelegate[r.size()]);
			}
		}
		return _crafts;
	}

	public BattleAutomatonDelegate[] getAutomatons() {
		if (_automatons == null && content != null) {
			ArrayList<BattleAutomatonDelegate> r = new ArrayList<BattleAutomatonDelegate>();
			for (MotBattleObjectDelegate c : content) {
				if (c instanceof BattleAutomatonDelegate) r.add((BattleAutomatonDelegate)c);
			}
			_automatons = (BattleAutomatonDelegate[]) r.toArray(new BattleAutomatonDelegate[r.size()]);
		}
		return (_automatons != null && _automatons.length == 0) ? null : _automatons;
	}

	public AnimatedBattleAreaDetails[] getAnnimatedBattleAreas() {
		if (_animatedBattleAreas ==  null && animatedBattleAreas != null && animatedBattleAreas.size() > 0) {
			_animatedBattleAreas = animatedBattleAreas.toArray(new AnimatedBattleAreaDetails[animatedBattleAreas.size()]);
		}
		return _animatedBattleAreas;
	}
	
	private void remove(MotBattleObjectDelegate v) {
		if (content.contains(v)) {
			_planet = null;
			_crafts = null;
			_automatons = null;
			content.remove(v);
			removedContent.add(v);
		}
	}
	
	private void remove(String destroyedName) {
		MotBattleObjectDelegate v = find(destroyedName, true, false);
		if (v != null) {
			remove(v);
		}
	}

	private void remove(AnimatedBattleAreaDetails abad) {
		AnimatedBattleAreaDetails v = findAnimatedBattleArea(abad);
		if (v != null) {
			animatedBattleAreas.remove(v);
			_animatedBattleAreas = null;
		}
	}

	public ImageObjectDelegate getImageObject(String name) {
		return find(name);
	}
	
	public BattleObjectDelegate getBattleObject(String name) {
		return find(name);
	}

	public BattleCraftDelegate getBattleCraft(String name) {
		return (BattleCraftDelegate) find(name);
	}

	public BattleCraftDelegate [] getBattleCraft(String name []) {
		if (name == null) return null;
		BattleCraftDelegate [] r = new BattleCraftDelegate[name.length];
		for (int i = 0; i < r.length; i++) {
			r[i] = getBattleCraft(name[i]);
		}
		return r;
	}

	public BattleAutomatonDelegate getBattleAutomaton(String name) {
		return (BattleAutomatonDelegate) find(name);
	}

    public ImageObjectDelegate getObjectAt(Location l, boolean preferPrimaryObjects) throws ApplicationError {
    	return getObjectAt(l.getX(), l.getY(), preferPrimaryObjects);
    }
    
    public ImageObjectDelegate getObjectAt(int x, int y, boolean preferPrimaryObjects) throws ApplicationError {
		ImageObjectDelegate r = null; 
		if (preferPrimaryObjects) {
			r = getBattleObjectAt(x,y);
			if (r == null) {
				r = getObjectAt(x,y);
			}
		} else {
			r = getObjectAt(x,y);
		}
		return r;
    }

	public ImageObjectDelegate getObjectAt(int x, int y) throws ApplicationError {
		BattleAutomatonDelegate[] automatons = getAutomatons();
        if (automatons != null && automatons.length > 0) {
            for (int i=0; i<automatons.length; i++) {
                BattleAutomatonDelegate automaton = automatons[i];
                if (automaton.getArea().contains(x,y)) {
                    return automaton;
                }
            }
        }
        return getBattleObjectAt(x,y);
	}

	public BattleObjectDelegate getBattleObjectAt(int x, int y) throws ApplicationError {
        if (content != null) {
            for (MotBattleObjectDelegate c : content) {
                Area area = c.getArea();
				if (area.contains(x,y)) {
                    return c;
                }
            }
        }
        return null;
	}

	public void update(StateContainer sc) {
		if (sc != null) {
			ValueObject[] sua = sc.getStateUpdates();
			if (sua != null) {
				for (int i = 0; i < sua.length; i++) {
					addOrUpdate(sc, sua[i]);
				}
			}
		}
	}
	
	public void update(BattleStateUpdateEventAdapter vo) throws ApplicationError {
		battleSandbox = null;
		if (vo instanceof BattleStateUpdateMultiEventAdapter) {
			BattleStateUpdateMultiEventAdapter vom = (BattleStateUpdateMultiEventAdapter)vo;
			StateContainer container = (vo instanceof StateContainer) ? (StateContainer)vo : null;
			NameCraftStatePair[] states = vom.fetchStateUpdates();
			ValueObject[] vos = vom.fetchValueObjectUpdates();
			if (states != null) {
				for (int i = 0; i < states.length; i++) {
					_update(states[i].getName(), states[i].getState(), null, container);
				}
			}
			if (vos != null) {
				for (int i = 0; i < vos.length; i++) {
					if (vom instanceof StateContainer) {
						StateContainer sc = (StateContainer)vom;
						addOrUpdate(sc, vos[i]);
					} else {
						addOrUpdate(null, vos[i]);
					}
				}
			}
		} else if (vo instanceof BattleStateUpdateSingleEventAdapter) {
			BattleStateUpdateSingleEventAdapter vos = (BattleStateUpdateSingleEventAdapter)vo;
			StateContainer container = (vo instanceof StateContainer) ? (StateContainer)vo : null;
			_update(vos.getUpdatedObjectName(), vos.fetchStateUpdate(), vos.fetchValueObjectUpdate(), container);
		}
		if (vo instanceof StateContainer) {
			StateContainer svo = (StateContainer)vo;
			ValueObject[] sua = svo.getStateUpdates();
			if (sua != null) {				
				for (int i = 0; i < sua.length; i++) {
					try {
						addOrUpdate(svo, sua[i]);
					} catch (Exception e) {
						Logger.logError("Unknown Cache Update "+sua[i].getClass().getName(), e);
					}
				}
			}
		}
		if (vo instanceof LinkedEventBase) {
			MotEvent[] ea = ((LinkedEventBase)vo).getLinkedEvents();
			if (ea != null) {
				for (int i = 0; i < ea.length; i++) {
					if (ea[i] instanceof BattleStateUpdateEventAdapter) {
						update((BattleStateUpdateEventAdapter)ea[i]);
					}
				}
			}
		}
		
		if (vo instanceof BattleDestroyedEvent) {
			BattleDestroyedEvent event = (BattleDestroyedEvent) vo;
			String destroyedName = event.getUpdatedObjectName();
			if (destroyedName != null) {
				remove(destroyedName);
			}
		} else if (vo instanceof BattleCraftLeaveEvent) {
			BattleCraftLeaveEvent event = (BattleCraftLeaveEvent) vo;
			String leaveName = event.getCraft();
			if (leaveName != null) {
				remove(leaveName);
			}
		} else if (vo instanceof BattleWeaponFireEvent) {
			BattleWeaponFireEvent event = (BattleWeaponFireEvent) vo;
			if (event.getWeaponTechnology().isSalvageBeam() && event.getData() != null && event.getData() instanceof WreckageDetails) {
				WreckageDetails wd = (WreckageDetails) event.getData();
				remove(wd);
			}
		} else if (vo instanceof BattleCraftNewEvent) {
			BattleCraftNewEvent event = (BattleCraftNewEvent) vo;
			String updatedObjectName = event.getUpdatedObjectName();
			if (updatedObjectName != null) {
				MotBattleObjectDelegate o = find(updatedObjectName);
				if (o != null && o instanceof MotBattleCraftDelegate) {
					MotBattleCraftDelegate cd = (MotBattleCraftDelegate) o;
					ValueObject[] stateUpdates = event.getStateUpdates();
					CraftVO craftVO = null;
					if (stateUpdates != null) {						
						for (int i = 0; i < stateUpdates.length; i++) {
							if (stateUpdates[i] instanceof CraftVO) craftVO = (CraftVO) stateUpdates[i];
						}
						if (craftVO != null) {
							MotCraftDelegate craft = new MotCraftDelegate(craftVO);
							cd.setCraft(craft);
						}
					}
				}
			}
		} else if (vo instanceof BattleDamageRingEvent) {
			BattleDamageRingEvent event = (BattleDamageRingEvent) vo;
			BattleObjectDelegate bo = getBattleObject(event.getSource());
			if (bo != null && bo instanceof BattleAutomatonDelegate) {
				remove(event.getSource());
			}
		}
	}

	private void _update(String updatedObjectName, CraftState stateUpdate, ValueObject valueObjectUpdate, StateContainer container) {
		if (updatedObjectName != null) {
			MotBattleObjectDelegate o = find(updatedObjectName);
			if (o == null) {
				if (valueObjectUpdate != null) {
					addOrUpdate(null, valueObjectUpdate);
				} else {
					// If this contains a value object for this value then we have been sent an update
					// This occurs for mines that are not visible on the battle map - can get an event for a mine that is not 
					// in the delegate cache.
					ValueObject vo = (container != null) ? container.getState(updatedObjectName) : null;
					if (vo == null) {
						Logger.logError("Object Not Found in Cache "+updatedObjectName);
					}
				}
			} else if (valueObjectUpdate != null) {
				o.setBattleObjectValue(valueObjectUpdate);
			} else if (stateUpdate != null){
				if (stateUpdate.isDestroyed() && !(o instanceof BattlePlanetDelegate) ) {
					o.setState(stateUpdate);
					remove(o);
				} else {
					o.setState(stateUpdate);
				}
			}
		}
		
	}

	public void update(StateContainer container, ValueObject vo) {
		addOrUpdate(container, vo);
	}
	
	public MotBattleDelegate getBattle() {
		return battle;
	}

	public void setBattle(MotBattleDelegate battle) {
		this.battle = battle;
	}

	public BattleSandbox getBattleSandbox() {
		if (battleSandbox == null) {
			battleSandbox = new BattleSandbox(this);
		}
		return battleSandbox;
	}

}
