/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import java.net.URL;

import mot.client.animation.MovementRegister;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.common.bean.CraftGraphics;
import mot.common.bean.PlayerColor;
import mot.common.bean.Size;
import mot.common.factory.GraphicsFactory;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.vo.BattleCraftVO;
import mot.common.vo.BattleObjectVO;
import mot.common.vo.BattleStationVO;
import mot.common.vo.ImageObjectVO;
import mot.common.vo.ValueObject;

public class MotBattleCraftDelegate extends MotBattleObjectDelegate implements BattleCraftDelegate {

	private CraftGraphics craftGraphics;
	private BattleCraftVO value;	
	private CraftDelegate craft;
	
	
	public MotBattleCraftDelegate() {
	}
	
	public MotBattleCraftDelegate(BattleCraftVO v, CraftDelegate c) {
		value = v;
		craft = c;
		if (craft == null) {
			Logger.logWarning("MotBattleCraftDelegate() No Craft Delegate for "+value);
		}
	}

	protected BattleObjectVO getBattleObjectValue() { return value;	}
	protected ImageObjectVO getImageObjectValue() {	return value; }	
	public BattleCraftVO getBattleCraftValue() { return value;	}
	public void setBattleObjectValue(ValueObject v) { value = (BattleCraftVO) v; }
	
	public void flushCache() {
		super.flushCache();
		craftGraphics = null;
		if (craft != null) {
			craft.flushImageCache();
		}
	}

	public CraftGraphics getCraftGraphics() throws ApplicationError {
    	if (craftGraphics == null) {
    		if (craft != null) {
		    	String raceName = value.getOwner();
		    	String hullName = craft.getCraftDesign().getHullName();
		    	craftGraphics = GraphicsFactory.getRaceGraphicsList().getHullGraphics(raceName, hullName);
    		} else {
		    	craftGraphics = GraphicsFactory.getRaceGraphicsList().getDefaultHullGraphics();
    		}
    	}
		return craftGraphics;
	}
	
	public Size getImageSize() throws ApplicationError {
		if (craft == null) {
			return null;
		} else {
			return craft.getImageSize();
		}
	}
	
	public String getImage() throws ApplicationError {
		PlayerDelegate owner = getOwnerDelegate();
    	PlayerColor pc = owner.getColor();
    	return getImage(pc, null);
	}

	public String getImage(PlayerColor pc, Boolean isVisible) throws ApplicationError {
		CraftGraphics craftGraphics = getCraftGraphics();
    	
        String colorName  = "";
        if (pc != null) colorName = pc.getName();

        String picture;
        if (isVisible != null) {
        	if (isVisible) {
        		picture = craftGraphics.getPictureNormal();
        	} else {
        		picture = craftGraphics.getPictureCloaked();
        	}
        } else {
        	if (isVisible()) {
        		picture = craftGraphics.getPictureNormal();
        	} else {
        		picture = craftGraphics.getPictureCloaked();
        	}
        }
		String image = CraftGraphics.CRAFT_IMAGE_FOLDER+picture + "_" + colorName + craftGraphics.getFileExtension();
		
    	URL fileUrl = ClassLoader.getSystemResource(image);
        if (fileUrl == null) {
            image = CraftGraphics.CRAFT_IMAGE_FOLDER+picture + craftGraphics.getFileExtension();
        }
    	
       	return image;
	}

	public double getMoveRemainingAsPercent() {
		return value.getState().getMoveRemainingAsPercent();
	}

	public CraftDelegate getCraft() throws ApplicationError {
		return craft;
	}

	public boolean isVisible() throws ApplicationError {
		String playerName = getPlayerName();
		String[] cloakedToPlayerNames = value.getState().getCloakedToPlayerNames();
		return !Comp.in(playerName, cloakedToPlayerNames);
	}

	public boolean isAggressor() {
		return value.isAggressor();
	}

	public boolean isHologram() {
		return value.isHologram();
	}

	public boolean isOrbital() {
		return value.isOrbital();
	}

    public int getAnimationRotation() throws ApplicationError {
        if (value instanceof BattleStationVO && ((BattleStationVO)value).isOrbital()) {
            return MovementRegister.getRotation();
        } else {
            return 0;
        }
    }

	public void setCraft(CraftDelegate craft) {
		this.craft = craft;
	}

	@Override
	public int getSpeed() {
		if (value.getSpeed() == null) {
			return 0;
		} else {
			return value.getSpeed().getSpeed();
		}
	}

	@Override
	public int getTurningSpeed() {
		if (value.getSpeed() == null) {
			return 0;
		} else {
			return value.getSpeed().getTurningSpeed();
		}
	}



}
