/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import mot.client.deligates.BattleAutomatonDelegate;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.BattlePlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.BattleDelegateCache;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.NameList;
import mot.common.event.NewBattleEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.BattleNextObjectToMoveVO;
import mot.common.vo.BattleStateVO;
import mot.common.vo.BattleVO;
import mot.common.vo.OrbitVO;

public class MotBattleDelegate implements BattleDelegate {
	
	private BattleVO value;
	private BattleStateVO battleState;
	private BattleNextObjectToMoveVO nextObject;
	private BattleDelegateCache cache;
	
	public MotBattleDelegate(NewBattleEvent battle, BattleDelegateCache cache) {
		this.value = battle.getBattle();
		this.battleState = battle.getBattleState();
		this.cache = cache;
	}

	public BattleVO getBattleValue() {
		return value;
	}
	
	public BattleStateVO getBattleState() {
		return battleState;
	}

	@Override
	public String getName() {
		BattleVO value = getBattleValue();
		return value.getName();
	}

	@Override
	public OrbitVO getOrbit() {
		BattleVO value = getBattleValue();
		return value.getOrbit();
	}

	@Override
	public BattlePlanetDelegate getPlanet() throws ApplicationError {
		return cache.getPlanet();
	}

	@Override
	public BattleCraftDelegate[] getCrafts() throws ApplicationError {
		return cache.getCrafts();
	}

	@Override
	public BattleAutomatonDelegate[] getAutomatons() throws ApplicationError {
		return cache.getAutomatons();
	}

	@Override
	public AnimatedBattleAreaDetails[] getAnnimatedBattleAreas() throws ApplicationError {
		return cache.getAnnimatedBattleAreas();
	}

	@Override
	public int getTurn() throws ApplicationError {
		BattleStateVO value = getBattleState();
		return value.getTurn();
	}

	@Override
	public boolean hasEnded() throws ApplicationError {
		BattleStateVO value = getBattleState();
		return value.isHasEnded();
	}

	@Override
	public boolean isWinner(PlayerDelegate player) throws ApplicationError {
		BattleStateVO value = getBattleState();
		String[] winners = value.getWinners();
		return player != null && Comp.in(player.getName(), winners);
	}

	@Override
	public boolean equals(BattleDelegate other) throws ApplicationError {
		BattleVO value = getBattleValue();
		return Comp.same(value.getName(), other.getName());
	}

	/**
	 * @return List of player names that are the attackers in this battle.
	 * @throws ApplicationError
	 */
	public NameList getAggressors() throws ApplicationError {
		BattleCraftDelegate[] crafts = getCrafts();
		if (crafts == null) return null;
		NameList r = null;
		for (int i = 0; i < crafts.length; i++) {
			boolean isAgressor = crafts[i].isAggressor();
			if (isAgressor) {
				String ownerName = crafts[i].getOwner();
				if (r == null) {
					r = new NameList();
				}
				if(!r.contains(ownerName)) {
					r.add(ownerName);
				}
			}
		}
		return r;
	}

	/**
	 * @return List of player names that were attacked in this battle.
	 * @throws ApplicationError
	 */
	public NameList getVictims() throws ApplicationError {
		BattleCraftDelegate[] crafts = getCrafts();
		if (crafts == null) return null;
		NameList r = null;
		for (int i = 0; i < crafts.length; i++) {
			boolean isAgressor = crafts[i].isAggressor();
			if (!isAgressor) {
				String ownerName = crafts[i].getOwner();
				if (r == null) {
					r = new NameList();
				}
				if(!r.contains(ownerName)) {
					r.add(ownerName);
				}
			}
		}
		return r;
	}

	public String getBattleName() {
		if (getOrbit() == null) return null;
		return "Battle of "+((getOrbit().getPlanet() != null) ? getOrbit().getPlanet() : getOrbit().getStellarObject());
	}

	@Override
	public BattleDelegateCache getDelegateCache() {
		return cache;
	}

	@Override
	public boolean containsStartingPlayer(PlayerDelegate owner) {
		return Comp.in(owner.getName(), value.getStartingPlayers());
	}
	
	@Override
    public BattleObjectDelegate getNextObjectToMove() throws ApplicationError {
		if (nextObject == null) {
			return null;
		}
		String craftName = nextObject.getNextObject();
		BattleObjectDelegate craft = cache.getBattleObject(craftName);
		return craft;
	}

	@Override
	public boolean inNebula() {
		return value.isInNebula();
	}

	public void setBattleState(BattleStateVO battleState) {
		this.battleState = battleState;
	}

	public BattleVO getValue() {
		return value;
	}

	public void setValue(BattleVO value) {
		this.value = value;
	}

	public BattleNextObjectToMoveVO getNextObject() {
		return nextObject;
	}

	public void setNextObject(BattleNextObjectToMoveVO nextObject) {
		this.nextObject = nextObject;
	}

}
