/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import mot.client.deligates.BattleDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.BattleObjectVO;
import mot.common.vo.BattleObjectWeaponStateVO;
import mot.common.vo.ValueObject;
import mot.common.vo.WeaponInstanceVO;

public abstract class MotBattleObjectDelegate extends MotImageObjectDelegate implements BattleObjectDelegate {

	protected abstract BattleObjectVO getBattleObjectValue();
	public abstract void setBattleObjectValue(ValueObject valueObjectUpdate);
	
	private transient PlayerDelegate owner;
	private transient BattleObjectDelegate delegate;
	
	public PlayerDelegate getOwnerDelegate() {
		if (owner == null) {
			BattleObjectVO value = getBattleObjectValue();
			if (value.getOwner() != null) {
				owner = DeligateCacheManager.getPlayer(value.getOwner());
			}
		}
		return owner;
	}
	
	public BattleObjectDelegate getBattleObjectDelegate() throws ApplicationError {
		if (delegate == null) {
			BattleObjectVO value = getBattleObjectValue();
			delegate = getBattle().getDelegateCache().getBattleObject(value.getName());
		}
		return delegate;
	}
	
	public void flushCache() {
		owner = null;
	}
	
	public boolean isSelectableBy(PlayerDelegate player) throws ApplicationError {
        return Comp.same(getOwner(), player.getName());
	}

	public String getBattleName() {
		BattleObjectVO value = getBattleObjectValue();
		return value.getBattle();
	}

	private BattleDelegate battle;
	public BattleDelegate getBattle() {
		if (battle == null) {
			battle = DeligateCacheManager.getBattle(getBattleName());
		}
		return battle;
	}

	public Location getLocation() {
		BattleObjectVO value = getBattleObjectValue();
		return value.getState().getLocation();
	}

	public int getDirection() throws ApplicationError {
		BattleObjectVO value = getBattleObjectValue();
		return value.getState().getDirection();
	}
	
	public String getOwner() throws ApplicationError {
		BattleObjectVO value = getBattleObjectValue();
		return value.getOwner();
	}

	public boolean isOwnedBy(PlayerDelegate p) throws ApplicationError {
		BattleObjectVO value = getBattleObjectValue();
        return Comp.same(value.getOwner(), p.getName());
	}

	public boolean isControlledBy(PlayerDelegate p) throws ApplicationError {
		BattleObjectVO value = getBattleObjectValue();
        return Comp.same(value.getController(), p.getName());
	}

	public WeaponInstanceVO [] getWeapons() throws ApplicationError {
		BattleObjectVO value = getBattleObjectValue();
		return value.getWeaponState().getWeapons();
	}
	
	public WeaponInstanceVO getWeapon(Weapon w) throws ApplicationError {
		BattleObjectVO value = getBattleObjectValue();
		WeaponInstanceVO[] weapons = value.getWeaponState().getWeapons();
		if (weapons == null) return null;
		for (int i = 0; i < weapons.length; i++) {
			if (weapons[i].getWeapon() == w) {
				return weapons[i];
			}
		}
		return null;
	}

	public CraftState getState() {
		BattleObjectVO value = getBattleObjectValue();
		return value.getState();
	}

	public void setState(CraftState state) {
		if (state != null) {
			BattleObjectVO value = getBattleObjectValue();
			value.setState(state);
		}
	}
	
	public boolean isDestroyed() {
		BattleObjectVO value = getBattleObjectValue();
		return value.isDestroyed();
	}

	public boolean hasFightersToLaunch() {
		BattleObjectVO value = getBattleObjectValue();
		return value.getWeaponState().isHasFightersToLaunch();
	}

	public WeaponInstanceVO getWeaponInstance(BattleObjectDelegate source, Weapon weapon) throws ApplicationError {
		WeaponInstanceVO[] weapons = source.getWeapons();
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
            	if (weapons[i].getWeapon() == weapon) {
            		return weapons[i];
            	}
            }
        }
        return null;
	}

    public int getAnimationRotation() throws ApplicationError {
        return 0;
    }

	public void update(BattleObjectWeaponStateVO weaponState) {
		owner = null;
		BattleObjectVO value = getBattleObjectValue();
		value.setWeaponState(weaponState);
	}
}
