/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import java.util.ArrayList;

import mot.client.bean.CraftGraphicsWrapper;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.deligates.debug.DeligateDebugState;
import mot.common.bean.CraftGraphics;
import mot.common.bean.PlayerColor;
import mot.common.bean.Size;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.designs.WeaponBaysDesign;
import mot.common.factory.GraphicsFactory;
import mot.common.technology.Engine;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.vo.CraftVO;
import mot.common.vo.MoveableOwnedObjectVO;
import mot.common.vo.OrbitVO;

public class MotCraftDelegate extends MotMoveableOwnedObjectDelegate implements CraftDelegate {

	private CraftVO value;
	
	private transient CraftGraphics craftGraphics;
	private transient String image = null;

	public MotCraftDelegate() {
	}
	
	public MotCraftDelegate(CraftVO vo) {
		this.value = vo;
	}

	@Override
	public CraftGraphics getCraftGraphics() throws ApplicationError {
    	if (craftGraphics == null) {
	    	String raceName = value.getOwner();
	    	String hullName = value.getDesign().getHullName();
	    	craftGraphics = GraphicsFactory.getRaceGraphicsList().getHullGraphics(raceName, hullName);
    	}
       	return craftGraphics;
    }

	@Override
	public Size getImageSize() throws ApplicationError {
		return getCraftGraphics().getSize();
	}

	@Override
    public String getImage() throws ApplicationError {
    	if (craftGraphics == null) {
	    	craftGraphics = getCraftGraphics();
    	}
        if (image == null) {
        	PlayerDelegate owner = DeligateCacheManager.getPlayer(value.getOwner());
        	PlayerColor pc = (owner != null) ? owner.getColor() : null;
        	if (pc == null) {
        		pc = PlayerColor.monsterColor;
        	}
            image = CraftGraphicsWrapper.getImage(craftGraphics, pc);
        }
    	
       	return image;
    }

	public String get3DImage() throws ApplicationError {
    	if (craftGraphics == null) {
	    	craftGraphics = getCraftGraphics();
    	}
		String picture3d = craftGraphics.getPicture3d();
		if (picture3d == null) {
			return null;
		} else {
			return CraftGraphics.CRAFT_IMAGE_FOLDER + picture3d;
		}
	}

	@Override
    public boolean canColonise(PlanetDelegate pd) throws ApplicationError {
		CraftDesign design = value.getDesign();
    	Specials specials = design.getSpecialTechs();
    	if (specials != null) {
    		Special colonyModule = specials.getBestCanColonise();
    		if (colonyModule != null) {
    			return colonyModule.getCanColoniseRating() <= pd.getTeraforming();
    		}
    	}
    	return false;
    }

	@Override
	public PlanetDelegate[] getColonisablePlanets(PlanetDelegate[] planets, boolean includeColonies) throws ApplicationError {
		ArrayList r = null;
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (canColonise(planets[i])) {
					if (includeColonies || !planets[i].hasOwner()) {
						if (r == null) r = new ArrayList();
						r.add(planets[i]);
					}
				}
			}
		}
		if (r == null) {
			return null;
		} else {
			return (PlanetDelegate[]) r.toArray(new PlanetDelegate[r.size()]);
		}
	}
	
	private static final int MAX_QTY_IN_LONG_DESCRIPTION = 4;
	private static final int MAX_QTY_IN_SHORT_DESCRIPTION = 2;

	@Override
	public String getDescription(int maxLines, boolean withName, boolean withEta) throws ApplicationError {
		CraftDesign design = value.getDesign();
		Engine engine = design.getEngineTech();
		Shields shields = design.getShieldTechs();
		WeaponBaysDesign weaponBays = design.getWeaponBays();
		StringBuffer description = new StringBuffer("<html>");
		
		if (withName) {
			description.append("Craft "+value.getName());
		}
		
		if (withEta) {
			String enrouteDescription = getEnrouteDescription();
			if (enrouteDescription != null) {
				description.append(" ");
				description.append(enrouteDescription);
			}
		}

		if (withName) {
			description.append(" : ");
		}

		if (engine != null) {
			description.append("Engine "+engine.getName()+" Speed " +engine.getWarpSpeed()+"/"+engine.getImpulseSpeed());
		}
		
		if (shields != null && shields.getQuantity() > 0) {
			if (maxLines > 2) {
				description.append("<br>");
			} else {
				description.append(" ");
			}
			description.append("Shields " + shields.getNames()+" Protection " + Formatter.whole(shields.getDamageBlocked())+"/"+Formatter.whole(shields.getPower()));
		}
		
		if (weaponBays != null && weaponBays.getQty() > 0) {
			if (maxLines > 1) {
				description.append("<br>Weapons ");
			} else {
				description.append(" Weapons ");
			}
			Weapons weapons = weaponBays.getBaysAsArray()[0].getWeaponsList();
			if (maxLines > 1) {
				addLongWeaponDescription(description, weapons);
			} else {
				addShortWeaponDescription(description, weapons);
			}
		}
		
		return description.toString();
	}

	private void addLongWeaponDescription(StringBuffer description, Weapons weapons) {
		if (weapons != null && weapons.getQuantity() > 0) {
			TechnologyBundle[] weaponsByType = weapons.getDistinct();
			addWeaponDescription(description, weaponsByType, MAX_QTY_IN_LONG_DESCRIPTION, true);
		}
	}

	private void addShortWeaponDescription(StringBuffer description, Weapons weapons) {
		if (weapons != null) {
			TechnologyBundle[] weaponsByType = weapons.getDistinct();
			if (weaponsByType == null) return;
			addWeaponDescription(description, weaponsByType, MAX_QTY_IN_SHORT_DESCRIPTION, false);
		}
	}

	private void addWeaponDescription(StringBuffer description, TechnologyBundle[] weaponsByType, int maxQty, boolean withDam) {
		int qty = Comp.min(weaponsByType.length, maxQty);
		for (int j = 0; j < qty; j++) {
			if (j > 0) description.append(", ");
			TechnologyBundle weapon = weaponsByType[j];
			description.append(weapon.getName()+"*"+weapon.getQty());
			if (withDam && weapon.getTechnology() instanceof Weapon) {
				Weapon w = (Weapon) weapon.getTechnology();
				description.append(" Dam "+w.getDamage());
			}
		}
	}
	
	@Override
	public double getDamageAsPercent() throws ApplicationError {
		return value.getDamageAsPercent();
	}

	@Override
	public double getBuildQualityAsPercent() throws ApplicationError {
		return value.getBuildQualityAsPercent();
	}

	@Override
	public String getOrigin() throws ApplicationError {
		return value.getOrigin();
	}
	
	public PlayerDelegate getOriginDelegate() throws ApplicationError {
		return DeligateCacheManager.getPlayer(value.getOrigin());
	}

	@Override
	public String getFleet() throws ApplicationError {
		return value.getFleet();
	}
	public FleetDelegate getFleetDelegate() throws ApplicationError {
		return (FleetDelegate) DeligateCacheManager.getMoo(value.getFleet());
	}

	@Override
	public CraftDesign getCraftDesign() throws ApplicationError {
		return value.getDesign();
	}

	@Override
	public int getMaintenanceCost() {
		return value.getMaintenanceCost();
	}

	@Override
	public boolean hasMoved() {
		return value.isHasMoved() || value.getNavigator() != null;
	}

	@Override
	public MoveableOwnedObjectVO getMoveableOwnedObjectVO() {
		return value;
	}

	public CraftVO getCraftValue() {
		return value;
	}

	@Override
	public void flushImageCache() {
		craftGraphics = null;
		image = null;
	}

	public void update(CraftVO vo) {
		value = vo;
		craftGraphics = null;
		image = null;
	}

	public boolean galaxyMenuCanAttackActive() throws ApplicationError {
		if (canAttack()) {
			FleetDelegate fleet = getFleetDelegate();
			boolean hasMoved = fleet == null ? hasMoved() : fleet.hasMoved();
			return (isInOrbitRoundDefendedEnemy() || isInOrbitWithEnemyCraft()) && !hasMoved;
		}
		return false;
	}
	
	public boolean galaxyMenuCanBombardActive() throws ApplicationError {
		if (canBombard()) {
			FleetDelegate fleet = getFleetDelegate();
			boolean hasMoved = fleet == null ? hasMoved() : fleet.hasMoved();
			return isInOrbitRoundUnDefendedEnemy() && !hasMoved;
		}
		return false;
	}
	
	public boolean galaxyMenuCanInvadeActive() {
		return false;
	}
	
	public boolean galaxyMenuCanColoniseActive() throws ApplicationError {
		OrbitVO orbit = getOrbit();
		if (orbit != null && getNavigator() == null) {
			if (orbit.getPlanet() != null) {
				PlanetDelegate planet = DeligateCacheManager.getPlanet(orbit);
				if (canColonise(planet) && !planet.hasOwner())  return true;
			}
		}
		return false;
	}
	
	public boolean galaxyMenuCanDeployActive() throws ApplicationError {
		if (canDeploy()) {
			return isInOrbitRoundUnDefendedEnemy() && !isInOrbitWithEnemyCraft();
		}
		return false;
	}
	
	public boolean galaxyMenuCanJoinFleetActive() throws ApplicationError {
		OrbitVO orbit = getOrbit();
		return (orbit != null) && !hasMoved();
	}
	
	public boolean galaxyMenuCanUpgradeActive() throws ApplicationError {
		OrbitVO orbit = getOrbit();
		PlayerDelegate owner = getOwnerDelegate();
		if (owner == null || orbit == null || !(owner instanceof UserPlayerDelegate)) return false;
		FleetDelegate fleet = getFleetDelegate();
		boolean hasMoved = fleet == null ? hasMoved() : fleet.hasMoved();
		if (hasMoved) return false;
		if (orbit.getPlanet() == null) return false;
		PlanetDelegate planet = DeligateCacheManager.getPlanet(orbit.getPlanet());
		if (planet == null || planet.getProjectDesign() != null || !planet.isOwnedBy(owner) || planet.isMiningColony()) return false;
		CraftDesigns craftDesigns = ((UserPlayerDelegate) owner).getCraftDesigns();
		if (craftDesigns == null) return false;
		CraftDesigns upgrades = craftDesigns.getCraftUpgradeDesignsFor(this);
		return upgrades != null && upgrades.getQuantity() > 0;
	}
	
	public boolean galaxyMenuCanDeleteActive() throws ApplicationError {
		FleetDelegate fleet = getFleetDelegate();
		boolean hasMoved = fleet == null ? hasMoved() : fleet.hasMoved();
		return !hasMoved;
	}

	public boolean canAttack() throws ApplicationError {
		CraftDesign cd = getCraftDesign();
		if (cd != null)  {
			if (cd.getQtyWeapons() > 0) return true;
			Specials specials = cd.getSpecialTechs();
			if (specials != null && specials.getQuantityWithGroup(Special.GROUP_FIGHTER_BAYS) > 0) return true;
		}
		return false;
	}

	public boolean canBombard() throws ApplicationError {
		CraftDesign cd = getCraftDesign();
		if (cd != null)  {
			if (cd.getQtyWeapons() > 0) return true;
		}
		return false;
	}

	public boolean canDeploy() throws ApplicationError {
		CraftDesign cd = getCraftDesign();
		if (cd != null)  {
			int qtyPsi = cd.getQuantityWeaponsWithGroup(Weapon.GROUP_PSI);
			if (qtyPsi > 0) return true;
			int qtyPlague = cd.getQuantityWeaponsWithGroup(Weapon.GROUP_PLAGUE);
			if (qtyPlague > 0) return true;
		}
		return false;
	}


	/**
	 * This state is for debugging only.
	 */
	private DeligateDebugState debugState;
	
	public void setDebugState(DeligateDebugState debugState) {
		this.debugState = debugState;
	}

	public DeligateDebugState getDebugState() {
		return debugState;
	}


}
