/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import mot.client.deligates.CraftDelegate;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.technology.Specials;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.FleetVO;
import mot.common.vo.MoveableOwnedObjectVO;
import mot.common.vo.OrbitVO;

public class MotFleetDelegate extends MotMoveableOwnedObjectDelegate implements FleetDelegate {

	private CraftDelegate[] crafts;
	private FleetVO value;
	
	public MotFleetDelegate() {
	}
	
	public MotFleetDelegate(FleetVO vo) {
		value = vo;
	}

	public String getImage() throws ApplicationError {
		CraftDelegate craft = getMostVisibleCraft();
		if (craft == null) {
			return null;
		} else {
			return craft.getImage();
		}
	}
	
	public String get3DImage() throws ApplicationError {
		CraftDelegate craft = getMostVisibleCraft();
		if (craft == null) {
			return null;
		} else {
			return craft.get3DImage();
		}
	}


	public Size getImageSize() throws ApplicationError {
		CraftDelegate craft = getMostVisibleCraft();
		if (craft == null) {
			return null;
		} else {
			return craft.getImageSize();
		}
	}

	public CraftDelegate[] getCrafts() throws ApplicationError {
		if (crafts == null) {
//			if (value.getName().equalsIgnoreCase("Fleet14")) {
//				Logger.logError("");x
//			}
			crafts = DeligateCacheManager.getCraftsInFleet(value.getName());
		}
		return crafts;
	}

	public MoveableOwnedObjectVO getMoveableOwnedObjectVO() {
		return value;
	}

	private static final int MAX_CHARS_IN_FLEET_TO_NAME_ALL = 90;
	
	public String getDescription(int maxLines, boolean withName, boolean withEta) throws ApplicationError {
		int MAX = MAX_CHARS_IN_FLEET_TO_NAME_ALL;
		if (MainWindow.getWidth() > 1600) {
			MAX = 150;
		} else if (MainWindow.getWidth() > 1100) {
			MAX = 110;
		}
		
        CraftDelegate[] crafts = getCrafts();
		StringBuffer description = new StringBuffer("<html>");
		if (withName) {
			description.append("Fleet "+getName()+" [ ");
		}
        if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {			
				if (description.length() > MAX) {
					description.append(L.PLUS_MORE);
					break;
				} else {
					if (i > 0) description.append(", ");
					description.append(crafts[i].getName());
				}
			}
        }
		if (withName) {
			description.append(" ]");
		}
		if (withEta) {
			String enrouteDescription = getEnrouteDescription();
			if (enrouteDescription != null) {
				description.append(" ");
				description.append(enrouteDescription);
			}
		}
		return description.toString();
	}
	
	public CraftDelegate getMostVisibleCraft() throws ApplicationError {
		CraftDelegate mostVisible = null;
		int visibility = -1;
        CraftDelegate[] crafts = getCrafts();
        if (crafts != null) {
	        for (int i = 0; i < crafts.length; i++) {
	        	int v = crafts[i].getVisibility();
				if (mostVisible == null || v > visibility) {
					mostVisible = crafts[i];
					visibility = v;
				}
			}
        }
        return mostVisible;
	}

	@Override
	public boolean hasMoved() {
		return value.isHasMoved() || value.getNavigator() != null;
	}

	@Override
	public void flushImageCache() {
	}

	public void update(FleetVO vo) {
		value = (FleetVO) vo;
		crafts = null;
	}

	public void remove(String name) {
		value.setCrafts(Comp.remove(value.getCrafts(), name));
		crafts = null;
	}

	public int getQtyCrafts() {
		String[] ca = value.getCrafts();
		return ca == null ? 0 : ca.length;
	}
	
	public CraftDelegate getBestColonyCraft() throws ApplicationError {
		CraftDelegate best = null;
		double colonisationRating = 9999;
        CraftDelegate[] crafts = getCrafts();
        if (crafts != null) {
	        for (int i = 0; i < crafts.length; i++) {
	        	Specials specials = crafts[i].getCraftDesign().getSpecialTechs();
	        	if (specials != null && specials.getCanColoniseRating() != null) {	        		
	        		double v = specials.getCanColoniseRating();
	        		if (best == null || v < colonisationRating) {
	        			best = crafts[i];
	        			colonisationRating = v;
	        		}
	        	}
			}
        }
        return best;
	}


	public boolean galaxyMenuCanAttackActive() throws ApplicationError {
        CraftDelegate[] crafts = getCrafts();
        if (crafts != null) {
        	for (int i = 0; i < crafts.length; i++) {
				if (crafts[i].galaxyMenuCanAttackActive()) return true;
			}
        }
		return false;
	}
	
	public boolean galaxyMenuCanBombardActive() throws ApplicationError {
        CraftDelegate[] crafts = getCrafts();
        if (crafts != null) {
        	for (int i = 0; i < crafts.length; i++) {
				if (crafts[i].galaxyMenuCanBombardActive()) return true;
			}
        }
		return false;
	}
	
	public boolean galaxyMenuCanInvadeActive() {
		return false;
	}
	
	public boolean galaxyMenuCanColoniseActive() throws ApplicationError {
        CraftDelegate[] crafts = getCrafts();
        if (crafts != null) {
        	for (int i = 0; i < crafts.length; i++) {
				if (crafts[i].galaxyMenuCanColoniseActive()) return true;
			}
        }
		return false;
	}
	
	public boolean galaxyMenuCanDeployActive() throws ApplicationError {
        CraftDelegate[] crafts = getCrafts();
        if (crafts != null) {
        	for (int i = 0; i < crafts.length; i++) {
				if (crafts[i].galaxyMenuCanDeployActive()) return true;
			}
        }
		return false;
	}
	
	public boolean galaxyMenuCanJoinFleetActive() throws ApplicationError {
		// Join Fleet Button only shows if there is another fleet to merge into
		OrbitVO orbit = getOrbit();
		PlayerDelegate owner = getOwnerDelegate();
		if (owner == null || orbit == null) return false;
		int qtyFleets = DeligateCacheManager.getQtyFleetsAt(owner, orbit);
		int qtyCrafts = DeligateCacheManager.getQtyCraftsAt(owner, orbit);
		return qtyFleets > 1 || qtyCrafts > 0;
	}
	
	public boolean galaxyMenuCanUpgradeActive() throws ApplicationError {
        CraftDelegate[] crafts = getCrafts();
        if (crafts != null) {
        	for (int i = 0; i < crafts.length; i++) {
				if (crafts[i].galaxyMenuCanUpgradeActive()) return true;
			}
        }
		return false;
	}
	
	public boolean galaxyMenuCanDeleteActive() {
		return !hasMoved();
	}

}
