/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import java.awt.Color;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.StellarObjectOrbitalDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.vo.MoveableOwnedObjectVO;
import mot.common.vo.NavigatorVO;
import mot.common.vo.OrbitVO;

public abstract class MotMoveableOwnedObjectDelegate implements MoveableOwnedObjectDelegate {

	public abstract MoveableOwnedObjectVO getMoveableOwnedObjectVO();
	
	public boolean equals(NamedObjectDelegate other) {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return Comp.same(value, other);
	}

	public String getName() {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getName();
	}

	public Location getLocation() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getLocation();
	}

	public boolean isOwnedBy(PlayerDelegate player) throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return Comp.same(player, value.getOwner());
	}

	public String getOwner() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getOwner();
	}

	public PlayerDelegate getOwnerDelegate() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return DeligateCacheManager.getPlayer(value.getOwner());
	}

	public Color getOwnerColor() throws ApplicationError {
		PlayerDelegate d = getOwnerDelegate();
		return d.getColor().getColor();
	}
	
	public boolean hasOrbit() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getOrbit() != null;
	}

	public OrbitVO getOrbit() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getOrbit();
	}

	public boolean hasNavigator() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getNavigator() != null;
	}

	public NavigatorVO getNavigator() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getNavigator();
	}

	public int getVisibility() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getVisibility();
	}

	public int getRange() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.getRange();
	}

	public boolean isSleeping() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return value.isSleeping();		
	}
	
	public boolean equals(MoveableOwnedObjectDelegate other) {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		return other != null && Comp.same(getName(), other.getName());
	}

	public String getEnrouteDescription() throws ApplicationError {
		MoveableOwnedObjectVO value = getMoveableOwnedObjectVO();
		NavigatorVO navigator = value.getNavigator();
		if (navigator != null) {
			return navigator.getDescription();
		} else {
			return null;
		}
	}
	
	public String toString() { return getName();  }

	public boolean isInOrbitRoundDefendedEnemy() throws ApplicationError {
		OrbitVO orbit = getOrbit();
		if (orbit != null) {
			PlayerDelegate owner = getOwnerDelegate();
			if (orbit.getPlanet() != null) {
				PlanetDelegate planet = DeligateCacheManager.getPlanet(orbit);
				if (planet != null && planet.hasDefences()) {
					PlayerDelegate po = planet.getOwner();
					if (po != null) {
						return owner.isEnemy(po.getName());
					}
				}
			} else if (orbit.getStellarObject() != null) {
				StellarObjectDelegate so = DeligateCacheManager.getStellarObject(orbit);
				if (so != null) {
					StellarObjectOrbitalDelegate soo = so.getStellarObjectOrbital();
					if (soo != null) {
						return owner.isEnemy(soo.getOwnerName());
					}
				}
			}
		}
		return false;
	}

	public boolean isInOrbitRoundUnDefendedEnemy() throws ApplicationError {
		OrbitVO orbit = getOrbit();
		if (orbit != null) {
			PlayerDelegate owner = getOwnerDelegate();
			if (orbit.getPlanet() != null) {
				PlanetDelegate planet = DeligateCacheManager.getPlanet(orbit);
				if (planet != null && !planet.hasDefences()) {
					PlayerDelegate po = planet.getOwner();
					if (po != null) {
						return owner.isEnemy(po.getName());
					}
				}
			} else if (orbit.getStellarObject() != null) {
				StellarObjectDelegate so = DeligateCacheManager.getStellarObject(orbit);
				if (so != null) {
					StellarObjectOrbitalDelegate soo = so.getStellarObjectOrbital();
					if (soo != null) {
						return owner.isEnemy(soo.getOwnerName());
					}
				}
			}
		}
		return false;
	}

	public boolean isOrbitingOwnersPlanet() throws ApplicationError {
		OrbitVO orbit = getOrbit();
		if (orbit != null) {
			PlayerDelegate owner = getOwnerDelegate();
			if (orbit.getPlanet() != null) {
				PlanetDelegate planet = DeligateCacheManager.getPlanet(orbit);
				if (planet != null) {
					PlayerDelegate po = planet.getOwner();
					if (po != null) {
						return owner == po;
					}
				}
			}
		}
		return false;
	}

	public boolean isOrbitingOwnersSun() throws ApplicationError {
		OrbitVO orbit = getOrbit();
		if (orbit != null) {
			PlayerDelegate owner = getOwnerDelegate();
			if (orbit.getPlanet() == null) {
				StellarObjectDelegate so = DeligateCacheManager.getStellarObject(orbit);
				if (so != null && so instanceof SolarSystemDelegate) {
					SolarSystemDelegate ss = (SolarSystemDelegate) so;
					return so.isOwnedBy(owner);
				}
			}
		}
		return false;
	}

	public boolean isInOrbitWithEnemyCraft() throws ApplicationError {
		PlayerDelegate owner = getOwnerDelegate();
		MoveableOwnedObjectDelegate [] enemyMoos = DeligateCacheManager.getEnemyMoosAt(owner, getOrbit());
		return enemyMoos != null && enemyMoos.length > 0;
	}
	
	public int compareTo(MoveableOwnedObjectDelegate o) {
		try {
			UserPlayerDelegate owner = MainWindow.getOwner();
			if (o.isOwnedBy(owner)) {
				if (isOwnedBy(owner)) {
					return getName().compareTo(o.getName());
				} else {
					return 1;
				}
			} else {
				if (isOwnedBy(owner)) {
					return -1;
				} else {
					return getName().compareTo(o.getName());
				}
			}
		} catch (ApplicationError e) {
			Logger.logError("Failed To Compate "+this+" to "+o, e);
			return 0;
		}
		
	}
	
}
