/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;

import mot.client.animation.MovementRegister;
import mot.client.bean.GalaxyMapLabelSizeDetails;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.renderers.CellRendererMoneyBonus;
import mot.client.swing.renderers.CellRendererProductionBonus;
import mot.client.swing.renderers.CellRendererReproductionBonus;
import mot.client.swing.renderers.CellRendererResearchBonus;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.PlanetGraphics;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.designs.ProjectDesign;
import mot.common.factory.PlanetGraphicsFactory;
import mot.common.lang.L;
import mot.common.resources.Resource;
import mot.common.resources.Resources;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.OrbitVO;
import mot.common.vo.PlanetCrisisVO;
import mot.common.vo.PlanetProjectVO;
import mot.common.vo.PlanetVO;

public class MotPlanetDelegate implements PlanetDelegate {

	private PlanetVO value;

	private transient String [] images = null;
	private transient PlanetGraphics planetGraphics = null;
	private transient PlayerDelegate owner;
	private transient SolarSystemDelegate solarSystem;
	private transient GalaxyMapLabelSizeDetails galaxyMapLabelSizeDetails = null;

	public MotPlanetDelegate() {
	}

	public MotPlanetDelegate(PlanetVO vo) {
		value = vo;
	}

	public boolean equals(NamedObjectDelegate other) {
		return Comp.same(other, value.getName());
	}

	public String getName() {
		return value.getName();
	}

	public void setName(String newName) {
		value.setName(newName);
	}
	
	public String getOwnerName() throws ApplicationError {
		return value.getOwner();
	}

	public String getOriginName() throws ApplicationError {
		return value.getOrigin();
	}

	public PlayerDelegate getOwner() throws ApplicationError {
		if (owner == null) {
			owner = DeligateCacheManager.getPlayer(value.getOwner());
		}
		return owner;
	}

	public PlayerDelegate getOrigin() throws ApplicationError {
		return DeligateCacheManager.getPlayer(value.getOrigin());
	}

	public SolarSystemDelegate getSolarSystem() {
		if (solarSystem == null) {
			solarSystem = (SolarSystemDelegate) DeligateCacheManager.getStellarObject(value.getSolarSystem());
		}
		return solarSystem;
	}
	
	public String getSolarSystemName() {
		return value.getSolarSystem();
	}

	public boolean getOwnerWantsNoProject() {
		return value.getOwnerWantsNoProject();
	}

	public ProjectDesign getProjectDesign() {
		return value.getProjectDesign();
	}

	public int getTurnsToCompleteProject() throws ApplicationError {
		PlanetProjectVO project = value.getProject();
		if (project != null) {			
			return project.getTurnsToComplete();
		}
		return 0;
	}
	
	public int getCostToBuild() throws ApplicationError {
		PlanetProjectVO project = value.getProject();
		if (project != null) {			
			return project.getCostToBuild();
		}
		return 0;
	}
	
	public int getTurnsToCompleteProject(ProjectDesign project) throws ApplicationError {
		if (project == null) {
			return 0;
		} else if ( value.getProject() == null || !project.equals(value.getProject().getDesign()) ) {
        	PlayerDelegate projectOwner = getOwner();
        	return getTurnsToCompleteProject(project.getCostToBuildOnPlanet());
		} else {
			return getTurnsToCompleteProject();
		}
	}

    public int getTurnsToCompleteProject(int productionRequiredToFinish) throws ApplicationError {
        int prodPerTurn = getProductionOutput();
        if (prodPerTurn > 0) {
        	int production = getProduction();
            double qty = Math.ceil((((double)productionRequiredToFinish - production)) / prodPerTurn);
            if (qty < 0) qty = 0;
			return (int)qty;
        } else {
            return Integer.MAX_VALUE;
        }
    }

	public boolean isBlockaded() throws ApplicationError {
		return value.isBlockaded() && getOwner() != null;
	}

	public boolean isGuarded() throws ApplicationError {
		return value.isBlockaded() && getOwner() == null;
	}

	@Override
	public boolean isViewable() throws ApplicationError {
		return value.isViewable();
	}

	public boolean isOwnedBy(PlayerDelegate p) throws ApplicationError {
		if (p == null) return false;
		return Comp.same(p.getName(), value.getOwner());
	}

	public boolean isColonisableBy(CraftDesign cd) throws ApplicationError {
		return cd.canColonise(value.getTeraforming());
    }
	
	public int getProductionOutput() throws ApplicationError {
		return value.getProductionOutput();
	}

	public int getLostProductionOutput() throws ApplicationError {
		return value.getLostProductionOutput();
	}

	public int getResearchOutput() throws ApplicationError {
		return value.getResearchOutput();
	}

	public int getLostResearchOutput() throws ApplicationError {
		return value.getLostResearchOutput();
	}

	public double getEcologyOutput() throws ApplicationError {
		return value.getEcologyOutput();
	}

	public double getDevelopmentOutput() throws ApplicationError {
		return value.getDevelopmentOutput();
	}

	public int getMoneyOutput() throws ApplicationError {
		return value.getMoneyOutput();
	}

    public int getMoneyWasted() throws ApplicationError {
    	return value.getMoneyWasted();
    }

	public double getMinimumEcologyOutput() throws ApplicationError {
		return value.getMinimumEcologyOutput();
	}

	public double getMinimumEcologyPercent() throws ApplicationError {
		return value.getMinimumEcologyPercent();
	}

	public double getTeraformingGrowth() throws ApplicationError {
		return value.getTeraformingGrowth();
	}

	public double getPopulationGrowth() throws ApplicationError {
		return value.getPopulationGrowth();
	}

	public double getFactoriesGrowth() throws ApplicationError {
		return value.getFactoriesGrowth();
	}

	public int getQtyColonistsOnrouteTo() throws ApplicationError {
		return value.getQtyColonistsOnrouteTo();
	}

	public int getColonistsEtaOnrouteTo() throws ApplicationError {
		return value.getColonistsEtaOnrouteTo();
	}
	
	public int getTotalOutput() throws ApplicationError {
		return (int) (getProductionOutput() + getResearchOutput() + getEcologyOutput() + getDevelopmentOutput() + getMoneyOutput());
	}
	
	public String getProjectDescription() throws ApplicationError {
		ProjectDesign project = getProjectDesign();
        if (project == null) {
            return L.NO_PROJECT;
        } else {
            int turns = getTurnsToCompleteProject();
            String turnsStr = (turns > 1000) ? L.INFINATE : ""+turns;
            return L.BUILD+" : "+project.getName()+" ["+turnsStr+" "+L.TURNS+"]";
        }
	}

	public int getProduction() {
		return value.getProduction();
	}

	public Weapons getWeapons() throws ApplicationError {
		return Technologies.getWeapons().getWeapons(value.getWeapons());
	}

	public Shields getShields() throws ApplicationError {
		return Technologies.getShields().getShields(value.getShields());
	}

	public Scanners getScanners() throws ApplicationError {
		return Technologies.getScanners().getScanners(value.getScanners());
	}

	public double getMaxPopulation() throws ApplicationError {
		return value.getMaxPopulation();
	}

	public double getPopulation() {
		return value.getPopulation();
	}

	public double getTeraforming() {
		return value.getTeraforming();
	}

	public int getSize() {
		return value.getSize();
	}

	public double getQtyFactories() {
		return value.getQtyFactories();
	}

	public double getPercentEcology() {
		return value.getPercentEcology();
	}

	public double getPercentDevelopment() {
		return value.getPercentDevelopment();
	}

	public double getPercentResearch() {
		return value.getPercentResearch();
	}

	public double getPercentProduction() {
		return value.getPercentProduction();
	}

	public double getPercentMoney() {
		return 1.0 - value.getPercentProduction() - value.getPercentResearch() - value.getPercentDevelopment() - value.getPercentEcology();
	}

	public boolean getNotifiedTerraformComplete() {
		return value.getNotifiedTerraformComplete();
	}

	public boolean getNotifiedFactoriesComplete() {
		return value.getNotifiedFactoriesComplete();
	}

	public double getProductionBonus() {
		return value.getProductionBonus();
	}

	public double getResearchBonus() {
		return value.getResearchBonus();
	}

	public double getReproductionBonus() {
		return value.getReproductionBonus();
	}

	public int getMoneyBonus() {
		return value.getMoneyBonus();
	}

	public boolean getAutoEcology() {
		return value.getAutoEcology();
	}

	public boolean getAutoDevelopment() {
		return value.getAutoDevelopment();
	}

	public boolean getVaryProduction() {
		return value.getVaryProduction();
	}
	
	public boolean getVaryResearch() {
		return value.getVaryResearch();
	}

	public int getInfluence() {
		return value.getInfluence();
	}

	public int getQtyGroundDefense() {
		return value.getQtyGroundDefense();
	}

	public Orbital getOrbital() throws ApplicationError {
		return value.getOrbital();
	}

	public int getRepeatProject() {
		return value.getRepeatProject();
	}

	public long getCostToPurchaseProject() throws ApplicationError {
		return value.getCostToPurchaseProject();
	}

	public PlanetDelegate getMiningColonyDelegate() throws ApplicationError {
		PlanetDelegate pd = DeligateCacheManager.getPlanet(value.getMiningColony());
		return pd;
	}

	public boolean hasMiningColony() {
		return value.getMiningColony() != null;
	}

	public Resource getResource() {
		return Resources.getResource(value.getResource());
	}

	public int getMiningPopulation() throws ApplicationError {
		return value.getMiningPopulation();
	}

	public int getMiningProduction() throws ApplicationError {
		PlanetDelegate mc = getMiningColonyDelegate();
		if (mc == null) return 0;
		return mc.getProductionOutput();
	}

	public int getMiningResearch() throws ApplicationError {
		PlanetDelegate mc = getMiningColonyDelegate();
		if (mc == null) return 0;
		return mc.getResearchOutput();
	}

	public int getMiningMoney() throws ApplicationError {
		PlanetDelegate mc = getMiningColonyDelegate();
		if (mc == null) return 0;
		return mc.getMoneyOutput();
	}

	public int getLostMiningProduction() throws ApplicationError {
		PlanetDelegate mc = getMiningColonyDelegate();
		if (mc == null) return 0;
		return mc.getLostProductionOutput();
	}

	public int getLostMiningResearch() throws ApplicationError {
		PlanetDelegate mc = getMiningColonyDelegate();
		if (mc == null) return 0;
		return mc.getLostResearchOutput();
	}
	
	public int getLostMiningMoney() throws ApplicationError {
		PlanetDelegate mc = getMiningColonyDelegate();
		if (mc == null) return 0;
		return mc.getMoneyWasted();
	}

	public boolean isDominated() {
		return value.isDominated();
	}
	
	public boolean equals(PlanetDelegate other) {
		return Comp.same(other, value.getName());
	}

	public boolean hasAsteroids() {
		return value.hasAsteroids();
	}

	public boolean hasDefences() throws ApplicationError {
		return hasWeapons() || value.getOrbital() != null;
	}

	private boolean hasWeapons() throws ApplicationError {
		Weapons weapons = getWeapons();
		return weapons != null && weapons.getQuantity() > 0;
	}

	public boolean hasNatives() {
		return value.hasNatives();
	}

    public int getImageSize() throws ApplicationError {
		int maxPlanetSize = PlanetGraphicsFactory.getMaxPlanetSize();
		double scale = PlanetGraphicsFactory.getScale(getSize());
		return (int)(maxPlanetSize * scale);
	}


	public boolean hasOwner() throws ApplicationError {
		return (value.getOwner() != null);
	}
	
    public Color getTextColor() throws ApplicationError {
    	return getOwner().getColor().getColor();
    }

	/**
	 * Because planets terraform automatically, at the end of each turn we need to
	 * check that the images for each planet are still correct and update them if 
	 * necessary.
	 */
	public void updateImages() throws ApplicationError {
    	if (images != null) {
        	double teraforming = getTeraforming();
    		PlanetGraphics pg = PlanetGraphicsFactory.getPlanetGraphics(teraforming);
    		if (planetGraphics != pg) {
    			planetGraphics = pg;
        		createImages(planetGraphics);
    		}
    	}
	}
	
    public String getImageForPlanet() throws ApplicationError {
    	return getImageForPlanet(MovementRegister.getTickNo());
    }
    
    public String getImageForPlanet(int animationStage) throws ApplicationError {
    	if (images == null) {
        	double teraforming = getTeraforming();
    		planetGraphics = PlanetGraphicsFactory.getPlanetGraphics(teraforming);
    		createImages(planetGraphics);
    	}
    	return images[animationStage%images.length];
    }

	private void createImages(PlanetGraphics planetGraphics) {
		String[] stages = planetGraphics.getStagesAsArray();
		images = new String[stages.length];
		for (int i = 0; i < stages.length; i++) {
			images[i] = planetGraphics.getImage() + stages[i] + planetGraphics.getImageType(); 
		}
	}

	public String getPlanetTypeName() {
		double teraforming = getTeraforming();
		if (teraforming >= 1.8) {
			return L.UTOPIA;
		} else if (teraforming >= 1.4) {
			return L.GIA;
		} else if (teraforming >= 1.0) {
			return L.TERRAN;
		} else if (teraforming >= 0.8) {
			return L.ARID;
		} else if (teraforming >= 0.6) {
			return L.DESERT;
		} else if (teraforming >= 0.4) {
			return L.BAREN;
		} else {
			return L.TOXIC;
		}
	}
	

	public String getMoneyBonusName() {
		int bonus = getMoneyBonus();
		if (bonus == 0) return L.NONE;
		return CellRendererMoneyBonus.getMoneyBonusName(bonus)+"(+"+bonus+")";
	}
	
	public String getProductionBonusName() {
		double bonus = getProductionBonus();
		return CellRendererProductionBonus.getProductionBonusName(bonus);
	}
	
	public String getResearchBonusName() {
		double bonus = getResearchBonus();
		return CellRendererResearchBonus.getResearchBonusName(bonus);
	}
	
	public String getFertilityBonusName() {
		double bonus = getReproductionBonus();
		return CellRendererReproductionBonus.getFertilityBonusName(bonus);
	}

	public OrbitVO getOrbit() {
		return new OrbitVO(getSolarSystemName(), getName());
	}

	public boolean isMiningColony() throws ApplicationError {
		return (hasOwner() &&
				getOrbital() != null && 
				getPopulation() == 0);
	}

	public PlanetDelegate getOwnerOfMiningStation() throws ApplicationError {
		PlanetDelegate[] planets = DeligateCacheManager.getPlanets();
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				String miningColony = planets[i].getMiningColony();
				if (miningColony != null) {
					if (Comp.same(miningColony, getName())) {
	                    return planets[i];
					}
				}
			}
		}
		return null;
	}

	public String getMiningColony() {
		return value.getMiningColony();
	}

	@Override
	public PlanetCrisisVO getCrisis() {
		return value.getCrisis();
	}

	public PlanetVO getPlanetVO() {
		return value;
	}

	public void update(PlanetVO vo) {
		value = vo;
		owner = null;
		images = null;
		planetGraphics = null;
		solarSystem = null;
		galaxyMapLabelSizeDetails = null;
	}

	public String toString() { return getName();  }

	public GalaxyMapLabelSizeDetails getGalaxyMapLabelSizeDetails(Graphics2D graphics) throws ApplicationError {
		if (galaxyMapLabelSizeDetails == null) {
	    	FontMetrics fontMetrics = graphics.getFontMetrics();
	    	int labelWidth = fontMetrics.stringWidth(getName());
	    	int labelHeight= fontMetrics.getHeight()-4;
			galaxyMapLabelSizeDetails = new GalaxyMapLabelSizeDetails(labelWidth, labelHeight);
		}
		return galaxyMapLabelSizeDetails;
	}

	public boolean isWayPoint() throws ApplicationError {
    	OrbitVO fleetWayPoint = MainWindow.getOwner().getFleetWayPoint();
    	boolean isWaypoint = fleetWayPoint != null && fleetWayPoint.getPlanet() == getName();
    	return isWaypoint;
	}

	public int getMaintenanceCost() throws ApplicationError {
		return getWeaponMaintenanceCost() + getShieldMaintenanceCost() + getScannerMaintenanceCost() + getOrbitalMaintenanceCost();
	}
	
	public double getPlanetMaintenanceFactor() throws ApplicationError {
		PlayerDelegate o = getOwner();
		if (o == null) {
			return 1.0;
		} else {
			return owner.getPlanetMaintenanceFactor();
		}
	}
	
    public int getWeaponMaintenanceCost() throws ApplicationError {
    	TechnologyList tl = getWeapons();
    	if (tl != null) {
            int mc = tl.getMaintenanceCost();
            double mcf = getPlanetMaintenanceFactor();
			return (int) (mc * mcf);
        } else {
        	return 0;
        }
    }
    
    public int getShieldMaintenanceCost() throws ApplicationError {
    	TechnologyList tl = getShields();
    	if (tl != null) {
            int mc = tl.getMaintenanceCost();
            double mcf = getPlanetMaintenanceFactor();
			return (int) (mc * mcf);
        } else {
        	return 0;
        }
    }
    
    public int getScannerMaintenanceCost() throws ApplicationError {
    	TechnologyList tl = getScanners();
    	if (tl != null) {
            int mc = tl.getMaintenanceCost();
            double mcf = getPlanetMaintenanceFactor();
			return (int) (mc * mcf);
        } else {
        	return 0;
        }
    }
    
    public int getOrbitalMaintenanceCost() throws ApplicationError {
    	Orbital tl = getOrbital();
    	if (tl != null) {
            int mc = tl.getMaintenanceCost();
            double mcf = getPlanetMaintenanceFactor();
			return (int) (mc * mcf);
        } else {
        	return 0;
        }
    }

    public String getWeaponIcon() throws ApplicationError {
    	Weapons weapons = getWeapons();
    	if (weapons == null) return null;
    	Technology weapon = weapons.getMostCommon();
    	if (weapon == null) {
    		return null;
    	} else {    		
    		return weapon.getImage();
    	}
    }
    
    public String getShieldIcon() throws ApplicationError {
    	Shields shields = getShields();
    	if (shields == null) return null;
    	Technology shield = shields.getMostCommon();
    	if (shield == null) {
    		return null;
    	} else {    		
    		return shield.getImage();
    	}
    }
    
    public String getScannerIcon() throws ApplicationError {
    	Scanners scanners = getScanners();
    	if (scanners == null) return null;
    	Technology scanner = scanners.getMostCommon();
    	if (scanner == null) {
    		return null;
    	} else {    		
    		return scanner.getImage();
    	}
    }
    
    public String getOrbitalIcon() throws ApplicationError {
    	Orbital orbital = getOrbital();
    	if (orbital == null) {
    		return null;
    	} else {    		
    		return orbital.getHull();
    	}
    }

}
