/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import java.util.ArrayList;

import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.RelationDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.PlayerCharacterBasic;
import mot.common.bean.PlayerColor;
import mot.common.bean.SalvageList;
import mot.common.morals.MoralChoice;
import mot.common.morals.MoralChoiceList;
import mot.common.morals.MoralChoices;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.Comp;
import mot.common.vo.PlayerConstantsVO;
import mot.common.vo.PlayerDetailsVO;
import mot.common.vo.PlayerVO;
import mot.common.vo.RelationResearchAgreementVO;
import mot.common.vo.RelationTradeAgreementVO;
import mot.common.vo.StrategistEventVO;

public abstract class MotPlayerDelegate implements PlayerDelegate {

	public PlayerDetailsVO getDetailsValue() { return detailsValue; }
	public PlayerConstantsVO getConstantsValue() { return constantsValue; }
	public abstract PlayerVO getPlayerValue();
	
	private PlayerConstantsVO constantsValue = null;
	
	//
	// NOT ALWAYS UP TO DATE - FETCHED WHEN WINDOW OPENED
	//
    private PlayerDetailsVO detailsValue = null;
	
	@Override
	public String getName() {
		PlayerVO value = getPlayerValue();
		return value.getName();
	}

	@Override
	public PlayerColor getColor() throws ApplicationError {
		PlayerConstantsVO value = getConstantsValue();
		if (value == null) return null;
		return value.getPlayerColor();
	}

	@Override
	public String getColorName() throws ApplicationError {
		PlayerConstantsVO value = getConstantsValue();
		return value.getColor();
	}

	@Override
	public String getAlignment() {
		PlayerConstantsVO value = getConstantsValue();
		return value.getAlignment();
	}

	@Override
	public int getFunds() {
		PlayerVO value = getPlayerValue();
		if (value == null) return 0;
		return value.getFunds();
	}

	@Override
	public int getBoralium() {
		PlayerVO value = getPlayerValue();
		if (value == null) return 0;
		return value.getBoralium();
	}

	@Override
	public boolean equals(PlayerDelegate other) {
		PlayerVO value = getPlayerValue();
		return Comp.same(other, value.getName());
	}

	@Override
	public boolean equals(NamedObjectDelegate other) {
		PlayerVO value = getPlayerValue();
		return Comp.same(other, value.getName());
	}
  
	@Override
	public SalvageList getSalvage() throws ApplicationError {
		// Only user players return a value
		return null;
	}

	@Override
	public PlayerCharacterBasic getPlayerCharacter() throws ApplicationError {
		PlayerConstantsVO value = getConstantsValue();
		if (value == null) return null;
		return value.getPlayerCharacter();
	}

	@Override
	public double getPlanetMaintenanceFactor() {
		PlayerConstantsVO value = getConstantsValue();
		if (value == null) return 1.0;
		return value.getPlanetMaintenanceFactor();
	}
	
	
	//
	// THESE VALUES NOT ALWAYS UP TO DATE BUT ALWAYS UP TO DATE FOR USER PLAYERS
	//
	

	@Override
	public int getTotalResearch() throws ApplicationError {
		PlayerVO value =  getPlayerValue();
		if (value == null) return 0;
		return value.getTotalResearch();
	}


    public double getTradeRouteBonus() throws ApplicationError {
    	return 0.0;
    }

	


	
	//
	// THESE VALUES NOT ALWAYS UP TO DATE
	//	

	@Override
	public int getQtySpiesEspionage(PlayerDelegate other) throws ApplicationError {
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return 0;
		BasicNameValueList qtySpiesEspionage = value.getQtySpiesEspionage();
		if (qtySpiesEspionage == null) {
			return 0;
		} else {
			return qtySpiesEspionage.getParameterAsValidInt(other.getName());
		}
	}

	@Override
	public int getQtySpiesSabotage(PlayerDelegate other) throws ApplicationError {
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return 0;
		BasicNameValueList qtySpiesSabotage = value.getQtySpiesSabotage();
		if (qtySpiesSabotage == null) {
			return 0;
		} else {
			return qtySpiesSabotage.getParameterAsValidInt(other.getName());
		}
	}

	@Override
	public StrategistEventVO [] getStrategistEvents(PlayerDelegate other) throws ApplicationError {
		ArrayList r = new ArrayList();
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return null;
		StrategistEventVO[] dea = value.getStrategistEvents();
		if (dea != null) {			
			for (int i = 0; i < dea.length; i++) {
				if (Comp.same(dea[i].getPlayer(), other.getName())) {
					r.add(dea[i]);
				}
			}
		}
		if (r.size() > 0) {
			return (StrategistEventVO[]) r.toArray(new StrategistEventVO[r.size()]);
		} else {
			return null;
		}
	}
	
//	@Override
//    public DiplomaticHistoryVO getDiplomaticHistory(PlayerDelegate other) throws ApplicationError {
//		ArrayList r = new ArrayList();
//		PlayerDetailsVO value = getDetailsValue();
//		if (value == null) return null;
//		DiplomaticHistoryVO[] dha = value.getDiplomaticHistory();
//		if (dha == null) return null;
//		for (int i = 0; i < dha.length; i++) {
//			if (dha[i].containsPlayers(other.getName(), getName())) {
//				return dha[i];
//			}
//		}
//		return null;
//    }

	@Override
	public MoralChoiceList getMoralChoices() throws ApplicationError {
		PlayerDetailsVO value =  getDetailsValue();
		if (value == null) return null;
		String[] mca = value.getMoralChoices();
		if (mca == null) return null;
		MoralChoiceList mcl = new MoralChoiceList();
		for (int i = 0; i < mca.length; i++) {
			MoralChoice mc = MoralChoices.getMoralChoice(mca[i]);
			mcl.add(mc);
		}
		return mcl;
	}

	@Override
	public int getTotalPopulation() throws ApplicationError {
		PlayerDetailsVO value =  getDetailsValue();
		if (value == null) return 0;
		return value.getTotalPopulation();
	}

	@Override
	public int getTotalProduction() throws ApplicationError {
		PlayerDetailsVO value =  getDetailsValue();
		if (value == null) return 0;
		return value.getTotalProduction();
	}
	
	@Override
	public int getTotalIncome(boolean includeTrade) throws ApplicationError {
		PlayerDetailsVO value =  getDetailsValue();
		if (value == null) return 0;
		if (includeTrade) {
			return value.getTotalIncomeWithTrade();
		} else {
			return value.getTotalIncomeWithoutTrade();
		}
	}

	@Override
	public int getTradeIncome() throws ApplicationError {
		PlayerDetailsVO value =  getDetailsValue();
		if (value == null) return 0;
		return value.getTradeIncome();
	}
	
	@Override
	public int getTotalMaintenanceCost() throws ApplicationError {
		PlayerDetailsVO value =  getDetailsValue();
		if (value == null) return 0;
		return value.getTotalMaintenanceCost();
	}

	@Override
	public int getSpiesMaintenanceCost() throws ApplicationError {
		PlayerDetailsVO value =  getDetailsValue();
		if (value == null) return 0;
		return value.getSpiesMaintenanceCost();
	}

	@Override
	public int getSpyRecruitCosts() {
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return 0;
		return value.getSpyRecruitCosts();
	}
	
	@Override
	public double getSpyRecruitChance() {
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return 0;
		return value.getSpyRecruitChance();
	}
	
	@Override
	public int getSpyRecruitTotal() {
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return 0;
		return value.getSpyRecruitTotal();
	}
	
	@Override
	public int getEspionageSuccessChance(String player) throws ApplicationError {
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return 0;
		return value.getEspionageSuccessChance(player);
	}
	
	@Override
	public int getSabotageSuccessChance(String player) throws ApplicationError {
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return 0;
		return value.getSabotageSuccessChance(player);
	}
	
	public Technology getEspionageTarget(String player) throws ApplicationError {
		PlayerDetailsVO value = getDetailsValue();
		if (value == null) return null;
		String techName = value.getEspionageTarget(player);
		if (techName == null) {
			return null;
		} else {
			return Technologies.getTechnology(techName);
		}
	}


	
	//
	// LOCAL METHODS
	//
	

	
	public PlanetDelegate[] getAttackablePlanets(PlanetDelegate[] planets, boolean includeMiningColonies, boolean mustHaveDefences) throws ApplicationError {
		ArrayList r = null;
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].hasOwner() && canAttack(planets[i], mustHaveDefences)) {
					if (planets[i].isMiningColony()) {
						if (includeMiningColonies) {
							if (r == null) r = new ArrayList();
							r.add(planets[i]);
						}
					} else {
						if (r == null) r = new ArrayList();
						r.add(planets[i]);
					}
				}
			}
		}
		if (r == null) {
			return null;
		} else {
			return (PlanetDelegate[]) r.toArray(new PlanetDelegate[r.size()]);
		}
	}
	
	/**
	 * @return All colonies owned by this player that ARE NOT mining colonies.
	 * @throws ApplicationError
	 */
	public PlanetDelegate[] getNonMiningColonies() throws ApplicationError {
		PlanetDelegate[] pa = DeligateCacheManager.getPlanets();
		if (pa == null) return null;
		ArrayList r = null;
		for (int i = 0; i < pa.length; i++) {
			if (pa[i].isOwnedBy(this) && !pa[i].isMiningColony()) {
				if (r == null) r = new ArrayList();
				r.add(pa[i]);
			}
		}
		if (r == null) {
			return null;
		} else {
			return (PlanetDelegate[]) r.toArray(new PlanetDelegate[r.size()]);
		}
	}

	/**
	 * @return All planets this player has visited (can see) that can be colonised.
	 * @throws ApplicationError
	 */
	public PlanetDelegate[] getVisibleHabitablePlanets() throws ApplicationError {
		PlanetDelegate[] pa = DeligateCacheManager.getPlanets();
		if (pa == null) return null;
		ArrayList r = null;
		for (int i = 0; i < pa.length; i++) {
			if (pa[i].getSolarSystem().isVisible() && canColonise(pa[i])) {
				if (r == null) r = new ArrayList();
				r.add(pa[i]);
			}
		}
		if (r == null) {
			return null;
		} else {
			return (PlanetDelegate[]) r.toArray(new PlanetDelegate[r.size()]);
		}
	}

	/**
	 * @return All planets owned by this player has visited (can see).
	 * @throws ApplicationError
	 */
	public PlanetDelegate[] getVisiblePlanets() throws ApplicationError {
		PlanetDelegate[] pa = DeligateCacheManager.getPlanets();
		if (pa == null) return null;
		ArrayList r = null;
		for (int i = 0; i < pa.length; i++) {
			if (pa[i].getSolarSystem().isVisible()) {
				if (r == null) r = new ArrayList();
				r.add(pa[i]);
			}
		}
		if (r == null) {
			return null;
		} else {
			return (PlanetDelegate[]) r.toArray(new PlanetDelegate[r.size()]);
		}
	}

	public boolean canColonise(PlanetDelegate planetDeligate) throws ApplicationError {
		UserPlayerDelegate owner = MainWindow.getOwner();
    	Specials specials = owner.getSpecials();
    	if (specials != null) {
    		Special colonyModule = specials.getBestCanColonise();
    		if (colonyModule != null) {
    			return colonyModule.getCanColoniseRating() <= planetDeligate.getTeraforming();
    		}
    	}
    	return false;
	}

	public PlanetDelegate[] getColonisablePlanets(PlanetDelegate[] planets) throws ApplicationError {
		if (planets == null) return null;
		ArrayList r = null;
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (canColonise(planets[i])) {
					if (r == null) r = new ArrayList();
					r.add(planets[i]);
				}
			}
		}
		if (r == null) {
			return null;
		} else {
			return (PlanetDelegate[]) r.toArray(new PlanetDelegate[r.size()]);
		}
	}
	
	public RelationDelegate getRelation(String other) throws ApplicationError {
		if (other == null) return null;
		RelationDelegate relation = DeligateCacheManager.getRelation(getName(), other);
		return relation;
	}
	
	public RelationDelegate getRelation(PlayerDelegate other) throws ApplicationError {
		if (other == null) return null;
		RelationDelegate relation = DeligateCacheManager.getRelation(getName(), other.getName());
		return relation;
	}
	
	public boolean hasDiplomaticRelationsWith(PlayerDelegate other) throws ApplicationError {
		RelationDelegate relation = getRelation(other);
		return relation != null;
	}
	
	public boolean canAttack(PlanetDelegate planetDeligate, boolean mustHaveDefences) throws ApplicationError {
		String ownerName = planetDeligate.getOwnerName();
		if (ownerName == null) return false;
		boolean isOwnedByAnEnemy = isEnemy(ownerName);
		if (mustHaveDefences) {
			return ( isOwnedByAnEnemy && planetDeligate.hasDefences() );
		} else {
			return isOwnedByAnEnemy;
		}
	}
	
	public boolean isEnemy(String otherPlayerName) throws ApplicationError {
		if (otherPlayerName == null) {
			return false;
		}
		if (otherPlayerName.equalsIgnoreCase("Monster")) {
			return true;
		}
		RelationDelegate relation = getRelation(otherPlayerName);
		if (relation == null) return false;
		return relation.isWar();
	}

	public boolean isEnemy(String otherPlayerNames []) throws ApplicationError {
		if (otherPlayerNames == null) {
			return false;
		}
		for (int i = 0; i < otherPlayerNames.length; i++) {
			if (isEnemy(otherPlayerNames[i])) {
				return true;
			}
		}
		return false;
	}

    public String [] getPlayersWithTradeAgreements() throws ApplicationError {
    	RelationDelegate[] relations = DeligateCacheManager.getRelations();
    	if (relations == null) return null;
    	ArrayList r = null;
    	for (int i = 0; i < relations.length; i++) {
			if (relations[i].contains(getName())) {
				RelationTradeAgreementVO tradeAgreement = relations[i].getTradeAgreement();
				if (tradeAgreement != null) {					
					if (r == null) r = new ArrayList();
					r.add(relations[i].getOtherPlayerName(getName()));
				}
			}
		}
    	if (r == null) {
    		return null;
    	} else {
    		return (String[]) r.toArray(new String [r.size()]);
    	}
    }
    
    public String [] getPlayersWithResearchAgreements() throws ApplicationError {
    	RelationDelegate[] relations = DeligateCacheManager.getRelations();
    	if (relations == null) return null;
    	ArrayList r = null;
    	for (int i = 0; i < relations.length; i++) {
			if (relations[i].contains(getName())) {
				RelationResearchAgreementVO researchAgreement = relations[i].getResearchAgreement();
				if (researchAgreement != null) {					
					if (r == null) r = new ArrayList();
					r.add(relations[i].getOtherPlayerName(getName()));
				}
			}
		}
    	if (r == null) {
    		return null;
    	} else {
    		return (String[]) r.toArray(new String [r.size()]);
    	}
    }
    
    public String [] getPlayersAllianceTreaty() throws ApplicationError {
    	RelationDelegate[] relations = DeligateCacheManager.getRelations();
    	if (relations == null) return null;
		String ownerName = getName();
    	ArrayList r = null;
    	for (int i = 0; i < relations.length; i++) {
			String name = getName();
			if (relations[i].contains(getName()) && Comp.same(ownerName, name)) {
				if (relations[i].isAliance()) {					
					if (r == null) r = new ArrayList();
					r.add(relations[i].getOtherPlayerName(ownerName));
				}
			}
		}
    	if (r == null) {
    		return null;
    	} else {
    		return (String[]) r.toArray(new String [r.size()]);
    	}
    }

	public String[] getEnemyPlayers() throws ApplicationError {
		PlayerDelegate[] players = DeligateCacheManager.getPlayers();
		if (players == null) return null;
		String ownerName = getName();
    	ArrayList r = null;
		for (int i = 0; i < players.length; i++) {
			String name = players[i].getName();
			if (!Comp.same(ownerName, name)) {
				if (isEnemy(players[i].getName())) {
					if (r == null) r = new ArrayList();
					r.add(name);
				}
			}
		}
    	if (r == null) {
    		return null;
    	} else {
    		return (String[]) r.toArray(new String [r.size()]);
    	}
	}


	public String toString() { return getName();  }

	public void update(PlayerDetailsVO vo) {
		detailsValue = vo;
	}

	public void update(PlayerConstantsVO vo) {
		constantsValue = vo;
	}


}
