/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;

import mot.client.bean.GalaxyMapLabelSizeDetails;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.OutpostDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.SpaceStationDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.StellarObjectOrbitalDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Location;
import mot.common.bean.StellarObjectGraphics;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.OrbitVO;
import mot.common.vo.OutpostVO;
import mot.common.vo.SpaceStationVO;
import mot.common.vo.StellarObjectOrbitalVO;
import mot.common.vo.StellarObjectVO;

public abstract class MotStellarObjectDelegate implements StellarObjectDelegate {

	protected abstract StellarObjectVO getStellObjectValue();
	
	protected transient StellarObjectOrbitalDelegate stellarObjectOrbitalDelegate;
	protected transient GalaxyMapLabelSizeDetails galaxyMapLabelSizeDetails;
	
	@Override
	public boolean hasOwner() throws ApplicationError {
		StellarObjectVO value = getStellObjectValue();
		return value.getOwners() != null;
	}

	@Override
	public Color getTextColor() throws ApplicationError {
		StellarObjectVO value = getStellObjectValue();
		return value.getTextColor();
	}

	@Override
	public boolean equals(NamedObjectDelegate other) {
		StellarObjectVO value = getStellObjectValue();
		return Comp.same(value.getName(), other);
	}

	@Override
	public String getName() {
		StellarObjectVO value = getStellObjectValue();
		return value.getName();
	}

	public void setName(String newName) {
		StellarObjectVO value = getStellObjectValue();
		value.setName(newName);
	}

	@Override
	public Location getLocation() {
		StellarObjectVO value = getStellObjectValue();
		return value.getLocation();
	}

	@Override
	public int getLastVisitedDate() throws ApplicationError {
		StellarObjectVO value = getStellObjectValue();
		return value.getLastVisitiedDate();
	}

	@Override
	public boolean isOwnedBy(PlayerDelegate player) throws ApplicationError {
		StellarObjectVO value = getStellObjectValue();
		return Comp.in(player.getName(), value.getOwners());
	}

	@Override
	public boolean isVisible() throws ApplicationError {
		StellarObjectVO value = getStellObjectValue();
		return value.isVisible();
	}

	@Override
	public String[] getOwners() throws ApplicationError {
		StellarObjectVO value = getStellObjectValue();
		String[] owners = value.getOwners();
		return owners;
	}

	@Override
	public PlayerDelegate[] getOwnerDelegates() throws ApplicationError {
		StellarObjectVO value = getStellObjectValue();
		String[] owners = value.getOwners();
		if (owners == null || owners.length == 0) {
			return null;
		}
		PlayerDelegate[] r = new PlayerDelegate[owners.length];
		for (int i = 0; i < r.length; i++) {
			r[i] = DeligateCacheManager.getPlayer(owners[i]);
		}
		return r;
	}

	@Override
	public StellarObjectOrbitalDelegate getStellarObjectOrbital() throws ApplicationError {
		StellarObjectVO value = getStellObjectValue();
		StellarObjectOrbitalVO sobv = value.getOrbital();
		if (sobv == null) return null;
		if (stellarObjectOrbitalDelegate == null) {
			if (sobv instanceof OutpostVO) stellarObjectOrbitalDelegate = new OutpostDelegate((OutpostVO)sobv);
			else if (sobv instanceof SpaceStationVO) stellarObjectOrbitalDelegate = new SpaceStationDelegate((SpaceStationVO)sobv);
			else throw new ApplicationError("Unknown stellar object orbital "+sobv);
		}
		return stellarObjectOrbitalDelegate;
	}
	
	@Override
    public boolean isGuardedByMonster() throws ApplicationError {
		PlayerDelegate mp = DeligateCacheManager.getMonsterPlayer();
		CraftDelegate[] crafts = DeligateCacheManager.getCraftsAt(mp, this);
		return crafts != null && crafts.length > 0;
	}


	@Override
	public boolean equals(StellarObjectDelegate other) {
		StellarObjectVO value = getStellObjectValue();
		return Comp.same(value.getName(), other);
	}

	public String toString() { return getName();  }

	public GalaxyMapLabelSizeDetails getGalaxyMapLabelSizeDetails(Graphics2D graphics) throws ApplicationError {
		if (galaxyMapLabelSizeDetails == null) {
	    	FontMetrics fontMetrics = graphics.getFontMetrics();
	    	int labelWidth = fontMetrics.stringWidth(getName());
	    	int labelHeight= fontMetrics.getHeight()-4;
			galaxyMapLabelSizeDetails = new GalaxyMapLabelSizeDetails(labelWidth, labelHeight);
		}
		return galaxyMapLabelSizeDetails;
	}

	public int getRange(PlayerDelegate owner) throws ApplicationError {
		StellarObjectDelegate[] soa = DeligateCacheManager.getStellarObjects();
		if (isOwnedBy(owner)) return -1;
		if (soa == null) return -1;
		Location loc = getLocation();
		int range = Integer.MAX_VALUE;
		for (int i = 0; i < soa.length; i++) {
			if (soa[i].isOwnedBy(owner)) {				
				int d = loc.getDistanceTo(soa[i].getLocation());
				if (d < range) {
					range = d; 
				}
			}
		}
		return range;
	}
	
	@Override
	public String getImage() throws ApplicationError {
		StellarObjectGraphics sog = getGraphics(true);
		return sog.getImage();
	}

	public boolean isWayPoint() throws ApplicationError {
    	OrbitVO fleetWayPoint = MainWindow.getOwner().getFleetWayPoint();
    	boolean isWaypoint = fleetWayPoint != null && fleetWayPoint.getPlanet() == null && Comp.same(fleetWayPoint.getStellarObject(), getName());
    	return isWaypoint;
	}
}
