/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import java.net.URL;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.TransportDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.CraftGraphics;
import mot.common.bean.PlayerColor;
import mot.common.bean.RaceGraphicsList;
import mot.common.bean.Size;
import mot.common.designs.TransportDesign;
import mot.common.factory.GraphicsFactory;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;
import mot.common.vo.MoveableOwnedObjectVO;
import mot.common.vo.NavigatorVO;
import mot.common.vo.OrbitVO;
import mot.common.vo.TransportVO;

public class MotTransportDelegate extends MotMoveableOwnedObjectDelegate implements TransportDelegate {

	private TransportVO value;

	private CraftGraphics graphics;
	private String image;
	
    private static final String IDLE_TEXT = "Idle";
    
	public MotTransportDelegate() {
	}
	
	public MotTransportDelegate(TransportVO vo) {
		value = vo;
	}

	public String getDescription(int maxLines, boolean withName, boolean withEta) throws ApplicationError {
		String taskDescription = getTaskDescription();
		String r;
        if (taskDescription == null) {
        	if (withName) {
                r = getName()+" : "+IDLE_TEXT;
        	} else {
        		r = IDLE_TEXT;
        	}
        } else {
        	if (withName) {
        		r = getName()+" : "+taskDescription;
        	} else {
        		r = taskDescription;
        	}
        }
		if (withEta) {
			String enrouteDescription = getEnrouteDescription();
			if (enrouteDescription != null) {
				r += " " + enrouteDescription;
			}
		}
        return r;
	}

	
    /**
     * @return the graphics used by this craft on the battle map and on the galaxy map.
     * This is changed in a transient attribute graphics and dynamically generated from
     * image.
     */
	public CraftGraphics getCraftGraphics() throws ApplicationError {
        String player = value.getOwner();
		if (graphics == null) {
			RaceGraphicsList graphicsList = GraphicsFactory.getRaceGraphicsList();
            graphics = graphicsList.getHullGraphics(player, "Transport");
		}
        if (graphics == null) {
            throw new ApplicationError("TransportDeligateHelperImpl.getCraftGraphics() No graphics defined for player="+player+" Transport");
        }
		return graphics;
	}
    
    public String getImage() throws ApplicationError {
        if (image == null) {
        	PlayerDelegate owner = DeligateCacheManager.getPlayer(value.getOwner());
        	PlayerColor pc = (owner != null) ? owner.getColor() : null;
            String colorName  = "";
            if (pc != null) colorName = pc.getName();
            image = CraftGraphics.CRAFT_IMAGE_FOLDER+getCraftGraphics().getPictureNormal() + "_" + colorName + ".png";
        	URL fileUrl = ClassLoader.getSystemResource(image);
            if (fileUrl == null) {
                image = CraftGraphics.CRAFT_IMAGE_FOLDER+getCraftGraphics().getPictureNormal() + ".png";
            }
        }
        return image;
    }

	public String get3DImage() throws ApplicationError {
    	if (graphics == null) {
    		graphics = getCraftGraphics();
    	}
		return CraftGraphics.CRAFT_IMAGE_FOLDER + graphics.getPicture3d();
	}


	public Size getImageSize() throws ApplicationError {
		return getCraftGraphics().getSize();
	}

	public int getQty() {
		return value.getQty();
	}

	public int getMaxQty() {
		return TransportDesign.getSpace();
	}

	public String getOrbitalHull() {
		return value.getOrbitalHull();
	}

	public Hull getOrbitalHullTech() throws ApplicationError {
		return Technologies.getHulls().getHull(value.getOrbitalHull());
	}

	public OrbitVO getDestination() throws ApplicationError {
		NavigatorVO navigator = getNavigator();
		if (navigator != null) {
			return navigator.getDestination();
		} else {
			return null;
		}
	}

	public String getOrigin() {
		return value.getOrigin();
	}
	
	public PlanetDelegate getOriginPlanet() {
		String origin = value.getOrigin();
		if (origin == null) {
			return null;
		} else {
			return DeligateCacheManager.getPlanet(origin);
		}
	}

	public String getTaskDescription() throws ApplicationError {
		OrbitVO destination = getDestination();
		if (containsColonists()) {
			int qty = value.getQty();
	        String d = L.MOVE_+qty+L._COLONISTS_;
	        if (destination != null) {
	            d += L._TO_+destination.getName();
	        }
	        return d;
		} else if (containsTroops()) {
			int qty = value.getQty();
	        String d = L.MOVE_+qty+L._TROOPS_;
	        if (destination != null) {
	            d += L._TO_+destination.getName();
	        }
	        return d;
		} else if (containsTradeRoute()) {
	        String d = L.CREATE_A_TRADE_ROUTE;
	        String origin = value.getOrigin();
	        if (origin != null) {
	            d += L._FOR_+origin;
	        }
	        return d;
		} else {
			String origin = value.getOrigin();
			String orbitalHull = value.getOrbitalHull();
	    	//if (destination == null) {
	    		if (origin != null) {
	    			return L.DEPLOY_STATION+orbitalHull + L._FOR_ + origin;
	    		} else {
	    			return L.DEPLOY_STATION+orbitalHull;
	    		}
//	    	} else {
//	    		if (origin != null) {
//	    			return L.DEPLOY_STATION+orbitalHull + L._FOR_ + origin + L._TO_ + destination;
//	    		} else {
//	        		return L.DEPLOY_STATION+orbitalHull + L._TO_ + destination;
//	    		}
//	    	}
		}
	}
	
	@Override
	public boolean hasMoved() {
		return value.getNavigator() != null;
	}


	public MoveableOwnedObjectVO getMoveableOwnedObjectVO() {
		return value;
	}

	@Override
	public void flushImageCache() {
		graphics = null;
		image = null;
	}

	public void update(TransportVO vo) {
		value = (TransportVO) vo;
	}

	
	public boolean galaxyMenuCanAttackActive() throws ApplicationError {
		return false;
	}
	
	public boolean galaxyMenuCanBombardActive() throws ApplicationError {
		return false;
	}
	
	public boolean galaxyMenuCanInvadeActive() throws ApplicationError {
		if (containsTroops()) {
			return isInOrbitRoundUnDefendedEnemy() && !isInOrbitWithEnemyCraft();
		}
		return false;
	}
	
	public boolean galaxyMenuCanColoniseActive() throws ApplicationError {
		return false;
	}
	
	public boolean galaxyMenuCanDeployActive() throws ApplicationError {
		if (containsColonists()) {
			return isOrbitingOwnersPlanet();
		} else if (containsSpaceStation()) {
			OrbitVO orbit = getOrbit();
			if (orbit != null) {
				if (orbit.getPlanet() != null) {
					PlanetDelegate planet = DeligateCacheManager.getPlanet(orbit.getPlanet());
					if (planet != null) {
						return planet.getOwner() == null || planet.isOwnedBy(MainWindow.getOwner());
					}
				} else if (orbit.getStellarObject() != null) {
					StellarObjectDelegate stellarObject = DeligateCacheManager.getStellarObject(orbit.getStellarObject());
					if (stellarObject != null) {
						return stellarObject.isOwnedBy(getOwnerDelegate()) || stellarObject.getOwners() == null;
					}
					
				}
				return true;
			}
		} else if (containsTradeRoute()) {
			return isOrbitingOwnersSun();
		}
		return false;
	}
	
	public boolean galaxyMenuCanJoinFleetActive() {
		return false;
	}
	
	public boolean galaxyMenuCanUpgradeActive() {
		return false;
	}
	
	public boolean galaxyMenuCanDeleteActive() {
		return true;
	}

	public char getTaskContent() {
		return value.getTaskContent();
	}
	
	public boolean containsColonists() {
		return value.getTaskContent() == TransportVO.CONTENT_TYPE_COLONISTS;
	}

	public boolean containsTroops() {
		return value.getTaskContent() == TransportVO.CONTENT_TYPE_TROOPS;
	}

	public boolean containsSpaceStation() {
		return value.getTaskContent() == TransportVO.CONTENT_TYPE_SPACE_STATION;
	}
	
	public boolean containsTradeRoute() {
		return value.getTaskContent() == TransportVO.CONTENT_TYPE_TRADE_ROUTE;
	}
}
