/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.deligates.mot;

import java.util.ArrayList;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.RelationDelegate;
import mot.client.deligates.ResearchProjectDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.common.bean.NameList;
import mot.common.bean.PlayerCharacterBasic;
import mot.common.bean.PlayerResources;
import mot.common.bean.SalvageList;
import mot.common.bean.Settings;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.lang.L;
import mot.common.leaders.GeneralDetailsList;
import mot.common.leaders.GeneralLeader;
import mot.common.leaders.GovenorDetailsList;
import mot.common.leaders.GovenorLeader;
import mot.common.technology.Biotechnologies;
import mot.common.technology.Computers;
import mot.common.technology.Engines;
import mot.common.technology.Factories;
import mot.common.technology.Hulls;
import mot.common.technology.Humanities;
import mot.common.technology.Laboratories;
import mot.common.technology.MiscTechnologies;
import mot.common.technology.Modules;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Specials;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Theories;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.CraftDesignVO;
import mot.common.vo.CraftDesignsVO;
import mot.common.vo.OrbitVO;
import mot.common.vo.PlayerConstantsVO;
import mot.common.vo.PlayerTechnologiesVO;
import mot.common.vo.PlayerVO;
import mot.common.vo.ResearchProjectUpdateVO;
import mot.common.vo.ResearchProjectVO;
import mot.common.vo.UserPlayerVO;

public class MotUserPlayerDelegate extends MotPlayerDelegate implements UserPlayerDelegate {

	private CraftDesignsVO playerCraftDesigns;
	private PlayerTechnologiesVO playerTechnologies;
	private UserPlayerVO player;

	public MotUserPlayerDelegate() {
	}
	
	public MotUserPlayerDelegate(UserPlayerVO vo) {
		player = vo;
	}

	@Override
	public boolean isAutomated() {
		return false;
	}

	@Override
	public boolean isMonsterPlayer() {
		return false;
	}

	@Override
    public String getTransportName() {
        return "Transport";
    }

	@Override
	public GeneralLeader getGeneralLeader() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		String generalName = value.getGeneralName();
		if (generalName != null) {
			return new GeneralLeader(GeneralDetailsList.getInstance().getGeneralDetails(generalName), value.getGeneralExperience());
		} else {
			return null;
		}
	}

	@Override
	public GovenorLeader getGovenorLeader() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		String govenorName = value.getGovenorName();
		if (govenorName != null) {
			return new GovenorLeader(GovenorDetailsList.getInstance().getGovenorDetails(govenorName), value.getGovenorExperience());
		} else {
			return null;
		}
	}

	@Override
	public CraftDesign getLastCraftDesigned() throws ApplicationError {
		CraftDesignsVO value = getPlayerCraftDesignsValue();
		String name = null;
		if (value != null) {
			name = value.getLastCraftDesigned();
		}
		if (name == null) return null;
		CraftDesigns craftDesigns = getCraftDesigns();
		if (craftDesigns == null) return null;
		return craftDesigns.get(name);
	}

	@Override
	public boolean canViewDetails(PlayerDelegate pd) throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		return pd != null && Comp.in(pd.getName(), value.getPlayerSpies());
	}

	@Override
	public OrbitVO getFleetWayPoint() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		return value.getFleetWayPoint();
	}

	@Override
	public NameList getWeaponsToExcludeFromFireAll() throws ApplicationError {
		String[] names = Settings.getInstance().getWeaponsToExcludeFromFireAll();
		return new NameList(names);
	}

	@Override
	public Biotechnologies getBiotechnologies() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getBiotechnologiesAsList();
	}

	@Override
	public Computers getComputers() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getComputersAsList();
	}

	@Override
	public Engines getEngines() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getEnginesAsList();
	}

	@Override
	public Factories getFactories() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getFactoriesAsList();
	}

	@Override
	public Hulls getHulls() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getHullsAsList();
	}

	@Override
	public Laboratories getLaboratories() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getLaboratoriesAsList();
	}

	@Override
	public Scanners getScanners() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getScannersAsList();
	}

	@Override
	public Shields getShields() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getShieldsAsList();
	}

	@Override
	public Specials getSpecials() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getSpecialsAsList();
	}

	@Override
	public Weapons getWeapons() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getWeaponsAsList();
	}

	@Override
	public Modules getModules() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getModulesAsList();
	}

	@Override
	public Humanities getHumanities() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getHumanitiesAsList();
	}

	@Override
	public Theories getTheories() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getTheoriesAsList();
	}

	@Override
	public MiscTechnologies getMiscTechnologies() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getMiscTechnologiesAsList();
	}
	
	@Override
	public TechnologyList getTechnologies() throws ApplicationError {
		PlayerTechnologiesVO value = getPlayerTechnologiesValue();
		return value.getAllAsList();
	}
	
	@Override
	public PlanetDelegate getHomePlanet() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		if (value == null) return null;
		String homePlanetName = value.getHomePlanet();
		return DeligateCacheManager.getPlanet(homePlanetName);
	}

	@Override
	public PlayerResources getPlayerResources() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		if (value == null) return null;
		return value.getPlayerResources();
	}

	@Override
	public TechnologyList getMissedTechnologies() throws ApplicationError {
		PlayerConstantsVO value = getConstantsValue();
		if (value == null) return null;
		PlayerCharacterBasic pc = value.getPlayerCharacter();
		if (pc == null) return null;
		TechnologyList mt = pc.getMissedTechnologiesAsList();
		return (mt == null || mt.getQuantity() == 0) ? null : mt;
	}

	@Override
	public SalvageList getSalvage() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		if (value == null) return null;
		String[] salvage = value.getSalvage();
		return new SalvageList(salvage);
	}

	@Override
	public ResearchProjectDelegate getResearchProject() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		ResearchProjectVO researchProjectVO = value.getResearchProject();
		if (researchProjectVO == null) {
			return null;
		} else {
			return new ResearchProjectDelegate(researchProjectVO);
		}
	}

	@Override
    public CraftDesigns getCraftDesigns() throws ApplicationError {
		CraftDesignsVO value = getPlayerCraftDesignsValue();
		return value == null ? null : value.getCraftDesigns();
    }
	
	@Override
	public double getBuildQuality(String name) {
		CraftDesignsVO value = getPlayerCraftDesignsValue();
		return value == null ? 0.0 : value.getBuildQuality(name);
	}
	
	@Override
	public int getNetIncome() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		if (value == null) return 0;
		return value.getNetIncome();
	}

	@Override
	public int getQtyFreeTransport() {
		UserPlayerVO value = getUserPlayerValue();
		if (value == null) return 0;
		return value.getQtyFreeTransport();
	}
	
	@Override
	public int getQtySpiesDefending() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		if (value == null) return 0;
		return value.getQtySpiesDefending();
	}

	@Override
	public double getChanceToDiscoverSalvage() {
		UserPlayerVO value = getUserPlayerValue();
		if (value == null) return 0;
		return value.getChanceToDiscoverSalvage();
	}

	@Override
	public int getTotalResearch() throws ApplicationError {
		UserPlayerVO value = getUserPlayerValue();
		if (value == null) return 0;
		return value.getTotalResearch();
	}
	
	//
	// STATE MAINTENAINCE METHODS
	//
	
    public double getTradeRouteBonus() throws ApplicationError {
    	Humanities humanities = getHumanities();
    	if (humanities != null) {
    		return humanities.getTradeRouteBonus();
    	} else {
    		return 0;
    	}
    }

	@Override
	public PlayerVO getPlayerValue() {
		return player;
	}

	public UserPlayerVO getUserPlayerValue() {
		return player;
	}
	
	public PlayerTechnologiesVO getPlayerTechnologiesValue() {
		return playerTechnologies;
	}
	
	public CraftDesignsVO getPlayerCraftDesignsValue() {
		return playerCraftDesigns;
	}

	public void update(CraftDesignsVO vo) {
		playerCraftDesigns = vo;
	}

	public void update(CraftDesignVO vo) {
		if (playerCraftDesigns == null) {
			playerCraftDesigns = new CraftDesignsVO();
		}
		playerCraftDesigns.update(vo);
	}

	public void update(PlayerTechnologiesVO vo) {
		playerTechnologies = vo;
	}

	public void update(ResearchProjectUpdateVO vo) {
		if (vo != null && player != null) {
			player.setResearchProject(vo.getProject());
		}
	}

	public void update(UserPlayerVO vo) {
		player = vo;
	}


	public void addTechnology(Technology t) throws ApplicationError {
		if (playerTechnologies == null) {
			playerTechnologies = new PlayerTechnologiesVO(getName(), null);
		}
		playerTechnologies.add(t);
	}

	public SolarSystemDelegate [] getSolarSystemsOwnedBy() throws ApplicationError {
		StellarObjectDelegate[] sos = DeligateCacheManager.getStellarObjects();
		if (sos == null) return null;
		ArrayList r = null;
		for (int i = 0; i < sos.length; i++) {
			if (sos[i] instanceof SolarSystemDelegate && sos[i].isOwnedBy(this)) {
				if (r == null) r = new ArrayList();
				r.add(sos[i]);
			}
		}
		if (r == null) {
			return null;
		} else {
			return (SolarSystemDelegate[]) r.toArray(new SolarSystemDelegate[r.size()]);
		}
	}

	@Override
	public boolean hasDiscovered(Technology tech) throws ApplicationError {
		TechnologyList techs = getTechnologies();
		return techs.contains(tech);
	}

	@Override
	public String[] getTreatyNames(PlayerDelegate craftOwner) throws ApplicationError {
        RelationDelegate relation = getRelation(craftOwner);
        ArrayList r = new ArrayList();
        if (relation == null) {
       		r.add(L.NO_DIPLOMATIC_RELATIONS);
        } else {
        	if (relation.isAliance()) r.add(L.TREATY_ALLIANCE);
        	if (relation.isNonAgression()) r.add(L.TREATY_NON_AGRESSION);
        	if (relation.isPeace()) r.add(L.TREATY_PEACE);
        	if (relation.isWar()) r.add(L.TREATY_WAR);
        	if (relation.getTradeAgreement() != null) r.add(L.TREATY_TRADE);
        	if (relation.getResearchAgreement() != null) r.add(L.TREATY_RESEARCH);
        }
        return (String[]) r.toArray(new String[r.size()]);
	}

}
