/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.event;

import mot.common.event.MotEvent;
import mot.common.util.ApplicationError;

/**
 * Interface defining a service that receives incoming events (from the engine to the client)
 * and processes them.
 * 
 * This is registered with the main window so it can poll the incoming event queue using an 
 * implementation  of this interface.
 * 
 */
public interface EventMailBox {

	/**
	 * Check for events in the client's incoming event queue and process them.
	 * This method should be called every frame in the main window to ensure all events are processed. 
	 * @throws ApplicationError
	 */
	void processQueuedEvents() throws ApplicationError;
	
	/**
	 * Is the next event in the client event queue a battle event?
	 */
	boolean isNextEventBattleEvent() throws ApplicationError;

	/**
	 * @return True iff the event main box contains messages from the server that are awaiting processing by the client (in galaxy mode).
	 */
	boolean hasGalaxyEventsInQueue();

	/**
	 * @return TRUE if there is an event in the queue to be processed of this class.
	 */
	public boolean hasEventTypeInQueue(Class eventClass);
	public boolean hasEventTypeInQueue(Class eventClass1, Class eventClass2);

	/**
	 * This method does not block and can return null.
	 * @param eventClass The type of event to search for in the event queue.
	 * @return Fetch the first event of type passed in that is in the event queue or return null if there are not any.
	 */
	MotEvent fetchFirstEventType(Class eventClass);
	
	/**
	 * This method does not block.
	 * @param eventClass The type of event to search for in the event queue.
	 * @return Process the first event of type passed in that is in the event queue if one exists.
	 * @throws ApplicationError 
	 */
	void processFirstEventType(Class eventClass) throws ApplicationError;
	
	/**
	 * Adds an event to the in queue for the GUI that will be processed. This is used for the GUI to send its self an event!
	 * @param event The new event to add to the queue
	 * @throws ApplicationError
	 */
	void addEventToQueue(MotEvent event) throws ApplicationError;
}
