/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.event;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.ProxyStore;
import mot.client.swing.window.battle.BattleWindowConfig;
import mot.client.swing.window.main.MainWindow;
import mot.client.tutorials.Tutorial;
import mot.client.tutorials.TutorialSystem;
import mot.common.coms.protocols.ClientEventProcessor;
import mot.common.event.BattleEndEvent;
import mot.common.event.MotEvent;
import mot.common.event.NewBattleEvent;
import mot.common.event.UrgentEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

/**
 * This class is the connection from the server to the client (for local games - engine 
 * and GUI in same VM).
 * 
 * MOT ENGINE
 * 	Uses ClientProxyManager to contain a list of of ClientProxy implementations.
 * 	This is a ClientProxy implementation that is contained in the Engine.
 * 
 * Client GUI
 * 	Uses ClientEventProcessor as the root source of all events.
 * 	ClientEventProcessor.process(Event e) must only be called in the client GUI Thread.
 * 
 * This is a proxy class for local clients. It uses a local copy (in process) of the
 * MotEngine to effect its commands.
 * All changes/commands issued by a player must go through a proxy so the client is
 * Separated from the engine allowing distributed operation.
 */
public class MotClientEventProcessor implements ClientEventProcessor, EventMailBox {


    /**
     * This is the object that consumes events - The Swing Event consumer or a Test Harness
     */
    protected MotEventConsumer eventConsumer;
    
    /**
     * The player who owns this proxy stub.
     */
    protected String owner;

	private BlockingQueue<MotEvent> queue = new LinkedBlockingQueue<MotEvent>();
	
	public MotClientEventProcessor(String owner, MotEventConsumer eventConsumer) {
        this.eventConsumer = eventConsumer;
        this.owner = owner;
	}

	/*
	 * ONLY CALLED IN CLIENT GUI THREAD
	 */


	/**
	 * A list of galaxy events that have been received while the battle window was open.
	 * We cannot process them so we store then until the galaxy window opens again.
	 */
	List<MotEvent> galaxyEventQueue = new ArrayList<MotEvent>();

	public boolean hasGalaxyEventsInQueue() {
		return (galaxyEventQueue.size() > 0) || isNextEventGalaxyEvent();
	}

	public boolean hasEventTypeInQueue(Class eventClass) {
		int qty = galaxyEventQueue.size();
		for (int i = 0; i < qty; i++) {			
			MotEvent event = galaxyEventQueue.get(i);
			if (eventClass.isInstance(event)) {
				return true;
			}
		}
		if (queue.size() > 0) {
			Object[] a = queue.toArray();
			for (int i = 0; i < a.length; i++) {
				if (eventClass.isInstance(a[i])) {
					return true;
				}
			}
		}
		return false;
	}
	
	public boolean hasEventTypeInQueue(Class eventClass1, Class eventClass2) {
		int qty = galaxyEventQueue.size();
		for (int i = 0; i < qty; i++) {			
			MotEvent event = galaxyEventQueue.get(i);
			if (eventClass1.isInstance(event) || eventClass2.isInstance(event)) {
				return true;
			}
		}
		if (queue.size() > 0) {
			Object[] a = queue.toArray();
			for (int i = 0; i < a.length; i++) {
				if (eventClass1.isInstance(a[i]) || eventClass2.isInstance(a[i])) {
					return true;
				}
			}
		}
		return false;
	}
	
	public MotEvent fetchFirstEventType(Class eventClass) {
		int qty = galaxyEventQueue.size();
		for (int i = 0; i < qty; i++) {			
			MotEvent event = galaxyEventQueue.get(i);
			if (eventClass.isInstance(event)) {
				galaxyEventQueue.remove(event);
				return event;
			}
		}
		return null;
	}
	
	public void processFirstEventType(Class eventClass) throws ApplicationError {
		//Logger.logDebug("MotClientEventProcessor.processFirstEventType() class="+eventClass);
		int qty = galaxyEventQueue.size();
		for (int i = 0; i < qty; i++) {			
			MotEvent event = galaxyEventQueue.get(i);
			if (eventClass.isInstance(event)) {
				galaxyEventQueue.remove(event);
				handle(event);
			}
		}
		try {			
			Iterator<MotEvent> iterator = queue.iterator();
			if (iterator != null) {
				while (iterator.hasNext()) {
					MotEvent event = iterator.next();
					if (eventClass.isInstance(event)) {
						queue.remove(event);
						handle(event);
						return;
					}
				}
			}
		} catch (Exception ex) {
			Logger.logError("IGNORED", ex);
		}
	}
	
	private boolean processingEvent = false;
	
	public void processQueuedEvents() throws ApplicationError {
    	// Events can be processed by the mail box timer AND command responses
    	// We do not want to process events from the event timer while processing an event from the command response
		if (!processingEvent) {
			// We will not process events from engine while window is open because user may be doing something and we will
			// close the window to open another.
			if (!MainWindow.hasDialogOpen()) {				
				try {
					processingEvent = true;
					_processQueuedEvents();
				} finally {
					processingEvent = false;
				}
			} else {
				MotEvent event = queue.peek();
				if (event != null && event instanceof UrgentEvent) {
					handle(queue.poll());
				}
			}
		}
	}
	
	private void _processQueuedEvents() throws ApplicationError {
		MotEvent event = null;
		MainWindow mainWin = MainWindow.getInstance();
		
		if (TutorialSystem.isActive()) {
			Tutorial openTip = TutorialSystem.getOpenTutorial();
			if (openTip != null) {
				Class [] c = openTip.getEventToProcess();
				if (c != null) {
					for (int i = 0; i < c.length; i++) {						
						processFirstEventType(c[i]);
					}
				}
				return;
			}
		}
		
		if (mainWin.isGalaxyMapConfigurationActive()) {
			while (galaxyEventQueue.size() > 0) {
				event = galaxyEventQueue.get(0);
				galaxyEventQueue.remove(event);
				handle(event);
			}
		}
		
		do {			
			event = queue.peek();
			if (event != null) {
				if (mainWin.isBattleMapConfigurationActive()) {
					boolean showingAnimations = mainWin.getBattleWinConfig().getListener().isShowingAnimations();
					// Do not want to process the end of battle event if showing animations
					if (showingAnimations && event instanceof BattleEndEvent) {
			            BattleWindowConfig config = mainWin.getBattleWinConfig();
			            if (config != null) config.getListener().setEndBattleRecieved(true);
						return;
					}
					// Method poll() does not block, it returns null if the queue is empty.
					event = queue.poll();
					if (!event.isBattleEvent()) {
						// Must not process galaxy events when battle window is open
						// Battle has just started BUT Server command queue contained an action that caused a galaxy event
						// which has now appeared as an event.
						if (Logger.isDebug()) {
							Logger.logDebug("MotClientEventProcessor.processQueuedEvents() - store galaxy event "+event);
						}
						galaxyEventQueue.add(event);
					} else {
						handle(event);
					}
				} else {
					// Do not want to open battle window when processing a command
					if (ProxyStore.hasInstance() && ProxyStore.getInstance().isBusy() &&  event instanceof NewBattleEvent) {
						return;
					}
					// Method poll() does not block, it returns null if the queue is empty.
					event = queue.poll();
					// Must not process battle events when galaxy window is still open
					// Probably means a battle has is about to end so hasEnded() is returning true. So the battle window closes
					// just before the last weapon fire event is received and processed.
					if (!event.isGalaxyEvent()) {
						// This happens when we complete a battle (all remaining battle events should be ignored).
						Logger.logDebug("MotClientEventProcessor.processQueuedEvents() has received event "+event+" when GalaxyWindow is open");
					} else {
						handle(event);
					}
				}
			}
		} while (event != null);
	}
	
	public boolean isNextEventBattleEvent() {
		MotEvent event = queue.peek();
		if (event != null) {
			return event.isBattleEvent();
		}
		return false;
	}
	
	public boolean isNextEventGalaxyEvent() {
		MotEvent event = queue.peek();
		if (event != null) {
			return event.isGalaxyEvent();
		}
		return false;
	}
	
	public void addEventToQueue(MotEvent event) throws ApplicationError {
		process(event);
	}	
	

    public String getOwner() {
        return owner;
    }	
	
	public void handle(MotEvent e) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("MotClientEventProcessor.handle() Process MotEvent="+e);
        }
		DeligateCacheManager.update(e);
        try {
            eventConsumer.process(e, null);
        } catch (ValidationError ex) {
            MainWindow.getExestingInstance().reportValidationError(ex);
        }
    }
    
	
	/**
	 * 
	 * 
	 * METHODS BELOW THIS POINT ARE CALLED BY THE ENGINE USING THE ENGINE OR COMMUNICATION RECEIVER THREAD
	 * 
	 * 
	 */
	
	

	
	public void process(MotEvent e) throws ApplicationError {
//		if (Logger.isDebug()) {
//            Logger.logDebug("ClientStubBase.process() Process MotEvent="+e.getClass().getName());
//        }
		if (e == null) {
			// Container Events can be null
			Logger.logDebug("LocalClientStubAndProxy.process(MotEvent) - Null Event");
			return;
		}
		try {
			queue.put(e);
		} catch (InterruptedException e1) {
			Logger.logError(e1.getMessage(), e1);
			throw new ApplicationError("MotClientEventProcessor.processQueuedEvents() Event Lost - "+e1);
		}
    }

    public String getOwnerName() {
		return owner;
	}

	public void setOwnerName(String ownerName) {
		this.owner = ownerName;
	}


}