/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.proxy;

import java.net.InetAddress;
import java.net.UnknownHostException;

import javax.swing.SwingUtilities;

import mot.client.coms.DebugCommandResponseHook;
import mot.client.coms.LocalServerConnection;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.TransportDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.BattleDelegateCache;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.event.MotClientEventProcessor;
import mot.client.event.MotEventConsumer;
import mot.client.sandbox.battle.BattleSandbox;
import mot.client.swing.event.SwingMotEventConsumer;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.NewGameData;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.command_delegates.AbandonCmdDeligate;
import mot.common.command_delegates.AcceptLeaderCmdDeligate;
import mot.common.command_delegates.AcceptMoralChoiceCmdDeligate;
import mot.common.command_delegates.AddCraftDesignDeligate;
import mot.common.command_delegates.BattleAttackCmdDeligate;
import mot.common.command_delegates.BattleAttackLocationCmdDeligate;
import mot.common.command_delegates.BattleAutomateCmdDeligate;
import mot.common.command_delegates.BattleCraftLaunchFightersCmdDeligate;
import mot.common.command_delegates.BattleCraftMoveToCmdDeligate;
import mot.common.command_delegates.BattleCraftWaitCmdDeligate;
import mot.common.command_delegates.BattleEndTurnCmdDeligate;
import mot.common.command_delegates.BattleGetNextObjectCmdDeligate;
import mot.common.command_delegates.BattleGetNextOrdersCmdDeligate;
import mot.common.command_delegates.BattleRetreatCmdDeligate;
import mot.common.command_delegates.ClientClosesWindowCmdDeligate;
import mot.common.command_delegates.CommandDeligate;
import mot.common.command_delegates.CraftAttackCmdDeligate;
import mot.common.command_delegates.CraftBombardPlanetCmdDeligate;
import mot.common.command_delegates.CraftColoniseCmdDeligate;
import mot.common.command_delegates.CraftLeaveFleetCmdDeligate;
import mot.common.command_delegates.DesignColonyCraftCmdDeligate;
import mot.common.command_delegates.DesignMillitaryCraftCmdDeligate;
import mot.common.command_delegates.DiplomaticRequestCmdDeligate;
import mot.common.command_delegates.DiplomaticResponseCmdDeligate;
import mot.common.command_delegates.EndTurnCmdDeligate;
import mot.common.command_delegates.FetchDelegateStateCmdDelegate;
import mot.common.command_delegates.FetchGalaxyPreview;
import mot.common.command_delegates.FetchPlayerDetailsCmdDelegate;
import mot.common.command_delegates.FetchRegisteredPlayersCmdDelegate;
import mot.common.command_delegates.FetchSavedGameFiles;
import mot.common.command_delegates.FetchTechnologiesCmdDelegate;
import mot.common.command_delegates.GetCraftDesignStatistics;
import mot.common.command_delegates.GetGraphCmdDeligate;
import mot.common.command_delegates.IncreaseDevelopmentCmdDeligate;
import mot.common.command_delegates.IncreaseEcologyCmdDeligate;
import mot.common.command_delegates.JoinFleetCmdDeligate;
import mot.common.command_delegates.LoadCmdDeligate;
import mot.common.command_delegates.MooDeleteCmdDeligate;
import mot.common.command_delegates.MooDeployCmdDeligate;
import mot.common.command_delegates.MooMoveCmdDeligate;
import mot.common.command_delegates.MooRenameCmdDeligate;
import mot.common.command_delegates.MooSleepCmdDeligate;
import mot.common.command_delegates.NewGameCmdDeligate;
import mot.common.command_delegates.NewGameUserPlayerCmdDeligate;
import mot.common.command_delegates.NotificationCommandDelegate;
import mot.common.command_delegates.PurchaseProjectCmdDeligate;
import mot.common.command_delegates.RegisterProxyCmdDeligate;
import mot.common.command_delegates.RemoveCraftDesignDeligate;
import mot.common.command_delegates.RenamePlanetCmdDeligate;
import mot.common.command_delegates.RenameStellarObjectCmdDeligate;
import mot.common.command_delegates.SaveCmdDeligate;
import mot.common.command_delegates.SellSalvageCmdDeligate;
import mot.common.command_delegates.SetFleetWayPointCmdDelegate;
import mot.common.command_delegates.SetPlanetAutoPercentsCmdDeligate;
import mot.common.command_delegates.SetPlanetPercentsCmdDeligate;
import mot.common.command_delegates.SetPlanetWantsNoProjectCmdDeligate;
import mot.common.command_delegates.SetPlayerCharacterPercentsDeligate;
import mot.common.command_delegates.SetProjectCmdDeligate;
import mot.common.command_delegates.SetRecruiteSpyEffortCmdDeligate;
import mot.common.command_delegates.SetResearchProjectCmdDeligate;
import mot.common.command_delegates.SpyRoleAddEspionageCmdDeligate;
import mot.common.command_delegates.SpyRoleAddPlayerCmdDeligate;
import mot.common.command_delegates.SpyRoleAddSabotageCmdDeligate;
import mot.common.command_delegates.SpyRoleAddStellarObjectCmdDeligate;
import mot.common.command_delegates.SpyRoleSetEspionageTargetCmdDeligate;
import mot.common.command_delegates.SpyStealableTechnologiesCmdDelegate;
import mot.common.command_delegates.TransportLoadCmdDeligate;
import mot.common.command_results.ApplicationErrorCR;
import mot.common.command_results.CommandResult;
import mot.common.command_results.ValidationErrorCR;
import mot.common.coms.protocols.RemoteServerConnection;
import mot.common.coms.protocols.ServerConnection;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.designs.ProjectDesign;
import mot.common.event.StateContainer;
import mot.common.lang.L;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.vo.OrbitVO;
import mot.common.vo.TechnologyListVO;
import mot.test.TestBase;

public class MotServerProxy implements ServerProxy, CommandResponseProcessor {
	
	public static final long TIME_OUT_PEROID = 10000;
	public static final long DELETE_PEROID = 10*60000;
	
	private ServerConnection connection;
	private DebugCommandResponseHook commandResponseHook;
	private MotClientEventProcessor eventProcessor;
	private UserPlayerDelegate owner;

	private MotServerProxyCommandList currentCommands = new MotServerProxyCommandList();
	
	private String clientId;

	public MotServerProxy(boolean useDebugHook, MainWindow mainWin, MotEventConsumer testEventConsumer) throws ApplicationError {
		if (testEventConsumer != null) {
			eventProcessor = new MotClientEventProcessor(null, testEventConsumer);
		} else {
			eventProcessor = new MotClientEventProcessor(null, new SwingMotEventConsumer(mainWin));
		}
		mainWin.setEventMailBox(eventProcessor);
		
		if (useDebugHook) {
			connection = new LocalServerConnection(this, eventProcessor);
			LocalServerConnection lc = (LocalServerConnection) connection;
			commandResponseHook = new DebugCommandResponseHook();
			lc.setDebugHook(commandResponseHook);
			clientId = ServerConnection.LOCAL_CLIENT_ID;
		}
		
	}
	
	@Override
	public void connect(String ip) throws ApplicationError, ValidationError {
		if (ip == null) {
			if (connection == null) {
				connection = new LocalServerConnection(this, eventProcessor);
			} else if (connection instanceof RemoteServerConnection) {
				connection.disconnect();
				connection = new LocalServerConnection(this, eventProcessor);
			}
			clientId = ServerConnection.LOCAL_CLIENT_ID;
		} else {
			if (connection == null) {
				connection = new RemoteServerConnection(this, eventProcessor);
			} else if (connection instanceof LocalServerConnection) {
				connection.disconnect();				
				connection = new RemoteServerConnection(this, eventProcessor);
			}
			connection.connect(ip);
			try {
				clientId = InetAddress.getLocalHost().getHostAddress();
			} catch (UnknownHostException e) {
				clientId = "" + Utils.randomDouble(1.0);
			}
		}
	}
	
	@Override
	public void disconnect() {
		if (connection != null) {
			connection.disconnect();
			connection = null;
		}
	}

    public boolean isConnected() { 
    	return (connection != null) && connection.isConnected(); 
    }
    
    public ServerConnection getConnection() throws ApplicationError { 
		if (connection == null) {
			try {
				connect(null);
			} catch (ValidationError e) {
				throw new ApplicationError(e.getMessage());
			}
		}
    	return connection; 
    }
    

	@Override
	public void processTimeouts() throws ApplicationError {
		if (currentCommands.getQuantity() > 0) {
			currentCommands.processTimeouts();
		}
	}

	private void send(CommandDeligate commandDelegate) throws ValidationError, ApplicationError {
		if (commandDelegate instanceof NotificationCommandDelegate) {
			if (connection == null) connect(null);
			connection.send(commandDelegate);
		} else {
			send(commandDelegate, null, null);
		}
	}
	
	private void send(CommandDeligate commandDelegate, CompletedCallback callback, ClientErrorHandler errorHandler) throws ValidationError, ApplicationError {
		if (commandResponseHook != null) commandResponseHook.reset();
		if (callback == null) {
			callback = new CompletedClientCallbackSyncroniser();
		}
		MotServerProxyCommand command = new MotServerProxyCommand(commandDelegate, callback, errorHandler);
		currentCommands.add(command);
		commandDelegate.setSequence(command.sequence);
		if (connection == null) connect(null);
		connection.send(commandDelegate);
	}

	public void processCommandResponse(CommandResult cr) throws ApplicationError, ValidationError {
		// Because events were generated and sent before this command result they must be processed first
		// otherwise state update order will be incorrect!
		if (!SwingUtilities.isEventDispatchThread() && !TestBase.__TEST_RUNNING) {
			Logger.logStackTrace();
			// TODO - REMOVE ME
		}
		MotServerProxyCommand command = currentCommands.getCommand(cr.getSequence(), true);
		if (eventProcessor != null) {
			eventProcessor.processQueuedEvents();
		}
		if (Logger.isFull()) {
			Logger.logDebug("MotServerProxy() Command Response: "+cr);
		}
		if (command == null) {
			if (commandResponseHook == null) {
				MainWindow.getInstance().showTemporaryMessage(L.LAST_COMMAND_CANCELLED, true, null, 0);
			}
			Logger.logDebug("Command Lost for "+cr);
			return;
		}
		if (cr instanceof ValidationErrorCR) {
			ValidationError ex = new ValidationError(((ValidationErrorCR)cr).getMessage());
			if (command.errorHandler != null) {
				command.errorHandler.reportValidationError(ex);
			} else {
				MainWindow.getExestingInstance().reportValidationError(ex);
			}
		} else if (cr instanceof ApplicationErrorCR) {
			ApplicationError ex = new ApplicationError(((ApplicationErrorCR)cr).getMessage());
			if (command.errorHandler != null) {
				command.errorHandler.reportApplicationError(ex);
			} else {
				MainWindow.getExestingInstance().reportApplicationError(ex);
			}
		} else {
			if (cr instanceof StateContainer) {
				DeligateCacheManager.update((StateContainer)cr);
			}
			if (command.callback != null) {
				CompletedCallback cb = command.callback;
				if (cb != null) {
					cb.complete(cr);
				}
			}
		}
	}
	
	public void _processCommandResponse() throws ApplicationError, ValidationError {
		while (!commandResponseHook.isCompleted()) {
			Thread.yield();
		}
		CommandResult cr = commandResponseHook.getCommandResponse();
		processCommandResponse(cr);
	}
	
	// For test harness
	public void _processQueuedEvents() throws ApplicationError {
		eventProcessor.processQueuedEvents();
	}
	
	public boolean isBusy() {
		return currentCommands.getQuantity() > 0;
	}

    public MotClientEventProcessor getClientEventProcessor() { return eventProcessor; }
    
    public boolean supportsLocalServerProxy() { return (connection.getLocalServerProxy() != null); }
    public LocalServerProxy getLocalServerProxy() { return connection.getLocalServerProxy(); }

    public RemoteConnectionServerProxy getRemoteConnectionServerProxy() throws ApplicationError { 
		if (connection == null) {
			getConnection();
		}
		return connection.getRemoteConnectionServerProxy();
    }


	@Override
	public void register(String playerName, boolean updateClient, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		CompletedCallback subCallback = new CompletedCallback() {
			private CompletedCallback mainCallback;
			private String playerName;
			public CompletedCallback init(CompletedCallback mainCallback, String playerName) { this.mainCallback = mainCallback; this.playerName = playerName; return this; }
			public void complete(Object result) throws ApplicationError, ValidationError {
				owner = (UserPlayerDelegate) DeligateCacheManager.getPlayer(playerName);
				mainCallback.complete(owner);
			}
		}.init(callback, playerName);
		if (eventProcessor != null) {
			eventProcessor.setOwnerName(playerName);
		}
//		if (eventConsumer != null) {
//	        eventProcessor = new MotClientEventProcessor(playerName, eventConsumer);
//	        connection.register(eventProcessor);
//	        if (MainWindow.hasInstance()) {
//	        	MainWindow.getInstance().setEventMailBox(eventProcessor);
//	        } else {
//	        	Logger.logWarning("MotServerProxy.register() No Main Window");
//	        	//Logger.logStackTrace();
//	        }
//		}
		if (playerName != null) {
			send(new RegisterProxyCmdDeligate(playerName, null, updateClient), subCallback, null);
		}
	}

	@Override
	public void newGameUserPlayer(NewGameUserPlayerData userData, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        send(new NewGameUserPlayerCmdDeligate(clientId, userData), callback, errorHandler);
	}

	@Override
	public void newGame(NewGameUserPlayerData userData, NewGameData gameData, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		if (connection == null) connect(null);
        send(new NewGameCmdDeligate(clientId, userData, gameData), callback, errorHandler);
	}

	@Override
	public void getSavedGameFiles(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
    	send(new FetchSavedGameFiles(), callback, errorHandler);
	}

	@Override
	public void load(String fileName, String playerName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
	    send( new LoadCmdDeligate(fileName, playerName), callback, errorHandler);		
	}

	@Override
	public void save(String fileName, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send( new SaveCmdDeligate(fileName), null, errorHandler);
	}

	@Override
	public void fetchGalaxyPreview(String fileName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
    	send(new FetchGalaxyPreview(fileName), callback, errorHandler);
	}
	
	@Override
	public void fetchRegisteredPlayers(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
    	send(new FetchRegisteredPlayersCmdDelegate(), callback, errorHandler);
	}

	@Override
	public UserPlayerDelegate getOwner() throws ApplicationError {
		return owner;
	}

	@Override
	public void getCraftDesignStatistics(CraftDesign design, PlayerDelegate owner, PlayerDelegate origin, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		if (origin == null) origin = owner;
		send(new GetCraftDesignStatistics(design, owner.getName(), origin.getName()), callback, errorHandler);
	}

	@Override
	public void designMilitaryCraft(String hullName, OrbitVO orbit, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new DesignMillitaryCraftCmdDeligate(hullName, orbit), callback, errorHandler);
	}

	@Override
	public void designColonyCraft(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new DesignColonyCraftCmdDeligate(), callback, errorHandler);
	}

	@Override
	public void getGraph(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new GetGraphCmdDeligate(), callback, errorHandler);
	}

	@Override
	public void mooDeleteCmd(MoveableOwnedObjectDelegate craft, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new MooDeleteCmdDeligate(craft.getName()), callback, errorHandler);
		
	}

	@Override
	public void craftAttackCmd(MoveableOwnedObjectDelegate craft, PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		String p = (planet == null) ? null : planet.getName();
		send(new CraftAttackCmdDeligate(craft.getName(), p), callback, errorHandler);
		
	}

	@Override
	public void craftBombardPlanetCmd(MoveableOwnedObjectDelegate craft, PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new CraftBombardPlanetCmdDeligate(craft.getName(), planet.getName()), callback, errorHandler);
		
	}

	@Override
	public void invadePlanet(TransportDelegate transport, PlanetDelegate planet, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		if (planet == null) return;
		send(new MooDeployCmdDeligate(transport.getName(), planet.getSolarSystemName(), planet.getName()), null, errorHandler);
	}

	@Override
	public void craftLeaveFleetCmd(CraftDelegate craft, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new CraftLeaveFleetCmdDeligate(craft.getName()), callback, errorHandler);
		
	}

	@Override
	public void joinFleetCmd(MoveableOwnedObjectDelegate craft, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new JoinFleetCmdDeligate(craft.getName()), callback, errorHandler);
		
	}

	@Override
	public void endTurnCmd(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new EndTurnCmdDeligate(), callback, errorHandler);
	}

	@Override
	public void craftColoniseCmd(MoveableOwnedObjectDelegate craft, PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new CraftColoniseCmdDeligate(craft.getName(), planet.getName()), callback, errorHandler);
	}

	@Override
	public void setResearchProjectCmd(Technology tech, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SetResearchProjectCmdDeligate(tech.getFullName()), callback, errorHandler);
	}

	private boolean autoOn = false;
	public void setResearchProjectAuto(boolean autoOn) { this.autoOn = autoOn; }
	public boolean isResearchProjectAuto() { return autoOn; }

	@Override
	public void setProjectCmd(PlanetDelegate planet, ProjectDesign project, int repeat, boolean allIdle, boolean useSalvage, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SetProjectCmdDeligate(planet.getName(), project, repeat, allIdle, useSalvage), callback, errorHandler);
		
	}
	
	@Override
	public void setProjectCmd(PlanetDelegate [] planets, ProjectDesign project, int repeat, boolean allIdle, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		String[] names = Comp.getNames(planets);
		send(new SetProjectCmdDeligate(names, project, repeat, allIdle), callback, errorHandler);		
	}

	@Override
	public void purchaseProjectCmd(PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new PurchaseProjectCmdDeligate(planet.getName()), callback, errorHandler);
		
	}

	@Override
	public void mooMoveCmd(MoveableOwnedObjectDelegate craft, StellarObjectDelegate destination, PlanetDelegate planet, boolean generateOrbit, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
    	String planetName = (planet == null) ? null : planet.getName();
        send(new MooMoveCmdDeligate(craft.getName(), destination.getName(), planetName, generateOrbit), callback, errorHandler);
		
	}

	@Override
	public void mooDeploy(MoveableOwnedObjectDelegate moo, StellarObjectDelegate destStellarObject, PlanetDelegate destPlanet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        String planetName = (destPlanet == null) ? null : destPlanet.getName();
        String soName = (destStellarObject == null) ? null : destStellarObject.getName();
        send( new MooDeployCmdDeligate(moo.getName(), soName, planetName), callback, errorHandler );
	}

	@Override
    public void mooSleep(MoveableOwnedObjectDelegate moo, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		MooSleepCmdDeligate mood = null;
		if (moo != null) {
			mood = new MooSleepCmdDeligate(moo.getName());
		} else {
			mood = new MooSleepCmdDeligate();
		}
		send( mood, callback, errorHandler );
		
	}

	@Override
	public void loadTransportCmd(OrbitVO source, OrbitVO destination, int qty, int type, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        send(new TransportLoadCmdDeligate(source, destination, qty, type), callback, errorHandler);
	}

	@Override
	public void loadTransportCmd(PlanetDelegate source, PlanetDelegate destination, int qty, int type, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        OrbitVO s = new OrbitVO(source.getSolarSystemName(), source.getName());
		OrbitVO d = destination == null ? null : new OrbitVO(destination.getSolarSystemName(), destination.getName());
		loadTransportCmd(s, d, qty, type, callback, errorHandler);
	}

	@Override
	public void addCraftDesign(CraftDesign cd, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new AddCraftDesignDeligate(cd), callback, errorHandler);
	}

	@Override
	public void removeCraftDesign(String name, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new RemoveCraftDesignDeligate(name), callback, errorHandler);
		CraftDesigns cdl = getOwner().getCraftDesigns();
		if (cdl != null) cdl.remove(name);
	}

	@Override
	public void setFleetWayPoint(OrbitVO orbit, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SetFleetWayPointCmdDelegate(orbit), callback, errorHandler);
	}

	@Override
	public void increaseEcologyCmd(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new IncreaseEcologyCmdDeligate(), callback, errorHandler);
	}

	@Override
	public void increaseDevelopmentCmd(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new IncreaseDevelopmentCmdDeligate(), callback, errorHandler);
	}

	@Override
	public void setPlayerCharacterPercents(Double ecology, Double development, Double research, Double production, Double money, Double improveEcology, Double improveDevelopment, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SetPlayerCharacterPercentsDeligate(ecology, development, research, production, money, improveEcology, improveDevelopment), callback, errorHandler);
	}

	@Override
	public void setPlanetAutoPercentsCmd(PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SetPlanetAutoPercentsCmdDeligate(planet.getName()), callback, errorHandler);
	}

	@Override
	public void setPlanetAutoPercentsCmd(PlanetDelegate planet, Boolean production, Boolean research, Boolean development, Boolean ecology, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SetPlanetAutoPercentsCmdDeligate(planet.getName(), production, research, development, ecology), callback, errorHandler);
	}

	@Override
	public void setPlanetPercentsCmd(PlanetDelegate planet, double developmentPercent, double ecologyPercent, double productionPercent, double researchPercent, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SetPlanetPercentsCmdDeligate(planet.getName(), developmentPercent, ecologyPercent, productionPercent, researchPercent), callback, errorHandler);
	}

	@Override
	public void setPlanetWantsNoProjectCmd(PlanetDelegate planet, boolean wantsNoProject, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        send(new SetPlanetWantsNoProjectCmdDeligate(planet.getName(), wantsNoProject), callback, errorHandler);
	}

	@Override
	public void abandonCmd(PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        send(new AbandonCmdDeligate(planet.getName()), callback, errorHandler);
	}

	@Override
	public void acceptLeader(String name, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        send(new AcceptLeaderCmdDeligate(name), callback, errorHandler);
	}

	@Override
	public void acceptMoralChoiceCmd(String name, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new AcceptMoralChoiceCmdDeligate(name), callback, errorHandler);		
	}

	@Override
	public void renamePlanet(PlanetDelegate planet, String newName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send( new RenamePlanetCmdDeligate(planet.getName(), newName), callback, errorHandler);		
	}

	@Override
	public void renameStellarObject(StellarObjectDelegate so, String newName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send( new RenameStellarObjectCmdDeligate(so.getName(), newName), callback, errorHandler );
	}
	
	@Override
    public void renameMoo(MoveableOwnedObjectDelegate moo, String newName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send( new MooRenameCmdDeligate(moo.getName(), newName), callback, errorHandler );
	}

	@Override
	public void setRecruiteSpyEffortCmd(int amount, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SetRecruiteSpyEffortCmdDeligate(amount), callback, errorHandler);
	}

	@Override
	public void addSpyEspionage(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SpyRoleAddEspionageCmdDeligate(other.getName(), 1), callback, errorHandler);
	}

	@Override
	public void removeSpyEspionage(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SpyRoleAddEspionageCmdDeligate(other.getName(), -1), callback, errorHandler);
	}

	@Override
	public void addSpySabotage(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SpyRoleAddSabotageCmdDeligate(other.getName(), 1), callback, errorHandler);
	}

	@Override
	public void removeSpySabotage(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SpyRoleAddSabotageCmdDeligate(other.getName(), -1), callback, errorHandler);
	}

	@Override
	public void addSpyStellarObject(StellarObjectDelegate so, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SpyRoleAddStellarObjectCmdDeligate(so.getName()), callback, errorHandler);
	}

	@Override
	public void addSpyPlayer(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SpyRoleAddPlayerCmdDeligate(other.getName()), callback, errorHandler);
	}

    public void setEspionageTarget(PlayerDelegate other, Technology target, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
    	String technologyName = target == null ? null : target.getFullName();
		send(new SpyRoleSetEspionageTargetCmdDeligate(other.getName(), technologyName), callback, errorHandler);
    }

	@Override
	public void sendDiplomaticRequest(PlayerDelegate other, DiplomaticProposal proposal, String message, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new DiplomaticRequestCmdDeligate(other.getName(), proposal, message), callback, errorHandler);
	}

	public void acceptDiplomaticRequest(PlayerDelegate other, DiplomaticProposal proposal, String message, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new DiplomaticResponseCmdDeligate(other.getName(), proposal, true, message), callback, errorHandler);
	}
	
	public void rejectDiplomaticRequest(PlayerDelegate other, DiplomaticProposal proposal, String message, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        send(new DiplomaticResponseCmdDeligate(other.getName(), proposal, false, message), callback, errorHandler);		
	}

	@Override
	public void battleGetNextObject(BattleDelegate battle, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new BattleGetNextObjectCmdDeligate(battle.getName()), callback, errorHandler);
	}

	@Override
	public void battleGetNextOrdersCmd(BattleDelegate battle, BattleObjectDelegate source, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new BattleGetNextOrdersCmdDeligate(battle.getName(), source.getName()), null, errorHandler);
	}

	@Override
	public void battleEndTurnCmd(BattleDelegate battle, BattleObjectDelegate source, boolean endMove, boolean dischargeWeapons, Weapon weapon, int qtyWeapons, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new BattleEndTurnCmdDeligate(battle.getName(), source.getName(), endMove, dischargeWeapons, weapon == null ? null : weapon.getName(), qtyWeapons), callback, errorHandler);
	}

	@Override
	public void battleRetreatCmd(BattleDelegate battle, BattleObjectDelegate source, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new BattleRetreatCmdDeligate(battle.getName(), source.getName()), callback, errorHandler);
	}

	@Override
	public void battleAutomateCmd(BattleDelegate battle, CompletedCallback callback, ClientErrorHandler errorHandle) throws ApplicationError, ValidationError {
		send(new BattleAutomateCmdDeligate(battle.getName()), callback, errorHandle);
	}

	@Override
	public void battleAttackCmd(BattleDelegate battle, BattleObjectDelegate source, ImageObjectDelegate target, boolean move, boolean discharge, boolean missileDefence, LocationAndDirection attackFrom, Weapon weapon, int qtyToFire, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        String weaponName = (weapon == null) ? null : weapon.getName();
        String targetName = (target == null) ? null : target.getName();
        send(new BattleAttackCmdDeligate(battle.getName(), source.getName(), targetName, move, discharge, missileDefence, attackFrom, weaponName, qtyToFire), callback, errorHandler);
	}

	@Override
	public void battleAttackLocationCmd(BattleDelegate battle, BattleObjectDelegate source, Location target, Weapon weapon, int qtyToFire, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
        String weaponName = (weapon == null) ? null : weapon.getName();
        send(new BattleAttackLocationCmdDeligate(battle.getName(), source.getName(), target, weaponName, qtyToFire), callback, errorHandler);
	}

	@Override
	public void battleCraftWaitCmd(BattleDelegate battle, BattleCraftDelegate source, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new BattleCraftWaitCmdDeligate(battle.getName(), source.getName()), callback, errorHandler);
	}

	@Override
	public void battleCraftLaunchFightersCmd(BattleDelegate battle, BattleObjectDelegate source, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new BattleCraftLaunchFightersCmdDeligate(battle.getName(), source.getName()), callback, errorHandler);
	}

	@Override
	public void battleCraftMoveToCmd(BattleDelegate battle, BattleCraftDelegate source, LocationAndDirection moveToDest, ImageObjectDelegate target, boolean stopOnMissileAttack, boolean missileDefence, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
    	String targetName = target == null ? null : target.getName();
		send(new BattleCraftMoveToCmdDeligate(battle.getName(), source.getName(), moveToDest, targetName, stopOnMissileAttack,  missileDefence), callback, errorHandler);
	}

	@Override
	public LocationAndDirection battleFetchCraftMoveActualLocation(BattleDelegate battle, BattleCraftDelegate source, LocationAndDirection moveToDest, BattleCraftDelegate target) throws ApplicationError, ValidationError {
		if (battle != null && source != null) {
			BattleDelegateCache cache = battle.getDelegateCache();
			BattleSandbox battleSandBox = cache.getBattleSandbox();
			return battleSandBox.getCraftMoveActualLocation(source, moveToDest, target);
		}
		return null;
	}

	@Override
	public void sellSavlage(TechnologyList salvage, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError {
		send(new SellSalvageCmdDeligate(salvage), callback, errorHandler);
	}

	@Override
	public void clientClosesWindow(BattleDelegate battle) throws ApplicationError, ValidationError {
		send(new ClientClosesWindowCmdDeligate(battle == null ? null : battle.getName()));
	}

	@Override
	public void fetchTechnologies(PlayerDelegate player, CompletedCallback callback, ClientErrorHandler errorHandler) throws ValidationError, ApplicationError {
		send(new FetchTechnologiesCmdDelegate(player.getName()), callback, errorHandler);		
	}

	@Override
	public void fetchStealableTechnology(PlayerDelegate giver, PlayerDelegate taker, TechnologyList exclude, CompletedCallback callback, ClientErrorHandler errorHandler) throws ValidationError, ApplicationError {
		TechnologyListVO tl = exclude != null ? new TechnologyListVO(exclude) : null;
		send(new SpyStealableTechnologiesCmdDelegate(giver.getName(), taker.getName(), tl), callback, errorHandler);		
	}

	@Override
	public void fetchPlayerDetails(PlayerDelegate player, CompletedCallback callback, ClientErrorHandler errorHandler) throws ValidationError, ApplicationError {
		String name = player == null ? null : player.getName();
		send(new FetchPlayerDetailsCmdDelegate(name), callback, errorHandler);		
	}

	@Override
	public void updateDelegates(CompletedCallback callback) throws ValidationError, ApplicationError {
		send(new FetchDelegateStateCmdDelegate(), callback, null);		
	}

	public MoveableOwnedObjectDelegate[] getMoos(boolean all) throws ApplicationError {
		if (all) {
			LocalServerProxy ld = getLocalServerProxy();
			if (ld != null) {
				return ld.getMoos();
			}
		}
		return DeligateCacheManager.getMoos();
	}

}
