/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.proxy;

import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.TransportDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.NewGameData;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.coms.protocols.ServerConnection;
import mot.common.designs.CraftDesign;
import mot.common.designs.ProjectDesign;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.common.vo.OrbitVO;

/**
 * 
 * Interface defining the operation of a server proxy.
 * This interface defines the operation of a server in the game engine. If a client
 * wishes to send a command to the server, the proxy submit method should be used.
 * For local implementations, this would be implemented by directly calling the engine.
 * However, a remote implementation would serialise the command to XML, send it to the
 * server and the server would have to deserialise and process the command.
 * 
 * New Game
 * --------
 * ServerProxy.getInstance()						or					ServerProxy.getInstance()
 * 																		ServerProxy.connect(ip)
 *  
 * ServerProxy.newGame(UserData, GameData)								ServerProxy.newGameUserPlayer(UserData)
 * - delegate cache populated
 * 
 * ServerProxy.register(UserName, NoClientUpdate, EventProcessor)		GameStartedEvent received
 * - create client proxy on server, event processor in client  			- create client proxy on server, event processor in client
 * 																		- delegate cache populated
 * 
 * Load Game
 * ---------
 * ServerProxy.getInstance()						or					ServerProxy.getInstance()
 *  																	ServerProxy.connect(ip)
 *  
 * ServerProxy.load(File)							
 * - return player name 
 * - delegate cache populated
 * 
 * ServerProxy.register(UserName, NoClientUpdate, EventProcessor)		ServerProxy.register(UserName, WithClientUpdate EventProcessor)
 * - create client proxy on server, event processor in client  
 * 
 * 
 **/
public interface ServerProxy {

	public void connect(String ip) throws ApplicationError, ValidationError;
    public boolean isConnected();
    public void disconnect();
    public ServerConnection getConnection() throws ApplicationError;
	public boolean isBusy();
    
    /**
     * @return True if the client is a local client and can support direct access to the server
     * (Note the editor requires update access to the server).
     */
    public boolean supportsLocalServerProxy();
    
    /**
     * @return An instance of a local server proxy that allows the client to update server state (directly) - Used by Editor.
     */
    public LocalServerProxy getLocalServerProxy();
	
    /**
     * @return An instance of a Remote Connection server proxy that allows a remote client to connect to this game (for multi-player).
     */
    public RemoteConnectionServerProxy getRemoteConnectionServerProxy() throws ApplicationError;

    
    
    /**
     * This is called to register a new server URL from this connection.
     * This also flushed the delegate cache!
     * @throws mot.common.util.ValidationError
     * @throws mot.common.util.ApplicationError
     * @param serverURL
     */
    public void register(String player, boolean updateClient, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Register the existence of a new user or updates the users player options in a game.
     * This should be called when a User Player is connecting to a remote game to update / register the existence of a new user player. 
     * @param raceName The name of the player which is a user to add to the game!
     * @param userOptions Options for this user player
     */
    public void newGameUserPlayer(NewGameUserPlayerData userData, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Register the existence of a new user or updates the users player options in a game and creates a new Game.
     * @param gameData The name of the player (in the game) used by the player that owns this client
     */
    public void newGame(NewGameUserPlayerData userData, NewGameData gameData, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * @return A list of files names on the same machine as the engine that could be used
     * to load a game from.
     */
    public void getSavedGameFiles(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * Load a saved game from the filing system into the engine this client is connected to.
     * When this method returns the instance of MotEngine of the server will contain the 
     * contents of the new game unless an error is returned.
     * NOTE - The file name must be contained in the list of files returned by getSavedGameFiles()
     * @param fileName The name of the file on the same machine the server is running on.
     * @param playerName The name of the user player in the game (in a multi player game). Can be null in a single player game.
     */
    public void load(String fileName, String playerName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * Save the current game on the filing system.
     * If the file in file name does not exist then a new file is created, otherwise the old
     * saved game in the file is replaced with the current game.
     * @param fileName The name of the file on the same machine the server is running on.
     */
    public void save(String fileName, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * Fetches details of a saved game. This will not modify the existing game loaded. It just
     * displays
     * @param fileName The name of the file on the same machine the server is running on.
     * @return Details of the galaxy stored in file called fileName.
     */
    public void fetchGalaxyPreview(String fileName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Fetch a list of user player that exist in this game.
     * NB - When a multi-player game is loaded, other clients need to connect and resister for each user player in the game.
     * @return A list of names who have connected to the server to play a game.
     * @throws ValidationError 
     */
    public void fetchRegisteredPlayers(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * @return The player that owns this proxy - This is the current clients player!
     * @throws ApplicationError 
     */
    public UserPlayerDelegate getOwner() throws ApplicationError;

    /**
     * @return Statistics about the performance of this craft design if it was built by player owner with race origin operating it.
     * @throws ValidationError 
     */
    public void getCraftDesignStatistics(CraftDesign design, PlayerDelegate owner, PlayerDelegate origin, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Design a military craft or Battle Station or Space Station. 
     * @param hullName If not null, the design will be the best available using this hull.
     * @param orbit If the hull type is a space station, then the orbit matters as the engine needs to identify the list of modules contained in the station. 
     */
    public void designMilitaryCraft(String hullName, OrbitVO orbit, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    public void designColonyCraft(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
	/**
	 * @return Graphical information displayed in the statistics window for each player.
	 * @throws ApplicationError
	 * @throws ValidationError 
	 */
	public void getGraph(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    
    /**
     * Delete craft from the game. The craft will be turned into scrap and sold so this will add funds to the players treasury.
     * @param craft The craft the be destroyed.
     * @throws ApplicationError
     * @throws ValidationError
     */
    public void mooDeleteCmd(MoveableOwnedObjectDelegate craft, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * This craft attacks all enemy crafts and planets at its current location.
     * @param craft The craft/fleet that will attack the enemy
     * @param enemy the player to attack
     */
    public void craftAttackCmd(MoveableOwnedObjectDelegate craft, PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * This instructs a craft/fleet owned by this player in the game to bombard a planet.
     * A BombardEvent will be sent back to the user containing the results.
     * @param craft The craft/fleet that will bombard the enemy planet
     * @param planet The planet to bombard
     **/
    public void craftBombardPlanetCmd(MoveableOwnedObjectDelegate craft, PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * The transport must contain colonists that are now considered troops and initiate ground combat on planet2
     * as part of an invasion.
     * The results are sent back to the client as part of a InvasionPlanetEvent.
     * @throws ValidationError 
     * @throws ApplicationError 
     */
	public void invadePlanet(TransportDelegate transport, PlanetDelegate destPlanet, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

	/**
     * Removes this craft from the fleet it currently is part of.
     * @param craft The craft to remove from the fleet.
     */
    public void craftLeaveFleetCmd(CraftDelegate craft, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Add the craft to a fleet. If there is not fleet at the crafts location then a new
     * fleet is created, otherwise it joint an existing fleet at this location.
     * @param craft The craft to join a fleet.
     **/
    public void joinFleetCmd(MoveableOwnedObjectDelegate craft, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Ends the current turn of this user. It checks to see if all human players have 
     * moved, and ends the game turn if they have. When the game turn ends it, moves all 
     * crafts, fleets, computer players, starts battles and calculates all the new research, 
     * money, and production for each colony.
     **/
    public void endTurnCmd(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * This is a issues to request a craft colonise a planet (on the galaxy map).
     * Converts the planet into a colony owned by the this player. The craft used to 
     * colonise this planet must be a coloniser and will be destroyed by this operation.
     * @param craft The colony craft creating this colony.
     * @param planet The planet to create this colony on.
     **/
    public void craftColoniseCmd(MoveableOwnedObjectDelegate craft, PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * Sets the research project for this player to discover the technology tech. Any previous
     * research project is deleted.
     * A player may have a single research project and each research project may have a single
     * technology that is the technology the player is researching.
     * @param tech The technology this player will attempt to research in the new research project.
     */
    public void setResearchProjectCmd(Technology tech, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
	public void setResearchProjectAuto(boolean autoOn);
	public boolean isResearchProjectAuto();

    /**
     * Assigns project as the current planetary project on planet.
	 * A planetary project is a task assigned to a planet to build an object in the game 
	 * - like a new craft.
     * @param planet The planet to assign the project to.
     * @param project The project to assign to the planet.
     * @param repeat The quantity of times this project should be repeated.
     * @param allIdle If true this project should be placed on all planets not building anything.
     **/
    public void setProjectCmd(PlanetDelegate planet, ProjectDesign project, int repeat, boolean allIdle, boolean useSalvage, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void setProjectCmd(PlanetDelegate [] planet, ProjectDesign project, int repeat, boolean allIdle, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Purchase the planets project so it will be completed next turn.
     * @param planet The planet that has a project that we want to complete now.
     */
    public void purchaseProjectCmd(PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
	 * This is a issues to move a craft/fleet by a player in the game (on the galaxy map).
	 * @param craft The moo (Movable Owned Object) must not be null and is the craft (or whatever) to move
	 * @param destination The destination must not be null and is the stellar object to move the moo to.
	 * @param planet The planet may be null but if it is not null then stellar object must be the solar system
	 * containing this planet and this planet should be the orbit when the moo arrives at the destination.
	 * @param generateOrbit If generateOrbit is true then planet must be null and this command will select the planetary orbit
	 * to use when this moo arrives at the destination.
     */
    public void mooMoveCmd(MoveableOwnedObjectDelegate craft, StellarObjectDelegate destination, PlanetDelegate planet, boolean generateOrbit, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * For a transport it means unload the contents and place onto the destination planet or stellar object. 
     * For a craft it means activate technology on the craft that can effect the target planet or stellar object.
     * @param moo The Transport or craft to deploy.
     * @param destStellarObject Null or the destination solar system to deploy onto.
     * @param destPlanet Null or the destination planet to deploy onto.
     */
    public void mooDeploy(MoveableOwnedObjectDelegate moo, StellarObjectDelegate destStellarObject, PlanetDelegate destPlanet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Instruct this craft / fleet / transport not to ask for a commend every turn. The end turn button moved focus to unmoved moo
     * unless they are asleep.
     * @param moo The Transport or craft to sleep.
     */
    public void mooSleep(MoveableOwnedObjectDelegate moo, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Transfer qty people onto a transport in orbit around the source planet from the planet's surface.
     * If there is an transport in orbit around the source planet that has space for more colonists, that
     * space will be filled before new transports are used to load colonists onto. Existing ransports are 
     * not used to load colonists if they have been assigned an order to move to another solar system (have 
     * a navigator).
     * @param source The planet or stellar object from which to take the colonists (start of the journey)
     * @param destination This is optional. If set, it is the destination of the transport (the end
     * of the journey). This gives the transport orders to leave the source for the destination 
     * Immediately.
     * @param qty The quantity of colonists to load onto transports. If there is not space to existing 
     * transports and not enough new transports to load all the people, then a validation error will be 
     * raised - but as many transports will be loaded as can fit and the error is simply that some 
     * colonists have been left behind.
     * @param type one of the following values 0 = colonists; 1 = troops; 2 = trade route 
     */
    public void loadTransportCmd(OrbitVO source, OrbitVO destination, int qty, int type, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void loadTransportCmd(PlanetDelegate source, PlanetDelegate destination, int qty, int type, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    public void addCraftDesign(CraftDesign cd, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void removeCraftDesign(String name, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    public void setFleetWayPoint(OrbitVO orbit, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    
  	public void increaseEcologyCmd(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
  	public void increaseDevelopmentCmd(CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
  	
    public void setPlayerCharacterPercents(Double ecology, Double development, Double research, Double production, Double money, Double improveEcology, Double improveDevelopment, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    public void setPlanetAutoPercentsCmd(PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void setPlanetAutoPercentsCmd(PlanetDelegate planet, Boolean production, Boolean research, Boolean development, Boolean ecology, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    public void setPlanetPercentsCmd(PlanetDelegate planet, double developmentPercent, double ecologyPercent, double productionPercent, double researchPercent, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError ;
    
    public void setPlanetWantsNoProjectCmd(PlanetDelegate planet, boolean wantsNoProject, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * This is used to abandon a players colony (or mining station).
     * Removes the colony from this planet - The planet still exists but it is no 
     * longer a colony of the owner. It removes the population, and orbital. It also 
     * sets the origin owner to nothing.
     **/
    public void abandonCmd(PlanetDelegate planet, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * The Leader whose name is passed in is added to the list of leaders owned
     * by this player. The leader must have been offered to this player (just 
     * received an OfferLeaderEvent for this leader).
     **/
    public void acceptLeader(String name, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * The player that owns this proxy adopts the moral policy who name is passed
     * in. The moral policy advisor must have recommended this policy prior to a player
     * adopting it (just received an RecommendMoralChoiceEvent for this policy).
     **/
    public void acceptMoralChoiceCmd(String name, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    public void renamePlanet(PlanetDelegate planet, String newName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void renameStellarObject(StellarObjectDelegate so, String newName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void renameMoo(MoveableOwnedObjectDelegate moo, String newName, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
        
    public void setRecruiteSpyEffortCmd(int amount, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void addSpyEspionage(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void removeSpyEspionage(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void addSpySabotage(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void removeSpySabotage(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void addSpyStellarObject(StellarObjectDelegate so, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void addSpyPlayer(PlayerDelegate other, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    public void setEspionageTarget(PlayerDelegate other, Technology target, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
	public void sendDiplomaticRequest(PlayerDelegate other, DiplomaticProposal proposal, String message, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
	public void acceptDiplomaticRequest(PlayerDelegate other, DiplomaticProposal proposal, String message, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
	public void rejectDiplomaticRequest(PlayerDelegate other, DiplomaticProposal proposal, String message, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
	
    /**
     * This will move all computer controlled crafts/planets in the battle until the 
     * it finds one owned by a user player. This may also end the battle turn 
     * and start the next turn. This will only return null if the battle has ended 
     * and there is nothing to move.
     * 
     * NOTE - The delegate cache does store the next craft to move that is updated 
     * after every command. However - should an error in one command cause an issue
     * the next command updates the value.
     * 
     * @return The next battle object to move in the battle owned by a user. 
     * @throws ApplicationError
     * @throws ValidationError 
     */
    public void battleGetNextObject(BattleDelegate battle, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * Asks the game engine to move the source craft. This only applies to this turn.
     * The next turn the user will control the craft or have to call this method again.
     * @param battle The battle in which the source craft is fighting.
     * @param source The craft this command will be applied to.
     */
    public void battleGetNextOrdersCmd(BattleDelegate battle, BattleObjectDelegate source, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * Discharges all weapons and set's move remaining to 0 on the source craft. This means the
     * source cannot move until the next turn in the battle.
     * If source is null then this indicates that the battle window has been closed.
     * @param battle The battle in which the source craft is fighting.
     * @param source The craft whose who wants its turn be be ended (if any).
     */
    public void battleEndTurnCmd(BattleDelegate battle, BattleObjectDelegate source, boolean endMove, boolean dischargeWeapons, Weapon weapon, int qtyWeapons, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
	/**
	 * If this is a normal craft it is retreated (removed) from the battle. If it is a hologram it 
	 * is destroyed. Craft that have retreated may be destroyed if you lose the battle!
     * @param battle The battle in which the source craft is fighting.
     * @param source The craft this command will be applied to.
	 */
    public void battleRetreatCmd(BattleDelegate battle, BattleObjectDelegate source, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * Automates this battle - complete this battle in a single action! 
     */
    public void battleAutomateCmd(BattleDelegate battle, CompletedCallback callback, ClientErrorHandler errorHandle) throws ApplicationError, ValidationError;

    /**
     * Issues the command for this craft to attack the enemy target passed in.
     * @param owner The player who has submitted this command.
     * @param battle The battle this attack is taking place in.
     * @param source The object to attack.
     * @param target The object to be attacked.
     * @param move If this command is permitted to move the source craft. 
     * @param discharge If all weapons should be discharged to end this craft's move 
     * @param attackFrom Not null if this command is permitted to move the source craft and is the 
     * location to move this craft towards before firing weapons.
     * @param weapon The weapon the source craft is to fire at the target. If this is null 
     * all weapons are fired.
     * @param qtyToFire The The quantity of weapon the source craft is to fire at the target.
     * If weapon is not null this is the quantity of this weapon to fire. If weapon is null
     * this is ignored
     */
    public void battleAttackCmd(BattleDelegate battle, BattleObjectDelegate source, ImageObjectDelegate target, boolean move, boolean discharge, boolean missileDefence, LocationAndDirection attackFrom, Weapon weapon, int qtyToFire, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

	/**
	 * Fires weapons at the target location.
     * @param target The location to fire this weapon at
     * @param weapon If null all weapons that have not been fired and are in range fire.
     * If weapon is not null only this weapon is fired. If this is null weapons are fired in 
     * the order they are places in the weapon instance array - Note - order is important so 
     * weapons like ShieldDrain get sap maximum power before the shields are weakened.
     * @param qtyToFire If weapon is not null this is the quantity of weapons in the weapon 
     * instance to fire. If weapon is null then this parameter is ignored.
	 */
    public void battleAttackLocationCmd(BattleDelegate battle, BattleObjectDelegate source, Location target, Weapon weapon, int qtyToFire, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * This command causes the source craft to postpone its move. The owner of this craft does not
     * want to move it straight away - and instead, would like a crft of lower initiative to move 
     * first.
     * @param battle The battle in which the source craft is fighting.
     * @param source The craft this command will be applied to.
     **/
    public void battleCraftWaitCmd(BattleDelegate battle, BattleCraftDelegate source, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
     * This command causes the source craft to launch fighters it has in the fighters bay.
     * The source craft must have fighters (special) ready to launch.
     * @param battle The battle in which the source craft is fighting.
     * @param source The craft this command will be applied to.
     **/
    public void battleCraftLaunchFightersCmd(BattleDelegate battle, BattleObjectDelegate source, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

    /**
	 * Rotates craft towards this direction then moves it to the closest empty square
	 * between the crafts current location and this location.
	 * The craft will stop before its destination it the space is occupied or the craft hits
	 * an active incumbent - such as a mine field. If the craft hits a mine field it may continue
	 * moving by re-calling the move method!
     * @param battle The battle in which the source craft is fighting.
     * @param source The craft this command will be applied to.
     * @param moveToDest The destination location to move this craft to or towards.
     * @param target If this movement is to attack an enemy craft then this is the target for the attack (can be null).
     * @param maxMovePercent The maximum percentage of the crafts move that should be used.
     * @param stopOnMissileAttack If this is true the move will stop before a missile strikes.
     * If the craft has weapons that can destroy missiles that have not been fired this turn it 
     * gives the owner an opportunity to destroy the missiles before they damage the craft.
     */
    public void battleCraftMoveToCmd(BattleDelegate battle, BattleCraftDelegate source, LocationAndDirection moveToDest, ImageObjectDelegate target, boolean stopOnMissileAttack, boolean missileDefence, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;
    
    /**
     * Fetched the actual location the craft source would move to if the user asked to move to craft to moveToLocation.
     * @param battle The battle this craft is in.
     * @param source The craft that may want to move.
     * @param moveToLocation The location we may want to move the craft source too.
     * @return 
     * @throws ApplicationError
     * @throws ValidationError
     */
    public LocationAndDirection battleFetchCraftMoveActualLocation(BattleDelegate battle, BattleCraftDelegate source, LocationAndDirection moveToDest, BattleCraftDelegate target) throws ApplicationError, ValidationError;
    
    /**
     * Instructs the engine to sell the salvage passed in. Funds are credited to the players.
     * The player must actually own each piece of salvage otherwise a validation error is produced.
     * @param salvage List of salvage to sell.
     */
	public void sellSavlage(TechnologyList salvage, CompletedCallback callback, ClientErrorHandler errorHandler) throws ApplicationError, ValidationError;

	/**
	 * Called to notify the engine that a client has closes the battle window. 
	 */
	public void clientClosesWindow(BattleDelegate battle) throws ApplicationError, ValidationError;

	/**
	 * Fetched all technologies discovered by a player.
	 * NOTE - UserPlayer Delegate for the owner HAS this information - This is used to fetch technologies another player has discovered.
	 * @throws ApplicationError 
	 * @throws ValidationError 
	 */
	public void fetchTechnologies(PlayerDelegate player, CompletedCallback callback, ClientErrorHandler errorHandler) throws ValidationError, ApplicationError;

	/**
	 * Fetched a list of technologies player taker would like from player giver. 
	 * NOTE - UserPlayer Delegate for the owner HAS this information - This is used to fetch technologies another player has discovered.
	 */
	public void fetchStealableTechnology(PlayerDelegate giver, PlayerDelegate taker, TechnologyList exclude, CompletedCallback callback, ClientErrorHandler errorHandler) throws ValidationError, ApplicationError;
	
	/**
	 * Fetched information displayed only in the Empire Window. 
	 */
	public void fetchPlayerDetails(PlayerDelegate player, CompletedCallback callback, ClientErrorHandler errorHandler) throws ValidationError, ApplicationError;
	
	/**
	 * Update all the delegates held on this client with information from the server.
	 */
	public void updateDelegates(CompletedCallback callback) throws ValidationError, ApplicationError;
	
	/**
	 * If there are any commands that have taken so long that we assume an error has occurred.
	 * @throws ApplicationError 
	 */
	public void processTimeouts() throws ApplicationError;
	
	
	/**
	 * @param all All the MOOs in the game or just the MOOs visible in normal game play
	 * @return MOOs to be displayed on in the galaxy map
	 * @throws ApplicationError 
	 */
	public MoveableOwnedObjectDelegate[] getMoos(boolean all) throws ApplicationError;


}