/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.sandbox.battle;

import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.BattlePlanetDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.cache.BattleDelegateCache;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.AsteroidFieldDetails;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.PlayerCharacterBasic;
import mot.common.bean.PlayerColor;
import mot.common.bean.SpaceDistortionDetails;
import mot.common.designs.CraftDesign;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.OrbitVO;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.incumbants.AsteroidField;
import mot.engine.battles.incumbants.IncumbentFactory;
import mot.engine.battles.incumbants.SpaceDistortion;
import mot.engine.common.Orbit;
import mot.engine.common.ServerImageObject;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.PlayerCharacter;

public class BattleSandbox {
	
	private Battle battle;
	private BattleDelegateCache cache;

	public BattleSandbox(BattleDelegateCache cache) {
		this.cache = cache;
		OrbitVO orbit = cache.getBattle().getOrbit();
		StellarObjectDelegate sod = DeligateCacheManager.getStellarObject(orbit.getStellarObject());
		SolarSystem so = new SolarSystem(sod.getName(), sod.getLocation().getX(), sod.getLocation().getY());
		Planet p = null;
		if (orbit.getPlanet() != null) {
			new Planet(orbit.getName(), null, so);
		}
		battle = new Battle(new Orbit(so, p));
		BattleCraftDelegate[] crafts = cache.getCrafts();
		if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {
				create(crafts[i]);
			}
		}
		BattlePlanetDelegate planet = cache.getPlanet();
		if (planet != null) {
			create(planet);
		}
		AnimatedBattleAreaDetails[] annimatedBattleAreas = cache.getAnnimatedBattleAreas();
		if (annimatedBattleAreas != null) {
			for (int i = 0; i < annimatedBattleAreas.length; i++) {
				create(annimatedBattleAreas[i]);
			}
		}
	}
	
	public BattleObject getBattleObject(String name) {
		return battle.getBattleObject(name);
	}
	
	public BattleCraft getBattleCraft(String name) {
		return (BattleCraft) battle.getBattleObject(name);
	}
	
	private void create(AnimatedBattleAreaDetails abad) {
		try {
			_create(abad);
		} catch (ApplicationError e) {
			Logger.logError("BattleSandbox Failed", e);
		}
	}
	
	private void _create(AnimatedBattleAreaDetails abad) throws ApplicationError {
		if (abad instanceof AsteroidFieldDetails) {
			AsteroidField af = IncumbentFactory.createAsteroidField(battle, abad);
		} else if (abad instanceof SpaceDistortionDetails) {
			SpaceDistortion sd = IncumbentFactory.createSpacialDistortion(battle, null, abad);
		}
	}

	private void create(BattleObjectDelegate bo) {
		try {
			_create(bo);
		} catch (ApplicationError e) {
			Logger.logError("BattleSandbox Failed", e);
		}
	}
	
	private MonsterPlayer craftOwner;
	
	private void _create(BattleObjectDelegate bo) throws ApplicationError {
		Orbit orbit = battle.getOrbit();
		if (craftOwner == null) {
			String playerName = "SandboxMonster";
			PlayerColor colour = PlayerColor.getColorForName(null);
			PlayerCharacter character = new PlayerCharacter();
			character.setBasicCharacter(new PlayerCharacterBasic());
			craftOwner = new MonsterPlayer(playerName, character, colour);
		}
		if (bo instanceof BattlePlanetDelegate) {
			BattlePlanetDelegate bp = (BattlePlanetDelegate) bo;
			Planet planet = new Planet(bp.getName(), craftOwner, (SolarSystem) orbit.getStellarObject());
			BattlePlanet battlePlanet = new BattlePlanet(planet, battle);
			battlePlanet.setLocation(bo.getLocation());
			battle.setPlanet(battlePlanet);
		} else if (bo instanceof BattleCraftDelegate) {
			BattleCraftDelegate bc = (BattleCraftDelegate) bo;
			CraftDelegate c = bc.getCraft();
			if (c == null) {
				Logger.logWarning("BattleSandbox._create() No craft delegate for "+bc+" OBJECT IGNORED");
			} else {
				CraftDesign craftDesign = c.getCraftDesign();
				CraftDesign cd = new CraftDesign(bc.getName(), craftDesign.getHullTech(), craftDesign.getEngineTech());
				Craft craft = new Craft(bc.getName(), craftOwner, cd, orbit, 0.0);
				int speed = bc.getSpeed();
				int turningSpeed = bc.getTurningSpeed();
				BattleCraft battleCraft = new SandboxBattleCraft(battle, craft, false, speed, turningSpeed);
				battleCraft.setDirection(bc.getDirection());
				battleCraft.setMoveRemaining(bc.getMoveRemainingAsPercent());
				battleCraft.setLocation(new Location(bo.getLocation()));
				battle.add(battleCraft);
			}
		}
	}


	public LocationAndDirection getCraftMoveActualLocation(BattleCraftDelegate source, LocationAndDirection moveToDest, BattleCraftDelegate target) throws ApplicationError {
		BattleObject sourceBO = battle.getBattleObject(source.getName());
		if (sourceBO instanceof BattleCraft) {
			BattleCraft sourceBC = (BattleCraft) sourceBO;
			ServerImageObject targetIO = null; 
			Location targetLoc = null;
			if (target != null)  {
				targetIO = battle.getBattleObject(target.getName());
				targetLoc = target.getLocation();
			}
			return sourceBC.getCraftMoveActualLocation(moveToDest, targetLoc, targetIO);
		} else {
			return null;
		}
	}

}
