/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.sounds;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import mot.common.bean.Settings;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Plays songs from a song list in sequence and when it reaches the 
 * end, starts over again.
 * This class is used to play the background music in the game.
 */
public class JukeBox implements SoundFinishedEventListener {

	private ArrayList<Sound> playList = new ArrayList<Sound>();
	private Sound playing;
	
	public void add(String fileName) throws ApplicationError {
		if (fileName != null) {
			if (fileName.startsWith("http:")) {
				try {
					add(new URL(fileName));
				} catch (MalformedURLException e) {
					throw new ApplicationError("JukeBox.add() Invalid URL: "+fileName);
				}
			} else {
				add(new File(fileName));
			}
		}
	}

	public void add(File file) throws ApplicationError {
		if (file.getName().endsWith("mid") || file.getName().endsWith("midi")) {
			add(new SoundMusicMidi(file));
		} else {
			add(new SoundMusicWav(file));
		}
	}

	public void add(URL file) throws ApplicationError {
		if (file.getPath().endsWith("mid") || file.getPath().endsWith("midi")) {
			add(new SoundMusicMidi(file));
		} else {
			add(new SoundMusicWav(file));
		}
	}

	public void add(Sound s) {
		try {
			playList.add(s);
			//s.addListener(this);
		} catch (Exception e) {
			if (Logger.isWarning())	Logger.logError("JukeBox.add()", e);
		}
	}
	
	public void deleteSongs() {
		playList.clear();
	}

	public void finished(Sound sound) throws ApplicationError {
		Logger.logDebug("Finish sound "+sound.toString());
		try {
			if (playList.size() == 0) {
				return;
			}
			int index = 0;
			for (Sound s : playList) {
				if (sound == s) break;
				index++;
			}
			index++;
			if (index >= playList.size()) index = 0;
			Sound s = playList.get(index);
			Logger.logDebug("Start sound "+s.toString());
			playing = s;
			s.replay();
		} catch (Exception e) {
			if (Logger.isWarning())	Logger.logError("JukeBox.finished()", e);
		}
	}
	
	public void start() throws ApplicationError {
		try {
			if (playList.size() == 0) {
				return;
			}
			Sound s = (playing != null) ? playing : playList.get(0);
			playing = s;
			s.play();
		} catch (Exception e) {
			if (Logger.isWarning())	Logger.logError("JukeBox.start()", e);
		}
	}
	
	public void stop() {
		try {
			if (playing != null) {
				playing.stop();
				playing = null;
			}
		} catch (Exception e) {
			if (Logger.isWarning())	Logger.logError("JukeBox.start()", e);
		}
	}
	
	public void setVolume(double v) {
		try {
			for (Sound s : playList) {
				s.setVolume(v);
			}
		} catch (Exception e) {
			if (Logger.isWarning())	Logger.logError("JukeBox.stop()", e);
		}
	}
	
	private static JukeBox instance;
	public static JukeBox getInstance() throws ApplicationError {
		if (instance == null) {
			instance = new JukeBox();
			String[] songs = Settings.getInstance().getSongs();
			for (int i = 0; i < songs.length; i++) {
				try {					
					instance.add(songs[i]);
				} catch (Exception e) {
					Logger.logError("Unable to start Duke Box Song "+songs[i], e);
				}
			}
		}
		return instance;
	}

}
