/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.sounds;

import java.io.File;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;

import mot.common.bean.Settings;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * This is a normal sound from a file such as wav, mp3 etc.
 * These sounds are used for effects such as explosions, weapons fire,
 * button clicks etc.
 * 
 * You may call start and stop as many times as you like! It caches
 * the sound data on creation so there is no delay when the sound is 
 * played.
 */
public class SoundEffect extends AbstractSound {

	private Clip clip;

	public SoundEffect(File file) throws ApplicationError {
		if (Logger.isDebug()) {
			Logger.logDebug("SoundEffect.init() "+file.getName());
		}
		try {
			if (file.exists()) {
				AudioInputStream stream = AudioSystem.getAudioInputStream(file);
				AudioFormat format = stream.getFormat();
				DataLine.Info info = new DataLine.Info(Clip.class, format);
				clip = (Clip) AudioSystem.getLine(info);
				clip.open(stream);
				setVolume(Settings.getInstance().getVolumeEffects());
//				LineListener listener = new LineListener() {
//					public void update(LineEvent event) {
//						if (event.getType() == LineEvent.Type.STOP) {
//							try {
//								broadcastFinishEvent();
//							} catch (ApplicationError e) {
//								Logger.logError("", e);
//							}
//						}
//					}
//				};
//				if (clip != null) {
//					clip.addLineListener(listener);
//				}
			} else {
				Logger.logError("File "+file+" does not exist", new Exception());
			}
		} catch (Throwable e) {
			if (Logger.isWarning())	Logger.logError("Unable to play sound "+file.getName(), e);
		}
	}

	public void replay() throws ApplicationError {
		play();
	}
	
	public void play() {
		if (clip == null) {
			return;
		}
		if (clip.isActive()) {
			clip.stop();
			clip.drain();
		}
		clip.setMicrosecondPosition(0);
		clip.start();
	}

	public void stop() {
		if (clip == null) {
			return;
		}
		clip.stop();
		clip.drain();
		clip.flush();
	}

	public boolean isPlaying() {
		return clip!= null && clip.isActive();
	}
	
	private static boolean first = true; 
	
	public void setVolume(double v) {
		if (clip == null) {
			return;
		}
		try {
			FloatControl gainControl = null;
			try {
				gainControl = (FloatControl) clip.getControl(FloatControl.Type.MASTER_GAIN);
			} catch (Exception e) {	
				Logger.logWarning(e.getMessage());
			}
			if (gainControl == null) {
				gainControl = (FloatControl) clip.getControl(FloatControl.Type.VOLUME);
			}
			float min = gainControl.getMinimum();
			float max = gainControl.getMaximum();
			float range = (max - min) / 2;
			float vol = (float) (min + range + range * v);
			gainControl.setValue(vol);
		} catch (Exception e) {
			if (first) {
				if (Logger.isWarning())	Logger.logError("Unable to set volume", e);
				first = false;
			} else {
				if (Logger.isWarning())	Logger.logError("Unable to set volume");				
			}
		}	    	
	}

}
