/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.sounds;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MetaEventListener;
import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiChannel;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequencer;
import javax.sound.midi.Synthesizer;

import mot.common.bean.Settings;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * This is background music in midi format.
 * 
 * You may call start and stop as many times as you like! It caches
 * the sound data on creation so there is no delay when the sound is 
 * played.
 */
public class SoundMusicMidi extends AbstractSound {

	private static final int CHANGE_VOLUME_MESSAGE = 7;
	private static final int END_OF_SEQUENCE_MESSAGE = 47;
	public static final int VOLUME_CONTROL = 7;
	
	private Sequencer sequencer;
	private Synthesizer synthesizer;

	public SoundMusicMidi(File file) throws ApplicationError {
		try {
			sequencer = MidiSystem.getSequencer(false);
			synthesizer = MidiSystem.getSynthesizer();
			sequencer.open();
	        synthesizer.open();
	        sequencer.getTransmitter().setReceiver(synthesizer.getReceiver());		    
	        sequencer.setSequence(MidiSystem.getSequence(file));
		    sequencer.open();
		} catch (IOException e) {
			Logger.logError("Unable to play sound "+file.getName(), e);
		} catch (MidiUnavailableException e) {
			Logger.logError("Unable to play sound "+file.getName(), e);
		} catch (InvalidMidiDataException e) {
			Logger.logError("Unable to play sound "+file.getName(), e);
		}
		init();
	}
	
	public SoundMusicMidi(URL file) throws ApplicationError {
		try {
			sequencer = MidiSystem.getSequencer(false);
			synthesizer = MidiSystem.getSynthesizer();
			sequencer.open();
	        synthesizer.open();
	        sequencer.getTransmitter().setReceiver(synthesizer.getReceiver());		    
	        sequencer.setSequence(MidiSystem.getSequence(file));
		    sequencer.open();
		} catch (IOException e) {
			Logger.logError("Unable to play sound "+file.getQuery(), e);
		} catch (MidiUnavailableException e) {
			Logger.logError("Unable to play sound "+file.getQuery(), e);
		} catch (InvalidMidiDataException e) {
			Logger.logError("Unable to play sound "+file.getQuery(), e);
		}
		init();
	}
	
	private void init() throws ApplicationError {
	    MetaEventListener listener = new MetaEventListener() {
			public void meta(MetaMessage meta) {
				Logger.logDebug("Midi Event = "+meta);
				int type = meta.getType();
				if (type == END_OF_SEQUENCE_MESSAGE) {
					try {
						broadcastFinishEvent();
					} catch (ApplicationError e) {
						Logger.logError("", e);
					}
				}
			}
		};
		sequencer.addMetaEventListener(listener);
	}

	public void replay() throws ApplicationError {
		play();
	}
	
	public void play() throws ApplicationError {
		sequencer.setMicrosecondPosition(0);
		sequencer.start();
		// Setting the volume only works when the music IS playing...
		// This is really shitty!!!!
		new Thread() {
			public void run() {
				try {
					Thread.sleep(500);
					setVolume(Settings.getInstance().getVolumeMusic());
				} catch (Exception e) {
					Logger.logError("Unable to set Volume", e);
				}
			}
		}.start();
	}

	public void stop() {
		sequencer.stop();
	}

	public void setVolume(double v) {
        MidiChannel[] channels = synthesizer.getChannels();
        
        for (int i=0; i<channels.length; i++) {
            channels[i].controlChange(CHANGE_VOLUME_MESSAGE, (int)(v*127));
        }
	}

}
