/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.sounds;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;

import mot.client.swing.common.TimerAbstract;
import mot.common.bean.Settings;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

/**
 * This is background music in midi format.
 * 
 * You may call start and stop as many times as you like! It caches
 * the sound data on creation so there is no delay when the sound is 
 * played.
 */
public class SoundMusicWav extends AbstractSound {

	private static final int CHANGE_VOLUME_MESSAGE = 7;
	private static final int END_OF_SEQUENCE_MESSAGE = 47;
	public static final int VOLUME_CONTROL = 7;
	
	private Clip clip;
	private boolean paused;
	private TimerAbstract restartTimer;

	public SoundMusicWav(File file) throws ApplicationError {
		try {
			AudioInputStream stream = AudioSystem.getAudioInputStream(file);
			AudioFormat format = stream.getFormat();
			DataLine.Info info = new DataLine.Info(Clip.class, format);
			clip = (Clip) AudioSystem.getLine(info);
			clip.open(stream);
			setVolume(Settings.getInstance().getVolumeMusic());
			LineListener listener = getFinishListener();
			if (clip != null) {
				clip.addLineListener(listener);
			}
			paused = true;
		} catch (IOException e) {
			Logger.logError("Unable to play sound "+file.getName(), e);
		} catch (LineUnavailableException e) {
			Logger.logError("Unable to play sound "+file.getName(), e);
		} catch (UnsupportedAudioFileException e) {
			Logger.logError("Unable to play sound "+file.getName(), e);
		}
	}

	public SoundMusicWav(URL file) throws ApplicationError {
		try {
			AudioInputStream stream = AudioSystem.getAudioInputStream(file);
			AudioFormat format = stream.getFormat();
			DataLine.Info info = new DataLine.Info(Clip.class, format);
			clip = (Clip) AudioSystem.getLine(info);
			clip.open(stream);
			setVolume(Settings.getInstance().getVolumeMusic());
			LineListener listener = getFinishListener();
			if (clip != null) {
				clip.addLineListener(listener);
			}
			paused = true;
		} catch (IOException e) {
			Logger.logError("Unable to play sound "+file.getQuery(), e);
		} catch (LineUnavailableException e) {
			Logger.logError("Unable to play sound "+file.getQuery(), e);
		} catch (UnsupportedAudioFileException e) {
			Logger.logError("Unable to play sound "+file.getQuery(), e);
		}
	}

	private LineListener getFinishListener() {
		LineListener listener = new LineListener() {
			public void update(LineEvent event) {
				if (event.getType() == LineEvent.Type.STOP) {
					try {
						broadcastFinishEvent();
					} catch (ApplicationError e) {
						Logger.logError("", e);
					}
				}
			}
		};
		return listener;
	}

	public void replay() throws ApplicationError {
		if (!paused) {
			play();
		}
	}
	
	public void play() throws ApplicationError {
		if (clip.isActive()) {
			clip.stop();
			clip.drain();
		}
		clip.start();
		clip.setMicrosecondPosition(0);
		paused = false;
		if (restartTimer == null) {
			restartTimer = new TimerAbstract(10000) {
				protected void handleTick() throws ApplicationError, ValidationError {
					if (!paused && !clip.isRunning()) {
						clip.stop();
						clip.flush();
						Logger.logDebug("Play Timer tick - Not running");
						play();
					}
				}
			};
		}
		// Setting the volume only works when the music IS playing...
		// This is really shitty!!!!
//		new Thread("WavVolumeThread") {
//			public void run() {
//				try {
//					Thread.sleep(500);
//					setVolume(Settings.getInstance().getVolumeMusic());
//				} catch (Exception e) {
//					Logger.logError("Unable to set Volume", e);
//				}
//			}
//		}.start();
	}

	public void stop() {
		clip.stop();
		clip.drain();
		restartTimer.stop();
		restartTimer = null;
	}

	public void setVolume(double v) {
		try {
			if (v == 0.0) {
				paused = true;
				clip.stop();
			} else {
				if (paused) {
					clip.start();
					paused = false;
				}
				FloatControl gainControl = null;
				try {
					gainControl = (FloatControl) clip.getControl(FloatControl.Type.MASTER_GAIN);
				} catch (Exception e) {	
					Logger.logWarning(e.getMessage());
				}
				if (gainControl == null) {
					gainControl = (FloatControl) clip.getControl(FloatControl.Type.VOLUME);
				}
				float min = gainControl.getMinimum();
				float max = gainControl.getMaximum();
				float range = (max - min) / 2;
				float vol = (float) (min + range + range * v);
				//float vol = (float) (10f * v - 20f);
				Logger.logDebug("SetVolume = "+vol);
				gainControl.setValue(vol);
			}
		} catch (Exception e) {
			if (Logger.isWarning())	Logger.logError("Unable to set volume", e);
		}	    	
	}

}
