/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.sounds;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;

import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Maintains a cache of all sounds defined in the game.
 */
public class Sounds {

   	private static final String WEAPON_FIRE_PREFIX = "WeaponFire";
   	private static final String WEAPON_FIRE_DEFAULT = "WeaponFireDefaut";
   	private static final String DAMAGE_PREFIX = "Damage";
	private static final String ACTION_SOUND = "ActionSound";
	private static final String ERROR_SOUND = "ErrorSound";
	public static final String ALARM_SOUND = "AlarmSound";
   	
	private static HashMap<String,SoundEffect> soundMap = new HashMap<String, SoundEffect>();
   	
	public static SoundEffect getSound(String name) throws ApplicationError {
		//Logger.logDebug("****SOUND "+name);
		SoundEffect sound = soundMap.get(name);
		if (sound == null) {
			String fileName = SoundConfigFactory.getInstance().getSoundFile(name);
			if (fileName != null) {
				sound = new SoundEffect(new File(fileName));
				soundMap.put(name, sound);
			}
		}
		return sound;
	}

	/**
	 * The name of sound is generated using the text WeaponFire_<weapon group>.
	 * The name is then looked up in sounds.xml to find the correct file name for 
	 * that weapon group.
	 * @return The sound used to animate the weapon w.
	 */
	public static Sound getWeaponSound(Weapon w) throws ApplicationError {
		if (w == null) return null;
		String weaponName = w.getSoundGroup();
		if (weaponName == null) {
			weaponName = w.getGroup();
		}
		String name = WEAPON_FIRE_PREFIX+weaponName;
		SoundEffect sound = soundMap.get(name);
		if (sound == null) {
			String fileName = SoundConfigFactory.getInstance().getSoundFile(name);
			if (fileName != null) {
				sound = new SoundEffect(new File(fileName));
				soundMap.put(name, sound);
			} else {
				fileName = SoundConfigFactory.getInstance().getSoundFile(WEAPON_FIRE_DEFAULT);
				if (fileName != null) {
					sound = new SoundEffect(new File(fileName));
					soundMap.put(name, sound);
				}				
			}
		}
		return sound;
	}

	/**
	 * The name of sound is generated using the text Damage_<weapon group>.
	 * The name is then looked up in sounds.xml to find the correct file name for 
	 * that weapon group.
	 * @return The sound used to animate the weapon w.
	 */
	public static Sound getDamageSound(Weapon w) throws ApplicationError {
		if (w == null) return null;
		String name = DAMAGE_PREFIX+w.getGroup();
		SoundEffect sound = soundMap.get(name);
		if (sound == null) {
			DamageSoundConfig config = SoundConfigFactory.getInstance().getDamageSoundConfig(name);
			if (config != null) {
				sound = new SoundEffect(new File(config.getValue()));
				soundMap.put(name, sound);
			}
		}
		return sound;
	}

	/**
	 * The name of sound is generated using the text Damage_<weapon group>.
	 * The name is then looked up in sounds.xml to find the correct file name for 
	 * that weapon group.
	 * @return The config used to animate the weapon w.
	 */
	public static DamageSoundConfig getDamageConfig(Weapon w) throws ApplicationError {
		String name = DAMAGE_PREFIX+w.getGroup();
		return SoundConfigFactory.getInstance().getDamageSoundConfig(name);
	}

	public static SoundEffect getActionSound() throws ApplicationError {
		return getSound(ACTION_SOUND);
	}
	
	public static SoundEffect getAlarmSound() throws ApplicationError {
		return getSound(ALARM_SOUND);
	}
	
	public static SoundEffect getErrorSound() throws ApplicationError {
		return getSound(ERROR_SOUND);
	}
	
	public static void playErrorSound() {
		try {
			SoundEffect s = getSound(ERROR_SOUND);
			if (s != null) s.play();
		} catch (Exception ex) {
	    	Logger.logError("Action failed", ex);
		}
	}

	public static void setVolume(double vol) {
		Collection<SoundEffect> sounds = soundMap.values();
		if (sounds != null) {			
			for (SoundEffect s : sounds) {
				if (s != null) {
					s.setVolume(vol);
				}
			}
		}
	}
	
}

