/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleCraftLeaveEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes animation animation of a craft as it leaves the battle.
 */
public class CraftLeaveBattleAnimation extends EventAnimation  {

    private BattleCraftDelegate craft;
    private Location loc;
    private int lastShiftDistance; 
    private float scalePercent; 
    private GraphicalAnimationContainer container;
    private int direction;
    private int desiredDirection;
    
    private static final int DIRECTION_CHANGE_DELTA = 45;
    private static final int LIFE = 14;
    private static final String SOUND_NAME = "LeaveBattle";
    
    public CraftLeaveBattleAnimation(BattleCraftLeaveEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, LIFE, true, 0, container, Sounds.getSound(SOUND_NAME));
        this.craft = DeligateCacheManager.getBattleCraft(event.getBattle(), event.getCraft());
        this.loc = new Location(craft.getLocation());
        this.lastShiftDistance = 10;
        this.scalePercent = 1.0f;
        this.container = container;
        this.direction = craft.getDirection();
        if (direction > 180) {
            this.desiredDirection = 90; 
        } else {
            this.desiredDirection = 270;
        }
        
        if (Logger.isDebug()) {
            Logger.logDebug("LeaveBattleAnimation.init() "+craft.getName()+" ["+loc+":"+craft.getDirection()+"]");
        }
    }

    public void tick() throws ApplicationError {
        super.tick();
        loc.shiftInDirection(direction, lastShiftDistance);
        lastShiftDistance = (int)(lastShiftDistance * 1.5);
        scalePercent -= 0.07f;
        if (direction != desiredDirection) {
            int diff = direction - desiredDirection;
            if (diff > 0 && diff < DIRECTION_CHANGE_DELTA) {
                direction = desiredDirection;
            } else if (diff < 0 && diff > -DIRECTION_CHANGE_DELTA) {
                direction = desiredDirection;
            } else if (diff < 0) {
                direction -= DIRECTION_CHANGE_DELTA;
            } else {
                direction += DIRECTION_CHANGE_DELTA;
            }
        }
    }

    /**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        Size imageSize = craft.getImageSize();
		int width = (int)(imageSize.getX()*scalePercent);
        int height = (int)(imageSize.getY()*scalePercent);
        int imageLeft = loc.getX()-width/2;
        int imageTop = loc.getY()-height/2;
        ImageUtils.drawRotatedImage(container.getContainer(), 
                                    graphics, 
                                    craft.getImage(), 
                                    imageLeft, 
                                    imageTop, 
                                    width, 
                                    height, 
                                    loc, 
                                    direction);

    }

    /**
     * Paint the target craft on the screen if has not been vetoed by an animation BEFORE this one.
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(craft, this)) {
            ImageUtils.drawBattleCraft(container, graphics, craft, container.getImageSelection(), craft.getState(), false);
            //Logger.logDebug("MoveCraftAnimation.paintNonActive() "+craft.getName()+" @ "+source.toString());
        }
        super.paintNonActive(painter, list);
    }
    

	public void toString(StringBuffer sb) {
		sb.append("LeaveBattleAnimation(");
		sb.append(craft.getName());
		sb.append(")");
	}

	public Area getArea() throws ApplicationError {
        Size imageSize = craft.getImageSize();
		return new Area(loc, imageSize.getX(), imageSize.getY());
	}
}