/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleCraftNewEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes animation animation of a craft as it leaves the battle.
 */
public class CraftMaterialiseAnimation extends EventAnimation  {

	private GraphicalAnimationContainer container;
    private BattleCraftDelegate craft;
    private float scalePercent; 
    private Location loc;
    
    private static final int SCALE_DELTA = 45;
    private static final int LIFE = 25;
    private static final String SOUND_NAME = "Materialise";
    
    public CraftMaterialiseAnimation(BattleCraftNewEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, LIFE, true, 0, container, Sounds.getSound(SOUND_NAME));
        this.craft = DeligateCacheManager.getBattleCraft(event.getBattle(), event.getBattleCraft().getName());
        this.loc = craft.getLocation();
        this.scalePercent = 0.04f;
        this.container = container;
        
        if (Logger.isDebug()) {
            Logger.logDebug("CraftMaterialiseAnimation.init() "+event.getBattleCraft().getName());
        }
    }

    public void tick() throws ApplicationError {
        super.tick();
        scalePercent += 0.05;
    }

    /**
     * Craft Materialise animation does not want the craft drawn because is does not exist until this
     * animation is executed.
     * @return True if the object passed in is the craft for this animation.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(craft);
    }
    
    /**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        Size imageSize = craft.getImageSize();
        int width = (int)(imageSize.getX()*scalePercent);
        int height = (int)(imageSize.getY()*scalePercent);
        if (container.isVisible(craft) && width > 0 && height > 0) {
            int imageLeft = loc.getX()-width/2;
            int imageTop = loc.getY()-height/2;
            ImageUtils.drawRotatedImage(container.getContainer(), 
                                        graphics, 
                                        craft.getImage(), 
                                        imageLeft, 
                                        imageTop, 
                                        width, 
                                        height, 
                                        loc, 
                                        craft.getDirection());

        } else {
            setStage(LIFE);
        }
    }

	public PlayerDelegate getAnimatedObjectOwner() throws ApplicationError {
		if (craft != null) {
			return DeligateCacheManager.getPlayer(craft.getOwner());
		} else {
			return null;
		}
	}
	
	public void toString(StringBuffer sb) {
		sb.append("CraftMaterialiseAnimation(");
		sb.append(craft.getName());
		sb.append(")");
	}
	
	public Area getArea() throws ApplicationError {
        Size imageSize = craft.getImageSize();
		return new Area(loc, imageSize.getX(), imageSize.getY());
	}

}