/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.Location;
import mot.common.bean.ObjectShift;
import mot.common.bean.Settings;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes craft shifting movement to be animated on the screen.
 */
public class CraftShiftAnimation extends CraftMoveAnimation  {

    private Location loc;
    private Location source;
    private Location destination;
    private int distance;
    
    private static final String SOUND_NAME = "CraftShift";

    public CraftShiftAnimation(BattleCraftMoveEvent event, ObjectShift movement, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, movement.getState(), calcLife(event, movement), container, Sounds.getSound(SOUND_NAME));
        this.source = movement.getState().getLocation();
        this.destination = movement.getDestination();
        this.loc = (source == null) ? null : new Location(source);
        this.distance = source.getDistanceTo(destination);
        
        if (Logger.isDebug()) {
            Logger.logDebug("CraftShiftAnimation.init() "+getCraft().getName()+" ["+source.toString()+"=>"+destination.toString()+"]");
        }
    }

    private static int calcLife(BattleCraftMoveEvent event, ObjectShift movement) throws ApplicationError {
    	int qtyFramesPerMove = Settings.getInstance().getQtyFramesPerTurn();
        int life = (int)(qtyFramesPerMove * movement.getMovePercent());
        if (life < 1) {
        	return 1;
        } else {
        	return life;
        }
    }


    public void tick() throws ApplicationError {
        super.tick();
        int remainingDistance = loc.getDistanceTo(destination); 
        int remainingLife = getLife() - getStage();
        if (remainingLife <= 0) {
            loc.shift(destination, remainingDistance);
        } else {
            loc.shift(destination, remainingDistance / remainingLife);
        }
    }

    /**
     * Paint the target craft on the screen
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        ImageUtils.drawBattleCraft(container, graphics, getCraft(), container.getImageSelection(), loc, state.getDirection(), state, false);
        //Logger.logDebug("MoveCraftAnimation.paintActive() "+craft.getName()+" @ "+loc.toString());
        super.paintActive(painter);
    }

    /**
     * Paint the target craft on the screen if has not been vetoed by an animation BEFORE this one.
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(getCraft(), this)) {
            ImageUtils.drawBattleCraft(container, graphics, getCraft(), container.getImageSelection(), source, state.getDirection(), state, false);
            //Logger.logDebug("MoveCraftAnimation.paintNonActive() "+craft.getName()+" @ "+source.toString());
        }
        super.paintNonActive(painter, list);
    }
    
    public Location getLocation() {
        return loc;
    }

    public Location getSource() {
        return source;
    }

    public void setSource(Location source) {
        this.source = source;
    }

    public Location getDestination() {
        return destination;
    }

    public void setDestination(Location destination) {
        this.destination = destination;
    }
    
}
