/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageAnimationConfig;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleCraftTeleportEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes damage to a craft to be display on the screen.
 */
public class CraftTeleportAnimation extends EventAnimation {

    private static final int MIDDLE_STAGE_LENGTH = 4;

	private GraphicalAnimationContainer container;

    /**
     * This is the craft that is moving
     */
    private BattleCraftDelegate craft;

    /**
     * The craft before this animations is taking place.
     */
    private CraftState oldState;
    
    /**
     * The craft at the time this animations is taking place (and after)
     */
    private CraftState newState;
    
    private static ImageAnimationConfig config = null;
    
    private static ImageAnimationConfig getConfig() throws ApplicationError {
		if (config == null) {
			config = AnimationConfigFactory.getInstance().getImageAnimationConfig("Teleport");
		}
		return config;
	}

    private static int _getLife() throws ApplicationError { 
		int qtyImages = getConfig().getQtyImages();
		return qtyImages*2+MIDDLE_STAGE_LENGTH; 
	}
	
    private static final String SOUND_NAME = "Teleport";

    public CraftTeleportAnimation(BattleCraftTeleportEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, _getLife(), true, 0, Sounds.getSound(SOUND_NAME));
        this.craft = DeligateCacheManager.getBattleCraft(event.getBattle(), event.getCraft());
        this.newState = event.getNewState();
        this.oldState = event.getOldState();
        this.container = container;
        
        // Some weapons (such as bots) do not inflict damage so only send oldState
        if (newState == null) {
        	this.newState = oldState;
        }

        if (Logger.isDebug()) {
            Logger.logDebug("CraftTeleportAnimation.init() "+craft.getName());
        }
    }

    /**
     * No other animation should be played at the same time as this animation.
     */
    public boolean isExclusive() {
        return true;
    }

    /**
     * Damage animation does not want the craft drawn because its damage / shields values 
     * may be different.
     * @return True if the object passed in is the craft for this animation.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(craft);
    }

    /**
     * Paints the damage animation and the target craft with its current damage
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        Size imageSize = craft.getImageSize();
		int width = imageSize.getX();
		int height = imageSize.getY();
		int qtyImages = getConfig().getQtyImages();
		int stage = getStage();
		if (stage >= (qtyImages + MIDDLE_STAGE_LENGTH)) {
			// Materialising at destination
			String imageName = config.getActualImage(getLife() - (stage+1));
			Location loc = newState.getLocation();
	        ImageUtils.drawBattleCraft(container, graphics, craft, container.getImageSelection(), newState, false);
	        ImageUtils.drawImageCenteredAt(container, graphics, imageName, loc, width, height);
		} else if (stage < qtyImages){
			// Dematerialising at origin
			String imageName = config.getActualImage(stage);
			Location loc = oldState.getLocation();
	        ImageUtils.drawBattleCraft(container, graphics, craft, container.getImageSelection(), oldState, false);
	        ImageUtils.drawImageCenteredAt(container, graphics, imageName, loc, width, height);
		}
    }

    /**
     * Paint the target craft on the screen with its damage before this this animation
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(craft, this)) {
            ImageUtils.drawBattleObject(container, graphics, craft, container.getImageSelection(), oldState, null);
        }
    }

	public void toString(StringBuffer sb) {
		sb.append("CraftTeleportAnimation(");
		sb.append(craft.getName());
		sb.append(")");
	}

	public GraphicalAnimationContainer getContainer() {
		return container;
	}

	public void setContainer(GraphicalAnimationContainer container) {
		this.container = container;
	}

	public BattleCraftDelegate getCraft() {
		return craft;
	}

	public void setCraft(BattleCraftDelegate craft) {
		this.craft = craft;
	}

	public CraftState getNewState() {
		return newState;
	}

	public void setNewState(CraftState newState) {
		this.newState = newState;
	}

	public CraftState getOldState() {
		return oldState;
	}

	public void setOldState(CraftState oldState) {
		this.oldState = oldState;
	}

	public Area getArea() throws ApplicationError {
        Size imageSize = craft.getImageSize();
		return new Area(oldState != null ? oldState.getLocation() : newState.getLocation(), imageSize.getX(), imageSize.getY());
	}

	public PlayerDelegate getAnimatedObjectOwner() throws ApplicationError {
		if (craft != null) {
			return DeligateCacheManager.getPlayer(craft.getOwner());
		} else {
			return null;
		}
	}
	

}