/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.ObjectTurn;
import mot.common.bean.Settings;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes craft turning movement to be animated on the screen.
 */
public class CraftTurnAnimation extends CraftMoveAnimation {

    private double direction;
    private double fromDirection;
    private double toDirection;
    private double distance;

    private static final String SOUND_NAME = "CraftTurn";

    public CraftTurnAnimation(BattleCraftMoveEvent event, ObjectTurn movement, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, movement.getState(), calcLife(event, movement), container, Sounds.getSound(SOUND_NAME));
        this.fromDirection = movement.getState().getDirection();
        this.toDirection = movement.getTo();
        this.direction = fromDirection;
        this.distance = toDirection-fromDirection;
        
        if (distance > 180) distance = distance - 360;
        if (distance < -180) distance = distance + 360;
        
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTurnAnimation.init() "+getCraft().getName()+" ["+fromDirection+"=>"+toDirection+"]");
        }
    }

    private static int calcLife(BattleCraftMoveEvent event, ObjectTurn movement) throws ApplicationError {
    	int qtyFramesPerMove = Settings.getInstance().getQtyFramesPerTurn(); // 20
        int life = (int)(qtyFramesPerMove * movement.getMovePercent());
        if (life < 1) {
        	return 1;
        } else {
        	return life;
        }
    }

    public void tick() throws ApplicationError {
        super.tick();
        int remainingLife = getLife() - getStage();
        if (remainingLife <= 0) {
        	direction = toDirection;
        } else {
        	direction += distance / getLife();
        }
    }

    /**
     * Paint the target craft on the screen
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        ImageUtils.drawBattleCraft(container, graphics, getCraft(), container.getImageSelection(), state.getLocation(), (int)direction, state, false);
        //Logger.logDebug("MoveCraftAnimation.paintActive() "+craft.getName()+" @ "+loc.toString());
        super.paintActive(painter);
    }

    /**
     * Paint the target craft on the screen if has not been vetoed by an animation BEFORE this one.
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(getCraft(), this)) {
            ImageUtils.drawBattleCraft(container, graphics, getCraft(), container.getImageSelection(), state.getLocation(), (int)direction, state, false);
            //Logger.logDebug("MoveCraftAnimation.paintNonActive() "+craft.getName()+" @ "+source.toString());
        }
        super.paintNonActive(painter, list);
    }

	public double getFromDirection() {
		return fromDirection;
	}

	public double getToDirection() {
		return toDirection;
	}

	public double getDirection() {
		return direction;
	}

}
