/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Color;
import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.BattlePlanetDelegate;
import mot.client.sounds.DamageSoundConfig;
import mot.client.sounds.Sound;
import mot.client.sounds.SoundEffect;
import mot.client.sounds.Sounds;
import mot.client.swing.animation.decorator.CraftAnimationDecorator;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.common.PenUtils;
import mot.client.swing.config.AnimationConfig;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageAnimationConfig;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleDamageEvent;
import mot.common.event.BattleObjectTriggerEvent;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;

/**
 * Causes damage to a craft to be display on the screen.
 */
public class DamageAnimation extends EventAnimation {

    private GraphicalAnimationContainer container;

    /**
     * This is the craft that has been damaged
     */
    private BattleObjectDelegate target;
    
    /**
     * This is the damage inflicted on the craft.
     */
    private double damage;
    
    /**
     * This is the amount of damage blocked by the shield
     */
    private double shieldBlockedDamage;
    
    /**
     * The craft damage at the time this animations is taking place.
     */
    private CraftState newState;

    /**
     * The craft damage before this animations is taking place.
     */
    private CraftState oldState;

    /**
     * True if this craft was destroyed by the damage animated here
     */
    private boolean destroyed;
    
    /**
     * The message displayed to users.
     */
    private String damageDescription;
    
    /**
     * The weapon that caused the damage
     */
	private Weapon weapon;
	
	private SoundEffect shieldHitSound;
	
    /**
     * Some damage animations have special ways to draw the craft while the 
     * damage animation is active. If this is not null, then when this animation
     * is active this decorator should be used to draw the craft.
     */
    private CraftAnimationDecorator craftDecorator;
    
    
    public static String getMessage(BattleDamageEvent event, boolean isAtTarget) {
        if (isAtTarget) {
            return event.getTargetDescription();
        } else {
          	return event.getSourceDescription();
        }
    }
    
    private static ImageAnimationConfig iac_full;
    private static ImageAnimationConfig iac_half;
    

    private static String getShieldDamageImage(double shieldPercent, int stage) throws ApplicationError {
    	if (iac_full == null) {
    		synchronized (DamageAnimation.class) {
				if (iac_full == null) {
					iac_full = AnimationConfigFactory.getInstance().getImageAnimationConfig("shield_damage_full"); 
					iac_half = AnimationConfigFactory.getInstance().getImageAnimationConfig("shield_damage_half"); 
				}
			}
    	}
    	if (shieldPercent > 0.5) {
    		if (stage < iac_full.getQtyImages()) {
    			return iac_full.getActualImage(stage);
    		}
    	} else {
    		if (stage < iac_half.getQtyImages()) {
    			return iac_half.getActualImage(stage);
    		}
    	}
    	return null;
    }
    
    public DamageAnimation(GraphicalAnimationContainer container, BattleObjectTriggerEvent event) throws ApplicationError {
        super(event, AnimationConfig.damageAnimation(), true, 0, null);
        damageDescription = event.getDescription();
        oldState = event.getOriginalState();
        newState = event.getNewState();
	}
    
    public DamageAnimation(BattleDamageEvent event, GraphicalAnimationContainer container, BattleObjectDelegate target, CraftState oldState, CraftState newState, boolean isAtTarget, Weapon weapon) throws ApplicationError {
        super(event, AnimationConfig.damageAnimation(), true, 0, null);
        this.target = target;
		this.weapon = weapon;
        this.damage = event.getDamage().getDamage();
        this.shieldBlockedDamage = event.getShieldBlockedDamage();
        this.newState = newState;
        this.oldState = oldState;
        this.destroyed = event.getDestroyed();
        this.container = container;
        this.damageDescription = getMessage(event, isAtTarget);
        
        // Some weapons (such as bots) do not inflict damage so only send oldState
        if (this.newState == null) {
        	this.newState = oldState;
        }
        if (this.newState == null) {
        	this.newState = new CraftState(null,null,0,0,0,0,0,event.getTargetLocation(target),0,(String)null,null,null,false, 0.0);
        }
        
        boolean targetIsHologram = false;
        if (target instanceof BattleCraftDelegate) targetIsHologram = ((BattleCraftDelegate)target).isHologram();
        
    	Sound sound = Sounds.getDamageSound(weapon);
    	if (sound != null && !targetIsHologram && (damage > 0 || shieldBlockedDamage > 0)) {
    		DamageSoundConfig config = Sounds.getDamageConfig(weapon);
    		if (config != null) {
    			addSound(sound, config.getStartPercent());
    		}
    	}

   		shieldHitSound = Sounds.getSound("ShieldHit");
    	
        if (Logger.isDebug()) {
        	if (event.getTarget() != null) {
        		Logger.logDebug("DamageAnimation.init() "+event.getTarget()+" "+damageDescription+" [dam="+damage+":"+shieldBlockedDamage+"]");
        	} else {
        		Logger.logDebug("DamageAnimation.init() with no taget "+damageDescription+" [dam="+damage+":"+shieldBlockedDamage+"]");
        	}
        }
    }

	/**
     * Damage animation does not want the craft drawn because its damage / shields values 
     * may be different.
     * @return True if the object passed in is the craft for this animation.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(target);
    }

    /**
     * Paints the damage animation and the target craft with its current damage
     */
    public void paintActive(Object painter) throws ApplicationError {
		super.paintActive(painter);
        Graphics2D graphics = (Graphics2D) painter;
        // Destroyed on a planet means the colony, not the actual planet that still needs painting
        //Logger.logDebug("DRAW "+target+" DAMAGE");
        if (target != null && (!destroyed || target instanceof BattlePlanetDelegate)) {
            if (craftDecorator != null) {
                craftDecorator.draw(container, graphics, getStage(), target, container.getImageSelection(), newState);
            } else {
            	String shieldActiveImage = null;
                if (shieldBlockedDamage > 0 && weapon != null && weapon.isBeam()) {
                	double shieldAsPercent = target.getState().getShieldPercent();
                	if (oldState != null && oldState.getShieldPercent() > shieldAsPercent) {
                		shieldAsPercent = oldState.getShieldPercent();
                	}
                	shieldActiveImage = getShieldDamageImage(shieldAsPercent, getStage());
                	if (shieldHitSound != null) {
                		addSound(shieldHitSound, 0);
                		shieldHitSound = null;
                	}
                }
                ImageUtils.drawBattleObject(container, graphics, target, container.getImageSelection(), newState, shieldActiveImage);
            }
            //Logger.logDebug("DamageAnimation.paintActive() "+((DamageEvent)getEvent()).getTarget().getName()+" {"+Comp.left(""+target.getDamageAsPercent())+","+Comp.left(""+target.getShieldAsPercent())+"}");
        }
        PenUtils.setFontBold(graphics);
        graphics.setColor(Color.RED);
        int stage = getStage();
        Location loc = newState.getLocation();
        graphics.drawString(damageDescription, loc.getX()-damageDescription.length()*4, loc.getY()-stage*3);
    }

    /**
     * Paint the target craft on the screen with its damage before this this animation
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
		super.paintNonActive(painter, list);
        Graphics2D graphics = (Graphics2D) painter;
        if (target != null && !list.vetoDrawObject(target, this)) {
            ImageUtils.drawBattleObject(container, graphics, target, container.getImageSelection(), oldState, null);
            //Logger.logDebug("DamageAnimation.paintNonActive() "+((DamageEvent)getEvent()).getTarget().getName()+" {"+Comp.left(""+oldState.getDamagePercent())+","+Comp.left(""+oldState.getShieldPercent())+"}");
        }
        //Logger.logDebug("DamageAnimation.paintNonActive() "+target.getName()+" @ "+target.getLocation().toString());
    }

    public double getDamage() {
        return damage;
    }

    public void setDamage(double damage) {
        this.damage = damage;
    }


    public void setTarget(BattleObjectDelegate target) {
        this.target = target;
    }


    public BattleObjectDelegate getTarget() {
        return target;
    }


    public void setShieldBlockedDamage(double shieldBlockedDamage) {
        this.shieldBlockedDamage = shieldBlockedDamage;
    }


    public double getShieldBlockedDamage() {
        return shieldBlockedDamage;
    }


    public void setCraftDecorator(CraftAnimationDecorator craftDecorator) {
        this.craftDecorator = craftDecorator;
    }


    public CraftAnimationDecorator getCraftDecorator() {
        return craftDecorator;
    }


    public void setDestroyed(boolean destroyed) {
        this.destroyed = destroyed;
    }


    public boolean isDestroyed() {
        return destroyed;
    }

	public String getMessage() {
		return damageDescription;
	}

	public void setMessage(String message) {
		this.damageDescription = message;
	}

	public CraftState getNewState() {
		return newState;
	}

	public void setNewState(CraftState newState) {
		this.newState = newState;
	}

	public CraftState getOldState() {
		return oldState;
	}

	public void setOldState(CraftState oldState) {
		this.oldState = oldState;
	}

	public boolean isAnimated(BattleObjectDelegate bo) {
		return Comp.same(target, bo);
	}

	public void toString(StringBuffer sb) {
		sb.append("DamageAnimation(");
		if (target != null) {
			sb.append(target.getName());
		}
		linkedAnimationsToString(sb);
		sb.append(")");
	}

	public Area getArea() throws ApplicationError {
		if (target == null) {
			Location location = (newState != null) ? newState.getLocation() : oldState.getLocation();
			return new Area(location, 50, 20);
		} else {
			Location location = (newState != null) ? newState.getLocation() : target.getLocation();
			Size imageSize = target.getImageSize();
			return new Area(location, imageSize.getX(), imageSize.getY());
		}
	}

}