/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleDroidMineDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageAnimationConfig;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleDroidMineLaunchEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * This animation does not render anything - It is used to veto the drawing
 * of the droid mine until the launch animation is reached - Otherwise the droid mine
 * would appear on the battle field before animations that were before the droid mine
 * launch - e.g. A craft moves and then fires a droid mine would appear as the craft
 * fires a droid mine (at its move destination) and then moves to the droid mine!
 */
public class DroidMineLaunchAnimation extends EventAnimation {

    private static final String SOUND_NAME = "LaunchMissile";

    private BattleDroidMineDelegate droidMine;
	private GraphicalAnimationContainer container;
	private CraftState state;

    private static ImageAnimationConfig config = null;
    private static ImageAnimationConfig getConfig() throws ApplicationError {
		if (config == null) {
			config = AnimationConfigFactory.getInstance().getImageAnimationConfig("Teleport");
		}
		return config;
	}

    public DroidMineLaunchAnimation(BattleDroidMineLaunchEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, getConfig().getQtyImages()/2, true, 0, Sounds.getSound(SOUND_NAME));
		this.container = container;
		this.state = event.getDroidMine().getState();
        this.droidMine = (BattleDroidMineDelegate) DeligateCacheManager.getBattleAutomaton(event.getBattle(), event.getDroidMine().getName());

        if (Logger.isDebug()) {
            Logger.logDebug("DroidMineLaunchAnimation.init() "+droidMine.getName());
        }
    }

    /**
     * No other animation should be played at the same time as this animation.
     */
    public boolean isExclusive() {
        return true;
    }

    /**
     * Veto the missile from being drawn.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(droidMine);
    }

    /**
     * Paint the target craft on the screen
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        Size imageSize = droidMine.getImageSize();
		int width = imageSize.getX();
		int height = imageSize.getY();
		int stage = getStage();
		
    	int direction = state.getDirection();
        Location location = state.getLocation();
		ImageUtils.drawRotatedImageDeligate(container.getContainer(), graphics, droidMine, location, direction);
		String imageName = config.getActualImage(getLife() - (stage+1));
        ImageUtils.drawImageCenteredAt(container, graphics, imageName, location, width, height);
    }

	public void toString(StringBuffer sb) {
		sb.append("DroidMineLaunchAnimation(");
		sb.append(droidMine.getName());
		sb.append(")");
	}

	public PlayerDelegate getAnimatedObjectOwner() throws ApplicationError {
		if (droidMine != null) {
			return droidMine.getOwnerDelegate();
		} else {
			return null;
		}
	}
	

}