/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Window;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleMineDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.animation.images.ExplosionImage;
import mot.client.swing.common.ImageCache;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Size;
import mot.common.bean.WreckageDetails;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.MotEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;

public class ExplodeAnimation extends EventAnimation  {

    private ExplosionImage image;
    private BattleObjectDelegate target;
    private CraftState oldState;
    
    /**
     * True if this animation should draw the target object when it is non active. 
     * This animation is often used in conjunction with the damage animation which
     * draws the target, so this can be false. Otherwise (e.g. automaton destroy)
     * this should be true!  
     */
    private boolean drawWhenNonActive;
    
    private GraphicalAnimationContainer container;

	private static int getLength() throws ApplicationError {
		return AnimationConfigFactory.getInstance().getQtyImagesAnimationConfig(ExplosionImage.NAME);
	}

    public static final String SOUND_NAME = "Explosion";

    public ExplodeAnimation(BattleDestroyedEvent event, boolean drawWhenNonActive) throws ApplicationError {
        super(event, getLength(), true, 1, Sounds.getSound(SOUND_NAME));
        target = DeligateCacheManager.getBattleObject(event.getBattle(), event.getTarget());
        oldState = event.getOldState();
        Size imageSize = target.getImageSize();
        this.image = new ExplosionImage(imageSize.getX(), imageSize.getY(),event.getLocation(target));
        this.drawWhenNonActive = drawWhenNonActive;
        this.container = MainWindow.getInstance();

        if (Logger.isDebug()) {
            Logger.logDebug("ExplodeAnimation.init() "+target.getName()+"@"+event.getLocation(target));
        }
    }
     
    public ExplodeAnimation(MotEvent event, BattleObjectDelegate target, boolean drawWhenNonActive) throws ApplicationError {
        super(event, getLength(), true, 1, Sounds.getSound(SOUND_NAME));
        this.target = target;
        Size imageSize = target.getImageSize();
        this.image = new ExplosionImage(imageSize.getX(), imageSize.getY(), target.getLocation());
        this.drawWhenNonActive = drawWhenNonActive;
        this.container = MainWindow.getInstance();

        if (Logger.isDebug()) {
            Logger.logDebug("ExplodeAnimation.init() "+target);
        }
    }

    /**
     * Must paint Droid Mines before their explosion is animated but after they have been destroyed.
     * NB Mines are not visible to everybody, so may not be drawn!
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
		super.paintNonActive(painter, list);
        if (!list.vetoDrawObject(target, this)) {
	        Graphics2D graphics = (Graphics2D) painter;
	        boolean paint = true;
	        if (target instanceof BattleMineDelegate) {
	        	BattleMineDelegate mine = (BattleMineDelegate) target;
	        	paint = mine.isOwnedBy(container.getWindowOwner());
	        }
	        if (paint) {
	            ImageUtils.drawBattleObject(container, graphics, target, container.getImageSelection(), oldState, null);
//	        	/ImageUtils.drawBattleObject(container.getContainer(), graphics, target, container.getImageSelection(), container.getWindowOwner(), null);
	        }
        }
	}

    /**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        image.setIndex(getStage()+1);
        ImageUtils.drawRotatedImage(container.getContainer(), graphics, image);
        //Logger.logDebug("CraftDestroyedAnimation.paintActive() "+((CraftDestroyedEvent)getEvent()).getCraft().getName());
    }
    
    @Override
	public boolean vetoDrawAnimatedArea(AnimatedBattleAreaDetails sdd) {
    	if (super.vetoDrawAnimatedArea(sdd)) {
    		return true;
    	}
    	if (sdd instanceof WreckageDetails) {
    		WreckageDetails wreckage = (WreckageDetails) sdd;
    		if (Comp.same(wreckage.getCraftName(), target.getName())) {
    			return true;
    		}
     	}
    	return false;
	}

	public static void loadAllImages(Window window) throws ApplicationError {
        ExplosionImage explosion = new ExplosionImage(null, null);
        for (int i=1; i<=getLength(); i++) {
            explosion.setIndex(i);
            String imageName = explosion.getImage();
            Image image = ImageCache.getImage(window.getToolkit(), imageName);
            window.getGraphics().drawImage(image, 1100, 1100, 2, 2, window);
        }
    }

	public void toString(StringBuffer sb) {
		sb.append("ExplodeAnimation(");
		sb.append(target.getName());
		linkedAnimationsToString(sb);
		sb.append(")");
	}

	public Area getArea() throws ApplicationError {
        Size imageSize = target.getImageSize();
		return new Area(target.getLocation(), imageSize.getX(), imageSize.getY());
	}

	public ExplosionImage getImage() {
		return image;
	}

	public BattleObjectDelegate getTarget() {
		return target;
	}

}