/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Container;
import java.awt.Graphics2D;

import mot.client.animation.EventAnimation;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.animation.images.FearImage;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageConfig;
import mot.common.bean.Area;
import mot.common.event.BattleFearEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes animation over a craft indicating craft's crew are frozen with fear.
 */
public class FearAnimation extends EventAnimation  {

    private Container container;
    private FearImage image;
    private BattleCraftDelegate target;

    public FearAnimation(BattleFearEvent event, Container container) throws ApplicationError {
    	this(event, container, AnimationConfigFactory.getInstance().getImageConfig("FearAnimation"));
    }
    
    private static final String SOUND_NAME = "Fear";
	
    public FearAnimation(BattleFearEvent event, Container container, ImageConfig config) throws ApplicationError {
        super(event, config.getLife(), true, 1, Sounds.getSound(SOUND_NAME));
        this.container = container;
        this.target = DeligateCacheManager.getBattleCraft(event.getBattle(), event.getSource());
		this.image = new FearImage(config, event.getLocation());

        if (Logger.isDebug()) {
            Logger.logDebug("FearAnimation.init() "+target.getName());
        }
    }

    /**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        ImageUtils.drawRotatedImage(container, graphics, image);
    }

    public boolean isExclusive() {
        return true;
    }

    public void toString(StringBuffer sb) {
		sb.append("FearAnimation(");
		sb.append(target.getName());
		sb.append(")");
	}
	
	public Area getArea() throws ApplicationError {
		return new Area(image.getLocation(), image.getWidth(), image.getHeight());
	}

	public FearImage getImage() {
		return image;
	}

	public BattleCraftDelegate getTarget() {
		return target;
	}

	public PlayerDelegate getAnimatedObjectOwner() throws ApplicationError {
		if (target != null) {
			return target.getOwnerDelegate();
		} else {
			return null;
		}
	}
	

}