/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimation;
import mot.client.animation.BattleAnimationList;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.AnimationConfig;
import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes craft turning movement to be animated on the screen.
 */
public class FlashAnimation extends BattleAnimation {

	private GraphicalAnimationContainer container;
    private BattleObjectDelegate target;
    private Location location;
    private int direction;
    
    private static int FLASH_INTERVAL = -1;
    private static int getFashInterval() throws ApplicationError {
    	if (FLASH_INTERVAL < 1) {
    		FLASH_INTERVAL = AnimationConfig.flashInterval();
    	}
    	return FLASH_INTERVAL;
    }

    public FlashAnimation(GraphicalAnimationContainer container, BattleObjectDelegate target, Location location, int direction, int life) throws ApplicationError {
        super(life, true, 0, null);
        
        this.container = container;
        this.target = target;
		this.location = location;
		this.direction = direction;
        
        if (Logger.isDebug()) {
            Logger.logDebug("FlashAnimation.init() "+target.getName());
        }
    }

    /**
     * Paint the target craft on the screen
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        getFashInterval();
        boolean on = ( (getStage()%(2*FLASH_INTERVAL)) < FLASH_INTERVAL );
        if (on) {
        	ImageUtils.drawRotatedImageDeligate(container.getContainer(), graphics, target, location, direction, 2.0);
        } else {
        	ImageUtils.drawRotatedImageDeligate(container.getContainer(), graphics, target, location, direction);
        }
    	//Logger.logDebug("FlashAnimation.paintActive() "+target.getName()+" @ "+target.getLocation());
    }

    /**
     * This does not effect the state so does not need to paint inactive animations.
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
    }

    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(target);
    }

    public boolean isExclusive() {
        return true;
    }

	public void toString(StringBuffer sb) {
		sb.append("FlashAnimation(");
		sb.append(target.getName());
		sb.append(")");
	}

	public Area getArea() throws ApplicationError {
        Size imageSize = target.getImageSize();
		return new Area(location, imageSize.getX(), imageSize.getY());
	}

	public BattleObjectDelegate getTarget() {
		return target;
	}
}
