/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Window;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.animation.images.HoloGoneImage;
import mot.client.swing.common.ImageCache;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Size;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.MotEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

public class HologramDestroyedAnimation extends EventAnimation  {

    private GraphicalAnimationContainer window;
    private HoloGoneImage image;

    /**
     * This is the craft that has been damaged
     */
    private BattleObjectDelegate target;

    /**
     * The craft damage before this animations is taking place.
     */
    private CraftState oldState;

    private static final String SOUND_NAME = "HoloCraftDestroyed";
 
	public HologramDestroyedAnimation(BattleDestroyedEvent event, GraphicalAnimationContainer window) throws ApplicationError {
        super(event, 7, true, 1, Sounds.getSound(SOUND_NAME));
        this.target = DeligateCacheManager.getBattleCraft(event.getBattle(), event.getTarget());
        this.oldState = event.getOldState();
        this.image = new HoloGoneImage(target, oldState);
        this.window = window;

        if (Logger.isDebug()) {
            Logger.logDebug("HologramDestroyedAnimation.init() "+event.getTarget());
        }
    }

	public HologramDestroyedAnimation(MotEvent event, BattleCraftDelegate target, CraftState oldState, GraphicalAnimationContainer window) throws ApplicationError {
        super(event, 7, true, 1, Sounds.getSound(SOUND_NAME));
        this.target = target;
        this.oldState = oldState;
        this.image = new HoloGoneImage(target, oldState);
        this.window = window;

        if (Logger.isDebug()) {
            Logger.logDebug("HologramDestroyedAnimation.init() "+target.getName());
        }
    }

    /**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        image.setIndex(getStage()+1);
        ImageUtils.drawRotatedImage(window.getContainer(), graphics, image);
        //Logger.logDebug("HologramDestroyedAnimation.paintActive() "+((CraftDestroyedEvent)getEvent()).getCraft().getName());
    }

    /**
     * Paint the target craft on the screen with its damage before this this animation
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(target, this)) {
            ImageUtils.drawBattleObject(window, graphics, target, window.getImageSelection(), oldState, null);
            //Logger.logDebug("HologramDestroyedAnimation.paintNonActive() "+((DamageEvent)getEvent()).getTarget().getName()+" {"+Comp.left(""+oldState.getDamagePercent())+","+Comp.left(""+oldState.getShieldPercent())+"}");
        }
        //Logger.logDebug("HologramDestroyedAnimation.paintNonActive() "+target.getName()+" @ "+target.getLocation().toString());
    }

    public static void loadAllImages(Window window) throws ApplicationError {
        HoloGoneImage images = new HoloGoneImage(new Size(1,1), null);
        for (int i=1; i<8; i++) {
            images.setIndex(i);
            String imageName = images.getImage();
            Image image = ImageCache.getImage(window.getToolkit(), imageName);
            window.getGraphics().drawImage(image, 1100, 1100, 2, 2, window);
        }
    }

	public void toString(StringBuffer sb) {
		sb.append("HologramDestroyedAnimation(");
		sb.append(target.getName());
		sb.append(")");
	}
	
	public Area getArea() throws ApplicationError {
		return new Area(image.getLocation(), image.getWidth(), image.getHeight());
	}
	
}