/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import mot.client.animation.BattleAnimation;
import mot.client.animation.EventAnimation;
import mot.client.sounds.Sound;

/**
 * A linked animation is an animation that is a sub part of another animation.
 * They are esentually owned.
 * 
 * The question is, when is the parent animation finished? When its life is up
 * or when it and all its child anuimations life's are up?
 * 
 * If the mustWait flag is true the parent animation is only finished when this
 * animation is finished as well. Otherwise a linked animation finishes when the 
 * parent finishes.
 * 
 * NOTE - Linked animations can be owned by more than one parent animation. To
 * make a linked animation have co-ownership, set lastOwner to the last animation
 * in the list of animations being displayed. Only the LAST owner must wait until 
 * this linked animation completes.
 */
public abstract class LinkedAnimation extends BattleAnimation {

	/**
	 * This is true if the animation  that contains this linked animation can
	 * only be complete when this sub animation is complete.
	 */
	private boolean mustWait;
	
	/**
	 * The last owner in the animation list that owns this animation
	 */
	private EventAnimation lastOwner;

	public LinkedAnimation(BattleAnimation parent, boolean mustWait, Sound sound) {
		this(parent.getLife(), parent.isAfter(), parent.getInterval(), mustWait, sound);
	}
	
	public LinkedAnimation(int life, boolean after, int interval, boolean mustWait, Sound sound) {
		super(life, after, interval, sound);
		this.mustWait = mustWait;
	}

	public boolean isMustWait() {
		return mustWait;
	}

	public void setMustWait(boolean mustWait) {
		this.mustWait = mustWait;
	} 
	
	/**
	 * @param animation The parent animation that owns this linked animations, is complete its self 
	 * but wants to know if it is this child will allow it to finish
	 * @return True if this linked animation will allow its parent to complete.
	 */
	public boolean canComplete(EventAnimation animation) {
		if (!mustWait) {
			return true;
		} else {
			if (isComplete()) {
				return true;
			}
			if (lastOwner == null) {
				return false;
			} else {
				return animation != lastOwner;
			}
		}
	}

	public EventAnimation getLastOwner() {
		return lastOwner;
	}

	public void setLastOwner(EventAnimation lastOwner) {
		this.lastOwner = lastOwner;
	}
}
