/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Color;
import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.sounds.DamageSoundConfig;
import mot.client.sounds.Sound;
import mot.client.sounds.Sounds;
import mot.client.swing.common.PenUtils;
import mot.common.bean.CraftState;
import mot.common.bean.Damage;
import mot.common.bean.Location;
import mot.common.event.BattleDamageEvent;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes damage to a craft to be display on the screen.
 * This animation differs from DamageAnimation because it is designed to be part of
 * another animation. It DOES NOT draw the craft that has been damaged. The craft
 * must be drawn by the parent animation.
 */
public class LinkedDamageAnimation extends LinkedAnimation {

    /**
     * This is the damage inflicted on the craft.
     */
    private double damage;
    
    /**
     * This is the amount of damage blocked by the shield
     */
    private double shieldBlockedDamage;

    /**
     * The craft damage at the time this animations is taking place.
     */
    private CraftState newState;

    /**
     * The message displayed to users.
     */
    private String message;
    
    private BattleObjectDelegate target;

    public LinkedDamageAnimation(BattleDamageEvent event, GraphicalAnimationContainer container, BattleObjectDelegate target, Damage damage, boolean isAtTarget, Weapon weapon) throws ApplicationError {
    	this(event, container, target, damage.getOldTargetState(), damage.getNewTargetState(), isAtTarget, weapon);
    }
    
    public LinkedDamageAnimation(BattleDamageEvent event, GraphicalAnimationContainer container, BattleObjectDelegate target, CraftState oldState, CraftState newState, boolean isAtTarget, Weapon weapon) throws ApplicationError {
        super(25, true, 0, true, null);
        
        this.newState = newState;
        this.message = (event.getSourceDescription() != null) ? event.getSourceDescription() : event.getTargetDescription();
        this.target = target;
        
    	Sound sound = Sounds.getDamageSound(weapon);
    	if (sound != null) {
    		DamageSoundConfig config = Sounds.getDamageConfig(weapon);
    		addSound(sound, config.getStartPercent());
    	}
    	
        if (Logger.isDebug()) {
            Logger.logDebug("LinkedDamageAnimation.init() "+event.getTarget()+" "+damage+" : "+shieldBlockedDamage+" msg="+message);
        }
    }

    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return false;
    }

    /**
     * Paints the damage animation and the target craft with its current damage
     * MUST OVERRIDE paintActive() so we DO NOT paint the craft
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        PenUtils.setFontBold(graphics);
        graphics.setColor(Color.RED);
        int stage = getStage();
        CraftState newState = getNewState();
        Location loc = newState.getLocation();
        String message = getMessage();
        graphics.drawString(message, loc.getX()-message.length()*4, loc.getY()-stage*3);
    }

    /**
     * Paint the target craft on the screen with its damage before this this animation
     * MUST OVERRIDE paintNonActive() so we DO NOT paint the craft
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
    }

	public double getDamage() {
		return damage;
	}

	public String getMessage() {
		return message;
	}

	public CraftState getNewState() {
		return newState;
	}

	public BattleObjectDelegate getTarget() {
		return target;
	}

	public double getShieldBlockedDamage() {
		return shieldBlockedDamage;
	}

	public boolean canBeTransferedTo(EventAnimation a) {
		return true;
	}

	public void toString(StringBuffer sb) {
		sb.append("LinkedDamageAnimation(");
		sb.append(target.getName());
		sb.append(")");
	}
	
}