/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Color;
import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.sounds.Sound;
import mot.client.sounds.Sounds;
import mot.client.swing.animation.decorator.CraftAnimationDecorator;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.common.PenUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleObjectNotificationEvent;
import mot.common.event.BattleObjectNotificationType;
import mot.common.event.MotEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;

/**
 * Causes damage to a craft to be display on the screen.
 */
public class MessageAnimation extends EventAnimation {

    private GraphicalAnimationContainer container;

    /**
     * This is the craft that has been damaged
     */
    private BattleObjectDelegate target;

    /**
     * Generic message intended for display above the target
     */
    private String message;
    
    /**
     * The type of this animation - used to determine sound and icon used!
     * ALSO - messages of different types do not display at the same time.
     */
    private BattleObjectNotificationType type;

    /**
     * The craft damage at the time this animations is taking place.
     */
    private CraftState newState;

    /**
     * The craft damage before this animations is taking place.
     */
    private CraftState oldState;

    /**
     * Some animations have special ways to draw the craft while the
     * animation is active. If this is not null, then when this animation
     * is active this decorator should be used to draw the craft.
     */
    private CraftAnimationDecorator craftDecorator;
    
    private static Sound getSound(BattleObjectNotificationType type) throws ApplicationError {
		if (type == BattleObjectNotificationType.plague) {
	    	return Sounds.getSound("WeaponFirePlague");
		} else if (type == BattleObjectNotificationType.repair) {
	    	return Sounds.getSound("Repair");
		} else if (type == BattleObjectNotificationType.surrender) {
	    	return Sounds.getSound("Fear");
		} else {
			return null; 
		}
    }

    public MessageAnimation(BattleObjectNotificationEvent event, GraphicalAnimationContainer container, BattleObjectDelegate target, CraftState oldState, CraftState newState) throws ApplicationError {
        super(event, 25, true, 0, getSound(event.getType()));
        this.target = target;
        this.newState = newState;
        this.oldState = oldState;
        this.container = container;
        this.message = event.getMessage();
        this.type = event.getType();

        if (Logger.isDebug()) {
            Logger.logDebug("MessageAnimation.init() "+event.getTarget()+" "+message);
        }
    }

    public MessageAnimation(MotEvent event, GraphicalAnimationContainer container, BattleObjectDelegate target, CraftState oldState, CraftState newState, String message, BattleObjectNotificationType type, int life) throws ApplicationError {
        super(event, life, true, 0, null);
        this.target = target;
        this.newState = newState;
        this.oldState = oldState;
        this.container = container;
        this.message = message;
        this.type = type;

        if (Logger.isDebug()) {
            Logger.logDebug("MessageAnimation.init() "+target.getName()+" "+message);
        }
    }

    private ImageConfig icon = null;
	private ImageConfig getIcon() {
		if (icon == null) {
			if (type == BattleObjectNotificationType.plague) {
				icon = ImageConfigFactory.getImageConfigNoError("BattleIconPlague");
			} else if (type == BattleObjectNotificationType.repair) {
				icon = ImageConfigFactory.getImageConfigNoError("BattleIconRepair");
			} else if (type == BattleObjectNotificationType.surrender) {
				icon = ImageConfigFactory.getImageConfigNoError("BattleIconSurrender");
			} else if (type == BattleObjectNotificationType.generic) {
				icon = null;
			} else {
				Logger.logDebug("MessageAnimation.getIcon() Bad Type "+type.getDeclaringClass()); 
			}
		}
		return icon;
	}

    /**
     * Message animation does not want the craft drawn because its damage / shields values
     * may be different.
     * @return True if the object passed in is the craft for this animation.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return oldState != null && bo.equals(target);
    }

    /**
     * Paints the damage animation and the target craft with its current damage
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (craftDecorator != null) {
            craftDecorator.draw(container, graphics, getStage(), target, container.getImageSelection(), newState);
        } else {
            ImageUtils.drawBattleObject(container, graphics, target, container.getImageSelection(), newState, null);
        }
        //Logger.logDebug("DamageAnimation.paintActive() "+((DamageEvent)getEvent()).getTarget().getName()+" {"+Comp.left(""+target.getDamageAsPercent())+","+Comp.left(""+target.getShieldAsPercent())+"}");
        graphics.setColor(Color.RED);
        PenUtils.setFontBold(graphics);
        int stage = getStage();
        Location loc = getMessageLocation();
        int y = loc.getY()-stage*3;
		graphics.drawString(message, loc.getX()-message.length()*4, y);
        ImageConfig icon = getIcon();
        if (icon != null) {
        	Size imageSize = icon.getImageSize();
			ImageUtils._drawImageCenteredAt(container.getContainer(), graphics, icon.getImage(), loc.getX(), y - imageSize.getY() - 5, imageSize.getX(), imageSize.getY());
        }
    }

	private Location getMessageLocation() {
		Location loc;
        if (newState != null) {
        	loc = newState.getLocation();
        } else {
        	loc = target.getLocation();
        }
		return loc;
	}

    /**
     * Paint the target craft on the screen with its damage before this this animation
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(target, this)) {
            ImageUtils.drawBattleObject(container, graphics, target, container.getImageSelection(), oldState, null);
            //Logger.logDebug("DamageAnimation.paintNonActive() "+((DamageEvent)getEvent()).getTarget().getName()+" {"+Comp.left(""+oldState.getDamagePercent())+","+Comp.left(""+oldState.getShieldPercent())+"}");
        }
        //Logger.logDebug("DamageAnimation.paintNonActive() "+target.getName()+" @ "+target.getLocation().toString());
    }

    public void setTarget(BattleObjectDelegate target) {
        this.target = target;
    }


    public BattleObjectDelegate getTarget() {
        return target;
    }

    public void setCraftDecorator(CraftAnimationDecorator craftDecorator) {
        this.craftDecorator = craftDecorator;
    }


    public CraftAnimationDecorator getCraftDecorator() {
        return craftDecorator;
    }

	public void toString(StringBuffer sb) {
		sb.append("MessageAnimation(");
		sb.append(target.getName());
		sb.append(")");
	}

	public Area getArea() throws ApplicationError {
		return (target == null) ? null : new Area(getMessageLocation(), 50, 10);
		
	}
	
	public boolean isAnimated(BattleObjectDelegate bo) {
		return Comp.same(target, bo);
	}

	public BattleObjectNotificationType getType() {
		return type;
	}

}