/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleWeaponMissEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes missile miss target to be animated on the screen.
 */
public class MissMissileAnimation extends EventAnimation {

    private ImageObjectDelegate missile;
    private int direction;
    private Location loc;
    private Location sourceLoc;
    private Location destinationLoc;
    private GraphicalAnimationContainer container;
    
    public MissMissileAnimation(BattleWeaponMissEvent event, GraphicalAnimationContainer container) throws ApplicationError {
    	this(event, DeligateCacheManager.getBattleObject(event.getBattle(), event.getSource()), DeligateCacheManager.getBattleObject(event.getBattle(), event.getTarget()), container);
    }
    
    public MissMissileAnimation(BattleWeaponMissEvent event, ImageObjectDelegate source, BattleObjectDelegate target, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, source.getLocation().getDistanceTo(target.getLocation())/15+6, true, 0, null);
        this.missile = source;
        this.sourceLoc = source.getLocation();
        this.destinationLoc = target.getLocation();
        this.loc = (source == null) ? null : new Location(sourceLoc);
        this.container = container;
        this.direction = sourceLoc.getDirectionFor(destinationLoc);
        
        destinationLoc.shiftInDirection(direction, 100);

        if (Logger.isDebug()) {
            Logger.logDebug("MissMissileAnimation.init() "+missile.getName());
        }
    }

    /**
     * No other animation should be played at the same time as this animation.
     */
    public boolean isExclusive() {
        return true;
    }
    
    /**
     * Move Missile animation does not want the missile drawn becuase this animation draws it.;
     * This animation must draw the missile when active becuase it is in a different location
     * @return True if the object passed in is the missile for this animation.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(missile);
    }

    public Location getSource() {
        return sourceLoc;
    }

    public void setSource(Location source) {
        this.sourceLoc = source;
    }

    public Location getDestination() {
        return destinationLoc;
    }

    public void setDestination(Location destination) {
        this.destinationLoc = destination;
    }

    public void tick() throws ApplicationError {
        super.tick();
        int distance = sourceLoc.getDistanceTo(destinationLoc);
        loc.shift(destinationLoc, distance / getLife());
    }
    
    /**
     * Not required
     */
    public void paintActive(Object painter) throws ApplicationError {
    }

    /**
     * Not required
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
    }

    /**
     * Paint the missile on the screen.
     */
    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        //if (!list.vetoDrawObject(missile, this)) {
            String image = missile.getImage();
            Size size = new Size(100,100);
            int width = size.getX();
            int height = size.getY();
            int x = loc.getX();
            int y = loc.getY();
            ImageUtils.drawRotatedImage(container.getContainer(), graphics, image, x-width/2, y-height/2, width, height, loc, direction);
            //Logger.logDebug("MissMissileAnimation.paintPostProcess() "+missile.getName()+" @ "+loc.toString());
        //}
    }

	public void toString(StringBuffer sb) {
		sb.append("MissMissileAnimation(");
		sb.append(missile.getName());
		sb.append(")");
	}

}