/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.SoundEffect;
import mot.client.sounds.Sounds;
import mot.client.swing.animation.strokes.WeaponStroke;
import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleWeaponMissEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes Weapon Fire to be display on the screen to a location is space close
 * to but not hitting the target craft (weapon miss animation).
 */
public class MissTargetAnimation extends EventAnimation {

    private GraphicalAnimationContainer container;
    private Location [] sourceLoc;
    private BattleObjectDelegate sourceCraft;
    private Location destinationLoc;
    private WeaponStroke stroke;

    public MissTargetAnimation(BattleWeaponMissEvent event, GraphicalAnimationContainer container, WeaponStroke stroke) throws ApplicationError {
    	this(event, DeligateCacheManager.getBattleObject(event.getBattle(), event.getSource()), DeligateCacheManager.getBattleObject(event.getBattle(), event.getTarget()), container, stroke);
    }
    
    public MissTargetAnimation(BattleWeaponMissEvent event, BattleObjectDelegate source, BattleObjectDelegate target, GraphicalAnimationContainer container, WeaponStroke stroke) throws ApplicationError {
        super(event, 20, true, 0, Sounds.getWeaponSound(event.getWeapon()));
        
        Location sloc = event.getSourceLocation();
        Location tloc = event.getTargetLocation();
		int sdir = event.getSourceDirection();
		Size imageSize = target.getImageSize();
		sourceCraft = source;
        sourceLoc = WeaponFireAnimation.getDishWeaponFireLocationFrom(source, sloc, sdir, tloc);
        destinationLoc  = WeaponFireAnimation.getWeaponFireLocationTo(source, tloc);
        destinationLoc = new Area(tloc, imageSize.getX(), imageSize.getY()).getFurthestEdgeTo(sloc);

        this.container = container;
        this.stroke = stroke;

    	SoundEffect sound = Sounds.getSound("WeaponMiss");
    	if (sound != null) {
    		addSound(sound, getLife()/2);
    	}

        if (Logger.isDebug()) {
            Logger.logDebug("MissTargetAnimation.init() "+event.getSource());
        }
    }

    /**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
    }
    
    /**
     * Paint the fire on the screen.
     */
    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
        if (active) {
        	Graphics2D graphics = (Graphics2D) painter;
            for (int i = 0; i < sourceLoc.length; i++) {
                stroke.paint(container.getContainer(), graphics, getStage(), sourceLoc[i], destinationLoc);
                //Logger.logDebug("MissTargetAnimation.paintPostProcess() "+event.getTarget().getName());				
			}
        }
    }

	public boolean isExclusive() {
		return true;
	}

	public void toString(StringBuffer sb) {
		sb.append("MissTargetAnimation");
	}

	public Location [] getSource() {
		return sourceLoc;
	}

	public Location getDestination() {
		return destinationLoc;
	}

	public PlayerDelegate getAnimatedObjectOwner() throws ApplicationError {
		if (sourceCraft != null) {
			return sourceCraft.getOwnerDelegate();
		} else {
			return null;
		}
	}

}