/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import mot.client.animation.EventAnimation;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.MissilesDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.common.event.BattleMissileLaunchEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * This animation does not render anything - It is used to veto the drawing
 * of the missile until the launch animation is reached - Otherwise the missile
 * would appear on the battle field before animations that were before the missile
 * launch - e.g. A craft moves and then fires a missile would appear as the craft
 * fires a missile (at its move destination) and then moves to the missile!
 */
public class MissileLaunchAnimation extends EventAnimation {

	private MissilesDelegate missile;

    private static final String SOUND_NAME = "LaunchMissile";
    
    public MissileLaunchAnimation(BattleMissileLaunchEvent event) throws ApplicationError {
        super(event, 1, true, 0, Sounds.getSound(SOUND_NAME));
        this.missile = (MissilesDelegate) DeligateCacheManager.getBattleAutomaton(event.getBattle(), event.getMissile().getName());

        if (Logger.isDebug()) {
            Logger.logDebug("MissileLaunchAnimation.init() "+missile.getName());
        }
    }

    /**
     * No other animation should be played at the same time as this animation.
     */
    public boolean isExclusive() {
        return true;
    }

    /**
     * Veto the missile from being drawn.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(missile);
    }

    /**
     * Paint the target craft on the screen
     */
    public void paintActive(Object painter) throws ApplicationError {
    }

	public PlayerDelegate getAnimatedObjectOwner() throws ApplicationError {
		if (missile != null) {
			return missile.getOwnerDelegate();
		} else {
			return null;
		}
	}
    
	public void toString(StringBuffer sb) {
		sb.append("MissileLaunchAnimation(");
		sb.append(missile.getName());
		sb.append(")");
	}

}