/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.util.ArrayList;
import java.util.Iterator;

import mot.client.animation.BattleAnimation;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.common.bean.ObjectMovement;
import mot.common.bean.ObjectShift;
import mot.common.bean.ObjectTurn;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.event.BattleCraftTeleportEvent;
import mot.common.event.BattleDamageEvent;
import mot.common.event.BattleDroidMineMoveEvent;
import mot.common.event.BattleMissileMoveEvent;
import mot.common.event.LinkedEventBase;
import mot.common.event.MotEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;


public class MoveCraftAnimationFactory {

	
	/**
	 * Creates animations that will display to the user the craft move detailed in the
	 * CraftMoveEvent passed in. A craft move event contains a number of shifts and turns.
	 * Each shift will generate a ShoftAnimation and each turn will generate a turn animation.
	 * The order of the animations will be the same as the order of the movements in the move
	 * event.
	 * 
	 * Craft movements often have linked animations. The following are handles by this
	 * factory;
	 * 
	 * 1 - Missile Movements
	 * 		This creates a LinkedMoveMissile animation for each MoveMissile event.
	 * 		Each move missile animation is owned all movement animations, this allows
	 * 		the missiles movement to be animated across the entire move instead of 
	 * 		just one segment.
	 * 
	 * 2 - Damage
	 * 		If a shift animation contains damage then an linked damage animation is created 
	 * 		for that shift. This damage animation is owned by the shift animation and all
	 * 		animations that follow that shift. This is so that the damage animation is not 
	 * 		halted too quickly. 
	 * 
	 * @param event
	 * @param container
	 * @return
	 * @throws ApplicationError
	 */
    public static BattleAnimation [] create(BattleCraftMoveEvent event, GraphicalAnimationContainer container) throws ApplicationError {
    	ObjectMovement [] movements = event.getMovements();
    	if (movements == null) return null;
    	ArrayList animations = new ArrayList();
    	ArrayList damageAnimations = null;
    	int totalLife = 0;
    	for (int i=0; i<movements.length; i++) {
    		if (movements[i] instanceof ObjectShift) {
    			ObjectShift shift = (ObjectShift)movements[i];
	    		CraftShiftAnimation shiftAnimation = new CraftShiftAnimation(event, shift, container);
	    		if (shift.getDamage() != null) {
	    			// Craft has moved through an asteroid field are been damaged as part of its movement
	    			BattleDamageEvent damageEvent = new BattleDamageEvent(event.getBattle(), event.getName(), shift.getDamage()); 
	    			BattleObjectDelegate craft = DeligateCacheManager.getBattleObject(event.getBattle(), event.getName());
	    			LinkedDamageAnimation damageAnimation = new LinkedDamageAnimation(damageEvent, container, craft, shift.getDamage(), false, null);
	    			shiftAnimation.setState(shift.getDamage().getNewTargetState());
    				if (damageAnimations == null) damageAnimations = new ArrayList();
    				damageAnimations.add(damageAnimation);
	    		}
	    		totalLife += shiftAnimation.getLife();
	    		animations.add( shiftAnimation );
    		} else if (movements[i] instanceof ObjectTurn) {
    			ObjectTurn turn = (ObjectTurn)movements[i];
    			CraftTurnAnimation turnAnimation = new CraftTurnAnimation(event, turn, container); 
	    		totalLife += turnAnimation.getLife();
	    		animations.add( turnAnimation );
    		}
    		if (damageAnimations != null) {
				LinkedAnimation [] damAnims = (LinkedAnimation []) damageAnimations.toArray(new LinkedDamageAnimation[damageAnimations.size()]);
				((EventAnimation)animations.get(i)).addLinkedAnimation(damAnims);
    		}
    	}
    	
		if (damageAnimations != null && animations.size() > 0) {
			Iterator i = damageAnimations.iterator();
			EventAnimation last = (EventAnimation)animations.get(animations.size()-1);
			while (i.hasNext()) {
				LinkedDamageAnimation a = (LinkedDamageAnimation) i.next();
				a.setLastOwner(last);
			}
		}
		
		EventAnimation[] r =  (EventAnimation[])animations.toArray(new EventAnimation[animations.size()]);

		if (r != null) {
			LinkedAnimation [] linkedAnimations = createLinkedAnimations(event, totalLife, container);
			for (int i = 0; i<r.length; i++) {
				r[i].addLinkedAnimation(linkedAnimations);
			}
		}
		
		return r;
    }

    private static int getQtyDamageAnimationsForCraft(ArrayList damageAnimations, BattleCraftDelegate craft) {
    	if (damageAnimations == null) {
    		return 0;
    	}
    	int qty = damageAnimations.size();
    	int r = 0;
    	for (int i = 0; i < qty; i++) {
    		LinkedDamageAnimation da = (LinkedDamageAnimation) damageAnimations.get(i);
    		if (Comp.same(da.getTarget(), craft)) {
    			r++;
    		}
		}
		return r;
	}

	private static LinkedAnimation [] createLinkedAnimations(LinkedEventBase event, int life, GraphicalAnimationContainer container) throws ApplicationError {
    	MotEvent [] linkedEvents = event.getLinkedEvents();
        if (linkedEvents == null) return null;
        ArrayList r = null;
        for (int i=0; i<linkedEvents.length; i++) {
        	LinkedAnimation la = createLinkedAnimation(linkedEvents[i], life, container);
        	if (la != null) {
        		if (r == null) r = new ArrayList();
        		r.add(la);
        	}
        }
        if (r != null) {
        	return (LinkedAnimation []) r.toArray(new LinkedAnimation [r.size()]);
        } else {
        	return null;
        }
    }
    
    private static LinkedAnimation createLinkedAnimation(MotEvent event, int life, GraphicalAnimationContainer container) throws ApplicationError {
    	if (event instanceof BattleMissileMoveEvent) {
    	    BattleMissileMoveEvent mme = (BattleMissileMoveEvent)event;
    	    return new MoveMissileLinkedAnimation(mme, container, life);
    	} else if (event instanceof BattleDroidMineMoveEvent) {
    		BattleDroidMineMoveEvent mdme = (BattleDroidMineMoveEvent)event;
    	    return new MoveDroidMineLinkedAnimation(mdme, container, life);
    	}

    	return null;
    }

    public static BattleAnimation create(BattleCraftTeleportEvent event, GraphicalAnimationContainer container) throws ApplicationError {
    	CraftTeleportAnimation r = new CraftTeleportAnimation(event, container);
		LinkedAnimation [] linkedAnimations = createLinkedAnimations(event, r.getLife(), container);
		r.addLinkedAnimation(linkedAnimations);
		return r;
    }
    
}
