/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleDroidMineDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.Location;
import mot.common.event.BattleDroidMineMoveEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes missile movement to be animated on the screen.
 */
public class MoveDroidMineAnimation extends EventAnimation {

    private BattleDroidMineDelegate droidMine;
    private Location loc;
    private Location source;
    private Location destination;
    private GraphicalAnimationContainer container;

    
    private static final String SOUND_NAME = "MoveDroidMine";

    public MoveDroidMineAnimation(BattleDroidMineMoveEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, event.getFrom().getDistanceTo(event.getTo())/15+6, true, 0, Sounds.getSound(SOUND_NAME));
        this.droidMine = (BattleDroidMineDelegate) DeligateCacheManager.getBattleAutomaton(event.getBattle(), event.getDroidMine());
        this.source = event.getFrom();
        this.destination = event.getTo();
        this.loc = (source == null) ? null : new Location(source);
        this.container = container;

        if (Logger.isDebug()) {
            Logger.logDebug("MoveDroidMineAnimation.init() "+droidMine.getName());
        }
    }

    /**
     * No other animation should be played at the same time as this animation.
     */
    public boolean isExclusive() {
        return true;
    }
    
    /**
     * Move DroidMine animation does not want the missile drawn because this animation draws it.;
     * This animation must draw the missile when active because it is in a different location
     * @return True if the object passed in is the missile for this animation.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(droidMine);
    }

    public Location getSource() {
        return source;
    }

    public void setSource(Location source) {
        this.source = source;
    }

    public Location getDestination() {
        return destination;
    }

    public void setDestination(Location destination) {
        this.destination = destination;
    }

    public void tick() throws ApplicationError {
        super.tick();
        int distance = source.getDistanceTo(destination);
        loc.shift(destination, distance / getLife());
    }
    
    /**
     * Paint the target craft on the screen because it may have been destroyed
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        // Damage animations will draw objects and there may be many
        //ImageUtils.drawBattleObject(container.getContainer(), graphics, droidMine.getTarget(), container.getImageSelection(), container.getWindowOwner(), state, null);
        //Logger.logDebug("MoveDroidMineAnimation.paintActive() "+missile.getName()+" @ "+source.toString()+" TARGET "+droidMine.getTarget().getName()+" {"+Comp.left(""+state.getDamagePercent())+","+Comp.left(""+state.getShieldPercent())+"}");
    }

    /**
     * Paint the target craft on the screen because it may have been destroyed
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        // Damage animations will draw objects and there may be many
//        if (!list.vetoDrawObject(droidMine.getTarget(), this)) {
//            ImageUtils.drawBattleObject(container.getContainer(), graphics, droidMine.getTarget(), container.getImageSelection(), container.getWindowOwner(), state, null);
//            //Logger.logDebug("MoveDroidMineAnimation.paintNonActive() "+droidMine.getName()+" @ "+source.toString()+" TARGET "+droidMine.getTarget().getName()+" {"+Comp.left(""+state.getDamagePercent())+","+Comp.left(""+state.getShieldPercent())+"}");
//        }
    }

    /**
     * Paint the missile on the screen.
     */
    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(droidMine, this)) {
            ImageUtils.drawRotatedImageDeligate(container.getContainer(), graphics, droidMine, loc, droidMine.getDirection(loc));
            //Logger.logDebug("MoveDroidMineAnimation.paintPostProcess() "+droidMine.getName()+" @ "+loc.toString());
        }
    }

	public void toString(StringBuffer sb) {
		sb.append("MoveDroidMineAnimation(");
		sb.append(droidMine.getName());
		sb.append(")");
	}

}