/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.MissilesDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.event.BattleMissileMoveEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes missile movement to be animated on the screen.
 */
public class MoveMissileAnimation extends EventAnimation {

    private MissilesDelegate missile;
    private Location loc;
    private Location source;
    private Location destination;
    private int direction;
    private GraphicalAnimationContainer container;

    /**
     * State of the automaton after this event / animation.
     */
    private CraftState state;
    
    /**
     * The state of the target after this animation.
     */
    private CraftState targetState;
    
    private static final String SOUND_NAME = "MoveMissile";
	private int distance;

    public MoveMissileAnimation(BattleMissileMoveEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, event.getFrom().getDistanceTo(event.getTo())/15+6, true, 0, Sounds.getSound(SOUND_NAME));
        this.missile = (MissilesDelegate) DeligateCacheManager.getBattleAutomaton(event.getBattle(), event.getMissile());
        this.source = event.getFrom();
        this.destination = event.getTo();
        this.direction = source.getDirectionFor(destination);
        this.loc = (source == null) ? null : new Location(source);
        this.container = container;
        this.state = event.getState();
        this.targetState = event.getTargetState();

        distance = source.getDistanceTo(destination);

        if (Logger.isDebug()) {
            Logger.logDebug("MoveMissileAnimation.init() "+missile.getName()+" "+source+"->"+destination);
        }
    }

    /**
     * No other animation should be played at the same time as this animation.
     */
    public boolean isExclusive() {
        return true;
    }
    
    /**
     * Move Missile animation does not want the missile drawn because this animation draws it.;
     * This animation must draw the missile when active because it is in a different location
     * @return True if the object passed in is the missile for this animation.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(missile);
    }

    public Location getSource() {
        return source;
    }

    public void setSource(Location source) {
        this.source = source;
    }

    public Location getDestination() {
        return destination;
    }

    public void setDestination(Location destination) {
        this.destination = destination;
    }

    public void tick() throws ApplicationError {
        super.tick();
        loc.shift(destination, distance / getLife());
    }
    
    /**
     * Paint the target craft on the screen because it may have been destroyed
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        //ImageUtils.drawBattleObject(container, graphics, missile.getTargetDelegate(), container.getImageSelection(), state, null);
        //Logger.logDebug("MoveMissileAnimation.paintActive() "+missile.getName()+" @ "+source.toString()+" TARGET "+missile.getTarget().getName()+" {"+Comp.left(""+state.getDamagePercent())+","+Comp.left(""+state.getShieldPercent())+"}");
    }

    /**
     * Paint the target craft on the screen because it may have been destroyed
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
//        BattleObjectDelegate targetDelegate = missile.getTargetDelegate();
//		if (!list.vetoDrawObject(targetDelegate, this)) {
//            ImageUtils.drawBattleObject(container, graphics, targetDelegate, container.getImageSelection(), state, null);
//            //Logger.logDebug("MoveMissileAnimation.paintNonActive() "+missile.getName()+" @ "+source.toString()+" TARGET "+missile.getTarget().getName()+" {"+Comp.left(""+state.getDamagePercent())+","+Comp.left(""+state.getShieldPercent())+"}");
//        }
    }

    /**
     * Paint the missile on the screen.
     */
    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(missile, this)) {
            ImageUtils.drawRotatedImageDeligate(container.getContainer(), graphics, missile, loc, direction);
            //Logger.logDebug("MoveMissileAnimation.paintPostProcess() "+missile.getName()+" @ "+loc.toString());
        }
    }
    
	public Area getArea() throws ApplicationError {
		if (source != null && destination != null) {			
			return new Area(source.getX(), source.getY(), destination.getX(), destination.getY());
		} else {
			return null;
		}
	}

	public void toString(StringBuffer sb) {
		sb.append("MoveMissileAnimation(");
		sb.append(missile.getName());
		sb.append(")");
	}

	public MissilesDelegate getMissile() {
		return missile;
	}

}