/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.MissilesDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.Location;
import mot.common.event.BattleMissileMoveEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;

/**
 * Causes missile movement to be animated on the screen as a linked animation to another animation.
 */
public class MoveMissileLinkedAnimation extends LinkedAnimation {

    private static final String SOUND_NAME = "MoveMissile";

    private MissilesDelegate missile;
    private Location loc;
    private Location source;
    private Location destination;
    private int direction;
    private GraphicalAnimationContainer container;

	private int distance;

    public MoveMissileLinkedAnimation(BattleMissileMoveEvent event, GraphicalAnimationContainer container, int life) throws ApplicationError {
        super(Comp.max(life, 8), true, 0, false, Sounds.getSound(SOUND_NAME));
        this.missile = (MissilesDelegate) DeligateCacheManager.getBattleAutomaton(event.getBattle(), event.getMissile());
        this.source = event.getFrom();
        this.destination = event.getTo();
        this.direction = source.getDirectionFor(destination);
        this.loc = (source == null) ? null : new Location(source);
        this.container = container;
        
        distance = source.getDistanceTo(destination);

        //Logger.logDebug("*** CraftMoveAnimation Life = "+getLife());
        
        if (Logger.isDebug()) {
            Logger.logDebug("MoveMissileLinkedAnimation.init() "+missile.getName()+" ["+source.toString()+"=>"+destination.toString()+"]");
        }
    }

    /**
     * Other animation should be played at the same time as this animation.
     */
    public boolean isExclusive() {
        return false;
    }
    
    /**
     * Move Missile animation does not want the missile drawn because this animation draws it.;
     * This animation must draw the missile when active because it is in a different location
     * @return True if the object passed in is the missile for this animation.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        return bo.equals(missile);
    }

    public Location getSource() {
        return source;
    }

    public void setSource(Location source) {
        this.source = source;
    }

    public Location getDestination() {
        return destination;
    }

    public void setDestination(Location destination) {
        this.destination = destination;
    }

    public void tick() throws ApplicationError {
        super.tick();
        loc.shift(destination, distance / getLife());
    }
    
    /**
     * Paint the target craft on the screen because it may have been destroyed
     */
    public void paintActive(Object painter) throws ApplicationError {
    }

    /**
     * Paint the target craft on the screen because it may have been destroyed
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
        //Logger.logDebug("MoveMissileLinkedAnimation.paintNonActive() "+missile.getName()+" @ "+source.toString()+" TARGET "+missile.getTarget().getName());
    }

    /**
     * Paint the missile on the screen.
     */
    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        if (!list.vetoDrawObject(missile, this)) {
            ImageUtils.drawRotatedImageDeligate(container.getContainer(), graphics, missile, loc, direction);
            //Logger.logDebug("MoveMissileLinkedAnimation.paintPostProcess() "+missile.getName()+" @ "+loc.toString());
        }
    }

	public void toString(StringBuffer sb) {
		sb.append("MoveMissileLinkedAnimation(");
		sb.append(missile.getName());
		sb.append(")");
	}

}