/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.ImageConfigFactory;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.event.BattlePulseEvent;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes a shock wave animation from a single point outwards until its damage is zero.
 */
public class PulseAnimation extends EventAnimation  {

    private String imageName;
    private GraphicalAnimationContainer container;
    private BattleObjectDelegate source;
    private CraftState oldState;
    private int stepDistance;
    private int stepWidth;
    
    private Location origin;
    private Location currentLocation;
    private int currentWidth;
    private int direction;
    private int range;
    private int angle;
    
    private static final String SOUND_NAME = "Pulse";

    public PulseAnimation(BattlePulseEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, 8, true, 0, Sounds.getSound(SOUND_NAME));
        imageName = ImageConfigFactory.getImageForConfig("Pulse");
        this.container = container;
        this.source = DeligateCacheManager.getBattleObject(event.getBattle(), event.getSource());
        Weapon weapon = event.getWeaponTechnology();
        range = weapon == null ? 150 : weapon.getRange();
        angle = weapon == null ? 60 : weapon.getParam1AsInt();
        int life = getLife();
		this.stepDistance = range / life;
        this.stepWidth = (range - source.getImageSize().getX()/2) / life;
        this.oldState = event.getOldState();
        this.origin = event.getOldState().getLocation();
        this.direction = event.getOldState().getDirection();
        this.currentLocation = new Location(origin);
        this.currentWidth = source.getImageSize().getX()/2;

        currentLocation.shiftInDirection(direction, source.getImageSize().getX()/2);
        
        if (Logger.isDebug()) {
            Logger.logDebug("PulseAnimation.init() "+event.getSource()+" range="+range+" @ "+origin);
        }
    }
    
    protected void stageChange() throws ApplicationError {
        currentLocation.shiftInDirection(direction, stepDistance);
        currentWidth += stepWidth;
	}

	/**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        
        int stage = getStage();
        int width = stage * stepDistance;
        
        //Logger.logDebug("DRAW "+source+" PULSE at "+currentLocation+" size "+currentWidth);
        
        ImageUtils.drawRotatedImage(container.getContainer(), graphics, imageName, currentWidth, currentWidth, currentLocation, direction);
    }
    
	public void toString(StringBuffer sb) {
		sb.append("PulseAnimation(");
		sb.append(source.getName());
		linkedAnimationsToString(sb);
		sb.append(")");
	}


	public BattleObjectDelegate getSource() {
		return source;
	}


	

}