/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;
import java.util.ArrayList;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.bean.BattleCraftDeligateStateTupple;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.DamageSoundConfig;
import mot.client.sounds.Sound;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.event.BattleDamageEvent;
import mot.common.event.BattleDamageRingEvent;
import mot.common.event.MotEvent;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;

/**
 * Causes a shock wave animation from a single point outwards until its damage is zero.
 */
public class ShieldBurstAnimation extends EventAnimation  {

	private ImageConfig config;
    private String imageName;
    private GraphicalAnimationContainer container;
    private BattleObjectDelegate source;
    private int step;
    
    private int x;
    private int y;
    
    private static final int SHIELD_BURST_LIFE = 8;
    private static final String CONFIG_NAME = "ShieldBurst";
    private static final String SOUND_NAME = "ShieldBurst";
    

    public ShieldBurstAnimation(BattleDamageRingEvent event, GraphicalAnimationContainer container, Weapon weapon) throws ApplicationError {
        super(event, AnimationConfigFactory.getInstance().getImageConfig(CONFIG_NAME).getLife(), true, 1, Sounds.getSound(SOUND_NAME));
        config = AnimationConfigFactory.getInstance().getImageConfig(CONFIG_NAME);
        imageName = config.getImage();
        this.container = container;
        this.source = DeligateCacheManager.getBattleObject(event.getBattle(), event.getSource());
        this.step = 24 / SHIELD_BURST_LIFE;
        this.x = event.getLocation().getX();
        this.y = event.getLocation().getY();

        if (Logger.isDebug()) {
            Logger.logDebug("ShieldBurstAnimation.init() "+event.getSource()+" dam="+event.getDamage()+" @ "+source.getLocation());
        }

    	Sound sound = Sounds.getDamageSound(weapon);
    	if (sound != null) {
    		DamageSoundConfig config = Sounds.getDamageConfig(weapon);
    		if (config != null) {
    			addSound(sound, SHIELD_BURST_LIFE);
    		}
    	}
    }
    
    
	/**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        
        int stage = getStage();
        if (stage < SHIELD_BURST_LIFE) {
            Size imageSize = source.getImageSize();
        	int radius = (stage * 3) + imageSize.getX() + 2;
        	if (stage > SHIELD_BURST_LIFE/2) {
        		radius = (SHIELD_BURST_LIFE/2-(stage-SHIELD_BURST_LIFE/2)) * 3 + imageSize.getY() + 2;
        	}
        	ImageUtils.drawImage(container.getContainer(), graphics, imageName, x-radius, y-radius, radius*2, radius*2);
        	BattleCraftDeligateStateTupple[] doa = getDamagedObjects();
			if (doa != null) {    				
				for (int i = 0; i < doa.length; i++) {
					drawObjectWithShake(container, graphics, doa[i], (stage%2==0));
				}
			}
        }
    }

    private void drawObjectWithShake(GraphicalAnimationContainer container2, Graphics2D graphics, BattleCraftDeligateStateTupple target, boolean even) throws ApplicationError {
    	BattleCraftDelegate craft = target.getCraft();
    	CraftState state = target.getState();
    	if (!state.isDestroyed()) {
			Location loc = new Location(state.getLocation());
			int direction = loc.getDirectionFor(source.getLocation());
			loc.shiftInDirection(direction, even ? -2 : 2);
			direction += 90;
			if (direction > 360) direction -= 360;
			int r = Utils.random(5);
			loc.shiftInDirection(direction, r>2 ? r-6 : r+2);
			ImageUtils.drawBattleCraft(container, graphics, craft, container.getImageSelection(), loc, state.getDirection(), state, false);
    	}
	}

	private ArrayList<BattleCraftDeligateStateTupple> damagedObjects = new ArrayList();
    
	public void addDamagedBattleObject(MotEvent event) {
		if (event instanceof BattleDamageEvent) {
			BattleDamageEvent de = (BattleDamageEvent) event;
			BattleObjectDelegate target = DeligateCacheManager.getBattleObject(de.getBattle(), de.getTarget());
			if (target != null && target instanceof BattleCraftDelegate) {
				if (de.getCraftState() != null) {
					// Must use New State because craft may be destroyed and we need to know
					damagedObjects.add( new BattleCraftDeligateStateTupple((BattleCraftDelegate) target, de.getNewTargetState()) );
				}
			}
// Must not display crafts that have been destroyed 			
//		} else if (event instanceof DestroyedEvent) {
//			DestroyedEvent de = (DestroyedEvent) event;
//			if (de.getTarget() instanceof BattleCraftDeligate) {
//				if (de.getTarget() != null && de.getOldState() != null) {
//					damagedObjects.add( new BattleCraftDeligateStateTupple((BattleCraftDeligate) de.getTarget(), de.getOldState()) );
//				}
//			}
		}
	}
	
	public BattleCraftDeligateStateTupple [] getDamagedObjects() {
		return damagedObjects.toArray(new BattleCraftDeligateStateTupple[damagedObjects.size()]);
	}
    
    /**
     * Veto the damaged crafts from being drawn when we are animating the shield burst ring.
     * This is because we paint the crafts wobbling around and do not want them painted twice.
     */
    public boolean vetoDrawObject(BattleObjectDelegate bo) {
        int stage = getStage();
        if (stage < SHIELD_BURST_LIFE) {        	
	    	if (MainWindow.getExestingInstance().isBattleMapConfigurationActive()) {
	    		BattleAnimationList al = MainWindow.getExestingInstance().getBattleWinConfig().getListener().getState().getAnimations();
	    		if (al != null && al.isActiveAnimation(this)) {    			
	    			BattleCraftDeligateStateTupple[] doa = getDamagedObjects();
	    			if (doa != null) {    				
	    				for (int i = 0; i < doa.length; i++) {
	    					if (bo != null && Comp.same(bo.getName(), doa[i].getCraft().getName())) {
	    						return true;
	    					}
	    				}
	    			}
	    		}
	    	}
        }
    	return false;
    }
    
	public void toString(StringBuffer sb) {
		sb.append("ShieldBurstAnimation(");
		sb.append(source.getName());
		linkedAnimationsToString(sb);
		sb.append(")");
	}

	public Area getArea() throws ApplicationError {
		return new Area(new Location(x,y), 50, 50);
	}


	public BattleObjectDelegate getSource() {
		return source;
	}


}