/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Window;

import mot.client.animation.BattleAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sound;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageCache;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageAnimationConfig;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.bean.TroopBattleResult;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.MotEvent;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.Utils;

public class TroopBattleAnimation extends BattleAnimation {

	private static final String TROOP_BATTLE = "TroopBattle";
	private static final int LABEL_OFFSET_X = 40;
	private static final int LABEL_WIDTH = 30;
	private static final int HEIGHT = 20;
	
	private GraphicalAnimationContainer container;
    private ImageAnimationConfig config;
    private TroopBattleResult tbr;
    
	private Location loc;

	private Sound activeSound;
	private Sound winSound;
	private Sound loseSound;
    
	private StatusLabel attackers;
	private StatusLabel defenders;
	private Rectangle animationArea;
	
    public TroopBattleAnimation(MotEvent event, GraphicalAnimationContainer container,  TroopBattleResult tbr) throws ApplicationError {
        super(INFINITE_LIFE, true, 0, null);
        BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) event;
        
        this.config = AnimationConfigFactory.getInstance().getImageAnimationConfig(TROOP_BATTLE);
        this.container = container;
        this.tbr = tbr;
        
        if (Logger.isDebug()) {
            Logger.logDebug("TroopBattleAnimation.init() "+config.getName());
        }
        
    	CraftState state = wfe.getOldTargetState();
    	if (state == null) state = wfe.getNewTargetState();
    	loc = new Location(state.getLocation());
    	if (wfe.getTarget() != null) {
    		BattleObjectDelegate target = DeligateCacheManager.getBattleObject(wfe.getBattle(), wfe.getTarget());
    		loc.shift(0, target.getImageSize().getY()/2+5);
    	} else {
    		loc.shift(0, 20);
    	}

        winSound = Sounds.getSound("Ground_Win");
        loseSound = Sounds.getSound("Ground_Lose");
    
        attackers = new StatusLabel(L.ATTACKERS, tbr.getQtyAttackers(), loc.getX()+(LABEL_OFFSET_X), loc.getY(), LABEL_WIDTH, HEIGHT);
        defenders = new StatusLabel(L.DEFENDERS, tbr.getQtyDefenders(), loc.getX()-(LABEL_OFFSET_X+LABEL_WIDTH), loc.getY(), LABEL_WIDTH, HEIGHT);
		BattleObjectDelegate source = DeligateCacheManager.getBattleObject(wfe.getBattle(), wfe.getSource());
        if (Comp.same(container.getWindowOwner().getName(), source.getOwner())) {
        	attackers.setHitSound(loseSound);
        	defenders.setHitSound(winSound);
        } else {
        	attackers.setHitSound(winSound);
        	defenders.setHitSound(loseSound);
        }
        
        animationArea = new Rectangle(loc.getX()-LABEL_OFFSET_X, loc.getY(), 2*LABEL_OFFSET_X, HEIGHT);
    }    

    public boolean isComplete() { 
    	int qtyAttackersToLose = attackers.qty - tbr.getRemainingAttackers();
    	int qtyDefendersToLose = defenders.qty - tbr.getRemainingDefenders();
    	return (qtyDefendersToLose == 0 && qtyAttackersToLose == 0);
    }
    
    public boolean needsPainting() { return true; }
    public boolean isExclusive() {return true; }

    private String getImage() throws ApplicationError {
    	String[] stages = config.getStagesMandatoryAsSplit();
    	String index = stages[getStage()%stages.length];
        return config.getImage()+index+config.getImageType();
    }

    protected void stageChange() throws ApplicationError {
    	if (getStage() % 10 == 0) {
	    	if (isComplete()) return;
	    	int qtyAttackersToLose = attackers.qty - tbr.getRemainingAttackers();
	    	int qtyDefendersToLose = defenders.qty - tbr.getRemainingDefenders();
	    	int r = Utils.random(qtyDefendersToLose+qtyAttackersToLose);
	    	if (r < qtyDefendersToLose) {
	    		// lose a defender
	    		defenders.hit(qtyDefendersToLose/2);
	    	} else {
	    		// lose an attacker
	    		attackers.hit(qtyAttackersToLose/2);
	    	}
    	}
	}
    
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        String image = getImage();
        ImageUtils.drawImage(container.getContainer(), graphics, image, animationArea.getLeft(), animationArea.getTop(), animationArea.getWidth(), animationArea.getHeight());
        attackers.paint(graphics);
        defenders.paint(graphics);
        //Logger.logDebug("TroopBattleAnimation.paintActive() "+loc);
	}

	public static void loadAllImages(Window window) throws ApplicationError {
		ImageAnimationConfig config = AnimationConfigFactory.getInstance().getImageAnimationConfig(TROOP_BATTLE);
		int qtyImages = config.getQtyImages();
        for (int i=0; i<qtyImages; i++) {
            String imageName = config.getActualImage(i);
            Image image = ImageCache.getImage(window.getToolkit(), imageName);
            window.getGraphics().drawImage(image, 1100, 1100, 2, 2, window);
        }
    }

	public void stopSound() {
		if (activeSound != null) activeSound.stop();
	}

    public TroopBattleResult getTbr() {
		return tbr;
	}

	public void toString(StringBuffer sb) {
		sb.append("TroopBattleAnimation(");
		sb.append(""+loc);
		sb.append(")");
	}

	public Area getArea() throws ApplicationError {
		return loc.getArea(100);
	}

    class StatusLabel {

		public int qty;
		private Sound sound;
		private int x;
		private int y;
		private int width;
		private int height;
		
		private Color bkCol;
		private Font font;
		private String title;
		
		public StatusLabel(String title, int qty, int x, int y, int width , int height) throws ApplicationError {
			this.title = title;
			this.qty = qty;
			this.x = x;
			this.y = y;
			this.width = width;
			this.height = height;
			
			bkCol = PropertyManager.getMandatoryAsColor("window.invadeWindow.counter.background");
	        font = new Font(Font.SERIF, Font.BOLD, 16);	        
		}
		
		public void paint(Graphics2D graphics) {
			graphics.setColor(bkCol);
			graphics.fillRect(x,y,width,height);
			graphics.setColor(Color.WHITE);
			String qtyText = ""+qty;
	        graphics.drawString(qtyText, x+width/2-qtyText.length()*4, y+height-4);
	        graphics.drawString(title, x+width/2-title.length()*4, y+2*height-4);
		}

		private void startSound() throws ApplicationError {
			if (sound != null) {
				stopSound();
				activeSound = sound;
				sound.play();
			}
		}

		public void setHitSound(Sound sound) {
			this.sound = sound;
		}

		public void hit(int maxQtyCanKill) throws ApplicationError {
			startSound();
			qty -= (Comp.min(5, Utils.random(maxQtyCanKill)+1));
			if (qty < 0) qty = 0;
		}
    }
}
