/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Color;
import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleDroidMineDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.deligates.mot.MotBattleMineDelegate;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.AnimationConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.common.bean.CraftState;
import mot.common.event.BattleDamageRingEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.BattleMineVO;

/**
 * Causes a shock wave animation from a single point outwards until its damage is zero.
 */
public class WarpCoreOverloadAnimation extends EventAnimation  {

    private String imageName;
    private GraphicalAnimationContainer container;
    private BattleObjectDelegate source;
    private CraftState oldState;
    private int step;
    
    private int x;
    private int y;
    
    private static final String SOUND_NAME = "Explosion";

    public WarpCoreOverloadAnimation(BattleDamageRingEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        super(event, AnimationConfig.warpCoreOverloadAnimation(), true, 0, Sounds.getSound(SOUND_NAME));
        imageName = ImageConfigFactory.getImageForConfig("FireRing");
        this.container = container;
        this.source = DeligateCacheManager.getBattleObject(event.getBattle(), event.getSource());
        if (this.source == null) {
        	// May be a mine that is invisible to me - but engine should have sent mine vo in damage ring vo
        	this.source = new MotBattleMineDelegate(new BattleMineVO("FakeMine", event.getBattle(), new CraftState(event.getLocation())));
        }
        this.step = (event.getDamage()-source.getImageSize().getX()) / getLife();
        this.oldState = event.getOldState();
        this.x = source.getLocation().getX();
        this.y = source.getLocation().getY();

        if (Logger.isDebug()) {
            Logger.logDebug("WarpCoreOverloadAnimation.init() "+event.getSource()+" dam="+event.getDamage()+" @ "+source.getLocation());
        }
    }
    
    
    /**
     * Must paint Droid Mines, Suicide Craft before their explosion is animated but after they have been destroyed.
     * NB Mines are not visible.
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
		super.paintNonActive(painter, list);
		if (!(source instanceof BattleDroidMineDelegate)) {
	        if (!list.vetoDrawObject(source, this)) {
		        Graphics2D graphics = (Graphics2D) painter;
	            ImageUtils.drawBattleObject(container, graphics, source, container.getImageSelection(), oldState, null);
	        }
		}
	}

    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
    	super.paintPostProcess(painter, active, list);
        if (!active && source instanceof BattleDroidMineDelegate) {
        	if (!list.vetoDrawObject(source, this)) {
        		Graphics2D graphics = (Graphics2D) painter;
        		ImageUtils.drawBattleObject(container, graphics, source, container.getImageSelection(), oldState, null);
        		//ImageUtils.drawBattleObject(container.getContainer(), graphics, source, container.getImageSelection(), container.getWindowOwner(), null);
        	}
        }
    }



	/**
     * paints this animation on the screen.
     * This object is part of the local swing GUI interface implementation so
     * object must be a Graphics2D object.
     */
    public void paintActive(Object painter) throws ApplicationError {
        Graphics2D graphics = (Graphics2D) painter;
        
        int stage = getStage();
        Color color = new Color(0, 0, (int)(255-(stage*20)/getLife()));
        int radius = (stage * step) + source.getImageSize().getX();
        
        ImageUtils.drawRotatedImage(container.getContainer(), graphics, imageName, x-radius, y-radius, radius*2, radius*2, null, 5*getStage());
    }
    
	public void toString(StringBuffer sb) {
		sb.append("WarpCoreOverloadAnimation(");
		sb.append(source.getName());
		linkedAnimationsToString(sb);
		sb.append(")");
	}


	public BattleObjectDelegate getSource() {
		return source;
	}



}