/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import java.awt.Graphics2D;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sound;
import mot.client.sounds.Sounds;
import mot.client.swing.animation.strokes.WeaponStroke;
import mot.client.swing.common.ImageUtils;
import mot.common.bean.CraftGraphics;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.bean.WreckageDetails;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponFirePoint;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * Causes Weapon Fire to be display on the screen.
 */
public class WeaponFireAnimation extends EventAnimation {

    private GraphicalAnimationContainer container;
    private Location [] sourceLoc;
    private Location destLoc;
    private WeaponStroke stroke;
    
    private BattleObjectDelegate source;
    private BattleObjectDelegate target;
    
    public static Sound getSound(Weapon w) throws ApplicationError {
    	return Sounds.getWeaponSound(w);
    }

    public WeaponFireAnimation(BattleWeaponFireEvent event, GraphicalAnimationContainer container, WeaponStroke stroke, int life, int interval, boolean reverse) throws ApplicationError {
        super(event, life, true, interval, getSound(event.getWeaponTechnology()));
        
        this.source = DeligateCacheManager.getBattleObject(event.getBattle(), event.getSource());
        this.target = DeligateCacheManager.getBattleObject(event.getBattle(), event.getTarget());
        
        Weapon weapon = event.getWeaponTechnology();
		Location sloc = event.getSourceLocation(source);
        Location tloc = event.getTargetLocation(target);
		
		if (reverse) {
            this.destLoc = getWeaponFireLocationTo(source, sloc);
            this.sourceLoc = getDefaultWeaponFireLocationFrom(getTarget(), tloc, sloc);
        } else {
        	if (getTarget() == null) {
                this.destLoc = tloc;
        	} else {
                this.destLoc = getWeaponFireLocationTo(getTarget(), tloc);
        	}
    		int sdir = event.getDirection();
        	if (weapon == null) {
        		this.sourceLoc = getBayWeaponFireLocationFrom(source, sloc, sdir, tloc);
        	} else if (weapon.usesDish()) {
                this.sourceLoc = getDishWeaponFireLocationFrom(source, sloc, sdir, tloc);
        	} else if (weapon.isAnimatedFromCenterToCenter()) {
                this.sourceLoc = toLocationArray(sloc);
    		} else {
                this.sourceLoc = getBayWeaponFireLocationFrom(source, sloc, sdir, tloc);
    		}
       }
        this.container = container;
        this.stroke = stroke;
        
        if (Logger.isDebug()) {
        	String targetStr = (getTarget() == null) ? tloc.toString() : getTarget().toString(); 
            Logger.logDebug("WeaponFireAnimation.init() "+source+" fires "+weapon.getName()+" at "+targetStr+" DAM "+event.getDamage()+" "+sloc+"->"+tloc);
        }
    }
    
    public Location [] getSource() {
        return sourceLoc;
    }

    public void setSource(Location [] source) {
        this.sourceLoc = source;
    }

    public Location getDestination() {
        return destLoc;
    }

    public void setDestination(Location destination) {
        this.destLoc = destination;
    }
    
    public BattleObjectDelegate getAgressor() {
    	return source;
    }
    
    public BattleObjectDelegate getTarget() {
    	return target;
    }
    
    /**
     * Must paint Wreckage before salvage beam, time warp beam is animated but after they have 
     * been destroyed.
     */
    public void paintNonActive(Object painter, BattleAnimationList list) throws ApplicationError {
		super.paintNonActive(painter, list);
		BattleWeaponFireEvent event = (BattleWeaponFireEvent) getEvent();
		if (event != null && event.getData() != null && event.getData() instanceof WreckageDetails) {
			WreckageDetails wreckage = (WreckageDetails)event.getData();
	        if (!list.vetoDrawAnimatedArea(wreckage)) {
	            Graphics2D graphics = (Graphics2D) painter;
				ImageUtils.drawAnimatedBattleArea(container.getContainer(), graphics, wreckage);
	        }
		}
	}
    /**
     * Paint the fire on the screen.
     */
    public void paintPostProcess(Object painter, boolean active, BattleAnimationList list) throws ApplicationError {
        if (active) {
            Graphics2D graphics = (Graphics2D) painter;
            for (int i = 0; i < sourceLoc.length; i++) {
                stroke.paint(container.getContainer(), graphics, getStage(), sourceLoc[i], destLoc);
                //Logger.logDebug("WeaponFireAnimation.paintPostProcess() "+((WeaponFireEvent)getEvent()).getTarget().getName());				
			}
        }
    }

    /**
     * @return the location from which weapon fire animation should be drawn from this image
     * object to the target location.
     * This returns the closest edge of this image object to the location passed in.
     */
	public static Location [] getBayWeaponFireLocationFrom(ImageObjectDelegate io, Location sourceLoc, int sourceDir, Location targetLoc) throws ApplicationError {
		if (io instanceof BattleCraftDelegate) {
			BattleCraftDelegate craft = (BattleCraftDelegate) io;
	    	CraftGraphics craftGraphics = craft.getCraftGraphics();
	    	if (craftGraphics != null) {
	    		WeaponFirePoint[] weaponFireOffsets =  craftGraphics.getWeaponFireOffset();
		        if (weaponFireOffsets == null) {
			        return getDefaultWeaponFireLocationFrom(io, sourceLoc, targetLoc);
		        } else {
			        Location [] r = new Location[weaponFireOffsets.length];
			        for (int i = 0; i < r.length; i++) {
			        	r[i] = rotateLocationUsingWeaponFirePoint(sourceLoc, sourceDir, weaponFireOffsets[i]);
					}
			        return r;
		        }
	    	}
		}
		return getDefaultWeaponFireLocationFrom(io, sourceLoc, targetLoc);
    }

    /**
     * @return the location from which weapon fire animation should be drawn from this image
     * object to the target location.
     * This returns the closest edge of this image object to the location passed in.
     */
	public static Location [] getDishWeaponFireLocationFrom(ImageObjectDelegate io, Location sourceLoc, int sourceDir, Location targetLoc) throws ApplicationError {
		if (io instanceof BattleCraftDelegate) {
			BattleCraftDelegate craft = (BattleCraftDelegate) io;
	    	CraftGraphics craftGraphics = craft.getCraftGraphics();
	    	if (craftGraphics != null) {
		        WeaponFirePoint weaponFireOffset = craftGraphics.getDishOffset();
		        if (weaponFireOffset == null) {
			        return getDefaultWeaponFireLocationFrom(io, sourceLoc, targetLoc);
		        } else {
		        	Location [] r = new Location[1]; 
		        	r[0] = rotateLocationUsingWeaponFirePoint(sourceLoc, sourceDir, weaponFireOffset);
		        	return r;
		        }
	    	}
		}
		return getDefaultWeaponFireLocationFrom(io, sourceLoc, targetLoc);
    }

	private static Location [] toLocationArray(Location l) {
    	Location [] r = new Location[1]; 
    	r[0] = l;
    	return r;
	}
	
	private static Location rotateLocationUsingWeaponFirePoint(Location sourceLoc, int sourceDir, WeaponFirePoint weaponFireOffset) throws ApplicationError {
		Location r = new Location(sourceLoc);
		r.shift(weaponFireOffset.getX(), weaponFireOffset.getY());
		double rotation = ImageUtils.directonToRotation(sourceDir);
		r.rotateAround(rotation, sourceLoc.getX(), sourceLoc.getY());
		return r;
	}
	
	
    public static final float PERCENT_FIRE_FROM_EDGE = 0.25f;

    public static Location [] getDefaultWeaponFireLocationFrom(ImageObjectDelegate io, Location sourceLoc, Location targetLoc) throws ApplicationError {
        Location [] r = new Location[1];
        r[0] = (sourceLoc == null) ? null : new Location(sourceLoc);
        if (io != null) {
        	Size imageSize = io.getImageSize();
        	float radius = (imageSize.getY() + imageSize.getX()) / 2 * PERCENT_FIRE_FROM_EDGE;
        	r[0].shift(targetLoc, (int)radius);
        }
        return r;
	}

    /**
     * @return the location to which weapon fire animation should be drawn from this image
     * object to the target location.
     * This returns the closest edge of this image object to the location passed in.
     */
	public static Location getWeaponFireLocationTo(ImageObjectDelegate io, Location loc) throws ApplicationError {
		if (loc != null) {
			return loc;
		} else {
			return new Location(io.getLocation());
		}
    }

	public PlayerDelegate getAnimatedObjectOwner() throws ApplicationError {
		if (source != null) {
			return source.getOwnerDelegate();
		} else {
			return null;
		}
	}

	public void toString(StringBuffer sb) {
		sb.append("WeaponFireAnimation(");
		if (source != null) {
			sb.append(source.getName());
		} else if (sourceLoc != null){
			sb.append(sourceLoc);
		}
		sb.append("->");
		if (getTarget() != null) {
			sb.append(getTarget().getName());
		} else if (destLoc != null) {
			sb.append(destLoc);
		}
		sb.append(")");
	}



}