/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation;

import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.animation.strokes.BeamStroke;
import mot.client.swing.animation.strokes.DamageBotStroke;
import mot.client.swing.animation.strokes.ImageStroke;
import mot.client.swing.animation.strokes.PlagueStroke;
import mot.client.swing.animation.strokes.TroopTransportStroke;
import mot.client.swing.animation.strokes.WeaponStroke;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.WeaponFireAnimationConfig;
import mot.common.bean.Location;
import mot.common.bean.SpaceDistortionDetails;
import mot.common.bean.TroopBattleResult;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.BattleWeaponMissEvent;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;

public class WeaponFireAnimationFactory  {
	
    public static void create(BattleWeaponFireEvent event, GraphicalAnimationContainer container, BattleAnimationList al) throws ApplicationError {
    	EventAnimation animation = create(event, container);
        if (animation != null) {
            al.add( animation );
        }
        if (event.getData() != null) {
        	if (event.getData() instanceof TroopBattleResult) {
        		TroopBattleResult tbr = (TroopBattleResult) event.getData();
                al.add( new TroopBattleAnimation(event, container, tbr) );
        	} else if (event.getData() instanceof SpaceDistortionDetails) {
            	SpaceDistortionDetails sdd = (SpaceDistortionDetails) event.getData();
            	if (sdd != null) {
            		animation.addLinkedAnimation(new CreateSpacialDistortionAnimation(container, sdd, animation, false));
            	}
            }
        }
    }

    public static EventAnimation create(BattleWeaponFireEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        Weapon weapon = event.getWeaponTechnology();
		String name = weapon.getName();
        String group = weapon.getGroup();
        String suffix = event.getAnimationConfigNameSuffix();
        AnimationConfigFactory configs = AnimationConfigFactory.getInstance();
        WeaponFireAnimationConfig config = configs.getWeaponFireAnimationConfig(name, group, suffix);

        if (Comp.in(group, configs.getNoWeaponFireAnimation())) {
            if (Comp.same(group, Weapon.GROUP_TIME_WARP)) {
            	return new TimeWarpAnimation(config, event, container);
            }
            return null;
        }
        
        if (config == null) {
         	Logger.logError("", new ApplicationError("No config for name "+name+" group "+group+" suffix "+suffix));
        	WeaponStroke stroke = new BeamStroke();
            return new WeaponFireAnimation(event, container, stroke, 7, 0, false);
        }
        
        if (config.getLife() == 0) {
        	// Has no animation
        	return null;
        }
        
        WeaponStroke stroke = getStroke(event, config);
        
        int life = config.getLife();
        int interval = config.getInterval();        
        boolean reverse = event.isReverseAnimation();
        WeaponFireAnimation anim = new WeaponFireAnimation(event, container, stroke, life, interval, reverse);
        return anim;
    }

    public static WeaponStroke getStroke(BattleWeaponFireEvent event, WeaponFireAnimationConfig config) throws ApplicationError {
    	BattleObjectDelegate source = DeligateCacheManager.getBattleObject(event.getBattle(), event.getSource());
    	BattleObjectDelegate target = DeligateCacheManager.getBattleObject(event.getBattle(), event.getTarget());
    	return getStroke(event.getWeaponTechnology(),
    					 event.getSourceLocation(source),
    					 event.getTargetLocation(target),
    					 config);
    }

    public static WeaponStroke getStroke(Weapon weapon, Location source, Location destination, WeaponFireAnimationConfig config) throws ApplicationError {
        if (weapon == null) {
            return new BeamStroke();
        }

        WeaponStroke stroke = null;
        String group = weapon.getGroup();

	    if (Comp.same(group, Weapon.GROUP_DAMAGE_BOT)) {
	        stroke = new DamageBotStroke(config, source, destination);
	    } else if (Comp.same(group, Weapon.GROUP_PLAGUE)) {
	        stroke = new PlagueStroke(config, source, destination);
	    } else if (Comp.same(group, Weapon.GROUP_TROOP_TRANSPORT)) {
	        stroke = new TroopTransportStroke(config, source, destination);
    	} else {
    		if (config.getStrokeImage() != null) {
        		stroke = new ImageStroke(config);
			} else {
	            stroke = new BeamStroke(config);
			}
        }
        
        return stroke;
    }

    
    public static void create(BattleWeaponMissEvent event, GraphicalAnimationContainer container, BattleAnimationList al) throws ApplicationError {
        EventAnimation weaponFireAnim = create(event, container);
        if (weaponFireAnim != null) {
            al.add( weaponFireAnim );
        }
    }

    public static EventAnimation create(BattleWeaponMissEvent event, GraphicalAnimationContainer container) throws ApplicationError {
        //return new MissTargetAnimation2(event, container);
        Weapon weapon = event.getWeapon();
        if (weapon == null || weapon.isMissile()) {
            return new MissTargetAnimation2(event, container, event.getSourceLocation());
        } else {
            AnimationConfigFactory configs = AnimationConfigFactory.getInstance();
            String group = weapon.getGroup();
            if (Comp.in(group, configs.getNoWeaponFireAnimation())) {
                return null;
            }
            WeaponFireAnimationConfig config = configs.getWeaponFireAnimationConfig(weapon, null);
            if (config == null) {
             	Logger.logError("", new ApplicationError("No config for name "+weapon.getName()));
            	WeaponStroke stroke = new BeamStroke();
                return new MissTargetAnimation(event, container, stroke);
            }
            if (config.getLife() == 0) {
            	// Has no animation
            	return null;
            }
            Location sloc = event.getSourceLocation();
			Location tloc = event.getTargetLocation();
			WeaponStroke stroke = getStroke(weapon, sloc, tloc, config);
            return new MissTargetAnimation(event, container, stroke);
        }
    }


    
}