/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation.scroll;

import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Image;

import mot.client.swing.common.ImageCache;
import mot.client.swing.config.ImageScrollAnimationConfig;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * This is the base class for all scrolls that are made of images tiled from
 * the source to the origin. An image scroll shifts the images from the source 
 * to the destination cutting the start of the image for the first
 * image draw and cutting the end of the last image painted in the sequence.
 */
public class ImageScroll {

    /**
     * The name of the file to use as an image for this stroke.
     */
    private String imageName;
    
    /**
     * The width to draw this image. Note, this is not the width of the image definition
     * this is the width the image will be scaled to.
     */
    private int imageWidth;

    /**
     * The height to draw this image. Note, this is not the height of the image definition
     * this is the width the image will be scaled to.
     */
    private int imageHeight;
 
    public ImageScroll(ImageScrollAnimationConfig config) {
        this.imageName = config.getImage();
        this.imageWidth = config.getSize().getX();
        this.imageHeight = config.getSize().getY();
	}

	public void paint(Container container, Graphics2D graphics, int scroll, Location source, Location destination) throws ApplicationError {        
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [imageScroll "+imageName+" scroll "+scroll+"]");
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        double direction = source.getAccurateDirectionFor(destination);
        double angleInRadians = Math.toRadians(direction);
        
        //int srcImageWidth = image.getWidth(container);
        //int srcImageHeight = image.getHeight(container);
        
        int remainingDistance = source.getDistanceTo(destination);
        if (remainingDistance < 1) {
        	Logger.logWarning("ImageScroll.paint() with zero distance "+source+"=>"+destination+" scroll "+scroll);
        	return;
        }
                
        if (angleInRadians != 0) {
            graphics.rotate(angleInRadians, source.getX(), source.getY());
        }
        
        //Logger.logDebug("START "+source+"=>"+destination+" scroll "+scroll);
        
        int imageLeft = source.getX()-imageWidth/2;
        int imageRight = source.getX()+imageWidth/2;
        int imageBottom = source.getY();
        int imageTop = source.getY();
        
        int srcImageBottom = scroll % imageHeight;
        
        while (remainingDistance > 0) {	    
            int srcImageTop = imageHeight;
            if (remainingDistance < srcImageTop - srcImageBottom) {
            	srcImageTop = srcImageBottom + remainingDistance;
            }
            int srcImageHeight = srcImageTop - srcImageBottom;
            imageTop -= srcImageHeight;

			graphics.drawImage(image, imageLeft, imageBottom, imageRight, imageTop, 0, srcImageBottom, imageWidth, srcImageTop, container);
    		
			//Logger.logDebug(""+imageLeft+","+imageBottom+" "+imageRight+","+imageTop+" from "+0+","+srcImageBottom+" "+imageWidth+","+srcImageTop);
    		imageBottom -= srcImageHeight;
    		remainingDistance -= srcImageHeight;

    		srcImageBottom = 0;
        }
        
        if (angleInRadians != 0) {
        	graphics.rotate(-angleInRadians, source.getX(), source.getY());
        }
    }

}