/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation.strokes;

import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Image;

import mot.client.swing.common.ImageCache;
import mot.client.swing.config.ImageDuelStrokeAnimationConfig;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;

/**
 * This is the base class for all strokes that are made of images tiled from
 * the source to the origin. All images must point north as direction 0 implies 
 * no rotation and 0 is north.
 */
public class DuelImageStroke implements WeaponStroke {

    /**
     * The name of the file to use as an image for this stroke.
     */
    private String imageName1;
    private String imageName2;
    
    /**
     * The width to draw this image. Note, this is not the width of the image definition
     * this is the width the image will be scaled to.
     */
    private int imageWidth;

    /**
     * The height to draw this image. Note, this is not the height of the image definition
     * this is the width the image will be scaled to.
     */
    private int imageHeight;
    
    /**
     * This is the gap between the start of an image and the start of the next image to be 
     * drawn in the stroke.
     */
    private int gap;
    
    /**
     * This is the quantity of pixels forward that the images will be drawn each iteration 
     * of animation. 
     */
    private int step;
    
    /**
     * This is the percent along the line that the stroke changes from image1 to image2
     */
	private double joinPercent;

    protected DuelImageStroke(String imageName1, String imageName2, int imageWidth, int imageHeight, int gap, int step, double joinPercent) {
        this.imageName1 = imageName1;
        this.imageName2 = imageName2;
        this.imageWidth = imageWidth;
        this.imageHeight = imageHeight;
        this.gap = gap;
        this.step = step;
		this.joinPercent = joinPercent;
    }
    
    public DuelImageStroke(ImageDuelStrokeAnimationConfig config, double joinPercent) {
        this(config.getStrokeImage1(), config.getStrokeImage2(), config.getStrokeSize().getX(), config.getStrokeSize().getY(), config.getStrokeGap(), config.getStrokeStep(), joinPercent);
    }
    
    public void paint(Container container, Graphics2D graphics, int stage, Location source, Location destination) throws ApplicationError {
        Location loc = new Location(source);
        loc.shift(destination, stage%(imageHeight+gap)*step);
        Image image1 = ImageCache.getImage(container.getToolkit(), imageName1);
        Image image2 = ImageCache.getImage(container.getToolkit(), imageName2);
        double direction = source.getAccurateDirectionFor(destination);
        double angleInRadians = Math.toRadians(direction);
        int distanceTotal = source.getDistanceTo(destination);
        int distancesoFar = 0;

        boolean last = loc.equals(destination);
        while (!last) {
            int x = loc.getX();
            int y = loc.getY();
            loc.shift(destination, imageHeight+gap);
            last = loc.equals(destination);
            distancesoFar += imageHeight+gap;
			int imageLeft = x-imageWidth/2;
			int imageTop = y-imageHeight/2;
            if (angleInRadians != 0) {
                graphics.rotate(angleInRadians, x, y);
            }
            if (last || distancesoFar >= distanceTotal*joinPercent) {
            	graphics.drawImage(image2, imageLeft, imageTop, imageWidth, imageHeight, container);
            } else {
            	graphics.drawImage(image1, imageLeft, imageTop, imageWidth, imageHeight, container);
            }
            if (angleInRadians != 0) {
                graphics.rotate(-angleInRadians, x, y);
            }
        }
    }

    public boolean isCacheable() { return true; }
   
}