/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation.strokes;

import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Image;

import mot.client.swing.common.ImageCache;
import mot.client.swing.config.ImageStrokeAnimationConfig;
import mot.client.swing.config.WeaponFireAnimationConfig;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;

/**
 * This is the base class for all strokes that are made of images tiled from
 * the source to the origin. All images must point north as direction 0 implies 
 * no rotation and 0 is north.
 */
public class ImageStroke implements WeaponStroke {

    /**
     * The name of the file to use as an image for this stroke.
     */
    private String imageName;
    
    /**
     * The width to draw this image. Note, this is not the width of the image definition
     * this is the width the image will be scaled to.
     */
    private int imageWidth;

    /**
     * The height to draw this image. Note, this is not the height of the image definition
     * this is the width the image will be scaled to.
     */
    private int imageHeight;
    
    /**
     * This is the gap between the start of an image and the start of the next image to be 
     * drawn in the stroke.
     */
    private int gap;
    
    /**
     * This is the quantity of pixels forward that the images will be drawn each iteration 
     * of animation. 
     */
    private int step;

    protected ImageStroke(String imageName, int imageWidth, int imageHeight) {
        this.imageName = imageName;
        this.imageWidth = imageWidth;
        this.imageHeight = imageHeight;
        this.gap = (imageHeight+imageWidth)/2+1;
        this.step = gap/3;
    }
    
    protected ImageStroke(String imageName, int imageWidth, int imageHeight, int gap, int step) {
        this.imageName = imageName;
        this.imageWidth = imageWidth;
        this.imageHeight = imageHeight;
        this.gap = gap;
        this.step = step;
    }
    
    public ImageStroke(WeaponFireAnimationConfig config) {
        this(config.getStrokeImage(), config.getStrokeSize().getX(), config.getStrokeSize().getY(), config.getStrokeGap(), config.getStrokeStep());
    }
    
    public ImageStroke(ImageStrokeAnimationConfig config) {
        this(config.getStrokeImage(), config.getStrokeSize().getX(), config.getStrokeSize().getY(), config.getStrokeGap(), config.getStrokeStep());
    }
    
    public void paint(Container container, Graphics2D graphics, int stage, Location source, Location destination) throws ApplicationError {
        Location loc = new Location(source);
        
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        double direction = source.getAccurateDirectionFor(destination);
        double angleInRadians = Math.toRadians(direction);
        
        
        int d = imageHeight+gap;
        if (d <= 0) d = 1; 
        int startDist = (stage*step)%d;
        if (startDist > (gap+1)) {
            int imageLeft = loc.getX()-imageWidth/2;
            int imageTop = loc.getY()-imageHeight/2;
            int imageRight = loc.getX()+imageWidth/2;
            int imageBottom = loc.getY()+imageHeight/2;
            if (angleInRadians != 0) {
                graphics.rotate(angleInRadians, loc.getX(), loc.getY());
            }
            graphics.drawImage(image, imageLeft, imageTop, imageRight, imageBottom, 0, 0, image.getWidth(container), startDist-gap, container);
            if (angleInRadians != 0) {
                graphics.rotate(-angleInRadians, loc.getX(), loc.getY());
            }
        }
		loc.shift(destination, startDist);

        while (!loc.equals(destination)) {
            int imageLeft = loc.getX()-imageWidth/2;
            int imageTop = loc.getY()-imageHeight/2;
            if (angleInRadians != 0) {
                graphics.rotate(angleInRadians, loc.getX(), loc.getY());
            }
            graphics.drawImage(image, imageLeft, imageTop, imageWidth, imageHeight, container);
            if (angleInRadians != 0) {
                graphics.rotate(-angleInRadians, loc.getX(), loc.getY());
            }
            loc.shift(destination, d);
        }
        
    }

    public boolean isCacheable() { return true; }

   
}