/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.animation.strokes;

import java.awt.Container;
import java.awt.Graphics2D;

import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.WeaponFireAnimationConfig;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * This is a stroke that is not painted from the origin to the destination
 * like normal strokes (such as ImageStroke). It occupies a small point in
 * space and move (animated) from the source to the destination.
 */
public class MovingStroke implements WeaponStroke  {

    /**
     * The name of the file to use as an image for this stroke.
     */
    private String imageName;
    
    /**
     * The width to draw this image. Note, this is not the width of the image definition
     * this is the width the image will be scaled to.
     */
    private int imageWidth;

    /**
     * The height to draw this image. Note, this is not the height of the image definition
     * this is the width the image will be scaled to.
     */
    private int imageHeight;

    private Location loc;
    private Location source;
    private Location destination;
    private int distance;
    int direction;
    
    /**
     * The quantity of frames to use moving the weapon fire from the source to the destination
     */
    private int life;
    
    public MovingStroke(String imageName, int imageWidth, int imageHeight, int life, Location source, Location destination) {
        this.imageName = imageName;
        this.imageWidth = imageWidth;
        this.imageHeight = imageHeight;
        this.source = source;
        this.destination = destination;
        this.loc = new Location(source);
        this.distance = source.getDistanceTo(destination);
        this.direction = source.getDirectionFor(destination); 
        this.life = life;
        
        if (Logger.isDebug()) {
            Logger.logDebug("MovingStroke.init() "+imageName+" ["+source+"=>"+destination+"]");
        }
    }

    public MovingStroke(WeaponFireAnimationConfig config, Location source, Location destination) {
        this.imageName = config.getStrokeImage();
        this.imageWidth = config.getStrokeSize().getX();
        this.imageHeight = config.getStrokeSize().getY();
        this.source = source;
        this.destination = destination;
        this.loc = new Location(source);
        this.distance = source.getDistanceTo(destination);
        this.direction = source.getDirectionFor(destination); 
        this.life = config.getLife();
        
        if (Logger.isDebug()) {
            Logger.logDebug("MovingStroke.init() "+imageName+" ["+source+"=>"+destination+"]");
        }
    }

    public void paint(Container container, Graphics2D graphics, int stage, Location x1, Location x2) throws ApplicationError {
        loc.shift(destination, distance / life);
        
        int imageLeft = loc.getX()-imageWidth/2;
        int imageTop = loc.getY()-imageHeight/2;

        ImageUtils.drawRotatedImage(container, graphics, imageName, imageLeft, imageTop, imageWidth, imageHeight, loc, direction);
    }

    public boolean isCacheable() { return false; }
}
