/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.buttons;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;

import mot.client.proxy.ClientErrorHandler;
import mot.client.sounds.Sound;
import mot.client.sounds.SoundEffect;
import mot.client.sounds.Sounds;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.panels.BackgroundImagePanel;
import mot.client.swing.window.galaxy.MotGalaxyMapActions;
import mot.client.swing.window.main.MainWindow;
import mot.client.tutorials.TutorialSystem;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

/**
 * This is the base class for all buttons in the game that are constructed using
 * two images - one for normal display and one for roll over display. These are
 * not 'real' buttons, they do not extends JButton.
 */
public abstract class MotImageButton extends BackgroundImagePanel implements ActionListener, MouseListener, MotGalaxyMapActions, ClientErrorHandler {

	private String command;
	private String image;
	private ArrayList listeners;
	
	public MotImageButton(String image, int size, String command) {
		this.command = command;
		this.image = image;
		PanelUtils.setTheBloodySize(this, size, size);
		//addMouseListener(this); This is a background image panel - that is a mouse listener!
		addActionListener(this);
	}
	
    public void actionPerformed(ActionEvent e) {
        runAction();
    }

	protected void runAction() {
        if (Logger.isDebug()) {
            Logger.logDebug("MotImageButton.runAction() Action="+getClass().getName());
        }
        Sound sound = null;
		try {
			sound = Sounds.getActionSound();
			if (sound != null) {
				sound.play();
			}
		} catch (ApplicationError e) {
        	Logger.logError("Sound failed", e);
		}
        try {
        	if (TutorialSystem.isActive()) {
        		TutorialSystem.validateCommand(this);
        	}
        	doPreAction();
        	_doAction();
        	doPostAction();
        } catch (ValidationError ex) {
        	if (sound != null) sound.stop();
            MainWindow.getExestingInstance().reportValidationError(ex);
        } catch (Exception ex) {
        	MainWindow.getExestingInstance().reportApplicationError(ex);
        }
    }

	public void reportValidationError(ValidationError ex) {
		try {
			SoundEffect sound = Sounds.getActionSound();
			if (sound != null && !sound.isPlaying()) {
				sound.play();
			}
		} catch (ApplicationError e) {
			Logger.logError("Stop Sound failed", e);
		}
        MainWindow.getExestingInstance().reportValidationError(ex);
	}
	
	public void reportApplicationError(Exception ex) {
    	MainWindow.getExestingInstance().reportApplicationError(ex);
	}

    protected void doPreAction() throws ApplicationError, ValidationError { }
    protected abstract void _doAction() throws ApplicationError, ValidationError;
    protected void doPostAction() throws ApplicationError, ValidationError { }

    protected void addActionListener(ActionListener listener) {
		if (listeners == null) listeners = new ArrayList();
		listeners.add(listener);
	}

	protected void drawSizeRelativeObjects() {
	}

	protected String getImage() {
		return image;
	}

	long timeLastClicked = 0;
	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
    	if (e.getClickCount() == 1) {
    		if (e.getButton() != MouseEvent.BUTTON1) {
    			return;
    		}
    		long timeNow = Calendar.getInstance().getTimeInMillis();
    		if (timeNow < timeLastClicked +500) {
    			// We ignore double clicks!
    			return;
    		}
    		timeLastClicked = timeNow;
    		Iterator i = listeners.iterator();
    		ActionEvent ae = new ActionEvent(this, -1, command);
    		while (i.hasNext()) {
    			ActionListener al = (ActionListener) i.next();
    			al.actionPerformed(ae);
    		}
    	}
	}

	public void mouseReleased(MouseEvent e) {
	}
	
}
