/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.combos;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;

import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.renderers.CellRendererImageConfig;
import mot.client.swing.window.etc.NewGameWindow;
import mot.common.bean.GameConstants;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;

public class LevelCombo extends JComboBox implements ItemListener {

	private JLabel descriptionLabel;
	private boolean includeTutorial;
	private DescriptionProvider desriptionProvider;
	private NewGameWindow parent;

    public LevelCombo(NewGameWindow parent, JLabel descriptionLabel, Size size, boolean includeTutorial, DescriptionProvider desriptionProvider) throws ApplicationError {
		this.descriptionLabel = descriptionLabel;
		this.includeTutorial = includeTutorial;
		this.desriptionProvider = desriptionProvider;
		this.parent = parent;
    	addItemListener(this);
    	
    	setRenderer(new CellRendererImageConfig(size));
        ImageConfigFactory imageConfig = ImageConfigFactory.getInstance();
        if (includeTutorial) {
        	addItem(imageConfig.getLevelIconConfig().getImageConfig(L.LEVEL_0));
        }
		addItem(imageConfig.getLevelIconConfig().getImageConfig(L.LEVEL_1));
		addItem(imageConfig.getLevelIconConfig().getImageConfig(L.LEVEL_2));
		addItem(imageConfig.getLevelIconConfig().getImageConfig(L.LEVEL_3));
		addItem(imageConfig.getLevelIconConfig().getImageConfig(L.LEVEL_4));
        setSelectedIndex(0);
    	if (descriptionLabel != null) {
    		descriptionLabel.setText(getDescription());
    	}
    }

    public ImageConfig getLevelImageConfig() {
    	ImageConfig config = (ImageConfig) getSelectedItem();
    	return config;
    }
    
    public Double getLevel() {
    	ImageConfig config = (ImageConfig) getSelectedItem();
        Double levelDouble = null;
        if (Comp.same(config.getName(), L.LEVEL_0)) {
            levelDouble = new Double(GameConstants.LEVEL_EASY_BONUS);
        } else if (Comp.same(config.getName(), L.LEVEL_1)) {
            levelDouble = new Double(GameConstants.LEVEL_EASY_BONUS);
        } else if (Comp.same(config.getName(), L.LEVEL_2)) {
        	levelDouble = new Double(GameConstants.LEVEL_NORMAL_BONUS);
        } else if (Comp.same(config.getName(), L.LEVEL_3)) {
            levelDouble = new Double(GameConstants.LEVEL_HARD_BONUS);
        } else if (Comp.same(config.getName(), L.LEVEL_4)) {
            levelDouble = new Double(GameConstants.LEVEL_REAL_HARD_BONUS);
        } else {
        	levelDouble = new Double(1.0);
        }
        return levelDouble;
    }
    
    public void setLevel(Double level, boolean hasTutorial) {
    	if (level == null) {
    		setSelectedIndex(-1);
    	} else if (level > 0.25 && level < 0.75) {
    		if (hasTutorial) {
    			setSelectedIndex(0);
    		} else {
    			setSelectedIndex(1);
    		}
    	} else if (level > 0.75 && level < 1.75) {
    		setSelectedIndex(2);    		
    	} else if (level > 1.75 && level < 2.75) {
    		setSelectedIndex(2);    		
    	} else {
    		setSelectedIndex(3);    		
    	}
    }
    
    public boolean hasTutorial() {
    	ImageConfig config = (ImageConfig) getSelectedItem();
        Double levelDouble = null;
        if (Comp.same(config.getName(), L.LEVEL_0)) {
            return true;
        } else {
        	return false;
        }
    }
    
    public String getDescription() {
    	ImageConfig config = (ImageConfig) getSelectedItem();
    	if (desriptionProvider == null) {
    		return config.getDescription();
    	} else {
    		return desriptionProvider.getDescription();
    	}
    }

    public void itemStateChanged(ItemEvent e) {
    	if (e.getStateChange() == ItemEvent.SELECTED) {
	    	if (descriptionLabel != null) {
	        	descriptionLabel.setText(getDescription());
	        	setDependentState();
	    	}
    	}
    }
    
    public void setDependentState() {
    	Double level = getLevel();
        if (level != null && parent != null) {
        	if (level <= 0.6) {
        		parent.setIsWormHoleEnabled(false);
        		parent.setIsSpaceKaanEnabled(false);
        	} else if (level <= 1.1) {
        		parent.setIsWormHoleEnabled(false);
        		parent.setIsSpaceKaanEnabled(true);
        	} else if (level <= 2.1) {
        		parent.setIsWormHoleEnabled(true);
        		parent.setIsSpaceKaanEnabled(true);
        	} else if (level >= 3.9) {
        		parent.setIsWormHoleEnabled(true);
        		parent.setIsSpaceKaanEnabled(true);
        	} else {
        		parent.setIsWormHoleEnabled(false);
        		parent.setIsSpaceKaanEnabled(true);
        	}
        }

    }
}