/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.combos;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Iterator;

import javax.swing.JComboBox;
import javax.swing.JLabel;

import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.renderers.CellRendererImageConfig;
import mot.common.bean.PlayerSummary;
import mot.common.bean.RaceGraphics;
import mot.common.bean.RaceGraphicsList;
import mot.common.bean.Size;
import mot.common.bean.UserPlayerSummary;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;

public class RaceCombo extends JComboBox implements ItemListener {

	RaceGraphicsList graphicsList;
	private JLabel descriptionLabel;
	private int qtyItemsInList;
	private DescriptionProvider desriptionProvider;
	private ImageConfig randomRaceConfig;
	
    public RaceCombo(RaceGraphicsList graphicsList, JLabel descriptionLabel, boolean includeRandom, Size size, boolean displayLabel, boolean autoSelectItem, DescriptionProvider desriptionProvider) throws ApplicationError {
    	this.graphicsList = graphicsList;
		this.descriptionLabel = descriptionLabel;
		this.desriptionProvider = desriptionProvider;
    	this.randomRaceConfig = ImageConfigFactory.getImageConfigNoError("RandomPlayer");
    	
		qtyItemsInList = 0;
		setRenderer(new CellRendererImageConfig(size, displayLabel));
        Iterator it = graphicsList.getGraphics().iterator();
        while (it.hasNext()) {
            RaceGraphics race = (RaceGraphics) it.next();
            if (Comp.same(race.getType(), RaceGraphics.TYPE_PLAYER)) {
                String name = race.getName();
                ImageConfig config = ImageConfigFactory.getInstance().getRaceImageConfig(name);
   				addItem(config);
   				qtyItemsInList++;
            }
        }
        if (includeRandom) {
        	if (randomRaceConfig != null) {
        		addItem(randomRaceConfig);
        		qtyItemsInList++;
        		if (autoSelectItem) {
        			setSelectedIndex(qtyItemsInList-1);
        		} else {
        			setSelectedIndex(-1);
        		}
        	}
	    } else {
	    	if (autoSelectItem) {
	    		setSelectedIndex(0);
			} else {
				setSelectedIndex(-1);
			}
	    }
        
        if (descriptionLabel != null) {
	    	descriptionLabel.setText(getDescription());
	    	addItemListener(this);
        }
    }


    public String getRace() {
    	ImageConfig config = (ImageConfig) getSelectedItem();
    	if (config == null) {
    		return null;
    	} else if (config == randomRaceConfig) {
    		return null;
    	} else {    		
    		return config.getName();
    	}
    }
    
    public void setRace(String raceName) {
    	int qty = getItemCount();
    	for (int i = 0; i < qty; i++) {
    		ImageConfig config = (ImageConfig) getItemAt(i);
    		if (Comp.same(raceName, config.getName())) {
    			setSelectedIndex(i);
    			return;
    		}
		}
		setSelectedIndex(-1);
    }

    public String getDescription() {
    	if (desriptionProvider == null) {
	    	Object selection = getSelectedItem();
	    	if (selection == null || !(selection instanceof ImageConfig)) {
	    		return null;
	    	}
	    	String race = getRace();
	    	if (race != null) {
		        RaceGraphics rg = graphicsList.getGraphicsFor(race);
		        if (rg != null) {
		            return rg.getDescription();
		        }
	    	}
	    	return null;
    	} else {
    		return desriptionProvider.getDescription();
    	}    		
    }

    public void itemStateChanged(ItemEvent e) {
    	if (e.getStateChange() == ItemEvent.SELECTED) {
	    	updateDescription();
    	}
    }

	public void updateDescription() {
		if (descriptionLabel != null) {
			descriptionLabel.setText("<html><body>"+getDescription()+"</body></html>");
		}
	}

	public void selectRandomPlayer() {
		setSelectedIndex(qtyItemsInList-1);
	}
	
	public boolean hasSelectedRandomPlayer() {
    	int i = getSelectedIndex();
    	Object selection = getSelectedItem();
    	return (i == qtyItemsInList-1);
	}

	public void updateContent(UserPlayerSummary[] upsa) throws ApplicationError {
		removeAllItems();
		qtyItemsInList = 0;
		if (upsa != null) {				
			for (int i = 0; i < upsa.length; i++) {
				String name = upsa[i].getRaceName();
                ImageConfig config = ImageConfigFactory.getInstance().getRaceImageConfig(name);
   				addItem(config);
   				qtyItemsInList++;
			}
		}
	}
	
	public void updateContent(PlayerSummary[] playerDetails) throws ApplicationError {
		removeAllItems();
		qtyItemsInList = 0;
		if (playerDetails == null) return;
		for (int i = 0; i < playerDetails.length; i++) {
			if (playerDetails[i].isTypeHuman()) {				
				RaceGraphics race = graphicsList.getGraphicsFor(playerDetails[i].getName());
				String name = race.getName();
				ImageConfig config = ImageConfigFactory.getInstance().getRaceImageConfig(name);
				addItem(config);
				qtyItemsInList++;
			}
		}
	}

}