/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.common;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.common.util.ApplicationError;

/**
 * Creates components in a table format.
 */
public class ComponentTable extends JPanel
{
    /**
     * An array of strings defining the layout of each column.
     * Each entry in the array defines the constraints for a single column and the
     * size of the array defines the quantity of columns.
     */
    private String [] colConstraints;
    
    /**
     * An array of strings defining the width of each column.
     */
    private double [] colWidths;

    /**
     * An array of strings defining the height of each row.
     */
    private double [] rowHeights;

    private int qtyRows;
    private int qtyCols;
    
    private int margin;
    private int cellHGap;
    private int cellVGap;

    /**
     * The column count. When this reaches the quantity of columns the next component
     * will automatically be added at the start of the next row.
     */
    private int colCount = 0;

    /**
     * The row count.
     */
    private int rowCount = 0;

    /**
     * The constraints used to layout this table.
     */
    private GridBagLayout gridBag;
    
    /**
     * Use all remaining columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_END_ROW = 'F';

    /**
     * Use all remaining rows in this column. Default is use a single row.
     */
    public static final char CONSTRAINT_END_COL = 'E';

    /**
     * Use 2 columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_COLSPAN2 = '2';
    
    /**
     * Use 3 columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_COLSPAN3 = '3';
    
    /**
     * Use 4 columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_COLSPAN4 = '4';

    /**
     * Use all columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_ALLCOLS = 'x';
    
    /**
     * Use 2 columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_ROWSPAN2 = 'a';
    
    /**
     * Use 3 columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_ROWSPAN3 = 'b';
    
    /**
     * Use 4 columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_ROWSPAN4 = 'c';
    
    /**
     * Use all columns in this row. Default is use a single column.
     */
    public static final char CONSTRAINT_ALLROWS = 'y';
    
    /**
     * Horizontally align this cell to the left. The default is centre.
     */
    public static final char CONSTRAINT_LEFT = 'L';

    /**
     * Horizontally align this cell to the right. The default is centre.
     */
    public static final char CONSTRAINT_RIGHT = 'R';

    /**
     * Vertically align this cell to the top. The default is centre.
     */
    public static final char CONSTRAINT_CENTERED = 'C';

    /**
     * Vertically align this cell to the top. The default is centre.
     */
    public static final char CONSTRAINT_TOP = 'T';

    /**
     * Vertically align this cell to the bottom. The default is centre.
     */
    public static final char CONSTRAINT_BOTTOM = 'B';

    /**
     * Vertically align this cell to the left and right. The default is centre.
     */
    public static final char CONSTRAINT_FILL_HORIZONTAL = 'H';

    /**
     * Vertically align this cell to the top and bottom. The default is centre.
     */
    public static final char CONSTRAINT_FILL_VERTICAL = 'V';

    /**
     * Allocate spare vertically space to this cell. The default is to allocate 
     * spare space to the bottom cell.
     */
    public static final char CONSTRAINT_USE_VERTICAL_SPACE = 'S';
    
    /**
     * Allocate spare horizontal space to this cell. The default is to allocate 
     * spare space to the right most cell.
     */
    public static final char CONSTRAINT_USE_HORIZONTAL_SPACE = 'P';

    // I am not even sure what these mean!    
    public static final char NEXT_COL_IS_SKIP = '%';
    public static final char NEXT_ROW_IS_SKIP = '!';
    
    public ComponentTable(String colConstraints, int qtyRows, int qtyCols, int margin, int cellHGap, int cellVGap) {
        this(colConstraints.split(","), qtyRows, qtyCols, margin, cellHGap, cellVGap);
    }
    
    public ComponentTable(String [] colConstraints, int qtyRows, int qtyCols, int margin, int cellHGap, int cellVGap) {
        super();
        this.gridBag = new GridBagLayout();
        this.colConstraints = colConstraints;
        this.qtyRows = qtyRows;
        this.qtyCols = qtyCols;
        this.margin = margin;
        this.cellHGap = cellHGap;
        this.cellVGap = cellVGap;
        
        setLayout(gridBag);
    }
    
    public void removeAll() {
    	super.removeAll();
    	colCount = 0;
    	rowCount = 0;
    }
    
    
    public void setGridWidths(String colWidths) {
        if (colWidths != null) {
            setGridWidths(colWidths.split(","));
        }
    }

    public void setGridWidths(String [] colWidths) {
        if (colWidths == null) return;
        double [] r = new double[colWidths.length];
        for (int i=0; i<colWidths.length; i++) {
            try {
                r[i] = Double.parseDouble(colWidths[i]);
            } catch (NumberFormatException ex) {
            }
        }
        setGridWidths(r);
    }

    public void setGridHeights(String rowHeights) {
        if (rowHeights != null) {
            setGridHeights(rowHeights.split(","));
        }
    }

    public void setGridHeights(String [] rowHeights) {
        if (rowHeights == null) return;
        double [] r = new double[rowHeights.length];
        for (int i=0; i<rowHeights.length; i++) {
            try {
                r[i] = Double.parseDouble(rowHeights[i]);
            } catch (NumberFormatException ex) {
            }
        }
        setGridHeights(r);
    }

    public void setGridHeights(double [] rowHeights) {
        this.rowHeights = rowHeights;
    }
    
    public void setGridWidths(double [] colWidths) {
        this.colWidths = colWidths;
    }
    
    public JPanel getWithBorder(String panelName) throws ApplicationError { 
        if (panelName != null) {
            new MotBorder(this, panelName);
        }
        return this; 
    }

    private boolean hasConstraint(String constraint, char v) {
        return constraint != null && constraint.indexOf(v) > -1;
    }
    

    private GridBagConstraints gridBagConstraints = new GridBagConstraints();
    
    private GridBagConstraints getConstraints(String constraint) {
        // Set the spaces around this cell
        int left = (colCount == 0) ? margin : cellHGap;
        int right = (colCount == qtyCols-1) ? margin : cellHGap;
        int top = (rowCount == 0) ? margin : cellVGap;
        int bottom = (rowCount == qtyRows-1) ? margin : cellVGap;
        gridBagConstraints.insets = new Insets(top,left,bottom,right);
        
        int qtyColsUsed = 1;        
        if (hasConstraint(constraint, CONSTRAINT_COLSPAN2)) {
        	qtyColsUsed = 2;
        }
        if (hasConstraint(constraint, CONSTRAINT_COLSPAN3)) {
        	qtyColsUsed = 3;
        }
        if (hasConstraint(constraint, CONSTRAINT_COLSPAN4)) {
        	qtyColsUsed = 4;
        }

        int qtyRowsUsed = 1;
        if (hasConstraint(constraint, CONSTRAINT_ROWSPAN2)) {
        	qtyRowsUsed = 2;
        }
        if (hasConstraint(constraint, CONSTRAINT_ROWSPAN3)) {
        	qtyRowsUsed = 3;
        }
        if (hasConstraint(constraint, CONSTRAINT_ROWSPAN4)) {
        	qtyRowsUsed = 4;
        }
        
        // set the cell parameters        
        boolean rowEnd = colCount == (qtyCols-qtyColsUsed) || hasConstraint(constraint, CONSTRAINT_END_ROW);
        boolean colEnd = rowCount == (qtyRows-qtyRowsUsed) || hasConstraint(constraint, CONSTRAINT_END_COL);
        if  (rowEnd) {
            gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
            gridBagConstraints.weightx = 1;
        } else {
            gridBagConstraints.gridwidth = qtyColsUsed;
            gridBagConstraints.weightx = 0;
        }
        if (colEnd) {
            gridBagConstraints.gridheight = GridBagConstraints.REMAINDER;
            gridBagConstraints.weighty = 1;
        } else {
            gridBagConstraints.gridheight = qtyRowsUsed;
            gridBagConstraints.weighty = 0;
        }
        
        if (colWidths != null) {
            gridBagConstraints.weightx = colWidths[colCount];
        }
        if (rowHeights != null) {
            gridBagConstraints.weighty = rowHeights[rowCount];
        }
        
        if (hasConstraint(constraint, CONSTRAINT_USE_VERTICAL_SPACE)) {
            gridBagConstraints.weighty = 100000;
        }
        if (hasConstraint(constraint, CONSTRAINT_USE_HORIZONTAL_SPACE)) {
            gridBagConstraints.weightx = 100000;
        }
        if (hasConstraint(constraint, CONSTRAINT_ALLCOLS)) {
            gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
        }
        if (hasConstraint(constraint, CONSTRAINT_ALLROWS)) {
            gridBagConstraints.gridheight = GridBagConstraints.REMAINDER;
        }
        if (hasConstraint(constraint, NEXT_COL_IS_SKIP)) {
            gridBagConstraints.gridwidth = GridBagConstraints.RELATIVE;
        }
        if (hasConstraint(constraint, NEXT_ROW_IS_SKIP)) {
            gridBagConstraints.gridheight = GridBagConstraints.RELATIVE;
        }
        
        
        // set the alignment in this cell
        if (hasConstraint(constraint, CONSTRAINT_LEFT)) {
            if (hasConstraint(constraint, CONSTRAINT_TOP)) {
                gridBagConstraints.anchor = GridBagConstraints.NORTHEAST;
            } else if (hasConstraint(constraint, CONSTRAINT_BOTTOM)) {
                gridBagConstraints.anchor = GridBagConstraints.SOUTHEAST;
            } else {
                gridBagConstraints.anchor = GridBagConstraints.EAST;
            }
        } else if (hasConstraint(constraint, CONSTRAINT_RIGHT)) {
            if (hasConstraint(constraint, CONSTRAINT_TOP)) {
                gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
            } else if (hasConstraint(constraint, CONSTRAINT_BOTTOM)) {
                gridBagConstraints.anchor = GridBagConstraints.SOUTHWEST;
            } else {
                gridBagConstraints.anchor = GridBagConstraints.WEST;
            }
        } else if (hasConstraint(constraint, CONSTRAINT_CENTERED)) {
        	if (hasConstraint(constraint, CONSTRAINT_LEFT)) {
        		gridBagConstraints.anchor = GridBagConstraints.EAST;
        	} else if (hasConstraint(constraint, CONSTRAINT_RIGHT)) {
        		gridBagConstraints.anchor = GridBagConstraints.WEST;
        	} else {
        		gridBagConstraints.anchor = GridBagConstraints.CENTER;
        	}
        } else {
            if (hasConstraint(constraint, CONSTRAINT_TOP)) {
                gridBagConstraints.anchor = GridBagConstraints.NORTH;
            } else if (hasConstraint(constraint, CONSTRAINT_BOTTOM)) {
                gridBagConstraints.anchor = GridBagConstraints.SOUTH;
            } else {
                gridBagConstraints.anchor = GridBagConstraints.CENTER;
            }
        }
        
        if (hasConstraint(constraint, CONSTRAINT_FILL_HORIZONTAL)) {
            if (hasConstraint(constraint, CONSTRAINT_FILL_VERTICAL)) {
                gridBagConstraints.fill = GridBagConstraints.BOTH;
            } else {
                gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
            }
        } else {
            if (hasConstraint(constraint, CONSTRAINT_FILL_VERTICAL)) {
                gridBagConstraints.fill = GridBagConstraints.VERTICAL ;
            } else {
                gridBagConstraints.fill = GridBagConstraints.NONE;
            }
        }

        if  (rowEnd) {
            colCount = 0;
            rowCount++;
        } else {
            colCount += qtyColsUsed;
        }

        if (hasConstraint(constraint, CONSTRAINT_ALLCOLS)) {
            colCount = 0;
            rowCount += qtyRowsUsed;
        }
        
        return gridBagConstraints;
    }

//    private void checkColAndRowEnd() {
//        boolean rowEnd = colCount == (qtyCols-1);
//        boolean colEnd = rowCount == (qtyRows-1);
//        if  (rowEnd) {
//            gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
//            gridBagConstraints.weightx = 1;
//        } else {
//            gridBagConstraints.gridwidth = 1;
//            gridBagConstraints.weightx = 0;
//        }
//        if (colEnd) {
//            gridBagConstraints.gridheight = GridBagConstraints.REMAINDER;
//            gridBagConstraints.weighty = 1;
//        } else {
//            gridBagConstraints.gridheight = 1;
//            gridBagConstraints.weighty = 0;
//        }
//    }
    
    public void add(JComponent c) {
        add(c, null);
    }

    public JLabel add(String txt) {
    	if (textSize != null) {
    		JLabel label = new MotLabel(txt, textSize, false);
    		add(label, null);
    		return label;
    	} else {
    		JLabel label = PanelUtils.createLabel(txt);
    		add(label, null);
    		return label;
    	}
    }

    public void add(String txt, String constraints) {
        add(PanelUtils.createLabel(txt), constraints);
    }

    public void add(JComponent c, String constraints) {
        if (constraints == null) {
            gridBag.setConstraints(c, getConstraints(colConstraints[colCount]));
        } else {
            gridBag.setConstraints(c, getConstraints(constraints));
        }
        super.add( c );
    }
    
    public void addWithExtraConstraints(String txt, String constraints) {
    	addWithExtraConstraints(PanelUtils.createLabel(txt), constraints);
    }
    
    public void addWithExtraConstraints(JComponent c, String constraints) {
        if (constraints == null) {
            gridBag.setConstraints(c, getConstraints(colConstraints[colCount]));
        } else {
            gridBag.setConstraints(c, getConstraints(colConstraints[colCount]+constraints));
        }
        super.add( c );
    }
    
    public void skip() {
        boolean rowEnd = colCount == (qtyCols-1);
        if  (rowEnd) {
            colCount = 0;
            rowCount++;
        } else {
            colCount++;
        }
    }

    public void clean() {
        colCount = 0;
        rowCount = 0;
        removeAll();
    }


    public void setCellHGap(int cellHGap) {
        this.cellHGap = cellHGap;
    }


    public int getCellHGap() {
        return cellHGap;
    }


    public void setCellVGap(int cellVGap) {
        this.cellVGap = cellVGap;
    }


    public int getCellVGap() {
        return cellVGap;
    }

    // this is an infinate loop
    //public void doLayout() {
    //    validate();
    //}
    
    private boolean logConstraints = false;

    public void logConstraints() { logConstraints = true; }

	public int getQtyCols() {
		return qtyCols;
	}

	public void setQtyCols(int qtyCols) {
		this.qtyCols = qtyCols;
	}

	public int getQtyRows() {
		return qtyRows;
	}

	public void setQtyRows(int qtyRows) {
		this.qtyRows = qtyRows;
	}
	
	public void addRow() { 
		this.qtyRows++; 
	}
	
	public void reset() {
		colCount = 0;
		rowCount = 0;
	}

	private Integer textSize;

	public void setTextSize(Integer textSize) {
		this.textSize = textSize;
	}
	
	private JScrollPane scrollPane;

	public JScrollPane getScrollPane() {
        if (scrollPane == null) {
            scrollPane = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        }
        return scrollPane;
    }

    public JScrollPane getScrollPane(int padding) {
        if (scrollPane == null) {
            scrollPane = new JScrollPane(new PadPanel(this, padding), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        }
        return scrollPane;
    }

    public JScrollPane getScrollPaneNoBorder() {
        if (scrollPane == null) {
            scrollPane = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            scrollPane.setBorder(null);
        }
        return scrollPane;
    }

    public JScrollPane getScrollPaneNoBorder(int padding) {
        if (scrollPane == null) {
            scrollPane = new JScrollPane(new PadPanel(this, padding), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            scrollPane.setBorder(null);
        }
        return scrollPane;
    }

}