/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.common;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Container;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Stroke;

import mot.client.animation.GraphicalAnimationContainer;
import mot.client.animation.MovementRegister;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.BattlePlanetDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.animation.AnimationSettings;
import mot.client.swing.config.ImageAnimationConfig;
import mot.client.swing.config.SelectionAnimation;
import mot.client.telemetry.Telemetry;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.Area;
import mot.common.bean.CraftState;
import mot.common.bean.Decorator;
import mot.common.bean.DecoratorList;
import mot.common.bean.ImageObject;
import mot.common.bean.ImageObjectSettings;
import mot.common.bean.Location;
import mot.common.bean.PlayerColor;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;

/**
 * This class provides methods to draw the images of objects on the screen.
 */
public class ImageUtils  {

    private static final int DECORATOR_SIZE = 8;

	public static void drawBattlePlanet(GraphicalAnimationContainer container, Graphics2D graphics, BattlePlanetDelegate bp, ImageObjectDelegate selected, boolean isNext) throws ApplicationError {
        Location loc = bp.getLocation();
        double damagePercent = bp.getState().getDamagePercent();
        double shieldPercent = bp.getState().getShieldPercent();
        drawBattleObject(container, graphics, bp, selected, damagePercent, shieldPercent, loc, NO_ROTATION_DIRECTION, -1, -1, 1.0, null, null, bp.getState().getDecorators(), null, isNext);
    }

//  public static void drawBattleCraft(Container container, Graphics2D graphics, BattleCraftDeligate bc, ImageObjectDeligate selected, PlayerDeligate windowOwner, CraftState state) throws ApplicationError {
//  Location loc = bc.getLocation();
//  double shieldPercent = bc.getShieldAsPercent();
//  double damagePercent = bc.getDamageAsPercent();
//  double moveDamagePercent = bc.getMovementDamageAsPercent(); 
//  double psiDamagePercent = bc.getPsiDamageAsPercent();
//  double crewPercent = bc.getCrewAsPercent();
//  int direction = bc.getDirection();
//  drawBattleObject(container, graphics, bc, selected, windowOwner, damagePercent, shieldPercent, loc, direction, moveDamagePercent, psiDamagePercent, crewPercent, null, null, bc.getVisibleDecorators(), null);
//}
	
    public static void drawBattleCraft(GraphicalAnimationContainer container, Graphics2D graphics, BattleCraftDelegate bc, ImageObjectDelegate selected, CraftState state, boolean isNext) throws ApplicationError {
        Location loc = state.getLocation();
        int direction = state.getDirection();
        drawBattleCraft(container, graphics, bc, selected, loc, direction, state, isNext);
    }
    
    public static void drawBattleCraft(GraphicalAnimationContainer container, Graphics2D graphics, BattleCraftDelegate bc, ImageObjectDelegate selected, Location loc, int direction, CraftState state, boolean isNext) throws ApplicationError {
        double shieldPercent = state.getShieldPercent();
        double damagePercent = state.getDamagePercent();
        double moveDamagePercent = state.getMovementDamagePercent(); 
        double psiDamagePercent = state.getPsiDamagePercent();
        double crewPercent = state.getCrewPercent();
        drawBattleObject(container, graphics, bc, selected, damagePercent, shieldPercent, loc, direction, moveDamagePercent, psiDamagePercent, crewPercent, state.getPlayerColor(), state.getCloakedToPlayerNames(), state.getDecorators(), null, isNext);
    }
    
    public static void drawBattleObject(GraphicalAnimationContainer container, Graphics2D graphics, BattleObjectDelegate bo, ImageObjectDelegate selected, CraftState state, String shieldActiveImage) throws ApplicationError {
    	if (state == null) {
            drawBattleObject(container, graphics, bo, selected, bo.getState().getDamagePercent(), bo.getState().getShieldPercent(), bo.getLocation(), bo.getDirection(), bo.getState().getMovementDamagePercent(), bo.getState().getPsiDamagePercent(), bo.getState().getCrewPercent(), null, null, bo.getState().getDecorators(), shieldActiveImage, false);
    		//drawBattleObject(container, graphics, bo, selected, windowOwner, shieldActiveImage);
    	} else {
    		drawBattleObject(container, graphics, bo, selected, state.getDamagePercent(), state.getShieldPercent(), state.getLocation(), state.getDirection(), state.getMovementDamagePercent(), state.getPsiDamagePercent(), state.getCrewPercent(), state.getPlayerColor(), state.getCloakedToPlayerNames(), state.getDecorators(), shieldActiveImage, false);
    	}
    }
    
//    public static void drawBattleObject(Container container, Graphics2D graphics, BattleObjectDeligate bo, ImageObjectDeligate selected, PlayerDeligate windowOwner, String shieldActiveImage) throws ApplicationError {
//        drawBattleObject(container, graphics, bo, selected, windowOwner, bo.getDamageAsPercent(), bo.getShieldAsPercent(), bo.getLocation(), bo.getDirection(), bo.getMovementDamageAsPercent(), bo.getPsiDamageAsPercent(), bo.getCrewAsPercent(), null, null, bo.getVisibleDecorators(), shieldActiveImage);
//    }
    
    private static final Stroke PLAIN_STROKE = new BasicStroke(1.0f);
    
    public static void drawBattleObject(GraphicalAnimationContainer container, Graphics2D graphics, BattleObjectDelegate bo, ImageObjectDelegate selected, double damagePercent, double shieldPercent, Location loc, int direction, double moveDamagePercent, double psiDamagePercent, double crewPercent, PlayerColor pc, String [] cloakedToPlayerNames, DecoratorList decorators, String shieldActiveImage, boolean isNext) throws ApplicationError {
    	Size imageSize = bo.getImageSize();
    	if (imageSize == null) {
    		Logger.logWarning("ImageUtils.drawBattleObject() No image size for "+bo.getName());
    		bo.getImageSize();
    		return;
    	}

//    	if (Logger.isDebug()) {
//    		if (bo instanceof BattleCraftDelegate) {
//    			CraftDelegate craft = ((BattleCraftDelegate)bo).getCraft();
//				if (craft != null) {
//					DeligateDebugState debugState = craft.getDebugState();
//					if (debugState != null) {
//						debugState.notifyDrawBattleObject(loc);
//					}
//				}
//    		}
//    	}
    	
        int width = imageSize.getX();
        int height = imageSize.getY();
        int imageLeft = loc.getX()-width/2;
        int imageTop = loc.getY()-height/2;
        int imageRight = imageLeft+width;
        int imageBottom = imageTop+height;
        
        Container _container = container.getContainer();
        UserPlayerDelegate windowOwner = container.getWindowOwner();
        
//		if (bo.getName().startsWith("Military58_")) {
//			Logger.logDebug("");
//		}
        
        String imageName;
        if (pc != null || cloakedToPlayerNames != null) {
        	Boolean isVisible = null;
        	if (cloakedToPlayerNames != null) {
        		if (Comp.same(windowOwner.getName(), bo.getOwner())) {
        			PlayerDelegate p = container.getOwnerOfActiveAnimation();
        			if (p != null && !Comp.same(windowOwner, p)) {
        				// display player's crafts as see by the owner of the active animation
            			isVisible = !Comp.in(p.getName(), cloakedToPlayerNames);
        			} else {
        				// player can always see his own craft
        				isVisible = true;
        			}
        		} else {
        			// player can see enemies craft if player is not in cloakedTo player names list (sent from the engine).
        			isVisible = !Comp.in(windowOwner.getName(), cloakedToPlayerNames);
        		}
        	}
        	imageName = bo.getImage(pc, isVisible);    
    	} else {
    		imageName = bo.getImage();
    	}

        
        direction += bo.getAnimationRotation();
        drawRotatedImage(_container, graphics, imageName, imageLeft, imageTop, width, height, loc, direction);
        
        graphics.setStroke(getBarStroke());
        if (shieldPercent >= 0.01) {
        	String shieldImage = MovementRegister.getShieldImage(shieldPercent);
            drawImage(_container, graphics, shieldImage, imageLeft-4, imageTop-4, width+8, height+8);
        }

        if (shieldActiveImage != null) {
            drawImage(_container, graphics, shieldActiveImage, imageLeft-5, imageTop-5, width+10, height+10);
        }
        
        if (decorators != null) {
        	Decorator[] da = decorators.getDecorators();
        	if (da != null) {        		
        		int left = imageLeft;
        		int decoratorY = imageTop-3;
        		for (int i = 0; i < da.length; i++) {
        			if (da[i].visibleToAll() || Comp.same(windowOwner.getName(),bo.getOwner())) {
        				ImageAnimationConfig imageConfig = da[i].getImageConfig();
        				drawImageCenteredAt(_container, graphics, imageConfig, left, decoratorY, DECORATOR_SIZE, DECORATOR_SIZE);
        				int qty = da[i].getQty();
						if (qty != 0) {
							int textWidth = drawLabel(_container, graphics, ""+qty, left+DECORATOR_SIZE/2, decoratorY+3, Color.WHITE, false, 6);
        					left += textWidth;
        				}
        				left += DECORATOR_SIZE;
        			}
        		}
        	}
        }
        
        if (damagePercent >= 0.001) {
            graphics.setColor(AnimationSettings.DAMAGE_BAR_COLOR_PHYSICAL);
            graphics.setStroke(PLAIN_STROKE);
            graphics.drawLine(imageLeft, imageBottom+2, (int)(imageLeft+(imageRight-imageLeft)*damagePercent), imageBottom+2);
        }
        if (moveDamagePercent >= 0.01) {
            graphics.setColor(AnimationSettings.DAMAGE_BAR_COLOR_MOVEMENT);
            graphics.setStroke(PLAIN_STROKE);
            graphics.drawLine(imageLeft, imageTop-1, (int)(imageLeft+(imageRight-imageLeft)*moveDamagePercent), imageTop-1);
        }
        if (psiDamagePercent >= 0.01) {
            graphics.setColor(AnimationSettings.DAMAGE_BAR_COLOR_PSI);
            graphics.setStroke(PLAIN_STROKE);
            graphics.drawLine(imageLeft, imageTop-2, (int)(imageLeft+(imageRight-imageLeft)*psiDamagePercent), imageTop-2);
        }
        if (crewPercent < 1.0) {
            graphics.setColor(AnimationSettings.DAMAGE_BAR_COLOR_CREW);
            graphics.setStroke(PLAIN_STROKE);
            graphics.drawLine(imageLeft, imageBottom+3, (int)(imageLeft+(imageRight-imageLeft)*(1.0-crewPercent)), imageBottom+3);
        }
        
        if (selected != null && selected.equals(bo)) {
            int selectWidth = width + ImageObjectSettings.INTER_CRAFT_GAP*2;
            int selectHeight = height + ImageObjectSettings.INTER_CRAFT_GAP*2;
            //graphics.setStroke(PLAIN_STROKE);
            //graphics.setColor(Color.LIGHT_GRAY);
            //graphics.drawRect(loc.getX()-selectWidth/2, loc.getY()-selectHeight/2, selectWidth, selectHeight);
            String image = SelectionAnimation.getImageLarge();
            ImageUtils.drawImage(_container, graphics, image, loc.getX()-selectWidth/2-5, loc.getY()-selectHeight/2-5, selectWidth+10, selectHeight+10);
        } else if (isNext) {
            int selectWidth = width + ImageObjectSettings.INTER_CRAFT_GAP*2;
            int selectHeight = height + ImageObjectSettings.INTER_CRAFT_GAP*2;
            String image = SelectionAnimation.getNextToMoveImage();
            ImageUtils.drawImage(_container, graphics, image, loc.getX()-selectWidth/2-5, loc.getY()-selectHeight/2-5, selectWidth+10, selectHeight+10);
        }
    }
    
    public static void drawRotatedImage(Container container, Graphics2D graphics, ImageObject io) throws ApplicationError {
        Location loc = io.getLocation();
        int direction = io.getDirection();
        drawRotatedImage(container, graphics, io, loc, direction);
    }
    
    public static void drawRotatedImage(Container container, Graphics2D graphics, ImageObject io, Location loc, int direction) throws ApplicationError {
        int width = io.getWidth();
        int height = io.getHeight();
        int imageLeft = loc.getX()-width/2;
        int imageTop = loc.getY()-height/2;
        String imageName = io.getImage();
        drawRotatedImage(container, graphics, imageName, imageLeft, imageTop, width, height, loc, direction);
    }

    public static void drawRotatedImageCenteredAt(Container container, Graphics2D graphics, String imageName, int x, int y, int width, int height, int direction) throws ApplicationError {
        int imageLeft = x-width/2;
        int imageTop = y-height/2;
        drawRotatedImage(container, graphics, imageName, imageLeft, imageTop, width, height, null, direction);
    }


//    public static void drawRotatedImageDeligate(Container container, Graphics2D graphics, ImageObjectDeligate io) throws ApplicationError {
//        Location loc = io.getLocation();
//        int direction = io.getDirection();
//        drawRotatedImageDeligate(container, graphics, io, loc, direction);
//    }
    
    public static void drawRotatedImageDeligate(Container container, Graphics2D graphics, ImageObjectDelegate io, Location loc, int direction) throws ApplicationError {
    	Size imageSize = io.getImageSize();
        int width = imageSize.getX();
        int height = imageSize.getY();
        int imageLeft = loc.getX()-width/2;
        int imageTop = loc.getY()-height/2;
        String imageName = io.getImage();
        drawRotatedImage(container, graphics, imageName, imageLeft, imageTop, width, height, loc, direction);
    }

    public static void drawRotatedImageDeligate(Container container, Graphics2D graphics, ImageObjectDelegate io, Location loc, int direction, double scale) throws ApplicationError {
    	Size imageSize = io.getImageSize();
        int width = (int)(imageSize.getX() * scale);
        int height = (int)(imageSize.getY() * scale);
        int imageLeft = loc.getX()-width/2;
        int imageTop = loc.getY()-height/2;
        String imageName = io.getImage();
        drawRotatedImage(container, graphics, imageName, imageLeft, imageTop, width, height, loc, direction);
    }

    public static void drawRotatedImage(Container container, Graphics2D graphics, String imageName, int imageLeft, int imageTop, int width, int height, Location loc, int direction) {
    	if (Telemetry.ON) Telemetry.qtyDrawRotatedImage++;
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [drawRotatedImage "+imageName+"]");
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        double angleInRadians = Math.toRadians(directonToRotation(direction));
        if (angleInRadians != 0) {
            if (loc == null) {
                graphics.rotate(angleInRadians, imageLeft+width/2, imageTop+height/2);
            } else {
                graphics.rotate(angleInRadians, loc.getX(), loc.getY());
            }
        }
        //Logger.logDebug("DRAW "+loc.getX()+", "+loc.getY()+" ["+size.getX()+" ,"+size.getY()+"] "+bo.getDirection()+" : "+bo.getImageRotationRequired()+" "+imageName);
        //Logger.logDebug("DRAW "+imageName+"@"+loc.getX()+", "+loc.getY());
        graphics.drawImage(image, imageLeft, imageTop, width, height, container);
        if (angleInRadians != 0) {
            if (loc == null) {
                graphics.rotate(-angleInRadians, imageLeft+width/2, imageTop+height/2);
            } else {
                graphics.rotate(-angleInRadians, loc.getX(), loc.getY());
            }
        }
    }

    public static void drawRotatedImage(Container container, Graphics2D graphics, String imageName, int width, int height, Location loc, int direction) {
    	if (Telemetry.ON) Telemetry.qtyDrawRotatedImage++;
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [drawRotatedImage "+imageName+"]");
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        double angleInRadians = Math.toRadians(directonToRotation(direction));
        if (angleInRadians != 0) {
            graphics.rotate(angleInRadians, loc.getX(), loc.getY());
        }
        int imageLeft = loc.getX()-width/2;
		int imageTop = loc.getY()-height/2;
		//Logger.logDebug("DRAW "+loc.getX()+", "+loc.getY()+" ["+size.getX()+" ,"+size.getY()+"] "+bo.getDirection()+" : "+bo.getImageRotationRequired()+" "+imageName);
        //Logger.logDebug("DRAW "+imageName+"@"+loc.getX()+", "+loc.getY());
        graphics.drawImage(image, imageLeft, imageTop , width, height, container);
        if (angleInRadians != 0) {
            graphics.rotate(-angleInRadians, loc.getX(), loc.getY());
        }
    }
    
    private static AlphaComposite alpha = null;
    private static AlphaComposite getAlpha5() {
    	if (alpha == null)  alpha = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);
		return alpha;
    }

    public static void drawImageAlpha5(Container container, Graphics2D graphics, String imageName, int imageLeft, int imageTop, int width, int height) {
    	if (Telemetry.ON) Telemetry.qtyDrawImage++;
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [drawImage "+imageName+"]");
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        Composite originalComposite = graphics.getComposite();
        graphics.setComposite(getAlpha5());
        graphics.drawImage(image, imageLeft, imageTop, width, height, container);
        graphics.setComposite(originalComposite);
    }
    
    public static void drawImage(Container container, Graphics2D graphics, String imageName, int imageLeft, int imageTop, int width, int height) {
    	if (Telemetry.ON) Telemetry.qtyDrawImage++;
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [drawImage "+imageName+"]");
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        graphics.drawImage(image, imageLeft, imageTop, width, height, container);
    }
    
    public static void drawCenteredImage(Container container, Graphics2D graphics, String imageName, int x, int y) {
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [drawCenteredImage "+imageName+"]");
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        int width = image.getWidth(container);
        int height = image.getHeight(container);
        int imageLeft = x-width/2;
        int imageTop = y-height/2;
        graphics.drawImage(image, imageLeft, imageTop, width, height, container);
    }

    public static void _drawImageAt(Container container, Graphics2D graphics, String imageName, int imageLeft, int imageTop, int width, int height) {
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [_drawImageAt "+imageName+"]");
    	if (imageName == null) return;
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        graphics.drawImage(image, imageLeft, imageTop, width, height, container);
    }


    public static void drawImageCenteredAt(GraphicalAnimationContainer container, Graphics2D graphics, ImageAnimationConfig image, int x, int y) {
    	Size size = image.getImageSize();
    	_drawImageCenteredAt(container.getContainer(), graphics, image.getActualImage(), x, y, size.getX(), size.getY());
    }
    
    public static void drawImageCenteredAt(GraphicalAnimationContainer container, Graphics2D graphics, ImageAnimationConfig image, int x, int y, int xdim, int ydim) {
    	_drawImageCenteredAt(container.getContainer(), graphics, image.getActualImage(), x, y, xdim, ydim);
    }
    
    public static void drawImageCenteredAt(GraphicalAnimationContainer container, Graphics2D graphics, String imageName, Location loc, int width, int height) {
    	_drawImageCenteredAt(container.getContainer(), graphics, imageName, loc.getX(), loc.getY(), width, height);    	
    }
    
    public static void _drawImageCenteredAlpha5At(Container container, Graphics2D graphics, String imageName, int x, int y, int width, int height) {
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [_drawImageCenteredAt "+imageName+"]");
    	if (imageName == null) return;
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        int imageLeft = x-width/2;
        int imageTop = y-height/2;
        Composite originalComposite = graphics.getComposite();
        graphics.setComposite(getAlpha5());
        graphics.drawImage(image, imageLeft, imageTop, width, height, container);
        graphics.setComposite(originalComposite);
    }

    public static void _drawImageCenteredAt(Container container, Graphics2D graphics, String imageName, int x, int y, int width, int height) {
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [_drawImageCenteredAt "+imageName+"]");
    	if (imageName == null) return;
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        int imageLeft = x-width/2;
        int imageTop = y-height/2;
        graphics.drawImage(image, imageLeft, imageTop, width, height, container);
    }

    public static void drawImageCenteredAt(Container container, Graphics2D graphics, ImageAnimationConfig imageConfig, int x, int y, int width, int height) {
    	if (imageConfig == null) {
    		return;
    	}
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [drawImageCenteredAt "+imageConfig.getImage()+"]");
    	String imageName = imageConfig.getActualImage(MovementRegister.getTickNo());
    	if (imageName == null) return;
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        int imageLeft = x-width/2;
        int imageTop = y-height/2;
        graphics.drawImage(image, imageLeft, imageTop, width, height, container);
    }

    private static int labelHeight = 0;
    
    public static void drawCenteredLabel(Container container, Graphics2D graphics, String text, int x, int y) {
    	FontMetrics fontMetrics = graphics.getFontMetrics();
    	int labelWidth = fontMetrics.stringWidth(text);
    	if (labelHeight == 0) {
    		labelHeight = fontMetrics.getHeight();
    	}
        int left = x - labelWidth/2;
        int top = y - labelHeight/2;
        //Logger.logDebug("actual Loc Y="+y+" label height="+labelHeight);
		graphics.drawString(text, left, top);
    }

    
    public static void drawCenteredLabel(Container container, Graphics2D graphics, String text, int centerX, int centerY, Color colour, boolean bold, int size) {
    	FontMetrics fontMetrics = graphics.getFontMetrics();
    	int labelWidth = fontMetrics.stringWidth(text);
//    	if (labelHeight == 0) {
//    		labelHeight = fontMetrics.getHeight();
//    	}
        int left = centerX - labelWidth/2;
        int top = centerY;// - labelHeight/2;
        Font font = graphics.getFont();
        graphics.setColor(colour);
        graphics.setFont(new Font(font.getName(), bold ? Font.BOLD : Font.PLAIN, size));
		graphics.drawString(text, left, top);
    }

    public static int drawLabel(Container container, Graphics2D graphics, String text, int left, int top, Color colour, boolean bold, int size) {
    	Font font = graphics.getFont();
    	graphics.setColor(colour);
    	graphics.setFont(new Font(font.getName(), bold ? Font.BOLD : Font.PLAIN, size));
    	FontMetrics fontMetrics = graphics.getFontMetrics();
    	int labelWidth = fontMetrics.stringWidth(text);
		graphics.drawString(text, left, top);
        graphics.setFont(font);
		return labelWidth;
    }

    public static void drawCenteredXYLabel(Container container, Graphics2D graphics, String text, int centerX, int centerY, Color colour, boolean bold, int size) {
    	FontMetrics fontMetrics = graphics.getFontMetrics();
    	int labelWidth = fontMetrics.stringWidth(text);
    	if (labelHeight == 0) {
    		labelHeight = fontMetrics.getHeight();
    	}
        int left = centerX - labelWidth/2;
        int top = centerY - labelHeight/2;
        Font font = graphics.getFont();
        graphics.setColor(colour);
        graphics.setFont(new Font(font.getName(), bold ? Font.BOLD : Font.PLAIN, size));
		graphics.drawString(text, left, top);
    }


    public static void tileImageNoScale(Container container, Graphics2D graphics, String imageName, Area area) {
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [tileImageNoScale "+imageName+"]");
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        int imageWidth = image.getWidth(container);
        int imageHeight = image.getHeight(container);
        if (imageWidth < 1) return;
        int left = area.getLeft();
        int top = area.getTop();
        int right = left + area.getWidth();
        int bottom = top + area.getHeight();
        int areaWidth = area.getWidth();
        int areaHeight = area.getHeight();
        int x = left;
    	while (x < right) {
    		int y = top;
        	while (y < bottom) {
                graphics.drawImage(image, x, y, container);
                y += imageHeight;
        	}
        	x += imageWidth;
    	}
    }

    public static void tileImage(Container container, Graphics2D graphics, String imageName, Area area) {
    	if (Logger.TRACE_ON) Logger.logDebug("PAINT [tileImage "+imageName+"]");
        Image image = ImageCache.getImage(container.getToolkit(), imageName);
        int imageWidth = image.getWidth(container);
        int imageHeight = image.getHeight(container);
        if (imageWidth < 1) return;
        int left = area.getLeft();
        int top = area.getTop();
        int right = left + area.getWidth();
        int bottom = top + area.getHeight();
        int areaWidth = area.getWidth();
        int areaHeight = area.getHeight();
        int x = left;
        int y = top;
    	while (x < right) {
    		int w = Comp.min(imageWidth, right-x);
        	while (y < bottom) {
        		int h = Comp.min(imageHeight, bottom-y);
                graphics.drawImage(image, x, y, w, h, container);
                y += imageHeight;
        	}
        	x += imageWidth;
    	}
    }
    
	public static void drawAnimatedBattleArea(Container container, Graphics2D graphics, AnimatedBattleAreaDetails aba) throws ApplicationError {
		if (aba.tileArea()) {
			String image = aba.getAnimationConfig().getActualImage();
			if (image == null) return;
			ImageUtils.tileImage(container, graphics, image, aba.getArea());
		} else {
			Location center = aba.getCenterLocation();
			String image = aba.getAnimationConfig().getActualImage();
			if (image == null) return;
			ImageUtils._drawImageCenteredAt(container, graphics, image, center.getX(), center.getY(), aba.getWidth(), aba.getHeight());
		}
	}
    
    public static final int NO_ROTATION_DIRECTION = 270;

    /**
     * @return The angle of rotation required for this image to be displayed on the screen.
     * All images point left, so the return is a value between 90 and -270 representing the
     * angle of turn required by an image.
     */
    public static double directonToRotation(int direction) {
        int v = direction - 270;
        if (v <= -180) {
            v += 360;
        }
        return v;
    }
    
    private static Stroke barStroke = null;
    
    private static Stroke getBarStroke() {
        if (barStroke == null) {
            barStroke = new BasicStroke(1.0f);
        }
        return barStroke;
    }

    private static final Color rangeColour = new Color(10, 10, 10);
    
	public static void drawMooRange(Graphics2D graphics, Location location, int range) {
        graphics.setColor(rangeColour);
		graphics.fillOval(location.getX() - range, location.getY() - range, range*2, range*2);
	}

}