/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.common;

import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.Frame;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JFrame;

import mot.client.swing.window.common.MotFrame;
import mot.common.bean.Settings;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

public class MotGraphicsEnvironment  {
	
	public static int getWindowSizeX() throws ApplicationError {
		return Settings.getInstance().getWindowSizeX();
	}

	public static int getWindowSizeY() throws ApplicationError {
		return Settings.getInstance().getWindowSizeY();
	}

    private static DisplayMode mode = null;

    private static void switchToGameMode(GraphicsDevice device) throws ApplicationError {
        if (mode == null && device.isDisplayChangeSupported()) {
            mode = new DisplayMode(getWindowSizeX(), getWindowSizeY(), 32, DisplayMode.REFRESH_RATE_UNKNOWN);
            try {
                device.setDisplayMode(mode);
            } catch (UnsupportedOperationException e) {
                Logger.logError("Unable to switch to graphics mode", e);
                mode = null;
            }
        } else {
            Logger.logWarning("Unable to switch to graphics mode");
        }
    }

    /**
     * Switch the game to normal window mode.
     * In main screen mode the game runs as a normal window. It uses the swing / OS
     * painting thread. The window is resizable just like any other operating system 
     * window.
     */
    public static void setMainScreen(MotFrame motFrame) throws ApplicationError {    
        if (motFrame != null) {
        	JFrame window = motFrame.getWindow();
        	if (Settings.getInstance().getMaximised() == 1) {
                window.setExtendedState(Frame.MAXIMIZED_BOTH);
                window.setResizable(false);
        	} else {
                window.setExtendedState(Frame.NORMAL);
                window.setSize(getWindowSizeX(), getWindowSizeY());
                window.setResizable(true);
        	}
            window.setUndecorated(true);
       		motFrame.maximise();
        }
    }

    /**
     * Switch the game into full screen mode.
     * In full screen mode the game takes complete control of the screen. It switches
     * off OS level repaints and uses an internal paint loop. It uses page flipping
     * and writes directly to the display device for maximum performance!
     */
    public static void setFullScreen(MotFrame motFrame) throws ApplicationError {    
        GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
        if (motFrame != null) {
        	JFrame window = motFrame.getWindow();
            // We use our own painting loop and swing is not thread safe so we turn off
            // OS level repainting - repaints will only originate from swing now!
            //window.setIgnoreRepaint(true);
        	
        	// Make the window so that is can only be maximised and remove the window
        	// decorations to minimise and restore the window
            window.setExtendedState(Frame.MAXIMIZED_BOTH);
            window.setUndecorated(true);
            window.setResizable(false);
        	motFrame.maximise();
            
            // Make this window the fill screen window on the computers main display
            JFrame.setDefaultLookAndFeelDecorated(false);
            device.setFullScreenWindow(window);
            switchToGameMode(device);
            
            // use Page flipping (double buffering)
            window.createBufferStrategy(2);
        }
    }
}

/**
 * This is a pathetic attempt to stop the user moving the window!
 */
class WindowPositionAdapter extends ComponentAdapter {
    private boolean locked = false;
    private int DISTANCE_TO_EDGE = 0;
	
    public void componentMoved(ComponentEvent evt) {
        if(locked) return;
        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        int nx = evt.getComponent().getX();
        int ny = evt.getComponent().getY();
        if(nx < 0) {
            nx = 0;
        }
        if(ny < 0) {
            ny = 0;
        }
        if(nx < 0+DISTANCE_TO_EDGE) {
            nx = 0;
        }
        if(nx > size.getWidth()-evt.getComponent().getWidth()-DISTANCE_TO_EDGE) {
            nx = (int)size.getWidth()-evt.getComponent().getWidth();
        }

        if(ny > size.getHeight()-evt.getComponent().getHeight()-DISTANCE_TO_EDGE) {
            ny = (int)size.getHeight()-evt.getComponent().getHeight();
        }
		// make sure we don't get into a recursive loop when the
		// set location generates more events
        locked = true;
        evt.getComponent().setLocation(nx,ny);
        locked = false;
    }
}