/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.common;

import java.awt.Font;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextField;

import mot.client.actions.MotTask;
import mot.client.swing.window.main.MainWindow;
import mot.client.validators.DoubleValidator;
import mot.client.validators.IntegerValidator;
import mot.client.validators.MotValidator;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

public class MotTextField extends JTextField implements FocusListener {

	private MotValidator validator;
	private MotTask changeTask;

	public MotTextField(int size) {
		super(size);
	}
	
	public MotTextField(int size, MotTask changeTask) {
		super(size);
		this.changeTask = changeTask;
		addFocusListener(this);
	}
	
	public MotTextField(int size, MotValidator validator) {
		super(size);
		this.validator = validator;
		addFocusListener(this);
	}
	
	public MotTextField(int size, int sizeDiff, MotValidator validator) {
		super(size);
		this.validator = validator;
		addFocusListener(this);
        if (sizeDiff != 0) {
            Font font = getFont();
            int style = font.getStyle();
            setFont(new Font(font.getName(), style, font.getSize()+sizeDiff));
        }
	}
	
	public int getTextAsInt() throws ValidationError {
		String text = getText();
		if (text == null || text.length() == 0) {
			return 0;
		}
		if (validator != null && validator instanceof IntegerValidator) {
			IntegerValidator iv = (IntegerValidator) validator;
			Integer v = iv.validate(text);
			return v;
		} else {
			try {			
				return Integer.parseInt(text);
			} catch (NumberFormatException e) {
				throw new ValidationError(L.INVALID_+" "+text);
			}
		}
	}

	public double getTextAsDouble() throws ValidationError {
		String text = getText();
		if (text == null || text.length() == 0) {
			return 0;
		}
		if (validator != null && validator instanceof DoubleValidator) {
			DoubleValidator iv = (DoubleValidator) validator;
			Double v = iv.validate(text);
			return v;
		} else {
			try {			
				return Double.parseDouble(text);
			} catch (NumberFormatException e) {
				throw new ValidationError(L.INVALID_+" "+text);
			}
		}
	}

	//
	// FOCUS LISTENER STUFF
	//
	
	public void focusGained(FocusEvent e) {
	}

	public void focusLost(FocusEvent e) {
		try {
			if (validator != null) {
				String text = getText();
				validator.validate(text);
			}
			if (changeTask != null) {
				changeTask.run();
			}
		} catch (ApplicationError ex) {
			MainWindow.getExestingInstance().reportApplicationError(ex);
		} catch (ValidationError ex) {
			MainWindow.getExestingInstance().reportValidationError(ex);
		}
	}

	public MotTask getChangeTask() {
		return changeTask;
	}

	public void setChangeTask(MotTask changeTask) {
		this.changeTask = changeTask;
	}
	
	
}
