/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.common;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeListener;

import mot.common.bean.Rectangle;
import mot.common.bean.Size;

public class PanelUtils  {
    

    public static JPanel createButtonInPanel(String label, ActionListener al) {
        JPanel p = new JPanel();
        p.add(createButton(label, al));
        return p;
    }

    public static JButton createButton(String label, ActionListener al) {
        return createButton(label, label, al);
    }
    
    public static JButton createButton(String label, String command, ActionListener al) {
        JButton b = new JButton(label);
        b.setActionCommand(command);
        b.addActionListener(al);
        return b;
    }

    public static JLabel createLabel(String text) {
        return new JLabel(text);
    }

    public static JLabel createLabel(String text, Color backgroundColour) {
        JLabel label = new JLabel(text);
        if (backgroundColour != null) {
            label.setBackground(backgroundColour);
            label.setOpaque(true);
        }
        return label;
    }

    public static JLabel createLabel(String text, int sizeDiff, int alignment) {
    	return createLabel(text, sizeDiff, null, null, alignment, false);
    }
    
    public static JLabel createLabel(String text, int sizeDiff, int alignment, boolean bold) {
    	return createLabel(text, sizeDiff, null, null, alignment, bold);
    }
    
    public static JLabel createLabel(String text, int alignment) {
    	return createLabel(text, 0, null, null, alignment, false);
    }
    
    public static JLabel createLabel(String text, int alignment, Color backgroundColour) {
        JLabel label = new JLabel(text, alignment);
        label.setBackground(backgroundColour);
        label.setOpaque(true);
    	return label;
    }
    
    public static JLabel createLabel(String text, int sizeDiff, Dimension min, Dimension pref, int alignment, boolean bold) {
        JLabel label = new JLabel(text);
        if (sizeDiff != 0 || bold) {
            Font font = label.getFont();
            int style = font.getStyle();
            if (bold) {
                style = Font.BOLD;
            }
            label.setFont(new Font(font.getName(), style, font.getSize()+sizeDiff));
        }
        if (min != null) {
            label.setMinimumSize(min);
        }
        if (pref != null) {
            label.setPreferredSize(pref);
        }
        label.setHorizontalAlignment(alignment);
        return label;
    }

    public static JTextField createTextField(int size, String text, ActionListener al) {
        JTextField b = new JTextField();
        b.setText(text);
        b.setColumns(size);
        if (al != null) {
            b.addActionListener(al);
        }
        return b;
    }

    public static JToggleButton createToggleButton(String label, ActionListener al, ButtonGroup bg) {
        return createToggleButton(label, label, al, bg, null);
    }
    
    public static JToggleButton createToggleButton(String label, ActionListener al, ButtonGroup bg, boolean selected) {
        return createToggleButton(label, label, al, bg, null, selected);
    }
    
    public static JToggleButton createToggleButton(String label, ActionListener al, ButtonGroup bg, Font font) {
        return createToggleButton(label, label, al, bg, font);
    }
    
    public static JToggleButton createToggleButton(String label, String command, ActionListener al, ButtonGroup bg, Font font) {
        return createToggleButton(label, command, al, bg, font, false);
    }
    
    public static JToggleButton createToggleButton(String label, String command, ActionListener al, ButtonGroup bg, Font font, boolean selected) {
        JToggleButton b = new JToggleButton(label);
        b.setActionCommand(command);
        b.addActionListener(al);
        bg.add(b);
        if (font != null) {
            b.setFont(font);
        }
        if (selected) {
            bg.setSelected(b.getModel(), true);
        }
        return b;
    }
    
    public static JRadioButton createRadioButton(String label, ActionListener al, ButtonGroup bg, boolean selected) {
        JRadioButton b = new JRadioButton(label);
        b.addActionListener(al);
        bg.add(b);
        if (selected) {
            bg.setSelected(b.getModel(), true);
        }
        return b;
    }
    
    public static JCheckBox createCheckBox(String label, Font font) {
        return createCheckBox(label, null, null, font);
    }
    
    public static JCheckBox createCheckBox(String label, String command, ActionListener al, Font font) {
        JCheckBox b = new JCheckBox(label);
        if (al != null) {
            b.setActionCommand(command);
            b.addActionListener(al);
        }
        if (font != null) {
            b.setFont(font);
        }
        return b;
    }

    public static JSlider createPercentSlider(ChangeListener cl, boolean withTicks) {
        JSlider slider = new JSlider(0, 100);
        slider.addChangeListener(cl);
        if (withTicks) {
            slider.setPaintTicks(true);
            slider.setMajorTickSpacing(10);
            slider.setLabelTable( slider.createStandardLabels(50) );
            //slider.setMinorTickSpacing(5);
        }
        return slider;
    }
    

    public static JPanel createComponentStack(JComponent label1, String text) {
        return createComponentStack(label1, createLabel(text), "H");
    }
    
    public static JPanel createComponentStack(JComponent label1, JComponent label2) {
        return createComponentStack(label1, label2, "H");
    }
    
    public static JPanel createComponentStack(JComponent label1, JComponent label2, String constraints) {
        ComponentTable panel = new ComponentTable(constraints, 2, 1, 0, 0, 0);
        panel.add(label1);
        panel.add(label2);
        return panel;
    }

    public static JPanel createNamedControlV(JLabel label1, JComponent label2, String constraints) {
        ComponentTable panel = new ComponentTable(constraints, 2, 1, 0, 0, 0);
        panel.setOpaque(false);
        panel.add(label1, "B"+constraints);
        panel.add(label2, "T"+constraints);
        return panel;
    }

    public static JPanel createNamedControlV(JLabel label1, JComponent label2, Color backgroundColor) {
    	label1.setOpaque(true);
    	label1.setBackground(backgroundColor);
    	label2.setOpaque(true);
    	label2.setBackground(backgroundColor);
        ComponentTable panel = new ComponentTable("H", 2, 1, 0, 0, 0);
        panel.setBackground(backgroundColor);
        panel.setOpaque(true);
        panel.add(label1);
        panel.add(label2);
        return panel;
    }

    public static JPanel createNamedControlV(JLabel label1, JComponent label2) {
        ComponentTable panel = new ComponentTable("H", 2, 1, 0, 0, 0);
        panel.setOpaque(false);
        panel.add(label1);
        panel.add(label2);
        return panel;
    }

    public static JPanel createNamedControlV(String label1, JComponent comp) {
        ComponentTable panel = new ComponentTable("H", 2, 1, 0, 0, 0);
        panel.setOpaque(false);
        panel.add(new JLabel(label1));
        panel.add(comp, "HVPS");
        return panel;
    }

    public static JPanel createNamedControlH(String label1, JComponent label2) {
        ComponentTable panel = new ComponentTable("RV,HV", 1, 2, 0, 3, 0);
        panel.setOpaque(false);
        panel.add(new JLabel(label1));
        panel.add(label2);
        return panel;
    }

    public static JPanel createNamedControlH(JLabel label1, JComponent label2) {
        ComponentTable panel = new ComponentTable("RV,HV", 1, 2, 0, 3, 0);
        panel.setOpaque(false);
        panel.add(label1);
        panel.add(label2);
        return panel;
    }

    public static JPanel createFiller() {
        JPanel panel = new JPanel();
        panel.setOpaque(false);
        return panel;
    }
    
    public static JPanel createFiller(int x, int y) {
        JPanel panel = new JPanel();
        panel.setOpaque(false);
        panel.setMinimumSize(new Dimension(x,y));
        return panel;
    }

    public static JPanel createFrame(JComponent content, Color background) {
    	ComponentTable panel = new ComponentTable("-", 1,1, 0,0,0);
        panel.setOpaque(true);
        panel.setBackground(background);
        panel.add(content);
        return panel;
    }

    public static void setTheBloodySize(JComponent comp, Size size) {
    	if (size != null) {
    		setTheBloodySize(comp, size.getX(), size.getY());
    	}
    }
    
    // There are so many circumstances where setMinimumSize is ignored I resort to this!
    public static void setTheBloodySize(JComponent comp, int x, int y) {
        Dimension s = new Dimension(x, y);
        comp.setSize(s);
        comp.setMinimumSize(s);
        comp.setPreferredSize(s);
    }
    
    public static Rectangle tooOnScreen(Rectangle rect, Component component) {
    	Point origin = new Point(rect.getLeft(), rect.getTop());
		SwingUtilities.convertPointToScreen(origin, component);
		return new Rectangle(origin.x, origin.y, rect.getWidth(), rect.getHeight());
    }

	public static Rectangle tooRectangle(java.awt.Rectangle r) {
	    return new  Rectangle(r.x, r.y, r.width, r.height);
	}

	public static void addCascadingMouseListener(JComponent c, MouseListener ml) {
		if (ml == null || c == null) return;
		c.addMouseListener(ml);
		Component [] children = c.getComponents();
		if (children != null) {
			for (int i = 0; i < children.length; i++) {
				if (children[i] instanceof JComponent) {					
					addCascadingMouseListener((JComponent)children[i], ml);
				}
			}
		}
	}


}