/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.config;

import java.util.Collection;
import java.util.HashMap;

import mot.common.bean.NameIntPair;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.xml.DomFactory;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;
import mot.common.xml.XMLDocumentHandler;

/**
 * Contains the list of animation configurations
 * @see WeaponFireAnimationConfig for details
 */
public class AnimationConfigFactory {
	
	private static final String XML_FILE = "xml/animation_config.xml";

	private static final int DEFAULT_LIFE = 10;
	
	private static AnimationConfigFactory instance;
	
	public static AnimationConfigFactory getInstance() throws ApplicationError {
		if (instance == null) {
			instance = (AnimationConfigFactory)XMLDocumentHandler.parseResource(XML_FILE);
		}
		return instance;
	}

	public static void preLoad() throws ApplicationError {
		instance = new AnimationConfigFactory(DomFactory.load(XML_FILE));
	}
    


	/**
	 * Maps the name of the interval to a name value pair containing the name and the life.
	 * This is used to store a generic list of life values for animations. Note that the life
	 * of an animation of the quantity of frames it remains during animation.
	 * List of NameIntPair.
	 */
	private HashMap intervals = new HashMap();

	/**
	 * Maps the name of the animation type to the config details for that name.
	 * List of WeaponFireAnimationConfig.
	 */
	private HashMap weaponFireConfigs = new HashMap();
	
	/**
	 * Maps the name of the animation type to the config details for that name.
	 * List of ImageAnimationConfig.
	 */
	private HashMap imageAnimationConfigs = new HashMap();
	
	/**
	 * Maps the name of the animation type to the config details for that name.
	 * List of ImageConfig.
	 */
	private HashMap imageConfigs = new HashMap();
	
	/**
	 * Maps the name of the animation type to the config details for that name.
	 * List of ImageStrokeAnimationConfig.
	 */
	private HashMap imageStrokeAnimations = new HashMap();
	private HashMap imageDuelStrokeAnimations = new HashMap();
	
	/**
	 * Maps the name of the animation type to the config details for that name.
	 * List of ImageScrollAnimationConfig.
	 */
	private HashMap imageScrollAnimations = new HashMap();
	
	/**
	 * This is a list of weapon groups that do not have any WEAPON fire animation.
	 */
	private String [] noWeaponFireAnimation;
	
	/**
	 * This is a list of weapon groups that do not have any WEAPON fire animation.
	 */
	private String [] noDamageAnimation;
	
    private ImageAnimationConfig shieldsConfig;
    private ImageAnimationConfig asteroidsConfig;
    private ImageAnimationConfig asteroidRingConfig;
    private ImageAnimationConfig spaceDistortionConfig;
    private ImageAnimationConfig craftWreckageConfig;

	public AnimationConfigFactory() {
	}

	public AnimationConfigFactory(DomTree doc) throws ApplicationError {
    	DomNode root = doc.getNodeByName("animation-config-list");

    	setNoWeaponFireAnimation(root.getNodeValueByName_String("no-weapon-fire-animation"));
    	setNoDamageAnimation(root.getNodeValueByName_String("no-damage-animation"));
    	
    	DomNode[] a1 = root.getNodesByName("interval");
    	if (a1 != null) {
    		for (int i = 0; i < a1.length; i++) {
    			setInterval(new NameIntPair(a1[i]));
			}
    	}
    	
    	DomNode[] a2 = root.getNodesByName("weapon-fire-animation-config");
    	if (a2 != null) {
    		for (int i = 0; i < a2.length; i++) {
    			setWeaponFireAnimationConfig(new WeaponFireAnimationConfig(a2[i]));
    		}
    	}
    	
    	DomNode[] a3 = root.getNodesByName("image-animation-config");
    	if (a3 != null) {
    		for (int i = 0; i < a3.length; i++) {
    			setImageAnimationConfig(new ImageAnimationConfig(a3[i]));
    		}
    	}
    	
    	DomNode[] a4 = root.getNodesByName("image-config");
    	if (a4 != null) {
    		for (int i = 0; i < a4.length; i++) {
    			setImageConfig(new ImageConfig(a4[i]));
    		}
    	}
    	
    	DomNode[] a5 = root.getNodesByName("image-stroke-animation-config");
    	if (a5 != null) {
    		for (int i = 0; i < a5.length; i++) {
    			setImageStrokeAnimationConfig(new ImageStrokeAnimationConfig(a5[i]));
    		}
    	}
    	
    	DomNode[] a6 = root.getNodesByName("image-scroll-animation-config");
    	if (a6 != null) {
    		for (int i = 0; i < a6.length; i++) {
    			setImageScrollAnimationConfig(new ImageScrollAnimationConfig(a6[i]));
    		}
    	}
    	
    	DomNode[] a7 = root.getNodesByName("image-duel-stroke-animation-config");
    	if (a7 != null) {
    		for (int i = 0; i < a7.length; i++) {
    			setImageDuelStrokeAnimationConfig(new ImageDuelStrokeAnimationConfig(a7[i]));
    		}
    	}
	}


    public void setWeaponFireAnimationConfig(WeaponFireAnimationConfig v) {
		if (v.getName() != null) {
			weaponFireConfigs.put(v.getName(), v);
			if (v.getGroup() != null && weaponFireConfigs.get(v.getGroup()) == null) {
				weaponFireConfigs.put(v.getGroup(), v);
			}
		} else {
			if (v.getGroup() != null) {
				weaponFireConfigs.put(v.getGroup(), v);
			}
		}
	}
	
	public void setImageAnimationConfig(ImageAnimationConfig v) {
		imageAnimationConfigs.put(v.getName(), v);
	}
	
	public void setImageConfig(ImageConfig v) {
		imageConfigs.put(v.getName(), v);
	}
	
	public WeaponFireAnimationConfig getWeaponFireAnimationConfig(Weapon w, String suffix) {
		return getWeaponFireAnimationConfig(w.getName(), w.getGroup(), suffix);
	}
	
	public WeaponFireAnimationConfig getWeaponFireAnimationConfig(String name, String group, String suffix) {
		WeaponFireAnimationConfig r = null;
		if (suffix != null) {
			r = (WeaponFireAnimationConfig) weaponFireConfigs.get(name+suffix);
			if (r == null) {
				r = (WeaponFireAnimationConfig) weaponFireConfigs.get(group+suffix);
			}
		} else {
			r = (WeaponFireAnimationConfig) weaponFireConfigs.get(name);
			if (r == null) {
				r = (WeaponFireAnimationConfig) weaponFireConfigs.get(group);
			}
		}
		return r;
	}

	public WeaponFireAnimationConfig [] getWeaponFireAnimationConfigs() {
		Collection c = weaponFireConfigs.values();
		if (c == null) return null;
		return (WeaponFireAnimationConfig []) c.toArray(new WeaponFireAnimationConfig [c.size()]);
	}
	
	public int getWeaponFireAnimationLife(String group) {
		WeaponFireAnimationConfig r = (WeaponFireAnimationConfig) weaponFireConfigs.get(group);
		if (r == null) {
			return 0;
		} else {
			return r.getLife();
		}
	}

	public ImageAnimationConfig getImageAnimationConfig(String name) {
		ImageAnimationConfig r = (ImageAnimationConfig) imageAnimationConfigs.get(name);
		return r;
	}

	public int getQtyImagesAnimationConfig(String name) {
		ImageAnimationConfig r = (ImageAnimationConfig) imageAnimationConfigs.get(name);
		if (r == null) {
			return 0;
		} else {
			return r.getQtyImages();
		}
	}
	
	public ImageAnimationConfig [] getImageAnimationConfigs() {
		Collection c = imageAnimationConfigs.values();
		if (c == null) return null;
		return (ImageAnimationConfig []) c.toArray(new ImageAnimationConfig[c.size()]);
	}

	public ImageConfig getImageConfig(String name) {
		ImageConfig r = (ImageConfig) imageConfigs.get(name);
		return r;
	}

	public int getImagesConfigLife(String name) {
		ImageConfig r = (ImageConfig) imageConfigs.get(name);
		if (r == null) {
			return 0;
		} else {
			return r.getLife();
		}
	}

	public ImageConfig [] getImageConfigs() {
		Collection c = imageConfigs.values();
		if (c == null) return null;
		return (ImageConfig []) c.toArray(new ImageConfig[c.size()]);
	}
	

	public String[] getNoWeaponFireAnimation() {
		return noWeaponFireAnimation;
	}

	public void setNoWeaponFireAnimation(String[] noWeaponFireAnimation) {
		this.noWeaponFireAnimation = noWeaponFireAnimation;
	}

	public void setNoWeaponFireAnimation(String commaSeperatedList) {
		this.noWeaponFireAnimation = commaSeperatedList.split(",");
	}

	public String[] getNoDamageAnimation() {
		return noDamageAnimation;
	}

	public void setNoDamageAnimation(String commaSeperatedList) {
		this.noDamageAnimation = commaSeperatedList.split(",");
	}

	public void setNoDamageAnimation(String[] noDamageAnimation) {
		this.noDamageAnimation = noDamageAnimation;
	}
	
	public void setInterval(NameIntPair v) {
		intervals.put(v.getName(), v);
	}
	
	public NameIntPair getInterval(String name) {
		return (NameIntPair) intervals.get(name);
	}

	public NameIntPair [] getIntervals() {
		Collection ic = intervals.values();
		if (ic == null) return null;
		return (NameIntPair []) ic.toArray(new NameIntPair[ic.size()]);
	}
	

	public int getLife(String name) {
		NameIntPair v = (NameIntPair) intervals.get(name);
		if (v == null) {
			return DEFAULT_LIFE;
		} else {
			return v.getValue();
		}
	}
	
	public void setImageStrokeAnimationConfig(ImageStrokeAnimationConfig v) {
		imageStrokeAnimations.put(v.getName(), v);
	}

	public void setImageScrollAnimationConfig(ImageScrollAnimationConfig v) {
		imageScrollAnimations.put(v.getName(), v);
	}

	public ImageScrollAnimationConfig getImageScrollAnimationConfig(String v) {
		return (ImageScrollAnimationConfig) imageScrollAnimations.get(v);
	}
	
	public ImageStrokeAnimationConfig getImageStrokeAnimationConfig(String v) {
		return (ImageStrokeAnimationConfig) imageStrokeAnimations.get(v);
	}
	
	public ImageStrokeAnimationConfig [] getImageStrokeAnimations() {
		Collection c = imageStrokeAnimations.values();
		if (c == null) return null;
		return (ImageStrokeAnimationConfig []) c.toArray(new ImageStrokeAnimationConfig[c.size()]);
	}

	public void setImageDuelStrokeAnimationConfig(ImageDuelStrokeAnimationConfig v) {
		imageDuelStrokeAnimations.put(v.getName(), v);
	}

	public ImageDuelStrokeAnimationConfig getImageDuelStrokeAnimationConfig(String v) {
		return (ImageDuelStrokeAnimationConfig) imageDuelStrokeAnimations.get(v);
	}
	
	public ImageDuelStrokeAnimationConfig [] getImageDuelStrokeAnimations() {
		Collection c = imageDuelStrokeAnimations.values();
		if (c == null) return null;
		return (ImageDuelStrokeAnimationConfig []) c.toArray(new ImageDuelStrokeAnimationConfig[c.size()]);
	}

    private static final String SHIELDS_PROPERTY = "shields";
    private static final String ASTEROID_PROPERTY = "asteroids";
    private static final String ASTEROID_RING_PROPERTY = "asteroid_ring";
    private static final String SPACE_DISTORTION_PROPERTY = "space_distortion";
    private static final String CRAFT_WRECKAGE_PROPERTY = "craft_wreckage";
    
    public ImageAnimationConfig getAsteroidRingAnimationConfig() throws ApplicationError  {
        if (asteroidRingConfig == null){
	        asteroidRingConfig = AnimationConfigFactory.getInstance().getImageAnimationConfig(ASTEROID_RING_PROPERTY);
        }
        return asteroidRingConfig;
    }

    public ImageAnimationConfig getAsteroidAnimationConfig() throws ApplicationError  {
        if (asteroidsConfig == null){
	        asteroidsConfig = AnimationConfigFactory.getInstance().getImageAnimationConfig(ASTEROID_PROPERTY);
        }
        return asteroidsConfig;
    }

    public ImageAnimationConfig getShieldsAnimationConfig() throws ApplicationError  {
        if (shieldsConfig == null){
        	shieldsConfig = AnimationConfigFactory.getInstance().getImageAnimationConfig(SHIELDS_PROPERTY);
        }
        return shieldsConfig;
    }

    public ImageAnimationConfig getSpaceDistortionAnimationConfig() throws ApplicationError  {
        if (spaceDistortionConfig == null) {
	        spaceDistortionConfig = AnimationConfigFactory.getInstance().getImageAnimationConfig(SPACE_DISTORTION_PROPERTY);
        }
        return spaceDistortionConfig;
    }
	
    public ImageAnimationConfig getCraftWreckageAnimationConfig() throws ApplicationError  {
        if (craftWreckageConfig == null) {
        	craftWreckageConfig = AnimationConfigFactory.getInstance().getImageAnimationConfig(CRAFT_WRECKAGE_PROPERTY);
        }
        return craftWreckageConfig;
    }

}
