/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.config;

import mot.client.animation.MovementRegister;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.xml.DomNode;
import mot.common.xml.XMLInput;

/**
 * This is the details of a sequence of images that are displayed one after the 
 * other to produce an animation.
 * 
 * e.g. Explosion, Time Warp
 */
public class ImageAnimationConfig implements XMLInput {

	/**
	 * This is the name of this animation
	 */
	private String name;
	
	/**
	 * This is the name of the image including the directory but not including 
	 * an index and the file extension.
	 * A complete image name is made up from image + index + imageType
	 */
	private String image;
	
	/**
	 * This is the file extension of the image for this animation including the
	 * dot.
	 */
	private String imageType;
	
	
	/**
	 * This is the size of these images. All images MUST be the same size.
	 */
	private Size imageSize;
	
	/**
	 * This is the quantity of images in this animation. The index that is used
	 * to make this animation of a number between 1 and qtyImages inclusive.
	 * Must either have qtyImages of stages attribute to animate!
	 */
	private int qtyImages;

	/**
	 * Comma seperated list of image indexes. 
	 * Must either have qtyImages of stages attribute to animate!
	 */
	private String stages;
	
	/**
	 * This is the actual images to use for each stage of this animation. This 
	 * prevents all the string manipulation every time an image is to be painted.
	 */
	private transient String [] actualImages = null;

	public ImageAnimationConfig() {
	}

	public ImageAnimationConfig(String name, String image, String imageType, Size imageSize, int qtyImages, String stages) {
		super();
		this.name = name;
		this.image = image;
		this.imageType = imageType;
		this.imageSize = imageSize;
		this.qtyImages = qtyImages;
		this.stages = stages;
		actualImages = null;
	}

	public ImageAnimationConfig(DomNode n) throws ApplicationError {
		this.name = n.getNodeValueByName_String("name");
		this.image = n.getNodeValueByName_String("image");
		this.imageType = n.getNodeValueByName_String("image-type");
		this.imageSize = n.getNodeValueByName_Size("image-size");
		this.qtyImages = n.getNodeValueByName_Int("qty-images");
		
		setStages(n.getNodeValueByName_String("stages"));
	}

	public String getImage() {
		return image;
	}

	public void setImage(String image) {
		this.image = image;
		actualImages = null;
	}

	public Size getImageSize() {
		return imageSize;
	}

	public void setImageSize(Size imageSize) {
		this.imageSize = imageSize;
	}

	public String getImageType() {
		return imageType;
	}

	public void setImageType(String imageType) {
		this.imageType = imageType;
		actualImages = null;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getQtyImages() {
		if (stages != null) {
			return getStagesAsSplit().length;
		} else {
			return qtyImages;
		}
	}

	public void setQtyImages(int qtyImages) {
		this.qtyImages = qtyImages;
		actualImages = null;
	}

	public String getStages() {
		return stages;
	}

	public void setStages(String stages) {
		this.stages = stages;
		actualImages = null;
	}
	
    public String [] getStagesMandatoryAsSplit() throws ApplicationError {
    	if (stages == null) throw new ApplicationError("Animation "+name+" does not have stages");
        return stages.split(",");
    }

    private String [] stagesAsSplit = null;
    
    public String [] getStagesAsSplit() {
        if (stages == null) {
            return null;
        }
        if (stagesAsSplit == null) {
        	stagesAsSplit = stages.split(",");
        }
        return stagesAsSplit; 
    }
    
    public String [] getActualImages() {
    	if (actualImages == null) {
    		String [] stages = getStagesAsSplit();
    		if (stages != null) {
        		actualImages = new String [stages.length];
        		for (int i = 0; i < stages.length; i++) {
        			actualImages[i] = image + stages[i] + imageType;
    			}
    		} else {
        		actualImages = new String [qtyImages];
        		for (int i = 0; i < qtyImages; i++) {
        			actualImages[i] = image + i + imageType;
    			}
    		}
    	}
    	return actualImages;
    }

    public String getActualImage() {
    	return getActualImage(MovementRegister.getTickNo());
    }
    
    public String getActualImage(int animationCounter) {
    	String [] actualImages  = getActualImages();
    	if (actualImages == null || actualImages.length == 0) {
    		return null;
    	} else {
    		return actualImages[animationCounter % actualImages.length];
    	}
    }
}
