/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.config;

import java.awt.Image;
import java.util.Collection;
import java.util.HashMap;

import mot.client.swing.common.ImageCache;
import mot.client.swing.window.main.MainWindow;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.DomFactory;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;
import mot.common.xml.XMLDocumentHandler;

/**
 * Contains the list of animation configurations
 * @see WeaponFireAnimationConfig for details
 */
public class ImageConfigFactory {
	
	private static final String XML_FILE = "xml/image_config.xml";

	private static final String ROOT_ELEMENT_NAME = "image-config-factory";

	private static ImageConfigFactory instance;
	
	public static ImageConfigFactory getInstance() throws ApplicationError {
		if (instance == null) {
			instance = (ImageConfigFactory)XMLDocumentHandler.parseResource(XML_FILE);
		}
		return instance;
	}

	public static void preLoad() throws ApplicationError {
		instance = new ImageConfigFactory(DomFactory.load(XML_FILE));
	}

	public ImageConfigFactory() {
	}

	public ImageConfigFactory(DomTree doc) throws ApplicationError {
    	DomNode root = doc.getNodeByName(ROOT_ELEMENT_NAME);
    	
    	DomNode[] ica = root.getNodesByName("image-config");
    	if (ica != null) {
    		for (int i = 0; i < ica.length; i++) {
				setImageConfig(new ImageConfig(ica[i]));
			}
    	}
    	
    	DomNode mc = root.getNodeByName("menu-config");
    	if (mc != null) menuConfig = new MenuConfig(mc);
    	
    	DomNode fc = root.getNodeByName("flag-config");
    	if (fc != null) flagConfig = new ImageConfigList(fc);
    	
    	DomNode ric = root.getNodeByName("race-image-config");
    	if (ric != null) raceImageConfig = new ImageConfigList(ric);
    	
    	DomNode ricc = root.getNodeByName("race-icon-config");
    	if (ricc != null) raceIconConfig = new ImageConfigList(ricc);
    	
    	DomNode qpic = root.getNodeByName("qty-players-icon-config");
    	if (qpic != null) qtyPlayersIconConfig = new ImageConfigList(qpic);
    	
    	DomNode lic = root.getNodeByName("level-icon-config");
    	if (lic != null) levelIconConfig = new ImageConfigList(lic);
    	
    	DomNode gsic = root.getNodeByName("galaxy-size-icon-config");
    	if (gsic != null) galaxySizeIconConfig = new ImageConfigList(gsic);
    	
	}

	public static ImageConfig getImageConfigNoError(String name) {
		try {
			return getInstance().getImageConfig(name);
		} catch (ApplicationError ex) {
			Logger.logError("No image config for "+name, ex);
			return null;
		}
	}

	public static ImageConfig getConfigForImage(String name) throws ApplicationError {
		return getInstance().getImageConfig(name);
	}
	
	public static String getImageForConfig(String name) throws ApplicationError {
		ImageConfig imageConfig = getInstance().getImageConfig(name);
		if (imageConfig == null) {
			return null;
		} else {
			return imageConfig.getImage();
		}
	}

	public static Image getIconForConfig(String name) throws ApplicationError {
		String imageName = getImageForConfig(name);
		if (imageName == null) {
			return null;
		} else {
			if (MainWindow.hasInstance()) {
				return ImageCache.getImage(MainWindow.getInstance().getContainer().getToolkit(), imageName);
			} else {
				return ImageCache.getImage(null, imageName);
			}
		}
	}

	public static Image getIconForConfigNoError(String name) {
		try {
			return getIconForConfig(name);
		} catch (ApplicationError e) {
			return null;
		}
	}
	
	/**
	 * Maps the name of the image to the config details for that image.
	 * List of ImageConfig.
	 */
	private HashMap imageConfigs = new HashMap();
	
	/**
	 * This is the menu configuration for the game. This defines all the images used 
	 * on the menu.
	 */
	private MenuConfig menuConfig;
	
	/**
	 * This is the flag configuration for the game. This defines all the images used 
	 * on the for players flags.
	 */
	private ImageConfigList flagConfig;
	
	/**
	 * This is the race image configuration for the game. This defines all the images used 
	 * for players on the diplomacy and setup window.
	 */
	private ImageConfigList raceImageConfig;
	
	/**
	 * This is the race icon configuration for the game. This defines all the icons used 
	 * for players on the tabs of the diplomacy window.
	 */
	private ImageConfigList raceIconConfig;
	
	/**
	 * This is the list of images used to display the Quantity of Computer players
	 * combo box on the game setup window.
	 */
	private ImageConfigList qtyPlayersIconConfig;
	
	/**
	 * This is the list of images used to display the game level combo box on the game 
	 * setup window.
	 */
	private ImageConfigList levelIconConfig;
	
	/**
	 * This is the list of images used to display the galaxy size combo box on the game 
	 * setup window.
	 */
	private ImageConfigList galaxySizeIconConfig;
	
	
	public void setImageConfig(ImageConfig v) {
		imageConfigs.put(v.getName(), v);
	}
	
	public ImageConfig getImageConfig(String name) {
		ImageConfig r = (ImageConfig) imageConfigs.get(name);
		return r;
	}

	public ImageConfigList getImageConfigs() {
		if (imageConfigs.size() > 0) {
			Collection c = imageConfigs.values();
			ImageConfig[] ca = (ImageConfig[]) c.toArray(new ImageConfig[c.size()]);
			return new ImageConfigList(ca);
		} else {
			return null;
		}
	}

	public MenuConfig getMenuConfig() {
		return menuConfig;
	}

	public void setMenuConfig(MenuConfig menuConfig) {
		this.menuConfig = menuConfig;
	}
	
	public ImageConfigList getFlagConfig() {
		return flagConfig;
	}
	
	public ImageConfig getFlagConfig(String name) {
		return getFlagConfig().getImageConfig(name);
	}
	
	public void setFlagConfig(ImageConfigList flagConfig) {
		this.flagConfig = flagConfig;
	}

	public ImageConfigList getRaceImageConfig() {
		return raceImageConfig;
	}
	
	public void setRaceImageConfig(ImageConfigList raceImageConfig) {
		this.raceImageConfig = raceImageConfig;
	}
	
	public ImageConfig getRaceImageConfig(String raceName) {
		return getRaceImageConfig().getImageConfig(raceName);
	}

	public ImageConfigList getRaceIconConfig() {
		return raceIconConfig;
	}

	public void setRaceIconConfig(ImageConfigList raceIconConfig) {
		this.raceIconConfig = raceIconConfig;
	}

	public ImageConfig getRaceIconConfig(String raceName) {
		return getRaceIconConfig().getImageConfig(raceName);
	}

	public ImageConfigList getGalaxySizeIconConfig() {
		return galaxySizeIconConfig;
	}

	public void setGalaxySizeIconConfig(ImageConfigList galaxySizeIconConfig) {
		this.galaxySizeIconConfig = galaxySizeIconConfig;
	}

	public ImageConfigList getLevelIconConfig() {
		return levelIconConfig;
	}

	public void setLevelIconConfig(ImageConfigList levelIconConfig) {
		this.levelIconConfig = levelIconConfig;
	}

	public ImageConfigList getQtyPlayersIconConfig() {
		return qtyPlayersIconConfig;
	}

	public void setQtyPlayersIconConfig(ImageConfigList qtyPlayersIconConfig) {
		this.qtyPlayersIconConfig = qtyPlayersIconConfig;
	}


}
