/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.config;

import java.util.ArrayList;

import mot.common.bean.MotColor;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.DomNode;
import mot.common.xml.XMLInput;

/**
 * This is the details of an image that is used to animate an event. Typically this
 * image is moved, such as the fear animation that rises the image above the craft.
 * e.g. Fear
 */
public class MenuConfig implements XMLInput {

	/**
	 * This is a list of menu image configs (A menu is made up of many menu items
	 * each one having its own image config)
	 */
	private ArrayList menuImageConfigList;
	private transient MenuImageConfig [] _menuImageConfigList;

	/**
	 * This is the size of these images. All images MUST be the same size.
	 */
	private Size imageSize;
	
	/**
	 * The color to use as a background for the menu.
	 */
	private MotColor background;
	
	/**
	 * The color to use as a background when painting a tool tip for a menu item.
	 */
	private MotColor tipBackground;
	
	public MenuConfig() {
	}

	public MenuConfig(Size imageSize) {
		super();
		this.imageSize = imageSize;
	}

	public MenuConfig(DomNode n) throws ApplicationError {
		DomNode[] mia = n.getNodesByName("menu-image-config");
		if (mia != null) {
			for (int i = 0; i < mia.length; i++) {
				setMenuImageConfig(new MenuImageConfig(mia[i]));
			}
		}
		
		imageSize = n.getNodeValueByName_Size("image-size");
		background = n.getNodeValuesByName_Colour("background");
		tipBackground = n.getNodeValuesByName_Colour("tip-background");
	}

	public Size getImageSize() {
		return imageSize;
	}

	public void setImageSize(Size imageSize) {
		this.imageSize = imageSize;
	}
	
	public void setMenuImageConfig(MenuImageConfig v) {
		if (menuImageConfigList == null) menuImageConfigList = new ArrayList();
		menuImageConfigList.add(v);
		_menuImageConfigList = null;
	}
	
	public MenuImageConfig[] getMenuImageConfigs() {
		if (_menuImageConfigList == null) {
			_menuImageConfigList = (MenuImageConfig [])menuImageConfigList.toArray(new MenuImageConfig[menuImageConfigList.size()]);
		}
		return _menuImageConfigList;
	}

	public MenuImageConfig getMenuImageConfig(String name) {
		MenuImageConfig[] v = getMenuImageConfigs();
		for (int i = 0; i < v.length; i++) {
			if (Comp.same(name, v[i].getName())) {
				return v[i];
			}
		}
		return null;
	}

	public MotColor getTipBackground() {
		return tipBackground;
	}

	public void setTipBackground(MotColor tipBackground) {
		this.tipBackground = tipBackground;
	}

	public MotColor getBackground() {
		return background;
	}

	public void setBackground(MotColor background) {
		this.background = background;
	}
}
