/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.config;
import java.awt.Color;

import mot.common.xml.DomNode;
import mot.common.xml.XMLInput;

/**
 * This class holds configuration information for battle aninations.
 * The images that drive the animation in a battle are 'external' to the code.
 * Therefore I want to make it possible for a non programmer to create and configure 
 * how these images are used.
 * 
 * NOTE - some animations do have hard coded information, but, I hope this is the
 * exception not the rule.
 */
public class WeaponFireAnimationConfig implements XMLInput {

	/**
	 * The group name of the config. This is the name of the group for this config data.
	 * Usually each group of weapon uses a different type of animation.
	 */
	private String group;
	
	/**
	 * The name of the weapon this config refers to. The name does not include the group
	 * name it is the simple technology name
	 */
	private String name;
	
	/**
	 * This is the quantity of frames this animation stays alive
	 */
	private int life;
	
	/**
	 * This is the quantity of frames between this animation takes to move to the next
	 * stage (picture) of the animation
	 */
	private int interval;

	/**
	 * This is the weapons stroke used by this animation. This may be null. 
	 * NOTE - This is not part of the configuration! It is a cached value produced
	 * from the stroke information in this configuration so animations do not have 
	 * to create the same stroke over and over again!
	 */
//	/private WeaponStroke stroke;
// used to cache strokes but its a bad ides because they contain locations
	
	/**
	 * This is the name of the image to use for this stroke.
	 * Note this may be null - some strokes use lines or other custom configs.
	 */
	private String strokeImage;
	
	/**
	 * This is the colour used for this animation.
	 * Note this may be null - usually an animation will have a color or an image but
	 * I guess it is possible to have both or neither - but there are not any at the
	 * point of me writing this!
	 */
	private Color strokeColor;
	
	/**
	 * This is the size of the image for this stroke.
	 * Note this may be null - some strokes use lines or other custom configs.
	 * NB - 
	 */
	private StrokeSize strokeSize;
	

    /**
     * This is the gap between the start of an image and the start of the next image to be 
     * drawn in the stroke.
	 * Note this may be null - some strokes use lines or other custom configs.
     */
    private int strokeGap;
    
    /**
     * This is the quantity of pixels forward that the images will be drawn each iteration 
     * of animation. 
	 * Note this may be null - some strokes use lines or other custom configs.
     */
    private int strokeStep;

	public WeaponFireAnimationConfig() {
	}

	public int getInterval() {
		return interval;
	}

	public void setInterval(int interval) {
		this.interval = interval;
	}

	public int getLife() {
		return life;
	}

	public void setLife(int life) {
		this.life = life;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

//	public WeaponStroke getStroke() {
//		return stroke;
//	}
//
//	public void setStroke(WeaponStroke stroke) {
//		this.stroke = stroke;
//	}

	public int getStrokeGap() {
		return strokeGap;
	}

	public void setStrokeGap(int strokeGap) {
		this.strokeGap = strokeGap;
	}

	public String getStrokeImage() {
		return strokeImage;
	}

	public void setStrokeImage(String strokeImage) {
		this.strokeImage = strokeImage;
	}

	public StrokeSize getStrokeSize() {
		return strokeSize;
	}

	public void setStrokeSize(StrokeSize strokeSize) {
		this.strokeSize = strokeSize;
	}

	public int getStrokeStep() {
		return strokeStep;
	}

	public void setStrokeStep(int strokeStep) {
		this.strokeStep = strokeStep;
	}

	public WeaponFireAnimationConfig(String name, int life, int interval, String strokeImage, StrokeSize strokeSize, int strokeGap, int strokeStep) {
		this.name = name;
		this.life = life;
		this.interval = interval;
		this.strokeImage = strokeImage;
		this.strokeSize = strokeSize;
		this.strokeGap = strokeGap;
		this.strokeStep = strokeStep;
	}

	public WeaponFireAnimationConfig(DomNode n) {
		this.name = n.getNodeValueByName_String("name");
		this.group = n.getNodeValueByName_String("group");
		this.life = n.getNodeValueByName_Int("life");
		this.interval = n.getNodeValueByName_Int("interval");
		this.strokeImage = n.getNodeValueByName_String("stroke-image");
		this.strokeGap = n.getNodeValueByName_Int("stroke-gap");
		this.strokeStep = n.getNodeValueByName_Int("stroke-step");
		
		DomNode ssn = n.getNodeByName("stroke-size");
		if (ssn != null) this.strokeSize = new StrokeSize(ssn);
		
		String scn = n.getNodeValueByName_String("stroke-color");
		if (scn != null) setStrokeColor(scn);
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public Color getStrokeColorColor() {
		return strokeColor;
	}

	public void setStrokeColor(Color color) {
		this.strokeColor = color;
	}
	
	public void setStrokeColor(String rgbCommaList) {
		String [] rgb = rgbCommaList.split(",");
		this.strokeColor = new Color(Integer.parseInt(rgb[0]),
							   Integer.parseInt(rgb[1]),
							   Integer.parseInt(rgb[2]) );
	}

}
