/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.event.processors;

import java.util.ArrayList;

import mot.client.animation.BattleAnimation;
import mot.client.animation.BattleAnimationList;
import mot.client.animation.EventAnimation;
import mot.client.deligates.BattleDroidMineDelegate;
import mot.client.deligates.BattleMineDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.event.MotEventConsumer;
import mot.client.swing.animation.DelayAnimation;
import mot.client.swing.animation.ExplodeAnimation;
import mot.client.swing.animation.ExplodeAnimationFactory;
import mot.client.swing.animation.FlashAnimation;
import mot.client.swing.animation.ShieldBurstAnimation;
import mot.client.swing.animation.WarpCoreOverloadAnimation;
import mot.client.swing.config.AnimationConfig;
import mot.client.swing.window.battle.BattleWindowConfig;
import mot.client.swing.window.main.MainWindow;
import mot.common.event.BattleDamageRingEvent;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.LinkedEventBase;
import mot.common.event.MotEvent;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class DamageRingEventProcessor extends BattleSwingEventProcessor {

	public void process(MotEvent e, MotEventConsumer processor, LinkedEventBase parent) throws ApplicationError, ValidationError {
        BattleDamageRingEvent event = (BattleDamageRingEvent)e;
		MainWindow mainWindow = MainWindow.getInstance();
        BattleWindowConfig config = mainWindow.getBattleWinConfig();
        if (config != null && !hasRecievedEndBattleEvent(config)) {
        	BattleObjectDelegate source = DeligateCacheManager.getBattleObject(event.getBattle(), event.getSource());
            BattleAnimationList al = config.getListener().getState().getAnimations();
            if (!al.lastAnimationIsExplodeFor(source, 0)) {
            	al.add( new DelayAnimation() );
            }
            if (source instanceof BattleMineDelegate) {
            	BattleMineDelegate mine = (BattleMineDelegate) source;
            	int life = AnimationConfig.warpCoreOverloadMineFlashing();
                al.add(new FlashAnimation(mainWindow, mine, mine.getLocation(), mine.getDirection(), life));
            }
            if (source instanceof BattleDroidMineDelegate) {
            	BattleDroidMineDelegate mine = (BattleDroidMineDelegate) source;
            	int life = AnimationConfig.warpCoreOverloadMineFlashing();
                al.add(new FlashAnimation(mainWindow, mine, mine.getLocation(), mine.getDirection(), life));
            }
            ShieldBurstAnimation sba = null;
            Weapon weapon = event.getWeaponTechnology();
            if (weapon != null && weapon.isShieldBurst()) {
				sba = new ShieldBurstAnimation(event, mainWindow, weapon);
				al.add( sba );
            } else {            	
            	//removeRedundantExplodeAnimation(al, source);
            	al.add( new WarpCoreOverloadAnimation(event, mainWindow) );
            }
            MotEvent[] links = event.getLinkedEvents();
            if (links != null) {
                for (int i=0; i<links.length; i++) {
                	if (sba != null) sba.addDamagedBattleObject(links[i]);
                    if (links[i] instanceof BattleDestroyedEvent) {
                    	BattleDestroyedEvent cde = (BattleDestroyedEvent)links[(i)];
                    	EventAnimation dea = ExplodeAnimationFactory.create(cde, al, false);
                    } else {
                    	processor.process(links[i], event);                    	
                    }
                }
            }
            al.add( new DelayAnimation() );
        }
	}

	/**
	 * Remove from the list of animations any destroy animations for the source object of this warp core overload
	 * This may well have exploded from damage inflicted by another warp core overload OR weapon fire that has added a destroy animation for this source.
	 * This animation will explode the source before the warp core overload! 
	 */
	private void removeRedundantExplodeAnimation(BattleAnimationList al, BattleObjectDelegate source) {
		ArrayList list = al.getList();
		if (list != null) {
			for (Object o : list) {
				if (o instanceof ExplodeAnimation) {
					ExplodeAnimation ea = (ExplodeAnimation) o;
					if (Comp.same(ea.getTarget().getName(), source.getName())) {
						BattleAnimation p = ea; 
						int i = list.indexOf(p);
						while (i > 0 && p != null) {
							i--;
							p = (BattleAnimation) list.get(i);
							if (p.isExclusive()) {
								return;
							} else if (p instanceof WarpCoreOverloadAnimation) {
								list.remove(ea);
								if (Logger.isDebug()) {
									Logger.logDebug("Remove ExplodeAnimation for "+source.getName());
								}
								return;
							}
						}
					}
				}
			}
		}
	}

}
