/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.icons;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.ImageObserver;

import javax.swing.ImageIcon;

import mot.common.bean.Size;

public class ScaledImageIcon extends ImageIcon {

    private static final int DEFAULT_ICON_SIZE = 80;
    
    /**
     * The size this image is when displayed inside this icon. 
     * Must be smaller than iconSize.
     */
	private Size size;
	
	/**
	 * This is the size of the icon containing the image within it (centred).
	 */
	private Size iconSize;

    public ScaledImageIcon(Image image, Size size) {
        super(image);
        this.size = size;
    }

    /**
     * This is a click able image centred inside the icon.
     * @param image The image to display in this icon
     * @param size	The size this image is when displayed inside this icon. 
     * 				Must be smaller than iconSize.
     * @param iconSize This is the size of the icon containing the image within it (centred).
     */
    public ScaledImageIcon(Image image, Size size, Size iconSize) {
        super(image);
        this.size = size;
		this.iconSize = iconSize;
    }

    public synchronized void paintIcon(Component c, Graphics g, int x, int y) {
        ImageObserver imageObserver = getImageObserver();
        Image image = getImage();
        int width = getIconWidth();
        int height = getIconHeight();
        int spareWidth = width - size.getX();
        int spareHeight = height - size.getY();
        int xd = (spareWidth == 0) ? 0 : spareWidth/2;
        int yd = (spareHeight == 0) ? 0 : spareHeight/2;
        if(imageObserver == null) {
           g.drawImage(image, x+xd, y+yd, size.getX(), size.getY(), c);
        } else {
           g.drawImage(image, x+xd, y+yd, size.getX(), size.getY(), imageObserver);
        }
    }

    public Size getSize() {
        return size;
    }

    public void setSize(Size size) {
        this.size = size;
    }
    
    public int getIconWidth() {
    	if (iconSize != null) {
    		return iconSize.getX();
    	} else {
    		return DEFAULT_ICON_SIZE;
    	}
    }

    public int getIconHeight() {
    	if (iconSize != null) {
    		return iconSize.getY();
    	} else {
    		return DEFAULT_ICON_SIZE;
    	}
    }
}