/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.labels;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.config.MenuConfig;
import mot.client.swing.window.common.MessagePanel;
import mot.client.swing.window.main.MainWindow;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

/**
 * A JLabel that is used to display the name of a technology on the screen.
 */
public class TechnologyLabel extends JLabel implements MouseListener, MouseMotionListener {

    /**
     * The listener to send mouse click events to when this technology label is clicked on.
     */
    protected TechnologyLabelClickListener clickListener;
    
    /**
     * The technology this label is displaying on the screen.
     */
    private Technology technology;

    
    /**
     * If this is true when a user overs the mouse over this label the description of the
     * technology appears in the message list of the main window.
     */
    private boolean displayDetailsInMessagePanel;
    private boolean displayDetailsToolTip;

	/**
	 * This is true when the mouse is inside the component.
	 * Use to roll over tipping of information!
	 */
	private boolean rollover;

	private static Color tipBackground;

	
	static {
		try {
			MenuConfig menuConfig = ImageConfigFactory.getInstance().getMenuConfig();
			tipBackground = menuConfig.getTipBackground().getColor();
		} catch (Exception e) {
			Logger.logError("", e);
		}
	}

	/**
     * Creates a JLabel that displays text on the screen (in the label) and calls clickListener
     * when this technology is clicked on passing the click listener the technology.
     * @param clickListener The mouse click listener to inform when this label is clicked on
     * @param technology The technology this text field represents
     * @param text The text to display on the screen
     */
    public TechnologyLabel(String text, Technology pTechnology, TechnologyLabelClickListener pClickListener, boolean displayDetailsInMessagePanel, boolean displayDetailsToolTip) {
        super(text);
        this.technology = pTechnology;
        this.clickListener= pClickListener;
        this.displayDetailsInMessagePanel = displayDetailsInMessagePanel;
		this.displayDetailsToolTip = displayDetailsToolTip;
        addMouseListener(this);
        addMouseMotionListener(this);
        setVerticalAlignment(SwingConstants.TOP);
        if (displayDetailsToolTip && technology != null) {
        	String description = technology.getDescription();
			setToolTipText(MessagePanel.toHtml(description, 100, false));
        }
    }

    /**
     * Creates a JLabel that displays the name of this technology on the screen (in the label) 
     * and calls clickListener when this technology is clicked on passing the click listener 
     * the technology.
     * @param clickListener The mouse click listener to inform when this label is clicked on
     * @param technology The technology this text field represents
     */
    public TechnologyLabel(Technology technology, TechnologyLabelClickListener clickListener, boolean displayDetailsInMessagePanel, boolean displayDetailsToolTip) {
        super(technology.getName());
        this.technology = technology;
        this.clickListener= clickListener;
        this.displayDetailsInMessagePanel = displayDetailsInMessagePanel;
		this.displayDetailsToolTip = displayDetailsToolTip;
        addMouseListener(this);
        setVerticalAlignment(SwingConstants.TOP);
    }

    /**
     * Creates an empty JLabel. The method setTechnology(String text, Technology tech) must be called 
     * before this label will display anything. It calls clickListener when this technology is clicked 
     * on passing the click listener the technology.
     * @param clickListener The mouse click listener to inform when this label is clicked on
     */
    public TechnologyLabel(TechnologyLabelClickListener clickListener, boolean displayDetailsInMessagePanel, boolean displayDetailsToolTip) {
        this.clickListener= clickListener;
        this.displayDetailsInMessagePanel = displayDetailsInMessagePanel;
		this.displayDetailsToolTip = displayDetailsToolTip;
        addMouseListener(this);
    }

	public void setTechnology(Technology technology) {
        this.technology = technology;
        if (technology == null) {
        	setText(null);
        	setToolTipText(null);
        	return;
        }
        setText(technology.getName());
        if (displayDetailsToolTip && technology != null) {
        	setToolTipText(technology.getDescription());
        }
    }

    public void setTechnology(String text, Technology technology) {
        setText(text);
        this.technology = technology;
        if (displayDetailsToolTip && technology != null) {
        	setToolTipText(technology.getDescription());
        }
    }

	public void mouseClicked(MouseEvent e) {   }

    public void mousePressed(MouseEvent e) {
    	if (e.getClickCount() == 1) {
            if (clickListener == null) return;
            if (e.getButton() == MouseEvent.BUTTON1) {
                clickListener.leftClick(technology);
            } else if (e.getButton() == MouseEvent.BUTTON3) {
                clickListener.rightClick(technology);
            }
    	}
    }
    
    public void mouseReleased(MouseEvent e) { }
    
	public void mouseEntered(MouseEvent e) {
		rollover = true;
	}

	public void mouseExited(MouseEvent e) {
		rollover = false;
	}

    
	public boolean containsMouse() {
		return rollover;
	}
	
	public void mouseDragged(MouseEvent e) {
	}

    public Timer timer;

    public void mouseMoved(MouseEvent e) {
    	if (!displayDetailsInMessagePanel || technology == null) {
    		return;
    	}
		if (timer == null) {
	    	ActionListener tl = new ActionListener() {
				public void actionPerformed(ActionEvent e) {
			    	if (Logger.isDebug()) {
			    		if (!SwingUtilities.isEventDispatchThread()) {
			    			Logger.logError("", new Exception("NOT IN EVENT THREAD"));
			    		}
			    	}
					timer.stop();
					try {
						MainWindow mainWin = MainWindow.getInstance();
						if (containsMouse()) {
							String text = technology.getDescription();
							mainWin.showTemporaryMessage(text, false, tipBackground, 0);
						}
					} catch (ApplicationError ex) {
						Logger.logError("Unable to display tip", ex);
					}
				}
	    	};
			timer = new Timer(1000, tl);
			timer.start();
		} else {
			timer.restart();
		}
	}
    
    public String toString() {
    	return getText();
    }

	public void changeFontSize(Integer labelTextSizeChange, boolean bold) {
        Font font = getFont();
        int style = font.getStyle();
        if (bold) {
            style = Font.BOLD;
        }
		setFont(new Font(font.getName(), style, font.getSize()+labelTextSizeChange));
		
	}


}