/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.lists;

import java.awt.Color;
import java.awt.event.MouseListener;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.border.LineBorder;

import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PadPanel;
import mot.common.bean.MotComparable;
import mot.common.util.PropertyManager;

public class FakeList extends ComponentTable {
	
	private MouseListener mouseListener;
	
	private ListCellRenderer renderer;
	
	private Object[] content;
	private Object selection;
	
	private static Color selectedBackgroundColour = null;
	public static Color getSelectedBackgroundColour() {
		if (selectedBackgroundColour == null) {
			selectedBackgroundColour = PropertyManager.getMandatoryAsColorNoError("list.color.background.selected");
		}
		return selectedBackgroundColour;
	}
	
	private static Color listBackgroundColour = null;
	public static Color getListBackgroundColour() {
		if (listBackgroundColour == null) {
			listBackgroundColour = PropertyManager.getMandatoryAsColorNoError("list.color.background");
		}
		return listBackgroundColour;
	}

	public Color getSelectedOutlineColour() {
		return Color.WHITE;
	}


    public FakeList() {
    	super("HT", 0,1, 0,0,0);
    	setOpaque(true);
    }
    
    public FakeList(ListCellRenderer renderer) {
    	this();
    	this.renderer = renderer;
	}

    public FakeList(Object [] content, Object selection, ListCellRenderer renderer) {
    	this();
    	this.renderer = renderer;
		setContent(content, selection);
    }
    
	public void clearContent() {
    	setQtyRows(0);
        removeAll();
        layoutListPanel();
    }

    public void setContent(JComponent content) {
    	JComponent [] pa = new JComponent[1];
    	pa[0] = content;
    	setContent(pa, null);
    }
    
    public void setContent(Object [] content, int selectionIndex) {
    	if (content != null && content.length > 0) {
        	setContent(content, content[0]);
    	} else {
        	setContent(content, null);
    	}
    }
    
    public void setContent(Object [] content, Object selection) {
    	this.content = content;
    	this.selection = selection;
    	
        if (content == null) {
        	String emptyContentMessage = getEmptyContentMessage();
        	if (emptyContentMessage != null) {
                removeAll();
            	setQtyRows(1);
            	reset();
                add(new JLabel(emptyContentMessage), "HTS");
                layoutListPanel();
        	} else {
                removeAll();
            	layoutListPanel();
        	}
        	return;
        }
        
        removeAll();
    	setQtyRows(content.length);
    	reset();
    	
    	for (int i=0; i<content.length; i++) {
    		JComponent line = null;
    		if (content[i] instanceof JComponent){
	    		boolean selected = false;
	    		if (selection != null) {	    			
	    			if (content[i] instanceof MotComparable && selection instanceof MotComparable) {
	    				selected = ((MotComparable)content[i]).same((MotComparable)selection);
	    			} else {
	    				selected = (selection == content[i]);
	    			}
	    		}
    			line = (JComponent) content[i];
    			if (selected) {
    				Color sbc = getSelectedBackgroundColour();
    				if (sbc != null) line.setBackground(sbc);
    				Color soc = getSelectedOutlineColour();
    				if (soc != null) line.setBorder(new LineBorder(soc));
    			} else {
    				line.setBackground(listBackgroundColour);
    				line.setBorder(null);
    			}
    		} else if (renderer == null) {
	        	line = new JLabel(content[i].toString());
	    	} else {
	    		boolean selected = (selection == content[i]);
	    		line = (JComponent) renderer.getListCellRendererComponent(null, content[i], i, selected, false);
	    	}
    		if (mouseListener != null) line.addMouseListener(mouseListener);
    		//if (Logger.isDebug()) Logger.logDebug("FakeLabelList - AddLine");
        	if (i == content.length-1) {
                add(line, "HTS");
        	} else {
                add(line);
        	}
    	}
    	
		//if (Logger.isDebug()) Logger.logDebug("FakeLabelList - Layout");
    	layoutListPanel();
    }

	private void layoutListPanel() {
		if (scrollPane != null) {
			scrollPane.validate();
		} else if (getParent() == null) {
			validate();
		} else {
			getParent().validate();
		}
	}
    
    
    
    protected String getEmptyContentMessage() {
        return null;
    }

    
    public void emptyContent() {
        removeAll();
    	setQtyRows(0);
    	validate();
    }

    public Object [] getContent() {
    	return content;
    }
    
    public void doLayout() {
        if (getComponentCount() > 0) { 
            super.doLayout();
        }
    }
    
    private JScrollPane scrollPane;
    
    public JScrollPane getScrollPane() {
        if (scrollPane == null) {
            scrollPane = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            if (mouseListener != null) {
            	scrollPane.addMouseListener(mouseListener);
            	scrollPane.getViewport().addMouseListener(mouseListener);
            }
        }
        return scrollPane;
    }

    public JScrollPane getScrollPane(int padding) {
        if (scrollPane == null) {
            scrollPane = new JScrollPane(new PadPanel(this, padding), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        }
        return scrollPane;
    }

    public JScrollPane getScrollPaneNoBorder() {
        if (scrollPane == null) {
            scrollPane = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            scrollPane.setBorder(null);
        }
        return scrollPane;
    }

    public JScrollPane getScrollPaneNoBorder(int padding) {
        if (scrollPane == null) {
            scrollPane = new JScrollPane(new PadPanel(this, padding), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            scrollPane.setBorder(null);
        }
        return scrollPane;
    }

	public ListCellRenderer getRenderer() {
		return renderer;
	}

	public void setRenderer(ListCellRenderer renderer) {
		this.renderer = renderer;
	}

	public Object getSelection() {
		return selection;
	}

	public void setSelection(Object selection) {
		setContent(content, selection);
	}


}