/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.lists;

import java.awt.Point;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.swing.common.ComponentTable;
import mot.common.util.ApplicationError;

/**
 * Displays a list of MOOs (crafts, fleets) in a panel with a scroll bar.
 * Crafts may be selected in the list by clicking on them of double 
 * clicking on them.
 * 
 * This is used on the Galaxy Window to display the list of crafts in the
 * MOO stack selected in the galaxy by the user.
 */
public class FakeMooList extends ComponentTable {
	
	private class ListItem {
		public MoveableOwnedObjectDelegate moo;
		public FakeMooListMooPanel panel;
		public ListItem(MoveableOwnedObjectDelegate moo, FakeMooListMooPanel panel) {
			this.moo = moo;
			this.panel = panel;
		}
	}
	
	private ListItem [] moos;
	private MooListOwner owner;
	
	private MoveableOwnedObjectDelegate selection;
	
	public FakeMooList(MooListOwner owner) throws ApplicationError {
		super("", 1,1, 0,0,0);
		setOwner(owner);
	}

	public void clearMoos() throws ApplicationError {
		removeAll();
		moos = null;
		getScrollPane().validate();
		getScrollPane().repaint();
	}

	public void setMoos(MoveableOwnedObjectDelegate[] moos, MoveableOwnedObjectDelegate selection) throws ApplicationError {
		if (moos != null) {
			setQtyRows(moos.length+1);
	        removeAll();
			this.moos = new ListItem[moos.length];
			for (int i=0; i < moos.length; i++ ) {
				FakeMooListMooPanel panel = new FakeMooListMooPanel(moos[i], owner);
                add(panel, "HV");
				this.moos[i] = new ListItem(moos[i], panel);
			}
			add(new JPanel(), "HVS");
			revalidate();
			//getScrollPane().validate();
			//getScrollPane().revalidate();
		}
		setSelection(selection);
		//getScrollPane().repaint();
	}
	
	public MooListOwner getOwner() {
		return owner;
	}

	public void setOwner(MooListOwner owner) {
		this.owner = owner;
	}

	public MoveableOwnedObjectDelegate[] getContent() {
        if (moos != null && moos.length > 0) {
        	MoveableOwnedObjectDelegate[] r = new MoveableOwnedObjectDelegate[moos.length];
        	for (int i = 0; i < r.length; i++) {
				r[i] = moos[i].moo;
			}
        	return r;
        } else {
        	return null;
        }
	}

	public boolean contains(MoveableOwnedObjectDelegate moo) {
        if (moos != null && moos.length > 0) {
            for (int i=0; i<moos.length; i++) {
                if (moos[i].moo.equals(moo)) {
                	return true;
                }
            }
        }
        return false;
	}	

	public MoveableOwnedObjectDelegate getNextSelectionInList(MoveableOwnedObjectDelegate moo) throws ApplicationError {
		if (moo == null) return null;
        if (moos != null && moos.length > 1) {
            for (int i=0; i<moos.length; i++) {
                if (moos[i].moo.equals(moo)) {
                	return _getNextInList(i+1, moo.getOwnerDelegate());
                }
            }
        }
        return null;
	}

	private MoveableOwnedObjectDelegate _getNextInList(int index, PlayerDelegate owner) throws ApplicationError {
		int i = index;
        while (i<moos.length) {
            if (moos[i].moo.isOwnedBy(owner)) {
            	return moos[i].moo;
            }
            i++;
        }
		i = 0;
        while (i<index) {
            if (moos[i].moo.isOwnedBy(owner)) {
            	return moos[i].moo;
            }
            i++;
        }
        return null;
	}
	
	public MoveableOwnedObjectDelegate getSelection() {
		return selection;
	}

	public void setSelection(MoveableOwnedObjectDelegate selection) {
		this.selection = selection;
        if (moos != null && moos.length > 0) {
            for (int i=0; i<moos.length; i++) {
                boolean selected = moos[i].moo.equals(selection);
               	moos[i].panel.setSelected(selected);
                if (selected) {
                	scrollToVisible(i, selection);
                }
            }
        }
	}
	
	private void scrollToVisible(int i, MoveableOwnedObjectDelegate selection) {
        if (selection != null) {
        	JViewport vp = getScrollPane().getViewport();
        	java.awt.Rectangle actual = moos[i].panel.getBounds();
        	java.awt.Rectangle visible = vp.getViewRect();
        	int actualBottom = actual.y+actual.height;
        	int visibleBottom = visible.y + visible.height;
        	if (actualBottom > visibleBottom) {
        		int y = visible.y + (actualBottom - visibleBottom) ;
        		vp.setViewPosition(new Point(visible.x, y));
        	} else if (actual.y <= visible.y) {
        		int y = actual.y;
        		vp.setViewPosition(new Point(visible.x, y));
        	}
        }
	}

	public void remove(MoveableOwnedObjectDelegate moo) throws ApplicationError {
		if (selection != null && selection.equals(moo)) {
			selection = null;
		}
        if (moos != null && moos.length > 0) {
        	int index = -1;
            for (int i=0; i<moos.length; i++) {
               	if (moos[i].moo.equals(moo)) index = i;
            }
            if (index >= 0) {
            	MoveableOwnedObjectDelegate [] newMoos = new MoveableOwnedObjectDelegate [moos.length-1];
                for (int i=0, j = 0; i<moos.length; i++) {
                	if (!moos[i].moo.equals(moo)) {
                		newMoos[j++] = moos[i].moo;
                	}
                }
                MoveableOwnedObjectDelegate newSelection = (index < newMoos.length) ? newMoos[index] : null;
                setMoos(newMoos, newSelection);
            }
        }
        getScrollPane().validate();
        getScrollPane().repaint();
	}

	/**
	 * Add a new MOO to the end of this list without losing the currently selected craft. 
	 * @param moo The new MOO to add to this list!
	 */
	public void add(MoveableOwnedObjectDelegate moo) throws ApplicationError {
        if (moo != null) {
        	boolean found = false;
            for (int i=0; i<moos.length; i++) {
               	if (moos[i].moo.equals(moo)) found = true;
            }
            if (!found) {
            	MoveableOwnedObjectDelegate [] newMoos = new MoveableOwnedObjectDelegate [moos.length+1];
                for (int i=0; i<moos.length; i++) {
               		newMoos[i] = moos[i].moo;
                }
                newMoos[moos.length] = moo;
                setMoos(newMoos, selection);
            }
        }
	}

	private JScrollPane scrollPane;
    public JScrollPane getScrollPane() {
    	if (scrollPane == null) {
        	scrollPane =  new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    	}
    	return scrollPane;
    }
}
