/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.lists;

import java.awt.Color;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.window.main.MainWindow;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;

/**
 * Displays a list of panels in a panel with a scroll bar aligned to the
 * top just like a list. This behaves like a JList but is a made construct
 * out of a component table!
 */
public class FakePanelList extends ComponentTable {
	
	private Color even;
	private Color odd;
	private JPanel [] items;
	private JPanel hoverItem;
	private Color hoverOriginal;
	private FakePanelListClickListener clickListener;
	private boolean useHover = true;
	private static Color hover = PropertyManager.getMandatoryAsColorNoError("list.color.background.hover");

	public FakePanelList(FakePanelListClickListener clickListener, boolean useBackground) throws ApplicationError {
		this();
		this.clickListener = clickListener;
		if (useBackground) {
			even = PropertyManager.getMandatoryAsColor("list.color.background.even");
			odd = PropertyManager.getMandatoryAsColor("list.color.background.odd");
		}
	}

	public FakePanelList(Color even, Color odd, boolean useHover) throws ApplicationError {
		this();
		this.even = even;
		this.odd = odd;
		this.useHover = useHover;
	}

	public FakePanelList() throws ApplicationError {
		super("HST", 1,1, 0,0,0);
	}

	public void clear() throws ApplicationError {
		removeAll();
		items = null;
		getScrollPane().validate();
		getScrollPane().repaint();
	}

	public void setContent(JPanel[] contents) throws ApplicationError {
		if (contents != null) {
			removeAll();
			setQtyRows(contents.length);
	        items = null;
			for (int i=0; i < contents.length; i++ ) {
				if (i == contents.length-1) {
	                add(contents[i], "HST");
				} else {
	                add(contents[i], "HV");
				}
				if (i%2 == 0) {
					if (even != null) {
						contents[i].setBackground(even);
						contents[i].setOpaque(true);
					}
				} else {
					if (odd != null) {
						contents[i].setBackground(odd);
						contents[i].setOpaque(true);
					}
				}
				new ItemMouseListener(contents[i]);
			}
			getScrollPane().validate();
		} else {
			removeAll();
			setQtyRows(0);
		}
		getScrollPane().repaint();
	}
	
	private void scrollToVisible(int i, MoveableOwnedObjectDelegate selection) {
        if (selection != null) {
        	JViewport vp = getScrollPane().getViewport();
        	java.awt.Rectangle actual = items[i].getBounds();
        	java.awt.Rectangle visible = vp.getViewRect();
        	int actualBottom = actual.y+actual.height;
        	int visibleBottom = visible.y + visible.height;
        	if (actualBottom > visibleBottom) {
        		int y = visible.y + (actualBottom - visibleBottom) ;
        		vp.setViewPosition(new Point(visible.x, y));
        	} else if (actual.y <= visible.y) {
        		int y = actual.y;
        		vp.setViewPosition(new Point(visible.x, y));
        	}
        }
	}

	public void remove(JPanel item) throws ApplicationError {
        if (items != null && items.length > 0) {
        	int index = -1;
            for (int i=0; i<items.length; i++) {
               	if (items[i] == item) index = i;
            }
            if (index >= 0) {
            	JPanel [] newItems = new JPanel [items.length-1];
                for (int i=0, j = 0; i<items.length; i++) {
                	if (items[i] != item) {
                		newItems[j++] = items[i];
                	}
                }
                setContent(newItems);
            }
        }
        getScrollPane().validate();
        getScrollPane().repaint();
	}

	public void add(JPanel item) throws ApplicationError {
        if (item != null) {
        	boolean found = false;
            for (int i=0; items != null && i<items.length; i++) {
               	if (items[i]== item) found = true;
            }
            if (!found) {
            	JPanel [] newItems = new JPanel [items.length+1];
                for (int i=0; i<items.length; i++) {
                	newItems[i] = items[i];
                }
                newItems[items.length] = item;
                setContent(newItems);
            }
        }
	}
	
	private JScrollPane scrollPane;
    public JScrollPane getScrollPane() {
    	if (scrollPane == null) {
        	scrollPane =  new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    	}
    	return scrollPane;
    }
    public JScrollPane getScrollPane(boolean hasVerticalSB, boolean hasHorizontalSB) {
    	if (scrollPane == null) {
        	scrollPane =  new JScrollPane(this, 
					  					  hasVerticalSB ? JScrollPane.VERTICAL_SCROLLBAR_ALWAYS : JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
					  					  hasHorizontalSB ? JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS : JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
    	}
    	return scrollPane;
    }


    class ItemMouseListener implements MouseListener {
    	
    	private JPanel listItem;
    	
    	public ItemMouseListener(JPanel listItem) {
    		this.listItem = listItem;
    		listItem.addMouseListener(this);
    	}
    	
    	public void mouseClicked(MouseEvent e) { 
    	}
    	
    	public void mousePressed(MouseEvent e) {
        	if (e.getClickCount() == 1) {
        		if (clickListener != null) {
    				try {
    					clickListener.click(listItem);
    	    		} catch (ValidationError ex) {
    	    			MainWindow.getExestingInstance().reportValidationError(ex);
    	    		} catch (Exception ex) {
    	    			MainWindow.getExestingInstance().reportApplicationError(ex);
    				}
        		}
        	}
    	}
    	public void mouseReleased(MouseEvent e) {	}

    	public void mouseEntered(MouseEvent e) {
    		if (useHover && hover != null && listItem.getBackground() != hover) {
    			if (hoverItem != null) {
    				hoverItem.setBackground(hoverOriginal);
    				hoverItem.repaint();
    			}
    			hoverOriginal = listItem.getBackground();
    			listItem.setBackground(hover);
    			listItem.repaint();
    			hoverItem = listItem;
    		}
    	}
    	
    	public void mouseExited(MouseEvent e) {
    		Point mp = listItem.getMousePosition();
    		if (mp == null && hoverOriginal != null) {
    			listItem.setBackground(hoverOriginal);
    			listItem.repaint();
    		}
    	}
    }

}

