/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.lists;

import java.awt.Color;
import java.awt.Point;

import javax.swing.JScrollPane;
import javax.swing.JViewport;

import mot.client.swing.common.ComponentTable;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;

/**
 * Displays a list of panels in a panel with a scroll bar aligned to the
 * top just like a list. This behaves like a JList but is a made construct
 * out of a component table!
 */
public class FakePanelSelectableList extends ComponentTable {
	
	private Color even;
	private Color odd;

	private FakePanelListItem [] items;
	private FakePanelListItem selection;
	
	public FakePanelSelectableList(boolean useBackground) throws ApplicationError {
		this();
		if (useBackground) {
			even = PropertyManager.getMandatoryAsColor("list.color.background.even");
			odd = PropertyManager.getMandatoryAsColor("list.color.background.odd");
		}
	}

	public FakePanelSelectableList(Color even, Color odd) throws ApplicationError {
		this();
		this.even = even;
		this.odd = odd;
	}

	public FakePanelSelectableList() throws ApplicationError {
		super("HST", 1,1, 0,0,0);
	}

	public void clear() throws ApplicationError {
		removeAll();
		items = null;
		getScrollPane().validate();
		getScrollPane().repaint();
	}

	public void setContent(FakePanelListItem[] contents, FakePanelListItem selection) throws ApplicationError {
		if (contents != null) {
			setQtyRows(contents.length);
	        removeAll();
	        items = new FakePanelListItem[contents.length];
			for (int i=0; i < contents.length; i++ ) {
				if (i == contents.length-1) {
	                add(contents[i].getPanel(), "HST");
				} else {
	                add(contents[i].getPanel(), "HV");
				}
				if (i%2 == 0) {
					if (even != null) {
						contents[i].getPanel().setBackground(even);
						contents[i].getPanel().setOpaque(true);
					}
				} else {
					if (odd != null) {
						contents[i].getPanel().setBackground(odd);
						contents[i].getPanel().setOpaque(true);
					}
				}
			}
			getScrollPane().validate();
		}
		setSelection(selection);
		getScrollPane().repaint();
	}
	
	public FakePanelListItem getSelection() {
		return selection;
	}

	public void setSelection(FakePanelListItem selection) {
		this.selection = selection;
        if (items != null && items.length > 0) {
            for (int i=0; i<items.length; i++) {
                boolean selected = items[i].equals(selection);
                items[i].setSelected(selected);
                if (selected) {
                	scrollToVisible(i, selection);
                }
            }
        }
	}
	
	private void scrollToVisible(int i, FakePanelListItem selection) {
        if (selection != null) {
        	JViewport vp = getScrollPane().getViewport();
        	java.awt.Rectangle actual = items[i].getPanel().getBounds();
        	java.awt.Rectangle visible = vp.getViewRect();
        	int actualBottom = actual.y+actual.height;
        	int visibleBottom = visible.y + visible.height;
        	if (actualBottom > visibleBottom) {
        		int y = visible.y + (actualBottom - visibleBottom) ;
        		vp.setViewPosition(new Point(visible.x, y));
        	} else if (actual.y <= visible.y) {
        		int y = actual.y;
        		vp.setViewPosition(new Point(visible.x, y));
        	}
        }
	}

	public void remove(FakePanelListItem item) throws ApplicationError {
		if (selection != null && selection.equals(item)) {
			selection = null;
		}
        if (items != null && items.length > 0) {
        	int index = -1;
            for (int i=0; i<items.length; i++) {
               	if (items[i].equals(item)) index = i;
            }
            if (index >= 0) {
            	FakePanelListItem [] newItems = new FakePanelListItem [items.length-1];
                for (int i=0, j = 0; i<items.length; i++) {
                	if (!items[i].equals(item)) {
                		newItems[j++] = items[i];
                	}
                }
                FakePanelListItem newSelection = (index < newItems.length) ? newItems[index] : null;
                setContent(newItems, newSelection);
            }
        }
        getScrollPane().validate();
        getScrollPane().repaint();
	}

	public void add(FakePanelListItem item) throws ApplicationError {
        if (item != null) {
        	boolean found = false;
            for (int i=0; i<items.length; i++) {
               	if (items[i].equals(item)) found = true;
            }
            if (!found) {
            	FakePanelListItem [] newItems = new FakePanelListItem [items.length+1];
                for (int i=0; i<items.length; i++) {
               		newItems[i] = items[i];
                }
                newItems[items.length] = item;
                setContent(newItems, selection);
            }
        }
	}
	
	private JScrollPane scrollPane;
    public JScrollPane getScrollPane() {
    	if (scrollPane == null) {
        	scrollPane =  new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    	}
    	return scrollPane;
    }
}
