/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.lists;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.deligates.PlanetDelegate;
import mot.client.swing.buttons.MotAction;
import mot.client.swing.buttons.PlanetSelectionListener;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.solarsystem.PlanetAnimationPanelWithDescription;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

/**
 * This is a list of planets which renders planets containing a picture and a very brief
 * description of the planet including terraform level, size, maximum population and any
 * bonuses such as rich or fertile this planet has.
 * 
 * This list if used by the solar system window.
 */
public class PlanetList extends FakeList {

	private JPanel [] planetPanels = null;
	private PlanetSelectionListener listener;
	
    public PlanetList(MotInternalWindow owner, PlanetDelegate[] planets, PlanetSelectionListener listener) throws ApplicationError {
    	this.listener = listener;
    	
		ArrayList<JPanel> panels = null;
    	PlanetAnimationPanelWithDescription first = null;
    	
        if (planets != null) {
            for (int i=0; i<planets.length; i++) {
            	PlanetAnimationPanelWithDescription planetPanel = new PlanetAnimationPanelWithDescription(owner, planets[i]);
            	planetPanel.setOpaque(true);
            	if (panels == null) {
            		panels = new ArrayList();
            		first = planetPanel;
            	}
            	PanelUtils.addCascadingMouseListener(planetPanel, new PlanetListSelectionListener(planetPanel));
            	panels.add(new PadPanel(70,1));
				panels.add( planetPanel );
            }
        }
        
        if (panels != null) {
        	planetPanels = panels.toArray(new JPanel[panels.size()]);
        	setContent(planetPanels, first);
        }
    }

    public JScrollPane getInScrollPane() {
        return new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    }

	public PlanetDelegate getSelectedPlanet() {
		Object selection = getSelection();
		if (selection != null && selection instanceof PlanetAnimationPanelWithDescription) {
			PlanetAnimationPanelWithDescription planetPanel = (PlanetAnimationPanelWithDescription) selection;
			return planetPanel.getPlanet();
		}
		return null;
	}

	public void selectPlanet(PlanetDelegate planetToSelect) throws ApplicationError, ValidationError {
		Object[] content = getContent();
		if (content == null) return;
		for (int i = 0; i < content.length; i++) {
			if (content[i] instanceof PlanetAnimationPanelWithDescription) {
				PlanetAnimationPanelWithDescription planetPanel = (PlanetAnimationPanelWithDescription) content[i];
				if (planetPanel.getPlanet() == planetToSelect) {
					selectPlanet(planetPanel);
				}
			}
		}
	}

    private void selectPlanet(PlanetAnimationPanelWithDescription planetPanel) throws ApplicationError, ValidationError {
		setContent(planetPanels, planetPanel);
		if (listener != null) {
			listener.select(planetPanel.getPlanet());
		}
	}
    
	public Color getSelectedOutlineColour() {
		return null;
	}

	public void handleTick() throws ApplicationError {
		if (planetPanels != null) {
			for (int i = 0; i < planetPanels.length; i++) {
				if (planetPanels[i] instanceof PlanetAnimationPanelWithDescription) {
					PlanetAnimationPanelWithDescription pap = (PlanetAnimationPanelWithDescription) planetPanels[i];
					pap.handleTick();
				}
			}
		}
	}



	class PlanetListSelectionListener extends MotAction implements MouseListener {
    
    	private PlanetAnimationPanelWithDescription planetPanel;

    	PlanetListSelectionListener(PlanetAnimationPanelWithDescription planet) {
			this.planetPanel = planet;
    	}

		public void mouseClicked(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
			runAction();
			setContent(planetPanels, planetPanel);
			repaint();
		}

		public void mouseReleased(MouseEvent e) {
		}

		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}

		protected void _doAction() throws ApplicationError, ValidationError {
			if (listener != null) {
				listener.select(planetPanel.getPlanet());
			}
		}
    	
    }
    
}