/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.panels;
import java.awt.Color;

import mot.client.sounds.Sound;
import mot.client.sounds.Sounds;
import mot.client.swing.config.ImageAnimationConfig;
import mot.client.swing.window.common.MotInternalWindow;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;

/**
 * This is the basic class that implements animated panels. An animated panel is a Background
 * Image Panel whose background changes every n milli seconds - producing an animation.
 * 
 * An animation life defines the name of the animation images to be displayed in sequence. The images
 * displayed are;
 *          name + life[0] + fileType
 *          name + life[1] + fileType
 *          ...
 *          name + life[n] + fileType
 * Thus the size of the array life defines the quantity of images in the animation and the 
 * names of each image.
 * 
 * There are two types of animation
 *  Infinite - The animation continues until the window is closed. When the animation reaches 
 *            the end of its life the animation start at the beginning of its life again.
 *  Finite - The animation completes after a single pass through its life.
 */
public class AnimationPanel extends PlainTextBackgroundImagePanel implements AnimationTimerInterface {

    private String fileType;
    private String name;
    
    private String [] life;
    private int stage;
	private boolean isInfinate = false;
	private Sound sound;

    protected static final String DEFAULT_IMAGE_FILE_TYPE = PropertyManager.getMandatoryNoError("animation.default.file_type");

    public AnimationPanel(MotInternalWindow owner, String message, ImageAnimationConfig config) throws ApplicationError {
        this(owner, message, (config == null) ? null : config.getImage(), (config == null) ? null : config.getStagesAsSplit(), (config == null) ? null : config.getImageType());
    }
    
    public AnimationPanel(MotInternalWindow owner, String message, String name, String [] life) throws ApplicationError {
        this(owner, message, null, 0, false, name, life, DEFAULT_IMAGE_FILE_TYPE, null);
    }

    public AnimationPanel(MotInternalWindow owner, String message, String name, String [] life, String fileType) throws ApplicationError {
        this(owner, message, null, 0, false, name, life, fileType, null);
    }
    
    public AnimationPanel(MotInternalWindow owner, String message, String name, String [] life, String fileType, String soundName) throws ApplicationError {
        this(owner, message, null, 0, false, name, life, fileType, soundName);
    }
    
    public AnimationPanel(MotInternalWindow owner, String message, Color messageColor, int messageSizeDiff, boolean messageBold, String name, String [] life, String fileType, String soundName) throws ApplicationError {
    	super(message, messageColor, messageSizeDiff, messageBold, 0.8, 0.15, 0.01, 0.98);
        this.name = name;
        this.life = life;
        this.fileType = fileType;
        this.stage = 0;
        
        if (soundName != null) {
        	sound = Sounds.getSound(soundName);
        	if (sound != null) sound.play();
        }

        testResize();
    }
    
    protected String getImage() {
    	if (name == null) {
    		return null;
    	}
        if (life == null) {
        	//Logger.logDebug(""+getClass()+" AnimationPanel.getImage() image="+name);
            return name;
        } else {
            if (stage >= life.length) {
            	//Logger.logDebug(""+getClass()+" getImage() AnimationPanel.image="+name+life[life.length-1]+fileType);
                return name+life[life.length-1]+fileType; 
            } else {
            	//Logger.logDebug(""+getClass()+" getImage() AnimationPanel.image="+name+life[stage]+fileType);
                return name+life[stage]+fileType; 
            }
        }
    }

    public void handleTick() throws ApplicationError {
        stage++;
    	//Logger.logDebug(""+getClass()+" handleTick() STAGE="+stage);
        if (life != null && stage >= life.length) {
            if (infinate()) {
                stage = 0;
                repaint();
            } else {
                if (sound != null) sound.stop();
                animationFinished();
            }
        } else {
            repaint();
        }
    }
    
    /**
     * Stop this animation
     */
    public void stop() {
    }
    
    protected boolean infinate() { return isInfinate ; }
    
    protected void animationFinished() {
    }


    public int getStage() {
        return stage;
    }

	public String getFileType() {
		return fileType;
	}

	public void setFileType(String fileType) {
		this.fileType = fileType;
	}

	public String[] getLife() {
		return life;
	}

	public void setLife(String[] life) {
		this.life = life;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public boolean isInfinate() {
		return isInfinate;
	}

	public void setInfinate(boolean isInfinate) {
		this.isInfinate = isInfinate;
	}


}