/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.panels;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JComponent;
import javax.swing.JPanel;

import mot.client.actions.MotAction;
import mot.client.swing.common.Alignment;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.window.main.MainWindow;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.Rectangle;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public abstract class BackgroundImagePanel extends JPanel implements MouseListener {

	/**
	 * When this panel is being used as padding panel there may be a minimum size we wish to display the image in.
	 * If this is not null the image should not be displayed if the panel size is small than this (in any dimension). 
	 */
	private Size minimumSizeToDisplayImage;
	
	/**
	 * If this image is to be displayed as a fixed size in the centre of the screen this is not null
	 * and is that size.
	 */
	private Size fixedSize;
	
	private Alignment verticalAlignment;
	
    protected int width;
    protected int height;
    
    private boolean first = true;

    protected BackgroundImagePanel() {
        setLayout(null);
        setOpaque(false);
        addMouseListener(this);
    }

    /**
     * @return A Value between 0 and 1 representing the percentage of the panel 
     * that this image should consume. These methods return 1 unless over-ridden
     * which means consume the full panel. If these values are less than one then
     * the image is centred.
     */
    protected double getXScale() { return 1.0; }
    protected double getYScale() { return 1.0; }

    protected void testResize() {
        Dimension winSize = getSize();
        if (winSize == null) return;

        int originalWidth = width;
        int originalHeight = height;
        width = (int)winSize.getWidth();
        height = (int)winSize.getHeight();
        
        if (first || height != originalHeight || width != originalWidth) {
            first = false;
            //Logger.logDebug("RESIZE "+originalHeight+"=>"+height+" & "+originalWidth+"=>"+width);
            drawSizeRelativeObjects();
            validate();
            //repaint();
        }
    }

    public void layout() {
        drawSizeRelativeObjects();
    }

    protected abstract void drawSizeRelativeObjects();

    protected void setPosition(JComponent component, double percentTop, double percentHeight) {
        component.setOpaque(false);
        component.setBounds(0, (int)(height*percentTop), width, (int)(height*percentHeight));
    }
    
    protected Rectangle getPosition(double percentTop, double percentHeight, double percentIndent, double percentWidth) {
        return new Rectangle((int)(width*percentIndent), (int)(height*percentTop), (int)(width*percentWidth), (int)(height*percentHeight));
    }

    protected void setPosition(JComponent component, double percentTop, double percentHeight, double percentIndent, double percentWidth) {
        component.setOpaque(false);
        component.setBounds((int)(width*percentIndent), (int)(height*percentTop), (int)(width*percentWidth), (int)(height*percentHeight));
    }
    
    protected void setPositionWithFixedHeight(JComponent component1,JComponent component2, double percentTop, int height1InPixels, int height2InPixels, double percentIndent, double percentWidth) {
        component1.setOpaque(false);
        component2.setOpaque(false);
        int top = (int)(height*percentTop);
        Dimension winSize = getSize();
        if (winSize != null && top + height1InPixels + height2InPixels > winSize.height) {
        	top = winSize.height - (height1InPixels + height2InPixels);
        }
        component1.setBounds((int)(width*percentIndent), top, (int)(width*percentWidth), height1InPixels);
        component2.setBounds((int)(width*percentIndent), top+height1InPixels, (int)(width*percentWidth), height2InPixels);
    }
    
    protected void setPositionWithFixedHeight(JComponent component, double percentTop, int heightInPixels, double percentIndent, double percentWidth) {
        component.setOpaque(false);
        component.setBounds((int)(width*percentIndent), (int)(height*percentTop), (int)(width*percentWidth), heightInPixels);
    }
    
    protected void setPosition(JComponent component, double valign, int height, int width, boolean opaque) {
        component.setOpaque(opaque);
        int indent = (this.width - width) / 2;
        int top = (int) ((this.height - height) * valign);
        component.setBounds(indent, top, width, height);
    }
    
    protected int getX(double percentWidth) {
        return (int)(width*percentWidth);
    }

    protected int getY(double percentHeight) {
        return (int)(height*percentHeight);
    }

    protected int getRotation() {
        return 0;
    }

    
    /**
     * @return The name of the image to display as a background.
     * @throws ApplicationError 
     */
    protected abstract String getImage() throws ApplicationError;

    private static long t = 0;
    public void paint(Graphics graphics) {
//    	long st = System.currentTimeMillis();
//    	try {
//			System.out.println("BackgroundImagePanel.paint() Interval "+(st-t)+" ["+getImage()+"]");
//		} catch (ApplicationError e1) {
//			e1.printStackTrace();
//		}
//    	t = st;
        testResize();
        Graphics2D g = (Graphics2D) graphics;
        prePaint(g);
        String image;
		try {
			image = getImage();
		} catch (ApplicationError e) {
			Logger.logStackTrace();
			image = null;
		}
        //Logger.logDebug(""+getClass()+" BackgroundImagePanel.getImage() image="+image);
        if (image != null && isPanelAboveMinimumSize()) {
        	int left = 0;
        	int top = 0;
        	int imageWidth = 0;
        	int imageHeight = 0;
        	if (fixedSize != null) {
        		imageWidth = fixedSize.getX();
        		imageHeight = fixedSize.getY();
        		left = (width - imageWidth) / 2;
        		if (verticalAlignment == Alignment.top) {
        			top = 0;
        		} else if (verticalAlignment == Alignment.bottom) {
        			top = height - imageHeight;
        		} else {
        			top = (height - imageHeight) / 2;
        		}
        	} else {
            	imageWidth = (int)(width * getXScale());
            	imageHeight = (int)(height * getYScale());
        		if (width > imageWidth) {
        			left = (width-imageWidth)/2;
        		}
        		if (height > imageHeight) {
        			top = (height-imageHeight)/2;
        		}
        	}
            int rotation = getRotation();
            if (rotation == 0) {
                ImageUtils.drawImage(this, g, image, left, top, imageWidth, imageHeight);
            } else {
                ImageUtils.drawRotatedImage(this, g, image, left, top, imageWidth, imageHeight, null, rotation);
            }
        }
        super.paint(graphics);
        postPaint(g);
        // On Linux the animation is jumps as it is not sync-ing enough
        Toolkit.getDefaultToolkit().sync();
//        st = System.currentTimeMillis();
//    	System.out.println("BackgroundImagePanel.paint()  Cost "+(st-t));
//    	t = st;
    }
    
    public void repaint() {
    	//System.out.println("repaint - "+getClass().getName());
    	super.repaint();
    }

	protected void prePaint(Graphics2D graphics){ }
    protected void postPaint(Graphics2D graphics){ }


    public int getPanelWidth() {
        return width;
    }

    
	public void mouseClicked(MouseEvent e) { 
	}

    private MotAction clickAction;
	public void motMouseClicked(MouseEvent e) throws ApplicationError, ValidationError {
		if (clickAction != null) {
			clickAction.click();
		}
 	}
	public void setClickAction(MotAction clickAction) {
		this.clickAction = clickAction;
	}
	
	public void mousePressed(MouseEvent e) {
    	if (e.getClickCount() == 1) {
    		try {
    			if (TutorialSystem.isActive()) {
    				TutorialSystem.validateBackgroundImageClicked(this);
    			}
    			motMouseClicked(e);
    		} catch (ValidationError e1) {
    			MainWindow.getExestingInstance().reportValidationError(e1);
    		} catch (Exception e1) {
    			MainWindow.getExestingInstance().reportApplicationError(e1);
    		}
    	}
	}

	public void mouseReleased(MouseEvent e) {	}

	public void mouseEntered(MouseEvent e) {	}

	public void mouseExited(MouseEvent e) { 	}

	
	public void setMinimumSizeToDisplayImage(Size size) {
		minimumSizeToDisplayImage = size;
	}
	
    private boolean isPanelAboveMinimumSize() {
		if (minimumSizeToDisplayImage == null) return true;
		return (width > minimumSizeToDisplayImage.getX() && height > minimumSizeToDisplayImage.getY());
	}

	public void setFixedSizeAtCenter(Size fixedSizeAtCenter) {
		this.fixedSize = fixedSizeAtCenter;
	}

	public Alignment getVerticalAlignment() {
		return verticalAlignment;
	}

	public void setVerticalAlignment(Alignment verticalAlignment) {
		this.verticalAlignment = verticalAlignment;
	}

}