/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.panels;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;

import javax.swing.JLabel;
import javax.swing.SwingConstants;

import mot.client.actions.CraftAction;
import mot.client.bean.CraftGraphicsWrapper;
import mot.client.deligates.CraftDelegate;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.common.MotLabel;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.window.common.MessagePanel;
import mot.client.swing.window.common.SplitHtmlResult;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftLeaveFleet;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

/**
 * Displays information about a technology. It has a pictorial background (prefereable animated
 * in the future) and in front of that displays details of the technology.
 */
public class CraftPicturePanel extends BackgroundImagePanel {

    private JLabel label;
    private String titleText;
    
    private boolean showName = true;
    private int labelSize = 0;
    
    private CraftDelegate craft;
	private String backgroundImage = null;
    
    public CraftPicturePanel(int labelSize, boolean use3DImage, CraftDelegate craft) throws ApplicationError {
    	this(labelSize, use3DImage);
    	update(craft, null, false);
    }
    
    public CraftPicturePanel(int labelSize, boolean use3DImage) throws ApplicationError {
    	this.labelSize = labelSize;
    	
    	if (use3DImage) {
    		this.backgroundImage = ImageConfigFactory.getImageForConfig("3d_moo_background");
    	}

    	label = new MotLabel(null, labelSize, null, null, JLabel.CENTER, false);
    	label.setOpaque(false);
    	label.setVerticalTextPosition(SwingConstants.BOTTOM);
    	label.setHorizontalTextPosition(SwingConstants.CENTER);
    	label.setVerticalAlignment(SwingConstants.BOTTOM);
    	label.setHorizontalAlignment(SwingConstants.CENTER);
        add(label);

        testResize();
    }

	protected void prePaint(Graphics2D g){
		if (backgroundImage != null) {
			ImageUtils.drawImage(this, g, backgroundImage, 0, 0, width, height);
		}
	}

    public void update(CraftDelegate craft, String titleText, boolean withRepaint) throws ApplicationError {
        this.craft = craft;
        this.titleText = titleText;
        
        drawSizeRelativeObjects();        
        
        if (withRepaint) {
            invalidate();
            repaint();
        }
    }

    private final static int FACTOR = 10;

    protected void drawSizeRelativeObjects() {
        if (width < 1) return;
        int maxLineLength = width/10;
        SplitHtmlResult html = MessagePanel.toHtml(titleText, null, maxLineLength, true);
		label.setText(html.getHtmlString());
        if (html.qtyLines > 3) {
            setPositionWithFixedHeight(label, 0.70, 55, 0.05, 0.95);
        } else if (html.qtyLines > 2) {
            setPositionWithFixedHeight(label, 0.80, 42, 0.05, 0.95);
        } else if (html.qtyLines > 1) {
            setPositionWithFixedHeight(label, 0.65, 35, 0.05, 0.95);
        } else {
            setPositionWithFixedHeight(label, 0.82, 15, 0.05, 0.95);
        }
        if (leaveFleetButton != null) {
        	int x = width - leaveFleetButton.getWidth() - 1;
        	int y = 1;
        	leaveFleetButton.setBounds(x, y, leaveFleetButton.getWidth(), leaveFleetButton.getHeight());
        }
		if (craft != null && backgroundImage != null) {
			try {
		        Size craftImageSize = craft.getImageSize();
		        int craftImageWidth = craftImageSize.getX();
		        Size biggestImage = CraftGraphicsWrapper.getBiggestImage();
		        double percentSize = ((double)craftImageWidth) / ((double)biggestImage.getX());  
		        int actualX = (int) (width * percentSize); 
		        int actualY = (int) (height * percentSize); 
		        setFixedSizeAtCenter(new Size(actualX, actualY));
			} catch (ApplicationError e) {
				Logger.logError("Unable to size Craft", e);
			}
		}
    }

    private static final int INDENT = 30;
    private static final int THUMB_WIDTH = 100;
    private static final int THUMB_HEIGHT = 100;
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

    }

    private CraftAction craftAction;
	private ActionButtonCraftLeaveFleet leaveFleetButton;
    
	public CraftAction getCraftAction() {
		return craftAction;
	}

	public void setCraftAction(CraftAction craftAction) {
		this.craftAction = craftAction;
	}

	public void motMouseClicked(MouseEvent e) throws ApplicationError, ValidationError {
		if (craftAction != null) {
			craftAction.click(craft);
		}
 	}

	protected String getImage() throws ApplicationError {
		if (backgroundImage != null) {
			String image = craft.get3DImage();
			if (image == null) image = craft.getCraftDesign().getHullTech().getImage();
			return image;
		} else {
			return craft.getCraftDesign().getHullTech().getImage();
		}
	}

	public void addLeaveFleetButton(ActionButtonCraftLeaveFleet leaveFleetButton) {
		this.leaveFleetButton = leaveFleetButton;
		add(leaveFleetButton);
	}


}