/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.panels;

import java.awt.Graphics;
import java.awt.Image;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import mot.client.swing.common.IconSequence;
import mot.client.swing.common.ImageCache;
import mot.common.util.Comp;

/**
 * An icon Panel is a horizontal panel used to represent pictorially the quantity of some resource.
 * It contains a list of icons (1 per quantity) moving from left to right.
 */
public class IconPanel extends JPanel {

    /**
     * This is the name and quantity of the first icon in the list.
     */
    private IconSequence icon1;

    /**
     * This is the name and quantity of the second icon in the list.
     */
    private IconSequence icon2;

    /**
     * This is the name and quantity of the third icon in the list.
     */
    private IconSequence icon3;
    
    /**
     * This is the name and quantity of the first icon in the list.
     */
    private IconSequence lossIcon1;

    /**
     * This is the name and quantity of the second icon in the list.
     */
    private IconSequence lossIcon2;

    /**
     * This is the name and quantity of the third icon in the list.
     */
    private IconSequence lossIcon3;
    
    /**
     * Icons may optionally have text which is centred in the icon panel.
     */
    private JLabel text;
    
    /**
     * The alignment of the icons. Possible values are SwingConstants.CENTER,
     * SwingConstants.BOTTOM and SwingConstants.TOP.
     * Default is SwingConstants.CENTER
     */
    private int vAlignment = SwingConstants.CENTER;

    public IconPanel(IconSequence icon1, IconSequence icon2, IconSequence icon3) {
        super(null);
        this.icon1 = icon1;
        this.icon2 = icon2;
        this.icon3 = icon3;
        
        //setMinimumSize(new Dimension(getMaxWidth()*5, (int)(getMaxHeight()*1.5)));
    }
    
    public void setLossIcons(IconSequence lossIcon1, IconSequence lossIcon2, IconSequence lossIcon3) {
        this.lossIcon1 = lossIcon1;
        this.lossIcon2 = lossIcon2;
        this.lossIcon3 = lossIcon3;
    }
    
    public int getMaxWidth() {
        int r = 0;
        if (icon1 != null && icon1.getWidth() > r) {
            r = icon1.getWidth();
        }
        if (icon2 != null && icon2.getWidth() > r) {
            r = icon2.getWidth();
        }
        if (icon3 != null && icon3.getWidth() > r) {
            r = icon3.getWidth();
        }
        if (lossIcon1 != null && lossIcon1.getWidth() > r) {
            r = lossIcon1.getWidth();
        }
        if (lossIcon2 != null && lossIcon2.getWidth() > r) {
            r = lossIcon2.getWidth();
        }
        if (lossIcon3 != null && lossIcon3.getWidth() > r) {
            r = lossIcon3.getWidth();
        }
        if (text != null && text.getWidth() > r) {
            r = text.getWidth();
        }
        return r;
    }

    public int getMaxHeight() {
        int r = 0;
        if (icon1 != null && icon1.getHeight() > r) {
            r = icon1.getHeight();
        }
        if (icon2 != null && icon2.getHeight() > r) {
            r = icon2.getHeight();
        }
        if (icon3 != null && icon3.getHeight() > r) {
            r = icon3.getHeight();
        }
        if (lossIcon1 != null && lossIcon1.getHeight() > r) {
            r = lossIcon1.getHeight();
        }
        if (lossIcon2 != null && lossIcon2.getHeight() > r) {
            r = lossIcon2.getHeight();
        }
        if (lossIcon3 != null && lossIcon3.getHeight() > r) {
            r = lossIcon3.getHeight();
        }
        if (text != null && text.getHeight() > r) {
            r = text.getHeight();
        }
        return r;
    }

    public int getTotalQty() {
        int r = 0;
        if (icon1 != null) {
            r += icon1.getQty();
        }
        if (icon2 != null) {
            r += icon2.getQty();
        }
        if (icon3 != null) {
            r += icon3.getQty();
        }
        if (lossIcon1 != null) {
            r += lossIcon1.getQty();
        }
        if (lossIcon2 != null) {
            r += lossIcon2.getQty();
        }
        if (lossIcon3 != null) {
            r += lossIcon3.getQty();
        }
        return r;
    }

    public void paintComponent(Graphics g) { 
        super.paintComponent(g); 
        int x = 0;
        if (icon1 != null) {
            x = paintIcon(g, icon1, x);
        }
        if (icon2 != null) {
            x = paintIcon(g, icon2, x);
        }
        if (icon3 != null) {
            x = paintIcon(g, icon3, x);
        }
        if (lossIcon1 != null) {
            x = paintIcon(g, lossIcon1, x);
        }
        if (lossIcon2 != null) {
            x = paintIcon(g, lossIcon2, x);
        }
        if (lossIcon3 != null) {
            x = paintIcon(g, lossIcon3, x);
        }
    }

    public int paintIcon(Graphics g, IconSequence icon, int x) {
        Image img = ImageCache.getImage(getToolkit(), icon.getImage());
        int qty = icon.getQty();
        int w = getWidth();
        int h = getHeight();
        int imageHeight = Comp.min(icon.getHeight(), h-2);
        int imageWidth = icon.getWidth();
        int y = 0;
        if (vAlignment == SwingConstants.CENTER) y = (h-imageHeight)/2;
        if (vAlignment == SwingConstants.BOTTOM) y = h-imageHeight;
        int totalQty = getTotalQty();
        int interval = totalQty == 0 ? icon.getWidth()+1 : (w-icon.getWidth()) / totalQty;
        if (interval > icon.getWidth()) interval = icon.getWidth()+1; 
        for (int i=0; i<qty; i++) {
            g.drawImage(img, x, y, imageWidth, imageHeight, this);
            x += interval;
        }
        return x;
    }


    public void setIcon1(IconSequence icon1) {
        this.icon1 = icon1;
    }


    public IconSequence getIcon1() {
        return icon1;
    }


    public void setIcon2(IconSequence icon2) {
        this.icon2 = icon2;
    }


    public IconSequence getIcon2() {
        return icon2;
    }


    public void setIcon3(IconSequence icon3) {
        this.icon3 = icon3;
    }


    public IconSequence getIcon3() {
        return icon3;
    }

	public int getVAlignment() {
		return vAlignment;
	}

	public void setVAlignment(int alignment) {
		vAlignment = alignment;
	}

	public void setText(String t) {
		if (t == null) {
			removeAll();
		} else {
			removeAll();
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			text = new JLabel(t);
			text.setOpaque(false);
			add(text);
		}
	}

	public IconSequence getLossIcon1() {
		return lossIcon1;
	}

	public void setLossIcon1(IconSequence lossIcon1) {
		this.lossIcon1 = lossIcon1;
	}

	public IconSequence getLossIcon2() {
		return lossIcon2;
	}

	public void setLossIcon2(IconSequence lossIcon2) {
		this.lossIcon2 = lossIcon2;
	}

	public IconSequence getLossIcon3() {
		return lossIcon3;
	}

	public void setLossIcon3(IconSequence lossIcon3) {
		this.lossIcon3 = lossIcon3;
	}
}

