/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;

import mot.client.deligates.CraftDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.swing.common.ImageCache;
import mot.client.swing.common.PaintedPanel;
import mot.common.bean.Rectangle;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;

public class MooImagePanel extends PaintedPanel {
	
    private static final int DEFAULT_SIZE = 80;
    
	private MoveableOwnedObjectDelegate moo;
	private Color healthBarBorderColor;
	private Color healthBarColor;
	private int barHeight;

	public MooImagePanel(MoveableOwnedObjectDelegate moo, boolean withHealthBar, Color backgroundColour) throws ApplicationError {
        this.moo = moo;
        
        if (withHealthBar) {
	        this.healthBarColor = PropertyManager.getMandatoryAsColor("damagebar.color");
	        this.healthBarBorderColor = PropertyManager.getMandatoryAsColor("outline.color");
	        this.barHeight = 2;
        }
		
        if (backgroundColour != null) {
            setBackground(backgroundColour);
            setOpaque(true);
        } else {
            setOpaque(false);
        }
        
        Size imageSize = moo.getImageSize();
        setPreferredSize(new Dimension( (int)(imageSize.getX()*1.25), (int)(imageSize.getY()*1.25) ));
        		
	}
	
	protected Rectangle getMooImageRect() throws ApplicationError {
		Dimension size = getSize();
		int panelHeight = (int) size.getHeight();
		int panelWidth = (int) size.getWidth();
		
        Size imageSize = moo.getImageSize();
        int imageWidth = imageSize.getX();
        int imageHeight = imageSize.getY();
        
		int spareWidth = panelWidth - imageWidth;
		int spareHeight = panelHeight - imageHeight;
        int xd = (spareWidth == 0) ? 0 : spareWidth/2;
        int yd = (spareHeight == 0) ? 0 : spareHeight/2;
        
		return new Rectangle(xd, yd, imageWidth, imageHeight);
	}

	private void drawHealthBar(Graphics2D g, CraftDelegate craft) throws ApplicationError {
		double damagePercent = craft.getDamageAsPercent();
		if (damagePercent > 0.0) {
	        Rectangle mr = getMooImageRect();        
	        int topBar = mr.getBottom();
	        
	        g.setColor(healthBarColor);
			g.fillRect(mr.getLeft(), topBar+1, (int)(mr.getWidth() * damagePercent), barHeight);
		}
	}

	protected void drawContent(Graphics2D g) throws ApplicationError {
		if (isOpaque()) {
			Dimension size = getSize();
			int panelHeight = (int) size.getHeight();
			int panelWidth = (int) size.getWidth();
        	Color background = getBackground();
        	g.setColor(background);
        	g.fillRect(0, 0, panelWidth, panelHeight);
		}
		
		String imageName = moo.getImage();
        Image image = ImageCache.getImage(getToolkit(), imageName);
        
        Rectangle mr = getMooImageRect();        
        
        g.drawImage(image, mr.getLeft(), mr.getTop(), mr.getWidth(), mr.getHeight(), this);
        
		if (moo instanceof CraftDelegate && healthBarColor != null) {
			drawHealthBar(g, (CraftDelegate) moo);
		}
	}


}
