/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.panels;
import java.awt.Color;
import java.awt.Font;

import javax.swing.JLabel;
import javax.swing.SwingConstants;

import mot.client.swing.common.PanelUtils;

/**
 * This is a panel whose background is an image (whose name is returned by
 * getImage() and contain a simple textual description of the image centred 
 * at the bottom of the screen.
 */
abstract public class PlainTextBackgroundImagePanel extends BackgroundImagePanel {

    private JLabel descriptionLabel;
    private String text;
    private Font font;
    
    private double percentTop;
    private double percentHeight;
    private double percentIndent;
    private double percentWidth;
    
    protected PlainTextBackgroundImagePanel(String text, double percentTop, double percentHeight, double percentIndent, double percentWidth) {
    	this(text, null, 0, false, percentTop, percentHeight, percentIndent, percentWidth, SwingConstants.CENTER, SwingConstants.BOTTOM);
    }

    protected PlainTextBackgroundImagePanel(String text) {
    	this(text, null, 0, false, 0.3, 0.65, 0.01, 0.99, SwingConstants.CENTER, SwingConstants.BOTTOM);
    }

    protected PlainTextBackgroundImagePanel(String text, Color textColor, int textSizeDiff, boolean textBold) {
    	this(text, textColor, textSizeDiff, textBold, 0.3, 0.65, 0.01, 0.99, SwingConstants.CENTER, SwingConstants.BOTTOM);
    }

    protected PlainTextBackgroundImagePanel(String text, Color textColor, int textSizeDiff, boolean textBold, double percentTop, double percentHeight, double percentIndent, double percentWidth) {
    	this(text, textColor, textSizeDiff, textBold, percentTop, percentHeight, percentIndent, percentWidth, SwingConstants.CENTER, SwingConstants.BOTTOM);
    }
    
    protected PlainTextBackgroundImagePanel(String text, Color textColor, int textSizeDiff, boolean textBold, double percentTop, double percentHeight, double percentIndent, double percentWidth, int hAlign, int vAlign) {
        this.text = text;
        this.percentTop = percentTop;
        this.percentHeight = percentHeight;
        this.percentIndent = percentIndent;
        this.percentWidth = percentWidth;
        
        descriptionLabel = PanelUtils.createLabel(text);
        descriptionLabel.setVerticalAlignment(vAlign);
        descriptionLabel.setHorizontalAlignment(hAlign);
        if (textSizeDiff != 0 || textBold) {
            font = descriptionLabel.getFont();
            int style = font.getStyle();
            if (textBold) {
                style = Font.BOLD;
            }
            descriptionLabel.setFont(new Font(font.getName(), style, font.getSize()+textSizeDiff));
        }
        if (textColor != null) {
        	descriptionLabel.setForeground(textColor);
        }
        add(descriptionLabel);
        testResize();
    }
    
    protected PlainTextBackgroundImagePanel() {
    }

    private final static int FACTOR = 10;

    protected void drawSizeRelativeObjects() {
    	if (descriptionLabel != null) {
    		setPosition(descriptionLabel, percentTop, percentHeight, percentIndent, percentWidth);
    	}
    }

	public String getText() {
		return text;
	}

	public void setText(Font font, String text) {
		setFont(font);
		setText(text);
	}

	public void setText(String text) {
		this.text = text;
    	if (descriptionLabel == null) descriptionLabel = new JLabel();
		descriptionLabel.setText(text);
		if (font != null) {
			descriptionLabel.setFont(font);
		}
	}

	public JLabel getDescriptionLabel() {
		return descriptionLabel;
	}

	public void setFont(Font font) {
		this.font = font;
	}
	
	public void changeFont(Font font, Color textColor) {
		this.font = font;
		setText(text);
		if (textColor != null && descriptionLabel != null) {
			descriptionLabel.setForeground(textColor);
		}
	}
	
	public void changeFont(int textSizeDiff, boolean textBold) {
        font = descriptionLabel.getFont();
        int style = font.getStyle();
        if (textBold) {
            style = Font.BOLD;
        }
        descriptionLabel.setFont(new Font(font.getName(), style, font.getSize()+textSizeDiff));
	}

}