/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.panels;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.JLabel;
import javax.swing.SwingConstants;

import mot.client.bean.CraftGraphicsWrapper;
import mot.client.bean.ResearchProject;
import mot.client.deligates.PlayerDelegate;
import mot.client.swing.common.ImageCache;
import mot.client.swing.common.MotLabel;
import mot.client.swing.window.common.MessagePanel;
import mot.client.swing.window.common.SplitHtmlResult;
import mot.client.swing.window.main.MainWindow;
import mot.common.lang.L;
import mot.common.technology.Biotechnology;
import mot.common.technology.Computer;
import mot.common.technology.Engine;
import mot.common.technology.Factory;
import mot.common.technology.Hull;
import mot.common.technology.Laboratory;
import mot.common.technology.Module;
import mot.common.technology.Scanner;
import mot.common.technology.Shield;
import mot.common.technology.Special;
import mot.common.technology.Technology;
import mot.common.technology.Theory;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;

/**
 * Displays information about a technology. It has a pictorial background (prefereable animated
 * in the future) and in front of that displays details of the technology.
 */
public class TechnologyPicturePanel extends BackgroundImagePanel {

    private JLabel descriptionLabel;
    private String descriptionTitle;
    private String descriptionText;
    private JLabel descriptionHeaderLabel;
    private String descriptionHeaderText;
    
    private String thumbImage;
    
    private boolean showTurnsToComplete = false;
    private boolean showDescriptionHeader = true;
    private boolean showDescription = true;
    private boolean showHullThump;
    private PlayerDelegate owner; 
    
    private Technology tech;
    
	private String noTechnologyBackgroundImage;
    
    public TechnologyPicturePanel(PlayerDelegate owner, boolean showTurnsToComplete, boolean showDescription, boolean showTitle, boolean showHullThump , Technology tech, String descriptionHeaderText) throws ApplicationError {
        this(owner, showTurnsToComplete, showDescription, 5, showTitle, showHullThump, descriptionHeaderText);
        update(tech);
    }

    public TechnologyPicturePanel(PlayerDelegate owner, boolean showTurnsToComplete, boolean showDescription, boolean showTitle, boolean showHullThump, String descriptionHeaderText) {
    	this(owner, showTurnsToComplete, showDescription, 5, showTitle, showHullThump, descriptionHeaderText);
    }
    
    public TechnologyPicturePanel(PlayerDelegate owner, boolean showTurnsToComplete, boolean showDescription, int descriptionSize, boolean showTitle, boolean showHullThump, String descriptionHeaderText) {
    	this.owner = owner;
    	this.showTurnsToComplete = showTurnsToComplete;
		this.showDescription = showDescription;
		this.showDescriptionHeader = showTitle;
		this.showHullThump = showHullThump;
		this.descriptionHeaderText = descriptionHeaderText;
		
        descriptionLabel = new MotLabel(null, descriptionSize, null, null, JLabel.CENTER, false);
        descriptionHeaderLabel = new MotLabel(null, 10, null, null, JLabel.CENTER, true);
        //descriptionLabel.setAlignmentY(0.5f);
        //descriptionLabel.setAlignmentX(0.5f);
        descriptionLabel.setOpaque(false);
        descriptionLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
        descriptionLabel.setHorizontalTextPosition(SwingConstants.CENTER);
        descriptionLabel.setVerticalAlignment(SwingConstants.BOTTOM);
        descriptionLabel.setHorizontalAlignment(SwingConstants.CENTER);
        add(descriptionLabel);
        if (descriptionHeaderText != null) {
        	descriptionHeaderLabel.setOpaque(false);
        	descriptionHeaderLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
        	descriptionHeaderLabel.setHorizontalTextPosition(SwingConstants.CENTER);
        	descriptionHeaderLabel.setVerticalAlignment(SwingConstants.BOTTOM);
        	descriptionHeaderLabel.setHorizontalAlignment(SwingConstants.CENTER);
        	descriptionHeaderLabel.setForeground(Color.WHITE);
	        add(descriptionHeaderLabel);
        }
        testResize();
    }

    public void update(Technology tech) throws ApplicationError {
        update(tech, 0, true);
    }

    public void update(Technology tech, boolean withRepaint) throws ApplicationError {
        update(tech, 0, withRepaint);
    }
    
    public void update(Technology tech, int qty, boolean withRepaint) throws ApplicationError {
        this.tech = tech;
        
        if (tech != null) {
        	if (showDescriptionHeader) {
        		if (qty <= 0) {
        			descriptionTitle = tech.getName();
        		} else{
        			descriptionTitle = tech.getName() + " x " + qty;
        		}
        	}
            if (showDescription) descriptionText = tech.getDescription();
    	} else {
    		descriptionTitle = null;
    		descriptionText = null;
        }
        if (owner != null && descriptionText != null && showTurnsToComplete) {
        	double researchPerTurn = owner.getTotalResearch();
        	double costToResearch = tech.getCostToResearch();
        	descriptionText += " [ " + Formatter.wholeRoundedUp(costToResearch/researchPerTurn) + " "+L.TURNS+" ]";
        }
        
        drawSizeRelativeObjects();
        
        if (withRepaint) {
            invalidate();
            repaint();
        }
    }

    public void update(ResearchProject researchProject, boolean withRepaint) throws ApplicationError {
    	this.tech = (researchProject == null) ? null : researchProject.getTechnology();
        
        if (tech == null) {
        	return;
        }
        
    	descriptionTitle = tech.getName();
        descriptionText = tech.getDescription();
        
        if (owner != null && researchProject != null && showTurnsToComplete) {
        	if (researchProject.isResearchingSalvage()) {
        		double chanceToDiscoverSalvage = MainWindow.getOwner().getChanceToDiscoverSalvage();
        		double researchPerTurn = owner.getTotalResearch();
        		double costToResearch = researchProject.getCostToResearch();
       			descriptionTitle +=  L.RESEARCH_SALVAGE + " [ Success "+Formatter.percentLabelNoSign(chanceToDiscoverSalvage)+" ]";
        	}
    		double researchPerTurn = owner.getTotalResearch();
    		double costToResearch = researchProject.getCostToResearch();
    		descriptionText += " [ " + Formatter.wholeRoundedUp(costToResearch/researchPerTurn) + " "+L.TURNS+" ]";
        }
        
        drawSizeRelativeObjects();
        
        if (withRepaint) {
            invalidate();
            repaint();
        }
    }

    private final static int FACTOR = 10;

    protected void drawSizeRelativeObjects() {
        int width = getPanelWidth()-20;
        if (width < 1) return;
        Graphics graphics = getGraphics();
		SplitHtmlResult r = MessagePanel.toHtml(descriptionTitle, descriptionText, width, true, graphics);
		descriptionLabel.setText(r.getHtmlString());
        if (descriptionHeaderText != null) {
        	descriptionHeaderLabel.setText(descriptionHeaderText);
        }
        int qtyNewLines = r.qtyLines;
        if (qtyNewLines > 3) {
            setPositionWithFixedHeight(descriptionHeaderLabel, descriptionLabel, 0.75, 50, 250, 0.05, 0.9);
        } else if (qtyNewLines > 2) {
            setPositionWithFixedHeight(descriptionHeaderLabel, descriptionLabel, 0.85, 50, 150, 0.05, 0.9);
        } else {
            setPositionWithFixedHeight(descriptionHeaderLabel, descriptionLabel, 0.9, 50, 100, 0.05, 0.9);
        }
    }

    private static final int INDENT = 30;
    private static final int THUMB_WIDTH = 100;
    private static final int THUMB_HEIGHT = 100;
    
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        if (thumbImage != null) {        	
        	Image image = ImageCache.getImage(getToolkit(), thumbImage);
            g.drawImage(image, width-THUMB_WIDTH-INDENT, INDENT, THUMB_WIDTH, THUMB_HEIGHT, this);
        }
    }

    private static final String HULL_IMAGE = "images/backgrounds/technology_hull.jpg";
    private static final String ENGINE_IMAGE = "images/backgrounds/technology_engine.png";
    private static final String COMPUTER_IMAGE = "images/backgrounds/technology_computer.jpg";
    private static final String WEAPON_IMAGE = "images/backgrounds/technology_weapon.png";
    private static final String SHIELD_IMAGE = "images/backgrounds/technology_shield.png";
    private static final String SCANNER_IMAGE = "images/backgrounds/technology_scanner.png";
    private static final String SPECIAL_IMAGE = "images/backgrounds/technology_special.jpg";
    private static final String FACTORY_IMAGE = "images/backgrounds/technology_factory.jpg";
    private static final String LABORATORY_IMAGE = "images/backgrounds/technology_laboratory.jpg";
    private static final String BIOTECHNOLOGY_IMAGE = "images/backgrounds/technology_biotechnology.jpg";
    private static final String THEORY_IMAGE = "images/backgrounds/technology_theory.jpg";
    private static final String MODULE_IMAGE = "images/backgrounds/technology_module.jpg";
    
    protected String getImage() {
    	if (tech instanceof Hull && showHullThump) {
    		if (width > 50) {
    			thumbImage = CraftGraphicsWrapper.getDefaultImageNoError(((Hull)tech).getName()) ;
    		} else {
    			thumbImage = CraftGraphicsWrapper.getImageThumbNoError(((Hull)tech).getName()) ;
    		}
		} else {
			thumbImage = null;
		}
		if (tech == null && noTechnologyBackgroundImage != null) {
			return noTechnologyBackgroundImage;	
		} else {
			return getImage(tech);	
		}
    }
    
	public static String getActualImageThumb(Technology tech) {
		String image = null;
		if (tech instanceof Hull) {
			Hull hull = (Hull) tech;
			image = CraftGraphicsWrapper.getImageThumbNoError(tech.getName());
		}
		if (image == null) {
			image = TechnologyPicturePanel.getImage(tech);
		}
		return image;
	}
	
    public static String getImage(Technology tech) { 
        if (tech == null) { 
            return null;
        } else if (tech.getImage() != null) {
            return tech.getImage();
        } else if (tech instanceof Hull) {
        	return HULL_IMAGE;
        } else if (tech instanceof Engine) {
            return ENGINE_IMAGE;
        } else if (tech instanceof Computer) {
            return COMPUTER_IMAGE;
        } else if (tech instanceof Weapon) {
            return WEAPON_IMAGE;
        } else if (tech instanceof Shield) {
            return SHIELD_IMAGE;
        } else if (tech instanceof Scanner) {
            return SCANNER_IMAGE;
        } else if (tech instanceof Special) {
            return SPECIAL_IMAGE;
        } else if (tech instanceof Factory) {
            return FACTORY_IMAGE;
        } else if (tech instanceof Laboratory) {
            return LABORATORY_IMAGE;
        } else if (tech instanceof Biotechnology) {
            return BIOTECHNOLOGY_IMAGE;
        } else if (tech instanceof Theory) {
            return THEORY_IMAGE;
        } else if (tech instanceof Module) {
            return MODULE_IMAGE;
        }
        return null;
    }

	public void setNoTechnologyBackground(String image) {
		noTechnologyBackgroundImage = image;
	}

	public String getDescriptionTitle() {
		return descriptionTitle;
	}

	public void setDescriptionTitle(String descriptionTitle) {
		this.descriptionTitle = descriptionTitle;
	}

}