/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.renderers;

import java.awt.Color;
import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.border.LineBorder;

import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PadPanel;
import mot.client.swing.labels.TechnologyLabel;
import mot.client.swing.labels.TechnologyLabelClickListener;
import mot.client.swing.panels.TechnologyIconPanel;
import mot.client.swing.window.common.MessagePanel;
import mot.common.bean.Size;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;

public class CellRendererTechnologyLabel extends JPanel implements ListCellRenderer {

	private boolean showFullNames;
	private boolean displayIcons;
	private boolean displayText;
	private Size iconSize;
	private boolean iconHasName;
	private Integer textSizeDiff;
	private boolean textBold;
	private ImageIcon textIcon; // Same icon for every image
	private TechnologyLabelClickListener listener; // if icon or label clicked, call this
	
	private Color textColour;
	private Color backgroundColour;
	private boolean descriptionAsToolTip;
	private Integer maxLineLength;
	private ListItemControlPanelFactory itemControlPanelFactory;
	
	
    public CellRendererTechnologyLabel(boolean showFullNames, boolean displayIcons, boolean iconHasName, Size iconSize, boolean displayText, Integer textSizeDiff, boolean textBold, Color textColour, ImageIcon textIcon, boolean displayDescriptionAsToolTip, TechnologyLabelClickListener listener, Integer maxLineLength, ListItemControlPanelFactory itemControlPanelFactory) {
		this.showFullNames = showFullNames;
		this.displayIcons = displayIcons;
		this.iconHasName = iconHasName;
		this.iconSize = iconSize;
		this.displayText = displayText;
		this.textSizeDiff = textSizeDiff;
		this.textBold = textBold;
		this.textColour = textColour;
		this.textIcon = textIcon;
		this.descriptionAsToolTip = displayDescriptionAsToolTip;
		this.listener = listener;
		this.maxLineLength = maxLineLength;
		this.itemControlPanelFactory = itemControlPanelFactory;
    }
    
     public Component getListCellRendererComponent(JList list,
                                                   Object value,            // value to display
                                                   int index,               // cell index
                                                   boolean isSelected,      // is the cell selected
                                                   boolean cellHasFocus)    // the list and the cell have the focus
     {
    	 String qtyText = null;
    	 if (value instanceof TechnologyBundle) {
    		 int qty = ((TechnologyBundle) value).getQty();
    		 if (qty > 1) {
    			 qtyText = " X "+qty;
    		 }
    		 value = ((TechnologyBundle) value).getTechnology();
    	 } else if (!(value instanceof Technology)) {
    		 Logger.logError("Not a Technology : "+((value == null) ? "NULL" : value.getClass().getName()), new Exception());
    		 return new JLabel("ERROR");
    	 }
    	 Technology tech = (Technology) value;
    	 
    	 TechnologyLabel label = null;
    	 if (displayText){
    		 String text = showFullNames ? tech.getFullName() : tech.getName();
    		 if (qtyText != null) {
    			 text = text + qtyText;
    		 }
  			if (maxLineLength != null) {
 				String html = MessagePanel.toHtml(text, maxLineLength, false);
 				label = new TechnologyLabel(html, tech, listener, false, true);
 			} else {
 				label = new TechnologyLabel(text, tech, listener, false, true);
 			}
         	 label.setOpaque(false);
         	 if (textSizeDiff != null) {
         		 label.changeFontSize(textSizeDiff, textBold);
         	 }
     		if (textIcon != null) {
     			label.setIcon(textIcon);
     		}
        	if (textColour != null) {
        		label.setForeground(textColour);
        	}
        	if (descriptionAsToolTip) {
        		label.setToolTipText(tech.getDescription());
        	}
    	 }
         	
    	 TechnologyIconPanel techIcon = null;
         if (displayIcons) {
    		techIcon = new TechnologyIconPanel(tech, iconSize);
        	if (descriptionAsToolTip) {
        		techIcon.setToolTipText(tech.getDescription());
        	}
         }
         
         if (techIcon != null && label != null) {
       	    if (itemControlPanelFactory != null) {
       	    	ComponentTable component = new ComponentTable("HV,HV,HV,HV,HV,HVSP", 1, 6, 0, 0, 0);
       	    	component.setOpaque(false);
       	    	component.add(itemControlPanelFactory.getControlPanel(value));
       	    	component.add(new PadPanel(1, false));
       	    	component.add(techIcon);
       	    	component.add(new PadPanel(1, false));
       	    	component.add(label);
       	    	component.add(new PadPanel(1, false),"HVSP");
       	    	component.doLayout();
       	    	return component;
       	    } else {
       	    	ComponentTable component = new ComponentTable("HV,HVSP", 1, 2, 0, 4, 0);
       	    	component.setOpaque(false);
       	    	component.add(techIcon);
       	    	component.add(label);
       	    	component.doLayout();
       	    	setBackground(isSelected, label);
       	    	return component;
       	    }
         } else if (techIcon != null) {
         	setBackground(isSelected, techIcon);
         	return techIcon;
         } else if (label != null) {
    	    if (itemControlPanelFactory != null) {
       	    	ComponentTable component = new ComponentTable("HV,HV,HV,HVSP", 1, 4, 0, 0, 0);
       	    	component.setOpaque(false);
       	    	//component.setBackground(Color.GREEN);
       	    	component.add(itemControlPanelFactory.getControlPanel(value));
       	    	component.add(new PadPanel(1, false));
       	    	component.add(label);
       	    	component.add(new PadPanel(),"HVSP");
       	    	component.invalidate();
       	    	component.doLayout();
       	    	return component;
       	    } else {
	         	setBackground(isSelected, label);
	         	return label;
       	    }
         } else {
        	return new JLabel("-");
         }
     }

	private void setBackground(boolean isSelected, JComponent component) {
		Color selectedBackgroundColor = PropertyManager.getSelectedBackgroundColor();
		if (isSelected) {
			if (selectedBackgroundColor != null) {
				component.setOpaque(true);
				component.setBackground(selectedBackgroundColor);
				component.setBorder(new LineBorder(Color.WHITE));
			}
		} else {
			if (backgroundColour != null) {
				component.setOpaque(true);
				component.setBackground(backgroundColour);
			}
		}
	}
 }
