/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.tables;

import java.awt.GridBagLayout;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mot.common.designs.CraftDesign;
import mot.common.technology.Computer;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.Engines;
import mot.common.technology.Hull;
import mot.common.technology.Hulls;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Technology;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;

public class CraftDesignTable extends JPanel implements ListSelectionListener {

    public static final int MAX_COST_WIDTH = 40;
    public static final int MIN_NAME_WIDTH = 100;
    public static final int MAX_SELECT_WIDTH = 25;
    public static final int MAX_QTY_WIDTH = 30;
    
    private boolean includeCosts;
    private ListSelectionListener listener;

    private TechnologyTablePanel hullTable;
    private TechnologyTablePanel engineTable;
    private TechnologyTablePanel computerTable;
    private WeaponBayTable weaponBayTable;
    private TechnologyTablePanel shieldTable;
    private TechnologyTablePanel scannerTable;
    private TechnologyTablePanel specialTable;

    public CraftDesignTable(CraftDesign cd, ListSelectionListener listener) throws ApplicationError {
        this(cd, false, listener);
    }
    
    public CraftDesignTable(CraftDesign cd, boolean includeCosts, ListSelectionListener listener) throws ApplicationError {
        this.includeCosts = includeCosts;
        this.listener = listener;

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        setDesign(cd);
    }
    
    public void updateDesign(CraftDesign cd) throws ApplicationError {
        Technology selection = getSelection();
        removeAll();
        setDesign(cd);
        setSelection(selection);
        // cannot update the model as some table may not exist becuase it did not have anthing before!
        //hullTable.getTable().getTechnologyModel().updateModel(new Hulls(cd.getHull()));
        //engineTable.getTable().getTechnologyModel().updateModel(new Engines(cd.getEngine()));
        //computerTable.getTable().getTechnologyModel().updateModel(cd.getComputers());
        //weaponBayTable.updateModel(cd.getWeaponBays());
        //shieldTable.getTable().getTechnologyModel().updateModel(cd.getShields());
        //scannerTable.getTable().getTechnologyModel().updateModel(cd.getScanners());
        //specialTable.getTable().getTechnologyModel().updateModel(cd.getSpecials());
    }

    private void setDesign(CraftDesign cd) throws ApplicationError {
        if (cd == null) {
            JPanel textPanel = new JPanel();
            textPanel.setLayout(new GridBagLayout());
            textPanel.add(new JLabel("Add or Select a Craft Design"));
            add(textPanel);
            return;
        }
        if (cd.getHull() != null) {
            addListLabel("Hull", this);
            hullTable = mot.client.swing.tables.HullTable.createWithHeader(cd.getHullTech(), includeCosts, this);
            add(hullTable);
        }
        if (cd.getEngine() != null) {
            addListLabel("Engine", this);
            engineTable = EngineTable.createWithHeader(cd.getEngineTech(), includeCosts, this, cd.getHullTech());
            add(engineTable);
        }
        if (cd.getComputers() != null) {
            addListLabel("Computers", this);
            computerTable = ComputerTable.createWithHeader(cd.getComputerTechs(), includeCosts, this);
            add(computerTable);
        }
        if (cd.getWeaponBays() != null && cd.getWeaponBays().getQtyWeapons() > 0) {
            //addListLabel("Weapons", panel);
            weaponBayTable = new WeaponBayTable(cd.getWeaponBays(), includeCosts, this);
            add(weaponBayTable);
        }
        if (cd.getShields() != null) {
            addListLabel("Shields", this);
            shieldTable = ShieldTable.createWithHeader(cd.getShieldTechs(), includeCosts, this);
            add(shieldTable);
        }
        if (cd.getScanners() != null) {
            addListLabel("Scanners", this);
            scannerTable = ScannerTable.createWithHeader(cd.getScannerTechs(), includeCosts, this);
            add(scannerTable);
        }
        if (cd.getSpecials() != null) {
            addListLabel("Specials", this);
            specialTable = SpecialTable.createWithHeader(cd.getSpecialTechs(), includeCosts, this);
            add(specialTable);
        }
    }
    
    private void addListLabel(String text, JPanel panel) {
        JLabel label = new JLabel(text);
        label.setFont(mot.client.swing.common.LocalDefaults.fontBold);
        //label.setBackground(Color.BLACK);
        //label.setForeground(Color.WHITE);
        label.setOpaque(true);
        //label.setHorizontalAlignment(JLabel.LEFT);
        //label.setMaximumSize(new Dimension(panel.getWidth(), 20));
        //label.setMinimumSize(new Dimension(panel.getWidth(), 20));
        panel.add(label, null);
    }
    
    /**
     * Called when a selection has changed in a list.
     */
    public void valueChanged(ListSelectionEvent e) {
        TechnologyListSelectionModel source = (TechnologyListSelectionModel) e.getSource();
        if (!source.getValueIsAdjusting()) return;
        if (hullTable != null && !(source.getAvailable() instanceof Hulls)) {
            hullTable.clearSelection();
        }
        if (engineTable != null && !(source.getAvailable() instanceof Engines)) {
            engineTable.clearSelection();
        }
        if (computerTable != null && !(source.getAvailable() instanceof Computers)) {
            computerTable.clearSelection();
        }
        if (weaponBayTable != null && !(source.getAvailable() instanceof Weapons)) {
            weaponBayTable.clearSelection(source.getSource());
        }
        if (shieldTable != null && !(source.getAvailable() instanceof Shields)) {
            shieldTable.clearSelection();
        }
        if (scannerTable != null && !(source.getAvailable() instanceof Scanners)) {
            scannerTable.clearSelection();
        }
        if (specialTable != null && !(source.getAvailable() instanceof Specials)) {
            specialTable.clearSelection();
        }
        if (listener != null) {
            listener.valueChanged(e);
        }
    }
    
    public void clearSelection() {
        if (hullTable != null) {
            hullTable.clearSelection();
        }
        if (engineTable != null) {
            engineTable.clearSelection();
        }
        if (computerTable != null) {
            computerTable.clearSelection();
        }
        if (weaponBayTable != null) {
            weaponBayTable.clearSelection();
        }
        if (shieldTable != null) {
            shieldTable.clearSelection();
        }
        if (scannerTable != null) {
            scannerTable.clearSelection();
        }
        if (specialTable != null) {
            specialTable.clearSelection();
        }
    }

    public Technology getSelection() {
        Technology selection = null;
        if (hullTable != null) {
            selection = hullTable.getSelectedTechnology();
            if (selection != null) return selection;
        }
        if (engineTable != null) {
            selection = engineTable.getSelectedTechnology();
            if (selection != null) return selection;
        }
        if (computerTable != null) {
            selection = computerTable.getSelectedTechnology();
            if (selection != null) return selection;
        }
        if (weaponBayTable != null) {
            selection = weaponBayTable.getSelectedTechnology();
            if (selection != null) return selection;
        }
        if (shieldTable != null) {
            selection = shieldTable.getSelectedTechnology();
            if (selection != null) return selection;
        }
        if (scannerTable != null) {
            selection = scannerTable.getSelectedTechnology();
            if (selection != null) return selection;
        }
        if (specialTable != null) {
            selection = specialTable.getSelectedTechnology();
            if (selection != null) return selection;
        }
        return selection;
    }

    public void setSelection(Technology tech) {
        if (tech == null) {
            clearSelection();
            return;
        }
        if (hullTable != null && tech instanceof Hull) {
            hullTable.setSelectedTechnology(tech);
        }
        if (engineTable != null && tech instanceof Engine) {
            engineTable.setSelectedTechnology(tech);
        }
        if (computerTable != null && tech instanceof Computer) {
            computerTable.setSelectedTechnology(tech);
        }
        if (weaponBayTable != null && tech instanceof Weapon) {
            weaponBayTable.setSelectedTechnology(tech);
        }
        if (shieldTable != null && tech instanceof Shield) {
            shieldTable.setSelectedTechnology(tech);
        }
        if (scannerTable != null && tech instanceof Scanner) {
            scannerTable.setSelectedTechnology(tech);
        }
        if (specialTable != null && tech instanceof Special) {
            specialTable.setSelectedTechnology(tech);
        }
    }

}