/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.battle;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.util.Date;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.animation.BattleAnimation;
import mot.client.animation.BattleAnimationList;
import mot.client.deligates.BattleAutomatonDelegate;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.BattleDroidMineDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.BattlePlanetDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.MissilesDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;

public class BattleMapPanel extends JPanel {
    
    private static final int OUTLINE_SIZE = 20;
    
    private static Cursor ATTACK_CURSOR_HIT = null;
    private static Cursor _getAttackCursorHit() {
		if (ATTACK_CURSOR_HIT == null) {
			Toolkit toolkit = Toolkit.getDefaultToolkit();
			Image image = ImageConfigFactory.getIconForConfigNoError("AttackCursorHit");
			if (image != null) {
				int cw = image.getWidth(null);
				int ch = image.getHeight(null);
				if (cw > 0 && ch > 0) {
					ATTACK_CURSOR_HIT = toolkit.createCustomCursor(image , new Point(cw/2, ch/2), "MotAttackCursor1");
				}
			}
			//ATTACK_CURSOR = new Cursor(Cursor.CROSSHAIR_CURSOR);
		}
		return ATTACK_CURSOR_HIT;
	}
	
    private static Cursor ATTACK_CURSOR_MISS = null;
    private static Cursor _getAttackCursorMiss() {
		if (ATTACK_CURSOR_MISS == null) {
			Toolkit toolkit = Toolkit.getDefaultToolkit();
			Image image = ImageConfigFactory.getIconForConfigNoError("AttackCursorMiss");
			if (image != null) {
				int cw = image.getWidth(null);
				int ch = image.getHeight(null);
				if (cw > 0 && ch > 0) {
					ATTACK_CURSOR_MISS = toolkit.createCustomCursor(image , new Point(cw/2, ch/2), "MotAttackCursor2");
				}
			}
			//ATTACK_CURSOR = new Cursor(Cursor.CROSSHAIR_CURSOR);
		}
		return ATTACK_CURSOR_MISS;
	}

	public Cursor getAttackCursor() {
		if (listener.getState().isCursorHit()) {
			return _getAttackCursorHit();
		} else {
			return _getAttackCursorMiss();
		}
	}

	
    private static Cursor EXPRESS_CURSOR_MISS = null;
    private static Cursor _getExpressCursorMiss() {
		if (EXPRESS_CURSOR_MISS == null) {
			Toolkit toolkit = Toolkit.getDefaultToolkit();
			Image image = ImageConfigFactory.getIconForConfigNoError("ExpressCursorMiss");
			if (image != null) {
				int cw = image.getWidth(null);
				int ch = image.getHeight(null);
				if (cw > 0 && ch > 0) {
					EXPRESS_CURSOR_MISS = toolkit.createCustomCursor(image , new Point(cw/2, ch/2), "MotExpressCursor1");
				}
			}
		}
		return EXPRESS_CURSOR_MISS;
	}
    
    private static Cursor EXPRESS_CURSOR_HIT = null;
    private static Cursor _getExpressCursorHit() {
		if (EXPRESS_CURSOR_HIT == null) {
			Toolkit toolkit = Toolkit.getDefaultToolkit();
			Image image = ImageConfigFactory.getIconForConfigNoError("ExpressCursorHit");
			if (image != null) {
				int cw = image.getWidth(null);
				int ch = image.getHeight(null);
				if (cw > 0 && ch > 0) {
					EXPRESS_CURSOR_HIT = toolkit.createCustomCursor(image , new Point(cw/2, ch/2), "MotExpressCursor2");
				}
			}
		}
		return EXPRESS_CURSOR_HIT;
	}

    private static Cursor busyCursor = null;
    private static Cursor getBusyCursor() {
		if (busyCursor == null) {
			busyCursor = new Cursor(Cursor.WAIT_CURSOR);
		}
		return busyCursor;
	}
    
	public Cursor getExpressCursor() {
		if (listener.getState().isCursorHit()) {
			return _getExpressCursorHit();
		} else {
			return _getExpressCursorMiss();
		}
	}

	
	private Cursor currentCursor = null;
	private Cursor defaultCursor = null;

	/**
     * True if the debugging log is on to tracer where time is being spent
     */
    private static boolean timerlogOn = false;
    
    private JScrollPane scrollPane = null;

	private BattleWindowListener listener;
    
    public BattleMapPanel(BattleWindowListener listener) throws ApplicationError {
    	this.listener = listener;

		setBackground(Color.BLACK);
        setOpaque(true);
        
        scrollPane = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED , JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        //scrollPane.addKeyListener(listener);

        scrollPane.getViewport().getView().addMouseMotionListener(listener);
        addMouseListener(listener);
        //addKeyListener(listener);
        
        defaultCursor = getCursor();
    }

	public void setSize(double scale) {
		int w = (int)(GalaxyDefinitions.BATTLE_BOUNDARY.getRight()*scale);
		int h = (int)(GalaxyDefinitions.BATTLE_BOUNDARY.getBottom()*scale);
		setSize(w, h);
        setPreferredSize(new Dimension(w, h));
	}
    
    public void paint(Graphics g) { 
        // JavaDoc: delegates to paintComponent, paintBorder, paintChildren
        // in that order
        super.paint(g); 
    }
    
    protected void paintComponent(Graphics g) {
        if (timerlogOn) Logger.logDebug("***** paint() Start "+new Date().getTime());
    	
//        if (Logger.isDebug()) {
//            Logger.logDebug("BattleWindow.painContent() "+MainWindow.getExestingInstance().getBattleWinConfig().getListener().getState().getAnimations());
//        }
        
        try {
        	// Must make sure all events are processed before a repaint because otherwise the
        	// events cannot veto obsolete craft status.
        	// Events are now processed before command responses (in Server Proxy)
            //MainWindow.getInstance().processMailBox();
            
            draw((Graphics2D)g);
        } catch (Exception e) {
            MainWindow.getExestingInstance().reportApplicationError(e);
        }
        
        if (timerlogOn) Logger.logDebug("***** paint() Finish "+new Date().getTime());
    }

    private String starMapNebula;
    private String starMap;
	private final String _getStarMap(BattleDelegate battle) throws ApplicationError {
		if (battle.inNebula()) {
			if (starMapNebula == null) {
				starMapNebula = ImageConfigFactory.getImageForConfig("BattleMapNebula");
			}
			return starMapNebula;
		} else {
			if (starMap == null) {			
				starMap = ImageConfigFactory.getImageForConfig("BattleMap");
			}
			return starMap;
		}
	}

    private String craftMovementIconHover;
	private final String _getCraftMovementIconHover() throws ApplicationError {
		if (craftMovementIconHover == null) {			
			craftMovementIconHover = ImageConfigFactory.getImageForConfig("CraftMovementIconHover");
		}
		return craftMovementIconHover;
	}
	
    private String craftMovementIconActual;
	private final String _getCraftMovementIconActual() throws ApplicationError {
		if (craftMovementIconActual == null) {			
			craftMovementIconActual = ImageConfigFactory.getImageForConfig("CraftMovementIconActual");
		}
		return craftMovementIconActual;
	}
	
    public void draw(Graphics2D graphics) throws ApplicationError {
    	MainWindow mainWin = MainWindow.getInstance();
    	if (!mainWin.isBattleMapConfigurationActive()) {
    		// Sometimes a repaint event can take time to arrive and arrive after the 
    		// battle configuration has been removed.
    		return;
    	}
    	BattleWindowListener listener = mainWin.getBattleWinConfig().getListener();
    	UserPlayerDelegate owner = listener.getOwner();
        BattleDelegate battle = listener.getBattle();
        BattleCraftDelegate [] crafts = battle.getCrafts();
        ImageObjectDelegate selected = listener.getState().getSelected();
        BattleAnimationList animations = listener.getState().getAnimations();
        int qtyAnimations = animations.getQuantity();
		if (crafts == null && qtyAnimations == 0) {
            return;
        }
        
    	BattleObjectDelegate next = null;
    	if (qtyAnimations == 0 && selected == null) {
        	next = battle.getNextObjectToMove();
        	PlayerDelegate nextOwner = null;
        	if (next != null) {
        		nextOwner = next.getOwnerDelegate();
        		if (nextOwner == owner) next = null;
        		else if (nextOwner.isAutomated()) next = null;
        	}
    	}
        
        String starMap = _getStarMap(battle);
		if (starMap != null) {
			ImageUtils.drawImage(this, graphics, starMap, 0, 0, getWidth(),getHeight());
			//ImageUtils.tileImage(this, graphics, starMap, new Area(0,0,getWidth(),getHeight()));
		}
        
        double scale = listener.getScale();
        graphics.scale(scale, scale);

        BattleAnimation [] nb = animations.getNonActiveAnimations(Boolean.FALSE);
        if (nb!= null) {
            for (int i=0; i<nb.length; i++) {
                nb[i].paintNonActive(graphics, animations);
            }
        }

        BattleAnimation [] ab = animations.getActiveAnimations(Boolean.FALSE);
        if (ab != null) {
            for (int i=0; i<ab.length; i++) {
            	//Logger.logDebug("Paint BEFORE "+ab[i]+" stage="+ab[i].getStage()+" life="+ab[i].getLife());
                ab[i].paintActive(graphics);
            }
        }

        AnimatedBattleAreaDetails [] abad = battle.getAnnimatedBattleAreas();
        if (abad != null) {
        	for (int i=0; i < abad.length; i++) {
        		if (!animations.vetoDrawAnimatedArea(abad[i])) {
        			ImageUtils.drawAnimatedBattleArea(this, graphics, abad[i]);
        		}
        	}
        }
        
        if (battle.getPlanet() != null) {
            BattlePlanetDelegate bp = battle.getPlanet();
            if (!animations.vetoDrawObject(bp, null)) {
            	boolean isNext = (bp == next);
                ImageUtils.drawBattlePlanet(mainWin, graphics, bp, selected, isNext);
            }
        }

        if (crafts != null) {
	        for (int i=0; i<crafts.length; i++) {
	            if (!animations.vetoDrawObject(crafts[i], null)) {
	            	boolean isNext = (crafts[i] == next);
	                ImageUtils.drawBattleCraft(mainWin, graphics, crafts[i], selected, crafts[i].getState(), isNext);
	            }
	        }
        }
        
        BattleAnimation [] na = animations.getNonActiveAnimations(Boolean.TRUE);
        if (na!= null) {
            for (int i=0; i<na.length; i++) {
                na[i].paintNonActive(graphics, animations);
            }
        }

        BattleAnimation [] aa = animations.getActiveAnimations(Boolean.TRUE);
        if (aa != null) {
            for (int i=0; i<aa.length; i++) {
            	//Logger.logDebug("Paint AFTER "+aa[i]+" stage="+aa[i].getStage()+" life="+aa[i].getLife());
                aa[i].paintActive(graphics);
            }
        }

        BattleAutomatonDelegate[] al = battle.getAutomatons();
        if (al != null) {
            for (int i=0; i<al.length; i++) {
                if (!animations.vetoDrawObject(al[i], null)) {
                	int direction = al[i].getDirection();
                    Location location = al[i].getLocation();
                	if (al[i] instanceof MissilesDelegate) {
                		MissilesDelegate d = (MissilesDelegate) al[i];
                		Location targetLocation = animations.getOldestLocationFor(d.getTarget());
                		if (targetLocation != null) {
                			direction = location.getDirectionFor(targetLocation);
                		}
                	} else if (al[i] instanceof BattleDroidMineDelegate) {
                		BattleDroidMineDelegate d = (BattleDroidMineDelegate) al[i];
                		Location targetLocation = animations.getOldestLocationFor(d.getTarget());
                		if (targetLocation != null) {
                			direction = location.getDirectionFor(targetLocation);
                		}
                	}
					ImageUtils.drawRotatedImageDeligate(this, graphics, al[i], location, direction);
                }
            }
        }
        
        BattleAnimation [] ppa = animations.getActiveAnimations(null);
        if (ppa != null) {
            for (int i=0; i<ppa.length; i++) {
                ppa[i].paintPostProcess(graphics, true, animations);
            }
        }

        BattleAnimation [] ppn = animations.getNonActiveAnimations(null);
        if (ppn != null) {
            for (int i=0; i<ppn.length; i++) {
                ppn[i].paintPostProcess(graphics, false, animations);
            }
        }
        
        graphics.scale(1.0, 1.0);
        
        if (listener.isShowingAnimations()) {
        	currentCursor = getBusyCursor(); 
			setCursor(currentCursor);
        } else if (selected != null && listener.isControlled(selected) && !listener.getView().getAutoButton().isSelected()) {
    		//if (defaultCursor == null) defaultCursor = getCursor();
			if (listener.isBattlePlanet(selected) || listener.isOrbital(selected) || listener.isAttack()) {
    			Cursor attackCursor = getAttackCursor();
    			if (attackCursor != null && currentCursor != attackCursor) {
    				setCursor(attackCursor);
    			}
    		} else {
    			if (listener.isExpress()) {
        			Cursor expressCursor = getExpressCursor();
        			if (expressCursor != null && currentCursor != expressCursor) {
        				setCursor(expressCursor);
        			}
    			} else {
    				if (defaultCursor != null && currentCursor != defaultCursor) {
    					setCursor(defaultCursor);
    				}
    			}
    			LocationAndDirection hoverLocation = listener.getState().getMouseOverMoveToDest();
    			String craftOutline = _getCraftMovementIconHover();
    			if (hoverLocation != null && craftOutline != null) {
    				int width = OUTLINE_SIZE, height = OUTLINE_SIZE;
    				if (selected != null && selected instanceof ImageObjectDelegate) {
    					Size imageSize = ((ImageObjectDelegate)selected).getImageSize(); 
    					width = imageSize.getX(); 
    					height = imageSize.getY(); 
    				}
    				int imageLeft = hoverLocation.getX()-width/2;
    				int imageTop = hoverLocation.getY()-height/2+1;
    				int direction = hoverLocation.getDirection();
    				ImageUtils.drawRotatedImage(this, graphics, craftOutline, imageLeft, imageTop, width, height, null, direction);
    				
    				LocationAndDirection actualLocation = listener.getState().getMouseOverActualDest();
    				String actualOutline = _getCraftMovementIconActual();
    				if (actualLocation != null) {
    					int actualLeft = actualLocation.getX()-width/2;
    					int actualTop = actualLocation.getY()-height/2;
    					int actualDirection = actualLocation.getDirection();
    					ImageUtils.drawRotatedImage(this, graphics, actualOutline, actualLeft, actualTop, width, height, null, actualDirection);
    				}
    			}
    		}
    	} else {
			if (defaultCursor != null && currentCursor != defaultCursor) {
				setCursor(defaultCursor);
			}
    	}
    }

	public JScrollPane getScrollPane() {
		return scrollPane;
	}


}
