/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.battle;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.util.Date;

import mot.client.animation.BattleAnimationList;
import mot.client.deligates.BattleAutomatonDelegate;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.BattleDroidMineDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.BattlePlanetDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.MissilesDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.RelationDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.sounds.Sounds;
import mot.client.swing.window.battle.actions.MotBattleMapActions;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowListener;
import mot.client.swing.window.main.MotListener;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.Area;
import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.ImageObjectSettings;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.NameList;
import mot.common.bean.Settings;
import mot.common.bean.WreckageDetails;
import mot.common.designs.CraftDesign;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.vo.WeaponInstanceVO;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;

public class BattleWindowListener extends MainWindowListener implements MotListener, MotBattleMapActions {

    /**
     * This is the battle being fought in this window.
     */
    private BattleDelegate battle;
    
	/**
	 * This holds the current state of the galaxy map.
	 * E.g. What MOO is selected
	 */
	private BattleWindowState state;
	
	/**
	 * This is the view that this listener uses to display its results.
	 */
	private BattleWindowView view;
	
	/**
	 * It is possible to pause the battle using the p key.
	 */
	boolean paused = false;

	/**
	 * The zoom scaling to be applied to the battle map panel.
	 */
	private Double scale = null;

	/**
	 * Events from battles are broadcast to all players....
	 * So we may receive events from another battle after the battle in this window has closed.
	 * As a the battle window does not close until after all animations have been shown - you will get to see half a battle!
	 */
    private boolean endBattleRecieved = false;

    /**
     * True if the debugging log is on to tracer where time is being spent
     */
    private static boolean timerlogOn = false;    

    private boolean focusOnAnimation = false;
    private boolean first = true;
    
	public BattleWindowListener() throws ApplicationError {
		this.state = new BattleWindowState(null);
		
        ticksToMoveACraft = Settings.getInstance().getCraftMoveDelay();
	}
	
    private static final int REFRESHES_BETWEEN_CRAFT_MOVES = 10;
    private int ticksToMoveACraft = REFRESHES_BETWEEN_CRAFT_MOVES;
    
	private int lastMouseX;
	private int lastMouseY;

    public void handleTick() throws ApplicationError, ValidationError {
        if (timerlogOn) Logger.logDebug("***** handleTick() Start "+new Date().getTime());
        if (paused) return;
        
        if (first) {
        	view.focus(GalaxyDefinitions.BATTLE_BOUNDARY.getCenterLocation().getArea(10));
        	first = false;
        }

        ticksToMoveACraft--;
        
        BattleAnimationList animations = state.getAnimations();
        
        // To avoid infinite loop caused by an animation over an area so big that it cannot all be displayed
        if (!focusOnAnimation) {        	
        	Area animationArea = animations.getActiveAnimationArea(null);
        	if (animationArea != null && !view.isVisible(animationArea)) {
				focusOnAnimation = true;
                //Logger.logDebug("NOT VISIBLE - "+animationArea+" not in "+view.getMapPanel().getScrollPane().getViewport().getViewRect());
				view.focus(animationArea);
				return;
        	}
        }
        
        focusOnAnimation = false;
        int speed = view.getControlPanel().getSelectWeaponPanel().getSpeed();

        if (animations.tick(speed)) {
            animations.playSounds();
            view.getMapWindow().repaint();
            if (timerlogOn) Logger.logDebug("***** handleTick() End after animations.tick() "+new Date().getTime());
            return;
        } else {
            animations.playSounds();
        }
        if (animations.getQuantity() > 0) {
            if (timerlogOn) Logger.logDebug("***** handleTick() End have animations "+new Date().getTime());
            return;
        }
        
        if (battle.hasEnded()) {
        	MainWindow mainWindow = MainWindow.getInstance();
        	// This is probably not good enough because the last few events of the battle make take time to arrive! 
            if (animations.getQuantity() == 0 && !mainWindow.getEventMailBox().isNextEventBattleEvent()) {
                mainWindow.getGalaxyMapConfiguration().getGalaxyMapPanel().rebuildContent();
    			mainWindow.toGalaxyWindow();
    			// Inform the server we are closing the battle window so it can broadcast any remaining events.
    			ProxyStore.getInstance().battleEndTurnCmd(battle, null, true, true, null, -1, null, mainWindow);
            }
            return;
        }

        if (ticksToMoveACraft <= 0) {
            autoMove();
            view.getMapWindow().repaint();
            ticksToMoveACraft = REFRESHES_BETWEEN_CRAFT_MOVES;
        } else {
        	view.getMapWindow().repaint();
        }
        
        if (timerlogOn) Logger.logDebug("***** handleTick() End "+new Date().getTime());
        return;
    }

    private void autoMove() throws ApplicationError, ValidationError {
    	BattleObjectDelegate next = battle.getNextObjectToMove();
    	ImageObjectDelegate selected = state.getSelected();
    	if (state.isUserSelected() && selected != null && selected instanceof BattleObjectDelegate) {
    		next = (BattleObjectDelegate) selected;
    	}
    	
    	UserPlayerDelegate owner = getOwner();
    	if (next != null) {
            boolean controlledByOwner = next.isControlledBy(owner);
        	if (controlledByOwner && view.getAutoButton().isSelected()) {
                if ( !view.isVisible(next) ) {
                    //Logger.logDebug("AUTO MOVE AI- focus "+craft);
                	view.focus(next);
                } else {
                    //Logger.logDebug("AUTO MOVE AI - orders "+craft);
                    ProxyStore.getInstance().battleGetNextOrdersCmd(battle, next, null);
                }
            } else if (selected != null && selected.isDestroyed()) {
            	state.setSelected(null, false);
            	CompletedCallback callback = new CompletedCallback() {
					public void complete(Object result) throws ApplicationError, ValidationError {
						next();
					}
				};
            	ProxyStore.getInstance().battleGetNextObject(battle, callback, null);
            } else {
				boolean isUserSelected = state.isUserSelected();
				if (controlledByOwner && !isUserSelected && !Comp.same(next, selected)) {
                    // User can select nothing and we should not then select the next craft if user has selected
                	// If craft is auto selected, must not re-select because user may select a particular weapon to fire
            		//Logger.logDebug("AUTO SELECT - "+next);
            		select(next, false);
                } else if (!controlledByOwner && !isUserSelected) {
            		//Logger.logDebug("AUTO SELECT - null");
            		select(null, false);
                } else if ( !view.isVisible(next) ) {
            		//Logger.logDebug("AUTO FOCUS - "+next);
            		view.focus(next);
                }
            }
    	}
    }

    public void autoSelectNext() {
    	state.setUserSelected(false);
    	ticksToMoveACraft = REFRESHES_BETWEEN_CRAFT_MOVES; 
    } 

    public void next() throws ApplicationError, ValidationError {
    	BattleObjectDelegate craft = battle.getNextObjectToMove();
        if (craft != null) {
        	UserPlayerDelegate owner = getOwner();
            if (craft.isControlledBy(owner)) {
                select(craft, false);
            } else {
            	view.focus( craft );
            }
        }
    }

    public void select(ImageObjectDelegate io, boolean isUserSelected) throws ApplicationError, ValidationError {
        //Logger.logDebug("Select "+io);
        state.setSelected(io, isUserSelected);
        view.focus(io);
        view.setDescription(io);
        view.getControlPanel().updateWeaponSelect(io);
        selectModeFor(io);
        calculateMouseOverInformation(lastMouseX, lastMouseY);
    }
    
    public boolean isExpress() {
    	boolean isExpress = view.getControlPanel().getModeButton().isExpress();
    	return isExpress;
    }
    
    public boolean isMove() {
    	boolean isMove = view.getControlPanel().getModeButton().isMove();
    	return isMove;
    }
    
    public boolean isAttack() {
    	boolean isAttack = view.getControlPanel().getModeButton().isAttack();
    	return isAttack;
    }
    
    public void moveToNextMode(BattleObjectDelegate selectedObject, Weapon selectedWeapon, ImageObjectDelegate target, boolean stopped) throws ApplicationError, ValidationError {
    	UserPlayerDelegate owner = getOwner();
		if (selectedObject.isControlledBy(owner)) {
	        if (isExpress()) {
	        	if (stopped) {
	        		// Craft was hit by a missile and halted its move, we have not performed requested task
	        		return;
	        	}
	        	if (target != null) {
		    		if (hasPrimaryWeaponsReadyFire(selectedObject)) {
		    			// Target I shot at did not consume my main weapons - must be more to do.
		    			return;
		    		}
		    		if (selectedWeapon != null && hasWeaponsToFire(selectedObject)) {
		    			// User has fired a single weapon type and has more to fire
		    			return;
		    		}
		    		if (target instanceof BattleAutomatonDelegate && hasWeaponsToFire(selectedObject)) {
		    			// User has attacked a missile but still has more weapons to fire 
		    			return;
		    		}
	        	}
	    		if (hasTimeWarpGeneratorToFire(selectedObject)) {
	                ProxyStore.getInstance().battleAttackCmd(battle, selectedObject, selectedObject, false, false, false, null, getTimeWarpWeapon(selectedObject).getWeapon(), 1, null, null);
	                return;
	    		}
	    		ProxyStore.getInstance().battleEndTurnCmd(battle, selectedObject, true, true, null, -1, null, null);
	            autoSelectNext();
	        } else if (isMove()) {
	        	if (!isCraftWithMoveRemaining(selectedObject)) {
	        		view.getControlPanel().getModeButton().setModeAttack();
	        	}
	        } else if (isAttack()) {
	    		if (hasTimeWarpGeneratorAndAtStartOfTurn(selectedObject)) {
	    			// Assume we have just fired Time Warp Generator to move again so we do not end this craft's turn.
	    			return;
	    		}
	        	if (!hasWeaponsToFire(selectedObject)) {
	        		autoSelectNext();
	        	}
	        }
		} else {
			// I only moved this because a mind control weapon allowed me to. 
			// Have now lost control - so move onto my next craft to move!
			autoSelectNext();
		}
    }

    public void selectModeFor(ImageObjectDelegate selectedImage) throws ApplicationError, ValidationError {
    	if (selectedImage instanceof BattleObjectDelegate) {
    		BattleObjectDelegate selectedObject = (BattleObjectDelegate) selectedImage;
	        if (isMove()) {
	        	if (!isCraftWithMoveRemaining(selectedObject)) {
	        		view.getControlPanel().getModeButton().setModeAttack();
	        	}
	        } else if (isAttack()) {
	        	if (isCraftWithMoveRemaining(selectedObject)) {
	        		view.getControlPanel().getModeButton().setModeMove();
	        	}
	        }
    	}
    }
    
    private boolean hasTimeWarpGeneratorToFire(BattleObjectDelegate bo) throws ApplicationError {
    	WeaponInstanceVO weapon = getTimeWarpWeapon(bo);
    	return (weapon != null &&  weapon.hasWeaponsToFire());
    }
        
    private boolean hasTimeWarpGeneratorAndAtStartOfTurn(BattleObjectDelegate bo) throws ApplicationError {
    	WeaponInstanceVO weapon = getTimeWarpWeapon(bo);
    	if (weapon == null) {
    		return false;
    	}
    	boolean hasFullMove = true;
    	if (bo instanceof BattleCraftDelegate) {
    		BattleCraftDelegate bc = (BattleCraftDelegate) bo;
    		hasFullMove = bc.getMoveRemainingAsPercent() > 0.99;
    	}
    	 return hasFullMove && allWeaponsReadyFire(bo);
    }
        
    private WeaponInstanceVO getTimeWarpWeapon(BattleObjectDelegate bo) throws ApplicationError {
    	WeaponInstanceVO [] weapons = bo.getWeapons();
    	if (weapons != null) {
	    	for (int i = 0; i < weapons.length; i++) {
				if (weapons[i].getWeapon().isTimeWarp()) {
					return weapons[i];
				}
			}
    	}
    	return null;
    }

    public boolean hasWeaponsToFire(BattleObjectDelegate bo) throws ApplicationError {
    	WeaponInstanceVO [] weapons = bo.getWeapons();
    	if (weapons != null) {
	    	for (int i = 0; i < weapons.length; i++) {
				if (weapons[i].hasWeaponsToFire()) {
					return true;
				}
			}
    	}
    	return false;
    }
        
    private boolean allWeaponsReadyFire(BattleObjectDelegate bo) throws ApplicationError {
    	WeaponInstanceVO [] weapons = bo.getWeapons();
    	if (weapons != null) {
	    	for (int i = 0; i < weapons.length; i++) {
				if (weapons[i].getQtyFired() == 0) {
					return false;
				}
			}
    	}
    	return true;
    }
        
    private boolean hasPrimaryWeaponsReadyFire(BattleObjectDelegate bo) throws ApplicationError {
    	WeaponInstanceVO [] weapons = bo.getWeapons();
    	if (weapons != null) {
	    	for (int i = 0; i < weapons.length; i++) {
	    		if (Comp.in(weapons[i].getWeapon().getGroup(), Weapon.STOP_END_TURN_ON_EXPRESS_GROUPS)) {
	    			if (weapons[i].getQtyFired() < weapons[i].getQty()) {
	    				return true;
	    			}	    			
	    		}
			}
    	}
    	return false;
    }
    
    public BattleObjectDelegate getBattleObject(String name) {
    	return battle.getDelegateCache().getBattleObject(name);
    }
        
    //
    // KEY LISTENER STUFF
    //
    public void motKeyTyped(KeyEvent e) throws ApplicationError, ValidationError { 
    	if (Character.toLowerCase(e.getKeyChar()) == 'p') {
    		paused = !paused;
    	} else if (Character.toLowerCase(e.getKeyChar()) == 'f') { // fire (no move)
    		attack(false);
    	} else if (Character.toLowerCase(e.getKeyChar()) == 'c') { // computer attack (fire with move)
    		attack(true);
    	} else if (Character.toLowerCase(e.getKeyChar()) == 'e') {
    		view.getControlPanel().getModeButton().setModeExpress();
    	} else if (Character.toLowerCase(e.getKeyChar()) == 'm') {
    		view.getControlPanel().getModeButton().setModeMove();
    	} else if (Character.toLowerCase(e.getKeyChar()) == 'a') {
    		view.getControlPanel().getModeButton().setModeAttack();
    	} else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) { // complete
    		complete();
    	} else if (Character.toLowerCase(e.getKeyChar()) == '1') {
    		if (Logger.isDebug()) {
	    		Battle b = MotEngine.instance.getBattles().getActiveBattle();
	    		BattleCraft c0 = b.getCraft("Military18_3");
	    		BattleCraft c1 = b.getCraft("Invader6_1");
	    		BattleCraft c2 = b.getCraft("ternimator47");
	    		BattleCraft c3 = b.getCraft("Invader9_3");
	    		BattleCraft c4 = b.getCraft("ternimator117");
	    		BattleCraft c5 = b.getCraft("fighter5102");
	    		if (c5 != null) {
	    			c5.setLocation(new Location(400, 500));
	    		}
	    		if (c3 != null) {
	    			//c3.setLocation(new Location(511,500));
	    			//c3.getCraft().setDamage(c1.getCraft().getMaxDamage()-1);
//	    			c3.setMoveRemaining(0.0);
//	    			c3.setLocation(new Location(600,550));
//	    			c3.setDirection(270);
//	    			c0.setMoveRemaining(1.0);
//	    			c0.setLocation(new Location(600,500));
//	    			c0.setDirection(270);
//	    			c0.moveTo(new LocationAndDirection(600,500, 90), 1.0, new Location(600,550), c3, false);
//	    			c0.fireAllWeapons(c3);
//	    			c0.moveTo(new LocationAndDirection(600,500, 270), 1.0, null, null, false);
	    			
//	    			c1.setMoveRemaining(0.0);
//	    			c1.getCraft().setDamage(c1.getCraft().getMaxDamage()-1);
//	    			c1.setLocation(new Location(400,500));
//	    			c2.setMoveRemaining(1.0);
//	    			c2.setLocation(new Location(350,500));
//	    			c3.setMoveRemaining(0.0);
//	    			c3.setLocation(new Location(350,450));
//	    			Weapon w = Technologies.getWeapon("ShieldBurst");
//	    			c2.fireWeapon(c1, c2.getWeapon(w), 1);
	    		}
    		}
    	}
    }

    public void motMouseClicked(MouseEvent e) {
        try {
        	try {
        		if (isShowingAnimations()) {
        			throw new ValidationError(L.INVALID_COMMAND_BUSY);
        		}
	            if (e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1) {
	                leftClicked(e);
	            }
	            if (e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON3) {
	                rightClicked(e);
	            }
	        } catch (ValidationError ex) {
	        	view.reportValidationError(ex);
	            try {
					Sounds.getErrorSound().play();
				} catch (ApplicationError exc) {
		        	Logger.logError("Sound failed", exc);
				}
        	}
        } catch (Exception ex) {
        	view.reportApplicationError(ex);
        }
    }

    public void leftClicked(MouseEvent e) throws ApplicationError, ValidationError {
    	BattleDelegate battle = getBattle();
    	ImageObjectDelegate selected = getState().getSelected();
    	double scale = getScale();
        int x = (int) (e.getX() / scale);
		int y = (int) (e.getY() / scale);
   		Location clickLocation = new Location(x, y);
        int modifiers = e.getModifiers();
		boolean preferPrimaryObjects = !((modifiers & ActionEvent.SHIFT_MASK) > 0);
		ImageObjectDelegate target = battle.getDelegateCache().getObjectAt(x, y, preferPrimaryObjects);
        if (Logger.isDebug()) {
            Logger.logDebug("BattleWindow.mouseClicked() "+x+","+y+" => "+((target == null)?"NULL":target.getName()));
        }
        if (selected == null) {
            select(target, true);
        } else {
	        Weapon weapon = view.getSelectWeaponPanel().getWeapon();
	        int qtyToFire = view.getSelectWeaponPanel().getQty();
            boolean missileDefenceOn = isMissileDefenceOn();
        	if ((modifiers & ActionEvent.ALT_MASK) > 0) {        		
            	if (selected instanceof BattleObjectDelegate) {
            		BattleObjectDelegate selectedObject = (BattleObjectDelegate)selected;
    				CompletedCallback callback = new BattleObjectCommandCallback(this, selectedObject, target, true);
        			if (isMove()) {
                		if (selected instanceof BattleCraftDelegate) {
                			BattleCraftDelegate selectedCraft = (BattleCraftDelegate)selected;
	        				Location sol = selectedCraft.getLocation();
	        				int sod = sol.getDirectionFor(clickLocation);
	        				ProxyStore.getInstance().battleCraftMoveToCmd(battle, selectedCraft, new LocationAndDirection(sol, sod), null, false, missileDefenceOn, callback, null);
                		}
        			} else {
        				WeaponInstanceVO shieldDrains = getShieldDrainWeapons(selectedObject.getWeapons());
        				if (shieldDrains != null) {
        					ProxyStore.getInstance().battleAttackCmd(battle, selectedObject, target, false, false, missileDefenceOn, null, shieldDrains.getWeapon(), -1, callback, null);
        				} else {
        					throw new ValidationError("Invalid Command - ALT key is TURN ONLY in move mode and Fire SHIELD DRAIN ONLY in express and attack mode");
        				}
        			}
        		}
        	} else if ( (modifiers & ActionEvent.CTRL_MASK) > 0 || (weapon != null && weapon.onlyAttackLocation()) ) {        		
            	if (selected instanceof BattleObjectDelegate) {
            		BattleObjectDelegate selectedObject = (BattleObjectDelegate)selected;
    				CompletedCallback callback = new BattleObjectCommandCallback(this, selectedObject, target, false);
            		if (isMove()) {
                        LocationAndDirection attackFrom = state.getMouseOverMoveToDestNotNull(clickLocation, selectedObject);
                        ProxyStore.getInstance().battleCraftMoveToCmd(battle, (BattleCraftDelegate)selectedObject, attackFrom, target, true, missileDefenceOn, callback, null);
            		} else {
            	        ProxyStore.getInstance().battleAttackLocationCmd(battle, selectedObject, clickLocation, weapon, qtyToFire, callback, null);
            		}
            	}
        	} else if (target == null) {
            	if (selected instanceof BattleCraftDelegate) {
            		BattleCraftDelegate selectedCraft = (BattleCraftDelegate)selected;
    				CompletedCallback callback = new BattleObjectCommandCallback(this, selectedCraft, target, false);
    				LocationAndDirection moveToDest = state.getMouseOverMoveToDestNotNull(new Location(x, y), selectedCraft);
            		ProxyStore.getInstance().battleCraftMoveToCmd(battle, selectedCraft, moveToDest, null, true, missileDefenceOn, callback, null);
                }
            } else {
            	UserPlayerDelegate owner = getOwner();
                if (target.isSelectableBy(owner)) {
                	select(target, true);
                } else {
                	if (selected instanceof BattleObjectDelegate) {
                    	BattleObjectDelegate selectedObject = (BattleObjectDelegate)selected;
                        LocationAndDirection attackFrom = state.getMouseOverMoveToDestNotNull(clickLocation, selectedObject);
                    	if (isMove()) {
                    		if (selectedObject instanceof BattleCraftDelegate) {
                				CompletedCallback callback = new BattleObjectCommandCallback(this, selectedObject, target, false);
                    			ProxyStore.getInstance().battleCraftMoveToCmd(battle, (BattleCraftDelegate)selectedObject, attackFrom, target, true, missileDefenceOn, callback, null);
                    		}
                    	} else if (isAttack()) {
            				CompletedCallback callback = new BattleObjectCommandCallback(this, selectedObject, target, true);
                    		ProxyStore.getInstance().battleAttackCmd(battle, selectedObject, target, false, false, missileDefenceOn, null, weapon, qtyToFire, callback, null);
                    	} else {
            				CompletedCallback callback = new BattleObjectCommandCallback(this, selectedObject, target, false);
                    		ProxyStore.getInstance().battleAttackCmd(battle, selectedObject, target, true, true, missileDefenceOn, attackFrom, weapon, qtyToFire, callback, null);
                    	}
                	}
                }
            }
        }
    }
    
    public void rightClicked(MouseEvent e) throws ApplicationError, ValidationError {
    	BattleDelegate battle = getBattle();
    	ImageObjectDelegate selected = getState().getSelected();
    	double scale = getScale();
        int x = (int) (e.getX() / scale);
		int y = (int) (e.getY() / scale);
		ImageObjectDelegate obj = battle.getDelegateCache().getObjectAt(x, y, false);
        select(obj, true);
        view.getMapWindow().repaint();
    }

    public boolean isCraftWithMoveRemaining(ImageObjectDelegate o) {
        if (o == null) {
        	return false;
        } else if (o instanceof BattleCraftDelegate && !((BattleCraftDelegate)o).isOrbital()) {
			BattleCraftDelegate selectedCraft = (BattleCraftDelegate)o;
        	return selectedCraft.getMoveRemainingAsPercent() > 0.0;
        } else {
        	return false;
        }
    	
    }

    public boolean isBattlePlanet(ImageObjectDelegate o) {
		if (o != null && o instanceof BattlePlanetDelegate) {
    		return true;
    	}
       	return false;
    }

    public boolean isOrbital(ImageObjectDelegate o) {
    	if (o != null && o instanceof BattleCraftDelegate && ((BattleCraftDelegate)o).isOrbital()) {
    		return true;
    	}
       	return false;
    }


    private WeaponInstanceVO getShieldDrainWeapons(WeaponInstanceVO[] weapons) throws ApplicationError {
    	for (int i = 0; i < weapons.length; i++) {
    		if (weapons[i].getWeapon().isShieldDrain()) {
    			return weapons[i];
    		}
		}
    	return null;
	}

    private void attack(boolean withMove) throws ApplicationError, ValidationError{
    	ImageObjectDelegate selected = state.getSelected();
    	BattleDelegate battle = getBattle();
    	if (selected instanceof BattleObjectDelegate) {
        	BattleObjectDelegate selectedObject = (BattleObjectDelegate)selected;
			CompletedCallback callback = new BattleObjectCommandCallback(this, selectedObject, null, false);
			ProxyStore.getInstance().battleAttackCmd(battle, selectedObject, null, withMove, withMove, true, null, null, -1, callback, null);
    	}
    }
    
    private void calculateMouseOverMoveToDest(ImageObjectDelegate mouseOverObject, Location mouseLocation) throws ApplicationError {
    	BattleWindowState state = getState();
    	ImageObjectDelegate selectedObject = state.getSelected();
    	state.setMouseOverObject(mouseOverObject);
    	state.setMouseOverMoveToDest(null);
    	if (selectedObject != null && selectedObject instanceof BattleCraftDelegate) {
    		BattleCraftDelegate selectedCraft = (BattleCraftDelegate) selectedObject;
	    	if (mouseOverObject != null && mouseOverObject instanceof BattleObjectDelegate) {
	    		BattleObjectDelegate mouseOverBO = (BattleObjectDelegate) mouseOverObject;
	    		UserPlayerDelegate owner = getOwner();
	    		if (selectedCraft.isControlledBy(owner) && isEnemy(mouseOverBO.getOwner())) {
	    			if (mouseLocation != null) {
	    				Location tl = mouseOverObject.getLocation();
	    				int direction = mouseLocation.getDirectionFor(tl);
	    				Location location = new Location(tl);
	    				int selectionSize = selectedObject.getImageSize().getX();
	    				int mouseOverObjectSize = mouseOverObject.getImageSize().getX();
	    				int cornerFactor =  Comp.pos((direction % 90) - 45) / 10;
						location.shiftNoLimit(mouseLocation, mouseOverObjectSize/2+selectionSize/2+ImageObjectSettings.INTER_CRAFT_GAP+cornerFactor+2);
						state.setMouseOverMoveToDest( new LocationAndDirection(location.getX(), location.getY(), direction) );
	    			}
	    		}
	    	} else {
				state.setMouseOverMoveToDest( new LocationAndDirection(mouseLocation, selectedCraft.getLocation().getDirectionFor(mouseLocation)) );
	    	}
    	}
    }
    
    private boolean isTargetAngleWithinWeaponFiringAngle(BattleObjectDelegate source, Location target) throws ApplicationError {
    	if (!(source instanceof BattleCraftDelegate)) return true;
    	BattleCraftDelegate sourceCraft = (BattleCraftDelegate) source;
    	
    	CraftDesign cd = sourceCraft.getCraft().getCraftDesign();
    	if (cd == null || cd.getHull() == null) return true;
    	Hull hull = cd.getHullTech();
    	WeaponBayDetails wbd = cd.getHullTech().getWeaponBayDetails(0);
    	
		if (wbd != null) {
			int targetDirection = sourceCraft.getLocation().getDirectionFor(target);
			int direction = sourceCraft.getDirection();
			double reqdDirectionChange = Comp.getDirectionChangeFor(direction, targetDirection);
			if (reqdDirectionChange < 0) reqdDirectionChange = -reqdDirectionChange;
			if (reqdDirectionChange > wbd.getMaxFireAngle()) {
				return false;
			}
    	}
		return true;
	}
    
    private boolean inWeaponRange(BattleObjectDelegate source, Location target) throws ApplicationError {
		int distance = source.getLocation().getDistanceTo( target );
		
        Weapon weapon = view.getSelectWeaponPanel().getWeapon();
        
        if (weapon != null) {
        	WeaponInstanceVO wi = source.getWeapon(weapon);
			int range = wi.getRange();
        	return distance < range;
        } else {
        	long maxRange = 0;
        	WeaponInstanceVO[] weapons = source.getWeapons();
        	if (weapons == null) {
        		return false;
        	}
        	for (int i=0; i<weapons.length; i++) {
        		if (weapons[i].hasWeaponsToFire()) {
        			int range = weapons[i].getRange();
        			if (range > maxRange) {
        				maxRange = range;
        			}
        		}
        	}
        	return distance < maxRange;
        }
        
    }
    
    public boolean isTargetAngleWithinWeaponFiringAngle(ImageObjectDelegate source, Location target) throws ApplicationError {
    	if (source instanceof BattleObjectDelegate) {
    		BattleObjectDelegate sourceBattleObject = (BattleObjectDelegate) source;
    		return isTargetAngleWithinWeaponFiringAngle(sourceBattleObject, target);
    	} else {
    		return false;
    	}
    }
    
    public boolean isControlled(ImageObjectDelegate io) throws ApplicationError {
    	if (io != null && io instanceof BattleObjectDelegate) {
        	UserPlayerDelegate owner = getOwner();
    		BattleObjectDelegate bo = (BattleObjectDelegate) io;
    		return bo.isControlledBy(owner);
    	}
    	return false;
    }
    
    public boolean isEnemy(String planerName) throws ApplicationError {
    	return isEnemy(DeligateCacheManager.getPlayer(planerName));
    }
    
    public boolean isEnemy(PlayerDelegate p) throws ApplicationError {
    	if (p == null) return false;
    	if (p.isMonsterPlayer()) return true;
    	UserPlayerDelegate owner = getOwner();
    	RelationDelegate relation = owner.getRelation(p);
    	if (relation == null) return false;
    	return relation.isWar();
    }

	public boolean selectedCraftIsOwnedByPlayer() throws ApplicationError {
		ImageObjectDelegate selected = state.getSelected();
    	UserPlayerDelegate owner = getOwner();
		return (selected instanceof BattleObjectDelegate && Comp.same(((BattleObjectDelegate)selected).getOwner(), owner.getName()));
	}

	private WreckageDetails getWreckageAt(int mouseX, int mouseY) throws ApplicationError {
		AnimatedBattleAreaDetails [] abad = battle.getAnnimatedBattleAreas();
		if (abad != null) {
			for (int i = 0; i < abad.length; i++) {
				if (abad[i] instanceof WreckageDetails && abad[i].contains(mouseX,  mouseY)) {
					return (WreckageDetails) abad[i];
				}
			}
		}
		return null;
	}


	private BattleAutomatonDelegate getMovingAutomatonAt(int mouseX, int mouseY) throws ApplicationError {
		BattleAutomatonDelegate[] aa = battle.getAutomatons();
		if (aa != null) {
			for (int i = 0; i < aa.length; i++) {
				if (aa[i] instanceof MissilesDelegate && aa[i].getArea().contains(mouseX,  mouseY)) {
					return (BattleAutomatonDelegate) aa[i];
				}
				if (aa[i] instanceof BattleDroidMineDelegate && aa[i].getArea().contains(mouseX,  mouseY)) {
					return (BattleAutomatonDelegate) aa[i];
				}
			}
		}
		return null;
	}


	public void complete() throws ApplicationError, ValidationError {
        CompletedCallback callback = new CompletedCallback() {
			public void complete(Object result) throws ApplicationError, ValidationError {
				finishComplete();
			}
		};
		ProxyStore.getInstance().battleAutomateCmd(battle, callback, null);
	}

    public void finishComplete() throws ApplicationError, ValidationError {
		stopAllAnimations();
		MainWindow mainWin = MainWindow.getInstance();
		mainWin.toGalaxyWindow();
		mainWin.getGalaxyMapConfiguration().getGalaxyMapPanel().rebuildContent();
		mainWin.getGalaxyMapConfiguration().getView().refresh();
    }
    
    public boolean isShowingAnimations() {
        BattleAnimationList al = getState().getAnimations();
        if (al != null && al.getQuantity() > 0) {
        	return true;
        }
    	return false;
    }
    
	public BattleDelegate getBattle() {
		return battle;
	}

	public void setBattle(BattleDelegate battle) {
		this.battle = battle;
	}

	public BattleWindowState getState() {
		return state;
	}

	public void setState(BattleWindowState state) {
		this.state = state;
	}

	public BattleWindowView getView() {
		return view;
	}

	public boolean isMissileDefenceOn() {
		return view.getControlPanel().getMissileDefenceButton().isOn();
	}

	public void setView(BattleWindowView view) {
		this.view = view;
		super.setView(view);
	}
	
	private boolean isATargetForWeaponsReadyToFireAndInRange(ImageObjectDelegate selection, ImageObjectDelegate mouseOverObject, int mouseX, int mouseY) throws ApplicationError {
		if (selection != null && selection instanceof BattleObjectDelegate) {
			BattleObjectDelegate selectedBattleObject = (BattleObjectDelegate) selection;
			int distanceToTarget = selection.getLocation().getDistanceTo( mouseOverObject != null ? mouseOverObject.getLocation() : new Location(mouseX, mouseY) );
	        Weapon selectedWeapon = view.getSelectWeaponPanel().getWeapon();
	        if (selectedWeapon != null) {
	        	WeaponInstanceVO wi = selectedBattleObject.getWeapon(selectedWeapon);
    			return isATargetForWeaponReadyToFireAndInRange(selectedBattleObject, wi, mouseOverObject, mouseX, mouseY, distanceToTarget);
	        } else {
	        	UserPlayerDelegate owner = getOwner();
	        	NameList weaponsToExcludeFromFireAll = owner.getWeaponsToExcludeFromFireAll();
	        	WeaponInstanceVO[] weapons = selectedBattleObject.getWeapons();
	        	if (weapons != null) {
	        		for (int i = 0; i < weapons.length; i++) {
	        			if (weaponsToExcludeFromFireAll != null && !weaponsToExcludeFromFireAll.contains(weapons[i].getWeapon().getGroup())) {
	        				if (isATargetForWeaponReadyToFireAndInRange(selectedBattleObject, weapons[i], mouseOverObject, mouseX, mouseY, distanceToTarget)) {
	        					return true;
	        				}
	        			}
	        		}
	        	}
	        }
		}
		return false;
	}
	
	private boolean isATargetForWeaponReadyToFireAndInRange(BattleObjectDelegate selectedBattleObject, WeaponInstanceVO wi, ImageObjectDelegate mouseOverObject, int mouseX, int mouseY, int distanceToTarget) throws ApplicationError {
		if (wi.hasWeaponsToFire()) {
			int range = wi.getRange();
			if (range < distanceToTarget) return false;
			boolean canAttackLocation = wi.canAttackLocation();
			boolean canAttackWreckage = wi.canAttackWreckage();
			boolean canAttackAutomaton = wi.canAttackAutomaton();
			if (canAttackLocation) {
				if (mouseOverObject == null) return true;
			}
			if (canAttackWreckage) {
				WreckageDetails wreckage = getWreckageAt(mouseX, mouseY);
				if (wreckage != null) return true;
			}
			if (canAttackAutomaton) {
				BattleAutomatonDelegate missiles = getMovingAutomatonAt(mouseX, mouseY);
				if (missiles != null) return true;
			}
			if (!canAttackLocation && !canAttackWreckage && !canAttackAutomaton) {
				if (mouseOverObject instanceof BattleObjectDelegate && !Comp.same(((BattleObjectDelegate)mouseOverObject).getOwner(), getOwner().getName())) return true;
			}
		}
		return false;
	}
	
	public void updateCursor(ImageObjectDelegate selection, Location mouseLocation, ImageObjectDelegate mouseOverObject) throws ApplicationError {
		if (isTargetAngleWithinWeaponFiringAngle(selection, mouseLocation)) {
			if (isATargetForWeaponsReadyToFireAndInRange(selection, mouseOverObject, mouseLocation.getX(), mouseLocation.getY())) {
				state.setCursorHit(true);
			} else {
				state.setCursorHit(false);
			}
		} else {
			state.setCursorHit(false);
		}
	}
	
	protected void _mouseMoved(int x, int y) throws ApplicationError, ValidationError {
		if (!ProxyStore.hasInstance()) {
			return;
		}
		lastMouseX = x;
		lastMouseY = y;
		BattleAnimationList animations = getState().getAnimations();
		if (animations != null && animations.getQuantity() > 0) {
			state.clearMouseOverInformation();
		} else {
			calculateMouseOverInformation(x, y);
		}
	}
	
	public void calculateMouseOverInformation() throws ApplicationError, ValidationError {
		calculateMouseOverInformation(lastMouseX, lastMouseY);
	}
	
	private void calculateMouseOverInformation(int mouseX, int mouseY) throws ApplicationError, ValidationError {
		double s = getScale(); 
		int x = (int) (mouseX / s); 
		int y = (int) (mouseY / s); 
		ImageObjectDelegate selection = state.getSelected();
		if (selection != null && selection instanceof BattleCraftDelegate && !((BattleCraftDelegate)selection).isOrbital()) {
			ImageObjectDelegate mouseOverObject = battle.getDelegateCache().getObjectAt(x, y, true);
			state.setMouseOverObject(mouseOverObject);
			Location mouseLocation = new Location(x, y);
			calculateMouseOverMoveToDest(mouseOverObject, mouseLocation);
			BattleCraftDelegate selectedCraft = (BattleCraftDelegate) selection;
			LocationAndDirection moveToDest = state.getMouseOverMoveToDestNotNull(mouseLocation, selectedCraft);
			BattleCraftDelegate mouseOverCraft = (mouseOverObject instanceof BattleCraftDelegate) ? (BattleCraftDelegate) mouseOverObject : null;
			LocationAndDirection actualLocation = ProxyStore.getInstance().battleFetchCraftMoveActualLocation(battle, selectedCraft, moveToDest, mouseOverCraft);
			state.setMouseOverActualDest(actualLocation);
			updateCursor(selection, mouseLocation, mouseOverObject);

			
			
		} else if (isBattlePlanet(selection) || isOrbital(selection)) {
			ImageObjectDelegate mouseOverObject = battle.getDelegateCache().getObjectAt(x, y, true);
			updateCursor(selection, new Location(x, y), mouseOverObject);
		} else {
			state.clearMouseOverInformation();
		}
	}
	
	public void mouseWheelMoved(MouseWheelEvent e) {
	}

	public void stopAllAnimations() {
        BattleAnimationList animations = state.getAnimations();
        if (animations != null) {
			animations.stopAll();
		}
	}

	public double getScale() throws ApplicationError {
		if (scale == null) {
			return calculateScale();
		} else {			
			return scale;
		}
	}

	public void componentResized() throws ApplicationError {
		 scale = null;
	}

	public void flushScale() throws ApplicationError {
		 scale = null;
	}
	
	private double calculateScale() throws ApplicationError {
		if (!MainWindow.hasInstance()) {
			return 1.0;
		}
		Dimension size = MainWindow.getInstance().getSize();
		if (size.getWidth() > 1100) {
			scale = (size.getWidth()-100.0) / ((double)GalaxyDefinitions.BATTLE_BOUNDARY.getWidth());
		} else {
			scale = 1.0;
		}
		getView().getMapWindow().setSize(scale);
		return scale;
	}

	public boolean isEndBattleRecieved() {
		return endBattleRecieved;
	}

	public void setEndBattleRecieved(boolean endBattleRecieved) {
		this.endBattleRecieved = endBattleRecieved;
	}

	protected String getKeyBindingsMessage() {
		return L.TT_KEYS_BATTLE;
	}


}
