/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.battle;

import mot.client.animation.BattleAnimationList;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.util.ApplicationError;

public class BattleWindowState {

    /**
     * This is the battle craft or battle planet currently selected on the window (if any).
     */
    private ImageObjectDelegate selected;
    
    /**
     * This is the image that the mouse is currently hovering over.
     */
    private ImageObjectDelegate mouseOverObject;
    private LocationAndDirection mouseOverMoveToDest;
    private LocationAndDirection mouseOverActualDest;
    
    /**
     * This is the type of icon to display on the galaxy map
     */
    private boolean isCursorHit;
    
    /**
     * This is true if the User has manually focused or selected a craft. 
     * When true the AI does not focus or select a craft until the user cancels the manual selection.
     * 
     * After a battle object has moved the UI selects the next BO to move and auto focuses 
     * on it. We do not want to repeatedly focus on the next bo to move because;
     * 1 - User may select a single weapon
     * 2 - User may select another BO to move
     */
    private boolean isUserSelected;
    
    /**
     * This list of animation that require displaying on the screen.
     */
    private BattleAnimationList animations = new BattleAnimationList();

	public BattleWindowState(ImageObjectDelegate selected) {
		this.selected = selected;
		this.isUserSelected = false;
	}
	
	/**
	 * Initialised the state of the battle window to empty. This MUST be called
	 * before the start of every battle so that animations from a previous battle
	 * are removed.
	 */
	public void init() {
		animations = new BattleAnimationList();
		selected = null;
		isUserSelected = false;
	}

	public BattleAnimationList getAnimations() {
		return animations;
	}

	public void setAnimations(BattleAnimationList animations) {
		this.animations = animations;
	}

	public ImageObjectDelegate getSelected() {
		return selected;
	}

	public void setSelected(ImageObjectDelegate selected, boolean isUserSelected) {
		this.selected = selected;
		this.isUserSelected = isUserSelected;
	}

	public boolean isUserSelected() {
		return isUserSelected;
	}

	public boolean setUserSelected(boolean v) {
		return isUserSelected = v;
	}

	public ImageObjectDelegate getMouseOverObject() {
		return mouseOverObject;
	}

	public void setMouseOverObject(ImageObjectDelegate mouseOverObject) {
		this.mouseOverObject = mouseOverObject;
	}

	public LocationAndDirection getMouseOverMoveToDest() {
		return mouseOverMoveToDest;
	}

	public LocationAndDirection getMouseOverMoveToDestNotNull(Location mouseLocation, BattleObjectDelegate selectedCraft) {
		if (mouseOverMoveToDest != null) {
			return mouseOverMoveToDest; 
		} else {			
			return new LocationAndDirection(mouseLocation, selectedCraft.getLocation().getDirectionFor(mouseLocation));
		}
	}

	public void setMouseOverMoveToDest(LocationAndDirection v) {
		this.mouseOverMoveToDest = v;
	}

	public LocationAndDirection getMouseOverActualDest() {
		return mouseOverActualDest;
	}

	public void setMouseOverActualDest(LocationAndDirection mouseOverActualDest) {
		this.mouseOverActualDest = mouseOverActualDest;
	}

	public void clearMouseOverInformation() {
		mouseOverMoveToDest = null;
		mouseOverActualDest = null;
		mouseOverObject = null;
	}

	public boolean isCursorHit() {
		return isCursorHit;
	}

	public void setCursorHit(boolean isCursorHit) {
		this.isCursorHit = isCursorHit;
	}

	public PlayerDelegate getOwnerOfActiveAnimation() throws ApplicationError {
		if (animations != null) {
			return animations.getOwnerOfActiveAnimation(true);
		} else {
			return null;
		}
	}


}
