/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.battle;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.KeyboardFocusManager;
import java.awt.Rectangle;

import javax.swing.JScrollPane;
import javax.swing.JToggleButton;

import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleObjectDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.MissilesDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.animation.AnimationSettings;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.panels.SelectWeaponPanel;
import mot.client.swing.window.battle.actions.ActionMode;
import mot.client.swing.window.battle.actions.MotBattleMapActions;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowMessageList;
import mot.client.swing.window.main.MainWindowView;
import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.vo.WeaponInstanceVO;

public class BattleWindowView extends MainWindowView implements MotBattleMapActions {

	private BattleWindowListener listener;
    private BattleWindowControlPanel controlPanel = null;
    private BattleMapPanel battleMapPanel = null;
    private MainWindowMessageList messageList;
    private ComponentTable bottom;
    
    private DamagePanel physicalDamage;
    private DamagePanel engineDamage;
    private DamagePanel psiDamage;
    private DamagePanel crewDamage;
    private DamagePanel shieldPower;
    private DamagePanel movementRemaining;

    public BattleWindowView(BattleWindowListener listener) throws ApplicationError {
    	this.listener = listener;
    	listener.setView(this);

        battleMapPanel = new BattleMapPanel(listener);
        battleMapPanel.setSize(listener.getScale());


        messageList = new MainWindowMessageList();
    
        physicalDamage = new DamagePanel(AnimationSettings.DAMAGE_BAR_COLOR_PHYSICAL, 0.0, true, "Physical");
        engineDamage = new DamagePanel(AnimationSettings.DAMAGE_BAR_COLOR_MOVEMENT, 0.0, true, "Engine");
        psiDamage = new DamagePanel(AnimationSettings.DAMAGE_BAR_COLOR_PSI, 0.0, true, "Psi");
        crewDamage = new DamagePanel(AnimationSettings.DAMAGE_BAR_COLOR_CREW, 0.0, true, "Crew");
        shieldPower = new DamagePanel(AnimationSettings.DAMAGE_BAR_COLOR_SHIELD, 0.0, true, "Shield");
        movementRemaining = new DamagePanel(AnimationSettings.DAMAGE_BAR_COLOR_SHIELD, 0.0, true, "Movement");
        
        ComponentTable infoPanel = new ComponentTable("HV, HV, HV, HV, HV, HV, HV, HV, HV, HV, HV, HV, HV", 2, 4, 1,3,0);
        //infoPanel.setGridWidths("50,50,50,50,50,50,50,50,50,50,50,50,50");
        infoPanel.add(messageList, "HVSPF");
        infoPanel.add(movementRemaining);
        infoPanel.add(shieldPower);
        infoPanel.add(physicalDamage);
        infoPanel.add(engineDamage);
        infoPanel.add(psiDamage);
        infoPanel.add(crewDamage);
    	
        controlPanel = new BattleWindowControlPanel(listener);
        controlPanel.setPreferredSize(new Dimension(500,60));
        controlPanel.setMinimumSize(new Dimension(500,60));
        bottom = new ComponentTable("HVS,HVS", 1, 2, 0,0,0);
        bottom.setGridWidths("600,400");
        bottom.add(infoPanel);
        bottom.add(controlPanel);
        bottom.setPreferredSize(new Dimension(2000,60));
    }
    
    public void deactivate() throws ApplicationError {
    	MainWindow win = MainWindow.getInstance();
    	win.getContentPane().removeAll();
    	
    	KeyboardFocusManager.getCurrentKeyboardFocusManager().removeKeyEventDispatcher(listener);
    }
    
    public void activate() throws ApplicationError {
    	MainWindow win = MainWindow.getInstance();
        win.getContentPane().add(bottom, BorderLayout.SOUTH);
        win.getContentPane().add(battleMapPanel.getScrollPane(), BorderLayout.CENTER);

    	KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(listener);
    }

    public void setDescription(ImageObjectDelegate io) throws ApplicationError {
        if (io != null) {
        	if (io instanceof BattleCraftDelegate) {
        		BattleCraftDelegate battleCraft = (BattleCraftDelegate) io;
        		CraftDelegate craft = battleCraft.getCraft();
        		UserPlayerDelegate owner = MainWindow.getOwner();
				if (battleCraft.isControlledBy(owner)) {
        			messageList.addTemporaryMessage(craft.getDescription(1, true, false), false, null, null, 2);
        		} else {
        			messageList.addTemporaryMessage(craft.getName(), false, null, null, 2);
        		}
        	} else if (io instanceof MissilesDelegate) {
        		MissilesDelegate md = (MissilesDelegate) io;
        		if (md.getWeapons() != null && md.getWeapons().length > 0) {
        			WeaponInstanceVO wi = md.getWeapons()[0];
					messageList.addTemporaryMessage(md.getName()+"x"+wi.getQty(), false, null, null, 2);
        		} else {
                	messageList.addTemporaryMessage(io.getName(), false, null, null, 2);
        		}
        	} else {
            	messageList.addTemporaryMessage(io.getName(), false, null, null, 2);
        	}
        }
    }

    public boolean isVisible(ImageObjectDelegate io) throws ApplicationError {
        if (io == null) {
            return false;
        }
        Location l = io.getLocation();
        Size imageSize = io.getImageSize();
        int width = imageSize.getX();
        int height = imageSize.getY();
        return isVisible(l.getX()-width/2, l.getY()-height/2, width, height);
    }

    public boolean isVisible(Area a) throws ApplicationError {
        if (a == null) {
            return false;
        }
    	return isVisible(a.getLeft(), a.getTop(), a.getWidth(), a.getHeight());
    }
    
    public boolean isVisible(int left, int top, int width, int height) throws ApplicationError {
        double scale = listener.getScale();
        Rectangle r = battleMapPanel.getScrollPane().getViewport().getViewRect();
        return r.contains(left*scale, top*scale) && r.contains((left+width)*scale, (top+height)*scale);
    }

    public void focus(ImageObjectDelegate io) throws ApplicationError {
        if (io != null) {
            if ( !isVisible(io) ) {
				focus(io.getArea());
            }
        }
        controlPanel.updateWeaponSelect(io);
        if (io instanceof BattleObjectDelegate) {
        	BattleObjectDelegate bc = (BattleObjectDelegate) io;
            physicalDamage.setDamage(bc.getState().getDamagePercent());
            engineDamage.setDamage(bc.getState().getMovementDamagePercent());
            psiDamage.setDamage(bc.getState().getPsiDamagePercent());
            crewDamage.setDamage(1.0-bc.getState().getCrewPercent());
            shieldPower.setDamage(1.0-bc.getState().getShieldPercent());
        } else {
            physicalDamage.setDamage(0);
            engineDamage.setDamage(0);
            psiDamage.setDamage(0);
            crewDamage.setDamage(0);
            shieldPower.setDamage(0);
        }
    }
    
    public void focus(Area a) throws ApplicationError {
    	if (a == null) return;
		a.scale(listener.getScale());
		Dimension bms = battleMapPanel.getSize();
		focus(new Area(0,0,(int)bms.getWidth(), (int)bms.getHeight()), a.getCenterLocation());
    }
    
    public void handleTick() throws ApplicationError {
        messageList.tick();
        ImageObjectDelegate selected = listener.getState().getSelected();
        boolean selectedCraftHasFightersToLaunch = false;
        if (selected != null && selected instanceof BattleObjectDelegate) {
        	BattleObjectDelegate selectedCraft = (BattleObjectDelegate) selected;
        	selectedCraftHasFightersToLaunch = selectedCraft.hasFightersToLaunch();
        }
		controlPanel.handleTick(selectedCraftHasFightersToLaunch);
    }

    public void reportApplicationError(Exception e) {
        Logger.logError("Failed due to error ", e);
		messageList.addMessage(e.getMessage());
	}

	public void reportValidationError(ValidationError e) {
		messageList.addErrorMessage(e.getMessage(), true);
	}

	public void showMessage(String msg, boolean flash, Color flashBkColor, String sound) {		
		if (flash) {
			messageList.addTemporaryMessage(msg, true, flashBkColor, sound, 0);
		}
		messageList.addMessage(msg);
	}

	public void showTemporaryMessage(String msg, boolean flash, Color bkColour, String sound, int additionalSecsToExpire) {		
		messageList.addTemporaryMessage(msg, flash, bkColour, sound, additionalSecsToExpire);
	}

	public boolean isDisplayingTemporaryMessage() {
		return messageList.isDisplayingTemporaryMessage();
	}
    
	public BattleWindowControlPanel getControlPanel() {
		return controlPanel;
	}

	public BattleMapPanel getMapWindow() {
		return battleMapPanel;
	}

	public void setMapWindow(BattleMapPanel mapWindow) {
		this.battleMapPanel = mapWindow;
	}

	public JScrollPane getMapPanelScrollPane() {
		return battleMapPanel.getScrollPane();
	}
	public BattleMapPanel getMapPanel() {
		return battleMapPanel;
	}

	public MainWindowMessageList getInfoField() {
		return messageList;
	}

	public JToggleButton getAutoButton() {
		return controlPanel.getAutoButton();
	}

	public ActionMode getExpressButton() {
		return controlPanel.getModeButton();
	}

	public SelectWeaponPanel getSelectWeaponPanel() {
		return controlPanel.getSelectWeaponPanel();
	}
	
    public void updateWeaponSelect(ImageObjectDelegate io) throws ApplicationError {
    	controlPanel.updateWeaponSelect(io);
    }

	public DamagePanel getCrewDamage() {
		return crewDamage;
	}

	public void setCrewDamage(DamagePanel crewDamage) {
		this.crewDamage = crewDamage;
	}

	public DamagePanel getEngineDamage() {
		return engineDamage;
	}

	public void setEngineDamage(DamagePanel engineDamage) {
		this.engineDamage = engineDamage;
	}

	public DamagePanel getPhysicalDamage() {
		return physicalDamage;
	}

	public void setPhysicalDamage(DamagePanel physicalDamage) {
		this.physicalDamage = physicalDamage;
	}

	public DamagePanel getPsiDamage() {
		return psiDamage;
	}

	public void setPsiDamage(DamagePanel psiDamage) {
		this.psiDamage = psiDamage;
	}

	public void refresh() {	}
	
}
