/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.battle.actions;

import java.awt.Color;

import mot.client.deligates.ImageObjectDelegate;
import mot.client.swing.window.battle.BattleActionButton;
import mot.client.swing.window.battle.BattleWindowListener;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;

public class ActionMode extends BattleActionButton {

	private static Color colorExpress;
	private static Color colorMove;
	private static Color colorAttack;
	
	private ActionModeStates mode;
	
	public ActionMode(BattleWindowListener listener) {
		super(listener, ' ');
		mode = ActionModeStates.express;
		if (colorExpress == null) {
			colorExpress = PropertyManager.getMandatoryAsColorNoError("window.battle.mode.express");
			colorMove = PropertyManager.getMandatoryAsColorNoError("window.battle.mode.move");
			colorAttack = PropertyManager.getMandatoryAsColorNoError("window.battle.mode.attack");
		}
		update();
	}

	private void update() {
		if (mode == ActionModeStates.express) {
			setText(MODE_BUTTON_EXPRESS);
			if (colorExpress != null) setBackground(colorExpress);
		} else if (mode == ActionModeStates.move) {
			setText(MODE_BUTTON_MOVE);
			if (colorMove != null) setBackground(colorMove);
		} else if (mode == ActionModeStates.attack) {
			setText(MODE_BUTTON_ATTACK);
			if (colorAttack != null) setBackground(colorAttack);
		} else {
			setText("ERR");
		} 
	}

	protected void __doAction() throws ApplicationError, ValidationError {
		ImageObjectDelegate selection = listener.getState().getSelected();
		if (mode == ActionModeStates.express) {
			if (listener.isCraftWithMoveRemaining(selection)) {
				mode = ActionModeStates.move;
			} else {				
				mode = ActionModeStates.attack;
			}
		} else if (mode == ActionModeStates.move) {
			mode = ActionModeStates.attack;
		} else if (mode == ActionModeStates.attack) {
			mode = ActionModeStates.express;
		} 
		update();
	}
	
	public boolean keyPressed(char key, int modifiers) {
		Character shortcut = getShortcut();
		if (shortcut != null && shortcut.charValue() == key) {
			// HACK - SPACE bar is the default press key shortcut so we are getting 2 actions when the shortcut is pressed 
			if (!hasFocus()) {
				actionPerformed(null);
			}
	        return true;
		} else {
			return false;
		}
	}


	protected String _getToolTipText() {
		return "Select command mode - One of Move, Attack or Express (move and attack).<br/>This button is intended for expert players (beginners should use the default express mode).<br/>Express finishes a craft's move using a single click (click enemy craft to attack or a space to move towards)<br/>Express keys: SPACE = Scroll to next mode, e = Express mode, m = Move mode, a = Attack mode";
	}
	public void setTip(Character shortcut) {
		String t = _getToolTipText();
		if (t != null) {			
			setToolTipText("<html>"+t+"</html>");
		}
	}

	public boolean isExpress() {
		return mode == ActionModeStates.express;
	}

	public boolean isMove() {
		return mode == ActionModeStates.move;
	}

	public boolean isAttack() {
		return mode == ActionModeStates.attack;
	}

	public void setModeExpress() {
		mode = ActionModeStates.express;
		update();
	}

	public void setModeMove() {
		mode = ActionModeStates.move;
		update();
	}

	public void setModeAttack() {
		mode = ActionModeStates.attack;
		update();
	}

}

enum ActionModeStates { move, attack, express }; 
