/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.common;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

import javax.swing.JLabel;

import mot.client.swing.common.ComponentTable;

public class MessagePanel extends ComponentTable {

	private static final int FACTOR = 10;

	private JLabel label;
	private int maxLineLength;
	
	public MessagePanel(String message, int maxLineLength, Font font, Color backgroundColour) {
		super("HV", 1,1, 0,0,0);
		String html = toHTML(message, maxLineLength, true);
		label = new JLabel(html, null, JLabel.CENTER);
		this.maxLineLength = maxLineLength;
        if (font != null) {
        	label.setFont(font);
        }
        if (backgroundColour != null) {
        	label.setBackground(backgroundColour);
        	label.setOpaque(true);
        }
        add(label);
	}
	
	public MessagePanel(String message, int maxLineLength, Font font, Color backgroundColour, String alignment) {
		super(alignment, 1,1, 0,0,0);
		label = new JLabel(toHTML(message, maxLineLength, false));
		this.maxLineLength = maxLineLength;
        if (font != null) {
        	label.setFont(font);
        }
        if (backgroundColour != null) {
        	label.setBackground(backgroundColour);
        	label.setOpaque(true);
        }
        add(label);
	}
	
	public void setHTMLText(String message) {
		label.setText(toHTML(message, maxLineLength, false));
	}
	
	/**
	 * Builds a String containing HTML mark up where each line is less that maxLineLength characters
	 * long and contains a <br/> element to start a new line.
	 * @param plainText The text to convert to HTML.
	 * @param maxLineLength The maximum length a line can be in characters.
	 * @return HTML version of the string passed in.
	 */
	public String toHTML(String plainText, int maxLineLength, boolean centered) {
		SplitHtmlResult html = toHtml(null, plainText, maxLineLength, centered);
		return html.getHtmlString();
	}
	
//	public static String toHTML(String plainTextTitle, String plainText, int maxLineLength, boolean centered) {
//        if (plainTextTitle == null && plainText == null) return null;
//        int lineLength = 0;
//        StringBuffer sb = new StringBuffer("<html>");
//        if (centered) sb.append("<center>");
//        if (plainTextTitle != null) {
//        	sb.append("<b>");
//        	sb.append(plainTextTitle);
//        	sb.append("</b>");
//        }
//        if (plainText != null) {
//        	if (plainTextTitle != null) sb.append("<br/>");
//        	String[] splitText = plainText.split(" ");
//	        if (splitText == null) {
//	   			sb.append(plainText);
//	        } else {
//	            for (int i = 0; i < splitText.length; i++) {
//	    			if (lineLength+splitText[i].length() > maxLineLength) {
//	    				sb.append("<br/>");
//	    				sb.append(splitText[i]);
//	    				lineLength = splitText[i].length();
//	    			} else {
//	    				if (lineLength != 0) {
//	    					sb.append(" ");
//	    				}
//	    				sb.append(splitText[i]);
//	    				lineLength += splitText[i].length();
//	    			}
//	    		}
//	        }
//        }
//        
//        if (centered) sb.append("</center>");
//        sb.append("</html>");
//        return sb.toString();
//	}
//

	public static String toHtml(JLabel label, String text, boolean centered) {
		FontMetrics fontMetrics = label.getFontMetrics(label.getFont());
		int textWidth = fontMetrics.stringWidth(text);
		int textLength = text.length();
		int avgWidthCharWidth =  textWidth / textLength;
		int labelWidth = label.getWidth();
		int maxLineLength = labelWidth / avgWidthCharWidth - 2;
		return toHtml(text, maxLineLength, centered);
	}

	public static String toHtml(String text, int maxLineLength, boolean centered) {
		SplitHtmlResult r = toHtml(null, text, maxLineLength, centered);
		return r.getHtmlString();
	}
	
	public static SplitHtmlResult toHtml(String heading, String text, int panelWidth, boolean centered) {
        if (heading == null && text == null) return new SplitHtmlResult(null);
        SplitHtmlResult r = new SplitHtmlResult(new StringBuffer("<html>"));
        if (centered) r.html.append("<center>");
        if (heading != null) {
        	r.html.append("<b>");
        	if (heading.length() > panelWidth) {
        		split(heading, panelWidth, centered, r);
        	} else {
        		r.html.append(heading);
            	r.qtyLines++;
        	}
        	r.html.append("</b>");
        }
        if (text != null) {
        	if (heading != null) r.html.append("<br/>");
        	split(text, panelWidth, centered, r);
        }
        
        if (centered) r.html.append("</center>");
        r.html.append("</html>");
        return r;
	}

	public static SplitHtmlResult toHtml(String heading, String text, int maxLineLength, boolean centered, Graphics graphics) {
        if (heading == null && text == null) return new SplitHtmlResult(null);
        SplitHtmlResult r = new SplitHtmlResult(new StringBuffer("<html>"));
        if (centered) r.html.append("<center>");
        if (heading != null) {
        	r.html.append("<b>");
        	
        	if (heading.length() > maxLineLength) {
        		split(heading, maxLineLength, centered, r, graphics);
        	} else {
        		r.html.append(heading);
            	r.qtyLines++;
        	}
        	r.html.append("</b>");
        }
        if (text != null) {
        	if (heading != null) r.html.append("<br/>");
        	split(text, maxLineLength, centered, r, graphics);
        }
        
        if (centered) r.html.append("</center>");
        r.html.append("</html>");
        return r;
	}

	
	private static void split(String text, int panelWidth, boolean centered, SplitHtmlResult r) {
        int lineLength = 0;
    	String[] splitText = text.split(" ");
        if (splitText == null) {
			if (r.qtyLines > 0) r.html.append("<br/>");
        	r.html.append(text);
        	r.qtyLines++;
        } else {
            for (int i = 0; i < splitText.length; i++) {
    			if (lineLength+splitText[i].length() > panelWidth) {
    				if (r.qtyLines > 0) r.html.append("<br/>");
    				r.html.append(splitText[i]);
    				lineLength = splitText[i].length();
    				r.qtyLines++;
    			} else {
    				if (lineLength != 0) {
    					r.html.append(" ");
    				}
    				r.html.append(splitText[i]);
    				lineLength += splitText[i].length();
    			}
    		}
        }

	}
	
	private static void split(String text, int maxLineLength, boolean centered, SplitHtmlResult r, Graphics graphics) {
    	String[] splitText = text.split(" ");
        if (splitText == null) {
			if (r.qtyLines > 0) r.html.append("<br/>");
        	r.html.append(text);
        	r.qtyLines++;
        } else {
        	StringBuffer nextLine = new StringBuffer();
            for (int i = 0; i < splitText.length; i++) {
            	FontMetrics fontMetrics = graphics.getFontMetrics();
            	String tmp = nextLine.length() == 0 ? splitText[i] : nextLine.toString() + splitText[i];
            	int lineWidth = fontMetrics.stringWidth(tmp);
    			if (lineWidth > maxLineLength) {
    				r.html.append("<br/>");
    				r.html.append(nextLine);
    				r.qtyLines++;
    				nextLine = new StringBuffer();
    				nextLine.append(splitText[i]);
    			} else {
   					r.html.append(" ");
    				r.html.append(splitText[i]);
    			}
    		}
			r.html.append(nextLine);
			r.qtyLines++;
        }

	}
	
}

