/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.common;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JLabel;
import javax.swing.JPanel;

import mot.client.swing.buttons.MotButton;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.ImagePanel;
import mot.client.swing.window.main.MotConditionalAction;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

public class MotConfirmationDialogPopup extends MotInternalWindow {

	private static Dimension SIZE = new Dimension(600,105);
	
	private MotConditionalAction yesAction;
	private MotConditionalAction noAction;
	
	public MotConfirmationDialogPopup(MotInternalWindow parent, String msg, String title, MotConditionalAction yesAction, MotConditionalAction noAction) throws ApplicationError {
		this(parent, null, msg, title, yesAction, noAction, null);
	}

	public MotConfirmationDialogPopup(MotInternalWindow parent, String imageName, String msg, String title, MotConditionalAction yesAction, MotConditionalAction noAction) throws ApplicationError {
		this(parent, imageName, msg, title, yesAction, noAction, null);
	}
	
	public MotConfirmationDialogPopup(MotInternalWindow parent, String imageName, String msg, String title, MotConditionalAction yesAction, MotConditionalAction noAction, JPanel other) throws ApplicationError {
		super(null, parent, title, SIZE);
		
    	this.yesAction = yesAction;
		this.noAction = noAction;
		
    	msg = "<HTML>" + msg.replaceAll("\n", "<br>") + "</HTML>";
    	
    	JLabel messageLabel = PanelUtils.createLabel(msg);

    	Size imageSize;
    	if (imageName == null) {    		
    		ImageConfig imageConfig = ImageConfigFactory.getInstance().getImageConfig("QuestionImage");
    		imageName = (imageConfig == null) ? "question.png" : imageConfig.getImage();
    		imageSize = (imageConfig == null) ? new Size(50,50) : imageConfig.getImageSize();
    	} else {    	
    		imageSize = new Size(50,50);
    	}
    	
    	ImagePanel imagePanel = new ImagePanel(imageName);
		PanelUtils.setTheBloodySize(imagePanel, imageSize);
		
    	ComponentTable buttonTable = new ComponentTable("-", 2,1, 5,0,0);
    	buttonTable.add(new MotConfirmationDialogButtonYes(this, yesAction));
    	buttonTable.add(new MotConfirmationDialogButtonNo(this, noAction));
    	ComponentTable contents = new ComponentTable("-,HVSP,HV", 1,3, 0,10,1);
		contents.add(new PadPanel(imagePanel,10));
		if (other == null) {
			contents.add(messageLabel);
		} else {
	    	ComponentTable contentPanel = new ComponentTable("HV", 2,1, 0,0,4);
	    	contentPanel.add(messageLabel);
	    	contentPanel.add(other);
			contents.add(contentPanel);
		}
    	contents.add(buttonTable);
		
		getContentPane().add(new PadPanel(contents, 4), BorderLayout.CENTER);
	}
	
	public boolean processKeyEventsGlobally() {
		return false;
	}
	
}

class MotConfirmationDialogButtonYes extends MotButton {

	private MotConfirmationDialogPopup window;
	private MotConditionalAction action;
	
	public MotConfirmationDialogButtonYes(MotConfirmationDialogPopup window, MotConditionalAction action) {
		this.window = window;
		this.action = action;
		setText(L.YES);
	}

	protected void _doAction() throws ApplicationError, ValidationError {
		if (action != null) action.execute();
		window.hide();
	}

}


class MotConfirmationDialogButtonNo extends MotButton {

	private MotConfirmationDialogPopup window;
	private MotConditionalAction action;
	
	public MotConfirmationDialogButtonNo(MotConfirmationDialogPopup window, MotConditionalAction action) {
		this.window = window;
		this.action = action;
		setText(L.NO);
	}

	protected void _doAction() throws ApplicationError, ValidationError {
		if (action != null) action.execute();
		window.hide();
	}

}
