/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.common;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowListener;
import java.awt.event.WindowStateListener;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JWindow;

import mot.client.swing.common.MotGraphicsEnvironment;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.window.main.MotAWTMouseListener;
import mot.client.swing.window.main.TestMainWindow;
import mot.common.bean.Settings;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;

/**
 * This is the base class for the main window of the application. 
 * 
 * The main window may be a JFrame (in debug / development mode) or
 * a full screen window in user mode. Therefore this class acts as a
 * delegate for wither a JFrame or a JWindow.
 *
 */
public class MotFrame implements MotParent {

    private JFrame frame;
    private JWindow window;

    /**
     * True if and only if the game is running in full window mode.
     */
    protected boolean isFullScreen;

    public MotFrame() throws ApplicationError {
       	frame = new JFrame("Master of Tactics");
       	//frame.setUndecorated(true);
       	frame.setSize(MotGraphicsEnvironment.getWindowSizeX(), MotGraphicsEnvironment.getWindowSizeY());
        WindowLocator.setDefault(frame);

        
        Image iconConfig = ImageConfigFactory.getIconForConfig("GameIcon");
        if (iconConfig != null) {
        	frame.setIconImage(iconConfig);
        }
        
        isFullScreen = Settings.getInstance().isFullScreen();
        if (isFullScreen && !(this instanceof TestMainWindow)) {
	        MotGraphicsEnvironment.setFullScreen(this);
        } else if (!PropertyManager.isDevelopment()) {
	        MotGraphicsEnvironment.setMainScreen(this);
        }

        // This stops a user moving the window when in full screen mode!
        if (isFullScreen) {
            frame.addComponentListener(new ComponentListener(){
    			public void componentHidden(ComponentEvent e) {	}
    			public void componentMoved(ComponentEvent e) { 
    				frame.setLocation(0,0);
    			}
    			public void componentResized(ComponentEvent e) { }
    			public void componentShown(ComponentEvent e) { }
            });
        }
        
        if (!Settings.getInstance().isAppletMode()) {
        	Toolkit.getDefaultToolkit().addAWTEventListener(new MotAWTMouseListener(), AWTEvent.MOUSE_EVENT_MASK);
        }
    }


    public JFrame getWindow() {
    	//if (window != null) {
    	//	return window;
    	//} else {
    		return frame;
    	//}
    }
    
    private JMenuBar lastMenu;
    
    public void setMenuBar(JMenuBar menu) {
    	setJMenuBar(menu);
    }
    
    public void setJMenuBar(JMenuBar menu) {
    	if (window != null) {
    		if (menu == null) {
    			if (lastMenu != null) {
    				window.getContentPane().remove(lastMenu);
    			}
    		} else {
	    		window.getContentPane().add(menu, BorderLayout.NORTH);
    		}
    	} else {
    		frame.setJMenuBar(menu);
    	}
    }
    
    public void setVisible(boolean v) {
    	if (window != null) {
    		window.setVisible(v);
    	} else {
    		frame.setVisible(v);
    	}
    }
    
    @SuppressWarnings("deprecation")
	public void hide() {
    	if (window != null) {
    		window.hide();
    	} else {
    		frame.hide();
    	}
    }

    public void invalidate() {
    	if (window != null) {
    		window.invalidate();
    	} else {
    		//frame.invalidate();
    		frame.validate();
    		frame.repaint();
    		frame.getContentPane().validate();
    		//frame.getContentPane().validate();
    		frame.getContentPane().repaint();
    	}
    }
    
    public void addWindowListener(WindowListener wl) {
    	if (window != null) {
    		window.addWindowListener(wl);
    	} else {
    		frame.addWindowListener(wl);
    	}
	}
    
    public void addComponentListener(ComponentListener cl) {
    	if (window != null) {
    		window.addComponentListener(cl);
    	} else {
    		frame.addComponentListener(cl);
    	}
	}
    
    public void addWindowStateListener(WindowStateListener cl) {
    	if (window != null) {
    		window.addWindowStateListener(cl);
    	} else {
    		frame.addWindowStateListener(cl);
    	}
	}
    
	public Container getContainer() {
    	if (window != null) {
    		return window;
    	} else {
    		return frame;
    	}
	}
	
	public Container getContentPane() {
    	if (window != null) {
    		return window.getContentPane();
    	} else {
    		return frame.getContentPane();
    	}
	}
	
	public Component getGlassPane() {
    	if (window != null) {
    		return window.getGlassPane();
    	} else {
    		return frame.getGlassPane();
    	}
	}

    public Point getLocationOnScreen() {
		if (window != null) {
			return window.getLocationOnScreen();
		} else {
			return frame.getLocationOnScreen();
		}
    }
	
    public Dimension getSize() {
		if (window != null) {
			return window.getSize();
		} else {
			return frame.getSize();
		}
    }
    
    public void repaint() {
		if (window != null) {
			window.repaint();
		} else {
			frame.repaint();
		}
    }
	
    public void doLayout() {
		if (window != null) {
			window.doLayout();
		} else {
			frame.doLayout();
			
		}
    }
	
	public void maximise() {
		Window window = getWindow();
        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        int nx = window.getX();
        int ny = window.getY();
        if(nx < 0) {
            nx = 0;
        }
        if(ny < 0) {
            ny = 0;
        }
        if(nx < 0) {
            nx = 0;
        }
        if(nx > size.getWidth()-window.getWidth()) {
            nx = (int)size.getWidth()-window.getWidth();
        }
        if(ny > size.getHeight()-window.getHeight()) {
            ny = (int)size.getHeight()-window.getHeight();
        }
	}



}