/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.common;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.plaf.basic.BasicInternalFrameUI;

import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.ProxyStore;
import mot.client.swing.buttons.MotActionListener;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.EventObserver;
import mot.client.swing.common.TimerEventPolling;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.PlainTextBackgroundImagePanel;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowView;
import mot.client.telemetry.Telemetry;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class MotInternalWindow extends JInternalFrame implements MotParent {
	
	private MotInternalWindow child;
	protected MotInternalWindow parent;
    protected UserPlayerDelegate owner;
    
    private boolean painted = false;
    private boolean showing = false;
    private boolean fullScreen = false;
    
    private TitlePanel titlePanel = null;
    
    /**
     * Used when an internal windows wishes to fetch an event of a particular type from the event queue.
     * Cannot use main or mailbox timer because thread may already be processing an event or from those timers.
     */
    private TimerEventPolling pollingTimer = null; 
    
    /**
     * Used to redraw this window when the frame is re-sized
     * NOTE - Its static so only one is added to the main window (not one per window and never removed)
     */
    private static MotInternalWindowResizeListener resizeListener;
    
    protected MotInternalWindow(UserPlayerDelegate player, MotInternalWindow parent, String title, Dimension size) throws ApplicationError {
        super(title, (size != null), (size != null));
        this.owner = player;
        this.parent = parent;
        
        if (parent != null) {
        	parent.setChild(this);
        }

        MainWindow mainWin = MainWindow.getInstance();
		JDesktopPane desktop = mainWin.getActiveConfiguration().getView().activateDesktopPane();
        desktop.add(this);

        if (resizeListener == null) {
        	resizeListener = new MotInternalWindowResizeListener();
        	mainWin.addWindowListener(resizeListener);
        }

        if (parent != null) {
        	setSelected(parent, false);
        	parent.setEnabled(false);
        }

        setSelected(this, true);
        
        if (size != null) {
	        setSize(size);
	        WindowLocator.setCenter(this);
        } else {
        	fullScreen = true;
	        setSize(desktop.getSize());
        	setLocation(0, 0);
        	BasicInternalFrameUI ui = (BasicInternalFrameUI)getUI();
        	ui.setNorthPane(null);
        	setBorder(BorderFactory.createLineBorder(Color.GRAY));
        }

		if (TutorialSystem.isActive()) {			
			TutorialSystem.notifyWindowOpen(this);
		}
    }
    
    public void checkForResize(JDesktopPane desktop) {
    	if (fullScreen && desktop != null) {
    		Dimension desktopSize = desktop.getSize();
    		Dimension size = getSize();
    		if (size.getWidth() != desktopSize.getWidth() || size.getHeight() != desktopSize.getHeight()) {
    			setSize(desktopSize);
    		}
    	}
    }
    
    public MotInternalWindow getInternalWindow() {
    	return this;
    }
    
    private void setSelected(JInternalFrame frame, boolean v) {
    	try { 
    		frame.setSelected(v); 
    	} catch (Exception e) {
    		// Nothing!!
    	}
    }
    
    public void setOwner(UserPlayerDelegate player) {
        this.owner = player;
    }

    public UserPlayerDelegate getOwner() {
        return owner;
    }

	public void show() {
        MainWindow mainWin = MainWindow.getExestingInstance();
        if (mainWin != null) {
    		MainWindowView view = mainWin.getActiveConfiguration().getView();
    		if (!view.isDesktopPaneActive()) {
    			JDesktopPane desktop = view.activateDesktopPane();
    			desktop.add(this);
    		}
        }
		super.show();
        showing = true;
	}
    
	public void hide() {
		// Swing can call while when opening (see JInternalFrame line 329)
		// So if this window is not open - ignore the hide!
		if (!showing) {
			//Logger.logError("Hiding Window "+getClass(), new ApplicationError());
			return;
		}
		if (child != null) {
			child.hide();
		}
		stopPollingTimer();
		super.hide();
		try {
			if (showing && MainWindow.hasInstance()) {
				MainWindow mainWin = MainWindow.getExestingInstance();
			    if (parent != null) {
			    	setSelected(parent, true);
			    	parent.setEnabled(true);
			    } else {
					mainWin.getActiveConfiguration().getView().deactivateDesktopPane();
			    }
			    mainWin.repaint();
			}
		} catch (Exception ex) {
			Logger.logError("Failed to close window", ex);
		}
		broadcastCloseEvent();
        showing = false;
        try {
        	if (TutorialSystem.isActive()) {			
        		TutorialSystem.notifyWindowClose(this);
        	}
		} catch (ApplicationError e) {
			Logger.logError("", e);
		}
	}

	private void stopPollingTimer() {
		if (pollingTimer != null) {
			pollingTimer.stop();
			pollingTimer = null;
		}
	}

	
    private static long t = 0;
	public void paint(Graphics g) {
//    	long st = System.currentTimeMillis();
//    	System.out.println("MotInternalWindow.paint() Interval "+(st-t));
//    	t = st;
    	
		if (Telemetry.ON) Telemetry.qtyPaintDialog++;
		super.paint(g);
		painted = true;
		
//        st = System.currentTimeMillis();
//    	System.out.println("MotInternalWindow.paint()  Cost "+(st-t));
//    	t = st;
	}
	
	protected void pollForEventOfType(Class eventClass, EventObserver observer) throws ApplicationError {
		pollingTimer = new TimerEventPolling(eventClass, observer, 100);
	}
	
    public void waitUntilClosed() {
    	//
    	// By ignoring the MotConfirmationDialogBox, a Dialog Box can be closed and the desktop pane removed
    	// despite MotConfirmationDialogBox remaining in the showing state.
    	// If that happens in the mail box timer, then we will not be processing messages from the server.
    	//
		while (showing && (!MainWindow.hasInstance() || MainWindow.getExestingInstance().getActiveConfiguration().getView().isDesktopPaneActive())) {
    		MotEventProcessor.processNextMessage();
    	}
		if (showing && MainWindow.hasInstance()) {
			Logger.logWarning("MotInternalWindow.waitUntilClosed() DesktopPane is not active but dialog box is showing");
			Logger.logStackTrace();
			hide();
		}
    }
    
    public void waitUntilClosedModal() {
		while (showing && (!MainWindow.hasInstance() || MainWindow.getExestingInstance().getActiveConfiguration().getView().isDesktopPaneActive())) {
    		MotEventProcessor.processNextMessageModel(this);
    	}
		if (showing && MainWindow.hasInstance()) {
			Logger.logWarning("MotInternalWindow.waitUntilClosed() DesktopPane is not active but dialog box is showing");
			Logger.logStackTrace();
			hide();
		}
    }

    
    public void waitUntilPainted() {
    	while (!painted) {
    		MotEventProcessor.processNextMessage();
    	}
    }
    
    /**
     * @return A new panel containing the content panel passed in with a
     * title bar at the top (if this window is in full screen mode). Otherwise
     * it just returns the content panel passed in.
     * @throws ApplicationError 
     */
    protected JComponent addTitleBar(JComponent content) throws ApplicationError {
    	if (fullScreen) {
    		ComponentTable panel = new ComponentTable("HV", 2, 1, 0,0,0);
    		titlePanel = new TitlePanel(hasNavigationTitle(), this);
    		if (getTitle() != null) {
    			titlePanel.setText(getTitle());
    		}
    		panel.add(titlePanel);
    		panel.add(content, "HVPS");
    		return panel;
    	} else {
    		return content;
    	}
    }
    
    protected boolean hasNavigationTitle() { return false; }

	public void setTitle(String title) {
		if (titlePanel != null) {
			titlePanel.setText(title);
			titlePanel.repaint();
		} else {
			super.setTitle(title);
		}
	}
	
	public void setEnabledRecursize(boolean enabled) {
		setComponentsEnabled(this, enabled);
	}
	
	private static void setComponentsEnabled(Container c, boolean en) {
	    Component[] components = c.getComponents();
	    for (Component comp: components) {
	        if (comp instanceof Container) {
	            setComponentsEnabled((Container) comp, en);
	        }
	        comp.setEnabled(en);
	    }
	}
	
	private ArrayList<MotInternalWindowListener> listeners = new ArrayList<MotInternalWindowListener>();
	
	public void addListener(MotInternalWindowListener l) {
		listeners.add(l);
	}
	
	public void removeListener(MotInternalWindowListener l) {
		listeners.remove(l);
	}
	
	private void broadcastCloseEvent() {
		if (listeners != null && listeners.size() > 0) {
			for (MotInternalWindowListener l : listeners) {
				l.hide();
			}
		}
	}

	protected MotInternalWindow getChild() {
		return child;
	}

	protected void setChild(MotInternalWindow child) {
		this.child = child;
	}

	/**
	 * Some windows want to prevent the the End Turn button from being pressed and some do not.
	 * Example - Set Research Project and Set Planet Project when opened by a NoProjectEvent do not
	 * want the EndTurn button pressed until after they windows and been closed (and a project selected).
	 * Example - Discover Prize window and Solar System will allow End Turn Button to be pressed.   
	 */
	private boolean stopEndTurnButton = false;
	
	public boolean stopEndTurnButton() {
		return stopEndTurnButton;
	}
	
	public void setStopEndTurnButton(boolean v) {
		stopEndTurnButton = v;
	}

	/**
	 * Called when user clicks in the left had side of the title bar.
	 * @throws ApplicationError 
	 * @throws ValidationError 
	 */
	public void navigateLeft() throws ApplicationError, ValidationError {
	}

	/**
	 * Called when user clicks in the right had side of the title bar.
	 */
	public void navigateRight() throws ApplicationError, ValidationError {
	}

	/**
	 * User has pressed the next button / key.
	 */
	public void next() throws ApplicationError, ValidationError {
	}
    
	/**
	 * User has pressed the next button / key.
	 */
	public void previous() throws ApplicationError, ValidationError {
	}

	/**
	 * @return TRUE when key events are to be processed globally - so this window cannot receive key input (text input).
	 */
	public boolean processKeyEventsGlobally() {
		return true;
	}

	/**
	 * Windows such as NewGame and Load when opened at start of a game, should not be closed by pressing escape because
	 * they leave the user in a null state.
	 * @return TRUE if pressing escape should close this window
	 * @throws ApplicationError 
	 */
	public boolean escapeKeyClosesWindow() throws ApplicationError {
		return ProxyStore.hasInstance() && ProxyStore.getInstance().isConnected();
	}
    
	/**
	 * Override this method to animate windows.
	 * Originally MOT used an animation timer which all animation panels would register a timer call back.
	 * However - One mistake and times could be left registered and leave un-registration to cleanup after the window has closed (which does not happen).
	 * @throws ApplicationError
	 */
    public void handleTick() throws ApplicationError {
    }
    
}

class TitlePanel extends PlainTextBackgroundImagePanel {

	private static Font TITLE_FONT = null;
	private static ImageConfig TITLE_IMAGE = null;
	private MotInternalWindow motInternalWindow;
	private boolean hasNavigation;
	
	public TitlePanel(boolean hasNavigation, MotInternalWindow motInternalWindow) throws ApplicationError {
		super("", null, 0, false, 0.001, 0.98, 0.1, 0.8, SwingConstants.CENTER, SwingConstants.CENTER);
		this.hasNavigation = hasNavigation;
		this.motInternalWindow = motInternalWindow;
		TITLE_IMAGE = ImageConfigFactory.getInstance().getImageConfig(hasNavigation ? "DialogTitleArrows" : "DialogTitleDots");
		if (TITLE_FONT == null) {
	        JLabel label = new JLabel();
            Font font = label.getFont();
	        int style = font.getStyle();
            style = Font.BOLD;
			TITLE_FONT = new Font(font.getName(), style, font.getSize()+10);
		}
		Size imageSize = TITLE_IMAGE.getImageSize();
		setPreferredSize(new Dimension(imageSize.getX(), imageSize.getY()));
		setMinimumSize(new Dimension(10, imageSize.getY()));
		setFont(TITLE_FONT);
	}

	public void motMouseClicked(MouseEvent e) {
		if (e.getButton() != MouseEvent.BUTTON1) {
			return;
		}
		if (!hasNavigation) return;
		MotActionListener left = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				motInternalWindow.navigateLeft();
			}
		};
		MotActionListener right = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				motInternalWindow.navigateRight();
			}
		};
		if (e.getX() < getWidth()/2) {
			left.runAction();
		} else {
			right.runAction();
		}
	}
	
	protected String getImage() {
		return TITLE_IMAGE.getImage();
	}
	
}

class MotInternalWindowResizeListener implements WindowListener {

	// Window Listener Methods
	public void windowOpened(WindowEvent e) {
	}
	public void windowClosing(WindowEvent e) {
	}
	public void windowClosed(WindowEvent e) {
	}
	public void windowIconified(WindowEvent e) {
	}
	public void windowDeiconified(WindowEvent e) {
	}
	public void windowActivated(WindowEvent e) {
    	try {
    		// This is stateless so we do not get a memory leak
    		if (MainWindow.hasInstance()) {
    			MainWindow mainWin = MainWindow.getInstance();
    			if (mainWin.isGalaxyMapConfigurationActive()) {
    				MotInternalWindow topWindow = mainWin.getGalaxyMapConfiguration().getGalaxyView().getTopWindow();
    				JDesktopPane desktopPane = mainWin.getGalaxyMapConfiguration().getGalaxyView().getDesktopPane();
    				if (topWindow != null && desktopPane != null) {    					
    					topWindow.checkForResize(desktopPane);
    				}
    			}
    		}
		} catch (Throwable e1) {
			Logger.logError("", e1);
		}
	}
	public void windowDeactivated(WindowEvent e) {
	}

}