/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.craft;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.Timer;

import mot.client.deligates.CraftDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.icons.TargetIcon;
import mot.client.swing.labels.TechnologyLabel;
import mot.client.swing.labels.TechnologyLabelClickListener;
import mot.client.swing.lists.FakeTechnologyLabelList;
import mot.client.swing.panels.BackgroundImagePanel;
import mot.client.swing.renderers.CellRendererTechnologyLabel;
import mot.client.swing.renderers.ListItemControlPanelFactory;
import mot.client.swing.renderers.ListItemControlPanelFactoryTechnology;
import mot.client.swing.renderers.ListItemControlPanelListener;
import mot.common.bean.CraftDesignStatistics;
import mot.common.bean.CraftGraphics;
import mot.common.bean.RaceGraphicsList;
import mot.common.bean.Rectangle;
import mot.common.bean.Size;
import mot.common.designs.CraftDesign;
import mot.common.factory.GraphicsFactory;
import mot.common.lang.L;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.WeaponBundle;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;

/**
 * A panel containing a schematic picture of a craft design with details of
 * a particular craft design laid over the top. This is a pictorial view 
 * of a craft design.
 */
public class CraftDesignPicturePanel extends BackgroundImagePanel {

    private static final ImageConfig BACKGROUND_IMAGE = ImageConfigFactory.getImageConfigNoError("CraftDesignPictureBackground");
    private static final ImageConfig BACKGROUND_IMAGE_HULL = ImageConfigFactory.getImageConfigNoError("CraftDesignPictureBackgroundHull");
    private static final ImageConfig BACKGROUND_IMAGE_ENGINE = ImageConfigFactory.getImageConfigNoError("CraftDesignPictureBackgroundEngine");
    private static final ImageConfig BACKGROUND_IMAGE_COMPUTER = ImageConfigFactory.getImageConfigNoError("CraftDesignPictureBackgroundComputer");
    private static final ImageConfig BACKGROUND_IMAGE_SCANNER = ImageConfigFactory.getImageConfigNoError("CraftDesignPictureBackgroundScanner");
    private static final ImageConfig BACKGROUND_IMAGE_SHIELDS = ImageConfigFactory.getImageConfigNoError("CraftDesignPictureBackgroundShields");
    private static final ImageConfig BACKGROUND_IMAGE_SPECIALS = ImageConfigFactory.getImageConfigNoError("CraftDesignPictureBackgroundSpecials");
    private static final ImageConfig BACKGROUND_IMAGE_WEAPONS = ImageConfigFactory.getImageConfigNoError("CraftDesignPictureBackgroundWeapons");

    public static final ImageConfig BACKGROUND_IMAGE_SPACE_STATION = ImageConfigFactory.getImageConfigNoError("SpaceStationDesignPictureBackground");
    public static final ImageConfig BACKGROUND_IMAGE_NO_SPACE_STATION = ImageConfigFactory.getImageConfigNoError("NoSpaceStationDesignPictureBackground");
    public static final ImageConfig BACKGROUND_IMAGE_NO_BATTLE_STATION = ImageConfigFactory.getImageConfigNoError("NoBattleStationDesignPictureBackground");

    private ImageConfig selectedBackgroundImage = BACKGROUND_IMAGE;
    
    public void selectHullBackground() { selectedBackgroundImage = BACKGROUND_IMAGE_HULL; repaint();}
    public void selectEngineBackground() { selectedBackgroundImage = BACKGROUND_IMAGE_ENGINE; repaint();}
    public void selectComputerBackground() { selectedBackgroundImage = BACKGROUND_IMAGE_COMPUTER; repaint();}
    public void selectScannerBackground() { selectedBackgroundImage = BACKGROUND_IMAGE_SCANNER; repaint();}
    public void selectShieldsBackground() { selectedBackgroundImage = BACKGROUND_IMAGE_SHIELDS; repaint();}
    public void selectSpecialsBackground() { selectedBackgroundImage = BACKGROUND_IMAGE_SPECIALS; repaint();}
    public void selectWeaponsBackground() { selectedBackgroundImage = BACKGROUND_IMAGE_WEAPONS; repaint();}

    public boolean isHullBackgroundSelected()  throws ApplicationError, ValidationError { return selectedBackgroundImage == BACKGROUND_IMAGE_HULL; }
    public boolean isEngineBackgroundSelected() { return selectedBackgroundImage == BACKGROUND_IMAGE_ENGINE; }
    public boolean isComputerBackgroundSelected() { return selectedBackgroundImage == BACKGROUND_IMAGE_COMPUTER; }
    public boolean isScannerBackgroundSelected() { return selectedBackgroundImage == BACKGROUND_IMAGE_SCANNER; }
    public boolean isShieldsBackgroundSelected() { return selectedBackgroundImage == BACKGROUND_IMAGE_SHIELDS; }
    public boolean isSpecialsBackgroundSelected() { return selectedBackgroundImage == BACKGROUND_IMAGE_SPECIALS; }
    public boolean isWeaponsBackgroundSelected() { return selectedBackgroundImage == BACKGROUND_IMAGE_WEAPONS; }

    protected String getImage() {
   		return selectedBackgroundImage.getImage(); 
    }
    
    public void setBackgroundImage(ImageConfig newImage) {
    	selectedBackgroundImage = newImage;
    }
    
    private PlayerDelegate owner;
    private TechnologyLabelClickListener listener;

    private FakeTechnologyLabelList weaponsList;
    private FakeTechnologyLabelList shieldsList;
    private FakeTechnologyLabelList specialsList;

    private FakeTechnologyLabelList computerList;
    private FakeTechnologyLabelList scannerList;

    private TechnologyLabel engineLabel;
    private TechnologyLabel hullLabel;
    private boolean showHullThumb;

    private MotLabel salvageLabel;
    private FakeTechnologyLabelList salvageList;

    private BuildQualityPanel buildQualityPanel;
    
    private TechnologyTypeSelectionPanel techTypeSelectionPanel;
    
    private String image;
    private Size size;
    
    private static Color textColour = Color.BLACK;

	private Timer flashTimer;
	
	/**
	 * If this design is for an existing craft, then is the the craft.
	 * IT MAY BE NULL - then this is just a design!
	 */
	private CraftDelegate craft;

    public CraftDesignPicturePanel(PlayerDelegate owner, PlayerDelegate origin, CraftDesign cd) throws ApplicationError {
    	this(owner, origin, cd, null, null, null, null, null, false, false, false);
    	if (cd == null) {
    		selectedBackgroundImage = BACKGROUND_IMAGE_NO_SPACE_STATION;
    	} else {
    		selectedBackgroundImage = BACKGROUND_IMAGE_SPACE_STATION;
    	}
    }
    
    public CraftDesignPicturePanel(PlayerDelegate owner, PlayerDelegate origin, CraftDesign cd, CraftDesignStatistics cdStats, TechnologyLabelClickListener listener, TechnologyTypeSelectionListener techTypeListener, ListItemControlPanelListener craftDesignerWindow, CraftDelegate craft, boolean showBuildQuality, boolean showHullThumb, boolean showSectionToolTips) throws ApplicationError {
        this.owner = owner; 
        this.listener = listener;
		this.craft = craft;
		this.showHullThumb = showHullThumb;
        
		selectedBackgroundImage = BACKGROUND_IMAGE;
				
		ListItemControlPanelFactory controlPanelFactory = (craftDesignerWindow == null) ? null : new ListItemControlPanelFactoryTechnology(craftDesignerWindow, false);
        
        CellRendererTechnologyLabel renderer = new CellRendererTechnologyLabel(false, false, false, null, true, 0, false, textColour, null, false, listener, null, controlPanelFactory);
        weaponsList = new FakeTechnologyLabelList(renderer);
        shieldsList = new FakeTechnologyLabelList(renderer);
        specialsList = new FakeTechnologyLabelList(renderer);
        computerList = new FakeTechnologyLabelList(renderer);
        scannerList = new FakeTechnologyLabelList(renderer);

        engineLabel = new TechnologyLabel(listener, false, true);
        engineLabel.setForeground(textColour);
        hullLabel = new TechnologyLabel(listener, false, true);
        hullLabel.setForeground(textColour);
        
        CellRendererTechnologyLabel salvageRenderer = new CellRendererTechnologyLabel(false, true, false, new Size(15,15), true, 1, false, Color.CYAN, null, false, listener, null, controlPanelFactory);
        salvageLabel = new MotLabel(null, 3, false);
        salvageLabel.setForeground(Color.CYAN);
        salvageList = new FakeTechnologyLabelList(salvageRenderer);

        if (showBuildQuality) buildQualityPanel = new BuildQualityPanel(textColour);

        weaponsList.setOpaque(false);
        shieldsList.setOpaque(false);
        specialsList.setOpaque(false);
        salvageLabel.setOpaque(false);
        salvageList.setOpaque(false);
        
        if (showSectionToolTips) {
	        hullLabel.setToolTipText(L.TT_CRAFT_DESIGN_HULL);
	        engineLabel.setToolTipText(L.TT_CRAFT_DESIGN_ENGINES);
	        computerList.setToolTipText(L.TT_CRAFT_DESIGN_COMPUTERS);
	        scannerList.setToolTipText(L.TT_CRAFT_DESIGN_SCANNERS);
	        weaponsList.setToolTipText(L.TT_CRAFT_DESIGN_WEAPONS);
	        shieldsList.setToolTipText(L.TT_CRAFT_DESIGN_SHIELDS);
	        specialsList.setToolTipText(L.TT_CRAFT_DESIGN_SPECIALS);
	        salvageList.setToolTipText(L.TT_CRAFT_FROM_SALVAGE);
        }
        
        if (cd != null) {
            setDesign(cd, origin, cdStats);
        }
        
        add(setup(shieldsList.getScrollPane()));

        add(setup(weaponsList.getScrollPane()));

        add(setup(computerList));

        add(setup(scannerList));

        add(setup(specialsList.getScrollPane()));

        add(setup(salvageLabel));
        add(setup(salvageList.getScrollPane()));
        
        add(setup(engineLabel));
        add(setup(hullLabel));

        if (buildQualityPanel != null) {
        	add(setup(buildQualityPanel));
        }
        
        if (techTypeListener != null) {
        	techTypeSelectionPanel = new TechnologyTypeSelectionPanel(techTypeListener);
        	add(setup(techTypeSelectionPanel));
//        	flashTimer = new Timer(1000, new MotActionListener() {
//        		protected boolean playSound() { return false; }
//				protected void _doAction() throws ApplicationError, ValidationError {
//		    		flashOn = !flashOn;
//					repaint();
//				}
//			});
        }

        testResize();

    }
    
    public void selectAreaAt(Point p) {
    	if ( isIn(p, shieldsList.getScrollPane()) ) {
    		selectShieldsBackground();
    	}
    }
    
    private boolean isIn(Point p, Component c) {
    	java.awt.Rectangle bounds = c.getBounds();
    	boolean r = bounds.contains(p);
		return r;
	}
    
	public void handleGlobalMouseClick(MouseEvent e, TechnologyTypeSelectionListener techTypeListener)  throws ApplicationError, ValidationError  {
		Point clickLock = e.getLocationOnScreen();		
		if (Comp.contained(clickLock, PanelUtils.tooOnScreen(getShieldsRect(), this))) {
			techTypeListener.selectShields();
		}
		if (Comp.contained(clickLock, PanelUtils.tooOnScreen(getHullRect(), this))) {
			techTypeListener.selectHull();
		}
		if (Comp.contained(clickLock, PanelUtils.tooOnScreen(getComputerRect(), this))) {
			techTypeListener.selectComputer();
		}
		if (Comp.contained(clickLock, PanelUtils.tooOnScreen(getWeaponsRect(), this))) {
			techTypeListener.selectWeapons();
		}
		if (Comp.contained(clickLock, PanelUtils.tooOnScreen(getScannerRect(), this))) {
			techTypeListener.selectScanner();
		}
		if (Comp.contained(clickLock, PanelUtils.tooOnScreen(getSpecialsRect(), this))) {
			techTypeListener.selectSpecials();
		}
		if (Comp.contained(clickLock, PanelUtils.tooOnScreen(getEngineRect(), this))) {
			techTypeListener.selectEngine();
		}
	}

	private Rectangle getShieldsRect() { return getPosition(0.11, 0.1, 0.73, 0.18); }
	private Rectangle getHullRect() { return getPosition(0.16, 0.2, 0.01, 0.2); }
	private Rectangle getComputerRect() { return getPosition(0.32, 0.11, 0.15, 0.2); }
	private Rectangle getWeaponsRect() { return getPosition(0.41, 0.15, 0.35, 0.3); }
	private Rectangle getScannerRect() { return getPosition(0.32, 0.11, 0.58, 0.2); }
	private Rectangle getSpecialsRect() { return getPosition(0.62, 0.2, 0.33, 0.35); }
	private Rectangle getEngineRect() { return getPosition(0.925, 0.07, 0.5, 0.3); }
	private Rectangle getSalvageRect() { return getPosition(0.62, 0.3, 0.01, 0.15); }

	protected void drawSizeRelativeObjects() {
        setPosition(shieldsList.getScrollPane(), 0.11, 0.1, 0.76, 0.18);
        setPosition(hullLabel, 0.23, 0.05, 0.02, 0.15);
        setPosition(computerList, 0.32, 0.11, 0.2, 0.2);
        setPosition(weaponsList.getScrollPane(), 0.41, 0.15, 0.35, 0.3);
        setPosition(scannerList, 0.32, 0.11, 0.62, 0.2);
        setPosition(specialsList.getScrollPane(), 0.62, 0.2, 0.33, 0.35);
        setPosition(engineLabel, 0.925, 0.07, 0.57, 0.1);
        setPosition(salvageLabel, 0.575, 0.05, 0.005, 0.15);
        setPosition(salvageList.getScrollPane(), 0.62, 0.3, 0.01, 0.15);
        if (buildQualityPanel != null) {
        	setPosition(buildQualityPanel, 0.011, 0.05, 0.405, 0.2);
        }

        shieldsList.doLayout();
        weaponsList.doLayout();
        specialsList.doLayout();
        shieldsList.getScrollPane().doLayout();
        weaponsList.getScrollPane().doLayout();
        specialsList.getScrollPane().doLayout();
        computerList.doLayout();
        scannerList.doLayout();
        salvageList.doLayout();
        salvageList.getScrollPane().doLayout();
        if (buildQualityPanel != null) {
        	buildQualityPanel.doLayout();
        }

        if (techTypeSelectionPanel != null) {
        	setPosition(techTypeSelectionPanel, 0.5, 0.5, 0.0, 0.15);
        	techTypeSelectionPanel.doLayout();
        }
        
	}
    
    protected void postPaint(Graphics2D g) {
        if (image != null) {
            int x = getX(0.11);
			int y = getY(0.12);
			int w = (int)(size.getX()*1.5);
			int h = (int)(size.getY()*1.5);
			ImageUtils._drawImageCenteredAt(this, g, image, x, y, w, h);
            if (craft != null) {
            	try {
					double damagePercent = craft.getDamageAsPercent();
					if (damagePercent > 0.0) {
				        Color healthBarColour = PropertyManager.getMandatoryAsColor("damagebar.color");
				        g.setColor(healthBarColour);
						g.fillRect(x-h/2, y+h/2+2, (int)(w * damagePercent), 4);
					}
					
				} catch (ApplicationError e) {
					Logger.logError("Unable to paint damage bar", e);
				}
            }
        }
    }
    
    private JScrollPane setup(JScrollPane component) {
        component.setOpaque(false);
        if (component.getViewport() != null) {
            component.getViewport().setOpaque(false);
            component.setBorder(null);
        }
        return component;
    }
    
    private JComponent setup(JComponent component) {
        component.setOpaque(false);
        return component;
    }

    public void setSalvage(TechnologyBundle[] salvage) throws ApplicationError {
    	if (salvage == null || salvage.length == 0) {
    		salvageList.clearContent();
    		salvageLabel.setText(null);
    	} else {
    		salvageList.setContent(salvage, null);
    		salvageLabel.setText(L.SALVAGE_AVAILABLE_FOR_USE);
    	}

        invalidate();
        doLayout();
        repaint();
        
        
    }

    public Technology [] getUsableSalvage() {
    	return salvageList.getContent();
    }
    
    public void updateDesign(CraftDesign cd, PlayerDelegate origin, CraftDesignStatistics stats) throws ApplicationError {
        setDesign(cd, origin, stats);
    }
    
    public void setDesign(CraftDesign cd, PlayerDelegate origin, CraftDesignStatistics stats) throws ApplicationError {
        TargetIcon targetIcon = TargetIcon.getInstance();

        weaponsList.removeAll();
        if (cd.getWeaponBays() != null) {
            WeaponBundle [] weapons = cd.getWeaponBays().getDistinctWeapons();
            weaponsList.setContent(weapons, null);
        }
        weaponsList.validate();
        
       	shieldsList.setContent(cd.getShieldTechs(), null);
       	specialsList.setContent(cd.getSpecialTechs(), null);
       	computerList.setContent(cd.getComputerTechs(), null);
       	scannerList.setContent(cd.getScannerTechs(), null);
        
        if (cd.getEngine() == null) {
            engineLabel.setTechnology("-", null);
        } else {
            engineLabel.setTechnology(cd.getEngine(), cd.getEngineTech());
        }

        if (cd.getHull() == null) {
            hullLabel.setTechnology("-", null);
        } else {
        	String text = cd.getHull();
        	if (craft != null) {
        		double damagePercent = craft.getDamageAsPercent();
        		if (damagePercent > 0.0 && stats != null) {
	        		int maxDamage = stats.getMaxDamage();
					text = text + " [ " + L.DAMAGE + " " + Formatter.whole(maxDamage * damagePercent) + " of " + maxDamage + "]";
        		}
        	}
            hullLabel.setTechnology(text, cd.getHullTech());
        }

        if (showHullThumb && owner.getName() != null && cd.getHull() != null) {
            RaceGraphicsList rgl = GraphicsFactory.getRaceGraphicsList(); 
            CraftGraphics cg = rgl.getHullGraphics(owner.getName(), cd.getHull());
            if (cg != null) {
                image = CraftGraphics.CRAFT_IMAGE_FOLDER+cg.getPictureNormal()+cg.getFileExtension();
                size = cg.getSize();
            }
        } else {
            image = null;
        }
        
        if (buildQualityPanel != null) {
	        if (craft != null) {
	        	buildQualityPanel.setContent(craft);
	        } else if (owner.getName() != null && cd.getHull() != null) {
	        	if (owner instanceof UserPlayerDelegate) {
	            	double buildQuality = ((UserPlayerDelegate)owner).getBuildQuality(cd.getName());
	            	buildQualityPanel.setContent(buildQuality);
	        	} else {
	            	buildQualityPanel.setContent(0);
	        	}
	        }
        }

        invalidate();
        doLayout();
        repaint();
    }

    public void clearDesign() throws ApplicationError {
        weaponsList.clearContent();
       	shieldsList.clearContent();
       	specialsList.clearContent();
       	computerList.clearContent();
       	scannerList.clearContent();
        
        engineLabel.setTechnology(null);
        hullLabel.setTechnology(null);

      	salvageList.clearContent();
      	salvageLabel.setText(null);
      	
        invalidate();
        doLayout();
        repaint();
    }



    private JScrollPane scrollPanel;
    public JScrollPane getScrollPane() {
        if (scrollPanel == null) {
            scrollPanel = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        }
        return scrollPanel;
    }
    
	public TechnologyTypeSelectionPanel getTechTypeSelectionPanel() {
		return techTypeSelectionPanel;
	}
	
}