/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.craft;

import java.awt.Graphics2D;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.deligates.PlayerDelegate;
import mot.client.swing.buttons.MotActionListener;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.BackgroundImagePanel;
import mot.common.bean.CraftDesignStatistics;
import mot.common.designs.CraftDesign;
import mot.common.lang.L;
import mot.common.technology.Shields;
import mot.common.technology.Specials;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.ValidationError;

public class CraftDesignStatsPanel extends BackgroundImagePanel {

    private static final ImageConfig BACKGROUND_IMAGE = ImageConfigFactory.getImageConfigNoError("CraftDesignStatsBackground");

    protected String getImage() { 
    	return BACKGROUND_IMAGE.getImage(); 
    }
    
    private PlayerDelegate owner;

    private JLabel warpSpeedLabel = new JLabel();
    private JLabel battleSpeedLabel = new JLabel();
    private JLabel manouverabilityLabel = new JLabel();
    private JLabel shieldBlockLabel = new JLabel();
    private JLabel attackerBonusLabel = new JLabel();
    private JLabel defenderBonusLabel = new JLabel();
    private JLabel hitPointsLabel = new JLabel();
    private JLabel crewLabel = new JLabel();
    private JLabel cloakLabel = new JLabel();
    private JLabel psiLabel = new JLabel();
    private JLabel initiativeLabel = new JLabel();
    private JLabel assaultLabel = new JLabel();

    private JLabel detailsLabel = new JLabel(" ");

    private ComponentTable infoPanel;
    
	private CraftDesign cd;
	private CraftDesignStatistics stats;
	
    
    public CraftDesignStatsPanel(PlayerDelegate owner, PlayerDelegate craftOwner, PlayerDelegate craftOrigin, CraftDesign cd, CraftDesignStatistics cdStats) throws ApplicationError {
        this.owner = owner; 
        
        if (cd != null) {
            setDesign(cd, craftOwner, craftOrigin, cdStats);
        }
        
        MotActionListener selectWarpSpeed = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getWarpSpeedDescription());
			}
		};
        MotActionListener attack = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getAttackBonusDescription());
			}
		};
        MotActionListener manouerability = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getManouverabilityDescription());
			}
		};
        MotActionListener crew = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(getCrewDescription());
			}

		};
        MotActionListener shieldBlock = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(getShieldBlockDescription());
			}
		};
        MotActionListener assault = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getAssaultBonusDescription());
			}
		};
        
        MotActionListener battleSpeed = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getBattleSpeedDescription());
			}
		};
        MotActionListener defender = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getDefenceBonusDescription());
			}
		};
        MotActionListener initiative = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getInitiativeDescription());
			}
		};
        MotActionListener psi = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(getPsiDescription());
			}
		};
        MotActionListener maxDamage = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getMaxDamageDescription());
			}
		};
        MotActionListener cloak = new MotActionListener() {
			protected void _doAction() throws ApplicationError, ValidationError {
				detailsLabel.setText(stats == null ? null : stats.getCloakDescription());
			}
		};
        infoPanel = new ComponentTable("TPR,TR,TR,TR,TR,TR,TR,TR,TR,TR,TR,TR,TR,TPR", 3, 14, 0, 3, 0);
        infoPanel.setOpaque(false);
        
        infoPanel.add(new JPanel());
        infoPanel.add(setup(createStatsToggle("Warp Spd", selectWarpSpeed)));
        infoPanel.add(setup(warpSpeedLabel));
        infoPanel.add(setup(createStatsToggle("Attack", attack)));
        infoPanel.add(setup(attackerBonusLabel));
        infoPanel.add(setup(createStatsToggle("Manvblty", manouerability)));
        infoPanel.add(setup(manouverabilityLabel));
        infoPanel.add(setup(createStatsToggle("Crew", crew)));
        infoPanel.add(setup(crewLabel));
        infoPanel.add(setup(createStatsToggle("Shield Block", shieldBlock)));
        infoPanel.add(setup(shieldBlockLabel));        
        infoPanel.add(setup(createStatsToggle("Assault", assault)));
        infoPanel.add(setup(assaultLabel));
        infoPanel.add(new JPanel());

        infoPanel.add(new JPanel());
        infoPanel.add(setup(createStatsToggle("Battle Spd", battleSpeed)));
        infoPanel.add(setup(battleSpeedLabel));
        infoPanel.add(setup(createStatsToggle("Defend", defender)));
        infoPanel.add(setup(defenderBonusLabel));
        infoPanel.add(setup(createStatsToggle("Initiative", initiative)));
        infoPanel.add(setup(initiativeLabel));
        infoPanel.add(setup(createStatsToggle("Psi", psi)));
        infoPanel.add(setup(psiLabel));
        infoPanel.add(setup(createStatsToggle("Max Damage", maxDamage)));
        infoPanel.add(setup(hitPointsLabel));
        infoPanel.add(setup(createStatsToggle("Cloak", cloak)));
        infoPanel.add(setup(cloakLabel));
        infoPanel.add(new JPanel());

        infoPanel.add(detailsLabel, "SPF");
        
        add(infoPanel);

        testResize();
    }
    
    private ButtonGroup craftStatsBG = new ButtonGroup();
    private JComponent createStatsToggle(String label, ActionListener action) {
		return PanelUtils.createRadioButton(label, action, craftStatsBG, false) ;   	
    }
    
    protected void drawSizeRelativeObjects() {
        setPosition(infoPanel, 0.1, 0.8, 0.05, 0.9);
        infoPanel.doLayout();
    }
    
    protected void postPaint(Graphics2D g) {
    }
    
      
    private JScrollPane setup(JScrollPane component) {
        component.setOpaque(false);
        if (component.getViewport() != null) {
            component.getViewport().setOpaque(false);
            component.setBorder(null);
        }
        return component;
    }
    
    private JComponent setup(JComponent component) {
        component.setOpaque(false);
        return component;
    }

    public void updateDesign(CraftDesign cd, PlayerDelegate craftOwner, PlayerDelegate origin, CraftDesignStatistics stats) throws ApplicationError {
        setDesign(cd, craftOwner, origin, stats);
    }
    
    public void setDesign(CraftDesign cd, PlayerDelegate craftOwner, PlayerDelegate origin, CraftDesignStatistics stats) throws ApplicationError {
        this.stats = stats;
        this.cd = cd;
    
        if (stats != null) {
	        warpSpeedLabel.setText(""+stats.getWarpSpeed());
	        battleSpeedLabel.setText(""+stats.getBattleSpeed());
	        manouverabilityLabel.setText(""+Formatter.whole(stats.getManouverability()));
	        hitPointsLabel.setText(""+stats.getMaxDamage());
	        cloakLabel.setText(""+stats.getCloak());
	        attackerBonusLabel.setText(Formatter.percentLabelNoSign(stats.getAttackBonus()));
	        initiativeLabel.setText(""+stats.getInitiative());
	        psiLabel.setText(""+stats.getPsi());
	        assaultLabel.setText(Formatter.fraction(stats.getAssaultBonus()));
        } else {
	        warpSpeedLabel.setText(L.NONE);
	        battleSpeedLabel.setText(L.NONE);
	        manouverabilityLabel.setText(L.NONE);
	        hitPointsLabel.setText(L.NONE);
	        cloakLabel.setText(L.NONE);
	        attackerBonusLabel.setText(L.NONE);
	        initiativeLabel.setText(L.NONE);
	        psiLabel.setText(L.NONE);
	        assaultLabel.setText(L.NONE);
        }
        
        if (cd.getShields() == null) {
            shieldBlockLabel.setText("-");
        } else {
            shieldBlockLabel.setText(""+cd.getShieldTechs().getDamageBlocked());
        }
        
        if (cd.getSpecials() == null) {
            defenderBonusLabel.setText("-");
            crewLabel.setText(Formatter.whole(cd.getHullTech().getCrew()));
        } else {
            Specials specialTechs = cd.getSpecialTechs();
			defenderBonusLabel.setText(Formatter.percentLabelNoSign(specialTechs.getDefenderModification()));
            crewLabel.setText(Formatter.whole(cd.getHullTech().getCrew()+specialTechs.getQtyBerthsAsInt()));
        }
        
        Enumeration<AbstractButton> e = craftStatsBG.getElements();
        while (e.hasMoreElements()) {
			AbstractButton b = e.nextElement();
			if (b.isSelected() && b.getActionListeners() != null) {
				b.getActionListeners()[0].actionPerformed(null);
			}
		}
        
        invalidate();
    }

	private String getPsiDescription() throws ApplicationError {
		if (stats == null) {
			return null;
		}
		String r = stats.getPsiDescription();
		Shields shieldTechs = cd.getShieldTechs();
		if (cd.getShields() != null && shieldTechs.getPsiRating() > 0.0) {
			r += " ("+Formatter.percentLabel(shieldTechs.getPsiRating())+" "+L.FOR_DEFENCE+" )";
		}
		return r;
	}

	private String getCrewDescription() throws ApplicationError {
		if (cd == null || cd.getHull() == null) {
			return null;
		} else {
			double specialsBonus = 0.0;
			if (cd.getSpecials() != null) {
				specialsBonus = cd.getSpecialTechs().getQtyBerths();
			}
			return L.CREW + " = " + L.HULL+" "+L.CREW+" "+cd.getHullTech().getCrew()+" + "+L.SPECIALS_BONUS+" "+Formatter.fraction(specialsBonus);
		}
	}
    

	private String getShieldBlockDescription() throws ApplicationError {
		if (cd == null) {
			return null;
		} else {
			double v = 0.0;
			if (cd.getShields() != null) {
				v = cd.getShieldTechs().getDamageBlocked();
			}
			double specialsBonus = 0.0;
			if (cd.getSpecials() != null) {
				//specialsBonus = cd.getSpecials().get;
			}
			return L.SHIELD_BLOCK + " = " + L.SHIELD+" "+v+" + "+L.SPECIALS_BONUS+" "+Formatter.percentLabelNoSign(specialsBonus);
		}
	}

	private JScrollPane scrollPanel;

    public JScrollPane getScrollPane() {
        if (scrollPanel == null) {
            scrollPanel = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        }
        return scrollPanel;
    }


}