/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.craft;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.renderers.ListItemControlPanelListener;
import mot.client.swing.tables.TechnologyListSelectionModel;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.craft.actions.CraftDesignButtonAuto;
import mot.client.swing.window.craft.actions.CraftDesignButtonCancel;
import mot.client.swing.window.craft.actions.CraftDesignButtonClose;
import mot.client.swing.window.craft.actions.CraftDesignButtonHullProvider;
import mot.client.swing.window.craft.actions.CraftDesignButtonSelectTech;
import mot.client.swing.window.craft.actions.CraftDesignSelectTechnology;
import mot.client.swing.window.main.MotAWTMouseListener;
import mot.common.bean.SalvageList;
import mot.common.designs.CraftDesign;
import mot.common.lang.L;
import mot.common.technology.Computer;
import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Shield;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class CraftDesignWindow extends MotInternalWindow implements ListSelectionListener, TechnologyTypeSelectionListener, CraftDesignButtonHullProvider, ListItemControlPanelListener {

	private CraftDesignPanel craftDesignPanel;
    private ComponentTable controlPanel;

    private JLabel totalSpace = new JLabel();
    private JLabel remainingSpace = new JLabel();
    private JLabel costToBuild = new JLabel();
    private JLabel costToBuildNow = new JLabel();
    
    private PlayerDelegate origin;
    private CraftDesign design;
    private Technology selection;
    
    public CraftDesignButtonSelectTech hullsButton;
    public CraftDesignButtonSelectTech enginesButton;
    public CraftDesignButtonSelectTech computersButton;
    public CraftDesignButtonSelectTech scannersButton;
    public CraftDesignButtonSelectTech weaponsButton;
    public CraftDesignButtonSelectTech shieldsButton;
    public CraftDesignButtonSelectTech specialsButton;
    public CraftDesignButtonSelectTech salvageButton;
    public MotButton closeButton;
    public MotButton cancelButton;
 
    private CraftDesignSelectTechnology technologySelector;
    
    /**
     * The list of items this player has salvaged and are available for re-use.
     */
	private SalvageList salvage;
    
    private static final String TITLE = L.CRAFT_DESIGN;
	private CraftDesignGlobalMouseListener globalMouseListener;

    public CraftDesign getCraftDesign() { return design; }
    
    /**
     * Design a craft type window
     * @param parent
     * @param modal
     * @throws ValidationError 
     */
    public CraftDesignWindow(MotInternalWindow parent, CraftDesign design, UserPlayerDelegate owner, PlayerDelegate origin) throws ApplicationError, ValidationError {
        super(owner, parent, TITLE, null);
        this.design = design;
        this.origin = origin;
        salvage = owner.getSalvage();
        
        if (owner.getPlayerCharacter().isResourceful()) {
        	if (design != null && design.getSpecials() != null) {
        		Specials specials = design.getSpecialTechs();
        		specials.removeGroup(Special.GROUP_REPAIR_UNIT);
        		design.setSpecials(specials);
        	}
        }
        
        init();
    }
    
    private Technology[] getAllWithGroup(TechnologyList techs , String group) {
    	if (techs == null) {
    		return null;
    	} else {
    		return techs.getAllWithGroup(group);
    	}
    }
    
    public void init() throws ApplicationError, ValidationError {
        ButtonGroup group = new ButtonGroup();
        hullsButton = new CraftDesignButtonSelectTech(
        		this, 
        		CraftDesignButtonSelectTech.ADD_HULL, 
        		getAllWithGroup(owner.getHulls(), Hull.GROUP_CRAFT), 
        		group);
        enginesButton = new CraftDesignButtonSelectTech(
        		this, 
        		CraftDesignButtonSelectTech.ADD_ENGINE, 
        		owner.getEngines(),
        		group);
        computersButton = new CraftDesignButtonSelectTech(this, 
        		CraftDesignButtonSelectTech.ADD_COMPUTER, 
        		getAllWithGroup(owner.getComputers(), Computer.GROUP_COMPUTER),
        		group);
        scannersButton = new CraftDesignButtonSelectTech(
        		this,  
        		CraftDesignButtonSelectTech.ADD_SCANNER, 
        		owner.getScanners(),
        		group);
        weaponsButton = new CraftDesignButtonSelectTech(
        		this, 
        		CraftDesignButtonSelectTech.ADD_WEAPON, 
        		owner.getWeapons().getCraftWeapons(null),
        		group);
        shieldsButton = new CraftDesignButtonSelectTech(
        		this, 
        		CraftDesignButtonSelectTech.ADD_SHIELD, 
        		owner.getShields(),
        		group);
        specialsButton = new CraftDesignButtonSelectTech(
        		this, 
        		CraftDesignButtonSelectTech.ADD_SPECIAL, 
        		owner.getSpecials(),
        		group);
        salvageButton = new CraftDesignButtonSelectTech(
        		this, 
        		CraftDesignButtonSelectTech.ADD_SALVAGE, 
        		salvage,
        		group);
        cancelButton = new CraftDesignButtonCancel(this);
        closeButton = new CraftDesignButtonClose(this);
        
        hullsButton.setMinimumSize(new Dimension(175,20));
        hullsButton.setPreferredSize(new Dimension(175,20));
        
        controlPanel = new ComponentTable("HV", 9, 1, 10, 5, 5);
        controlPanel.add(hullsButton);
        controlPanel.add(enginesButton);
        controlPanel.add(computersButton);
        controlPanel.add(scannersButton);
        controlPanel.add(weaponsButton);
        controlPanel.add(shieldsButton);
        controlPanel.add(specialsButton);
        controlPanel.add(salvageButton);
        controlPanel.add(new JPanel(), "HTS");
        //controlPanel.setMinimumSize(new Dimension(195,700));
        //controlPanel.setPreferredSize(new Dimension(195,1200));
        
        hullsButton.setToolTipText(L.TT_CRAFT_DESIGN_HULL);
        enginesButton.setToolTipText(L.TT_CRAFT_DESIGN_ENGINES);
        computersButton.setToolTipText(L.TT_CRAFT_DESIGN_COMPUTERS);
        scannersButton.setToolTipText(L.TT_CRAFT_DESIGN_SCANNERS);
        weaponsButton.setToolTipText(L.TT_CRAFT_DESIGN_WEAPONS);
        shieldsButton.setToolTipText(L.TT_CRAFT_DESIGN_SHIELDS);
        specialsButton.setToolTipText(L.TT_CRAFT_DESIGN_SPECIALS);
        salvageButton.setToolTipText(L.TT_CRAFT_DESIGN_SALVAGE);
        
        technologySelector = new CraftDesignSelectTechnology(this);
        craftDesignPanel = new CraftDesignPanel(owner, owner, origin, design, null, technologySelector, this, this, null);

        JPanel bottomButtonPanel = new JPanel();
        bottomButtonPanel.add(new CraftDesignButtonAuto(this, origin));
        bottomButtonPanel.add(new CraftDesignButtonAuto(this, origin, this));
        bottomButtonPanel.add(PanelUtils.createComponentStack(new JLabel(L.__TOTAL_SPACE__), totalSpace));
        bottomButtonPanel.add(PanelUtils.createComponentStack(new JLabel(L.__REMAINING_SPACE__), remainingSpace));
        bottomButtonPanel.add(PanelUtils.createComponentStack(new JLabel(L.__BUILD_COST__), costToBuild));
        bottomButtonPanel.add(PanelUtils.createComponentStack(new JLabel(L.__BUILD_COST_NOW__), costToBuildNow));
        totalSpace.setHorizontalAlignment(SwingConstants.CENTER);
        remainingSpace.setHorizontalAlignment(SwingConstants.CENTER);
        costToBuild.setHorizontalAlignment(SwingConstants.CENTER);
        costToBuildNow.setHorizontalAlignment(SwingConstants.CENTER);

        ComponentTable detailsPanel = new ComponentTable("HV", 2, 1, 0, 0, 0);
        detailsPanel.add(getCraftDesignPanel(), "HVS");
        detailsPanel.add(bottomButtonPanel);
        
        JScrollPane controlPanelScrollPane = new JScrollPane(controlPanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        //controlPanelScrollPane.setPreferredSize(new Dimension(175,1200));
        controlPanelScrollPane.setMinimumSize(new Dimension(210,700));

        ComponentTable rightButtonPanel = new ComponentTable("HV", 3, 1, 5, 5, 5);
        rightButtonPanel.add(controlPanelScrollPane,"HVS");
        rightButtonPanel.add(closeButton);
        rightButtonPanel.add(cancelButton);


        ComponentTable main = new ComponentTable("HV,HVP", 1, 2, 5, 3, 3);
        main.add(rightButtonPanel);
        main.add(detailsPanel);

        getContentPane().add(addTitleBar(main));
        calculate(design);
        
        globalMouseListener = new CraftDesignGlobalMouseListener(this, this);
		MotAWTMouseListener.addListener(globalMouseListener );

        show();
        waitUntilClosed();
    }
    
    /**
     * Called when a selection has changed in a list.
     */
    public void valueChanged(ListSelectionEvent e) {
        TechnologyListSelectionModel source = (TechnologyListSelectionModel) e.getSource();
        selection = source.getSelectedTechnology();
        try {
			calculate(design);
		} catch (ApplicationError ex) {
			Logger.logError("Faled", ex);
		}
    }
    
    /**
     * @param tech The technology to modify
     * @param qty A value of -1 = remove, +1 = add, -2 = remove all and +2 = max qty
     */
    public void changeListItem(Technology tech, int qty) throws ValidationError, ApplicationError {
        if (tech == null) {
            return;
        }
        if (qty == 1) {
            design.add(tech, true);
        } else if (qty == -1) {
            design.remove(tech);
        } else if (qty == 2) {
            design.fill(tech);
        } else if (qty == -2) {
            design.removeAll(tech);
        }
        selection = tech;
        if (tech instanceof Weapon && weaponsButton.isSelected()) {
        	weaponsButton.selectTechType(true);
        }
        getCraftDesignPanel().updateDesign(design, null, owner, origin);
        //getCraftDesignPanel().setSelection(selection);
        calculate(design);
    }

	public void calculate(CraftDesign cd) throws ApplicationError {
        if (cd == null) {
            totalSpace.setText("");
            remainingSpace.setText("");
            costToBuild.setText("");
            costToBuildNow.setText("");
        } else {
            totalSpace.setText(""+cd.getHullTech().getSpace());
            remainingSpace.setText(""+cd.getRemainingSpace());
            costToBuild.setText(""+cd.getCostToBuild());
            costToBuildNow.setText(""+cd.getCostToBuildUsingExistingTechs(salvage));
        }
    }

	public ComponentTable getControlPanel() {
		return controlPanel;
	}

	public CraftDesign getDesign() {
		return design;
	}

	public void setDesign(CraftDesign cd) {
		design = cd;
	}

	public Technology getSelection() {
		return selection;
	}

	public void setSelection(Technology selection) {
		this.selection = selection;
	}

	public void updateDesign(CraftDesign design) throws ApplicationError, ValidationError {
		this.design = design;
		//getCraftDesignPanel().setSelection(null);
		getCraftDesignPanel().updateDesign(design, null, owner, origin);
		calculate(design);
	}

    private void selectTechType(Technology tech) throws ApplicationError, ValidationError {
    	if (tech instanceof Hull) {
    		selectHull();
    	} else if (tech instanceof Engine) {
    		selectEngine();
    	} else if (tech instanceof Computer) {
    		selectComputer();
    	} else if (tech instanceof Scanner) {
    		selectScanner();
    	} else if (tech instanceof Shield) {
    		selectShields();
    	} else if (tech instanceof Weapon) {
    		selectWeapons();
    	} else if (tech instanceof Special) {
    		selectSpecials();
    	}
	}


	//
	// Methods implementing TechnologyTypeSelectionListener
	// Called when user clicks on a tech area of the picture. 
	//
	public void selectHull() throws ApplicationError, ValidationError {
		hullsButton.selectTechType(false);
		getCraftDesignPanel().getPicturePanel().selectHullBackground();
		getCraftDesignPanel().getPicturePanel().getTechTypeSelectionPanel().selectHull();
	}
	public void selectEngine() throws ApplicationError, ValidationError {
		enginesButton.selectTechType(false);
		getCraftDesignPanel().getPicturePanel().selectEngineBackground();
		getCraftDesignPanel().getPicturePanel().getTechTypeSelectionPanel().selectEngine();
	}
	public void selectComputer() throws ApplicationError, ValidationError {
		computersButton.selectTechType(false);
		getCraftDesignPanel().getPicturePanel().selectComputerBackground();
		getCraftDesignPanel().getPicturePanel().getTechTypeSelectionPanel().selectComputer();
	}
	public void selectScanner() throws ApplicationError, ValidationError {
		scannersButton.selectTechType(false);
		getCraftDesignPanel().getPicturePanel().selectScannerBackground();
		getCraftDesignPanel().getPicturePanel().getTechTypeSelectionPanel().selectScanner();
	}
	public void selectShields() throws ApplicationError, ValidationError {
		shieldsButton.selectTechType(false);
		getCraftDesignPanel().getPicturePanel().selectShieldsBackground();
		getCraftDesignPanel().getPicturePanel().getTechTypeSelectionPanel().selectShields();
	}
	public void selectSpecials() throws ApplicationError, ValidationError {
		specialsButton.selectTechType(false);
		getCraftDesignPanel().getPicturePanel().selectSpecialsBackground();
		getCraftDesignPanel().getPicturePanel().getTechTypeSelectionPanel().selectSpecials();
	}
	public void selectWeapons() throws ApplicationError, ValidationError {
		weaponsButton.selectTechType(false);
		getCraftDesignPanel().getPicturePanel().selectWeaponsBackground();
		getCraftDesignPanel().getPicturePanel().getTechTypeSelectionPanel().selectWeapons();
	}

	public CraftDesignPanel getCraftDesignPanel() {
		return craftDesignPanel;
	}
	
	public void hide() {
		super.hide();
		if (globalMouseListener != null) globalMouseListener.removeListener();
	}

	public String getCurrentHullName() {
		return design.getHullName();
	}

    
}

class PicturePanelGlassPane extends Component implements MouseListener {
	
	private CraftDesignWindow craftDesignWindow;
	private CraftDesignPicturePanel picturePanel;
	
	public PicturePanelGlassPane(CraftDesignWindow craftDesignWindow, CraftDesignPicturePanel picturePanel) {
		this.craftDesignWindow = craftDesignWindow;
		this.picturePanel = picturePanel;
		addMouseListener(this);
	}
	
	public void mouseClicked(MouseEvent e) {
	}
	public void mousePressed(MouseEvent e) {
    	if (e.getClickCount() == 1) {
    		Point p = SwingUtilities.convertPoint(craftDesignWindow, e.getPoint(), picturePanel);
    		picturePanel.selectAreaAt(p);
    	}
	}
	public void mouseReleased(MouseEvent e) {	 }
	public void mouseEntered(MouseEvent e) {	}
	public void mouseExited(MouseEvent e) {   }
}