/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.craft;
import java.awt.Dimension;

import javax.swing.JScrollPane;

import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.lists.CraftDesignList;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.common.actions.CloseButton;
import mot.client.swing.window.craft.actions.CloseAndUpdateAllCraftDesignButton;
import mot.client.swing.window.craft.actions.DeleteCraftDesignButton;
import mot.client.swing.window.craft.actions.NewCraftDesignButton;
import mot.client.swing.window.craft.actions.UpdateCraftDesignButton;
import mot.client.swing.window.main.MotConditionalAction;
import mot.client.swing.window.solarsystem.actions.ProjectChangeListener;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.designs.CraftDesignsSupplier;
import mot.common.designs.ProjectDesign;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

public class CraftDesignsWindow extends MotInternalWindow implements ProjectChangeListener {

	private PlayerDelegate origin;
	private CraftDesignsSupplier designsSupplier;
	
    private CraftDesignPanel craftDesignPanel;
    private CraftDesignList craftDesignList;
    
    private CraftDesign currentSelection;
	private CraftDesigns restrictedDesignsList;
    
    public CraftDesignsWindow(MotInternalWindow parent, UserPlayerDelegate owner, PlayerDelegate origin) throws ApplicationError, ValidationError {
        super(owner, parent, L.CRAFT_DESIGN_LIST_WINDOW, null);
        this.origin = origin;
        this.designsSupplier = owner;
		this.restrictedDesignsList = null;
        init(false);
    }

    public CraftDesignsWindow(MotInternalWindow parent, UserPlayerDelegate owner, PlayerDelegate origin, CraftDesignsSupplier designsSupplier, CraftDesigns designs) throws ApplicationError, ValidationError {
        super(owner, parent, L.AVAILABLE_CRAFT_DESIGNS, null);
        this.origin = origin;
        this.designsSupplier = designsSupplier;
		this.restrictedDesignsList = designs;
        init(true);
    }

    private void init(boolean selectButton) throws ApplicationError, ValidationError {
    	CraftDesigns designs = (restrictedDesignsList == null) ? designsSupplier.getCraftDesigns() : restrictedDesignsList;
		CraftDesign [] craftDesigns = (designs != null) ? designs.getCraftDesigns() : null;
        currentSelection = null;
        if (craftDesigns != null && craftDesigns.length > 0) {
        	currentSelection = craftDesigns[0];
        }
        craftDesignList = new CraftDesignList(craftDesigns, currentSelection, this);
        
        JScrollPane craftDesignScrollPane = craftDesignList.getScrollPane();

        ComponentTable controlPanel = new ComponentTable("H", 6, 1, 10, 5, 5);
        controlPanel.add(new NewCraftDesignButton(this, origin));
        controlPanel.add(new UpdateCraftDesignButton(this, origin));
        controlPanel.add(new DeleteCraftDesignButton(this));
        controlPanel.add(craftDesignScrollPane, "HVSP");
        if (selectButton) {
        	controlPanel.add(new CloseButton(L.SELECT, this));
        	controlPanel.add(new CancelButton(this));
        } else {
        	controlPanel.add(new CloseAndUpdateAllCraftDesignButton(this));
        	controlPanel.add(new CloseButton(this));
        }
        Dimension size = new Dimension(175,2000);
        controlPanel.setPreferredSize(size);
        controlPanel.setMinimumSize(size);

        craftDesignPanel = new CraftDesignPanel(owner, owner, origin, currentSelection, null, null, null, null, null);
        
        ComponentTable main = new ComponentTable("T,HVP", 1, 2, 5, 3, 3);
        //PanelUtils.setBorder(main, "MAIN");
        main.add(controlPanel);
        main.add(craftDesignPanel);
        

        getContentPane().add(addTitleBar(main));

        show();
    }

    public void updateDesign(CraftDesign design) throws ApplicationError, ValidationError {
       	craftDesignPanel.updateDesign(design, null, owner, origin);
    }
    
	public void changeDesignsSelection(ProjectDesign newProject) throws ApplicationError, ValidationError {
		currentSelection = (CraftDesign) newProject;
        if (currentSelection != null) {
            updateDesign(currentSelection);
        }
    	CraftDesigns designs = (restrictedDesignsList == null) ? designsSupplier.getCraftDesigns() : restrictedDesignsList;
		craftDesignList.update(designs, currentSelection);
		craftDesignList.repaint();
    }
    

	public CraftDesignList getCraftDesignList() {
		return craftDesignList;
	}

//	public CraftDesignPanel getCraftDesign() {
//		return craftDesignPanel;
//	}

	public CraftDesign getSelectedCraftDesign() {
		return currentSelection;
	}
	
	public class CancelButton extends MotButton implements MotConditionalAction {

		private CraftDesignsWindow parent;

		public CancelButton(CraftDesignsWindow parent) {
			super(L.CANCEL);
			this.parent = parent;
		}
		
		public void execute() throws ApplicationError, ValidationError {
			currentSelection = null;
			parent.hide();
		}

		@Override
		protected void _doAction() throws ApplicationError, ValidationError {
			execute();
		}
	}
	

}