/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.empire;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.deligates.CraftDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.RelationDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.icons.MotImageIcon;
import mot.client.swing.icons.MotImageIconNullable;
import mot.client.swing.panels.CraftPicturePanel;
import mot.client.swing.panels.IconPanelMoney;
import mot.client.swing.panels.TechnologyPicturePanel;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.common.actions.CloseButton;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.solarsystem.PlanetAnimationPanel;
import mot.common.bean.Settings;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Shield;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;

public class BalanceSheetWindow extends MotInternalWindow {

	private MotLabel expensesHeader;
	private MotLabel incomeHeader;
	private MotLabel spiesMaintenanceCost;
	private MotLabel spiesRecruitEfortPerTurn;

	public BalanceSheetWindow(MotInternalWindow parent, UserPlayerDelegate owner) throws ApplicationError, ValidationError {
        super(owner, parent, L.EMPIRE_BALANCE_SHEET, null);

        init();
        show();

        CompletedCallback fetchComplete = new CompletedCallback() {
			public void complete(Object result) throws ApplicationError, ValidationError {
				update();
			}
		};
        
        ProxyStore.getInstance().fetchPlayerDetails(MainWindow.getOwner(), fetchComplete, null);

	}

    private JPanel newLabel(String text, Color tc, Color bc, int align) {
        JLabel l = new JLabel(text);
        l.setForeground(tc);
        JPanel p = new JPanel(new FlowLayout(align));
        p.setBackground(bc);
        p.add(l);
        return p;
    }

    private JLabel newText(String text, Color tc) {
        JLabel l = new JLabel(text);
        l.setForeground(tc);
        return l;
    }

    private void init() throws ApplicationError {
        Color expensesBackgroundColor = PropertyManager.getMandatoryAsColor("window.balancesheet.expenses.background");
        Color incomeBackgroundColor = PropertyManager.getMandatoryAsColor("window.balancesheet.income.background");
        Color headerBackgroundColor = PropertyManager.getMandatoryAsColor("window.balancesheet.header.background");

        expensesHeader = new MotLabel(L.EXPENSE, 2, null, null, 0, true);
        incomeHeader = new MotLabel(L.INCOME, 2, null, null, 0, true);
        
        expensesHeader.setBackground(headerBackgroundColor);
        expensesHeader.setOpaque(true);
        incomeHeader.setBackground(headerBackgroundColor);
        incomeHeader.setOpaque(true);

        spiesMaintenanceCost = new MotLabel();
        spiesRecruitEfortPerTurn = new MotLabel();

        ComponentTable main = new ComponentTable("-,-", 2, 2, 0, 0, 0);
        main.add(expensesHeader, "PHT");
        main.add(incomeHeader, "PHT");
        main.add(createCostPanel(expensesBackgroundColor), "PHVS");
        main.add(createIncomePanel(incomeBackgroundColor), "PHVS");

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(new CloseButton(this), null);

        JScrollPane scrollPane = new JScrollPane(new PadPanel(main), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scrollPane.getVerticalScrollBar().setUnitIncrement(Settings.getInstance().getScrollSpeed());

    	getContentPane().add(addTitleBar(scrollPane), null);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
    }

    public JPanel createCostPanel(Color expensesBackgroundColor) throws ApplicationError {
        PlanetDelegate[] planets = DeligateCacheManager.getPlanets(owner);
        CraftDelegate[] crafts = DeligateCacheManager.getCrafts(owner);
        int qtyPlanets = (planets == null) ? 0 : planets.length;
        int qtyCrafts = (crafts == null) ? 0 : crafts.length;

		String imageMissile = TechnologyPicturePanel.getActualImageThumb(Technologies.getWeapons().getTechnologyForGroup(Weapon.GROUP_MISSILE));
		String imageShield = TechnologyPicturePanel.getActualImageThumb(Technologies.getShields().getTechnologyForGroup(Shield.GROUP_SHIELD));
		String imageScanner = TechnologyPicturePanel.getActualImageThumb(Technologies.getScanners().getTechnologyForGroup(Scanner.GROUP_SCANNER));
		String imageOrbital = TechnologyPicturePanel.getActualImageThumb(Technologies.getHulls().getTechnologyForGroup(Hull.GROUP_SPACE_STATION));
		Size iconSize = new Size(25,25);

        ComponentTable panel = new ComponentTable("HV,HV,HV,HV,HVP", qtyPlanets*5+qtyCrafts+4, 5, 5, 3, 3);
        panel.setTextSize(new Integer(2));
        panel.setBackground(expensesBackgroundColor);
        int totalPlanets = 0;
        int totalCrafts = 0;
		if (planets != null && planets.length > 0) {
            for (int i=0; i<planets.length; i++) {
        		int weaponCost = planets[i].getWeaponMaintenanceCost();
                panel.add(new PlanetAnimationPanel(this, planets[i], iconSize));
                panel.add(planets[i].getName());
                panel.add(new MotImageIconNullable(imageMissile, iconSize));
                panel.add(""+weaponCost);
				panel.add(new IconPanelMoney(weaponCost, expensesBackgroundColor));
				
				int shieldCost = planets[i].getShieldMaintenanceCost();
                panel.add(new PadPanel());
                panel.add(new PadPanel());
                panel.add(new MotImageIconNullable(imageShield, iconSize));
                panel.add(""+shieldCost);
                panel.add(new IconPanelMoney(shieldCost, expensesBackgroundColor));
                
                int scannerCost = planets[i].getScannerMaintenanceCost();
                panel.add(new PadPanel());
                panel.add(new PadPanel());
                panel.add(new MotImageIconNullable(imageScanner, iconSize));
                panel.add(""+scannerCost);
				panel.add(new IconPanelMoney(scannerCost, expensesBackgroundColor));
                
				int orbitalCost = planets[i].getOrbitalMaintenanceCost();
                panel.add(new PadPanel());
                panel.add(new PadPanel());
                panel.add(new MotImageIconNullable(imageOrbital, iconSize));
                panel.add(""+orbitalCost);
				panel.add(new IconPanelMoney(orbitalCost, expensesBackgroundColor));
				
				totalPlanets += planets[i].getMaintenanceCost();
            }
        }

        panel.add(L.PLANETS_TOTAL, "HV2");
        panel.add(new PadPanel());
        panel.add(""+totalPlanets);
		panel.add(new IconPanelMoney(totalPlanets, expensesBackgroundColor));


        if (crafts != null && crafts.length > 0) {
            for (int i=0; i<crafts.length; i++) {
                int cost = crafts[i].getMaintenanceCost();
                CraftPicturePanel craftImage = new CraftPicturePanel(0, true, crafts[i]);
                PanelUtils.setTheBloodySize(craftImage, iconSize);
				panel.add(craftImage);
                panel.add(crafts[i].getName());
                panel.add(new PadPanel());
                panel.add(""+cost);
				panel.add(new IconPanelMoney(cost, expensesBackgroundColor));
				totalCrafts += cost;
            }
        }

        panel.add(L.CRAFTS_TOTAL, "HV2");
        panel.add(new PadPanel());
        panel.add(""+totalCrafts);
		panel.add(new IconPanelMoney(totalCrafts, expensesBackgroundColor));

		int spiesCosts = owner.getSpiesMaintenanceCost();
        panel.add(L.SPY_RECRUIT, "HV2");
        panel.add(new PadPanel());
		panel.add(""+spiesCosts);
		panel.add(new IconPanelMoney(spiesCosts, expensesBackgroundColor));

        panel.add(new PadPanel());
        panel.add(new PadPanel());
        panel.add(new PadPanel());
        panel.add(new PadPanel());
        panel.add(new PadPanel(), "HVSP");

        return panel;
    }

    public JPanel createIncomePanel(Color incomeBackgroundColor) throws ApplicationError {
        PlanetDelegate[] planets = DeligateCacheManager.getPlanets(owner);
        String[] trades = owner.getPlayersWithTradeAgreements();
        int qtyPlanets = (planets == null) ? 0 : planets.length;
        int qtyTrades = (trades == null) ? 0 : trades.length;

        ImageConfig tradeIcon = ImageConfigFactory.getConfigForImage("TradeImage");
		Size iconSize = new Size(25,25);

        ComponentTable panel = new ComponentTable("HV,HV,HV,HVP", qtyPlanets+qtyTrades+3, 4, 5, 3, 3);
        panel.setTextSize(new Integer(2));
        panel.setBackground(incomeBackgroundColor);
        int total = 0;
        if (planets != null && planets.length > 0) {
            for (int i=0; i<planets.length; i++) {
                int cost = (int)planets[i].getMoneyOutput();
                panel.add(new PlanetAnimationPanel(this, planets[i], iconSize));
                panel.add(planets[i].getName());
                panel.add(""+cost);
                panel.add(new IconPanelMoney(cost, incomeBackgroundColor));
                total += cost;
            }
        }
        panel.add(L.PLANETS_TOTAL, "HV2");
        panel.add(""+total);
        panel.add(new IconPanelMoney(total, incomeBackgroundColor));
        total = 0;
        if (trades != null && trades.length > 0) {
            for (int i=0; i<trades.length; i++) {
            	RelationDelegate relation = DeligateCacheManager.getRelation(owner.getName(), trades[i]);
            	int money = relation.getTradeAgreement().getMoney();
                panel.add(new MotLabel(new MotImageIcon(tradeIcon, iconSize)));
                panel.add(trades[i]);
                panel.add(""+money);
                panel.add(new IconPanelMoney(money, incomeBackgroundColor));
                total += money;
            }
        }
        panel.add(L.TRADE_TOTAL, "HV2");
        panel.add(""+total);
        panel.add(new IconPanelMoney(total, incomeBackgroundColor));
        
        panel.add(new PadPanel());
        panel.add(new PadPanel());
        panel.add(new PadPanel());
        panel.add(new PadPanel(),"HVSP");

        return panel;
    }

	private void update() {
		spiesRecruitEfortPerTurn.setText(""+MainWindow.getOwner().getSpyRecruitCosts());
	}

}