/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.empire;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.buttons.MotClickTechnologyLabelAction;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.labels.HtmlLabel;
import mot.client.swing.lists.FakeList;
import mot.client.swing.lists.FakeTechnologyLabelList;
import mot.client.swing.lists.MoralChoiceFakeList;
import mot.client.swing.panels.AncientRacePanel;
import mot.client.swing.panels.IconPanelBoralium;
import mot.client.swing.panels.IconPanelMoney;
import mot.client.swing.panels.IconPanelPopulation;
import mot.client.swing.panels.IconPanelResearch;
import mot.client.swing.panels.RacePicturePanel;
import mot.client.swing.panels.RacePicturePanelWithFlag;
import mot.client.swing.renderers.CellRendererTechnologyLabel;
import mot.client.swing.renderers.ListItemControlPanelFactory;
import mot.client.swing.renderers.ListItemControlPanelFactoryTechnology;
import mot.client.swing.renderers.ListItemControlPanelListener;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.common.actions.CloseButton;
import mot.client.swing.window.common.actions.OpenBalanceSheetWindowButton;
import mot.client.swing.window.common.actions.OpenPlanetsWindowButton;
import mot.client.swing.window.empire.actions.SellAllSalvageButton;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.PlayerCharacterBasic;
import mot.common.bean.SalvageList;
import mot.common.bean.Size;
import mot.common.event.EmpireWindowEvent;
import mot.common.lang.L;
import mot.common.leaders.GeneralLeader;
import mot.common.leaders.GovenorLeader;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.ValidationError;

public class EmpireWindow extends MotInternalWindow implements ListItemControlPanelListener {



	public EmpireWindow(UserPlayerDelegate owner) throws ApplicationError, ValidationError {
        this((MotInternalWindow)null, owner);
    }

    public EmpireWindow(MotInternalWindow parent, UserPlayerDelegate owner) throws ApplicationError, ValidationError {
        super(owner, parent, L.TITLE_EMPIRE_WINDOW, null);
        
        init(owner);
        
        show();
        update();
        
        CompletedCallback fetchComplete = new CompletedCallback() {
			public void complete(Object result) throws ApplicationError, ValidationError {
				update();
			}
		};
        
        ProxyStore.getInstance().fetchPlayerDetails(owner, fetchComplete, null);
        pollForEventOfType(EmpireWindowEvent.class, null);

    }
    
    private FakeTechnologyLabelList salvageList;
    
	private IconPanelPopulation popPanel;
	private IconPanelResearch resPanel;
	private IconPanelMoney incomePanel;
	private IconPanelMoney expensesPanel;
	private IconPanelMoney profitPanel;
	private IconPanelBoralium boraliumPanel;
	private MotLabel populationLabel;
	private MotLabel researchLabel;
	private MotLabel incomeLabel;
	private MotLabel expenseLabel;
	private MotLabel profitLabel;
	private MotLabel boraliumLabel;
	private LeaderGeneralPanel generalPanel;
	private LeaderGovenorPanel govenorPanel;
	private HtmlLabel generalLabel;
	private HtmlLabel generalDescriptionLabel;
	private HtmlLabel govenorLabel;
	private HtmlLabel govenorDescriptionLabel;

    private void init(UserPlayerDelegate owner) throws ApplicationError {


        //
        // Create Moral Choices Details Panel
        //

        MoralChoiceFakeList moralChoicePanel = new MoralChoiceFakeList(owner.getMoralChoices());
        JScrollPane moralChoiceScrollPane = moralChoicePanel.getScrollPaneNoBorder(3);
        moralChoiceScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        moralChoiceScrollPane.setMinimumSize(new Dimension(500,100));
        moralChoiceScrollPane.setPreferredSize(new Dimension(700,100));
        //JPanel moralChoiceArea = new PadPanel(moralChoiceScrollPane, 0);

        //
        // Create Production Output Panel
        //
        
        popPanel = new IconPanelPopulation(0);
        resPanel = new IconPanelResearch(0, 0);
        incomePanel = new IconPanelMoney(0,0);
        expensesPanel = new IconPanelMoney(0,0);
        profitPanel = new IconPanelMoney(0,0);
        boraliumPanel = new IconPanelBoralium(0);
        boraliumPanel.setVAlignment(SwingConstants.TOP);
        
        ComponentTable table = new ComponentTable("TR,T,HV",6,3, 3, 3,3);
        Dimension min = new Dimension(50, 15);
        table.add(new MotLabel(L.POPULATION1, 0, min, null, JLabel.LEFT, false, L.EMPIRE_POPULATION));
		table.add(populationLabel = new MotLabel("", L.EMPIRE_POPULATION));
        table.add(popPanel);
        table.add(new MotLabel(L.RESEARCH, 0, min, null, JLabel.LEFT, false, L.EMPIRE_RESEARCH));
		table.add(researchLabel = new MotLabel("", L.EMPIRE_RESEARCH));
        table.add(resPanel);
        table.add(new MotLabel(L.TOTAL_INCOME, 0, min, null, JLabel.LEFT, false, L.EMPIRE_INCOME));
		table.add(incomeLabel = new MotLabel("", L.EMPIRE_INCOME));
        table.add(incomePanel);
        table.add(new MotLabel(L.TOTAL_EXPENSES, 0, min, null, JLabel.LEFT, false, L.EMPIRE_EXPENSES));
		table.add(expenseLabel = new MotLabel("", L.EMPIRE_EXPENSES));
        table.add(expensesPanel);
        table.add(new MotLabel(L.NET_INCOME, 0, min, null, JLabel.LEFT, false, L.EMPIRE_PROFIT));
		table.add(profitLabel = new MotLabel("", L.EMPIRE_PROFIT));
        table.add(profitPanel);
        table.add(new MotLabel(L.BORALIUM, 0, min, null, JLabel.LEFT, false, L.EMPIRE_BORALIUM));
		table.add(boraliumLabel = new MotLabel("", L.EMPIRE_BORALIUM));
        table.add(boraliumPanel);
        //table.setMinimumSize(new Dimension(500,200));
        //table.setPreferredSize(new Dimension(1000,200));

        popPanel.setToolTipText(L.EMPIRE_POPULATION);
        resPanel.setToolTipText(L.EMPIRE_RESEARCH);
        incomePanel.setToolTipText(L.EMPIRE_INCOME);
        expensesPanel.setToolTipText(L.EMPIRE_EXPENSES);
        profitPanel.setToolTipText(L.EMPIRE_PROFIT);
        boraliumPanel.setToolTipText(L.EMPIRE_BORALIUM);
        
        //
        // Create Player Details Panel
        //
        
        RacePicturePanel racePicturePanel = new RacePicturePanelWithFlag(owner);
        if (MainWindow.isScreen900High()) {
        	PanelUtils.setTheBloodySize(racePicturePanel, 200,300);
        } else {
        	PanelUtils.setTheBloodySize(racePicturePanel, 100,180);
        }

		ListItemControlPanelFactory controlPanelFactory = new ListItemControlPanelFactoryTechnology(this, true);
		SellSalvageListener sellSalvageListener = new SellSalvageListener();
        salvageList = new FakeTechnologyLabelList(new CellRendererTechnologyLabel(false, true, false, new Size(15,15), true, 2, false, null, null, true, sellSalvageListener, null, controlPanelFactory));
        salvageList.setToolTipText(L.EMPIRE_SALVAGE);
        
        PlayerCharacterBasic playerCharacter = owner.getPlayerCharacter();
        FakeList characterBonusList = new FakeList();
        characterBonusList.setContent(playerCharacter.getDescription(owner.getPlayerResources()), null);
        characterBonusList.setToolTipText(L.EMPIRE_BONUSUS);
        characterBonusList.getScrollPane().setMinimumSize(new Dimension(200,80));
        
        //
        // Create Leader Details Panel
        //

        AncientRacePanel ancientPicturePanel = new AncientRacePanel(owner, false);
        
        String ancientName = owner.getAlignment();
        JLabel ancientRaceDescription = new HtmlLabel(SwingConstants.CENTER, 4, false);
        if (ancientName != null) {
        	//RaceGraphics ancientConfig = GraphicsFactory.getRaceGraphicsList().getGraphicsFor(ancientName);
        	ancientRaceDescription.setText(ancientName);
        } else {
        	ancientRaceDescription.setText(L.NOT_ALIGNED_TO_ANY_ANCIENT_RACES);
        }

        generalPanel = new LeaderGeneralPanel(null);
        govenorPanel = new LeaderGovenorPanel(null);
        if (MainWindow.isScreen900High()) {
            PanelUtils.setTheBloodySize(ancientPicturePanel, 200,200);
	        PanelUtils.setTheBloodySize(generalPanel, 200,200);
	        PanelUtils.setTheBloodySize(govenorPanel, 200,200);
        } else {
            PanelUtils.setTheBloodySize(ancientPicturePanel, 150,150);
	        PanelUtils.setTheBloodySize(generalPanel, 150,150);
	        PanelUtils.setTheBloodySize(govenorPanel, 150,150);
        }

        generalLabel = new HtmlLabel(SwingConstants.CENTER, 4, false);
        govenorLabel = new HtmlLabel(SwingConstants.CENTER, 4, false);
        generalDescriptionLabel = new HtmlLabel("", SwingConstants.CENTER);
        govenorDescriptionLabel = new HtmlLabel("", SwingConstants.CENTER);

//        PanelUtils.setTheBloodySize(generalLabel, 125,190);
//        PanelUtils.setTheBloodySize(govenorLabel, 125,190);
        generalLabel.setMinimumSize(new Dimension(100,60));
        generalLabel.setPreferredSize(new Dimension(250,80));
        govenorLabel.setMinimumSize(new Dimension(100,60));
        govenorLabel.setPreferredSize(new Dimension(250,80));

        ComponentTable leaderDetailsPanel = new ComponentTable("P,P,P", 3,3, 0, 3,1);
        leaderDetailsPanel.add(ancientPicturePanel);
        leaderDetailsPanel.add(generalPanel);
        leaderDetailsPanel.add(govenorPanel);
        leaderDetailsPanel.add(ancientRaceDescription, "HV");
        leaderDetailsPanel.add(generalLabel, "HV");
        leaderDetailsPanel.add(govenorLabel, "HV");
        leaderDetailsPanel.add(new PadPanel(), "HTS");
        leaderDetailsPanel.add(generalDescriptionLabel, "HTS");
        leaderDetailsPanel.add(govenorDescriptionLabel, "HTS");
        
//        PanelUtils.setTheBloodySize(leaderDetailsPanel, 2000,300);
        //leaderDetailsPanel.setMinimumSize(new Dimension(500,200));
        //leaderDetailsPanel.setPreferredSize(new Dimension(1000,200));
        
        

        //
        // Create Main Panel
        //

        ComponentTable topLeft = new ComponentTable("HV", 2, 1, 3,0,5);
        topLeft.add(new MotBorder(moralChoiceScrollPane, L.MORAL_CHOICES));
        topLeft.add(new MotBorder(table, L.TOTAL_PRODUCTION_OUTPUT));

        ComponentTable topRight = new ComponentTable("HV", 2, 1, 3,0,5);
        topRight.add(racePicturePanel);
        topRight.add(PanelUtils.createNamedControlV(owner.getName()+L.BONUSES, characterBonusList.getScrollPane()),"HVS");
        topRight.setMinimumSize(new Dimension(250, 150));

        ComponentTable top = new ComponentTable("HVP,HV", 1,2, 3, 0,0);
        top.add(topLeft);
        top.add(topRight);

        ComponentTable middle = new ComponentTable("HV,HV", 1,2, 3, 0,3);
        middle.setGridWidths("0.35,0.65");
		middle.add(PanelUtils.createNamedControlV(L.SALVAGE_COLLECTED, salvageList.getScrollPane()));
        middle.add(leaderDetailsPanel);
        

        ComponentTable main = new ComponentTable("HV", 2, 1, 0,0,0);
        //main.setGridHeights("0.25,0.75");
        main.add(top);
        main.add(new MotBorder(middle, L.EMPIRE_S_LEADERS_DETAILS),"HVS");
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(new CloseButton(this), null);
        buttonPanel.add(new SellAllSalvageButton(this), null);
        buttonPanel.add(new OpenPlanetsWindowButton(this), null);
        buttonPanel.add(new OpenBalanceSheetWindowButton(this), null);

        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        getContentPane().add(addTitleBar(main), null);        
    }
    
    public void update() throws ApplicationError {
//    	getContentPane().removeAll();
//    	init(getOwner());
//    	getContentPane().validate();
        String race = owner.getName();
        String alignment = owner.getAlignment();
        int population = (int) owner.getTotalPopulation();
        int income = (int) owner.getTotalIncome(true);
        int expenses = (int) owner.getTotalMaintenanceCost();
        int profit = (int) owner.getNetIncome();
        int research = (int) owner.getTotalResearch();
        int boralium = (int) owner.getBoralium();

        popPanel.update(population);
        resPanel.update(research, 0);
        incomePanel.update(income,0);
        expensesPanel.update(expenses,0);
        profitPanel.update(profit,0);
        boraliumPanel.update(boralium);
        
		populationLabel.setText(""+population);
		researchLabel.setText(Formatter.whole(research));
		incomeLabel.setText(Formatter.whole(income));
		expenseLabel.setText(Formatter.whole(expenses));
		profitLabel.setText(Formatter.whole(profit));
		boraliumLabel.setText(Formatter.whole(boralium));

        GeneralLeader generalLeader = owner.getGeneralLeader(); 
        GovenorLeader govenorLeader = owner.getGovenorLeader(); 
        generalPanel.update(generalLeader);
        govenorPanel.update(govenorLeader);

        if (generalLeader == null) {
            generalLabel.setText(L.NO_GENERAL);
            generalDescriptionLabel.setText("");
        } else {
        	String txt = generalLeader.getName()+L.__LEVEL_+generalLeader.getLevel()+" ]"; 
            generalLabel.setTextCentered(txt);
            generalDescriptionLabel.setTextCentered(generalLeader.getDetails().getDescription(), 30);
        }
        if (govenorLeader == null) {
            govenorLabel.setText(L.NO_GOVENOR);
            govenorDescriptionLabel.setText("");
        } else {
        	String txt = govenorLeader.getName()+L.__LEVEL_+govenorLeader.getLevel() + " ]"; 
            govenorLabel.setTextCentered(txt);
            govenorDescriptionLabel.setTextCentered(govenorLeader.getDetails().getDescription(), 30);
        }

        SalvageList salvage = owner.getSalvage();
        TechnologyList salvageSorted = Comp.sortByName(salvage);
        TechnologyBundle[] tb = (salvageSorted != null) ? salvageSorted.getDistinct() : null; 
        salvageList.setContent(tb, null);
        
        repaint();

    }

    /**
     * Change quantity of salvage
     * @param tech The technology to modify
     * @param qty A value of -1 = remove, +1 = add, -2 = remove all and +2 = max qty
     */
	public void changeListItem(Technology technology, int i) throws ApplicationError, ValidationError {
		SalvageList salvage = owner.getSalvage();
		if (salvage != null && salvage.contains(technology)) {
			int qty = (i == -2) ? salvage.getQuantity(technology) : 1;
			SalvageList salvageToSell = new SalvageList(technology, qty);
			CompletedCallback callback = new SellSalvageCallback(technology, qty);
			ServerProxy serverProxy = ProxyStore.getInstance();
			serverProxy.sellSavlage(salvageToSell, callback, null);
		}
	}

    private class SellSalvageCallback implements CompletedCallback {
		private Technology technology;
		private int qty;
		public SellSalvageCallback(Technology technology, int qty) {
			this.technology = technology;
			this.qty = qty;
		}
		public void complete(Object result) throws ApplicationError, ValidationError {
			if (result != null && result instanceof Integer) {
				int moneyRaised = (Integer) result;
				MainWindow.getInstance().showMessageGalaxy(L.SOLD_SALVAGE+technology.getFullNameSpaceSeperated()+L._FOR_+moneyRaised, true);
			}
			update();
			MainWindow.getInstance().getGalaxyMapConfiguration().getGalaxyView().updateStatus(getOwner());
		}
    	
    };
    
    private class SellSalvageListener extends MotClickTechnologyLabelAction {
		protected void selectTechnology(Technology technology) throws ApplicationError, ValidationError {
			SalvageList salvage = owner.getSalvage();
			if (salvage != null && salvage.contains(technology)) {
				CompletedCallback callback = new SellSalvageCallback(technology, 1);
				ServerProxy serverProxy = ProxyStore.getInstance();
				serverProxy.sellSavlage(new SalvageList(technology), callback, null);
			}
		}
    };
    

}