/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.empire;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotTransparentLabel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.labels.TechnologyLabelClickListener;
import mot.client.swing.lists.FakeTechnologyLabelList;
import mot.client.swing.panels.TechnologyPicturePanel;
import mot.client.swing.renderers.CellRendererTechnologyLabel;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Size;
import mot.common.command_results.TechnologyListCR;
import mot.common.lang.L;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class EspionageReportWindow extends MotInternalWindow implements ActionListener, TechnologyLabelClickListener {

    private static final Dimension SIZE = new Dimension(650, 375);

    private PlayerDelegate player;
    private TechnologyList data;
    
    private FakeTechnologyLabelList list = new FakeTechnologyLabelList(new CellRendererTechnologyLabel(false, true, false, new Size(20,20), true, 0, false, null, null, true, this, null, null));

    private Technology selectedTechnology;
	private TechnologyPicturePanel technologyPicturePanel;

	private MotButtonPanel selectTechnologyButtonPanel;

	private SelectTechnologyButton selectTechnologyButton;

	private RaceRelationPanel raceRelationPanel;

	class SelectTechnologyButton extends MotButton implements CompletedCallback {
		public SelectTechnologyButton() { super("Select for Espionage"); }
		protected void _doAction() throws ApplicationError, ValidationError {
			ProxyStore.getInstance().setEspionageTarget(player, selectedTechnology, this, this);
		}
		public void complete(Object result) throws ApplicationError, ValidationError {
			raceRelationPanel.updateEspionageTarget();
		}
	}


    public EspionageReportWindow(MotInternalWindow parent, RaceRelationPanel raceRelationPanel, UserPlayerDelegate owner, PlayerDelegate player) throws ApplicationError, ValidationError {
        super(owner, parent, L.ESPIONAGE_REPORT_WINDOW + "[ "+player.getName()+" ]", SIZE);
		this.raceRelationPanel = raceRelationPanel;

        this.player = player;

        init();
        show();
    }

    private void init() throws ApplicationError, ValidationError {
    	ComponentTable warsPanel = new ComponentTable("HV",1,1,0,3,3);
        warsPanel.setBackground(new Color(30,0,0));
        warsPanel.setOpaque(true);
        
        String [] enemies = player.getEnemyPlayers();
        warsPanel.addRow();
        warsPanel.add(new MotTransparentLabel(L.WARS,3,JLabel.CENTER,true));
        if (enemies != null && enemies.length > 0) {
            for (int i=0; i<enemies.length;i++) {
                warsPanel.addRow();
                warsPanel.add(new MotTransparentLabel(enemies[i],JLabel.CENTER));
            }
        } else {
            warsPanel.addRow();
            warsPanel.add(new MotTransparentLabel(L.NONE,JLabel.CENTER));
        }
        warsPanel.add(new JPanel());
        
        ComponentTable tradersPanel = new ComponentTable("HV",1,1,0,3,3);
        tradersPanel.setBackground(new Color(0,0,30));
        tradersPanel.setOpaque(true);
        String [] traders = player.getPlayersWithTradeAgreements();
        tradersPanel.addRow();
        tradersPanel.add(new MotTransparentLabel(L.TRADERS,3,JLabel.CENTER,true));
        if (traders != null && traders.length > 0) {
            for (int i=0; traders != null && i<traders.length;i++) {
                tradersPanel.addRow();
                tradersPanel.add(new MotTransparentLabel(traders[i],JLabel.CENTER));
            }
        } else {
            tradersPanel.addRow();
        	tradersPanel.add(new MotTransparentLabel(L.NONE,JLabel.CENTER));
        }
        tradersPanel.add(new JPanel());
        
        ComponentTable researchersPanel = new ComponentTable("HV",1,1,0,3,3);
        researchersPanel.setBackground(new Color(20,20,20));
        researchersPanel.setOpaque(true);
        String [] researchers = player.getPlayersWithTradeAgreements();
        researchersPanel.addRow();
        researchersPanel.add(new MotTransparentLabel(L.RESEARCHERS,3,JLabel.CENTER,true));
        if (researchers != null && researchers.length > 0) {
            for (int i=0; researchers != null && i<traders.length;i++) {
            	researchersPanel.addRow();
            	researchersPanel.add(new MotTransparentLabel(researchers[i],JLabel.CENTER));
            }
        } else {
        	researchersPanel.addRow();
        	researchersPanel.add(new MotTransparentLabel(L.NONE,JLabel.CENTER));
        }
        researchersPanel.add(new JPanel());
        
        ComponentTable alliancesPanel = new ComponentTable("HV",1,1,0,3,3);
        alliancesPanel.setBackground(new Color(0,30,0));
        alliancesPanel.setOpaque(true);
        String [] alliances = player.getPlayersAllianceTreaty();
        alliancesPanel.addRow();
        alliancesPanel.add(new MotTransparentLabel(L.ALLIANCES,3,JLabel.CENTER,true));
        if (alliances != null && alliances.length > 0) {
            for (int i=0; i<alliances.length;i++) {
                alliancesPanel.addRow();
                alliancesPanel.add(new MotTransparentLabel(alliances[i],JLabel.CENTER));
            }
        } else {
            alliancesPanel.addRow();
        	alliancesPanel.add(new MotTransparentLabel(L.NONE,JLabel.CENTER));
        }
        alliancesPanel.add(new JPanel());
        
        ComponentTable infoPanel = new ComponentTable("HV", 5, 1, 5, 3, 3);
        infoPanel.add(warsPanel);
        infoPanel.add(tradersPanel);
        infoPanel.add(researchersPanel);
        infoPanel.add(alliancesPanel);
        infoPanel.add(new JPanel(),"HVPS");
        PanelUtils.setTheBloodySize(infoPanel, new Size(150, 250));
        
        technologyPicturePanel = new TechnologyPicturePanel(owner, false, false, true, false, null);
        selectTechnologyButtonPanel = new MotButtonPanel();
        selectTechnologyButton = new SelectTechnologyButton();
        
    	ComponentTable selectTechnologyPanel = new ComponentTable("HV",2,1,0,1,1);
        selectTechnologyPanel.add(technologyPicturePanel,"HVSP");
        selectTechnologyPanel.add(selectTechnologyButtonPanel,"HV");
        
        JScrollPane listScrollPane = list.getScrollPane();
        PanelUtils.setTheBloodySize(listScrollPane, new Size(200, 200));

        ComponentTable main = new ComponentTable("HV,HV,HVSP", 1, 3, 5, 3, 3);
        main.add(new MotBorder(infoPanel, L.TREATY_DETAILS));
        main.add(listScrollPane);
        main.add(selectTechnologyPanel);
        
        this.getContentPane().add(main, BorderLayout.CENTER);

        JPanel buttonPanel = new JPanel();
        ButtonGroup bg = new ButtonGroup();
        buttonPanel.add(PanelUtils.createButton(L.CLOSE, this));
        buttonPanel.add(PanelUtils.createToggleButton(L.ALL, this, bg, true));
        buttonPanel.add(PanelUtils.createToggleButton(L.UNDISCOVERED_BY_YOU, this, bg));
        buttonPanel.add(PanelUtils.createToggleButton(L.UNDISCOVERED_BY_THEM, this, bg));
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        
        ProxyStore.getInstance().fetchTechnologies(player, new CompletedCallback() {
			public void complete(Object result) throws ApplicationError {
				data = ((TechnologyListCR)result).getResultAsList();
				list.setContent((TechnologyListCR)result);
			}
		}, null);
    }

    public void actionPerformed(ActionEvent e) {
        try {
            doActionPerformed(e);
        } catch (ApplicationError ex) {
            Logger.logError("Error "+ex, ex);
        //} catch (ValidationError ex) {
        //    JOptionPane.showMessageDialog(this, ex.getMessage(), "Invalid Comment", JOptionPane.OK_OPTION);
        }
    }

    public void doActionPerformed(ActionEvent e) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("EspionageReportWindow.actionPerformed() "+e.getActionCommand());
        }
        if (Comp.same(e.getActionCommand(), L.CLOSE)) {
            this.hide();
        } else if (Comp.same(e.getActionCommand(), L.ALL)) {
        	if (data != null) {        		
        		list.setContent(data.getAllAsArray(false), null);
        	}
        } else if (Comp.same(e.getActionCommand(), L.UNDISCOVERED_BY_YOU)) {
        	if (data != null) {        		
        		TechnologyList l = new GeneralTechnologyList(data);
        		l.remove(owner.getTechnologies());
        		list.setContent(l.getAllAsArray(), null);
        	}
        } else if (Comp.same(e.getActionCommand(), L.UNDISCOVERED_BY_THEM)) {
        	if (data != null) {        		
        		TechnologyList l = new GeneralTechnologyList(owner.getTechnologies());
        		l.remove(data);
        		list.setContent(l.getAllAsArray(), null);
        	}
        }
    }

	public void rightClick(Technology technology) {	}

	/**
	 * User selects a technology in list to espionage
	 */
	public void leftClick(Technology technology) {	
		try {
			list.setSelection(technology);
			technologyPicturePanel.update(technology);
			selectedTechnology = technology;
			selectTechnologyButtonPanel.removeAll();
			if (!owner.hasDiscovered(technology)) {
				selectTechnologyButtonPanel.add(selectTechnologyButton);
			}
			selectTechnologyButtonPanel.validate();
		} catch (ApplicationError e) {
			MainWindow.getExestingInstance().reportApplicationError(e);
		}
	}
	
}