/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.empire;

import java.awt.Color;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Comparator;

import javax.swing.JLabel;
import javax.swing.JScrollPane;

import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.RelationDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.buttons.MotAction;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotButtonGeneric;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.BackgroundImagePanelFinal;
import mot.client.swing.panels.RacePicturePanelWithFlag;
import mot.client.swing.window.common.actions.UpdatablePlayerContainer;
import mot.client.swing.window.empire.actions.CloseRelationsWindow;
import mot.client.swing.window.empire.actions.EspionagePlayerButton;
import mot.client.swing.window.empire.actions.OpenEspionageReportWindowButton;
import mot.client.swing.window.empire.actions.SendButton;
import mot.client.swing.window.empire.actions.SendWithCutomMessageButton;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.DatedMessage;
import mot.common.bean.DatedMessageBean;
import mot.common.bean.DiplomaticHistory;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;
import mot.common.vo.RelationResearchAgreementVO;
import mot.common.vo.RelationTradeAgreementVO;
import mot.common.vo.StrategistEventVO;
import mot.common.vo.TechnologyListVO;

public class RaceRelationPanel extends ComponentTable implements UpdatablePlayerContainer {

	private PlayerDelegate player;
    private RelationsWindow parent;
    
    private RaceRelationBar relBar;
    
    private DiplomaticProposal proposal = new DiplomaticProposal();
    private MotLabel proposalText;

    private ComponentTable historyTable;
    
    private EspionagePanel spyTab;
    private RecruitePanel recruiteTab;
    private TreatyPanel treatyPanel;
    private TradePanel tradePanel;
	private MotButtonGeneric yesButton;
	private MotButtonGeneric noButton;
	private MotButtonPanel buttonPanel;
	private MotButtonPanel mainButtonPanel;
	
	private SendButton sendButton;
	private SendWithCutomMessageButton sendButtonWithCustomMessage;
	
    public RaceRelationPanel(PlayerDelegate player, RelationsWindow parent) throws ApplicationError, ValidationError {
    	super("HV", 4, 1, 3, 0, 0);
    	
        this.player = player;
        this.parent = parent;
        
        boolean high = MainWindow.isScreen1000High();
        proposalText = new MotLabel(null, high ? 10 : 6, JLabel.CENTER, false);
        proposalText.setHorizontalTextPosition(JLabel.CENTER);
        proposalText.setVerticalTextPosition(JLabel.CENTER);
        
		UserPlayerDelegate owner = MainWindow.getOwner();

		mainButtonPanel = new MotButtonPanel();

        if (owner.hasDiplomaticRelationsWith(player)) {
	        Color highlightBackgroundColour = PropertyManager.getMandatoryAsColor("window.relations.color.background.highlight");

	        treatyPanel = new TreatyPanel(parent, player, this, proposal, highlightBackgroundColour);
	        tradePanel = new TradePanel(parent, this, proposal, highlightBackgroundColour);
	        
	        ComponentTable negotiationTable = new ComponentTable("HV", 3, 1, 3, 0, 0);
	        yesButton = new MotButtonGeneric(L.YES, null);
	        noButton = new MotButtonGeneric(L.NO, null);
	        sendButton = new SendButton(this);
	    	sendButtonWithCustomMessage = new SendWithCutomMessageButton(parent, this);
	        buttonPanel = new MotButtonPanel();
	        negotiationTable.add(proposalText);
	        negotiationTable.add(buttonPanel);
	        negotiationTable.add(new PadPanel(), "HVSP");
	        JScrollPane negotiationTableSP = new JScrollPane(negotiationTable, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	        
	        PanelUtils.setTheBloodySize(buttonPanel, new Size(400,40));
	        
	        historyTable = new ComponentTable("HV,HVP", 1,2, 2,4,0);
	        JScrollPane historySP = new JScrollPane(historyTable, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	        historySP.setBackground(Color.BLACK);

	        RacePicturePanelWithFlag racePicturePanel = new RacePicturePanelWithFlag(player);
	        
	        //mainTable.setGridWidths("0.3,0.7");
	        ComponentTable left = new ComponentTable("HV", 2, 1, 0, 0, 0);
	        ComponentTable right = new ComponentTable("HV", 3, 1, 0, 0, 0);
	        PanelUtils.setTheBloodySize(racePicturePanel, new Size(400,250));
	        //PanelUtils.setTheBloodySize(negotiationTable, new Size(450,250));
	        left.add(racePicturePanel);
	        left.add(historySP);
	        right.add(new PadPanel(treatyPanel, 2, highlightBackgroundColour));
	        right.add(new PadPanel(tradePanel, 2, highlightBackgroundColour));
	        right.add(negotiationTableSP,"HVSP");
	        
	        boolean wide = MainWindow.isScreen1600Wide();
			ComponentTable mainTable = new ComponentTable("HV,HVSP,HV", 1, wide ? 3 : 2, 0, 0, 0);
	        mainTable.add(new MotBorder(left, "Player Information"));
	        mainTable.add(right);
	        if (wide)  {
	        	BackgroundImagePanelFinal padPanel = new BackgroundImagePanelFinal(ImageConfigFactory.getImageForConfig("GameBannerV"));
	        	PanelUtils.setTheBloodySize(padPanel, new Size(200,200));
				mainTable.add(padPanel);
	        }

	        ComponentTable spyTable = new ComponentTable("HVSP,HV", 1, 2, 0, 0, 0);
	        spyTab = new EspionagePanel(parent, parent);
	        recruiteTab = new RecruitePanel(parent, parent);
	        spyTable.add(new MotBorder(spyTab, L.ESPIONAGE));	
	        spyTable.add(new MotBorder(recruiteTab, L.RECRUIT_SPIES));	
	        
	        RelationDelegate rel = owner.getRelation(player);
	        if (rel != null) {
	        	relBar = new RaceRelationBar(rel.getValue());
	        } else {
	        	relBar = new RaceRelationBar(0);
	        }
	        
	        add(mainTable,"HVSP");
	        add(spyTable);	
	        add(new PadPanel(relBar));
	        add(mainButtonPanel);
	        
	        negotiationTable.setOpaque(true);
	        negotiationTable.setBackground(highlightBackgroundColour);
	        proposalText.setOpaque(true);
	        proposalText.setBackground(highlightBackgroundColour);

        } else {
        	add(new NoRelationsPanel(), "HVSP");
	        add(mainButtonPanel);
        }
        
        updatePlayerContainer(true);
        updateButtonPanel();
    }

    public void updateButtonPanel() throws ApplicationError {
    	if (mainButtonPanel != null) {
	    	mainButtonPanel.removeAll();
	    	mainButtonPanel.add(new CloseRelationsWindow(parent));
	    	UserPlayerDelegate owner = MainWindow.getOwner();
	    	if (owner.canViewDetails(player)) {
	    		mainButtonPanel.add(new OpenEspionageReportWindowButton(this));
	    	} else {
	    		mainButtonPanel.add(new EspionagePlayerButton(this));
	    	}
	    	validate();
	    	repaint();
    	}
    }
    
    public void updatePlayerContainer(boolean updateProposalText) throws ApplicationError, ValidationError {
    	if (relBar != null) {
    		UserPlayerDelegate owner = MainWindow.getOwner();
    		StrategistEventVO [] strategistEvents = owner.getStrategistEvents(player);
    		//DiplomaticHistoryVO diplomaticHistory = owner.getDiplomaticHistory(player);
    		RelationDelegate relation = DeligateCacheManager.getRelation(owner.getName(), player.getName());
			DiplomaticHistory diplomaticHistory = relation.getDiplomaticHistory(); 
    		DatedMessage [] diplomaticHistoryMessages = diplomaticHistory == null ? null : diplomaticHistory.getOneLineSummaries(owner.getName(), player.getName());
	        
	        ArrayList<DatedMessage> content = new ArrayList();
	        if (strategistEvents != null) {
		        for (int i = 0; i < strategistEvents.length; i++) {				
		        	content.add(strategistEvents[i]);
				}
	        }
	        if (diplomaticHistoryMessages != null) {
		        for (int i = 0; i < diplomaticHistoryMessages.length; i++) {				
		        	content.add(diplomaticHistoryMessages[i]);
				}
	        }
	        RelationTradeAgreementVO tradeAgreement = relation.getTradeAgreement();
	        if (tradeAgreement != null) {
	        	content.add(new DatedMessageBean(DeligateCacheManager.getDate()+0.91, "Trade Income generated "+tradeAgreement.getMoney()));
	        }
	        RelationResearchAgreementVO researchAgreement = relation.getResearchAgreement();
	        if (researchAgreement != null) {
	        	TechnologyListVO voidedMissedTechs1 = researchAgreement.getVoidedMissedTechnologies(owner.getName());
	        	TechnologyListVO voidedMissedTechs2 = researchAgreement.getVoidedMissedTechnologies(player.getName());
	        	if (voidedMissedTechs1 != null) content.add(new DatedMessageBean(researchAgreement.getDate()+0.1, owner.getName()+" can research "+voidedMissedTechs1.getFullNamesAsString()));
	        	if (voidedMissedTechs2 != null) content.add(new DatedMessageBean(researchAgreement.getDate()+0.2, player.getName()+" can research "+voidedMissedTechs2.getFullNamesAsString()));
				if (researchAgreement != null) content.add(new DatedMessageBean(DeligateCacheManager.getDate()+0.92, "Research generated "+researchAgreement.getResearch()));
	        }
	        // Java prior to 1.8 does not have sort method on List
	        content = Comp.sort(content, new Comparator<DatedMessage>() {
				public int compare(DatedMessage o1, DatedMessage o2) {
					double d1 = o1.getDate();
					double d2 = o2.getDate();
					double r = (d2 - d1);
					if (r < 0.0) return -1;
					else if (r > 0.0) return 1;
					else return 0;
				}
			});

	        historyTable.removeAll();

	        if (content.size() > 0) {
	        	historyTable.setQtyRows(content.size()+1);
		        for (DatedMessage datedMessage : content) {
		        	historyTable.add(new MotLabel(""+(int)datedMessage.getDate(), JLabel.LEFT, JLabel.TOP));
	        		historyTable.add(new MotLabel(datedMessage.getMessage()));
				}
	        	historyTable.add(new MotLabel(""));
	    		historyTable.add(new MotLabel(""));
	        }
	        
	        buttonPanel.removeAll();
	        
	        if (updateProposalText && proposal.containsProposal()) {
	        	updateProposalText();
	        }
	        
	        boolean hasYesOrNo = false;
	        if (yesButton.hasAction()) {
	        	buttonPanel.add(yesButton);
	        	hasYesOrNo = true;
	        }
	        if (noButton.hasAction()) {
	        	buttonPanel.add(noButton);
	        	hasYesOrNo = true;
	        }
	        if (!hasYesOrNo && proposal != null && !proposal.isExecuted() && proposal.containsProposal()) {
	        	buttonPanel.add(sendButton);
	        	buttonPanel.add(sendButtonWithCustomMessage);
	        }
	        
	        spyTab.update(player);
	        recruiteTab.update(player);
	        treatyPanel.update(player);
	        tradePanel.update(player);
	        
	        RelationDelegate rel = owner.getRelation(player);
	        if (rel != null) {
	        	relBar.setValue(rel.getValueOtherPlayer(owner.getName()));
	        } else {
	        	relBar.setValue(0);
	        }
	        
	        validate();
	        repaint();
    	}
    }
    
//    public void clearConversationStatements() {
//    	statements.clearConversationStatements();
//    	validate();
//    	repaint();
//    }
//    
    public Insets getInsets() {
    	return new Insets(5,5,5,5);
    }


	public PlayerDelegate getSelectedPlayer() {
		return player;
	}

	public void addDiplomaticMsgToProposalText(String msg, String colour) throws ApplicationError, ValidationError {
		String oldMsg = proposalText.getText();
		if (oldMsg == null) {
			proposalText.setText(msg);
			return;
		}
		int i = oldMsg.indexOf("</html>");
		if (i < 2) {
			proposalText.setText(msg);
			return;
		}
		if (colour == null) colour="red";
		String newMsg = oldMsg.substring(0, i-1) + "<br/><br/><font color=\""+colour+"\">" + msg + "</font></html>";
		proposalText.setText(newMsg);
		updatePlayerContainer(false);
	}

	public void setDiplomaticProposalText(DiplomaticProposal proposal, String playerName, String text) throws ApplicationError {
		this.proposal = proposal;
		proposalText.setText(text);
	}
	
	public void setDiplomaticProposalText(DiplomaticProposal proposal, String playerName, MotAction yesAction, MotAction noAction) throws ApplicationError, ValidationError {
		this.proposal = proposal;
		if (proposal == null) {
			proposalText.setText(null);
			return;
		}
		int width = Comp.max(getWidth()/12, 50);
		String html = proposal.getProposalTextHTML(playerName);
		if (yesButton != null) yesButton.setAction(yesAction);
		if (noButton != null) noButton.setAction(noAction);
		proposalText.setText(html);
		updatePlayerContainer(false);
	}

	public void updateProposalText() throws ApplicationError {
		String html = proposal.getProposalTextHTML(player.getName());
//		int width = Comp.max(getWidth()/12, 50);
//		String html = MessagePanel.toHtml(msg, width, false);
		proposalText.setText(html);
	}

	

	public SendButton getSendButton() {
		return sendButton;
	}

	public MotButtonGeneric getNoButton() {
		return noButton;
	}

	public RecruitePanel getRecruiteTab() {
		return recruiteTab;
	}

	public DiplomaticProposal getProposal() {
		return proposal;
	}

	public DiplomaticProposal getProposalNotNull() {
		if (proposal == null) proposal = new DiplomaticProposal();
		return proposal;
	}

	public RelationsWindow getParentWindow() {
		return parent;
	}

	public void updateEspionageTarget() throws ApplicationError {
		spyTab.update(player);
	}

}