/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.empire;

import java.awt.BorderLayout;
import java.util.HashMap;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.swing.buttons.MotAction;
import mot.client.swing.icons.RaceIconFactory;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.common.actions.UpdatablePlayerContainer;
import mot.common.bean.DiplomaticProposal;
import mot.common.event.DiplomaticResponseEvent;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;

public class RelationsWindow extends MotInternalWindow implements UpdatablePlayerContainer {

    private JTabbedPane tabs;
    private HashMap indexToNameMap = new HashMap();
    private HashMap nameToTabMap = new HashMap();

    private RelationsWindow(MotInternalWindow parent, UserPlayerDelegate owner) throws ApplicationError, ValidationError {
        super(owner, parent, L.DIPLOMACY_WINDOW, null);
        
        init();
        show();
        
        CompletedCallback fetchComplete = new CompletedCallback() {
			public void complete(Object result) throws ApplicationError, ValidationError {
				updatePlayerContainer(true);
			}
		};
        
        ProxyStore.getInstance().fetchPlayerDetails(owner, fetchComplete, null);
        pollForEventOfType(DiplomaticResponseEvent.class, null);
    }

    private static RelationsWindow instance = null;
	private JPanel buttonPanel;
    
    public static boolean hasShowingInstance() {
    	if (instance != null && instance.isShowing()) {
    		return true;
    	} else {
    		return false;
    	}
    }

//    public static void open(MotInternalWindow parent, UserPlayerDeligate owner)  throws ApplicationError {
//    	RelationsWindow win = getInstanceNotNull(parent, owner);
//    	win.waitUntilClosed();
//    	instance = null;
//    }
    
    public static synchronized RelationsWindow getInstanceNotNull(MotInternalWindow parent, UserPlayerDelegate owner)  throws ApplicationError, ValidationError {
    	if (hasShowingInstance()) {
    		return instance; 
    	} else {
    		instance = newInstance(parent, owner);
    		return instance;
    	}
    }
    
    public static synchronized RelationsWindow newInstance(MotInternalWindow parent, UserPlayerDelegate owner) throws ApplicationError, ValidationError {
        instance = new RelationsWindow(parent, owner);
        return instance;
    }
    
	public static synchronized void close() {
		if (instance != null) {
			instance.hide();
		}
		instance = null;
	}

    private void init() throws ApplicationError, ValidationError {
        tabs = new JTabbedPane();
        PlayerDelegate[] players = DeligateCacheManager.getPlayers();
        int j = 0;
        for (int i=0; i<players.length; i++) {
            PlayerDelegate p = players[i];
            if (Comp.same(p.getName(), owner.getName())) {
                continue;
            }
            RaceRelationPanel panel = new RaceRelationPanel(p, this);
            indexToNameMap.put(new Integer(j), p);
            nameToTabMap.put(p.getName(), panel);
			tabs.addTab(p.getName(), RaceIconFactory.getIcon(p), panel);
            tabs.setForegroundAt(j, p.getColor().getColor());
            j++;
        }
        
        this.getContentPane().add(addTitleBar(tabs), BorderLayout.CENTER);
//
//        buttonPanel = new JPanel();
//        buttonPanel.add(new CloseRelationsWindow(this));
//        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
    }
    
    public void updatePlayerContainer(boolean updateProposalText) throws ApplicationError, ValidationError {
        int qty = tabs.getTabCount();
        for (int i=0; i<qty; i++) {
            RaceRelationPanel tab = (RaceRelationPanel)tabs.getComponentAt(i);
            tab.updatePlayerContainer(updateProposalText);
        }
    }
    
    public PlayerDelegate getSelectedPlayer() {
    	return (PlayerDelegate)indexToNameMap.get(new Integer(tabs.getSelectedIndex()));
    }

    public RaceRelationPanel selectedPlayer(String name) {
    	RaceRelationPanel panel = (RaceRelationPanel)nameToTabMap.get(name);
    	return panel;
    }

    public RaceRelationPanel selectedTab() {
    	PlayerDelegate selectedPlayer = getSelectedPlayer();
    	if (selectedPlayer != null) {
    		return (RaceRelationPanel)nameToTabMap.get(selectedPlayer.getName());
    	} else {    		
    		return null;
    	}
    }

	public void setDiplomaticMsg(String playerName, String msg, boolean updateContainer) throws ApplicationError, ValidationError {
		RaceRelationPanel playerTab = selectedPlayer(playerName);
		if (playerTab == null) return;
		tabs.setSelectedComponent(playerTab);
		playerTab.setDiplomaticProposalText(null, playerName, msg);
		if (updateContainer) playerTab.updatePlayerContainer(false);
	}
	
	public void addDiplomaticMsg(String playerName, String msg, String colour) throws ApplicationError, ValidationError {
		RaceRelationPanel playerTab = selectedPlayer(playerName);
		if (playerTab == null) return;
		tabs.setSelectedComponent(playerTab);
		playerTab.addDiplomaticMsgToProposalText(msg,colour);
	}
	
	public void askQuestion(String playerName, DiplomaticProposal proposal, MotAction yesAction, MotAction noAction) throws ApplicationError, ValidationError {
		RaceRelationPanel playerTab = selectedPlayer(playerName);
		if (playerTab == null) return;
		tabs.setSelectedComponent(playerTab);
		playerTab.setDiplomaticProposalText(proposal, playerName, yesAction, noAction);
	}

	public HashMap getNameToTabMap() {
		return nameToTabMap;
	}


}