/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc;

import java.awt.Dimension;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import javax.swing.JTextField;

import mot.client.swing.buttons.MotRolloverLabelButton;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.lists.FakeList;
import mot.common.bean.SavedGameFile;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;

public class FileSelectionPanel extends ComponentTable {

	private static final String DATE_SEPARATOR = " - ";
	private ActionListener selectFileListener;
	private ActionListener loadFileListener;
	private JTextField fileField;
	private FakeList fileList;
	
	/**
	 * @param fileSelectedListener Calls this action on double click - load saved game file
	 * @param selectFileListener Calls this action on single click - preview saved game file
	 */
	public FileSelectionPanel(ActionListener loadFileListener, ActionListener selectFileListener) {
		super("HV", 2, 1, 4, 0, 4);
		
		this.selectFileListener = selectFileListener;
		this.loadFileListener = loadFileListener;
		
		fileField = PanelUtils.createTextField(40, "", loadFileListener);
		fileList = new FakeList();
		//JScrollPane fileListScrollPane = new JScrollPane(fileList, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		add(PanelUtils.createNamedControlH(L.FILE_NAME+" : ", fileField));
		add(fileList.getScrollPane(), "HVSP");
		
		setMinimumSize(new Dimension(100,500));
	}
	
	public String getFileName() {
		return fileField.getText();
	}
	
	public void clearFileName() {
		fileField.setText(null);
	}
	
	public void update(SavedGameFile [] files) throws ApplicationError {
		if (files == null) {
			fileList.setContent(null, null);
			return;
		}
		MotRolloverLabelButton [] labels = new MotRolloverLabelButton[files.length];
        for (int i=files.length-1; i>=0; i--) {
    		JLabel fileLabel = new JLabel(files[i].getFileName()+"   ( " + files[i].getDate() + " )");
			labels[i] = new MotRolloverLabelButton(null, fileLabel, null, files[i]) {
    			protected void _doAction() throws ApplicationError, ValidationError {
    				selectFile((SavedGameFile)getSubject());
    			}
    		    protected void _doDoubleClickAction() throws ApplicationError, ValidationError {
    		    	openFile((SavedGameFile)getSubject());
    		    }
            };
        }
        fileList.setContent(labels, null);
	}
	
	private String currentSelection = null;

	public void selectFile(SavedGameFile file) {
		if (!Comp.same(currentSelection, file)) {
			currentSelection = file.getFileName();
			fileField.setText(file.getFileName());
			selectFileListener.actionPerformed(null);
		}
	}

	public void openFile(SavedGameFile file) {
		fileField.setText(file.getFileName());
		loadFileListener.actionPerformed(null);
	}

	public void setFocusOnFileField() {
		fileField.requestFocus();
	}
}

