/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc;

import java.awt.Dimension;
import java.awt.event.ActionListener;

import javax.swing.JPanel;

import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.other.SummaryStellarObjectDeligate;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.RemoteConnectionServerProxy;
import mot.client.proxy.ServerProxy;
import mot.client.swing.buttons.MotActionListener;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.combos.RaceCombo;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.NameControlH;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.BackgroundImagePanelFinal;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.common.MotMessageDialog;
import mot.client.swing.window.etc.actions.LoadGameButton;
import mot.common.bean.GalaxyDetails;
import mot.common.bean.GalaxySummary;
import mot.common.bean.PlayerSummary;
import mot.common.bean.SavedGameFile;
import mot.common.bean.SectorOwnerGrid;
import mot.common.bean.Size;
import mot.common.bean.StellarObjectSummary;
import mot.common.command_results.FileNamesCR;
import mot.common.factory.GraphicsFactory;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class LoadWindow extends MotInternalWindow {

	
	private FileSelectionPanel filePanel;
    private GalaxyWindowPreviewPanel galaxyViewPanel;
    private GalaxyPlayerSummaryPanel galaxyPlayerSummaryPanel;
    private MotButton loadButton, cancelButton;
	private CompletedCallback loadCallback;
	private RaceCombo raceComboBox;

	private boolean isMultiPlayer;

    ActionListener previewGalaxyAction = new MotActionListener() {
    	protected boolean playSound() { return false; }
    	protected void _doAction() throws ApplicationError, ValidationError {
    		String fileName = filePanel.getFileName();
    		if (fileName != null) {
    			CompletedCallback callback = new CompletedCallback() {
					public void complete(Object result) throws ApplicationError, ValidationError {
						GalaxySummary details = (GalaxySummary) result;
		    			if (details != null) {
		    				update(details);
		    			}
					}
				};
    			ProxyStore.getInstance().fetchGalaxyPreview(fileName, callback, null);
    		}
    	}
    };
    
    public LoadWindow(MotInternalWindow parent, UserPlayerDelegate owner, CompletedCallback loadCallback, boolean isMultiPlayer) throws ApplicationError, ValidationError {
        super(owner, parent, L.LOAD_WINDOW, null);
		this.loadCallback = loadCallback;
		this.isMultiPlayer = isMultiPlayer;
        init();
    	show();
        waitUntilClosed();
    }

	private MotInternalWindow getLoadWindow() {
		return this;
	}

	private void init() throws ApplicationError {
    	loadButton = new MotButton(L.LOAD) {
    		protected void _doAction() throws ApplicationError, ValidationError {
				String fileName = filePanel.getFileName();
				String playerName = raceComboBox.getRace();
				LoadGameButton.load(fileName, playerName, loadCallback, null, getLoadWindow(), isMultiPlayer);
    		}

    	};
    	cancelButton = new MotButton(L.CANCEL) {
    		protected void _doAction() throws ApplicationError, ValidationError {
    			filePanel.clearFileName();
    			getInternalWindow().hide();
    			if ( parent != null ) {
    				parent = new StartupWindow();
    			}
    		}
    	};
    	
    	filePanel = new FileSelectionPanel(loadButton, previewGalaxyAction);
    	galaxyViewPanel = new GalaxyWindowPreviewPanel();
    	galaxyPlayerSummaryPanel = new GalaxyPlayerSummaryPanel();
    	
    	filePanel.setPreferredSize(new Dimension(500, 600));
    	filePanel.setMinimumSize(new Dimension(300, 400));
    	PanelUtils.setTheBloodySize(galaxyViewPanel, new Size(500,400));
        raceComboBox = new RaceCombo(GraphicsFactory.getRaceGraphicsList(), null, false, new Size(25,25), true, false, null);

    	ComponentTable left = new ComponentTable("HV", isMultiPlayer ? 2 : 1,1, 0,4,0);
    	left.add(filePanel, "HVSP");
        if (isMultiPlayer) {
        	left.add(new NameControlH("Player Selection:  ", 3, raceComboBox));
        }

    	ComponentTable right = new ComponentTable("HV", 2,1, 0,4,0);
    	right.add(galaxyViewPanel);
    	right.add(galaxyPlayerSummaryPanel.getScrollPane(), "HVSP");

    	ComponentTable main = new ComponentTable("HV,HV,HVSP", 1,3, 4,0,4);
    	main.add(left);
    	main.add(right);
    	BackgroundImagePanelFinal padPanel = new BackgroundImagePanelFinal(ImageConfigFactory.getImageForConfig("GameBannerV"));
    	padPanel.setMinimumSizeToDisplayImage(new Size(200,200));
		main.add(padPanel);
    	
    	JPanel buttonPanel = new MotButtonPanel(loadButton, cancelButton);
    	
    	ComponentTable outer = new ComponentTable("HV", 2,1, 0,4,0);
    	outer.add(main, "HVSP");
    	outer.add(buttonPanel);

    	getContentPane().add(addTitleBar(outer), null);

    	filePanel.setFocusOnFileField();

    	updateFiles();

        if (isMultiPlayer) {
			ServerProxy sp = ProxyStore.getInstance();
			RemoteConnectionServerProxy multiPlayerProxy = sp.getRemoteConnectionServerProxy();
			multiPlayerProxy.startListingForConnections();
        }
	}

    public void updateFiles() {
    	try {
			CompletedCallback callback = new CompletedCallback() {
				public void complete(Object result) throws ApplicationError, ValidationError {
					SavedGameFile[] files = ((FileNamesCR) result).getResult();
					if (Logger.isDebug()) {
						Logger.logDebug("LoadWindow.updateFiles() files="+Comp.toString(files)+" "+System.currentTimeMillis()); 
					}
					filePanel.update(files);
				}
			};
    		ProxyStore.getInstance().getSavedGameFiles(callback, null);
		} catch (ApplicationError e) {
			Logger.logError("Update Failed", e);
		} catch (ValidationError e) {
			MotMessageDialog.showError(this, e);
		}
    }
    
    public void update(GalaxySummary details) throws ApplicationError {
    	SectorOwnerGrid sog = details.getSectorOwnerGrid();
    	StellarObjectSummary [] stellarObjects = details.getStellarObjects();
    	GalaxyDetails galaxyDetails = details.getGalaxyDetails();
    	StellarObjectDelegate [] sods = new StellarObjectDelegate[stellarObjects.length];
    	for (int i = 0; i < stellarObjects.length; i++) {
    		sods[i] = new SummaryStellarObjectDeligate(stellarObjects[i]);
		}
		PlayerSummary[] players = details.getPlayers();
		update(players, sog, sods, galaxyDetails);
    }
    
    public void update(PlayerSummary [] playerDetails, SectorOwnerGrid sog, StellarObjectDelegate [] stellarObjects, GalaxyDetails galaxyDetails) throws ApplicationError {
    	galaxyPlayerSummaryPanel.update(playerDetails);
    	galaxyViewPanel.update(playerDetails, sog, stellarObjects, galaxyDetails);
        raceComboBox.updateContent(playerDetails);
        raceComboBox.setSelectedIndex(-1);
    }
    
    public String getSelectedFile() {
    	return filePanel.getFileName();
    }
        
	public boolean processKeyEventsGlobally() {
		return false;
	}

}

