/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mot.client.bean.GameEditor;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.combos.AncientRaceCombo;
import mot.client.swing.combos.ColourCombo;
import mot.client.swing.combos.RaceCombo;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.MotTextField;
import mot.client.swing.common.NameControlV;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.labels.HtmlLabel;
import mot.client.swing.lists.SetupOptionsList;
import mot.client.swing.panels.BackgroundImagePanelFinal;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.etc.actions.ConnectButton;
import mot.client.swing.window.etc.actions.ConnectCancelButton;
import mot.client.swing.window.etc.actions.ConnectJoinButton;
import mot.client.swing.window.etc.actions.ConnectJoinWaitButton;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.bean.PlayerColor;
import mot.common.bean.Settings;
import mot.common.bean.Size;
import mot.common.event.StartGameEvent;
import mot.common.factory.GraphicsFactory;
import mot.common.lang.L;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;


public class MultiPlayerConnectWindow extends MotInternalWindow {

    private MotTextField serverIP;
    private MotLabel connectionStatus;
	private ConnectButton connectButton;
    
	private RaceCombo raceComboBox;
    private AncientRaceCombo ancientComboBox;
    private ColourCombo colourComboBox;

    private SetupOptionsList optionsList;
    
    private JLabel raceLabel = PanelUtils.createLabel("", 5, SwingConstants.CENTER); 
    private JLabel infoLabel = new HtmlLabel(JLabel.LEFT, 2, false);
    private JLabel alignmentDescriptionLabel = new HtmlLabel(JLabel.LEFT, 2, false);
    
    public boolean isCancel;
    
    private MultiPlayerConnectStatus status = new MultiPlayerConnectStatus();
	private ConnectCancelButton cancelButton;
	
	/**
	 * When TRUE - User must select a player, alignment, colour and settings to join a new game. 
	 * (used when starting a new game)
	 * When FALSE - User must select a only a player. The alignment, colour and settings will are already know for this player. 
	 * (used when loading an existing game)
	 */
	public boolean newGame;

    public MultiPlayerConnectWindow(MotInternalWindow parent, boolean newGame) throws ApplicationError, ValidationError {
        super(null, parent, L.PLAYER_CONNECT_WINDOW, null);
		this.newGame = newGame;
        setListData();
        init();
        changeRace();
        optionsList.setOptionsFromLast();
        isCancel = false;
        show();
        pollForEventOfType(StartGameEvent.class, null);
        waitUntilClosed();
    }

    public void setListData() throws ApplicationError {
    	ChangeListener changeOptionsListener = new ChangeListener() {
    	    public void stateChanged(ChangeEvent e) {
    	        try {
    	            connectButton.notifyDetailsChanged();
    	        } catch (Exception ex) {
    	            Logger.logError("Failed", ex);
    	        }
    	    }
		};
    	
        Color galaxyBackground = PropertyManager.getMandatoryAsColor("window.setup.color.background.galaxy");
        
        raceComboBox = new RaceCombo(GraphicsFactory.getRaceGraphicsList(), null, false, new Size(100,100), false, newGame, null);
        ancientComboBox = new AncientRaceCombo(GraphicsFactory.getRaceGraphicsList(), alignmentDescriptionLabel, new Size(100,100));
        colourComboBox = new ColourCombo(new Size(100,100));
        optionsList = new SetupOptionsList();
        optionsList.createOptions(changeOptionsListener);
        
        raceComboBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				changeRace();
	            connectButton.notifyDetailsChanged();
			}
		});

        colourComboBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
	            connectButton.notifyDetailsChanged();
			}
		});

        ancientComboBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
	            connectButton.notifyDetailsChanged();
			}
		});

        serverIP = new MotTextField(40,12,null);
        connectionStatus = new MotLabel("", 5, JLabel.LEFT, true);
        
        connectButton = newGame ? new ConnectJoinWaitButton(serverIP, this) : new ConnectJoinButton(serverIP, this);
        connectButton.setFont(new Font(connectButton.getFont().getName(), connectButton.getFont().getStyle(), connectButton.getFont().getSize()+8));
        cancelButton = new ConnectCancelButton(this);
        cancelButton.setFont(new Font(cancelButton.getFont().getName(), cancelButton.getFont().getStyle(), cancelButton.getFont().getSize()+8));
        
        Settings settings = Settings.getInstance();
        
        Integer raceIndex = settings.getGenericSettingInt("setupwin-mp-race");
        Integer ancientIndex = settings.getGenericSettingInt("setupwin-mp-ancient");
        Integer colourIndex = settings.getGenericSettingInt("setupwin-mp-colour");
        String ip = settings.getGenericSetting("setupwin-mp-ip");
        
        if (raceIndex != null) 		raceComboBox.setSelectedIndex(raceIndex.intValue());
        if (ancientIndex != null) 	ancientComboBox.setSelectedIndex(ancientIndex.intValue());
        if (colourIndex != null) 	colourComboBox.setSelectedIndex(colourIndex.intValue());
        if (ip != null) 			serverIP.setText(ip);

    }
    

    private void init() throws ApplicationError {
        setTitle(L.GAME_SETUP);
        
        Color raceBackground = PropertyManager.getMandatoryAsColor("window.setup.color.background.race");
        int comboTitleTextSize = isBigScreenConfig() ? 3 : 0;

        ComponentTable serverSettings = new ComponentTable("HV,HVSP", 1, 2, 10,15,0);
        serverSettings.add(new PadPanel(new NameControlV(L.SERVER_IP, 8, serverIP, raceBackground, 0, 0, 10), 5, raceBackground));
        serverSettings.add(new PadPanel(connectionStatus, 5, raceBackground));
        serverSettings.setBackground(raceBackground);
        
        ComponentTable playerDescription = new ComponentTable("HV", 2, 1, 0,0,5);
        JPanel raceDescription = PanelUtils.createNamedControlV(raceLabel, infoLabel, raceBackground);
        playerDescription.setBackground(raceBackground);
        playerDescription.add(raceDescription);
        playerDescription.add(alignmentDescriptionLabel);

        raceComboBox.setBackground(raceBackground);

        ComponentTable playerSettings;
        if (!newGame) {
        	playerSettings = new ComponentTable("HV,HVSP", 1, 2, 10,15,0);
        	playerSettings.add(new NameControlV(L.RACE_SELECTION, comboTitleTextSize, raceComboBox, raceBackground));
        	playerSettings.add(playerDescription);
        } else if (isBigScreenConfig()) {
        	playerSettings = new ComponentTable("HV,HV,HV,HVSP", 1, 4, 10,15,0);
        	playerSettings.add(new NameControlV(L.RACE_SELECTION, comboTitleTextSize, raceComboBox, raceBackground));
        	playerSettings.add(new NameControlV(L.COLOUR, comboTitleTextSize, colourComboBox, raceBackground));
        	playerSettings.add(new NameControlV(L.ANCIENT_RACE_ALIGNMENT, comboTitleTextSize, ancientComboBox, raceBackground));
        	playerSettings.add(playerDescription);
        } else {
        	playerSettings = new ComponentTable("HV,HV,HV", 1, 3, 10,15,0);
        	playerSettings.add(new NameControlV(L.RACE_SELECTION, comboTitleTextSize, raceComboBox, raceBackground));
        	playerSettings.add(new NameControlV(L.COLOUR, comboTitleTextSize, colourComboBox, raceBackground));
        	playerSettings.add(new NameControlV(L.ANCIENT_RACE_ALIGNMENT, comboTitleTextSize, ancientComboBox, raceBackground));
        }
        playerSettings.setBackground(raceBackground);
        playerSettings.setOpaque(true);
        
        MotButtonPanel buttonPanel = new MotButtonPanel();
        buttonPanel.add(connectButton);
        buttonPanel.add(new PadPanel(10));
		buttonPanel.add(cancelButton);
        buttonPanel.setBackground(raceBackground);
        
    	BackgroundImagePanelFinal padPanel = null;
    	if (GameEditor.hasEditor()){
    		padPanel = new BackgroundImagePanelFinal(ImageConfigFactory.getImageForConfig("GameBannerH"));
    	} else {
    		// This is free version - display advert as padding
    		padPanel = new BackgroundImagePanelFinal(ImageConfigFactory.getImageForConfig("GameBannerH"));
    	}
    	
    	
        ComponentTable left = new ComponentTable("HV", 4, 1, 0,5,10);
        left.setBackground(raceBackground);
        left.setOpaque(true);
		left.add(new MotBorder(serverSettings, "Server Settings", raceBackground));
		left.add(new MotBorder(playerSettings, "Player Settings", raceBackground));
        left.add(buttonPanel);
        left.add(padPanel,"HVSP");

        Color ancientBackground = PropertyManager.getMandatoryAsColor("window.setup.color.background.ancient");
		//alignmentPanel.setBackground(ancientBackground);
        alignmentDescriptionLabel.setBackground(ancientBackground);
        
        //alignmentPanel.setOpaque(true);
        alignmentDescriptionLabel.setOpaque(true);
        
        if (newGame) {
	        JPanel right = new PadPanel(optionsList, 15);
	
	        ComponentTable main = new ComponentTable("HV,HV", 1, 2, 0,0,0);
	        main.setGridWidths("0.65,0.35");
	        
	        main.add(new MotBorder(left));
	        main.add(right);
	        
	        getContentPane().add(addTitleBar(main), null);
        } else {
	        getContentPane().add(addTitleBar(left), null);
        }
    }
    
	private boolean isBigScreenConfig() throws ApplicationError {
		return MainWindow.getInstance().getSize().width > 1200;
	}

    public void registerNewPlayerOnServer(CompletedCallback callback) throws ApplicationError, ValidationError {
        ServerProxy serverProxy = ProxyStore.getInstance();
        SetupOptions userOptions = getOptions();
        String alignment = getAlignment();
        String raceName = raceComboBox.getRace();
        PlayerColor playerColor = colourComboBox.getPlayerColor();
        NewGameUserPlayerData userPlayerData = new NewGameUserPlayerData(raceName, userOptions, playerColor, alignment);
		serverProxy.newGameUserPlayer(userPlayerData, callback, null);
    }

    private void changeRace() {
        try {
            String race = raceComboBox.getRace();
            raceLabel.setText(race);
            optionsList.changeRace(race);
            String description = raceComboBox.getDescription();
            if (description != null) {
                infoLabel.setText("<html><body>"+description+"</body></html>");
            }
            connectButton.notifyDetailsChanged();
        } catch (ApplicationError err) {
            Logger.logError("Failed", err);
        }
    }
    
    public SetupOptions getOptions() throws ApplicationError {
    	ImageConfig config = (ImageConfig) raceComboBox.getSelectedItem();
        return optionsList.getOptions();
    }
    
    public String getRace() { return raceComboBox.getRace(); }
    public String getAlignment() { return ancientComboBox.getRace(); }
    public PlayerColor getPlayerColor() throws ApplicationError { return colourComboBox.getPlayerColor(); }

	public void saveSelection() throws ApplicationError {
        Settings settings = Settings.getInstance();

        settings.addGenericSetting("setupwin-mp-race", ""+raceComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-mp-ancient", ""+ancientComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-mp-colour", ""+colourComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-mp-ip", serverIP.getText());

        optionsList.saveSelection();
        
        settings.save();
	}

	public MultiPlayerConnectStatus getStatus() {
		return status;
	}

	public void setEditable(boolean status) {
		raceComboBox.setEditable(status);
		ancientComboBox.setEditable(status);
		colourComboBox.setEditable(status);
	}

	public boolean processKeyEventsGlobally() {
		return false;
	}

	public RaceCombo getRaceComboBox() {
		return raceComboBox;
	}

	public AncientRaceCombo getAncientComboBox() {
		return ancientComboBox;
	}

	public void updateAlignmentDescription(String alignment) {
		String description = ancientComboBox.getDescription(alignment);
		alignmentDescriptionLabel.setText( description );
	}


}