/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mot.client.alarms.AlarmSystem;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.RemoteConnectionServerProxy;
import mot.client.proxy.ServerProxy;
import mot.client.swing.combos.AncientRaceCombo;
import mot.client.swing.combos.ColourCombo;
import mot.client.swing.combos.GalaxySizeCombo;
import mot.client.swing.combos.LevelCombo;
import mot.client.swing.combos.QtyComputerPlayersCombo;
import mot.client.swing.combos.RaceCombo;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotCheckBoxSimple;
import mot.client.swing.common.NameControlV;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.labels.HtmlLabel;
import mot.client.swing.lists.SetupOptionsList;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.etc.actions.NewCustomGameButton;
import mot.client.swing.window.etc.actions.NewGameButton;
import mot.client.swing.window.etc.actions.NewGameCancelButton;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.GameConstants;
import mot.common.bean.NewGameDataComputerPlayerDetails;
import mot.common.bean.NewGameDataUserPlayerDetails;
import mot.common.bean.PlayerColor;
import mot.common.bean.Settings;
import mot.common.bean.Size;
import mot.common.event.NewGameUserPlayerEvent;
import mot.common.factory.GraphicsFactory;
import mot.common.lang.L;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;

public class NewGameWindow extends MotInternalWindow {

	private RaceCombo raceComboBox;
    private AncientRaceCombo ancientComboBox;
    private QtyComputerPlayersCombo qtyComputerPlayersComboBox;
    private GalaxySizeCombo sizeComboBox;
    private LevelCombo levelComboBox;
    private ColourCombo colourComboBox;
    private SetupOptionsList playerOptionsList;
	private SetupWindowOtherPlayerList otherPlayersList;

	private MotCheckBoxSimple checkboxWormhole = new MotCheckBoxSimple("Wormhole", null);
	private MotCheckBoxSimple checkboxSpaceKaan = new MotCheckBoxSimple("Space Kaan", null);

    private JLabel raceLabel = PanelUtils.createLabel("", 5, SwingConstants.CENTER); 
    private JLabel infoLabel = new HtmlLabel(JLabel.LEFT, 2, false);
    private JLabel costLabel = PanelUtils.createLabel(null);
    private JLabel sizeDescriptionLabel = new HtmlLabel(JLabel.LEFT, 0, false);
    private JLabel qtyPlayersDescriptionLabel = new HtmlLabel(JLabel.LEFT, 0, false);
    private JLabel alignmentDescriptionLabel = new HtmlLabel(JLabel.LEFT, 2, false);
    private JLabel levelDescriptionLabel = new HtmlLabel(JLabel.LEFT, 0, false);
    
    public static final int MAX_COST = 11;
     
    public boolean isCancel;
	private boolean isMultiPlayer;

    /**
     * Display a New Game Setup window. 
     * @param parent The parent window
     * @param url The url of the server (if this game is remote).
     */
    public NewGameWindow(MotInternalWindow parent, boolean isMultiPlayer) throws ApplicationError {
        super(null, parent, L.NEW_GAME_WINDOW, null);
		this.isMultiPlayer = isMultiPlayer;

        setListData();
        init();
        changeRace();
        playerOptionsList.setOptionsFromLast();
        setCost();
        isCancel = false;
        pollForEventOfType(NewGameUserPlayerEvent.class, null);
        AlarmSystem.getInstance().notifySetupWindowOpened();
        show();
        waitUntilClosed();
    }

    public void setListData() throws ApplicationError {
    	ChangeListener changeOptionsListener = new ChangeListener() {
    	    public void stateChanged(ChangeEvent e) {
    	        try {
    	            setCost();
    	        } catch (Exception ex) {
    	            Logger.logError("Failed", ex);
    	        }
    	    }
		};
    	
        Color galaxyBackground = PropertyManager.getMandatoryAsColor("window.setup.color.background.galaxy");
        
        raceComboBox = new RaceCombo(GraphicsFactory.getRaceGraphicsList(), null, false, new Size(100,100), false, true, null);
        ancientComboBox = new AncientRaceCombo(GraphicsFactory.getRaceGraphicsList(), alignmentDescriptionLabel, new Size(100,100));
        qtyComputerPlayersComboBox = new QtyComputerPlayersCombo(qtyPlayersDescriptionLabel);
        sizeComboBox = new GalaxySizeCombo(sizeDescriptionLabel);
        levelComboBox = new LevelCombo(this, levelDescriptionLabel, new Size(100,100), true, null);
        colourComboBox = new ColourCombo(new Size(100,100));
        playerOptionsList = new SetupOptionsList();
        playerOptionsList.createOptions(changeOptionsListener);
        otherPlayersList = new SetupWindowOtherPlayerList(this, galaxyBackground);    	
        
        raceComboBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				changeRace();
			}
		});

        qtyComputerPlayersComboBox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				try {
					changeQtyComputerPlayers();
				} catch (ApplicationError ex) {
		            Logger.logError("Failed", ex);
				}
			}
		});
        
        Settings settings = Settings.getInstance();
        
        Integer raceIndex = settings.getGenericSettingInt("setupwin-race");
        Integer ancientIndex = settings.getGenericSettingInt("setupwin-ancient");
        Integer qtyIndex = settings.getGenericSettingInt("setupwin-qty");
        Integer sizeIndex = settings.getGenericSettingInt("setupwin-size");
        Integer levelIndex = settings.getGenericSettingInt("setupwin-level");
        Integer colourIndex = settings.getGenericSettingInt("setupwin-colour");
        
        if (raceIndex != null) 		raceComboBox.setSelectedIndex(raceIndex.intValue());
        if (ancientIndex != null) 	ancientComboBox.setSelectedIndex(ancientIndex.intValue());
        if (qtyIndex != null) 		qtyComputerPlayersComboBox.setSelectedIndex(qtyIndex.intValue());
        if (sizeIndex != null) 		sizeComboBox.setSelectedIndex(sizeIndex.intValue());
        if (levelIndex != null) 	levelComboBox.setSelectedIndex(levelIndex.intValue());
        if (colourIndex != null) 	colourComboBox.setSelectedIndex(colourIndex.intValue());
        
        int qtyComputerPlayers = qtyComputerPlayersComboBox.getQtyPlayers();
		otherPlayersList.loadFromSettings(settings, qtyComputerPlayers);        
		
		levelComboBox.setDependentState();
		String wormholeTxt = settings.getGenericSetting("setupwin-wornhole-enabled");
		String spacekaanTxt = settings.getGenericSetting("setupwin-spacekaan-enabled");
		String isolationTxt = settings.getGenericSetting("setupwin-isolation-enabled");
		
		if (wormholeTxt != null) {
			setIsWormHoleEnabled(wormholeTxt.equalsIgnoreCase("true"));
		}
		if (spacekaanTxt != null) {
			setIsSpaceKaanEnabled(spacekaanTxt.equalsIgnoreCase("true"));
		}
    }
    

    private void init() throws ApplicationError {
        setTitle(L.GAME_SETUP);
        
        ComponentTable main = new ComponentTable("HV,HV", 1, 2, 0,0,0);
        main.setGridWidths("0.65,0.35");
        
        Color raceBackground = PropertyManager.getMandatoryAsColor("window.setup.color.background.race");
        Color galaxyBackground = PropertyManager.getMandatoryAsColor("window.setup.color.background.galaxy");
        
        int comboTitleTextSize = isBigScreenConfig() ? 3 : 0;
        JPanel galaxySizePanel = new NameControlV(L.GALAXY_SIZE, comboTitleTextSize, sizeComboBox, raceBackground);
        JPanel qtyPlayersPanel = new NameControlV(L.QTY_COMPUTER_PLAYERS, comboTitleTextSize, qtyComputerPlayersComboBox, raceBackground);
        JPanel difficultyPanel = new NameControlV(L.DIFFICULTY, comboTitleTextSize, levelComboBox, raceBackground);

        ComponentTable gameDescriptions = new ComponentTable("HV", 3, 1, 0, 0, 5);
        gameDescriptions.setBackground(raceBackground);
        gameDescriptions.add(sizeDescriptionLabel);
        gameDescriptions.add(qtyPlayersDescriptionLabel);
        gameDescriptions.add(levelDescriptionLabel);
        
        Font descriptionFont = new Font(sizeDescriptionLabel.getFont().getName(), Font.PLAIN, sizeDescriptionLabel.getFont().getSize()+3);
        sizeDescriptionLabel.setFont(descriptionFont);
        qtyPlayersDescriptionLabel.setFont(descriptionFont);
        levelDescriptionLabel.setFont(descriptionFont);
        
        ComponentTable galaxySettings;
        if (isBigScreenConfig()) {
        	galaxySettings = new ComponentTable("HV,HV,HV,HP", 1, 4, 10,15,0);
        	//galaxySettings.setGridWidths("25,25,25,25");
        	galaxySettings.setOpaque(true);
        	galaxySettings.setBackground(galaxyBackground);
        	galaxySettings.add(galaxySizePanel);
        	galaxySettings.add(qtyPlayersPanel);
        	galaxySettings.add(difficultyPanel);
        	galaxySettings.add(gameDescriptions);
        } else {
        	galaxySettings = new ComponentTable("HV,HV,HV", 1, 3, 10,5,0);
        	//galaxySettings.setGridWidths("25,25,25,25");
        	galaxySettings.setOpaque(true);
        	galaxySettings.setBackground(galaxyBackground);
        	galaxySettings.add(galaxySizePanel);
        	galaxySettings.add(qtyPlayersPanel);
        	galaxySettings.add(difficultyPanel);
        }
        
        raceComboBox.setBackground(raceBackground);

        ComponentTable playerDescription = new ComponentTable("HV", 2, 1, 0,0,5);
        JPanel raceDescription = PanelUtils.createNamedControlV(raceLabel, infoLabel, raceBackground);
        playerDescription.setBackground(raceBackground);
        playerDescription.add(raceDescription);
        playerDescription.add(alignmentDescriptionLabel);

        ComponentTable playerSettings;
        if (isBigScreenConfig()) {
        	playerSettings = new ComponentTable("HV,HV,HV,HVSP", 1, 4, 10,15,0);
        	playerSettings.add(new NameControlV(L.RACE_SELECTION, comboTitleTextSize, raceComboBox, raceBackground));
        	playerSettings.add(new NameControlV(L.COLOUR, comboTitleTextSize, colourComboBox, raceBackground));
        	playerSettings.add(new NameControlV(L.ANCIENT_RACE_ALIGNMENT, comboTitleTextSize, ancientComboBox, raceBackground));
        	playerSettings.add(playerDescription);
        } else {
        	playerSettings = new ComponentTable("HV,HV,HV", 1, 3, 10,15,0);
        	playerSettings.add(new NameControlV(L.RACE_SELECTION, comboTitleTextSize, raceComboBox, raceBackground));
        	playerSettings.add(new NameControlV(L.COLOUR, comboTitleTextSize, colourComboBox, raceBackground));
        	playerSettings.add(new NameControlV(L.ANCIENT_RACE_ALIGNMENT, comboTitleTextSize, ancientComboBox, raceBackground));
        }
        playerSettings.setBackground(raceBackground);
        playerSettings.setOpaque(true);
        
        ComponentTable left = new ComponentTable("HV", 3, 1, 0,5,10);
        left.setBackground(raceBackground);
        left.setOpaque(true);
		left.add(new MotBorder(galaxySettings, "Galaxy Settings", raceBackground));
		left.add(new MotBorder(playerSettings, "Player Settings", raceBackground));
		left.add(new MotBorder(otherPlayersList.getScrollPane(), "Other Players", raceBackground),"HVSP");

        Color ancientBackground = PropertyManager.getMandatoryAsColor("window.setup.color.background.ancient");
		//alignmentPanel.setBackground(ancientBackground);
        alignmentDescriptionLabel.setBackground(ancientBackground);
        
        //alignmentPanel.setOpaque(true);
        alignmentDescriptionLabel.setOpaque(true);
        
		galaxySizePanel.setBackground(galaxyBackground);
        sizeDescriptionLabel.setBackground(galaxyBackground);
        qtyPlayersPanel.setBackground(galaxyBackground);
        qtyPlayersDescriptionLabel.setBackground(galaxyBackground);
        otherPlayersList.setBackground(galaxyBackground);
        levelDescriptionLabel.setBackground(galaxyBackground);
        
        galaxySizePanel.setOpaque(true);
        sizeDescriptionLabel.setOpaque(true);
        qtyPlayersPanel.setOpaque(true);
        qtyPlayersDescriptionLabel.setOpaque(true);
        otherPlayersList.setOpaque(true);
        levelDescriptionLabel.setOpaque(true);

        ComponentTable gameOptionsList = new ComponentTable("HV", 3, 1, 0,0,0);
		gameOptionsList.add(checkboxWormhole);
		gameOptionsList.add(checkboxSpaceKaan);

		checkboxWormhole.setToolTipText("Select if this galaxy can have a worm hole. A worm hole is a stellar object that can be used to move around the galaxy.");
		checkboxSpaceKaan.setToolTipText("Select if this galaxy can a space Kaan. A Space Kaan is a space dragon that emerge from nebula after a battle wakes them. It will wonder the galaxy destroying crafts until killed.");
		
        ComponentTable optionsPanel = new ComponentTable("HV", 2, 1, 0,0,0);
        optionsPanel.add(new MotBorder(gameOptionsList, "Game Features"));
        optionsPanel.add(playerOptionsList,"HVSP");
        JPanel right = new PadPanel(optionsPanel, 15, 0);

        main.add(new MotBorder(left));
        main.add(right);
        
        //boolean canHaveMultipleUsers = ProxyStore.canHaveMultipleUsers();
        
        ComponentTable southPanel = new ComponentTable("H,L", 1,2,0,0,0);
        MotButtonPanel buttonPanel = new MotButtonPanel();
        buttonPanel.add(new NewGameButton(this));
        buttonPanel.add(new NewCustomGameButton(this));
        buttonPanel.add(new NewGameCancelButton(this));
        //if (canHaveMultipleUsers) buttonPanel.add(new ServerSocketThreadStatusButton(this));
        southPanel.add(buttonPanel,"HP");
        southPanel.add(costLabel,"L");
        

        getContentPane().add(addTitleBar(main), null);
        getContentPane().add(southPanel, BorderLayout.SOUTH);

        if (isMultiPlayer) {
			ServerProxy sp = ProxyStore.getInstance();
			RemoteConnectionServerProxy multiPlayerProxy = sp.getRemoteConnectionServerProxy();
			multiPlayerProxy.startListingForConnections();
        }

    }
    
    private void changeQtyComputerPlayers() throws ApplicationError {
    	int qtyComputerPlayers = qtyComputerPlayersComboBox.getQtyPlayers();
    	otherPlayersList.changeQtyComputerPlayers(qtyComputerPlayers);
    }

	private boolean isBigScreenConfig() throws ApplicationError {
		return MainWindow.getInstance().getSize().width > 1200;
	}

    public void setCost() throws ApplicationError {
    	SetupOptions options = getOptions();
        int cost = options.getCost();
        costLabel.setText(""+cost+ L._OF_+MAX_COST);
    }
    

    private void changeRace() {
        try {
            String race = raceComboBox.getRace();
            raceLabel.setText(race);
            playerOptionsList.changeRace(race);
            String description = raceComboBox.getDescription();
            if (description != null) {
            	if (description.startsWith("<html>")) {
            		infoLabel.setText(description);
            	} else {
            		infoLabel.setText("<html><body>"+description+"</body></html>");
            	}
            }
        } catch (ApplicationError err) {
            Logger.logError("Failed", err);
        }
    }
    
    public SetupOptions getOptions() throws ApplicationError {
    	ImageConfig config = (ImageConfig) raceComboBox.getSelectedItem();
        return playerOptionsList.getOptions();
    }
    
    public String getRace() { return raceComboBox.getRace(); }
    public String getAlignment() { return ancientComboBox.getRace(); }
    public int getQtyComputerPlayers() { return qtyComputerPlayersComboBox.getQtyPlayers(); }
    public int getQtyPlanets() { return sizeComboBox.getQtyPlanets(); }
    public int getQtyNebula() { return sizeComboBox.getQtyNebula(); }
    public Size getGalaxySize() { return sizeComboBox.getGalaxySize(); }
    public Double getLevel() { return levelComboBox.getLevel(); }
    public boolean isHardLevel() { return levelComboBox.getLevel() >= GameConstants.LEVEL_HARD_BONUS; }
    public boolean hasTutorial() { return levelComboBox.hasTutorial(); }
    public PlayerColor getPlayerColor() throws ApplicationError { return colourComboBox.getPlayerColor(); }
	public NewGameDataComputerPlayerDetails[] getComputerPlayerDetails() { return otherPlayersList.getComputerPlayerDetails(); }
	public NewGameDataUserPlayerDetails[] getUserPlayerDetails() { return otherPlayersList.getUserPlayerDetails(); }
	public boolean isWormHoleEnabled() {  return checkboxWormhole.isSelected(); /*(getLevel() >= 2.0)*/ }
	public boolean isSpaceKaanEnabled() { return checkboxSpaceKaan.isSelected(); /*(getLevel() > 0.5)*/ }

	public void setIsWormHoleEnabled(boolean v) { checkboxWormhole.setSelected(v); }
	public void setIsSpaceKaanEnabled(boolean v) { checkboxSpaceKaan.setSelected(v); }

	public void saveSelection() throws ApplicationError {
        Settings settings = Settings.getInstance();

        settings.addGenericSetting("setupwin-race", ""+raceComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-ancient", ""+ancientComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-qty", ""+qtyComputerPlayersComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-size", ""+sizeComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-level", ""+levelComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-colour", ""+colourComboBox.getSelectedIndex());
        settings.addGenericSetting("setupwin-wornhole-enabled", isWormHoleEnabled() ? "true" : "false");
        settings.addGenericSetting("setupwin-spacekaan-enabled", isSpaceKaanEnabled() ? "true" : "false");
        otherPlayersList.saveToSettings(settings);        

        playerOptionsList.saveSelection();
        
        settings.save();
	}

	public SetupWindowOtherPlayerList getOtherPlayersList() {
		return otherPlayersList;
	}

}