/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc;

import java.awt.BorderLayout;
import java.awt.Color;
import java.io.File;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JRadioButton;

import mot.client.actions.MotTask;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.ProxyStore;
import mot.client.sounds.JukeBox;
import mot.client.sounds.Sounds;
import mot.client.swing.buttons.MotAction;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotFlowPanel;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.MotTextField;
import mot.client.swing.window.common.MotConfirmationDialog;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.common.WindowLocator;
import mot.client.swing.window.etc.actions.CloseOptionsWindowButton;
import mot.client.swing.window.main.MainWindow;
import mot.client.validators.FileValidator;
import mot.client.validators.IntegerValidator;
import mot.common.bean.PlayerCharacterBasic;
import mot.common.bean.Settings;
import mot.common.lang.L;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class OptionsWindow extends MotInternalWindow {

	private MotTextField percentTerraformTextField;
    private MotTextField percentDevelopmentTextField;
    private MotTextField percentResearchTextField;
    private MotTextField percentProductionTextField;

    private JCheckBox moveCraftWarning;

    private JRadioButton fullScreenOption;
    private JRadioButton maximisedOption;
    private JRadioButton windowOption;
    private MotTextField screenWidth;
    private MotTextField screenHeight;
    
    private MotTextField effectsVolume;
    private MotTextField musicVolume;
    private MotTextField musicFile;
    
    private MotTextField autoSaveInterval;
    
    private static final String OPTIONS_WINDOW = "Options Window";
    private static final String [] VALID_EXTENSIONS = {"midi", "mid", "wav"};

    public OptionsWindow(MotInternalWindow parent, UserPlayerDelegate owner) throws ApplicationError {
        super(owner, parent, OPTIONS_WINDOW, null);

        WindowLocator.setDefault(this);
        
        init();
        setup();
    }

    private void init() throws ApplicationError {

    	int fontSize = 0;
    	if (MainWindow.getWidth() > 1600) fontSize = 6;
    	else if (MainWindow.getWidth() > 1170) fontSize = 2;
    			
    	percentTerraformTextField = new MotTextField(5, fontSize, new IntegerValidator("Terraform Percent", 0, 100));
    	percentDevelopmentTextField = new MotTextField(5, fontSize, new IntegerValidator("Development Percent", 0, 100));
    	percentResearchTextField = new MotTextField(5, fontSize, new IntegerValidator("Research Percent", 0, 100));
    	percentProductionTextField = new MotTextField(5, fontSize, new IntegerValidator("Production Percent", 0, 100));
    	
    	fullScreenOption = new JRadioButton();
    	maximisedOption = new JRadioButton();
    	windowOption = new JRadioButton();
    	ButtonGroup bg = new ButtonGroup();
    	bg.add(fullScreenOption);
    	bg.add(maximisedOption);
    	bg.add(windowOption);
    	screenWidth = new MotTextField(4, fontSize, new IntegerValidator("Screen Width", 1024, 2400));
    	screenHeight = new MotTextField(4, fontSize, new IntegerValidator("Screen Height", 768, 1900));
    	
    	effectsVolume = new MotTextField(3, fontSize, new IntegerValidator("Effects Volume", 0, 100));
    	musicVolume = new MotTextField(3, fontSize, new IntegerValidator("Music Volume", 0, 100));
    	musicFile = new MotTextField(20, fontSize, new FileValidator("Music File", VALID_EXTENSIONS));
    	
    	autoSaveInterval = new MotTextField(2, fontSize, new IntegerValidator("Autosave Interval", 0, 99));
    	
        percentTerraformTextField.setToolTipText("The percentage of a planets production used to terraform the planet. This will value will be added to the ecology percent when the planet can perform terraforming and auto is on.");
        percentDevelopmentTextField.setToolTipText("This is the percentage of a planets production that will be used to build factories when a planet can have more factories built and auto is on.");
        
        moveCraftWarning = new JCheckBox();
        moveCraftWarning.setToolTipText("When the player presses End Turn - Warn play if there are unmoved crafts");
        
        ComponentTable table = new ComponentTable("TR,TR,TRP", 12, 3, 10, 5, 10);
        table.add(new MotLabel("Planet Percent Terraform", fontSize, JLabel.LEFT, false));
        table.add(new MotFlowPanel(percentTerraformTextField, new MotLabel("%", fontSize, JLabel.LEFT, false)));
        table.add(new MotLabel("Percentage of planet's output allocated when Auto-Terraform is ON", fontSize, JLabel.LEFT, false));
        
        table.add(new MotLabel("Planet Percent Development", fontSize, JLabel.LEFT, false));
        table.add(new MotFlowPanel(percentDevelopmentTextField, new MotLabel("%", fontSize, JLabel.LEFT, false)));
        table.add(new MotLabel("Percentage of planet's output allocated when Auto-Development is ON", fontSize, JLabel.LEFT, false));
        
        table.add(new MotLabel("Move Craft Warning", fontSize, JLabel.LEFT, false));
        table.add(moveCraftWarning);
        table.add(new MotLabel("When the player presses End Turn - Warn play if there are unmoved crafts", fontSize, JLabel.LEFT, false));
        
        table.add(new MotLabel("Full Screen Mode", fontSize, JLabel.LEFT, false));
        table.add(fullScreenOption);
        table.add(new MotLabel("Restart game for Window Mode to take effect", fontSize, null, null, JLabel.LEFT, false, Color.RED, null));
        table.add(new MotLabel("Maximised Window", fontSize, JLabel.LEFT, false));
        table.add(maximisedOption);
        table.add(new MotLabel("!! NOT ALL MODES WILL WORK ON ALL SYSTEMS !!", fontSize, null, null, JLabel.LEFT, false, Color.RED, null));
        table.add(new MotLabel("Normal Window", fontSize, JLabel.LEFT, false));
        table.add(windowOption);
        table.add(new MotLabel("If game fails to restart delete settings.xml in the data folder", fontSize, null, null, JLabel.LEFT, false, Color.RED, null));
        
        table.add(new MotLabel("Screen Width", fontSize, JLabel.LEFT, false));
        table.add(new MotFlowPanel(screenWidth, new MotLabel("px", fontSize, JLabel.LEFT, false)));
        table.add(new MotLabel("Only used in 'Normal' Window Mode", fontSize, JLabel.LEFT, false));

        table.add(new MotLabel("Screen Height", fontSize, JLabel.LEFT, false));
        table.add(new MotFlowPanel(screenHeight, new MotLabel("px", fontSize, JLabel.LEFT, false)));
        table.add(new MotLabel("Only used in 'Normal' Window Mode", fontSize, JLabel.LEFT, false));

        table.add(new MotLabel("Sounds Effects Volums", fontSize, JLabel.LEFT, false));
        table.add(new MotFlowPanel(effectsVolume, new MotLabel("%", fontSize, JLabel.LEFT, false)));
        table.add(new MotLabel("Effects volume when 0 is quiet and 100 is full volume", fontSize, JLabel.LEFT, false));
        
        table.add(new MotLabel("Music Volume", fontSize, JLabel.LEFT, false));
        table.add(new MotFlowPanel(musicVolume, new MotLabel("%", fontSize, JLabel.LEFT, false)));
        table.add(new MotLabel("Music volume when 0 is silent and 100 is full volume", fontSize, JLabel.LEFT, false));
        
        table.add(new MotLabel("Music File", fontSize, JLabel.LEFT, false));
        table.add(musicFile);
        table.add(new MotLabel("Background Music ("+Comp.toString(VALID_EXTENSIONS, ",")+")", fontSize, JLabel.LEFT, false));
        
        table.add(new MotLabel("Autosave Interval", fontSize, JLabel.LEFT, false));
        table.add(autoSaveInterval);
        table.add(new MotLabel("The time in turns between auto save", fontSize, JLabel.LEFT, false));
        
        getContentPane().add(table, BorderLayout.CENTER);
        
        MotAction yesAction = new MotAction() {
			protected void _doAction() throws ApplicationError, ValidationError {
				save();
	            hide();
			}
		};
		MotAction noAction = new MotAction() {
			protected void _doAction() throws ApplicationError, ValidationError {
	            hide();
			}
		};
		CloseOptionsWindowButton yesButton = new CloseOptionsWindowButton(L.SAVE, yesAction);
		CloseOptionsWindowButton noButton = new CloseOptionsWindowButton(L.CANCEL, noAction);

        MotButtonPanel motButtonPanel = new MotButtonPanel(yesButton);
        getContentPane().add(motButtonPanel, BorderLayout.SOUTH);
    }
    
    private void setup() throws ApplicationError {
        PlayerCharacterBasic pc = owner.getPlayerCharacter();
        percentTerraformTextField.setText(Formatter.whole(pc.getPlayerEcologyPercent()*100));
        percentDevelopmentTextField.setText(Formatter.whole(pc.getPlayerDevelopmentPercent()*100));
        Settings settings = Settings.getInstance();
        if (settings.isFullScreen()) {
        	fullScreenOption.setSelected(true);
        } else if (settings.getMaximised() > 0) {
        	maximisedOption.setSelected(true);
    	} else {
    		windowOption.setSelected(true);
        }
        screenWidth.setText(Formatter.whole(settings.getWindowSizeX()));
        screenHeight.setText(Formatter.whole(settings.getWindowSizeY()));
        effectsVolume.setText(Formatter.whole(settings.getVolumeEffects()*100));
        musicVolume.setText(Formatter.whole(settings.getVolumeMusic()*100));
        autoSaveInterval.setText(Formatter.whole(settings.getAutosaveInterval()));
        
        moveCraftWarning.setSelected(settings.isMoveCraftWarning());
        
        String[] songs = settings.getSongs();
        if (songs != null && songs.length > 0) {
        	musicFile.setText(songs[0]);
        }
      
        effectsVolume.setChangeTask(new MotTask() {
			public void run() throws ApplicationError, ValidationError {
	            double vol = effectsVolume.getTextAsDouble()/100.0;
	            Settings settings = Settings.getInstance();
	            if (vol != settings.getVolumeEffects()) {	            	
	            	if (vol >= 0.0 && vol <= 1.0) {
	            		settings.setVolumeEffects(vol);
	            		Sounds.setVolume(vol);
	            	} else {
	            		Logger.logWarning("SHOULD NOT GET HERE");
	            	}
	            	Sounds.getWeaponSound(Technologies.getWeapons().getWeaponAtIndex(0)).play();
	            }
			}
		});
        
        musicVolume.setChangeTask(new MotTask() {
			public void run() throws ApplicationError, ValidationError {
	            double vol = musicVolume.getTextAsDouble()/100.0;
	            Settings settings = Settings.getInstance();
	            if (vol != settings.getVolumeMusic()) {	            	
		            if (vol >= 0.0 && vol <= 1.0) {
						settings.setVolumeMusic(vol);
						JukeBox.getInstance().setVolume(vol);
		            } else {
		            	Logger.logWarning("SHOULD NOT GET HERE");
		            }
	            }
			}
		});
        
        musicFile.setChangeTask(new MotTask() {
			public void run() throws ApplicationError, ValidationError {
	            String song = musicFile.getText();
            	Settings settings = Settings.getInstance();
	            if (song != null && !Comp.in(song, settings.getSongs())) {
	            	File file = new File(song);
	            	if (!file.exists()) {
	            		throw new ValidationError("File does not exist");
	            	}
	            	JukeBox jukeBox = JukeBox.getInstance();
	            	settings.deleteSongs();
	            	jukeBox.deleteSongs();
					jukeBox.stop();
					settings.setSong(song);
					jukeBox.add(file);
					jukeBox.start();
	            } else {
	            	Logger.logWarning("SHOULD NOT GET HERE");
	            }
			}
		});
        
        show();
    }
    
    private void save() throws ApplicationError, ValidationError {
    	Settings settings = Settings.getInstance();
        try {
            if (fullScreenOption.isSelected()) {
            	int currentFullScreen = settings.getFullScreen();
            	boolean r = (currentFullScreen == 1) || MotConfirmationDialog.showInPopup(this, "Screen Settings take effect on restart<br/>Not all settings are supported by all systems<br/>Should the game fail on startup delete the file settings.xml to revert to defaults.", L.CONFIRM);
            	if (!r) {
            		throw new ValidationError("No Changes made");
            	}
            	settings.setFullScreen(1);
            	settings.setMaximised(0);
            } else if (maximisedOption.isSelected()) {
            	settings.setFullScreen(0);
            	settings.setMaximised(1);
            } else {
            	settings.setFullScreen(0);
            	settings.setMaximised(0);
            }
            settings.setWindowSizeX(screenWidth.getTextAsInt());
            settings.setWindowSizeY(screenHeight.getTextAsInt());
            settings.setVolumeEffects(effectsVolume.getTextAsDouble()/100);
            settings.setVolumeMusic(musicVolume.getTextAsDouble()/100);
            settings.setAutosaveInterval(autoSaveInterval.getTextAsInt());
            settings.setMoveCraftWarning(moveCraftWarning.isSelected());
        	PlayerCharacterBasic pc = owner.getPlayerCharacter();
            Double pe = null;
            String txt = percentTerraformTextField.getText();
            if (txt != null) {
                pe = new Double(Double.parseDouble(txt)/100.0);
            }
            Double pd = null;
            txt = percentDevelopmentTextField.getText();
            if (txt != null) {
                pd = new Double(Double.parseDouble(txt)/100.0);
            }
            ProxyStore.getInstance().setPlayerCharacterPercents(null, null, null, null, null, pe, pd, null, null);
            settings.save();
        } catch (NumberFormatException e) {
            throw new ValidationError("Please enter numbers as terraform and development percents");
        }
    }
}