/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc;

import javax.swing.JLabel;
import javax.swing.JPanel;

import mot.client.proxy.ProxyStore;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotLabel;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.BackgroundImagePanel;
import mot.client.swing.panels.ComponentTableWithBackground;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.common.actions.ExitGameButton;
import mot.client.swing.window.common.actions.NewGameButton;
import mot.client.swing.window.etc.actions.LoadGameButton;
import mot.client.swing.window.etc.actions.OpenMultiPlayerConnectWindowButton;
import mot.common.bean.Settings;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;

public class StartupWindow extends MotInternalWindow {

	public static final String GAME_BACKGROUND_IMAGE = PropertyManager.getMandatoryNoError("image.background.game");
    
    public static StartupWindow instance;
    
    private boolean isMultiPlayer;

    public StartupWindow() throws ApplicationError {
        super(null, null, L.WELCOME, null);

        instance = this;
        setup();

        show();
    }

    private void setup() throws ApplicationError {
        boolean canHaveMultipleUsers = ProxyStore.canHaveMultipleUsers();

        ComponentTable buttonPanel;
        
        if (canHaveMultipleUsers && isMultiPlayer) {
            buttonPanel = new ComponentTable("", 6, 1, 8, 0, 5);
            buttonPanel.add(new SinglePlayerButton(), "HT");
            buttonPanel.add(new NewGameButton(this, true), "HT");
            buttonPanel.add(new OpenMultiPlayerConnectWindowButton(this, true), "HT");
            buttonPanel.add(new LoadGameButton(this, null, null, true), "HT");
            buttonPanel.add(new OpenMultiPlayerConnectWindowButton(this, false), "HT");
            buttonPanel.add(new ExitGameButton(this), "HT");
        } else {
            String lastGame = Settings.getInstance().getLastGameSaved();
            String lastPlayer = Settings.getInstance().getLastGamePlayer();
            int qtyRows = canHaveMultipleUsers ? 4 : 3;
            if (lastGame != null) qtyRows++;
            buttonPanel = new ComponentTable("", qtyRows, 1, 8, 0, 5);
            if (canHaveMultipleUsers) { 
            	buttonPanel.add(new MultiPlayerButton(), "HT");
            }
            buttonPanel.add(new NewGameButton(this, false), "HT");
            buttonPanel.add(new LoadGameButton(this, null, null, false), "HT");
            if (lastGame != null) buttonPanel.add(new LoadGameButton(this, lastGame, lastPlayer, false), "HT");
            buttonPanel.add(new ExitGameButton(this), "HT");
        }
        buttonPanel.setOpaque(false);
        
        
        ComponentTable controlPanel = new ComponentTable("", 2, 1, 0, 0, 0);
        String version = PropertyManager.getVersion();
        controlPanel.add(buttonPanel, "HVSP");
        controlPanel.add(new MotLabel(version, -2, null, null, JLabel.CENTER, false), "H");
        controlPanel.setOpaque(false);
        
        String image = ImageConfigFactory.getImageConfigNoError("Border200x250").getImage();
        ComponentTable contentPanel = new ComponentTableWithBackground("", 1, 1, 25, 0, 0, image);
        contentPanel.add(controlPanel, "HVSP");
        
        int height = canHaveMultipleUsers || isMultiPlayer ? 275 : 225;
        GameBackgroundPanel backgroundPanel = new GameBackgroundPanel(contentPanel, height);

        getContentPane().removeAll();
        getContentPane().add(backgroundPanel, null);
        
        getContentPane().doLayout();
        getContentPane().repaint();
    }
    
    class MultiPlayerButton extends MotButton {
    	public MultiPlayerButton() {
    		super("Multi Player Menu");
    	}
    	@Override
    	protected void _doAction() throws ApplicationError, ValidationError {
    		isMultiPlayer = true;
    		setup();
    	}    	
    }
    
    class SinglePlayerButton extends MotButton {
    	public SinglePlayerButton() {
    		super("Single Player Menu");
    	}
    	@Override
    	protected void _doAction() throws ApplicationError, ValidationError {
    		isMultiPlayer = false;
    		setup();
    	}    	
    }
}


class GameBackgroundPanel extends BackgroundImagePanel {

    private JPanel controlPanel;
    private int height;

    public GameBackgroundPanel(JPanel controlPanel, int height) {
        this.controlPanel = controlPanel;
        this.height = height;
        
        setLayout(null);
        
        add(this.controlPanel);
        
//        String backgroundImage = ImageConfigFactory.getImageConfigNoError("Border250x350").getImage();
//        imagePanel = new ImagePanel(backgroundImage);
    }
    
    protected void drawSizeRelativeObjects() {
        setPosition(controlPanel, 0.7, height, 200, false);        
    }
    
    protected String getImage() {
        return StartupWindow.GAME_BACKGROUND_IMAGE;
    }
    
}

