/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc.actions;

import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.common.MotTextField;
import mot.client.swing.window.etc.MultiPlayerConnectWindow;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.UserPlayerSummary;
import mot.common.command_results.UserPlayerSummaryCR;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;

public class ConnectJoinButton extends ConnectButton implements CompletedCallback {
	
	private MotTextField serverIP;
	private MultiPlayerConnectWindow parent;
	
	// This is a list of available user players on the server this client can connect as
	private UserPlayerSummary[] userPlayerSummaries;
	
	public ConnectJoinButton(MotTextField serverIP, MultiPlayerConnectWindow parent) {
		super(L.CONNECT);
		this.serverIP = serverIP;
		this.parent = parent;
	}

	protected void _doAction() throws ApplicationError, ValidationError {
		if (parent.getStatus().isInit()) {
			ServerProxy serverProxy = ProxyStore.getInstance();
			if (!serverProxy.isConnected()) {
				String ip = serverIP.getText();
				if (ip == null || ip.length() < 4) {
					throw new ValidationError(L.INVALID_IP_ADDRESS);
				}
				serverProxy.connect(ip);
			}
			if (serverProxy.isConnected()) {
				serverProxy.fetchRegisteredPlayers(this, this);
			}
		} else if (parent.getStatus().isConnected() || parent.getStatus().isWaiting()) {
			String playerName = parent.getRace();
			LoadGameButton.register(playerName, true);
			parent.saveSelection();
		}
	}

	@Override
	public void complete(Object result) throws ApplicationError, ValidationError {
		if (result == null) {
			setText(L.UPDATE);
			MainWindow.getInstance().reportValidationError(new ValidationError("No Game Loaded - Update after a game is loaded"));
		} else if (result instanceof UserPlayerSummaryCR) {
			UserPlayerSummaryCR cr = (UserPlayerSummaryCR) result;
			userPlayerSummaries = cr.getResult();
			parent.getRaceComboBox().updateContent(userPlayerSummaries);
			parent.getStatus().moveToConnected();
			parent.updateAlignmentDescription( getAncientRace() );
			setText(L.JOIN);
		}
	}

	private String getAncientRace() {
		String race = parent.getRaceComboBox().getRace();
		if (race != null && userPlayerSummaries != null) {
			for (int i = 0; i < userPlayerSummaries.length; i++) {
				if (Comp.same(userPlayerSummaries[i].getRaceName(), race)) {
					return userPlayerSummaries[i].getAlignment();
				}
			}
		}
		return null;
	}

	@Override
	public void notifyDetailsChanged() {
		parent.updateAlignmentDescription( getAncientRace() );
		if (parent.getStatus().isWaiting()) {
			setText(L.UPDATE);
		}
	}

	

}
