/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc.actions;

import java.awt.event.ActionListener;

import mot.client.animation.GalaxyAnimationRegister;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.ClientErrorHandler;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.window.common.MessageWindow;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.etc.LoadWindow;
import mot.client.swing.window.galaxy.GalaxyWindowListener;
import mot.client.swing.window.galaxy.GalaxyWindowView;
import mot.client.swing.window.galaxy.MotGalaxyMapActions;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Settings;
import mot.common.command_results.StringCR;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class LoadGameButton extends MotButton implements MotGalaxyMapActions, ActionListener {

    
    private MotInternalWindow parent;
	private String savedGameFile;
	private String savedGamePlayer;
	private boolean isMultiPlayer;

    public LoadGameButton(MotInternalWindow parent, String savedGameFile, String savedGamePlayer, boolean isMultiPlayer) {
		setText(savedGameFile == null ? L.LOAD : L.CONTINUE);
		this.parent = parent;
		this.savedGameFile = savedGameFile;
		this.savedGamePlayer = savedGamePlayer;
		this.isMultiPlayer = isMultiPlayer;
	}

	protected void _doAction() throws ApplicationError, ValidationError {
		//ServerProxy serverProxy = ProxyStore.getLocalInstance();
        parent.hide();
        CompletedCallback loadCallback = new CompletedCallback() {
			public void complete(Object result) throws ApplicationError, ValidationError {
		        String playerName = ((StringCR) result).getValue();
	        	if (Logger.isDebug()) {
	        		Logger.logDebug("LoadGameButton.complete() Loaded player="+playerName);
	        	}
		        if (playerName != null) {
		        	MainWindow.getExestingInstance().toGalaxyWindow();
		        	register(playerName, false);
		        } else {
		        	parent.show();
		        }
			}
		};
        load(savedGameFile, savedGamePlayer, loadCallback, this, parent, isMultiPlayer);
	}

    public static void register(String playerName, boolean updateClient) throws ApplicationError, ValidationError {
        CompletedCallback registerCallback = new CompletedCallback() {
			public void complete(Object result) throws ApplicationError, ValidationError {
	            UserPlayerDelegate player = (UserPlayerDelegate) result;
	        	MainWindow mainWin = MainWindow.getInstance();         
	            mainWin.setWindowOwner(player.getName());
	            try {
	            	GalaxyWindowListener gwl = MainWindow.getInstance().getGalaxyMapListener();
	            	gwl.clearSelection();
	            	gwl.setMapWindowSize();
	            	gwl.focusOnHomeSolarSystem();
	                mainWin.getGalaxyMapConfiguration().getGalaxyView().updateStatus(player);
	                mainWin.getGalaxyMapConfiguration().getGalaxyView().getMessageList().removeAllMessages();
	                
	            	GalaxyWindowView view = mainWin.getGalaxyMapConfiguration().getGalaxyView();
	                view.getMapPanel().rebuildContent();
	        		GalaxyAnimationRegister.clean();
	        		view.getRadarPanel().repaint();
	        		
					view.closeAllWindows();
	            } catch (ApplicationError ex) {
	            	Logger.logError("", ex);
	            }
			}
        };
    	MainWindow mainWin = MainWindow.getInstance();         
    	ServerProxy serverProxy = ProxyStore.getInstance();
    	serverProxy.register(playerName, updateClient, registerCallback, null);
    }
    
    public static void load(String savedGameFile, String savedGamePlayer, CompletedCallback loadCallback, ClientErrorHandler errorHandler, MotInternalWindow parent, boolean isMultiPlayer) throws ApplicationError, ValidationError {
        if (savedGameFile != null) {
        	if (Logger.isDebug()) {
        		Logger.logDebug("LoadGameButton.load() Load file="+savedGameFile+" player="+savedGamePlayer);
        	}
        	MessageWindow messageWin = new MessageWindow(null, parent, "Loading.....");
	    	messageWin.waitUntilPainted();
       		Settings.getInstance().updateLastGameSaved(savedGameFile, savedGamePlayer);
           	ServerProxy serverProxy = ProxyStore.getInstance();
           	DeligateCacheManager.flush();
           	CompletedCallback splashCallback = new LoadSplashScreenCallback(messageWin, parent, loadCallback);
           	LoadSplashScreenErrorCallback splashErrorCallback = new LoadSplashScreenErrorCallback(serverProxy, messageWin, errorHandler);
           	serverProxy.load(savedGameFile, isMultiPlayer ? savedGamePlayer : null, splashCallback, splashErrorCallback);
        } else {
    		new LoadWindow(parent, null, loadCallback, isMultiPlayer);
        }
    }
    
}

class LoadSplashScreenCallback implements CompletedCallback {
	
	private MessageWindow messageWin;
	private CompletedCallback loadCallback;
	private MotInternalWindow parent;
	
	LoadSplashScreenCallback(MessageWindow messageWin, MotInternalWindow parent, CompletedCallback loadCallback) {
		this.messageWin = messageWin;
		this.parent = parent;
		this.loadCallback = loadCallback;
	}
	
	public void complete(Object result) throws ApplicationError, ValidationError {
		messageWin.hide();
		if (parent != null) {
			parent.setVisible(false);
		}
		loadCallback.complete(result);
	}
}

class LoadSplashScreenErrorCallback implements ClientErrorHandler, CompletedCallback {
	
	private ServerProxy serverProxy;
	private MessageWindow messageWin;
	private ClientErrorHandler errorCallback;
	
	LoadSplashScreenErrorCallback(ServerProxy serverProxy, MessageWindow messageWin, ClientErrorHandler errorCallback) {
		this.serverProxy = serverProxy;
		this.messageWin = messageWin;
		this.errorCallback = errorCallback;
	}
	
	private void updateDelegates() {
		try {
			serverProxy.updateDelegates(this);
		} catch (ValidationError e) {
			MainWindow.getExestingInstance().reportValidationError(e);
		} catch (ApplicationError e) {
			MainWindow.getExestingInstance().reportApplicationError(e);
		}
	}

	public void reportValidationError(ValidationError e) throws ValidationError {
		updateDelegates();
		messageWin.hide();
		if (errorCallback == null) {
			MainWindow.getExestingInstance().reportValidationError(e);
		} else {
			errorCallback.reportValidationError(e);
		}
	}
	
	public void reportApplicationError(Exception e) {
		updateDelegates();
		messageWin.hide();
		if (errorCallback == null) {
			MainWindow.getExestingInstance().reportApplicationError(e);
		} else {
			errorCallback.reportApplicationError(e);
		}
	}

	@Override
	public void complete(Object result) throws ApplicationError, ValidationError {
		// Delegates have been updated
	}
	
}


