/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.etc.actions;

import mot.client.animation.GalaxyAnimationRegister;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.etc.NewGameWindow;
import mot.client.swing.window.galaxy.GalaxyWindowListener;
import mot.client.swing.window.galaxy.GalaxyWindowView;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.solarsystem.ColoniseGraphicWindow;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.NewGameData;
import mot.common.bean.NewGameDataComputerPlayerDetails;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.bean.PlayerColor;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class NewGameButton extends MotButton {

	protected NewGameWindow parent;

	protected NewGameButton(String buttonName, NewGameWindow parent) {
		super(buttonName);
		this.parent = parent;
	}
	
	public NewGameButton(NewGameWindow parent) {
		super(L.NEW_GAME);
		this.parent = parent;
	}

	protected void _doAction() throws ApplicationError, ValidationError {
        boolean hasTutorial = parent.hasTutorial();
        int qtyPlanets = parent.getQtyPlanets();
        int qtyNebula = parent.getQtyNebula();
        Size galaxySize = parent.getGalaxySize();
        int qtyComputerPlayers = parent.getQtyComputerPlayers();
        Double level = parent.getLevel();
        boolean hasWormHole = parent.isWormHoleEnabled();
        boolean hasSpaceKaan = parent.isSpaceKaanEnabled();
        NewGameDataComputerPlayerDetails[] computerPlayers = parent.getComputerPlayerDetails();
        
        try {
        	parent.saveSelection();
        } catch (Exception e) {
        	Logger.logError("Unable to save settings.xml", e);
        }
        
        boolean openEditor = (this instanceof NewCustomGameButton);
		NewGameData gameData = new NewGameData(computerPlayers, level, hasTutorial, qtyPlanets, qtyNebula, galaxySize, hasWormHole, hasSpaceKaan);
        
        createNewGame(parent, parent.getOptions(), gameData, openEditor);
	}

    /**
     * Create a new Game consisting of a single user player using a local
     * client (GUI window system).
     */
    private static void createNewGame(NewGameWindow setupWin, SetupOptions userOptions, NewGameData gameData, boolean openEditor) throws ValidationError, ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("SetupWindow.startNewGame()");
        }

        String playerAlignment = setupWin.getAlignment();
		PlayerColor playerColor = setupWin.getPlayerColor();
		String playerName = setupWin.getRace();

        ServerProxy serverProxy = ProxyStore.getInstance();
        
        int cost = userOptions.getCost();
        if (cost > NewGameWindow.MAX_COST) {
            throw new ValidationError(L.YOU_CAN_ONLY_SELECT_OPTIONS_UPTO_A_TOTAL_COST_OF+NewGameWindow.MAX_COST);
        }
        
		_hasTutorial = gameData.isHasTutorial();
		
        NewGameUserPlayerData userData = new NewGameUserPlayerData(playerName, userOptions, playerColor, playerAlignment);
        NewGameButtonCompleteCallback callback = new NewGameButtonCompleteCallback(setupWin, openEditor, playerName);
        
        // Create an instance of the game - If this is a multi player game we may need to wait for other players!
        serverProxy.newGame(userData, gameData, callback, null);
    }
    
    private static boolean _hasTutorial = false;
    
	public static void startNewGameAnimation() throws ApplicationError, ValidationError {
		MainWindow mainWindow = MainWindow.getInstance();
    	GalaxyWindowListener gwl = mainWindow.getGalaxyMapListener();
    	GalaxyWindowView view = mainWindow.getGalaxyMapConfiguration().getGalaxyView();
    	UserPlayerDelegate user = mainWindow.getWindowOwner();
    	
        if (_hasTutorial) {
        	//I have decided to switch of saving loading of tutorial state.
        	//serverProxy.setTutorial(new TutorialStatus());
        	TutorialSystem.setup();
        }
        
		gwl.focusOnHomeSolarSystem();
		view.updateStatus(user);
        
        // Finally show the new Colony window to give the user a chance to change the home
        // colonies name.
        PlanetDelegate [] colonies = DeligateCacheManager.getPlanets(user);
        if (colonies != null && colonies.length > 0) {
            new ColoniseGraphicWindow(null, colonies[0], user, (colonies[0] == null) ? null : L.COLONISE_PLANET+" "+colonies[0].getName());
        }
	}	

	public static void registerInNewGame(MotInternalWindow setupWin, boolean openEditor, String playerName) throws ApplicationError, ValidationError {
        //UserPlayerDelegate user = (UserPlayerDelegate) DeligateCacheManager.getPlayer(playerName);
        
        // Close setup window 
        if (setupWin != null) {
            setupWin.hide();
        }
        
        RegisterCompleteCallback registerCallback = new RegisterCompleteCallback(openEditor);
        // Create a proxy which acts as the glue between the user interface and the engine.
        MainWindow mainWindow = MainWindow.getInstance();         
        ServerProxy serverProxy = ProxyStore.getInstance();
        serverProxy.register(playerName, false, registerCallback, null);
	}

}

class NewGameButtonCompleteCallback implements CompletedCallback {
	
	private NewGameWindow setupWin;
	private boolean openEditor;
	private String playerName;

	public NewGameButtonCompleteCallback(NewGameWindow setupWin, boolean openEditor, String playerName) {
		this.setupWin = setupWin;
		this.openEditor = openEditor;
		this.playerName = playerName;
	}

	@Override
	public void complete(Object result) throws ApplicationError, ValidationError {
		NewGameButton.registerInNewGame(setupWin, openEditor, playerName);
	}	
}

class RegisterCompleteCallback implements CompletedCallback {
	
	private boolean openEditor;

	public RegisterCompleteCallback(boolean openEditor) {
		this.openEditor = openEditor;
	}

	@Override
	public void complete(Object result) throws ApplicationError, ValidationError {
		UserPlayerDelegate player = (UserPlayerDelegate) result;
		
        // Create or show the Main Window
        MainWindow mainWindow = MainWindow.getInstance();         
        mainWindow.setWindowOwner(player.getName());
    	GalaxyWindowListener gwl = mainWindow.getGalaxyMapListener();
    	gwl.clearSelection();

    	GalaxyWindowView view = mainWindow.getGalaxyMapConfiguration().getGalaxyView();
        view.getMapPanel().rebuildContent();
		GalaxyAnimationRegister.clean();
		
		view.getRadarPanel().repaint();
    	
        gwl.setMapWindowSize();
		gwl.focusOnHomeSolarSystem();
		view.updateStatus(player);
        
		if (openEditor) {
			mainWindow.toEditorWindow(true);
		} else {
			NewGameButton.startNewGameAnimation();
		}
	}	
}

