/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import mot.client.actions.ActiveRectangle;
import mot.client.bean.MoveableObjectLocation;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowConfig;
import mot.client.swing.window.main.MainWindowListener;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.common.vo.NavigatorVO;

public class GalaxyMapMoo extends ActiveRectangle {

	public static final int CRAFT_WIDTH = 20;
	public static final int CRAFT_HEIGHT = 20;

	/**
	 * The Moveable Object location to which this active rectangle was built.
	 */
	private MoveableObjectLocation mol;

	/**
	 * Other MOOs at this location.
     * NB each of these will have their own ActiveRectangle.
	 */
    private MoveableOwnedObjectDelegate[] moos;
    private int qty = 1;
    
    /**
     * The moo that owns this Active Rectangle (each moo has its own ActiveRectangle).
     */
    private MoveableOwnedObjectDelegate mostVisibleMoo;
    
    private ActiveRectangle orbitedObject;
    
    /**
     * Is this object visible on the screen (within view port).
     */
    private boolean visible;
    
    /**
     * If this MoL has two active rectangles (players list and other list) this is the screen location offset to apply to this list.
     */
	private int offset = 0;
		
	public GalaxyMapMoo(MoveableObjectLocation mol, MoveableOwnedObjectDelegate [] moos, MoveableOwnedObjectDelegate mostVisibleMoo, int x, int y, boolean visible, ActiveRectangle orbitedObject, int offset) throws ApplicationError {
		this(mol, moos, mostVisibleMoo, x, y, visible, orbitedObject);
		this.offset = offset;
	}
	
	public GalaxyMapMoo(MoveableObjectLocation mol, MoveableOwnedObjectDelegate [] moos, MoveableOwnedObjectDelegate mostVisibleMoo, int x, int y, boolean visible, ActiveRectangle orbitedObject) throws ApplicationError {
		super(x, y, CRAFT_WIDTH, CRAFT_HEIGHT);
		this.mol = mol;
		this.moos = moos;
		this.mostVisibleMoo = mostVisibleMoo;
		this.orbitedObject = orbitedObject;
		this.visible = visible;
		this.qty = getQty(moos);

		update();
	}

	private int getQty(MoveableOwnedObjectDelegate[] ma) {
		if (ma == null) return 0;
		if (ma.length == 0) return 0;
		int qty = 0;
		for (int i = 0; i < ma.length; i++) {
			if (ma[i] instanceof FleetDelegate) {
				qty += ((FleetDelegate)ma[i]).getQtyCrafts();
			} else {
				qty++;
			}
		}
		return qty;
	}

	public void click() throws ApplicationError, ValidationError {
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateClick(this);
    	}
		MainWindowConfig config = MainWindow.getInstance().getActiveConfiguration();
		MainWindowListener listener = config.getListener();
    	if (moos == null) return;
   		listener.click(mostVisibleMoo);
		listener.playSound("SelectMoo");
	}

	public void doubleClick() throws ApplicationError, ValidationError {
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateDoubleClick(this);
    	}
		MainWindowConfig config = MainWindow.getInstance().getActiveConfiguration();
		config.getListener().doubleClick(mostVisibleMoo);
	}

	public boolean contains(NamedObjectDelegate o) {
		String name = o.getName();
		return Comp.same(name, mostVisibleMoo.getName()) || Comp.in(name, moos);
	}

	public boolean contains(String name) {
		return Comp.same(name, mostVisibleMoo.getName()) || Comp.in(name, moos);
	}

	public NamedObjectDelegate getContents() {
		return mostVisibleMoo;
	}
	
	public MoveableOwnedObjectDelegate getMostVisibleMoo() {
		return mostVisibleMoo;
	}
	
	public MoveableOwnedObjectDelegate getMooHeadingTo(StellarObjectDelegate dest) throws ApplicationError {
		if (moos == null) return null;
		for (int i = 0; i < moos.length; i++) {
			NavigatorVO navigator = moos[i].getNavigator();
			if (navigator != null && Comp.same(navigator.getDestination().getStellarObject(), dest.getName()) ) {
				return moos[i];
			}
		}
		return null;
	}
	
//	public void setMoo(MoveableOwnedObjectDelegate moo) {
//		this.moo = moo;
//	}
	
	public MoveableOwnedObjectDelegate [] getMoos() {
		return moos;
	}
	
	public MoveableOwnedObjectDelegate [] getAllMoos() {
		return mol.getMoos();
	}

	public int getQtyMoos() {
		return qty;
	}
	
	public boolean isVisible() {
		return visible;
	}
	
	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	/**
	 * Active rectangle use the observer pattern. This method is called when the moo this
	 * observer is watching has changed.
	 */
	public void update() throws ApplicationError {
		if (orbitedObject != null) {
	        int x = orbitedObject.getLeft();
	        int y = orbitedObject.getTop();
	        NavigatorVO navigator = mostVisibleMoo.getNavigator();
            if (navigator != null) {
            	x -= CRAFT_WIDTH;
            } else {
            	x += orbitedObject.getWidth();
            }
            y -= CRAFT_HEIGHT;
            y += offset;
    		setLeft(x);
    		setTop(y);
		}
	}
	
	/**
	 * Negative value implies this MOO is left of orbited object.
	 * Positive value implies this MOO is right of orbited object.
	 * @return The distance on X coordinate from the orbited object to this MOO.
	 */
	public int getXDistanceFromOrbitedObject() {
		if (orbitedObject != null) {
			Location loc1 = getCenterLocation();
			Location loc2 = orbitedObject.getCenterLocation();
			return loc1.getX() - loc2.getX();
		}
		return 0;
	}

	//public boolean isDefault() { return isDefaultMoo; }

	public boolean isSleeping() throws ApplicationError {
		if (moos != null) {
			if (moos != null) {
				for (int i = 0; i < moos.length; i++) {
					if (!moos[i].isSleeping()) return false;
				}
			}
		}
		return true;
	}

}
