/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics2D;
import java.util.ArrayList;

import mot.client.actions.ActiveRectangle;
import mot.client.actions.ActiveRectangleList;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.TransportDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.window.galaxy.actions.GalaxyMapButton;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftAttack;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftBombard;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftColonise;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftDelete;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftDeploy;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftInvade;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftJoinFleet;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftSleep;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftTransportColonists;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftTransportTroops;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftUpgrade;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftViewAFleetDetails;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Rectangle;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;

public class GalaxyMapMooList {

	public static final int BUTTON_GAP = 5;
	private static final int COMMAND_GAP = 5;

	private static final int WIDTH = 3;

	public static final int CRAFT_NAME_HEIGHT = 8;
	public static final int MAX_DISPLAID_IN_LIST = 4;
	
	/**
	 * Content of this MOO list.
	 */
	private MoveableOwnedObjectDelegate[] moos;
	
	/**
	 * Item selected in this list.
	 */
	private MoveableOwnedObjectDelegate selected;
	
	/**
	 * The rectangle in the screen clicked that caused this list to be rendered.
	 */
	private ActiveRectangle selectedAR;

	/**
	 * List of active rectangles currently being displayed on the screen.
	 */
	private GalaxyMapMooListItem [] visibleContent;

	/**
	 * The draggable bar in the screen to scroll this list's content.
	 */
	private GalaxyMapMooScrollBar scrollBar;

	/**
	 * This parent of this list.
	 */
	private ActiveRectangleList activeRectangleList;


	private int panelWidth;
	private int panelHeight;
	private int left;
	private int top;
	
	/**
	 * TRUE => This list is below the Select Planet Moo (top or middle of screen)
	 * FALSE => This list is above the Select Planet Moo (bottom of screen)
	 */
	private boolean originIsTop;
	
	private static ImageConfig imageTopConfig;
	private static String imageTop;
	private static String imageMiddle;
	private static String imageMiddleSelected;
	private static String imageBottom;
	private static int topHeight;
	
	public GalaxyMapMooList(ActiveRectangleList activeRectangleList, MoveableOwnedObjectDelegate [] moos, MoveableOwnedObjectDelegate selected, ActiveRectangle selectedAR, int left, int top, int panelWidth, int panelHeight, boolean originIsTop) throws ApplicationError {
		this.activeRectangleList = activeRectangleList;
		this.selectedAR = selectedAR;
		this.left = left;
		this.top = top;
		this.panelWidth = panelWidth;
		this.panelHeight = panelHeight;
		this.originIsTop = originIsTop;
		
		if (imageTopConfig == null) {
			imageTopConfig = ImageConfigFactory.getConfigForImage("MooMenuTop");
			imageTop = imageTopConfig.getImage();
			imageMiddle = ImageConfigFactory.getImageForConfig("MooMenuMiddle");
			imageMiddleSelected = ImageConfigFactory.getImageForConfig("MooMenuMiddleSelected");
			imageBottom = ImageConfigFactory.getImageForConfig("MooMenuBottom");
			topHeight = (int) (((double)imageTopConfig.getImageSize().getY()) / ((double)panelHeight) * imageTopConfig.getImageSize().getY());
		}
		
		setContent(moos, selected);
	}

	public void setContent(MoveableOwnedObjectDelegate [] moos, MoveableOwnedObjectDelegate selected) throws ApplicationError {
		this.moos = moos;
		this.selected = selected;
		
		if (moos != null) {			
			if (moos.length > MAX_DISPLAID_IN_LIST) {			
				double percent = ((double)MAX_DISPLAID_IN_LIST) / ((double)moos.length);
				int sbLeft = left + panelWidth + 2;
				int sbSize = MAX_DISPLAID_IN_LIST * panelHeight;
				int sbBarHeight = (int) (((double)sbSize) * percent);
				scrollBar = new GalaxyMapMooScrollBar(sbLeft, top, sbSize, sbBarHeight, WIDTH, moos, this);
			} else {
				scrollBar = null;
			}
		}
		
		setContentToScrollBar();
		
		if (selected != null) {
			setSelection(selected);
		}
	}
	
	public void setContentToScrollBar() throws ApplicationError {
		activeRectangleList.setButtons(null);
		if (scrollBar != null) {
			double scrollPercent = scrollBar.getPercentScroll();
			double start = ((moos.length * panelHeight) * scrollPercent)/panelHeight;
			int startIndex = Comp.roundDown(start);
			double percentOfFirst = 1.0 - (start - startIndex);
			int displayHeight = Comp.min(moos.length, MAX_DISPLAID_IN_LIST) * panelHeight;
			ArrayList<GalaxyMapMooListItem> r = new ArrayList(); 
			GalaxyMapMooListItem i = createMooListItem(moos[startIndex], left, top, panelWidth, (int)(percentOfFirst*panelHeight), panelHeight);
			r.add(i);
			int index = startIndex + 1;
			int heightCount = i.getHeight(); 
			while (heightCount <= displayHeight - panelHeight && index < moos.length) {
				i = createMooListItem(moos[index], left,  top + heightCount, panelWidth, panelHeight, panelHeight);
				heightCount += panelHeight; 
				index++;
				r.add(i);
			}
			if (heightCount <= displayHeight && index < moos.length) {
				i = createMooListItem(moos[index], left,  top + heightCount, panelWidth, (top + heightCount) - displayHeight, panelHeight);
				r.add(i);
			}
			
			visibleContent = r.toArray(new GalaxyMapMooListItem[r.size()]) ;
		} else {
			int qty = Comp.min(moos.length, MAX_DISPLAID_IN_LIST);
			int actualTop = top + (MAX_DISPLAID_IN_LIST - qty) * panelHeight;
			visibleContent = new GalaxyMapMooListItem[qty];
			for (int i = 0; i < qty; i++) {
				if (originIsTop) {
					visibleContent[i] = createMooListItem(moos[i], left, top + i*panelHeight, panelWidth, panelHeight, panelHeight);
				} else {
					visibleContent[i] = createMooListItem(moos[i], left, actualTop + i*panelHeight, panelWidth, panelHeight, panelHeight);
				}
			}
		}

	}

	private GalaxyMapMooListItem createMooListItem(MoveableOwnedObjectDelegate moo, int x, int y, int panelWidth, int panelHeight, int maxPanelHeight) throws ApplicationError {
		GalaxyMapMooListItem gmm = new GalaxyMapMooListItem(this, moo, x, y, panelWidth, panelHeight);
		if (panelHeight == maxPanelHeight) {
			createButtons(moo, x, y, panelWidth, panelHeight);
		}
		return gmm;
	}
	
	private void createButtons(MoveableOwnedObjectDelegate moo, int x, int y, int panelWidth, int panelHeight) throws ApplicationError {
		boolean isSelected = Comp.same(moo, selected);
		if (isSelected) {
			UserPlayerDelegate owner = MainWindow.getOwner();
			boolean galaxyMenuCanAttackActive = moo.galaxyMenuCanAttackActive() && moo.isOwnedBy(owner);
			boolean galaxyMenuCanBombardActive = moo.galaxyMenuCanBombardActive() && moo.isOwnedBy(owner);
			boolean galaxyMenuCanInvadeActive = moo.galaxyMenuCanInvadeActive() && moo.isOwnedBy(owner);
			boolean galaxyMenuCanColoniseActive = moo.galaxyMenuCanColoniseActive() && moo.isOwnedBy(owner);
			boolean galaxyMenuCanDeployActive = moo.galaxyMenuCanDeployActive() && moo.isOwnedBy(owner);
			boolean galaxyMenuCanJoinFleetActive = moo.galaxyMenuCanJoinFleetActive() && moo.isOwnedBy(owner);
			boolean galaxyMenuCanUpgradeActive = moo.galaxyMenuCanUpgradeActive() && moo.isOwnedBy(owner);
			boolean galaxyMenuCanDeleteActive = moo.galaxyMenuCanDeleteActive() && moo.isOwnedBy(owner);
			boolean galaxyMenuCanViewFleetActive = moo instanceof FleetDelegate;
			TransportPlanetPair galaxyMenuCanTransportColoniststActive = null;
			TransportPlanetPair galaxyMenuCanTransportTroopsActive = null;
			boolean galaxyMenuSleepActive = !moo.hasNavigator() && moo.isOwnedBy(owner) && !moo.isSleeping();
			
			int qtyFreeTransport = owner.getQtyFreeTransport();
			if (moo instanceof TransportDelegate && moo.getOrbit() != null && moo.getOrbit().getPlanet() != null && moo.isOwnedBy(owner)) {
				TransportDelegate transport = (TransportDelegate) moo;
		        PlanetDelegate planet = DeligateCacheManager.getPlanet(moo.getOrbit().getPlanet());
				if (transport.containsTroops() && planet.isOwnedBy(owner) && !planet.isMiningColony() && planet.getQtyGroundDefense() > 6) {
					galaxyMenuCanTransportTroopsActive = new TransportPlanetPair(transport, planet);
				}
				if (transport.containsColonists() && planet.isOwnedBy(owner) && !planet.isMiningColony() && planet.getPopulation() > 6) {
					galaxyMenuCanTransportColoniststActive = new TransportPlanetPair(transport, planet);
				}
			}

			int qty = 0;
			if (galaxyMenuCanAttackActive) qty++;
			if (galaxyMenuCanBombardActive)qty++;
			if (galaxyMenuCanInvadeActive) qty++;
			if (galaxyMenuCanColoniseActive) qty++;
			if (galaxyMenuCanDeployActive) qty++;
			if (galaxyMenuCanJoinFleetActive) qty++;
			if (galaxyMenuCanUpgradeActive) qty++;
			if (galaxyMenuCanDeleteActive) qty++;
			if (galaxyMenuCanTransportColoniststActive != null) qty++;
			if (galaxyMenuCanTransportTroopsActive != null) qty++;
			if (galaxyMenuSleepActive) qty++;
			
			int qtyRows, qtyCols;
			if (qty < 4) {
				qtyRows = 1;
				qtyCols = qty;
			} else if (qty < 7) {
				qtyRows = 2;
				qtyCols = 3;
			} else {
				qtyRows = 2;
				qtyCols = Comp.ceil(qty / 2.0);
			}
			
			ArrayList<GalaxyMapButton> r = new ArrayList();
			GalaxyWindowListener listener = MainWindow.getInstance().getGalaxyMapListener();
			int buttonSize = getButtonSize();
			
			int row = 0;
			int col = 0;
			
			boolean onRight = selectedAR.getRight() < x;
			int panelLeft = onRight ? x + panelWidth + COMMAND_GAP : x - COMMAND_GAP - buttonSize;
			int direction = onRight ? 1 : -1;
			int panelTop = y;
			
			if (galaxyMenuCanAttackActive) {
				r.add(new GalaxyMapButtonCraftAttack(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanBombardActive) {
				r.add(new GalaxyMapButtonCraftBombard(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanInvadeActive) {
				r.add(new GalaxyMapButtonCraftInvade(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanColoniseActive) {
				r.add(new GalaxyMapButtonCraftColonise(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanDeployActive) {
				r.add(new GalaxyMapButtonCraftDeploy(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanJoinFleetActive) {
				r.add(new GalaxyMapButtonCraftJoinFleet(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanUpgradeActive) {
				r.add(new GalaxyMapButtonCraftUpgrade(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanDeleteActive) {
				r.add(new GalaxyMapButtonCraftDelete(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanViewFleetActive) {
				r.add(new GalaxyMapButtonCraftViewAFleetDetails(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanTransportColoniststActive != null) {
				r.add(new GalaxyMapButtonCraftTransportColonists(galaxyMenuCanTransportColoniststActive.planet, listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuCanTransportTroopsActive != null) {
				r.add(new GalaxyMapButtonCraftTransportTroops(galaxyMenuCanTransportTroopsActive.planet, listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			if (galaxyMenuSleepActive) {
				r.add(new GalaxyMapButtonCraftSleep(listener, panelLeft + col * (buttonSize+BUTTON_GAP) * direction, panelTop + row * (buttonSize+BUTTON_GAP)));
				col++;
				if (col == qtyCols) { col = 0; row++; }
			}
			activeRectangleList.setButtons(r.toArray(new GalaxyMapButton[r.size()]));
		}
	}

	public GalaxyMapMooListItem [] getContent() {
		return visibleContent;
	}

	int z = 0;
	public void render(Container container, Graphics2D graphics, Rectangle visible) throws ApplicationError {
		if (visibleContent != null && visibleContent.length >= 1) {
			int left = visibleContent[0].getLeft();
			int top = visibleContent[0].getTop(); 
			//Logger.logDebug("Render List top="+top+" panelSize="+panelSize);
            ImageUtils.drawImage(container, graphics, imageTop, left, top-topHeight, panelWidth, topHeight);
            java.awt.Rectangle originalClip = graphics.getClipBounds();
            graphics.setClip(left-10, top, panelWidth+20, MAX_DISPLAID_IN_LIST * panelHeight);
			for (int i = 0; i < visibleContent.length; i++) {
				GalaxyMapMooListItem so = visibleContent[i];
				if (visible.intersects(so)) {
					//Logger.logDebug("Render Item ="+so.getMoo().getName()+" top="+top);
					int panelTop = i == 0 ? so.getBottom() - panelHeight : so.getTop();
					boolean isSelected = Comp.same(so.getContents(), selected);
					if (isSelected) {
						ImageUtils.drawImage(container, graphics, imageMiddleSelected, left, panelTop, panelWidth, panelHeight);					
					} else {
						ImageUtils.drawImage(container, graphics, imageMiddle, left, panelTop, panelWidth, panelHeight);					
					}
					int centerX = left+panelWidth/2;
					int centerY = panelTop+(panelHeight-CRAFT_NAME_HEIGHT)/2;
			        boolean sleeping = so.getMoo().isSleeping();
			        if (sleeping) {
			        	ImageUtils._drawImageCenteredAlpha5At(container, graphics, so.getPicture(), centerX, centerY, so.getImageSize(), so.getImageSize());					
			        } else {
			        	ImageUtils._drawImageCenteredAt(container, graphics, so.getPicture(), centerX, centerY, so.getImageSize(), so.getImageSize());					
			        }
					//Logger.logDebug("Draw Label ="+so.getMoo().getName()+" top="+top+"panelSize="+panelSize);
					//Color ownerColour = so.getMoo().getOwnerColor();
					ImageUtils.drawCenteredLabel(container, graphics, so.getMoo().getName(), centerX, panelTop+panelHeight-4, Color.WHITE, false, 10);
				}
			}
			graphics.setClip(originalClip.x, originalClip.y, originalClip.width, originalClip.height);
            ImageUtils.drawImage(container, graphics, imageBottom, left, visibleContent[visibleContent.length-1].getBottom(), panelWidth, topHeight);
            if (scrollBar != null) {
            	double percent = ((double)MAX_DISPLAID_IN_LIST) / ((double)visibleContent.length);
            	int sbTop = scrollBar.getTop();
            	int sbLeft = scrollBar.getLeft();
            	int sbBarTop = scrollBar.getBarTop();
            	int sbBarHeight = scrollBar.getBarHeight();
            	if (sbBarTop > sbTop) {            		
            		graphics.setColor(Color.GRAY);
            		graphics.fillRect(sbLeft, sbTop, WIDTH, sbBarTop-sbTop);
            	}
            	graphics.setColor(Color.WHITE);
            	graphics.fillRect(sbLeft, sbBarTop, WIDTH, sbBarHeight);
            	int sbBottom = scrollBar.getBottom();
            	int border = sbBarTop+sbBarHeight;
            	if (sbBottom > border) {            		
            		graphics.setColor(Color.GRAY);
            		graphics.fillRect(sbLeft, border, WIDTH, sbBottom-border);
            	}
            }
		}	
	}
	
	private static int buttonSize = 0;
	public static int getButtonSize() {
		if (buttonSize == 0) {
			buttonSize = GalaxyMapButton.getSize(MotGalaxyMapActions.MENU_CRAFT_COLONISE);
		}
		return buttonSize;
	}

	public void setSelection(MoveableOwnedObjectDelegate moo) throws ApplicationError {
		if (visibleContent != null && visibleContent.length > 0) {
			GalaxyMapMooListItem mooPanel = isFullyVisibleInScrollPanel(moo);
			if (mooPanel != null) {
				selected = moo;
				int x = mooPanel.getLeft();
				int y = mooPanel.getTop();
				int panelWidth = mooPanel.getWidth();
				int panelHeight = mooPanel.getHeight();
				activeRectangleList.setButtons(null);
				createButtons(moo, x, y, panelWidth, panelHeight);
			} else if (scrollBar != null) {
				int newSelectedIndex = getIndexInList(moo);
				int newStartIndex = newSelectedIndex - MAX_DISPLAID_IN_LIST /2;
				if (newStartIndex < 0) newStartIndex = 0;
				if (newStartIndex > moos.length - MAX_DISPLAID_IN_LIST) newStartIndex = moos.length - MAX_DISPLAID_IN_LIST;
				double scrollPercent = newStartIndex / ((double)moos.length); 
				scrollBar.setScrollPercent(scrollPercent);
				setContentToScrollBar();
				selected = moo;
			}
		}
	}

	private int getIndexInList(MoveableOwnedObjectDelegate moo) {
		if (moos != null) {
			for (int i = 0; i < moos.length; i++) {
				if (Comp.same(moo, moos[i])) return i;
			}
		}
		return 0;
	}

	public MoveableOwnedObjectDelegate getSelection() {
		return selected;
	}

	public MoveableOwnedObjectDelegate getNextSelectionInList(MoveableOwnedObjectDelegate moo) throws ApplicationError {
		if (moo == null) return null;
        if (moos != null && moos.length > 1) {
            for (int i=0; i<moos.length; i++) {
                if (moos[i].equals(moo)) {
                	return _getNextInList(i+1, moo.getOwnerDelegate());
                }
            }
        }
        return null;
	}

	private MoveableOwnedObjectDelegate _getNextInList(int index, PlayerDelegate owner) throws ApplicationError {
		int i = index;
        while (i<moos.length) {
            if (moos[i].isOwnedBy(owner)) {
            	return moos[i];
            }
            i++;
        }
		i = 0;
        while (i<index) {
            if (moos[i].isOwnedBy(owner)) {
            	return moos[i];
            }
            i++;
        }
        return null;
	}

	public GalaxyMapMooScrollBar getScrollBar() {
		return scrollBar;
	}

	public boolean isVisibleInScrollPanel(MoveableOwnedObjectDelegate newSelection) {
		if (visibleContent == null) return false;
		for (int i = 0; i < visibleContent.length; i++) {
			if (visibleContent[i].contains(newSelection)) {
				return true;
			}
		}
		return false;
	}
	
	public GalaxyMapMooListItem isFullyVisibleInScrollPanel(MoveableOwnedObjectDelegate newSelection) { 
		if (visibleContent == null) return null;
		for (int i = 0; i < visibleContent.length; i++) {
			if (visibleContent[i].contains(newSelection)) {
				if  (visibleContent[i].getHeight() == panelHeight) return visibleContent[i];
			}
		}
		return null;
	}

	public boolean contains(NamedObjectDelegate moo) {
		return Comp.in(moo, moos);
	}

	
}

class TransportPlanetPair {
	public TransportPlanetPair(TransportDelegate transport, PlanetDelegate planet) {
		this.transport = transport;
		this.planet = planet;
	}
	public TransportDelegate transport;
	public PlanetDelegate planet;
}
