/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.util.Iterator;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;

import mot.client.actions.ActiveRectangle;
import mot.client.actions.ActiveRectangleList;
import mot.client.actions.IndexedSelectionPolicy;
import mot.client.alarms.AlarmSystem;
import mot.client.animation.GalaxyAnimation;
import mot.client.animation.GalaxyAnimationList;
import mot.client.animation.GalaxyAnimationRegister;
import mot.client.animation.GalaxyOrbitDescription;
import mot.client.bean.CraftGraphicsWrapper;
import mot.client.bean.GalaxyMapLabelSizeDetails;
import mot.client.bean.MoveableObjectLocation;
import mot.client.bean.MoveableObjectLocationFactory;
import mot.client.deligates.LabeledObjectDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.StellarObjectOrbitalDelegate;
import mot.client.deligates.TradeRouteDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.config.SelectionAnimation;
import mot.client.swing.window.common.MotConfirmationDialog;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowListener;
import mot.client.telemetry.Telemetry;
import mot.common.bean.Area;
import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.Location;
import mot.common.bean.Nebula;
import mot.common.bean.Rectangle;
import mot.common.bean.SectorInfluenceItem;
import mot.common.bean.SectorOwnerGrid;
import mot.common.bean.Settings;
import mot.common.bean.Size;
import mot.common.bean.StellarObjectGraphics;
import mot.common.designs.Orbital;
import mot.common.designs.ProjectDesign;
import mot.common.factory.GraphicsFactory;
import mot.common.factory.PlanetGraphicsFactory;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;
import mot.common.vo.NavigatorVO;
import mot.common.vo.OrbitVO;

public class GalaxyMapPanel extends JPanel implements GalaxyZoomedPanel {

	/**
	 * The amount of zoom before planets are displayed orbiting the sun
	 */
	public static final int ZOOM_TO_DISPLAY_PLANETS = 3;

    private static final int PLANET_TITLE_X_OFFSET_PER_CHAR = 3;
    private static final int PLANET_TEXT_GAP = 1;
    
    // The amount larger the suns orbit is to a planets orbit (space between the sun and a planet)
    private static final float SUN_ORBIT_FACTOR = 1.0f;
    // The amount smaller a planet is than a planet orbit
    private static final float PLANET_SIZE_FACTOR = 0.9f;
    // The amount smaller a sun is than a planet orbit
    private static final float SUN_SIZE_FACTOR = 1f;

    // The width and the height of the size of the image drawn on the screen for an orbital.
    private static final int ORBITAL_SIZE = 20;
	
	/**
	 * The current zoom the user has selected. A value of one implies no zoom
	 * one pixel in the galaxy is one square in the game and Solar systems are
	 * displayed as a simple star with craft icons around it. A value of more
	 * then DISPLAY_PLANETS causes each solar system to be display as a sun 
	 * orbited by planets and crafts a visible orbiting planets.
	 */
    private float zoomScale = GalaxyZoomSlider.DEFAULT_ZOOM;
    
    /**
     * This is the list of objects painted on the screen
     * Each object on the screen is displayed using an image contained in a rectangle.
     */
    private ActiveRectangleList objects;
    
    /**
     * These are other pieces of information built once then used on each paint request
     * to increase performance.
     */
    private Size galaxySize;
    
    /**
     * This is the nebula contained in this galaxy.
     */
    private Nebula [] nebulae;
    
    /**
     * This is a list of animations that are being displayed on the screen.
     */
    private GalaxyAnimationList animations = new GalaxyAnimationList();
    
    /**
     * This is an integer value that increments every paint so it can be used for
     * animation images.
     */
    private int animationStage = 0;
    
    /**
     * This is the font used to write text in this panel
     */
    private Font font;
    
    /**
     * The owner of this panel (the owner of the window that owns this panel)
     */
    private String owner;
    
    private JScrollPane scrollPane = null;

    private Settings settings;
    
    private WindowShowingNotification nowShowingNotification= null;

    /**
     * All the content being displayed will only be rebuilt if this flag is true.
     * Otherwise it will be updated.
     * The content must be rebuilt if the zoom scale is modified or a craft's orbit status
     * changes etc.
     */
    private boolean rebuild = true;

    /**
     * If this is true this map shows ALL stellar object names in the solar system with names.
     * If this is false this map only shows the names of stellar objects visited by the owner of the window.
     */
    private boolean showAllNames;
    
    /**
     * If this is true this map shows ALL crafts and fleets in the game.
     * If this is false this map only shows crafts and fleets that are in scan range of the owner of this window.
     */
    private boolean showAllMoos;

    
    private static Color orbitRingColor = PropertyManager.getMandatoryAsColorNoError("window.color.galaxy.map.orbit.ring");
	private static Color PROGRESS_BAR_DONE_COLOUR = new Color(10,100,10);
    
	private AlphaComposite labelAlphaComposite;	
	
	private String wayPointImage;


    public GalaxyMapPanel(GalaxyMapListener listener, boolean showAllNames, boolean showAllMoos) throws ApplicationError {
    	this.objects = new ActiveRectangleList(new IndexedSelectionPolicy());
    	this.showAllNames = showAllNames;
    	this.showAllMoos = showAllMoos;
    	
		scrollPane = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_NEVER , JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		settings = Settings.getInstance();
		
		addMouseListener(listener);
		addMouseMotionListener(listener);

    	scrollPane.getViewport().getView().addMouseMotionListener(listener);
    	labelAlphaComposite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.4f);
    	
    	wayPointImage = ImageConfigFactory.getImageForConfig("WayPoint");
    }
    
    public void paintComponent(Graphics g) {
        try {
            super.paintComponent(g); 
	        if (owner == null) {
	        	return;
	        }
	        
	        if (rebuild) {
	        	buildContent();
	        	rebuild = false;
	        	AlarmSystem.getInstance().notifyContentRebuilt();
	        } else {
	        	updateContent();
	        }
	        
	        long start = Logger.TRACE_ON ? System.currentTimeMillis() : 0;
	        
	        String starMap = ImageConfigFactory.getImageForConfig("GalaxyMap");
			if (starMap != null) {
				//ImageUtils.drawImage(this, (Graphics2D)g, starMap, 0, 0, getWidth(),getHeight());
				ImageUtils.tileImageNoScale(this, (Graphics2D)g, starMap, new Area(0,0,getWidth(),getHeight()));
			}
			
			if (nebulae != null) {
				for (int i = 0; i < nebulae.length; i++) {
					String nebulaImage = nebulae[i].getNedulaGraphics().getPicture(animationStage);
					Location center = nebulae[i].getLocation();
			        int x = (int)(center.getX()*zoomScale);
			        int y = (int)(center.getY()*zoomScale);
			        int diameter = (int)(nebulae[i].getDiameter()*zoomScale);
					ImageUtils._drawImageCenteredAt(this, (Graphics2D)g, nebulaImage, x, y, diameter, diameter);
				}
			}
			
        	if (nowShowingNotification != null) {
        		nowShowingNotification.nowShowingNotification();
        		nowShowingNotification = null;
        	}
        	
	    	if (font == null) {
	            font = g.getFont();
	            font = new Font(font.getName(), Font.BOLD, font.getSize());
	            g.setFont(font);
        	}
            g.setColor(Color.CYAN);
            drawContent((Graphics2D)g);
            
            if (Logger.TRACE_ON) {
            	long end = System.currentTimeMillis();
            	Logger.logDebug("PAINT ["+(end-start)+"]");
            }

        } catch (Exception e) {
            Logger.logError("MainWindow.painContent() Failed due to error ",e);
            Logger.flushLog();
			try {
				boolean r = MotConfirmationDialog.showInPopup(null, "Error "+e+"\nShutdown application?", "Error Dialog");
	            if (r) {
	                System.exit(0);
	            }
			} catch (Exception e1) { }
        }
        animationStage++;
    }    
    
    private void drawContent(Graphics2D graphics) throws ApplicationError, ValidationError {
    	if (Telemetry.ON) Telemetry.qtyDrawGalaxyMap++;
    	graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
    	graphics.setRenderingHint(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_QUALITY);
    	
        NamedObjectDelegate selection = getSelection();
    	UserPlayerDelegate owner = MainWindow.getOwner();

        // Draw grid and Zones of Control
        if (DeligateCacheManager.getSectorOwnerGrid() == null) return;
    	drawGrid(graphics);
    	
        // Only draw visible content 
		Rectangle visible = PanelUtils.tooRectangle(scrollPane.getViewport().getViewRect());
		
    	// First draw animations
		GalaxyAnimation [] gaa = animations.getAll();
    	if (gaa != null) {
    		for (int j = 0; j < gaa.length; j++) {
    			gaa[j].paint(graphics);
			}
    	}
    	
        // Now draw actual objects
    	Iterator i = objects.getAll();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			if (ar instanceof GalaxyMapMoo) {
				GalaxyMapMoo moo = (GalaxyMapMoo) ar;
				if (showAllMoos || moo.isVisible()) {
					draw(moo, selection, graphics);
				}
			} else if (ar instanceof GalaxyMapPlanet) {
				GalaxyMapPlanet planet = (GalaxyMapPlanet) ar;
				if (visible.intersects(ar)) {
					if (planet.isWaypoint()) {
						ImageUtils.drawImage(this, graphics, wayPointImage, planet.getLeft()-10, planet.getTop()-10, planet.getWidth()+20, planet.getHeight()+20);
					}
			        ImageUtils.drawImage(this, graphics, planet.getPicture(), planet.getLeft(), planet.getTop(), planet.getWidth(), planet.getHeight());
			        PlanetDelegate pd = planet.getPlanet();
					if (pd.getSolarSystem().getLastVisitedDate() >= 0 || showAllNames) {
			        	drawLabel(pd, planet.getLeft(), planet.getTop(), planet.getWidth(), planet.getHeight(), planet.getProject() , owner, graphics);
			    		Orbital orbital = pd.getOrbital();
			    		if (orbital != null) {
				        	drawOrbital(graphics, orbital, pd.getOwner(), ar);
			    		}
			    		if (pd.hasAsteroids()) {
			    			String asteroidRingImage = AnimationConfigFactory.getInstance().getAsteroidRingAnimationConfig().getActualImage();
					        ImageUtils.drawImage(this, graphics, asteroidRingImage, planet.getLeft()-3, planet.getTop()-3, planet.getWidth()+6, planet.getHeight()+6);
			    		}
			        }
			        if (planet.contains(selection)) {
			            String image = SelectionAnimation.getImageLarge();
			            ImageUtils.drawImage(this, graphics, image, planet.getLeft()-3, planet.getTop()-3, planet.getWidth()+6, planet.getHeight()+6);
			        }
				}
			} else if (ar instanceof GalaxyMapStellarObject) {
				if (visible.intersects(ar)) {
					GalaxyMapStellarObject so = (GalaxyMapStellarObject) ar;
					if (so.isWaypoint()) {
						ImageUtils.drawImage(this, graphics, wayPointImage, so.getLeft()-5, so.getTop()-5, so.getWidth()+10, so.getHeight()+10);
					}
					Location rotateAbout = so.getRotateAbout();
					if (rotateAbout != null) {
						ImageUtils.drawRotatedImage(this, graphics, so.getPicture(), so.getLeft(), so.getTop(), so.getWidth(), so.getHeight(), rotateAbout, animationStage % 360);
					} else {						
						ImageUtils.drawImage(this, graphics, so.getPicture(), so.getLeft(), so.getTop(), so.getWidth(), so.getHeight());
					}
			        StellarObjectDelegate sod = so.getStellarObject();
			        if (sod.getLastVisitedDate() >= 0 || showAllNames) {
			        	drawLabel(sod, so.getLeft(), so.getTop(), so.getWidth(), so.getHeight(), null, owner, graphics);		        	
			        	StellarObjectOrbitalDelegate outpost = sod.getStellarObjectOrbital();
			    		if (outpost != null) {
				        	drawOrbital(graphics, outpost.getOrbital(), outpost.getOwner(), ar);
			    		}
			        }
			        boolean zoomedIn = zoomScale >= ZOOM_TO_DISPLAY_PLANETS; 
					if (zoomedIn && sod instanceof SolarSystemDelegate && settings.isDrawOrbitRingsInGalaxyMap()) {
						drawOrbitRings(graphics, so, (SolarSystemDelegate)sod);
			        }
			        if (so.contains(selection)) {
			            String image = SelectionAnimation.getImage();
			            ImageUtils.drawImage(this, graphics, image, so.getLeft()-3, so.getTop()-3, so.getWidth()+6, so.getHeight()+6);
			        }
				}
			}
		}
		
		GalaxyMapMooList mooList = objects.getMooList();
		if (mooList != null) {
			mooList.render(this, graphics, visible);
		}
		objects.render(this, graphics, visible);
    }

	private void drawOrbital(Graphics2D graphics, Orbital orbital, PlayerDelegate owner, ActiveRectangle orbitedObject) throws ApplicationError {
		if (orbital == null || owner == null) return;
        String image = CraftGraphicsWrapper.getSmallImage(owner.getName(), orbital.getHullTech().getName(), owner.getColor());
        int x = orbitedObject.getLeft() + orbitedObject.getWidth()/2 - ORBITAL_SIZE/2;
        int y = orbitedObject.getTop() - ORBITAL_SIZE;
		ImageUtils.drawRotatedImage(this, graphics, image, x, y, ORBITAL_SIZE, ORBITAL_SIZE, null, (animationStage % 90) * 4);
	}
    
    private void drawOrbitRings(Graphics2D graphics, Rectangle rect, SolarSystemDelegate sod) {
//        if (sod.getName().equals("Ormiston")) {
//        	Logger.logDebug("*** ");
//        }
    	graphics.setColor(orbitRingColor);
	    int sunRadius = (int)(rect.getWidth()*SUN_ORBIT_FACTOR);
	    int planetRadius = calcPlanetRadius();
    	int qtyOrbits = GalaxyAnimationRegister.getQtyOrbitIndexes(sod.getName());
    	Location center = rect.getCenterLocation();
        for (int orbitIndex = 0; orbitIndex < qtyOrbits; orbitIndex++) {
    	    int planetOrbit = sunRadius + planetRadius * (orbitIndex+1);
    		int x = center.getX()-planetOrbit;
			int y = center.getY()-planetOrbit;
			int size = planetOrbit*2;
//            if (sod.getName().equals("Ormiston")) {
//            	Logger.logDebug("*** center "+x+" "+y+" "+size);
//            }    	
			//graphics.drawArc(x, y, size, size, 0, 360);
			graphics.drawOval(x, y, size, size);
		}
    }

    private void drawLabel(LabeledObjectDelegate object, int x, int y, int width, int height, GalaxyMapProject project, UserPlayerDelegate owner, Graphics2D graphics) throws ApplicationError {
    	if (Telemetry.ON) Telemetry.qtyDrawGalaxyLabels++;
        String labelText = object.getName();
    	GalaxyMapLabelSizeDetails labelSize = object.getGalaxyMapLabelSizeDetails(graphics);
    	int labelWidth = labelSize.getWidth();
    	int labelHeight= labelSize.getHeight();
    	int labelTop = y + height + PLANET_TEXT_GAP;
        int labelLeft = x + width/2 - labelWidth/2;
        int labelBottom = labelTop + labelHeight;
        int progressBarLeft = labelLeft-2;
        if (project != null) {
        	Size size = project.getImageSize();
        	project.setRectangle(labelLeft - size.getX()/2 - 10, labelTop, size.getX(), size.getY());
        	ImageUtils.drawImage(this, graphics, project.getImage(), project.getLeft(), project.getTop(), project.getWidth(), project.getHeight());
        	Composite old = graphics.getComposite();
        	float percentComplete = project.getPercentComplete();
        	int progressBarWidth1 = (int)((labelWidth+4) * percentComplete);
        	int progressBarWidth2 = (int)((labelWidth+4) * (1.0-percentComplete));
			graphics.setComposite(labelAlphaComposite);
			if (progressBarWidth1 > 0) {
				graphics.setColor(PROGRESS_BAR_DONE_COLOUR);
				//graphics.fillRect(progressBarLeft, labelTop-1, progressBarWidth1, labelHeight+2);
				graphics.fillRoundRect(progressBarLeft, labelTop-1, progressBarWidth1, labelHeight+2, 4, 4);
			}
			graphics.setColor(Color.BLACK);
        	//graphics.fillRect(progressBarLeft+progressBarWidth1, labelTop-1, progressBarWidth2, labelHeight+2);
        	graphics.fillRoundRect(progressBarLeft+progressBarWidth1, labelTop-1, progressBarWidth2, labelHeight+2, 4, 4);
        	graphics.setComposite(old);
        } else if (object instanceof PlanetDelegate && ((PlanetDelegate)object).isOwnedBy(owner) ){
        	Composite old = graphics.getComposite();
			graphics.setComposite(labelAlphaComposite);
			graphics.setColor(Color.BLACK);
        	graphics.fillRoundRect(progressBarLeft, labelTop-1, labelWidth+4, labelHeight+2, 4, 4);
        	graphics.setComposite(old);
        }
        Color color = Color.LIGHT_GRAY;
		if (object.hasOwner()) {
            color = object.getTextColor();
        }
        graphics.setColor(color);
		graphics.drawString(labelText, labelLeft, labelBottom-(int)labelHeight*0.2f);
    }

    private void draw(GalaxyMapMoo r, NamedObjectDelegate selection, Graphics2D graphics) throws ApplicationError {
    	if (Telemetry.ON) Telemetry.qtyDrawGalaxyMoo++;
    	MoveableOwnedObjectDelegate moo = r.getMostVisibleMoo();
        String picture = moo.getImage();
        if (picture == null) return;
        boolean sleeping = r.isSleeping();

        if (sleeping) {
        	ImageUtils.drawImageAlpha5(this, graphics, picture, r.getLeft(), r.getTop(), r.getWidth(), r.getHeight());
        } else {
        	ImageUtils.drawImage(this, graphics, picture, r.getLeft(), r.getTop(), r.getWidth(), r.getHeight());
        }
        
        if (moo != null && moo.equals(selection)) {
            String image = SelectionAnimation.getImage();
            ImageUtils.drawImage(this, graphics, image, r.getLeft()-3, r.getTop()-3, r.getWidth()+6, r.getHeight()+6);
        }
        
		int qty = r.getQtyMoos();
		GalaxyMapMooList mooList = objects.getMooList();
		if  (qty > 1 && (mooList == null || !mooList.contains(moo))) {
			ImageUtils.drawLabel(this, graphics, ""+qty, r.getRight()+2, r.getBottom()-5, Color.LIGHT_GRAY, false, 10);
        }
    }
    
    private NamedObjectDelegate getSelection() throws ApplicationError {
//      GalaxyWindowListener gwl = MainWindow.getInstance().getGalaxyMapListener();
//      MoveableOwnedObjectDeligate selection = gwl.getSelection();
    	MainWindowListener listener = MainWindow.getInstance().getActiveConfiguration().getListener();
    	if (listener instanceof GalaxyMapListener) {
    		GalaxyMapListener gml = (GalaxyMapListener) listener;
    		return gml.getSelection();
    	}
    	return null;
    }

	private void drawGrid(Graphics2D g) throws ApplicationError {
        Rectangle visible = PanelUtils.tooRectangle(scrollPane.getViewport().getViewRect());
        JViewport viewport = scrollPane.getViewport();
		int width = viewport.getWidth();
        int height = viewport.getHeight();
        SectorOwnerGrid sog = DeligateCacheManager.getSectorOwnerGrid();
		int qtyRows = sog.getQtyRows();
		int qtyCols = sog.getQtyCols();
		Dimension windowSize = getSize();
		double winXDim = Comp.min(windowSize.getWidth(), galaxySize.getX()*zoomScale);
		double winYDim = Comp.min(windowSize.getHeight(), galaxySize.getY()*zoomScale);
		int sectorWidth = (int)(winXDim/qtyCols);
		int sectorHeight = (int)(winYDim/qtyRows);
		
		int startX = (int)Math.floor(visible.getLeft()/(double)sectorWidth);
		int startY = (int)Math.floor(visible.getTop()/(double)sectorHeight);
		if (startX < 0) startX = 0;
		if (startY < 0) startY = 0;
		int endX = startX + (int)Math.ceil(width/(double)sectorWidth) + 1; 
		int endY = startY + (int)Math.ceil(height/(double)sectorHeight) + 1; 

	  	Composite old = g.getComposite();
	  	AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER,0.35f);
	  	g.setComposite(ac);
		for (int col = startX; col < endX && col < qtyCols; col++) {
			for (int row = startY; row < endY && row < qtyRows; row++) {
				SectorInfluenceItem owner = sog.getSectorOwner(row, col);
				if (owner != null) {
					PlayerDelegate player = DeligateCacheManager.getPlayer(owner.getPlayer());
					if (player != null) {
						Color ownerColor = player.getColor().getSectorColor();
						g.setColor(ownerColor);
						int x = col*sectorWidth;
						int y = row*sectorHeight;
						g.fillRect(x, y, sectorWidth, sectorHeight);
					}
				}
			}		
		}
		g.setColor(Color.GRAY);
		for (int col = startX; col < endX && col < qtyCols; col++) {
			int x = col*sectorWidth;
			g.drawLine(x, 0, x, (int)winYDim);
		}
		for (int row = startY; row < endY && row < qtyRows; row++) {
			int y = row*sectorHeight;
			g.drawLine(0, y, (int)winXDim, y);
		}
	  	g.setComposite(old);
	}

	



    private void updateContent() throws ApplicationError, ValidationError {
		Iterator i = objects.getAll();
		while (i.hasNext()) {
			ActiveRectangle ar = (ActiveRectangle) i.next();
			ar.update();
		}
		GalaxyAnimation [] gaa = animations.getAll();
    	if (gaa != null) {
    		for (int j = 0; j < gaa.length; j++) {
    			if (gaa[j].isComplete()) {
    				animations.remove(gaa[j]);
    			} else {
    				gaa[j].tick();
    			}
			}
    	}
    }    

    public void refresh() throws ApplicationError {
    	buildContent();
    }
    
    private void buildContent() throws ApplicationError {
    	galaxySize = DeligateCacheManager.getGalaxyDetails().getSize();
    	nebulae = DeligateCacheManager.getGalaxyDetails().getNebulaeAsArray();
    	
    	objects.removeAll();
		animations.removeAll();

		// build all the planets / space stations etc
        ServerProxy proxy = ProxyStore.getInstance();
        //if (Logger.isDebug()) Logger.logDebug("REBUILD Galaxy Map "+Comp.toString(proxy.getMoos(showAllMoos)));
		StellarObjectDelegate[] sos = DeligateCacheManager.getStellarObjects();
        for (int i=0; sos != null && i<sos.length; i++) {
       		buildStellarObject(sos[i]);
        }
        
        // build crafts / fleets
        MoveableOwnedObjectDelegate[] allMoos = proxy.getMoos(showAllMoos);
        MoveableObjectLocation [] mols = MoveableObjectLocationFactory.sortMoosByLocation(allMoos, true, zoomScale >= ZOOM_TO_DISPLAY_PLANETS);
        if (mols != null) {
            for (int i=0; mols != null && i<mols.length; i++) {
                buildMoveableOwnedObject(mols[i], true);
            }
        }
        
        buildMiningTrails();
        buildTradeRoutes();
    }

	private void buildStellarObject(StellarObjectDelegate so) throws ApplicationError {
		boolean zoomedIn = zoomScale >= ZOOM_TO_DISPLAY_PLANETS;
		StellarObjectGraphics starGraphics = so.getGraphics(zoomedIn);
        Size starGrahicsSize = starGraphics.getSize();
        
        int width;
        int height;
        if (zoomedIn) {
	        width = height = (int)(calcPlanetRadius()*SUN_SIZE_FACTOR);
        } else {
	        width = (int)(starGrahicsSize.getX()*Comp.max(1.0,zoomScale));
	        height = (int)(starGrahicsSize.getY()*Comp.max(1.0,zoomScale));
        }

        int x = (int)(so.getLocation().getX()*zoomScale);
        int y = (int)(so.getLocation().getY()*zoomScale);
        
        Location rotateAbout = null;
        Location ra = starGraphics.getRotateAbout();
		if (ra != null) {
	        width = (int)(starGrahicsSize.getX());
	        height = (int)(starGrahicsSize.getY());
	        int xo = (int)(ra.getX());
	        int yo = (int)(ra.getY());
			if (!zoomedIn) {
				width = width/4;
				height = height/4;
				xo = xo/4;
				yo = yo/4;
			}
			rotateAbout = new Location(x - width/2 + xo, y - height/2 + yo);
		}
        
    	boolean isWaypoint = so.isWayPoint();

        GalaxyMapStellarObject ar = new GalaxyMapStellarObject(so, x, y, width, height, !zoomedIn, zoomedIn, starGraphics, rotateAbout, animationStage, isWaypoint);
        objects.add(ar);
        
    	boolean visible = so.isVisible();
		ar.setVisible(visible);
    	
        if (zoomScale >= ZOOM_TO_DISPLAY_PLANETS && so instanceof SolarSystemDelegate) {
        	SolarSystemDelegate ss = (SolarSystemDelegate) so;
        	PlanetDelegate [] planets = ss.getPlanets();
        	if (planets != null) {
        		for (int i=0; i<planets.length; i++) {
        			buildPlanet(x, y, (int)(width*SUN_ORBIT_FACTOR), planets[i], visible);
        		}
        	}
        }
    }
	
	/**
	 * The average dimension of a planet used when rendering on the screen.
	 */
	private static final double MEAN_PLANET_SIZE = 20;
   
    private void buildPlanet(int x, int y, int sunRadius, PlanetDelegate planet, boolean visible) throws ApplicationError {
        GalaxyOrbitDescription od = GalaxyAnimationRegister.getOrbitFor(planet.getName(), planet.getSolarSystem().getName());
        int orbitIndex = od.getOrbitIndex();
        int planetRadius = calcPlanetRadius();
        int planetOrbit = sunRadius + planetRadius * orbitIndex;
        int size = planet.getImageSize();
        int scale = planet.getSize() / 5;
		size = (int) (MEAN_PLANET_SIZE + scale);
    	OrbitVO fleetWayPoint = MainWindow.getOwner().getFleetWayPoint();
    	boolean isWaypoint = fleetWayPoint != null && Comp.same(fleetWayPoint.getPlanet(), planet.getName());

		//int maxPlanetSize = (int)(planetRadius * PLANET_SIZE_FACTOR);
		//if (size > maxPlanetSize) size = maxPlanetSize;
    	GalaxyMapPlanet ar = new GalaxyMapPlanet(planet, x, y, od, planetOrbit, size, isWaypoint);
        objects.add(ar);
        ar.setVisible(visible);
        if (planet.isViewable()) {        	
        	ProjectDesign project = planet.getProjectDesign();
        	if (project != null) {
        		GalaxyMapProject par = new GalaxyMapProject(ar, project);
        		ar.setProject(par);
        		objects.add(par);
        	}
        }
    }
    
    private void buildMiningTrails() throws ApplicationError {
    	for (Iterator i = objects.getAll(); i.hasNext();) {
    		ActiveRectangle ar = (ActiveRectangle) i.next();
    		if (ar instanceof GalaxyMapPlanet) {
    			GalaxyMapPlanet gp = (GalaxyMapPlanet) ar;
    			if (gp.getPlanet().hasMiningColony()) {
    				GalaxyMapPlanet minePlanet = findGalaxyMapPlanet(gp.getPlanet().getMiningColonyDelegate().getName());
    				if (minePlanet != null) {
	    				TrailAnimationResourceMine anim = new TrailAnimationResourceMine(minePlanet, gp, this);
	    				animations.add(anim);
    				} else {
    					Logger.logWarning("No Planet Active Rectangle for "+gp.getPlanet().getName());
    				}
    			}
    		}
		}
    }
    
    private void buildTradeRoutes() throws ApplicationError {
    	TradeRouteDelegate[] tradeRoutes = DeligateCacheManager.getTradeRoutes();if (tradeRoutes != null) {
    		for (int i = 0; i < tradeRoutes.length; i++) {
    			GalaxyMapStellarObject ss1 = findGalaxyMapStellarObject(tradeRoutes[i].getSolarSystem1Name());
    			GalaxyMapStellarObject ss2 = findGalaxyMapStellarObject(tradeRoutes[i].getSolarSystem2Name());
    			if (ss1 != null && ss2 != null) {
        			TrailAnimationTradeRoute anim1 = new TrailAnimationTradeRoute(tradeRoutes[i], ss1, ss2, this);
    				animations.add(anim1);
    				TrailAnimationTradeRoute anim2 = new TrailAnimationTradeRoute(tradeRoutes[i], ss2, ss1, this);
    				animations.add(anim2);
    			}
			}
    	}
    }
    
	public ActiveRectangle getDraggableObjectAt(int x, int y) {
		GalaxyMapMooList mooList = objects.getMooList();
		if (mooList != null) {
			GalaxyMapMooScrollBar sb = mooList.getScrollBar();
			if (sb != null && sb.contains(x, y)) {
				return sb;
			}
		}
		return null;
	}

    private ActiveRectangle findActiveRectangleFor(String name, String stellarObjectName) {
    	ActiveRectangle r = null;
    	for (Iterator i = objects.getAll(); i.hasNext();) {
    		ActiveRectangle ar = (ActiveRectangle) i.next();
    		if (name != null && ar instanceof GalaxyMapPlanet) {
    			GalaxyMapPlanet gp = (GalaxyMapPlanet) ar;
    			if (Comp.same(gp.getPlanet().getName(), name)) {
    				return ar;
    			}
    		}
    		if (ar instanceof GalaxyMapStellarObject) {
    			GalaxyMapStellarObject gs = (GalaxyMapStellarObject) ar;
    			if (Comp.same(gs.getStellarObject().getName(), stellarObjectName)) {
    				r = gs;
    			}
    		}
		}
    	return r;
    }

    private GalaxyMapPlanet findGalaxyMapPlanet(String planet) {
    	for (Iterator i = objects.getAll(); i.hasNext();) {
    		ActiveRectangle ar = (ActiveRectangle) i.next();
    		if (ar instanceof GalaxyMapPlanet) {
    			GalaxyMapPlanet gp = (GalaxyMapPlanet) ar;
    			if (Comp.same(gp.getPlanet().getName(), planet)) {
    				return gp;
    			}
    		}
		}
    	return null;
    }

    private GalaxyMapStellarObject findGalaxyMapStellarObject(String name) {
    	for (Iterator i = objects.getAll(); i.hasNext();) {
    		ActiveRectangle ar = (ActiveRectangle) i.next();
    		if (ar instanceof GalaxyMapStellarObject) {
    			GalaxyMapStellarObject gso = (GalaxyMapStellarObject) ar;
    			if (Comp.same(gso.getStellarObject().getName(), name)) {
    				return gso;
    			}
    		}
		}
    	return null;
    }

    private static int MAX_PLANET_RADIUS;

    static {
    	try {
			MAX_PLANET_RADIUS = PlanetGraphicsFactory.getMaxPlanetSize()/2;
		} catch (ApplicationError e) {
			Logger.logError("Failed to load Planet Config", e);
			MAX_PLANET_RADIUS = 50;
		}
    }
    
    /**
     * Calculate the radius from the sun for each planet's orbit
     */
	private int calcPlanetRadius() {
		float radius = (int)(zoomScale * (GalaxyDefinitions.MIN_DISTANCE_BETWEEN_SOLAR_SYSTEMS/4f) / (GalaxyOrbitDescription.MAX_PLANET_ORBITS+1));
		if (radius > MAX_PLANET_RADIUS) radius = MAX_PLANET_RADIUS;
		return (int)radius;
	}
    
    private void buildMoveableOwnedObject(MoveableObjectLocation mol, boolean visible) throws ApplicationError {
        GalaxyWindowListener gwl = MainWindow.getInstance().getGalaxyMapListener();
        MoveableOwnedObjectDelegate selection = gwl.getSelectedMoo();
        
    	OrbitVO orbit = mol.getOrbit();
    	NavigatorVO navigator = mol.getNavigator();
		ActiveRectangle orbitedObject = null;
		if (orbit == null) {
	        int x = (int)(mol.getLocation().getX()*zoomScale)-GalaxyMapMoo.CRAFT_WIDTH/2;
	        int y = (int)(mol.getLocation().getY()*zoomScale)-GalaxyMapMoo.CRAFT_HEIGHT/2;
	        MoveableOwnedObjectDelegate[] moos = mol.getMoos();
	        MoveableOwnedObjectDelegate visibleMoo = null;
	        if (selection != null && mol.contains(selection)) {
	        	visibleMoo = selection;
	        } else {
	        	visibleMoo = mol.getMostVisible(); 
	        }
	        GalaxyMapMoo ar = new GalaxyMapMoo(mol, moos, visibleMoo, x, y, visible, orbitedObject);
	        objects.add(ar);
	    	buildWarpTrails(ar);
		} else {
			if (zoomScale >= ZOOM_TO_DISPLAY_PLANETS) {
				String planet = orbit.getPlanet();
				if (planet != null) {
					// Orbit the selected planet
					orbitedObject = objects.get(planet);
				} else {
					// Solar System does not have any planets - orbit sun
					orbitedObject = objects.get(orbit.getStellarObject());
				}
			} else {
				// orbit sun / space station
				orbitedObject = objects.get(orbit.getStellarObject());
			}
			if (navigator == null) {
				MoveableOwnedObjectDelegate[] playerMoos = mol.getPlayerMoos();
				int offset = 0;
				if (playerMoos != null) {
			        MoveableOwnedObjectDelegate visibleMoo = null;
			        if (selection != null && mol.playerListContains(selection)) {
			        	visibleMoo = selection;
			        } else {
			        	visibleMoo = mol.getPlayerMooMostVisible(); 
			        }
			        GalaxyMapMoo ar = new GalaxyMapMoo(mol, playerMoos, visibleMoo, 0, 0, visible, orbitedObject);
			        objects.add(ar);
			        offset = GalaxyMapMoo.CRAFT_WIDTH;
				}
				MoveableOwnedObjectDelegate[] otherMoos = mol.getOtherMoos();
				if (otherMoos != null) {
			        MoveableOwnedObjectDelegate visibleMoo = null;
			        if (selection != null && mol.otherListContains(selection)) {
			        	visibleMoo = selection;
			        } else {
			        	visibleMoo = mol.getOtherMooMostVisible(); 
			        }
			        GalaxyMapMoo ar = new GalaxyMapMoo(mol, otherMoos, visibleMoo, 0, 0, visible, orbitedObject, offset);
			        objects.add(ar);
				}
			} else {
		        MoveableOwnedObjectDelegate[] moos = mol.getMoos();
		        MoveableOwnedObjectDelegate visibleMoo = null;
		        if (selection != null && mol.contains(selection)) {
		        	visibleMoo = selection;
		        } else {
		        	visibleMoo = mol.getMostVisible(); 
		        }
		        GalaxyMapMoo ar = new GalaxyMapMoo(mol, moos, visibleMoo, 0, 0, visible, orbitedObject);
		        objects.add(ar);
		    	buildWarpTrails(ar);
			}
		}
        
    }

	private void buildWarpTrails(GalaxyMapMoo ar) throws ApplicationError {
		MoveableOwnedObjectDelegate[] moos = ar.getMoos();
		if (moos == null) return;
		for (int i = 0; i < moos.length; i++) {
			NavigatorVO navigator = moos[i].getNavigator();
			if (navigator != null) {
				String planetName = navigator.getDestination().getPlanet();
				String stellarObjectName = navigator.getDestination().getStellarObject();
				ActiveRectangle dest = findActiveRectangleFor(planetName, stellarObjectName);
				if (!animations.containsTrailAnimation(ar, dest)) {
					TrailAnimationWarp anim = new TrailAnimationWarp(ar, dest, this);
					//anim.tick();
					animations.add(anim);
				}
			}
		}
	}
    
	public MoveableOwnedObjectDelegate [] getSelectionList(MoveableOwnedObjectDelegate craft) {
		GalaxyMapMoo moo = objects.getMoo(craft);
		if (moo != null) {
			return moo.getAllMoos();
		}
		return null;
	}

    /**
     * Create a list on the galaxy map of all MOOS at this location for the user to select a single object 
     * (from the many) to command. 
     */
	public void setSelection(MoveableOwnedObjectDelegate selection) throws ApplicationError {
		if (selection == null) {
			objects.clearSelection();
			return;
		}
    	
    	GalaxyMapMoo moo = objects.getMoo(selection);
    	if (moo == null) return;
    	
    	MoveableOwnedObjectDelegate [] listContent = moo.getAllMoos();
    	
    	GalaxyMapMooList mooList = objects.getMooList();
		if (mooList != null && mooList.isVisibleInScrollPanel(selection)) {
			mooList.setSelection(selection);
		} else {
			ActiveRectangle selectedAR = objects.get(selection);
			
			if (selectedAR == null) return;

			int left = 0;
			int top = 0;
			
			int panelWidth = GraphicsFactory.getRaceGraphicsList().getBiggestCraftSize() + 2;			
			int panelHeight = GraphicsFactory.getRaceGraphicsList().getBiggestCraftSize() + GalaxyMapMooList.CRAFT_NAME_HEIGHT + 2;			
			Rectangle visible = PanelUtils.tooRectangle(scrollPane.getViewport().getViewRect());
			
			if (selectedAR.getRight() > visible.getRight() - (panelWidth+120)) {
				// Close to right hand side of screen - must use left
				left = selectedAR.getLeft()-panelWidth-10;
			} else if (selectedAR.getLeft() < visible.getLeft() + (panelWidth+120)) {
				// Close to left hand side of screen - must use right
				left = selectedAR.getRight()+5;
			} else if (moo.getXDistanceFromOrbitedObject() < 0) {
				// Can use either side - but will use left hand side because orbited planet is on the right
				left = selectedAR.getLeft()-panelWidth-10;
			} else {
				// Can use either side - but will use right hand side because orbited planet is on the left
				left = selectedAR.getRight()+5;
			}
			
			boolean originIsTop = true;
			int heightRequired = Comp.min(listContent.length, GalaxyMapMooList.MAX_DISPLAID_IN_LIST)*panelHeight;
			if (selectedAR.getTop() + heightRequired < visible.getBottom()) {
				// top or middle
				top = selectedAR.getTop();
			} else {
				// bottom
				top = selectedAR.getTop() - heightRequired;
				originIsTop = false;
			}
			
			objects.createMooList(listContent, selection, selectedAR, left, top, panelWidth, panelHeight, originIsTop);
		}
    }
    
	public void setSelection(PlanetDelegate selection) throws ApplicationError {
		if (selection == null) {
			objects.clearSelection();
			return;
		}
    	
    	GalaxyMapPlanet planet= objects.getPlanet(selection);
    	if (planet == null) return;
    	
		int left = 0;
		int buttonSize = GalaxyMapMooList.getButtonSize();
		Rectangle visible = PanelUtils.tooRectangle(scrollPane.getViewport().getViewRect());
		
		if (planet.getRight() > visible.getRight() - ((buttonSize*3) + planet.getWidth() + 10)) {
			// Close to right hand side of screen - must use left
			left = planet.getLeft()-buttonSize-10;
		} else {
			// Close to left hand side of screen - must use right
			left = planet.getRight()+5;
		}
		
		int top = planet.getTop() + planet.getHeight() /2 - buttonSize /2;
		
		objects.createPlanetButtons(selection, planet, left, top);
    }
    
	public void setSelection(StellarObjectDelegate selection) throws ApplicationError {
		if (selection == null) {
			objects.clearSelection();
			return;
		}
    	
    	GalaxyMapStellarObject so= objects.getStellarObject(selection);
    	if (so == null) return;
    	
		int left = 0;
		int buttonSize = GalaxyMapMooList.getButtonSize();
		Rectangle visible = PanelUtils.tooRectangle(scrollPane.getViewport().getViewRect());
		
		if (so.getRight() > visible.getRight() - (buttonSize*3)) {
			// Close to right hand side of screen - must use left
			left = so.getLeft()-buttonSize-10;
		} else {
			// Close to left hand side of screen - must use right
			left = so.getRight()+5;
		}
		
		int top = so.getTop() + so.getHeight() /2 - buttonSize /2;
		
		objects.createStellarObjectButtons(selection, so, left, top);
    }
    
    /**
     * The contents in the MOO list has changed - any objects that no longer exist should be removed 
     * and any additional objects at this location should be added.
     * NB - This is called after a command on a MOO has been completed
     * @throws ApplicationError 
     */
    public void updateMooList(MoveableOwnedObjectDelegate newSelection) throws ApplicationError {
    	objects.updateMooList(newSelection);
    	rebuildContent();
    }
    
    public void updateSelection() throws ApplicationError {
    	NamedObjectDelegate selection = getSelection();
    	if (selection instanceof PlanetDelegate) setSelection((PlanetDelegate)selection);
    	if (selection instanceof StellarObjectDelegate) setSelection((StellarObjectDelegate)selection);
    	if (selection instanceof MoveableOwnedObjectDelegate) updateMooList((MoveableOwnedObjectDelegate)selection);
    }

    public boolean isShowingMooListOrButtons() {
    	return objects.getMooList() != null || objects.getButtons() != null;
    }
    
    public boolean isShowingMooListWithTargetDestination() throws ApplicationError {
    	GalaxyMapMooList mooList = objects.getMooList();
		if (mooList != null) {
			MoveableOwnedObjectDelegate selection = mooList.getSelection();
			if (selection != null) {
				return selection.hasNavigator();
			}
		} 
		return false;
    }
    

    public Rectangle getViewArea() {
        java.awt.Rectangle visibleRect = scrollPane.getViewport().getViewRect();
        int x = (int)(visibleRect.x/zoomScale);
        int y = (int)(visibleRect.y/zoomScale);
        int width = (int)(visibleRect.width/zoomScale);
        int height = (int)(visibleRect.height/zoomScale);
        return new Rectangle(x,y,width,height);
    }
    
    public JScrollPane getScrollPane() {
    	return scrollPane;
    }

	public ActiveRectangleList getObjects() {
		return objects;
	}

	public float getZoomScale() {
		return zoomScale;
	}
	
	public void setZoomScale(float value) {
		this.zoomScale = value;
	}

	public void setOwner(String owner) {
		this.owner = owner;
	}

	public void rebuildContent() {
		this.rebuild = true;
	}

	public WindowShowingNotification getNowShowingNotification() {
		return nowShowingNotification;
	}

	public void setNowShowingNotification(WindowShowingNotification nowShowingNotification) {
		this.nowShowingNotification = nowShowingNotification;
	}

	public GalaxyAnimationList getAnimations() {
		return animations;
	}

}

