/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import mot.client.animation.GalaxyOrbitDescription;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowConfig;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.LocationWithMemento;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;

public class GalaxyMapPlanet extends GalaxyMapDestination {

	/**
	 * The planet that has been drawn on the panel
	 */
	private PlanetDelegate planet;
	
	private GalaxyOrbitDescription od;
	
	/**
	 * Is the stellar object visible to player owner.
	 * Returns the same value as - pd.isVisible(owner)
	 */
	private boolean visible;

	/**
	 * This is the location of the solar system containing this planet. This
	 * location must be modified because planets are animated as orbiting the
	 * solar system. The actual location this planet is drawn is calculated as
	 * this location shifted out using the radius stored in planetOrbit by an 
	 * angle dependent on the animation cycle.
	 */
	private LocationWithMemento location;
	
	/**
	 * This is the distance from the location that this planet must be actually drawn.
	 * This value is the radius (distance from the centre).
	 */
	private int planetOrbit;
	
	/**
	 * The project associated with this planet (if any).
	 */
	private GalaxyMapProject project;

	private boolean isWaypoint;
	
    public GalaxyMapPlanet(PlanetDelegate planet, int x, int y, GalaxyOrbitDescription od, int planetOrbit, int size, boolean isWaypoint) throws ApplicationError {
		super(x, y, size, size);
		this.planet = planet;
		this.od = od;
		this.location = new LocationWithMemento(x, y);
		this.planetOrbit = planetOrbit;
		this.visible = false;
		this.isWaypoint = isWaypoint;
		update();
    }
    
	public void click() throws ApplicationError, ValidationError {
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateClick(this);
    	}
		MainWindowConfig config = MainWindow.getInstance().getActiveConfiguration();
		config.getListener().click(planet);
	}

	public void doubleClick() throws ApplicationError, ValidationError {
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateDoubleClick(this);
    	}
		MainWindowConfig config = MainWindow.getInstance().getActiveConfiguration();
		config.getListener().doubleClick(planet, visible);
	}

	public boolean contains(NamedObjectDelegate o) {
		if (o == null) return false;
		return Comp.same(o.getName(), planet.getName());
	}

	public boolean contains(String name) {
		return Comp.same(name, planet.getName());
	}

	public PlanetDelegate getPlanet() {
		return planet;
	}

	public NamedObjectDelegate getContents() {
		return planet;
	}

	public String getPicture() throws ApplicationError {
		return planet.getImageForPlanet();
	}

	public boolean isVisible() {
		return visible;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	/**
	 * Active rectangle use the observer pattern. This method is called when the moo this
	 * observer is watching has changed.
	 */
	public void update() throws ApplicationError {
		location.restore();
        location.shiftInDirectionWithDouble(od.getRotation(), planetOrbit);
        setPosition(location);
	}
	
	public String toString() {
		return planet.getName();
	}
	
	public String getName() {
		return planet.getName();
	}

	public GalaxyMapProject getProject() {
		return project;
	}

	public void setProject(GalaxyMapProject project) {
		this.project = project;
	}

	public boolean isWaypoint() {
		return isWaypoint;
	}

	public void setWaypoint(boolean isWaypoint) {
		this.isWaypoint = isWaypoint;
	}

}
