/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import mot.client.actions.ActiveRectangle;
import mot.client.bean.CraftGraphicsWrapper;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.swing.window.main.MainWindow;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.Size;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ProjectDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.TransportDesign;
import mot.common.designs.WeaponDesign;
import mot.common.technology.Hull;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;

public class GalaxyMapProject extends ActiveRectangle {

	/**
	 * The planet this project belongs too.
	 */
	private GalaxyMapPlanet planet;
	
	/**
	 * The project this planet is building.
	 */
	private ProjectDesign project;
	
	private String image;
	private float percentComplete;
	
	private static final String IMAGE_WEAPON = "images/decorators/weapon_decorator.png";
	private static final String IMAGE_SHIELD = "images/decorators/shield_decorator.png";
	private static final String IMAGE_SCANNER = "images/decorators/scanner_decorator.png";
	private static final String IMAGE_OTHER = "images/decorators/busy_decorator.png";
	
    public GalaxyMapProject(GalaxyMapPlanet planet, ProjectDesign project) throws ApplicationError {
		super(0, 0, 0, 0);
		this.planet = planet;
		this.project = project;
		update();
    }
    
	public void click() throws ApplicationError, ValidationError {
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateClick(this);
    	}
	}

	public void doubleClick() throws ApplicationError, ValidationError {
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateDoubleClick(this);
    	}
        if (planet != null) {
        	MainWindow.getInstance().getGalaxyMapConfiguration().getListener().openSelectProjectWindow(planet.getPlanet());
        }
	}

	/**
	 * Active rectangle use the observer pattern. This method is called when the project this
	 * observer is watching has changed.
	 */
	public void update() throws ApplicationError {
		image = _getImage(project);
		PlanetDelegate p = planet.getPlanet();
		if (project != null && p != null) {
			PlayerDelegate owner = p.getOwner();
			int cost = p.getCostToBuild();
			percentComplete = (float) (p.getProduction() / (double)cost);
			if (percentComplete < 0.0) percentComplete = 0.0f;
			else if (percentComplete > 1.0) percentComplete = 1.0f;
		} else {
			percentComplete = 0.0f;
		}

	}

	public boolean contains(NamedObjectDelegate o) {
		if (planet == null) return false;
		return Comp.same(o, planet.getPlanet());
	}

	public boolean contains(String name) {
		if (name == null || planet == null) return false;
		return Comp.same(name, planet.getPlanet().getName());
	}

	public NamedObjectDelegate getContents() {
		return null;
	}

	private static Size size = new Size(12,12);
	public Size getImageSize() {
		return size;
	}

	public String getImage() {
		return image;
	}
	
	public static String _getImage(ProjectDesign project) throws ApplicationError {
		if (project instanceof WeaponDesign) {
			return IMAGE_WEAPON;
		} else if (project instanceof ScannerDesign) {
			return IMAGE_SCANNER;
		} else if (project instanceof ShieldDesign) {
			return IMAGE_SHIELD;
		} else if (project instanceof CraftDesign) {
			Hull hull = ((CraftDesign)project).getHullTech();
			String image = CraftGraphicsWrapper.getImageThumbNoError(hull.getName());
			return image;
		} else if (project instanceof CraftUpgradeDesign) {
			Hull hull = ((CraftUpgradeDesign)project).getDesign().getHullTech();
			String image = CraftGraphicsWrapper.getImageThumbNoError(hull.getName());
			return image;
		} else if (project instanceof TransportDesign) {
			String image = CraftGraphicsWrapper.getImageThumbNoError("Transport");
			return image;
		} else if (project instanceof BattleStationDesign) {
			Hull hull = ((BattleStationDesign)project).getBattleStation();
			String image = CraftGraphicsWrapper.getImageThumbNoError(hull.getName());
			return image;
		} else if (project instanceof SpaceStationDesign) {
			Hull hull = ((SpaceStationDesign)project).getSpaceStation();
			String image = CraftGraphicsWrapper.getImageThumbNoError(hull.getName());
			return image;
		} else if (project instanceof OutpostDesign) {
			Hull hull = ((OutpostDesign)project).getOutpost();
			String image = CraftGraphicsWrapper.getImageThumbNoError(hull.getName());
			return image;
		} else {
			return IMAGE_OTHER;
		}
	}

	public float getPercentComplete() throws ApplicationError {
		return percentComplete;
	}


}
