/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowConfig;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.Location;
import mot.common.bean.StellarObjectGraphics;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;

public class GalaxyMapStellarObject extends GalaxyMapDestination {

	private StellarObjectDelegate so;
	
	/**
	 * Is the stellar object visible to player owner.
	 * Returns the same value as - so.isVisible(owner)
	 */
	private boolean visible;
	
	/**
	 * True if when a user clicks on this object to send a craft here it should 
	 * automatically select a planetary orbit. 
	 */
	private boolean autoselectPlanetOrbit;
	
	/**
	 * The graphics used to display this object
	 */
	private StellarObjectGraphics starGraphics;
	
	/**
	 * True if this stellar object should used the zoomed in image to display it.
	 * When zoomed in a sun is displayed while when zoomed out a star is displayed!
	 */
	private boolean zoomedIn;
	
	private int animationStage;
	
	/**
	 * If this stellar object is to animated through rotation this is the centre of the rotation.
	 */
	private Location rotateAbout;
	private int rotateDelta;

	private boolean isWaypoint;
	
	public GalaxyMapStellarObject(StellarObjectDelegate so, int x, int y, int width, int height, boolean autoselectPlanetOrbit, boolean zoomedIn, StellarObjectGraphics starGraphics, Location rotateAbout, int animationStage, boolean isWaypoint) throws ApplicationError {
		super(x, y, width, height);
		this.so = so;
		this.visible = false;
		this.autoselectPlanetOrbit = autoselectPlanetOrbit;
		this.zoomedIn = zoomedIn;
		this.animationStage = animationStage;
		this.starGraphics = starGraphics;
		this.rotateAbout = rotateAbout;
		this.isWaypoint = isWaypoint;
		
		if (rotateAbout != null) {
			rotateDelta = 1;
		}
	}

	public void click() throws ApplicationError, ValidationError {
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateClick(this);
    	}
		MainWindowConfig config = MainWindow.getInstance().getActiveConfiguration();
		config.getListener().click(so, autoselectPlanetOrbit);
	}

	public void doubleClick() throws ApplicationError, ValidationError {
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateDoubleClick(this);
    	}
		MainWindowConfig config = MainWindow.getInstance().getActiveConfiguration();
		config.getListener().doubleClick(so);
	}

	public boolean contains(NamedObjectDelegate o) {
		if (o == null) return false;
		return Comp.same(o.getName(), so.getName());
	}

	public boolean contains(String name) {
		return Comp.same(name, so.getName());
	}


	public StellarObjectDelegate getStellarObject() {
		return so;
	}

	public NamedObjectDelegate getContents() {
		return so;
	}

	public String getName() {
		if (so != null) {
			return so.getName();
		} else {			
			return null;
		}
	}

	public boolean isVisible() {
		return visible;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	public String getPicture() {
		String picture = starGraphics.getPicture(animationStage);
        return picture;
	}
	
	public StellarObjectGraphics getStellarObjectGraphics() {
		return starGraphics;
	}


	/**
	 * Active rectangle use the observer pattern. This method is called when the moo this
	 * observer is watching has changed.
	 */
	public void update() throws ApplicationError {
		animationStage++;
	}
	
	public String toString() {
		return so.getName();
	}

	public Location getRotateAbout() {
		return rotateAbout;
	}

	public int getRotateDelta() {
		return rotateDelta;
	}

	public boolean isWaypoint() {
		return isWaypoint;
	}

	public void setWaypoint(boolean isWaypoint) {
		this.isWaypoint = isWaypoint;
	}

}
