/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Dimension;

import javax.swing.JDesktopPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.alarms.AlarmSystem;
import mot.client.deligates.PlayerDelegate;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowConfig;
import mot.client.swing.window.main.MainWindowView;
import mot.common.util.ApplicationError;

/**
 * The main window is used to display the galaxy map as well as the battle window (to fight
 * a battle at the end of a turn). Therefore the MainWindow has pluggable contents. Each 
 * type of content is called a configuration. This class represents a configuration on the
 * main window. Each configuration consists of;
 * 		- A View : Menus and control laid out in panels
 * 		- A Listener : Definitions of actions and current state
 * 
 * To switch from one configuration to another the old configuration must be de-activated
 * and then the new configuration activated (using the methods defined on the configuration).
 * The de-activation removes the menus, components and key listeners. While the activate adds
 * the menus, components and key listeners.
 * 
 * After starting a game you must set the owner of a galaxy window configuration. The contents
 * of the galaxy map will NOT be drawn until the owner is set!
 */
public class GalaxyWindowConfig implements MainWindowConfig {

	private static final String NAME = "GalaxyWindowConfig";
	
	/**
	 * This is the listener of this configuration.
	 */
	private GalaxyWindowListener listener;
	
	/**
	 * This is the view of this configuration.
	 */
	private GalaxyWindowView view;
	
    public GalaxyWindowConfig() throws ApplicationError {
    	listener = new GalaxyWindowListener();
    	view = new GalaxyWindowView(listener);
    	listener.setView(view);
    }
    
    public void activate() throws ApplicationError {
    	view.activate();
    }
    
    public void deactivate() throws ApplicationError {
    	listener.clearSelection();
    	view.deactivate();
    }
    
    public void setOwner(String owner) {
    	listener.setOwner(owner);
    	view.getMapPanel().setOwner(owner);
    }
    
    public void handleTick() throws ApplicationError {
        view.handleTick();
        AlarmSystem.getInstance().tick();
    }

	public String getName() { return NAME; }


	public GalaxyWindowListener getListener() {
		return listener;
	}

	public GalaxyWindowView getGalaxyView() {
		return view;
	}
	
	public MainWindowView getView() {
		return view;
	}
	
    public JPanel getMainPanel() {
    	return view.getMapPanel();
    }

	public Dimension getMainPanelSize() throws ApplicationError {
		JDesktopPane desktopPane = view.getDesktopPane();
		if (desktopPane != null) {
			if (!desktopPane.isValid()) {
				MainWindow.getInstance().getContainer().validate();
			}
			return desktopPane.getSize();
		} else {
			JScrollPane mapWindowPanel = view.getMapPanelScrollPane();
			if (!mapWindowPanel.isValid()) {
				MainWindow.getInstance().getContainer().validate();
			}
			return mapWindowPanel.getSize(); 
		}		
	}
	
    public GalaxyMapPanel getGalaxyMapPanel() {
    	return view.getMapPanel();
    }

	public PlayerDelegate getOwnerOfActiveAnimation() {
		return null;
	}

}
