/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import mot.client.deligates.FleetDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.TransportDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftAttack;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftBombard;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftColonise;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftDelete;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftDeploy;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftInvade;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftJoinFleet;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftSleep;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftUpgrade;
import mot.client.swing.window.galaxy.actions.ActionButtonCraftViewAFleetDetails;
import mot.client.swing.window.galaxy.actions.ActionButtonRemoveWaypoint;
import mot.client.swing.window.galaxy.actions.ActionButtonSetWaypoint;
import mot.client.swing.window.galaxy.actions.ActionButtonTransportColonists;
import mot.client.swing.window.galaxy.actions.ActionButtonTransportTroops;
import mot.client.swing.window.main.MainWindow;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;

public class GalaxyWindowContextButtonPanel extends ComponentTable {
	
	private GalaxyWindowListener listener;

	public GalaxyWindowContextButtonPanel(GalaxyWindowListener listener) throws ApplicationError {
		super("-,-,-", 1, 3, 4, 4, 4);
		this.listener = listener;
	}

	public void setContent(MoveableOwnedObjectDelegate moo) throws ApplicationError {
		removeAll();
		if (moo != null && moo.isOwnedBy(listener.getOwner())) {
			boolean galaxyMenuCanAttackActive = moo.galaxyMenuCanAttackActive();
			boolean galaxyMenuCanBombardActive = moo.galaxyMenuCanBombardActive();
			boolean galaxyMenuCanInvadeActive = moo.galaxyMenuCanInvadeActive();
			boolean galaxyMenuCanColoniseActive = moo.galaxyMenuCanColoniseActive();
			boolean galaxyMenuCanDeployActive = moo.galaxyMenuCanDeployActive();
			boolean galaxyMenuCanJoinFleetActive = moo.galaxyMenuCanJoinFleetActive();
			boolean galaxyMenuCanUpgradeActive = moo.galaxyMenuCanUpgradeActive();
			boolean galaxyMenuCanDeleteActive = moo.galaxyMenuCanDeleteActive();
			boolean galaxyMenuCanViewFleetActive = moo instanceof FleetDelegate;
			
			UserPlayerDelegate owner = MainWindow.getOwner();
			int qtyFreeTransport = owner.getQtyFreeTransport();
			ActionButtonTransportTroops loadTroops = null;
			ActionButtonTransportColonists loadColonists = null;
			if (moo instanceof TransportDelegate && moo.getOrbit() != null && moo.getOrbit().getPlanet() != null) {
				TransportDelegate transport = (TransportDelegate) moo;
		        PlanetDelegate planet = DeligateCacheManager.getPlanet(moo.getOrbit().getPlanet());
				if (transport.containsTroops() && planet.isOwnedBy(owner) && !planet.isMiningColony() && planet.getQtyGroundDefense() > 6) {
					loadTroops = new ActionButtonTransportTroops(planet, listener);
				}
				if (transport.containsColonists() && planet.isOwnedBy(owner) && !planet.isMiningColony() && planet.getPopulation() > 6) {
					loadColonists = new ActionButtonTransportColonists(planet, listener);
				}
			}

			int qty = 1;
			if (galaxyMenuCanAttackActive) qty++;
			if (galaxyMenuCanBombardActive)qty++;
			if (galaxyMenuCanInvadeActive) qty++;
			if (galaxyMenuCanColoniseActive) qty++;
			if (galaxyMenuCanDeployActive) qty++;
			if (galaxyMenuCanJoinFleetActive) qty++;
			if (galaxyMenuCanUpgradeActive) qty++;
			if (galaxyMenuCanDeleteActive) qty++;
			if (loadTroops != null) qty++;
			if (loadColonists != null) qty++;
			setQtyRows(Comp.ceil(qty/3.0f));
			
			if (galaxyMenuCanAttackActive) add(new ActionButtonCraftAttack(listener));
			if (galaxyMenuCanBombardActive)add(new ActionButtonCraftBombard(listener));
			if (galaxyMenuCanInvadeActive) add(new ActionButtonCraftInvade(listener));
			if (galaxyMenuCanColoniseActive) add(new ActionButtonCraftColonise(listener));
			if (galaxyMenuCanDeployActive) add(new ActionButtonCraftDeploy(listener));
			if (galaxyMenuCanJoinFleetActive) add(new ActionButtonCraftJoinFleet(listener));
			if (galaxyMenuCanUpgradeActive) add(new ActionButtonCraftUpgrade(listener));
			if (galaxyMenuCanDeleteActive) add(new ActionButtonCraftDelete(listener));
			if (galaxyMenuCanViewFleetActive) add(new ActionButtonCraftViewAFleetDetails(listener));
			if (loadTroops != null) add(loadTroops);
			if (loadColonists != null) add(loadColonists);
			add(new ActionButtonCraftSleep(listener));
		}
		validate();
		repaint();
	}

	public void setContent(PlanetDelegate planet, GalaxyWindowStateHoldingPanel holdingPanel) throws ApplicationError {
		removeAll();
		if (planet != null) {
			UserPlayerDelegate owner = MainWindow.getOwner();
			int qtyFreeTransport = owner.getQtyFreeTransport();
			boolean loadTroops = (planet.isOwnedBy(owner) && !planet.isMiningColony() && planet.getQtyGroundDefense() > 6);
			boolean loadColonists = (planet.isOwnedBy(owner) && !planet.isMiningColony() && planet.getPopulation() > 6);
			int qty = 1;
			if (loadTroops) qty++;
			if (loadColonists) qty++;
			setQtyRows(Comp.ceil(qty/3.0f));
			if (!planet.isWayPoint()) {
				add(new ActionButtonSetWaypoint(listener, holdingPanel));
			} else {
				add(new ActionButtonRemoveWaypoint(listener, holdingPanel));
			}
			if (loadTroops) add(new ActionButtonTransportTroops(planet, listener));
			if (loadColonists) add(new ActionButtonTransportColonists(planet, listener));
		}
		validate();
		repaint();
	}

}
