/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Color;

import javax.swing.BoxLayout;

import mot.client.actions.SimpleAction;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.swing.common.BoxPanel;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.panels.TechnologyPicturePanel;
import mot.client.swing.window.battle.DamagePanel;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;
import mot.common.vo.NavigatorVO;
import mot.common.vo.OrbitVO;

public class GalaxyWindowContextCraftPanel extends ComponentTable implements GalaxyWindowStateHoldingPanel {
	
	private static Color healthBarColour;

	private GalaxyWindowListener listener;
	
	private MotBorder craftBorder;
	private TechnologyPicturePanel hullImagePanel;
	private MotLabel hitPointsLabel; 
	private MotLabel spaceLabel; 
	//private GalaxyWindowContextButtonPanel buttonPanel;
	private BoxPanel headerPanel;
	private BoxPanel detailsPanel;
	private DamagePanel damagePanel;

	private CraftDelegate selection;


	public GalaxyWindowContextCraftPanel(GalaxyWindowListener listener) throws ApplicationError {
		super("HV", 5, 1, 0, 0, 0);
		
		if (healthBarColour == null) {
			healthBarColour = PropertyManager.getMandatoryAsColor("damagebar.color");
		}
		
        this.listener = listener;
        
        hullImagePanel = new TechnologyPicturePanel(listener.getOwner(), false, false, 2, true, false, null);
        PanelUtils.setTheBloodySize(hullImagePanel, new Size(100, 100));
        hitPointsLabel = new MotLabel();
        spaceLabel = new MotLabel();
        //buttonPanel = new GalaxyWindowContextButtonPanel(listener);
		headerPanel = new BoxPanel(BoxLayout.Y_AXIS);
		detailsPanel = new BoxPanel();
		
		ComponentTable craftPanel = new ComponentTable("HV,HV", 4, 2, 1, 5, 0);
		craftPanel.add(hullImagePanel, "HV2F");
		damagePanel = new DamagePanel(healthBarColour, 0);
		craftPanel.add(damagePanel, "HV2F");
		craftPanel.add(L.HIT_PONTS);
		craftPanel.add(hitPointsLabel);
		craftPanel.add(L.SPACE);
		craftPanel.add(spaceLabel);
		
        
        craftBorder = new MotBorder(craftPanel, L.HULL);
        
		add(craftBorder);
        //add(new MotBorder(buttonPanel, L.COMMANDS));
        add(new PadPanel(5));
        add(headerPanel);
        add(new PadPanel(5));
        add(detailsPanel, "HVPS");
        
        //setBackground(menuBackgroundColor);
	}

    public CraftDelegate getSelection() {
    	return selection;
    }

    public void setSelection(CraftDelegate newSelection) throws ApplicationError {
		
    	this.selection = newSelection;
    	
    	Hull hull = newSelection.getCraftDesign().getHullTech();
		hullImagePanel.update(hull, false);
		craftBorder.setPanelName(newSelection.getName());
		hitPointsLabel.setText(""+hull.getMaxDamage());
		spaceLabel.setText(""+hull.getSpace());
		double damage = newSelection.getDamageAsPercent();
		if (damage > 0.0) {
			damagePanel.setDamage(damage);
			damagePanel.setLabel(L.DAMAGE);
			PanelUtils.setTheBloodySize(damagePanel, 100,15);
		} else {
			damagePanel.setDamage(0.0);
			damagePanel.setLabel(null);
			PanelUtils.setTheBloodySize(damagePanel, 100,1);
		}
    	
    	updateHeaderPanel(headerPanel, newSelection);
    	//buttonPanel.setContent(newSelection);
		
		detailsPanel.removeAll();
		if (newSelection != null) {
			if (newSelection.isOwnedBy(listener.getOwner())) {
		        int maxQtyTechPanels = 3;
				if (MainWindow.hasInstance()) {			
					MainWindow mainWin = MainWindow.getExestingInstance();
					double screenHeight = mainWin.getSize().getHeight();
					if (screenHeight < 900) {
						maxQtyTechPanels = 1;
					} else if (screenHeight < 1000) {
						maxQtyTechPanels = 2;
					}
				} else {
					maxQtyTechPanels = 0;
				}
				detailsPanel.add(new GalaxyWindowContextCraftDetailsPanel(newSelection, maxQtyTechPanels));
			} else {
				detailsPanel.add(new GalaxyWindowContextCraftOwnerPanel(newSelection));
			}
		}
		
		
		SimpleAction clickHullImageAction = new SimpleAction() {
			public void select() throws ApplicationError, ValidationError {
				MainWindow.getExestingInstance().getGalaxyMapListener().openCraftDetailsWindow(null, selection, true);
			}
		};
		hullImagePanel.setClickAction(clickHullImageAction);

		revalidate();
    }

	public static void updateHeaderPanel(BoxPanel headerPanel, MoveableOwnedObjectDelegate moo) throws ApplicationError {
		headerPanel.removeAll();
		if (moo != null) {
			NavigatorVO navigator = moo.getNavigator();
			if (navigator != null && navigator.getDestination() != null) {
				OrbitVO destination = navigator.getDestination();
				String name = destination.getPlanet() == null ? destination.getStellarObject() : destination.getPlanet();
				BoxPanel destinationPanel = new BoxPanel(BoxLayout.Y_AXIS);
				destinationPanel.add(new MotLabel(" "+name + " [ETA " + navigator.getEta()+"]"));
				destinationPanel.add(new PadPanel(2));
				headerPanel.add(new MotBorder(destinationPanel, L.CAP_HEADING_TO_));
			}
		}
	}

	@Override
	public void clearState() throws ApplicationError {
	}

	@Override
	public void handleTick() throws ApplicationError {
	}

	@Override
	public void updateSelection() throws ApplicationError {
		setSelection(selection);
	}

}
