/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.BoxLayout;
import javax.swing.JPanel;

import mot.client.deligates.CraftDelegate;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.TransportDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.ComponentTableWithBackground;
import mot.client.swing.panels.ImagePanel;
import mot.client.swing.window.main.MainWindow;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;

public class GalaxyWindowContextPanel extends JPanel {
	
	private GalaxyWindowListener listener;
	private Color menuBackgroundColor;
	private String mainMenuBackground;
	private ImageConfig mainMenuTop;
	private ImageConfig mainMenuItem;
	private ImageConfig mainMenuEmpty;
	private ImageConfig mainMenuBottom;
	
    private GalaxyWindowButtonMenu fileMenu;
    private GalaxyWindowButtonMenu empireMenu;
    private GalaxyWindowContextCraftPanel craftPanel;
    private GalaxyWindowContextTransportPanel transportPanel;
    private GalaxyWindowContextFleetPanel fleetPanel;
    private GalaxyWindowContextPlanetPanel planetPanel;
    private GalaxyWindowContextStellarObjectPanel stellarObjectPanel;
	
	private GalaxyWindowStateHoldingPanel state = null;

	public GalaxyWindowContextPanel(GalaxyWindowListener listener, Color menuBackgroundColor) throws ApplicationError {
        this.listener = listener;
		this.menuBackgroundColor = menuBackgroundColor;
       	mainMenuBackground = ImageConfigFactory.getImageForConfig("MainMenuBackground");
       	mainMenuTop = ImageConfigFactory.getImageConfigNoError("MainMenuTop");
       	mainMenuItem = ImageConfigFactory.getImageConfigNoError("MainMenuItem");
       	mainMenuEmpty = ImageConfigFactory.getImageConfigNoError("MainMenuEmpty");
       	mainMenuBottom = ImageConfigFactory.getImageConfigNoError("MainMenuBottom");
       	
		fileMenu = new GalaxyWindowFileButtonMenu(listener, this);
        empireMenu = new GalaxyWindowEmpireButtonMenu(listener, this);
        craftPanel = new GalaxyWindowContextCraftPanel(listener);
        transportPanel = new GalaxyWindowContextTransportPanel(listener);
        fleetPanel = new GalaxyWindowContextFleetPanel(listener);
        planetPanel = new GalaxyWindowContextPlanetPanel(listener);
        stellarObjectPanel = new GalaxyWindowContextStellarObjectPanel(listener);
        
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
       	setBackground(menuBackgroundColor);
  
		if (MainWindow.hasInstance()) {			
			MainWindow mainWin = MainWindow.getExestingInstance();
			double screenHeight = mainWin.getSize().getHeight();
			if (screenHeight < 1024) {
		       	PanelUtils.setTheBloodySize(this, new Size(170,600));
			} else {
		       	PanelUtils.setTheBloodySize(this, new Size(150,600));
			}
		}
		
	}
	
	public void showMenu(ArrayList<GalaxyWindowMenuButton> buttons) {
        ComponentTable headerMenuPanel = new ComponentTableWithBackground("-,-,-,-,-", 1, 5, 0, 0, 0, mainMenuTop);
        headerMenuPanel.setGridWidths("0.2,0.2,0.2,0.2,0.2");
        headerMenuPanel.add(new PadPanel());
        headerMenuPanel.add(fileMenu);
        headerMenuPanel.add(new PadPanel());
        headerMenuPanel.add(empireMenu);
        headerMenuPanel.add(new PadPanel());

        int maxQtyButtons = buttons.size();
//		if (MainWindow.hasInstance()) {			
//			MainWindow mainWin = MainWindow.getExestingInstance();
//			double screenHeight = mainWin.getSize().getHeight();
//			if (screenHeight < 900) {
//				maxQtyButtons = 5;
//			} else if (screenHeight < 1000) {
//				maxQtyButtons = 7;
//			}
//		} else {
//			maxQtyButtons = 0;
//		}
        int qtyRows = maxQtyButtons*2+4;
        
		ComponentTable menuPanel = new ComponentTableWithBackground("HV", 11, 1, 0, 0, 0, mainMenuBackground);
		menuPanel.removeAll();
		menuPanel.setQtyRows(qtyRows);
		menuPanel.add(headerMenuPanel);
		menuPanel.add(new ImagePanel(mainMenuEmpty.getImage()), "HVSP");
		int j = 0;
		for (Iterator i = buttons.iterator(); i.hasNext();) {
			if (j >= maxQtyButtons) break;
			GalaxyWindowMenuButton button = (GalaxyWindowMenuButton) i.next();
			ComponentTable menuItem = new ComponentTableWithBackground("-", 1, 1, 0, 0, 0, mainMenuItem);
			menuItem.add(button);
			menuPanel.add(menuItem);
			menuPanel.add(new ImagePanel(mainMenuEmpty.getImage()), "HVSP");
			j++;
		}
		menuPanel.add(new ImagePanel(mainMenuEmpty == null ? null : mainMenuEmpty.getImage()), "HVS");
		ImagePanel bottomPanel = new ImagePanel(mainMenuBottom);
		//PanelUtils.setTheBloodySize(bottomPanel, new Size(150,20));
		menuPanel.add(bottomPanel, "HV");
		
		removeAll();
		add(menuPanel.getScrollPaneNoBorder());
		validate();
		repaint();
	}

	public void showEmpireMenu() throws ApplicationError {
		showMenu(empireMenu.getButtons());
		if (state != null) state.clearState();
		state = null;
		TutorialSystem.notifyClearSelection();
	}

	public void showFileMenu() throws ApplicationError {
		showMenu(fileMenu.getButtons());
		if (state != null) state.clearState();
		state = null;
	}

	public void showMoo(MoveableOwnedObjectDelegate moo) throws ApplicationError {
    	if (moo instanceof CraftDelegate) {
    		CraftDelegate craft = (CraftDelegate) moo;
    		if (craft.getFleet() != null) {
    			showMoo(DeligateCacheManager.getMoo(craft.getFleet()));
    			return;
    		}
    	}
    	
		removeAll();
		
    	if (moo instanceof CraftDelegate) {
    		add(craftPanel);
    		craftPanel.setSelection((CraftDelegate)moo);
    		state = craftPanel;
    	} else if (moo instanceof TransportDelegate) {
    		add(transportPanel);
    		transportPanel.setSelection((TransportDelegate)moo);
    		state = transportPanel;
    	} else if (moo instanceof FleetDelegate) {
    		add(fleetPanel);
    		fleetPanel.setSelection((FleetDelegate)moo);
    		state = fleetPanel;
    	}
		
		validate();
		repaint();
	}

//	public boolean containsMoo(MoveableOwnedObjectDelegate moo) {
//		if (state == moosPanel) {
//			return moosPanel.getMooList().contains(moo);
//		} else {
//			return false;
//		}
//	}

	public void showPlanet(PlanetDelegate planet) throws ApplicationError {
		state = planetPanel;
		planetPanel.setSelection(planet);
				
		removeAll();
		add(planetPanel);
		validate();
		repaint();
	}

	public void showStellarObject(StellarObjectDelegate so) throws ApplicationError {
		state = stellarObjectPanel;
		stellarObjectPanel.setSelection(so);
				
		removeAll();
		add(stellarObjectPanel);
		validate();
		repaint();
	}

	// remove
	public NamedObjectDelegate getSelection() {
		if (state == craftPanel) {
			return craftPanel.getSelection();
		} else if (state == transportPanel) {
			return transportPanel.getSelection();
		} else if (state == fleetPanel) {
			return fleetPanel.getSelection();
		} else if (state == planetPanel) {
			return planetPanel.getSelection();
		} else if (state == stellarObjectPanel) {
			return stellarObjectPanel.getSelection();
		} else {
			return null;
		}
	}

	public MoveableOwnedObjectDelegate getMooSelection() {
		if (state == craftPanel) {
			return craftPanel.getSelection();
		} else if (state == transportPanel) {
			return transportPanel.getSelection();
		} else if (state == fleetPanel) {
			return fleetPanel.getSelection();
		} else {
			return null;
		}
	}

	public void updateSelectionState() throws ApplicationError {
		if (state == craftPanel) {
			CraftDelegate selection = craftPanel.getSelection();
			if (selection != null && DeligateCacheManager.getMoo(selection.getName()) != null) {
				craftPanel.setSelection(selection);
			} else {
				showEmpireMenu();
			}
		} else if (state == transportPanel) {
			TransportDelegate selection = transportPanel.getSelection();
			if (selection != null && DeligateCacheManager.getMoo(selection.getName()) != null) {
				//transportPanel.setSelection(selection);
			} else {
				showEmpireMenu();
			}
		} else if (state == fleetPanel) {
			FleetDelegate selection = fleetPanel.getSelection();
			if (selection != null && DeligateCacheManager.getMoo(selection.getName()) != null) {
				fleetPanel.setSelection(selection);
			} else {
				showEmpireMenu();
			}
		} else if (state == planetPanel) {
			PlanetDelegate selection = planetPanel.getSelection();
			if (selection != null) {
				planetPanel.setSelection(selection);
			} else {
				showEmpireMenu();
			}
		} else if (state == stellarObjectPanel) {
			StellarObjectDelegate selection = stellarObjectPanel.getSelection();
			if (selection != null) {
				stellarObjectPanel.setSelection(selection);
			} else {
				showEmpireMenu();
			}
		}
		
	}

	public void updateSelectionState(PlanetDelegate planet) throws ApplicationError {
		if (state == planetPanel) {
			PlanetDelegate selection = planetPanel.getSelection();
			if (selection == planet) {
				planetPanel.setSelection(selection, false);
			}
		}
	}

    public void handleTick() throws ApplicationError {
		if (state != null) state.handleTick();
	}

	public void updateSelection() throws ApplicationError {
		if (state != null) state.updateSelection();
	}

}
