/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Color;

import javax.swing.JPanel;

import mot.client.actions.MotAction;
import mot.client.actions.SimpleAction;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PadPanel;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.lists.FakeTechnologyLabelList;
import mot.client.swing.panels.ImagePanel;
import mot.client.swing.panels.ProjectProgressBar;
import mot.client.swing.renderers.CellRendererTechnologyLabel;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.solarsystem.PlanetAnimationPanelWithDescription;
import mot.common.bean.SalvageList;
import mot.common.bean.Size;
import mot.common.designs.Orbital;
import mot.common.designs.ProjectDesign;
import mot.common.lang.L;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class GalaxyWindowContextPlanetPanel extends ComponentTable implements GalaxyWindowStateHoldingPanel {

	private GalaxyWindowListener listener;
	private PlanetDelegate selectedPlanet;
	
	private MotLabel populationLabel = new MotLabel();
    private PadPanel maxPopulationIconPanel = new PadPanel();
	private MotLabel maxPopulationLabel = new MotLabel();
    private MotLabel productionLabel = new MotLabel();
    private PadPanel lostProductionIconPanel = new PadPanel();
    private MotLabel lostProductionLabel = new MotLabel();
    private MotLabel researchLabel = new MotLabel();
    private PadPanel lostResearchIconPanel = new PadPanel();
    private MotLabel lostResearchLabel = new MotLabel();
    private MotLabel moneyLabel = new MotLabel();
    private CellRendererTechnologyLabel renderer = new CellRendererTechnologyLabel(false, true, false, new Size(30,30), true, 0, false, null, null, true, null, 16, null);
    private FakeTechnologyLabelList defenceList = new FakeTechnologyLabelList(renderer);
    private ComponentTable planetPanel;
    private PlanetAnimationPanelWithDescription planetPanelContent;
    private ProjectProgressBar projectProgress;
	//private GalaxyWindowContextButtonPanel buttonPanel;

	private ImagePanel maxPopulationIcon;
	private ImagePanel lostProductionIcon;
	private ImagePanel lostResearchIcon;

	public GalaxyWindowContextPlanetPanel(GalaxyWindowListener gwl) throws ApplicationError {
		super("HV", 8, 1, 0, 0, 0);
		
        this.listener = gwl;
        
        maxPopulationIcon = new ImagePanel(ImageConfigFactory.getImageForConfig("MaxPop"), new Size(15,20));
        lostProductionIcon = new ImagePanel(ImageConfigFactory.getImageForConfig("LostProduction"), new Size(15,20));
        lostResearchIcon = new ImagePanel(ImageConfigFactory.getImageForConfig("LostResearch"), new Size(15,20));

        ComponentTable detailsTable = new ComponentTable("HV,HV,HV,HVSP", 5, 4, 0, 4, 2);
        detailsTable.add(new ImagePanel(ImageConfigFactory.getImageForConfig("Man"), new Size(15,20)));
        detailsTable.add(populationLabel);
		detailsTable.add(maxPopulationIconPanel);
        detailsTable.add(maxPopulationLabel);
        detailsTable.add(new ImagePanel(ImageConfigFactory.getImageForConfig("Hammer"), new Size(15,20)));
        detailsTable.add(productionLabel);
        detailsTable.add(lostProductionIconPanel);
        detailsTable.add(lostProductionLabel);
        detailsTable.add(new ImagePanel(ImageConfigFactory.getImageForConfig("Tube"), new Size(15,20)));
        detailsTable.add(researchLabel);
        detailsTable.add(lostResearchIconPanel);
        detailsTable.add(lostResearchLabel);
        detailsTable.add(new ImagePanel(ImageConfigFactory.getImageForConfig("Gold"), new Size(15,20)));
        detailsTable.add(moneyLabel);
        detailsTable.add(new PadPanel());
        detailsTable.add(new PadPanel());
        
		MotAction openProjectWindowAction = new MotAction() {
			public void doubleClick() throws ApplicationError, ValidationError { }
			public void click() throws ApplicationError, ValidationError { listener.openSelectProjectWindow(selectedPlanet); setSelection(selectedPlanet); }
		};
		
        projectProgress = new ProjectProgressBar(true, openProjectWindowAction);

		planetPanel = new ComponentTable("-", 1,1, 0,0,0);

		//buttonPanel = new GalaxyWindowContextButtonPanel(listener);
		
		add(planetPanel);
		//add(projectLabel);
		add(projectProgress);
		add(new PadPanel(1));
		add(new MotBorder(detailsTable, L.DETAILS));
		add(new PadPanel(1));
		add(new MotBorder(defenceList, L.DEFENSE));
		add(new PadPanel(1));
        //add(new MotBorder(buttonPanel, L.COMMANDS));
		//add(buttonPanel);
        add(new JPanel(), "HVPS");
        
        //setBackground(menuBackgroundColor);
	}

    public void setSelection(PlanetDelegate planet) throws ApplicationError {
    	setSelection(planet, true);
    }
    
    public void setSelection(PlanetDelegate planet, boolean updateRotatingPlanet) throws ApplicationError {
    	this.selectedPlanet = planet;
    	
    	if (updateRotatingPlanet && planet != null) {
    		if (planetPanelContent == null) {
    			planetPanelContent = new PlanetAnimationPanelWithDescription(planet);
    			planetPanel.add(planetPanelContent);
    		} else {
    			planetPanelContent.setPlanet(planet);
    		}
    	}
    	
    	if (planet != null && planet.isViewable()) {
            boolean isMiningColony = planet.isMiningColony();
    		if (!isMiningColony) {
    	        double population = planet.getPopulation();
    	        double populationGrowth = planet.getPopulationGrowth();
    	        double maxPopulation = planet.getMaxPopulation();
    	        int productionOutput = planet.getProductionOutput();
    	        int lostProductionOutput = planet.getLostProductionOutput();
    	        int researchOutput = planet.getResearchOutput();
    	        int lostResearchOutput = planet.getLostResearchOutput();
    	        long moneyOutput = planet.getMoneyOutput();
    			populationLabel.setText( Formatter.whole(population) + Formatter.fractionWithSign(populationGrowth) );
    			maxPopulationIconPanel.removeAllAndAdd(maxPopulationIcon);
    			maxPopulationLabel.setText(Formatter.whole(maxPopulation));
    			productionLabel.setText( Formatter.whole(productionOutput) );
    			if (lostProductionOutput > 0.0) {
	    			lostProductionIconPanel.removeAllAndAdd(lostProductionIcon);
	    			lostProductionLabel.setText(Formatter.whole(lostProductionOutput));
    			} else {
        			lostProductionIconPanel.removeAll();
        			lostProductionLabel.setText(null);
    			}
    			researchLabel.setText( Formatter.whole(researchOutput) );
    			if (lostResearchOutput > 0.0) {
	    			lostResearchIconPanel.removeAllAndAdd(lostResearchIcon);
	    			lostResearchLabel.setText(Formatter.whole(lostResearchOutput));
    			} else {
        			lostResearchIconPanel.removeAll();
        			lostResearchLabel.setText(null);
    			}
    			moneyLabel.setText( Formatter.whole(moneyOutput) );
    			if (moneyOutput < 0) {
    				moneyLabel.setFontColor(Color.RED);
    			} else {
    				moneyLabel.setFontColor(Color.WHITE);
    			}
        		SalvageList salvage = planet.getOwner().getSalvage();
        		ProjectDesign project = planet.getProjectDesign();
        		projectProgress.updateStatus(planet);
//        		if (project != null) {
//					int qtyTurns = planet.getTurnsToCompleteProject();
//					String txt = "Complete in "+qtyTurns;
//					if (qtyTurns > 1) txt += " turns"; else  txt += " turn";
//					projectLabel.setText(txt);
//        		} else {
//					projectLabel.setText(null);
//        		}
    		} else {
    			PlanetDelegate mineOwner = planet.getOwnerOfMiningStation();
    			if (mineOwner == null) {
    				Logger.logWarning("Station around planet "+planet.getName()+" does not have an owner");
    				return;
    			}
    	        int populationOutput = mineOwner.getMiningPopulation();
    	        int productionOutput = mineOwner.getMiningProduction();
    	        int researchOutput = mineOwner.getMiningResearch();
    	        long moneyOutput = mineOwner.getMiningMoney();
    			populationLabel.setText(Formatter.whole(populationOutput));
    			maxPopulationIconPanel.removeAll();
    			maxPopulationLabel.setText(null);
    			productionLabel.setText( Formatter.whole(productionOutput) );
    			lostProductionIconPanel.removeAll();
    			lostProductionLabel.setText(null);
    			researchLabel.setText( Formatter.whole(researchOutput) );
    			lostResearchIconPanel.removeAll();
    			lostResearchLabel.setText(null);
    			moneyLabel.setText( Formatter.whole(moneyOutput) );
   				moneyLabel.setFontColor(Color.WHITE);
        		projectProgress.clear("N/A");
				//projectLabel.setText(null);
    		}
    	} else if (planet != null && planet.hasOwner()){
	        double population = planet.getPopulation();
	        double maxPopulation = planet.getMaxPopulation();
			populationLabel.setText(Formatter.whole(population));
			maxPopulationIconPanel.removeAllAndAdd(maxPopulationIcon);
			maxPopulationLabel.setText(Formatter.whole(maxPopulation));
    		productionLabel.setText(L.UNKNOWN);
			lostProductionIconPanel.removeAll();
			lostProductionLabel.setText(null);
			researchLabel.setText(L.UNKNOWN);
			lostResearchIconPanel.removeAll();
			lostResearchLabel.setText(null);
    		moneyLabel.setText(L.UNKNOWN);
			moneyLabel.setFontColor(Color.WHITE);
            projectProgress.clear("Unknown Project");
			//projectLabel.setText(null);
    	} else {
            populationLabel.setText("N/A");
			maxPopulationIconPanel.removeAll();
			maxPopulationLabel.setText(null);
    		productionLabel.setText("N/A");
			lostProductionIconPanel.removeAll();
			lostProductionLabel.setText(null);
    		researchLabel.setText("N/A");
			lostResearchIconPanel.removeAll();
			lostResearchLabel.setText(null);
    		moneyLabel.setText("N/A");
			moneyLabel.setFontColor(Color.WHITE);
            projectProgress.clear("Not Colonised");
			//projectLabel.setText(null);
    	}
    	
        boolean viewable = (planet != null) && planet.isViewable();
		if (viewable) {
			TechnologyList tl = null;
	        Weapons weapons = planet.getWeapons();
	        if (weapons != null && weapons.getQuantity() > 0) {
	        	if (tl == null) tl = new GeneralTechnologyList();
	            tl.add(weapons);
	        }
	        
	        Shields shields = planet.getShields();
	        if (shields != null && shields.getQuantity() > 0) {
	        	if (tl == null) tl = new GeneralTechnologyList();
	            tl.add(shields);
	        }
	        
	        Scanners scanners = planet.getScanners();
	        if (scanners != null && scanners.getQuantity() > 0) {
	        	if (tl == null) tl = new GeneralTechnologyList();
	            tl.add(scanners);
	        }
	
	    	Orbital orbital = planet.getOrbital();
			if (orbital != null) {
	        	if (tl == null) tl = new GeneralTechnologyList();
	            tl.add(orbital.getHullTech());
			}
	        if (tl != null) {
	        	defenceList.setContent(tl, null);
	        } else {
	        	//defenceList.clearContent();
	        	defenceList.setContent(new MotLabel(L.NO_DEFENCES));
	        }
	        if (planetPanelContent != null) {	        	
	        	SimpleAction clickPlanetImageAction = new SimpleAction() {
	        		public void select() throws ApplicationError, ValidationError {
	        			MainWindow.getExestingInstance().getGalaxyMapListener().openSolarSystemWindow(selectedPlanet);
	        		}
	        	};
	        	planetPanelContent.setClickAction(clickPlanetImageAction);
	        }

			revalidate();
		} else if (planet != null) {
			CraftDelegate[] ca = DeligateCacheManager.getCraftsAt(MainWindow.getOwner(), planet.getSolarSystem());
			if (ca != null) {
				String txt = planet.hasDefences() ? L.DEFENDED : L.NO_DEFENCES;
				defenceList.setContent(new MotLabel(txt));
			} else if (planet.hasOwner()) {
				defenceList.setContent(new MotLabel(L.UNKNOWN));
			} else {
				defenceList.setContent(new MotLabel("N/A"));
			}
			
		} else {
        	defenceList.clearContent();
		}

    	//buttonPanel.setContent(planet, this);
    	
		revalidate();
    }

	@Override
	public void clearState() throws ApplicationError {
		defenceList.clearContent();
	}

	@Override
	public void handleTick() throws ApplicationError {
		if (planetPanelContent != null) {
			planetPanelContent.handleTick();
		}
	}

	public PlanetDelegate getSelection() {
		return selectedPlanet;
	}

	@Override
	public void updateSelection() throws ApplicationError {
		setSelection(selectedPlanet);
	}

}
