/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import mot.client.deligates.BlackHoleDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.StellarObjectOrbitalDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.BoxPanel;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.icons.RaceIconFactory;
import mot.client.swing.panels.BackgroundImagePanelFinal;
import mot.client.swing.panels.PicturePanel;
import mot.client.swing.panels.TechnologyPicturePanel;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Location;
import mot.common.bean.SectorInfluenceItem;
import mot.common.bean.SectorInfluenceList;
import mot.common.bean.SectorOwnerGrid;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;

public class GalaxyWindowContextStellarObjectPanel extends ComponentTable implements GalaxyWindowStateHoldingPanel {

	private GalaxyWindowListener listener;
	
	private MotLabel nameLabel = new MotLabel(null, 2, JLabel.CENTER, false);
	private PicturePanel image;
	private BoxPanel contentPanel;
    private BoxPanel influencePanel;
    //private ComponentTable buttonPanel;

	private StellarObjectDelegate selection;

	public GalaxyWindowContextStellarObjectPanel(GalaxyWindowListener listener) throws ApplicationError {
		super("HV", 9, 1, 0, 0, 0);
		
        this.listener = listener;
        
        image = new PicturePanel();
        PanelUtils.setTheBloodySize(image, new Size(110,110));

        contentPanel = new BoxPanel(BoxLayout.Y_AXIS);
        influencePanel = new BoxPanel();
        //buttonPanel = new ComponentTable("c,c", 1, 1, 4, 4, 4);

        add(new PadPanel(image, 0,0,0,0));
        add(nameLabel);
        add(new PadPanel(5));
        add(contentPanel);
        add(new PadPanel(5));
        add(new MotBorder(influencePanel, "Sector Influence"));
        add(new PadPanel(5));
        //add(buttonPanel);
        add(new PadPanel(5));
        add(new JPanel(), "HVPS");
        
        //setBackground(menuBackgroundColor);
	}

    public StellarObjectDelegate getSelection() {
    	return selection;
    }

    public void setSelection(StellarObjectDelegate so) throws ApplicationError {
		
    	this.selection = so;
    	
    	image.setImage(so.getImage());
    	
    	StringBuilder text = new StringBuilder("<html><b>"+so.getName()+"</b>");
    	if (so instanceof BlackHoleDelegate) {
    		BlackHoleDelegate bh = (BlackHoleDelegate) so;
    		if (bh.getResearchBonus() > 0.0) {    			
    			text.append("<br/>Research ");
    			text.append(Formatter.percentLabel((double)bh.getResearchBonus()));
    		}
    		if (bh.hasSpaceJumpAbility()) {    			
    			text.append("<br/>Space Jump");
    		}
    	}
    	text.append("</html>");
		nameLabel.setText(text.toString());
    	
    	contentPanel.removeAll();
    	
    	ComponentTable infoPanel = new ComponentTable("HV,HVP", 2, 2, 0, 5, 2);
    	
    	infoPanel.add("Last Visited");
    	int lastVisitedDate = so.getLastVisitedDate();
    	if (lastVisitedDate > 0) {
        	infoPanel.add(""+lastVisitedDate);
    	} else {
        	infoPanel.add("Never");
    	}
    	
    	infoPanel.add("Range");
    	int range = so.getRange(MainWindow.getOwner());
    	if (range > 0) {
    		infoPanel.add(""+range);
    	} else {
    		infoPanel.add("N/A");
    	}

    	contentPanel.add(infoPanel);

    	StellarObjectOrbitalDelegate orbital = so.getStellarObjectOrbital();
    	if (orbital != null) {
    		TechnologyPicturePanel panel = new TechnologyPicturePanel(listener.getOwner(), false, false, true, false, null);
            PanelUtils.setTheBloodySize(panel, new Size(100, 100));
            panel.update(orbital.getOrbital().getHullTech());
            contentPanel.add(new MotBorder(panel, "Orbital"));
    	} else if (!so.hasOwner()) {
            PicturePanel panel = new PicturePanel(ImageConfigFactory.getImageForConfig("NoOrbitalBackground"));
            PanelUtils.setTheBloodySize(panel, new Size(100, 100));
			contentPanel.add(new MotBorder(panel, "Orbital"));
    	}

    	contentPanel.invalidate();
    	
    	Location loc = so.getLocation();
    	SectorOwnerGrid sog = DeligateCacheManager.getSectorOwnerGrid();
		SectorInfluenceList cell = sog.getCellByLocation(loc.getX(), loc.getY());
		influencePanel.removeAll();
		if (cell != null) {
			SectorInfluenceItem[] influence = cell.getItemsArray();
			if (influence != null) {
				ComponentTable c = new ComponentTable("HV,HV,HVS", influence.length, 3, 0, 3, 2);
				for (int i = 0; i < influence.length; i++) {					
					PlayerDelegate player = DeligateCacheManager.getPlayer(influence[i].getPlayer());
			        ImageConfig flagImage = ImageConfigFactory.getInstance().getFlagConfig(player.getColorName());
			        c.add(new BackgroundImagePanelFinal(flagImage.getImage(), new Size(20,20)));
					c.add(new MotLabel(RaceIconFactory.getIcon(player)));
					MotLabel v = new MotLabel(""+influence[i].getInfluence());
					c.add(v);
				}
				influencePanel.add(c);
			}
		} else {
			influencePanel.add(new MotLabel("None"));
		}
		
		UserPlayerDelegate owner = MainWindow.getOwner();
    	
//		buttonPanel.removeAll();
//		boolean createTradeRoute = (so.isOwnedBy(owner) && so instanceof SolarSystemDelegate && ((SolarSystemDelegate)so).canCreateTradeRoute());
//		buttonPanel.setQtyCols(createTradeRoute ? 2 : 1);
//		if (!so.isWayPoint()) {
//			buttonPanel.add(new ActionButtonSetWaypoint(listener, this));
//		} else {
//			buttonPanel.add(new ActionButtonRemoveWaypoint(listener, this));
//		}
//		if (createTradeRoute) {
//			buttonPanel.add(new ActionButtonCreateTradeRoute(so, listener));			
//		}

		revalidate();
		invalidate();
    }

	@Override
	public void clearState() throws ApplicationError {
	}

	@Override
	public void handleTick() throws ApplicationError {
	}

	@Override
	public void updateSelection() throws ApplicationError {
		setSelection(selection);
	}

}
