/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.util.ArrayList;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JScrollPane;

import mot.client.actions.ActiveRectangle;
import mot.client.actions.ActiveRectangleList;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.Delegate;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.craft.CraftDetailsWindow;
import mot.client.swing.window.craft.FleetDetailsWindow;
import mot.client.swing.window.galaxy.actions.GalaxyMapButton;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowListener;
import mot.client.swing.window.solarsystem.SelectPlanetWindow;
import mot.client.swing.window.solarsystem.SelectProjectWindow;
import mot.client.swing.window.solarsystem.SolarSystemWindow;
import mot.client.swing.window.tech.ResearchWindow;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.GalaxyDetails;
import mot.common.bean.Location;
import mot.common.bean.SalvageList;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyTree;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.vo.OrbitVO;
import mot.engine.Cheat;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;

public class GalaxyWindowListener extends MainWindowListener implements MotGalaxyMapActions, GalaxyMapListener {


	/**
	 * This is the view that this listener uses to display its results.
	 */
	private GalaxyWindowView view;
	

	public GalaxyWindowListener() throws ApplicationError {
        setMapWindowSize();
	}
	
    //
    // KEY LISTENER STUFF
    //
    public void motKeyTyped(KeyEvent e) throws ApplicationError, ValidationError {
		if (view.isDesktopPaneActive()) {
	        if (Character.toLowerCase(e.getKeyChar()) == 'n') { // next
	    		MotInternalWindow dialog = view.getTopWindow();
	    		if (dialog != null) {
	    			dialog.next();
	    		}
	    	} else if (Character.toLowerCase(e.getKeyChar()) == 'v') { // previous
	    		MotInternalWindow dialog = view.getTopWindow();
	    		if (dialog != null) {
	    			dialog.previous();
	    		}    		
	    	} else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) { // close
	    		MotInternalWindow dialog = view.getTopWindow();
	    		if (dialog != null && dialog.escapeKeyClosesWindow()) {
	    			dialog.hide();
	    		}    		
	    		
	        }
		} else {
			ActiveRectangleList arl = getView().getMapPanel().getObjects();
			if (arl != null) {
				boolean processed = arl.keyPressed(e.getKeyChar(), e.getModifiers());
				if (processed) return;
			}
	        if (e.getKeyChar() == '^') {
	        	try {
	        		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
					if (localDelegate != null) {						
			        	UserPlayerDelegate owner = getOwner();
						Cheat.cheat(owner);
						MainWindow.getExestingInstance().getEventMailBox().processQueuedEvents();
						view.repaintMap();
						view.getStatusPanel().updateStatus(owner);
					}
	        	} catch (ApplicationError ex) {
	        		view.reportApplicationError(ex);
	        	}
	    	} else if (Character.toLowerCase(e.getKeyChar()) == 'n') { // next
    			next();
	    	} else if (Character.toLowerCase(e.getKeyChar()) == 'v') { // previous
    			previous();
	    	} else if (Character.toLowerCase(e.getKeyChar()) == '1') {
	    		if (Logger.isDebug()) {
	    			Craft c1 = MotEngine.instance.getCrafts().getCraft("Military39_2");
	    			if (c1 != null) {
	    				c1.moveTo(new Orbit(MotEngine.instance.getStellarObjects().getStellarObject("Moggill")));
	    				//AlarmSystem.getInstance().notifyEndTurnButtonPressed();
	    			}
	    		}
	        }
		}
    }
    
    public void keyReleased(KeyEvent e) { }

    /**
     * Called when the user rolls the mouse wheel
     */
	public void mouseWheelMoved(MouseWheelEvent e) {
		view.getZoomSliderListener().mouseWheelMoved(e);
	}

    public void motMouseClicked(MouseEvent e) {
    	MainWindow mainWin = null;
    	try {
    		mainWin = MainWindow.getInstance();
    	} catch (Exception ex) {
    		Logger.logError("FAIL", ex);
    	}
        try {
        	GalaxyMapPanel mapWindow = view.getMapPanel();
            int x = e.getX();
            int y = e.getY();
            if (e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1) {
    			ActiveRectangle o = mapWindow.getObjects().click(x,y);
    			if (o != null && !o.isDraggable()) {
    				// Animation stops when button is down (and mouse event can be lost when dialog is open)
    				mouseButton1Down = false;
    			}
            } else if (e.getClickCount() > 1 && e.getButton() == MouseEvent.BUTTON1) {
            	// Animation stops when button is down (and mouse event can be lost when dialog is open)
            	mouseButton1Down = false;
                ActiveRectangle o = mapWindow.getObjects().doubleClick(x,y);
            } else if (e.getButton() == MouseEvent.BUTTON3) {
            	clearSelection();
                view.repaintMap();
            }
        } catch (ValidationError ex) {
        	MainWindow.getExestingInstance().reportValidationError(ex);
        } catch (Exception ex) {
            MainWindow.getExestingInstance().reportApplicationError(ex);
        }
    }
    
	public void componentResized() throws ApplicationError {
		JDesktopPane desktopPane = view.getDesktopPane();
		if (desktopPane != null) {
			JInternalFrame[] frames = desktopPane.getAllFrames();
			if (frames != null) {
				for (int i = 0; i < frames.length; i++) {
					if (frames[i] instanceof MotInternalWindow)
					((MotInternalWindow)frames[i]).checkForResize(desktopPane);
				}
			}
		}
		//System.out.println("Here");
	}

	private GalaxyMapButton last = null;
	protected void _mouseMoved(int x, int y) throws ApplicationError, ValidationError {
		if (last != null) last.setMouseIn(false);
		ActiveRectangleList arl = getView().getMapPanel().getObjects();
		if (arl != null) {
			GalaxyMapButton ar = arl.getButtonAt(x, y);
			if (ar != null) {
				ar.setMouseIn(true);
				last = ar;
			}
		}
	}

	protected void mousePausedAt(int x, int y) throws ApplicationError {
		ActiveRectangleList arl = getView().getMapPanel().getObjects();
		if (arl != null) {
			GalaxyMapButton ar = arl.getButtonAt(x, y);
			if (ar != null) {
				ar.showToolTip();
			}
		}
	}
	
	protected ActiveRectangle getDraggableObjectAt(int x, int y) { 
		return getView().getMapPanel().getDraggableObjectAt(x, y); 
	}

	public boolean isMouseButton1Down() {
		return mouseButton1Down;
	}


    //
    // HELPER METHODS
    //

    public PlanetDelegate openSelectPlanetForColonisation(SolarSystemDelegate ss, CraftDelegate craft, boolean produceErrors) throws ValidationError, ApplicationError {
        if (craft == null) {
            MoveableOwnedObjectDelegate selection = getSelectedMoo();
            if (selection != null && selection instanceof CraftDelegate) {
            	craft = (CraftDelegate) selection;
            } else {
            	return null;
            }
        }
        PlanetDelegate[] planets = craft.getColonisablePlanets(ss.getPlanets(), false);
        if (planets != null && planets.length > 0 && craft != null) {
        	UserPlayerDelegate owner = getOwner();
        	SolarSystemWindow ssWin = new SolarSystemWindow(null, ss, owner);
        	ssWin.waitUntilPainted();
        	SelectPlanetWindow spw = new SelectPlanetWindow(owner, ssWin, planets, "<html>"+L.COLONISER_CRAFT_HAS_ARRIVED_AT+" <center><b>"+ss.getName()+"</b></center> "+L.SELECT_A_PLANET_TO_COLONISE+"</html>");
            spw.waitUntilClosedModal();
            ssWin.hide();
            if (spw.getSelectedPlanet() != null) {
                PlanetDelegate planet = spw.getSelectedPlanet();
                MainWindow.getInstance().showMessageGalaxy(L.COLONISE_PLANET+" "+planet.getName(), false);
                ProxyStore.getInstance().craftColoniseCmd(craft, planet, null, MainWindow.getExestingInstance());
            }
            return spw.getSelectedPlanet();
        } else if (produceErrors) {
        	throw new ValidationError(L.YOU_CANNOT_COLONISE_ANYTHING_HERE);
        }
        return null;
    }
    
    public void openResearchWindow(UserPlayerDelegate player, boolean stopEndTurnUntilWinClosed, boolean onlyIfTechsAvailableForResearch) throws ApplicationError, ValidationError {
    	UserPlayerDelegate owner = getOwner();
        Technology [] availableTechs = DeligateCacheManager.getTechsAvailableForResearch();
        Technology [] missedTechs = TechnologyTree.getInstance().getVisibleMissedTechnology(owner);
        SalvageList salvage = player.getSalvage();
        salvage.removeAllTechsDiscoveredBy(player);
        salvage.removeAll(availableTechs);
        Technology[] recentMissedTechs = TechnologyTree.getInstance().getRecentlyMissedTechnology(missedTechs, owner);
		
        if (onlyIfTechsAvailableForResearch) {
        	if (availableTechs == null || availableTechs.length == 0) {
        		// No techs available for research, do not open window!
        		return;
        	}
        }
        
		new ResearchWindow(null, player, player.getResearchProject(), availableTechs, salvage, missedTechs, recentMissedTechs, stopEndTurnUntilWinClosed);
    }

	public void openSelectProjectWindow(PlanetDelegate planet) throws ApplicationError, ValidationError {
    	playActionSound();
    	UserPlayerDelegate owner = MainWindow.getOwner();
        new SelectProjectWindow(null, planet);
        rebuildGalaxyMap();			
	}

    public CraftDelegate getSelectedCraftInOrbit() throws ApplicationError, ValidationError {
        MoveableOwnedObjectDelegate selection = getSelectedMoo();
        if (selection == null || !(selection instanceof CraftDelegate)) {
            throw new ValidationError(L.INVALID_COMMAND_PLEASE_SELECT_A_CRAFT_FIRST);
        }
        CraftDelegate craft = (CraftDelegate)selection;
        OrbitVO orbit = craft.getOrbit();
        if (orbit == null) {
            throw new ValidationError(L.INVALID_COMMAND_CRAFT_MUST_BE_IN_ORBIT_OF_A_SOLAR_SYSTEM);
        }
        return craft;
    }

    public MoveableOwnedObjectDelegate getSelectedCraftBaseInOrbit() throws ApplicationError, ValidationError {
        MoveableOwnedObjectDelegate selection = getSelectedMoo();
        if (selection == null || !(selection instanceof MoveableOwnedObjectDelegate)) {
            throw new ValidationError(L.INVALID_COMMAND_PLEASE_SELECT_A_CRAFT_FIRST);
        }
        MoveableOwnedObjectDelegate craft = (MoveableOwnedObjectDelegate)selection;
        OrbitVO orbit = craft.getOrbit();
        if (orbit == null) {
            throw new ValidationError(L.INVALID_COMMAND_CRAFT_MUST_BE_IN_ORBIT_OF_A_SOLAR_SYSTEM);
        }
        return craft;
    }

    public MoveableOwnedObjectDelegate getSelectedMooInOrbit() throws ApplicationError, ValidationError {
        MoveableOwnedObjectDelegate selection = getSelectedMoo();
        if (selection == null || !(selection instanceof MoveableOwnedObjectDelegate)) {
            throw new ValidationError(L.INVALID_COMMAND_PLEASE_SELECT_A_OBJECT_FIRST);
        }
        MoveableOwnedObjectDelegate moo = (MoveableOwnedObjectDelegate)selection;
        OrbitVO orbit = moo.getOrbit();
        if (orbit == null) {
            throw new ValidationError(L.INVALID_COMMAND_CRAFT_MUST_BE_IN_ORBIT_OF_A_SOLAR_SYSTEM);
        }
        return moo;
    }

    public MoveableOwnedObjectDelegate getSelectedMooIfInOrbit() throws ApplicationError, ValidationError {
        MoveableOwnedObjectDelegate selection = getSelectedMoo();
        if (selection == null || !(selection instanceof MoveableOwnedObjectDelegate)) {
            return null;
        }
        MoveableOwnedObjectDelegate moo = (MoveableOwnedObjectDelegate)selection;
        OrbitVO orbit = moo.getOrbit();
        if (orbit == null) {
            throw new ValidationError(L.INVALID_COMMAND_CRAFT_MUST_BE_IN_ORBIT_OF_A_SOLAR_SYSTEM);
        }
        return moo;
    }

    public PlanetDelegate getColonyToAttack(OrbitVO orbit, boolean includeMiningColonies, boolean mustHaveDefences) throws ApplicationError, ValidationError {
        StellarObjectDelegate so = DeligateCacheManager.getStellarObject(orbit.getStellarObject());
        if (so instanceof SolarSystemDelegate) {
        	SolarSystemDelegate ss = (SolarSystemDelegate) so;
        	PlanetDelegate[] colonies = getOwner().getAttackablePlanets(ss.getPlanets(), includeMiningColonies, mustHaveDefences);
        	if (colonies == null || colonies.length == 0) {
        		return null;
        	}
        	if (colonies.length == 1) {
        		return colonies[0];
        	} else if (orbit.getPlanet() != null && Comp.in(orbit.getPlanet(), colonies)) {
        		return DeligateCacheManager.getPlanet(orbit.getPlanet());
        	} else {
            	UserPlayerDelegate owner = getOwner();
        		SelectPlanetWindow spw = new SelectPlanetWindow(owner, null, colonies, L.SELECT_A_COLONY_TO_ATTACK);
        		spw.waitUntilClosed();
        		return spw.getSelectedPlanet();
        	}
        }
        return null;
    }

    public void setMapWindowSize() throws ApplicationError {
    	if (ProxyStore.hasInstance()) {    		
    		ServerProxy server = ProxyStore.getInstance();
    		if (server.isConnected() && view != null) {
    			GalaxyMapPanel mapWindow = view.getMapPanel();
    			JScrollPane mapWindowScrollPane = view.getMapPanelScrollPane();
    			GalaxyDetails galaxyDetails = DeligateCacheManager.getGalaxyDetails();
    			if (galaxyDetails != null) {    				
    				Size s = galaxyDetails.getSize();
    				int x = (int)(s.getX()*mapWindow.getZoomScale());
    				int y = (int)(s.getY()*mapWindow.getZoomScale());
    				mapWindow.setPreferredSize(new Dimension(x, y));
    				mapWindowScrollPane.getViewport().setViewSize(new Dimension(x,y));
    			}
    		}
    	}
    }
    
    public void focusOnHomeSolarSystem() throws ApplicationError {
    	UserPlayerDelegate owner = getOwner();
    	PlanetDelegate homePlanet = owner.getHomePlanet();
    	if (homePlanet != null) {
    		SolarSystemDelegate ss = homePlanet.getSolarSystem();
    		if (ss != null) {
    			focusOn(ss);
    		}
    		
    	}
    }

    public void focusOn(Delegate deligate) throws ApplicationError {
    	if (deligate == null) return;
    	Location loc = null;
    	if (deligate instanceof StellarObjectDelegate) loc = ((StellarObjectDelegate)deligate).getLocation();
    	if (deligate instanceof PlanetDelegate) loc = ((PlanetDelegate)deligate).getSolarSystem().getLocation();
    	if (deligate instanceof MoveableOwnedObjectDelegate) loc = ((MoveableOwnedObjectDelegate)deligate).getLocation();
    	if (loc != null) {
        	GalaxyMapPanel mapWindow = view.getMapPanel();
        	JScrollPane mapWindowScrollPane = view.getMapPanelScrollPane();
	    	float zoomScale = mapWindow.getZoomScale();
	        int x = (int)(loc.getX()*zoomScale);
	        int y = (int)(loc.getY()*zoomScale);
	        view.focus(x, y);
    	}
    }

    public void click(MoveableOwnedObjectDelegate moo) throws ApplicationError {
    	setSelection(moo);
    	playSound("SelectMoo");
    }

    public void doubleClick(MoveableOwnedObjectDelegate moo) throws ApplicationError, ValidationError {
    	openCraftDetailsWindow(null, moo, true);
    }

    public void openCraftDetailsWindow(MotInternalWindow parent, MoveableOwnedObjectDelegate moo, boolean rebuild) throws ApplicationError, ValidationError {
    	UserPlayerDelegate owner = getOwner();
        if (!moo.isOwnedBy(owner)) {
            throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
        }
        if (moo instanceof CraftDelegate) {
			playActionSound();
            new CraftDetailsWindow(parent, owner, (CraftDelegate)moo);
            if (rebuild) rebuildGalaxyMap();
        } else if (moo instanceof FleetDelegate) {
			playActionSound();
            new FleetDetailsWindow(parent, owner, (FleetDelegate)moo);
            if (rebuild) rebuildGalaxyMap();
        }
    }
    
    class InstructMooCallback implements CompletedCallback {
		private MoveableOwnedObjectDelegate selection;
		private StellarObjectDelegate so;
		private PlanetDelegate planet;

		public InstructMooCallback(MoveableOwnedObjectDelegate selection, StellarObjectDelegate so, PlanetDelegate planet) {
			this.selection = selection;
			this.so = so;
			this.planet = planet;
		}

		public void complete(Object result) throws ApplicationError, ValidationError {
            validateGalaxyCraftList();
            playSound("InstructMoo");
			clearSelection();
			rebuildGalaxyMap();
			if (so != null) {
	        	if (TutorialSystem.isActive()) {			
	        		TutorialSystem.notifyInstructMoo(selection, so, null);
	        	}
			} else if (planet != null) {
	        	if (TutorialSystem.isActive()) {			
	        		TutorialSystem.notifyInstructMoo(selection, planet.getSolarSystem(), planet);
	        	}
			}
		}    	
    }
    
	public static void validateGalaxyCraftList() throws ApplicationError, ValidationError {
		MainWindow mainWindow = MainWindow.getInstance();
		if (mainWindow.isGalaxyMapConfigurationActive()) {
			mainWindow.getGalaxyMapConfiguration().getGalaxyMapPanel().updateMooList(null);
		}
	}
	
	public void click(StellarObjectDelegate so, boolean autoselectPlanetOrbit) throws ApplicationError, ValidationError { 
		MoveableOwnedObjectDelegate selection = getSelectedMoo();
        if (selection != null) {
        	InstructMooCallback callback = new InstructMooCallback(selection, so, null);
            ProxyStore.getInstance().mooMoveCmd(selection, so, null, autoselectPlanetOrbit, callback, null);
        } else {
			playActionSound();
        	view.getContextPanel().showStellarObject(so);
    		view.getMapPanel().setSelection(so);
            rebuildGalaxyMap();
        }
	}
	
    public void doubleClick(StellarObjectDelegate so) throws ApplicationError, ValidationError { 
        if (so != null && so.getLastVisitedDate() >= 0) {
            if (so instanceof SolarSystemDelegate) {
    			playActionSound();
                SolarSystemDelegate ss = (SolarSystemDelegate)so;
            	UserPlayerDelegate owner = getOwner();
                new SolarSystemWindow(null, ss, owner);
    			rebuildGalaxyMap();			
            }
        }
    }

	public void click(PlanetDelegate planet) throws ApplicationError, ValidationError { 
		MoveableOwnedObjectDelegate selection = getSelectedMoo();
        if (selection != null) {
        	InstructMooCallback callback = new InstructMooCallback(selection, null, planet);
            ProxyStore.getInstance().mooMoveCmd(selection, planet.getSolarSystem(), planet, false, callback, null);
        } else {
			playActionSound();
        	view.getContextPanel().showPlanet(planet);
    		view.getMapPanel().setSelection(planet);
            rebuildGalaxyMap();

        }
	}
	
    public void doubleClick(PlanetDelegate planet, boolean visible) throws ApplicationError, ValidationError { 
        if (planet != null && planet.getSolarSystem().getLastVisitedDate() >= 0) {
			playActionSound();
        	UserPlayerDelegate owner = MainWindow.getOwner();
            new SolarSystemWindow(null, planet, owner);
			rebuildGalaxyMap();			
        }
    }

    public void openSolarSystemWindow(PlanetDelegate planet) throws ApplicationError, ValidationError { 
			playActionSound();
        	UserPlayerDelegate owner = MainWindow.getOwner();
            new SolarSystemWindow(null, planet, owner);
    }

    public MoveableOwnedObjectDelegate getSelectedMoo() {
    	NamedObjectDelegate selection = getSelection();
    	if (selection instanceof MoveableOwnedObjectDelegate) {
    		return (MoveableOwnedObjectDelegate) selection;
    	} else {
    		return null;
    	}
    }
    
    public PlanetDelegate getSelectedPlanet() {
    	NamedObjectDelegate selection = getSelection();
    	if (selection instanceof PlanetDelegate) {
    		return (PlanetDelegate) selection;
    	} else {
    		return null;
    	}
    }
    
    public StellarObjectDelegate getSelectedStellarObject() {
    	NamedObjectDelegate selection = getSelection();
    	if (selection instanceof StellarObjectDelegate) {
    		return (StellarObjectDelegate) selection;
    	} else {
    		return null;
    	}
    }
    
    public NamedObjectDelegate getSelection() {
    	return view.getContextPanel().getSelection();
    }
    
    public void clearSelection() throws ApplicationError {
    	view.getMapPanel().getObjects().clearSelection();
		view.getContextPanel().showEmpireMenu();
    }
    
    public void setSelection(MoveableOwnedObjectDelegate moo) throws ApplicationError {
    	view.getContextPanel().showMoo(moo);
    	view.getMapPanel().getObjects().clearSelection();
		view.getMapPanel().setSelection(moo);
        rebuildGalaxyMap();
	}
    
	public void next() throws ApplicationError, ValidationError {
		ActiveRectangleList objects = view.getMapPanel().getObjects();
		GalaxyMapMoo moo = objects.getNextMoo(getSelectedMoo());
		if (moo != null) {
			setSelection(moo.getMostVisibleMoo());
			focusOn(moo.getMostVisibleMoo());
		}
	}
    
	public void previous() throws ApplicationError, ValidationError {
		ActiveRectangleList objects = view.getMapPanel().getObjects();
		GalaxyMapMoo moo = objects.getPreviousMoo(getSelectedMoo());
		if (moo != null) {
			setSelection(moo.getMostVisibleMoo());
			focusOn(moo.getMostVisibleMoo());
		}
	}

	protected void rebuildGalaxyMap() throws ApplicationError {
		MainWindow mainWindow = MainWindow.getInstance();
		GalaxyWindowConfig config = mainWindow.getGalaxyMapConfiguration();
		config.getGalaxyView().getMapPanel().rebuildContent();
        mainWindow.repaint();
	}
	
	public void setView(GalaxyWindowView view) {
		this.view = view;
		super.setView(view);
	}

	protected String getKeyBindingsMessage() {
		return L.TT_KEYS_GALAXY;
	}

	public GalaxyWindowView getView() {
		return view;
	}

	public MoveableOwnedObjectDelegate[] getMoos(UserPlayerDelegate owner, boolean mustBeInOrbit) throws ApplicationError {
		GalaxyMapMoo[] gmma = view.getMapPanel().getObjects().getAllMoos(owner, mustBeInOrbit);
		if (gmma == null  || gmma.length == 0) return null;
		MoveableOwnedObjectDelegate [] r = new MoveableOwnedObjectDelegate[gmma.length];
		for (int i = 0; i < r.length; i++) {
			r[i] = gmma[i].getMostVisibleMoo();
		}
		return r;
	}


	public MoveableOwnedObjectDelegate[] getCrafts(UserPlayerDelegate owner, boolean mustBeInOrbit) throws ApplicationError {
		GalaxyMapMoo[] gmma = view.getMapPanel().getObjects().getAllMoos(owner, mustBeInOrbit);
		if (gmma == null  || gmma.length == 0) return null;
		ArrayList r = null;
		for (int i = 0; i < gmma.length; i++) {
			if (gmma[i].getMostVisibleMoo() instanceof MoveableOwnedObjectDelegate) {
				if (r == null) r = new ArrayList();
				r.add(gmma[i].getMostVisibleMoo());
			}
		}
		if (r == null) {
			return null;
		} else {			
			return (MoveableOwnedObjectDelegate[]) r.toArray(new MoveableOwnedObjectDelegate[r.size()]);
		}
	}


}
