/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.SwingUtilities;
import javax.swing.Timer;

import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.buttons.MotRolloverImageButton;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.config.MenuConfig;
import mot.client.swing.config.MenuImageConfig;
import mot.client.swing.window.galaxy.actions.ShortcutListener;
import mot.client.swing.window.main.MainWindow;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public abstract class GalaxyWindowMenuButton extends MotRolloverImageButton implements ShortcutListener, MouseMotionListener {



	/**
	 * This is the action listener for the galaxy window
	 */
	protected GalaxyWindowListener listener;

	/**
	 * This is the view that this listener uses to display its results.
	 */
	protected GalaxyWindowView view;
	
	
	private Character shortcut;
	private String tip;
	
	private static Color tipBackground;
	
	static {
		try {
			MenuConfig menuConfig = ImageConfigFactory.getInstance().getMenuConfig();
			tipBackground = menuConfig.getTipBackground().getColor();
		} catch (Exception e) {
			Logger.logError("", e);
		}
	}
	
	private int size;
	
	private static int getSize(String name) {
		int size = 55;		
		try {
			MenuConfig menuConfig = ImageConfigFactory.getInstance().getMenuConfig();
			MenuImageConfig imageConfig = menuConfig.getMenuImageConfig(name);
			if (imageConfig != null && imageConfig.getImageSize() != null) {
				size = imageConfig.getImageSize().getX();
			} else {
				size = menuConfig.getImageSize().getX();
			}
		} catch (Exception e) {
			Logger.logError("", e);
		}
		return size;
	}
	
	public static Color getTipBackground() { return tipBackground; }

	public GalaxyWindowMenuButton(GalaxyWindowListener listener, String name, Character shortcut) throws ApplicationError {
		super(ImageConfigFactory.getInstance().getMenuConfig().getMenuImageConfig(name).getImage(), getSize(name), name);
		this.size = getSize(name);
		this.listener = listener;
		this.view = listener.getView();
		this.shortcut = shortcut;
		if (shortcut != null) {
			listener.addShortcutLisneter(this);
		}
		//Custom tool tips have been disabled
		//addMouseMotionListener(this);
		addMouseWheelListener(listener);
	}

	public boolean keyPressed(char key, int modifiers) {
		if (shortcut != null && shortcut.charValue() == key) {
	        runAction();
	        return true;
		} else {
			return false;
		}
	}

    protected UserPlayerDelegate getOwner() {
    	return view.getListener().getOwner();
    }

	@Override
	public Character getShortcutKey() {
		return shortcut;
	}

    /**
     * If an internal window is open it throws a validation error and closes it.
     * @throws ValidationError 
     */
    protected void doPreAction() throws ApplicationError, ValidationError {
        ServerProxy server = ProxyStore.getInstance();
        if (!server.isConnected()) {
        	throw new ValidationError(L.CREATE_A_NEW_GAME_FIRST);
        }
		if (view.isDesktopPaneActive()) {
			// user has clicked on a menu item while a dialog window is open.
			// There may be events queued from the engine waiting to be processed after the window
			// is closed. These must be processed before we can open any new windows or perform
			// actions.
			view.deactivateDesktopPane();
			// So I will process the mail box instead of throwing an error.
			MainWindow.getInstance().processMailBox();
			//throw new ValidationError(L.CLOSE_WINDOW_FIRST_I_WILL_DO_IT_FOR_YOU_THIS_TIME);
		}
	}
	
	public String getTip() {
		return tip;
	}

	public void setTip(String t) {
		if (shortcut != null) {
			tip = "<html>" + t + "<br/>"+L.EXPRESS_KEY_+shortcut+"</html>";
		} else {
			tip = "<html>"+t+"</html>";
		}
		if (tip != null) {
			setToolTipText(tip);
		}
	}

	//
	// MOUSE MOTION LISTENER USED TO DISPLAY TOOL TIPS
	//
	
	public void mouseDragged(MouseEvent e) {
	}

    public Timer timer;

    public void mouseMoved(MouseEvent e) {
    	if (getTip() == null) {
    		return;
    	}
		if (timer == null) {
	    	ActionListener tl = new ActionListener() {
				public void actionPerformed(ActionEvent e) {
			    	if (Logger.isDebug()) {
			    		if (!SwingUtilities.isEventDispatchThread()) {
			    			Logger.logError("", new Exception("NOT IN EVENT THREAD"));
			    		}
			    	}
					timer.stop();
					try {
						MainWindow mainWin = MainWindow.getInstance();
						if (containsMouse() && !mainWin.isDisplayingTemporaryMessage()) {
							mainWin.showTemporaryMessage(getTip(), false, tipBackground, 0);
						}
					} catch (ApplicationError ex) {

						Logger.logError("Unable to display tip", ex);
					}
				}
	    	};
			timer = new Timer(1000, tl);
			timer.start();
		} else {
			timer.restart();
		}
	}

    protected void doPostAction() throws ApplicationError, ValidationError {
		MainWindow mainWindow = MainWindow.getInstance();
		if (mainWindow.isGalaxyMapConfigurationActive()) {
			UserPlayerDelegate owner = mainWindow.getWindowOwner();
			mainWindow.getGalaxyMapConfiguration().getGalaxyView().updateStatus(owner);
		}
    	view.getMapPanel().rebuildContent();
	}
    
}
