/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ImageUtils;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowConfig;
import mot.client.swing.window.main.MainWindowView;
import mot.common.bean.GalaxyDetails;
import mot.common.bean.GalaxyGraphicsList;
import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.bean.SectorInfluenceItem;
import mot.common.bean.SectorOwnerGrid;
import mot.common.bean.Size;
import mot.common.bean.StellarObjectGraphics;
import mot.common.factory.GraphicsFactory;
import mot.common.util.ApplicationError;

/**
 * This is a panel that shows the universe in a single small window. It only 
 * shows solar systems and radar coverage. It gives the user information about 
 * what part of the galaxy he or she is currently viewing. It also allows the
 * user to select a different area of the galaxy go view.  
 */
public class GalaxyWindowRadarPanel extends JPanel implements MouseListener, MouseMotionListener {


	private static final int SIZE_X = 15;
	private static final int SIZE_Y = 15;
	private static final int CARROT_SIZE = 15;
	
    private static GalaxyGraphicsList galaxyGraphics;
    
    private boolean drawViewRect;
    
    /**
     * If this is not null then the carrot should be displayed on the galaxy map.
     * This carrot allows the user to identify the location in the galaxy of an 
     * object he or she has selected.
     */
    private GalaxyWindowRadarCarrot carrot;
    private int carrotRotation = 0;

    public GalaxyWindowRadarPanel(int size) throws ApplicationError {
    	this(true, true, size);
    }

    public GalaxyWindowRadarPanel(boolean drawViewRect, boolean updatable, int size) throws ApplicationError {
    	this.drawViewRect = drawViewRect;
    	
    	if (galaxyGraphics == null) {
    		galaxyGraphics = GraphicsFactory.getGalaxyGraphicsList();
    	}
        PanelUtils.setTheBloodySize(this, size, size);

        setBorder(BorderFactory.createLineBorder(Color.WHITE));
        
        if (updatable) {
	        addMouseListener(this);
	        addMouseMotionListener(this);
        }
        
    }
    

    public StellarObjectDelegate[] getStellarObjects() throws ApplicationError {
		return DeligateCacheManager.getStellarObjects();
	}

	protected GalaxyDetails getGalaxyDetails() throws ApplicationError {
		return DeligateCacheManager.getGalaxyDetails();
	}
	
	protected UserPlayerDelegate getOwner() throws ApplicationError {
		UserPlayerDelegate owner = MainWindow.getOwner();
		return owner;
	}
	
	protected boolean readyForPainting() throws ApplicationError {
    	MainWindow mainWin = MainWindow.getInstance();
        GalaxyWindowConfig config = mainWin.getGalaxyMapConfiguration();
    	UserPlayerDelegate owner = getOwner(); 
        return (config != null && owner != null);
	}

    // !!!!!HACK!!!!!!
    // temporary variables that I do not want to pass around!
    private int width;
    private int height;
    private double scaleX;
    private double scaleY;
    
	public void paintComponent(Graphics g) { 
    	super.paintComponent(g);
    	
        try {
        	if (!readyForPainting()) {
        		return;
        	}
        	GalaxyDetails galaxyDetails = getGalaxyDetails();
            if (galaxyDetails == null || galaxyDetails.getSize() == null) {
            	return;
            }
            MainWindow mainWin = MainWindow.getInstance();
            Container container = mainWin.getContainer();
            
        	Size galaxySize = galaxyDetails.getSize();
        	width = getWidth();
        	height = getHeight();
        	scaleX = ((double)width) / galaxySize.getX();
        	scaleY = ((double)height) / galaxySize.getY();
        	drawGrid((Graphics2D)g);
        	StellarObjectDelegate [] sos = getStellarObjects();
        	for (int i=0; sos != null && i<sos.length; i++) {
        		drawStellarObject(container, (Graphics2D)g, galaxySize, sos[i]);
        	}
        	if (drawViewRect) {
        		MainWindowView activeView = MainWindow.getInstance().getActiveConfiguration().getView();
        		if (activeView instanceof GalaxyView) {
        			GalaxyMapPanel mapPanel = ((GalaxyView)activeView).getMapPanel();
        			drawViewRectangle((Graphics2D)g, galaxySize, mapPanel);
        		}
        	}
        	
	        if (carrot != null) {
	        	if (carrot.hasExpired()) {
	        		carrot = null;
	        	} else {
	        		Location loc = carrot.getLocation();
	                int x = (int) (loc.getX() * scaleX);
	                int y = (int) (loc.getY() * scaleY);
	        		String image = carrot.getCarrotImage();
					ImageUtils.drawRotatedImageCenteredAt(mainWin.getContainer(), (Graphics2D)g, image, x,y, CARROT_SIZE, CARROT_SIZE, carrotRotation);
					carrotRotation += 5;
					if (carrotRotation >= 360) carrotRotation = 0;
	        	}
	        }
        } catch (Exception e) {
            MainWindow.getExestingInstance().reportApplicationError(e);
        }
    }

	protected SectorOwnerGrid getSectorOwnerGrid() throws ApplicationError {
		return DeligateCacheManager.getSectorOwnerGrid();
	}

	protected Color getPlayerColour(String playerName) throws ApplicationError {
		PlayerDelegate player = DeligateCacheManager.getPlayer(playerName);
		if (player != null) {
			Color ownerColor = player.getColor().getSectorColor();
			return ownerColor;
		} else {
			return null;
		}
	}

	private void drawGrid(Graphics2D g) throws ApplicationError {
        SectorOwnerGrid sog = getSectorOwnerGrid();
        if (sog == null) return;
		int qtyRows = sog.getQtyRows();
		int qtyCols = sog.getQtyCols();
		double intervalX = width / (double)qtyCols; 
		double intervalY = height / (double)qtyRows; 
		for (int row = 0; row< qtyRows; row++) {
			for (int col = 0; col< qtyCols; col++) {
				SectorInfluenceItem owner = sog.getSectorOwner(row, col);
				if (owner != null) {
					Color ownerColor = getPlayerColour(owner.getPlayer());
					if (ownerColor != null) {
						g.setColor(ownerColor);
						int y = (int)(row * intervalY);
						int x = (int)(col * intervalX);
						g.fillRect(x, y, (int)intervalX, (int)intervalY);
					}
				}
			}		
		}
		g.setColor(Color.GRAY);
		for (int j = 0; j< qtyRows; j++) {
			int p = (int)(j * intervalY);
			g.drawLine(0, p, width, p);
		}
		for (int i = 0; i< qtyCols; i++) {
			int p = (int)(i * intervalX);
			g.drawLine(p, 0, p, height);
		}
	}
	
	private void drawStellarObject(Container container, Graphics2D g, Size galaxySize, StellarObjectDelegate so) throws ApplicationError {
		if (so == null || so.getGraphics(false) == null) {
			StellarObjectGraphics gr = so.getGraphics(false);
			throw new RuntimeException("");
			// TODO - REMOVE ME
		}
        StellarObjectGraphics starGraphics = so.getGraphics(false);
        String picture = starGraphics.getPicture(0);
        int x = (int) (so.getLocation().getX() * scaleX);
        int y = (int) (so.getLocation().getY() * scaleY);
        ImageUtils._drawImageCenteredAt(container, g, picture, x, y, SIZE_X, SIZE_Y);
	}

	private void drawViewRectangle(Graphics2D g, Size galaxySize, GalaxyMapPanel mapPanel) throws ApplicationError {
        Rectangle view = mapPanel.getViewArea();
        g.setColor(Color.WHITE);
        g.drawRect((int)(view.getLeft()*scaleX), (int)(view.getTop()*scaleY), (int)(view.getWidth()*scaleX), (int)(view.getHeight()*scaleY));
	}

    //private static long REPAINT_RADAR_INTERVAL = 1000;
    //private static long lastRadarPaint;

    public void handleTick() throws ApplicationError {
//        long time = System.currentTimeMillis();
//        if (time - lastRadarPaint > REPAINT_RADAR_INTERVAL) {
//        	lastRadarPaint = time;
//        	repaint();
//        }
    }	


	//
	// Mouse Listener
	//
	
	public void mouseClicked(MouseEvent e) {
	}
	public void mouseEntered(MouseEvent e) {
	}
	public void mouseExited(MouseEvent e) {
	}
	public void mousePressed(MouseEvent e) {
    	if (e.getClickCount() <= 1) {
    		try {
            	MainWindow mainWin = MainWindow.getInstance();
                MainWindowConfig activeConfiguration = mainWin.getActiveConfiguration();
                if (activeConfiguration == null || getGalaxyDetails() == null) return;
                MainWindowView view = activeConfiguration.getView();
                if ( !(view instanceof GalaxyView) ) return;
                GalaxyView galaxyView = (GalaxyView) view;
    	        Size galaxySize = getGalaxyDetails().getSize();
    			double scaleX = ((double)getWidth()) / galaxySize.getX();
    			double scaleY = ((double)getHeight()) / galaxySize.getY();
    			int x = (int)(e.getX() / scaleX);
    			int y = (int)(e.getY() / scaleY);
    			galaxyView.focusOnMap(x, y);
    		} catch (ApplicationError ex) {
    			MainWindow.getExestingInstance().getActiveConfiguration().getView().reportApplicationError(ex);
    		}
    	}
	}
	public void mouseReleased(MouseEvent e) {
	}

	//
	// Mouse Motion Listener
	//
	
	public void mouseDragged(MouseEvent e) {
		mousePressed(e);
	}
	public void mouseMoved(MouseEvent e) {
	}

	public GalaxyWindowRadarCarrot getCarrot() {
		return carrot;
	}

	public void setCarrot(GalaxyWindowRadarCarrot carrot) {
		if (this.carrot != carrot) {
			repaint();
		}
		this.carrot = carrot;
	}
}
