/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.KeyboardFocusManager;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.actions.ActiveRectangleList;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.BackgroundImagePanel;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.galaxy.actions.ActionChangeZoom;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowMessageList;
import mot.client.swing.window.main.MainWindowView;
import mot.client.tutorials.TutorialSystem;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.util.ValidationErrorSettings;

public class GalaxyWindowView extends MainWindowView implements MotGalaxyMapActions, GalaxyView {
	
	private GalaxyWindowListener listener;
    private Color backgroundColour = Color.BLACK;
    private GalaxyMapPanel mapPanel = null;
    private GalaxyWindowStatusPanel statusPanel;
    private MainWindowMessageList messageList = new MainWindowMessageList();
    private GalaxyZoomSlider zoomSlider;
    private GalaxyWindowRadarPanel radarPanel;
    private JPanel mainPanel;
    
    private ComponentTable controlPanel;
    private GalaxyWindowContextPanel contextPanel;

	/**
	 * If this window is currently displaying one or more dialog boxes this is the
	 * desktop pane containing the dialog boxes. Otherwise this is null and the main
	 * panel is displaying the galaxy map. 
	 */
	private JDesktopPane desktopPane;

	private Color menuBackgroundColor;
	private String menuBackgroundTop;
	private String menuBackgroundCorner;
	private String menuBackgroundRight;
	
	
    public GalaxyWindowView(GalaxyWindowListener listener) throws ApplicationError {
    	this.listener = listener;
    	listener.setView(this);
    	
        menuBackgroundColor = ImageConfigFactory.getInstance().getMenuConfig().getBackground().getColor();
       	menuBackgroundTop = ImageConfigFactory.getImageForConfig("MainMenuBackgroundTop");
       	menuBackgroundCorner = ImageConfigFactory.getImageForConfig("MainMenuBackgroundCorner");
       	menuBackgroundRight = ImageConfigFactory.getImageForConfig("MainMenuBackgroundRight");

       	contextPanel = new GalaxyWindowContextPanel(listener, menuBackgroundColor);
       	contextPanel.showEmpireMenu();
       	
        statusPanel = new GalaxyWindowStatusPanel(listener);
        messageList.setPreferredSize(new Dimension(800,50));

        
        mapPanel = new GalaxyMapPanel(listener, false, false);
        mapPanel.setOpaque(true);
        mapPanel.setBackground(backgroundColour);
        
        mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(mapPanel.getScrollPane(), BorderLayout.CENTER);
        mainPanel.add(messageList, BorderLayout.SOUTH);
        
        radarPanel = new GalaxyWindowRadarPanel(150);
        zoomSlider = new GalaxyZoomSlider(this);
        
        BackgroundImagePanel menuCornerPanel = new BackgroundImagePanel() {
			protected void drawSizeRelativeObjects() {
			}
			protected String getImage() {
				return menuBackgroundCorner;
			}
        };
        
        controlPanel = new ComponentTable("HV", 4, 1, 0, 3, 3);
        controlPanel.add(radarPanel, "HV");
        //controlPanel.add(new JPanel(), "HV");
        controlPanel.add(zoomSlider, "HV");
        controlPanel.add(contextPanel, "HVPS");
        controlPanel.add(statusPanel, "HV");
        controlPanel.setBackground(menuBackgroundColor);
        
        mapPanel.addMouseWheelListener(listener);
        mapPanel.getScrollPane().addMouseWheelListener(listener);
        mapPanel.getScrollPane().getViewport().addMouseWheelListener(listener);
        contextPanel.addMouseWheelListener(listener);
        controlPanel.addMouseWheelListener(listener);
        radarPanel.addMouseWheelListener(listener);
        mainPanel.addMouseWheelListener(listener);
    	
    }
    
    private void addImageButton(ComponentTable buttonPanel, GalaxyWindowMenuButton button, boolean last) throws ApplicationError {
    	if (last) {
        	buttonPanel.add(button, "HST");
    	} else { 
        	buttonPanel.add(button, "HV");
    	}
    }
    
    public void displaySettingsMenu() {
    	
    }

    public void displayCommonMenu() {
    	
    }
    
//    private JMenuItem newMenuItem(String name, ActionListener al, Character shortcut) {
//        return newMenuItem(name, null, al, shortcut);
//    }
//    
//    private JMenuItem newMenuItem(String name, String text, ActionListener al, Character shortcut) {
//        JMenuItem mi = new JMenuItem(text == null ? name : text);
//        mi.setActionCommand(name);
//        mi.addActionListener(al);
//        if (shortcut != null) {
//            mi.setMnemonic(shortcut.charValue());
//            mi.setAccelerator(KeyStroke.getKeyStroke(shortcut.charValue()));
//        }
//        return mi;
//    }

    public void deactivate() throws ApplicationError {
    	MainWindow win = MainWindow.getInstance();
    	win.getContentPane().removeAll();

    	KeyboardFocusManager.getCurrentKeyboardFocusManager().removeKeyEventDispatcher(listener);    	
    }
    
    public void activate() throws ApplicationError {
    	MainWindow win = MainWindow.getInstance();
    	win.getContentPane().add(mainPanel, BorderLayout.CENTER);
    	win.getContentPane().add(controlPanel, BorderLayout.EAST);

    	KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(listener);
    	mapPanel.setNowShowingNotification(new WindowShowingNotification( ) {
			public void nowShowingNotification() throws ApplicationError {
				if (zoomSlider != null) zoomSlider.requestFocusInWindow();
			}
		});  
    }

    public void handleTick() throws ApplicationError {
   		// Must not paint window while mouse is dragging screen around.
    	if (listener.mouseButton1Down) {
    		ActiveRectangleList objects = mapPanel.getObjects();
    		if (objects != null && objects.getMooList() != null && objects.getMooList().getScrollBar() != null) {
    			mapPanel.repaint();
    		}
    		return;
    	}
    	
   		if (desktopPane != null) {
   			JInternalFrame[] dialogs = desktopPane.getAllFrames();
   			if (dialogs != null) {
   				for (int i = 0; i < dialogs.length; i++) {
					if (dialogs[i] instanceof MotInternalWindow) {
						((MotInternalWindow) dialogs[i]).handleTick();
					}
				}
   			}
   		}
   		
   		contextPanel.handleTick();
        mapPanel.repaint();
        radarPanel.handleTick();
        messageList.tick();
        statusPanel.getEndTurnButton().handleTick();
    }
    
    public void repaintMap() {
        mapPanel.repaint(); 
        radarPanel.repaint();
    }
    
    /**
     * Set the centre of view port to scroll pane on location x,y.
     * @param x the centre to scroll to (x is in game engine coordinates)
     * @param y the centre to scroll to (y is in game engine coordinates)
     */
    public void focusOnMap(int x, int y) {
    	int mx = (int)(x * mapPanel.getZoomScale());
    	int my = (int)(y * mapPanel.getZoomScale());
    	focus(mx, my);
    }

	public void reportApplicationError(Exception e) {
		Logger.logError("MainWindow.actionPerformed() Failed due to error ", e);
		messageList.addMessage(e.getMessage());
	}

	public void reportValidationError(ValidationError e) {
		if (e.getSettings() == null) {
			messageList.addErrorMessage(e.getMessage(), true);
		} else {
			ValidationErrorSettings settings = e.getSettings();
			messageList.addErrorMessage(e.getMessage(), true, (int)(MainWindowMessageList.TIME_FOR_TEMP_MSGS_TO_EXPIRE*settings.getDurationPercent()));
		}
	}

	public void showMessage(String msg, boolean flash, Color flashBkColor, String sound) {
		if (flash) {
			messageList.addTemporaryMessage(msg, true, flashBkColor, sound, 0);
		}
		messageList.addMessage(msg);
	}

	public void showTemporaryMessage(String msg, boolean flash, Color bkColor, String sound, int additionalSecsToExpire) {		
		messageList.addTemporaryMessage(msg, flash, bkColor, sound, additionalSecsToExpire);
	}

	public boolean isDisplayingTemporaryMessage() {
		return messageList.isDisplayingTemporaryMessage();
	}
	
	
    public void updateStatus(UserPlayerDelegate owner) throws ApplicationError, ValidationError {
    	statusPanel.updateStatus(owner);
    }
    
	public MainWindowMessageList getMessageList() {
		return messageList;
	}

	public GalaxyMapPanel getMapPanel() {
		return mapPanel;
	}

	public JScrollPane getMapPanelScrollPane() {
		return mapPanel.getScrollPane();
	}

	public GalaxyZoomSlider getZoomSlider() {
		return zoomSlider;
	}
	
	public ActionChangeZoom getZoomSliderListener() {
		return zoomSlider.getListener();
	}

//	public GalaxyWindowMooList getMooList() {
//		return mooList;
//	}

	public GalaxyWindowRadarPanel getRadarPanel() {
		return radarPanel;
	}

	public GalaxyWindowStatusPanel getStatusPanel() {
		return statusPanel;
	}

	public GalaxyWindowListener getListener() {
		return listener;
	}
	
	public GalaxyWindowContextPanel getContextPanel() {
		return contextPanel;
	}
	
	public JDesktopPane activateDesktopPane() {
		if (desktopPane == null) {
			desktopPane = new JDesktopPane();
	    	mainPanel.remove(mapPanel.getScrollPane());
	    	mainPanel.add(desktopPane, BorderLayout.CENTER);
	    	mainPanel.validate();
	    	mainPanel.repaint();
		}
		return desktopPane;
	}

	public void deactivateDesktopPane() throws ApplicationError {
		if (desktopPane != null) {
	    	mainPanel.remove(desktopPane);
	        mainPanel.add(mapPanel.getScrollPane(), BorderLayout.CENTER);
			desktopPane = null;
	    	mainPanel.validate();
	    	mainPanel.repaint();
	    	if (zoomSlider != null) zoomSlider.requestFocusInWindow();
	    	if (TutorialSystem.isActive()) {
	    		TutorialSystem.notifySwitchToGalaxyMap();
	    	}
	    	mapPanel.rebuildContent();
//	        try {
//				EndTurnButton.validateGalaxyCraftList();
//			} catch (ValidationError e) {
//				Logger.logError("Ignored", e);
//			}
		}
	}
	
	/**
	 * This must be called when the main window is resized to make sure that any  
	 * open dialog boxes are resized.
	 */
	public void validate() {
		if (desktopPane != null) {
			Component[] components = desktopPane.getComponents();
			if (components != null) {
				for (int i = 0; i < components.length; i++) {
					if (components[i] instanceof MotInternalWindow) {
						MotInternalWindow miw = (MotInternalWindow) components[i];
						miw.checkForResize(desktopPane);
					}
				}
			}
		}	
		//zoomSlider.minZoomNeedsRecalculating = true;
	}
	
	public MotInternalWindow getTopWindow() {
		if (isDesktopPaneActive()) {
			JDesktopPane desktopPane = getDesktopPane();
			Component[] dialogs = desktopPane.getComponents();
			if (dialogs != null && dialogs.length > 0) {
				for (int i = 0; i < dialogs.length ; i++) {					
					if (dialogs[i] instanceof MotInternalWindow) {
						return (MotInternalWindow) dialogs[i];
					}
				}
			}
		}
		return null;
	}

	public void closeAllWindows() {
		if (isDesktopPaneActive()) {
			JDesktopPane desktopPane = getDesktopPane();
			Component[] dialogs = desktopPane.getComponents();
			if (dialogs != null && dialogs.length > 0) {
				for (int i = 0; i < dialogs.length ; i++) {					
					if (dialogs[i] instanceof MotInternalWindow) {
						((MotInternalWindow) dialogs[i]).hide();
					}
				}
			}
		}
	}
    

	public boolean hasOpenDialogThatStopsEndTurnButton() {
		if (isDesktopPaneActive()) {
			JDesktopPane desktopPane = getDesktopPane();
			Component[] dialogs = desktopPane.getComponents();
			if (dialogs != null && dialogs.length > 0) {
				for (int i = 0; i < dialogs.length; i++) {					
					if (dialogs[i] instanceof MotInternalWindow) {
						if ( ((MotInternalWindow) dialogs[i]).stopEndTurnButton() ) {
							return true;
						}
					}
				}
			}
		}
		return false;
	}
    
	public boolean isEndTurnInProgress() throws ApplicationError {
		return statusPanel.getEndTurnButton().isEndTurnInProgress();
	}

	/**
	 * @return TRUE when key events are to be processed globally - so this window cannot receive key input (text input).
	 */
	public boolean processKeyEventsGlobally() {
		MotInternalWindow topWindow = getTopWindow();
		if (topWindow != null) {
			return topWindow.processKeyEventsGlobally();
		} else {
			return true;
		}
	}

	public JDesktopPane getDesktopPane() {
		return desktopPane;
	}

	public JPanel getMainPanel() {
		return mainPanel;
	}

    public boolean isDesktopPaneActive() {
    	return (desktopPane != null);
    }

	public void refresh() throws ApplicationError, ValidationError {
		mapPanel.refresh();
	}
	
}
