/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy;

import java.awt.Container;
import java.awt.Graphics2D;

import mot.client.animation.GalaxyAnimation;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.animation.strokes.DuelImageStroke;
import mot.client.swing.animation.strokes.ImageStroke;
import mot.client.swing.animation.strokes.WeaponStroke;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageDuelStrokeAnimationConfig;
import mot.client.swing.config.ImageStrokeAnimationConfig;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;

public class TrailAnimationRange extends GalaxyAnimation {

	private static final String CONFIG_WARP = "WarpTrail";
	private static ImageStrokeAnimationConfig warpConfig;
	private static ImageStrokeAnimationConfig getWarpConfig() throws ApplicationError {
		if (warpConfig == null) {			
			warpConfig = AnimationConfigFactory.getInstance().getImageStrokeAnimationConfig((CONFIG_WARP));
		}
		return warpConfig;
	}
	
	private static final String CONFIG = "RangeTrail";
	private static ImageDuelStrokeAnimationConfig rangeConfig;
	private static ImageDuelStrokeAnimationConfig getConfig() throws ApplicationError {
		if (rangeConfig == null) {			
			rangeConfig = AnimationConfigFactory.getInstance().getImageDuelStrokeAnimationConfig((CONFIG));
		}
		return rangeConfig;
	}

	private static final int WIDTH = 5;
	
	private GalaxyMapMoo source;
	private Location srcScreenLocation;
	private Location destScreenLocation;
	private Container container;
	
	private WeaponStroke stroke;


	public TrailAnimationRange(GalaxyMapMoo source, GalaxyMapDestination destination, Container container) throws ApplicationError {
		this(source, destination, container, getConfig());
	}
	
	public TrailAnimationRange(GalaxyMapMoo source, GalaxyMapDestination destination, Container container, ImageDuelStrokeAnimationConfig config) throws ApplicationError {
		super(config.getLife(), config.getInterval());

		this.source = source;
		this.container = container;
		
		StellarObjectDelegate so = null;
    	if (destination instanceof GalaxyMapStellarObject) {
    		so = ((GalaxyMapStellarObject) destination).getStellarObject(); 
    	} else if (destination instanceof GalaxyMapPlanet) {
    		so = ((GalaxyMapPlanet)destination).getPlanet().getSolarSystem();
    	}
		
		MoveableOwnedObjectDelegate moo = source.getMostVisibleMoo();
		int range = moo.getRange();
		Location srcLocation = (moo.getOrbit() != null) ? DeligateCacheManager.getStellarObject(moo.getOrbit().getStellarObject()).getLocation() : moo.getLocation();
		Location destLocation = so.getLocation();
								
		srcScreenLocation = source.getCenterLocation();
		destScreenLocation = destination.getCenterLocation();
		
		int distance = srcLocation.getDistanceTo(destLocation);
		if (range < distance) {
			double percent = ((double)range) / ((double)distance);
			this.stroke = new DuelImageStroke(config, percent);
		} else {
			this.stroke = new ImageStroke(getWarpConfig());
		}
		
	}

	public void paint(Graphics2D graphics) throws ApplicationError {
		if (srcScreenLocation == null || destScreenLocation == null) {
			return;
		}
		Location s = new Location(srcScreenLocation);
		double direction = s.getAccurateDirectionFor(destScreenLocation);
		s.shiftInDirectionWithDouble(direction, WIDTH);
		stroke.paint(container, graphics, getStage(), s, destScreenLocation);
	}
	
    public boolean isComplete() {
    	return !source.contains(srcScreenLocation);
    }
}
