/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy.actions;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.window.galaxy.GalaxyMapListener;
import mot.client.swing.window.galaxy.GalaxyMapPanel;
import mot.client.swing.window.galaxy.GalaxyView;
import mot.client.swing.window.galaxy.GalaxyZoomSlider;
import mot.client.swing.window.galaxy.GalaxyZoomedPanel;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowListener;
import mot.common.bean.GalaxyDetails;
import mot.common.bean.Settings;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;

public class ActionChangeZoom implements ChangeListener, AdjustmentListener, MouseWheelListener {
	
	/**
	 * The panel that is being zoomed by this control
	 */
	private GalaxyZoomSlider zoomSlider;
	
	/**
	 * The view containing this zoom slider
	 */
	private GalaxyView view;

    /*
     * This is the point on the screen that was at the centre of the user's view
     * the last time the user moved the scroll bars.
     * It is important to remember this because the user may zoom right out to view
     * the galaxy, losing the centre focus (because it was not the centre of the 
     * galaxy) but when the user zooms in again he would like to zoom right back 
     * to where he was!
     */
    private double lastCenterX = -1, lastCenterY = -1;
    
    /*
     * When the zoom scale is being moved it will mode the scroll bars, but this 
     * move should not be considered as a move by the user of the scroll bars. 
     */
    private boolean _ignoreScrollBarChange = false;

    public ActionChangeZoom(GalaxyZoomSlider zoomSlider, GalaxyView view) {
		this.zoomSlider = zoomSlider;
		this.view = view;
		
		GalaxyZoomedPanel zoomedPanel = view.getMapPanel();
      	zoomedPanel.setZoomScale(calculateZoomScale());
		
	}
	
    private float calculateZoomScale() {
        int value = zoomSlider.getValue();
      	float zoomScale = GalaxyZoomSlider.MIN_ZOOM + (GalaxyZoomSlider.SCALE_FACTOR * value)/15.0f;
      	float minZoomScale = calculateMinimumZoom();
      	if (minZoomScale > zoomScale) {
      		return minZoomScale;
      	} else {
      		return zoomScale;
      	}
    }
    
    /**
     * Called when the zoom slider is modified on the main window!
     */
    public void stateChanged(ChangeEvent e) {
    	try {
			if (!ProxyStore.getInstance().isConnected()) {
				return;
			}
		} catch (ApplicationError e1) {
			Logger.logError("Zoom change broken", e1);
		}
    	_ignoreScrollBarChange = true;
        if (lastCenterX < 0.0 || lastCenterY < 0.0) {
        	adjustmentValueChanged(null);
        }
		GalaxyZoomedPanel zoomedPanel = view.getMapPanel();
      	float zoomScale = calculateZoomScale();
      	zoomedPanel.setZoomScale(zoomScale);
        try {
        	MainWindowListener listener = MainWindow.getInstance().getActiveConfiguration().getListener();
			if (listener instanceof GalaxyMapListener) {
				((GalaxyMapListener)listener).setMapWindowSize();
			}
//        	GalaxyWindowListener gwl = MainWindow.getInstance().getGalaxyMapListener();
//        	gwl.setMapWindowSize();
        } catch (ApplicationError ex) {
        	Logger.logError("", ex);
        }
        Rectangle viewRect = view.getMapPanelScrollPane().getViewport().getViewRect();
        double viewWidth = viewRect.getWidth()/zoomScale;
        double viewHeight = viewRect.getHeight()/zoomScale;
		double x = lastCenterX - viewWidth/2.0;
		double y = lastCenterY - viewHeight/2.0;
        double r = lastCenterX + viewWidth/2.0;
        double b = lastCenterY + viewHeight/2.0;
        try {
        	final Size galaxySize = DeligateCacheManager.getGalaxyDetails().getSize();
        	if (r > galaxySize.getX()) {
        		double delta =  r - galaxySize.getX();
        		x -=  delta;
        		lastCenterX -= delta; 
        	}
        	if (b > galaxySize.getY()) {
        		double delta =  b - galaxySize.getY();
        		y -= delta;
        		lastCenterY -= delta;
        	}
		} catch (Exception e2) {
			Logger.logError("Unable to check zoom value", e2);
		}
        if (x < 0.0) {
        	lastCenterX -= x;
        	x = 0.0;
        }
        if (y < 0.0) {
        	lastCenterY -= y;
        	y = 0.0;
        }
    	Point topLeft = new Point((int)(x*zoomScale), (int)(y*zoomScale));
    	view.getMapPanelScrollPane().getViewport().setViewPosition(topLeft);
    	_ignoreScrollBarChange = false;
    	view.getMapPanel().rebuildContent();
		view.getRadarPanel().repaint();
    }

    /**
     * Called when the one of the scroll bars is moved on the main window!
     */
	public void adjustmentValueChanged(AdjustmentEvent event) {
		if (!_ignoreScrollBarChange) {
	    	Rectangle r = view.getMapPanelScrollPane().getViewport().getViewRect();
			GalaxyZoomedPanel zoomedPanel = view.getMapPanel();
	    	float zoomScale = zoomedPanel.getZoomScale();
	    	lastCenterX = r.getCenterX()/zoomScale;
	    	lastCenterY = r.getCenterY()/zoomScale;
		}
    	view.getMapPanel().rebuildContent();
		view.getRadarPanel().repaint();
	}

    /**
     * Called when the user rolls the mouse wheel
     */
	public void mouseWheelMoved(MouseWheelEvent e) {
		int speed;
		try {
			speed = Settings.getInstance().getMouseScrollWheelSpeed();
		} catch (ApplicationError e1) {
			Logger.logError("ActionChangeListener.mouseWheelMoved() Unable to get speed", e1);
			speed = 3;
		}
		int notches = e.getWheelRotation()*speed;
		zoomSlider.setValue(zoomSlider.getValue()-notches);
	}

	/**
	 * Calculates the minimum value that zoom can be on the Galaxy Map Panel.
	 * NB - Must not be able to zoom out so far that the galaxy is smaller than the window size. 
	 * @throws ApplicationError 
	 */
	private float calculateMinimumZoom() {
		float minZoom = 1;
		try {
			GalaxyMapPanel mapPanel = view.getMapPanel();
			Dimension mapPanelSize = mapPanel.getScrollPane().getViewport().getExtentSize();
			if (ProxyStore.hasInstance()) {
				ServerProxy serverProxy = ProxyStore.getInstance();
				if (serverProxy.isConnected()) {
					GalaxyDetails galaxyDetails = DeligateCacheManager.getGalaxyDetails();
					if (galaxyDetails != null) {
						final Size galaxySize = galaxyDetails.getSize();
						float minZoomX =  (float) (mapPanelSize.getWidth() / galaxySize.getX());
						float minZoomY =  (float) (mapPanelSize.getHeight() / galaxySize.getY());
						minZoom = Comp.max(minZoomX, minZoomY);
						if (minZoom < 1) minZoom = 1;
					}
				}
			}
		} catch (Exception e) {
			Logger.logError("ActionZoomChange.calculateMinimumZoom() Unable to recalculate minimum zoom due to error", e);
		}
		return minZoom;
	}

}
