/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy.actions;

import java.awt.Color;
import java.awt.event.ActionListener;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.window.galaxy.GalaxyMapPanel;
import mot.client.swing.window.galaxy.GalaxyWindowView;
import mot.client.swing.window.galaxy.MotGalaxyMapActions;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowMessageList;
import mot.client.tutorials.TutorialSystem;
import mot.common.bean.Settings;
import mot.common.event.NewBattleEvent;
import mot.common.event.StartTurnEvent;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;
import mot.common.util.ValidationErrorSettings;

public class EndTurnButton extends MotButton implements MotGalaxyMapActions, ActionListener {

	private boolean endTurnInProgress = false;
	private boolean newTurnStarted = false;
	private boolean flashOn = false;
	private long lastEndTurnTime = 0;
	
	private boolean sleepWarningIssued = false;
	
	public EndTurnButton() {
		updateTextEndTurn();
		//setToolTipText(L.END_TURN_HELP);
	}

	protected void _doAction() throws ApplicationError, ValidationError {
		long time = System.currentTimeMillis();
		if (!endTurnInProgress && time - lastEndTurnTime > 300) {
			if(Settings.getInstance().isMoveCraftWarning() && !sleepWarningIssued && !TutorialSystem.isActive()) {
				if (!allMoosMoved()) {
					sleepWarningIssued = true;
					throw new ValidationError(L.END_TURN_WARNING_NOT_ALL_CRAFTS_MOVED, new ValidationErrorSettings(2.0f));
				}
			}
			if (sleepWarningIssued) {
				MainWindowMessageList messageList = MainWindow.getExestingInstance().getGalaxyMapConfiguration().getGalaxyView().getMessageList();
				messageList.removeTemporaryMessageWithText(L.END_TURN_WARNING_NOT_ALL_CRAFTS_MOVED);
			}
			sleepWarningIssued = false;
			endTurnInProgress = true;
			newTurnStarted = false;
			lastEndTurnTime = time;
			flashOn = true;
			intervalCount = 0;
    		GalaxyWindowView galaxyView = MainWindow.getInstance().getGalaxyMapConfiguration().getGalaxyView();
    		GalaxyMapPanel mapPanel = galaxyView.getMapPanel();
			if (mapPanel.isShowingMooListWithTargetDestination()) galaxyView.getListener().clearSelection();
			galaxyView.getMessageList().notifyEndTurn();
			ProxyStore.getInstance().endTurnCmd(null, this);
		} else {
			throw new ValidationError(L.BUSY_NEW_TURN_STARTES_AFTER_ALL_REQUESTS_ARE_COMPLETED);
		}
	}
	
	public void _testDoAction() throws ApplicationError, ValidationError {
		lastEndTurnTime = 0;
		_doAction();
	}
	
	private boolean allMoosMoved() throws ApplicationError {
        ServerProxy proxy = ProxyStore.getInstance();
        MoveableOwnedObjectDelegate[] moos = proxy.getMoos(false);
        if (moos == null) return true;
        UserPlayerDelegate owner = MainWindow.getOwner();
        for (int i = 0; i < moos.length; i++) {
        	if (moos[i].isOwnedBy(owner)) {        		
        		if (!(moos[i].isSleeping() || moos[i].hasMoved())) {
        			return false;
        		}
        	}
		}
		return true;
	}

	public void reportValidationError(ValidationError ex) {
		try {
			if (!isEndTurnInProgress()) {
				endTurnInProgress = false;
				updateTextEndTurn();
			}
		} catch (ApplicationError e) {
			Logger.logError("EndTurn", e);
		}
		super.reportValidationError(ex);
	}
	
	public void reportApplicationError(Exception ex) {
		try {
			if (!isEndTurnInProgress()) {
				endTurnInProgress = false;
				updateTextEndTurn();
			}
		} catch (ApplicationError e) {
			Logger.logError("EndTurn", e);
		}
		super.reportApplicationError(ex);
	}

	private static Color busyColour = null;	
	private static final Color getBusyColour() throws ApplicationError {
		if (busyColour == null) {
			busyColour = PropertyManager.getMandatoryAsColor("window.main.endturn_busy.background");
		}
		return busyColour;
	}

	private static Color normalColour = null;	
	private static final Color getNormalColour() throws ApplicationError {
		if (normalColour == null) {
			normalColour = PropertyManager.getMandatoryAsColor("window.main.colour");
		}
		return normalColour;
	}

	private static int intervalCount = 0;
	
	public void handleTick() throws ApplicationError {
		if (endTurnInProgress) {			
			boolean busy = isEndTurnInProgress();
			if (busy) {
				if (MainWindow.hasInstance() && MainWindow.getExestingInstance().getEventMailBox().hasEventTypeInQueue(StartTurnEvent.class, NewBattleEvent.class)) {
					setText(L.WAITING);
					setBackground(getNormalColour());
				} else if (intervalCount < 1) {
					intervalCount = 5;
					setText(L.WAITING);
					if (flashOn) {
						setBackground(getNormalColour());
						flashOn = false;
					} else {
						setBackground(getBusyColour());
						flashOn = true;
					}
				} else {
					intervalCount--;
				}
			} else {
				endTurnInProgress = false;
				setBackground(getNormalColour());
				updateTextEndTurn();
				if (TutorialSystem.isActive()) {			
					TutorialSystem.notifyEndTurnComplete();
				}
			}
		}
	}

	public boolean isEndTurnInProgress() throws ApplicationError {
		if (!newTurnStarted) return true;
		boolean r = false;
		if (MainWindow.hasInstance()) {
			MainWindow mainWindow = MainWindow.getExestingInstance();
			if (mainWindow.getEventMailBox() != null) {
				r = mainWindow.getEventMailBox().hasGalaxyEventsInQueue();
			}
			if (!r && mainWindow.getGalaxyMapListener() != null) {
				r = mainWindow.getGalaxyMapListener().getView().hasOpenDialogThatStopsEndTurnButton();
			}
		}
		return r;
	}

	public void notifyTurnStarted() {
		newTurnStarted = true;
	}

	public void setEndTurnInProgress(boolean v) {
		endTurnInProgress = v;
	}

	public void updateTextEndTurn() {
		setText(L.END_TURN);
	}
	


}
