/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy.actions;

import mot.client.actions.ActiveRectangle;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.proxy.ClientErrorHandler;
import mot.client.sounds.SoundEffect;
import mot.client.sounds.Sounds;
import mot.client.swing.common.MotToolTip;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.config.MenuConfig;
import mot.client.swing.config.MenuImageConfig;
import mot.client.swing.window.galaxy.GalaxyMapPanel;
import mot.client.swing.window.galaxy.GalaxyWindowListener;
import mot.client.swing.window.main.MainWindow;
import mot.client.tutorials.TutorialSystem;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public abstract class GalaxyMapButton extends ActiveRectangle implements ShortcutListener, ClientErrorHandler {

	protected GalaxyWindowListener listener;
	private MenuImageConfig menuImageConfig;
	private Character shortcut;

	public GalaxyMapButton(GalaxyWindowListener listener, String name, Character shortcut, int x, int y) throws ApplicationError {
		super(x,y, getSize(name), getSize(name), true);
		
		this.listener = listener;
		this.menuImageConfig = ImageConfigFactory.getInstance().getMenuConfig().getMenuImageConfig(name);
		this.shortcut = shortcut;
		if (shortcut != null) {
			listener.addShortcutLisneter(this);
		}
		//Custom tool tips have been disabled
		//addMouseMotionListener(this);
		//addMouseWheelListener(listener);
	}

	private static int size = 0;
	private String tipText;		
	public static int getSize(String name) {
		if (size > 0) {
			return size;
		}
		size = 55;
		try {
			MenuConfig menuConfig = ImageConfigFactory.getInstance().getMenuConfig();
			MenuImageConfig imageConfig = menuConfig.getMenuImageConfig(name);
			if (imageConfig != null && imageConfig.getImageSize() != null) {
				size = imageConfig.getImageSize().getX();
			} else {
				size = menuConfig.getImageSize().getX();
			}
		} catch (Exception e) {
			Logger.logError("", e);
		}
		return size;
	}
	
	protected void setTip(String tipText) {
		if (shortcut == null) {
			this.tipText = "<html>"+tipText+"</html>";
		} else {
			this.tipText = "<html>"+tipText + " [Shortcut "+shortcut+"]"+"</html>";
		}
	}

	protected abstract void _doAction() throws ApplicationError, ValidationError;
	
	public void mouseClicked(int x, int y) throws ApplicationError, ValidationError { 
		click();
	}

	public void click() throws ApplicationError, ValidationError {
		try {
			SoundEffect sound = Sounds.getActionSound();
			if (sound != null) {
				sound.play();
			}
		} catch (ApplicationError e) {
        	Logger.logError("Sound failed", e);
		}
		
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.validateClick(this);
    	}
		
    	_doAction();
	}

	public void doubleClick() throws ApplicationError, ValidationError {
	}

	public boolean keyPressed(char key, int modifiers) throws ApplicationError, ValidationError {
		if (shortcut != null && shortcut.charValue() == key) {
			_doAction();
	        return true;
		} else {
			return false;
		}
	}

	public Character getShortcutKey() {
		return shortcut;
	}

	/**
	 * Active rectangle use the observer pattern. This method is called when the moo this
	 * observer is watching has changed.
	 */
	public void update() throws ApplicationError {
	}

	public boolean contains(NamedObjectDelegate o) {
		return false;
	}

	public boolean contains(String name) {
		return false;
	}

	public NamedObjectDelegate getContents() {
		return null;
	}

	public void reportValidationError(ValidationError ex) {
		try {
			SoundEffect sound = Sounds.getActionSound();
			if (sound != null && !sound.isPlaying()) {
				sound.play();
			}
		} catch (ApplicationError e) {
			Logger.logError("Stop Sound failed", e);
		}
        MainWindow.getExestingInstance().reportValidationError(ex);
	}
	
	public void reportApplicationError(Exception ex) {
    	MainWindow.getExestingInstance().reportApplicationError(ex);
	}

	private static String ROLL_OVER = "_rollover";
	private static String EXTENSION = ".png";
	
	private String normalImage;
	private String rolloverImage;
	
	private boolean mouseIn;
	
	private String getRolloverImage() {
		if (rolloverImage == null) {
			rolloverImage = menuImageConfig.getImage()+ROLL_OVER+EXTENSION;
		}
		return rolloverImage;
	}

	private String getNormalImage() {
		if (normalImage == null) {
			normalImage = menuImageConfig.getImage()+EXTENSION;
		}
		return normalImage;
	}

	public String getPicture() {
		return mouseIn ? getRolloverImage() : getNormalImage();
	}

	public boolean isMouseIn() {
		return mouseIn;
	}

	public void setMouseIn(boolean mouseIn) {
		this.mouseIn = mouseIn;
	}

	public void showToolTip() {
		int x = getLeft();
		int y = getBottom() - 5;
		GalaxyMapPanel mapPanel = listener.getView().getMapPanel();
		MotToolTip.showTipWindow(mapPanel, x, y, tipText);
	}

	public Character getShortcut() {
		return shortcut;
	}

	public void setShortcut(Character shortcut) {
		this.shortcut = shortcut;
	}

}
