/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.galaxy.actions;

import java.util.ArrayList;

import mot.client.deligates.CraftDelegate;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.swing.window.craft.CraftDesignsWindow;
import mot.client.swing.window.galaxy.GalaxyMapPanel;
import mot.client.swing.window.galaxy.GalaxyWindowListener;
import mot.client.swing.window.galaxy.GalaxyWindowView;
import mot.client.swing.window.galaxy.MotGalaxyMapActions;
import mot.client.swing.window.main.MainWindow;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.designs.CraftDesignsSupplier;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.common.vo.CraftVO;

public class GalaxyMapButtonCraftUpgrade extends GalaxyMapButton implements MotGalaxyMapActions, CompletedCallback, CraftDesignsSupplier {

	public GalaxyMapButtonCraftUpgrade(GalaxyWindowListener listener, int x, int y) throws ApplicationError {
		super(listener, MENU_CRAFT_UPGRADE, null, x, y);
		setTip(L.UPGRADE_THE_CURRENTLY_SELECTED_CRAFT);
	}

	private MoveableOwnedObjectDelegate selection;

	protected void _doAction() throws ApplicationError, ValidationError {
    	MainWindow mainWindow = MainWindow.getInstance();
        selection = listener.getSelectedMoo();
        if (selection == null) {
        	throw new ValidationError(L.INVALID_COMMAND_PLEASE_SELECT_A_CRAFT_FIRST);
        }
        if (selection.getOrbit() == null || selection.getOrbit().getPlanet() == null) {
        	throw new ValidationError(L.INVALID_COMMAND_CRAFT_MUST_BE_IN_ORBIT_OF_A_PLANET);
        }
        PlanetDelegate planet = DeligateCacheManager.getPlanet(selection.getOrbit().getPlanet());
        if (planet == null) {
        	throw new ValidationError(L.INVALID_COMMAND_CRAFT_MUST_BE_IN_ORBIT_OF_A_PLANET);
        }
        if (planet.getProjectDesign() != null) {
        	throw new ValidationError(L.YOU_CANNOT_UPGRADE_A_CRAFT_ON_A_PLANET_THAT_HAS_A_PROJET);
        }
        UserPlayerDelegate owner = MainWindow.getOwner();
		CraftDesigns possibleDesigns = getCraftDesigns();
        if (possibleDesigns == null || possibleDesigns.getQuantity() == 0) {
        	MainWindow.getExestingInstance().showTemporaryMessage("No Craft Designs Suitable (must have same hull)", true, null, 0);
        }
		CraftDesignsWindow win = new CraftDesignsWindow(null, owner, owner, this, possibleDesigns);
		win.waitUntilClosed();
		CraftDesign cd = win.getSelectedCraftDesign();
		if (cd != null) {
			CraftVO[] craftsToUpgrade = getCraftsToUpgrade(cd, selection);
			if (craftsToUpgrade == null) {
	        	throw new ValidationError(L.INVALID_COMMAND);
			}
			CraftUpgradeDesign project = new CraftUpgradeDesign(cd);
			project.setCrafts(craftsToUpgrade);
			ProxyStore.getInstance().setProjectCmd(planet, project, 0, false, false, this, this);
		}
	}
	
	@Override
	public void complete(Object result) throws ApplicationError, ValidationError {
		GalaxyWindowView view = listener.getView();
    	GalaxyMapPanel mapPanel = view.getMapPanel();
        view.getStatusPanel().updateStatus(MainWindow.getOwner());
        listener.clearSelection();
        mapPanel.rebuildContent();
	}

	public void doubleClick() throws ApplicationError, ValidationError {
	}

	public CraftDesigns getCraftDesigns() throws ApplicationError, ValidationError {
		UserPlayerDelegate owner = MainWindow.getOwner();
        MoveableOwnedObjectDelegate moo = listener.getSelectedCraftBaseInOrbit();
		if (owner == null || moo == null) return null;
		CraftDesigns cds = owner.getCraftDesigns();
		if (cds == null || cds.getQuantity() == 0) return null;
		return cds.getCraftUpgradeDesignsFor(moo);
	}

	private CraftVO [] getCraftsToUpgrade(CraftDesign craftDesign, MoveableOwnedObjectDelegate moo) throws ApplicationError {
		if (moo instanceof CraftDelegate) {
			CraftDelegate craft = (CraftDelegate) moo;
			CraftVO [] r = new CraftVO [1];
			r[0] = craft.getCraftValue();
			return r;
		} else if (moo instanceof FleetDelegate) {
			FleetDelegate fleet = (FleetDelegate)moo;
			CraftDelegate[] crafts = fleet.getCrafts();
			if (crafts != null) {
				ArrayList<CraftVO> r = null;
				for (int i = 0; i < crafts.length; i++) {
					if (CraftDesigns.canUpgradeTo(crafts[i], craftDesign)) {
						if (r == null) r = new ArrayList();
						r.add(crafts[i].getCraftValue());
					}
				}
				if (r != null) {
					return r.toArray(new CraftVO[r.size()]); 
				}
			}
		}
		return null;
	}

}
