/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.main;

import java.awt.Color;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowStateListener;

import javax.swing.ToolTipManager;

import mot.client.animation.AnimationContainer;
import mot.client.animation.GalaxyAnimationRegister;
import mot.client.animation.GraphicalAnimationContainer;
import mot.client.animation.MovementRegister;
import mot.client.bean.GameEditor;
import mot.client.bean.GameEditorConfig;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.event.EventMailBox;
import mot.client.proxy.ClientErrorHandler;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.MotServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.sounds.JukeBox;
import mot.client.sounds.Sounds;
import mot.client.swing.animation.ExplodeAnimation;
import mot.client.swing.animation.HologramDestroyedAnimation;
import mot.client.swing.animation.TimeWarpAnimation;
import mot.client.swing.animation.TroopBattleAnimation;
import mot.client.swing.common.TimerMailBox;
import mot.client.swing.common.TimerMain;
import mot.client.swing.window.battle.BattleWindowConfig;
import mot.client.swing.window.common.MotFrame;
import mot.client.swing.window.etc.actions.NewGameButton;
import mot.client.swing.window.galaxy.GalaxyWindowConfig;
import mot.client.swing.window.galaxy.GalaxyWindowListener;
import mot.client.swing.window.galaxy.GalaxyWindowView;
import mot.client.telemetry.Telemetry;
import mot.client.tutorials.TutorialSystem;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;


/**
 * The Main Window for the game. In full screen mode (see settings.xml for details) this will
 * cover the entire window and take control of the graphics device. Therefore this is a 
 * singleton class.
 * 
 * The main window is used to display the galaxy map as well as the battle window (to fight
 * a battle at the end of a turn). Therefore the MainWindow has plugable contents. Each 
 * type of content is called a configuration. The attribute configurations defined the list
 * of all configurations defined in the main window. 
 * 
 * Each configuration consists of;
 * 		- A View : Menus and control laid out in panels
 * 		- A Listener : Definitions of actions and current state
 * 
 * To switch from one configuration to another the old configuration must be decactivated
 * and then the new configuration activated (using the methods defined on the configuration).
 * The deactivation removes the menus, components and key listeners. While the activate adds
 * the menus, components and key listeners.
 * 
 * There can only be one active configuration - the window configuration that is currently 
 * active (and there should always be an active configuration - otherwise there is no main
 * window being displayed!).
 * 
 * This window also contains the game timer that is used to animate all aspects of the game.
 * It is the timer contained in this window that causes 
 * 		- The main window to be repainted x times a second
 * 		- Panels in dialog boxes to be painted every y milli seconds causing an animation
 * 
 * When in full screen mode custom painting is used so the main window keeps a record of 
 * every sub window opened using the WindowRegister. This allows all sub windows to be painted
 * in the custom drawing loop for full screen mode.
 * 
 * The main window is a animation container and so implements the GraphicalAnimationContainer 
 * interface. This is required on the battle field for weapon fire animations, craft destroyed
 * Animations etc.
 */
public class MainWindow extends MotFrame implements WindowListener, ComponentListener, GraphicalAnimationContainer, ClientErrorHandler, WindowStateListener {

    private static MainWindow instance = null;
    
    /**
     * The MainWindow is a singleton class because in Full Window Mode the a 
     * single game window takes control of the entire video device. This method
     * is used to create or fetch the single instance of the MainWindow.
     */
    public static MainWindow getInstance() throws ApplicationError {
    	if (instance == null) {
    		synchronized (MainWindow.class) {
    	    	if (instance == null) {
    	    		instance = new MainWindow();
    	    		instance.init();
    		        instance.activeConfiguration.activate();
    		        new ProxyStore(new MotServerProxy(false, instance, null));
    	    	}
    	    	JukeBox.getInstance().start();
			}
    	}
    	return instance;
    }
    
	public static MainWindow getExestingInstance() {
    	if (instance == null) {
    		Logger.logError("", new ApplicationError("No Instance exists"));
    	}
    	return instance;
    }
	
    /**
     * THIS IS MEANT FOR TEST PURPOSES ONLY
     */
	public static MainWindow __createTestMainWindow(MainWindowConfig config) throws ApplicationError {
    	instance = new TestMainWindow(config);
    	return instance;
    }

    
    public static boolean hasInstance() {
    	return (instance != null);
    }

	public static boolean hasDialogOpen() {
		if (instance == null) return false;
		MainWindowView view = instance.getActiveConfiguration().getView();
		return view != null && view.isDesktopPaneActive();
	}


    /**
     * The queue of events sent by the engine to this client and waiting to be processed.
     * Must check the mail box every frame and process pending events. 
     */
    private EventMailBox eventMailBox;
    
    /**
     * This is the timer that causes the windows re-painted every xx milli seconds.
     */
    private TimerMain timer;
    
    /**
     * This is the timer that polls for events sent to this client from the engine to be processed..
     */
    private TimerMailBox mailBoxTimer;
    
    /**
     * This is the configuration in the list of configurations that is active.
     * There should always be one active configuration!
     */
    private MainWindowConfig activeConfiguration;
    
    /**
     * This configuration defines the contents of the main window when displaying the
     * Galaxy Map.
     */
    private GalaxyWindowConfig galaxyWinConfig;

    /**
     * This configuration defines the contents of the main window when displaying a
     * battle field.
     */
    private BattleWindowConfig battleWinConfig;
    
    /**
     * List of messages to be displayed in the galaxy window.
     * When the battle window is open, messages for the galaxy window are held in this pending list until the active configuration is changed.
     * This is to prevent messages from being lost by being sent when the battle window is just about to close.
     */
    private PendingMessageList pendingGalaxyMessages = new PendingMessageList();
    
    /**
     * List of messages to be displayed in the battle window.
     * When the galaxy window is open, messages for the battle window are held in this pending list until the active configuration is changed.
     */
    private PendingMessageList pendingBattleMessages = new PendingMessageList();

    /**
     * True if the window is paused - all animation stops. 
     * This is normally false and primarily intended for debugging.
     */
	private boolean paused = false;
    
    /**
     * This is a singleton class - Use the status method getInstance().
     */
    private MainWindow() throws ApplicationError {
    	// Do not initialise in constructor because menu cannot determine screen size
    }
    
    private void init() throws ApplicationError {
        // Start the timer (this runs the entire game)!
        timer = new TimerMain();
        // Start the timer (this receives events from the engine)!
        mailBoxTimer = new TimerMailBox();
        
        // Create the configurations
        galaxyWinConfig = new GalaxyWindowConfig();
        battleWinConfig = new BattleWindowConfig();
        activeConfiguration = galaxyWinConfig;
        
        addWindowListener(this);
        addComponentListener(this);
        addWindowStateListener(this);
            
        setVisible(true);

        ToolTipManager.sharedInstance().setDismissDelay(30000);
        ExplodeAnimation.loadAllImages(getWindow());
        TroopBattleAnimation.loadAllImages(getWindow());
        HologramDestroyedAnimation.loadAllImages(getWindow());
        TimeWarpAnimation.loadAllImages(getWindow());
    }

    protected MainWindow(MainWindowConfig config) throws ApplicationError {
        activeConfiguration = config;
        
        if (config instanceof TestBattleWindowConfig) {
	        battleWinConfig = (TestBattleWindowConfig)config;
        } else if (config instanceof TestGalaxyWindowConfig) {
        	galaxyWinConfig = (GalaxyWindowConfig) config;
        }
    }

    private static long t = 0;
	/**
     * repaints the screen fetching the new craft information. 
     */
    public void handleTick() throws ApplicationError, ValidationError {
//    	long st = System.currentTimeMillis();
//    	System.out.println("MainWindow.handleTick() Interval "+(st-t));
//    	t = st;
    	if (Telemetry.ON) {
    		Telemetry.checkIntervalComplete();
    		Telemetry.qtyTick++;
    	}
    	if (paused) return;
    	MovementRegister.tick();  // planets spinning
        activeConfiguration.handleTick(); // moo list
      	GalaxyAnimationRegister.tick(); // planets orbiting the sun
    	if (TutorialSystem.isActive()) {
    		TutorialSystem.handleTick();
    	}
//    	st = System.currentTimeMillis();
//    	System.out.println("MainWindow.handleTick() Cost "+(st-t));
//    	t = st;
    }
        
    /**
     * poll and process events sent from the engine to this client.  
     */
    public void processMailBox() throws ApplicationError, ValidationError {
		ServerProxy proxyStore = ProxyStore.getInstance();
		if (proxyStore != null) {
			proxyStore.processTimeouts();
		}
        if (eventMailBox != null) {
        	eventMailBox.processQueuedEvents();
        }
    }
    
    /**
     * Make the galaxy window configuration the active configuration being displayed.
     * @throws ValidationError 
     */
    public void toGalaxyWindow() throws ApplicationError, ValidationError {
    	boolean showNewGameAnimation = (activeConfiguration instanceof GameEditorConfig) && 
    								   ((GameEditorConfig)activeConfiguration).showNewGameAnimationOnClose();
    	// In case any crafts have changed owner during the battle we must flush image caches
    	flushMooImages();
    	BattleDelegate battle = null;
    	if (activeConfiguration == battleWinConfig) {
    		battle = battleWinConfig.getListener().getBattle();
    	}
    	if (activeConfiguration != galaxyWinConfig) {
    		Logger.logDebug("MainWindow.toGalaxyWindow()");
        	activeConfiguration.deactivate();
        	activeConfiguration = galaxyWinConfig;
        	activeConfiguration.activate();
    	}
    	GalaxyWindowView galaxyView = galaxyWinConfig.getGalaxyView();
		if (galaxyView.isDesktopPaneActive()) {
			galaxyView.deactivateDesktopPane();
    	}
    	getWindow().validate();
    	galaxyView.validate();
    	// Messages may be displayed in the galaxies message list when the 
    	// battle window is open so it may need rebuilding.
    	galaxyView.getMessageList().update(true);
    	galaxyView.getMapPanel().rebuildContent();
		galaxyView.getRadarPanel().repaint();
    	getWindow().repaint();
    	pendingBattleMessages.empty();
    	pendingGalaxyMessages.postAll(galaxyWinConfig.getGalaxyView());
    	try {
			ServerProxy proxyStore = ProxyStore.getInstance();
			proxyStore.clientClosesWindow(battle);
		} catch (ValidationError e) {
		}
    	if (showNewGameAnimation) {
    		NewGameButton.startNewGameAnimation();
    	}
    }
    
    private void flushMooImages() throws ApplicationError {
    	MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos();
    	if (moos == null) return;
    	for (int i = 0; i < moos.length; i++) {
    		// This flushed images
			moos[i].flushImageCache();
		}
    }
    
    public void _setActiveConfiguration(MainWindowConfig testConfig) {
    	activeConfiguration = testConfig;
        if (testConfig instanceof TestBattleWindowConfig) {
	        battleWinConfig = (TestBattleWindowConfig)testConfig;
        } else if (testConfig instanceof TestGalaxyWindowConfig) {
        	galaxyWinConfig = (GalaxyWindowConfig) testConfig;
        }
    }

	/**
     * Make the battle window configuration the active configuration being displayed.
	 * @throws ValidationError 
     */
    public void toBattleWindow(BattleDelegate battle) throws ApplicationError, ValidationError {
		Logger.logDebug("MainWindow.toBattleWindow()");
    	activeConfiguration.deactivate();
    	activeConfiguration = battleWinConfig;
    	battleWinConfig.activate(battle);
    	getWindow().validate();
    	getWindow().repaint();
    	pendingBattleMessages.postAll(battleWinConfig.getBattleView());
    }
    
	/**
     * Make the battle window configuration the active configuration being displayed.
	 * @throws ValidationError 
     */
    public void toEditorWindow(boolean showNewGameAnimationOnClose) throws ApplicationError, ValidationError {
		Logger.logDebug("MainWindow.toEditorWindow()");
		if (GameEditor.hasEditor()) {
			if (ProxyStore.getInstance().supportsLocalServerProxy()) {
				activeConfiguration.deactivate();
				activeConfiguration = GameEditor.getEditorConfig(getOwner());
				activeConfiguration.activate();
				((GameEditorConfig)activeConfiguration).setShowNewGameAnimationOnClose(showNewGameAnimationOnClose);
				getWindow().validate();
				getWindow().repaint();
			} else {
				throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
			}
		} else {
			throw new ValidationError(L.ONLY_PAID_VERSION_HAS_EDITOR);
		}
    }
    
    /**
     * Override hide so we can stop the timer!
     */
    public void hide() {
    	shutdownCleanup();
    	super.hide();
    }
    
    public void shutdownCleanup() {
    	if (Logger.isDebug()) Logger.logDebug("SHUTDOWN");
        timer.stop();
        mailBoxTimer.stop();
        if (Telemetry.ON) Telemetry.save();
        Logger.flushLog();
    }

    public static UserPlayerDelegate getOwner() {
    	if (instance != null) {
    		return instance.getWindowOwner();
    	} else {
    		return null;
    	}
    }
    
    public static String getOwnerName() {
    	if (instance != null) {
    		return instance.getWindowOwnerName();
    	} else {
    		return null;
    	}
    }
    
//    public UserPlayerDelegate _getOwner() {
//    	if (galaxyWinConfig == null) {
//    		return null;
//    	} else {
//    		return galaxyWinConfig.getListener().getOwner();
//    	}
//    }

    public void _setWindowOwner(PlayerDelegate owner) {
    	_setWindowOwner(owner == null ? null : owner.getName());
    }

    public void _setWindowOwner(String owner) {
    	GalaxyWindowConfig gwc = getGalaxyMapConfiguration();
		if (gwc != null) gwc.setOwner(owner);
        BattleWindowConfig bwc = getBattleWinConfig();
		if (bwc != null) bwc.setOwner(owner);
    }

    // For test harness
    public void __setWindowOwner(String owner) throws ApplicationError {
		if (galaxyWinConfig == null) galaxyWinConfig = new TestGalaxyWindowConfig();
		_setWindowOwner(owner);
    }
    
    public void setWindowOwner(String owner) {
        getGalaxyMapConfiguration().setOwner(owner);
        getBattleWinConfig().setOwner(owner);
    }
    
	public UserPlayerDelegate getWindowOwner() {
    	if (galaxyWinConfig == null) {
    		return null;
    	} else {
    		return galaxyWinConfig.getListener().getOwner();
    	}
	}
	public String getWindowOwnerName() {
    	if (galaxyWinConfig == null) {
    		return null;
    	} else {
    		return galaxyWinConfig.getListener().getOwnerName();
    	}
	}


    
    // WINDOW LISTENER STUFF

    public void windowOpened(WindowEvent e) { }
    public void windowClosing(WindowEvent e) { shutdownCleanup(); System.exit(0); }
    public void windowClosed(WindowEvent e) { }
    public void windowIconified(WindowEvent e) { }
    public void windowDeiconified(WindowEvent e) { }
    public void windowActivated(WindowEvent e) { }
    public void windowDeactivated(WindowEvent e) { }

    // COMPONENT LISTENER STUFF
	
    public void componentResized(ComponentEvent e) { 
		try {
			MainWindow.getInstance().invalidate();
	    	if (activeConfiguration != null) {
				try {
					activeConfiguration.getListener().componentResized();
				} catch (ApplicationError e1) {
					Logger.logError("ComponentResize failed", e1);
				}
	    	}
		} catch (ApplicationError e1) {
			Logger.logError("ComponentResize failed", e1);
		}
    }

    // STATE LISTENER STUFF
    
    public void windowStateChanged(WindowEvent e) {
		try {
			MainWindow.getInstance().invalidate();
	    	if (activeConfiguration != null) {
				try {
					activeConfiguration.getListener().componentResized();
				} catch (ApplicationError e1) {
					Logger.logError("ComponentResize failed", e1);
				}
	    	}
		} catch (ApplicationError e1) {
			Logger.logError("ComponentResize failed", e1);
		}
	}
    
    public void componentMoved(ComponentEvent e) {  }
    public void componentShown(ComponentEvent e) {  }
    public void componentHidden(ComponentEvent e) {   }
    
	public boolean isGalaxyMapConfigurationActive() {
		return activeConfiguration == galaxyWinConfig;
	}

	public boolean isBattleMapConfigurationActive() {
		return activeConfiguration == battleWinConfig;
	}

	public MainWindowConfig getActiveConfiguration() {
		return activeConfiguration;
	}

	public GalaxyWindowListener getGalaxyMapListener() {
		return galaxyWinConfig.getListener();
	}

	public GalaxyWindowConfig getGalaxyMapConfiguration() {
		return galaxyWinConfig;
	}

	public BattleWindowConfig getBattleWinConfig() {
		return battleWinConfig;
	}

	public void setBattleWinConfig(BattleWindowConfig battleWinConfig) {
		this.battleWinConfig = battleWinConfig;
	}
	
	// GRAPHICAL CONTAINER STUFF

	public ImageObjectDelegate getImageSelection() {
		return battleWinConfig.getListener().getState().getSelected();
	}

	public boolean isVisible(ImageObjectDelegate imageObject) throws ApplicationError {
		return battleWinConfig.getBattleView().isVisible(imageObject);
	}
	
	public void showMessageActive(String msg, boolean flash) {
		activeConfiguration.getView().showMessage(msg, flash, null, null);
	}

	public void showMessageGalaxy(String msg, boolean flash) {
		if (!isGalaxyMapConfigurationActive()) {
			pendingGalaxyMessages.add(new PendingMessage(msg, flash, null));
		} else {
			galaxyWinConfig.getGalaxyView().showMessage(msg, flash, null, null);
		}
	}

	public void showMessageBattle(String msg, boolean flash) {
		if (isGalaxyMapConfigurationActive()) {
			pendingBattleMessages.add(new PendingMessage(msg, flash, null));
		} else {
			battleWinConfig.getBattleView().showMessage(msg, flash, null, null);
		}
	}

	public void showMessageBothPassive(String msg, boolean flash, String sound) {
		if (isGalaxyMapConfigurationActive()) {
			galaxyWinConfig.getGalaxyView().showMessage(msg, flash, null, sound);
		} else {
			pendingGalaxyMessages.add(new PendingMessage(msg, flash, sound));
			battleWinConfig.getBattleView().showMessage(msg, flash, null, sound);
		}
	}

	public void showMessageBothActive(String msg, boolean flash, String sound) {
		if (isGalaxyMapConfigurationActive()) {
			pendingBattleMessages.add(new PendingMessage(msg, false, null));
			galaxyWinConfig.getGalaxyView().showMessage(msg, flash, null, sound);
		} else {
			pendingGalaxyMessages.add(new PendingMessage(msg, false, null));
			battleWinConfig.getBattleView().showMessage(msg, flash, null, sound);
		}
	}

	public void showTemporaryMessage(String msg, boolean flash, Color bkColor, int additionalSecsToExpire) {
		activeConfiguration.getView().showTemporaryMessage(msg, flash, bkColor, null, additionalSecsToExpire);
	}
	
	public boolean isDisplayingTemporaryMessage() {
		return activeConfiguration.getView().isDisplayingTemporaryMessage();
	}

	// All errors should be reported to the user using this method
	
	public void reportApplicationError(Exception e) {
    	Logger.logError("Action failed", e);
		activeConfiguration.getView().reportApplicationError(e);
	}

	public void reportValidationError(ValidationError e) {
		activeConfiguration.getView().reportValidationError(e);
        try {
			Sounds.getErrorSound().play();
		} catch (ApplicationError ex) {
        	Logger.logError("Sound failed", ex);
		}
	}

    public EventMailBox getEventMailBox() {	
    	return eventMailBox; 
    }
    
	public void setEventMailBox(EventMailBox eventMailBox) { 
		this.eventMailBox = eventMailBox; 
	}

	public AnimationContainer getAnimationContainer() {
		return getBattleWinConfig();
	}
	
	public void reversePause() {
		if (paused) {
			paused = false;
		} else {
			paused = true;
		}
		
	}

	public static int getWidth() {
        if (instance != null) {
        	return (int) instance.getSize().getWidth();
        } else {
        	return 0;
        }
	}
	
	public static boolean isScreen1200Wide() {
        if (instance != null) {
        	return instance.getSize().width >= 1200;
        } else {
        	return false;
        }
	}

	public static boolean isScreen1600Wide() {
        if (instance != null) {
        	return instance.getSize().width >= 1600;
        } else {
        	return false;
        }
	}

	public static boolean isScreen1000High() {
        if (instance != null) {
        	return instance.getSize().height > 1000;
        } else {
        	return false;
        }
	}

	public static boolean isScreen900High() {
        if (instance != null) {
        	return instance.getSize().height > 900;
        } else {
        	return false;
        }
	}


	public PlayerDelegate getOwnerOfActiveAnimation() throws ApplicationError {
		return activeConfiguration.getOwnerOfActiveAnimation();
	}

	public int getQtyAnimations() throws ApplicationError {
		if (activeConfiguration == battleWinConfig) {
			return battleWinConfig.getAnimations().getQuantity();
		} else {
			return 0;
		}
	}

	/**
	 * When a client GUI sends a command to the engine, the request is not complete immediately. However, once complete the GUI usually
	 * needs to update the information being displayed. This method create a callback that will be called once the request is complete. 
	 * @return A callback that will update the GUI display.
	 */
	public CompletedCallback createUpdateCallback() {
		return null;
	}

	
}