/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.main;

import java.awt.Dimension;

import javax.swing.JPanel;

import mot.client.deligates.PlayerDelegate;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

/**
 * Defines a layout of the main window.
 * Because the main window is multi-purpose (used for the battle window and
 * the galaxy window) we need to be able to lay it out in different ways. 
 * Implementations of this class layout the main window for a particular use. 
 * Note that when laying out the main window you cannot use it was empty before 
 * hand - it is likely that it was in use by another window config.
 * 
 * To switch from one configuration to another the old configuration must be decactivated
 * and then the new configuration activated (using the methods defined on the configuration).
 * The deactivation removes the menus, components and key listeners. While the activate adds
 * the menus, components and key listeners.
 * 
 * Dialogs in the game are implemented using internal frames (@see JInternalFrame).
 * When a dialog is open the main content panel of the active configuration must 
 * contains a desktop panel which will contain the dialog stack. When a dialog is
 * open the main content pain will not display ANY other content. The desktop panel
 * can be activated and deactivated using the activateDesktopPane() and deactivateDesktopPane()
 * methods on the view.
 */
public interface MainWindowConfig {

    public void activate() throws ApplicationError;
    public void deactivate() throws ApplicationError;
    
	/**
	 * @return The name of the configuration - MUST BE UNIQUE
	 */
	public String getName();
	
	/**
	 * This should be called on each animation tick on the active configuration
	 * on the main window. This should cause this configuration to repaint its
	 * self and thus performing annotation.
	 * @throws ApplicationError
	 * @throws ValidationError
	 */
    public void handleTick() throws ApplicationError, ValidationError;
    
    /**
     * @return The main content panel for this window config. For a battle config it
     * is the panel containing the battle map and for a galaxy config it is the panel
     * containing the galaxy map.
     */
    public JPanel getMainPanel();
    
    /**
     * @return The size of the main panel of the game (the panel that contains the Galaxy Map or the Battle Field).
     * @throws ApplicationError 
     */
	public Dimension getMainPanelSize() throws ApplicationError;
	
	/**
     * @return The view component of this configuration
     */
    public MainWindowView getView();
    
	/**
     * @return The listener of this configuration
     */
    public MainWindowListener getListener();
    
	/**
	 * @return The owner (player who owned the object being animated) of the active animation in the window.
	 * Returns null if there are is nothing being animated;
	 * @throws ApplicationError 
	 */
	public abstract PlayerDelegate getOwnerOfActiveAnimation() throws ApplicationError;

}
