/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.main;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.ToolTipManager;

import mot.client.actions.ActiveRectangle;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.Sound;
import mot.client.sounds.Sounds;
import mot.client.swing.common.MotToolTip;
import mot.client.swing.window.galaxy.GalaxyWindowButtonMenu;
import mot.client.swing.window.galaxy.GalaxyWindowListener;
import mot.client.swing.window.galaxy.actions.GalaxyMapButtonCraftSleep;
import mot.client.swing.window.galaxy.actions.ShortcutListener;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

/**
 * Basic implementation of a window listener.
 * If listens to events of an MapWindowView and implements scrolling of the
 * view's panel by hovering over the edges of the map as well as dragging the
 * map using the mouse.
 */
public abstract class MainWindowListener implements MotListener {
	
	private MainWindowView view;
	private String owner;
	public boolean mouseButton1Down = false;
	public boolean mouseButton2Down = false;
	public ActiveRectangle currentDraggable = null;
	
	protected MainWindowListener() throws ApplicationError {
		setup();
	}
	
	protected MainWindowListener(String owner, MainWindowView view) throws ApplicationError {
		this.owner = owner;
		this.view = view;
		setup();
	}
	
	private void setup() throws ApplicationError {
		if (this instanceof GalaxyWindowListener) {
			shortcutListeners.add(GalaxyMapButtonCraftSleep.getShortcutListener((GalaxyWindowListener)this));
		}
	}
	
	public UserPlayerDelegate getOwner() {
		PlayerDelegate player = DeligateCacheManager.getPlayer(owner);
		if (player instanceof UserPlayerDelegate) {
			return (UserPlayerDelegate) player;
		} else {
			// Probably loading a game that has different player of same name
			return null;
		}
	}

	public String getOwnerName() {
		return owner;
	}

	public void setOwner(String owner) {
		this.owner = owner;
	} 

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {
		if (movedTimer != null) movedTimer.stop();
		mouseButton1Down = false;
		mouseButton2Down = false;
		currentDraggable = null;
	}

	private int lastDragX, lastDragY;
	private long lastTimestamp;
	public void mousePressed(MouseEvent e) {
		lastDragX = e.getXOnScreen();
		lastDragY = e.getYOnScreen();
		lastTimestamp = e.getWhen();
		int button = e.getButton();
		if (button == MouseEvent.BUTTON1) {
			mouseButton1Down = true;
			currentDraggable = null;
		} else if (button == MouseEvent.BUTTON3) {
			mouseButton2Down = true;
		}
		if (e.getClickCount() == 1) {
			motMouseClicked(e);
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		mouseButton1Down = false;
		mouseButton2Down = false;
		currentDraggable = null;
		if (e.getClickCount() == 2) {
			motMouseClicked(e);
		}
	}
	
    public void mouseClicked(MouseEvent e) {
    	// Not used for selecting as clicks get lost (smallest drag does not get converted to a click)
    }

    public abstract void motMouseClicked(MouseEvent e);
    
	//
    // Global Key Listener : KeyEventDispatcher interface!
	// This callback is registered on view.activate() and unregistered on view.deactivate(). 
	//
    public boolean dispatchKeyEvent(KeyEvent e) {
    	boolean discardEvent = false;
    	if (view.processKeyEventsGlobally()) {
    		try {
    			if (e.getID() == KeyEvent.KEY_TYPED) {
    				if (Character.toLowerCase(e.getKeyChar()) == 'p') {
    					MainWindow.getExestingInstance().reversePause();
    				} else if (e.getKeyChar() == ':') {
    					Logger.TRACE_ON = !Logger.TRACE_ON;
    				} else if (e.getKeyChar() == '?') {
    					String message = getKeyBindingsMessage();
    					if (message != null) {
    						Color c = GalaxyWindowButtonMenu.getTipBackground();
    						view.showTemporaryMessage(message, false, c, null, 4);
    					}
    				} else if (Character.toLowerCase(e.getKeyChar()) == 'x' && MainWindow.getExestingInstance().isBattleMapConfigurationActive()) {
    					MainWindow.getExestingInstance().getBattleWinConfig().getListener().stopAllAnimations();
    				} else {        		
    					boolean processed = false;
    					Iterator i = shortcutListeners.iterator();
    					while (i.hasNext() && !processed) {
    						ShortcutListener l = (ShortcutListener) i.next();
    						processed = l.keyPressed(Character.toLowerCase(e.getKeyChar()), e.getModifiers());
    					}
    					if (!processed) {
    						motKeyTyped(e);
    					}
    				}
    			} else if (e.getID() == KeyEvent.KEY_PRESSED) {
    				// Avoid duplicate key handling - but ESCAPE is never typed only pressed!
    				if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
    					motKeyTyped(e);
    				}
    			}
    		} catch (ValidationError ex) {
    			view.reportValidationError(ex);
    			try {
    				Sounds.getErrorSound().play();
    			} catch (ApplicationError exc) {
    				Logger.logError("Sound failed", exc);
    			}
    		} catch (Exception ex) {
    			view.reportApplicationError(ex);
    		}
    	}
        return discardEvent;
    }

	protected abstract void motKeyTyped(KeyEvent e) throws ApplicationError, ValidationError;

    /**
     * @return The text of a message containing the key bindings for commands currently available.
     */
	protected abstract String getKeyBindingsMessage();

	protected ArrayList<ShortcutListener> shortcutListeners = new ArrayList();

    /**
     * Buttons in the game with shortcuts call this to register them selves
     * to receive key events.
     */
    public void addShortcutLisneter(ShortcutListener shortcutListener) {
    	Character key = shortcutListener.getShortcutKey();
    	if (key != null) {    		
    		for (ShortcutListener i : shortcutListeners) {
    			if (i.getShortcutKey() == key) {
    				shortcutListeners.remove(i);
    				break;
    			}
    		}
    	}
    	shortcutListeners.add(shortcutListener);
    }
    
	//
	// Mouse Motion Listener
	//
	
	public synchronized void mouseDragged(MouseEvent e) {
		try {
			//Logger.logDebug("mx="+e.getX()+" my="+e.getY()+" xOS="+e.getXOnScreen()+" cc="+e.getClickCount()+" button="+e.getButton()+" ts="+e.getWhen()+" point="+e.getPoint()+" "+e.getModifiers()+" id="+e.getID()+" "+e.isConsumed()+" src="+e.getSource());
			int dx = lastDragX - e.getXOnScreen();
			int dy = lastDragY - e.getYOnScreen();
			if (dx != 0 || dy != 0) {
				lastDragX = e.getXOnScreen();
				lastDragY = e.getYOnScreen();
				lastTimestamp = e.getWhen();
				int x = e.getX();
				int y = e.getY();
				if (mouseButton1Down) {
					if (currentDraggable != null) {
						currentDraggable.mouseDragged(dx, dy);
					} else {
						currentDraggable = getDraggableObjectAt(x,y);
						if (currentDraggable != null) {
							currentDraggable.mouseDragged(dx, dy);
						} else {
							view.moveFocus(dx, dy);			
						}
					}
				} else if (mouseButton2Down) {
					mouseButton2Dragged(x, y, -dx, -dy);
				}
			}
		} catch (Exception ex) {
			Logger.logError("MouseDragged Fail", ex);
		}
	}

	protected ActiveRectangle getDraggableObjectAt(int x, int y) { return null; }
	protected void mouseButton2Dragged(int x, int y, int dx, int dy) { }
	
	private static final int SCROLL_AMOUNT = 5;
	private static final int TIME_TO_START_SCROLLING = 500;
	private static final int TIME_TO_SCROLLING = 5;
	private Timer movedTimer;
	private int mousePosX, mousePosY;
	
	public void mouseMoved(MouseEvent e) {
		mousePosX = e.getX();
		mousePosY = e.getY();
		try {
			_mouseMoved(mousePosX, mousePosY);
		} catch (Exception ex) {
			Logger.logError("MouseMoved Fail", ex);
		}
			if (movedTimer != null) { 
				movedTimer.restart();
			} else {
				movedTimer = new Timer(TIME_TO_SCROLLING, new MousePausedAction());
				movedTimer.setInitialDelay(TIME_TO_START_SCROLLING);
				movedTimer.start();
			}
		// Java Tool tips are not closing like they should - I will force them to close
		closeJavaToolTip();
		// Close my own tool tips
		MotToolTip.hideTipWindow();
	}
	
	private long lastTimeMillis = 0;
	private void closeJavaToolTip() {
        long currentTimeMillis = System.currentTimeMillis();
        if (currentTimeMillis - lastTimeMillis > 1000) {
	        JPanel component = view.getMapPanel();
			MouseEvent phantom = new MouseEvent( component, MouseEvent.MOUSE_MOVED, currentTimeMillis, 0, 10, 10, 0, false);
	        ToolTipManager.sharedInstance().mouseMoved(phantom);
        }
    }
		
	protected void _mouseMoved(int x, int y) throws ApplicationError, ValidationError {
	}

	private static final int SCROLL_WIDTH = 20;
	
//	public boolean isInTopScrollRegion(int x, int y) {
//		java.awt.Rectangle rect = view.getMapPanelScrollPane().getViewport().getViewRect(); 
//		return rect.contains(x,y) && (y < rect.y + SCROLL_WIDTH);
//	}
//
//	public boolean isInBottomScrollRegion(int x, int y) {
//		java.awt.Rectangle rect = view.getMapPanelScrollPane().getViewport().getViewRect();
//		Point p = view.getMapPanelScrollPane().getViewport().getMousePosition();
//		return rect.contains(x,y) && (y > rect.y + rect.height - SCROLL_WIDTH);
//	}
//	
//	public boolean isInLeftScrollRegion(int x, int y) {
//		java.awt.Rectangle rect = view.getMapPanelScrollPane().getViewport().getViewRect(); 
//		return rect.contains(x,y) && (x < rect.x + SCROLL_WIDTH);
//	}
//
//	public boolean isInRightScrollRegion(int x, int y) {
//		java.awt.Rectangle rect = view.getMapPanelScrollPane().getViewport().getViewRect(); 
//		return rect.contains(x,y) && (x > rect.x + rect.width - SCROLL_WIDTH);
//	}

	public void setView(MainWindowView view) {
		this.view = view;
	}

	private class MousePausedAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
	    	if (Logger.isDebug()) {
	    		if (!SwingUtilities.isEventDispatchThread()) {
	    			Logger.logError("", new Exception("NOT IN EVENT THREAD"));
	    		}
	    	}
//	    	boolean r = autoScroll();
//	    	if (!r) {
	    		try {
					mousePausedAt(mousePosX, mousePosY);
				} catch (ApplicationError ex) {
		            MainWindow.getExestingInstance().reportApplicationError(ex);
				}
	    	}
//		}
	}

	protected void mousePausedAt(int mousePosX, int mousePosY) throws ApplicationError {
	}
	
//	protected boolean autoScroll() {
//		int dx = 0;
//		int dy = 0;
//		if (isInTopScrollRegion(mousePosX, mousePosY)) {
//			dy = -SCROLL_AMOUNT;
//		}
//		if (isInBottomScrollRegion(mousePosX, mousePosY)) {
//			dy = SCROLL_AMOUNT;
//		}
//		if (isInLeftScrollRegion(mousePosX, mousePosY)) {
//			dx = -SCROLL_AMOUNT;
//		}
//		if (isInRightScrollRegion(mousePosX, mousePosY)) {
//			dx = SCROLL_AMOUNT;
//		}
//		if (dx == 0 && dy == 0) {
//			return false;
//		} else {
//			mousePosX += dx;
//			mousePosY += dy;
//			view.moveFocus(dx, dy);
//			return true;
//		}
//	}
	
	public void playSound(String name) {
		try {
			Sound sound = Sounds.getSound(name);
			sound.play();
		} catch (ApplicationError e) {
	    	Logger.logError("Sound "+name+" failed", e);
		}
	}
	
	public void playActionSound() {
		try {
	    	Sounds.getActionSound().play();
		} catch (ApplicationError e) {
	    	Logger.logError("Sound action failed", e);
		}
	}
	
	public abstract void componentResized() throws ApplicationError;
	
	/**
	 * User has single clicked on a single MOO in the main panel.  
	 */
	public void click(MoveableOwnedObjectDelegate moo) throws ApplicationError, ValidationError { }
	
	/**
	 * User has double clicked on the MOO in the main panel.  
	 */
    public void doubleClick(MoveableOwnedObjectDelegate moo) throws ApplicationError, ValidationError { }
	
	/**
	 * User has single clicked on the SO in the main panel.  
	 */
	public void click(StellarObjectDelegate so, boolean autoselectPlanetOrbit) throws ApplicationError, ValidationError {  }
	
	/**
	 * User has double clicked on the SO in the main panel.  
	 */
    public void doubleClick(StellarObjectDelegate so) throws ApplicationError, ValidationError {  }
	
	/**
	 * User has single clicked on the SO in the main panel.  
	 */
	public void click(PlanetDelegate planet) throws ApplicationError, ValidationError {  }
	
	/**
	 * User has double clicked on the Planet in the main panel.  
	 */
    public void doubleClick(PlanetDelegate planet, boolean visible) throws ApplicationError, ValidationError {  }

}
