/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.main;

import java.awt.Color;
import java.awt.Container;

import javax.swing.JComponent;

public class MainWindowMessageCustom implements MainWindowMessage {

	/**
	 * This is the text of the message to be displayed
	 */
	private JComponent message;
	
	/**
	 * This is the time that this message should be removed from the list.
	 */
	private long createdTime;
	
	/**
	 * This is the time that this message should be removed from the list.
	 */
	private long expireTime;
	
	/**
	 * This is only used if the message is flashing (has more than one background color
	 * It is the quantity of times this message should flash. Null implies that the message
	 * flashes until it expires while a value represents the quantity of times the message
	 * should flash until it stops.
	 */
	private Integer qtyFlashes;
	
	/**
	 * Message priority - Higher priority messages will not be removed to make way for lower 
	 * priority messages.
	 */
	private int priority;

	/**
	 * Sound to play during this message
	 */
	private String sound;
	
	/**
	 * This is the background colour.
	 * If this message should flash then there should be more than one background color.
	 */
	private static Color [] background = {
		new Color(50,0,0),
		new Color(0,0,0)
	};

	/**
	 * This is the foreground colour.
	 */
	private Color foreground =  Color.WHITE;
	
	/**
	 * The current background colour being used. This should be one colour in the
	 * array background.
	 */
	private Color currentBackgroundColor = background[0];
	
	private static final int timeToChangeColor = 400;

	public MainWindowMessageCustom(JComponent message, long milliSecondsToExpire, int qtyFlashes, int priority, String sound) {
		this.message = message;
		this.priority = priority;
		this.sound = sound;
		
		this.createdTime = System.currentTimeMillis();
		this.expireTime = createdTime + milliSecondsToExpire;
		this.qtyFlashes = qtyFlashes;
		
        //setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		message.setBackground(currentBackgroundColor);
		message.setForeground(foreground);
	}
	
	public void resetExpireTimes() {
		long duration = expireTime - createdTime;
		createdTime = System.currentTimeMillis();
		expireTime = createdTime + duration;
	}


	public void setColours(Color [] backgrounds) {
		background = backgrounds;
		currentBackgroundColor = background[0];
	}
	
	/**
	 * This should be called frequently for any message that requires animation (flashing).
	 * @return True if and only if the message has expired.
	 */
	public boolean tick(Container container) {
		long time = System.currentTimeMillis();
		if (time > expireTime) {
			return true;
		}
		int age = (int)(time - createdTime);
		if (background.length <= 1) {
			return false;
		}
		if (qtyFlashes == null || qtyFlashes.intValue() > 0) {
			int index = (age / timeToChangeColor)%background.length;
			Color newBackground =  background[index];
			if (newBackground != currentBackgroundColor) {
				currentBackgroundColor = newBackground;
				message.setBackground(currentBackgroundColor);
				container.repaint();
				if (qtyFlashes != null) {
					qtyFlashes = new Integer(qtyFlashes.intValue()-1);
				}
			}
		}
		return false;
	}

	public Color getCurrentBackgroundColor() {
		return currentBackgroundColor;
	}

	public void setCurrentBackgroundColor(Color currentBackgroundColor) {
		this.currentBackgroundColor = currentBackgroundColor;
	}

	public JComponent getComponent() {
		return message;
	}

	public int getPriority() {
		return priority;
	}

	public String getSound() {
		return sound;
	}

	
}
