/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.main;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JScrollPane;
import javax.swing.JViewport;

import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.sounds.SoundEffect;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ComponentTable;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;

/**
 * This displays a list of messages at the bottom of the galaxy window. These messages 
 * range from errors to notification of events. This displays two types of information
 *  1 - Temporary Message
 *  		Examples of temporary messages are error messages. These have special 
 *  		highlighting and usually flash to get the user's attention. They are only
 *  		displayed in the list for a short period of time and ONLY one temporary 
 *  		message can be displayed at a time.
 *	2 - Plain Messages
 *			Examples are notification of events such as declarations of war etc.
 *			These message remain displayed in the list (chronologically ordered) for a
 *			Period of time.
 */
public class MainWindowMessageList extends ComponentTable {
	
	private static final int SECOND = 1000;
	public static final int TIME_FOR_QUESTIONS_TO_EXPIRE = 30*SECOND;
	public static final int TIME_FOR_TEMP_MSGS_TO_EXPIRE = 4*SECOND;
	public static final int TIME_FOR_NORMAL_MSGS_TO_EXPIRE = 10*60*SECOND;
	
	private static final Color [] ERROR_MSG_COLOURS_DEFAULT = {
		new Color(0,0,0),
		new Color(100,0,0)
	};
	private static final Color [] NORMAL_MSG_COLOURS_DEFAULT = {
		new Color(0,0,0),
		new Color(29,34,52)
	};
	private static Color [] ERROR_MSG_COLOURS;
	private static Color [] NORMAL_MSG_COLOURS;
	
	private static Color [] getErrorMessageColours() {
		if (ERROR_MSG_COLOURS == null) {
			ERROR_MSG_COLOURS = new Color[2];
			try {
				ERROR_MSG_COLOURS[0] = PropertyManager.getMandatoryAsColor("window.main.background");
				ERROR_MSG_COLOURS[1] = PropertyManager.getMandatoryAsColor("window.main.message.error.background");
			} catch (ApplicationError e) {
				Logger.logError("Unable to get Color", e);
				ERROR_MSG_COLOURS = ERROR_MSG_COLOURS_DEFAULT;
			}
		}
		return ERROR_MSG_COLOURS;
	}
	
	private static Color [] getNormalMessageColours() {
		if (NORMAL_MSG_COLOURS == null) {
			NORMAL_MSG_COLOURS = new Color[2];
			try {
				NORMAL_MSG_COLOURS[0] = PropertyManager.getMandatoryAsColor("window.main.background");
				NORMAL_MSG_COLOURS[1] = PropertyManager.getMandatoryAsColor("window.main.message.flash.background");
			} catch (ApplicationError e) {
				Logger.logError("Unable to get Color", e);
				NORMAL_MSG_COLOURS = NORMAL_MSG_COLOURS_DEFAULT;
			}
		}
		return NORMAL_MSG_COLOURS;
	}
	
	/**
	 * This is the list of messages being displayed to the user at the
	 * bottom of the galaxy window. Each element in this list of of type
	 * GalaxyWindowMessage.
	 */
	private ArrayList messages = new ArrayList();
	
	/**
	 * This is the current temporary message being displayed (or null) if
	 * no temporary message is being displayed.
	 */
	private MainWindowMessage temporaryMessage;
	private ArrayList<MainWindowMessage> temporaryMessageQueue = new ArrayList<MainWindowMessage>();
	
	/**
	 * This is a tutorial tip message being displayed (or null) if
	 * no temporary message is being displayed.
	 */
	private MainWindowMessage tipMessage;
	
	public MainWindowMessageList() throws ApplicationError {
		super("HVS", 1, 1, 1, 1, 1);
		//PanelUtils.setTheBloodySize(this, 1000,40);
		setMinimumSize(new Dimension(100,40));
        setPreferredSize(new Dimension(1000,40));
        
        setOpaque(true);
        setBackground(Color.BLACK);
	}
	
	public void removeAllMessages() {
		temporaryMessageQueue.clear();
		temporaryMessage = null;
		tipMessage = null;
		messages.clear();
		invalidate();
		repaint();
	}

    public void update(boolean scrollToBottom) {
        removeAll();
        if (temporaryMessage != null) {
            add(new JScrollPane(temporaryMessage.getComponent(), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
        } else if (tipMessage != null) {
            add(new JScrollPane(tipMessage.getComponent(), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));
        } else {
        	ComponentTable contents = new ComponentTable("HT", messages.size(),1, 0,1,1);
        	contents.setOpaque(true);
        	contents.setBackground(Color.BLACK);
        	JScrollPane scrollPane;
            if (scrollToBottom) {
            	scrollPane = new JScrollToBottomPane(contents, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            } else {
            	scrollPane = new JScrollPane(contents, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            }
        	scrollPane.getVerticalScrollBar().setUnitIncrement(20);
        	scrollPane.getVerticalScrollBar().setBlockIncrement(40);
        	Iterator i = messages.iterator();
            while (i.hasNext()) {
            	MainWindowMessage msg = (MainWindowMessage) i.next();
                if (i.hasNext()) {
                	contents.add(msg.getComponent(), "HT");
                } else {
                	contents.add(msg.getComponent(), "HST");
                }
            }
            add(scrollPane);
        }
        validate();
        repaint();
    }
	
	/**
	 * This should be called every time the window is repainted or very frequently.
	 * This method animates the messages and expires messages that have been around
	 * too long.
	 */
	public void tick() throws ApplicationError {
        if (temporaryMessage != null) {
            if (temporaryMessage.tick(this)) {
            	temporaryMessage = null;
            	if (tipMessage == null) {
            		temporaryMessage = nextTemporaryMessageInQueue();
            		if (temporaryMessage == null) {
            			tickMessages();
            		}
            	}
            	update(tipMessage == null && temporaryMessage == null);
            }
        } else if (tipMessage != null) {
            if (tipMessage.tick(this)) {
            	tipMessage = null;
            	if (temporaryMessage == null) {
            		temporaryMessage = nextTemporaryMessageInQueue();
            	}
            	if (temporaryMessage == null) {
            		tickMessages();
            		update(true);
            	} else {
            		update(false);
            	}
            }
        } else {
        	tickMessages();
        }
	}
	
	private MainWindowMessage nextTemporaryMessageInQueue() {
		if (temporaryMessageQueue.size() > 0) {
			MainWindowMessage r = temporaryMessageQueue.remove(0);
			r.resetExpireTimes();
			playSound(r);
			return r;
		} else {            		
			return null;
		}
	}

	private void tickMessages() {
		Iterator i = messages.iterator();
		while (i.hasNext()) {
			MainWindowMessage msg = (MainWindowMessage) i.next();
			if (msg.tick(this)) {
				i.remove();
			}
		}
	}
	
	public void addErrorMessage(String msg, boolean flash) {
		if (flash) {
			temporaryMessage = new MainWindowMessageText(msg, TIME_FOR_TEMP_MSGS_TO_EXPIRE, getErrorMessageColours(), null, null);
		} else {
			temporaryMessage = new MainWindowMessageText(msg, TIME_FOR_TEMP_MSGS_TO_EXPIRE);
		}
    	update(false);
	}
	
	public void addErrorMessage(String msg, boolean flash, int duration) {
		if (flash) {
			temporaryMessage = new MainWindowMessageText(msg, duration, getErrorMessageColours(), null, null);
		} else {
			temporaryMessage = new MainWindowMessageText(msg, duration);
		}
    	update(false);
	}
	
	public void addTemporaryMessage(String msg, boolean flash, Color bkColor, String sound, int additionalSecsToExpire) {
		MainWindowMessageText newMsg;
		if (flash) {
			newMsg = new MainWindowMessageText(msg, TIME_FOR_TEMP_MSGS_TO_EXPIRE + additionalSecsToExpire*1000, getNormalMessageColours(), null, sound);
		} else {
			newMsg = new MainWindowMessageText(msg, TIME_FOR_TEMP_MSGS_TO_EXPIRE + additionalSecsToExpire*1000, bkColor, sound);
		}
		addTemporaryMessage(newMsg);
	}

	public void addTemporaryMessage(MainWindowMessage message) {	
		if (tipMessage != null) {
			return;
		}
		if (temporaryMessage != null) {
			temporaryMessageQueue.add(message);
		} else {
			temporaryMessage = message;
			playSound(temporaryMessage);
	    	update(false);
		}
	}
	
	public void showTipMessage(MainWindowMessage message) {	
		tipMessage = message;
		temporaryMessage = null;
    	update(false);
	}
	
	public void addMessageNoUpdate(String msg) {
		if (msg == null) {
			return;
		}
		String dateStr = "";
		try {
			dateStr = L.DATE_+DeligateCacheManager.getDate()+" : ";
		} catch (Exception ex) { }
		messages.add( new MainWindowMessageText(dateStr+msg, TIME_FOR_NORMAL_MSGS_TO_EXPIRE) );
	}

	public void addMessage(String msg) {
		if (msg == null) {
			return;
		}
		addMessageNoUpdate(msg);
    	update(true);
	}

	private void playSound(MainWindowMessage msg) {
		String sound = msg.getSound();
		if (sound != null) {
			try {
				SoundEffect s = Sounds.getSound(sound);
				if (s != null) {
					s.play();
				} else {
					Logger.logWarning("Sound missing "+sound);
				}
			} catch (Exception exc) {
				Logger.logError("Sound failed", exc);
			}
		}
	}


	public boolean isDisplayingTemporaryMessage() {
		return temporaryMessage != null || tipMessage != null;
	}
	
	public boolean isDisplayingTipMessage() {
		return tipMessage != null;
	}
	
	public void removeTemporaryMessageWithText(String text) {
		if (temporaryMessage != null && temporaryMessage instanceof MainWindowMessageText) {
			MainWindowMessageText currentTextMessage = (MainWindowMessageText) temporaryMessage;
			if (currentTextMessage.getMessage() == text) {				
				temporaryMessage = nextTemporaryMessageInQueue();
				update(true);
				return;
			}
		}
		if (temporaryMessageQueue != null) {
			for (Iterator<MainWindowMessage> i = temporaryMessageQueue.iterator(); i.hasNext(); ) {
				MainWindowMessage currentMessage = i.next();
				if (currentMessage instanceof MainWindowMessageText) {
					MainWindowMessageText currentTextMessage = (MainWindowMessageText) currentMessage;
					if (currentTextMessage.getMessage() == text) {				
						i.remove();
						return;
					}
				}
			}
		}
	}
	
	public void removeTemporaryMessage() {
		temporaryMessage = nextTemporaryMessageInQueue();
    	update(true);
	}
	
	public void removeTipMessage() {
		tipMessage = null;
		temporaryMessage = nextTemporaryMessageInQueue();
    	update(true);
	}

	public MainWindowMessage getTipMessage() {
		return tipMessage;
	}

	public void notifyEndTurn() {
		temporaryMessageQueue.clear();
	}

}

/**
 * A scroll pane that always scrolls to the bottom when laid out.
 */
class JScrollToBottomPane extends JScrollPane {

	public JScrollToBottomPane(Component view, int vsbPolicy, int hsbPolicy) {
		super(view, vsbPolicy, hsbPolicy);
	}

	@SuppressWarnings("deprecation")
	public void layout() {
		super.layout();
		JViewport viewport = getViewport();
    	int viewHeight = viewport.getHeight();
    	int height = viewport.getView().getHeight();
		viewport.setViewPosition(new Point(0, height-viewHeight));
	}
	
	
	
}
