/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.main;

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;

import javax.swing.JComponent;
import javax.swing.JLabel;

public class MainWindowMessageText extends JLabel implements MainWindowMessage {

	/**
	 * This is the text of the message to be displayed
	 */
	private String message;
	
	/**
	 * This is the time that this message should be removed from the list.
	 */
	private long createdTime;
	
	/**
	 * This is the time that this message should be removed from the list.
	 */
	private long expireTime;
	
	/**
	 * This is only used if the message is flashing (has more than one background color
	 * It is the quantity of times this message should flash. Null implies that the message
	 * flashes until it expires while a value represents the quantity of times the message
	 * should flash until it stops.
	 */
	private Integer qtyFlashes;
	
	/**
	 * This is the background colour.
	 * If this message should flash then there should be more than one background color.
	 */
	private Color [] background =  { Color.BLACK };

	/**
	 * This is the foreground colour.
	 */
	private Color foreground =  Color.WHITE;
	
	/**
	 * The current background colour being used. This should be one colour in the
	 * array background.
	 */
	private Color currentBackgroundColor = background[0];
	
	private static final int timeToChangeColor = 400;

	private String sound;

	public MainWindowMessageText(String message, long milliSecondsToExpire) {
		this.message = message;
		this.createdTime = System.currentTimeMillis();
		this.expireTime = createdTime + milliSecondsToExpire;
		
        //setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		setBackground(currentBackgroundColor);
		setForeground(foreground);
		setHorizontalAlignment(JLabel.LEFT);
		
		setText(message);
		setOpaque(true);
	}
	
	public MainWindowMessageText(String message, long milliSecondsToExpire, Color [] background, Integer qtyFlashes, String sound){
		this(message, milliSecondsToExpire);
		this.background = background;
		this.currentBackgroundColor = background[background.length-1];
		this.qtyFlashes = qtyFlashes;
		this.sound = sound;

		setBackground(currentBackgroundColor);
		
		int fontSize = 0;
		if (MainWindow.getWidth() > 1500) fontSize = 4;
		else if (MainWindow.getWidth() > 1200) fontSize = 2;
		
        Font font = getFont();
        setFont(new Font(font.getName(), font.getStyle(), font.getSize()+fontSize));
	}
	
	public MainWindowMessageText(String message, long milliSecondsToExpire, Color newBackground, String sound) {
		this(message, milliSecondsToExpire);
		this.background = new Color[1];
		this.background[0] = newBackground;
		this.currentBackgroundColor = background[background.length-1];
		this.sound = sound;

		setBackground(currentBackgroundColor);
		
		int fontSize = 0;
		if (MainWindow.getWidth() > 1500) fontSize = 4;
		else if (MainWindow.getWidth() > 1200) fontSize = 2;
		
        Font font = getFont();
        setFont(new Font(font.getName(), font.getStyle(), font.getSize()+fontSize));
	}
	
	public void resetExpireTimes() {
		long duration = expireTime - createdTime;
		createdTime = System.currentTimeMillis();
		expireTime = createdTime + duration;
	}


	/**
	 * This should be called frequently for any message that requires animation (flashing).
	 * @return True if and only if the message has expired.
	 */
	public boolean tick(Container container) {
		long time = System.currentTimeMillis();
		if (time > expireTime) {
			return true;
		}
		int age = (int)(time - createdTime);
		if (background.length <= 1) {
			return false;
		}
		if (qtyFlashes == null || qtyFlashes.intValue() > 0) {
			int index = (age / timeToChangeColor)%background.length;
			Color newBackground =  background[index];
			if (newBackground != currentBackgroundColor) {
				currentBackgroundColor = newBackground;
				setBackground(currentBackgroundColor);
				container.repaint();
				if (qtyFlashes != null) {
					qtyFlashes = new Integer(qtyFlashes.intValue()-1);
				}
			}
		}
		return false;
	}

	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}

	public Color getCurrentBackgroundColor() {
		return currentBackgroundColor;
	}

	public void setCurrentBackgroundColor(Color currentBackgroundColor) {
		this.currentBackgroundColor = currentBackgroundColor;
	}

	public JComponent getComponent() {
		return this;
	}

	public int getPriority() {
		return 0;
	}

	public String getSound() {
		return sound;
	}

	public void setSound(String sound) {
		this.sound = sound;
	}
	
}
