/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.main;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;

import javax.swing.JDesktopPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

/**
 * This is a base class for all views that contain a map in a scrollable viewport.
 * It provides support for scrolling, centring on a location etc.
 */
public abstract class MainWindowView {
	
	/**
	 * When the main window wishes to change the active configuration this must be
	 * called on the old configuration before activate on the new configuration. 
	 * @throws ApplicationError
	 */
    public abstract void deactivate() throws ApplicationError;
	
	/**
	 * When the main window wishes to change the active configuration this must be
	 * called to activate the new configuration. 
	 * @throws ApplicationError
	 */
    public abstract void activate() throws ApplicationError;
    
    /**
     * Called by any code to report an error
     * @throws ApplicationError 
     */
	public abstract void showMessage(String msg, boolean flash, Color flashBkColor, String sound);

    /**
     * Called by any code to display a message on the screen for a short period of time
     * If background color is null then the default background colour is used
     * @throws ApplicationError 
     */
	public abstract void showTemporaryMessage(String msg, boolean flash, Color flashBkColor, String sound, int additionalSecsToExpire);

	/**
	 * This is used to unimportant messages like tool tips do not get displayed over
	 * important messages like error messages! Remember - that only one temporary message
	 * can be displayed at a time!
	 * @return True if the message box being displayed on the window is showing an temporary
	 * message.
	 */
	public abstract boolean isDisplayingTemporaryMessage();
	
    /**
     * Called by any code to report an error
     */
	public abstract void reportApplicationError(Exception e);

    /**
     * Called by any code to report an error
     * @throws ApplicationError 
     */
	public abstract void reportValidationError(ValidationError e);
	
	/**
	 * All maps are contained in a scroll pane
	 * @return The scroll pane this map is contained in.
	 */
	public abstract JScrollPane getMapPanelScrollPane();
	public abstract JPanel getMapPanel();

    /**
     * Centre the view port of the scroll pane on location x,y.
     * @param scrollPane The scroll pane to display the centre x,y
     * @param boundary The boundary of the scroll pane that this method must not display outside
     * @param x the centre to scroll to
     * @param y the centre to scroll to
     */
    public void focus(Area boundary, int x, int y) {
    	focus(boundary, new Location(x, y));
    }
    
    
    public void focus(Area boundary, Location loc) {
        // Stupid swing has a bug that scrolling only works forwards!
    	JScrollPane scrollPane = getMapPanelScrollPane();
        if (boundary == null) {
        	// This is the size of the panel inside the view port of which only a part is visible.
        	Dimension viewSize = scrollPane.getViewport().getViewSize();
        	boundary = new Area(0,0, viewSize.width, viewSize.height);
        }
        // This is the size of the view port
        Dimension size = scrollPane.getViewport().getSize();
        Rectangle r = new Rectangle(loc, size.width, size.height);
        if (r.getLeft() < boundary.getLeft()) {
            r.setLeft(boundary.getLeft());
        }
        if (r.getRight() > boundary.getRight()) {
            r.setRight(boundary.getRight());
        }
        if (r.getTop() < boundary.getTop()) {
            r.setTop(boundary.getTop());
        }
        if (r.getBottom() > boundary.getBottom()) {
            r.setBottom(boundary.getBottom());
        }
        loc = r.getTopLeftLocation();
        if (loc.getX() < 0) loc.setX(0);
        if (loc.getY() < 0) loc.setY(0);
        //Logger.logDebug("FOCUS "+loc+" in "+r);
        scrollPane.getViewport().setViewPosition(new Point(loc.getX(), loc.getY()));
    }

    /**
     * Set the centre the view port of the scroll pane on location x,y.
     * @param x the centre to scroll to
     * @param y the centre to scroll to
     */
    public void focus(int x, int y) {
    	JScrollPane scrollPane = getMapPanelScrollPane();
        Dimension viewSize = scrollPane.getViewport().getViewSize();
        Dimension portSize = scrollPane.getViewport().getSize();
        x -= portSize.width/2;
        y -= portSize.height/2;
        if (x < 0) {
        	x = 0;
        }
        if (y < 0) {
        	y = 0;
        }
        if (x+portSize.width > viewSize.width) {
        	x = viewSize.width-portSize.width;
        }
        if (y+portSize.height > viewSize.height) {
        	y = viewSize.height-portSize.height;
        }
        scrollPane.getViewport().setViewPosition(new Point(x, y));
    }

    /**
     * Shift the current part of the map being displayed in the viewport by dx,dy.
     * @param dx The amount to move the screen focus on the x axis (x is in screen coordinates)
     * @param dy The amount to move the screen focus on the y axis (y is in screen coordinates)
     */
    public void moveFocus(int dx, int dy) {
    	java.awt.Rectangle r = getMapPanelScrollPane().getViewport().getViewRect();
    	int centerX = (int)r.getCenterX();
    	int centerY = (int)r.getCenterY();
    	focus(centerX+dx, centerY+dy);
    }

    /**
     * The main content panel is consumed by a desktop panel that can be used to
     * display dialog windows. The normal content (such as the battle or galaxy map)
     * is no longer displayed.
     * @return A desktop pane that is currently active in the main panel.
     * @throws ApplicationError 
     */
    public JDesktopPane activateDesktopPane() {
    	return null;
    }
    
    /**
     * The desktop panel is removed from the main panel and normal content is
     * redisplayed. For example, in the galaxy map, all dialogs are closed, the
     * desktop panel is removed and the galaxy map is redisplayed in the main
     * content panel.
     * @throws ApplicationError 
     */
    public void deactivateDesktopPane() throws ApplicationError {
    }
    
    /**
     * The desktop pane is active if a dialog is being displayed in the main panel area!
     * @return True only if the desktop pane is active
     */
    public boolean isDesktopPaneActive() {
    	return false;
    }

    /** 
     * Called when something being displayed in this view may have been changed or deleted.
     * @throws ValidationError 
     * @throws ApplicationError 
     */
	public abstract void refresh() throws ApplicationError, ValidationError;

	/**
	 * @return TRUE when key events are to be processed globally - so this window cannot receive key input (text input).
	 */
	public boolean processKeyEventsGlobally() {
		return true;
	}

	/**
	 * Notification that the user has resized the screen.
	 */
	protected void resizeWindow() throws ApplicationError {
		
	}
}
