/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;

import javax.swing.JLabel;

import mot.client.deligates.UserPlayerDelegate;
import mot.client.sounds.Sound;
import mot.client.sounds.Sounds;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageAnimationConfig;
import mot.client.swing.panels.AnimationPanel;
import mot.client.swing.panels.AnimationTimerInterface;
import mot.client.swing.window.common.MotInternalWindow;
import mot.common.event.PlanetInvadeEvent;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.PropertyManager;
import mot.common.util.Utils;
import mot.common.util.ValidationError;

public class InvadeGraphicWindow extends MotInternalWindow implements ActionListener, AnimationTimerInterface {

	private static final long CHANGE_INTERVAL = 500;
    
    private static String IMAGE_TYPE = null;
    private static String IMAGE_NAME = null;
    private static String [] STAGES = null;

    private AnimationPanel animationPanel;
    
    private PlanetInvadeEvent event;
	private String message;
	
	private StatusLabel attackers;
	private StatusLabel defenders;
	
	private Sound activeSound;
	private Sound winSound;
	private Sound loseSound;

    public InvadeGraphicWindow(MotInternalWindow parent, PlanetInvadeEvent event, UserPlayerDelegate user)throws ApplicationError, ValidationError {
        super(user, parent, L.INVADE_PLANET, null);
        this.event = event;
        if (IMAGE_NAME == null) {
        	ImageAnimationConfig config = AnimationConfigFactory.getInstance().getImageAnimationConfig("Invade");
			IMAGE_NAME = config.getImage(); 
        	STAGES = config.getStagesMandatoryAsSplit();
        	IMAGE_TYPE = config.getImageType();
        }
        
        winSound = Sounds.getSound("Ground_Win");
        loseSound = Sounds.getSound("Ground_Lose");
        
        animationPanel = new AnimationPanel(this, L.INVASION_OF_+event.getTarget(), IMAGE_NAME, STAGES, IMAGE_TYPE);
        animationPanel.setInfinate(true);
        Font font = new Font(Font.SERIF, Font.BOLD, 28);
        animationPanel.changeFont(font, Color.BLACK);
        
        getContentPane().add(animationPanel, BorderLayout.CENTER);
        
        ComponentTable buttonPanel = new ComponentTable("HV,PC,HV",1,3,0,0,0);
        attackers = new StatusLabel(L.ATTACKERS+": "+event.getAttackerRace(), event.getQtyAttackers());
        defenders = new StatusLabel(L.DEFENDERS+": "+event.getDefenderRace(), event.getQtyDefenders());
        if (Comp.same(owner.getName(), event.getAttackerRace())) {
        	attackers.setHitSound(loseSound);
        	defenders.setHitSound(winSound);
        } else {
        	attackers.setHitSound(winSound);
        	defenders.setHitSound(loseSound);
        }
        buttonPanel.add(attackers);
        buttonPanel.add(PanelUtils.createButton(L.CLOSE, this));
        buttonPanel.add(defenders);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        
        lastTickTime = Calendar.getInstance().getTimeInMillis() + CHANGE_INTERVAL*2;
        
        show();
        waitUntilClosed();
    }
    
	public void hide() {
		if (isShowing()) {
	    	stopSound();
	        animationPanel.stop();
		}
		super.hide();
	}

    public void actionPerformed(ActionEvent e) {
        hide();
    }
    
    private synchronized void stopSound() {
		if (activeSound != null) {
			activeSound.stop();
			activeSound = null;
		}
	}

	private long lastTickTime = 0;
	public void handleTick() throws ApplicationError {
		//Logger.logWarning("InvadeGraphWindow.tick()");
		if (animationPanel != null) {
			animationPanel.handleTick();
		}
		if (isShowing()) {
			long timeNow = Calendar.getInstance().getTimeInMillis();
			if (timeNow - lastTickTime < CHANGE_INTERVAL) {
				return;
			}
			lastTickTime = timeNow;
			int qtyAttackersToLose = attackers.qty - event.getRemainingAttackers();
			int qtyDefendersToLose = defenders.qty - event.getRemainingDefenders();
			if (qtyDefendersToLose == 0 && qtyAttackersToLose == 0) {
				return;
			}
			int r = Utils.random(qtyDefendersToLose+qtyAttackersToLose);
			if (r < qtyDefendersToLose) {
				// lose a defender
				defenders.hit(qtyDefendersToLose);
			} else {
				// lose an attacker
				attackers.hit(qtyAttackersToLose);
			}
		}
	}
    
    class StatusLabel extends ComponentTable {

		public JLabel titleLabel;
		public JLabel qtyLabel;
		public int qty;
		private Sound sound;

		public StatusLabel(String title, int qty) throws ApplicationError {
			super("", 2,1,5,0,5);
			this.qty = qty;
			Color bkCol = PropertyManager.getMandatoryAsColor("window.invadeWindow.counter.background");
			titleLabel =  new JLabel(title);
			qtyLabel =  new JLabel(""+qty, 0);
	        Font font = new Font(Font.SERIF, Font.BOLD, 28);
	        qtyLabel.setFont(font);
	        qtyLabel.setBackground(bkCol);
	        qtyLabel.setOpaque(true);
			add(titleLabel,"C");
			add(qtyLabel, "HV");
		}
		
		private synchronized void startSound() throws ApplicationError {
			if (sound != null) {
				stopSound();
				activeSound = sound;
				sound.play();
			}
		}

		public void setHitSound(Sound sound) {
			this.sound = sound;
		}

		public void hit(int maxQtyCanKill) throws ApplicationError {
			startSound();
			qty -= (Utils.random(Comp.min(5, maxQtyCanKill))+1);
			if (qty < 0) qty = 0;
			qtyLabel.setText(""+qty);
		}
    	
    }
}