/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import java.awt.Color;
import java.awt.FlowLayout;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import mot.client.actions.SimpleAction;
import mot.client.deligates.PlanetDelegate;
import mot.client.swing.common.BoxPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.window.common.MotInternalWindow;
import mot.common.bean.MotComparable;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.PropertyManager;
import mot.common.vo.PlanetCrisisVO;

public class PlanetAnimationPanelWithDescription extends BoxPanel implements MotComparable {
	
	private MotInternalWindow owner;
	private PlanetDelegate planet;
	private PlanetAnimationPanel planetAnimationPanel;

	public PlanetAnimationPanelWithDescription(PlanetDelegate planet) throws ApplicationError {
		super(BoxLayout.Y_AXIS);
		
		this.planet = planet;
		
		if (planet != null) {
			setup();
		}
	}
	
	public PlanetAnimationPanelWithDescription(MotInternalWindow owner, PlanetDelegate planet) throws ApplicationError {
		super(BoxLayout.Y_AXIS);
		
		this.owner = owner;
		this.planet = planet;
		
		setup();
	}
	
	public void setPlanet(PlanetDelegate planet) throws ApplicationError {
		this.planet = planet;
		removeAll();
		if (planet != null) {
			setup();
		}
		invalidate();
		repaint();
	}
	
	public void setup() throws ApplicationError {
        String text1 = planet.getPlanetTypeName()+" "+Formatter.whole(planet.getSize())+
                "*"+Formatter.fraction(planet.getTeraforming());
        
        
		 add( createTitleLabel(planet, text1) );
		
		 if (planet.getSolarSystem().isVisible() ) {			 
			 if (planet.getProductionBonus() != 0) {
				 add( createTitleLabel(planet, planet.getProductionBonusName()) );
			 }
			 if (planet.getResearchBonus() != 0) {
				 add( createTitleLabel(planet, planet.getResearchBonusName()) );
			 }
			 if (planet.getReproductionBonus() != 0) {
				 add( createTitleLabel(planet, planet.getFertilityBonusName()) );
			 }
			 if (planet.getMoneyBonus() > 0) {
				 add( createTitleLabel(planet, planet.getMoneyBonusName()) );
			 }
			 if (planet.getResource() != null) {
				 add( createTitleLabel(planet, planet.getResource().getTitle()) );
			 }
			 if (planet.hasNatives()) {
				 add( createTitleLabel(planet, L.NATIVES) );
			 }
		 }
		 
		 planetAnimationPanel = new PlanetAnimationPanel(owner, planet);
		 PlanetCrisisVO crisis = planet.getCrisis();
		 
		 if (crisis != null) {
        	 planetAnimationPanel.setText(crisis.getName());
        	 planetAnimationPanel.changeFont(0,true);
         } else if (planet.isBlockaded()) {
        	 planetAnimationPanel.setText(L.BLOCKADED_CAPS);
        	 planetAnimationPanel.changeFont(0,true);
         } else if (planet.isGuarded()) {
        	 planetAnimationPanel.setText(L.GUARDED_CAPS);
        	 planetAnimationPanel.changeFont(0,true);
         } else if (planet.isDominated()) {
        	 planetAnimationPanel.setText(L.DOMINATED_CAPS);
        	 planetAnimationPanel.changeFont(0,true);
         }

		JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
		panel.add(planetAnimationPanel);
		panel.setOpaque(false);
        
        add(panel);

        setOpaque(false);
	}
	
    private JPanel createTitleLabel(PlanetDelegate planet, String text) {
        JLabel label = PanelUtils.createLabel(text);
        label.setForeground(getColor(planet));
        label.setOpaque(false);
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
        panel.add(label);
        panel.setOpaque(false);
        return panel;
    }

    private Color getColor(PlanetDelegate planet) {
        try {
            if (planet.getOwner() == null) {
                return PropertyManager.getMandatoryAsColor("window.color.text.disabled");
            } else {
                return PropertyManager.getMandatoryAsColor("window.color.text.enabled");
            }
        } catch (ApplicationError ex) {
            return Color.BLACK;
        }
    }

	public PlanetDelegate getPlanet() {
		return planet;
	}

	@Override
	public boolean same(MotComparable other) {
		return other != null && other instanceof PlanetAnimationPanelWithDescription && ((PlanetAnimationPanelWithDescription)other).getPlanet() == planet;
	}

	public void handleTick() throws ApplicationError {
		if (planetAnimationPanel != null) {
			planetAnimationPanel.handleTick();
		}
	}

	public void setClickAction(SimpleAction clickPlanetImageAction) {
		if (planetAnimationPanel != null) {
			planetAnimationPanel.setClickAction(clickPlanetImageAction);
		}
	}

}

