/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import javax.swing.JLabel;
import javax.swing.JScrollPane;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.lists.FakeTechnologyLabelList;
import mot.client.swing.panels.IconPanelMoney;
import mot.client.swing.panels.IconPanelPopulation;
import mot.client.swing.panels.IconPanelProduction;
import mot.client.swing.panels.IconPanelResearch;
import mot.client.swing.renderers.CellRendererTechnologyLabel;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.craft.CraftDesignPicturePanel;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.Size;
import mot.common.command_results.CraftDesignCR;
import mot.common.designs.Orbital;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.technology.Modules;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.ValidationError;
import mot.common.vo.OrbitVO;

public class PlanetBattleStationPanel extends ComponentTable {


	private JLabel titleLabel = new MotLabel("", 5, JLabel.CENTER, true);
    
    private JLabel populationLabel = PanelUtils.createLabel(null);
    private JLabel productionLabel = PanelUtils.createLabel(null);
    private JLabel researchLabel = PanelUtils.createLabel(null);
    private JLabel moneyLabel = PanelUtils.createLabel(null);

    private IconPanelPopulation populationIconPanel = new IconPanelPopulation(0);
    private IconPanelProduction productionIconPanel = new IconPanelProduction(0);
    private IconPanelResearch researchIconPanel = new IconPanelResearch(0,0);
    private IconPanelMoney moneyIconPanel = new IconPanelMoney(0,0);

    private FakeTechnologyLabelList moduleList;
    private CraftDesignPicturePanel stationDesignPicturePanel;

    public PlanetBattleStationPanel(MotInternalWindow parent) throws ApplicationError {
        super("-", 1,2, 5,3,3);
       
        ComponentTable leftPanel = new ComponentTable("HV", 4,1, 0, 3,3);
        PanelUtils.setTheBloodySize(leftPanel, new Size(200,200));

        ComponentTable popPanel = new ComponentTable("RT,T,HV", 1,3, 0, 3,3);
        popPanel.add(L.POPULATION);
        popPanel.add(populationLabel);
        popPanel.add(populationIconPanel);
        leftPanel.add(popPanel);
        
        ComponentTable prodPanel = new ComponentTable("RT,T,HV", 1,3, 0, 3,3);
        prodPanel.add(L.PRODUCTION);
        prodPanel.add(productionLabel);
        prodPanel.add(productionIconPanel);
        leftPanel.add(prodPanel);
       
        ComponentTable resPanel = new ComponentTable("RT,T,HV", 1,3, 0, 3,3);
        resPanel.add(L.RESEARCH);
        resPanel.add(researchLabel);
        resPanel.add(researchIconPanel);
        leftPanel.add(resPanel);
       
        moduleList = new FakeTechnologyLabelList(new CellRendererTechnologyLabel(false, true, false, new Size(30,30), true, 2, false, null, null, true, null, null, null));
        JScrollPane moduleListScrollPane = new JScrollPane(moduleList, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED , JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        leftPanel.add(moduleListScrollPane, "HVSP");
        
        ComponentTable rightPanel = new ComponentTable("HV", 2,1, 0, 3,3);
        stationDesignPicturePanel = new CraftDesignPicturePanel(parent.getOwner(), parent.getOwner(), null);
        rightPanel.add(titleLabel, "TF");
        rightPanel.add(stationDesignPicturePanel,"HVSP");

        add(leftPanel,"HV");
		add(rightPanel, "HVSP");
    }


    public void selectPlanet(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError, ValidationError {
        if (planet != null && planet.isViewable()) {
        	Orbital orbital = planet.getOrbital();
			if (orbital != null) {
                titleLabel.setText("Battle Station at "+planet.getName());
                double productionBonus = planet.getProductionBonus();
                int moneyBonus = planet.getMoneyBonus();
                int population = orbital.getPopulation(productionBonus, moneyBonus);
                int production = orbital.getProduction();
                int research = orbital.getResearch();
                populationLabel.setText( Formatter.whole(population) );
                populationIconPanel.update(population);
                productionLabel.setText( Formatter.whole(production) );
                productionIconPanel.update(production, 0);
                researchLabel.setText( Formatter.whole(research) );
                researchIconPanel.update(research, 0);
                Modules modules = orbital.getModuleTechs();
                if (modules != null) {
                    moduleList.setContent(modules, null);
                } else {
                    moduleList.clearContent();
                }
                fillStationDesignDetails(orbital, new OrbitVO(null, planet.getName()));
        	} else {
                titleLabel.setText("No Battle Station");
                populationLabel.setText( "-" );
                populationIconPanel.update(0);
                productionLabel.setText( "-" );
                productionIconPanel.update(0, 0);
                researchLabel.setText( "-" );
                researchIconPanel.update(0, 0);
                moduleList.emptyContent();
                moduleList.repaint();
    			stationDesignPicturePanel.setBackgroundImage(CraftDesignPicturePanel.BACKGROUND_IMAGE_NO_BATTLE_STATION);
    			stationDesignPicturePanel.clearDesign();
        	}
        } else {
            titleLabel.setText(L.UNKNOWN);
            populationLabel.setText( "-" );
            populationIconPanel.update(0);
            productionLabel.setText( "-" );
            productionIconPanel.update(0, 0);
            researchLabel.setText( "-" );
            researchIconPanel.update(0, 0);
            moduleList.emptyContent();
            moduleList.repaint();
			stationDesignPicturePanel.setBackgroundImage(CraftDesignPicturePanel.BACKGROUND_IMAGE_NO_BATTLE_STATION);
			stationDesignPicturePanel.clearDesign();
        }
    }


    private void fillStationDesignDetails(Orbital orbital, OrbitVO orbit) throws ApplicationError, ValidationError {
    	if (orbital != null) {
    		Hull hull = orbital.getHullTech();
    		CompletedCallback designSpaceStationCallback = new CompletedCallback() {
				public void complete(Object result) throws ApplicationError, ValidationError {
					if (result != null && result instanceof CraftDesignCR) {
						CraftDesignCR cr = (CraftDesignCR) result;
						stationDesignPicturePanel.setBackgroundImage(CraftDesignPicturePanel.BACKGROUND_IMAGE_SPACE_STATION);
						stationDesignPicturePanel.setDesign(cr.getResult(), null, null);
					}
				}
			};    		
    		ProxyStore.getInstance().designMilitaryCraft(hull.getName(), orbit, designSpaceStationCallback, MainWindow.getExestingInstance());
    	} else {
			stationDesignPicturePanel.setBackgroundImage(CraftDesignPicturePanel.BACKGROUND_IMAGE_NO_BATTLE_STATION);
			stationDesignPicturePanel.clearDesign();
    	}
	}


	public void handleTick() throws ApplicationError {
    }
}
