/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import javax.swing.JLabel;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.panels.IconPanelFactory;
import mot.client.swing.panels.IconPanelMoney;
import mot.client.swing.panels.IconPanelPollution;
import mot.client.swing.panels.IconPanelPopulation;
import mot.client.swing.panels.IconPanelProduction;
import mot.client.swing.panels.IconPanelResearch;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.PlayerCharacterBasic;
import mot.common.lang.L;
import mot.common.technology.Biotechnologies;
import mot.common.technology.Factories;
import mot.common.technology.Laboratories;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

/**
 * This is a panel that contains a rows of icon panels that display the
 * population, output, development, research etc of a planet. Exactly
 * which icons panels this panel displays is configurable.
 * 
 *  It is used on the solar system window as well as the planets list.
 */
public class PlanetIconPanel extends ComponentTable {

	private JLabel maxPopLabel;
	private JLabel popLabel;
	private JLabel factoryLabel;
	private JLabel poluteLabel;
	
	private JLabel moneyText = PanelUtils.createLabel(L.MONEY);
	
	private JLabel maxPopulationLabel = PanelUtils.createLabel(null);
    private JLabel populationLabel = PanelUtils.createLabel(null);
    private JLabel factoriesLabel = PanelUtils.createLabel(null);
    private JLabel pollutionLabel = PanelUtils.createLabel(null);
    private JLabel productionLabel = PanelUtils.createLabel(null);
    private JLabel researchLabel = PanelUtils.createLabel(null);
    private JLabel moneyLabel = PanelUtils.createLabel(null);

    private JLabel maxPopulationChangeLabel = PanelUtils.createLabel(null);
    private JLabel populationChangeLabel = PanelUtils.createLabel(null);
    private JLabel factoriesChangeLabel = PanelUtils.createLabel(null);
    private JLabel minEcologyLabel = PanelUtils.createLabel(null);

    private IconPanelPopulation populationIconPanel = new IconPanelPopulation(0);
    private IconPanelPopulation maxPopulationIconPanel = new IconPanelPopulation(0);
    private IconPanelFactory factoryIconPanel = new IconPanelFactory(0);
    private IconPanelPollution minEcologyIconPanel = new IconPanelPollution(0);
    private IconPanelProduction productionIconPanel = new IconPanelProduction(0);
    private IconPanelResearch researchIconPanel = new IconPanelResearch(0,0);
    private IconPanelMoney moneyIconPanel = new IconPanelMoney(0,0);
    
	public PlanetIconPanel() {
		this(true, true, true, true, true, true, true, true);
	}	
	
	public PlanetIconPanel(boolean withChangeLabels, boolean maxPop, boolean pop, boolean fact, boolean eco, boolean prod, boolean res, boolean money) {
		super("RT,T,T,HV", 1, withChangeLabels ? 4 : 3, 0, 3,3);
        
		int qtyRows = 0;
		if (maxPop) qtyRows++;
		if (pop) qtyRows++;
		if (fact) qtyRows++;
		if (eco) qtyRows++;
		if (prod) qtyRows++;
		if (res) qtyRows++;
		if (money) qtyRows++;
		setQtyRows(qtyRows);
		
		if (maxPop) {
			String tt = "<html>" + L.TT_PLANET_MAX_POPULATION + "</html>";
			maxPopLabel = add(L.MAX_POP);
			maxPopLabel.setToolTipText(tt);
	        add(maxPopulationLabel);
	        maxPopulationLabel.setToolTipText(tt);
	        if (withChangeLabels) {
	        	add(maxPopulationChangeLabel);
	        	maxPopulationChangeLabel.setToolTipText(tt);
	        }
	        add(maxPopulationIconPanel, "HV");
	        maxPopulationIconPanel.setToolTipText(tt);
		}        

		if (pop) {
			String tt = "<html>" + L.TT_PLANET_POPULATION + "</html>";
			popLabel = add(L.POPULATION);
			popLabel.setToolTipText(tt);
			populationLabel.setToolTipText(tt);
			add(populationLabel);
			if (withChangeLabels) {
				add(populationChangeLabel);
				populationChangeLabel.setToolTipText(tt);
			}
			add(populationIconPanel, "HV");
			populationIconPanel.setToolTipText(tt);
		}		
		
		if (fact) {
			String tt = "<html>" + L.TT_PLANET_FACTORIES + "</html>";
			factoryLabel = add(L.FACTORIES);
			factoryLabel.setToolTipText(tt);
			add(factoriesLabel);
			factoriesLabel.setToolTipText(tt);
			if (withChangeLabels) {
				add(factoriesChangeLabel);
				factoriesChangeLabel.setToolTipText(tt);
			}
			add(factoryIconPanel, "HV");
			factoryIconPanel.setToolTipText(tt);
		}	
		
		if (eco) {
			String tt = "<html>" + L.TT_PLANET_MIN_ECOLOGY + "</html>";
			poluteLabel = add(L.MIN_ECOLOGY);
			poluteLabel.setToolTipText(tt);
			add(pollutionLabel);
			pollutionLabel.setToolTipText(tt);
			if (withChangeLabels) {
				add(minEcologyLabel);
				minEcologyLabel.setToolTipText(tt);
			}
			add(minEcologyIconPanel, "HV");
			minEcologyIconPanel.setToolTipText(tt);
		}		
		
		if (prod) {
			add(L.PRODUCTION).setToolTipText(L.TT_PLANET_PRODUCTION);
			add(productionLabel, withChangeLabels ? "T2" : "T");
			productionLabel.setToolTipText(L.TT_PLANET_PRODUCTION);
			add(productionIconPanel, "HV");
			productionIconPanel.setToolTipText(L.TT_PLANET_PRODUCTION);
		}		
		
		if (res) {
			add(L.RESEARCH).setToolTipText(L.TT_PLANET_RESEARCH);
			add(researchLabel, withChangeLabels ? "T2" : "T");
			researchLabel.setToolTipText(L.TT_PLANET_RESEARCH);
			add(researchIconPanel, "HV");
			researchIconPanel.setToolTipText(L.TT_PLANET_RESEARCH);
		}		
		
		if (money) {
			String tt = "<html>" + L.TT_PLANET_MONEY + "</html>";
			moneyText.setToolTipText(L.TT_PLANET_MONEY);
			add(moneyText);
			add(moneyLabel, withChangeLabels ? "T2" : "T");
			moneyLabel.setToolTipText(L.TT_PLANET_MONEY);
			add(moneyIconPanel, "HV");
			moneyIconPanel.setToolTipText(L.TT_PLANET_MONEY);
		}			
    }
	
    public void setOpaque(boolean v) {
    	if (maxPopulationLabel != null) {
	    	maxPopulationLabel.setOpaque(v);
	    	populationLabel.setOpaque(v);
	    	factoriesLabel.setOpaque(v);
	    	pollutionLabel.setOpaque(v);
	    	productionLabel.setOpaque(v);
	    	researchLabel.setOpaque(v);
	    	moneyLabel.setOpaque(v);
	
	    	maxPopulationChangeLabel.setOpaque(v);
	    	populationChangeLabel.setOpaque(v);
	    	factoriesChangeLabel.setOpaque(v);
	    	minEcologyLabel.setOpaque(v);
	
	    	populationIconPanel.setOpaque(v);
	    	maxPopulationIconPanel.setOpaque(v);
	    	factoryIconPanel.setOpaque(v);
	    	minEcologyIconPanel.setOpaque(v);
	    	productionIconPanel.setOpaque(v);
	    	researchIconPanel.setOpaque(v);
	    	moneyIconPanel.setOpaque(v);
    	}
    	super.setOpaque(v);
    }


    public void update(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError, ValidationError {
        if (planet != null && planet.isViewable()) {
        	if (planet.isMiningColony()) {
            	updateMiningColony(planet);
        	} else {
            	updateNormalColony(planet);
        	}
        } else {
        	updateNoColony(planet);
        }
    }

    private void updateNormalColony(PlanetDelegate planet) throws ApplicationError, ValidationError {
        //ProxyStore.getInstance().setPlanetAutoPercentsCmd(planet);

        double maxPopulation = planet.getMaxPopulation();
        double qtyFactories = planet.getQtyFactories();
        double minimumEcologyOutput = planet.getMinimumEcologyOutput();
        int productionOutput = planet.getProductionOutput();
        int lostProductionOutput = planet.getLostProductionOutput();
        int researchOutput = planet.getResearchOutput();
        int lostResearchOutput = planet.getLostResearchOutput();
        int moneyOutput = planet.getMoneyOutput();
        int wastedMoneyOutput = planet.getMoneyWasted();
        double population = planet.getPopulation();
        
        double teraformingGrowth = planet.getTeraformingGrowth();
        double populationGrowth = planet.getPopulationGrowth();
        double factoriesGrowth = planet.getFactoriesGrowth();
        double minimumEcologyPercent = planet.getMinimumEcologyPercent();
        
		maxPopulationLabel.setText( Formatter.whole(maxPopulation) );
		populationLabel.setText( Formatter.whole(population) );
		factoriesLabel.setText( Formatter.whole((int)qtyFactories) );
		pollutionLabel.setText( Formatter.whole(minimumEcologyOutput) );
		productionLabel.setText( Formatter.whole(productionOutput) );
		researchLabel.setText( Formatter.whole(researchOutput) );
		moneyLabel.setText( Formatter.whole(moneyOutput) );

		maxPopulationChangeLabel.setText( Formatter.fractionWithSign(teraformingGrowth ));
		populationChangeLabel.setText( Formatter.fractionWithSign(populationGrowth ));
		factoriesChangeLabel.setText( Formatter.fractionWithSign(factoriesGrowth ));
		minEcologyLabel.setText( Formatter.percentLabelNoSign(minimumEcologyPercent) );

        maxPopulationIconPanel.update((int)maxPopulation);
        populationIconPanel.update((int)population);
        factoryIconPanel.update((int)qtyFactories);
        minEcologyIconPanel.update((int)minimumEcologyOutput);
        productionIconPanel.update(productionOutput, lostProductionOutput);
        researchIconPanel.update(researchOutput, lostResearchOutput);
        moneyIconPanel.update((int)moneyOutput, wastedMoneyOutput);
        
        populationIconPanel.setText(null);
        factoryIconPanel.setText(null);
        minEcologyIconPanel.setText(null);
        
        {
        	StringBuffer tt = new StringBuffer("<html>");
        	tt.append(L.TT_PLANET_MAX_POPULATION);
        	int size = planet.getSize();
        	double teraforming = planet.getTeraforming();
        	double productionBonus = planet.getProductionBonus();
        	int moneyBonus = planet.getMoneyBonus();
        	int battleStationPop = (planet.getOrbital() == null) ? 0 : planet.getOrbital().getPopulation(productionBonus, moneyBonus);  
        	int miningColonyPop = planet.getMiningPopulation();  
			tt.append("<br/><br/>Terrestrial Population: Size ("+size+") x Terraform ("+Formatter.fraction(teraforming)+") = "+Formatter.fraction(size*teraforming)+" (not including bonuses)");
			tt.append("<br/>Battle Station Population: "+battleStationPop);
			tt.append("<br/>Mining Colony Population: "+miningColonyPop);
			tt.append("</html>");
			String ttText = tt.toString();
			
			maxPopLabel.setToolTipText(ttText);
	        maxPopulationLabel.setToolTipText(ttText);
	        if (maxPopulationChangeLabel != null) maxPopulationChangeLabel.setToolTipText(ttText);
	        maxPopulationIconPanel.setToolTipText(ttText);
        }
        
        {
			String tt = "<html>" + L.TT_PLANET_MONEY + "<br/>Income generated: " + planet.getMoneyOutput() + "<br/>Money wasted: " + planet.getMoneyWasted() + "</html>";
			moneyText.setToolTipText(tt);
			moneyLabel.setToolTipText(tt);
			moneyIconPanel.setToolTipText(tt);	
        }

        PlayerDelegate origin = planet.getOrigin();
        if (origin != null) {
        	StringBuffer tt = new StringBuffer("<html>");
        	tt.append(L.TT_PLANET_POPULATION);
            if (origin != null && origin instanceof UserPlayerDelegate) {
            	UserPlayerDelegate up = (UserPlayerDelegate) origin;
            	PlayerCharacterBasic pc = up.getPlayerCharacter();
            	if (pc != null) {
            		tt.append("<br/>");
            		if (pc.getBattleSpeedBonus() > 0.0) tt.append("<br/>Battle Speed Bonus: "+Formatter.fraction(pc.getBattleSpeedBonus()));
            		if (pc.getBattleTurningBonus() > 0.0) tt.append("<br/>Battle Turning Bonus: "+Formatter.fraction(pc.getBattleTurningBonus()));
            		if (pc.getInitiativeBonus() > 0.0) tt.append("<br/>Initiative Bonus: "+Formatter.fraction(pc.getInitiativeBonus()));
            		if (pc.getAssaultBonus() > 0.0) tt.append("<br/>Assault Bonus: "+Formatter.fraction(pc.getAssaultBonus()));
            		if (pc.getCloakLevel() > 0) tt.append("<br/>Cloak Level: "+Formatter.fraction(pc.getCloakLevel()));
            		if (pc.getSpeedBonus() > 0.0) tt.append("<br/>Speed Bonus: "+Formatter.fraction(pc.getSpeedBonus()));
            		if (pc.getPsiRating() > 0.0) tt.append("<br/>Psi Rating: "+Formatter.fraction(pc.getPsiRating()));
            		if (pc.getEcologyBonus() > 0.0) tt.append("<br/>Ecology Bonus: "+Formatter.fraction(pc.getEcologyBonus()));
            		if (pc.getDevelopmentBonus() > 0.0) tt.append("<br/>Development Bonus: "+Formatter.fraction(pc.getDevelopmentBonus()));
            		if (pc.getProductionBonus() > 0.0) tt.append("<br/>Production Bonus: "+Formatter.fraction(pc.getProductionBonus()));
            		if (pc.getReproductionBonus() > 0.0) tt.append("<br/>Reproduction Bonus: "+Formatter.fraction(pc.getReproductionBonus()));
            		if (pc.getFactoryPerPopulationBonus() > 0.0) tt.append("<br/>Factory Per Population Bonus: "+Formatter.fraction(pc.getFactoryPerPopulationBonus()));
            		if (pc.getResearchBonus() > 0.0) tt.append("<br/>Research Bonus: "+Formatter.fraction(pc.getResearchBonus()));
            		if (pc.getSpyBonus() > 0.0) tt.append("<br/>Spy Bonus: "+Formatter.fraction(pc.getSpyBonus()));
            		if (pc.getTradingBonus() > 0.0) tt.append("<br/>Trading Bonus: "+Formatter.fraction(pc.getTradingBonus()));
            		if (pc.getEloquenceBonus() > 0.0) tt.append("<br/>Eloquence Bonus: "+Formatter.fraction(pc.getEloquenceBonus()));
            	}
            }
			tt.append("</html>");
			String ttText = tt.toString();
			
			popLabel.setToolTipText(ttText);
	        populationLabel.setToolTipText(ttText);
	        if (populationChangeLabel != null) maxPopulationChangeLabel.setToolTipText(ttText);
	        populationIconPanel.setToolTipText(ttText);
        }


		Factories factories = MainWindow.getOwner().getFactories();
		Laboratories laboratories = MainWindow.getOwner().getLaboratories();
		Biotechnologies biologies = MainWindow.getOwner().getBiotechnologies();
		if (factories != null) {
			if (factoryLabel != null) {
				StringBuffer tt = new StringBuffer("<html>");
				tt.append(L.TT_PLANET_FACTORIES);
				PlayerCharacterBasic character = planet.getOrigin().getPlayerCharacter();
				double factoriesPerPopBonus = 0.0;
				if (character == null) {
					Logger.logWarning("PlanelIconPanel.update() No Player Character for "+planet.getOrigin());
				} else {					
					factoriesPerPopBonus = character.getFactoryPerPopulationBonus();
				}
				double qtyPerPopulation = factories.getQtyPerPopulation();
				double maxQtyFactories = population*(1.0+factoriesPerPopBonus)*qtyPerPopulation;
				tt.append("<br/><br/>Maximum Factories per Population: "+qtyPerPopulation+" + player bonusus "+factoriesPerPopBonus+" (Maximum Current Usage "+Formatter.fraction(maxQtyFactories)+")");
				int prodPerFactory = factories.getProduction();
				tt.append("<br/>Production per Factory: "+Formatter.fraction(prodPerFactory));
				double qtyFactoriesInUse = Comp.min(qtyFactories, maxQtyFactories);
				tt.append("<br/>Total Factory Production: "+Formatter.fraction(qtyFactoriesInUse*prodPerFactory)+" (with all output allocated to production)");
				if (laboratories != null) {
					int prodPerLaboratory = laboratories.getProduction();
					tt.append("<br/>Total Laboratory Research: "+Formatter.fraction(qtyFactoriesInUse*prodPerLaboratory)+" (with all output allocated to research)");
				}
				tt.append("<br/>Potential Factory Production: "+Formatter.fraction(maxQtyFactories*prodPerFactory)+" (with all output allocated to production and maximum factories)");
				if (laboratories != null) {
					int prodPerLaboratory = laboratories.getProduction();
					tt.append("<br/>Potential Laboratory Research: "+Formatter.fraction(maxQtyFactories*prodPerLaboratory)+" (with all output allocated to research and maximum factories)");
				}
				tt.append("<br/>Population and other technologies / bonuses also add to output in addition to factories.");
				tt.append("</html>");
				String ttText = tt.toString();
				
				factoryLabel.setToolTipText(ttText);
				factoriesLabel.setToolTipText(ttText);
				if (factoriesChangeLabel != null)  factoriesChangeLabel.setToolTipText(ttText);
				factoryIconPanel.setToolTipText(ttText);
			}
			
			if (poluteLabel != null) {
				StringBuffer tt = new StringBuffer("<html>");
				tt.append(L.TT_PLANET_MIN_ECOLOGY);
				double minEcologyWork = factories.getMinEcologyWork();
				tt.append("<br/><br/>Current Minimum Ecology Work: "+Formatter.percentLabelNoSign(minEcologyWork)+" (percent of production lost maintaining your ecology)");
				double maxTerraform = 1.0;
				if (biologies != null) {
					maxTerraform = biologies.getTerraformRating();
				}
				double teraforming = planet.getTeraforming();
				double size = planet.getSize();
				tt.append("<br/>Maximum Terraform: "+Formatter.fraction(maxTerraform)+" (as you discover new technologies you will be able to increase your maximum terraform)");
				tt.append("<br/>Current Terraform: "+Formatter.fraction(teraforming)+" (Maximum population ignoring bonuses and extra terrestrial population "+Formatter.fraction(teraforming*size)+")");
				tt.append("</html>");
				String ttText = tt.toString();
				
				poluteLabel.setToolTipText(ttText);
				pollutionLabel.setToolTipText(ttText);
				if (minEcologyLabel != null) minEcologyLabel.setToolTipText(ttText);
				minEcologyIconPanel.setToolTipText(ttText);
			}
			
		}

    }
    
    private void updateMiningColony(PlanetDelegate planet) throws ApplicationError, ValidationError {
		PlanetDelegate mineOwner = planet.getOwnerOfMiningStation();
		if (mineOwner == null) {
			Logger.logWarning("Station around planet "+planet.getName()+" does not have an owner");
			return;
		}
		double maxPopulation = mineOwner.getMiningPopulation();
        int productionOutput = mineOwner.getMiningProduction();
        int researchOutput = mineOwner.getMiningResearch();
        int moneyOutput = mineOwner.getMiningMoney();
        int lostProduction = planet.getLostProductionOutput();
        int lostResearch = planet.getLostResearchOutput();
        int lostMoney = planet.getMoneyWasted();

		maxPopulationLabel.setText( Formatter.whole(maxPopulation) );
		populationLabel.setText("N/A");
		factoriesLabel.setText("N/A");
		pollutionLabel.setText("N/A");
		productionLabel.setText( Formatter.whole(productionOutput) );
		researchLabel.setText( Formatter.whole(researchOutput) );
		moneyLabel.setText( Formatter.whole(moneyOutput) );

		maxPopulationChangeLabel.setText("");
		populationChangeLabel.setText("");
		factoriesChangeLabel.setText("");
		minEcologyLabel.setText("");

        maxPopulationIconPanel.update((int)maxPopulation);
        populationIconPanel.update(0);
        factoryIconPanel.update(0);
        minEcologyIconPanel.update(0);
        productionIconPanel.update((int)productionOutput, lostProduction);
        researchIconPanel.update((int)researchOutput, lostResearch);
        moneyIconPanel.update((int)moneyOutput, lostMoney);

        populationIconPanel.setText(L.MINING_COLONY);
        factoryIconPanel.setText(L.MINING_COLONY);
        minEcologyIconPanel.setText(L.MINING_COLONY);
        
		
        if (popLabel != null) {
			String ttText = "<html>"+L.TT_PLANET_POPULATION+"</html>";
			popLabel.setToolTipText(ttText);
	        populationLabel.setToolTipText(ttText);
	        if (populationChangeLabel != null) maxPopulationChangeLabel.setToolTipText(ttText);
	        populationIconPanel.setToolTipText(ttText);
        }
        
        if (maxPopLabel != null) {
			String ttText = "<html>"+L.TT_PLANET_MAX_POPULATION+"</html>";
			maxPopLabel.setToolTipText(ttText);
	        maxPopulationLabel.setToolTipText(ttText);
	        if (maxPopulationChangeLabel != null) maxPopulationChangeLabel.setToolTipText(ttText);
	        maxPopulationIconPanel.setToolTipText(ttText);
        }

		if (factoryLabel != null) {
			String ttText = "<html>"+L.TT_PLANET_FACTORIES+"</html>";
			factoryLabel.setToolTipText(ttText);
			factoriesLabel.setToolTipText(ttText);
			if (factoriesChangeLabel != null)  factoriesChangeLabel.setToolTipText(ttText);
			factoryIconPanel.setToolTipText(ttText);
		}

		if (poluteLabel != null) {
			String tt = "<html>" + L.TT_PLANET_MIN_ECOLOGY + "</html>";
			poluteLabel.setToolTipText(tt);
			pollutionLabel.setToolTipText(tt);
			if (minEcologyLabel != null) minEcologyLabel.setToolTipText(tt);
			minEcologyIconPanel.setToolTipText(tt);
		}

    }
    
    private void updateNoColony(PlanetDelegate planet) throws ApplicationError, ValidationError {
        double maxPopulation = planet == null ? 0.0 : planet.getMaxPopulation();

        maxPopulationLabel.setText( Formatter.whole(maxPopulation) );
        populationLabel.setText("");
        factoriesLabel.setText("");
        pollutionLabel.setText("");
		productionLabel.setText("");
		researchLabel.setText("");
		moneyLabel.setText("");

        maxPopulationChangeLabel.setText("");
        populationChangeLabel.setText("");
        factoriesChangeLabel.setText("");
        minEcologyLabel.setText("");

        maxPopulationIconPanel.update((int)maxPopulation);
        populationIconPanel.update(0);
        factoryIconPanel.update(0);
        minEcologyIconPanel.update(0);
        productionIconPanel.update(0, 0);
        researchIconPanel.update(0, 0);
        moneyIconPanel.update(0, 0);
        
        populationIconPanel.setText(null);
        factoryIconPanel.setText(null);
        minEcologyIconPanel.setText(null);
        
		
        if (popLabel != null) {
			String ttText = "<html>"+L.TT_PLANET_POPULATION+"</html>";
			popLabel.setToolTipText(ttText);
	        populationLabel.setToolTipText(ttText);
	        if (populationChangeLabel != null) maxPopulationChangeLabel.setToolTipText(ttText);
	        populationIconPanel.setToolTipText(ttText);
        }
        
        if (maxPopLabel != null) {
			String ttText = "<html>"+L.TT_PLANET_MAX_POPULATION+"</html>";
			maxPopLabel.setToolTipText(ttText);
	        maxPopulationLabel.setToolTipText(ttText);
	        if (maxPopulationChangeLabel != null) maxPopulationChangeLabel.setToolTipText(ttText);
	        maxPopulationIconPanel.setToolTipText(ttText);
        }
        
        if (factoryLabel != null) {
			String ttText = "<html>"+L.TT_PLANET_FACTORIES+"</html>";
			factoryLabel.setToolTipText(ttText);
			factoriesLabel.setToolTipText(ttText);
			if (factoriesChangeLabel != null)  factoriesChangeLabel.setToolTipText(ttText);
			factoryIconPanel.setToolTipText(ttText);
		}
		
		if (poluteLabel != null) {
			String tt = "<html>" + L.TT_PLANET_MIN_ECOLOGY + "</html>";
			poluteLabel.setToolTipText(tt);
			pollutionLabel.setToolTipText(tt);
			if (minEcologyLabel != null) minEcologyLabel.setToolTipText(tt);
			minEcologyIconPanel.setToolTipText(tt);
		}

    }
	
}
