/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import javax.swing.JLabel;
import javax.swing.JPanel;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.TransportDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.labels.PopulationLabel;
import mot.client.swing.panels.IconPanelCulture;
import mot.client.swing.panels.IconPanelDefense;
import mot.client.swing.panels.ImagePanel;
import mot.client.swing.window.empire.actions.EspionageSolarSystemButton;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionButtonAbandon;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionButtonRename;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionButtonRenamePlanet;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionButtonTransfer;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionRemoveWaypoint;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionSetWaypoint;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.ValidationError;
import mot.common.vo.OrbitVO;
import mot.common.vo.PlanetCrisisVO;

public class PlanetOtherPanel extends ComponentTable {

	private static final int TEXT_SIZE = 3;
	
    public JLabel crisisDescriptionLabel = new MotLabel("", TEXT_SIZE, JLabel.LEFT, true);
	public JLabel solarSystemNameLabel = new MotLabel("", TEXT_SIZE, JLabel.LEFT, true);
    public JLabel planetNameLabel = new MotLabel("", TEXT_SIZE, JLabel.LEFT, true);
    public PopulationLabel populationDetailsLabel = new PopulationLabel(TEXT_SIZE, JLabel.LEFT, true);
    public JLabel defenseLabel = new MotLabel("", TEXT_SIZE, JLabel.LEFT, true);
    public JLabel waypointLabel = new MotLabel("", TEXT_SIZE, JLabel.LEFT, true);
    PadPanel waypointAnimationPanel = new PadPanel(5);
    PadPanel transportPopulationPanel = new PadPanel(5);
    PadPanel transportTroopsPanel = new PadPanel(5);

	private JLabel qtyGroundDefenseLabel = new MotLabel("", TEXT_SIZE, JLabel.LEFT, true);
    private IconPanelDefense qtyGroundDefenseIconPanel = new IconPanelDefense(0);

    private JLabel qtyCultureLabel = new MotLabel("", TEXT_SIZE, JLabel.LEFT, true);
    private IconPanelCulture qtyCultureIconPanel = new IconPanelCulture(0);
    
    private PlanetAnimationPanel waypointPlanetAnimationPanel;
    private StellarObjectAnimationPanel waypointStellarObjectAnimationPanel;

	private SolarSystemWindow parent;
    
	public PlanetOtherPanel(SolarSystemWindow parent, SolarSystemWindowState state) throws ApplicationError {
		super("HV", 6,1, 5, 5,5);
		this.parent = parent;

        ComponentTable crisisPanel = new ComponentTable("HVP", 1,1, 0, 3,3);
        crisisPanel.add(crisisDescriptionLabel);

        ComponentTable outputPanel = new ComponentTable("HV", 2,2, 0, 3,3);
        outputPanel.add(new MotLabel(L.INFLUENCE, TEXT_SIZE, JLabel.LEFT, true));        
        outputPanel.add(PanelUtils.createNamedControlH(qtyCultureLabel, qtyCultureIconPanel), "HV");
        outputPanel.add(new MotLabel(L.GROUND_DEFENSES, TEXT_SIZE, JLabel.LEFT, true));        
        outputPanel.add(PanelUtils.createNamedControlH(qtyGroundDefenseLabel, qtyGroundDefenseIconPanel), "HV");
        
        qtyCultureLabel.setToolTipText(L.TT_PLANET_INFLUENCE);
        qtyCultureIconPanel.setToolTipText(L.TT_PLANET_INFLUENCE);
        qtyGroundDefenseLabel.setToolTipText(L.TT_PLANET_GROUND_DEFENCES);
        qtyGroundDefenseIconPanel.setToolTipText(L.TT_PLANET_GROUND_DEFENCES);

		ComponentTable renamePanel = new ComponentTable("R,HV,HV,HVP", 4,3, 7, 3,3);
		
        SolarSystemActionButtonRename b1 = new SolarSystemActionButtonRename(parent, this, state);
        b1.setToolTipText(L.TT_PLANET_RENAME);
        renamePanel.add(new PadPanel(b1, 5));
		renamePanel.add(new MotLabel(L.SOLAR_SYSTEM_NAME_, L.TT_PLANET_RENAME, TEXT_SIZE, JLabel.LEFT, true));
        solarSystemNameLabel.setToolTipText(L.TT_PLANET_RENAME);
        renamePanel.add(solarSystemNameLabel);
        
        SolarSystemActionButtonRenamePlanet b2 = new SolarSystemActionButtonRenamePlanet(parent, this, state);
        b2.setToolTipText(L.TT_PLANET_RENAME);
        renamePanel.add(new PadPanel(b2, 5));
        renamePanel.add(new MotLabel(L.PLANET_NAME_, L.TT_PLANET_RENAME, TEXT_SIZE, JLabel.LEFT, true));
        planetNameLabel.setToolTipText(L.TT_PLANET_RENAME);
        renamePanel.add(planetNameLabel);

        renamePanel.add(new PadPanel(new SolarSystemActionButtonAbandon(parent, this, state), 5));
        renamePanel.add(new JPanel());
        renamePanel.add(new JPanel());

        renamePanel.add(new PadPanel(new EspionageSolarSystemButton(parent, state), 5));
        renamePanel.add(new JPanel());
        renamePanel.add(new JPanel());
        
		ComponentTable transportPanel = new ComponentTable("R,-,RTP", 2,3, 7, 3,3);
		
        SolarSystemActionButtonTransfer b3 = new SolarSystemActionButtonTransfer(L.TRANSFER_COLONISTS, parent, this, state, false);
        b3.setToolTipText(L.TT_PLANET_LOAD_TRANSPORT_COLONISTS);
        transportPanel.add(new PadPanel(b3, 5));
        populationDetailsLabel.setToolTipText(L.TT_PLANET_LOAD_TRANSPORT_COLONISTS);
        transportPanel.add(populationDetailsLabel);
        transportPanel.add(transportPopulationPanel);
        
        SolarSystemActionButtonTransfer b4 = new SolarSystemActionButtonTransfer(L.TRANSFER_TROOPS, parent, this, state, true);
        b4.setToolTipText(L.TT_PLANET_LOAD_TRANSPORT_TROOPS);
        transportPanel.add(new PadPanel(b4, 5));
        defenseLabel.setToolTipText(L.TT_PLANET_LOAD_TRANSPORT_TROOPS);
        transportPanel.add(defenseLabel);
        transportPanel.add(transportTroopsPanel);

		ComponentTable waypointButtonsPanel = new ComponentTable("R", 3,1, 5, 5,5);
		waypointButtonsPanel.add(new SolarSystemActionSetWaypoint(parent, this, state, false));
		waypointButtonsPanel.add(new SolarSystemActionSetWaypoint(parent, this, state, true));
		waypointButtonsPanel.add(new SolarSystemActionRemoveWaypoint(parent, this, state));
		
		ComponentTable waypointPlanetPanel = new ComponentTable("-", 2,1, 7, 3,3);
		waypointPlanetPanel.add(waypointAnimationPanel,"HV");
		waypointLabel.setToolTipText(L.TT_FLEET_WAY_POINT);
		waypointPlanetPanel.add(waypointLabel);

		ComponentTable waypointPanel = new ComponentTable("-,-,HVP", 1,3, 7, 3,3);
		waypointPanel.add(waypointButtonsPanel);
		waypointPanel.add(waypointPlanetPanel);
		waypointPanel.add(new JLabel(""));

		
		add(new MotBorder(crisisPanel, L.CRISIS));
		add(new MotBorder(outputPanel, L.OUTPUT));
		add(new MotBorder(renamePanel, L.MISCELLANEOUS));
		add(new MotBorder(transportPanel, L.TRANSPORTS));
		add(new MotBorder(waypointPanel, L.WAY_POINT));
        add(new JLabel(), "HVS");
	}

	
    public void selectPlanet(PlanetDelegate planet, UserPlayerDelegate owner) throws ApplicationError, ValidationError {
        boolean isViewable = (planet != null) && planet.isViewable();
        if (owner != null && isViewable && planet != null) {
        	PlanetCrisisVO crisis = planet.getCrisis();
			if (crisis != null) {
                crisisDescriptionLabel.setText(crisis.getDescription());
        	} else {
        		crisisDescriptionLabel.setText(L.NONE);
        	}
            solarSystemNameLabel.setText(planet.getSolarSystemName());
            planetNameLabel.setText(planet.getName());
            populationDetailsLabel.update(planet);
            defenseLabel.setText("of "+planet.getQtyGroundDefense());
            transportPopulationPanel.removeAll();
            transportTroopsPanel.removeAll();
            TransportDelegate [] transports = DeligateCacheManager.getTransportsAt(owner, planet);
            if (transports != null && transports.length > 0) {
            	for (int i = 0; i < transports.length; i++) {
					String taskDescription = transports[i].getTaskDescription();
					if (taskDescription != null && taskDescription.contains("troops")) {
						ComponentTable t = new ComponentTable("HV,HVSP", 1, 2, 2, 10, 0);
						t.add(new ImagePanel(transports[i].getImage(), new Size(30,30)));
						t.add(new MotLabel(taskDescription, TEXT_SIZE, JLabel.LEFT, true));
						transportTroopsPanel.add(t);						
					} else if (taskDescription != null && taskDescription.contains("colonists")) {
						ComponentTable t = new ComponentTable("HV,HVSP", 1, 2, 2, 10, 0);
						t.add(new ImagePanel(transports[i].getImage(), new Size(30,30)));
						t.add(new MotLabel(taskDescription, TEXT_SIZE, JLabel.LEFT, true));
						transportPopulationPanel.add(t);
					}
				}
            }
            qtyCultureIconPanel.update(planet.getInfluence());
            qtyCultureLabel.setText(Formatter.whole(planet.getInfluence()));
            qtyGroundDefenseIconPanel.update(planet.getQtyGroundDefense());
            qtyGroundDefenseLabel.setText(Formatter.whole(planet.getQtyGroundDefense()));
        } else {
            crisisDescriptionLabel.setText("");
            solarSystemNameLabel.setText((planet == null) ? "" : planet.getName());
            planetNameLabel.setText("");
            populationDetailsLabel.setText("");
            defenseLabel.setText("");
            transportPopulationPanel.removeAll();
            transportTroopsPanel.removeAll();
            qtyCultureIconPanel.update(0);
            qtyCultureLabel.setText("");
            qtyGroundDefenseIconPanel.update(0);
            qtyGroundDefenseLabel.setText("");
        }
        OrbitVO fleetWayPoint = owner.getFleetWayPoint();
        waypointAnimationPanel.removeAll();
		waypointPlanetAnimationPanel = null;
		waypointStellarObjectAnimationPanel = null;
        if (fleetWayPoint == null) {
        	waypointLabel.setText("");
        } else {
        	if (fleetWayPoint.getPlanet() != null) {
        		PlanetDelegate p = DeligateCacheManager.getPlanet((fleetWayPoint.getPlanet()));
        		waypointPlanetAnimationPanel = new PlanetAnimationPanel(parent, p);
        		waypointAnimationPanel.add(waypointPlanetAnimationPanel);
        		waypointLabel.setText(fleetWayPoint.getPlanet());
        	} else if (fleetWayPoint.getStellarObject() != null) {
        		StellarObjectDelegate stellarObject = DeligateCacheManager.getStellarObject(fleetWayPoint);
        		waypointStellarObjectAnimationPanel = new StellarObjectAnimationPanel(parent, stellarObject);
        		waypointAnimationPanel.add(waypointStellarObjectAnimationPanel);
        		waypointLabel.setText(fleetWayPoint.getStellarObject());
        	} else {
        		waypointLabel.setText("");
        	}
        }
    }
    
    public void handleTick() throws ApplicationError {
		if (waypointPlanetAnimationPanel != null) {
			waypointPlanetAnimationPanel.handleTick();
		}
		if (waypointStellarObjectAnimationPanel != null) {
			waypointStellarObjectAnimationPanel.handleTick();
		}
    }
}
