/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import java.awt.Color;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSlider;
import javax.swing.event.ChangeListener;

import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PadPanel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.window.galaxy.GalaxyWindowContextPanel;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionAutoDevelopment;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionAutoEcology;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionBuyProject;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionChangeProject;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionVaryProduction;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionVaryResearch;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;

public class PlanetOutputPanel extends ComponentTable {

    //public static final String DEV_AUTO_CHECK_BOX = "DEV_AUTO_CHECK_BOX";
    //public static final String ECO_AUTO_CHECK_BOX = "ECO_AUTO_CHECK_BOX";

	private JLabel productionLabel = new JLabel();
    private JLabel researchLabel = new JLabel();
    private JLabel ecologyLabel = new JLabel();
    private JLabel developmentLabel = new JLabel();
    private JCheckBox ecoCheckBox;
    private JCheckBox devCheckBox;
    
    private JLabel surplessLabel = new JLabel();
    private JCheckBox resCheckBox;
    private JCheckBox prodCheckBox;
    
    private JSlider productionPercent;
    private JSlider researchPercent;
    private JSlider ecologyPercent;
    private JSlider developmentPercent;
    

    private JLabel projectLabel;
    private JLabel repeatLabel;
    
    private JProgressBar projectProgress;

    private ProjectImagePanel imagePanel;
    
	private PlanetIconPanel planetIconPanel;
    
    public PlanetOutputPanel(SolarSystemWindow parent, ChangeListener cl, PlanetIconPanel planetIconPanel) throws ApplicationError {
        super("HT", 2,1, 0,0,3);
        
		this.planetIconPanel = planetIconPanel;
        
        imagePanel = new ProjectImagePanel(parent);
        
        productionPercent = PanelUtils.createPercentSlider(cl, false);
        researchPercent = PanelUtils.createPercentSlider(cl, false);
        ecologyPercent = PanelUtils.createPercentSlider(cl, false);
        developmentPercent = PanelUtils.createPercentSlider(cl, false);

        productionLabel.setToolTipText(L.TT_PRODUCTION_PERCENT);
        productionPercent.setToolTipText(L.TT_PRODUCTION_PERCENT);
        researchLabel.setToolTipText(L.TT_PRODUCTION_PERCENT);
        researchPercent.setToolTipText(L.TT_RESEARCH_PERCENT);
        ecologyLabel.setToolTipText(L.TT_PRODUCTION_PERCENT);
        ecologyPercent.setToolTipText(L.TT_ECOLOGY_PERCENT);
        developmentLabel.setToolTipText(L.TT_PRODUCTION_PERCENT);
        developmentPercent.setToolTipText(L.TT_DEVELOPMENT_PERCENT);
        
        projectProgress = new JProgressBar(JProgressBar.VERTICAL);
        projectProgress.setToolTipText(L.TT_PROJECT_PROGRESS);

        devCheckBox = new SolarSystemActionAutoDevelopment(parent, parent.getState()); 
        ecoCheckBox = new SolarSystemActionAutoEcology(parent, parent.getState());

        resCheckBox = new SolarSystemActionVaryResearch(parent, parent.getState()); 
        prodCheckBox = new SolarSystemActionVaryProduction(parent, parent.getState()); 

        resCheckBox.setToolTipText(L.TT_VARY_PRODUCTION);
        prodCheckBox.setToolTipText(L.TT_VARY_PRODUCTION);
        surplessLabel.setToolTipText(L.TT_VARY_PRODUCTION);
        devCheckBox.setToolTipText(L.TT_AUTO_ECOLOGY_PERCENT);
        ecoCheckBox.setToolTipText(L.TT_AUTO_DEVELOPMENT_PERCENT);

        ComponentTable slidersPanel = new ComponentTable("R,-,HVP,R,R", 4,5, 0, 3,1);
        
        slidersPanel.add(L.PRODUCTION);
        slidersPanel.add(productionLabel);
        slidersPanel.add(productionPercent);
        slidersPanel.add(prodCheckBox);
        slidersPanel.add("Surplus","C");
        
        slidersPanel.add(L.RESEARCH);
        slidersPanel.add(researchLabel);
        slidersPanel.add(researchPercent);
        slidersPanel.add(resCheckBox);
        slidersPanel.add(surplessLabel,"C");
        
        slidersPanel.add(L.ECOLOGY);
        slidersPanel.add(ecologyLabel);
        slidersPanel.add(ecologyPercent);
        slidersPanel.add(ecoCheckBox,"R2");
        
        slidersPanel.add(L.DEVELOPMENT);
        slidersPanel.add(developmentLabel);
        slidersPanel.add(developmentPercent);
        slidersPanel.add(devCheckBox,"R2");

        double windowWidth = MainWindow.getInstance().getSize().getWidth();
        int fontSize = 12;
        if (windowWidth < 1200) fontSize = 2;
        else if (windowWidth < 1600) fontSize = 4;
        
        projectLabel = new MotLabel(null, fontSize, null, null, JLabel.LEFT, true);
        repeatLabel = new MotLabel(null, fontSize, null, null, JLabel.LEFT, true);
        
        JPanel projectHeaderPanel = new ComponentTable("HV,-,HVSP,-,HV", 1,5, 2, 0,0);
        projectHeaderPanel.add(new MotLabel(L.PROJECT_+":", fontSize, null, null, 0, true));
        projectHeaderPanel.add(PanelUtils.createFiller(5,5));
        projectHeaderPanel.add(projectLabel);
        projectHeaderPanel.add(PanelUtils.createFiller(5,5));
        projectHeaderPanel.add(repeatLabel);

        ComponentTable projectControlPanel = new ComponentTable("-", 3, 1, 0, 0, 4);
        projectControlPanel.add(new SolarSystemActionChangeProject(parent, parent.getState()));
        projectControlPanel.add(new SolarSystemActionBuyProject(parent, parent.getState()));
        PadPanel progressPadPanel = new PadPanel(projectProgress, 5, 5, 0, 4);
        projectControlPanel.add(progressPadPanel, "HVSP");
        
        ComponentTable projectPanel = new ComponentTable("-,-", 2,2, 3, 3,3);
        projectPanel.add(projectHeaderPanel, "RHV2");
        projectPanel.add(projectControlPanel, "HV");
        MotBorder projectBorder = new MotBorder(imagePanel, L.PROJECT);
		projectPanel.add(new PadPanel(projectBorder, 4, 4, 0, 4), "HVSP");
        
        Color projectBackground = PropertyManager.getMandatoryAsColor("window.solarsystem.project.background");
        projectPanel.setBackground(projectBackground);
        repeatLabel.setBackground(projectBackground);
        projectBorder.setBackground(projectBackground);
        imagePanel.setBackground(Color.BLACK);
        projectHeaderPanel.setBackground(projectBackground);
        projectPanel.setBackground(projectBackground);
        projectLabel.setBackground(projectBackground);
        projectControlPanel.setBackground(projectBackground);
        projectProgress.setBackground(projectBackground);
        progressPadPanel.setBackground(projectBackground);
        
        add(slidersPanel);
        add(projectPanel, "HVSP");
    }

    public void selectPlanet(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError, ValidationError {
        boolean viewable = (planet != null) && planet.isViewable();
        boolean isMiningColony = (viewable && planet.isMiningColony());
		if (viewable && !isMiningColony) {
            productionLabel.setText( Formatter.whole(planet.getPercentProduction()*100)+"%" );
            researchLabel.setText( Formatter.whole(planet.getPercentResearch()*100)+"%" );
            developmentLabel.setText( Formatter.whole(planet.getPercentDevelopment()*100)+"%" );
            ecologyLabel.setText( Formatter.whole(planet.getPercentEcology()*100)+"%" );
    
            developmentPercent.setValue((int)(planet.getPercentDevelopment()*100));
            ecologyPercent.setValue((int)(planet.getPercentEcology()*100));
            researchPercent.setValue((int)(planet.getPercentResearch()*100));
            productionPercent.setValue((int)(planet.getPercentProduction()*100));
            
            developmentPercent.setEnabled(!planet.getAutoDevelopment());
            ecologyPercent.setEnabled(!planet.getAutoEcology());
            devCheckBox.setSelected(planet.getAutoDevelopment());
            ecoCheckBox.setSelected(planet.getAutoEcology());
            
            prodCheckBox.setSelected(planet.getVaryProduction());
            resCheckBox.setSelected(planet.getVaryResearch());
            surplessLabel.setText(Formatter.whole(planet.getPercentMoney()*100)+"%");
            
            try {
                projectLabel.setText(planet.getProjectDescription());
            } catch(ApplicationError ex) {
                Logger.logError("Error", ex);
            }
            
            if (planet.getRepeatProject() > 0) {
                repeatLabel.setText("X"+(planet.getRepeatProject()+1));
            } else {
                repeatLabel.setText("");
            }
            
            updateProjectProgressBar(planet, owner);
            
        } else {
            developmentPercent.setEnabled(false);
            ecologyPercent.setEnabled(false);
            productionPercent.setEnabled(false);
            researchPercent.setEnabled(false);
            devCheckBox.setSelected(false);
            ecoCheckBox.setSelected(false);

            productionLabel.setText("");
            researchLabel.setText("");
            developmentLabel.setText("");
            ecologyLabel.setText("");
            
            developmentPercent.setValue(50);
            ecologyPercent.setValue(50);
            researchPercent.setValue(50);
            productionPercent.setValue(50);

            projectLabel.setText("");
            repeatLabel.setText("");
            
            projectProgress.setValue(0);
        }
        
        imagePanel.setProject(planet, viewable);
    }

	protected void updateProjectProgressBar(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError {
        if (planet.getProjectDesign() != null) {
        	// MUST GET SERVER CALC BECAUSE MAY HAVE USED SALVAGE!
    		double percent = planet.getProduction() / (double)planet.getCostToBuild();
    		projectProgress.setValue((int)(percent*100.0));
        } else {
        	projectProgress.setValue(0);
        }
	}
    
    public void updateProject(PlanetDelegate planet) throws ApplicationError, ValidationError {
        if (planet.getRepeatProject() > 0) {
            repeatLabel.setText("X"+(planet.getRepeatProject()+1));
        } else {
            repeatLabel.setText("");
        }
        projectLabel.setText(planet.getProjectDescription());
        UserPlayerDelegate owner = MainWindow.getOwner();
        boolean viewable = planet.isViewable();
        imagePanel.setProject(planet, viewable);
        updateProjectProgressBar(planet, owner);
        GalaxyWindowContextPanel galaxyContextPanel = MainWindow.getExestingInstance().getGalaxyMapConfiguration().getGalaxyView().getContextPanel();
		NamedObjectDelegate selection = galaxyContextPanel.getSelection();
		if (selection != null && selection.equals(planet)) {
			galaxyContextPanel.updateSelectionState();
		}
        planetIconPanel.update(planet, owner);

    }

    boolean ignore = false;
    public void updateProdCheckBox(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError, ValidationError {
    	if (ignore) return;
        boolean prodSelected = prodCheckBox.isSelected();
        boolean resSelected = resCheckBox.isSelected();
        if (prodSelected && resSelected) {
        	ignore = true;
        	resCheckBox.setSelected(false);
        	resSelected = false; 
        	ignore = false;
        }
		ProxyStore.getInstance().setPlanetAutoPercentsCmd(planet, new Boolean(prodSelected), new Boolean(resSelected), null, null, new UpdateAutoPercentsCallback(planet), null);
    }

    public void updateResCheckBox(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError, ValidationError {
    	if (ignore) return;
        boolean prodSelected = prodCheckBox.isSelected();
        boolean resSelected = resCheckBox.isSelected();
        if (prodSelected && resSelected) {
        	ignore = true;
        	prodCheckBox.setSelected(false);
        	prodSelected = false; 
        	ignore = false;
        }
        ProxyStore.getInstance().setPlanetAutoPercentsCmd(planet, new Boolean(prodCheckBox.isSelected()), new Boolean(resCheckBox.isSelected()), null, null, new UpdateAutoPercentsCallback(planet), null);
    }

    public void updateDevCheckBox(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError, ValidationError {
        developmentPercent.setEnabled(!devCheckBox.isSelected());
        ProxyStore.getInstance().setPlanetAutoPercentsCmd(planet, null, null, new Boolean(devCheckBox.isSelected()), null, new UpdateAutoPercentsCallback(planet), null);
        if (devCheckBox.isSelected()) {
            developmentLabel.setText( Formatter.whole(planet.getPercentDevelopment()*100)+"%" );
            developmentPercent.setValue((int)(planet.getPercentDevelopment()*100));
        }
    }

    public void updateEcoCheckBox(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError, ValidationError {
        ecologyPercent.setEnabled(!ecoCheckBox.isSelected());
        ProxyStore.getInstance().setPlanetAutoPercentsCmd(planet, null, null, null, new Boolean(ecoCheckBox.isSelected()), new UpdateAutoPercentsCallback(planet), null);
        if (ecoCheckBox.isSelected()) {
            ecologyLabel.setText( Formatter.whole(planet.getPercentEcology()*100)+"%" );
            ecologyPercent.setValue((int)(planet.getPercentEcology()*100));
        }
    }
    
    public void setPlanetPercents(PlanetDelegate planet, PlayerDelegate owner, CompletedCallback callback) throws ApplicationError, ValidationError {
        ProxyStore.getInstance().setPlanetPercentsCmd(planet, 
                                                      developmentPercent.getValue()/100.0, 
                                                      ecologyPercent.getValue()/100.0, 
                                                      productionPercent.getValue()/100.0, 
                                                      researchPercent.getValue()/100.0,
                                                      callback, null);
    }

    public boolean inAutoPercentsCallback = false;
    private class UpdateAutoPercentsCallback implements CompletedCallback {
		private PlanetDelegate planet;
		public UpdateAutoPercentsCallback(PlanetDelegate planet) {
			this.planet = planet;
		}
		@Override
		public void complete(Object result) throws ApplicationError, ValidationError {
			inAutoPercentsCallback = true;
	        developmentLabel.setText( Formatter.whole(planet.getPercentDevelopment()*100)+"%" );
	        developmentPercent.setValue((int)(planet.getPercentDevelopment()*100));
	        ecologyLabel.setText( Formatter.whole(planet.getPercentEcology()*100)+"%" );
	        ecologyPercent.setValue((int)(planet.getPercentEcology()*100));
	        planetIconPanel.update(planet, MainWindow.getOwner());
	        inAutoPercentsCallback = false;
		}
    }


    public void handleTick() throws ApplicationError {
//		if (waypointPlanetAnimationPanel != null) {
//			waypointPlanetAnimationPanel.handleTick();
//		}
    }


}