/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import mot.client.bean.Updatable;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.swing.buttons.ActionObserver;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.solarsystem.actions.ActionButtonTransfer;
import mot.client.swing.window.solarsystem.actions.ActionButtonViewPlanet;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.ValidationError;

/**
 * This is a panel that is used to display details about a planet. It contains a picture of
 * the planet on the left with population, production, research and money output on the right.
 * This panel is used on the Empire Details window.
 */
public class PlanetPanel extends ComponentTable implements Updatable {
	
	//
    // Which details to show for each planet
    //
    private static boolean maxPop = true;
    private static boolean pop = true;
    private static boolean fact = false;
    private static boolean eco = false;
    private static boolean prod = true;
    private static boolean res = true;
    private static boolean money = false;

    private PlanetDelegate planet;
    private PlayerDelegate windowOwner;

    private JLabel titleLabel = PanelUtils.createLabel(null, 5, SwingConstants.CENTER);
    private JLabel projectLabel = PanelUtils.createLabel(null);
    private PlanetIconPanel planetIconPanel;
	private MotInternalWindow parent;
	
	private PlanetAnimationPanel planetAnimationPanel;
	
	private ActionButtonViewPlanet viewButton;
	private ActionButtonTransfer transferButton;
    
    public PlanetPanel(MotInternalWindow parent, ActionObserver actionObserver, PlanetDelegate planet, PlayerDelegate windowOwner) throws ApplicationError, ValidationError {
    	super("HV,HVSP,HV", 1,3, 3, 2,2);
    	
		this.parent = parent;
        this.planet = planet;
        this.windowOwner = windowOwner;
        
        //setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));

        transferButton = new ActionButtonTransfer(planet, actionObserver);
        viewButton = new ActionButtonViewPlanet(planet, parent, this);
        ComponentTable buttonPanel = new ComponentTable("HSP", 2,1, 0, 2,2);
		buttonPanel.add(transferButton);
		buttonPanel.add(viewButton);
        buttonPanel.setOpaque(false);

        planetAnimationPanel = new PlanetAnimationPanel(parent, planet);
		add(planetAnimationPanel);
		add(getDetailsPanel());
        add(buttonPanel);
        
        update();
     }
       
    private JPanel getDetailsPanel() throws ApplicationError {
        ComponentTable detailsPanel = new ComponentTable("HV", 3,1, 0, 4,3);
        detailsPanel.setOpaque(false);

        planetIconPanel = new PlanetIconPanel(true, maxPop, pop, fact, eco, prod, res, money);
        planetIconPanel.setOpaque(false);
        
        detailsPanel.add(titleLabel);
        detailsPanel.add(planetIconPanel, "HVSP");
        detailsPanel.add(projectLabel);
        
        return detailsPanel;
     }        
    
    public void updateAll() throws ApplicationError, ValidationError {
    	if (parent instanceof PlanetsWindow) {
    		PlanetsWindow pw = (PlanetsWindow) parent;
    		pw.setup();
    	} else {
    		update();
    	}
    }
    
     public void update() throws ApplicationError, ValidationError {
    	 String titleText = planet.getName() + " [" + planet.getPlanetTypeName() + " " +
    	 					Formatter.whole(planet.getSize())+
    	 					"*"+Formatter.fraction(planet.getTeraforming());
         if (planet.getProductionBonus() != 0) {
        	 titleText += " "+planet.getProductionBonusName();
         }
         if (planet.getResearchBonus() != 0) {
        	 titleText += " "+planet.getResearchBonusName();
         }
         if (planet.getMoneyBonus() > 0) {
        	 titleText += " "+planet.getMoneyBonusName();
         }
         if (planet.getResource() != null) {
        	 titleText += " "+planet.getResource().getTitle();
         }
         if (planet.getReproductionBonus() != 0) {
        	 titleText += " "+planet.getFertilityBonusName();
         }
         if (planet.hasNatives()) {
        	 titleText += " "+L.NATIVES;
         }
         titleText += "]";
    	 titleLabel.setText(titleText);
    	 
    	 planetIconPanel.update(planet, windowOwner);

         boolean viewable = planet.isViewable();
         if (viewable) {
             projectLabel.setText(L.PROJECT_+" "+planet.getProjectDescription());
         } else {
             projectLabel.setText(L.PROJECT_UNKNOWN);
         }
     }

	public void viewPlanet() {
		viewButton.actionPerformed(null);
	}
	
	public void handleTick() throws ApplicationError {
		if (planetAnimationPanel != null) {
			planetAnimationPanel.handleTick();
		}
	}

 }
