/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import java.util.ArrayList;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.TradeRouteDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.window.common.MotInternalWindow;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

public class PlanetTradeRoutesPanel extends ComponentTable {

	private PlanetTradeRoutePanel totals;
	private TradeRouteList list = new TradeRouteList();
	private MotInternalWindow parent;

    public PlanetTradeRoutesPanel(MotInternalWindow parent) throws ApplicationError {
        super("-", 2,1, 5,3,3);
		this.parent = parent;
		this.totals = new PlanetTradeRoutePanel(parent);
       
		add(new MotBorder(totals, "Total Trade Route Output"), "HV");
		add(new MotBorder(list.getScrollPane(), "Trade Output per Trade Route"), "HVSP");
    }


    public void selectPlanet(PlanetDelegate planet, PlayerDelegate owner) throws ApplicationError, ValidationError {
        if (planet != null && planet.isViewable()) {
        	TradeRouteDelegate[] tradeRoutes = DeligateCacheManager.getTradeRoutes();
        	if (tradeRoutes == null) {
        		list.clearContent();
        		totals.selectTradeRoute(planet.getSolarSystem(), 0, 0, 0);
        	} else {
        		ArrayList<TradeRouteDelegate> content = null;
        		for (int i = 0; i < tradeRoutes.length; i++) {
					if (tradeRoutes[i].contains(planet.getSolarSystemName())) {
						if (content == null) content = new ArrayList();
						content.add(tradeRoutes[i]);
					}
				}
        		if (content == null) {
        			list.clearContent();
        		} else {
        			list.clearContent();
        			list.setQtyRows(content.size());
            		for (int i = 0; i < content.size(); i++) {
            			TradeRouteDelegate  tradeRoute = content.get(i);
						PlanetTradeRoutePanel c = new PlanetTradeRoutePanel(parent);
						SolarSystemDelegate otherSolarSystem = (planet.getSolarSystem() == tradeRoute.getSolarSystem1()) ? tradeRoute.getSolarSystem2() : tradeRoute.getSolarSystem1();
						c.selectTradeRoute(planet, otherSolarSystem, tradeRoute, parent.getOwner());
						list.add(c);
            		}
        		}
        		SolarSystemDelegate solarSystem = planet.getSolarSystem();
				totals.selectTradeRoute(planet.getSolarSystem(), getTotalProduction(solarSystem, tradeRoutes), getTotalResearch(solarSystem, tradeRoutes), getTotalMoney(solarSystem, tradeRoutes));
        	}
        } else {
        	if (planet != null) {
        		TradeRouteDelegate[] tradeRoutes = DeligateCacheManager.getTradeRoutes();
            	SolarSystemDelegate solarSystem = planet.getSolarSystem();
    			totals.selectTradeRoute(planet.getSolarSystem(), getTotalProduction(solarSystem, tradeRoutes), getTotalResearch(solarSystem, tradeRoutes), getTotalMoney(solarSystem, tradeRoutes));
        	} else {
        		totals.selectTradeRoute(null, 0, 0, 0);
        	}
            list.clearContent();
        }
    }

	private int getTotalProduction(SolarSystemDelegate ss, TradeRouteDelegate[] tradeRoutes) throws ApplicationError {
		PlanetDelegate[] planets = ss.getPlanets();
		if (planets == null || tradeRoutes == null) return 0;
		int r = 0;
		for (int j = 0; j < planets.length; j++) {
			PlanetDelegate planet = planets[j];
			for (int i = 0; i < tradeRoutes.length; i++) {
				r += (int) (tradeRoutes[i].getIndustryOutput(planet.getName()) * planet.getPercentProduction());
			}
		}
		return r;
	}

	private int getTotalResearch(SolarSystemDelegate ss, TradeRouteDelegate[] tradeRoutes) throws ApplicationError {
		PlanetDelegate[] planets = ss.getPlanets();
		if (planets == null || tradeRoutes == null) return 0;
		int r = 0;
		for (int j = 0; j < planets.length; j++) {
			PlanetDelegate planet = planets[j];
			for (int i = 0; i < tradeRoutes.length; i++) {
				r +=  (int) (tradeRoutes[i].getResearchOutput(planet.getName()) * planet.getPercentResearch());
			}			
		}
		return r;
	}

	private int getTotalMoney(SolarSystemDelegate ss, TradeRouteDelegate[] tradeRoutes) throws ApplicationError {
		PlanetDelegate[] planets = ss.getPlanets();
		if (planets == null || tradeRoutes == null) return 0;
		int r = 0;
		for (int j = 0; j < planets.length; j++) {
			PlanetDelegate planet = planets[j];
			for (int i = 0; i < tradeRoutes.length; i++) {
				r +=  (int) (tradeRoutes[i].getIndustryOutput(planet.getName()) * planet.getPercentMoney());
			}
		}
		return r;
	}

	public void handleTick() throws ApplicationError {
    }
}
