/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlanetListDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.buttons.ActionObserver;
import mot.client.swing.buttons.GenericAction;
import mot.client.swing.combos.PlanetListOrderCombo;
import mot.client.swing.common.ActionClose;
import mot.client.swing.common.MotCheckBoxSimple;
import mot.client.swing.lists.FakePanelList;
import mot.client.swing.lists.FakePanelListClickListener;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.galaxy.GalaxyWindowRadarCarrotFactory;
import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class PlanetsWindow extends MotInternalWindow implements ActionObserver, FakePanelListClickListener {


    private FakePanelList planetsList = new FakePanelList(this, true);
    private PlanetListOrderCombo orderCombo = new PlanetListOrderCombo();
    
    private boolean all = false;
	private MotCheckBoxSimple habitableCheckBox;
    
    public PlanetsWindow(UserPlayerDelegate owner) throws ApplicationError, ValidationError {
        this((MotInternalWindow)null, owner);
    }

    public PlanetsWindow(MotInternalWindow parent, UserPlayerDelegate owner) throws ApplicationError, ValidationError {
        super(owner, parent, L.PLANETS_WINDOW, null);
        this.owner = owner;

        init();
        setup();
        show();
    }

    public PlanetsWindow(MotInternalWindow parent, UserPlayerDelegate owner, boolean all) throws ApplicationError, ValidationError {
        super(owner, parent, L.PLANETS_WINDOW, null);
        this.owner = owner;
        this.all = all;

        init();
        setup();
        show();
    }
    
    private PlanetDelegate [] getPlanetsToDisplay() throws ApplicationError {
    	if (all) {
    		if (habitableCheckBox.isSelected()) {
    			return owner.getVisibleHabitablePlanets();
    		} else {
        		return owner.getVisiblePlanets();
    		}
    	} else {
    		if (habitableCheckBox.isSelected()) {
    			return owner.getNonMiningColonies();
    		} else {
    			return DeligateCacheManager.getPlanets(owner);
    		}
    	}
    }
    
    private PlanetDelegate [] getPlanets() throws ApplicationError {
    	PlanetDelegate[] pda = getPlanetsToDisplay();
        PlanetListDelegate pl =  new PlanetListDelegate(pda);
        if (orderCombo.isByName()) {
            return pl.orderByName();
        } else if (orderCombo.isByPopulation()) {
            return pl.orderByPopulation();
        } else if (orderCombo.isByProductionOutput()) {
            return pl.orderByProductionOutput();
        } else if (orderCombo.isByTerraform()) {
            return pl.orderByTerraform();
        } else {
            return pl.getPlanets();
        }
    }

    public void setup() throws ApplicationError, ValidationError {
        PlanetDelegate[] planets = getPlanets();
        if (planets != null && planets.length > 0) {
        	JPanel [] planetPanels = new JPanel[planets.length];
            for (int i=0; i<planets.length; i++) {
            	planetPanels[i] = new PlanetPanel(this, this, planets[i], owner);
            	Location loc = planets[i].getSolarSystem().getLocation();
				GalaxyWindowRadarCarrotFactory.create(loc, planetPanels[i]);
            }
            planetsList.setContent(planetPanels);
        } else {
        	planetsList.clear();
        }
    }

    private void init() throws ApplicationError {
        JScrollPane planetsScrollPane = planetsList.getScrollPane();

        // actionNotification() will be called when this combo changes
        orderCombo.addActionListener(new GenericAction(this));

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(new ActionClose(this), null);
        buttonPanel.add(orderCombo, null);
        habitableCheckBox = new MotCheckBoxSimple("Habitable", null);
        habitableCheckBox.setSelected(!all);
        habitableCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					setup();
					getContentPane().validate();
					getContentPane().repaint();
					// I have no idea why we need to do this!
					setup();
					getContentPane().validate();
					getContentPane().repaint();
				} catch (Exception e1) {
					Logger.logError("ChangeButton Failed", e1);
				}
			}
		});
        buttonPanel.add(habitableCheckBox, null);

    	getContentPane().add(addTitleBar(planetsScrollPane), null);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
    }

	public void actionNotification() throws ValidationError, ApplicationError {
		setup();
	}

	public void notifyEndTurnButtonPressed() throws ApplicationError, ValidationError {
		setup();
	}

	public void click(JPanel p) {
		if (p instanceof PlanetPanel) {
			PlanetPanel pp = (PlanetPanel) p;
			pp.viewPlanet();
		}
	}
	
	public void handleTick() throws ApplicationError {
		Component[] listContents = planetsList.getComponents();
		if (listContents != null) {
			for (int i = 0; i < listContents.length; i++) {				
				if (listContents[i] instanceof PlanetPanel) {
					PlanetPanel pp = (PlanetPanel) listContents[i];
					pp.handleTick();
				}
			}
		}
	}

}