/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.swing.config.ImageConfigFactory;
import mot.client.swing.panels.ImagePanel;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.craft.CraftDesignPicturePanel;
import mot.common.designs.AsteroidDesign;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.ConstructPlanetDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ProjectDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.TransportDesign;
import mot.common.designs.WeaponDesign;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Shield;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

/**
 * This is a panel that displays a planetary project. It contains an image 
 * of the project and a description.
 * 
 * DO not confuse this panel with a ProjectPanel that displays this image 
 * inside a tab control along with a technology table of the details
 */
public class ProjectImagePanel extends JPanel {

	private static String NO_PROJECT_IMAGE = null;
	private MotInternalWindow parent; 
	
    public ProjectImagePanel(MotInternalWindow parent) throws ApplicationError {
        this.parent = parent;
        
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        if (NO_PROJECT_IMAGE == null) {
        	NO_PROJECT_IMAGE = ImageConfigFactory.getInstance().getImageConfig("NoProject").getImage();
        }
        
        add(new ImagePanel(NO_PROJECT_IMAGE));
    }
    
    public void setProject(PlanetDelegate planet, boolean viewable) throws ApplicationError, ValidationError {
    	if (planet != null && viewable) {
    		PlayerDelegate owner = planet.getOwner();
    		//if (owner instanceof UserPlayerDelegate) { 
    			setProject(owner, planet.getOrigin(), planet.getProjectDesign());
    		//} else {
    		//	throw new ValidationError("This does not belong to you");
    		//}
    	} else {
    		setContents(null);
    	}
    }

    private void setProject(PlayerDelegate owner, PlayerDelegate origin, ProjectDesign project) throws ApplicationError {
        if (project == null) {
            setContents(null);
        } else if (project instanceof TransportDesign) {
            setProject((TransportDesign) project);
        } else if (project instanceof CraftDesign) {
            setProject(owner, origin, (CraftDesign) project);
        } else if (project instanceof CraftUpgradeDesign) {
            setProject(owner, origin, (CraftUpgradeDesign) project);
        } else if (project instanceof WeaponDesign) {
            setProject(((WeaponDesign)project).getWeapon());
        } else if (project instanceof ShieldDesign) {
            setProject(((ShieldDesign)project).getShield());
        } else if (project instanceof ScannerDesign) {
            setProject(((ScannerDesign)project).getScanner());
        } else if (project instanceof BattleStationDesign) {
            setProject(((BattleStationDesign)project).getBattleStation());
        } else if (project instanceof SpaceStationDesign) {
            setProject(((SpaceStationDesign)project).getSpaceStation());
        } else if (project instanceof OutpostDesign) {
            setProject(((OutpostDesign)project).getOutpost());
        } else if (project instanceof ConstructPlanetDesign) {
            setProject(((ConstructPlanetDesign)project));
        } else if (project instanceof AsteroidDesign) {
            setProject(((AsteroidDesign)project));
        } else if (project instanceof TransportDesign) {
            setProject((TransportDesign) project);
    	} else {
    		setContents(null);
    	}
    }

    public void setProject(TransportDesign project) {
    	ProjectTransportOverviewPanel overview = new ProjectTransportOverviewPanel(project);
        setContents(overview);
    }

    public void setProject(PlayerDelegate owner, PlayerDelegate origin, CraftDesign project) throws ApplicationError {
    	CraftDesignPicturePanel overview = new CraftDesignPicturePanel(owner, origin, project, null, null, null, null, null, true, true, true);
        setContents(overview.getScrollPane());
    }

    public void setProject(PlayerDelegate owner, PlayerDelegate origin, CraftUpgradeDesign project) throws ApplicationError {
    	CraftDesignPicturePanel overview = new CraftDesignPicturePanel(owner, origin, project.getDesign(), null, null, null, null, null, true, true, true);
        setContents(overview.getScrollPane());
    }

    public void setProject(Weapon project) {
    	ProjectWeaponOverviewPanel overview = new ProjectWeaponOverviewPanel(project);
        setContents( overview);
    }

    public void setProject(Shield project) {
    	ProjectShieldOverviewPanel overview = new ProjectShieldOverviewPanel(project);
        setContents(overview);
    }

    public void setProject(Scanner project) {
    	ProjectScannerOverviewPanel overview = new ProjectScannerOverviewPanel(project);
        setContents(overview);
    }

    public void setProject(Hull project) {
    	ProjectBattleStationOverviewPanel overview = new ProjectBattleStationOverviewPanel(project);
        setContents(overview);
    }

    public void setProject(ConstructPlanetDesign project) throws ApplicationError {
    	ProjectConstructPlanetOverviewPanel overview = new ProjectConstructPlanetOverviewPanel(project.getPlanetConstructionTech());
        setContents(overview);
    }

    public void setProject(AsteroidDesign project) throws ApplicationError {
    	ProjectConstructAsteroidsOverviewPanel overview = new ProjectConstructAsteroidsOverviewPanel(project.getAsteroids());
        setContents(overview);
    }
    
    private void setContents(JComponent contents) {
        removeAll();
        if (contents == null) {
            add(new ImagePanel(NO_PROJECT_IMAGE));
        } else {
            add(contents);
        }
        validate();
        repaint();
    }	
}
