/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.window.craft.CraftDesignPanel;
import mot.common.bean.SalvageList;
import mot.common.designs.AsteroidDesign;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.ConstructPlanetDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ProjectDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.TransportDesign;
import mot.common.designs.WeaponDesign;
import mot.common.technology.AsteroidFieldConstruction;
import mot.common.technology.Hull;
import mot.common.technology.Module;
import mot.common.technology.PlanetConstruction;
import mot.common.technology.Scanner;
import mot.common.technology.Shield;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

/**
 * This is a panel that displays a planetary project. This panel is used in windows
 * to select a project on a planet. It contains a two tabs, one contains an image 
 * of the project and a description and the other containing a table of detailed 
 * information about the project being built.
 * 
 * DO not confuse this panel with a ProjectImagePanel that displays the image of 
 * the project without the tab control allowing the user to navigate to the technology 
 * table of details.
 */
public class ProjectPanel extends JPanel {

    private TechnologyBundle[] usableSalvage;

	public ProjectPanel() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    }

    public void setProject(PlanetDelegate planet, UserPlayerDelegate owner, ProjectDesign project) throws ApplicationError, ValidationError {
        if (project instanceof TransportDesign) {
            setProject((TransportDesign) project);
        } else if (project instanceof CraftDesign) {
            setProject(planet, owner, (CraftDesign) project);
        } else if (project instanceof CraftUpgradeDesign) {
            setProject(planet, owner, (CraftUpgradeDesign) project);
        } else if (project instanceof WeaponDesign) {
            setProject(((WeaponDesign)project).getWeapon());
        } else if (project instanceof ShieldDesign) {
            setProject(((ShieldDesign)project).getShield());
        } else if (project instanceof ScannerDesign) {
            setProject(((ScannerDesign)project).getScanner());
        } else if (project instanceof BattleStationDesign) {
            setProject(((BattleStationDesign)project).getBattleStation());
        } else if (project instanceof SpaceStationDesign) {
            setProject(((SpaceStationDesign)project).getSpaceStation());
        } else if (project instanceof OutpostDesign) {
            setProject(((OutpostDesign)project).getOutpost());
        } else if (project instanceof ConstructPlanetDesign) {
            setProject(((ConstructPlanetDesign)project).getPlanetConstructionTech());
        } else if (project instanceof AsteroidDesign) {
            setProject(((AsteroidDesign)project).getAsteroids());
        } else {
        	setContents(new ProjectNonePanel(), false);
        }
    }

	public void setProject(TransportDesign project) {
		usableSalvage = null;
        setContents(new ProjectTransportPanel(project), false);
    }

    public void setProject(PlanetDelegate planet, UserPlayerDelegate owner, CraftDesign project) throws ApplicationError, ValidationError {
    	PlayerDelegate origin = planet.getOrigin();
        CraftDesignPanel cdp = new CraftDesignPanel(owner, owner, origin, project, null, null, null, null, null);
        SalvageList salvage = owner.getSalvage();
        usableSalvage = project.getUsableSalvage(planet, salvage);
		if (usableSalvage != null) {
			cdp.setSalvage(usableSalvage);
		}
		setContents(cdp, false);
    }

    public void setProject(PlanetDelegate planet, UserPlayerDelegate owner, CraftUpgradeDesign project) throws ApplicationError, ValidationError {
		usableSalvage = null;
    	PlayerDelegate origin = planet.getOrigin();
        setContents(new CraftDesignPanel(owner, owner, origin, project.getDesign(), null, null, null, null, null), false);
    }

    public void setProject(Weapon project) throws ApplicationError {
		usableSalvage = null;
        setContents(new ProjectWeaponPanel(project), false);
    }

    public void setProject(Shield project) throws ApplicationError {
		usableSalvage = null;
        setContents(new ProjectShieldPanel(project), false);
    }

    public void setProject(Scanner project) throws ApplicationError {
		usableSalvage = null;
        setContents(new ProjectScannerPanel(project), false);
    }

    public void setProject(Hull project) throws ApplicationError {
		usableSalvage = null;
        setContents(new ProjectBattleStationPanel(project), false);
    }

    public void setProject(Module project) throws ApplicationError {
		usableSalvage = null;
        setContents(new ProjectModulePanel(project), false);
    }

    public void setProject(PlanetConstruction project) {
		usableSalvage = null;
        setContents(new ProjectConstructPlanetPanel(project), false);
    }
    
    public void setProject(AsteroidFieldConstruction asteroids) {
		usableSalvage = null;
        setContents(new ProjectConstructAsteroidsPanel(asteroids), false);
	}
    
    private void setContents(JComponent contents, boolean inScrollPane) {
        removeAll();
        if (inScrollPane) {
            add(new JScrollPane(contents));
        } else {
            add(contents);
        }
        validate();
        repaint();
    }

	public TechnologyBundle[] getUsableSalvage() {
		return usableSalvage;
	}

}