/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import java.awt.Dimension;
import java.util.ArrayList;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.CompletedCallback;
import mot.client.proxy.ProxyStore;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.buttons.MotLabelButton;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.MotLabel;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.craft.CraftDesignsWindow;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.solarsystem.actions.ProjectChangeListener;
import mot.client.swing.window.solarsystem.actions.SelectProjectCraft;
import mot.client.swing.window.solarsystem.actions.SelectProjectOther;
import mot.client.swing.window.solarsystem.actions.SelectProjectScanner;
import mot.client.swing.window.solarsystem.actions.SelectProjectShield;
import mot.client.swing.window.solarsystem.actions.SelectProjectStation;
import mot.client.swing.window.solarsystem.actions.SelectProjectTransport;
import mot.client.swing.window.solarsystem.actions.SelectProjectWeapon;
import mot.common.designs.ConstructPlanetDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.designs.Orbital;
import mot.common.designs.ProjectDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.WeaponDesign;
import mot.common.lang.L;
import mot.common.technology.AsteroidFieldConstruction;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Hull;
import mot.common.technology.Hulls;
import mot.common.technology.MiscTechnologies;
import mot.common.technology.MiscTechnology;
import mot.common.technology.PlanetConstruction;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

/**
 * This window is used to select new projects for planets.
 */
public class SelectProjectWindow extends MotInternalWindow implements ProjectChangeListener {
    

	private static final int HEADER_FONT_SIZE_INCREASE = 5;

	private PlanetDelegate planet;
	
    public ProjectDesign project;
    private int repeat = 0;
    private String transportName;

    private JLabel buildCostLabel = new MotLabel("", HEADER_FONT_SIZE_INCREASE, JLabel.LEFT, true);
    private JLabel turnsLabel = new MotLabel("", HEADER_FONT_SIZE_INCREASE, JLabel.LEFT, true);
    private JLabel repeatLabel = new MotLabel("", HEADER_FONT_SIZE_INCREASE, JLabel.LEFT, true);

    private ProjectList list = new ProjectList();
    private ProjectPanel detailsPanel;
    
    private PlanetDefensePanel defenceList = new PlanetDefensePanel();

	private JPanel buttonPanel;

    
    //Player owner;
    public SelectProjectWindow(MotInternalWindow parent, PlanetDelegate planet) throws ApplicationError, ValidationError {
        //super("Select Planetary Project");
        super((UserPlayerDelegate)planet.getOwner(), parent, L.SELECT_PROJECT_ON_+((planet == null)?"":planet.getName()), null);

        this.planet = planet;
        this.project = planet.getProjectDesign();
                
        init();
        
        defenceList.selectPlanet(planet, owner);
        
        if (owner == null) {
            throw new ApplicationError("SelectProjectWindow.SelectProjectWindow() Owner is null for planet "+planet.getName());
        }

        updateProjectDisplay();
        
        show();
        waitUntilClosed();
    }

    private void init() throws ApplicationError {
        defenceList.setPreferredSize(new Dimension(100, 150));

        ComponentTable left = new ComponentTable("H", 3, 1, 3, 3, 3);
        ButtonGroup projectTypeButtonGroup = new ButtonGroup();
        //JScrollPane listScrollPane1 = new JScrollPane(list, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        left.add(new MotBorder(defenceList, L.DEFENSE));
        left.add(new MotLabel(L.SELECT_A_PROJECT, HEADER_FONT_SIZE_INCREASE, JLabel.LEFT, true));
        left.add(list.getScrollPane(), "HVS");
        left.setPreferredSize(new Dimension(100, 2000));
        left.setMaximumSize(new Dimension(100, 2000));
        
        updateProjectList();

        ComponentTable cost = new ComponentTable("R,R,R,R,R,R", 1, 6, 2, 5, 0);
        cost.add(new MotLabel(L.BUILD_COST, HEADER_FONT_SIZE_INCREASE, JLabel.LEFT, true));
        cost.add(buildCostLabel, "HP");
        cost.add(new MotLabel(L.TURNS_TO_BUILD, HEADER_FONT_SIZE_INCREASE, JLabel.LEFT, true));
        cost.add(turnsLabel, "HP");
        cost.add(new MotLabel(L.REPEAT, HEADER_FONT_SIZE_INCREASE, JLabel.LEFT, true));
        cost.add(repeatLabel, "HP");
        
        detailsPanel = new ProjectPanel();

        buttonPanel = new JPanel();        
        buttonPanel.add(new MotButton(L.CANCEL) {
			protected void _doAction() throws ApplicationError, ValidationError {
				closeProjectWindow();
			}
        });
        buttonPanel.add(new MotButton(L.BUILD){
        	{ setToolTipText(L.TT_BUTTON_BUILD); }
			protected void _doAction() throws ApplicationError, ValidationError {
				ProjectDesign existingProject = planet.getProjectDesign();
				if (existingProject == null || !existingProject.same(project)) { 
					ProxyStore.getInstance().setProjectCmd(planet, project, repeat, false, false, new CloseWindowCallback(), this);
				} else {
					closeProjectWindow();
				}
			}
        });
        buttonPanel.add(new MotButton(L.BUILD_WITH_SALVAGE){
        	{ setToolTipText(L.TT_BUTTON_BUILD); }
			protected void _doAction() throws ApplicationError, ValidationError {
				ProjectDesign existingProject = planet.getProjectDesign();
				int costToBuild = getCostToBuildUsingSalvage(project);
				if (existingProject == null || !existingProject.same(project) || planet.getCostToBuild() > costToBuild) { 
					ProxyStore.getInstance().setProjectCmd(planet, project, repeat, false, true, new CloseWindowCallback(), this);
				} else {
					closeProjectWindow();
				}
			}
        });
        buttonPanel.add(new MotButton(L.BUILD_ON_IDLE){
        	{ setToolTipText(L.TT_BUTTON_SELECT_ON_IDLE); }
			protected void _doAction() throws ApplicationError, ValidationError {
	            ProxyStore.getInstance().setProjectCmd(planet, project, repeat, true, false, new CloseWindowCallback(), this);
			}
        });
        buttonPanel.add(new MotButton(L.REPEAT){
        	{ setToolTipText(L.TT_BUTTON_REPEAT); }
			protected void _doAction() throws ApplicationError, ValidationError {
	            if (project != null && (project instanceof WeaponDesign || project instanceof CraftDesign || project instanceof SpaceStationDesign)) {
	                repeat++;
	                repeatLabel.setText(""+repeat);
	            }
			}
        });
        buttonPanel.add(new MotButton(L.NO_PROJECT){
        	{ setToolTipText(L.TT_BUTTON_NO_PROJECT); }
			protected void _doAction() throws ApplicationError, ValidationError {
	            ProxyStore.getInstance().setPlanetWantsNoProjectCmd(planet, true, new CloseWindowCallback(), this);
			}
        });
        buttonPanel.add(new MotButton(L.CRAFT_DESIGN){
			protected void _doAction() throws ApplicationError, ValidationError {
	            SelectProjectWindow parent = getSelectProjectWindow();
				CraftDesignsWindow win = new CraftDesignsWindow(parent, owner, planet.getOrigin());
	            win.waitUntilClosed();
	            try {
	            	CraftDesign cd = win.getSelectedCraftDesign();
	            	if (cd != null) {
	            		setProject(cd);
	            		updateProjectDisplay();
	            	}
	            	updateProjectList();
	                updateProjectDisplay();
	            } catch (ValidationError ex) { }
			}
        });
        
        ComponentTable right = new ComponentTable("HV", 3, 1, 3, 0, 0);
        right.add(detailsPanel, "HVS");
        right.add(new MotBorder(cost));
        right.add(buttonPanel);

        ComponentTable main = new ComponentTable("HV,HV", 1, 2, 0, 0, 5);
        main.setGridWidths("100,450");
        main.add(left);
        main.add(right);

        getContentPane().add(addTitleBar(main), null);
    }
    
	private void closeProjectWindow() {
		hide();
	}

	private SelectProjectWindow getSelectProjectWindow() {
		return this;
	}
	
    private int getTurnsToCompleteProject() {
        if (project == null) return 0;
        try {
            return planet.getTurnsToCompleteProject(project);
        } catch (ApplicationError e) {
            Logger.logError("",e);
            return -1;
        }
    }
    
    private int getTurnsToCompleteProject(int workToCompleteProject) {
        if (project == null) return 0;
        try {
            return planet.getTurnsToCompleteProject(workToCompleteProject);
        } catch (ApplicationError e) {
            Logger.logError("",e);
            return -1;
        }
    }

    private int getCostToBuildUsingSalvage(ProjectDesign design) throws ApplicationError {
		if (design != null && design instanceof CraftDesign) {
			return getCostToBuildUsingSalvage((CraftDesign)design);
		} else {
			return design.getCostToBuildOnPlanet();
		}
    }
    
    private int getCostToBuildUsingSalvage(CraftDesign cd) throws ApplicationError {
		TechnologyBundle[] usableSalvage = detailsPanel.getUsableSalvage();
		GeneralTechnologyList salvageTechs = new GeneralTechnologyList(usableSalvage);
		int costUsingSalvage = cd.getCostToBuildUsingExistingTechs(salvageTechs);
		return costUsingSalvage;
    }
    
    public void updateProjectDisplay() throws ApplicationError, ValidationError {
        if (project != null) {
            detailsPanel.setProject(planet, owner, project);
            detailsPanel.invalidate();
            detailsPanel.repaint();
            if (project != null) {
                int cost = project.getCostToBuildOnPlanet();
                int turns = getTurnsToCompleteProject();
				String costText = ""+cost;
				String turnsText = ""+turns;
                TechnologyBundle[] usableSalvage = detailsPanel.getUsableSalvage();
				if (usableSalvage != null && project instanceof CraftDesign) {
					CraftDesign cd = (CraftDesign) project;
					int costUsingSalvage = getCostToBuildUsingSalvage(cd);
					int turnsUsingSalvage = (int) Math.ceil((double)costUsingSalvage / planet.getProductionOutput());
					costText += "  ("+costUsingSalvage+" using salvage)";
					turnsText += "  ("+turnsUsingSalvage+" using salvage)";
				}
				turnsLabel.setText(turnsText);
				buildCostLabel.setText(""+costText);
            } else {
                turnsLabel.setText("");
                buildCostLabel.setText("");
            }
        } else {
            turnsLabel.setText("");
            buildCostLabel.setText("");
            repeatLabel.setText("");
            detailsPanel.setProject(planet, owner, (ProjectDesign)null);
        }
    }
    
    public void updateProjectList() throws ApplicationError {
		UserPlayerDelegate owner = MainWindow.getOwner();
		ArrayList<MotLabelButton> labels = new ArrayList<MotLabelButton>();
		
		// Add Craft Designs
        CraftDesigns craftsDesigns = owner.getCraftDesigns();
        if ( craftsDesigns != null && craftsDesigns.getQuantity() > 0 ) {
        	CraftDesign [] pa = craftsDesigns.getCraftDesigns();
        	if ( pa == null ) return;
        	for (int i=pa.length-1; i>=0; i--) {
        		labels.add(new SelectProjectCraft(this, pa[i]));
        	}
        	labels.add(new SelectProjectTransport(this));
        }

    	// Add Weapons
        Weapons weapons = owner.getWeapons();
        if (weapons != null) {
	        Technology[] currentWeapons = weapons.getPlanetWeapons(getPlanet().getWeapons());
	        for (int i=currentWeapons.length-1; i>=0; i--) {
	        	labels.add(new SelectProjectWeapon(this, (Weapon)currentWeapons[i]));
	        }
        }

    	// Add Shields
        Shields shields = owner.getShields();
        if (shields != null) {
    		Shields existingShields = planet.getShields();
			if (existingShields == null) {
	        	Shield bestShield = (Shield) shields.getMostAdvancedWithGroup(Shield.GROUP_SHIELD);
		    	if (bestShield != null) {
		    		labels.add(new SelectProjectShield(this, bestShield));
				}
	    	}
        }

    	// Add Scanners
        Scanners scanners = owner.getScanners();
        if (scanners != null) {        	
        	Scanners existingScanner = planet.getScanners();
        	if (existingScanner == null) {
        		Scanner bestScanner = (Scanner) scanners.getMostAdvancedWithGroup(Scanner.GROUP_SCANNER);
        		if (bestScanner != null) {
        			labels.add(new SelectProjectScanner(this, bestScanner));
        		}
        	}
        }

    	// Add Stations
        Hulls hulls = owner.getHulls();
        if (hulls != null) {        	
        	Orbital existingOrbital = planet.getOrbital();
        	if (existingOrbital == null) {
	        	Hull bs = hulls.getMostAdvancedBattleStationHull();
	        	if (bs != null) {
					labels.add(new SelectProjectStation(this, bs));
				}
        	}
        	Hull outpost = hulls.getMostAdvancedOutpostHull();
        	if (outpost != null) {
				labels.add(new SelectProjectStation(this, outpost));
			}
        	Hull spaceStation = hulls.getMostAdvancedSpaceStationHull();
        	if (spaceStation != null) {
				labels.add(new SelectProjectStation(this, spaceStation));
			}
        }

    	// Add Other
		MiscTechnologies discoveredOtherTechs = owner.getMiscTechnologies();
		if (discoveredOtherTechs != null) {
			MiscTechnology [] planetConstruction = Technologies.getMiscTechnologies().getMiscTechnology(PlanetConstruction.class);
			for (int i = 0; i < planetConstruction.length; i++) {
				if (discoveredOtherTechs.contains(planetConstruction[i]) && owner.getBoralium() >= ConstructPlanetDesign.BORALIUM_COST) {
					labels.add(new SelectProjectOther(this, planetConstruction[i]));
				}
			}
			MiscTechnology [] asteroidsConstruction = Technologies.getMiscTechnologies().getMiscTechnology(AsteroidFieldConstruction.class);
			for (int i = 0; i < asteroidsConstruction.length; i++) {
				if (discoveredOtherTechs.contains(asteroidsConstruction[i]) && !planet.hasAsteroids()) {
					labels.add(new SelectProjectOther(this, asteroidsConstruction[i]));
				}
			}
		}
			
			
        if (labels.size() > 0) {
        	MotLabelButton[] la = labels.toArray(new MotLabelButton[labels.size()]);
        	list.setContent(la, null);
        } else {
        	list.emptyContent();
        }

    }

	public ProjectDesign getProject() {
		return project;
	}

	public void setProject(ProjectDesign project) {
		this.project = project;
	}

	public PlanetDelegate getPlanet() {
		return planet;
	}

	public void setPlanet(PlanetDelegate planet) {
		this.planet = planet;
	}

	public int getRepeat() {
		return repeat;
	}

	public void setRepeat(int repeat) {
		this.repeat = repeat;
	}

	public ProjectList getProjectList() {
		return list;
	}

	public void setProjectList(ProjectList list) {
		this.list = list;
	}

	public void changeDesignsSelection(ProjectDesign newProject) throws ApplicationError, ValidationError {
        setProject(newProject);
        updateProjectDisplay();
	}

	private class CloseWindowCallback implements CompletedCallback {
		public void complete(Object result) throws ApplicationError, ValidationError {
			closeProjectWindow();
			MainWindow.getInstance().getGalaxyMapConfiguration().getGalaxyView().getStatusPanel().updateStatus(MainWindow.getOwner());
		}

	}
    
}