/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.solarsystem;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.proxy.CompletedCallback;
import mot.client.swing.buttons.PlanetSelectionListener;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotBorder;
import mot.client.swing.common.PadPanel;
import mot.client.swing.lists.PlanetList;
import mot.client.swing.lists.PlanetListOwner;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.common.MotMessageDialog;
import mot.client.swing.window.common.actions.UpdatableContainer;
import mot.client.swing.window.galaxy.GalaxyWindowRadarCarrotFactory;
import mot.client.swing.window.galaxy.GalaxyWindowView;
import mot.client.swing.window.galaxy.actions.EndTurnButtonListener;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.solarsystem.actions.SolarSystemActionClose;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.vo.PlanetCrisisVO;

public class SolarSystemWindow extends MotInternalWindow implements FocusListener, PlanetListOwner, ChangeListener, UpdatableContainer, EndTurnButtonListener  {

    //private static final Dimension SIZE = new Dimension(550, 400);

    private SolarSystemWindowState state;

    private PlanetList planetList;
    
    private JTabbedPane tabs;
    private PlanetOutputPanel planetOutput;
    //private PlanetDefensePanel planetDefense;
    private PlanetMiningStationPanel miningPanel;
    private PlanetBattleStationPanel battleStationPanel;
    private PlanetTradeRoutesPanel tradeRoutesPanel;
    private PlanetOtherPanel planetOtherPanel;

	private PlanetIconPanel planetIconPanel = new PlanetIconPanel();

    private PlanetDefensePanel defenceList = new PlanetDefensePanel();

	private boolean playSelectPlanetSound = true;
	
    /**
     * Displays a dialog box to view and edit Solar Systems
     * The planet passed in is preselected from the list of planets.
     */
    public SolarSystemWindow(MotInternalWindow parent, PlanetDelegate planet, UserPlayerDelegate user) throws ApplicationError, ValidationError {
        super(user, parent, L.SOLAR_SYSTEM_WINDOW, null);
        init(planet.getSolarSystem(), user, planet);
        show();
    }
    
    /**
     * Displays a dialog box to view and edit Solar Systems
     */
    public SolarSystemWindow(MotInternalWindow parent, SolarSystemDelegate solarSystem, UserPlayerDelegate user) throws ApplicationError, ValidationError {
        super(user, parent, L.SOLAR_SYSTEM_WINDOW, null);
        init(solarSystem, user, null);
        show();
    }

    /**
     * Displays a dialog box to view and edit Solar Systems
     */
    private void init(SolarSystemDelegate solarSystem, UserPlayerDelegate user, PlanetDelegate planetToSelect) throws ApplicationError, ValidationError {
        PlanetDelegate [] planets = solarSystem.getPlanets();
        PlanetSelectionListener selectPlanetAction = new PlanetSelectionListener() {
			public void select(PlanetDelegate planet) throws ApplicationError, ValidationError {
		        if (planet == null) return;
	            selectPlanet(planet);
			}
        };
        
        planetList = new PlanetList(this, planets, selectPlanetAction);
        state = new SolarSystemWindowState(solarSystem, planetToSelect);
        
        // Must make sure we are using the *same* planet delegate object otherwise the model will not find it!
//        if (planetToSelect != null) {
//            for (int i=0; i<planets.length; i++) {
//                if (Comp.same(planets[i].getName(), planetToSelect.getName())) {
//                    planetToSelect = planets[i];
//                }
//            }
//        }
        
        
        planetOutput = new PlanetOutputPanel(this, this, planetIconPanel);
        //planetDefense = new PlanetDefensePanel(this);
        miningPanel = new PlanetMiningStationPanel(this);
        battleStationPanel = new PlanetBattleStationPanel(this);
        tradeRoutesPanel = new PlanetTradeRoutesPanel(this);
        planetOtherPanel = new PlanetOtherPanel(this, state);
        tabs = new JTabbedPane();
        tabs.addTab(L.OUTPUT, planetOutput);
        tabs.addTab(L.MINING, miningPanel);
        tabs.addTab(L.BATTLE_STATION, battleStationPanel);
        tabs.addTab(L.TRADE_ROUTES, tradeRoutesPanel);
        tabs.addTab(L.OTHER,  new JScrollPane(planetOtherPanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
        
        ComponentTable topPanel = new ComponentTable("HVP,HV", 1,2, 0, 0,0);
        defenceList.setPreferredSize(new Dimension(200, 60));
        topPanel.add(new MotBorder(planetIconPanel, L.DETAILS));
        topPanel.add(new MotBorder(defenceList, L.DEFENSE));

        ComponentTable right = new ComponentTable("-", 2,1, 0,0,0);
        right.add(topPanel, "HV");
        right.add(tabs, "HVS");

        solarSystemPanel = new SolarSystemPanel(this, solarSystem, planetList);
        
        ComponentTable left = new ComponentTable("HVSP", 2,1, 0, 3,3);
		left.add(solarSystemPanel);
        left.add(new PadPanel(new SolarSystemActionClose(this, state), 20), "-");

        JPanel main = new JPanel();
        main.setLayout(new BoxLayout(main, BoxLayout.X_AXIS));
        //left.setSize(new Dimension(150,400));
        left.setMinimumSize(new Dimension(150,200));
        left.setMaximumSize(new Dimension(150,800));
        main.add(left);
        main.add(right);
        
        getContentPane().removeAll();
        getContentPane().add(addTitleBar(main));

        if (planetToSelect == null && planets != null && planets.length > 0) {
            planetToSelect = planets[0];
        }
        
        if (planetToSelect == null) {
            selectPlanet(null);
        } else {
        	playSelectPlanetSound = false;
            planetList.setSelection(planetToSelect);
            playSelectPlanetSound = true;
        }
        
        setTitle();
        GalaxyWindowRadarCarrotFactory.create(solarSystem.getLocation(), this);
        
        planetList.selectPlanet(planetToSelect);

    }

    public void setTitle() throws ApplicationError {
        SolarSystemDelegate solarSystem = state.getSolarSystem();
        PlanetDelegate currentPlanet = state.getCurrentPlanet();
        String core = L.SOLAR_SYSTEM_+solarSystem.getName();
        if (currentPlanet != null) {
            core += " [ "+currentPlanet.getName();
            if (currentPlanet.getOwner() != null) {
                core += " "+currentPlanet.getOwnerName();
                if (!Comp.same(currentPlanet.getOriginName(), currentPlanet.getOwnerName())) {
                    core += L._GOVERNING_+currentPlanet.getOriginName();
                }
            }
            if (currentPlanet.getCrisis() != null) {
            	PlanetCrisisVO crisis = currentPlanet.getCrisis();
            	core += " "+crisis.getName();
            }
//            if (currentPlanet.isBlockaded()) {
//            	core += " "+L.BLOCKADED_CAPS;
//            }
//            if (currentPlanet.isDominated()) {
//            	core += " "+L.DOMINATED_CAPS;
//            }
            setTitle(core+" ] ");
        } else {
            setTitle(core);
        }
    }

    public void selectPlanet() throws ApplicationError, ValidationError {
        PlanetDelegate currentPlanet = state.getCurrentPlanet();
        selectPlanet(currentPlanet);
    }
    
    public void selectPlanet(PlanetDelegate planet) throws ApplicationError, ValidationError {
//        if (Logger.isDebug()) {
//            String name = (planet == null) ? "No Planet" : planet.getName();
//            Logger.logDebug("SolarSystemWindow.selectPlanet() name="+name);
//        }
        
        inSetPlanet = true;
        state.setCurrentPlanet(planet);
        
        planetIconPanel.update(planet, owner);
        planetOutput.selectPlanet(planet, owner);
        miningPanel.selectPlanet(planet, owner);
        battleStationPanel.selectPlanet(planet, owner);
        tradeRoutesPanel.selectPlanet(planet, owner);
        defenceList.selectPlanet(planet, owner);

        planetOtherPanel.selectPlanet(planet, owner);
    
        setTitle();
        
        inSetPlanet = false;
        
        updateGalaxyWindowStatus();
    }
    
    
    public void updateGalaxyWindowStatus() throws ApplicationError, ValidationError {
        if (MainWindow.getInstance().isGalaxyMapConfigurationActive()) {
        	MainWindow.getInstance().getGalaxyMapConfiguration().getGalaxyView().updateStatus(owner);
        }
    }

    /**
     * setPlanet changes values but we do not want to setPlanetPercents!
     */
    private boolean inSetPlanet = false;

	private SolarSystemPanel solarSystemPanel;

    /**
     * Implementation of ChangeLisneter - called when a slider changes position. 
     */
    public void stateChanged(ChangeEvent e) {
//        if (Logger.isDebug()) {
//            Logger.logDebug("SolarSystemWindow.stateChanged()");
//        }
        //JSlider source = (JSlider)e.getSource();
        PlanetDelegate currentPlanet = state.getCurrentPlanet();
        if (currentPlanet == null || inSetPlanet) {
            return;
        }
        if (planetOutput.inAutoPercentsCallback) return;
        try {
            planetOutput.setPlanetPercents(currentPlanet, owner, new ChangePlanetPercentsCallback());
        } catch (ValidationError ex) {
            MotMessageDialog.showError(this, ex);
        } catch (NumberFormatException ex) {
            MotMessageDialog.showError(this, ex);
        } catch (ApplicationError ex) {
            Logger.logError("", ex);
        }
    }
    
    public void focusGained(FocusEvent e) {
    }

    public void focusLost(FocusEvent e) {
//        if (Logger.isDebug()) {
//            Logger.logDebug("SolarSystemWindow.focusLost()");
//        }
        try {
            PlanetDelegate currentPlanet = state.getCurrentPlanet();
            planetOutput.setPlanetPercents(currentPlanet, owner, new ChangePlanetPercentsCallback());
        } catch (ValidationError ex) {
            MotMessageDialog.showError(this, ex);
        } catch (NumberFormatException ex) {
            MotMessageDialog.showError(this, ex);
        } catch (ApplicationError ex) {
            Logger.logError("", ex);
        }
    }

	public SolarSystemWindowState getState() {
		return state;
	}

	public PlanetOutputPanel getPlanetOutput() {
		return planetOutput;
	}

	public PlanetMiningStationPanel getMiningPanel() {
		return miningPanel;
	}

	public PlanetOtherPanel getPlanetOtherPanel() {
		return planetOtherPanel;
	}

	public void notifyEndTurnButtonPressed() throws ApplicationError, ValidationError {
		setTitle();
		selectPlanet();
	}
  
	protected boolean hasNavigationTitle() { return true; }
    
	/**
	 * Move to previous Solar System
	 */
	public void navigateLeft() throws ApplicationError, ValidationError {
		SolarSystemDelegate[] sos = owner.getSolarSystemsOwnedBy();
		if (sos == null) return;
		int index = Comp.findIndexInArray(state.getSolarSystem(), sos);
		if (index > 0) {
			navigateTo(sos[index-1]);
		} else if (index == 0 && sos.length > 1) {
			navigateTo(sos[sos.length-1]);
		}
	}


	/**
	 * Move to next Solar System
	 */
	public void navigateRight() throws ApplicationError, ValidationError {
		SolarSystemDelegate[] sos = owner.getSolarSystemsOwnedBy();
		if (sos == null) return;
		int index = Comp.findIndexInArray(state.getSolarSystem(), sos);
		if (index < sos.length-1) {
			navigateTo(sos[index+1]);
		} else if (index == sos.length-1 && sos.length > 1) {
			navigateTo(sos[0]);
		}
	}
	
	private void navigateTo(SolarSystemDelegate so) throws ApplicationError, ValidationError {
		PlanetDelegate[] pa = so.getPlanets();
		if (pa != null) {
			for (int i = 0; i < pa.length; i++) {
				if (pa[i].isOwnedBy(owner) && !pa[i].isMiningColony()) {
					init(so, owner, pa[i]);
					return;
				}
			}
		}
		init(so, owner, null);
	}

	public void next() throws ApplicationError, ValidationError {
		navigateRight();
	}
    
	public void previous() throws ApplicationError, ValidationError {
		navigateLeft();
	}

	@Override
	public void update() throws ApplicationError, ValidationError {
		if (state != null && state.getSolarSystem() != null) {
			init(state.getSolarSystem(), owner, null);
		}
	}
	
    private class ChangePlanetPercentsCallback implements CompletedCallback {
		@Override
		public void complete(Object result) throws ApplicationError, ValidationError {
            selectPlanet();
            GalaxyWindowView view = MainWindow.getInstance().getGalaxyMapConfiguration().getGalaxyView();
			view.getStatusPanel().updateStatus(getOwner());
			PlanetDelegate planet = state.getCurrentPlanet();
			if (planet != null) view.getContextPanel().updateSelectionState(planet);
		}
    }

    public void handleTick() throws ApplicationError {
    	planetList.handleTick();
    	Component selectedTab = tabs.getSelectedComponent();
		if (selectedTab == planetOutput) {
			planetOutput.handleTick();
    	} else if (selectedTab == miningPanel) {
    		miningPanel.handleTick();
    	} else if (selectedTab == planetOtherPanel) {
    		planetOtherPanel.handleTick();
    	}
		if (solarSystemPanel != null) {
			solarSystemPanel.handleTick();
		}
    }
    
}
