/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.swing.window.tech;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;

import mot.client.bean.PotentialResearchProject;
import mot.client.bean.ResearchProject;
import mot.client.deligates.ResearchProjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.buttons.MotClickTechnologyLabelAction;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.lists.FakeTechnologyLabelList;
import mot.client.swing.panels.TechnologyPicturePanel;
import mot.client.swing.renderers.CellRendererTechnologyLabel;
import mot.client.swing.tables.GeneralTechnologyTable;
import mot.client.swing.window.main.MainWindow;
import mot.common.bean.SalvageList;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;

/**
 * A Panel that displays a list of technologies down the left and the actual technology
 * selected from the list is displayed on the right in a picture (uses TechnologyPanel).
 * The user may use a tab to select a table view of the technology instead of the summary
 * view which is more graphical!
 */
public class SelectableTechnologyPanel extends ComponentTable {

    MotClickTechnologyLabelAction selectionListener = new MotClickTechnologyLabelAction() {
		protected void selectTechnology(Technology technology) throws ApplicationError, ValidationError {
			salvageList.setSelection(null);
			availableList.setSelection(technology);
			double chanceToDiscoverSalvage = MainWindow.getOwner().getChanceToDiscoverSalvage();
            updateTechnologyPanel(new PotentialResearchProject(technology));            
		}
	};
    CellRendererTechnologyLabel renderer = new CellRendererTechnologyLabel(false, true, false, new Size(20,20), true, 2, false, null, null, false, selectionListener, null, null);
    CellRendererTechnologyLabel rendererNonSelectable = new CellRendererTechnologyLabel(false, true, false, new Size(20,20), true, 2, false, Color.RED, null, false, null, null, null);
    
    MotClickTechnologyLabelAction salvageSelectionListener = new MotClickTechnologyLabelAction() {
		protected void selectTechnology(Technology technology) throws ApplicationError, ValidationError {
			availableList.setSelection(null);
			salvageList.setSelection(technology);
			double chanceToDiscoverSalvage = MainWindow.getOwner().getChanceToDiscoverSalvage();
            updateTechnologyPanel(new PotentialResearchProject(technology, chanceToDiscoverSalvage));            
		}
	};
    CellRendererTechnologyLabel rendererSalvage = new CellRendererTechnologyLabel(false, true, false, new Size(20,20), true, 2, false, Color.CYAN, null, false, salvageSelectionListener, null, null);
    
	private FakeTechnologyLabelList availableList;
    private JScrollPane availableScrollPane;

	private FakeTechnologyLabelList missedList;
    private JScrollPane missedScrollPane;

	private FakeTechnologyLabelList salvageList;
    private JScrollPane salvageScrollPane;

    private TechnologyPicturePanel technologyPanel;
    private JScrollPane detailsScrollPane;
    
	private SalvageList salvage;
	private TechnologyList availableTechnologies;
	private ResearchProjectDelegate currentResearchProject;

    public SelectableTechnologyPanel(UserPlayerDelegate owner, boolean showTurnsToComplete, Technology [] techs, SalvageList salvage, ResearchProjectDelegate currentResearchProject, Technology [] recentMissedTechs) throws ApplicationError {
        super("V,HPV", 1, 2, 0, 0, 0);
		this.currentResearchProject = currentResearchProject;
		this.availableTechnologies = new GeneralTechnologyList(techs);
		this.salvage = salvage;
        
    	availableList = new FakeTechnologyLabelList(renderer);
    	availableScrollPane = new JScrollPane(availableList, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    	missedList = new FakeTechnologyLabelList(rendererNonSelectable);
    	missedList.setMinimumSize(new Dimension(100,100));
    	missedScrollPane = new JScrollPane(missedList, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

   		salvageList = new FakeTechnologyLabelList(rendererSalvage);
   		salvageScrollPane = new JScrollPane(salvageList, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    	
        technologyPanel = new TechnologyPicturePanel(owner, showTurnsToComplete, true, false, false, null);
        detailsScrollPane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        
        availableScrollPane.setPreferredSize(new Dimension(210,400));
        PanelUtils.setTheBloodySize(salvageScrollPane, new Size(210,200));
        
        boolean hasRecentMissedTechs = (recentMissedTechs != null && recentMissedTechs.length > 0);
        boolean hasSalvage = (salvage != null && salvage.getQuantity() > 0);
        int qtyRows = 2;
        if(hasRecentMissedTechs) qtyRows += 2;
        if(hasSalvage) qtyRows += 2;
        
        ComponentTable listPanel = new ComponentTable("HSV", qtyRows, 1, 0, 0, 0);
        listPanel.add(new MotLabel(L.TECHNOLOGIES, 4, true), "R");
        listPanel.add(availableScrollPane);
        if (hasSalvage) {
            listPanel.add(new MotLabel(L.SALVAGE, 4, true, Color.CYAN), "R");
            listPanel.add(salvageScrollPane, "HV");
        }
        if (hasRecentMissedTechs) {
            listPanel.add(new MotLabel(L.RECENTLY_MISSED, 4, true), "R");
            listPanel.add(missedScrollPane, "HV");
        }
        listPanel.setMinimumSize(new Dimension(210, 200));
        
        JTabbedPane tabs = new JTabbedPane();
        tabs.addTab(L.OVERVIEW, technologyPanel);
        tabs.addTab(L.DETAILS, detailsScrollPane);
        
        add(listPanel);
        add(tabs);
        
        update(techs, salvage, currentResearchProject, recentMissedTechs);
    }
    
    /**
     * Makes this window display the technologies in the list techs. This should be 
     * the list of technologies discovered by this player.
     * @throws ApplicationError 
     */
    public void update(Technology [] techs, SalvageList salvage, ResearchProject currentResearchProject, Technology [] recentMissedTechs) throws ApplicationError {
    	if (currentResearchProject == null && techs != null) {
    		currentResearchProject = new PotentialResearchProject(techs[0]);
    	}
    	
    	Technology selection = (currentResearchProject == null) ? null : currentResearchProject.getTechnology();
    	availableList.setContent(techs, selection);
    	if (salvage != null) {
    		salvageList.setContent(salvage.getAllNoDuplicates(), selection);
    	}
    	if (recentMissedTechs != null) {
    		missedList.setContent(recentMissedTechs, null);
    	}
   		updateTechnologyPanel(currentResearchProject);
    }
    
    public void updateTechnologyPanel(ResearchProject currentResearchProject) throws ApplicationError {
        technologyPanel.update(currentResearchProject, false);
        
        detailsScrollPane.getViewport().removeAll();
        Technology selection = (currentResearchProject == null) ? null : currentResearchProject.getTechnology();
        if (selection != null) {
            JTable table = new GeneralTechnologyTable(selection);
            detailsScrollPane.getViewport().add(table);
        }
        detailsScrollPane.validate();
        
        validate();
        repaint();
    }

    private JScrollPane scrollPanel;
    public JScrollPane getScrollPane() {
        if (scrollPanel == null) {
            scrollPanel = new JScrollPane(this, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        }
        return scrollPanel;
    }

	public Technology getSelection() {
		Technology r = (Technology) availableList.getSelection();
		if (r == null) r = (Technology) salvageList.getSelection();
		return r;
	}
}