/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.client.tutorials;

import java.util.ArrayList;

import mot.client.actions.ActiveRectangle;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.buttons.MotImageButton;
import mot.client.swing.buttons.MotToggleButton;
import mot.client.swing.panels.BackgroundImagePanel;
import mot.client.swing.panels.ProgressBarPanel;
import mot.client.swing.window.common.MotInternalWindow;
import mot.client.swing.window.etc.actions.CloseOptionsWindowButton;
import mot.client.swing.window.galaxy.GalaxyWindowConfig;
import mot.client.swing.window.galaxy.GalaxyWindowEmpireButtonMenu;
import mot.client.swing.window.galaxy.GalaxyWindowFileButtonMenu;
import mot.client.swing.window.galaxy.actions.ActionButtonSettings;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowMessage;
import mot.client.swing.window.main.MainWindowMessageList;
import mot.client.swing.window.solarsystem.SelectPlanetWindow;
import mot.common.bean.GameConstants;
import mot.common.designs.ProjectDesign;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;

public abstract class TutorialSystem {

	private static Class [] exclude = {ActionButtonSettings.class, GalaxyWindowFileButtonMenu.class, GalaxyWindowEmpireButtonMenu.class, CloseOptionsWindowButton.class, TutorialButtonStop.class, TutorialButtonSkip.class};

	private static ArrayList<Class> available = null;
	private static boolean isActive = false;
	private static int dateOfLastTip = 0;
	
	public static void clear()  {
		available = null;
	}

	public static boolean isActive() throws ApplicationError {
		return isActive;
	}
	
	public static void setIsActive(boolean v) throws ApplicationError {
		isActive = v;
	}
	
	public static void setup() throws ApplicationError, ValidationError {
		//tutorial = ProxyStore.getInstance().getTutorial();
		MainWindow.getInstance().getGalaxyMapConfiguration().getGalaxyView().getContextPanel().showEmpireMenu();
		new A0IntroTip().display();
		available = new ArrayList();
		available.add(B0ColoniserArrivedTip.class);
		available.add(C1MonstersAboutTip.class);
		available.add(D1CraftDesignTip.class);
		isActive  = true;
	}
	
	public static void removeTip(Class c) {
		available.remove(c);
		if (available.size() == 0) available = null;	
	}
	
	public static void validateClick(ActiveRectangle ar) throws ApplicationError, ValidationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.validateClick(ar);
		}
	}
	
	public static void validateDoubleClick(ActiveRectangle ar) throws ApplicationError, ValidationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.validateDoubleClick(ar);
		}
	}
	
	public static void validateCommand(MotButton command) throws ApplicationError, ValidationError {
		if (!Comp.in(command.getClass(), exclude)) {
			Tutorial tip = getOpenTutorial();
			if (tip != null) {
				tip.validateCommand(command);
			}
		}
	}
	
	public static void validateCommand(MotImageButton command) throws ApplicationError, ValidationError {
		if (!Comp.in(command.getClass(), exclude)) {
			Tutorial tip = getOpenTutorial();
			if (tip != null) {
				tip.validateCommand(command);
			}
		}
	}

	public static void validateCommand(MotToggleButton command) throws ApplicationError, ValidationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.validateCommand(command);
		}
	}

	
	public static void validateBackgroundImageClicked(BackgroundImagePanel parent) throws ApplicationError, ValidationError {
		if (!Comp.in(parent.getClass(), exclude)) {
			Tutorial tip = getOpenTutorial();
			if (tip != null) {
				tip.validateBackgroundImageClicked(parent);
			}
		}
	}

	public static void validateProgressBarClicked(ProgressBarPanel parent) throws ApplicationError, ValidationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.validateProgressBarClicked(parent);
		}
	}

	public static void notifyEndTurn() throws ApplicationError, ValidationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.notifyEndTurn();
		} else {
			if (available != null && available.contains(D1CraftDesignTip.class)) {				
				UserPlayerDelegate owner = MainWindow.getOwner();
				PlanetDelegate [] planets = DeligateCacheManager.getPlanets(owner);
				if (planets != null) {
					for (int i = 0; i < planets.length; i++) {						
						ProjectDesign project = planets[i].getProjectDesign();
						if (project == null) {
							removeTip(D1CraftDesignTip.class);
							new D1CraftDesignTip(planets[i]).display();
							return;
						}
					}
				}
			}
		}
	}

	public static void notifyEndTurnComplete() throws ApplicationError {
	}

	public static void notifyWindowOpen(MotInternalWindow win) throws ApplicationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.notifyWindowOpen(win);
		} else if (win instanceof SelectPlanetWindow) {
			if (available != null && available.contains(B0ColoniserArrivedTip.class)) {
				removeTip(B0ColoniserArrivedTip.class);
				new B0ColoniserArrivedTip((SelectPlanetWindow)win).display();
			}
		}
	}

	public static void notifyClearSelection() throws ApplicationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.notifyClearSelection();
		}
	}

	public static void notifyWindowClose(MotInternalWindow win) throws ApplicationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.notifyWindowClose(win);
		}
	}
	
	public static void notifyInstructMoo(MoveableOwnedObjectDelegate moo, StellarObjectDelegate so, PlanetDelegate planet) throws ApplicationError {
		Tutorial tip = getOpenTutorial();
		if (tip != null) {
			tip.notifyInstructMoo(moo, so, planet);
		} else if (available != null && available.contains(C1MonstersAboutTip.class)) {
			UserPlayerDelegate owner = MainWindow.getOwner();
			if (so != null &&  Comp.same(so.getName(), GameConstants.TUTORIAL_FIRST_GUIRDED_NAME)) {
				removeTip(C1MonstersAboutTip.class);
				new C1MonstersAboutTip(so).display();
			}
		}
	}
	
	public static void notifyPlanetColonised(PlanetDelegate pd) throws ApplicationError {
	}

	public static void notifySwitchToGalaxyMap() throws ApplicationError {
	}
	
    public static void handleTick() throws ApplicationError, ValidationError {
		MainWindow mainWin = MainWindow.getInstance();
//		if (B6OpenSSTip.getColony() != null && !mainWin.getEventMailBox().hasGalaxyEventsInQueue() && !mainWin.getGalaxyMapConfiguration().getView().isDesktopPaneActive()) {
//			// B5EndTurnTip sets this value when this tip is ready to be displayed.
//			B6OpenSSTip.setColony(null);
//			new B6OpenSSTip().display();
//		}
    }
    
	public static Tutorial getOpenTutorial() throws ApplicationError {
		MainWindowMessageTutorial tipMessage = getOpenTipMessage();
		if (tipMessage != null) {
			return tipMessage.getTutorial();
		} else {
			return null;
		}
	}

	private static MainWindowMessageTutorial getOpenTipMessage() throws ApplicationError {
		if (!MainWindow.hasInstance()) {
			return null;
		}
		GalaxyWindowConfig gwc = MainWindow.getExestingInstance().getGalaxyMapConfiguration();
		if (gwc == null) {
			return null;
		}
		MainWindowMessageList ml = gwc.getGalaxyView().getMessageList();
		MainWindowMessage tipMessage = ml.getTipMessage();
		if (tipMessage != null && tipMessage instanceof MainWindowMessageTutorial) {
			return (MainWindowMessageTutorial) tipMessage;
		} else {
			return null;
		}
	}

	public static void closeTip() throws ApplicationError {
		MainWindowMessageList ml = MainWindow.getInstance().getGalaxyMapConfiguration().getGalaxyView().getMessageList();
		ml.removeTipMessage();
	}

	public static void skipTip() throws ApplicationError {
		MainWindowMessageTutorial tipMessage = getOpenTipMessage();
		if (tipMessage != null && tipMessage.getTutorial() != null) {
			tipMessage.getTutorial().next();
		}
	}

	public static int getDateOfLastTip() {
		return dateOfLastTip;
	}

	public static void setDateOfLastTip(int dateOfLastTip) {
		TutorialSystem.dateOfLastTip = dateOfLastTip;
	}

}
