/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.awt.Rectangle;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * Represents an area of a map.
 * The origin is the top left corner (point 0, 0).
 */
public class Area extends XMLBean implements IRectangle {

	private int top;
	private int left;
	private int bottom;
	private int right;

	public Area() {
	}
	
	public Area(Location center, int width, int height) {
		int centerX = center.getX();
		int centerY = center.getY();
		this.left = centerX - width/2;
		this.top = centerY - height/2;
		this.right = centerX + width/2;
		this.bottom = centerY + height/2;
	}	
	
	public Area(int left, int top, int right, int bottom) {
		if (left > right) {
			int t = left;
			left = right;
			right = t;
		}
		if (top > bottom) {
			int t = top;
			top = bottom;
			bottom = t;
		}
		this.top = top;
		this.left = left;
		this.bottom = bottom;
		this.right = right;
	}

	public Area(Location center, Size size) {
		this(center, size.getX(), size.getY());
	}

	public static Area createArea(AreaSrc [] aa) throws ApplicationError {
		Area r = null;
    	for (int i = 0; i < aa.length; i++) {
    		Area a = aa[i].getArea();
    		if (a != null) {    			
    			if (r == null) {
    				r = a.cloneArea();
    			} else {
    				r.add(a);
    			}
    		}
		}
    	return r;
	}
	
	public int getTop() { return top; }
	public int getLeft() { return left; }
	public int getBottom() { return bottom; }
	public int getRight() { return right; }

	public int getWidth() { return Comp.pos(right - left); }
	public int getHeight() { return Comp.pos(bottom - top); }
	public Size getSize() { return new Size(getWidth(), getHeight()); }
	
	public void setTop(int v) { top = v; }
	public void setLeft(int v) { left = v; }
	public void setBottom(int v) { bottom = v; }
	public void setRight(int v) { right = v; }

	public void shiftTop(int v) { top += v; }
	public void shiftLeft(int v) { left += v; }
	public void shiftBottom(int v) { bottom += v; }
	public void shiftRight(int v) { right += v; }
	
    public Location getCenterLocation() {
        return new Location(left + (right-left)/2, top + (bottom-top)/2);
    }

    /**
     * A point is assumed to be contained if it lays on one of the lines.
     * @return True if this contains the the point l.
     */
	public boolean contains(Location l) {
		return contains(l.getX(), l.getY());
	}

    /**
     * A point is assumed to be contained if it lays on one of the lines.
     * @return True if this contains the the point x,y.
     */
	public boolean contains(int x, int y) {
		return (x >= left && x <= right && y >= top && y <= bottom);
	}
    
    /**
     * A area is assumed to be contained if it is the same area.
     * @return True if this contains the whole of the area o.
     */
    public boolean contains(Area o) {
        return o.getLeft() >= left &&
               o.getRight() <= right &&
               o.getTop() >= top &&
               o.getBottom() <= bottom;
    }

    /**
     * @return True if this area intersects the area a and false otherwise.
     */
	public boolean intersects(Area a) {
        boolean doesNotIntersect = a.getTop() > getBottom() || 
                                   a.getBottom() < getTop() ||
                                   a.getLeft() > getRight() ||
                                   a.getRight() < getLeft();
		return !doesNotIntersect;
	}

    /**
     * Same as getClosestEdgeTo(int x, int y);
     */
	public Location getClosestEdgeTo(Location l) {
        return getClosestEdgeTo(l.getX(), l.getY());
    }
    
    /**
     * @return the location of the point on the edge of this Area (rectangle) that
     * is closest to the point x,y passed in.
     */
	public Location getClosestEdgeTo(int x, int y) {
		if (contains(x,y)) {
			if (x > left) x = left-1;
			if (x < right) x = right+1;
			
			if (y > top) y = top-1;
			if (y < bottom) y = bottom+1;			
		} else {
			if (x < left) x = left-1;
			if (x > right) x = right+1;
			
			if (y < top) y = top-1;
			if (y > bottom) y = bottom+1;			
		}

		return new Location(x, y);
	}
    
    /**
     * Same as getFurthestEdgeTo(int x, int y);
     */
	public Location getFurthestEdgeTo(Location l) {
        return getFurthestEdgeTo(l.getX(), l.getY());
    }
    
    /**
     * @return the location of the point on the edge of this Area (rectangle) that
     * is furthest away to the point x,y passed in.
     */
	public Location getFurthestEdgeTo(int x, int y) {
        int mx = (left + right)/2;
        int my = (top + bottom)/2;

		if (x < mx) x = right+2;
		else x = left-2;

		if (y < my) y = bottom+2;
		else y = top-2;

		return new Location(x, y);
	}
	
	/**
	 * This calculates the points of intersection of a line from location from to location to.
	 * It returns the closest location to from if more than one point of intersection is found.
	 * @param from The location the line is from
	 * @param to The location the line is to
	 * @return The first location that intersects this area on the line from to.
	 */
	public Location getFirstIntersection(Location from, Location to) {
		Location topLeft = new Location(top,left);
		Location topRight = new Location(top,right);
		Location bottomRight = new Location(bottom,right);
		Location bottomLeft = new Location(bottom,left);
		Location i1 = Geometry.getIntersection(from, to, topLeft, topRight);
		Location i2 = Geometry.getIntersection(from, to, topRight, bottomRight);
		Location i3 = Geometry.getIntersection(from, to, bottomRight, bottomLeft);
		Location i4 = Geometry.getIntersection(from, to, bottomLeft, topLeft);
		Location best = null;
		int bestDist = -1;
		if (i1 != null) {
			int d = i1.getDistanceTo(from);
			if (best == null || d < bestDist) {
				best = i1;
				bestDist = d;
			}
		}
		if (i2 != null) {
			int d = i2.getDistanceTo(from);
			if (best == null || d < bestDist) {
				best = i2;
				bestDist = d;
			}
		}
		if (i3 != null) {
			int d = i3.getDistanceTo(from);
			if (best == null || d < bestDist) {
				best = i3;
				bestDist = d;
			}
		}
		if (i4 != null) {
			int d = i4.getDistanceTo(from);
			if (best == null || d < bestDist) {
				best = i4;
				bestDist = d;
			}
		}
		return best;
	}
    
	public Location getTopIntersection(int x, int y, int d) {
        return getIntersection(x, y, d, 0.0, top);
    }
    
	public Location getBottomIntersection(int x, int y, int d) {
        return getIntersection(x, y, d, 0.0, bottom);
    }
    
	public Location getLeftIntersection(int x, int y, int d) {
        return getIntersection(x, y, d, Double.MAX_VALUE, Integer.MIN_VALUE);
    }
    
	public Location getRightIntersection(int x, int y, int d) {
        return getIntersection(x, y, d, Double.MAX_VALUE, Integer.MIN_VALUE);
    }
    
    /**
     * Given an objects location x,y and direction d it returns null if this
     * object will not intersect the line defined by gradient lineGraient and
     * intersection linec.
     * A direction if a value in degrees between 0 and 359 where 0 is north.
     * @return The point of intersection with the vector produced from point x,y in 
     * direction d (with infinite distance) and the top line of this area.
     */
	public static Location getIntersection(int x, int y, int d, double lineGradient, int linec) {
        double radians = Math.tan(Math.toRadians(d));
        double gradient = (radians == 0.0) ? Double.MAX_VALUE :  1.0/radians;
        if (gradient == lineGradient) {
            return null;
        }
        double c = calculateC(x, y, gradient);
        int rx = (int)((linec - c) / (gradient - lineGradient));
        int ry = (int)(gradient * x + c);
        return new Location (rx, ry);	
    }
    
    private static int calculateC(int x, int y, double gradient){
        return (int)(y - ( x * gradient ));
    }

    
    /**
     * Converts this area to a AWT Rectangle.
     * @return A Rectangle from the AWT library for this area.
     */
    public Rectangle getRectangle() {
        return new Rectangle(left, top, right-left, bottom-top);        
    }

    public mot.common.bean.Rectangle getMotRectangle() {
        return new mot.common.bean.Rectangle(left, top, right-left, bottom-top);        
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("top", getTop());
		writer.addElement("left", getLeft());
		writer.addElement("bottom", getBottom());
		writer.addElement("right", getRight());
	}

	protected String getElementName() { return "area"; }
    
    public String toString() { return "Area ["+left+","+top+" "+right+","+bottom+"]"; }

    /**
     * @return The maximum distance of any line within this area.
     */
	public int getDiameter() {
		return (int) Math.sqrt(Math.pow(right - left, 2) + Math.pow(bottom - top, 2));
	}

	public void shift(LocationDelta shiftDelta) {
		left += shiftDelta.getDx();
		right += shiftDelta.getDx();
		top += shiftDelta.getDy();
		bottom += shiftDelta.getDy();
	}

	public void scale(double scale) {
		left  = (int) (left * scale);
		right  = (int) (right * scale);
		top  = (int) (top * scale);
		bottom  = (int) (bottom * scale);
	}

	public Area cloneArea() {
		return new Area(left, top, right, bottom);
	}

	public void add(Area area) {
		if (area == null) return;
		if (area.getLeft() < left) left = area.getLeft();
		if (area.getTop() < top) top = area.getTop();
		if (area.getRight() > right) right = area.getRight();
		if (area.getBottom() > bottom) bottom = area.getBottom();
	}

	public Location getRandomLocation() {
		return new Location(left + Utils.random(right-left), top + Utils.random(bottom-top));
	}

	public Area getInsetArea(int inset) {
		int newLeft = left+inset;
		int newTop = top+inset;
		int newRight = right-inset;
		int newBottom = bottom-inset;
		if (newLeft >= newRight || newTop >= newBottom) {
			Logger.logDebug("Area.getInsetArea() INSET INVALID "+inset);
			return null;
		} else {			
			return new Area(newLeft, newTop, newRight, newBottom);
		}
	}

}