/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.util.ArrayList;

import mot.common.technology.WeaponFirePoint;
import mot.common.util.ApplicationError;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * Contains the image used to display a craft on the screen.
 * The name of this object is the name of the hull to which this
 * image corresponds.
 */
public class CraftGraphics extends NamedObject {

    public static final String CRAFT_IMAGE_FOLDER = "images/crafts/";

	/**
	 * Diagram of this hull
	 */
	private String pictureNormal;

	/**
	 * Diagram of this hull
	 */
	private String pictureCloaked;

	/**
	 * Diagram of this hull
	 */
	private String pictureSmall;

	/**
	 * Diagram of this hull
	 */
	private String picture3d;

	/**
	 * The screen space consumed by this hull.
	 */
	private Size size;

	/**
	 * This is the list of offsets from the centre of the image to locations 
	 * from which weapon fire animations should be drawn. Two values would 
	 * imply this hull should have its weapon fire animated from two locations.  
	 * A value of 10,10 would imply drawing weapon fire from image.x+10, image.y+10 
	 * and target.x, target.y. Likewise A value of 5,-5 would imply drawing 
	 * weapon fire from image.x+5, image.y-5 and target.x, target.y.
	 * NOTE - image.x, image.y should be the centre of the image!
	 *      - This should not be used to animate weapons fire from the HULL's dish!
	 *      - The weapon fire location must be rotated to the crafts firing position
	 */
	private ArrayList weaponFireOffsetList;
	private transient WeaponFirePoint [] weaponFireOffset;
	
	/**
	 * This is the location of the dish in the craft's image. A value of null implies 
	 * this hull does not have a dish - and as such weapons that are fired from the
	 * dish should not be placed in this hull. That being said it does not crash, it
	 * just uses a default location!
	 */
	private WeaponFirePoint dishOffset;
	
	/**
	 * The file type used for this image file.
	 */
	private String fileExtension;
	
	public CraftGraphics() {
	}

	public CraftGraphics(String name, String pictureNormal, String pictureCloaked, String pictureSmall, Size size, String fileExtension) {
		super(name);
		this.pictureNormal = pictureNormal;
		this.pictureCloaked = pictureCloaked;
		this.pictureSmall = pictureSmall;
		this.size = new Size(size);
		this.fileExtension = fileExtension;
	}

	public CraftGraphics(String name, String pictureNormal, String pictureCloaked, String pictureSmall, String picture3d, int size, String fileExtension, int wfo1_x, int wfo1_y, int wfo2_x, int wfo2_y, int dfo_x, int dfo_y) {
		super(name);
		this.pictureNormal = pictureNormal;
		this.pictureCloaked = pictureCloaked;
		this.pictureSmall = pictureSmall;
		this.picture3d = picture3d;
		this.size = new Size(size,size);
		this.fileExtension = fileExtension;
		
		if (wfo1_x != 0 || wfo1_y != 0) setWeaponFireOffset(new WeaponFirePoint(wfo1_x, wfo1_y));
		if (wfo2_x != 0 || wfo2_y != 0) setWeaponFireOffset(new WeaponFirePoint(wfo2_x, wfo2_y));
		if (dfo_x != 0 || dfo_y != 0) setDishOffset(new WeaponFirePoint(dfo_x, dfo_y));
	}

	public CraftGraphics(CraftGraphics other) {
		super(other.getName());
		this.pictureNormal = other.getPictureNormal();
		this.pictureCloaked = other.getPictureCloaked();
		this.pictureSmall = other.getPictureSmall();
		this.picture3d = other.getPicture3d();
		this.size = new Size(other.getSize());
		this.fileExtension = other.getFileExtension();
	}

	public CraftGraphics(DomNode n) throws ApplicationError {
		super(n.getNodeValueByName_String("name"));
		this.pictureNormal = n.getNodeValueByName_String("picture-normal");
		this.pictureCloaked = n.getNodeValueByName_String("picture-cloaked");
		this.pictureSmall = n.getNodeValueByName_String("picture-small");
		this.picture3d = n.getNodeValueByName_String("picture-3d");
		this.fileExtension = n.getNodeValueByName_String("file-extension");
		
		size = n.getNodeValueByName_Size("size");
		
		DomNode [] wfo = n.getNodesByName("weapon-fire-offset");
		if (wfo != null) {
			for (int i = 0; i < wfo.length; i++) {				
				setWeaponFireOffset(new WeaponFirePoint(wfo[i]));
			}
		}

		DomNode fo = n.getNodeByName("dish-offset");
		if (fo != null) dishOffset = new WeaponFirePoint(fo);

	}

	public String getPictureNormal() { return pictureNormal; }
	public void setPictureNormal(String v) { pictureNormal = v; }

	public String getPictureCloaked() { return pictureCloaked; }
	public void setPictureCloaked(String v) { pictureCloaked = v; }

	public String getFileExtension() { return (fileExtension == null) ? ".jpg" : fileExtension; }
	public void setFileExtension(String v) { fileExtension = v; }

	public Size getSize() { return size; }
	public void setSize(Size v) { size = v; }
    
    public WeaponFirePoint getDishOffset() { return dishOffset; }
    public void setDishOffset(WeaponFirePoint v) { dishOffset = v; }
    public boolean hasDish() { return (dishOffset != null); }
    
    public WeaponFirePoint [] getWeaponFireOffset() { 
    	if (weaponFireOffset == null) {
        	if (weaponFireOffsetList != null) {
        		weaponFireOffset = (WeaponFirePoint[]) weaponFireOffsetList.toArray(new WeaponFirePoint[weaponFireOffsetList.size()]);
        	} else {
        		weaponFireOffset = new WeaponFirePoint[0];
        	}
    	}
    	if (weaponFireOffset == null || weaponFireOffset.length == 0) {
        	return null;
    	} else {
        	return weaponFireOffset;
    	}
    }

    public void setWeaponFireOffset(WeaponFirePoint v) {
    	weaponFireOffset = null;
    	if (weaponFireOffsetList == null) weaponFireOffsetList = new ArrayList();
    	weaponFireOffsetList.add(v);
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		if (pictureNormal != null) writer.addElement("picture-normal", getPictureNormal());
		if (pictureCloaked != null) writer.addElement("picture-cloaked", getPictureCloaked());
		if (pictureSmall != null) writer.addElement("picture-small", getPictureSmall());
		if (picture3d != null) writer.addElement("picture-3d", getPicture3d());
		if (size != null) {
			size.toXML(writer);
		}
		getWeaponFireOffset();
		if (weaponFireOffset != null) {
			for (int i = 0; i < weaponFireOffset.length; i++) {
				weaponFireOffset[i].toXML(writer, "weapon-fire-offset");
			}
		}
		if (dishOffset != null) {
			dishOffset.toXML(writer, "dish-offset");
		}
		if (fileExtension != null) writer.addElement("file-extension", fileExtension);
	}

	protected String getElementName() { return "craft-graphics"; }

	public String getPictureSmall() {
		return pictureSmall;
	}

	public void setPictureSmall(String pictureSmall) {
		this.pictureSmall = pictureSmall;
	}

	public String getPicture3d() {
		return picture3d;
	}

	public void setPicture3d(String picture3d) {
		this.picture3d = picture3d;
	}
}