/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.io.Serializable;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;


/**
 * This contains the display-able state of a craft at a point in time.
 * It is common for the state of a craft to be out of step with the state of the craft 
 * that needs painting in a user interface. Therefore, Animations must paint the crafts
 * not in their current state, but in the state at the point of time the animation 
 * took place.
 */
public class CraftState implements Serializable {

	
	/**
	 * The owner of the craft
	 */
    private String owner;
    
    /**
     * The controller of the craft
     */
    private String controller;
    
    /**
     * The craft damage before this animations is taking place.
     */
    private double damagePercent;

    /**
     * The craft shield power before this animations is taking place.
     */
    private double shieldPercent;

    /**
     * The craft damage before this animations is taking place.
     */
    private double movementDamagePercent;

	/**
     * The craft shield power before this animations is taking place.
     */
    private double psiDamagePercent;

    /**
     * The location this craft was at.
     */
    private Location location;
    
    /**
     * The direction this craft is pointing in.
     */
    private int direction;
    
    /**
     * The quantity of crew in the craft.
     */
    private double crewPercent;
    
    /**
     * A list of image decoration to display on this object.
     */
    private DecoratorList decorators;
    
    /**
     * String the name of the owner.
     */
    private String color;

    /**
     * A list of player names this craft is cloaked to on the battle field.
     */
	private String [] cloakedToPlayerNames;
	
	/**
	 * If this craft is destroyed
	 */
	private boolean isDestroyed;

	/**
	 * The amount of move remaining to this craft / object.
	 */
	private double moveRemainingAsPercent;
	
    public CraftState() {
    }

    public CraftState(String owner, String controller, double damagePercent, double shieldPercent, double movementDamagePercent, double psiDamagePercent, double crewPercent, Location loc, int direction, PlayerColor pc, String [] cloakedToPlayerNames, DecoratorList decorators, boolean isDestroyed, double moveRemainingAsPercent) {
        this.owner = owner;
		this.controller = controller;
		this.damagePercent = damagePercent;
        this.shieldPercent = shieldPercent;
        this.movementDamagePercent = movementDamagePercent;
        this.psiDamagePercent = psiDamagePercent;
        this.crewPercent = crewPercent;
        this.location = (loc == null) ? null : new Location(loc);
        this.direction = direction;
        this.color = pc == null ? null : pc.getName();
        this.cloakedToPlayerNames = cloakedToPlayerNames;
        this.decorators = decorators;
        this.isDestroyed = isDestroyed;
        this.moveRemainingAsPercent = moveRemainingAsPercent;
    }

    public CraftState(String owner, String controller, double damagePercent, double shieldPercent, double movementDamagePercent, double psiDamagePercent, double crewPercent, Location loc, int direction, String playerColour, String [] cloakedToPlayerNames, DecoratorList decorators, boolean isDestroyed, double moveRemainingAsPercent) {
        this.owner = owner;
		this.controller = controller;
		this.damagePercent = damagePercent;
        this.shieldPercent = shieldPercent;
        this.movementDamagePercent = movementDamagePercent;
        this.psiDamagePercent = psiDamagePercent;
        this.crewPercent = crewPercent;
        this.location = (loc == null) ? null : new Location(loc);
        this.direction = direction;
        this.color = playerColour;
        this.cloakedToPlayerNames = cloakedToPlayerNames;
        this.decorators = decorators;
        this.isDestroyed = isDestroyed;
        this.moveRemainingAsPercent = moveRemainingAsPercent;
    }

    public CraftState(Location loc) {
        this.location = (loc == null) ? null : new Location(loc);
	}

	public void setDamagePercent(double damagePercent) {
        this.damagePercent = damagePercent;
    }


    public double getDamagePercent() {
        return damagePercent;
    }


    public void setShieldPercent(double shieldPercent) {
        this.shieldPercent = shieldPercent;
    }


    public double getShieldPercent() {
        return shieldPercent;
    }


    public void setMovementDamagePercent(double movementDamagePercent) {
        this.movementDamagePercent = movementDamagePercent;
    }


    public double getMovementDamagePercent() {
        return movementDamagePercent;
    }


    public void setPsiDamagePercent(double psiDamagePercent) {
        this.psiDamagePercent = psiDamagePercent;
    }


    public double getPsiDamagePercent() {
        return psiDamagePercent;
    }


    public void setLocation(Location location) {
        this.location = location;
    }


    public Location getLocation() {
        return location;
    }


    public void setDirection(int direction) {
        this.direction = direction;
    }


    public int getDirection() {
        return direction;
    }


    public void setCrewPercent(double crewPercent) {
        this.crewPercent = crewPercent;
    }


    public double getCrewPercent() {
        return crewPercent;
    }


    public void setDecorators(DecoratorList decorators) {
        this.decorators = decorators;
    }


    public DecoratorList getDecorators() {
        return decorators;
    }

	public PlayerColor getPlayerColor() throws ApplicationError {
		return color == null ? null : PlayerColor.getColorForName(color);
	}

	public String getColor() {
		return color;
	}

	public void setColor(String color) {
		this.color = color;
	}

	public String [] getCloakedToPlayerNames() {
		return cloakedToPlayerNames;
	}

	public void setCloakedToPlayerNames(String [] cloakedToPlayerNames) {
		this.cloakedToPlayerNames = cloakedToPlayerNames;
	}

	public String toString() {
		return "CraftState { loc=" + location+ " dam%=" + damagePercent + " shield%=" + shieldPercent + "colour = " + color + " cloakedTo=" + Comp.toString(cloakedToPlayerNames) + " }";
	}

	public boolean isDestroyed() {
		return isDestroyed;
	}

	public void setDestroyed(boolean isDestroyed) {
		this.isDestroyed = isDestroyed;
	}

	public String getOwner() {
		return owner;
	}

	public void setOwner(String owner) {
		this.owner = owner;
	}

	public String getController() {
		return controller;
	}

	public void setController(String controller) {
		this.controller = controller;
	}

	public double getMoveRemainingAsPercent() {
		return moveRemainingAsPercent;
	}

	public void setMoveRemainingAsPercent(double moveRemainingAsPercent) {
		this.moveRemainingAsPercent = moveRemainingAsPercent;
	}

}