/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.ValidationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * @author Paul Davis
 * 
 * Contains the details of an address.
 */
public class DateTime extends XMLBean {

    private int minute;
    private int hour;
    private int day;
    private int month;
    private int year;
    private boolean containsTime;

    public DateTime() {
    }

    public DateTime(DateTime other) throws ValidationError {
        containsTime = other.getContainsTime();
        setMinute(other.getMinute());
        setHour(other.getHour());
        setDay(other.getDay());
        setMonth(other.getMonth());
        setYear(other.getYear());
    }

    public DateTime(int day, int month, int year) throws ValidationError {
        containsTime = false;
        setMinute(0);
        setHour(0);
        setDay(day);
        setMonth(month);
        setYear(year);
    }

    public DateTime(String day, String month, String year) throws ValidationError {
        containsTime = false;
        setMinute(0);
        setHour(0);
		setDay(day);
		setMonth(month);
		setYear(year);
	}

    public void setDay(String day) throws ValidationError {
		try {
			setDay(Integer.parseInt(day));
		} catch (NumberFormatException e) {
			throw new  ValidationError("Invalid day "+day+" - Must be a number");
		}
    }

    public void setMonth(String month) throws ValidationError {
		try {
			setMonth(Integer.parseInt(month));
		} catch (NumberFormatException e) {
			throw new ValidationError("Invalid month "+month+" - Must be a number");
		}
    }

    public void setYear(String year) throws ValidationError {
		try {
			setYear(Integer.parseInt(year));
		} catch (NumberFormatException e) {
			throw new ValidationError("Invalid year "+year+" - Must be a number");
		}
    }

    public DateTime(String minute, String hour, String day, String month, String year)  throws ValidationError {
		this(day, month, year);
        containsTime = true;
        setMinute(minute);
        setHour(hour);
    }

    public void setMinute(String minute)  throws ValidationError {
		try {
			setMinute(Integer.parseInt(minute));
		} catch (NumberFormatException e) {
			throw new ValidationError("Invalid minute "+minute+" - Must be a number");
		}
    }

    public void setHour(String hour)  throws ValidationError {
		try {
			setHour(Integer.parseInt(hour));
		} catch (NumberFormatException e) {
			throw new ValidationError("Invalid hour "+hour+" - Must be a number");
		}
    }

    public DateTime(Date date) throws ValidationError {
        Calendar cal = Calendar.getInstance();
		cal.setTime(date);
        containsTime = false;
        setMinute(0);
        setHour(0);
        setDay(cal.get(Calendar.DAY_OF_MONTH));
        setMonth(cal.get(Calendar.MONTH)+1);
        setYear(cal.get(Calendar.YEAR));
	}

    public DateTime(Calendar cal) throws ValidationError {
		this(cal, true);
	}

    public DateTime(Calendar cal, boolean containsTime) throws ValidationError {
        this.containsTime = containsTime;
        if (containsTime) {
			setMinute(cal.get(Calendar.MINUTE));
			setHour(cal.get(Calendar.HOUR_OF_DAY));
		} else {
			setMinute(0);
			setHour(0);
		}
        setDay(cal.get(Calendar.DAY_OF_MONTH));
        setMonth(cal.get(Calendar.MONTH)+1);
        setYear(cal.get(Calendar.YEAR));
	}

    public int getMinute() { return minute; }
    public int getHour() { return hour; }
    public int getDay() { return day; }
    public int getMonth() { return month; }
    public int getYear() { return year; }
    public String getMonthName() { return monthNames[month-1]; }

    public void setMinute(int v) throws ValidationError {
        minute = v;
		if (minute < 0 || minute > 59) {
			throw new ValidationError("Invalid minute "+minute+" - Must be between 0 and 59");
		}
    }
    public void setHour(int v) throws ValidationError {
        hour = v;
		if (hour < 0 || hour > 23) {
			throw new ValidationError("Invalid hour "+hour+" - Must be between 0 and 23");
		}
    }
    public void setDay(int v) throws ValidationError {
        day = v;
		if (day < 1 || day > 31) {
			throw new ValidationError("Invalid day "+day+" - Must be between 1 and 31");
		}
    }
    public void setMonth(int v) throws ValidationError {
        month = v;
		if (month < 1 || month > 12) {
			throw new ValidationError("Invalid month "+month+" - Must be between 1 and 12");
		}
    }
    public void setYear(int v) throws ValidationError {
        year = v;
		if (year < 1900) {
			throw new ValidationError("Invalid year "+year+" - Must be greater than 1900");
		}
    }

    private static final String[] monthNames = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

    public GregorianCalendar getCalendar() throws ValidationError {
        GregorianCalendar cal = new GregorianCalendar();
        cal.setLenient(false);
        try {
            cal.set(year, month-1, day, hour, minute);

            // I will get a Date objecty to make sure it calculates the Date Value here
            // and so it must be a valid date!!!
            cal.getTime();
        } catch (Exception e) {
            String m = monthNames[month-1];
            throw new ValidationError("Invalid Date - Day "+day+" month "+m+" year "+year);
        }

        return cal;
    }

	public java.sql.Date getSqlDate() throws ValidationError {
		long v = getCalendar().getTime().getTime();
		return new java.sql.Date(v);
	}

	public java.sql.Timestamp getSqlTimestamp() throws ValidationError {
		long v = getCalendar().getTime().getTime();
		return new java.sql.Timestamp(v);
	}

	/**
	 * Get day of week for this date..
	 */
    public int getDayOfWeek() throws ValidationError {
		return getCalendar().get(Calendar.DAY_OF_WEEK);
	}

	/**
	 * Get day of week for first day of this month.
	 */
    public int getFirstDayOfWeek() throws ValidationError {
		Calendar cal = new DateTime(1, month, year).getCalendar();
		return cal.get(Calendar.DAY_OF_WEEK);
	}

    public boolean after(DateTime dt) throws ValidationError {
        return after(dt.getCalendar());
    }

    public boolean before(DateTime dt) throws ValidationError {
        return before(dt.getCalendar());
    }

	private static final long SEC = 60*1000;

    public boolean after(Calendar dt) throws ValidationError {
		long mine = getCalendar().getTime().getTime();
		long other = dt.getTime().getTime();

		// round to minutes
		mine = mine / SEC;
		other = other / SEC;

		return (mine > other);
	}

    public boolean before(Calendar dt) throws ValidationError {
		long mine = getCalendar().getTime().getTime();
		long other = dt.getTime().getTime();

		// round to minutes
		mine = mine / SEC;
		other = other / SEC;

		return (mine < other);
	}

    public boolean sameDate(DateTime other) {
		if (other == null) return false;
        return ( year == other.getYear()  && month == other.getMonth() && day == other.getDay() );
    }

    public boolean getContainsTime() {
        return containsTime;
    }

    int monthLength[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

	/**
	 * Get qty days for this month.
	 */
    public int getMaxDaysForMonth() throws ValidationError {
		int max = monthLength[month-1];
		if (month == 0) {
			if (getCalendar().isLeapYear(year)) {
				max = 29;
			}
		}
		return max;
	}

    public String toString() {
        if (containsTime) {
			return getDateTimeString();
		} else {
			return getDateString();
		}
    }

    public String getDateString() {
        return ""+day+" of "+getMonthName()+" "+year;
    }

    public String getDateTimeString() {
        return ""+day+" of "+getMonthName()+" "+year+" "+hour+":"+minute;
    }

    public String getDateStringSQL() {
        return "TO_DATE('"+day+" "+getMonthName()+" "+year+"')";
    }

    public String getTimestampStringSQL() {
        return "TO_DATE('"+day+" "+getMonthName()+" "+year+" "+hour+":"+minute+"')";
    }

    /**
     * Returns a date time object containing the date today with no time element.
     */
    public static DateTime getInstance() throws ValidationError {
        return new DateTime( Calendar.getInstance().getTime() );
    }

    /**
     * Returns a date time object containing the date and time of the call.
     */
    public static DateTime getTimeInstance() throws ValidationError  {
        return new DateTime( Calendar.getInstance() );
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError {
    	BasicNameValueList list = getAttributesNotNull(generateClassAttribute);
        list.add("minute", ""+minute);
        list.add("hour", ""+hour);
        list.add("day", ""+day);
        list.add("month", ""+month);
        list.add("year", ""+year);
        return list; 
    }
    
	protected String getElementName() { return "date-time"; }
    protected boolean beanHasBody() { return false; }
}