/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.util.ArrayList;
import java.util.Arrays;

import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * A list of the relationship between to players.
 * e.g.
 * 	Date 3000 Diplomatic Relations Formed
 *  Date 3050 War Declared
 *  Date 3060 Peace Treaty Signed
 *
 */
public class DiplomaticHistory extends XMLBean {

	/**
	 * The list of diplomatic events recorded in this history.
	 */
	private DiplomaticHistoryEvent [] events;
	
	public DiplomaticHistory() {
	}
	
	public DiplomaticHistory(DiplomaticHistory o) {
		if (o != null) {
			DiplomaticHistoryEvent[] ea = o.getEvents();
			if (ea != null) {				
				events = Arrays.copyOf(ea, ea.length);
			}
		}
	}

	public void addDiplomaticHistoryEvent(double date, DiplomaticProposal proposal, String playerOfferMaker) {
		setDiplomaticHistoryEvent(new DiplomaticHistoryEvent(date, proposal, playerOfferMaker));
	}
	
	public void addDiplomaticHistoryEvent(double date, String description) {
		setDiplomaticHistoryEvent(new DiplomaticHistoryEvent(date, description));
	}
	
	public void setDiplomaticHistoryEvent(DiplomaticHistoryEvent e) {
		if (events == null) {
			events = new DiplomaticHistoryEvent [1];
			events[0] = e;
		} else {
			events = Arrays.copyOf(events, events.length+1);
			events[events.length-1] = e;
		}
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (events != null) {
			for (int i = 0; i < events.length; i++) {
				events[i].toXML(writer);
			}
		}
	}

	protected String getElementName() {
		return "diplomatic-history";
	}

	public DiplomaticHistoryEvent [] getEvents() {
		return events;
	}

	public void setEvents(DiplomaticHistoryEvent [] events) {
		this.events = events;
	}

	public int getQuantityOfEvents() {
		if (events != null) {
			return events.length;
		} else {
			return 0;
		}
	}

	public DatedMessage[] getOneLineSummaries(String player1, String player2) throws ApplicationError {
		ArrayList<DatedMessage> r = null;
		if (events != null) {
			for (int i = 0; i < events.length; i++) {
				if (r == null) r = new ArrayList();
				r.add(new DatedMessageBean(events[i].getDate(), events[i].getDiplomaticHistorySummary(player1, player2)));
			}
		}
		if (r == null) {
			return null;
		} else {
			return r.toArray(new DatedMessageBean[r.size()]);
		}
	}
}
