/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * A list of the relationship between to players.
 * e.g.
 * 	Date 3000 Diplomatic Relations Formed
 *  Date 3050 War Declared
 *  Date 3060 Peace Treaty Signed
 *
 */
public class DiplomaticHistoryEvent extends XMLBean {

	/**
	 * The date the proposal was accepted - Note it is a decimal as several things can occur in a single year
	 */
	private double date;
	
	/**
	 * The diplomatic proposal that was agreed to between the players
	 */
	private DiplomaticProposal proposal;
	
	/**
	 * This is a textual description of the event. This may be used INSTEAD OF a proposal
	 */
	private String description;
	
	/**
	 * The name of the player that made the offer. As the diplomatic history is stored in a relationship, we 
	 */
	private String playerOfferMaker;

	public DiplomaticHistoryEvent() {
	}
	
	public DiplomaticHistoryEvent(double date, DiplomaticProposal proposal, String playerOfferMaker) {
		super();
		this.date = date;
		this.proposal = new DiplomaticProposal(proposal);
		this.playerOfferMaker = playerOfferMaker;
	}
	
	public DiplomaticHistoryEvent(double date, String description) {
		super();
		this.date = date;
		this.description = description;
	}

	public DiplomaticHistoryEvent(DiplomaticHistoryEvent diplomaticHistory) {
		this.date = diplomaticHistory.getDate();
		this.proposal = new DiplomaticProposal(diplomaticHistory.getProposal());
		this.playerOfferMaker = diplomaticHistory.getPlayerOfferMaker();
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("date", date);
		writer.addElement("player-offer-maker", playerOfferMaker);
		if (proposal != null) proposal.toXML(writer, "proposal");
		if (description != null) writer.addElement("description", description);
	}

	public String getDiplomaticHistorySummary(String player1, String player2) throws ApplicationError {
		if (proposal == null) {
			return description;
		}
		return proposal.getDiplomaticHistorySummary(playerOfferMaker, getOtherPlayer(player1, player2));
	}

	private String getOtherPlayer(String player1, String player2) {
		if (Comp.same(playerOfferMaker, player1)) {
			return player2;
		} else {
			return player1;
		}	
	}

	protected String getElementName() {
		return "diplomatic-history-event";
	}

	public double getDate() {
		return date;
	}

	public void setDate(double date) {
		this.date = date;
	}

	public DiplomaticProposal getProposal() {
		return proposal;
	}

	public void setProposal(DiplomaticProposal proposal) {
		this.proposal = proposal;
	}

	public String getPlayerOfferMaker() {
		return playerOfferMaker;
	}

	public void setPlayerOfferMaker(String playerOfferMaker) {
		this.playerOfferMaker = playerOfferMaker;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

}
