/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.io.Serializable;

import mot.common.lang.L;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.vo.TechnologyListVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

public class DiplomaticProposal extends XMLBean implements Serializable {

	public static DiplomaticProposal getWar() { DiplomaticProposal r = new DiplomaticProposal(); r.setWar(true); return r; }
	public static DiplomaticProposal getPeace() { DiplomaticProposal r = new DiplomaticProposal(); r.setPeace(true); return r; }
	public static DiplomaticProposal getNonAggression() { DiplomaticProposal r = new DiplomaticProposal(); r.setNonAggression(true); return r; }
	public static DiplomaticProposal getAlliance() { DiplomaticProposal r = new DiplomaticProposal(); r.setAlliance(true); return r; }
	public static DiplomaticProposal getTrade() { DiplomaticProposal r = new DiplomaticProposal(); r.setTrade(true); return r; }
	public static DiplomaticProposal getResearch() { DiplomaticProposal r = new DiplomaticProposal(); r.setResearch(true); return r; }

	public static DiplomaticProposal gift(Technology tech, int money, String planet) throws ApplicationError { return new DiplomaticProposal(new PlayerAssets(tech, money, planet), null); }
	public static DiplomaticProposal gift(Technology tech) throws ApplicationError { return new DiplomaticProposal(new PlayerAssets(tech), null); }
	public static DiplomaticProposal gift(String planet) throws ApplicationError { return new DiplomaticProposal(new PlayerAssets(planet), null); }
	public static DiplomaticProposal gift(int money) throws ApplicationError { return new DiplomaticProposal(new PlayerAssets(money), null); }
	public static DiplomaticProposal gift(NameList crafts) { return new DiplomaticProposal(new PlayerAssets(crafts), null); }
	
	public static DiplomaticProposal demand(Technology tech, int money, String planet) throws ApplicationError { return new DiplomaticProposal(null, new PlayerAssets(tech, money, planet)); }
	public static DiplomaticProposal demand(Technology tech) throws ApplicationError { return new DiplomaticProposal(null, new PlayerAssets(tech)); }
	public static DiplomaticProposal demand(String planet) throws ApplicationError { return new DiplomaticProposal(null, new PlayerAssets(planet)); }
	public static DiplomaticProposal demand(int money) throws ApplicationError { return new DiplomaticProposal(null, new PlayerAssets(money)); }
	public static DiplomaticProposal demand(NameList crafts) { return new DiplomaticProposal(null, new PlayerAssets(crafts)); }
	public static DiplomaticProposal demand(PlayerAssets wanted) { return new DiplomaticProposal(null, wanted); }

	public static DiplomaticProposal swap(Technology offeredTech, int offeredMoney, String offeredPlanet, Technology wantTech, int wantMoney, String wantPlanet) throws ApplicationError { return new DiplomaticProposal(new PlayerAssets(offeredTech, offeredMoney, offeredPlanet), new PlayerAssets(wantTech, wantMoney, wantPlanet)); }
	public static DiplomaticProposal swap(Technology offeredTech, Technology wantTech) throws ApplicationError { return new DiplomaticProposal(new PlayerAssets(offeredTech), new PlayerAssets(wantTech)); }
	public static DiplomaticProposal swap(int offeredMoney, int wantMoney) throws ApplicationError { return new DiplomaticProposal(new PlayerAssets(offeredMoney), new PlayerAssets(wantMoney)); }
	public static DiplomaticProposal swap(String offeredPlanet, String wantPlanet) throws ApplicationError { return new DiplomaticProposal(new PlayerAssets(offeredPlanet), new PlayerAssets(wantPlanet)); }

	private boolean war;
	private boolean peace;
	private boolean nonAggression;
	private boolean alliance;
	private boolean trade;
	private boolean research;
	
    private PlayerAssets offered; 
    private PlayerAssets wanted;
    
    /**
     * Has this proposal already been performed (war declared, assets tranfered etc). 
     */
    private boolean executed = false;

    public DiplomaticProposal() {
    }

    public DiplomaticProposal(PlayerAssets offered, PlayerAssets wanted) {
        this.offered = offered;
        this.wanted = wanted;
	}

    public DiplomaticProposal(DiplomaticProposal proposal) {
    	war = proposal.isWar();
    	peace = proposal.isPeace();
    	nonAggression = proposal.isNonAggression();
    	alliance = proposal.isAlliance();
    	trade = proposal.isTrade();
    	research = proposal.isResearch();
    	if (proposal.getOffered() != null) offered = new PlayerAssets(proposal.getOffered());
    	if (proposal.getWanted() != null) wanted = new PlayerAssets(proposal.getWanted());
	}
    
	public void setOffered(PlayerAssets offered) {
        this.offered = offered;
    }

    public PlayerAssets getOffered() {
        return offered;
    }

    public PlayerAssets getOfferedNotNull() {
    	if (offered == null) offered = new PlayerAssets();
        return offered;
    }

    public void setWanted(PlayerAssets wanted) {
        this.wanted = wanted;
    }

    public PlayerAssets getWanted() {
        return wanted;
    }

    public PlayerAssets getWantedNotNull() {
    	if (wanted == null) wanted = new PlayerAssets();
        return wanted;
    }

	public boolean isWar() {
		return war;
	}

	public void setWar(boolean war) {
		this.war = war;
	}

	public boolean isWarWithNoDemand() {
		return war && wanted == null;
	}

	public boolean isDemand() {
		return offered == null && wanted != null;
	}

	public boolean isSwap() {
		return offered != null && wanted != null;
	}

	public boolean isWarWithDemand() {
		return war && wanted != null;
	}

	public boolean isFreeGift() {
		return !hasTreaty() && offered != null && wanted == null;
	}

	public boolean isPeace() {
		return peace;
	}

	public void setPeace(boolean peace) {
		this.peace = peace;
	}

	public boolean isNonAggression() {
		return nonAggression;
	}

	public void setNonAggression(boolean nonAggression) {
		this.nonAggression = nonAggression;
	}

	public boolean isAlliance() {
		return alliance;
	}

	public void setAlliance(boolean alliance) {
		this.alliance = alliance;
	}

	public boolean isTrade() {
		return trade;
	}

	public void setTrade(boolean trade) {
		this.trade = trade;
	}

	public boolean isResearch() {
		return research;
	}

	public void setResearch(boolean research) {
		this.research = research;
	}

	public boolean hasTreaty() {
		return war || peace || nonAggression || alliance || trade || research;
	}

	public void removeTreaies() {
		war = false;
		peace = false;
		nonAggression = false;
		alliance = false;
		trade = false;
		research = false;
	}

	public void clear() {
		removeTreaies();
		offered = null;
		wanted = null;
		executed = false;
	}

    public String getProposalTextHTML(String playerName) throws ApplicationError {
    	if (isWar()) {
    		if (wanted != null && offered != null) {
    			return "<html>Give us:<br/> "+getAssetsText(wanted, "&nbsp;&nbsp;&nbsp;")+"<br/>In exchange for:<br/>"+getAssetsText(offered, "&nbsp;&nbsp;&nbsp;")+"<br/>Otherwise we share declare War</html>";
    		} else if (wanted != null) {
    			return "<html>Give us:<br/> "+getAssetsText(wanted, "&nbsp;&nbsp;&nbsp;")+"<br/>Otherwise we will declare WAR</html>";
    		} else if (offered != null) {
    			return "<html>If you do not accept our gift of:<br/>"+getAssetsText(offered, true, true)+"<br/>We shall declare WAR [diplomatic blunder!]</html>";
    		} else {
    			return "<html>Declare War</html>";
    		}
    	} else {
    		String start = null;
	        if (isAlliance()) {
	        	start =  "<html>"+L.WOULD_YOU_CONSIDER_SIGNING_A_+L.ALLIENCE_TREATY;
	        }
	        if (isNonAggression()) {
	        	start =  "<html>"+L.WOULD_YOU_CONSIDER_SIGNING_A_+L.NON_AGRESSION_TREATY;
	        }
	        if (isPeace()) {
	        	start =  "<html>"+L.WOULD_YOU_CONSIDER_SIGNING_A_+L.PEACE_TREATY;
	        }
	        if (isTrade()) {
	        	start =  "<html>"+L.WOULD_YOU_CONSIDER_SIGNING_A_+L.TRADE_TREATY;
	        }
	        if (isResearch()) {
	        	start =  "<html>"+L.WOULD_YOU_CONSIDER_SIGNING_A_+L.RESEARCH_TREATY;
	        }
    		if (wanted != null && offered != null) {
    			if (start != null) {
    				return start + " with an exchange of assets.<br/>We offer:<br/> "+getAssetsText(offered, "&nbsp;&nbsp;&nbsp;")+"<br/>In exchange for:<br/>"+getAssetsText(wanted, "&nbsp;&nbsp;&nbsp;")+"<br/>Do we have a deal?</html>";
    			} else {
    				return "<html>Would you consider an exchange?<br/>Give us:<br/> "+getAssetsText(offered, "&nbsp;&nbsp;&nbsp;")+"<br/>In exchange for:<br/>"+getAssetsText(wanted, "&nbsp;&nbsp;&nbsp;")+"<br/>Do we have a deal?</html>";
    			}
    		} else if (wanted != null) {
    			if (start != null) {
    				return start + " in exchange for:<br/>"+getAssetsText(wanted, "&nbsp;&nbsp;&nbsp;")+"<br/>Do you accept?</html>";
    			} else {
    				return "<html>We demand you give us:<br/>"+getAssetsText(wanted, "&nbsp;&nbsp;&nbsp;")+"<br/>Do you accept?</html>";
    			}
    		} else if (offered != null) {
    			if (start != null) {
    				return start + " with a gift of:<br/>"+getAssetsText(offered, "&nbsp;&nbsp;&nbsp;")+"<br/>Do you  accept?</html>";
    			} else {
    				return "<html>We offer you a free gift of:<br/>"+getAssetsText(offered, "&nbsp;&nbsp;&nbsp;")+"<br/>as a token of our good will</html>";
    			}
    		} else {
    			if (start == null) {
    				return null;
    			} else {
    				return start+"?</html>";
    			}
    		}
    	}
    }

	public String getDiplomaticHistorySummary(String playerOfferMaker, String playerOther) throws ApplicationError {
	   	if (isWar()) {
    		if (wanted != null && offered != null) {
    			String offeredMsg = getAssetsText(wanted, "&nbsp;&nbsp;&nbsp;");
				String wantedMsg = getAssetsText(offered, "&nbsp;&nbsp;&nbsp;");
    			return "<html>"+playerOther+" agreed to asset swap "+playerOfferMaker+" under threat of war<br/>Offered: "+offeredMsg+"<br/>Received: "+wantedMsg+"</html>";
    		} else if (wanted != null) {
				String wantedMsg = getAssetsText(wanted, "&nbsp;&nbsp;&nbsp;");
    			return "<html>"+playerOther+" agreed to asset demand by "+playerOfferMaker+" under threat of war<br/>Received: "+wantedMsg+"</html>";
    		} else if (offered != null) {
    			String offeredMsg = getAssetsText(offered, "&nbsp;&nbsp;&nbsp;");
    			return "<html>"+playerOther+" agreed to take a gift "+playerOfferMaker+" under threat of war<br/>Offered: "+offeredMsg+"</html>";
    		} else {
    			return playerOfferMaker+L._DECLARED_WAR_ON_+playerOther;
    		}
	   	} else if (wanted != null && wanted.hasAssets() && offered != null && offered.hasAssets()) {
	   		String offeredMsg = getAssetsText(offered, false, false);
	   		String wantedMsg = getAssetsText(wanted, false, false);
        	if (hasTreaty()) {
        		String treatiesText = getTreatyString();
				return "<html>"+playerOther+" agreed to asset swap that "+playerOfferMaker+" proposed<br/>Offered: "+offeredMsg+"<br/>Received: "+wantedMsg+"<br/>Treaties: "+treatiesText+"</html>";
        	} else {
				return "<html>"+playerOther+" agreed to asset swap that "+playerOfferMaker+" proposed<br/>Offered: "+offeredMsg+"<br/>Received: "+wantedMsg+"</html>";
        	}
        } else if (offered != null && offered.hasAssets() && (wanted == null || !wanted.hasAssets())) {
        	String offeredMsg = getAssetsText(offered, false, false);
        	if (hasTreaty()) {
        		String treatiesText = getTreatyString();
        		return "<html>"+playerOther+" agreed to "+getTreatyString()+"from "+playerOfferMaker+" with gifts<br/>Offered: "+offeredMsg+"<br/>Treaties: "+treatiesText+"</html>";
        	} else {
        		return "<html>"+playerOfferMaker+" gave a gift to "+playerOther+"<br/>Offered: "+offeredMsg+"</html>";
        	}
        } else if (wanted != null && wanted.hasAssets() && (offered == null || !offered.hasAssets())) {
        	String wantedMsg = getAssetsText(wanted, false, false);
        	if (hasTreaty()) {
        		String treatiesText = getTreatyString();
        		return "<html>"+playerOther+" agreed to "+getTreatyString()+"from "+playerOfferMaker+" with demand<br/>Received: "+wantedMsg+"<br/>Treaties: "+treatiesText+"</html>";
        	} else {
        		return "<html>"+playerOfferMaker+" demanded a gift from "+playerOther+"<br/>Received: "+wantedMsg+"</html>";
        	}
        } else {
        	String r =  playerOther;
        	if (isAlliance()) {
            	r = r + " agreed to an " + L.ALLIENCE_TREATY;
            } else if (isNonAggression()) {
            	r = r + " agreed to a " + L.NON_AGGRESSION_TREATY;
            } else if (isPeace()) {
            	r = r + " agreed to a " + L.PEACE_TREATY;
            } else if (isTrade()) {
            	r = r + " agreed to a " + L.TRADE_TREATY;
            } else if (isResearch()) {
            	r = r + " agreed to a " + L.RESEARCH_TREATY;
            }
           return r + " from "+playerOfferMaker;
        }
	}
	

    public String getOneLineProposalText(String playerName) throws ApplicationError {
        if (wanted != null && wanted.hasAssets() && offered != null && offered.hasAssets()) {
            String offeredMsg = getAssetsText(offered, false, true);
            String wantedMsg = getAssetsText(wanted, false, true);
			return L.YOU_OFFERED_+playerName+" "+offeredMsg+L._IN_EXCHANGE_FOR+wantedMsg;
        } else if (offered != null && offered.hasAssets() && (wanted == null || !wanted.hasAssets())) {
            String offeredMsg = getAssetsText(offered, false, true);
            return L.YOU_GIFTED_+playerName+L._WITH_+offeredMsg;
        } else if (wanted != null && wanted.hasAssets() && (offered == null || !offered.hasAssets())) {
            String wantedMsg = getAssetsText(wanted, false, true);
        	return L.YOU_DEMANDED_+playerName+L._GIVE_YOU_+wantedMsg;
        } else {
        	String r = null; 
        	if (isWar()) {
    			r =  L.YOU_DECLARED_WAR_ON+playerName;
            } else {
            	r = L.YOU_OFFERED_+playerName;
            }
        	if (isAlliance()) {
            	r = r + " "+L.ALLIENCE_TREATY;
            } else if (isNonAggression()) {
            	r = r + " "+L.NON_AGGRESSION_TREATY;
            } else if (isPeace()) {
            	r = r + " "+L.PEACE_TREATY;
            } else if (isTrade()) {
            	r = r + " "+L.TRADE_TREATY;
            } else if (isResearch()) {
            	r = r + " "+L.RESEARCH_TREATY;
            }
           return r;
        }
    }
    
    private String getAssetsText(PlayerAssets assets, String indent) throws ApplicationError {
    	StringBuilder r = new StringBuilder();
    	boolean first = true;
	    if (assets.getMoney() > 0) {
	    	if (indent != null) r.append(indent);
	    	r.append( L.MONEY_+assets.getMoney() );
	    	first = false;
	    }
	    TechnologyListVO techs = assets.getTechnologies();
		if (techs != null) {
	    	if (!first) r.append("<br/>");
	    	Technology[] tl = techs.getAllAsArray();
	    	if (tl != null) {	    		
	    		for (int i = 0; i < tl.length; i++) {
	    	    	if (i > 0) r.append("<br/>");
	    	    	if (indent != null) r.append(indent);	    			
	    	    	r.append( L.TECHNOLOGY_+tl[i].getName() );
	    		}
	    	}
    		first = false;
	    }
	    if (assets.getPlanet() != null) {
	    	if (!first) r.append("<br/>");
	    	if (indent != null) r.append(indent);
	    	r.append(L.PLANET+" "+assets.getPlanet());
    		first = false;
	    }
	    if (assets.getCraftNames() != null) {
	    	if (!first) r.append("<br/>");
	    	if (indent != null) r.append(indent);
	    	r.append(L.CRAFT_+assets.getCraftNames()) ;
    		first = false;
	    }
	    return r.toString();
	}
    
    private String getAssetsText(PlayerAssets assets, boolean html, boolean withTreaties) throws ApplicationError {
    	StringBuilder r = new StringBuilder();
    	boolean first = true;
	    if (assets.getMoney() > 0) {
	    	r.append( L.MONEY_+assets.getMoney() );
    		first = false;
	    }
	    if (assets.getTechnologies() != null) {
	    	if (!first) if (html) r.append("<br/>"); else r.append(" ");
	    	r.append( L.TECHNOLOGY_+assets.getTechnologies().getNames(", ") );
    		first = false;
	    }
	    if (assets.getPlanet() != null) {
	    	if (!first) if (html) r.append("<br/>"); else r.append(" ");
	    	r.append(L.PLANET_+assets.getPlanet());
    		first = false;
	    }
	    if (assets.getCraftNames() != null) {
	    	if (!first) if (html) r.append("<br/>"); else r.append(" ");
	    	r.append(L.CRAFT_+assets.getCraftNames()) ;
    		first = false;
	    }
	    if (withTreaties) {
	    	if (peace) {
	    		if (!first) if (html) r.append("<br/>"); else r.append(" ");
	    		r.append(" "+L.PEACE_TREATY);
	    		first = false;
	    	}
	    	if (nonAggression) {
	    		if (!first) if (html) r.append("<br/>"); else r.append(" ");
	    		r.append(" "+L.NON_AGGRESSION_TREATY);
	    		first = false;
	    	}
	    	if (alliance) {
	    		if (!first) if (html) r.append("<br/>"); else r.append(" ");
	    		r.append(" "+L.ALLIENCE_TREATY);
	    		first = false;
	    	}
	    	if (trade) {
	    		if (!first) if (html) r.append("<br/>"); else r.append(" ");
	    		r.append(" "+L.TRADE_TREATY);
	    		first = false;
	    	}
	    	if (research) {
	    		if (!first) if (html) r.append("<br/>"); else r.append(" ");
	    		r.append(" "+L.RESEARCH_TREATY);
	    		first = false;
	    	}
	    }
	    return r.toString();
	}
    
    public String getGalaxyWindowRequestMessage(String playerName) throws ApplicationError {
        if (isWar()) {
            return L.PLAYER_+playerName+L.HAS_DECLARED_WAR_ON_YOU;
        } else if (isAlliance()) {
        	return L.PLAYER_+playerName+L.REQUESTS_AN_ALLIANCE_WITH_YOU;
        } else if (isNonAggression()) {
           	return L.PLAYER_+playerName+L.REQUESTS_A_NON_AGRESSION_PACT_WITH_YOU;
        } else if (isPeace()) {
            return L.PLAYER_+playerName+L.REQUESTS_A_PEACE_TREATY_WITH_YOU;
        } else if (isTrade()) {
            return L.PLAYER_+playerName+L.REQUEST_A_TRADE_AGREEMENT_WITH_YOU;
        } else if (isResearch()) {
            return L.PLAYER_+playerName+L.REQUESTS_A_RESEAECH_TREATY_WITH_YOU;
        } else {
    		if (wanted != null && offered != null) {
    	        return L.PLAYER_+playerName+L.REQUESTS_AN_EXCHANGE_OFFERED+getAssetsText(offered, null)+L.WANTED+getAssetsText(wanted, null)+L.WOULD_YOU_LIKE_TO_ACCEPT;
    		} else if (offered != null) {
        		return L.PLAYER_+playerName+L.HAS_GIVEN_YOU+getAssetsText(offered, null)+L.AS_A_GESTURE_OF_GOODWILL;
            } else if (wanted != null){
            	return L.PLAYER_+playerName+L.DEMANDS+getAssetsText(wanted, null)+L.WILL_YOU_CONCEDE;
            } else {
            	return "Received unknown diplomatic request from "+playerName;
            }
        }
    }

    public String getGalaxyWindowEnemyResponseMessage(String playerName, boolean accepted) throws ApplicationError {
    	String start = null;
        if (isWar()) {
    		if (wanted != null && offered != null) {
            	if (accepted) {
                	return "You rejected exchange or WAR with player "+playerName+" [Now at WAR]";
            	} else {
                	return "You accepted exchange or WAR with player "+playerName+" [WAR averted]";
            	}
    		} else if (offered != null) {
            	return L.PLAYER_+playerName+L.HAS_DECLARED_WAR_ON_YOU+" with a gift";
            } else if (wanted != null){
            	if (accepted) {
                	return "You rejected demand or WAR with player "+playerName+" [Now at WAR]";
            	} else {
                	return "You accepted demand or WAR with player "+playerName+" [WAR averted]";
            	}
            } else {
            	return L.PLAYER_+playerName+L.HAS_DECLARED_WAR_ON_YOU;
            }
        } else if (isAlliance()) {
        	if (accepted) {
        		start = L.ALLIANCE_SIGNED_WITH+playerName;
        	} else {
        		start = L.ALLIANCE_REJECTED_WITH+playerName;
        	}
        } else if (isNonAggression()) {
        	if (accepted) {
        		start = L.NON_AGRESSION_PACT_SIGNED_WITH+playerName;
        	} else {
        		start = L.NON_AGRESSION_PACT_REJECTED_WITH+playerName;
        	}
        } else if (isPeace()) {
        	if (accepted) {
        		start = L.PEACE_TREATY_SIGNED_WITH+playerName;
        	} else {
        		start = L.PEACE_TREATY_REJECTED_WITH+playerName;
        	}
        } else if (isTrade()) {
        	if (accepted) {
        		start = L.TRADE_AGREEMENT_SIGNED_WITH+playerName;
        	} else {
        		start = L.TRADE_AGREEMENT_REJECTED_WITH+playerName;
        	}
        } else if (isResearch()) {
        	if (accepted) {
        		start = L.RESEARCH_TREATY_SIGNED_WITH+playerName;
        	} else {
        		start = L.RESEARCH_TREATY_REJECTED_WITH+playerName;
        	}
        }
		if (wanted != null && offered != null) {
			if (start != null) {
				return start + L._AND_+L.EXCHANGE_TREATY;
			} else {
				return start;
			}
		} else if (offered != null) {
			if (start != null) {
				return start + L._AND_+L.GIFT_TREATY;
			} else {
				return start;
			}
        } else if (wanted != null){
			if (start != null) {
				return start + L._AND_+L.DEMAND_TREATY;
			} else {
				return start;
			}
        } else {
        	return "Received unknown diplomatic request from "+playerName;
        }
    }
    

	public String getTreatyString() {
    	String r = "";
    	if (war) r += L.WAR+" ";
    	if (peace) r += L.PEACE_TREATY+" ";
    	if (nonAggression) r += L.NON_AGGRESSION_TREATY+" ";
    	if (alliance) r += L.ALLIENCE_TREATY+" ";
    	if (trade) r += L.TRADE_TREATY+" ";
    	if (research) r += L.RESEARCH_TREATY+" ";
    	return r;
    }

    public String toString() {
    	return "DiplomaticProposal ["+getTreatyString()+(wanted == null ? "" : ", wanted "+wanted)+(offered == null ? "" : ", offered "+offered)+"]";
    }

	public int getQtyTreaties() {
    	int r = 0;
    	if (war) r += 1;
    	if (peace) r += 1;
    	if (nonAggression) r += 1;
    	if (alliance) r += 1;
    	if (trade) r += 1;
    	if (research) r += 1;
    	return r;
	}
	
	public boolean isExecuted() {
		return executed;
	}
	
	public void setExecuted(boolean executed) {
		this.executed = executed;
	}
	
	public void removeAll() {
    	offered.removeAll();
    	wanted.removeAll();
    	war = false;
    	peace = false;
    	nonAggression = false;
    	alliance = false;
    	trade = false;
    	research = false;
	}
	
	public boolean canAddAllianceTreaty() {
		return !war && !alliance;
	}
	
	public boolean canAddNonAgressionTreaty() {
		return !war && !nonAggression;
	}
	
	public boolean canAddPeaceTreaty() {
		return !peace;
	}
	
	public boolean canAddResearchTreaty() {
		return !war && !research;
	}
	
	public boolean canAddTradeTreaty() {
		return !war && !trade;
	}
	
	public boolean canAddWarTreaty() {
		return !war && !trade && !alliance && !nonAggression && !peace && !research;
	}

    public void addWantedPlanet(String o) {
    	if (wanted == null) wanted = new PlayerAssets();
        wanted.setPlanet(o);
    }
    
    public void addOfferedPlanet(String o) {
    	if (offered == null) offered = new PlayerAssets();
        offered.setPlanet(o);
    }
        
    public void addWantedCraft(String o) {
    	if (wanted == null) wanted = new PlayerAssets();
        wanted.addCraft(o);
    }
    
    public void addOfferedCraft(String o) {
    	if (offered == null) offered = new PlayerAssets();
        offered.addCraft(o);
    }

    public void addWantedMoney(int amount) {
    	if (wanted == null) wanted = new PlayerAssets();
        wanted.setMoney(amount);
    }

    public void addOfferedMoney(int amount) {
    	if (offered == null) offered = new PlayerAssets();
        offered.setMoney(amount);
    }
    
    public void addWantedTechnology(Technology o) {
    	if (wanted == null) wanted = new PlayerAssets();
        wanted.addTechnology(o);
    }
    
    public void addOfferedTechnology(Technology o) {
    	if (offered == null) offered = new PlayerAssets();
        offered.addTechnology(o);
    }
    
	public boolean isEmpty() {
		return offered == null && wanted == null && !hasTreaty();
	}
    
	public boolean containsProposal() {
		return offered != null || wanted != null || hasTreaty();
	}
	
	public TechnologyList getOfferedTechnologList() throws ApplicationError {
		if (offered != null) {
			return offered.getTechnologList();
		} else {
			return null;
		}
	}
	
	public TechnologyList getWantedTechnologList() throws ApplicationError {
		if (wanted != null) {
			return wanted.getTechnologList();
		} else {
			return null;
		}
	}
	
	public NameList getOfferedCraftNames() {
		if (offered == null) {
			return null;
		} else {
			return offered.getCraftNames();
		}
	}
	
	public NameList getWantedCraftNames() {
		if (wanted == null) {
			return null;
		} else {
			return wanted.getCraftNames();
		}
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (war) writer.addElement("war", war);
		if (peace) writer.addElement("peace", peace);
		if (nonAggression) writer.addElement("non-aggression", nonAggression);
		if (alliance) writer.addElement("alliance", alliance);
		if (trade) writer.addElement("trade", trade);
		if (research) writer.addElement("research", research);
		if (wanted != null) wanted.toXML(writer, "wanted");
		if (offered != null) offered.toXML(writer, "offered");		
	}

	protected String getElementName() {
		return "diplomatic-proposal";
	}
}
