/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * Contains graphical configuration information.
 */
public class GalaxyGraphicsList extends XMLBean {

	private static final String ELEMENT_NAME = "star-graphics-list";
	private ArrayList starGraphics;
	private ArrayList sunGraphics;
    
    private transient StellarObjectGraphics [] starGraphicsArray = null;
    private transient StellarObjectGraphics [] sunGraphicsArray = null;
    
    private StellarObjectGraphics blackHoleGraphics;
    private StellarObjectGraphics wormHoleGraphics;
    private StellarObjectGraphics nebulaGraphics;

	public GalaxyGraphicsList() {
	}

	public GalaxyGraphicsList(DomTree doc) throws ApplicationError {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getAllNodes();
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				if (Comp.same("star-graphics", n[i].getNodeName())) {
					setStarGraphics(new StellarObjectGraphics(n[i]));
				} else if (Comp.same("sun-graphics", n[i].getNodeName())) {
					setSunGraphics(new StellarObjectGraphics(n[i]));
				} else if (Comp.same("nebula-graphics", n[i].getNodeName())) {
					nebulaGraphics = new StellarObjectGraphics(n[i]);
				} else if (Comp.same("black-hole-graphics", n[i].getNodeName())) {
					blackHoleGraphics = new StellarObjectGraphics(n[i]);
				} else if (Comp.same("worm-hole-graphics", n[i].getNodeName())) {
					wormHoleGraphics = new StellarObjectGraphics(n[i]);
				}
			}
		}
	}

	public ArrayList getStarGraphics() { return starGraphics; }
	public void setStarGraphics(ArrayList v) { starGraphics = v; }
    
    public void setStarGraphics(StellarObjectGraphics sg) {
        if (starGraphics == null) starGraphics = new ArrayList();
        starGraphicsArray = null;
        starGraphics.add(sg);
    }

	public StellarObjectGraphics [] getStarGraphicsAsArray() {
		if (starGraphics == null) {
			return null;
		} else {
            if (starGraphicsArray == null) {
                starGraphicsArray = (StellarObjectGraphics []) starGraphics.toArray(new StellarObjectGraphics [starGraphics.size()]);
            }
            return starGraphicsArray;
		}
	}

	public StellarObjectGraphics getStarGraphicsFor(String name) {
		if (blackHoleGraphics.sameName(name)) {
			return blackHoleGraphics;
		}
		if (wormHoleGraphics.sameName(name)) {
			return wormHoleGraphics;
		}
		StellarObjectGraphics [] a = getStarGraphicsAsArray();
		for (int i=0; i<a.length; i++) {
			if ( Comp.same(a[i].getName(), name) ) {
				return a[i];
			}
		}
		return null;
	}

    public void setSunGraphics(StellarObjectGraphics sg) {
        if (sunGraphics == null) sunGraphics = new ArrayList();
        sunGraphicsArray = null;
        sunGraphics.add(sg);
    }
    
	public StellarObjectGraphics [] getSunGraphicsAsArray() {
		if (sunGraphics == null) {
			return null;
		} else {
            if (sunGraphicsArray == null) {
            	sunGraphicsArray = (StellarObjectGraphics []) sunGraphics.toArray(new StellarObjectGraphics [sunGraphics.size()]);
            }
            return sunGraphicsArray;
		}
	}

	public StellarObjectGraphics getSunGraphicsFor(String name) {
		if (blackHoleGraphics.sameName(name)) {
			return blackHoleGraphics;
		}
		if (wormHoleGraphics.sameName(name)) {
			return wormHoleGraphics;
		}
		StellarObjectGraphics [] a = getSunGraphicsAsArray();
		for (int i=0; i<a.length; i++) {
			if ( Comp.same(a[i].getName(), name) ) {
				return a[i];
			}
		}
		return null;
	}

	public StellarObjectGraphics getStellarObjectGraphics(String name, boolean zoomedIn) {
        if (zoomedIn) {
	        return getSunGraphicsFor(name);
        } else {
	        return getStarGraphicsFor(name);
        }
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		StellarObjectGraphics [] list = getStarGraphicsAsArray();
		for (int i=0; i<list.length; i++) {
			list[i].toXML(writer);
		}
		if (blackHoleGraphics != null) blackHoleGraphics.toXML(writer, "black-hole-graphics");
		if (wormHoleGraphics != null) wormHoleGraphics.toXML(writer, "worm-hole-graphics");
		if (nebulaGraphics != null) nebulaGraphics.toXML(writer, "nebula-graphics");
	}

	protected String getElementName() { return ELEMENT_NAME; }

	public void setBlackHoleGraphics(StellarObjectGraphics blackHoleGraphics) {
		this.blackHoleGraphics = blackHoleGraphics;
	}

	public StellarObjectGraphics getBlackHoleGraphics() {
		return blackHoleGraphics;
	}

	public void setNebulaGraphics(StellarObjectGraphics nebulaGraphics) {
		this.nebulaGraphics = nebulaGraphics;
	}

	public StellarObjectGraphics getNebulaGraphics() {
		return nebulaGraphics;
	}

	public StellarObjectGraphics getWormHoleGraphics() {
		return wormHoleGraphics;
	}

	public void setWormHoleGraphics(StellarObjectGraphics wormHoleGraphics) {
		this.wormHoleGraphics = wormHoleGraphics;
	}
}