/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.awt.geom.Line2D;

/**
 * 
 */
public class Geometry {
	
	private static double MIN = 0.0001;

	public static Location getIntersection(Location line1Start, Location line1End, Location line2Start, Location line2End) {
		return getIntersection(line1Start.getX(), line1Start.getY(), line1End.getX(), line1End.getY(), line2Start.getX(), line2Start.getY(), line2End.getX(), line2End.getY());
	}

	public static Location getIntersection(double x1line1, double y1line1, double x2line1, double y2line1, double x1line2, double y1line2, double x2line2, double y2line2) {
		if (!Line2D.linesIntersect(x1line1, y1line1, x2line1, y2line1, x1line2, y1line2, x2line2, y2line2)) {
			return null;
		}
		
		if ((x1line1==x1line2) && (y1line1==y1line2)) {
			return new Location(x1line1, y1line1);
		}
		if ((x1line1==x2line2) && (y1line1==y2line2)) {
			return new Location(x1line1, y1line1);
		}
		if ((x2line1==x1line2) && (y2line1==y1line2)) {
			return new Location(x2line1, y2line1);
		}
		if ((x2line1==x2line2) && (y2line1==y2line2)) {
			return new Location(x2line1, y2line1);
		}

		double dyline1 = -( y2line1 - y1line1 );
		double dxline1 = x2line1 - x1line1;

		double dyline2 = -( y2line2 - y1line2 );
		double dxline2 = x2line2 - x1line2;

		double e = -(dyline1 * x1line1) - (dxline1 * y1line1);
		double f = -(dyline2 * x1line2) - (dxline2 * y1line2);

		// compute the intersection point using  ax+by+e = 0 and cx+dy+f = 0
        // If there is more than 1 intersection point between two lines
		
		double det = dyline1 * dxline2 - dyline2 * dxline1;
		if(det < MIN && det > -MIN) {
			return null;
		} else {
			return (new Location(
					(-(e * dxline2 - dxline1 * f)/det),
					(-(dyline1 * f - dyline2 * e)/det)
					));
		}		
	}
	
	public static Location getProjectedIntersection(Location line1Start, Location line1End, Location line2Start, Location line2End) {
		return getProjectedIntersection(line1Start.getX(), line1Start.getY(), line1End.getX(), line1End.getY(), line2Start.getX(), line2Start.getY(), line2End.getX(), line2End.getY());
	}

	public static Location getProjectedIntersection(double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4) {
		double d1 = det(x1, y1, x2, y2);
		double d2 = det(x3, y3, x4, y4);
		double d3 = det(x1 - x2, y1 - y2, x3 - x4, y3 - y4);
		if (d3 < MIN && d3 > -MIN) return null;
		double x = det(d1, x1 - x2,	d2, x3 - x4)/d3;
		double y = det(d1, y1 - y2,	d2, y3 - y4)/d3;
		return new Location((int)x, (int)y);
	}
	
	private static double det(double a, double b, double c, double d) {
		return a * d - b * c;
	}

	public static int getMinDistance(double x1, double y1, double x2, double y2, double px, double py) {
		return (int) Line2D.ptLineDist(x1, y1, x2, y2, px, py);
	}

	public static Location getClosestPoint(Location lineStart, Location lineEnd, Location point) {
		return getClosestPoint(lineStart.getX(), lineStart.getY(), lineEnd.getX(), lineEnd.getY(), point.getX(), point.getY());
	}

	public static Location getClosestPoint(double x1, double y1, double x2, double y2, double px, double py) {
		double rx = 0.0;
		double ry = 0.0;
		
        // segment is a point
        if(y1 == y2 && x1 == x2) {
            return new Location(x1, y1);
        }
        // segment is a point
        if(y1 == y2 && x1 == x2) {
            rx = x1;
            ry = y1;
            return new Location(rx, ry);
        }
        // segment is horizontal
        if(y1 == y2) {
            ry = y1;
            rx = mid(x1, x2, px);
            return new Location(rx, ry);
        }
        // segment is vertical
        if(x1 == x2) {
            rx = x1;
            ry = mid(y1, y2, py);
            return new Location(rx, ry);
        }
        double dx = x2 - x1;
        double dy = y2 - y1;
        rx = dy * (dy * x1 - dx * (y1 - py)) + dx * dx * px;
        rx = rx / (dx * dx + dy * dy);
        ry = (dx * (px - rx)) / dy + py;

        if (x2 > x1) {
            if (rx > x2) {
                rx = x2;
                ry = y2;
            } else if (rx < x1) {
                rx = x1;
                ry = y1;
            }
        } else {
            if (rx < x2) {
                rx = x2;
                ry = y2;
            } else if (rx > x1) {
                rx = x1;
                ry = y1;
            }
        }
        
        return new Location(rx, ry);
    }
	
    private static double mid(double a, double b, double c) {
        if(a <= b) {
            if(b <= c)
                return b;
            else if(c <= a)
                return a;
            else
                return c;
        }
        if(b >= c)
            return b;
        else if(c >= a)
            return a;
        else
            return c;
    }
    
    /**
     * Calculate the intersection locations betwenn the rectangle and the line from source to 
     * destination. The first location will always be closer to the source location than the
     * destination location.
     */
    public static IntersectionLocations getIntersections(IRectangle rectangle, Location source, Location destination) {
    	Location firstEdge = null, secondEdge = null;
		Location l = Geometry.getIntersection(
				rectangle.getLeft(), 
				rectangle.getTop(),
				rectangle.getLeft()+rectangle.getWidth(), 
				rectangle.getTop(),
				source.getX(),
				source.getY(),
				destination.getX(),
				destination.getY() );
		if (l != null) {
			firstEdge = l;
		}
		l = Geometry.getIntersection(
				rectangle.getLeft()+rectangle.getWidth(), 
				rectangle.getTop(),
				rectangle.getLeft()+rectangle.getWidth(), 
				rectangle.getTop()+rectangle.getHeight(),
				source.getX(),
				source.getY(),
				destination.getX(),
				destination.getY() );
		if (l != null) {
			if (firstEdge == null) {
				firstEdge = l;
			} else {
				secondEdge = l;
				return _createIntersectionLocations(source, firstEdge, secondEdge);
			}
		}
		l = Geometry.getIntersection(
				rectangle.getLeft()+rectangle.getWidth(), 
				rectangle.getTop()+rectangle.getHeight(),
				rectangle.getLeft(), 
				rectangle.getTop()+rectangle.getHeight(),
				source.getX(),
				source.getY(),
				destination.getX(),
				destination.getY() );
		if (l != null) {
			if (firstEdge == null) {
				firstEdge = l;
			} else {
				secondEdge = l;
				return _createIntersectionLocations(source, firstEdge, secondEdge);
			}
		}
		l = Geometry.getIntersection(
				rectangle.getLeft(), 
				rectangle.getTop()+rectangle.getHeight(),
				rectangle.getLeft(), 
				rectangle.getTop(),
				source.getX(),
				source.getY(),
				destination.getX(),
				destination.getY() );
		if (l != null) {
			if (firstEdge == null) {
				firstEdge = l;
			} else {
				secondEdge = l;
				return _createIntersectionLocations(source, firstEdge, secondEdge);
			}
		}
		return _createIntersectionLocations(source, firstEdge, secondEdge);
    }
    
    private static IntersectionLocations _createIntersectionLocations(Location source, Location firstEdge, Location secondEdge) {
    	if (firstEdge == null && secondEdge == null) {
    		return null;
    	} else if (secondEdge != null && firstEdge == null) {
			firstEdge = secondEdge;
			secondEdge = null;
    	} else if (secondEdge != null && firstEdge != null && source.getDistanceTo(secondEdge) < source.getDistanceTo(firstEdge)) {
			Location tmp = firstEdge;
			firstEdge = secondEdge;
			secondEdge = tmp;
		}
		return new IntersectionLocations(firstEdge, secondEdge);
    }
}
