/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.io.Serializable;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.DomNode;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * Based class for all objects that have a location.
 */
public class Location extends XMLBean implements Serializable {

	/**
	 * The X position on the map.
	 */
	private int x;

	/**
	 * The Y position on the map.
	 */
	private int y;

	public Location() {
	}

	public Location(int x, int y) {
        this.x = x;
        this.y = y;
	}

	public Location(double x, double y) {
        this.x = (int)x;
        this.y = (int)y;
	}

	public Location(Location l) {
        this.x = l.getX();
        this.y = l.getY();
	}

	public Location(DomNode n) {
		x = n.getNodeValueByName_Int("x");
		y = n.getNodeValueByName_Int("y");
	}

	public int getX() { return x; }
	public int getY() { return y; }

	public void setX(int v) { x = v; }
	public void setY(int v) { y = v; }
    
    public String getHashKey() { return ""+x+"_"+y; }

	public boolean notEqual(Location o) { return o == null || !equals(o); }
	public boolean equals(Location o) { return o != null && equals(o.getX(), o.getY()); }
	public boolean equals(int x, int y) { return (this.x == x && this.y == y); }

	public void setLocation(Location o) { this.x = o.getX(); this.y = o.getY(); }

	public int getDistanceTo(Location o) { return  (o == null) ? 0 : getDistanceTo(o.getX(), o.getY()); }
	public int getDistanceTo(int dx, int dy) { return (int) Math.round(getDistanceBetween(x, y, dx, dy)); }

	public double getDistanceTo2(Location o) { return getDistanceTo2(o.getX(), o.getY()); }
	public double getDistanceTo2(int dx, int dy) { return getDistanceBetween(x, y, dx, dy); }
    
	public static double getDistanceBetween(int ox, int oy, int dx, int dy) {
		double x = new Double(dx-ox).doubleValue();
		double y = new Double(dy-oy).doubleValue();
		double dist = Math.sqrt(x*x + y*y);
		return dist;
	}

	public void shift(int dx, int dy) {
		x += dx;
		y += dy;
	}

	public void shift(LocationDelta shiftDelta) {
		if (shiftDelta == null) return;
		x += shiftDelta.getDx();
		y += shiftDelta.getDy();
	}

	/**
	 * Move in the direction of dest the distance passed in.
	 */
	public void shift(Location dest, int distance) {
		//int speed = object.getEngine().getWarpSpeed();
		double dist = getDistanceTo2(dest);
		if (dist <= distance) {
			x = dest.getX();
			y = dest.getY();
		} else {
			double dx = dest.getX() - x;
			double dy = dest.getY() - y;
			double cosTheta = dx / dist;
			double sinTheta = dy / dist;
			dx = ((double)distance) * cosTheta;
			dy = ((double)distance) * sinTheta;
			shift((int)Math.round(dx), (int)Math.round(dy));
		}
    }

	public void shiftNoLimit(Location dest, int distance) {
		double dist = getDistanceTo2(dest);
		double dx = dest.getX() - x;
		double dy = dest.getY() - y;
		double cosTheta = dx / dist;
		double sinTheta = dy / dist;
		dx = ((double)distance) * cosTheta;
		dy = ((double)distance) * sinTheta;
		shift((int)Math.round(dx), (int)Math.round(dy));
    }

	/**
	 * Get the shift delta for this Location to move in the direction of dest the distance passed in.
	 * Produced a delta object that if added to this location produces the same as shift(dest, distance).
	 */
	public LocationDelta shiftDelta(Location dest, int distance) {
		//int speed = object.getEngine().getWarpSpeed();
		double dist = getDistanceTo2(dest);
		if (dist <= distance) {
			return new LocationDelta(dest.getX() - x, dest.getY() - y);
		} else {
			double dx = dest.getX() - x;
			double dy = dest.getY() - y;
			double cosTheta = dx / dist;
			double sinTheta = dy / dist;
			dx = ((double)distance) * cosTheta;
			dy = ((double)distance) * sinTheta;
			return new LocationDelta((int)Math.round(dx), (int)Math.round(dy));
		}
    }

	/**
	 * Move this location to dest.
	 */
	public void shiftTo(Location dest) {
        x = dest.getX();
        y = dest.getY();
    }

    /**
     * Move this location in the direction for a distance of.
     * @param distance The distance measures in pixels to move this location.
     * @param direction The direction to move this location measured in degrees (0 - 359)
     * where 0 is north.
     */
	public void shiftInDirection(int direction, int distance) {
        // cosTheta = dx / distance;
        // sinTheta = dy / distance;
        double theta = Math.toRadians(direction-90);
        double cosTheta = Math.cos(theta);
        double sinTheta = Math.sin(theta);
        int dx = (int)(cosTheta * distance);
        int dy = (int)(sinTheta * distance);
        shift(dx, dy);
    }

	public void shiftInDirectionOneSpace(int direction) {
        double theta = Math.toRadians(direction-90);
        double cosTheta = Math.cos(theta);
        double sinTheta = Math.sin(theta);
        int dx = (int)(cosTheta);
        int dy = (int)(sinTheta);
        if (dx == 0 && dy == 0) {
            dx = (int)(cosTheta * 2);
            dy = (int)(sinTheta * 2);
        }
        shift(dx, dy);
    }
	
    /**
     * Move this location in the direction for a distance of.
     * @param distance The distance measures in pixels to move this location.
     * @param direction The direction to move this location measured in degrees (0 - 359)
     * where 0 is north.
     */
	public void shiftInDirectionWithDouble(double direction, int distance) {
        // cosTheta = dx / distance;
        // sinTheta = dy / distance;
        double theta = Math.toRadians(direction-90);
        double cosTheta = Math.cos(theta);
        double sinTheta = Math.sin(theta);
        int dx = (int)(cosTheta * distance);
        int dy = (int)(sinTheta * distance);
        shift(dx, dy);
    }

	/**
	 * Same as getDirectionFor() but also takes the existing direction so if this location is 
	 * the SAME as the target location the existing direction is returned. i.e. no change.
	 */
	public int getDirectionFor(int existingDirection, Location l) {
		if (l.getX() == x && l.getY() == y) {
			return existingDirection;
		} else {
			return getDirectionFor(l.getX(), l.getY());
		}
	}
	
    /**
     * Alternative to getDirectionFor(int x, int y);
     */
	public int getDirectionFor(Location l) {
        return getDirectionFor(l.getX(), l.getY());
    }
    
	/**
	 * @return the angle in degrees a craft would need as a heading to move
	 * to location px,py. The return value is between 0 and 359.
	 */
	public int getDirectionFor(int px, int py) {
		double dx = (double)(x - px);
		double dy = (double)(y - py);
        // tanTheta = dx / dy;
		if (dx >= 0 && dy < 0) {
			double tanTheta = dx / dy;
			double degrees = Math.toDegrees(Math.atan(tanTheta));
			if (degrees < 0) degrees = -degrees;
			return 180 + (int) degrees;
		} else if (dx > 0 && dy >= 0) {
			double tanTheta = dy / dx;
			double degrees = Math.toDegrees(Math.atan(tanTheta));
			if (degrees < 0) degrees = -degrees;
			return 270 + (int) degrees;
		} else if (dx <= 0 && dy > 0) {
			double tanTheta = dx / dy;
			double degrees = Math.toDegrees(Math.atan(tanTheta));
			if (degrees < 0) degrees = -degrees;
			return (int) degrees;
		} else {
			double tanTheta = dy / dx;
			double degrees = Math.toDegrees(Math.atan(tanTheta));
			if (degrees < 0) degrees = -degrees;
			return 90 + (int) degrees;
        }
	}
	
    /**
     * Alternative to getDirectionFor(int x, int y);
     */
	public double getAccurateDirectionFor(Location l) {
        return getAccurateDirectionFor(l.getX(), l.getY());
    }
    
	/**
	 * @return the angle in degrees a craft would need as a heading to move
	 * to location px,py. The return value is between 0 and 359.
	 */
	public double getAccurateDirectionFor(int px, int py) {
		double dx = (double)(x - px);
		double dy = (double)(y - py);
        // tanTheta = dx / dy;
		if (dx >= 0 && dy < 0) {
			double tanTheta = dx / dy;
			double degrees = Math.toDegrees(Math.atan(tanTheta));
			if (degrees < 0) degrees = -degrees;
			return 180 + degrees;
		} else if (dx > 0 && dy >= 0) {
			double tanTheta = dy / dx;
			double degrees = Math.toDegrees(Math.atan(tanTheta));
			if (degrees < 0) degrees = -degrees;
			return 270 + degrees;
		} else if (dx <= 0 && dy > 0) {
			double tanTheta = dx / dy;
			double degrees = Math.toDegrees(Math.atan(tanTheta));
			if (degrees < 0) degrees = -degrees;
			return degrees;
		} else {
			double tanTheta = dy / dx;
			double degrees = Math.toDegrees(Math.atan(tanTheta));
			if (degrees < 0) degrees = -degrees;
			return 90 + degrees;
        }
	}
	
	/**
	 * @return A positive number being the difference in degrees between the two directions d1 and d2.
	 */
	public static int getDirectionDeltaPos(int d1, int d2) {
		int delta = Comp.pos(d1-d2);
		if (delta > 180) {
			delta = -(360 - delta);
		}
		return delta;
	}
	
	/**
	 * @return The direction change required to change from d1 to d2 where a negative number implies anti-clockwise spin and a positive number a clockwise spin.
	 */
	public static int getDirectionDelta(int d1, int d2) {
		int delta = d1-d2;
		if (delta > 180) {
			delta = -(180 - delta%180);
		} else if (delta < -180) {
			delta = 180 - (-delta)%180;
		}
		return delta;
	}
	
	/**
	 * Rotate the current location degrees where this location is on the circumference 
	 * of a circle with centre x,y.
	 * E.G. Rotate the this location marked by a character * around the centre marked
	 * using the character x by 90 degrees.
	 * 
	 *  		*											
	 *  						BECOMES							*
	 *  			x									x
	 */
	public void rotateAround(double degrees, int centerX, int centerY) {
		x -= centerX;
		y -= centerY;
		double angle = Math.toRadians(degrees);
		double xtemp = (x * Math.cos(angle)) - (y * Math.sin(angle));
		y = (int) Math.round((x * Math.sin(angle)) + (y * Math.cos(angle)));
		x = (int) Math.round(xtemp);
		x += centerX;
		y += centerY;
	}
	
	public Rectangle createRectangleAround(int size) {
		return new Rectangle(this, size);
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("x", x);
		writer.addElement("y", y);
	}

	protected String getElementName() { return "location"; }
    
    public String toString() { return "("+x+","+y+")"; }

	public Area getArea(int size) {
		return new Area(x-size, y-size, x+size, y+size);
	}

}