/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

public class NameList extends XMLBean implements Serializable {

	private ArrayList<String> names;

	public ArrayList getNames() { return names; }
	public void setNames(ArrayList v) { names = v; }

	public String [] getNamesAsArray() {
        if (names == null || names.size() == 0) {
            return null; 
        } else {
            return (String []) names.toArray(new String[names.size()]); 
        }
    }
	
	public void add(String name) {
        setName(name);
    }
    
	public void setName(String name) {
		if (names == null) {
			names = new ArrayList();
		}
		names.add(name);
	}

	public NameList() {
	}
	
	public NameList(String [] names) {
		if (names != null) {
			for (int i = 0; i < names.length; i++) {
				add(names[i]);
			}
		}
	}
    
    public NameList(ArrayList<String> names) {
		if (names != null) {
			for (String name : names) {
				add(name);
			}
		}
	}
    
	public NameList(TechnologyList tl) {
		if (tl != null) {
			Technology[] all = tl.getAllAsArray();
			if (all != null) {				
				for (int i = 0; i < all.length; i++) {
					add(all[i].getName());
				}
			}
		}
	}
    
	public NameList(NameList names) {
		if (names != null) {
			String[] all = names.getNamesAsArray();
			if (all != null) {				
				for (int i = 0; i < all.length; i++) {
					add(all[i]);
				}
			}
		}
	}
	
	public NameList(String name) {
		if (name != null) {
			add(name);
		}
	}
	
	public NameList(String name1, String name2) {
		if (name1 != null)  add(name1);
		if (name2 != null)  add(name2);
	}
	
	public NameList(String name1, String name2, String name3) {
		if (name1 != null)  add(name1);
		if (name2 != null)  add(name2);
		if (name3 != null)  add(name3);
	}
	
	public boolean contains(String name) {
        if (names == null) return false;
        Iterator i = names.iterator();
        while (i.hasNext()) {
            String v = (String) i.next();
            if (Comp.same(v, name)) {
                return true;
            }
        }
        return false;
    }

    public void remove(NameList nl) {
		if (names == null || nl == null) {
			return;
		}
		String[] na = nl.getNamesAsArray();
		if (na == null) return;
		for (int i = 0; i < na.length; i++) {
			removeAll(na[i]);
		}
    }
    
	public void removeAll(String name) {
		if (names != null) {
			String[] na = getNamesAsArray();
			if (na == null) return;
			for (int i = 0; i < na.length; i++) {
				if (Comp.same(name, na[i])) {
					names.remove(na[i]);
				}
			}
		}
	}

	public void removeFirst(String name) {
		if (names != null) {
			String[] na = getNamesAsArray();
			if (na == null) return;
			for (int i = 0; i < na.length; i++) {
				if (Comp.same(name, na[i])) {
					names.remove(na[i]);
					return;
				}
			}
		}
	}

	public void remove(NamedObject name) {
		if (names != null) {
			String[] na = getNamesAsArray();
			if (na == null) return;
			for (int i = 0; i < na.length; i++) {
				if (Comp.same(name, na[i])) {
					names.remove(na[i]);
				}
			}
		}
	}
	

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (names != null) {
			Iterator i = names.iterator();
			while (i.hasNext()) {
				String name = (String) i.next();
				writer.addElement("name", name);
			}
		}
	}

	protected String getElementName() { return "name-list"; }

	public int getQuantity() {
		if (names != null) {
			return names.size();
		} else {			
			return 0;
		}
	}
	
	public boolean same(NameList other) {
        if (names == null) return false;
        int qty = names.size();
        if (qty != other.getQuantity()) return false;
        for (int i = 0; i < qty; i++) {
        	String v1 = names.get(i);
        	String v2 = other.getAtIndex(i);
			if (!Comp.same(v1, v2)) {
				return false;
			}
		}
        return true;
	}

    
	public int getQuantity(String name) throws ApplicationError {
		int r = 0;
		if (names != null) {
			String[] na = getNamesAsArray();
			if (na != null) {
				for (int i = 0; i < na.length; i++) {
					if ( Comp.same(na[i], name) ) {
						r++;
					}
				}
			}
		}
		return r;
	}

	private String getAtIndex(int index) {
		if (names == null || index >= names.size()) return null;
		return (String) names.get(index);
	}
	
	public String getNamesSpaceSeparated() {
		return Comp.toString(getNamesAsArray(), " ");
	}
	
	public String getNamesSeparated(String separator) {
		return Comp.toString(getNamesAsArray(), separator);
	}
	
	public String toString() {
		return getNamesSeparated(",");
	}

}