/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

import mot.client.deligates.NamedObjectDelegate;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This is one of the generic base classes in the game. Most of the objects in the game are
 * sub classes of this, such as Craft, Planet, Solar System etc. It represents any object in
 * the game that has a name.
 * 
 * This also provides basic support for changing a name. See oldNames for details.
 */
public abstract class NamedObject extends XMLBean implements NamedObjectI, Serializable {

    /**
     * This is a list of old names that this object used to be called.
     * Because of name references, we must keep track of old names. E.g. A SolarSystem
     * reference of a User Delegate may not know that the name has changed and search for
     * an object with the old name. The method didHaveName() may be used to determine 
     * if a given name was used in the past.
     * This is a list of String.
     */
    private transient ArrayList oldNames;

    /**
     * This is the current name of this object.
     */
	private String name;

	public String getName() { return name; }
	public void setName(String v) { name = v; }

	public NamedObject() {
	}

	public NamedObject(NamedObject o) {
        name = o.getName();
	}

	public NamedObject(String name) {
		this.name = name;
	}

	public boolean sameName(String v) {
		if (v == name) return true;
		return v.equals(name);
	}
	
    public boolean is(String v) {
		if (v == name) return true;
		return v.equals(name);
	}



	public boolean notEqual(NamedObject o) {
		if (o == null) {
			return true;
		} else {
			return !(o.sameName(getName()));
		}
	}

	public boolean equals(NamedObject o) {
		if (o == null) {
			return false;
		} else {
			return o.sameName(getName());
		}
	}

	public boolean equals(NamedObjectDelegate o) {
		if (o == null) {
			return false;
		} else {
			return sameName(o.getName());
		}
	}


    /**
     * This changes the name of this object to name. A record is kept until this 
     * game is serialized of the old name. Use the method didHaveName() when finding
     * an object by a name that does not exist. This will allow objects to be located
     * by old names.
     * @param newName The name to change this object's name to.
     */
    public void changeNameTo(String newName) {
         if (oldNames == null) oldNames = new ArrayList();
         oldNames.add(name);
         name = newName;
    }
    
    /**
     * @return True if this object has the name in the past. 
     * @param name The name this object may have had.
     */
    public boolean didHaveName(String name) {
        if (oldNames == null) return false;
        Iterator i = oldNames.iterator();
        while (i.hasNext()) {
            String oldName = (String) i.next();
            if (Comp.same(oldName, name)) {
                return true;
            }
        }
        return false;
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", name);
	}

    public void setOldNames(ArrayList oldNames) {
        this.oldNames = oldNames;
    }

    public ArrayList getOldNames() {
        return oldNames;
    }

    public String toString() { return name; }
}