/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import mot.common.util.Formatter;

/**
 * Represents the details of an objects movement from on point to another 
 * point.
 * 
 * When a craft moves on a battle field it may pass through incumbents. Therefore
 * its speed may change or events may occur whole performing a single move. Therefore
 * a single movement is constructed from one or more ObjectShifts. Each ObjectShift 
 * defines a origin and destination location and a speed at which the craft should move 
 * from the origin to the destination.
 * 
 * An event may also occur between one movement shift and another, e.g. damage when 
 * entering an asteroid field. An object shift 
 *
 */
public class ObjectShift implements ObjectMovement {

	/**
	 * The location the object has moved to in this shift.
	 */
	private Location destination;
	
	/**
	 * The state of this craft at the start of this shift.
	 */
	private CraftState state;
	
	/**
	 * The percentage (0.0 to 1.0) of the objects move taken by this shift.
	 */
	private double movePercent;

	/**
	 * The change in speed to apply to the animation during this shift.
	 * No change implies a value of 1.0. A value of 0.5 would mean move
	 * at half the normal speed for this craft.
	 */
	private double speedModification;
	
	/**
	 * This is any damage that has been inflicted on this craft / BO during the shift.
	 * This is normally null - BUT, if the craft has passed through an incumbent then
	 * damage could have been inflicted during the move.
	 */
	private Damage damage;

	public ObjectShift(Location destination, double movePercent, CraftState state, double speedModification) {
		super();
		this.destination = new Location(destination);
		this.state = state;
		this.movePercent = movePercent;
		this.speedModification = speedModification;
	}

	public ObjectShift(Location destination, double movePercent, CraftState state) {
		super();
		this.destination = new Location(destination);
		this.state = state;
		this.movePercent = movePercent;
		this.speedModification = 1.0;
	}

	public Location getDestination() {
		return destination;
	}

	public Location getOrigin() {
		return state.getLocation();
	}

	public void setDestination(Location destination) {
		this.destination = destination;
	}

	public double getSpeedModification() {
		return speedModification;
	}

	public void setSpeedModification(double speedModification) {
		this.speedModification = speedModification;
	}

	public double getMovePercent() {
		return movePercent;
	}

	public void setMovePercent(double movePercent) {
		this.movePercent = movePercent;
	}

	public Damage getDamage() {
		return damage;
	}

	public void setDamage(Damage damage) {
		this.damage = damage;
	}

	public CraftState getState() {
		return state;
	}

	public void setState(CraftState state) {
		this.state = state;
	}

	public String toString() {
		if (state != null) {
			return "Move "+state.getLocation()+"=>"+destination+" move%="+Formatter.fraction(movePercent)+" speedMod="+speedModification+" dam="+damage;
		} else {
			return "Move dest="+destination+" move%="+Formatter.fraction(movePercent)+" speedMod="+speedModification+" dam="+damage;
		}
	}

}
