/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import mot.common.util.ApplicationError;
import mot.common.xml.XMLInput;

/**
 * Contains graphical image configuration information used to display a planet on 
 * the screen.
 * 
 * The image used to display a planet depends on the planets terraform level and
 * the quantity of images in the planets animation cycle. This configuration specifies
 * the minimum (inclusive) terraform level to use this configuration (image).
 * 
 * The image file name is constructed from the picture + animation stage + picture
 * type. The animation stage is a including number in the stages contained here.
 * The stages are repeated providing animation!
 */
public class PlanetGraphics implements XMLInput {

	/**
	 * This is the min (inclusive) amount of terraform required by a 
	 * planet to use this image.
	 * Say there are two planets graphics configurations - One for 
	 * tarraform 100 and one for tarraform 120. Planets of tarraform 
	 * 100 to 119 would use the config with tarraform 100 while planets 
	 * with tarraform 120 or more would use the config with tarraform 120.
	 */
	private double minTerraform;

	/**
	 * This is the max (exclusive) amount of terraform required by a 
	 * planet to use this image.
	 */
	private double maxTerraform;
	
	/**
	 * picture of this planet
	 */
	private String image;

	/**
	 * The image type of the picture
	 */
	private String imageType;

	/**
	 * The screen space consumed by this hull.
	 */
	private Size imageSize;

	/**
	 * Comma separated list of image indexes. 
	 * Must either have qtyImages of stages attribute to animate!
	 */
	private String stages;
	
	private transient String [] _stages;

	public PlanetGraphics(String picture, Size size) {
		this.image = picture;
		this.imageSize = size;
	}

	public PlanetGraphics() {
	}

	public String getImage() {
		return image;
	}

	public void setImage(String image) {
		this.image = image;
	}

	public String getImageType() {
		return imageType;
	}

	public void setImageType(String imageType) {
		this.imageType = imageType;
	}

	public String getStages() {
		return stages;
	}

	public void setStages(String stages) {
		_stages = null;
		this.stages = stages;
	}
	
    public String [] getStagesMandatoryAsSplit() throws ApplicationError {
    	if (_stages != null) {
    		return _stages;
    	}
    	if (stages == null) throw new ApplicationError("PlanetGraphics "+minTerraform+" does not have stages");
    	_stages = stages.split(",");
    	return _stages;
    }

    public String [] getStagesAsArray() {
    	if (_stages != null) {
    		return _stages;
    	}
        if (stages == null) {
            return null;
        }
    	_stages = stages.split(",");
    	return _stages;
    }

	public double getMaxTerraform() {
		return maxTerraform;
	}

	public void setMaxTerraform(double maxTerraform) {
		this.maxTerraform = maxTerraform;
	}

	public double getMinTerraform() {
		return minTerraform;
	}

	public void setMinTerraform(double minTerraform) {
		this.minTerraform = minTerraform;
	}

	public Size getImageSize() {
		return imageSize;
	}

	public void setImageSize(Size imageSize) {
		this.imageSize = imageSize;
	}

}