/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;
import java.io.Serializable;

import mot.common.lang.L;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.vo.TechnologyListVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This is a container for a things that a player owns that are considered assets of 
 * that player such as;
 *      Technology
 *      Money
 *      Systems
 * A player may give or swap their assets to another player. For details of how this
 * is used see the give command and the swap command.
 */
public class PlayerAssets extends XMLBean implements Serializable {

    /**
     * A list of technologies on offer.
     */
    private TechnologyListVO technologies;
    
    /**
     * An amount of money on offer.
     */
    private int money;
    
    /**
     * A planet on offer.
     */
    private String planet;
    
    /**
     * A list of craft names
     */
    private NameList craftNames;

    public PlayerAssets() {
    }

    public PlayerAssets(int money) throws ApplicationError {
        this((TechnologyListVO)null, money, null);
    }
    
    public PlayerAssets(String planet) throws ApplicationError {
        this((TechnologyListVO)null, 0, planet);
    }
    
    public PlayerAssets(NameList crafts) {
        craftNames = crafts;
    }
    
    public PlayerAssets(Technology technology) throws ApplicationError {
        this(technology == null ? null : new TechnologyListVO(technology), 0, null);
    }
    
    public PlayerAssets(Technology technology, int money, String planet) throws ApplicationError {
        this(technology == null ? null : new TechnologyListVO(technology), money, planet);
    }
    
    public PlayerAssets(TechnologyList technologies, int money, String planet) throws ApplicationError {
        this(technologies == null ? null : new TechnologyListVO(technologies), money, planet);
    }
    
    public PlayerAssets(TechnologyListVO technologies, int money, String planet) throws ApplicationError {
        this.technologies = new TechnologyListVO(technologies);
        this.money = money;
        this.planet = planet;
    }
    
    public PlayerAssets(Technology technology1, Technology technology2) {
   	 	addTechnology(technology1);
   	 	addTechnology(technology2);
	}

	public PlayerAssets(PlayerAssets a) {
		technologies = a.getTechnologies();
		money = a.getMoney();
		planet = a.getPlanet();
	}

	public boolean hasAssets() {
        return money != 0 || planet != null || (technologies != null && technologies.getQuantity() > 0) || (craftNames != null && craftNames.getQuantity() > 0);
    }

    public int getQuantity() {
        int qty = 0;
        if (money != 0) qty++;
        if (planet != null) qty++;
        if (technologies != null) qty += technologies.getQuantity();
        return qty;
    }

    public void removeAll() {
        money = 0;
        planet = null;
        technologies = null;
        craftNames = null;
    }
    
    public void removeTechnology(Technology tech) {
        if (technologies != null) {
            technologies.remove(tech);
        }
    }

    public void addTechnology(Technology t) {
    	if (t == null) return;
        if (technologies == null) {
            technologies = new TechnologyListVO();
        }
        technologies.add(t);
    }

    public void setTechnologies(TechnologyListVO technologies) {
        this.technologies = technologies;
    }


    public TechnologyListVO getTechnologies() {
    	if (technologies == null || technologies.getQuantity() == 0) {
    		return null;
    	} else {
    		return technologies;
    	}
    }

    public TechnologyList getTechnologList() throws ApplicationError {
    	if (technologies == null) {
    		return null;
    	} else {
    		return technologies.getAllAsList();
    	}
    }

    public void setMoney(int money) {
        this.money = money;
    }

    public int getMoney() {
        return money;
    }

    public void setPlanet(String planet) {
        this.planet = planet;
    }

    public String getPlanet() {
        return planet;
    }

	public String getNameList() {
		StringBuilder sb = null;
        if (money > 0) {
        	sb = new StringBuilder();
        	sb.append("Money ");
        	sb.append(money);
        }
        if (planet != null) {
        	if (sb == null) {
        		sb = new StringBuilder();
        	} else {
        		sb.append(", ");
        	}
        	sb.append("Planet ");
        	sb.append(planet);
        }
        if (technologies != null) {
        	if (sb == null) {
        		sb = new StringBuilder();
        	} else {
        		sb.append(", Technology ");
        	}
        	sb.append(technologies.getFullNamesAsString());
        }
        if (craftNames != null) {
        	if (sb == null) {
        		sb = new StringBuilder();
        	} else {
        		sb.append(", Craft ");
        	}
        	sb.append(craftNames.getNamesSeparated(", "));
        }
    	if (sb == null) {
    		return null;
    	} else {
    		return  sb.toString();
    	}
	}
	
    public String getDescription() throws ApplicationError {
    	return "" +
	       ((money > 0) ? L.MONEY_+money : "") +
	       ((technologies != null) ? L.TECHNOLOGY_+technologies.getNames(", ") : "") +
	       ((planet != null) ? L.PLANET+planet : "") +
	       ((craftNames != null) ? L.CRAFT_+craftNames : "") ;
    }
    
	public String toString() {
		return getNameList();
	}

	public boolean isEmpty() {
		return (technologies == null || technologies.getQuantity() == 0) && money == 0 && planet == null && (craftNames == null || craftNames.getQuantity() == 0);
	}

	public NameList getCraftNames() {
		return craftNames;
	}

	public void setCraftNames(NameList craftNames) {
		this.craftNames = craftNames;
	}
	
	public void addCraft(String name) {
		if (craftNames == null) craftNames = new NameList();
		craftNames.add(name);
	}

	public void removeCraft(String name) {
		if (craftNames == null) return;
		craftNames.removeFirst(name);
		if (craftNames.getQuantity() == 0) craftNames = null;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (technologies != null) technologies.toXML(writer,"technologies");
		if (money > 0) writer.addElement("money", money);
		if (planet != null) writer.addElement("planet", planet);
		if (craftNames != null) craftNames.toXML(writer, "craft-names");
	}

	protected String getElementName() {
		return "player-assets";
	}

}