/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.util.ArrayList;

import mot.common.lang.L;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOption;
import mot.common.setup.SetupOptions;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Laboratories;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;

/**
 * The bonuses this player's characteristics provides to its owner.
 */
public class PlayerCharacterBasic extends NamedObject {

	// These values can be found in relics.xml
	private static final String BIO_CONFORMITY = "Bio Conformity";
	private static final String BIO_GENETIC_TRANSFORMATION = "Bio Genetic Transformation";
	private static final String ELECTROLITIC_AUGUMENTATION = "Electrolitic Augumentation";
	private static final String CRYSTALINE_AUGUMENTATED_METALS = "Crystaline Augumentated Metals";

	// These values can be found in ancient_alignment_gifts.xml
	private static final String PSI_EMHANCEMENT = "Psi Enhancment";
	private static final String CLOAKING_SPACE_INVERTER = "Cloaking Space Inverter";

	/**
	 * Increase in production on planets
	 * A value of 0.1 implies an increase of 10%
	 */
	private double productionBonus;
	private double productionLevelBonus;

	/**
	 * Increase in research
	 * A value of 0.1 implies an increase of 10%
	 */
	private double researchBonus;
	private double researchLevelBonus;

	/**
	 * Increase in developing a planet's factories
	 * A value of 0.1 implies an increase of 10%
	 */
	private double developmentBonus;
	private double developmentLevelBonus;

	/**
	 * Increase in population growth
	 * A value of 0.1 implies an increase of 10%
	 */
	private double reproductionBonus;

	/**
	 * Bonus offered to the qty factory units allowed per population unit.
	 * A value of 0.1 implies an increase of 10%
	 */
	private double factoryPerPopulationBonus;

	/**
	 * The increase in ecology work performed per turn for this player.
	 * A value of 0.1 implies an increase of 10%
	 */
	private double ecologyBonus;

    /**
     * The Maximum population bonus for players with this character.
	 * A value of 0.1 implies an increase of 10%
     */
    private double maxPopulationBonus;

	/**
	 * The percentage of output produced by a planet that is used to terraform.
     * Note this is in addition to the minimum ecology percent that is automatically allocated.
	 */
	private double playerEcologyPercent = 0.1;

	/**
	 * The percentage of output produced by a planet that is used to develop.
     * This will only be allocated when there is development work to be performed on a planet.
	 */
	private double playerDevelopmentPercent = 0.1;

	/**
	 * The percentage of the players production output put into research after ecology
     * development and money percentages have been removed.
     * playerResearchPercent + playerProductionPercent = 100.
	 */
	private double playerResearchPercent = 0.5;
	private double playerIsolationistResearchPercent = 0.75;

	/**
	 * The percentage of the players production output put into production after ecology
     * development and money percentages have been removed.
     * playerResearchPercent + playerProductionPercent = 100.
	 */
	private double playerProductionPercent = 0.5;
	private double playerIsolationistProductionPercent = 0.25;

	/**
	 * The percentage of the players production output not used (put into money).
	 */
	private double playerMoneyPercent = 0.05;


	/**
	 * The Psi rating for this player. Each craft owned by this player emits a 
	 * background psi wave of this power.
	 */
	private int psiRating;


	/**
	 * Bonus to turning speed of craft in battle.
	 * A value of 0.1 implies an increase of 10%
	 */
	private double battleTurningBonus;

	/**
	 * Bonus to speed of craft in battle.
	 * A value of 0.1 implies an increase of 10%
	 */
	private double battleSpeedBonus;

	/**
	 * Bonus to maximum hit points of all crafts owned by this player.
	 * A value of 0.1 implies an increase of 10%
	 */
	private double maxDamageBonus;

	/**
	 * Bonus to speed of craft on galaxy map (between stellar objects).
	 * A value of 0.1 implies an increase of 10%
	 */
	private double speedBonus;

	/**
	 * This is the amount of cloak added to all crafts owned by this player. For details on
     * how cloak works see
	 */
	private int cloakLevel;

    /**
     * Bonus to the initiative of a craft in battle.
	 * A value of 0.1 implies an increase of 10%
     **/
	private double initiativeBonus;

    /**
     * Bonuses to trading this player receives.
	 * A value of 0.1 implies an increase of 10%
     */
    private double tradingBonus;

    /**
     * The maximum amount of trade routes this player can support between players
     */
    private int maxPlayerTradeAgreements;

    /**
     * The maximum amount of trade routes this player can support between the players planets (per planet).
     */
    private int maxSolarSystemTradeRoutes;

    /**
     * Bonuses to ground combat operations.
	 * A value of 0.1 implies an increase of 10%
     */
    private double assaultBonus;
    
    /**
     * Bonus to spying operations.
	 * A value of 0.1 implies an increase of 10%
     */
    private double spyBonus;
    
    /**
     * The charisma for this player.
     * This is not used in the game - Charisma is used by MotFactory to calculate the base relationships
     * stored in each player character. It is for reference purposes only.
     */
    private int charisma;
    
    /**
     * If true then this player is resourceful. Resourceful players are know to be creative thinkers
     * and tend to get themselves out of tricky situations. Resource players has a chance or repairing 
     * a crafts damage when it is low, and a chance of restoring a crafts shields when they are low.
     */
    private boolean resourceful;
    
    /**
     * Bonus to the influence of this player.
	 * A value of 0.1 implies an increase of 10%
     **/
    private double eloquenceBonus;

    /**
     * This is the type of immunity this player enjoys or null if this player is not immune to any type 
     * of attack.
     */
    private ArrayList<Immunity> immunities = new ArrayList<Immunity>();
    
    /**
     * This is a list of technologies that this player is not allowed to discover. All technologies in 
     * this list should never appear in the players 'available for research list'. However a player may
     * acquire these technologies from other players either by using research treaties or espionage.
     */
    private NameList missedTechnologies;
    private transient TechnologyList _missedTechnologies;
    
    /**
     * The probability that this player will discover each technology he salvages from a battle.
	 * A value of 0.1 implies an increase of 10%
     **/
    private double chanceToDiscoverSalvagedTech = 0.5;

    public PlayerCharacterBasic() {
	}

	public PlayerCharacterBasic(PlayerCharacterBasic pc) {
		super(pc.getName());
		this.productionBonus = pc.getProductionBonus();
		this.productionLevelBonus = pc.getProductionLevelBonus();
		this.researchBonus = pc.getResearchBonus();
		this.researchLevelBonus = pc.getResearchLevelBonus();
		this.developmentBonus = pc.getDevelopmentBonus();
		this.developmentLevelBonus = pc.getDevelopmentLevelBonus();
		this.reproductionBonus = pc.getReproductionBonus();
		this.factoryPerPopulationBonus = pc.getFactoryPerPopulationBonus();
		this.ecologyBonus = pc.getEcologyBonus();
		this.maxPopulationBonus = pc.getMaxPopulationBonus();
		this.playerEcologyPercent = pc.getPlayerEcologyPercent();
		this.playerDevelopmentPercent = pc.getPlayerDevelopmentPercent();
		this.playerResearchPercent = pc.getPlayerResearchPercent();
		this.playerIsolationistResearchPercent = pc.getPlayerIsolationistResearchPercent();
		this.playerProductionPercent = pc.getPlayerProductionPercent();
		this.playerIsolationistProductionPercent = pc.getPlayerIsolationistProductionPercent();
		this.playerMoneyPercent = pc.getPlayerMoneyPercent();
		this.psiRating = pc.getPsiRating();
		this.battleTurningBonus = pc.getBattleTurningBonus();
		this.battleSpeedBonus = pc.getBattleSpeedBonus();
		this.maxDamageBonus = pc.getMaxDamageBonus();
		this.speedBonus = pc.getSpeedBonus();
		this.cloakLevel = pc.getCloakLevel();
		this.initiativeBonus = pc.getInitiativeBonus();
		this.tradingBonus = pc.getTradingBonus();
		this.assaultBonus = pc.getAssaultBonus();
		this.spyBonus = pc.getSpyBonus();
		this.maxPlayerTradeAgreements = pc.getMaxPlayerTradeAgreements();
		this.maxSolarSystemTradeRoutes = pc.getMaxSolarSystemTradeRoutes();
		this.charisma = pc.getCharisma();
		this.resourceful = pc.isResourceful();
		this.eloquenceBonus = pc.getEloquenceBonus();
		this.immunities = pc.getImmunitiesList();
		this.missedTechnologies = pc.getMissedTechnologies();
		this._missedTechnologies = null;
		this.chanceToDiscoverSalvagedTech = pc.getChanceToDiscoverSalvagedTech();
	}

	public void setProductionBonus(double v) { productionBonus = v; }
	public void setProductionLevelBonus(double v) { productionLevelBonus = v; }
	public void setResearchBonus(double v) { researchBonus = v; }
	public void setResearchLevelBonus(double v) { researchLevelBonus = v; }
	public void setDevelopmentBonus(double v) { developmentBonus = v; }
	public void setDevelopmentLevelBonus(double v) { developmentLevelBonus = v; }
	public void setReproductionBonus(double v) { reproductionBonus = v; }
	public void setFactoryPerPopulationBonus(double v) { factoryPerPopulationBonus = v; }
	public void setEcologyBonus(double v) { ecologyBonus = v; }
    public void setMaxPopulationBonus(double v) { maxPopulationBonus = v; }

	public void setPlayerEcologyPercent(double v) { playerEcologyPercent = v; }
	public void setPlayerDevelopmentPercent(double v) { playerDevelopmentPercent = v; }
	public void setPlayerResearchPercent(double v) { playerResearchPercent = v; }
	public void setPlayerIsolationistResearchPercent(double v) { playerIsolationistResearchPercent = v; }
	public void setPlayerProductionPercent(double v) { playerProductionPercent = v; }
	public void setPlayerIsolationistProductionPercent(double v) { playerIsolationistProductionPercent = v; }
	public void setPlayerMoneyPercent(double v) { playerMoneyPercent = v; }

	public void setPsiRating(int v) { psiRating = v; }
	
	public void setMissedTechnologies(TechnologyList v) { missedTechnologies = v == null ? null : v.getFullNamesList(); _missedTechnologies = null; }
	public void setMissedTechnologies(GeneralTechnologyList v ) { missedTechnologies = v == null ? null : v.getFullNamesList(); _missedTechnologies = null; }
	public void setMissedTechnologies(NameList v) { missedTechnologies = v; _missedTechnologies = null; }
	public void setMissedTechnology(String v) { addMissedTechnology(v); }
	public void addMissedTechnology(Technology v) { addMissedTechnology(v.getFullName()); }
	public void addMissedTechnology(String v) { 
		if (missedTechnologies == null) missedTechnologies = new NameList(); 
		missedTechnologies.add(v); 
		_missedTechnologies = null; 
	}
	public void flushMissedTechnologiesCache() { _missedTechnologies = null; }
    
	public void setBattleTurningBonus(double v) { battleTurningBonus = v; }
	public void setBattleSpeedBonus(double v) { battleSpeedBonus = v; }
	public void setInitiativeBonus(double v) { initiativeBonus = v; }
	public void setSpeedBonus(double v) { speedBonus = v; }
	public void setMaxDamageBonus(double v) { maxDamageBonus = v; }
	public void setCloakLevel(int v) { cloakLevel = v; }
    public void setTradingBonus(double tradingBonus) { this.tradingBonus = tradingBonus; }
    public void setAssaultBonus(double assaultBonus) { this.assaultBonus = assaultBonus; }
    public void setSpyBonus(double spyBonus) { this.spyBonus = spyBonus; }

    public void setResourceful(boolean v) { resourceful = v; }
    public void setImmunity(Immunity v) { if (v == null) immunities = new ArrayList<Immunity>(); else immunities.add(v); }
    public void setImmunityBio(ImmunityBio  v) { immunities.add(v); }
    public void setImmunityPsi(ImmunityPsi  v) { immunities.add(v); }
    public void setImmunityGravity(ImmunityGravity  v) { immunities.add(v); }
    public void setImmunityMindControl(ImmunityMindControl  v) { immunities.add(v); }
    public void setImmunityReplicaBeam(ImmunityReplicaBeam  v) { immunities.add(v); }
    public void setImmunityTroopTransport(ImmunityTroopTransport v) { immunities.add(v); }

	public double getLevel() { return productionLevelBonus+1.0; }
	public double getProductionBonus() { return productionBonus; }
	public double getProductionLevelBonus() { return productionLevelBonus; }
	public double getProductionTotalBonus() { return productionBonus + productionLevelBonus; }
	public double getResearchBonus() { return researchBonus; }
	public double getResearchLevelBonus() { return researchLevelBonus; }
	public double getResearchTotalBonus() { return researchBonus + researchLevelBonus; }
	public double getDevelopmentBonus() { return developmentBonus; }
	public double getDevelopmentLevelBonus() { return developmentLevelBonus; }
	public double getDevelopmentTotalBonus() { return developmentBonus + developmentLevelBonus; }
	
	public double getReproductionBonus() { return reproductionBonus; }
	public double getFactoryPerPopulationBonus() { return factoryPerPopulationBonus; }
	public double getEcologyBonus() { return ecologyBonus; }
    public double getMaxPopulationBonus() { return maxPopulationBonus; }

	public double getPlayerEcologyPercent() { return playerEcologyPercent; }
	public double getPlayerDevelopmentPercent() { return playerDevelopmentPercent; }
	public double getPlayerResearchPercent() { return playerResearchPercent; }
	public double getPlayerIsolationistResearchPercent() { return playerIsolationistResearchPercent; }
	public double getPlayerProductionPercent() { return playerProductionPercent; }
	public double getPlayerIsolationistProductionPercent() { return playerIsolationistProductionPercent; }
	public double getPlayerMoneyPercent() { return playerMoneyPercent; }

	public int getPsiRating() { return psiRating; }
	
	public NameList getMissedTechnologies() { return missedTechnologies; }
	public TechnologyList getMissedTechnologiesAsList() throws ApplicationError { 
		if (_missedTechnologies == null && missedTechnologies != null) _missedTechnologies = new GeneralTechnologyList(missedTechnologies); 
		return _missedTechnologies; 
	}

    public double getBattleTurningBonus() { return battleTurningBonus; }
    public double getBattleSpeedBonus() { return battleSpeedBonus; }
	public double getInitiativeBonus() { return initiativeBonus; }
    public double getSpeedBonus() { return speedBonus; }
    public double getMaxDamageBonus() { return maxDamageBonus; }
    public int getCloakLevel() { return cloakLevel; }
    public double getTradingBonus() { return tradingBonus; }
    public double getAssaultBonus() { return assaultBonus; }
    public double getSpyBonus() { return spyBonus; }

    public boolean getIsResourceful() { return resourceful; }
    public boolean isResourceful() { return resourceful; }
    
    public boolean isImune(Weapon weapon) { 
        if (immunities != null) {
        	for (Immunity imunity : immunities) {				
        		if (imunity.isImune(weapon)) return true;
			}
        }
        return false;
    }

    private static final double MIN_PERCENT = 0.01;    
    private static final double MAX_PERCENT = 0.5;
    
	public void setPlayerEcologyPercent(String v) throws ValidationError { 
        try {
            if (v == null) {
                throw new ValidationError("Please enter a valid fraction between "+MIN_PERCENT+" and "+MAX_PERCENT+" for the ecology percent");
            }
            double dv = Double.parseDouble(v);
            if (dv < MIN_PERCENT || dv > MAX_PERCENT) {
                throw new ValidationError("Please enter a valid fraction between "+MIN_PERCENT+" and "+MAX_PERCENT+" for the ecology percent");
            }
            playerEcologyPercent = dv; 
        } catch (NumberFormatException e) {
            throw new ValidationError("Please enter a valid fraction between "+MIN_PERCENT+" and "+MAX_PERCENT+" for the ecology percent");
        }
        
    }

	public void setPlayerDevelopmentPercent(String v) throws ValidationError { 
        try {
            if (v == null) {
                throw new ValidationError("Please enter a valid fraction between "+MIN_PERCENT+" and "+MAX_PERCENT+" for the development percent");
            }
            double dv = Double.parseDouble(v);
            if (dv < MIN_PERCENT || dv > MAX_PERCENT) {
                throw new ValidationError("Please enter a valid fraction between "+MIN_PERCENT+" and "+MAX_PERCENT+" for the development percent");
            }
            playerDevelopmentPercent = dv; 
        } catch (NumberFormatException e) {
            throw new ValidationError("Please enter a valid fraction between "+MIN_PERCENT+" and "+MAX_PERCENT+" for the development percent");
        }
        
    }

	/**
     * Constructs a players character from the setup options for this player's race.
     * Note some of the player statistics change for different levels.
     * 
     * The relationships must be defined before constructing a player character becuase
     * it uses the settings in relationships as default settings for this player.
     * 
     * @param myName The name of this new player
     * @param options The setup options to configure this player
     * @param level The level at which this game is being played
     * @param playerNames The list of players in this game.
     * @throws mot.common.util.ApplicationError
     */
	public PlayerCharacterBasic(String myName, RaceOptions options, Double level, String [] playerNames) throws ApplicationError {
        super(myName);

		productionBonus = options.getDoubleForName("production-bonus");
		researchBonus = options.getDoubleForName("research-bonus");
		developmentBonus = options.getDoubleForName("development-bonus");
		reproductionBonus = options.getDoubleForName("reproduction-bonus");
		factoryPerPopulationBonus = options.getDoubleForName("factory-per-population-bonus");
		battleTurningBonus = options.getDoubleForName("battle-turning-bonus");
		battleSpeedBonus = options.getDoubleForName("battle-speed-bonus");
		initiativeBonus = options.getDoubleForName("initiative-bonus");
		speedBonus = options.getDoubleForName("speed-bonus");
		maxDamageBonus = options.getDoubleForName("max-damage-bonus");
		cloakLevel = options.getIntForName("cloak-level");
		maxPopulationBonus = options.getDoubleForName("max-population-bonus");
		tradingBonus = options.getDoubleForName("trading-bonus");
		maxPlayerTradeAgreements = options.getIntForName("max-player-trade-agreements");
		maxSolarSystemTradeRoutes = options.getIntForName("max-solar-system-trade-routes");
        assaultBonus = options.getDoubleForName("assault-bonus"); 
        spyBonus = options.getDoubleForName("spy-bonus"); 
		ecologyBonus = options.getDoubleForName("ecology-bonus");
		psiRating = options.getIntForName("psi-rating");
        resourceful = options.getBoolForName("is-resourceful");
        eloquenceBonus = options.getDoubleForName("eloquence");
		chanceToDiscoverSalvagedTech = options.getDoubleForName("chance-to-discover-salvaged-tech");

        SetupOptions raceOptions = options.getRaceOptions();
        if (raceOptions != null) {
			String [] immunities = raceOptions.getAllStringsForName("immunity");
	        if (immunities != null) {
	        	for (int i = 0; i < immunities.length; i++) {				
	        		Immunity immunity = Immunity.getForName(immunities[i]);
	        		setImmunity(immunity);
				}
	        }
        }
        
        if (level != null) {
            productionLevelBonus = level.doubleValue()-1.0;
            researchLevelBonus   = level.doubleValue()-1.0;
            developmentLevelBonus = level.doubleValue()-1.0;
        } else {
            productionLevelBonus = 0.0;
            researchLevelBonus   = 0.0;
            developmentLevelBonus = 0.0;
        }
		playerEcologyPercent = options.getDoubleForName("percent-ecology");
		playerDevelopmentPercent = options.getDoubleForName("percent-development");
		playerResearchPercent = options.getDoubleForName("percent-research");
		playerIsolationistResearchPercent = options.getDoubleForName("isolationist-percent-research");
		playerProductionPercent = options.getDoubleForName("percent-production");
		playerIsolationistProductionPercent = options.getDoubleForName("isolationist-percent-production");
		playerMoneyPercent = options.getDoubleForName("percent-money");

        double percentTechsMissed = options.getDoubleForName("qty-missed-techs");
        if (percentTechsMissed > 0) {
            // Will always miss battle stations or terraforming
//        	if (Utils.random(0.5)) {
//        		Biotechnologies biotechnologies = Technologies.getBiotechnologies();
//        		biotechnologies.getAllWithGroup(Biotechnology.)
//        	} else {
//        		Hulls hulls = Technologies.getHulls();
//        	}
            TechnologyList all = Technologies.getTechnologies(false);
            int totalQty = all.getQuantity();
            int qty = (int)(totalQty * percentTechsMissed);
            missedTechnologies = new NameList();
            _missedTechnologies = null;
            while (qty > 0) {
                int i = Utils.random(totalQty);
                Technology tech = all.getTechnologyAtIndex(i);
                // We do not add techs with restrictions to the exclude list because these
                // are usually special techs like theory and alignment specials.
                if (tech.getRestriction() == null) {
                    totalQty--;
                    qty--;
                    all.remove(tech);
                    missedTechnologies.add(tech.getFullName());
                }
            }
            if (chanceToDiscoverSalvagedTech >= 1.0) {
            	Technology[] salvageAnalysers = Technologies.getLaboratories().getAllWithGroup(Laboratories.SALVAGE_ANALYSER);
            	if (salvageAnalysers != null) {
            		for (int i = 0; i < salvageAnalysers.length; i++) {						
            			missedTechnologies.add(salvageAnalysers[i].getFullName());
					}
            	}
            }
        }
        
        // Reference only
        charisma = options.getIntForName("charisma");
    }



	public void increasesBonusesFromOptions(SetupOption option) throws ApplicationError {
		increasesBonusesFromOptions(new SetupOptions(option));
	}
	
	/**
	 * Increase the basic bonuses in this player's character for the options passed in.
	 * This is used when a Relic Prize is discovered and adds bonuses o this players
	 * character.
	 * @param options The options whose bonuses will be used to increase the existing bonuses.
	 * @throws ApplicationError 
	 */
	public void increasesBonusesFromOptions(SetupOptions options) throws ApplicationError {
		if (options.hasOption(CRYSTALINE_AUGUMENTATED_METALS)) {
			maxDamageBonus += options.getDoubleForName(CRYSTALINE_AUGUMENTATED_METALS);
		}
		if (options.hasOption(ELECTROLITIC_AUGUMENTATION)) {
			speedBonus += options.getDoubleForName(ELECTROLITIC_AUGUMENTATION);
			battleSpeedBonus += options.getDoubleForName(ELECTROLITIC_AUGUMENTATION);
		}
		if (options.hasOption(BIO_GENETIC_TRANSFORMATION)) {
			maxPopulationBonus += options.getDoubleForName(BIO_GENETIC_TRANSFORMATION);
		}
		if (options.hasOption(BIO_CONFORMITY)) {
			spyBonus += options.getDoubleForName(BIO_CONFORMITY);
		}
		if (options.hasOption(PSI_EMHANCEMENT)) {
			psiRating += options.getIntForName(PSI_EMHANCEMENT);
		}
		if (options.hasOption(CLOAKING_SPACE_INVERTER)) {
			cloakLevel += options.getIntForName(CLOAKING_SPACE_INVERTER);
		}
	}

//	public void _encode(XMLWriter writer) throws ApplicationError {
//		encode(writer);
//	}
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("production-bonus", productionBonus);
		writer.addElement("production-level-bonus", productionLevelBonus);
		writer.addElement("research-bonus", researchBonus);
		writer.addElement("research-level-bonus", researchLevelBonus);
		writer.addElement("development-bonus", developmentBonus);
		writer.addElement("development-level-bonus", developmentLevelBonus);
		writer.addElement("reproduction-bonus", reproductionBonus);
		writer.addElement("factory-per-population-bonus", factoryPerPopulationBonus);
		writer.addElement("ecology-bonus", ecologyBonus);
		writer.addElement("player-ecology-percent", playerEcologyPercent);
		writer.addElement("player-development-percent", playerDevelopmentPercent);
		writer.addElement("player-research-percent", playerResearchPercent);
		writer.addElement("player-isolationist-research-percent", playerIsolationistResearchPercent);
		writer.addElement("player-production-percent", playerProductionPercent);
		writer.addElement("player-isolationist-production-percent", playerIsolationistProductionPercent);
		writer.addElement("psi-rating", psiRating);
		writer.addElement("battle-turning-bonus", battleTurningBonus);
		writer.addElement("battle-speed-bonus", battleSpeedBonus);
		writer.addElement("initiative-bonus", initiativeBonus);
		writer.addElement("assault-bonus", assaultBonus);
		writer.addElement("speed-bonus", speedBonus);
		writer.addElement("max-damage-bonus", maxDamageBonus);
		writer.addElement("cloak-level", cloakLevel);
		writer.addElement("max-population-bonus", maxPopulationBonus);
		writer.addElement("trading-bonus", tradingBonus);
		writer.addElement("max-player-trade-agreements", maxPlayerTradeAgreements);
		writer.addElement("max-solar-system-trade-routes", maxSolarSystemTradeRoutes);
        writer.addElement("charisma", charisma);
		writer.addElement("spy-bonus", spyBonus);
		writer.addElement("resourceful", resourceful);
		writer.addElement("eloquence-bonus", eloquenceBonus);
        if (immunities != null) {
        	for (Immunity imunity : immunities) {				
        		imunity.toXML(writer);
			}
        }
        if (missedTechnologies != null) {
        	String[] na = missedTechnologies.getNamesAsArray();
        	if (na != null) {
        		for (int i = 0; i < na.length; i++) {
        			writer.addElement("missed-technology", na[i]);
				}
        	}
        }
		writer.addElement("chance-to-discover-salvaged-tech", chanceToDiscoverSalvagedTech);
	}

	/**
	 * This includes bonuses from resources.
	 * @param owner 
	 * @return A list of all bonuses / characteristics contained in
	 * this player character.
	 * @throws ApplicationError 
	 */
	public String [] getDescription(PlayerResources resources) throws ApplicationError {
		ArrayList r = new ArrayList();
		if (productionBonus != 0.0) r.add(L.PLANETRY_PRODUCTION_BONUS+toPercent(productionBonus));
		if (developmentBonus != 0.0) r.add(L.PLANETRY_DEVELOPMENT_BONUS+toPercent(developmentBonus));
		if (reproductionBonus != 0.0) r.add(L.PLANETRY_REPRODUCTION_BONUS+toPercent(reproductionBonus));
		if (ecologyBonus != 0.0) r.add(L.PLANETRY_ECOLOGY_BONUS+toPercent(ecologyBonus));
		if (researchBonus != 0.0) r.add(L.PLANETRY_RESEARCH_BONUS+toPercent(researchBonus));
		if (maxPopulationBonus != 0.0) r.add(L.PLANETRY_MAXIMUM_POPULATION_BONUS+toPercent(maxPopulationBonus));
		if (factoryPerPopulationBonus != 0.0) r.add(L.FACTORY_PER_POPULATION_BONUS+toPercent(factoryPerPopulationBonus));
		if (psiRating != 0.0) r.add(L.PSI_RATING+psiRating);
		if (battleTurningBonus != 0.0) r.add(L.TURNING_BONUS_IN_BATTLE+toPercent(battleTurningBonus));
		if (battleSpeedBonus != 0.0) r.add(L.MOVEMENT_BONUS_IN_BATTLE+toPercent(battleSpeedBonus));
		if (initiativeBonus != 0.0) r.add(L.INITIATIVE_BONUS_IN_BATTLE+toPercent(initiativeBonus));
		if (assaultBonus != 0.0) r.add(L.HAND_TO_HAND_COMBAT_BONUS+toPercent(assaultBonus));
		if (speedBonus != 0.0) r.add(L.WARP_SPEED_BONUS+toPercent(speedBonus));
		if (maxDamageBonus != 0.0) r.add(L.CRAFT_MAXIMUM_DAMAGE_BONUS+toPercent(maxDamageBonus));
		if (cloakLevel != 0.0) r.add(L.CRAFT_CLOAK_BONUS+cloakLevel);
		if (tradingBonus != 0.0) r.add(L.TRADING_INCOME_BONUS+toPercent(tradingBonus));
		r.add(L.MAXIMUM_PLAYER_TRADE_ROUTES+maxPlayerTradeAgreements);
		r.add(L.MAXIMUM_SOLAR_SYSTEM_TRADE_ROUTES+maxSolarSystemTradeRoutes);
		if (spyBonus != 0.0) r.add(L.ESPIONAGE_BONUS+toPercent(spyBonus));
		if (resourceful) r.add(L.IS_RESOURCEFUL);
		if (eloquenceBonus != 0.0) r.add(L.INFLUENCE_BONUS+toPercent(eloquenceBonus)+L._ELOQUENCE);
        if (immunities != null) {
        	for (Immunity imunity : immunities) {				
                r.add(imunity.getDescription());
			}
        }
		r.add(L.CHANCE_TO_DISCOVER_SALVAGED_TECHNOLOGY+chanceToDiscoverSalvagedTech);
		if (resources != null) {
			if (resources.getCloakBonus() > 0.0) r.add(L.CLOAK_PALM + " - " + L.CLOAK + " " +toPercent(resources.getCloakBonus()));
			if (resources.getDamageBonus() > 0.0) r.add(L.ANTI_MATTER + " - " + L.DAMAGE + " " +toPercent(resources.getDamageBonus()));
			if (resources.getMaxDamageBonus() > 0.0) r.add(L.TRITANIUM_ORE + " - " + L.MAX_DAMAGE + " " +toPercent(resources.getMaxDamageBonus()));
			if (resources.getPsiBonus() > 0.0) r.add(L.PSI_FUNGUS + " - " + L.PSI + " " +toPercent(resources.getPsiBonus()));
			if (resources.getSpeedBonus() > 0.0) r.add(L.WARP_CRYSTAL + " - " + L.SPEED + " " + toPercent(resources.getSpeedBonus()));
			if (resources.getResearchBonus() > 0.0) r.add(L.BLACK_HOLE + " - " + L.RESEARCH + " " +toPercent(resources.getResearchBonus()));
		}
        return (String []) r.toArray(new String[r.size()]);
	}
	
	private String toPercent(double v) {
		return Formatter.percentLabel(v);
	}
	
	protected String getElementName() { return "basic-character"; }

    public int getMaxPlayerTradeAgreements() {
        return maxPlayerTradeAgreements;
    }

    public void setMaxPlayerTradeAgreements(int maxTradeRoutes) {
        this.maxPlayerTradeAgreements = maxTradeRoutes;
    }

    public int getMaxSolarSystemTradeRoutes() {
        return maxSolarSystemTradeRoutes;
    }

    public void setMaxSolarSystemTradeRoutes(int maxTradeRoutes) {
        this.maxSolarSystemTradeRoutes = maxTradeRoutes;
    }

    // Method for backwards compatibility in saved game files
    public void setMaxTradeRoutes(int maxTradeRoutes) {
        this.maxSolarSystemTradeRoutes = maxTradeRoutes;
        this.maxPlayerTradeAgreements = maxTradeRoutes;
    }
    // Method for backwards compatibility in saved game files
    public void setMaxPlayerTradeRoutes(int maxTradeRoutes) {
        this.maxPlayerTradeAgreements = maxTradeRoutes;
    }

    public int getCharisma() {
        return charisma;
    }

    public void setCharisma(int charisma) {
        this.charisma = charisma;
    }

	public double getEloquenceBonus() {
		return eloquenceBonus;
	}

	public void setEloquenceBonus(double eloquenceBonus) {
		this.eloquenceBonus = eloquenceBonus;
	}

	public void setChanceToDiscoverSalvagedTech(double chanceToDiscoverSalvagedTech) {
		this.chanceToDiscoverSalvagedTech = chanceToDiscoverSalvagedTech;
	}

	public double getChanceToDiscoverSalvagedTech() {
		return chanceToDiscoverSalvagedTech;
	}
	
	public ArrayList<Immunity> getImmunitiesList() {
		return immunities;
	}

	public Immunity [] getImmunities() {
		if (immunities.size() == 0) {
			return null;
		} else {
			return immunities.toArray(new Immunity[immunities.size()]);
		}
	}
}