/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.awt.Color;

import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;

public class PlayerColor {

    public static PlayerColor monsterColor = new PlayerColor(L.COLOUR_MONSTER);	

    private static PlayerColor [] colors;	

    public static synchronized PlayerColor [] gePlayerColors() throws ApplicationError {
        if (colors == null) {
            colors = new PlayerColor [] { 
                new PlayerColor(L.COLOUR_BLUE), 
                new PlayerColor(L.COLOUR_RED), 
                new PlayerColor(L.COLOUR_GREEN), 
                new PlayerColor(L.COLOUR_ORANGE), 
                new PlayerColor(L.COLOUR_YELLOW), 
                new PlayerColor(L.COLOUR_PINK), 
                new PlayerColor(L.COLOUR_GREY)
                
            };
        }
        return colors;
    }
    
	public static PlayerColor getColorForName(String v) throws ApplicationError {
		if (v == null) return monsterColor;
		if (Comp.same(v, L.COLOUR_MONSTER)) {
			return monsterColor;
		}
		PlayerColor[] pcs = gePlayerColors();
		for (int i = 0; i < pcs.length; i++) {
			if (Comp.same(v, pcs[i].getName())) return pcs[i];
		}
		Logger.logWarning("PlayerColour() No colour for "+v);
		return null;
	}

	public static PlayerColor getAvailableColor(PlayerColourList usedColours) throws ApplicationError {
		PlayerColor[] pca = gePlayerColors();
		if (pca == null) return null;
		for (int i = 0; i < pca.length; i++) {
			if (!usedColours.contains(pca[i])) {
				usedColours.add(pca[i]);
				return pca[i];
			}
		}
		throw new ApplicationError("No Colours Available "+usedColours);
	}



	
	
	
	
    private String name;
    
    /**
     * This is the colour to use for this player to draw the names of planets owned
     * by the player. 
     */
    private transient Color color;
    
    /**
     * This is the colour to use for this player to fill the background of the sector 
     * map. 
     */
    private transient Color sectorColor;

    public PlayerColor() {
    }

    private PlayerColor(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }
    
    public void setName(String v) {
        name = v;
    }

    public Color getColor() throws ApplicationError {
        if (color == null) {
        	if (Comp.same(name, L.COLOUR_BLUE)) {
        		color = PropertyManager.getMandatoryAsColor("player.color.blue");
        	} else if (Comp.same(name, L.COLOUR_RED)) {
        		color = PropertyManager.getMandatoryAsColor("player.color.red");
        	} else if (Comp.same(name, L.COLOUR_GREEN)) {
        		color = PropertyManager.getMandatoryAsColor("player.color.green");
        	} else if (Comp.same(name, L.COLOUR_ORANGE)) {
        		color = PropertyManager.getMandatoryAsColor("player.color.orange");
        	} else if (Comp.same(name, L.COLOUR_YELLOW)) {
        		color = PropertyManager.getMandatoryAsColor("player.color.yellow");
        	} else if (Comp.same(name, L.COLOUR_PINK)) {
        		color = PropertyManager.getMandatoryAsColor("player.color.pink");
        	} else if (Comp.same(name, L.COLOUR_GREY)) {
        		color = PropertyManager.getMandatoryAsColor("player.color.white");
        	} else if (Comp.same(name, L.COLOUR_MONSTER)) {
        		color = Color.MAGENTA;
        	} else {
        		Logger.logWarning("NO COLOUR "+name);
        		color = PropertyManager.getMandatoryAsColor("player.color.white");
        	}
        }
        return color;
    }

    public Color getSectorColor() throws ApplicationError {
        if (sectorColor == null) {
        	if (Comp.same(name, L.COLOUR_BLUE)) {
        		sectorColor = PropertyManager.getMandatoryAsColor("player.color.blue", 0.3);
        	} else if (Comp.same(name, L.COLOUR_RED)) {
        		sectorColor = PropertyManager.getMandatoryAsColor("player.color.red", 0.3);
        	} else if (Comp.same(name, L.COLOUR_GREEN)) {
        		sectorColor = PropertyManager.getMandatoryAsColor("player.color.green", 0.3);
        	} else if (Comp.same(name, L.COLOUR_ORANGE)) {
        		sectorColor = PropertyManager.getMandatoryAsColor("player.color.orange", 0.3);
        	} else if (Comp.same(name, L.COLOUR_YELLOW)) {
        		sectorColor = PropertyManager.getMandatoryAsColor("player.color.yellow", 0.3);
        	} else if (Comp.same(name, L.COLOUR_PINK)) {
        		sectorColor = PropertyManager.getMandatoryAsColor("player.color.pink", 0.3);
        	} else if (Comp.same(name, L.COLOUR_GREY)) {
        		sectorColor = PropertyManager.getMandatoryAsColor("player.color.white", 0.3);
        	} else {
        		Logger.logWarning("NO COLOUR "+name);
        		sectorColor = PropertyManager.getMandatoryAsColor("player.color.white", 0.3);
        	}
        }
        return sectorColor;
    }

    public String toString() { return name; }

}