/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * Contains graphical configuration information.
 */
public class RaceGraphics extends NamedObject {

    public static final String ELEMENT_NAME = "race-graphics";
	public static final String TYPE_PLAYER = "player";
    public static final String TYPE_ANCIENT = "ancient";
    public static final String TYPE_MONSTER = "monster";

	/**
	 * the type of this race.
	 */
	private String type;

	/**
	 * Diagram of this race
	 */
	private String picture;

	/**
	 * List of graphical information for crafts owned by this race.
	 */
	private ArrayList craftGraphics;
    
    private transient CraftGraphics [] craftGraphicsArray = null;

    /**
     * Image details to be displayed in the solar system window for
     * this race!
     */
    private RaceGraphicsForColony colonyGraphics;
    
    /**
     * The description of the race.
     */
    private String description;

	public RaceGraphics() {
	}

	public RaceGraphics(DomNode domNode) throws ApplicationError {
		super(domNode.getNodeValueByName_String("name"));
		
		type = domNode.getNodeValueByName_String("type");
		description = domNode.getNodeValueByName_String("description");
		
		DomNode[] cga = domNode.getNodesByName("craft-graphics");
		if (cga != null) {
			for (int i = 0; i < cga.length; i++) {				
				setCraftGraphics(new CraftGraphics(cga[i]));
			}
		}
		
		DomNode cg = domNode.getNodeByName("colony-graphics");
		if (cg != null) colonyGraphics = new RaceGraphicsForColony(cg);
		
		String [] hullName = domNode.getNodeValuesByName_String("copy-craft-graphics");
		if (hullName != null) {
			for (int i = 0; i < hullName.length; i++) {				
				setCopyCraftGraphics(hullName[i]);
			}
		}
		
		String [] raceName = domNode.getNodeValuesByName_String("copy-all-craft-graphics");
		if (raceName != null) {
			for (int i = 0; i < raceName.length; i++) {				
				setCopyAllCraftGraphics(raceName[i]);
			}
		}
	}

	public String getPicture() { return picture; }
	public void setPicture(String v) { picture = v; }

	public CraftGraphics [] getCraftGraphicsAsArray() {
		if (craftGraphics == null) {
			return null;
		} else {
            if (craftGraphicsArray == null) {
                craftGraphicsArray = (CraftGraphics []) craftGraphics.toArray(new CraftGraphics [craftGraphics.size()]);
            }
            return craftGraphicsArray;
		}
	}

    /**
     * This is used by the XML document handler to inform this RaceGraphics that its CraftGraphics 
     * are to be copied from the first raceGraphics.
     */
    public void setCopyCraftGraphics(String hullName) throws ApplicationError {
        if (RaceGraphicsList.referenceRaceGraphics != null) {
            CraftGraphics cg = RaceGraphicsList.referenceRaceGraphics.getHullGraphics(hullName);
            if (cg == null) {
                throw new ApplicationError("RaceGraphics.setCopyCraftGraphics() craft-graphics with name "+hullName+" is not defined in first race-graphics element");
            }
            setCraftGraphics(cg);
        } else {
            throw new ApplicationError("RaceGraphics.setCopyCraftGraphics() RaceGraphicsList.referenceRaceGraphics is null");
        }
    }
    
    /**
     * This is used by the XML document handler to inform this RaceGraphics that its CraftGraphics 
     * are to be copied from the first raceGraphics.
     */
    public void setCopyAllCraftGraphics(String raceName) throws ApplicationError {
        RaceGraphics rg = RaceGraphicsList.referenceRaceGraphics; // GraphicsFactory.getRaceGraphicsList().getGraphicsFor(raceName); This was an infinite loo
        if (rg != null) {
            CraftGraphics [] cg = rg.getCraftGraphicsAsArray();
            if (cg == null) {
                throw new ApplicationError("RaceGraphics.setCopyAllCraftGraphics() craft-graphics with name "+raceName+" is not defined in first race-graphics element");
            }
            for (int i=0; i<cg.length; i++) {
                setCraftGraphics(cg[i]);                
            }
        } else {
            throw new ApplicationError("RaceGraphics.setCopyAllCraftGraphics() RaceGraphics is null");
        }
    }
    
    public void setCraftGraphics(CraftGraphics cg) {
        if (craftGraphics == null) craftGraphics = new ArrayList();
        craftGraphics.add(cg);
        craftGraphicsArray = null;
    }

	public CraftGraphics getHullGraphics(String hullName) {
		CraftGraphics [] images = getCraftGraphicsAsArray();
        if (images == null) return null;
		for (int i=0; i<images.length; i++) {
			if ( Comp.same(hullName, images[i].getName()) ) {
				return images[i];
			}
		}
		return null;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("type", type);
		writer.addElement("picture", picture);
		CraftGraphics [] list = getCraftGraphicsAsArray();
		for (int i=0; i<list.length; i++) {
			list[i].toXML(writer);
		}
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }


    public void setType(String type) {
        this.type = type;
    }


    public String getType() {
        return type;
    }

	public RaceGraphicsForColony getColonyGraphics() {
		return colonyGraphics;
	}

	public void setColonyGraphics(RaceGraphicsForColony colonyGraphics) {
		this.colonyGraphics = colonyGraphics;
	}
}