/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.util.ArrayList;

import mot.common.factory.GraphicsFactory;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * Contains graphical configuration information.
 */
public class RaceGraphicsList extends XMLBean {

    private static final String ROOT_ELEMENT_NAME = "graphics";

	/**
     * This contains the first RaceGraphics in the list.
     */
    public static RaceGraphics referenceRaceGraphics = null;

	private ArrayList graphics;
    
    private transient RaceGraphics [] graphicsArray = null;

	public RaceGraphicsList() {
	}

	public RaceGraphicsList(DomTree doc) throws ApplicationError {
    	DomNode root = doc.getNodeByName(ROOT_ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(RaceGraphics.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setRaceGraphics(new RaceGraphics(n[i]));
			}
		}
	}

	public ArrayList getGraphics() { return graphics; }
	public void setGraphics(ArrayList v) { 
        graphics = v; 
        if (v != null && v.size() > 0) {
            referenceRaceGraphics = (RaceGraphics)v.get(0); 
        } else {
            referenceRaceGraphics = null;
        }
        graphicsArray = null;
    }
    
    public void setRaceGraphics(RaceGraphics v) {
        if (graphics == null) graphics = new ArrayList();
        graphics.add(v);
        if (referenceRaceGraphics == null) referenceRaceGraphics = v;
    }

	public RaceGraphics [] getRaceGraphicsAsArray() {
		if (graphics == null) {
			return null;
		} else {
            if (graphicsArray == null) {
                graphicsArray = (RaceGraphics []) graphics.toArray(new RaceGraphics [graphics.size()]);
            }
            return graphicsArray;
		}
	}

	public RaceGraphics getGraphicsFor(String raceName) {
		RaceGraphics [] a = getRaceGraphicsAsArray();
		if (raceName != null) {
			for (int i=0; i<a.length; i++) {
				if ( Comp.same(a[i].getName(), raceName) ) {
					return a[i];
				}
			}
		}
		if (a.length > 0){
			return a[0];
		}
		return null;
	}

	public RaceGraphics [] getGraphicsType(String type) {
		RaceGraphics [] a = getRaceGraphicsAsArray();
        ArrayList r = null;
		for (int i=0; i<a.length; i++) {
			if ( Comp.same(a[i].getType(), type) ) {
                if (r == null) r = new ArrayList();
				r.add(a[i]);
			}
		}
        if (r == null) {
            return null;
        } else {
            return (RaceGraphics []) r.toArray(new RaceGraphics [r.size()]);
        }
	}

	public CraftGraphics getHullGraphics(String raceName, String hullName) {
		RaceGraphics rg = getGraphicsFor(raceName);
        if (rg != null) {
            return rg.getHullGraphics(hullName);
        } else {
            return null;
        }
	}
	
	private CraftGraphics _biggest = null;
	public CraftGraphics getBiggestCraft() {
		RaceGraphics [] a = getRaceGraphicsAsArray();
		for (int i=0; i<a.length; i++) {
			CraftGraphics[] cga = a[i].getCraftGraphicsAsArray();
			if (cga != null) {				
				for (int j = 0; j < cga.length; j++) {				
					if (_biggest == null || cga[j].getSize().isLarger(_biggest.getSize())) {
						_biggest = cga[j];
					}
				}
			}
		}
		if (_biggest != null){
			return _biggest;
		} else {
			return null;
		}
	}

	public int getBiggestCraftSize() throws ApplicationError {
		CraftGraphics biggest = GraphicsFactory.getRaceGraphicsList().getBiggestCraft();
		return (biggest == null) ? 50 : biggest.getSize().getY();
	}
	
	public CraftGraphics getDefaultHullGraphics() {
		RaceGraphics rg = getGraphicsFor("Monster");
        if (rg != null) {
            return rg.getHullGraphics("Flyer60");
        } else {
            return null;
        }
	}


	protected void encode(XMLWriter writer) throws ApplicationError {
		RaceGraphics [] list = getRaceGraphicsAsArray();
		for (int i=0; i<list.length; i++) {
			list[i].toXML(writer);
		}
	}

	protected String getElementName() { return ROOT_ELEMENT_NAME; }

}