/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.io.Serializable;

/**
 * @author Paul Davis
 */
public class Rectangle implements Serializable, IRectangle {

	private int left;
    private int top;
    private int width;
    private int height;

    public Rectangle() {
    }

    public Rectangle(Location loc, int size) {
    	if (size == 0) {
            this.left = loc.getX();
            this.top = loc.getY();
    	} else {
            this.left = loc.getX()-size/2;
            this.top = loc.getY()-size/2;
    	}
        this.width = size;
        this.height = size;
    }

	public Rectangle(Location centerLoc, int width, int height) {
	    this.left = centerLoc.getX()-width/2;
	    this.top = centerLoc.getY()-height/2;
	    this.width = width;
	    this.height = height;
    }
    
    public Rectangle(int left, int top, int width, int height) {
        this.left = left;
        this.top = top;
        this.width = width;
        this.height = height;
    }

    public Rectangle(Rectangle v) {
        this.left = v.getLeft();
        this.top = v.getTop();
        this.width = v.getWidth();
        this.height = v.getHeight();
    }

    public Area getArea() {
        return new Area(left, top, left+width, top+height);
    }
    
    public Location getCenterLocation() {
        return new Location(left + width/2, top + height/2);
    }

    public Location getTopLeftLocation() {
        return new Location(left, top);
    }
    
    /**
     * A point is assumed to be contained if it lays on one of the lines.
     * @return True if this contains the the point l.
     */
	public boolean contains(Location l) {
		return contains(l.getX(), l.getY());
	}

    /**
     * A point is assumed to be contained if it lays on one of the lines.
     * @return True if this contains the the point x,y.
     */
	public boolean contains(int px, int py) {
		return (px >= left && px <= (left+width) && py >= top && py <= top+height);
	}
	
	public boolean equals(Rectangle r) {
		return (left == r.getLeft() && top == r.getTop() && width == r.getWidth() && height == r.getHeight());
	}

	public void update(Rectangle r) {
		left = r.getLeft();
		top = r.getTop();
		width = r.getWidth();
		height = r.getHeight();
	}

	/**
	 * Increase the size of the rectangle by xd on the x-dimension and yd
	 * on the y-dimension. The centre of the rectangle is NOT modified. All
	 * corners are moves out by half the increase (xd or yd);
	 * @param xd
	 * @param yd
	 */
	public void expand(int xd, int yd) {
		left -= xd/2;
		top -= yd/2;
		width += xd;
		height += xd;
	}
	
    /**
     * @return True if this area intersects the area a and false otherwise.
     */
	public boolean intersects(Rectangle a) {
		int r = left+width;
		int b = top+height;
		int ol = a.getLeft();
		int or = ol + a.getWidth();
		int ot = a.getTop();
		int ob = ot + a.getHeight();
        boolean doesNotIntersect = ot > b || 
                                   ob < top ||
                                   ol > r ||
                                   or < left;
		return !doesNotIntersect;
	}

	/**
	 * Increase the size of the rectangle by xd on the x-dimension and yd
	 * on the y-dimension. The centre of the rectangle is NOT modified. All
	 * corners are moves out by half the increase (xd or yd);
	 * @param xd
	 * @param yd
	 */
	public void scale(double v) {
		left = (int)(left * v);
		top = (int)(top * v);
		width = (int)(width * v);
		height = (int)(height * v);
	}

    public void setLeft(int x) {
        this.left = x;
    }


    public int getLeft() {
        return left;
    }

    public void setRight(int x) {
        this.left = x - width;
    }


    public int getRight() {
        return left + width;
    }


    public void setTop(int y) {
        this.top = y;
    }


    public int getTop() {
        return top;
    }

    public void setBottom(int y) {
        this.top = y - height;
    }


    public int getBottom() {
        return top + height;
    }


    public void setWidth(int width) {
        this.width = width;
    }


    public int getWidth() {
        return width;
    }


    public void setHeight(int height) {
        this.height = height;
    }


    public int getHeight() {
        return height;
    }
    
    public void setRectangle(int left, int top, int width, int height) {
        this.left = left;
        this.top = top;
        this.width = width;
        this.height = height;
    }

    

	public String toString() {
		return "Rectangle [left=" + left + ", top=" + top + ", width=" + width
				+ ", height=" + height + "]";
	}
}