/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.ValueObject;

/**
 * The galaxy window display a map of the galaxy showing ownership of the galaxy.
 * The galaxy is split into sectors each sector having a single owner, the player
 * with the most influence in that sector.
 * 
 * This class contains a table of all the sectors in the galaxy and details of the 
 * owner of each sector. It is used to display the galaxy map.
 */
public class SectorOwnerGrid implements ValueObject {

	private int sectorSize;
	private SectorInfluenceList [] [] grid;

	public SectorOwnerGrid(int width, int height, int sectorSize) {
		this.sectorSize = sectorSize;
		this.grid = new SectorInfluenceList [height] [width];
	}
	
	public int getQtyCols() { return grid[0].length; }
	public int getQtyRows() { return grid.length; }
	
	public SectorInfluenceList getCellBySector(int row, int col) {
		return grid[row][col];
	}

	public SectorInfluenceList getCellBySectorNotNull(int row, int col) {
		SectorInfluenceList r = grid[row][col];
		if (r != null) {
			return r;
		} else {
			r = new SectorInfluenceList();
			grid[row][col] = r;
			return r;
		}
	}

	public void clear() throws ApplicationError {
		for (int i = 0; i < grid.length; i++) {
			for (int j = 0; j < grid[i].length; j++) {
				if (grid[i][j] != null) {
					grid[i][j].clear();
				}
			}
		}
	}


	public SectorInfluenceItem getSectorOwner(int row, int col) {
		if (row >= grid.length || row < 0) {
			Logger.logWarning("Array out of bounds row="+row+" col="+col);
			Logger.logStackTrace();
			return null;
		}
		if (col >= grid[0].length || col < 0) {
			Logger.logWarning("Array out of bounds row="+row+" col="+col);
			Logger.logStackTrace();
			return null;
		}
		SectorInfluenceList l = grid[row][col];
		if (l == null) {
			return null;
		} else {
			return l.getOwner();
		}
	}	
	
	public SectorInfluenceList getCellByLocation(int x, int y) {
		int col = x / sectorSize;
		int row = y / sectorSize;
		if (row >= grid.length || col >= grid[0].length) {
			return null;
		} else {
			return grid[row][col];
		}
	}

	public SectorInfluenceList [] [] getGrid() {
		return grid;
	}

	public void setGrid(SectorInfluenceList [] [] grid) {
		this.grid = grid;
	}

	public void setCell(int row, int col, SectorInfluenceList cell) {
		grid[row][col] = cell;
	}

	public String toString() {
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < grid.length; i++) {
			for (int j = 0; j < grid[i].length; j++) {
				if (j>0) buf.append(",");
				if (grid[i][j] != null) {
					buf.append(grid[i][j].toString());
				} else {
					buf.append("null");
				}
			}
			buf.append("\n");
		}
		return buf.toString();
	}


}
