/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import mot.client.telemetry.Telemetry;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLDocumentHandler;
import mot.common.xml.XMLWriter;

/**
 * This class is a singleton class that contains all the players settings,
 * e.g. the speed at which the user wished the screen to scroll etc.
 */
public class Settings extends XMLBean {

    private static final String SETTINGS_FILE_NAME = "data/settings.xml";
    private static final String SETTINGS_ORIGINAL_FILE_NAME = "data/settings_original.xml";
    
	private static final int DEFAULT_QTY_COLONISTS_PER_SHUTTLE = 5;
	/**
     * The time interval between screen refreshes on the Main Window.
     */
    public static final int DEFAULT_FRAMES_PER_SECOND = 20;
    public static final int MIN_FRAMES_PER_SECOND = 10;
    public static final int MAX_FRAMES_PER_SECOND = 100;
	private static final double DEFAULT_VOLUME = 0.25;
	private static final double MAX_VOLUME = 1.0;
	private static final double MIN_VOLUME = 0.0;
	public static final int DEFAULT_MOUSE_SCROLL_WHEEL_SPEED = 3;
	public static final int DEFAULT_AUTO_SAVE_INTERVAL = 10;
	
    public Settings() {
    }
    
    private static Settings instance;
    
    /**
	  * The quantity of frames per second used to animate the main window.
	  * A high value such as 50 would produce smooth animation but take lots 
	  * of CPU time while a low value like 10 would produce jumpy animation
	  * but consume only a small amount of CPU.
     */
    private int framesPerSecond;
    
    /**
     * The quantity of frames to pause between moving each craft on the battle field.
     * It is also the delay between focusing on a craft and that craft moving.
     */
    private int craftMoveDelay;
    
    /**
	 * The quantity of frames used to animate a complete craft move on the battle field.
	 * I high value will cause crafts to move slowly while a low value will be quick.
     */
    private int qtyFramesPerTurn;
    
    /**
	  * Value 2 = Warning, 4 = Info, 5 = Debug.
     */
    private int logLevel = 2;
    
    /**
	  * A value of 1 implies the game will be played in full screen mode (take complete
	  * control of the desktop in exclusive mode and use its own painting loop (disable
	  * OS level painting). Any other value implies the game will be run in a normal
	  * application window.
     */
    private int fullScreen;

    /**
	  * A value of 1 implies the window is to be maximised. full screen takes presidency over this setting.
     */
    private int maximised;

    /**
     * The width of the main window.
     * In  full screen mode this is the resolution width of the screen (so must be 
     * supported by the video driver).
     * In main screen mode it is the width of the window.
     */
    private int windowSizeX;
    
    /**
     * The height of the main window.
     * In  full screen mode this is the resolution height of the screen (so must be 
     * supported by the video driver).
     * In main screen mode it is the height of the window.
     */
    private int windowSizeY;
    
    /**
     * If this is true the galaxy map displays orbit rings
     */
    private boolean drawOrbitRingsInGalaxyMap;
    
    /**
     * The quantity of colonists to load into a transport each time the user clicks 
     * the Transport button on a planet.
     */
    private int qtyColonistsToLoadPerShuttle = DEFAULT_QTY_COLONISTS_PER_SHUTTLE;
    
    /**
     * This is the volume of sound effects in the game.
     * This is a value between 0.0 and 1.0.
     */
	private double volumeEffects = DEFAULT_VOLUME;
    
    /**
     * This is the volume of sound background music in the game.
     * This is a value between 0.0 and 1.0.
     */
	private double volumeMusic;
	
	/**
	 * This is a list of songs played during the game in the background.
	 */
	private List<String> songs = new ArrayList<String>();
    
	/**
	 * This is a list of all weapon groups that should not be fired when the user
	 * uses the express button to inform a craft to attack another craft using all
	 * its weapons. This is important because some weapons groups can also damage
	 * the craft that is firing the weapon, e.g. Droid Mine, Spatial Distortion etc.
	 */
	private String [] weaponsToExcludeFromFireAll;
	
    /**
     * The size of zoom performed on the galaxy window for every click of the mouse wheel.
     */
    private int mouseScrollWheelSpeed = DEFAULT_MOUSE_SCROLL_WHEEL_SPEED;
	
    /**
     * The size of zoom performed on the galaxy window for every click of the mouse wheel.
     */
    private int autosaveInterval = DEFAULT_AUTO_SAVE_INTERVAL;

    /**
     * List generic settings from other parts of the system.
     */
	private Map<String, NameValuePair> genericList = new HashMap<String, NameValuePair>();
	
	/**
	 * This is the latest version of the game that has been released and is know to this installation.
	 * It is used to issue a warning to the player update the game currently installed. 
	 */
	private String latestVersion;

	/**
	 * This is the time in millis the last version was checked (web site contacted to check for updates). 
	 */
	private long timeVersionChecked;
	
	/**
	 * The last game saved (used by the continue game button at the start). 
	 */
	private String lastGameSaved;
	
	/**
	 * In a multi-player game this is the name of the player (in a single player game can be null) in the last game saved.
	 */
	private String lastGamePlayer;
	
	/**
	 * Does the end turn button give a warning when pressed an not all crafts have been moved (or sleeping).
	 */
	private boolean moveCraftWarning;
	
	
	private boolean appletMode = false;
	
	/**
     * Loads the settings from the settings.xml file if they have not been already loaded.
     * @throws ApplicationError
     */
    public static Settings getInstance() throws ApplicationError {
		if (instance == null) {
			File file = new File(SETTINGS_FILE_NAME);
			if (!file.exists()) {
				file = new File(SETTINGS_ORIGINAL_FILE_NAME);
			}
			instance = (Settings)XMLDocumentHandler.parse(file, false, null);
		}
		return instance;
    }

    public static boolean hasInstance() {
    	return (instance != null);
    }

	public static void newInstance(int framesPerSecond, int windowSizeX, int windowSizeY, String music, String weaponsToExcludeFromFireAll, int logLevel) {
		instance = new Settings();
		instance.setFramesPerSecond(framesPerSecond);
		instance.setQtyFramesPerTurn(DEFAULT_QTY_FRAMES_PER_MOVE_TURN);
		instance.setAutosaveInterval(-1);
		instance.setFullScreen(0);
		instance.setMaximised(0);
		instance.setWindowSizeX(windowSizeX);
		instance.setWindowSizeY(windowSizeY);
		instance.setMouseScrollWheelSpeed(DEFAULT_MOUSE_SCROLL_WHEEL_SPEED);
		instance.setDrawOrbitRingsInGalaxyMap(true);
		instance.setQtyColonistsToLoadPerShuttle(DEFAULT_QTY_COLONISTS_PER_SHUTTLE);
		instance.setVolumeEffects((MAX_VOLUME - MIN_VOLUME) / 2);
		instance.setVolumeMusic((MAX_VOLUME - MIN_VOLUME) / 2);
		instance.setSong(music);
		instance.setLogLevel(logLevel);
		instance.setWeaponsToExcludeFromFireAll(weaponsToExcludeFromFireAll);
		instance.setAppletMode(true);
		instance.setMoveCraftWarning(true);
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("frames-per-second", framesPerSecond);
        writer.addElement("craft-move-delay", craftMoveDelay);
        writer.addElement("qty-frames-per-turn", qtyFramesPerTurn);
        writer.addElement("autosave-interval", autosaveInterval);
        writer.addElement("full-screen", fullScreen);
        writer.addElement("maximised", maximised);
        writer.addElement("window-size-x", windowSizeX);
        writer.addElement("window-size-y", windowSizeY);
        writer.addElement("mouse-scroll-wheel-speed", mouseScrollWheelSpeed);
        writer.addElement("draw-orbit-rings-in-galaxy-map", drawOrbitRingsInGalaxyMap);
        writer.addElement("qty-colonists-to-load-per-shuttle", qtyColonistsToLoadPerShuttle);
        writer.addElement("volume-effects", volumeEffects);
        writer.addElement("volume-music", volumeMusic);
        for (Iterator i = songs.iterator(); i.hasNext();) {
        	String songName = (String) i.next();
            writer.addElement("song", songName);
		}
        writer.addElement("weapons-to-exclude-from-fire-all", Comp.toString(weaponsToExcludeFromFireAll,","));
        writer.addElement("log-level", logLevel);
        if (latestVersion != null) writer.addElement("latest-version", latestVersion);
        if (timeVersionChecked > 0) writer.addElement("time-version-checked", timeVersionChecked);
        if (lastGameSaved != null) writer.addElement("last-game-saved", lastGameSaved);
        if (lastGamePlayer != null) writer.addElement("last-game-player", lastGamePlayer);
        Collection<NameValuePair> values = genericList.values();
        if (values != null) {
        	for (Iterator i = values.iterator(); i.hasNext();) {
				NameValuePair v = (NameValuePair) i.next();
				v.toXML(writer, "generic-setting");
			}
        }
        if (Telemetry.ON) writer.addElement("move-craft-warning", moveCraftWarning);
        if (Telemetry.ON) writer.addElement("telemetry", 1);
	}

    public void save() throws ApplicationError {
    	if (!appletMode) {
	    	File file = new File(SETTINGS_FILE_NAME);
	        XMLWriter writer = new XMLWriter(null, getElementName());
	        toXML(writer);
	        writer.save(file);
    	}
    }

	protected String getElementName() { return "settings"; }

    public int getCraftMoveDelay() {
        return craftMoveDelay;
    }

    public void setCraftMoveDelay(int craftMoveDelay) {
        this.craftMoveDelay = craftMoveDelay;
    }

	public boolean isFullScreen() {
		return fullScreen == 1;
	}

	public int getFullScreen() {
		return fullScreen;
	}

	public void setFullScreen(int fullScreen) {
		this.fullScreen = fullScreen;
	}

	public int getWindowSizeX() {
		return windowSizeX;
	}

	public void setWindowSizeX(int windowSizeX) {
		this.windowSizeX = windowSizeX;
	}

	public int getWindowSizeY() {
		return windowSizeY;
	}

	public void setWindowSizeY(int windowSizeY) {
		this.windowSizeY = windowSizeY;
	}

    private static int DEFAULT_QTY_FRAMES_PER_MOVE_TURN = 20;
    public int getQtyFramesPerTurn() {
    	if (qtyFramesPerTurn <= 0) {
    		qtyFramesPerTurn = DEFAULT_QTY_FRAMES_PER_MOVE_TURN;
    	}
    	return qtyFramesPerTurn;
    }

	public void setQtyFramesPerTurn(int qtyFramesPerMoveTurn) {
		this.qtyFramesPerTurn = qtyFramesPerMoveTurn;
	}

	public int getFramesPerSecond() {
		if (framesPerSecond < MIN_FRAMES_PER_SECOND || framesPerSecond > MAX_FRAMES_PER_SECOND) {
			framesPerSecond = DEFAULT_FRAMES_PER_SECOND;
		}
		return framesPerSecond;
	}

	public void setFramesPerSecond(int framesPerSecond) {
		this.framesPerSecond = framesPerSecond;
	}

	public boolean isDrawOrbitRingsInGalaxyMap() {
		return drawOrbitRingsInGalaxyMap;
	}

	public void setDrawOrbitRingsInGalaxyMap(boolean drawOrbitRingsInGalaxyMap) {
		this.drawOrbitRingsInGalaxyMap = drawOrbitRingsInGalaxyMap;
	}

	public int getQtyColonistsToLoadPerShuttle() {
		return qtyColonistsToLoadPerShuttle;
	}

	public void setQtyColonistsToLoadPerShuttle(int qtyColonistsToLoadPerShuttle) {
		this.qtyColonistsToLoadPerShuttle = qtyColonistsToLoadPerShuttle;
	}

	public double getVolumeEffects() {
		return volumeEffects;
	}

	public void setVolumeEffects(double v) {
		if (v >= MIN_VOLUME && v <= MAX_VOLUME) {
			volumeEffects = v;
		} 
	}

	public double getVolumeMusic() {
		return volumeMusic;
	}

	public void setVolumeMusic(double v) {
		if (v >= MIN_VOLUME && v <= MAX_VOLUME) {
			volumeMusic = v;
		} 
	}
	
	public void setSong(String v) {
		songs.add(v);
	}

	public String [] getSongs() {
		return songs.toArray(new String[songs.size()]);
	}
	
	public void deleteSongs() {
		 songs.clear();
	}

	public String [] getWeaponsToExcludeFromFireAll() {
		return weaponsToExcludeFromFireAll;
	}

	public void setWeaponsToExcludeFromFireAll(String v) {
		this.weaponsToExcludeFromFireAll = v.split(",");
	}

	public int getMouseScrollWheelSpeed() {
		return mouseScrollWheelSpeed;
	}

	public void setMouseScrollWheelSpeed(int mouseScollWheelSpeed) {
		this.mouseScrollWheelSpeed = mouseScollWheelSpeed;
	}

	public int getAutosaveInterval() {
		return autosaveInterval;
	}

	public void setAutosaveInterval(int autosaveInterval) {
		this.autosaveInterval = autosaveInterval;
	}

	public void setGenericSetting(NameValuePair v) {
		if (v == null) return;
		NameValuePair old = genericList.get(v.getName());
		if (old != null) {
			old.setValue(v.getValue());
		} else {
			genericList.put(v.getName(), v);
		}
	}
	
	public void addGenericSetting(String name, String value) {
		NameValuePair v = genericList.get(name);
		if (v != null) {
			v.setValue(value);
		} else {
			genericList.put(name, new NameValuePair(name, value));
		}
	}
	
	public String getGenericSetting(String name) {
		NameValuePair v = genericList.get(name);
		if (v == null) {
			return null;
		} else {
			return v.getValue();
		}
	}

	public Integer getGenericSettingInt(String name) {
		String vs = getGenericSetting(name);
		if (vs == null) return null;
		try {
			return new Integer(vs);
		} catch (NumberFormatException nfe) {
			return null;
		}
	}

	public NameValuePair [] getAllGenericSettings() {
        Collection<NameValuePair> values = genericList.values();
        if (values != null) {
        	return values.toArray(new NameValuePair[values.size()]);
        } else {
        	return null;
        }
	}

	public int getMaximised() {
		return maximised;
	}

	public void setMaximised(int maximised) {
		this.maximised = maximised;
	}

	public int getLogLevel() {
		return logLevel;
	}

	public void setLogLevel(int logLevel) {
		this.logLevel = logLevel;
	}

	public String getLatestVersion() {
		return latestVersion;
	}

	public void setLatestVersion(String latestVersion) {
		this.latestVersion = latestVersion;
	}

	public long getTimeVersionChecked() {
		return timeVersionChecked;
	}

	public void setTimeVersionChecked(long timeVersionChecked) {
		this.timeVersionChecked = timeVersionChecked;
	}

	public String getLastGameSaved() {
		return lastGameSaved;
	}

	public void setLastGameSaved(String lastGameSaved) {
		this.lastGameSaved = lastGameSaved;
	}

	public void updateLastGameSaved(String fileName, String playerName) throws ApplicationError {
		if (fileName != null &&  !Comp.same(fileName, lastGameSaved)) {
			this.lastGameSaved = fileName;
			this.lastGamePlayer = playerName;
			save();
		}
	}

	public String getLastGamePlayer() {
		return lastGamePlayer;
	}

	public void setLastGamePlayer(String lastGamePlayer) {
		this.lastGamePlayer = lastGamePlayer;
	}

	public boolean isAppletMode() {
		return appletMode;
	}

	public void setAppletMode(boolean appletMode) {
		this.appletMode = appletMode;
	}

	public void setTelemetry(int v) {
		if (v == 1) Telemetry.ON = true;
	}

	public int getScrollSpeed() {
		return 16;
	}

	public boolean isMoveCraftWarning() {
		return moveCraftWarning;
	}

	public void setMoveCraftWarning(boolean sleepMode) {
		this.moveCraftWarning = sleepMode;
	}

	
}