/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import mot.common.util.ApplicationError;
import mot.common.util.Utils;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * Contains graphical configuration information.
 */
public class StellarObjectGraphics extends NamedObject {

    public static final int QTY_STAR_GRAPHICS = 3;
    public static final String STAR_TYPE_YELLOW = "Yellow";
    public static final String STAR_TYPE_BLUE = "Blue";
    public static final String STAR_TYPE_RED = "Red";
    
    public static String getRandomStarType() {
        switch (Utils.random(StellarObjectGraphics.QTY_STAR_GRAPHICS)) {
            case 0:
                return STAR_TYPE_YELLOW;
            case 1:
                return STAR_TYPE_BLUE;
            case 2:
                return STAR_TYPE_RED;
            default:
                return STAR_TYPE_YELLOW;
        }
    }

	/**
	 * Diagram of this star
	 */
	private String picture;
	
	/**
	 * The type of the image or null if the picture contains the type
	 */
	private String pictureType;

	/**
	 * The screen space consumed by this hull.
	 */
	private Size size;
	
	/**
	 * If this image is animated then this is the quantity of images in the
	 * animation. This is optional and may be 0. 
	 * qtyAnimations = 0 and stages = null implies no animation.
	 */
	private int qtyAnimations = 0;
	
	/**
	 * If this image is animated then this the names of the file indexes used in the animation.
	 * This is optional and may be null. If this is null and qtyAnimation = 5 then the stages 
	 * are assumed to be 1,2,3,4,5.
	 * qtyAnimations = 0 and stages = null implies no animation.
	 */
	private String stages;
	private String [] _stages;
	
	private transient String [] pictures = null;
	
	/**
	 * If this image is to be animated by rotation, this is the centre around which to rotate it.
	 */
	private Location rotateAbout;

	
	/**
	 * The type of the 3d background image of this object
	 */
	private String image;

	
	public StellarObjectGraphics() {
	}

	public StellarObjectGraphics(DomNode domNode) throws ApplicationError {
		super(domNode.getNodeValueByName_String("name"));
		 picture = domNode.getNodeValueByName_String("picture");
		 pictureType = domNode.getNodeValueByName_String("picture-type");
		 qtyAnimations = domNode.getNodeValueByName_Int("qty-animations");
		 stages = domNode.getNodeValueByName_String("stages");
		 size = domNode.getNodeValueByName_Size("size");
		 image = domNode.getNodeValueByName_String("image");
		 
		 DomNode rn = domNode.getNodeByName("rotate-about");
		 if (rn != null) rotateAbout = new Location(rn);
	}

	public String getPicture() { return picture; }
	public void setPicture(String v) { picture = v; }

	public Size getSize() { return size; }
	public void setSize(Size v) { size = v; }
    
	public String getStarType() { return getName(); }

    public String [] getStagesAsArray() {
    	if (_stages != null) {
    		return _stages;
    	}
        if (stages == null) {
            return null;
        }
    	_stages = stages.split(",");
    	return _stages;
    }

	public String getPicture(int animationStage) {
		if (pictures == null) {
	        if (qtyAnimations > 0) {
				pictures = new String[qtyAnimations];
				for (int i = 0; i < qtyAnimations; i++) {
			        String picture = getPicture() + (i+1);
			        if (pictureType != null) {
			        	picture += pictureType;
			        }
			        pictures[i] = picture;
				}
	        } else if (stages != null) {
	        	String [] sa = getStagesAsArray();
	        	pictures = new String[sa.length];
				for (int i = 0; i < sa.length; i++) {
			        String picture = getPicture() + sa[i];
			        if (pictureType != null) {
			        	picture += pictureType;
			        }
			        pictures[i] = picture;
				}
	        } else {
				pictures = new String[1];
		        String picture = getPicture();
		        if (pictureType != null) {
		        	picture += pictureType;
		        }
		        pictures[0] = picture;
	        }
		}
        return pictures[animationStage%pictures.length];
	}
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("picture", picture);
		writer.addElement("picture-type", pictureType);
		writer.addElement("qty-animations", qtyAnimations);
		if (size != null) {
			size.toXML(writer);
		}
		writer.addElement("image", image);
	}

	protected String getElementName() { return "star-graphics"; }

	public int getQtyAnimations() {
		return qtyAnimations;
	}

	public void setQtyAnimations(int qtyAnimations) {
		this.qtyAnimations = qtyAnimations;
	}

	public String getPictureType() {
		return pictureType;
	}

	public void setPictureType(String pictureType) {
		this.pictureType = pictureType;
	}

	public String getStages() {
		return stages;
	}

	public void setStages(String stages) {
		this.stages = stages;
	}

	public Location getRotateAbout() {
		return rotateAbout;
	}

	public void setRotateAbout(Location rotateAbout) {
		this.rotateAbout = rotateAbout;
	}

	public String getImage() {
		return image;
	}

	public void setImage(String image) {
		this.image = image;
	}
}