/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean;

import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * A tutorial is a collection of tips. Each tip has a unique ID.
 * This class is a map from tip ID to status (Used or Unused).
 * 
 * This class is transfered between the client and the engine
 * to save / load the tutorial status information.
 */
public class TutorialStatus extends XMLBean {

	private static final String SEPARATOR = ",";
	private Boolean [] tipStatus;

	public TutorialStatus() {
	}

	public void setUsedTip(int index) {
		if (tipStatus == null || index >= tipStatus.length) {
			Boolean [] oldTipStatus = tipStatus;
			tipStatus = new Boolean[index+1];
			if (oldTipStatus != null) {
				for (int i = 0; i < oldTipStatus.length; i++) {
					tipStatus[i] = oldTipStatus[i];
				}
				for (int i = oldTipStatus.length; i < tipStatus.length; i++) {
					tipStatus[i] = Boolean.FALSE;
				}
			}
		}
		tipStatus[index] = Boolean.TRUE;
	}
	
	public boolean hasUsedTipId(int index) {
		return tipStatus != null && tipStatus[index] != null && tipStatus[index].booleanValue();
	}
	
	private static final String USED = "1";
	private static final String UNUSED = "0";
	
	public void setUsedTips(String statusString) {
		String[] ida = statusString.split(SEPARATOR);
		tipStatus = new Boolean[ida.length];
		for (int i = 0; i < ida.length; i++) {
			tipStatus[i] = (ida[i].equals(USED));
		}
	}
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		if (tipStatus == null) return;
				
		StringBuffer sb = new StringBuffer();
		for (int i=0; i<tipStatus.length; i++) {
			if (tipStatus[i] != null && tipStatus[i].booleanValue()) {
				sb.append(USED);
			} else {
				sb.append(UNUSED);
			}
			if (i < tipStatus.length-1)	sb.append(SEPARATOR);
		}
		writer.addElement("used-tips", sb.toString());
	}
	

	protected String getElementName() {
		return "tutorial";
	}

	
	
}
