/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean.stats;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import mot.common.technology.Hull;
import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * A collection of historical information collected to score a game.
 */
public class GameScoreData extends XMLBean implements Serializable {

	/**
	 * The list of craft this player has built during the game.
	 */
	private List<CraftQuantityData> craftBuiltList = new ArrayList();
	
	/**
	 * The list of craft this player has destroyed during the game.
	 */
	private List<CraftQuantityData> craftDestroyedList = new ArrayList();
		
	/**
	 * The list of craft this player has Invaded during the game.
	 */
	private List<CraftQuantityData> craftInvadedList = new ArrayList();

	/**
	 * Has this player invaded the ancients planet?
	 */
	private boolean ancientsConquered;
	
	public GameScoreData() {
	}
	
	public GameScoreData(Hull built, int builtQty, Hull destroyed, int destroyedQty, Hull invaded, int invadedQty) throws ApplicationError {
		addCraftBuilt(built, builtQty);
		addCraftDestroyed(destroyed, destroyedQty);
		addCraftInvaded(invaded, invadedQty);
	}
	
	public CraftQuantityData getCraftBuilt(Hull hull) throws ApplicationError {
		if (craftBuiltList == null) return null; 
		int qty = craftBuiltList.size();
		for (int i = 0; i < qty; i++) {
			CraftQuantityData craftQuantityData = craftBuiltList.get(i);
			if (craftQuantityData.getHullTech() == hull) {
				return craftQuantityData; 
			}
		}
		return null;
	}

	public void setCraftBuilt(CraftQuantityData craftBuilt) {
		if (craftBuiltList == null) craftBuiltList = new ArrayList(); 
		this.craftBuiltList.add(craftBuilt);
	}

	public void addCraftBuilt(Hull hull, int qty) throws ApplicationError {
		CraftQuantityData cd = getCraftBuilt(hull);
		if (cd != null) {
			cd.increaseQty(qty);
		} else {
			setCraftBuilt(new CraftQuantityData(hull, qty));
		}
	}
	
	public List<CraftQuantityData> getCraftBuiltList() {
		return craftBuiltList;
	}

	public CraftQuantityData [] getCraftBuilt() {
		if (craftBuiltList == null || craftBuiltList.size() == 0) return null; 
		return craftBuiltList.toArray(new CraftQuantityData [craftBuiltList.size()]);
	}

	public void setCraftBuiltList(List<CraftQuantityData> craftBuilt) {
		this.craftBuiltList = craftBuilt;
	}
	
	
	public CraftQuantityData getCraftDestroyed(Hull hull) throws ApplicationError {
		if (craftDestroyedList == null) return null; 
		int qty = craftDestroyedList.size();
		for (int i = 0; i < qty; i++) {
			CraftQuantityData craftQuantityData = craftDestroyedList.get(i);
			if (craftQuantityData.getHullTech() == hull) {
				return craftQuantityData; 
			}
		}
		return null;
	}

	public void setCraftDestroyed(CraftQuantityData craftBuilt) {
		if (craftDestroyedList == null) craftDestroyedList = new ArrayList(); 
		this.craftDestroyedList.add(craftBuilt);
	}
	
	public void addCraftDestroyed(Hull hull, int qty) throws ApplicationError {
		CraftQuantityData cd = getCraftDestroyed(hull);
		if (cd != null) {
			cd.increaseQty(qty);
		} else {
			setCraftDestroyed(new CraftQuantityData(hull, qty));
		}
	}
	
	public List<CraftQuantityData> getCraftDestroyedList() {
		return craftDestroyedList;
	}

	public void setCraftDestroyedList(List<CraftQuantityData> craftDestroyed) {
		this.craftDestroyedList = craftDestroyed;
	}
	
	public CraftQuantityData [] getCraftDestroyed() {
		if (craftDestroyedList == null || craftDestroyedList.size() == 0) return null; 
		return craftDestroyedList.toArray(new CraftQuantityData [craftDestroyedList.size()]);
	}

	public CraftQuantityData getCraftInvaded(Hull hull) throws ApplicationError {
		if (craftInvadedList == null) return null; 
		int qty = craftInvadedList.size();
		for (int i = 0; i < qty; i++) {
			CraftQuantityData craftQuantityData = craftInvadedList.get(i);
			if (craftQuantityData.getHullTech() == hull) {
				return craftQuantityData; 
			}
		}
		return null;
	}

	public void setCraftInvaded(CraftQuantityData craftInvaded) {
		if (craftInvadedList == null) craftInvadedList = new ArrayList(); 
		craftInvadedList.add(craftInvaded);
	}
	
	public void addCraftInvaded(Hull hull, int qty) throws ApplicationError {
		CraftQuantityData cd = getCraftInvaded(hull);
		if (cd != null) {
			cd.increaseQty(qty);
		} else {
			setCraftInvaded(new CraftQuantityData(hull, qty));
		}
	}
	
	public List<CraftQuantityData> getCraftInvadedList() {
		return craftInvadedList;
	}

	public CraftQuantityData [] getCraftInvaded() {
		if (craftInvadedList == null || craftInvadedList.size() == 0) return null; 
		return craftInvadedList.toArray(new CraftQuantityData [craftInvadedList.size()]);
	}

	public void setCraftInvadedList(List<CraftQuantityData> v) {
		craftInvadedList = v;
	}

	public boolean isAncientsConquered() {
		return ancientsConquered;
	}

	public void setAncientsConquered(boolean ancientsConquered) {
		this.ancientsConquered = ancientsConquered;
	}

	public int getTotalQtyCraftsBuilt() {
		if (craftBuiltList == null) return 0;
		int r = 0;
		int qty = craftBuiltList.size();
		for (int i = 0; i < qty; i++) {
			CraftQuantityData craftQuantityData = craftBuiltList.get(i);
			r += craftQuantityData.getQty();
		}
		return r;
	}

	public int getTotalQtyCraftsDestroyed() {
		if (craftDestroyedList == null) return 0;
		int r = 0;
		int qty = craftDestroyedList.size();
		for (int i = 0; i < qty; i++) {
			CraftQuantityData craftQuantityData = craftDestroyedList.get(i);
			r += craftQuantityData.getQty();
		}
		return r;
	}

	public int getTotalQtyCraftsInvaded() {
		if (craftInvadedList == null) return 0;
		int r = 0;
		int qty = craftInvadedList.size();
		for (int i = 0; i < qty; i++) {
			CraftQuantityData craftQuantityData = craftInvadedList.get(i);
			r += craftQuantityData.getQty();
		}
		return r;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (craftBuiltList != null) {
			int qty = craftBuiltList.size();
			for (int i = 0; i < qty; i++) {
				CraftQuantityData craftQuantityData = craftBuiltList.get(i);
				craftQuantityData.toXML(writer, "craft-built");
			}
		}
		if (craftDestroyedList != null) {
			int qty = craftDestroyedList.size();
			for (int i = 0; i < qty; i++) {
				CraftQuantityData craftQuantityData = craftDestroyedList.get(i);
				craftQuantityData.toXML(writer, "craft-destroyed");
			}
		}
		if (craftInvadedList != null) {
			int qty = craftInvadedList.size();
			for (int i = 0; i < qty; i++) {
				CraftQuantityData craftQuantityData = craftInvadedList.get(i);
				craftQuantityData.toXML(writer, "craft-invaded");
			}
		}
		writer.addElement("ancients-conquered", ancientsConquered);
	}

	protected String getElementName() {
		return "game-score-data";
	}

}
