/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean.stats;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This class collected statistics for a given player so that a graph of time verses population,
 * production, research etc may be drawn for a player. A graph consists of a number of lines.
 */
public class Graph extends XMLBean implements Serializable {

    /**
     * This contains a list of lines contained in the graph.
     * Maps the line name to the Line object containing the list of statistics (date / value
     * pairs) from which a graph can be drawn.
     */
    private HashMap<String, Line> lines = new HashMap();
    
    /**
     * The last date a statistic was added to this graph.
     */
    private int lastDate;

    public Graph() {
    }
    
    /**
     * Adds a new statistic to the line with this name.
     * If no line of this name exists, new one is created.
     * @param name The name of the line
     * @param date The date of the statistic
     * @param value The value of the statistic
     */
    public void add(String name, int date, long value) {
        add(name, new Statistic(date, value));
    }

    /**
     * Adds a new statistic to the line with this name.
     * If no line of this name exists, new one is created.
     * @param name The name of the line
     * @param value The statistic to add to this line.
     */
    public void add(String name, Statistic value) {
        Line line = (Line)lines.get(name);
        if (line == null) {
            line = new Line(name);
            lines.put(name, line);
        }
        line.add(value);
    }

    /**
     * Adds the line to the list of lines in this graph.
     * @param line The line to add to the graph.
     */
    public void addLine(Line line) {
        String name = line.getName();
        lines.put(name, line);
    }

    public void setLine(Line line) {
        addLine(line);
    }
    
    /**
     * @param name The name of the line to search.
     * @return The statistic with the highest (most recent) date in the line with
     * the name passed in.
     */
    public Statistic getLast(String name) {
        Line line = (Line)lines.get(name);
        if (line == null) {
            return null;
        }
        return line.getLast();
    }
    
    /** 
     * @returns The minimum date of all lines in the graph.
     */
    public int getMinimumDate() {
        int min = -1;
        if (lines == null || lines.size() == 0) {
            return 3000;
        }
        Iterator i = lines.keySet().iterator();
        while (i.hasNext()) {
            String name = (String)i.next();
            Line line = (Line)lines.get(name);
            Statistic s = line.getFirst();
            if (s != null) {
                int t = s.getDate();
                if (min == -1 || t < min) {
                    min = t;
                }
            }
        }
        return min;
    }
    
    public int getMaximumValue(String name) {
        int max = -1;
        if (lines == null || lines.size() == 0) {
            return 100;
        }
        Line line = (Line)lines.get(name);
        if (line != null) {
            Statistic s = line.getBiggest();
            if (s != null) {
                int t = (int) s.getValue();
                if (max == -1 || t > max) {
                    max = t;
                }
            }
        }
        return max;
    }

    public Line [] getGraphLines() {
        if (lines == null || lines.size() == 0) {
            return null;
        } else {
            Set keys = lines.keySet();
            Iterator i = keys.iterator();
            Line [] r = new Line [keys.size()];
            int j = 0;
            while (i.hasNext()) {
                String k = (String) i.next();
                Line l = (Line) lines.get(k);
                r[j++] = l;
            }
            return r;
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("last-date", lastDate);
        Set keys = lines.keySet();
        Iterator i = keys.iterator();
        while (i.hasNext()) {
            String k = (String) i.next();
            Line line = (Line) lines.get(k);
            line.toXML(writer);
        }
	}

	protected String getElementName() { return "graph"; }


    public void setLastDate(int lastDate) {
        this.lastDate = lastDate;
    }


    public int getLastDate() {
        return lastDate;
    }

	public HashMap<String, Line> getLines() {
		return lines;
	}

	public void setLines(HashMap<String, Line> lines) {
		this.lines = lines;
	}
	
	public String toString() {
		return ""+lines;
	}
	
}