/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.bean.stats;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This class collected statistics for a given player so that a graph of time verses population,
 * production, research etc may be drawn for a player.
 */
public class Line extends XMLBean implements Serializable {

    /**
     * This is a list of statistic objects (date / value pairs) that can be used to draw a line
     * on a graph.
     * pairs from which a grap
     * h can be drawn.
     */
    private ArrayList stats;
    
    /**
     * This is the name of this line on the graph. On a graphical diagram is would probably be 
     * used as the key.
     */
    private String name;

    public Line() {
    }
    
    public Line(String name) {
        this.name = name;
    }
    
    /**
     * Adds a new statistic to this line.
     * @param value The statistic to add.
     */
    public void add(Statistic value) {
        if (stats == null) {
            stats = new ArrayList();
        }
        stats.add(value);
    }
    
    public void addStatistic(Statistic value) {
        add(value);
    }

    public void setStatistic(Statistic value) {
        add(value);
    }

    /**
     * @return The statistic with the highest (most recent) date in this line.
     */
    public Statistic getLast() {
        Statistic best = null;
        if (stats != null) {
            Iterator i = stats.iterator();
            while (i.hasNext()) {
                Statistic stat = (Statistic)i.next();
                if (best == null || stat.getDate() > best.getDate()) {
                    best = stat;
                }
            }
        }
        return best;
    }

    /**
     * @return The statistic with the highest value in this line.
     */
    public Statistic getBiggest() {
        Statistic best = null;
        if (stats != null) {
            Iterator i = stats.iterator();
            while (i.hasNext()) {
                Statistic stat = (Statistic)i.next();
                if (best == null || stat.getValue() > best.getValue()) {
                    best = stat;
                }
            }
        }
        return best;
    }

    /**
     * @return The statistic with the lowest (oldest) date in this line.
     */
    public Statistic getFirst() {
        if (stats != null) {
            return (Statistic) stats.get(0);
        }
        return null;
    }

    public Statistic [] getStatsData() {
        if (stats == null || stats.size() == 0) {
            return null;
        } else {
            return (Statistic []) stats.toArray(new Statistic[stats.size()]);
        }
    }


	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", name);
        if (stats != null) {
            Iterator i = stats.iterator();
            while (i.hasNext()) {
                Statistic stat = (Statistic)i.next();
                stat.toXML(writer);
            }
        }
	}

	protected String getElementName() { return "line"; }


    public void setName(String name) {
        this.name = name;
    }


    public String getName() {
        return name;
    }

	public ArrayList getStats() {
		return stats;
	}

	public void setStats(ArrayList stats) {
		this.stats = stats;
	}
}