/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.coms.protocols;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;


public class JDKEncoderDecoder implements XMLEncoderDecoder {

    /**
     * Converts a Java object to its serialized form.
     * @return The serialized data form, Typically XML.
     * @param o The Java object to be serialised.
     */
    public String toString(Object o) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        XMLEncoder d = new XMLEncoder(new BufferedOutputStream(baos));
        d.writeObject(o);
        d.close();
        String xml =  baos.toString();
        if (MotSocket.log) {
            Logger.logDebug("WRITE XML for "+o.getClass().getName()+":\n"+xml);
        }
        return xml;
    }
    
    /**
     * Converts serialized data to a Java object.
     * @return The Java object that has been serialised.
     * @param o The serialized data form, Typically XML..
     */
    public Object toObject(String xml) throws IOException {
        return toObject(new ByteArrayInputStream(xml.getBytes()));
    }

    /**
     * Converts serialized data to a Java object.
     * @return The Java object that has been serialised.
     * @param o The serialized data form, Typically XML..
     */
    public Object toObject(InputStream is) throws IOException {
        if (MotSocket.log) {
            is = _debug(is);
        }
        XMLDecoder d = new XMLDecoder(is);
        Object o = d.readObject();
        d.close();
        return o;
    }

    private static final int MAX_BUFF = 2048;

    private InputStream _debug(InputStream is) throws IOException {
        byte[] buffer = new byte[MAX_BUFF];
        StringBuffer sb = new StringBuffer();
        int bytesRead;
        do {
            bytesRead = is.read(buffer, 0, MAX_BUFF);
            if (bytesRead > 0) {
               sb.append(new String(buffer, 0, bytesRead));
            }
        } while (bytesRead >= 0);
        is.close();
        String xml = sb.toString();
        Logger.logDebug("READ XML:\n"+xml);
        return new ByteArrayInputStream(xml.getBytes());
    }

    /**
     * @return A string exactly 6 bytes containing the length of the contents 
     * of the serialized data.  
     */
    public String getLength(String xml) throws ApplicationError {
        if (xml == null) return "000000";
        int length = xml.length();
        if (length < 10) {
            return "00000"+length;
        } else if (length < 100) {
            return "0000"+length;
        } else if (length < 1000) {
            return "000"+length;
        } else if (length < 10000) {
            return "00"+length;
        } else if (length < 100000) {
            return "0"+length;
        } else if (length < 1000000) {
            return ""+length;
        } else {
            throw new ApplicationError("Size overflow error");
        }
    }
    
}