/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.coms.protocols;

import mot.client.coms.SendResponseToGuiThread;
import mot.client.event.MotClientEventProcessor;
import mot.client.proxy.CommandResponseProcessor;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.RemoteConnectionServerProxy;
import mot.common.command_delegates.CommandDeligate;
import mot.common.command_results.CommandResult;
import mot.common.event.MotEvent;
import mot.common.event.ServerConnectionErrorEvent;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;

public class RemoteServerConnection implements ServerConnection {

	private MotSocket socket;
	
	/**
	 * The client object that will process each command response coming back from the server.
	 */
    private CommandResponseProcessor responseProcessor;
    
	/**
	 * The client object that will process all events coming back from the server.
	 */
	private MotClientEventProcessor eventProcessor;
	
    private SocketHandlerThreadClient receiveThread;

	public RemoteServerConnection(CommandResponseProcessor responseProcessor, MotClientEventProcessor eventProcessor) throws ApplicationError, ValidationError {
		this.responseProcessor = responseProcessor;		
		this.eventProcessor = eventProcessor;
    }

	@Override
    public void connect(String ip) throws ApplicationError, ValidationError {
        socket = new MotSocket(ip);
        
		if (receiveThread == null) {
			receiveThread = new SocketHandlerThreadClient();
			receiveThread.start();
		}
    }
    
	@Override
	public void disconnect() {
	}

	@Override
	public boolean isConnected() {
		return socket != null;
	}


//	@Override
//	public void register(MotClientEventProcessor eventProcessor) {
//		this.eventProcessor = eventProcessor;
//	}
	
    public void send(CommandDeligate param) throws ApplicationError, ValidationError {
    	if (socket == null) {
    		throw new ValidationError(L.NOT_TO_CONNECT_TO_A_GAME_SERVER);
    	}
        try {
            if (param != null) {
            	socket._write(param);
            }
        } catch (Exception e) {
            Logger.logError("Connection to server failed: ",e);
            throw new ApplicationError("Failed to connect to the server");
        }
    }    
    
	public LocalServerProxy getLocalServerProxy() {
		return null;
	}
	
	public RemoteConnectionServerProxy getRemoteConnectionServerProxy() {
		return null;
	}
	
	public boolean supportsRemoteConnection() {
		return false;
	}


    private class SocketHandlerThreadClient extends Thread {
    	
    	private int logCount = 0;
    	
    	public SocketHandlerThreadClient() {
    		super("SocketHandlerThreadClient");
    	}
    	
    	public void run() {
    		while (true) {
    			try {
    				Object o = socket._read();
    				logCount = 0;
    				if (o instanceof CommandResult) {
    					CommandResult cr = (CommandResult) o;
    					SendResponseToGuiThread.send(responseProcessor, cr);
    				} else if (o instanceof MotEvent) {
    					if (eventProcessor == null) {
    	    				Logger.logError("RemoteServerConnection.ReceiveThread.run() No Event Processor: "+o);
    					} else {
    						eventProcessor.process((MotEvent) o);
    					}
    				}
    			} catch (Throwable t) {
    				if (logCount < 10) {
    					Logger.logError("RemoteServerConnection.ReceiveThread.run() "+t.getMessage(), t);
    					if (logCount == 0) {
    						try {
    							eventProcessor.process(new ServerConnectionErrorEvent(t.getMessage()));
    						} catch (ApplicationError e) {
    							Logger.logError("RemoteServerConnection.ReceiveThread.run() "+t.getMessage(), t);
    						}
    					}
    					logCount++;
    				}
    			}
    		}
    		
    		
    	}
    }


}