/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.designs;

import java.io.Serializable;
import java.util.ArrayList;

import mot.client.deligates.PlanetDelegate;
import mot.common.bean.MotComparable;
import mot.common.bean.NameList;
import mot.common.bean.NamedObject;
import mot.common.bean.SalvageList;
import mot.common.technology.Computer;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.TechnologyContainer;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBundle;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.vo.CraftDesignVO;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;


/**
 * Hold the design of a craft as produced in a factory. Note that crafts contain their
 * own copied of all the information in its design so they can be re-fitted! A craft design
 * may not be identical to the craft design of that name.
 */
public class CraftDesign extends NamedObject implements ProjectDesign, MotComparable, Serializable {

	/**
	 * Allows the default image defined for this hull to be overridden.
	 */
	private String image;

	/**
	 * The craft hull.
	 */
	private String hull;

	/**
	 * The engines on this movable object.
	 */
	private String engine;

	/**
	 * The computers mounted on this craft.
	 */
	private NameList computers;

	/**
	 * The weapons mounted on this craft.
	 */
	private WeaponBaysDesign weaponBays;

	/**
	 * The shields mounted on this craft.
	 */
	private NameList shields;

	/**
	 * The scanners mounted on this craft.
	 */
	private NameList scanners;

	/**
	 * The transporters etc. mounted on this craft.
	 */
	private NameList specials;
	
	public CraftDesign() {
	}

	public CraftDesign(String name) {
		super(name);
	}

	public CraftDesign(String name, Technology... techs) throws ApplicationError {
		super(name);
		for (int i = 0; i < techs.length; i++) {
			if (techs[i] instanceof Hull) {
		        setHull((Hull) techs[i]);
			} else if (techs[i] instanceof Engine) {
		        setEngine((Engine) techs[i]);
			} else if (techs[i] instanceof Computer) {
		        setComputer((Computer) techs[i]);
			} else if (techs[i] instanceof Weapon) {
		        setWeapon((Weapon) techs[i]);
			} else if (techs[i] instanceof Scanner) {
		        setScanner((Scanner) techs[i]);
			} else if (techs[i] instanceof Special) {
		        setSpecial((Special) techs[i]);
			} else if (techs[i] instanceof Shield) {
		        setShield((Shield) techs[i]);
			}
		}
		if (getHull() == null) throw new ApplicationError("Craft Design must have a Hull");
	}

    /**
     * Copy constructor
     */
	public CraftDesign(CraftDesign cd) throws ApplicationError {
        this(cd, true, true, true);
    }

    /**
     * Copy constructor
     */
	public CraftDesign(CraftDesign cd, boolean withComputers, boolean withScanners, boolean withShields) throws ApplicationError {
		super(cd.getName());
        setImage(cd.getImage());
        setHull(cd.getHull());
        setEngine(cd.getEngine());
        if (cd.getWeaponBays() != null) setWeaponBays(new WeaponBaysDesign(cd.getWeaponBays()));
        if (cd.getSpecials() != null) setSpecials(new Specials(cd.getSpecials()));
        if (cd.getComputers() != null && withComputers) setComputers(new Computers(cd.getComputers()));
        if (cd.getShields() != null && withShields) setShields(new Shields(cd.getShields()));
        if (cd.getScanners() != null && withScanners) setScanners(new Scanners(cd.getScanners()));
	}
	
	public CraftDesign(CraftDesignVO vo) throws ApplicationError {
		super(vo.getName());
        setImage(vo.getImage());
        setHull(vo.getHull());
        setEngine(vo.getEngine());
        if (vo.getWeapons() != null) setWeaponBays(new WeaponBaysDesign(vo.getWeapons()));
        if (vo.getSpecials() != null) setSpecials(new NameList(vo.getSpecials()));
        if (vo.getComputers() != null) setComputers(new NameList(vo.getComputers()));
        if (vo.getShields() != null) setShields(new NameList(vo.getShields()));
        if (vo.getScanners() != null) setScanners(new NameList(vo.getScanners()));
	}

	public CraftDesign(DomNode n) throws ApplicationError {
		super(n.getNodeValueByName_String("name"));
        setImage(n.getNodeValueByName_String("image"));
        setHull(n.getNodeValueByName_String("hull"));
        setEngine(n.getNodeValueByName_String("engine"));
        
        DomNode wbn = n.getNodeByName("weapon-bays");
        if (wbn != null) setWeaponBays(new WeaponBaysDesign(wbn));
        
        String [] specials = n.getNodeValuesByName_String("special");
        if (specials != null) {
        	for (int i = 0; i < specials.length; i++) {
				setSpecial(specials[i]);
			}
        }
        
        String [] computers = n.getNodeValuesByName_String("computer");
        if (computers != null) {
        	for (int i = 0; i < computers.length; i++) {
				setComputer(computers[i]);
			}
        }
        
        String [] shields = n.getNodeValuesByName_String("shield");
        if (shields != null) {
        	for (int i = 0; i < shields.length; i++) {
				setShield(shields[i]);
			}
        }
        
        String [] scanners = n.getNodeValuesByName_String("scanner");
        if (scanners != null) {
        	for (int i = 0; i < scanners.length; i++) {
				setScanner(scanners[i]);
			}
        }        
	}

	public String getImage() { return image; }
	public String getHull() { return hull; }
	public String getEngine() { return engine; }
	public NameList getComputers() { return computers; }
	public WeaponBaysDesign getWeaponBays() { return weaponBays; }
	public NameList getShields() { return shields; }
	public NameList getScanners() { return scanners; }
	public NameList getSpecials() { return specials; }
	
	public void setImage(String v) { image = v; }
	public void setHull(String v) { hull = v; }
	public void setEngine(String v) { engine = v; }
	public void setComputers(NameList v) { computers = v; }
	public void setWeaponBays(WeaponBaysDesign v) { weaponBays = v; }
	public void setShields(NameList v) { shields = v; }
	public void setScanners(NameList v) { scanners = v; }
	public void setSpecials(NameList v) { specials = v; }

	public void setHull(Hull v) { hull = v == null ? null : v.getName(); }
	public void setEngine(Engine v) { engine = v == null ? null : v.getName(); }
	public void setComputer(Computer v) { if (v != null) { if (computers == null) computers = new NameList(); computers.add(v.getName()); } }
	public void setWeapon(Weapon v) { if (weaponBays == null) weaponBays = new WeaponBaysDesign(); weaponBays.addWeapon(v,0); }
	public void setShield(Shield v) { if (v != null) { if (shields == null) shields = new NameList(); shields.add(v.getName()); } }
	public void setScanner(Scanner v) { if (v != null) { if (scanners == null) scanners = new NameList(); scanners.add(v.getName()); } }
	public void setSpecial(Special v) { if (v != null) { if (specials == null) specials = new NameList(); specials.add(v.getName()); } }
   
	public String getHullName() { return (hull == null) ? "None" : hull; }
	public String getComputersName() { return (computers == null) ? "None" : computers.getNamesSeparated(" ,"); }
	public String getEngineName() { return (engine == null) ? "None" : engine; }
	public String getWeaponsName() { return (weaponBays == null) ? "None" : weaponBays.getWeaponNames(); }
	public String getShieldsName() { return (shields == null) ? "None" : shields.getNamesSeparated(" ,"); }
	public String getScannersName() { return (scanners == null) ? "None" : scanners.getNamesSeparated(" ,"); }
	public String getSpecialsName() { return (specials == null) ? "None" : specials.getNamesSeparated(" ,"); }

	public String getHullTechName() { return hull; }
	public String getEngineTechName() { return engine; }
	public String [] getComputersTechNames() { return (computers == null) ? null : computers.getNamesAsArray(); }
	public String [] getShieldsTechNames() { return (shields == null) ? null : shields.getNamesAsArray(); }
	public String [] getScannersTechNames() { return (scanners == null) ? null : scanners.getNamesAsArray(); }
	public String [] getSpecialsTechNames() { return (specials == null) ? null : specials.getNamesAsArray(); }
	public ArrayList<String> getComputerNamesAsList() { return (computers == null) ? null : computers.getNames(); }
	public ArrayList<String> getShieldNamesAsList() { return (shields == null) ? null : shields.getNames(); }
	public ArrayList<String> getScannerNamesAsList() { return (scanners == null) ? null : scanners.getNames(); }
	public ArrayList<String> getSpecialNamesAsList() { return (specials == null) ? null : specials.getNames(); }

	public int getQtyWeapons() throws ApplicationError {
		if (weaponBays != null) {
			return weaponBays.getQtyWeapons();
		} else {
			return 0;
		}
	}
	
	public Weapon [] getWeapons() throws ApplicationError {
		if (weaponBays != null) {
			return weaponBays.getAllWeapons();
		} else {
			return null;
		}
	}
	
	public Weapons getWeaponTechs() throws ApplicationError {
		if (weaponBays != null) {
			return weaponBays.getAllWeaponsAsList();
		} else {
			return null;
		}
	}

	public Weapons getWeaponTechsNoDuplicates() throws ApplicationError {
		if (weaponBays != null) {
			return weaponBays.getAllWeaponsNoDuplicates();
		} else {
			return null;
		}
	}

	public WeaponBundle [] getDistinctWeapons() throws ApplicationError {
		if (weaponBays != null) {
			return weaponBays.getDistinctWeapons();
		} else {
			return null;
		}
	}

	public void setWeapon(Weapon v, int bayNo, int qty) { 
        if (weaponBays == null) weaponBays = new WeaponBaysDesign();
        for (int i=0; i<qty;i++) {
            weaponBays.addWeapon(v, bayNo);
        }
    }

	public void fillWithWeapon(Weapon v, int bayNo) throws ApplicationError { 
        if (weaponBays == null) weaponBays = new WeaponBaysDesign();
        int space = v.getSpace();
        int qty = getRemainingSpace() / space; 
        for (int i=0; i<qty; i++) {
            weaponBays.addWeapon(v, bayNo);
        }
    }

	public void setComputer(String v) throws ApplicationError {
        if (computers == null) computers = new NameList();
		computers.add(v);
	}
	
	public void setShield(String v) throws ApplicationError {
        if (shields == null) shields = new NameList();
		shields.add(v);
	}
	
	public void setScanner(String v) throws ApplicationError {
        if (scanners == null) scanners = new NameList();
		scanners.add(v);
	}
	
	public void setSpecial(String v) throws ApplicationError {
        if (specials == null) specials = new NameList();
		specials.add(v);
	}

	public void addSpecials(Special v, int qty) throws ApplicationError {
        if (specials == null) specials = new NameList();
        while (qty > 0) {
        	setSpecial(v);
        	qty--;
        }
	}

	public Hull getHullTech() throws ApplicationError {
		if (hull != null) {
			return Technologies.getHull(hull);
		} else {
			return null;
		}
	}
	
	public Engine getEngineTech() throws ApplicationError {
		if (engine != null) {
			return Technologies.getEngine(engine);
		} else {
			return null;
		}
	}
	
	public Computers getComputerTechs() throws ApplicationError {
        if (computers == null) { 
        	return null;
        } else {        	
        	return new Computers(computers);
        }
	}

	public Computers getComputerTechsNoDuplicates() throws ApplicationError {
        if (computers == null) { 
        	return null;
        } else {        	
        	Computers r = new Computers(computers);
        	r.removeDuplicates();
			return r;
        }
	}

	public Shields getShieldTechs() throws ApplicationError {
        if (shields == null) { 
        	return null;
        } else {        	
        	return new Shields(shields);
        }
	}

	public Shields getShieldTechsNoDuplicates() throws ApplicationError {
        if (shields == null) { 
        	return null;
        } else {        	
        	Shields r = new Shields(shields);
        	r.removeDuplicates();
			return r;
        }
	}

	public Scanners getScannerTechs() throws ApplicationError {
        if (scanners == null) { 
        	return null;
        } else {        	
        	return new Scanners(scanners);
        }
	}

	public Scanners getScannerTechsNoDuplicates() throws ApplicationError {
        if (scanners == null) { 
        	return null;
        } else {        	
        	Scanners r = new Scanners(scanners);
        	r.removeDuplicates();
			return r;
        }
	}

	public Specials getSpecialTechs() throws ApplicationError {
        if (specials == null) { 
        	return null;
        } else {        	
        	return new Specials(specials);
        }
	}

	public Specials getSpecialTechsNoDuplicates() throws ApplicationError {
        if (specials == null) { 
        	return null;
        } else {        	
        	Specials r = new Specials(specials);
        	r.removeDuplicates();
			return r;
        }
	}

	public void setComputers(Computers v) throws ApplicationError {
        if (v == null) { 
        	computers = null;
        } else {        	
        	computers = v.getNamesList();
        }
	}

	public void setShields(Shields v) throws ApplicationError {
        if (v == null) { 
        	shields = null;
        } else {        	
        	shields = v.getNamesList();
        }
	}

	public void setScanners(Scanners v) throws ApplicationError {
        if (v == null) { 
        	scanners = null;
        } else {        	
        	scanners = v.getNamesList();
        }
	}

	public void setSpecials(Specials v) throws ApplicationError {
        if (v == null) { 
        	specials = null;
        } else {        	
        	specials = v.getNamesList();
        }
	}

	public TechnologyBundle[] getUsableSalvage(DesignBuilder planet, SalvageList planetOwnersSalvage) throws ApplicationError {
		if (planetOwnersSalvage == null) return null;
        TechnologyBundle[] usableSalvage = planetOwnersSalvage.getDistinct(this);
        return usableSalvage;
	}

    public int getCostToBuildOnPlanet() throws ApplicationError {
        return getCostToBuild();
    }

	/**
	 * Calculates the cost to build this craft design.
	 * @throws ApplicationError 
	 */
	public int getCostToBuild() throws ApplicationError {
    	Hull hullTech = getHullTech();
    	Engine engineTech = getEngineTech();
    	Computers computerTechs = getComputerTechs();
    	Weapons weaponTechs = getWeaponTechs();
    	Shields shieldTechs = getShieldTechs();
    	Scanners scannerTechs = getScannerTechs();
    	Specials specialTechs = getSpecialTechs();
    	int cost = 0;
		double factor = 1.0;
    	if (hullTech != null) {
    		cost += hullTech.getCostToBuild();
    		factor = hullTech.getCostToBuildFactor();
    	}
    	if (engineTech != null) cost += engineTech.getCostToBuild(factor);
    	if (computerTechs != null) cost += computerTechs.getCostToBuild(factor);
    	if (weaponTechs != null) cost += weaponTechs.getCostToBuild(factor);
    	if (shieldTechs != null) cost += shieldTechs.getCostToBuild(factor);
    	if (scannerTechs != null) cost += scannerTechs.getCostToBuild(factor);
    	if (specialTechs != null) cost += specialTechs.getCostToBuild(factor);
		return cost;
	}

	/**
	 * Calculates the cost to build this craft design.
	 * @throws ApplicationError 
	 */
	public int getCostToBuildUsingExistingTechs(TechnologyList freeTechs) throws ApplicationError {
		WorkTupple work = getTechsToBuild(freeTechs);
		Technology[] ta = work.buildList.getAllAsArray();
		Hull h = Technologies.getHull(hull);
		if (h == null || ta == null) return 0;
		double factor = h.getCostToBuildFactor();
		int cost = 0;
		for (int i = 0; i < ta.length; i++) {
			cost += ta[i].getCostToBuild(factor);
		}
		return cost;
	}
	

	/**
	 * Note - If this craft has 10 mass drivers then the list returned will have two mass drivers.
	 * However, if we already have 3 mass drivers built, these should be in freeTechs so we only 
	 * need to build 7 mass drivers and this method will return 7.
	 * @param freeTechs These are technologies that can be used to build this craft that do not 
	 * need to be built.
	 * @return The work that needs to be done to construct a craft of this design.
	 * @throws ApplicationError
	 */
	private WorkTupple getTechsToBuild(TechnologyList freeTechs) throws ApplicationError {
		WorkTupple result = new WorkTupple();
		Hull hullTech = getHullTech();
		result.fill(hullTech, freeTechs);
		Engine engineTech = getEngineTech();
		result.fill(engineTech, freeTechs);
		Computers computerTechs = getComputerTechs();
		if (computerTechs != null) {
			result.fill(computerTechs.getDistinct(), freeTechs);
		}
		WeaponBaysDesign weaponBays = getWeaponBays();
		if (weaponBays != null) {
			result.fill(weaponBays.getDistinctWeapons(), freeTechs);
		}
		Shields shieldTechs = getShieldTechs();
		if (shieldTechs != null) {
			result.fill(shieldTechs.getDistinct(), freeTechs);
		}
		Scanners scannerTechs = getScannerTechs();
		if (scannerTechs != null) {
			result.fill(scannerTechs.getDistinct(), freeTechs);
		}
		Specials specialTechs = getSpecialTechs();
		if (specialTechs != null) {
			result.fill(specialTechs.getDistinct(), freeTechs);
		}
		return result;

	}

	public int getRemainingSpace() throws ApplicationError {
    	return getRemainingSpace(getHullTech(), getEngineTech(), getComputerTechs(), weaponBays, getShieldTechs(), getScannerTechs(), getSpecialTechs());
    }
    
    public static int getRemainingSpace(Hull hull, Engine engine, Computers computers, WeaponBaysDesign weaponBays, Shields shields, Scanners scanners, Specials specials) throws ApplicationError {
        int totalSpace = hull.getSpace();
		double factor = hull.getCostToBuildFactor();
		int space = (engine != null ? engine.getSpace(factor) : 0) +
                    (computers != null ? computers.getSpace() : 0) +
                    (weaponBays != null ? weaponBays.getSpace() : 0) +
                    (shields != null ? shields.getSpace() : 0) +
                    (scanners != null ? scanners.getSpace() : 0) +
                    (specials != null ? specials.getSpace() : 0);
        return totalSpace - space;
    }
    
    public void fill(Technology tech) throws ApplicationError {
        if (tech == null) {
            Logger.logWarning("CraftDesign.fill() Cannot add technology null");
            return;
        }
        if (Logger.isDebug()) {
            Logger.logDebug("CraftDesign.fill() type="+tech.getTechType()+" name="+tech.getName());
        }
    	int spaceHave = getRemainingSpace();
    	int spaceTech = tech.getSpace();
    	int qty = spaceHave / spaceTech;
    	for (int i=0; i < qty; i++) {
    		add(tech);
    	}
    }

    /**
     * Adds a new piece of technology to this craft design.
     * If it is a hull or engine then it replaces the existing tech, otherwise it is added
     * to the list of technologies of that type. If it is a weapon then it is added
     * to the first weapons bay.
     */
    public void add(Technology tech) throws ApplicationError {
        add(tech, false);
    }
    
    /**
     * Adds a new piece of technology to this craft design.
     * If it is a hull or engine then it replaces the existing tech, otherwise it is added
     * to the list of technologies of that type. If it is a weapon then it is added
     * to the first weapons bay.
     * If validate is true then It checks that the technology being added to this design is 
     * not a duplicate. For example it is not legal to have special AugumentedEnginesI and
     * special AugumentedEnginesII.
     */
    public void add(Technology tech, boolean replace) throws ApplicationError {
        if (tech == null) {
            Logger.logWarning("CraftDesign.add() Cannot add technology null");
            return;
        }
        if (Logger.isDebug()) {
            Logger.logDebug("CraftDesign.add() type="+tech.getTechType()+" name="+tech.getName());
        }
        if (tech instanceof Weapon) {
            addWeapon((Weapon)tech);
        } else if (tech instanceof Shield) {
            if (replace) {
                String group = tech.getGroup();
                Specials specialTechs = getSpecialTechs();
                if (group != null && specialTechs != null) {
                    if (specialTechs.containsGroup(group)) {
                    	specialTechs.removeGroup(group);
                    	setSpecials(specialTechs);
                    }
                }
            }
            if (shields == null) shields = new NameList();
            shields.add(tech.getName());
        } else if (tech instanceof Computer) {
            if (replace) {
                String group = tech.getGroup();
                Computers computerTechs = getComputerTechs();
                if (group != null && computerTechs != null) {
                    if (computerTechs.containsGroup(group)) {
                    	computerTechs.removeGroup(group);
                    	setComputers(computerTechs);
                    }
                }
            }
            if (computers == null) computers = new NameList();
            computers.add(tech.getName());
        } else if (tech instanceof Scanner) {
            if (replace) {
                String group = tech.getGroup();
                Scanners scannerTechs = getScannerTechs();
                if (group != null && scannerTechs != null) {
                    if (scannerTechs.containsGroup(group)) {
                    	scannerTechs.removeGroup(group);
                    	setScanners(scannerTechs);
                    }
                }
            }
            if (scanners == null) scanners = new NameList();
            scanners.add(tech.getName());
        } else if (tech instanceof Special) {
            if (replace) {
                String group = tech.getGroup();
                Specials specialTechs = getSpecialTechs();
                if (group != null && specialTechs != null) {
                    if (specialTechs.containsGroup(group)) {
                    	specialTechs.removeGroup(group);
                    	setSpecials(specialTechs);
                    }
                }
            }
            if (specials == null) specials = new NameList();
            specials.add(tech.getName());
        } else if (tech instanceof Hull) {
            hull = tech.getName();
        } else if (tech instanceof Engine) {
            engine = tech.getName();
        } else {
            Logger.logWarning("CraftDesign.add() Cannot add technology "+tech.getName());
        }
    }

    /**
     * Adds a new piece of technology to this craft design.
     * It is added to the first weapons bay.
     */
    public void addWeapon(Weapon tech) throws ApplicationError {
        if (weaponBays == null) weaponBays = new WeaponBaysDesign();
        if (weaponBays.getQty() < 1) {
            weaponBays.setWeaponBay(new WeaponBayDesign());
        }
        weaponBays.addWeapon(tech, 0);
    }

    /**
     * Removes an existing piece of technology from this craft design.
     */
    public void remove(Technology tech) throws ApplicationError {
        if (tech == null) {
            Logger.logWarning("CraftDesign.remove() Cannot add technology null");
            return;
        }
        if (Logger.isDebug()) {
            Logger.logDebug("CraftDesign.remove() type="+tech.getTechType()+" name="+tech.getName());
        }
        if (tech instanceof Weapon) {
            removeWeapon((Weapon)tech);
        } else if (tech instanceof Shield) {
            if (shields == null) return;
            shields.remove(tech);
        } else if (tech instanceof Computer) {
            if (computers == null) return;
            computers.remove(tech);
        } else if (tech instanceof Scanner) {
            if (scanners == null) return;
            scanners.remove(tech);
        } else if (tech instanceof Special) {
            if (specials == null) return;
            specials.remove(tech);
        } else if (tech instanceof Hull) {
            // Nothing
        } else if (tech instanceof Engine) {
            // Nothing
        } else {
            Logger.logWarning("CraftDesign.remove() Cannot remove technology "+tech.getName());
        }
    }

    public void removeAll(Technology tech) throws ApplicationError {
        if (tech == null) {
            Logger.logWarning("CraftDesign.removeAll() Cannot remove technology null");
            return;
        }
        if (Logger.isDebug()) {
            Logger.logDebug("CraftDesign.removeAll() type="+tech.getTechType()+" name="+tech.getName());
        }
        if (tech instanceof Weapon) {
            if (weaponBays == null) return;
            weaponBays.removeAll((Weapon)tech);
        } else if (tech instanceof Shield) {
            if (shields == null) return;
            shields.removeAll(tech.getName());
        } else if (tech instanceof Computer) {
            if (computers == null) return;
            computers.removeAll(tech.getName());
        } else if (tech instanceof Scanner) {
            if (scanners == null) return;
            scanners.removeAll(tech.getName());
        } else if (tech instanceof Special) {
            if (specials == null) return;
            specials.removeAll(tech.getName());
        } else if (tech instanceof Hull) {
            // Nothing
        } else if (tech instanceof Engine) {
        	// Nothing
        } else {
            Logger.logWarning("CraftDesign.remove() Cannot add technology "+tech.getName());
        }
    }
    
    /**
     * Adds a new piece of technology to this craft design.
     * It is added to the first weapons bay.
     */
    public void removeWeapon(Weapon tech) throws ApplicationError {
        if (weaponBays == null) return;
        weaponBays.removeFirstWeapon(tech, 0);
    }

    /**
     * Removes all weapons from this craft design.
     */
    public void removeAllWeapons() {
        weaponBays = null;
    }

    public boolean canBeBuiltBy(TechnologyContainer player) throws ApplicationError {
        if (getHull() != null) {
            if (player.getHulls() == null) return false;
            if ( !player.getHulls().contains(getHull()) ) return false;
        }
        if (getEngine() != null) {
            if (player.getEngines() == null) return false;
            if ( !player.getEngines().contains(getEngine()) ) return false;
        }
        if (getWeaponBays() != null) {
            if (player.getWeapons() == null) return false;
            if ( !player.getWeapons().contains(getWeaponBays().getAllWeapons()) ) return false;
        }
        if (getComputers() != null) {
            if (player.getComputers() == null) return false;
            if ( !player.getComputers().contains(getComputerTechs()) ) return false;
        }
        if (getShields() != null) {
            if (player.getShields() == null) return false;
            if ( !player.getShields().contains(getShieldTechs()) ) return false;
        }
        if (getScanners() != null) {
            if (player.getScanners() == null) return false;
            if ( !player.getScanners().contains(getScannerTechs()) ) return false;
        }
        if (getSpecials() != null) {
            if (player.getSpecials() == null) return false;
            if ( !player.getSpecials().contains(getSpecialTechs()) ) return false;
        }
        return true;
	}

	public boolean canColonise() throws ApplicationError {
        Specials s = getSpecialTechs();
        if (s == null) return false;
		return (s.getCanColoniseRating() != null);
	}

	public boolean canColonise(PlanetDelegate planet) throws ApplicationError {
        return canColonise(planet.getTeraforming());
	}

	public boolean canColonise(double terraformRating) throws ApplicationError {
        Specials s = getSpecialTechs();
        if (s == null) return false;
		Double x = s.getCanColoniseRating();
        if (x == null) return false;
		double canColoniseRating = x.doubleValue();
        return (terraformRating >= canColoniseRating);
    }
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		if (hull != null) {
            writer.addElement("hull", hull);
		}
		if (engine != null) {
            writer.addElement("engine", engine);
		}
		if (computers != null) {
			String[] na = computers.getNamesAsArray();
			if (na != null) {
				for (int i = 0; i < na.length; i++) {					
					writer.addElement("computer", na[i]);
				}
			}
		}
		if (weaponBays != null) {
			weaponBays.toXML(writer);
		}
		if (shields != null) {
			String[] na = shields.getNamesAsArray();
			if (na != null) {
				for (int i = 0; i < na.length; i++) {					
					writer.addElement("shield", na[i]);
				}
			}
		}
		if (scanners != null) {
			String[] na = scanners.getNamesAsArray();
			if (na != null) {
				for (int i = 0; i < na.length; i++) {					
					writer.addElement("scanner", na[i]);
				}
			}
		}
		if (specials != null) {
			String[] na = specials.getNamesAsArray();
			if (na != null) {
				for (int i = 0; i < na.length; i++) {					
					writer.addElement("special", na[i]);
				}
			}
		}
	}

	protected String getElementName() { return "craft-design"; }


    /**
     * Convenient for debugging and tracing....
     */
    public void log(String playerName) throws ApplicationError {
        StringBuffer sb = new StringBuffer();
        if (getHull() != null) sb.append("\nHull ="+getNameAndSpace(getHullTech()));
        if (getEngine() != null) sb.append("\nEngine ="+getNameAndSpace(getEngineTech()));
        if (getComputers() != null) sb.append("\nComputers ="+getNameAndSpace(getComputerTechs()));
        if (getShields() != null) sb.append("\nShields ="+getNameAndSpace(getShieldTechs()));
        if (getScanners() != null) sb.append("\nScanners ="+getNameAndSpace(getScannerTechs()));
        if (getSpecials() != null) sb.append("\nSpecials ="+getNameAndSpace(getSpecialTechs()));
        if (getWeaponBays() != null) sb.append("\nWeapons = "+getWeaponBays().getWeaponNames()+"[ space "+getWeaponBays().getSpace()+"]");
        Logger.logDebug("Player "+playerName+" Designed New Craft "+getName()+" [remaining space="+getRemainingSpace()+"]"+sb.toString());
    }

    private String getNameAndSpace(Technology tech) {
        return tech.getName() + " [space "+tech.getSpace()+"]";
    }
    
    private String getNameAndSpace(TechnologyList techs) {
        if (techs == null || techs.getQuantity() == 0) return "None";
        Technology [] array = techs.getAllAsArray(true);
        StringBuffer sb = new StringBuffer();
        for (int i=0; i<array.length; i++) {
            sb.append(getNameAndSpace(array[i]));
            sb.append(", ");
        }
        return sb.toString() + "[total space "+techs.getSpace()+"]";
    }
    
    /**
     * @return True if these two designs are the same object or have the same technologies.
     */
	public boolean same(CraftDesign other) {
		if (other == null) return false;
		if (!Comp.same(hull, other.getHull())) {
			return false;
		}
		if (!Comp.same(engine, other.getEngine())) {
			return false;
		}
		if (!Comp.same(computers, other.getComputers())) {
			return false;
		}
		if (weaponBays != other.getWeaponBays()) {
			if (weaponBays == null ||  !weaponBays.same(other.getWeaponBays())) {
				return false;
			}
		}
		if (!Comp.same(shields, other.getShields())) {
			return false;
		}
		if (!Comp.same(scanners, other.getScanners())) {
			return false;
		}
		if (!Comp.same(specials, other.getSpecials())) {
			return false;
		}
		return true;
	}
    
	public String getProjectElementName() {
        return getElementName();
    }

	public void update(CraftDesign newDesign) throws ApplicationError {
		if (newDesign == null) return;
		hull = newDesign.getHull();
		engine = newDesign.getEngine();
		if (newDesign.getComputers() != null) {
			computers = new NameList(newDesign.getComputers());
		} else {
			computers = null;
		}
		if (newDesign.getShields() != null) {
			shields = new NameList(newDesign.getShields());
		} else {
			shields = null;
		}
		if (newDesign.getScanners() != null) {
			scanners = new NameList(newDesign.getScanners());
		} else {
			scanners = null;
		}
		if (newDesign.getSpecials() != null) {
			specials = new NameList(newDesign.getSpecials());
		} else {
			specials = null;
		}
		if (newDesign.getWeaponBays() != null) {
			weaponBays = new WeaponBaysDesign(newDesign.getWeaponBays());
		} else {
			weaponBays = null;
		}
	}

	public boolean same(MotComparable other) {
		return other != null && other instanceof CraftDesign && same((CraftDesign)other);
	}

	public int getQuantityWeaponsWithGroup(String group) throws ApplicationError {
		if (weaponBays != null) {
			return weaponBays.getQuantityWeaponsWithGroup(group);
		}
		return 0;
	}

	public TechnologyList getAllTechnologiesNoDuplicates() throws ApplicationError {
		TechnologyList r = new GeneralTechnologyList();
		r.add(getHullTech());
		r.add(getEngineTech());
		r.add(getWeaponTechsNoDuplicates());
		r.add(getComputerTechsNoDuplicates());
		r.add(getShieldTechsNoDuplicates());
		r.add(getScannerTechsNoDuplicates());
		r.add(getSpecialTechsNoDuplicates());
		return r;
	}

	public boolean contains(Technology t) throws ApplicationError {
		if (t == null) return false;
		String name = t.getName();
		if (t == getHullTech()) return true;
		if (t == getEngineTech()) return true;
		if (weaponBays != null) if (weaponBays.contains(name)) return true;
		if (computers != null) if (computers.contains(name)) return true;
		if (shields != null) if (shields.contains(name)) return true;
		if (scanners != null) if (scanners.contains(name)) return true;
		if (specials != null) if (specials.contains(name)) return true;
		return false;
	}

	public int getQty(Technology t) throws ApplicationError {
		if (t == null) return 0;
		String name = t.getName();
		if (t == getHullTech()) return 1;
		if (t == getEngineTech()) return 1;
		if (t instanceof Weapon) {
			if (weaponBays != null) return weaponBays.getQuantity(name);
		} else if (t instanceof Computer) {
			if (computers != null) return computers.getQuantity(name);
		} else if (t instanceof Shield) {
			if (shields != null) return shields.getQuantity(name);
		} else if (t instanceof Scanner) {
			if (scanners != null) return scanners.getQuantity(name);
		} else if (t instanceof Special) {
			if (specials != null) return specials.getQuantity(name);
		}
		return 0;
	}

	public boolean same(ProjectDesign project) {
		if (project == null) return false;
		if  (!(project instanceof CraftDesign)) return false;
		CraftDesign other = (CraftDesign) project;
		return same(other);
	}

	public boolean isMilitary() throws ApplicationError {
		return containsWeapons() || containsFighters();
	}

    public boolean containsFighters() throws ApplicationError {
        Specials specials = getSpecialTechs();
        if (specials == null) return false;
        return (specials.getQtyFighters() != null);
    }

    public boolean containsWeapons() {
        WeaponBaysDesign weaponBays = getWeaponBays();
        if (weaponBays == null) return false;
        return (weaponBays.getQtyWeapons() > 0);
    }

	


}