/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.designs;

import java.util.ArrayList;
import java.util.Iterator;

import mot.client.deligates.CraftDelegate;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.DomFactory;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLDocumentHandler;
import mot.common.xml.XMLWriter;

/**
 * Contains the list of designed craft.
 * To get the list of pre-defined craft designs use the static method getInstance().
 */
public class CraftDesigns extends XMLBean {

    private static final String XML_FILE = "xml/craft_designs.xml";

	private ArrayList list;
    
    private static CraftDesigns instance = null;

	/**
	 * Get the list of predefined craft designs.
	 */
	public static synchronized CraftDesigns getInstance() throws ApplicationError {
        if (instance == null) {
            instance = (CraftDesigns) XMLDocumentHandler.parseResource(XML_FILE);
        }
        return instance;
	}

	public static synchronized void setInstance(CraftDesigns cds) {
		instance = cds;
	}
	
	public static void preLoad() throws ApplicationError {
		instance = new CraftDesigns(DomFactory.load(XML_FILE));
	}

	public static void flush() {
		instance = null;
	}

    public CraftDesigns() {
    }

    public CraftDesigns(CraftDesign [] cds) {
        if (cds != null && cds.length > 0) {
            list = new ArrayList();
            for (int i=0; i<cds.length; i++) {
                list.add(cds[i]);
            }
        }
    }

    public CraftDesigns(DomTree doc) throws ApplicationError {
    	DomNode root = doc.getNodeByName("craft-designs");
    	
    	DomNode[] cda = root.getNodesByName("craft-design");
    	if (cda != null) {
    		for (int i = 0; i < cda.length; i++) {
    			setCraftDesign(new CraftDesign(cda[i]));
			}
    	}
	}

	public ArrayList getList() { return list; }
    public void setList(ArrayList v) { list = v; }
    
    /**
     * If a craft design of the same name as the craft design passed in exists that 
     * design is replaced by the design passed in. Otherwise the design passed in is added to the list.
     * @param cd The craft design to add or update.
     */
    public void addOrUpdate(CraftDesign cd) {
        CraftDesign old = get(cd.getName());
        if (old == null) {
            setCraftDesign(cd);
        } else {
            list.remove(old);
            setCraftDesign(cd);
        }
    }

    public void add(CraftDesign cd) {
        setCraftDesign(cd);
    }
    
    public void setCraftDesign(CraftDesign cd) {
        if (list == null) list = new ArrayList();
        list.add(cd);
        craftDesigns = null;
    }

    public void remove(String cdName) {
        remove(get(cdName));
    }

    public void remove(CraftDesign cd) {
    	if (cd == null) return;
        if (list == null) return;
        list.remove(cd);
        craftDesigns = null;
    }

    private transient CraftDesign [] craftDesigns;
    public CraftDesign [] getCraftDesigns() {
		if (craftDesigns == null && list != null) {
			craftDesigns = (CraftDesign []) list.toArray( new CraftDesign[list.size()]);
		}
		return craftDesigns;
	}

    public CraftDesign get(String name) { return find(name); }
    
    public CraftDesign find(String name) {
		CraftDesign [] craftDesigns = getCraftDesigns();
        
        if (craftDesigns == null) {
            return null;
        }
        
		for (int i=0; i<craftDesigns.length; i++) {
			if ( Comp.same(name, craftDesigns[i].getName()) ) {
				return craftDesigns[i];
			}
		}

		return null;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                CraftDesign cd  = (CraftDesign) i.next();
                cd.toXML(writer);
            }
        }
	}

	protected String getElementName() { return "craft-designs"; }


	public int getQuantity() {
		if (list == null) {
			return 0;
		} else {
			return list.size();
		}
	}

	public CraftDesigns getCraftUpgradeDesignsFor(MoveableOwnedObjectDelegate moo) throws ApplicationError {
		CraftDesign[] cda = getCraftDesigns();
		if (cda == null) return null;
		CraftDesigns r = null;
		for (int i = 0; i < cda.length; i++) {
			if (includeCraftDesign(cda[i], moo)) {
				if (r == null) r = new CraftDesigns();
				r.add(cda[i]);
			}
		}
		return r;
	}
	
	private boolean includeCraftDesign(CraftDesign craftDesign, MoveableOwnedObjectDelegate moo) throws ApplicationError {
		if (moo instanceof CraftDelegate) {
			CraftDelegate craft = (CraftDelegate) moo;
			return canUpgradeTo(craft, craftDesign);
		} else if (moo instanceof FleetDelegate) {
			FleetDelegate fleet = (FleetDelegate)moo;
			CraftDelegate[] crafts = fleet.getCrafts();
			if (crafts != null) {
				for (int i = 0; i < crafts.length; i++) {
					if (canUpgradeTo(crafts[i], craftDesign)) {
						return true;
					}
				}
			}
		}
		return false;
	}

	public static boolean canUpgradeTo(CraftDelegate craft, CraftDesign craftDesign) throws ApplicationError {
		if (craft.getCraftDesign().getHullTech() != craftDesign.getHullTech()) {
			return false;
		}
		if (craft.getCraftDesign().same(craftDesign)) {
			return false;
		}
		return true;
	}

}