/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.designs;

import java.io.Serializable;
import java.util.ArrayList;

import mot.common.bean.MotComparable;
import mot.common.bean.NamedObject;
import mot.common.bean.SalvageList;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Hull;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Specials;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.CraftVO;
import mot.common.xml.XMLWriter;

public class CraftUpgradeDesign extends NamedObject implements ProjectDesign, MotComparable, Serializable {
	
	/**
	 * The design to upgrade crafts to
	 */
	private CraftDesign design;
	
	/**
	 * The crafts to upgrade
	 */
	private CraftVO [] crafts;

	public CraftUpgradeDesign() {
		super("Craft Upgrade");
	}
	
	public CraftUpgradeDesign(CraftDesign cd) {
		super("Craft Upgrade");
		design = cd;
	}

	public CraftUpgradeDesign(CraftDesign cd, CraftVO craft) {
		super("Craft Upgrade");
		design = cd;
		crafts = new CraftVO[1];
		crafts[0] = craft;
	}

	public CraftUpgradeDesign(CraftDesign cd, CraftVO craft1, CraftVO craft2) {
		super("Craft Upgrade");
		design = cd;
		crafts = new CraftVO[2];
		crafts[0] = craft1;
		crafts[1] = craft2;
	}

	public void addCraft(CraftVO craft) throws ApplicationError {
		if (crafts == null) {
			crafts = new CraftVO[1];
			crafts[0] = craft;
		} else {
			CraftVO[] old = crafts;
			crafts = new CraftVO[old.length+1];
			for (int i = 0; i < old.length; i++) {
				crafts[i] = old[i];
			}
			crafts[old.length] = craft;
		}
	}
	
	public void addCrafts(CraftUpgradeDesign other) throws ApplicationError {
		CraftVO[] oc = other.getCrafts();
		ArrayList r = new ArrayList();
		if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {
				r.add(crafts[i]);
			}
		}
		if (oc != null) {
			for (int i = 0; i < oc.length; i++) {
				r.add(oc[i]);
			}
		}
		if (r.size() > 0) {
			crafts = (CraftVO[]) r.toArray(new CraftVO[r.size()]);
		} else {
			crafts = null;
		}
	}

	@Override
	public boolean same(MotComparable other) {
		if (other == null || !(other instanceof CraftUpgradeDesign)) {			
			return false;
		}
		CraftUpgradeDesign o = (CraftUpgradeDesign) other;
		if (!o.getDesign().same(getDesign())) {
			return false;
		}
		if (o.getCrafts() == null && crafts == null) {
			return true;
		}
		if (o.getCrafts() == null || crafts == null) {
			return false;
		}
		for (int i = 0; i < crafts.length; i++) {
			if (!o.containsCraft(crafts[i].getName())) {
				return false;
			}
		}
		return true;
	}

	public boolean containsCraft(String name) {
		if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {
				if (Comp.same(name, crafts[i].getName())) {
					return true;
				}
			}			
		}
		return false;
	}

	@Override
	public int getCostToBuildOnPlanet() throws ApplicationError {
		int r = 0;
		if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {
				r += getCostToBuildOnPlanet(crafts[i]);
			}			
		}
		return r;
	}

	private int getCostToBuildOnPlanet(CraftVO craft) throws ApplicationError {
		if (design == null) {
			return 0;
		}
		GeneralTechnologyList newFreeTechs = new GeneralTechnologyList();
		Hull hull = craft.getHull();
		newFreeTechs.add(hull);
		Engine engine = craft.getEngine();
		newFreeTechs.add(engine);
		Computers computers = craft.getComputers();
		newFreeTechs.add(computers);
		Weapons weapons = craft.getWeapons();
		newFreeTechs.add(weapons);
		Shields shields = craft.getShields();
		newFreeTechs.add(shields);
		Scanners scanners = craft.getScanners();
		newFreeTechs.add(scanners);
		Specials specials = craft.getSpecials();
		newFreeTechs.add(specials);
		return design.getCostToBuildUsingExistingTechs(newFreeTechs);
	}

	@Override
	protected String getElementName() {
		return "craft-design-upgrade";
	}

	public CraftDesign getDesign() {
		return design;
	}

	public void setDesign(CraftDesign design) {
		this.design = design;
	}

	public void setCraftDesign(CraftDesign design) {
		this.design = design;
	}

	public void setCraft(CraftVO craft) {
		if (crafts == null || crafts.length == 0) {
			crafts = new CraftVO[1];
			crafts[0] = craft;
		} else {
			CraftVO[] nca = new CraftVO[crafts.length+1];
			for (int i = 0; i < crafts.length; i++) {
				nca[i] = crafts[i];
			}
			nca[crafts.length] = craft;
			crafts = nca;
		}
	}

	public CraftVO [] getCrafts() {
		return crafts;
	}

	public void setCrafts(CraftVO [] crafts) {
		this.crafts = crafts;
	}

	public int getQtyCrafts() {
		if (crafts == null || crafts.length == 0) {
			return 0;			
		} else {
			return crafts.length;			
		}
	}
	
	public boolean upgradingCraftInAFleet() {
		if (crafts == null || crafts.length == 0) {
			return false;			
		} else {
			for (int i = 0; i < crafts.length; i++) {
				if (crafts[i].getFleet() != null) {
					return true;
				}
			}
		}
		return false;
	}

	public int getCostToBuild(SalvageList salvage) throws ApplicationError {
		if (design == null || crafts == null || crafts.length == 0) {
			return 0;
		}
		WorkTupple work = new WorkTupple();
		for (int i = 0; i < crafts.length; i++) {
			work = getTechsToBuild(crafts[i], salvage, work);
		}
		Technology[] ta = work.buildList.getAllAsArray();
		Hull h = design.getHullTech();
		if (h == null) return 0;
		double factor = h.getCostToBuildFactor();
		int cost = 0;
		if (ta != null) {
			for (int i = 0; i < ta.length; i++) {
				cost += ta[i].getCostToBuild(factor);
			}
		}
		return cost;
	}

	public TechnologyList getFreeTechsUsedInBuild(SalvageList salvage) throws ApplicationError {
		if (design == null || crafts == null || crafts.length == 0) {
			return null;
		}
		WorkTupple work = new WorkTupple();
		for (int i = 0; i < crafts.length; i++) {
			work = getTechsToBuild(crafts[i], salvage, work);
		}
		return work.useList;
	}

	/**
	 * Note - If this craft has 10 mass drivers then the list returned will have two mass drivers.
	 * However, if we already have 3 mass drivers built, these should be in freeTechs so we only 
	 * need to build 7 mass drivers and this method will return 7.
	 * @param freeTechs These are technologies that can be used to build this craft that do not 
	 * need to be built.
	 * @return The work that needs to be done to construct a craft of this design.
	 * @throws ApplicationError
	 */
	public WorkTupple getTechsToBuild(CraftVO craft, TechnologyList freeTechs, WorkTupple result) throws ApplicationError {
		if (result == null) result = new WorkTupple();
		Engine engineTech = design.getEngineTech();
		if (craft.getEngine() != engineTech) {
			result.fill(engineTech, freeTechs);
		}
		Computers computerTechs = design.getComputerTechs();
		if (computerTechs != null) {
			result.fill(computerTechs.getDistinct(craft.getComputers()), freeTechs);
		}
		WeaponBaysDesign weaponBays = design.getWeaponBays();
		if (weaponBays != null) {
			result.fill(weaponBays.getDistinct(craft.getWeapons()), freeTechs);
		}
		Shields shieldTechs = design.getShieldTechs();
		if (shieldTechs != null) {
			result.fill(shieldTechs.getDistinct(craft.getShields()), freeTechs);
		}
		Scanners scannerTechs = design.getScannerTechs();
		if (scannerTechs != null) {
			result.fill(scannerTechs.getDistinct(craft.getScanners()), freeTechs);
		}
		Specials specialTechs = design.getSpecialTechs();
		if (specialTechs != null) {
			result.fill(specialTechs.getDistinct(craft.getSpecials()), freeTechs);
		}
		return result;

	}

	
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		if (design != null) {
			design.toXML(writer);
		}
		if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {
				crafts[i].toXML(writer, "craft");
//				if (crafts[i] instanceof CraftVOAdapter) {
//					CraftVO cvo = new CraftVO(crafts[i].getName(), crafts[i].getOwner(), crafts[i].getLocation(), crafts[i].getNavigator(), crafts[i].getOrbit(), crafts[i].getVisibility(), crafts[i].getRange(),
//							crafts[i].getOrigin(), crafts[i].getFleet(), crafts[i].getDesign(), crafts[i].getMaintenanceCost(), crafts[i].getDamageAsPercent(), crafts[i].getBuildQualityAsPercent(),
//							crafts[i].isTemporaryCraft());
//					cvo.toXML(writer, "craft");
//				} else {
//					crafts[i].toXML(writer, "craft");
//				}
			}
		}
	}

	public boolean same(ProjectDesign project) {
		if (project == null) return false;
		return (project instanceof CraftUpgradeDesign && Comp.same(design.getName(), ((CraftUpgradeDesign)project).getDesign().getName()));
	}

}
