/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.designs;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.swing.window.common.MotInternalWindow;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;

/**
 * Factory class used to create project designs. Using a technology this class has methods
 * to create each type of project design.
 */
public class DesignFactory {

	private MotInternalWindow parent;

    public DesignFactory(MotInternalWindow parent) {
		this.parent = parent;
    }

    public ProjectDesign createCraftProjectDesign(UserPlayerDelegate owner, String craftName) throws ApplicationError {
        if (Comp.same(craftName, owner.getTransportName())) {
            TransportDesign transport = new TransportDesign();
            return transport;
        } else {
            CraftDesigns cds = owner.getCraftDesigns();
            if (cds == null) return null;
            CraftDesign cd = cds.find(craftName);
            return cd;
        }
    }

    public ProjectDesign createWeaponProjectDesign(UserPlayerDelegate owner, PlanetDelegate planet, String weaponName) throws ApplicationError, ValidationError {
        Weapons l = owner.getWeapons();
        if (l != null) {
            Weapon w = l.getWeapon(weaponName);
            Weapons pl = planet.getWeapons();
            return new WeaponDesign(w);
        }
        return null;
    }

    public ProjectDesign createWeaponProjectDesign(UserPlayerDelegate owner, PlanetDelegate planet, Weapon w) throws ApplicationError, ValidationError {
        Weapons l = owner.getWeapons();
        if (l != null) {
            Weapons pl = planet.getWeapons();
            return new WeaponDesign(w);
        }
        return null;
    }

    public ProjectDesign createShieldProjectDesign(UserPlayerDelegate owner, PlanetDelegate planet, Shield w) throws ApplicationError, ValidationError {
        Shields l = owner.getShields();
        if (l != null) {
            Shields pl = planet.getShields();
            if (planet.getShields() != null && planet.getShields().containsGroup(w.getGroup())) {
                throw new ValidationError(L.YOU_CANNOT_ADD_THIS_SHIELD_BECUASE_YOU_ALREADY_HAVE_ONE_OF_THIS_GROUP);
            }
            return new ShieldDesign(w);
        }
        return null;
    }

    public ProjectDesign createShieldProjectDesign(UserPlayerDelegate owner, PlanetDelegate planet, String shieldName) throws ApplicationError, ValidationError {
        Shields l = owner.getShields();
        if (l != null) {
            Shield w = l.getShield(shieldName);
            Shields pl = planet.getShields();
            if (planet.getShields() != null && planet.getShields().containsGroup(w.getGroup())) {
                throw new ValidationError(L.YOU_CANNOT_ADD_THIS_SHIELD_BECUASE_YOU_ALREADY_HAVE_ONE_OF_THIS_GROUP);
            }
            return new ShieldDesign(w);
        }
        return null;
    }

    public ProjectDesign createScannerProjectDesign(UserPlayerDelegate owner, PlanetDelegate planet, Scanner w) throws ApplicationError, ValidationError {
        Scanners l = owner.getScanners();
        if (l != null) {
            Scanners pl = planet.getScanners();
            if (planet.getScanners() != null && planet.getScanners().containsGroup(w.getGroup())) {
                throw new ValidationError(L.YOU_CANNOT_ADD_THIS_SCANNER_BECUASE_YOU_ALREADY_HAVE_ONE_OF_THIS_GROUP);
            }
            return new ScannerDesign(w);
        }
        return null;
    }

    public ProjectDesign createScannerProjectDesign(UserPlayerDelegate owner, PlanetDelegate planet, String scannerName) throws ApplicationError, ValidationError {
        Scanners l = owner.getScanners();
        if (l != null) {
            Scanner w = l.getScanner(scannerName);
            Scanners pl = planet.getScanners();
            if (planet.getScanners() != null && planet.getScanners().containsGroup(w.getGroup())) {
                throw new ValidationError(L.YOU_CANNOT_ADD_THIS_SCANNER_BECUASE_YOU_ALREADY_HAVE_ONE_OF_THIS_GROUP);
            }
            return new ScannerDesign(w);
        }
        return null;
    }

    public ProjectDesign createStationProjectDesign(UserPlayerDelegate owner, PlanetDelegate planet, String hullName) throws ApplicationError, ValidationError {
        if (owner.getHulls() == null) return null;
        Hull station = owner.getHulls().getHull(hullName);
        return createStationProjectDesign(planet, station);
    }

    public ProjectDesign createStationProjectDesign(PlanetDelegate planet, Hull station) throws ApplicationError, ValidationError {
        if (station == null) {
            return null;
        }
        if (station.isBattleStation()) { 
            return new BattleStationDesign(station);
        } else if (station.isOutpost()) {
            return new OutpostDesign(station); 
        } else if (station.isSpaceStation()) {
            return new SpaceStationDesign(station); 
        }
        return null;
    }

//    private Technology getTechnologyToUpgrade(TechnologyList tl, Technology t) throws ValidationError, ApplicationError {
//        if (tl == null) {
//            throw new ValidationError(L.THERE_ARE_NOT_TECHNOLOGIES_OF_THIS_TYPE_TO_UPGRADE);
//        }
//        
//        if (designer.manualUpgradeSelection()) {
//            return getTechnologyToUpgrade(tl.getDistinct());
//        } else {
//            if (!tl.containsGroup(t.getGroup())) {
//               throw new ValidationError(L.THERE_ARE_NOT_TECHNOLOGIES_OF_THIS_GROUP_TO_UPGRADE);
//            }
//            return tl.getTechnologyForGroup(t.getGroup());
//        }
//    }
//
//    private Technology getTechnologyToUpgrade(TechnologyBundle[] tb) throws ValidationError, ApplicationError {
//        Weapon old = null;
//        if (tb == null) {
//            throw new ValidationError(L.THERE_ARE_NOT_TECHNOLOGIES_OF_THIS_TYPE_TO_UPGRADE);
//        } else if (tb.length == 1) {
//            return tb[0].getTechnology();                    
//        } else {
//            String [] techNames = new String[tb.length];
//            for (int i=0; i<tb.length;i++) {
//                techNames[i] = tb[i].getName();
//            }
//            String techName = designer.selectUpgrade(techNames, parent);
//            for (int i=0; i<tb.length;i++) {
//                if (Comp.same(tb[i].getName(), techName)) {
//                    return tb[i].getTechnology();
//                }
//            }
//            return null;
//        }
//    }
}