/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.designs;

import java.io.Serializable;
import java.util.ArrayList;

import mot.common.bean.CraftGraphics;
import mot.common.bean.PlayerColor;
import mot.common.factory.GraphicsFactory;
import mot.common.resources.Resource;
import mot.common.technology.Hull;
import mot.common.technology.Module;
import mot.common.technology.Modules;
import mot.common.technology.Scanner;
import mot.common.technology.Technologies;
import mot.common.technology.TechnologyContainer;
import mot.common.technology.TechnologyReference;
import mot.common.technology.TechnologyReferenceList;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * An orbital is an object like a battle station or space station in orbit around a planet.
 * There are no real differences between a battle station and a space station, the purpose
 * of a orbital is defined by the modules it contains.
 * 
 * A battle station is an orbital that contains a hull and a population module.
 * A space station is an orbital that contains a hull, population module, scanner, and mining / research modules.
 * A listening post is an orbital that contains a scanner only.
 * 
 * NOTE - Modules must be added to an orbital using the addModule() in this class. You
 * must not use getModules().add(). The serialised state will not be updates.
 */
public class Orbital extends XMLBean implements Serializable {

	/**
	 * This is the amount of influence added to the planet that owns this orbital / mining station.
	 * It is a fixed amount (does not accumulate).
	 * It is also the initial influence added to a planet when a station is deployed.  
	 */
    private static final int INFLUENCE = PropertyManager.getMandatoryNoErrorAsInt("influence.space_station",1);

	/**
	 * This is the amount of influence added to the planet this orbital is mining per turn.
	 * The actual influence used is accumulated on the mining colony. 
	 */
    private static final double INFLUENCE_GROWTH = PropertyManager.getMandatoryNoErrorAsDouble("influence_growth.space_station",0.01);

    private transient Hull hullTech;
    private String hull;
    
    private transient Scanner scannerTech;
    private String scanner;
    
    private transient Modules moduleTechs;
    private ArrayList<String> modules;

    public Orbital() {
    }

    public Orbital(Hull hull, Scanner scanner, Modules modules) {
        setHull(hull);
        setScanner(scanner);
        setModules(modules);
    }
    
    /**
     * @return The name of the image to be used to displays this craft. This should be the
     * name of an image file in this games distribution (see images folder).
     */
    public String getImage(String playerName, PlayerColor playerColour) throws ApplicationError {
        String colorName  = "";
        if (playerColour != null) colorName = playerColour.getName();
        Hull station = getHullTech();
        CraftGraphics outpostGraphics = GraphicsFactory.getStationGraphics(playerName, station);
        String image = CraftGraphics.CRAFT_IMAGE_FOLDER+outpostGraphics.getPictureNormal() + "_" + colorName + ".png";
        return image;
    }
    
    public static final double MIN_PROD_BONUS_FOR_HABITAT = 1.0;
    
    public int getPopulation(double productionBonus, int moneyBonus) throws ApplicationError {
        Modules modules = getModuleTechs();
        if (modules == null) {
            return 0;
        } else {
            int pop = modules.getPopulation();
			if (productionBonus >= MIN_PROD_BONUS_FOR_HABITAT || moneyBonus > 0) {
                return pop;
            } else {
                return pop / 2;
            }
        }

    }
    
    public int getProduction() throws ApplicationError {
        Modules modules = getModuleTechs();
        if (modules == null) {
            return 0;
        } else {
            return modules.getProduction();
        }
    }
    
    public int getResearch() throws ApplicationError {
        Modules modules = getModuleTechs();
        if (modules == null) {
            return 0;
        } else {
            return modules.getResearch();
        }
    }
    
    /**
     * You may not add duplicate modules to an orbital. You may add to habitat modules of a 
     * different type (but same group) but you may not add two habitat II modules.
     * @return True if this orbital can have module added to its list of modules and false otherwise. 
     * @param module The module that may or may not be added to this orbital
     */
    public boolean canHaveModule(Module module) throws ApplicationError {
        Modules modules = getModuleTechs();
        if (modules == null) {
            return true;
        } else {
            return modules.contains(module);
        }
    }

    public boolean hasModule(Module module) throws ApplicationError {
        Modules modules = getModuleTechs();
        if (modules == null) {
            return false;
        } else {
            return modules.contains(module);
        }
    }
    

    /**
     * This method is called if a planet that has this object in orbit is colonised.
     * If there are any particular modules that cannot be used on a populated planet then
     * this would be a great place to check (and remove them).
     */
    public void notifyPlanetColonised() {
        
    }
    
    public String getName() throws ApplicationError {
        Hull hull = getHullTech();
        if (hull == null) {
            return "None";
        } else {
            return hull.getName();
        }
    }

    public int getMaintenanceCost() throws ApplicationError {
        Hull hull = getHullTech();
        if (hull == null || hull.getMaintenanceCost() == null) {
            return 0;
        } else {
            return hull.getMaintenanceCost().intValue();
        }
    }

    public void setHull(Hull v) {
        this.hullTech = v;
        if (v != null) {
            hull = v.getName();
        } else {
        	hull = null; 
        }
    }

    public Hull getHullTech() throws ApplicationError {
        if (hullTech == null && getHull() != null) {
            hullTech = Technologies.getHull(getHull());
            return hullTech;
        } else {
            return hullTech;
        }
    }

    public void setScanner(Scanner v) {
        this.scannerTech = v;
        if (v != null) {
            scanner = v.getName();
        } else {
            scanner = null;
        }
    }


    public Scanner getScannerTech() throws ApplicationError {
        if (scannerTech == null && getScanner() != null) {
            scannerTech = Technologies.getScanner(getScanner());
            return scannerTech;
        } else {
            return scannerTech;
        }
    }


    public void setModules(Modules v) {
        this.moduleTechs = v;
        if (v != null) {
            modules = v.getNamesAsList();
        } else {
            modules = null;
        }
    }

    public Modules getModuleTechs() throws ApplicationError {
        if (moduleTechs == null && getModules() != null) {
            moduleTechs = new Modules(getModules());
        }
        return moduleTechs;
    }
    
    public void setModule(Module module) throws ApplicationError {
    	addModule(module);
    }
    
    public void addModule(Module module) throws ApplicationError {
    	getModuleTechs();
        if (moduleTechs == null) moduleTechs = new Modules();
        if (!moduleTechs.contains(module)) {
            moduleTechs.add(module);
        }
        setModules(moduleTechs.getNamesAsList());
    }


    /**
     * @return the strength of the scanners placed on this planet.
     * This determines how far out this planets view of the galaxy extends.
     */
	public int getScanStrength() throws ApplicationError {
        getScannerTech();
		if (scannerTech == null) {
			return 0;
		} else {
			return scannerTech.getStrength();
		}
	}

    protected void encode(XMLWriter writer) throws ApplicationError {
        if (getHull() != null) writer.addElement("hull", getHull());
        if (getScanner() != null) writer.addElement("scanner", getScanner());
        if (getModules() != null) {
        	for (String moduleName : getModules()) {
        		writer.addElement("module", moduleName);
        	}
        }
    }

    protected String getElementName() {
        return "orbital";
    }

	public int getInfluence(Resource resource) {
		if (resource != null) {
			return INFLUENCE * 2;
		} else {
			return INFLUENCE;
		}
	}

	public double getInfluenceGrowth(Resource resource) {
		if (resource != null) {
			return INFLUENCE_GROWTH * 2;
		} else {
			return INFLUENCE_GROWTH;
		}
	}
	
	public boolean isAsGood(Hull otherHull) throws ApplicationError {
		if (otherHull == null) return true;
		Hull thisHull = getHullTech();
		return thisHull != null && thisHull.getMaxDamage() >= otherHull.getMaxDamage();
	}

	private static final String DEFAULT_HABITAT_TECH = "Core Habitat";

	public void addCoreModule() throws ApplicationError {
		Modules m = Technologies.getModules();
		if (m == null) return;
		Module module = m.getModule(DEFAULT_HABITAT_TECH);
		if (module == null) return;
		if (moduleTechs == null || !moduleTechs.contains(module)) {
			addModule(module);
		}
	}


	public void addSpaceStationModules(TechnologyContainer tc) throws ApplicationError {
		if (tc == null) return;
		Modules m = tc.getModules();
		if (m == null) return;
		Module[] ma = m.getModules();
		if (ma == null) return;
		for (int i = 0; i < ma.length; i++) {
			if (ma[i].isOnSpaceStation()) {
				addModule(ma[i]);
			}
		}
	}

	public void addBattleStationModules(TechnologyContainer tc) throws ApplicationError {
		if (tc == null) return;
		Modules m = tc.getModules();
		if (m == null) return;
		Module[] ma = m.getModules();
		if (ma == null) return;
		for (int i = 0; i < ma.length; i++) {
			if (ma[i].isOnBattleStation()) {
				addModule(ma[i]);
			}
		}
	}

	public void addOutpostModules(TechnologyContainer tc) throws ApplicationError {
		if (tc == null) return;
		Modules m = tc.getModules();
		if (m == null) return;
		Module[] ma = m.getModules();
		if (ma == null) return;
		for (int i = 0; i < ma.length; i++) {
			if (ma[i].isOnOutpost()) {
				addModule(ma[i]);
			}
		}
	}

	public String getHull() {
		return hull;
	}

	public void setHull(String hull) {
		this.hull = hull;
		hullTech = null;
	}

	public void setHullReference(TechnologyReference hull) {
		this.hull = hull.getName();
		hullTech = null;
	}

	public String getScanner() {
		return scanner;
	}

	public void setScanner(String scanner) {
		this.scanner = scanner;
		scannerTech = null;
	}
	
	public void setScannerReference(TechnologyReference scanner) {
		if (scanner != null) {
			setScanner(scanner.getName());
		}
	}

	public int getQtyFighterBays() throws ApplicationError {
		if (modules == null) return 0;
		int r = 0;
		for (String m : modules) {
			Module module = Technologies.getModules().getModule(m);
			if (module.getQtyFighterBays() != null) {
				r += module.getQtyFighterBays();
			}
		}
		return r;
	}

	public int getQtySentinals() throws ApplicationError {
		if (modules == null) return 0;
		int r = 0;
		for (String m : modules) {
			Module module = Technologies.getModules().getModule(m);
			if (module.getQtySentinels() != null) {
				r += module.getQtySentinels();
			}
		}
		return r;
	}

	public ArrayList<String> getModules() {
		return modules;
	}

	public void setModules(ArrayList<String> modules) {
		this.modules = modules;
		moduleTechs = null;
	}

	public void setModule(String module) throws ApplicationError {
		setModule(Technologies.getModule(module));
	}
	
	public void setModulesReference(TechnologyReferenceList modules) throws ApplicationError {
		if (modules != null) {
			TechnologyReference[] ta = modules.getAllAsArray();
			if (ta != null) {
				for (int j = 0; j < ta.length; j++) {					
					setModuleReference(ta[j]);
				}
			}
		}
	}

	public void setModuleReference(TechnologyReference module) throws ApplicationError {
		if (module != null) {
			setModule(module.getName());
		}
	}

}