/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.designs;

import java.io.Serializable;

import mot.common.bean.NameList;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.WeaponBayVO;
import mot.common.xml.DomNode;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This contains a list of weapons in a particular weapons bay on a craft.
 * Each craft hull type has a number of bays, and each hull type may have restrictions
 * on the things that may be stored in different bays and the quantity allowed. Also
 * some bays may be more accurate than others for different types of weapon.
  * 
 * SIMPLE SUGGESTION - 
 *      Only one type of weapon is allowed in a weapon bay?
 *      Hull defines how many weapons bays a craft has and each has different accuracy details.
*/
public class WeaponBayDesign extends XMLBean implements Serializable {

	/**
	 * The list of weapons.
	 */
	private NameList weapons;
	
	public WeaponBayDesign() {
	}

	public WeaponBayDesign(WeaponBayDesign wb) throws ApplicationError {
		weapons = new NameList( wb.getWeapons() );
	}

	public WeaponBayDesign(Weapons weapons) {
		this.weapons = (weapons == null ? null : weapons.getNamesList());
	}

	public WeaponBayDesign(NameList weapons) {
		this.weapons = weapons;
	}

	public WeaponBayDesign(WeaponBayVO wb) throws ApplicationError {
		if (wb != null) {
			weapons = wb.getNamesList();
		}
	}

	public WeaponBayDesign(DomNode n) throws ApplicationError {
        String [] weapons = n.getNodeValuesByName_String("weapon");
        if (weapons != null) {
        	for (int i = 0; i < weapons.length; i++) {
				setWeapon(weapons[i]);
			}
        }
	}

	public NameList getWeapons() { return weapons; }
	public void setWeapons(NameList v) { weapons = v; }

	public void setWeapons(Weapons v) {
		if (v != null) {
			Weapon[] wa = v.getWeapons();
			if (wa != null) {
				for (int i = 0; i < wa.length; i++) {
			        if (weapons == null) weapons = new NameList();
					weapons.add(wa[i].getName());	
				}
			}
		}
	}

	public Weapons getWeaponsList() throws ApplicationError {
		if (weapons == null || weapons.getQuantity() == 0) {
			return null;
		} else {
			return new Weapons(weapons.getNamesAsArray());
		}
	}

	public void setWeapon(String v) throws ApplicationError {
        if (weapons == null) weapons = new NameList();
		weapons.add(v);
	}
    
    public void add(Weapon w) {
        setWeapon(w);
    }
    
    public void setWeapon(Weapon w) {
    	if (w != null) {
	        if (weapons == null) weapons = new NameList();
			weapons.add(w.getName());
    	}
    }

    public void removeFirst(Weapon w) {
    	if (w != null) {
	        if (weapons == null) return;
			weapons.removeFirst(w.getName());
    	}
    }
    
	public void removeAll(Weapon w) {
        if (weapons == null || w == null) return;
		weapons.removeAll(w.getName());
	}


	/**
     * @return The total quantity of weapons
     */
	public int getQtyWeapons() {
		if (weapons == null) {
			return 0;
		} else {
			return weapons.getQuantity();
		}
	}
    
	public int getQuantityWeaponsWithGroup(String group) throws ApplicationError {
		int r = 0;
		if (weapons != null) {
			String[] na = weapons.getNamesAsArray();
			if (na != null) {
				for (int i = 0; i < na.length; i++) {
					Weapon weapon = Technologies.getWeapon(na[i]);
					if ( Comp.same(weapon.getGroup(), group) ) {
						r++;
					}
				}
			}
		}
		return r;
	}
    
	public int getQuantity(String name) throws ApplicationError {
		if (weapons != null) {
			return weapons.getQuantity(name);
		}
		return 0;
	}
	
    public boolean contains(String name) throws ApplicationError {
        Weapon w = Technologies.getWeapon(name);
        return contains(w);
    }

    public boolean contains(Weapon w) throws ApplicationError {
        if (weapons == null || w == null) {
            return false;
        } else {
            return weapons.contains(w.getName());
        }
    }

	public boolean containsGroup(String groupName) throws ApplicationError {
		if (weapons != null) {
			String[] na = weapons.getNamesAsArray();
			if (na != null) {
				for (int i = 0; i < na.length; i++) {
					Weapon weapon = Technologies.getWeapon(na[i]);
					if ( Comp.same(weapon.getGroup(), groupName) ) {
						return true;
					}
				}
			}
		}
		return false;
	}
	
    public boolean same(WeaponBayDesign other) {
		if (other == null) return false;
        if (weapons == null && other.getWeapons() == null) return true;
        if (weapons == null || other.getWeapons() == null) return false;
		return weapons.same(other.getWeapons());
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		if (weapons != null) {
			String[] na = weapons.getNamesAsArray();
			if (na != null) {
				for (int i = 0; i < na.length; i++) {
					writer.addElement("weapon", na[i]);
				}
			}
		}
	}

	protected String getElementName() { return "weapon-bay"; }

    public String toString() {
        if (weapons == null) {
        	return "[]";
        } else {
        	return "["+weapons.toString()+"]";
        }
    }

}