/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.designs;

import java.io.Serializable;
import java.util.ArrayList;

import mot.common.bean.NameList;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBundle;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.vo.WeaponBayVO;
import mot.common.xml.DomNode;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This is a list of Weapons bays.
 */
public class WeaponBaysDesign extends XMLBean implements Serializable {

	/**
	 * The bays that this craft contains.
	 */
	private ArrayList bays;
    
	private transient WeaponBayDesign [] baysArray;

	public WeaponBaysDesign() {
	}

	public WeaponBaysDesign(WeaponBayDesign wb) throws ApplicationError {
        if (wb != null) {
            setWeaponBay(wb);
        }
	}

	public WeaponBaysDesign(WeaponBaysDesign wb) throws ApplicationError {
        WeaponBayDesign [] list = wb.getBaysAsArray();
        if (list != null) {
            for (int i=0; i<list.length; i++) {
                setWeaponBay(new WeaponBayDesign(list[i]));
            }
        }
	}

   	public WeaponBaysDesign(WeaponBayVO[] bays) throws ApplicationError {
        if (bays != null) {
            for (int i=0; i<bays.length; i++) {
                setWeaponBay(new WeaponBayDesign(bays[i]));
            }
        }
	}

	public WeaponBaysDesign(DomNode n) throws ApplicationError {
		DomNode [] na = n.getNodesByName("weapon-bay");
        if (na != null) {
        	for (int i = 0; i < na.length; i++) {
        		setWeaponBay( new WeaponBayDesign(na[i]));
			}
        }
	}

	public void setBays(ArrayList v) { bays = v; }

	public ArrayList getBays() { return bays; }
	public WeaponBayDesign [] getBaysAsArray() { 
        if (bays == null) {
            return null;
        } else {
            if (baysArray == null) {
                baysArray = (WeaponBayDesign[])bays.toArray(new WeaponBayDesign[(bays.size())]); 
            }
            return baysArray;
        }
    }
    
	public void add(WeaponBayDesign v) { 
        setWeaponBay(v);
    }
    
	public void setWeaponBay(WeaponBayDesign v) { 
        if (bays == null) bays = new ArrayList();
        baysArray = null;
        bays.add(v); 
    }
    
    public void addWeapon(Weapon w, int bayNo) {
        int qty = getQty();
        if (bayNo > qty) {
            Logger.logError("NO BAY "+bayNo+" - Weapon Ignored", new ApplicationError());
            return;
        } else if (bayNo == qty) {
            WeaponBayDesign wb = new WeaponBayDesign();
            wb.add(w);
            setWeaponBay(wb);
        } else {
            WeaponBayDesign [] wbs = getBaysAsArray();
            wbs[bayNo].add(w);
        }
    }
    
    public WeaponBayDesign getWeaponBay(int bayNo) {
        int qty = getQty();
        if (bayNo >= qty) {
            return null;
        } else {
            WeaponBayDesign [] wbs = getBaysAsArray();
            return wbs[bayNo];
        }
    }

    public void removeFirstWeapon(Weapon w) throws ApplicationError {
        removeFirstWeapon(w, 0);
    }
    
    public void removeFirstWeapon(Weapon w, int bayNo) throws ApplicationError {
        WeaponBayDesign [] wbs = getBaysAsArray();
        if (bayNo >= wbs.length) {
            throw new ApplicationError("WeaponsBays.addWeapon() Unable to add to bay number "+bayNo);
        } else {
            wbs[bayNo].removeFirst(w);
        }
    }

	public void removeAll(Weapon tech) {
		if (bays != null) {
            WeaponBayDesign [] bays = getBaysAsArray();
            for (int i=0; i<bays.length; i++) {
            	bays[i].removeAll(tech);
            }
		}
	}


    /**
     * @return The total quantity of weapons bays
     */
	public int getQty() {
		if (bays == null) {
			return 0;
		} else {
			return bays.size();
		}
	}
    
    /**
     * @return The total quantity of weapons in all weapons bays
     */
	public int getQtyWeapons() {
		if (bays == null) {
			return 0;
		} else {
            int sum = 0;
            WeaponBayDesign [] bays = getBaysAsArray();
            for (int i=0; i<bays.length; i++) {
                sum += bays[i].getQtyWeapons();
            }
			return sum;
		}
	}
	
	public int getQuantityWeaponsWithGroup(String group) throws ApplicationError {
		if (bays == null) {
			return 0;
		} else {
            int sum = 0;
            WeaponBayDesign [] bays = getBaysAsArray();
            for (int i=0; i<bays.length; i++) {
                sum += bays[i].getQuantityWeaponsWithGroup(group);
            }
			return sum;
		}
	}
    
	public int getQuantity(String name) throws ApplicationError {
		if (bays == null) {
			return 0;
		} else {
            int sum = 0;
            WeaponBayDesign [] bays = getBaysAsArray();
            for (int i=0; i<bays.length; i++) {
                sum += bays[i].getQuantity(name);
            }
			return sum;
		}
	}
    
    /**
     * @return The total quantity of damage inflicted from all weapons in all weapons 
     * bays. So if there are three weapons one with damage 1 and two with damage 2 this
     * would return 3.
     * @throws ApplicationError 
     */
//	public int getDamageSum(boolean includeLimitedShots) throws ApplicationError {
//		if (bays == null) {
//			return 0;
//		} else {
//            int sum = 0;
//            WeaponBayDesign [] bays = getBaysAsArray();
//            for (int i=0; i<bays.length; i++) {
//                sum += bays[i].getDamageSum(includeLimitedShots);
//            }
//			return sum;
//		}
//	}
    
    /**
     * Returns the space consumed by all weapons in these bays
     * @throws ApplicationError 
     */
    public int getSpace() throws ApplicationError {
        Weapon l [] = getAllWeapons();
        if (l == null) return 0;
        int space = 0;
        for (int i=0; i<l.length; i++) {
            space += l[i].getSpace();
        }
        return space;
    }
    
    /**
     * Returns the maximum range of any weapon in this weapons list.
     */
//    public long getMaxRange() throws ApplicationError {
//        WeaponBayDesign [] bays = getBaysAsArray();
//        long max = 0;
//		for (int i=0; i<bays.length; i++) {
//			if (bays[i].getMaxRange() > max) {
//				max = bays[i].getMaxRange();
//			}
//		}
//		return max;
//    }

    public boolean contains(String name) throws ApplicationError {
        Weapon w = Technologies.getWeapon(name);
        return contains(w);
    }

    public boolean contains(Technology t) throws ApplicationError {
    	if (t instanceof Weapon) {
    		return contains((Weapon)t);
    	}
    	return false;
    }
    
    public boolean contains(Weapon w) throws ApplicationError {
        WeaponBayDesign [] bays = getBaysAsArray();
        
        if (bays == null) return false;
        
        for (int i=0; i<bays.length; i++) {
            if (bays[i].contains(w)) return true;
        }
        
        return false;
    }

	public boolean containsGroup(String groupName) throws ApplicationError {
        WeaponBayDesign [] bays = getBaysAsArray();
        
        if (bays == null) return false;
        
        for (int i=0; i<bays.length; i++) {
            if (bays[i].containsGroup(groupName)) return true;
        }
        
        return false;
	}

    /**
     * @return The total cost to build all items in this list. 
     * @throws ApplicationError 
     */
   	public int getCostToBuild(double costToBuildHullFactor) throws ApplicationError {
        WeaponBayDesign [] bays = getBaysAsArray();
        int total = 0;
		for (int i=0; i<bays.length; i++) {
			total = bays[i].getWeaponsList().getCostToBuild(costToBuildHullFactor);
		}
        return total;
    }

    /**
     * @return the weapons lists for all bays. Weapons list at index 0 is from bay 0,
     * Weapons list at index 1 is from bay 1, and so on.
     * @throws ApplicationError 
     */
//   	public Weapons[] getWeapons() throws ApplicationError {
//        WeaponBayDesign [] bays = getBaysAsArray();
//        if (bays == null || bays.length == 0) return null;
//        ArrayList<Weapons> r = null;
//		for (int i=0; i<bays.length; i++) {
//			if (bays[i] != null && bays[i].getQtyWeapons() > 0) {
//				if (r == null) r = new ArrayList<Weapons>();
//				r.add(bays[i].getWeaponsList());
//			}
//		}
//		if (r == null) {
//			return null;
//		} else {
//			return r.toArray(new Weapons[r.size()]);
//		}
//    }

    /**
     * @return an array of all weapons in all bays.
     * @throws ApplicationError 
     */
   	public Weapon[] getAllWeapons() throws ApplicationError {
        WeaponBayDesign [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        ArrayList r = null;
		for (int i=0; i<bays.length; i++) {
			NameList weapons = bays[i].getWeapons();
            if (weapons != null) {
                String[] all = weapons.getNamesAsArray();
                for (int j=0; all != null && j<all.length; j++) {
                	if (r == null) r = new ArrayList();
                    r.add( Technologies.getWeapon(all[j]) );
                }
            }
		}
		if (r == null) {
			return null;
		} else {
			return (Weapon[])r.toArray(new Weapon[(r.size())]);
		}
    }

    /**
     * @return an array of all weapons of type v in all bays.
     * @throws ApplicationError 
     */
   	public Weapon[] getAllWeapons(Weapon v) throws ApplicationError {
        WeaponBayDesign [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        ArrayList r = null;
		for (int i=0; i<bays.length; i++) {
			NameList weapons = bays[i].getWeapons();
            if (weapons != null) {
                String[] all = weapons.getNamesAsArray();
                for (int j=0; all != null && j<all.length; j++) {
                	Weapon weapon = Technologies.getWeapon(all[j]);
                    if (weapon == v) {
                    	if (r == null) r = new ArrayList();
                        r.add(weapon);
                    }
                }
            }
		}
		if (r == null) {
			return null;
		} else {
			return (Weapon[])r.toArray(new Weapon[(r.size())]);
		}
    }

    /**
     * @return an array of all weapons in all bays.
     * @throws ApplicationError 
     */
   	public Weapons getAllWeaponsAsList() throws ApplicationError {
        WeaponBayDesign [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        Weapons r = null;
		for (int i=0; i<bays.length; i++) {
			NameList weapons = bays[i].getWeapons();
            if (weapons != null) {
                String[] all = weapons.getNamesAsArray();
                for (int j=0; all != null && j<all.length; j++) {
                	if (r == null) r = new Weapons();
                	Weapon weapon = Technologies.getWeapon(all[j]);
                    r.add(weapon);
                }
            }
		}
        return r;
    }
    
   	public Weapons getAllWeaponsNoDuplicates() throws ApplicationError {
        WeaponBayDesign [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        Weapons r = null;
		for (int i=0; i<bays.length; i++) {
			NameList weapons = bays[i].getWeapons();
            if (weapons != null) {
                String[] all = weapons.getNamesAsArray();
                for (int j=0; all != null && j<all.length; j++) {
                	Weapon weapon = Technologies.getWeapon(all[j]);
                	if (r == null || !r.contains(weapon)) {
	                	if (r == null) r = new Weapons();
	                    r.add(weapon);
                	}
                }
            }
		}
        return r;
    }
    
    /**
     * The return value contains a single weapon bundle for each type of weapon. The weapon
     * bundle contains the quantity of this weapon type.
     * @return A list of all weapons in this list
     * @throws ApplicationError 
     */
    public WeaponBundle [] getDistinctWeapons() throws ApplicationError {
        Weapons weapons = getAllWeaponsAsList();
        if (weapons == null) {
            return null;
        }
        return weapons.getDistinctWeapons();
    }

    public TechnologyBundle [] getDistinct(TechnologyList exclude) throws ApplicationError {
        Weapons weapons = getAllWeaponsAsList();
        if (weapons == null) {
            return null;
        }
        return weapons.getDistinct(exclude);
    }


    public String getWeaponNames() {
        return getWeaponNames(" ,");
    }

    public String getWeaponNames(String seperator) {
        WeaponBayDesign [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        StringBuffer sb = new StringBuffer();
        for (int i=0; i<bays.length; i++) {
            sb.append("Weapons Bay "+i+" (");
            NameList weapons = bays[i].getWeapons();
            if (weapons != null) {
            	sb.append(weapons.getNamesSeparated(seperator));
            }
            sb.append(") ");
        }
        return sb.toString();
    }

    /**
     * @return True iff other has the same quantity and type of weapons as this.
     */
	public boolean same(WeaponBaysDesign other) {
		if (other == null) return false;
        WeaponBayDesign [] bays = getBaysAsArray();
        WeaponBayDesign [] obays = other.getBaysAsArray();
        if (bays == null && obays == null) return true;
        if (bays == null || obays == null) return false;
        if (bays.length != obays.length) return false;
		for (int i=0; i<bays.length; i++) {
			if (!bays[i].same(obays[i])) {
				return false;
			}
		}
		return true;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
        WeaponBayDesign [] bays = getBaysAsArray();
		if (bays != null) {
			for (int i=0; i<bays.length; i++) {
				bays[i].toXML(writer);
			}
		}
	}

	protected String getElementName() { return "weapon-bays"; }

    public WeaponBaysDesign cloneTechnology() throws ApplicationError {
        try {
            return (WeaponBaysDesign)this.clone();
        } catch (CloneNotSupportedException e) {
            throw new ApplicationError("WeaponsBaysDesign.cloneTechnology() Failed due to error "+e);
        }
    }
    
    public String toString() {
        WeaponBayDesign [] list = getBaysAsArray();
        if (list == null) {
        	return "[]";
        } else {
        	return "["+Comp.toStringOnAll(list, ",")+"]";
        }
    }

}

