/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.event;

import mot.common.bean.CraftState;
import mot.common.bean.ObjectMovement;
import mot.common.vo.ValueObject;

/**
 * This event is sent to all players when a craft has moved on the battle map. This gives UIs a
 * chance to animate craft movement.
 *
 * Note percentTurn + percentMove does not have to be equal to one because not all of the
 * crafts turn may have been taken.
 */
public class BattleCraftMoveEvent extends LinkedEventBase implements BattleStateUpdateSingleEventAdapter {

	/**
	 * The name of the battle this event takes part in.
	 */
	private String battle;
	
	/**
	 * The name of the craft that has moved.
	 */
	private String name;
	
	/**
	 * The final state of the craft after the move
	 */
	private CraftState craftState;

	/**
	 * The components of the move
	 */
    private ObjectMovement [] movements;

    public BattleCraftMoveEvent(String battle, String name, CraftState craftState, ObjectMovement [] movements) {
        this.battle = battle;
		this.name = name;
		this.craftState = craftState;
        this.movements = movements;
    }

    public void setCraftState(CraftState craftState) {
        this.craftState = craftState;
    }

    public CraftState getCraftState() {
        return craftState;
    }

	public ObjectMovement[] getMovements() {
		return movements;
	}

	public void setMovements(ObjectMovement[] movements) {
		this.movements = movements;
	}
	
	public String getMovementsAsString() {
        if (movements == null) return "None";
        StringBuilder sb = new StringBuilder();
        boolean first = true;
        for (ObjectMovement i : movements) {
            if (i != null) {
            	if (first) {
            		first = false;
            	} else {
            		sb.append(" + ");
            	}
           		sb.append(i.toString());
            }
        }
        if (sb == null || sb.length() == 0) {
        	return "None";
        } else {
        	return sb.toString();
        }
	}
	
	public String toString() {
		return "CraftMoveEvent: craft="+name+" movements="+getMovementsAsString()+super.toString();
	}

	public boolean isGalaxyEvent() {
		return false;
	}

	public boolean isBattleEvent() {
		return true;
	}
	
	@Override
	public String getUpdatedObjectName() {
		return name;
	}

	@Override
	public CraftState fetchStateUpdate() {
		return craftState;
	}

	@Override
	public ValueObject fetchValueObjectUpdate() {
		return null;
	}

	public String getBattle() {
		return battle;
	}

	public void setBattle(String battle) {
		this.battle = battle;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}	
}