/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.event;

import mot.common.bean.CraftState;
import mot.common.bean.Damage;
import mot.common.bean.LocatedObject;
import mot.common.bean.Location;
import mot.common.vo.ValueObject;

/**
 * This event is caused when a craft is damaged - and causes the damage animation to be 
 * destroyed. This event is not used when an object is damaged by weapon fire - A Weapon
 * FireEvent is created for damage inflicted by a weapon - This is used for things like
 * a warp core breach or any explosion.
 * 
 * NOTE - Using WeaponFireEvent reduces the total quantity of events sent to the client.
 */
public class BattleDamageEvent extends LinkedEventBase implements BattleStateUpdateSingleEventAdapter {

	/**
	 * The name of the battle this event takes part in.
	 */
	private String battle;
	
    /**
     * The craft that has been damaged.
     */
    private String target;
    
    /**
     * The details of the damage
     */
    private Damage damage;
    
	public BattleDamageEvent(String battle, String target, Damage damage) {
        this.battle = battle;
		this.target = target;
        this.damage = damage;
	}

	public String toString() {
		return "DamageEvent: target="+target+" dam="+damage+super.toString();
	}
	
    public String getTargetDescription() {
        return damage.getTargetDescription();
    }

    public double getWeaponDamage() {
        return damage.getDamage();
    }

    public double getShieldBlockedDamage() {
        return damage.getShieldBlockedDamage();
    }

    public String getTarget() {
        return target;
    }

    public void setTarget(String target) {
        this.target = target;
    }

    public boolean getDestroyed() {
        return damage.getDestroyed();
    }


    public CraftState getOldTargetState() {
        return damage.getOldTargetState();
    }


    public CraftState getNewTargetState() {
        return damage.getNewTargetState();
    }

    public CraftState getCraftState() {
        if (damage.getOldTargetState() != null) {
        	return damage.getOldTargetState();
        } else {
        	return damage.getNewTargetState();
        }
    }

    public String getSourceDescription() {
        return null;
    }

	public Location getTargetLocation(LocatedObject defaultLocationProvider) {
		if (damage.getTargetLocation() != null) {
			return damage.getTargetLocation();
		} else {
			if (damage.getOldTargetState() != null) {
				return damage.getOldTargetState().getLocation();
			} else if (damage.getNewTargetState() != null) {
				return damage.getNewTargetState().getLocation();
			} else if (defaultLocationProvider != null) {
				return defaultLocationProvider.getLocation();
			} else {
				return null;
			}
		}
	}

	public boolean isGalaxyEvent() {
		return false;
	}

	public boolean isBattleEvent() {
		return true;
	}

	public String getBattle() {
		return battle;
	}

	public void setBattle(String battle) {
		this.battle = battle;
	}

	@Override
	public String getUpdatedObjectName() {
		return target;
	}

	@Override
	public CraftState fetchStateUpdate() {
		return damage.getNewTargetState();
	}

	@Override
	public ValueObject fetchValueObjectUpdate() {
		return null;
	}

	public Damage getDamage() {
		return damage;
	}

	public void setDamage(Damage damage) {
		this.damage = damage;
	}

}