/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.event;

import mot.common.bean.CraftState;
import mot.common.bean.LocatedObject;
import mot.common.bean.Location;
import mot.common.vo.ValueObject;

/**
 * Caused when a craft is destroyed, either in a battle or on the main map
 * due to some other event.
 */
public class BattleDestroyedEvent extends LinkedEventBase implements BattleStateUpdateSingleEventAdapter {

	/**
	 * The name of the battle this event takes part in.
	 */
	private String battle;
	
    /**
     * This is the state before the craft was destroyed.
     * Note if this event is being sent in conjunction with a WeaponFireEvent then this may be null
     * because any animation based on this will not be required to maintain old state.
     */
	public CraftState oldState;
    
    /**
     * This is the craft that has been destroyed.
     */
	public String target;
    
    /**
     * True if this craft was a hologram.
     */
    private boolean hologram;

	public BattleDestroyedEvent(String battle, String target, CraftState oldState, boolean isHologram) {
		this.battle = battle;
		this.target = target;
        this.oldState = oldState;
        this.hologram = isHologram;
	}


	public Location getLocation(LocatedObject defaultLocationProvider) {
		if (oldState != null) {
			return oldState.getLocation();
		} else if (defaultLocationProvider != null) {
			return defaultLocationProvider.getLocation();
		} else {			
			return null;
		}
	}

	public String toString() {
		return "BattleDestroyedEvent: target="+target+" "+super.toString();
	}
	
    public String getTarget() {
        return target;
    }

    public void setTarget(String target) {
        this.target = target;
    }


    public void setHologram(boolean hologram) {
        this.hologram = hologram;
    }


    public boolean isHologram() {
        return hologram;
    }


    public void setOldState(CraftState oldState) {
        this.oldState = oldState;
    }


    public CraftState getOldState() {
        return oldState;
    }

	public boolean isGalaxyEvent() {
		return false;
	}

	public boolean isBattleEvent() {
		return true;
	}


	public String getBattle() {
		return battle;
	}


	public void setBattle(String battle) {
		this.battle = battle;
	}


	@Override
	public CraftState fetchStateUpdate() {
		return null;
	}

	@Override
	public ValueObject fetchValueObjectUpdate() {
		return null;
	}

	@Override
	public String getUpdatedObjectName() {
		return target;
	}


}