/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.event;

import mot.common.bean.CraftState;
import mot.common.bean.Damage;
import mot.common.bean.LocatedObject;
import mot.common.bean.Location;
import mot.common.bean.NameCraftStatePair;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.ValueObject;

/**
 * Caused when a craft fires a weapon in a battle.
 * The battle craft source has fired on battle craft target using weapon weapon
 * and has inflicted damage on the hull of target. The shields on target stopped
 * shieldBlockedDamage extra damage from being inflicted on it.
 */
public class BattleWeaponFireEvent extends BattleDamageEvent implements BattleStateUpdateMultiEventAdapter {

    /**
     * The craft that has fired the weapon.
     * DO NOT USE THIS - ALWAYS USE THE STATE OBJECTS BECAUSE THE VALUES IN THIS OFTEN RELATE TO A DIFFERENT POINT IN TIME.
     */
    private String source;

    /**
     * The weapon that has been fired.
     */
    private String weapon;
    
    /**
     * The source craft state before this animations is taking place.
     */
    private CraftState oldSourceState;

    /**
     * The source craft state while this animations is taking place.
     */
    private CraftState newSourceState;

    /**
     * True if the animation should be the reverse of its normal direction.
     * Normally animation flow from the source to the target, but animations like ShieldDrain can
     * flow both ways.
     */
    private boolean reverseAnimation = false;
    
    /**
     * Some weapon fire animations have special messages for the user.
     */
    private String sourceDescription;

    /**
     * Weapons that can use different animation images for different times the weapon is fired you can specify
     * a suffix to the name in the configuration for this weapon file animation. e.g. ReplicaBeam. 
     */
    private String animationConfigNameSuffix;
    
    /**
     * This is extra information pertaining the this weapon fire event. It is untypes and its
     * contents are entirely dependent on the type of weapon that generated this event.
     * Example Uses :
     * 	1 - When a spatial distortion weapon fires it passed details of the spatial distortion 
     *  	is has created
     *  2 - When salvage beam removes wreckage, the wreckage still needs animating until this event 
     * 		is animated.
     */
    private Object data;

    public BattleWeaponFireEvent(String battle, String source, String target, String weapon, double damage, double shieldBlockedDamage, boolean destroyed, Location targetLocation, boolean reverseAnimation, CraftState oldSourceState, CraftState oldTargetState, CraftState newSourceState, CraftState newTargetState, String sourceDescription, String targetDescription, String animationConfigNameSuffix, Object data) {
        super(battle, target, new Damage(targetDescription, damage, shieldBlockedDamage, destroyed, targetLocation, oldTargetState, newTargetState));
		this.source = source;
        this.weapon = weapon;
        this.reverseAnimation = reverseAnimation;
        this.oldSourceState = oldSourceState;
        this.newSourceState = newSourceState;
        this.sourceDescription = sourceDescription;
        this.animationConfigNameSuffix = animationConfigNameSuffix;
        this.data = data;
	}
    
	public String toString() {
		return "WeaponFireEvent source="+source+" target="+getTarget()+" Src Dam="+getSourceDescription()+" Target Dam="+getTargetDescription()+" [weapon="+weapon+":"+getSourceLocation(null)+"->"+getTargetLocation(null)+"]";
	}

	public Location getSourceLocation(LocatedObject defaultLocationProvider) {
		if (newSourceState != null) {
			return newSourceState.getLocation();
		} else if (oldSourceState != null) {
			return oldSourceState.getLocation();
		} else {
			Logger.logWarning("WeaponFireEvent() weapon="+weapon+" No source state");
			if (defaultLocationProvider != null) {
				return defaultLocationProvider.getLocation();
			} else {
				return null;
			}
		}
	}
	
	public int getDirection() throws ApplicationError {
		if (newSourceState != null) {
			return newSourceState.getDirection();
		} else if (oldSourceState != null) {
			return oldSourceState.getDirection();
		} else {
			Logger.logWarning("WeaponFireEvent() weapon="+weapon+" No source state");
			return 0;
		}
	}
	
    public CraftState getCraftState() {
        if (getDamage().getOldTargetState() != null) {
        	return getDamage().getOldTargetState();
        } else {
        	return getDamage().getNewTargetState();
        }
    }

	public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public Weapon getWeaponTechnology() throws ApplicationError {
        return Technologies.getWeapon(weapon);
    }

    public String getWeapon() {
        return weapon;
    }

    public void setWeapon(String weapon) {
        this.weapon = weapon;
    }


    public void setReverseAnimation(boolean reverseAnimation) {
        this.reverseAnimation = reverseAnimation;
    }


    public boolean isReverseAnimation() {
        return reverseAnimation;
    }


    public void setOldSourceState(CraftState oldSourceState) {
        this.oldSourceState = oldSourceState;
    }


    public CraftState getOldSourceState() {
        return oldSourceState;
    }


    public void setNewSourceState(CraftState newSourceState) {
        this.newSourceState = newSourceState;
    }


    public CraftState getNewSourceState() {
        return newSourceState;
    }


    public void setSourceDescription(String sourceDescription) {
        this.sourceDescription = sourceDescription;
    }


    public String getSourceDescription() {
        return sourceDescription;
    }

	public Object getData() {
		return data;
	}

	public void setData(Object data) {
		this.data = data;
	}


	public void setAnimationConfigNameSuffix(String animationConfigNameSuffix) {
		this.animationConfigNameSuffix = animationConfigNameSuffix;
	}

	public String getAnimationConfigNameSuffix() {
		return animationConfigNameSuffix;
	}

	@Override
	public NameCraftStatePair [] fetchStateUpdates() {
		NameCraftStatePair [] r = new NameCraftStatePair[2];
		r[0] = new NameCraftStatePair(source, newSourceState);
		r[1] = new NameCraftStatePair(getTarget(), getDamage().getNewTargetState());
		return r;
	}

	@Override
	public ValueObject [] fetchValueObjectUpdates() {
		if (data != null && data instanceof ValueObject) {
			ValueObject [] r = new ValueObject[1];
			r[0] = (ValueObject)data;
			return r;
		} else {
			return null;
		}
	}

	public boolean isGalaxyEvent() {
		return false;
	}

	public boolean isBattleEvent() {
		return true;
	}
}