/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.event;

import java.util.ArrayList;

import mot.common.bean.SectorOwnerGrid;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.NamedValueObject;
import mot.common.vo.ValueObject;

/**
 * This is an event that contains game objects such as crafts, planets, players that require a state update.
 * These events contain zero or more ValueObjects that can be used to update the client state.
 */
public abstract class StateUpdateContainerEvent implements MotEvent, StateContainer, ValueObject {

	private ArrayList<ValueObject> state;
	
	public void addStateUpdate(ValueObject v) throws ApplicationError {
		if (state == null) state = new ArrayList<ValueObject>();
		state.add(v);
	}

	public void addStateUpdate(ValueObject [] vol) throws ApplicationError {
		if (vol == null) return;
		for (int i = 0; i < vol.length; i++) {
			addStateUpdate(vol[i]);
		}		
	}
	
	public ValueObject [] getStateUpdates() {
		if (state == null) return null;
		return state.toArray(new ValueObject[state.size()]);
	}

	public ValueObject getState(String name) {
		if (state == null) return null;
		for (ValueObject vo : state) {
			if (vo instanceof NamedValueObject && Comp.same(((NamedValueObject)vo).getName(), name)) {
				return (NamedValueObject) vo;
			}
		}
		return null;
	}

	public ValueObject getType(Class clazz) {
		if (state == null) return null;
		for (ValueObject vo : state) {
			if (clazz.isAssignableFrom(vo.getClass())) {
				return vo;
			}
//			if (vo.getClass() == clazz) {
//				return vo;
//			}
		}
		return null;
	}
	
	public ArrayList<ValueObject> getState() {
		return state;
	}

	public void setState(ArrayList<ValueObject> state) {
		this.state = state;
	}

	public String getStateString() {
		String r = " Empty";
    	if (state != null) {
    		r = "";
    		for (ValueObject v : state) {
    			if (v instanceof SectorOwnerGrid) {
    				r = r + " SectorOwnerGrid";
    			} else {
    				String s = v.toString();
    				if (s != null) r = r + " "+ v;
    			}
    		}
    	}
		return "[" + r + "]";
	}
	
	@Override
	public String toString() {
		String name = getClass().getSimpleName();
		return name+" " + getStateString();
	}
}