/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.leaders;

import mot.common.util.ApplicationError;

/**
 * A General is a leader that provides benefits to a player's crafts. Example improvements
 * may be increase warp speed, increase manouverabuility, etc. All benefits are player wide
 * and effect all crafts owned by this player.
 * 
 * A General gains gains some experience when a battle is won but lots of experience 
 * when a battle is lost.
 */
public class GeneralLeader extends Leader {

    /**
     * These are the details describing this leaders abilities.
     */
    private transient GeneralDetails details;

    public GeneralLeader() {
    }

    public GeneralLeader(GeneralDetails details) {
        super(details.getName());
        this.details = details;
    }
    
    public GeneralLeader(GeneralDetails details, int experience) {
        super(details.getName());
        this.details = details;
        setExperience(experience);
    }
    
    public GeneralDetails getDetails() throws ApplicationError {
        if (details == null) {
            details = GeneralDetailsList.getInstance().getGeneralDetails(getName());
        }
        if (details == null) {
            throw new ApplicationError("General.getDetails() Unable to find General "+getName());
        }
        return details;
    }
    
    protected String getElementName() { return "general-leader"; }




    public boolean hasBattleTurningBonus() throws ApplicationError {
        return getDetails().getBattleTurningBonus() != null;
    }

    public boolean hasBattleSpeedBonus() throws ApplicationError {
        return getDetails().getBattleSpeedBonus() != null;
    }

    public boolean hasMaxDamageBonus() throws ApplicationError {
        return getDetails().getMaxDamageBonus() != null;
    }

    public boolean hasSpeedBonus() throws ApplicationError {
        return getDetails().getSpeedBonus() != null;
    }

    public boolean hasCloakLevel() throws ApplicationError {
        return getDetails().getCloakLevel() != null;
    }

    public boolean hasInitiativeBonus() throws ApplicationError {
        return getDetails().getInitiativeBonus() != null;
    }

    public boolean hasAssaultBonus() throws ApplicationError {
        return getDetails().getAssaultBonus() != null;
    }

    public boolean hasDamageBonus() throws ApplicationError {
        return getDetails().getDamageBonus() != null;
    }

    public boolean hasShieldPowerBonus() throws ApplicationError {
        return getDetails().getShieldPowerBonus() != null;
    }




    public double getBattleTurningBonus() throws ApplicationError {
        return applyLevel(getDetails().getBattleTurningBonus());
    }

    public double getBattleSpeedBonus() throws ApplicationError {
        return applyLevel(getDetails().getBattleSpeedBonus());
    }

    public double getMaxDamageBonus() throws ApplicationError {
        return applyLevel(getDetails().getMaxDamageBonus());
    }

    public double getSpeedBonus() throws ApplicationError {
        return applyLevel(getDetails().getSpeedBonus());
    }

    public int getCloakLevel() throws ApplicationError {
        return applyLevel(getDetails().getCloakLevel());
    }

    public double getInitiativeBonus() throws ApplicationError {
        return applyLevel(getDetails().getInitiativeBonus());
    }

    public double getAssaultBonus() throws ApplicationError {
        return applyLevel(getDetails().getAssaultBonus());
    }

    public double getDamageBonus() throws ApplicationError {
        return applyLevel(getDetails().getDamageBonus());
    }

    public double getShieldPowerBonus() throws ApplicationError {
        return applyLevel(getDetails().getShieldPowerBonus());
    }
}