/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.leaders;

import mot.common.util.ApplicationError;

/**
 * A Govenor is a leader that provides benefits to a player's planets. Example improvements
 * may be increase production, increase maximum population, etc. All benefits are player wide
 * and effect all planets owned by this player.
 * 
 * A Govenor gains some experience each turn but gains a more experience from colonising a planet
 * and when a planet is blockaded or lost.
 */
public class GovenorLeader extends Leader {

    /**
     * These are the details describing this leaders abilities.
     */
    private transient GovenorDetails details;

    public GovenorLeader() {
    }

    public GovenorLeader(GovenorDetails details) {
        super(details.getName());
        this.details = details;
    }
    
    public GovenorLeader(GovenorDetails details, int experience) {
        super(details.getName());
        this.details = details;
        setExperience(experience);
    }

    public GovenorDetails getDetails() throws ApplicationError {
        if (details == null) {
            details = GovenorDetailsList.getInstance().getGovenorDetails(getName());
        }
        if (details == null) {
            throw new ApplicationError("Govenor.getDetails() Unable to find Govenor "+getName());
        }
        return details;
    }

    protected String getElementName() { return "govenor-leader"; }






    public boolean hasProductionBonus() throws ApplicationError {
        return getDetails().getProductionBonus() != null;
    }

    public boolean hasResearchBonus() throws ApplicationError {
        return getDetails().getResearchBonus() != null;
    }

    public boolean hasDevelopmentBonus() throws ApplicationError {
        return getDetails().getDevelopmentBonus() != null;
    }

    public boolean hasReproductionBonus() throws ApplicationError {
        return getDetails().getReproductionBonus() != null;
    }

    public boolean hasFactoryPerPopulationBonus() throws ApplicationError {
        return getDetails().getFactoryPerPopulationBonus() != null;
    }

    public boolean hasEcologyBonus() throws ApplicationError {
        return getDetails().getEcologyBonus() != null;
    }

    public boolean hasMaxPopulationBonus() throws ApplicationError {
        return getDetails().getMaxPopulationBonus() != null;
    }

    public boolean hasPsiRating() throws ApplicationError {
        return getDetails().getPsiRating() != null;
    }

    public boolean hasTradingBonus() throws ApplicationError {
        return getDetails().getTradingBonus() != null;
    }

    public boolean hasSpyBonus() throws ApplicationError {
        return getDetails().getSpyBonus() != null;
    }

    public boolean hasMaxTradeRoutes() throws ApplicationError {
        return getDetails().getMaxTradeRoutes() != null;
    }




    public double getProductionBonus() throws ApplicationError {
        return applyLevel(getDetails().getProductionBonus());
    }

    public double getResearchBonus() throws ApplicationError {
        return applyLevel(getDetails().getResearchBonus());
    }

    public double getDevelopmentBonus() throws ApplicationError {
        return applyLevel(getDetails().getDevelopmentBonus());
    }

    public double getReproductionBonus() throws ApplicationError {
        return applyLevel(getDetails().getReproductionBonus());
    }

    public double getFactoryPerPopulationBonus() throws ApplicationError {
        return applyLevel(getDetails().getFactoryPerPopulationBonus());
    }

    public double getEcologyBonus() throws ApplicationError {
        return applyLevel(getDetails().getEcologyBonus());
    }

    public double getMaxPopulationBonus() throws ApplicationError {
        return applyLevel(getDetails().getMaxPopulationBonus());
    }

    public int getPsiRating() throws ApplicationError {
        return applyLevel(getDetails().getPsiRating());
    }

    public double getTradingBonus() throws ApplicationError {
        return applyLevel(getDetails().getTradingBonus());
    }

    public double getSpyBonus() throws ApplicationError {
        return applyLevel(getDetails().getSpyBonus());
    }

    public int getMaxTradeRoutes() throws ApplicationError {
        return applyLevel(getDetails().getMaxTradeRoutes());
    }

}