/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.leaders;

import mot.common.bean.NamedObject;
import mot.common.util.ApplicationError;
import mot.common.xml.XMLWriter;

/**
 * A leader is a game object (a representation of an individual) that provides benefits
 * to its owner. The game has a library of leaders and each player may have a set quantity
 * of leaders (usually one) in each position. Each leader can only belong to one player and 
 * when assigned to a player they provide player bonuses.
 * 
 * Leaders gain experience through the game and the more experience a leader has the
 * more powerful he becomes.
 * 
 * The level of a leader is calculated from the experience.
 */
public abstract class Leader extends NamedObject {

    /**
     * This is the quantity of experience this leader has achieved.
     * Leader gain experience through the game and the more experience a leader has the
     * more powerful he becomes.
     * All leaders start of with 0 experience and then gain experience over time.
     */
    private int experience;

    public Leader() {
    }

    public Leader(String name) {
        super(name);
        this.experience = 0;
    }
    
    private static final int K = 1000;
    private static final int LEVEL2_EXPERIENCE = 100;
    private static final int LEVEL3_EXPERIENCE = 500;
    private static final int LEVEL4_EXPERIENCE = 2*K;
    private static final int LEVEL5_EXPERIENCE = 10*K;
    private static final int LEVEL6_EXPERIENCE = 50*K;
    private static final int LEVEL7_EXPERIENCE = 100*K;
    private static final int LEVEL8_EXPERIENCE = 500*K;
    private static final int LEVEL9_EXPERIENCE = 1000*K;
    private static final int LEVEL10_EXPERIENCE = 10000*K;
    
    /**
     * @return The level this leader has attained. This is calculated from the leaders 
     * experience.
     */
    public int getLevel() {
        if (experience < LEVEL2_EXPERIENCE) {
            return 1;
        } else if (experience < LEVEL3_EXPERIENCE) {
            return 2;
        } else if (experience < LEVEL4_EXPERIENCE) {
            return 3;
        } else if (experience < LEVEL5_EXPERIENCE) {
            return 4;
        } else if (experience < LEVEL6_EXPERIENCE) {
            return 5;
        } else if (experience < LEVEL7_EXPERIENCE) {
            return 6;
        } else if (experience < LEVEL8_EXPERIENCE) {
            return 7;
        } else if (experience < LEVEL9_EXPERIENCE) {
            return 8;
        } else {
            return 9;
        }
    }
    
    protected double applyLevel(Double bonus) {
        if (bonus == null) {
            return 1.0;
        } else {
            return bonus.doubleValue()*getLevel();
        }
    }

    protected int applyLevel(Integer bonus) {
        if (bonus == null) {
            return 1;
        } else {
            return bonus.intValue()*getLevel();
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        super.encode(writer);
		writer.addElement("experience", experience);
	}


    public void setExperience(int experience) {
        this.experience = experience;
    }


    public int getExperience() {
        return experience;
    }
    
    public void addExperience(int qty) {
        experience += qty;
    }
}