/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.morals;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

public class MoralChoiceList extends XMLBean {

	private static final String ELEMENT_NAME = "moral-choice-list";
	
	private static final String ROOT_ELEMENT_NAME = "moral-choices";

	private transient Double productionBonus = null;
	private transient Double influenceBonus = null;
	private transient Double damageBonus = null;
	private transient Integer relationsBonus = null;

	private ArrayList list = null;
	
	private transient MoralChoice [] array = null;
    
    public MoralChoiceList() {
    }

    /**
     * Constructs a new moral choice list containing a single moral choice t.
     * @param mc The moral choice to add to this list
     */
    public MoralChoiceList(MoralChoice mc) {
        if (mc != null) {
            add(mc);
        }
    }
    
    /**
     * Performs a shallow clone on this moral choice list.
     * @param mcl The list to topy - This new list will contain all the techs in tl.
     * @throws mot.common.util.ApplicationError
     */
    public MoralChoiceList(MoralChoiceList mcl) {
        if (mcl == null) return;
        MoralChoice [] mc = mcl.getAllAsArray();
        if (mc != null) {
            for (int i=0; i<mc.length; i++) {
                add(mc[i]);
            }
        }
    }
    
    /**
     * Performs a shallow clone on this moral choice list.
     * @param tl The list to topy - This new list will contain all the techs in tl.
     * @throws mot.common.util.ApplicationError
     */
    public MoralChoiceList(MoralChoice [] mcl) throws ApplicationError {
        if (mcl != null) {
            for (int i=0; i<mcl.length; i++) {
                add(mcl[i]);
            }
        }
    }
    
    /**
     * Performs a shallow clone on this moral choice list.
     * @param tl The list to topy - This new list will contain all the techs in tl.
     * @throws mot.common.util.ApplicationError
     */
    public MoralChoiceList(String [] mcl) throws ApplicationError {
        if (mcl != null) {
            for (int i=0; i<mcl.length; i++) {
            	MoralChoice mc = MoralChoices.getMoralChoice(mcl[i]);
                add(mc);
            }
        }
    }
    
    public MoralChoiceList(DomTree doc) {
    	DomNode root = doc.getNodeByName(ROOT_ELEMENT_NAME);
    	
    	DomNode[] mca = root.getNodesByName("moral-choice");
    	if (mca != null) {
    		for (int i = 0; i < mca.length; i++) {
				add(new MoralChoice(mca[i]));
			}
    	}
	}


	/**
     * Adds a new moral choice item to the list of items. Note any cached values such as the 
     * total build cost for all item in the list are reset to null so the next time it is 
     * requested it must be recalculated. See method changed().
     */
	public void add(MoralChoice v) {
        if (v == null) return;
		if (list == null) list = new ArrayList();
		list.add(v);
        changed();
	}

    /**
     * Adds all moral choice items in the list to this list of items. This uses the add(moral choice)
     * method to add each moral choice.
     */
	public void add(MoralChoiceList v) {
        if (v == null) return;
        MoralChoice [] mc = v.getAllAsArray();
        if (mc == null) return;
        for (int i=0; i<mc.length; i++) {
            add(mc[i]);
        }
	}

    /**
     * Adds all moral choice items in the list to this list of items. This uses the add(moral choice)
     * method to add each moral choice.
     */
	public void add(MoralChoice [] mcl) {
        if (mcl == null) return;
        for (int i=0; i<mcl.length; i++) {
            add(mcl[i]);
        }
	}

	public void setMoralChoice(MoralChoice mc) { add(mc); } 

    /**
     * Removes a moral choice item from the list of items. Note any cached values such as the 
     * total build cost for all item in the list are reset to null so the next time it is 
     * requested it must be recalculated. See method changed().
     */
	public void remove(MoralChoice v) {
		if (list == null) return;
		list.remove(v);
        changed();
	}

    /**
     * Removes all moral choice items in the list to this list of items. This uses the remove(moral choice)
     * method to remove each moral choice.
     */
	public void remove(MoralChoiceList v) {
        if (v == null) return;
        MoralChoice [] mcl = v.getAllAsArray();
        if (mcl == null) return;
        for (int i=0; i<mcl.length; i++) {
            remove(mcl[i]);
        }
	}
    
    /**
     * Remove all elements from this list.
     */
    public void removeAll() {
        list = null;
        changed();
    }

    /**
     * @return moral choices in the list at index i
     */
	public MoralChoice get(int i) {
        if (list != null && i < list.size()) {
        	return (MoralChoice) list.get(i);
        } else {
        	return null;
        }
	}

    /**
     * @return All moral choices in the list
     */
	public MoralChoice [] getAllAsArray() {
        if (list == null) return null;
        if (array == null) {
            array = (MoralChoice[])list.toArray(new MoralChoice[list.size()]);
        }
        return array;
	}

    /**
     * @return All moral choices in the list
     */
	public String [] getAllNames() {
        MoralChoice [] mca = getAllAsArray();
        if (mca != null) {
        	return Comp.toStrings(mca);
        } else {
        	return null;
        }
	}

    /**
     * Returns the first moral choice in the list whose name is the name passed in.
     * @param name The name of a type of moral choice to search for.
     * @return A moral choice item.
     */
	public MoralChoice get(String name) {
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				MoralChoice v = (MoralChoice)i.next();
				if ( Comp.same(v.getName(), name) ) {
					return v;
				}
			}
		}
		return null;
	}

    /**
     * @return The quantity of items in this list.
     */
    public int getQuantity() {
        if (list == null) {
            return 0;
        } else {
            return list.size();
        }
    }
    
	/**
	 * The bonus to planetary production used for the player having made these choices
	 * A value of 0.1 implies a 10% bonus
	 */
	public double getProductionBonus() {
		if (productionBonus == null) {
			MoralChoice [] mca = getAllAsArray();
			if (mca == null) {
				productionBonus = new Double(0.0);
			} else {
                double r = 0.0;
				for (int i=0; i<mca.length; i++) {
					if (mca[i].getProductionBonus() != null) {
						r += mca[i].getProductionBonus().doubleValue();
					}
				}
				productionBonus = new Double(r);
			}
		}
		return productionBonus.doubleValue();
	}

	/**
	 * The bonus to influence used for the player having made these choices
	 * A value of 0.1 implies a 10% bonus
	 */
	public double getInfluenceBonus() {
		if (influenceBonus == null) {
			MoralChoice [] mca = getAllAsArray();
			if (mca == null) {
				influenceBonus = new Double(0.0);
			} else {
                double r = 0.0;
				for (int i=0; i<mca.length; i++) {
					if (mca[i].getInfluenceBonus() != null) {
						r += mca[i].getInfluenceBonus().doubleValue();
					}
				}
				influenceBonus = new Double(r);
			}
		}
		return influenceBonus.doubleValue();
	}

	/**
	 * The bonus to weapon damage used for the player having made these choices
	 * A value of 0.1 implies a 10% bonus
	 */
	public double getDamageBonus() {
		if (damageBonus == null) {
			MoralChoice [] mca = getAllAsArray();
			if (mca == null) {
				damageBonus = new Double(0.0);
			} else {
                double r = 0.0;
				for (int i=0; i<mca.length; i++) {
					if (mca[i].getDamageBonus() != null) {
						r += mca[i].getDamageBonus().doubleValue();
					}
				}
				damageBonus = new Double(r);
			}
		}
		return damageBonus.doubleValue();
	}

	/**
	 * The bonus to diplomatic relations used for the player having made these choices
	 * A value of 10 implies a all relations are improved by 10
	 */
	public int getRelationsBonus() {
		if (relationsBonus == null) {
			MoralChoice [] mca = getAllAsArray();
			if (mca == null) {
				relationsBonus = new Integer(1);
			} else {
                int r = 0;
				for (int i=0; i<mca.length; i++) {
					if (mca[i].getRelationsBonus() != null) {
						r += mca[i].getRelationsBonus().doubleValue();
					}
				}
				relationsBonus = new Integer(r);
			}
		}
		return relationsBonus.intValue();
	}

	/**
	 * @return true if an only if ANY techology in the list tl is contained in this list.
	 */
	public boolean containsAny(MoralChoiceList mcl) {
        if (mcl == null) return false;
        MoralChoice [] mca = mcl.getAllAsArray();
		if (mca != null) {
		    for (int i = 0; i < mca.length; i++) {
				if ( contains(mca[i]) ) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @return true if an only if all techology in the list tl are contained in this list.
	 */
	public boolean contains(MoralChoiceList mcl) {
        if (mcl == null) return false;
        MoralChoice [] mca = mcl.getAllAsArray();
		if (mca != null) {
		    for (int i = 0; i < mca.length; i++) {
				if ( !contains(mca[i]) ) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * Returns true if an only if a techology with this name is contained in this list.
	 */
	public boolean contains(MoralChoice mc) {
		if (mc == null) return true;
        MoralChoice [] mca = getAllAsArray();
		if (mca != null) {
		    for (int i = 0; i < mca.length; i++) {
				if ( mca[i] == mc ) {
					return true;
				}
			}
		}
		return false;
	}

    private void changed() {
		productionBonus = null;
		influenceBonus = null;
		damageBonus = null;
		relationsBonus = null;
        array = null;
    }

    
    public String getNames() {
        return getNames("\n");
    }
    
    public String getNames(String seperator) {
        StringBuffer sb = new StringBuffer();
        MoralChoice [] mca = getAllAsArray();
		if (mca != null) {
	        boolean first = true;
		    for (int i = 0; i < mca.length; i++) {
	            if (first) {
	                first = false;
	            } else {
	                sb.append(seperator);
	            }
	            sb.append(mca[i].getName());
			}
		}
        return sb.toString();
    }

	protected String getElementName() {
		return ELEMENT_NAME;
	}
   
	protected void encode(XMLWriter writer) throws ApplicationError {
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				MoralChoice v = (MoralChoice)i.next();
				v.toXML(writer);
			}
		}
	}
    
    public void toXML(XMLWriter writer) throws ApplicationError {
		referenceToXML(writer);
	}

    
    
    // Rather than using moral choice reference everywhere or cloning a moral choice list,
    // you can serialise a reference to a list of technologies and this class has the 
    // smarts to de serialise into the original list.
    
	public void referenceToXML(XMLWriter writer) throws ApplicationError {
        referenceToXML(writer, getElementName());
    }
    
    public void referenceToXML(XMLWriter writer, String elementName) throws ApplicationError {
        writer.openElement(elementName, getAttributes(writer.generateClassAttribute(), writer.generateSchemaAttributes()), false);
        MoralChoice [] list = getAllAsArray();
        for (int i=0; list != null && i<list.length; i++) {
            writer.addElement("reference-name", list[i].getName());
        }
        writer.closeElement(elementName);
    }
    
    public void setReferenceName(String name) throws ApplicationError {
        add(MoralChoices.getMoralChoice(name));
    }
    
    public String toString() {
        return getNames(", ");
    }


}