/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.proxy;

import mot.common.event.MotEvent;
import mot.common.util.ApplicationError;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattlePlanet;
import mot.engine.event.BattleEndEF;
import mot.engine.event.FundsLowEF;
import mot.engine.event.MotEventFactory;
import mot.engine.event.PlanetInvadeEF;
import mot.engine.planets.Planet;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.relations.Relations;

/**
 * This is a proxy class for computer players. It uses a local copy (in process) of the
 * MotEngine to effect its commands.
 * All changes/commands issued by a player must go through a proxy so the client is
 * separated from the engine allowing distributed operation.
 */
public class ComputerPlayerProxy extends ClientProxyBase {

    public ComputerPlayerProxy(ComputerPlayer owner) {
        super(owner);
    }

    /**
     * The engine informs clients of changes (weapons firing, craft destruction etc)
     * by generating events.
     */
	public void notify(MotEvent e) throws ApplicationError {
        process(e);
    }

	public void notify(MotEventFactory e) throws ApplicationError {
        //if (Logger.isDebug()) {
        //    Logger.logDebug("ComputerPlayerProxy.process() MotEvent="+e.getClass().getName());
        //}
        ComputerPlayer owner = (ComputerPlayer) getOwner();
        MotEngine engine = MotEngine.instance;
        if (e instanceof FundsLowEF) {
	        FundsLowEF event = (FundsLowEF)e;
	        // Boy don't you hate it when players cheat!!!
	        owner.addFunds(owner.getTotalMaintenanceCost()*5);
//        } else if (e instanceof DiplomaticEventAllianceRequestEF) {
//            DiplomaticEventAllianceRequestEF event = (DiplomaticEventAllianceRequestEF)e;
//            owner.requestAliance(event.getPlayer(), event.getMessage(), null);
//        } else if (e instanceof DiplomaticEventPeaceRequestEF) {
//            DiplomaticEventPeaceRequestEF event = (DiplomaticEventPeaceRequestEF)e;
//            owner.requestPeace(event.getPlayer(), event.getMessage(), null);
//        } else if (e instanceof DiplomaticEventNonAgressionRequestEF) {
//            DiplomaticEventNonAgressionRequestEF event = (DiplomaticEventNonAgressionRequestEF)e;
//            owner.requestNonAgression(event.getPlayer(), event.getMessage(), null);
//        } else if (e instanceof DiplomaticEventResearchTreatyRequestEF) {
//            DiplomaticEventResearchTreatyRequestEF event = (DiplomaticEventResearchTreatyRequestEF)e;
//            owner.requestResearchTreaty(event.getPlayer(), event.getMessage(), null);
//        } else if (e instanceof DiplomaticEventTradeRouteRequestEF) {
//            DiplomaticEventTradeRouteRequestEF event = (DiplomaticEventTradeRouteRequestEF)e;
//            owner.requestTradeRoute(event.getPlayer(), event.getMessage(), null);
//        } else if (e instanceof DiplomaticEventSwapAssetsRequestEF) {
//            DiplomaticEventSwapAssetsRequestEF event = (DiplomaticEventSwapAssetsRequestEF)e;
//            PlayerAssets offered = event.getOfferedAssets();
//            PlayerAssets wanted = event.getWantedAssets();
//            owner.requestSwapAssets(event.getPlayer(), offered, wanted);
//        } else if (e instanceof DiplomaticEventDemandAssetsEF) {
//            DiplomaticEventDemandAssetsEF event = (DiplomaticEventDemandAssetsEF)e;
//            PlayerAssets wanted = event.getWantedAssets();
//            owner.demandAssets(event.getPlayer(), wanted);
        } else if (e instanceof BattleEndEF) {
            BattleEndEF event = (BattleEndEF)e;
            Battle battle = event.getBattle();
			BattlePlanet battlePlanet = battle.getPlanet();
            if (battlePlanet != null && battlePlanet.getOwner() != null) {
                Player planetOwner = battlePlanet.getOwner();
                Relations rels = MotEngine.instance.getRelations();
                if (planetOwner != getOwner() && rels.isWar(planetOwner, owner)) {
                    if (battle.isWinner(owner) && !battlePlanet.getPlanet().hasDefenses()) {
                        // Lets request troop transports to invade
                        Planet planet = battlePlanet.getPlanet();
                        owner.notifyNeedTroops(planet);
                        int qtyTroops = owner.getTransportManagerNotNull().getQtyTroopsWithDestination(battlePlanet.getPlanet());
                        if (qtyTroops == 0) {
                        	// I will bombard this planet
                        	owner.getGeneral().convertAttackToBombardAt(planet);
                        }
                    }
                }
            }
            if (battle.containsStartingPlayer(owner)) {
                owner.getStratagist().notiftBattleEnd(battle);
            }
        } else if (e instanceof PlanetInvadeEF) {
        }    	
    }

	public void process(MotEvent e) throws ApplicationError {
		throw new ApplicationError("Dont do that!!!!");
	}

}