/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.setup;

import java.util.HashMap;

import mot.common.util.ApplicationError;
import mot.common.xml.XMLDocumentHandler;

/**
 * Rather than loading the Race Options by parsing the XML document yourself ask this 
 * factory to give you the options for a race and it will load the default options then
 * apply the race specific overrides for you. It will cache the results so that it is 
 * not done more than once.
 */
public class OptionsFactory  {

    /**
     * This is a map from race name to selected race options.
     */
    private static HashMap mapSO = new HashMap();
    
    /**
     * This is a map from race name to race options.
     */
    private static HashMap mapRO = new HashMap();
    
    /**
     * These are the general setup options (non race specific).
     */
    private static SetupOptions generalOptions;

	public static void flush() {
		generalOptions = null;
		mapSO.clear();
		mapRO.clear();
	}

    public static synchronized void setGeneralOptions(SetupOptions options) {
        generalOptions = options;
    }

    public static synchronized boolean generalOptionsLoaded() {
        return (generalOptions != null);
    }

    private static synchronized SetupOptions _getGeneralOptions() throws ApplicationError {
        return (SetupOptions)XMLDocumentHandler.parseResource("xml/options_general.xml", true, "xml/options.xsd");
    }

    public static synchronized SetupOptions getGeneralOptions() throws ApplicationError {
        if (generalOptions == null) {
            generalOptions = _getGeneralOptions();
        }
        return generalOptions;
    }

    public static synchronized void setSetupOptions(String name, SetupOptions raceOptions) throws ApplicationError {
        mapSO.put(name, raceOptions);
        mapRO.put(name, new RaceOptions(name, raceOptions));
    }
    
    public static synchronized SetupOptions getSetupOptions(String race) throws ApplicationError {
    	SetupOptions options = (SetupOptions)mapSO.get(race);
        if (options == null) {
            options = (SetupOptions)XMLDocumentHandler.parseResource("xml/options_" + race.toLowerCase() + ".xml", true, "xml/options.xsd");
            mapSO.put(race, options);
        }
        return options;
    }

    public static synchronized RaceOptions getRaceOptions(String race) throws ApplicationError {
    	RaceOptions options = (RaceOptions)mapRO.get(race);
        if (options == null) {
            SetupOptions so = (SetupOptions)XMLDocumentHandler.parseResource("xml/options_" + race.toLowerCase() + ".xml", true, "xml/options.xsd");
            options = new RaceOptions(race, so, new SetupOptions(getGeneralOptions()));
            mapRO.put(race, options);
        }
        return options;
    }

}