/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.setup;
import mot.common.bean.NamedObject;
import mot.common.util.ApplicationError;
import mot.engine.setup.OptionsList;

/**
 * These are a set of race configuration options that also contains details of 
 * the race to which they belong AND the default values. This is used as a convenient 
 * bin of options from which a race character may be built because it contains the default
 * options as well as the race specific overrides - You may select each option and get the
 * correct value for the player character - It may come from the race override options or
 * the default options - but all options are defined.
 */
public class RaceOptions extends NamedObject implements OptionsList {

    /**
     * These are the general options containing all possible settings and default values for each race.
     */
    private SetupOptions generalOptions;
    
    /**
     * These are the race specific options overriding the values in the general options.
     */
    protected SetupOptions raceOptions;
    
    public RaceOptions(String name, SetupOptions raceOptions, SetupOptions generalOptions) {
        super(name);
        this.raceOptions = raceOptions;
        this.generalOptions = generalOptions;
    }
    
    public RaceOptions(String name, SetupOptions raceOptions) throws ApplicationError {
        super(name);
        this.raceOptions = raceOptions;
        this.generalOptions = OptionsFactory.getGeneralOptions();
    }
    
    public SetupOptions getGeneralOptions() {
        return generalOptions;
    }

    public void setGeneralOptions(SetupOptions v) {
        this.generalOptions = v;
    }


	public double getDoubleForName(String name) throws ApplicationError {
        if (raceOptions != null && raceOptions.getStringForName(name) != null) {
            return raceOptions.getDoubleForName(name);
        } else {
            return generalOptions.getDoubleForName(name);
        }
    }
    
	public int getIntForName(String name) throws ApplicationError {
        if (raceOptions != null && raceOptions.getStringForName(name) != null) {
            return raceOptions.getIntForName(name);
        } else {
            return generalOptions.getIntForName(name);
        }
    }
    
	public boolean getBoolForName(String name) throws ApplicationError {
        String v = getStringForName(name);
        return (v != null && v.equalsIgnoreCase("true"));
    }
    
	public String getStringForName(String name) throws ApplicationError {
        if (raceOptions != null) {
            String v = raceOptions.getStringForName(name);
            if (v != null) {
                return v;
            }
        }
        
        return generalOptions.getStringForName(name);
    }
    
	public String [] getSplitStringForName(String name, String delimiter) throws ApplicationError {
        String v = getStringForName(name);
        if (v != null) {
            return v.split(delimiter);
        } else {
            return null;
        }
    }
    
	public SetupOption getForName(String name) throws ApplicationError {
        if (raceOptions != null) {
            SetupOption v = raceOptions.getForName(name);
            if (v != null) {
                return v;
            }
        }
        
        return generalOptions.getForName(name);
    }
    
    
    public SetupOptions getRaceOptions() {
        return raceOptions;
    }

    public void setRaceOptions(SetupOptions v) {
        this.raceOptions = v;
    }

    /**
     * @return Returns the cost of all options selected in the race options.
     */
    public int getCost() {
        if (raceOptions == null) {
            return 0;
        } else {
            return raceOptions.getCost();
        }
    }

    public String getElementName() {
        return "race-options";
    }
}