/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.setup;

import mot.common.bean.NamedObject;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValuePairList;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * Contains race setup option information.
 */
public class SetupOption extends NamedObject {

	/**
	 * The amount of this option.
	 */
	private String value;

	/**
	 * The cost if selecting this option when starting the game.
	 */
	private int cost;

	/**
	 * If set this value is to be used for computer players when the game is played
	 * at this level. This allows higher bonuses to be specified for computer players
	 * at higher levels. Note - if no value for this level is found then the default
	 * no level (level = null) value is used.
     * For details of the values level may take see MotEngine.
	 */
	private Double level;

	/**
	 * An option may have a list of string parameter values.
     * e.g. Starting techs option has a name value pair list of tech type and tech name.
	 */
	private BasicNameValuePairList parameters;
    
    /**
     * This is the label describing this option to place onto the screen.
     */
    private String label;
    
    /**
     * This is a textual description this option to place used to display to users.
     * There only needs to be one description per name. This if there are 5 options
     * for a given setting, then only one needs to have a description.
     */
    private String description;
    
    /**
     * This is an optional long description textual description of this option to be 
     * used in a tool tip.
     */
    private String explanation;

    
    public SetupOption() {
    }

    public SetupOption(String name, String value, int cost) {
        setName(name);
        this.value = value;
        this.cost = cost;
    }

	public SetupOption(DomNode n) {
        setName(n.getNodeValueByName_String("name"));
        value = n.getNodeValueByName_String("value");
        cost = n.getNodeValueByName_Int("cost");
        //level = n.getNodeValueByName_Double("level");
        label = n.getNodeValueByName_String("label");
        description = n.getNodeValueByName_String("description");
        explanation = n.getNodeValueByName_String("explanation");
	}

	public String getValue() { return value; }
	public void setValue(String v) { value = v; }

	public int getCost() { return cost; }
	public void setCost(int v) { cost = v; }

	// for use by XML document handler
	public void setCost(String v) { cost = Integer.parseInt(v); }

	public Double getLevel() { return level; }
	public void setLevel(Double v) { level = v; }

	// for use by XML document handler
	public void setLevel(String v) { level = new Double(v); }

	public BasicNameValuePairList getParameters() { return parameters; }
	public void setParameters(BasicNameValuePairList v) { parameters = v; }

	public void setParameter(NameValuePair v) { 
        if (parameters == null) parameters = new BasicNameValuePairList();
        parameters.add(v); 
    }

	protected String getElementName() { return "setup-option"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
        writer.addElement("value", value);
        writer.addElement("cost", cost);
        writer.addElement("level", level);
        if (parameters != null) {
            parameters.toXML(writer);
        }
	}

    public String getLabel() {
        if (label == null) {
            return value;
        } else {
            return label;
        }
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

	public String getExplanation() {
		return explanation;
	}

	public void setExplanation(String explanation) {
		this.explanation = explanation;
	}
}