/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.setup;

import java.io.Serializable;
import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.setup.OptionsList;

/**
 * Contains race setup option information.
 */
public class SetupOptions extends XMLBean implements OptionsList, Serializable {

	private ArrayList options;
	private transient SetupOption [] array = null;

	public ArrayList getOptions() { return options; }
    
    public SetupOptions() {
    }

    public SetupOptions(SetupOptions other) {
        SetupOption [] others = other.getOptionsAsArray();
        options = new ArrayList();
        for (int i=0; i < others.length; i++) {
            options.add(others[i]);
        }
        array = null; 
    }

    public SetupOptions(SetupOption option) {
        options = new ArrayList();
        options.add(option);
        array = null; 
	}

    public SetupOptions(DomTree doc) {
    	DomNode root = doc.getNodeByName("options");
    	
    	DomNode[] oa = root.getNodesByName("option");
    	if (oa != null) {
    		for (int i = 0; i < oa.length; i++) {
    			setOption(new SetupOption(oa[i]));
			}
    	}
	}

	public void setOptions(ArrayList v) { 
        options = v; 
        array = null; 
    }

	public void setOption(SetupOption v) { 
        if (options == null) options = new ArrayList(); 	
        options.add(v); 
        array = null; 
    }
    
	public void removeOption(SetupOption v) { 
        if (options == null) return; 
        options.remove(v); 
        array = null; 
    }

	public SetupOption [] getOptionsAsArray() {
		if (array == null && options != null && options.size() > 0) {
			array = (SetupOption[])options.toArray(new SetupOption[(options.size())]);
		}
		return array;
	}
    
    /**
     * Add the list of options passed in to the current list of options.
     * @param so The list of options to add
     * @param override If true options specified in so will replace those in options otherwise they will not.
     */
    public void add(SetupOptions so, boolean override) throws ApplicationError {
        if (so == null || options == null) {
            return;
        }
        SetupOption [] a = so.getOptionsAsArray();
        for (int i=0; i<a.length; i++) {
            SetupOption existing = getForName(a[i].getName());
            if (existing != null && override) {
                removeOption(existing);
                setOption(a[i]);
            } else if (existing == null) {
                setOption(a[i]);
            }
        }
    }

    /**
     * If an option with name does not exist it creates a new option with name name
     * and value value, otherwise it updates the option name with value value.
     */
    public void setOption(String name, String value, int cost) throws ApplicationError {
        SetupOption so = getForName(name);
        if (so == null) {
            setOption(new SetupOption(name, value, cost));
        } else {
            so.setValue(value);
        }
    }

    public void addOption(String name, String value, int cost) throws ApplicationError {
        setOption(new SetupOption(name, value, cost));
    }

	public double getDoubleForName(String name) throws ApplicationError {
		String v = getStringForName(name);
		if (v == null) {
			return 0;
		} else {
			try {
				return Double.parseDouble(v);
			} catch (NumberFormatException e) {
				throw new ApplicationError("SetupOptions.getDoubleForName() option "+name+" is not a valid double");
			}
		}
	}

	public int getIntForName(String name) throws ApplicationError {
		String v = getStringForName(name);
		if (v == null) {
			return 0;
		} else {
			try {
				return Integer.parseInt(v);
			} catch (NumberFormatException e) {
				throw new ApplicationError("SetupOptions.getIntForName() option "+name+" is not a valid integer "+v);
			}
		}
	}

	public String getStringForName(String name) throws ApplicationError {
		SetupOption so = getForName(name);
		if (so != null) {
			return so.getValue();
		} else {
			return null;
		}
	}

	public String[] getAllStringsForName(String name) throws ApplicationError {
		SetupOption [] so = getAllForName(name);
		if (so == null) {
            return null;
        }
        String [] r = new String[so.length];
        for (int i=0; i<so.length; i++) {
            r[i] = so[i].getValue();
        }
        return r;
	}
	
	public boolean hasOption(String name) throws ApplicationError {
		SetupOption so = getForName(name);
		return (so != null);
	}

    /**
     * Returns all setup options in the list that have the name passed in.
     * @param name The name of the option
     * @return All setup option with the name name or null.
     * @throws mot.common.util.ApplicationError
     */
	public SetupOption [] getAllForName(String name) throws ApplicationError {
		SetupOption [] so = getOptionsAsArray();
        ArrayList result = null;
		if (so != null) {
			for (int i=0; i<so.length; i++) {
				if ( Comp.same(so[i].getName(), name) ) {
                    if (result == null) result = new ArrayList();
					result.add(so[i]);
				}
			}
		}
        if (result == null) {
            return null;
        } else {
            return (SetupOption[])result.toArray(new SetupOption[(result.size())]);
        }
	}
    
    /**
     * Returns the description for the setup option name passed in. If no description ecists the 
     * name is used.
     * @param name The name of the option
     * @return Returns the description for the setup option name passed in. If no description ecists the 
     * name is used.
     * @throws mot.common.util.ApplicationError
     */
	public String getDescriptionForName(String name) throws ApplicationError {
		SetupOption [] so = getAllForName(name);
		if (so != null && so.length > 0) {
			for (int i=0; i<so.length; i++) {
				if ( Comp.same(so[i].getName(), name) && so[i].getDescription() != null) {
                    return so[i].getDescription();
				}
			}
            return so[0].getName();
		}
        return null;
	}
    
	public String getExplanationForName(String name) throws ApplicationError {
		SetupOption [] so = getAllForName(name);
		if (so != null && so.length > 0) {
			for (int i=0; i<so.length; i++) {
				if ( Comp.same(so[i].getName(), name) && so[i].getExplanation() != null) {
                    return so[i].getExplanation();
				}
			}
		}
        return null;
	}
    
    /**
     * Returns the setup option in this list that has the name passed in. If multiple setup options
     * exist with the same name it returns the option with cost closest to zero.
     * @param name The name of the option
     * @return The best setup options or null if no option exists for this name
     * @throws mot.common.util.ApplicationError
     */
	public SetupOption getForName(String name) throws ApplicationError {
		SetupOption [] so = getOptionsAsArray();
        SetupOption best = null;
		if (so != null) {
			for (int i=0; i<so.length; i++) {
				if ( Comp.same(so[i].getName(), name) ) {
                    if (best == null || betterMatch(0, best, so[i]) ) {
                        best = so[i];
                    }
				}
			}
		}
		return best;
	}

    /**
     * @return Returns the cost of all options in this list.
     */
    public int getCost() {
		SetupOption [] so = getOptionsAsArray();
        int cost = 0;
		if (so != null) {
			for (int i=0; i<so.length; i++) {
				cost += so[i].getCost();
			}
		}
		return cost;
    }
    
    
    /**
     * Returns true of and only if the parameter so has a cost is closer to 
     * the parameter cost than the parameter best does.
     */
    public boolean betterMatch(int cost, SetupOption best, SetupOption so) {
        int dif1 = cost - best.getCost();
        int dif2 = cost - so.getCost();
        
        if (dif1 < 0) dif1 = -dif1;
        if (dif2 < 0) dif2 = -dif2;
        
        return (dif2 < dif1);
    }

	public String [] getNamesStartingWith(String prefix) {
		SetupOption [] so = getOptionsAsArray();
		if (so != null) {
			ArrayList list = new ArrayList();
			for (int i=0; i<so.length; i++) {
				if ( prefix == null || Comp.startsWith(so[i].getName(), prefix) ) {
					list.add( so[i].getName() );
				}
			}
			return (String []) list.toArray( new String[list.size()] );
		}
		return null;
	}

	protected String getElementName() { return "setup-options"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
		for (int i=0; i<options.size(); i++) {
			SetupOption r = (SetupOption)options.get(i);
            if (r != null) {
                r.toXML(writer);
            }
		}
	}

	public int getQuantity() {
		if (options == null) {
			return 0;
		} else {
			return options.size();
		}
	}

	public SetupOption getAtIndex(int index) {
        if (options == null || index >= options.size()) {
            return null;
        } else {
            return (SetupOption)options.get(index);
        }
	}

	public String toString() {
		if (options == null) return "options=None";
		return "options=" + Comp.toString(options.iterator(),", ");
	}

	public boolean getBoolForName(String name) throws ApplicationError {
        String v = getStringForName(name);
        return (v != null && v.equalsIgnoreCase("true"));
    }

	public String [] getSplitStringForName(String name, String delimiter) throws ApplicationError {
        String v = getStringForName(name);
        if (v != null) {
            return v.split(delimiter);
        } else {
            return null;
        }
    }


}