/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.Collection;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Biotechnologies extends TechnologyList {

	private static final String ELEMENT_NAME = "biotechnologies";
	
	private transient Double terraformRating = null;
	private transient Integer psiRating = null;
    private transient Double reproductionBonus = null;
    private transient Integer spyRating = null;

    public Biotechnologies() {
        
    }

    public Biotechnologies(Biotechnology w) {
        add(w);
    }

    public Biotechnologies(Biotechnologies o) throws ApplicationError {
        super(o);
    }

    public Biotechnologies(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getBiotechnology(names[i]) );
            }
        }
    }

    public Biotechnologies(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Biotechnology.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setBiotechnology(new Biotechnology(n[i]));
			}
		}
	}

	/**
     * The amount of terraform this player can perform which is calculated by the
     * formula : 1.0 + the sum  of all terraform in this list.
     * @return The max terraform level all the techs in this list allows.
     */
	public double getTerraformRating() {
		if (terraformRating != null) {
			return terraformRating.doubleValue();
		}

		Collection list = getAll(true);
		if (list == null) return 1.0;
		double result = 1.0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Biotechnology v = (Biotechnology)i.next();
            if (v.getTerraformRating() != null) {
                result = result + v.getTerraformRating().doubleValue();
            }
		}
		terraformRating = new Double(result);
		return result;
	}

    /**
     * The amount of psi ability this player can perform which is calculated by the
     * formula : 0.0 + the sum  of all psi ratings in this list.
     * @return The sum of psi ratings in all the techs in this list.
     */
	public int getPsiRating() {
		if (psiRating != null) {
			return psiRating.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Biotechnology v = (Biotechnology)i.next();
            if (v.getPsiRating() != null) {
                result = result + v.getPsiRating().intValue();
            }
		}
		psiRating = new Integer(result);
		return result;
	}

    /**
     * The reproduction bonus of this which is calculated by the
     * formula : 1.0 + the sum  of all re-production bonuses in this list.
     * @return The sum of reproduction bonuses in all the techs in this list.
     */
    public double getReproductionBonus() {
		if (reproductionBonus != null) {
			return reproductionBonus.doubleValue();
		}

		Collection list = getAll(false);
		if (list == null) return 0.0;
		double result = 0.0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Biotechnology v = (Biotechnology)i.next();
            if (v.getReproductionBonus() != null) {
                result = result + v.getReproductionBonus().doubleValue();
            }
		}
		reproductionBonus = new Double(result);
		return result;
    }

    /**
     * The spy rating of this which is calculated by the formula : 1.0 + the sum 
     * of all spy ratings in this list.
     * @return The sum of spy ratings in all the techs in this list.
     */
	public int getSpyRating() {
		if (spyRating != null) {
			return spyRating.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Biotechnology v = (Biotechnology)i.next();
            if (v.getSpyRating() != null) {
                result = result + v.getSpyRating().intValue();
            }
		}
		spyRating = new Integer(result);
		return result;
	}


	protected void changed() {
		terraformRating = null;
		psiRating = null;
        reproductionBonus = null;
        spyRating = null;
	}

	public void setBiotechnology(String v) throws ApplicationError {
		add( Technologies.getBiotechnologies().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError {
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_bios.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public Biotechnology getBiotechnology(String name) {
        return (Biotechnology)super.get(name);
    }

    public void setBiotechnology(Biotechnology v) {
        add(v);
    }

    public Biotechnology getBiotechnologyAtIndex(int index) {
        return (Biotechnology)getTechnologyAtIndex(index);
    }

}