/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * The terraforming level of a player is calculated to be the product of all bio techs
 * containing a terraforming rating. However the group attribute of technology is used
 * to make some technologies obsolete.
 * The terraforming group is used in calculating the total terraforming level allowed
 * in a list of terraforming technologies. Only a single terraforming rating can be used 
 * from a single group.
 * Note that group is optional, if no group is specified thia terraforming rating will be 
 * used. If this field exist then only the maximum value in this group will be used.
 * This allows the game to make some technologies obsolete when this technology is discovered.
 */
public class Biotechnology extends Technology {

	public static final String ELEMENT_NAME = "biotechnology";

	/**
	 * The terraforming index permitted by this technology.
	 */
	private Double terraformRating;

	/**
	 * The psi rating for this technology. If a player has this technology his
	 * psi rating is increased by this amount.
	 *
	 * It Calculate the psi ratings to determine if the people still have the nerve to
	 * attack. It has the following effect;
	 * 1 - If the defending player has Psi Rating more than PSI_MISS_TURN_LEVEL greater
	 * than the attacking player then there is a probability the attack may not happen
	 * but the weapons will be marked as fired.
	 * Calculation = Random between PSI_MISS_TURN_LEVEL and PSI_SURRENDER_RATING > psi difference.
	 * 2 - If the attacking player has Psi rating more than PSI_SURRENDER_RATING grater
	 * than the defender then there is a chance the defender may surrender.
	 * Calculation = Random between PSI_SURRENDER_RATING and PSI_MAX_RATING > psi difference.
	 */
	private Integer psiRating;
    
    /**
     * This is a factor applied to planets owned by races with this technology that increases
     * their reproduction rate. e.g. a value of 0.1 would increase population growth by 10%.
     */
    private Double reproductionBonus;

    /**
     * spy rating is the ability this technology has to improve this races espionage. See 
     * @see mot.engine.spies.SpyRole for details of how spying works.
     * The difference in the spy level between the two players provided the spy advantage
     * which in turn defines the effectiveness of each spy.
     */
    private Integer spyRating;


	public static final int PSI_MISS_TURN_LEVEL = 10;
	public static final int PSI_MAX_MISS_TURN_RATING = 100;

	public static final int PSI_SURRENDER_RATING = 30;
	public static final int PSI_MAX_RETREAT_RATING = 50;
	public static final int PSI_MAX_SURRENDER_RATING = 100;

	public Biotechnology() {
	}
	
	public Biotechnology(DomNode node) {
		super(node);
		terraformRating = node.getNodeValueByName_Double("terraform-rating");
		psiRating = node.getNodeValueByName_Integer("psi-rating");
		reproductionBonus = node.getNodeValueByName_Double("reproduction-bonus");
		spyRating = node.getNodeValueByName_Integer("spy-rating");
	}
	
	public Double getTerraformRating() { return terraformRating; }
	public Integer getPsiRating() { return psiRating; }
    public Double getReproductionBonus() { return reproductionBonus; }
	public Integer getSpyRating() { return spyRating; }

	public void setTerraformRating(Double v) { terraformRating = v; }
	public void setPsiRating(Integer v) { psiRating = v; }
    public void setReproductionBonus(Double v) { reproductionBonus = v; }
    public void setSpyRating(Integer v) { spyRating = v; }
    
	public void setTerraformRating(double v) { terraformRating = new Double(v); }
	public void setPsiRating(int v) { psiRating = new Integer(v); }
    public void setReproductionBonus(double v) { reproductionBonus = new Double(v); }
	public void setSpyRating(int v) { spyRating = new Integer(v); }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", getDescription());
		writer.addElement("terraform-rating", terraformRating);
		writer.addElement("psi-rating", psiRating);
		writer.addElement("reproduction-bonus", reproductionBonus);
		writer.addElement("spy-rating", spyRating);
        writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("group", getGroup());
		Restriction restriction = getRestriction();
		if (restriction != null && restriction instanceof XMLBean) {
			((XMLBean)restriction).toXML(writer);
		}
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[9];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("terraform-rating", terraformRating);
		r[3] = new NameValuePair("psi-rating", psiRating);
		r[4] = new NameValuePair("reproduction-bonus", reproductionBonus);
		r[5] = new NameValuePair("spy-rating", spyRating);
        r[6] = new NameValuePair("cost to build", getCostToBuild());
		r[7] = new NameValuePair("cost to research", getCostToResearch());
		r[8] = new NameValuePair("group", getGroup());
		return r;
	}


	protected String getElementName() { return ELEMENT_NAME; }
    
    public String getTechType() {
        return mot.common.technology.Technologies.TECH_TYPE_BIOTECHNOLOGY;
    }
}