/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * @author Paul Davis
 *
 */
public class Computer extends Technology {

	public static final String ELEMENT_NAME = "computer";

	/**
	 * This is the power of this computer, so for example 1 would be a very basic computer,
	 * 2 would be twice as powerful, 4 would be four times as powerful etc. 
     * The difference in computer power between two craft effects the manoeuvrability and thus the 
     * damage inflicted when one attacks the other. See BattleCraft for the details on manoeuvrability,
     * but basically it is the (impulse speed of the craft + the turning speed of the craft)/20 + computer power.
	 **/
	private Integer power;
    
    /**
     * Computers are used to driver machinery, produce films, run the internet etc. In effect better computers 
     * can also provide a race more influence. The more influence a player has the more spies a race recruits.
     * A race recruits 1 spy for every 100 population per influence. Thus a race with 200 people and influence 
     * 4 would have 8 spies. Allows players to build techs like GalaxyNet.
     * 
     * @see mot.engine.visitors.Visitor for details of spying operation.
     */
    private Integer influence;

    /**
     * Computers are used to trade commodities and currencies as a normal part of galactic trade.
     * Better computer technology can increase the commerce performed and this increase the benefits
     * a player receives from trade. Allows players to build techs like GalacticStockExchange.
     * 
     * @see mot.engine.relations.TradeAgreementTreaty for details of trading.
     */
    private Integer commerce;

	public Computer() {
	}
	
	public Computer(DomNode node) {
		super(node);
		power = node.getNodeValueByName_Integer("power");
		influence = node.getNodeValueByName_Integer("influence");
		commerce = node.getNodeValueByName_Integer("commerce");
	}
	
	public Integer getPower() { return power; }
	public void setPower(Integer v) { power = v; }
    public int getPowerAsInt() { return (power == null) ? 0 : power.intValue(); }

    public Integer getInfluence() { return influence; }
    public void setInfluence(Integer influence) { this.influence = influence; }
    public int getInfluenceAsInt() { return (influence == null) ? 0 : influence.intValue(); }

    public Integer getCommerce() { return commerce; }
    public void setCommerce(Integer commerce) { this.commerce = commerce; }
    public int getCommerceAsInt() { return (commerce == null) ? 0 : commerce.intValue(); }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", getDescription());
		writer.addElement("power", power);
		writer.addElement("influence", influence);
		writer.addElement("commerce", commerce);
        writer.addElement("space", getSpace());
        writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("group", getGroup());
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[9];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("power", power);
		r[3] = new NameValuePair("influence", influence);
		r[4] = new NameValuePair("commerce", commerce);
        r[5] = new NameValuePair("space", getSpace());
        r[6] = new NameValuePair("cost to build", getCostToBuild());
		r[7] = new NameValuePair("cost to research", getCostToResearch());
		r[8] = new NameValuePair("group", getGroup());
		return r;
	}
	
    public String getStatsText() {
        return "Computer "+getName()+" power "+power+" influence="+influence;
    }
    
    public String getDescription() {
        String description = super.getDescription();
        if (description != null) {
            return description;
        } else {
            String r = "Computer "+getName();
            if (power != null) {
                r += " has battle tactical analysis "+power; 
            }
            if (influence != null) {
                r += " allows cultural influence of "+influence; 
            }
            if (commerce != null) {
                r += " has commercial benefit of "+commerce; 
            }
            return r;
        }
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_COMPUTER;
    }

    
    public static final String GROUP_COMPUTER = "Computer";
}