/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import mot.common.bean.NameList;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Computers extends TechnologyList {

	private static final String ELEMENT_NAME = "computers";
	
	private transient Integer power = null;
	private transient Integer influence = null;
	private transient Integer commerce = null;

    public Computers() {

    }

    public Computers(Computers o) throws ApplicationError {
        super(o);
    }

    public Computers(Computer w) {
        add(w);
    }

    public Computers(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getComputer(names[i]) );
            }
        }
    }

    public Computers(ArrayList<String> names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.size(); i++) {
                add( Technologies.getComputer(names.get(i)) );
            }
        }
    }

	public Computers(NameList names) throws ApplicationError {
        if (names != null) {
        	String[] na = names.getNamesAsArray();
        	if (na != null) {
	            for (int i=0; i<na.length; i++) {
	                add( Technologies.getComputer(na[i]) );
	            }
        	}
        }
	}

	public Computers(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Computer.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setComputer(new Computer(n[i]));
			}
		}
	}

	public int getPower() {
		if (power != null) {
			return power.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Computer v = (Computer)i.next();
			if (v.getPower() != null) {
                result += Comp.nnv(v.getPower(), 0);
            }
		}
		power = new Integer(result);
		return result;
	}

	public int getInfluence() {
		if (influence != null) {
			return influence.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Computer v = (Computer)i.next();
			if (v.getInfluence() != null) {
                result += Comp.nnv(v.getInfluence(), 0);
            }
		}
		influence = new Integer(result);
		return result;
	}

	public int getCommerce() {
		if (commerce != null) {
			return commerce.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Computer v = (Computer)i.next();
			if (v.getCommerce() != null) {
                result += Comp.nnv(v.getCommerce(), 0);
            }
		}
		commerce = new Integer(result);
		return result;
	}

	protected void changed() {
		power = null;
		influence = null;
	}

	public void setComputer(String v) throws ApplicationError {
		add( Technologies.getComputers().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_computers.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public Computer getComputer(String name) {
        return (Computer)super.get(name);
    }

    public Computer getComputerAtIndex(int index) {
        return (Computer)getTechnologyAtIndex(index);
    }

    public void setComputer(Computer v) {
        add(v);
    }
    
    public Computer getMostPowerfulComputer() {
        Computer best = null;
        Collection c = getCurrentList();
        if (c != null) {
            Iterator i = c.iterator();
            while (i.hasNext()) {
                Computer v = (Computer)i.next();
                if (v.getPower() != null) {
                    if (best == null || v.getPower().intValue() > best.getPower().intValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
    }

}