/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 *  @author Paul Davis
 *
 */
public class Engine extends Technology {

	public static final String ELEMENT_NAME = "engine";
	
	public static final String GROUP_ANTI_MATTER = "AntiMatter";
	public static final String GROUP_TELEPORT = "Teleport";

	public Engine() {
	}

	public Engine(DomNode node) {
		super(node);
		
		maxTravelDistance = node.getNodeValueByName_Integer("max-travel-distance");
		warpSpeed = node.getNodeValueByName_Int("warp-speed");
		impulseSpeed = node.getNodeValueByName_Int("impulse-speed");
		turningSpeed =  node.getNodeValueByName_Int("turning-speed");
		//costToBuildFactor = node.getNodeValueByName_Double("build-cost-hull-factor");
	}
	
	public boolean isTeleport() { return Comp.same(GROUP_TELEPORT, getGroup()); }
	
	/**
	 * The distance these engines can travel without re-fueling.
	 */
	private int maxTravelDistance;

	public int getMaxTravelDistance() {	return maxTravelDistance; }
	public void setMaxTravelDistance(int v) { maxTravelDistance = v; }

	/**
	 * The Speed of the engines - distance moveable per turn between star systems.
	 */
	private int warpSpeed;

	public int getWarpSpeed() { return warpSpeed; }
	public void setWarpSpeed(int v) { warpSpeed = v; }

	/**
	 * The Speed of the engines at sub warp (speed in battle). Qty squares per move.
	 * The battle speed of a craft would be the engines impulse speed divided by the hulls
	 * movement cost.
	 */
	private int impulseSpeed;

	public int getImpulseSpeed() { return impulseSpeed;	}
	public void setImpulseSpeed(int v) { impulseSpeed = v;	}

	/**
	 * The Speed of the engines can turn the craft around (in degrees 0-359). The quantity of
	 * degrees of turn each movement speed creates for turning.
	 * The turning speed of a craft would be the engines turning power divided by the nulls
	 * turning cost.
	 */
	private int turningSpeed;

	public int getTurningSpeed() { return turningSpeed;	}
	public void setTurningSpeed(int v) { turningSpeed = v;	}

	/**
	 * The size of a hull can effect the cost to build some technologies on the ship.
	 * This is the factor to which technologies that this does apply to (see Technology
	 * base class for details of which tech this effects)
	 */
	private double costToBuildFactor;

	public double getCostToBuildFactor() { return costToBuildFactor; }
	public void setCostToBuildFactor(double v) { costToBuildFactor = v; }

    /**
     * @return The size of the shock wave generated by this engine when a warp core explosion
     * is generated. See WeaponInstanceDetonator for details.
     */
	public int getWarpCoreOverloadDamage() { 
        int v = (int)(impulseSpeed * 0.5);
        return 25 + v;
    }

    public String getDescription() {
        String description = super.getDescription();
        if (description != null) {
            return description;
        } else {
            return "Engine "+getName()+" is capable of warp "+warpSpeed+" and impulse "+impulseSpeed+". It has a range of "+maxTravelDistance;
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", super.getDescription());
		writer.addElement("max-travel-distance", maxTravelDistance);
		writer.addElement("warp-speed", warpSpeed);
		writer.addElement("impulse-speed", impulseSpeed);
		writer.addElement("turning-speed", turningSpeed);
        writer.addElement("space", getSpace());
        writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("use-build-cost-hull-factor", getUseBuildCostHullFactor());
		writer.addElement("group", getGroup());
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[10];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("max travel distance", maxTravelDistance);
		r[3] = new NameValuePair("warp speed", warpSpeed);
		r[4] = new NameValuePair("impulse speed", impulseSpeed);
		r[5] = new NameValuePair("turning speed", turningSpeed);
        r[6] = new NameValuePair("space", getSpace());
        r[7] = new NameValuePair("cost to build", getCostToBuild());
		r[8] = new NameValuePair("cost to research", getCostToResearch());
		r[9] = new NameValuePair("group", getGroup());
		return r;
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_ENGINE;
    }

}