/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.Collection;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Engines extends TechnologyList {

	private static final String ELEMENT_NAME = "engines";
	private transient Integer maxTravelDistance = null;

    public Engines() {

    }

    public Engines(Engines o) throws ApplicationError {
        super(o);
    }

    public Engines(Engine w) {
        add(w);
    }

    public Engines(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getEngine(names[i]) );
            }
        }
    }

	public Engines(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Engine.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setEngine(new Engine(n[i]));
			}
		}
	}

	public int getMaxTravelDistance() {
		if (maxTravelDistance != null) {
			return maxTravelDistance.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Engine v = (Engine)i.next();
			if (v.getMaxTravelDistance() > result) {
				result = v.getMaxTravelDistance();
			}
		}
		maxTravelDistance = new Integer(result);
		return result;
	}

	protected void changed() {
		maxTravelDistance = null;
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_engines.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public Engine getEngine(String name) {
        return (Engine)super.get(name);
    }

    public Engine getEngineAtIndex(int index) {
        return (Engine)getTechnologyAtIndex(index);
    }

    public void setEngine(Engine v) {
        add(v);
    }

	public Engine getFastestEngine(String group) {
        Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        Engine best = null;
        for (int i=0; i<list.length; i++) {
            Engine c = (Engine)list[(i)];
            if (Comp.same(group, c.getGroup())) {
                if (best == null || best.getWarpSpeed() < c.getWarpSpeed()) {
                    best = c;
                } else if (best.getWarpSpeed() == c.getWarpSpeed()) {
                    if (best.getMaxTravelDistance() < c.getMaxTravelDistance()) {
                        best = c;
                    }
                }
            }
        }
		return (Engine)best;
	}

	public Engine getFastestEngine() {
        Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        Engine best = null;
        for (int i=0; i<list.length; i++) {
            Engine c = (Engine)list[(i)];
            if (best == null || best.getWarpSpeed() < c.getWarpSpeed()) {
                best = c;
            } else if (best.getWarpSpeed() == c.getWarpSpeed()) {
                if (best.getMaxTravelDistance() < c.getMaxTravelDistance()) {
                    best = c;
                }
            }
        }
		return (Engine)best;
	}

	public Engine getLongestRangeEngine() {
        Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        Engine best = null;
        for (int i=0; i<list.length; i++) {
            Engine c = (Engine)list[(i)];
            if (best == null || best.getMaxTravelDistance() < c.getMaxTravelDistance()) {
                best = c;
            } else if (best.getMaxTravelDistance() == c.getMaxTravelDistance()) {
                if (best.getWarpSpeed() < c.getWarpSpeed()) {
                    best = c;
                }
            }
        }
		return (Engine)best;
	}

}