/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.Collection;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Factories extends TechnologyList {

    private static final String ELEMENT_NAME = "factories";

	public Factories() {
        
    }

    public Factories(Factory w) {
        add(w);
    }

    public Factories(Factories o) throws ApplicationError {
        super(o);
    }

    public Factories(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getFactory(names[i]) );
            }
        }
    }

    public Factories(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Factory.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setFactory(new Factory(n[i]));
			}
		}
	}

	private transient Double qtyPerPopulation = null;
    private transient Integer production = null;
    private transient Double minEcologyWork = null;
    private transient Double buildQuality = null;
    private transient Double propaganda = null;
    private transient Integer costToBuild = null;
    
	//public Factory getBestFactory() {
	//	return (Factory)getBest();
	//}
    
    /**
     * @return The quantity of factories per unit of population for a given player that has discovered all factories in this list.
     */
    public double getQtyPerPopulation() {
		if (qtyPerPopulation == null) {
			Collection list = getAll(true);
			if (list == null) {
				qtyPerPopulation = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Factory v = (Factory)i.next();
		            result = result + v.getQtyPerPopulation();
				}
				qtyPerPopulation = new Double(result);
			}
		}
		return qtyPerPopulation.doubleValue();
    }
    
    /**
     * @return Total production per factory for a given player that has discovered all factories in this list.
     */
    public int getProduction() {
		if (production == null) {
			Collection list = getAll(true);
			if (list == null) {
				production = new Integer(0);
			} else {
				Iterator i = list.iterator();
		        int result = 0;
				while (i.hasNext()) {
					Factory v = (Factory)i.next();
		            result = result + v.getProduction();
				}
				production = new Integer(result);
			}
		}
		return production.intValue();
    }
    
    /**
     * @return Minimum ecology work per factory for a given player that has discovered all factories in this list.
     */
    public double getMinEcologyWork() {
		if (minEcologyWork == null) {
			Collection list = getAll(true);
			if (list == null) {
				minEcologyWork = 0.5;
			} else {
				Iterator i = list.iterator();
				while (i.hasNext()) {
					Factory v = (Factory)i.next();
					if (minEcologyWork == null || (v.getMinEcologyWork() > 0.1 && v.getMinEcologyWork() < minEcologyWork)) {
						minEcologyWork = v.getMinEcologyWork();
					}
				}
				if (minEcologyWork == null) minEcologyWork = 0.5;
			}
		}
		return minEcologyWork.doubleValue();
    }
    
    public double getBuildQuality() {
		if (buildQuality == null) {
			Collection list = getAll(true);
			if (list == null) {
				buildQuality = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Factory v = (Factory)i.next();
		            result = result + v.getBuildQuality();
				}
				buildQuality = new Double(result);
			}
		}
		return buildQuality.doubleValue();
    }
    
    public double getPropaganda() {
		if (propaganda == null) {
			Collection list = getAll(true);
			if (list == null) {
				propaganda = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0.0;
				while (i.hasNext()) {
					Factory v = (Factory)i.next();
		            result = result + v.getPropaganda();
				}
				propaganda = new Double(result);
			}
		}
		return propaganda.doubleValue();
    }
    
    private static final int DEFAULT_COPST_TO_BUILD = 50;
    public int getCostToBuild() {
		if (costToBuild == null) {
			Collection list = getAll(true);
			if (list != null) {
				Iterator i = list.iterator();
				int r = Integer.MAX_VALUE;
				while (i.hasNext()) {
					Factory v = (Factory)i.next();
					if (Comp.same(v.getGroup(), Factory.GROUP_PRODUCTION)) {
						if (v.getCostToBuild() < r) {
							r = v.getCostToBuild();
						}
					}
				}
				costToBuild = new Integer(r);
			}
		}
		if (costToBuild == null) {
			Logger.logWarning("Factories.getCostToBuild() No Production factory!");
			return DEFAULT_COPST_TO_BUILD;
		} else {
			return costToBuild.intValue();
		}
    }
    
	protected void changed() {
	    qtyPerPopulation = null;
	    production = null;
	    minEcologyWork = null;
	    buildQuality = null;
	    propaganda = null;
	    costToBuild = null;
	}

	public void setFactory(String v) throws ApplicationError {
		add( Technologies.getFactories().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_factories.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public Factory getFactory(String name) {
        return (Factory)super.get(name);
    }

    public void setFactory(Factory v) {
        add(v);
    }

    public Factory getFactoryAtIndex(int index) {
        return (Factory)getTechnologyAtIndex(index);
    }

}