/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * @author Paul Davis
 *
 */
public class Factory extends Technology {

	public static final String ELEMENT_NAME = "factory";
	public static final String GROUP_PRODUCTION = "Production";
	public static final String GROUP_OTHER = "Other";
	
	/**
	 * The qty of these factories that can be operated per unit of population.
	 */
	private Double qtyPerPopulation;

	/**
	 * The quantity of production required to perform ecology work per factory of this type to 
     * remove the pollution it produces.
	 */
	private Double minEcologyWork;

	/**
	 * The amount of production per turn this factory produces
	 */
	private Integer production;
	
	/**
	 * This is the build quality bonus that effects a crafts performance. 
	 */
	private Double buildQuality;

	/**
	 * This is the % bonus to influence this infrastructure provides this player. 
	 * A value of 0.1 implies an increase of 10%
	 */
	private Double propaganda;

	public Factory() {
	}
	
	public Factory(DomNode node) {
		super(node);
		
		qtyPerPopulation = node.getNodeValueByName_Double("qty-per-population");
		minEcologyWork = node.getNodeValueByName_Double("min-ecology-work");
		production = node.getNodeValueByName_Integer("production");
		buildQuality = node.getNodeValueByName_Double("build-quality");
		propaganda = node.getNodeValueByName_Double("propaganda");
	}
	
	public double getQtyPerPopulation() { return (qtyPerPopulation == null) ? 0.0 : qtyPerPopulation.doubleValue(); }
	public double getMinEcologyWork() { return (minEcologyWork == null) ? 0.0 : minEcologyWork.doubleValue(); }
	public int getProduction() { return (production == null) ? 0 : production.intValue(); }
	public double getBuildQuality() { return (buildQuality == null) ? 0.0 : buildQuality.doubleValue(); }
	public double getPropaganda() { return (propaganda == null) ? 0.0 : propaganda.doubleValue(); }
	
	public void setQtyPerPopulation(Double v) { qtyPerPopulation = v; }
	public void setMinEcologyWork(Double v) { minEcologyWork = v; }
	public void setProduction(Integer v) { production = v; }
	public void setBuildQuality(Double v) {	buildQuality = v; }
	public void setPropaganda(Double v) {	propaganda = v; }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("qty-per-population", qtyPerPopulation);
		writer.addElement("min-ecology-work", minEcologyWork);
		writer.addElement("production", production);
		writer.addElement("build-quality", buildQuality);
		writer.addElement("propaganda", propaganda);
		writer.addElement("image", getImage());
        writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("description", super.getDescription());
		writer.addElement("group", getGroup());
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[11];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("qty per population", qtyPerPopulation);
		r[3] = new NameValuePair("min ecology work", minEcologyWork);
		r[4] = new NameValuePair("production", production);
		r[5] = new NameValuePair("build quality", buildQuality);
		r[6] = new NameValuePair("propaganda", propaganda);
		r[7] = new NameValuePair("image", getImage());
        r[8] = new NameValuePair("cost to build", getCostToBuild());
		r[9] = new NameValuePair("cost to research", getCostToResearch());
		r[10] = new NameValuePair("group", getGroup());
		return r;
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_FACTORY;
    }

    public String getDescription() {
        String description = super.getDescription();
        if (description != null) {
            return description;
        } else {
            return "Factory "+getName()+" uses "+1/qtyPerPopulation+" people and produces "+production+" production and " +minEcologyWork+" damage to the environment per factory";
        }
    }

}