/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.crafts.WeaponBays;

/**
 * This represents a Craft's hull (body) in the game. It is a shell into which all 
 * the other parts that constitute an operable craft are mounted. The hull selected
 * to build a craft with determined:
 * 	1 - The image used to display the craft
 * 	2 - The amount of stuff that can be put into the craft. Stuff takes space and hulls
 * 		have different amounts of space.
 *  3 - Weapons fire animation. Some hulls have a single weapon bay, and all weapons MUST
 *      fire from there. Others have multiple weapon bays so when they fire a weapon the
 *      weapons fire is animated from multiple points.
 *  4 - Has a dish. Some weapons are fired from a HULL's dish (not weapon bay). For a 
 *  	craft to support these weapons it must have a dish! A Hull is deemed to have a dish
 *  	if its dish offset is not null.
 *
 */
public class Hull extends Technology {

	public static final String ELEMENT_NAME = "hull";

	/**
	 * The cost of turning this craft during a battle.
	 * The turning speed of a craft would be the engines turning power divided by the hulls
	 * turning cost.
	 * A small craft turns fast and would have a value of about 0.5 while a huge craft turns
	 * slowly and so would have a value of about 1.5.
	 */
	private double battleTurningCost;

	/**
	 * The cost of moving this craft during a battle.
	 * The battle speed of a craft would be the engines impulse speed divided by the hulls
	 * movement cost.
	 * A small craft turns fast and would have a value of about 0.8 while a huge craft
	 * turns slowly and so  would have a value of about 1.2.
	 */
	private double battleMovementCost;

	/**
	 * Returns the qty of hit points this hull can withstand before being destroyed.
	 */
	private int maxDamage;

	/**
	 * A list of weapon bays on this hull.
	 */
	private WeaponBays weaponBays;

    /**
     * Some technology on a craft cost more to build depending on the size of the hull.
     * For those technologies this is the factor for this hull.
     */
    private double costToBuildFactor;
    
    /**
     * This is the maximum quantity of crew that this hull can contain. 
     */
    private int crew;
    
    /**
     * This is information multiple purposes. Different hulls can use this for different reasons.
     * The current values it contains are;
     *  1 - For battle stations
     *      The name of a module to be associated with the creation of this battle station.
     */
    private String meta;

	/**
	 * Specification of the different weapons bays this hull contains. Each weapons bay
	 * details will be used to create a weapons bay on a craft created with this hull
	 * that uses these details. Certain bays may only be used to contain certain types
	 * of weapons, fire at certain angles (from firing, rear firing etc). 
	 */
	private WeaponBayDetails [] weaponBayDetails;
	
	/**
	 * When this hull is destroyed - does it cause wreckage (and salvage).
	 */
	private boolean destructonCanCauseWreckage = true;
	
	public Hull() {
    }
 
	public Hull(DomNode node) {
		super(node);
		battleTurningCost = node.getNodeValueByName_Double("battle-turning-cost");
		battleMovementCost = node.getNodeValueByName_Double("battle-movement-cost");
		maxDamage = node.getNodeValueByName_Integer("max-damage");
		crew = node.getNodeValueByName_Integer("crew");
		costToBuildFactor = node.getNodeValueByName_Double("cost-to-build-factor");
		meta = node.getNodeValueByName_String("meta");
		DomNode [] wbn = node.getNodesByName("weapon-bay-details");
		if (wbn != null) {
			weaponBayDetails =  new WeaponBayDetails[wbn.length];
			for (int i = 0; i < wbn.length; i++) {				
				weaponBayDetails[i] = new WeaponBayDetails(wbn[i]);
			}
		}
		destructonCanCauseWreckage = node.getNodeValueByName_Bool("destructon-can-cause-wreckage", true);
	}

	public double getBattleTurningCost() { return battleTurningCost; }
	public double getBattleMovementCost() { return battleMovementCost; }
	public int getMaxDamage() { return maxDamage; }
	public double getCostToBuildFactor() { return costToBuildFactor; }
    public WeaponBays getWeaponBays() { return weaponBays; }
    
	public void setBattleTurningCost(double v) { battleTurningCost = v; }
	public void setBattleMovementCost(double v) { battleMovementCost = v; }
	public void setMaxDamage(int v) { maxDamage = v; }
	public void setCostToBuildFactor(double v) { costToBuildFactor = v; }
    public void setWeaponBays(WeaponBays v) { weaponBays = v; }
    
    public String getDescription() {
        String description = super.getDescription();
        if (description != null) {
            return description;
        } else {
            return "Hull "+getName()+" has space "+getSpace()+", maximum damage of "+maxDamage+" and crew "+getCrew();
        }
    }

    /**
     * Returns the space that would be remaining in this hull is this technology was 
     * fitted (and no other).
     */
    public int getRemainingSpaceWith(Technology [] tech) {
        if (tech == null) {
            return getSpace();
        }
        
        int space = getSpace();
        
        for (int i=0; i<tech.length; i++) {
            if (tech[i].getUseBuildCostHullFactor()) {
                space -= tech[i].getSpace(costToBuildFactor);
            } else {
                space -= tech[i].getSpace();
            }
        }
        
        return space;
    }
    
    /**
     * Returns the space that would be remaining in this hull is this technology was 
     * fitted (and no other).
     */
    public int getRemainingSpaceWith(Technology tech) {
        if (tech == null) {
            return getSpace();
        }
        
        if (tech.getUseBuildCostHullFactor()) {
            return getSpace() - tech.getSpace(costToBuildFactor);
        } else {
            return getSpace() - tech.getSpace();
        }
    }
    
    public boolean isCraft() {
        String group = getGroup();
        return group != null && GROUP_CRAFT.equalsIgnoreCase(group);
    }
    
    public boolean isFighter() {
        String group = getGroup();
        return group != null && GROUP_FIGHTER.equalsIgnoreCase(group);
    }
    
    public boolean isStation() {
        String group = getGroup();
        return group != null && (GROUP_OUTPOST.equalsIgnoreCase(group) || GROUP_BATTLE_STATION.equalsIgnoreCase(group) || GROUP_SPACE_STATION.equalsIgnoreCase(group));
    }
    
    public boolean isBattleStation() {
        String group = getGroup();
        return group != null &&  GROUP_BATTLE_STATION.equalsIgnoreCase(group);
    }
    
    public boolean isOutpost() {
        String group = getGroup();
        return group != null && GROUP_OUTPOST.equalsIgnoreCase(group) ;
    }
    
    public boolean isSpaceStation() {
        String group = getGroup();
        return group != null &&  GROUP_SPACE_STATION.equalsIgnoreCase(group);
    }
    
    public boolean isSentinel() {
        String group = getGroup();
        return group != null && GROUP_SENTINEL.equalsIgnoreCase(group);
    }
    
    public String getStatsText() {
        return "Hull "+getName()+" Hit Points "+maxDamage;
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", super.getDescription());
		writer.addElement("image", super.getImage());
		writer.addElement("battle-turning-cost", battleTurningCost);
		writer.addElement("battle-movement-cost", battleMovementCost);
		writer.addElement("max-damage", maxDamage);
		writer.addElement("space", getSpace());
		writer.addElement("crew", crew);
		if (weaponBayDetails != null) {
			for (int i = 0; i < weaponBayDetails.length; i++) {
				weaponBayDetails[i].toXML(writer);
			}
		}
        writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("maintenance-cost", getMaintenanceCost());
		writer.addElement("cost-to-build-factor", costToBuildFactor, true);
		writer.addElement("group", getGroup());
		writer.addElement("meta", getMeta());
		writer.addElement("destructon-can-cause-wreckage", destructonCanCauseWreckage);

		Restriction restriction = getRestriction();
		if (restriction != null && restriction instanceof XMLBean) {
			((XMLBean)restriction).toXML(writer);
		}
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[15];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("image", super.getImage());
		r[3] = new NameValuePair("battle turning cost", battleTurningCost);
		r[4] = new NameValuePair("battle movement cost", battleMovementCost);
		r[5] = new NameValuePair("max damage", maxDamage);
		r[6] = new NameValuePair("space", getSpace());
		r[7] = new NameValuePair("crew", crew);
        r[8] = new NameValuePair("cost to build", getCostToBuild());
		r[9] = new NameValuePair("cost to research", getCostToResearch());
		r[10] = new NameValuePair("maintenance cost", getMaintenanceCost());
		r[11] = new NameValuePair("cost to build factor", costToBuildFactor);
		r[12] = new NameValuePair("group", getGroup());
		r[13] = new NameValuePair("meta", getMeta());
		r[14] = new NameValuePair("destructon-can-cause-wreckage", destructonCanCauseWreckage);
		return r;
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_HULL;
    }


    public void setCrew(int crew) {
        this.crew = crew;
    }


    public int getCrew() {
        return crew;
    }

    public static final String GROUP_CRAFT = "craft";
    public static final String GROUP_FIGHTER = "fighter";
    public static final String GROUP_OUTPOST = "outpost";
    public static final String GROUP_BATTLE_STATION = "battle_station";
    public static final String GROUP_SPACE_STATION = "space_station";
    public static final String GROUP_SENTINEL = "sentinel";

    public void setMeta(String meta) {
        this.meta = meta;
    }


    public String getMeta() {
        return meta;
    }

	public WeaponBayDetails[] getWeaponBayDetails() {
		return weaponBayDetails;
	}

	public void setWeaponBayDetails(WeaponBayDetails[] weaponBayDetails) {
		this.weaponBayDetails = weaponBayDetails;
	}

	public void setWeaponBayDetails(WeaponBayDetails v) {
		if (weaponBayDetails == null || weaponBayDetails.length == 0) {
			weaponBayDetails = new WeaponBayDetails[1];
			weaponBayDetails[0] = v;
		} else {
			WeaponBayDetails [] newArray = new WeaponBayDetails[weaponBayDetails.length+1];
			for (int i = 0; i < weaponBayDetails.length; i++) {
				newArray[i] = weaponBayDetails[i];
			}
			newArray[weaponBayDetails.length] = v;
			weaponBayDetails = newArray;
		}
	}

	public WeaponBayDetails getBayWithBestWeaponFireAngle() {
		if (weaponBayDetails == null || weaponBayDetails.length == 0) {
			return null;
		} else {
			WeaponBayDetails best = weaponBayDetails[0];
			for (int i = 1; i < weaponBayDetails.length; i++) {
				if (weaponBayDetails[i].getMaxFireAngle() > best.getMaxFireAngle()) {
					best = weaponBayDetails[i];
				}
			}
			return best;
		}
	}

	public WeaponBayDetails getWeaponBayDetails(int index) {
		if (weaponBayDetails == null || weaponBayDetails.length == 0) {
			return null;
		}
		if (index >= weaponBayDetails.length) {			
			return weaponBayDetails[weaponBayDetails.length-1];
		} else {
			return weaponBayDetails[index];
		}
	}

	public boolean isDestructonCanCauseWreckage() {
		return destructonCanCauseWreckage;
	}

	public void setDestructonCanCauseWreckage(boolean v) {
		this.destructonCanCauseWreckage = v;
	}


}