/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Hulls extends TechnologyList {

    private static final String ELEMENT_NAME = "hulls";
	private transient Hull [] craftHulls;
    private transient Hull [] stationHulls;
    
    private transient Hull bestFighter;
    private transient Hull biggestHull;

    public Hulls() {

    }

    public Hulls(Hulls o) throws ApplicationError {
        super(o);
    }

    public Hulls(Hull w) {
        add(w);
    }

    public Hulls(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getHull(names[i]) );
            }
        }
    }

	public Hulls(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Hull.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setHull(new Hull(n[i]));
			}
		}
	}

	protected void changed() {
        craftHulls = null;
        stationHulls = null;
	}

	public void setHull(String v) throws ApplicationError {
		add( Technologies.getHulls().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_hulls.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

	public Hull getBiggestCraftHull() {
        Collection list = getAll(true);
        Hull best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isCraft()) {
	                if (best == null || best.getSpace() < hull.getSpace()) {
	                    best = hull;
	                } else if (best.getSpace() == hull.getSpace() && best.getCostToResearch() < hull.getCostToResearch()) {
	                    best = hull;
	                }
                }
            }
        }
        return best;
	}

	public Hull getFastestCraftHull() {
        Collection list = getAll(true);
        Hull best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isCraft()) {
	                if (best == null || best.getBattleMovementCost() > hull.getBattleMovementCost()) {
	                    best = hull;
	                }
                }
            }
        }
        return best;
	}

	public Hull getBiggestBattleStationHull() {
        Collection list = getAll(true);
        Hull best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isBattleStation() && (best == null || best.getSpace() < hull.getSpace())) {
                    best = hull;
                }
            }
        }
        return best;
	}

	public Hull getBiggestOutpostHull() {
        Collection list = getAll(true);
        Hull best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isOutpost() && (best == null || best.getSpace() < hull.getSpace())) {
                    best = hull;
                }
            }
        }
        return best;
	}

	public boolean hasDiscoveredStation() {
		return getBiggestSpaceStationHull() != null;
	}

	public Hull getBiggestSpaceStationHull() {
        Collection list = getAll(true);
        Hull best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isSpaceStation() && (best == null || best.getSpace() < hull.getSpace())) {
                    best = hull;
                }
            }
        }
        return best;
	}

	public Hull getBiggestSentinelHull() {
        Collection list = getAll(true);
        Hull best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isSentinel() && (best == null || best.getSpace() < hull.getSpace())) {
                    best = hull;
                }
            }
        }
        return best;
	}


    public Hull getHull(String name) {
        return (Hull)super.get(name);
    }

    public Hull getHullAtIndex(int index) {
        return (Hull)getTechnologyAtIndex(index);
    }

    public void setHull(Hull v) {
        add(v);
    }

    public Hull [] getCraftHulls() {
        if (craftHulls == null) {
            Collection list = getAll(true);
            if (list != null) {
                ArrayList results = null;
                Iterator i = list.iterator();
                while (i.hasNext()) {
                    Hull hull = (Hull)i.next();
                    if (hull.isCraft()) {
                        if (results == null) {
                            results = new ArrayList();
                        }
                        results.add(hull);
                    }
                }
                if (results != null) {
                    craftHulls = (Hull[])results.toArray(new Hull[(results.size())]);
                }
            }
        }
        return craftHulls;
    }
    
    
    public Hull [] getStationHulls() {
        if (stationHulls == null) {
            Collection list = getAll(true);
            if (list != null) {
                ArrayList results = null;
                Iterator i = list.iterator();
                while (i.hasNext()) {
                    Hull hull = (Hull)i.next();
                    if (hull.isStation()) {
                        if (results == null) {
                            results = new ArrayList();
                        }
                        results.add(hull);
                    }
                }
                if (results != null) {
                    stationHulls = (Hull[])results.toArray(new Hull[(results.size())]);
                }
            }
        }
        return stationHulls;
    }
    
    public Hull [] getCurrentStationHulls() {
    	Hull [] r = null;
        Collection list = getAll(true);
        if (list != null) {
            ArrayList results = null;
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isStation()) {
                    if (results == null) {
                        results = new ArrayList();
                    }
                    results.add(hull);
                }
            }
            if (results != null) {
                r = (Hull[])results.toArray(new Hull[(results.size())]);
            }
        }
        return r;
    }
    
    
    public Hull getMostAdvancedBattleStationHull() {
    	Hull r = null;
        Collection list = getAll(true);
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isBattleStation()) {
                	if (r == null || hull.getCostToResearch() > r.getCostToResearch()) {
                		r = hull;
                	}
                }
            }
        }
        return r;
    }

    public Hull getMostAdvancedSpaceStationHull() {
    	Hull r = null;
        Collection list = getAll(true);
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isSpaceStation()) {
                	if (r == null || hull.getCostToResearch() > r.getCostToResearch()) {
                		r = hull;
                	}
                }
            }
        }
        return r;
    }
    
    public Hull getMostAdvancedOutpostHull() {
    	Hull r = null;
        Collection list = getAll(true);
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Hull hull = (Hull)i.next();
                if (hull.isOutpost()) {
                	if (r == null || hull.getCostToResearch() > r.getCostToResearch()) {
                		r = hull;
                	}
                }
            }
        }
        return r;
    }
    

    
    /**
     * @return The fighter hull with the largest hit points.
     */
    public Hull getBestFighterHull() {
        if (bestFighter == null) {
            Collection list = getAll(true);
            if (list != null) {
                Iterator i = list.iterator();
                while (i.hasNext()) {
                    Hull hull = (Hull)i.next();
                    if (hull.isFighter()) {
                        if (bestFighter == null) {
                            bestFighter = hull;
                        } else if (hull.getMaxDamage() > bestFighter.getMaxDamage()) {
                            bestFighter = hull;
                        }
                    }
                }
            }
        }
        return bestFighter;
    }
    
    /**
     * @return The smallest hull in this list that has space for this angine and this special.
     */
    public Hull getSmallestCraftHullWithSpaceFor(Engine engine, Technology special) {
        Hull [] hulls = getCraftHulls();
        Hull best = null;
        for (int i=0; i<hulls.length; i++) {
            int space = hulls[i].getRemainingSpaceWith(engine);
            if (space >= special.getSpace()) {
                if (best == null || hulls[i].getSpace() < best.getSpace()) {
                    best = hulls[i];
                }
            }
        }
        return best;
    }



}