/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Humanities extends TechnologyList {

    private static final String ELEMENT_NAME = "humanities";
    
	private transient Integer maxTradeRouteDistance;
	private transient Integer maxSolarSystemTradeRoutes;
	private transient Double tradeRouteBonus;

    public Humanities() {

    }

    public Humanities(Humanities o) throws ApplicationError {
        super(o);
    }

    public Humanities(Humanity w) {
        add(w);
    }

    public Humanities(Humanity... humanities) {
		if (humanities != null) {
			for (int i = 0; i < humanities.length; i++) {
				add(humanities[i]);
			}
		}
	}

    public Humanities(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getHumanity(names[i]) );
            }
        }
    }

    public Humanities(ArrayList<String> names) throws ApplicationError {
        if (names != null) {
        	int size = names.size();
            for (int i=0; i<size; i++) {
                add( Technologies.getHumanity(names.get(i)) );
            }
        }
    }

	public Humanities(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Humanity.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setHumanity(new Humanity(n[i]));
			}
		}
	}

	private transient Humanity [] humanities = null;

    /**
     * @return The shield in the list
     */
    public Humanity [] getHumanities() {
        if (humanities == null) {
			Collection list = getAll(true);
			if (list == null) return null;
			humanities = new Humanity[list.size()];
			Iterator i = list.iterator();
            int index = 0;
			while (i.hasNext()) {
				Humanity v = (Humanity)i.next();
                humanities[index++] = v;
			}
		}
        return humanities;
	}

    public void setHumanity(String v) throws ApplicationError {
        add( Technologies.getHumanities().get(v) );
	}

    /**
     * @return The total population for all modules in this list.
     */
    public int getMaxSolarSystemTradeRouteDistance() {
        if (maxTradeRouteDistance != null) {
            return maxTradeRouteDistance.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Humanity v = (Humanity)i.next();
            if (v.getMaxTradeRouteDistance() != null) {
                result += v.getMaxTradeRouteDistance().intValue();
            }
		}
		maxTradeRouteDistance = new Integer(result);
        return result;
	}

    public int getMaxSolarSystemTradeRoutes() {
        if (maxSolarSystemTradeRoutes != null) {
            return maxSolarSystemTradeRoutes.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Humanity v = (Humanity)i.next();
            if (v.getMaxSolarSystemTradeRoutes() != null) {
                result += v.getMaxSolarSystemTradeRoutes().intValue();
            }
		}
		maxSolarSystemTradeRoutes = new Integer(result);
        return result;
	}

    public double getTradeRouteBonus() {
        if (tradeRouteBonus != null) {
            return tradeRouteBonus.doubleValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		double result = 0.0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Humanity v = (Humanity)i.next();
            if (v.getTradeRouteBonus() != null) {
                result += v.getTradeRouteBonus().doubleValue();
            }
		}
		tradeRouteBonus = new Double(result);
        return result;
	}

	protected void changed() {
		maxTradeRouteDistance = null;
		maxSolarSystemTradeRoutes = null;
		tradeRouteBonus = null;
	}
    
    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_humanities.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

    protected String getElementName() { return ELEMENT_NAME; }

    public Humanity getHumanity(String name) {
        return (Humanity)super.get(name);
    }

    public void setHumanity(Humanity v) {
        add(v);
    }

}