/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * @author Paul Davis
 *
 */
public class Humanity extends Technology {

    public static final String ELEMENT_NAME = "humanity";

	/**
     * The maximum distance a trade of route between a players planets.
     * Until a player has discovered a social science with a maximum distance 
     * a player cannot create any trade routes between their planets.
     */
    private Integer maxTradeRouteDistance;
    
	/**
     * The maximum quantity of additional trade routes that may be supported per solar system.
     */
    private Integer maxSolarSystemTradeRoutes;
    
    /**
     * Bonus to industrial output on planets using the trade route.
     * A value of 0.1 if a 10% increase in output
     */
    private Double tradeRouteBonus;

    public Humanity() {
    }

    public Humanity(DomNode node) {
		super(node);
		maxTradeRouteDistance = node.getNodeValueByName_Integer("max-trade-route-distance");
		maxSolarSystemTradeRoutes = node.getNodeValueByName_Integer("max-solar-system-trade-routes");
	}

	public String getDescription() {
        String description = super.getDescription();
        if (description != null) {
            return description;
        } else {
            String r = "Humanity "+getName();
            if (maxTradeRouteDistance != null)  r += " permits trade route between planets upto a distance of "+maxTradeRouteDistance;
            if (tradeRouteBonus != null)  r += " Increasesindustrial bonus of trade routes by "+tradeRouteBonus;
            return r;
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", getDescription());
		writer.addElement("image", getImage());
		writer.addElement("max-trade-route-distance", maxTradeRouteDistance);
		writer.addElement("max-solar-system-trade-routes", maxSolarSystemTradeRoutes);
		writer.addElement("trade-route-bonus", tradeRouteBonus);
        writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("group", getGroup());
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[10];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("image", super.getImage());
		r[3] = new NameValuePair("max-trade-route-distance", maxTradeRouteDistance);
		r[4] = new NameValuePair("max-solar-system-trade-routes", maxSolarSystemTradeRoutes);
		r[5] = new NameValuePair("trade-route-bonus", tradeRouteBonus);
        r[6] = new NameValuePair("cost to build", getCostToBuild());
		r[7] = new NameValuePair("cost to research", getCostToResearch());
		r[8] = new NameValuePair("maintenance cost", getMaintenanceCost());
		r[9] = new NameValuePair("group", getGroup());
		return r;
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_HUMANITIES;
    }

    public void setMaxTradeRouteDistance(Integer maxTradeRouteDistance) {
        this.maxTradeRouteDistance = maxTradeRouteDistance;
    }

    public Integer getMaxTradeRouteDistance() {
        return maxTradeRouteDistance;
    }

	public Double getTradeRouteBonus() {
		return tradeRouteBonus;
	}

	public void setTradeRouteBonus(Double v) {
		this.tradeRouteBonus = v;
	}

	public Integer getMaxSolarSystemTradeRoutes() {
		return maxSolarSystemTradeRoutes;
	}

	public void setMaxSolarSystemTradeRoutes(Integer maxSolarSystemTradeRoutes) {
		this.maxSolarSystemTradeRoutes = maxSolarSystemTradeRoutes;
	}

}