/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.Collection;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Laboratories extends TechnologyList {

	public static String SALVAGE_ANALYSER = "Salvage Analyser";

    private static final String ELEMENT_NAME = "laboratories";
	/**
     * Contains the laboratory that produces the most research in this list of technologies.
     */
    private transient Integer production = null;
    private transient Double chanceToDiscoverSalvagedTechBonus = null;

    public Laboratories() {
        
    }

    public Laboratories(Laboratory w) {
        add(w);
    }

    public Laboratories(Laboratories o) throws ApplicationError {
        super(o);
    }

    public Laboratories(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getLaboratory(names[i]) );
            }
        }
    }

    public Laboratories(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Laboratory.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setLaboratory(new Laboratory(n[i]));
			}
		}
	}

	public int getProduction() {
		if (production == null) {
			Collection list = getAll(true);
			if (list == null) {
				production = new Integer(0);
			} else {
				Iterator i = list.iterator();
		        int result = 0;
				while (i.hasNext()) {
					Laboratory v = (Laboratory)i.next();
		            result = result + v.getProduction();
				}
				production = new Integer(result);
			}
		}
		return production.intValue();
    }
    
    public double getChanceToDiscoverSalvagedTechBonus() {
		if (chanceToDiscoverSalvagedTechBonus == null) {
			Collection list = getAll(true);
			if (list == null) {
				chanceToDiscoverSalvagedTechBonus = new Double(0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Laboratory v = (Laboratory)i.next();
					if (v.getChanceToDiscoverSalvagedTechBonus() != null) {
						result = result + v.getChanceToDiscoverSalvagedTechBonus().doubleValue();
					}
				}
				chanceToDiscoverSalvagedTechBonus = new Double(result);
			}
		}
		return chanceToDiscoverSalvagedTechBonus.doubleValue();
    }
	
	protected void changed() {
        production = null;
        chanceToDiscoverSalvagedTechBonus = null;
	}

	public void setLaboratory(String v) throws ApplicationError {
		add( Technologies.getLaboratories().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_labs.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public Laboratory getLaboratory(String name) {
        return (Laboratory)super.get(name);
    }

    public void setLaboratory(Laboratory v) {
        add(v);
    }

    public Laboratory getLaboratoryAtIndex(int index) {
        return (Laboratory)getTechnologyAtIndex(index);
    }

}