/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.xml.DomNode;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * 
 * The technology that owns this restriction is not available for research until 
 * such a time as all technologies costing less than minResearchCost have been discovered.
 */
public class MinCostRestriction extends XMLBean implements Restriction {

    public static final String ELEMENT_NAME = "min-research-cost-restriction";
    
	/**
     * The minimum research cost of any technology available for research for
     * this technology to be available for research.
     */
    private int minCost;

    public MinCostRestriction() {
    }

    public MinCostRestriction(int minCost) {
		this.minCost = minCost;
    }

    public MinCostRestriction(DomNode node) {
    	minCost = node.getNodeValueByName_Int("min-cost");
	}

	/**
     * Removes tech from the available list if the available list has technologies that cost 
     * less than minCost to research.
     * @param tech The technology whom this restriction applies.
     * @param available The list of technologies that are available for research to this player.
     * @param player The player who wants to discover a technology from the available list.
     */
    public void apply(Technology tech, TechnologyList list, TechnologyOwner player) throws ApplicationError {
        Technology[] techs = list.getAllAsArray(true);
        if (techs != null) {
            for (int i=0; i<techs.length; i++) {
                if (techs[i].getCostToResearch() < minCost) {
                    // Must make sure not to exclude this tech because of another tech which 
                    // is going to be excluded.
                    Restriction subResctriction = techs[i].getRestriction();
                    if (subResctriction != null) {
                        subResctriction.apply(techs[i], list, player);
                        if (list.contains(techs[i])) {
                            list.remove(tech);
                        }
                    } else {
                        list.remove(tech);
                    }
                    return;
                }
            }
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("min-cost", minCost);
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public void setMinCost(int minResearchCost) {
        this.minCost = minResearchCost;
    }

    public int getMinCost() {
        return minCost;
    }
}