/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class MiscTechnologies extends TechnologyList {

    private static final String ELEMENT_NAME = "misc-technologies";

	public MiscTechnologies() {
    }

    public MiscTechnologies(MiscTechnology w) {
        add(w);
    }

    public MiscTechnologies(MiscTechnologies o) throws ApplicationError {
        super(o);
    }

    public MiscTechnologies(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getMiscTechnology(names[i]) );
            }
        }
    }

	public MiscTechnologies(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
    	DomNode[] n = root.getAllNodes();
    	if (n != null) {
    		for (int i = 0; i < n.length; i++) {				
    			setMiscTechnology(MiscTechnology.create(n[i]));
			}
    	}
	}

	public MiscTechnology [] getMiscTechnology(Class techClass) throws ApplicationError {
		Technology[] techs = getAllAsArray();
    	if (techs == null) return null;
    	ArrayList r = null;
    	for (int i=0; i<techs.length; i++) {
    		if (techs[i].getClass() == techClass) {
    			if (r == null) {
    				r = new ArrayList();
    			}
   				r.add(techs[i]);
			}
    	}
    	if (r == null) {
    		return null;
    	} else {
    		return (MiscTechnology []) r.toArray(new MiscTechnology[r.size()]);
    	}
	}

	protected void changed() {
	}

    public void setMiscTechnology(String v) throws ApplicationError {
        add( Technologies.getMiscTechnologies().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_misc.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public MiscTechnology getMiscTechnology(String name) {
        return (MiscTechnology)super.get(name);
    }

    public void setMiscTechnology(MiscTechnology v) {
        add(v);
    }
    
    public void setMineField(MineField v) {
        add(v);
    }

    public void setStationMissileBase(StationMissileBase v) {
        add(v);
    }

    public void setGalacticLore(GalacticLore v) {
        add(v);
    }

    public void setNewsSystem(NewsSystem v) {
        add(v);
    }

    public void setKamikaze(Kamikaze v) {
        add(v);
    }

    public void setAsteroidFieldConstruction(AsteroidFieldConstruction v) {
        add(v);
    }

    public MiscTechnology getMiscTechnologyAtIndex(int index) {
        return (MiscTechnology)getTechnologyAtIndex(index);
    }
    
    public void setPlanetConstruction(PlanetConstruction v) {
    	add(v);
    }
    
    public void setExtendedSpaceJump(ExtendedSpaceJump v) {
    	add(v);
    }
    
    public MineField getMineField() {
    	Technology [] techs = getAllAsArray();
    	if (techs == null) return null;
    	for (int i=0; i<techs.length; i++) {
    		if (techs[i] instanceof MineField) {
    			MineField miscTech = (MineField) techs[i];
				return miscTech;
			}
    	}
    	return null;
    }
    
    public double getInfluence() {
    	Technology [] techs = getAllAsArray();
    	if (techs == null) return 0.0;
    	for (int i=0; i<techs.length; i++) {
    		if (techs[i] instanceof NewsSystem) {
    			NewsSystem miscTech = (NewsSystem) techs[i];
				return miscTech.getInfluence();
			}
    	}
    	return 0.0;
    }

    public double getSuicidePercent() {
    	Technology [] techs = getAllAsArray();
    	if (techs == null) return 0.0;
    	for (int i=0; i<techs.length; i++) {
    		if (techs[i] instanceof Kamikaze) {
    			Kamikaze miscTech = (Kamikaze) techs[i];
				return miscTech.getSuicidePercent();
			}
    	}
    	return 0.0;
    }

    public GalacticLore getGalacticLoreTechnology() {
    	Technology [] techs = getAllAsArray();
    	if (techs == null) return null;
    	for (int i=0; i<techs.length; i++) {
    		if (techs[i] instanceof GalacticLore) {
    			GalacticLore miscTech = (GalacticLore) techs[i];
				return miscTech;
			}
    	}
    	return null;
    }

    public Kamikaze getKamikaziTechnology() {
    	Technology [] techs = getAllAsArray();
    	if (techs == null) return null;
    	for (int i=0; i<techs.length; i++) {
    		if (techs[i] instanceof Kamikaze) {
    			Kamikaze miscTech = (Kamikaze) techs[i];
				return miscTech;
			}
    	}
    	return null;
    }
    
    public ExtendedSpaceJump getExtendedSpaceJump() {
    	Technology [] techs = getAllAsArray();
    	if (techs == null) return null;
    	for (int i=0; i<techs.length; i++) {
    		if (techs[i] instanceof ExtendedSpaceJump) {
    			ExtendedSpaceJump miscTech = (ExtendedSpaceJump) techs[i];
				return miscTech;
			}
    	}
    	return null;
    }

}