/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * 
 * @author Paul Davis
 *
 */
public class Module extends Technology {

    public static final String ELEMENT_NAME = "module";

	/**
     * This is the quantity of people that this module provides living quarters for.
     * A population of 10 would increase the maximum population of the planet that
     * controls this module by 10.
     * Note This is NOT modified by the bonuses of the planet it is in orbit around.
     * The orbit may or may not be the planet that controls this orbital.
     */
    private Integer population;

    /**
     * This is the amount of production this module adds the the production output of
     * the planet that controls it.
     * Note This is modified by the bonuses of the planet it is in orbit around.
     * The orbit may or may not be the planet that controls this orbital.
     */
    private Integer production;

    /**
     * This is the amount of research this module adds the the production output of
     * the planet that controls it.
     * Note This is modified by the bonuses of the planet it is in orbit around.
     * The orbit may or may not be the planet that controls this orbital.
     */
    private Integer research;

    /**
     * This is the quantity of fighter bays this module can deploy in a battle when attacked.
     */
    private Integer qtyFighterBays;

    /**
     * This is the quantity of defensive buoys deploy in a battle to defend the space station.
     */
    private Integer qtySentinels;
    
    /**
     * Some modules may be restricted to particular types of stellar object.
     * Must be one of the values;
     * STELLAR_OBJECT_TYPE_SOLAR_SYSTEM
     * STELLAR_OBJECT_TYPE_BLACK_HOLE
     * STELLAR_OBJECT_TYPE_WORM_HOLE
     * A value of null means can be placed on anything.
     */
    private String StellarObjectTypeRestriction;
    
    public static final String STELLAR_OBJECT_TYPE_SOLAR_SYSTEM = "SolarSystem";
    public static final String STELLAR_OBJECT_TYPE_BLACK_HOLE = "BlackHole";
    public static final String STELLAR_OBJECT_TYPE_WORM_HOLE = "WormHole";

    private boolean onSpaceStation = false;
    private boolean onBattleStation = false;
    private boolean onOutpost = false;

    public Module() {
    }

    public Module(DomNode node) {
		super(node);
		population = node.getNodeValueByName_Integer("population");
		production = node.getNodeValueByName_Integer("production");
		research = node.getNodeValueByName_Integer("research");
		qtyFighterBays = node.getNodeValueByName_Integer("qty-fighter-bays");
		qtySentinels = node.getNodeValueByName_Integer("qty-sentinels");
		onSpaceStation = node.getNodeValueByName_Bool("on-space-station");
		onBattleStation = node.getNodeValueByName_Bool("on-battle-station");
		onOutpost = node.getNodeValueByName_Bool("on-outpost");
	}

	public String getDescription() {
        String description = super.getDescription();
        if (description != null) {
            return description;
        } else {
            String r = "Module "+getName();
            if (population != null)  r += " increases maximum population by "+population;
            if (production != null)  r += " increases production by "+production;
            if (research != null)  r += " increases research by "+research;
            if (qtyFighterBays != null)  r += " deploys "+qtyFighterBays+" fighter bays to defend the station";
            if (qtySentinels != null)  r += " deploys "+qtySentinels+" buoys to defend the station";
            return r;
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", getDescription());
		writer.addElement("image", getImage());
		writer.addElement("population", population);
		writer.addElement("production", production);
		writer.addElement("research", research);
		writer.addElement("qty-fighter-bays", qtyFighterBays);
		writer.addElement("qty-sentinels", qtySentinels);
		writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("group", getGroup());
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[13];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("image", super.getImage());
		r[3] = new NameValuePair("population", population);
		r[4] = new NameValuePair("production", production);
		r[5] = new NameValuePair("research", research);
		r[6] = new NameValuePair("qty fighter bays", qtyFighterBays);
		r[7] = new NameValuePair("qty sentinels", qtySentinels);
		r[8] = new NameValuePair("space", getSpace());
        r[9] = new NameValuePair("cost to build", getCostToBuild());
		r[10] = new NameValuePair("cost to research", getCostToResearch());
		r[11] = new NameValuePair("maintenance cost", getMaintenanceCost());
		r[12] = new NameValuePair("group", getGroup());
		return r;
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_MODULE;
    }


    public void setPopulation(Integer population) {
        this.population = population;
    }


    public Integer getPopulation() {
        return population;
    }


    public void setProduction(Integer production) {
        this.production = production;
    }


    public Integer getProduction() {
        return production;
    }


    public void setResearch(Integer research) {
        this.research = research;
    }


    public Integer getResearch() {
        return research;
    }

	public Integer getQtyFighterBays() {
		return qtyFighterBays;
	}

	public void setQtyFighterBays(Integer v) {
		this.qtyFighterBays = v;
	}

	public Integer getQtySentinels() {
		return qtySentinels;
	}

	public void setQtySentinels(Integer v) {
		this.qtySentinels = v;
	}

	public boolean isOnSpaceStation() {
		return onSpaceStation;
	}

	public void setOnSpaceStation(boolean onSpaceStation) {
		this.onSpaceStation = onSpaceStation;
	}

	public boolean isOnBattleStation() {
		return onBattleStation;
	}

	public void setOnBattleStation(boolean onBattleStation) {
		this.onBattleStation = onBattleStation;
	}

	public boolean isOnOutpost() {
		return onOutpost;
	}

	public void setOnOutpost(boolean onOutpost) {
		this.onOutpost = onOutpost;
	}

	public String getStellarObjectTypeRestriction() {
		return StellarObjectTypeRestriction;
	}

	public void setStellarObjectTypeRestriction(String stellarObjectTypeRestriction) {
		StellarObjectTypeRestriction = stellarObjectTypeRestriction;
	}

}