/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Modules extends TechnologyList {

    private static final String ELEMENT_NAME = "modules";
	private transient Integer population;
    private transient Integer production;
    private transient Integer research;

    public Modules() {

    }

    public Modules(Modules o) throws ApplicationError {
        super(o);
    }

    public Modules(Module w) {
        add(w);
    }

    public Modules(Module... modules) {
		if (modules != null) {
			for (int i = 0; i < modules.length; i++) {
				add(modules[i]);
			}
		}
	}

    public Modules(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getModule(names[i]) );
            }
        }
    }

    public Modules(ArrayList<String> names) throws ApplicationError {
        if (names != null) {
        	int size = names.size();
            for (int i=0; i<size; i++) {
                add( Technologies.getModule(names.get(i)) );
            }
        }
    }

	public Modules(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Module.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setModule(new Module(n[i]));
			}
		}
	}

	private transient Module [] modules = null;

    /**
     * @return The shield in the list
     */
    public Module [] getModules() {
        if (modules == null) {
			Collection list = getAll(true);
			if (list == null) return null;
            modules = new Module[list.size()];
			Iterator i = list.iterator();
            int index = 0;
			while (i.hasNext()) {
                Module v = (Module)i.next();
                modules[index++] = v;
			}
		}
        return modules;
	}

    public void setModule(String v) throws ApplicationError {
        add( Technologies.getModules().get(v) );
	}

    /**
     * @return The total population for all modules in this list.
     */
    public int getPopulation() {
        if (population != null) {
            return population.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
            Module v = (Module)i.next();
            if (v.getPopulation() != null) {
                result += v.getPopulation().intValue();
            }
		}
        population = new Integer(result);
        return result;
	}

    /**
     * @return The total production for all modules in this list.
     */
    public int getProduction() {
        if (production != null) {
            return production.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
            Module v = (Module)i.next();
            if (v.getProduction() != null) {
                result += v.getProduction().intValue();
            }
		}
        production = new Integer(result);
        return result;
	}

    /**
     * @return The total population for all modules in this list.
     */
    public int getResearch() {
        if (research != null) {
            return research.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
            Module v = (Module)i.next();
            if (v.getResearch() != null) {
                result += v.getResearch().intValue();
            }
		}
        research = new Integer(result);
        return result;
	}

	protected void changed() {
        population = null;
        production = null;
        research = null;
        modules = null;
	}
    
    public String getBonusDescription() {
        return "Population +"+getPopulation()+" Production +"+getProduction()+" Research +"+getResearch();
    }

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_modules.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

    protected String getElementName() { return ELEMENT_NAME; }

    public Module getModule(String name) {
        return (Module)super.get(name);
    }

    public void setModule(Module v) {
        add(v);
    }

    public Module getModuleAtIndex(int index) {
        return (Module)getTechnologyAtIndex(index);
    }

	public int getQtyFighterBays() {
		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
            Module v = (Module)i.next();
            if (v.getQtyFighterBays() != null) {
                result += v.getQtyFighterBays().intValue();
            }
		}
		return result;
	}

	public int getQtySentinels() {
		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
            Module v = (Module)i.next();
            if (v.getQtySentinels() != null) {
                result += v.getQtySentinels().intValue();
            }
		}
		return result;
	}

	public Module getBestSentinels() {
		Module[] modules = getModules();
		Module best = null;
        for (int i=0; modules != null && i<modules.length; i++) {
            if (modules[i].getQtySentinels() != null && modules[i].getQtySentinels() > 0) {
				if (best == null || best.getQtySentinels() <= modules[i].getQtySentinels()) {
                	best = modules[i];
                }
            }
        }
        return best;
	}


}