/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * @author Paul Davis
 *
 */
public class Scanner extends Technology {

	public static final String ELEMENT_NAME = "scanner";
	public static final String GROUP_SCANNER = "Scanner";

	/**
	 * This is the strength of the scanner.
	 * Crafts are visible if the distance from the closest scanner is less than the strength. The 
     * cloak level for this craft is subtracted from the scan strength.
     *          VISIBLE IF : distance < (scan strength - cloak level).
	 */
	private int strength;

	public Scanner() {
	}
	
	public Scanner(DomNode node) {
		super(node);
		strength = node.getNodeValueByName_Integer("strength");
	}
	
	public Scanner(String name, int strength) {
		setName(name);
		setStrength(strength);
		setGroup("Scanner");
	}
	
	public int getStrength() { return strength; }
	public void setStrength(int v) { strength = v; }

    public String getDescription() {
        String description = super.getDescription();
        if (description != null) {
            return description;
        } else {
            return "Scanner "+getName()+" has a scan strength of "+strength;
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", super.getDescription());
		writer.addElement("strength", strength);
		writer.addElement("space", getSpace());
        writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("maintenance-cost", getMaintenanceCost());
		writer.addElement("group", getGroup());
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[9];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("image", super.getImage());
		r[3] = new NameValuePair("strength", strength);
		r[4] = new NameValuePair("space", getSpace());
        r[5] = new NameValuePair("cost to build", getCostToBuild());
		r[6] = new NameValuePair("cost to research", getCostToResearch());
		r[7] = new NameValuePair("maintenance cost", getMaintenanceCost());
		r[8] = new NameValuePair("group", getGroup());
		return r;
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_SCANNER;
    }
    
    /**
     * @return True iff this scanner is as good or better than the scanner s passed in.
     */
	public boolean isAsGood(Scanner s) {
		if (s == null) return true;
		return (strength >= s.getStrength());
	}

}