/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import mot.common.bean.NameList;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Scanners extends TechnologyList {

	public static final String ELEMENT_NAME = "scanners";
	private transient Integer strength;

    public Scanners() {
        
    }

    public Scanners(Scanners o) throws ApplicationError {
        super(o);
    }

    public Scanners(Scanner w) {
        add(w);
    }

    public Scanners(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getScanner(names[i]) );
            }
        }
    }

    public Scanners(ArrayList<String> names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.size(); i++) {
                add( Technologies.getScanner(names.get(i)) );
            }
        }
    }

	public Scanners(NameList names) throws ApplicationError {
        if (names != null) {
        	String[] na = names.getNamesAsArray();
        	if (na != null) {
	            for (int i=0; i<na.length; i++) {
	                add( Technologies.getScanner(na[i]) );
	            }
        	}
        }
	}

	public Scanners(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Scanner.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setScanner(new Scanner(n[i]));
			}
		}
	}

	private transient Scanner [] scanners = null;

    /**
     * @return The shield in the list
     */
	public Scanner [] getScanners() {
		if (scanners == null) {
			Collection list = getAll(true);
			if (list == null) return null;
			scanners = new Scanner[list.size()];
			Iterator i = list.iterator();
            int index = 0;
			while (i.hasNext()) {
				Scanner v = (Scanner)i.next();
				scanners[index++] = v;
			}
		}
		return scanners;
	}

	public Scanner[] getCurrentScanners() {
        Collection l = getAll(false);
        if (l == null) return null;
        return (Scanner[])l.toArray(new Scanner[(l.size())]);
	}

	public void setScanner(String v) throws ApplicationError {
		add( Technologies.getScanners().get(v) );
	}

    /**
     * The scan strength of a list of scanners is defined as the scan strength of 
     * the most powerful scanner in the list.
     * @return The scan strength of this list of scanners.
     */
	public int getStrength() {
		if (strength != null) {
			return strength.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Scanner v = (Scanner)i.next();
            if (v.getStrength() > result) {
                result = v.getStrength();
            }
		}
		strength = new Integer(result);
		return result;
	}

    public int getStrength(int dist) {
        int v = getStrength() - dist;
        if (v < 0) v = 0;
        return v;
    }

	protected void changed() {
		strength = null;
        scanners = null;
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_scanners.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

	public Scanner getBestScanner() {
		return (Scanner)getBest();
	}

    public Scanner getScanner(String name) {
        return (Scanner)super.get(name);
    }

	public Scanners getScanners(String[] names) {
        if (names == null || names.length == 0) {
            return null;
        }
        Scanners r = new Scanners();
        for (int i = 0; i < names.length; i++) {
			r.add(getScanner(names[i]));
		}
		return r;
	}
	
    public void setScanner(Scanner v) {
        add(v);
    }
    
    public Scanner getScannerAtIndex(int index) {
        return (Scanner)getTechnologyAtIndex(index);
    }

	/**
	 * Calculates in the scanner s is out dated technology when compared with this list.
	 * @return True iff this list of scanners contains a scanner as good or better than the field passed in.
	 */
	public boolean containsAsGood(Scanner s) {
        if (getQuantity() == 0) {
            return false;
        }
        
        Scanner[] scanners = getScanners();
        for (int i=0; i<scanners.length; i++) {
            if (scanners[i].isAsGood(s)) {
                return true;
            }
        }
		return false;
	}

}