/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * @author Paul Davis
 *
 */
public class Shield extends Technology {

	public static final String ELEMENT_NAME = "shield";
	public static final String GROUP_PSI = "Psi";
	public static final String GROUP_SHIELD = "Shield";
	public static final String GROUP_CLOAK = "Cloak";

	public static final String HOLO_SHIELD = "FakeShield";
	
	/**
	 * The amount of damage blocked per weapon fire. So a weapon firing 10 points
	 * of damage at a ship with shield blocking 5 would cause 5 points of damage.
	 */
	private double damageBlocked;

	/**
	 * This is the maximum value that power can take.
     * This gets its value from power the first time power is set (at load time!).
	 */
	private double power;

	/**
	 * The power added to the shields every turn.
	 */
	private double rechargeRate;

	/**
	 * The percentage of an enemies psi power they block. A value of 0.25 will reduce
     * the psi attack by 25%.
	 */
	private double psiRating;

	/**
	 * The percentage of the damage that reduced the shields power. e.g. Damage 100 on shields with power 50
	 * and discharge rate 0.1 could mean power is reduced by 10% of the damage , i.e. 10.
	 * A discharge rate of 1 would mean the shield power does down by the amount of damage blocked.
	 * This is a very powerful setting has I low discharge rate makes shields difficult to penetrate.
	 * Most shields should have a value of 1 or close to it.
	 */
	private double dischargeRate;

	/**
	 * The amount these shields allow a ship to avoid detection by enemy scanners.
	 * Crafts are visible if the distance from the closest scanner is less than the scanner strength. The 
     * cloak level for this craft is subtracted from the scan strength. The cloak level for a craft is the
     * shield cloak level + the player character cloak level.
     *          VISIBLE IF : distance < (scan strength - cloak level).
     * NOTE - Cloak is a big advantage in battle. If cloak > 0 for a craft then you can only reliably fire
     * at it within you scanner range - losing you your planet weapons range bonus. So a cloak shield should
     * be BIG.
	 */
	private int cloakLevel;

	public Shield() {
	}

	public Shield(DomNode node) {
		super(node);
		damageBlocked = node.getNodeValueByName_Double("damage-blocked");
		power = node.getNodeValueByName_Double("power");
		rechargeRate = node.getNodeValueByName_Double("recharge-rate");
		dischargeRate = node.getNodeValueByName_Double("discharge-rate");
		psiRating = node.getNodeValueByName_Double("psi-rating");
		cloakLevel = node.getNodeValueByName_Int("cloak-level");
	}
	
	public double getDamageBlocked() { return damageBlocked; }
	public double getPower() { return power; }
	public double getRechargeRate() { return rechargeRate; }
	public double getPsiRating() { return psiRating; }
	public double getDischargeRate() { return dischargeRate; }
	public int getCloakLevel() { return cloakLevel; }

	public void setDamageBlocked(double v) { damageBlocked = v; }
	public void setPower(double v) { power = v; }
	public void setRechargeRate(double v) { rechargeRate = v; }
	public void setPsiRating(double v) { psiRating = v; }
	public void setDischargeRate(double v) { dischargeRate = v; }
	public void setCloakLevel(int v) { cloakLevel = v; }

    public String getDescription() {
        String description = super.getDescription();
        if (description != null) {
            return description;
        } else {
            String r = "Shield "+getName();
            if (power > 0.0) r += " has a maximum power of "+power;
            if (damageBlocked > 0.0) r += " blocks damage "+damageBlocked;
            if (damageBlocked > 0.0) r += " and recharges at "+rechargeRate+" per turn";
            if (cloakLevel > 0.0) r += " provides a cloak of "+cloakLevel;
            if (psiRating > 0.0) r += " reduces psi by "+psiRating*100+"%";
            return r;
        }
    }
    
	public boolean isNormal() { return Comp.same(getGroup(), GROUP_SHIELD); }
	public boolean isPsi() { return Comp.same(getGroup(), GROUP_PSI); }
	public boolean isCloak() { return Comp.same(getGroup(), GROUP_CLOAK); }

	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", super.getDescription());
		writer.addElement("image", getImage());
		writer.addElement("damage-blocked", damageBlocked, true);
		writer.addElement("power", power, true);
		writer.addElement("recharge-rate", rechargeRate, true);
		writer.addElement("discharge-rate", dischargeRate, true);
		writer.addElement("psi-rating", psiRating, true);
		writer.addElement("cloak-level", cloakLevel);
		writer.addElement("space", getSpace());
		writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("maintenance-cost", getMaintenanceCost());
		writer.addElement("group", getGroup());
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[14];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("image", super.getImage());
		r[3] = new NameValuePair("damage blocked", damageBlocked);
		r[4] = new NameValuePair("power", power);
		r[5] = new NameValuePair("recharge rate", rechargeRate);
		r[6] = new NameValuePair("discharge rate", dischargeRate);
		r[7] = new NameValuePair("psi rating", psiRating);
		r[8] = new NameValuePair("cloak level", cloakLevel);
		r[9] = new NameValuePair("space", getSpace());
        r[10] = new NameValuePair("cost to build", getCostToBuild());
		r[11] = new NameValuePair("cost to research", getCostToResearch());
		r[12] = new NameValuePair("maintenance cost", getMaintenanceCost());
		r[13] = new NameValuePair("group", getGroup());
		return r;
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_SHIELD;
    }
    
    /**
     * @return True iff this shield is as good or better than the shield s passed in.
     */
	public boolean isAsGood(Shield s) {
		if (s == null) return true;
		if (s.isCloak()) {
			return cloakLevel >= s.getCloakLevel();
		} else if (s.isPsi()) {
			return psiRating >= s.getPsiRating();
		} else {
			return damageBlocked >= s.getDamageBlocked();
		}
	}

}