/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import mot.common.bean.NameList;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Shields extends TechnologyList {

	private static final String ELEMENT_NAME = "shields";
	private transient Double damageBlocked = null;
	private transient Double power = null;
	private transient Double rechargeRate = null;
	private transient Integer cloakLevel = null;
	private transient Double psiRating = null;

    public Shields() {
        
    }

    public Shields(Shields o) throws ApplicationError {
        super(o);
    }

    public Shields(Shield w) {
        add(w);
    }
    
    public Shields(Shield [] w) throws ApplicationError {
        super(w);
    }
    
    public Shields(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getShield(names[i]) );
            }
        }
    }

    public Shields(ArrayList<String> names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.size(); i++) {
                add( Technologies.getShield(names.get(i)) );
            }
        }
    }

	public Shields(NameList names) throws ApplicationError {
        if (names != null) {
        	String[] na = names.getNamesAsArray();
        	if (na != null) {
	            for (int i=0; i<na.length; i++) {
	                add( Technologies.getShield(na[i]) );
	            }
        	}
        }
	}

	public Shields(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Shield.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setShield(new Shield(n[i]));
			}
		}
	}

	private transient Shield[] shields = null;

    /**
     * @return The shield in the list
     */
	public Shield[] getShields() {
		if (shields == null) {
			Collection list = getAll(true);
			if (list == null) return null;
			shields = new Shield[list.size()];
			Iterator i = list.iterator();
            int index = 0;
			while (i.hasNext()) {
				Shield v = (Shield)i.next();
				shields[index++] = v;
			}
		}
		return shields;
	}

	public double getDamageBlocked() {
		if (damageBlocked != null) {
			return damageBlocked.doubleValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		double result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Shield v = (Shield)i.next();
			result = result + v.getDamageBlocked();
		}
		damageBlocked = new Double(result);
		return result;
	}

    /**
     * Returns the sum power of all shields in this list.
     */
	public double getPower() {
		if (power != null) {
			return power.doubleValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		double result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Shield v = (Shield)i.next();
			result = result + v.getPower();
		}
		power = new Double(result);
		return result;
	}

	public double getRechargeRate() {
		if (rechargeRate != null) {
			return rechargeRate.doubleValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		double result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Shield v = (Shield)i.next();
			result = result + v.getRechargeRate();
		}
		rechargeRate = new Double(result);
		return result;
	}

	public int getCloakLevel() {
		if (cloakLevel != null) {
			return cloakLevel.intValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		int result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Shield v = (Shield)i.next();
			result = result + (int)v.getCloakLevel();
		}
		cloakLevel = new Integer(result);
		return result;
	}

	public double getPsiRating() {
		if (psiRating != null) {
			return psiRating.doubleValue();
		}

		Collection list = getAll(true);
		if (list == null) return 0;
		double result = 0;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Shield v = (Shield)i.next();
			result = result + v.getPsiRating();
		}
		if (result > 1.0) {
			result = 1.0;
		}
		psiRating = new Double(result);
		return result;
	}

	protected void changed() {
		damageBlocked = null;
		power = null;
		rechargeRate = null;
		psiRating = null;
        shields = null;
	}

    /**
     * @return The last shield in the list of technologies.
     */
	public Shield getBestShield() {
		return (Shield)getBest();
	}

    /**
     * @return The shield with the highest defense rating.
     * defense rating is measured as damage blocked * power.
     */
	public Shield getBestDefenseShield() {
        if (getQuantity() == 0) {
            return null;
        }
        
        Shield[] shields = getShields();
        double rating = shields[0].getDamageBlocked() * shields[0].getPower();
        int bestIndex = 0;
        for (int i=1; i<shields.length; i++) {
            double r = shields[i].getDamageBlocked() * shields[i].getPower();
            if (r > rating) {
                rating = r;
                bestIndex = i;
            }
        }
        return shields[bestIndex];
    }

	public Shield getBestCloakShield() {
        if (getQuantity() == 0) {
            return null;
        }
        
        Shield[] shields = getShields();
        int rating = shields[0].getCloakLevel();
        int bestIndex = 0;
        for (int i=1; i<shields.length; i++) {
        	int r = shields[i].getCloakLevel();
            if (r > rating) {
                rating = r;
                bestIndex = i;
            }
        }
        if (rating == 0) return null;
        return shields[bestIndex];
    }


	/**
	 * Calculates in the shield s is out dated technology when compared with this list.
	 * @return True iff this list of shields contains a shield as good or better than the field passed in.
	 */
	public boolean containsAsGood(Shield s) {
        if (getQuantity() == 0) {
            return false;
        }
        
        Shield[] shields = getShields();
        for (int i=0; i<shields.length; i++) {
            if (shields[i].isAsGood(s)) {
                return true;
            }
        }
		return false;
	}
	
    /**
     * @return The shield with the highest psi defence rating.
     */
	public Shield getBestPsiShield() {
        if (getQuantity() == 0) {
            return null;
        }
        
        Shield[] shields = getShields();
        double rating = 0;
        int bestIndex = 0;
        for (int i=0; i<shields.length; i++) {
            double r = shields[i].getPsiRating();
            if (r > 0 && r > rating) {
                rating = r;
                bestIndex = i;
            }
        }
        if (rating > 0) {
        	return shields[bestIndex];
        } else {
        	return null;
        }
    }

	public void setShield(String v) throws ApplicationError {
		add( Technologies.getShields().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_shields.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public Shield getShield(String name) {
        return (Shield)super.get(name);
    }
    
	public Shields getShields(String[] names) {
        if (names == null || names.length == 0) {
            return null;
        }
        Shields r = new Shields();
        for (int i = 0; i < names.length; i++) {
			r.add(getShield(names[i]));
		}
		return r;
	}

    public void setShield(Shield v) {
        add(v);
    }
    
    public Shield getShieldAtIndex(int index) {
        return (Shield)getTechnologyAtIndex(index);
    }

}