/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * @author Paul Davis
 *
 */
public class Special extends Technology {

	public static final String ELEMENT_NAME = "special";
	public static final String GROUP_HOLO_EMITER = "HoloEmiter";
	public static final String GROUP_INCINERATOR = "Incinerator";
	public static final String GROUP_SHIELD_SPAN = "ShieldSpan";
	public static final String GROUP_IMMUNITY = "Imunity";
	public static final String GROUP_MISSILE_GUIDANCE = "MissileGuidance";
	public static final String GROUP_FIGHTER_BAYS = "FighterBays";
	public static final String GROUP_REPAIR_UNIT = "RepairUnit";
	public static final String GROUP_COLONISER = "ColonyModule";
	public static final String GROUP_COMMAND_CENTER = "CommandCenter";
	

	public Special() {
		isCommandModule = null;
		speedBonus = null;
		battleTurningBonus = null;
		battleSpeedBonus = null;
		defenderModification = null;
		attackerModification = null;
        rangeModification = null;
		initiativeBonus = null;
        maxDamageBonus = null;
		canColoniseRating = null;
        qtyBerths = null;
        qtyFighters = null;
        qtyHoloCraft = null;
        influence = null;
        immunity = null;
        repairEffect = null;
        missileRetarget = null;
	}

    public Special(DomNode node) {
		super(node);
		
		speedBonus = node.getNodeValueByName_Double("speed-bonus");
		battleSpeedBonus = node.getNodeValueByName_Double("battle-speed-bonus");
		battleTurningBonus = node.getNodeValueByName_Double("battle-turning-bonus");
		defenderModification = node.getNodeValueByName_Double("defender-modification");
		attackerModification = node.getNodeValueByName_Double("attacker-modification");
		rangeModification = node.getNodeValueByName_Double("range-modification");
		initiativeBonus = node.getNodeValueByName_Double("initiative-bonus");
		maxDamageBonus = node.getNodeValueByName_Double("max-damage-bonus");
		canColoniseRating = node.getNodeValueByName_Double("can-colonise-rating");
		qtyBerths = node.getNodeValueByName_Integer("qty-berths");
		qtyFighters = node.getNodeValueByName_Integer("qty-fighters");
		qtyHoloCraft = node.getNodeValueByName_Integer("qty-holo-craft");
		influence = node.getNodeValueByName_Double("influence");
		salvageEffect = node.getNodeValueByName_Double("salvage-effect");
		repairEffect = node.getNodeValueByName_Double("repair-effect");
		missileRetarget = node.getNodeValueByName_Boolean("missile-retarget");
		fleetBattleSpeedBonus = node.getNodeValueByName_Double("fleet-battle-speed-bonus");
		fleetBattleTurningBonus = node.getNodeValueByName_Double("fleet-battle-turning-bonus");
		fleetDefenderModification = node.getNodeValueByName_Double("fleet-defender-modification");
		fleetAttackerModification = node.getNodeValueByName_Double("fleet-attacker-modification");
		fleetRangeModification = node.getNodeValueByName_Double("fleet-range-modification");
		fleetInitiativeBonus = node.getNodeValueByName_Double("fleet-initiative-bonus");
		
		setImmunity(node.getNodeValuesByName_String("immunity"));
		
		DomNode ssn = node.getNodeByName("shield-span");
		if (ssn != null) {			
			shieldSpan = new ShieldSpan(ssn);
		}
	}

	/**
     * If this is true then the bonuses provided by this special are FLEET WIDE (apply to the entire fleet).
     * If this is false then the bonuses provided by this special are for THIS CRAFT ONLY.
     * This is usually false (or null) implying the bonuses are intended for this craft only.
     */
    private Boolean isCommandModule;
	
    /**
     * A bonus in warp speed generated by this special. Warp speed is the speed in
     * the galaxy map. When on a craft in a fleet this special will effect the speed
     * of the whole fleet. The best speed bonuses of all the crafts in the fleet is used
     * to calculate the speed of a fleet.
     * Value 0 indicates no change
     */
    private Double speedBonus;

    /**
     * Bonus (%) to truning speed of craft in battle.
     * Value 0 indicates no change
     */
    private Double battleTurningBonus;

    /**
     * Bonus (%) to speed of craft in battle.
     * Value 0 indicates no change
     */
    private Double battleSpeedBonus;


    /**
     * The amount this special modifies damage on this craft.
     * The source craft source has fired on the target craft (these specials belong to the target
     * craft) inflicting damage amount of damage. The value is a percent change between 0 and 1.
     * Value 0 indicates no change
     *
     * The damage is reduced by a defender modification before the shields are hit so it also has
     * the effects of reducing damage to the shields.
     */
    private Double defenderModification;

    /**
     * The amount this special modifies damage from this crafts weapons.
     * The source craft source has fired on the target craft (these specials belong to the source
     * craft) inflicting damage amount of damage.  The value is a percent change between 0 and 1.
     * Value 0 indicates no change
     */
    private Double attackerModification;

    /**
     * The amount this special modifies range for this crafts weapons.
     * The craft with this rating has its range extended by this value. The value is a percent change
     * between 0 and 1.
     * Value 0 indicates no change
     */
    private Double rangeModification;

    /**
     * Bonus (%) to the initiative of a craft in battle.
     * Value 0 indicates no change
     */
    private Double initiativeBonus;

    /**
     * Bonus (%) added to the maximum damage a craft with can sustain before being destroyed.
     * Value 0 indicates no change
     */
    private Double maxDamageBonus;

    /**
     * This special allows the craft to colonise a planet of terraform level canColoniseRating
     * or higher. See Planet.getPlanetType() for details of what values this may take.
     * A value of NULL means this special cannot be used to colonise.
     */
    private Double canColoniseRating;

    /**
     * This is the qty of units of population that can be loaded onto this craft. This is used
     * for ground assault.
     */
    private Integer qtyBerths;

    /**
     * This is the qty of fighter crafts that may be stored in a hanger on the crafts this special
     * contains. These may be launched in a battle anf fight independently to the mother craft.
     */
    private Integer qtyFighters;

    /**
     * The quantity of Holo Crafts created by this craft when it enters a battle. A Holo Craft
     * is a fake craft used as a decoy. See HoloCraft for details.
     */
    private Integer qtyHoloCraft;
    
    /**
     * The influence this special produces. Influence in propaganda using a news service.
     * Influence is used to determine the morale of a craft / planet in a battle.
     */
    private Double influence;

    /**
     * When a craft is destroyed some of its technology its left in the wreckage and is 
     * available for salvage. This is the effect this special has on the amount of technology 
     * salvaged from its owning craft. A value of 1.0 implies no change, a value of 0.0 would mean
     * removes all technology.
     */
    private Double salvageEffect;
    
    /**
     * The shield span this special produces. The Shield span capability allows a craft to
     * transfer the shield recharge power not used on this craft to other craft.
     */
    private ShieldSpan shieldSpan;
    
    /**
     * A List of weapon groups that this special provides immunity too.
     */
    private List<String> immunity;

    /**
     * The percent of damage each turn that this special will repair.
     * Should be between 0 and 1 - where 0 means no repair and 1 means fully repaired.
     * This is used on automated repair units.
     */
    private Double repairEffect;
    
    /**
     * if this contains a value true then missiles shot by a craft containing this special will auto
     * retarget if their target is destroyed.
     */
    private Boolean missileRetarget;

    /**
     * BattleTurningBonus that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetBattleTurningBonus;

    /**
     * BattleSpeedBonus that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetBattleSpeedBonus;

    /**
     * AttackerModification that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetAttackerModification;

    /**
     * DefenderModification that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetDefenderModification;

    /**
     * RangeModification that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetRangeModification;

    /**
     * InitiativeBonus that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetInitiativeBonus;
    
    public Double getSpeedBonus() { return speedBonus; }
    public Double getBattleTurningBonus() { return battleTurningBonus; }
    public Double getBattleSpeedBonus() { return battleSpeedBonus; }
    public Double getDefenderModification() { return defenderModification; }
    public Double getAttackerModification() { return attackerModification; }
    public Double getRangeModification() { return rangeModification; }
    public Double getInitiativeBonus() { return initiativeBonus; }
	public Double getMaxDamageBonus() { return maxDamageBonus; }
	public Double getCanColoniseRating() { return canColoniseRating; }
	public Integer getQtyBerths() { return qtyBerths; }
    public Integer getQtyFighters() { return qtyFighters; }
    public Integer getQtyHoloCraft() { return qtyHoloCraft; }
	public Double getInfluence() { return influence; }

    public void setSpeedBonus(Double v) { speedBonus = v; }
    public void setBattleTurningBonus(Double v) { battleTurningBonus = v; }
    public void setBattleSpeedBonus(Double v) { battleSpeedBonus = v; }
    public void setDefenderModification(Double v) { defenderModification = v; }
    public void setAttackerModification(Double v) { attackerModification = v; }
    public void setRangeModification(Double v) { rangeModification = v; }
    public void setInitiativeBonus(Double v) { initiativeBonus = v; }
	public void setMaxDamageBonus(Double v) { maxDamageBonus = v; }
	public void setCanColoniseRating(Double v) { canColoniseRating = v; }
	public void setQtyBerths(Integer v) { qtyBerths = v; }
    public void setQtyFighters(Integer v) { qtyFighters = v; }
    public void setQtyHoloCraft(Integer v) { qtyHoloCraft = v; }
	public void setInfluence(Double v) { influence = v; }

	public void setSpeedBonus(String v) { speedBonus = new Double(v); }
	public void setBattleTurningBonus(String v) { battleTurningBonus = new Double(v); }
	public void setBattleSpeedBonus(String v) { battleSpeedBonus = new Double(v); }
	public void setDefenderModification(String v) { defenderModification = new Double(v); }
	public void setAttackerModification(String v) { attackerModification = new Double(v); }
	public void setRangeModification(String v) { rangeModification = new Double(v); }
	public void setInitiativeBonus(String v) { initiativeBonus = new Double(v); }
	public void setMaxDamageBonus(String v) { maxDamageBonus = new Double(v); }
	public void setCanColoniseRating(String v) { canColoniseRating = new Double(v); }
	public void setInfluence(String v) { influence = new Double(v); }



	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", super.getDescription());
		writer.addElement("image", getImage());
		writer.addElement("speed-bonus", speedBonus);
		writer.addElement("battle-speed-bonus", battleSpeedBonus);
		writer.addElement("battle-turning-bonus", battleTurningBonus);
		writer.addElement("defender-modification", defenderModification);
		writer.addElement("attacker-modification", attackerModification);
		writer.addElement("range-modification", rangeModification);
		writer.addElement("initiative-bonus", initiativeBonus);
		writer.addElement("max-damage-bonus", maxDamageBonus);
		writer.addElement("can-colonise-rating", canColoniseRating);
		writer.addElement("qty-berths", qtyBerths);
        writer.addElement("qty-fighters", qtyFighters);
        writer.addElement("qty-holo-craft", qtyHoloCraft);
        writer.addElement("influence", influence);
        writer.addElement("salvage-effect", salvageEffect);
        if (shieldSpan != null) {
        	shieldSpan.toXML(writer);
        }
        if (immunity != null) {
        	for (Iterator i = immunity.iterator(); i.hasNext();) {
		        writer.addElement("immunity", (String) i.next());
			}
        }
        writer.addElement("repair-effect", repairEffect);
        writer.addElement("missile-retarget", missileRetarget);
        writer.addElement("fleet-battle-speed-bonus", fleetBattleSpeedBonus);
        writer.addElement("fleet-battle-turning-bonus", fleetBattleTurningBonus);
        writer.addElement("fleet-defender-modification", fleetDefenderModification);
        writer.addElement("fleet-attacker-modification", fleetAttackerModification);
        writer.addElement("fleet-range-modification", fleetRangeModification);
        writer.addElement("fleet-initiative-bonus", fleetInitiativeBonus);
		writer.addElement("space", getSpace());
		writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("group", getGroup());
	}

	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[31];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("image", super.getImage());
		r[3] = new NameValuePair("speed-bonus", speedBonus);
		r[4] = new NameValuePair("battle-speed-bonus", battleSpeedBonus);
		r[5] = new NameValuePair("battle-turning-bonus", battleTurningBonus);
		r[6] = new NameValuePair("defender-modification", defenderModification);
		r[7] = new NameValuePair("attacker-modification", attackerModification);
		r[8] = new NameValuePair("range-modification", rangeModification);
		r[9] = new NameValuePair("initiative-bonus", initiativeBonus);
		r[10] = new NameValuePair("max-damage-bonus", maxDamageBonus);
		r[11] = new NameValuePair("can-colonise-rating", canColoniseRating);
		r[12] = new NameValuePair("qty-berths", qtyBerths);
        r[13] = new NameValuePair("qty-fighters", qtyFighters);
        r[14] = new NameValuePair("qty-holo-craft", qtyHoloCraft);
        r[15] = new NameValuePair("influence", influence);
        r[16] = new NameValuePair("salvage-effect", salvageEffect);
        r[17] = new NameValuePair("shield span", shieldSpan != null ? ""+shieldSpan.getRange() : "");
        r[18] = new NameValuePair("immunity", immunity != null ? ""+Comp.toString(immunity) : "");
        r[19] = new NameValuePair("repair-effect", repairEffect);
        r[20] = new NameValuePair("missile-retarget", missileRetarget);
        r[21] = new NameValuePair("fleet-battle-speed-bonus", fleetBattleSpeedBonus);
        r[22] = new NameValuePair("fleet-battle-turning-bonus", fleetBattleTurningBonus);
        r[23] = new NameValuePair("fleet-defender-modification", fleetDefenderModification);
        r[24] = new NameValuePair("fleet-attacker-modification", fleetAttackerModification);
        r[25] = new NameValuePair("fleet-range-modification", fleetRangeModification);
        r[26] = new NameValuePair("fleet-initiative-bonus", fleetInitiativeBonus);
		r[27] = new NameValuePair("space", getSpace());
        r[28] = new NameValuePair("cost to build", getCostToBuild());
		r[29] = new NameValuePair("cost to research", getCostToResearch());
		r[30] = new NameValuePair("group", getGroup());
		return r;
	}

	protected String getElementName() { return ELEMENT_NAME; }

    public String getTechType() {
        return Technologies.TECH_TYPE_SPECIAL;
    }
    
	public void setSalvageEffect(Double salvageEffect) {
		this.salvageEffect = salvageEffect;
	}
	
	public Double getSalvageEffect() {
		return salvageEffect;
	}
	
	public void setShieldSpan(ShieldSpan shieldSpan) {
		this.shieldSpan = shieldSpan;
	}
	
	public ShieldSpan getShieldSpan() {
		return shieldSpan;
	}
	
	public void setImunity(String weaponGroup) {
		if (immunity == null) immunity = new ArrayList<String>();
		immunity.add(weaponGroup);
	}
	
	public void setImmunity(String weaponGroup) {
		if (immunity == null) immunity = new ArrayList<String>();
		immunity.add(weaponGroup);
	}
	
	public void setImmunity(String [] weaponGroups) {
		if (weaponGroups != null) {			
			if (immunity == null) immunity = new ArrayList<String>();
			for (int i = 0; i < weaponGroups.length; i++) {				
				immunity.add(weaponGroups[i]);
			}
		} else {
			weaponGroups = null;
		}
	}
	
	public boolean isImmuneTo(String weaponGroup) {
		return immunity != null && immunity.contains(weaponGroup);
	}
	
	public void setIsCommandModule(boolean isCommandModule) {
		this.isCommandModule = isCommandModule;
	}
	
	public Boolean isCommandModule() {
		return isCommandModule;
	}
	
	public Double getRepairEffect() {
		return repairEffect;
	}
	
	public void setRepairEffect(Double repairEffect) {
		this.repairEffect = repairEffect;
	}
	
	public Boolean getMissileRetarget() {
		return missileRetarget;
	}
	
	public void setMissileRetarget(Boolean missileRetarget) {
		this.missileRetarget = missileRetarget;
	}
	public Double getFleetBattleSpeedBonus() {
		return fleetBattleSpeedBonus;
	}
	public void setFleetBattleSpeedBonus(Double fleetBattleSpeedBonus) {
		this.fleetBattleSpeedBonus = fleetBattleSpeedBonus;
	}
	public Double getFleetAttackerModification() {
		return fleetAttackerModification;
	}
	public void setFleetAttackerModification(Double fleetAttackerModification) {
		this.fleetAttackerModification = fleetAttackerModification;
	}
	public Double getFleetDefenderModification() {
		return fleetDefenderModification;
	}
	public void setFleetDefenderModification(Double fleetDefenderModification) {
		this.fleetDefenderModification = fleetDefenderModification;
	}
	public Double getFleetRangeModification() {
		return fleetRangeModification;
	}
	public void setFleetRangeModification(Double fleetRangeModification) {
		this.fleetRangeModification = fleetRangeModification;
	}
	public Double getFleetInitiativeBonus() {
		return fleetInitiativeBonus;
	}
	public void setFleetInitiativeBonus(Double fleetInitiativeBonus) {
		this.fleetInitiativeBonus = fleetInitiativeBonus;
	}
	public Double getFleetBattleTurningBonus() {
		return fleetBattleTurningBonus;
	}
	public void setFleetBattleTurningBonus(Double fleetBattleTurningBonus) {
		this.fleetBattleTurningBonus = fleetBattleTurningBonus;
	}
	
}