/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import mot.common.bean.NameList;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * @author Paul Davis
 *
 */
public class Specials extends TechnologyList {

	private static final String STATION_FIGHTERS_BAYS = "Station Fighters Bays";
	private static final String ELEMENT_NAME = "specials";
	private transient Double battleTurningBonus = null;
	private transient Double battleSpeedBonus = null;
	private transient Double speedBonus = null;
	private transient Double attackerModification = null;
	private transient Double defenderModification = null;
	private transient Double rangeModification = null;
	private transient Double initiativeBonus = null;
	private transient Double maxDamageBonus = null;
	private transient Double canColoniseRating = null;
	private transient Integer qtyBerths = null;
	private transient Integer qtyFighters = null;
    private transient Integer qtyHoloEmiters = null;
    private transient Double influence = null;
    private transient Double salvageEffect = null;
    private transient ShieldSpan shieldSpan = null;
    private transient Double repairEffect = null;
    private transient Boolean missileRetarget = null;
    
    private transient Special[] specials;

    public Specials() {

    }

    public Specials(Specials o) throws ApplicationError {
        super(o);
    }

    public Specials(Special w) {
        add(w);
    }

    public Specials(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getSpecial(names[i]) );
            }
        }
    }

    public Specials(ArrayList<String> names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.size(); i++) {
                add( Technologies.getSpecial(names.get(i)) );
            }
        }
    }

	public Specials(NameList names) throws ApplicationError {
        if (names != null) {
        	String[] na = names.getNamesAsArray();
        	if (na != null) {
	            for (int i=0; i<na.length; i++) {
	                add( Technologies.getSpecial(na[i]) );
	            }
        	}
        }
	}

	public Specials(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Special.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setSpecial(new Special(n[i]));
			}
		}
	}

	protected void changed() {
		battleTurningBonus = null;
		battleSpeedBonus = null;
		speedBonus = null;
		attackerModification = null;
		defenderModification = null;
		rangeModification = null;
		initiativeBonus = null;
		maxDamageBonus = null;
		canColoniseRating = null;
        specials = null;
        qtyBerths = null;
        qtyFighters = null;
        qtyHoloEmiters = null;
        influence = null;
        salvageEffect = null;
        shieldSpan = null;
        repairEffect = null;
        missileRetarget = null;
}

	/**
	 * Returns a bonus in speed generated by these specials. Speed is defined as the pace at which
	 * a craft moves in a straight line.
	 */
	public Double getSpeedBonusAsDouble() {
		if (speedBonus == null) {
			Collection list = getAll(true);
			if (list == null) {
				speedBonus = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
		            result = result + Comp.nnv(v.getSpeedBonus(), 0.0);
				}
				speedBonus = new Double(result);
			}
		}
		return speedBonus;
	}

    /**
     * See getSpeedBonusAsDouble()
     * @return Speed Bonus
     */
	public double getSpeedBonus() { return getSpeedBonusAsDouble().doubleValue(); }

	

	/**
     * Bonus (%) to turning speed of craft in battle.
	 */
    public double getBattleTurningBonus() {
		if (battleTurningBonus == null) {
			Collection list = getAll(true);
			if (list == null) {
				battleTurningBonus = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
		            result = result + Comp.nnv(v.getBattleTurningBonus(), 0.0);
				}
				battleTurningBonus = new Double(result);
			}
		}
		return battleTurningBonus.doubleValue();
    }

    /**
     * Bonus (%) to speed of craft in battle.
     */
    public double getBattleSpeedBonus() {
		if (battleSpeedBonus == null) {
			Collection list = getAll(true);
			if (list == null) {
				battleSpeedBonus = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
		            result = result + Comp.nnv(v.getBattleSpeedBonus(), 0.0);
				}
				battleSpeedBonus = new Double(result);
			}
		}
		return battleSpeedBonus.doubleValue();
	}

    public Special getBestBattleSpeedBonus() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v.getBattleSpeedBonus() != null) {
                    if (best == null || best.getBattleSpeedBonus().doubleValue() < v.getBattleSpeedBonus().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}


	/**
	 * The source craft source has fired on the target craft (these specials belong to the target
	 * craft) inflicting damage amount of damage. This method returns the new damage inflicted
	 * after the specials modified it.
	 */
	public double getDefenderModification() {
		if (defenderModification == null) {
			Collection list = getAll(true);
			if (list == null) {
				defenderModification = new Double(0);
			} else {
				Iterator i = list.iterator();
                double r = 0.0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
					r += Comp.nnv(v.getDefenderModification(), 0.0);
				}
				defenderModification = new Double(r);
			}
		}
		return defenderModification.doubleValue();
	}


	/**
	 * The source craft source has fired on the target craft (these specials belong to the source
	 * craft) inflicting damage amount of damage. This method returns the new damage inflicted
	 * after the specials modified it.
	 */
	public double getAttackerModification() {
		if (attackerModification == null) {
			Collection list = getAll(true);
			if (list == null) {
				attackerModification = new Double(0);
			} else {
				Iterator i = list.iterator();
                double r = 0.0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
					r += Comp.nnv(v.getAttackerModification(), 0.0);
				}
				attackerModification = new Double(r);
			}
		}
		return attackerModification.doubleValue();
	}

	/**
	 * The quantity the weapon range is modified by this list of specials.
	 */
	public double getRangeModification() {
		if (rangeModification == null) {
			Collection list = getAll(true);
			if (list == null) {
				rangeModification = new Double(0);
			} else {
				Iterator i = list.iterator();
                double r = 0.0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
					r += Comp.nnv(v.getRangeModification(), 0.0);
				}
				rangeModification = new Double(r);
			}
		}
		return rangeModification.doubleValue();
	}

    /**
     * Bonus (%) to the initiative of a craft in battle.
     */
    public double getInitiativeBonus() {
		if (initiativeBonus == null) {
			Collection list = getAll(true);
			if (list == null) {
				initiativeBonus = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
		            result = result + Comp.nnv(v.getInitiativeBonus(), 0);
				}
				initiativeBonus = new Double(result);
			}
		}
		return initiativeBonus.doubleValue();
    }

    /**
     * Bonus (%) to the maximum damage a craft can sustain in battle.
     */
	public double getMaxDamageBonus() {
		if (maxDamageBonus == null) {
			Collection list = getAll(true);
			if (list == null) {
				maxDamageBonus = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
		            result = result + Comp.nnv(v.getMaxDamageBonus(), 0);
				}
				maxDamageBonus = new Double(result);
			}
		}
		return maxDamageBonus.doubleValue();
	}

    /**
     * Returns the colonisation rating for this list of specials.
     */
    public Double getCanColoniseRating() {
		if (canColoniseRating == null) {
			Collection list = getAll(true);
			if (list == null) {
				canColoniseRating = new Double(-1);
			} else {
				Iterator i = list.iterator();
				while (i.hasNext()) {
					Special v = (Special)i.next();
					if (v.getCanColoniseRating() != null) {
						if (canColoniseRating == null || v.getCanColoniseRating().doubleValue() < canColoniseRating.doubleValue()) {
		            		canColoniseRating = v.getCanColoniseRating();
						}
					}
				}
				if (canColoniseRating == null) {
					canColoniseRating = new Double(-1);
				}
			}
		}
        if (canColoniseRating.doubleValue() == -1) {
            return null;
        } else {
            return canColoniseRating;
        }
	}

    /**
     * Returns the Quantity of berths rating from this list of specials.
     */
    public int getQtyBerthsAsInt() {
        Integer r = getQtyBerths();
        if (r == null) {
            return 0;
        } else {
            return r.intValue();
        }
    }
    
    /**
     * Returns the Quantity of berths rating from this list of specials.
     */
    public Integer getQtyBerths() {
		if (qtyBerths == null) {
			Collection list = getAll(true);
            int result = 0;
			if (list != null) {
				Iterator i = list.iterator();
				while (i.hasNext()) {
					Special v = (Special)i.next();
					if (v != null && v.getQtyBerths() != null) {
                        result += v.getQtyBerths().intValue();
					}
				}
			}
            qtyBerths = new Integer(result);
		}
        return qtyBerths;
	}

    /**
     * Returns the Quantity of fighters in this list of specials.
     */
    public Integer getQtyFighters() {
		if (qtyFighters == null) {
			Collection list = getAll(true);
            int result = 0;
			if (list != null) {
				Iterator i = list.iterator();
				while (i.hasNext()) {
					Special v = (Special)i.next();
					if (v != null && v.getQtyFighters() != null) {
                        result += v.getQtyFighters().intValue();
					}
				}
			}
            qtyFighters = new Integer(result);
		}
        return qtyFighters;
	}

    /**
     * Returns the special in this list with the best colonisation ability.
     */
    public Special getBestCanColonise() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getCanColoniseRating() != null) {
                    if (best == null || best.getCanColoniseRating().doubleValue() > v.getCanColoniseRating().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the most berths.
     */
    public Special getBestQtyBerths() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getQtyBerths() != null) {
                    if (best == null || best.getQtyBerths().intValue() < v.getQtyBerths().intValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the lowest negative space.
     */
    public Special getBestSpacePod() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getSpace() < 0) {
                    if (best == null || best.getSpace() > v.getSpace()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    public Special getBestHoloEmiter() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getQtyHoloCraft() != null && v.getQtyHoloCraft() > 0) {
                    if (best == null || best.getQtyHoloCraft() < v.getQtyHoloCraft()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the highest defender modification.
     */
    public Special getBestDefenderModification() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getDefenderModification() != null) {
                    if (best == null || best.getDefenderModification().doubleValue() < v.getDefenderModification().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the highest attacker modification.
     */
    public Special getBestAttackerModification() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getAttackerModification() != null) {
                    if (best == null || best.getAttackerModification().doubleValue() < v.getAttackerModification().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the highest max damage bonus.
     */
    public Special getBestMaxDamageBonus() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getMaxDamageBonus() != null) {
                    if (best == null || best.getMaxDamageBonus().doubleValue() < v.getMaxDamageBonus().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the highest speed bonus.
     */
    public Special getBestSpeedBonus() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getSpeedBonus() != null) {
                    if (best == null || best.getSpeedBonus().doubleValue() < v.getSpeedBonus().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

	public Special getBestRepairEffect() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v != null && v.getRepairEffect() != null) {
                    if (best == null || best.getRepairEffect().doubleValue() < v.getRepairEffect().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}
	
    /**
     * Returns the quantity of holo emiters in this list of specials.
     */
    public int getQtyHoloEmiter() {
        if (qtyHoloEmiters == null) {
            Collection list = getAll(true);
            if (list == null) {
                qtyHoloEmiters = new Integer(0);
                return 0;
            }
            Iterator i = list.iterator();
            int qty = 0;
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (Comp.same(v.getGroup(), Special.GROUP_HOLO_EMITER) && v.getQtyHoloCraft() != null) {
                    qty += v.getQtyHoloCraft().intValue();
                }
            }
            qtyHoloEmiters = new Integer(qty);
            return qty;
        } else {
            return qtyHoloEmiters.intValue();
        }
	}

	/**
     * The total influence in the list.
	 */
    public double getInfluence() {
		if (influence == null) {
			Collection list = getAll(true);
			if (list == null) {
				influence = new Double(0.0);
			} else {
				Iterator i = list.iterator();
		        double result = 0;
				while (i.hasNext()) {
					Special v = (Special)i.next();
		            result = result + Comp.nnv(v.getInfluence(), 0.0);
				}
				influence = new Double(result);
			}
		}
		return influence.doubleValue();
    }

    
	public void setSpecial(String v) throws ApplicationError {
		add( Technologies.getSpecials().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_specials.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    public Special getSpecial(String name) {
        return (Special)super.get(name);
    }
    public Special getSpecialAtIndex(int index) {
        return (Special)getTechnologyAtIndex(index);
    }
    public void setSpecial(Special v) {
        add(v);
    }
    
    public Special[] getSpecials() {
        if (specials == null) {
            Collection list = getAll(true);
            if (list != null && list.size() > 0) {
                specials = (Special[])list.toArray(new Special[(list.size())]);
            }
        }
        return specials;
    }

    /**
     * When a craft is destroyed some of its technology its left in the wreckage and is 
     * This is the effect these specials have on the amount of technology salvaged from 
     * its owning craft. A value of 1.0 implies no change, a value of 0.0 would mean
     * removes all technology.
     */
	public Double getSalvageEffect() {
		if (salvageEffect == null) {
			Special[] specials = getSpecials();
	        double r = 1.0;
	        for (int i=0; specials != null && i<specials.length; i++) {
	            if (specials[i].getSalvageEffect() != null) {
	                r = r * specials[i].getSalvageEffect().doubleValue();
	            }
	        }
	        salvageEffect = r;
		}
		return salvageEffect;
	}

	/**
	 * @return The shield span in this list that has the largest span percent.
	 */
	public ShieldSpan getBestShieldSpan() {
		if (shieldSpan == null) {
			Special[] specials = getSpecials();
	        for (int i=0; specials != null && i<specials.length; i++) {
	            if (specials[i].getShieldSpan() != null) {
	            	if (shieldSpan == null) {
	            		shieldSpan = specials[i].getShieldSpan();
	            	} else if (shieldSpan.getSpanPercent() < specials[i].getShieldSpan().getSpanPercent()) {
	            		shieldSpan = specials[i].getShieldSpan();
	            	}
	            }
	        }
		}
		return shieldSpan;
	}

	public boolean isImune(String group) {
		if (group == null) return false;
		Special[] specials = getSpecials();
		if (specials == null) return false;
        for (int i=0; i<specials.length; i++) {
            if (specials[i].isImmuneTo(group)) {
                return true;
            }
        }
        return false;
	}

    public Double getRepairEffect() {
		if (repairEffect == null) {
			Special[] specials = getSpecials();
	        double r = 0;
	        for (int i=0; specials != null && i<specials.length; i++) {
	            if (specials[i].getRepairEffect() != null) {
	                r = r + specials[i].getRepairEffect().doubleValue();
	            }
	        }
	        repairEffect = r;
		}
		return repairEffect;
	}

	public boolean hasMissileRetarget() {
		if (missileRetarget == null) {
			Special[] specials = getSpecials();
	        boolean r = false;
	        for (int i=0; !r && specials != null && i<specials.length; i++) {
	            if (specials[i].getMissileRetarget() != null) {
	                r = specials[i].getMissileRetarget().booleanValue();
	            }
	        }
	        missileRetarget = r;
		}
		return missileRetarget == null ? false : missileRetarget.booleanValue();
	}

    /**
     * Returns the special in this list with the highest fleet Battle Speed Bonus.
     */
    public Special getBestFleetBattleSpeedBonus() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v.getFleetBattleSpeedBonus() != null) {
                    if (best == null || best.getFleetBattleSpeedBonus().doubleValue() < v.getFleetBattleSpeedBonus().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the highest fleet attacker modification.
     */
    public Special getBestFleetAttackerModification() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v.getFleetAttackerModification() != null) {
                    if (best == null || best.getFleetAttackerModification().doubleValue() < v.getFleetAttackerModification().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the highest fleet defender modification.
     */
    public Special getBestFleetDefenderModification() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v.getFleetDefenderModification() != null) {
                    if (best == null || best.getFleetDefenderModification().doubleValue() < v.getFleetDefenderModification().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the highest fleet Range Modification.
     */
    public Special getBestFleetRangeModification() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v.getFleetRangeModification() != null) {
                    if (best == null || best.getFleetRangeModification().doubleValue() < v.getFleetRangeModification().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

    /**
     * Returns the special in this list with the highest fleet Initiative Bonus.
     */
    public Special getBestFleetInitiativeBonus() {
        Collection list = getAll(true);
        Special best = null;
        if (list != null) {
            Iterator i = list.iterator();
            while (i.hasNext()) {
                Special v = (Special)i.next();
                if (v.getFleetInitiativeBonus() != null) {
                    if (best == null || best.getFleetInitiativeBonus().doubleValue() < v.getFleetInitiativeBonus().doubleValue()) {
                        best = v;
                    }
                }
            }
        }
        return best;
	}

	public boolean hasBattleBonuses() {
		Special s = getBestFleetAttackerModification();
		if (s != null && s.getFleetAttackerModification() != null) {
			return true;
		}
		s = getBestFleetDefenderModification();
		if (s != null && s.getFleetDefenderModification() != null) {
			return true;
		}
		s = getBestFleetBattleSpeedBonus();
		if (s != null && s.getFleetBattleSpeedBonus() != null) {
			return true;
		}
		s = getBestFleetRangeModification();
		if (s != null && s.getFleetRangeModification() != null) {
			return true;
		}
		s = getBestFleetInitiativeBonus();
		if (s != null && s.getFleetInitiativeBonus() != null) {
			return true;
		}
		return false;
	}


	public Special getStationFighterBays() {
		Special r = getSpecial(STATION_FIGHTERS_BAYS);
		if (r == null) Logger.logWarning("Special "+STATION_FIGHTERS_BAYS+" missing");
		return r;
	}

	public Technology getFighterBays() {
		Technology s = getMostAdvancedWithGroup(Special.GROUP_FIGHTER_BAYS);
		return s;
	}
	
}