/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.io.File;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.DomFactory;
import mot.common.xml.XMLDocumentHandler;

/**
 * Defines the complete set of technology on offer in the game.
 */
public class Technologies {
	
	private static final String TECH_MISC_FILENAME = "xml/tech_misc.xml";
	private static final String TECH_MODULES_FILENAME = "xml/tech_modules.xml";
	private static final String TECH_HUMANITIES_FILENAME = "xml/tech_humanities.xml";
	private static final String TECH_THEORIES_FILENAME = "xml/tech_theories.xml";
	private static final String TECH_WEAPONS_FILENAME = "xml/tech_weapons.xml";
	private static final String TECH_SPECIALS_FILENAME = "xml/tech_specials.xml";
	private static final String TECH_SHIELDS_FILENAME = "xml/tech_shields.xml";
	private static final String TECH_SCANNERS_FILENAME = "xml/tech_scanners.xml";
	private static final String TECH_LABS_FILENAME = "xml/tech_labs.xml";
	private static final String TECH_HULLS_FILENAME = "xml/tech_hulls.xml";
	private static final String TECH_FACTORIES_FILENAME = "xml/tech_factories.xml";
	private static final String TECH_ENGINES_FILENAME = "xml/tech_engines.xml";
	private static final String TECH_COMPUTERS_FILENAME = "xml/tech_computers.xml";
	private static final String TECH_BIOS_FILENAME = "xml/tech_bios.xml";
	
	private static Biotechnologies biotechnologies;
	private static Computers computers;
	private static Engines engines;
	private static Factories factories;
	private static Hulls hulls;
	private static Laboratories laboratories;
	private static Scanners scanners;
	private static Shields shields;
	private static Specials specials;
    private static Weapons weapons;
    private static Modules modules;
    private static Humanities humanities;
    private static Theories theories;
    private static MiscTechnologies miscTechnologies;

	public static void flush() {
        biotechnologies = null;
        computers = null;
        engines = null;
        factories = null;
        hulls = null;
        laboratories = null;
        scanners = null;
        shields = null;
        weapons = null;
        theories = null;
        modules = null;
        humanities = null;
        specials = null;
        miscTechnologies = null;
    }

	public static Biotechnologies getBiotechnologies() throws ApplicationError {
		if (biotechnologies == null) {
			File file = new File("updates/tech_bios.xml");
			if (file.exists()) {
				biotechnologies = (Biotechnologies)XMLDocumentHandler.parseFile("updates/tech_bios.xml", true, "xml/tech_bios.xsd");
			} else {
				biotechnologies = (Biotechnologies)XMLDocumentHandler.parseResource(TECH_BIOS_FILENAME);
			}
		}
		return biotechnologies;
	}

	public static Biotechnology getBiotechnology(String name) throws ApplicationError {
		Biotechnologies techs = getBiotechnologies();
		if (techs == null) {
			throw new ApplicationError("Technologies.getBiotechnology() No biotechs defined");
		}
		return (Biotechnology)techs.get(name);
	}

	public static Computers getComputers() throws ApplicationError {
		if (computers == null) {
			File file = new File("updates/tech_computers.xml");
			if (file.exists()) {
				computers = (Computers)XMLDocumentHandler.parseFile("updates/tech_computers.xml", true, "xml/tech_computers.xsd");
			} else {
				computers = (Computers)XMLDocumentHandler.parseResource(TECH_COMPUTERS_FILENAME);
			}

		}
		return computers;
	}

	public static Computer getComputer(String name) throws ApplicationError {
		Computers techs = getComputers();
		if (techs == null) {
			throw new ApplicationError("Technologies.getComputer() No Computers defined");
		}
		return (Computer)techs.get(name);
	}

	public static Engines getEngines() throws ApplicationError {
		if (engines == null) {
			File file = new File("updates/tech_engines.xml");
			if (file.exists()) {
				engines = (Engines)XMLDocumentHandler.parseFile("updates/tech_engines.xml", true, "xml/tech_engines.xsd");
			} else {
				engines = (Engines)XMLDocumentHandler.parseResource(TECH_ENGINES_FILENAME);
			}
		}
		return engines;
	}

	public static Engine getEngine(String name) throws ApplicationError {
		Engines techs = getEngines();
		if (techs == null) {
			throw new ApplicationError("Technologies.getEngine() No Engines defined");
		}
		return (Engine)techs.get(name);
	}

	public static Factories getFactories() throws ApplicationError {
		if (factories == null) {
			File file = new File("updates/tech_factories.xml");
			if (file.exists()) {
				factories = (Factories)XMLDocumentHandler.parseFile("updates/tech_factories.xml", true, "xml/tech_factories.xsd");
			} else {
				factories = (Factories)XMLDocumentHandler.parseResource(TECH_FACTORIES_FILENAME);
			}

		}
		return factories;
	}

	public static Factory getFactory(String name) throws ApplicationError {
		Factories techs = getFactories();
		if (techs == null) {
			throw new ApplicationError("Technologies.getFactory() No Factories defined");
		}
		return (Factory)techs.get(name);
	}

	public static Hulls getHulls() throws ApplicationError {
		if (hulls == null) {
			File file = new File("updates/tech_hulls.xml");
			if (file.exists()) {
				hulls = (Hulls)XMLDocumentHandler.parseFile("updates/tech_hulls.xml", true, "xml/tech_hulls.xsd");
			} else {
				hulls = (Hulls)XMLDocumentHandler.parseResource(TECH_HULLS_FILENAME);
			}

		}
		return hulls;
	}

	public static Hull getHull(String name) throws ApplicationError {
		Hulls techs = getHulls();
		if (techs == null) {
			throw new ApplicationError("Technologies.getHull() No Hulls defined");
		}
		return (Hull)techs.get(name);
	}

	public static Laboratories getLaboratories() throws ApplicationError {
		if (laboratories == null) {
			File file = new File("updates/tech_labs.xml");
			if (file.exists()) {
				laboratories = (Laboratories)XMLDocumentHandler.parseFile("updates/tech_labs.xml", true, "xml/tech_labs.xsd");
			} else {
				laboratories = (Laboratories)XMLDocumentHandler.parseResource(TECH_LABS_FILENAME);
			}
		}
		return laboratories;
	}

	public static Laboratory getLaboratory(String name) throws ApplicationError {
		Laboratories techs = getLaboratories();
		if (techs == null) {
			throw new ApplicationError("Technologies.getLaboratory() No laboratories defined");
		}
		return (Laboratory)techs.get(name);
	}

	public static Scanners getScanners() throws ApplicationError {
		if (scanners == null) {
			File file = new File("updates/tech_scanners.xml");
			if (file.exists()) {
				scanners = (Scanners)XMLDocumentHandler.parseFile("updates/tech_scanners.xml", true, "xml/tech_scanners.xsd");
			} else {
				scanners = (Scanners)XMLDocumentHandler.parseResource(TECH_SCANNERS_FILENAME);
			}
		}
		return scanners;
	}

	public static Scanner getScanner(String name) throws ApplicationError {
		Scanners techs = getScanners();
		if (techs == null) {
			throw new ApplicationError("Technologies.getScanner() No Scanners defined");
		}
		return (Scanner)techs.get(name);
	}

	public static Shields getShields() throws ApplicationError {
		if (shields == null) {
			File file = new File("updates/tech_shields.xml");
			if (file.exists()) {
				shields = (Shields)XMLDocumentHandler.parseFile("updates/tech_shields.xml", true, "xml/tech_shields.xsd"); 
			} else {
				shields = (Shields)XMLDocumentHandler.parseResource(TECH_SHIELDS_FILENAME); 
			}
		}
		return shields;
	}

	public static Shield getShield(String name) throws ApplicationError {
		Shields techs = getShields();
		if (techs == null) {
			throw new ApplicationError("Technologies.getShield() No Shields defined");
		}
		return (Shield)techs.get(name);
	}

	public static Specials getSpecials() throws ApplicationError {
		if (specials == null) {
			File file = new File("updates/tech_specials.xml");
			if (file.exists()) {
				specials = (Specials)XMLDocumentHandler.parseFile("updates/tech_specials.xml", true, "xml/tech_specials.xsd");
			} else {
				specials = (Specials)XMLDocumentHandler.parseResource(TECH_SPECIALS_FILENAME);
			}
		}
		return specials;
	}

	public static Special getSpecial(String name) throws ApplicationError {
		Specials techs = getSpecials();
		if (techs == null) {
			throw new ApplicationError("Technologies.getSpecial() No Specials defined");
		}
		return (Special)techs.get(name);
	}

	public static Weapons getWeapons() throws ApplicationError {
		if (weapons == null) {
			File file = new File("updates/tech_weapons.xml");
			if (file.exists()) {
				weapons = (Weapons)XMLDocumentHandler.parseFile("updates/tech_weapons.xml", true, "xml/tech_weapons.xsd");
			} else {
				weapons = (Weapons)XMLDocumentHandler.parseResource(TECH_WEAPONS_FILENAME);
			}
		}
		return weapons;
	}

	public static Weapon getWeapon(String name) throws ApplicationError {
		Weapons techs = getWeapons();
		if (techs == null) {
			throw new ApplicationError("Technologies.getWeapon() No Weapons defined");
		}
		return (Weapon)techs.get(name);
	}

	public static Theories getTheories() throws ApplicationError {
		if (theories == null) {
			File file = new File("updates/tech_theories.xml");
			if (file.exists()) {
				theories = (Theories)XMLDocumentHandler.parseFile("updates/tech_theories.xml", true, "xml/tech_theories.xsd");
			} else {
				theories = (Theories)XMLDocumentHandler.parseResource(TECH_THEORIES_FILENAME);
			}
		}
		return theories;
	}

	public static Theory getTheory(String name) throws ApplicationError {
		Theories techs = getTheories();
		if (techs == null) {
			throw new ApplicationError("Technologies.getTheory() No Theory defined");
		}
		return (Theory)techs.get(name);
	}

	public static Modules getModules() throws ApplicationError {
		if (modules == null) {
			File file = new File("updates/tech_modules.xml");
			if (file.exists()) {
				modules = (Modules)XMLDocumentHandler.parseFile("updates/tech_modules.xml", true, "xml/tech_modules.xsd");
			} else {
				modules = (Modules)XMLDocumentHandler.parseResource(TECH_MODULES_FILENAME);
			}
		}
		return modules;
	}

	public static Module getModule(String name) throws ApplicationError {
		Modules techs = getModules();
		if (techs == null) {
			throw new ApplicationError("Technologies.getModule() No Modules defined");
		}
		return (Module)techs.get(name);
	}

	public static Humanities getHumanities() throws ApplicationError {
		if (humanities == null) {
			File file = new File("updates/tech_humanities.xml");
			if (file.exists()) {
				humanities = (Humanities)XMLDocumentHandler.parseFile("updates/tech_humanities.xml", true, "xml/tech_humanities.xsd");
			} else {
				humanities = (Humanities)XMLDocumentHandler.parseResource(TECH_HUMANITIES_FILENAME);
			}
		}
		return humanities;
	}

	public static Humanity getHumanity(String name) throws ApplicationError {
		Humanities techs = getHumanities();
		if (techs == null) {
			throw new ApplicationError("Technologies.getHumanity() No humanities defined");
		}
		return (Humanity)techs.get(name);
	}

	public static MiscTechnologies getMiscTechnologies() throws ApplicationError {
		if (miscTechnologies == null) {
			File file = new File("updates/tech_misc.xml");
			if (file.exists()) {
				miscTechnologies = (MiscTechnologies)XMLDocumentHandler.parseFile("updates/tech_misc.xml", true, "xml/tech_misc.xsd");
			} else {
				miscTechnologies = (MiscTechnologies)XMLDocumentHandler.parseResource(TECH_MISC_FILENAME);
			}
		}
		return miscTechnologies;
	}

	public static MiscTechnology getMiscTechnology(String name) throws ApplicationError {
		MiscTechnologies techs = getMiscTechnologies();
		if (techs == null) {
			throw new ApplicationError("Technologies.getMiscTechnologies() No Misc techs defined");
		}
		return (MiscTechnology)techs.get(name);
	}

	public static Kamikaze getMiscKamikazeTechnology() throws ApplicationError {
		MiscTechnologies techs = getMiscTechnologies();
		if (techs == null) {
			throw new ApplicationError("Technologies.getMiscTechnologies() No Misc techs defined");
		}
		return techs.getKamikaziTechnology();
	}

	public static Humanity getHumanTradeRoute() throws ApplicationError {
		Humanities techs = getHumanities();
		if (techs == null) {
			throw new ApplicationError("Technologies.getHumanTradeRoute() No Humanity techs defined");
		}
		return techs.getHumanity("Human Trade Route");
	}


	
	public static final String TECH_TYPE_FACTORY = "factory";
	public static final String TECH_TYPE_LABORATORY = "laboratory";
	public static final String TECH_TYPE_BIOTECHNOLOGY = "biotechnology";
	public static final String TECH_TYPE_HULL = "hull";
	public static final String TECH_TYPE_COMPUTER = "computer";
	public static final String TECH_TYPE_WEAPON = "weapon";
	public static final String TECH_TYPE_SHIELD = "shield";
	public static final String TECH_TYPE_SCANNER = "scanner";
	public static final String TECH_TYPE_SPECIAL = "special";
    public static final String TECH_TYPE_ENGINE = "engine";
    public static final String TECH_TYPE_THEORY = "theory";
    public static final String TECH_TYPE_MODULE = "module";
    public static final String TECH_TYPE_HUMANITIES = "humanity";
    public static final String TECH_TYPE_MISC = "misc";

    public static Technology getTechnology(String techType, String techName) throws ApplicationError {
		if (Comp.same(techType, TECH_TYPE_FACTORY)) {
			return getFactory(techName);
		} else if (Comp.same(techType, TECH_TYPE_LABORATORY)) {
			return getLaboratory(techName);
		} else if (Comp.same(techType, TECH_TYPE_BIOTECHNOLOGY)) {
			return getBiotechnology(techName);
		} else if (Comp.same(techType, TECH_TYPE_HULL)) {
			return getHull(techName);
		} else if (Comp.same(techType, TECH_TYPE_COMPUTER)) {
			return getComputer(techName);
		} else if (Comp.same(techType, TECH_TYPE_WEAPON)) {
			return getWeapon(techName);
		} else if (Comp.same(techType, TECH_TYPE_SHIELD)) {
			return getShield(techName);
		} else if (Comp.same(techType, TECH_TYPE_SCANNER)) {
			return getScanner(techName);
		} else if (Comp.same(techType, TECH_TYPE_SPECIAL)) {
			return getSpecial(techName);
		} else if (Comp.same(techType, TECH_TYPE_ENGINE)) {
			return getEngine(techName);
		} else if (Comp.same(techType, TECH_TYPE_THEORY)) {
			return getTheory(techName);
		} else if (Comp.same(techType, TECH_TYPE_MODULE)) {
			return getModule(techName);
		} else if (Comp.same(techType, TECH_TYPE_HUMANITIES)) {
			return getHumanity(techName);
		} else if (Comp.same(techType, TECH_TYPE_MISC)) {
			return getMiscTechnology(techName);
		} else {
			throw new ApplicationError("Technologies.add() unknown tech type "+techType);
		}
    }

    public static Technology getTechnology(String techFullName) throws ApplicationError {
        int sepIndex = techFullName.indexOf(getSeperator());
        if (sepIndex < 1) {
            throw new ApplicationError("Technologies.getTechnology() Full name does not have seperator "+techFullName);
        }
        String techType = techFullName.substring(0, sepIndex);
        String techName = techFullName.substring(sepIndex+getSeperator().length());
        return getTechnology(techType, techName);
    }
    
    public static final String getSeperator() {
        return " , ";
    }

    /**
     * @return Returns a list containing all technology in the game.
     */
    public static TechnologyList getTechnologies() throws ApplicationError {
        return getTechnologies(true);
    }
    
    /**
     * @return Returns a list containing all technology in the game.
     */
    public static TechnologyList getTechnologies(boolean includeTheory) throws ApplicationError {
        TechnologyList techList = new GeneralTechnologyList();
        if (getFactories() != null) techList.add(factories);
        if (getLaboratories() != null) techList.add(laboratories);
        if (getBiotechnologies() != null) techList.add(biotechnologies);
        if (getHulls() != null) techList.add(hulls);
        if (getComputers() != null) techList.add(computers);
        if (getWeapons() != null) techList.add(weapons);
        if (getShields() != null) techList.add(shields);
        if (getScanners() != null) techList.add(scanners);
        if (getSpecials() != null) techList.add(specials);
        if (getEngines() != null) techList.add(engines);
        if (getModules() != null) techList.add(modules);
        if (getHumanities() != null) techList.add(humanities);
        if (getTheories() != null && includeTheory) techList.add(theories);
        if (getMiscTechnologies() != null) techList.add(miscTechnologies);
        return techList;
    }

    /**
     * Returns an array of all technology that have the same research cost that is higher than
     * the parameter min. The cost will be the closest cost of any technology above the value min.
     */
    public static Technology [] getCheapestToResearch(int min) throws ApplicationError {
        GeneralTechnologyList list = new GeneralTechnologyList();
        if (getBiotechnologies() != null) list = biotechnologies.getCheapestToResearch(min, list);
        if (getSpecials() != null) list = specials.getCheapestToResearch(min, list);
        if (getComputers() != null) list = computers.getCheapestToResearch(min, list);
        if (getEngines() != null) list = engines.getCheapestToResearch(min, list);
        if (getFactories() != null) list = factories.getCheapestToResearch(min, list);
        if (getLaboratories() != null) list = laboratories.getCheapestToResearch(min, list);
        if (getShields() != null) list = shields.getCheapestToResearch(min, list);
        if (getWeapons() != null) list = weapons.getCheapestToResearch(min, list);
        if (getScanners() != null) list = scanners.getCheapestToResearch(min, list);
        if (getHulls() != null) list = hulls.getCheapestToResearch(min, list);
        if (getModules() != null) list = modules.getCheapestToResearch(min, list);
        if (getHumanities() != null) list = humanities.getCheapestToResearch(min, list);
        if (getTheories() != null) list = theories.getCheapestToResearch(min, list);
        if (getMiscTechnologies() != null) list = miscTechnologies.getCheapestToResearch(min, list);

        if (list.getQuantity() == 0) {
            return null;
        } else {
            return list.getAllAsArray(true);
        }
    }

	public static void setBiotechnologies(Biotechnologies v) { biotechnologies = v; }
	public static void setComputers(Computers  v) { computers = v; }
	public static void setEngines(Engines  v) { engines = v; }
	public static void setFactories(Factories  v) { factories = v; }
	public static void setHulls(Hulls  v) { hulls = v; }
	public static void setLaboratories(Laboratories  v) { laboratories = v; }
	public static void setScanners(Scanners  v) { scanners = v; }
	public static void setShields(Shields  v) { shields = v; }
	public static void setSpecials(Specials  v) { specials = v; }
    public static void setWeapons(Weapons  v) { weapons = v; }
    public static void setTheories(Theories  v) { theories = v; }
    public static void setModules(Modules  v) { modules = v; }
    public static void setHumanities(Humanities  v) { humanities = v; }
    public static void setMiscTechnologies(MiscTechnologies  v) { miscTechnologies = v; }

	public static boolean notInitialised() {
		return (weapons == null);
	}

	public static void removeUpdates() {
		try {
			File file = new File("updates/tech_bios.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_computers.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}   
		try {
			File file = new File("updates/tech_engines.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_factories.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_hulls.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_labs.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_scanners.xml");   
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_scanners.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_shields.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_specials.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);  
		}
		try {
			File file = new File("updates/tech_weapons.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_theories.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_modules.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_humanities.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
		try {
			File file = new File("updates/tech_misc.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
	}

	public static void preLoad() throws ApplicationError {
        biotechnologies = new Biotechnologies(DomFactory.load(TECH_BIOS_FILENAME));
        computers = new Computers(DomFactory.load(TECH_COMPUTERS_FILENAME));
        engines = new Engines(DomFactory.load(TECH_ENGINES_FILENAME));
        factories = new Factories(DomFactory.load(TECH_FACTORIES_FILENAME));
        hulls = new Hulls(DomFactory.load(TECH_HULLS_FILENAME));
        laboratories = new Laboratories(DomFactory.load(TECH_LABS_FILENAME));
        scanners = new Scanners(DomFactory.load(TECH_SCANNERS_FILENAME));
        shields = new Shields(DomFactory.load(TECH_SHIELDS_FILENAME));
        weapons = new Weapons(DomFactory.load(TECH_WEAPONS_FILENAME));
        theories = new Theories(DomFactory.load(TECH_THEORIES_FILENAME));
        modules = new Modules(DomFactory.load(TECH_MODULES_FILENAME));
        humanities = new Humanities(DomFactory.load(TECH_HUMANITIES_FILENAME));
        specials = new Specials(DomFactory.load(TECH_SPECIALS_FILENAME));
        miscTechnologies = new MiscTechnologies(DomFactory.load(TECH_MISC_FILENAME));        
	}


}