/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.io.Serializable;

import mot.common.bean.NamedObject;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * Base class for all technologies defined in the game
 *
 */
public abstract class Technology extends NamedObject implements Cloneable, TechTreeNodeParent, Serializable {

	/**
	 * The cost to build this piece of technology.
	 */
	private int costToBuild;

	/**
	 * Some technologies have their build cost effected by the hull size.
	 * If this applies to this technology then this value must be set to true.
	 * The default value is false.
	 */
	private boolean useBuildCostHullFactor = false;

	/**
	 * The cost to research this piece of technology.
	 */
	private int costToResearch;

	/**
	 * Returns the amount of space in this hull or this technology would consume in a hull.
     * Determines how many weapons, shields etc may be installed.
	 */
	private int space;

    /**
     * The description of this piece of technology.
     * This is optional for all tech types but can be useful to display to user about this
     * technology.
     */
    private String description;

    /**
     * Some technologies have an image specific for this technology. Other technologies use a single image for 
     * all technologies of that type. For example, at this point in time all engine technologies have the same
     * image displayed on the research screen (technology details screen), while special technologies have a different
     * image for each special.
     * If there is an image for this technology item, then this attribute will store its name.     
     */
    private String image;
    

	/**
     * Only a single technology from the same group can be used. Group is optional and it is not used
     * then this technology once discovered will always be available.
     * This allows the game to make some technologies obsolete when this technology is discovered.
     * Only a single piece of technology from a group is allowed on a craft at the same time.
	 */
	private String group;
    
    /**
     * This is the up keep cost of this technology per turn. So for example each craft with a 
     * Hull containing an upkeep cost of 2 would debit the users money each turn by 2. Usally 
     * crafts, and things installed on a planet have a maintenance cost (e.g. weapons, shields).
     */
    private Integer maintenanceCost;
    
    /**
     * Technologies may have restrictions applied to them. Restriction reduce a players ability to 
     * discover a technology. It may be a temporary description such as you cannot discover this 
     * tech until a certain condition has been satisfied or a permanent description, such as a player 
     * needs a player characteristic to discover this technology.
     * 
     * For details see the Restriction class.
     */
    private Restriction restriction;
    
    /**
     * All technologies can be discovered through espionage unless that technology is explicitly forbidden
     * by this having a value of TRUE.
     */
    private boolean forbidEspionage = false;

    protected Technology() {
    }

    protected Technology(Technology o) {
        super(o);
        costToBuild = o.getCostToBuild();
        costToResearch = o.getCostToResearch();
        useBuildCostHullFactor = o.getUseBuildCostHullFactor();
    }


	public Technology(DomNode node) {
		setName(node.getNodeValueByName_String("name"));
		
		description = node.getNodeValueByName_String("description");
		space = node.getNodeValueByName_Int("space");
		maintenanceCost = node.getNodeValueByName_Integer("maintenance-cost");
		useBuildCostHullFactor = node.getNodeValueByName_Bool("use-build-cost-hull-factor");
		image = node.getNodeValueByName_String("image");

		costToBuild = node.getNodeValueByName_Int("cost-to-build");
		costToResearch = node.getNodeValueByName_Int("cost-to-research");
		group = node.getNodeValueByName_String("group");
		
		forbidEspionage = node.getNodeValueByName_Bool("forbid-espionage");
		
		DomNode mcr = node.getNodeByName(MinCostRestriction.ELEMENT_NAME);
		if (mcr != null) restriction = new MinCostRestriction(mcr);

		DomNode ar = node.getNodeByName(AlignmentRestriction.ELEMENT_NAME);
		if (ar != null) restriction = new AlignmentRestriction(ar);

		DomNode mr = node.getNodeByName(MandatoryRestriction.ELEMENT_NAME);
		if (mr != null) restriction = new MandatoryRestriction(mr);

	}

	public int getCostToBuild() { return costToBuild; }
	public int getCostToResearch() { return costToResearch; }
	public boolean getUseBuildCostHullFactor() { return useBuildCostHullFactor; }
    public int getSpace() { return space; }
    
	public void setCostToBuild(int v) { costToBuild = v; }
	public void setCostToResearch(int v) { costToResearch = v; }
	public void setUseBuildCostHullFactor(boolean v) { useBuildCostHullFactor = v; }
    public void setSpace(int v) { space = v; }

    public Technology cloneTechnology() throws ApplicationError {
        try {
            return (Technology)clone();
        } catch (CloneNotSupportedException e) {
            throw new ApplicationError("Technology.cloneTechnology() clone not supported "+e+" on "+toString());
        }
    }

    public int getSpace(Hull hull) { 
        if (hull == null) {
            return space;
        } else {
            return getSpace(hull.getCostToBuildFactor());
        }
    }
    
	public int getSpace(double costToBuildHullFactor) {
		if (useBuildCostHullFactor) {
			return (int) (space * costToBuildHullFactor);
		} else {
			return space;
		}
	}

	public int getCostToBuild(double costToBuildHullFactor) {
		if (useBuildCostHullFactor) {
			return (int) (costToBuild * costToBuildHullFactor);
		} else {
			return costToBuild;
		}
	}

	public void setUseBuildCostHullFactor(String v) {
		useBuildCostHullFactor = Comp.same(v, "true");
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("description", description);
		writer.addElement("cost-to-research", costToResearch);
		writer.addElement("cost-to-build", costToBuild);
		writer.addElement("use-build-cost-hull-factor", useBuildCostHullFactor);
		writer.addElement("maintenance-cost", maintenanceCost);
		writer.addElement("group", group);
	}

    public abstract String getTechType();

    public String toString() {
        return getFullName();
    }

    public String getFullName() {
        return getTechType() + Technologies.getSeperator() + getName();
    }

    public String getFullNameSpaceSeperated() {
        return getTechType() + " " + getName();
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }
    
	public String getImage() {
		return image;
	}

	public void setImage(String image) {
		this.image = image;
	}

	public String getGroup() { return group; }
	public void setGroup(String v) { group = v; }

    public Restriction getRestriction() { return restriction; }
    public void setRestriction(Restriction restriction) { this.restriction = restriction; }
    public void setMinResearchCostRestriction(MinCostRestriction restriction) { this.restriction = restriction; }
    public void setAlignmentRestriction(AlignmentRestriction restriction) { this.restriction = restriction; }
    public void setMandatoryRestriction(MandatoryRestriction restriction) { this.restriction = restriction; }

	public Integer getMaintenanceCost() { return maintenanceCost; }
	public void setMaintenanceCost(Integer v) { maintenanceCost = v; }

    /**
     * @param tech The technology to upgrade this to.
     * @return Returns the cost to upgrade this technology to technology tech.
     */
    public int getCostToUpgradeTo(Technology tech) {
        if (tech == null) return 0;
        int diff = tech.getCostToBuild() - getCostToBuild();
        if (diff < 0) diff = -diff;
        return tech.getCostToBuild() / 10 + diff;
    }

    public boolean isForbidEspionage() {
		return forbidEspionage;
	}

	public void setForbidEspionage(boolean forbidEspionage) {
		this.forbidEspionage = forbidEspionage;
	}

	/**
     * This is the amount by which the money spent to purchase production is 
     * higher than the cost to build the product.
     * If cost to build was 100 then cost to purchase would be 150 
     * @see Planet.getCostToPurchaseProject()
     */
    public static final double COST_TO_PURCHASE_FACTOR = 1.5;
    
    /**
     * This is the amount by which the money spent to purchase production is 
     * higher than the cost to build the product.
     * If cost to build was 100 then value would be 66.666
     * @see Craft.getValue()
     */
    public static final double COST_TO_SELL_FACTOR = 1.45;

    /**
     * @return The definition of this technology (fields name value in XML file name value pairs).
     */
	public abstract NameValuePair[] getAsData();

    
}