/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import mot.common.bean.NameList;
import mot.common.designs.CraftDesign;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.vo.TechnologyListVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * Generic list of technologies.....
 *
 */
public abstract class TechnologyList extends XMLBean implements Serializable {

	private transient Integer costToBuild = null;
	private transient Integer costToResearch = null;
	private transient Integer space = null;
	private transient Integer maintenanceCost = null;
	private transient Collection currentList = null;

	private ArrayList list = null;
    
    public TechnologyList() {
        
    }

    /**
     * Constructs a new technology list containing a single technology t.
     * @param t The technology to add to this list
     */
    public TechnologyList(Technology t) {
        if (t != null) {
            add(t);
        }
    }

    public TechnologyList(Technology t, int qty) {
		for (int i = 0; i<qty; i++) {
			add(t);
		}
    }

    /**
     * Performs a shallow clone on this technology list. This must be a shallow clone otherwise
     * Player.stealTechnology() will not work when it tries to get a list of all technology owned
     * by one player but not by another - It uses object equality to determine it two techs are the
     * same.
     * @param tl The list to copy - This new list will contain all the techs in tl.
     * @throws mot.common.util.ApplicationError
     */
    public TechnologyList(TechnologyList tl) {
        if (tl == null) return;
        Technology[] t = tl.getAllAsArray(true);
        if (t != null) {
            for (int i=0; i<t.length; i++) {
                add(t[i]);
            }
        }
    }
    
    /**
     * Performs a shallow clone on this technology list. This must be a shallow clone otherwise
     * Player.stealTechnology() will not work when it tries to get a list of all technology owned
     * by one player but not by another - It uses object equality to determine it two techs are the
     * same.
     * @param tl The list to topy - This new list will contain all the techs in tl.
     * @throws mot.common.util.ApplicationError
     */
    public TechnologyList(Technology[] t) throws ApplicationError {
        if (t != null) {
            for (int i=0; i<t.length; i++) {
                add(t[i]);
            }
        }
    }
    
    /**
     * Constructs a technology list from the list of technology names passed in. The
     * names must be full technology names and all technologies must exist in otherwise
     * an error occurs.
     * @param tl The full names of the technologies.
     * @throws mot.common.util.ApplicationError
     */
    public TechnologyList(String [] fullNames) throws ApplicationError {
        if (fullNames != null) {
            for (int i=0; i<fullNames.length; i++) {
                Technology tech = Technologies.getTechnology(fullNames[i]);
                add(tech);
            }
        }
    }
    
    public TechnologyList(TechnologyBundle[] t) {
        if (t != null) {
            for (int i=0; i<t.length; i++) {
                add(t[i].getTechnology(), t[i].getQty());
            }
        }
	}

	public void flushCache() {
		_changed();
		costToBuild = null;
		costToResearch = null;
        currentList = null;
        space = null;
    }

    /**
     * Add qty technologies v to the list. SO it v is mass driver and qty if 5 we will
     * add mass driver to the list 5 times (yes the list will have atleast five duplicate 
     * entries).
     * @param v The technology to add
     * @param qty The quantity to add
     */
	public void add(Technology v, int qty) {
		for (int i = 0; i<qty; i++) {
			add(v);
		}
	}
	
    /**
     * Adds a new technology item to the list of items. Note any cached values such as the 
     * total build cost for all item in the list are reset to null so the next time it is 
     * requested it must be recalculated. See method changed().
     */
	public void add(Technology v) {
        if (v == null) return;
		if (list == null) list = new ArrayList();
		list.add(v);
        flushCache();
	}

    /**
     * Adds all technology items in the list to this list of items. This uses the add(Technology)
     * method to add each technology.
     */
	public void add(TechnologyList v) {
        if (v == null) return;
        Technology[] techs = v.getAllAsArray(true);
        if (techs == null) return;
        for (int i=0; i<techs.length; i++) {
            add(techs[i]);
        }
	}

    /**
     * Adds all technology items in the list to this list of items. This uses the add(Technology)
     * method to add each technology.
     */
	public void add(Technology[] techs) {
        if (techs == null) return;
        for (int i=0; i<techs.length; i++) {
            add(techs[i]);
        }
	}
	
	public void removeAll(Technology[] techsToRemove) {
		if (techsToRemove == null) return;
		for (int i = 0; i < techsToRemove.length; i++) {
			removeAll(techsToRemove[i]);
		}
	}

    public void removeAll(Technology technology) {
		if (list == null || technology == null) return;
    	Technology[] ta = getAllAsArray();
    	if (ta == null) return;
    	removeAll();
    	for (int i = 0; i < ta.length; i++) {
			if (ta[i] != technology) {
				add(ta[i]);
			}
		}
        flushCache();
	}

	/**
     * Removes the first instance of technology v from this list. Note any cached values 
     * such as the total build cost for all item in the list are reset to null so the next 
     * time it is requested it must be recalculated. See method changed().
     */
	public void remove(Technology v) {
		if (list == null) return;
		list.remove(v);
        flushCache();
	}

    /**
     * Removes all technology items in the list to this list of items. This uses the remove(Technology)
     * method to remove each technology.
     */
	public void remove(TechnologyList v) {
        if (v == null) return;
        Technology[] techs = v.getAllAsArray(true);
        if (techs == null) return;
        for (int i=0; i<techs.length; i++) {
            remove(techs[i]);
        }
	}
    
	public void remove(TechnologyListVO v) throws ApplicationError {
        if (v == null) return;
        String[] techs = v.getFullNames();
        if (techs == null) return;
        for (int i=0; i<techs.length; i++) {
        	Technology t = Technologies.getTechnology(techs[i]);
            remove(t);
        }
	}
    
	public void remove(TechnologyBundle[] techs) {
		if (techs == null) return;
		for (int i = 0; i < techs.length; i++) {
			remove(techs[i].getTechnology(), techs[i].getQty());
		}
	}

    private void remove(Technology technology, int qty) {
		while (qty > 0) {
			remove(technology);
			qty--;
		}
	}

	/**
     * Removes all technology items in the list with group. This uses the remove(Technology)
     * method to remove each technology.
     */
	public void removeGroup(String group) {
        if (group == null) return;
        Technology[] techs = getAllWithGroup(group);
        if (techs == null) return;
        for (int i=0; i<techs.length; i++) {
            remove(techs[i]);
        }
	}
    
    /**
     * Remove all element from this list.
     */
    public void removeAll() {
        list = null;
        flushCache();
    }

	public void removeDuplicates() {
        Technology[] techs = getAllAsArray();
        if (techs == null) return;
        removeAll();
        for (int i=0; i<techs.length; i++) {
        	if (!contains(techs[i])) {
        		remove(techs[i]);
        	}
        }
	}

    /**
     * @return The list of technologies in the list that have not been made obsolete.
     */
	public Collection getCurrentList() {
        if (currentList == null && list != null && list.size() > 0) {
            Iterator i = list.iterator();
            HashMap map = new HashMap();
            while (i.hasNext()) {
                Technology v = (Technology)i.next();
                Technology o = (Technology)map.get(v.getGroup());
                if ( o != null ) {
                    if (o.getCostToResearch() < v.getCostToResearch()) {
                        map.remove(o.getGroup());
                        map.put(v.getGroup(), v);
                    }
                } else {
                    map.put(v.getGroup(), v);
                }
            }
            currentList = map.values();
        }
        return currentList;
    }
    
    /**
     * @return All technologies in the list.
     */
	public Collection getAll(boolean includeObsolete) {
        if (includeObsolete) {
            return list;
        } else {
           return getCurrentList(); 
        }
	}

    /**
     * @return All technologies in the list if obsolete includeObsolete is true otherwise
     * it returns getCurrentList().
     */
	public Technology[] getAllAsArray() {
        return getAllAsArray(true);
    }
    
    /**
     * @return All technologies in the list if obsolete includeObsolete is true otherwise
     * it returns getCurrentList().
     */
	public Technology[] getAllAsArray(boolean includeObsolete) {
        if (list == null) return null;
        if (includeObsolete) {
            return (Technology[])list.toArray(new Technology[(list.size())]);
        } else {
            Collection l = getAll(includeObsolete);
            if (l == null) return null;
            return (Technology[])l.toArray(new Technology[(l.size())]);
        }
	}

    public Technology[] getAllNoDuplicates() {
        Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        ArrayList<Technology> result = new ArrayList();
        for (int i=0; i<list.length; i++) {
            if (!result.contains(list[i])) {
                result.add(list[i]);
            }
        }
        if (result.size() == 0) {
        	return null;
        } else {
        	return result.toArray(new Technology[result.size()]);
        }
    }


    /**
     * @return All technologies in the list whose group is the same as the group
     * passed in.
     */
	public Technology[] getAllWithGroup(String group) {
        Collection l = getAll(true);
        if (l == null) return null;
        Iterator i = l.iterator();
        if (i == null) return null;
        ArrayList r = null;
        while (i.hasNext()) {
            Technology tech = (Technology)i.next();
            if (Comp.same(group, tech.getGroup())) {
                if (r == null) r = new ArrayList();
                r.add(tech);
            }
        }
        if (r == null) {
            return null;
        } else {
            return (Technology[])r.toArray(new Technology[(r.size())]);
        }
	}

    /**
     * @return All technologies in the list whose group is the same as the group
     * passed in.
     */
	public Technology[] getAllWithGroups(String [] groups) {
        Collection l = getAll(true);
        if (l == null) return null;
        Iterator i = l.iterator();
        if (i == null) return null;
        ArrayList r = null;
        while (i.hasNext()) {
            Technology tech = (Technology)i.next();
            if (Comp.in(tech.getGroup(), groups)) {
                if (r == null) r = new ArrayList();
                r.add(tech);
            }
        }
        if (r == null) {
            return null;
        } else {
            return (Technology[])r.toArray(new Technology[(r.size())]);
        }
	}

    /**
     * @return All technologies in the list whose group is the same as the group
     * passed in.
     */
	public TechnologyList getAllExcluding(TechnologyList exclude) {
        Collection l = getAll(true);
        if (l == null) return null;
        Iterator i = l.iterator();
        if (i == null) return null;
        TechnologyList r = null;
        while (i.hasNext()) {
            Technology tech = (Technology)i.next();
            if (!exclude.contains(tech)) {
                if (r == null) r = new GeneralTechnologyList();
                r.add(tech);
            }
        }
        return r;
	}

    /**
     * @return All technologies in the list whose group is the same as the group
     * passed in.
     */
	public TechnologyList getAllExcluding(TechnologyList exclude, int minResearchCost, int maxResearchCost) {
        Collection l = getAll(true);
        if (l == null) return null;
        Iterator i = l.iterator();
        if (i == null) return null;
        TechnologyList r = null;
        while (i.hasNext()) {
            Technology tech = (Technology)i.next();
            if (tech.getCostToResearch() > minResearchCost && tech.getCostToResearch() < maxResearchCost) {            	
            	if (!exclude.contains(tech)) {
            		if (r == null) r = new GeneralTechnologyList();
            		r.add(tech);
            	}
            }
        }
        return r;
	}

    /**
     * @return The technology in the list whose group is the same as the group
     * passed in and has the highest research cost in of technologies in this group.
     */
	public Technology getMostAdvancedWithGroup(String group) {
        Collection l = getAll(true);
        if (l == null) return null;
        Iterator i = l.iterator();
        if (i == null) return null;
        Technology r = null;
        while (i.hasNext()) {
            Technology tech = (Technology)i.next();
            if (Comp.same(group, tech.getGroup())) {
                if (r == null || r.getCostToResearch() < tech.getCostToResearch()) {
                    r = tech;
                }
            }
        }
        return r;
	}


    /**
     * @return The first technology in the list whose group is the same as the group
     * passed in. If none is found it throws an application error. This will NOT return 
     * null!
     * @throws ApplicationError 
     */
	public Technology getFirstInGroupMandatory(String group) throws ApplicationError {
        Collection l = getAll(true);
        if (l == null) throw new ApplicationError("TechnologyList.getWithGroupMandatory() No technology with group "+group);
        Iterator i = l.iterator();
        if (i == null) throw new ApplicationError("TechnologyList.getWithGroupMandatory() No technology with group "+group);
        while (i.hasNext()) {
            Technology tech = (Technology)i.next();
            if (Comp.same(group, tech.getGroup())) {
            	return tech;
            }
        }
        throw new ApplicationError("TechnologyList.getWithGroupMandatory() No technology with group "+group);
	}
	

	
    /**
     * Returns the first technology in the list whose name is the name passed in.
     * @param name The name of a type of technology to search for.
     * @return A technology item.
     */
	public Technology get(String name) {
		Collection list = getAll(true);
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( Comp.same(v.getName(), name) ) {
					return v;
				}
			}
		}
		return null;
	}

    public Technology getMostCommon() {
    	TechnologyBundle[] bundle = getDistinct();
    	if (bundle == null) return null;
    	TechnologyBundle best = null;
    	for (int i = 0; i < bundle.length; i++) {
			if (best == null || best.getQty() < bundle[i].getQty()) {
				best = bundle[i];
			}
		}
    	return best.getTechnology();
    }
    
    /**
     * To save space screen space/user effort weapons are displayed to the user as quantity, type pairs.
     * So a craft can have 100 lasers without a list of 100 lasers!
     * @return TechnologyBundle of technologies in this list where.
     */
    public TechnologyBundle[] getDistinct() {
        Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        HashMap map = new HashMap();
        ArrayList<TechnologyBundle> result = new ArrayList();
        for (int i=0; i<list.length; i++) {
            String name = list[i].getFullName();
            TechnologyBundle tb = (TechnologyBundle)map.get(name);
            if (tb == null) {
                tb = new TechnologyBundle(list[i]);
				map.put(name, tb);
                result.add(tb);
            } else {
                tb.qty++;
            }
        }
        if (result.size() == 0) {
        	return null;
        } else {
        	return result.toArray(new TechnologyBundle[result.size()]);
        }
    }

    public TechnologyBundle[] getDistinct(TechnologyList exclude) {
    	if (exclude == null) {
    		return getDistinct();
    	} else {
    		return getDistinct(exclude.getDistinct());
    	}
    }
    
    /**
     * e.g. We have 3xMass driver 1 and in exclude there is 2xMass Driver 1 => Result has 1xMass Driver 1
     * @return A list of technology and quantity pairs in this list with exclude extracted.
     */
    public TechnologyBundle[] getDistinct(TechnologyBundle [] exclude) {
    	TechnologyBundle[] tba = getDistinct();
    	if (tba == null || exclude == null) return tba;
    	
    	ArrayList<TechnologyBundle> result = new ArrayList();
    	for (int i = 0; i < tba.length; i++) {
			int index = _find(tba[i], exclude);
			if (index >= 0) {
				if (exclude[index].getQty() < tba[i].getQty()) {
					tba[i].setQty( tba[i].getQty()-exclude[index].getQty() );
					result.add(tba[i]);
				}
			} else {
				result.add(tba[i]);
			}
		}
    	
        if (result.size() == 0) {
        	return null;
        } else {
        	return result.toArray(new TechnologyBundle[result.size()]);
        }
    }
    
	public TechnologyBundle[] getDistinct(CraftDesign project) throws ApplicationError {
    	TechnologyBundle[] tba = getDistinct();
    	if (tba == null || project == null) return null;
    	
    	ArrayList<TechnologyBundle> result = new ArrayList();
    	for (int i = 0; i < tba.length; i++) {
			int qtyInCraftDesign = project.getQty(tba[i].getTechnology());
			int qtyInSalvage = tba[i].getQty();
			int qty = Comp.min(qtyInCraftDesign, qtyInSalvage) ;
			if (qty > 0) {
				result.add(new TechnologyBundle(tba[i].getTechnology(), qty));
			}
		}
    	
        if (result.size() == 0) {
        	return null;
        } else {
        	return result.toArray(new TechnologyBundle[result.size()]);
        }
	}


    private int _find(TechnologyBundle tb, TechnologyBundle[] tba) {
    	for (int i = 0; i < tba.length; i++) {
			if (tb.getTechnology() == tba[i].getTechnology()) {
				return i;
			}
		}
		return -1;
	}

	public TechnologyList get(int fromIndex, int toIndex) {
        Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        if (list.length <= fromIndex) return null;
        if (list.length <= toIndex) toIndex = list.length-1;
        TechnologyList r = new GeneralTechnologyList();
        for (int i = fromIndex; i < toIndex; i++) {
			r.add(list[i]);
		}
        return r;
    }
    
	public Technology get(int index) {
        Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        if (list.length <= index) return null;
		return list[index];
    }
    
    /**
     * @param tech The technology to upgrade every item in this list to.
     * @return Returns the cost to upgrade all items in this list to technology tech.
     */
    public int getCostToUpgradeTo(Technology tech) {
        int r = 0;
		Collection list = getAll(true);
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
                Technology t = (Technology)i.next();
                r += t.getCostToUpgradeTo(tech);
            }
        }
        return r;
    }
    
    /**
     * @return Return true if this list's best technology of group has a lowest cost to research than costToResearch passed in.
     */
	public boolean needsUpgrading(String group, int costToResearch) {
        int r = 0;
		Collection list = getAll(true);
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
                Technology t = (Technology)i.next();
                if (t.getCostToResearch() >= costToResearch) {
                	return false;
                }
            }
        }
		return true;
	}

    
    /**
     * Upgrades all technologies in this list to the technology tech.
     * If this list contains 2 MassDriver1 and 1 Laser and tech was Fusion Beam
     * then this list would contains 3 Fusion Beams.
     * @param tech The technology to upgrade to.
     */
    public void upgradeTo(Technology tech) {
        int qty = 0;
        if (list != null) {
            qty = list.size();
            list.clear();
        }
        for (int i=0; i<qty; i++) {
            add(tech);
        }
    }

    /**
     * Upgrades all technologies of type from in this list to the technology to.
     * If this list contains 2 MassDriver1 and 1 Laser and tech was Fusion Beam
     * then this list would contains 3 Fusion Beams.
     * @param fromTech The technology to upgrade from.
     * @param toTech The technology to upgrade to.
     */
    public void upgradeTo(Technology fromTech, Technology toTech) {
        if (fromTech == toTech) {
            Logger.logWarning("TechnologyList.upgradeTo() from and to techs are the same");
            return;
        }
		Collection col = getAll(true);
		if (col != null) {
			Iterator i = list.iterator();
            removeAll();
			while (i.hasNext()) {
                Technology t = (Technology)i.next();
                if (t == fromTech) {
                    add(toTech);
                } else {
                    add(t);
                }
            }
        }
    }

    /**
     * @return The quantity of items in this list.
     */
    public int getQuantity() {
        if (list == null) {
            return 0;
        } else {
            return list.size();
        }
    }
    
    /**
     * @return The quantity of items in this list of type tech.
     */
    public int getQuantity(Technology tech) {
        int qty = 0;
		Collection list = getAll(true);
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
                Technology t = (Technology)i.next();
                if (t == tech) {
                    qty++;
                }
            }
        }
        return qty;
    }
    
	public int getQuantityWithGroup(String group) {
        int qty = 0;
		Collection list = getAll(true);
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
                Technology t = (Technology)i.next();
                if (Comp.same(group, t.getGroup())) {
                    qty++;
                }
            }
        }
        return qty;
	}



    /**
     * @return The quantity of technologies in the list that are also in the list ol passed in.
     */
	public int getQuantityFrom(TechnologyList ol) {
		if (ol == null) return 0;
		Technology[] ta = getAllAsArray();
		if (ta == null) return 0;
		int r = 0;
		for (int i = 0; i < ta.length; i++) {
			if (ol.contains(ta[i])) r++;
		}
		return r;
	}
    
    /**
     * @return The technology in this list at the index passed in. 
     * @param index Index of the technology to fetch (value between 0 and size - 1).
     */
    public Technology getTechnologyAtIndex(int index) {
        if (list == null || index >= list.size()) {
            return null;
        } else {
            return (Technology)list.get(index);
        }
    }
    
    /**
     * @return The technology in this list at the index passed in or -1 if it does not exist. 
     * @param index Index of the technology to fetch (value between 0 and size - 1).
     */
    public int getIndexOfTechnology(Technology tech) {
        if (list == null) {
            return -1;
        } else {
            return list.indexOf(tech);
        }
    }
    
	/**
	 * The cost to build this piece of technology.
	 * The parameter costToBuildHullFactor is the percent that the cost to build this technology
	 * increases due to the ship hull (e.g. engines cost more as ships get larger).
	 */
	public int getCostToBuild(double costToBuildHullFactor) {
		if (costToBuild == null) {
			Collection list = getAll(true);
			if (list == null) {
				costToBuild = new Integer(0);
			} else {
				Iterator i = list.iterator();
                int cost = 0;
				while (i.hasNext()) {
					Technology v = (Technology)i.next();
					cost += v.getCostToBuild(costToBuildHullFactor);
				}
				costToBuild = new Integer(cost);
			}
		}
		return costToBuild.intValue();
	}

	/**
	 * The cost to research this list of technology.
	 */
	public int getCostToResearch() {
		if (costToResearch == null) {
			Collection list = getAll(true);
			if (list == null) {
				costToResearch = new Integer(0);
			} else {
				Iterator i = list.iterator();
                int cost = 0;
				while (i.hasNext()) {
					Technology v = (Technology)i.next();
					cost += v.getCostToResearch();
				}
				costToResearch = new Integer(cost);
			}
		}
		return costToResearch.intValue();
	}

	/**
	 * The space consumed by this piece of technology.
	 */
	public int getSpace() {
		if (space == null) {
			Collection list = getAll(true);
			if (list == null) {
				space = new Integer(0);
			} else {
				Iterator i = list.iterator();
                int cost = 0;
				while (i.hasNext()) {
					Technology v = (Technology)i.next();
					cost += v.getSpace();
				}
				space = new Integer(cost);
			}
		}
		return space.intValue();
	}

	/**
	 * The total maintenance cost for all tech items in this list.
	 */
	public int getMaintenanceCost() {
		if (maintenanceCost == null) {
			Collection list = getAll(true);
			if (list == null) {
				maintenanceCost = new Integer(0);
			} else {
				Iterator i = list.iterator();
                int cost = 0;
				while (i.hasNext()) {
					Technology v = (Technology)i.next();
                    if (v.getMaintenanceCost() != null) {
                        cost += v.getMaintenanceCost().intValue();
                    }
				}
				maintenanceCost = new Integer(cost);
			}
		}
		return maintenanceCost.intValue();
	}

    /**
     * @return The best technology in this list
     */
	protected Technology getBest() {
		if (list == null || list.size() == 0) {
			return null;
		} else {
			return (Technology)list.get(list.size() - 1);
        }
	}
	
	public Technology getRandom() {
		if (list == null || list.size() == 0) return null;
        return (Technology) Utils.random(list);
	}

	/**
	 * @return true if an only if ANY technology in the list tl is contained in this list.
	 */
	public boolean containsAny(TechnologyList tl) {
        if (tl == null) return false;
		Collection c = tl.getAll(true);
		if (c != null) {
			Iterator i = c.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( contains(v) ) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @return true if an only if all technology in the list tl are contained in this list.
	 */
	public boolean contains(TechnologyList tl) {
        if (tl == null) return true;
		Collection c = tl.getAll(true);
		if (c != null) {
			Iterator i = c.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( !contains(v) ) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * @return true if an only if all technology in the list tl are contained in this list.
	 */
	public boolean contains(Technology [] tl) {
        if (tl == null) return true;
        for (int i = 0; i < tl.length; i++) {
			Technology v = tl[i];
			if ( !contains(v) ) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @return true if an only if all technologies in all lists tl are contained in this list.
	 */
	public boolean contains(TechnologyList[] tl) {
		if (tl != null) {
            for (int i=0; i<tl.length; i++) {
                if (!contains(tl[i])) {
                    return false;
                }
            }
		}
		return true;
	}

	/**
	 * @return true if an only if this list contains technologies other than the one passed in.
	 */
	public boolean containsOther(Technology t) {
        if (t == null) return false;
		Collection c = getAll(true);
		if (c != null) {
			Iterator i = c.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( t != v ) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @return true if an only if this technology is contained in this list.
	 */
	public boolean contains(Technology t) {
        if (t == null) return false;
		return contains(t.getName());
	}

	/**
	 * Returns true if an only if a technology with this name is contained in this list.
	 */
	public boolean contains(String name) {
		Collection list = getAll(true);
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( Comp.same(v.getName(), name) ) {
					return true;
				}
			}
		}
		return false;
	}
    
    /**
     * @param group The group this list may contain
     * @return Returns true if this list contains one or more technologies from this group.
     */
    public boolean containsGroup(String group) {
		Collection list = getAll(true);
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( Comp.same(v.getGroup(), group) ) {
					return true;
				}
			}
		}
		return false;
    }
    
    /**
     * @return True if this collection contains one or more copies of the same technology. 
     */
    public boolean containsDuplicates() {
        Technology [] all = getAllAsArray(true);
        if (all == null) return false;
        for (int i=0; i<all.length; i++) {
            for (int j=0; j<all.length; j++) {
                if (i != j && all[i] == all[j]) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * @param group The group this list may contain
     * @return Returns the first technology it finds in the list that is from this group.
     */
    public Technology getTechnologyForGroup(String group) {
		Collection list = getAll(true);
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( Comp.same(v.getGroup(), group) ) {
					return v;
				}
			}
		}
		return null;
    }
    
	public String[] getGroups() {
		Collection list = getAll(true);
		if (list == null || list.size() == 0) {
			return null;
		}
		ArrayList<String> r = null;
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Technology v = (Technology)i.next();
			String group = v.getGroup();
			if ( !Comp.in(group, r) ) {
				if (r == null) r = new ArrayList<String>();
				r.add(group);
			}
		}
		if (r == null) {
			return null;
		} else {
			return r.toArray(new String[r.size()]);
		}
	}



	/**
	 * @return true if an only if these two lists contain the same data.
	 */
	public boolean same(TechnologyList tl) {
        if (tl == null) return false;
        if (tl == this) return true;
        return same(tl.getAllAsArray());
	}

	/**
	 * @return true if an only if these two lists contain the same data.
	 */
	public boolean same(Technology [] ta) {
        if (ta == null) return false;
        if (ta.length != getQuantity()) return false;
        for (int i = 0; i < ta.length; i++) {
			if (!contains(ta[i])) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Notify sub classes to recalculate cached values as list has changed.
	 */
	protected abstract void changed();
    
    private void _changed() {
        costToBuild = null;
        costToResearch = null;
        space = null;
        changed();
    }

    public int getHighestResearchValueBelow() {
        return getHighestResearchValueBelow(Integer.MAX_VALUE);
    }
    
    public int getHighestResearchValueBelow(int min) {
		Collection list = getAll(true);
        int best = Integer.MAX_VALUE;
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( v.getCostToResearch() < min ) {
                    if (best == Integer.MAX_VALUE || best < v.getCostToResearch()) { 
                        best = v.getCostToResearch();
                    }
				}
			}
		}
		return (Integer.MAX_VALUE == best) ? -1 : best;
    }
   
    public int getLowestResearchValueAbove() {
        return getLowestResearchValueAbove(-1);
    }
    
    public int getLowestResearchValueAbove(int min) {
		Collection list = getAll(true);
        int best = Integer.MAX_VALUE;
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				if ( v.getCostToResearch() > min ) {
                    if (best == Integer.MAX_VALUE || best > v.getCostToResearch()) { 
                        best = v.getCostToResearch();
                    }
				}
			}
		}
		return (Integer.MAX_VALUE == best) ? -1 : best;
    }
   

    /**
     * Returns an array of all technology that have the same research cost that is higher than 
     * the parameter min. The cost will be the closest cost of any technology above the value min.
     * The matching technologies will be added the the ArrayList passed in and this array list is 
     * returned as the result. If null is passed in as the ArrayList it creates a new ArrayList.
     * @return The list of technologies results passed in as parameter 'results' with a new technologies added
     */
    public GeneralTechnologyList getCheapestToResearch(int min, GeneralTechnologyList results) {
		Technology[] list = getAllAsArray(true);
        if (list == null) return results;
        
        int best = Integer.MAX_VALUE;
        
        if (results != null && results.getQuantity() > 0) {
            best = results.getTechnologyAtIndex(0).getCostToResearch();
        }

        for (int i=0; i<list.length; i++) {
            Technology v = list[i];
            if ( v.getCostToResearch() >= min ) {
                if (best == Integer.MAX_VALUE || best > v.getCostToResearch()) {
                    if (results != null) results.removeAll();
                    best = v.getCostToResearch();
                }
            }
        }

        if (Integer.MAX_VALUE == best) {
            return results;
        }

        for (int i=0; i<list.length; i++) {
            Technology v = list[i];
            if ( v.getCostToResearch() == best ) {
                if (results == null) results = new GeneralTechnologyList();
                results.add(v);
            }
        }
        
		return results;
    }
    
    public Technology getCheapestToResearch() {
		Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        
        Technology result = null;
        int best = Integer.MAX_VALUE;
        
        for (int i=0; i<list.length; i++) {
            Technology v = list[i];
            int c = v.getCostToResearch();
			if ( result == null || c < best ) {
                best = c;
                result = v;
            }
        }
       
		return result;
    }

    public Technology getMostExpensiveToResearch() {
		Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        
        Technology result = null;
        int best = Integer.MAX_VALUE;
        
        for (int i=0; i<list.length; i++) {
            Technology v = list[i];
            int c = v.getCostToResearch();
			if ( result == null || c > best ) {
                best = c;
                result = v;
            }
        }
       
		return result;
    }
    

    public String getNames() {
        return getNames("\n");
    }
    
    public String getNames(String seperator) {
		Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        StringBuffer sb = new StringBuffer();
        boolean first = true;
        for (int i=0; i<list.length; i++) {
            if (first) {
                first = false;
            } else {
                sb.append(seperator);
            }
            sb.append(list[i].getName());
        }
        return sb.toString();
    }
    
    public String getDescriptions() {
        return getDescriptions("\n");
    }
    
    public String getDescriptions(String seperator) {
		Technology[] list = getAllAsArray(true);
        if (list == null) return null;
        StringBuffer sb = new StringBuffer();
        boolean first = true;
        for (int i=0; i<list.length; i++) {
            if (first) {
                first = false;
            } else {
                sb.append(seperator);
            }
            sb.append(list[i].getDescription());
        }
        return sb.toString();
    }
    
    public ArrayList<String> getNamesAsList() {
		Technology[] list = getAllAsArray(true);
        if (list == null || list.length == 0) return null;
        ArrayList<String> result = new ArrayList<String>();
        for (int i=0; i<list.length; i++) {
            result.add(list[i].getName());
        }
        return result;
    }
    
    public NameList getNamesList() {
		Technology[] list = getAllAsArray(true);
        if (list == null || list.length == 0) return null;
        return new NameList(this);
    }

    public NameList getFullNamesList() {
		Technology[] list = getAllAsArray(true);
        if (list == null || list.length == 0) return null;
        NameList result = new NameList();
        for (int i=0; i<list.length; i++) {
            result.add(list[i].getFullName());
        }
        return result;
    }

    public String [] getNamesAsArray() {
		Technology[] list = getAllAsArray(true);
        if (list == null || list.length == 0) return null;
        String [] result = new String[list.length];
        for (int i=0; i<list.length; i++) {
            result[i] = list[i].getName();
        }
        return result;
    }
    
    public String [] getFullNamesAsArray() {
		Technology[] list = getAllAsArray(true);
        if (list == null || list.length == 0) return null;
        String [] result = new String[list.length];
        for (int i=0; i<list.length; i++) {
            result[i] = list[i].getFullName();
        }
        return result;
    }
    
    public Vector getListModel() {
        Technology[] list = getAllAsArray(true);
        Vector data = new Vector();
        if (list != null) {
            for (int i=list.length-1;i>=0; i--) {
                data.add(list[i].getName());
            }
        }
        return data;
    }

    public Vector getListModel(String [] groups) {
        Technology[] list = getAllAsArray(true);
        Vector data = new Vector();
        if (list != null) {
            for (int i=list.length-1;i>=0; i--) {
                if (Comp.in(list[i].getGroup(), groups)) {
                    data.add(list[i].getName());
                }
            }
        }
        return data;
    }

    public Vector getListModelResearchCostMoreThan(Technology tech) {
        Technology[] list = getAllAsArray(true);
        Vector data = new Vector();
        for (int i=0; list != null && i<list.length; i++) {
            if (list[i].getCostToResearch() > tech.getCostToResearch()) {
                data.add(list[i].getName());
            }
        }
        return data;
    }

    public Vector getDistrictListModel() {
        TechnologyBundle[] list = getDistinct();
        Vector data = new Vector();
        for (int i=0; list != null && i<list.length; i++) {
            if (list[i].getQty() > 1) {
                data.add(list[i].getName()+"x"+list[i].getQty());
            } else {
                data.add(list[i].getName());
            }
        }
        return data;
    }

    public String [] getDistinctList() {
        TechnologyBundle[] list = getDistinct();
        ArrayList data = new ArrayList();
        for (int i=0; list != null && i<list.length; i++) {
            if (list[i].getQty() > 1) {
                data.add(list[i].getName()+"x"+list[i].getQty());
            } else {
                data.add(list[i].getName());
            }
        }
        if (data.size() == 0) {
        	return null;
        } else {        	
        	return (String [])data.toArray(new String[data.size()]);
        }
    }

    public void applyRestrictions(TechnologyOwner owner) throws ApplicationError {
        Technology[] list = getAllAsArray(true);
        if (list != null) {
            for (int i=0; i<list.length; i++) {
                if (list[i].getRestriction() != null) {
                    list[i].getRestriction().apply(list[i], this, owner);
                }
            }
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				Technology v = (Technology)i.next();
				v.toXML(writer);
			}
		}
	}
    
    
    
    // Rather than using Technology reference everywhere or cloning a technology list,
    // you can serialise a reference to a list of technologies and this class has the 
    // smarts to de serialise into the original list.
    
    private static final String SEPERATOR = "_%_";
    
    public void referenceToXML(XMLWriter writer) throws ApplicationError {
        referenceToXML(writer, getElementName());
    }
    
    public void referenceToXML(XMLWriter writer, String elementName) throws ApplicationError {
        writer.openElement(elementName, getAttributes(writer.generateClassAttribute(), writer.generateSchemaAttributes()), false);
		Technology[] list = getAllAsArray(true);
        for (int i=0; list != null && i<list.length; i++) {
            String v = list[i].getTechType() + SEPERATOR + list[i].getName();
            writer.addElement("reference-name", v);
        }
        writer.closeElement(elementName);
    }
    
    public void setReferenceName(String ref) throws ApplicationError {
        String type = ref.substring(0, ref.indexOf(SEPERATOR));
        String name = ref.substring(ref.indexOf(SEPERATOR)+SEPERATOR.length());
        add(Technologies.getTechnology(type, name));
    }
    
    public void sortByName() {
        if (list == null) return;
        String [] names = getNamesAsArray();
        Arrays.sort(names);
        ArrayList newList = new ArrayList();
        for (int i=0; i<names.length; i++) {
            Technology next = get(names[i]);
            remove(next);
            newList.add(next);            
        }
        list = newList;
        flushCache();
    }
    
    public String toString() {
        return getNames(", ");
    }

	public TechnologyList getEspionageForbiddenTechnologies() {
        Collection l = getAll(true);
        if (l == null) return null;
        Iterator i = l.iterator();
        if (i == null) return null;
        GeneralTechnologyList r = null;
        while (i.hasNext()) {
            Technology tech = (Technology)i.next();
            if (tech.isForbidEspionage()) {
                if (r == null) r = new GeneralTechnologyList();
                r.add(tech);
            }
        }
        return r;
	}

}