/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.io.File;
import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.xml.DomFactory;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLDocumentHandler;
import mot.common.xml.XMLWriter;

/**
 * This maintains a dependency tree of technologies in the Game.
 * Before discovering most technologies you will need to discover pre-requisites
 * these are called parents in the tree.
 * Nodes with no parents are called starting nodes and are those initially available 
 * for research.
 */
public class TechnologyTree extends XMLBean {

	private static final String ROOT_ELEMENT_NAME = "tech-tree";
	private static final String NODE_ELEMENT_NAME = "tech-tree-node";

	private static final String FILE_NAME = "xml/tech_tree.xml";
	
	/**
	 * This is the technology tree that is only loaded once.
     * This should be accessed using the singleton pattern.
	 */
	private static TechnologyTree tree;

    /**
     * Generates a list of technologies to be discovered in order of cheapest to research to most 
     * expensive.
     */
	public static synchronized TechnologyTree getInstance() throws ApplicationError {
		if (tree == null) {
			File file = new File("updates/tech_tree.xml");
			if (file.exists()) {
				tree = (TechnologyTree)XMLDocumentHandler.parseFile("updates/tech_tree.xml", true, "xml/tech_tree.xsd");
			} else {
				tree = (TechnologyTree)XMLDocumentHandler.parseResource(FILE_NAME);
			}
		}
		return tree;
	}
    
    public static void flush() {
        tree = null;
    }
    
	public static void preLoad() throws ApplicationError {
		tree = new TechnologyTree(DomFactory.load(FILE_NAME));

	}


    public static synchronized void setInstance(TechnologyTree v) { tree = v; }


    /**
     * The list of nodes in this tree. This is a list of TechnologyTreeNode objects.
     */
    private ArrayList nodes;
    private transient TechnologyTreeNode [] nodesArray;
    
    public TechnologyTree() {
    }

    public TechnologyTree(TechnologyTree other) {
    	nodesArray = null;
    	TechnologyTreeNode[] na = other.getNodesAsArray();
    	if (na != null) {
    		for (int i = 0; i < na.length; i++) {
				setNode(na[i]);
			}
    	}
	}

	public TechnologyTree(DomTree doc) throws ApplicationError {
    	DomNode root = doc.getNodeByName(ROOT_ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(NODE_ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setNode(new TechnologyTreeNode(n[i]));
			}
		}
	}

	public ArrayList getNodes() {
        return nodes;
    }

    public TechnologyTreeNode [] getNodesAsArray() {
        if (nodesArray == null && nodes != null) {
            nodesArray = (TechnologyTreeNode[])nodes.toArray(new TechnologyTreeNode[(nodes.size())]);
        }
        return nodesArray;
    }

    public void setNodes(ArrayList nodes) {
        this.nodes = nodes;
        nodesArray = null;
    }

    public void setTechTreeNode(TechnologyTreeNode node) {
        setNode(node);
    }
    
    public void setNode(TechnologyTreeNode node) {
        if (nodes == null) nodes = new ArrayList();
        nodes.add(node);
        nodesArray = null;
    }
    
    public void removeNode(Technology t) {
        if (nodes == null || t == null) {
            return;
        }
        TechnologyTreeNode[] nodes = getNodesAsArray();
        for (int i=0; i<nodes.length; i++) {
            TechnologyTreeNode node = nodes[i];
            if (node.getNode() == t) {
                this.nodes.remove(node);
            }
        }
    	nodesArray = null;
    }
    
    public TechnologyTreeNode[] getParentNodesAsArray(TechnologyTreeNode node) throws ApplicationError {
    	Technology[] r = node.getParentsAsArray();
    	if (r == null || r.length == 0) {
    		return null;
    	} else {
    		TechnologyTreeNode[] nr = new TechnologyTreeNode[r.length];
    		for (int i = 0; i < nr.length; i++) {
				nr[i] = getNode(r[i]);
			}
    		return nr;
    	}
    }


    private boolean discoveredOrExcluded(TechnologyOwner player, Technology parent) throws ApplicationError {
		Technology tech = (Technology) parent;
        if (player.hasDiscovered(tech)) {
            return true;
        }
        TechnologyList exclude = player.getMissedTechnologies();
        return (exclude != null && exclude.contains(tech));
    }
    
    private boolean allDiscoveredOrExcluded(TechnologyOwner player, Technology [] parents) throws ApplicationError {
        if (parents == null) return true;
        for (int i=0; i<parents.length; i++) {
            if (!discoveredOrExcluded(player, parents[i])) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * @return the list of techs that can be discovered using technology tech 
     * passed in and has not been discovered by the player passed in.
     */
    public Technology [] getTechnologyAvailableForResearch(TechnologyOwner player) throws ApplicationError {
         TechnologyTreeNode [] nodes = getNodesAsArray();
         GeneralTechnologyList r = new GeneralTechnologyList();
         TechnologyList exclude = player.getMissedTechnologies();
         
         if (nodes == null) {
             return null;
         }
         
         for (int i=0; i<nodes.length; i++) {
             if (isAvailableForResearch(player, nodes[i], exclude)) {
            	 r.add(nodes[i].getNode());
             }
         }
         
         r.applyRestrictions(player);
         
         if (r.getQuantity() == 0) {
             return null;
         } else {
            return r.getAllAsArray(true);
         }
    }

	private boolean isAvailableForResearch(TechnologyOwner player, TechnologyTreeNode node, TechnologyList exclude) throws ApplicationError {
		 Technology tech = node.getNode();
		 if (player.hasDiscovered(tech)) {
		     // This player has already discovered this tech
		 } else if (allDiscoveredOrExcluded(player, node.getParentsAsArray())) {
		     // This player has not already discovered this tech but has 
		     // discovered all techs its research is dependent on.
		     if (exclude == null || !exclude.contains(tech)) {
		    	 // This technology is not in the player's exclude list
		    	 if (node.getCondition() != null) {
		    		 return node.getCondition().isSatisfied(player, exclude);
		    	 } else {		    		 
		    		 return true;
		    	 }
		     }
		 }
		 return false;
	}

    /**
     * @return All technologies that are missed for for the player visible at this point in time (you can only see a 
     * tech is missing once it should be available for research.
     * More formally it is defined as - All the technologies that this player could research now (dependencies have been
     * satisfied - see technology tree for details) but they are on the missed list.
     */
    public Technology [] getVisibleMissedTechnology(TechnologyOwner player) throws ApplicationError {
        TechnologyList r =  getVisibleMissedTechnologyAsList(player);        
        if (r == null || r.getQuantity() == 0) {
            return null;
        } else {
           return r.getAllAsArray(true);
        }
    }
    
    /**
     * @return All technologies that are missed for for the player visible at this point in time (you can only see a 
     * tech is missing once it should be available for research.
     * More formally it is defined as - All the technologies that this player could research now (dependencies have been
     * satisfied - see technology tree for details) but they are on the missed list.
     */
    public TechnologyList getVisibleMissedTechnologyAsList(TechnologyOwner player) throws ApplicationError {
        TechnologyList include = player.getMissedTechnologies();
        if (include == null || include.getQuantity() == 0) {
        	return null;
        }

        Technology [] candidates = include.getAllAsArray();
        GeneralTechnologyList r = new GeneralTechnologyList();
        
        if (nodes == null) {
            return null;
        }
        
        for (int i=0; i<candidates.length; i++) {
            Technology tech = candidates[i];
            TechnologyTreeNode node = getNode(tech);
            if (node == null) {
            	//All the ancient and fake techs are not in the tech tree - they should be ignored!
            	//Logger.logWarning("**** Technology "+tech+" does not have a Tech Tree Node");
            } else if (isAvailableForResearch(player, node, null)) {
                r.add(tech);
            }
        }
        
        r.applyRestrictions(player);
        
        if (r.getQuantity() == 0) {
            return null;
        } else {
           return r;
        }
    }
    
    public Technology [] getRecentlyMissedTechnology(Technology [] missedTechs, TechnologyOwner player) throws ApplicationError {
    	Theories theories = player.getTheories();
    	if (theories == null) return null;
    	Technology currentLevel = theories.getMostExpensiveToResearch();
    	if (currentLevel == null) return null;
    	
       	if (missedTechs == null) return null;

        GeneralTechnologyList r = new GeneralTechnologyList();
        
        for (int i=0; i<missedTechs.length; i++) {
            TechnologyTreeNode node = getNode(missedTechs[i]);
            if (node == null) {
            	Logger.logWarning("**** Technology "+missedTechs[i]+" does not have a Tech Tree Node");
            } else if (node.containsParent(currentLevel)) {
                r.add(missedTechs[i]);
            }
        }
        
        r.applyRestrictions(player);
        
        if (r.getQuantity() == 0) {
            return null;
        } else {
           return r.getAllAsArray(true);
        }
    }
    
    /**
     * @return the list of all technologies in this technology tree.
     */
    public Technology [] getTechnologies() throws ApplicationError {
         TechnologyTreeNode [] nodes = getNodesAsArray();
         GeneralTechnologyList r = new GeneralTechnologyList();
         
         if (nodes == null) {
             return null;
         }
         
         for (int i=0; i<nodes.length; i++) {
             TechnologyTreeNode node = nodes[i];
             Technology tech = node.getNode();
             r.add(tech);
         }
         
         if (r.getQuantity() == 0) {
             return null;
         } else {
            return r.getAllAsArray(true);
         }
    }

	public boolean contains(String tech) throws ApplicationError {
		return getNode(tech) != null;
	}
	
	public TechnologyTreeNode getNode(String tech) throws ApplicationError {
        if (tech == null) return null;
        TechnologyTreeNode [] nodes = getNodesAsArray();
        
        for (int i=0; i<nodes.length; i++) {
            TechnologyTreeNode node = nodes[i];
            if (node == null || node.getNode() == null) {
                throw new ApplicationError("TechnologyTree.getNode() Have NULL node for node after "+nodes[i-1].getNode().getFullName()+" at "+i+" CHECK tech_tree.xml contains valid technology name");
            }
            if ( Comp.same(node.getNode().getFullName(), tech) ) {
                return node;
            }
        }
        
        return null;
	}

    /**
     * @return the node of the tech passed in.
     */
    public TechnologyTreeNode getNode(Technology tech) throws ApplicationError {
        if (tech == null) return null;
         TechnologyTreeNode [] nodes = getNodesAsArray();
         
         for (int i=0; i<nodes.length; i++) {
             TechnologyTreeNode node = nodes[i];
             if (node == null || node.getNode() == null) {
                 throw new ApplicationError("TechnologyTree.getNode() Have NULL node for node after "+nodes[i-1].getNode().getFullName()+" at "+i+" CHECK tech_tree.xml contains valid technology name");
             }
             if ( node.getNode() == tech || Comp.same(node.getNode().getFullName(), tech.getFullName()) ) {
                 return node;
             }
         }
         
         return null;
    }

    /**
     * @return All technologies that are directly dependent on this technology in the tree.
     */
	public TechnologyList getChildren(Technology tech) {
        if (tech == null) return null;
        TechnologyTreeNode [] nodes = getNodesAsArray();
        TechnologyList r = new GeneralTechnologyList();
        for (int i=0; i<nodes.length; i++) {
            TechnologyTreeNode node = nodes[i];
            if ( node.containsParent(tech) ) {
                r.add(node.getNode());
            }
        }
        return r;
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	BasicNameValueList nvl = new UnsortedNameValuePairList();
        nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
        nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_tree.xsd") );
        return nvl;
    }
    
	protected String getElementName() {
		return ROOT_ELEMENT_NAME;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		TechnologyTreeNode[] nodes = getNodesAsArray();
        if (nodes == null) return;
        
		for (int i=0; i<nodes.length; i++) {
            TechnologyTreeNode node = nodes[i];
            node.toXML(writer);
        }
	}

	public int getQuantity() {
		if (nodes != null) {
			return nodes.size();
		} else {
			return 0;
		}
	}

	public static void removeUpdates() {
		try {
			File file = new File("updates/tech_tree.xml");
			if (file.exists()) file.delete();
		} catch (Exception e) {
			Logger.logError("FAIL", e);
		}
	}


}