/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.DomNode;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * A node in the TechnologyTree.
 * This class is designed to be de-serialised from XML so has set methods with String paremeters.
 */
public class TechnologyTreeNode extends XMLBean {

    /**
     * This node.
     */
    private Technology node;

    /**
     * The technology this node depends on - List of Technology.
     * Things that must be discovered to research this technology.
     */
    private ArrayList  parents;
    
    private transient Technology[] parentsArray;
    
    /**
     * If researching this technology has a condition attached to it.
     */
    private TechnologyTreeCondition condition;

    public TechnologyTreeNode() {
    }

    public TechnologyTreeNode(Technology node) {
    	this(node, (Technology [])null);
    }
    
    public TechnologyTreeNode(Technology node, TechnologyList parents, TechnologyTreeCondition condition) {
    	this(node, parents == null ? null : parents.getAllAsArray());
		this.condition = condition;
    }
    
    public TechnologyTreeNode(Technology node, TechnologyList parents) {
    	this(node, parents == null ? null : parents.getAllAsArray());
    }
    
    public TechnologyTreeNode(Technology node, Technology [] parents) {
		super();
		this.node = node;
		this.parents = new ArrayList();
		
		if (parents != null) {
			for (int i = 0; i < parents.length; i++) {
				this.parents.add(parents[i]);
			}
		}
	}

    public TechnologyTreeNode(DomNode ttn) throws ApplicationError {
    	node = ttn.getNodeValueByName_Technology("node");
    	
    	setParent(ttn.getNodeValuesByName_Technology("parent"));
    	
    	DomNode pc = ttn.getNodeByName("condition-percentage");
    	if (pc != null) setCondition(new TechnologyTreeConditionPercentage(pc));
	}

	public TechnologyTreeNode clone() {
    	TechnologyTreeNode r = new TechnologyTreeNode();
    	r.setNode(getNode());
    	Technology[] pa = getParentsAsArray();
		if (pa != null) {
			for (int i = 0; i < pa.length; i++) {
				r.setParent(pa[i]);
			}
		}
		return r;
    }

	public Technology getNode() {
        return node;
    }

    public void setNode(Technology node) {
        this.node = node;
    }

    public void setNode(String node) throws ApplicationError {
        this.node = Technologies.getTechnology(node);
        if (this.node == null) {
            Logger.logWarning("TechnologyTree.setNode() node = "+node+" Cannot find technology");
        }
    }


    public ArrayList getParents() {
        return parents;
    }

    public Technology[] getParentsAsArray() {
        if (parentsArray == null && parents != null) {
            parentsArray = (Technology[])parents.toArray(new Technology[(parents.size())]);
        }
        return parentsArray;
    }


    public void setParents(ArrayList parents) {
        this.parents = parents;
        parentsArray = null;
    }

    public void setParent(String parent) throws ApplicationError {
        if (parents == null) parents = new ArrayList();
        Technology tech = Technologies.getTechnology(parent);
        if (tech == null) {
        	Logger.logWarning("No tech "+parent);
        	return;
        }
		parents.add(tech);
        parentsArray = null;
    }

    public void setParent(Technology parent) {
        if (parents == null) parents = new ArrayList();
        parents.add(parent);
        parentsArray = null;
    }
    
    public void setParent(Technology[] pa) {
    	if (pa != null) {
	        if (parents == null) parents = new ArrayList();
	        for (int i = 0; i < pa.length; i++) {				
	        	parents.add(pa[i]);
			}
	        parentsArray = null;
    	}
    }
    
    
    public String toString() {
    	return node.toString();
    }

	public TechnologyTreeCondition getCondition() {
		return condition;
	}

	public void setCondition(TechnologyTreeCondition condition) {
		this.condition = condition;
	}

	public void setConditionPercentage(TechnologyTreeConditionPercentage condition) {
		this.condition = condition;
	}
	
	public boolean containsParent(Technology tech) {
		Technology[] pa = getParentsAsArray();
		if (pa == null) return false;
		for (int i = 0; i < pa.length; i++) {
			if (pa[i] == tech) {
				return true;
			}
		}
		return false;
	}

	protected String getElementName() {
		return "tech-tree-node";
	}
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("node", node.getFullName());
		Technology[] pa = getParentsAsArray();
		if (pa != null) {
			for (int i = 0; i < pa.length; i++) {
				if (pa[i] != null) {
					writer.addElement("parent", pa[i].getFullName());
				} else {
					Logger.logWarning("Null Parent");
				}
			}
		}
		if (condition != null) {
			condition.toXML(writer);
		}
	}

}