/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * 
 */
public class Weapon extends Technology {

	public static final String ELEMENT_NAME = "weapon";

	public static final double DEGREDATION_DISTANCE_FACTOR = 100;
    
	public static final String GROUP_PROJECTILE = "Projectile";
	public static final String GROUP_BEAM = "Beam";
	public static final String GROUP_MISSILE = "Missile";
	public static final String GROUP_SHIELD_DRAIN = "ShieldDrain";
	public static final String GROUP_IMMOBILISER = "Immobiliser";
	public static final String GROUP_GIRATOR = "Girator";
	public static final String GROUP_TRANSPORT = "Transport";
	public static final String GROUP_PSI = "Psi";
	public static final String GROUP_DETONATOR = "Detonator";
	public static final String GROUP_TROOP_TRANSPORT = "TroopTransport";
	public static final String GROUP_BIO = "Bio";
	public static final String GROUP_SPACIAL_DESTORTION = "SpacialDistortion";
	public static final String GROUP_TIME_WARP = "TimeWarp";
	public static final String GROUP_TIME_WARP_BEAM = "TimeWarpBeam";
	public static final String GROUP_PLAGUE = "Plague";
	public static final String GROUP_DAMAGE_BOT = "DamageBot";
	public static final String GROUP_MIND_CONTROL = "MindControl";
	public static final String GROUP_BERSERKER = "Berserker";
	public static final String GROUP_COWARDICE = "Cowardice";
	public static final String GROUP_DROID_MINE = "DroidMine";
	public static final String GROUP_RETARGET = "Retarget";
	public static final String GROUP_SALVAGE_BEAM = "SalvageBeam";
	public static final String GROUP_REPLICA_BEAM = "ReplicaBeam";
	public static final String GROUP_SHIELD_BURST = "ShieldBurst";
	public static final String GROUP_TELEPORT = "Teleport";
	public static final String GROUP_SLEEP = "Sleep";
	public static final String GROUP_PULSE = "Pulse";

	public static final String TYPE_NEUTRON_BLASTER = "Neutron Blaster";
	public static final String TYPE_FUSION_BEAM = "Fusion Beam";
	public static final String TYPE_PHASOR = "Phasor";
	public static final String TYPE_ANCIENT = "ancient";
	
	public static final String HOLO_WEAPON = "FakeWeapon";

    /**
     * A list of weapon groups that can inflict damage in a normal attack.
     */
    public static String [] BEAM_OR_PROJECTILE_GROUPS = { GROUP_BEAM, GROUP_PROJECTILE };

    public static String [] MISSILE_DEFENCE_ONLY_GROUPS = { GROUP_RETARGET };
    
    public static String [] STOP_END_TURN_ON_EXPRESS_GROUPS = { GROUP_BEAM, GROUP_PROJECTILE, GROUP_MISSILE, GROUP_GIRATOR };
	
    /**
     * This is a list of groups that should be fired before the attack weapons.
     * Note order is important - Weapons like ShieldDrain are best fired before attack weapon because
     * the attack weapons are reducing the shield power that can be sapped. Weapons like transport are
     * best fired after attack weapons because they have no effect is shields are raised.
     */
    public static String [] FIRE_BEFORE_GROUPS = { GROUP_SHIELD_DRAIN, GROUP_IMMOBILISER, GROUP_PSI, GROUP_SHIELD_BURST };

    public static String [] FIRE_CREW_1 = { GROUP_BIO };
    public static String [] FIRE_CREW_2 = { GROUP_TROOP_TRANSPORT };
    
    /**
     * A list of weapon groups that can inflict damage in a normal attack.
     */
    public static String [] ATTACK_GROUPS = { GROUP_BEAM, GROUP_PROJECTILE, GROUP_MISSILE, GROUP_DROID_MINE, GROUP_PULSE };

    /**
     * This is a list of groups that should be fired after the attack weapons.
     * Note order is important - Weapons like ShieldDrain are best fired before attack weapon because
     * the attack weapons are reducing the shield power that can be sapped. Weapons like transport are
     * best fired after attack weapons because they have no effect is shields are raised.
     */
    public static String [] FIRE_AFTER_GROUPS = { GROUP_TRANSPORT, GROUP_GIRATOR, GROUP_DETONATOR, GROUP_PLAGUE, GROUP_DAMAGE_BOT, GROUP_MIND_CONTROL, GROUP_BERSERKER, GROUP_COWARDICE, GROUP_SLEEP, GROUP_REPLICA_BEAM };

    /**
     * This is a list of groups that should be fired after all other types of weapons.
     */
    public static String [] FIRE_LAST_GROUPS_1 = { GROUP_TELEPORT };
    public static String [] FIRE_LAST_GROUPS_2 = { GROUP_TIME_WARP };

    /**
     * This is a list of groups that should be fired at wreckage on the battle map.
     */
    public static String [] FIRE_AT_WRECKAGE_GROUPS = { GROUP_SALVAGE_BEAM, GROUP_TIME_WARP_BEAM };

    /**
     * This is a list of groups that should be fired at movable automatons. These have no effect
     * on crafts or planets!
     */
    //public static String [] ONLY_FIRE_AT_MOVEABLE_AUTOMITONS_GROUPS = { GROUP_RETARGET };

    /**
     * This is a list of groups that should be fired at movable automatons. These have no effect
     * on crafts or planets!
     */
    public static String [] ONLY_FIRE_AT_LOCATION_GROUPS = { GROUP_SALVAGE_BEAM, GROUP_TIME_WARP_BEAM, GROUP_SPACIAL_DESTORTION };
    
    /**
     * These are weapons that can only be built on a planet (they cannot be placed onto a craft).
     */
    public static String [] PLANETARY_GROUPS = { GROUP_SPACIAL_DESTORTION };

    public static String [] MIND_GROUPS = { GROUP_MIND_CONTROL, GROUP_BERSERKER, GROUP_COWARDICE };

	/**
	 * The damage this weapon inflicts.
	 */
	private double damage;

	/**
	 * The maximum distance from which this weapon can be fired.
	 */
	private int range;

	/**
 	 * The % value (a value between 0 and 1) per DEGREDATION_DISTANCE_FACTOR squares that the damage is reduced.
 	 * A value of 1 implies the damage is not effected by range, while a value of 0 would imply even
	 * fired 1 square away all damage would be lost.
	 */
	private double degredation;

	/**
	 * The amount by which the damage produced by this weapon may vary each time it is fired.
	 * This value is between 0 and 1, and represents the amount of random element in each weapons
	 * fire. A value of 0 means all damage will always be the same, while a value of 1 would means
	 * the damage varies between 0 and double the normal damage.
	 */
	private double variance;
    
    /**
     * This is the maximum quantity of times this weapon can be used in a single battle.
     * If this is null then this weapon can be shot an infinite number of times.
     */
    private Integer qtyShots;
    
    /**
     * This this is true then this weapon is fired using the HULL's dish NOT its weapon bays.
     * Therefore this weapons can only be installed on hulls with a dish!
     * @see mot.common.technology.Hull for more details.
     */
    private boolean usesDish = false;
    
    /**
     * This is the name group name to use (instead of the actual group) for generating the
     * name of the sound to use when animating this weapon fire. This is optional, and if not
     * present the normal weapon group is used.
     * The name of the sound file used to animate weapon fire is generated by @see Sounds
     */
    private String soundGroup;
    
    private static int ANIMATION_TYPE_WEAPON_FIRE_POINTS_TO_CENTER = 0;
    private static int ANIMATION_TYPE_CENTER_TO_CENTER = 1;
    private static int ANIMATION_TYPE_RADIAL_TO_CENTER = 2;
    
    /**
     * This does not have any effect on the operation of this weapon type and is not
     * mandatory for any weapon. It may take any of the following values:
     * 0 = ANIMATION_TYPE_WEAPON_FIRE_POINTS_TO_CENTER : Animate the weapon fire from the 
     * 				weapon fire points on the source craft's image (defined on the hull)
     * 				to the center of the target image. This is the default value and the 
     * 				value used if this field contains an invalid value.
     * 				This is one or more values defined by the hull.
     * 1 = ANIMATION_TYPE_CENTER_TO_CENTER : Animate the weapon fire from the center of
     * 				the source's image to the center of the target's image. 
     * 2 = ANIMATION_TYPE_RADIAL_TO_CENTER : Animate the weapon fire from the radial point
     * 				of the source's image to the center of the target's image. 
     * 				The radial point on an image is calculated to be 20% out from the center 
     * 				of the image in the direction of the target.
     */
    private int animationType;
 
    /**
     * Most weapons do not use these fields. However the use to which they are put varies
     * between weapons. See the subclass of WeaponInstance for details.
     */
    private String param1;

    /**
     * Most weapons do not use these fields. However the use to which they are put varies
     * between weapons. See the subclass of WeaponInstance for details.
     */
    private String param2;
    
    /**
     * Some weapons can only be used in particular ways - e.g. some weapons can only have a 
     * single instance on a craft. E.g. A craft can only have a single instance of a time 
     * warp beam.
     * 
     * CURRENTLY NOT USED
     */
    private WeaponRestrictions restrictions;
    
    /**
     * When a craft is destroyed some of its technology its left in the wreckage and is 
     * available for salvage. This is the effect this weapon has on the amount of technology 
     * salvaged from its owning craft. A value of 1.0 implies no change, a value of 0.0 would mean
     * removes all technology.
     */
    private Double salvageEffect;
    
    /**
     * Maximum quantity of these weapons allowed on a craft. Value -1 implies no limit.
     */
    private int maxQtyOnCraft;
    
    /**
     * Maximum quantity of these weapons allowed on a planet. Value -1 implies no limit.
     */
    private int maxQtyOnPlanet;

    public Weapon() {
    }
    
    public Weapon(DomNode node) {
		super(node);
		damage = node.getNodeValueByName_Double("damage");
		range = node.getNodeValueByName_Integer("range");
		degredation = node.getNodeValueByName_Double("degredation");
		variance = node.getNodeValueByName_Double("variance");
		salvageEffect = node.getNodeValueByName_Double("salvage-effect");
		qtyShots = node.getNodeValueByName_Integer("qty-shots");
		usesDish = node.getNodeValueByName_Bool("uses-dish");
		animationType = node.getNodeValueByName_Int("animation-type");
		param1 = node.getNodeValueByName_String("param1");
		param2 = node.getNodeValueByName_String("param2");
		maxQtyOnCraft = node.getNodeValueByName_Integer("max-qty-on-craft");
		maxQtyOnPlanet = node.getNodeValueByName_Integer("max-qty-on-planet");
		soundGroup = node.getNodeValueByName_String("sound-group");
	}

public boolean isAnimatedFromCenterToCenter() {
	   return animationType == ANIMATION_TYPE_CENTER_TO_CENTER;
   }
   
   public boolean isAnimatedFromRadialToCenter() {
	   return animationType == ANIMATION_TYPE_RADIAL_TO_CENTER;
   }
    
	public void setDamage(double v) { damage = v; }
	public void setRange(int v) { range = v; }
	public void setDegredation(double v) { degredation = v; }
	public void setVariance(double v) { variance = v; }

	public double getDamage() { return damage; }
	public int getRange() { return range; }
	public double getDegredation() { return degredation; }
	public double getVariance() { return variance; }

    public boolean isMissile() {
        return Comp.same(getGroup(), GROUP_MISSILE);
    }
    
    public boolean isBeam() {
        return Comp.same(getGroup(), GROUP_BEAM);
    }
    
    public boolean isProjectile() {
        return Comp.same(getGroup(), GROUP_PROJECTILE);
    }

    public boolean isDroidMine() {
        return Comp.same(getGroup(), GROUP_DROID_MINE);
    }

    public boolean isBeamOrProjectile() {
    	return isBeam() || isProjectile();
    }
    
    public boolean isShieldDrain() {
        return Comp.same(getGroup(), GROUP_SHIELD_DRAIN);
    }
    
    public boolean isImmobiliser() {
        return Comp.same(getGroup(), GROUP_IMMOBILISER);
    }
    
    public boolean isDamageBot() {
        return Comp.same(getGroup(), GROUP_DAMAGE_BOT);
    }
    
    public boolean isPlague() {
        return Comp.same(getGroup(), GROUP_PLAGUE);
    }
    
    public boolean isPulse() {
        return Comp.same(getGroup(), GROUP_PULSE);
    }
    
    public boolean isBio() {
        return Comp.same(getGroup(), GROUP_BIO);
    }
    
    public boolean isSleep() {
        return Comp.same(getGroup(), GROUP_SLEEP);
    }
    
    public boolean isTroopTransport() {
        return Comp.same(getGroup(), GROUP_TROOP_TRANSPORT);
    }
    
    public boolean isShieldBurst() {
        return Comp.same(getGroup(), GROUP_SHIELD_BURST);
    }
    
    public boolean isTimeWarp() {
        return Comp.same(getGroup(), GROUP_TIME_WARP);
    }
    
    public boolean isTimeWarpBeam() {
        return Comp.same(getGroup(), GROUP_TIME_WARP_BEAM);
    }
    
    public boolean isSpacialDistortion() {
        return Comp.same(getGroup(), GROUP_SPACIAL_DESTORTION);
    }
    
    public boolean isSalvageBeam() {
        return Comp.same(getGroup(), GROUP_SALVAGE_BEAM);
    }
    
    public boolean isRetargetBeam() {
        return Comp.same(getGroup(), GROUP_RETARGET);
    }
    
    public boolean onlyTargetsMovingAutomitons() {
        return Comp.in(getGroup(), MISSILE_DEFENCE_ONLY_GROUPS);
    }

	public boolean onlyAttackLocation() {
        return Comp.in(getGroup(), ONLY_FIRE_AT_LOCATION_GROUPS);
	}
    
    public boolean isDetonator() {
        return Comp.same(getGroup(), GROUP_DETONATOR);
    }

    
    public boolean isMindGroup() {
        return Comp.in(getGroup(), Weapon.MIND_GROUPS);
    }

    public boolean isAttackGroup() {
        return Comp.in(getGroup(), Weapon.ATTACK_GROUPS);
    }
    
    public boolean fireBeforeAttackGroup() {
        return Comp.in(getGroup(), Weapon.ATTACK_GROUPS);
    }
    
    public boolean isPlanetryGroup() {
        return Comp.in(getGroup(), Weapon.PLANETARY_GROUPS);
    }
    
    public String getTechType() {
        return Technologies.TECH_TYPE_WEAPON;
    }

	protected String getElementName() { return ELEMENT_NAME; }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("name", getName());
		writer.addElement("description", getDescription());
		writer.addElement("image", getImage());
		writer.addElement("damage", damage, true);
		writer.addElement("range", range);
		writer.addElement("degredation", degredation);
		writer.addElement("variance", variance);
		writer.addElement("salvage-effect", salvageEffect);
        writer.addElement("space", getSpace());
        writer.addElement("qty-shots", qtyShots);
        if (usesDish) writer.addElement("uses-dish", usesDish);
        if (animationType != 0) writer.addElement("animation-type", animationType);
        writer.addElement("param1", param1);
        writer.addElement("param2", param2);
        writer.addElement("max-qty-on-craft", maxQtyOnCraft);
        writer.addElement("max-qty-on-planet", maxQtyOnPlanet);
        writer.addElement("cost-to-build", getCostToBuild());
		writer.addElement("cost-to-research", getCostToResearch());
		writer.addElement("maintenance-cost", getMaintenanceCost());
		writer.addElement("group", getGroup());
		writer.addElement("sound-group", soundGroup);
    }


	public NameValuePair[] getAsData() {
		NameValuePair[] r = new NameValuePair[20];
		r[0] = new NameValuePair("name", getName());
		r[1] = new NameValuePair("description", getDescription());
		r[2] = new NameValuePair("image", super.getImage());
		r[3] = new NameValuePair("damage", damage);
		r[4] = new NameValuePair("range", range);
		r[5] = new NameValuePair("degredation", degredation);
		r[6] = new NameValuePair("variance", variance);
		r[7] = new NameValuePair("salvage effect", salvageEffect);
        r[8] = new NameValuePair("space", getSpace());
        r[9] = new NameValuePair("qty shots", qtyShots);
        r[10] = new NameValuePair("uses dish", usesDish);
        r[11] = new NameValuePair("animation type", animationType);
        r[12] = new NameValuePair("param1", param1);
        r[13] = new NameValuePair("param2", param2);
        r[14] = new NameValuePair("max qty on craft", maxQtyOnCraft);
        r[15] = new NameValuePair("max qty on planet", maxQtyOnPlanet);
        r[16] = new NameValuePair("cost to build", getCostToBuild());
		r[17] = new NameValuePair("cost to research", getCostToResearch());
		r[18] = new NameValuePair("maintenance cost", getMaintenanceCost());
		r[19] = new NameValuePair("group", getGroup());
		return r;
	}


    public void setQtyShots(int v) { qtyShots = new Integer(v); }
    public void setQtyShots(Integer v) { qtyShots = v; }
    public Integer getQtyShots() { return qtyShots; }
    public String getQtyShotsAsString() { return (qtyShots == null) ? "-" : ""+qtyShots; }
	public boolean usesDish() { return usesDish;	}
	public boolean isUsesDish() { return usesDish;	}
	public void setUsesDish(boolean usesDish) {	this.usesDish = usesDish;	}
	
	public int getAnimationType() {
		return animationType;
	}
	public void setAnimationType(int animationType) {
		this.animationType = animationType;
	}

	public String getSoundGroup() {
		return soundGroup;
	}

	public void setSoundGroup(String soundGroup) {
		this.soundGroup = soundGroup;
	}

	public void setParam1(String param1) {
		this.param1 = param1;
	}

	public String getParam1() {
		return param1;
	}

	public void setParam2(String param2) {
		this.param2 = param2;
	}

	public String getParam2() {
		return param2;
	}

	public int getParam1AsInt() throws ApplicationError {
		if (param1 == null) {
			return 0;
		} else {
			try {				
				return Integer.parseInt(param1);
			} catch (Exception e) {
				throw new ApplicationError("Weapon.getParam1AsInt() Weapon "+getName()+" does not have a valid param1 value "+param1);
			}
		}
			
	}

	public int getParam2AsInt() throws ApplicationError {
		if (param2 == null) {
			return 0;
		} else {
			try {				
				return Integer.parseInt(param2);
			} catch (Exception e) {
				throw new ApplicationError("Weapon.getParam2AsInt() Weapon "+getName()+" does not have a valid param2 value "+param2);
			}
		}
			
	}

	public void setRestrictions(WeaponRestrictions restrictions) {
		this.restrictions = restrictions;
	}

	public WeaponRestrictions getRestrictions() {
		return restrictions;
	}

	public void setSalvageEffect(Double salvageEffect) {
		this.salvageEffect = salvageEffect;
	}

	public Double getSalvageEffect() {
		return salvageEffect;
	}

	public boolean unlimitedQtyOnCraft() {
		return maxQtyOnCraft < 0;
	}

	public int getMaxQtyOnCraft() {
		return maxQtyOnCraft;
	}

	public void setMaxQtyOnCraft(int maxQtyOnCraft) {
		this.maxQtyOnCraft = maxQtyOnCraft;
	}

	public boolean unlimitedQtyOnPlanet() {
		return maxQtyOnPlanet < 0;
	}

	public int getMaxQtyOnPlanet() {
		return maxQtyOnPlanet;
	}

	public void setMaxQtyOnPlanet(int maxQtyOnPlanet) {
		this.maxQtyOnPlanet = maxQtyOnPlanet;
	}

}