/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.io.Serializable;

import mot.common.bean.NamedObject;
import mot.common.util.ApplicationError;
import mot.common.xml.DomNode;
import mot.common.xml.XMLWriter;

/**
 * Container for weapon fire characteristics for a particular weapons bay.
 * Each weapons bay on a craft may allow the weapon to fire from different angles 
 * (e.g. forwards only, all around etc). In the future a given weapons bay may 
 * only contain certain types of weapon or effect the damage a weapon does...
 */
public class WeaponBayDetails extends NamedObject implements Serializable {

	public static final int DEFAULT_MAX_FIRE_ANGLE = 180;

	/**
	 * A value between 0 and 180 representing the quantity of degrees 
	 * from the from direction this craft is pointing that it may fire
	 * its weapons.
	 * A value of 180 implies that a craft can fire weapons in any direction 
	 * around it (i.e. -180 to +180 degrees).
	 */
	private int maxFireAngle = DEFAULT_MAX_FIRE_ANGLE;

	public WeaponBayDetails() {
	}

	public WeaponBayDetails(String name, int maxFireAngle) {
		super(name);
		this.maxFireAngle = maxFireAngle;
	}

	public WeaponBayDetails(DomNode domNode) {
		super(domNode.getNodeValueByName_String("name"));
		maxFireAngle = domNode.getNodeValueByName_Int("max-fire-angle");
	}

	public int getMaxFireAngle() {
		return maxFireAngle;
	}

	public void setMaxFireAngle(int maxFireAngle) {
		this.maxFireAngle = maxFireAngle;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("max-fire-angle", maxFireAngle);
	}

	protected String getElementName() {
		return "weapon-bay-details";
	}
	
}
