/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.technology;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.Comp;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;
import mot.common.vo.WeaponBayVO;
import mot.common.xml.DomNode;
import mot.common.xml.DomTree;

/**
 * 
 */
public class Weapons extends TechnologyList {

	private static final String ELEMENT_NAME = "weapons";

	private transient Weapon[] weapons = null;
    
    /**
     * The maximum weapons range of any weapon in this list.
     */
    private transient Long maxRange = null;
    
    /**
     * The maximum weapons damage of any weapon in this list.
     */
    private transient Weapon maxDamageAttackWeapon = null;
    private transient Weapon maxDamageBeamWeapon = null;
    
    /**
     * The maximum weapons damage of any weapon in this list what does not have a limited number of shots.
     */
    private transient Weapon infinateMaxDamageAttackWeapon = null;

    /**
     * When a craft is destroyed some of its technology its left in the wreckage and is 
     * This is the effect these weapons have on the amount of technology salvaged from 
     * its owning craft. A value of 1.0 implies no change, a value of 0.0 would mean
     * removes all technology.
     */
    private transient Double salvageEffect = null;

    public Weapons() {
        
    }

    public Weapons(Weapons o) {
        super(o);
    }

    public Weapons(Weapon w) {
        add(w);
    }

    public Weapons(Weapon w, Weapon... wa) {
		add(w);
        if (wa != null) {
        	for (int i = 0; i < wa.length; i++) {
				add(wa[i]);
			}
        }
    }

    public Weapons(Weapon w, int qty) {
    	for (int i = 0; i < qty; i++) {			
    		add(w);
		}
    }

    public Weapons(String [] names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.length; i++) {
                add( Technologies.getWeapon(names[i]) );
            }
        }
    }

    public Weapons(ArrayList<String> names) throws ApplicationError {
        if (names != null) {
            for (int i=0; i<names.size(); i++) {
                add( Technologies.getWeapon(names.get(i)) );
            }
        }
    }

    public Weapons(String name, int qty) throws ApplicationError {
        if (name != null) {
            for (int i=0; i<qty; i++) {
                add( Technologies.getWeapon(name) );
            }
        }
    }

	public Weapons(Weapon[] wa) {
        if (wa != null) {
            for (int i=0; i<wa.length; i++) {
                add( wa[i] );
            }
        }
	}

	public Weapons(WeaponBayVO[] wba) throws ApplicationError {
        if (wba != null) {
            for (int i=0; i<wba.length; i++) {
                add( wba[i].getWeaponsAsList() );
            }
        }
	}

	public Weapons(DomTree doc) {
    	DomNode root = doc.getNodeByName(ELEMENT_NAME);
		DomNode [] n = root.getNodesByName(Weapon.ELEMENT_NAME);
		if (n != null) {
			for (int i = 0; i < n.length; i++) {
				setWeapon(new Weapon(n[i]));
			}
		}
	}

	protected void changed() {
		weapons = null;
        maxRange = null;
        maxDamageAttackWeapon = null;
        maxDamageBeamWeapon = null;
        infinateMaxDamageAttackWeapon = null;
        salvageEffect = null;
	}

    /**
     * @return The weapons in the list
     */
	public Weapon[] getWeapons() {
		if (weapons == null) {
			Collection list = getAll(true);
			if (list == null) return null;
			weapons = new Weapon[list.size()];
			Iterator i = list.iterator();
            int index = 0;
			while (i.hasNext()) {
				Weapon v = (Weapon)i.next();
				weapons[index++] = v;
			}
		}
		return weapons;
	}


    /**
     * @return The weapons in the list excluding obsolete technologies
     */
	public Weapon[] getCurrentWeapons() {
		Collection list = getAll(false);
		if (list == null) return null;
		Weapon[] r = new Weapon[list.size()];
		Iterator i = list.iterator();
        int index = 0;
		while (i.hasNext()) {
			Weapon v = (Weapon)i.next();
			r[index++] = v;
		}
		return r;
	}

	/**
     * @return The maximum range of any weapon in this weapons list.
     */
    public long getMaxRange() throws ApplicationError {
        if (maxRange == null) {
            Weapon[] weapons = getWeapons();
            for (int i=0; weapons != null && i<weapons.length; i++) {
                if (maxRange == null || maxRange.intValue() < weapons[i].getRange()) {
                    maxRange = new Long(weapons[i].getRange());
                }
            }
        }
        if (maxRange == null) {
            return 0;
        } else {
            return maxRange.intValue();
        }
    }

    /**
     * @return The total quantity of damage inflicted from all weapons. 
     * So if there are three weapons one with damage 1 and two with damage 2 this
     * would return 3.
     */
	public int getDamageSum(boolean includeLimitedShots) {
        Weapon[] l = getWeapons();
        if (l == null) {
            return 0;
        }
        int r = 0;
        for (int i=0; i<l.length; i++) {
            if (includeLimitedShots) {
                r += l[i].getDamage();
            } else if (l[i].getQtyShots() == null) {
                r += l[i].getDamage();
            }
        }
        return r;
	}

	public void setWeapon(String v) throws ApplicationError {
		add( Technologies.getWeapons().get(v) );
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
    	if (generateSchemaAttribute) {
    		BasicNameValueList nvl = new UnsortedNameValuePairList();
    		nvl.add( new NameValuePair("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance") );
    		nvl.add( new NameValuePair("xsi:noNamespaceSchemaLocation", "xml/tech_weapons.xsd") );
    		return nvl;
    	} else {
    		return super.getAttributes(generateClassAttribute, generateSchemaAttribute);
    	}
    }

	protected String getElementName() { return ELEMENT_NAME; }

    /**
     * @return Returns the weapon with the highest damage.
     */
	public Weapon getMaxDamageAttackWeapon() {
        if (maxDamageAttackWeapon == null) {
            Weapon[] weapons = getWeapons();
            for (int i=0; weapons != null && i<weapons.length; i++) {
                if (weapons[i].isAttackGroup()) {
                    if (maxDamageAttackWeapon == null || maxDamageAttackWeapon.getDamage() <= weapons[i].getDamage()) {
                        maxDamageAttackWeapon = weapons[i];
                    }
                }
            }
        }
        return maxDamageAttackWeapon;
	}
	
	public Weapon getMaxDamageBeamWeapon() {
        if (maxDamageBeamWeapon == null) {
            Weapon[] weapons = getWeapons();
            for (int i=0; weapons != null && i<weapons.length; i++) {
                if (weapons[i].isBeamOrProjectile()) {
                    if (maxDamageBeamWeapon == null || maxDamageBeamWeapon.getDamage() <= weapons[i].getDamage()) {
                        maxDamageBeamWeapon = weapons[i];
                    }
                }
            }
        }
        return maxDamageBeamWeapon;
	}

	public Weapon getMaxDamageMissile() {
        Weapon[] weapons = getWeapons();
        Weapon maxDamageMissile = null;
        for (int i=0; weapons != null && i<weapons.length; i++) {
            if (weapons[i].isMissile()) {
				if (maxDamageMissile == null || maxDamageMissile.getDamage() <= weapons[i].getDamage()) {
                	maxDamageMissile = weapons[i];
                }
            }
        }
        return maxDamageMissile;
	}

	public Weapon getMaxDamageWeaponWithGroup(String groupName) {
		Weapon best = null;
        Weapon[] weapons = getWeapons();
        for (int i=0; weapons != null && i<weapons.length; i++) {
            if (Comp.same(weapons[i].getGroup(), groupName)) {
                if (best == null || best.getDamage() <= weapons[i].getDamage()) {
                	best = weapons[i];
                }
            }
        }
        return best;
	}

    /**
     * @return Returns the weapon with the highest damage that does not have a limited number of shots and 
     * is of group Beam or Projectile.
     */
	public Weapon getMaxDamageAttackWeaponWithInfinateShots() {
        if (infinateMaxDamageAttackWeapon == null) {
            Weapon[] weapons = getWeapons();
            for (int i=0; weapons != null && i<weapons.length; i++) {
                if (weapons[i].isBeamOrProjectile() && weapons[i].getQtyShots() == null) {
                    if (infinateMaxDamageAttackWeapon == null || infinateMaxDamageAttackWeapon.getDamage() <= weapons[i].getDamage()) {
                        infinateMaxDamageAttackWeapon = weapons[i];
                    }
                }
            }
        }
        return infinateMaxDamageAttackWeapon;
	}

    /**
     * @return Returns the weapon with the highest damage that can be used in ground combat.
     */
	public Weapon getBestGroundWeaponDamage() {
		return getMaxDamageAttackWeaponWithInfinateShots();
	}

	public Weapon getMaxDamageAntiMissileWeaponNotRetarget() throws ApplicationError {
		Weapon r = null;
        Weapon[] weapons = getWeapons();
        for (int i=0; weapons != null && i<weapons.length; i++) {
            if (weapons[i].isRetargetBeam() && weapons[i].getParam1AsInt() == 0) {
                if (r == null || r.getDamage() <= weapons[i].getDamage()) {
                    r = weapons[i];
                }
            }
        }
        return r;
	}


    public Weapon getWeapon(String name) {
        return (Weapon)super.get(name);
    }

	public Weapons getWeapons(String[] names) {
        if (names == null || names.length == 0) {
            return null;
        }
        Weapons r = new Weapons();
        for (int i = 0; i < names.length; i++) {
			r.add(getWeapon(names[i]));
		}
		return r;
	}

    public void setWeapon(Weapon v) {
        add(v);
    }

    public Weapon getWeaponAtIndex(int index) {
        return (Weapon)getTechnologyAtIndex(index);
    }

    /**
     * The return value contains a single weapon bundle for each type of weapon. The weapon
     * bundle contains the quantity of this weapon type.
     * @return A list of all weapons in this list
     */
    public WeaponBundle [] getDistinctWeapons() {
        TechnologyBundle [] techs = getDistinct();
        if (techs == null || techs.length == 0) {
            return null;
        }
        WeaponBundle [] r =  new WeaponBundle[techs.length];
        for (int i=0; i<techs.length; i++) {
            r[i] = new WeaponBundle((Weapon)techs[i].getTechnology(), techs[i].getQty());
        }
        return r;
    }

    public String getDescription(boolean summary) {
        int weaponsQty = getQuantity();
        if (weaponsQty == 0) {
            return "-";
        }
        if (summary) {
            long damage = getDamageSum(false) / weaponsQty;
            return ""+damage+"X"+weaponsQty;
        } else {
            TechnologyBundle [] tba = getDistinct();
            String r = null;
            for (int i=0; i<tba.length; i++) {
                if (r == null) {
                    r = "";
                } else {
                    r += ", ";
                }
                r += tba[i].getName()+"X"+tba[i].getQty();
            }
            return r;
        }
    }

	public double getSalvageEffect() {
		if (salvageEffect == null) {
	        Weapon[] weapons = getWeapons();
	        double r = 1.0;
	        for (int i=0; weapons != null && i<weapons.length; i++) {
	            if (weapons[i].getSalvageEffect() != null) {
	                r = r * weapons[i].getSalvageEffect().doubleValue();
	            }
	        }
	        salvageEffect = r;
		}
		return salvageEffect;
	}

	/**
	 * If existing is not null then it only returns what can be built now if the planet already has existing qty.
	 * @return All weapons that can be built on a planet (max-qty-on-planet != 0)
	 */
	public Weapon[] getPlanetWeapons(Weapons existing) {
		Weapon[] wa = getCurrentWeapons();
		if (wa == null) return null;
		ArrayList r = null;
		for (int i = 0; i < wa.length; i++) {
			int maxQtyOnPlanet = wa[i].getMaxQtyOnPlanet();
			if (maxQtyOnPlanet != 0) {
				if (existing == null) {
					if (r == null) r = new ArrayList();
					r.add(wa[i]);
				} else {
					if (maxQtyOnPlanet < 0 || existing.getQuantity(wa[i]) < maxQtyOnPlanet) {
						if (r == null) r = new ArrayList();
						r.add(wa[i]);
					}
				}
			}
		}
		if (r == null) {
			return null;
		} else {			
			return (Weapon[]) r.toArray(new Weapon[r.size()]);
		}
	}

	/**
	 * If existing is not null then it only returns what can be built now if the craft already has existing qty.
	 * @return All weapons that can be built on a planet (max-qty-on-craft != 0)
	 */
	public Weapon[] getCraftWeapons(Weapons existing) {
		Weapon[] wa = getWeapons();
		if (wa == null) return null;
		ArrayList r = null;
		for (int i = 0; i < wa.length; i++) {
			int maxQtyOnCraft = wa[i].getMaxQtyOnCraft();
			if (maxQtyOnCraft != 0) {
				if (existing == null) {
					if (r == null) r = new ArrayList();
					r.add(wa[i]);
				} else {
					if (maxQtyOnCraft < 0 || existing.getQuantity(wa[i]) < maxQtyOnCraft) {
						if (r == null) r = new ArrayList();
						r.add(wa[i]);
					}
				}
			}
		}
		if (r == null) {
			return null;
		} else {			
			return (Weapon[]) r.toArray(new Weapon[r.size()]);
		}
	}

}