/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.util;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * 
 */
public class BasicNameValuePairList extends XMLBean implements BasicNameValueList, Serializable {
    private HashMap list = new HashMap();

    public BasicNameValuePairList() {
    }

    public HashMap getList() { return list; }
    public void setList(HashMap v) { list=v; }
    public Collection getKeys() { return list.keySet(); }

    public int getSize() {
        if (list == null) {
            return 0;
        } else {
            return list.size();
        }
    }

    public NameValuePair [] getAsArray() {
		if (list == null) {
			return null;
		}
		ArrayList r = new ArrayList();
		Collection c = list.keySet();
		if ( c != null) {
			Iterator i = c.iterator();
			while (i.hasNext()) {
				String key = (String) i.next();
				String value = (String)list.get(key);
				r.add( new NameValuePair(key, value) );
			}
		}
		return (NameValuePair []) r.toArray(new NameValuePair [r.size()]);
	}

    public String getAsSeperatedList(String seperator) {
    	String r = "";

        boolean first = true;
		if (list != null) {
			Collection c = list.keySet();
			if ( c != null) {
				Iterator i = c.iterator();
				while (i.hasNext()) {
					String key = (String) i.next();
					String value = (String)list.get(key);
					if (first) {
						first = false;
					} else {
						r += seperator;
					}
					r += key+"="+value;
				}
			}
		}

		return r;
	}

    public String getAsAttributeList() {
    	StringBuffer r = new StringBuffer();

        boolean first = true;
		if (list != null) {
			Collection c = list.keySet();
			if ( c != null) {
				Iterator i = c.iterator();
				while (i.hasNext()) {
					String key = (String) i.next();
					String value = (String)list.get(key);
					if (first) {
						first = false;
					} else {
						r.append(" ");
					}
					r.append(key);
					r.append("=\"");
					r.append(value);
					r.append("\"");
				}
			}
		}

		return r.toString();
	}

	public void add(BasicNameValueList v) {
		if (v != null) {
			Collection c = v.getKeys();
			if ( c != null) {
				Iterator i = c.iterator();
				while (i.hasNext()) {
					String key = (String) i.next();
					add(key, (String)v.getParameter(key));
				}
			}
		}
	}

	public void add(NameValuePair nvp) {
		add(nvp.getName(), nvp.getValue());
	}

	public void add(String name, String value) {
		if (name == null || value == null) return;

		String v = (String) list.get(name);

		if (v == null) {
			list.put(name, value);
		} else {
			if ( !v.equals(value) ) {
				list.remove(name);
				list.put(name, value);
			}
		}
	}

    public void add(String name, String [] values) throws ApplicationError {
        if (name == null || values == null || values.length == 0) {
            return;
        }
        for (int i=0; i<values.length; i++) {
            add(name+i, values[i]);
        }
    }


    public void add(String name, Integer value) throws ApplicationError {
    	if (value != null) {
    		add(name, ""+value);
    	}
    }

    public void add(String name, Long value) throws ApplicationError {
    	if (value != null) {
    		add(name, ""+value);
    	}
    }

    public void add(String name, Double value) throws ApplicationError {
    	if (value != null) {
    		add(name, ""+value);
    	}
    }

    public void add(String name, int value) {
  		add(name, ""+value);
    }

    public void add(String name, boolean value) {
    	if (value) {
    		add(name, "Y");
    	} else {
    		add(name, "N");
    	}
    }

	public String getParameter(String name) {
		if (name == null) return null;
		return (String) list.get(name);
	}

	public String [] getParameterValues(String name) {
		if (name == null) return null;
        ArrayList list = new ArrayList();

        String v = getParameter(name);
        if (v != null) {
            list.add(v);
        }

        v = getParameter(name+"0");
        int i = 0;
        while (v != null) {
            list.add(v);
            i++;
            v = getParameter(name+i);
        }

        if (list.size() == 0) {
            return null;
        } else {
            return (String []) list.toArray(new String[list.size()]);
        }
	}

	public int getParameterAsValidInt(String name) throws ApplicationError {
		String v = getParameter(name);
		if (v == null) return 0;
		return Integer.parseInt(v);
	}

	protected String getElementName() { return "setup-option"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
        NameValuePair [] l = getAsArray();
        for (int i=0; l != null && i < l.length; i++) {
            if (l[i] != null) {
                l[i].toXML(writer);
            }
        }
	}

	public String[] getParameterNames() {
		Collection keys = getKeys();
		if (keys == null) {
			return null;
		}
		Iterator i = keys.iterator();
		String [] r = new String[keys.size()];
		int j = 0;
		while (i.hasNext()) {
			r[j++] = (String) i.next();
		}
		return r;
	}

	public Technology[] getParameterNamesAsTechs() throws ApplicationError {
		Collection keys = getKeys();
		if (keys == null) {
			return null;
		}
		Iterator i = keys.iterator();
		Technology [] r = new Technology[keys.size()];
		int j = 0;
		while (i.hasNext()) {
			String techFullName = (String) i.next();
            if (techFullName == null) {
                throw new ApplicationError("BasicNameValuePairList.getParameterNamesAsTechs() Unknown technology type "+techFullName);
            }
            //Logger.logDebug(techFullName);
            r[j++] = Technologies.getTechnology(techFullName);
		}
		return r;
	}

}