/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.util;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.common.bean.DateTime;
import mot.common.bean.Location;
import mot.common.bean.NameList;
import mot.common.bean.NamedObject;
import mot.common.bean.NamedObjectI;
import mot.common.bean.Rectangle;
import mot.common.bean.SavedGameFile;
import mot.common.bean.UserPlayerSummary;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.TechnologyList;
import mot.common.technology.TechnologyTree;
import mot.common.technology.TechnologyTreeNode;
import mot.engine.planets.Planet;

/**
 * 
 */
public class Comp {

	public static boolean same(NamedObjectI v1, NamedObjectI v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.getName().equalsIgnoreCase(v2.getName()));
	}

	public static boolean same(NamedObject v1, NamedObject v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.getName().equalsIgnoreCase(v2.getName()));
	}

	public static boolean same(NamedObject v1, String v2) {
		if (v1 == null && v2 == null) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.getName().equalsIgnoreCase(v2));
	}

	public static boolean same(String v1, String v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.equalsIgnoreCase(v2));
	}

	public static boolean same(NameList v1, NameList v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return v1.same(v2);
	}

	public static boolean same(String v1, String v2, String v3) {
		if (v1 == v2) return true;
		if (v1 == v3) return true;
		if (v1 == null) return false;
        if (v2 != null) {
            if (v1.equalsIgnoreCase(v2)) {
                return true;
            }
        }
        if (v3 != null) {
            if (v1.equalsIgnoreCase(v3)) {
                return true;
            }
        }
        return false;
	}

	public static boolean same(Integer v1, Integer v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.intValue() == v2.intValue());
	}

	public static boolean same(Double v1, Double v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.doubleValue() == v2.doubleValue());
	}

	public static boolean same(Long v1, Long v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.longValue() == v2.longValue());
	}

	public static boolean same(DateTime v1, DateTime v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.sameDate(v2));
	}
    
	public static boolean same(String v1, int v2) {
        int v = 0;
        if (v1 != null) {
            try {
                v = Integer.parseInt(v1);
            } catch (NumberFormatException e) {
            }
        }
		return v == v2;
	}

	public static boolean same(String v1, NamedObject v2) {
        if (v2 == null || v1 == null) {
            return false;
        }
        return same(v1, v2.getName()); 
    }

	public static boolean same(String v1, NamedObjectDelegate v2) {
        if (v2 == null || v1 == null) {
            return false;
        }
        return same(v1, v2.getName()); 
    }

	public static boolean same(NamedObject v1, NamedObjectDelegate v2) {
		if (v1 == null && v2 == null) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.getName().equalsIgnoreCase(v2.getName()));
	}

	public static boolean same(NamedObjectDelegate v1, String v2) {
		if (v1 == null && v2 == null) return true;
		if (v1 == null || v2 == null) return false;
		return (v1.getName().equalsIgnoreCase(v2));
	}

	public static boolean in(String v1, String [] v2) {
		if (v1 == null || v2 == null) return false;
		for (int i=0; i<v2.length; i++) {
			if (same(v1, v2[i])) {
				return true;
			}
		}
		return false;
	}

	public static boolean in(Technology v1, Technology [] v2) {
		if (v1 == null || v2 == null) return false;
		for (int i=0; i<v2.length; i++) {
			if (v1 == v2[i]) {
				return true;
			}
		}
		return false;
	}

	public static boolean in(NamedObject v, ArrayList list) {
		if (v == null || list == null) return false;
		for (Object o : list) {
			if (o instanceof NamedObject) {
				NamedObject no = (NamedObject) o;
				if (same(v, no)) {
					return true;
				}
			}
		}
		return false;
	}

	public static boolean in(String v1, ArrayList<String> v2) {
		if (v1 == null || v2 == null) return false;
		for (String v : v2) {
			if (same(v1, v)) {
				return true;
			}
		}
		return false;
	}

	public static boolean in(String v1, BasicNameValueList v2) {
        if (v2 == null) return false;
        return in(v1, v2.getAsArray());
    }
    
	public static boolean in(String v1, NamedObjectDelegate[] v2) {
		if (v1 == null || v2 == null) return false;
		for (int i=0; i<v2.length; i++) {
			if (same(v1, v2[i].getName())) {
				return true;
			}
		}
		return false;
	}

	public static boolean in(NamedObjectDelegate v1, NamedObjectDelegate[] v2) {
		if (v1 == null || v2 == null) return false;
		for (int i=0; i<v2.length; i++) {
			if (same(v1, v2[i])) {
				return true;
			}
		}
		return false;
	}

	public static boolean in(String v1, NamedObject [] v2) {
		if (v1 == null || v2 == null) return false;
		for (int i=0; i<v2.length; i++) {
			if (same(v1, v2[i].getName())) {
				return true;
			}
		}
		return false;
	}

	public static boolean in(NamedObject v1, NamedObject [] v2) {
		if (v1 == null || v2 == null) return false;
		for (int i=0; i<v2.length; i++) {
			if (v1 == v2[i]) {
				return true;
			}
		}
		return false;
	}

	public static boolean in(Class v1, Class[] v2) {
		if (v1 == null || v2 == null) return false;
		for (int i=0; i<v2.length; i++) {
			if (v1 == v2[i]) {
				return true;
			}
		}
		return false;
	}
	
	
	public static NamedObjectDelegate getNameStartingWith(String name, NamedObjectDelegate[] items) {
		if (name == null || items == null) return null;
		for (int i=0; i<items.length; i++) {
			if (items[i] != null && items[i].getName() != null && items[i].getName().startsWith(name)) {
				return items[i];
			}
		}
		return null;
	}

	public static int findIndexInArray(NamedObjectDelegate ss, NamedObjectDelegate[] sos) {
		if (sos == null) return -1;
		for (int i = 0; i < sos.length; i++) {
			if (same(sos[i], ss)) {
				return i;
			}
		}
		return -1;
	}
	
	public static int findIndexInArray(String name, NamedObjectDelegate[] sos) {
		if (sos == null) return -1;
		for (int i = 0; i < sos.length; i++) {
			if (same(sos[i], name)) {
				return i;
			}
		}
		return -1;
	}
	
	
	public static NamedObjectDelegate find(NamedObjectDelegate ss, NamedObjectDelegate[] sos) {
		if (sos == null || ss == null) return null;
		for (int i = 0; i < sos.length; i++) {
			if (same(sos[i], ss)) {
				return sos[i];
			}
		}
		return null;
	}
	
	public static NamedObjectI findIn(String name, NamedObjectI[] sos) {
		if (sos == null) return null;
		for (int i = 0; i < sos.length; i++) {
			if (same(sos[i].getName(), name)) {
				return sos[i];
			}
		}
		return null;
	}
	
	public static String find(String name, String [] da) {
		if (da == null) return null;
		for (int i = 0; i < da.length; i++) {
			if (same(da[i], name)) {
				return da[i];
			}
		}
		return null;
	}
	
	public static NamedObjectDelegate find(String name, NamedObjectDelegate[] da) {
		if (da == null) return null;
		for (int i = 0; i < da.length; i++) {
			if (same(da[i], name)) {
				return da[i];
			}
		}
		return null;
	}
	
	public static NamedObject find(String name, NamedObject[] da) {
		if (da == null) return null;
		for (int i = 0; i < da.length; i++) {
			if (da[i] != null && same(da[i].getName(), name)) {
				return da[i];
			}
		}
		return null;
	}

	public static boolean same(NamedObjectDelegate v1, NamedObjectDelegate v2) {
		if (v1 == v2) return true;
		if (v1 == null || v2 == null) return false;
		return Comp.same(v1.getName(), v2.getName());
	}

    public static boolean isEmpty(String s) { return ((s == null) || (s.length() == 0)); }
    public static  boolean isEmpty(Integer i) { return (i == null); }
    public static  boolean isEmpty(String s1, String s2) { return (isEmpty(s1) && isEmpty(s2)); }

    public static boolean notEmpty(String s) { return !isEmpty(s); }

    public static String join(String p1, String p2) {
		if (p1 == null && p2 == null) {
			return null;
		} else if (p1 != null && p2 == null) {
			return p1;
		} else if (p1 == null && p2 != null) {
			return p2;
		} else {
			return p1+" "+p2;
		}
	}

	public static int nnv(Integer v) {
		if (v == null) {
			return 0;
		} else {
			return v.intValue();
		}
	}

	public static int nnv(Integer v, int d) {
		if (v == null) {
			return d;
		} else {
			return v.intValue();
		}
	}

	public static long nnv(Long v) {
		if (v == null) {
			return 0;
		} else {
			return v.longValue();
		}
	}

	public static double nnv(Double v) {
		if (v == null) {
			return 0;
		} else {
			return v.doubleValue();
		}
	}

	public static double nnv(Double v, double d) {
		if (v == null) {
			return d;
		} else {
			return v.doubleValue();
		}
	}

	public static String nnv(String v) {
		if (v == null) {
			return "";
		} else {
			return v;
		}
	}

	public static String nnv(String v1, String v2) {
		if (v1 != null) {
			return v1;
		} else if (v2 != null) {
			return v2;
		} else {
			return "";
		}
	}

    public static String [] toStrings(NamedObject[] v) {
		if (v == null) {
            return null;
        }
        String [] names = new String[v.length];
        for (int i=0; i<v.length; i++) {
            names[i] = v[i].getName();
        }
        return names;
	}

    public static String [] toStrings(NamedObjectDelegate[] v) {
		if (v == null) {
            return null;
        }
        String [] names = new String[v.length];
        for (int i=0; i<v.length; i++) {
            names[i] = v[i].getName();
        }
        return names;
	}

    public static String [] toStrings(NamedObjectDelegate[] v, String exclude) {
		if (v == null) {
            return null;
        }
        ArrayList r = null;
        for (int i=0; i<v.length; i++) {
            if (!same(v[i].getName(), exclude)) {
                if (r == null) r = new ArrayList();
                r.add(v[i].getName());
            }
        }
        if (r == null) {
            return null;
        } else {
            return (String []) r.toArray(new String[r.size()]);
        }
	}

    public static String [] toStrings(NamedObjectDelegate[] v, String [] exclude) {
		if (v == null) {
            return null;
        }
        ArrayList r = null;
        for (int i=0; i<v.length; i++) {
            if (!in(v[i].getName(), exclude)) {
                if (r == null) r = new ArrayList();
                r.add(v[i].getName());
            }
        }
        if (r == null) {
            return null;
        } else {
            return (String []) r.toArray(new String[r.size()]);
        }
	}

    public static String [] toStrings(NamedObjectDelegate[] v, NamedObjectDelegate[] exclude) {
		if (v == null) {
            return null;
        }
        ArrayList r = null;
        for (int i=0; i<v.length; i++) {
            if (!in(v[i].getName(), exclude)) {
                if (r == null) r = new ArrayList();
                r.add(v[i].getName());
            }
        }
        if (r == null) {
            return null;
        } else {
            return (String []) r.toArray(new String[r.size()]);
        }
	}


    public static String toString(List<String> v) {
    	if (v == null) return null;
		return toString(v.toArray(new String[v.size()]), " ");
	}

    public static String toString(String [] v) {
		return toString(v, " ");
	}

    public static String toString(String [] v, String seperator) {
		if (v == null) {
			return null;
		}

		if (v.length == 1) {
			return v[0];
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length; i++) {
			sb.append(v[i]);
			if (i < v.length-1)	sb.append(seperator);
		}

		return sb.toString();
	}

    public static String toString(NamedObject [] v) {
		return toString(v, " ");
	}

    public static String toString(NamedObject [] v, String seperator) {
		if (v == null) {
			return null;
		}

		if (v.length == 1) {
			if (v[0] == null) {
				return null;
			} else {
				return v[0].getName();
			}
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length; i++) {
			if (v[i] != null) {
				if (i != 0) sb.append(seperator);
				sb.append(v[i].getName());
			}
		}

		return sb.toString();
	}

    
    public static String toString(NamedObject [] v, String seperator, int maxQty, String tooManyMessage) {
		if (v == null) {
			return null;
		}

		if (v.length == 1) {
			if (v[0] == null) {
				return null;
			} else {
				return v[0].getName();
			}
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length && i < maxQty; i++) {
			if (v[i] != null) {
				if (i != 0) sb.append(seperator);
				sb.append(v[i].getName());
			}
		}
		if (v.length > maxQty) {
			sb.append(tooManyMessage);
		}

		return sb.toString();
	}

    public static String toString(Location [] v) {
		return toString(v, " ");
	}

    public static String toString(Location [] v, String seperator) {
		if (v == null) {
			return null;
		}

		if (v.length == 1) {
			return v[0].toString();
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length; i++) {
			if (i != 0) sb.append(seperator);
			sb.append(v[i].toString());
		}

		return sb.toString();
	}
    
	public static String toString(TechnologyBundle[] v) {
		if (v == null) {
			return null;
		}

		if (v.length == 1) {
			return v[0].toString();
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length; i++) {
			if (i != 0) sb.append(",");
			sb.append(v[i].toString());
		}

		return sb.toString();
	}

	public static String toString(TechnologyTree techTree) {
		StringBuffer sb = new StringBuffer();
		TechnologyTreeNode[] v = techTree.getNodesAsArray();
		for (int i=0; i<v.length; i++) {
			if (i != 0) sb.append(";");
			sb.append(v[i].toString());
		}

		return sb.toString();
	}

	public static String toString(UserPlayerSummary[] v) {
		return toString(v, " ");
	}

	public static String toString(UserPlayerSummary[] v, String seperator) {
		if (v == null) {
			return "";
		}

		if (v.length == 1) {
			return v[0].getRaceName();
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length; i++) {
			if (i != 0) sb.append(seperator);
			sb.append(v[i].getRaceName());
		}

		return sb.toString();
	}


    public static String toString(NamedObjectDelegate [] v) {
		return toString(v, " ");
	}

    public static String toString(NamedObjectDelegate [] v, String seperator) {
		if (v == null) {
			return "";
		}

		if (v.length == 1) {
			return v[0].getName();
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length; i++) {
			if (i != 0) sb.append(seperator);
			sb.append(v[i].getName());
		}

		return sb.toString();
	}

    public static String toStringOnAll(Object [] v) {
		return toStringOnAll(v, " ");
	}

    public static String toStringOnAll(Object [] v, String seperator) {
		if (v == null) {
			return null;
		}

		if (v.length == 1) {
			return v[0].toString();
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length; i++) {
			sb.append(v[i].toString());
			sb.append(seperator);
		}

		return sb.toString();
	}

/**
	 * The parameter v must be a list of Strings.
	 */
    public static String toStringNamedItems(Collection c) {
        return toStringNamedItems(c, ",");
    }
    
	/**
	 * The parameter v must be a list of Strings.
	 */
    public static String toStringNamedItems(Collection c, String seperator) {
		if (c == null) {
			return null;
		}

        Iterator i = c.iterator();
		StringBuffer sb = new StringBuffer();
		while (i.hasNext()) {
			NamedObject v = (NamedObject) i.next();
            if (v != null) {
                sb.append(v.getName());
                sb.append(seperator);
            }
		}

		return sb.toString();
    }

	/**
	 * The parameter v must be a list of Strings.
	 */
    public static String toString(Collection c, String seperator) {
		if (c == null) {
			return null;
		}

        Iterator i = c.iterator();
		StringBuffer sb = new StringBuffer();
		while (i.hasNext()) {
			Object v = i.next();
            if (v != null) {
                sb.append(v.toString());
                sb.append(seperator);
            }
		}

		return sb.toString();
    }

	/**
	 * The parameter v must be a list of Strings.
	 */
    public static String toString(Iterator i, String seperator) {
		if (i == null) {
			return null;
		}

		StringBuffer sb = new StringBuffer();
		while (i.hasNext()) {
			String v = (String) i.next();
			sb.append(v);
			sb.append(seperator);
		}

		return sb.toString();
	}

	public static String toString(SavedGameFile[] v) {
		if (v == null) {
			return "";
		}

		if (v.length == 1) {
			return v[0].getFileName();
		}

		StringBuffer sb = new StringBuffer();
		for (int i=0; i<v.length; i++) {
			if (i != 0) sb.append(",");
			sb.append(v[i].getFileName());
		}

		return sb.toString();
	}


    public static boolean notNull(String v) { return v!=null && v.length()>0; }
    public static boolean notNull(Integer v) { return v!=null; }
    public static boolean notNull(Double v) { return v!=null; }


	// Much faster than Integer.pareseInt() as it will never construct an object!
	public static boolean isInteger(String v) {
		if (v == null) return false;
		for (int i=0; i<v.length(); i++) {
			if (!isNumber(v.charAt(i))) {
				return false;
			}
		}
		return true;
	}

	public static boolean bigger(Double v1, Double v2) {
        if (v1 == null) return false;
        if (v2 == null) return true;
        return (v1.doubleValue() > v2.doubleValue());
    }

	public static boolean equal(int v1, int v2, int variance) {
        return (v1 >= (v2-variance) && v2 >= (v1-variance));
    }

	public static boolean isNumber(char v) {
		return (v == '0' || v == '1' || v == '2' || v == '3' || v == '4' || v == '5' || v == '6' || v == '7' || v == '8' || v == '9');
	}

	public static boolean isWhiteSpace(char v) {
		return (v == ' ' || v == '\n' || v == '\t' || v == '\r');
	}

	public static boolean containsNumber(String v) {
        if (v == null) return false;

        char [] ca = v.toCharArray();
        for (int i=0; i<ca.length; i++) {
            if (isNumber(ca[i])) {
                return true;
            }
        }

        return false;
    }
    
    public static boolean startsWith(String value, String prefix) {
        if (value == null) return false;
        if (prefix == null) return false;
        
        return value.startsWith(prefix);
    }

    public static int min(int v1, int v2) {
        return (v1 < v2) ? v1 : v2;
    }

    public static long min(long v1, long v2) {
        return (v1 < v2) ? v1 : v2;
    }

    public static int max(int v1, int v2) {
        return (v1 > v2) ? v1 : v2;
    }

    public static double min(double v1, double v2) {
        return (v1 < v2) ? v1 : v2;
    }

    public static double max(double v1, double v2) {
        return (v1 > v2) ? v1 : v2;
    }

    public static float min(float v1, float v2) {
        return (v1 < v2) ? v1 : v2;
    }

    public static float max(float v1, float v2) {
        return (v1 > v2) ? v1 : v2;
    }

    public static int pos(int v) {
        return (v < 0) ? -v : v;
    }

    public static double pos(double v) {
        return (v < 0.0) ? -v : v;
    }
    
    public static double difference(double v1, double v2) {
        return (v1 > v2) ? v1-v2 : v2-v1;
    }

    public static int difference(int v1, int v2) {
        return (v1 > v2) ? v1-v2 : v2-v1;
    }

    public static double inRange(double low, double high, double v) {
        v = (v > high) ? high : v; 
        v = (v < low) ? low : v;
        return v;
    }

    public static int ceil(float v) {
        return (int) (Math.ceil(v));
    }
    
    public static int ceil(double v) {
        return (int) (Math.ceil(v));
    }
    
    public static int floor(float v) {
        return (int) (Math.floor(v));
    }

    public static int floor(double v) {
        return (int) (Math.floor(v));
    }

    public static String [] toStringArray(int v) {
        if (v > 0) {
            String [] r = new String[v];
            for (int i=0; i<v; i++) {
                r[i] = ""+(i+1);
            }
            return r;
        } else {
            return null;
        }
    }
    
    public static String left(String v) {
        return left(v, 3);
    }
    
    public static String left(String v, int len) {
        if (v == null) return null;
        if (v.length() <= len) {
            return v;
        } else {
            return v.substring(0, len);
        }
    }
    

    public static Object [] copy(Object [] to, Object [] from) {
        if (from  == null || to == null) return null;
        int qty = min(from.length, to.length);
        for (int i=0; i<qty; i++) {
            to[i] = from[i];
        }
        return to;
    }

	public static String[] add(String[] a, String v) {
    	if (a == null) {
    		a = new String[1];
    		a[0] = v;
    		return a;
    	} else {
    		String [] na = new String[a.length+1];
    		for (int i = 0; i < a.length; i++) {
    			na[i] = a[i];
    		}
    		na[a.length] = v;
    		return na;
    	}
	}

	public static ArrayList add(ArrayList r, Object[] va) {
		if (va != null) {
			for (int i = 0; i < va.length; i++) {
				if (r == null) r = new ArrayList();
				r.add(va[i]);
			}
		}
		return r;
	}

	public static boolean contained(Point clickLock, Rectangle rect) {
		if (clickLock == null || rect == null) return false;
		return rect.contains(clickLock.x, clickLock.y);
	}

	/**
	 * Round an angle to the closest of 0, 90, 180 or 270.
	 */
	public static int roundAngle(int dir) {
    	if (dir >= 45 && dir < 135) {
    		return 90;
    	} else if (dir >= 135 && dir < 225) {
    		return 180;
    	} else if (dir >= 225 && dir < 315) {
    		return 270;
    	} else {
    		return 0;
    	} 
	}

	public static boolean isKey(char c1, char c2) {
		return Character.toLowerCase(c1) == c2;
	}

	public static int getDirectionChangeFor(int direction, int reqdDirection) {
		int reqdDirectionChange = reqdDirection - direction;
        if (reqdDirectionChange < -180) reqdDirectionChange += 360;
        if (reqdDirectionChange > 180) reqdDirectionChange -= 360;
        return reqdDirectionChange;
	}

	// BROKEN???
	public static double round3dp(double v) {
		long t = (long)(v * 1000.0);
		double r = (((double)v)+0.5)/1000.0;
		return r;
	}

	public static double roundToWhole(double v) {
		long r = (long)(v+0.5);
		return r;
	}

	public static int roundDown(double d) {
		return (int)d;
	}

	public static boolean isWhole(double v) {
		long v2 = (long)v;
		return (v == ((double)v2));
	}
	
	public static String [] getNames(NamedObject [] oa) {
		if (oa == null) return null;
		String [] r = new String[oa.length];
		for (int i = 0; i < r.length; i++) {
			r[i] = oa[i].getName();
		}
		return r;
	}

	public static String [] getNames(NamedObjectDelegate [] oa) {
		if (oa == null) return null;
		String [] r = new String[oa.length];
		for (int i = 0; i < r.length; i++) {
			r[i] = oa[i].getName();
		}
		return r;
	}

	public static ArrayList<String> getNamesAsList(NamedObjectDelegate [] oa) {
		if (oa == null) return null;
		ArrayList<String>  r = new ArrayList<String>();
		for (int i = 0; i < oa.length; i++) {
			r.add(oa[i].getName());
		}
		return r;
	}

	public static NamedObjectDelegate [] sortByName(NamedObjectDelegate[] l) {
		if (l == null || l.length < 2) return l;
		for (int i = 0; i < l.length-1; i++) {
			for (int j = 0; j < l.length - 1 - i; j++) {
				if (l[j].getName().compareTo(l[j+1].getName()) > 0) {
					NamedObjectDelegate t = l[j];
					l[j] = l[j+1];
					l[j+1] = t;
				}
			}			
		}
		return l;
	}

	public static TechnologyList sortByName(TechnologyList l) throws ApplicationError {
		if (l != null) {
			return new GeneralTechnologyList(sortByName(l.getAllAsArray()));
		} else {
			return null;
		}
	}

	public static Technology [] sortByName(Technology [] l) {
		if (l == null || l.length < 2) return l;
		for (int i = 0; i < l.length-1; i++) {
			for (int j = 0; j < l.length - 1 - i; j++) {
				if (l[j].getName().compareTo(l[j+1].getName()) > 0) {
					Technology t = l[j];
					l[j] = l[j+1];
					l[j+1] = t;
				}
			}			
		}
		return l;
	}
	
	public static String [] sortByName(String[] l) {
		if (l == null || l.length < 2) return l;
		for (int i = 0; i < l.length-1; i++) {
			for (int j = 0; j < l.length - 1 - i; j++) {
				if (l[j].compareTo(l[j+1]) > 0) {
					String t = l[j];
					l[j] = l[j+1];
					l[j+1] = t;
				}
			}			
		}
		return l;
	}

	public static ArrayList sort(ArrayList l, Comparator c) throws ApplicationError {
		if (l == null) return null;
		if (l.size() == 1) return l;
		Object[] a = l.toArray();
		Arrays.sort(a, c);
		ArrayList r = new ArrayList();
		for (int i = 0; i < a.length; i++) {
			r.add(a[i]);
		}
		return r;
	}


	public static String[] remove(String[] names, String name) {
		if (names == null) return null;
		ArrayList<String> r = new ArrayList();
		for (int i = 0; i < names.length; i++) {
			if (!Comp.same(name, names[i])) {
				r.add(names[i]);
			}
		}
		if (r.size() > 0) {
			return r.toArray(new String[r.size()]);
		} else {
			return null;
		}
	}

	public static ArrayList toList(Object[] v) {
		if (v == null) return null;
		 ArrayList r = new ArrayList();
		 for (int i = 0; i < v.length; i++) {
			r.add(v[i]);
		}
		return r;
	}

	public static int getQuantity(TechnologyList list, Technology t) {
		if (list == null) {
			return 0;
		} else {
			return list.getQuantity(t);
		}
	}

	public static boolean same(String fileName, SavedGameFile file) {
		if (fileName == null && file == null) return true;
		if (fileName == null || file == null) return false;
		return Comp.same(fileName, file.getFileName());
	}

	public static MoveableOwnedObjectDelegate[] remove(MoveableOwnedObjectDelegate[] moos, MoveableOwnedObjectDelegate moo) {
		if (moos == null) return null;
		if (moos.length == 1) {
			if (moos[0].equals(moo)) {
				return null;
			} else {
				return moos;
			}
		}
    	ArrayList<MoveableOwnedObjectDelegate> r = new ArrayList();
    	for (int i = 0; i < moos.length; i++) {
    		if (!moos[i].equals(moo)) {
    			r.add(moos[i]);
    		}
		}
		if (r.size() > 0) {
			return r.toArray(new MoveableOwnedObjectDelegate[r.size()]);
		} else {
			return null;
		}
	}

	public static Planet[] remove(Planet[] planets, Planet planet) {
		if (planets == null) return null;
		if (in(planet, planets)) {
			if (planets.length == 1) {
				return null;
			} else {
				Planet[] r = new Planet[planets.length-1];
				int j = 0;
				for (int i = 0; i < planets.length; i++) {
					if (planets[i] != planet) r[j++] = planets[i];
				}
				return r;
			}
		} else {
			return planets;
		}
	}

}