/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import javax.swing.Timer;

import mot.client.MotCommandLineOptions;
import mot.common.bean.Settings;

/**
 * Utility class that is used by the system to log information to a file.
 * This information includes debug, progress, error and warning. The amount
 * of information being logged can be changed at run time this allowing debugging
 * to be switched on or off while the system is running.
 */
public class Logger {
	
	private static boolean inMemory = true;
	private static boolean logToStdOut = false;
	private static Timer timer;

	private static StringBuilder contents =  new StringBuilder(200000);
	
    /**
     * No information is logged.
     */
    public static final int LEVEL_SILENT = 0;

    /**
     * Only error information is logged.
     */
    public static final int LEVEL_ERROR = 1;

    /**
     * Only error and warning information is logged.
     */
    public static final int LEVEL_WARNING = 2;

    /**
     * Logs the results of actions performed, i.e. A single like containing
     * the IDs for each registration or amendment, even if this operation
     * consisted of serial HTTP requests.
     */
    public static final int LEVEL_RESULT  = 3;

    /**
     * A log of all tasks performed as well as warning and error
     * information are logged.
     */
    public static final int LEVEL_INFO  = 4;

    /**
     * All information is logged including all the debugging information.
     * This may produce a very verbose log.
     */
    public static final int LEVEL_DEBUG = 5;

    /**
     * All information available will be logged. This may produce a very verbose log.
     */
    public static final int LEVEL_FULL = 6;

    /**
     * This is the current level of logging and must be one of the
     * values above. The default level depends on the type of deployment.
     * For Development builds then all logging information is displayed
     * but for live/test builds then only warning and error information
     * is displayed.
     */
    private static int loggingLevel = LEVEL_WARNING;
    private static boolean loggingLevelLoaded = false;
    static {
        try {
            if (Settings.hasInstance()) {
            	loggingLevel = Settings.getInstance().getLogLevel();
            } else {
           		loggingLevel = PropertyManager.getMandatoryAsInt("log.level");
            }
        } catch (ApplicationError e) { }
    }
    
    private static String logFile = null;
    private static String getLogFile() {
    	if (logFile == null) {
    		logFile = MotCommandLineOptions.getLogFile();
    		if (logFile == null) {
    			logFile = PropertyManager.get("log.file");
    		}
    	}
    	return logFile;
    }

    /**
     * If archive is true, the log files are archived at the end
     * of each day to a file with a date extended name. Otherwise
     * the log files are truncated at the end of each day.
     */
    private static boolean archive = false;

    /**
     * The current day for which the logs are being written.
     */
    private static int day = -1;
	private static boolean changed = false;

    public static void setSilent() { loggingLevel = LEVEL_SILENT; }
    public static void setError() { loggingLevel = LEVEL_ERROR; }
    public static void setWarning() { loggingLevel = LEVEL_WARNING; }
    public static void setResult() { loggingLevel = LEVEL_RESULT; }
    public static void setInfo() { loggingLevel = LEVEL_INFO; }
    public static void setDebug() { loggingLevel = LEVEL_DEBUG; }
    public static void setFull() { loggingLevel = LEVEL_FULL; }
    public static void setLogLevelLoaded() { loggingLevelLoaded = true; }

    public static boolean isSilent() { return (loggingLevel == LEVEL_SILENT); }
    public static boolean isError() { return (loggingLevel >= LEVEL_ERROR); }
    public static boolean isWarning() { return (loggingLevel >= LEVEL_WARNING); }
    public static boolean isResult() { return (loggingLevel >= LEVEL_RESULT); }
    public static boolean isInfo() { return (loggingLevel >= LEVEL_INFO); }
    public static boolean isDebug() { return (loggingLevel >= LEVEL_DEBUG); }
    public static boolean isFull() { return (loggingLevel == LEVEL_FULL); }

    public static void logError(String msg) { log(msg, LEVEL_ERROR, new ApplicationError(msg)); }
    public static void logError(String msg, Throwable e) { log(msg, LEVEL_ERROR, e); }
    public static void logWarning(String msg) { log(msg, LEVEL_WARNING); }
    public static void logResult(String msg) { log(msg, LEVEL_RESULT); }
    public static void logInfo(String msg) { log(msg, LEVEL_INFO); }
    public static void logDebug(String msg) { log(msg, LEVEL_DEBUG);}
    public static void logFull(String msg) { log(msg, LEVEL_FULL); }

    private static final SimpleDateFormat formatter = new SimpleDateFormat ("[yyyy.MM.dd.HH:mm:ss]");
    
	public static boolean TRACE_ON = false;

    public static void logDebug(InputStream is) {
        try {
            int length = is.available();
            byte [] buffer = new byte[length];
            is.read(buffer, 0, length);
            String msg = new String(buffer);
            log(msg, LEVEL_DEBUG);
        } catch (IOException ex) {
            Logger.logError("Logger.logDebug() Unable to display message due to error",ex);
        }
    }

    public static synchronized void logStackTrace() {
        StackTraceElement [] stackTrace = new Exception("Test").getStackTrace();;
        for (int i=0; i < stackTrace.length; i++) {
            logDebug(stackTrace[i].toString());
        }
    }

    /**
     * Logs information to the log file if the level is currently being
     * logged.
     * @param msg The message to be logged.
     * @param level The level at which the information is logged.
     */
    private static void log(String msg, int level) {
        log(msg, level, null);
    }

    /**
     * Logs information to the log file if the level is currently being
     * logged. Also dumps a stack trace of the exception.
     * @param msg The message to be logged.
     * @param level The level at which the information is logged.
     */
    private static void log(String msg, int level, Throwable error) {
        try {
        	// Tests may never load settings!
	    	if (!loggingLevelLoaded && Settings.hasInstance()) {
	    		loggingLevel = Settings.getInstance().getLogLevel();
	    		loggingLevelLoaded = true;
	    	}
	        if (level <= loggingLevel) {
	            _log(msg, level, error);
	        }
        } catch (Exception e) {
            System.out.println("Unable to log message "+msg+" at level "+level+" due to error "+e);
            e.printStackTrace();
        }
    }
    
    private static synchronized void _log(String msg, int level, Throwable error) throws IOException {
        Calendar now = Calendar.getInstance();
        if (now.get(Calendar.DAY_OF_MONTH) != day) {
            if (day >= 0) archiveFile();
            day = now.get(Calendar.DAY_OF_MONTH);
        }
        String dateString = formatter.format(new Date());
        writeLine(getLoggingLevelPrefix(level) + dateString+" "+msg, error);
    }

    /**
     * Logs information to the log file regardless of the logging level. This
     * method should not be used. It is intended for objects like the domain 
     * manager that is used by the logger to initialise and therefore cannot use
     * normal logger functionality.
     * @param msg The message to be logged.
     */
    public static void logDirect(String msg) {
        try {
            String dateString = formatter.format(new Date());
            writeLine(getLoggingLevelPrefix(-1) + dateString+" "+msg);
        } catch (Exception e) {
            System.out.println("Unable to log direct message "+msg+" due to error "+e);
            e.printStackTrace();
        }
    }
    
    private static void writeLine(String line) throws IOException {
        writeLine(line, null);
    }

    private static synchronized void writeLine(String line, Throwable error) throws IOException {
    	if (inMemory) {
    		contents.append(line);
			contents.append("\n");
    		if (error != null) {
    			contents.append(error.toString());
    			contents.append("\n");
    			StackTraceElement[] sta = error.getStackTrace();
    			for (int i = 0; sta != null && i < sta.length; i++) {
    				contents.append(sta[i].toString());
    				contents.append("\n");
				}
    		}
    		changed = true;
    		if (timer == null) {
    	        timer = new Timer(5000, new LogTimer());
    	        timer.start();
    		}
    	} else {
    		if (logToStdOut) {
    			System.out.println(line);
    			if (error != null) {
    				error.printStackTrace(System.out);
    			}
    		} else {
    			String fileName = getLogFile();
    			PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName, true)));
    			out.println(line);
    			if (error != null) {
    				error.printStackTrace(out);
    			}
    			out.flush();
    			out.close();
    		}
    	}
    }
    
    public static synchronized void flushLog() {
        try {
        	if (inMemory && changed) {
    			String c = contents.toString();
        		if (logToStdOut) {
        			System.out.println(c);
        		} else {
        			String fileName = getLogFile();
        			PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName, true)));
        			out.println(c);
        			out.flush();
        			out.close();
        		}
        		contents.delete(0, contents.length());
        		changed = false;
        	}
        } catch (Exception e) {
            System.out.println("Unable to close log file due to error "+e);
            e.printStackTrace();
        }
    }

    public static String getLevelString() {
        if (loggingLevel == LEVEL_DEBUG) return "debug";
        else if (loggingLevel == LEVEL_INFO) return "info";
        else if (loggingLevel == LEVEL_RESULT) return "result";
        else if (loggingLevel == LEVEL_WARNING) return "warning";
        else if (loggingLevel == LEVEL_ERROR) return "error";
        else if (loggingLevel == LEVEL_FULL) return "full";
        else return "silent";
    }

	private static String getLoggingLevelPrefix(int level) {
        if (level == LEVEL_DEBUG) return "D";
        else if (level == LEVEL_INFO) return "I";
        else if (level == LEVEL_RESULT) return "R";
        else if (level == LEVEL_WARNING) return "W";
        else if (level == LEVEL_ERROR) return "E";
        else if (level == LEVEL_FULL) return "F";
        else return "X";
	}


    private static void archiveFile() {
        try {
            String fileName = getLogFile();
            File file = new File(fileName);
            if (archive) {
                SimpleDateFormat sdf = new SimpleDateFormat ("yyyy_MM_dd");
                String dateString = formatter.format(new Date());
                file.renameTo(new File(dateString+".log"));
            } else {
				long K = 1000;
				if (file.length() > 1000*K) {
					clearContents(500*K);
				}
            }
        } catch (Exception e) { }
    }

    public static void clearContents() {
        String fileName = getLogFile();
		new File(fileName).delete();
	}

    public static synchronized void clearContents(long qtyLinesToKeep) {
        String fileName = getLogFile();
        try {
            PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName, false)));
            BufferedReader in = new BufferedReader(new FileReader(fileName));
            String line = in.readLine();
            int size = 0;
            while (line != null) {
				size += line.length();
				if (size >= qtyLinesToKeep) {
					out.println(line);
				}
                line = in.readLine();
            }
            out.flush();
            out.close();
            in.close();
        } catch (IOException e) {}
    }

    public static String getContents() {
		return getContents(LEVEL_DEBUG);
	}

    public static synchronized String getContents(int level) {
        String fileName = getLogFile();
        try {
            BufferedReader in = new BufferedReader(new FileReader(fileName));
            StringBuffer sb = new StringBuffer();
            String line = in.readLine();
            while (line != null) {
				boolean excludeLine = false;
				if (line.length() > 1 && level < LEVEL_FULL) {
					char c = line.charAt(0);
					int charLevel = LEVEL_DEBUG;
					if (c == 'D') charLevel = LEVEL_DEBUG;
					else if (c == 'I') charLevel = LEVEL_INFO;
					else if (c == 'R') charLevel = LEVEL_RESULT;
					else if (c == 'W') charLevel = LEVEL_WARNING;
					else if (c == 'E') charLevel = LEVEL_ERROR;
					else if (c == 'F') charLevel = LEVEL_FULL;
					if (charLevel > level) excludeLine = true;
				}

				if (!excludeLine) {
					sb.append(line);
					sb.append("\n");
				}
                line = in.readLine();
            }
            in.close();
            return sb.toString();
        } catch (Exception e) {
            return null;
        }
    }

    public static void deleteLogFile() throws ApplicationError {
        String fileName = getLogFile();
        try {
            File file = new File(fileName);
            if (file.exists()) {
                file.delete();
            }
        } catch (Exception e) {
            throw new ApplicationError("Logger.deleteLogFile() could not delete file "+fileName);
        }
    }
	public static boolean isInMemory() {
		return inMemory;
	}
	public static void setInMemory(boolean v) {
		inMemory = v;
	}
	
	public static void setLogToStdOut(boolean v) {
		logToStdOut = v;
		if (logToStdOut) inMemory = false;
	}
	
	public static boolean isLogToStdOut() {
		return logToStdOut;
	}
	
}

class LogTimer implements ActionListener {

	public void actionPerformed(ActionEvent e) {
		Logger.flushLog();
	}
	
}