/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.util;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;

import mot.common.bean.DateTime;

/**
 * Implements a name value list from a BasicNameValueList.
 */
public class NameValuePairList implements NameValueList, Serializable {


	private BasicNameValueList list;
    
    public int getSize() {
        if (list == null) {
            return 0;
        } else {
            return list.getSize();
        }
    }

    public NameValuePair [] getAsArray() {
        if (list == null) {
            return null;
        } else {
            return list.getAsArray();
        }
    }

    public NameValuePairList() {
		this.list = new BasicNameValuePairList();
    }

    public NameValuePairList(BasicNameValueList list) {
		this.list = list;
    }

    public String getParameter(String name) {
		return list.getParameter(name);
    }

    public String [] getParameterValues(String name) {
		return list.getParameterValues(name);
    }

    public void add(String name, String value) throws ApplicationError {
		list.add(name, value);
    }

    public void add(String name, String [] values) throws ApplicationError {
        list.add(name, values);
    }

    public void add(String name, Integer value) throws ApplicationError {
		list.add(name, value);
    }

    public void add(String name, Long value) throws ApplicationError {
		list.add(name, value);
    }

    public void add(String name, Double value) throws ApplicationError {
		list.add(name, value);
    }

    public void add(String name, boolean value) throws ApplicationError {
		list.add(name, value);
    }

    public void add(BasicNameValueList v) throws ApplicationError {
    	if (v != null) {
    		list.add( v );
    	}
    }

	public void add(NameValuePair nameValuePair) {
		list.add(nameValuePair);
	}

    public String getAsSeperatedList(String seperator) {
		return list.getAsSeperatedList(seperator);
    }

    public Collection getKeys() {
		return list.getKeys();
    }


    public BasicNameValueList getBasicList() {
    	return list;
    }

    public void add(NameValueList v) throws ApplicationError {
    	if (v != null) {
    		list.add( v.getBasicList() );
    	}
    }

    public String getMandatoryParameter(String name) throws ApplicationError {
		String v = getParameter(name);
		if (v == null || v.length() == 0) throw new ApplicationError("NameValuePairList.getMandatoryParameter() Failed becuase parameter "+name+" not not exist or is empty");
		return v;
    }

    public String getParameterAsUpper(String name)  {
        String value = getParameter(name);
        if (value != null) value = value.toUpperCase();
        return value;
    }

    public boolean getParameterAsYesNo(String name) {
        String v = getParameter(name);
        return (v != null) && v.equalsIgnoreCase("yes");
    }

    public boolean getParameterAsYesNo(String name, boolean defaultValue) {
        String v = getParameter(name);
        if (v == null) {
            return defaultValue;
        } else {
            return v.equalsIgnoreCase("yes");
        }
    }

    public Boolean getParameterAsYesNoNull(String name) {
        String v = getParameter(name);
        if (v == null) {
            return null;
        } else {
            return new Boolean(v.equalsIgnoreCase("yes"));
        }
    }


    public int getParameterAsNonMandatoryInt(String name) {
		try {
			return getParameterAsInt(name);
		} catch (ValidationError e) {
			return 0;
		}
	}

    public int getParameterAsIntWithDefailt(String name, int defaultValue) {
		String valueString = getParameter(name);
		if (valueString != null) {
			try {
                return Integer.parseInt(valueString);
			} catch(NumberFormatException e) {
				return defaultValue;
            }
		}

		return defaultValue;
	}

    public int getParameterAsInt(String name) throws ValidationError {
		String valueString = getParameter(name);
		int value = 0;

		if (valueString != null) {
			try {
                value = Integer.parseInt(valueString);
			} catch(NumberFormatException e) {
				throw new ValidationError("Invalid numeric value "+valueString+" - Must be a valid whole number");
            }
		}

		return value;
    }

    public int getParameterAsValidInt(String name) throws ApplicationError {
		String valueString = getParameter(name);
		int value = 0;

		if (valueString != null) {
			try {
                value = Integer.parseInt(valueString);
			} catch(NumberFormatException e) {
				throw new ApplicationError("Invalid numeric value "+valueString+" - Must be a valid whole number");
            }
		}

		return value;
    }

    public Integer getParameterAsInteger(String name) throws ValidationError {
		String valueString = getParameter(name);
		Integer value = null;

		if (valueString != null && valueString.length() > 0) {
			try {
				value = new Integer(valueString);
			} catch(NumberFormatException e) {
				throw new ValidationError("Invalid numeric value "+valueString+" - Must be a valid whole number");
			}
		}

		return value;
    }

    public int getParameterAsMandatoryInt(String name) throws ApplicationError {
		String valueString = getParameter(name);
		int value = 0;

		if (valueString != null && valueString.length() > 0) {
			try {
			value = Integer.parseInt(valueString);
			} catch(NumberFormatException e) {
			throw new ApplicationError("NameValuePairList.getParameterAsMandatoryInt() Parameter "+name+" must be a whole number (integer)");
			}
		} else {
			throw new ApplicationError("NameValuePairList.getParameterAsMandatoryInt() Parameter "+name+" is mandatory");
		}

		return value;
    }

    public Integer getParameterAsMandatoryInteger(String name) throws ApplicationError {
		String valueString = getParameter(name);
		Integer value = null;

		if (valueString != null && valueString.length() > 0) {
			try {
			value = new Integer(valueString);
			} catch(NumberFormatException e) {
			throw new ApplicationError("NameValuePairList.getParameterAsMandatoryInteger() Parameter "+name+" must be a whole number (integer)");
			}
		} else {
			throw new ApplicationError("NameValuePairList.getParameterAsMandatoryInteger() Parameter "+name+" is mandatory");
		}

		return value;
    }

    public Long getParameterAsLong(String name) throws ValidationError {
		String valueString = getParameter(name);
		Long value = null;

		if (valueString != null) {
			try {
                value = new Long(valueString);
			} catch(NumberFormatException e) {
				throw new ValidationError("Invalid numeric value "+valueString+" - Must be a valid whole number");
            }
		}

		return value;
    }

    public Long getParameterAsMandatoryLong(String name, String fieldName) throws ValidationError {
		Long v = getParameterAsLong(name);
		if (v == null) {
			throw new ValidationError("Mandatory Field "+fieldName+" - Must be a valid whole number");
		} else {
			return v;
		}
    }

    public boolean getParameterAsBoolean(String name) throws ValidationError {
        return getParameterAsBoolean(name, false);
    }

    public boolean getParameterAsBoolean(String name, boolean defaultValue) throws ValidationError {
		String valueString = getParameter(name);

		if (valueString == null) {
			return defaultValue;
		} else {
			return valueString.equalsIgnoreCase("true") ||
                   valueString.equalsIgnoreCase("yes") ||
                   valueString.equalsIgnoreCase("y");
		}
    }

    public DateTime getDate(String name) throws ValidationError {
    	return getDate(name, true);
    }

    public DateTime getDate(String name, boolean mandatory) throws ValidationError {
        String day = getParameter(name+"Day");
        String month = getParameter(name+"Month");
        String year = getParameter(name+"Year");

        if (mandatory || day != null) {
        	return new DateTime(day, month, year);
        } else {
        	return null;
        }
    }

    public String getAsInputList(boolean hidden) {
		Iterator keys = list.getKeys().iterator();
		StringBuffer result = new StringBuffer();

		while (keys.hasNext()) {
			String name = (String) keys.next();
			String value = list.getParameter(name);
            if (value != null) {
                //value = HTMLEncoder.encode(value);
                if (hidden) {
                    result.append("<input type=\"hidden\" name=\""+name+"\" value=\""+value+"\"");
                } else {
                    result.append("<input type=\"text\" name=\""+name+"\" value=\""+value+"\"");
                }
                result.append("\n");
            }
		}

		return result.toString();
    }

	public String getAsAttributeList() {
		return list.getAsAttributeList();
	}


}