/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.util;

import java.awt.Color;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import mot.common.bean.Size;

/**
 * Utility class that loads and stores all the configuration parameters
 * the system uses. All properties should be loaded through the property
 * manager.
 *
 * The Property Manager can also determine if the current deployment is a Live deployment
 * or a Development deployment.
 */
public class PropertyManager {
	private static ResourceBundle properties = null;
	private static ResourceBundle version = null;

    public static Size getMandatoryAsSize(String name) throws ApplicationError {
        return new Size( PropertyManager.getMandatoryAsInt(name+".width"), 
                          PropertyManager.getMandatoryAsInt(name+".height") );
    }

    public static Color getMandatoryAsColorNoError(String name) {
    	try {
    		return getMandatoryAsColor(name);
    	} catch (ApplicationError ex) {
    		Logger.logWarning("PropertyManager.getMandatoryAsColorNoError() Invalid Property : Color "+name+" does not exist");
    		return Color.WHITE;
    	}
    }
    
    public static Color getMandatoryAsColor(String name) throws ApplicationError {
        return new Color( PropertyManager.getMandatoryAsInt(name+".red"), 
                          PropertyManager.getMandatoryAsInt(name+".green"), 
                          PropertyManager.getMandatoryAsInt(name+".blue") );
    }
    
    public static Color getMandatoryAsColor(String name, double factor) throws ApplicationError {
        return new Color( (int)(PropertyManager.getMandatoryAsInt(name+".red")*factor), 
        				  (int)(PropertyManager.getMandatoryAsInt(name+".green")*factor), 
        				  (int)(PropertyManager.getMandatoryAsInt(name+".blue")*factor) );
    }
    
    public static int getMandatoryAsInt(String name) throws ApplicationError {
        String r = getMandatory(name);
        try {
            return Integer.parseInt(r);
        } catch (NumberFormatException ex) {
            throw new ApplicationError("PropertyManager.getMandatoryAsInt() property "+name+" is not a valid int");
        }
    }
    
    public static String [] getMandatoryAsSplit(String name, String seperator) throws ApplicationError {
        String r = getMandatory(name);
        return r.split(seperator);
    }

    public static String [] getAsSplit(String name, String seperator) {
        String r = get(name);
        if (r == null) {
            return null;
        }
        return r.split(seperator);
    }
    
    public static float getMandatoryAsFloat(String name) throws ApplicationError {
        String r = getMandatory(name);
        try {
            return Float.parseFloat(r);
        } catch (NumberFormatException ex) {
            throw new ApplicationError("PropertyManager.getMandatoryAsFloat() property "+name+" is not a valid float");
        }
    }
    
    public static double getMandatoryAsDouble(String name) throws ApplicationError {
        String r = getMandatory(name);
        try {
            return Double.parseDouble(r);
        } catch (NumberFormatException ex) {
            throw new ApplicationError("PropertyManager.getMandatoryAsDouble() property "+name+" is not a valid double");
        }
    }
    
    public static String getMandatory(String name) throws ApplicationError {
        String r = get(name);
        if (r == null) {
            throw new ApplicationError("PropertyManager.getMandatory() property "+name+" does not exist");
        }
        return r;
    }
    
    public static String getMandatoryNoError(String name) {
        String r = get(name);
        if (r == null) {
            Logger.logError("", new ApplicationError("PropertyManager.getMandatory() property "+name+" does not exist"));
        }
        return r;
    }
    
    public static String getMandatoryWithSpaces(String name) throws ApplicationError  {
    	int index = name.indexOf(' ');
    	if (index > -1) {
    		name = name.replace(' ', '_');
    	}
    	return getMandatory(name);
    }
    

    public static int getMandatoryNoErrorAsInt(String name) {
        try {
	        int r = getMandatoryAsInt(name);
            return r;
        } catch (Exception ex) {
            Logger.logError("PropertyManager.getMandatoryAsInt() property "+name+" is not a valid int");
        }
        return 0;
    }

    public static double getMandatoryNoErrorAsDouble(String name, double def) {
        try {
        	double r = getMandatoryAsDouble(name);
            return r;
        } catch (Exception ex) {
            Logger.logError("PropertyManager.getMandatoryNoErrorAsDouble() property "+name+" is not a valid int");
        }
        return def;
    }
    
    public static int getMandatoryNoErrorAsInt(String name, int def) {
        try {
	        int r = getMandatoryAsInt(name);
            return r;
        } catch (Exception ex) {
            Logger.logError("PropertyManager.getMandatoryAsInt() property "+name+" is not a valid int");
        }
        return def;
    }

    public static Size getMandatoryNoErrorAsSize(String name) {
        try {
        	Size r = getMandatoryAsSize(name);
            return r;
        } catch (Exception ex) {
            Logger.logError("PropertyManager.getMandatoryNoErrorAsSize() property "+name+" is not a valid int");
        }
        return null;
    }
 
    public static String [] getMandatoryAsSplitNoError(String name, String seperator) {
        String [] r = getAsSplit(name, seperator);
        if (r == null) {
            Logger.logError("", new ApplicationError("PropertyManager.getMandatory() property "+name+" does not exist"));
        }
        return r;
    }

    public static String get(String name) {
        if (properties == null) {
            properties = ResourceBundle.getBundle("mot");
        }
        try {
			return properties.getString(name);
		} catch (MissingResourceException e) {
			return null;
		}
    }

    public static boolean isDevelopment() throws ApplicationError {
        String answer = get("development");
        return (answer != null && answer.equalsIgnoreCase("true"));
    }

    public static String getVersion() {
        if (version == null) {
        	version = ResourceBundle.getBundle("version");
        }
        try {
			return version.getString("version");
		} catch (MissingResourceException e) {
			return null;
		}
    }

	private static Color selectedBackgroundColor = null;
	public static Color getSelectedBackgroundColor() {
		if (selectedBackgroundColor == null) {
			selectedBackgroundColor = PropertyManager.getMandatoryAsColorNoError("list.color.background.selected");
		}
		return selectedBackgroundColor;
	}
}