/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.util;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * 
 */
public class UnsortedNameValuePairList extends XMLBean implements Serializable, BasicNameValueList {
    private ArrayList list = new ArrayList();

    public UnsortedNameValuePairList() {
    }

    public int getSize() {
        if (list == null) {
            return 0;
        } else {
            return list.size();
        }
    }

    public NameValuePair [] getAsArray() {
		if (list == null) {
			return null;
		}
		return (NameValuePair []) list.toArray(new NameValuePair [list.size()]);
	}

    public String getAsSeperatedList(String seperator) {
    	String r = "";

        boolean first = true;
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				NameValuePair nvp = (NameValuePair) i.next();
				String name = nvp.getName();
				String value = nvp.getValue();
				if (first) {
					first = false;
				} else {
					r += seperator;
				}
				r += name+"="+value;
			}
		}

		return r;
	}

    public String getAsAttributeList() {
    	StringBuffer r = new StringBuffer();

        boolean first = true;
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				NameValuePair nvp = (NameValuePair) i.next();
				String name = nvp.getName();
				String value = nvp.getValue();
				if (first) {
					first = false;
				} else {
					r.append(" ");
				}
				r.append(name);
				r.append("=\"");
				r.append(value);
				r.append("\"");
			}
		}

		return r.toString();
	}

	public void add(NameValuePair nvp) {
		list.add(nvp);
	}

	public void add(String name, String value) {
		if (name == null || value == null) return;
		list.add(new NameValuePair(name, value));
	}

    public void add(String name, Integer value) throws ApplicationError {
    	if (value != null) {
    		add(name, ""+value);
    	}
    }

    public void add(String name, Long value) throws ApplicationError {
    	if (value != null) {
    		add(name, ""+value);
    	}
    }

    public void add(String name, Double value) throws ApplicationError {
    	if (value != null) {
    		add(name, ""+value);
    	}
    }

    public void add(String name, int value) {
  		add(name, ""+value);
    }

    public void add(String name, boolean value) {
    	if (value) {
    		add(name, "Y");
    	} else {
    		add(name, "N");
    	}
    }

	public void add(BasicNameValueList v) {
		if (v != null) {
			Collection c = v.getKeys();
			if ( c != null) {
				Iterator i = c.iterator();
				while (i.hasNext()) {
					String key = (String) i.next();
					add(key, (String)v.getParameter(key));
				}
			}
		}
	}
	
    public void add(String name, String [] values) throws ApplicationError {
        if (name == null || values == null || values.length == 0) {
            return;
        }
        for (int i=0; i<values.length; i++) {
            add(name+i, values[i]);
        }
    }

	public String getParameter(String name) {
		if (name == null) return null;
		if (list == null) return null;
		for (Iterator i = list.iterator(); i.hasNext();) {
			NameValuePair nvp = (NameValuePair) i.next();
			if (Comp.same(nvp.getName(), name)) {
				return nvp.getValue();
			}
		}
		return null;
	}

	public String [] getParameterValues(String name) {
		if (name == null) return null;
        ArrayList list = new ArrayList();

        String v = getParameter(name);
        if (v != null) {
            list.add(v);
        }

        v = getParameter(name+"0");
        int i = 0;
        while (v != null) {
            list.add(v);
            i++;
            v = getParameter(name+i);
        }

        if (list.size() == 0) {
            return null;
        } else {
            return (String []) list.toArray(new String[list.size()]);
        }
	}

	protected String getElementName() { return "nvp-list"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
        NameValuePair [] l = getAsArray();
        for (int i=0; l != null && i < l.length; i++) {
            if (l[i] != null) {
                l[i].toXML(writer);
            }
        }
	}

	public Collection getKeys() {
		List r = null;
		if (list != null) {
			Iterator i = list.iterator();
			while (i.hasNext()) {
				NameValuePair nvp = (NameValuePair) i.next();
				if (r == null) {
					r = new ArrayList();
				}
				r.add(nvp.getName());
			}
		}

		return r;
	}

	public int getParameterAsValidInt(String name) throws ApplicationError {
		String v = getParameter(name);
		if (v == null) return 0;
		return Integer.parseInt(v);
	}


}