/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.util;

import java.util.ArrayList;
import java.util.Date;
import java.util.Random;

import javax.management.RuntimeErrorException;

import mot.common.bean.LocatedArea;
import mot.common.bean.Location;

/**
 * Common utilities class.
 */
public class Utils {

	private static Random random = null;

	private static synchronized Random getRandom() {
		if (random == null) {
			random = new Random(new Date().getTime());
		}
		return random;
	}

    /**
     * Used for testing to predetermine what random event will occur.
     */
	private static class NextRandomProbabilityTupple { public double value; public Double check; public NextRandomProbabilityTupple(double value, Double check) { this.value=value; this.check=check; }} 
    private static ArrayList<NextRandomProbabilityTupple> nextRandomProbability = null;

	/**
	 * The parameter probability must be a number between 0 and 1.
	 * Returns true if any only if a random number between 0 and 1 is less than probability.
	 */
	public static boolean random(double probability) {
        if (nextRandomProbability != null && nextRandomProbability.size() > 0) {
        	NextRandomProbabilityTupple tupple = nextRandomProbability.get(0);
        	nextRandomProbability.remove(0);
            if (tupple.check != null && (tupple.check < probability-0.01 || tupple.check > probability+0.01)) {
            	throw new RuntimeErrorException(new Error(), "Expected "+tupple.check+" got "+probability);
            }
            double v = tupple.value;
            return (v < probability);
        } else {
            double v = getRandom().nextDouble();
            return (v < probability);
        }
	}

    public static void clearAndSetNextRandomProbability(double p) {
    	clearAndSetNextRandomProbability(p, null);
    }
    
    public static void setNextRandomProbability(double p) {
    	setNextRandomProbability(p, null);
    }
 
    public static void clearAndSetNextRandomProbability(double p, Double check) {
        clearNextRandomProbability();
        setNextRandomProbability(p, check);
    }
    
    public static void setNextRandomProbability(double p, Double check) {
        if (nextRandomProbability == null) nextRandomProbability = new ArrayList();
        nextRandomProbability.add(new NextRandomProbabilityTupple(p, check));
    }
 
    public static void clearNextRandomProbability() {
        nextRandomProbability = null;
    }

    /**
     * Used for testing to predetermine what random event will occur.
     */
    private static ArrayList nextRandomDouble = null;

	/**
	 * returns a random number between 0 and p-1 (inclusive).
	 */
	public static double randomDouble(double p) {
        if (nextRandomDouble != null && nextRandomDouble.size() > 0) {
            double r = ((Double)nextRandomDouble.get(0)).doubleValue();
            nextRandomDouble.remove(0);
            return r*p;
        } else {
            return (getRandom().nextDouble())*p;
        }
	}

    public static void clearAndSetNextRandomDouble(double p) {
    	clearNextRandomDouble();
    	setNextRandomDouble(p);
    }
 
    public static void setNextRandomDouble(double p) {
        if (nextRandomDouble == null) nextRandomDouble = new ArrayList();
        nextRandomDouble.add(new Double(p));
    }
 
    public static void clearNextRandomDouble() {
        nextRandomDouble = null;
    }

   
    /**
     * Used for testing to predetermine what random event will occur.
     */
    private static ArrayList<NextRandomIntTupple> nextRandomInt = null;

	/**
	 * returns a random number between 0 and p-1 (inclusive).
	 */
	public static int random(int p) {
        if (p == 0) return 0;
        if (nextRandomInt != null && nextRandomInt.size() > 0) {
            NextRandomIntTupple tupple = nextRandomInt.get(0);
            nextRandomInt.remove(0);
            if (tupple.paramCheck != null && tupple.paramCheck != p) {
            	throw new RuntimeErrorException(new Error(), "Expected "+tupple.paramCheck+" got "+p);
            }
            return tupple.returnValue;
        } else {
            return getRandom().nextInt(p);
        }
	}
    
    public static void clearAndSetNextRandomInts(int p1, int p2, int p3, int p4, int p5) {
    	nextRandomInt = new ArrayList();
    	setNextRandomInt(p1, null);
    	setNextRandomInt(p2, null);
    	setNextRandomInt(p3, null);
    	setNextRandomInt(p4, null);
    	setNextRandomInt(p5, null);
    }
    
    public static void clearAndSetNextRandomInt(int p) {
    	nextRandomInt = new ArrayList();
    	setNextRandomInt(p, null);
    }
    
    public static void clearAndSetNextRandomInt(int p, Integer valueCheck) {
    	nextRandomInt = new ArrayList();
    	setNextRandomInt(p, valueCheck);
    }
    
    public static void setNextRandomInt(int p) {
    	setNextRandomInt(p, null);
    }
    
    public static void setNextRandomInt(int p, Integer valueCheck) {
        if (nextRandomInt == null) nextRandomInt = new ArrayList();
        nextRandomInt.add(new NextRandomIntTupple(p, valueCheck));
    }

    public static void clearNextRandomInt() {
        nextRandomInt = null;
    }

    public static void clearAll() {
    	clearNextRandomInt();
    	clearNextRandomDouble();
    	clearNextRandomProbability();
    }
    
	/**
	 * returns a random number between 0 and p-1 (inclusive).
	 */
	public static long random(long p) {
        long v = getRandom().nextLong();
        if (v < 0) v = -v;
		return v % p;
	}

	/**
	 * Select a random item from the list.
	 */
	public static Object random(ArrayList items) {
		if (items == null || items.size() == 0) return null;
		int index = random(items.size());
		return items.get(index);
	}

	public static Object random(Object[] items) {
		if (items == null || items.length == 0) return null;
		int index = random(items.length);
		return items[index];
	}

	public static LocatedArea getClosest(LocatedArea [] locationOccupiers, Location location) {
		if (locationOccupiers == null) return null;
		if (locationOccupiers.length == 1) return locationOccupiers[0];
		LocatedArea r = locationOccupiers[0];
		int dist = locationOccupiers[0].getLocation().getDistanceTo(location);
		for (int i = 1; i < locationOccupiers.length; i++) {
			int d = locationOccupiers[i].getLocation().getDistanceTo(location);
			if (d < dist) {
				r = locationOccupiers[i];
				dist = d;
			}
		}
		return r;
	}
}

class NextRandomIntTupple {
	public NextRandomIntTupple(int returnValue, Integer paramCheck) {
		this.returnValue = returnValue;
		this.paramCheck = paramCheck;
	}
	public int returnValue;
	public Integer paramCheck;
}

