/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.vo;

import mot.common.bean.NamedObjectI;
import mot.common.designs.CraftDesign;
import mot.common.designs.WeaponBayDesign;
import mot.common.designs.WeaponBaysDesign;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Specials;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;

public class CraftDesignVO implements ValueObject, NamedObjectI {

    private String name;
	private String image;

	private String hull;
	private String engine;

    private WeaponBayVO [] weapons;
    
	private String [] computers;
	private String [] shields;
	private String [] scanners;
	private String [] specials;
	
	private double buildQuality;
	

	private transient Hull hullTech;
	private transient Engine engineTech;

	private transient Computers computersList;
	private transient Scanners scannersList;
	private transient Shields shieldsList;
	private transient Specials specialsList;
    
    public CraftDesignVO() {
    }
    
    public CraftDesignVO(CraftDesign cd, double buildQuality) {
        this.buildQuality = buildQuality;
		name = cd.getName();
        image = cd.getImage();
        hull = cd.getHullTechName();
        engine = cd.getEngineTechName();
        computers = cd.getComputersTechNames();
        shields = cd.getShieldsTechNames();
        scanners = cd.getScannersTechNames();
        specials = cd.getSpecialsTechNames();
        WeaponBaysDesign wbList = cd.getWeaponBays();
        if (wbList != null) {
            WeaponBayDesign [] wbs = wbList.getBaysAsArray();
            if (wbs != null) {
                weapons = new WeaponBayVO [wbs.length];
                for (int i=0; i<wbs.length; i++) {
                    weapons[i] = new WeaponBayVO(wbs[i]);
                }
            }
        }
    }
    
    public CraftDesign getCraftDesign() throws ApplicationError {
        CraftDesign cd = new CraftDesign(name);
        cd.setImage(image);
        cd.setHull(hull);
        cd.setEngine(engine);
        cd.setComputers(getComputersAsList());
        cd.setShields(getShieldsAsList());
        cd.setScanners(getScannersAsList());
        cd.setSpecials(getSpecialsAsList());
        if (weapons != null) {
            WeaponBaysDesign bays = new WeaponBaysDesign();
            for (int i=0; i<weapons.length; i++) {
                WeaponBayDesign bay = weapons[i].getWeaponBayDesign();
                bays.add(bay);
            }
            cd.setWeaponBays(bays);
        }
        return cd;
    }

    public void setName(String name) {
        this.name = name;
    }


    public String getName() {
        return name;
    }


    public void setImage(String image) {
        this.image = image;
    }


    public String getImage() {
        return image;
    }


    public void setHull(String hull) {
        this.hull = hull;
    }


    public String getHull() {
        return hull;
    }

    public Hull getHullTech() throws ApplicationError {
        if (hullTech == null && hull != null) {
            hullTech = Technologies.getHull(hull);
        }
        return hullTech;
    }

    public void setEngine(String engine) {
        this.engine = engine;
    }


    public String getEngine() {
        return engine;
    }

    public Engine getEngineTech() throws ApplicationError {
        if (engineTech == null && engine != null) {
            engineTech = Technologies.getEngine(engine);
        }
        return engineTech;
    }

    public void setComputers(String[] computers) {
        this.computers = computers;
    }

    public Computers getComputersAsList() throws ApplicationError {
        if (computersList == null && computers != null) {
            computersList = new Computers(computers);
        }
        return computersList;
    }

    public String[] getComputers() {
        return computers;
    }

    public void setScanners(String[] scanners) {
        this.scanners = scanners;
    }

    public Scanners getScannersAsList() throws ApplicationError {
        if (scannersList == null && scanners != null) {
            scannersList = new Scanners(scanners);
        }
        return scannersList;
    }

    public String[] getScanners() {
        return scanners;
    }

    public void setShields(String[] shields) {
        this.shields = shields;
    }

    public Shields getShieldsAsList() throws ApplicationError {
        if (shieldsList == null && shields != null) {
            shieldsList = new Shields(shields);
        }
        return shieldsList;
    }

    public String[] getShields() {
        return shields;
    }

    public void setSpecials(String[] specials) {
        this.specials = specials;
    }

    public Specials getSpecialsAsList() throws ApplicationError {
        if (specialsList == null && specials != null) {
            specialsList = new Specials(specials);
        }
        return specialsList;
    }

    public String[] getSpecials() {
        return specials;
    }


    public void setWeapons(WeaponBayVO[] weapons) {
        this.weapons = weapons;
    }


    public WeaponBayVO[] getWeapons() {
        return weapons;
    }

	public double getBuildQuality() {
		return buildQuality;
	}

	public void setBuildQuality(double buildQuality) {
		this.buildQuality = buildQuality;
	}

	@Override
	public String toString() {
		return "CraftDesignVO ["+getName()+"]";
	}

}