/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.vo;

import java.util.Arrays;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.BasicNameValuePairList;

public class PlayerDetailsVO implements ValueObject {
	
	private String name;
	
    private int totalIncomeWithTrade;
    private int totalIncomeWithoutTrade;
    private int tradeIncome;
    private int totalMaintenanceCost;
    private int spiesMaintenanceCost;
	
    private int totalProduction;
    private int totalPopulation;
    
    private String [] moralChoices;

    private int spyRecruitTotal;
	private int spyRecruitCosts;
	private double spyRecruitChance;
	private BasicNameValuePairList espionageSucessChances;
	private BasicNameValuePairList sabotageSucessChances;
	private BasicNameValuePairList espionageTargets;
	
    /**
     * A list of name value pairs where each name is the name of the player with whom this player 
     * has assigned spies to espionage and the value is the quantity.
     */
    private BasicNameValueList qtySpiesEspionage;

    /**
     * A list of name value pairs where each name is the name of the player with whom this player 
     * has assigned spies to sabotage and the value is the quantity.
     */
    private BasicNameValueList qtySpiesSabotage;

    /**
     * This is a list of things that a computer player remembers and effect the relations of the player who owns the proxy this information is being sent to. 
     */
    private StrategistEventVO [] strategistEvents;

	public PlayerDetailsVO() {
	}

	public PlayerDetailsVO(String name, int totalIncomeWithTrade, int totalIncomeWithoutTrade, int tradeIncome, int totalMaintenanceCost, int spiesMaintenanceCost, BasicNameValuePairList espionageSucessChances, BasicNameValuePairList sabotageSucessChances, BasicNameValuePairList espionageTargets) {
		this.name = name;
		this.totalIncomeWithTrade = totalIncomeWithTrade;
		this.totalIncomeWithoutTrade = totalIncomeWithoutTrade;
		this.tradeIncome = tradeIncome;
		this.totalMaintenanceCost = totalMaintenanceCost;
		this.spiesMaintenanceCost = spiesMaintenanceCost;
		this.espionageSucessChances = espionageSucessChances;
		this.sabotageSucessChances = sabotageSucessChances;		
		this.espionageTargets = espionageTargets;
	}

	public String toString() {
		return "PlayerDetailsVO ["+name+"]";
	}

    public void setTotalIncomeWithTrade(int totalIncomeWithTrade) {
        this.totalIncomeWithTrade = totalIncomeWithTrade;
    }


    public int getTotalIncomeWithTrade() {
        return totalIncomeWithTrade;
    }


    public void setTotalIncomeWithoutTrade(int totalIncomeWithoutTrade) {
        this.totalIncomeWithoutTrade = totalIncomeWithoutTrade;
    }


    public int getTotalIncomeWithoutTrade() {
        return totalIncomeWithoutTrade;
    }


    public void setTradeIncome(int tradeIncome) {
        this.tradeIncome = tradeIncome;
    }


    public int getTradeIncome() {
        return tradeIncome;
    }

    public void setTotalMaintenanceCost(int totalMaintenanceCost) {
        this.totalMaintenanceCost = totalMaintenanceCost;
    }


    public int getTotalMaintenanceCost() {
        return totalMaintenanceCost;
    }

    public void setSpiesMaintenanceCost(int spiesMaintenanceCost) {
        this.spiesMaintenanceCost = spiesMaintenanceCost;
    }


    public int getSpiesMaintenanceCost() {
        return spiesMaintenanceCost;
    }
    
    public void setTotalProduction(int totalProduction) {
        this.totalProduction = totalProduction;
    }


    public int getTotalProduction() {
        return totalProduction;
    }


    public void setTotalPopulation(int totalPopulation) {
        this.totalPopulation = totalPopulation;
    }


    public int getTotalPopulation() {
        return totalPopulation;
    }


	public String[] getMoralChoices() {
		return moralChoices;
	}

	public void setMoralChoices(String[] moralChoices) {
		this.moralChoices = moralChoices;
	}


	public int getSpyRecruitCosts() {
		return spyRecruitCosts;
	}

	public void setSpyRecruitCosts(int spyRecruitCosts) {
		this.spyRecruitCosts = spyRecruitCosts;
	}

	public double getSpyRecruitChance() {
		return spyRecruitChance;
	}

	public void setSpyRecruitChance(double spyRecruitChance) {
		this.spyRecruitChance = spyRecruitChance;
	}

    public void setSpyRecruitTotal(int spyRecruitTotal) {
        this.spyRecruitTotal = spyRecruitTotal;
    }

    public int getSpyRecruitTotal() {
        return spyRecruitTotal;
    }

	public BasicNameValuePairList getEspionageSucessChances() {
		return espionageSucessChances;
	}

	public void setEspionageSucessChances(BasicNameValuePairList espionageSucessChances) {
		this.espionageSucessChances = espionageSucessChances;
	}

	public BasicNameValuePairList getSabotageSucessChances() {
		return sabotageSucessChances;
	}

	public void setSabotageSucessChances(BasicNameValuePairList sabotageSucessChances) {
		this.sabotageSucessChances = sabotageSucessChances;
	}

	public BasicNameValuePairList getEspionageTargets() {
		return espionageTargets;
	}

	public void setEspionageTargets(BasicNameValuePairList espionageTargets) {
		this.espionageTargets = espionageTargets;
	}

	public int getEspionageSuccessChance(String player) throws ApplicationError {
    	if (espionageSucessChances != null) {
   			return espionageSucessChances.getParameterAsValidInt(player);
    	} else {
    		return 0;
    	}
	}
	
	public int getSabotageSuccessChance(String player) throws ApplicationError {
    	if (sabotageSucessChances != null) {
   			return sabotageSucessChances.getParameterAsValidInt(player);
    	} else {
    		return 0;
    	}
	}

	public String getEspionageTarget(String player) throws ApplicationError {
    	if (espionageTargets != null) {
   			return espionageTargets.getParameter(player);
    	} else {
    		return null;
    	}
	}

    public void setQtySpiesEspionage(BasicNameValueList qtySpiesEspionage) {
        this.qtySpiesEspionage = qtySpiesEspionage;
    }

    public BasicNameValueList getQtySpiesEspionage() {
        return qtySpiesEspionage;
    }

    public void setQtySpiesSabotage(BasicNameValueList qtySpiesSabotage) {
        this.qtySpiesSabotage = qtySpiesSabotage;
    }


    public BasicNameValueList getQtySpiesSabotage() {
        return qtySpiesSabotage;
    }

    public void setStrategistEvents(StrategistEventVO[] diplomaticEvents) {
        this.strategistEvents = diplomaticEvents;
    }

    public StrategistEventVO[] getStrategistEvents() {
        return strategistEvents;
    }
    
	public void setStrategistEvent(StrategistEventVO v) {
		if (strategistEvents == null) {
			strategistEvents = new StrategistEventVO[1];
			strategistEvents[0] = v;
		} else {
			strategistEvents = Arrays.copyOf(strategistEvents, strategistEvents.length+1);
			strategistEvents[strategistEvents.length-1] = v;
		}
	}

	public void addStrategistEvents(StrategistEventVO[] dea) {
		if (dea == null) return;
		for (int j = 0; j < dea.length; j++) {
			setStrategistEvent(dea[j]);
		}
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

}
