/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.vo;

import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

public class TechnologyListVO extends XMLBean implements ValueObject {

    private String [] fullNames;
    private transient TechnologyList _technologyList; 

    public TechnologyListVO() {
    }

    public TechnologyListVO(TechnologyList tl) {
    	if (tl != null) {
    		this.fullNames = tl.getFullNamesAsArray();
    	}
    }

    public TechnologyListVO(Technology t) {
        this.fullNames = new String[1];
        this.fullNames[0] = t.getFullName();
    }

    public TechnologyListVO(TechnologyListVO technologies) throws ApplicationError {
    	if (technologies != null) {
    		Technology[] ta = technologies.getAllAsArray();
    		if (ta != null) {
    			for (int i = 0; i < ta.length; i++) {
					add(ta[i]);
				}
    		}
    	}
	}

	public void setFullName(String fullName) {
    	add(fullName);
    }

    public void setFullNames(String[] fullNames) {
        this.fullNames = fullNames;
    }

    public String getFullNamesAsString() {
        return Comp.toString(fullNames, ", ");
    }

    public String[] getFullNames() {
        return fullNames;
    }

	public String getNames(String separator) throws ApplicationError {
		TechnologyList l = getAllAsList();
		if (l == null) return null;
		return l.getNames(", ");
	}

    public TechnologyList getAllAsList() throws ApplicationError {
        if (_technologyList == null && fullNames != null && fullNames.length > 0) {
            _technologyList = new GeneralTechnologyList(fullNames);
        }
        return _technologyList;
    }

    
	@Override
	public String toString() {
		return "TechnologyListVO ["+Comp.toString(fullNames)+"]";
	}

	public int getQuantity() {
		if (fullNames == null) {
			return 0;
		} else {
			return fullNames.length;
		}
	}

	public void remove(Technology tech) {
		if (tech != null) {
			remove(tech.getFullName());
		}
	}

	public void remove(String tech) {
		if (tech != null) {
			fullNames = Comp.remove(fullNames, tech);
			_technologyList = null;
		}
	}

	public void add(Technology tech) {
		if (tech != null) {
			add(tech.getFullName());
		}
	}

	public void add(String tech) {
		if (tech != null) {
			fullNames = Comp.add(fullNames, tech);
			_technologyList = null;
		}
	}

	public Technology [] getAllAsArray() throws ApplicationError {
		TechnologyList l = getAllAsList();
		if (l != null) {
			return l.getAllAsArray();
		} else {
			return null;
		}
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (fullNames != null) {
			for (int i = 0; i < fullNames.length; i++) {				
				writer.addElement("full-name", fullNames[i]);
			}
		}	
	}

	protected String getElementName() {
		return "technology-list-vo";
	}

}