/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.xml;

import java.util.ArrayList;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import mot.common.bean.MotColor;
import mot.common.bean.Size;
import mot.common.setup.SetupOption;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;

public class DomNode {

	private Node node;

	public DomNode(Node node) {
		this.node = node;
	}

	public String getNodeName() {
		return node.getNodeName();
	}
	
	public String getNodeValueByName_String(String name) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				return child.getTextContent();
			}
		}
		return null;
	}

	public int getNodeValueByName_Int(String name) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				String v = child.getTextContent();
				try {					
					return Integer.parseInt(v);
				} catch (NumberFormatException e) {
					Logger.logDebug("On Node"+child.toString());
					throw e;
				}
			}
		}
		return 0;
	}

	public int getNodeValueByName_Int(String name, int defaultValue) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				String v = child.getTextContent();
				if (v == null) {
					return defaultValue;
				} else {
					return Integer.parseInt(v);
				}
			}
		}
		return 0;
	}

	public boolean getNodeValueByName_Bool(String name) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				return Comp.same("true", child.getTextContent());
			}
		}
		return false;
	}

	public boolean getNodeValueByName_Bool(String name, boolean defaultValue) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				return Comp.same("true", child.getTextContent());
			}
		}
		return defaultValue;
	}

	public Boolean getNodeValueByName_Boolean(String name) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				return Comp.same("true", child.getTextContent());
			}
		}
		return null;
	}

	public Double getNodeValueByName_Double(String name) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				return new Double(child.getTextContent());
			}
		}
		return null;
	}

	public Integer getNodeValueByName_Integer(String name) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				return new Integer(child.getTextContent());
			}
		}
		return null;
	}


	public Technology getNodeValueByName_Technology(String name) throws ApplicationError {
		String techFullName = getNodeValueByName_String(name);
		if (techFullName == null) return null;
		return Technologies.getTechnology(techFullName);
	}

	public Size getNodeValueByName_Size(String name) throws ApplicationError {
		DomNode n = getNodeByName(name);
		if (n == null) return null;
		return new Size(n);
	}

	public SetupOption getNodeValueByName_SetupOption(String name) throws ApplicationError {
		DomNode optionNode = getNodeByName(name);
		if (optionNode != null) {
			return new SetupOption(optionNode);
		} else {
			return null;
		}
	}

	public String [] getNodeValuesByName_String(String name) {
		NodeList children = node.getChildNodes();
		ArrayList<String> r = null;  
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				if (r == null) r= new ArrayList<String>();
				r.add(child.getTextContent());
			}
		}
		if (r == null) {
			return null;
		} else {
			return r.toArray(new String[r.size()]);
		}
	}


	public Technology [] getNodeValuesByName_Technology(String name) throws ApplicationError {
		NodeList children = node.getChildNodes();
		ArrayList<Technology> r = null;  
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				if (r == null) r= new ArrayList<Technology>();
				String techFullName = child.getTextContent();
				r.add(Technologies.getTechnology(techFullName));
			}
		}
		if (r == null) {
			return null;
		} else {
			return r.toArray(new Technology[r.size()]);
		}
	}

	public Size [] getNodeValuesByName_Size(String name) {
		DomNode [] n = getNodesByName(name);
		if (n != null) {
			Size [] r = new Size[n.length];
			for (int i = 0; i < n.length; i++) {
				r[i] = new Size(n[i]);
			}
			return r;
		} else {
			return null;
		}
			
	}
	
	public MotColor getNodeValuesByName_Colour(String name) {
		DomNode n = getNodeByName(name);
		if (n != null) {
			return new MotColor(n);
		} else {
			return null;
		}
			
	}
	
	public DomNode getNodeByName(String name) {
		NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				return new DomNode(child);
			}
		}
		return null;
	}

	public DomNode [] getNodesByName(String name) {
		NodeList children = node.getChildNodes();
		ArrayList<DomNode> r = null;  
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (Comp.same(name, child.getNodeName())) {
				if (r == null) r= new ArrayList<DomNode>();
				r.add(new DomNode(child));
			}
		}
		if (r == null) {
			return null;
		} else {
			return r.toArray(new DomNode[r.size()]);
		}
	}

	public DomNode [] getAllNodes() {
		NodeList children = node.getChildNodes();
		ArrayList<DomNode> r = null;  
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			if (r == null) r= new ArrayList<DomNode>();
			r.add(new DomNode(child));
		}
		if (r == null) {
			return null;
		} else {
			return r.toArray(new DomNode[r.size()]);
		}
	}

	public String toString() {
		StringBuilder sb = new StringBuilder(node.getNodeName());
		NodeList children = node.getChildNodes();
		if (children != null) {			
			sb.append("[");
			for (int i = 0; i < children.getLength(); i++) {
				Node child = children.item(i);
				sb.append(new DomNode(child).toString());
			}
			sb.append("]");
		} else {
			sb.append("=");
			sb.append(node.getTextContent());
		}
		return sb.toString();
	}
}
