/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.xml;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;

import mot.common.util.Logger;

/**
 * @author Paul Davis
 * 
 * Wraps up the introspection of a bean so objects can create
 * beans and call their get/set methods using a single piece
 * of code.
 */
public class IntrospectionBean {

    private Object theBean;
    private Class  theClass;

    public Object getBean() {
		return theBean;
	}

    public Class getBeanClass() {
        return theClass;
    }

    public IntrospectionBean(String className) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
		theClass = Class.forName(className);
		theBean = theClass.newInstance();
    }

    public IntrospectionBean(Object theBean) {
		this.theClass = theBean.getClass();
		this.theBean = theBean;
    }


    /**
     * Returns the list of attributes contained in a bean
     **/
    public String[] getAttributes() {
		Field [] fields = theClass.getFields();
		String [] names = new String[fields.length];
		for (int i=0; i<fields.length; i++) {
			names[i] = fields[i].getName();
		}
		return names;
    }

    /**
     * Returns the list of methods starting with the string get.
     **/
    public String[] getListOfGetMethods() {
		Method[] methods = theClass.getDeclaredMethods();
		ArrayList names = new ArrayList();
		for (int i=0; i<methods.length; i++) {
			String name = methods[i].getName();
			if (name.startsWith("get")) {
				names.add(name);
			}
		}
		return (String[]) names.toArray(new String[names.size()]);
    }

    /**
     * Calls a beans set method for an attribute.
     * @param attribute The name of the beans attribute on which to call the set
     *        method. It assumes the set method uses a standard java name, so
     *	      if the attribute is called fredFlint the method will be called
     *        setFredFlint(String). Note it assumes all parameters are strings.
     * @param value The value to set the attributes value to.
     * @param searchForType If true it will use reflection to attempt to determine the type of the field otherwise it will assume the type is String.
     **/
    public void callSet(String attribute, String value, boolean searchForType) throws SecurityException, NoSuchMethodException, InvocationTargetException, ClassNotFoundException, IllegalAccessException, InstantiationException, NoSuchFieldException {
        if (searchForType) {
            Class[] paramClassArray = new Class[1];
            try {
                String stdAttributeName = convertToJavaName(attribute);
                paramClassArray[0] = getAttributeType(theClass, stdAttributeName);
                callSet(attribute, value, paramClassArray);
            } catch (IllegalArgumentException e) {
                // There is an attribute of this name, but no setMethod containing one of our
                // predefined types.
                // Attempt to call a set method using the default string parameter type.
                Class [] paramClassArray2 = new Class[] { String.class };
                try {
                    callSet(attribute, value, paramClassArray2);
                } catch (NoSuchMethodException e2) {
                    Logger.logError("IntrospectionBean.callSet() Previously tried parameter type "+paramClassArray[0]+" and got error "+e);
                    throw e2;
                }
            } catch (NoSuchFieldException e) {
                // The field does not have the same name as the attribute in the XML
                // Attempt to call a set method using the default string parameter type.
                Class [] paramClassArray2 = new Class[] { String.class };
                try {
                    callSet(attribute, value, paramClassArray2);
                } catch (NoSuchMethodException e2) {
                    try {
                    	String methodName = getSetMethodFromAttribute(attribute);
                    	Class [] paramClassArray3 = getAttributeTypeFromSetMethod(methodName);
                    	Object[] paramsArray = null;
                    	callSet(attribute, value, paramClassArray3);
                    } catch (NoSuchMethodException e3) {
	                    Logger.logError("IntrospectionBean.callSet() Previously tried parameter type "+paramClassArray[0]+" and got error "+e);
	                    throw e3;
                    }
                }
            } catch (NoSuchMethodException e) {
                // There is no set method for the attribute type - This is not a standard
                // Java Bean - Attempt to use parameter String
                Class [] paramClassArray2 = new Class[] { String.class };
                try {
                    callSet(attribute, value, paramClassArray2);
                } catch (NoSuchMethodException e2) {
                    Logger.logError("IntrospectionBean.callSet() Previously tried parameter type "+paramClassArray[0]+" and got error "+e);
                    throw e2;
                }
            }
        } else {
            Class[] paramClassArray = new Class[] { String.class };
            callSet(attribute, value, paramClassArray);
        }
    }

	private void callSet(String attribute, String value, Class[] paramClassArray) throws SecurityException, NoSuchMethodException, InvocationTargetException, ClassNotFoundException, IllegalAccessException, InstantiationException, NumberFormatException {
		String methodName = getSetMethodFromAttribute(attribute);

        Object[] paramsArray = null;
        if (paramClassArray[0].getName().equalsIgnoreCase("int")) {
            paramsArray = new Object [] { new Integer(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("long")) {
            paramsArray = new Object [] { new Long(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("double")) {
            paramsArray = new Object [] { new Double(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("float")) {
            paramsArray = new Object [] { new Float(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("boolean")) {
            paramsArray = new Object [] { new Boolean(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("short")) {
            paramsArray = new Object [] { new Short(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("java.lang.Double")) {
            paramsArray = new Object [] { new Double(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("java.lang.Integer")) {
            paramsArray = new Object [] { new Integer(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("java.lang.Long")) {
            paramsArray = new Object [] { new Long(value) };
        } else if (paramClassArray[0].getName().equalsIgnoreCase("java.lang.Boolean")) {
            paramsArray = new Object [] { new Boolean(value) };
        } else {
            paramsArray = new Object[] { value };
        }

        if (Logger.isFull()) {
			Logger.logFull("IntrospectionBean.callSet() : Call "+theClass.getName()+"."+methodName);
		}

        callSet(theClass, methodName, paramClassArray, paramsArray);
    }

    public void callSet(String attribute, IntrospectionBean value, String methodName) throws SecurityException, NoSuchMethodException, InvocationTargetException, ClassNotFoundException, IllegalAccessException, InstantiationException {
		Class[] paramClassArray = new Class[] { value.getBeanClass() };
		if (methodName == null) {
			methodName = getSetMethodFromAttribute(attribute);
		}
		Object[] paramsArray = new Object[] { value.getBean() };
		if (Logger.isFull()) {
			Logger.logFull("IntrospectionBean.callSet() : Call "+theClass.getName()+"."+methodName);
		}

		try {
            callSet(theClass, methodName, paramClassArray, paramsArray);
		} catch (NoSuchMethodException e) {
            try {
                // There was no set method with the paramater type of value.
                // Check for a set method with a super class of value.
                Class superClass = value.getBeanClass().getSuperclass();
                paramClassArray = new Class[] { superClass };
                callSet(theClass, methodName, paramClassArray, paramsArray);
            } catch (NoSuchMethodException tmpError) {
                throw e;
            }
        }
    }

    private void callSet(Class theClass, String methodName, Class[] paramClassArray, Object[] paramsArray) throws SecurityException, NoSuchMethodException, InvocationTargetException, ClassNotFoundException, IllegalAccessException, InstantiationException {
		try {
			Method method = theClass.getMethod( methodName, paramClassArray );
			method.invoke(theBean, paramsArray);
		} catch (NoSuchMethodException e) {
			// Try to find the method in a super class
			Class superClass = theClass.getSuperclass();
			if (superClass != null) {
                try {
                    callSet(superClass, methodName, paramClassArray, paramsArray);
                } catch (NoSuchMethodException e2) {
                    throw e;
                }
			} else {
				throw e;
			}
		} catch (InvocationTargetException e) {
			Throwable e2 = e.getTargetException();
			if (Logger.isError()) {
				Logger.logError("IntrospectionBean.callSet() : Method "+methodName+" Failed due to error "+e2,e2);
			}
			throw e;
		}
    }

    /**
     * Finds the type of the attribute with name passed in by introspecting the class.
     * @param name The name of the attribute for which to find the type
     * @return the class representing this type (not class objects are also used for primitive types).
     */
    private Class getAttributeType(Class theClass, String name) throws NoSuchFieldException {
        try {
            Field field = theClass.getDeclaredField(name);
            return field.getType();
        } catch (NoSuchFieldException e) {
            Class superClass = theClass.getSuperclass();
            if (superClass != null) {
                return getAttributeType(superClass, name);
            } else {
                throw e;
            }
        }
    }

    private Class[] getAttributeTypeFromSetMethod(String methodName) {
        Method[] methods = theClass.getDeclaredMethods();
        if (methods == null) {
        	return null;
        }
        for (int i = 0; i < methods.length; i++) {
        	if (methodName.equals(methods[i].getName())) {        		
        		Class[] paramTypes = methods[i].getParameterTypes();
        		if (paramTypes != null && paramTypes.length == 1) {
        			return paramTypes;
        		} else {
        			return null;
        		}
        	}
		}
		return null;
    }


    private String getGetMethodFromAttribute(String attrName) {
		String name = "get"+
				  attrName.substring(0,1).toUpperCase()+
				  attrName.substring(1);
        name = convertToJavaName(name);
		return name;
    }

    public String getAttributeNameFromSetMethod(String method) {
        String name = method.substring(3);
        return name.substring(0,1).toLowerCase()+name.substring(1);
    }

    public String getSetMethodFromAttribute(String attrName) {
		String name = "set"+
				  attrName.substring(0,1).toUpperCase()+
				  attrName.substring(1);
        name = convertToJavaName(name);
		return name;
    }

    private String convertToJavaName(String name) {
		if (name.indexOf('_') > 0) {
			name = convertToJavaName('_', name);
		}
		if (name.indexOf('-') > 0) {
			name = convertToJavaName('-', name);
		}
		return name;
    }

    private String convertToJavaName(char seperatorChar, String name) {
		int index = name.indexOf(seperatorChar);
		while (index > 0) {
			if (name.length() >= (index+2)) {
			   name = name.substring(0, index)+
			   name.substring(index+1, index+2).toUpperCase()+
			   name.substring(index+2);
			} else if (name.length() >= (index+1)) {
				name = name.substring(0, index)+
			    name.substring(index+1);
			} else {
				name = name.substring(0, index);
			}
			index = name.indexOf(seperatorChar);
		}
		return name.toString();
    }

    /**
     * Calls a beans get method for an attribute.
     * @param attribute The name of the beans attribute on which to call the set
     *        method. It assumes the set method uses a standard java name, so
     *	      if the attribute is called fredFlint the method will be called
     *        Object getFredFlint(). Note it assumes all parameters are strings.
     * @param value The value to set the attributes value to.
     **/
    public Object callGetByFieldName(String fieldName) throws SecurityException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {
		String methodName = getGetMethodFromAttribute(fieldName);
		return callGetByMethod(fieldName);
    }

    /**
     * Calls a beans get method for an attribute.
     * @param methodName The name of the method to call.
     **/
    public Object callGetByMethod(String methodName) throws SecurityException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {
		Class[] paramClassArray = new Class[] { };
		Object[] paramsArray = new Object[] { };
		if (Logger.isFull()) {
			Logger.logFull("IntrospectionBean.callGet() : Call "+theClass.getName()+"."+methodName);
		}
		return callGet(theClass, methodName, paramClassArray, paramsArray);
	}

	private Object callGet(Class theClass, String methodName, Class[] paramClassArray, Object[] paramsArray) throws SecurityException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {
		try {
			Method method = theClass.getMethod( methodName, paramClassArray );
			return method.invoke(theBean, paramsArray);
		} catch (NoSuchMethodException e) {
			// Try to find the method in a super class
			Class superClass = theClass.getSuperclass();
			if (superClass != null) {
				return callGet(superClass, methodName, paramClassArray, paramsArray);
			} else {
				throw e;
			}
		} catch (InvocationTargetException e) {
			Throwable e2 = e.getTargetException();
			if (Logger.isError()) {
				Logger.logError("IntrospectionBean.callGet() : Method "+methodName+" Failed due to error "+e2,e2);
			}
			throw e;
		}
    }
}
