/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.xml;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.util.UnsortedNameValuePairList;

/**
 * @author Paul Davis
 * 
 * Base class for all beans serializable to XML.
 */
public abstract class XMLBean implements XMLBeanInterface {

  /**
   * This must be overridden by all Bean implementations. Its implementation should
   * write all XML element definitions contained in the bean. This would typically be
   * an element for each bean property.
   * Note - All beans contained with this bean (by calling the addBean() method) will
   * be serialised to XML automatically, encode should not implement.
   * @param writer The writer being used to serialise to XML.
   */
  protected abstract void encode(XMLWriter writer) throws ApplicationError;

  /**
   * This must be overridden by all Bean implementations. Its implementation should
   * return the name of the XML element generated by this bean.
   */
  protected abstract String getElementName();

  /**
   * Returns the default attributes.
   */
  private BasicNameValueList getDefaultAttributes(boolean generateClassAttribute) throws ApplicationError { 
      if (generateClassAttribute) {
    	  UnsortedNameValuePairList nvl = new UnsortedNameValuePairList();
		  nvl.add( new NameValuePair("class", getClassAttribute()) );
          return nvl;
      }
      return null; 
  }

  /**
    * @return The full name of this class (including package names) if the class attribute 
    * provided by this implementation.
    */
  protected String getClassAttribute() {
      return this.getClass().getName();
  }

  /**
   * If your been has attributes then override this method.
   * By default an XML bean does not have any attributes.
   * @param generateSchemaAttribute TODO
   */
  protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError { 
      return getDefaultAttributes(generateClassAttribute);
  }

  /**
   * Returns a BasicNameValuePairList containing the default attributes defined in 
   * XMLBean.getAttribute() and will never be null (but could be an empty list of 
   * no default attributes were defined).
   */
  protected BasicNameValueList getAttributesNotNull(boolean generateClassAttribute) throws ApplicationError { 
	  BasicNameValueList nvl = getDefaultAttributes(generateClassAttribute);
      if (nvl == null) {
          nvl = new UnsortedNameValuePairList();
      }
      return nvl;
  }

  /**
   * If your been has embeded XML element and needs encode to be called then use
   * the default implementation (true). If you want your bean to use a short definition
   * and have no context e.g. <person gender="m"/> then override this and return false.
   */
  protected boolean beanHasBody() { return true; }

  /**
   * If element name has been defined writes the XML to open the element tag.
   * Must be overridden if the default constructor is used to construct the bean.
   * @param writer The writer being used to serialise to XML.
   */
  protected void openElement(XMLWriter writer) {
      if ( getElementName() != null ) writer.openElement(getElementName());
  }

  /**
   * If element name has been defined writes the XML to close the element tag.
   * Must be overridden if the default constructor is used to construct the bean.
   * @param writer The writer being used to serialise to XML.
   */
  protected void closeElement(XMLWriter writer) {
      if ( getElementName() != null ) writer.closeElement(getElementName());
  }

  /**
   * Converts the bean to XML.
   * @param writer The writer being used to serialise to XML.
   */
  public void toXML(XMLWriter writer) throws ApplicationError {
      toXML(writer, getElementName());
  }
  
  /**
   * Converts the bean to XML.
   * @param writer The writer being used to serialise to XML.
   * @param The name of the XML element to open for this object.
   */
  public void toXML(XMLWriter writer, String elementName) throws ApplicationError {
	  BasicNameValueList attributes = getAttributes(writer.generateClassAttribute(), writer.generateSchemaAttributes());
	  if (beanHasBody()) {
          writer.openElement(elementName, attributes, false);
		  encode(writer);
		  writer.closeElement(elementName);
	  } else {
          writer.openElement(elementName, attributes, true);
	  }
  }

}
