/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.xml;

import java.util.Date;
import java.util.Iterator;

import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;

/**
 * @author Paul Davis
 * 
 * Converts an arbitrary bean into XML. It uses introspection to get all the
 * attribute values and writes an XML document for the bean. It also serializes
 * any aggregated beans and data types.
 * The big question is - How does it find out all the beans state to serialise to
 * XML? This is implemented using an AttributeIterator. There are currently two
 * AttributeIterator called MethodAttributeIterator and PropertyAttributeIterator.
 * MethodAttributeIterator - Assumes every method starting with the word get returns
 * an item of state and uses them.
 * PropertyAttributeIterator - Assumes every class attribute should be serialised
 * and attempts to generate get methods for the state items to call.
 */
public class XMLDocumentGenerator {

    GenericWriter writer;
    String rootElement;
    AttributeIterator ai;

    public XMLDocumentGenerator(AttributeIterator ai) throws ApplicationError {
        Object bean = ai.getBean();
		String className = bean.getClass().getName();
		String dtd = PropertyManager.get(className+".dtd");
		writer = new XMLWriter(dtd, getRootElement(bean));

		this.ai = ai;

		generate(bean);
    }

    public XMLDocumentGenerator(AttributeIterator ai, GenericWriter writer) throws ApplicationError {
		this.writer = writer;
		this.ai = ai;
		generate(ai.getBean());
    }

    private void generate(Object bean) throws ApplicationError {
		// We only generate the root element if it is not a collection!
		if (bean instanceof java.util.Collection) {
			generateElement(null, bean);
		} else {
			writer.openElement(getRootElement(bean));
				while (ai.hasNext()) {
					ai.next();
					generateElement(ai.getName(), ai.getValue());
				}
			writer.closeElement(getRootElement(bean));
		}
    }

    private void generateElement(String name, Object value) throws ApplicationError  {
		if (value instanceof java.lang.String) {
			writer.addElement(name, (String) value);
		} else if (value instanceof java.lang.Number) {
			writer.addElement(name, (Number) value);
		} else if (value instanceof java.lang.Boolean) {
			writer.addElement(name, (Boolean) value);
		} else if (value instanceof java.util.Collection) {
			Iterator i = ((java.util.Collection)value).iterator();
			while (i.hasNext()) {
			Object o = i.next();
			generateElement(name, o);
			}
		} else if (value instanceof java.util.Date) {
			writer.addElement(name, (Date) value);
		} else if (value instanceof java.io.Serializable) {
			XMLDocumentGenerator xdg = new XMLDocumentGenerator(ai.clone(value), writer);
		}
	}

	public GenericWriter getWriter() {
		return writer;
	}

	public String getDocument() {
		return writer.getDocument();
	}

	private String getRootElement(Object bean) {
		if (rootElement == null) {
			if (bean instanceof java.util.Collection) {
			Iterator i = ((java.util.Collection)bean).iterator();
			if (i.hasNext()) {
				getRootElement(i.next());
			}
			} else {
			rootElement = getClassName(bean.getClass());
			}
		}
		return rootElement;
    }

    private String getClassName(Class theClass) {
        String className = getShortClassName(theClass.getName());

        // For EJB's the bean is wrapped in machine generated classes whose names
        // are extrensions of the root names seperated with _'s. So I will strip
        // underscore seperated names.
        int index = className.indexOf("_");
        if (index > 1) {
            className = className.substring(0, index);
        }

        return className;
    }

    public static String getShortClassName(String className) {
        return className.substring(className.lastIndexOf('.')+1);
    }


}
