/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.xml;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;

/**
 * @author Paul Davis
 * 
/*
 * Parses an XML document using the SAX 2 API and returns a Java
 * Bean constructed from the document. Note the DTD in the XML must
 * contain class declarations.
 */
public class XMLDocumentHandler extends DefaultHandler implements ContentHandler, ErrorHandler {

    /**
     * The details of an XML element being processed.
     */
    private class XMLElement {
		public XMLElement(String name, String className, String setMethodName) throws SAXException {
			this.name = name;
			this.className = className;
			this.setMethodName = setMethodName;
			if (className != null) {
				try {
					this.instance = new IntrospectionBean(className);
				} catch (Exception e) {
                    Logger.logError("XMLElement.XMLElement() construct introspection bean for class "+className+" due to error ", e);
					throw new SAXException("XMLElement.XMLElement() construct introspection bean for class "+className+" due to error "+e);
				}
			}
		}
		public String name;
		public String className;
		public String setMethodName;
		public IntrospectionBean instance;
		public String content;
    };

    /**
     * A LIFO stack of XMLElements being processed. As XML elements are
     * processed (startElement is called) they are placed onto the stack
     * and when processing has been finished (endElement is called) they
     * are removed from the stack.
     */
    private ArrayList stack = new ArrayList();

    /**
     * List of errors that occured during processing the XML document.
     */
    private ArrayList errors = new ArrayList();

    public XMLDocumentHandler() {
    }

    private static final String [] ATTR_IGNORE_LIST = {"class", "setMethod", "xsi:noNamespaceSchemaLocation", "xsi:SchemaLocation"};

    public void startElement(String uri, String localpart, String name, Attributes attributes) throws SAXException {
		String className = attributes.getValue("class");
		className = BackwardsCompatibility.replaceClassName(className);
		String setMethodName = attributes.getValue("setMethod");
        if (Logger.isFull()) {
            Logger.logFull("XMLDocumentHandler.startElement() : "+name+" ["+className+"]");
        }

		XMLElement element = new XMLElement(name, className, setMethodName);
		stack.add(element);

		int length = attributes.getLength();
		for (int i = 0; i < length; i++) {
			//String attrURI = attributes.getURI(i);
			//String attrLocalpart = attributes.getLocalName(i);
			String attrName = attributes.getQName(i);
            if (Comp.in(attrName, ATTR_IGNORE_LIST)) {
                continue;
            }
			if (Logger.isFull()) {
				Logger.logFull("XMLDocumentHandler.startElement() : process attribute "+attrName);
			}
			try {
				String content = attributes.getValue(i);
				content = BackwardsCompatibility.replaceContent(content);
				element.instance.callSet(attrName, content, true);
			} catch (Exception e) {
				throw new SAXException("Unable to call set method for attribute "+attrName+" on class "+className+" due to error "+e);
			}
		}
    }

    public void characters(char[] ch, int offset, int length) throws SAXException {
		String chars = new String(ch, offset, length);
		if (Logger.isFull()) {
			Logger.logFull("XMLDocumentHandler.characters() : "+chars);
		}
		XMLElement element = (XMLElement) stack.get(stack.size()-1);
		if (element.content == null) {
			element.content = chars;
		} else {
			element.content += chars;
		}
    }

    public void ignorableWhitespace(char[] ch, int offset, int length) throws SAXException {
    }

    public void endElement(String uri, String localpart, String name) throws SAXException {
		if (Logger.isFull()) {
			Logger.logFull("XMLDocumentHandler.endElement() : "+name);
		}

		int stackSize = stack.size();

		XMLElement element = (XMLElement) stack.get(stack.size()-1);
		if (stackSize > 1) {
			stack.remove(element);
		}

		if (stackSize > 1) {
			XMLElement outerElement = (XMLElement) stack.get(stack.size()-1);
			if (outerElement.instance == null) {
				throw new SAXException("XMLDocumentHandler.endElement() No class attribute specified on "+outerElement.name);
			}
			if (element.instance != null) {
				try {
					outerElement.instance.callSet(element.name, element.instance, element.setMethodName);
				} catch (Exception e) {
					Logger.logError("XMLDocumentHandler.endElement() Unable to call set method for attribute "+element.name+" on class "+outerElement.className+" due to error ",e);
					throw new SAXException("XMLDocumentHandler.endElement() Unable to call set method for attribute "+element.name+" on class "+outerElement.className+" due to error "+e);
				}
            } else if (element.content != null) {
				try {
					String content = BackwardsCompatibility.replaceContent(element.content);
					outerElement.instance.callSet(element.name, content, true);
				} catch (Exception e) {
					Logger.logError("XMLDocumentHandler.endElement() Unable to call set method for content "+element.name+" on class "+outerElement.className+" due to error ",e);
					throw new SAXException("XMLDocumentHandler.endElement() Unable to call set method for content "+element.name+" on class "+outerElement.className+" due to error "+e);
				}
			}
		}
    }

    public Object getParsedObject() {
		XMLElement element = (XMLElement) stack.get(stack.size()-1);
		return element.instance.getBean();
	}

    public void warning(SAXParseException ex) {
		Logger.logError("XMLDocumentHandler.warning() Error on line "+ex.getLineNumber(), ex);
		//errors.add(ex.getMessage());
    }

    public void error(SAXParseException ex) {
		Logger.logError("XMLDocumentHandler.warning() Error on line "+ex.getLineNumber(), ex);
		errors.add(ex.getMessage());
    }

    public void fatalError(SAXParseException ex) {
		Logger.logError("XMLDocumentHandler.warning() Error on line "+ex.getLineNumber(), ex);
		errors.add(ex.getMessage());
    }

    public static Object parseFile(String fileName, String schemaPath) throws ApplicationError {
        return parseFile(fileName, true, schemaPath);
    }
    
    public static Object parseFile(String fileName, boolean validate, String schemaPath) throws ApplicationError {
    	File file = new File(fileName);
        if (!file.exists()) {
        	file = new File(PropertyManager.getMandatory("directory.xml.relative")+fileName);
        }
        return parse(file, validate, schemaPath);
    }

    public static Object parseFileCurrentDirectory(String fileName, boolean validate) throws ApplicationError {
        File file = new File(fileName);
        return parse(file, validate, null);
    }

    public static Object parse(File file) throws ApplicationError {
        return parse(file, true, null);
    }
    
    public static Object parse(File file, boolean validate, String schemaPath) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("XMLDocumentHandler.parse() file="+file.getName());
        }
        try {
        	URL schemaLocation = null;
        	if (schemaPath != null) {
        		schemaLocation = ClassLoader.getSystemResource(schemaPath);
        	}
			BufferedReader in = new BufferedReader( new FileReader(file) );
			return parse(new InputSource(in), validate, schemaLocation);
        } catch (IOException e) {
            throw new ApplicationError("Unable to parse XML file "+file+" due to error "+e);
        }
	}

	public static Object parseResource(String path) throws ApplicationError {
		return parseResource(path, true, null);
	}

	public static Object parseResource(String path, boolean validate, String schemaPath) throws ApplicationError {
    	if (schemaPath == null) {
    		schemaPath = path.replaceAll(".xml", ".xsd");
    	}
    	//This does not work in Java Web Start
    	//URL schemaLocation = ClassLoader.getSystemResource(schemaPath);
    	ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
    	URL schemaLocation = contextClassLoader.getResource(schemaPath);
    	
    	//This does not work in Java Web Start
    	//InputStream is = ClassLoader.getSystemResourceAsStream(path);
		InputStream is = contextClassLoader.getResourceAsStream(path);

		if (is == null) {
			throw new ApplicationError("Unable to open resource "+path);
		}
		
        if (Logger.isDebug()) {
            Logger.logDebug("XMLDocumentHandler.parseResource() resource="+path+" schemaLocation="+schemaPath);
        }
        
		return parse(new InputSource(is), validate, schemaLocation);
	}

	public static final String DTD_URL_MARKER = "__DTD_URL__";

    public static Object parse(String xml) throws ApplicationError {
        return parse(xml, true);
    }
    
    public static Object parse(String xml, boolean validate) throws ApplicationError {
        if (Logger.isFull()) {
            Logger.logFull("XMLDocumentHandler.parse()\n"+xml);
        }
        if (xml == null || xml.length() == 0) {
            return null;
        }
		int index = xml.indexOf(DTD_URL_MARKER);
		if (index >= 0) {
			//String dtdUrl = PropertyManager.getContext();
            //if (Logger.isDebug()) {
            //    Logger.logDebug("XMLDocumentHandler.parse() Parse document "+EntityEncoder.encode(getHeader(xml)));
            //}
			//xml = xml.substring(0, index) + dtdUrl + xml.substring(index+DTD_URL_MARKER.length());
		}
		if (Logger.isFull()) {
			Logger.logFull("XMLDocumentHandler.parse() : xml=\n"+xml);
		}
        return parse( new InputSource(new StringReader(xml)), validate, null);
	}

    public static String getHeader(String xml) {
		int index = xml.indexOf("\n");
		if (index >= 0) {
            return xml.substring(0,index);
        } else {
            return null;
        }
    }

    static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
    static final String XML_SCHEMA_FEATURE = "http://apache.org/xml/features/validation/schema";

    public static Object parse(InputSource xml) throws ApplicationError {
        return parse(xml, true, null);
    }

    public static Object parse(InputSource xml, boolean validate, URL schemaLocation) throws ApplicationError {
        // construct handler
        XMLDocumentHandler handler = new XMLDocumentHandler();

        SAXParserFactory factory = SAXParserFactory.newInstance();
        factory.setValidating(validate);
        factory.setNamespaceAware(true);


        try {
            if (validate) {
                //JAXP 1.1
                //factory.setFeature(XML_SCHEMA_FEATURE, true);
    
                SAXParser parser = factory.newSAXParser();
                
                // If schemas are in class path - must set location
                if (schemaLocation != null) {
                	parser.setProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation",  schemaLocation.toExternalForm());
                }
                
                //JAXP 1.2
                parser.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
                parser.parse(xml, handler);
            } else {
                SAXParser parser = factory.newSAXParser();
                parser.parse(xml, handler);
            }
		} catch (Exception e) {
			Logger.logError("XMLDocumentHandler.parse() Failed due to error ", e);
            if (xml != null && xml.getByteStream() != null) {
            	Logger.logDebug(xml.getByteStream());
            }
			throw new ApplicationError("XMLDocumentHandler.parse() Failed due to error "+e);
		} finally {
		}

		return handler.getParsedObject();
    }

}
