/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.common.xml;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;

import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.Comp;
import mot.common.util.PropertyManager;
import mot.common.util.UnsortedNameValuePairList;

/**
 * @author Paul Davis
 * 
 * Builds an XML Document and entity encodes the information.
 * Please do not manually write XML in classes - If a class wants to generate
 * an XML element, or an entire XML document, please use this XML writer. You may
 * use it two write XML elements contained in other objects that also use this
 * XML writer. Using this class means that you get the formatting for free and
 * the project gets everything entity encoding using the same object.
 */
public class XMLWriter extends GenericWriter {

//TODO - addElement for int, boolean, double etc should use setXXX(int v) etc by writing an attribute for the type

    private StringBuffer header = new StringBuffer();
    private StringBuffer sb = new StringBuffer();
    private boolean hasDtd;
    private boolean hasSchema;
    private int qtyElements = 0;
    
    public boolean generateClassAttribute() { return !hasDtd && !hasSchema; }
    public boolean generateSchemaAttributes() { return qtyElements == 0; }

    public XMLWriter() {
		this(null, null, null, false);
    }

    public XMLWriter(String dtd, String rootEelement) {
		this(dtd, rootEelement, null, false);
    }

    public XMLWriter(String dtd, String rootEelement, String xsl, boolean hasSchema) {
		String charset = PropertyManager.get("xml.charset");
		if (charset == null) charset = "ISO-8859-1";
		String version = PropertyManager.get("xml.version");
		if (version == null) version = "1.0";
		header.append("<?xml version=\""+version+"\" encoding=\""+charset+"\"?>\n");
		String descriptor = PropertyManager.get("xml.dtd.descriptor");
        hasDtd = (dtd != null);
		if (hasDtd) {
			header.append("<!DOCTYPE "+rootEelement+" PUBLIC \""+descriptor+"\" \""+dtd+"\">\n");
        }
		if (xsl != null) {
			header.append("<?xml-stylesheet href=\""+xsl+"\" type=\"text/xsl\"?>\n");
		}
		this.hasSchema = hasSchema;
    }

    public void openElement(String name) {
		openElement(name, false);
    }

    public void openElement(String name, UnsortedNameValuePairList attributes) {
		openElement(name, attributes, false);
    }

    public void openElement(String name, boolean shortcut) {
		if (shortcut) {
			sb.append(getIndent()+"<"+name+"/>\n");
		} else {
			sb.append(getIndent()+"<"+name+">\n");
			indentUp();
		}
		qtyElements++;
    }

    public void openElement(String name, BasicNameValueList attributes, boolean shortcut) {
		String attributesString = "";
		if (attributes != null) {
			attributesString = attributes.getAsAttributeList();
		}
		if (shortcut) {
			if (attributesString != null && attributesString.length() > 0) {
				sb.append(getIndent()+"<"+name+" "+attributesString+"/>\n");
			} else {
				sb.append(getIndent()+"<"+name+"/>\n");
			}
		} else {
			if (attributesString != null && attributesString.length() > 0) {
				sb.append(getIndent()+"<"+name+" "+attributesString+">\n");
			} else {
				sb.append(getIndent()+"<"+name+">\n");
			}
			indentUp();
		}
		qtyElements++;
    }

    public void closeElement(String name) {
		indentDown();
		sb.append(getIndent()+"</"+name+">\n");
    }

    public void addElement(String name, String value) {
		if (value != null) {
			sb.append(getIndent()+"<"+name+">"+entityEncode(value)+"</"+name+">\n");
			qtyElements++;
		}
    }

    public void addElement(String name, Number value) {
        if (value != null) {
            sb.append(getIndent()+"<"+name+">"+value+"</"+name+">\n");
        }
    }

    public void addElement(String name, int value) {
		sb.append(getIndent()+"<"+name+">"+value+"</"+name+">\n");
		qtyElements++;
    }

    public void addElement(String name, long value) {
		sb.append(getIndent()+"<"+name+">"+value+"</"+name+">\n");
		qtyElements++;
    }

    public void addElement(String name, double value) {
   		sb.append(getIndent()+"<"+name+">"+value+"</"+name+">\n");
		qtyElements++;
    }

    public void addElement(String name, double value, boolean removeUnusedDecimalPlaces) {
    	if (removeUnusedDecimalPlaces && Comp.isWhole(value)) {
    		sb.append(getIndent()+"<"+name+">"+((long)value)+"</"+name+">\n");
    	} else {
    		sb.append(getIndent()+"<"+name+">"+value+"</"+name+">\n");
    	}
		qtyElements++;
    }

    public void addElement(String name, Boolean value) {
    	if (value != null) {
    		addElement(name, value.booleanValue());
    		qtyElements++;
    	}
    }

    public void addElement(String name, boolean value) {
		sb.append(getIndent()+"<"+name+">"+(value?"true":"false")+"</"+name+">\n");
		qtyElements++;
    }

    public void addElement(String name, Date value) {
		openElement(name);
		Calendar cal = Calendar.getInstance();
		cal.setTime(value);
		addElement("hour", cal.get(Calendar.HOUR_OF_DAY));
		addElement("minute", cal.get(Calendar.MINUTE));
		addElement("day", cal.get(Calendar.DATE));
		addElement("month", cal.get(Calendar.MONTH));
		addElement("year", cal.get(Calendar.YEAR));
		closeElement(name);
    }

    /**
     * All textual information in an XML document should be entity encoded. This prevents
     * things like &, < etc in the document for breaking the XML syntax and making the document
     * unparsable.
     */
    private String entityEncode(String value) {
        return EntityEncoder.encode(value);
    }

    /**
     * Returns the data writen into the XML Writer as an XML Document. The document
     * will be complete with XML Headers, DTD declaration and XSL declaration as depending
     * on the information provided when constructing this XMLWriter.
     */
    public String getDocument() {
        return header.toString() + sb.toString();
    }

    /**
     * Returns a String containing the XML element declarations for the information entered
     * into the XML writer. Note this is not a valid XML document as it does not contain any
     * of the XML headers, but it may be used to be added into an XML document.
     */
    public String getElementDeclarations() {
        return sb.toString();
    }

    /**
     * Writes the XML to the file.
     */
    public void save(File file) throws ApplicationError {
        String xml = getDocument();
        if (!Comp.isEmpty(xml)) {
            try {
                BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file));
                bos.write(xml.getBytes());
                bos.flush();
                bos.close();
            } catch (IOException e) {
                throw new ApplicationError("Unable to Write file "+file.getPath()+" "+file.getName()+" due to error "+e);
            }
        }
    }

}
