/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.editor.swing.window;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.swing.combos.NamedObjectCombo;
import mot.client.swing.combos.OrbitCombo;
import mot.client.swing.combos.PlayerCombo;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.MotTextField;
import mot.client.validators.IntegerValidator;
import mot.common.bean.GalaxyDetails;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.vo.NavigatorVO;
import mot.common.vo.OrbitVO;
import mot.editor.swing.controls.CraftDesignCombo;
import mot.editor.swing.window.actions.DeleteCraftButton;
import mot.editor.swing.window.actions.NewCraftButton;
import mot.editor.swing.window.actions.UpdateEditorButton;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftBase;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.Navigator;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.generals.CraftTask;
import mot.engine.setup.MotFactory;

public class EditorCraftPanel extends JTabbedPane implements UpdatableEditorPanel {

	private JPanel editorContent;
	private EditorWindowListener listener;
	private MoveableOwnedObjectDelegate craft;
	
	private NamedObjectCombo selectionCombo;
	private NamedObjectCombo subSelectionCombo;
	private OrbitCombo orbitCombo;
	private OrbitCombo destinationCombo;
	private MotTextField x;
	private MotTextField y;
	
	private DeleteCraftButton deleteButton;
	private PlayerCombo newCraftOwnerSelect;
	private CraftDesignCombo newCraftDesignSelectCombo;
	private OrbitCombo newCraftOrbitSelectCombo;

	public EditorCraftPanel(EditorWindowListener listener, MoveableOwnedObjectDelegate craft, MoveableOwnedObjectDelegate[] selectionList) throws ApplicationError {
		this.listener = listener;
		this.craft = craft;
		
		GalaxyDetails galaxyDetails = DeligateCacheManager.getGalaxyDetails();
		
		selectionCombo = new NamedObjectCombo(selectionList, false);
		selectionCombo.setSelection(craft);

		subSelectionCombo = new NamedObjectCombo(null, false);

		orbitCombo = new OrbitCombo(new Size(25, 25), true);
		destinationCombo = new OrbitCombo(new Size(25, 25), true);

		x = new MotTextField(3, new IntegerValidator("X Position", MotFactory.MIN_DIST_TO_EDGE, galaxyDetails.getSize().getX()-MotFactory.MIN_DIST_TO_EDGE));
		y = new MotTextField(3, new IntegerValidator("Y Position", MotFactory.MIN_DIST_TO_EDGE, galaxyDetails.getSize().getY()-MotFactory.MIN_DIST_TO_EDGE));
		
		deleteButton = new DeleteCraftButton(listener);

		ComponentTable locPanel = new ComponentTable("HV, HVP", 8, 2, 0, 5, 4);
		ComponentTable newPanel = new ComponentTable("HV", 7, 1, 0, 5, 4);
		ComponentTable designPanel = new ComponentTable("HV,HVP", 1, 1, 0, 5, 4);
				
	    newCraftOwnerSelect = new PlayerCombo(new Size(25, 25), false, true);
	    newCraftOwnerSelect.setSelectedIndex(0);
		newCraftDesignSelectCombo = new CraftDesignCombo();
		newCraftOrbitSelectCombo = new OrbitCombo(new Size(25, 25), false);
		
		newCraftOwnerSelect.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				PlayerDelegate player = newCraftOwnerSelect.getSelectedPlayer();
				try {
					newCraftDesignSelectCombo.setSelection(player);
				} catch (Exception e1) {
					Logger.logError("FAIL", e1);
				}
			}
		});

		locPanel.add(new MotLabel("Craft Selection"));
		locPanel.add(selectionCombo);
		locPanel.add(new MotLabel("Orbit"));
		locPanel.add(orbitCombo);
		locPanel.add(new MotLabel("Destination"));
		locPanel.add(destinationCombo);
		locPanel.add(new MotLabel("X Position"));
		locPanel.add(x);
		locPanel.add(new MotLabel("Y Position"));
		locPanel.add(y);
		locPanel.add(new MotButtonPanel(deleteButton),"HVx");		
		locPanel.add(new MotButtonPanel(new UpdateEditorButton(this)),"HVx");		
		locPanel.add(new JPanel(),"HVx");

		newPanel.add(new MotLabel("Owner:"));
		newPanel.add(newCraftOwnerSelect);
		newPanel.add(new MotLabel("Design:"));
		newPanel.add(newCraftDesignSelectCombo);
		newPanel.add(new MotLabel("Orbit:"));
		newPanel.add(newCraftOrbitSelectCombo);
		newPanel.add(new MotButtonPanel(new NewCraftButton(listener, newCraftOwnerSelect, newCraftDesignSelectCombo, newCraftOrbitSelectCombo)));
		
		designPanel.add(new MotButtonPanel(new MotLabel("Coming Soon")));
		
		add("Loc", locPanel);
		add("New", newPanel);
		add("Design", designPanel);
		
		orbitCombo.setContent(true, true);
		destinationCombo.setContent(true, true);
		newCraftOrbitSelectCombo.setContent(true, true);
		
		setValues(craft);
	}
	
	private void setValues(MoveableOwnedObjectDelegate craft) throws ApplicationError {
		if (craft == null) {
			selectionCombo.clear();
			orbitCombo.clear();
			destinationCombo.clear();
			x.setText(null);
			y.setText(null);
			deleteButton.setCraftDelegate(null);
		} else {
			selectionCombo.setSelection(craft);
			OrbitVO orbit = craft.getOrbit();
			if (orbit != null) {
				if (orbit.getPlanet() != null) {
					orbitCombo.setSelection(DeligateCacheManager.getPlanet(orbit));
				} else {
					orbitCombo.setSelection(DeligateCacheManager.getStellarObject(orbit));
				}
			} else {
				orbitCombo.setSelection(null);
			}
			NavigatorVO navigator = craft.getNavigator();
			if (navigator != null) {
				OrbitVO destination = navigator.getDestination();
				if (destination.getPlanet() != null) {
					destinationCombo.setSelection(DeligateCacheManager.getPlanet(destination));
				} else {
					destinationCombo.setSelection(DeligateCacheManager.getStellarObject(destination));
				}
			} else {
				destinationCombo.setSelection(null);
			}
			x.setText(""+craft.getLocation().getX());
			y.setText(""+craft.getLocation().getY());
			deleteButton.setCraftDelegate(craft);
		}
	}

	public void update() throws ValidationError, ApplicationError {
		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
		if (localDelegate == null) {
			throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
		}
		MoveableOwnedObjectDelegate selection = (MoveableOwnedObjectDelegate) selectionCombo.getSelectedItem();
		if (localDelegate.getPlayer(selection.getOwner()) instanceof MonsterPlayer) {
			throw new ValidationError("Can't edit crafts owned by Ancients or Pirates");
		}
		
		int xp = x.getTextAsInt();
		int yp = y.getTextAsInt();
		
		boolean movedLoc = false;
		boolean canRemoveOrbit = false;
		
		MoveableOwnedObject moo = localDelegate.getMoveableOwnedObject(selection);
		if (moo instanceof Craft && ((Craft)moo).getFleet() != null) moo = ((Craft)moo).getFleet();
		NamedObjectDelegate newOrbit = orbitCombo.getNamedObject();
		Orbit oldDestination = moo.getNavigator() == null ? null : moo.getNavigator().getDestination();
		
		if (newOrbit != null && newOrbit instanceof PlanetDelegate) {
			Planet p = localDelegate.getPlanet((PlanetDelegate)newOrbit);
			if (moo.getOrbit() == null || moo.getOrbit().getPlanet() != p) {
				canMoveMoo(moo);
				moo.setOrbit(new Orbit(p));
				moo.setLocation(new Location(p.getSolarSystem().getLocation()));
				if (moo instanceof Fleet) {
					normalise((Fleet)moo);
				}
				movedLoc = true;
			}
		} else if (newOrbit != null && newOrbit instanceof StellarObjectDelegate) {
			StellarObject ss = localDelegate.getStellarObject((StellarObjectDelegate)newOrbit);
			if (moo.getOrbit() == null || moo.getOrbit().getStellarObject() != ss || moo.getOrbit().getPlanet() != null) {
				canMoveMoo(moo);
				moo.setOrbit(new Orbit(ss));
				moo.setLocation(new Location(ss.getLocation()));
				if (moo instanceof Fleet) {
					normalise((Fleet)moo);
				}
				movedLoc = true;
			}
		} else {
			canRemoveOrbit = true;
		}
		
		NamedObjectDelegate newDestination = destinationCombo.getNamedObject();
		boolean sameDestination = atSameStellarObject(newOrbit,newDestination);
		
		if (!sameDestination && newDestination != null && newDestination instanceof PlanetDelegate) {
			Planet p = localDelegate.getPlanet((PlanetDelegate)newDestination);
			if (moo.getNavigator() == null || moo.getNavigator().getDestination().getPlanet() != p) {
				canMoveMoo(moo);
				Orbit destination = new Orbit(p);
				if (moo.getNavigator() != null) {
					moo.getNavigator().setDestination(destination);
				} else {
					moo.setNavigator(new Navigator(moo, getOrigin(moo), destination));
				}
				normaliseTask(moo, oldDestination);
				if (moo instanceof Fleet) {
					normalise((Fleet)moo);
				} else if (moo instanceof Craft && ((Craft)moo).getFleet() != null) {
					normalise(((Craft)moo).getFleet());
				}
			}
		} else if (!sameDestination && newDestination != null && newDestination instanceof SolarSystemDelegate) {
			SolarSystem ss = localDelegate.getSolarSystem((SolarSystemDelegate)newDestination);
			if (moo.getNavigator() == null || moo.getNavigator().getDestination().getStellarObject() != ss || moo.getNavigator().getDestination().getPlanet() != null) {
				canMoveMoo(moo);
				Orbit destination = new Orbit(ss);
				if (moo.getNavigator() != null) {
					moo.getNavigator().setDestination(destination);
				} else {
					moo.setNavigator(new Navigator(moo, getOrigin(moo), destination));
				}
				normaliseTask(moo, oldDestination);
				if (moo instanceof Fleet) {
					normalise((Fleet)moo);
				} else if (moo instanceof Craft && ((Craft)moo).getFleet() != null) {
					normalise(((Craft)moo).getFleet());
				}
			}
		} else if (newDestination == null) {
			Orbit o = orbitCombo.getOrbit();
			if (o == null) {
				throw new ValidationError("Craft must be in orbit to remove destination");
			}
			canMoveMoo(moo);
			moo.setNavigator(null);
			moo.setOrbit(o);
			moo.setLocation(o.getStellarObject().getLocationClone());
			if (moo instanceof Fleet) {
				normalise((Fleet)moo);
			}
		}

		if (!moo.getLocation().equals(xp, yp) && !movedLoc) {
//			StellarObject soAt = MotEngine.instance.getStellarObjectsNotNull().getStellarObjectAt(xp, yp);
//			if (soAt != null) {
//				throw new ValidationError("To move craft at this location set orbit to "+soAt.getName());
//			}
			if (!canRemoveOrbit && moo.getOrbit() != null) {
				throw new ValidationError("Craft with in orbit must have same location as Solar System");
			}
			if (moo.getNavigator() == null) {
				throw new ValidationError("Craft at warp must have a destination");
			}
			moo = canMoveMoo(moo);
			if (canRemoveOrbit) moo.setOrbit(null);
			moo.setLocation(new Location(xp, yp));
			if (moo instanceof Fleet) {
				normalise((Fleet)moo);
			}
		}
		
		
		//GalaxyAnimationRegister.clean();
		CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();
		ProxyStore.getInstance().updateDelegates(callbackSync);
        callbackSync.waitTillComplete();
		listener.getView().getMapPanel().rebuildContent();
		
		Logger.logDebug("Move "+moo.getName()+" to "+moo.getLocation());
		
		setValues(DeligateCacheManager.getMoo(craft.getName()));
	}

	private boolean atSameStellarObject(NamedObjectDelegate newOrbit, NamedObjectDelegate newDestination) {
		if (newOrbit instanceof StellarObjectDelegate) {
			if (newDestination instanceof StellarObjectDelegate) {
				return Comp.same((StellarObjectDelegate)newOrbit,(StellarObjectDelegate)newDestination);
			} else if (newDestination instanceof PlanetDelegate){
				return Comp.same((StellarObjectDelegate)newOrbit,((PlanetDelegate)newDestination).getSolarSystem());
			}
		} else if (newOrbit instanceof PlanetDelegate){
			if (newDestination instanceof StellarObjectDelegate) {
				return Comp.same(((PlanetDelegate)newOrbit).getSolarSystem(),(StellarObjectDelegate)newDestination);
			} else if (newDestination instanceof PlanetDelegate){
				return Comp.same(((PlanetDelegate)newOrbit).getSolarSystem(),((PlanetDelegate)newDestination).getSolarSystem());
			}
		}
		return false;
	}

	private Orbit getOrigin(MoveableOwnedObject moo) throws ApplicationError {
		Orbit origin = null;
		if (moo.getNavigator() != null) {
			origin = moo.getNavigator().getOrigin(); 
		} else if (moo.getOrbit() != null) {
			origin = moo.getOrbit();
		}
		return origin;
	}

	private void normaliseTask(MoveableOwnedObject moo, Orbit oldDestination) throws ApplicationError {
		if (!(moo instanceof CraftBase)) return;
		
		CraftBase craft = (CraftBase) moo;
		CraftTask task = craft.getTask();
		Navigator navigator = craft.getNavigator();
		
		if (task == null) return;
		if (navigator == null) return;

		if (task.getTarget() != null && task.getTarget().equals(oldDestination)) {
			Orbit newTarget = navigator.getDestination();
			task.setTarget(newTarget);
		}
	}
	
	private void normalise(Fleet fleet) throws ApplicationError {
		if (fleet == null) return;
		Craft[] crafts = fleet.getCrafts();
		if (crafts == null) return;
		for (int i = 0; i < crafts.length; i++) {
			crafts[i].setLocation(new Location(fleet.getLocation()));
			if (fleet.getOrbit() == null) {
				crafts[i].setOrbit(null);
			} else {
				crafts[i].setOrbit(new Orbit(fleet.getOrbit()));
			}
			if (fleet.getNavigator() == null) {
				crafts[i].setNavigator(null);
			} else {
				crafts[i].setNavigator(new Navigator(fleet.getNavigator()));
			}
		}
	}

	private MoveableOwnedObject canMoveMoo(MoveableOwnedObject moo) throws ApplicationError, ValidationError {
		if (moo == null) throw new ValidationError("No craft selected");
		if (moo instanceof Craft) {
			Craft craft = (Craft) moo;
			if (craft.getFleet() != null) {
				return craft.getFleet();
			}
		}
		return moo;
	}


}
