/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.editor.swing.window;

import javax.swing.JLabel;
import javax.swing.JPanel;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.swing.combos.PlayerCombo;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotCheckBoxSimple;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.MotTextField;
import mot.client.swing.renderers.CellRendererMoneyBonus;
import mot.client.swing.renderers.CellRendererNumber;
import mot.client.swing.renderers.CellRendererProductionBonus;
import mot.client.swing.renderers.CellRendererReproductionBonus;
import mot.client.swing.renderers.CellRendererResearchBonus;
import mot.client.validators.DoubleValidator;
import mot.common.lang.L;
import mot.common.resources.Resource;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.editor.swing.controls.BonusCombo;
import mot.editor.swing.controls.MoneyBonusCombo;
import mot.editor.swing.controls.PlanetSizeCombo;
import mot.editor.swing.controls.ResourceCombo;
import mot.editor.swing.window.actions.DeletePlanetButton;
import mot.editor.swing.window.actions.UpdateEditorButton;
import mot.engine.MotEngine;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

public class EditorPlanetPanel extends ComponentTable implements UpdatableEditorPanel {

	private JPanel editorContent;
	private EditorWindowListener listener;
	private PlanetDelegate planet;
	
	private PlayerCombo ownerCombo;
	private PlayerCombo originCombo;
	private PlanetSizeCombo size;
	private MotTextField terraform;
	private MotTextField population;
	private MotTextField factories;
	private BonusCombo productionBonus;
	private BonusCombo researchBonus;
	private BonusCombo reproductionBonus;
	private MoneyBonusCombo moneyBonus;
	private ResourceCombo resource;
	private MotCheckBoxSimple natives;
	private DeletePlanetButton deleteButton;
	
	public EditorPlanetPanel(EditorWindowListener listener, PlanetDelegate planet) throws ApplicationError {
		super("HV,HVP", 16, 2, 0, 5, 4);
		this.listener = listener;
		this.planet = planet;

		ownerCombo = new PlayerCombo(true, false);
		originCombo = new PlayerCombo(true, false);
		size = new PlanetSizeCombo(new CellRendererNumber(false));
		terraform = new MotTextField(12, new DoubleValidator(L.TERRAFORM, 0.2, 1.6, null));
		population = new MotTextField(12, new DoubleValidator(L.POPULATION, 0.0, 300.0, null));
		factories = new MotTextField(12, new DoubleValidator(L.FACTORIES, 0.0, 300.0, null));
		productionBonus = new BonusCombo(new CellRendererProductionBonus());
		researchBonus = new BonusCombo(new CellRendererResearchBonus());
		reproductionBonus = new BonusCombo(new CellRendererReproductionBonus());
		moneyBonus = new MoneyBonusCombo(new CellRendererMoneyBonus());
		resource = new ResourceCombo();
		natives = new MotCheckBoxSimple(L.NATIVES, null);
		deleteButton = new DeletePlanetButton(listener);
		
		add(new MotLabel(L.NAME_));
		add(new MotLabel(planet.getName(), 2, null, null, JLabel.LEFT, false));
		add(new MotLabel(L.OWNER));
		add(ownerCombo);
		add(new MotLabel(L.ORIGIN));
		add(originCombo);
		add(new MotLabel(L.SIZE));
		add(size);
		add(new MotLabel(L.TERRAFORM));
		add(terraform);
		add(new MotLabel(L.POPULATION));
		add(population);
		add(new MotLabel(L.FACTORIES));
		add(factories);
		add(new MotLabel(L.PRODUCTION_BONUS));
		add(productionBonus);
		add(new MotLabel(L.RESEARCH_BONUS));
		add(researchBonus);
		add(new MotLabel(L.FERTILITY_BONUS));
		add(reproductionBonus);
		add(new MotLabel(L.MONEY_BONUS));
		add(moneyBonus);
		add(new MotLabel(L.RESOURCE));
		add(resource);
		add(natives,"Rx");
		add(new MotButtonPanel(deleteButton),"HVx");		
		add(new MotButtonPanel(new UpdateEditorButton(this)),"HVx");		
		add(new JPanel(),"HVx");
		
		setValues(planet);
	}
	
	private void setValues(PlanetDelegate planet) throws ApplicationError {
		if (planet == null) {
			ownerCombo.clear();
			originCombo.clear();
			size.clear();
			terraform.setText(null);
			population.setText(null);
			factories.setText(null);
			productionBonus.clear();
			researchBonus.clear();
			reproductionBonus.clear();
			moneyBonus.clear();
			resource.clear();
			natives.setSelected(false);
			deleteButton.setPlanetDelegate(null);
		} else {
			ownerCombo.setSelection(planet.getOwner());
			originCombo.setSelection(planet.getOrigin());
			size.setSelection(planet.getSize());
			terraform.setText(""+planet.getTeraforming());
			population.setText(""+planet.getPopulation());
			factories.setText(""+planet.getQtyFactories());
			productionBonus.setSelection(planet.getProductionBonus());
			researchBonus.setSelection(planet.getResearchBonus());
			reproductionBonus.setSelection(planet.getReproductionBonus());
			moneyBonus.setSelection(planet.getMoneyBonus());
			resource.setSelection(planet.getResource());
			natives.setSelected(planet.hasNatives());
			deleteButton.setPlanetDelegate(planet);
		}
	}

	public void update() throws ValidationError, ApplicationError {
		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
		if (localDelegate == null) {
			throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
		}
		
		int newPlanetSize = size.getPlanetSize();
		double newTerraform = terraform.getTextAsDouble();
		double newPopulation = population.getTextAsDouble();
		double newQtyFactories = factories.getTextAsDouble();
		double newProdBonus = productionBonus.getBonus();
		double newResBonus = researchBonus.getBonus();
		double newFerBonus = reproductionBonus.getBonus();
		int newMoneyBonus = moneyBonus.getBonus();
		Resource newResource = resource.getResource();
		boolean newHasNatives = natives.isSelected();
		PlayerDelegate newOwner = ownerCombo.getSelectedPlayer();
		PlayerDelegate newOrigin = originCombo.getSelectedPlayer();
		
		Planet p = localDelegate.getPlanet(planet.getName());
		Player owner = p.getOwner();
		
		if (newHasNatives != p.hasNatives()) {
			if (owner != null) {
				throw new ValidationError("Natives can only be placed on planets that are not colonised");
			}
		}
		
		if (newTerraform < 0.1 || newTerraform > 1.5) throw new ValidationError("Terraform must be between 0.1 and 1.5");
		
		if (newOwner == null) {
			if (newPopulation > 0) throw new ValidationError("Population must be zero unless planet has an owner");
			if (newQtyFactories > 0) throw new ValidationError("Factories must be zero unless planet has an owner");
			if (newOrigin != null) throw new ValidationError("Origin must be None unless planet has an owner");
		} else {
			if (newPopulation <= 0 && !p.isMiningColony()) throw new ValidationError("Population must be more then 0 unless planet has an owner");
		}
		if (newQtyFactories < 0) throw new ValidationError("Factories must be zero or more unless planet has an owner");
		
		Player newPlayer = localDelegate.getPlayer(newOwner);
		boolean ownerChanged = !Comp.same(newPlayer, p.getOwner());

		p.setOwner(newPlayer);
		p.setOrigin(localDelegate.getPlayer(newOrigin));
		p.setSize(newPlanetSize);
		p.setTeraforming(newTerraform);
		p.setPopulation(newPopulation);
		p.setQtyFactories(newQtyFactories);
		p.setProductionBonus(newProdBonus);
		p.setResearchBonus(newResBonus);
		p.setReproductionBonus(newFerBonus);
		p.setMoneyBonus(newMoneyBonus);
		p.setResource(newResource);

		if (newHasNatives && !p.hasNatives()) {
			p.setHasNatives(true);
			p.setQtyNatives(planet.getSize());		
		}
		if (!newHasNatives) {
			p.setHasNatives(false);
			p.setQtyNatives(0);		
		}
		
		if (ownerChanged) {
			if (newOwner != null) {
				p.setDateColonised(MotEngine.instance.getDate());
				if (p.getInfluence() == 0) {
					p.setInfluence(1);
				}
				p.getSolarSystem().addVisitor(newPlayer, false);
				if (p.getOrigin() == null) {
					p.setOrigin(newPlayer);
				}
			} else {
				p.setDateColonised(0);
				p.setInfluence(0);
				p.setOrigin(null);
			}
		}

		MotEngine.instance.getInfluence().update();
		DeligateCacheManager.flush();
		CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();
		ProxyStore.getInstance().updateDelegates(callbackSync);
        callbackSync.waitTillComplete();
		//GalaxyAnimationRegister.clean();
		listener.getView().getMapPanel().rebuildContent();
	}

}
