/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.editor.swing.window;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import mot.client.deligates.PlayerDelegate;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.swing.buttons.MotButton;
import mot.client.swing.combos.TechnologyCombo;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.MotTextField;
import mot.client.swing.common.PanelUtils;
import mot.client.swing.lists.TechList;
import mot.client.validators.IntegerValidator;
import mot.common.lang.L;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.editor.swing.window.actions.UpdateEditorButton;
import mot.engine.players.Player;
import mot.engine.setup.PlayerFactory;

public class EditorPlayerPanel extends JTabbedPane implements UpdatableEditorPanel {

	private EditorWindowListener listener;
	private PlayerDelegate player;
	private MotTextField moneyField;
	private MotTextField boraliumField;
	private TechList discoveredList;
	private TechnologyCombo technologyCombo;
	private TechList missedList;
	private TechnologyCombo missedCombo;

	public EditorPlayerPanel(EditorWindowListener listener, PlayerDelegate pd) throws ApplicationError, ValidationError {
		this.listener = listener;
		this.player = pd;

		MotButton addTechButton = new AddDiscoveredTechButton();
		MotButton removeTechButton = new RemoveDiscoveredTechButton();
		
		MotButton addMissedTechButton = new AddMissedTechButton();
		MotButton removeMissedTechButton = new RemoveMissedTechButton();
		
		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
		if (localDelegate == null) {
			throw new ApplicationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
		}
		Player p = localDelegate.getPlayer(player);
		
		moneyField = new MotTextField(10, new IntegerValidator(L.MONEY, 0, 999999));
		boraliumField = new MotTextField(10, new IntegerValidator(L.BORALIUM, 0, 9999));
		discoveredList = new TechList(p.getTechnologies(), null, null, true);
		missedList = new TechList(p.getPlayerCharacter().getMissedTechnologies(), null, null, true);
		technologyCombo = new TechnologyCombo(Technologies.getTechnologies(), true);
		missedCombo = new TechnologyCombo(Technologies.getTechnologies(), true);
		
		JScrollPane discoveredScrollPane = discoveredList.getScrollPane();
		PanelUtils.setTheBloodySize(discoveredScrollPane, 200, 150);
		
		JScrollPane missedScrollPane = missedList.getScrollPane();
		PanelUtils.setTheBloodySize(missedScrollPane, 200, 150);
		
		ComponentTable mainPanel = new ComponentTable("HV,HV", 3, 2, 0, 5, 4);
		ComponentTable discoveredTechsPanel = new ComponentTable("HV,HV", 5, 2, 0, 5, 4);
		ComponentTable missedTechsPanel = new ComponentTable("HV,HV", 5, 2, 0, 5, 4);
		
		mainPanel.add(new MotLabel(L.MONEY));
		mainPanel.add(moneyField);
		mainPanel.add(new MotLabel(L.BORALIUM));
		mainPanel.add(boraliumField);
		mainPanel.add(new MotButtonPanel(new UpdateEditorButton(this)),"HVx");
		
		discoveredTechsPanel.add(new MotLabel(L.DISCOVERED, 4, true), "Rx");
		discoveredTechsPanel.add(discoveredScrollPane, "HVx");
		discoveredTechsPanel.add(new MotButtonPanel(removeTechButton), "HVx");
		discoveredTechsPanel.add(technologyCombo, "HVx");
		discoveredTechsPanel.add(new MotButtonPanel(addTechButton), "HVx");
		
		missedTechsPanel.add(new MotLabel(L.MISSED_TECHNOLOGIES, 4, true), "Rx");
		missedTechsPanel.add(missedScrollPane, "HVx");
		missedTechsPanel.add(new MotButtonPanel(removeMissedTechButton), "HVx");
		missedTechsPanel.add(missedCombo, "HVx");
		missedTechsPanel.add(new MotButtonPanel(addMissedTechButton), "HVx");
		
		add("main", mainPanel);
		add("techs", discoveredTechsPanel);
		add("missed", missedTechsPanel);
		
		setContent(pd);
	}

	public void setContent(PlayerDelegate pd) throws ApplicationError, ValidationError {
		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
		if (localDelegate == null) {
			throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
		}
		Player p = localDelegate.getPlayer(pd.getName());
		
		moneyField.setText( ""+p.getFunds() );
		boraliumField.setText( ""+p.getBoralium() );
		
		discoveredList.update(p.getTechnologies());
		missedList.update(p.getPlayerCharacter().getMissedTechnologies());

	}

	public void update() throws ApplicationError, ValidationError {
		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
		if (localDelegate == null) {
			throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
		}
		Player p = localDelegate.getPlayer(player);
		
		int m = moneyField.getTextAsInt();
		int b = boraliumField.getTextAsInt();
		
		p.setFunds(m);
		p.setBoralium(b);
		
	}

	
	private final class RemoveMissedTechButton extends MotButton {
		private RemoveMissedTechButton() {
			super(L.REMOVE);
		}

		protected void _doAction() throws ApplicationError, ValidationError {
			Technology tech = missedList.getSelectedTech();
			if (tech != null) {
				LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
				if (localDelegate == null) {
					throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
				}
				Player p = localDelegate.getPlayer(player);
				p.getPlayerCharacter().getMissedTechnologies().remove(tech);
				missedList.update(p.getPlayerCharacter().getMissedTechnologies());
			}
		}
	}

	private final class AddMissedTechButton extends MotButton {
		private AddMissedTechButton() {
			super(L.ADD);
		}

		protected void _doAction() throws ApplicationError, ValidationError {
			Technology tech = missedCombo.getSelectedTech();
			if (tech != null) {
				LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
				if (localDelegate == null) {
					throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
				}
				Player p = localDelegate.getPlayer(player);
				p.getPlayerCharacter().getMissedTechnologies().add(tech);
				missedList.update(p.getPlayerCharacter().getMissedTechnologies());
			}
		}
	}

	private final class RemoveDiscoveredTechButton extends MotButton {
		private RemoveDiscoveredTechButton() {
			super(L.REMOVE);
		}

		protected void _doAction() throws ApplicationError, ValidationError {
			Technology tech = discoveredList.getSelectedTech();
			if (tech != null) {
				LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
				if (localDelegate == null) {
					throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
				}
				Player p = localDelegate.getPlayer(player);
				Technology[] st = PlayerFactory.getStartingTechnologies(p);
				GeneralTechnologyList startingTechnologies = new GeneralTechnologyList(st);
				if (startingTechnologies.contains(tech)) {
					throw new ValidationError("Cannot remove a starting technology from a player");
				}
				p.removeTechnology(tech);
				discoveredList.update(p.getTechnologies());
			}
		}
	}

	private final class AddDiscoveredTechButton extends MotButton {
		private AddDiscoveredTechButton() {
			super(L.ADD);
		}

		protected void _doAction() throws ApplicationError, ValidationError {
			Technology tech = technologyCombo.getSelectedTech();
			if (tech != null) {
				LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
				if (localDelegate == null) {
					throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
				}
				Player p = localDelegate.getPlayer(player);
				p.addTechnology(tech);
				discoveredList.update(p.getTechnologies());
			}
		}
	}


}
