/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.editor.swing.window;

import javax.swing.JLabel;
import javax.swing.JPanel;

import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.MotButtonPanel;
import mot.client.swing.common.MotCheckBoxSimple;
import mot.client.swing.common.MotLabel;
import mot.client.swing.common.MotTextField;
import mot.client.validators.IntegerValidator;
import mot.common.bean.GalaxyDetails;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.editor.swing.controls.PrizeCombo;
import mot.editor.swing.controls.StarTypeCombo;
import mot.editor.swing.window.actions.NewPlanetButton;
import mot.editor.swing.window.actions.UpdateEditorButton;
import mot.engine.MotEngine;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;
import mot.engine.prizes.Prize;
import mot.engine.setup.MotFactory;

public class EditorSolarSystemPanel extends ComponentTable implements UpdatableEditorPanel {

	private JPanel editorContent;
	private EditorWindowListener listener;
	private SolarSystemDelegate solarSystem;
	
	private StarTypeCombo starType;
	private MotTextField x;
	private MotTextField y;
	private MotCheckBoxSimple [] visited;
	private PrizeCombo prize;
	
	private static int getQtyPlayers() {
		return MotEngine.instance.getPlayersNotNull().getQuantity();
	}
	
	public EditorSolarSystemPanel(EditorWindowListener listener, SolarSystemDelegate ssd) throws ApplicationError {
		super("HV", 4+getQtyPlayers(), 1, 0, 5, 4);
		this.listener = listener;
		this.solarSystem = ssd;

		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
		if (localDelegate == null) {
			return;
		}
		
		GalaxyDetails galaxyDetails = DeligateCacheManager.getGalaxyDetails();
		SolarSystem ss = localDelegate.getSolarSystem(ssd);
		Prize p = (ss.getPrizes() == null || ss.getPrizes().length == 0) ? null : ss.getPrizes()[0];
		
		starType = new StarTypeCombo();
		x = new MotTextField(3, new IntegerValidator("X Position", MotFactory.MIN_DIST_TO_EDGE, galaxyDetails.getSize().getX()-MotFactory.MIN_DIST_TO_EDGE));
		y = new MotTextField(3, new IntegerValidator("Y Position", MotFactory.MIN_DIST_TO_EDGE, galaxyDetails.getSize().getY()-MotFactory.MIN_DIST_TO_EDGE));
		prize = new PrizeCombo(p);
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		visited = new MotCheckBoxSimple[players.length]; 
		for (int i = 0; i < players.length; i++) {
			visited[i] = new MotCheckBoxSimple(players[i].getName()+" visited", null);
		}

		ComponentTable topPanel = new ComponentTable("HV,HVP", 5, 2, 0, 5, 4);
		topPanel.add(new MotLabel(L.NAME));
		topPanel.add(new MotLabel(ssd.getName(), 2, null, null, JLabel.LEFT, false));
		topPanel.add(new MotLabel("Start Type"));
		topPanel.add(starType);
		topPanel.add(new MotLabel("X Position"));
		topPanel.add(x);
		topPanel.add(new MotLabel("Y Position"));
		topPanel.add(y);
		topPanel.add(new MotLabel("Prize"));
		topPanel.add(prize);
		add(topPanel);
		for (int i = 0; i < players.length; i++) {
			add(visited[i]);
		}
		add(new MotButtonPanel(new UpdateEditorButton(this)));		
		add(new MotButtonPanel(new NewPlanetButton(listener, ssd)));		
		add(new JPanel(),"HVS");

		starType.setSelection(ssd.getGraphicsName());
		x.setText(""+ssd.getLocation().getX());
		y.setText(""+ssd.getLocation().getY());
		for (int i = 0; i < players.length; i++) {
			visited[i].setSelected(ss.getLastVisitedDate(players[i]) > 0);
		}
		
	}

	public void update() throws ValidationError, ApplicationError {
		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
		if (localDelegate == null) {
			throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
		}
		
		String st = starType.getStarType();
		int xp = x.getTextAsInt();
		int yp = y.getTextAsInt();
		
		SolarSystem ss = localDelegate.getSolarSystem(solarSystem);
		
		if (!ss.getLocation().equals(xp, yp)) {
			ss.changeLocationTo(xp,yp);
			DeligateCacheManager.flush();
		}
		ss.setStarType(st);
		
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		for (int i = 0; i < players.length; i++) {
			if (visited[i].isSelected()) {
				ss.addVisitor(players[i], false);
			} else {
				ss.removeVisitor(players[i]);
			}
		}
		
		Prize p = prize.getPrize();
		ss.clearAndSetPrize(p);
		
		CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();
		ProxyStore.getInstance().updateDelegates(callbackSync);
        callbackSync.waitTillComplete();
		//DeligateCacheManager.flush();
		//GalaxyAnimationRegister.clean();
		listener.getView().getMapPanel().rebuildContent();
	}
	
}
