/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.editor.swing.window;

import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.util.ArrayList;

import javax.swing.JScrollPane;

import mot.client.actions.ActiveRectangle;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.window.galaxy.GalaxyMapListener;
import mot.client.swing.window.galaxy.GalaxyMapPanel;
import mot.client.swing.window.galaxy.GalaxyMapStellarObject;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowListener;
import mot.common.bean.Location;
import mot.common.bean.SectorOwnerGrid;
import mot.common.bean.Size;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.StellarObject;
import mot.engine.players.Player;

public class EditorWindowListener extends MainWindowListener implements GalaxyMapListener {


	/**
	 * This holds the current state of the galaxy map.
	 */
	private EditorWindowState state;
	
	/**
	 * This is the view that this listener uses to display its results.
	 */
	private EditorWindowView view;
	

	public EditorWindowListener() throws ApplicationError {
	}

    private ArrayList shortcutListeners = new ArrayList();

    /**
     * Called when the user rolls the mouse wheel
     */
	public void mouseWheelMoved(MouseWheelEvent e) {
		view.getZoomSliderListener().mouseWheelMoved(e);
	}

    /**
     * Buttons in the game with shortcuts call this to register them selves
     * to receive key events.
     */
//    public void addShortcutLisneter(ShortcutListener shortcutListener) {
//    	shortcutListeners.add(shortcutListener);
//    }
    
    //
    // MOUSE LISTENER STUFF
    //

    public void motMouseClicked(MouseEvent e) {
    	MainWindow mainWin = null;
    	try {
    		mainWin = MainWindow.getInstance();
    	} catch (Exception ex) {
    		Logger.logError("FAIL", ex);
    	}
        try {
        	GalaxyMapPanel mapWindow = view.getMapPanel();
            if (e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1) {
                int x = e.getX();
                int y = e.getY();
                ActiveRectangle o = mapWindow.getObjects().click(x,y);
                if (o == null) {
	                // remove selection
	                clearSelection();
	                view.repaintMap();
                }
            }
        } catch (ValidationError ex) {
        	MainWindow.getExestingInstance().reportValidationError(ex);
        } catch (Exception ex) {
            MainWindow.getExestingInstance().reportApplicationError(ex);
        }
    }
    
	public void componentResized() {
		// TODO
	}

	ActiveRectangle dragSelection = null;
	public void mousePressed(MouseEvent e) {
		super.mousePressed(e);
		try {
			if (e.getButton() == MouseEvent.BUTTON3) {
				int x = e.getX(); 
				int y = e.getY();
				GalaxyMapPanel mapWindow = view.getMapPanel();
				dragSelection = mapWindow.getObjects().getObjectItemAt(x,y);
			}
		} catch (Exception ex) {
			Logger.logError("", ex);
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		super.mouseReleased(e);
		dragSelection = null;
	}
	
	protected void mouseButton2Dragged(int x, int y, int dx, int dy) {
		try {
			if (dragSelection != null && dragSelection instanceof GalaxyMapStellarObject) {
				StellarObjectDelegate sod = ((GalaxyMapStellarObject)dragSelection).getStellarObject();
				LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
				if (localDelegate != null) {
					UserPlayerDelegate owner = getOwner();
					StellarObject ss = localDelegate.getStellarObject(sod);
					float scale = view.getMapPanel().getZoomScale();
					ss.changeLocationTo((int)(x/scale), (int)(y/scale));
					Player o = localDelegate.getPlayer(owner);
					sod.update(EngineVoFactory.getStellarObject(ss, o));
					SectorOwnerGrid sog = localDelegate.getEngine().getInfluence().getSectorOwnerGrid();
					DeligateCacheManager.update(new StateUpdateCommandResult(sog));
					getView().getMapPanel().rebuildContent();
					getView().getRadarPanel().repaint();
				}
			}
		} catch (Exception e) {
			Logger.logError("", e);
		}
	}
	
	public boolean isMouseButton1Down() {
		return mouseButton1Down;
	}


    //
    // HELPER METHODS
    //


    public void setMapWindowSize() throws ApplicationError {
        ServerProxy server = ProxyStore.getInstance();
        if (server != null && server.isConnected()) {
        	GalaxyMapPanel mapWindow = view.getMapPanel();
        	JScrollPane mapWindowScrollPane = view.getMapPanelScrollPane();
	        Size s = DeligateCacheManager.getGalaxyDetails().getSize();
	        int x = (int)(s.getX()*mapWindow.getZoomScale());
	        int y = (int)(s.getY()*mapWindow.getZoomScale());
	        mapWindow.setPreferredSize(new Dimension(x, y));
	        mapWindowScrollPane.getViewport().setViewSize(new Dimension(x,y));
	    }
    }
    
    public void focusOnHomeSolarSystem() throws ApplicationError {
		UserPlayerDelegate owner = getOwner();
    	PlanetDelegate homePlanet = owner.getHomePlanet();
    	if (homePlanet != null) {
    		SolarSystemDelegate ss = homePlanet.getSolarSystem();
    		if (ss != null) {
    			GalaxyMapPanel mapWindow = view.getMapPanel();
    			JScrollPane mapWindowScrollPane = view.getMapPanelScrollPane();
    			Location loc = ss.getLocation();
    			float zoomScale = mapWindow.getZoomScale();
    			int x = (int)(loc.getX()*zoomScale);
    			int y = (int)(loc.getY()*zoomScale);
    			view.focus(x, y);
    		}
    	}
    }
    
    public NamedObjectDelegate getSelection() {
    	return state.getSelection();
    }
    
    public void clearSelection() throws ApplicationError {
		state.setSelection(null);
		try {
			view.getEditorPanel().setContent(MainWindow.getOwner(), null);
		} catch (ValidationError e) {
			MainWindow.getExestingInstance().reportValidationError(e);
		}
    }
    
    public void setSelection(NamedObjectDelegate selection) throws ApplicationError {
    	state.setSelection(selection);
    }
    
	public void setView(EditorWindowView view) {
		this.view = view;
		this.state = new EditorWindowState(null);
		super.setView(view);
	}

	public EditorWindowState getState() {
		return state;
	}

	public EditorWindowView getView() {
		return view;
	}

	public void click(MoveableOwnedObjectDelegate moo) throws ApplicationError, ValidationError { 
		setSelection(moo);
		MoveableOwnedObjectDelegate[] selectionList = view.getMapPanel().getSelectionList(moo);
		view.getEditorPanel().setContent(moo, selectionList);
	}

	public void click(StellarObjectDelegate so, boolean autoselectPlanetOrbit) throws ApplicationError, ValidationError { 
		playActionSound();
		setSelection(so);
		view.getEditorPanel().setContent(so, null);
	}
	
	public void click(PlanetDelegate planet) throws ApplicationError, ValidationError {
		playActionSound();
		setSelection(planet);
		view.getEditorPanel().setContent(planet, null);
	}

	protected String getKeyBindingsMessage() {
		return null;
	}

	protected void motKeyTyped(KeyEvent e) throws ApplicationError, ValidationError {
	}

}
