/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.editor.swing.window;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import mot.client.swing.combos.PlayerCombo;
import mot.client.swing.common.ComponentTable;
import mot.client.swing.common.NameControlH;
import mot.client.swing.window.galaxy.GalaxyMapPanel;
import mot.client.swing.window.galaxy.GalaxyView;
import mot.client.swing.window.galaxy.GalaxyWindowRadarPanel;
import mot.client.swing.window.galaxy.GalaxyZoomSlider;
import mot.client.swing.window.galaxy.MotGalaxyMapActions;
import mot.client.swing.window.galaxy.WindowShowingNotification;
import mot.client.swing.window.galaxy.actions.ActionChangeZoom;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.MainWindowMessageList;
import mot.client.swing.window.main.MainWindowView;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;

public class EditorWindowView extends MainWindowView implements MotGalaxyMapActions, GalaxyView {
	
	private EditorWindowListener listener;
    private Color backgroundColour = Color.BLACK;
    private GalaxyMapPanel mapPanel = null;
    private GalaxyZoomSlider zoomSlider;
    private GalaxyWindowRadarPanel radarPanel;
    private MainWindowMessageList messageList = new MainWindowMessageList();
    private ComponentTable controlPanel;
    private PlayerCombo playerSelect;
    private EditorPanel editorPanel;
    private JPanel mainPanel;
    
    public EditorWindowView(EditorWindowListener listener) throws ApplicationError, ValidationError {
    	this.listener = listener;
    	listener.setView(this);
    	
        mapPanel = new GalaxyMapPanel(listener, true, true);
        mapPanel.setOpaque(true);
        mapPanel.setBackground(backgroundColour);
        
        mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(mapPanel.getScrollPane(), BorderLayout.CENTER);
        mainPanel.add(messageList, BorderLayout.SOUTH);
        messageList.setPreferredSize(new Dimension(800,50));
        
        radarPanel = new GalaxyWindowRadarPanel(200);
        zoomSlider = new GalaxyZoomSlider(this);
        playerSelect = new PlayerCombo(true, false);
        editorPanel = new EditorPanel(listener);

        controlPanel = new ComponentTable("HV", 4, 1, 0, 3, 3);
        controlPanel.add(radarPanel, "HV");
        controlPanel.add(zoomSlider, "HV");
        controlPanel.add(new NameControlH(L.PLAYER, playerSelect), "HV");
        controlPanel.add(editorPanel, "HVPS");
        controlPanel.setBackground(PropertyManager.getMandatoryAsColor("window.main.background"));
        
        mapPanel.addMouseWheelListener(listener);
        mapPanel.getScrollPane().addMouseWheelListener(listener);
        mapPanel.getScrollPane().getViewport().addMouseWheelListener(listener);
        editorPanel.addMouseWheelListener(listener);
        controlPanel.addMouseWheelListener(listener);
        radarPanel.addMouseWheelListener(listener);
        mainPanel.addMouseWheelListener(listener);
        playerSelect.addActionListener(new EditorPlayerListener(playerSelect, editorPanel));
    	
    }
    
    public void deactivate() throws ApplicationError {
    	MainWindow win = MainWindow.getInstance();
    	win.getContentPane().removeAll();
    }
    
    public void activate() throws ApplicationError {
    	MainWindow win = MainWindow.getInstance();
    	win.getContentPane().add(mainPanel, BorderLayout.CENTER);
    	win.getContentPane().add(controlPanel, BorderLayout.EAST);

    	mapPanel.setNowShowingNotification(new WindowShowingNotification( ) {
			public void nowShowingNotification() throws ApplicationError {
				if (zoomSlider != null) zoomSlider.requestFocusInWindow();
			}
		});  
    }

    public void handleTick() throws ApplicationError {
    	// Must not paint window while mouse is dragging screen around.
    	if (listener.mouseButton1Down) return;
        mapPanel.repaint();
        radarPanel.handleTick();
        messageList.tick();
    }
    
    public void repaintMap() {
        mapPanel.repaint(); 
        radarPanel.repaint();
    }
    
    /**
     * Set the centre of view port to scroll pane on location x,y.
     * @param x the centre to scroll to (x is in game engine coordinates)
     * @param y the centre to scroll to (y is in game engine coordinates)
     */
    public void focusOnMap(int x, int y) {
    	int mx = (int)(x * mapPanel.getZoomScale());
    	int my = (int)(y * mapPanel.getZoomScale());
    	focus(mx, my);
    }

	public void reportApplicationError(Exception e) {
		Logger.logError("MainWindow.actionPerformed() Failed due to error ", e);
		messageList.addMessage(e.getMessage());
	}

	public void reportValidationError(ValidationError e) {
		messageList.addErrorMessage(e.getMessage(), true);
	}

	public void showMessage(String msg, boolean flash, Color flashBkColor, String sound){
		if (flash) {
			messageList.addTemporaryMessage(msg, true, flashBkColor, sound, 0);
		}
		messageList.addMessage(msg);
	}

	public void showTemporaryMessage(String msg, boolean flash, Color bkColor, String sound, int additionalSecsToExpire) {		
		messageList.addTemporaryMessage(msg, flash, bkColor, sound, additionalSecsToExpire);
	}

	public boolean isDisplayingTemporaryMessage() {
		return messageList.isDisplayingTemporaryMessage();
	}
	
	public JScrollPane getMapPanelScrollPane() {
		return mapPanel.getScrollPane();
	}

	public ActionChangeZoom getZoomSliderListener() {
		return zoomSlider.getListener();
	}
	
	public GalaxyMapPanel getMapPanel() {
		return mapPanel;
	}
	
	public EditorWindowListener getListener() {
		return listener;
	}

	public void refresh() throws ApplicationError, ValidationError {
		mapPanel.refresh();
	}

	public EditorPanel getEditorPanel() {
		return editorPanel;
	}

	public GalaxyWindowRadarPanel getRadarPanel() {
		return radarPanel;
	}
	
}
