/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.editor.swing.window.actions;

import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.swing.buttons.MotButton;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.editor.swing.window.EditorWindowListener;
import mot.engine.MotEngine;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

public class DeletePlanetButton extends MotButton {
    
	private EditorWindowListener listener;
	private PlanetDelegate planetDelegate;

	public DeletePlanetButton(EditorWindowListener listener, PlanetDelegate planet) {
		super("Delete Planet");
		this.listener = listener;
		this.planetDelegate = planet;
	}

	public DeletePlanetButton(EditorWindowListener listener) {
		super("Delete Planet");
		this.listener = listener;
	}

	protected void _doAction() throws ApplicationError, ValidationError {
		LocalServerProxy localDelegate = ProxyStore.getInstance().getLocalServerProxy();
		if (localDelegate == null) {
			throw new ValidationError(L.EDITOR_REQUIRES_LOCAL_CLIENT);
		}
		
		if (planetDelegate == null) return;
		
		Planet planet = localDelegate.getPlanet(planetDelegate);
		
		if (MotEngine.instance.getMoveableOwnedObjects().getMOOsAt(planet) != null) {
			throw new ValidationError("Can only delete planets with nothing in orbit");
		}
		
		if (MotEngine.instance.getMoveableOwnedObjects().getMOOsWithDestination(planet) != null) {
			throw new ValidationError("Can only delete planets with nothing traveling to it");
		}
		
		SolarSystem ss = planet.getSolarSystem();
		Player owner = planet.getOwner();
		if (MotEngine.instance.getTradeRoutes().getTradeRoutes(ss) != null && ss.getQtyColonies(owner, false) < 2) {
			throw new ValidationError("Can only delete planets that do not have a trade route");
		}
		
		ComputerPlayer[] computerPlayers = MotEngine.instance.getPlayersNotNull().getComputerPlayers();
		if (computerPlayers != null) {
			for (int i = 0; i < computerPlayers.length; i++) {
				if (computerPlayers[i].getStratagist().referencesPlanet(planet)) {
					throw new ValidationError("Can only delete planets with no owner");
				}
			}
		}
		
		planet.removeColony(true);
		
		MotEngine.instance.getPlanets().removePlanet(planet);
		
		MotEngine.instance.getInfluence().update();
		DeligateCacheManager.flush();
		CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();
		ProxyStore.getInstance().updateDelegates(callbackSync);
        callbackSync.waitTillComplete();
		//GalaxyAnimationRegister.clean();
		listener.getView().getMapPanel().rebuildContent();
		
		planetDelegate = null;
	}

	public PlanetDelegate getPlanetDelegate() {
		return planetDelegate;
	}

	public void setPlanetDelegate(PlanetDelegate planetDelegate) {
		this.planetDelegate = planetDelegate;
	}
}
