/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Deque;
import java.util.Iterator;

import mot.common.proxy.ClientProxy;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.event.MotEventFactory;
import mot.engine.players.Player;

/**
 * This class manages all the connection from clients and is used to broadcast information
 * to the clients.
 */
public class ClientProxyManager  {

    private class EventQueueEntry {
    
        public MotEventFactory event;
        public Player player;
    
        public EventQueueEntry(MotEventFactory event) {
            this.event = event;
        }
        
        public EventQueueEntry(MotEventFactory event, Player player) {
            this.event = event;
            this.player = player;
        }
    }

    /**
     * The human users playing the game (List of objects that are a subclass of Proxy).
     * Note the Unit test suite also registers its self as a proxy to view events.
     * This is not serialised when saving the game. Each user must individually connect
     * to the game. Note a new game turn cannot start until all players have moved so
     * all players must connect using their own proxy before being able to continue with
     * a game.
     */
    private transient ArrayList proxies;
    
    /**
     * This is a list of queued events. See scheduleBroadcast() for details.
     * This array contains a list of EventQueueEntry objects.
     * NOTE - It is more than possible that events will be added to this queue while broadcasting the 
     * existing events in the queue.
     */
    private  Deque<EventQueueEntry> eventQueue = new ArrayDeque();

    public ClientProxyManager() {
    }


    private boolean isEventQueueHead(MotEventFactory event) {
        if (eventQueue.size() == 0) return false;
        EventQueueEntry eqe = eventQueue.peek();
        return (eqe.event == event);
    }
    
    public void broadcastQueue() throws ApplicationError {
        while (eventQueue.size() > 0) {
            EventQueueEntry eqe = eventQueue.removeFirst();
            try {
				_broadcast(eqe.event, eqe.player);
			} catch (Exception e) {
				// We do not want to lose events so we consume this error and assume the next event 
				// will be processed successfully!
				Logger.logError("Failed to send event: "+eqe.event.toString(), e);
			}
        }
        eventQueue.clear();
    }
    
	/**
	 * Removes this event from the list of events scheduled to be broadcast.
	 * If this event was the head of the queue then the event queue is broadcast.
	 * @throws ApplicationError 
	 */
	public void deleteScheduledBroadcast(MotEventFactory event) throws ApplicationError {
		if (isEventQueueHead(event)) {
			// There may be more instances of this event waiting to be sent to other players
			while (remove(event, null));
			broadcastQueue();
		} else {
			// There may be more instances of this event waiting to be sent to other players
			while (remove(event, null));
		}
	}
	
	private boolean remove(MotEventFactory event, Player player) throws ApplicationError {
		EventQueueEntry eqe = find(event, player);
		if (eqe != null ) {
			eventQueue.remove(eqe);
			return true;
		} else {
			return false;
		}
	}
	
    private EventQueueEntry find(MotEventFactory event, Player player) {
        Iterator i = eventQueue.iterator();
        while (i != null && i.hasNext()) {
            EventQueueEntry eqe = (EventQueueEntry) i.next();
            if (player == null) {
            	if (eqe.event == event) {
            		return eqe;
            	}
            } else {
            	if (eqe.event == event && eqe.player == player) {
            		return eqe;
            	}
            }
        }
        return null;
    }
    
    private boolean contains(MotEventFactory event, Player player) {
        Iterator i = eventQueue.iterator();
        while (i != null && i.hasNext()) {
            EventQueueEntry eqe = (EventQueueEntry) i.next();
            if (eqe.event == event && eqe.player == player) return true;
        }
        return false;
    }
    
    private LinkedEventFactoryBase getEventToContain(MotEventFactory event) {
        Iterator i = eventQueue.iterator();
        while (i != null && i.hasNext()) {
            EventQueueEntry eqe = (EventQueueEntry) i.next();
            if (eqe.event instanceof LinkedEventFactoryBase) {
                LinkedEventFactoryBase leb = (LinkedEventFactoryBase) eqe.event;
                if (leb.wantsToLink(event)) return leb;
            }
        }
        return null;
    }
    
    private void addToQueue(MotEventFactory event, Player player) throws ApplicationError {
        if (contains(event, player)) {
            return;
        }

        LinkedEventFactoryBase masterEvent = getEventToContain(event);

        if (masterEvent != null) {
        	if (masterEvent.containsLinkedEvent(event)) {
        		return;
        	}
            if (Logger.isDebug()) {
                Logger.logDebug("ClientProxyManager.addToQueue() Event "+event.getClass().getName()+" linked to existing event");
            }
            masterEvent.link(event);
            return;
        }

        if (Logger.isDebug()) {
            if (player == null) {
                Logger.logDebug("ClientProxyManager.addToQueue() Event "+event.getClass().getName()+" to all players");
            } else {
                Logger.logDebug("ClientProxyManager.addToQueue() Event "+event.getClass().getName()+" to "+player.getName());
            }
        }

        eventQueue.add(new EventQueueEntry(event, player));
    }

    /**
     * This is a shortcut for broadcast(MotEventFactory, Player) where player is null
     * so that all player receive this event.
     */
	public void broadcast(MotEventFactory e) throws ApplicationError {
        broadcast(e, (Player)null);
    }

	public void multicast(MotEventFactory e, Player [] players) throws ApplicationError {
    	if (players != null) {
    		for (int i = 0; i < players.length; i++) {
    			if (players[i] != null) {
    				broadcast(e, players[i]);
    			}
    		}
    	}
	}
	
    /**
     * Send an event to all proxies in the game. Events are used to inform individual user 
     * interfaces (GUI's) to display some graphic or open a screen to display information 
     * to a user.
     * @param e This is an event factory that generates the correct event for this proxy.
     * Because different proxies use different types of event (local proxies need local events
     * and remote proxies need remote events) this parameter must be an event factory that can
     * generate the correct type of events for each type of proxy.
     * @param player If this is null then this event is broadcast to the proxies of all players
     * in the game. If player is not null then this event is only sent to proxies owned by this player.
     * @throws mot.common.util.ApplicationError
     */
	public void broadcast(MotEventFactory e, Player player) throws ApplicationError {
        if (eventQueue.size() > 0) {
            if (isEventQueueHead(e)) {
            	Logger.logWarning("Should send schedule: "+e);
                broadcastQueue();
            } else {
                addToQueue(e, player);
            }
        } else {
            _broadcast(e, player);
        }
    }
        
	private void _broadcast(MotEventFactory e, Player player) throws ApplicationError {
        if (Logger.isDebug()) {
            if (player == null) {
                Logger.logDebug("ClientProxyManager._broadcast() Event "+e.getClass().getName()+" to all players");
            } else {
                Logger.logDebug("ClientProxyManager._broadcast() Event "+e.getClass().getName()+" to "+player.getName());
            }
        }
		if (proxies != null) {
			Iterator i = proxies.iterator();
			while (i != null && i.hasNext()) {
				ClientProxy p = (ClientProxy)i.next();
                if (player == null || player == p.getOwner()) {
                    p.notify(e);
                }
			}
		}
	}
    
    private ClientProxy _getProxy(Player player) throws ApplicationError {
		if (proxies != null) {
			Iterator i = proxies.iterator();
			while (i != null && i.hasNext()) {
				ClientProxy p = (ClientProxy)i.next();
                if (player == null || player == p.getOwner()) {
                    return p;
                }
			}
		}
        return null;
    }

    /**
     * @throws mot.common.util.ApplicationError
     * @return An array of names for all players who have registered a proxy.
     */
    public String [] getPlayerNames() throws ApplicationError {
		if (proxies == null) {
            return null;
        }
        String [] r = new String [proxies.size()];
        for (int i=0; i<proxies.size(); i++) {
            ClientProxy p = (ClientProxy)proxies.get(i);
            r[i] = p.getOwner().getName();
        }
        return r;
    }

    /**
     * @throws mot.common.util.ApplicationError
     * @return An array of ClientProxy for all players who have registered a proxy.
     */
    public ClientProxy [] getProxies() throws ApplicationError {
		if (proxies == null) {
            return null;
        }
        ClientProxy [] r = new ClientProxy [proxies.size()];
        for (int i=0; i<proxies.size(); i++) {
            ClientProxy p = (ClientProxy)proxies.get(i);
            r[i] = p;
        }
        return r;
    }


    /**
     * @throws mot.common.util.ApplicationError
     * @return An array of ClientProxy for all players who have registered a proxy.
     */
    public ClientProxy [] getProxies(Player exclude) throws ApplicationError {
		if (proxies == null) {
            return null;
        }
        ArrayList<ClientProxy> r = new ArrayList();
        for (int i=0; i<proxies.size(); i++) {
            ClientProxy p = (ClientProxy)proxies.get(i);
            if (p.getOwner() != exclude) {
            	r.add(p);
            }
        }
        if (r == null || r.size() == 0) {
        	return null;
        } else {
        	return r.toArray(new ClientProxy [r.size()]);
        }
    }

    /**
     * @throws mot.common.util.ApplicationError
     * @return The proxy for player p.
     */
    public ClientProxy getProxy(Player p) throws ApplicationError {
		if (proxies == null) {
            return null;
        }
        for (int i=0; i<proxies.size(); i++) {
            ClientProxy proxy = (ClientProxy)proxies.get(i);
            if (proxy.getOwner() == p) {
            	return proxy;
            }
        }
       	return null;
    }

    public void scheduleMulticast(MotEventFactory e, Player [] players) throws ApplicationError {
    	if (players != null) {
    		for (int i = 0; i < players.length; i++) {
    			if (players[i] != null) {
    				scheduleBroadcast(e, players[i]);
    			}
    		}
    	}
    }
    /**
     * This schedule an event for broadcasting - but does not broadcast it yet. It is added to a send queue
     * and will only be sent when broadcast is called later (broadcasting the same event). All other events
     * broadcast after this event are added to the send queue behind this event and are sent after this event.
     * 
     * Note - This is used when the order in which events are sent is important but this event is not in its 
     * final state ready for sending but in preparing the event other events will be sent that must arrive 
     * after this event.
     * 
     * @param e The event to broadcast at a later time.
     * @param player The player to send this event to at a later time (or null for all players).
     */
    public void scheduleBroadcast(MotEventFactory e, Player player) throws ApplicationError {
    	//Logger.logDebug("ClientProxyManager.scheduleBroadcast()");
    	//Logger.logStackTrace();
        addToQueue(e, player);
    }

	public void sendScheduledEvent(MotEventFactory e) throws ApplicationError {
        if (eventQueue.size() > 0) {
            if (isEventQueueHead(e)) {
            	broadcastQueue();
            }
        }
	}


//	public void add(ClientProxy [] p) {
//        if (p == null) return; broken
//        for (int i=0; i<p.length; i++) {
//            add(p[i]);
//        }
//	}

	public void add(ClientProxy p) {
		if (proxies == null) proxies = new ArrayList();
		proxies.add(p);
	}

	public void remove(ClientProxy p) {
		if (proxies == null) return;
		proxies.remove(p);
	}


	public void removeAll() {
		proxies = new ArrayList();
	}




}