/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import mot.common.bean.GameConstants;
import mot.common.bean.PlayerColor;
import mot.common.bean.Settings;
import mot.common.bean.Size;
import mot.common.bean.TutorialStatus;
import mot.common.designs.CraftDesigns;
import mot.common.event.StateContainer;
import mot.common.proxy.ComputerPlayerProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.technology.GeneralTechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.NameGenerator;
import mot.common.util.PropertyManager;
import mot.common.util.ValidationError;
import mot.common.vo.DateVO;
import mot.common.vo.FlushCraftCacheVO;
import mot.common.vo.PlayerTechnologiesVO;
import mot.common.vo.TechnologyAvailableForResearchVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLDocumentHandler;
import mot.common.xml.XMLWriter;
import mot.engine.adapter.ComputerPlayerVOAdapter;
import mot.engine.battles.Battle;
import mot.engine.battles.Battles;
import mot.engine.command_handlers.FetchGalaxyPreviewCmdHandler;
import mot.engine.command_handlers.FetchStealableTechnologiesHistory;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.MoveableOwnedObjects;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Crafts;
import mot.engine.crafts.Fleet;
import mot.engine.event.EndTurnEF;
import mot.engine.event.EventContainerEF;
import mot.engine.event.GameEndEF;
import mot.engine.event.MotEventFactory;
import mot.engine.event.PlayersWaitingForEndTurnEF;
import mot.engine.event.StateUpdateEF;
import mot.engine.leaders.LeaderFactory;
import mot.engine.planets.InfluenceGrid;
import mot.engine.planets.Planet;
import mot.engine.planets.Planets;
import mot.engine.planets.StellarObjects;
import mot.engine.planets.TradeRoutes;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.PlayerCharacter;
import mot.engine.players.Players;
import mot.engine.players.ResearchProject;
import mot.engine.players.UserPlayer;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.spies.Spies;
import mot.engine.transports.Transport;

/**
 * This is the running game. This is a singleton class - see getInstance() for details.
 * A Game may contain a any number of ComputerPlayers and UserPlayers and each Player
 * may own any number of StellarObjects and Crafts/Fleets.
 * All Players in the Game share a single map represented by the list of SolarSystems in the
 * game.
 * This is a turn based game and battles are executed at the end of each turn. A game 
 * therefore also consists of a list of battles waiting to be executed at the end of the 
 * turn. Once a battle has been executed it will be removed from the list of battles.
 * 
 * Client programs should communicate with the engine using a ServerProxy. The server proxy 
 * can be used to create new games / load game as well as issue commands to the engine during
 * a game.
 * 
 * The engine responds to clients by (sometimes) returning a value from a command, but usually
 * by issuing events. Events a sent to all interested parties when generated. Clients do not 
 * use server side objects directly - they use delegates to the server objects. Lookup delegate 
 * design pattern for details. If a client requests a player from the server's proxy the proxy 
 * will communicate with the server on the clients behalf then return the client a PlayerDeligate. 
 * 
 * The ServerProxy and all the delegates are interfaces that have multiple implementations. The
 * two basic flavors are local and remote. Local implementation are just thin wrappers around
 * the server objects while the remote implementation use IPC to get Value Objects (see design 
 * pattern for details) from the server.
 * 
 */
public class MotEngine extends XMLBean {

    public static final int START_DATE = 3000;

	/**
     * The List of Players in the game.
     */
    private Players players;

    /**
     * The List of Players in the game that have been killed.
     * We keep a record of these players for statistical data AND player origin information.
     */
    private Players deadPlayers;

    /**
     * This is the monster player in this game.
     */
    private MonsterPlayer monsterPlayer;

    /**
     * The List of Stellar Objects in the game.
     */
    private StellarObjects stellarObjects;

    /**
     * The List of Planets in the game.
     */
    private Planets planets;

    /**
     * The List of Crafts in the game.
     */
    private Crafts crafts;

    /**
     * A List of battles that will occur at the end of this turn. At the end of each
     * turn all the battles are then deleted.
     */
    private Battles battles;
    
    /**
     * A List of relationships between the players. This includes diplomatic status
     * such as war, No aggression treaty etc and trade/research treaties.
     */
    private Relations relations;

    /**
     * Current date in game.
     */
    private int date = START_DATE;

    /**
     * The level the game is being played at. This is a value between 0.1 and +n used
     * as a percentage change to resources by computer players and their relationship
     * to the user player.
     */
    private double level = 1;
    
    /**
     * If the game is in tutorial mode - then this not not null and contains tutorial details.
     * This is normally null.
     */
    private TutorialStatus tutorial;
    
    /**
     * This is used by games containing a single user. If this is null then this is a multi
     * user game and the user must supply his user name and password.
     * It is the name of the single user for this game.
     */
    private String userPlayerName;

    /**
     * The proxy manager manages all the proxies to our clients and is used to send events
     * to the GUI.
     */
    private ClientProxyManager proxyManager = new ClientProxyManager();

    /**
     * The object used to generate new names. This is used to make sure that no duplicates
     * are generated.
     */
     private NameGenerator nameGenerator;
     
    /**
     * This is the list of spies in the game. Spies are owned by players and assigned to other 
     * players in a SpyRole. Each player has a predefined quantity of spies based upon his total
     * population and his influence. These spies are allocated to tasks using the Intelligence
     * interface for computer players and the intelligence window for user players.
     */
     private Spies spies;
     
     /**
      * This is a 'conceptual' container for Movable Owned Objects.
      * It does not actually contains anything, its stateless, but is used to locate / access
      * Moveable Owned Objects.
      */
     private MoveableOwnedObjects moos;
     
     /**
 	 * Influence is the player's ability to convince the galaxy's
 	 * population of their opinions / objectives. If player A has a higher 
 	 * influence in a sector of the galaxy where they both contains a populated
 	 * planet. The player with a lower influence will find that some of his
 	 * population sympathise with the other players cause and this lose some
 	 * production to the player with more influence.
 	 * 
 	 * To speed things up influence is only calculated at the end of every 
 	 * turn. The influence for each player in the galaxy is stored in a class
 	 * called InfluenceGrid. It contains a table. Each cell in the table is 
 	 * called a sector and is a region on the galaxy map. At the end of every
 	 * turn the table is updated with the latest information on influence.
      */
     private InfluenceGrid influence;
     
     /**
      * This is a list of all the trade routes in the galaxy.
      */
     private TradeRoutes tradeRoutes;

     /**
      * List of technologies suggested to swap / give / take by
      * FetchStealableTechnologiesCmdHamdler.
      * NOT SAVED
      */
 	 public Map<String,FetchStealableTechnologiesHistory> fetchHistory = new HashMap();

    /**
     * Ends the current turn of the user passed in. It checks to see if all human
     * players have moved, and ends the game turn if they have. When the game turn
     * ends it, moves all crafts, fleets, computer players, starts battles and calculates 
     * all the new research, money, and production for each colony.
     * Note It may contain BattleProxy objects as well as normal proxies. Thus a single
     * player may have more than one proxy!
     */
    public void endTurn(Player player) throws ApplicationError, ValidationError {
		player.setTurnEnded(true);
		boolean allPlayersMoved = getPlayers().allUserPlayersMoved();
		if (allPlayersMoved) {
            autosave();

			EndTurnEF[] endTurnEvents = getPlayers().createEndTurnEventForEachUserPlayer();
            date++;
            if (Logger.isDebug()) {
                Logger.logDebug("MotEngine.endTurn() *** ALL USER PLAYERS ENDED TURN *** ["+date+"]");
            }
			// Move players first so priorities can be calculated
			getPlayers().endTurn();
			if (getPlanets() != null) getPlanets().endTurn();
			if (getCrafts() != null) getCrafts().endTurn();
			getPlayers().endFleetsTurn();
            getStellarObjects().endTurn();
			if (getSpies() != null) getSpies().endTurn();
            getMonsterPlayer().endTurn();
            LeaderFactory.endTurn();
            getRelationsNotNull().endTurn();
            getInfluence().update();
            getTradeRoutes().endTurn();
            
            for (int i = 0; i < endTurnEvents.length; i++) {				
            	MotEngine.instance.addStateUpdate(endTurnEvents[i], endTurnEvents[i].getPlayer(), true, false, false, true, true, true, true, false, false, true, false);
			}
            MotEngine.instance.getProxyManager().broadcastQueue();
            
			if (battles != null && battles.getQtyBattles() > 0) {
				Battle battle = battles.start(true);
                if (!battle.hasHumanPlayer()) {
                    battle.automate();
                    startNextBattleInQueue();
                }
                // Make sure all remaining events are broadcast to the clients.
                MotEngine.instance.getProxyManager().broadcastQueue();
			} else {
                getPlayers().startTurn();
            }
            MotEngine.instance.getProxyManager().broadcastQueue();
            
            getPlayers().destroyFinishedPlayers();
            if (hasEnded()) {
            	broadcastEndGame();            
            }
            
            if (Logger.isDebug()) {
                logSummary();
            }            
		} else {
			UserPlayer[] upa = getPlayers().getUserPlayers();
			if (upa != null) {
				for (int i = 0; i < upa.length; i++) {
					if (upa[i] != player && !upa[i].getTurnEnded()) {
						broadcast(new PlayersWaitingForEndTurnEF(), upa[i]);
					}
				}
			}
		}
	}

    public void __endTurn_CraftsFleetsBattles() throws ApplicationError, ValidationError {
		EndTurnEF[] endTurnEvents = getPlayers().createEndTurnEventForEachUserPlayer();
        date++;
        if (Logger.isDebug()) {
            Logger.logDebug("MotEngine.endTurn() *** ALL USER PLAYERS ENDED TURN *** ["+date+"]");
        }
        getPlayers().endTurn();
		if (getCrafts() != null) getCrafts().endTurn();
		getPlayers().endFleetsTurn();
        
        for (int i = 0; i < endTurnEvents.length; i++) {				
        	MotEngine.instance.addStateUpdate(endTurnEvents[i], endTurnEvents[i].getPlayer(), true, false, false, true, true, true, true, false, false, true, false);
		}
        MotEngine.instance.getProxyManager().broadcastQueue();
        
		if (battles != null && battles.getQtyBattles() > 0) {
			Battle battle = battles.start(true);
            if (!battle.hasHumanPlayer()) {
                battle.automate();
                startNextBattleInQueue();
            }
            // Make sure all remaining events are broadcast to the clients.
            MotEngine.instance.getProxyManager().broadcastQueue();
		} else {
            getPlayers().startTurn();
        }
        MotEngine.instance.getProxyManager().broadcastQueue();
        
        getPlayers().destroyFinishedPlayers();
        if (hasEnded()) {
        	broadcastEndGame();            
        }
        
        if (Logger.isDebug()) {
            logSummary();
        }
	}

	public void _sendStateUpdateEvents(Player p, boolean updatePlayers, boolean updatePlanets, boolean updateCrafts, boolean updateStellarObjects, boolean updateSOG, boolean includeStaticData) throws ApplicationError {
		EventContainerEF [] ca = new EventContainerEF [1];
		ca[0] = new EventContainerEF();
        ca[0].setTarget(p);
		sendStateUpdateEvents(ca, updatePlayers, updatePlanets, updateCrafts, updateStellarObjects, updateSOG, includeStaticData);
	}
	
	private void sendStateUpdateEvents(EventContainerEF[] schedulesEventContainers, boolean updatePlayers, boolean updatePlanets, boolean updateCrafts, boolean updateStellarObjects, boolean updateSOG, boolean includeStaticData) throws ApplicationError {
		if (schedulesEventContainers == null) return;
		for (int i = 0; i < schedulesEventContainers.length; i++) {
			StateUpdateEF event = new StateUpdateEF();
			Player target = schedulesEventContainers[i].getTarget();
			addStateUpdate(event, target, updatePlayers, false, false, updatePlanets, updateCrafts, updateStellarObjects, updateSOG, includeStaticData, false, false, false);
			schedulesEventContainers[i].setEvent(event);
	        MotEngine.instance.broadcast(schedulesEventContainers[i], target);
		}
	}

	public void addStateUpdate(StateContainer event, Player viewer, boolean updatePlayers, boolean updateCraftDesigns, boolean updatePlayerTechnologies, boolean updatePlanets, boolean updateCrafts, boolean updateStellarObjects, boolean updateSOG, boolean includeStaticData, boolean includeTechnologyAvailableForResearch, boolean includeDate, boolean includePlayerConstants) throws ApplicationError {
		if (updatePlayers) {
			event.addStateUpdate(EngineVoFactory.getPlayers(getPlayers().getPlayers()));
			if (deadPlayers != null) {
				event.addStateUpdate(EngineVoFactory.getDeadPlayers(deadPlayers.getPlayers()));
			}
			Relation[] rels = MotEngine.instance.getRelationsNotNull().getRelations(viewer);
			event.addStateUpdate(EngineVoFactory.getRelations(rels, viewer));
		}
		if (includePlayerConstants) {			
			event.addStateUpdate(EngineVoFactory.getPlayerConstants(getPlayers().getPlayers()));
			Players dps = getDeadPlayers();
			if (dps != null) {
				event.addStateUpdate(EngineVoFactory.getPlayerConstants(dps.getPlayers()));
			}
		}
		if (updateCraftDesigns && viewer instanceof UserPlayer) {
			CraftDesigns craftDesigns = ((UserPlayer)viewer).getCraftDesigns();
			if (craftDesigns != null) {
				event.addStateUpdate(EngineVoFactory.getCraftDesign((UserPlayer)viewer, viewer));
			}
		}
		if (updatePlayerTechnologies) {
			event.addStateUpdate(new PlayerTechnologiesVO(viewer.getName(), viewer));
		}
		if (includeDate) {
			event.addStateUpdate(new DateVO(date));
		}
		if (updatePlanets && planets != null) {
			event.addStateUpdate(EngineVoFactory.getPlanets(planets.getPlanets(), viewer));
			event.addStateUpdate(EngineVoFactory.getTradeRoutes(MotEngine.instance.getTradeRoutes(), viewer));
		}
		if (updateCrafts) {
			event.addStateUpdate(new FlushCraftCacheVO());
			event.addStateUpdate(EngineVoFactory.getMoos(getMoveableOwnedObjects().getMOOs(viewer, true, true), viewer));
		}
		if (updateStellarObjects && stellarObjects != null) {
			event.addStateUpdate(EngineVoFactory.getStellarObjects(getStellarObjects().getStellarObjects(), viewer));
		}
		if (updateSOG) {
			InfluenceGrid ig = getInfluence();
			if (ig != null) event.addStateUpdate(ig.getSectorOwnerGrid());
		}
		if (includeStaticData) {
			event.addStateUpdate(getStellarObjects().getGalaxyDetails());
			event.addStateUpdate(new ComputerPlayerVOAdapter(getMonsterPlayer()));
		}
		if (includeTechnologyAvailableForResearch) {
			event.addStateUpdate(new TechnologyAvailableForResearchVO(new GeneralTechnologyList(viewer.getTechnologyAvailableForResearch())));
		}
	}
	
	/**
     * @return The player who control the monsters in this game.
     * @throws mot.common.util.ApplicationError
     * @throws mot.common.util.ValidationError
     */
    public synchronized MonsterPlayer getMonsterPlayer() throws ApplicationError {
        if (monsterPlayer == null) {
            RaceOptions monsterOptions = OptionsFactory.getRaceOptions("monster");
            PlayerCharacter monsterCharacter = new PlayerCharacter("Monster", monsterOptions, null, null, null);
            monsterPlayer = new MonsterPlayer("Monster", monsterCharacter, PlayerColor.monsterColor);
        }
        return monsterPlayer;
    }

    public void logSummary() throws ApplicationError {
        StringBuffer sb = new StringBuffer();
        Player [] players = getPlayers().getPlayers();
        for (int i=0; i<players.length; i++) {
            sb.append("\nPlayer Name = "+players[i].getName()+"\n");
            if (players[i].getResearchProject() == null) {
                sb.append("No Research Project\n");                    
            } else {
                sb.append("Research = "+players[i].getResearchProject().getProjectDescription()+"\n");
            }
            if (players[i] instanceof ComputerPlayer) {
                sb.append("Craft Tasks : ");
                ComputerPlayer cp = (ComputerPlayer) players[i];
                if (cp.getGeneral() != null) {
                    sb.append(cp.getGeneral().getDescription());
                }
                sb.append("\n");
            }
            Planet [] planets = players[i].getPlanets();
            for (int j=0; planets != null && j<planets.length; j++) {
                sb.append("Has Planet="+planets[j].getName()+" project="+planets[j].getProjectDescription()+"\n");
            }
            Craft [] crafts = players[i].getCrafts();
            for (int j=0; crafts != null && j<crafts.length; j++) {
                if (crafts[j].getFleet() == null) {
                    sb.append("Has Craft="+crafts[j].getName()+" task="+crafts[j].getTaskDescription()+ " (task="+crafts[j].getTaskName()+")\n");
                }
            }
            Fleet [] fleets = players[i].getFleet();
            for (int j=0; fleets != null && j<fleets.length; j++) {
                sb.append("Has Fleet="+fleets[j].getName()+" task="+fleets[j].getTaskDescription()+" Craft=[");
                Craft [] fleetCraft = fleets[j].getCrafts();
                sb.append(Comp.toString(fleetCraft, ", "));
                sb.append("] ("+((fleets[j].getTask()==null) ? "No Task" : fleets[j].getTask().getName())+")\n");
            }
            Transport[] transports = players[i].getTransportManagerNotNull().getTransportsAsArray();
            for (int j=0; transports != null && j<transports.length; j++) {
                if (transports[j].getTask() != null) {
                	sb.append("Transport "+transports[j].getName()+": ");
                	sb.append(transports[j].getTask().getDescription());
                	sb.append("\n");
                }
            }            
            sb.append("Money "+players[i].getFunds()+" upkeep "+players[i].getTotalMaintenanceCost()+" income "+players[i].getTotalIncome(true)+"(funds "+players[i].getFunds()+")\n");
        }
        Logger.logDebug(sb.toString());
    }
    
    /**
     * A game is said to have ended if there are no human players in the game
     * OR there is only one player left.
     * @return True if an only if this game has ended.
     */
    public boolean hasEnded() {
    	if (players == null) {
    		return true;
    	}
    	UserPlayer[] userPlayers = players.getUserPlayers();
        return (players.getQuantity() <= 1 || userPlayers == null || userPlayers.length == 0);
    }

    public boolean hasBattleInQueue() throws ApplicationError {
    	return (battles != null && battles.getQtyBattles() > 0);
    }
        
    /**
     * Ends the battle at the solar system. If this is the last battle in progress then
     * the next turn begins.
     */
    public void startNextBattleInQueue() throws ApplicationError {
        if (battles == null) {
            return;
        }

        if (battles.getQtyBattles() > 0) {
            Battle newBattle = getBattles().start(true);
            if (!newBattle.hasHumanPlayer()) {
                newBattle.automate();
                // Make sure all remaining events are broadcast to the clients.
                MotEngine.instance.getProxyManager().broadcastQueue();
                startNextBattleInQueue();
            }
        } else {
            instance.getPlayers().startTurn();
        }

        getPlayers().destroyFinishedPlayers();
        if (hasEnded()) {
        	broadcastEndGame();            
        }
	}

    /**
     * Use this method to always return a Battles object.
     */
    public Battles getBattlesNotNull() { 
        if (battles == null) battles = new Battles();
        return battles; 
    }

    /**
     * Use this method to always return a Spies object.
     */
    public Spies getSpiesNotNull() { 
        if (spies == null) spies = new Spies();
        return spies; 
    }
    
    public Relations getRelationsNotNull() { 
        if (relations == null) relations = new Relations();
        return relations; 
    }

    public Players getPlayersNotNull() { 
        if (players == null) players = new Players();
        return players; 
    }

    public Players getDeadPlayersNotNull() { 
        if (deadPlayers == null) deadPlayers = new Players();
        return deadPlayers; 
    }

    public StellarObjects getStellarObjectsNotNull() { 
        if (stellarObjects == null) stellarObjects = new StellarObjects();
        return stellarObjects; 
    }

    private boolean spaceKaanEnabled = false;

    public Players getPlayers() { return players; }
    public Players getDeadPlayers() { return deadPlayers; }
    public StellarObjects getStellarObjects() { return stellarObjects; }
    public Planets getPlanets() { return planets; }
    public Crafts getCrafts() { if (crafts == null) crafts = new Crafts(); return crafts; }
    public Relations getRelations() { return relations; }
    public Battles getBattles() { return battles; }
    public int getDate() { return date; }
    public double getLevel() { return level; }
    public NameGenerator getNameGenerator() { if (nameGenerator == null) nameGenerator = new NameGenerator(); return nameGenerator; }
    public Spies getSpies() { return spies; }
    public boolean isSpaceKaanEnabled() { return spaceKaanEnabled;	}
    public MoveableOwnedObjects getMoveableOwnedObjects() { if (moos == null) moos = new MoveableOwnedObjects(); return moos; }

    public void setPlayers(Players v) { players = v; }
    public void setDeadPlayers(Players v) { deadPlayers = v; }
    public void setStellarObjects(StellarObjects v) { stellarObjects = v; }
    public void setPlanets(Planets v) { planets = v; }
    public void setCrafts(Crafts v) { crafts = v; }
    public void setRelations(Relations v) { relations = v; }
    public void setBattles(Battles v) { battles = v; }
    public void setDate(int v) { date = v; }
    public void setLevel(double v) { level = v; }
    public void setNameGenerator(NameGenerator v) { nameGenerator = v; }
    public void setSpies(Spies v) { spies = v; }
	public void setSpaceKaanEnabled(boolean v) { spaceKaanEnabled = v;	}

    public boolean isLevelEasy() { return level == GameConstants.LEVEL_EASY_BONUS; }
    public boolean isLevelNormal() { return level == GameConstants.LEVEL_NORMAL_BONUS; }
    public boolean isAHardLevel() { return level > GameConstants.LEVEL_NORMAL_BONUS; }
    
    public static double getPlanetMaintenanceFactor() {
    	if (instance == null) {
    		return 1.0;
    	} else {
            if (instance.isLevelEasy()) {
            	return 0.3;
            } else if (instance.isLevelNormal()) {
            	return 0.6;
            } else {
            	return 1.0;
            }
    	}
    }

    private void broadcastEndGame() throws ApplicationError {
    	String [] winners = (players == null) ? null : players.getPlayerNames();
        GameEndEF event = new GameEndEF(winners);
        MotEngine.instance.broadcast(event);            
	}

    
    /**
     * This is a short cut for proxyManager.broadcast(MotEventFactory).
     */
	public void broadcast(MotEventFactory e) throws ApplicationError {
        proxyManager.broadcast(e);
    }
    
    /**
     * This is a short cut for proxyManager.broadcast(MotEventFactory, Player).
     */
	public void multicast(MotEventFactory e, Player [] players) throws ApplicationError {
        proxyManager.multicast(e, players);
    }
    
    /**
     * This is a short cut for proxyManager.broadcast(MotEventFactory, Player).
     */
	public void broadcast(MotEventFactory e, Player player) throws ApplicationError {
        proxyManager.broadcast(e, player);
    }
	
	public void broadcastToUserPlayers(MotEventFactory e, Player exclude) throws ApplicationError {
		UserPlayer[] players = getPlayersNotNull().getUserPlayers();
		for (int i = 0; i < players.length; i++) {
			if (exclude == null || players[i] != exclude) {
				broadcast(e, players[i]);
			}
		}
	}

    
    /**
     * Creates proxies for all the computer players.
     */
    public void setupComputerPlayerProxies() {
        if (players != null) {
            Player [] list = players.getPlayers();
            for (int i=0; i<list.length; i++) {
                if (list[i] instanceof ComputerPlayer) {
                    ComputerPlayer cp = (ComputerPlayer)list[i];
                    proxyManager.add(new ComputerPlayerProxy(cp));
                }
            }
        }
    }

    /**
     * Running instance of the game.
     */
	public static MotEngine instance = null;

    /**
     * Get a the current instance of the Game.
     * Note - only one instance of a game can exist in any JVM.
     */
    public static MotEngine getInstance() {
		return instance;
    }

    /**
     * Replace the current instance stored in this game with the instance passed in.
     */
    public static void restoreInstance(MotEngine otherInstance) {
		instance = otherInstance;
    }

    /**
     * Generate a new game. If an instance of a game already exists it will be deleted.
     * Note - only one instance of a game can exist in any JVM.
     */
    public static MotEngine newInstance() {
		new MotEngine();
		return instance;
    }
    
    public static MotEngine newInstance(String userPlayerName) {
		new MotEngine();
        instance.userPlayerName = userPlayerName;
		return instance;
    }

    /**
     * Constructing an instance of MotEngine sets the instance variable so loading a game
     * has the correct instance.
     */
    public MotEngine() {
        instance = this;
    }
    
    /**
     * Create a new instance of MotEngine and load the saved game in file into the 
     * new instance.
     * @param file The file to load
     */
    public static MotEngine load(File file) throws ApplicationError {
    	MotEngine previewEngine = FetchGalaxyPreviewCmdHandler.getPreviewEngine(file);
		if (previewEngine != null) {
	        instance = previewEngine;
		} else {
	        instance = (MotEngine) XMLDocumentHandler.parse(file, false, null);
		}
		instance.fixMissingTecyhnologies();
		instance.getInfluence();
        instance.setupComputerPlayerProxies();
        return instance;
    }
    
    private void fixMissingTecyhnologies() {
    	Player[] pa = getPlayers().getPlayers();
    	for (int i = 0; i < pa.length; i++) {
			ResearchProject rp = pa[i].getResearchProject();
			if (rp != null && rp.getTechnology() == null) {
				Logger.logWarning("MotEngine.fixMissingTecyhnologies() delete Research Project for "+pa[i]);
				pa[i].setResearchProject(null);
			}
		}
	}

	private static final String AUTOSAVE_FILE_NAME = "saves/autosave.mot";

    private void autosave() {
        try {
        	int autosaveInterval = Settings.getInstance().getAutosaveInterval();
        	if (date % autosaveInterval == 0) {
        		save(new File(AUTOSAVE_FILE_NAME));
        	}
        } catch (Exception e) {
        	Logger.logError("autosave failed", e);
        }
    }

    private static final String BACKUP_FILE_NAME = "saves/backup.mot";

    /**
     * Save the current MotEngine into the file.
     * @param file The file to save
     */
    public void save(File file) throws ApplicationError {
        if (file.exists()) {
            File backup = new File(BACKUP_FILE_NAME);
            backup.delete();
            file.renameTo(backup);
        }
        XMLWriter writer = new XMLWriter(null, getElementName());
        toXML(writer);
        writer.save(file);
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("save-file-version", getCurrentSaveFileVersion());
		if (players != null) {
			players.toXML(writer);
		}
		if (deadPlayers != null) {
			deadPlayers.toXML(writer, "dead-players");
		}
        if (monsterPlayer != null) {
            monsterPlayer.toXML(writer);
        }
		if (planets != null) {
			planets.toXML(writer);
		}
		if (stellarObjects != null) {
			stellarObjects.toXML(writer);
		}
		if (crafts != null) {
			crafts.toXML(writer);
		}
		if (relations != null) {
			relations.toXML(writer);
		}
		if (nameGenerator != null) {
			nameGenerator.toXML(writer);
		}
		if (spies != null) {
			spies.toXML(writer);
		}
		if (tradeRoutes != null) {
			tradeRoutes.toXML(writer);
		}
        writer.addElement("date", date);
        writer.addElement("level", level);
        writer.addElement("space-kaan-enabled", spaceKaanEnabled);
        writer.addElement("user-player-name", userPlayerName);
        
        if (tutorial != null) tutorial.toXML(writer);
	}

	public void setSaveFileVersion(String fileVersion) throws ApplicationError {
		if (!Comp.same(PropertyManager.getMandatory("save.file.version"), fileVersion)) {
			// TODO
		}
	}
	
	private String getCurrentSaveFileVersion() throws ApplicationError {
		return PropertyManager.getMandatory("save.file.version");
	}

	protected String getElementName() { return "mot-engine"; }


    public void setUserPlayerName(String userPlayerName) {
        this.userPlayerName = userPlayerName;
    }


    public String getUserPlayerName() {
        return userPlayerName;
    }


    public void setMonsterPlayer(MonsterPlayer monsterPlayer) {
        this.monsterPlayer = monsterPlayer;
    }


    public ClientProxyManager getProxyManager() {
        return proxyManager;
    }

	public InfluenceGrid getInfluence() throws ApplicationError {
		if (influence == null) {
			Size galaxySize = stellarObjects.getSize();
			int sectorSize = stellarObjects.getSectorSize();
			influence = new InfluenceGrid(galaxySize.getX(), galaxySize.getY(), sectorSize, true);
		}
		return influence;
	}

	public void setInfluence(InfluenceGrid influence) {
		this.influence = influence;
	}

	public TutorialStatus getTutorial() {
		return tutorial;
	}

	public void setTutorial(TutorialStatus v) {
		tutorial = v;
	}

	public TradeRoutes getTradeRoutes() {
		if (tradeRoutes == null) tradeRoutes = new TradeRoutes();
		return tradeRoutes;
	}

	public void setTradeRoutes(TradeRoutes tradeRoutes) {
		this.tradeRoutes = tradeRoutes;
	}

}