/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.engine.event.BattleDestroyedEF;
import mot.engine.event.MotLinkedEventFactory;

/**
 * An Automaton is an object in the battle (whose life is usually fairly temporary (but does 
 * not have to be) is not under the control of a player. Examples are mines and missiles. They
 * are controlled by their own AI.
 */
abstract public class Automaton extends BattleObject implements MoveLinkedObject {

    /**
     * This is the percentage of the move of this automaton has to move this turn.
     * It is a value between 0.0 and 1.0.
     */
    private double moveRemaining;
    
    /**
     * This is the object this automaton is linked to if any.
     */
    private BattleObject link;

    public Automaton(Battle battle, String name, Location loc, BattleObject link) {
		super(battle, name, loc);
        
        this.link = link;
    }

    /**
     * Called at the end of every turn to allow the automaton to move. It moves the automaton
     * the remaining part of its move.
     * @throws mot.common.util.ApplicationError
     */
	public void move() throws ApplicationError {
        move(moveRemaining, null);
    }

    /**
     * This method moves the automaton percent of its move.
     * If this automaton is linked to another movable object then when the other object moves
     * this method is used to move the linked automaton by the same percentage of its move.
     * @param percent
     * @throws mot.common.util.ApplicationError
     */
    public void move(double percent, MotLinkedEventFactory rootEvent) throws ApplicationError {
        if (isDestroyed()) {
        	// Linked objects may be moved after they are destroyed by being destroyed by
        	// another automaton while moving - e.g. half way through moving the automatons.
        	return;
        }
        if (percent > moveRemaining) percent = moveRemaining;
        if (percent <= 0.0) return;
        moveRemaining -= percent;
        performMove(percent, rootEvent);
    }
    
    /**
     * This must be implemented in all automatons to actually perform the move.
     * @param percent The amount of the move to perform. This will be between 0.0 and
     * 1.0.
     * @throws mot.common.util.ApplicationError
     */
    protected abstract void performMove(double percent, MotLinkedEventFactory rootEvent) throws ApplicationError;
    
	/**
	 * Ends this turn on the battle and starts a new turn.
	 */
	public void startTurn() {
        moveRemaining = 1.0;
	}

	public void setMoveRemaining(double v) {
        moveRemaining = v;
	}

    /**
     * Notify this automaton that this object has been destroyed (or retreated). When ever 
     * an object is destroyed all automatons are notified giving each automaton the ability 
     * to destroy its self (or whatever its wants to do). However objects like missile can 
     * no longer target a craft if it is destroyed!
     * This default implementation does nothing - If an automaton want to do something 
     * override this method.
     * @param bo The object that has been deleted.
     * @throws mot.common.util.ApplicationError
     */
    public void notifyGoneFromBattle(BattleObject bo, BattleDestroyedEF destroyedEvent) throws ApplicationError {
        // nothing
    }

    public BattleObject getLink() {
        return link;
    }

    public void setLink(BattleObject link) {
        this.link = link;
    }
    
}