/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.Area;
import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.GameConstants;
import mot.common.bean.Location;
import mot.common.bean.NamedLocatedObject;
import mot.common.bean.SalvageList;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.event.StateContainer;
import mot.common.technology.MineField;
import mot.common.technology.MiscTechnologies;
import mot.common.technology.Scanners;
import mot.common.technology.Specials;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.vo.BattleCraftSpeedUpdateVO;
import mot.common.vo.BattleCraftSpeedVO;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.adapter.BattleCraftSpeedVOAdapter;
import mot.engine.adapter.BattleStateVOAdapter;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.battles.incumbants.AsteroidField;
import mot.engine.battles.incumbants.Incumbent;
import mot.engine.battles.incumbants.IncumbentDetailsFactory;
import mot.engine.battles.incumbants.IncumbentFactory;
import mot.engine.battles.incumbants.Incumbents;
import mot.engine.battles.incumbants.SpaceDistortion;
import mot.engine.battles.triggers.Triggers;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.ServerImageObject;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.crafts.Fleet;
import mot.engine.event.BattleEndEF;
import mot.engine.event.BattleNewTurnEF;
import mot.engine.event.BattleStateUpdateContainerEF;
import mot.engine.event.RoutedEF;
import mot.engine.event.StateUpdateEF;
import mot.engine.event.StatefulEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.Players;
import mot.engine.players.UserPlayer;
import mot.engine.relations.Relations;
import mot.engine.technology.WeaponInstance;
import mot.engine.technology.WeaponInstanceSalvageBeam;

/**
 * A battle occurs at a stellar object in a special battle window. It may contains
 * space stations, battle stations, a planet and many crafts. The players fight until
 * there are no crafts with any enemy still in the battle. Each craft moves in turn 
 * around the battle field moving to intercept enemies and fire upon them.
 * 
 * To use this class;
 * 1 - Create an instance
 * 2 - Add members by calling join().
 * 3 - Call start();
 * 4 - Call Craft c = getNextCraft(); c.getNextOrders() repeatedly or automate(). 
 * 	   If c is null move the planet.
 * 5 - Battle.endTurn() will be called when the last enemy object is destroyed BUT some events may still be queued.
 * 
 * At the end of a battle the loser of a battle is routed. A loser is defined as any 
 * player that has no crafts still in the battle at the end and who have an enemy who 
 * does have crafts remaining at the end.
 * 
 * 
 * BATTLE LAYOUT:
 * 
 *   100     200            400 < Mines < 800                   x=900
 *   
 *           craft3       500          700 to 850             craft
 *           craft1       sentinels    asteroids              craft
 *   planet  station                                          craft
 *           craft2                                           craft
 *           craft4
 * 
 */
public class Battle extends NamedLocatedObject {

	/**
	 * The turn of this battle
	 */
	private int turn = 1;

	/**
	 * A battle always takes place in a stellar object. This contains the stellar object 
	 * this battle is located at. It may also contain the planet this battle is located at
	 *  but note that a battle does not have to take place around a planet (so planet can 
	 *  be null). 
	 */
	private Orbit orbit;
	
	private boolean noShields = false;
	private boolean noCloak = false;

	/**
	 * The list of craft in this battle. This is a list of BattleCraft objects.
	 */
	private ArrayList<BattleCraft> craftList = new ArrayList();
	private transient BattleCraft [] crafts;

	/**
	 * If there is a planet also involved in the battle, this is the planet.
	 */
	private BattlePlanet planet;
    
    /**
     * This is a list of automatons currently in this battle. They will be moved at
     * the end of every turn. This contains a list of Automaton objects.
     */
    private ArrayList automatons = new ArrayList();
    
    /**
     * This is a list of players in the this battle. Every craft that joins the battle 
     * should have its owner added to this list. At the end of the game this list will
     * be used to determine which player lost and route them.
     */
    private Players startingPlayers = new Players();
    private Players startingUserPlayers = new Players();
    
    /**
     * This is a list of all crafts that joined the battle. When crafts are destroyed or
     * leave the battle they are not removed from this list. This list is created once
     * when the battle is started.
     */
    private Craft [] startingCrafts;
    
    /**
     * This is a list of crafts that have issued a wait command. A wait command informs the 
     * AI that the craft does want to more straight away and instead wished to wait until after
     * other craft of the planet that are of lower initiative.
     * This is an ArrayList of BattleCraft objects.
     */
    private ArrayList<BattleCraft> waitingCraft;
    
    /**
     * This is a list of battle crafts that have retreated from this battle.
     */
    private BattleCraftList retreatList;
    
    /**
     * This is set to true once the EndBattle event has been sent to make sure the event is 
     * never sent twice.
     */
    private boolean sentEndBattleEvent = false;
    
    /**
     * Maintains a list of all incumbents defined in the battle.
     */
    private Incumbents incumbents = new Incumbents();
    
    /**
     * Maintains a list of all crafts wreckage in the battle.
     */
    private Wreckages wreckages = new Wreckages();
    
    /**
     * Defines the list of triggers that exist in this battle field.
     */
    private Triggers triggers;
    
    /**
     * A Map from player to bonuses in this battle.
     * This map caches transient bonuses that may change during battle.
     */
    private HashMap<String, PlayerBattleBonuses> playerBonuses = new HashMap();
    
    private NextObjectToMoveCache nextObjectToMoveCache;
    
    /**
     * When a craft is added or removed, all crafts need their speed updated on the client.
     * This is because one craft can effect another crafts speed (command modules).
     */
    private boolean _allCraftsNeedSpeedUpdate = false;

    /**
     * IF this is a mining colony then this is the planet that owns this mining battle station.
     */
    private Planet mineColonyOwner;

    /**
     * If this battle has started (NewBattleEvent sent to clients).
     * If false clients do not know about this battle and the turn may not yet have ended!
     */
	private boolean started = false;
    
	private static String getNewName(String prefix) {
		if (MotEngine.instance == null) {
			return prefix+"DefaultName";
		} else {
			return MotEngine.instance.getNameGenerator().getNextName(prefix);
		}
	}
	
	public Battle(Orbit orbit) {
		super(getNewName("Battle"), orbit.getStellarObject().getLocation());
		this.orbit = orbit;
		if (isInNebula()) {
			noShields = true;
			noCloak = true;
		}
		nextObjectToMoveCache = new NextObjectToMoveCache(this);
	}

	public boolean isInNebula() {
		StellarObject stellarObject = orbit.getStellarObject();
		return stellarObject != null && stellarObject.isInNebula();
	}
    
    public String getDescription() {
        return getName() + " at "+getActualOrbit().getName()+" containing["+Comp.toString(getCrafts(), ", ")+"]";
    }

    public BattlePlanet join(Planet planet) throws ApplicationError {
        if (this.planet != null) {
            // Can only have one planet.
            return null;
        }
        
        if (planet.getOwner() == null) {
        	return null;
        }
    
		if (Logger.isDebug()) {
            Logger.logDebug("Battle.join() Location="+getActualOrbit().getName()+" planet="+planet.getName());
        }

        if (planet.getPopulation() > 0.0 || planet.hasWeapons()) {
    		this.planet = new BattlePlanet(planet, this);
        }
        
        if (planet.getOrbital() != null && !containsStation(planet.getOrbital())) {
            BattleStation bs = BattleStationFactory.createBattleStation(this, planet);
            craftList.add( 0, bs );
            crafts = null;
    		mineColonyOwner = MotEngine.instance.getPlanets().getOwnerOfMiningStation(planet.getOrbital());
        }
        StellarObjectOrbital outpost = planet.getStellarObject().getOrbital();
        if (outpost != null && !containsStation(outpost.getOrbital())) {
            BattleStation bs = BattleStationFactory.createStellarObjectStation(this, outpost);
            craftList.add( 0, bs );
            crafts = null;
        }
        
        Player planetOwner = planet.getOwner();
		if (!startingPlayers.contains(planetOwner)) {
            startingPlayers.add(planetOwner);
             if (planetOwner instanceof UserPlayer) startingUserPlayers.add(planetOwner);
        }
        
        if (planet.getHasAsteroids()) {
        	IncumbentFactory.createAsteroidField(this);
        }
        
        if (getQtyMines() == 0) {
	        MiscTechnologies miscTechs = planetOwner.getMiscTechnologies();
	        if (miscTechs != null) {
		        MineField mineField = miscTechs.getMineField();
		        if (mineField != null) {
		        	BattleMineFactory.create(planetOwner, this, mineField.getRange(), mineField.getQty());
		        }
	        }
        }
        
        return this.planet;
	}
    
	public BattleStation join(StellarObjectOrbital orbital) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Battle.join() Location="+getActualOrbit().getName()+" orbital="+orbital.getName());
        }

		BattleStation bs = null;
        if (orbital != null && !containsStation(orbital.getOrbital())) {
            bs = BattleStationFactory.createStellarObjectStation(this, orbital);
            craftList.add( 0, bs );
            crafts = null;
        }
        
        Player orbitalOwner = orbital.getOwner();
		if (!startingPlayers.contains(orbitalOwner)) {
            startingPlayers.add(orbitalOwner);
            if (orbitalOwner instanceof UserPlayer) startingUserPlayers.add(orbitalOwner);
        }
        
        if (getQtyMines() == 0 && orbitalOwner != null) {
	        MiscTechnologies miscTechs = orbitalOwner.getMiscTechnologies();
	        if (miscTechs != null) {
		        MineField mineField = miscTechs.getMineField();
		        if (mineField != null) {
		        	BattleMineFactory.create(orbitalOwner, this, mineField.getRange(), mineField.getQty());
		        }
	        }
        }

        return bs;
    }

    public BattleCraft join(Craft craft, boolean attacker) throws ApplicationError {
    	if (craft == null) {
    		Logger.logError("NULL craft joining", new Exception("Error"));
    		return null;
    	}
    	
        if (getCraft(craft.getName()) != null) {
            return null;
        }
        
		if (Logger.isDebug()) {
            Logger.logDebug("Battle.join() Location="+getActualOrbit().getName()+" craft="+craft.getName());
        }
        
        BattleCraft bc = new BattleCraft(this, craft, attacker);
		craftList.add( bc );
		crafts = null;
        
        Player craftOwner = craft.getOwner();
		if (!startingPlayers.contains(craftOwner)) {
            startingPlayers.add(craftOwner);
            if (craftOwner instanceof UserPlayer) startingUserPlayers.add(craftOwner);
        }
		
		if (started && turn > 1) {
			// Had a bug - When when a colony is lost crafts can be gifted then defend 
			// the planet by attacking and join a battle after it has started
			Logger.logWarning("Battle.join() Craft joining battle after start of battle "+craft);
			Location loc = getClosestFreeLocationInLine(GalaxyDefinitions.BATTLE_BOUNDARY.getCenterLocation(), bc);
			bc.setLocation(loc);
		}

        return bc;
	}
    
    public void add(BattleCraft bc) {
		craftList.add( bc );
		crafts = null;
    }

    public void join(Craft [] crafts, boolean attacker) throws ApplicationError {
		for (int i=0; i<crafts.length; i++) {
			join(crafts[i], attacker);
		}
	}

    public void join(Fleet fleet, boolean attacker) throws ApplicationError {
		Craft [] crafts = fleet.getCrafts();
		join(crafts, attacker);
	}
    
    protected void addToCraftList(BattleCraft craft) {
		craftList.add( craft );
		crafts = null;
    }
    
    /**
     * Creates a new craft in the battle (Battle Craft) using the craft design passed in.
     * The new Craft is not placed in the list of crafts owned by this player and is not
     * added to the MotEngine. This craft is intended to exist only for the life time of
     * this battle.
     * 
     * This method is intended for techs such as missile and Fighter craft launched from 
     * a mother craft in battle (Fighter Bays special).
     * 
     * The direction is set to point the craft at the closest enemy craft in the battle.
     * 
     * @param cd The craft design used to produce this craft.
     * @return The new BattleCraft created.
     * @param loc The location in the battle to create this new craft
     */
    public BattleCraft newCraft(Player owner, CraftDesign cd, Location loc) throws ApplicationError, ValidationError {
        Craft newCraft = CraftFactory.build(cd, owner, getOrbit());
        BattleCraft bc = join(newCraft, true);
        bc.setLocation(loc);
        BattleCraft enemyCraft = getClosestEnemyCraft(owner, loc, false);
        if (enemyCraft == null) {
            BattlePlanet bp = getPlanet();
            if (bp != null) {
                int reqdDirection = loc.getDirectionFor(bp.getLocation());
                bc.setDirection(reqdDirection);
            }
        } else {
    		int reqdDirection = loc.getDirectionFor(enemyCraft.getLocation());
            bc.setDirection(reqdDirection);
        }
		Specials specials = bc.getSpecials();
		if (specials != null && specials.hasBattleBonuses()) {
			updateBattleBonuses(bc.getOwner());
		}
		_allCraftsNeedSpeedUpdate = true;
        return bc;
    }

	public void _remove(BattleCraft craft){
		crafts = null;
		craftList.remove(craft);
		if (waitingCraft != null && waitingCraft.contains(craft)) {
			removeFromWaitList(craft);
		}
	}
	
    /**
     * Removes this craft from the battle.
     * This does not remove this craft from the game.
     * @throws ApplicationError 
     */
	public void remove(BattleCraft craft) throws ApplicationError {
		_remove(craft);
		Specials specials = craft.getSpecials();
		if (specials != null && specials.hasBattleBonuses()) {
			updateBattleBonuses(craft.getOwner());
		}
		_allCraftsNeedSpeedUpdate = true;
		if (started && hasEnded()) {
			endBattle();
		}
	}

    /**
     * Removes this planet from the battle.
     * This does not remove this planet from the game.
     * @throws ApplicationError 
     */
	public void remove(BattlePlanet planet) throws ApplicationError {
		planet = null;
		if (started && hasEnded()) {
			endBattle();
		}
	}

    /**
     * Removes this craft from the battle.
     */
	public void remove(Automaton a) {
		if (automatons != null) {
            automatons.remove(a);
        }
	}

    /**
     * Adds a new automaton to the battle.
     */
	public void add(Automaton a) {
		if (automatons == null) {
            automatons = new ArrayList();
        }
        automatons.add(a);
	}
    
    public void addToRetreatList(BattleCraft craft) {
        if (retreatList == null) retreatList = new BattleCraftList();
        retreatList.add(craft);
    }

    public boolean hasRetreated(BattleCraft craft) {
        if (retreatList == null) return false;
        return retreatList.contains(craft.getCraft());
    }

    public BattleCraft [] getRetreatedCraft() {
        if (retreatList == null) {
            return null;
        } else {
            return retreatList.getAllCraft();           
        }
    }

	public void setTurn(int v) { turn = v; }
	public void setCraftList(ArrayList v) { craftList = v; }
	public void setPlanet(BattlePlanet v) { planet = v; }

	public int getTurn() { return turn; }
	public ArrayList getCraftList() { return craftList; }
	public BattlePlanet getPlanet() { return planet; }
	public Incumbents getIncumbents() { return incumbents; }
	public Triggers getTriggers() { return triggers; }
	
	public Triggers getTriggersNotNull() {
		if (triggers == null) triggers = new Triggers();
		return triggers; 
	}

    public BattleStation getBattleStation() throws ApplicationError {
		BattleCraft[] ca = getCrafts();
		if (ca == null) return null;
		for (int i = 0; i < ca.length; i++) {
			if (ca[i] instanceof BattleStation) {
				return (BattleStation)ca[i];
			}
		}
		return null;
	}


	public void sendStateUpdateEvents(Player player, boolean updateCrafts, boolean updateAutomatons, boolean updateAnimatedBattleAreas) throws ApplicationError {
		BattleStateUpdateContainerEF event = new BattleStateUpdateContainerEF(getName());
		addStateUpdate(event, player, updateCrafts, updateAutomatons, updateAnimatedBattleAreas);
        MotEngine.instance.broadcast(event, player);
	}
	
	public void addStateUpdate(StateContainer event, Player player, boolean updateCrafts, boolean updateAutomatons, boolean updateAnimatedBattleAreas) throws ApplicationError {
		if (updateCrafts) {
			event.addStateUpdate(EngineVoFactory.getBattleCrafts(getCrafts(), player));
		}
		if (updateAutomatons) {
			event.addStateUpdate(EngineVoFactory.getAutomitons(getAutomatons(), player));
		}
		if (updateAnimatedBattleAreas) {
			event.addStateUpdate(getAnnimatedBattleAreas());
		}
		nextObjectToMoveCache.checkForUpdates(event);
	}
    
    /**
     * @return true if and only if this battle contains something belonging to this player.
     */
    public boolean contains(Player player) throws ApplicationError {
        if (planet != null && planet.getOwner() == player) {
            return true;
        }
        
        BattleCraft [] crafts = getCrafts();
        if (crafts == null) {
            return false;
        }
        
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getOwner() == player) {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * @return True if this battle contains an instance of a battle station created from orbital.  
     */
    public boolean containsStation(Orbital orbital) {
        BattleCraft [] crafts = getCrafts();
        if (crafts == null) {
            return false;
        }
        
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i] instanceof BattleStation) {
                BattleStation bs = (BattleStation) crafts[i];
                if (bs.getOrbital().getOrbital() == orbital) {
                    return true;
                }
            }
        }
        
        return false;
    }

    /**
     * @return true if and only if this battle contains automaton.
     */
    public boolean contains(Automaton automiton) {
        return (automatons != null && automatons.contains(automiton));
    }

    /**
     * @return true if and only if this battle contains this battle object.
     */
    //public boolean contains(BattleObject bo) {
    //    if (bo instanceof BattleCraft) {
    //        what about battle station?
    //        return contains(((BattleCraft)bo).getCraft());
    //    } else if (bo instanceof BattlePlanet) {
    //        return (bo == planet);
    //    } else if (bo instanceof Automiton) {
    //        return (automitons != null && automitons.contains(bo));
    //    }
    //    return false;
    //}

    /**
     * @return true if and only if this battle contains something belonging to this player.
     */
    public boolean contains(Craft craft) {
        BattleCraft [] crafts = getCrafts();
        if (crafts == null) {
            return false;
        }
        
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getCraft() == craft) {
                return true;
            }
        }
        
        return false;
    }

    /**
     * @return true if and only if this battle contains something belonging to this player.
     */
    public boolean contains(MoveableOwnedObject moo) {
        if (moo instanceof Craft) {
            return contains((Craft)moo);
        } else if (moo instanceof Fleet) {
            Craft [] crafts = ((Fleet)moo).getCrafts();
            if (crafts != null) {
                for (int i=0; i<crafts.length; i++) {
                    if (contains(crafts[i])) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    /**
     * Returns true if and only if this battle contains something belonging to a human 
     * player - At this point in time.
     */
    public boolean hasHumanPlayer() throws ApplicationError {
        if (planet != null && planet.getOwner() != null && !planet.getOwner().isComputerPlayer()) {
            return true;
        }
        
        BattleCraft [] crafts = getCrafts();
        if (crafts == null) {
            return false;
        }
        
        for (int i=0; i<crafts.length; i++) {
            if (!crafts[i].getOwner().isComputerPlayer()) {
                return true;
            }
        }
        
        return false;
    }

	public int getQtyUserPlayers() throws ApplicationError {
		int qtyUserPlayers = 0;
		Player[] players = getPlayers();
		if (players != null) {
			for (int i = 0; i < players.length; i++) {
				if (players[i] instanceof UserPlayer) {
					qtyUserPlayers++;
				}
			}
		}
		return qtyUserPlayers;
	}

	public UserPlayer [] getUserPlayers() throws ApplicationError {
		ArrayList<UserPlayer> r = null;
		Player[] players = getPlayers();
		if (players != null) {
			for (int i = 0; i < players.length; i++) {
				if (players[i] instanceof UserPlayer) {
					if (r == null) r = new ArrayList();
					r.add((UserPlayer)players[i]);
				}
			}
		}
		if (r == null) {
			return null;
		} else {			
			return r.toArray(new UserPlayer[r.size()]);
		}
	}

    /**
     * @return A array of all the player names currently in this battle. This value will change over time.
     */
    public Player [] getPlayers() throws ApplicationError {
        ArrayList list = new ArrayList();
        // Planet colony can be destroyed in battle (so we have a planet with no owner).
        if (planet != null && planet.getOwner() != null) {
            list.add(planet.getOwner());
        }
        
        BattleCraft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                Player p = crafts[i].getOwner();
                if (!list.contains(p)) {
                    list.add(p);
                }
            }
        }
        
        return (Player []) list.toArray(new Player[list.size()]);
    }


    /**
     * Returns the list of players who joined this battle. NOTE - Some of them may have 
     * been destroyed several turns before.
     */
    public Player [] getStartingPlayers() {
        return startingPlayers.getPlayers();
    }

    public Player [] getStartingUserPlayers() {
        return startingUserPlayers.getPlayers();
    }
    
	public void _addUserPlayerForTestHarness(ComputerPlayer player) {
		if (!startingUserPlayers.contains(player)) {
			startingUserPlayers.add(player);
		}
	}


    /**
     * @return A array of all the player currently in this battle that can take an active part in it.
     * This is players with a planet that has weapons or a craft that is operational. 
     * NB - A planet with defences does not make a player active because he can not take any actions. A planet MUST have
     * weapons to make a player active.
     */
    public Players getActivePlayers() throws ApplicationError {
    	Players list = new Players();
        if (planet != null && (planet.hasWeapons() || planet.hasFightersToLaunch())) {
            list.add(planet.getOwner());
        }
        
        BattleCraft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                Player p = crafts[i].getOwner();
                if (!list.contains(p) && crafts[i].isOperating()) {
                    list.add(p);
                }
            }
        }
        
        return list;
    }


    /**
     * Returns true if this player joined this battle. NOTE - Some of them may have 
     * been destroyed several turns before.
     */
    public boolean containsStartingPlayer(Player p) {
        return (startingPlayers != null) && startingPlayers.contains(p);
    }

    /**
     * Returns the list of players who joined this battle. NOTE - Some of them may have 
     * been destroyed several turns before.
     */
    public Craft [] getStartingCrafts() {
        return startingCrafts;
    }

    /**
     * @return true if this player joined this battle. NOTE - Some of them may have 
     * been destroyed several turns before.
     */
    public boolean containsStartingCraft(Craft c) {
        if (startingCrafts == null) return false;
        for (int i=0; i < startingCrafts.length; i++) {
            if (startingCrafts[i] == c) {
                return true;
            }
        }
        return false;
    }

	/**
	 * A battle always takes place in a stellar object. This returns the stellar object for
	 * this battle.
	 */
    public Orbit getOrbit() {
        return orbit;
    }

	/**
	 * A battle always takes place in a stellar object. This returns the stellar object for
	 * this battle.
	 */
    public StellarObject getActualOrbit() {
        getOrbit();
        if (orbit != null) {
	        return orbit.getStellarObject();
        } else {
	        return null;
        }
    }

    public AsteroidField getAsteroidField() {
    	return (AsteroidField) getIncumbents().getType(AsteroidField.class);
    }

    public SpaceDistortion [] getSpaceDistortions() {
    	return getIncumbents().getSpaceDistortions();
    }

    /**
     * Examples of these types of animations are special distortions, asteroid fields and wreckage.
     * Not to be confused with event animations that exist only on the client.
     * @return A list of areas on the battle map that contain animated background objects. 
     */
    public AnimatedBattleAreaDetails [] getAnnimatedBattleAreas() throws ApplicationError {
    	ArrayList<AnimatedBattleAreaDetails> r = getIncumbents().getAllIncumbentDetails();
    	if (wreckages != null) {
    		r = wreckages.add(r);
    	}
    	if (r == null) {
        	return null;
    	} else {
    		return (AnimatedBattleAreaDetails []) r.toArray(new AnimatedBattleAreaDetails[r.size()]);
    	}
    }
    
	public AnimatedBattleAreaDetails getAnnimatedBattleAreaAt(int x, int y) throws ApplicationError {
		Location loc = new Location(x,y);
		Wreckage wreckage = wreckages.getWreckageAt(loc);
		if (wreckage != null) {
			return WreckageDetailsFactory.create(wreckage);
		}
		Incumbent[] incumbents = getIncumbents().getIncumbents(loc);
		if (incumbents != null && incumbents.length > 0) {
			return IncumbentDetailsFactory.create(incumbents[0]);
		}
		return null;
	}

    /**
     * @return an array of all craft contained in the battle. Craft that have been destroyed 
     * in this battle are not included.
     */
  	public BattleCraft [] getCrafts() {
        if (this.crafts != null) {
          return crafts;
        }
    
        if (craftList == null) {
          return null;
        }
    
        BattleCraft [] crafts = new BattleCraft[ craftList.size() ];
        for (int i=0; i<craftList.size(); i++) {
          crafts[i] = (BattleCraft) craftList.get(i);
        }
            this.crafts = crafts;
            
        return crafts;
    }

  	public BattleObject [] getBattleObjects() {
        ArrayList<BattleObject> r = null;
    
        BattleCraft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	if (r == null) r = new ArrayList<BattleObject>();
            	r.add(crafts[i]);
            }
        }
        
        if (planet != null) {
        	if (r == null) r = new ArrayList<BattleObject>();
        	r.add(planet);
        }
        
        if (automatons != null && automatons.size() > 0) {
        	for (int i = 0; i < automatons.size(); i++) {
            	if (r == null) r = new ArrayList<BattleObject>();
            	Automaton automaton = (Automaton) automatons.get(i);
            	r.add(automaton);
			}
        }
    
        if (r != null) {
        	return (BattleObject[]) r.toArray(new BattleObject[r.size()]);
        } else {
        	return null;
        }
    }

    /**
     * @return an array of all craft contained in the battle that are owned by the player 
     * passed in.
     */
  	public BattleCraft [] getCrafts(Player player) throws ApplicationError {
        BattleCraft [] crafts = getCrafts();
        if (crafts == null) {
            return null;
        }
    
        ArrayList r = null;
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].isOwnedBy(player)) {
                if (r == null) r = new ArrayList();
                r.add(crafts[i]);
            }
        }
        
        if (r == null) {
            return null;
        } else {
            return (BattleCraft []) r.toArray(new BattleCraft[r.size()]);
        }
    }

    /**
     * @return True if player contains crafts in the battle.
     */
  	public boolean containsCrafts(Player player, boolean mustBeOperating) throws ApplicationError {
        BattleCraft [] crafts = getCrafts();
        if (crafts == null) {
            return false;
        }
    
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].isOwnedBy(player)) {
                if (mustBeOperating) {
                    if (crafts[i].isOperating()) return true;
                } else {
                    return true;
                }
            }
        }
        
        return false;
    }

    /**
     * @return the craft with name contained in the battle nor null.
     */
    public BattleCraft getCraft(String name) {
        BattleCraft [] crafts = getCrafts();
        if (crafts == null) return null;
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getCraft().sameName(name)) {
                return crafts[i];
            }
        }
        return null;
    }

    public Automaton getAutomiton(String name) {
        Automaton [] autos = getAutomatons();
        if (autos == null) return null;
        for (int i=0; i<autos.length; i++) {
            if (autos[i].sameName(name)) {
                return autos[i];
            }
        }
        return null;
    }

    public Automaton getAutomatonAt(int x, int y) throws ApplicationError {
        if (automatons != null && automatons.size() > 0) {
            for (int i=0; i<automatons.size(); i++) {
                Automaton automaton = (Automaton) automatons.get(i);
                if (automaton.isInside(x,y)) {
                    return automaton;
                }
            }
        }
        return null;
    }

    public MovingAutamaton getMovingAutomatonTargettingMeAt(Player player, int x, int y) throws ApplicationError {
        if (automatons != null && automatons.size() > 0) {
            for (int i=0; i<automatons.size(); i++) {
                Automaton automaton = (Automaton) automatons.get(i);
                if (automaton instanceof MovingAutamaton) {
                	MovingAutamaton ma = (MovingAutamaton) automaton;
                	if (ma.isInside(x,y) && ma.isTargetingPlayer(player)) {
                		return ma;
                	}
                }
            }
        }
        return null;
    }


    public int getQtyMines() {
        Automaton [] autos = getAutomatons();
        if (autos == null) return 0;
        int r = 0;
        for (int i=0; i<autos.length; i++) {
            if (autos[i] instanceof BattleMine) {
                r++;
            }
        }
        return r;
	}



    /**
     * Returns the object located at position x,y in the battle (or null if one
     * does not exist). The point x,y must be within the rectangle on the returned
     * object's image.
     * The objects returns the first object found in this order at this location
     * 	1 - Automaton (e.g. missile)
     *  2 - Craft
     *  3 - Planet
     */
    public ServerImageObject getObjectAt(int x, int y) throws ApplicationError {
        if (automatons != null && automatons.size() > 0) {
            for (int i=0; i<automatons.size(); i++) {
                Automaton automaton = (Automaton) automatons.get(i);
                if (automaton.isInside(x,y)) {
                    return automaton;
                }
            }
        }
        BattleCraft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (crafts[i].isInsideArea(x,y)) {
                    return crafts[i];
                }
            }
        }
        if (planet != null && planet.isInsideArea(x,y)) {
            return planet;
        }
        return null;
    }

    /**
     * Returns the object located at position x,y in the battle (or null if one
     * does not exist). The point x,y must be within the rectangle on the returned
     * object's image.
     * There should only be one thing at this location a Craft or a Planet
     */
    public BattleObject getBattleObjectAt(int x, int y) throws ApplicationError {
        BattleCraft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (crafts[i].isInsideArea(x,y)) {
                    return crafts[i];
                }
            }
        }
        if (planet != null && planet.isInsideArea(x,y)) {
            return planet;
        }
        return null;
    }

    public BattleObject getBattleObject(String name) {
        BattleCraft craft = getCraft(name);
        if (craft != null) {
            return craft;
        }
        if (Comp.same(name, getPlanet())) {
            return getPlanet();
        }
        Automaton auto = getAutomiton(name);
        if (auto != null) {
            return auto;
        }
        return null;
    }
    
    /**
     * The player is assumed to be a winner if 
     *  1 - He still has crafts in the battle that are not disabled.
     *  OR
     *  2 - The planet belong to player and has defences.
     *  OR
     *  3 - All enemy combatants were destroyed (planet and crafts) and
     *      this player has at least one craft that was not destroyed.
     * NOTE - A battle may have multiple winners and losers.
     * @param player The player who may have been a winner
     * @return True if the player passed in is a winner.
     */
    public boolean isWinner(Player player) throws ApplicationError {
        if (containsCrafts(player, true)) {
            return true;
        }
		if (planet != null && planet.getOwner() == player && planet.isColonyWithDefenses()) {
            return true;
        }
        if (planet != null && planet.getOwner() != null && planet.getOwner().isEnemy(player) && planet.isColonyWithDefenses()) {
            return false;
        }
        Craft [] startingCrafts = getStartingCrafts();
        if (startingCrafts == null) return true;
        int qtySurvivingEnemy = 0;
        int qtySurvivingPlayer = 0;
        for (int i=0; i<startingCrafts.length; i++) {
            Craft c = startingCrafts[i];
            if (!c.isDestroyed()) {
            	Player owner = c.getOwner();
				if (owner == player) {
	            	qtySurvivingPlayer++;
	            } else  if (owner.isEnemy(player)) {
	            	qtySurvivingEnemy++;
	            }
            }
        }
        return (qtySurvivingPlayer > 0) && (qtySurvivingEnemy == 0);
    }
    
    /**
     * @return True if craft is one of the crafts that have retreated from this battle.
     */
    public boolean hasRetreated(Craft craft) {
        if (retreatList == null) return false;
        return retreatList.contains(craft);
    }
    
	/**
	 * Start the battle.
	 * This involves, setting the start positions of all craft, all craft directions and
	 * the the position of the planet if any.
	 */
	public void start() throws ApplicationError {
        if (Logger.isDebug()) {
			Logger.logDebug("Battle.start() "+getName()+" *************************************************************************");
        }
        

        //Logger.logError("Battle.start() Start "+getName(), new Exception());
        started = true;

        // Must rebuild the planet weapons just in case a project has completed since the planet joined the battle
        if (planet != null) {
        	Planet p = planet.getPlanet();
            planet.setShields(p.getShields(), true);
            planet.setWeapons(p.getWeapons(), true, true, null);
            planet.setSpecials(p);
        }
        
        setup();
        
    }
    
	/**
	 * Layout all the craft on the battle field and then calls startTurn() so everything
     * is waiting to run. Creates any holo craft that may be generated by crafts holo emiters.
     * The next task should be getNextCraft().
	 */
	public void setup() throws ApplicationError {
        BattleLayoutManager blm = new LinedBattleLayoutManager(this);

        BattleCraft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                BattleCraft craft = crafts[i];
                if (craft.hasHoloEmiters()) {
                    double chanceToSucceed = getChanceToDeployHoloCraft(craft.getOwner());
                    if (Utils.random(chanceToSucceed)) {
                        notifyEnemiesHoloCraftTrickWorked(craft.getOwner());
                        HoloCraftFactory.create(this, craft, false);
                    }
                }
            }
        }
        
		if (planet != null) {
			blm.setPlanetLocation(planet);
		}

		crafts = getCrafts();
        startingCrafts = new Craft [crafts.length];
		for (int i=0; i<crafts.length; i++) {
			blm.setNextFreeLocation(crafts[i]);
			crafts[i].startBattle();
            startingCrafts[i] = crafts[i].getCraft();
		}
        
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                BattleCraft craft = crafts[i];
                craft.setup();
            }
        }

        if (planet != null) {
        	planet.setup();
		}
        
        BattleStation bs = getBattleStation();
        if (bs != null) {
        	int qtySentinels = bs.getQtySentinals();
        	if (qtySentinels > 0) {
        		BattleSentinelFactory.createSentinels(this, bs.getOwner(), qtySentinels);
                crafts = null;        		
        	}
        }

        startTurn();
	}
    
	/**
     * @return A value between 0 and 1 which is the probability that player p will
     * fool his enemy players into believing his holocraft are real crafts.
     * @param p The player trying to trick his enemies using holo crafts.
     */
    public double getChanceToDeployHoloCraft(Player p) throws ApplicationError {
        Player [] players = getPlayers();
        if (players == null) return 0.0;
        Player enemy = null;
        for (int i=0; i<players.length; i++) {
            if (players[i] != p && players[i].isEnemy(p)) {
                if (enemy == null || players[i].getChanceHoloCraftWork() < enemy.getChanceHoloCraftWork()) {
                    enemy = players[i];
                }
            }
        }
        if (enemy == null) return 0.0;
        return enemy.getChanceHoloCraftWork();
    }

    /**
     * Notifies all enemies of player p that they have fallen for the holocraft decoy.
     * Allows players to update the chance they will fall for this trick in the future.
     */
    public void notifyEnemiesHoloCraftTrickWorked(Player p) throws ApplicationError {
        Player [] players = getPlayers();
        if (players == null) return;
        for (int i=0; i<players.length; i++) {
            if (players[i] != p && players[i].isEnemy(p)) {
                players[i].notifyHoloCraftTrickWorked();
            }
        }
    }
    
    /**
     * Influence in a battle is generated by broadcasting a news service and it
     * effects the morale of crafts and planets (population / crew).
     * @param p The player to return the influence for
     * @return A value between -1.0 and +1.0 that represents the influence in this 
     * battle the player passed in is generating.
     */
    public double getInfluence(Player p) throws ApplicationError {
    	double r = 0.0;
    	if (planet != null && planet.isOwnedBy(p)) {
	    	MiscTechnologies mts = p.getMiscTechnologies();
	    	if (mts != null) {
	    		r += mts.getInfluence();
	    	}
    	}
        BattleCraft [] crafts = getCrafts();
        for (int i=0; i<crafts.length; i++) {
        	if (crafts[i].isOwnedBy(p)) {
            	Specials specials = crafts[i].getSpecials();
            	if (specials != null) {
                	r += specials.getInfluence();
            	}
        	}
        }
        if (r < -1.0) r = -1.0;
        if (r > 1.0) r = 1.0;
        return r;
    }
    
    /**
     * Destroys all crafts in this battle that are holograms and that are projected from
     * holo emiters placed in the craft parent.
     * @param parent The craft containing holo emiters for which we will destroy all projections.
     */
    public void destroyHoloCraftFor(BattleCraft parent) throws ApplicationError {
        BattleCraft [] crafts = getCrafts();
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i] instanceof HoloCraft) {
                HoloCraft hc = (HoloCraft) crafts[i];
                if (hc.getParent() == parent.getCraft()) {
                    hc.destroy(null, hc.getCraftState(), null);
                }
            }
        }
    }
    
    /**
     * Causes all crafts belonging to this player to retreat from the battle.
     * @param player The player whose crafts are going to retreat.
     */
    public void retreat(Player player) throws ApplicationError {
        BattleCraft [] crafts = getCrafts();
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getOwner() == player) {
                crafts[i].retreat();
            }
        }
    }
    
    /**
     * Automates this battle.
     * All players must be a computer player 
     */
    public void automate() throws ApplicationError {
        try {
			while(!hasEnded()) {
			    BattlePrimaryObject craft = getNextToMove(false);
			    if (!hasEnded()) {
			        if (craft != null) {
			            craft.getNextOrders();
			        }
			    }
			}
		} catch (Exception e) {
			// If there is more than one battle queued then I do not want all of them to be ignored!
			Logger.logError("Battle.automate() Failed: "+toString()+" failed ", e);
		}
        endTurn();
    }

	/**
	 * Ends this turn on the battle and starts a new turn.
     * If there are any automatons they are moved before the turn is ended.
	 */
	public void endTurn() throws ApplicationError {
        if (automatons != null && automatons.size() > 0) {
            Automaton [] list = getAutomatons();
            for (int i=0; i<list.length; i++) {
                list[i].move();
            }
        }
    
        if (Logger.isDebug()) {
			Logger.logDebug("Battle.endTurn() turn="+turn);
        }
		turn++;
        
        if (hasEnded()) {
            endBattle();
        } else {
        	if (triggers != null) {
        		triggers.execute();
        	}
            startTurn();
            MotEngine.instance.multicast( new BattleNewTurnEF(this), getStartingUserPlayers());
        }
        
    }
    
	/**
	 * Starts a new turn in the battle. It calls start turn for each craft in the
	 * battle as well as the planet (if one exists) and all automatons. Starting
	 * a new turn on a craft causes all weapons to be recharged and move remaining
	 * to be set to 1.0.
	 */
	public void startTurn() throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		for (int i=0; i<crafts.length; i++) {
			crafts[i].startTurn();
		}
        if (planet != null) {
            planet.startTurn();
        }
        if (automatons != null && automatons.size() > 0) {
            Automaton [] list = getAutomatons();
            for (int i=0; i<list.length; i++) {
                list[i].startTurn();
            }
        }
	}

	public void removeCraftFromQueuedBattle(Craft craft) throws ApplicationError {
		if (Logger.isDebug()) {
			Logger.logDebug("Battle.removeCraftFromQueuedBattle() craft="+craft);
		}
		if (craft != null) {
			BattleCraft bc = getCraft(craft.getName());
			if (bc != null) {
				crafts = null;
				craftList.remove(bc);
			}			
		}
	}

	public void removePlayerFromQueuedBattle(Player player) throws ApplicationError {
		if (Logger.isDebug()) {
			Logger.logDebug("Battle.removePlayerFromQueuedBattle() player="+player);
		}
		BattleCraft[] bca = getCrafts();
		if (bca != null) {
			for (int i = 0; i < bca.length; i++) {
				if (bca[i].isOwnedBy(player)) {
					crafts = null;
					craftList.remove(bca[i]);
				}
			}
		}
		if (planet != null && planet.isOwnedBy(player)) {
			planet = null;
		}
		startingPlayers.remove(player);
		if (player instanceof UserPlayer) {
			startingUserPlayers.remove(player);
		}
	}

    /**
     * @return true if any of the battle end conditions have been met.
     * The conditions are defined as
     *  - Maximum quantity of turns has been exceeded.
     *  - There are no enemies left to fight each other in the battle.
     */
    public boolean hasEnded() throws ApplicationError {
        BattleCraft [] crafts = getCrafts();
        
        if (crafts == null || crafts.length == 0) {
            return true;
        }
        
        if (turn >= GameConstants.MAX_BATTLE_TURNS) {
            return true;
        }

//        if (planet != null) {
//            if (hasEnemyCraft(planet.getOwner(), true)) {
//                return false;
//            }
//        }
        
        Players players = getActivePlayers();
		if (players == null || players.getQuantity() < 2) {
        	return true;
        }
        
        if (players.hasEnemies()) {
            return false;
        }
        
        return true;
    }
    
    /**
     * This routes all crafts in the battle that have retreated from the battle for
     * any player that lost.
     * Any crafts that have been rendered inoperable (have no crew) at the end of 
     * the battle surrendered to the closest craft at the end of the battle.
     * This should be called when a battle has ended.
     */
    public void endBattle() throws ApplicationError {
        if (sentEndBattleEvent) {
        	return;
        }
        sentEndBattleEvent = true;

        if (Logger.isDebug()) {
            Logger.logDebug("Battle.endBattle() "+getName());
        }
        
        BattleCraft[] wreckageArray = wreckages.getWreckageCraft();
        Player[] pa = getStartingPlayers();
        BattleEndEF [] endBattleEvents = new BattleEndEF[pa.length];
        for (int i = 0; i < pa.length; i++) {			
        	endBattleEvents[i] = new BattleEndEF(this, wreckageArray);
        	// Must not broadcast queue as battle ends as a result of weapons fire and they have not finished yet.
        	MotEngine.instance.getProxyManager().scheduleBroadcast(endBattleEvents[i], pa[i]);
		}
        
        PlayerSalvageMap playerToSalageMap = new PlayerSalvageMap();
        
        BattleCraft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (!crafts[i].isOperating() && !isWinner(crafts[i].getOwner())) {
                    BattleCraft closestEnemy = getClosestEnemyCraft(crafts[i].getOwner(), crafts[i].getLocation(), true);
                    if (closestEnemy != null) {
                        crafts[i].surrender(closestEnemy.getOwner(), true);
                    }
                }
                if (crafts[i].isOperating() && isWinner(crafts[i].getOwner())) {                	
                	WeaponInstance sb = crafts[i].getWeaponSalvageBeam();
                	if (sb != null && sb instanceof WeaponInstanceSalvageBeam) {
                		WeaponInstanceSalvageBeam salvageBeam = (WeaponInstanceSalvageBeam) crafts[i].getWeaponSalvageBeam();
                		int qtySalvageToCollect = salvageBeam.getQtySalvageCanCollect();
                		SalvageList salvage = getAndRemoveBattleSalvage(qtySalvageToCollect);
                    	Player owner = crafts[i].getOwner();
                    	if (salvage != null) playerToSalageMap.add(owner, salvage);
                	}
                }
            }
        }
        
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	Player owner = crafts[i].getOwner();
            	SalvageList salvage = crafts[i].getSalvage();
            	if (salvage != null) playerToSalageMap.add(owner, salvage);
            }
        }
        
        BattleStation battleStation = getBattleStation();
        if (wreckages != null && (planet != null || battleStation != null)) {
        	Player salvageOwner = null;
        	if (planet != null) {
				salvageOwner = planet.getOwner();
        	}
        	if (salvageOwner == null && battleStation != null) {
				salvageOwner = battleStation.getOwner();
        	}
        	if (salvageOwner != null) {
	            Wreckage[] wa = wreckages.getWrecks();
	            if (wa != null) {
	            	for (int i = 0; i < wa.length; i++) {
	            		if ((wa[i] instanceof WreckageCraft)) {
	            			WreckageCraft craftWreckage = (WreckageCraft) wa[i];
	            			SalvageList salvage = craftWreckage.getSalvage();
	            			if (salvage != null) {
	            				playerToSalageMap.add(salvageOwner, salvage);
	            			}
	            		}
	            	}
	            }
        	}
        }
        
        boolean foundTechnology = false;
        Player[] playersWithSalvage = playerToSalageMap.getPlayersWithSalavge();
        if (playersWithSalvage != null) {
        	for (int i = 0; i < playersWithSalvage.length; i++) {
        		SalvageList salvage = playerToSalageMap.getSalvage(playersWithSalvage[i]);
            	playersWithSalvage[i].addSalvage(salvage);
			}
        }
        
        BattleCraft [] retreaters = getRetreatedCraft();
        if (retreaters != null) {
            HashMap<Player,RoutedEF> routedList = new HashMap();
            for (int i=0; i<retreaters.length; i++) {
                Player owner = retreaters[i].getOwner();
                if (!isWinner(owner) && hasEnemyCraft(owner, true)) {
                	if (retreaters[i].getRetreatTurn() != null && retreaters[i].getRetreatTurn() >= turn) {
                		retreaters[i]._destroy();                    
                		RoutedEF routedEF = routedList.get(owner);
                		if (routedEF == null) {
                			routedEF = new RoutedEF(owner, getOrbit());
                			routedList.put(owner, routedEF);
                		}
                		routedEF.addCraftName(retreaters[i].getName());
                	}
                } else {
                	owner.addSalvage(retreaters[i].getSalvage());
                }
            }
            for (Player p : routedList.keySet()) {
            	RoutedEF routedEF = routedList.get(p);
            	if (routedEF != null) {
        			MotEngine.getInstance().broadcast(routedEF, p);
            	}
            }
        }
        
        Player [] startingPlayers = getStartingPlayers();
        for (int i=0; i<startingPlayers.length; i++) {
        	boolean isWinner = isWinner(startingPlayers[i]);
        	if (isWinner && !(startingPlayers[i] instanceof MonsterPlayer)) {
        		orbit.getStellarObject().addVisitor(startingPlayers[i], isWinner);
        	}
        }

        for (int i = 0; i < pa.length; i++) {			
        	if (pa[i] instanceof UserPlayer) {
        		addEndBattleStateUpdate(endBattleEvents[i], (UserPlayer)pa[i], foundTechnology);
        	}
		}
        
        // Must update state on all user players NOT IN THE BATTLE
        // Otherwise they will see only craft / mining trails etc that do not exist!
        Players players = MotEngine.instance.getPlayers();
        if (players != null) {
        	UserPlayer[] upa = players.getUserPlayers();
        	for (int i = 0; i < upa.length; i++) {				
        		if (!Comp.in(upa[i], pa)) {
        			StateUpdateEF stateUpdateEvent = new StateUpdateEF();
            		addEndBattleStateUpdate(stateUpdateEvent, upa[i], false);
            		MotEngine.instance.broadcast(stateUpdateEvent, upa[i]);
        		}
			}
        }

        MotEngine.instance.getBattles().end(this);
    }
    
    private void addEndBattleStateUpdate(StatefulEventFactory event, UserPlayer up, boolean foundTechnology) throws ApplicationError {
    	event.addStateUpdate(new BattleStateVOAdapter(this));
    	StellarObject actualOrbit = getActualOrbit();
    	// Update all planets at this stellar object because blockades may end
		if (actualOrbit != null) {
			event.addStateUpdate(EngineVoFactory.getStellarObject(actualOrbit, up));
			if (actualOrbit instanceof SolarSystem) {
				SolarSystem ss = (SolarSystem) actualOrbit;
				Planet[] planets = ss.getPlanets();
				if (planets != null) {
					for (int i = 0; i < planets.length; i++) {						
						event.addStateUpdate(new PlanetVOAdapter(planets[i], up));
					}
				}
			}
		}
		if (mineColonyOwner != null) {
			event.addStateUpdate(new PlanetVOAdapter(mineColonyOwner, up));
		}
		MotEngine.instance.addStateUpdate(event, up, false, false, foundTechnology, false, true, false, true, false, foundTechnology, false, false);
    }
    
    public SalvageList getAndRemoveBattleSalvage(int totalSalvageCanCollect){
        Wreckage[] wa = wreckages.getWrecks();
        if (wa != null && totalSalvageCanCollect > 0) {
        	SalvageList r = new SalvageList();
        	for (int i = 0; i < wa.length && totalSalvageCanCollect > 0; i++) {
        		wreckages.remove(wa[i]);
                if ((wa[i] instanceof WreckageCraft)) {
                    WreckageCraft craftWreckage = (WreckageCraft) wa[i];
                    SalvageList salvage = craftWreckage.getSalvage();
                    if (salvage != null) {
                    	int qty = salvage.addSalvageTo(totalSalvageCanCollect, r);
                    	totalSalvageCanCollect -= qty;
                    }
                    if (totalSalvageCanCollect <= 0) {
                    	return r;
                    }
                }
        	}
        }
        return null;
    }
	/**
     * Craft objects are moved in initiative order during a battle. Once all craft in this 
     * battle turn have been moved then the planet is moved (if any) and a new turn is 
     * started.
     * This method returns the craft with the highest priority that is not waiting. If a planet 
     * exists and is owned by a UserPlayer then the planet is returned because it is the next 
     * object to move!
	 * If the next craft or plant to move is a computer controlled craft, then, if moveComputerPlayer
	 * is true it is moved and then this method tries again with the next craft etc. If moveComputerPlayer
	 * is false it will return the craft or planet that is to move next, regardless of ownership.
     * 
     * When a user client interface wishes to fetch the next craft to move in a battle it should 
     * set the parameter moveComputerPlayer to true. This will then move computer craft prior to 
     * returning. All the usually move events will be returned to the client so the necessary animations
     * occur.
     * @return The next object (craft/planet) to move in this battle.
     * @param moveComputerPlayer If this is true and the next craft to move is a computer players
     * craft then the craft is moved before the method returns.
     * @throws mot.common.util.ApplicationError
     */
	public BattlePrimaryObject getNextToMove(boolean moveComputerPlayer) throws ApplicationError {
		BattlePrimaryObject best = _getNextToMove();

		if (best == null) {
			return null;
		} else if (moveComputerPlayer && best.controlIsAutomated()) {
	        while(!hasEnded()) {
	        	best.getNextOrders();
	    		best = _getNextToMove();
	    		if (best == null) {
	    	        endBattle();
	    			return null;
	    		} else if (!best.controlIsAutomated()) {
	    			return best;
	    		}
	        }
	        endBattle();
			return null;
		} else {
			return best;
		}

	}
    
	public BattlePrimaryObject _getNextToMove() throws ApplicationError {
		BattleCraft best = getCraftWithHighestPriority();

		if (best == null) {
	        // All the crafts have moved, the planet is about to move so this is the 
	        // end of the wait list.
			if (waitingCraft != null && waitingCraft.size() > 0) {
				best = getFirstWaitingCraft();
				if (best != null) {
					return best;
				}
	        } else if (planet != null && planet.getOwner() != null && (planet.hasWeaponsToFire() || planet.hasFightersToLaunch())) {
                return planet;                
	        }
	        endTurn();
    		best = getCraftWithHighestPriority();
    		if (best == null) {
    	        if (planet != null && planet.getOwner() != null && (planet.hasWeaponsToFire() || planet.hasFightersToLaunch())) {
    	        	return planet;
    	        }
    		}
    		return best;
		} else {
			return best;
		}
		
	}

    /**
     * @return The craft with the highest initiative that has not been moved (see BattleCraft.hasMoved()
     * for details).
     */
    public BattleCraft getCraftWithHighestPriority() throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		BattleCraft best = null;
		int bestInitiative = -1;

		for (int i=0; i<crafts.length; i++) {
			if (higherInitiative(crafts[i], bestInitiative, best) && !crafts[i].hasMoved()) {
                if (waitingCraft == null || !waitingCraft.contains(crafts[i])) {
                    best = crafts[i];
                    bestInitiative = crafts[i].getInitiative();
                }
			}
		}
        
        return best;
    }
    
    private boolean higherInitiative(BattleCraft candidate, int bestInitiative, BattleCraft best) throws ApplicationError {
    	int initiative = candidate.getInitiative();
		if (initiative == bestInitiative) {
			return (candidate.getName().compareTo(best.getName()) < 0);
		} else {
			return (initiative > bestInitiative);
		}
    }
    
    /**
     * A Craft can be the next to move if it is the craft with the highest initiative that 
     * has not moved, OR, There are crafts with with a higher initiative but they all belong 
     * to the owner of this craft.
     * 
     * Therefore - A player can usurp the move order of his own crafts.
     * 
     * @return True if this craft can move next.
     */
    public boolean canMoveNext(BattleCraft craft) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		int craftInitiative = craft.getInitiative();
        Player owner = craft.getController();

		for (int i=0; i<crafts.length; i++) {
			if (!crafts[i].isControlledBy(owner) && !crafts[i].hasMoved()) {
                int initiative = crafts[i].getInitiative();
                if (initiative > craftInitiative) {
                	if (!isWaitingCraft(crafts[i])) {
                		return false;
                	}
                }
			}
		}
        return true;
    }

    /**
     * @return If all crafts in the battle have moved (must be planets turn or can start next turn).
     */
    public boolean allCraftHaveMoved() throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		if (crafts == null) {
			return true;
		}

		for (int i=0; i<crafts.length; i++) {
			if (!crafts[i].hasMoved()) {
           		return false;
			}
		}
        return true;
    }

    /**
     * NOTE - we cannot remove a craft from the wait list until it has finished its move because the GUI may ask for the 
     * next craft again and again without moving it.
     * @return The first craft in the wait list that has move remaining.
     */
	private BattleCraft getFirstWaitingCraft() {
		BattleCraft first = (BattleCraft) waitingCraft.get(0);
		while (first != null && first.hasMoved()) {
			removeFromWaitList(first);
			if (waitingCraft != null) {
				first = (BattleCraft) waitingCraft.get(0);
			} else {
				first = null;
			}
		}
		return first;
	}
	
    /**
     * Add craft to the list of crafts waiting until after other crafts of lower initiative 
     * have moved. @see Battle.waitingCraft for details.
     * @param craft The craft that does not want to move immediately.
     */
    public void addToWaitList(BattleCraft craft) {
        if (waitingCraft == null) waitingCraft = new ArrayList();
        waitingCraft.add(craft);
    }
    
    public void removeFromWaitList(BattleCraft craft) {
        if (waitingCraft == null) return;
        waitingCraft.remove(craft);
        if (waitingCraft.size() == 0) {
        	waitingCraft = null;
        }
    }

    public void clearWaitList() {
        if (waitingCraft == null) return;
        waitingCraft.clear();
    }
    

	public BattleCraft [] getWaitingCraft() {
        if (waitingCraft == null) {
        	return null;
        } else {
        	return (BattleCraft[]) waitingCraft.toArray(new BattleCraft[waitingCraft.size()]);
        }
	}

	public boolean isWaitingCraft(BattleCraft craft) {
        if (waitingCraft == null) {
        	return false;
        } else {
        	for (BattleCraft c : waitingCraft) {
				if (c == craft) return true;
			}
        	return false;
        }
	}

	/**
	 * @return true if this player has enemy craft in the battle.
     * An enemy craft is a craft whose owner is at war with this player.
     * @param owner - The player we wish to determine the existence of enemies.
     * @param mustBeOperating - If true enemy craft must be operable (have crew, not disabled)
     * to be considered an enemy!
	 */
	public boolean hasEnemyCraft(Player owner, boolean mustBeOperating) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();

		for (int i=0; i<crafts.length; i++) {
			if (r.isWar(crafts[i].getOwner(), owner)) {
                if (mustBeOperating && crafts[i].isOperating()) {
                    return true;
                }
			}
		}

		return false;

	}

    /**
     * @return True if there is a planet and the planet in the battle is owned by an enemy 
     * of owner has and has weapons or shields.   
     * @param owner - The player we wish to determine the existence of enemies.
     */
	public boolean hasEnemyPlanet(Player owner) throws ApplicationError {
        if (planet != null && owner.isEnemy(planet.getOwner())) {
            return (planet.hasWeapons() || planet.hasShields() || planet.hasFightersToLaunch());
        }
        return false;
    }

	/**
	 * @return true if this player has craft in the battle.
     * @param owner - The player we wish to determine the existence of enemies.
     * @param enemyMustBeOperating - If true craft must be operable (have crew, not disabled)
     * to be considered!
	 */
	public boolean hasCraft(Player owner, boolean mustBeOperating) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();

		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].getOwner() == owner) {
                if (mustBeOperating && crafts[i].isOperating()) {
                    return true;
                }
			}
		}

		return false;

	}

	
	/**
	 * @return the closest enemy craft to the location parameter in the battle.
     * An enemy craft is a craft whose owner is at war with this player.
	 */
	public BattleCraft getClosestEnemyCraft(Player owner, Location srcLocation, boolean mustBeOperating) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();

		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (r.isWar(crafts[i].getOwner(), owner)) {
				Location craftLoc = crafts[i].getLocation();
				int dist = craftLoc.getDistanceTo(srcLocation);
				if (closest == null || dist < closestDist) {
					if (mustBeOperating) {
						if (crafts[i].isOperating()) {
							closest = crafts[i];
							closestDist = dist;
						}
					} else {
						closest = crafts[i];
						closestDist = dist;
					}
				}
			}
		}
		
		return closest;
	}

	/**
	 * @return the closest enemy craft to the location parameter in the battle.
     * An enemy craft is a craft whose owner is at war with this player.
	 */
	public BattleCraft getClosestEnemyCraft(Player owner, ServerImageObject source, boolean mustBeOperating, ExclusionZoneList exclusionZones) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();
		Location srcLocation = source.getLocation();

		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (source != crafts[i] && r.isWar(crafts[i].getOwner(), owner)) {
				Location craftLoc = crafts[i].getLocation();
				int dist = craftLoc.getDistanceTo(srcLocation);
				if (closest == null || dist < closestDist) {
					if (exclusionZones == null || !exclusionZones.cotains(craftLoc)) {						
						if (mustBeOperating) {
							if (crafts[i].isOperating()) {
								closest = crafts[i];
								closestDist = dist;
							}
						} else {
							closest = crafts[i];
							closestDist = dist;
						}
					}
				}
			}
		}
		
		return closest;
	}

	/**
	 * @return the biggest enemy craft to the location parameter in the battle.
     * An enemy craft is a craft whose owner is at war with this player.
	 */
	public BattleCraft getMostPowerfulEnemyCraftAwayFromMyCraftInRange(Player owner, BattleCraft source, boolean mustBeOperating, int minDistToFriendlyCraft, ExclusionZoneList exclusionZones) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();
		Location srcLoc = source.getLocation();
		int sourceSpeed = source.getSpeed(true);
		
		BattleCraft best = null;
		int bestPower = 0;

		for (int i=0; i<crafts.length; i++) {
			if (source != crafts[i] && r.isWar(crafts[i].getOwner(), owner)) {
				Location craftLoc = crafts[i].getLocation();
				int dist = craftLoc.getDistanceTo(srcLoc);
				if (dist < sourceSpeed) {
					int closestDistToMine = getClosestDistanceToCraftOwnedBy(crafts[i], owner);
					if (closestDistToMine > minDistToFriendlyCraft) {						
						if (exclusionZones == null || !exclusionZones.cotains(craftLoc)) {						
							if (mustBeOperating) {
								if (crafts[i].isOperating()) {
									int power = crafts[i].getCraft().getMaxDamage();
									if (best == null || bestPower < power) {
										best = crafts[i];
										bestPower = power;
									}
								}
							} else {
								int power = crafts[i].getCraft().getMaxDamage();
								if (best == null || bestPower < power) {
									best = crafts[i];
									bestPower = power;
								}
							}
						}
					}
				}
			}
		}
		
		return best;
	}

	/**
	 * @return the closest enemy craft to the location parameter in the battle.
     * An enemy craft is a craft whose owner is at war with this player.
	 */
	public BattleCraft getClosestEnemyCraftAwayFromMyCraft(Player owner, ServerImageObject source, boolean mustBeOperating, int minDistToFriendlyCraft, ExclusionZoneList exclusionZones) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();
		Location srcLoc = source.getLocation();

		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (source != crafts[i] && r.isWar(crafts[i].getOwner(), owner)) {
				Location craftLoc = crafts[i].getLocation();
				int dist = craftLoc.getDistanceTo(srcLoc);
				if (closest == null || dist < closestDist) {
					int closestDistToMine = getClosestDistanceToCraftOwnedBy(crafts[i], owner);
					if (closestDistToMine > minDistToFriendlyCraft) {						
						if (exclusionZones == null || !exclusionZones.cotains(craftLoc)) {						
							if (mustBeOperating) {
								if (crafts[i].isOperating()) {
									closest = crafts[i];
									closestDist = dist;
								}
							} else {
								closest = crafts[i];
								closestDist = dist;
							}
						}
					}
				}
			}
		}
		
		return closest;
	}


	/**
	 * @return the closest enemy craft to the location parameter in the battle.
     * An enemy craft is a craft whose owner is at war with this player.
	 */
	public BattleObject getClosestEnemyOperatingCraftWithinFiringAngle(Player player, BattleObject source) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();

		BattleObject closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (player.isEnemy(crafts[i].getOwner()) && !crafts[i].isControlledBy(player)) {
				int dist = crafts[i].getLocation().getDistanceTo(source.getLocation());
				if (closest == null || dist < closestDist) {
					if (source.isTargetAngleWithinWeaponFiringAngleOfAllWeapons(crafts[i])) {
                    	if (crafts[i].isOperating()) {
	                        closest = crafts[i];
	                        closestDist = dist;
                    	}
                    }
				}
			}
		}
		
		if (planet != null && planet.getOwner() != null && source.getOwner().isEnemy(planet.getOwner())) {
			int dist = planet.getLocation().getDistanceTo(source.getLocation());
			if (closest == null || dist < closestDist) {
				if (source.isTargetAngleWithinWeaponFiringAngleOfAllWeapons(planet)) {
                    closest = planet;
                    closestDist = dist;
                }
			}
		}

		return closest;

	}

	public BattleObject getClosestEnemyOperatingCraftWithinFiringAngleForWeapon(Player player, BattleObject source, WeaponInstance weapon) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();

		BattleObject closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (player.isEnemy(crafts[i].getOwner()) && !crafts[i].isControlledBy(player)) {
				int dist = crafts[i].getLocation().getDistanceTo(source.getLocation());
				if (closest == null || dist < closestDist) {
					if (source.isTargetAngleWithinWeaponFiringAngleOfAllWeapons(crafts[i])) {
                    	if (crafts[i].isOperating()) {
                    		if (weapon.willHaveAnEffectOn(crafts[i])) {
		                        closest = crafts[i];
		                        closestDist = dist;
                    		}
                    	}
                    }
				}
			}
		}
		
		if (planet != null && planet.getOwner() != null && source.getOwner().isEnemy(planet.getOwner())) {
			int dist = planet.getLocation().getDistanceTo(source.getLocation());
			if (closest == null || dist < closestDist) {
				if (source.isTargetAngleWithinWeaponFiringAngleOfAllWeapons(planet)) {
            		if (weapon.willHaveAnEffectOn(planet)) {
	                    closest = planet;
	                    closestDist = dist;
            		}
                }
			}
		}

		return closest;

	}
	


	/**
	 * @return the closest enemy craft to the location parameter in the battle.
     * An enemy craft is a craft whose owner is at war with this player.
	 */
	public BattleCraft getClosestEnemyCraftWithCrewDamage(Player owner, Location loc, BattleCraft exclude) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();

		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (exclude != crafts[i] && r.isWar(crafts[i].getOwner(), owner) && crafts[i].getCrewAsPercent() < 1.0) {
				int dist = crafts[i].getLocation().getDistanceTo(loc);
				if (closest == null || dist < closestDist) {
                    closest = crafts[i];
                    closestDist = dist;
				}
			}
		}

		return closest;

	}

	/**
	 * @return the closest enemy craft to the location parameter in the battle.
     * An enemy craft is a craft whose owner is atr war with this player.
	 */
	public int getQtyEnemyCraft(Player owner) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
		Relations r = MotEngine.instance.getRelations();

		int qty = 0;

		for (int i=0; i<crafts.length; i++) {
			if (r.isWar(crafts[i].getOwner(), owner)) {
				qty++;
			}
		}

		return qty;

	}

	/**
	 * @return the closest enemy craft to the location parameter in the battle.
     * An enemy craft is a craft whose owner is at war with this player.
	 */
	public BattleCraft getMyClosestCraft(Player owner, Location loc, boolean mustBeOperating, BattleCraft exclude) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();

		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (exclude != crafts[i] && crafts[i].getOwner() == owner) {
				int dist = crafts[i].getLocation().getDistanceTo(loc);
				if (closest == null || dist < closestDist) {
                    if (mustBeOperating && crafts[i].isOperating()) {
                        closest = crafts[i];
                        closestDist = dist;
                    } else {
                        closest = crafts[i];
                        closestDist = dist;
                    }
				}
			}
		}

		return closest;
	}

	/**
	 * @return the closest craft to craft bc parameter in the battle.
	 */
	public BattleCraft getClosestCraft(BattleCraft bc) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();

		Location craftLocation = bc.getLocation();
		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (crafts[i] != bc) {
				int dist = crafts[i].getLocation().getDistanceTo(craftLocation);
				if (closest == null || dist < closestDist) {
                    closest = crafts[i];
                    closestDist = dist;
				}
			}
		}

		return closest;

	}

	/**
	 * @return the closest craft to craft bc parameter in the battle that is owned by owner
	 * and is not the craft bc passed in.
	 */
	public BattleCraft getClosestCraftOwnedBy(ServerImageObject bc, Player owner) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();

		Location craftLocation = bc.getLocation();
		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (crafts[i] != bc && crafts[i].isOwnedBy(owner)) {
				int dist = crafts[i].getLocation().getDistanceTo(craftLocation);
				if (closest == null || dist < closestDist) {
                    closest = crafts[i];
                    closestDist = dist;
				}
			}
		}

		return closest;

	}
	
	public int getClosestDistanceToCraftOwnedBy(ServerImageObject bc, Player owner) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();

		Location craftLocation = bc.getLocation();
		BattleCraft closest = null;
		int closestDist = Integer.MAX_VALUE;

		for (int i=0; i<crafts.length; i++) {
			if (crafts[i] != bc && crafts[i].isOwnedBy(owner)) {
				int dist = crafts[i].getLocation().getDistanceTo(craftLocation);
				if (closest == null || dist < closestDist) {
                    closest = crafts[i];
                    closestDist = dist;
				}
			}
		}

		return closestDist;

	}
	
	/**
	 * @return the closest craft to loc parameter in the battle that is not contained in
	 * the exclude list.
	 */
	public BattleCraft getClosestCraft(Location loc, List exclude) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();

		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (exclude == null || !exclude.contains(crafts[i])) {
				int dist = crafts[i].getLocation().getDistanceTo(loc);
				if (closest == null || dist < closestDist) {
                    closest = crafts[i];
                    closestDist = dist;
				}
			}
		}

		return closest;
	}

	/**
	 * @return the closest craft to loc parameter in the battle that is not contained in
	 * the exclude list and is owned by owner.
	 */
	public BattleCraft getClosestCraftOwnedBy(Location loc, Player owner, List exclude) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();

		BattleCraft closest = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].isOwnedBy(owner) && !exclude.contains(crafts[i])) {
				int dist = crafts[i].getLocation().getDistanceTo(loc);
				if (closest == null || dist < closestDist) {
                    closest = crafts[i];
                    closestDist = dist;
				}
			}
		}

		return closest;

	}
	
	/**
	 * @return the quantity of crafts in the battle at this point in time.
	 */
	public int getQtyCraft() throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
        if (crafts == null) {
            return 0;
        } else {
            return crafts.length;
        }
    }

	/**
	 * @return the quantity of fighter crafts player owner has in the battle at this point in time.
	 */
	public int getQtyCraft(Player owner) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
        int qty = 0;
		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].getOwner() == owner) {
                qty++;
			}
		}
		return qty;
	}

	/**
	 * @return the quantity of fighter crafts this player has.
	 */
	public int getQtyFighterCraft(Player owner) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
        int qty = 0;
		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].getOwner() == owner) {
                qty++;
			}
		}
		return qty;
	}

	/**
	 * @return null if and only if this craft has space on the battle map to exist
	 * at this location. Otherwise it returns a list of battle craft that occupy some portion 
     * of the space this craft would need to be at this location.
     * Note the list will never include the craft its self!
	 */
	public ServerImageObject [] getLocationOccupiers(Location loc, ServerImageObject craft) throws ApplicationError {
		Area areaOccupied = craft.getAreaAt(loc);
		return getAreaOccupiers(craft, areaOccupied);
	}

	public ServerImageObject getClosestLocationOccupier(Location loc, ServerImageObject craft) throws ApplicationError {
		ServerImageObject[] locationOccupiers = getLocationOccupiers(loc, craft);
		if (locationOccupiers != null && locationOccupiers.length > 0) {
			ServerImageObject occupier = (ServerImageObject) Utils.getClosest(locationOccupiers, craft.getLocation());
			return occupier;
		} else {
			return null;
		}
	}
	
	public ServerImageObject [] getLocationOccupiers_ShiftLocToAvoidOldOccupant(Location loc, ServerImageObject craft, ServerImageObject oldOccupant, int shiftDir) throws ApplicationError {
		ServerImageObject[] locationOccupiers = getLocationOccupiers(loc, craft);
		if (locationOccupiers != null && locationOccupiers.length > 0) {
			ServerImageObject occupier = locationOccupiers[0];
			if (occupier == oldOccupant) {
				while( craft.getAreaAt(loc).intersects(occupier.getArea()) ) loc.shiftInDirection(shiftDir, 2);
				locationOccupiers = getLocationOccupiers(loc, craft);
				if (locationOccupiers != null && locationOccupiers.length > 0) {
					occupier = locationOccupiers[0];
				} else {
					occupier = null;
				}
			}
			return locationOccupiers;
		} else {
			return null;
		}
	}
	
	public static boolean noCraftBetween(BattleObject battleObject, ServerImageObject originalTarget) {
		return false;
	}

	
	/**
	 * @return null if and only if this craft has space on the battle map to exist
	 * at this location. Otherwise it returns a list of battle craft that occupy some portion 
     * of the space this craft would need to be at this location.
     * Note the list will never include the craft its self!
	 */
	public ServerImageObject [] getAreaOccupiers(ServerImageObject excludeObject, Area area) throws ApplicationError {
		BattleCraft [] crafts = getCrafts();
        ArrayList list = null;

		for (int i=0; i<crafts.length; i++) {
			if (crafts[i] != excludeObject) {
				Area craftArea = crafts[i].getArea();
                //Logger.logDebug("INTERSECTS "+areaOccupiedA+"  "+areaOccupiedB+"  => "+areaOccupiedB.intersects(areaOccupiedA));
				if (craftArea != null && craftArea.intersects(area)) {
                    if (list == null) list = new ArrayList();
					list.add(crafts[i]);
				}
			}
		}
        
        if (planet != null) {
            Area craftArea = planet.getArea();
            //Logger.logDebug("INTERSECTS "+areaOccupiedA+"  "+areaOccupiedB+"  => "+areaOccupiedB.intersects(areaOccupiedA));
            if (craftArea.intersects(area)) {
                if (list == null) list = new ArrayList();
                list.add(planet);
            }
        }

        if (list == null) {
            return null;
        } else {
            return (ServerImageObject []) list.toArray(new ServerImageObject [list.size()]);
        }
	}

	
	/**
	 * When multiple locations are possible, the finalTarget location is used to determine which is best.
	 * This method will choose the one with the minimum distance and minimum turning distance to the finalTarget
	 * passed in.
	 * To avoid infinite loops on a congested battlefield we define a maximum depth this will traverse.
     * @param moveToDest A location the craft was originally asked to move to.
     * @param actualDest The closest location to moveToDest that is within range and that we are considering moving to.
     * @param currentDirection The direction the craft is pointing now.
     * @param craft The craft that is moving.
     * @param targetDest The original destination the user / computer selected as the target.
     * This is usually the location of the craft or planet under attack.
	 * @returns the closest location where this craft has space on the battle map to exist
	 * in the area between this crafts location and the location loc passed in.
	 */
	public Location getFreeLocationInArea(Location moveToDest, Location actualDest, int currentDirection, ServerImageObject craft, Location targetDest, MovementCostCalculator mcc, int depth) throws ApplicationError {
		if (depth > 3) {
			return getLocation();
		}
		ServerImageObject occupant = getClosestLocationOccupier(actualDest, craft);
		if (occupant != null) {
			// I attempt to locate the craft either side of the image object that occupies the space I 
			// would like to use.
			int occupantImageSize = occupant.getMaxSpaceDimension();
			int myImageSize = craft.getMaxSpaceDimension();
			
			// Now calculate two possible locations either side of the object that occupies the space 
			// we would like to move to.
			
			int dir1 = (currentDirection + 90)%360;
			Location tryLoc1 = new Location(occupant.getLocation());
			tryLoc1.shiftInDirection(dir1, occupantImageSize+myImageSize+1);
			
			int dir2 = (currentDirection - 90 + 360)%360;
			Location tryLoc2 = new Location(occupant.getLocation());
			tryLoc2.shiftInDirection(dir2, occupantImageSize+myImageSize+1);
			
			// Determine the if either of the two locations we have calculated are free for us to use 
			ServerImageObject [] occupants1 = getLocationOccupiers_ShiftLocToAvoidOldOccupant(tryLoc1, craft, occupant, dir1);
			ServerImageObject [] occupants2 = getLocationOccupiers_ShiftLocToAvoidOldOccupant(tryLoc2, craft, occupant, dir2);
			ServerImageObject [] occupants3 = null;
			
			if (occupants1 != null && occupants1.length > 0) {
				// Location tryLoc1 is occupied
				// Try moving away from occupier to find free space
				Location tl = craft.getClosestLocationTo(tryLoc1, occupants1[0], occupants1[0].getMaxOfHeightAndWidth()-2);
				if (tl != null) {					
					tryLoc1 = tl;
					occupants1 = getLocationOccupiers_ShiftLocToAvoidOldOccupant(tryLoc1, craft, occupant, dir1);
				}
			}
			if (occupants2 != null && occupants2.length > 0) {
				// Location tryLoc1 is occupied
				// Try moving away from occupier to find free space
				Location tl = craft.getClosestLocationTo(tryLoc2, occupants2[0], occupants2[0].getMaxOfHeightAndWidth()-2);
				if (tl != null) {					
					tryLoc2 = tl;
					occupants2 = getLocationOccupiers_ShiftLocToAvoidOldOccupant(tryLoc2, craft, occupant, dir2);
				}
			}
			
			// Determine which one is best using angle to point to final target and distance.
			Location scoreToLocation = (targetDest == null) ? moveToDest : targetDest;
			int score1 = mcc.getScoreToTarget(craft, tryLoc1, scoreToLocation);
			int score2 = mcc.getScoreToTarget(craft, tryLoc2, scoreToLocation);

			if (occupants1 != null && occupants1.length > 0) score1 = Integer.MAX_VALUE;
			if (occupants2 != null && occupants2.length > 0) score2 = Integer.MAX_VALUE;
			
			if (score1 <= score2 && score1 < Integer.MAX_VALUE) {
				return tryLoc1;
			} else if (score2 < Integer.MAX_VALUE) {
				return tryLoc2;
			} else {
				// recurse and try the next location further away from the target towards the source
	            actualDest = craft.getClosestLocationTo(occupant);
	            actualDest.shift(craft.getLocation(), 1);
	            ServerImageObject [] occupants = getLocationOccupiers(actualDest, craft);
	            if (occupants != null && actualDest.notEqual(craft.getLocation())) {
	                actualDest = getFreeLocationInArea(moveToDest, actualDest, currentDirection, craft, targetDest, mcc, depth+1);
	            }
			}
			
		}

		return actualDest;
	}
    
	/**
	 * @returns the closest location where this craft has space on the battle map to exist
	 * on a straight line between this crafts location and the location loc passed in.
	 */
	public Location getClosestFreeLocationNoDirChange(Location loc, ServerImageObject craft) throws ApplicationError {
		ServerImageObject [] occupants = getLocationOccupiers(loc, craft);
		Location tryLoc = loc;
		if (occupants != null) {
			Location craftLocation = craft.getLocation();
			int direction = craftLocation.getDirectionFor(loc);
			int distance = craftLocation.getDistanceTo(loc);
            while (distance > 0 && occupants != null) {
            	distance--;
            	tryLoc = new Location(craftLocation);
            	tryLoc.shiftInDirection(direction, distance);
            	occupants = getLocationOccupiers(tryLoc, craft);
            }            
		}

		return tryLoc;
	}

	/**
	 * @returns the closest location where craft has space on the battle map to exist
	 * on a straight line.
	 */
	public Location getClosestFreeLocationInLine(Location loc, ServerImageObject craft) throws ApplicationError {
		ServerImageObject [] occupants = getLocationOccupiers(loc, craft);
		if (occupants != null) {
            loc = craft.getClosestLocationTo(occupants[0]);
            occupants = getLocationOccupiers(loc, craft);
            while (occupants != null && loc.notEqual(craft.getLocation())) {
                loc.shift(craft.getLocation(), 1);                
                occupants = getLocationOccupiers(loc, craft);
            }
		}

		return loc;
	}
    
    /**
     * The extra distance added to a planets scan ability.
     */
    private static int PLANET_SCAN_BONUS = 150;

    /**
     * @return the scan highest strength for all players that are an enemy of player at this 
     * location in the battle field.
     * @see getScannerStrengthAt(Player player, Location loc) for a definition of a players 
     * scan strength.
     */
    public int getEnemyScannerStrengthAt(Player player, Location loc) throws ApplicationError {
        Player [] players = getPlayers();
        if (players == null) return 0;
        int best = 0;
        for (int i=0; i<players.length; i++) {
            if (players[i] != player && player.isEnemy(players[i])) {
                int s = getScannerStrengthAt(players[i], loc);
                if (s > best) best = s;
            }
        }
        return best;
    }

    /**
     * @return the scan strength for this player at this location in the battle field.
     * This is defined as the maximum scanner strength at the location the craft is positioned.
     * The scanner strength is defined as the scanner power minus the distance from the scanner.
     */
    public int getScannerStrengthAt(Player player, Location loc) throws ApplicationError {
        int strength = 0;
        
        if (planet != null && planet.getOwner() == player) {
            Scanners scanners = planet.getPlanet().getScanners();
            strength = PLANET_SCAN_BONUS - loc.getDistanceTo(planet.getLocation());
            if (scanners != null) {
                strength += scanners.getStrength();
            }
            if (strength < 0) strength = 0;
        }

        BattleCraft [] crafts = getCrafts();
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getOwner() == player) {
                Scanners scanners = crafts[i].getCraft().getScanners();
                if (scanners != null && scanners.getStrength() > strength) {
                    int v = scanners.getStrength(loc.getDistanceTo(crafts[i].getLocation()));
                    if (v > strength) strength = v;
                }
            }
        }

        return strength;
    }
    

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("turn", turn);
		if (getCrafts() != null) {
            for (int i=0; i<crafts.length; i++) {
                crafts[i].toXML(writer);                
            }
		}
		if (planet != null) {
			planet.toXML(writer);
		}
		if (startingPlayers != null) {
			startingPlayers.toXML(writer);
		}
	}

	protected String getElementName() { return "battle"; }


    public Automaton []  getAutomatons() {
        if (automatons == null || automatons.size() == 0) {
            return null;
        } else {
            return (Automaton []) automatons.toArray(new Automaton [automatons.size()]);
        }
    }

    /**
     * Missiles are always visible but mines are only visible to the person who
     * laid them.
     * @return All automatons in this battle that are visible to player passed in.
     */
    public Automaton []  getVisibleAutomitons(Player player) throws ApplicationError {
        if (automatons == null || automatons.size() == 0) {
            return null;
        } else {
        	Automaton [] automitons = getAutomatons();
        	ArrayList r = null;
        	for (int i=0; i<automitons.length; i++) {
        		if (automitons[i].isVisible(player)) {
        			if (r == null) r = new ArrayList();
        			r.add(automitons[i]);
        		}
        	}
        	if (r == null) {
	            return null;
        	} else {
	            return (Automaton []) r.toArray(new Automaton [r.size()]);
        	}
        }
    }

    /**
     * @param player Any automaton returned by this method must be owned by an enemy 
     * of this player. 
     * @param location The location we wish to find an automaton closest to.
     * @param mustBeMoveable If this is true automatons returned must be subclasses 
     * of MoveableAutomiton.
     * @return The closest automaton that meets the criteria to location in this 
     * battle.
     * @throws ApplicationError 
     */
	public Automaton getClosestEnemyAutomiton(Player player, Location location, boolean mustBeMoveable, boolean mustBeVisible) throws ApplicationError {
        if (automatons == null || automatons.size() == 0) {
            return null;
        } else {
        	Automaton [] automitons = getAutomatons();
        	Automaton closest = null;
        	int closestDist = Integer.MAX_VALUE;
        	for (int i=0; i<automitons.length; i++) {
        		if (!automitons[i].getOwner().isEnemy(player)) {
        			continue;
        		}
        		if (mustBeVisible && !automitons[i].isVisible(player)) {
        			continue;
        		}
        		if (mustBeMoveable && !(automitons[i] instanceof MovingAutamaton)) {
        			continue;
        		}
        		int dist = location.getDistanceTo(automitons[i].getLocation());
    			if (closest == null || closestDist > dist) {
    				closest = automitons[i];
    				closestDist = dist;
    			}
        	}
        	return closest;
        }
	}

	public MovingAutamaton getClosestEnemyMovingAutomatonInWeaponFireAngleAimedAtMe(BattleObject source, WeaponInstance weaponInstance, boolean mustBeVisible) throws ApplicationError {
        if (automatons == null || automatons.size() == 0) {
            return null;
        } else {
        	Automaton [] automitons = getAutomatons();
        	MovingAutamaton closest = null;
        	int closestDist = Integer.MAX_VALUE;
        	Player owner = source.getOwner();
        	for (int i=0; i<automitons.length; i++) {
        		if (!automitons[i].getOwner().isEnemy(owner)) {
        			continue;
        		}
        		if (mustBeVisible && !automitons[i].isVisible(owner)) {
        			continue;
        		}
        		if (!(automitons[i] instanceof MovingAutamaton)) {
        			continue;
        		}
        		MovingAutamaton ma = (MovingAutamaton) automitons[i];
        		if (!ma.getTarget().isOwnedBy(owner)) {
        			continue;
        		}
        		Location automatonLoc = ma.getLocation();
        		Location sourceLoc = source.getLocation();
				int dist = sourceLoc.getDistanceTo(automatonLoc);
    			if (closest == null || closestDist > dist) {
            		if (source.isTargetAngleWithinWeaponFiringAngle(weaponInstance, ma)) {
	    				closest = ma;
	    				closestDist = dist;
            		}
    			}
        	}
        	return closest;
        }
	}
	
	public Wreckages getWreckages() {
		return wreckages;
	}

	public String toString() {
		String craftsNames = Comp.toString(getCrafts());
		return "Battle at "+getOrbit()+" containing "+craftsNames;
	}

	public boolean hasSentEndBattleEvent() {
		return sentEndBattleEvent;
	}
	
	public PlayerBattleBonuses getPlayerBonuses(Player p) throws ApplicationError {
		PlayerBattleBonuses pb = playerBonuses.get(p.getName());
		if (pb == null) {
			pb = new PlayerBattleBonuses();
			pb.update(p, this);
			playerBonuses.put(p.getName(), pb);
		}
		return pb;
	}

	private void updateBattleBonuses(Player p) throws ApplicationError {
		PlayerBattleBonuses pb = getPlayerBonuses(p);
		pb.update(p, this);
	}
	
	public BattleCraftSpeedUpdateVO [] getSpeedUpdates() throws ApplicationError {
		ArrayList<BattleCraftSpeedUpdateVO> r = null;
		BattleCraft[] ca = getCrafts();
		if (ca != null) {
			for (int i = 0; i < ca.length; i++) {
				if (_allCraftsNeedSpeedUpdate || ca[i].isClientNeedSpeedUpdate()) {
					ca[i].setClientNeedSpeedUpdate(false);
					if (r == null) r = new ArrayList();
					BattleCraftSpeedVO vo = new BattleCraftSpeedVOAdapter(ca[i]);
					r.add(new BattleCraftSpeedUpdateVO(ca[i].getName(), getName(), vo));
				}
			}
		}
		_allCraftsNeedSpeedUpdate = false;
		if (r == null) {
			return null;
		} else {
			return r.toArray(new BattleCraftSpeedUpdateVO[r.size()]);
		}
	}

	public boolean isNoShields() {
		return noShields;
	}

	public boolean isNoCloak() {
		return noCloak;
	}

	/**
	 * THIS CAN RETURN NULL!!!
	 * @return Return a random area on this battlefield that contains space for this craft (area is not occupied).
	 */
	public Location getRandomFreeLocationFor(BattleObject craft) throws ApplicationError {
		Area availableArea = GalaxyDefinitions.BATTLE_BOUNDARY.getInsetArea(craft.getHeight());
		if (availableArea == null) return null;
		int count = 0;
		while (count < 20) {
			Location r = availableArea.getRandomLocation();
			ServerImageObject[] occupiers = getLocationOccupiers(r, craft);
			if (occupiers == null) {
				return r;
			}
		}
		return null;
	}

	public NextObjectToMoveCache getNextObjectToMoveCache() {
		return nextObjectToMoveCache;
	}

	public ExclusionZoneList getExclusionZones(Player owner) throws ApplicationError {
        BattleCraft [] crafts = getCrafts();
        if (crafts == null) {
            return null;
        }
        ExclusionZoneList r = null;
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].isOwnedBy(owner)) {
				ExclusionZone ez = crafts[i].getExclusionZone();
				if (ez != null) {
					if (r == null) r = new ExclusionZoneList();
					r.add(ez);
				}
			}
		}
		return r;
	}

	public boolean isStarted() {
		return started;
	}



}