/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import java.util.ArrayList;

import mot.common.bean.CraftState;
import mot.common.bean.Damage;
import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.ImageObjectSettings;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.ObjectMovement;
import mot.common.bean.ObjectShift;
import mot.common.bean.ObjectTurn;
import mot.common.bean.SalvageList;
import mot.common.bean.Size;
import mot.common.bean.WreckageDetails;
import mot.common.bean.stats.GameScoreData;
import mot.common.designs.CraftDesign;
import mot.common.lang.L;
import mot.common.leaders.GovenorLeader;
import mot.common.technology.Biotechnologies;
import mot.common.technology.Biotechnology;
import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Technology;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.incumbants.Incumbent;
import mot.engine.battles.incumbants.IncumbentEdge;
import mot.engine.battles.triggers.Triggers;
import mot.engine.common.ServerImageObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftDesignAdapter;
import mot.engine.crafts.CraftReference;
import mot.engine.crafts.WeaponBay;
import mot.engine.crafts.WeaponBays;
import mot.engine.event.BattleCraftLeaveEF;
import mot.engine.event.BattleCraftMoveEF;
import mot.engine.event.BattleCraftSurrenderEF;
import mot.engine.event.BattleCraftTeleportEF;
import mot.engine.event.BattleDamageEF;
import mot.engine.event.BattleDestroyedEF;
import mot.engine.event.BattleFearEF;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.leaders.EngineGeneralLeader;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.ResourcefulProcessor;
import mot.engine.players.UserPlayer;
import mot.engine.technology.WeaponInstance;
import mot.engine.technology.WeaponInstanceDetonator;

/**
 * This is a wrapper used by the battle module to represent a craft in battle.
 * The location is the location on the battle field instead of the map.
 */
public class BattleCraft extends BattlePrimaryObject {
    /**
     * The attacker has a but of an edge when moving. This is a smaller advantage than
     * having a BattleStationI special on the craft though!
     */
	private static final int INITIATIVE_FOR_ATTACKER = 5;
    
    /**
     * This is the percentage of weapon fires that miss a craft that is not visible (cloaked).
     */
    private static final double CHANCE_TO_MISS_WHEN_CLOAKED = 0.75;
    
    /**
     * The maximum percentage loss of movement a craft will use from its condition
     * A craft with speed 100 and 50% damaged would imply actual speed of 87.5
     */
    private static final double MAX_MOVEMNT_LOSS_DUE_TO_CONDITION = 0.25;

	/**
	 * A craft involved in a battle.
	 **/
	private CraftReference craftReference;

	/**
	 * True if this craft is one of the attacking craft. If a craft is an attacker then
	 * it has a slight initiative bonus.
	 **/
	private boolean attacker;

	/**
	 * The proportion of the move that is remaining (value between 0 and 1). 
     * Initially a battle craft with have move remaining of 1, then as the craft moves, its 
     * remaining move is reduced. Once remaining move is 0 the craft has to wait until the 
     * next turn before it can move again.
     * Note a proportion is used because turning costs and move costs are measured very 
     * differently but all consume the crafts move.
	 */
	private double moveRemaining;

	/**
	 * The direction the craft is pointing. A value between 0 and 359.
	 * A value of zero implies the craft is pointing North, and then it
	 * is measured in positive degrees.
	 */
	private int direction = 0;

	/**
	 * This is true if an only if this craft has been destroyed. If this is true
	 * then this craft will have been removed from the crafts and battle craft lists.
	 * However it still may be referenced from with a call stack (temporarily) and
	 * those methods need to know this craft is a gonner.
	 */
	private boolean destroyed = false;

    
    /**
     * When the first weapon is fired by this craft, psi is calculated and may produce
     * fear or surrender - All subsequent weapon fires will not calculate the psi.   
     */
    private boolean firedAWeaponThisTurn = false;

	/**
	 * The date in battle this craft retreated from the battle or null if this craft has not retreated.
	 */
	private Integer retreatTurn;
    
    /**
     * True iff this craft is a fighter craft. Note fighter craft are not added to the
     * players list of craft and do not cause debris when destroyed.
     */
    private boolean isFighter = false;
    
    /**
     * This is amount of damage inflicted on the craft that will reduce its movement.
     * A craft's impulse and turning speed is reduced by movementDamage.
     * Also a craft cannot exit a battle if movementDamage > 0.
     */
    private int movementDamage = 0;
    
    /**
     * This is amount of damage inflicted on the craft that will increase fear among the crew.
     * Once psiDamage reaches PSI_MAX_SURRENDER_RATING then the craft will surrender to its opponents.
     */
    private double psiDamage = 0.0;
    
    /**
     * The quantity of people manning this craft. If this reaches zero then the craft cannot 
     * start a new turn. An enemy craft can kill crew by landing its own crew on a craft and fighting
     * hand to hand or using Bio Weapons.
     */
    private int crew;
    
    /**
     * This is the controller of this craft at this point in time. A value of null implies the
     * owner of this craft is the controller. Otherwise possible states are Stasis, MindControl, 
     * Berzerker, which either delegate control to another player or control the craft in their
     * own way.
     */
    private BattleCraftController controllerDetails;
    
	public BattleCraft(Battle battle, Craft craft, boolean attacker) throws ApplicationError {
        this(battle, craft, attacker, false);
    }
    
	public BattleCraft(Battle battle, Craft craft, boolean attacker, boolean heavyWeapons) throws ApplicationError {
		super(battle);
		this.craftReference = new CraftReference(craft);
		this.attacker = attacker;

        setShields(craft.getShields(), true);
        setWeapons(craft.getWeaponBays(), heavyWeapons, false);
        
        crew = craft.getHull().getCrew();
        if (craft.getSpecials() != null && craft.getSpecials().getQtyBerths() != null) {
            crew += craft.getSpecials().getQtyBerths().intValue();
        }
        
        if (hasAutoRepair()) {
        	craft.setDamage(0);
        }
    }
    
    public Size getSize() throws ApplicationError {
        return getCraft().getCraftGraphics().getSize();
    }
    
    public boolean isOwnedBy(Player owner) throws ApplicationError {
        return getCraft().isOwnedBy(owner);
    }
    
    public boolean isControlledBy(Player p) throws ApplicationError {
	    if (isOwnedBy(p)) {
			return (controllerDetails == null || controllerDetails.canBeComandedBy(p));
	    } else {
			return (controllerDetails != null && controllerDetails.canBeComandedBy(p));
	    }
    }
    
	public Player getController() throws ApplicationError {
    	if (controllerDetails != null) {
    		return controllerDetails.getController();
    	} else {
    		return getOwner();
    	}
	}

	public CraftReference getCraftReference() { return craftReference; }
	public Craft getCraft() { return craftReference.getCraft(); }
	public boolean getAttacker() { return attacker; }
	public int getDirection() { return direction; }
	public double getMoveRemaining() { return moveRemaining; }

	public void setLocationDirection(Location l, int v) { setLocation(l); direction = v; }
	public void setCraftReference(CraftReference v) { this.craftReference = v; }
	public void setAttacker(boolean v) { this.attacker = v; }
	public void setDirection(int v) { direction = v; }
	public void _setMoveRemaining(double v) { moveRemaining = v; }
	public void setMoveRemaining(double v) throws ApplicationError {
        if (moveRemaining > v) {
            moveLinkedObjects(moveRemaining-v, null, null);
        }
        moveRemaining = v;
        if (moveRemaining < 0.0) moveRemaining = 0.0;
    }
	
	public void endTurn() throws ApplicationError {
		dischargeWeapons(); 
		setMoveRemaining(0.0);
		battle.removeFromWaitList(this);
	}

    public CraftStateFactory getCraftState() throws ApplicationError {
        return new CraftStateFactory(getOwnerName(), getControllerName(), getDamageAsPercent(), getShieldAsPercent(), getMovementDamageAsPercent(), getPsiDamageAsPercent(), getCrewAsPercent(), getLocationClone(), getDirection(), getOwner().getColor(), getIsCloakedToList(), getDecorators(), isDestroyed(), getMoveRemaining());
    }

    public CraftState getCraftState(Player player) throws ApplicationError {
   		return new CraftState(getOwnerName(), getControllerName(), getDamageAsPercent(), getShieldAsPercent(), getMovementDamageAsPercent(), getPsiDamageAsPercent(), getCrewAsPercent(), getLocationClone(), getDirection(), getOwner().getColor(), getIsCloakedToList(), getVisibleDecorators(player), isDestroyed(), getMoveRemaining());
    }

    public CraftState getCraftStateForGenericPlayer() throws ApplicationError {
   		return new CraftState(getOwnerName(), getControllerName(), getDamageAsPercent(), getShieldAsPercent(), getMovementDamageAsPercent(), getPsiDamageAsPercent(), getCrewAsPercent(), getLocationClone(), getDirection(), getOwner().getColor(), getIsCloakedToList(), null, isDestroyed(), getMoveRemaining());
    }

    // convenience
    public Player getOwner() throws ApplicationError { return getCraft().getOwner(); }
    public Player getOriginPlayer() throws ApplicationError {  return getCraft().getOrigin();  }
    public String getName() { return craftReference.getCraft().getName(); }
    public BattleCraftController getControllerDetails() { return controllerDetails; }

    public String getControllerName() throws ApplicationError { 
    	Player controller = getController(); 
    	return (controller == null) ? null : controller.getName();
    }

    public void setControllerDetails(BattleCraftController v) throws ApplicationError {
    	controllerDetails = v; 
    	getDecorators().update();
    }

    public boolean controlIsAutomated() throws ApplicationError {
    	if (controllerDetails != null) {
    		return controllerDetails.controlIsAutomated();
    	} else {
    		return getOwner().isComputerPlayer();
    	}
    }
    
    /**
     * Over rides BattleObject method.
     * @param bayIndex The index of the weapons bay we are seeking to get weapons bay details.
     * @return The details for the weapons bay (from the hull of this craft - if it is one) at index bayIndex;
     */
    protected WeaponBayDetails getWeaponBayDetailsForIndex(int bayIndex) {
    	return getCraft().getHull().getWeaponBayDetails(bayIndex);
    }
    
    /**
     * Removes technology tech from this battle craft and the craft this battle craft represents.
     * If this technology is a shield or weapon the instance is removed.
     * @param tech The technology to add.
     */
    public void remove(Technology tech) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft.remove() "+getName()+" Loses "+tech.getName());
        }
        if (tech instanceof Weapon) {
            Weapon t = (Weapon)tech;
            removeWeaponInstance(t);
            if (getCraft().getWeaponBays() != null) {
                getCraft().getWeaponBays().removeWeapon(t, 0);
            }
        } else if (tech instanceof Shield) {
            Shield t = (Shield)tech;
            removeShieldInstance(t);
            if (getCraft().getShields() != null) {
                getCraft().getShields().remove(t);
            }
        } else if (tech instanceof Special) {
            Special t = (Special)tech;
            if (getCraft().getSpecials() != null) {
                getCraft().getSpecials().remove(t);
            }
        } else {
            Logger.logWarning("BattleCraft.remove() Unable to remove tech "+tech.getFullName());
        }
    }
    
    /**
     * Adds technology tech to this battle craft and the craft this battle craft represents.
     * If this technology is a shield or weapon a new instance is created for this technology.
     * @param tech The technology to add.
     */
    public void add(Technology tech) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft.add() "+getName()+" Gains "+tech.getName());
        }
        Craft craft = getCraft();
		if (tech instanceof Weapon) {
            Weapon t = (Weapon)tech;
            WeaponBays bays = craft.getWeaponBays();
			if (bays == null) {
                WeaponBays wbs = new WeaponBays();
                craft.setWeaponBays(wbs);
            }
            bays.addWeapon(t, 0);
            addWeaponInstance(t, getWeaponBayDetailsForIndex(0) );
        } else if (tech instanceof Shield) {
            Shield t = (Shield)tech;
            addShieldInstance(t);
            if (craft.getShields() != null) {
                craft.getShields().add(t);
            } else {
                craft.setShields(new Shields(t));
            }
        } else if (tech instanceof Special) {
            Special t = (Special)tech;
            if (craft.getSpecials() != null) {
                craft.getSpecials().add(t);
            } else {
                craft.setSpecials(new Specials(t));
            }
        } else {
            Logger.logWarning("BattleCraft.add() Unable to add tech "+tech.getFullName());
        }
    }
    
	/**
	 * @return The damage this object has sustained.
	 */
	public double getDamage() {
        return getCraft().getDamage();
    }
    
    /**
     * @return A description of this object. This is used to display text in the info field of a UI.
     */
    public String getDescription() {
        Craft c = getCraft();
        int hitPoints = c.getHull().getMaxDamage();
        double damage = c.getDamage();
        double shieldPower = 0.0;
        double shieldMaxPower = 0.0;
        double blocked = 0.0;
        if (shields != null) {
            shieldPower = getShieldPower();
            shieldMaxPower = c.getShields().getPower();
            blocked = c.getShields().getDamageBlocked();
        }
        String weaponsDescription = "-";
        if (getCraft().getWeaponBays() != null) {
            Weapons weapons = c.getWeaponBays().getAllWeaponsAsList();
            weaponsDescription = weapons.getDescription(false);
        }
        return "Craft "+getName()+" DAMAGE "+Formatter.fraction(damage)+" of "+Formatter.fraction(hitPoints)+"  DEFENSE "+Formatter.fraction(blocked)+" ["+Formatter.fraction(shieldPower)+" of "+Formatter.fraction(shieldMaxPower)+"]  ATTACK "+weaponsDescription;
    }
    
    /**
     * This is an index representing this crafts manouverability in a battle. It is
     * based on the engine speed and hull size plus any race specials (bonuses to battle
     * turning costs and movement). The computer level also effects the manoeuvrability.
     **/
    public int getManoeuvrability() throws ApplicationError {
        int avgSpeed = (int) ((getSpeed() + getTurningSpeed())/2);
        int computerPower = (getCraft().getComputers() == null) ? 0 : getCraft().getComputers().getPower();
        return avgSpeed + computerPower;
    }

    public String getManoeuvrabilityDescription() throws ApplicationError {
        int avgSpeed = (int) ((getSpeed() + getTurningSpeed())/2);
        int computerPower = (getCraft().getComputers() == null) ? 0 : getCraft().getComputers().getPower();
        return L.MANOUVERABILITY+" = "+L.AVG_SPEED+" "+avgSpeed+
        		" + "+L.COMPUTER_POWER+" "+computerPower;
    }

    private boolean clientsNeedSpeedUpdate = false;
    private int _lastSpeedCalculated = -1;

    public boolean isClientNeedSpeedUpdate() { return clientsNeedSpeedUpdate; }
    public void setClientNeedSpeedUpdate(boolean v) { clientsNeedSpeedUpdate = v; }
    
    /**
     * Calculate the speed in battle for this craft.
     **/
    public int getSpeed() throws ApplicationError {
        int speed = getSpeed(true);
        if (_lastSpeedCalculated >= 0 && _lastSpeedCalculated != speed) {
        	clientsNeedSpeedUpdate = true;
        }
        _lastSpeedCalculated = speed;
		return speed;
    }
    
    /**
     * Calculate the speed in battle for this craft.
     **/
    public int getSpeed(boolean includeMovementDamage) throws ApplicationError {
        Craft craft = getCraft();
        Engine engine = craft.getEngine();
        Hull hull = craft.getHull();
        if (engine == null) {
            return 0;
        }
        double speed = engine.getImpulseSpeed() / hull.getBattleMovementCost();

        double specialsBonus = (craft.getSpecials() == null) ? 1.0 : (1.0 + craft.getSpecials().getBattleSpeedBonus());
        double playerBonus = 1.0 + getCraft().getOrigin().getOriginBattleSpeedBonus() + getOwner().getOwnerBattleSpeedBonus();
        
        double fleetBonus = 1.0;
        if (battle != null) {
	        PlayerBattleBonuses fleetBonuses = battle.getPlayerBonuses(getOwner());
	        if (fleetBonuses != null && fleetBonuses.getFleetBattleSpeedBonus() != null) fleetBonus += fleetBonuses.getFleetBattleSpeedBonus();
        }
        
        speed = speed * playerBonus * specialsBonus * fleetBonus;
        if (includeMovementDamage) {
            speed -= movementDamage;
        	double condition = getCondition();
            if (condition < 1.0) {
                speed -= speed * (1.0 - condition) * MAX_MOVEMNT_LOSS_DUE_TO_CONDITION;
            }
        }

        return (int) speed;
    }
    
    public String getSpeedDescription() throws ApplicationError {
        Craft craft = getCraft();
        Engine engine = craft.getEngine();
        Hull hull = craft.getHull();
        if (engine == null) {
            return null;
        }
        double speed = engine.getImpulseSpeed() / hull.getBattleMovementCost();

        double specialsBonus = (craft.getSpecials() == null) ? 0 : (craft.getSpecials().getBattleSpeedBonus());
        double playerBonus = getCraft().getOrigin().getOriginBattleSpeedBonus();
        float resourceBonus = getOwner().getResourceCache().getSpeedBonus();
        EngineGeneralLeader generalLeader = getOwner().getGeneralLeader();
        double leaderBonus = 0;
        if (generalLeader != null && generalLeader.hasBattleSpeedBonus()) {
            leaderBonus = generalLeader.getBattleSpeedBonus();  
        }
        
        return L.SPEED+" = "+Formatter.whole(speed)+
        		" + "+L.SPECIALS_BONUS+" "+Formatter.percentLabelNoSign(specialsBonus) +
        		" + "+L.PLAYER_BONUS+" "+Formatter.percentLabelNoSign(playerBonus) +
        		" + "+L.RESOURCE_BONUS+" "+Formatter.percentLabelNoSign(resourceBonus) +
        		" + "+L.LEADER_BONUS+" "+Formatter.percentLabelNoSign(leaderBonus) ;
    }

    /**
     * Calculate the turning speed in battle for this craft.
     **/
    public int getTurningSpeed() throws ApplicationError {
        Craft craft = getCraft();
        Engine engine = craft.getEngine();
        if (engine == null) {
            return 0;
        }
        Hull hull = craft.getHull();
        double speed = engine.getTurningSpeed() / hull.getBattleTurningCost();

        double specialsBonus = (craft.getSpecials() == null) ? 1.0 : (1.0 + craft.getSpecials().getBattleTurningBonus());
        double playerBonuses = 1.0 + craft.getOrigin().getOriginBattleTurningBonus() + getOwner().getOwnerBattleTurningBonus();
        
        double fleetBonus = 1.0;
        if (battle != null) {
	        PlayerBattleBonuses fleetBonuses = battle.getPlayerBonuses(getOwner());
	        if (fleetBonuses != null && fleetBonuses.getFleetBattleTurningBonus() != null) fleetBonus += fleetBonuses.getFleetBattleTurningBonus();
        }

        speed = speed * playerBonuses * specialsBonus * fleetBonus;
        speed -= movementDamage;

        double condition = getCondition();
        if (condition < 1.0) {
            speed -= speed * (1.0 - condition) * MAX_MOVEMNT_LOSS_DUE_TO_CONDITION;
        }

        return (int) speed;
    }

	/**
	 * @return the initiative for this craft in battle. A crafts initiative determines how
	 * early in each battles turn a craft moves. The initiative is based on
     * 1 - The size/manoeuvrability of the craft.
     * 2 - Any specials a craft may contain.
	 */
	public int getInitiative() throws ApplicationError {
		
        double specialsBonus = (getCraft().getSpecials() == null) ? 1.0 : (1.0 + getCraft().getSpecials().getInitiativeBonus());
        double playerBonus = 1.0 + getCraft().getOrigin().getOriginInitiativeBonus() + getOwner().getOwnerInitiativeBonus();
        
        double fleetBonus = 1.0;
        if (battle != null) {
	        PlayerBattleBonuses fleetBonuses = battle.getPlayerBonuses(getOwner());
	        if (fleetBonuses != null && fleetBonuses.getFleetInitiativeBonus() != null) fleetBonus += fleetBonuses.getFleetInitiativeBonus();
        }

        double v = getManoeuvrability() * specialsBonus * playerBonus * fleetBonus;
        if (attacker) v += INITIATIVE_FOR_ATTACKER;
        return (int) v;
	}

	public String getInitiativeDescription() throws ApplicationError {
        double specialsBonus = (getCraft().getSpecials() == null) ? 0.0 : (getCraft().getSpecials().getInitiativeBonus());
        double playerBonus = getCraft().getOrigin().getOriginInitiativeBonus();
        double leaderBonus = getOwner().getOwnerInitiativeBonus();
        int attackerBonus = (attacker) ? INITIATIVE_FOR_ATTACKER : 0;
        int v = getManoeuvrability();
        return L.INITIATIVE+" = "+L.MANOUVERABILITY+" "+v+
        		" + "+L.ATTACKER_BONUS+" + "+attackerBonus +
        		" + "+L.PLAYER_BONUS+" "+Formatter.percentLabelNoSign(playerBonus) +
        		" + "+L.SPECIALS_BONUS+" "+Formatter.percentLabelNoSign(specialsBonus) +
        		" + "+L.LEADER_BONUS+" "+Formatter.percentLabelNoSign(leaderBonus) ;
	}

    /**
     * @return the visibility of this craft in the battle field. A crafts visibility is effected by 
     * its shields and the players cloaking bonus.
     */
    public int getCloakLevel() throws ApplicationError {
    	if (battle != null && battle.isNoCloak()) {
    		return 0;
    	} else {
    		return getCraft().getCloak();
    	}
    }

    /**
     * @return The psi level of the craft 
     */
    public int getPsiLevel() throws ApplicationError {
        return getCraft().getOrigin().getOriginPsiRating() + getOwner().getOwnerPsiRating();
    }

    public String getPsiLevelDescription() throws ApplicationError {
    	int playerPsi = getCraft().getOrigin().getOriginPsiRating();

        int leaderBonus = 0;
        GovenorLeader govenorLeader = getOwner().getGovenorLeader();
        if (govenorLeader != null && govenorLeader.hasPsiRating()) {
        	leaderBonus = govenorLeader.getPsiRating();  
        }
        int bioPsi = 0;
        Biotechnologies biotechnologies = getOwner().getBiotechnologies();
		if (biotechnologies != null) {
			bioPsi = biotechnologies.getPsiRating();
		}
		int resourceBonus = getOwner().getResourceCache().getPsiBonus();
		
        return L.PSI+" = "+L.PLAYER+" "+Formatter.whole(playerPsi)+
        		" + "+L.TECH_BONUS+" "+bioPsi +
        		" + "+L.RESOURCE_BONUS+" "+resourceBonus +
        		" + "+L.LEADER_BONUS+" "+leaderBonus ;
    }

    /**
     * This is the state of repair this battle craft is in. It is a value between 0.0 and +1.0 
     * where 0.0 is a very poor and 1.0 is full working order. A craft normally works much 
     * better when it is in good condition.
     */
    public double getCondition() throws ApplicationError {
    	Craft craft = getCraft();
    	double buildQuality = craft.getBuildQuality();
    	double damage = craft.getDamage();
    	double maxDamage = craft.getMaxDamage();
		double r = buildQuality - (damage / maxDamage);
		if (r < 0.0) r = 0.0;
		if (r > 1.0) r = 1.0;
		return r;
	}
    
    /**
     * A craft has moved during this turn in the battle if move remaining is 0 and it 
     * cannot fire any weapons (note this could mean either it has no weapons, its weapons
     * have already been fired or it has weapons to fire, BUT, there is no enemy craft 
     * in range.
     * @return True if this craft finished its battle turn
     */
    public boolean hasMoved() { 
        return (moveRemaining <= 0.0 && !hasWeaponsToFire());
    }
    
    /**
     * NOTE - This is not equal to !hasMoved() - It is quite different.
     * @return True if this craft has not moved anything - It has not fired any weapons
     * and its move remaining is 1.0 (full).
     */
    public boolean hasNotMoved() { 
        return (moveRemaining >= 1.0 && allWeaponsReadyToFire());
    }
    
    /**
     * A craft can fire weapons if it has no weapons and its weapons have not already 
     * been fired and there is an enemy craft in range.
     * @return True if this craft can fire weapons.
     */
    public boolean canFireWeapons() throws ApplicationError { 
        if (weapons == null || !hasWeaponsToFire()) {
            return false;
        }
        Location myLoc = getLocation();
        ServerImageObject closestEnemy = getBattle().getClosestEnemyCraft(getOwner(), myLoc, false);
        BattlePlanet bp = getBattle().getPlanet();
        if (bp != null && bp.getOwner().isEnemy(getOwner())) {
            if (closestEnemy == null || myLoc.getDistanceTo(closestEnemy.getLocation()) > myLoc.getDistanceTo(bp.getLocation())) {
                closestEnemy = bp;
            }
        }
        return (closestEnemy != null && inWeaponRange(closestEnemy.getLocation()));
    }
    
    /**
     * Crafts are considered non operational if;
     *  1 - They have no crew
     * @return True if this craft is capable of operation in this battle. 
     */
    public boolean isOperating() {
        return (crew > 0);
    }
    
	/**
	 * Notify this craft a battle is about to start. This should be called before
	 * the start of each battle this craft is to take part in.
	 * @throws ApplicationError 
	 */
	public void startBattle() throws ApplicationError {
        if (isOperating()) {
            rechargeWeapons();
            rechargeShieldsFull();
        }
	}

	/**
	 * Ends this turn on the battle and starts a new turn.
     * If the owner of this craft is resourceful then the players resourceful process is executed.
	 */
	public void startTurn() throws ApplicationError {
        if (crew > 0) {
            rechargeWeapons();
            rechargeShields();
            if (!isOrbital()) moveRemaining = 1;
            Player owner = getOwner();
            if (owner.isResourceful() || hasAutoRepair()) {
                ResourcefulProcessor rp = owner.getResourcefulProcessor();
                if (rp != null) {
                    rp.execute(this);
                }
            }
        }
	}

    public void rechargeWeapons() {
    	super.rechargeWeapons();
        firedAWeaponThisTurn = false;
    }
    
	private boolean hasAutoRepair() {
		return (getSpecials() != null) && (getSpecials().getRepairEffect() != null) && (getSpecials().getRepairEffect() > 0);
	}

	/**
	 * Retreat from the battle.
	 */
	public void retreat() throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft.retreat() "+getName());
        }
        if (movementDamage > 0) {
            Logger.logWarning("BattleCraft.retreat() "+getName()+" CANNOT RETREAT AS IT HAS MOVEMENT DAMAGE");
            return;
        }
        if (getController() != getOwner()) {
        	if (Logger.isDebug()) {
        		Logger.logDebug("BattleCraft.retreat() "+getName()+" CANNOT RETREAT WHEN UNDER THE CONTROL OF ANOTHER PLAYER");
        	}
            setMoveRemaining(0);
            dischargeWeapons();
            return;        	
        }
        if (getCraft().getHull().isStation()) {
            return;
        }
        Battle battle = getBattle();
		MotEngine.instance.multicast( new BattleCraftLeaveEF(this), battle.getStartingUserPlayers() );
        battle.addToRetreatList(this);
        retreatTurn = battle.getTurn();
        battle.remove(this);
        deleteAutomitonsTargetedAtMe(null);
	}

	/**
	 * Inflicts the parameter damage amount of damage on this craft. This may destroy the craft
	 * or destroy any of the crafts instruments and equipment.
	 */
	public WeaponDamage damage(double damage, Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft.damage() craft "+getName()+" max dam "+getCraft().getMaxDamage()+" dam "+getCraft().getDamage()+" + "+damage);
        }
		getCraft().damage(damage, agressor, this, oldState, parent);
		return null;
	}
    
	/**
	 * Inflicts the parameter damage amount of psi damage to this craft - psi blocked 
     * by the shields. If psi damage reaches PSI_MAX_RETREAT_RATING then the craft 
     * retreats from the battle. The psi damage inflicted on a craft causes the craft 
     * to miss turns more and surrender more - See fireWeapon() for details.
	 */
    public void psiDamage(double damage) throws ApplicationError {
        psiDamage += damage;
        if (psiDamage <= 0) psiDamage = 0;
        if (psiDamage >= Biotechnology.PSI_MAX_RETREAT_RATING) {
            retreat();
            psiDamage = Biotechnology.PSI_MAX_RETREAT_RATING;
        }
    }
    
    /**
     * Most types of battle craft produce debris. Holocraft for example however do not leave 
     * debris behind nor do fighters (craft launched from a mother ship).
     * @return True iff when this craft is destroyed wreckage should be created on the battle field.
     */
    protected boolean destructonCausesWreckage() {
		Craft craft = getCraft();
		if (!craft.getHull().isDestructonCanCauseWreckage()) return false;
		Specials specials = craft.getSpecials();
		if (specials != null) {
			if (specials.containsGroup(Special.GROUP_INCINERATOR)) return false;
		}
		return true;
	}

	/**
	 * Destroy this craft (removes it from the game) and causes it to explode during the battle.
	 * It also triggers the destruction of generated holo crafts and detonators detonate.
	 * It also creates debris if required.
	 * @param agressor 
	 */
	public void destroy(Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft.destroy() craft "+getName());
        }
        if (destroyed) {
            return;
        }
        BattleDestroyedEF destroyedEvent = getCraftDestroyedEvent(oldState);
        if (destructonCausesWreckage()) {
        	Wreckages wreckages = getBattle().getWreckages();
        	SalvageList salvage = SalvageFactory.create(this);
			WreckageCraft wreckage = new WreckageCraft(this, battle.getTurn(), salvage );
			wreckages.add(wreckage);
			WreckageDetails wd = WreckageDetailsFactory.create(wreckage);
			destroyedEvent.addStateUpdate(wd);
        }
        if (parent == null) {
        	MotEngine.instance.multicast( destroyedEvent, battle.getStartingUserPlayers() );
        } else {
        	parent.link(destroyedEvent);
        }
        destroyed = true;
        deleteAutomitonsTargetedAtMe(destroyedEvent);
        Triggers triggers = battle.getTriggers();
        if (triggers != null) {
        	triggers.removeAllWithTarget(this);
        }
        if (hasDetonator()) {
            WeaponInstanceDetonator.explode(this);
        }
        if (hasHoloEmiters()) {
            getBattle().destroyHoloCraftFor(this);
        }
        Craft craft = getCraft();
        craft.destroy();
		getBattle().remove(this);
		if (agressor != null && agressor instanceof UserPlayer && !isFighter && !(this instanceof HoloCraft) && !(this instanceof BattleStation)) {
			UserPlayer up = (UserPlayer) agressor;
			GameScoreData gsd = up.getGameScoreDataNotNull();
			gsd.addCraftDestroyed(craft.getHull(), 1);
		}
	}
    
	public void _destroy() throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft._destroy() craft "+getName());
        }
        if (destroyed) {
            return;
        }
        destroyed = true;
		getBattle().remove(this);
		getCraft().destroy();
	}
	
	
	/**
	 * This craft must have been previously destroyed otherwise this method 
	 * does not do anything.
	 * This craft is added to the battle as a new craft, but is not added to
	 * the players list of crafts so it will not exist after the battle.
	 * The player owner is in control of this craft after it has been
	 * resurrected. Its wreckage is removed from the battle field.
	 * @throws ApplicationError 
	 */
	public void resurect(Player owner) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft.resurect() craft "+getName()+" for player "+owner.getName());
        }
		if (!isDestroyed()) return;
		Wreckages wreckages = battle.getWreckages();
		WreckageCraft wreckage = wreckages.getWreckage(this);
		if (wreckage == null) return;
		Craft craft = getCraft();
		battle.addToCraftList(this);
		if (getOwner() != owner) {
			craft.setOwner(owner);
		}
		craft.setDamage(0);
		destroyed = false;
		rechargeShieldsFull();
		wreckages.remove(wreckage);
	}
	
    public BattleDestroyedEF getCraftDestroyedEvent(CraftStateFactory oldState) throws ApplicationError {
        return new BattleDestroyedEF(this, oldState, false);
    }
    
    /**
     * Creates a new battle craft, that is part of the same battle, positioned between location loc and this craft but 
     * as close to this craft as possible. The new BattleCraft has a new instance of craft that is NOT added to any players
     * lists so it only exists for the duration of this battle. 
     * @param owner The player who will own this new Craft
     * @param origin There is a bias towards placing the craft on the origin side of the targetLocation.
     * @param targetLocation The location to attempt to place the craft at.
     * @return A newly created BattleCraft
     * @throws ApplicationError 
     */
    public BattleCraft duplicate(Player owner, Location targetLocation, Location origin) throws ApplicationError {
    	Battle battle = getBattle();
		CraftDesign cd = new CraftDesignAdapter(getCraft());
        Craft craft = new Craft(owner, cd, battle.getOrbit());
        craft.setOrigin(getOwner());
        craft.arrivedAt(battle.getOrbit(), false);
        BattleCraft bc = battle.join(craft, false);
        
        int radius = bc.getImageRadius() + getImageRadius() + ImageObjectSettings.INTER_CRAFT_GAP;
        SquareBattleLayoutManager slm = new SquareBattleLayoutManager(battle, getLocation(), 0, radius);
        slm.setNextFreeLocation(bc);
        
        int direction = bc.getLocation().getDirectionFor(getLocation());
        bc.setDirection(direction);
    	return bc;
    }
    
	/**
	 * Moves the craft to a position on the battle map next to but not overlapping
     * to the craft passed in. It uses moveTo(Location).
	 */
	public boolean moveTo(BattleCraft craft) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft.moveTo() "+getName()+" => "+craft.getName());
        }
        
        //Location loc = getClosestLocationTo(craft);
        Location loc = craft.getLocation();
        Location cloc = craft.getLocation();
		return moveTo(new LocationAndDirection(loc, loc.getDirectionFor(cloc)), 1.0, cloc, craft, false);
    }
    
	/**
	 * Moves the craft to a position on the battle map next to but not overlapping
     * to the planet passed in. It uses moveTo(Location).
	 */
	public boolean  moveTo(BattlePlanet planet) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleCraft.moveTo() "+getName()+" => "+planet.getName());
        }
        
        //Location loc = getClosestLocationTo(planet);
        return moveTo(new LocationAndDirection(planet.getLocation()), 1.0, planet.getLocation(), planet, false);
    }
    
	public boolean _moveTo(Location dest) throws ApplicationError {
        return moveTo(new LocationAndDirection(dest), 1.0);
    }
	
	public boolean _moveTo(Location dest, double maxMovePercent) throws ApplicationError { 
        return moveTo(new LocationAndDirection(dest), maxMovePercent);
	}
	
	/**
	 * Rotates craft towards this direction then moves it to the closest empty square
	 * between the crafts current location and the dest location passed in. If there is
	 * enough movement it then turns the craft in the direction contained in dest.
	 * The turning cost of a craft can be calculated by
	 * (total degrees to turn) * (hull turning cost) / (engine turn power) / (player character turning bonus)
     * All of the crafts remaining move will be used to move to this location if the location
     * is out of range. Otherwise only the percentage of the move required to rotate the craft towards
     * dest and move there will be used.
     * @param dest The destination location to move this craft to or towards.
	 */
	public boolean moveTo(LocationAndDirection moveToDest) throws ApplicationError {
        return moveTo(moveToDest, 1.0);
    }
    
	
	public boolean moveTo(LocationAndDirection moveToDest, double maxMovePercent) throws ApplicationError { 
		return moveTo(moveToDest, maxMovePercent, null, null, false); 
	}
	
	/**
	 * Rotates craft towards this direction then moves it to the closest empty square
	 * between the crafts current location and this location.
	 * The craft will stop before its destination it the space is occupied or the craft hits
	 * am active incumbent - such as a mine field. If the craft hits a mine field it may continue
	 * moving by re-calling the move method!
	 * @return True if this craft can keep moving towards its destination and false otherwise.
     * @param moveToDest The destination location to move this craft to or towards.
     * @param maxMovePercent The maximum percentage of the crafts move that should be used.
	 */
	public boolean moveTo(LocationAndDirection moveToDest, double maxMovePercent, Location targetDest, ServerImageObject target, boolean isDryRun) throws ApplicationError {
        if (Logger.isDebug() && !isDryRun) {
            String percentStr = ""+maxMovePercent;
            if (percentStr.length() > 4) percentStr = percentStr.substring(0,5);
            Logger.logDebug("BattleCraft.moveTo("+percentStr+") "+getName()+" @ "+getLocation()+":"+direction+" => "+moveToDest+" targetDest = "+targetDest+" target = "+target);
        }
        
        // first we must do some obvious checks to make sure the player can move 
        // to the destination, otherwise it will activate mines it does not actually
        // reach while moving AND can point in the wrong direction while shifting!
        
		if ( !GalaxyDefinitions.BATTLE_BOUNDARY.contains(moveToDest) ) {
			moveToDest.setLocation(GalaxyDefinitions.BATTLE_BOUNDARY.getClosestEdgeTo(moveToDest));
		}
		
		Craft craft = getCraft();
		if (craft.getEngine() == null) {
			// Must be a battle station
			return false;
		}
		
		if (target == null) {			
			ServerImageObject occupier = battle.getBattleObjectAt(moveToDest.getX(), moveToDest.getY());
			if (occupier != null && occupier != this) {
				target = occupier;
				if (targetDest == null) {
					targetDest = target.getLocation();
				}
			}
		} 
		if (targetDest == null) {
			targetDest = new Location(moveToDest);
			if (moveToDest.getDirection() != null) {
				targetDest.shiftInDirection(moveToDest.getDirection(), 25);
			}
		}

		try {
			boolean r;
			if (craft.getEngine().isTeleport()) {
				r = _teleportTo(moveToDest, targetDest, target, isDryRun);
			} else {
				r = _moveTo(moveToDest, maxMovePercent, targetDest, target, isDryRun);
			}
			int speed = getSpeed();
	        if (moveRemaining < 0.01 || speed*moveRemaining < 2) {
	        	// With a lot of movement damage, a full move can be no move 
	        	setMoveRemaining(0.0);
	        }
			if (MotEngine.instance != null) {
				MotEngine.instance.getProxyManager().broadcastQueue();
			}
			return r;
		} catch (RuntimeException e) {
			if (MotEngine.instance != null) {
				MotEngine.instance.getProxyManager().broadcastQueue();
			}
			throw e;
		} catch (ApplicationError e) {
			if (MotEngine.instance != null) {
				MotEngine.instance.getProxyManager().broadcastQueue();
			}
			throw e;
		}
	}

	private boolean _teleportTo(Location moveToDest, Location targetDest, ServerImageObject target, boolean isDryRun) throws ApplicationError {
        if (Logger.isDebug() && !isDryRun) {
            String percentStr = ""+moveRemaining;
            if (percentStr.length() > 4) percentStr = percentStr.substring(0,5);
            Logger.logDebug("BattleCraft.teleportTo("+percentStr+") "+getName()+" @ "+getLocation()+":"+direction+" => "+moveToDest+" [ dist "+getLocation().getDistanceTo(moveToDest)+" speed "+getSpeed()+" ]");
        }
        
        CraftStateFactory originalState = getCraftState();
        
        double percentMoveToUse = Comp.min(moveRemaining, 1.0);
        if (percentMoveToUse < 1.0) {
        	setMoveRemaining(0.0);
        	return false;
        }

        BattleCraftTeleportEF event = null;
		if (!isDryRun) {
	        event = new BattleCraftTeleportEF(this, originalState, percentMoveToUse);
	        MotEngine.instance.getProxyManager().scheduleMulticast(event, battle.getStartingUserPlayers());
		}

		MovementCostCalculator mcc = new MovementCostCalculator(this, percentMoveToUse);
		Location destBeforeMove = moveToDest;
        Location dest = getClosestFreeSpaceTo(moveToDest, mcc, targetDest, target);
        
		int reqdDirection = direction;
		if (targetDest != null) {
			reqdDirection = getLocation().getDirectionFor(targetDest);
		} else {
			reqdDirection = getLocation().getDirectionFor(dest);
		}
		
		int distance = getLocation().getDistanceTo(dest);
		if (distance < 2) {
	        // Teleporting does not have any turning costs - get required direction automatically
			if (Comp.pos(direction-reqdDirection) < 2) {
	        	setMoveRemaining(0.0);
	        	if (!isDryRun) {
	        		MotEngine.instance.getProxyManager().deleteScheduledBroadcast(event);
	        	}
				return false;
			}
			direction = reqdDirection;
		} else {
			// Teleporting does not have any turning costs - get required direction automatically
			direction = reqdDirection;

			// Teleporting moves directly to the destination and uses all move to get there
			getLocation().shiftTo(dest);
			
			if (!destBeforeMove.equals(dest)) {
				// Teleporting does not have any turning costs - get required direction automatically
				if (targetDest != null) {
					direction = getLocation().getDirectionFor(targetDest);
				}
			}
		}
		
		if (!isDryRun) {
			setMoveRemaining(0.0);
			CraftStateFactory newState = getCraftState();
			event.setNewState(newState);
	        MotEngine.instance.getProxyManager().sendScheduledEvent(event);
	
	        activateIncumbentsAt(dest);
		}
		
        return false;
	}

	public void activateIncumbentsAt(Location dest) throws ApplicationError {
		Incumbent[] incumbents = battle.getIncumbents().getIncumbents(dest);
        if (incumbents != null) {
        	for (int i = 0; i < incumbents.length; i++) {
				incumbents[i].activate(this);
			}
        }
	}
	
	private boolean _moveTo(Location moveToDest, double maxMovePercent, Location targetDest, ServerImageObject target, boolean isDryRun) throws ApplicationError {
        double originalMoveRemaining = moveRemaining;
        double percentMoveRemainingToUse = Comp.min(moveRemaining, maxMovePercent);
        
        Location start = getLocationClone();

        Location dest = new Location(moveToDest);
		MovementCostCalculator mcc = new MovementCostCalculator(this, percentMoveRemainingToUse);
        dest = getClosestFreeSpaceTo(dest, mcc, targetDest, target);
		double speed = getSpeed();
        boolean willShift = (getLocation().getDistanceTo(dest) > 1) && speed > 0;

        if (Logger.isDebug() && !isDryRun) {
            String percentStr = ""+maxMovePercent;
            if (percentStr.length() > 4) percentStr = percentStr.substring(0,5);
            Logger.logDebug("BattleCraft.moveTo("+percentStr+") "+getName()+" @ "+getLocation()+":"+direction+" => "+dest+" [ dist "+getLocation().getDistanceTo(dest)+" speed "+getSpeed()+" ] actual="+dest);
        }

		boolean canKeepMoving = false;
        
        // Schedule the move event so that happens first
        BattleCraftMoveEF event = null;
		if (!isDryRun) {
	        event = new BattleCraftMoveEF(this, null, null);
	        MotEngine.instance.getProxyManager().scheduleMulticast(event, battle.getStartingUserPlayers());
		}
        
        // Must stop the movement of this craft where a moving automaton will strike
        MovementLinkStrike automatonStrike = null;
        if (!isDryRun) {
        	automatonStrike = getPercentMoveBeforeAutomatonStrike();
        	if (automatonStrike != null && automatonStrike.getMovePercent() < percentMoveRemainingToUse) {
    			canKeepMoving = true;
        		percentMoveRemainingToUse = automatonStrike.getMovePercent();
                if (Logger.isDebug()) {
                    String percentStr = ""+percentMoveRemainingToUse;
                    if (percentStr.length() > 4) percentStr = percentStr.substring(0,4);
                    Logger.logDebug("BattleCraft.moveTo() Reduce MaxMovePercent to "+percentMoveRemainingToUse+" due to strike by "+automatonStrike.getAutomaton());
                }
        	} else {
        		automatonStrike = null;
        	}
        }

		ArrayList movements = new ArrayList();
		
        // First turn in the direction in which we shall move
		// This is to avoid craft moving while pointing in another direction.
        if (willShift) {
        	ObjectTurn turn1 = _doTurn(dest, percentMoveRemainingToUse, isDryRun);
        	if (turn1 != null) {
				movements.add(turn1);
		        percentMoveRemainingToUse = percentMoveRemainingToUse - turn1.getMovePercent();
				if (percentMoveRemainingToUse < 0.001) percentMoveRemainingToUse = 0.0;
        	}
        }

		// Now move towards the destination
		
		IncumbentEdge ie = null;
		
        if (willShift) {
            ie = battle.getIncumbents().getFirstActiveEncumbent(this, getLocation(), dest);
    		if (ie != null) {
    			if (Logger.isDebug()) {
    				Logger.logDebug("BattleCraft.moveTo() stop craft at "+ie.getLoc()+" due to incumbent "+ie.getIncumbant());
    			}
    			dest = ie.getLoc();
    			canKeepMoving = true;
    		}
    		
    		if (!getLocation().equals(dest)) {
    			if (percentMoveRemainingToUse > 0.0) {
    				// We can stop half way to our destination because distance is too great or incumbents slowed us down
    				// We must make sure we do not stop on top of another object in the battle field.
    		        int maxDist = (int) (speed * percentMoveRemainingToUse);
    		        mcc = new MovementCostCalculator(this, percentMoveRemainingToUse);
    				dest = getClosestFreeSpaceTo_NO_DIR_CHANGE(dest, maxDist);
    				percentMoveRemainingToUse = _doShifts(dest, movements, percentMoveRemainingToUse, isDryRun);
    			}
    		}
        }

        // Turn pointing towards the target to which we were asked to move
		// This ensures that weapons can fire as craft is pointing towards the target
        ObjectTurn turn2 = null;
        if (targetDest != null) {
        	turn2 = _doTurn(targetDest, percentMoveRemainingToUse, isDryRun);
        }
		if (turn2 != null) {
			movements.add(turn2);
		}
		
		if (!isDryRun) {
			if (movements.size() > 0) {
				ObjectMovement [] ma = (ObjectMovement []) movements.toArray(new ObjectMovement [movements.size()]);
				event.setMovements(ma, getCraftState());
		        moveLinkedObjects(originalMoveRemaining-moveRemaining, event, automatonStrike);
		        MotEngine.instance.getProxyManager().sendScheduledEvent(event);
			} else {
				MotEngine.instance.getProxyManager().deleteScheduledBroadcast(event);
			}
		}
		
		if (ie != null && ie.getIncumbant().inside(getLocation()) && !isDryRun) {
			ie.getIncumbant().activate(this);
			if (isDestroyed()) {
				canKeepMoving = false;
			}
		}
		
		return canKeepMoving && !isDestroyed();
	}

	private ObjectTurn _doTurn(Location dest, double percentMoveToUse, boolean isDryRun) throws ApplicationError {
		
		if (percentMoveToUse < 0.001) {
			return null;
		}
		
		CraftState originalState = getCraftStateForGenericPlayer();
        double originalMoveRemaining = moveRemaining;

        // First rotate craft to destination
        int reqdDirection = getLocation().getDirectionFor(direction, dest);
		double reqdDirectionChange = Comp.getDirectionChangeFor(direction, reqdDirection);

		int turningSpeed = getTurningSpeed();
        
		if (Comp.pos(reqdDirectionChange) >= turningSpeed*percentMoveToUse) {
			// use all movement points turning craft as much as we can
            double directionChange = (reqdDirectionChange < 0) ? -turningSpeed : turningSpeed;
			direction += directionChange*percentMoveToUse;
            moveRemaining -= percentMoveToUse;
		} else {
            // use required movement points turning craft to desired direction
            direction = reqdDirection;
            moveRemaining -= Comp.pos(reqdDirectionChange) / turningSpeed;
        }

        if (direction < 0) direction += 360;
        if (direction > 360) direction -= 360;
        if (moveRemaining < 0.0) moveRemaining = 0.0;

        // You can get rounding errors so make sure the turn is worth making.
		if (Comp.pos(reqdDirectionChange) > 2) {
	        double percentTurn = originalMoveRemaining - moveRemaining;
			return new ObjectTurn(direction, percentTurn, originalState);
		} else {
			return null;
		}
	}
	
    private double _doShifts(Location dest, ArrayList movements, double percentMoveToUse, boolean isDryRun) throws ApplicationError {
    	Location loc = getLocation();
		IncumbentEdge [] ies = battle.getIncumbents().getMovementIncumbentEdgeList(loc, dest);
		
		double speed = getSpeed();
        int maxDist = (int) (speed * percentMoveToUse);
		Location originalLoc = new Location(loc);
		
		if (ies != null && ies.length > 0) {
			Location last = originalLoc;
			for (int j=0; j<ies.length && percentMoveToUse > 0.0 && !isDestroyed(); j++) {
				Incumbent incumbent = ies[j].getIncumbant(); 
				if (!incumbent.inside(getLocation())) incumbent = null;
				ObjectShift movement = _doShift(last, ies[j].getLoc(), speed, percentMoveToUse, incumbent, isDryRun);
				if (movement != null) {
					movements.add(movement);
					percentMoveToUse -= movement.getMovePercent();
					last = movement.getDestination();
				}
			}
			if (percentMoveToUse > 0.0 && !isDestroyed()) {
				Incumbent incumbentInside = battle.getIncumbents().getIncumbentInside(dest);
//				Incumbent incumbent = ies[ies.length-1].getIncumbant(); 
//				if (!incumbent.inside(getLocation())) incumbent = null;
				ObjectShift movement = _doShift(last, dest, speed, percentMoveToUse, incumbentInside, isDryRun);
				if (movement != null) {
					movements.add(movement);
					percentMoveToUse -= movement.getMovePercent();
				}
			}
		} else {
			Incumbent incumbentInside = battle.getIncumbents().getIncumbentInside(loc);
			ObjectShift movement = _doShift(originalLoc, dest, speed, percentMoveToUse, incumbentInside, isDryRun);
			if (movement == null) {
				// craft has been destroyed
				return 0.0;
			}
			movements.add(movement);
			percentMoveToUse -= movement.getMovePercent();
		}
		return percentMoveToUse;
    }

    private ObjectShift _doShift(Location from, Location to, double speed, double percentMoveToUse, Incumbent incumbent, boolean isDryRun) throws ApplicationError {
    	CraftStateFactory originalCraftState = getCraftState();
    	Damage craftDamage = null;
		
        if (incumbent != null) {
            if (incumbent.getMovementDamageEffect() > 0.0 && !isDryRun) {
            	double damage = getCraft().getHull().getMaxDamage() * incumbent.getMovementDamageEffect();
                double damageBlockedByShields = WeaponInstance.blockedDamageByShields(getShields(), damage, 1);
                damage -= damageBlockedByShields;
        		BattleDamageEF de = new BattleDamageEF(this);
                MotEngine.instance.getProxyManager().scheduleMulticast(de, battle.getStartingUserPlayers());
                damage(damage, null, originalCraftState, null);
                String damageDescription = Formatter.getDamageDescription(damage,	damageBlockedByShields);
                craftDamage = new Damage(damageDescription, damage, damageBlockedByShields, isDestroyed(), originalCraftState.getCraftStateForGenericPlayer(), getCraftStateForGenericPlayer());
                if (isDestroyed()) {
	                de.setDamage(damageDescription, damage, damageBlockedByShields, isDestroyed(), originalCraftState, getCraftState());
	                MotEngine.instance.getProxyManager().sendScheduledEvent(de);
	            	return null;
                } else {
                	MotEngine.instance.getProxyManager().deleteScheduledBroadcast(de);
                }
            }
        }
        
        int maxDist = (incumbent == null) ? (int)(speed * percentMoveToUse)
        		                          : (int)(speed * percentMoveToUse * incumbent.getMovementEffect());
        int fullMoveMaxDist = (incumbent == null) ? (int)speed
                	           					  : (int)(speed * incumbent.getMovementEffect());

        // If the target location is occupied then get as close as we can to it
        // NB This can happen because incumbents slow the craft down causing the final location to change.
        Location newTo = getClosestFreeSpaceTo_NO_DIR_CHANGE(to, maxDist);

        double actualDistance = shift(newTo, maxDist);
        double percent = actualDistance / fullMoveMaxDist;
        // This line prevents rounding errors effecting the results
        if (percentMoveToUse - percent < 0.011) percent = percentMoveToUse;
        moveRemaining -= percent;
        if (moveRemaining <= 0.0) moveRemaining = 0.0;
        
        ObjectShift shift = new ObjectShift(getLocation(), percent, originalCraftState.getCraftStateForGenericPlayer());
        if (incumbent != null) {
        	shift.setSpeedModification(incumbent.getMovementEffect());
            if (craftDamage != null) {
                shift.setDamage(craftDamage);
            }
        }
        
        return shift;
    }

	/**
	 * @return If this craft was asked to move to location moveToDest - this method returns the location that it would actually move to.
	 * @throws ApplicationError 
	 */
	public LocationAndDirection getCraftMoveActualLocation(LocationAndDirection moveToDest, Location targetDest, ServerImageObject target) throws ApplicationError	{
		int oldDirection = direction;
		Location oldLocation = getLocationClone();
		double oldMoveRemaining = moveRemaining;
		
		moveTo(moveToDest, moveRemaining, targetDest, target, true);
		LocationAndDirection r = new LocationAndDirection(getLocation(), direction);
		
		direction = oldDirection;
		setLocation(oldLocation);
		moveRemaining = oldMoveRemaining;
		return r;
	}
	

	/**
	 * This craft surrenders in the battle to the owner passed in.
	 * This can be caused by a Psi attack or invading a craft with troops.
	 */
	public void surrender(Player newOwner, boolean sendEvent) throws ApplicationError {
        setMoveRemaining(0);
        // Must check to make sure this is not destroyed as ending my move will move linked automatons and may destroy me
        if (isDestroyed()) {
			if (Logger.isDebug()) {
				Logger.logDebug("BattleCraft.surrender() "+getName()+" destroyed before surrender");
			}
        } else {
	        Player oldOwner = getCraft().getOwner();
	        CraftStateFactory oldState = getCraftState();
			Craft craft = getCraft();
			craft.surrender(newOwner);
			battle.removeFromWaitList(this);
			if (sendEvent) {
		        CraftStateFactory newState = getCraftState();
		        BattleCraftSurrenderEF event = new BattleCraftSurrenderEF(this, oldState, newState);
				MotEngine.instance.multicast( event, battle.getStartingUserPlayers() );
			}
			if (newOwner instanceof UserPlayer && !isFighter && !(this instanceof HoloCraft) && !(this instanceof BattleStation)) {
				UserPlayer up = (UserPlayer) newOwner;
				GameScoreData gsd = up.getGameScoreDataNotNull();
				gsd.addCraftInvaded(craft.getHull(), 1);
			}
        }
	}

	/**
	 * Returns true if an only if this craft has bean destroyed (by weapons or anything else).
	 */
	public boolean isDestroyed() {
		return destroyed;
	}
	
    /**
     * @throws ApplicationError 
     * @returns The shield charge as a percent of the max charge value. The value is between 0 and 1.
     */
    public double getShieldAsPercent() throws ApplicationError {
        if (shields == null) return 0.0;
        double maxPower = 0.0;
        for (int i=0; i<shields.length; i++) {
            maxPower += shields[i].getMaxShieldCharge();
        }
        double power = 0.0;
        for (int i=0; i<shields.length; i++) {
            power += shields[i].getPower();
        }
        double r = power / maxPower;
        if (r > 1.0) r = 1.0;
        return r;
    }

    /**
     * @returns The craft damage as a percent of the max damage. The value is between 0 and 1.
     */
    public double getDamageAsPercent() throws ApplicationError {
        double maxDamage = getCraft().getMaxDamage();
        double damage = getCraft().getDamage();
        double r = damage / maxDamage;
        if (r > 1.0) r = 1.0;
        return r;
    }

    public double getDamageRequiredToDestroy() throws ApplicationError {
        double maxDamage = getCraft().getMaxDamage();
        double damage = getCraft().getDamage();
        double r = maxDamage - damage;
        return r;
    }

    /**
     * @returns The craft movement damage as a percent of the max damage. The value is between 0 and 1.
     */
    public double getMovementDamageAsPercent() throws ApplicationError {
        double maxDamage = getSpeed(false);
        double r = movementDamage / maxDamage;
        if (r > 1.0) r = 1.0;
        return r;
    }

    /**
     * @returns The craft psi damage as a percent of the max damage. The value is between 0 and 1.
     */
    public double getPsiDamageAsPercent() {
        double maxDamage = Biotechnology.PSI_MAX_RETREAT_RATING;
        double r = psiDamage / maxDamage;
        if (r > 1.0) r = 1.0;
        return r;
    }

    /**
     * @returns The crafts current crew as a percent of the max crew this craft holds. The value is between 0 and 1.
     */
    public double getCrewAsPercent() {
        double maxCrew = getCraft().getHull().getCrew();
        double r = crew / maxCrew;
        if (r > 1.0) r = 1.0;
        return r;
    }
    
    /**
     * Issues the command for this craft to attack the enemy craft bc passed in.
     * If the craft is within range, the craft will fire all weapons at the enemy craft.
     * If the enemy is not in range of any weapon, a validation error will be issued.
     * @param enemy The enemy object to attack
     * @param weapon The weapon the source craft is to fire at the target. If this is null 
     * all weapons are fired.
     * @param qtyToFire The The quantity of weapon the source craft is to fire at the target.
     * If weapon is null this is ignored
     * @param excludeGroups This is a list of all weapon groups that should not be fired when the user
	 * does not specify a weapon to attack another craft using. This is important because some weapons 
	 * groups can also damage the craft that is firing the weapon, e.g. Droid Mine, Special 
	 * Distortion etc.
     * @throws mot.common.util.ApplicationError
     */
    public void attack(ServerImageObject target, WeaponInstance weapon, int qtyToFire) throws ApplicationError {
        if (inWeaponRange(target.getLocation())) {
            if (hasWeaponsToFire()) {
                if (target instanceof BattleCraft) {
                    BattleCraft bc = (BattleCraft) target;
                    fireWeapons(bc, weapon, qtyToFire);
                } else if (target instanceof BattleObject) {
                    BattleObject bo = (BattleObject) target;
                    fireWeapons(bo, weapon, qtyToFire);
                }
            }
        }
    }
    
    
	/**
	 * Fires this crafts weapons at craft target object.
     * 
	 * This method calculates the psi difference and performs miss turn or surrender
	 * operation that may take place as a result of psi effects. See biotechnology
	 * class for details.
     * 
     * If the craft has not missed a turn or the enemy surrendered then is calls
     * BattleObject.fireAllWeapons(target). For more details read that method.
     * 
     * @param target The craft to attack
     * @param weapon If null all weapons that have not been fired and are in range fire.
     * If weapon is not null only this weapon is fired.
	 */
	public boolean fireWeapons(BattleCraft target, WeaponInstance weapon, int qtyToFire) throws ApplicationError {
        if (isDestroyed()) return false;

        int psiDamageSource = 0;
        int psiDamageTarget = 0;

        if (!firedAWeaponThisTurn) {
        	firedAWeaponThisTurn = true;
        	int targetPsi = target.getPsiLevel();
        	int craftPsi = getPsiLevel();
        	int psiDifference = targetPsi - craftPsi;
        	
        	psiDamageSource = (int)(psiBlockShields(psiDifference) + psiDamage);
        	psiDamageTarget = (int)(target.psiBlockShields(-psiDifference) + target.getPsiDamage());
        	
        	if (Logger.isDebug()) {
        		Logger.logDebug("BattleCraft.fireWeapons() "+getName()+" target="+target.getName()+" psi diff="+(-psiDifference)+" Source PsiDam="+psiDamageSource+" Target PsiDam="+psiDamageTarget);
        	}
        }
        
        
		if (psiDamageSource >= Biotechnology.PSI_MISS_TURN_LEVEL) {
			// Attacker may loose nerve to attack
			int r = Utils.random(Biotechnology.PSI_MAX_MISS_TURN_RATING);
			if (r < psiDamageSource) {
                MotEngine.instance.multicast( new BattleFearEF(this, getLocation(), target), battle.getStartingUserPlayers() );
				dischargeWeapons();
                setMoveRemaining(0);
                return false;
			}
		}

        boolean firedAWeapon = super.fireWeapons(target, weapon, qtyToFire);
        
        if (firedAWeapon && psiDamageTarget >= Biotechnology.PSI_SURRENDER_RATING && !(target.getOwner() instanceof MonsterPlayer) && !target.isDestroyed()) {
			// Defender may surrender
			int r = Utils.random(Biotechnology.PSI_MAX_SURRENDER_RATING);
			if (r < (psiDamageTarget-Biotechnology.PSI_SURRENDER_RATING)) {
				if (Logger.isDebug()) {
					Logger.logDebug("BattleCraft.fireWeapons() Target "+target.getName()+" surrenders psi="+psiDamageTarget);
				}
				target.surrender(getOwner(), true);
                return firedAWeapon;
			}
		}

        return firedAWeapon;
	}

    public boolean isTargetAngleWithinWeaponFiringAngleOfAllWeapons(BattleObject target) {
    	if (weapons == null) return false;
    	for (int i = 0; i < weapons.length; i++) {
			if (!isTargetAngleWithinWeaponFiringAngle(weapons[i], target)) {
				return false;
			}
		}
    	return true;
    }
    
	/**
	 *@return True if the craft is pointing in a direction that allows this weapon to fire at this target.
	 */
    public boolean isTargetAngleWithinWeaponFiringAngle(WeaponInstance weapon, BattleObject target) {
    	WeaponBayDetails wbd = weapon.getWeaponBayDetails();
		if (wbd != null) {
			int targetDirection = getLocation().getDirectionFor(target.getLocation());
			double reqdDirectionChange = Comp.getDirectionChangeFor(direction, targetDirection);
			if (reqdDirectionChange < 0) reqdDirectionChange = -reqdDirectionChange;
			if (reqdDirectionChange > wbd.getMaxFireAngle()) {
				return false;
			}
    	}
		return true;
	}

	
    /**
     * @param psiPower The amount of psi attack directed at this craft
     * @return The amount of psi attack directed at this craft that gets through the shields.
     */
    public double psiBlockShields(double psiPower) {
        if (psiPower <= 0) {
            return 0;
        }
        if (!hasShields()) {
        	// If in nebula, we have no shields, and so should not block psi
        	return psiPower;
        }
        Shields shields = getCraft().getShields();
        if (shields != null && shields.getPsiRating() > 0.0) {
            return psiPower * (1.0 - shields.getPsiRating());
        } else {
            return psiPower;
        }
        
    }

    /**
     * The manouverability advantage is limited to 2.0. Battle Station have no manoverability
     * so they always take twice the damage because they always have the maximum (dis)advantage.
     * @return The manouverability advantage this object has over the target object.
     */
	public double getManouverabilityAdvantage(BattleObject target) throws ApplicationError {
        if (target instanceof BattleCraft) {
            BattleCraft bc = (BattleCraft) target;
            double manv1 = (double) getManoeuvrability();
            double manv2 = (double) bc.getManoeuvrability();
            if (manv2 < manv1/2) manv2 = manv1/2;
            if (manv1 < manv2/2) manv1 = manv2/2;
            double manvAdvantage = manv1 / manv2;
            return manvAdvantage;
        } else {
            return 1.0;
        }
    }
    	
    /**
     * @return The specials owned by this object.
     */
	public Specials getSpecials() { return getCraft().getSpecials(); }

    /**
     * @return the weapons range bonus enjoyed by this craft. 
     */
    public double getWeaponRangeBonus() {
        double rangeBonus = 1.0;
        Specials specials = getCraft().getSpecials();
        if (specials != null) {
            rangeBonus += specials.getRangeModification();
        }
		return rangeBonus;
    }
    
    /**
     * Returns true if the craft passed in is within weapons range of all weapons on
     * this craft.
     */
    public boolean allInWeaponRange(Location loc) throws ApplicationError {
        if (weapons == null) {
            return false;
        }
		int distance = getLocation().getDistanceTo( loc );
        long minDistance = 0;
        for (int i=0; i<weapons.length; i++) {
            long tmpMaxDistance = weapons[i].getRange();
            if (tmpMaxDistance < minDistance) {
                minDistance = tmpMaxDistance;
            }
        }
        double rangeBonus = 1.0;
        Specials specials = getCraft().getSpecials();
        if (specials != null) {
            rangeBonus += specials.getRangeModification();
        }
		return distance < (minDistance*rangeBonus);
    }
    
    private static final int MAX_TRANSPORT_DISTANCE = 100;
    
    /**
     * Troop are used to invade an enemies crafts and defend against craft invasion.
     * @return True if this craft contains specials to carry troops.
     */
    public boolean hasTroopTransports() {
        WeaponInstance[] wa = this.getWeapons();
        if (wa == null) return false;
        for (int i = 0; i < wa.length; i++) {
			if (wa[i].getWeapon().isTroopTransport()) {
				return true;
			}
		}
        return false;
    }
    
    /**
     * This craft gets is next orders from the player in this battle.
     * For user players an event is sent to the user interface asking for orders.
     * For computer players new orders are set on the battle craft directly.
     */
    public void getNextOrders() throws ApplicationError {
    	if (controllerDetails != null) {
    		controllerDetails.setNextBattleOrders();
    	} else {
            getCraft().getOwner().setNextBattleOrders(this);
    	}
    }
    
    /**
     * Returns true if and only if this craft's image displayed on the battle map is 
     * larger than the image for battle craft other passed in. See graphics_race.xml 
     * for details of the size of images used to render crafts.
     */
    public boolean isLarger(BattleCraft other) throws ApplicationError {
        Size sizeThis = getCraft().getCraftGraphics().getSize();
        Size sizeOther = other.getCraft().getCraftGraphics().getSize();
        return sizeThis.isLarger(sizeOther);
    }
    
    /**
     * ONLY USED IN TEST HARNESS
     * Removes all existing weapons on the battle craft and adds the weapon passed
     * in as the crafts only weapon - ready to fire!
     * @throws ApplicationError 
     */
    public void _replaceAllWeaponsWith(Weapon w, boolean heavy, boolean infinateShots) throws ApplicationError {
    	int bayIndex = 0;
        WeaponBay bay = getCraft().getWeaponBays().getBaysAsArray()[bayIndex];
		Weapons weapons1 = bay.getWeapons();
        weapons1.removeAll();
        weapons1.add(w);
        setWeapons(weapons1, heavy, infinateShots, getWeaponBayDetailsForIndex(bayIndex));
    }

    
    protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("attacker", attacker);
		writer.addElement("move-remaining", moveRemaining);
		writer.addElement("direction", direction);
		writer.addElement("destroyed", destroyed);
		if (craftReference != null) {
			craftReference.toXML(writer);
		}
	}

	protected String getElementName() { return "battle-craft"; }
    
    public String toString() { return getName()+"@"+getLocation(); }

    public void setMovementDamage(int v) throws ApplicationError {
        movementDamage = v;
        int speed = getSpeed(false);
        if (movementDamage >= speed) {
            movementDamage = speed;
        }
    }


    public int getMovementDamage() {
        return movementDamage;
    }


    public void setPsiDamage(double psiDamage) {
        this.psiDamage = psiDamage;
    }


    public double getPsiDamage() {
        return psiDamage;
    }


    public void setCrew(int crew) throws ApplicationError {
    	boolean wasZero = (this.crew == 0);
        this.crew = crew;
        if (wasZero || crew == 0) createOrUpdateDecotatorList();
    }


    public int getCrew() {
        return crew;
    }

	public void setFighter(boolean isFighter) {
		this.isFighter = isFighter;
	}

	public boolean isFighter() {
		return isFighter;
	}

	public boolean isSuicideFighter() {
		if (!hasDetonator()) return false;
		WeaponInstance[] wa = getWeapons();
		if (wa == null) return true;
		return (wa.length == 1) || (wa.length == 2 && hasDroadMine());
	}


	public Integer getRetreatTurn() {
		return retreatTurn;
	}

	public boolean isOrbital() {
		return getCraft().getHull().isStation();
	}

}