/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.Decorator;
import mot.common.util.ApplicationError;
import mot.engine.players.Player;

/**
 * This represents the state a craft may find its self in.
 * Example statuses are
 * 		Stasis
 * 		Mind Control
 * Read about each for details of how each status effects a 
 * craft's operations.
 * 
 * Note - The Player object that ownes a craft must check its 
 * status BEFORE giving it a command - the battle craft its self
 * when told to move simply moves - but when a craft has status 
 * mind control the owner should not ask the craft to move. 
 *  		
 */
public interface BattleCraftController {

	/**
	 * Give orders to the craft bc in the battle.
	 */
	void setNextBattleOrders() throws ApplicationError;

	/**
	 * Control is automated if the controlling player is a computer player OR
	 * the implementation of this interface will controll the craft directly.
	 * The setNextBattleOrders() method should only be called if this returns
	 * TRUE!
	 * @return True if and only is the control of crafts using this is automated.
	 */
    boolean controlIsAutomated() throws ApplicationError;
    
    /**
     * @return True if crafts controlled by this controller can be commanded by player p.
     */
    boolean canBeComandedBy(Player p);
    
    /**
     * A Null value imples this controller will control the craft directly.
     * @return The name of player that is in control of the craft (if any). 
     * MAY NOT BE THE OWNER AND MAY BE NULL
     */
    String getControllerName();
    
    /**
     * A Null value imples this controller will control the craft directly
     * @return The player that is in control of the craft (if any). 
     * MAY NOT BE THE OWNER AND MAY BE NULL
     */
    Player getController();
    
    /**
     * @return The decorator used in a battle to display above crafts that are controlled by 
     * this controller. 
     */
    Decorator getDecorator();

}
