/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.battles.incumbants.IncumbentFactory;
import mot.engine.battles.incumbants.PerimeterAlarm;
import mot.engine.battles.incumbants.PerimeterNotification;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.event.MotLinkedEventFactory;
import mot.engine.players.Player;
import mot.engine.technology.WeaponInstanceDetonator;

/**
 * A mine is a battle object containing a detonator detonator weapon (see 
 * WeaponInstanceDetonator for details). It uses a PerimeterAlarm incumbent to
 * stop crafts when they enter the mines activation region and explodes the
 * detonator damaging all crafts close by.
 * 
 * A Mine explodes by any craft comes within its active region but is visibly
 * to its owner. It is not visible to any other player.
 */
public class BattleMine extends Automaton implements PerimeterNotification {
	
	private static final int DAMAGE_TO_DESTROY = 10;

	/**
	 * The damage this object has sustained
	 */
	private double damage;
	
	/**
	 * This is the incumbent on the battle field that defines this mine's
	 * sensor area to activate if any craft enters.
	 */
	private PerimeterAlarm alarm;
	
	/**
	 * This is the player that created the mine.
	 * This is the only player that can see the mine in the mine field!
	 */
	private Player owner;
	
	private boolean destroyed = false;
	
	private static final Size SIZE = new Size(15,15);
	
	public BattleMine(Player owner, Battle battle, String name, Location loc, int range) throws ApplicationError {
		super(battle, name, loc, null);
		this.owner = owner;
		this.alarm = IncumbentFactory.createPerimeterAlarm(this, battle, loc, range);
	}

	public WeaponDamage damage(double amount, Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError { 
        double damageToDestroy = getDamageToDestroy();
        damage += amount;
        if (Logger.isDebug()) {
            Logger.logDebug("BattleMine.damage() "+getName()+" amount="+amount+" (total Damage "+damage+")");
        }
        if (damage >= damageToDestroy) {
            destroy(null, oldState, parent);
        }
        return null;
	}

	private double getDamageToDestroy() {
		return DAMAGE_TO_DESTROY;
	}

	public void destroy(Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleMine.destroy() mine "+getName());
        }
		getBattle().remove(this);
		getBattle().getIncumbents().remove(alarm);
		WeaponInstanceDetonator.explode(this);
		destroyed = true;
	}

	public boolean isDestroyed() {
		return destroyed;
	}
	
	public Player getOriginPlayer() throws ApplicationError {
		return owner;
	}

	public Player getOwner() throws ApplicationError {
		return owner;
	}

	public boolean isOwnedBy(Player owner) throws ApplicationError {
		return (this.owner == owner);
	}

	public int getDirection() {
		return 0;
	}

	public Size getSize() throws ApplicationError {
		return SIZE;
	}

	protected String getElementName() {
		return "mine";
	}

	protected void performMove(double percent, MotLinkedEventFactory rootEvent) throws ApplicationError {
	}

	public double getPercentMoveBeforeStrike() throws ApplicationError {
		return 1.0;
	}
	
    public double getMorale() {
		return 0.0;
	}
    
    public double getCondition() {
		return 1.0;
	}
    
	
	/**
	 * Called when craft has activated the mine. This should cause the mine to 
	 * explode.
	 */
	public void perimeterNotification(BattleCraft craft) throws ApplicationError {
		if (Logger.isDebug()) {
			Logger.logDebug("BattleMine.perimeterNotification() craft "+craft.getName()+" activates "+getName());
		}
		destroy(null, getCraftState(), null);
	}

	public PerimeterAlarm getAlarm() {
		return alarm;
	}

	public void setAlarm(PerimeterAlarm alarm) {
		this.alarm = alarm;
	}

	public void setLocation(Location v) {
		alarm.setCenter(v);
		super.setLocation(v); 
	}

	/**
	 * A mine is only visible to its owner
	 */
    public boolean isVisible(Player player) throws ApplicationError {
		return (player == getOwner());
	}

	public void strike(MotLinkedEventFactory rootEvent) throws ApplicationError {
	}

	
}
