/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import java.util.ArrayList;

import mot.common.bean.CraftState;
import mot.common.bean.DecoratorList;
import mot.common.bean.Location;
import mot.common.bean.PlayerColor;
import mot.common.bean.SalvageList;
import mot.common.technology.Shield;
import mot.common.technology.ShieldSpan;
import mot.common.technology.Shields;
import mot.common.technology.Specials;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.ServerImageObject;
import mot.engine.crafts.WeaponBay;
import mot.engine.crafts.WeaponBays;
import mot.engine.event.BattleDestroyedEF;
import mot.engine.event.BattleWeaponMissEF;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.event.MotLinkedEventFactory;
import mot.engine.players.Player;
import mot.engine.technology.ShieldInstance;
import mot.engine.technology.WeaponInstance;
import mot.engine.technology.WeaponInstanceFactory;
import mot.engine.technology.WeaponInstanceTimeWarp;

/**
 * This is an object on the battle field. Objects in a battle are the targets of weapons 
 * fire and usually contain their own weapons and shields. Examples are
 * 	Planets
 * 	Crafts
 * 	Automatons (Missiles / Mines)
 * 
 * Attributes of a battle field such as incumbents ARE NOT battle objects. Battle Objects
 * are owned by a player.
 */
abstract public class BattleObject extends ServerImageObject {

    /**
     * This is the percentage of weapon fires that miss a craft that is not visible (cloaked).
     */
    private static final double CHANCE_TO_MISS_WHEN_CLOAKED = 0.5;
    
     /**
      * This is an instance of a weapon in the battle.
      * Weapons have state during a battle on a craft by craft basis (such as hasFired). 
      * Therefore we need an instance of a class around Weapon to keep this extra state.
      */
    protected WeaponInstance[] weapons;

    /**
     * A battle craft must keep a track of the shield charge. This is because through the
     * process of battle the charge may go up and down.
     */
    protected ShieldInstance[] shields;

    /**
     * The battle this object is part of.
     */
    protected Battle battle;
    
    /**
     * This is a list of MoveLinkedObject objects whose move is linked to this battle object.
     * If this battle object moves 0.8 percent of its move then all linked automatons will
     * move 0.8 percent of their move at the same time.
     */
    private ArrayList moveLinks;
    
    /**
     * This is dynamically generated from moveLinks and acts as a cache.
     */
    private transient MoveLinkedObject[] moveLinksArray;

    /**
     * This is the mood the crew are in. It is a value between -1.0 and +1.0 where -1.0 is
     * a very low morale and +1.0 is a very high morale. Crew perform a better job of manning
     * a craft if they have good morale. 
     * @see setup() for details on the value morale is set to. 
     */
    private double morale;
    
    /**
     * This is the list of decorators that are on this craft at this point in time.
     * Any object that changes the state of a battle craft must remember to change the decorator 
     * list.
     */
    private ServerDecoratorList decorators = null;
    

    /**
     * This is a list of technology that this craft has salvaged from this battle field.
     * NB - Salvage comes from the wreckage of other craft that have been destroyed. See
     * class Wreckages for more details.
     */
    private SalvageList salvage;

    /**
     * If a commander has instructed this craft then it may create an exclusion zone so that other crafts
     * try not to attack items at this location.
     * Weapons like the detonator are dangerous to other friendly craft.
     */
    private ExclusionZone exclusionZone = null;
    
    public BattleObject(Battle battle) {
        this.battle = battle;
    }

    public BattleObject(Battle battle, String name, Location loc) {
        super(name, loc);
        this.battle = battle;
    }

	/**
	 * This should be called before the battle actually starts but after all crafts / 
	 * planets etc have been created. A craft may use this to perform any initialisation 
	 * it desires.
	 * 
	 * This method sets the craft morale. 
	 * The morale is 0 if no players or all plays have an active news service 
	 * The morale is -1 if only enemy players have an active news service
	 * The morale is +1 if only this player has an active news service
	 */
	public void setup() throws ApplicationError {
		Player owner = getOwner();
		double myInfluence = battle.getInfluence(owner);
		double enemyInfluence = 0.0;
		Player [] players = battle.getPlayers();
		for (int i=0; i<players.length; i++) {
			if (players[i] != null && players[i].isEnemy(owner)) {
				enemyInfluence = battle.getInfluence(players[i]);
				break;
			}
		}
		setMorale(myInfluence - enemyInfluence);
	}
	
    
    /**
     * Makes all weapons available to fire.
     */
    public void rechargeWeapons() {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                weapons[i].recharge();
            }
        }
    }

    /**
     * Makes all weapons fired.
     */
    public void dischargeWeapons() {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                weapons[i].discharge();
            }
        }
    }

    public void dischargeWeapon(Weapon weapon, int qty) {
		WeaponInstance wi = getWeapon(weapon);
		if (wi != null) {
			wi.discharge(qty);
		}
    }
    
    public void dischargeWeaponsIfNoTimeWarpFired() {
    	if (!hasFiredTimeWarpThisCommand()) {
    		dischargeWeapons();
    	}
    }

    public void dischargeWeaponIfNoTimeWarpFired(Weapon weapon, int qty) {
    	if (!hasFiredTimeWarpThisCommand()) {
    		WeaponInstance wi = getWeapon(weapon);
    		if (wi != null) {
    			wi.discharge(qty);
    		}
    	}
    }

    /**
     * Most battle objects each weapon produces a single instance of a weapon in the battle.
     * However for object like planet a single weapon is a weapons battery and this method should 
     * be overridden to provide a batter size.
     */
    protected double getQtyWeaponsBonus() { return 1; }

    /**
     * @param bayIndex The index of the weapons bay we are seeking to get weapons bay details.
     * @return The details for the weapons bay (from the hull of this craft - if it is one) at index bayIndex;
     */
    protected WeaponBayDetails getWeaponBayDetailsForIndex(int bayIndex) {
    	return null;
    }
    
    /**
     * Adds all weapons in the weaponsBays to this object's list of weapons.
     * @param weaponsBays All weapons in this list of bays will be added to the battle object.
     */
    protected void setWeapons(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) throws ApplicationError {
        weapons = new WeaponInstance[1];
        weapons[0] = WeaponInstanceFactory.create(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }
    
    /**
     * Adds all weapons in the weaponsBays to this object's list of weapons.
     * @param weaponsBays All weapons in this list of bays will be added to the battle object.
     */
    public void setWeapons(WeaponBays weaponsBays, boolean heavy, boolean infiniteShots) throws ApplicationError {
        if (weaponsBays == null) {
            return;
        }
        WeaponBay[] bays = weaponsBays.getBaysAsArray();
        if (bays == null) {
        	return;
        }
        for (int i = 0; i < bays.length; i++) {
			Weapons weapons = bays[i].getWeapons();
	        setWeapons(weapons, heavy, infiniteShots, getWeaponBayDetailsForIndex(i));
		}
    }

    /**
     * Adds all weapons in the weaponsBays to this object's list of weapons.
     * @param weaponsBays All weapons in this list of bays will be added to the battle object.
     */
    public void setWeapons(Weapons weapons, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) throws ApplicationError {
        if (weapons != null) {
            TechnologyBundle[] bundle = weapons.getDistinct();
            if (bundle != null) {
                setWeapons(bundle, heavy, infiniteShots, weaponBayDetails);
            } else {
            	this.weapons = null;
            }
        } else {
        	this.weapons = null;
        }
    }
    
    /**
     * Sets the weapon attribute on this battle object. The weapons are ordered, non missiles then
     * missiles. This is allows missiles to be aimed at a different craft if the target is destroyed 
     * by the immediate firing of the beam weapons.
     * @param bundle The list of weapons to add
     * @param heavy If these weapons should all be heavy (heavy weapons have a longer range and do more damage).
     * @param infiniteShots If the missile weapons here should have a infinite number of shots.
     */
    protected void setWeapons(TechnologyBundle[] bundle, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) throws ApplicationError {
        ArrayList result = null;
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.FIRE_BEFORE_GROUPS, null, weaponBayDetails);
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.FIRE_CREW_1, null, weaponBayDetails);
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.FIRE_CREW_2, null, weaponBayDetails);
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.ATTACK_GROUPS, Boolean.FALSE, weaponBayDetails);
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.ATTACK_GROUPS, Boolean.TRUE, weaponBayDetails);
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.FIRE_AFTER_GROUPS, null, weaponBayDetails);
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.MISSILE_DEFENCE_ONLY_GROUPS, null, weaponBayDetails);
        if (this instanceof BattlePlanet) {
            result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.PLANETARY_GROUPS, null, weaponBayDetails);
        }
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.FIRE_AT_WRECKAGE_GROUPS, null, weaponBayDetails);
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.FIRE_LAST_GROUPS_1, null, weaponBayDetails);
        result = _buildWeaponsList(result, bundle, heavy, infiniteShots, Weapon.FIRE_LAST_GROUPS_2, null, weaponBayDetails);
        if (result != null) {
            this.weapons = (WeaponInstance[])result.toArray(new WeaponInstance[(result.size())]);
        } else {
            this.weapons = null;
        }
    }

    private ArrayList _buildWeaponsList(ArrayList result, TechnologyBundle[] bundle, boolean heavy, boolean infiniteShots, String [] groups, Boolean missiles, WeaponBayDetails weaponBayDetails) throws ApplicationError {
        for (int j=0; j<bundle.length; j++) {
            Weapon weapon = (Weapon)bundle[(j)].getTechnology();
            if (Comp.in(weapon.getGroup(), groups)) {
                if (missiles == null || weapon.isMissile() == missiles.booleanValue()) {
                    if (weapon.isBeam() || weapon.isProjectile()) {
                        bundle[j].qty *= getQtyWeaponsBonus();
                    }
                    WeaponInstance wi = WeaponInstanceFactory.create(bundle[j], heavy, infiniteShots, weaponBayDetails);
                    if (result == null) result = new ArrayList();
                    result.add(wi);
                }
            }
        }
        return result;
    }

    /**
     * Adds a weapon to the list of weapons instances already on this battle object.
     * @param v The weapon to create a weapon instance for this craft
     */
    public void addWeaponInstance(Weapon v, WeaponBayDetails weaponBayDetails) throws ApplicationError {
        WeaponInstance wi = getWeapon(v);
        if (wi != null) {
            wi.setQty(wi.getQty()+1);
            return;
        }
        
        int index = 0;
        WeaponInstance [] newWeapons = null;
        if (weapons == null) {
            newWeapons = new WeaponInstance[1];
        } else {
            newWeapons = new WeaponInstance[weapons.length+1];
            Comp.copy(newWeapons, weapons);
            index = weapons.length;
        }
        newWeapons[index] = WeaponInstanceFactory.create(v, 1, false, false, weaponBayDetails);
        weapons = newWeapons;
    }

    /**
     * Removes a weapon from the list of weapon instances already on this battle object.
     * @param v The shield to remove a weapon instance from this craft
     */
    public void removeWeaponInstance(Weapon v) {
        if (weapons == null) {
            return;
        }
        WeaponInstance wi = getWeapon(v);
        if (wi != null) {
            if (wi.getQty() > 1) {
                wi.setQty(wi.getQty()-1);
            } else {
                removeWeaponInstance(wi);
            }
        }
    }
    
    /**
     * Removes a shield from the list of weapon instances already on this battle object.
     * @param v The weapon to remove a shield instance from this craft
     */
    private void removeWeaponInstance(WeaponInstance v) {
        if (weapons == null) {
            return;
        }
        ArrayList r = new ArrayList();
        boolean found = false;
        for (int i=0; i<weapons.length; i++) {
            if (!found && weapons[i] == v) {
                found = true;
            } else {
                r.add(weapons[i]);
            }
        }
        if (r.size() == 0) {
            weapons = null;
        } else {
            weapons = (WeaponInstance[]) r.toArray(new WeaponInstance[r.size()]);
        }
    }
    
    /**
     * @param w The weapon to get the weapon instance for.
     * @return The weapon instance for weapon w or null if it does not exist.
     */
    public WeaponInstance getWeapon(Weapon w) {
        WeaponInstance [] wi = getWeapons();
        if (wi != null) {
            for (int i=0; i<wi.length; i++) {
                if (wi[i].getWeapon() == w) {
                    return wi[i];
                }
            }
        }
        return null;
    }

    public WeaponInstance getWeaponSpaceDistortion() {
        WeaponInstance [] wi = getWeapons();
        if (wi != null) {
            for (int i=0; i<wi.length; i++) {
                if (wi[i].getWeapon().isSpacialDistortion()) {
                    return wi[i];
                }
            }
        }
        return null;
    }

    public WeaponInstance getWeaponShieldBurst() {
        WeaponInstance [] wi = getWeapons();
        if (wi != null) {
            for (int i=0; i<wi.length; i++) {
                if (wi[i].getWeapon().isShieldBurst()) {
                    return wi[i];
                }
            }
        }
        return null;
    }

    public WeaponInstance getWeaponSalvageBeam() {
        WeaponInstance [] wi = getWeapons();
        if (wi != null) {
            for (int i=0; i<wi.length; i++) {
                if (wi[i].getWeapon().isSalvageBeam()) {
                    return wi[i];
                }
            }
        }
        return null;
    }

    public WeaponInstance getWeaponTroopTransport() {
        WeaponInstance [] wi = getWeapons();
        if (wi != null) {
            for (int i=0; i<wi.length; i++) {
                if (wi[i].getWeapon().isTroopTransport()) {
                    return wi[i];
                }
            }
        }
        return null;
    }

    public WeaponInstance getWeaponDroidMine() {
        WeaponInstance [] wi = getWeapons();
        if (wi != null) {
            for (int i=0; i<wi.length; i++) {
                if (wi[i].getWeapon().isDroidMine()) {
                    return wi[i];
                }
            }
        }
        return null;
    }

    public WeaponInstance getWeaponDetonator() {
        WeaponInstance [] wi = getWeapons();
        if (wi != null) {
            for (int i=0; i<wi.length; i++) {
                if (wi[i].getWeapon().isDetonator()) {
                    return wi[i];
                }
            }
        }
        return null;
    }

 	/**
	 * Returns true if an only if this object has weapons that have not fired this turn yet.
	 */
	public boolean hasWeaponsToFire() {
        if (isDestroyed()) return false;
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                if (weapons[i].hasWeaponsToFire()) {
                    return true;
                }
            }
        }
		return false;
	}
    
	/**
     * Missiles that have been exhausted (all fires used) are not included.
	 * @return true if an only if this object has weapons has not fired any of its weapons 
     * for this turn.
	 */
	public boolean allWeaponsReadyToFire() {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                if (!weapons[i].hasWeaponsToFire() && weapons[i].canRecharge()) {
                    return false;
                }
            }
        }
		return true;
	}
    
    /**
     * @return True if the craft has a weapon of group detonator and false otherwise.
     */
    public boolean hasDetonator() {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                if ( Comp.same(weapons[i].getWeapon().getGroup(), Weapon.GROUP_DETONATOR) ) {
                    return true;
                }
            }
        }
		return false;
    }

    public boolean hasDroadMine() {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                if ( Comp.same(weapons[i].getWeapon().getGroup(), Weapon.GROUP_DROID_MINE) ) {
                    return true;
                }
            }
        }
		return false;
    }

    public boolean hasBeamWeapons() {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                if ( Comp.same(weapons[i].getWeapon().getGroup(), Weapon.GROUP_BEAM) ) {
                    return true;
                }
            }
        }
		return false;
    }

    public boolean hasShieldBurst() {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                if ( Comp.same(weapons[i].getWeapon().getGroup(), Weapon.GROUP_SHIELD_BURST) ) {
                    return true;
                }
            }
        }
		return false;
    }
    
    /**
     * @return True if the craft has a weapon of group TimeWarp and false otherwise.
     */
    public boolean hasTimeWarp() {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                if ( Comp.same(weapons[i].getWeapon().getGroup(), Weapon.GROUP_TIME_WARP) ) {
                    return true;
                }
            }
        }
		return false;
    }
    
	/**
     * @return True if the craft has a special of group holo emiter.
     */
    public boolean hasHoloEmiters() {
        Specials craftSpecials = getSpecials();
        return (craftSpecials != null && craftSpecials.getQtyHoloEmiter() > 0);
    }
    
    /**
     * Objects like battle craft can have specials that extend the range of their weapons
     * Any objects that can extend the range of their weapon must extend this method.
     * @return the weapons range bonuses enjoyed by this object. 
     */
    public double getWeaponRangeBonus() {
        return 1.0;
    }
    
    /**
     * @return The manoeuvrability advantage this object has over the target object.
     */
	public double getManouverabilityAdvantage(BattleObject target) throws ApplicationError { return 1.0; }

    /**
     * @return The specials owned by this object.
     */
	public Specials getSpecials() { return null; }

    /**
     * @return the shield charge as a percent of the max charge.
     * The value is between 0 and 1 where 0 implies shields are at full power.
     * @throws ApplicationError 
     */
    public double getShieldAsPercent() throws ApplicationError {
        return 0.0;
    }
    
    /**
     * @return the damage as a percent of the max damage.
     * The value is between 0 and 1 where 0 implies no damage.
     */
    public double getDamageAsPercent() throws ApplicationError {
        return 0.0;
    }

    /**
     * @return The craft movement damage as a percent of the max damage. 
     * The value is between 0 and 1 where 0 implies no damage.
     */
    public double getMovementDamageAsPercent() throws ApplicationError {
        return 0.0;
    }

    /**
     * @return The craft psi damage as a percent of the max damage.
     * The value is between 0 and 1 where 0 implies no damage.
     */
    public double getPsiDamageAsPercent() {
         return 0.0;
    }
   
    /**
     * @return The craft crew as a percent of the max crew.
     * The value is between 0 and 1 where 0 implies a full crew complement.
     */
    public double getCrewAsPercent() {
         return 1.0;
    }
   
    /**
     * @return The state of this object at this point in time.
     */
    public CraftStateFactory getCraftState() throws ApplicationError {
        String ownerName = getOwnerName();
		PlayerColor pc = (getOwner() == null) ? null : getOwner().getColor();
		return new CraftStateFactory(ownerName, ownerName, getDamageAsPercent(), getShieldAsPercent(), getMovementDamageAsPercent(), getPsiDamageAsPercent(), 1.0, getLocationClone(), getDirection(), pc, getIsCloakedToList(), getDecorators(), isDestroyed(), 0.0);
    }

    public CraftState getCraftState(Player viewer) throws ApplicationError {
        String ownerName = getOwnerName();
		DecoratorList decorators = getVisibleDecorators(viewer);
		PlayerColor pc = (getOwner() == null) ? null : getOwner().getColor();
		return new CraftState(ownerName, ownerName, getDamageAsPercent(), getShieldAsPercent(), getMovementDamageAsPercent(), getPsiDamageAsPercent(), 1.0,  getLocationClone(), getDirection(), pc, getIsCloakedToList(), decorators, isDestroyed(), 0.0);
    }

    public CraftState getCraftStateForGenericPlayer() throws ApplicationError {
        String ownerName = getOwnerName();
		PlayerColor pc = (getOwner() == null) ? null : getOwner().getColor();
   		return new CraftState(ownerName, ownerName, getDamageAsPercent(), getShieldAsPercent(), getMovementDamageAsPercent(), getPsiDamageAsPercent(), getCrewAsPercent(), getLocationClone(), getDirection(), pc, getIsCloakedToList(), null, isDestroyed(), 0.0);
    }

	/**
	 * Inflicts the parameter damage amount of damage on this object.
	 */
	public abstract WeaponDamage damage(double damage, Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError;
    
    /**
     * Causes this object to be removed from the battle and the game and issues the correct
     * destroy event (e.g. CraftDestroyedEvent) to be issued.
     * @throws mot.common.util.ApplicationError
     */
	public abstract void destroy(Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError;
	
    
	/**
	 * @return The damage this object has sustained or -1 if this object does not
     * maintain a damage assessment.
	 */
	public double getDamage() {
        return -1;
    }
    
    /**
     * @return True if this object is destroyed (no longer a valid weapon target).
     */
    public abstract boolean isDestroyed();
    
    /**
     * @return the visibility of this craft in the battle field. A crafts visibility is effected by 
     * its shields and the players cloaking bonus.
     */
    public int getCloakLevel() throws ApplicationError {
        return 0;
    }
    
    /**
     * @return true if and only if this player's scanners a strong enough to see this object through
     * its shields at the distance from this players crafts / planets.
     */
    public boolean isVisible(Player player) throws ApplicationError {
        if (getOwner() == player) {
            return true;
        }
        int cloakLevel = getCloakLevel();
        if (cloakLevel <= 0) {
            return true;
        }
		int playerSight = getBattle().getScannerStrengthAt(player, getLocation());
		return (playerSight >= cloakLevel);
    }
    
    /**
     * NOTE - This returns NULL if the craft has no cloaking (and is visible to all)
     * @return A list of player names that cannot see this craft on the battle map.
     * @throws ApplicationError
     */
    public String [] getIsCloakedToList() throws ApplicationError {
        int cloakLevel = getCloakLevel();
        if (cloakLevel <= 0) {
            return null;
        }
        ArrayList r = null;
        Player[] players = getBattle().getPlayers();
        if (players != null) {
	        for (int i = 0; i < players.length; i++) {
				if (!isVisible(players[i])) {
					if (r == null) r = new ArrayList();
					r.add(players[i].getName());
				}
			}
        }
        if (r != null) {
        	return (String []) r.toArray(new String[r.size()]);
        } else {
        	return null;
        }
    }
    
    /**
     * Called when the user issues an command to this craft before any actions are carried out.
     * This is used by time warp to avoid discharging weapons after a command to fire all and discharge!
     * @return Return true if the command is to be ignored.
     */
    public boolean notifyStartCommand() {
    	boolean stopCommand = false;
    	if (weapons != null) {
	        for (int i=0; i<weapons.length; i++) {
            	if (weapons[i].notifyStartCommand()) {
            		stopCommand = true;
            	}
	        }
    	}
        return stopCommand;
    }

    /**
     * @return Return true if the command issues by the owner has caused a time warp to be fired.
     */
	public boolean hasFiredTimeWarpThisCommand() {
    	if (weapons != null) {
	        for (int i=0; i<weapons.length; i++) {
	            if (weapons[i] instanceof WeaponInstanceTimeWarp) {
	            	WeaponInstanceTimeWarp tw = (WeaponInstanceTimeWarp) weapons[i];
	            	if (tw.hasFiredTimeWarpThisCommand()) {
	            		return true;
	            	}
	            }
	        }
    	}
        return false;
	}

    

    /**
     * @return true if the location passed in is within weapons range of any weapon on
     * this object.
     */
    public boolean inWeaponRange(Location loc) throws ApplicationError {
        if (weapons == null) {
            return false;
        }
		int distance = getLocation().getDistanceTo( loc );
        long maxDistance = 0;
        for (int i=0; i<weapons.length; i++) {
            long tmpMaxDistance = weapons[i].getRange();
            if (tmpMaxDistance > maxDistance) {
                maxDistance = tmpMaxDistance;
            }
        }
        double rangeBonus = getWeaponRangeBonus();
		return distance < (maxDistance*rangeBonus);
    }

    public int getWeaponRange(WeaponInstance wi) throws ApplicationError {
        double rangeBonus = getWeaponRangeBonus();
    	return (int) (wi.getRange() * rangeBonus);
    }
    

    /**
     * This is a short cut for fireWeapons() where weapon is null so all weapons are fired.
     * If this craft has a time warp weapon then this will fire all the weapons on this craft
     * twice, once before firing the time warp and once afterwards.
     */
	public boolean fireAllWeapons(BattleObject target) throws ApplicationError {
		boolean firedAWeapon = _fireAllWeapons(target);
        if (hasTimeWarp()) {
        	boolean fired = _fireAllWeapons(target);
        	if (fired) firedAWeapon = true;
        }
        return firedAWeapon;
    }

	private boolean _fireAllWeapons(BattleObject target) throws ApplicationError {
        if (target instanceof BattleCraft) {
            BattleCraft bc = (BattleCraft) target;
            return fireWeapons(bc, null, -1);
        } else {
        	return fireWeapons(target, null, -1);
        }
    }

    /**
     * When BattleObject.fireAllWeapons() calls fireWeapons(BattleCraft, WeaponInstance, int) it 
     * must call the overridden version in BattleCraft - Without this it will call
     * fireWeapons(BattleObject, WeaponInstance, int)
     */
	public boolean fireWeapons(BattleCraft target, WeaponInstance weapon, int qtyToFire) throws ApplicationError {
        return fireWeapons((BattleObject)target, weapon, qtyToFire);
    }

	/**
	 * Fires weapons at the target object.
	 * If the target is destroyed before all the weapons have been fires, then the
	 * the unused weapons will not have been fired.
	 *
	 * This method calculates the visibility of the target and if the target is not 
     * visible there is a change the target will be missed. If the target is missed
     * the weapons are discharged with firing and a MissTargetEvent is generated. 
     * Otherwise the weapons fire are fired as normal and a weapon fire event is generated.
     * 
     * This method uses the method fire(WeaponInstance, BattleCraft, WeaponFireEvent) 
     * for each weapon to fire it. For extra details on weapons fire read its docos.
     * 
     * @param target The craft to attack
     * @param weapon If null all weapons that have not been fired and are in range fire.
     * If weapon is not null only this weapon is fired. If this is null weapons are fired in 
     * the order they are places in the weapon instance array - Note - order is important so 
     * weapons like ShieldDrain get sap maximum power before the shields are weakened.
     * @param qtyToFire If weapon is not null this is the quantity of weapons in the weapon 
     * instance to fire. If weapon is null then this parameter is ignored.
	 */
	public boolean fireWeapons(BattleObject target, WeaponInstance weapon, int qtyToFire) throws ApplicationError {
        if (isDestroyed()) return false;
        
//        if (Logger.isDebug()) {
//            Logger.logDebug("BattleObject.fireWeapons() "+getName()+" target="+target.getName());
//        }
        
        if (weapons == null) {
            return false;
        }
        
        boolean firedAWeapon = false;
        
        if (weapon != null) {
            return fireWeapon(target, weapon, qtyToFire);
        } else {
            Player originalOwner = target.getOwner();
            for (int i=0; i<weapons.length; i++) {
                String group = weapons[i].getWeapon().getGroup();
        		String [] excludeGroups = getOwner().getWeaponsToExcludeFromFireAll(target);
				if (excludeGroups != null && Comp.in(group, excludeGroups)) {
                    continue;
                }
                if (originalOwner != target.getOwner()) {
                    // Owner has changed, stop firing the rest of the weapons!
                    return false;
                }
                if (weapons[i].willHaveAnEffectOn(target)) {
	                int qty = (qtyToFire > 0) ? Comp.min(qtyToFire, weapons[i].getQty()) : weapons[i].getQty();
	                boolean fired = fireWeapon(target, weapons[i], qty);
	                if (fired) firedAWeapon = true;
                } else {
                	if (Logger.isDebug()) {
                		Logger.logDebug("BattleObject.fireWeapons() Weapon "+weapons[i].getName()+" No Effect on "+target);
                	}
                }
            }
        }
        
		MotEngine.instance.getProxyManager().broadcastQueue();
		
		return firedAWeapon;
	}

	/**
	 * A craft cannot fire its weapon at a target if the craft is not pointing in the right 
	 * direction. E.g. The weapon bay may only allow this weapon to fire forwards!
	 * ONLY effects crafts - so it is overridden for crafts!  
	 * @param weapon The weapon to fire
	 * @param target The target to attack with the weapon
	 * @return True if this BO can fire the weapon passed in at the target.
	 */
	public boolean isTargetAngleWithinWeaponFiringAngle(WeaponInstance weapon, BattleObject target) {
		return true;
	}
	
	public boolean isTargetAngleWithinWeaponFiringAngleOfAllWeapons(BattleObject target) {
		return true;
	}

    /**
     * Return weapons intended for use against missiles in firing angle, in range and ready to fire at target.
     */
	public WeaponInstance [] getWeaponsReadyToFireAt(MovingAutamaton target, String [] weaponGroups) throws ApplicationError {
		ArrayList r = null;
        if (weapons != null && target != null) {
            for (int i=0; i<weapons.length; i++) {
                if ( Comp.in(weapons[i].getWeapon().getGroup(), weaponGroups) ) {
                	if (weapons[i].hasWeaponsToFire()) {                		
                		if (isTargetAngleWithinWeaponFiringAngle(weapons[i], target)) {
                			if (weapons[i].inRange(this, target.getLocation())) {
                				if (r == null) r = new ArrayList();
                				r.add(weapons[i]);
                			}
                		}
                	}
                }
            }
        }
        if (r != null) {
        	return (WeaponInstance[]) r.toArray(new WeaponInstance[r.size()]);
        } else {
        	return null;
        }
	}
	
    /**
     * If this craft has weapons that can attack incoming missiles fire at them.
     * e.g. Anti missile rockets or retarget beam (that are ready to fire).
     */
	public void fireMissileDefenceWeapons(MovingAutamaton priorityTarget) throws ApplicationError {
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
                if ( Comp.in(weapons[i].getWeapon().getGroup(), Weapon.MISSILE_DEFENCE_ONLY_GROUPS) ) {
                	int qtyToFire = weapons[i].getQtyWeaponsToFire();
                	boolean keepFiring = true;
                	for (int j = 0; j < qtyToFire && keepFiring; j++) {
	                	if (priorityTarget != null && priorityTarget.isTargetingPlayer(getOwner())) {
	                		fireWeapons(priorityTarget, weapons[i], 1);
	                	} else {
	                		keepFiring = fireMissileDefenceWeapon(weapons[i]);
                		}
                	}
                }
            }
        }
	}
    
    private boolean fireMissileDefenceWeapon(WeaponInstance weaponInstance) throws ApplicationError {
    	MovingAutamaton enemyMissile = battle.getClosestEnemyMovingAutomatonInWeaponFireAngleAimedAtMe(this, weaponInstance, true);
    	if (enemyMissile != null) {
   			fireWeapons(enemyMissile, weaponInstance, 1);
   			return true;
    	} else {
    		return false;
    	}
	}
	
	private Weapon _getDefaultWeapon() {
    	if (weapons == null || weapons.length == 0) return null;
    	for (int i = 0; i < weapons.length; i++) {
			Weapon weapon = weapons[i].getWeapon();
			if (weapon.isBeam() || weapon.isProjectile()) {
				return weapon;
			}
		}
		return weapons[0].getWeapon();
	}

	/**
     * Fires weapon at the target. If the target is destroyed or the weapon is not charged then the 
     * weapon is not fired.
     * CHANGE - This now discharges the weapon after it has been fired.
     * @param target The target craft to fire at
     * @param weapon The weapon to fire.
     * @throws mot.common.util.ApplicationError
     */
	public boolean fireWeapon(BattleObject target, WeaponInstance weapon, int qtyToFire) throws ApplicationError {
		if (qtyToFire < 0) {
			qtyToFire = weapon.getQtyWeaponsToFire();
		}
		
        if (Logger.isDebug()) {
            if (weapon != null) {
                Logger.logDebug("BattleObject.fireWeapon() "+getName()+" ["+weapon.getWeapon().getName()+"X"+qtyToFire+"] target="+target.getName());
            } else {
                Logger.logDebug("BattleObject.fireWeapon() "+getName()+" target="+target.getName());
            }
        }

        if (!weapon.hasWeaponsToFire()) {
            return false;
        }
        if (!weapon.inRange(this, target.getLocation())) {
            return false;
        }
        if (target.isDestroyed()) {
            return false;
        }
        if (weapon.usesWeaponBayFireAngle() && !isTargetAngleWithinWeaponFiringAngle(weapon, target)) {
          	// This object cannot fire at the target because the target is not within angle range 
          	// for the weapons bay its is being fired from.
          	if (!getOwner().isComputerPlayer() && allWeaponsReadyToFire()) {
          		// When weapon is null or a missile client uses MissTargetAnimation2
                MotEngine.instance.multicast( new BattleWeaponMissEF(this, target, weapon.getWeapon(), getLocationClone(), getDirection(), target.getLocationClone()), target.getBattle().getStartingUserPlayers() );
          	}
          	weapon.discharge(qtyToFire);
           	return false;
        }

        boolean canMiss = (weapon == null) ? true : weapon.canMiss();
        if (!target.isVisible(getOwner())) {
            if (canMiss && Utils.random(CHANCE_TO_MISS_WHEN_CLOAKED)) {
                Weapon w = weapon == null ? _getDefaultWeapon() : weapon.getWeapon();
                if (w != null) {
                    MotEngine.instance.broadcast( new BattleWeaponMissEF(this, target, w, getLocationClone(), getDirection(), target.getLocationClone()) );
                }
                weapon.discharge();
                return false;
            }
        }

        weapon.fire(this, target, qtyToFire);
        //weapon.discharge(qtyToFire); this is a double discharge we we fired one one 2 weapons
        
        return (weapon.getWeapon() != null) && weapon.getWeapon().isBeamOrProjectile();
    }

	

	/**
	 * Fires weapons at the target location.
     * This method uses the method fire(WeaponInstance, BattleCraft, WeaponFireEvent) 
     * for each weapon to fire it. For extra details on weapons fire read its docos.
     * 
     * @param target The location to fire this weapon at
     * @param weapon If null all weapons that have not been fired and are in range fire.
     * If weapon is not null only this weapon is fired. If this is null weapons are fired in 
     * the order they are places in the weapon instance array - Note - order is important so 
     * weapons like ShieldDrain get sap maximum power before the shields are weakened.
     * @param qtyToFire If weapon is not null this is the quantity of weapons in the weapon 
     * instance to fire. If weapon is null then this parameter is ignored.
	 */
	public void fireWeapons(Location target, WeaponInstance weapon, int qtyToFire) throws ApplicationError {
        if (isDestroyed()) return;
        
        if (Logger.isDebug()) {
            Logger.logDebug("BattleObject.fireWeapons() "+getName()+" target="+target);
        }
        
        if (weapons == null) {
        	WeaponInstance sdw = getWeaponSpaceDistortion();
        	if (sdw != null && sdw.hasWeaponsToFire()) {
        		weapon = sdw;
        	} else {
            	WeaponInstance sbw = getWeaponSalvageBeam();
        		if (sbw != null && sbw.hasWeaponsToFire()) {
        			weapon = sbw;
        		}
        	}
            return;
        }

        if (weapon != null) {
            fireWeapon(target, weapon, qtyToFire);
        } else {
            for (int i=0; i<weapons.length; i++) {
                int qty = weapons[i].getQty();
                fireWeapon(target, weapons[i], qty);
            }
        }
	}

    /**
     * Fires weapon at the location.
     * @param target The target location to fire at
     * @param weapon The weapon to fire.
     * @throws mot.common.util.ApplicationError
     */
	public void fireWeapon(Location target, WeaponInstance weapon, int qtyToFire) throws ApplicationError {
        if (weapon == null) {
            return;
        }
        if (Logger.isDebug()) {
            Logger.logDebug("BattleObject.fireWeapon() "+getName()+" ["+weapon.getWeapon().getName()+"X"+qtyToFire+"] target="+target);
        }

        if (!weapon.hasWeaponsToFire()) {
            return;
        }
        
        weapon.fire(this, target, qtyToFire);
    }

	/**
	 * @return True if this craft has at least one weapon that can fire at a location and has charge.
	 */
	public boolean hasWeaponsThatAttackLocation() {
		if (weapons == null) {
			return false;
		}
        for (int i=0; i<weapons.length; i++) {
            if (weapons[i].canAttackLocationAndHasCharge()) {
            	return true;
            }
        }
		return false;
	}

	
    /**
     * Recharges all shields on the craft to their maximum levels.
     * See ShieldInstance of Shield for details.
     * @throws ApplicationError 
     */
    public void rechargeShieldsFull() throws ApplicationError {
        if (shields == null) return;
        for (int i=0; i<shields.length; i++) {
            shields[i].rechargeFull();
        }
    }

    /**
     * Recharges all shields on the craft.
     * See ShieldInstance of Shield for details.
     * @throws ApplicationError 
     */
    public void rechargeShields() throws ApplicationError {
        if (shields == null) return;
        double unusedPower = 0.0;
        for (int i=0; i<shields.length; i++) {
            unusedPower += shields[i].recharge();
        }
        if (unusedPower > 0) {
        	if (getSpecials() != null) {
        		ShieldSpan ss = getSpecials().getBestShieldSpan();
        		if (ss != null) {
        			ShieldInstance.executeShieldSpan(this, ss, unusedPower);
        		}
        	}
        }
    }
    
    public void dischargeShields() {
        if (shields == null) return;
        for (int i=0; i<shields.length; i++) {
            shields[i].discharge();
        }
    }


    /**
     * Initialises the shields attribute to be the list of shields passed in.
     * @throws ApplicationError 
     */
    public void setShields(Shields v, boolean doNoShieldsCheck) throws ApplicationError {
    	if (doNoShieldsCheck) {
    		if (battle != null && battle.isNoShields()) {
                shields = null;
                return;
    		}
    	}
        if (v == null) {
            shields = null;
            return;
        }
        Shield[] array = v.getShields();
        if (array == null) {
            shields = null;
            return;
        }
        shields = new ShieldInstance[array.length];
        for (int i=0; i<array.length; i++) {
            shields[i] = new ShieldInstance(this, array[i]);
        }
	}
    
    /**
     * Adds a shield to the list of shield instances already on this battle object.
     * @param v The shield to create a shield instance for this craft
     * @throws ApplicationError 
     */
    public void addShieldInstance(Shield v) throws ApplicationError {
        int index = 0;
        ShieldInstance [] newShields = null;
        if (shields == null) {
            newShields = new ShieldInstance[1];
        } else {
            newShields = new ShieldInstance[shields.length+1];
            Comp.copy(newShields, shields);
            index = shields.length;
        }
        newShields[index] = new ShieldInstance(this, v);
        shields = newShields;
    }

    /**
     * Removes a shield from the list of shield instances already on this battle object.
     * @param v The shield to remove a shield instance from this craft
     */
    public void removeShieldInstance(Shield v) {
        if (shields == null) {
            return;
        }
        ArrayList r = new ArrayList();
        boolean found = false;
        for (int i=0; i<shields.length; i++) {
            if (!found && shields[i].getShield() == v) {
                found = true;
            } else {
                r.add(shields[i]);
            }
        }
        if (r.size() == 0) {
            shields = null;
        } else {
            shields = (ShieldInstance[]) r.toArray(new ShieldInstance[r.size()]);
        }
    }

    /**
     * @return the current shield power (this sum of current power to all shields).
     */
    public double getShieldPower() {
        if (shields == null) return 0;
        double sum = 0;
        for (int i=0; i<shields.length; i++) {
            sum += shields[0].getPower();
        }
        return sum;
    }
    
    public void setShieldPower(int shield, double power) {
        shields[shield].setPower(power); 
    }
    

    public ShieldInstance[] getShields() { return shields; }
    public WeaponInstance[] getWeapons() { return weapons; }

    public ShieldInstance[] getShields(String group) { 
        if (shields == null) {
            return null;
        }
        ArrayList r = null;
        for (int i=0; i<shields.length; i++) {
            if (Comp.same(group, shields[i].getShield().getGroup())) {
                if (r == null) r = new ArrayList();
                r.add(shields[i]);
            }
        }
        if (r == null) {
            return null;
        } else {
            return (ShieldInstance[]) r.toArray(new ShieldInstance[r.size()]);
        }
    }
    
    public ShieldInstance getShieldsWithHighestPower(String group) { 
        if (shields == null) {
            return null;
        }
        ShieldInstance best = null;
        for (int i=0; i<shields.length; i++) {
            if (Comp.same(group, shields[i].getShield().getGroup())) {
                if (best == null || best.getPower() < shields[i].getPower()) {
                    best = shields[i];
                }
            }
        }
        return best;
    }
    
    public ShieldInstance getShieldsWithLowestPower(String group) { 
        if (shields == null) {
            return null;
        }
        ShieldInstance best = null;
        for (int i=0; i<shields.length; i++) {
            if (Comp.same(group, shields[i].getShield().getGroup())) {
                if (best == null || best.getPower() > shields[i].getPower()) {
                    best = shields[i];
                }
            }
        }
        return best;
    }
    
	/**
	 * @return true if an only if this object has weapons.
	 */
	public boolean hasWeapons() {
        return (weapons != null && weapons.length > 0);
	}

	/**
	 * @return true if an only if this planet has shields.
	 */
	public boolean hasShields() {
        return (shields != null && shields.length > 0);
	}


	public ShieldInstance getShieldsThatRequireRecharging() throws ApplicationError {
		if (shields != null && shields.length > 0) {
			for (int i = 0; i < shields.length; i++) {
				if (shields[i].getShield().isNormal() && !shields[i].fullyCharged()) {
					return shields[i];
				}
			}
		}
		return null;
	}

	public boolean hasShieldPower() {
		ShieldInstance[] shields = getShields();
		if (shields == null) return false;
		for (int i = 0; i < shields.length; i++) {
			if (shields[i].getPower() > 0.0) return true;
		}
		return false;
	}

	/**
     * Deletes all missiles in the battle that are targeting this craft.
     * @throws mot.common.util.ApplicationError
     */
    public void deleteAutomitonsTargetedAtMe(BattleDestroyedEF destroyedEvent) throws ApplicationError {
        Automaton [] aa = getBattle().getAutomatons();
        if (aa == null) return;
        for (int i=0;i < aa.length; i++) {
            aa[i].notifyGoneFromBattle(this, destroyedEvent);
        }
    }
    

    /**
     * Return the closest location to dest from this object that is no more than maxDist away.
     * NOTE - This location may be occupied.
     * @param dest A destination we wish to get as close to as possible.
     * @throws ApplicationError 
     */
	private Location getClosestLocToDestInRange(Location dest, int maxDist) throws ApplicationError {
		Location loc = getLocation();
		int distance = loc.getDistanceTo(dest);
        int actualDistance = Comp.min(distance, maxDist);

        Location actualLoc = new Location(loc);
        actualLoc.shift(dest, actualDistance);
		return actualLoc;
	}
	
    /**
     * @param moveToDest The location to move this object towards
     * @param maxDist The maximum distance to move this object towards the destination.
     * @param targetDest The original destination the user / computer selected as the target.
     * This is usually the location of the craft or planet under attack.
     * @return The closest free space to dest on the battle field between this object's current 
     * position and dest that is no further than maxDist from its current location.
     */
    public Location getClosestFreeSpaceTo(Location moveToDest, MovementCostCalculator mcc, Location targetDest, ServerImageObject target) throws ApplicationError {

    	// Crop movement to within range
        Location actualDest = getClosestLocToDestInRange(moveToDest, mcc.getMovement());
        
        // Avoid doing micro movements
        // When attacking an adjacent craft we may click a few pixels out of angle causing the craft
        // to turn 90 degrees move a few pixels then turn 90 degrees back to target!
		Location currentLocation = getLocation();
		if (targetDest != null && moveToDest.getDistanceTo(currentLocation) < getMaxSpaceDimension()*2+1) {
			// The moveToLocation is right next to the current location and NO CLOSER to the target
			// We will not move.
			return battle.getClosestFreeLocationNoDirChange(actualDest, this);
        }
        
        Battle battle = getBattle();
		ServerImageObject[] locationOccupiers = battle.getLocationOccupiers(actualDest, this);
		if (locationOccupiers == null || locationOccupiers.length == 0) {
			// There is nothing stopping us going to this location.
			return actualDest;
		}
		
		// If target is one of the location occupiers then move away from it
		if (target != null && Comp.in(target, locationOccupiers)) {
			actualDest = getClosestLocationTo(actualDest, target);
			locationOccupiers = battle.getLocationOccupiers(actualDest, this);
			if (locationOccupiers == null || locationOccupiers.length == 0) {
				return actualDest;
			}
		}
		
		Location closestFreeSpaceInStraightLine = battle.getClosestFreeLocationNoDirChange(actualDest, this);
		
		// Try moving the actualDest parallel to the edge of the target (try to move around object between me and destination)
		// to find a free location
		Location tryDest1 = null;
		if (targetDest != null) {
			int dirToTarget = actualDest.getDirectionFor(targetDest);
			int dirDelta1 = Comp.roundAngle((dirToTarget + 90)%360);
			int dirDelta2 = Comp.roundAngle((dirToTarget - 90 + 360)%360);
			tryDest1 = getClosestFreeLocationToInDirections(battle, this, actualDest, locationOccupiers[0], dirDelta1, dirDelta2, targetDest, mcc);
		}
		
		int currentDirection = getDirection();
		ServerImageObject occupier = (ServerImageObject) Utils.getClosest(locationOccupiers, currentLocation);
		Location tryDest2 = battle.getFreeLocationInArea(moveToDest, actualDest, currentDirection, this, targetDest, mcc, 0);
		
		int d1 = (tryDest1 == null) ? Integer.MAX_VALUE : tryDest1.getDistanceTo(targetDest) + tryDest1.getDistanceTo(actualDest);
		int d2 = (tryDest2 == null) ? Integer.MAX_VALUE : tryDest2.getDistanceTo(targetDest) + tryDest2.getDistanceTo(actualDest);
		int d3 = closestFreeSpaceInStraightLine.getDistanceTo(targetDest) + closestFreeSpaceInStraightLine.getDistanceTo(actualDest);
		
		if (d1 < d2 && d1 < d3) {
			return tryDest1;
		} else if (d2 < d3) {
			return tryDest2;
		} else {
			return closestFreeSpaceInStraightLine;
		}
    }

	/**
     * Does not allow the craft to change direction to get the closest location.
     * So the closest location is calculated from a straight line between origin and destination. 
     */
    protected Location getClosestFreeSpaceTo_NO_DIR_CHANGE(Location dest, int maxDist) throws ApplicationError {
        Location actualLoc = getClosestLocToDestInRange(dest, maxDist);
        Battle battle = getBattle();
		if (battle.getLocationOccupiers(actualLoc, this) != null) {
			actualLoc = battle.getClosestFreeLocationNoDirChange(actualLoc, this);
		}

        return actualLoc;
    }


    /**
     * Shift this object in the direction of dest. The craft will not move further than
     * maxDist so may not reach the destination.
     * @param dest The location the craft wished to move to
     * @param maxDist The maximum distance the craft can move
     */
    protected int shift(Location dest, int maxDist) throws ApplicationError {
        Location loc = getLocation();
		int distance = loc.getDistanceTo(dest);
        int actualDistance = Comp.min(distance, maxDist);
		loc.shift(dest, actualDistance);
		return actualDistance;
    }

    /**
     * Shift this object towards the destination dest. The location this object is moved to may 
     * also containing other overlapping image objects.
     * @param dest The location to move this object towards
     * @param maxDist The maximum distance to move this object towards the destination.
     * @return The actual distance moved.
     */
    protected int shiftWithOverlap(Location dest, int maxDist) throws ApplicationError {
        Location loc = getLocation();
		int distance = loc.getDistanceTo(dest);
        int actualDistance = Comp.min(distance, maxDist);
		loc.shift(dest, actualDistance);
        return actualDistance;
    }

    /**
     * Add a move linked object to this battle object.
     * @param v The object to have its moves linked to this object.
     */
    public void add(MoveLinkedObject v) {
        if (moveLinks == null) moveLinks = new ArrayList();
        moveLinks.add(v);
        moveLinksArray = null;
    }
    
    /**
     * Remove a move linked object to from battle object's list of linked objects.
     * @param v The object to have its moves linked to this object.
     */
    public void remove(MoveLinkedObject v) {
        if (moveLinks == null) return;
        moveLinks.remove(v);
        moveLinksArray = null;
    }
    
    /**
     * @return All Objects that have their movement linked to the movement of this object.
     */
    public MoveLinkedObject[] getMoveLinks() {
        if (moveLinksArray == null && moveLinks != null) {
            moveLinksArray = (MoveLinkedObject[])moveLinks.toArray(new MoveLinkedObject[(moveLinks.size())]);
        }
        return moveLinksArray;
    }
    
	/**
	 * @return The percentage move this craft can make before an automaton strikes this craft.
	 * @throws ApplicationError 
	 */
	public MovementLinkStrike getPercentMoveBeforeAutomatonStrike() throws ApplicationError {
        MoveLinkedObject [] links = getMoveLinks();
        if (links == null) {
        	return null;
        }
        double minPercent = 1.0;
        MoveLinkedObject minLink = null;
		for (int i=0; i<links.length; i++) {
			double percent = links[i].getPercentMoveBeforeStrike();
			if (percent < minPercent) {
				minPercent = percent;
				minLink = links[i];
			}
		}	
		if (minPercent < 1.0) {
			return new MovementLinkStrike(minLink, minPercent);
		} else {
			return null;
		}
	}

    /**
     * This moves all objects whose move is linked to this object by percent
     * amount of their total move.
     * @param automatonStrike This is an automaton that has stopped movement of the craft to strike (may be null)
     */
    public void moveLinkedObjects(double percent, MotLinkedEventFactory event, MovementLinkStrike automatonStrike) throws ApplicationError {
        if (isDestroyed()) return;
        
        // Movement has halted for this automaton to strike
        MoveLinkedObject strikingAutomaton = null;
        if (automatonStrike != null) {
        	strikingAutomaton = automatonStrike.getAutomaton();
        	strikingAutomaton.strike(event);
        }
        
        MoveLinkedObject [] links = getMoveLinks();
        if (links != null) {
            for (int i=0; i<links.length;i++) {
            	// moving one droid mine can cause it to explode and destroy the next one before it moves.
            	if (!links[i].isDestroyed() && links[i] != strikingAutomaton) {            		
            		links[i].move(percent, event);  
            	}
            }
        }
    }

    /**
     * Decorators are only displayed in windows with the same owner as this Image Object. A 
     * decorator allows the owner of the craft know that, his craft is cloaked (to other players)
     * or that his object is not a real craft but a hologram.
     * @return The names of images (decorators) that should be displayed on top of this crafts image.
     * @throws mot.common.util.ApplicationError
     */
    public ServerDecoratorList getDecorators() throws ApplicationError {
    	if (decorators == null) {
    		decorators = new ServerDecoratorList(this);
    	}
        return decorators;
    }

    public void createOrUpdateDecotatorList() throws ApplicationError {
    	if (decorators == null) {
    		decorators = new ServerDecoratorList(this);
    	} else {
    		decorators.update();
    	}
    }

    /**
     * @return Return a list containing only the decorators that should be displayed to player p
     */
    public DecoratorList getVisibleDecorators(Player p) throws ApplicationError {
    	return getDecorators().getVisibleDecorators(p);
    }
    
    public ExclusionZone getExclusionZone() {
    	return exclusionZone;
    }
    
    public void setExclusionZone(ExclusionZone v) {
    	exclusionZone = v;
    }
    
    /**
     * @return Return the player who built this battle object. This is usually the
     * owner, but if the owner has changed will be the original owner.
     */
    public abstract Player getOriginPlayer() throws ApplicationError;

    /**
     * @return Return the player that owns this object.
     */
    public abstract Player getOwner() throws ApplicationError;

    public String getOwnerName() throws ApplicationError { 
    	Player owner = getOwner(); 
    	return (owner == null) ? null : owner.getName();
    }
    
    /**
     * @return true if the player passed in (owner) is the owner of this battle object.
     */
    public abstract boolean isOwnedBy(Player owner) throws ApplicationError;
    
    /**
     * Normally only the owner can control an object - BUT, some mind control weapons
     * can make a craft controllable by another player.
     * @return True if the player p is can give commands to this battle object, e.g. tell
     * the object to move or fire etc.
     */
    public boolean isControlledBy(Player p) throws ApplicationError {
    	return isOwnedBy(p);
    }
    
    /**
     * Normally only the owner can control an object - BUT, some mind control weapons
     * can make a craft controllable by another player or even null.
     * @return The player that is controlling this battle object.
     */
    public Player getController() throws ApplicationError {
    	return getOwner();
    }
   
    public double getMorale() {
		return morale;
	}
    
    public void setMorale(double v) {
		morale = v;
    	if (morale < -1.0) morale = -1.0;
    	if (morale > 1.0) morale = 1.0;
	}
    
    public void modifyMorale(double v) {
    	setMorale(morale+v);
    }

    
    /**
     * Morale effects the performance of the crew. It is a value between 0.0 and +1.0.
     * A value of 0.0 would mean deralict while a value of 1.0 would mean perfect.
     * This effects weapons @see {@link WeaponInstance} for details.
     * @return The condition of this battle object.
     */
    public abstract double getCondition() throws ApplicationError;
    
    public Battle getBattle() {
        return battle;
    }

    public void setBattle(Battle battle) {
        this.battle = battle;
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (battle != null) {
			new BattleReference(battle).toXML(writer);
		}
	}

	public void setSalvage(SalvageList salvage) {
		this.salvage = salvage;
	}

	public SalvageList getSalvage() {
		return salvage;
	}

	public void addSalvage(SalvageList sl) {
		Technology[] ta = sl.getAllAsArray();
		if (ta != null) {
			for (int i = 0; i < ta.length; i++) {
				addSalvage(ta[i]);
			}
		}
	}

	public void addSalvage(Technology technology) {
        if (Logger.isDebug()) {
            Logger.logDebug("BattleObject.addSalvage() "+getName()+" "+technology.getName());
        }
		if (salvage == null) {
			salvage = new SalvageList();
		}
		salvage.add(technology);
		
	}

	/**
     * Some objects such as crafts can be non - operational but not destroyed.
     * e.g. A craft with no crew
     * @return True if this object is capable of operation in this battle. 
	 */
	public boolean isOperating() {
		return true;
	}
	
	protected boolean weaponStateDirty = false;

	/**
	 * @return TRUE if any weapon states requires an updating on each user client (tests dirty flag)
	 */
	public boolean clientsNeedWeaponStateUpdate() {
		if (weaponStateDirty) {
			return true;
		}
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
            	if (weapons[i].clientsNeedStateUpdate()) {
            		return true;
            	}
            }
        }
		return false;
	}

	/**
	 * Called to mark all client weapon states as updated (not dirty)
	 */
	public void clientsWeaponStateUpdated() {
		weaponStateDirty = false;
        if (weapons != null) {
            for (int i=0; i<weapons.length; i++) {
            	weapons[i].clientsWeaponStateUpdated();
            }
        }
	}

}