/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.PlanetGraphics;
import mot.common.bean.Size;
import mot.common.event.BattleObjectNotificationType;
import mot.common.factory.PlanetGraphicsFactory;
import mot.common.lang.L;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.event.BattleObjectNotificationEF;
import mot.engine.event.DestroyColonyEF;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.planets.CrisisFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.PlanetReference;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

/**
 * This is a wrapper used by the battle module to represent a planet in battle.
 * The location is the location on the battle field instead of the map.
 */
public class BattlePlanet extends BattlePrimaryObject {

	/**
	 * If there is a planet also involved in the battle, this is the planet.
	 */
	private PlanetReference planet;
    
    /**
     * The damage inflicted on this planet.
     * Once damage is greater than the cost of a weapon to build, that weapon is destroyed.
     * When there are no weapons, once the damage is greater than the cost to build the shields the shields are destroyed.
     * When there are no shields, once the damage is greater than the cost to build the scanners the scanners are destroyed.
     * When there are no scanners, for every POP_DESTROYED_PER_DAMAGE amount of damage a unit of population is destroyed.
     */
    private double damage;

    /**
     * Graphical information about the images used to animate this planet
     */
    private PlanetGraphics planetGraphics;
    
    /**
     * List of specials placed on this planet
     */
    private Specials specials = null;
    
	public Specials getSpecials() { return specials; }
	public void setSpecials(Specials specials) { this.specials = specials;	}
	public void addSpecial( Special special ) {
		if (specials == null) specials = new Specials();
		specials.add(special);
	}
    public void setSpecials(Planet p) throws ApplicationError {
    	specials = null;
    	Player owner = p.getOwner();
        if (owner != null && owner.getSpecials() != null) {
        	Specials sl = owner.getSpecials();
			Special missileGuidanceUnit = (Special) sl.getMostAdvancedWithGroup(Special.GROUP_MISSILE_GUIDANCE);
        	if (missileGuidanceUnit != null) {
        		addSpecial( missileGuidanceUnit );
        	}
        	Special fighterBays = (Special) sl.getMostAdvancedWithGroup(Special.GROUP_FIGHTER_BAYS);
        	if (fighterBays != null && p.hasScanners() && p.hasShields()) {
        		addSpecial( fighterBays );
        	}
        }
	}


    public static final double POPULATION_DESTROYED_PER_DAMAGE = 0.05;
    public static final double FACTORIES_DESTROYED_PER_DAMAGE = 0.05;
    public static final double QTY_WEAPONS_IN_PLANTRY_WEAPON = 2;

	public void setPlanetReference(PlanetReference v) { planet = v; }
	public void setPlanet(PlanetReference v) { planet = v; }
	public Planet getPlanet() { return planet.getPlanet(); }
	public PlanetReference getPlanetReference() { return planet; }

	// convenience
	public Player getOwner() throws ApplicationError { return planet.getPlanet().getOwner(); }
    public Player getOriginPlayer() throws ApplicationError {  return planet.getPlanet().getOrigin();  }
    public String getName() { return planet.getPlanet().getName(); }


	public BattlePlanet(Planet p, Battle battle) throws ApplicationError {
        super(battle);
        setName(p.getName());
		planet = new PlanetReference(p);
        setShields(p.getShields(), true);
        setWeapons(p.getWeapons(), true, true, null);
        setSpecials(p);
        planetGraphics = PlanetGraphicsFactory.getPlanetGraphics(getPlanet().getTeraforming());
	}

	public Size getSize() throws ApplicationError {
    	double terraforming = planet.getPlanet().getTeraforming();
        return PlanetGraphicsFactory.getPlanetGraphics(terraforming).getImageSize();
    }
    
    public boolean isOwnedBy(Player owner) throws ApplicationError {
        return getPlanet().isOwnedBy(owner);
    }

	/**
	 * Ends this turn on the battle and starts a new turn.
	 * @throws ApplicationError 
	 */
	public void startTurn() throws ApplicationError {
		rechargeWeapons();
		rechargeShields();
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		if (planet != null) {
			planet.toXML(writer);
		}
	}

    /**
     * Returns a description of this object. This is used to display text in the info field of a UI.
     */
    public String getDescription() {
        Planet p = getPlanet();
        double pop = p.getPopulation();
        double shieldPower = 0;
        double shieldMaxPower = 0;
        double blocked = 0;
        if (shields != null) {
            shieldPower = getShieldPower();
            shieldMaxPower = p.getShields().getPower();
            blocked = p.getShields().getDamageBlocked();
        }
        String weaponsDescription = "-";
        if (getPlanet().getWeapons() != null) {
            weaponsDescription = p.getWeapons().getDescription(false);
        }
        return "Planet "+getName()+" POP "+Formatter.whole(pop)+"  DEFENCE "+Formatter.fraction(blocked)+" ["+Formatter.fraction(shieldPower)+" of "+Formatter.fraction(shieldMaxPower)+"]  ATTACK "+weaponsDescription;
    }

    /**
     * Removed the weapon passed in from this planet (battle planet and the actual 
     * planet object). If multiple weapons of this type exist then the first in the
     * list is removed.
     * @param weapon The weapon to remove.
     */
    public void removeWeapon(Weapon weapon) throws ApplicationError {
        Weapons weapons = getPlanet().getWeapons();
        weapons.remove(weapon);
        setWeapons(weapons, true, true, null);
    }
    
    /**
     * This much damage is inflicted on this planet.
     * The shields are ignored - This method is for use after the shields have been used
     * to inflict the final damage. For details of how damage effects a planet see the details 
     * for attribute damage.
     */
    public WeaponDamage damage(double d, Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("BattlePlanet.damage() Planet "+getName()+" damage="+d);
        }
        damage += d;

        // Destroy weapons        
        Weapons weapons = getPlanet().getWeapons();
        int qtyWeapons = 0;
        if (weapons != null) {
            Weapon w = weapons.getMaxDamageAttackWeapon();
            if (w == null) {
            	w = weapons.getWeaponAtIndex(0);
            }
            while (w != null && w.getCostToBuild() < damage) {
                if (Logger.isDebug()) {
                    Logger.logDebug("BattlePlanet.damage() Planet "+getName()+" destroy weapon "+w.getName());
                }
                weaponStateDirty = true;
                removeWeapon(w);
                damage -= w.getCostToBuild();
                w = weapons.getMaxDamageAttackWeapon();
                qtyWeapons++;
            }
        }
        
        if (weapons != null && weapons.getQuantity() > 0) {
            return new WeaponDamageGeneric(L.WEAPONS+"x"+Formatter.fraction(qtyWeapons)+" "+L.DESTROYED);
        }
        
        // Destroy shields        
        Shields shields = getPlanet().getShields();
        int qtyShields = 0;
        if (shields != null) {
            Shield s = shields.getBestShield();
            while (s != null && s.getCostToBuild() < damage) {
                if (Logger.isDebug()) {
                    Logger.logDebug("BattlePlanet.damage() Planet "+getName()+" destroy shield "+s.getName());
                }
                shields.remove(s);
                setShields(shields, true);
                damage -= s.getCostToBuild();
                s = shields.getBestShield();
                qtyShields++;
            }
        }
        if (shields != null && shields.getQuantity() > 0) {
            return new WeaponDamageGeneric(L.SHIELDS+"x"+Formatter.fraction(qtyShields)+" "+L.DESTROYED);
        }
        
        // Destroy scanners        
        Scanners scanners = getPlanet().getScanners();
        int qtyScanners = 0;
        if (scanners != null) {
            Scanner s = scanners.getBestScanner();
            while (s != null && s.getCostToBuild() < damage) {
                if (Logger.isDebug()) {
                    Logger.logDebug("BattlePlanet.damage() Planet "+getName()+" destroy scanner "+s.getName());
                }
                scanners.remove(s);
                damage -= s.getCostToBuild();
                s = scanners.getBestScanner();
                qtyScanners++;
            }
        }
        if (scanners != null && scanners.getQuantity() > 0) {
            return new WeaponDamageGeneric(L.SCANNERS+"x"+Formatter.fraction(qtyScanners)+" "+L.DESTROYED);
        }
        
        // Destroy Population / Factories
        double factories = getPlanet().getQtyFactories();
        factories = factories - damage * FACTORIES_DESTROYED_PER_DAMAGE;
        if (factories < 0) factories = 0;
        if (Logger.isDebug()) {
            Logger.logDebug("BattlePlanet.damage() Planet "+getName()+" destroy factories "+getPlanet().getQtyFactories()+" => "+factories);
        }
        getPlanet().setQtyFactories(factories);

        double qtyPeopleKilled = damage * POPULATION_DESTROYED_PER_DAMAGE;
        killPopulation(qtyPeopleKilled, agressor);
        
        if (battle != null && isDestroyed()) {
        	MotEngine.instance.multicast(new BattleObjectNotificationEF(this, L.COLONY_DESTROYED, BattleObjectNotificationType.generic, null, null), battle.getStartingPlayers());
        } else {
        }
        
        return new WeaponDamageGeneric(Formatter.fraction(d)+" ["+L.POPULATION+" "+Formatter.fraction(qtyPeopleKilled)+" "+L.KILLED+"]");
    }
    
    public static int estimateQtyPopulationKilledByDamage(int damage) {
    	return  (int) (damage * Bombardment.BOMBARD_DAMAGE_FACTOR * POPULATION_DESTROYED_PER_DAMAGE); 
    }
    
    private static final float QTY_SOLDIERS_PER_POP_KILLED = 1.5f;
    
    /**
     * Kills qty people on the planet. If qty >= the population then the colony is destroyed.
     * @param qty The quantity if people to destroy.
     * @param agressor The player who has killed the people.
     */
    public void killPopulation(double qtyPeopleKilled, Player agressor) throws ApplicationError {
        double population = getPlanet().getPopulation();
        if (population == 0.0) return;
        population = population - qtyPeopleKilled;
        if (population < 0.0) population = 0.0;
        if (Logger.isDebug()) {
            Logger.logDebug("BattlePlanet.killPopulation() Planet "+getName()+" destroy population "+getPlanet().getPopulation()+" => "+population);
        }
        getPlanet().setPopulation(population);
        
        damage = 0.0;
        
        int qtyMarinesKilled =  (int)(qtyPeopleKilled * QTY_SOLDIERS_PER_POP_KILLED);
        if (qtyMarinesKilled < 1) qtyMarinesKilled = 1;
        int qtyMarines = getPlanet().getQtyGroundDefense()-qtyMarinesKilled;
        if (qtyMarines < 0) {
            qtyMarines = 0;
        }
        getPlanet().setQtyGroundDefense(qtyMarines);
        
        if (population == 0.0) {
            Player oldOwner = planet.getPlanet().getOwner();
            planet.getPlanet().removeColony(true);
            if (oldOwner != null) {
                oldOwner.notifyColonyLost(planet.getPlanet(), agressor, true);
                Player[] players = MotEngine.instance.getPlayers().getPlayers();
                for (int i = 0; i < players.length; i++) {
					if (players[i] instanceof ComputerPlayer && players[i] != oldOwner && players[i] != agressor) {
						((ComputerPlayer)players[i]).getStratagist().notifyOfGenocide(planet.getPlanet(), agressor, oldOwner);
					}
				}
            }
            MotEngine.getInstance().broadcast(new DestroyColonyEF(getPlanet()));
        } else {
        	if (planet.getPlanet().getCrisis() == null) {
        		CrisisFactory.createEmergencyCrisis(planet.getPlanet());
        	}
        }
    }
    
    
    /**
     * No rotation is required for this object. 
     * @throws ApplicationError 
     */
    public int getDirection() throws ApplicationError {
    	return 270; 
    }
    
    /**
     * Returns the image to used to display this planet in a battle.
     * @param player The player who owns the window displaying this image.
     * @return The name of the image to use.
     */
    public String getImage(boolean isVisible) {
		return planetGraphics.getImage();
    }
    
    /**
     * @return True if this planet does have a colony (population is 0).
     */
    public boolean isColony() {
        return (planet.getPlanet().getPopulation() > 0.0);
    }
    
    /**
     * @return True if this planet does not have a colony (population is 0).
     */
    public boolean isDestroyed() {
        return (planet.getPlanet().getPopulation() <= 0.0);
    }

    public double getCondition() {
		return 1.0;
	}
    
    public boolean controlIsAutomated() throws ApplicationError {
   		return getOwner().isComputerPlayer();
    }

    /**
     * This planet gets its next orders from the player in this battle.
     * For user players an event is sent to the user interface asking for orders.
     * For computer players new orders are set on the battle craft directly.
     */
    public void getNextOrders() throws ApplicationError {
        getPlanet().getOwner().setNextBattleOrders(this);
    }

    protected double getQtyWeaponsBonus() { return QTY_WEAPONS_IN_PLANTRY_WEAPON; }

	protected String getElementName() { return "battle-planet"; }

    public double getDamage() {
        return damage;
    }

    public void setDamage(double v) {
        damage = v;
    }

	public void destroy(Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
    }
    
    public boolean isColonyWithDefenses() {
        return (isColony() && (hasWeapons() || hasShields() || hasFightersToLaunch()));
    }
}