/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.ImageObjectSettings;
import mot.common.designs.CraftDesign;
import mot.common.technology.MiscTechnologies;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.event.BattleCraftNewEF;
import mot.engine.players.Player;

public abstract class BattlePrimaryObject extends BattleObject {

    /**
     * This is true if this has fighters and has launched them in this battle and 
     * false otherwise.
     */
    private boolean launchedFighters = false;
    

	public BattlePrimaryObject(Battle battle) {
		super(battle);
	}

    public boolean hasCommandModule() {
        Specials specials = getSpecials();
        if (specials == null) return false;
        return specials.containsGroup(Special.GROUP_COMMAND_CENTER);
    }

	/**
	 * @return true if this ship is carrying fighters AND has fighters to launch.
	 */
    public boolean hasFightersToLaunch() {
        Specials specials = getSpecials();
        if (specials == null) return false;
        return (specials.getQtyFighters() != null && specials.getQtyFighters().intValue() > 0 && !hasLaunchedFighters());
    }

    public BattleCraft [] launchFighters() throws ApplicationError {
        Specials specials = getSpecials();
        if (specials == null || specials.getQtyFighters() == null || specials.getQtyFighters().intValue() == 0) {
            return null;
        }
        if (launchedFighters) {
            return null;
        }

        Battle battle = getBattle();
        Player owner = getOwner();
        Orbit orbit = battle.getOrbit();
        CraftDesign fighterDesign = owner.getFighterDesign();
        if (fighterDesign == null) {
            Logger.logWarning("BattleCraft.launchFighters() Unable to design fighter craft");
            return null;
        }
        
        int qty = specials.getQtyFighters().intValue();
        boolean isPlanet = this instanceof BattlePlanet;
		if (isPlanet) {
        	qty = qty * 2;
        }
		
		CraftDesign suicideDesign = null;
		int qtySuicide = 0;
		Weapons discoveredWeapons = owner.getWeapons();
		MiscTechnologies discoveredMiscTechnologies = owner.getMiscTechnologies();
		if (isPlanet && discoveredMiscTechnologies != null && discoveredWeapons != null) {
			boolean discoveredDetonator = discoveredWeapons.getQuantityWithGroup(Weapon.GROUP_DETONATOR) > 0;
			double suicidePercent = discoveredMiscTechnologies.getSuicidePercent();
			if (discoveredDetonator && suicidePercent > 0.0) {
				qtySuicide = (int) (qty * suicidePercent);
				suicideDesign = owner.getSuicideFighterDesign();
			}
		}
		
        BattleCraft[] fighters = _launchFighters(battle, owner, orbit, fighterDesign, qty, suicideDesign, qtySuicide);
        
        launchedFighters = true;
        weaponStateDirty = true;
        
		Player[] startingPlayers = battle.getStartingUserPlayers();
		for (int i = 0; i < fighters.length; i++) {
			BattleCraftNewEF event = new BattleCraftNewEF(fighters[i], false);
			MotEngine.instance.multicast(event, startingPlayers);
		}
        
        return fighters;
    }

	public BattleCraft[] _launchFighters(Battle battle, Player owner, Orbit orbit, CraftDesign fighterDesign, int qty, CraftDesign suicideDesign, int qtySuicide) throws ApplicationError {
		int totalQty = qty + qtySuicide;
		BattleCraft [] fighters = new BattleCraft[totalQty];
        for (int i=0; i<totalQty; i++) {
        	if (i < qty) {
        		fighters[i] = _launchFighter(battle, owner, orbit, fighterDesign);
        	} else if (suicideDesign != null){
        		fighters[i] = _launchFighter(battle, owner, orbit, suicideDesign);
        	}
        }
		return fighters;
	}

	private BattleCraft _launchFighter(Battle battle, Player owner, Orbit orbit, CraftDesign fighterDesign) throws ApplicationError {
        Craft fighterCraft = CraftFactory.build(fighterDesign, owner, orbit);
        BattleCraft fighter = battle.join(fighterCraft, true);
        fighter.setFighter(true);
        int radius = getImageRadius() + fighter.getImageRadius() + ImageObjectSettings.INTER_CRAFT_GAP;
        boolean isPlanet = this instanceof BattlePlanet;
        int direction = isPlanet ? 90 : getDirection();
		SquareBattleLayoutManager slm = new SquareBattleLayoutManager(battle, getLocation(), direction, radius);
        slm.setNextFreeLocation(fighter);
        fighter.setDirection(direction);
        fighter.startTurn();
        return fighter;
	}
	
    public void setLaunchedFighters(boolean launchedFighters) {
        this.launchedFighters = launchedFighters;
    }


    public boolean hasLaunchedFighters() {
        return launchedFighters;
    }

	public abstract Specials getSpecials();

    /**
     * Instructs this object to get its next orders from its controller in this battle.
     * For user players an event is sent to the user interface asking for orders.
     * For computer players new orders are set on the battle craft directly.
     */
    public abstract void getNextOrders() throws ApplicationError;

    /**
     * Note - something does not have to be owned  by a computer player to be controlled by one!
     * See MindControl! 
     * @return True iff this is controlled by a computer player.
     * @throws ApplicationError
     */
    public abstract boolean controlIsAutomated() throws ApplicationError;
    
}
