/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.Location;
import mot.common.designs.CraftDesign;
import mot.common.designs.WeaponBaysDesign;
import mot.common.technology.Computer;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.Engines;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.engine.crafts.Craft;
import mot.engine.players.Player;

/**
 * This factory builds a craft design or battle craft from a hull. Note that although
 * is does not HAVE to be, the hull should be of group station.
 * The craft design generated will be based upon this hull with the best shields and beam
 * weapons - but no engines.
 */
public class BattleSentinelFactory  {

    private static final String SENTINEL_ENGINE = "_SentinelEngine_";

	/**
     * Create a Sentinels for this battle
     * @throws ApplicationError 
     */
	public static void createSentinels(Battle battle, Player owner, int qtySentinels) throws ApplicationError {
		int interval = GalaxyDefinitions.BATTLE_BOUNDARY.getHeight() / (qtySentinels+1);	
        CraftDesign cd = createBattleSentinelDesign(owner, true, true, true);
        
        if (cd == null) {
        	Logger.logWarning("Unable to create sentinels");
        }
        
        int xLoc = interval;
        for (int i = 0; i < qtySentinels; i++) {			
        	// Create the station behind the Battle Station
        	// Note this is not added to the owners list of crafts!!
        	Craft station = new Craft(owner, cd, battle.getOrbit());
        	station.arrivedAt(battle.getOrbit(), false);
        	
        	// Create a new battle sentinel
        	BattleStation battleSentinal= new BattleStation(battle, station, false, null);
        	battleSentinal.setup();
        	
        	battleSentinal.setLocation(new Location(500, xLoc));
        	battleSentinal.setDirection(90);
            battle.add(battleSentinal);

            Logger.logWarning("BattleSentinelFactory.createSentinels()  player  "+battleSentinal);
            
            xLoc += interval;
		}
	}
	
    public static CraftDesign createBattleSentinelDesign(Player player, boolean withShields, boolean withComputers, boolean withScanners) throws ApplicationError {
        CraftDesign cd = new CraftDesign("BattleSentinel");
        
        Hull hull = Technologies.getHulls().getBiggestSentinelHull();
        if (hull == null) {
        	Logger.logWarning("BattleSentinelFactory.createBattleSentinelDesign() No Sentinel Hull");
        	return null;
        }
        cd.setHull(hull);
        
        Engines engines = Technologies.getEngines();
        Engine engine = engines.getEngine(SENTINEL_ENGINE);
		if (engine == null) {
	        engine = new Engine();
	        engine.setName(SENTINEL_ENGINE);
	        engine.setMaxTravelDistance(0);
	        engine.setWarpSpeed(0);
	        engine.setImpulseSpeed(0);
	        engine.setTurningSpeed(180);
	        engine.setSpace(0);
	        engine.setCostToBuild(0);
	        engine.setCostToResearch(0);
	        engine.setGroup("AntiMatter");
	        engines.add(engine);
		}
		cd.setEngine(engine);
        
        if (withComputers && player.getComputers() != null) {
            Computer computer = player.getComputers().getMostPowerfulComputer();
            cd.setComputers(new Computers(computer));
        }

        if (withShields && player.getShields() != null) {
            Shield shield1 = player.getShields().getBestDefenseShield();
            if (shield1 != null) {
            	cd.add(shield1);
            }
        }

        if (withScanners && player.getScanners() != null) {
            Scanner scanner = player.getScanners().getBestScanner();
            if (scanner != null) {
                cd.setScanners(new Scanners(scanner));
            }
        }
        
        Weapons weapons = player.getWeapons();
        if (weapons == null || weapons.getQuantity() == 0) {
            Logger.logWarning("BattleSentinelFactory.createBattleSentinelDesign()  player  "+player.getName()+" does not have any weapons");
            return null;
        }
        
        WeaponBaysDesign weaponBays = new WeaponBaysDesign();
        cd.setWeaponBays(weaponBays);
        
        int remainingSpace = cd.getRemainingSpace();
        
        Weapon weapon = player.getWeapons().getMaxDamageAttackWeaponWithInfinateShots();
        if (weapon == null) {
            Logger.logWarning("BattleSentinelFactory.createBattleSentinelDesign()  player  "+player.getName()+" does not have any beam weapons");
            return cd;
        }
        
        int weaponSpace = weapon.getSpace();
        int qtyWeapons = Comp.max(1, remainingSpace / weaponSpace);
        for (int i=0; i<qtyWeapons; i++) {
        	weaponBays.addWeapon(weapon, 0);
        }

        Weapon antiMissileWeapon = player.getWeapons().getMaxDamageAntiMissileWeaponNotRetarget();
        if (antiMissileWeapon != null) {
        	weaponBays.addWeapon(antiMissileWeapon, 0);
        	weaponBays.addWeapon(antiMissileWeapon, 0);
        }

        if (withShields && player.getShields() != null) {
            Shield shield2 = player.getShields().getBestPsiShield();
            if (shield2 != null) {
            	cd.add(shield2);
            }
        }
        
        if (Logger.isFull()) {
            cd.log(player.getName());
        }
        
        return cd;
    }

}