/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.designs.Orbital;
import mot.common.util.ApplicationError;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.planets.OrbitalReference;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * A Battle Station is a huge orbital space station around a planet. In a battle it acts as a
 * craft with no engines (it cannot move) but with shields and weapons. Battle stations are 
 * constructed automatically at the start of each battle.
 */
public class BattleStation extends BattleCraft {

    /**
     * This is the reference to the orbital that this battle station represents.
     */
    private OrbitalReference orbital;

    public BattleStation(Battle battle, Craft craft, boolean attacker, OrbitalReference orbital) throws ApplicationError {
        super(battle, craft, attacker, true);
        this.orbital = orbital;
    }

    /**
     * This overrides the normal implementation in BattleCraft.
     * Battle Stations always have a very low manoverability because they do not have engines.
     * This means they gen heavy damage from enemy fire, but when attacking an enemy engines 
     * are not taken into account. This implementation simply uses the difference in computer
     * power.
     * The advantage is limited to 1.5 rather than 2.0 in the normal implementation.
     * @return The manoeuvrability advantage this object has over the target object.
     */
	public double getManouverabilityAdvantage(BattleObject target) {
        if (target instanceof BattleCraft) {
            BattleCraft bc = (BattleCraft) target;
            double cp1 = (getCraft().getComputers() == null) ? 0 : getCraft().getComputers().getPower();
            double cp2 = (bc.getCraft().getComputers() == null) ? 0 : bc.getCraft().getComputers().getPower();
            if (cp2 < cp1/2) cp2 = cp1/2;
            if (cp1 < cp2/2) cp1 = cp2/2;
            double manvAdvantage = cp1 / cp2;
            return manvAdvantage;
        } else {
            return 1.0;
        }
    }

	/**
	 * Destroy this battle station.
     * This method calls BattleCraft.destroy() to create a craft destroyed event and all other
     * usual actions when a craft is destroyed.
     * This method also removed the battle station from the planet in this battle.
	 */
	public void destroy(Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
		if (orbital != null) {
			Planet planet = orbital.getPlanet();
			if (planet != null) {
				planet.destroyOrbital();
			} else {
				StellarObject stellarObject = orbital.getStellarObject();
				if (stellarObject != null) {
					Player owner = getOwner();
					stellarObject.destroyOutpost(owner);
				}
			}
		}
        super.destroy(agressor, oldState, parent);
    }

    public void setOrbital(OrbitalReference orbital) {
        this.orbital = orbital;
    }

    public OrbitalReference getOrbital() {
        return orbital;
    }

    public Orbital getOrbitalObject() {
		if (orbital != null) {
			return orbital.getOrbital();
		} else {
			return null;
		}
    }

	public int getQtySentinals() throws ApplicationError {
    	Orbital orbital = getOrbitalObject();
    	if (orbital != null) {
    		return orbital.getQtySentinals();
    	} else {
    		return 0;
    	}
	}
    
}