/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.designs.WeaponBaysDesign;
import mot.common.technology.Computer;
import mot.common.technology.Computers;
import mot.common.technology.Hull;
import mot.common.technology.Modules;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Special;
import mot.common.technology.StationMissileBase;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.engine.common.SpaceStation;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.crafts.Craft;
import mot.engine.planets.OrbitalReference;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * This factory builds a craft design or battle craft from a hull. Note that although
 * is does not HAVE to be, the hull should be of group station.
 * The craft design generated will be based upon this hull with the best shields and beam
 * weapons - but no engines.
 */
public class BattleStationFactory  {

    private static final String STATION_MISSILE_BASE = "Station Missile Base";

    /**
     * Create a BattleStation or SpaceStation orbiting a Planet
     */
	public static BattleStation createBattleStation(Battle battle, Planet planet) throws ApplicationError {
        Orbital orbital = planet.getOrbital();
        Player owner = planet.getOwner();
        
        int qtyFighters = 0;
        if (orbital != null) {
        	Modules modules = orbital.getModuleTechs();
        	if (modules != null) {
        		qtyFighters = modules.getQtyFighterBays();
        	}
        }
        
        CraftDesign cd = createBattleStationDesign(orbital.getHullTech(), owner, true, true, true, qtyFighters);
        
        // Create the station behind the Battle Station
        // Note this is not added to the owners list of crafts!!
        Craft station = new Craft(owner, cd, battle.getOrbit());
        
		// Create a new battle station
		BattleStation battleStation = new BattleStation(battle, station, false, new OrbitalReference(planet));
		station.arrivedAt(battle.getOrbit(), false);

        return battleStation;
    }
    
    /**
     * Create a Outpost or SpaceStation orbiting a Sun
     */
    public static BattleStation createStellarObjectStation(Battle battle, StellarObjectOrbital soa) throws ApplicationError {
        Orbital orbital = soa.getOrbital();
        Player owner = soa.getOwner();
        
        boolean withShields = false;
        boolean withComputers = false;
        int qtyFighters = 0;
        
        if (soa instanceof SpaceStation) {
        	SpaceStation ss = (SpaceStation) soa;
        	withShields = true;
        	withComputers = true;
        	qtyFighters = soa.getQtyFighterBays();
        }
        
        CraftDesign cd = createBattleStationDesign(orbital.getHullTech(), owner, withShields, withComputers, true, qtyFighters);
        
        // Create the station behind the Battle Station
        // Note this is not added to the owners list of crafts!!
        Craft station = new Craft(owner, cd, battle.getOrbit());
        
		// Create a new battle station
		BattleStation battleStation = new BattleStation(battle, station, false, new OrbitalReference(battle.getOrbit().getStellarObject()));
		station.arrivedAt(battle.getOrbit(), false);

        return battleStation;
    }
    
    public static CraftDesign createBattleStationDesign(Hull hull, Player player, boolean withShields, boolean withComputers, boolean withScanners, int qtyFighterBays) throws ApplicationError {
        CraftDesign cd = new CraftDesign("BattleStation");
        cd.setHull(hull);
        
        if (withComputers && player.getComputers() != null) {
            Computer computer = player.getComputers().getMostPowerfulComputer();
            cd.setComputers(new Computers(computer));
        }

        if (withShields && player.getShields() != null) {
            Shield shield1 = player.getShields().getBestDefenseShield();
            if (shield1 != null) {
            	cd.add(shield1);
            }
        }

        if (withScanners && player.getScanners() != null) {
            Scanner scanner = player.getScanners().getBestScanner();
            if (scanner != null) {
                cd.setScanners(new Scanners(scanner));
            }
        }
        
        
        Weapons weapons = player.getWeapons();
        if (weapons == null || weapons.getQuantity() == 0) {
            Logger.logWarning("BattleStationFactory.createBattleStationDesign()  player  "+player.getName()+" does not have any weapons");
            return null;
        }
        
        WeaponBaysDesign weaponBays = new WeaponBaysDesign();
        cd.setWeaponBays(weaponBays);
        
        int remainingSpace = cd.getRemainingSpace();
        
        Weapon weapon = player.getWeapons().getMaxDamageAttackWeaponWithInfinateShots();
        if (weapon == null) {
            Logger.logWarning("BattleStationFactory.createBattleStationDesign()  player  "+player.getName()+" does not have any beam weapons");
            return cd;
        }
        
        int weaponSpace = weapon.getSpace();
        int qtyWeapons = Comp.max(1, remainingSpace / weaponSpace);
        for (int i=0; i<qtyWeapons; i++) {
        	weaponBays.addWeapon(weapon, 0);
        }
        
        if (player.getMiscTechnologies() != null && player.getMiscTechnologies().contains(STATION_MISSILE_BASE)) {
        	StationMissileBase missileBase = (StationMissileBase) player.getMiscTechnologies().get(STATION_MISSILE_BASE);
        	Weapon missile = player.getWeapons().getMaxDamageWeaponWithGroup(Weapon.GROUP_MISSILE);
        	if (missile != null) {
        		int qtyMissiles = missileBase.getQtyMissiles();
        		for (int i = 0; i < qtyMissiles; i++) {					
        			weaponBays.addWeapon(missile, 0);
				}
        	}
        }
        
        if (qtyFighterBays > 0) {
        	Special fighters = Technologies.getSpecials().getStationFighterBays();
        	if (fighters == null) {
        		Logger.logWarning("Space Station missing fighters");
        	} else {
        		cd.addSpecials(fighters, qtyFighterBays);
        	}
        }
        
        if (withShields && player.getShields() != null) {
            Shield shield2 = player.getShields().getBestPsiShield();
            if (shield2 != null) {
            	cd.add(shield2);
            }
        }
        
        if (Logger.isFull()) {
            cd.log(player.getName());
        }
        
        return cd;
    }

}