/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.engine.players.Player;

/**
 * This lays outs out crafts for a single player a line between locUp and locDown. Each craft is
 * placed as close to centerLoc as possible. This the lines are build as follows;
 *                !                  *                  !
 *                !                  **                 !
 *                !                 ***                 !
 *                !                 ****                !
 *                !                *****                !
 */
class BattlerLine  {

    public Player player;
    public Battle battle;
    public Location centerLoc;
    public Location locUp;
    public Location locDown;
    public int direction;
    public int shiftAmount;

    public BattlerLine(Player player, Battle battle, Location centerLoc, Location locUp, Location locDown, int direction, int shiftAmount) {
        this.player = player;
        this.battle = battle;
        this.centerLoc = centerLoc;
        this.locUp = locUp;
        this.locDown = locDown;
        this.direction = direction;
        this.shiftAmount = shiftAmount;
    }
    
    /**
     * Shifts this line toward the center by the amount passed in.
     * This method can be used to start beuilding a new line of crafts in front of the
     * last line when the last line is full.
     * @param amount The quantity of pixels to move the line
     */
    public void shiftLine(int amount) {
        if (direction == 90) {
            centerLoc.shift(amount, 0);
            locUp.shift(amount, 0);
            locDown.shift(amount, 0);
        } else if (direction == 270) {
            centerLoc.shift(-amount, 0);
            locUp.shift(-amount, 0);
            locDown.shift(-amount, 0);
        } else if (direction == 180) {
            centerLoc.shift(0, amount);
            locUp.shift(0, amount);
            locDown.shift(0, amount);
        } else if (direction == 0) {
            centerLoc.shift(0, -amount);
            locUp.shift(0, -amount);
            locDown.shift(0, -amount);
        }
    }
    
    /**
     * returns the next free location in the line that craft could be placed in, or
     * null if there is no such location!
     * @param craft The craft to place in the battle field.
     * @throws mot.common.util.ApplicationError
     */
    public Location getNextFreeLocation(BattleCraft craft) throws ApplicationError {
        craft.setLocation(locUp);
        Location loc1 = battle.getClosestFreeLocationInLine(centerLoc, craft);
        craft.setLocation(locDown);
        Location loc2 = battle.getClosestFreeLocationInLine(centerLoc, craft);
        
        if (loc1.equals(locUp) && loc2.equals(locDown)) {
            shiftLine(shiftAmount);
            loc1 = battle.getClosestFreeLocationInLine(centerLoc, craft);
            loc2 = battle.getClosestFreeLocationInLine(centerLoc, craft);
        }
        
        int d1 = loc1.getDistanceTo(centerLoc);
        int d2 = loc2.getDistanceTo(centerLoc);
        
        craft.setDirection(direction);
        if (d1 < d2) {
            return loc1;
        } else {
            return loc2;
        }
    }

    /**
     * Sets the craft location to the next free location in the line.
     * @param craft The craft to place in the battle field.
     * @throws mot.common.util.ApplicationError
     */
    public Location setNextFreeLocation(BattleCraft craft) throws ApplicationError {
        Location loc = getNextFreeLocation(craft);
        if (loc != null) {
            craft.setLocation(new Location(loc));
        }
        return loc;
    }

    /**
     * Sets the craft location to the next free location in the line in front of the current line.
     * @param craft The craft to place in the battle field.
     * @throws mot.common.util.ApplicationError
     */
    public Location setNextFreeLocationInFront(BattleCraft craft) throws ApplicationError {
        shiftLine(shiftAmount);
        Location loc = setNextFreeLocation(craft);
        shiftLine(-shiftAmount);
        return loc;
    }

    public void setBattle(Battle battle) {
        this.battle = battle;
    }


    public Battle getBattle() {
        return battle;
    }


    public void setCenterLoc(Location centerLoc) {
        this.centerLoc = centerLoc;
    }


    public Location getCenterLoc() {
        return centerLoc;
    }


    public void setLocUp(Location locUp) {
        this.locUp = locUp;
    }


    public Location getLocUp() {
        return locUp;
    }


    public void setLocDown(Location locDown) {
        this.locDown = locDown;
    }


    public Location getLocDown() {
        return locDown;
    }


    public void setPlayer(Player player) {
        this.player = player;
    }


    public Player getPlayer() {
        return player;
    }


    public void setDirection(int direction) {
        this.direction = direction;
    }


    public int getDirection() {
        return direction;
    }

}