/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import java.util.ArrayList;

import mot.client.coms.LocalServerConnection;
import mot.client.proxy.ProxyStore;
import mot.common.bean.Nebula;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftBase;
import mot.engine.crafts.Fleet;
import mot.engine.event.BattleStateUpdateContainerEF;
import mot.engine.event.NewBattleEF;
import mot.engine.planets.Planet;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.generals.CraftTask;

/**
 * This class contains the list of battles scheduled to be played at the end of
 * this game turn. There can only be a single battle per solar system - so if three
 * or more players have a battle at a single stellar object then they will all be
 * in the same battle.
 */
public class Battles extends XMLBean {

    private static final int MIN_TIME_BETWEEN_KAAN_RELEASES = 50;
    public static final int FIRST_TIME_BETWEEN_KAAN_RELEASES = 3100;
    
	/**
     * The list of battles in this battle list.
     */
    private ArrayList battleList;
    private transient Battle [] battles;

    /**
     * True if battle 0 in the list has been started.
     */
    private boolean battleActive = false;

    public ArrayList getBattleList() { return battleList; }
    public void setBattleList(ArrayList v) { battleList = v; }
    
    public Battles() {
	}

    public Battle [] getBattles() {
		if (battles == null && battleList != null && battleList.size() > 0) {
			battles = (Battle []) battleList.toArray(new Battle [battleList.size()]);
		}
		return battles;
	}

	public int getQtyBattles() {
        Battle [] battles = getBattles();
		if (battles == null) {
			return 0;
		} else {
			return battles.length;
		}
    }

    public Battle getBattleAt(Orbit orbit) {
		return getBattleAt(orbit, false);
	}

    /**
     * Returns the battle located at stellar object so. If no battle exists and create is
     * false it returns null, otherwise, if create is true it create a new battle at that
     * solar system and returns it.
     */
    public Battle getBattleAt(Orbit orbit, boolean create) {
		Battle [] battles = getBattles();
        if (battles != null) {
            for (int i=0; i<battles.length; i++) {
                if (battles[i].getActualOrbit() == orbit.getStellarObject()) {
                    return battles[i];
                }
            }
        }
		if (create) {
            Battle battle = new Battle(orbit);
            if (battleList == null) battleList = new ArrayList();
			battleList.add(battle);
            this.battles = null;
            return battle;
		} else {
			return null;
		}
	}

	public Battle getBattle(String name) {
		Battle [] battles = getBattles();
		if (battles != null) {			
			for (int i=0; i<battles.length; i++) {
				if ( Comp.same(battles[i].getName(), name) ) {
					return battles[i];
				}
			}
		}
		return null;
	}

    public static boolean contains(CraftBase cb) throws ApplicationError {
    	if (cb instanceof Craft) {
    		return contains((Craft)cb);
    	} else if (cb instanceof Fleet) {
    		Craft[] crafts = ((Fleet)cb).getCrafts();
    		if (crafts == null) return true;
    		for (int i = 0; i < crafts.length; i++) {
				if (contains(crafts[i])) {
					return true;
				}
			}
    		return false;
    	}
    	Logger.logWarning("Battles.contains() Does not work for "+cb);
    	return false;
    }
    
    public static boolean contains(Craft craft) throws ApplicationError {
		Orbit orbit = craft.getOrbit();
		if (orbit == null) {
			return false;
		}
		Battle battle = MotEngine.instance.getBattlesNotNull().getBattleAt(orbit, false);
		if (battle == null) {
			return false;
		} else {
			return battle.contains(craft);
		}
	}

    public static void join(Planet planet) throws ApplicationError {
		Battle battle = MotEngine.instance.getBattlesNotNull().getBattleAt(planet.getOrbit(), true);
		battle.join(planet);
	}

    public static void join(Orbit orbit, Craft craft) throws ApplicationError { join(orbit, craft, false); }
    public static void join(Orbit orbit, Craft [] crafts) throws ApplicationError { join(orbit, crafts, false); }
    public static void join(Orbit orbit, Fleet fleet) throws ApplicationError { join(orbit, fleet, false); }
    public static void join(Orbit orbit, CraftBase cb) throws ApplicationError { join(orbit, cb, false); }
    
    public static void join(Orbit orbit, CraftBase cb, boolean attacker) throws ApplicationError { 
        if (cb instanceof Craft) {
            join(orbit, (Craft) cb, attacker); 
        } else if (cb instanceof Fleet) {
            join(orbit, (Fleet) cb, attacker); 
        }
    }

    public static void join(Orbit orbit, Craft craft, boolean attacker) throws ApplicationError {
		Battle battle = MotEngine.instance.getBattlesNotNull().getBattleAt(orbit, true);
		battle.join(craft, attacker);
	}

    public static void join(Orbit orbit, Craft [] crafts, boolean attacker) throws ApplicationError {
		Battle battle = MotEngine.instance.getBattlesNotNull().getBattleAt(orbit, true);
		battle.join(crafts, attacker);
	}

    public static void join(Orbit orbit, Fleet fleet, boolean attacker) throws ApplicationError {
		Battle battle = MotEngine.instance.getBattlesNotNull().getBattleAt(orbit, true);
		battle.join(fleet, attacker);
	}

    public static void join(Orbit orbit, StellarObjectOrbital outpost) throws ApplicationError {
		Battle battle = MotEngine.instance.getBattlesNotNull().getBattleAt(orbit, true);
		battle.join(outpost);
	}

	public void removeCraftFromQueuedBattles(Craft craft) throws ApplicationError {
		Battle [] ba = getBattles();
		if (ba != null && ba.length > 0) {
			int start = battleActive ? 1 : 0;
			for (int i=start; i<ba.length; i++) {
				if (ba[i].contains(craft)) {
					ba[i].removeCraftFromQueuedBattle(craft);
					if (ba[i].hasEnded()) {
						battles = null;
				        battleList.remove(ba[i]);
					}
				}
			}
		}
	}

	public boolean playerIsInQueuedBattles(Player player) throws ApplicationError {
		Battle [] ba = getBattles();
		if (ba != null) {
			for (int i=0; i<ba.length; i++) {
				if (ba[i].contains(player)) {
					return true;
				}
			}
		}
		return false;
	}
	
	public void removePlayerFromQueuedBattles(Player player) throws ApplicationError {
		Battle [] ba = getBattles();
		if (ba != null) {
			for (int i=0; i<ba.length; i++) {
				if (ba[i].contains(player)) {
					ba[i].removePlayerFromQueuedBattle(player);
					if (ba[i].hasEnded()) {
						battles = null;
				        battleList.remove(ba[i]);
					}
				}
			}
		}
	}

	/**
	 * End this battle, sets the active flag to false and removes this battle from the list of 
     * battles. The next battle is not started you must call start() to start the next battle.
     * It also adds all winners of the battle to the list of visitor to this solar system (see
     * Battle.isWinner() for details). It also calls informBattleFinished for all craft's tasks
     * that survived this battle. This gives each crafts task the opportunity to asses its new
     * position.
	 */
	public void end(Battle b) throws ApplicationError {
        if (Logger.isDebug()) {
			Logger.logDebug("Battles.end() End "+b.getName()+" at "+b.getOrbit().getDescription());
			Logger.logDebug("Battles.end() ***************************************************************");
        }
        
        Player [] players = b.getStartingPlayers();
        for (int i=0; i<players.length; i++) {
        	boolean isWinner = b.isWinner(players[i]);
        	players[i].notifyBattleEnded(b, isWinner);
        }
        
        Craft [] crafts = b.getStartingCrafts();
        ArrayList<CraftTask> informed = new ArrayList();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                Craft craft = crafts[i];
                if (craft != null && craft.getTask() != null) {
                	if (!informed.contains(craft.getTask())) {
	                	informed.add(craft.getTask());
	                    craft.getTask().informBattleFinished(b);
                	}
                }
            }
        }
        
        if (b.getOrbit() != null && b.getOrbit().getStellarObject() != null) {
        	StellarObject so = b.getOrbit().getStellarObject();
        	Nebula nebula = so.getNebula();
        	int date = MotEngine.instance.getDate();
        	if (nebula != null && nebula.getDateOfLastKaanRelease() < (date - MIN_TIME_BETWEEN_KAAN_RELEASES) && MotEngine.instance.isSpaceKaanEnabled() && date > FIRST_TIME_BETWEEN_KAAN_RELEASES) {
        		// Release Nebula Kaan to Maraud the galaxy.
        		nebula.setDateOfLastKaanRelease(date);
        		MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
        		int qtyKaan = mp.getQtyKaans();
        		if (qtyKaan < 2) {
        			mp.dispatchKaan(nebula, so.getOwners());
        		}
        	}
        }

        _end(b);
        battleActive = false;

        // Start the next battle if one exists
        // Do not start the battle in this call stack as we are half way through a weapon fire
        //MotEngine.instance.startNextBattleInQueue();
        if (MotEngine.instance.hasBattleInQueue()) {
        	((LocalServerConnection)ProxyStore.getInstance().getConnection()).addStartNextBattleToQueue();
        } else {
        	MotEngine.instance.getPlayers().startTurn();
        }
	}

	public void _end(Battle b) throws ApplicationError {
		battles = null;
        battleList.remove(b);
	}
	

	/**
	 * Start the battles starting with battle 0.
     * Generates a StartBattleEvent for the battle starting.
	 */
	public Battle start(boolean updateNextCraftToMove) throws ApplicationError {
        Battle [] battles = getBattles();
        if (battles == null || battles.length == 0) {
            return null;
        }
        
        if (Logger.isDebug()) {
			Logger.logDebug("Battles.start() Start "+battles[0].getDescription());
        }
        
        Battle battle = battles[0];
        
        if (!battle.isStarted()) {        	
        	battle.start();
        	battleActive = true;
        	Player[] startingPlayers = battle.getStartingUserPlayers();
        	MotEngine.instance.multicast( new NewBattleEF(battle), startingPlayers );
        	
        	if (updateNextCraftToMove) {
        		BattleStateUpdateContainerEF event = new BattleStateUpdateContainerEF(battle.getName());
        		battle.getNextObjectToMoveCache().checkForUpdates(event);
        		MotEngine.instance.multicast(event, startingPlayers);
        	}
        }
		
        return battle;
	}
    
    /**
     * Returns the battle that is currently being fought.
     * Note that this is currently just the first battle in the list.
     */
    public Battle getActiveBattle() {
        Battle [] battles = getBattles();
        if (battleActive) {
            return battles[0];
        } else {
            return null;
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (getBattles() != null) {
			Battle [] battles = getBattles();
			for (int i=0; i<battles.length; i++) {
				if (battles[i] != null) {
					battles[i].toXML(writer);
				}
			}
		}
	}

	protected String getElementName() { return "battles"; }
}