/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.adapter.SolarSystemVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.crafts.Craft;
import mot.engine.crafts.WeaponBays;
import mot.engine.event.PlanetBombardedEF;
import mot.engine.planets.OrbitalReference;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRoute;
import mot.engine.planets.TradeRoutes;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * This is not actually part of a battle - but uses the BattleCraft object.
 * Conceptually it is sitting in orbit bombing a defenceless planet. The planet is assumes to 
 * have no shields and no weapons during bombardment. To same time endlessly attacking a planet
 * bombardment is like a express operation of the galaxy map to execute the attacking of
 * a planet.
 */
public class Bombardment  {

    public static final int BOMBARD_DAMAGE_FACTOR = 10;

    private BattlePlanet planet;
    private ArrayList crafts = new ArrayList();

    public Bombardment(Planet planet, Craft [] cl) throws ApplicationError {
        this.planet = new BattlePlanet(planet, null);
        add(cl);
    }

    public Bombardment(Planet planet, Craft craft) throws ApplicationError {
        this.planet = new BattlePlanet(planet, null);
        crafts.add(craft);
    }

    public void add(Craft [] cl) {
        for (int i=0; i<cl.length; i++) {
            add(cl[i]);
        }
    }
    
    public void add(Craft craft) {
        if (!crafts.contains(craft)) {
            crafts.add(craft);
        }
    }
    
    public void run() throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("Bombardment.run() planet="+planet.getName());
        }
        
        SolarSystem solarSystem = planet.getPlanet().getSolarSystem();
		TradeRoutes tradeRoutes = MotEngine.instance.getTradeRoutes();
		Planet[] effectedPlanets = tradeRoutes.getAllPlanetsEffectedByTradeRoutesIn(solarSystem);
		TradeRoute[] effectedTradeRoutes = MotEngine.instance.getTradeRoutes().getTradeRoutes(solarSystem);

        OrbitalReference originalMiningStation = planet.getPlanet().getMiningStation();
        double originalPop = planet.getPlanet().getPopulation();
        double originalFactories = planet.getPlanet().getQtyFactories();
        Player planetOwner = planet.getOwner();
        Player craftOwner = null;
        for (Iterator i = crafts.iterator(); i.hasNext(); ) {
            Craft craft = (Craft) i.next();
            craftOwner = craft.getOwner();
            WeaponBays weaponBays = craft.getWeaponBays();
            if (weaponBays != null) {            	
            	Weapon[] weapons = weaponBays.getAllWeapons();
            	for (int j=0; weapons != null && j < weapons.length; j++) {
            		double dam = weapons[j].getDamage();
            		planet.damage(dam*BOMBARD_DAMAGE_FACTOR, craftOwner, planet.getCraftState(), null);
            	}
            }
        }
        double bomPop = originalPop - planet.getPlanet().getPopulation();
        double bomFactories = originalFactories - planet.getPlanet().getQtyFactories();
        
        if (effectedTradeRoutes != null) {
        	for (int i = 0; i < effectedTradeRoutes.length; i++) {
        		tradeRoutes.remove(effectedTradeRoutes[i]);
			}
        }
        
        PlanetBombardedEF event1 = new PlanetBombardedEF(planet.getPlanet(), bomPop, bomFactories);
        if (planetOwner instanceof UserPlayer) {
        	event1.addStateUpdate(new SolarSystemVOAdapter(planet.getPlanet().getSolarSystem(), planetOwner));
        	event1.addStateUpdate(new PlanetVOAdapter(planet.getPlanet(), planetOwner));
        	if (originalMiningStation != null && originalMiningStation.getPlanet() != null) {
            	event1.addStateUpdate(new SolarSystemVOAdapter(originalMiningStation.getPlanet().getSolarSystem(), planetOwner));
            	event1.addStateUpdate(new PlanetVOAdapter(originalMiningStation.getPlanet(), planetOwner));
        	}
    		if (effectedPlanets != null) {
    			event1.addStateUpdate(EngineVoFactory.getPlanets(effectedPlanets, planetOwner) );
    		}
    		if (effectedTradeRoutes != null) {
    			event1.addStateUpdate(EngineVoFactory.getInvalidTradeRoutes(effectedTradeRoutes) );
    		}
        }
        
        PlanetBombardedEF event2 = new PlanetBombardedEF(planet.getPlanet(), bomPop, bomFactories);
        if (craftOwner instanceof UserPlayer) {
        	event2.addStateUpdate(new SolarSystemVOAdapter(planet.getPlanet().getSolarSystem(), craftOwner));
        	event2.addStateUpdate(new PlanetVOAdapter(planet.getPlanet(), craftOwner));        	
        	if (originalMiningStation != null && originalMiningStation.getPlanet() != null) {
            	event2.addStateUpdate(new SolarSystemVOAdapter(originalMiningStation.getPlanet().getSolarSystem(), craftOwner));
            	event2.addStateUpdate(new PlanetVOAdapter(originalMiningStation.getPlanet(), craftOwner));
        	}
    		if (effectedPlanets != null) {
    			event2.addStateUpdate(EngineVoFactory.getPlanets(effectedPlanets, craftOwner) );
    		}
    		if (effectedTradeRoutes != null) {
    			event2.addStateUpdate(EngineVoFactory.getInvalidTradeRoutes(effectedTradeRoutes) );
    		}
        }
        
        MotEngine.instance.broadcast(event1, planetOwner);
        MotEngine.instance.broadcast(event2, craftOwner);
    }


    public void setPlanet(BattlePlanet planet) {
        this.planet = planet;
    }


    public BattlePlanet getPlanet() {
        return planet;
    }


    public void setCrafts(ArrayList crafts) {
        this.crafts = crafts;
    }


    public ArrayList getCrafts() {
        return crafts;
    }
    
    
    public boolean contains(Craft craft) {
        return crafts.contains(craft);
    }
    
    public boolean containsAny(Craft [] l) {
        for (int i=0; i<l.length; i++) {
            if (crafts.contains(l[i])) {
                return true;
            }
        }
        return false;
    }
}