/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.CraftState;
import mot.common.bean.DecoratorList;
import mot.common.bean.Location;
import mot.common.bean.PlayerColor;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.engine.players.Player;

/**
 * CraftState class is container of the state of a craft viewable by a particular player.
 * NB - The state viewed by one player is often different that the state viewed by another player.
 * The server collects state in this StateFactory class and generated the right State of each player on request.
 */
public class CraftStateFactory {

	/**
	 * The owner of the craft
	 */
    private String owner;
    
    /**
     * The controller of the craft
     */
    private String controller;
    
    /**
     * The craft damage before this animations is taking place.
     */
    private double damagePercent;

    /**
     * The craft shield power before this animations is taking place.
     */
    private double shieldPercent;

    /**
     * The craft damage before this animations is taking place.
     */
    private double movementDamagePercent;

	/**
     * The craft shield power before this animations is taking place.
     */
    private double psiDamagePercent;

    /**
     * The location this craft was at.
     */
    private Location location;
    
    /**
     * The direction this craft is pointing in.
     */
    private int direction;
    
    /**
     * The quantity of crew in the craft.
     */
    private double crewPercent;
    
    /**
     * A list of image decoration to display on this object.
     */
	private ServerDecoratorList decorators;

    /**
     * String the name of the owner.
     */
    private String color;

    /**
     * A list of player names this craft is cloaked to on the battle field.
     */
	private String [] cloakedToPlayerNames;
	
	/**
	 * If this craft is destroyed
	 */
	private boolean isDestroyed;

	/**
	 * The amount of move remaining to this craft / object.
	 */
	private double moveRemainingAsPercent;
	
	public CraftStateFactory(String owner, String controller, double damagePercent, double shieldPercent, double movementDamagePercent, double psiDamagePercent, double crewPercent, Location loc, int direction, PlayerColor pc, String [] cloakedToPlayerNames, ServerDecoratorList decorators, boolean isDestroyed, double moveRemainingAsPercent) {
        this.owner = owner;
		this.controller = controller;
		this.damagePercent = damagePercent;
        this.shieldPercent = shieldPercent;
        this.movementDamagePercent = movementDamagePercent;
        this.psiDamagePercent = psiDamagePercent;
        this.crewPercent = crewPercent;
        this.location = (loc == null) ? null : new Location(loc);
        this.direction = direction;
        this.color = pc == null ? null : pc.getName();
        this.cloakedToPlayerNames = cloakedToPlayerNames;
        this.decorators = decorators == null ? null : new ServerDecoratorList(decorators);
        this.isDestroyed = isDestroyed;
        this.moveRemainingAsPercent = moveRemainingAsPercent;
	}
	
    public CraftStateFactory(String owner, String controller, double damagePercent, double shieldPercent, double movementDamagePercent, double psiDamagePercent, double crewPercent, Location loc, int direction, PlayerColor pc, String [] cloakedToPlayerNames, DecoratorList decorators, boolean isDestroyed, double moveRemainingAsPercent) {
        this.owner = owner;
		this.controller = controller;
		this.damagePercent = damagePercent;
        this.shieldPercent = shieldPercent;
        this.movementDamagePercent = movementDamagePercent;
        this.psiDamagePercent = psiDamagePercent;
        this.crewPercent = crewPercent;
        this.location = (loc == null) ? null : new Location(loc);
        this.direction = direction;
        this.color = pc == null ? null : pc.getName();
        this.cloakedToPlayerNames = cloakedToPlayerNames;
        this.decorators = decorators == null ? null : new ServerDecoratorList(decorators);
        this.isDestroyed = isDestroyed;
        this.moveRemainingAsPercent = moveRemainingAsPercent;
    }
    
	public void setDestroyed(boolean isDestroyed) {
		this.isDestroyed = isDestroyed;
	}

	public CraftState getCraftStateForGenericPlayer() {
   		return new CraftState(owner, controller, damagePercent, shieldPercent, movementDamagePercent, psiDamagePercent, crewPercent, location, direction, color, cloakedToPlayerNames, null, isDestroyed, moveRemainingAsPercent);
	}

	public CraftState getCraftState(Player viewer) throws ApplicationError {
		DecoratorList dl = null;
		if (decorators != null) {
			dl = decorators.getVisibleDecorators(viewer);
		}
   		return new CraftState(owner, controller, damagePercent, shieldPercent, movementDamagePercent, psiDamagePercent, crewPercent, location, direction, color, cloakedToPlayerNames, dl, isDestroyed, moveRemainingAsPercent);
	}
	
	public String toString() {
		return "CraftState { loc=" + location+ " dam%=" + damagePercent + " shield%=" + shieldPercent + "colour = " + color + " cloakedTo=" + Comp.toString(cloakedToPlayerNames) + " }";
	}



	/**
	 * You must call this method before passing the CraftState to each player - because the correct
	 * Decorator state must be generated!
	 * @param p
	 * @throws ApplicationError
	 */
//	public void updateDecoratorsFor(Player p) throws ApplicationError {
//		if (decorators == null) {
//			setDecorators( null );
//		} else {
//			setDecorators( decorators.getVisibleDecorators(p) );
//		}
//	}

}
