/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.technology.Engines;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.event.BattleDroidMineMoveEF;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.event.MotEventFactory;
import mot.engine.players.Player;
import mot.engine.technology.WeaponInstance;

/**
 * This is a mine fired from a Droid Catapult Weapon on a craft / planet 
 * during a battle. Unlike a normal mine it targets a particular craft,
 * and moves towards its target. When it gets close enough to its target
 * it detonates like a normal mine. They move slow speed so crafts can
 * out run them, but it means the craft cannot stop and wait for a period
 * of time without being caught.
 */
public class DroidMine extends MovingAutamaton {
    
	/**
	 * The damage this object has sustained
	 */
	private double damage;
	
    public DroidMine(BattleObject origin, BattleObject target, WeaponInstance w, BattleObject link) throws ApplicationError {
		super(MotEngine.instance.getNameGenerator().getNextName("DroidMine"), origin, target, w, link);
    }
    
    public static final float SPEED_BONUS = 0.5f;
    public static final int DAMAGE_TO_DESTROY_A_DROID_MINE = 20;
    
    /**
     * @returns The distance this missile may move each turn.
     */
    public int getSpeed() throws ApplicationError {
        int speed = 0;
        Engines engines = getOwner().getEngines();
        if (engines != null && engines.getQuantity() > 0) {
            speed += (int)(engines.getFastestEngine().getImpulseSpeed() * SPEED_BONUS);
        }
        if (speed < 25) speed = 25;
        return speed;
    }
    
    public double getDamageToDestroy() {
        WeaponInstance[] weapons = getWeapons();
        if (weapons == null || weapons.length == 0) {
            return DAMAGE_TO_DESTROY_A_DROID_MINE;
        } else {
            return weapons[0].getDamage() + DAMAGE_TO_DESTROY_A_DROID_MINE;
        }
    }
    
    public WeaponDamage damage(double amount, Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
        double damageToDestroy = getDamageToDestroy();
        damage += amount;
        if (Logger.isDebug()) {
            Logger.logDebug("DroidMine.damage() "+getName()+" amount="+amount+" (total Damage "+damage+")");
        }
        if (damage >= damageToDestroy) {
            destroy(null, getCraftState(), parent);
        }
        return null;
    }
    
	private static ImageConfig imageConfig = null;
	
    public Size getSize() throws ApplicationError {
		if (imageConfig == null) {
			imageConfig = ImageConfigFactory.getInstance().getImageConfig("DroidMine");
		}
		return imageConfig.getImageSize();
    }

    protected String getElementName() { return "droid-mine"; }

	protected MotEventFactory createMovementEvent(Location from, Location to, CraftStateFactory targetState) {
		return new BattleDroidMineMoveEF(this, from, to, targetState);
	}

}