/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.designs.CraftDesign;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.event.BattleDestroyedEF;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.players.Player;

/**
 * A HoloCraft is a 'fake' craft in a battle. It only has Fake Weapons and can
 * only be used as a decoy to distract the enemy. Crafts that contain holo
 * emiters automatically deploy holo crafts when a new battle starts.
 * 
 * Holo Crafts are linked to their parent crafts, and are destroyed when fired
 * at or their parents are destroyed.
 */
public class HoloCraft extends BattleCraft {

	private static final float HOLOCRAFT_INITIATIVE_BONUS = 1.5f;
	private Craft parent;

	/**
	 * Create an instance of a HoloCraft.
	 * 
	 * @param attacker
	 *            True if the parent is an attacking craft.
	 * @param parent
	 *            The parent craft that contains a holo emiter that is
	 *            projecting (creating) this holo craft.
	 * @param battle
	 *            The battle this craft is part of.
	 * @param cd
	 *            The Design this holo craft will be an instance of.
	 */
	public HoloCraft(Battle battle, Craft parent, boolean attacker, CraftDesign cd) throws ApplicationError {
		super(battle, CraftFactory.build(cd, parent.getOwner(), battle.getOrbit()), attacker);
		this.parent = parent;
	}

	/**
	 * Hologram crafts do not take damage. This destroys the craft.
	 */
	public WeaponDamage damage(double damage, Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
		if (Logger.isDebug()) {
			Logger.logDebug("HoloCraft.damage() craft " + getName() + " dam " + getCraft().getDamage() + " + " + damage);
		}
		destroy(agressor, oldState, parent);
		return null;
	}

	/**
	 * Holo craft have a slight initiative bonus - so they usually move first.
	 */
	public int getInitiative() throws ApplicationError {
		int r = super.getInitiative();
		return (int) (r * HOLOCRAFT_INITIATIVE_BONUS);
	}
	
	public BattleDestroyedEF getCraftDestroyedEvent(CraftStateFactory oldState) throws ApplicationError {
		return new BattleDestroyedEF(this, oldState, true);
	}

	protected boolean destructonCausesWreckage() {
		return false;
	}

	public void setParent(Craft parent) {
		this.parent = parent;
	}

	public Craft getParent() {
		return parent;
	}

}