/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.players.Player;

/**
 * This Layouts crafts and planets in lines.
 * Objects are layed out from the center of the map in a middle out strategy.
 * Players lined up on the left and the right of the map have their crafts arranged
 * top to bottom so the location is shifted along the Y axis.
 * Players lined up on the top and the bottom of the map have their crafts arranged
 * left to right so the location is shifted along the X axis.
 * The Map should look something like;
 * 
 *          C C
 *      
 *                  C
 *   C              C
 *   C              C
 *                  C
 *                  
 *          C
 * 
 * This shows 4 players in the battle each with their craft starting at difference coordinates.
 */
public class LinedBattleLayoutManager implements BattleLayoutManager  {

    /**
     * The battle this manager is laying out.
     */
    private Battle battle;
    
    private BattlerLine left = null;
    private BattlerLine right = null;
    private BattlerLine top = null;
    private BattlerLine bottom = null;

    // constants!
    int shiftAmount = 100;
    
    public LinedBattleLayoutManager(Battle battle) throws ApplicationError {
        this.battle = battle;
    }
    
    /**
     * Note this layout manager only lays out a maximum of four players. This returns null
     * if there are already four players with crafts and this craft does not belong to any 
     * of them.
     * The defending player should be the owner of the first craft for which a line becuase
     * the first line allocated is on the left.
     * @param object The craft that needs laying out.
     * @return The BattlerLine that can be used to create the starting location for a craft
     * in a battle.
     */
    private BattlerLine getLineFor(Player owner) throws ApplicationError {
        if (left == null) {
            left = new BattlerLine( owner,
                                    battle, 
                                    new Location(tenPercentLeft, my), 
                                    new Location(tenPercentLeft, tenPercentTop), 
                                    new Location(tenPercentLeft, tenPercentBottom),
                                    90,
                                    shiftAmount);
            return left;
        }
        
        if (left.getPlayer() == owner) {
            return left;
        }

        if (right == null) {
            right = new BattlerLine( owner,
                                    battle, 
                                    new Location(tenPercentRight, my), 
                                    new Location(tenPercentRight, tenPercentTop), 
                                    new Location(tenPercentRight, tenPercentBottom),
                                    270,
                                    shiftAmount);
            return right;
        }
        
        if (right.getPlayer() == owner) {
            return right;
        }

        if (top == null) {
            top = new BattlerLine( owner,
                                    battle, 
                                    new Location(mx, tenPercentTop), 
                                    new Location(tenPercentLeft, tenPercentTop), 
                                    new Location(tenPercentRight, tenPercentTop),
                                    180,
                                    shiftAmount);
            return top;
        }
        
        if (top.getPlayer() == owner) {
            return top;
        }

        if (bottom == null) {
            bottom = new BattlerLine( owner,
                                    battle, 
                                    new Location(mx, tenPercentBottom), 
                                    new Location(tenPercentLeft, tenPercentBottom), 
                                    new Location(tenPercentRight, tenPercentBottom),
                                    0,
                                    shiftAmount);
            return bottom;
        }
        
        if (bottom.getPlayer() == owner) {
            return bottom;
        }
        
        return null;
    }

    /**
     * Places the battle craft in the next free location on the map.
     * For location type details see BattleLayoutManager.
     */
    public void setNextFreeLocation(BattleCraft craft) throws ApplicationError {
        BattlerLine line = getLineFor(craft.getOwner());
        
        if (line == null) {
            Logger.logWarning("LinedBattleLayoutManager.setNextFreeLocation() No free line for craft "+craft.getName()+" *** REMOVED FROM BATTLE ***");
            battle.remove(craft);
            return;
        }

        Location loc;
        if (craft instanceof BattleStation) {
            loc = line.setNextFreeLocationInFront(craft);    
        } else {
            loc = line.setNextFreeLocation(craft);    
        }
        
        if (Logger.isDebug()) {
            Logger.logDebug("LinedBattleLayoutManager.setNextFreeLocation() craft "+craft.getName()+"@"+loc+" - "+craft.getOwner().getName());
        }
    }

    /**
     * Places the planet onto the battlefield.
     * Note - The planet should always belong to the left player.
     */
    public void setPlanetLocation(BattlePlanet planet) throws ApplicationError {
        BattlerLine line = getLineFor(planet.getOwner());
        Location loc = new Location(line.getCenterLoc());
        planet.setLocation(loc);

        if (Logger.isDebug()) {
            Logger.logDebug("LinedBattleLayoutManager.setPlanetLocation() craft "+planet.getName()+"@"+loc);
        }
    }
}