/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.client.swing.config.ImageConfig;
import mot.client.swing.config.ImageConfigFactory;
import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.technology.Engines;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.event.BattleMissileMoveEF;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.event.MotEventFactory;
import mot.engine.players.Player;
import mot.engine.technology.WeaponInstance;

/**
 * This is a group of missiles fired by a single craft headed for a single target.
 * Each turn the move() method should be called - this will move the missiles to 
 * the target and when they reach it the missile will fire the its weapons and destroy
 * its self.
 */
public class Missiles extends MovingAutamaton {

    /**
     * The damage that has been inflicted on this automaton by weapon fire.
     */
    private double damage;

    public Missiles(BattleObject origin, BattleObject target, WeaponInstance w, BattleObject link) throws ApplicationError {
		super(MotEngine.instance.getNameGenerator().getNextName("Missile"), origin, target, w, link);
    }

    public static final float SPEED_BONUS = 1.5f;
    public static final int DAMAGE_TO_DESTROY_A_MISSILE = 10;
    
    /**
     * @returns The distance this missile may move each turn.
     */
    public int getSpeed() throws ApplicationError {
        int speed = 0;
        Engines engines = getOwner().getEngines();
        if (engines != null && engines.getQuantity() > 0) {
            speed += (int)(engines.getFastestEngine().getImpulseSpeed() * SPEED_BONUS);
        }
        if (speed < 250) speed = 250;
        return speed;
    }
    
    public double getDamageToDestroy() {
        WeaponInstance[] weapons = getWeapons();
        if (weapons == null || weapons.length == 0) {
            return DAMAGE_TO_DESTROY_A_MISSILE;
        } else {
            return Comp.floor((float)weapons[0].getDamage()) + DAMAGE_TO_DESTROY_A_MISSILE;
        }
    }
    
    public WeaponDamage damage(double amount, Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
        double damageToDestroy = getDamageToDestroy();
        int qtyMissilesDestroyed = (int)((damage + amount) / damageToDestroy);
        damage = ((damage + amount) % damageToDestroy);
        int qty = weapons[0].getQty() - qtyMissilesDestroyed;
        if (qty < 0) qty = 0;
        if (Logger.isDebug()) {
            Logger.logDebug("Misssiles.damage() "+getName()+" damage "+amount+" (qty "+weapons[0].getQty()+"=>"+qty+")");
        }
        weapons[0].setQty(qty);
        if (qty == 0) {
            destroy(agressor, getCraftState(), parent);
        }
		return new WeaponDamageGeneric(Formatter.fraction(amount)+" ["+Formatter.fraction(qtyMissilesDestroyed)+" "+L.DESTROYED+"]");

    }
    
	private static ImageConfig imageConfig = null;
	
    public Size getSize() throws ApplicationError {
		if (imageConfig == null) {
			imageConfig = ImageConfigFactory.getInstance().getImageConfig("Missile");
		}
		return imageConfig.getImageSize();
    }

    protected String getElementName() { return "missiles"; }

	protected MotEventFactory createMovementEvent(Location from, Location to, CraftStateFactory targetState) {
		return new BattleMissileMoveEF(this, from, to, targetState);
	}


	public double getDamage() {
		return damage;
	}

	public void setDamage(double damage) {
		this.damage = damage;
	}
	
}