/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.engine.common.ServerImageObject;

public class MovementCostCalculator {

	private int movement;
	private int turn;

	public MovementCostCalculator(BattleCraft bc) throws ApplicationError {
		this(bc, bc.getMoveRemaining());
	}

	public MovementCostCalculator(BattleCraft bc, double maxMovePercent) throws ApplicationError {
		double speed = bc.getSpeed();
		int turningSpeed = bc.getTurningSpeed();
		movement = (int)(speed*maxMovePercent);
		turn = (int)(turningSpeed*maxMovePercent);
	}
	
	public MovementCostCalculator(int turn, int movement) {
		this.movement = movement;
		this.turn = turn;
	}

	public Location getClosestLocationTo(BattleObject bc, Location to) throws ApplicationError {
		Location from = bc.getLocation();
		int requiredDirection = from.getDirectionFor(to);
		int dirDelta = Location.getDirectionDeltaPos(bc.getDirection(), requiredDirection);
		int maxDist = getMovementRemainingAfterTurn(dirDelta);
		Location actual = bc.getClosestFreeSpaceTo_NO_DIR_CHANGE(to, maxDist);
		return actual;
	}
	
	public boolean canMoveTo(ServerImageObject bc, Location to) throws ApplicationError {
		return canMoveFromTo(bc.getLocationAndDirection(), to);
	}
	
	public boolean canMoveFromTo(LocationAndDirection from, Location to) {
		int requiredDirection = from.getDirectionFor(to);
		int dirDelta = Location.getDirectionDeltaPos(from.getDirection(), requiredDirection);
		int maxDist = getMovementRemainingAfterTurn(dirDelta);
		int dist = from.getDistanceTo(to);
		return maxDist >= dist;
	}
	
	public int getMovementRemainingAfterTurn(BattleObject bc, Location turnToLocation) throws ApplicationError {
		Location from = bc.getLocation();
		int requiredDirection = from.getDirectionFor(turnToLocation);
		int dirDelta = Location.getDirectionDeltaPos(bc.getDirection(), requiredDirection);
		int maxDist = getMovementRemainingAfterTurn(dirDelta);
		return maxDist;
	}
	
	public int getScoreToTarget(ServerImageObject craft, Location to, Location targetLoc) throws ApplicationError {
		Location from = craft.getLocation();
		int requiredDirection1 = from.getDirectionFor(to);
		int dirDelta1 = Location.getDirectionDeltaPos(craft.getDirection(), requiredDirection1);
		int maxDist = getMovementRemainingAfterTurn(dirDelta1);
		int dist = from.getDistanceTo(to);
		if (maxDist < dist) {
			return Integer.MAX_VALUE;
		}
		int requiredDirection2 = to.getDirectionFor(targetLoc);
		int dirDelta2 = Location.getDirectionDeltaPos(craft.getDirection(), requiredDirection1);
		if (canTurn(dirDelta2, maxDist / movement)) {
			return to.getDistanceTo(targetLoc);
		} else {
			return craft.getSpaceX()*2 + to.getDistanceTo(targetLoc); 
		}
	}
	
	public int getMovementRemainingAfterTurn(int turnAmount) {
		double t = Comp.pos(turnAmount);
		if (t < 0.1) {
			return movement;
		}
		double turnPercent = t / turn;
		if (turnPercent > 0.99) {
			return 0;
		}
		double r = movement - movement*turnPercent;
		return (int) r;
	}
	
	public boolean canTurn(int turnAmount, double percentTurnAvailable) {
		double t = Comp.pos(turnAmount);
		if (t < 0.1) {
			return true;
		}
		int maxTurn = (int) (turn * percentTurnAvailable);
		return maxTurn > turnAmount;
	}
	
	public int getMovement() {
		return movement;
	}

	public void setMovement(int movement) {
		this.movement = movement;
	}

	public int getTurn() {
		return turn;
	}

	public void setTurn(int turn) {
		this.turn = turn;
	}
	
}
