/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.Location;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.event.BattleDestroyedEF;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.event.MotEventFactory;
import mot.engine.event.MotLinkedEventFactory;
import mot.engine.players.Player;
import mot.engine.technology.WeaponFireResult;
import mot.engine.technology.WeaponInstance;
import mot.engine.technology.WeaponInstanceDetonator;

/**
 * This is a base class for types of automaton that move towards a target then
 * attack when they arrive using their weapons.
 * It contains common code between Droid mine and missiles!
 */
public abstract class MovingAutamaton extends Automaton {

	/**
     * This is the object this automaton is aimed at and wishes to destroy.
     */
    private BattleObject target;
    
    /**
     * The Craft this automaton was fired from.
     */
    private BattleObject origin;
    
    /**
     * This is the player that owns these missiles 
     * This is important to remember because the origin may be destroyed, 
     * change owner or is planet become a non colony.  
     */
    private Player owner;
    
    
    private boolean hasHitTarget = false;
    private boolean destroyed = false;
    
    
    public MovingAutamaton(String name, BattleObject origin, BattleObject target, WeaponInstance w, BattleObject link) throws ApplicationError {
		super(origin.getBattle(), name, new Location(origin.getLocation()), link);
        this.target = target;
        this.origin = origin;
        this.owner = origin.getOwner();
        
        setWeapons(w.getWeapon(), w.getQty(), false, false, null);
        startTurn();
    }

    // for test purposes
	public void _performMove(double percent, MotLinkedEventFactory rootEvent) throws ApplicationError {
		performMove(percent, rootEvent);
	}
	
	protected void performMove(double percent, MotLinkedEventFactory rootEvent) throws ApplicationError {
        if (target.isDestroyed()) {
            if (Logger.isDebug()) {
                Logger.logDebug("MovingAutomiton.move() "+getName()+" Target "+target.getName()+" has been destroyed");
            }
            destroy(true, getCraftState(), null);
            return;
        }
        
        if (Logger.isDebug()) {
            String percentStr = ""+percent;
            if (percentStr.length() > 4) percentStr = percentStr.substring(0,5);
            Logger.logDebug("MovingAutomiton.move("+percentStr+") "+getName()+" @ "+getLocation()+" => "+target.getName()+" @ "+target.getLocation()+" [ dist "+getLocation().getDistanceTo(target.getLocation())+" ]");
            if (isDestroyed()) {
            	Logger.logError("", new ApplicationError("IS DESTROYED"));
            }
        }
        
        int dist = getLocation().getDistanceTo(target.getLocation()) - target.getImageRadius();
        int speed = getSpeed();
        int distRemaining = (int) (percent*speed);
        if (dist < distRemaining) {
            attack(target, rootEvent);
        } else {
            moveTo(target.getLocation(), target, distRemaining, rootEvent);
        }
    }

	/**
	 * Moves this missile towards the location dest a distance of distance and generates a
     * move missile event to animate this movement.
	 * The turning cost of a craft can be calculated by
	 * (total degrees to turn) * (hull turning cost) / (engine turn power) / (player character turning bonus)
	 */
	public void moveTo(Location dest, BattleObject target, int distance, MotLinkedEventFactory rootEvent) throws ApplicationError {
		if ( !GalaxyDefinitions.BATTLE_BOUNDARY.contains(dest) ) {
			dest = GalaxyDefinitions.BATTLE_BOUNDARY.getClosestEdgeTo(dest);
		}
        Location old = getLocationClone();
		shiftWithOverlap(dest, distance);
        
    	MotEventFactory event = createMovementEvent(old, getLocationClone(), target.getCraftState());
        if (rootEvent == null) {
            MotEngine.instance.multicast( event, battle.getStartingUserPlayers() );
        } else {
            rootEvent.link( event );
        }
    }
	
	protected abstract int getSpeed() throws ApplicationError;
    
    /**
     * @return The percentage of this object will have to perform of a move before it strikes its 
     * target. A missile with speed 300, of distance 200 from its target would return 0.66666.
     */
    public double getPercentMoveBeforeStrike() throws ApplicationError {
    	if (target instanceof BattleCraft) {
    		BattleCraft tc = (BattleCraft) target;
    		int targetSpeed = tc.getSpeed();
    		int targetDirection = tc.getDirection();
    		int automatonSpeed = getSpeed();
    		int automatonDirection = getLocation().getDirectionFor(target.getLocation());
    		int dirDelta = Comp.getDirectionChangeFor(automatonDirection, targetDirection);
    		double aggregateSpeed = 0.0;
    		if (dirDelta > 135 || dirDelta < -135) {
    			// missile and target are pointing in opposite directions
    			aggregateSpeed = targetSpeed + automatonSpeed;
    		} else if (dirDelta > 45 || dirDelta < -45) {
    			// missile is coming at target from the side 
    			aggregateSpeed = automatonSpeed * 1.1;
    		} else {
    			// missile are target heading is same direction (missile must catch up target)
    			aggregateSpeed = automatonSpeed - targetSpeed;
    			if (aggregateSpeed < 0) {
    				// craft is faster than missile
    				return 1.0;
    			}
    		}
    		double dist = getLocation().getDistanceTo(target.getLocation());
    		return dist / aggregateSpeed;
    	} else {
    		double speed = getSpeed();
    		double dist = getLocation().getDistanceTo(target.getLocation());
    		return dist / speed;
    	}
    }
    
    protected abstract MotEventFactory createMovementEvent(Location from, Location to, CraftStateFactory targetState);
    
    /**
     * This explodes the missile causing it to damage the target battle object and be
     * destroyed its self. A Missile Explosion automation is generated as well as the
     * usual damage animations.
     */
    public void attack(BattleObject bo, MotLinkedEventFactory rootEvent) throws ApplicationError {
    	if (Logger.isDebug()) {
    		Logger.logDebug("MovingAutomaton.attack() "+this+"->"+target);
    	}
    	MotEventFactory moveEvent = createMovementEvent(getLocationClone(), target.getLocationClone(), target.getCraftState());
        if (rootEvent == null) {
            MotEngine.instance.multicast( moveEvent, battle.getStartingUserPlayers());
        } else {
            rootEvent.link( moveEvent );
        }
        
        hasHitTarget = true;

        getLocation().shiftTo(target.getLocation());
        CraftStateFactory oldTargetState = target.getCraftState();
        int qtyToFire = weapons[0].getQty();
        WeaponFireResult result = weapons[0]._fire(this, target, 1.0, qtyToFire, false);
        if (result != null) {
        	CraftStateFactory newSourceState = getCraftState();
        	newSourceState.setDestroyed(true);
        	String damageDescription = Formatter.getDamageDescription(result.getDamage(), result.getShieldBlockedDamage());
            BattleWeaponFireEF event = new BattleWeaponFireEF(this, target, weapons[0].getWeapon(), result.getDamage(), result.getShieldBlockedDamage(), null, damageDescription, false, null, oldTargetState, newSourceState, null);
            MotEngine.instance.getProxyManager().scheduleMulticast(event, battle.getStartingUserPlayers());
            if (result.getDamage() > 0.0) {
                WeaponDamage damage = target.damage(result.getDamage(), owner, oldTargetState, null);
                if (damage != null) {
                	event.setTargetDescription(damage.getDescription());
                }
            }
            if (target.isDestroyed()) event.setDestroyed(true);
            event.setNewTargetState(target.getCraftState());
            MotEngine.instance.getProxyManager().sendScheduledEvent(event);
        }
        destroy(false, getCraftState(), null);
    }
    
	public void strike(MotLinkedEventFactory rootEvent) throws ApplicationError {
		attack(target, rootEvent);
	}

    
    /**
     * This automaton should be destroyed if it is targeting the object passed in and there is no alternative target.
     * @param bo The object that has been destroyed.
     * @throws mot.common.util.ApplicationError
     */
    public void notifyGoneFromBattle(BattleObject bo, BattleDestroyedEF destroyedEvent) throws ApplicationError {
        if (target == bo && !hasHitTarget) {
            BattleObject alternativeTarget = null;
            if (origin.getSpecials() != null && origin.getSpecials().hasMissileRetarget()) {
            	alternativeTarget = getBattle().getClosestCraftOwnedBy(bo, target.getOwner());
            }
        	if (alternativeTarget != null) {
        		if (Logger.isDebug()) {
        			Logger.logDebug("MovingAutomiton.notifyDestroyed() name "+getName()+" Old Target "+bo.getName()+" New Target "+alternativeTarget.getName());
        		}
        		target = alternativeTarget;
        		target.add(this);
        	} else {
        		destroy(true, getCraftState(), destroyedEvent);
        	}
        }
    }

	public void destroy(Player agressor, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
		destroy(true, oldState, parent);
	}
	
    /**
	 * Destroy this craft - Causes it to explode during the battle.
	 * If the destroyedEvent is not null then the destruction of this automaton is linked to the
	 * destruction of the battle object whose destroy event is passed in.  A linked animation 
	 * containing the destruction of this automaton is added to the event. 
	 */
	public void destroy(boolean produceEvent, CraftStateFactory oldState, LinkedEventFactoryBase parentEvent) throws ApplicationError {
		if (destroyed) {
			return;
		}
        if (Logger.isDebug()) {
            Logger.logDebug("MovingAutomiton.destroy() name "+getName());
        }
		Battle battle = getBattle();
		_destroy();
        if (hasDetonator() && !battle.hasEnded()) {
        	// Must not fire detonator if this object is being destroyed because 
        	// the battle has ended!
            WeaponInstanceDetonator.explode(this);
        } else if (parentEvent != null && produceEvent) {
        	parentEvent.link(new BattleDestroyedEF(this, getCraftState(), false));
        } else if (produceEvent) {
        	MotEngine.instance.multicast(new BattleDestroyedEF(this, getCraftState(), false), battle.getStartingUserPlayers());
        }
	}

	private void _destroy() {
		destroyed = true;
		Battle battle = getBattle();
		battle.remove(this);
        target.remove(this);
		
	}
	
	/**
	 * A missile is always visible
	 */
    public boolean isVisible(Player player) {
		return true;
	}

    /**
     * @return The direction this missile is pointing in. Missile have no turning
     * costs so they are always pointing towards the target.
     */
    public int getDirection() {
		Location loc = getLocation();
		return loc.getDirectionFor(target.getLocation());
    }
    
    public int getDirection(Location loc) {
		return loc.getDirectionFor(target.getLocation());
    }
    
    public Player getOwner() throws ApplicationError {
        return owner;
    }
    
    public Player getOriginPlayer() throws ApplicationError {
        return owner;
    }
    
    public boolean isOwnedBy(Player p) throws ApplicationError {
        return owner == p;
    }
    
    public boolean isDestroyed() {
        return !getBattle().contains(this);
    }

    public double getCondition() {
		return 1.0;
	}
    
    protected abstract double getDamageToDestroy();
    
    public String getDescription() {
        Weapon w = weapons[0].getWeapon();
        return getName()+" "+w.getDamage()+" X "+weapons[0].getQty()+"  Target "+target.getName();
    }

    public int getQty() {
        Weapon w = weapons[0].getWeapon();
        return weapons[0].getQty();
    }

    public void dischargeWeapons() {
        // I have no weapons because I am a weapon!
    }

    public void retargetTo(BattleObject newTarget) {
    	target.remove(this); 
        target = newTarget;
    	target.add(this); 
    }
    
    public void setTarget(BattleObject target) {
        this.target = target;
    }

    public BattleObject getTarget() {
        return target;
    }

    public void setOrigin(BattleObject origin) {
        this.origin = origin;
    }


    public BattleObject getOrigin() {
        return origin;
    }

	public boolean isTargetingPlayer(Player player) throws ApplicationError {
		return target.isOwnedBy(player);
	}

    public String toString() { return getName()+"@"+getLocation(); }

}