/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.command_results.StateUpdateCommandResult;
import mot.common.event.StateContainer;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.BattleCraftSpeedUpdateVO;
import mot.common.vo.BattleNextObjectToMoveVO;
import mot.common.vo.BattleObjectWeaponStateUpdateVO;
import mot.common.vo.BattleObjectWeaponStateVO;
import mot.common.vo.ValueObjectList;
import mot.engine.MotEngine;
import mot.engine.adapter.BattleObjectWeaponStateVOAdapter;
import mot.engine.event.BattleStateUpdateContainerEF;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

public class NextObjectToMoveCache {

	private Battle battle;
	
	private BattlePrimaryObject lastNextObject;

	public NextObjectToMoveCache(Battle battle) {
		this.battle = battle;
	}

	public void checkForUpdates(StateContainer event) throws ApplicationError {
        BattlePrimaryObject next = battle.getNextToMove(true);
        logState(next);

        BattleNextObjectToMoveVO stateUpdate = null;
        if (next != lastNextObject) {
        	lastNextObject = next;
        	stateUpdate = new BattleNextObjectToMoveVO(battle.getName(), next.getName());
        	event.addStateUpdate(stateUpdate);
        }
        
        ValueObjectList weaponStateUpdates = getWeaponStateUpdates();
        if (weaponStateUpdates != null) {
        	event.addStateUpdate(weaponStateUpdates.getStateUpdates());
        }
        
	}

	public void checkForUpdates(StateUpdateCommandResult cr, Player owner) throws ApplicationError {
        BattlePrimaryObject next = battle.getNextToMove(true);
        logState(next);
        
        BattleNextObjectToMoveVO stateUpdate = null;
        if (next != lastNextObject) {
        	lastNextObject = next;
        	stateUpdate = new BattleNextObjectToMoveVO(battle.getName(), next == null ? null : next.getName());
        	cr.addStateUpdate(stateUpdate);
        }
        
        ValueObjectList weaponStateUpdates = getWeaponStateUpdates();
        if (weaponStateUpdates != null) {
        	cr.addStateUpdate(weaponStateUpdates);
        }
        
        BattleCraftSpeedUpdateVO[] speedUpdates = battle.getSpeedUpdates();
        if (speedUpdates != null) {
        	cr.addStateUpdate(speedUpdates);
        }
        
        if (stateUpdate != null || weaponStateUpdates != null) {        	
        	BattleStateUpdateContainerEF stateUpdateEvent = new BattleStateUpdateContainerEF(battle.getName());
        	if (stateUpdate != null) {
        		stateUpdateEvent.addStateUpdate(stateUpdate);
        	}
        	if (weaponStateUpdates != null) {
        		stateUpdateEvent.addStateUpdate(weaponStateUpdates);
        	}
	        if (speedUpdates != null) {
	        	stateUpdateEvent.addStateUpdate(speedUpdates);
	        }
        	
        	Player[] startingPlayers = battle.getStartingUserPlayers();
        	for (int i = 0; i < startingPlayers.length; i++) {
        		if (startingPlayers[i] != owner && startingPlayers[i] instanceof UserPlayer) {
        			MotEngine.instance.broadcast(stateUpdateEvent, startingPlayers[i]);
        		}
        	}
        }
	}

	private void logState(BattlePrimaryObject next) {
		Logger.logDebug("Last Next = "+lastNextObject+" New Next = "+next);
	}

	private ValueObjectList getWeaponStateUpdates() throws ApplicationError {
        BattleObject[] boa = battle.getBattleObjects();
        if (boa == null) return null;
        ValueObjectList weaponStateUpdates = null;
        for (int i = 0; i < boa.length; i++) {
           	if (boa[i].clientsNeedWeaponStateUpdate()) {
               	BattleObjectWeaponStateVO ws = new BattleObjectWeaponStateVOAdapter(boa[i]);
               	BattleObjectWeaponStateUpdateVO wsu = new BattleObjectWeaponStateUpdateVO(battle.getName(), boa[i].getName(), ws);
               	if (weaponStateUpdates == null) {
               		weaponStateUpdates = new ValueObjectList();
               	}
               	weaponStateUpdates.addStateUpdate(wsu);
               	boa[i].clientsWeaponStateUpdated();
            }
		}
        return weaponStateUpdates;
	}

	public BattlePrimaryObject getLastNextObject() {
		return lastNextObject;
	}

	public void setLastNextObject(BattlePrimaryObject lastNextObject) {
		this.lastNextObject = lastNextObject;
	}

}
