/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.util.ApplicationError;
import mot.engine.players.Player;

/**
 * This is a cache for the bonuses a player's craft is receiving in the battle that may change
 * during the battle. These bonuses need to be recalculated every time something happens that
 * may effect their value, eg
 *   Fleet Bonuses must be re-calculated each time a craft is deleted.
 */
public class PlayerBattleBonuses {

    /**
     * BattleTurningBonus that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetBattleTurningBonus;

    /**
     * BattleSpeedBonus that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetBattleSpeedBonus;

    /**
     * AttackerModification that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetAttackerModification;

    /**
     * DefenderModification that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetDefenderModification;

    /**
     * RangeModification that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetRangeModification;

    /**
     * InitiativeBonus that applies to all crafts owned by the same player in a battle.
     */
    private Double fleetInitiativeBonus;
    
	public Double getFleetBattleTurningBonus() {
		return fleetBattleTurningBonus;
	}

	public void setFleetBattleTurningBonus(Double fleetBattleTurningBonus) {
		this.fleetBattleTurningBonus = fleetBattleTurningBonus;
	}

	public Double getFleetBattleSpeedBonus() {
		return fleetBattleSpeedBonus;
	}

	public void setFleetBattleSpeedBonus(Double fleetBattleSpeedBonus) {
		this.fleetBattleSpeedBonus = fleetBattleSpeedBonus;
	}

	public Double getFleetAttackerModification() {
		return fleetAttackerModification;
	}

	public void setFleetAttackerModification(Double fleetAttackerModification) {
		this.fleetAttackerModification = fleetAttackerModification;
	}

	public Double getFleetDefenderModification() {
		return fleetDefenderModification;
	}

	public void setFleetDefenderModification(Double fleetDefenderModification) {
		this.fleetDefenderModification = fleetDefenderModification;
	}

	public Double getFleetRangeModification() {
		return fleetRangeModification;
	}

	public void setFleetRangeModification(Double fleetRangeModification) {
		this.fleetRangeModification = fleetRangeModification;
	}

	public void update(Player p, Battle battle) throws ApplicationError {
		BattleCraft[] crafts = battle.getCrafts(p);
		nullify();
		if (crafts == null) {
			return;
		}
		for (int i = 0; i < crafts.length; i++) {
			Specials specials = crafts[i].getSpecials();
			if (specials != null) {
				Special s;
				s = specials.getBestFleetAttackerModification();
				if (s != null && (fleetAttackerModification == null || fleetAttackerModification < s.getFleetAttackerModification())) {
					fleetAttackerModification = s.getFleetAttackerModification();
				}
				s = specials.getBestFleetDefenderModification();
				if (s != null && (fleetDefenderModification == null || fleetDefenderModification < s.getFleetDefenderModification())) {
					fleetDefenderModification = s.getFleetDefenderModification();
				}
				s = specials.getBestFleetBattleSpeedBonus();
				if (s != null && (fleetBattleSpeedBonus == null || fleetBattleSpeedBonus < s.getFleetBattleSpeedBonus())) {
					fleetBattleSpeedBonus = s.getFleetBattleSpeedBonus();
					fleetBattleTurningBonus = s.getFleetBattleTurningBonus();
				}
				s = specials.getBestFleetRangeModification();
				if (s != null && (fleetRangeModification == null || fleetRangeModification < s.getFleetRangeModification())) {
					fleetRangeModification = s.getFleetRangeModification();
				}
				s = specials.getBestFleetInitiativeBonus();
				if (s != null && (fleetInitiativeBonus == null || fleetInitiativeBonus < s.getFleetInitiativeBonus())) {
					fleetInitiativeBonus = s.getFleetInitiativeBonus();
				}
			}
		}
	}

	public Double getFleetInitiativeBonus() {
		return fleetInitiativeBonus;
	}

	public void setFleetInitiativeBonus(Double fleetInitiativeBonus) {
		this.fleetInitiativeBonus = fleetInitiativeBonus;
	}
    
	private void nullify() {
		fleetBattleTurningBonus = null;
		fleetBattleSpeedBonus = null;
		fleetAttackerModification = null;
		fleetDefenderModification = null;
		fleetRangeModification = null;
		fleetInitiativeBonus = null;
	}

}
