/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.SalvageList;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Specials;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapons;
import mot.common.util.Utils;
import mot.engine.crafts.Craft;
import mot.engine.crafts.WeaponBays;

/**
 * When a craft has been destroyed in a battle wreckage is created. This wreckage 
 * can contain technology that is available for salvage by participants in the battle.
 * This factory created the list of salvage from a destroyed battle craft. 
 */
public class SalvageFactory {

	/**
	 * @return The percentage of technology in this craft that can be salvaged.
	 */
	public static Double getPercentToSalvage(BattleCraft battleCraft) {
		Craft craft = battleCraft.getCraft();
		double r = 1.0;
		WeaponBays weaponBays = craft.getWeaponBays();
		if (weaponBays != null) {
			Weapons weapons = weaponBays.getAllWeaponsAsList();
			if (weapons != null) {
				double weaponsEffect = weapons.getSalvageEffect();
				r = r * weaponsEffect;
			}
		}
		Specials specials = craft.getSpecials();
		if (specials != null) {
			double specialsEffect = specials.getSalvageEffect();
			r = r * specialsEffect;
		}
		if (r == 1.0) {
			return null;
		} else {
			return r;
		}
	}
	
	public static SalvageList create(BattleCraft battleCraft) {
		
		Double percentToSalvage = getPercentToSalvage(battleCraft);

		int qty = Utils.random(3);
		if (qty == 0) return new SalvageList();
		
		Craft craft = battleCraft.getCraft();
		TechnologyList available = new GeneralTechnologyList();
		available.add(craft.getHull());
		available.add(craft.getEngine());
		available.add(craft.getComputers());
		available.add(craft.getScanners());
		available.add(craft.getShields());
		available.add(craft.getSpecials());
		WeaponBays weaponBays = craft.getWeaponBays();
		if (weaponBays != null) {
			available.add(weaponBays.getAllWeapons());
		}
		SalvageList salvage = battleCraft.getSalvage();
		if (salvage != null) {
			available.add(salvage.getAllAsArray());
		}
		
		SalvageList selected = new SalvageList();
		
		if (percentToSalvage == null || Utils.random(percentToSalvage)) {
			Technology tech = available.getRandom();
			selected.add(tech);
			available.remove(tech);
		}
		
		if (qty == 2 && available.getQuantity() > 0) {
			if (percentToSalvage == null || Utils.random(percentToSalvage)) {
				Technology tech = available.getRandom();
				selected.add(tech);
			}
		}
		
		return selected;
	}

}
