/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.Decorator;
import mot.common.bean.DecoratorCommandCenter;
import mot.common.bean.DecoratorDamageBot;
import mot.common.bean.DecoratorDisabled;
import mot.common.bean.DecoratorHologram;
import mot.common.bean.DecoratorList;
import mot.common.bean.DecoratorPlague;
import mot.common.bean.DecoratorSuicideCraft;
import mot.common.util.ApplicationError;
import mot.engine.battles.triggers.DamageBot;
import mot.engine.battles.triggers.Plague;
import mot.engine.players.Player;


/**
 * This is a list of all decorators that are applicable to a given craft.
 * Because not all decorators should be displayed to all players, contains
 * two separate decorator lists - 
 * 
 * 	The owner list. This is the list of decorators that should be displayed 
 * 	to the owner
 * 
 *  The all list. This is the list of decorators that should be displayed to
 *  everybody.
 *  
 *  Note - The owner list MUSt contain all the decorators in the all list!
 */
public class ServerDecoratorList {

	/**
	 * The owner of this decorator list
	 */
	private BattleObject owner;
	
	/**
	 * List of decorators to display to the owner (in a window view by the owner 
	 * of the battle object)
	 */
	private DecoratorList ownerList;
	
	/**
	 * List of decorators to display to all players (in a window view by the owner 
	 * of the battle object)
	 */
	private DecoratorList allList;
	
	public ServerDecoratorList(BattleObject owner) {
		this.owner = owner;
		update();
	}
	
	public ServerDecoratorList(ServerDecoratorList decorators) {
		owner = decorators.getOwner();
		ownerList = decorators.getOwnerListCopy();
		allList = decorators.getAllListCopy();
	}

	public ServerDecoratorList(DecoratorList decorators) {
		allList = decorators;
	}

	public void update() {
		if (owner != null) {
			if (ownerList != null) ownerList.removeAll();
			if (allList != null) allList.removeAll();
	        if (owner instanceof HoloCraft) {
	            addOwner(DecoratorHologram.getInstance());
	        }
	        if (owner.hasDetonator()) {
	            addOwner(DecoratorSuicideCraft.getInstance());
	        }
	        if (owner instanceof BattleCraft) {
	        	BattleCraft craft = (BattleCraft) owner;
		        if (!craft.isOperating()) {
		            addBoth(DecoratorDisabled.getInstance());
		        }
		        if (craft.hasCommandModule()) {
		            addOwner(DecoratorCommandCenter.getInstance());
		        }
	        	BattleCraftController controllerDetails = craft.getControllerDetails();
				if (controllerDetails != null) {
					Decorator decorator = controllerDetails.getDecorator();
		            addBoth(decorator);
	        	}
				Decorator decorator = craft.getBattle().getTriggersNotNull().getDecotator(craft);
				if (decorator != null) {
					if (decorator.visibleToAll()) addAll(decorator);
					addOwner(decorator);
				}
	        }
	        Battle battle = owner.getBattle();
	        if (battle != null) {	        	
	        	DamageBot damageBotTrigger = (DamageBot) battle.getTriggersNotNull().getBattleObjectTriggerOn(owner, DamageBot.class);
	        	if (damageBotTrigger != null) addBoth(DecoratorDamageBot.getInstance(damageBotTrigger.getDamage()));
	        	Plague damagePlague = (Plague) battle.getTriggersNotNull().getBattleObjectTriggerOn(owner, Plague.class);
	        	if (damagePlague != null) addBoth(DecoratorPlague.getInstance(damagePlague.getQtyKilledPerTurn()));
	        }
		}
	}

	private void addOwner(Decorator decorator) {
		if (ownerList == null) ownerList = new DecoratorList();
		ownerList.add(decorator);
	}

	private void addAll(Decorator decorator) {
		if (allList == null) allList = new DecoratorList();
		allList.add(decorator);
	}

	private void addBoth(Decorator decorator) {
		if (allList == null) allList = new DecoratorList();
		if (ownerList == null) ownerList = new DecoratorList();
		allList.add(decorator);
		ownerList.add(decorator);
	}
	
	public DecoratorList getVisibleDecorators(Player p) throws ApplicationError {
		if (owner != null) {
			if (p == owner.getOwner()) {
				return ownerList;
			} else {
				return allList;
			}
		} else {
			return allList;
		}
	}

	private BattleObject getOwner() {
		return owner;
	}

	public DecoratorList getOwnerList() {
		return ownerList;
	}

	public DecoratorList getAllList() {
		return allList;
	}
	
	public DecoratorList getOwnerListCopy() {
		if (ownerList == null) {
			return null;
		} else {
			return new DecoratorList(ownerList);
		}
	}

	public DecoratorList getAllListCopy() {
		if (allList == null) {
			return null;
		} else {
			return new DecoratorList(allList);
		}
	}
}
