/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;
import mot.common.bean.Location;
import mot.common.util.ApplicationError;

/**
 * This Layouts crafts and planets clusterer round a single point.
 * Objects are layed out aruond a single point on the map and each new object layed
 * out is placed as close to that object as possible. So say M locates the spot around
 * which the new crafts are to be layed out, then, the layout would look like;
 *      
 *      a b c d e
 *      C C C C f
 *      C C M C g
 *        C C C h
 *            j i    
 *            
 * The character a, b, ... etc mark the next locations it would use!
 * 
 * This is used to place fighter crafts around the mother craft in a battle!
 */
public class SquareBattleLayoutManager implements BattleLayoutManager {

    private Location center;
    private int radius;
    private int startDirection;

    private BattlerLine left = null;
    private BattlerLine right = null;

    int shiftAmount = 50;

    public SquareBattleLayoutManager(Battle battle, Location center, int startDirection, int startRadius) {
        this.center = center;
        this.radius = startRadius;
        
        if (startDirection >= 0 && startDirection <= 180) {
            this.startDirection = 90;
        } else {
            this.startDirection = 270;
        }

        left = new BattlerLine( null,
                                battle, 
                                new Location(center.getX()-radius, center.getY()), 
                                new Location(center.getX()-radius, tenPercentTop), 
                                new Location(center.getX()-radius, tenPercentBottom),
                                270,
                                shiftAmount);

        right = new BattlerLine( null,
                                battle, 
                                new Location(center.getX()+radius, center.getY()), 
                                new Location(center.getX()+radius, tenPercentTop), 
                                new Location(center.getX()+radius, tenPercentBottom),
                                90,
                                shiftAmount);

    }

    /**
     * Sets the crafts location to be the closest location to the center of the square that
     * is not occupied by any object on the battle field.
     * @param craft The craft to place on the battle field.
     * @throws mot.common.util.ApplicationError
     */
    public void setNextFreeLocation(BattleCraft craft) throws ApplicationError {
        Location locLeft = left.getNextFreeLocation(craft);
        int distLeft = center.getDistanceTo(locLeft);
        if (startDirection != left.getDirection()) {
            distLeft += 25;
        }

        Location locRight = right.getNextFreeLocation(craft);
        int distRight = center.getDistanceTo(locLeft);
        if (startDirection != right.getDirection()) {
            distRight += 25;
        }

        Location best = null;
        int bestDist = Integer.MAX_VALUE;
        
        if (locLeft != null) {
            best = locLeft;
            bestDist = distLeft;
        }
        
        if (locRight != null && bestDist > distRight) {
            best = locRight;
            bestDist = distRight;
        }
        
        craft.setLocation(best);
    }

    /**
     * Does nothing as this does not layout planets!
     */
    public void setPlanetLocation(BattlePlanet planet) throws ApplicationError {
    }
}