/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles;

import mot.common.bean.TroopBattleResult;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.players.Player;

/**
 * Represents and Hand to Hand battle between two players. This can be;
 * - Troops landing on a planet in a troop transport (fight on a planet)
 * - Troops transported to a craft during battle (fight on a craft)
 */
public class TroopBattle  {

    private int qtyAttackers;
    private int qtyDefenders;
    
    private int remainingAttackers;
    private int remainingDefenders;
    
    private int attackerWeaponDamage;
    private int defenderWeaponDamage;
    private double attackerAdvantage;
    
    private int getWeaponDamage(Player p) {
    	if (p.getWeapons() != null && p.getWeapons().getBestGroundWeaponDamage() != null) {
        	Weapon weapon = p.getWeapons().getBestGroundWeaponDamage();
			return (int) weapon.getDamage();
		} else {
			Logger.logWarning("TroopBattle - using default weapon damage for "+p.getName());
			return 4;
		}
    }

    public TroopBattle(Player attacker, Player defender, int qtyAttackers, int qtyDefenders) throws ApplicationError {
        this(attacker, defender, qtyAttackers, qtyDefenders, 2.0);
    }
    
    public TroopBattle(Player attacker, Player defender, int qtyAttackers, int qtyDefenders, double maxRandomAttackerBonus) throws ApplicationError {
        this.qtyAttackers = qtyAttackers;
        this.qtyDefenders = qtyDefenders;
        this.attackerWeaponDamage = getWeaponDamage(attacker);
        this.defenderWeaponDamage = getWeaponDamage(defender);

        if (maxRandomAttackerBonus > 0.0) {
            this.attackerAdvantage = (1.0+Utils.randomDouble(maxRandomAttackerBonus)) + (double)(attackerWeaponDamage - defenderWeaponDamage);
        } else if (maxRandomAttackerBonus < 0.0) {
            this.attackerAdvantage = 0.25 + Utils.randomDouble(-1.0/maxRandomAttackerBonus);
            if (attackerWeaponDamage > defenderWeaponDamage) {
                this.attackerAdvantage += 1.0 - 1.0/(attackerWeaponDamage - defenderWeaponDamage);
            } else if (attackerWeaponDamage < defenderWeaponDamage) {
                this.attackerAdvantage -= 1.0 - 1.0/(defenderWeaponDamage - attackerWeaponDamage);
            }
        }

        double attackerBonus = 1.0 + attacker.getAssaultBonus();
        double defenderBonus = 1.0 + defender.getAssaultBonus();
        if (attackerBonus > 1.0) {
            attackerAdvantage = attackerAdvantage * attackerBonus;
        }
        if (defenderBonus > 1.0) {
            attackerAdvantage = attackerAdvantage / defenderBonus;
        }
    }
    
    public TroopBattleResult execute() throws ApplicationError {
        if (attackerAdvantage > 0) {
            remainingAttackers = qtyAttackers - Comp.min((int)(qtyDefenders/attackerAdvantage), qtyAttackers);
            remainingDefenders = qtyDefenders - Comp.min(qtyDefenders, (int)(qtyAttackers*attackerAdvantage));
        } else if (attackerAdvantage < 0) {
            attackerAdvantage = -attackerAdvantage;
            remainingAttackers = qtyAttackers - Comp.min((int)(qtyDefenders*attackerAdvantage), qtyAttackers);
            remainingDefenders = qtyDefenders - Comp.min(qtyDefenders, (int)(qtyAttackers/attackerAdvantage));
        } else {
            remainingAttackers = qtyAttackers - Comp.min(qtyDefenders, qtyAttackers);
            remainingDefenders = qtyDefenders - Comp.min(qtyDefenders, qtyAttackers);
        }
        return new TroopBattleResult(qtyAttackers, qtyDefenders, remainingAttackers, remainingDefenders);
    }


    public void setQtyAttackers(int qtyAttackers) {
        this.qtyAttackers = qtyAttackers;
    }


    public int getQtyAttackers() {
        return qtyAttackers;
    }


    public void setQtyDefenders(int qtyDefenders) {
        this.qtyDefenders = qtyDefenders;
    }


    public int getQtyDefenders() {
        return qtyDefenders;
    }


    public void setRemainingAttackers(int remainingAttackers) {
        this.remainingAttackers = remainingAttackers;
    }


    public int getRemainingAttackers() {
        return remainingAttackers;
    }


    public void setRemainingDefenders(int remainingDefenders) {
        this.remainingDefenders = remainingDefenders;
    }


    public int getRemainingDefenders() {
        return remainingDefenders;
    }


    public void setAttackerAdvantage(double attackerAdvantage) {
        this.attackerAdvantage = attackerAdvantage;
    }


    public double getAttackerAdvantage() {
        return attackerAdvantage;
    }
}