/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles.incumbants;

import mot.common.bean.AsteroidFieldDetails;
import mot.common.bean.Rectangle;
import mot.common.util.ApplicationError;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;

/**
 * A field of rocks that are spread out across a battle field 
 * that inflict damage on crafts travelling through them AND slows
 * them down. 
 * This damage is a proportion of the hit points the hull can sustain. 
 * A craft with a maximum hit points of 200 passing through an 
 * asteroid field of damage 0.1 would receive damage 20 per move.
 */
public class AsteroidField extends IncumbantRectangle {
	
	/**
	 * The damage this causes to a craft per move in the asteroid field.
	 */
	private double movementDamage;

	/**
	 * The damage percentage of movement a craft receives while passing 
	 * through this field.
	 */
	private double movementEffect;


	public AsteroidField(Rectangle rectangle, double movementDamage, double movementEffect) {
		super(rectangle);
		this.movementDamage = movementDamage;
		this.movementEffect = movementEffect;
	}
	
	public boolean effectsMovement() {
		return true;
	}

	public boolean effectsWeapons() {
		return false;
	}

	public double getWeaponEffect() {
		return 0.0;
	}

	public double getMovementEffect() {
		return movementEffect;
	}

	public double getMovementDamageEffect() {
		return movementDamage;
	}
	
	public void activate(BattleCraft craft) throws ApplicationError {
		double movementDamageEffect = getMovementDamageEffect();
		if (movementDamageEffect >= 0.0) {
			double damage = craft.getCraft().getHull().getMaxDamage() * movementDamageEffect;
			craft.damage(damage, null, craft.getCraftState(), null);
		}
	}

    public boolean isActive(BattleObject bo) {
    	return false;
    }	

    public AsteroidFieldDetails getAsteroidFieldDetails() {
    	return new AsteroidFieldDetails(getRectangle());
    }
}
