/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles.incumbants;

import mot.common.bean.IntersectionLocations;
import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.util.ApplicationError;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;

/**
 * An incumbant is an area on the battle map that performs an action when a 
 * craft moves through its area or attempts to fire through it. For example 
 * an asteriod belt inflicts damage as crafts move through it and weapons 
 * cannot be fired through it.
 */
public interface Incumbent {
	
	public boolean effectsWeapons();
	
	public boolean effectsMovement();
	
	/**
	 * @return The percentage of weapons that fail to pass through the incumbant. 
	 * This should only be used if effectsWeapons() returns true.
	 */
	public double getWeaponEffect();
	
	/**
	 * @return The percentage of speed lost while passing through this incumberence. 
	 * This should only be used if effectsMovement() returns true.
	 */
	public double getMovementEffect();
	
	/**
	 * @return The amount of damage inflicted on a craft as it passed through the incumberence. 
	 * This should only be used if effectsMovement() returns true.
	 */
	public double getMovementDamageEffect();
	
	/**
	 * notifies that the craft passed in has activated the incumberence.
	 * The effect depends on the type of incumberence.
	 */
	public void activate(BattleCraft craft) throws ApplicationError;

	/**
	 * The first edge is closest edge of this incumbant when moving from source to destination.
	 * The second edge is furthest edge of this incumbant when moving from source to destination.
	 * @return The location of the intersections of the line from source to destination and the edges of this shape.
	 */
	public IntersectionLocations getIntersections(Location source, Location destination);
    
	/**
	 * @return The location of the closest point on the line from source to 
	 * destination to the center of this rectangle IF it is within this circle's
	 * radius. If it is outside this circle's radius then null is returned.
	 */
    public Location getClosestPointToCenter(Location source, Location destination);
    
    /**
     * @return True if the location is inside this incumbent.
     */
    public boolean inside(Location loc);
    
    /**
     * Causes the craft to stop its movement at the closest point to the center of this
     * incumbent. The craft does not lose any turn but the owner will have to ask the craft
     * to move towards the destination again!
     * @return True if a craft's movement is paused at the boundary to the incumbent.
     */
    public boolean isActive(BattleObject bo) throws ApplicationError;

    /**
     * @return True if any of the area represented by rectangle rect intersects with any
     * part of this incumbent.
     */
    public boolean overlaps(Rectangle rect);
    
}
