/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles.incumbants;

import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.util.ApplicationError;
import mot.common.util.Utils;
import mot.engine.battles.Battle;
import mot.engine.players.Player;

public class IncumbentFactory {

	private static final double ASTEROID_DAMAGE = 0.25;
	private static final double ASTEROID_MOVEMENT_EFFECT = 0.5;
	private static final double SPACEDIST_MOVEMENT_EFFECT = 0.5;
	
	private static final Rectangle DEFAULT_ASTEROID_RECT = new Rectangle(700,150,50,700);
//	private static final Rectangle DEFAULT_MINE_RECT = new Rectangle(510,300,80,400);
	
    /**
     * Creates an asteroid field in the battle passed in. The asteroid is located 
     * in a fixed location (a rectangle 450,150 -> 550,850).
     */
    public static AsteroidField createAsteroidField(Battle battle) throws ApplicationError {
    	AsteroidField field = new AsteroidField(new Rectangle(DEFAULT_ASTEROID_RECT), ASTEROID_DAMAGE, ASTEROID_MOVEMENT_EFFECT);
    	battle.getIncumbents().add(field);
    	return field;
    }
	
    public static AsteroidField createAsteroidField(Battle battle, Rectangle rect) throws ApplicationError {
    	AsteroidField field = new AsteroidField(rect, ASTEROID_DAMAGE, ASTEROID_MOVEMENT_EFFECT);
    	battle.getIncumbents().add(field);
    	return field;
    }
	
    /**
     * Creates an asteroid field in the battle passed in. They are created
     * using special weapons that can only be deployed on planets. The rectangle
     * the incumbent covers is determined by the weapon fire target location and power
     * of the weapon (see WeaponInstance for details).
     */
    public static SpaceDistortion createSpacialDistortion(Battle battle, Player owner, Rectangle rect) throws ApplicationError {
    	SpaceDistortion field = new SpaceDistortion(owner, rect, SPACEDIST_MOVEMENT_EFFECT);
    	battle.getIncumbents().add(field);
    	return field;
    }
	
    /**
     * Creates an asteroid field in the battle passed in owned by owner. The mine is
     * located randomly within the rectangle 510,300 -> 590,700
     */
//    public static PerimeterAlarm createPerimeterAlarm(PerimeterNotification client, Battle battle, int radius) throws ApplicationError {
//    	return createPerimeterAlarm(client, battle, DEFAULT_MINE_RECT, radius);
//    }
	
    /**
     * Creates an asteroid field in the battle passed in owned by owner. The mine is
     * located randomly within the rectangle passed in
     */
    public static PerimeterAlarm createPerimeterAlarm(PerimeterNotification client, Battle battle, Rectangle rect, int radius) throws ApplicationError {
    	Location loc = createRandom(rect); 
    	return createPerimeterAlarm(client, battle, loc, radius);
    }
    
    /**
     * Creates an asteroid field in the battle passed in owned by owner. The mine is
     * located at the location passed in.
     */
    public static PerimeterAlarm createPerimeterAlarm(PerimeterNotification client, Battle battle, Location loc, int radius) throws ApplicationError {
    	PerimeterAlarm alarm = new PerimeterAlarm(client, loc, radius);
    	battle.getIncumbents().add(alarm);
    	return alarm;
    }
    
    private static Location createRandom(Rectangle rect) {
    	return new Location( rect.getLeft()+Utils.random(rect.getWidth()), 
    			             rect.getTop()+Utils.random(rect.getHeight()) );
    }
	
}
