/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles.incumbants;

import java.util.ArrayList;

import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.IntersectionLocations;
import mot.common.bean.LocatedArea;
import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.bean.SpaceDistortionDetails;
import mot.common.util.ApplicationError;
import mot.engine.battles.BattleObject;

/**
 * Container for the list of incumbants defines in a battle.
 */
public class Incumbents {
	
	/**
	 * This is the list of incumbants.
	 */
	private ArrayList incumbentsList = new ArrayList();
	
	/**
	 * This is a cache of the list of incumbants. This is dynamically generated
	 * from the attribute incumbantsList. Use method getIncumbants() to fetch its
	 * contents!
	 */
	private transient Incumbent [] incumbents;
	
	/**
	 * Add a new incumbant to this list of incumbants.
	 */
	public void add(Incumbent v) {
		incumbents = null;
		incumbentsList.add(v);
	}

	/**
	 * Add a new incumbant to this list of incumbants.
	 */
	public void remove(Incumbent v) {
		incumbents = null;
		incumbentsList.remove(v);
	}

	/**
	 * Fetch all the incumbants in this list.
	 */
	public Incumbent [] getIncumbants() {
		if (incumbents == null && incumbentsList.size() > 0) {
			incumbents = (Incumbent []) incumbentsList.toArray(new Incumbent [incumbentsList.size()]);
		}
		return incumbents;
	}

    /**
     * Note - If a craft is located inside an incumbent, but close enough to the edge for its
     * image to extend outside the incumbent, then this craft is not considered to be incumbered. 
     * @param source The location the weapon is being fired from
     * @param destination The location the weapon is being fired too
     * @return The location of the first incumbant in the battle that 
     * a weapon fired from source to target would find. It returns 
     * null if no incumbant is found. If the distance to an incumbant 
     * is zero (source location lies inside an incumbant) then the 
     * source location is returned!
     */
    public Location getFirstWeaponIncumbentEdge(LocatedArea source, LocatedArea destination) throws ApplicationError {
    	Incumbent [] incumbants = getIncumbants();
    	if (incumbants == null) return null;
    	Location loc = null;
    	int distance = 0;
    	for (int i=0; i<incumbants.length; i++) {
    		if (incumbants[i].effectsWeapons()) {
    			Location block = blocksLineOfSight(incumbants[i], source, destination); 
    			if (block != null) {
    				int newDistance = source.getLocation().getDistanceTo(block);
    				if (loc == null || newDistance < distance) {
    					loc = block;
    					distance = newDistance;
    				}
    			}
    		}
    	}
    	return loc;
    }
    
    /**
     * An incumbent blocks the line of sight between an source craft and a destination craft if
     * it is located in a direct line detween the source and the destination and no part of the 
     * craft is visible posing out from the incumbent.
     * 
     *   In the maps below I marks the area of the incumbent while * marks the source and dest
     * 
     *        IIIIIII
     * 	  *** IIIIIII  ***          INCUMBERED - Case 1
     *        IIIIIII
     * 
     *        IIIIIII
     * 	      II***II  ***          INCUMBERED - Case 2
     *        IIIIIII
     * 
     *        IIIIIII
     * 	      IIII***     ***       NOT INCUMBERED - Case 3
     *        IIIIIII
     * 
     *        IIIIIII
     * 	      IIIIIII ***    ***    NOT INCUMBERED - Case 4
     *        IIIIIII
     * 
     *        IIIIIIIIIIIIIIII
     * 	      II***III****IIII     INCUMBERED - Case 5
     *        IIIIIIIIIIIIIIII
     *        
     * @param incumbent The Area marked above with the letter I
     * @param source One of the area marked with three stars
     * @param destination One of the area marked with three stars
     * @return the location of the blockage in the line of sight if the incumbent passed 
     * in blocks the line of sight between the source and the destination.
     */
    private Location blocksLineOfSight(Incumbent incumbent, LocatedArea source, LocatedArea destination) throws ApplicationError {
		IntersectionLocations intersections = incumbent.getIntersections(source.getLocation(), destination.getLocation());
		if (intersections == null) {
			if (incumbent.inside(source.getLocation())) {
				// Case 5
				return source.getLocation();
			} else {				
				return null;
			}
		}
		Location first = intersections.getFirstEdge();
		if (first != null) {
			Location second = intersections.getSecondEdge();
			if (second == null) {
				if ((incumbent.inside(source.getLocation()) && source.isInside(first))) {
					// case 3
					return null;
				}
				if (incumbent.inside(destination.getLocation()) && destination.isInside(first)) { 
					// case 3
					return null;
				}
				// case 2
				return first;
			} else {
				// Case 1
				return first;
			}
		} else {
			if (incumbent.inside(source.getLocation())) {
				// Case 5
				return source.getLocation();
			} else {
				// Case 4
				return null;
			}
		}
    }
	
    /**
     * @param source The location moving from
     * @param destination The location moving too
     * @return A list of IncumbentEdge objects that define the list of incumbent
     * boundaries crossed when moving from the source location to the destination 
     * location. Note that they are in order of closest to the source location to
     * farthest!
     */
    public IncumbentEdge [] getMovementIncumbentEdgeList(Location source, Location destination) {
    	Incumbent [] incumbants = getIncumbants();
    	if (incumbants == null) return null;
    	ArrayList results = null;
    	for (int i=0; i<incumbants.length; i++) {
    		if (incumbants[i].effectsMovement()) {
    			IntersectionLocations intersections = incumbants[i].getIntersections(source, destination);
    			if (intersections != null) {
					Location loc = intersections.getFirstEdge();
					if (loc != null) {
						int distance = source.getDistanceTo(loc);
						results = new ArrayList();
						addIncumbentEdge(results, incumbants[i], loc, distance);
						loc = intersections.getSecondEdge();
						if (loc != null) {
							distance = source.getDistanceTo(loc);
							addIncumbentEdge(results, incumbants[i], loc,
									distance);
						}
					}
				}
    		}
    	}
    	if (results == null) {
	    	return null;
    	} else {
	    	return (IncumbentEdge []) results.toArray(new IncumbentEdge [results.size()]);
    	}
    }
    
    private void addIncumbentEdge(ArrayList results, Incumbent incumbant, Location loc, int distance) {
    	int qty = results.size();
    	for (int i=0; i<qty; i++) {
    		IncumbentEdge ie = (IncumbentEdge) results.get(i);
    		if (ie.getDistance() > distance) {
    			results.add(i, new IncumbentEdge(incumbant, loc, distance));
    			return;
    		}
    	}
		results.add(new IncumbentEdge(incumbant, loc, distance));
    }

    /**
     * An active encumbent is an incumbent that has some effect on the battle map
     * and the craft must stop at the closest point to the encumbent so the event
     * of triguring the encumbent can occur. An example is a Mine. Note - The craft
     * may continue its move afterwards!
     * @param source The location the weapon is being fired from
     * @param destination The location the weapon is being fired too
     * @return The location of the first incumbant in the battle that
     * would effect the craft passed in and isActive() returns true.
     */
    public IncumbentEdge getFirstActiveEncumbent(BattleObject bo, Location source, Location destination) throws ApplicationError {
    	Incumbent [] incumbants = getIncumbants();
    	if (incumbants == null) return null;
    	Incumbent closest = null;
    	int closestDistance = 0;
    	Location closestLocation = null;
    	for (int i=0; i<incumbants.length; i++) {
    		if (incumbants[i].isActive(bo)) {
    			Location loc = incumbants[i].getClosestPointToCenter(source, destination);
    			if (incumbants[i].inside(loc)) {
        			int dist = loc.getDistanceTo(source);
        			if (closest == null || dist < closestDistance) {
        				closestDistance = dist;
        				closestLocation = loc;
        				closest = incumbants[i];
        			}
    				
    			}
    		}
    	}
    	if (closest != null) {
    		return new IncumbentEdge(closest, closestLocation, closestDistance);
    	} else {
    		return null;
    	}
    }
    
    /**
     * @return True if the area represented by rectangle rect does not have any 
     * incumbents in it (entirely or partly).
     */
    public boolean isEmpty(Rectangle rect) {
    	Incumbent [] incumbents = getIncumbants();
    	if (incumbents == null) return true;
    	for (int i=0; i<incumbents.length; i++) {
    		if (incumbents[i].overlaps(rect)) {
    			return false;
    		}
    	}    	
    	return true;
    }
    
    /**
     * @return All incumbents that contain the location loc.
     * @param activeOnly If this is true only active inbcumbents are returned.
     */
    public Incumbent [] getIncumbents(Location loc) {
    	Incumbent [] incumbents = getIncumbants();
    	if (incumbents == null) return null;
    	ArrayList r = null;
    	for (int i=0; i<incumbents.length; i++) {
    		if (incumbents[i].inside(loc)) {
    			if (r == null) r = new ArrayList();
    			r.add(incumbents[i]);
    		}
    	}    	
    	if (r == null) {
        	return null;
    	} else {
        	return (Incumbent[]) r.toArray(new Incumbent[r.size()]);
    	}
    }
    
    public Incumbent getType(Class incumbentType) {
    	Incumbent [] incumbents = getIncumbants();
    	if (incumbents == null) return null;
    	for (int i=0; i<incumbents.length; i++) {
    		if (incumbents[i].getClass() == incumbentType) {
    			return incumbents[i];
    		}
    	}    	
    	return null;
    }    
	
    public SpaceDistortion [] getSpaceDistortions() {
    	Incumbent [] incumbents = getIncumbants();
    	if (incumbents == null) return null;
    	ArrayList r = null;
    	for (int i=0; i<incumbents.length; i++) {
    		if (incumbents[i] instanceof SpaceDistortion) {
    			if (r == null) r = new ArrayList();
    			r.add(incumbents[i]);
    		}
    	}    	
    	if (r == null) {
        	return null;
    	} else {
    		return (SpaceDistortion []) r.toArray(new SpaceDistortion[r.size()]);
    	}
    }    
	
    public SpaceDistortionDetails [] getSpaceDistortionDetails() {
    	Incumbent [] incumbents = getIncumbants();
    	if (incumbents == null) return null;
    	ArrayList r = null;
    	for (int i=0; i<incumbents.length; i++) {
    		if (incumbents[i] instanceof SpaceDistortion) {
    			if (r == null) r = new ArrayList();
    			r.add(incumbents[i]);
    		}
    	}    	
    	if (r == null) {
        	return null;
    	} else {
    		SpaceDistortionDetails [] ra = new SpaceDistortionDetails[r.size()];
    		for (int i=0; i<r.size(); i++) {
    			SpaceDistortion sd = (SpaceDistortion) r.get(i);
    			ra[i] = new SpaceDistortionDetails(sd.getRectangle());
    		}
    		return ra;
    	}
    }    
	
    public ArrayList<AnimatedBattleAreaDetails> getAllIncumbentDetails() throws ApplicationError {
    	ArrayList<AnimatedBattleAreaDetails> r = null;
    	Incumbent [] incumbents = getIncumbants();
    	if (incumbents != null) {
	    	for (int i=0; i<incumbents.length; i++) {
	    		if (r == null) r = new ArrayList();
	    		AnimatedBattleAreaDetails incumbent = IncumbentDetailsFactory.create(incumbents[i]);
	    		if (incumbent != null) {
	    			r.add(incumbent);
	    		}
	    	}
    	}
    	return r;
    }

	public Incumbent getIncumbentInside(Location loc) {
    	Incumbent [] incumbents = getIncumbants();
    	if (incumbents != null) {
	    	for (int i=0; i<incumbents.length; i++) {
	    		if (incumbents[i].inside(loc)) {
	    			return incumbents[i];
	    		}
	    	}
    	}
    	return null;
	}    
    
}

