/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles.triggers;

import mot.common.util.ApplicationError;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleObject;
import mot.engine.players.Player;

/**
 * This is a base class that attempts to extract out some of the common code for 
 * defining a trigger on a craft in a battle field. It is assumed that most triggers 
 * will apply to a single craft so it makes sense to factor out some common code.
 * 
 * It is intended that sub classes override the execute() method to perform the 
 * desired for the trigger but you must remember to call super.execute() or increment
 * stage by one.
 */
public abstract class CraftTriggerBase implements Trigger {
	
	/**
	 * This is the player that caused / ownes this trigger.
	 */
	protected Player owner;
	
	/**
	 * Use this value for the life field if it has an infinite life.
	 */
	public static final int INFINATE_LIFE = -1;

	/**
	 * This is the battle that this trigger is defined in.
	 */
	protected Battle battle;

	/**
	 * This is the craft in the battle that this trigger applies to.
	 */
	protected BattleObject battleObject;
	
	/**
	 * The quantity of times this trigger will be executed before is expires.
	 * A value of INFINATE_LIFE means it will not expire.
	 */
	protected int life;

	/**
	 * The quantity of times this trigger has been executed so far.
	 */
	protected int stage;

	public CraftTriggerBase(Player owner, Battle battle, BattleObject craft, int life) {
		this.owner = owner;
		this.battle = battle;
		this.battleObject = craft;
		this.life = life;
		this.stage = 0;
	}
	
	

	public void execute() throws ApplicationError {
		stage++;
	}

	public boolean isComplete() throws ApplicationError {
		if (battleObject != null && battleObject.isDestroyed()) {
			return true;
		}
		if (life == INFINATE_LIFE) {
			return false;
		} else {
			return stage >= life;
		}
	}



	public Battle getBattle() {
		return battle;
	}

	public void setBattle(Battle battle) {
		this.battle = battle;
	}

	public BattleObject getBattleObject() {
		return battleObject;
	}

	public void setBattleObject(BattleObject craft) {
		this.battleObject = craft;
	}

	public int getLife() {
		return life;
	}

	public void setLife(int life) {
		this.life = life;
	}

	public int getStage() {
		return stage;
	}

	public void setStage(int stage) {
		this.stage = stage;
	}

	public Player getOwner() {
		return owner;
	}

	public void setOwner(Player owner) {
		this.owner = owner;
	}

	public BattleObject getTarget() {
		return battleObject;
	}
}
