/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles.triggers;

import mot.common.bean.Decorator;
import mot.common.bean.NamedObjectList;
import mot.common.event.BattleObjectNotificationType;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleObjectNotificationEF;
import mot.engine.event.BattleObjectTriggerEF;
import mot.engine.players.Player;
import mot.engine.technology.WeaponInstanceBio;

public class Plague extends CraftTriggerBase {

	/**
	 * The quantity of people to kill each turn
	 */
    private int qtyKilledPerTurn;
    
    /**
     * The battle turn to ignore
     */
    private int ignoreTurn;
    
	public Plague(Player owner, Battle battle, BattleObject craft, int life, int qtyKilledPerTurn) {
		super(owner, battle, craft, life);
		this.qtyKilledPerTurn = qtyKilledPerTurn;
		this.ignoreTurn = -1;
	}

	public void execute() throws ApplicationError {
		if (battleObject == null || battleObject.isDestroyed() || !battleObject.isOperating()) {
			return;
		}
		
		if (battle.getTurn() == ignoreTurn) {
			return;
		}
		
		if (Logger.isDebug()) {
			Logger.logDebug("Plague.execute() on craft "+battleObject.getName());
		}
		
		CraftStateFactory originalState = battleObject.getCraftState();
        int remainingCrew = WeaponInstanceBio.killCrew(battleObject, qtyKilledPerTurn, owner);
		super.execute();
		CraftStateFactory newState = battleObject.getCraftState();
		String message = "Plague : Crew killed "+qtyKilledPerTurn+" ("+remainingCrew+")";
		BattleObjectTriggerEF event = new BattleObjectTriggerEF(battleObject, Weapon.GROUP_PLAGUE, originalState, newState, message);
        MotEngine.instance.multicast( event, battle.getStartingUserPlayers() );
	}

	public void postExecute() throws ApplicationError {
		BattleCraft bc = battle.getClosestCraftOwnedBy(battleObject.getLocation(), battleObject.getOwner(), new NamedObjectList(battleObject));
		if (bc != null) {
			if (Logger.isDebug()) {
				Logger.logDebug("Plague.execute() spread to craft "+bc.getName());
			}
			if (bc.isOperating()) {				
				Plague existingPlague = (Plague) battle.getTriggers().getBattleObjectTriggerOn(bc, Plague.class);
				CraftStateFactory state = bc.getCraftState();
				if (existingPlague != null) {
					if (existingPlague.getQtyKilledPerTurn() < qtyKilledPerTurn) {
						MotEngine.instance.multicast( new BattleObjectNotificationEF(bc, "Plague Spread", BattleObjectNotificationType.plague, state, state), battle.getStartingUserPlayers() );
					}
				} else {
					Plague newPlague = new Plague(owner, battle, bc, CraftTriggerBase.INFINATE_LIFE, qtyKilledPerTurn);
					newPlague.setIgnoreTurn(battle.getTurn());
					battle.getTriggers().add(newPlague);
					MotEngine.instance.multicast( new BattleObjectNotificationEF(bc, "Plague Spread", BattleObjectNotificationType.plague, state, state), battle.getStartingUserPlayers() );
				}
			}
		}
	}

	public String toString() { return "Plague on craft "+battleObject.getName(); }

	public int getQtyKilledPerTurn() {
		return qtyKilledPerTurn;
	}

	public void setQtyKilledPerTurn(int qtyKilledPerTurn) {
		this.qtyKilledPerTurn = qtyKilledPerTurn;
	}
	
	public Decorator getDecotator() {
		return null;
	}

	public int getIgnoreTurn() {
		return ignoreTurn;
	}

	public void setIgnoreTurn(int ignoreTurn) {
		this.ignoreTurn = ignoreTurn;
	}

}
