/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles.triggers;

import mot.common.bean.Decorator;
import mot.common.util.ApplicationError;
import mot.engine.battles.BattleObject;
import mot.engine.players.Player;

/**
 * A Trigger is an action that executes at the end of each battle turn. A
 * trigger may be implemented to do anything but the execute method will 
 * be called at the end of turn allowing the trigger to execute at the
 * end of every turn. 
 * 
 * An example of its usage is a plague. At the end of each battle turn
 * a craft containing a plague has a number of crew killed.
 */
public interface Trigger {
	/**
	 * This will be called at the end of a battle turn before the start turn
	 * method.
	 */
	public void execute() throws ApplicationError;

	/**
	 * This method will be called after the execute method at the end of a turn
	 * and before the start of the next turn. It can be used to make a trigger 
	 * expire after a peroid of time.
	 * @return True of this trigger has completed and can be removed from the
	 * list of triggers.
	 */
	public boolean isComplete() throws ApplicationError;
	
	/**
	 * Not all triggers have an owner so this may be null.
	 * The owner of a trigger is the player that created it. So for example if 
	 * it is a plague on a craft it is the player who created that plague.
	 * @return The owner of this trigger
	 */
	public Player getOwner() throws ApplicationError;

	/**
	 * For a Damage Bot this would be the battle craft it is damaging each turn.
	 * @return The battle object this trigger is associated with (if any).
	 */
	public BattleObject getTarget();

	/**
	 * @return The decorator crafts with this target should display or null if no decorator is used
	 */
	public Decorator getDecotator();

	/**
	 * Called after all triggers have been executed to give each trigger the opportunity of doing some
	 * other tasks.
	 * NOTE - Triggers are sorted so that trigger events are sent order of type. If a trigger send some other events
	 * they should be send from the postExecute() method!
	 * @throws ApplicationError 
	 */
	public void postExecute() throws ApplicationError;
}
