/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.battles.triggers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import mot.common.bean.Decorator;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.battles.BattleObject;
import mot.engine.players.Player;

/**
 * Maintains the list of all triggers defined on a battle.
 * For details of what a trigger is see the interface Trigger.
 */
public class Triggers {
	
	/**
	 * All the triggers defined in this list of triggers.
	 * The contents of this is cached in the _triggers attribute.
	 */
	private ArrayList triggers = new ArrayList();
	private Trigger [] _triggers;
	
	public void add(Trigger t) {
		if (Logger.isDebug()) {
			Logger.logDebug("Triggers.add() "+t);
		}
		triggers.add(t);
		_triggers = null;
		Collections.sort(triggers, new Comparator() {
			public int compare(Object o1, Object o2) {
				if (o1 == o2) return 0;
				if (o1 == null || o2 == null) return 0;
				return o1.getClass().getName().compareTo(o2.getClass().getName());
			}
		});
	}
	
	public void remove(Trigger t) {
		if (Logger.isDebug()) {
			Logger.logDebug("Triggers.remove() "+t);
		}
		triggers.remove(t);
		_triggers = null;
	}
	
	public Trigger [] getTriggers() {
		if (_triggers == null && triggers.size() > 0) {
			_triggers = (Trigger[]) triggers.toArray(new Trigger[triggers.size()]);
		}
		return _triggers;
	}
	
	public CraftTriggerBase getBattleObjectTriggerOn(BattleObject bo) {
		getTriggers();
		if (_triggers == null) return  null;
		for (int i=0; i<_triggers.length; i++) {
			if (_triggers[i] instanceof CraftTriggerBase) {
				CraftTriggerBase trigger = (CraftTriggerBase) _triggers[i];
				if (trigger.getBattleObject() == bo) {
					return trigger;
				}
			}
		}
		return null;
	}
	
	public CraftTriggerBase getBattleObjectTriggerOn(BattleObject bo, Class c) {
		getTriggers();
		if (_triggers == null) return  null;
		for (int i=0; i<_triggers.length; i++) {
			if (_triggers[i] instanceof CraftTriggerBase) {
				CraftTriggerBase trigger = (CraftTriggerBase) _triggers[i];
				if (trigger.getBattleObject() == bo && _triggers[i].getClass() == c) {
					return trigger;
				}
			}
		}
		return null;
	}
	
	public Trigger getTriggerOfType(Class c) {
		getTriggers();
		if (_triggers == null) return  null;
		for (int i=0; i<_triggers.length; i++) {
			if (_triggers[i].getClass() == c) {
				return _triggers[i];
			}
		}
		return null;
	}

	public Trigger getTriggerOfType(Class c, Player owner) throws ApplicationError {
		getTriggers();
		if (_triggers == null) return  null;
		for (int i=0; i<_triggers.length; i++) {
			if (_triggers[i].getClass() == c && _triggers[i].getOwner() == owner) {
				return _triggers[i];
			}
		}
		return null;
	}

	public void execute() throws ApplicationError {
		Trigger [] ta = getTriggers();
		if (ta == null) return;
		for (int i=0; i<ta.length; i++) {
			ta[i].execute();
			if (ta[i].isComplete()) {
				remove(ta[i]);
			}
		}
		for (int i=0; i<ta.length; i++) {
			ta[i].postExecute();
		}
	}

	public void removeAllWithTarget(BattleObject target) {
		Trigger [] ta = getTriggers();
		if (ta == null) return;
		for (int i=0; i<ta.length; i++) {
			if (ta[i].getTarget() == target) {
				remove(ta[i]);
			}
		}
	}

	public Decorator getDecotator(BattleObject target) {
		Trigger [] ta = getTriggers();
		if (ta == null) return null;
		for (int i=0; i<ta.length; i++) {
			if (ta[i].getTarget() == target) {
				Decorator d = ta[i].getDecotator();
				if (d != null) return d;
			}
		}
		return null;
	}

	
}
