/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.adapter.SolarSystemVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.planets.OrbitalReference;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRoute;
import mot.engine.players.Player;

/**
 * This is used to abandon a players colony (or mining station).
 * @see Planet.removeColony() for details.
 **/
public class AbandonCmd extends PlanetCmd {


    public AbandonCmd(Planet planet, Player owner) throws ApplicationError {
        super(planet, owner);
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        Planet planet = getPlanet();
        if (owner != planet.getOwner()) {
            throw new ValidationError(L.INVALID_COMMAND_THIS_PLANET_THAT_DOES_NOT_BELONG_TO_YOU);
        }
        
        SolarSystem solarSystem = planet.getSolarSystem();
        boolean hasTradeRoute = solarSystem.hasTradeRoute();
		TradeRoute[] effectedTradeRoutes = MotEngine.instance.getTradeRoutes().getTradeRoutes(solarSystem);
		Planet[] effectedPlanets = hasTradeRoute ? MotEngine.instance.getTradeRoutes().getAllPlanetsEffectedByTradeRoutesIn(solarSystem) : null;
		
       	Planet miningPlanet = planet.getMiner();
       	OrbitalReference miningStation = planet.getMiningStation();
        planet.removeColony(false);
        
        StateUpdateCommandResult r = new StateUpdateCommandResult(EngineVoFactory.getPlayer(owner));
		r.addStateUpdate(new SolarSystemVOAdapter(solarSystem, owner));
		if (hasTradeRoute) {
			r.addStateUpdate(EngineVoFactory.getPlanets(effectedPlanets, owner) );
		} else {
			r.addStateUpdate(new PlanetVOAdapter(planet, owner));
		}
        if (miningPlanet != null) {
        	r.addStateUpdate(new PlanetVOAdapter(miningPlanet, owner));
        	r.addStateUpdate(new SolarSystemVOAdapter(miningPlanet.getSolarSystem(), owner));
        }
        if (miningStation != null && miningStation.getPlanet() != null) {
        	Planet mine = miningStation.getPlanet();
        	r.addStateUpdate(new PlanetVOAdapter(mine, owner));
        	r.addStateUpdate(new SolarSystemVOAdapter(mine.getSolarSystem(), owner));
        }
		if (effectedTradeRoutes != null) {
			// Not all effected trade routes may be deleted
			r.addStateUpdate(EngineVoFactory.getTradeRoutes(MotEngine.instance.getTradeRoutes(), owner));
		}
		return r;
    }


}