/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.leaders.GeneralDetails;
import mot.common.leaders.GovenorDetails;
import mot.common.leaders.LeaderDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.common.vo.MoveableOwnedObjectVO;
import mot.common.vo.PlanetVO;
import mot.engine.adapter.PlayerDetailsVOAdapter;
import mot.engine.adapter.UserPlayerVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.leaders.LeaderFactory;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * This is a command issued by a user to the engine.
 * This command causes the engine to add the Leader with name leader to
 * player.
 **/
public class AcceptLeaderCmd extends Command {

    private String leader;

    public AcceptLeaderCmd(Player player, String leader) {
        super(player);
        this.leader = leader;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        Player owner = getOwner();
        if (owner instanceof UserPlayer) {
            UserPlayer up = (UserPlayer) owner;
            if (!Comp.same(leader, up.getOfferedLeader())) {
                throw new ValidationError("Leader "+leader+" was not offered to you");
            }
        }
        LeaderDetails ld = LeaderFactory.getLeader(leader);
        if (ld instanceof GeneralDetails) {
            GeneralDetails general = (GeneralDetails) ld; 
            LeaderFactory.addGeneralLeaderTo(owner, general);
        } else if (ld instanceof GovenorDetails) {
            GovenorDetails govenor = (GovenorDetails) ld; 
            LeaderFactory.addGovenorLeaderTo(owner, govenor);
        }
        StateUpdateCommandResult cr = new StateUpdateCommandResult(EngineVoFactory.getPlayer(owner), new PlayerDetailsVOAdapter(owner));
        if (ld instanceof GeneralDetails) {
        	MoveableOwnedObjectVO[] moos = EngineVoFactory.getMoos(owner.getCraftAndFleetObjects(), owner);
        	cr.addStateUpdate(moos);
        } else if (ld instanceof GovenorDetails) {
        	PlanetVO[] planets = EngineVoFactory.getPlanets(owner.getPlanets(), owner);
        	cr.addStateUpdate(planets);
        }

        if (owner instanceof UserPlayer) {
        	cr.addStateUpdate(new UserPlayerVOAdapter((UserPlayer)owner));
        }

		return cr;
	}


    public void setLeader(String leader) {
        this.leader = leader;
    }


    public String getLeader() {
        return leader;
    }

}