/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.lang.L;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBundle;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.common.EngineVoFactory;
import mot.engine.players.UserPlayer;

public class AddCraftDesign extends Command {

	private UserPlayer owner;
    private CraftDesign cd;

    public AddCraftDesign(CraftDesign cd, UserPlayer player) {
        super(player);
        this.owner = player;
        this.cd = cd;
    }

    protected CommandResult execute() throws ApplicationError, ValidationError {
        if (cd == null) {
            throw new ValidationError(L.INVALID_COMMAND_NO_CRAFT_DESIGN_SPECIFIED);
        }
        
        if (cd.getHull() == null) {
        	throw new ValidationError(L.INVALID_CRAFT_DESIGN_HAS_NO_HULL);
        }
        
        if (cd.getEngine() == null) {
        	throw new ValidationError(L.INVALID_CRAFT_DESIGN_HAS_NO_ENGINE);
        }
        
        if (cd.getWeaponBays() != null && cd.getWeaponBays().getQtyWeapons() > 0) {
	        WeaponBundle[] weapons = cd.getWeaponBays().getDistinctWeapons();
	        for (int i = 0; i < weapons.length; i++) {
				Weapon weapon = weapons[i].getWeapon();
				if (weapon.isSpacialDistortion()) {
					throw new ValidationError(L.INVALID_CRAFT_DESIGN_SPATIAL_DISTORTIONS_CANNOT_BE_PLACED_ON_A_CRAFT);
				}
				int maxQtyPerCraft = weapon.getMaxQtyOnCraft();
				if (maxQtyPerCraft >= 0) {
					if (weapons[i].getQty() > maxQtyPerCraft) {
						throw new ValidationError(L.INVALID_CRAFT_DESIGN_YOU_CAN_ONLY_INSTALL_+maxQtyPerCraft+" "+weapon.getName()+L._PER_CRAFT);
					}
				}
			}
        }
        
//        if (cd.getShields() != null) {
//        	Shields shields = cd.getShieldTechs();
//        	String[] groups = shields.getGroups();
//        	if (groups != null) {
//        		for (int i = 0; i < groups.length; i++) {
//        			int qty = shields.getQuantityWithGroup(groups[i]);
//					if (qty > 1) {
//			            throw new ValidationError(L.INVALID_CRAFT_DESIGN_ONLY_ONE_SHIELD_OF_ANY_TYPE);
//					}
//				}
//        	}
//        }
        
        if (cd.getRemainingSpace() < 0) {
            throw new ValidationError(L.INVALID_CRAFT_DESIGN_NOT_ENOUGH_SPACE_TO_STORE_ALL_THE_EQUIPMENT);
        }
        
        if (getOwner().isResourceful()) {
        	Specials specialTechs = cd.getSpecialTechs();
			if (specialTechs != null && specialTechs.getRepairEffect() != null && specialTechs.getRepairEffect().doubleValue() > 0.0) {
                throw new ValidationError(L.INVALID_CRAFT_RESOURCEFUL_PLAYERS_GET_REPAIR_UNITS_AUTOMATICALLY);
        	} else {
        		Special repairUnit = (owner.getSpecials() == null) ? null : owner.getSpecials().getBestRepairEffect();
        		if (repairUnit != null) {
        			cd.add(repairUnit);
        		}
        	}
        }
        
        CraftDesigns p = owner.getCraftDesigns();
        if ( p == null ) {
            p = new CraftDesigns();
            owner.setCraftDesigns(p);
        }
        p.addOrUpdate(cd);
        
        //double buildQuality = owner.getCraftBuildExperienceNotNull().getBuildQuality(owner, cd.getName());
        return new StateUpdateCommandResult(EngineVoFactory.getCraftDesigns(owner.getCraftDesigns(), owner));
    }

}