/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.command_results.BooleanCR;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.common.vo.BattleObjectStateUpdateVO;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.MovingAutamaton;
import mot.engine.common.ServerImageObject;
import mot.engine.players.Player;
import mot.engine.players.commanders.CommanderBasic;
import mot.engine.technology.WeaponInstance;

/**
 * This command causes the source craft to attack the enemy craft in a battle.
 * All weapons in the source are fired at the target providing they are in range.
 **/
public class BattleAttackCmd extends BattleCommand {


	/**
     * The source of this command.
     */
    protected ServerImageObject source;

    /**
     * The target of this command.
     */
    protected ServerImageObject target;

    /**
     * This is null if the craft is not to be moved and the location to which to move the craft if the craft can be moved.
     * NOTE - This is not the location of the attack target! This is a location around the target (because a target 
     * is attacked for a particular direction).
     */
    private LocationAndDirection attackFrom;

    /**
     * The weapon the source craft is to fire at the target.
     * If this is null all weapons are fired.
     */
    protected Weapon weapon;

    /**
     * The The quantity of weapon the source craft is to fire at the target.
     * If weapon is not null this is the quantity of this weapon to fire. If weapon is null
     * this is ignored
     */
    protected int qtyToFire;

    /**
     * If this command is permitted to move the source craft towards the 
     * destination.
     */
	private boolean move;
    
    /**
     * If this command will discharge all weapons (providing time warp has not fired to restart the turn again)
     * and set move remaining to 0.
     */
	private boolean discharge;
    
    /**
     * If this command will automatically fire missile defence weapons at incoming automatons.
     */
	private boolean missileDefence;

	private boolean updateClient;
    

    public ServerImageObject getSource() { return source; }
    public void setSource(ServerImageObject v) { source = v; }

    public ServerImageObject getTarget() { return target; }
    public void setTarget(ServerImageObject target) { this.target = target; }


    /**
     * @param owner The player who has submitted this command.
     * @param battle The battle this attack is taking place in.
     * @param source The object to attack.
     * @param target The object to be attacked.
     * @param move If the this command will more towards the target before firing.
     * @param weapon The weapon the source craft is to fire at the target. If this is null 
     * all weapons are fired.
     * @param qtyToFire The The quantity of weapon the source craft is to fire at the target.
     * If weapon is not null this is the quantity of this weapon to fire. If weapon is null
     * this is ignored
     */
	public BattleAttackCmd(Player owner, Battle battle, ServerImageObject source, ServerImageObject target, boolean move, boolean discharge, boolean missileDefence, LocationAndDirection attackFrom, Weapon weapon, int qtyToFire, boolean updateClient) {
        super(owner, battle);
        this.source = source;
        this.target = target;
		this.move = move;
		this.discharge = discharge;
		this.missileDefence = missileDefence;
        this.attackFrom = attackFrom;
        this.weapon = weapon;
        this.qtyToFire = qtyToFire;
		this.updateClient = updateClient;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult executeBattleCommand()  throws ApplicationError, ValidationError {
		if (source instanceof BattleCraft) {
			BattleCraft sourceCraft = (BattleCraft) source;
	        validateCanGiveCommand(sourceCraft);
	        if (!sourceCraft.isOperating()) throw new ValidationError(L.NOT_OPERATING);
	        if (!sourceCraft.hasWeaponsToFire()) throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANY_WEAPONS_READY_TO_FIRE);
	        if (target == null) {
	        	target = battle.getClosestEnemyOperatingCraftWithinFiringAngle(owner, sourceCraft);
	        	if (target == null) {
	        		target = battle.getClosestEnemyCraft(owner, source, true, null);
	        	}
	        }
		} else if (source instanceof BattlePlanet) {
			BattlePlanet sourcePlanet = (BattlePlanet) source;
	        if (!sourcePlanet.isOwnedBy(owner)) throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
	        if (!sourcePlanet.hasWeaponsToFire()) throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANY_WEAPONS_READY_TO_FIRE);
	        BattleCraft bestInitiative = battle.getCraftWithHighestPriority();
	        if (bestInitiative != null) throw new ValidationError(L.THERE_ARE_OTHER_CRAFT_WITH_HIGHER_INITIATIVE_THAT_HAVE_NOT_MOVED_YET);
	        if (target == null) {
	       		target = battle.getClosestEnemyCraft(owner, source, true, null);
	        }
		} else {
			throw new ValidationError(L.YOU_CANNOT_COMMAND);
		}
		
    	if (target == null || !(target instanceof BattleObject)) {
			// This is an automated command because it does not have a target
			if (source instanceof BattleObject) {
				if (source instanceof BattleCraft) {
					BattleCraft sourceCraft = (BattleCraft) source;
					sourceCraft.setMoveRemaining(0.0);
				}
				BattleObject sourceBO = (BattleObject) source;
				sourceBO.dischargeWeapons();
				cleanup(sourceBO);
				StateUpdateCommandResult cr = new StateUpdateCommandResult();
				if (updateClient) {
					battle.getNextObjectToMoveCache().checkForUpdates(cr, owner);
				}
				cr.addStateUpdate(new BattleObjectStateUpdateVO(source.getName(), battle.getName(), sourceBO.getCraftState(owner)));
				return cr;
			} else {
    			return null;    			
			}
    	}
    	
		boolean moved = false;
		BattleObject sourceBattleObject = (BattleObject) source; 
        
		
        boolean stopped = false;
        
        sourceBattleObject.notifyStartCommand();
        
		if (sourceBattleObject instanceof BattleCraft) {
			BattleCraft sourceCraft = (BattleCraft) sourceBattleObject;
	        
			battle.removeFromWaitList(sourceCraft);
			
	        LocationAndDirection originalLAD = source.getLocationAndDirection();
	        if (sourceCraft.getMoveRemaining() > 0.0) {
	            if (missileDefence) sourceCraft.fireMissileDefenceWeapons(null);
	            if (attackFrom != null && !sourceCraft.allInWeaponRange(target.getLocation())) {
	            	stopped = sourceCraft.moveTo(attackFrom, 1.0, target.getLocation(), target, false);
	            } else if (move) {
	            	stopped = sourceCraft.moveTo(new LocationAndDirection(target.getLocation()), 1.0, null, null, false);
	            }
	            if (discharge && !stopped) {
	            	sourceCraft.setMoveRemaining(0.0);
	            }
	        }
	        moved = !sourceCraft.getLocationAndDirection().equals(originalLAD);
		}
		
		if (stopped) {
			return new BooleanCR(true);
		}
		
        if (weapon != null) {
            WeaponInstance w = sourceBattleObject.getWeapon(weapon);
            if (w == null) {
                throw new ValidationError(L.THE_SOURCE_CRAFT_DOES_NOT_HAVE_WEAPON+weapon.getName());
            }
            if (qtyToFire > (w.getQty() - w.getQtyFired())) {
                throw new ValidationError(L.THE_SOURCE_CRAFT_DOES_NOT_HAVE_THIS_MANY_WEAPON_AVAILABLE_TO_FIRE);
            }
            if (!moved && !w.inRange(sourceBattleObject, target.getLocation())) {
                throw new ValidationError(L.TARGET_IS_OUT_OF_RANGE_OF_WEAPON+w.getWeapon().getName());
            }
            attack((BattleObject)target, w, qtyToFire);
        } else {
        	boolean finishedFiring = false;
			if (!finishedFiring && target instanceof MovingAutamaton) {
				WeaponInstance [] mdwa = sourceBattleObject.getWeaponsReadyToFireAt((MovingAutamaton)target, Weapon.MISSILE_DEFENCE_ONLY_GROUPS);
				if (mdwa != null) {
					attack((BattleObject)target, mdwa);
				} else {
					WeaponInstance [] bwa = sourceBattleObject.getWeaponsReadyToFireAt((MovingAutamaton)target, Weapon.BEAM_OR_PROJECTILE_GROUPS);
					if (bwa != null) {
						attack((BattleObject)target, bwa);
					}
				}
				finishedFiring = true;
				discharge = false;
			}
			if (!finishedFiring) {				
				attack((BattleObject)target, null, -1);
				if (target != null) {
					Location tl = target.getLocation();
					MovingAutamaton missile = battle.getMovingAutomatonTargettingMeAt(owner, tl.getX(), tl.getY());
					sourceBattleObject.fireMissileDefenceWeapons(missile);
				}
			}
        }

        if (missileDefence) {
        	sourceBattleObject.fireMissileDefenceWeapons(null);
        }
        
        cleanup(sourceBattleObject);
        
        if (discharge && weapon == null) {
        	if (source instanceof BattleObject) {
        		BattleObject sourceBO = (BattleObject)source;
    		    if (!sourceBO.hasFiredTimeWarpThisCommand()) {
		        	// Find alternative targets for any weapons that have not been fired
		        	CommanderBasic.findAlternateTargetForUnFiredWeapons(sourceBO, target);
	        	}
    		}
        	sourceBattleObject.dischargeWeaponsIfNoTimeWarpFired();
        }
        
		StateUpdateCommandResult cr = new StateUpdateCommandResult();
		if (updateClient) {
			battle.getNextObjectToMoveCache().checkForUpdates(cr, owner);
		}
		return cr;
	}

    private void attack(BattleObject t, WeaponInstance  [] wa) throws ApplicationError {
		if (wa != null) {
			for (int i = 0; i < wa.length; i++) {
				attack(t, wa[i], wa[i].getQtyWeaponsToFire());
			}			
		}
    }
    
    private void attack(BattleObject t, WeaponInstance w, int qty) throws ApplicationError {
		if (source instanceof BattleCraft) {
			BattleCraft sourceCraft = (BattleCraft) source;
			sourceCraft.attack(target, w, qty);
		} else if (source instanceof BattlePlanet) {
			BattlePlanet sourcePlanet = (BattlePlanet) source;
			sourcePlanet.fireWeapons((BattleObject)target, w, qty);
		}
	}
    
	public LocationAndDirection getMoveTo() {
        return attackFrom;
    }

    public void setMoveTo(LocationAndDirection moveTo) {
        this.attackFrom = moveTo;
    }

    public void setWeapon(Weapon weapon) {
        this.weapon = weapon;
    }


    public Weapon getWeapon() {
        return weapon;
    }


    public void setQtyToFire(int qtyToFire) {
        this.qtyToFire = qtyToFire;
    }


    public int getQtyToFire() {
        return qtyToFire;
    }
}