/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.bean.Location;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.players.Player;
import mot.engine.technology.WeaponInstance;
import mot.engine.technology.WeaponInstanceSpacialDistortion;

/**
 * This command causes the source planet to attack the enemy object in a battle.
 * All weapons in the source are fired at the target providing they are in range.
 **/
public class BattleAttackLocationCmd extends BattleCommand {


	/**
     * The source of this command.
     */
    protected BattleObject source;

    /**
     * The target of this command.
     */
    protected Location target;
    
    /**
     * If this command is permitted to move the source craft towards the destination.
     * This command will only move the craft when there are some weapons out of range
     * and this is true.
     */
    private boolean canMove;

    /**
     * The weapon the source craft is to fire at the target.
     * If this is null all weapons are fired.
     */
    protected Weapon weapon;

    /**
     * The The quantity of weapon the source craft is to fire at the target.
     * If weapon is not null this is the quantity of this weapon to fire. If weapon is null
     * this is ignored
     */
    protected int qtyToFire;

    public BattleObject getSource() { return source; }
    public void setSource(BattleObject v) { source = v; }

    public Location getTarget() { return target; }
    public void setTarget(Location target) { this.target = target; }

    /**
     * @param owner The player who has submitted this command.
     * @param battle The battle this attack is taking place in.
     * @param source The object to attack.
     * @param target The object to be attacked.
     * @param weapon The weapon the source craft is to fire at the target. If this is null 
     * all weapons are fired.
     * @param qtyToFire The The quantity of weapon the source craft is to fire at the target.
     * If weapon is not null this is the quantity of this weapon to fire. If weapon is null
     * this is ignored
     */
	public BattleAttackLocationCmd(Player owner, Battle battle, BattleObject source, Location target, boolean canMove, Weapon weapon, int qtyToFire) {
        super(owner, battle);
        this.source = source;
        this.target = target;
        this.canMove = canMove;
        this.weapon = weapon;
        this.qtyToFire = qtyToFire;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult executeBattleCommand()  throws ApplicationError, ValidationError {
		if (source instanceof BattleCraft) {
	        validateCanGiveCommand((BattleCraft)source);
			battle.removeFromWaitList((BattleCraft)source);
		} else {
	        if (!source.isOwnedBy(owner)) throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
	        BattleCraft bestInitiative = battle.getCraftWithHighestPriority();
	        if (bestInitiative != null) throw new ValidationError(L.THERE_ARE_CRAFT_WITH_HIGHER_INITIATIVE_THAT_HAVE_NOT_MOVED_YET);
		}
        if (!source.hasWeaponsToFire()) throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANY_WEAPONS_READY_TO_FIRE);
        if (weapon != null) {
            if (weapon.isSpacialDistortion() && target.getDistanceTo(source.getLocation()) < WeaponInstanceSpacialDistortion.MIN_DIST_TO_PLANET) {
                throw new ValidationError(L.TARGET_TOO_CLOSE_TO_PLANET);
            }            
            WeaponInstance w = source.getWeapon(weapon);
            if (w == null) {
                throw new ValidationError(L.THE_SOURCE_CRAFT_DOES_NOT_HAVE_WEAPON_+weapon.getName());
            }
            if (qtyToFire > (w.getQty() - w.getQtyFired())) {
                throw new ValidationError(L.THE_SOURCE_CRAFT_DOES_NOT_HAVE_THIS_MANY_WEAPON_AVAILABLE_TO_FIRE);
            }
        	if (!w.canAttackLocationAndHasCharge()) {
                throw new ValidationError(L.THE_SOURCE_CRAFT_CANNOT_ATTACK_A_LOCATION);
        	}
            source.fireWeapons(target, w, qtyToFire);
        } else {
        	if (!source.hasWeaponsThatAttackLocation()) {
                throw new ValidationError(L.THE_SOURCE_CRAFT_CANNOT_ATTACK_A_LOCATION);
        	}
            source.fireWeapons(target, null, -1);
        }
        cleanup(source);
		StateUpdateCommandResult cr = new StateUpdateCommandResult();
		battle.getNextObjectToMoveCache().checkForUpdates(cr, owner);
		return cr;
	}

    public void setWeapon(Weapon weapon) {
        this.weapon = weapon;
    }


    public Weapon getWeapon() {
        return weapon;
    }


    public void setQtyToFire(int qtyToFire) {
        this.qtyToFire = qtyToFire;
    }


    public int getQtyToFire() {
        return qtyToFire;
    }

    public boolean isCanMove() {
        return canMove;
    }

    public void setCanMove(boolean canMove) {
        this.canMove = canMove;
    }

    
}