/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import java.util.ArrayList;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.event.BattleAutomatedEF;
import mot.engine.event.PlayersWaitingForBattleAutomateEF;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * This command causes the battle to be executed automatically.
 **/
public class BattleAutomateCmd extends BattleCommand {
	
	// List of user players in a battle that have requested to automate it.
	// Once all user players have requested to automate - the battle is automated. 
	private static AutomateRequestStore automateRequestStore = null;

    public BattleAutomateCmd(Battle battle, Player owner) throws ApplicationError {
        super(owner, battle);
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult executeBattleCommand()  throws ApplicationError, ValidationError {
		
		UserPlayer [] userPlayers = battle.getUserPlayers();
		if (userPlayers != null && userPlayers.length > 1) {
			if (automateRequestStore == null || !Comp.same(automateRequestStore.getBattleName(), battle.getName())) {
				automateRequestStore = new AutomateRequestStore(battle.getName());
				automateRequestStore.add(owner.getName());
				for (int i = 0; i < userPlayers.length; i++) {
					if (userPlayers[i] != owner) {
						MotEngine.instance.broadcast(new PlayersWaitingForBattleAutomateEF(battle), userPlayers[i]);
					}
				}
				throw new ValidationError("Automate Request Pending - All users must request complete battle");
			}
			automateRequestStore.add(owner.getName());
			if (!automateRequestStore.containsAllUserPlayers(userPlayers)) {
				throw new ValidationError("Automate Request Pending - All users must request comlete battle");
			}
			for (int i = 0; i < userPlayers.length; i++) {
				if (userPlayers[i] != owner) {
					MotEngine.instance.broadcast(new BattleAutomatedEF(battle), userPlayers[i]);
				}
			}
		}
		
		battle.clearWaitList();
		
        battle.automate();
        battle.endBattle();
        MotEngine.instance.getProxyManager().broadcastQueue();
        StateUpdateCommandResult cr = new StateUpdateCommandResult();
        MotEngine.instance.addStateUpdate(cr, owner, true, false, true, true, true, true, true, false, true, true, false);
		return cr;
	}
	
	private class AutomateRequestStore {
		private String battleName;
		private ArrayList<String> players;

		public AutomateRequestStore(String battleName) {
			this.setBattleName(battleName);
		}

		public boolean containsAllUserPlayers(UserPlayer[] userPlayers) {
			if (userPlayers == null) {
				return true;
			} else {
				for (UserPlayer up : userPlayers) {
					if (!containUserPlayer(up.getName())) {
						return false;
					}
				}
				return true;
			}
		}

		public void add(String name) {
			if (players == null) {
				players = new ArrayList();
				players.add(name);
			} else if (!containUserPlayer(name)) {
				players.add(name);
			}
		}

		private boolean containUserPlayer(String userPlayer) {
			if (players == null) {
				return false;
			} else {
				for (String n : players) {
					if (Comp.same(n, userPlayer)) {
						return true;
					}
				}
				return false;
			}
		}

		public String getBattleName() {
			return battleName;
		}

		public void setBattleName(String battleName) {
			this.battleName = battleName;
		}
	}

}