/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.technology.Specials;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePrimaryObject;
import mot.engine.players.Player;

/**
 * This command causes the source craft to launch fighters it has in the fighters bay.
 **/
public class BattleCraftLaunchFightersCmd extends BattleCommand {

	/**
     * The source of this command.
     */
    protected BattleObject source;

    public BattleObject getSource() { return source; }
    public void setSource(BattleObject v) { source = v; }


    /**
     * @param owner The player who has submitted this command.
     * @param battle The battle this attack is taking place in.
     * @param source The object to attack.
     */
	public BattleCraftLaunchFightersCmd(Player owner, Battle battle, BattleObject source) {
        super(owner, battle);
        this.source = source;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult executeBattleCommand()  throws ApplicationError, ValidationError {
        validateCanGiveCommand(source);
        
        if (!(source instanceof BattlePrimaryObject)) {
        	throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANY_FIGHTERS_READY_TO_LAUNCH);
        }
        
    	BattlePrimaryObject bpo = (BattlePrimaryObject) source;
    	Specials specials = bpo.getSpecials();
    	
    	if (bpo.hasLaunchedFighters()) throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANY_FIGHTERS_READY_TO_LAUNCH);
        if (specials == null || specials.getQtyFighters() == null || specials.getQtyFighters().intValue() == 0) throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANY_FIGHTERS);

        int qtyFighters = specials.getQtyFighters().intValue();
        
        BattleCraft [] fighters = bpo.launchFighters();
        if (fighters == null || fighters.length == 0) {
            throw new ValidationError(L.YOU_HAVE_NOT_REQUIRED_ALL_THE_TECHNOLOGY_TO_USE_FIGHTERS);
        }
        
        cleanup(source);

		StateUpdateCommandResult cr = new StateUpdateCommandResult();
		battle.getNextObjectToMoveCache().checkForUpdates(cr, owner);
		return cr;
	}


}