/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.bean.LocationAndDirection;
import mot.common.command_results.BooleanCR;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.common.ServerImageObject;
import mot.engine.players.Player;

/**
 * This command causes the source craft to attack the enemy craft in a battle.
 * All weapons in the source are fired at the target providing they are in range.
 **/
public class BattleCraftMoveToCmd extends BattleCommand {


	/**
     * The source of this command.
     */
    protected BattleCraft source;

    /**
     * The target of this command.
     */
    protected LocationAndDirection moveToDest;
    
    /**
     * If this is true the move will stop at STOP_ON_ATTACK% of the move before a missile strikes
     * if the craft has weapons that can destroy missiles that have not been fired this turn.
     * This gives the owner an opportunity to destroy the missiles before they damage the craft.
     */
    private boolean stopOnMissileAttack;

    /**
     * If this command should automatically fire missile defences before and after the move.
     */
    private boolean missileDefence;
    
	private ServerImageObject target;

	private boolean updateClient;


    public BattleCraft getSource() { return source; }
    public void setSource(BattleCraft v) { source = v; }

    public LocationAndDirection getTarget() { return moveToDest; }
    public void setTarget(LocationAndDirection target) { this.moveToDest = target; }


	public BattleCraftMoveToCmd(Player owner, Battle battle, BattleCraft source, LocationAndDirection moveToDest, ServerImageObject target, boolean stopOnMissileAttack, boolean missileDefence, boolean updateClient) {
        super(owner, battle);
        this.source = source;
        this.moveToDest = moveToDest;
		this.target = target;
        this.stopOnMissileAttack = stopOnMissileAttack;
		this.missileDefence = missileDefence;
		this.updateClient = updateClient;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult executeBattleCommand()  throws ApplicationError, ValidationError {
        validateCanGiveCommand(source);
        if (source.getMoveRemaining() <= 0) throw new ValidationError(L.THIS_HAS_ALREADY_MOVED);
        if (source.isOrbital()) throw new ValidationError(L.THIS_CANNOT_MOVE);
        
        battle.removeFromWaitList(source);

        //MoveLinkedObject [] links = source.getMoveLinks();
        double maxMovePercent = 1.0;
        
        // We must stop on missile attack otherwise it will look silly a missile going slow to hit target right next to it!
        if (missileDefence) {
        	source.fireMissileDefenceWeapons(null);
        }
        
        boolean stopped = false;
        
        if (target != null) {
        	stopped = source.moveTo(moveToDest, maxMovePercent, target.getLocation(), target, false);
        } else {
        	stopped = source.moveTo(moveToDest, maxMovePercent, null, null, false);
        }
        
        if (missileDefence && !stopped) {
        	source.fireMissileDefenceWeapons(null);
        }
        
        cleanup(source);

        StateUpdateCommandResult cr = null;
		if (stopped) {
			cr = new BooleanCR(true);
		} else {			
			cr = new StateUpdateCommandResult();
		}
		if (updateClient) {
			battle.getNextObjectToMoveCache().checkForUpdates(cr, owner);
		}
		return cr;
	}

}