/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.event.BattleCraftUpdateEF;
import mot.engine.players.Player;
import mot.engine.technology.WeaponInstance;

/**
 * This command causes the source craft end its turn in this battle.
 **/
public class BattleEndTurnCmd extends BattleCommand {

    /**
     * The source of this command.
     */
    protected BattleObject source;
	private boolean endMove;
	private boolean dischargeWeapons;
	private Weapon weapon;
	private int qtyWeapons;

    public BattleObject getSource() { return source; }
    public void setSource(BattleObject v) { source = v; }

    public BattleEndTurnCmd(Battle battle, BattleObject source, Player owner, boolean endMove, boolean dischargeWeapons, Weapon weapon, int qtyWeapons)  throws ApplicationError {
        super(owner, battle);
        this.source = source;
		this.endMove = endMove;
		this.dischargeWeapons = dischargeWeapons;
		this.weapon = weapon;
		this.qtyWeapons = qtyWeapons;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult executeBattleCommand()  throws ApplicationError, ValidationError {
		if (source == null) {
			// Client has closed the battle window
		} else if (source instanceof BattleCraft) {
            BattleCraft selectedCraft = (BattleCraft) source;
            if (!selectedCraft.isControlledBy(owner)) throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
            if (endMove) {
        		selectedCraft.fireMissileDefenceWeapons(null);
            	selectedCraft.setMoveRemaining(0);
            }
            if (dischargeWeapons) {
            	dischargeWeapons(selectedCraft);
            }
            BattleCraftUpdateEF updateEvent = new BattleCraftUpdateEF(selectedCraft, selectedCraft.getCraftState());
            MotEngine.instance.multicast(updateEvent, battle.getStartingUserPlayers());
        } else if (source instanceof BattlePlanet) {
            BattlePlanet selectedPlanet = (BattlePlanet) source;
            if (!selectedPlanet.isOwnedBy(owner)) throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
        	dischargeWeapons(selectedPlanet);
        	if (battle.allCraftHaveMoved()) {
        		battle.endTurn();
        	}
        }
		
        cleanup(source);
        
		StateUpdateCommandResult cr = new StateUpdateCommandResult();
		battle.getNextObjectToMoveCache().checkForUpdates(cr, owner);
		return cr;
	}
	
	private void dischargeWeapons(BattleObject bo) {
		if (weapon != null) {
			WeaponInstance wi = bo.getWeapon(weapon);
			if (wi != null) {
				if (qtyWeapons > 0) {
					wi.discharge(qtyWeapons);
				} else {
					wi.discharge();
				}
			}
		} else {
			bo.dischargeWeapons();
		}
	}

}