/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.players.Player;

/**
 * This is a command issued by a user to the engine.
 * A command is independent of the type of UI that asked for the command to be
 * performed. A command must be an XMLBean so that all commands issued to the
 * system can be saved in a Trace and we can re-run a game play.
 **/
public abstract class Command {

    /**
     * The player issuing this command.
     */
    protected Player owner;

    protected Command(Player owner) {
		this.owner = owner;
    }

    /**
     * This method issues this command to the active MOT engine.
     */
	public CommandResult run() throws ApplicationError, ValidationError {
		try {
			CommandResult r = execute();
	        sendMessageQueue();
			return r;
		} catch (ApplicationError e) {
	        sendMessageQueue();
			throw e;
		} catch (ValidationError e) {
	        sendMessageQueue();
			throw e;
		} catch (RuntimeException e) {
	        sendMessageQueue();
			throw e;
		}
    }

	private void sendMessageQueue() {
		if (MotEngine.instance != null && MotEngine.instance.getProxyManager() != null) {        	
			try {
				MotEngine.instance.getProxyManager().broadcastQueue();
			} catch (Throwable e) {
				Logger.logError("Unable to send message queue", e);
			}
		}
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected abstract CommandResult execute() throws ApplicationError, ValidationError;


    public Player getOwner() {
        return owner;
    }

    public void setOwner(Player owner) {
        this.owner = owner;
    }

}