/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftBase;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;


/**
 * A Command issues to a players craft or fleet to attack the enemy. This causes the
 * CraftBase object (craft or fleet) to attack all enemy craft and the outpost located
 * at the stellar object of the enemies.
 **/
public class CraftAttackCmd extends CraftCmd {

    private Planet planet;

	public CraftAttackCmd(CraftBase craft, Planet planet, Player owner) {
        super(craft, owner);
		this.planet = planet;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        CraftBase craft = getCraft();
        Player owner = getOwner();
        if (craft.getOwner() != owner) {
            throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
        }
        StellarObject orbit = craft.getOrbitStellarObject();
        if (orbit == null) {
            throw new ValidationError(L.YOU_CANNOT_ATTACK_CRAFTS_AT_WARP);
        }
        
        craft.setSleeping(false);
        
        Craft [] enemyCrafts = owner.getEnemyCraftAt(orbit, true);
        StellarObjectOrbital orbital = orbit.getOrbital();

        boolean noEnemyOutpost = (orbital == null || !orbital.getOwner().isEnemy(owner)); 
        if (planet == null && enemyCrafts == null && noEnemyOutpost) {
            throw new ValidationError(L.INVALID_COMMAND_YOU_DO_NOT_HAVE_AN_ENEMY_AT_THIS_LOCATION);
        }

        if (planet == null && noEnemyOutpost && orbit instanceof SolarSystem) {
            SolarSystem ss = (SolarSystem) orbit;
            Planet [] planets =ss.getPlanets();
            if (planets != null) {
                for (int i=0; i<planets.length; i++) {
                    if (planets[i].getOwner() != null && planets[i].getOwner().isEnemy(owner) && planets[i].hasDefenses()) {
                        throw new ValidationError(L.INVALID_COMMAND_YOU_MUST_ATTACK_AN_ENEMY_PLANET);
                    }
                }
            }
        }
        
        if (planet != null) {
            Player planetOwner = planet.getOwner();
    		if (owner == planetOwner) {
                throw new ValidationError(L.INVALID_COMMAND_YOU_CANNOT_ATTACK_YOUR_OWN_PLANET);
            }
            if (!planetOwner.isEnemy(owner)) {
                throw new ValidationError(L.INVALID_COMMAND_NO_ENEMY_ON_THIS_PLANET);
            }
            if (!planet.hasDefenses()) {
                throw new ValidationError(L.INVALID_COMMAND_YOU_MUST_BOMBARD_DEFENCELESS_PLANETS);
            }
            craft.attack(planet, enemyCrafts);
        } else {        	
        	craft.attack(orbit, enemyCrafts);
        }
        
        craft.setHasMoved(true);

    	return new StateUpdateCommandResult(EngineVoFactory.getMoo(craft, getOwner()));
	}

}