/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.common.vo.RemoveMooVO;
import mot.engine.adapter.CraftVOAdapter;
import mot.engine.adapter.FleetVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftBase;
import mot.engine.crafts.Fleet;
import mot.engine.players.Player;

/**
 * This is a command issued by a user to add a craft to a fleet.
 **/
public class CraftJoinFleetCmd extends CraftCmd {

    public CraftJoinFleetCmd(CraftBase craft, Player owner) throws ApplicationError {
        super(craft, owner);
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        CraftBase craftBase = getCraft();
        if (craftBase.getOwner() != getOwner()) {
            throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
        }
        if (craftBase.getNavigator() != null) {
            throw new ValidationError(L.INVALID_COMMAND_HAS_ORDERS);
        }
        craftBase.setSleeping(false);
        if (craftBase instanceof Craft) {
            Craft craft = (Craft) craftBase;
            Orbit orbit = craft.getOrbit();
            Fleet [] fleets = craft.getOwner().getFleetAt(orbit, false);
            Fleet fleet = null;
            if (fleets == null) {
                fleet = new Fleet(craft.getOwner(), craft.getOrbit());
                fleet.add(craft);
                craft.getOwner().setFleet(fleet);
            } else {
            	fleet = fleets[0]; 
                fleet.add(craft);
            }
            return new StateUpdateCommandResult(new FleetVOAdapter(fleet, owner), new CraftVOAdapter(craft, owner));
        } else if (craftBase instanceof Fleet) {
        	Fleet srcFleet = (Fleet) craftBase;
        	StateUpdateCommandResult cr = new StateUpdateCommandResult(new FleetVOAdapter(srcFleet, owner));
        	Orbit orbit = srcFleet.getOrbit();
            Fleet [] fleets = srcFleet.getOwner().getFleetAt(orbit, false);
            if (fleets == null || fleets.length < 2) {
            	// Join all crafts at this location
                Craft [] crafts = srcFleet.getOwner().getCraftsAt(srcFleet.getOrbit(), false, false);
                if (crafts == null || crafts.length == 0) {
                	throw new ValidationError("Invalid Command - No craft to join this fleet");
                }
        		srcFleet.add(crafts);
        		cr.addStateUpdate(new FleetVOAdapter(srcFleet, owner));
        		cr.addStateUpdate(EngineVoFactory.getCrafts(srcFleet.getCrafts(), owner));
                return cr;
            } else {
            	Craft [] crafts = null;
            	Fleet destFleet = null;
            	if (srcFleet == fleets[0]) {
            		destFleet = fleets[1];
                    crafts = fleets[0].getCrafts();
            		cr.addStateUpdate(new RemoveMooVO(fleets[0].getName()));
            		fleets[0].destroy();
            	} else {
            		destFleet = fleets[0];
                    crafts = fleets[1].getCrafts();
            		cr.addStateUpdate(new RemoveMooVO(fleets[1].getName()));
            		fleets[1].destroy();
            	}
            	destFleet.add(crafts);
        		cr.addStateUpdate(new FleetVOAdapter(destFleet, owner));
        		cr.addStateUpdate(EngineVoFactory.getCrafts(crafts, owner));
                return cr;
            }
        }
        return null;
	}

}