/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.PlayerAssets;
import mot.common.command_results.CommandResult;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.relations.AllianceTreaty;
import mot.engine.relations.NonAgressionTreaty;
import mot.engine.relations.Relations;

/**
 * This command causes the player to request the receiver to swap all the assets in
 * wantedAssets with those in offeredAssets.
 * Note a response does not come back immediately - An event will be sent some
 * time soon containing the response.
 */
public class DiplomaticRequestCmd extends Command {

    /**
     * The player receiving the request to swap assets.
     */
    private Player receiver;

    /**
     * The proposal that is being made, treaties and assets.
     */
    private DiplomaticProposal proposal;

    /**
     * The assets the player that owns this command wants to receive from the receiver.
     */
    private String message;

    public DiplomaticRequestCmd(Player player, Player receiver, DiplomaticProposal proposal, String message) {
        super(player);
        this.receiver = receiver;
        this.proposal = proposal;
        this.message = message;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        if (receiver == null || owner == null) {
            throw new ValidationError("Must specify two players - Giver and receiver");
        }
        if (!owner.hasDiplomaticRelationsWith(receiver)) {
            throw new ValidationError("You do not have diplomatic relations with "+receiver.getName());
        }

        PlayerAssets offeredAssets = proposal.getOffered();
        PlayerAssets wantedAssets = proposal.getWanted();
        
        if (offeredAssets != null) {
        	owner.hasAssets(offeredAssets);
            if (offeredAssets.getTechnologies() != null) {
                if (offeredAssets.getTechnologList().containsDuplicates()) {
                    throw new ValidationError("You cannot have duplicates");
                }
            }
        }
        
        if (wantedAssets != null) {
        	receiver.hasAssets(wantedAssets);
            if (wantedAssets.getTechnologies() != null) {
                if (wantedAssets.getTechnologList().containsDuplicates()) {
                    throw new ValidationError("You cannot have duplicates");
                }
                if (wantedAssets.getTechnologList().containsAny(owner.getTechnologies())) {
                    throw new ValidationError("You cannot ask for technology you already have");
                }
            }
        }

        if (offeredAssets != null && wantedAssets != null) {
        	if (offeredAssets.getTechnologies() != null && wantedAssets.getTechnologies() != null) {
                if (offeredAssets.getTechnologList().containsAny(wantedAssets.getTechnologList())) {
                    throw new ValidationError("Cannot give and receive the same technology");
                }
        	}
        }

		if (proposal.isWar() && (proposal.isAlliance() || proposal.isNonAggression() || proposal.isResearch() || proposal.isTrade() || proposal.isPeace())) {
            throw new ValidationError("Cannot declare war and other treaties "+receiver.getName());
        }

        Relations relations = MotEngine.instance.getRelations();
		if (proposal.isAlliance() || proposal.isNonAggression() || proposal.isResearch() || proposal.isTrade()) {
            if (relations.isWar(owner, receiver)) {
                throw new ValidationError("You are at war with player "+receiver.getName());
            }
        }

        if (proposal.isResearch()) {
        	if (relations.hasMaximumResearchers(getOwner())) {
                throw new ValidationError("You have already used your maximum quantity of research agreements");
            }
        }

        if (proposal.isTrade()) {
        	if (relations.hasMaximumPlayerTradeRoutes(getOwner())) {
                throw new ValidationError("You have already used you maximum quantity of trade routes");
            }
        }
        
        if (proposal.isPeace()) {
            if (!relations.isWar(owner, receiver)) {
                throw new ValidationError("You are not at ware with player "+receiver.getName());
            }
        }
        
        if (proposal.isWar()) {
    		if (relations.isWar(owner, receiver)) {
                throw new ValidationError("You are at war with player "+receiver.getName());
            }
    		NonAgressionTreaty nat = relations.getNonAggressionTreaty(owner, receiver);
    		if (nat != null && !nat.isOutOfDate()) {
    			ComputerPlayer[] players = MotEngine.instance.getPlayersNotNull().getComputerPlayers();
    			if (players != null) {
    				for (int i = 0; i < players.length; i++) {
    					players[i].getStratagist().notifyBreakNonAggression(owner);
    				}
    			}
    		}
    		AllianceTreaty at = relations.getAllianceTreaty(owner, receiver);
    		if (at != null && !at.isOutOfDate()) {
    			ComputerPlayer[] players = MotEngine.instance.getPlayersNotNull().getComputerPlayers();
    			if (players != null) {
    				for (int i = 0; i < players.length; i++) {
    					players[i].getStratagist().notifyBreakAlliance(owner);
    				}
    			}
    		}        
    	}

        receiver.diplomaticRequest(owner, proposal, message);

        return null;
	}
    
}