/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.PlayerAssets;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.common.EngineVoFactory;
import mot.engine.players.Player;

/**
 * This is a command issued by a user to accept or reject a diplomatic proposal.
 * The player other must have already made the request and this command
 * is issued by the receiver of the proposal to accept it.
 **/
public class DiplomaticResponseCmd extends Command {

    /**
     * The proposal that is being made, treaties and assets.
     */
    private DiplomaticProposal proposal;

    /**
     * The other player (who made the proposal and is waiting for this response).
     */
    protected Player other;
    
    /**
     * Is the proposal accepted
     */
    protected boolean accept;
    
    /**
     * A human readable text message
     */
    protected String message;


    public DiplomaticResponseCmd(Player player, Player other, DiplomaticProposal proposal, boolean accept, String message) {
    	super(player);
		this.other = other;
		this.proposal = proposal;
		this.accept = accept;
		this.message = message;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        if (other == null || owner == null) {
            throw new ValidationError("Must specify two players");
        }
        
        if (accept) {
            PlayerAssets offeredAssets = proposal.getOffered();
            PlayerAssets wantedAssets = proposal.getWanted();
            
            if (offeredAssets != null) {
            	other.hasAssets(offeredAssets);
                if (offeredAssets.getTechnologies() != null) {
                    if (offeredAssets.getTechnologList().containsDuplicates()) {
                        throw new ValidationError("You cannot have duplicates");
                    }
                }
            }
            
            if (wantedAssets != null) {
            	owner.hasAssets(wantedAssets);
                if (wantedAssets.getTechnologies() != null) {
                    if (wantedAssets.getTechnologList().containsDuplicates()) {
                        throw new ValidationError("You cannot have duplicates");
                    }
                    if (wantedAssets.getTechnologList().containsAny(other.getTechnologies())) {
                        throw new ValidationError("You cannot ask for technology you already have");
                    }
                }
            }
        }

        other.diplomaticResponse(getOwner(), proposal, accept, message);
        
        StateUpdateCommandResult cr = new StateUpdateCommandResult();
        cr.addStateUpdate( EngineVoFactory.getPlayerAssetsUpdate(proposal, owner, other) );
		return cr;
	}

}