/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import java.io.File;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StringCR;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.players.Player;
import mot.engine.players.Players;
import mot.engine.players.UserPlayer;
import mot.engine.validators.SavedGameFileNameValidator;

/**
 * This is a command issued by a user to the engine.
 **/
public class LoadCmd extends Command {

    private String fileName;
	private String playerName;
	private String oldPlayerName;

	public LoadCmd(String oldPlayerName, String fileName, String playerName) {
        super(null);
		this.oldPlayerName = oldPlayerName;
        this.fileName = fileName;
		this.playerName = playerName;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
		File file = SavedGameFileNameValidator.validate(fileName, false);	
		
		if (MotEngine.instance != null) {
			UserPlayer [] oldPlayers = MotEngine.instance.getPlayersNotNull().getUserPlayers();
			if (oldPlayers != null && oldPlayers.length > 1) {
        		throw new ValidationError(L.MULTIPLAYER_GAME_IN_POROGRESS);
			}
		}
        
		MotEngine originalGame = MotEngine.instance;
		Players players = null;
		try {
	        MotEngine.load(file);

	        players = MotEngine.instance.getPlayers();
			if (players == null || players.getQuantity() == 0) {
				throw new ValidationError(L.NO_USER_PLAYERS_EXIST_IN_THIS_GAME);
			}
			if (playerName != null) {
				Player player = players.getPlayer(playerName);
				if (player == null) throw new ValidationError(L.GAME_LOADED_BUT_COULD_NOT_FIND_PLAYER_+playerName);
			} else {
				UserPlayer[] userPlayers = players.getUserPlayers();
				if (Logger.isDebug()) {
					Logger.logDebug("LoadCmd.execute() User players="+Comp.toString(userPlayers));
				}
				if (userPlayers == null || userPlayers.length > 1) {
					throw new ValidationError(L.GAME_LOADED_BUT_NO_USER_SELECTED);
				}
				playerName = userPlayers[0].getName();
			}
		} catch (ValidationError e) {
			MotEngine.restoreInstance(originalGame);
			throw e;
		} catch (Exception e) {
			Logger.logError("", e);
			MotEngine.restoreInstance(originalGame);
            throw new ValidationError("File "+fileName+" is not a valid game");
		}
		
		Player player = players.getPlayer(playerName);
		StringCR r = new StringCR(playerName);
		MotEngine.instance.addStateUpdate(r, player, true, true, true, true, true, true, true, true, true, true, true);
		return r;
	}


}