/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.adapter.TransportVOAdapter;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftBase;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRouteFactory;
import mot.engine.players.Player;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTask;
import mot.engine.transports.TransportTaskCreateTradeRoute;
import mot.engine.transports.TransportTaskDeployStation;
import mot.engine.transports.TransportTaskInvade;
import mot.engine.transports.TransportTaskMoveColonists;


/**
 * This is a issues to deploy the contents of a transport to the stellar object or
 * planet the transport is currently in orbit at.
 **/
public class MooDeployCmd extends Command {

	private MoveableOwnedObject moo;
    private StellarObject destStellarObject; 
    private Planet destPlanet;
    
    public MooDeployCmd(MoveableOwnedObject moo, StellarObject destStellarObject, Planet destPlanet, Player owner) {
        super(owner);
        this.moo = moo;
        this.destStellarObject = destStellarObject;
        this.destPlanet = destPlanet;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        if (moo.getOwner() != getOwner()) {
            throw new ValidationError(L.THIS_TRANSPORT_DOES_NOT_BELONG_TO_YOU);
        }
        if (moo instanceof Transport) {
        	return deployTransport((Transport) moo);
        } else if (moo instanceof CraftBase){
        	return deployCraft((CraftBase) moo);
        } else {
            throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANYTHING_TO_DEPLOY);
        }
	}
	
	private CommandResult deployTransport(Transport transport)  throws ApplicationError, ValidationError {
        TransportTask task = transport.getTask();
        
        if (task == null) {
            throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANYTHING_TO_DEPLOY);
        }
        
        if (transport.getOrbit() == null) {
            throw new ValidationError(L.TRANSPORT_MUST_BE_IN_ORBIT_TO_DEPLOY);
        }
        
        if (destPlanet == null) {
            if (destStellarObject == null) {
                throw new ValidationError(L.INVALID_COMMAND_YOU_MUST_SPECIFY_A_STELLAR_OBJECT);
            }
            if (!transport.getTask().canDeployTo(destStellarObject)) {
                throw new ValidationError(L.INVALID_COMMAND_YOU_MUST_SPECIFY_A_DEPLOY_TARGET);
            }
        } else {
            if (destStellarObject == null) {
                destStellarObject = destPlanet.getStellarObject();
            }
        }

        if (destStellarObject == null) {
            throw new ValidationError(L.INVALID_COMMAND_YOU_MUST_SPECIFY_A_STELLAR_OBJECT);
        }
        
        moo.setSleeping(false);

        if (task instanceof TransportTaskMoveColonists) {
        	return deployTransportMoveColonists(transport);
        } else if (task instanceof TransportTaskInvade) {
        	return deployTransportMoveInvade(transport);
        } else if (task instanceof TransportTaskDeployStation) {
        	return deployTransportDeployStation(transport);
        } else if (task instanceof TransportTaskCreateTradeRoute) {
        	return deployTransportCreateTradeRoute(transport);
        }
                
        throw new ValidationError(L.INVALID_COMMAND);

	}

	private CommandResult deployTransportMoveColonists(Transport transport)  throws ApplicationError, ValidationError {
        TransportTask task = transport.getTask();

        Orbit orbit = transport.getOrbit();
    	int qtyEnemyCraft = transport.getOwner().getQtyEnemyCraftAt(orbit.getStellarObject());
    	if (qtyEnemyCraft > 0) {
    		throw new ValidationError(L.INVALID_COMMAND_THIS_TRANSPORT_CANNOT_BE_DEPLOYED_CLOSE_TO_ENEMY_ACVTIVITY);
    	}

        if (destPlanet == null || destPlanet.getOwner() != getOwner()) {
            throw new ValidationError(L.INVALID_COMMAND_YOU_CAN_ONLY_MOVE_SETTLERS_TO_YOUR_OWN_COLONIES);
        }
        if (destPlanet.isMiningColony()) {
            throw new ValidationError(L.INVALID_COMMAND_YOU_CAN_ONLY_MOVE_SETTLERS_TO_YOUR_OWN_COLONIES);
        }

        StateUpdateCommandResult cr = task.deploy(transport, destStellarObject, destPlanet);
        
		return cr;

	}

	private CommandResult deployTransportMoveInvade(Transport transport)  throws ApplicationError, ValidationError {
        TransportTask task = transport.getTask();

        if (destPlanet == null || destPlanet.getOwner() == null || destPlanet.getOwner() == getOwner()) {
            throw new ValidationError(L.INVALID_COMMAND_YOU_CAN_ONLY_INVADE_PLANETS_OWNED_BY_AN_ENEMY);
        }
        if (!MotEngine.instance.getRelations().isWar(destPlanet.getOwner(), getOwner())) {
            throw new ValidationError(L.INVALID_COMMAND_YOU_CANNOT_INVADE_A_PLANET_UNLESS_YOU_ARE_AT_WAR);
        }
        if (destPlanet.hasDefenses()) {
            throw new ValidationError(L.INVALID_COMMAND_CANNOT_INVADE_A_PLANET_THAT_HAS_DEFENCES);
        }

        StateUpdateCommandResult cr = task.deploy(transport, destStellarObject, destPlanet);
        
		return cr;

	}

	private CommandResult deployTransportDeployStation(Transport transport)  throws ApplicationError, ValidationError {
        TransportTask task = transport.getTask();

        Orbit orbit = transport.getOrbit();
    	int qtyEnemyCraft = transport.getOwner().getQtyEnemyCraftAt(orbit.getStellarObject());
    	if (qtyEnemyCraft > 0) {
    		throw new ValidationError(L.INVALID_COMMAND_THIS_TRANSPORT_CANNOT_BE_DEPLOYED_CLOSE_TO_ENEMY_ACVTIVITY);
    	}

		TransportTaskDeployStation deployTask = (TransportTaskDeployStation) task;
        if (destStellarObject.hasOwner() && !destStellarObject.isOwnedBy(getOwner())) {
            throw new ValidationError(L.INVALID_COMMAND_THE_CURRENT_OWNER_PREVENTS_YOU_FROM_DEPLOYING_HERE);
        }
        
        if (destPlanet != null) {
	        Player planetOwner = destPlanet.getOwner();
			if (planetOwner != null) {
	        	if (planetOwner == owner) {
	        		if (destPlanet.isMiningColony()) {
	        			throw new ValidationError(destPlanet.getName()+L.THIS_PLANET_ALREADY_HAS_A_MINING_STATION);
	        		}
	        		// Special case - rebase the transport
	        		deployTask.setOrigin(destPlanet);
	        		return new StateUpdateCommandResult(new TransportVOAdapter(transport));
	        	} else {
	        		throw new ValidationError(L.INVALID_COMMAND_THIS_IS_ALREADY_TAKEN);
	        	}
	        }
	        if (deployTask.getOrigin() != null && deployTask.getOrigin().getPlanet().getMiningStation() != null) {
	            throw new ValidationError(deployTask.getOrigin().getPlanet().getName()+L.THIS_PLANET_ALREADY_HAS_A_MINING_STATION);
	        }
        }
        
        if (destPlanet == null && destStellarObject.getOrbital() != null) {
        	StellarObjectOrbital orbital = destStellarObject.getOrbital();
			if (orbital.getOrbital().getHullTech().isOutpost() && orbital.isOwnedBy(getOwner())) {
				// Special case - we allow outposts to be replaced with space stations!
			} else {
				throw new ValidationError(L.INVALID_COMMAND_THIS_IS_ALREADY_TAKEN);
			}
        }

		TransportTaskDeployStation t = (TransportTaskDeployStation) task;
		if (transport.getOrbit().getStellarObject() != destStellarObject) {
			Orbit destOrbit = new Orbit(destStellarObject, destPlanet);
			t.setDestination(destOrbit);
			transport.moveTo(destOrbit);
			return null;
		}

        StateUpdateCommandResult cr = task.deploy(transport, destStellarObject, destPlanet);
        
		return cr;

	}

	private CommandResult deployTransportCreateTradeRoute(Transport transport)  throws ApplicationError, ValidationError {
        TransportTask task = transport.getTask();

        Orbit orbit = transport.getOrbit();
    	int qtyEnemyCraft = transport.getOwner().getQtyEnemyCraftAt(orbit.getStellarObject());
    	if (qtyEnemyCraft > 0) {
    		throw new ValidationError(L.INVALID_COMMAND_THIS_TRANSPORT_CANNOT_BE_DEPLOYED_CLOSE_TO_ENEMY_ACVTIVITY);
    	}

    	if (!(destStellarObject instanceof SolarSystem)) {
    		throw new ValidationError(L.INVALID_COMMAND_CANNOT_BUILD_THIS_TRADE_ROUTE);
    	}
    	SolarSystem dest = (SolarSystem) destStellarObject;
    	TransportTaskCreateTradeRoute ctrt = (TransportTaskCreateTradeRoute) task;
    	SolarSystem origin = ((TransportTaskCreateTradeRoute) task).getOrigin().getSolarSystem();
		if (origin.getSingleOwnerOrNull() != owner || dest.getSingleOwnerOrNull() != owner) {
			throw new ValidationError(L.INVALID_COMMAND_CANNOT_BUILD_THIS_TRADE_ROUTE_MOST_OWN);
    	}
		if (origin == dest) {
			throw new ValidationError(L.INVALID_COMMAND_CANNOT_BUILD_THIS_TRADE_ROUTE);
		}
		int maxDistance = owner.getMaxSolarSystemTradeRouteDistance();
		int distance = origin.getDistanceTo(dest);
		if (distance > maxDistance) {
			throw new ValidationError("Invalid Command - Maximum distance between solar systems for a Trade Route is "+maxDistance+" ["+distance+"]");
    	}
		if (!TradeRouteFactory.canCreateTradeRoute(origin, dest)) {
			throw new ValidationError(L.INVALID_COMMAND_CANNOT_BUILD_THIS_TRADE_ROUTE);
    	}

        StateUpdateCommandResult cr = task.deploy(transport, destStellarObject, destPlanet);
        
		return cr;

	}

	private CommandResult deployCraft(CraftBase craftBase)  throws ApplicationError, ValidationError {
        if (destPlanet.getOwner() == null) {
            throw new ValidationError(L.INVALID_COMMAND_THIS_PLANET_HAS_NO_OWNER);
        }
        if (!MotEngine.instance.getRelations().isWar(getOwner(), destPlanet.getOwner())) {
            throw new ValidationError(L.INVALID_COMMAND_YOU_ARE_NOT_AT_WAR_WITH_PLAYER+destPlanet.getOwner().getName());
        }
        if (destPlanet.hasWeapons()) {
            throw new ValidationError(L.INVALID_COMMAND_CANNOT_ATTACK_A_PLANET_THAT_HAS_DEFENSES);
        }
        if (destPlanet.getCrisis() != null) {
            throw new ValidationError(L.INVALID_COMMAND_CANNOT_DEPLOY_TO_A_PLANET_IN_CRISIS);
        }
        Craft [] enemies = getOwner().getEnemyCraftAt(destPlanet.getStellarObject(), false);
        if (enemies != null && enemies.length > 0) {
            throw new ValidationError(L.INVALID_COMMAND_CANNOT_BOMBARD_A_PLANET_WHEN_ENEMY_CRAFTS_ARE_IN_ORBIT_WITH_YOU);
        }
        if (craftBase.hasMoved()) {
            throw new ValidationError(L.THIS_HAS_ALREADY_MOVED);
        }
		if (!craftBase.containsDeployOnPlanet()) {
            throw new ValidationError(L.THIS_DOES_NOT_HAVE_ANYTHING_TO_DEPLOY);
		}
		craftBase.setHasMoved(true);
		craftBase.deploy(destPlanet);
		return null;
		
	}
	
}