/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.common.vo.RemoveMooVO;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.CraftOrbitFactoryBasic;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.StellarObjects;
import mot.engine.planets.TradeRoute;
import mot.engine.players.Player;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskCreateTradeRoute;
import mot.engine.transports.TransportTaskMoveColonists;


/**
 * This is a issues to move a craft/fleet by a player in the game (on the galaxy map).
 * The moo (Movable Owned Object) must not be null and is the craft (or whatever) to move
 * The destination must not be null and is the stellar object to move the moo to.
 * The planet may be null but if it is not null then stellar object must be the solar system
 * containing this planet and this planet should be the orbit when the moo arrives at the destination.
 * If generateOrbit is true then planet must be null and this command will select the planetary orbit
 * to use when this moo arrives at the destination.
 **/
public class MooMoveCmd extends MooCmd {

	private StellarObject destination;
    private Planet planet;
    private boolean generateOrbit;

    public MooMoveCmd(MoveableOwnedObject moo, StellarObject destination, Planet planet, boolean generateOrbit, Player owner) {
        super(moo, owner);
        this.destination = destination;
        this.planet = planet;
        this.generateOrbit = generateOrbit;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        MoveableOwnedObject moo = getMoo();
        if (moo.getOrbit() == null) {
            throw new ValidationError(L.INVALID_COMMAND_THIS_CANNOT_BE_GIVEN_ORDERS_WHILE_AT_WARP);
        }
        if (moo.getOwner() != getOwner()) {
            throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
        }
        if (moo.getEngine() == null) {
            // This object has been deleted!
            return null;
        }
		if (!moo.canTravelTo(destination)) {
			throw new ValidationError(L.INVALID_COMMAND_DESTINATION_IS_OUT_OF_RANGE);
		}
        if (moo.isInABattle()) {
			throw new ValidationError(L.THIS_OBJECT_IS_UNDER_ATTACK);
        }
        if (moo instanceof Transport) {
        	Transport transport = (Transport) moo;
        	if (transport.getTask() instanceof TransportTaskMoveColonists) {
            	if (planet!= null && !planet.isOwnedBy(owner)) {
        			throw new ValidationError(L.CANNOT_TRANSPORT_COLONISTS_TO_A_PLANET_THAT_DOOES_NOT_BELONG_TO_YOU);
            	}
            	if (destination!= null && !destination.isOwnedBy(owner)) {
        			throw new ValidationError(L.CANNOT_TRANSPORT_COLONISTS_TO_A_PLANET_THAT_DOOES_NOT_BELONG_TO_YOU);
            	}
        	} else if (transport.getTask() instanceof TransportTaskCreateTradeRoute) {
        		int maxTradeRouteDistance = owner.getMaxSolarSystemTradeRouteDistance();
        		int distance = moo.getOrbit().getStellarObject().getDistanceTo(destination);
				if (distance > maxTradeRouteDistance) {
        			throw new ValidationError("Invalid Command - Maximum distance between solar systems for a Trade Route is "+maxTradeRouteDistance+" ["+distance+"]");
        		}
        	}
        }
        if (planet == null && generateOrbit && destination instanceof SolarSystem) {
        	SolarSystem ss = (SolarSystem) destination;
        	planet = new CraftOrbitFactoryBasic().getOrbit(ss, moo);
        }
        StellarObjects sos = MotEngine.getInstance().getStellarObjects();
        if (sos.hasGalacticIncumbentsBetween(moo.getLocation(), destination.getLocation(), BlackHole.class)) {
			throw new ValidationError("Crew refuse to navigate through a black hole");
        }
        
        StellarObject actualOrbit = moo.getActualOrbit();
        boolean sameSS = destination == actualOrbit;
        
        moo.setSleeping(false);
        
        moo.moveTo(new Orbit(destination, planet));

        StateUpdateCommandResult cr = new StateUpdateCommandResult();
        if (moo instanceof Transport) {
        	if (planet != null) {
        		cr.addStateUpdate( new PlanetVOAdapter(planet, owner) );
        	}
        	if (destination != null && destination instanceof SolarSystem) {
        		SolarSystem ss = (SolarSystem) destination;
        		TradeRoute[] tradeRoutes = ss.getTradeRoutes();
				if (tradeRoutes != null) {
					cr.addStateUpdate(EngineVoFactory.getTradeRoutes(tradeRoutes, owner));
        		}
        	}
        	if ( ((Transport)moo).getTask() == null ) {
        		cr.addStateUpdate( new RemoveMooVO(moo.getName()) );
        		return cr;
        	}
        }
		if (sameSS) {
			// May have just visited the solar system
        	cr.addStateUpdate(EngineVoFactory.getStellarObject(actualOrbit, owner));
        }

		EngineVoFactory.addMooDeepStateUpdate(moo, owner, cr);
		
		return cr;
	}

}