/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.bean.NewGameData;
import mot.common.bean.NewGameDataComputerPlayerDetails;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.event.StartGameEvent;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.common.vo.FlushDelegateCacheVO;
import mot.engine.ConnectionList;
import mot.engine.MotEngine;
import mot.engine.SocketHandlerThreadEngine;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.setup.MotFactory;
import mot.engine.setup.PlayerRegistry;

/**
 * This is a command issued by a user to the engine.
 * It causes the engine to start a new game with the settings passed in. This creates
 * a single user player and a new game for the userOptions/alignment passed in plus one
 * user player for evenly NewGameUserPlayerCmd executed to date. It will create
 * qtyComputerPlayers computer players in a game at level, qty planets and galaxy size passed in.
 **/
public class NewGameCmd extends Command {

	private String clientId;
	private NewGameUserPlayerData userPlayerData;
	private NewGameData gameData;
	private String oldPlayerName;

    public NewGameCmd(String clientId, String oldPlayerName, NewGameUserPlayerData userPlayerData, NewGameData gameData) {
        super(null);
		this.clientId = clientId;
		this.oldPlayerName = oldPlayerName;
		this.userPlayerData = userPlayerData;
		this.gameData = gameData;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
		
		if (MotEngine.instance != null) {
			UserPlayer [] oldPlayers = MotEngine.instance.getPlayersNotNull().getUserPlayers();
			if (oldPlayers != null && oldPlayers.length > 1) {
        		throw new ValidationError(L.MULTIPLAYER_GAME_IN_POROGRESS);
			}
		}
		
		new NewGameUserPlayerCmd(clientId, userPlayerData).run();
		
		String raceName = userPlayerData.getRaceName();
		String raceColour = userPlayerData.getColour();
        NewGameUserPlayerData [] users = PlayerRegistry.getUserPlayerDetails();
        
        NewGameDataComputerPlayerDetails[] computerPlayers = gameData.getComputerPlayers();
        if (computerPlayers != null) {
        	for (int i = 0; i < computerPlayers.length; i++) {
        		if (computerPlayers[i].getRaceName() != null) {        			
        			NewGameUserPlayerData v =  PlayerRegistry.getPlayerByName(computerPlayers[i].getRaceName(), null);
        			if (v != null) {
        				throw new ValidationError("Duplicate Race: "+computerPlayers[i].getRaceName());
        			}
        			for (int j = 0; j < computerPlayers.length; j++) {
        				if (i != j && Comp.same(computerPlayers[i].getRaceName(), computerPlayers[j].getRaceName())) {
        					throw new ValidationError("Duplicate Race: "+computerPlayers[i].getRaceName());
        				}
        			}
        		}
			}
        }
        
        if (users == null) {
			throw new ValidationError("No User Players");
        }
        
        if (gameData.isHasTutorial() && users.length > 1) {
			throw new ValidationError("Multi player game cannot have a tutorial");
        }
        
        if (MotEngine.instance != null && MotEngine.instance.getProxyManager() != null) {
        	MotEngine.instance.getProxyManager().removeAll();
        }
        
        MotFactory.create(gameData, users);
        
        PlayerRegistry.clear();
        
        owner = MotEngine.instance.getPlayersNotNull().getUserPlayer(raceName);
        StateUpdateCommandResult r = new StateUpdateCommandResult();
        r.addStateUpdate(new FlushDelegateCacheVO());
        MotEngine.instance.addStateUpdate(r, owner, true, true, true, true, true, true, true, true, true, true, true);
        
        SocketHandlerThreadEngine [] remoteClients = ConnectionList.getRemoteServerConnections();
    	for (int i = 0; i < remoteClients.length; i++) {		
    		Player p = MotEngine.instance.getPlayersNotNull().getPlayer( remoteClients[i].getOwnerName() );
    		StartGameEvent startGameEvent = new StartGameEvent();
    		MotEngine.instance.addStateUpdate(startGameEvent, p, true, true, true, true, true, true, true, true, true, true, true);
    		remoteClients[i].process(startGameEvent);
		}

        return r;
	}

	private int count(NewGameDataComputerPlayerDetails[] computerPlayers, String cpr) {
		if (computerPlayers == null) return 0;
		int qty = 0;
		for (int i = 0; i < computerPlayers.length; i++) {
			if (Comp.same(cpr, computerPlayers[i].getRaceName())) {
				qty++;
			}
		}
		return qty;
	}



}