/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.client.coms.LocalServerConnection;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.command_results.CommandResult;
import mot.common.coms.protocols.ClientEventProcessor;
import mot.common.coms.protocols.ServerConnection;
import mot.common.event.NewGameUserPlayerEvent;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.engine.ConnectionList;
import mot.engine.MotEngine;
import mot.engine.setup.PlayerRegistry;

/**
 * This is a command issued by a user to the engine.
 * It stores up the options selected by a user players as they register for a new game, then
 * when a NewGameCmd is fired, the accumulated options for all the users are consumed to create
 * a game.
 * 
 * NOTE - This command deletes the game currently in this engine.
 **/
public class NewGameUserPlayerCmd extends Command {
	
	private String clientId;
	private NewGameUserPlayerData userPlayerData;

    public NewGameUserPlayerCmd(String clientId, NewGameUserPlayerData userPlayerData) {
        super(null);
		this.clientId = clientId;
        this.userPlayerData = userPlayerData;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        if (MotEngine.getInstance() != null) MotEngine.newInstance();
		if (userPlayerData == null || userPlayerData.getRaceName() == null || userPlayerData.getUserOptions() == null || userPlayerData.getColour() == null) {
			throw new ValidationError("Player setup options not specified");
		}
		String raceName = userPlayerData.getRaceName();
    	NewGameUserPlayerData v1 =  PlayerRegistry.getPlayerByName(raceName, clientId);
    	if (v1 != null) {
    		throw new ValidationError("Duplicate Race: "+raceName);
    	}
    	NewGameUserPlayerData v2 =  PlayerRegistry.getPlayerByColour(userPlayerData.getColour(), clientId);
    	if (v2 != null) {
    		throw new ValidationError("Duplicate Colour: "+userPlayerData.getColour());
    	}
        PlayerRegistry.update(clientId, userPlayerData);
        
        if (!Comp.same(ServerConnection.LOCAL_CLIENT_ID, clientId)) {        	
        	LocalServerConnection lsc = ConnectionList.getLocalServerConnection();
        	if (lsc != null) {
        		ClientEventProcessor ep = lsc.getClientEventProcessor();
        		if (ep != null) {
        			ep.process(new NewGameUserPlayerEvent(clientId, userPlayerData));
        		}
        	}
        }
        
        return null;
	}
    
}