/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import java.util.ArrayList;

import mot.common.bean.NamedObject;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.common.vo.RenameListVO;
import mot.common.vo.RenameVO;
import mot.engine.MotEngine;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Fleet;
import mot.engine.event.StateUpdateEF;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;

/**
 * This command renames an object to new name.
 * Currently this is only implements for Planet and SolarSystem.
 **/
public class RenameCmd extends Command {


	/**
     * The object to be given a new name.
     */
    private NamedObject object;

    /**
     * The quantity of times this planet should repeat this project.
     */
    private String newName;

    public RenameCmd(NamedObject object, String newName, Player owner) throws ApplicationError {
        super(owner);
        this.object = object;
        this.newName = newName;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        if (newName == null) throw new ValidationError(L.YOU_CANNOT_CHANGE_A_NAME_TO_NOTHING);
        
        RenameListVO s = new RenameListVO();

        if (object instanceof Planet) {
            Planet existing = MotEngine.instance.getPlanets().getPlanet(newName);
            if (existing != null) {
                throw new ValidationError(L.THIS_NAME_IS_ALREADY_IN_USE);
            }
            Planet planet = (Planet)object; 
            if (!planet.isOwnedBy(getOwner())) {
                throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
            }
            object.changeNameTo(newName);
            s.addPlanet(planet.getName(), newName);
        } else if (object instanceof SolarSystem) {
            StellarObject existing = MotEngine.instance.getStellarObjects().getStellarObject(newName);
            if (existing != null) {
                throw new ValidationError(L.THIS_NAME_IS_ALREADY_IN_USE);
            }
            SolarSystem ss = (SolarSystem)object; 
            if (!ss.isOwnedBy(getOwner())) {
                throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
            }
            Planet [] planets = ss.getPlanets();
            if (planets != null) {
                String ssName = ss.getName();
                for (int i=0; i<planets.length; i++) {
                    String planetName = planets[i].getName();
                    int index = planetName.indexOf(ssName);
                    int length = ssName.length();
                    if (index < 0) {
                    	ArrayList oldNames = ss.getOldNames();
                    	if (oldNames != null && oldNames.size() > 0) {
                    		String oldSSName = (String) oldNames.get(0);
                    		index = planetName.indexOf(oldSSName);
                    		length = oldSSName.length();
                    	}
                    }
                    if (index >= 0) {
                        String newPlanetName = newName + planetName.substring(index+length);
                        String oldPlanetName = planets[i].getName();
                        planets[i].changeNameTo(newPlanetName);
                        s.addPlanet(oldPlanetName, newPlanetName);
                    }
                }
            }
            String oldSSName = ss.getName();
            object.changeNameTo(newName);
            s.setSolarSystem(new RenameVO(oldSSName, newName));
        } else if (object instanceof Fleet) {
        	Fleet fleet = (Fleet) object;
            MoveableOwnedObject existing = MotEngine.instance.getMoveableOwnedObjects().getMOO(newName);
            if (existing != null) {
                throw new ValidationError(L.THIS_NAME_IS_ALREADY_IN_USE);
            }
            if (!fleet.isOwnedBy(getOwner())) {
                throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
            }
            String oldName = fleet.getName();
            fleet.changeNameTo(newName);
            s.setMoo(oldName, newName);
        } else {
            throw new ValidationError(L.YOU_CAN_ONLY_RENAME_A_PLANET_OR_A_SOLAR_SYSTEM);
        }
        
        MotEngine.instance.broadcastToUserPlayers(new StateUpdateEF(s), owner);
        
        return new StateUpdateCommandResult(s);
    }


    public void setObject(NamedObject object) {
        this.object = object;
    }


    public NamedObject getObject() {
        return object;
    }


    public void setNewName(String newName) {
        this.newName = newName;
    }


    public String getNewName() {
        return newName;
    }



}