/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * If development is not null, then planet auto development percent is set to development.
 * If ecology is not null, then planet auto ecology percent is set to ecology.
 * If planet auto development percent is set to automatic, this command causes development
 * on a planet allocated to be calculated automatically.
 * If planet auto ecology percent percent is set to automatic, this command causes ecology
 * on a planet allocated to be calculated automatically.
 **/
public class SetPlanetPercentsCmd extends PlanetCmd {

	private double developmentPercent;
    private double ecologyPercent;
    private double productionPercent;
    private double researchPercent;

    public SetPlanetPercentsCmd(Planet planet, double developmentPercent, double ecologyPercent, double productionPercent, double researchPercent, Player owner) throws ApplicationError {
        super(planet, owner);
        this.developmentPercent = developmentPercent;
        this.ecologyPercent = ecologyPercent;
        this.productionPercent = productionPercent;
        this.researchPercent = researchPercent;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        Planet planet = getPlanet();
        if (productionPercent > 1) {
            throw new ValidationError(L.PRODUCTION_PERCENT_CANNOT_EXCEED_100);
        }
        planet.setPercentDevelopment( developmentPercent );
        planet.setPercentEcology( ecologyPercent );
        planet.setPercentProduction( productionPercent );
        planet.setPercentResearch( researchPercent );
        planet.setTargetSurplusPercent(null);
        planet.setVaryProduction(false);
        planet.setVaryResearch(false);
        return new StateUpdateCommandResult(new PlanetVOAdapter(planet, owner), EngineVoFactory.getPlayer(owner));
    }
}