/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.SalvageList;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.designs.AsteroidDesign;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.ConstructPlanetDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.ProjectDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.WeaponDesign;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.vo.CraftVO;
import mot.common.vo.FlushCraftCacheVO;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.Project;
import mot.engine.planets.ProjectCraftUpgrade;
import mot.engine.planets.ProjectFactory;
import mot.engine.planets.ProjectSpaceStation;
import mot.engine.players.Player;

/**
 * This is used for a user's planets and what its next project should be.
 **/
public class SetProjectCmd extends Command {

    /**
     * This is the craft/fleet to which this command is being issued.
     */
    private Planet [] planets;


	/**
     * The project this planet should build.
     */
    private ProjectDesign project;

    /**
     * The quantity of times this planet should repeat this project.
     */
    private int repeat;
    
    /**
     * If true this project is to be set on all planets with no project.
     * If false this project should be set on this planet only.
     */
    private boolean allIdle;


	private boolean useSalvage;

    public SetProjectCmd(Planet planet, ProjectDesign project, int repeat, Player owner, boolean allIdle, boolean useSalvage) throws ApplicationError {
    	super(owner);
        this.planets = new Planet[1];
        this.planets[0] = planet;
        this.project = project;
        this.repeat = repeat;
        this.allIdle = allIdle;
        this.useSalvage = useSalvage;
	}

    public SetProjectCmd(Planet [] planets, ProjectDesign project, int repeat, Player owner, boolean allIdle, boolean useSalvage) throws ApplicationError {
    	super(owner);
        this.planets = planets;
        this.project = project;
        this.repeat = repeat;
        this.allIdle = allIdle;
		this.useSalvage = useSalvage;
	}

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
		Planet [] planets = getPlanets();
		StateUpdateCommandResult r = new StateUpdateCommandResult();
		for (int i = 0; i < planets.length; i++) {
			r = execute(planets[i], project, allIdle, r);
		}
		return r;
	}
	
    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	private StateUpdateCommandResult execute(Planet planet, ProjectDesign project, boolean setOnAllIdle, StateUpdateCommandResult r)  throws ApplicationError, ValidationError {
        if (owner != planet.getOwner()) {
            throw new ValidationError(L.YOU_CANNOT_SET_THE_PROJECT_ON_A_PLANET_THAT_DOES_NOT_BELONG_TO_YOU);
        }
        if (project instanceof ProjectSpaceStation) {
            if (planet.getMiningStation() != null) {
                throw new ValidationError(L.THIS_PLANET_ALREADY_HAS_A_MINING_STATION);
            }
        } else if (project instanceof ConstructPlanetDesign) {
        	if (owner.getBoralium() < ConstructPlanetDesign.BORALIUM_COST) {
                throw new ValidationError(L.YOU_DO_NOT_HAVE_ENOUGH_BORALIUM_CRYSTALS_100_REQUIRED);        		
        	}
    		if (planet.getSolarSystem().getQtyPlanets() == GalaxyDefinitions.MAX_PLANETS_PER_SOLAR_SYSTEM) {
                throw new ValidationError(L.YOU_CANNOT_BUILD_MORE_PLANETS_IN_THIS_SOLAR_SYSTEM);        		
    		}
    		owner.addBoralium(-ConstructPlanetDesign.BORALIUM_COST);
        } else if (project instanceof CraftDesign) {
        	CraftDesign cd = (CraftDesign) project;
        	SalvageList salvage = useSalvage ? owner.getSalvage() : null;
        	int qty = 1 + repeat;
        	_checkQuantity(salvage, cd, qty);
        } else if (project instanceof CraftUpgradeDesign) {
        	CraftUpgradeDesign cud = (CraftUpgradeDesign) project;
        	SalvageList salvage = useSalvage ? owner.getSalvage() : null;
        	Project existing = planet.getProject();
        	int qtyCrafts = cud.getQtyCrafts();
        	if (qtyCrafts == 0) {
        		throw new ValidationError(L.INVALID_COMMAND_PLEASE_SELECT_A_CRAFT_FIRST);
        	}
			if (repeat > 0 || setOnAllIdle) {
				throw new ValidationError(L.YOU_CANNOTBUILD_THIS_HERE);
			}
			_checkQuantity(salvage, cud.getDesign(), qtyCrafts);
			if (planet.getProject() != null && planet.getProject() instanceof ProjectCraftUpgrade) {
				CraftUpgradeDesign existingCUP = ((ProjectCraftUpgrade)planet.getProject()).getCraftDesign();
				Hull hull = cud.getDesign().getHullTech();
				Hull existingHull = existingCUP.getDesign().getHullTech();
				if (existingHull != hull) {
					throw new ValidationError(L.YOU_CANNOTBUILD_THIS_HERE);
				}
			}
			_checkHullsMatch(cud.getDesign().getHullTech(), cud.getCrafts());
		} else if (project instanceof ShieldDesign) {
			ShieldDesign d = (ShieldDesign) project;
			Shields shields = planet.getShields();
			if (shields != null && (shields.containsGroup(d.getShield().getGroup()) || shields.contains(d.getShield()))) {
				throw new ValidationError(L.YOU_CANNOTBUILD_THIS_HERE);
			}
        } else if (project instanceof ScannerDesign) {
			ScannerDesign d = (ScannerDesign) project;
			Scanners scanners = planet.getScanners();
			if (scanners != null && (scanners.containsGroup(d.getScanner().getGroup()) || scanners.contains(d.getScanner()))) {
				throw new ValidationError(L.YOU_CANNOTBUILD_THIS_HERE);
			}
		} else if (project instanceof WeaponDesign) {
			WeaponDesign d = (WeaponDesign) project;
			Weapons planetWeapons = planet.getWeapons();
			if (planetWeapons != null) {
				Weapon bestOfGroup = planet.getWeapons().getMaxDamageWeaponWithGroup(d.getWeapon().getGroup());
				if (bestOfGroup != null && bestOfGroup.getDamage() > d.getWeapon().getDamage()) {
					throw new ValidationError(L.YOU_CANNOTBUILD_THIS_HERE);
				}
				int qty = planetWeapons.getQuantityWithGroup(d.getWeapon().getGroup());
				int max = d.getWeapon().getMaxQtyOnPlanet();
				if (qty + 1 + repeat > max) {
					throw new ValidationError(L.MAX_QTY_EXCEEDED);
				}
			}
		} else if (project instanceof BattleStationDesign) {
			BattleStationDesign d = (BattleStationDesign) project;
			if (planet.getOrbital() != null) {
				throw new ValidationError(L.YOU_CANNOTBUILD_THIS_HERE);
			}
		} else if (project instanceof AsteroidDesign) {
			AsteroidDesign d = (AsteroidDesign) project;
			if (planet.getHasAsteroids()) {
				throw new ValidationError(L.YOU_CANNOTBUILD_THIS_HERE);
			}
		}
        
        Project p = ProjectFactory.getProject(project, planet, useSalvage);
        planet.setProject(p);
       	planet.setRepeatProject(repeat);
       	
       	r.addStateUpdate(EngineVoFactory.getPlayer(owner)); // may have used salvage
       	
       	if (Logger.isDebug()) {
       		Logger.logDebug("SetProjectCmd.run() assign "+p+" to "+planet);
       	}
        
        if (setOnAllIdle) {
        	setProjectOnAllIdlePlanets(r);
        	r.addStateUpdate(EngineVoFactory.getPlanets(owner.getPlanets(), owner));
        } else {        	
        	r.addStateUpdate(new PlanetVOAdapter(planet, owner));
        }
        
        if (project instanceof CraftUpgradeDesign) {
        	CraftUpgradeDesign cud = (CraftUpgradeDesign) project;
			r.addStateUpdate(new FlushCraftCacheVO());
			r.addStateUpdate(EngineVoFactory.getMoos(MotEngine.instance.getMoveableOwnedObjects().getMOOs(owner, true, true), owner));
        }
        return r;
    }


	private void _checkHullsMatch(Hull hull, CraftVO[] crafts) throws ApplicationError, ValidationError {
		if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {
				if (hull != crafts[i].getHull()) {
					throw new ValidationError(L.INVALID_COMMAND_PLEASE_SELECT_A_CRAFT_FIRST);
				}
			}
		}
	}

	private void setProjectOnAllIdlePlanets(StateUpdateCommandResult r) throws ApplicationError {
		if (owner == null) return;
		Planet[] planets = owner.getPlanets();
		if (planets == null) return;
		for (int i = 0; i < planets.length; i++) {
			Planet p = planets[i];
			// Dont set a project on a mining station
			if (p.getPopulation() > 0 && p.getProject() == null) {
				try {
					ProjectDesign newProject = project;
					if (project instanceof ShieldDesign) {
						ShieldDesign d =(ShieldDesign)project;
						if (p.getShields() != null) {
							newProject = null;
						}
					} else if (project instanceof ScannerDesign) {
						ScannerDesign d =(ScannerDesign)project;
						if (p.getScanners() != null) {
							newProject = null;
						}
					} else if (project instanceof BattleStationDesign) {
						BattleStationDesign d =(BattleStationDesign)project;
						if (p.getOrbital() != null) {
							newProject = null;
						}
					} else if (project instanceof WeaponDesign) {
						WeaponDesign d =(WeaponDesign)project;
						if (p.getWeapons() != null) {
							newProject = null;
						}
					}
					if (newProject != null) {
						execute(p, newProject, false, r);
					}
				} catch (ApplicationError e) {
					Logger.logError("Unable to set nested project", e);
				} catch (ValidationError e) {
					// No project will be set on this planet because we can't
					//Logger.logError("", e);
				}
			}

		}
	}

	private void _checkQuantity(SalvageList salvage, CraftDesign cd, int qtyCrafts) throws ApplicationError, ValidationError {
    	_checkQuantity(salvage, cd.getHullTech(), qtyCrafts);
    	_checkQuantity(salvage, cd.getEngineTech(), qtyCrafts);
    	_checkQuantity(salvage, cd.getDistinctWeapons(), qtyCrafts);
    	if (cd.getShields() != null) _checkQuantity(salvage, cd.getShieldTechs().getDistinct(), qtyCrafts);
    	if (cd.getComputers() != null) _checkQuantity(salvage, cd.getComputerTechs().getDistinct(), qtyCrafts);
    	if (cd.getScanners() != null) _checkQuantity(salvage, cd.getScannerTechs().getDistinct(), qtyCrafts);
    	if (cd.getSpecials() != null) _checkQuantity(salvage, cd.getSpecialTechs().getDistinct(), qtyCrafts);
	}
	
	private void _checkQuantity(SalvageList salvage, Technology tech, int qtyCrafts) throws ApplicationError, ValidationError {
		if (tech == null) return;
		if (!owner.hasDiscovered(tech)) {
			if (salvage == null) {
		        _throwValidationError(tech);
			}				
			int qtyInSalvage = salvage.getQuantity(tech);
			if (qtyInSalvage < qtyCrafts) {
		        _throwValidationError(tech);
			}
		}
	}

	private void _checkQuantity(SalvageList salvage, TechnologyBundle[] a, int qtyCrafts) throws ValidationError, ApplicationError {
		if (a == null) return;
		for (int i = 0; i < a.length; i++) {
			if (!owner.hasDiscovered(a[i].getTechnology())) {
				if (salvage == null) {
			        _throwValidationError(a[i].getTechnology());
				}				
				int qtyInSalvage = salvage.getQuantity(a[i].technology);
				if (qtyInSalvage < a[i].getQty() * qtyCrafts) {
			        _throwValidationError(a[i].getTechnology());
				}				
			}
		}
	}

	private void _throwValidationError(Technology a) throws ValidationError {
		throw new ValidationError(L.YOU_HAVE_NOT_DISCOVERED_+a.getFullNameSpaceSeperated()+L._OR_HAVE_ENOUGH_IN_SALVAGE);
	}

    public void setPlanets(Planet [] planets) {
        this.planets = planets;
    }

    public Planet [] getPlanets() {
        return planets;
    }

    public void setProject(ProjectDesign project) {
        this.project = project;
    }


    public ProjectDesign getProject() {
        return project;
    }


    public void setRepeat(int repeat) {
        this.repeat = repeat;
    }


    public int getRepeat() {
        return repeat;
    }


}