/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.adapter.PlayerDetailsVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.spies.Spies;
import mot.engine.spies.SpyRole;
import mot.engine.spies.SpyRoleEspionage;

/**
 * Moves a spy from the defend pool belonging to player to an espionage spy role 
 * owned by player and targeting player other..
 */
public class SpyRoleAddEspionageCmd extends SpyRoleCmd {

    public SpyRoleAddEspionageCmd(Player player, Player target, int qty) {
        super(player, target, qty);
    }
    
    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        UserPlayer owner = (UserPlayer) getOwner();
        
        if (target == null || owner == null) {
            throw new ValidationError("Must specify two players - Spy owner and spy taget");
        }
        
        if (qty  == 0) {
            throw new ValidationError("Must specify a quantity of spies");
        }
        
        Spies spies = MotEngine.instance.getSpiesNotNull();
        if (qty > 0) {
            int qtyDefending = spies.getQtySpiesDefending(owner);
            if (qty > qtyDefending) {
                throw new ValidationError("You do not have this many spies to assign");
            }
        } else {
            int qtyInRole = spies.getQtySpiesEspionage(owner, target);
            if ((-qty) > qtyInRole) {
                throw new ValidationError("You do not have this many spies to remove");
            }
        }
        
        SpyRole [] roles = spies.getSpies(owner, target);
        int remainingQty = qty;
        for (int i=0; roles != null && i<roles.length; i++) {
            if (roles[i] instanceof SpyRoleEspionage) {
                SpyRoleEspionage sre = (SpyRoleEspionage) roles[i];
                remainingQty = sre.addQtySpies(qty);
                if (sre.getQtySpies() == 0) {
                    spies.removeSpyRole(sre);
                }
            }
        }
        
        if (remainingQty > 0) {
            spies.addSpyRole(new SpyRoleEspionage(owner, target, remainingQty));
        }
        if (qty > 0) {
            SpyRole defenders = spies.getDefendingSpies(owner);
            defenders.addQtySpies(-qty);
            if (defenders.getQtySpies() == 0) {
                spies.removeSpyRole(defenders);
            }
        }
        if (qty < 0) {
            spies.addDefendingSpies(owner, -qty);
        }
        return new StateUpdateCommandResult(EngineVoFactory.getPlayer(owner), new PlayerDetailsVOAdapter(owner));
	}
}