/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;
import mot.common.command_results.CommandResult;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.adapter.PlayerDetailsVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.spies.Spies;
import mot.engine.spies.SpyCenter;
import mot.engine.spies.SpyRole;

/**
 * Allocates some spies from owner to spy on the target player. This will allow the owner
 * to view the espionage report details for the target player.
 */
public class SpyRoleAddPlayerCmd extends Command {

	private Player target;

    public SpyRoleAddPlayerCmd(Player owner, Player target) {
        super(owner);
        this.target = target;
    }

    /**
     * This is the quantity of spies this will cost.
     */
    private static final int COST = 3;

    /**
     * Each command in the system must implement this method. Its implementation
     * must execute the command.
     */
	protected CommandResult execute()  throws ApplicationError, ValidationError {
        UserPlayer owner = (UserPlayer) getOwner();

        if (target == null || owner == null) {
            throw new ValidationError("Must specify two players - Spy owner and spy taget");
        }

        Spies spies = MotEngine.instance.getSpiesNotNull();
        int qtyDefending = spies.getQtySpiesDefending(owner);
        
        if (COST > qtyDefending) {
            throw new ValidationError(L.YOU_DO_NOT_HAVE_THIS_MANY_SPIES_TO_ASSIGN);
        }

        SpyCenter sc = owner.getSpyCenter();
        if (sc == null) {
            sc = new SpyCenter();
        }
        
        if (sc.canViewDetailsOf(target)) {
            throw new ValidationError(L.YOU_HAVE_ALREADY_ASSIGNED_SPIES_TO_THIS_PLAYER);
        }
        
        sc.addPlayerSpy(target);

        SpyRole defenders = spies.getDefendingSpies(owner);
        defenders.addQtySpies(-COST);
        if (defenders.getQtySpies() == 0) {
            spies.removeSpyRole(defenders);
        }
        
        return new StateUpdateCommandResult(EngineVoFactory.getPlayer(owner), new PlayerDetailsVOAdapter(owner));
	}
}