/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command;

import mot.common.command_results.CommandResult;
import mot.common.command_results.NameListCR;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.Orbit;
import mot.engine.event.MessageEF;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRouteFactory;
import mot.engine.players.Player;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportManager;
import mot.engine.transports.TransportTask;
import mot.engine.transports.TransportTaskCreateTradeRoute;
import mot.engine.transports.TransportTaskInvade;
import mot.engine.transports.TransportTaskMoveColonists;

public class TransportLoadCmd extends Command {

	private Orbit source;
    private Orbit destination;
    private int qty;
	private int type;

    public TransportLoadCmd(Orbit source, Orbit destination, int qty, Player owner, int type) {
        super(owner);
        this.source = source;
        this.destination = destination;
        this.qty = qty;
		this.type = type;
    }

    protected CommandResult execute() throws ApplicationError, ValidationError {
        TransportManager tm = owner.getTransportManagerNotNull();
        
        Planet sourcePlanet = source.getPlanet();
        Planet destinationPlanet = destination == null ? null : destination.getPlanet();
        
        Transport transport = null;
        if (type == 0|| type == 1) {
        	if (!sourcePlanet.isOwnedBy(owner)) {
        		throw new ValidationError(L.ORIGIN_PLANET_DOES_NOT_BELONG_TO_YOU);
        	}
        	if (qty > 0 && sourcePlanet.getPopulation() <= (qty + 5)) {
        		throw new ValidationError(L.ORIGIN_PLANET_DOES_NOT_HAVE_ENOUGH_POPULATION_TO_TRANSFER_+qty+L._PEOPLE);
        	}
        	transport = tm.getMoveColonistsTransportWithSpareCapacityAt(sourcePlanet, type == 1);
        } else {
            if (!source.isOwnedBy(owner)) {
                throw new ValidationError(L.ORIGIN_PLANET_DOES_NOT_BELONG_TO_YOU);
            }
        }
        
        if (transport == null) {
        	transport = tm.getFreeTransport();
        }
        if (transport == null) {
            throw new ValidationError(L.YOU_DO_NOT_HAVE_ANY_TRANSPORTS_AVAILABLE);
        }
        
        if (destination != null && !destination.isOwnedBy(owner)) {
            throw new ValidationError(L.DESTINATION_PLANET_DOES_NOT_BELONG_TO_YOU);
        }
        
        NameListCR r = new NameListCR();
        if (type == 0 || type == 1) {
        	loadPeople(sourcePlanet, destinationPlanet, transport, r);
        } else if (type == 2) {
        	if (!(source.getStellarObject() instanceof SolarSystem)) {
        		throw new ValidationError("Must be a Solar System");
        	}
        	if (destination != null && !(destination.getStellarObject() instanceof SolarSystem)) {
        		throw new ValidationError("Must be a Solar System");
        	}
        	SolarSystem sourceSolarSystem = (SolarSystem) source.getStellarObject();
        	SolarSystem destinationSolarSystem = destination != null ? (SolarSystem) destination.getStellarObject() : null;
    		if (TradeRouteFactory.canCreateTradeRoute(sourceSolarSystem, destinationSolarSystem)) {
    			new TransportTaskCreateTradeRoute(transport, sourceSolarSystem, destinationSolarSystem);
    		} else {
    			throw new ValidationError(L.INVALID_COMMAND_YOU_CANNOT_CREATE_A_NEW_TRADE_ROUTE);
    		}
    		r.setName(transport.getName());
        } else {
        	throw new ValidationError("Invalid Transport cargo"+type);        	
        }
       
        if (sourcePlanet != null) r.addStateUpdate(new PlanetVOAdapter(sourcePlanet, owner));
        if (destinationPlanet != null) r.addStateUpdate(new PlanetVOAdapter(destinationPlanet, owner));
        r.addStateUpdate(EngineVoFactory.getPlayer(owner));
        Transport[] transports = tm.getTransportsInUse();
        if (transports != null) {
        	r.addStateUpdate(EngineVoFactory.getMoos(transports, owner));
        }
		return r;
    }
    
    public void loadPeople(Planet sourcePlanet, Planet destinationPlanet, Transport transport, NameListCR r) throws ValidationError, ApplicationError {
        TransportManager tm = owner.getTransportManagerNotNull();
        
        while (transport != null && qty > 0) {
            int qtyOnTransport = Comp.min(qty, transport.getSpaceAvailable());
            TransportTask task = transport.getTask();
            if (type == 1) {
            	if (qty > sourcePlanet.getQtyGroundDefense()-1) {
            		throw new ValidationError(L.THIS_PLANET_DOES_NOT_HAVE_ENOUGH_TROOPS);
            	}
    			if (task != null && task instanceof TransportTaskInvade) {
    				TransportTaskInvade tti = (TransportTaskInvade) task;
    				tti.addTroops(transport, sourcePlanet, qtyOnTransport);
    			} else if (task != null && task instanceof TransportTaskMoveColonists) {
    				throw new ValidationError(L.THIS_TRANSPORT_CONTAINS_COLONISTS);
    			} else {
    	            new TransportTaskInvade(transport, sourcePlanet, destinationPlanet, qtyOnTransport);
    			}
            } else if (type == 0) {
            	if (qty > sourcePlanet.getPopulation()-1) {
            		throw new ValidationError(L.THIS_PLANET_DOES_NOT_HAVE_ENOUGH_COLONISTS);
            	}
    			if (task != null && task instanceof TransportTaskMoveColonists) {
    				TransportTaskMoveColonists ttmc = (TransportTaskMoveColonists) task;
    				ttmc.addColonists(transport, sourcePlanet, qtyOnTransport);
    			} else if (task != null && task instanceof TransportTaskInvade) {
    				throw new ValidationError(L.THIS_TRANSPORT_CONTAINS_TROOPS);
    			} else {
    	            new TransportTaskMoveColonists(transport, sourcePlanet, destinationPlanet, qtyOnTransport);
    			}
            }
            r.setName(transport.getName());
            qty -= qtyOnTransport;
            transport = tm.getFreeTransport();
        }
        
        if (qty > 0) {
            MotEngine.instance.broadcast(new MessageEF(L.SOME_COLONISTS_HAVE_BEEN_LEFT_BEHIND_BECAUSE_YOU_DO_NOT_HAVE_ENOUGH_TRANSPORTS), owner);
        }
    }
}