/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command_handlers;

import java.io.File;

import mot.common.bean.GalaxyDetails;
import mot.common.bean.GalaxySummary;
import mot.common.bean.PlayerSummary;
import mot.common.bean.SectorOwnerGrid;
import mot.common.bean.StellarObjectSummary;
import mot.common.command_delegates.FetchGalaxyPreview;
import mot.common.command_delegates.Request;
import mot.common.command_results.CommandResult;
import mot.common.coms.protocols.ClientEventProcessor;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.common.xml.XMLDocumentHandler;
import mot.engine.MotEngine;
import mot.engine.adapter.PlayerSummaryAdapter;
import mot.engine.adapter.StellarObjectSummaryAdapter;
import mot.engine.common.StellarObject;
import mot.engine.players.Player;
import mot.engine.validators.SavedGameFileNameValidator;

/**
 * This is a command issued by a user to the engine.
 **/
public class FetchGalaxyPreviewCmdHandler extends MessageHandlerBase {

	private static MotEngine previewEngine;
	private static String previewFileName;
	private static long previewFileTime;
	
	public static MotEngine getPreviewEngine(File file) {
		if (previewEngine != null && Comp.same(previewFileName, file.getAbsolutePath())) {
			if (previewFileTime == file.lastModified()) {
				return previewEngine;
			}
		}
		return null;
	}
	
    public FetchGalaxyPreviewCmdHandler() {
	}

	public CommandResult process(Request o, String playerName, ClientEventProcessor connection) throws ApplicationError, ValidationError {
		FetchGalaxyPreview gp = (FetchGalaxyPreview) o;
		String fileName = gp.getFileName();
		File file = SavedGameFileNameValidator.validate(fileName, true);
		
		//
		// Load game and fetch preview information
		// Should I save the engine instance JUST in case it is corrupted 
		// during loading of the game (any references use e.g. PlanetReference)?
		//
		
		Player [] players;
		SectorOwnerGrid sog;
		StellarObject [] stellarObjects;
		GalaxyDetails galaxyDetails;
		PlayerSummary [] playersSummary;
		StellarObjectSummary [] stellarObjectSummaries;
		
		MotEngine originalGame = MotEngine.instance;
		try {
			MotEngine.newInstance();
			previewEngine = (MotEngine) XMLDocumentHandler.parse(file, false, null);
			players = previewEngine.getPlayersNotNull().getPlayers();
			sog = previewEngine.getInfluence().getSectorOwnerGrid();
			stellarObjects = previewEngine.getStellarObjectsNotNull().getStellarObjects();
			galaxyDetails = previewEngine.getStellarObjects().getGalaxyDetails();
			int qtyPlayers = players.length;
			playersSummary = new PlayerSummary [qtyPlayers];
			for (int i=0; i<qtyPlayers; i++) {
				playersSummary[i] = new PlayerSummaryAdapter(players[i]);
			}

			int qtyStellarObjects = stellarObjects.length;
			stellarObjectSummaries = new StellarObjectSummary [qtyStellarObjects];
			for (int i=0; i<qtyStellarObjects; i++) {
				stellarObjectSummaries[i] = new StellarObjectSummaryAdapter(stellarObjects[i]);
			}
			previewFileName = file.getAbsolutePath();
			previewFileTime = file.lastModified();
		} catch (Exception e) {
			MotEngine.restoreInstance(originalGame);
            throw new ValidationError("File "+fileName+" is not a valid game");
		}
		MotEngine.restoreInstance(originalGame);
		
		//
		// Send information back to client
		//

		return new GalaxySummary(playersSummary, sog, stellarObjectSummaries, galaxyDetails);
	}

}