/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command_handlers;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.common.vo.OrbitVO;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.ServerImageObject;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftBase;
import mot.engine.crafts.Crafts;
import mot.engine.crafts.Fleet;
import mot.engine.planets.Planet;
import mot.engine.planets.Planets;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.StellarObjects;
import mot.engine.players.Player;
import mot.engine.players.Players;
import mot.engine.players.UserPlayer;
import mot.engine.transports.Transport;

public abstract class MessageHandlerBase implements MessageHandler {

    protected Battle getBattle(String name) throws ApplicationError {
        Battle battle = MotEngine.instance.getBattlesNotNull().getBattle(name);
        return battle;
    }

    protected Planet [] getPlanets(String [] names) throws ApplicationError, ValidationError {
        if (names == null) {
            return null;
        }
        Planet [] r = new Planet [names.length];
        for (int i=0; i<names.length; i++) {
            r[i] = getPlanet(names[i]);
        }
        return r;
    }


    protected Craft [] getCrafts(String [] names) throws ApplicationError, ValidationError {
        if (names == null) {
            return null;
        }
        Craft [] r = new Craft [names.length];
        for (int i=0; i<names.length; i++) {
            r[i] = getCraft(names[i]);
        }
        return r;
    }

    protected Player [] getPlayers(String [] names) throws ApplicationError {
        if (names == null) {
            return null;
        }
        Player [] r = new Player [names.length];
        for (int i=0; i<names.length; i++) {
            r[i] = getPlayer(names[i]);
        }
        return r;
    }

    protected Player getPlayer(String name) throws ApplicationError {
    	if (name  == null) return null;
        Players players = MotEngine.instance.getPlayers();
        if (players == null) {
            throw new ApplicationError("MessageHandlerBase.getPlayer() No players exist");
        }
        Player player = players.getPlayer(name);
        if (player == null) {
            Players deadPlayers = MotEngine.instance.getDeadPlayers();
            if (deadPlayers != null) {
            	player = deadPlayers.getPlayer(name);
            }
            if (player == null) {
            	throw new ApplicationError("MessageHandlerBase.getPlayer() Player "+name+" does not exist");
            }
        }
        return player;
    }

    protected UserPlayer getUserPlayer(String name) throws ApplicationError {
    	if (name  == null) return null;
        Player player = getPlayer(name);
        if (player == null) {
            Players deadPlayers = MotEngine.instance.getDeadPlayers();
            if (deadPlayers != null) {
            	player = deadPlayers.getPlayer(name);
            }
        }
        if (player instanceof UserPlayer) {
            return (UserPlayer) player;
        } else {
            throw new ApplicationError("MessageHandlerBase.getPlayer() Player "+name+" is not a user player");
        }
    }

    protected CraftBase getCraftOrFleet(String name) throws ApplicationError, ValidationError {
    	if (name  == null) return null;
    	MoveableOwnedObject moo = getMoveableOwnedObject(name);
    	if (moo instanceof CraftBase) {
    		return (CraftBase) moo;
    	} else {
    		throw new ValidationError("Invalid Command for "+name);
    	}
    }
    
    protected Craft getCraft(String name) throws ApplicationError, ValidationError {
    	if (name  == null) return null;
    	MoveableOwnedObject moo = getMoveableOwnedObject(name);
    	if (moo instanceof Craft) {
    		return (Craft) moo;
    	} else {
    		throw new ValidationError("Invalid Command for "+name);
    	}
    }

    protected Fleet getFleet(String name) throws ApplicationError, ValidationError {
    	if (name  == null) return null;
    	MoveableOwnedObject moo = getMoveableOwnedObject(name);
    	if (moo instanceof Fleet) {
    		return (Fleet) moo;
    	} else {
    		throw new ValidationError("Invalid Command for "+name);
    	}
    }

    protected MoveableOwnedObject getMoveableOwnedObject(String name) throws ApplicationError, ValidationError {
    	if (name  == null) return null;
        Crafts crafts = MotEngine.instance.getCrafts();
        if (crafts != null) {
	        Craft craft = crafts.getCraft(name);
	        if (craft != null) {
	            return craft;
	        }
        }
        Fleet fleet = MotEngine.instance.getPlayers().getFleet(name);
        if (fleet != null) {
            return fleet;
        }
        Transport transport = MotEngine.instance.getPlayers().getTransport(name);
        if (transport != null) {
        	return transport;
        }
        throw new ValidationError(name+" does not exist");
    }
    
    public static Planet getPlanet(String name) throws ApplicationError {
    	if (name  == null) return null;
        Planets planets = MotEngine.instance.getPlanets();
        if (planets == null) {
            throw new ApplicationError("MessageHandlerBase.getPlanet() No planets exist");
        }
        Planet planet =planets.getPlanet(name);
        if (planet == null) {
            throw new ApplicationError("MessageHandlerBase.getPlanet() Planet "+name+" does not exist");
        }
        return planet;
    }

    public static StellarObject getStellarObject(String name) throws ApplicationError {
    	if (name  == null) return null;
        StellarObjects sos = MotEngine.instance.getStellarObjects();
        if (sos == null) {
            throw new ApplicationError("MessageHandlerBase.StellarObject() No stellar objects exist");
        }
        StellarObject so =sos.getStellarObject(name);
        if (so == null) {
            throw new ApplicationError("MessageHandlerBase.StellarObject() Stellar Object "+name+" does not exist");
        }
        return so;
    }

    protected SolarSystem getSolarSystem(String name) throws ApplicationError {
    	if (name  == null) return null;
        StellarObject so = getStellarObject(name);
        if (so instanceof SolarSystem) {
            return (SolarSystem) so;
        } else {
            throw new ApplicationError("MessageHandlerBase.StellarObject() Stellar Object "+name+" is not a SolarSystemt");
        }
    }
    
    public  Orbit getOrbit(OrbitVO orbit) throws ApplicationError {
    	if (orbit  == null) return null;
    	String planetName = orbit.getPlanet();
		if (planetName != null) {
			return new Orbit(getPlanet(planetName));
		} else {
			String stellarObjectName = orbit.getStellarObject();
			if (stellarObjectName != null) {
				return new Orbit(getStellarObject(stellarObjectName));
			}
        }
        return null;
    }


    protected BattlePlanet getBattlePlanet(String battleName, String planetName) throws ApplicationError {
    	if (planetName  == null) return null;
        Battle battle = getBattle(battleName);
    	if (battle  == null) return null;
		BattlePlanet planet = battle.getPlanet();
        if (planet == null) {
            throw new ApplicationError("MessageHandlerBase.getBattleCraft() No planet is in this battle");
        }
        if (!Comp.same(planetName, planet.getName())) {
            throw new ApplicationError("MessageHandlerBase.getBattleCraft() BattlePlanet "+planetName+" does not exist");
        }
        return planet;
    }

    protected BattleCraft getBattleCraft(String battleName, String craftName) throws ApplicationError {
    	if (craftName  == null) return null;
        Battle battle = getBattle(battleName);
    	if (battle  == null) return null;
        BattleCraft craft = battle.getCraft(craftName);
        if (craft == null) {
            throw new ApplicationError("MessageHandlerBase.getBattleCraft() BattleCraft "+craftName+" does not exist");
        }
        return craft;
    }

    protected BattleObject getBattleObject(String battleName, String boName) throws ApplicationError {
        BattleObject bo = getBattleObjectNoError(battleName, boName);
        if (bo == null) {
            throw new ApplicationError("MessageHandlerBase.getBattleCraft() BattleObject "+boName+" does not exist");
        }
        return bo;
    }

    protected BattleObject getBattleObjectNoError(String battleName, String boName) throws ApplicationError {
    	if (boName  == null) return null;
        Battle battle = getBattle(battleName);
    	if (battle  == null) return null;
        BattleObject bo = battle.getBattleObject(boName);
        return bo;
    }

    protected ServerImageObject getLocatedAreaObject(String battleName, String imageObjectName) throws ApplicationError {
    	if (imageObjectName  == null) return null;
        Battle battle = getBattle(battleName);
    	if (battle  == null) return null;
        BattleObject bo = battle.getBattleObject(imageObjectName);
        if (bo == null) {
            throw new ApplicationError("MessageHandlerBase.getImageObject() ImageObject "+imageObjectName+" does not exist");
        }
        return bo;
    }


    protected ServerImageObject getImageObject(String battleName, String imageObjectName) throws ApplicationError {
    	if (imageObjectName  == null) return null;
        Battle battle = getBattle(battleName);
    	if (battle  == null) return null;
        BattleObject bo = battle.getBattleObject(imageObjectName);
        if (bo == null) {
            throw new ApplicationError("MessageHandlerBase.getImageObject() ImageObject "+imageObjectName+" does not exist");
        }
        return bo;
    }
    
    protected ServerImageObject getImageObject(Battle battle, String imageObjectName) throws ApplicationError {
        BattleObject bo = battle.getBattleObject(imageObjectName);
        if (bo == null) {
            throw new ApplicationError("MessageHandlerBase.getImageObject() ImageObject "+imageObjectName+" does not exist");
        }
        return bo;
    }

}