/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.command_handlers;

import mot.common.command_delegates.Request;
import mot.common.command_delegates.SpyStealableTechnologiesCmdDelegate;
import mot.common.command_results.CommandResult;
import mot.common.command_results.TechnologyListCR;
import mot.common.coms.protocols.ClientEventProcessor;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * This is a command issued by a user to the engine.
 **/
public class SpyStealableTechnologiesCmdHandler extends MessageHandlerBase {
	
    public CommandResult process(Request o, String playerName, ClientEventProcessor connection) throws ApplicationError, ValidationError {
        UserPlayer owner = MotEngine.instance.getPlayersNotNull().getUserPlayer(playerName);
    	SpyStealableTechnologiesCmdDelegate p = (SpyStealableTechnologiesCmdDelegate)o;
    	String takerPlayer = p.getTakerPlayer();
    	if (takerPlayer == null) {
    		throw new ValidationError("Must specify source player");
    	}
    	Player taker = MotEngine.instance.getPlayersNotNull().getPlayer(takerPlayer);
    	if (taker == null) {
    		throw new ValidationError("Must specify source player");
    	}
    	String giverPlayer = p.getGiverPlayer();
    	if (giverPlayer == null) {
    		throw new ValidationError("Must specify target player");
    	}
    	Player giver = MotEngine.instance.getPlayersNotNull().getPlayer(giverPlayer);
    	if (giver == null) {
    		throw new ValidationError("Must specify target player");
    	}
    	TechnologyList techs = giver.getStealableTechnologies(taker);
    	techs.remove(p.getExclude());
    	if (techs.getQuantity() == 0) {
    		throw new ValidationError("We will not discuss our technologies with you");
    	}
    	if ((owner == taker && owner.isViewableBy(giver)) || (owner == giver && owner.isViewableBy(taker))) {
   			return new TechnologyListCR(techs);
    	} else {
    		int currentDate = MotEngine.instance.getDate();
    		if (techs != null && techs.getQuantity() > 0) {
    	    	String key = giverPlayer+takerPlayer;
				FetchStealableTechnologiesHistory history = MotEngine.instance.fetchHistory.get(key);
				removeDiscoveredTechnologies(taker, history);
    			if (history == null) {
    				history = new FetchStealableTechnologiesHistory();
    				MotEngine.instance.fetchHistory.put(key, history);
    				Technology tech = techs.getRandom();
    				history.technologies = new GeneralTechnologyList(tech);
    				history.date = currentDate;
    				return new TechnologyListCR(tech);
    			} else if ((double)history.technologies.getQuantity() < techs.getQuantity()/4.0) {
    				techs.remove(history.technologies);
    				Technology tech = techs.getRandom();
    				history.technologies.add(tech);
    				return new TechnologyListCR(tech);
    			} else {
    				Integer lastRequestDate = history.date;
    				if (lastRequestDate != null && currentDate > lastRequestDate + 2) {
    					Technology tech = techs.getRandom();
    					history.technologies.removeAll();
    					history.technologies.add(tech);
    					history.date = currentDate;
    					return new TechnologyListCR(tech);            		
    				} else {
    					GeneralTechnologyList offeredHistory = new GeneralTechnologyList(history.technologies);
    					offeredHistory.remove(p.getExclude());
    					Technology tech = offeredHistory.getRandom();
    					if (tech == null) {
    						throw new ValidationError("Maximum technology suggestion has already been offered!");
    					} else {
    						return new TechnologyListCR(tech);
    					}
    				}
    			}
    		} else {
    			return null;
    		}
    	}
    }

	private void removeDiscoveredTechnologies(Player taker, FetchStealableTechnologiesHistory history) throws ApplicationError {
		if (history == null) return;
		Technology[] techs = history.technologies.getAllAsArray();
		if (techs == null) return;
		for (int i = 0; i < techs.length; i++) {
			if (taker.hasDiscovered(techs[i])) history.technologies.remove(techs[i]);
		}
	}
}