/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.common;

import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.mot.MotCraftDelegate;
import mot.client.deligates.mot.MotFleetDelegate;
import mot.client.deligates.mot.MotTransportDelegate;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.PlayerAssets;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.event.StateContainer;
import mot.common.technology.GeneralTechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.vo.BattleAutomatonVO;
import mot.common.vo.BattleCraftVO;
import mot.common.vo.BattleObjectVO;
import mot.common.vo.BattleObjectWeaponStateUpdateVO;
import mot.common.vo.CraftDesignVO;
import mot.common.vo.CraftDesignsVO;
import mot.common.vo.CraftVO;
import mot.common.vo.DeadPlayerVO;
import mot.common.vo.FleetVO;
import mot.common.vo.FlushCraftCacheVO;
import mot.common.vo.ImageObjectVO;
import mot.common.vo.MoveableOwnedObjectVO;
import mot.common.vo.NavigatorVO;
import mot.common.vo.OutpostVO;
import mot.common.vo.PlanetProjectVO;
import mot.common.vo.PlanetVO;
import mot.common.vo.PlayerConstantsVO;
import mot.common.vo.PlayerTechnologiesVO;
import mot.common.vo.PlayerVO;
import mot.common.vo.RelationVO;
import mot.common.vo.ResearchProjectVO;
import mot.common.vo.SpaceStationVO;
import mot.common.vo.StellarObjectOrbitalVO;
import mot.common.vo.StellarObjectVO;
import mot.common.vo.TechnologyAvailableForResearchVO;
import mot.common.vo.TradeRouteInvalidVO;
import mot.common.vo.TradeRouteListVO;
import mot.common.vo.TradeRouteVO;
import mot.common.vo.TransportVO;
import mot.common.vo.UserPlayerVO;
import mot.common.vo.ValueObjectList;
import mot.common.vo.VoFactory;
import mot.common.vo.WeaponInstanceVO;
import mot.engine.MotEngine;
import mot.engine.adapter.AutomitonVOAdapter;
import mot.engine.adapter.BattleCraftVOAdapter;
import mot.engine.adapter.BattleMineVOAdapter;
import mot.engine.adapter.BattleObjectVOAdapter;
import mot.engine.adapter.BattleObjectWeaponStateVOAdapter;
import mot.engine.adapter.BattlePlanetVOAdapter;
import mot.engine.adapter.BattleStationVOAdapter;
import mot.engine.adapter.BlackHoleVOAdapter;
import mot.engine.adapter.ComputerPlayerVOAdapter;
import mot.engine.adapter.CraftVOAdapter;
import mot.engine.adapter.DroidMineVOAdapter;
import mot.engine.adapter.FleetVOAdapter;
import mot.engine.adapter.ImageObjectVOAdapter;
import mot.engine.adapter.MissilesVOAdapter;
import mot.engine.adapter.OrbitVOAdapter;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.adapter.PlayerConstantsVOAdapter;
import mot.engine.adapter.PlayerDetailsVOAdapter;
import mot.engine.adapter.RelationVOAdapter;
import mot.engine.adapter.ResearchProjectVOAdapter;
import mot.engine.adapter.SolarSystemVOAdapter;
import mot.engine.adapter.TradeRouteVOAdapter;
import mot.engine.adapter.TransportVOAdapter;
import mot.engine.adapter.UserPlayerVOAdapter;
import mot.engine.adapter.WeaponInstanceVOAdapter;
import mot.engine.battles.Automaton;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleMine;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.BattlePrimaryObject;
import mot.engine.battles.BattleStation;
import mot.engine.battles.DroidMine;
import mot.engine.battles.Missiles;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.Navigator;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.Project;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRoute;
import mot.engine.planets.TradeRoutes;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.ResearchProject;
import mot.engine.players.UserPlayer;
import mot.engine.relations.Relation;
import mot.engine.technology.WeaponInstance;
import mot.engine.transports.Transport;

public class EngineVoFactory extends VoFactory {

    public static final ImageObjectVO getImageObject(ServerImageObject  a, Player viewer) throws ApplicationError {
        if (a == null) return null;
        if (a instanceof BattleObject) {
            return getBattleObject((BattleObject)a, viewer);   
        } else {
            return new ImageObjectVOAdapter(a, viewer);
        }

    }

    public static final BattleObjectVO getBattleObject(BattleObject  a, Player viewer) throws ApplicationError {
        if (a == null) return null;
        if (a instanceof BattleStation) {
        	BattleStation bs = (BattleStation)a;
            return new BattleStationVOAdapter(bs, viewer, bs.getOrbital() != null);   
        } else if (a instanceof BattleCraft) {
            return getBattleCraft((BattleCraft)a, viewer);   
        } else if (a instanceof BattlePlanet) {
            return new BattlePlanetVOAdapter((BattlePlanet)a, viewer);   
        } else if (a instanceof Automaton) {
            return getBattleAutomiton((Automaton)a, viewer);   
        } else {
            return new BattleObjectVOAdapter(a, viewer);
        }

    }

    public static final BattleObjectVO getBattlePrimaryObject(BattlePrimaryObject  a, Player viewer) throws ApplicationError {
        if (a == null) return null;
        if (a instanceof BattleCraft) {
        	return getBattleCraft((BattleCraft)a, viewer);
        } else if (a instanceof BattlePlanet) {
        	return new BattlePlanetVOAdapter((BattlePlanet)a, viewer);
        } else {
        	throw new ApplicationError(""+a.getClass().getName());
        }
    }
    
    public static final BattleCraftVO getBattleCraft(BattleCraft a) throws ApplicationError {
        return getBattleCraft(a, null);
    }    
    
    public static final BattleCraftVO getBattleCraft(BattleCraft a, Player viewer) throws ApplicationError {
        if (a == null) return null;
        if (a instanceof BattleStation) {
        	BattleStation bs = (BattleStation)a;
            return new BattleStationVOAdapter(bs, viewer, bs.getOrbital() != null);   
        }
        return new BattleCraftVOAdapter(a, viewer);
    }

    public static final PlanetProjectVO getPlanetProject(Planet p, Project a) throws ApplicationError {
        if (a == null) {
            return null;
        }
        int costToBuildOnPlanet = a.getCostToBuildOnPlanet();
		return new PlanetProjectVO(a.getProjectDesign(), costToBuildOnPlanet, p.getTurnsToCompleteProject(costToBuildOnPlanet), p.getCostToPurchaseProject());
    }

    public static final PlayerVO getPlayer(Player a) throws ApplicationError {
        if (a == null) return null;
        if (a instanceof UserPlayer) {
            return new UserPlayerVOAdapter((UserPlayer)a);
        } else if (a instanceof ComputerPlayer) {
            return new ComputerPlayerVOAdapter((ComputerPlayer)a);
        } else {
            return new PlayerVO(a.getName());
        }
    }


    public static final PlayerTechnologiesVO getPlayerTechs(Player a) throws ApplicationError {
        if (a == null) return null;
        return new PlayerTechnologiesVO(a.getName(), a);
    }

    public static final ValueObjectList getPlayerAssetsUpdate(DiplomaticProposal proposal, Player viewer, Player other) throws ApplicationError {
        PlayerAssets offeredAssets = proposal.getOffered();
        PlayerAssets wantedAssets = proposal.getWanted();
    	ValueObjectList r = new ValueObjectList();
		r.addStateUpdate(EngineVoFactory.getPlayer(viewer));
    	r.addStateUpdate(new PlayerDetailsVOAdapter(viewer));
    	r.addStateUpdate(EngineVoFactory.getRelation(MotEngine.instance.getRelationsNotNull().getRelationship(viewer, other), viewer));
        getPlayerAssetsUpdate(r, offeredAssets, viewer);
        getPlayerAssetsUpdate(r, wantedAssets, viewer);
        return r;
    }

    private static final ValueObjectList getPlayerAssetsUpdate(ValueObjectList r, PlayerAssets assets, Player player) throws ApplicationError {
   		if (assets == null) return r;
    	if (assets.getPlanet() != null) {
    		String planetName = assets.getPlanet();
    		Planet planet = MotEngine.instance.getPlanets().getPlanet(planetName);
			r.addStateUpdate(new PlanetVOAdapter(planet, player));
    		r.addStateUpdate(new SolarSystemVOAdapter(planet.getSolarSystem(), player));
    		if (planet.getMiningStation() != null) {
    			Planet mine = planet.getMiningStation().getPlanet();
    			r.addStateUpdate(new PlanetVOAdapter(mine, player));
        		r.addStateUpdate(new SolarSystemVOAdapter(mine.getSolarSystem(), player));
    		}
    	}
    	if (assets.getTechnologies() != null) {
    		r.addStateUpdate(new PlayerTechnologiesVO(player.getName(), player));
    		r.addStateUpdate(new TechnologyAvailableForResearchVO(new GeneralTechnologyList(player.getTechnologyAvailableForResearch())));
    		// May have updated planetary shields etc
    		r.addStateUpdate(getPlanets(player.getPlanets(), player));
    	}
    	if (assets.getNameList() != null) {
			r.addStateUpdate(new FlushCraftCacheVO());
			r.addStateUpdate(EngineVoFactory.getMoos(MotEngine.instance.getMoveableOwnedObjects().getMOOs(player, true, true), player));
    	}
    	return r;
    }

    public static final ValueObjectList getPlayerAssetsUpdate(PlayerAssets wantedAssets, PlayerAssets offeredAssets, Player player) throws ApplicationError {
    	ValueObjectList r = new ValueObjectList();
    	if (wantedAssets == null) wantedAssets = new PlayerAssets();
    	if (offeredAssets == null) offeredAssets = new PlayerAssets();
		r.addStateUpdate(EngineVoFactory.getPlayer(player));
    	r.addStateUpdate(new PlayerDetailsVOAdapter(player));
    	if (wantedAssets.getPlanet() != null) {
    		String planetName = wantedAssets.getPlanet();
    		Planet planet = MotEngine.instance.getPlanets().getPlanet(planetName);
			r.addStateUpdate(new PlanetVOAdapter(planet, player));
			r.addStateUpdate(new SolarSystemVOAdapter(planet.getSolarSystem(), player));
    		if (planet.getMiningStation() != null) {
    			Planet mine = planet.getMiningStation().getPlanet();
    			r.addStateUpdate(new PlanetVOAdapter(mine, player));
        		r.addStateUpdate(new SolarSystemVOAdapter(mine.getSolarSystem(), player));
    		}
    	}
    	if (offeredAssets.getPlanet() != null) {
    		String planetName = offeredAssets.getPlanet();
    		Planet planet = MotEngine.instance.getPlanets().getPlanet(planetName);
    		r.addStateUpdate(new PlanetVOAdapter(planet, player));
			r.addStateUpdate(new SolarSystemVOAdapter(planet.getSolarSystem(), player));
    		if (planet.getMiningStation() != null) {
    			Planet mine = planet.getMiningStation().getPlanet();
    			r.addStateUpdate(new PlanetVOAdapter(mine, player));
        		r.addStateUpdate(new SolarSystemVOAdapter(mine.getSolarSystem(), player));
    		}
    	}
    	if (wantedAssets.getTechnologies() != null || offeredAssets.getTechnologies() != null) {
    		r.addStateUpdate(new PlayerTechnologiesVO(player.getName(), player));
    		r.addStateUpdate(new TechnologyAvailableForResearchVO(new GeneralTechnologyList(player.getTechnologyAvailableForResearch())));
    	}
    	if (wantedAssets.getNameList() != null || offeredAssets.getNameList() != null) {
			r.addStateUpdate(new FlushCraftCacheVO());
			r.addStateUpdate(EngineVoFactory.getMoos(MotEngine.instance.getMoveableOwnedObjects().getMOOs(player, true, true), player));
    	}
    	return r;
    }

    public static final StellarObjectVO getStellarObject(StellarObject a, Player viewer) throws ApplicationError {
        if (a == null) {
            return null;
        } else if (a instanceof SolarSystem) {
            return new SolarSystemVOAdapter((SolarSystem)a, viewer);   
        } else if (a instanceof BlackHole) {
            return new BlackHoleVOAdapter((BlackHole)a, viewer);   
        } else {
            return new StellarObjectVO(a.getName(), a.getLocation(), Comp.getNames(a.getOwners()), a.getTextColor(), a.isVisible(viewer), a.getLastVisitedDate(viewer), EngineVoFactory.getOrbitalVO(a.getOrbital()));
        }
    }

    public static final CraftDesignsVO getCraftDesigns(CraftDesigns a, Player viewer) throws ApplicationError {
        if (a == null) return null;
        CraftDesign [] cds = a.getCraftDesigns();
        if (cds == null || cds.length == 0) {
            return null;
        }
        CraftDesignVO [] r = new CraftDesignVO[cds.length];
        for (int i=0; i<cds.length; i++) {
        	double buildQuality = viewer.getCraftBuildExperienceNotNull().getBuildQuality(viewer, cds[i].getName());
            r[i] = getCraftDesign(cds[i], buildQuality);
        }
        String lastCraftDesignedName = null;
        if (viewer instanceof UserPlayer) {
        	lastCraftDesignedName = ((UserPlayer)viewer).getLastCraftDesignedName();
        }
        return new CraftDesignsVO(viewer.getName(), r, lastCraftDesignedName);
    }

//    public static final ResearchProjectVO getResearchProject(ResearchProject a) {
//        if (a == null) return null;
//        return new ResearchProjectVOAdapter(a);
//    }

    public static final BattleAutomatonVO getBattleAutomiton(Automaton a, Player viewer) throws ApplicationError {
        if (a == null) return null;
        if (a instanceof Missiles) {
            return new MissilesVOAdapter((Missiles)a, viewer);
        } else if (a instanceof DroidMine) {
            return new DroidMineVOAdapter((DroidMine)a, viewer);
        } else if (a instanceof BattleMine) {
            return new BattleMineVOAdapter((BattleMine)a, viewer);
        } else {
			return new AutomitonVOAdapter(a, viewer);
        }
    }

    public static final void addMooDeepStateUpdate(MoveableOwnedObject [] moo, Player viewer, StateContainer container) throws ApplicationError {
    	if (moo != null) {
    		for (int i = 0; i < moo.length; i++) {
    			addMooDeepStateUpdate(moo[i], viewer, container);
			}
    	}
    }
    
    public static final void addMooDeepStateUpdate(MoveableOwnedObject moo, Player viewer, StateContainer container) throws ApplicationError {
    	container.addStateUpdate( EngineVoFactory.getMoo(moo, viewer) );
        if (moo instanceof Fleet) {
        	Fleet f = (Fleet) moo;
        	container.addStateUpdate(EngineVoFactory.getCrafts(f.getCrafts(), viewer));
        }
    }
    
    public static final MoveableOwnedObjectVO getMoo(MoveableOwnedObject a, Player viewer) throws ApplicationError {
        if (a == null) return null;
        if (a instanceof Craft) {
            return new CraftVOAdapter((Craft)a, viewer);
        } else if (a instanceof Fleet) {
            return new FleetVOAdapter((Fleet)a, viewer);
        } else if (a instanceof Transport) {
            return new TransportVOAdapter((Transport)a);
        } else {
            NavigatorVO navigator = null;
            Navigator n = a.getNavigator();
            if (n != null) {
                navigator = new NavigatorVO(new OrbitVOAdapter(n.getDestination()), n.getETA());
            }
            return new MoveableOwnedObjectVO(
            								 a.getName(), 
            								 a.getOwner().getName(), 
                                             a.getLocation(), 
                                             navigator, 
                                             new OrbitVOAdapter(a.getOrbit()),
                                             a.getVisibility(viewer),
                                             a.getMaxTravelDistance(),
                                             a.isSleeping()
                                             );
        }
    }
    
    public static final RelationVO getRelation(Relation a, Player viewer) throws ApplicationError {
        if (a == null) return null;
        return new RelationVOAdapter(a, viewer);
    }

	public static RelationVO [] getRelations(Relation[] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        RelationVO [] r = new RelationVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = getRelation(a[i], viewer);
        }
        return r;
	}








    public static final BattleAutomatonVO [] getAutomitons(Automaton [] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        BattleAutomatonVO [] r = new BattleAutomatonVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = getBattleAutomiton(a[i], viewer);
        }
        return r;
    }

    public static final BattleCraftVO [] getBattleCrafts(BattleCraft [] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        BattleCraftVO [] r = new BattleCraftVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = getBattleCraft(a[i], viewer);
        }
        return r;
    }

    public static final BattleObjectWeaponStateUpdateVO [] getBattleObjectWeaponStates(BattleObject [] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        BattleObjectWeaponStateUpdateVO [] r = new BattleObjectWeaponStateUpdateVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new BattleObjectWeaponStateUpdateVO(a[i].getBattle().getName(), a[i].getName(), new BattleObjectWeaponStateVOAdapter(a[i]));
        }
        return r;
    }

    public static final WeaponInstanceVO [] getWeaponInstance(WeaponInstance [] a) {
        if (a == null || a.length == 0) {
            return null;
        }
        WeaponInstanceVO [] r = new WeaponInstanceVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new WeaponInstanceVOAdapter(a[i]);
        }
        return r;
    }

    public static final PlanetVO [] getPlanets(Planet [] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        PlanetVO [] r = new PlanetVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new PlanetVOAdapter(a[i], viewer);
        }
        return r;
    }

    public static final CraftVO [] getCrafts(Craft [] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        CraftVO [] r = new CraftVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new CraftVOAdapter(a[i], viewer);
        }
        return r;
    }

    public static final DeadPlayerVO [] getDeadPlayers(Player [] a) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        DeadPlayerVO [] r = new DeadPlayerVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new DeadPlayerVO(getPlayer(a[i]));
        }
        return r;
    }

    public static final PlayerVO [] getPlayers(Player [] a) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        PlayerVO [] r = new PlayerVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = getPlayer(a[i]);
        }
        return r;
    }

    public static final UserPlayerVO [] getUserPlayers(UserPlayer [] a) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        UserPlayerVO [] r = new UserPlayerVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new UserPlayerVOAdapter(a[i]);
        }
        return r;
    }
    
    public static final PlayerConstantsVO [] getPlayerConstants(Player [] a) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        PlayerConstantsVO [] r = new PlayerConstantsVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new PlayerConstantsVOAdapter(a[i]);
        }
        return r;
    }
    
    public static final StellarObjectVO [] getStellarObjects(StellarObject [] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        StellarObjectVO [] r = new StellarObjectVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = getStellarObject(a[i], viewer);
        }
        return r;
    }

    public static final MoveableOwnedObjectVO [] getMoos(MoveableOwnedObject [] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        MoveableOwnedObjectVO [] r = new MoveableOwnedObjectVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = getMoo(a[i], viewer);
        }
        return r;
    }

    public static final MoveableOwnedObjectDelegate [] getMooDelegates(MoveableOwnedObject [] a, Player viewer) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        MoveableOwnedObjectDelegate [] r = new MoveableOwnedObjectDelegate[a.length];
        for (int i=0; i<a.length; i++) {
        	MoveableOwnedObjectVO vo = getMoo(a[i], viewer);
			if (vo instanceof CraftVO) {					
				r[i] = new MotCraftDelegate((CraftVO)vo);
			} else if (vo instanceof FleetVO) {					
				r[i] = new MotFleetDelegate((FleetVO)vo);
			} else if (vo instanceof TransportVO) {					
				r[i] = new MotTransportDelegate((TransportVO)vo);
			}
        }
        return r;
    }


	public static StellarObjectOrbitalVO getOrbitalVO(StellarObjectOrbital orbital) throws ApplicationError {
		if (orbital  == null) {
			return null;
		} else if (orbital instanceof Outpost) {
			return new OutpostVO(orbital.getOwner().getName(), orbital.getOrbital());
		} else if (orbital instanceof SpaceStation) {
			return new SpaceStationVO(orbital.getOwner().getName(), orbital.getOrbital());
		} else {
			return new StellarObjectOrbitalVO(orbital.getOwner().getName(), orbital.getOrbital());
		}
	}

	public static CraftDesignsVO getCraftDesign(UserPlayer owner, Player viewer) throws ApplicationError {
		CraftDesigns craftDesigns = owner.getCraftDesigns();
		if (craftDesigns == null) return null;
		CraftDesign[] cda = craftDesigns.getCraftDesigns();
		CraftDesignVO [] voa = new CraftDesignVO[cda.length];
		for (int i = 0; i < cda.length; i++) {
			voa[i] = VoFactory.getCraftDesign(cda[i], viewer.getCraftBuildExperienceNotNull().getBuildQuality(viewer, cda[i].getName()), owner.getName());
		}
		String name = owner.getName();
		String lastCraftDesignedName = owner.getLastCraftDesignedName();
		return new CraftDesignsVO(name, voa, lastCraftDesignedName);
	}

	public static ResearchProjectVO getResearchProject(ResearchProject researchProject) {
		if (researchProject != null) {
			return new ResearchProjectVOAdapter(researchProject);
		} else {
			return null;
		}
	}

	public static TradeRouteVO[] getTradeRoutes(TradeRoute [] a, Player player) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        TradeRouteVO [] r = new TradeRouteVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new TradeRouteVOAdapter(a[i], player);
        }
        return r;
	}

	public static TradeRouteListVO getTradeRoutes(TradeRoutes tradeRoutes, Player player) throws ApplicationError {
		if (tradeRoutes == null) {
			return null;
		} else {
			TradeRouteVO[] tra = getTradeRoutes(tradeRoutes.getTradeRoutes(), player);
			return new TradeRouteListVO(tra);
		}
	}

    public static final TradeRouteInvalidVO [] getInvalidTradeRoutes(TradeRoute [] a) throws ApplicationError {
        if (a == null || a.length == 0) {
            return null;
        }
        TradeRouteInvalidVO [] r = new TradeRouteInvalidVO[a.length];
        for (int i=0; i<a.length; i++) {
            r[i] = new TradeRouteInvalidVO(a[i].getSolarSystem1Name(), a[i].getSolarSystem2Name());
        }
        return r;
    }


 }