/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.common;

import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.technology.Engine;
import mot.common.technology.Engines;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.crafts.CraftOrbitFactory;
import mot.engine.crafts.Navigator;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;

/**
 * Based class for all objects that have a name, owner and a location that can change.
 * Craft and Fleet objects are a subclass of MoveableOwnedObject but Planets and Stellar 
 * Objects are not (because they do not move).
 * 
 * This object encapsulates the functionality that enables an object to move. Thus
 * MoveableOwnedObjects have 
 *      - An engine and possibly specials that determine its speed.
 *      - An location on the galaxy map
 *      - An orbit or navigator. If a movable object is travelling between solar systems 
 *          then it will contain a Navigator. If a movable object is in orbit around a 
 *          stellar object then orbit will be set.
 * 
 * If this object has specials that effect its speed (at warp), then override the 
 * getSpecials() method.
 */
public abstract class MoveableOwnedObject extends OwnedObject {

	/**
	 * The position on the map.
	 */
	private Location location;

	/**
	 * The engines on this movable object.
	 */
	private Engine engine;
	
	/**
	 * Normally each craft will ask the owner for a task if idle. If an object is sleeping
	 * the game will not ask the player what he wished to do with the craft this turn. 
	 */
	private boolean sleeping = false;

	public MoveableOwnedObject() {
	}

	public MoveableOwnedObject(String name, Player owner, Engine engine, Orbit orbit) {
		super(name, owner);
		this.engine = engine;
		this.orbit = orbit;
        if (orbit != null && orbit.getStellarObject() != null) {
            this.location = new Location(orbit.getStellarObject().getLocation());
        }
	}


	public Location getLocation() throws ApplicationError { return location; }
	public void setLocation(Location v) { location = v; }
	public boolean sameLocation(Location v) { return location.equals(v); }
	public int getDistanceTo(Location v) { return location.getDistanceTo(v); }
    
	public Engine getEngine() { return engine; }
	public void setEngine(Engine v) { engine = v; }
    
	/**
	 * Creates a new reference to this object, e.g. a CraftReference..
	 */
	public abstract MoveableOwnedObjectReference createNewReference() throws ApplicationError;

	/**
	 * If a movable object is travelling between solar systems then it will contain
	 * a Navigator.
     * Generally - A movable object will either have an orbit or a navigator, but
     * if a craft has been assigned an order to leave a solar system but not yet
     * left (end turn has not been pressed) then it may have an orbit and a navigator!
	 */
	private Navigator navigator;

	public Navigator getNavigator() throws ApplicationError { return navigator; }
	public void setNavigator(Navigator v) { navigator = v; }

	/**
	 * If a movable object is located in a solar systems then it will contain
	 * an orbit. 
	 * @see navigator for how an orbit an navigator interrelate.
	 */
	private Orbit orbit;

	public void setOrbit(Orbit v) { orbit = v; }
	public void setOrbit(StellarObject so, Planet p) { orbit = new Orbit(so, p); }
	public Orbit getOrbit() throws ApplicationError { return orbit;	}
	
	/**
	 * WARNING - This will not return the correct orbit of a craft in a fleet. Use getOrbit() or  getOrbitStellarObject().
	 * @return The value of the orbit field for this object.
	 */
	public StellarObject getActualOrbit() { return orbit == null ? null : orbit.getStellarObject(); }

	public boolean isInOrbit() throws ApplicationError {
		Orbit o = getOrbit();
		if (o == null) {
			return false;
		} else {
			return true;
		}
	}

	public boolean isOrbiting(StellarObject so) throws ApplicationError {
		Orbit o = getOrbit();
		if (o == null) {
			return false;
		}
		return o.getStellarObject() == so;
	}
	
	public boolean isOrbiting(Planet planet) throws ApplicationError {
		Orbit o = getOrbit();
		if (o == null) {
			return false;
		}
		return o.getPlanet() == planet;
	}
	
	public StellarObject getOrbitStellarObject() throws ApplicationError {
		Orbit o = getOrbit();
		if (o == null) {
			return null;
		}
		return o.getStellarObject();
	}
	
    public String getTaskDescription() throws ApplicationError {
        Navigator naviogator = getNavigator();
        if (navigator != null) {
            return "Enroute to "+navigator.getDestination().getDescription()+" dist="+navigator.getDistanceRemaining();
        } else if (orbit != null) {
            return "In orbit of "+orbit.getDescription();
        } else {
            return "None";
        }
    }

	/**
	 * When an object leaves orbit to move to a new Solar System it must call depart.
     * This removes the objects orbit. The navigator should already have been setup
     * and the navigator is the thing that calls depart() when moving the object to 
     * a new location.
	 * @throws ApplicationError 
	 */
	public void depart(Navigator nav) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("MoveableOwnedObject.depart() "+getName()+" for "+nav.getDestination().getDescription());
        }
        Orbit orbit = getOrbit();
        if (orbit != null && orbit.getStellarObject() != null && !orbit.getStellarObject().getLocation().equals(getLocation())) {
        	// If someone has modified the save file, moving this stellar object, then this craft must move with it.
        	setLocation(new Location(orbit.getStellarObject().getLocation()));
        }
		setOrbit(null);
        // This may be a craft that is part of a fleet so I will set the navigator
        // So crafts is a fleet have a reference to the fleets navigator for convenience.
        setNavigator(nav);
	}
    
	/**
	 * When an object arrives at the destination arrivedAt must be called.
     * This sets the objects orbit and removes the navigator.
     * If the planet is not null then the planet is specified on the orbit created.  
	 */
	public void arrivedAt(Orbit orbit) throws ApplicationError {
		setOrbit(orbit);
        setNavigator(null);
        Location loc = getLocation();
        loc.setX(orbit.getLocation().getX());
        loc.setY(orbit.getLocation().getY());
        Player owner = getOwner();
        StellarObject stellarObject = orbit.getStellarObject();
        MonsterPlayer monsterPlayer = MotEngine.instance.getMonsterPlayer();
        int qtyMonsterCrafts = monsterPlayer.getQtyCraftsAt(stellarObject);
        if (qtyMonsterCrafts == 0) {
            stellarObject.addVisitor(owner, true);
        }
        if (orbit.getPlanet() == null && stellarObject instanceof SolarSystem) {
	        CraftOrbitFactory orbitFactory = owner.getCraftOrbitFactory();
	        if (orbitFactory != null) {
	        	SolarSystem ss = (SolarSystem) stellarObject;
	        	orbit.setPlanet(orbitFactory.getOrbit(ss, this));
	        }
        }
	}
    
	public void setLocationInSpace(Location loc, Orbit dest) throws ApplicationError {
		setOrbit(null);
		setLocation(new Location(loc));
		if (dest != null) {			
			Navigator nav = new Navigator();
			nav.setDestination(dest);
			nav.setObjectReference(createNewReference());
			setNavigator(nav);
		}
	}
	
	/**
     * @return True if this object is in a battle at this solar system. This includes
     * battles that have started of battles that have not yet started (will start at the
     * end of the turn).
     */
    public boolean isInABattle() throws ApplicationError {
        Orbit orbit = getOrbit();
        if (orbit != null) {
            if (MotEngine.instance.getBattles() != null) {
                Battle b = MotEngine.instance.getBattles().getBattleAt(orbit);
                if (b != null && b.contains(this)) {
                    // This craft cannot depart a solar system while it is being attacked!
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * @return The battle this MOO is going to take part in at the end of this turn.
     */
    public Battle getBattle() throws ApplicationError {
        Orbit orbit = getOrbit();
        if (orbit != null) {
            if (MotEngine.instance.getBattles() != null) {
                Battle b = MotEngine.instance.getBattles().getBattleAt(orbit);
                if (b != null && b.contains(this)) {
                    return b;
                }
            }
        }
        return null;
    }

    /**
     * Moves this object to the stellar object so. It does not take any notice of turns
     * or distance - it simple does a direct transfer to the destination. 
     * @param so The object this should be located at.
     * @param planet If so is a solar system with may be a planet (or null).
     */
    public void transferTo(Orbit orbit) throws ApplicationError {
        if (getLocation() == null) {
            setLocation(new Location(orbit.getLocation()));
        } else {
            getLocation().shiftTo(orbit.getLocation());
        }
        setOrbit(orbit);
        setNavigator(null);
    }

    /**
     * @return the size of the bitmap used to display this craft for this player on the screen.
     */
    public abstract Size getImageSize() throws ApplicationError;
    
    /**
     * @return a the speed bonuses for this objects.
     * Some MOOs may contains specials that effect the speed, if so, override this method to provide 
     * the speed bonus. The default implementation provided here does not have any specials so receives 
     * no speed advantages.
     */
    public double getSpeedBonus() throws ApplicationError {
        return 1.0;
    }

    public String getSpeedBonusDescription() throws ApplicationError {
        return null;
    }
    
    /**
     * This speed is calculated as the speed of the engine multiplied by the speed bonus for the player.
     * @return The warp speed this craft can travel at.
     */
    public int getWarpSpeed() throws ApplicationError {
		Engine e = getEngine();
		if (e == null) {
			// This must be a fleet with no craft
			return 10;
		}
		int speed = e.getWarpSpeed();
        double speedBonus = getSpeedBonus();
        speed = (int) (speed * speedBonus);
        return speed;
    }

    public String getWarpSpeedDescription() throws ApplicationError {
		Engine e = getEngine();
		if (e == null) {
			// This must be a fleet with no craft
			return null;
		}
		int speed = e.getWarpSpeed();
        String bonusDescriptions = getSpeedBonusDescription();
        return L.SPEED+" = "+Formatter.whole(speed)+bonusDescriptions;
    }
    
	/**
	 * @return the visibility of this object by player at this point in time (on the galaxy map).
	 */
	public abstract int getVisibility(Player player) throws ApplicationError;
	
	/**
	 * End the turn of this object - If this object has a navigator this method will move the object through space.
	 * @throws ApplicationError
	 */
	public abstract void endTurn() throws ValidationError, ApplicationError;

    /**
     * Gives the order for this object to move to the destination star system from its current position.
     * If the destination is the current star system it is in orbit around, then any command to move to
     * another solar system is cancelled.
     */
	public void moveTo(Orbit orbit) throws  ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("MoveableOwnedObject.moveTo() "+getName()+" to "+orbit.getDescription());
        }
        if (getEngine() == null) {
            Logger.logWarning("MoveableOwnedObject.moveTo() Fleet "+getName()+" has no engine [maybe no crafts have been added]");
            return;
        }
        if (orbit.getStellarObject() == getOrbitStellarObject()) {
            // Interpreted as cancel move command or move to another planet is SS
            arrivedAt(orbit);
        } else {
            setNavigator(new Navigator(this, this.orbit, orbit));
        }
	}
    
    /**
     * @return true if this object is heading to the solar system ss.
     */
    public boolean isOnrouteTo(StellarObject stellarObject) throws ApplicationError {
        return getNavigator() != null && getNavigator().getDestination().atSameSO(stellarObject);
    }
    
    /**
     * @return true if and only is this craft can move to this solar system.
     */
	public boolean canMoveTo(StellarObject stellarObject) throws ApplicationError {
		return canTravelTo(stellarObject);
    }

	/**
	 * @return true iff this engine technology can travel to this solar system
	 * for this player.
	 */
	public boolean canTravelTo(StellarObject stellarObject) throws ApplicationError {
        if (engine == null) return false;
        Player owner = getOwner();
		if (owner.getResourceCache().isInfinateRangeAbility()) {
			return true;
		}
		StellarObject so = owner.getClosestStellarObject(stellarObject.getLocation());
		if (so == null) return false;
		int distance = so.getLocation().getDistanceTo(stellarObject.getLocation());
		return (distance <= getMaxTravelDistance());
	}
    
	/**
	 * Remove this object from the game. If this object is a single item, then it 
	 * is equivalent to destroy(). However, if this represents many objects (like 
	 * a fleet), then it is equivalent to destroy on each item. 
	 */
	public void destroyAll() throws ApplicationError {
		destroy();
	}
	
	/**
	 * Remove this object from the game. It will be removed from the engine not 
	 * no notifications will be sent to any clients.  
	 */
	public abstract void destroy() throws ApplicationError;
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		if (location != null) {
			location.toXML(writer);
		}
		if (orbit != null) {
			orbit.toXML(writer);
		}
		if (engine != null) {
            writer.addElement("engine", engine.getName());
			//engine.toXML(writer);
		}
		//if (sleeping) writer.addElement("sleeping", true);
		if (navigator != null) {
			navigator.toXML(writer);
		}
	}

    public void setEngine(String engineName) throws ApplicationError {
		Engines engines = Technologies.getEngines();
        engine = engines.getEngine(engineName);
    }

	public int getMaxTravelDistance() throws ApplicationError {
		if (engine == null) {
			return 0;
		} else {
			return engine.getMaxTravelDistance();
		}
	}

	public String getDescription() {
		return getName() + "@" + ((orbit == null) ? (location == null ? "?" : location.toString()) : orbit.getDescription());
	}

	public boolean isSleeping() {
		return sleeping;
	}

	public void setSleeping(boolean sleeping) {
		this.sleeping = sleeping;
	}

}