/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.common;

import java.util.ArrayList;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.engine.MotEngine;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.Navigator;
import mot.engine.planets.Planet;
import mot.engine.players.Player;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportManager;

/**
 * This class is used to collect / locate movable owned objects.
 * It is designed to be a container, similar to Crafts, but the things it conceptually
 * contains actually come from other containers that hold all the Movable Owned Objects
 * such as Crafts and Transports.
 */
public class MoveableOwnedObjects  {
    
    public MoveableOwnedObjects() {
    }
    
    /**
     * Crafts are located on the galaxy may in the same x,y position as the craft objects position 
     * unless it is in orbit around a stellar object.
     * 
     * Note - this may not find crafts located in a stellar object because their image is located
     * close the the stellar object (not over it). To locate a craft at a stellar object locate a
     * stellar object the user close to the users click and then find visibly crafts at that location.
     * 
     * @return an array of crafts located at x,y or null. 
     */
    public MoveableOwnedObject [] getMOOsAt(int x, int y) throws ApplicationError {
        ArrayList r = null;
        Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if ( crafts[i].getLocation().equals(x,y) ) {
                    if (r  == null) r = new ArrayList();
                    Fleet fleet = crafts[i].getFleet();
                    if (fleet != null) {
                        if (!r.contains(fleet)) {
                            r.add(fleet);
                        }
                    } else {
                        r.add(crafts[i]);
                    }
                }
            }
        }
        
        if (r == null) {
            return null;
        } else {
            return (MoveableOwnedObject []) r.toArray(new MoveableOwnedObject[r.size()]);
        }
    }

    public MoveableOwnedObject [] getMOOsAt(StellarObject stellarObject) throws ApplicationError {
    	Location loc = stellarObject.getLocation();
    	return getMOOsAt(loc.getX(), loc.getY());
    }

    /**
     * @return All moos that are visible by Player player or null OR all moos in the galaxy
     */
    public MoveableOwnedObject [] getMOOs(Player player, boolean mustBeVisible, boolean includeCraftsInFleet) throws ApplicationError {
        ArrayList r = null;
        Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	Fleet fleet = crafts[i].getFleet();
				boolean visible = fleet == null ? crafts[i].isVisible(player) : fleet.isVisible(player);
                if ( player == null || crafts[i].isVisible(player) || !mustBeVisible ) {
                    if (r  == null) r = new ArrayList();
                    if (fleet != null) {
                        if (includeCraftsInFleet) r.add(crafts[i]);
                    } else {
                    	r.add(crafts[i]);
                    }
                }
            }
        }
        Player [] players = MotEngine.instance.getPlayersNotNull().getPlayers();
        for (int i = 0; i < players.length; i++) {			
        	if (players[i].getTransportManager() != null) {
        		Transport [] transports = players[i].getTransportManager().getTransportsAsArray();
        		if (transports != null) {
        			for (int j=0; j<transports.length; j++) {
        				if (transports[j].displayOnGalaxyMap()) {
        	                if ( player == null || transports[j].isVisible(player) || !mustBeVisible ) {
        	                	if (r  == null) r = new ArrayList();
        	                	r.add(transports[j]);
        	                }
        				}
        			}
        		}
        	}
        	Fleet[] fleets = players[i].getFleet();
        	if (fleets != null) {
        		for (int j = 0; j < fleets.length; j++) {
                    if ( player == null || fleets[j].isVisible(player) || !mustBeVisible ) {
                    	r.add(fleets[j]);
                    }
				}
        	}
		}
        
        if (r == null) {
            return null;
        } else {
            return (MoveableOwnedObject []) r.toArray(new MoveableOwnedObject[r.size()]);
        }
    }

    /**
     * @return All moos that are owned by Player player or null OR all moos in the galaxy
     */
    public MoveableOwnedObject [] getMOOs(Player player, boolean includeCraftsInFleet) throws ApplicationError {
        ArrayList r = null;
        Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if ( player == null || crafts[i].isOwnedBy(player) ) {
                    if (r  == null) r = new ArrayList();
                    Fleet fleet = crafts[i].getFleet();
                    if (fleet != null) {
                        if (!r.contains(fleet)) {
                            r.add(fleet);
                        }
                        if (includeCraftsInFleet) r.add(crafts[i]);
                    } else {
                    	r.add(crafts[i]);
                    }
                }
            }
        }
        Player [] players = MotEngine.instance.getPlayersNotNull().getPlayers();
        for (int i = 0; i < players.length; i++) {			
        	if (players[i].getTransportManager() != null) {
        		Transport [] transports = players[i].getTransportManager().getTransportsAsArray();
        		if (transports != null) {
        			for (int j=0; j<transports.length; j++) {
        				if (transports[j].displayOnGalaxyMap()) {
        	                if ( player == null || transports[j].isOwnedBy(player) ) {
        	                	if (r  == null) r = new ArrayList();
        	                	r.add(transports[j]);
        	                }
        				}
        			}
        		}
        	}
		}
        
        if (r == null) {
            return null;
        } else {
            return (MoveableOwnedObject []) r.toArray(new MoveableOwnedObject[r.size()]);
        }
    }

    /**
     * Crafts are locatred on the galaxy may in the same x,y position as the craft objects position 
     * unless it is in orbit around a stellar object.
     * 
     * Note - this may not find crafts located in a steller object becuase their image is located
     * close the the stellar object (not over it). To locate a craft at a stellar object locate a
     * stellar object the user close to the users click and then find visibly crafts at that location.
     * 
     * @return an array of crafts located at x,y and visible by Player player or null. 
     */
    public MoveableOwnedObject [] getVisibleMOOsAt(int x, int y, Player player) throws ApplicationError {
        ArrayList r = null;
        Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if ( crafts[i].getLocation().equals(x,y) && crafts[i].isVisible(player) ) {
                    if (r  == null) r = new ArrayList();
                    Fleet fleet = crafts[i].getFleet();
                    if (fleet != null) {
                        if (!r.contains(fleet)) {
                            r.add(fleet);
                        }
                    } else {
                        r.add(crafts[i]);
                    }
                }
            }
        }
        if (player.getTransportManager() != null) {
            Transport [] transports = player.getTransportManager().getTransportsAsArray();
            if (transports != null) {
                for (int i=0; i<transports.length; i++) {
                    if (transports[i].displayOnGalaxyMap()) {
                        if (transports[i].getLocation().equals(x,y)) {
                            r.add(transports[i]);
                        }
                    }
                }
            }
        }
        
        if (r == null) {
            return null;
        } else {
            return (MoveableOwnedObject []) r.toArray(new MoveableOwnedObject[r.size()]);
        }
    }

    /**
     * @return An array of crafts in orbit around a stellar object and visible by Player player
     * or null.
     */
    public MoveableOwnedObject [] getVisibleMOOsAt(StellarObject stellarObject, Player player) throws ApplicationError {
        if (stellarObject == null) {
            return null;
        }
        Location loc = stellarObject.getLocation();
        return getVisibleMOOsAt(loc.getX(), loc.getY(), player);
    }

	public MoveableOwnedObject getMOO(String name) throws ApplicationError {
        Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if ( Comp.same(name, crafts[i]) ) {
                    return crafts[i];
                }
            }
        }
        Player [] players = MotEngine.instance.getPlayersNotNull().getPlayers();
        if (players != null) {
        	for (int i = 0; i < players.length; i++) {
				Fleet fleet = players[i].getFleet(name);
				if (fleet != null) {
					return fleet;
				}
			}
        }
        Fleet fleet = MotEngine.instance.getMonsterPlayer().getFleet(name);
        if (fleet != null) {
			return fleet;
		}
        for (int i = 0; i < players.length; i++) {			
        	if (players[i].getTransportManager() != null) {
        		Transport [] transports = players[i].getTransportManager().getTransportsAsArray();
        		if (transports != null) {
        			for (int j=0; j<transports.length; j++) {
        				if (transports[j].displayOnGalaxyMap() &&  Comp.same(name, transports[j].getName())) {
        	                return transports[j];
        				}
        			}
        		}
        	}
		}

		return null;
	}


	public MoveableOwnedObject [] getMOOsAt(Planet planet) throws ApplicationError {
        ArrayList r = null;
        Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                Orbit orbit = crafts[i].getOrbit();
				if ( orbit != null && orbit.getPlanet() == planet ) {
                    if (r  == null) r = new ArrayList();
                    Fleet fleet = crafts[i].getFleet();
                    if (fleet != null) {
                        if (!r.contains(fleet)) {
                            r.add(fleet);
                        }
                    } else {
                        r.add(crafts[i]);
                    }
                }
            }
        }
        Player[] players = MotEngine.instance.getPlayersNotNull().getPlayers();
        for (int i = 0; i < players.length; i++) {			
        	TransportManager transportManager = players[i].getTransportManager();
			if (transportManager != null) {
        		Transport [] transports = transportManager.getTransportsAsArray();
        		if (transports != null) {
        			for (int j=0; j<transports.length; j++) {
    					Orbit orbit = transports[j].getOrbit();
						if (orbit != null && orbit.getPlanet() == planet) {
    						r.add(transports[j]);
    					}
        			}
        		}
        	}
		}
        
        if (r == null) {
            return null;
        } else {
            return (MoveableOwnedObject []) r.toArray(new MoveableOwnedObject[r.size()]);
        }
	}


	public MoveableOwnedObject [] getMOOsWithDestination(Planet planet) throws ApplicationError {
        ArrayList r = null;
        Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if ( crafts[i].getNavigator() != null && crafts[i].getNavigator().getDestination().getPlanet() == planet) {
                    if (r  == null) r = new ArrayList();
                    Fleet fleet = crafts[i].getFleet();
                    if (fleet != null) {
                        if (!r.contains(fleet)) {
                            r.add(fleet);
                        }
                    } else {
                        r.add(crafts[i]);
                    }
                }
            }
        }
        Player[] players = MotEngine.instance.getPlayersNotNull().getPlayers();
        for (int i = 0; i < players.length; i++) {			
        	TransportManager transportManager = players[i].getTransportManager();
			if (transportManager != null) {
        		Transport [] transports = transportManager.getTransportsAsArray();
        		if (transports != null) {
        			for (int j=0; j<transports.length; j++) {
    					Navigator navigator = transports[j].getNavigator();
						if (navigator != null && navigator.getDestination() != null && navigator.getDestination().getPlanet() == planet) {
    						r.add(transports[j]);
    					}
        			}
        		}
        	}
		}
        
        if (r == null) {
            return null;
        } else {
            return (MoveableOwnedObject []) r.toArray(new MoveableOwnedObject[r.size()]);
        }
	}


    
}