/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.common;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.vo.OrbitVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.planets.Planet;
import mot.engine.planets.PlanetReference;
import mot.engine.planets.SolarSystemReference;
import mot.engine.players.Player;

/**
 * This class represents the rotation of a movable owned object around a stellar object. 
 * If an orbit is around a solar system then it may also contain a planet.
 * 
 * When the galaxy map is zoomed in, MOOs can be seen as well as individual planet in the 
 * solar system. The engine consider all craft at a stellar object to be at the 'same' location,
 * however, a planet may be specified for user effect - when the user clicks on a 
 * planet the craft moves to orbit that planet and not the sun or another planet in the solar 
 * system!
 */
public class Orbit extends XMLBean {

	/**
	 * This is the stellar object this orbit is located at
	 */
	private StellarObjectReference stellarObject;
	
	/**
	 * This is the stellar object this orbit is located at
	 */
	private PlanetReference planet;

	public Orbit() {
	}

	public Orbit(Orbit other) {
		this(other.getStellarObject(), other.getPlanet());
	}

	
	public Orbit(StellarObjectReference stellarObject, PlanetReference planet) {
		this.stellarObject = stellarObject;
		this.planet = planet;
	}

	public Orbit(Planet planet) {
		this(planet.getStellarObject(), planet);
	}
	
	public Orbit(StellarObject so) {
		this(so, null);
	}
	
	public Orbit(StellarObject stellarObject, Planet planet) {
		this.stellarObject = stellarObject.getObjectReference();
		if (planet != null) {
			this.planet = new PlanetReference(planet);
		}
	}
	
	public Orbit(OrbitVO orbit) {
		if (orbit != null) {
			if (orbit.getStellarObject() != null) {
				StellarObject so = MotEngine.instance.getStellarObjectsNotNull().getStellarObject(orbit.getStellarObject());
				stellarObject = so.getObjectReference();
			}
			if (orbit.getPlanet() != null) {
				planet = new PlanetReference(orbit.getPlanet());
			}
		}
	}

	public boolean atSameSO(StellarObject stellarObject) {
		return (this.stellarObject.getStellarObject() == stellarObject);
	}

	public boolean atSameSO(Orbit orbit) {
		return (stellarObject.getStellarObject() == orbit.getStellarObject());
	}

	public boolean equals(Orbit orbit) {
		if (orbit == null) return false;
		if (planet != null) {
			return planet.getPlanet() == orbit.getPlanet();
		} else {
			if (orbit.getPlanet() != null) return false;
			return (stellarObject != null && stellarObject.getStellarObject() == orbit.getStellarObject());
		}
	}

	public Location getLocation() {
		if (stellarObject == null) {
			if (planet != null) {
				return planet.getPlanet().getStellarObject().getLocation();
			} else {
				return null;
			}
		}
		return stellarObject.getStellarObject().getLocation();
	}

	public Planet getPlanet() {
		if (planet == null) {
			return null;
		} else {
			return planet.getPlanet();
		}
	}

	public void setPlanet(Planet planet) {
		if (planet == null) return;
		this.planet = new PlanetReference(planet);
	}

	public void setPlanet(PlanetReference planet) {
		this.planet = planet;
	}

	public StellarObject getStellarObject() {
		if (stellarObject == null) {
			if (planet != null) {
				return planet.getPlanet().getStellarObject();
			} else {
				return null;
			}
		} else {
			return stellarObject.getStellarObject();
		}
	}

	public void setStellarObjectReference(StellarObjectReference stellarObject) {
		this.stellarObject = stellarObject;
	}

	public void setStellarObject(StellarObjectReference stellarObject) {
		this.stellarObject = stellarObject;
	}

	public void setStellarObject(SolarSystemReference stellarObject) {
		this.stellarObject = stellarObject;
	}

	public void setStellarObject(StellarObject stellarObject) {
		if (stellarObject != null) {
			this.stellarObject = stellarObject.getObjectReference();
		}
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (stellarObject != null) {
			stellarObject.toXML(writer, "stellar-object");
		}
		if (planet != null) {
			planet.toXML(writer, "planet");
		}
	}

	protected String getElementName() {
		return "orbit";
	}
	
	public String getDescription() {
		String planetStr = (planet == null) ? "" : "[planet="+planet.getName()+"]";
		if (stellarObject == null) {
			return planetStr;
		} else {
			return stellarObject.getName() + planetStr;
		}
	}
	
	public String getShortDescription() {
		if (planet != null) {
			return planet.getName();
		} else if (stellarObject != null) {
			return stellarObject.getName();
		} else {
			return "None";
		}
	}
	
	/**
	 * A player is defined to own an orbit if the orbit is in a stellar object owned
	 * by the player.
	 * @param player The player that may own this orbit
	 * @return True if the player owned this orbit and false otherwise
	 */
	public boolean isOwnedBy(Player player) throws ApplicationError {
		return stellarObject.getStellarObject().isOwnedBy(player);
	}
	
	public String toString() {
		if (planet != null) {
			return "Orbit "+planet;
		} else {
			return "Orbit "+stellarObject;
		}
	}

}
