/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.common;

import mot.common.bean.NamedObject;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.players.Player;
import mot.engine.players.PlayerReference;

/**
 * Base class for all Objects in the system that have an Owner. This includes most 
 * things in the system including Planets, Crafts, Fleets. The owner of an object is
 * a Player and may be a UserPlayer or a ComputerPlayer.
 * 
 * The owner of most objects can change over time, e.g. a Planet can be invaded, a 
 * craft can surrender or be boarded. This an owned object also has an origin. An
 * origin is the owner who first owned this object. An objects origin usually does
 * not change.
 * 
 * It is a object's origin that determines its bonuses not its owner. Thus for a planet
 * or a craft that changes owner still contains the same population/crew it had before
 * and thus carries the same bonuses.
 */
public abstract class OwnedObject extends NamedObject {

	/**
	 * The player to whom this object belongs. This may change over time.
	 */
	private PlayerReference ownerReference;

	/**
	 * This is the original owner. For a planet it is the race that populated the
     * planet. For a craft it is the player from whom the craft's crew belong - usually
     * the player that built the craft.
     * 
     * Player bonuses should use the origin player not the owner player for calculation.
     * Just because a planet changes owner, its race does not change. Therefore its 
     * bonuses should not change.
     * 
     * This attribute is not used (has a null value) until the owner changes.
	 */
	private PlayerReference originReference;

    /**
     * @return The owner of this object 
     */
	public Player getOwner() throws ApplicationError {	
        if (ownerReference == null) 
            return null; 
        else 
            return ownerReference.getPlayer(); 
    }

    /**
     * @return The owner of this object 
     */
	public PlayerReference getOwnerReference() {	return ownerReference; }

    /**
     * Sets the owner of this object
     * Do not use this method to change the owner of an object - use changeOwner().
     */
	public void setOwner(Player v) { ownerReference = (v == null) ? null : new PlayerReference(v); }

    /**
     * Sets the owner of this object
     * Do not use this method to change the owner of an object - use changeOwner().
     */
	public void setOwnerReference(PlayerReference v) {	ownerReference = v; }

    /**
     * If origin reference is null then the owner has not changed for this object and
     * the owner is returned.
     * @return The origin player of this object 
     */
	public Player getOrigin() throws ApplicationError {	
        if (originReference == null) {
            return getOwner(); 
        } else { 
            Player origin = originReference.getPlayer();
            if (origin == null) {
                // This player no longer exists - Check dead players!
                origin = MotEngine.instance.getDeadPlayersNotNull().getPlayer(originReference.getName());
            }
            if (origin == null) {
                Logger.logWarning("OwnedObject.getOrigin() PLAYER "+originReference.getName()+" DOES NOT EXIST : set origin to owner");
                originReference = null;
                return getOwner();
            } else {
                originReference = new PlayerReference(origin);
                return origin;
            }
        }
    }

    /**
     * @return The origin of this object 
     */
	public PlayerReference getOriginReference() {	return originReference; }

    /**
     * Sets the origin of this object
     */
	public void setOrigin(Player v) { originReference = (v == null) ? null : new PlayerReference(v); }

    /**
     * Sets the origin of this object
     */
	public void setOriginReference(PlayerReference v) {	originReference = v; }

	public OwnedObject() {
	}

	public OwnedObject(String name, Player owner) {
		super(name);
		if (owner != null) {
            setOwner(owner);
        }
	}

	public OwnedObject(Player owner) {
		super(owner);
		if (owner != null) {
            setOwner(owner);
        }
	}

    /**
     * Changes the owner of this object from its current owner to newOwner. The origin
     * player of this object is set to the old owner.
     * @throws mot.common.util.ApplicationError
     * @param newOwner The new owner for this object.
     */
	public void changeOwner(Player newOwner) throws ApplicationError {
        if (Logger.isDebug()) {
            String oldOwnerName = "None";
    		if (getOwner() != null) oldOwnerName = getOwner().getName();
            String newOwnerName = "None";
    		if (newOwner != null) newOwnerName = newOwner.getName();
            Logger.logDebug("OnwedObject.changeOwner() Object "+getName()+" change owner from "+oldOwnerName+" to "+newOwnerName);
        }
        Player owner = getOwner();
		if (owner != null) {
            owner._remove(this);
        }
        if (newOwner != null) {
            newOwner.add(this);
            if (originReference == null) {
                setOrigin(owner);
            }
        }
        setOwner(newOwner);
	}

    /**
     * @return true if player passed in is the owner of this object.
     */
	public boolean isOwnedBy(Player player) throws ApplicationError {
		return (player == getOwner());
	}

	public boolean hasOwner() throws ApplicationError {
		return (ownerReference != null);
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		if (ownerReference != null) {
			ownerReference.toXML(writer, "owner-reference");
		}
		if (originReference != null) {
			originReference.toXML(writer, "origin-reference");
		}
	}
}