/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.common;

import mot.common.bean.Area;
import mot.common.bean.ImageObjectSettings;
import mot.common.bean.LocatedArea;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.LocationDelta;
import mot.common.bean.NamedLocatedObject;
import mot.common.bean.Rectangle;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleObject;
import mot.engine.battles.MovementCostCalculator;
import mot.engine.players.Player;

/**
 * This is a server class that represents the server side of the class ImageObject.
 * This class does not contain any image information because the server does not care
 * about images - however it does care about the presence, location and size of the 
 * images that are being displayed on the screen to a user - This object contains that
 * information! 
 */
public abstract class ServerImageObject extends NamedLocatedObject implements LocatedArea {
    public ServerImageObject() {
    }

	public ServerImageObject(String name, int x, int y) {
		super(name, y, y);
	}

	public ServerImageObject(String name, Location l) {
		super(name, l);
	}
	
    /**
     * @return the closest location this craft can occupy next to the craft passed in.
     * The location returned will be on a straight line between this craft and the target craft, but as close 
     * to the target craft as we can get. 
     * NOTE - This location may be occupied by another object!
     */
	public Location getClosestLocationTo(ServerImageObject io) throws ApplicationError {
        Area occupiedArea = io.getArea();
        Location loc = occupiedArea.getClosestEdgeTo(getLocation());
        loc.shift(getLocation(), getMaxSpaceDimension());
        Area area = getAreaAt(loc);
        while (area.intersects(occupiedArea)) {
            LocationDelta shiftDelta = loc.shiftDelta(getLocation(), 2);
            area.shift(shiftDelta);
            loc.shift(shiftDelta);
            if (shiftDelta.getDx() == 0 && shiftDelta.getDy() == 0) {
            	// These two images overlapping exactly on the edge
            	int direction = io.getLocation().getDirectionFor(getLocation());
            	loc.shiftInDirection(direction, 2);
            }
        }
        return loc;
    }

    /**
     * @return the closest location to loc this craft can occupy that does not overlap with io passed in.
     * NOTE - This method preserves direction 
     * NOTE - if loc is at the centre of io then we cannot get a direction so we use the this objects location 
     * NOTE - This location may be occupied by another object!
     */
	public Location getClosestLocationTo(Location loc, ServerImageObject io) throws ApplicationError {
		Location newLoc = io.getLocationClone();
		int dir = loc.equals(io.getLocation()) ? newLoc.getDirectionFor(getLocation()) : newLoc.getDirectionFor(loc);
		newLoc.shiftInDirection(dir, getMaxSpaceDimension() + io.getMaxSpaceDimension());
		
		Area area = io.getArea();
        while (area.intersects(getAreaAt(newLoc))) {
    		newLoc.shiftInDirectionOneSpace(dir);
        }
        
        return newLoc;
    }

	public Location getClosestLocationTo(Location loc, ServerImageObject io, int maxShiftDistance) throws ApplicationError {
		Location l = getClosestLocationTo(loc, io);
		if (l.getDistanceTo(loc) > maxShiftDistance) {
			return null;
		} else {
			return l;
		}
	}
	
	/**
	 * NOTE - The location returned may be occupied by another object
	 * @param moveToLoc The destination that we want to find a location for this object next too.
	 * @param occupier Another object that intersects the area occupied by this object if it were at acualDest (another craft that's in the way).
	 * @param dir1 A direction to try
	 * @param dir2 A direction to try
	 * @param targetDest The target destination this craft wants to move to.
	 * @param closestFreeSpaceInStraightLine 
	 * @return The closest free location in direction dirDelta1 or dirDelta12 to actualDest that does not intersect serverImageObject.
	 * @throws ApplicationError 
	 */
	public Location getClosestFreeLocationToInDirections(Battle battle, BattleObject craft, Location moveToLoc, ServerImageObject occupier, int dir1, int dir2, Location targetDest, MovementCostCalculator mcc) throws ApplicationError {
		
		if (targetDest == null) {
			return null;
		}
		
		Location newLoc1 = new Location(moveToLoc);
		int d1 = occupier.getOverlapInDirection(moveToLoc, craft, dir1);
		newLoc1.shiftInDirection(dir1, d1);
		ServerImageObject occupier1 = occupier;
		
		Location newLoc2 = new Location(moveToLoc);
		int d2 = occupier.getOverlapInDirection(moveToLoc, craft, dir2);
		newLoc2.shiftInDirection(dir2, d2);
		ServerImageObject occupier2 = occupier;
		
		Area occupiedArea1 = occupier1.getArea();
		Area occupiedArea2 = occupier2.getArea();
		int maxDistance = 4;
		int distance = 1;
        while (distance < maxDistance) {
        	Location r1 = null;
        	Location r2 = null;
    		newLoc1.shiftInDirectionOneSpace(dir1);
    		if (!occupiedArea1.intersects(getAreaAt(newLoc1))) {
    			occupier1 = battle.getClosestLocationOccupier(newLoc1, this);
    			if (occupier1 == null && mcc.canMoveTo(craft, newLoc1)) {
    				r1 = newLoc1;
    			} else if (occupier1 != null) {
    				occupiedArea1 = occupier1.getArea();
    			}
    		}
    		newLoc2.shiftInDirectionOneSpace(dir2);
    		if (!occupiedArea2.intersects(getAreaAt(newLoc2))) {
    			occupier2 = battle.getClosestLocationOccupier(newLoc2, this);
    			if (occupier2 == null && mcc.canMoveTo(craft, newLoc2)) {
    				r2 = newLoc2;
    			} else if (occupier2 != null){
    				occupiedArea2 = occupier2.getArea();
    			}
    		}
    		
			if (r1 != null && r2 != null) {
				int r1ToTarget = r1.getDistanceTo(targetDest);
				int r2ToTarget = r2.getDistanceTo(targetDest);
				if (r1ToTarget <= r2ToTarget) {
					return r1;
				} else {
					return r2;
				}
			} else if (r1 != null) {
				return r1;
			} else if (r2 != null) {
				return r2;
    		}
			
    		distance++;
    	}
        
        return null;
	}

    /**
     * @return true if this craft were located at loc and loc is too close to the 
     * craft other. Otherwise it returns false.
     */
    public boolean isTooClose(Location loc, ServerImageObject other) throws ApplicationError {
        int dist = loc.getDistanceTo( other.getLocation() );
        int minDist = getSpaceX() + other.getSpaceX();
        //Logger.logDebug("isTooClose "+getName()+" "+other.getName()+" => "+dist+" "+minDist);
        return (dist < minDist);
    }

	public boolean isInside(Location l) throws ApplicationError {
		return isInside(l.getX(), l.getY());
	}

	public boolean isInside(int x, int y) throws ApplicationError {
		Size size = getSize();
		int width = size.getX();
		int height = size.getY();
		int left = getLocation().getX() - width/2;
		int top = getLocation().getY() - height/2;
		int right = left + width;
		int bottom = top + height;
		return (x >= left && x <= right && y >= top && y <= bottom);
	}
	
	public boolean isInsideArea(Location l) throws ApplicationError {
		return isInsideArea(l.getX(), l.getY());
	}

	public boolean isInsideArea(int x, int y) throws ApplicationError {
		int width = getSpaceX();
		int height = getSpaceY();
		int left = getLocation().getX() - width;
		int top = getLocation().getY() - height;
		int right = left + width*2;
		int bottom = top + height*2;
		return (x >= left && x <= right && y >= top && y <= bottom);
	}
	
	/**
	 * @return the area occupied by this craft on the battle map.
	 */
	public Area getArea() throws ApplicationError {
		return getAreaAt(getLocation());
	}

    /**
     * Same as getAreaAt(int x, int y)
     */
	public Area getAreaAt(Location loc) throws ApplicationError {
        if (loc == null) {
            return null;
        } else {
            return getAreaAt(loc.getX(), loc.getY());
        }
    }

	/**
	 * @return the area occupied by this craft - If this craft was at this location.
     * This is not the size of the image - Space is left around the image of each craft
     * to display shields and a selected rectangle.
	 */
	public Area getAreaAt(int x, int y) throws ApplicationError {
        Size size = getSize();
        int left = x-getSpaceX();
        int right = x+getSpaceX();
        int top = y-getSpaceY();
        int bottom = y+getSpaceY();
        return new Area(left, top, right, bottom);
	}


	/**
	 * @return the rectangle occupied by this craft on the battle map.
	 */
	public Rectangle getRectangle() throws ApplicationError {
		Location loc = getLocation();
		int x = loc.getX();
		int y = loc.getY();
        Size size = getSize();
        int left = x-getSpaceX();
        int right = x+getSpaceX();
        int top = y-getSpaceY();
        int bottom = y+getSpaceY();
		return new Rectangle(left, top, right-left, bottom-top);
	}

	/**
     * @return the space this craft requires on the map. This returns the distance 
     * between its X location the closest any other crafts SpaceX can be to this craft.
     */
    public int getSpaceX() throws ApplicationError {
        Size size = getSize();
        return size.getX()/2+ImageObjectSettings.INTER_CRAFT_GAP;
    }

    /**
     * @return the space this craft requires on the map. This returns the distance 
     * between its Y location the closest any other crafts SpaceY can be to this craft.
     */
    public int getSpaceY() throws ApplicationError {
        Size size = getSize();
        return size.getY()/2+ImageObjectSettings.INTER_CRAFT_GAP;
    }

    /**
     * Calculates the space in any direction that this object requires measured from the centre of the object (its location)
     * to its outer edge in either the X or Y coordinate.
     * NOTE - This is not the edge of the image - its the edge of the space occupied by this craft including the inter craft gap.
     * @return The maximum of getSpaceX() and getSpaceY()
     * @throws ApplicationError
     */
    public int getMaxSpaceDimension() throws ApplicationError {
    	int spaceX = getSpaceX();
    	int spaceY = getSpaceY();
    	if (spaceX > spaceY) {
    		return spaceX;
    	} else {
    		return spaceY;
    	}
    }

    /**
     * 
     * @param moveToLoc The location craft to move to.
     * @param craft The craft we want to move to moveToLoc and calculate the overlap
     * @param dir The direction we want to calculate the overlap when craft move to location moveTo
     * @return The distance moveToLoc must shift in direction dir so that the image would not overlap this crafts image.
     * @throws ApplicationError
     */
	public int getOverlapInDirection(Location moveToLoc, ServerImageObject craft, int dir) throws ApplicationError {
        Location loc = getLocation();        
        
    	if (dir >= 45 && dir < 135) {
            int right = loc.getX() + getSpaceX();
            int left = moveToLoc.getX() - craft.getSpaceX();
    		return right - left;
    	} else if (dir >= 135 && dir < 225) {
            int top = moveToLoc.getY() - craft.getSpaceY();
            int bottom = loc.getY() + getSpaceY();
    		return bottom - top;
    	} else if (dir >= 225 && dir < 315) {
            int right = moveToLoc.getX() + craft.getSpaceX();
            int left = loc.getX() - getSpaceX();
    		return right - left;
    	} else {
            int top = loc.getY() - getSpaceY();
            int bottom = moveToLoc.getY() + craft.getSpaceY();
    		return bottom - top;
    	} 
	}

    public int getDistanceToEdgeFrom(int x, int y, int dir) throws ApplicationError {
        int sx = getSpaceX();
        int sy = getSpaceY();
        Location loc = getLocation();
    	if (dir >= 45 && dir < 135) {
    		return loc.getX() + sx - x;
    	} else if (dir >= 135 && dir < 225) {
    		return loc.getY() + sy - y;
    	} else if (dir >= 225 && dir < 315) {
    		return x - (loc.getX() - sx);
    	} else {
    		return y - (loc.getY() - sy);
    	} 
	}
    
	public abstract Size getSize() throws ApplicationError;
    
    /**
     * Calculates the gap required between this object and the object passed in.
     * @return The minimum distance between this object and the object passed in before the images start overlapping.
     */
    public int getMinDistanceTo(ServerImageObject other) throws ApplicationError {
    	return getMaxSpaceDimension() + other.getMaxSpaceDimension();
    }
    
    /**
     * @return the average of the image height and image width.
     */
    public int getImageRadius() throws ApplicationError {
        Size size = getSize();
        return (size.getX() + size.getY())/2;
    }

    /**
     * @return the width of the image to display on the screen.
     */
    public int getWidth() throws ApplicationError {
        Size size = getSize();
        return size.getX();
    }

    /**
     * @return the height of the image to display on the screen.
     */
    public int getHeight() throws ApplicationError {
        Size size = getSize();
        return size.getY();
    }

    /**
     * @return the height of the image to display on the screen.
     */
    public int getMaxOfHeightAndWidth() throws ApplicationError {
        Size size = getSize();
        if (size.getY() > size.getX()) {
        	return size.getY();
        } else {
        	return size.getX();
        }
    }

	public LocationAndDirection getLocationAndDirection() throws ApplicationError { 
		return new LocationAndDirection(getLocation(), getDirection()); 
	}

    /**
     * @return The direction this object is pointing. A value between 0 and 359.
	 * A value of zero implies the craft is pointing North, and then it is measured 
     * in positive degrees.
     * @throws ApplicationError 
     */
    public abstract int getDirection() throws ApplicationError;
    
    /**
     * @return The direction this object would be pointing if it was located at loc. 
     * A value between 0 and 359. A value of zero implies the craft is pointing North, 
     * and then it is measured in positive degrees.
     * NOTE - Objects like missile do not have a direction and always point to the target.
     * Therefore for such objects we need to know what direction there would be pointing if
     * they were located in a different position then their current location
     * @throws ApplicationError 
     */
    public int getDirection(Location loc) throws ApplicationError {
        return getDirection();
    }
    
    
    /**
     * @param player The player who may be trying to select this object on the screen
     * @return True if the player passed in can select this object in the window.
     * @throws ApplicationError
     */
    public boolean isSelectableBy(Player player) throws ApplicationError {
        if (this instanceof BattleObject) {
            BattleObject bo = (BattleObject) this;
            return bo.isOwnedBy(player);
        } else {
            return false;
        }
    }
    
}