/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.common;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Iterator;

import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.bean.NamedLocatedObject;
import mot.common.bean.Nebula;
import mot.common.bean.Size;
import mot.common.bean.StellarObjectGraphics;
import mot.common.designs.Orbital;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;
import mot.engine.prizes.BoraliumPrize;
import mot.engine.prizes.CraftPrize;
import mot.engine.prizes.MoneyPrize;
import mot.engine.prizes.Prize;
import mot.engine.prizes.RelicPrize;
import mot.engine.prizes.TechnologyPrize;
import mot.engine.relations.Relations;

/**
 * This is the base class for all objects that are visible on the galaxy map.
 * All stellar objects are also a destination of MoveableOwnedObjects (such as 
 * crafts, fleets, monsters etc).
 * 
 * Two main types of stellar object are
 *  Solar System
 *  Space Station
 *
 * Usually a stellar object has a single owner, however some stellar objects
 * such as solar systems can have multiple owners. Players are not allowed to colonise
 * a solar system if it already has a colony from another player, but, a player can 
 * invade a planet making the solar system shared in ownership. Other stellar objects
 * like space stations can only have a single owner.
 * 
 * A Space station will always have a single owner, but a solar system may have no owner,
 * one owner, or more than one owner.
 */
public abstract class StellarObject extends NamedLocatedObject {

    /**
     * This is a list of all players who have visited this stellar object.
     * This is a list of Visitor objects.
     */
    private ArrayList visitors;
    
    /**
     * This is a list of artifacts, surprises (good and bad) that will be released
     * or given to the first player that visits this object.
     * This is a list of Prize objects.
     */
    private ArrayList prizes;
    
    /**
     * This is an orbital associated with a stellar object instead of a planet. It is used
     * to extend the range of crafts and act as a listening post (contains scanners and scans
     * the space around it).
     */
    private StellarObjectOrbital orbital;

    /**
     * This is the orbit for objects around this planet. This is dynamically generated as an 
     * orbit is defined as a stellar object and an optional planet. Not that orbits around 
     * a planet do not use the same instance - so do not use equality operator to compare!
     * I have defined this to reduce the quantity of construction of class Orbit. e.g. When
     * a craft wishes to move to a new location can use the orbit of the planet or stellar 
     * object it is going to.
     */
    private transient Orbit orbit;

    public Orbit getOrbit() { 
    	if (orbit == null) orbit = new Orbit(this, null); 
    	return orbit; 
    }
    
    public StellarObject() {
    }

	public StellarObject(String name, int x, int y) {
		super(name, x, y);
	}
    
    /**
     * @return true if and only if this stellar object has one or more owners.
     */
    public abstract boolean hasOwner() throws ApplicationError;

    /**
     * @return true if player is the owner of this stellar object.
     */
	public abstract boolean isOwnedBy(Player player) throws ApplicationError;

	public boolean isOwnedByAnyOf(Player [] players) throws ApplicationError {
		if (players == null) return false;
		for (int i = 0; i < players.length; i++) {
			if (isOwnedBy(players[i])) return true;
		}
		return false;
	}
	
    /**
     * @return the list of owners of this stellar object.
     */
    public abstract Player [] getOwners() throws ApplicationError;

    /**
     * @return the name of the bitmap used to display this solar system on the map.
     */
    public abstract StellarObjectGraphics getStellarObjectGraphics() throws ApplicationError;

    /**
     * Generates new reference to this stellar object.
     * So a solar system would generate a new SolaarSystemReference etc.
     */
    public abstract StellarObjectReference getObjectReference();
    
    /**
     * @return the strength of the scanners placed in this stellar object.
     * This determines how far out this planets view of the galaxy extends.
     */
	public int getScanStrength(int distance, Player player) throws ApplicationError {
        int bestStrength = 0;
        if (this instanceof SolarSystem) {
            Planet [] planets = ((SolarSystem)this).getPlanets();
            if (planets != null) {
                for (int i=0; i<planets.length; i++) {
                    if (planets[i].isOwnedBy(player)) {
                        int v = planets[i].getScanStrength(distance);
                        if (v > bestStrength) bestStrength = v; 
                    }
                }
            }
        }
		if (orbital != null && orbital.isOwnedBy(player)) {
            int v = orbital.getScanStrength(distance);
                if (v > bestStrength) {
                    bestStrength = orbital.getScanStrength(distance); 
                }
		}
        return bestStrength;
	}

    /**
     * @return the strength of the scanners placed on this planet at location loc.
     * This determines how powerful this planets view of this galaxy location.
     */
	public int getScannerStrengthAt(Location loc, Player player) throws ApplicationError {
		int v = getScanStrength(getLocation().getDistanceTo(loc), player);
		if (v < 0) v = 0;
		return v;
	}
    
    /**
     * Places a satellite in orbit around this stellar object. The stellar object will
     * be owned by this player because the player will be assumed to have an outpost 
     * at this location.
     * @param orbital The satellite to deploy at this stellar object
     */
    public void deploy(Orbital orbital, Player owner) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("StellarObject.deploy() StellarObject "+getName()+" has outpost "+orbital.getName());
        }
        
        if (orbital.getHullTech().isOutpost()) {
        	setOrbital(new Outpost(orbital, owner));
        } else {
        	setOrbital(new SpaceStation(orbital, owner));
        }
        
        owner.flushCache();
    }
    
    /**
     * Removes the outpost located at this stellar object. It then removes this
     * from the owners list of stellar objects (if required).
     * @param owner This must be the owner of the outpost at this stellar object.
     */
    public void destroyOutpost(Player owner) {
        if (Logger.isDebug()) {
            Logger.logDebug("StellarObject.destroyOutpost() at so="+getName());
        }
        
        setOrbital(null);
        owner.flushCache();
    }

    /**
     * @return The colour this stellar objects name should be painted. This is defined as the
     * colour of the owner or white if there is no owner
     */
    public Color getTextColor() throws ApplicationError {
        Color color = Color.WHITE;
        //PropertyManager.getMandatoryAsColor("window.color.galaxy.map.text");
        if (hasOwner()) {
            color = getOwners()[0].getColor().getColor();
        }
        return color;
    }
    
    /**
     * @return The area on the galaxy map occupied by this object.
     * @throws ApplicationError
     */
    public Area getArea() throws ApplicationError {
    	Size size = getStellarObjectGraphics().getSize();
    	Location loc = getLocation();
    	return new Area(loc, size);
    }

    /**
     * @param p The player who may have an enemy
     * @return Returns true if the player p has an enemy in among the owners of this
     * stellar object.
     */
    public boolean hasEnemyOwner(Player p) throws ApplicationError {
        if (!hasOwner()) {
            return false;
        }
        Player [] enemies = getOwners();
        Relations rels = MotEngine.instance.getRelations();
        for (int i=0; i<enemies.length; i++) {
            if (rels.isWar(enemies[i], p)) {
                return true;
            }
        }
        return false;
    }
    
    public abstract boolean hasDefendedEnemyOwner(Player p) throws ApplicationError;
    
    /**
     * A point is assumed to be contained if it lays on one of the lines.
     * @return True if this contains the the point x,y.
     * @throws ApplicationError 
     */
	public boolean isInside(Location tLoc) throws ApplicationError {
		int x = tLoc.getX();
		int y = tLoc.getY();
		return isInside(x,y);
	}
	
    /**
     * A point is assumed to be contained if it lays on one of the lines.
     * @return True if this contains the the point x,y.
     * @throws ApplicationError 
     */
	public boolean isInside(int x, int y) throws ApplicationError {
        Size size = getStellarObjectGraphics().getSize();
        Location loc = getLocation();
        int top = loc.getY() - size.getY()/2;
        int left = loc.getX() - size.getX()/2;
        int bottom = top + size.getY();
        int right = left + size.getX();
		return (x >= left && x <= right && y >= top && y <= bottom);
	}

	/**
	 * @return True iff this stellar object is inside a nebula
	 */
	public boolean isInNebula() {
		return getNebula() != null;
	}

	/**
	 * @return The nebula if this stellar object is inside a nebula otherwise returns null
	 */
	public Nebula getNebula() {
		if (MotEngine.instance == null) {
			return null;
		}
		Location location = getLocation();
		Nebula nebula = MotEngine.instance.getStellarObjectsNotNull().getNebula(location);
		return nebula;
	}
	
	/**
	 * This is the list of objects that this black hole may damage in its current 
	 * location (as it passes through the galaxy). Note - it may damage several
	 * stellar objects as it passes through.
	 * @return All stellar objects (not including this one) that are inside the area occupied by 
	 * this black hole.
	 * @throws ApplicationError 
	 */
    public StellarObject [] getIntersectingStellarObjects() throws ApplicationError {
    	StellarObject [] sos = MotEngine.getInstance().getStellarObjectsNotNull().getStellarObjects();
    	ArrayList candidates = null;
		for (int i = 0; i < sos.length; i++) {
			if (sos[i] != this && getArea().intersects(sos[i].getArea())) {
				if (candidates == null) candidates = new ArrayList();
				candidates.add(sos[i]);
			}
		}
		if (candidates == null || candidates.size() == 0) {
			return null;
		} else {
			return (StellarObject[]) candidates.toArray(new StellarObject [candidates.size()]);
		}
	}

	/**
     * Used by XMLDocumentHandler to load saved games.
     */
    public void setVisitor(Visitor v) {
        if (visitors == null) visitors = new ArrayList();
        visitors.add(v);
    }

    /**
     * Same as addVisitor(Player, int) with qtySpies equal to 0.
     */
    public void addVisitor(Player player, boolean releasePrizes) throws ApplicationError {
        addVisitor(player, 0, releasePrizes);
    }
    
    /**
     * Adds the player as a visitor to this object at the current date. If this player has
     * already visited this object then the date of visit is updated to the current date (and
     * no new visit is created).
     * This method also releases any prizes at this stellar object (see Prize for details).
     */
    public void addVisitor(Player player, int qtySpies, boolean releasePrizes) throws ApplicationError {
    	if (is("Bunya")) {
    		Logger.logDebug("");
    	}
        Visitor v = getVisitor(player);
        if (v == null) {
            v = new Visitor(player, MotEngine.getInstance().getDate());
            setVisitor(v);
        } else {
            v.setDate(MotEngine.getInstance().getDate());
        }
        if (qtySpies > 0) {
            v.addSpies(qtySpies);
        }
        if (prizes != null && releasePrizes) {
            Iterator i = prizes.iterator();
            while (i.hasNext()) {
                Prize p = (Prize) i.next();
                i.remove(); // CraftPrize will call arriveAt() which will call this causing an infinite loop without this
                p.release(player, this);
            }
            prizes = null;
        }
    }

    /**
     * Remove visitor object in this stellar object for player p
     */
    public void removeVisitor(Player p) {
        Visitor v = getVisitor(p);
        if (v != null) {
            visitors.remove(v);
        }
    }

	/**
	 * @return the visit object of the time the player passed in visited this object. It the
     * player has not visited this object it will return null.
	 */
	protected Visitor getVisitor(Player player) {
		if (visitors == null) return null;
        Iterator i = visitors.iterator();
        while (i.hasNext()) {
            Visitor v = (Visitor) i.next();
            if (v.getPlayer() == player) {
                return v;
            }
        }
        return null;
	}
    
	/**
	 * @return the date of the time the player passed in visited this object. It the
     * player has not visited this object it will return -1.
	 */
	public int getLastVisitedDate(Player player) {
		Visitor v = getVisitor(player);
        if (v != null) {
            return v.getDate();
        } else {
            return -1;
        }
	}
    
	/**
	 * @return the quantity of spies assigned to this stellar object by player.
	 */
	public int getQtySpies(Player player) {
		Visitor v = getVisitor(player);
        if (v != null) {
            return v.getQtySpies();
        } else {
            return 0;
        }
	}
    
	/**
	 * @return the visibility of this stellar object by player at this point in time.
     * The visibility of a craft is determined by the closest scanner strength 
     * (on a planet or a craft). Note that shields do not have the effect of cloaking 
     * a planet!
	 */
	public int getVisibility(Player player) throws ApplicationError {
		return player.getScannerStrengthAt(getLocation());
	}

	/**
     * When a stellar object can be seen by a player then that player can double click on the
     * stellar object to view its details.
     * 
     * A player can view a stellar object if a) Its the owner b) Its within scanning range or
     * c) Has visited the object.
     * 
     * This should not be confused with if a planet is viewable. If a solar system is visible then
     * the planets may be viewed, but not the population/production/defense details. If a planet is 
     * viewable then all details of the planet may be viewed.
     * 
	 * @return true if this stellar object can be seen by this player at this point in time.
	 */
	public boolean isVisible(Player player) throws ApplicationError {
		return (isOwnedBy(player)) || 
               (getLastVisitedDate(player) >= 0) ||
               (getVisibility(player) > 0);
	}

    /**
     * @return A list of all player names for that isVisible(Player) would return true. 
     */
    public String [] getVisibleToPlayers() throws ApplicationError {
        Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
        ArrayList list = null;
        for (int i=0; i<players.length; i++) {
            if (isVisible(players[i])) {
                if (list == null) list = new ArrayList();
                list.add(players[i].getName());
            }
        }
        if (list == null) {
            return null;
        } else {
            return (String []) list.toArray(new String[list.size()]);
        }
    }

    /**
     * Adds a new prize to the list of prizes at this object;
     */
    public void setPrize(Prize v) {
        if (prizes == null) prizes = new ArrayList();
        if (v != null) prizes.add(v);
    }

    public void clearAndSetPrize(Prize v) {
        prizes = new ArrayList();
        if (v != null) prizes.add(v);
    }

    /**
     * @return the array of prizes;
     */
    public Prize [] getPrizes() {
        if (prizes == null) {
            return null;
        } else {
            return (Prize []) prizes.toArray(new Prize[prizes.size()]);
        }
    }
    
	public void changeLocationTo(int xp, int yp) throws ApplicationError {
		Craft[] ca = MotEngine.instance.getCrafts().getAt(this);
		setLocation(new Location(xp,yp));
		for (int i = 0; ca != null && i < ca.length; i++) {
			ca[i].setLocation(new Location(xp,yp));
			if (ca[i].getFleet() != null) {
				ca[i].getFleet().setLocation(new Location(xp,yp));
			}
		}
		MotEngine.instance.getInfluence().update();
	}

  	/**
	 * Performs any operations these objects need to do at the end of every turn.
     * This method does not do anything but may be overriden by stellar objects that
     * want to perform operations at the end of every turn.
  	 * @throws ApplicationError 
	 */
	public void endTurn() throws ApplicationError {
    }

    public void setCraftPrize(CraftPrize v) { setPrize(v); }
    public void setMoneyPrize(MoneyPrize v) { setPrize(v); }
    public void setTechnologyPrize(TechnologyPrize v) { setPrize(v); }
    public void setBoraliumPrize(BoraliumPrize v) { setPrize(v); }
    public void setRelicPrize(RelicPrize v) { setPrize(v); }

	protected void encode(XMLWriter writer) throws ApplicationError {
        super.encode(writer);
        if (visitors != null) {
            Iterator i = visitors.iterator();
            while (i.hasNext()) {
                Visitor v = (Visitor) i.next();
                v.toXML(writer);
            }
        }
        if (prizes != null) {
            Iterator i = prizes.iterator();
            while (i.hasNext()) {
                Prize v = (Prize) i.next();
                v.toXML(writer);
            }
        }
        if (orbital != null) {
        	orbital.toXML(writer);
        }
    }

    public StellarObjectOrbital getOrbital() {  return orbital;   }
    public void setOrbital(StellarObjectOrbital orbital) {  this.orbital = orbital;  }
    public void setOutpost(Outpost orbital) {  this.orbital = orbital;   }
    public void setSpaceStation(SpaceStation orbital) {  this.orbital = orbital;   }

}