/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.common;

import mot.common.bean.CraftGraphics;
import mot.common.designs.Orbital;
import mot.common.factory.GraphicsFactory;
import mot.common.technology.ExtendedSpaceJump;
import mot.common.technology.Hull;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;
import mot.common.xml.XMLWriter;
import mot.engine.planets.InfluenceGenerator;
import mot.engine.players.Player;

/**
 * An outpost is an orbital that is associated with a stellar object instead of a
 * planet and is used to extend the range of crafts and a players scanners.
 */
public abstract class StellarObjectOrbital extends OwnedObject implements InfluenceGenerator {

	private transient String image;
    
    private Orbital orbital;

    public StellarObjectOrbital(Orbital orbital, Player owner) {
        super(owner);
        this.orbital = orbital;
    }

    public StellarObjectOrbital() {
    }

    /**
     * @return the graphics used by this craft on the battle map and on the galaxy map.
     * This is changed in a transient attribute graphics and dynamically generated from
     * image.
     */
	public CraftGraphics getGraphics() throws ApplicationError {
		return GraphicsFactory.getStationGraphics(getOwner().getName(), orbital.getHullTech());
	}

    /**
     * @return the strength of the scanners placed on this planet.
     * This determines how far out this planets view of the galaxy extends.
     */
	public int getScanStrength() throws ApplicationError {
    	return orbital.getScanStrength();
	}

    /**
     * @return the strength of the scanners placed on this planet.
     * This determines how far out this planets view of the galaxy extends.
     */
	public int getScanStrength(int distance) throws ApplicationError {
        int v = getScanStrength() - distance;
        if (v < 0) v = 0;
        return v;
	}

	public int getQtyFighterBays() throws ApplicationError {
		return orbital.getQtyFighterBays();
	}

	public int getQtySentinals() throws ApplicationError {
		return orbital.getQtySentinals();
	}

	public boolean hasSpaceJumpAbility() throws ApplicationError {
		Player owner = getOwner();
		ExtendedSpaceJump extendedSpaceJump = Technologies.getMiscTechnologies().getExtendedSpaceJump();
		return owner != null && owner.hasDiscovered(extendedSpaceJump);
	}

    protected void encode(XMLWriter writer) throws ApplicationError {
        if (orbital != null) orbital.toXML(writer);
        super.encode(writer);
    }


    public void setOrbital(Orbital orbital) {
        this.orbital = orbital;
    }


    public Orbital getOrbital() {
        return orbital;
    }

	public Hull getHull() throws ApplicationError {
		if (orbital != null) {
			return orbital.getHullTech();
		} else {
			return null;
		}
	}

}