/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.crafts;

import mot.common.bean.Area;
import mot.common.bean.CraftGraphics;
import mot.common.bean.Location;
import mot.common.bean.RaceGraphicsList;
import mot.common.bean.Size;
import mot.common.designs.CraftDesign;
import mot.common.designs.WeaponBaysDesign;
import mot.common.factory.GraphicsFactory;
import mot.common.lang.L;
import mot.common.leaders.GeneralLeader;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.technology.Hulls;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Specials;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.Battles;
import mot.engine.battles.Bombardment;
import mot.engine.battles.CraftStateFactory;
import mot.engine.common.MoveableOwnedObjectReference;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.event.LinkedEventFactoryBase;
import mot.engine.leaders.EngineGeneralLeader;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;
import mot.engine.players.generals.CraftTask;

/**
 * This is a Craft in the game. All Crafts are a subclass of this class, including 
 * military, colony, user and computer crafts.
 * 
 * Note that Craft contains its own list of Weapons, Shields, Hull, Engine etc and does
 * not depend on a craft design. This is to allow crafts to be easily modified during
 * a game by a player wishing to recycle crafts by upgrading instead of building new ones.
 */
public class Craft extends CraftBase {

	public static final double DEFAULT_BUILD_QUALITY = 1.0;
	
	/**
	 * The graphic used by this craft design.
	 * These are dynamically loaded from the GraphicsFactory and are not part of the
	 * serialisable state.
	 */
	private transient CraftGraphics graphics;

    /**
     * This is the name of the bitmap used to display this craft for this player on the screen.
     */
    private transient String image = null;
    
    /**
     * This is the size of the bitmap used to display this craft for this player on the screen.
     */
    private transient Size imageSize = null;
    
	/**
	 * The fleet this craft is part of (if any).
	 */
	private FleetReference fleetReference;

	/**
	 * The hull of this craft.
	 */
	private Hull hull;

	/**
	 * The computers mounted on this craft.
	 */
	private Computers computers;

	/**
	 * The weapons mounted on this craft.
	 */
	private WeaponBays weaponBays;

	/**
	 * The shields mounted on this craft.
	 */
	private Shields shields;

	/**
	 * The scanners mounted on this craft.
	 */
	private Scanners scanners;

	/**
	 * The transporters etc. mounted on this craft.
	 */
	private Specials specials;

	/**
	 * The damage that has been inflicted on this craft.
	 * The craft is destroyed if this value is higher than getMaxDamage().
	 */
	private double damage;
	
	/**
	 * This is the condition of the craft. This value effects the condition of a craft in
	 * a battle. Crafts with a low build quality start a battle with a lower condition.
	 * The condition of a craft in a battle effects its speed and weapon damage. 
	 */
	private double buildQuality;
	
	/**
	 * The name of the design that created this craft.
	 */
	private String designName;
    
	public Craft() {
	}

	public Craft(Player owner, Player origin, CraftDesign design, Orbit orbit) throws ApplicationError {
        this(owner, design, orbit, DEFAULT_BUILD_QUALITY);
        setOrigin(origin);
    }

	public Craft(Player owner, Player origin, CraftDesign design, Orbit orbit, double buildQuality) throws ApplicationError {
        this(owner, design, orbit, buildQuality);
        setOrigin(origin);
    }
	
	public Craft(Player owner, CraftDesign design, Orbit orbit) throws ApplicationError {
        this(owner, design, orbit, DEFAULT_BUILD_QUALITY);
	}

	public Craft(Player owner, CraftDesign design, Orbit orbit, double buildQuality) throws ApplicationError {
		this(getNewName(design.getName()), owner, design, orbit, buildQuality);
	}
	
	public Craft(String newCraftName, Player owner, CraftDesign design, Orbit orbit, double buildQuality) throws ApplicationError {
		super(newCraftName, owner, design.getEngineTech(), orbit);
		this.hull = design.getHullTech();
		if (design.getComputers() != null) {
            this.computers = new Computers(design.getComputers());
        }
        if (design.getWeaponBays() != null) {
            this.weaponBays = new WeaponBays(design.getWeaponBays());
        }
        if (design.getShields() != null) {
            this.shields = new Shields(design.getShields());
        }
        if (design.getScanners() != null) {
            this.scanners = new Scanners(design.getScanners());
        }
        if (design.getSpecials() != null) {
            this.specials = new Specials(design.getSpecials());
        }
        this.buildQuality = buildQuality;
        this.designName = design.getName();
	}

	public FleetReference getFleetReference() { return fleetReference; }
	public Fleet getFleet() throws ApplicationError {
		if (fleetReference == null) {
			return null;
		} else {
			return fleetReference.getFleet();
		}
	}

    /**
     * @return The speed bonuses for this craft based upon the specials and player advantages.
     */
    public double getSpeedBonus() throws ApplicationError {
        double playerBonus = 1.0 + getOrigin().getOriginSpeedBonus() + getOwner().getOwnerSpeedBonus();
        if (specials != null && specials.getSpeedBonusAsDouble() != null) {
            double specialsBonus = 1.0 + specials.getSpeedBonusAsDouble().doubleValue();
            return specialsBonus * playerBonus;
        } else {
            return playerBonus;
        }
    }

    public String getSpeedBonusDescription() throws ApplicationError {
        double playerBonus = getOrigin().getOriginSpeedBonus();
        double leaderBonus = 0;
        EngineGeneralLeader generalLeader = getOwner().getGeneralLeader();
        if (generalLeader != null && generalLeader.hasSpeedBonus()) {
            leaderBonus = generalLeader.getSpeedBonus();  
        }
        double resourceBonus = getOwner().getResourceCache().getSpeedBonus();
        double specialsBonus = 0.0;
        if (specials != null && specials.getSpeedBonusAsDouble() != null) {
            specialsBonus = specials.getSpeedBonusAsDouble().doubleValue();
        }
        return  " + "+L.SPECIALS_BONUS+" "+Formatter.percentLabelNoSign(specialsBonus) +
        		" + "+L.PLAYER_BONUS+" "+Formatter.percentLabelNoSign(playerBonus) +
        		" + "+L.RESOURCE_BONUS+" "+Formatter.percentLabelNoSign(resourceBonus) +
        		" + "+L.LEADER_BONUS+" "+Formatter.percentLabelNoSign(leaderBonus) ;
    }
    
	public Hull getHull() { return hull; }
	public Computers getComputers() { return computers; }
	public WeaponBays getWeaponBays() { return weaponBays; }
	public Shields getShields() { return shields; }
	public Scanners getScanners() { return scanners; }
	public Specials getSpecials() { return specials; }

	public WeaponBaysDesign getWeaponBaysDesign() {
		if (weaponBays == null) {
			return null;
		} else {
			return weaponBays.getWeaponBaysDesign();
		}
	}

	public void setFleetReference(FleetReference v) { fleetReference = v; }

	public void setHull(Hull v) { hull = v; }
	public void setComputers(Computers v) { computers = v; }
	public void setWeaponBays(WeaponBays v) { weaponBays = v; }
	public void setShields(Shields v) { shields = v; }
	public void setScanners(Scanners v) { scanners = v; }
	public void setSpecials(Specials v) { specials = v; }
    public void setImage(String v) { image = v; }
    
    /**
     * @return the location of this craft
     * Note - This overrides Moveable Owned Object getLocation() because when a craft 
     * is part of a fleet its location is not used.
     */
	public Location getLocation() throws ApplicationError {
        if (fleetReference != null) {
            return fleetReference.getFleet().getLocation();
        } else {
            return super.getLocation(); 
        }
    }
    
    /**
     * @return The location of this craft
     * Note - This overrides Moveable Owned Object getOrbit() because when a craft 
     * is part of a fleet its orbit is not used.
     */
	public Orbit getOrbit() throws ApplicationError {
        if (fleetReference != null) {
            return fleetReference.getFleet().getOrbit();
        } else {
            return super.getOrbit(); 
        }
    }

    /**
     * @return the navigator for this craft
     * Note - This overrides MoveableOwnedObject.getNavigator() because when a craft 
     * is part of a fleet its orbit is not used.
     */
	public Navigator getNavigator() throws ApplicationError {
        if (fleetReference != null) {
            return fleetReference.getFleet().getNavigator();
        } else {
            return super.getNavigator(); 
        }
    }

    /**
     * Note If this craft is assigned to a fleet this will be the task of the fleet.
     * @return The task to which this craft has been assigned or null of this craft does not 
     * have a task. Note only computer players have a task - see CraftTask for details.
     * @throws mot.common.util.ApplicationError
     */
    public CraftTask getTask() throws ApplicationError {
        if (fleetReference != null) {
            return fleetReference.getFleet().getTask();
        } else {
            return super.getTask(); 
        }
    }

	public MoveableOwnedObjectReference createNewReference() {
		return new CraftReference(this);
	}

	/**
	 * @return true if this ship can colonise a planet (i.e. is a colony ship).
	 */
	public boolean canColonise() {
        Specials s = getSpecials();
        if (s == null) return false;
		return (s.getCanColoniseRating() != null);
	}

	/**
	 * @return true if this ship has a colony module capable of colonising this planet.
     * It does not check that the planet is in a solar system the owner can reach or
     * does not have an owner.
	 */
	public boolean canColonise(Planet planet) {
        Specials s = getSpecials();
        if (s == null) return false;
		Double x = s.getCanColoniseRating();
        if (x == null) return false;
		double canColoniseRating = x.doubleValue();
        return (planet.getTeraforming() >= canColoniseRating);
	}
    
	/**
	 * @return true if this ship can invade another craft (is an invader).
     * An invader craft is defined as a craft with a Troop Transport weapon.
	 * @throws ApplicationError 
	 */
	public boolean canInvade() throws ApplicationError {
        WeaponBays weaponBays = getWeaponBays();
        if (weaponBays == null) return false;
		return weaponBays.containsGroup(Weapon.GROUP_TROOP_TRANSPORT);
	}

	public boolean hasDamageBots() throws ApplicationError {
        WeaponBays weaponBays = getWeaponBays();
        if (weaponBays == null) return false;
		return weaponBays.containsGroup(Weapon.GROUP_DAMAGE_BOT);
	}

	/**
	 * @return true if this ship is carrying marines.
	 */
    public boolean containsMarines() {
        Specials s = getSpecials();
        if (s == null) return false;
        return (s.getQtyBerths() != null && s.getQtyBerths().intValue() > 0);
    }

	/**
	 * @return true if this ship is carrying weapons.
	 */
    public boolean containsWeapons() {
        WeaponBays weaponBays = getWeaponBays();
        if (weaponBays == null) return false;
        return (weaponBays.getQtyWeapons() > 0);
    }

    public boolean containsWeapon(Weapon w) throws ApplicationError {
        WeaponBays weaponBays = getWeaponBays();
        if (weaponBays == null) return false;
        return (weaponBays.contains(w));
    }

    public boolean containsWeaponGroup(String group) throws ApplicationError {
        WeaponBays weaponBays = getWeaponBays();
        if (weaponBays == null) return false;
        return (weaponBays.containsGroup(group));
    }


	/**
	 * @return true if this ship is carrying fighters.
	 */
    public boolean containsFighters() {
        Specials specials = getSpecials();
        if (specials == null) return false;
        return (specials.getQtyFighters() != null);
    }

	public boolean isMilitary() throws ApplicationError {
		return containsWeapons();
	}

	/**
     * The best planet is defined by the planet quality score. See Planet for details.
     * @return The best Planet this ship can colonise in the in the solar system the craft is located at.
     */
	public Planet getBestPlanetCanColonise() throws ApplicationError {
        StellarObject orbit = getOrbitStellarObject();
        if ( orbit == null || !(orbit instanceof SolarSystem) ) {
            return null;
        }
        Planet [] planets = ((SolarSystem) orbit).getPlanets();
        
        Planet best = null;
        int bestScore = 0;
        for (int i=0; i<planets.length; i++) {
            SolarSystem ss = planets[i].getSolarSystem();
    		if (canColonise(planets[i]) && canMoveTo(ss)) {
                if (!ss.hasOwner() || ss.isOwnedBy(getOwner())) {
                    int score = planets[i].getQualityScore();
                    if (score > bestScore && planets[i].getOwner() == null) {
                        bestScore = score;
                        best = planets[i]; 
                    }
                }
            }
        }
        return best;
	}

	/**
	 * @return true if this craft can be seen by this player at this point in time.
	 */
	public boolean isVisible(Player player) throws ApplicationError {
		if (fleetReference != null) {
			return fleetReference.getFleet().isVisible(player);
		} else {
			return _isVisible(player);
		}
	}

	public boolean _isVisible(Player player) throws ApplicationError {
		if ((player == getOwner()) || (getVisibility(player) > 0)) return true;
		Orbit orbit = getOrbit();
		return (orbit != null && orbit.isOwnedBy(player) || (orbit != null && player.getQtyMoosAt(orbit.getStellarObject()) > 0));
	}
	
	/**
	 * @return the visibility of this craft by player at this point in time.
     * The visibility of a craft is determined by the closest scanner strength 
     * (on a planet or a craft) and the shields cloak value of this craft. The
     * player cloak bonus also effects a crafts visibility.
	 */
	public int getVisibility(Player player) throws ApplicationError {
		if (player == null) return 100;
		int playerSight = player.getScannerStrengthAt(getLocation());
		int cloakLevel = getCloak();
		return (playerSight - cloakLevel);
	}

    /**
     * @return The cloak that is applied to this craft. This is cloaking from the craft's
     * shields plus the origin owners cloak.
     */
	public int getCloak() throws ApplicationError {
		int cloakLevel = getOrigin().getOriginCloakLevel();
        cloakLevel += getOwner().getOwnerCloakLevel();
		if (shields != null) {
			cloakLevel += shields.getCloakLevel();
		}
        return cloakLevel;
    }
    
	public String getCloakDescription() throws ApplicationError {
		int shieldsCloak = 0;
		if (shields != null) {
			shieldsCloak = shields.getCloakLevel();
		}
		int playerCloakLevel = getOrigin().getOriginCloakLevel();
		int leaderCloak = 0;
		EngineGeneralLeader generalLeader = getOwner().getGeneralLeader();
        if (generalLeader != null && generalLeader.hasCloakLevel()) {
        	leaderCloak = generalLeader.getCloakLevel();  
        }
        int resourceCloak = getOwner().getResourceCache().getCloakBonus();
        return L.CLOAK+" = "+L.SHIELDS+" "+shieldsCloak+
        		" + "+L.PLAYER_BONUS+" "+playerCloakLevel +
        		" + "+L.RESOURCE_BONUS+" "+resourceCloak +
        		" + "+L.LEADER_BONUS+" "+leaderCloak;
    }
    
    /**
     * @return The upkeep cost of this craft per turn.
     */
	public int getMaintenanceCost() {
        if (getHull() != null && getHull().getMaintenanceCost() != null) {
            return getHull().getMaintenanceCost().intValue();
        } else {
            return 0;
        }
    }
    
	/**
     * This is the amount of money the player would get if this craft was sold.
	 * @return The value of this craft.
	 */
	public int getValue() {
		double factor = hull.getCostToBuildFactor();
		long v=(getEngine() != null ? getEngine().getCostToBuild(factor) : 0) +
			   (computers != null ? computers.getCostToBuild(factor) : 0) +
			   (weaponBays != null ? weaponBays.getCostToBuild(factor) : 0) +
			   (shields != null ? shields.getCostToBuild(factor) : 0) +
			   (scanners != null ? scanners.getCostToBuild(factor) : 0) +
			   (specials != null ? specials.getCostToBuild(factor) : 0) +
			   (hull != null ? hull.getCostToBuild(factor) : 0);
        return (int)(v / Technology.COST_TO_SELL_FACTOR);

	}
    
    /**
     * @return the area on the map occupied by this craft.
     */
    public Area getArea() throws ApplicationError {
        Size size = getCraftGraphics().getSize();
        Location loc = getLocation();
        int top = loc.getY();
        int left = loc.getX();
        int bottom = top + size.getY();
        int right = left + size.getX();
        return new Area(left, top, right, bottom);
    }

	/**
	 * @return the scanner strength of this craft.
	 */
	public int getScanStrength() {
        return getScanStrength(0);
    }
    
	/**
	 * @return the scanner strength of this craft at distance away from the craft.
	 */
	public int getScanStrength(int distance) {
		if (scanners == null) {
			return 0;
		} else {
			return scanners.getStrength(distance);
		}
	}

	/**
	 * @return the scanner strength of this craft at this location.
	 * Note a crafts scanners scan into space to see surounding crafts.
	 */
	public int getScannerStrengthAt(Location loc) throws ApplicationError {
		int v = getScanStrength(getLocation().getDistanceTo(loc));
		if (v < 0) v = 0;
		return v;
	}

	/**
	 * moves the craft to the planets solar system and once there colonised the
	 * planet.
	 */
	public void colonise(Planet planet) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("Craft.colonise() craft "+getName()+" commanded to colonise "+planet.getName());
        }
		if (planet.getSolarSystem() != getOrbitStellarObject()) {
			moveTo(planet.getOrbit());
		} else {
			planet.colonise(this);
		}
	}
	
	public void changeOwner(Player newOwner) throws ApplicationError {
        Fleet fleet = getFleet();
		if (fleet != null) {
			fleet.remove(this);
			if (fleet.getQtyCrafts() == 0) {
				fleet.destroy();
			}
        }
		if (Battles.contains(this)) {
			MotEngine.instance.getBattlesNotNull().removeCraftFromQueuedBattles(this);
		}
		graphics = null;
		CraftTask task = getTask();
		if (task != null) {
			task.removeCraft(this);
		}
		setTask(null);
		super.changeOwner(newOwner);
	}
	

	public void surrender(Player newOwner) throws ApplicationError {
		changeOwner(newOwner);
	}

	public void arrivedAt(Orbit orbit) throws ApplicationError {
        arrivedAt(orbit, true);
    }
    
	public void arrivedAt(Orbit orbit, boolean withOrders) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Craft.arrivedAt() "+getName()+" now at "+orbit.getDescription());
        }
        
        super.arrivedAt(orbit);
        
        if (withOrders) {
            getOwner().setNextCraftOrders(this);
        }
	}

    /**
     * This craft bombards the planet. For details on bombardment see the 
     * battle.Bombard class.
     * @param planet The planet to be bombarded.
     * @throws mot.common.util.ApplicationError
     */
	public void bombard(Planet planet) throws ApplicationError {
		Orbit orbit = getOrbit();
		if (orbit == null) {
			moveTo(planet.getOrbit());
		}
		StellarObject stellarObject = orbit.getStellarObject();
		if (stellarObject == null || stellarObject != planet.getStellarObject()) {
			moveTo(planet.getOrbit());
		} else {
            Player owner = planet.getOwner();
            if (owner != null) {
                new Bombardment(planet, this).run();
            }
		}
		setHasMoved(true);
	}
    
    public int getDamageRepairPerTurn() {
        return getHull().getMaxDamage() / 10;
    }

	public void endTurn() throws ValidationError, ApplicationError {
        if (damage > 0) {
            damage -= getDamageRepairPerTurn();
        	Orbit orbit = getOrbit();
        	if (orbit != null && orbit.getStellarObject() != null) {
        		Player owner = getOwner();
				if (orbit.getStellarObject().isOwnedBy(owner)) {
        			if (owner.getQtyEnemyCraftAt(orbit.getStellarObject()) == 0) {
        	            damage -= getDamageRepairPerTurn()*2;        				
        			}
        		}
        	}
            if (damage < 0) damage = 0;
        }
        if (getFleet() != null) {
            return;
        }
		if (Logger.isDebug()) {
            Logger.logDebug("Craft.endTurn() name="+getName());
        }
    	if (getNavigator() == null && getOrbit() == null) {
    		// somebody has hacked the save file
    		Location loc = getLocation();
    		if (loc == null || MotEngine.getInstance().getStellarObjects().getStellarObjectAt(loc) != null) {
    			setOrbit(new Orbit(MotEngine.getInstance().getStellarObjects().getStellarObjectAt(loc)));
    		} else if (getOrbit() == null) {
    			setOrbit(new Orbit(getOwner().getCapitol()));
    		}
    	}
		if (getNavigator() != null) {
			getNavigator().move();
		}
		setHasMoved(false);
	}

	public void setDamage(double v) {
		damage = v;
	}

	public void addDamage(double v) {
		damage += v;
        if (damage <= 0) damage = 0;
	}

	public double getDamage() {
		return damage;
	}

	public double getDamageAsPercent() throws ApplicationError {
		int maxDamage = getMaxDamage();
		return damage / maxDamage;
	}

	public int getMaxDamage() throws ApplicationError {
		int md = hull.getMaxDamage();
		if (specials != null) {
			md = (int) (md * ( 1.0 + specials.getMaxDamageBonus()));
		}
        double bonus = 1.0 + getOrigin().getOriginMaxDamageBonus() + getOwner().getOwnerMaxDamageBonus();
        if (bonus > 1.0) {
            md = (int)(md *bonus);
        }
		return md;
	}
    
	public String getMaxDamageDescription() throws ApplicationError {
		int md = hull.getMaxDamage();
		double specialsBonus = 0.0;
		if (specials != null) {
			specialsBonus = specials.getMaxDamageBonus();
		}
		double playerBonus = getOrigin().getOriginMaxDamageBonus();;
		double resourceBonus = getOwner().getResourceCache().getMaxDamageBonus();
		double leaderBonus = 0.0;
		GeneralLeader generalLeader = getOwner().getGeneralLeader();
		if (generalLeader != null && generalLeader.hasMaxDamageBonus()) {
			leaderBonus = generalLeader.getMaxDamageBonus();  
		}
		
        return L.MAX_DAMAGE+" = "+md+
        		" + "+L.SPECIALS_BONUS+" "+Formatter.percentLabelNoSign(specialsBonus) +
        		" + "+L.PLAYER_BONUS+" "+Formatter.percentLabelNoSign(playerBonus) +
        		" + "+L.RESOURCE_BONUS+" "+Formatter.percentLabelNoSign(resourceBonus) +
        		" + "+L.LEADER_BONUS+" "+Formatter.percentLabelNoSign(leaderBonus) ;
	}
    
    public boolean isDestroyed() throws ApplicationError {
        return damage > getMaxDamage();
    }

	public void repair(double amount) {
		damage -= amount;
		if (damage < 0) damage = 0;
	}

	public void damage(double newDamage, Player agressor, BattleCraft bc, CraftStateFactory oldState, LinkedEventFactoryBase parent) throws ApplicationError {
		damage += newDamage;
		if (isDestroyed()) {
			if (bc != null) {
				bc.destroy(agressor, oldState, parent);
			} else {
				destroy();
			}
		}
	}

    /**
     * Removes this craft from the game.
     * The craft is removed from the list of crafts in the game and the owner craft list.
     * If the craft is in a fleet then it is removed from the fleet. If it is the last 
     * craft in a fleet the fleet is also destroyed.
     */
	public void destroy() throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("Craft.destroy() destroy craft "+getName());
        }
		if (fleetReference != null) {
            Fleet fleet = fleetReference.getFleet();
			fleet.remove(this);
            Craft [] crafts = fleet.getCrafts();
            if (crafts == null || crafts.length == 0) {
                getOwner().removeFleet(fleet);
            }
		}
        int maxDamage = getMaxDamage();
        if (damage < maxDamage) {
            damage = maxDamage+1;
        }
        Player owner = getOwner();
		MotEngine.instance.getCrafts().remove(this);
		owner._remove(this);
        MotEngine.instance.getPlanets().flushBlockadeCache();
        Battle battle = getBattle();
        if (battle != null) {
        	BattleCraft battleCraft = battle.getCraft(getName());
        	battle.remove(battleCraft);
        	if (battle.hasEnded()) {
        		MotEngine.instance.getBattles()._end(battle);
        	}
        }
	}

    /**
     * @return the graphics used by this craft on the battle map and on the galaxy map.
     * This is changed in a transient attribute graphics and dynamically generated from
     * image.
     */
	public CraftGraphics getCraftGraphics() throws ApplicationError {
        Player player = getOwner();
		if (graphics == null) {
			RaceGraphicsList graphicsList = GraphicsFactory.getRaceGraphicsList();
            graphics = graphicsList.getHullGraphics(player.getName(), hull.getName());
		}
        if (graphics == null) {
            throw new ApplicationError("Craft.getCraftGraphics() No graphics defined for player="+player.getName()+" Hull="+hull.getName());
        }
		return graphics;
	}
    
    public Size getImageSize() throws ApplicationError {
        if (imageSize == null) {
            imageSize = getCraftGraphics().getSize();
        }
        return imageSize;
    }

    /**
     * Upgrade the engine on all crafts in this list owned by player player and that 
     * have an engine that has a worse engine. Worse is defined as;
     * a - Slower
     * b - The same speed with a lower range
     */
    public void upgradeEngine(Engine engine) throws ApplicationError, ValidationError {
        Engine e = getEngine();
        if (e.getWarpSpeed() < engine.getWarpSpeed()) {
            upgradeTo(engine);   
        } else if ((e.getWarpSpeed() == engine.getWarpSpeed()) && (e.getMaxTravelDistance() < engine.getMaxTravelDistance())) {
            upgradeTo(engine);   
        }
    }
    
    /**
     * This Upgrade the engine on this craft to the engine passed in. The players funds are
     * reduced by the amount required to upgrade to this engine. If there is not enough money
     * a validation error is thrown.
     * @param engine The new engine to use on this craft.
     */
    private void upgradeTo(Engine engine) throws ApplicationError, ValidationError {
        int cost = getEngine().getCostToUpgradeTo(engine);
        cost = (int)(cost * Technology.COST_TO_PURCHASE_FACTOR);
        if (cost > getOwner().getFunds()) {
            throw new ValidationError("You have insuffecient - Not all engines have been upgraded");
        }
        getOwner().addFunds(-cost);
        setEngine(engine);
        Fleet fleet = getFleet();
        if (fleet != null) {
            fleet.setEngine(fleet.getFleetEngine());
        }
    }

	@Override
	public int getQuantityWeaponsWithGroup(String group) {
		WeaponBays wb = getWeaponBays();
		if (wb != null) {
			Weapons weapons = wb.getAllWeaponsAsList();
			if (weapons != null) {
				return weapons.getQuantityWithGroup(group);
			}
		}
		return 0;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("damage", ""+damage);
		writer.addElement("build-quality", ""+buildQuality);
		writer.addElement("design-name", designName);
		if (fleetReference != null) {
			fleetReference.toXML(writer);
		}
		if (hull != null) {
            writer.addElement("hull", hull.getName());
			//hull.toXML(writer);
		}
		if (computers != null) {
			computers.referenceToXML(writer);
		}
		if (weaponBays != null) {
			weaponBays.toXML(writer);
		}
		if (shields != null) {
			shields.referenceToXML(writer);
		}
		if (scanners != null) {
			scanners.referenceToXML(writer);
		}
		if (specials != null) {
			specials.referenceToXML(writer);
		}
	}

	protected String getElementName() { return "craft"; }

    /**
     * Used by XMLDocumentHanler to load saved games.
     */
    public void setHull(String hullName) throws ApplicationError {
		Hulls hulls = Technologies.getHulls();
        hull = hulls.getHull(hullName);
    }

	public double getBuildQuality() {
		return buildQuality;
	}

	public void setBuildQuality(double buildQuality) {
		this.buildQuality = buildQuality;
	}

	// DELETE ME - I am just for save file backwards compatibility
	public void setWeaponBays(mot.common.technology.WeaponBays wb) throws ApplicationError {
		weaponBays = new WeaponBays(wb);
	}

	public String getDesignName() {
		// DELETE ME - I am just for save file backwards compatibility
		if (designName == null) {
			String name = getName();
			int i = name.length()-1;
			while (i > 0 && designName == null) {
				if (!Character.isDigit(name.charAt(i))) {
					if (i == name.length()-1) {
						designName = name;
					} else {
						designName = name.substring(0, i+1);
					}
				}
				i--;
			}
			if (designName == null) {
				designName = name;
			}
		}
		return designName;
	}

	public void setDesignName(String designName) {
		this.designName = designName;
	}

}