/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.crafts;

import mot.common.technology.Engine;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.battles.Battles;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.event.PlanetCrisisEF;
import mot.engine.planets.CrisisFactory;
import mot.engine.planets.Planet;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.players.generals.CraftTask;
import mot.engine.players.generals.CraftTaskReference;
import mot.engine.relations.Relations;

/**
 * Base craft for fleet and craft objects. Implements functionality common to all
 * craft type objects. This objects only purpose is to prevent duplicate code between
 * Craft and Fleet.
 */
public abstract class CraftBase extends MoveableOwnedObject {
    public CraftBase() {
    }

	public CraftBase(String name, Player owner, Engine engine, Orbit orbit) {
		super(name, owner, engine, orbit);
	}
    
    /**
     * This is used by the general task to assign a task to a craft. A task is esentially
     * a purpose to which crafts are assigned such as escort, attack an enemy planet etc.
     */
    private CraftTaskReference task;

	/**
	 * This is true if this craft has bombarded (or some other action that takes the turn) a
	 * planet this turn. It means that this craft's turn has been taken and it can no longer 
	 * be given any orders (until next turn).
	 */
	private boolean hasMoved;

	protected static String getNewName(String prefix) {
		return MotEngine.getInstance().getNameGenerator().getNextName(prefix);
	}
	
	public void attack(Planet planet) throws ApplicationError {
		Craft[] enemyCrafts = getOwner().getEnemyCraftAt(planet.getStellarObject(), true);
		attack(planet, enemyCrafts);
	}
	
    /**
     * If this craft is not at the of the target this craft moves to their location
     * Otherwise
     * This craft attacks the planet passed in. A battle is used or created including
     * this craft, the planet and any other craft belonging to the owner of the planet 
     * at this location.
     * @param planet The planet to attack
     * @param enemyCrafts 
     * @throws mot.common.util.ValidationError
     * @throws mot.common.util.ApplicationError
     */
	public void attack(Planet planet, Craft[] enemyCrafts) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("CraftBase.attack() "+getName()+" attack "+planet.getName());
        }
		StellarObject stellarObject = getOrbitStellarObject();
		if (stellarObject != null && stellarObject == planet.getSolarSystem()) {
			if (!MotEngine.instance.getRelations().isWar(getOwner(), planet.getOwner())) {
				planet.getOwner().startWar( getOwner(), null, false);
			}
			//Battles battles = MotEngine.instance.getBattlesNotNull();
            Battles.join(planet);
            Battles.join(getOrbit(), this, true);
            if (enemyCrafts != null) {
                Battles.join(getOrbit(), enemyCrafts, true);
            }
            //Craft [] defendendingCraft = planet.getOwner().getCraftsAt(stellarObject);
		} else {
			moveTo(planet.getOrbit());
		}
	}

    /**
     * If this craft is not at the of the target this craft moves to their location
     * Otherwise
     * This craft attacks the crafts passed in at the stellar object stellarObject and
     * the outpost (if owed by the same player). The crafts must be located at stellarObject. 
     * If enemyCrafts is null then this craft attack the outpost located at stellarObject. A 
     * battle is used or created including this craft, the enemy craft passed in.
     * @param stellarObject The location this battle will take place at
     * @param enemyCraft the craft for this craft to attack
     * @throws mot.common.util.ValidationError
     * @throws mot.common.util.ApplicationError
     */
	public void attack(StellarObject stellarObject, Craft [] enemyCraft) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("CraftBase.attack() "+getName()+" attack "+Comp.toString(enemyCraft, ", "));
        }
        
		Orbit orbit = getOrbit();
		if (orbit != null && orbit.atSameSO(stellarObject)) {
			StellarObjectOrbital orbital = stellarObject.getOrbital();
            Player enemy = null;
            if (enemyCraft != null && enemyCraft.length > 0) {
                enemy = enemyCraft[0].getOwner();
            }
            if (enemy == null && orbital != null) {
                enemy = orbital.getOwner();
            }
            if (enemy == null) {
                Logger.logWarning("CraftBase.attack() NO ENEMY");
                return;
            }
            Player owner = getOwner();
			if (owner == enemy) {
                Logger.logWarning("CraftBase.attack() ATTACKING MYSELF");
                return;
            }
			if (!MotEngine.instance.getRelations().isWar(owner, enemy)) {
				enemy.startWar(owner, null, false);
			}
			// Find planet that that this battle should take place around
			// Should be the planet the attacked craft are at and not the planet 
			// of the attacking craft.
			if (enemyCraft != null && enemyCraft.length > 0 && enemyCraft[0].getOrbit() != null) {
				orbit = enemyCraft[0].getOrbit();
			}
			Planet planet = orbit.getPlanet();
			if (planet != null && planet.isOwnedBy(enemy) && planet.hasDefenses()) {
				Battles.join(orbit.getPlanet());
			}
			//Battles battles = MotEngine.instance.getBattlesNotNull();
            if (orbital != null && orbital.getOwner() != null) {
                if (orbital.getOwner().isEnemy(owner)) {  
                    Battles.join(orbit, orbital);
                }
            }
            Battles.join(orbit, this, true);
            if (enemyCraft != null && enemyCraft.length > 0) {
                Battles.join(orbit, enemyCraft);
            }
		} else {
			moveTo(orbit);
		}
	}

	public void blockade(StellarObject stellarObject) throws ApplicationError {
		StellarObject orbit = getOrbitStellarObject();
		if (orbit == null || orbit != stellarObject) {
			moveTo(stellarObject.getOrbit());
		} else {
			// Simply the presence here is a blockade
		}
	}

	public boolean isBlockading(Planet planet) throws ApplicationError {
		Orbit orbit = getOrbit();
		if (orbit == null) return false;
		StellarObject pss = planet.getSolarSystem();
		StellarObject so = orbit.getStellarObject();
		Relations relations = MotEngine.instance.getRelations();
		if  (so != null && so == pss && relations != null) {
			return relations.isWar(getOwner(), planet.getOwner()) && isMilitary() ;
		} else {
			return false;
		}
	}

	public boolean isBlockading() throws ApplicationError {
		Orbit orbit = getOrbit();
		if (orbit == null) return false;
		StellarObject so = orbit.getStellarObject();
		Player[] enemyPlayers = getOwner().getEnemyPlayers();
		return ( so != null && so.isOwnedByAnyOf(enemyPlayers) );
	}

	/**
	 * @return True iff this craft is a military craft (has weapons).
	 */
	public abstract boolean isMilitary() throws ApplicationError;
	
	public abstract void bombard(Planet planet) throws ApplicationError;

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("has-moved", hasMoved);
        if (task != null) {
            task.toXML(writer);
        }
    }

    public void setCraftTaskReference(CraftTaskReference r) {
        setTaskReference(r);
    }

    public void setTaskReference(CraftTaskReference task) {
        this.task = task;
    }

    public CraftTaskReference getTasReference() {
        return task;
    }
    
    public void setTask(CraftTask v) {
        if (v == null) {
            task = null;
        } else {
            task = new CraftTaskReference(v);
        }
    }
    
    /**
     * @return The task to which this craft has been assigned or null of this craft does not 
     * have a task. Note only computer players have a task - see CraftTask for details.
     * @throws mot.common.util.ApplicationError
     */
    public CraftTask getTask() throws ApplicationError {
        if (task == null) {
            return null;
        } else {
            return task.getTask();
        }
    }

    /**
     * Note - The name is not the type such as explore but each individual task is given a 
     * unique name by the name generator which is returned here.
     * @return The task name this object is assigned to.
     * @throws mot.common.util.ApplicationError
     */
    public String getTaskName() throws ApplicationError {
        CraftTask task = getTask();
        if (task == null) {
            return "None"; 
        } else {
            return task.getName();
        }
    }

	public abstract int getQuantityWeaponsWithGroup(String group);

    /**
     * @return TRUE iff this craft / fleet contains weapons that can be deployed on a planet
     * (biological / chemical weapons).
     */
	public boolean containsDeployOnPlanet() {
		int qtyPsi = getQuantityWeaponsWithGroup(Weapon.GROUP_PSI);
		int qtyPlague = getQuantityWeaponsWithGroup(Weapon.GROUP_PLAGUE);
		return qtyPsi + qtyPlague > 0;
	}


	/**
	 * Deploy chemical / biological weapons on this planet. This returns a CommandResult containing state updates for the client
	 * @param planet The planet to deploy on.
	 * @throws ApplicationError 
	 */
	public void deploy(Planet planet) throws ApplicationError {
		int qtyPsi = getQuantityWeaponsWithGroup(Weapon.GROUP_PSI);
		int qtyPlague = getQuantityWeaponsWithGroup(Weapon.GROUP_PLAGUE);
		if (qtyPsi > 0) {
			CrisisFactory.createPsiCrisis(planet);
		}
		if (qtyPlague > 0) {
			CrisisFactory.createPlagueCrisis(planet);
		}
		if (qtyPsi > 0 || qtyPlague > 0) {
			Player craftOwner = getOwner();
			ComputerPlayer[] computerPlayers = MotEngine.instance.getPlayers().getComputerPlayers();
			if (computerPlayers != null) {
				for (int i = 0; i < computerPlayers.length; i++) {
					if (computerPlayers[i] != craftOwner) {
						computerPlayers[i].getStratagist().notifyUseOfBiologicalWeapons(craftOwner);
					}
				}
			}
			Player planetOwner = planet.getOwner();
			PlanetCrisisEF event1 = new PlanetCrisisEF(planet);
			if (planetOwner instanceof UserPlayer) {
				event1.addStateUpdate(new PlanetVOAdapter(planet, planetOwner));
			}
			MotEngine.instance.broadcast(event1, planetOwner);
			PlanetCrisisEF event2 = new PlanetCrisisEF(planet);
			if (craftOwner instanceof UserPlayer) {
				event2.addStateUpdate(new PlanetVOAdapter(planet, craftOwner));
				EngineVoFactory.addMooDeepStateUpdate(this, craftOwner, event2);
			}
			MotEngine.instance.broadcast(event2, craftOwner);
		}
	}


	public boolean hasMoved() {
		return hasMoved;
	}

	public void setHasMoved(boolean hasMoved) {
		this.hasMoved = hasMoved;
	}

    
}