/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.crafts;

import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.util.ApplicationError;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.CraftBuildExperience;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * This creates new instances of a craft from a craft design in craft_designes.xml.
 * NOTE - This factory is ONLY used to create crafts based on predefined craft designs
 * and is used during the game setup and for monsters - It is not used to create crafts
 * during a game. During a game crafts are creates using CraftDesign.getCraftInstance().
 */
public class CraftFactory  {

    /**
     * Create a new craft owned by player player, at the home stellar object of this player
     * and conforming to design design. The craft is orbiting the home planet of the owner.
     */
    public static Craft create(Player player, Player origin, String design, boolean withOrders, boolean useBuiltQuality) throws ApplicationError {
        Planet planet = player.getCapitol();
        if (planet == null) {
            throw new ApplicationError("CraftFactory.create() Player "+player.getName()+" does not have any planets");
        }
        SolarSystem so = planet.getSolarSystem();
        return create(design, player, origin, new Orbit(so, planet), withOrders, useBuiltQuality);

    }

    /**
     * Create a new craft owned by player player, at stellar object so and conforming to
     * design design.
     */
    public static Craft create(String design, Player owner, Player origin, Orbit orbit, boolean withOrders, boolean useBuiltQuality) throws ApplicationError {
        CraftDesigns craftDesigns = CraftDesigns.getInstance();
        CraftDesign craftDesign = craftDesigns.find(design);
        if (craftDesign == null) {
        	throw new ApplicationError("No such design "+design);
        }
        Craft craft = getCraftInstance(craftDesign, owner, origin, orbit, withOrders, useBuiltQuality);
        if (owner instanceof UserPlayer) {
            UserPlayer up = (UserPlayer) owner;
            CraftDesigns cds = up.getCraftDesigns();
            if (cds == null) {
                cds = new CraftDesigns();
                up.setCraftDesigns(cds);
            }
            cds.setCraftDesign( new CraftDesign(craftDesign) );
        }
        return craft;
    }

	/**
	 * Constructs a craft object for this design located at this stellar object.
	 * The new craft is added to the game and its owner's list of crafts.
	 * Once created the craft is located at the stellar object passed in orbiting the
	 * planet passed in. Note planet may be null, in which case the craft is not orbiting
	 * a planet!
	 */
	public static Craft getCraftInstance(CraftDesign design, Player owner, Player origin, Orbit orbit, boolean withOrders, boolean useBuiltQuality) throws ApplicationError {
		// Create a new craft
		double buildQuality = Craft.DEFAULT_BUILD_QUALITY;
		if (useBuiltQuality && design.getHullName() != null) {
			String hullName = design.getHullName();
			CraftBuildExperience cbe = owner.getCraftBuildExperienceNotNull();
			buildQuality = cbe.getBuildQuality(owner, hullName);
			cbe.addDesignExperience(hullName);
		}
		
		Craft craft = new Craft(owner, origin, design, orbit, buildQuality);
		// Add the new craft to the game.
		Crafts crafts = MotEngine.getInstance().getCrafts();
        if (crafts == null) {
            crafts = new Crafts();
            MotEngine.instance.setCrafts(crafts);
        }
        crafts.add(craft);
        
        if (orbit != null) {
        	craft.arrivedAt(orbit, withOrders);
        }
		
        return craft;
	}

    /**
     * This method is called when a craft has been launched in a battle. This craft does
     * not exist outside the battle (e.g. A Fighter Craft launched from a FighterBay special).
     * The craft built is not added to MotEngine or the players list of crafts!
     */
    public static Craft build(CraftDesign design, Player owner, Orbit orbit) throws ApplicationError {
		Craft craft = new Craft(owner, design, orbit);
		craft.setOrbit(orbit);
        return craft;
    }

}