/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.crafts;

import java.util.ArrayList;

import mot.common.bean.Location;
import mot.common.technology.Engine;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.StellarObject;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

public class Crafts extends XMLBean {

	/**
	 * The list craft currently in this game.
	 */
    private ArrayList list;

    public ArrayList getList() { return list; }
    public void setList(ArrayList v) { list = v; }

    public Crafts() {
	}

    private transient Craft [] crafts;
    public Craft [] getCrafts() {
        if (list == null || list.size() == 0) return null;
		if (crafts == null) {
			crafts = (Craft []) list.toArray( new Craft[list.size()]);
		}
		return crafts;
	}

    /**
     * Used by XMLDocumentHandler to load saved games.
     * Longcut for add(Craft).
     */
    public void setCraft(Craft v) throws ApplicationError {
        add(v);
    }

    /**
     * Adds a new Craft to this list.
     * Flushes the array cache of Crafts.
     */
    public void add(Craft craft) throws ApplicationError {
		crafts = null;
        if (list == null) list = new ArrayList();
		list.add(craft);
        if (craft.getOwner() == null) {
            Logger.logError("");
        }
		craft.getOwner().add(craft);
	}

    public void remove(Craft craft) throws ApplicationError {
		crafts = null;
		list.remove(craft);
		craft.getOwner()._remove(craft);
	}

    public Craft getCraft(String name) {
		Craft [] crafts = getCrafts();
        if (crafts == null) return null;
		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].getName().equalsIgnoreCase(name)) {
				return crafts[i];
			}
		}
		return null;
	}

	public void endTurn() throws ValidationError, ApplicationError {
		Craft [] crafts = getCrafts();
        if (crafts == null) return;
		for (int i=0; i<crafts.length; i++) {
			crafts[i].endTurn();
		}
        MotEngine.instance.getPlanets().flushBlockadeCache();
	}

    /**
     * @return True if there is a craft in this list of crafts that is blockading the
     * planet passed in. A planet is blockaded if there is an enemy craft in orbit in
     * the stellar object.
     * @param planet The planet that may be being blockaded.
     */
	public boolean isBlockaded(Planet planet) throws ApplicationError {
		Craft [] crafts = getCrafts();
        if (crafts == null) return false;
		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].isBlockading(planet)) {
				return true;
			}
		}
		return false;
	}

    /**
     * @return The crafts at location loc.
     */
	public Craft [] getAt(Location loc) throws ApplicationError {
		Craft [] crafts = getCrafts();
        if (crafts == null || loc == null) return null;
        ArrayList list = null;
		for (int i=0; i<crafts.length; i++) {
            if (loc.equals(crafts[i].getLocation())) {
                if (list == null) list = new ArrayList();
                list.add(crafts[i]);
            }
		}
        if (list == null) {
            return null;
        } else {
            return (Craft []) list.toArray(new Craft[list.size()]);
        }
	}
    
    /**
     * @return The crafts at with orbit.
     */
	public Craft [] getAt(StellarObject orbit) throws ApplicationError {
		Craft [] crafts = getCrafts();
        if (crafts == null || orbit == null) return null;
        ArrayList list = null;
		for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getOrbit() != null && crafts[i].getOrbit().getStellarObject() == orbit) {
                if (list == null) list = new ArrayList();
                list.add(crafts[i]);
            }
		}
        if (list == null) {
            return null;
        } else {
            return (Craft []) list.toArray(new Craft[list.size()]);
        }
	}
    
    /**
     * @return The qty crafts at location loc.
     */
	public int getQtyAt(Location loc) throws ApplicationError {
		Craft [] crafts = getCrafts();
        if (crafts == null || loc == null) return 0;
        int qty = 0;
		for (int i=0; i<crafts.length; i++) {
            if (loc.equals(crafts[i].getLocation())) {
                qty++;
            }
		}
        return qty;
	}
    
    /**
     * @return The qty crafts at stellar object orbit.
     */
	public int getQtyAt(StellarObject so) throws ApplicationError {
		Craft [] crafts = getCrafts();
        if (crafts == null || so == null) return 0;
        int qty = 0;
		for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getOrbit() != null && crafts[i].getOrbit().getStellarObject() == so) {
                qty++;
            }
		}
        return qty;
	}
    
    /**
     * @return The qty military crafts at stellar object orbit.
     */
	public int getQtyMilitaryAt(StellarObject so) throws ApplicationError {
		Craft [] crafts = getCrafts();
        if (crafts == null || so == null) return 0;
        int qty = 0;
		for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getOrbit() != null && crafts[i].getOrbit().getStellarObject() == so) {
            	if (crafts[i].isMilitary()) {
            		qty++;
            	}
            }
		}
        return qty;
	}
    
    /**
     * Upgrade the engine on all crafts in this list owned by player player and that 
     * have an engine that has a worse engine. Worse is defined as;
     * a - Slower
     * b - The same speed with a lower range
     */
    public void upgradeEngines(Player player, Engine engine) throws ApplicationError, ValidationError {
		Craft [] crafts = getCrafts();
        if (crafts == null) return;
		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].isOwnedBy(player)) {
				crafts[i].upgradeEngine(engine);
			}
		}
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (getCrafts() != null) {
			Craft [] crafts = getCrafts();
			for (int i=0; crafts != null && i<crafts.length; i++) {
				if (crafts[i] != null) {
					crafts[i].toXML(writer);
				}
			}
		}
	}

	protected String getElementName() { return "crafts"; }
}