/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.crafts;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.bean.Location;
import mot.common.bean.Size;
import mot.common.technology.Engine;
import mot.common.technology.Specials;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.battles.Bombardment;
import mot.engine.common.MoveableOwnedObjectReference;
import mot.engine.common.Orbit;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * This is a Fleet in the game. A fleet is a list of Craft objects. When a craft
 * is part of a fleet the fleets navigator is used to control it.
 * 
 * Fleets move using the slowest warp speed of all crafts in the fleet. However,
 * the specials speed bonuses from all crafts are added to improve the speed. Thus 
 * a single craft with a speed bonus special can improve the speed of all craft in 
 * the fleet.
 */
public class Fleet extends CraftBase {

	public Fleet() {
	}

	public Fleet(Player owner, Orbit orbit) {
		super(getNewName("Fleet"), owner, null, orbit);
        if (Logger.isDebug()) {
            Logger.logDebug("Fleet.init() Create fleet "+getName());
        }
	}

	/**
	 * The list of craft in this fleet. This is a list of CraftReference objects.
	 */
	private ArrayList craftList = new ArrayList();
    private transient Craft [] crafts = null;

	public ArrayList getCraftList() { return craftList; }
	public void setCraftList(ArrayList v) { craftList = v; }

	public int getQtyCrafts() {
        if (craftList == null) {
            return 0;
        } else {        	
        	return craftList.size();
        }
        
	}
	
	public Craft [] getCrafts() {
        if (crafts != null) {
            return crafts;
        }
        
		if (craftList == null) {
			return null;
		}

		crafts = new Craft[ craftList.size() ];
		for (int i=0; i<craftList.size(); i++) {
			CraftReference cr = (CraftReference) craftList.get(i);
			crafts[i] = cr.getCraft();
		}

		return crafts;
	}

	public String [] getCraftNames() {
        Craft[] ca = getCrafts();
        if (ca == null) return null;
		String [] r = new String[ca.length];
		for (int i = 0; i < r.length; i++) {
        	if (ca[i] != null) {
        		r[i] = ca[i].getName();
        	}
		}
		return r;
	}

    public int getSize() {
        return craftList.size();
    }

	public void add(Craft [] crafts) throws ApplicationError {
		if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {
				add(crafts[i]);
			}
		}
	}
	
    /**
     * Adds a new Craft to the fleet.
     * This may change the speed at which the fleet moves, as a fleet moves at the 
     * speed of the slowest craft.
     */
	public void add(Craft craft) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Fleet.add() Fleet="+getName()+" craft="+craft.getName());
        }
		craftList.add( new CraftReference(craft) );
        craft.setFleetReference( new FleetReference(this) );
        crafts = null;
        setEngine(getFleetEngine());
	}
    
    /**
     * Used by XMLDocumentHandles to load a saved game.
     * Does same as add(Craft).
     */
    public void setCraftReference(CraftReference cr) {
		craftList.add( cr );
        crafts = null;
    }

    /**
     * Removes this craft from the fleet and removes the fleet reference in the craft.
     * @param craft The craft to remove from the fleet.
     * @throws ApplicationError 
     */
	public void remove(Craft craft) throws ApplicationError {
		crafts = null;
        craft.setFleetReference( null );
        Iterator i = craftList.iterator();
        while (i.hasNext()) {
            CraftReference cr = (CraftReference) i.next();
            if (cr.getCraft() == craft) {
                craftList.remove(cr);
                craft.setLocation(new Location(getLocation()));
                Orbit orbit = getOrbit();
                if (orbit != null) {
                	craft.setOrbit(new Orbit(orbit));
                } else {
                	craft.setOrbit(null);
                }
                Navigator navigator = getNavigator();
                if (navigator != null) {
                    craft.setNavigator(new Navigator(navigator));
				} else {
	                craft.setNavigator(null);
				}
                break;
            }
        }
        setEngine(getFleetEngine());
	}
    
    /**
     * This deletes this fleet. The craft contained in it are not deleted but are removed
     * from the fleet. The fleet is removed from its owners list of fleets.
     */
    public void destroy() throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	if (crafts[i] != null) {            		
            		crafts[i].setFleetReference(null);
            		Location location = getLocation();
            		if (location != null) {
            			crafts[i].setLocation(new Location(location));
            		}
            		Navigator navigator = getNavigator();
            		if (navigator != null) {
            			Navigator newNavigator = new Navigator(navigator);
            			newNavigator.setObjectReference(new CraftReference(crafts[i]));
						crafts[i].setNavigator(newNavigator);
            		} else {
            			crafts[i].setNavigator(null);
            		}
            		Orbit orbit = getOrbit();
            		if (orbit != null) {
            			crafts[i].setOrbit(new Orbit(orbit));
            		} else {
            			crafts[i].setOrbit(null);
            		}
            	}
            }
        }
        getOwner().removeFleet(this);
    }

    @Override
	public void destroyAll() throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	crafts[i].destroy();
            }
        }
	}

    /**
     * Returns true if this fleet contains this craft.
     */
	public boolean contains(Craft craft) {
        Iterator i = craftList.iterator();
        while (i.hasNext()) {
            CraftReference cr = (CraftReference) i.next();
            if (cr.getCraft() == craft) {
                return true;
            }
        }
        return false;
	}
    
	/**
	 * A fleet is visible if any craft it contains is visible.
	 * @return true if this fleet can be seen by this player at this point in time.
	 */
	public boolean isVisible(Player player) throws ApplicationError {
		Craft[] ca = getCrafts();
		if (ca == null) return false;
		for (int i = 0; i < ca.length; i++) {
        	if (ca[i] != null) {
				if (ca[i]._isVisible(player)) {
					return true;
				}
        	}
		}
		return false;
	}
	
	public MoveableOwnedObjectReference createNewReference() throws ApplicationError {
		return new FleetReference(this);
	}

    /**
     * This is the size of the bitmap used to display this craft for this player on the screen.
     */
    transient Size imageSize = null;
    
    public Size getImageSize() throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (imageSize == null) {
            imageSize = crafts[0].getCraftGraphics().getSize();
        }
        return imageSize;
    }
    
    /**
     * This is the engine that will determine the speed at which this fleet moves at warp.
     * @return The slowest engine at warp of the crafts in the fleet.
     */
    public Engine getFleetEngine() {
        Craft [] crafts = getCrafts();
        if (crafts == null) return null;
        Engine slowest = null;
        for (int i=0; i<crafts.length; i++) {
        	if (crafts[i] != null) {
	            Engine engine = crafts[i].getEngine();
	            if (slowest == null || slowest.getWarpSpeed() > engine.getWarpSpeed()) {
	                slowest = engine;
	            }
        	}
        }
        return slowest;
    }

    /**
     * @return The visibility of the most visible craft in this fleet.
     */
    public int getVisibility(Player player) throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts == null) return 0;
        int visibility = 0;
        for (int i=0; i<crafts.length; i++) {
        	if (crafts[i] != null) {
	        	int craftVisibility = crafts[i].getVisibility(player);
	            if (craftVisibility > visibility) {
	            	visibility = craftVisibility;
	            }
        	}
        }
        return visibility;
	}
    
    /**
     * The best speed bonus of all crafts is used as the speed bonus for the fleet.
     * @return Returns the speed bonus that applies to this fleet.
     */
    public double getSpeedBonus() throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts == null) return 1.0;
        Double best = null;
        for (int i=0; i<crafts.length; i++) {
            Specials craftSpecials = crafts[i].getSpecials();
            if (craftSpecials != null && craftSpecials.getQuantity() > 0) {
                Double v = craftSpecials.getSpeedBonusAsDouble();
                if (best == null) {
                    best = v;
                } else if (best.doubleValue() < v.doubleValue()) {
                    best = v;
                }
            }
        }
        double playerBonus = 1.0 + getOrigin().getOriginSpeedBonus() + getOwner().getOwnerSpeedBonus();
        if (best != null) {
            return (1.0 + best.doubleValue()) * playerBonus;
        } else {
            return playerBonus;
        }
    }


	/**
	 * See MoveableOwnedObject.depart() for details.
     * In addition this method calls depart() on all crafts belonging to the fleet.
	 * @throws ApplicationError 
	 */
	public void depart(Navigator nav) throws ApplicationError {
        super.depart(nav);
        //Craft [] crafts = getCrafts();
		//if (crafts != null) {
        //    for (int i=0; i<crafts.length; i++) {
        //        crafts[i].depart(nav);
        //    }
		//}
	}
    
	/**
	 * Returns true if any craft in this fleet can colonise the planet passed in.
	 */
	public boolean canColonise(Planet planet) {
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (crafts[i].canColonise(planet)) {
                    return true;
                }
            }
		}
        return false;
	}
    
	/**
	 * See MoveableOwnedObject.arrivedAt() for details.
     * In addition this method calls arrivedAt() on all crafts belonging to the fleet.
	 */
	public void arrivedAt(Orbit orbit) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Fleet.arrivedAt() stellar object="+orbit.getDescription());
        }
        super.arrivedAt(orbit);
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                crafts[i].arrivedAt(orbit, false);
            }
		}
		getOwner().setNextFleetOrders(this);
	}

    /**
     * Each craft in this fleet bombards the planet. For details on bombardment
     * see the battle.Bombard class.
     * @param planet The planet to be bombarded.
     * @throws mot.common.util.ApplicationError
     */
	public void bombard(Planet planet) throws ApplicationError {
		Orbit orbit = getOrbit();
		if (orbit == null) {
			return;
		} else if (orbit.getStellarObject() != planet.getStellarObject()) {
			moveTo(planet.getOrbit());
		} else {
            Player owner = planet.getOwner();
            if (owner != null) {
                new Bombardment(planet, getCrafts()).run();
            }
		}
	}
	
	public boolean isMilitary() throws ApplicationError {
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (crafts[i].containsWeapons()) {
                	return true;
                }
            }
		}
		return false;
	}

	@Override
	public int getQuantityWeaponsWithGroup(String group) {
		int qty = 0;
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                qty += crafts[i].getQuantityWeaponsWithGroup(group);
            }
		}
		return qty;
	}

	public int getQtyMilitaryCraft() throws ApplicationError {
		int qty = 0;
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (crafts[i].isMilitary()) qty++;
            }
		}
		return qty;
	}

	public int getQtyColinserCraft() throws ApplicationError {
		int qty = 0;
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (crafts[i].canColonise()) qty++;
            }
		}
		return qty;
	}

	public Craft getColoniser() {
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (crafts[i].canColonise()) {
                	return crafts[i];
                }
            }
		}
		return null;
	}


	public void endTurn() throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Fleet.endTurn() name="+getName());
        }
        if (getSize() <= 0) {
            destroy();
            return;
        }
		if (getNavigator() != null) {
			getNavigator().move();
		}
		setHasMoved(false);
	}

	public void setHasMoved(boolean hasMoved) {
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	if (crafts[i] != null) {
            		crafts[i].setHasMoved(hasMoved);
            	}
            }
		}
		super.setHasMoved(hasMoved);
	}
	
	public void setSleeping(boolean sleeping) {
		super.setSleeping(sleeping);
        Craft [] crafts = getCrafts();
		if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	if (crafts[i] != null) {
            		crafts[i].setSleeping(sleeping);
            	}
            }
		}
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);

		if (craftList != null) {
            for (int i=0; i<craftList.size(); i++) {
                CraftReference cr = (CraftReference) craftList.get(i);
                cr.toXML(writer);
            }
		}
	}

	protected String getElementName() { return "fleet"; }

	public String toString() {
		try {
			Orbit orbit = getOrbit();
			Location location = getLocation();
			return getName()+"[crafts=" + Comp.toString(getCrafts())+ "]@"+ ((orbit == null) ? (location == null ? "?" : location.toString()) : orbit.getDescription());
		} catch (ApplicationError e) {
			Logger.logError("toString() Failed",e);
			return getName()+"[crafts=" + Comp.toString(getCrafts())+ "]@FAILED";
		}
	}

	
}