/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.crafts;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.MoveableOwnedObjectReference;
import mot.engine.common.Orbit;
import mot.engine.planets.BlackHole;
import mot.engine.planets.GalacticIncumbent;
import mot.engine.planets.StellarObjects;
import mot.engine.transports.TransportReference;

/**
 * A Navigator pilots a Movable Owned Object (such as a Craft or Fleet) from one 
 * solar system to another.
 */
public class Navigator extends XMLBean {

	/**
	 * This is the object being navigated across the map.
	 */
	private MoveableOwnedObjectReference objectReference;

	/**
	 * The origin is the location from where the object has moved.
	 */
	private Orbit origin;

	/**
	 * The destination star system.
	 */
	private Orbit destination;
	
	public MoveableOwnedObjectReference getObjectReference() { return objectReference; }
	public void setObjectReference(MoveableOwnedObjectReference v) { objectReference = v; }
	public MoveableOwnedObject getObject() throws ApplicationError { return objectReference.getObject(); }

	public Navigator() {
	}

	public Navigator(Navigator other) {
		this.origin = other.getOrigin();
		this.destination = other.getDestination();
		this.objectReference = other.getObjectReference();
	}

	public Navigator(MoveableOwnedObject object, Orbit origin, Orbit destination) throws ApplicationError {
		this.origin = origin;
		this.destination = destination;
		this.objectReference = object.createNewReference();
	}

    public int getDistanceRemaining() throws ApplicationError {
        MoveableOwnedObject moo = objectReference.getObject();
        if (moo == null) {
        	return 999;
        }
        Location loc = moo.getLocation();
        Location dest = destination.getStellarObject().getLocation();
        return loc.getDistanceTo(dest);
    }
    
    /**
     * Moves this object at warp speed from its current location towards the destination.
     * If the object arrives at the destination is calls the arrivedAt() method.
     * If the object has an orbit, it calls the depart() method so the object can leave orbit.
     */
	public void move() throws ApplicationError  {
		if (objectReference == null) return;
        MoveableOwnedObject moo = objectReference.getObject();
        if (moo == null) {
        	return;
        }
        
        Orbit orbit = moo.getOrbit();
		if (orbit != null) {
            if (moo.isInABattle()) {
                // This craft cannot depart a solar system while it is being attacked!
                return;
            }
            if (orbit.getStellarObject() != null && orbit.getStellarObject() instanceof BlackHole) {
            	BlackHole bh = (BlackHole) orbit.getStellarObject();
            	if (bh.canSpaceJumpTo(moo, destination.getStellarObject())) {
            		moo.arrivedAt(destination);
            		return;
            	}
            }
            moo.depart(this);
        }
    
		int speed = moo.getWarpSpeed();
		Location dest = getDestination().getLocation();
		Location currentLoc = moo.getLocation();
		Location newLoc = new Location(currentLoc);
		newLoc.shift(dest, speed);
		
		StellarObjects sos = MotEngine.getInstance().getStellarObjects();
		GalacticIncumbent[] incumbents = sos.getGalacticIncumbentsBetween(currentLoc, newLoc);
		if (incumbents != null) {
			for (int i = 0; i < incumbents.length; i++) {
				if (incumbents[i].handleEntry(moo)) {
					if (Logger.isDebug()) {
			            Logger.logDebug("Navigator.move() "+moo.getName()+" destroyed in "+incumbents[i].getName());
			        }
					return;
				}
			}
		}

		currentLoc.setLocation(newLoc);

        if (Logger.isDebug()) {
            Logger.logDebug("Navigator.move() "+moo.getName()+" speed "+speed+" to "+getDestination().getDescription()+" [remaining "+getDistanceRemaining()+"]");
        }

		if (currentLoc.equals(dest)) {
			//object.getLocation().setLocation(dest);
			moo.arrivedAt(getDestination());
		}
	}
    
	/**
     * @return The quantity of turns before this navigator will arrive at the destination. 
     */
    public int getETA() throws ApplicationError {
        MoveableOwnedObject moo = objectReference.getObject();
        Orbit orbit = moo.getOrbit();
        if (orbit != null && orbit.getStellarObject() != null && orbit.getStellarObject() instanceof BlackHole) {
        	BlackHole bh = (BlackHole) orbit.getStellarObject();
        	if (bh.canSpaceJumpTo(moo, destination.getStellarObject())) {
        		return 1;
        	}
        }
    	float speed = moo.getWarpSpeed();
    	float dist = moo.getLocation().getDistanceTo(destination.getLocation());
    	return (int) Math.ceil(dist / speed);
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (origin != null) {
			origin.toXML(writer, "origin");
		}
		if (destination != null) {
			destination.toXML(writer, "destination");
		}
		if (objectReference != null) {
			objectReference.toXML(writer);
		}
	}

	protected String getElementName() { return "navigator"; }
    
    /**
     * Used by XMLDocumentHandler to load saved games.
     * Longcut for setObjectReference(MoveableOwnedObjectReference).
     */
    public void setCraftReference(CraftReference v) {
        setObjectReference(v);
    }
    
    /**
     * Used by XMLDocumentHandler to load saved games.
     * Longcut for setObjectReference(MoveableOwnedObjectReference).
     */
    public void setFleetReference(FleetReference v) {
        setObjectReference(v);
    }
    
    /**
     * Used by XMLDocumentHandler to load saved games.
     * Longcut for setDestinationReference(StellarObjectReference).
     */
    public void setTransportReference(TransportReference v) {
        setObjectReference(v);
    }
    
	public Orbit getDestination() {
		return destination;
	}
	public void setDestination(Orbit destination) {
		this.destination = destination;
	}
	public Orbit getOrigin() {
		return origin;
	}
	public void setOrigin(Orbit origin) {
		this.origin = origin;
	}
    
	public String toString() {
		try {
			return " origin="+origin+" dest="+destination+" object="+((objectReference == null) ? "null" : objectReference.getObject());
		} catch (ApplicationError e) {
			Logger.logError("toString()", e);
			return"Error";
		}
	}
}