/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.crafts;

import mot.common.designs.WeaponBayDesign;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.vo.WeaponBayVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
* This contains a list of weapons in a particular weapons bay on a craft.
 * Each craft hull type has a number of bays, and each hull type may have restrictions
 * on the things that may be stored in different bays and the quantity allowed. Also
 * some bays may be more accurate than others for different types of weapon.
  * 
 * SIMPLE SUGGESTION - 
 *      Only one type of weapon is allowed in a weapon bay?
 *      Hull defines hw many weapons bays a craft has and each has different accuracy details.
*/
public class WeaponBay extends XMLBean {

	/**
	 * The list of weapons.
	 */
	private Weapons weapons;
    
	public WeaponBay() {
	}

	public WeaponBay(WeaponBay wb) throws ApplicationError {
        setWeapons( new Weapons(wb.getWeapons()) );
	}

	public WeaponBay(Weapons weapons) {
		this.weapons = weapons;
	}

	public WeaponBay(WeaponBayVO wb) throws ApplicationError {
		if (wb != null) {
			weapons = new Weapons(wb.getWeapons());
		}
	}

	public WeaponBay(WeaponBayDesign wb) throws ApplicationError {
		if (wb != null) {
			weapons = wb.getWeaponsList();
		}
	}

	public Weapons getWeapons() { return weapons; }
	public void setWeapons(Weapons v) { weapons = v; }
 
	public void setWeapon(String v) throws ApplicationError {
        if (weapons == null) weapons = new Weapons();
		weapons.add(Technologies.getWeapon(v));
	}
    
    public void add(Weapon w) {
        setWeapon(w);
    }
    
    public void setWeapon(Weapon w) {
        if (weapons == null) weapons = new Weapons();
		weapons.add(w);
    }

    public void remove(Weapon w) {
        if (weapons == null) return;
		weapons.remove(w);
    }
    
    /**
     * @return The total quantity of weapons
     */
	public int getQtyWeapons() {
		if (weapons == null) {
			return 0;
		} else {
			return weapons.getQuantity();
		}
	}
    
    /**
     * @return The total quantity of damage inflicted from all weapons. 
     * So if there are three weapons one with damage 1 and two with damage 2 this
     * would return 3.
     */
	public int getDamageSum(boolean includeLimitedShots) {
		if (weapons == null) {
			return 0;
		} else {
			return weapons.getDamageSum(includeLimitedShots);
		}
	}
    
    public boolean contains(String name) throws ApplicationError {
        Weapon w = Technologies.getWeapon(name);
        return contains(w);
    }

    public boolean contains(Weapon w) throws ApplicationError {
        if (weapons == null) {
            return false;
        } else {
            return weapons.contains(w);
        }
    }

	public boolean containsGroup(String groupName) {
        if (weapons == null) {
            return false;
        } else {
            return weapons.containsGroup(groupName);
        }
	}
	
    /**
     * @return True iff other has the same quantity and type of weapons as this.
     */
    public boolean same(WeaponBay other) {
		if (other == null) return false;
        if (weapons == null && other.getWeapons() == null) return true;
        if (weapons == null || other.getWeapons() == null) return false;
		return weapons.same(other.getWeapons());
    }
    
    /**
     * Returns the maximum range of any weapon in this weapons list.
     */
    public long getMaxRange() throws ApplicationError {
		return weapons.getMaxRange();
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		if (weapons != null) {
			weapons.referenceToXML(writer);
		} 
	}

	protected String getElementName() { return "weapon-bay"; }

    public String toString() {
        if (weapons == null) {
        	return "[]";
        } else {
        	return "["+weapons.toString()+"]";
        }
    }

	public WeaponBayDesign getWeaponBayDesign() {
		return new WeaponBayDesign(getWeapons());
	}

}