/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.crafts;

import java.util.ArrayList;

import mot.common.designs.WeaponBayDesign;
import mot.common.designs.WeaponBaysDesign;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBundle;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.vo.WeaponBayVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This is a list of Weapons bays.
 */
public class WeaponBays extends XMLBean {

	/**
	 * The bays that this craft contains.
	 */
	private ArrayList bays;
    
	private transient WeaponBay [] baysArray;

	public WeaponBays() {
	}

	public WeaponBays(WeaponBay wb) throws ApplicationError {
        if (wb != null) {
            this.setWeaponBay(wb);
        }
	}

	public WeaponBays(WeaponBays wb) throws ApplicationError {
        WeaponBay [] list = wb.getBaysAsArray();
        if (list != null) {
            for (int i=0; i<list.length; i++) {
                this.setWeaponBay(new WeaponBay(list[i]));
            }
        }
	}

   	public WeaponBays(WeaponBayVO[] bays) throws ApplicationError {
        if (bays != null) {
            for (int i=0; i<bays.length; i++) {
                this.setWeaponBay(new WeaponBay(bays[i]));
            }
        }
	}

	public WeaponBays(WeaponBaysDesign bays) throws ApplicationError {
        if (bays != null) {
        	WeaponBayDesign[] ba = bays.getBaysAsArray();
        	if (ba != null) {        		
        		for (int i=0; i<ba.length; i++) {
        			this.setWeaponBay(new WeaponBay(ba[i]));
        		}
        	}
        }
	}

	public void setBays(ArrayList v) { bays = v; }

	public ArrayList getBays() { return bays; }
	public WeaponBay [] getBaysAsArray() { 
        if (bays == null) {
            return null;
        } else {
            if (baysArray == null) {
                baysArray = (WeaponBay[])bays.toArray(new WeaponBay[(bays.size())]); 
            }
            return baysArray;
        }
    }
    
	public void add(WeaponBay v) { 
        setWeaponBay(v);
    }
    
	public void setWeaponBay(WeaponBay v) { 
        if (bays == null) bays = new ArrayList();
        baysArray = null;
        bays.add(v); 
    }
    
    public void addWeapon(Weapon w, int bayNo) {
        int qty = getQty();
        if (bayNo > qty) {
            Logger.logError("NO BAY "+bayNo+" - Weapon Ignored", new ApplicationError());
            return;
        } else if (bayNo == qty) {
            WeaponBay wb = new WeaponBay();
            wb.add(w);
            setWeaponBay(wb);
        } else {
            WeaponBay [] wbs = getBaysAsArray();
            wbs[bayNo].add(w);
        }
    }
    
    public WeaponBay getWeaponBay(int bayNo) {
        int qty = getQty();
        if (bayNo >= qty) {
            return null;
        } else {
            WeaponBay [] wbs = getBaysAsArray();
            return wbs[bayNo];
        }
    }

    public void removeWeapon(Weapon w) throws ApplicationError {
        removeWeapon(w, 0);
    }
    
    public void removeWeapon(Weapon w, int bayNo) throws ApplicationError {
        WeaponBay [] wbs = getBaysAsArray();
        if (bayNo >= wbs.length) {
            throw new ApplicationError("WeaponsBays.addWeapon() Unable to add to bay number "+bayNo);
        } else {
            wbs[bayNo].remove(w);
        }
    }
    
    /**
     * @return The total quantity of weapons bays
     */
	public int getQty() {
		if (bays == null) {
			return 0;
		} else {
			return bays.size();
		}
	}
    
    /**
     * @return The total quantity of weapons in all weapons bays
     */
	public int getQtyWeapons() {
		if (bays == null) {
			return 0;
		} else {
            int sum = 0;
            WeaponBay [] bays = getBaysAsArray();
            for (int i=0; i<bays.length; i++) {
                sum += bays[i].getQtyWeapons();
            }
			return sum;
		}
	}
    
    /**
     * @return The total quantity of damage inflicted from all weapons in all weapons 
     * bays. So if there are three weapons one with damage 1 and two with damage 2 this
     * would return 3.
     */
	public int getDamageSum(boolean includeLimitedShots) {
		if (bays == null) {
			return 0;
		} else {
            int sum = 0;
            WeaponBay [] bays = getBaysAsArray();
            for (int i=0; i<bays.length; i++) {
                sum += bays[i].getDamageSum(includeLimitedShots);
            }
			return sum;
		}
	}
    
    /**
     * Returns the space consumed by all weapons in these bays
     */
    public int getSpace() {
        Weapons l [] = getWeapons();
        if (l == null) return 0;
        int space = 0;
        for (int i=0; i<l.length; i++) {
            space += l[i].getSpace();
        }
        return space;
    }
    
    /**
     * Returns the maximum range of any weapon in this weapons list.
     */
    public long getMaxRange() throws ApplicationError {
        WeaponBay [] bays = getBaysAsArray();
        long max = 0;
		for (int i=0; i<bays.length; i++) {
			if (bays[i].getMaxRange() > max) {
				max = bays[i].getMaxRange();
			}
		}
		return max;
    }

    public boolean contains(String name) throws ApplicationError {
        Weapon w = Technologies.getWeapon(name);
        return contains(w);
    }

    public boolean contains(Weapon w) throws ApplicationError {
        WeaponBay [] bays = getBaysAsArray();
        
        if (bays == null) return false;
        
        for (int i=0; i<bays.length; i++) {
            if (bays[i].contains(w)) return true;
        }
        
        return false;
    }

	public boolean containsGroup(String groupName) {
        WeaponBay [] bays = getBaysAsArray();
        
        if (bays == null) return false;
        
        for (int i=0; i<bays.length; i++) {
            if (bays[i].containsGroup(groupName)) return true;
        }
        
        return false;
	}

    /**
     * @return The total cost to build all items in this list. 
     */
   	public int getCostToBuild(double costToBuildHullFactor) {
        WeaponBay [] bays = getBaysAsArray();
        int total = 0;
		for (int i=0; i<bays.length; i++) {
			Weapons weapons = bays[i].getWeapons();
			if (weapons != null) {
				total = weapons.getCostToBuild(costToBuildHullFactor);
			}
		}
        return total;
    }

    /**
     * @return the weapons lists for all bays. Weapons list at index 0 is from bay 0,
     * Weapons list at index 1 is from bay 1, and so on.
     */
   	public Weapons[] getWeapons() {
        WeaponBay [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        Weapons[] r = new Weapons[bays.length];
		for (int i=0; i<bays.length; i++) {
			r[i] = bays[i].getWeapons();
		}
        return r;
    }

    /**
     * @return an array of all weapons in all bays.
     */
   	public Weapon[] getAllWeapons() {
        WeaponBay [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        ArrayList r = new ArrayList();
		for (int i=0; i<bays.length; i++) {
			Weapons weapons = bays[i].getWeapons();
            if (weapons != null) {
                Weapon[] all = weapons.getWeapons();
                for (int j=0; all != null && j<all.length; j++) {
                    r.add( all[j]);
                }
            }
		}
        return (Weapon[])r.toArray(new Weapon[(r.size())]);
    }

    /**
     * @return an array of all weapons of type v in all bays.
     */
   	public Weapon[] getAllWeapons(Weapon v) {
        WeaponBay [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        ArrayList r = new ArrayList();
		for (int i=0; i<bays.length; i++) {
			Weapons weapons = bays[i].getWeapons();
            if (weapons != null) {
                Weapon[] all = weapons.getWeapons();
                for (int j=0; all != null && j<all.length; j++) {
                    if (all[j] == v) {
                        r.add( all[j]);
                    }
                }
            }
		}
        return (Weapon[])r.toArray(new Weapon[(r.size())]);
    }

    /**
     * @return an array of all weapons in all bays.
     */
   	public Weapons getAllWeaponsAsList() {
        WeaponBay [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        Weapons r = new Weapons();
		for (int i=0; i<bays.length; i++) {
			Weapons weapons = bays[i].getWeapons();
            if (weapons != null) {
                Weapon[] all = weapons.getWeapons();
                for (int j=0; all != null && j<all.length; j++) {
                    r.add( all[j]);
                }
            }
		}
        return r;
    }
    
    /**
     * The return value contains a single weapon bundle for each type of weapon. The weapon
     * bundle contains the quantity of this weapon type.
     * @return A list of all weapons in this list
     */
    public WeaponBundle [] getDistinctWeapons() {
        Weapons weapons = getAllWeaponsAsList();
        if (weapons == null) {
            return null;
        }
        return weapons.getDistinctWeapons();
    }

    public String getWeaponNames() {
        return getWeaponNames(" ,");
    }

    public String getWeaponNames(String seperator) {
        WeaponBay [] bays = getBaysAsArray();
        if (bays == null || bays.length == 0) return null;
        StringBuffer sb = new StringBuffer();
        for (int i=0; i<bays.length; i++) {
            sb.append("Weapons Bay "+i+" (");
            Weapons weapons = bays[i].getWeapons();
            if (weapons != null) {
            	sb.append(weapons.getNames(seperator));
            }
            sb.append(") ");
        }
        return sb.toString();
    }

	public WeaponBaysDesign getWeaponBaysDesign() {
		WeaponBay[] ba = getBaysAsArray();
		if (ba == null) {
			return null;
		}
		WeaponBaysDesign r = new WeaponBaysDesign();
		for (int i = 0; i < ba.length; i++) {
			r.add(ba[i].getWeaponBayDesign());
		}
		return r;
	}

    /**
     * @return True iff other has the same quantity and type of weapons as this.
     */
	public boolean same(WeaponBays other) {
		if (other == null) return false;
        WeaponBay [] bays = getBaysAsArray();
        WeaponBay [] obays = other.getBaysAsArray();
        if (bays == null && obays == null) return true;
        if (bays == null || obays == null) return false;
        if (bays.length != obays.length) return false;
		for (int i=0; i<bays.length; i++) {
			if (!bays[i].same(obays[i])) {
				return false;
			}
		}
		return true;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
        WeaponBay [] bays = getBaysAsArray();
		if (bays != null) {
			for (int i=0; i<bays.length; i++) {
				bays[i].toXML(writer);
			}
		}
	}

	protected String getElementName() { return "weapon-bays"; }

    public WeaponBays cloneTechnology() throws ApplicationError {
        try {
            return (WeaponBays)this.clone();
        } catch (CloneNotSupportedException e) {
            throw new ApplicationError("WeaponsBays.cloneTechnology() Failed due to error "+e);
        }
    }
    
    public String toString() {
        WeaponBay [] list = getBaysAsArray();
        if (list == null) {
        	return "[]";
        } else {
        	return "["+Comp.toStringOnAll(list, ",")+"]";
        }
    }

}