/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.event;
import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.event.BattleDamageRingEvent;
import mot.common.event.MotEvent;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.adapter.BattleMineVOAdapter;
import mot.engine.battles.BattleMine;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.players.Player;

/**
 * This event is created when a crafts engines have overloaded causing an explosion.
 * The craft whose warp core is overloaded will have been destroyed. The explosion
 * may have caused several other crafts to be damaged - This event will contains a number
 * of linked CraftDamageEvents containing a craft damage event for each craft that was
 * damaged in the core breach.
 */
public class BattleDamageRingEF extends LinkedEventFactoryBase {

    /**
     * The object that has caused the warp core breach or Shield Burst.
     */
    private BattleObject source;
    
    /**
     * The location the craft was at when the event occurred.
     */
    private Location location;

    /**
     * This is the amount of explosive damage produced by the blast.
     */
    private int damage;

    /**
     * This is the weapon that caused this Damage Ring.
     * This may be NULL - Mines do not have weapons and cause Warp Core Overloads.
     */
	private Weapon weapon;

	/**
	 * The state before the damage ring event.
	 */
	private CraftStateFactory oldState;
	
	/**
	 * The state before the damage ring event.
	 */
	private CraftStateFactory newState;


    public BattleDamageRingEF(BattleObject source, CraftStateFactory oldState, Location location, int damage, Weapon weapon) throws ApplicationError {
        this.source = source;
		this.oldState = oldState;
		this.newState = source.getCraftState();
		this.location = location;
        this.damage = damage;
		this.weapon = weapon;
        if (Logger.isDebug()) {
        	Logger.logDebug("BattleDamageRingEF() source="+source+"@"+location);
        }
    }

    public MotEvent createEvent(Player owner) throws ApplicationError {
    	String weaponName = (weapon == null) ? null : weapon.getName();
    	CraftState os = (oldState != null) ? oldState.getCraftState(owner) : null;
    	CraftState ns = (newState != null) ? newState.getCraftState(owner) : null;
    	BattleDamageRingEvent event = new BattleDamageRingEvent(source.getBattle().getName(), source.getName(), os, ns, location, damage, weaponName);
        setLinkedEvents(event, owner);
        if (source instanceof BattleMine) {
        	event.addStateUpdate(new BattleMineVOAdapter((BattleMine)source, owner));
        }
        return event;
    }

    /**
     * WarpCoreOverload events want craft destroyed events generated while this event is
     * queued to be linked to the warp overload event instead of sent independently.
     * @return Returns true if the event if an CraftDestroyed event.
     * @param event The event that may be linked to this event
     */
// Stupid idea - what if we have two events catching? which one owns the new event?
//    public boolean wantsToLink(MotEventFactory event) {
//        return event instanceof BattleDestroyedEF;
//    }
}