/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.event;

import mot.common.bean.CraftState;
import mot.common.bean.Location;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.MotEvent;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.players.Player;

/**
 * Caused when a craft fires a weapon in a battle.
 * The battle craft source has fired on battle craft target using weapon weapon
 * and has inflicted damage on the hull of target. The shields on target stopped
 * shieldBlockedDamage extra damage from being inflicted on it.
 */
public class BattleWeaponFireEF extends LinkedEventFactoryBase {

    private BattleObject source;
    private BattleObject target;
    private Weapon weapon;
    
    /**
     * The damage inflicted on the targets hull
     */
    private double damage;

    /**
     * The damage inflicted on the targets shields
     */
    private double shieldBlockedDamage;

    /**
     * True is this craft has been destroyed at the point of this animation
     */
    private boolean destroyed;

    /**
     * This is the target location to which to draw the weapons fire to.
     * This is usually null, in which case the weapon fire is drawn to the target
     * battle object.
     */
    private Location targetLocation;

    /**
     * The source craft state before this animations is taking place.
     */
    private CraftStateFactory oldSourceState;

    /**
     * The target craft state before this animations is taking place.
     */
    private CraftStateFactory oldTargetState;

    /**
     * The source craft state while this animations is taking place.
     */
    private CraftStateFactory newSourceState;

    /**
     * The target craft state while this animations is taking place.
     */
    private CraftStateFactory newTargetState;

    /**
     * True if the animation should be the reverse of its normal direction.
     * Normally animation flow from the source to the target, but animations like ShieldDrain can
     * flow both ways.
     */
    private boolean reverseAnimation = false;

    /**
     * Some weapon fire animations have special messages for the user.
     */
    private String sourceDescription;

    /**
     * Some weapon fire animations have special messages for the user.
     */
    private String targetDescription;
    
    /**
     * Weapons that can use different animation images for different times the weapon is fired you can specify
     * a suffix to the name in the configuration for this weapon file animation. e.g. ReplicaBeam. 
     */
    private String animationConfigNameSuffix;
    
    /**
     * This is extra information pertaining the this weapon fire event. It is untypes and its
     * contents are entirely dependent on the type of weapon that generated this event.
     * Example Uses :
     * 	1 - When a spatial distortion weapon fires it passed details of the spatial distortion 
     *  	is has created
     */
    private Object data;
    
    public BattleWeaponFireEF(BattleObject source, BattleObject target, Weapon weapon, double damage, double shieldBlockedDamage, String sourceDescription, String targetDescription, boolean destroyed, CraftStateFactory oldSourceState, CraftStateFactory oldTargetState, CraftStateFactory newSourceState, CraftStateFactory newTargetState) {
        this.source = source;
        this.target = target;
        this.weapon = weapon;
        this.damage = damage;
        this.shieldBlockedDamage = shieldBlockedDamage;
        this.destroyed = destroyed;
        this.oldSourceState = oldSourceState;
        this.oldTargetState = oldTargetState;
        this.newSourceState = newSourceState;
        this.newTargetState = newTargetState;
        this.sourceDescription = sourceDescription;
        this.targetDescription = targetDescription;
        
        if (Logger.isDebug()) {
        	Logger.logDebug("BattleWeaponFireEF() "+source+" -> "+target+" Old Target State="+oldTargetState+" New Target State="+newTargetState);
        }
	}
    
    public MotEvent createEvent(Player owner) throws ApplicationError {
    	if (source != null && newSourceState == null) newSourceState = source.getCraftState(); 
    	if (target != null && newTargetState == null) newTargetState = target.getCraftState();
    	if (oldSourceState == null) oldSourceState = newSourceState;
    	if (oldTargetState == null) oldTargetState = newTargetState;
    	CraftState ots = (oldTargetState != null) ? oldTargetState.getCraftState(owner) : null;
    	CraftState nts = (newTargetState != null) ? newTargetState.getCraftState(owner) : null;
    	CraftState oss = (oldSourceState != null) ? oldSourceState.getCraftState(owner) : null;
    	CraftState nss = (newSourceState != null) ? newSourceState.getCraftState(owner) : null;
    	BattleWeaponFireEvent event = new BattleWeaponFireEvent(
        											source.getBattle().getName(),
        											source.getName(),
        											(target != null) ? target.getName() : null,
                                                    weapon.getName(),
                                                    damage,
                                                    shieldBlockedDamage,
                                                    destroyed,
                                                    targetLocation,
                                                    reverseAnimation,
                                                    oss,
                                                    ots,
                                                    nss,
                                                    nts,
                                                    sourceDescription,
                                                    targetDescription,
                                                    animationConfigNameSuffix,
                                                    data);
        setLinkedEvents(event, owner);
        return event;
    }



    /**
     * Creates a version of this event for remote clients.
     * Remote clients are those in a different JVM so RPC is required.
     * Therefore all MotEvents must be Serializable.
     */



    public void setReverseAnimation(boolean reverseAnimation) {
        this.reverseAnimation = reverseAnimation;
    }


    public boolean isReverseAnimation() {
        return reverseAnimation;
    }


    public void setSource(BattleObject source) {
        this.source = source;
    }


    public BattleObject getSource() {
        return source;
    }


    public void setTarget(BattleObject target) {
        this.target = target;
    }


    public BattleObject getTarget() {
        return target;
    }


    public void setWeapon(Weapon weapon) {
        this.weapon = weapon;
    }


    public Weapon getWeapon() {
        return weapon;
    }


    public void setShieldBlockedDamage(double shieldBlockedDamage) {
        this.shieldBlockedDamage = shieldBlockedDamage;
    }


    public double getShieldBlockedDamage() {
        return shieldBlockedDamage;
    }


    public void setDestroyed(boolean destroyed) {
        this.destroyed = destroyed;
    }


    public boolean isDestroyed() {
        return destroyed;
    }


    public void setOldSourceState(CraftStateFactory oldSourceState) {
        this.oldSourceState = oldSourceState;
    }


    public CraftStateFactory getOldSourceState() {
        return oldSourceState;
    }


    public void setOldTargetState(CraftStateFactory oldTargetState) {
        this.oldTargetState = oldTargetState;
    }


    public CraftStateFactory getOldTargetState() {
        return oldTargetState;
    }


    public void setNewSourceState(CraftStateFactory newSourceState) {
        this.newSourceState = newSourceState;
    }


    public CraftStateFactory getNewSourceState() {
        return newSourceState;
    }


    public void setNewTargetState(CraftStateFactory newTargetState) {
        this.newTargetState = newTargetState;
    }


    public CraftStateFactory getNewTargetState() {
        return newTargetState;
    }


    public void setSourceDescription(String sourceDescription) {
        this.sourceDescription = sourceDescription;
    }


    public String getSourceDescription() {
        return sourceDescription;
    }


    public void setTargetDescription(String targetDescription) {
        this.targetDescription = targetDescription;
    }


    public String getTargetDescription() {
        return targetDescription;
    }

	public Location getTargetLocation() {
		return targetLocation;
	}

	public void setTargetLocation(Location targetLocation) {
		this.targetLocation = targetLocation;
	}

	public Object getData() {
		return data;
	}

	public void setData(Object data) {
		this.data = data;
	}

	public void setAnimationConfigNameSuffix(String animationConfigNameSuffix) {
		this.animationConfigNameSuffix = animationConfigNameSuffix;
	}

	public String getAnimationConfigNameSuffix() {
		return animationConfigNameSuffix;
	}



}