/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.event;

import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.event.MotEvent;
import mot.common.event.NewBattleEvent;
import mot.common.util.ApplicationError;
import mot.engine.adapter.BattlePlanetVOAdapter;
import mot.engine.adapter.BattleStateVOAdapter;
import mot.engine.adapter.BattleVOAdapter;
import mot.engine.adapter.CraftVOAdapter;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.BattleStation;
import mot.engine.battles.HoloCraft;
import mot.engine.common.EngineVoFactory;
import mot.engine.players.Player;

/**
 * Notifies player that a new battle is about to start.
 * At the end of each turn a NewBattleEvent will be generated for each battle that must take
 * place AND has a UserPlayer involved. If more than one battle must be fought at the end of
 * a turn only a single battle will be played at a time -  so NewBattleEvent will be sent out
 * for the first battle but subsequent battles will not have a NewBattleEvent generated until
 * the first battle has completed.
 */
public class NewBattleEF implements MotEventFactory {

    private Battle battle;

    public NewBattleEF(Battle battle) {
        this.battle = battle;
	}

    public MotEvent createEvent(Player owner) throws ApplicationError {
    	NewBattleEvent event = new NewBattleEvent(new BattleVOAdapter(battle), new BattleStateVOAdapter(battle));
        BattleCraft[] crafts = battle.getCrafts();
        if (crafts != null) {
        	for (int i = 0; i < crafts.length; i++) {
        		if (crafts[i] instanceof HoloCraft || crafts[i] instanceof BattleStation) {
                    event.addStateUpdate(new CraftVOAdapter(crafts[i].getCraft(), owner));
        		}
                event.addStateUpdate(EngineVoFactory.getBattleCraft(crafts[i], owner));
			}
        }
        BattlePlanet planet = battle.getPlanet();
        if (planet != null) {
            event.addStateUpdate(new BattlePlanetVOAdapter(planet, owner));
        }
        Automaton[] automatons = battle.getAutomatons();
        if (automatons != null) {
        	for (int i = 0; i < automatons.length; i++) {
        		if (automatons[i].isVisible(owner)) {
        			event.addStateUpdate(EngineVoFactory.getBattleAutomiton(automatons[i], owner));
        		}
			}
        }
        AnimatedBattleAreaDetails[] annimatedBattleAreas = battle.getAnnimatedBattleAreas();
        event.setAnnimatedBattleAreas(annimatedBattleAreas);
		return event;
    }

	public String toString() {
		return "NewBattleEF battle="+battle;
	}


}