/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.leaders;

import mot.common.leaders.GeneralDetails;
import mot.common.leaders.GeneralDetailsList;
import mot.common.leaders.GeneralLeader;
import mot.common.leaders.GovenorDetails;
import mot.common.leaders.GovenorDetailsList;
import mot.common.leaders.GovenorLeader;
import mot.common.leaders.LeaderDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.players.Player;

/**
 * This class is used to assign leaders (generals and governors) to players.
 * 
 * A leader can only be assigned to a single player. This class search all existing players 
 * to exclude existing leaders from those that are still available.
 * 
 * For details of how leaders work @see mot.common.leaders.Leader
 */
public class LeaderFactory {

    private static final int QTY_BATTLE_FOR_SMALL_CHANCE = 1;
    private static final int QTY_BATTLE_FOR_OK_CHANCE = 5;
    
    public static Player _debugMustOfferLeader = null;

    /**
     * Uses a random number generator to allocate a chance to each player to see if the 
     * player should be offered a leader from the available list.
     * 
     * A Player is offered a leader on the following conditions;
     *      1 - The year is divisible by 11 (with no remainder)
     *      2a - The player has fought a battle and A random number between 0 and 5 (exclusive) is 0 (20% of the time)
     *      2b - Or, the player has fought 5 battles and A random number between 0 and 3 (exclusive) is 0 (33% of the time)
     *      3 - This player does not have a general or governor.
     *      
     *  If the player does not have a governor or general then its a 50% chance for each.
     *  If the player has one and not the other then the missing one will be offered. It will
     *  be a random leader from the list of leaders of the type allocated.
     */
    public static void endTurn() throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("LeaderFactory.endTurn()");
        }
        if (_debugMustOfferLeader != null) {
            offerLeaderTo(_debugMustOfferLeader);
            _debugMustOfferLeader = null;
            return;
        }
        int date = MotEngine.instance.getDate();
        if (date % 11 == 0) {
            Player [] players = MotEngine.instance.getPlayers().getPlayers();
            if (players == null) return;
            for (int i=0; i<players.length; i++) {
                if (players[i].canHaveNewLeader()) {
                    int qtyBattlesFought = players[i].getQtyBattles(); 
                    if (qtyBattlesFought >= QTY_BATTLE_FOR_OK_CHANCE) {
                        if (Utils.random(3) == 0) {
                            offerLeaderTo(players[i]);
                        }
                    } else if (qtyBattlesFought >= QTY_BATTLE_FOR_SMALL_CHANCE) {
                        if (Utils.random(5) == 0) {
                            offerLeaderTo(players[i]);
                        }
                    }
                }
            }
        }
    }

    public static void offerLeaderTo(Player player) throws ApplicationError {
        if (player.getGeneralLeader() == null && player.getGovenorLeader() == null) {
            if (Utils.random(2) == 0) {
                offerGeneralLeaderTo(player);
            } else {
                offerGeneralLeaderTo(player);
            }
        } else if (player.getGeneralLeader() == null) {
            offerGeneralLeaderTo(player);
        } else if (player.getGovenorLeader() == null) {
            offerGovenorLeaderTo(player);
        }
    }

    public static void offerGeneralLeaderTo(Player player) throws ApplicationError {
        GeneralDetails [] available = getAvailableGenerals();
        if (available != null && available.length > 0) {
            GeneralDetails leader = available[Utils.random(available.length)];
            if (Logger.isDebug()) {
                Logger.logDebug("LeaderFactory.offerGeneralLeaderTo() Player "+player.getName()+" offered leader "+leader.getName());
            }
            player.offerGeneralLeaderTo(leader);
        }
    }

    public static void addGeneralLeaderTo(Player player, GeneralDetails leader) {
        if (Logger.isDebug()) {
            Logger.logDebug("LeaderFactory.addGeneralLeaderTo() Player "+player.getName()+" getting leader "+leader.getName());
        }
        player.setGeneralLeader(new EngineGeneralLeader(leader));
    }

    public static void offerGovenorLeaderTo(Player player) throws ApplicationError {
        GovenorDetails [] available = getAvailablGovenors();
        if (available != null && available.length > 0) {
            GovenorDetails leader = available[Utils.random(available.length)];
            player.offerGovenorLeaderTo(leader);
            if (Logger.isDebug()) {
                Logger.logDebug("LeaderFactory.offerGovenorLeaderTo() Player "+player.getName()+" offered leader "+leader.getName());
            }
        }
    }

    public static void addGovenorLeaderTo(Player player, GovenorDetails leader) {
        if (Logger.isDebug()) {
            Logger.logDebug("LeaderFactory.addGovenorLeaderTo() Player "+player.getName()+" getting leader "+leader.getName());
        }
        player.setGovenorLeader(new EngineGovenorLeader(leader));
    }
    
    /**
     * @throws mot.common.util.ApplicationError
     * @return A leader called name or null if one does not exist.
     * @param name The name of the leader to find.
     */
    public static LeaderDetails getLeader(String name) throws ApplicationError {
        if (name == null) {
            return null;
        }
        LeaderDetails r = GeneralDetailsList.getInstance().getGeneralDetails(name);
        if (r == null) {
            r = GovenorDetailsList.getInstance().getGovenorDetails(name);
        }
        return r;
    }

    /**
     * @throws mot.common.util.ApplicationError
     * @return A list of all General that are not in use by a player
     */
    public static GeneralDetails [] getAvailableGenerals() throws ApplicationError {
        GeneralDetails [] generals = GeneralDetailsList.getInstance().getGenerals();
        if (generals == null) return null;
        GeneralDetailsList available = new GeneralDetailsList(generals);
        Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return generals;
        for (int i=0; i<players.length; i++) {
            GeneralLeader general = players[i].getGeneralLeader();
            if (general != null) {
                available.remove(general.getName());
            }
        }
        return available.getGenerals();
    }

    /**
     * @throws mot.common.util.ApplicationError
     * @return A list of all General that are not in use by a player
     */
    public static GovenorDetails [] getAvailablGovenors() throws ApplicationError {
        GovenorDetails [] govenors = GovenorDetailsList.getInstance().getGovenors();
        if (govenors == null) return null;
        GovenorDetailsList available = new GovenorDetailsList(govenors);
        Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return govenors;
        for (int i=0; i<players.length; i++) {
            GovenorLeader govenor = players[i].getGovenorLeader();
            if (govenor != null) {
                available.remove(govenor.getName());
            }
        }
        return available.getGovenors();
    }

}