/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.StellarObjectGraphics;
import mot.common.factory.GraphicsFactory;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.XMLWriter;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.common.StellarObjectReference;
import mot.engine.players.Player;

/**
 * This represents a Black Hole on the galaxy map.
 * A Black Hole is a gravity well that moves across the galaxy. Crafts cannot pass through 
 * it AND it damages planets in its path.  
 */
public class BlackHole extends StellarObject {

	/**
	 * Research bonus this resource provides any player that controls (owns) it.
	 */
	private float researchBonus;

    /**
     * true => Player has ability to move in one turn to any other solar system from the black hole and infinite maximum travel distance (range).
     */
    private boolean spaceJumpAbility;
    
	public BlackHole() {
	}

	public BlackHole(String name, int x, int y, float researchBonus,  boolean spaceJumpAbility) {
		super(name, x, y);
		this.researchBonus = researchBonus;
		this.spaceJumpAbility = spaceJumpAbility;
	}
    
    /**
     * Generates new a new StellarObjectReference to this solar system.
     */
    public StellarObjectReference getObjectReference() {
        return new BlackHoleReference(this);
    }
    
	protected String getElementName() { return "black-hole"; }
	
	private transient StellarObjectGraphics stellarObjectGraphics = null;
    
    public StellarObjectGraphics getStellarObjectGraphics() throws ApplicationError {
        if (stellarObjectGraphics == null) {
        	if (spaceJumpAbility) {
        		stellarObjectGraphics = GraphicsFactory.getGalaxyGraphicsList().getWormHoleGraphics();
        	} else {
        		stellarObjectGraphics = GraphicsFactory.getGalaxyGraphicsList().getBlackHoleGraphics();
        	}
            if (stellarObjectGraphics == null) {
                throw new ApplicationError("SolarSystem.getImage() No image defined for "+getName());
            }
        }
        return stellarObjectGraphics;
    }
    
	public boolean hasOwner() throws ApplicationError {
    	StellarObjectOrbital orbital = getOrbital();
        if (orbital != null) {
            return true;
        }
        return false;
	}

	public boolean isOwnedBy(Player player) throws ApplicationError {
    	StellarObjectOrbital orbital = getOrbital();
        if (orbital != null) {
        	return Comp.same(player, orbital.getOwner());
        } else {
        	return false;
        }
	}

	public Player[] getOwners() throws ApplicationError {
    	StellarObjectOrbital orbital = getOrbital();
        if (orbital != null) {
        	Player[] r = new Player[1];
        	r[0] = orbital.getOwner();
        	return r;
        } else {
        	return null;
        }
	}
	
	/**
	 * @param moo The object that wants to move to the destination
	 * @param dest The destination in the galaxy
	 * @return true iff the object moo can space jump to the destination if it was
	 * in orbit around this stellar object.
	 * @throws ApplicationError 
	 */
	public boolean canSpaceJumpTo(MoveableOwnedObject moo, StellarObject dest) throws ApplicationError {
		if (moo.getOrbitStellarObject() == dest) return true;
		Player mooOwner = moo.getOwner();
		if (isSpaceJumpAbility() && isOwnedBy(mooOwner)) {
			return moo.canTravelTo(dest);
		} else {
			return false;
		}
	}

	public boolean hasDefendedEnemyOwner(Player p) throws ApplicationError {
		return hasEnemyOwner(p);
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
        super.encode(writer);
        writer.addElement("research-bonus", researchBonus);
        writer.addElement("space-jump-ability", spaceJumpAbility);
	}
	
	public float getResearchBonus() {
		return researchBonus;
	}

	public void setResearchBonus(float researchBonus) {
		this.researchBonus = researchBonus;
	}

	public boolean isSpaceJumpAbility() {
		return spaceJumpAbility;
	}

	public void setSpaceJumpAbility(boolean spaceJumpAbility) {
		this.spaceJumpAbility = spaceJumpAbility;
	}

//	public int getInfluence(Player p) throws ApplicationError {
//    	StellarObjectOrbital orbital = getOrbital();
//        if (orbital != null) {
//        	return orbital.getInfluence();
//        } else {
//        	return 0;
//        }
//	}

}