/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import java.util.ArrayList;

import mot.common.bean.Location;
import mot.common.bean.SectorInfluenceItem;
import mot.common.bean.SectorInfluenceList;
import mot.common.bean.SectorOwnerGrid;
import mot.common.util.ApplicationError;
import mot.common.util.PropertyManager;
import mot.engine.MotEngine;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.players.Player;

/**
 * This calculates the influence each player has in each sector of the
 * galaxy map. Influence is the player's ability to convince the galaxy's
 * population of their opinions / objectives. If player A has a higher 
 * influence in a sector of the galaxy where they both contains a populated
 * planet. The player with a lower influence will find that some of his
 * population sympathize with the other players cause and this lose some
 * production to the player with more influence.
 * 
 * To speed things up influence is only calculated at the end of every 
 * turn. The influence for each player in the galaxy is stored in a class
 * called InfluenceGrid.
 * 
 * This is a server side implementation of the InfluenceGrid.
 * The class InfluenceGrid is just a data container that holds a map of
 * galactic influence by player. This class also contains code to calculate 
 * the grid with data from the engine.
 */
public class InfluenceGrid {
	
	/**
	 * This is the quantity of influence generation required in a sector before it overflows
	 * in to a neighbouring sector.
	 */
	private static int POPULATION_TO_INFLUENCE_SECTOR;
	
	protected int sectorSize;
	
	/**
	 * This is a data structure used by the clients to display the galaxy map.
	 * It contains information about which player owns which sector.
	 */
	private SectorOwnerGrid sog;

	static {
		try {
			POPULATION_TO_INFLUENCE_SECTOR = PropertyManager.getMandatoryAsInt("influence.cross_sector");
		} catch (Exception e) {
			POPULATION_TO_INFLUENCE_SECTOR = 100;
		}
	}
	
	public InfluenceGrid(int galaxyWidth, int galaxyHeight, int sectorSize, boolean initialise) throws ApplicationError {
		this.sectorSize = sectorSize;
		int width = (int) Math.ceil(galaxyWidth / ((double)sectorSize));
		int height = (int) Math.ceil(galaxyHeight / ((double)sectorSize));
		sog = new SectorOwnerGrid(width,height,sectorSize);
		if (initialise) {
			update();
		}
	}	
	
	public void clear() throws ApplicationError {
		sog.clear();
		Planet [] planets = MotEngine.instance.getPlanets().getPlanets();
		for (int i = 0; i < planets.length; i++) {
			planets[i].setDominated(null);
		}
	}
	
	/**
	 * Populate this influence grid with up to date information about the influence
	 * each player exerts over each sector of the galaxy map.
	 */
	public void update() throws ApplicationError {
		Planets planetsList = MotEngine.instance.getPlanets();
		if (planetsList == null) {
			return;
		}
		clear();
		Planet [] planets = planetsList.getPlanets();
		for (int i = 0; i < planets.length; i++) {
			add(planets[i]);
		}
		StellarObject[] sos = MotEngine.instance.getStellarObjects().getStellarObjects();
		for (int i = 0; i < sos.length; i++) {
			StellarObjectOrbital outpost = sos[i].getOrbital();
			if (outpost != null) {
				Location loc = sos[i].getLocation();
				int col = loc.getX() / sectorSize;
				int row = loc.getY() / sectorSize;
				add(outpost, col, row);
			}
		}
		for (int i = 0; i < planets.length; i++) {
			Player owner = planets[i].getOwner(); 
			if (owner != null) {
				Location loc = planets[i].getStellarObject().getLocation();
				SectorInfluenceList cell = sog.getCellByLocation(loc.getX(), loc.getY());
				if (cell != null && !cell.isOwnedBy(owner.getName())) {
					planets[i].setDominated(Boolean.TRUE);
				}
			}
		}
	}

	private void add(Planet planet) throws ApplicationError {
		Location loc = planet.getSolarSystem().getLocation();
		int col = loc.getX() / sectorSize;
		int row = loc.getY() / sectorSize;
		SectorInfluenceList[][] grid = sog.getGrid();
		if (row >= grid.length) row =  grid.length-1;
		if (col >= grid[0].length) col =  grid[0].length-1;
		if (planet.getOwner() != null) {
			add(planet, col, row);
		}
		OrbitalReference miningStation = planet.getMiningStation();
		if (miningStation != null) {
			OrbitalInfluenceGeneratorAdapter ig = new OrbitalInfluenceGeneratorAdapter(planet, miningStation.getOrbital());
			add(ig, col, row);
		}
	}
	
	private void add(InfluenceGenerator generator, int col, int row) throws ApplicationError {
		if (generator.getOwner() == null) {
			return;
		}
		int influence = generator.getInfluence();
		int originalInfluence = getInfluenceBySector(row, col, generator.getOwner());
		int totalInfluence = influence + originalInfluence;
		SectorInfluenceList cell = sog.getCellBySectorNotNull(row, col);
		String owner = generator.getOwner().getName();
		cell.add(owner, influence);
		int radius = 0;
		int requiredInfluenceToOverflowIntoNextSector = POPULATION_TO_INFLUENCE_SECTOR;
		while (totalInfluence > requiredInfluenceToOverflowIntoNextSector) {
			 radius++;
			 if (originalInfluence > requiredInfluenceToOverflowIntoNextSector) {
				 addSurroundingInfluence(generator, row, col, radius, influence);
			 } else {
				 addSurroundingInfluence(generator, row, col, radius, totalInfluence - requiredInfluenceToOverflowIntoNextSector);
			 }
			 totalInfluence -= requiredInfluenceToOverflowIntoNextSector;
			 originalInfluence -= requiredInfluenceToOverflowIntoNextSector;
			 requiredInfluenceToOverflowIntoNextSector += POPULATION_TO_INFLUENCE_SECTOR*(radius+1);
		}
	}
	
	private void addSurroundingInfluence(InfluenceGenerator generator, int row, int col, int radius, int influence) throws ApplicationError {
		int qtyRows =  getQtyRows();
		int qtyCols =  getQtyCols();
		if (row - radius >= 0) {
			// Add influence row along the top
			addHorizontalInfluence(generator, row-radius, col, radius, influence, qtyCols);
		}
		if (row + radius < qtyRows) {
			// Add influence row along the bottom
			addHorizontalInfluence(generator, row+radius, col, radius, influence, qtyCols);
		}
		if (col - radius >= 0) {
			// Add influence row down the left
			addVerticalInfluence(generator, row, col-radius, radius-1, influence, qtyRows);
		}
		if (col + radius < qtyCols) {
			// Add influence row down the right
			addVerticalInfluence(generator, row, col+radius, radius-1, influence, qtyRows);
		}
	}

	private void addHorizontalInfluence(InfluenceGenerator generator, int row, int centerCol, int radius, int influence, int qtyCols) throws ApplicationError {
		int left = centerCol - radius;
		if (left < 0) left = 0;
		int right = centerCol + radius;
		if (right >= qtyCols) right = qtyCols-1;
		for (int i = left; i <= right; i++) {
			SectorInfluenceList cell = sog.getCellBySectorNotNull(row, i);
			String owner = generator.getOwner().getName();
			cell.add(owner, influence);
		}
	}
	
	private void addVerticalInfluence(InfluenceGenerator generator, int centerRow, int col, int radius, int influence, int qtyRows) throws ApplicationError {
		int top = centerRow - radius;
		if (top < 0) top = 0;
		int bottom = centerRow + radius;
		if (bottom >= qtyRows) bottom = qtyRows-1;
		for (int i = top; i <= bottom; i++) {
			SectorInfluenceList cell = sog.getCellBySectorNotNull(i, col);
			String owner = generator.getOwner().getName();
			cell.add(owner, influence);
		}
	}
	
	public int getInfluenceBySector(int row, int col, Player player) throws ApplicationError {
		SectorInfluenceList cell = sog.getCellBySector(row, col);
		if (cell == null) {
			return 0;
		} else {
			return cell.getInfluence(player.getName());
		}
	}
	
	public int getInfluenceByLocation(int x, int y, Player player) throws ApplicationError {
		if (player == null) return 0;
		SectorInfluenceList cell = sog.getCellBySector(y/sectorSize, x/sectorSize);
		if (cell == null) {
			return 0;
		} else {
			return cell.getInfluence(player.getName());
		}
	}
	
	public Player getOwnerBySector(int row, int col) throws ApplicationError {
		SectorInfluenceItem cell = sog.getSectorOwner(row, col);
		if (cell == null) {
			return null;
		} else {
			return MotEngine.instance.getPlayersNotNull().getPlayer(cell.getPlayer());
		}
	}
	
	public SectorInfluenceItem getSectorOwnersInfluenceByLocation(int x, int y) throws ApplicationError {
		SectorInfluenceItem cell = sog.getSectorOwner(y/sectorSize, x/sectorSize);
		return cell;
	}

	public Player getOwnerByLocation(int x, int y) throws ApplicationError {
		SectorInfluenceItem cell = sog.getSectorOwner(y/sectorSize, x/sectorSize);
		if (cell == null) {
			return null;
		} else {
			return MotEngine.instance.getPlayersNotNull().getPlayer(cell.getPlayer());
		}
	}

	public Player getOwnerByLocation(Location l) throws ApplicationError {
		return  getOwnerByLocation(l.getX(), l.getY());
	}
	
	public Player getOwnerByStellarObject(StellarObject so) throws ApplicationError {
		Location l = so.getLocation();
		return  getOwnerByLocation(l.getX(), l.getY());
	}

	/**
	 * @return A list of influence sectors that lie between location start and finish inclusively.
	 * @throws ApplicationError 
	 */
	public Player[] getSectorOwnersBetween(Location start, Location finish) throws ApplicationError {
		int distance = start.getDistanceTo(finish);
		int step = sectorSize-1;
		ArrayList<Player> r = new ArrayList();
		Location l = new Location(start);
		for (int i = 0; i < distance; i+=step) {
			l.shift(finish, step);
			Player owner = getOwnerByLocation(l);
			if (!r.contains(owner)) {
				r.add(owner);
			}
		}
		if (r.size() == 0) {
			return null;
		} else {
			return r.toArray(new Player[r.size()]);
		}			
	}
	
	public int getQtyCols() {
		return sog.getQtyCols();
	}
	
	public int getQtyRows() {
		return sog.getQtyRows();
	}

	public SectorOwnerGrid getSectorOwnerGrid() {
		return sog;
	}

	public String toString() {
		return sog.toString();
	}

}
