/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.Orbital;
import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectReference;

/**
 * This is a reference to a planet / stellar object and its satellite (orbital). It 
 * contains a reference to a planet (for planet orbitals - space stations / battle 
 * stations) or a reference to a stellar object (for stellar object references - 
 * outposts). 
 * It provides convenient access to an orbital satellite and its owner.
 */
public class OrbitalReference extends XMLBean {

    /**
     * This is a reference to the planet that this orbital is stationed around.
     * This will be null for stations not placed in orbit around a planet.
     */
    private PlanetReference planetReference;
    
    /**
     * This is a reference to the planet that this orbital is stationed around.
     * This will be null for station in orbit around a planet.
     */
    private StellarObjectReference stellarObjectReference;
    
    public OrbitalReference() {
    }

    /**
     * @param planet The planet the orbital referred to by this object is in orbit of.
     */
	public OrbitalReference(Planet planet) {
        planetReference = new PlanetReference(planet);
    }
    
	public OrbitalReference(StellarObject so) {
        stellarObjectReference = so.getObjectReference();
    }
    
    public Planet getPlanet() {
        if (planetReference == null) {
            return null;
        } else {
            return planetReference.getPlanet();
        }
    }
    
    public StellarObject getStellarObject() {
        if (stellarObjectReference == null) {
            return planetReference.getPlanet().getStellarObject();
        } else {
            return stellarObjectReference.getStellarObject();
        }
    }
    
    public Orbital getOrbital() {
        if (planetReference != null) {
            Planet p = planetReference.getPlanet();
            if (p == null) {
                return null;
            }
            return p.getOrbital();
        } else if (stellarObjectReference != null) {
            StellarObject so = stellarObjectReference.getStellarObject();
            if (so == null) {
                return null;
            }
            if (so.getOrbital() != null) {
                return so.getOrbital().getOrbital();
            }
        }
        return null;
    }

    protected void encode(XMLWriter writer) throws ApplicationError {
        if (planetReference != null) {
            planetReference.toXML(writer);
        }
        if (stellarObjectReference != null) {
            stellarObjectReference.toXML(writer);
        }
    }

    protected String getElementName() {
        return "orbital-reference";
    }


    public void setPlanetReference(PlanetReference planetReference) {
        this.planetReference = planetReference;
    }


    public PlanetReference getPlanetReference() {
        return planetReference;
    }


    public void setStellarObjectReference(StellarObjectReference stellarObjectReference) {
        this.stellarObjectReference = stellarObjectReference;
    }


    public StellarObjectReference getStellarObjectReference() {
        return stellarObjectReference;
    }
}