/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.Location;
import mot.common.designs.Orbital;
import mot.common.designs.ProjectDesign;
import mot.common.lang.L;
import mot.common.resources.Resource;
import mot.common.resources.ResourceBoralium;
import mot.common.technology.Factories;
import mot.common.technology.Hull;
import mot.common.technology.Hulls;
import mot.common.technology.Laboratories;
import mot.common.technology.Module;
import mot.common.technology.Modules;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Technology;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;
import mot.engine.Cheat;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.common.OwnedObject;
import mot.engine.common.ProjectBuilder;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Crafts;
import mot.engine.event.MessageEF;
import mot.engine.event.SabotageEF;
import mot.engine.players.Player;
import mot.engine.players.PlayerCharacter;

/**
 * This is a planet in a game. This is one of the fundamental objects in the game.
 * It implements populations, population growth, production, research output and 
 * ownership. It contains the complete status that a planet is currently in.
 * 
 * If a planet is in a sector of space that is owned by another player, it may suffer
 * from a trade imbalance (cultural domination). For details of sector ownership @see
 * InfluenceGrid. If a planet has a trade imbalance then a percentage of its production
 * transfered to the planet that dominates its sector.
 * 
 */
public class Planet extends OwnedObject implements ProjectBuilder, InfluenceGenerator {


	/**
	 * The solar system to which this planet belongs.
	 */
	private SolarSystemReference solarSystemReference;

	/**
	 * The design of craft that this plant is producing.
     * These may be one of the following things; CraftDesign, Weapon, Shield, Scanner.
     * Please use changeProjectTo() to switch production on the planet from one thing 
     * to another instead of setProject().
	 */
	private Project project;

	/**
	 * The quantity of units built up so far producing the project.
	 */
	private int production;

	/**
	 * The weapons mounted on this plant.
	 */
	private Weapons weapons;

	/**
	 * The shields mounted on this plant.
	 */
	private Shields shields;

	/**
	 * The scanners mounted on this plant.
	 */
	private Scanners scanners;

	/**
	 * Qty People on this planet at this point in time. This would normally be a value between
	 * zero and the the maximum population. However, this may be higher than the maximum 
	 * population - for example when a planet's terraform has been reduced due to bio weapons
	 * or industrial population.
	 */
	private double population;

	/**
	 * The quality of the environment this planet provides. It is a fraction applied
	 * to the size of a planet that is used to calculate the maximum population this
	 * planet can support. For example if a planet was size 100 and has a terraform of
	 * 0.8 then it could support a population of 80. 
	 * Terraform quality is associated with a name e.g. Toxic is 0.2 or less, Utopia is
	 * 1.8 or higher, Terran is 1.0 to 1.1.
	 * For details on colonisation-ability of a planet see the documentation on
	 * Specials.
	 */
	private double teraforming;

	/**
	 * The planet size.
	 * This is defined as the quantity of people who could live on this planet if
	 * its environment was teraformed to terran norms. 100 would be a typical home
	 * planet size.
	 */
	private int size;

	/**
	 * Qty factories / industrial units on this planet.
	 */
	private double qtyFactories;

	/**
	 * The percentage of planets effort that is being expended on each task.
     * Any resources left over are added to the players money pool.
     * These should be a value between 0 and 1.
	 */
	private double percentEcology = 0.25;
	private double percentDevelopment = 0.25;
	private double percentResearch = 0.25;
	private double percentProduction = 0.25;

	/**
	 * Flags to indicate if the planets owner should be notified when production
	 * is finished.
	 */
	private boolean notifiedTerraformComplete = true;
	private boolean notifiedFactoriesComplete = true;


    /**
     * If true then at the start and end of every turn this planet will validate the
     * ecology percentage. The ecology percent will be set to the minimum required to
     * maintain a stable ecology when;
     *  1 - Not enough effort was being spent on ecology to maintain the environment
     *  2 - No teraforming is required but the ecology percent was higher than the minimum.
     *  For all other circumstances the percentage will not be modified.
     */
    private boolean autoEcology;

    /**
     * If true then at the start and end of every turn this planet will set the development 
     * percentage. The development percent will be set to zero if no more factories can be 
     * built otherwise it will be set to the development percent in the player character.
     */
    private boolean autoDevelopment;

    /**
     * When true, targetSurplusPercent must have a value and calculate percents will vary the production
     * percent to maintain an unused percent of development output is targetSurplusPercent.
     */
    private boolean varyProduction;

    /**
     * When true, targetSurplusPercent must have a value and calculate percents will vary the research
     * percent to maintain an unused percent of development output is targetSurplusPercent.
     */
    private boolean varyResearch;

    /**
     * This is a percentage of output that the auto calculate percents will modify production or
     * research percent to keep the unused percent at targetSurplusPercent.
     * A value of 0.0 if 0% and a value of 1.0 is 100%. The value must be between 0 and 1.0
     */
    private Double targetSurplusPercent;

	/**
	 * The bonuses this planet provides to its owner.
     * A Value of 0.0 implies no bonus, a value of -0.5 would mean poor (half output)
     * while a value of 1.0 would mean good (double output).
	 */
	private double productionBonus = 0.0;
	private double researchBonus = 0.0;
	private double reproductionBonus = 0.0;
	private int moneyBonus = 0;
	private int boraliumBonus = 0;
    
    /**
     * This is true if the owner of this planet does not want this planet to have a project.
     * Used by user interfaces to avoid prompting a user to select a project every turn when
     * no project has been selected.
     */
    public transient boolean ownerWantsNoProject;

    /**
     * This is the date this planet was colonised;
     */
    private int dateColonised;
    
    /**
     * This is the quantity of ground defence units defending this planet against invasion.
     * This has a default maximum of population but can be increased by technologies like
     * robotic soldiers.
     * This values grows at a value of 1 per turn up to the max value. These units are used to 
     * defend against ground assaults from enemies trying to take this planet.
     */
    private int qtyGroundDefense;
    
    /**
     * This is the quantity of times this planet should repeat the build task assigned to it 
     * as its project.
     */
    private int repeatProject;
    
    /**
     * If this is not null this planet has an artificial satellite.
     * There are multiple types
     * 1 - Battle Station
     *      Used for defence of a planet and increases the population slightly.
     *      When this planet is attacked a new craft will be constructed with no engines, 
     *      and the best shields / beam weapons this player has. @see Battle or 
     *      BattleStationFactory for details.
     * 2 - Mining Station
     *      Used around a planet that does not have any surface population. These act as
     *      outposts or mining stations boosting the production of the 'owning' planet.
     * 3 - Outpost
     *      Used around a planet that does not have any surface population. These act as
     *      refeuling stations for crafts and extend the range.
     */
    private Orbital orbital;
    
    /**
     * If this is not null then this planet has a mining station located on a remote planet.
     * A mining station is a space station orbiting a planet (not its owner) and provides
     * extra research, production, special resources to the that owns it (this planet).
     * 
     * Note that the planet this mining station is in orbit around will have the mining
     * station as its orbital!
     */
    private OrbitalReference miningStation;
    
    /**
     * Resources are rare minerals / planets / crystals etc that provide bonuses to any
     * player controlling them. A player with Resource X on any planet it owns will get 
     * player wide bonuses for X.
     */
    private Resource resource;
    
    /**
     * Determines if this planet has asteroids.
     * If a planet has asteroids then battles located at this planet has Asteroids Fields.
     * Asteroids fields are incumbents that damage all crafts passing through them as well
     * as slow them down. They do not effect weapons fire.
     */
    private boolean hasAsteroids;
    
    /**
     * This is the total accumulated influence on this planet
     */
    private double influence;
    
    /**
     * True iff this planet has natives.
     * If True then qtyNatives is the quantity of natives this planet will use add to the population once this
     * planet is colonised. qtyNatives is not used if this is false.
     */
    private boolean hasNatives = false;
    private int qtyNatives;

    /**
     * A planet is considered to be dominated if it is in a sector controlled by another
     * player. This value is dynamically generated while calculating the influence every 
     * turn.
     */
    private transient Boolean dominated; 

    /**
     * A Planet is blockaded if it has an enemy planet in orbit around it. This requires
     * searching all the crafts in the system and identifying if they belong to an enemy 
     * craft and are located at this solar system - a reasonable timely calculation.
     * This is a cached value that prevents this planet recalculating its blockade status.
     * When a Craft is destroyed and after crafts move Planet.flushBlockadeCache() must be 
     * called.
     * 
     * If this is null it implies the cache needs calculating. Otherwise the method isBlockaded()
     * can be used to calculate its status.
     */
    private transient Blockade blockaded;
    
    /**
     * This is the orbit for objects around this planet. This is dynamically generated as an 
     * orbit is defined as a stellar object and an optional planet. Note that orbits around 
     * a planet do not use the same instance - so do not use equality operator to compare!
     * I have defined this to reduce the quantity of construction of class Orbit. e.g. When
     * a craft wishes to move to a new location can use the orbit of the planet or stellar 
     * object it is going to.
     */
    private transient Orbit orbit;
    
    /**
     * The crisis this planet is experiencing at the moment if one exists.
     */
    private Crisis crisis;

    public Orbit getOrbit() { 
    	if (orbit == null) orbit = new Orbit(getStellarObject(), this); 
    	return orbit; 
    }
    
	public Planet() {
		hasNatives = false;
	}

	public Planet(String name, Player owner, SolarSystem solarSystem) {
		super(name, owner);
		this.solarSystemReference = new SolarSystemReference(solarSystem);

        autoEcology = true;
        autoDevelopment = true;
        
        if (owner != null && MotEngine.instance != null) {
            dateColonised = MotEngine.instance.getDate();
            influence = 1;
        }
        
        hasNatives = false;
	}

	public void setSolarSystem(SolarSystem v) { solarSystemReference = new SolarSystemReference(v); }
	public void setSolarSystemReference(SolarSystemReference v) { solarSystemReference = v; }
	public void setProduction(int v) { production = v; }
	public void setWeapons(Weapons v) { weapons = v; }
	public void setShields(Shields v) { shields = v; }
	public void setScanners(Scanners v) { scanners = v; }
	public void setPopulation(double v) { population = v; }
	public void setTeraforming(double v) { teraforming = v; }
	public void setSize(int v) { size = v; }
	public void setQtyFactories(double v) { qtyFactories = v; }
	public void setPercentEcology(double v) { percentEcology = v; }
	public void setPercentDevelopment(double v) { percentDevelopment = v; }
	public void setPercentResearch(double v) { percentResearch = v; }
	public void setPercentProduction(double v) { percentProduction = v; }
	public void setNotifiedTerraformComplete(boolean v) { notifiedTerraformComplete = v; }
	public void setNotifiedFactoriesComplete(boolean v) { notifiedFactoriesComplete = v; }
	public void setProductionBonus(double v) { productionBonus = v; }
	public void setResearchBonus(double v) { researchBonus = v; }
	public void setReproductionBonus(double v) { reproductionBonus = v; }
	public void setMoneyBonus(int v) { moneyBonus = v; }
	public void setBoraliumBonus(int v) { boraliumBonus = v; }
	public void setAutoEcology(boolean v) { autoEcology = v; }
	public void setAutoDevelopment(boolean v) { autoDevelopment = v; }
	public void setQtyGroundDefense(int v) { qtyGroundDefense = v; }
    public void setHasAsteroids(boolean v) { hasAsteroids = v; }

	public SolarSystem getSolarSystem() { return solarSystemReference.getSolarSystem(); }
	public StellarObject getStellarObject() { return solarSystemReference.getSolarSystem(); }
	public SolarSystemReference getSolarSystemReference() { return solarSystemReference; }
	public int getProduction() { return production; }
	public Weapons getWeapons() { return weapons; }
	public Shields getShields() { return shields; }
	public Scanners getScanners() { return scanners; }
	public double getPopulation() { return population; }
	public double getTeraforming() { return teraforming; }
	public int getSize() { return size; }
	public double getQtyFactories() { return qtyFactories; }
	public double getPercentEcology() { return percentEcology; }
	public double getPercentDevelopment() { return percentDevelopment; }
	public double getPercentResearch() { return percentResearch; }
	public double getPercentProduction() { return percentProduction; }
	public boolean getNotifiedTerraformComplete() { return notifiedTerraformComplete; }
	public boolean getNotifiedFactoriesComplete() { return notifiedFactoriesComplete; }
	public double getProductionBonus() { return productionBonus; }
	public double getResearchBonus() { return researchBonus; }
	public double getReproductionBonus() { return reproductionBonus; }
	public int getMoneyBonus() { return moneyBonus; }
	public int getBoraliumBonus() { return boraliumBonus; }
    public Project getProject() { return project; }
    public ProjectDesign getProjectDesign() { return (project == null) ? null : project.getProjectDesign(); }
	public boolean getAutoEcology() { return autoEcology; }
	public boolean getAutoDevelopment() { return autoDevelopment; }
	public boolean getVaryProduction() { return varyProduction; }
	public boolean getVaryResearch() { return varyResearch; }
	public int getQtyGroundDefense() { return qtyGroundDefense; }
    public boolean getHasAsteroids() { return hasAsteroids; }

    public boolean hasWeapons() { return (weapons != null && weapons.getQuantity() > 0); }
    public boolean hasShields() { return (shields != null && shields.getQuantity() > 0); }
    public boolean hasScanners() { return (scanners != null && scanners.getQuantity() > 0); }
    public boolean hasDefenses() { return (hasWeapons() || orbital != null); }

	public void setVaryProduction(boolean v) { 
		if (v) {
			varyResearch = false;
			targetSurplusPercent = getUnusedPercent();
			if (targetSurplusPercent < 0.0) targetSurplusPercent = 0.0;
			if (targetSurplusPercent > 1.0) targetSurplusPercent = 1.0;
		}
		varyProduction = v; 
	}
	
	public void setVaryResearch(boolean v) { 
		if (v) {
			varyProduction = false;
			targetSurplusPercent = getUnusedPercent();
			if (targetSurplusPercent < 0.0) targetSurplusPercent = 0.0;
			if (targetSurplusPercent > 1.0) targetSurplusPercent = 1.0;
		}
		varyResearch = v; 
	}


    /**
     * A planet is defended if it has defences OR crafts are in orbit to protected it.
     */
    public boolean isDefended() throws ApplicationError { 
        int qtyCrafts = getOwner().getQtyCraftsAt(getStellarObject());
        return hasDefenses() || qtyCrafts > 0;
    }
    
    public int getQtyWeapons() throws ApplicationError { 
        if (weapons != null) {
        	return weapons.getQuantity();
        } else {
        	return 0;
        }
    }
    
    private PlayerCharacter getPlayerCharacter() throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) {
            return null;
        } else {
            return owner.getPlayerCharacter();
        }
    }

    /**
     * Used by XMLDocumentHanler to load saved games. Longcut to setProject().
     */
	public void setWeaponProject(ProjectWeapon v) { setProject(v); }
	public void setShieldProject(ProjectShield v) { setProject(v); }
	public void setScannerProject(ProjectScanner v) { setProject(v); }
    public void setCraftProject(ProjectCraft cd) { setProject(cd);  }
    public void setTransportProject(ProjectTransport v) { setProject(v); }
    public void setTransportDesign(ProjectTransport v) { setProject(v); }
    public void setBattleStationProject(ProjectBattleStation v) { setProject(v); }
    public void setOutpostProject(ProjectOutpost v) { setProject(v); }
    public void setSpaceStationProject(ProjectSpaceStation v) { setProject(v); }
    public void setAsteroidProject(ProjectAsteroids v) { setProject(v); }
    public void setConstructPlanetProject(ProjectConstructPlanet v) { setProject(v); }
    public void setCraftUpgradeProject(ProjectCraftUpgrade v) { setProject(v); }

	public void upgradeWeapons(Weapon to) throws ApplicationError {
		if (weapons != null) {
			Weapon from = weapons.getMaxDamageWeaponWithGroup(to.getGroup());
			if (from != null && from != to) {
				if (to.getCostToResearch() > from.getCostToResearch()) {
					if (Logger.isDebug()) {
						Logger.logDebug("Planet.upgradeWeapons() Planet "+getName()+" upgrade from "+from+" to "+to);
					}
					weapons.upgradeTo(from,to);
				}
			}
		}
		if (project != null && project instanceof ProjectWeapon) {
			Weapon from = ((ProjectWeapon) project).getWeapon();
			if (Comp.same(to.getGroup(), from.getGroup()) && to.getCostToResearch() > from.getCostToResearch()) {
				if (Logger.isDebug()) {
					Logger.logDebug("Planet.upgradeWeapons() Planet Project "+getName()+" upgrade from "+from+" to "+to);
				}
				((ProjectWeapon) project).getWeaponDesign().setName(to);
			}
		}
	}
	
	public void upgradeShield(Shield to) throws ApplicationError {
		if (shields != null) {
			Shield from = shields.getBestDefenseShield();
			if (from != null && from != to) {
				if (to.getPower() > from.getPower()) {
					if (Logger.isDebug()) {
						Logger.logDebug("Planet.upgradeShield() Planet "+getName()+" upgrade from "+from+" to "+to);
					}
					shields.upgradeTo(from,to);
				}
			}
		}
		if (project != null && project instanceof ProjectShield) {
			Shield from = ((ProjectShield) project).getShield();
			if (Comp.same(to.getGroup(), from.getGroup()) && to.getPower() > from.getPower()) {
				if (Logger.isDebug()) {
					Logger.logDebug("Planet.upgradeShield() Planet Project "+getName()+" upgrade from "+from+" to "+to);
				}
				((ProjectShield) project).getShieldDesign().setName(to);
			}
		}
	}

	public void upgradeScanner(Scanner to) throws ApplicationError {
		if (scanners != null) {
			Scanner from = scanners.getBestScanner();
			if (from != null && from != to) {
				if (to.getStrength() > from.getStrength()) {
					if (Logger.isDebug()) {
						Logger.logDebug("Planet.upgradeScanner() Planet "+getName()+" upgrade from "+from+" to "+to);
					}
					if (scanners != null) {
						scanners.upgradeTo(from,to);
					}
				}
			}
		}
		if (project != null && project instanceof ProjectScanner) {
			Scanner from = ((ProjectScanner) project).getScanner();
			if (Comp.same(to.getGroup(), from.getGroup()) && to.getStrength() > from.getStrength()) {
				if (Logger.isDebug()) {
					Logger.logDebug("Planet.upgradeScanner() Planet Project "+getName()+" upgrade from "+from+" to "+to);
				}
				((ProjectScanner) project).getScannerDesign().setName(to);
			}
		}
	}

	public void upgradeOrbital(Hull to) throws ApplicationError {
		if (orbital != null) {
			Hull from = orbital.getHullTech();
			if (from.isBattleStation() && to.getSpace() > from.getSpace()) {
				if (Logger.isDebug()) {
					Logger.logDebug("Planet.upgradeBattleStation() Planet "+getName()+" upgrade from "+from+" to "+to);
				}
				orbital.setHull(to);
			}
		}
		if (project != null && project instanceof ProjectBattleStation) {
			Hull from = ((ProjectBattleStation) project).getBattleStation();
			if (Comp.same(to.getGroup(), from.getGroup()) && to.getSpace() > from.getSpace()) {
				if (Logger.isDebug()) {
					Logger.logDebug("Planet.upgradeBattleStation() Planet Project "+getName()+" upgrade from "+from+" to "+to);
				}
				((ProjectBattleStation) project).getBattleStationDesign().setName(to);
			}
		}
	}

	public void upgradeOrbital(Modules to) throws ApplicationError {
		if (orbital != null && to != null && to.getQuantity() > 0) {
			Module[] ma = to.getModules();
			for (int i = 0; i < ma.length; i++) {
				if (!orbital.hasModule(ma[i])) {
					orbital.addModule(ma[i]);
				}
			}
		}
	}
	
    /**
     * Makes the spare production capacity this planet generates work on the project v passed in.
     * @param v The project to start building.
     */
	public void setProject(Project v) { 
		project = v; 
		ownerWantsNoProject = false; 
	}

	/**
     * Changes the project this planet is building to this new project.
     * This is similar to setProject() but maybe one day I will choose to
     * lose production when you change the project.
     */
    public void changeProjectTo(Project p) {
        setProject(p);
    }

    /**
     * @returns if a private attribute as its type is fluid, but you can get a description 
     * for the project.
     */
	public String getProjectDescription() throws ApplicationError { 
        if (project == null) {
            return L.NO_PROJECT;
        } else {
            int turns = getTurnsToCompleteProject();
            String turnsStr = (turns > 1000) ? L.INFINATE : ""+turns;
            return L.BUILD+" : "+project.getName()+" ["+turnsStr+" "+L.TURNS+"]";
        }
    }

	/**
	 * @returns the quantity turns required to build the current project at the
	 * current work done per turn.
	 * Note This must search for blockading craft - so if you have this calculated
	 *      rewrite this method and produce two methods!
	 */
    public int getTurnsToCompleteProject() throws ApplicationError {
        if (project == null) {
            return 0;
        } else {
        	return getTurnsToCompleteProject(project.getCostToBuildOnPlanet());
        }
    }
    
	/**
	 * @returns the quantity turns required to build the project passed in at the
	 * current work done per turn.
	 */
    public int getTurnsToCompleteProject(Project project) throws ApplicationError {
        if (project == null) {
            return 0;
        } else {
            return getTurnsToCompleteProject(project.getProjectDesign());        	
        }
    }

	/**
	 * @returns the quantity turns required to build the project passed in at the
	 * current work done per turn.
	 */
    public int getTurnsToCompleteProject(ProjectDesign project) throws ApplicationError {
        if (project == null) {
            return 0;
        } else {
        	return getTurnsToCompleteProject(project.getCostToBuildOnPlanet());
        }
    }

    public int getTurnsToCompleteProject(int costToBuildOnPlanet) throws ApplicationError {
        int prodPerTurn = getProductionOutput();
        if (prodPerTurn > 0) {
            double qty = Math.ceil((((double)costToBuildOnPlanet - production)) / prodPerTurn);
            if (qty < 0) qty = 0;
			return (int)qty;
        } else {
            return Integer.MAX_VALUE;
        }
    }
    

    /**
     * @throws ApplicationError 
     * @returns the total effort still remaining to be completed on the current project.
     */
    public int getRemainingProjectProduction() throws ApplicationError {
        if (project == null) {
            return 0;
        } else {
            return project.getCostToBuildOnPlanet() - getProduction();
        }
    }

    /**
     * @throws ApplicationError 
     * @returns the cost to purchase the project at this point in time.
     */
    public int getCostToPurchaseProject() throws ApplicationError {
        return (int) (getRemainingProjectProduction() * Technology.COST_TO_PURCHASE_FACTOR);
    }

    /**
     * Uses the finds owned by this player to purchase the project.
     * This allows a project to be built after on a planet but costs the player money!
     */
    public void purchaseProject() throws ApplicationError, ValidationError {
        if (getTurnsToCompleteProject() == 0) {
            return;
        }
        int cost = getCostToPurchaseProject();
        if (Logger.isDebug()) {
            Logger.logDebug("Planet.purchaseProject() Planet="+getName()+" purchase = "+getProjectDescription()+" for "+cost);
        }
        Player owner = getOwner();
        if (owner.hasFunds(cost)) {
            owner.addFunds(-cost);
            setProduction(project.getCostToBuildOnPlanet());
        } else {
            throw new ValidationError("Insuffecient Funds : Cost is "+cost+" but you only have "+owner.getFunds());
        }
    }
    
    /**
     * The player player has sabotaged this planet. This method destroys 
     * something on this planet and sends a Sabotage event to the owner 
     * of this planet and the player performing the sabotage.
     * @param player The player sabotaging this planet.
     */
    public void sabotage(Player player) throws ApplicationError {
        SabotageEF event = new SabotageEF(this, player);
        int productionSabotaged = 0;
        int factoriesSabotaged = 0;
        int populationSabotaged = 0;
        int r = Utils.random(3);
        if (r == 0 && project != null) {
            // Destroy production
            productionSabotaged = (int)production;
            production = 0;
        } else if (r == 1 && population > 5.0) {
            // Destroy population
            double original = population;
            population = population-(Utils.random(10) + 5);
            if (population < 1) population = 1;
            populationSabotaged = (int)(original - population); 
        } else {
            // Destroy factories
            double original = qtyFactories;
            qtyFactories = Comp.max(0.0, qtyFactories - (Utils.random(10) + 5));
            factoriesSabotaged = (int)(original - qtyFactories);
        }
        if (Logger.isDebug()) {
            Logger.logDebug("Planet.sabotage() "+getName()+" sabotaged by "+player.getName()+" production="+productionSabotaged+" population="+populationSabotaged+" factories="+factoriesSabotaged);
        }
        
        player.notifySabotage(this, player, productionSabotaged, factoriesSabotaged, populationSabotaged);
        getOwner().notifySabotage(this, player, productionSabotaged, factoriesSabotaged, populationSabotaged);
    }


    /**
     * @return an integer value representing the quality of this planet. This is used to 
     * compare planets for colonisation. A planet with a higher score would make a better 
     * colony.
     */
    public int getQualityScore() {
    	int resourceScore = 0;
    	if (resource != null) resourceScore = 100;
        return (int)(size*teraforming + (productionBonus)*20 + (researchBonus)*10) + resourceScore;
    }

    /**
     * @return an integer value representing the quality of this planet's minerals. This is 
     * used to compare planets for mining with a space station. A planet with a higher score 
     * would make a better mining station.
     */
    public int getMiningScore() {
        int resourceScore = (resource == null) ? 0 : 1000; 
        return resourceScore + (int)((productionBonus)*20 + (researchBonus)*10);
    }

    /**
     * @return the percentage of resources unused. Note this may be negative if the amount of 
     * total percents for research, development, production and ecology is > 100.
     * This means this planet will consume money.
     */
    public double getUnusedPercent() {
        double total = percentEcology + percentDevelopment + percentResearch + percentProduction;
        return 1.0-total;
    }


    /**
     * The results of this calculation are cached so that it is only calculated once per turn.
     * @return True iff this planet is blockaded. A planet is blockaded if an enemy has a craft
     * in orbit around this planet.
     */
    public boolean isBlockaded() throws ApplicationError {
        if (blockaded == null) {
            boolean r = false;
            Crafts crafts = MotEngine.instance.getCrafts();
            if (crafts != null) {
                r = crafts.isBlockaded(this);
            }
            blockaded = BlockadeFactory.create(r);
        }
        return blockaded.isBlockaded();
    }

    public void flushBlockadeCache() { blockaded = null; }

    public int getOrbitalPopulation() throws ApplicationError {
        if (orbital == null) {
            return 0;
        } else {
        	double productionBonus = getProductionBonus();
        	int moneyBonus = getMoneyBonus();
            return orbital.getPopulation(productionBonus, moneyBonus);
        }
    }

	public int getMiningStationPopulation() throws ApplicationError {
        if (miningStation != null) {
            Planet mine = miningStation.getPlanet();
            if (mine != null) {
            	return mine.getOrbitalPopulation();
            }
        }
		return 0;
	}


	/**
	 * @return The Maximum population based on the current terraforming and player bonuses.
	 * The qty people who can live on the planet by its owner.
	 */
	public double getMaxPopulation() throws ApplicationError {
		double playerBonus = 1;
        Player origin = getOrigin();
		if (origin != null) {
			playerBonus += origin.getOriginMaxPopulationBonus();
		}
		Player owner = getOwner();
		if (owner != null) {
			playerBonus += owner.getOwnerMaxPopulationBonus();
		}
        int stationPopulation = getOrbitalPopulation();
        int miningStationPopulation = getMiningStationPopulation();
		return size * teraforming * playerBonus + stationPopulation + miningStationPopulation;
	}

	/**
	 * @return The max population this plant could maintain if terraformed to the owners
	 * maximum terraforming ability. This includes the owner population bonuses.
	 */
	public double getHighestMaxPopulation() throws ApplicationError {
		double playerBonus = 1;
        Player origin = getOrigin();
		if (origin != null) {
			playerBonus += origin.getOriginMaxPopulationBonus();
		}
		Player owner = getOwner();
		if (owner != null) {
			playerBonus += owner.getOwnerMaxPopulationBonus();
		}
        if (owner != null && owner.getBiotechnologies() != null) {
            playerBonus *= owner.getBiotechnologies().getTerraformRating();
        }
        int stationPopulation = getOrbitalPopulation();
        int miningStationPopulation = getMiningStationPopulation();
		return size * playerBonus + stationPopulation + miningStationPopulation;
	}

	/**
	 * @return The maximum qty factories allowed on this planet if it is fully populated.
	 */
	public double getMaxFactories() throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) return 0.0;
		Factories factories = owner.getFactories();
        if (factories == null) return 0.0;
		double playerBonus = 1.0;
        Player origin = getOrigin();
		if (origin != null) {
			playerBonus += origin.getFactoryPerPopulationBonus();
		}
		return getMaxPopulation() * factories.getQtyPerPopulation() * playerBonus;
	}

	/**
	 * @return The qty of factories in use on this planet.
     * Note we may have more factories in the planet than the population can use
     * so we cannot simly use qtyFactories!
	 */
	public double getQtyFactoriesInUse() throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) return 0.0;
		Factories factories = owner.getFactories();
        if (factories == null) return 0.0;
		double playerBonus = 1;
        Player origin = getOrigin();
		if (origin != null) {
			playerBonus += origin.getFactoryPerPopulationBonus();
		}
        double qtyPerPop = factories.getQtyPerPopulation();
		return Comp.min(qtyFactories, population * qtyPerPop * playerBonus);
	}
    
	private static double dominationPercent;
	static {
		try {
			dominationPercent = PropertyManager.getMandatoryAsFloat("production.lost.when.dominated");
		} catch (Exception e) {
			dominationPercent = 0.25;
		}
	}
	
	/**
     * A planets output is divided up between development, production 
     * and ecology based on the percents. The value returned is gross
     * and has not been divided.
	 * @return The quantity of work this plant is doing per turn.
	 */
	public double getIndustryWork() throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) return 0.0;
        double r = population;
		Factories factories = owner.getFactories();
        if (factories != null) {
	        int factoryProduction = factories.getProduction();
			r += getQtyFactoriesInUse()*factoryProduction;
        }
    	double tradeBonus = MotEngine.instance.getTradeRoutes().getIndustryBonus(getSolarSystem());
    	if (tradeBonus > 0.0) r = r * (1.0+tradeBonus);
        if (isDominated()) {
        	double prodLost = r * dominationPercent;
        	r -= prodLost;
        }
        return r;
	}

	/**
	 * @return Industry work from population and factories without deductions from domination
	 * or trade bonuses.
	 */
	public double getCoreIndustryWork() throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) return 0.0;
        double r = population;
		Factories factories = owner.getFactories();
        if (factories != null) {
	        int factoryProduction = factories.getProduction();
			r += getQtyFactoriesInUse()*factoryProduction;
        }
        return r;
	}


	public double getMaxIndustryWork() throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) return 0.0;
        double r = population;
		Factories factories = owner.getFactories();
        if (factories != null) {
	        int factoryProduction = factories.getProduction();
			r += getMaxFactories()*factoryProduction;
        }
        return r;
	}

	/**
     * A planets output is divided up between development, production 
     * and ecology based on the percents. The value returned is gross
     * and has not been divided.
	 * @return The quantity of work this plant is doing per turn with laboratory bonuses applied.
	 */
	public double getResearchWork(boolean includeDomination) throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) return 0.0;
        double r = population;
		Laboratories labs = owner.getLaboratories();
        if (labs != null) {
	        int labProduction = labs.getProduction();
			r +=getQtyFactoriesInUse()*labProduction;
        }
    	double tradeBonus = MotEngine.instance.getTradeRoutes().getIndustryBonus(getSolarSystem());
    	if (tradeBonus > 0.0) r = r * (1.0+tradeBonus);
        if (includeDomination && isDominated()) {
        	double prodLost = r * dominationPercent;
        	r -= prodLost;
        }
        return r;
	}


    /**
     * @return the amount of money generated this turn from the orbital station.
     */
    public int getMiningStationMoney() throws ApplicationError {
        if (miningStation == null) {
            return 0;
        } else {
            Planet mine = miningStation.getPlanet();
            double blockadedChange = mine.isBlockaded() ? BLOCKADED_PERCENT_CHANGE : 1.0;
            return (int)(mine.getMoneyBonus() * blockadedChange);
        }
    }
    
    public int getLostMiningStationMoney() throws ApplicationError {
        if (miningStation == null) {
            return 0;
        } else {
            Planet mine = miningStation.getPlanet();
            double blockadedChange = mine.isBlockaded() ? 1.0 - BLOCKADED_PERCENT_CHANGE : 0.0;
            return (int)(mine.getMoneyBonus() * blockadedChange);
        }
    }

    public int getMoneyWasted() throws ApplicationError {
        boolean blockaded = isBlockaded();
		double blockadedChange = blockaded ? BLOCKADED_PERCENT_CHANGE : 1.0;

		double industryWork = getIndustryWork();
		double bonuses = getPlanetProductionBonuses();
		double unusedPercent = getUnusedPercent();
		double amount = industryWork * bonuses * blockadedChange;
		if (project == null) {
        	unusedPercent += getPercentProduction();
        }

        if (unusedPercent <= 0.05) {
        	return 0;
        } else if (unusedPercent <= 0.15) {
        	return (int) (amount * (unusedPercent - 0.05) * 0.3);
        } else if (unusedPercent <= 0.25) {
        	return (int) (amount * 0.1 * 0.3 + amount * (unusedPercent - 0.15) * 0.6);
        } else {
        	return (int) (amount * 0.1 * 0.3 + amount * 0.1 * 0.6 + amount * (unusedPercent - 0.25) * 0.9);
        }
    }

	/**
     * @return the amount of money generated this turn.
     * Note - If the percent effort on this planet are greater than 100% this value may be negative.
     */
    public int getMoneyOutput() throws ApplicationError {
        boolean blockaded = isBlockaded();
		double blockadedChange = blockaded ? BLOCKADED_PERCENT_CHANGE : 1.0;

		double industryWork = getIndustryWork();
		double bonuses = getPlanetProductionBonuses();
		double unusedPercent = getUnusedPercent();
		double amount = industryWork * unusedPercent * bonuses * blockadedChange;
        
		if (project == null) {
            amount += getProductionOutput(true);
        }
        
		double wastedMoney = getMoneyWasted();
		amount = amount - wastedMoney;
		
        int miningStationMoney = getMiningStationMoney();
        
        return (int) Math.round((amount + getMoneyBonus() + miningStationMoney));
    }
 
    private static final double POLUTION_PER_UNIT_POPULATION = 0.1;

	/**
	 * @return the minimum amount of work required on this plant to maintain the planets current
	 * terraform status. i.e. The quantity of ecology work required not prevent pollution of this
	 * plant.
	 */
	public double getMinimumEcologyOutput() throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) return 0.0;
		Factories factories = owner.getFactories();
        if (factories == null) return 0.0;
        int productionPerFactory = factories.getProduction();
        double minEcoPerFactory = factories.getMinEcologyWork();
		double qtyFactoriesInUse = getQtyFactoriesInUse();
		double r = population*POLUTION_PER_UNIT_POPULATION + qtyFactoriesInUse*productionPerFactory*minEcoPerFactory;
        return r;
	}

	/**
	 * @return the minimum percent of planets effort required to prevent planet maximum size
	 * from falling due to pollution.
	 */
	public double getMinimumEcologyPercent() throws ApplicationError {
        double iw = getIndustryWork();
        if (iw == 0.0) {
            return 0.5;
        }
        
		double playerBonus = 1.0;
        Player origin = getOrigin();
        Player owner = getOwner();
		if (owner != null) {
			playerBonus += owner.getOwnerEcologyBonus();
		}
		if (origin != null) {
			playerBonus += origin.getOriginEcologyBonus();
		}
		double r = getMinimumEcologyOutput() / (iw * playerBonus);
        return r;
	}

	public static final double BLOCKADED_PERCENT_CHANGE = 0.5;

	/**
	 * @return the ecology work performed this turn.
	 */
	public double getEcologyOutput() throws ApplicationError {
		if (getOwner() == null) {
			return 0;
		}

		// Calculate the work being performed
        double bonus = 1.0 + getOrigin().getOriginEcologyBonus() + getOwner().getOwnerEcologyBonus();
		double r = getIndustryWork()*percentEcology*bonus;
		return r;
	}

    /**
     * This is the ecology effort required to terraform the lanet by 1 population unit.
     * As a planet it lower quality then it is herder to teraform.
     */
	private static final double WORK_PER_PERCENT_TERRAFORM_10 = 2500; // i.e. 10 Eco Effort => MaxPop+0.004)
	private static final double WORK_PER_PERCENT_TERRAFORM_8 = 5000; // i.e. 10 Eco Effort => MaxPop+0.002)
	private static final double WORK_PER_PERCENT_TERRAFORM_6 = 7000; // i.e. 10 Eco Effort => MaxPop+0.0014)
	private static final double WORK_PER_PERCENT_TERRAFORM_0 = 10000; // i.e. 10 Eco Effort => MaxPop+0.001)

	/**
	 * @return the terraforming ratio growth this turn.
	 */
	public double getTeraformingGrowth() throws ApplicationError {
		Player owner = getOwner();
		if (owner == null) {
			return 0.0;
		}
        boolean blockaded = isBlockaded();
		double blockadedChange = blockaded ? BLOCKADED_PERCENT_CHANGE : 1.0;
		double ecologyWork = getEcologyOutput();
        double workPerPercentTeraform = WORK_PER_PERCENT_TERRAFORM_10;
        if (teraforming < 0.6) workPerPercentTeraform = WORK_PER_PERCENT_TERRAFORM_0;
        else if (teraforming < 0.8) workPerPercentTeraform = WORK_PER_PERCENT_TERRAFORM_6;
        else if (teraforming < 1.0) workPerPercentTeraform = WORK_PER_PERCENT_TERRAFORM_8;
		double terraformWork = ecologyWork - getMinimumEcologyOutput();
		double r = terraformWork / workPerPercentTeraform * blockadedChange;
		
		double maxTerraforming = owner.getMaxTerraforming();
		if (r+teraforming > maxTerraforming) {
			return maxTerraforming - teraforming; 
		} else {
			return r;
		}
	}

	/**
	 * @return the population growth this turn.
	 */
	public double getPopulationGrowth() throws ApplicationError {
		if (getOwner() == null) {
			return 0;
		}

		double r;
		double mp = getMaxPopulation();
		if (population == mp) {
			r = 0;
		} else if (population > mp) {
			r = (mp - population) / 10;
		} else {	
			// Calculate the increase in population
			double reproductionBonus = 1.0 + getReproductionBonus() + 
									   getOwner().getOwnerReproductionBonus() +
									   getOrigin().getOriginReproductionBonus();
	        boolean blockaded = isBlockaded();
			double blockadedChange = blockaded ? BLOCKADED_PERCENT_CHANGE : 1.0;
			double halfMaxPop =  mp / 2;
			if (population < halfMaxPop) {
				r = popGrowthFormula(population) * reproductionBonus * blockadedChange;
			} else {
				r = popGrowthFormula(mp - population) * reproductionBonus * blockadedChange;
			}
		}
		
		if (crisis != null) {
			Double pe = crisis.populationEffect();
			if (pe != null) {				
				r = r - population * pe;
			}
		}
		
		return r;
	}

	private static final double popGrowthFormula(double v) {
		return (v * 0.015) + 0.25;
	}

    /**
     * @return The quantity of colonists that are in transporter crafts enroute to this planet. 
     */
    public int getQtyColonistsOnrouteTo() throws ApplicationError {
        Player owner = getOwner();
        if (owner != null && owner.getTransportManager() != null) {
            return owner.getTransportManager().getQtyColonistsOnrouteTo(this);
        } else {
            return 0;
        }
    }

    /**
     * @return The the quantity of turns the colonists onroute to this planet will take. 
     */
    public int getColonistsEtaOnrouteTo() throws ApplicationError {
        Player owner = getOwner();
        if (owner != null && owner.getTransportManager() != null) {
            return owner.getTransportManager().getColonistsEtaOnrouteTo(this);
        } else {
            return 0;
        }
    }

	/**
	 * @return the amount of work done this turn in developing this planets infrastructure.
	 */
	public double getDevelopmentOutput() throws ApplicationError {
		Player owner = getOwner();
		if (owner == null) {
			return 0;
		}

        boolean blockaded = isBlockaded();
		double blockadedChange = blockaded ? BLOCKADED_PERCENT_CHANGE : 1.0;
        
		// Calculate the production and research
		Player origin = getOrigin();
		boolean includeLevelBonus = (owner == origin);
		double bonus = 1.0 + owner.getOwnerDevelopmentBonus() + origin.getOriginDevelopmentBonus(includeLevelBonus);
		double development = getIndustryWork()*percentDevelopment*bonus;

		return development * blockadedChange;
	}

	/**
	 * @return the factories built this turn.
	 */
	public double getFactoriesGrowth() throws ApplicationError {
        Player owner = getOwner();
        if (owner == null) return 0.0;
		Factories factories = owner.getFactories();
        if (factories == null) return 0.0;

        double cost = factories.getCostToBuild();
		double development = getDevelopmentOutput();

        boolean blockaded = isBlockaded();
		double blockadedChange = blockaded ? BLOCKADED_PERCENT_CHANGE : 1.0;
		return (development / cost) * blockadedChange;
	}

    /**
     * @return The production output produced by the orbital around this planet. 
     */
    public int getMiningStationProduction(double blockadedChange) throws ApplicationError {
        if (miningStation == null) {
            return 0;
        } else {
            Planet mine = miningStation.getPlanet();
            Orbital orbital = mine.getOrbital();
            if (orbital == null || mine.getOwner() == null) return 0;
            Player owner = getOwner();
            Player origin = getOrigin();
    		boolean includeLevelBonus = (owner == origin);
			double bonuses = 1.0 + mine.getProductionBonus() + 
            			     origin.getOriginProductionBonus(includeLevelBonus) + 
            			     owner.getOwnerProductionBonus();
            return (int)(orbital.getProduction() * bonuses * blockadedChange);
        }
    }
    
    public int getMiningStationProduction() throws ApplicationError {
        if (miningStation == null) {
            return 0;
        } else {
        	Planet mine = miningStation.getPlanet();
        	double blockadedChange = mine.isBlockaded() ? BLOCKADED_PERCENT_CHANGE : 1.0;
        	return getMiningStationProduction(blockadedChange);
        }
    }
    
    public int getLostMiningStationProduction() throws ApplicationError {
        if (miningStation == null) {
            return 0;
        } else {
        	Planet mine = miningStation.getPlanet();
        	double blockadedChange = mine.isBlockaded() ? 1.0 - BLOCKADED_PERCENT_CHANGE : 0.0;
        	return getMiningStationProduction(blockadedChange);
        }
    }
    
    public double getPlanetProductionBonuses() throws ApplicationError {
		Player owner = getOwner();
		if (owner == null) {
			return 0;
		}

		Player origin = getOrigin();
		boolean includeLevelBonus = (owner == origin);
		double bonuses = 1.0 + getProductionBonus() + 
						 origin.getOriginProductionBonus(includeLevelBonus) + 
						 owner.getOwnerProductionBonus();
		return bonuses;
    }

    /**
     * Production Output is the total work done on this planets Project each turn.
     * @return The production output produced by this planet. 
     */
	public int getProductionOutput(boolean removeLostProductionFromDominationon) throws ApplicationError {
        boolean blockaded = removeLostProductionFromDominationon ? isBlockaded() : false;
		double blockadedChange = blockaded ? BLOCKADED_PERCENT_CHANGE : 1.0;

		double bonuses = getPlanetProductionBonuses();
		
		if (crisis != null && crisis.productionEffect() != null) bonuses += crisis.productionEffect();
		double v = 0.0;
		if (removeLostProductionFromDominationon) {
			v = getIndustryWork()*percentProduction*bonuses;
		} else {
			v = getCoreIndustryWork()*percentProduction*bonuses;
		}
        
        if (miningStation != null) {
            v += getMiningStationProduction();
        }
        
		return (int) (v * blockadedChange);
	}

	public int getProductionOutput() throws ApplicationError { return getProductionOutput(true); }

    /**
     * @return The research output produced by the orbital around this planet. 
     */
    public int getMiningStationResearch(double blockadedChange) throws ApplicationError {
        if (miningStation == null) {
            return 0;
        } else {
            Planet mine = miningStation.getPlanet();
            Orbital orbital = mine.getOrbital();
            Player owner = mine.getOwner();
			if (orbital == null || owner == null) return 0;
            Player origin = getOrigin();
    		boolean includeLevelBonus = (owner == origin);
            double bonuses = 1.0 + owner.getResearchBonus(includeLevelBonus) + mine.getResearchBonus();
            return (int)(orbital.getResearch() * bonuses * blockadedChange);
        }
    }
    
    public int getMiningStationResearch() throws ApplicationError {
        if (miningStation == null) {
            return 0;
        } else {
            Planet mine = miningStation.getPlanet();
            double blockadedChange = mine.isBlockaded() ? BLOCKADED_PERCENT_CHANGE : 1.0;
            return getMiningStationResearch(blockadedChange);
        }
    }
    
    public int getLostMiningStationResearch() throws ApplicationError {
        if (miningStation == null) {
            return 0;
        } else {
            Planet mine = miningStation.getPlanet();
            double blockadedChange = mine.isBlockaded() ? 1.0 - BLOCKADED_PERCENT_CHANGE : 0.0;
            return getMiningStationResearch(blockadedChange);
        }
    }

    /**
     * @return The research output produced by this planet. 
     */
    public double getPlanetaryResearch(boolean removeLostOutput) throws ApplicationError {
		Player owner = getOwner();
		if (owner == null) {
			return 0;
		}
		
		Player origin = getOrigin();
		boolean includeLevelBonus = (owner == origin);
		
		double bonuses = 1.0 + getResearchBonus() + origin.getResearchBonus(includeLevelBonus);
		double v = getResearchWork(removeLostOutput)*percentResearch*bonuses;
		return v;
    }
    
    /**
     * Research Output is the total work done on this players research Project each 
     * turn by this planet.
     * @return The research output produced by this planet. 
     */
	public int getResearchOutput(boolean removeLostOutput) throws ApplicationError {
		Player owner = getOwner();
		if (owner == null) {
			return 0;
		}

		boolean blockaded = removeLostOutput ? isBlockaded() : false;
		double blockadedChange = blockaded ? BLOCKADED_PERCENT_CHANGE : 1.0;

		double v = getPlanetaryResearch(removeLostOutput)*blockadedChange;

        if (miningStation != null) {
            v += getMiningStationResearch();
        }
        
        float resourceResearchBonus = owner.getResourceCache().getResearchBonus();
        if (resourceResearchBonus > 0.0) {
        	v = v * (1.0f + resourceResearchBonus);
        }

		return (int) v;
	}

	public int getResearchOutput() throws ApplicationError { return getResearchOutput(true); }

	public boolean canHaveMoreFactories() throws ApplicationError {
		return qtyFactories < getMaxFactories();
	}

	public boolean canHaveMorePopulation() throws ApplicationError {
		return getPopulation() < getMaxPopulation();
	}

	public boolean canHaveMoreTerraforming() throws ApplicationError {
        Player owner = getOwner();
		if (owner == null) return false;
        double maxTerraforming = owner.getMaxTerraforming();
        return (teraforming < maxTerraforming);
		//return getMaxPopulation() < getHighestMaxPopulation();
	}
    
    public int getMaxQtyGroundDefense() {
        // TODO This should be improvable by researching techs
        return (int)population;
    }

	public boolean isMiningColony() {
		return population == 0.0;
	}

    /**
     * Ends this turn for this planet. It is on endTurn that planets add this 
     * turns effort to their projects and calculate population growth etc.
     */
	public void endTurn() throws ApplicationError, ValidationError {
		Player owner = getOwner();
		if (owner == null) return;
		if (isMiningColony()) {
	        if (resource != null && resource instanceof ResourceBoralium) {
	        	owner.addBoralium(1);
	        }
	        if (orbital != null) {	        	
	        	influence += orbital.getInfluenceGrowth(getResource()); 
	        }
	        return;
		}

        if (Logger.isDebug()) {
            Logger.logDebug("Planet.endTurn() name="+getName());
        }

        if (autoDevelopment) {
            setDevelopmentPercent();
        }
        if (autoEcology) {
            setEcologyPercent();
        }
        
        // Any resources not being used may be converted to money!
        getOwner().addFunds(getMoneyOutput());

		double teraformGrowth = getTeraformingGrowth();
		teraforming += teraformGrowth;
		if (Logger.isDebug()) {
            Logger.logDebug("Planet.endTurn() teraforming growth="+teraformGrowth+" ["+teraforming+"]");
        }
        double maxTerraforming = getOwner().getMaxTerraforming();
        if (teraforming >= maxTerraforming) {
            teraforming = maxTerraforming;
			if (!notifiedTerraformComplete) {
                double r = percentEcology / 2;
				_setPercents(0, percentDevelopment, percentResearch+r, percentProduction+r);
				getOwner().notifyEcologyFinished(this);
				notifiedTerraformComplete = true;
			}
		} else if (teraforming < 0.2) {
			teraforming = 0.2;
		}

        double popGrowth = getPopulationGrowth();
        if (popGrowth > 0.0) {
			if (canHaveMorePopulation()) {
				population += popGrowth;
				if (Logger.isDebug()) {
	                Logger.logDebug("Planet.endTurn() population growth="+popGrowth+" ["+population+"]");
	            }
	            double mp = getMaxPopulation();
	            if (population > mp) {
	                population = mp;
	            }
			}
        } else if (popGrowth < 0.0) {
        	if (population > 1.0) {
				population += popGrowth;
        		if (population < 1.0) population = 1.0;
				if (Logger.isDebug()) {
	                Logger.logDebug("Planet.endTurn() population growth="+popGrowth+" ["+population+"]");
	            }
        	}
        }

		if (canHaveMoreFactories()) {
			double factoryGrowth = getFactoriesGrowth();
			qtyFactories += factoryGrowth;
			if (Logger.isDebug()) {
                Logger.logDebug("Planet.endTurn() factory growth="+factoryGrowth+" ["+qtyFactories+"]");
            }
		} else if (percentDevelopment > 0) {
			if (!notifiedFactoriesComplete) {
                double r = percentDevelopment / 2;
				_setPercents(percentEcology, 0, percentResearch+r, percentProduction+r);
				getOwner().notifyFactoriesFinished(this);
				notifiedFactoriesComplete = true;
			}
		}
		
		if (crisis != null) {
			if (crisis.hasExpired()) {
				crisis = null;
			} else {
				crisis.endTurn();
			}
		}

		// Add research to players total
		getOwner().addResearch(getResearchOutput());

		// Produce next item
		addProduction(getProductionOutput());
        
        if (qtyGroundDefense < getMaxQtyGroundDefense()) {
            qtyGroundDefense++;
        }

        if (varyProduction || varyResearch) {
        	varyProductionResearchPercents();
        }
        if (autoEcology) {
            setEcologyPercent();
        }
        
        influence += getInfluenceGrowth(); 
        
        if (resource != null && resource instanceof ResourceBoralium) {
        	getOwner().addBoralium(1);
        }
	}

    /**
     * Adds a quantity of production to the planets project. When the quantity of production added
     * if >= the cost of the project the project is built (calls build method on the project)
     * If repeat > 0 then the this project is still active an will attempt to build another.
     * Each object build repeat is decreased by one until repeat is 0. When a project (and all repeats)
     * is complete project is set to null and a no project event is sent to the owner.
     * @param amount The amount of production to add.
     * @throws mot.common.util.ApplicationError
     * @throws mot.common.util.ValidationError
     */
	public void addProduction(int amount) throws ApplicationError, ValidationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Planet.addProduction() amount="+amount+" to "+getProjectDescription());
        }
		if (project == null && amount > 0) {
			getOwner().notifyNoProject(this, null);
		}

		if (project == null) {
			// This effort has already been credited as money
			return;
		}

		production += amount;

		int buildCost = project.getCostToBuildOnPlanet();
		if (buildCost < 1) {
			buildCost = 1;
		}
		if (production >= buildCost) {
            if (project.canBuildProjectNow(this)) {
            	production = production % buildCost;
                project.build(this);
                String description = L.PLANET+" "+getName()+L.HAS_COMPLETED_PROJECT+project.getProjectDesign().getName();
                if (repeatProject > 0) {
                    repeatProject--;
                    MotEngine.instance.broadcast(new MessageEF(description), getOwner());
                } else {
                    project = null;
                    getOwner().notifyNoProject(this, description);
                }
            }
		}
	}
    
    /**
     * Sets the ecology percent to the minimum percentage if it is lower than the minimum level.
     * @param percentIncreaseForTerraforming The amount above the minimum ecology percent to
     * set ecology to if terrforming is required.
     */
    public void setEcologyPercent() throws ApplicationError {
        PlayerCharacter pc = getPlayerCharacter();
        double percentIncreaseForTerraforming = (pc == null) ? 0.1 : pc.getPlayerEcologyPercent();
        double output = getEcologyOutput();
        double min = getMinimumEcologyOutput();
        if (output < (min+percentIncreaseForTerraforming)) {
            if (canHaveMoreTerraforming()) {
                setPercentEcology(getMinimumEcologyPercent()+percentIncreaseForTerraforming);
            } else {
                setPercentEcology(getMinimumEcologyPercent());
            }
        } else if (output > min) {
            if (canHaveMoreTerraforming()) {
                setPercentEcology(getMinimumEcologyPercent()+percentIncreaseForTerraforming);
            } else {
                setPercentEcology(getMinimumEcologyPercent());
            }
        }
    }

    public void setDevelopmentPercent() throws ApplicationError {
//    	if (getName().startsWith("Yugar")) {
//    		Logger.logFull("");
//    	}
        if (canHaveMoreFactories()) {
            PlayerCharacter pc = getPlayerCharacter();
            setPercentDevelopment(pc.getPlayerDevelopmentPercent());
            notifiedFactoriesComplete = true;
        } else {
            setPercentDevelopment(0);
        }
        
    }

    /**
     * Sets the planetary percentages based on a best effort algorithm using the player
     * character defined percentages. This is defined so planets on self manage or for
     * use by computers players. Call this function and you should know that the percentages
     * are sensible.
     */
	public void setPercentsAuto(boolean includeProduction, boolean includeResearch, boolean useIsolationist) throws ApplicationError {
        if (getOwner() == null || population == 0.0) {
            return;
        }

        PlayerCharacter pc = getPlayerCharacter();
        setPercentDevelopment(pc.getPlayerDevelopmentPercent());

        setEcologyPercent();
        setDevelopmentPercent();
        
        double remaining = 1.0 - (percentEcology + percentDevelopment + pc.getPlayerMoneyPercent());
        
        if (useIsolationist) {
        	if (includeProduction) {
        		setPercentProduction(pc.getPlayerIsolationistProductionPercent() * remaining);
        	}
        	if (includeResearch) {
        		setPercentResearch(pc.getPlayerIsolationistResearchPercent() * remaining);
        	}
        } else {
        	if (includeProduction) {
        		setPercentProduction(pc.getPlayerProductionPercent() * remaining);
        	}
        	if (includeResearch) {
        		setPercentResearch(pc.getPlayerResearchPercent() * remaining);
        	}
        }

        // This would have changed the minimum ecology percent!
        setEcologyPercent();

		if (Logger.isDebug()) {
            Logger.logDebug("Planet.setPercentsAuto() Eco="+percentEcology+" Dev="+percentDevelopment+" Res="+percentResearch+" Prod="+percentProduction);
        }
    }

	public void setPercents(double pEcology,
							double pDevelopment,
							double pResearch,
							double pProduction) {
		_setPercents(pEcology, pDevelopment, pResearch, pProduction);
		notifiedTerraformComplete = true;
		notifiedFactoriesComplete = true;
	}

	private void _setPercents(double pEcology,
				 	 		  double pDevelopment,
							  double pResearch,
							  double pProduction) {
		if (Logger.isDebug()) {
            Logger.logDebug("Planet._setPercents()");
        }
		percentEcology 		= pEcology;
		percentDevelopment 	= pDevelopment;
		percentResearch 	= pResearch;
		percentProduction 	= pProduction;
	}
	
	public void varyProductionResearchPercents() throws ApplicationError {
		if (varyProduction && targetSurplusPercent != null) {
			double unusedPercent = getUnusedPercent();
			double difference = Comp.difference(unusedPercent, targetSurplusPercent);
            if (difference > 0.01) {
            	percentProduction += unusedPercent - targetSurplusPercent;
                if (autoEcology) {
                	setEcologyPercent();
                }
            }
		} else if (varyResearch && targetSurplusPercent != null) {
			double unusedPercent = getUnusedPercent();
			double difference = Comp.difference(unusedPercent, targetSurplusPercent);
            if (difference > 0.01) {
            	percentResearch += unusedPercent - targetSurplusPercent;
            }
		}
	}
	
	private static double InfluenceGeneratedPerPopulation = -1;

	private static double getInfluenceGeneratedPerPopulation() throws ApplicationError {
		if (InfluenceGeneratedPerPopulation < 0.0) {
			InfluenceGeneratedPerPopulation = PropertyManager.getMandatoryAsFloat("influence.prod_cost");
		}
		return InfluenceGeneratedPerPopulation;
	}

    /**
     * The influence is calculated using the formula below;
     * Influence = Total Pop in Area * influence bonuses
     * @return The influence this planet generates per turn 
     */
	public double getInfluenceGrowth() throws ApplicationError {
		double bonus = 1.0 + getOwner().getInfluenceBonus();
		return population * getInfluenceGeneratedPerPopulation() * bonus;
	}

    public int getInfluence() { return (int) influence;  }
    public double getInfluenceAsDouble() { return influence;  }
    public void setInfluence(double v) { influence = v;  }
	
    /**
     * Converts this planet into a colony owned by the player who owns the craft used to 
     * colonise this planet.
     * @param craft The colony craft creating this colony.
     * @throws mot.common.util.ApplicationError
     */
	public void colonise(Craft craft) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("Planet.colonise() Planet "+getName()+" colonised by "+craft.getName());
        }
        
        Craft []  enemies = craft.getOwner().getEnemyCraftAt(getStellarObject(), false);
        if (enemies != null && enemies.length > 0) {
            Logger.logWarning("Planet.colonise() Planet "+getName()+" cannot be colonised by "+craft.getName()+" because enemies are in orbit");
            return;
        }
        if (getOwner() != null) {
            Logger.logWarning("Planet.colonise() Planet "+getName()+" cannot be colonised by "+craft.getName()+" because it is already colonised / mined");
            return;
        }
        if ( getStellarObject().hasOwner() && !getStellarObject().isOwnedBy(craft.getOwner()) ) {
            Logger.logWarning("Planet.colonise() Planet "+getName()+" cannot be colonised by "+craft.getName()+" because solar system is owned by another player");
            return;
        }
		// colonise planet
        dateColonised = MotEngine.instance.getDate();
		setOwner(craft.getOwner());
		setOrigin(null);
		
		if (hasNatives) {
			population = qtyNatives;
			qtyNatives = 0;
			if (population < 5) population  = 5; // colonised for a second time
			qtyFactories = population / 2;
		} else {
			population = 5;
			qtyFactories = 0;
		}
		production = 0;
		crisis = null;
		influence = 1;
        
        if (orbital != null) {
            orbital.notifyPlanetColonised();
        }

		// notify owner
        getOwner().flushCache();
		getOwner().notifyPlanetColonised(this);

		// delete craft
		craft.destroy();
	}
	
    /**
     * Places a satellite in orbit around this planet. If this planet does not have 
     * an owner then the owner is set to the owner of the orbital.
     * @param owner The owner of the orbital structure to deploy
     * @param orbital The satellite to deploy around this planet
     */
    public void deploy(Planet ownerPlanet, Orbital orbital) throws ApplicationError {
    	if (orbital == null) {
    		return;
    	}
    	Player newOwner = ownerPlanet.getOwner();
    	if (newOwner == null) {
    		// Owner planet destroyed during flight to deploy
    		return;
    	}
        if (Logger.isInfo()) {
            Logger.logInfo("Planet.deploy() Planet "+getName()+" deploy orbital "+orbital.getName());
        }
        
        if (getOwner() != null) {
            Logger.logWarning("Planet.deploy() Planet "+getName()+" cannot be orbited by "+orbital.getName()+" becuase it is already owned");
            return;
        }

        if (ownerPlanet.getMiningStation() == null) {
            ownerPlanet.setMiningStation(new OrbitalReference(this));
        }
		setOwner(newOwner);
        setOrbital(orbital);
        influence = orbital.getInfluence(resource);
        getOwner().flushCache();
        population = 0.0;
        crisis = null;
    }
    
    /**
     * This is called to destroy the orbital. This removes this orbital from the planet which 
     * it orbits and removes any references any other planets may have to this orbital.
     */
    public void destroyOrbital() throws ApplicationError {
        Orbital orbital = getOrbital();
        if (orbital != null) {
            MotEngine.instance.getPlanets().removeMiningColony(orbital);
            setOrbital(null);
            if (population == 0.0) {
                getOwner().flushCache();
                setOwner(null);
            }
        }
    }
    
    
    /**
     * Removes the colony from this planet - The planet still exists but it is no 
     * longer a colony of a player. This removes this planet from the owner's list
     * of planets and removes the population, and orbital. It also set the origin
     * and owner to null.
     */
    public void removeColony(boolean mustRemoveTradeRoutes) throws ApplicationError {
        if (miningStation != null) {
            miningStation.getPlanet().destroyOrbital();
        }
        if (orbital != null) { 
            MotEngine.instance.getPlanets().removeMiningColony(orbital);
        }
        SolarSystem solarSystem = getSolarSystem();
        if (mustRemoveTradeRoutes || isOnlyColonyInSolarSystem()) {
        	MotEngine.instance.getTradeRoutes().remove(solarSystem);
        }
        Player owner = getOwner();
        if (owner != null) {
            owner.flushCache();
        }
        setOwner(null);
        setOrigin(null);
        setPopulation(0.0);
        setOrbital(null);
        setMiningStation(null);
        setQtyFactories(0.0);
        setInfluence(0);
        setProject(null);
        setQtyGroundDefense(0);
        setWeapons(null);
        setShields(null);
        setScanners(null);
        crisis = null;
    }

    public boolean isOnlyColonyInSolarSystem() {
    	Planet[] planets = getSolarSystem().getPlanets();
    	for (int i = 0; i < planets.length; i++) {
			if (planets[i] != this && planets[i].getPopulation() > 0.0) {
				return false;
			}
		}
		return true;
	}

	/**
     * @return the strength of the scanners placed on this planet.
     * This determines how far out this planets view of the galaxy extends.
     */
	public int getScanStrength(int distance) {
		if (scanners == null) {
			return 0;
		} else {
			return scanners.getStrength(distance);
		}
	}

    /**
     * @return the strength of the scanners placed on this planet at location loc.
     * This determines how powerful this planets view of this galaxy location.
     */
	public int getScannerStrengthAt(Location loc) {
		int v = getScanStrength(getSolarSystem().getLocation().getDistanceTo(loc));
		if (v < 0) v = 0;
		return v;
	}

    /**
     * A Planets viewability should not be confused with a solar systems visibility. 
     * If a solar system is visible then the planets may be viewed, but not the 
     * population/production/defence details. If a planet is viewable then all details of 
     * the planet may be viewed.
     * 
     * A planet is viewable by a player if that player owns it or has spies assigned to it.
     * 
     * @return true if player p can view the details of this planet.
     * If true the player can use the SolarSystemWindow to examine the planets details.
     */
	public boolean isViewableBy(Player p) throws ApplicationError {
		return (isOwnedBy(p) || getStellarObject().getQtySpies(p) > 0);
	}

	public void changeOwner(Player newOwner) throws ApplicationError {
		super.changeOwner(newOwner);
		flushBlockadeCache();
        StellarObject so = getStellarObject();
        if (so.getLastVisitedDate(newOwner) < 0) {
        	so.addVisitor(newOwner, false);
        }
        if (miningStation != null && miningStation.getPlanet() != null) {
        	miningStation.getPlanet().changeOwner(newOwner);
        }
        Weapons newOwnerWeapons = newOwner.getWeapons();
        if (weapons != null && newOwnerWeapons != null) {
        	String [] groups = weapons.getGroups();
        	if (groups != null) {
        		for (int i = 0; i < groups.length; i++) {
        			Weapon newWeapon = (Weapon) newOwnerWeapons.getMostAdvancedWithGroup(groups[i]);
        			if (newWeapon != null) {
        				upgradeWeapons(newWeapon);
        			}
				}
        	}
        }
        Shields newOwnerShields = newOwner.getShields();
        if (shields != null && newOwnerShields != null) {
        	String [] groups = shields.getGroups();
        	if (groups != null) {
        		for (int i = 0; i < groups.length; i++) {
        			Shield newShield = (Shield) newOwnerShields.getMostAdvancedWithGroup(groups[i]);
        			if (newShield != null) {
        				upgradeShield(newShield);
        			}
				}
        	}
        }
        Scanners newOwnerScanners = newOwner.getScanners();
        if (scanners != null && newOwnerScanners != null) {
        	String [] groups = scanners.getGroups();
        	if (groups != null) {
        		for (int i = 0; i < groups.length; i++) {
        			Scanner newScanner = (Scanner) newOwnerScanners.getMostAdvancedWithGroup(groups[i]);
        			if (newScanner != null) {
        				upgradeScanner(newScanner);
        			}
				}
        	}
        }
        Hulls newOwnerHulls = newOwner.getHulls();
        if (newOwnerHulls != null) {
        	Hull newHull = newOwnerHulls.getBiggestBattleStationHull();
        	if (newHull != null) {
        		upgradeOrbital(newHull);
        	}
        }
        Modules newOwnerModules = newOwner.getModules();
        if (newOwnerModules != null) {
        	upgradeOrbital(newOwnerModules);
        }
        crisis = null;
        if (!isMiningColony()) {
        	SolarSystem solarSystem = getSolarSystem();
        	MotEngine.instance.getTradeRoutes().remove(solarSystem);
        }
	}
	
	/**
     * @return The total maintenance cost for all items installed on this planet.
     * This is the sum of the upkeep cost for weapons, shields and scanners.
	 */
	public int getMaintenanceCost() throws ApplicationError {
        int cost = 0;
        if (weapons != null) {
            cost += weapons.getMaintenanceCost();
        }
        if (shields != null) {
            cost += shields.getMaintenanceCost();
        }
        if (scanners != null) {
            cost += scanners.getMaintenanceCost();
        }
        if (orbital != null && orbital.getMaintenanceCost() > 0) {
            cost += orbital.getMaintenanceCost();
        }
       	double mcf = MotEngine.getPlanetMaintenanceFactor();
		return (int) (cost * mcf);
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("production", production);
		writer.addElement("population", population);
		writer.addElement("teraforming", teraforming);
		writer.addElement("size", size);
		writer.addElement("qty-factories", qtyFactories);
		writer.addElement("percent-ecology", percentEcology);
		writer.addElement("percent-development", percentDevelopment);
		writer.addElement("percent-research", percentResearch);
		writer.addElement("percent-production", percentProduction);
		writer.addElement("notified-terraform-complete", notifiedTerraformComplete);
		writer.addElement("notified-factories-complete", notifiedFactoriesComplete);
		writer.addElement("production-bonus", productionBonus);
		writer.addElement("research-bonus", researchBonus);
		writer.addElement("reproduction-bonus", reproductionBonus);
		writer.addElement("money-bonus", moneyBonus);
		writer.addElement("boralium-bonus", boraliumBonus);
		writer.addElement("auto-ecology", autoEcology);
		writer.addElement("auto-development", autoDevelopment);
		writer.addElement("vary-production", varyProduction);
		writer.addElement("vary-research", varyResearch);
		writer.addElement("target-surplus-percent", targetSurplusPercent);
		writer.addElement("date-colonised", dateColonised);
		writer.addElement("qtyGroundDefense", qtyGroundDefense);
		writer.addElement("repeat-project", repeatProject);
		writer.addElement("has-asteroids", hasAsteroids);
		writer.addElement("influence", influence);
		if (hasNatives) {
			writer.addElement("has-natives", hasNatives);
			writer.addElement("qty-natives", qtyNatives);
			
		}
		if (solarSystemReference != null) {
			solarSystemReference.toXML(writer);
		}
		if (project != null) {
			project.toXML(writer);
		}
		if (weapons != null) {
			weapons.referenceToXML(writer);
		}
		if (shields != null) {
			shields.referenceToXML(writer);
		}
		if (scanners != null && !scanners.contains(Cheat.getSuperScanner())) {
			scanners.referenceToXML(writer);
		}
        if (orbital != null) {
            orbital.toXML(writer);
        }
        if (miningStation != null) {
            miningStation.toXML(writer, "mining-station");
        }
        if (resource != null) {
            resource.toXML(writer, "resource");
        }
		if (crisis != null) {
			crisis.toXML(writer);
		}

	}

	protected String getElementName() { return "planet"; }


    public String toStringPlanetDetails() {
        try {
            return 
                "\nPlanet Name = "+getName()+
                "\nmaxPop = "+getMaxPopulation()+
                "\nPopulation = "+population+"\t+"+getPopulationGrowth()+
                "\nFactories = "+qtyFactories+"\t+"+getFactoriesGrowth()+
                "\nTeraforming = "+teraforming+"\t+"+getTeraformingGrowth()+"\tminEcoOutput = "+getMinimumEcologyOutput()+
                "\nOutputs= Prod "+getProductionOutput()+" Res "+getResearchOutput()+" Eco "+getEcologyOutput()+" Dev "+getDevelopmentOutput()+"\tMoney "+getMoneyOutput()+
                "\nProject = "+getProjectDescription();
        } catch (ApplicationError e) { return "Error"+e; }
    }

    public int getDateColonised() {
        return dateColonised;
    }

    public void setDateColonised(int dateColonised) {
        this.dateColonised = dateColonised;
    }

    public void setRepeatProject(int repeatProject) {
        this.repeatProject = repeatProject;
    }

    public int getRepeatProject() {
        return repeatProject;
    }


    public void setOrbital(Orbital orbital) {
        this.orbital = orbital;
    }

    public Orbital getOrbital() {
        return orbital;
    }


    public void setMiningStation(OrbitalReference miningStation) {
        this.miningStation = miningStation;
    }


    public OrbitalReference getMiningStation() {
        return miningStation;
    }
    
    /**
     * @return The miner of this planet IF this planet is a mining colony, or null otherwise.
     * @throws ApplicationError
     */
    public Planet getMiner() throws ApplicationError {
        if (orbital != null) {
        	return MotEngine.instance.getPlanets().getOwnerOfMiningStation(orbital);
        } else {
        	return null;
        }
    }


    public void setResource(Resource resource) {
        this.resource = resource;
    }


    public Resource getResource() {
        return resource;
    }

	public Boolean getDominated() {
		return dominated;
	}

	public void setDominated(Boolean v) {
		dominated = v;
	}
	
	public boolean isDominated() { return dominated != null && dominated.booleanValue(); }

	public boolean hasNatives() {
		return hasNatives;
	}

	public void setHasNatives(boolean hasNatives) {
		this.hasNatives = hasNatives;
	}

	public int getQtyNatives() {
		return qtyNatives;
	}

	public void setQtyNatives(int qtyNatives) {
		this.qtyNatives = qtyNatives;
	}

	public Crisis getCrisis() {
		return crisis;
	}

	public void setCrisis(Crisis crisis) {
		this.crisis = crisis;
	}

	public void setCrisisPlague(CrisisPlague crisis) {
		this.crisis = crisis;
	}

	public void setCrisisPsi(CrisisPsi crisis) {
		this.crisis = crisis;
	}

	public void setCrisisEmergency(CrisisEmergency crisis) {
		this.crisis = crisis;
	}

	public Double getTargetSurplusPercent() {
		return targetSurplusPercent;
	}

	public void setTargetSurplusPercent(Double targetSurplusPercent) {
		this.targetSurplusPercent = targetSurplusPercent;
	}



}