/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.GameConstants;
import mot.common.setup.RaceOptions;
import mot.common.technology.PlanetConstruction;
import mot.common.util.ApplicationError;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.players.Player;
import mot.engine.players.PlayerCharacter;
import mot.engine.setup.SetupPlayerCharacter;

/**
 * This creates new instances of planets around a solar system.
 */
public class PlanetFactory  {
	
	public static final String [] PLANET_SUFFIXES = {"A","B","C","D","E","F","G"};

    public static String getDefaultName(SolarSystem ss) throws ApplicationError {
    	if (ss.getQtyPlanets() > PLANET_SUFFIXES.length) {
    		throw new ApplicationError("Too many plants in an SS "+ss.getQtyPlanets());
    	}
    	String name = ss.getName() + "-" + (PLANET_SUFFIXES[ss.getQtyPlanets()]);
    	return name;
    }
	
    public static Planet createNewPlanet(Player owner, PlanetConstruction planetConstructionTech, SolarSystem ss) throws ApplicationError {
    	String name = getDefaultName(ss);
    	Planet planet = new Planet(name, owner, ss);
        planet.setSize( planetConstructionTech.getPlanetSize() );
        planet.setPopulation( (owner != null) ? planetConstructionTech.getPlanetPopulation() : 0 );
        planet.setTeraforming( planetConstructionTech.getPlanetTerraform() );
        planet.setQtyFactories( (owner != null) ? planetConstructionTech.getPlanetQtyFactories() : 0 );
        planet.setProductionBonus( planetConstructionTech.getPlanetProductionBonus() );
        planet.setResearchBonus( planetConstructionTech.getPlanetResearchBonus() );
        planet.setReproductionBonus( planetConstructionTech.getPlanetReproductionBonus() );
        
        add( planet );
        
        return planet;
    }

    /**
     * Create a new Planet that is the home world for the race passed in. All the player bonuses
     * for the home world set in the players character are used to create this planet.
     * @throws mot.common.util.ApplicationError
     */
    public static Planet createHomePlanet(Player owner, SolarSystem ss, Double gameLevel) throws ApplicationError {
    	String name = getDefaultName(ss);
    	return createHomePlanet(name, owner, ss, gameLevel);
    }
    
    public static Planet createHomePlanet(String name, Player owner, SolarSystem ss, Double gameLevel) throws ApplicationError {
        Planet planet = new Planet(name, owner, ss);
        RaceOptions raceOptions = null;
        PlayerCharacter pc = owner.getPlayerCharacter();
        if (pc instanceof SetupPlayerCharacter) {
            SetupPlayerCharacter spc = (SetupPlayerCharacter) pc;
            raceOptions = spc.raceOptions;
        } else {
            throw new ApplicationError("PlanetFactory.createHomePlanet() Player Character should be an instance of SetupPlayerCharacter while creating a game");
        }
        planet.setSize( raceOptions.getIntForName("home-planet-size") );
        planet.setPopulation( raceOptions.getDoubleForName("home-planet-population") );
        planet.setTeraforming( raceOptions.getDoubleForName("home-planet-teraforming") );
        planet.setQtyFactories( raceOptions.getIntForName("home-planet-qty-factories") );
        planet.setProductionBonus( raceOptions.getDoubleForName("home-planet-production-bonus") );
        planet.setResearchBonus( raceOptions.getDoubleForName("home-planet-research-bonus") );
        planet.setReproductionBonus( raceOptions.getDoubleForName("home-planet-reproduction-bonus") );
        
        if (gameLevel != null) {        	
        	if (gameLevel <= GameConstants.LEVEL_EASY_BONUS) {
        		planet.setMoneyBonus(GameConstants.MONEY_BONUS_GEMS);
        	} else if (gameLevel <= GameConstants.LEVEL_NORMAL_BONUS) {
        		planet.setMoneyBonus(GameConstants.MONEY_BONUS_DIAMONDS);
        	}
        }
        
        add( planet );
        
        return planet;
    }

    /**
     * Create a new Planet that is the home world for the race passed in. All the player bonuses
     * for the home world set in the players character are used to create this planet.
     * @throws mot.common.util.ApplicationError
     */
    public static Planet createRandomPlanet(SolarSystem ss) throws ApplicationError {
    	String name = getDefaultName(ss);
    	Planet planet = new Planet(name, null, ss);
        planet.setSize( getRandomPlanetSize() );
        planet.setPopulation( 0 );
        planet.setTeraforming( getRandomPlanetQuality() );
        planet.setMoneyBonus(getRandomMoneyBonus());
        planet.setQtyFactories( 0 );
        planet.setProductionBonus( getRandomProductionBonus(planet.getTeraforming()) );
        planet.setResearchBonus( getRandomResearchBonus(planet.getTeraforming()) );
        planet.setReproductionBonus( getRandomReproductionBonus(planet.getTeraforming()) );
        
        if (planet.getSize() <= 40) {
        	addNatives(planet);
        }
        
        add( planet );
        
        return planet;
    }

    public static Planet createBonusPlanet(SolarSystem ss) throws ApplicationError {
    	String name = getDefaultName(ss);
    	Planet planet = new Planet(name, null, ss);
        planet.setSize( 100 );
        planet.setPopulation( 0 );
        planet.setTeraforming( 0.9 );
        planet.setMoneyBonus( 0 );
        planet.setQtyFactories( 0 );
        planet.setProductionBonus( 2.0 );
        planet.setResearchBonus( 0.0 );
        planet.setReproductionBonus( 0.0 );
        
        add( planet );
        
        return planet;
    }

    public static Planet createTutorialFirstColony(SolarSystem ss) throws ApplicationError {
    	String name = getDefaultName(ss);
    	Planet planet = new Planet(name, null, ss);
        planet.setSize( 100 );
        planet.setPopulation( 0 );
        planet.setTeraforming( 1.0 );
        planet.setMoneyBonus( GameConstants.MONEY_BONUS_GOLD );
        planet.setQtyFactories( 0 );
        planet.setProductionBonus( 0.0 );
        planet.setResearchBonus( 0.0 );
        planet.setReproductionBonus( 1.0 );
        
        add( planet );
        
        return planet;
    }

	public static Planet createTutorialGuardedPlanet(SolarSystem ss) throws ApplicationError {
    	String name = getDefaultName(ss);
    	Planet planet = new Planet(name, null, ss);
        planet.setSize( 110 );
        planet.setPopulation( 0 );
        planet.setTeraforming( 1.0 );
        planet.setMoneyBonus( getRandomMoneyBonus() );
        planet.setQtyFactories( 0 );
        planet.setProductionBonus( 2.0 );
        planet.setResearchBonus( 0.0 );
        planet.setReproductionBonus( 0 );
        
        add( planet );

    	String name2 = getDefaultName(ss);
    	Planet planet2 = new Planet(name2, null, ss);
        planet2.setSize( 90 );
        planet2.setPopulation( 0 );
        planet2.setTeraforming( 0.7);
        planet2.setMoneyBonus( 0 );
        planet2.setQtyFactories( 0 );
        planet2.setProductionBonus( 0 );
        planet2.setResearchBonus( 1.0 );
        planet2.setReproductionBonus( 0 );
        
        add( planet2 );

        return planet;
	}
    

	private static int getRandomPlanetSize() {
		int v = Utils.random(100);
		if (v <= 10) {
			return 20; // 10%
		} else if (v <= 20) {
			return 40; // 10%
		} else if (v <= 40) {
			return 60; // 20%
		} else if (v <= 60) {
			return 80; // 20%
		} else if (v <= 70) {
			return 90; // 10%
		} else if (v <= 80) {
			return 100; // 10%
		} else if (v <= 90) {
			return 110; // 10%
		} else if (v <= 95) {
			return 120; // 5%
		} else {
			return 130; // 5%
		}
	}

	private static double getRandomPlanetQuality() {
		int v = Utils.random(100);
        if (v >= 90) return 1.0;
        else if (v >= 80) return 0.9;
        else if (v >= 65) return 0.8;
        else if (v >= 50) return 0.7;
        else if (v >= 35) return 0.6;
        else if (v >= 20) return 0.5;
        else if (v >= 10) return 0.4;
        else return 0.2;
	}

	private static int getRandomMoneyBonus() {
		int v = Utils.random(100);
        if (v >= 95) return GameConstants.MONEY_BONUS_DIAMONDS;
        else if (v >= 80) return GameConstants.MONEY_BONUS_GOLD;
        else return GameConstants.MONEY_BONUS_NONE;
	}

	private static double getRandomProductionBonus(double terraform) {
        if (terraform < 0.8) {
            return getRandomBonus(50);
        } else {
            return getRandomBonus(10);
        }
	}

	private static double getRandomResearchBonus(double terraform) {
		return getRandomBonus(10);
	}

	private static double getRandomReproductionBonus(double terraform) {
		if (terraform < 0.7) {
			return 0.0;
		} else {			
			return getRandomBonus(20);
		}
	}

	private static double getRandomBonus(int percentChance) {
		int v = Utils.random(100);
        if (v >= percentChance) {
            return 0.0;
        }
        
		v = Utils.random(100);
		if (v <= 10) {
			return -0.5; // 10%
		} else if (v <= 30) {
			return -0.25; // 20%
		} else if (v <= 50) {
			return 0.5; // 20%
		} else if (v <= 70) {
			return 1.0; // 20%
		} else if (v <= 90) {
			return 2.0; // 20%
		} else {
			return 3.0; // 10%
		}
	}

	private static void addNatives(Planet planet) {
		planet.setReproductionBonus(0.0);
		planet.setTeraforming(1.0);
		planet.setHasNatives(true);
		planet.setQtyNatives(planet.getSize());
	}
	
    
    private static void add(Planet p) throws ApplicationError {
        MotEngine engine = MotEngine.getInstance();
        Planets planets = engine.getPlanets();
        if (planets == null) {
            planets = new Planets();
            engine.setPlanets(planets);
        }
        planets.add(p);
    }

    public static Planet createAndAddPlanetToGalaxy(String name, SolarSystem ss, Player owner, int size, int population, double terraform) throws ApplicationError {
        Planet planet = new Planet(name, owner, ss);
        planet.setSize( size );
        planet.setPopulation( population );
        planet.setTeraforming( terraform );
        planet.setMoneyBonus(0);
        planet.setQtyFactories( 0 );
        planet.setProductionBonus(0);
        planet.setResearchBonus(0);
        planet.setReproductionBonus(0);
        
        add( planet );
        
        return planet;
    }

}