/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.NamedObject;
import mot.common.references.PlanetReferenceI;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.NameValuePair;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;

/**
 * This is a reference to a planet. It uses the name of the planet to locate
 * the planet in the list of planets in the game and then caches it for future use. 
 * If it cannot find a planet for the given name, it searches all the planets for 
 * old names of this given name.
 */
public class PlanetReference extends NamedObject {

	private transient Planet planet;

    /**
     * @return The planet for this reference 
     */
	public Planet getPlanet() {
		if (planet == null) {
			planet = MotEngine.instance.getPlanets().getPlanet(getName());
		}
		return planet;
	}

	public PlanetReference() {
	}

	public PlanetReference(PlanetReferenceI planet) {
		super(planet.getName());
	}

	public PlanetReference(String planet) {
		super(planet);
	}

	public PlanetReference(Planet planet) {
		super(planet.getName());
		this.planet = planet;
	}

	public SolarSystem getSolarSystem() {
		Planet p = getPlanet();
		if (p != null) {
			return p.getSolarSystem();
		} else {
			return null;
		}
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		// does not have a bean body
	}

    protected BasicNameValueList getAttributes(boolean generateClassAttribute, boolean generateSchemaAttribute) throws ApplicationError {
		if (getPlanet() == null) {
			return null;
		}

		BasicNameValueList params = getAttributesNotNull(generateClassAttribute);
		params.add( new NameValuePair("name", planet.getName()) );

		return params;
	}

	protected String getElementName() { return "planet-reference"; }
    protected boolean beanHasBody() { return false; }

}