/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import java.util.ArrayList;
import java.util.Collection;

import mot.common.bean.Location;
import mot.common.designs.Orbital;
import mot.common.technology.Module;
import mot.common.util.ApplicationError;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.MoveableOwnedObjects;
import mot.engine.players.Player;

/**
 * This is the list of planets in the game. It acts as a container for planets and 
 * allows planets to be found by name, owner, status and much more.
 */
public class Planets extends XMLBean {

    private ArrayList list;

    public ArrayList getList() { return list; }
    public void setList(ArrayList v) { list = v; }

    public Planets() {
	}

    private transient Planet [] planets;
    public Planet [] getPlanets() {
		if (planets == null) {
			planets = (Planet []) list.toArray( new Planet[list.size()]);
		}
		return planets;
	}

    /**
     * Used by XMLDocumentHandler to load saved games
     * Longcut to add(Planet)
     */
    public void setPlanet(Planet v) throws ApplicationError {
        add(v);
    }

    /**
     * Adds a new planet to this list of planets.
     * This will flush the Array of planets cache.
     */
    public void add(Planet planet) throws ApplicationError {
		planets = null;
        if (list == null) list = new ArrayList();
		list.add(planet);
        if (planet.getOwner() != null) {
            planet.getOwner().flushCache();
        }
        // When loading a game planets are loaded before stellar objects!
        if (MotEngine.instance.getStellarObjects() != null && planet.getSolarSystem() != null) {
        	planet.getSolarSystem().flushCache();
        }
	}

    public void removePlanet(Planet planet) throws ApplicationError {
		planets = null;
        if (list == null) return;
		list.remove(planet);
        if (planet.getOwner() != null) {
            planet.getOwner().flushCache();
        }
        // When loading a game planets are loaded before stellar objects!
        if (MotEngine.instance.getStellarObjects() != null && planet.getSolarSystem() != null) {
        	planet.getSolarSystem().flushCache();
        }
	}
    
    /**
     * If this method cannot find a Solar System with this name it searches 
     * old names for a Solar System of this name.
     * @return the Planet with name or null if it does not exist.
     */
    public Planet getPlanet(String name) {
		Planet [] planets = getPlanets();
		for (int i=0; i<planets.length; i++) {
			if (planets[i].getName().equalsIgnoreCase(name)) {
				return planets[i];
			}
		}
		for (int i=0; i<planets.length; i++) {
			if (planets[i].didHaveName(name)) {
				return planets[i];
			}
		}
		return null;
	}

    /**
     * @return an array of Planets owned by player p, or, returns null if none exist.
     */
    public Planet [] getPlanetsOwnedBy(Player p) throws ApplicationError {
		Planet [] planets = getPlanets();
        ArrayList results = new ArrayList();
		for (int i=0; i<planets.length; i++) {
			if (planets[i].isOwnedBy(p)) {
				results.add(planets[i]);
			}
		}
		return (Planet [])results.toArray(new Planet[results.size()]);
    }

    /**
     * @return an array of Planets owned by player p, or, returns null if none exist.
     */
    public Planet [] getColonies() throws ApplicationError {
		Planet [] planets = getPlanets();
        ArrayList results = null;
		for (int i=0; i<planets.length; i++) {
			if (planets[i].getOwner() != null) {
                if (results == null) results = new ArrayList();
				results.add(planets[i]);
			}
		}
        if (results == null) {
            return null;
        } else {
            return (Planet [])results.toArray(new Planet[results.size()]);
        }
    }

    /**
     * This calls entTurn on each planet in the list.
     * It is on endTurn that planets add this turns effort to their projects
     * and calculate population growth etc.
     */
	public void endTurn() throws ApplicationError, ValidationError {
		Planet [] planets = getPlanets();
		for (int i=0; i<planets.length; i++) {
			planets[i].endTurn();
		}
	}

    /**
     * @return The closest planet to location loc, that satisfies the following conditions:
     *   - Has a mining station that can have module added to it.
     *   - Is owned by player
     */
	public Planet getClosestPlanetWithMiningStation(Location loc, Player player, Module module) throws ApplicationError {
		int minDist = Integer.MAX_VALUE;
        Planet best = null;
		if (getPlanets() != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].isOwnedBy(player) && planets[i].getMiningStation() != null) {
                    OrbitalReference or = planets[i].getMiningStation();
                    if (or.getOrbital().canHaveModule(module)) {
                        int dist = loc.getDistanceTo(planets[i].getSolarSystem().getLocation());
                        if (dist < minDist) {
                            minDist = dist;
                            best = planets[i];
                        }
                    }
				}
			}
		}
		return best;
	}
    
    /**
     * The planet that owns the mining station is not the planet the mining is located
     * in orbit around.
     * @return The planet that owns this orbital passed in.
     */
	public Planet getOwnerOfMiningStation(Orbital orbital) throws ApplicationError {
		if (getPlanets() != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].getMiningStation() != null) {
					Orbital mineStation = planets[i].getMiningStation().getOrbital();
                    if (mineStation == orbital) {
                        return planets[i];
                    }
				}
			}
		}
		return null;
	}
    
    /**
     * Removes the mining colony reference to the orbital passed in.
     * @param orbital The orbital to remove all mining references to.
     */
    public void removeMiningColony(Orbital orbital) {
		if (getPlanets() != null) {
			for (int i=0; i<planets.length; i++) {
                OrbitalReference or = planets[i].getMiningStation();
                if (or != null && or.getOrbital() == orbital) {
                    planets[i].setMiningStation(null);
                    return;
                }
            }
        }
    }

	/**
	 * The quantity in the list will be a maximum of qtyPlanets. If there are enough matching planets
	 * in this list there will be qtyPlanets returned. However, if not enought matching planets are found
	 * the length of the list will be less or even a null return value.
	 * @param noMoos If this is true only planets without any crafts in the solar system will be returned.
	 * @return A random collection of planets that have terraform of maxTerraform or less.
	 */
	public Planet [] getRandomPlanets(int qtyPlanets, double maxTerraform, boolean noMoos) throws ApplicationError {
		getPlanets();
		if (planets == null) return null;
        MoveableOwnedObjects moos = MotEngine.instance.getMoveableOwnedObjects();
        ArrayList candidates = new ArrayList(getPlanetsWithMaxTerraForm(maxTerraform));
        ArrayList results = new ArrayList();
        while (results.size() < qtyPlanets && candidates.size() > 0) {
			Planet p = (Planet) Utils.random(candidates);
			results.add(p);
			candidates.remove(p);
		}
        return (Planet[]) results.toArray(new Planet[results.size()]);
	}
	
	/**
	 * A planet is considered unguarded if there are no crafts at its solar system.
	 * A Planet is considered empty if the solar system is not owned. 
	 * @param ignoreList 
	 * @return
	 * @throws ApplicationError
	 */
	public Planet getRandomUnGuardedEmptyPlanet(ArrayList ignoreList) throws ApplicationError {
		getPlanets();
		if (planets == null) return null;
		ArrayList<Planet> candidates = new ArrayList();
		for (int i = 0; i < planets.length; i++) {
			if (!planets[i].getSolarSystem().hasOwner()) {
				if ( MotEngine.instance.getCrafts().getQtyAt(planets[i].getSolarSystem()) == 0 && (ignoreList == null || !ignoreList.contains(planets[i].getSolarSystem())) ) {
					candidates.add(planets[i]);
				}
			}
		}
        if (candidates.size() > 0) {
			Planet p = (Planet) Utils.random(candidates);
			return p;
		} else {
			return null;
		}
	}
	
	/**
	 * @return All the planets that have terraform of maxTerraform or less.
	 */
	public Collection<Planet> getPlanetsWithMaxTerraForm(double maxTerraform) {
		if (getPlanets() == null) return null;
		ArrayList<Planet> r = new ArrayList();
		for (int i = 0; i < planets.length; i++) {
			if (planets[i].getTeraforming() <= maxTerraform) {
				r.add(planets[i]);
			}
		}
		return r;
	}
	
	public int getQtyPlanets() {
		if (getPlanets() == null) return 0;
		return planets.length;
	}

	public double getTotalEspionageEffect(Player owner) throws ApplicationError {
		double r = 0;
		if (getPlanets() != null) {
			for (int i=0; i<planets.length; i++) {
                if (planets[i].isOwnedBy(owner)) {
                	if (planets[i].getCrisis() != null) {
                		Double v = planets[i].getCrisis().espionageEffect();
                		if (v != null) {
                			r += v;
                		}
                	}
                }
            }
        }
		return r;
	}

    /**
     * Flushes the blockade cache for every planet in the solara system.
     * This should be called when a craft is destroyed and after crafts 
     * move at the end of every turn.
     */
    public void flushBlockadeCache() {
		if (getPlanets() != null) {
			for (int i=0; i<planets.length; i++) {
                planets[i].flushBlockadeCache();
            }
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (getPlanets() != null) {
			Planet [] planets = getPlanets();
			for (int i=0; i<planets.length; i++) {
				if (planets[i] != null) {
					planets[i].toXML(writer);
				}
			}
		}
	}

	protected String getElementName() { return "planets"; }
}