/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.ProjectDesign;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.common.xml.XMLBeanInterface;
import mot.engine.common.ProjectBuilder;

/**
 * This contains the details of a planetary project. A planetary project is
 * a task assigned to a planet to build an object in the game - like a new craft.
 * This is a server side object - It contains a method getDesign() to get design 
 * details. The project but contains the build() method to actually build the 
 * project.
 * 
 * Some projects are build the moment they are complete. An example is a craft project.
 * Once the required amount of effort has been spend by the builder on the project
 * notifyBuildComplete() will return true and build() will be called to complete the 
 * project. A new craft will be created and the builder will request its next project.
 * 
 * Some projects need player coordination or intervention before they are actually built.
 * For example a module (used to produce / improve battle stations). A module can only 
 * actually be produced when a transport exists to to move it to its destination. Therefore
 * production may be delayed until a transport is ready.
 */
public interface Project extends XMLBeanInterface {

    /**
     * This method is called when the planet (builder) is ready to build the project but
     * has not yet actually called build() to build the object. If this method returns true
     * then build will be called immediately, and the builder will start the next project or
     * send an event asking for the next project. If this method returns false no action will
     * be taken on building the project.
     * This is important for projects like BuildSpaceStation that require a transport to be available
     * to complete the build. It allows for build to be delayed a turn or two!
     * @param builder The planet is ready to build the object.
     * @return true if the builder should build this project immediately on completion.
     */
    public boolean canBuildProjectNow(ProjectBuilder builder) throws ApplicationError, ValidationError;

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError, ValidationError;
    
    /**
     * NB - If freeList contains 5 fusion beams this implies that the project may use up to 
     * 5 of them without having the cost added to the cost to build this structure.
     * @param freeTechs A list of technologies to use on this project. 
     * @return The cost to build this object. See Planet production for details.
     * @throws ApplicationError 
     */
    public int getCostToBuildOnPlanet() throws ApplicationError;

    /**
     * @return The name of this object being built. For a CraftDesign it may be
     * something like FighterI.
     */
    public String getName();

    /**
     * @return The name of an xml element that is used to store this project. 
     */
//	public String getProjectElementName();
	
	/**
	 * @return The design of the object being built by this project
	 */
	public ProjectDesign getProjectDesign();
    
}