/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.BattleStationDesign;
import mot.common.designs.Orbital;
import mot.common.designs.ProjectDesign;
import mot.common.technology.Hull;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyReference;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.common.ProjectBuilder;

/**
 * This is a planetary project to build a BattleStation on a planet.
 */
public class ProjectBattleStation extends XMLBean implements Project {

	private BattleStationDesign battleStationDesign;
	
    public ProjectBattleStation() {
    }

    public ProjectBattleStation(BattleStationDesign design) {
        this.battleStationDesign = design;
    }

//    public ProjectBattleStation(Hull battleStation) {
//        this.battleStation = battleStation;
//    }

    public boolean canBuildProjectNow(ProjectBuilder builder) { return true; }
    
    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet/thing which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("BattleStation.build() Build BattleStation "+getName()+" on planet "+builder.getName());
        }
        Hull battleStation = battleStationDesign.getBattleStation();
        Orbital orbital = new Orbital(battleStation, null, null);
        orbital.addCoreModule();
        orbital.addBattleStationModules(builder.getOwner());        
        builder.setOrbital(orbital);
    }

	public int getCostToBuildOnPlanet() throws ApplicationError {
		return battleStationDesign.getCostToBuildOnPlanet();
	}

	/**
     * This is the amount of money the player would get if this was sold.
	 * @return The value of this project (once complete).
	 */
	public int getValue() throws ApplicationError {
		int v = battleStationDesign.getBattleStation().getCostToBuild();
        return (int) (v / Technology.COST_TO_SELL_FACTOR);

	}

	public ProjectDesign getProjectDesign() {
		return battleStationDesign;
	}
    
	public String getName() {
		return battleStationDesign.getName();
	}

	protected String getElementName() { return "battle-station-project"; }
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		battleStationDesign.toXML(writer);
	}

	public Hull getBattleStation() throws ApplicationError {
		return battleStationDesign.getBattleStation();
	}

	public BattleStationDesign getBattleStationDesign() {
		return battleStationDesign;
	}

	public void setBattleStationDesign(BattleStationDesign design) {
		this.battleStationDesign = design;
	}
	
	public void setTechnologyReference(TechnologyReference v) throws ApplicationError {
		if (battleStationDesign == null) {
			battleStationDesign = new BattleStationDesign((Hull)v.getTechnology());
		} else {
			battleStationDesign.setName(v.getName());
		}
	}

}