/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.GalaxyDefinitions;
import mot.common.designs.ConstructPlanetDesign;
import mot.common.designs.ProjectDesign;
import mot.common.technology.PlanetConstruction;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.ProjectBuilder;

/**
 * This is a planetary project to build a new planet in a solar system.
 */
public class ProjectConstructPlanet extends XMLBean implements Project {

	private ConstructPlanetDesign constructPlanetDesign;
	
    public ProjectConstructPlanet() {
    }

    public ProjectConstructPlanet(ConstructPlanetDesign design) {
    	this.constructPlanetDesign = design;
    }

    public ProjectConstructPlanet(PlanetConstruction planetConstructionTech, String solarSystem) {
    	this.constructPlanetDesign = new ConstructPlanetDesign(planetConstructionTech, solarSystem);
    }

    public boolean canBuildProjectNow(ProjectBuilder builder) { return true; }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet/thing which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
    	SolarSystem ss = MotEngine.instance.getStellarObjects().getSolarSystem(constructPlanetDesign.getSolarSystem());
    	if (ss == null) {
    		Logger.logError("", new ApplicationError("ProjectConstructPlanet.build() - no solar system with name "+constructPlanetDesign.getSolarSystem()));
    		return;
    	}
    	if (ss.getQtyPlanets() > GalaxyDefinitions.MAX_PLANETS_PER_SOLAR_SYSTEM-1) {
    		Logger.logError("", new ApplicationError("ProjectConstructPlanet.build() - Too many planets in "+constructPlanetDesign.getSolarSystem()));
    		return;
    	}
    	Planet newPlanet = PlanetFactory.createNewPlanet(null, constructPlanetDesign.getPlanetConstructionTech(), ss);
    }
    
    public int getCostToBuildOnPlanet() throws ApplicationError {
        return constructPlanetDesign.getCostToBuildOnPlanet();
    }

    public String getName() {
        return constructPlanetDesign.getName();
    }

    protected String getElementName() {
        return "construct-planet-project";
    }

    protected void encode(XMLWriter writer) throws ApplicationError {
        constructPlanetDesign.toXML(writer);
    }
    
	public String getProjectElementName() {
        return getElementName();
    }

	public ProjectDesign getProjectDesign() {
		return constructPlanetDesign;
	}

	public ConstructPlanetDesign getConstructPlanetDesign() {
		return constructPlanetDesign;
	}

	public void setConstructPlanetDesign(ConstructPlanetDesign design) {
		this.constructPlanetDesign = design;
	}
}