/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.SalvageList;
import mot.common.bean.stats.GameScoreData;
import mot.common.designs.CraftDesign;
import mot.common.designs.ProjectDesign;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.TechnologyBundle;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.common.Orbit;
import mot.engine.common.ProjectBuilder;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * This is a project to build a craft on a planet.
 */
public class ProjectCraft extends XMLBean implements Project {

    /**
     * The design of the craft that this project is building.
     */
    private CraftDesign craftDesign;
    
    /**
     * The cost to build this project.
     */
    private int costToBuild;

    public ProjectCraft() {
    }

    public ProjectCraft(CraftDesign design, ProjectBuilder builder, boolean useSalvage) throws ApplicationError, ValidationError {
        Player owner = builder.getOwner();
		SalvageList salvage = owner.getSalvage();

		this.craftDesign = design;
		TechnologyBundle[] salvageUsed = useSalvage ? design.getUsableSalvage(builder, salvage) : null;
        setCostToBuild(design.getCostToBuildUsingExistingTechs(new GeneralTechnologyList(salvageUsed)));
        
        if (salvage != null && salvageUsed != null) {
        	if (Logger.isDebug() && salvageUsed.length > 0) {
        		Logger.logDebug("ProjectCraft.init() Use salvage "+Comp.toString(salvageUsed)+" to construct "+design.getName()+" on "+builder.getName());
        	}
        	salvage.remove(salvageUsed);
        }
    }

    public boolean canBuildProjectNow(ProjectBuilder builder) { return true; }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("CraftDesign.build() Build Craft "+getName()+" on planet "+builder.getName());
        }
        Player owner = builder.getOwner();
        Player origin = owner;
        if (builder instanceof Planet) {
            Planet planet = (Planet) builder;
            origin = planet.getOrigin();
        }
        
        Craft craft = getCraftInstance(owner, origin, builder.getOrbit());
        
		if (owner instanceof UserPlayer) {			
			Orbit fleetWayPoint = ((UserPlayer)owner).getFleetWayPoint();
			if (fleetWayPoint != null) {
				craft.moveTo(fleetWayPoint);
			}
		}
		
		if (owner instanceof UserPlayer) {
			UserPlayer up = (UserPlayer) owner;
			GameScoreData gsd = up.getGameScoreDataNotNull();
			gsd.addCraftBuilt(craftDesign.getHullTech(), 1);
		}
    }

	public Craft getCraftInstance(Player owner, Player origin, Orbit orbit) throws ApplicationError {
        return CraftFactory.getCraftInstance(craftDesign, owner, origin, orbit, true, true);
    }

    public int getCostToBuildOnPlanet() throws ApplicationError {
        return costToBuild;
    }

    public String getName() {
        return craftDesign.getName();
    }

    public void setCraftDesign(CraftDesign design) {
        this.craftDesign = design;
    }

    public CraftDesign getCraftDesign() {
        return craftDesign;
    }

    protected String getElementName() {
        return "craft-project";
    }

    protected void encode(XMLWriter writer) throws ApplicationError {
        craftDesign.toXML(writer);
        writer.addElement("cost-to-build", costToBuild);
    }
    
	public String getProjectElementName() {
        return getElementName();
    }

	public ProjectDesign getProjectDesign() {
		return craftDesign;
	}

	public void setCostToBuild(int costToBuild) {
		this.costToBuild = costToBuild;
	}

	public int getCostToBuild() {
		return costToBuild;
	}

	public String toString() {
		return "ProjectCraft "+getName();
	}

	
}