/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.SalvageList;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.ProjectDesign;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.CraftVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.common.ProjectBuilder;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.players.Player;

/**
 * This is a project to build a craft on a planet.
 */
public class ProjectCraftUpgrade extends XMLBean implements Project {

    /**
     * The design of the craft that this project is building.
     */
    private CraftUpgradeDesign design;
    
    /**
     * The cost to build this project.
     */
    private int costToBuild;

    public ProjectCraftUpgrade() {
    }

    public ProjectCraftUpgrade(CraftUpgradeDesign design, ProjectBuilder builder) throws ApplicationError {
		this.design = design;
        _update(builder);
    }

	public void addCrafts(CraftUpgradeDesign design, ProjectBuilder builder) throws ApplicationError {
		this.design.addCrafts(design);
        _update(builder);
	}

	private void _update(ProjectBuilder builder) throws ApplicationError {
        Player owner = builder.getOwner();
		SalvageList salvage = owner.getSalvage();

		setCostToBuild(design.getCostToBuild(salvage));
        
        TechnologyList salvageUsed = design.getFreeTechsUsedInBuild(salvage);
        if (salvage != null && salvageUsed != null) {
        	if (Logger.isDebug() && salvageUsed.getQuantity() > 0) {
        		Logger.logDebug("ProjectCraft.init() Use salvage "+salvageUsed+" to construct "+design.getName()+" on "+builder.getName());
        	}
        	salvage.remove(salvageUsed);
        }
        
        CraftVO[] ca = design.getCrafts();
        if (ca != null) {
        	for (int i = 0; i < ca.length; i++) {
        		Craft craft = MotEngine.instance.getCrafts().getCraft(ca[i].getName());
        		if (craft != null) {
        			craft.destroy();
        		}
			}
        }
        
	}

    public boolean canBuildProjectNow(ProjectBuilder builder) { return true; }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("CraftUpgradeDesign.build() Build Craft "+getName()+" on planet "+builder.getName());
        }
        Player owner = builder.getOwner();
        
        CraftVO[] crafts = design.getCrafts();
        for (int i = 0; i < crafts.length; i++) {
			Player origin = MotEngine.instance.getPlayersNotNull().getPlayerDeadOrAlive(crafts[i].getOrigin());
			if (origin == null) origin = owner;
        	Craft craft = getCraftInstance(owner, origin, builder.getOrbit());
//			if (owner instanceof UserPlayer) {			
//				Orbit fleetWayPoint = ((UserPlayer)owner).getFleetWayPoint();
//				if (fleetWayPoint != null) {
//					craft.moveTo(fleetWayPoint);
//				}
//			}		
		}
        
    }

	public Craft getCraftInstance(Player owner, Player origin, Orbit orbit) throws ApplicationError {
        return CraftFactory.getCraftInstance(design.getDesign(), owner, origin, orbit, true, true);
    }

    public int getCostToBuildOnPlanet() throws ApplicationError {
        return costToBuild;
    }

    public String getName() {
        return design.getName();
    }

    public void setCraftUpgradeDesign(CraftUpgradeDesign design) {
        this.design = design;
    }

    public void setCraftDesignUpgrade(CraftUpgradeDesign design) {
        this.design = design;
    }
    
    public CraftUpgradeDesign getCraftDesign() {
        return design;
    }

    protected String getElementName() {
        return "craft-upgrade-project";
    }

    protected void encode(XMLWriter writer) throws ApplicationError {
        design.toXML(writer);
        writer.addElement("cost-to-build", costToBuild);
    }
    
	public String getProjectElementName() {
        return getElementName();
    }

	public ProjectDesign getProjectDesign() {
		return design;
	}

	public void setCostToBuild(int costToBuild) {
		this.costToBuild = costToBuild;
	}

	public int getCostToBuild() {
		return costToBuild;
	}

	
}