/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.GalaxyDefinitions;
import mot.common.designs.ConstructPlanetDesign;
import mot.common.designs.ProjectDesign;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.ProjectBuilder;

/**
 * This is a planetary project to build a weapon on a planet.
 */
public class ProjectCreatePlanet extends XMLBean implements Project {

    private static final int PLANET_SIZE = 130;
    private static final int PLANET_POP = 10;
    private static final double PLANET_TERAFORM = 0.6;

    
    private ConstructPlanetDesign constructPlanetDesign;
    
	public ProjectCreatePlanet() {
    }

//    public ProjectCreatePlanet(ConstructPlanetDesign design, PlanetConstruction tech) {
//        super(tech, solarSystem.getName());
//    }

    public ProjectCreatePlanet(ConstructPlanetDesign design) {
        this.constructPlanetDesign = design;
	}

	public boolean canBuildProjectNow(ProjectBuilder builder) { return true; }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet/thing which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
		SolarSystem solarSystem = MotEngine.instance.getStellarObjects().getSolarSystem(constructPlanetDesign.getSolarSystem());
		if (solarSystem.getQtyPlanets() == GalaxyDefinitions.MAX_PLANETS_PER_SOLAR_SYSTEM) {
	        if (Logger.isInfo()) {
	            Logger.logInfo("CreatePlanet.build() Build "+getName()+" on planet "+builder.getName()+" abandoned because we have reached the maximum quantity of solar systems");
	        }
			return;
		}
        if (Logger.isDebug()) {
            Logger.logDebug("CreatePlanet.build() Build "+getName()+" on planet "+builder.getName());
        }
        String name = solarSystem.getName() +"-"+ (solarSystem.getQtyPlanets()+1);
		Planet planet = PlanetFactory.createAndAddPlanetToGalaxy(name, solarSystem, builder.getOwner(), PLANET_SIZE, PLANET_POP, PLANET_TERAFORM);
    }
    
	public int getCostToBuildOnPlanet() throws ApplicationError {
		return constructPlanetDesign.getCostToBuildOnPlanet();
	}
    
	public ProjectDesign getProjectDesign() {
		return constructPlanetDesign;
	}
	
	public String getName() {
		return constructPlanetDesign.getName();
	}
        
	protected String getElementName() { return "create-planet-project"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
		constructPlanetDesign.toXML(writer);
	}

	public ConstructPlanetDesign getConstructPlanetDesign() {
		return constructPlanetDesign;
	}

	public void setConstructPlanetDesign(ConstructPlanetDesign design) {
		this.constructPlanetDesign = design;
	}
}