/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.AsteroidDesign;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.ConstructPlanetDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ProjectDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.TransportDesign;
import mot.common.designs.WeaponDesign;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.common.ProjectBuilder;

/**
 * This factory creates Project from project designs.
 * A project design is the specification of a project and are a client side object sent
 * from the UI to the engine and vise versa.
 * A project is a server side object that can actually be assigned to a project builder
 * (a planet) to be built.
 */
public class ProjectFactory  {

    public static Project getProject(ProjectDesign pd, ProjectBuilder builder, boolean useSalvage) throws ApplicationError, ValidationError {
        if (pd == null) {
            return null;
        } else if (pd instanceof CraftDesign) {
        	CraftDesign cd = (CraftDesign) pd;
            return new ProjectCraft(cd, builder, useSalvage);
        } else if (pd instanceof CraftUpgradeDesign) {
        	CraftUpgradeDesign cud = (CraftUpgradeDesign) pd;
        	Project existing = builder.getProject();
        	if (existing != null && existing instanceof ProjectCraftUpgrade) {
        		ProjectCraftUpgrade existingCUP = (ProjectCraftUpgrade) existing;
        		existingCUP.addCrafts(cud, builder);
        		return existingCUP;
        	} else {
        		return new ProjectCraftUpgrade(cud, builder);
        	}
        } else if (pd instanceof WeaponDesign) {
            return new ProjectWeapon((WeaponDesign)pd);
        } else if (pd instanceof BattleStationDesign) {
            return new ProjectBattleStation((BattleStationDesign)pd);
        } else if (pd instanceof ShieldDesign) {
            return new ProjectShield((ShieldDesign)pd);
        } else if (pd instanceof ScannerDesign) {
            return new ProjectScanner((ScannerDesign)pd);
        } else if (pd instanceof TransportDesign) {
            return new ProjectTransport();
        } else if (pd instanceof OutpostDesign) {
            return new ProjectOutpost((OutpostDesign)pd, null);
        } else if (pd instanceof SpaceStationDesign) {
            return new ProjectSpaceStation((SpaceStationDesign)pd);
        } else if (pd instanceof AsteroidDesign) {
            return new ProjectAsteroids((AsteroidDesign)pd);
        } else if (pd instanceof ConstructPlanetDesign) {
            return new ProjectConstructPlanet((ConstructPlanetDesign)pd);
        } else if (pd instanceof ConstructPlanetDesign) {
        	if (builder instanceof Planet) {
	            return new ProjectCreatePlanet((ConstructPlanetDesign)pd);
        	} else {
                throw new ApplicationError("Project type "+pd.getClass().getName()+" builder must be a planet");
        	}
        } else {
            throw new ApplicationError("Unknown project type "+pd.getClass().getName());
        }
    }

}