/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.Orbital;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ProjectDesign;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Technologies;
import mot.common.technology.TechnologyReference;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.ProjectBuilder;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectReference;
import mot.engine.event.MessageEF;
import mot.engine.players.Player;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskDeployStation;

/**
 * This is a planetary project to build a Outpost on a planet.
 */
public class ProjectOutpost extends XMLBean implements Project {

	private OutpostDesign outpostDesign;
	private StellarObjectReference target;
	
    public ProjectOutpost() {
    }

    public ProjectOutpost(OutpostDesign design, StellarObject target) {
        this.outpostDesign = design;
        this.setTarget(target);
    }

//    public ProjectOutpost(Hull outpost, StellarObject target) {
//        this.outpost = outpost;
//        this.setTarget(target);
//    }

    public boolean canBuildProjectNow(ProjectBuilder builder) throws ApplicationError {
        Player owner = builder.getOwner();
        if (owner.getTransportManagerNotNull().getFreeTransport() != null) {
        	return true;
        } else if (outpostDesign.isIncludeTransport()) {
        	return true;        	
        } else {
        	outpostDesign.setIncludeTransport(true);
            MessageEF event = new MessageEF("Project "+getName()+L.HAS_BEEN_UPGRADED_TO_INCLUDE_TRANSPORT_ON+builder.getName()+L.BECAUSE_THERE_ARE_NO_FREE_TRANSPORTS);
            MotEngine.instance.broadcast(event, builder.getOwner());
            return false;
        }
    }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet/thing which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
        build(builder, true);
    }

    public void build(ProjectBuilder builder, boolean dispatchTransport) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("OutpostProject.build() Build Outpost "+getName()+" on planet "+builder.getName()+" deploy to "+getTarget());
        }
        Hull outpost = outpostDesign.getOutpost();
		String scannerName = outpost.getMeta();
        Scanner scanner = Technologies.getScanner(scannerName);
        Orbital orbital = new Orbital(outpost, scanner, null);
        Player owner = builder.getOwner();
        Transport transport;
        if (outpostDesign.isIncludeTransport()) {
	        transport = new Transport(owner);
	        owner.getTransportManagerNotNull().setTransport(transport);
        } else {
            transport = owner.getTransportManagerNotNull().getFreeTransport();
        }
        if (transport == null) {
            if (owner.isComputerPlayer()) {
                transport = new Transport(owner);
                owner.getTransportManagerNotNull().setTransport(transport);
            } else {
                Logger.logWarning("ProjectOutpost.build() No Transport to build project - SHOULD NOT GET HERE");
                return;
            }
        }
        new TransportTaskDeployStation(transport, (Planet) builder, (getTarget() == null) ? null : getTarget().getOrbit(), orbital, dispatchTransport);
    }

	public int getCostToBuildOnPlanet() throws ApplicationError {
		return outpostDesign.getCostToBuildOnPlanet();
	}

	public ProjectDesign getProjectDesign() {
		return outpostDesign;
	}
	
	public String getName() {
		return outpostDesign.getName();
	}
        
	protected String getElementName() { return "outpost-project"; }
	
    protected void encode(XMLWriter writer) throws ApplicationError {
		outpostDesign.toXML(writer);
    	if (target != null) target.toXML(writer, "target");
    }

	public void setTarget(StellarObject v) {
		if (v != null) {
			target = v.getObjectReference();
		} else {
			target = null;
		}
	}
	
	public void setTarget(StellarObjectReference v) {
		target = v;
	}

	public StellarObject getTarget() {
		if (target == null) {
			return null;
		} else {
			return target.getStellarObject();
		}
	}

	public Hull getOutpost() throws ApplicationError {
		return outpostDesign.getOutpost();
	}

	public boolean isIncludeTransport() {
		return outpostDesign.isIncludeTransport();
	}
	
	public void setIncludeTransport(boolean includeTransport) {
		outpostDesign.setIncludeTransport( includeTransport );
	}

	public OutpostDesign getOutpostDesign() {
		return outpostDesign;
	}

	public void setOutpostDesign(OutpostDesign design) {
		this.outpostDesign = design;
	}

	public void setTechnologyReference(TechnologyReference v) throws ApplicationError {
		if (outpostDesign == null) {
			outpostDesign = new OutpostDesign((Hull)v.getTechnology());
		} else {
			outpostDesign.setName(v.getName());
		}
	}

}