/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.ProjectDesign;
import mot.common.designs.ScannerDesign;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyReference;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.common.ProjectBuilder;

/**
 * This is a planetary project to build a scanner on a planet.
 */
public class ProjectScanner extends XMLBean implements Project {

	private ScannerDesign scannerDesign;
	
    public ProjectScanner() {
    }

    public ProjectScanner(ScannerDesign design) {
        this.scannerDesign = design;
    }

//    public ProjectScanner(Scanner scanner) {
//        this.scanner = scanner;
//    }

    public boolean canBuildProjectNow(ProjectBuilder builder) { return true; }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet/thing which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("Scanner.build() Build Scanner "+getName()+" on planet "+builder.getName());
        }
        Scanners scanners = builder.getScanners();
        if (scanners == null) {
            scanners = new Scanners();
            builder.setScanners(scanners);
        }
        scanners.add(scannerDesign.getScanner());
    }

	public int getCostToBuildOnPlanet() throws ApplicationError {
		return scannerDesign.getCostToBuildOnPlanet();
	}

	/**
     * This is the amount of money the player would get if this was sold.
	 * @return The value of this project (once complete).
	 */
	public int getValue() throws ApplicationError {
		int v = scannerDesign.getScanner().getCostToBuild()*ScannerDesign.PLANET_COST_FACTOR;
        return (int) (v / Technology.COST_TO_SELL_FACTOR);

	}

	public ProjectDesign getProjectDesign() {
		return scannerDesign;
	}
	
	public String getName() {
		return scannerDesign.getName();
	}
    
	protected String getElementName() { return "scanner-project"; }
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		scannerDesign.toXML(writer);
	}

	public Scanner getScanner() throws ApplicationError {
		return scannerDesign.getScanner();
	}

	public ScannerDesign getScannerDesign() {
		return scannerDesign;
	}

	public void setScannerDesign(ScannerDesign design) {
		this.scannerDesign = design;
	}

	public void setTechnologyReference(TechnologyReference v) throws ApplicationError {
		if (scannerDesign == null) {
			scannerDesign = new ScannerDesign((Scanner)v.getTechnology());
		} else {
			scannerDesign.setName(v.getName());
		}
	}

}