/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.ProjectDesign;
import mot.common.designs.ShieldDesign;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyReference;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.common.ProjectBuilder;

/**
 * This is a planetary project to build a shield on a planet.
 */
public class ProjectShield extends XMLBean implements Project {

	private ShieldDesign shieldDesign; 
	
    public ProjectShield() {
    }

    public ProjectShield(ShieldDesign design) {
        this.shieldDesign = design;
    }

//    public ProjectShield(Shield shield) {
//        this.shield = shield;
//    }

    public boolean canBuildProjectNow(ProjectBuilder builder) { return true; }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet/thing which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("Shield.build() Build Shield "+getName()+" on planet "+builder.getName());
        }
        Shields shields = builder.getShields();
        if (shields == null) {
            shields = new Shields();
            builder.setShields(shields);
        }
        shields.add(shieldDesign.getShield());
    }

	public int getCostToBuildOnPlanet() throws ApplicationError {
		return shieldDesign.getCostToBuildOnPlanet();
	}

	/**
     * This is the amount of money the player would get if this was sold.
	 * @return The value of this project (once complete).
	 */
	public int getValue() throws ApplicationError {
		int v = shieldDesign.getShield().getCostToBuild()*ShieldDesign.PLANET_COST_FACTOR;
        return (int) (v / Technology.COST_TO_SELL_FACTOR);

	}

	public ProjectDesign getProjectDesign() {
		return shieldDesign;
	}
	
	public String getName() {
		return shieldDesign.getName();
	}
        
	protected String getElementName() { return "shield-project"; }
	
	protected void encode(XMLWriter writer) throws ApplicationError {
		shieldDesign.toXML(writer);
	}

	public Shield getShield() throws ApplicationError {
		return shieldDesign.getShield();
	}

	public ShieldDesign getShieldDesign() {
		return shieldDesign;
	}

	public void setShieldDesign(ShieldDesign design) {
		this.shieldDesign = design;
	}

	public void setTechnologyReference(TechnologyReference v) throws ApplicationError {
		if (shieldDesign == null) {
			shieldDesign = new ShieldDesign((Shield)v.getTechnology());
		} else {
			shieldDesign.setName(v.getName());
		}
	}

}