/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.Orbital;
import mot.common.designs.ProjectDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.lang.L;
import mot.common.technology.Hull;
import mot.common.technology.TechnologyReference;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.common.ProjectBuilder;
import mot.engine.event.MessageEF;
import mot.engine.players.Player;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskDeployStation;

/**
 * This is a planetary project to build a SpaceStation on a planet.
 */
public class ProjectSpaceStation extends XMLBean implements Project {
	
	private SpaceStationDesign spaceStationDesign;

	/**
	 * The planet we wish to deploy this space station onto once it is built.
	 */
	private Orbit target;

    public ProjectSpaceStation() {
    }

    public ProjectSpaceStation(SpaceStationDesign design) {
        this.spaceStationDesign = design;
    }

    public ProjectSpaceStation(SpaceStationDesign design, Orbit target) {
        this.spaceStationDesign = design;
        setTarget(target);
    }

    public boolean canBuildProjectNow(ProjectBuilder builder) throws ApplicationError { 
        Player owner = builder.getOwner();
        if (owner.getTransportManagerNotNull().getQtyFreeTransport() > 0) {
        	return true;
        } else if (spaceStationDesign.isIncludeTransport()) {
        	return true;        	
        } else {
        	spaceStationDesign.setIncludeTransport(true);
            MessageEF event = new MessageEF("Project "+getName()+L.HAS_BEEN_UPGRADED_TO_INCLUDE_TRANSPORT_ON+builder.getName()+L.BECAUSE_THERE_ARE_NO_FREE_TRANSPORTS);
            MotEngine.instance.broadcast(event, builder.getOwner());
            return false;
        }
    }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet/thing which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
        build(builder, true);
    }

    public void build(ProjectBuilder builder, boolean dispatchTransport) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("SpaceStation.build() Build SpaceStation "+getName()+" on planet "+builder.getName());
        }
        Orbital orbital = new Orbital(spaceStationDesign.getSpaceStation(), null, null);
        Player owner = builder.getOwner();
        Transport transport;
        if (spaceStationDesign.isIncludeTransport()) {
	        transport = new Transport(owner);
	        owner.getTransportManagerNotNull().setTransport(transport);
        } else {
            transport = owner.getTransportManagerNotNull().getFreeTransport();
        }
        if (transport == null) {
            if (owner.isComputerPlayer()) {
                transport = new Transport(owner);
                owner.getTransportManagerNotNull().setTransport(transport);
            } else {
                sendNoTransportMessage(builder);
            }
        }
        new TransportTaskDeployStation(transport, (Planet) builder, target, orbital, dispatchTransport);
    }
    
    private void sendNoTransportMessage(ProjectBuilder builder) throws ApplicationError {
        MessageEF event = new MessageEF(builder.getName()+L.WAS_UNABLE_TO_COMPLETE_PROJECT+getName()+L.BECAUSE_THERE_ARE_NO_FREE_TRANSPORTS);
        MotEngine.instance.broadcast(event, builder.getOwner());
    }

	public int getCostToBuildOnPlanet() throws ApplicationError {
		return spaceStationDesign.getCostToBuildOnPlanet();
	}

	public ProjectDesign getProjectDesign() {
		return spaceStationDesign;
	}
	
	public String getName() {
		return spaceStationDesign.getName();
	}
        
    protected void encode(XMLWriter writer) throws ApplicationError {
		spaceStationDesign.toXML(writer);
    	if (target != null) target.toXML(writer, "target");
    }
    
    protected String getElementName() { return "space-station-project"; }

	public void setTarget(Orbit target) {
		this.target = target;
	}
	
	public void setTarget(PlanetReference target) {
		this.target = new Orbit(null, target);
	}
	
	public Orbit getTarget() {
		return target;
	}

	public Hull getSpaceStation() throws ApplicationError {
		return spaceStationDesign.getSpaceStation();
	}

	public boolean isIncludeTransport() {
		return spaceStationDesign.isIncludeTransport();
	}

	public SpaceStationDesign getSpaceStationDesign() {
		return spaceStationDesign;
	}

	public void setSpaceStationDesign(SpaceStationDesign design) {
		this.spaceStationDesign = design;
	}
	
	public void setTechnologyReference(TechnologyReference hull) throws ApplicationError {
		if (spaceStationDesign == null) {
			spaceStationDesign = new SpaceStationDesign((Hull)hull.getTechnology());
		} else {
			spaceStationDesign.setName(hull.getName());
		}
	}

	public void setIncludeTransport(boolean v) {
		spaceStationDesign.setIncludeTransport(v);
	}
}