/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.designs.ProjectDesign;
import mot.common.designs.WeaponDesign;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyReference;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.common.ProjectBuilder;

/**
 * This is a planetary project to build a weapon on a planet.
 */
public class ProjectWeapon extends XMLBean implements Project {

	private WeaponDesign weaponDesign;
	
    public ProjectWeapon() {
    }

    public ProjectWeapon(WeaponDesign design) {
        this.weaponDesign = design;
    }

//    public ProjectWeapon(Weapon weapon) {
//        this.weapon = weapon;
//    }

    public boolean canBuildProjectNow(ProjectBuilder builder) { return true; }

    /**
     * This method is called when the project has been completed to build the object.
     * @param builder The planet/thing which has built the object.
     */
    public void build(ProjectBuilder builder) throws ApplicationError {
        if (Logger.isInfo()) {
            Logger.logInfo("Weapon.build() Build Weapon "+getName()+" on planet "+builder.getName());
        }
        Weapons weapons = builder.getWeapons();
        if (weapons == null) {
            weapons = new Weapons();
            builder.setWeapons(weapons);
        }
        weapons.add(weaponDesign.getWeapon());
    }
    
	public int getCostToBuildOnPlanet() throws ApplicationError {
		return weaponDesign.getCostToBuildOnPlanet();
	}
    
	/**
     * This is the amount of money the player would get if this was sold.
	 * @return The value of this project (once complete).
	 */
	public int getValue() throws ApplicationError {
		int v = weaponDesign.getWeapon().getCostToBuild()*WeaponDesign.PLANET_COST_FACTOR;
        return (int) (v / Technology.COST_TO_SELL_FACTOR);

	}

	public ProjectDesign getProjectDesign() {
		return weaponDesign;
	}
	
	public String getName() {
		return weaponDesign.getName();
	}
        
	protected String getElementName() { return "weapon-project"; }
	
    protected void encode(XMLWriter writer) throws ApplicationError {
		weaponDesign.toXML(writer);
    }

	public Weapon getWeapon() throws ApplicationError {
		return weaponDesign.getWeapon();
	}

	public WeaponDesign getWeaponDesign() {
		return weaponDesign;
	}

	public void setWeaponDesign(WeaponDesign design) {
		this.weaponDesign = design;
	}
	
	public void setTechnologyReference(TechnologyReference v) throws ApplicationError {
		if (weaponDesign == null) {
			weaponDesign = new WeaponDesign((Weapon)v.getTechnology());
		} else {
			weaponDesign.setName(v.getName());
		}
	}
}