/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import java.util.ArrayList;

import mot.common.bean.GameConstants;
import mot.common.bean.StellarObjectGraphics;
import mot.common.factory.GraphicsFactory;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Utils;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.common.StellarObjectReference;
import mot.engine.crafts.Craft;
import mot.engine.players.Player;
import mot.engine.relations.Relations;

/**
 * This represents a solar system on the galaxy map.
 * A solar system is a star containing a number of planets in orbit around it.
 * I a player has one or more colonies in the solar system it is said that 
 * the player is an owner of the solar system. 
 * Players are not allowed to colonise a solar system if it already has a colony from another 
 * player, but, a player can invade a planet making the solar system shared in ownership. 
 */
public class SolarSystem extends StellarObject {


	/**
	 * List of planets contained in the solar system. Dynamically generated from Planets
	 * belonging to this solar system.
	 */
	private transient Planet [] planets;

    /**
     * This is the type of star at the centre of this solar system.
     * This is used for display purposes.
     */
    private String starType;

	public SolarSystem() {
	}

	public SolarSystem(String name, int x, int y) {
		super(name, x, y);
	}
    
    public String getStarType() { return starType; }
    public void setStarType(String v) { starType = v; starGraphics = null; }
    
    /**
     * Gets the list of planets in this solar system. 
     */
	public Planet [] getPlanets() {
		if (planets == null) {
			Planets planetsList = MotEngine.instance.getPlanets();
			if (planetsList == null) {
				return null;
			}
			ArrayList list = new ArrayList();
			Planet [] all = planetsList.getPlanets();
			for (int i=0; i<all.length; i++) {
                SolarSystem ss = all[i].getSolarSystem();
				if (ss == this) {
					list.add(all[i]);
				}
			}
			planets = (Planet []) list.toArray(new Planet[list.size()]);
		}
        return planets;
	}

    /**
     * Gets the list of planets in this solar system. This method returns a new ArrayList
     * every time it is called.
     * @return An ArrayList containing the planets for this player.
     */
	public ArrayList getPlanetsAsList() {
        ArrayList list = new ArrayList();
        Planet [] all = MotEngine.instance.getPlanets().getPlanets();
        for (int i=0; i<all.length; i++) {
            SolarSystem ss = all[i].getSolarSystem();
            if (ss == this) {
                list.add(all[i]);
            }
        }
        return list;
	}

    /**
     * Gets the list of planets in this solar system that are colonisable by the craft
     * passed in.
     * @return An array of planets.
     */
	public Planet [] getColonisablePlanetsBy(Craft craft) throws ApplicationError {
        ArrayList list = null;
        Planet [] all = MotEngine.instance.getPlanets().getPlanets();
        for (int i=0; i<all.length; i++) {
            SolarSystem ss = all[i].getSolarSystem();
            if (ss == this) {
                if (all[i].getOwner() == null && craft.canColonise(all[i])) {
                    if (list == null) {
                        list = new ArrayList();
                    }
                    list.add(all[i]);
                }
            }
        }
        if (list == null) {
            return null;
        } else {
            return (Planet []) list .toArray(new Planet[list.size()]);
        }
	}

    /**
     * Gets the list of planets in this solar system are owned by enemies of this player.
     * @return An array of planets.
     */
	public Planet [] getEnemyPlanetsBy(Player owner) throws ApplicationError {
        ArrayList list = null;
        Planet [] all = MotEngine.instance.getPlanets().getPlanets();
        Relations rels = MotEngine.instance.getRelations();
        for (int i=0; i<all.length; i++) {
            SolarSystem ss = all[i].getSolarSystem();
            if (ss == this) {
                Player p = all[i].getOwner();
                if ( p != null && rels.isWar(owner, p) ) {
                    if (list == null) {
                        list = new ArrayList();
                    }
                    list.add(all[i]);
                }
            }
        }
        if (list == null) {
            return null;
        } else {
            return (Planet []) list .toArray(new Planet[list.size()]);
        }
	}

    /**
     * Gets the list of planets in this solar system that are colonies (contain an owner)
     * For details of what a colony is see Planet.isColony().
     * @return An array of planets.
     */
	public Planet [] getColonies(boolean includeMiningColonies) throws ApplicationError {
        ArrayList list = null;
        Planet [] all = getPlanets();
        for (int i=0; i<all.length; i++) {
            if (all[i].hasOwner()) {
            	if (all[i].getPopulation() > 0 || (includeMiningColonies && all[i].isMiningColony())) {            		
            		if (list == null) {
            			list = new ArrayList();
            		}
            		list.add(all[i]);
            	}
            }
        }
        if (list == null) {
            return null;
        } else {
            return (Planet []) list .toArray(new Planet[list.size()]);
        }
	}


	public int getQtyColonies(boolean includeMiningColonies) throws ApplicationError {
        Planet [] all = getPlanets();
        int qty = 0;
        for (int i=0; i<all.length; i++) {
            if (all[i].hasOwner()) {
                if (all[i].getPopulation() > 0.0) {
                	qty++;
                } else if (includeMiningColonies && all[i].isMiningColony()) {
                	qty++;
                }
            }
        }
		return qty;
	}

	public int getQtyColonies(Player owner, boolean includeMiningColonies) throws ApplicationError {
        Planet [] all = getPlanets();
        int qty = 0;
        for (int i=0; i<all.length; i++) {
            if (all[i].getOwner() == owner) {
                if (all[i].getPopulation() > 0.0) {
                	qty++;
                } else if (includeMiningColonies && all[i].isMiningColony()) {
                	qty++;
                }
            }
        }
		return qty;
	}

	public Planet [] getColonies(Player owner) throws ApplicationError {
        ArrayList list = null;
        Planet [] all = getPlanets();
        for (int i=0; i<all.length; i++) {
            if (all[i].getOwner() == owner) {
                if (list == null) {
                    list = new ArrayList();
                }
                list.add(all[i]);
            }
        }
        if (list == null) {
            return null;
        } else {
            return (Planet []) list .toArray(new Planet[list.size()]);
        }
	}
	
	public boolean containsMiningColony() throws ApplicationError {
        Planet [] all = getPlanets();
        for (int i=0; i<all.length; i++) {
            if (all[i].hasOwner()) {
                if (all[i].isMiningColony()) {
                	return true;
                }
            }
        }
		return false;
	}

	public Planet getRandomPlanet() {
		Planet [] pa = getPlanets();
		if (pa == null || pa.length == 0) return null;
		if (pa.length == 1) return pa[0];
		int ss = Utils.random(pa.length);
		return pa[ss];
	}

	public Planet getRandomColony() throws ApplicationError {
		Planet [] pa = getColonies(false);
		if (pa == null) return null;
		if (pa.length == 1) return pa[0];
		int ss = Utils.random(pa.length);
		return pa[ss];
	}

	public Planet getRandomColony(Player owner) throws ApplicationError {
		Planet [] pa = getColonies(owner);
		if (pa == null) return null;
		if (pa.length == 1) return pa[0];
		int ss = Utils.random(pa.length);
		return pa[ss];
	}

	public void flushCache() {
		planets = null;
	}

	public int getQtyPlanets() {
		Planet [] planets = getPlanets();
		if (planets != null) {
			return planets.length;
		} else {
			return 0;
		}
	}

    /**
     * Returns true if and only if this stellar object has one or more owners.
     */
    public boolean hasOwner() throws ApplicationError {
    	StellarObjectOrbital orbital = getOrbital();
        if (orbital != null) {
            return true;
        }
		getPlanets();
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].getOwner() != null) {
					return true;
				}
			}
		}
        return false;
    }

    /**
     * Returns true if player is the owner of this solar system.
     */
	public boolean isOwnedBy(Player player) throws ApplicationError {
		StellarObjectOrbital orbital = getOrbital();
        if (orbital != null && orbital.isOwnedBy(player)) {
            return true;
        }
		getPlanets();
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].isOwnedBy(player)) {
					return true;
				}
			}
		}
        return false;
	}

    /**
     * Returns the list of owners of this solar system.
     */
    public Player [] getOwners() throws ApplicationError {
		getPlanets();
		ArrayList list = null;
		StellarObjectOrbital orbital = getOrbital();
        if (orbital != null) {
            if (list == null) list = new ArrayList();
            list.add( orbital.getOwner() );
        }
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				Player owner = planets[i].getOwner();
				if (owner != null) {
                    if (list == null) list = new ArrayList();
                    if (!Comp.in(owner, list)) {
                    	list.add( owner );
                    }
				}
			}
		}
        if (list == null) {
            return null;
        } else {
    		return (Player [])list.toArray(new Player [list.size()]);
        }
    }

    public Player getSingleOwnerOrNull() throws ApplicationError {
		getPlanets();
		Player owner = null;
		StellarObjectOrbital orbital = getOrbital();
        if (orbital != null) {
            owner = orbital.getOwner();
        }
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				Player po = planets[i].getOwner();
				if (po != null) {
					if (owner != null && po != owner) return null;
					owner = po;
				}
			}
		}
        return owner;
    }
    
	public int getPopulation(Player owner) throws ApplicationError {
		getPlanets();
		int r = 0;
//		StellarObjectOrbital orbital = getOrbital();
//        if (orbital != null) {
//            r = orbital.getOrbital().getPopulation(0, 0);
//        }
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].isOwnedBy(owner)) {
					r += (int) planets[i].getPopulation();
				}
			}
		}
        return r;
	}

	public int getMaxIndustryWork(Player owner) throws ApplicationError {
		getPlanets();
		int r = 0;
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].isOwnedBy(owner)) {
					r += (int) planets[i].getMaxIndustryWork();
				}
			}
		}
        return r;
	}

	public boolean isBlockaded() throws ApplicationError {
		getPlanets();
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].isBlockaded()) return true;
			}
		}
		return false;
	}


	public boolean hasDefendedEnemyOwner(Player p) throws ApplicationError {
		getPlanets();
		StellarObjectOrbital orbital = getOrbital();
        if (orbital != null) {
            if (p.isEnemy(orbital.getOwner())) {
            	return true;
            }
        }
		if (planets != null) {
			for (int i=0; i<planets.length; i++) {
				if (planets[i].getOwner() != null) {
		            if (p.isEnemy(planets[i].getOwner()) && planets[i].hasDefenses()) {
		            	return true;
		            }
				}
			}
		}
		return false;
	}

	public boolean hasTradeRoute() {
		TradeRoutes tradeRoutes = MotEngine.instance.getTradeRoutes();
		if (tradeRoutes == null) return false;
		return tradeRoutes.getQtyTradeRoutes(this) > 0;
	}

	public TradeRoute [] getTradeRoutes() {
		TradeRoutes tradeRoutes = MotEngine.instance.getTradeRoutes();
		if (tradeRoutes == null) return null;
		return tradeRoutes.getTradeRoutes(this);
	}

	public boolean isPartOfTutorial() {
		return MotEngine.instance.getDate() < MotEngine.START_DATE + 5 && Comp.same(getName(), GameConstants.TUTORIAL_FIRST_COLONY_NAME);
	}


    /**
     * This is the name of the bitmap used to display this star system on the screen.
     */
    private transient StellarObjectGraphics starGraphics = null;
    
    public StellarObjectGraphics getStellarObjectGraphics() throws ApplicationError {
        if (starGraphics == null) {
            starGraphics = GraphicsFactory.getGalaxyGraphicsList().getStarGraphicsFor(starType);
            if (starGraphics == null) {
                throw new ApplicationError("SolarSystem.getImage() No image defined for start type "+starType);
            }
        }
        return starGraphics;
    }
    
    /**
     * Generates new a new SolaarSystemReference to this solar system.
     */
    public StellarObjectReference getObjectReference() {
        return new SolarSystemReference(this);
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
        writer.addElement("star-type", starType);
	}

	protected String getElementName() { return "solar-system"; }

}