/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.bean.Area;
import mot.common.bean.GalaxyDetails;
import mot.common.bean.Geometry;
import mot.common.bean.IntersectionLocations;
import mot.common.bean.Location;
import mot.common.bean.NamedLocatedObject;
import mot.common.bean.Nebula;
import mot.common.bean.Size;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.PropertyManager;
import mot.common.util.Utils;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.players.Player;

/**
 * This contains the list fo solar systems defined int he galaxy.
 * The size of the galaxy is defined as size and is set in the MotFactory.
 */
public class StellarObjects extends XMLBean {

	private static final int SECTOR_SIZE = PropertyManager.getMandatoryNoErrorAsInt("sector.size");
	
    /**
     * The list of stellar objects
     */
    private ArrayList list;
    
    /**
     * The size of the galaxy
     */
    private GalaxyDetails galaxyDetails;
    
    public ArrayList getList() { return list; }
    public void setList(ArrayList v) { list = v; }

    public Size getSize() { return getGalaxyDetails().getSize(); }

    public void setSize(Size v) {
    	if (galaxyDetails == null) {
        	galaxyDetails = new GalaxyDetails(v, SECTOR_SIZE);
    	} else {
        	galaxyDetails.setSize(v); 
    	}
    }
    
    public int getSectorSize() {
    	return SECTOR_SIZE;
    }

	public GalaxyDetails getGalaxyDetails() {
    	if (galaxyDetails == null) {
        	galaxyDetails = new GalaxyDetails(new Size(0,0), SECTOR_SIZE);
    	}
		return galaxyDetails;
	}
    
    public StellarObjects() {
	}

    public int getQty() {
		if (list == null) {
			return 0;
		} else {
			return list.size();
		}
	}

    /**
     * A list of all the stellar objects
     * Dynamically generated from attribute list.
     */
    private transient StellarObject [] stellarObjects;
    public StellarObject [] getStellarObjects() {
        if (list == null) return null;
		if (stellarObjects == null) {
			stellarObjects = (StellarObject []) list.toArray( new StellarObject[list.size()]);
		}
		return stellarObjects;
	}

    /**
     * A list of all the solar systems
     * Dynamically generated from attribute list.
     */
    private transient SolarSystem [] solarSystems;
    public SolarSystem [] getSolarSystems() {
        if (list == null) return null;
		if (solarSystems == null) {
            ArrayList candidates = new ArrayList();
            Iterator i = list.iterator();
            while (i.hasNext()) {
                StellarObject so = (StellarObject) i.next();
                if (so instanceof SolarSystem) {
                    candidates.add(so);
                }
            }
			solarSystems = (SolarSystem []) candidates.toArray( new SolarSystem[candidates.size()]);
		}
		return solarSystems;
	}
    
    /**
     * A list of all the nebula in the galaxy
     */
    public Nebula [] getNebulae() {
        return getGalaxyDetails().getNebulaeAsArray();
	}
    
    /**
     * A list of all the nebula
     * Dynamically generated from attribute list.
     */
    private transient BlackHole [] blackHoles;
    public BlackHole [] getBlackHoles() {
        if (list == null) return null;
		if (blackHoles == null) {
            ArrayList candidates = new ArrayList();
            Iterator i = list.iterator();
            while (i.hasNext()) {
                StellarObject so = (StellarObject) i.next();
                if (so instanceof BlackHole) {
                    candidates.add(so);
                }
            }
            blackHoles = (BlackHole []) candidates.toArray( new BlackHole[candidates.size()]);
		}
		return blackHoles;
	}
    
    /**
     * Used by XMLDocumentHandler to load saved games.
     * Longcut for add(StellarObject).
     */
    public void setSolarSystem(SolarSystem ss) { add(ss);  }
    public void setBlackHole(BlackHole ss) { add(ss); }
    public void setNebula(Nebula n) { getGalaxyDetails().setNebulae(n); }

    /**
     * Adds a new StellarObject to this list.
     * Flushes the array cache of stellarObjects and solarSystems.
     */
    public void add(StellarObject stellarObject) {
		stellarObjects = null;
		solarSystems = null;
		blackHoles = null;
        if (list == null) list = new ArrayList();
		list.add(stellarObject);
	}

    /**
     * Removes the StellarObject from this list.
     * Flushes the array cache of stellarObjects and solarSystems.
     */
    public void remove(StellarObject stellarObject) {
		stellarObjects = null;
		solarSystems = null;
		blackHoles = null;
        if (list == null) return;
		list.remove(stellarObject);
	}

    /**
     * If this method cannot find a Stellar object with this name it searches 
     * old names for a stellar object of this name.
     * @return a StellarObject with this name if it exists or null.
     */
    public StellarObject getStellarObject(String name) {
		StellarObject [] sos = getStellarObjects();
		for (int i=0; i<sos.length; i++) {
			if (sos[i].getName().equalsIgnoreCase(name)) {
				return sos[i];
			}
		}
		for (int i=0; i<sos.length; i++) {
			if (sos[i].didHaveName(name)) {
				return sos[i];
			}
		}
		return null;
	}

    /**
     * If this method cannot find a Solar System with this name it searches 
     * old names for a Solar System of this name.
     * @return a Solar System with this name if it exists or null.
     */
    public SolarSystem getSolarSystem(String name) {
    	SolarSystem [] sos = getSolarSystems();
    	if (sos == null) return null;
		for (int i=0; i<sos.length; i++) {
			if (sos[i].getName().equalsIgnoreCase(name)) {
				return sos[i];
			}
		}
		for (int i=0; i<sos.length; i++) {
			if (sos[i].didHaveName(name)) {
				return sos[i];
			}
		}
		return null;
	}

    /**
     * @return a Nebula with this name if it exists or null.
     */
    public Nebula getNebula(String name) {
    	Nebula [] sos = getNebulae();
    	if (sos == null) return null;
		for (int i=0; i<sos.length; i++) {
			if (sos[i].getName().equals(name)) {
				return sos[i];
			}
		}
		return null;
	}

    /**
     * @return a nebula at this location if it exists or null.
     */
	public Nebula getNebula(Location location) {
		Nebula[] nebulae = getNebulae();
		if (nebulae != null) {
			for (int i = 0; i < nebulae.length; i++) {
				if (nebulae[i].contains(location)) {
					return nebulae[i];
				}
			}
		}
		return null;
	}
	
    /**
     * @return a Nebula with this name if it exists or null.
     */
    public BlackHole getBlackHole(String name) {
    	BlackHole [] sos = getBlackHoles();
    	if (sos == null) return null;
		for (int i=0; i<sos.length; i++) {
			if (sos[i].getName().equals(name)) {
				return sos[i];
			}
		}
		return null;
	}

	public GalacticIncumbent [] getGalacticIncumbentsBetween(Location start, Location finish) throws ApplicationError {
		ArrayList<GalacticIncumbent> r = new ArrayList();
		StellarObject[] sos = getStellarObjects();
    	if (sos == null) return null;
		for (int i = 0; i < sos.length; i++) {
			if (sos[i] instanceof GalacticIncumbent) {
				GalacticIncumbent gi = (GalacticIncumbent) sos[i];
				Area area = gi.getArea();
				if (area.contains(start) || area.contains(finish)) {
					r.add(gi);
				} else {					
					IntersectionLocations intersections = Geometry.getIntersections(area, start, finish);
					if (intersections != null && intersections.getFirstEdge() != null) {
						r.add(gi);
					}
				}
			}
		}
		if (r == null || r.size() == 0) {
			return null;
		} else {
			return r.toArray(new GalacticIncumbent[r.size()]);
		}		
	}
	
	public boolean hasGalacticIncumbentsBetween(Location start, Location finish, Class type) throws ApplicationError {
		StellarObject[] sos = getStellarObjects();
    	if (sos == null) return false;
		for (int i = 0; i < sos.length; i++) {
			if (sos[i] instanceof GalacticIncumbent && sos[i].getClass() == type) {
				GalacticIncumbent gi = (GalacticIncumbent) sos[i];
				Area area = gi.getArea();
				IntersectionLocations intersections = Geometry.getIntersections(area, start, finish);
				if (intersections != null && intersections.getFirstEdge() != null) {
					return true;
				}
			}
		}
		return false;		
	}
    /**
     * 
     * @param x The position of the stellar object
     * @param y The position of the stellar object
     * @param player The player the MOOs must be visible by
     * @param withOrders If null all MOOS at this stellar object are returned. If it is true only
     * MOOs with an order to move to a new location are returned. If false only MOOs with no orders
     * are returned.
     * @return A list of visible MoveableOwnedObjects located at a solar system within 
     * position x, x+solar system xdim and y+solar system ydim. The MOOs must be visible 
     * by player passed in.
     * @throws mot.common.util.ApplicationError
     */
    public MoveableOwnedObject [] getVisibleMOOsAtStellarObjectAt(int x, int y, Player player, Boolean withOrders) throws ApplicationError {
        StellarObject so = getStellarObjectAt(x, y);
        MoveableOwnedObject [] moos = null;
        if (so == null) {
            return null;
        } else {
            MotEngine engine = MotEngine.getInstance();
            moos = engine.getMoveableOwnedObjects().getVisibleMOOsAt(so, player);
        }
        
        if (moos == null || withOrders == null) {
            return moos;
        }
        
        // This will include all craft in orbit around this solar system.
        // So we must remove crafts with no orders if withOrders = true
        // or remove crafts with orders if withOrders = false
        
        ArrayList list = new ArrayList();
        for (int i=0; i<moos.length; i++) {
            if (withOrders.booleanValue()) {
                if (moos[i].getNavigator() != null) list.add(moos[i]);
            } else {
                if (moos[i].getNavigator() == null) list.add(moos[i]);
            }
        }
        
        if (list.size() == 0) {
            return null;
        }
        
        return (MoveableOwnedObject [])list.toArray(new MoveableOwnedObject[list.size()]);
	}

    /**
     * Returns a solar system located within position x, x+solar system xdim and y+solar system ydim.
     */
    public StellarObject getStellarObjectAt(int x, int y) throws ApplicationError {
		StellarObject [] sos = getStellarObjects();
    	if (sos == null) return null;
		for (int i=0; i<sos.length; i++) {
			if (sos[i].isInside(x,y)) {
				return sos[i];
			}
		}
		return null;
	}


    /**
     * Returns a stellar object located at position x,y.
     */
    public StellarObject getStellarObjectAt(Location loc) {
		StellarObject [] sos = getStellarObjects();
    	if (sos == null) return null;
		for (int i=0; i<sos.length; i++) {
			if (sos[i].getLocation().equals(loc)) {
				return sos[i];
			}
		}
		return null;
	}

    /**
     * returns the maximum position of any planet along the x axis.
     * This is used by the main window to determine the seize of the panel to render.
     */
    public int getMaxX() {
		StellarObject [] sos = MotEngine.instance.getStellarObjects().getStellarObjects();
		int x = 0;
		for (int i=0; i<sos.length; i++) {
            if ( x < sos[i].getLocation().getX() ) {
                x = sos[i].getLocation().getX();
            }
		}
		return x+25;
    }

    /**
     * returns the maximum position of any planet along the y axis.
     * This is used by the main window to determine the seize of the panel to render.
     */
    public int getMaxY() {
		StellarObject [] sos = getStellarObjects();
		int y = 0;
		for (int i=0; i<sos.length; i++) {
            if ( y < sos[i].getLocation().getY() ) {
                y = sos[i].getLocation().getY();
            }
		}
		return y+25;
    }
    
    /**
     * @return The closest stellar object to this location in the game.
     */
	public StellarObject getClosestStellarObject(Location loc) throws ApplicationError {
		StellarObject [] sos = getStellarObjects();
		if (sos == null) return null;
		StellarObject closest = null;
		for (int i=0; i<sos.length; i++) {
			if ( (closest == null || sos[i].getDistanceTo(loc) < closest.getDistanceTo(loc)) ) {
				closest = sos[i];
			}
		}
		return closest;
	}
    
    /**
     * NOTE - Range is passed in because some craft can travel further than others!
     * @return The closest stellar objects not owned by player that is at most range away from a solar system owned by player. 
     */
    public StellarObject getClosestWithinRange(Player player, int range) throws ApplicationError {
        StellarObject[] mySos = player.getStellarObjects();
		StellarObject [] sos = getStellarObjects();
		StellarObject best = null;
		int bestDist = -1;
		for (int i=0; i<sos.length; i++) {
            int dist = sos[i].getClosestDistance(mySos);
            if (dist <= range) {
                if (best == null || dist < bestDist) {
                	best = sos[i];
                	bestDist = dist;
                }
            }
        }        
		return best;
    }

    public StellarObject getClosestWithinRangeWithNoOwner(Player player, int range) throws ApplicationError {
        StellarObject[] mySos = player.getStellarObjects();
		StellarObject [] sos = getStellarObjects();
		StellarObject best = null;
		int bestDist = -1;
		for (int i=0; i<sos.length; i++) {
            int dist = sos[i].getClosestDistance(mySos);
            if (dist <= range) {
            	if (sos[i].getOwners() == null) {            		
            		if (best == null || dist < bestDist) {
            			best = sos[i];
            			bestDist = dist;
            		}
            	}
            }
        }        
		return best;
    }

    public Planet getClosestStellarObjectWithinRangeOf(Player owner, Player traveller, StellarObject closestToTarget) throws ApplicationError {
    	if (traveller == null || traveller.getEngines() == null) {
    		return null;
    	}
    	int range = traveller.getEngines().getMaxTravelDistance();
    	        
		Planet best = null;
		int bestDistance = 0;
        StellarObject[] sos = owner.getStellarObjects();
        if (sos == null) {
        	// owner has no stellar objects!
        	return null;
        }
		for (int i=0; i<sos.length; i++) {
            StellarObject closest = traveller.getClosestStellarObject(sos[i].getLocation());
            if (closest == null) {
            	// traveller has no stellar objects!
            	return null;
            }
			int dist = closest.getDistanceTo(sos[i].getLocation());
            if (dist <= range) {
            	if (sos[i] != closestToTarget && sos[i] instanceof SolarSystem) {
            		SolarSystem so = (SolarSystem) sos[i];
            		Planet planet = so.getRandomPlanet();
            		if (closestToTarget == null) {
            			Logger.logWarning("StellarObjects.getClosestStellarObjectWithinRangeOf() closestToTarget is NULL");
            			return planet;
            		}
            		if (planet != null && so.getQtyPlanets() > 0) {
            			if (best == null) {
            				best = planet;
            				bestDistance = planet.getSolarSystem().getDistanceTo(closestToTarget);
            			} else {
            				int newDist = planet.getSolarSystem().getDistanceTo(closestToTarget);
            				if (bestDistance > newDist) {
                				best = planet;
                				bestDistance = newDist;
            				}
            			}
            		}
            	}
            }
        }  
		
		return best;
    }
    

    public Planet getRandomPlanetWithinRangeOf(Player owner, Player traveller) throws ApplicationError {
    	if (traveller == null || traveller.getEngines() == null) {
    		return null;
    	}
    	int range = traveller.getEngines().getMaxTravelDistance();
    	        
		ArrayList candidates = null;
        StellarObject[] sos = owner.getStellarObjects();
        if (sos == null) {
        	// owner has no stellar objects!
        	return null;
        }
		for (int i=0; i<sos.length; i++) {
            StellarObject closest = traveller.getClosestStellarObject(sos[i].getLocation());
            if (closest == null) {
            	// traveller has no stellar objects!
            	return null;
            }
			int dist = closest.getDistanceTo(sos[i].getLocation());
            if (dist <= range) {
            	if (sos[i] instanceof SolarSystem) {
            		SolarSystem so = (SolarSystem) sos[i];
            		Planet planet = so.getRandomColony(owner);
            		if (planet != null) {
            			if (candidates == null) {
            				candidates = new ArrayList();
            			}
            			candidates.add(planet);            			
            		}
            	}
            }
        }        
		if (candidates == null || candidates.size() == 0) {
			return null;
		} else {
            int ss = Utils.random(candidates.size());
            Planet planet = (Planet) candidates.get(ss);
			return planet;
		}		
    }
    
    public BlackHole getWormHoleWithinRangeOf(Player owner, Player traveller) throws ApplicationError {
    	if (traveller == null || traveller.getEngines() == null) {
    		return null;
    	}
    	int range = traveller.getEngines().getMaxTravelDistance();
    	        
        StellarObject[] sos = owner.getStellarObjects();
        if (sos == null) {
        	// owner has no stellar objects!
        	return null;
        }
		for (int i=0; i<sos.length; i++) {
            StellarObject closest = traveller.getClosestStellarObject(sos[i].getLocation());
            if (closest == null) {
            	// traveller has no stellar objects!
            	return null;
            }
			int dist = closest.getDistanceTo(sos[i].getLocation());
            if (dist <= range) {
            	if (sos[i] instanceof BlackHole) {
            		BlackHole bh = (BlackHole) sos[i];
            		if (bh.isSpaceJumpAbility()) {
            			return bh;
            		}
            	}
            }
        }        
		return null;
    }
    
    public BlackHole getBlackHoleWithinRangeOf(Player owner, Player traveller) throws ApplicationError {
    	if (traveller == null || traveller.getEngines() == null) {
    		return null;
    	}
    	int range = traveller.getEngines().getMaxTravelDistance();
    	        
        StellarObject[] sos = owner.getStellarObjects();
        if (sos == null) {
        	// owner has no stellar objects!
        	return null;
        }
		for (int i=0; i<sos.length; i++) {
            StellarObject closest = traveller.getClosestStellarObject(sos[i].getLocation());
            if (closest == null) {
            	// traveller has no stellar objects!
            	return null;
            }
			int dist = closest.getDistanceTo(sos[i].getLocation());
            if (dist <= range) {
            	if (sos[i] instanceof BlackHole) {
            		BlackHole bh = (BlackHole) sos[i];
           			return bh;
            	}
            }
        }        
		return null;
    }
    
    /**
     * NOTE - Range is passed in because some craft can travel further than others!
     * @return A list of all stellar objects not owned by player that are at most range away from a solar system owned by player. 
     */
    public StellarObject [] getAllWithinRange(Player player, int range) throws ApplicationError {
        StellarObject[] mySos = player.getStellarObjects();
		ArrayList result = null;
		StellarObject [] sos = getStellarObjects();
		for (int i=0; i<sos.length; i++) {
            int dist = sos[i].getClosestDistance(mySos);
            if (dist <= range) {
                if (result == null) {
                    result = new ArrayList();
                }
                result.add(sos[i]);
            }
        }        
		if (result == null || result.size() == 0) {
			return null;
		} else {
			return (StellarObject[]) result.toArray(new StellarObject[result.size()]);
		}		
    }

    /**
     * @param mySos A list of my stellar objects, all returned stellar objects must be
     * within range of these.
     * @param range The maximum distance from the closest mySos object.
     * @return The A list of Stellar Objects with range of mySos.
     */
    public ArrayList getNotVistedWithinRange(Player player, NamedLocatedObject [] mySos, int range, boolean noCrafts) throws ApplicationError {
        if (list == null || mySos.length == 0) return null;
		ArrayList result = null;
		StellarObject [] sos = getStellarObjects();
		for (int i=0; i<sos.length; i++) {
            int date = sos[i].getLastVisitedDate(player);
            if (date == -1 && !Comp.in(sos[i], mySos)) {
                int dist = sos[i].getClosestDistance(mySos);
                if (dist <= range) {
                    int qtyCrafts = 0;
                    if (noCrafts && MotEngine.instance.getCrafts() != null) {
                        qtyCrafts = MotEngine.instance.getCrafts().getQtyAt(sos[i].getLocation());
                    }
                    if (qtyCrafts == 0) {
                        if (result == null) {
                            result = new ArrayList();
                        }
                        result.add(sos[i]);
                    }
                }
            }
        }        
        return result;
    }

    /**
     * @param mySos A list of stellar objects my stellar objects, all returned stellar objects must be
     * within range of these.
     * @param range The maximum distance from the closest mySos object.
     * @return The closest stellar object to any stellar object in mySos and less than range in distance.
     */
    public StellarObject getClosestNotVistedWithinRange(Player player, NamedLocatedObject [] mySos, int range, boolean noCrafts) throws ApplicationError {
        if (list == null || mySos.length == 0) return null;
		StellarObject best = null;
        int bestDist = Integer.MAX_VALUE;
		StellarObject [] sos = getStellarObjects();
		for (int i=0; i<sos.length; i++) {
            int date = sos[i].getLastVisitedDate(player);
            if (date == -1 && !Comp.in(sos[i], mySos)) {
                int qtyCrafts = 0;
                if (noCrafts && MotEngine.instance.getCrafts() != null) {
                    qtyCrafts = MotEngine.instance.getCrafts().getQtyAt(sos[i].getLocation());
                }
                if (qtyCrafts == 0) {
                    int dist = sos[i].getClosestDistance(mySos);
                    if (dist < range && dist < bestDist) {
                        best = sos[i];
                        bestDist = dist;
                    }
                }
            }
        }        
        return best;
    }
    
    /**
     * @return A random stellar object from all the stellar objects.
     * @throws mot.common.util.ApplicationError
     */
    public StellarObject getRandomStellarObject(boolean canHaveOwner) throws ApplicationError {
		StellarObject [] sos = getStellarObjects();
		if (sos == null) return null;
		ArrayList candidates = new ArrayList();
		for (int i=0; i<sos.length; i++) {
			if ( canHaveOwner || !sos[i].hasOwner() ) {
				candidates.add(sos[i]);
			}
		}
		if (candidates.size() == 0) {
            return null;
        }
        int i = Utils.random(candidates.size());
        return (StellarObject) candidates.get(i);
    }
    
    /**
     * @param p The player the selected stellar object must belong to.
     * @return A random stellar object from all the stellar objects in this list that belongs to player p.
     * @throws mot.common.util.ApplicationError
     */
    public StellarObject getRandomStellarObjectOwnedBy(Player p) throws ApplicationError {
		StellarObject [] sos = getStellarObjects();
		if (sos == null) return null;
		ArrayList candidates = new ArrayList();
		for (int i=0; i<sos.length; i++) {
			if ( sos[i].isOwnedBy(p) ) {
				candidates.add(sos[i]);
			}
		}
		if (candidates.size() == 0) {
            return null;
        }
        int i = Utils.random(candidates.size());
        return (StellarObject) candidates.get(i);
    }
    
    /**
     * Remove all visitor objects in all stellar objects for player p
     */
    public void removeVisitors(Player p) {
		StellarObject [] sos = getStellarObjects();
        if (sos == null) return;
		for (int i=0; i<sos.length; i++) {
            sos[i].removeVisitor(p);
		}
    }

    /**
     * Remove all outposts in all stellar objects for player p
     */
    public void removeStellarObjectOrbitals(Player p) throws ApplicationError {
		StellarObject [] sos = getStellarObjects();
        if (sos == null) return;
		for (int i=0; i<sos.length; i++) {
            if (sos[i].getOrbital() != null) {
            	StellarObjectOrbital outpost = sos[i].getOrbital();
                if (outpost.getOwner() == p) {
                    sos[i].setOutpost(null);
                }
            }
		}
    }

	/**
	 * Performs any operations these objects need to do at the end of every turn.
     * Currently this only includes moving all the spies.
  	 * @throws ApplicationError 
	 */
	public void endTurn() throws ApplicationError {
		StellarObject [] sos = getStellarObjects();
		if (sos == null) return;
		for (int i=0; i<sos.length; i++) {
            sos[i].endTurn();
		}
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (getStellarObjects() != null) {
			StellarObject [] sos = getStellarObjects();
			for (int i=0; i<sos.length; i++) {
				if (sos[i] != null) {
					sos[i].toXML(writer);
				}
			}
		}
		if (galaxyDetails != null) {
			Size size = galaxyDetails.getSize();
			Nebula[] nebulae = galaxyDetails.getNebulaeAsArray();
			if (size != null) size.toXML(writer);
			if (nebulae != null) {
				for (int i = 0; i < nebulae.length; i++) {
					nebulae[i].toXML(writer);
				}
			}
		}
	}

	protected String getElementName() { return "stellar-objects"; }
	
}