/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.technology.Humanities;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;
import mot.common.util.NameValuePair;
import mot.common.util.NameValuePairList;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.players.Player;

/**
 * This is a trade route between two different planets that generates income.
 * 
 * @author paul
 *
 */
public class TradeRoute extends XMLBean {
	public static final double BONUS_EFFECT = 0.05;
	
	private SolarSystemReference solarSystemReference1;
	private SolarSystemReference solarSystemReference2;
	private int startDate;
	private int blockadeTurns;
	
	public TradeRoute() {
		
	}
	
	public TradeRoute(SolarSystemReference ss1, SolarSystemReference ss2, int startDate) {
		this.solarSystemReference1 = ss1;
		this.solarSystemReference2 = ss2;
		this.startDate = startDate;
	}
	
	public TradeRoute(SolarSystem ss1, SolarSystem planet2, int startDate) {
		this.solarSystemReference1 = new SolarSystemReference(ss1);
		this.solarSystemReference2 = new SolarSystemReference(planet2);
		this.startDate = startDate;
	}
	
	public boolean isBlockaded() throws ApplicationError {
		SolarSystem ss1 = getSolarSystem1();
		SolarSystem ss2 = getSolarSystem2();
		return ss1.isBlockaded() || ss2.isBlockaded();
	}
	
	public void endTurn() throws ApplicationError {
		if (isBlockaded()) {
			blockadeTurns++;
		} else {
			blockadeTurns = 0;
		}
	}
	
	/**
	 * Determines if this trade route is valid (being that all sectors inclusive between 
	 * planet1 and planet 2 are controlled by their owner or players with a trade treaty 
	 * with the owner.
	 * @return TRUE iff this trade route is valid
	 * @throws ApplicationError 
	 */
	public boolean isValid() throws ApplicationError {
		SolarSystem ss1 = getSolarSystem1();
		SolarSystem ss2 = getSolarSystem2();
		if (blockadeTurns > 1) return false;
		return TradeRouteFactory.controlAllSectorsBetween(ss1, ss2);
	}
	
//	public Player getSolarSystem1SingleOwnerOrNull() throws ApplicationError {
//		SolarSystem v = getSolarSystem1();
//		if (v == null) return null;
//		return v.getSingleOwnerOrNull();
//	}
//
//	public Player getSolarSystem2SingleOwnerOrNull() throws ApplicationError {
//		SolarSystem v = getSolarSystem2();
//		if (v == null) return null;
//		return v.getSingleOwnerOrNull();
//	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (solarSystemReference1 != null) solarSystemReference1.toXML(writer, "solar-system-reference1");
		if (solarSystemReference2 != null) solarSystemReference2.toXML(writer, "solar-system-reference2");
		writer.addElement("start-date", startDate);
		writer.addElement("blockade-turns", blockadeTurns);
	}

	protected String getElementName() {
		return "trade-route";
	}

	public SolarSystem getSolarSystem1() {
		return solarSystemReference1 == null ? null : solarSystemReference1.getSolarSystem();
	}

	public SolarSystemReference getSolarSystemReference1() {
		return solarSystemReference1;
	}

	public void setSolarSystemReference1(SolarSystemReference planet1) {
		this.solarSystemReference1 = planet1;
	}

	public SolarSystem getSolarSystem2() {
		return solarSystemReference2 == null ? null : solarSystemReference2.getSolarSystem();
	}

	public SolarSystemReference getSolarSystemReference2() {
		return solarSystemReference2;
	}

	public void setSolarSystemReference2(SolarSystemReference v) {
		this.solarSystemReference2 = v;
	}

	public int getStartDate() {
		return startDate;
	}

	public void setStartDate(int startDate) {
		this.startDate = startDate;
	}

	@Override
	public String toString() {
		return "TradeRoute [" + solarSystemReference1 + "<=>" + solarSystemReference2+ "]";
	}

	public String getSolarSystem1Name() {
		return (solarSystemReference1 == null) ? null : solarSystemReference1.getName();
	}

	public String getSolarSystem2Name() {
		return (solarSystemReference2 == null) ? null : solarSystemReference2.getName();
	}

	public boolean hasSolarSystem(SolarSystem p) {
		return solarSystemReference1 != null && solarSystemReference1.equals(p) || solarSystemReference2 != null && solarSystemReference2.equals(p);
	}

	public Player getOwner() throws ApplicationError {
		SolarSystem v = getSolarSystem1();
		if (v == null) return null;
		return v.getSingleOwnerOrNull();
	}

	public boolean same(SolarSystem ss1, SolarSystem ss2) {
		return (getSolarSystem1() == ss1 && getSolarSystem2() == ss2) ||
			   (getSolarSystem1() == ss2 && getSolarSystem2() == ss1);
	}

	/**
	 * @return The economic production contributed by the trade route to each colony owned by viewer
	 * @throws ApplicationError 
	 */
	public NameValuePairList getIndustryOutput(Player viewer) throws ApplicationError {
		NameValuePairList result = new NameValuePairList();
		getIndustryOutput(solarSystemReference1.getSolarSystem(), result, viewer);
		getIndustryOutput(solarSystemReference2.getSolarSystem(), result, viewer);
		if (result.getSize() == 0) {
			return null;
		} else {
			return result;
		}
	}

	private static void getIndustryOutput(SolarSystem solarSystem, NameValuePairList result, Player viewer) throws ApplicationError {
		if (solarSystem == null) return;
		Planet[] planets = solarSystem.getPlanets();
		if (planets == null) return;
		Player owner = solarSystem.getSingleOwnerOrNull();
		if (owner == null) return;
		double tradeRouteBonus = TradeRoute.BONUS_EFFECT;
		Humanities humanities = owner.getHumanities();
		if (humanities != null) {
			tradeRouteBonus += humanities.getTradeRouteBonus();
		}
    	if (tradeRouteBonus <= 0.0) return;
		for (int i = 0; i < planets.length; i++) {
			if (planets[i].isOwnedBy(viewer) && !planets[i].isMiningColony()) {
				int r = (int) (planets[i].getCoreIndustryWork() * planets[i].getPlanetProductionBonuses() * tradeRouteBonus);
				if (r > 0) result.add(new NameValuePair(planets[i].getName(), r));
			}
		}
	}

	/**
	 * @return The economic production contributed by the trade route to each colony owned by viewer
	 * @throws ApplicationError 
	 */
	public NameValuePairList getResearchOutput(Player viewer) throws ApplicationError {
		NameValuePairList result = new NameValuePairList();
		getResearchOutput(solarSystemReference1.getSolarSystem(), result, viewer);
		getResearchOutput(solarSystemReference2.getSolarSystem(), result, viewer);
		if (result.getSize() == 0) {
			return null;
		} else {
			return result;
		}
	}

	private static void getResearchOutput(SolarSystem solarSystem, NameValuePairList result, Player viewer) throws ApplicationError {
		if (solarSystem == null) return;
		Planet[] planets = solarSystem.getPlanets();
		if (planets == null) return;
		double tradeRouteBonus = TradeRoute.BONUS_EFFECT + Technologies.getHumanities().getTradeRouteBonus();
    	if (tradeRouteBonus <= 0.0) return;
		for (int i = 0; i < planets.length; i++) {
			if (planets[i].isOwnedBy(viewer) && !planets[i].isMiningColony()) {
				int r = (int) (planets[i].getPlanetaryResearch(false) * tradeRouteBonus);
				if (r > 0) result.add(new NameValuePair(planets[i].getName(), r));
			}
		}
	}

	public int getBlockadeTurns() {
		return blockadeTurns;
	}

	public void setBlockadeTurns(int blockadeTurns) {
		this.blockadeTurns = blockadeTurns;
	}


}
