/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.engine.MotEngine;
import mot.engine.players.Player;
import mot.engine.relations.Relations;

public class TradeRouteFactory {

	public static TradeRoute create(SolarSystem ss1, SolarSystem ss2) {
		TradeRoutes trs = MotEngine.instance.getTradeRoutes();
		int date = MotEngine.instance.getDate();
		TradeRoute tr = new TradeRoute(ss1, ss2, date);
		trs.setTradeRoute(tr);
		return tr;
	}

	/**
	 * Can the solar system ss build a trade route to any other solar system in the galaxy at this point in time.
	 * @param ss The solar system establishing a new trade route
	 * @throws ApplicationError 
	 */
	public static boolean canCreateTradeRoute(SolarSystem ss) throws ApplicationError {
		if (!canSupportTradeRoute(ss)) return false;
		if (ss.isBlockaded()) return false;
		Player owner = ss.getSingleOwnerOrNull();
		if (owner == null) return false;
		int maxDistance = owner.getMaxSolarSystemTradeRouteDistance();
		Planet[] planets = owner.getPlanets();
		if (planets == null) return false;
		for (int i = 0; i < planets.length; i++) {
			SolarSystem css = planets[i].getSolarSystem();
			int distance = ss.getDistanceTo(css);
			if (distance < maxDistance && canSupportTradeRoute(css)) return true;
		}
		return false;
	}
	
	/**
	 * Can a trade route be created by origin and connect to destination
	 * @param origin The solar system establishing a new trade route
	 * @param destination The destination of the trade route 
	 * THIS MAY BE NULL
	 * When null it returns if planet origin can support a new trade route
	 * @return
	 * @throws ApplicationError 
	 */
	public static boolean canCreateTradeRoute(SolarSystem origin, SolarSystem destination) throws ApplicationError {
		if (origin == destination) return false;
		if (!canSupportTradeRoute(origin)) return false;
		if (origin.isBlockaded()) return false;
		
		if (destination != null) {
			if (!canSupportTradeRoute(destination)) return false;
			if (destination.isBlockaded()) return false;
			
			int distance = origin.getLocation().getDistanceTo(destination.getLocation());
			int max = origin.getSingleOwnerOrNull().getMaxSolarSystemTradeRouteDistance();
			if (distance > max) return false;
			TradeRoute existing = MotEngine.instance.getTradeRoutes().getTradeRoute(origin, destination);
			if (existing != null) return false;
			if (!controlAllSectorsBetween(origin, destination)) return false;
		}
		
		return true;
	}

	private static boolean canSupportTradeRoute(SolarSystem ss) throws ApplicationError {
		if (ss == null) return false;
		Player owner = ss.getSingleOwnerOrNull();
		if (owner == null) return false;
		
		if (ss.isBlockaded()) return false;
		if (ss.getQtyColonies(false) == 0) return false;
		
		int max = owner.getMaxSolarSystemTradeRoutes();
		if (max == 0) return false;
		SolarSystem capitol = owner.getCapitolSolarSystem();
		if (ss == capitol) max++;
		int qty = MotEngine.instance.getTradeRoutes().getQtyTradeRoutes(ss);
		if (qty >= max) return false;
		
		return true;
	}

	public static boolean controlAllSectorsBetween(SolarSystem ss1, SolarSystem ss2) throws ApplicationError {
		if (!ss1.hasOwner() || !ss2.hasOwner()) return true;
		Location start = ss1.getLocation();
		Location finish = ss2.getLocation();
		Player [] owners = MotEngine.instance.getInfluence().getSectorOwnersBetween(start, finish);
		if (owners == null) return true;
		for (int i = 0; i < owners.length; i++) {
			if (!isValidTrader(ss1, ss2, owners[i])) {
				return false;
			}
		}
		return true;
	}

	private static boolean isValidTrader(SolarSystem ss1, SolarSystem ss2, Player player) throws ApplicationError {
		Player owner1 = ss1.getSingleOwnerOrNull();
		Player owner2 = ss2.getSingleOwnerOrNull();
		if (owner1 == null || owner2 == null) return false;
		Relations rels = MotEngine.instance.getRelationsNotNull();
		return rels.canTradeWith(owner1, player) && rels.canTradeWith(owner2, player);
	}

}
