/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.planets;

import java.util.ArrayList;
import java.util.HashMap;

import mot.common.technology.Humanities;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.common.StellarObject;
import mot.engine.players.Player;

/**
 * This is a list of all trade routes the game.
 * 
 * @author paul
 *
 */
public class TradeRoutes extends XMLBean {

	private ArrayList<TradeRoute> tradeRoutes = new ArrayList();
	private TradeRoute[] _tradeRoutes = null;

	private HashMap<String, TradeRoute[]> _solarSystemToRouteMap;

	/**
	 * @return All trade routes in the game
	 */
	public TradeRoute[] getTradeRoutes() {
		if (_tradeRoutes == null && tradeRoutes.size() > 0) {
			_tradeRoutes = tradeRoutes.toArray(new TradeRoute[tradeRoutes.size()]);
		}
		return _tradeRoutes;
	}

	public void setTradeRoute(TradeRoute tr) {
		tradeRoutes.add(tr);
		refreshMaps();
	}

	private void addTradeRouteMap(SolarSystem ss, TradeRoute tr) {
		TradeRoute[] tra = _solarSystemToRouteMap.get(ss.getName());
		if (tra == null) {
			tra = new  TradeRoute[1];
			tra[0] = tr;
			_solarSystemToRouteMap.put(ss.getName(), tra);
		} else {
			TradeRoute[] tra2 = new  TradeRoute[tra.length+1];
			for (int i = 0; i < tra.length; i++) {
				tra2[i] = tra[i];
			}
			tra2[tra.length] = tr;
			_solarSystemToRouteMap.put(ss.getName(), tra2);
		}
	}

	public void remove(TradeRoute tr) {
		tradeRoutes.remove(tr);
		refreshMaps();
	}

	private void buildPlanetRouteMap() {
		_solarSystemToRouteMap = new HashMap();
		TradeRoute[] tra = getTradeRoutes();
		if (tra != null) {
			for (int i = 0; i < tra.length; i++) {
				addTradeRouteMap(tra[i].getSolarSystem1(), tra[i]);
				addTradeRouteMap(tra[i].getSolarSystem2(), tra[i]);
			}
		}
	}

	public void refreshMaps() {
		_tradeRoutes = null;
		_solarSystemToRouteMap = null;
	}

	public TradeRoute[] getTradeRoutes(StellarObject so) {
		if (so instanceof SolarSystem) {
			return getTradeRoutes((SolarSystem)so);
		} else {
			return null;
		}
	}
	
	/**
	 * @return All trade routes the containing SolarSystem p
	 */
	public TradeRoute[] getTradeRoutes(SolarSystem p) {
		if (p == null) {
			Logger.logWarning("No SolarSystem Name");
			return null;
		}
		if (_solarSystemToRouteMap == null) {
			buildPlanetRouteMap();
		}
		TradeRoute[] tra = _solarSystemToRouteMap.get(p.getName());
		return tra;
	}
	
	/**
	 * @return All trade routes the containing planet p
	 */
	public TradeRoute getTradeRoute(SolarSystem ss1, SolarSystem ss2) {
		if (ss1 == null || ss2 == null) {
			Logger.logWarning("No SolarSystem Names");
			return null;
		}
		TradeRoute[] trl = getTradeRoutes();
		if (trl == null) return null;
		for (int i = 0; i < trl.length; i++) {
			if (trl[i].same(ss1,ss2)) {
				return trl[i];
			}
		}
		return null;
	}
	
	/**
	 * Removes any trade routes that have become invalid and notifies the owner.
	 * @throws ApplicationError
	 */
	public void endTurn() throws ApplicationError {
		TradeRoute[] tra = getTradeRoutes();
		if (tra != null) {
			for (int i = 0; i < tra.length; i++) {
				if (!tra[i].isValid()) {
					Player planet1Owner = tra[i].getOwner();
					remove(tra[i]);
					planet1Owner.notifyTradeRouteInvalid(tra[i]);
				} else {
					tra[i].endTurn();
				}
			}
		}
	}

	/**
	 * @param planet The planet that enjoys the bonuses returned
	 * @return The bonuses provided by trade routes to planet p. The value 0.1 is a increase of 10%.
	 * @throws ApplicationError 
	 */
	public double getIndustryBonus(SolarSystem ss) throws ApplicationError {
		getTradeRoutes();
		double r = 0;
		if (_tradeRoutes != null) {
			for (int i = 0; i < _tradeRoutes.length; i++) {
				if (_tradeRoutes[i].hasSolarSystem(ss)) {
					Player owner = _tradeRoutes[i].getOwner();
					if (owner != null) {					
						double tradeRouteBonus = TradeRoute.BONUS_EFFECT;
						Humanities humanities = owner.getHumanities();
						if (humanities != null) {
							tradeRouteBonus += humanities.getTradeRouteBonus();
						}
						r+= tradeRouteBonus;
					}
				}
			}
		}
		return r;
	}

	public int getQtyTradeRoutes(SolarSystem p) {
		getTradeRoutes();
		int r = 0;
		if (_tradeRoutes != null) {
			for (int i = 0; i < _tradeRoutes.length; i++) {
				if (_tradeRoutes[i].hasSolarSystem(p)) r++;
			}
		}
		return r;
	}

	@Override
	protected void encode(XMLWriter writer) throws ApplicationError {
		getTradeRoutes();
		if (_tradeRoutes != null) {
			for (int i = 0; i < _tradeRoutes.length; i++) {
				_tradeRoutes[i].toXML(writer);
			}
		}
	}

	public void remove(SolarSystem solarSystem) throws ApplicationError {
		getTradeRoutes();
		if (_tradeRoutes != null) {
			for (int i = 0; i < _tradeRoutes.length; i++) {
				if (_tradeRoutes[i].hasSolarSystem(solarSystem)) {
					tradeRoutes.remove(_tradeRoutes[i]);
				}
			}
		}
		_tradeRoutes = null;
	}

	@Override
	protected String getElementName() {
		return "trade-routes";
	}

	public int getQtyTradeRoutes() {
		return tradeRoutes.size();
	}

	public Planet [] getAllPlanetsEffectedByTradeRoutesIn(StellarObject so) {
		if (so instanceof SolarSystem) {
			return getAllPlanetsEffectedByTradeRoutesIn((SolarSystem)so);
		} else {
			return null;
		}
	}
	
	public Planet [] getAllPlanetsEffectedByTradeRoutesIn(SolarSystem solarSystem) {
        ArrayList list = null;
		getTradeRoutes();
		if (_tradeRoutes != null) {
			for (int i = 0; i < _tradeRoutes.length; i++) {
				if (_tradeRoutes[i].hasSolarSystem(solarSystem)) {
					Planet[] planets1 = _tradeRoutes[i].getSolarSystem1().getPlanets();
					for (int j = 0; j < planets1.length; j++) {
						if (planets1[j].getPopulation() > 0) {
							if (list == null) {
			                    list = new ArrayList();
			                }
			                list.add(planets1[j]);
						}
					}
					Planet[] planets2 = _tradeRoutes[i].getSolarSystem2().getPlanets();
					for (int j = 0; j < planets2.length; j++) {
						if (planets2[j].getPopulation() > 0) {
							if (list == null) {
			                    list = new ArrayList();
			                }
			                list.add(planets2[j]);
						}
					}
				}
			}
		}
        if (list == null) {
            return null;
        } else {
            return (Planet []) list .toArray(new Planet[list.size()]);
        }
	}

}
