/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.common.OwnedObject;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;

/**
 * Manages a player's trade routes.
 * Checks that all trade routes available for use by this player have been used
 * and if there are any trade routes not used assigns them to a new player.
 */
public abstract class Banker extends OwnedObject {

	public static final int CALCULATE_INTERVAL = 8;
	public static final int CALCULATE_START = 10;

	/**
	 * The date of the last time this player calculates its priorities.
	 */
	protected int dateOfLastCalculation;

	public int getDateOfLastCalculation() { return dateOfLastCalculation; }
	public void setDateOfLastCalculation(int v) { dateOfLastCalculation = v; }

	public Banker() {
	}

	public Banker(Player owner) throws ApplicationError {
		super(owner);
	}

	/**
	 * Checks that all trade routes available for use by this player have been used
     * and if there are any trade routes not used assigns them to a new player.
	 */
	public abstract void assignTradeRoutes() throws ApplicationError, ValidationError;

    /**
     * Convenience method for canFormTradeRoute(Player p1, Player p2, Player [] traders).
     */
    public static boolean canFormTradeRoute(Player p1, Player p2) throws ApplicationError {
        Player [] traders = MotEngine.instance.getRelations().getTradersWith(p1);
        return canFormTradeRoute(p1, p2, traders);
    }
    
    /**
     * Two player can form a trade route if
     *  - They are not the same player
     *  - Have established diplomatic relations
     *  - Are not at war
     *  - Do not already have a trade route
     *  - Have not reached maximum quantity of trade routes already
     * @param p1 Player wishing to form a trade route
     * @param p2 Player wishing to form a trade route
     * @param traders The current list of players trading with p1
     * @return True if and only if player p1 and p2 can form a trade routes at this point in time
     */
    public static boolean canFormTradeRoute(Player p1, Player p2, Player [] traders) throws ApplicationError {
        if (p1 == p2) {
            return false;
        }
        
        if (!p1.hasDiplomaticRelationsWith(p2)) {
            return false;
        }
        
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(p1, p2);
        if (!rels.canHaveTradeTreaty(rel)) {
            return false;
        }
        
        if (traders !=  null) {
            for (int j=0; j<traders.length; j++) {
                if (traders[j] == p2) {
                   return false;
                }
            }
        }
        
        if (rels.hasMaximumPlayerTradeRoutes(p1) || rels.hasMaximumPlayerTradeRoutes(p2)) {
        	return false;
        }
        
        return true;
    }

}