/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.util.ArrayList;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.DiplomaticStatements;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.players.strategists.StratagistEvent;
import mot.engine.players.strategists.StrategistEventManager;
import mot.engine.players.strategists.StrategistEventRequestTradeTreaty;

/**
 * Manages a player's trade routes.
 * Checks that all trade routes available for use by this player have been used
 * and if there are any trade routes not used assigns them to a new player.
 */
public class BankerBasic extends Banker {

	public BankerBasic() {
	}

	public BankerBasic(Player owner) throws ApplicationError {
		super(owner);
	}

	/**
	 * Checks that all trade routes available for use by this player have been used
     * and if there are any trade routes not used assigns them to a new player.
	 */
	public void assignTradeRoutes() throws ApplicationError, ValidationError {
		int date = MotEngine.getInstance().getDate();
		if ( (date - dateOfLastCalculation) < CALCULATE_INTERVAL) {
			return;
		}
		if (MotEngine.instance.getLevel() <= 1.0 && date < MotEngine.START_DATE+CALCULATE_START) {
			// Lets not bombard beginners with treaty requests.
			return;
		} 
        dateOfLastCalculation = date;

        Player owner = getOwner();
        int maxTradeRoutes = owner.getMaxPlayerTradeRoutes();
        Player []  traders = MotEngine.instance.getRelations().getTradersWith(owner);
        if (traders == null || traders.length < maxTradeRoutes) {
            try {
                int qtyTraders = (traders == null) ? 0 : traders.length;
                int qty = maxTradeRoutes - qtyTraders;
                createTradeRoute(qty);
            } catch (ValidationError e) {
                // Ignore - there is nobody we can create a trade route with!
            }
        }
    }
    
    /**
     * Creates a trade route between this player and the biggest player we have diplomatic 
     * contact with
     */
    private void createTradeRoute(int qty) throws ApplicationError, ValidationError {
        Player owner = getOwner();
        Player [] traders = MotEngine.instance.getRelations().getTradersWith(owner);
        Player [] all = MotEngine.instance.getPlayers().getPlayers();        
        
        // First create a list of all players that are eligible for us to form a
        // trade relationship with.
        ArrayList candidates = null;
        for (int i=0; i<all.length; i++) {
            if (!Banker.canFormTradeRoute(getOwner(), all[i], traders)) {
                continue;
            }
            if (owner instanceof ComputerPlayer) {
            	ComputerPlayer cp = (ComputerPlayer)owner;
				StrategistEventManager em = cp.getStratagist().getEventManagerNotNull();
				StratagistEvent lastTradeRequest = em.getLastDiplomaticEvent(all[i], StrategistEventRequestTradeTreaty.class);
				if (lastTradeRequest != null) {
					continue;
				}
            }
            if (candidates == null) candidates = new ArrayList();
            candidates.add(all[i]);
        }
        
        
        while (qty > 0) {
        	if (candidates == null || candidates.size() == 0) {
        		Logger.logDebug("BankerBasic.createTradeRoute() No eligable players to create trade route with");
        		return;
        	}
        	
            Player player = (Player) Utils.random(candidates);
            
            if (Logger.isDebug()) {
                Logger.logDebug("BankerBasic.createTradeRoute() request trade route between "+owner.getName()+" and "+player.getName());
            }
            
            PlayerCharacter pc = owner.getPlayerCharacter();
            String msg = pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_TRADE);
            player.diplomaticRequest(getOwner(), DiplomaticProposal.getTrade(), msg);

        	qty--;
        	candidates.remove(player);
        }

    }

    public String getElementName() {
        return "banker-basic";
    }

}