/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.PlayerColor;
import mot.common.designs.Orbital;
import mot.common.lang.L;
import mot.common.leaders.GeneralDetails;
import mot.common.leaders.GovenorDetails;
import mot.common.morals.MoralChoice;
import mot.common.technology.Biotechnology;
import mot.common.technology.Module;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattlePlanet;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftOrbitFactory;
import mot.engine.crafts.CraftOrbitFactoryBasic;
import mot.engine.crafts.Fleet;
import mot.engine.event.MessageEF;
import mot.engine.leaders.LeaderFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.TradeRoute;
import mot.engine.players.commanders.Commander;
import mot.engine.players.commanders.CommanderBasic;
import mot.engine.players.generals.General;
import mot.engine.players.generals.GeneralBasic;
import mot.engine.players.mayor.Mayor;
import mot.engine.players.mayor.MayorBasic;
import mot.engine.players.strategists.Stratagist;
import mot.engine.players.strategists.StratagistBasic;
import mot.engine.players.strategists.StrategistResponse;
import mot.engine.relations.Relation;
import mot.engine.transports.Transport;

/**
 * This class is a Computer Player in the Game. A Computer player does not do much
 * its self, it contains;
 * A Single General - To assign tasks to crafts/fleets on the galaxy map.
 * A Single Commander - To assign tasks to crafts in battle.
 * A Single Mayor - To assign tasks to planets.
 * A Scientist - To determine what technologies to research.
 * A Stratagist - To determine who and when this player should go to war with and try to agree peace with.
 * A Banker - To manage the players trade routes.
 * Currently the a computer player only manages research projects (using a research script)
 * and event notifications (see mot.engine.event package for details).
 */
public class ComputerPlayer extends Player implements AutomatedPlayer {

	/**
	 * The priority manager for the player's crafts / fleets on the galaxy map.
     * Generals manage colonisation as well as defence / conquest.
	 */
	private General general;
    
	/**
	 * Commanders manage crafts during combat.
	 */
	private Commander commander;
    
    /**
     * Manages this players planets.
     */
    private Mayor mayor;

	/**
	 * Manages the players research.
	 */
	private Scientist scientist;
    
	/**
	 * The priority manager for the player's diplomacy.
	 */
	private Stratagist stratagist;
    
	/**
	 * Manages the players spies.
	 */
    private IntelligenceBasic intelligence;

	/**
	 * Manages the players spies.
	 */
    private BankerBasic banker;

    /**
     * Is this player isolationist at this point in time.
     */
    private boolean isolationist = false;
    
    /**
     * The quantity of spies this player has built
     */
    private int qtyBaseSpies = 0;
    private int qtyAdditionalSpies = 0;

	public ComputerPlayer() throws ApplicationError {
		this.general = new GeneralBasic(this);
        this.commander = new CommanderBasic(this);
        this.mayor = new MayorBasic(this);
        this.scientist = new ScientistBasic(this);
		this.stratagist = new StratagistBasic(this);
		this.intelligence = new IntelligenceBasic(this);
		this.banker = new BankerBasic(this);
	}

	public ComputerPlayer(String name, PlayerCharacter character, PlayerColor color, String alignment) throws ApplicationError {
        super(name, character, color, alignment);
		this.general = new GeneralBasic(this);
        this.commander = new CommanderBasic(this);
        this.mayor = new MayorBasic(this);
        this.scientist = new ScientistBasic(this);
		this.stratagist = new StratagistBasic(this);
		this.intelligence = new IntelligenceBasic(this);
		this.banker = new BankerBasic(this);
		
		stratagist.notifyEmpireEstablished();
	}


	public General getGeneral() { return general; }
	public Mayor getMayor() { return mayor; }
	public Commander getCommander() { return commander; }
	public Stratagist getStratagist() { return stratagist; }
	public Scientist getScientist() { return scientist; }
	public Intelligence getIntelligence() { return intelligence; }
	public Banker getBanker() { return banker; }

	public void setStratagist(Stratagist v) { stratagist = v; }

    /**
     * This is required by XMLDocumentHandler to load saved games.    
     */
    public void setGeneralBasic(GeneralBasic v) { general = v; }

    /**
     * This is required by XMLDocumentHandler to load saved games.    
     */
    public void setCommanderBasic(CommanderBasic v) { commander = v; }

    /**
     * This is required by XMLDocumentHandler to load saved games.    
     */
    public void setMayorBasic(MayorBasic v) { mayor = v; }

    /**
     * This is required by XMLDocumentHandler to load saved games.    
     */
    public void setScientistBasic(ScientistBasic v) { scientist = v; }

    /**
     * This is required by XMLDocumentHandler to load saved games.    
     */
    public void setStratagistBasic(StratagistBasic v) { stratagist = v; }

    /**
     * This is required by XMLDocumentHandler to load saved games.    
     */
    public void setIntelligenceBasic(IntelligenceBasic v) { intelligence = v; }


    /**
     * This is required by XMLDocumentHandler to load saved games.    
     */
    public void setBankerBasic(BankerBasic v) { banker = v; }

	public void endTurn() throws ValidationError, ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("ComputerPlayer.endTurn() player="+getName());
        }
        
        calculateQtySpies();

		stratagist.endTurn();
        intelligence.assignSpies();
        banker.assignTradeRoutes();
        scientist.assignResearchTreaty();
        general.endTurn();
        mayor.endTurn();

		// for each idle craft, get its next task.
		Craft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                Craft craft = crafts[i];
                if (craft.getNavigator() == null && craft.getFleet() == null) {
                    setNextCraftOrders(craft);
                }
            }
        }

		// for each idle fleet, get its next task.
		Fleet [] fleets = getFleet();
        if (fleets != null) {
            for (int i=0; i<fleets.length; i++) {
                Fleet fleet = fleets[i];
                if (fleet.getNavigator() == null) {
                    setNextFleetOrders(fleet);
                }
            }
        }

        Planet planets [] = getPlanets();
        if (planets != null) {
            for (int i=0; i<planets.length; i++) {
                Planet planet = planets[i];
                mayor.setPercents(planet);
            }
        }
        
		super.endTurn();

        // assign any spies to tasks that have not already 
        // been assigned.
        intelligence.assignSpies();
	}

	public void calculateQtySpies() throws ApplicationError {
		long pop = getTotalPopulation();
        int minQtySpies = (int)((pop / 51));
        if (qtyBaseSpies < minQtySpies) qtyBaseSpies = minQtySpies;
	}

	/**
	 * Sets the next technology to be researched.
	 */
	public void notifyNoResearchProject(boolean updateTechsToResearch) throws ApplicationError {
		setResearchProject( scientist.getNextResearchProject(getLastDiscovery(), this) );
	}

	/**
	 * Returns the next craft to build. This may be a colony ship or a war ship.
	 * @throws ValidationError 
	 */
	public void notifyNoProject(Planet planet, String reason) throws ApplicationError, ValidationError {
        mayor.notifyNoProject(planet);
	}

	public void notifyEcologyFinished(Planet planet) throws ApplicationError, ValidationError {
        mayor.setPercents(planet);
	}

	public void notifyFactoriesFinished(Planet planet) throws ApplicationError, ValidationError {
        mayor.setPercents(planet);
	}

	public void notifyTechnologyDiscovered(Technology tech, boolean fromSalvage) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("ComputerPlayer.notifyTechnologyDiscovered() discovered tech = "+tech.getName());
        }
		super.notifyTechnologyDiscovered(tech, fromSalvage);
		if (tech instanceof Biotechnology) {
			Biotechnology biotech = (Biotechnology)tech;
			if (biotech.getTerraformRating() != null) {
				if (Logger.isDebug()) {
                    Logger.logDebug("ComputerPlayer.notifyTechnologyDiscovered() set all planet percents to perform terraforming");
                }
				PlayerCharacter pc = getPlayerCharacter();
				double pEcology 	= pc.getPlayerEcologyPercent();
				double pDevelopment = pc.getPlayerDevelopmentPercent();
				double pResearch 	= pc.getPlayerResearchPercent();
				double pProduction 	= pc.getPlayerProductionPercent();
				Planet [] planets = getPlanets();
                if (planets != null) {
                    for (int i=0; i<planets.length; i++) {
                        planets[i].setPercents(pEcology, pDevelopment, pResearch, pProduction);
                    }
                }
			}
		} else if (tech instanceof Scanner) {
            Planet [] planets = getPlanets();
            if (planets != null) {
                for (int i=0; i<planets.length; i++) {
                    planets[i].setScanners(null);
                    planets[i].setScanners(new Scanners((Scanner)tech));
                }
            }
        }
	}

	public void notifyPlanetColonised(Planet planet)  throws ApplicationError {
		PlayerCharacter pc = getPlayerCharacter();
		planet.setPercents( pc.getPlayerEcologyPercent(),
							pc.getPlayerDevelopmentPercent(),
							pc.getPlayerResearchPercent(),
							pc.getPlayerProductionPercent() );
        super.notifyPlanetColonised(planet);
	}

	public void notifyColonyLost(Planet colony, Player agressor, boolean genocide) throws ApplicationError {
		super.notifyColonyLost(colony, agressor, genocide);
        stratagist.notifyColonyLost(colony, agressor, genocide);
        mayor.notifyColonyLost(colony, agressor);
    }

	public void notifyTradeRouteInvalid(TradeRoute tradeRoute) throws ApplicationError {
	}
	
	/**
     * Called when player has been caught spying. This event is sent to the perpetrator 
     * and the victim.
     * @param player The player caught spying
     * @param tech The technology stolen by the player
     */
	public void notifySpying(Player player, Technology tech) throws ApplicationError {
        if (player != this) {
            stratagist.notifySpying(player, tech);
        }
    }

	public void notifySpySuccess(Player player, Technology tech) throws ApplicationError {	}
	
	/**
     * Called when player has been caught sabotaging installations. This event is 
     * sent to the perpetrator and the victim.
     * @param planet The planet that has been sabotaged
     * @param player The player caught sabotaging
     */
	public void notifySabotage(Planet planet, Player player, int productionSabotaged, int factoriesSabotaged, int populationSabotaged)  throws ApplicationError {
        if (player != this) {
            stratagist.notifySabotage(player);
        }
    }
    
    /**
     * Called when troops (troop transports) are required to invade a planet.
     * @param planet The planet that troop transports are required to invade.
     */
    public void notifyNeedTroops(Planet planet) throws ApplicationError {
        mayor.notifyNeedTroops(planet);
    }

	/**
	 * Called when a craft has been built or is idle to get its next task.
	 */
	public void setNextCraftOrders(Craft craft) throws ApplicationError {
        if (craft.getFleet() == null) {
        	if (craft.getNavigator() == null && craft.getOrbit() == null) {
        		// Who edited the save file? This craft is broken - craft end turn will fix it.
        		return;
        	}
            general.assignTaskToCraft(craft);
        }
	}

	/**
	 * Assigns order to military craft.
	 * See General class for details.
	 */
	public void setNextFleetOrders(Fleet fleet) throws ApplicationError {
        general.assignTaskToFleet(fleet);
	}

	/**
	 * Assigns order to military craft.
	 * See General class for details.
	 */
	public void setNextBattleOrders(BattleCraft craft) throws ApplicationError {
		commander.assignTaskToCraft(craft);
	}
    
	/**
	 * Assigns order to planets in battle.
	 * See General class for details.
	 */
	public void setNextBattleOrders(BattlePlanet planet) throws ApplicationError {
		commander.assignTaskToPlanet(planet);
    }

	/**
	 * Gets bonuses applied to a player due to the level of the game.
	 * Currently only computer players get a bonus.
	 */
	public double getLevelBonus() {
		double level = MotEngine.instance.getLevel();
        if (level < 1) level = 1;
        return level;
	}

	public void notificationOfWar(Player aggressor, Player victim) throws ApplicationError {
        stratagist.notificationOfWar(aggressor, victim);
    }

    public void diplomaticRequestNotification(Player player, DiplomaticProposal proposal, String message) throws ApplicationError {
		StrategistResponse r = stratagist.diplomaticRequest(player, proposal); 
		if (Logger.isDebug()) {
            Logger.logDebug("ComputerPlayer.notifyDiplomaticRequest() player "+player.getName()+" requests "+proposal+" response="+r);
        }
        player.diplomaticResponse(this, proposal, r.isAccepted(), r.getResponse());
    }

    public void diplomaticResponseNotification(Player player, DiplomaticProposal proposal, boolean accepted, String message) throws ApplicationError {
        stratagist.notifyDiplomaticResponse(player, proposal, accepted);
    }
    
	/**
	 * Called when this player is being offered a leader. To accept this leader return true
     * otherwise return false.
	 */
	public void offerGeneralLeaderTo(GeneralDetails leader) throws ApplicationError {
        LeaderFactory.addGeneralLeaderTo(this, leader);
    }
    
	/**
	 * Called when this player is being offered a leader. To accept this leader return true
     * otherwise return false.
	 */
	public void offerGovenorLeaderTo(GovenorDetails leader) throws ApplicationError {
        LeaderFactory.addGovenorLeaderTo(this, leader);
    }
    
	/**
	 * Accepts the recommendation of the Moral Policy Advisor if the moral choice is listed
	 * as acceptable for this player in its character.
	 */
	public void recommendMoralChoice(MoralChoice mc) throws ApplicationError {
		String [] yesList = getPlayerCharacter().getAcceptableMoralChoices();
		if ( Comp.in(mc.getName(), yesList) ) {
			add(mc);
			MotEngine.instance.getProxyManager().broadcast(new MessageEF(L.PLAYER_+getName()+L.HAS_ADOPTED_MORAL_POLICY+mc.getName()));
		}
	}

	/**
	 * Called when another a craft has arrived at an enemies solar system. It must return
     * which planet to attack in the solar system.
     * It returns the planet with the highest population.
	 */
    public Planet getPlanetToAttack(Planet [] planets) throws ApplicationError {
		return general.getPlanetToAttack(planets);
    }

    /**
     * Called when a Space Station has been built to find out which planet the owner would
     * like to deploy the station to.
     */
    public Planet getPlanetToDeploySpaceStation(Transport transport, Orbital station) throws ApplicationError {
        return general.getBestPlanetCanDeployMiningStation(transport);
    }

    /**
     * Called when a Space Station has been built to find out which planet the owner would
     * like to deploy the station to.
     */
    public StellarObject getStellarObjectToDeployOutpost(Transport transport, Orbital station) throws ApplicationError {
        return general.getBestStellarObjectToDeployOutpost();
    }

    /**
     * Called when a Module has been built to find out which space station the owner would
     * like to add the module to.
     * 
     * Currently computer players only build modules on planets that own a mining station
     * so modules built by a planet should always goto that planets mining station.
     */
    public Planet getStationToAddModule(Transport transport, Planet origin, Module module) throws ApplicationError {
        if (origin != null && origin.getMiningStation() != null) {
            Planet destination = origin.getMiningStation().getPlanet();
            return destination;
        } else {
            return MotEngine.instance.getPlanets().getClosestPlanetWithMiningStation(transport.getLocation(), transport.getOwner(), module);
        }
    }
    
    private CraftOrbitFactory craftOrbitFactory = null;
    public CraftOrbitFactory getCraftOrbitFactory() {
    	if (craftOrbitFactory == null) {
    		craftOrbitFactory = new CraftOrbitFactoryBasic();
    	}
    	return craftOrbitFactory;
    }
    public void setCraftOrbitFactory(CraftOrbitFactory v) { craftOrbitFactory = v; }

	public Relation getRelationshipWith(Player other) throws ApplicationError {
		Relation rel = super.getRelationshipWith(other);
		if (rel == null) return null;
		Relation r = new Relation(rel);
		r.setValue(stratagist.getRating(other, true));
		return r;
	}

	public boolean isComputerPlayer() {
		return true;
	}

    public void surrendetTo(Player other) throws ApplicationError {
    	if (general != null) {
    		general.removeAllTasks();
    	}
    	super.surrendetTo(other);
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		if (general != null) {
			general.toXML(writer);
		}
		if (stratagist != null) {
			stratagist.toXML(writer);
		}
        if (intelligence != null) {
            intelligence.toXML(writer);
        }
        writer.addElement("isolationist", isolationist);
        writer.addElement("qty-base-spies", qtyBaseSpies);
        writer.addElement("qty-additional-spies", qtyAdditionalSpies);
        
	}

	protected String getElementName() { return "computer-player"; }

	public boolean isIsolationist() {
		return isolationist;
	}

	public void setIsolationist(boolean isolationist) {
		this.isolationist = isolationist;
	}

    public void setQtyBaseSpies(int v) throws ApplicationError {
        qtyBaseSpies = v;
    }
	
    public int getQtyBaseSpies() throws ApplicationError {
        return qtyBaseSpies;
    }
	
	public void addQtyAditionalSpies(int qty) {
		qtyAdditionalSpies += qty;
	}
	
	public void setQtyAdditionalSpies(int qtyAdditionalSpies) {
		this.qtyAdditionalSpies = qtyAdditionalSpies;
	}
	
    public int getQtyAdditionalSpies() throws ApplicationError {
    	return qtyAdditionalSpies;
    }
    
    public int getQtyBuiltSpies() throws ApplicationError {
    	return getQtyBaseSpies() + qtyAdditionalSpies;
    }

}