/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.bean.NameIntPair;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This maintains a record of all crafts build by this player through the game.
 * For each hull this player has used to design a craft it holds the quantity of
 * crafts that have been designed using that hull.
 * 
 * New craft designs usually suffer from build problems. One a player has built a
 * few craft of a particular design them quality improves.
 * 
 * This class calculates the build quality for each craft built by the owner. The
 * initial build quality is MIN_BUILD_QUALITY. Each time the player builds a new 
 * craft the build quality is increased by BUILD_QUALITY_CHANGE_PER_CRAFT_BUILT until
 * MAX_BUILD_QUALITY is reached.
 * 
 * So build quality is a value between MIN_BUILD_QUALITY and MAX_BUILD_QUALITY where 
 * MAX_BUILD_QUALITY is a perfect craft.
 * 
 * NOTE - It actually stores the quantity of craft built using a particular hull. This
 * is a course grained record of experience building craft designs which I think is right.
 */
public class CraftBuildExperience extends XMLBean {

	public static final double MIN_BUILD_QUALITY = 0.7; 
	public static final double BUILD_QUALITY_CHANGE_PER_CRAFT_BUILT = 0.1; 
	public static final double MAX_BUILD_QUALITY = 1.0; 

	/**
	 * This is a list of hull name / quantity of times that hull has been built.
	 * This enable us to find the quantity of times any arbitary hull has been
	 * use to build a craft.
	 * 
	 * This is a list of NameIntPair objects.
	 */
	private ArrayList designList;

	public void addDesign(NameIntPair design) {
		setDesign(design);
	}

	public void setDesign(NameIntPair design) {
		if (designList == null) designList = new ArrayList();
		designList.add(design);
	}
	
	/**
	 * @return The quantity of hulls of name built by this player
	 */
	public NameIntPair getDesignExperience(String designName) {
		if (designList != null) {
			Iterator i = designList.iterator();
			while (i.hasNext()) {
				NameIntPair nip = (NameIntPair)i.next();
				if (Comp.same(designName, nip.getName())) {
					return nip;
				}
			}
		}
		return null;
	}

	/**
	 * @return The built quality for player owner if he was building a craft using hull
	 * hullName.
	 */
	public double getBuildQuality(Player owner, String designName) {
		NameIntPair nip = getDesignExperience(designName);
		double r;
		if (nip == null) {
			r = MIN_BUILD_QUALITY; 
		} else {
			r = MIN_BUILD_QUALITY + BUILD_QUALITY_CHANGE_PER_CRAFT_BUILT * nip.getValue();
		}
		if (owner.getFactories() != null) {
			double factoryBuildQuality = owner.getFactories().getBuildQuality();
			r += factoryBuildQuality;
		}
		if (r > MAX_BUILD_QUALITY) r = MAX_BUILD_QUALITY;
		if (r < MIN_BUILD_QUALITY) r = MIN_BUILD_QUALITY;
		return r;
	}
	
	/**
	 * Add a one to the quantity of crafts built using the hull name passed in.
	 */
	public void addDesignExperience(String designName) {
		NameIntPair design = getDesignExperience(designName);
		if (design == null) {
			design = new NameIntPair(designName, 1);
			addDesign(design);
		} else {
			design.setValue(design.getValue()+1);
		}
	}
		
	protected void encode(XMLWriter writer) throws ApplicationError {
		if (designList != null) {
			Iterator i = designList.iterator();
			while (i.hasNext()) {
				NameIntPair nip = (NameIntPair)i.next();
				nip.toXML(writer, "design");
			}
		}
	}
	
	protected String getElementName() { return "craft-build-experience"; }

	
}
