/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.spies.Spies;
import mot.engine.spies.SpyRoleDefend;
import mot.engine.spies.SpyRoleEspionage;
import mot.engine.spies.SpyRoleSabotage;

/**
 * Manages a player's spies.
 * It assigns a players spies to defend the player as well as attack enemy players
 * or keep an eye on friendly players.
 */
public class IntelligenceBasic extends Intelligence {

	public IntelligenceBasic() {
	}

	public IntelligenceBasic(Player owner) throws ApplicationError {
		super(owner);
	}

	/**
	 * Checks that all spies are currently in use AND assigns a task to each spy based 
     * upon the players position.
     * 
     * This method allocates percentages to defend, attack the most hated player and attack the
     * largest player. The percentage for each is taken from the players character being the 
     * following properties;
     *  - spy-defend-percent
     *  - spy-attack-most-hated-percent
     *  - spy-attack-largest-percent
     *  
     *  If at war with most hated player then the spy-attack-largest-percent will be used for sabotage
     *  on the most hated player instead of spying on the largest player.
	 */
	public void assignSpies() throws ApplicationError, ValidationError {
		if ( (MotEngine.getInstance().getDate() - dateOfLastCalculation) <= CALCULATE_INTERVAL) {
			return;
		}
        dateOfLastCalculation = MotEngine.getInstance().getDate();
        
        ComputerPlayer owner = (ComputerPlayer) getOwner();
        Spies spies = MotEngine.getInstance().getSpiesNotNull();
        spies.removeSpies(owner);

        int qty = owner.getQtySpies();
        double percentDefend = owner.getPlayerCharacter().getSpyDefendPercent();
        double spyAttackMostHatedPercent = owner.getPlayerCharacter().getSpyAttackMostHatedPercent();
        double spyAttackLargestPercent = owner.getPlayerCharacter().getSpyAttackLargestPercent();
        
        Player mostHated = owner.getStratagist().getMostHatedPlayer().player;
        if (mostHated == null) {
            if (Logger.isDebug()) {
                Logger.logDebug("IntelligenceBasic.assignSpies() Player "+owner.getName()+" No Other players => defend="+qty);
            }
            spies.addSpyRole(new SpyRoleDefend(owner, qty));
            return;
        }

        int qtyDefend = (int)(qty*percentDefend);
        int qtyMostHated = (int)(qty*spyAttackMostHatedPercent);
        int qtyLargest = (int)(qty*spyAttackLargestPercent);
        
        if (qty > ( qtyDefend+qtyMostHated+qtyLargest )) {
            qtyDefend++;
        }
        if (qty > ( qtyDefend+qtyMostHated+qtyLargest )) {
            qtyMostHated++;
        }
        
        Player largestPlayer = MotEngine.instance.getPlayers().getLargestPlayer();
        boolean atWarWithMostHated = MotEngine.instance.getRelations().isWar(owner, mostHated);
        if (largestPlayer == owner) {
            largestPlayer = mostHated;
        }
        if (largestPlayer == mostHated && !atWarWithMostHated) {
            qtyMostHated += qtyLargest;
            qtyLargest = 0;
        }
        
        if (Logger.isDebug()) {
            Logger.logDebug("IntelligenceBasic.assignSpies() Player "+owner.getName()+" defend="+qtyDefend+" mostHated="+qtyMostHated+" largest="+qtyLargest);
        }

        spies.addSpyRole(new SpyRoleDefend(owner, qtyDefend));
        spies.addSpyRole(new SpyRoleEspionage(owner, mostHated, qtyMostHated));
        if (qtyLargest > 0) {
            if (atWarWithMostHated) {
                spies.addSpyRole(new SpyRoleSabotage(owner, mostHated, qtyLargest));
            } else {
                spies.addSpyRole(new SpyRoleEspionage(owner, largestPlayer, qtyLargest));
            }
        }
    }

    public String getElementName() {
        return "intelligence-basic";
    }

}