/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.util.ArrayList;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Nebula;
import mot.common.bean.PlayerAssets;
import mot.common.bean.PlayerColor;
import mot.common.designs.Orbital;
import mot.common.gifts.AncientGift;
import mot.common.gifts.AncientGiftsFactory;
import mot.common.lang.L;
import mot.common.leaders.GeneralDetails;
import mot.common.leaders.GovenorDetails;
import mot.common.morals.MoralChoice;
import mot.common.technology.Module;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.vo.PlayerTechnologiesVO;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattlePlanet;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.event.MessageEF;
import mot.engine.event.PrizeGiftEF;
import mot.engine.planets.Planet;
import mot.engine.planets.TradeRoute;
import mot.engine.players.commanders.Commander;
import mot.engine.players.commanders.CommanderBasic;
import mot.engine.players.generals.CraftTaskMaraud;
import mot.engine.players.generals.CraftTaskMaraudCraft;
import mot.engine.players.generals.General;
import mot.engine.players.generals.GeneralMonster;
import mot.engine.transports.Transport;
import mot.test.TestComputerPlayer;


/**
 * This class is a holder for the monster player. The monster player is a special player
 * contained as an attribute in the engine - so all players share the same monster player.
 * The monster player hates everyone and is always at war with everyone.
 * The monster player is also all ancient players - that controls the crafts that guards 
 * planets of the ancients.
 * 
 * The monster player does not have any planets or solar systems but does govern crafts.
 * 
 * There are no relationships defined for the monster player but the relations have hard 
 * coded knowledge of the monster player. 
 * 
 * This class is a Computer Player in the Game. A Computer player does not do much
 * its self, it contains;
 * A Single General - To assign tasks to military crafts/fleets on the galaxy map.
 * A Single Commander - To assign tasks to crafts in battle.
*/

public class MonsterPlayer extends Player implements AutomatedPlayer {

	private static final int MIN_INTERVAL_BETWEEN_DEFEND_FLEETS = 50;

	/**
	 * The priority manager for the player's crafts / fleets on the galaxy map.
     * Generals manage colonisation as well as defence / conquest.
	 */
	private GeneralMonster general;

	/**
	 * Commanders manage crafts during combat.
	 */
	private Commander commander;
    
    /**
     * This is a list of all the ancient races controlled by the monster player.
     */
    private ArrayList ancients = new ArrayList();
    
    /**
     * This is a list of all the ancient players in the game that is dynamically 
     * generated from ancients array list.
     */
    private transient AncientPlayer [] ancientsArray;
    
    /**
     * This is the date the next ancient attack is scheduled.
     * On this date the endTurn() method will dispatch a fleet of ancients to attack a 
     * random player.
     */
    private int nextAttackDate;
    
    /**
     * This is the date of the next scheduled Kaan Attack
     */
    private int nextKaanAttackDate;

    private static final int GIFT_1_DATE = 3075;
    private static final int GIFT_2_DATE = 3250;

	public MonsterPlayer() {
		this.general = new GeneralMonster(this);
        this.commander = new CommanderBasic(this);
    }
    
	public MonsterPlayer(String name, PlayerCharacter character, PlayerColor color) throws ApplicationError {
        super(name, character, color, null);
		this.general = new GeneralMonster(this);
        this.commander = new CommanderBasic(this);
        
        // All Players MUST have an beam weapon - this is used to calculate ground combat ability plus other things.
        addWeapon(Technologies.getWeapons().getWeapon("ancient"));
	}


	public General getGeneral() { return general; }
	public Commander getCommander() { return commander; }

    public void setGeneral(GeneralMonster general) { this.general = general; }
    public void setGeneralMonster(GeneralMonster general) { this.general = general; }
    public void setCommander(Commander commander) { this.commander = commander;  }

	public void endTurn() throws ValidationError, ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("MonsterPlayer.endTurn()");
        }
		
        general.endTurn();
        
        Craft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                if (crafts[i].getFleet() == null) {
                    general.assignTaskToCraft(crafts[i]);
                }
            }
        }
        Fleet [] fleets = getFleet();
        if (fleets != null) {
            for (int i=0; i<fleets.length; i++) {
                general.assignTaskToFleet(fleets[i]);
            }
        }
        int date = MotEngine.instance.getDate();
        if (date >= nextAttackDate) {
            dispatchAttack();
            setNextAttackDate();
        }
        
        if (date == GIFT_1_DATE) {
        	doAncientsGifts(1);
        } else if (date == GIFT_2_DATE) {
        	doAncientsGifts(2);
        }
        
        if (date > LOWEST_KAAN_ATTACK_DATE && date >= nextKaanAttackDate && MotEngine.instance.isSpaceKaanEnabled()) {
        	Nebula[] nebulae = MotEngine.instance.getStellarObjectsNotNull().getNebulae();
        	if (nebulae != null && nebulae.length > 0) {
        		Nebula nebula = (Nebula) Utils.random(nebulae);
        		dispatchKaan(nebula, null);
        	}
        	setNextKaanAttackDate();
        }
	}

	private void doAncientsGifts(int round) throws ApplicationError {
		AncientPlayer[] aa = getAncients();
		if (aa == null) {
			Logger.logWarning("MonsterPlayer.doAncientsGifts() - NO ANCIENT GIFTS DEFINED");
		} else {
			for (int i = 0; i < aa.length; i++) {
				String ancientName = aa[i].getName();
				AncientGift gift = AncientGiftsFactory.selectRandomGift(round, ancientName);
				if (gift == null) {
					Logger.logWarning("MonsterPlayer.doAncientsGifts() - NO ANCIENT GIFT To Give for round "+round+" name "+ancientName);
				} else {
					Player[] pa = MotEngine.instance.getPlayersNotNull().getPlayersWithAlignment(ancientName);
					if (pa != null) {
						for (int j = 0; j < pa.length; j++) {
							doAncientGift(gift, pa[j]);
						}
					}
				}
			}
		}
	}
    
    private void doAncientGift(AncientGift gift, Player player) throws ApplicationError {
		if (Logger.isDebug()) {
			Logger.logDebug("MonsterPlayer.doAncientGift "+player.getName()+" gift="+gift.getName());
		}
    	String description = gift.getDescription();
    	if (description == null && gift.getTechnology() != null) description = gift.getTechnology().getFullName();
		String text = L.ANCIENT_ + gift.getAncientName() + L.HAS_GIVEN_ALL_PLAYERS_ALIGNED_TO_THEM + description;
    	String techName = gift.getTechnology() == null ? null : gift.getTechnology().getFullName();
    	PrizeGiftEF event = new PrizeGiftEF(false, gift.getAncientName(), techName, text);
		if (gift.getOption() != null) {
            player.getPlayerCharacter().increasesBonusesFromOptions(gift.getOption());
		} else if (gift.getTechnology() != null) {
            player.addTechnology(gift.getTechnology());
            event.addStateUpdate(new PlayerTechnologiesVO(player.getName(), player));
		} else {
			Logger.logWarning("MonsterPlayer.doAncientsGift() - gift "+gift+" does not have anything to give");
		}
		MotEngine.instance.broadcast(event, player);
	}

	public void setAncientPlayer(AncientPlayer ap) {
        addAncient(ap);
    }
    
    public void addAncient(AncientPlayer ap) {
        ancients.add(ap);
        ancientsArray = null;
    }
    
    public AncientPlayer [] getAncients() {
        if (ancientsArray == null) {
            ancientsArray = (AncientPlayer[]) ancients.toArray(new AncientPlayer[ancients.size()]);
        }
        return ancientsArray;
    }

    public AncientPlayer getAncientPlayer(String name) {
        AncientPlayer[] aa = getAncients();
        if (aa != null) {
            for (int i=0; i<aa.length; i++) {
                if (Comp.same(name, aa[i].getName())) {
                    return aa[i];
                }
            }
        }
        return null;
    }


    /**
     * Sends a fleet of ancient crafts to maraud the aggressor if the aggressor is of 
     * the same alignment as the victim and the ancients of that alignment have not 
     * been defeated.
     * @param agressor The player who declared war.
     * @param victim The player who has had war declared on himself.
     * @throws mot.common.util.ApplicationError
     */
	public void notifyWarDeclared(Player agressor, Player victim) throws ApplicationError {
		if (agressor.isComputerPlayer() && !(agressor instanceof TestComputerPlayer)) {
			// To remove this condition the stratagist would have to be improved (its too hard).
			return;
		}
        if ( agressor.getAlignment() != null && Comp.same(agressor.getAlignment(), victim.getAlignment()) ) {
            AncientPlayer ap = getAncientPlayer(agressor.getAlignment());
            if (ap == null) {
                // Assume the ancient has been destroyed.
                return;
            }
            CraftTaskMaraud task = general.createMaraudTaskFor(constructWrathFleet(ap), agressor, 2);
            if (!task.complete()) {
            	MotEngine.instance.broadcast(new MessageEF(L.ANCIENTS_ATTACK_MSG_WAR), agressor);
            } else {
            	general.remove(task);
            }
        }
	}
    
    private static final long VERY_SMALL_DATE = 3050;
    private static final long SMALL_DATE = 3100;
    private static final long MEDIUM_DATE = 3200;
    private static final long LARGE_DATE = 3300;
    
    private Fleet constructWrathFleet(AncientPlayer ap) throws ApplicationError {
        int date = MotEngine.instance.getDate();
        int dateDelay = 0;
        if (MotEngine.instance.isLevelEasy()) dateDelay = 200;
        if (MotEngine.instance.isLevelNormal()) dateDelay = 100;
        Orbit orbit = new Orbit(ap.getHome(), null);
        Craft craft1 = null;
        Craft craft2 = null;
        Craft craft3 = null;
        Craft craft4 = null;
        Craft craft5 = null;
        if (date < dateDelay + VERY_SMALL_DATE) {
            craft1 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
        } else if (date < dateDelay + SMALL_DATE) {
            craft1 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
            craft2 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
        } else if (date < dateDelay + MEDIUM_DATE) {
            craft1 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
            craft2 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
            craft3 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
        } else if (date < dateDelay + LARGE_DATE) {
            craft1 = CraftFactory.create("medium_"+ap.getName(), this, this, orbit, false, false);
            craft2 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
            craft3 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
            craft4 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
        } else {
            craft1 = CraftFactory.create("medium_"+ap.getName(), this, this, orbit, false, false);
            craft2 = CraftFactory.create("medium_"+ap.getName(), this, this, orbit, false, false);
            craft3 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
            craft4 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
            craft5 = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
        }
        Fleet fleet = FleetFactory.create(craft1);
        if (craft2 != null) fleet.add(craft2);
        if (craft3 != null) fleet.add(craft3);
        if (craft4 != null) fleet.add(craft4);
        if (craft5 != null) fleet.add(craft5);
        
        return fleet;
    }
    
	public void notifyNoResearchProject(boolean updateTechsToResearch) throws ApplicationError {
	}
	public void notifyNoProject(Planet planet, String reason) throws ApplicationError {
	}
	public void notifyEcologyFinished(Planet planet) throws ValidationError {
	}
	public void notifyFactoriesFinished(Planet planet) throws ValidationError {
	}
	public void notifyTradeRouteInvalid(TradeRoute tradeRoute) throws ApplicationError {
	}
	public void notifySpying(Player player, Technology tech) throws ApplicationError {
    }
	public void notifySpySuccess(Player player, Technology tech) throws ApplicationError {	
	}
	public void notifySabotage(Planet planet, Player player, int productionSabotaged, int factoriesSabotaged, int populationSabotaged)  throws ApplicationError {
    }
    public void notifyNeedTroops(Planet planet) throws ApplicationError {
    }
	public void setNextCraftOrders(Craft craft) throws ApplicationError {
        general.assignTaskToCraft(craft);
	}
	public void setNextFleetOrders(Fleet fleet) throws ApplicationError {
        general.assignTaskToFleet(fleet);
	}

	/**
	 * Assigns order to military craft.
	 * See General class for details.
	 */
	public void setNextBattleOrders(BattleCraft craft) throws ApplicationError {
		commander.assignTaskToCraft(craft);
	}
    
	public void setNextBattleOrders(BattlePlanet planet) throws ApplicationError {
    }
    public int getQualityScore(Planet p) throws ApplicationError {
        return 0;
    }

	protected void diplomaticRequestNotification(Player player, DiplomaticProposal proposal, String message) throws ApplicationError { }
	protected void diplomaticResponseNotification(Player player, DiplomaticProposal proposal, boolean accepted, String message) throws ApplicationError {	}
	
    public Planet getPlanetToAttack(Planet [] planets) throws ApplicationError {
		return general.getPlanetToAttack(planets);
    }
    public Planet getPlanetToDeploySpaceStation(Transport transport, Orbital station) throws ApplicationError {
        return null;
    }
    public StellarObject getStellarObjectToDeployOutpost(Transport transport, Orbital station) throws ApplicationError {
        return null;
    }
    public Planet getStationToAddModule(Transport transport, Planet origin, Module module) throws ApplicationError {
        return null;
    }
    public void giveFrom(Player player, PlayerAssets offered) {
    }
    public int getQtyBuiltSpies() { return 0; }

	public void offerGeneralLeaderTo(GeneralDetails leader) throws ApplicationError {    }
	public void offerGovenorLeaderTo(GovenorDetails leader) throws ApplicationError {    }

	public void recommendMoralChoice(MoralChoice mc) throws ApplicationError {	}
	
	public boolean isComputerPlayer() {
		return true;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		if (general != null) {
			general.toXML(writer);
		}
        AncientPlayer[] aa = getAncients();
        if (aa != null) {
            for (int i=0; i<aa.length; i++) {
                aa[i].toXML(writer);
            }
        }
        writer.addElement("next-attack-date", nextAttackDate);
        writer.addElement("next-kaan-attack-date", nextKaanAttackDate);
        super.encode(writer);
	}

	protected String getElementName() { return "monster-player"; }

    /**
     * @param victim 
     * @return The ancient race that is used to attack the victim (must not be aligned to the victim).
     * @throws mot.common.util.ApplicationError
     */
    private AncientPlayer getAncientAttacker(Player victim) throws ApplicationError {
    	AncientPlayer[] apa = getAncients();
    	if (apa == null || apa.length == 0) return null;
    	int index = (apa.length == 1) ? 0 : Utils.random(apa.length);
    	if (ancientCanAttack(apa[index], victim)) {
    		return apa[index];
    	} else {
    		index = 0;
    		while (index < apa.length) {
	        	if (ancientCanAttack(apa[index], victim)) {
	        		return apa[index];
	        	}
	        	index++;
    		}
    	}
    	return null;
    }

    private boolean ancientCanAttack(AncientPlayer ap, Player victim) throws ApplicationError {
		if (!Comp.same(ap.getName(), victim.getAlignment())) {
			StellarObject home = ap.getHome();
			if (home != null && !home.hasOwner()) {
				int qtyAncientGuradingCraft = getQtyCraftsAt(home);
				return qtyAncientGuradingCraft > 0; 
			}
		}
		return false;
	}

	/**
     * Creates a fleet of small ancient crafts to attack an random player in the game.
     * If the game level is greater than 1 then it is more likely to be a user player.
     */
    private void dispatchAttack() throws ApplicationError {
        int date = MotEngine.instance.getDate();
        int lowestAttackDate = getLowestAttackDate();
        if (date < lowestAttackDate) {
            return;
        }
        Player victim = selectVictim();
        if (victim == null) {
            return;
        }
        AncientPlayer ap = getAncientAttacker(victim);
        if (ap == null) {
            // Assume the ancient has been destroyed.
            return;
        }
        Orbit orbit= new Orbit(ap.getHome(), null);
        Craft craft = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
        Fleet fleet = FleetFactory.create(craft);
        int qtyCraftsfactor =  MotEngine.instance.isAHardLevel() ? 60 : 100;
        int qty = (int)((date - lowestAttackDate) / qtyCraftsfactor);
        while (qty > 0) {
            Craft craftx = CraftFactory.create("small_"+ap.getName(), this, this, orbit, false, false);
            fleet.add(craftx);
            qty--;
        }
		if (Logger.isDebug()) {
            Logger.logDebug("MonsterPlayer.dispatchAttack() attack "+victim.getName()+" fleet="+fleet.getName());
        }
        general.createMaraudTaskFor(fleet, victim, 1);
    }
    
    /**
     * Create a new fleet of a single space craft (the nebula kaan) with task of marauding the galaxy.
     * Initial target will be the first player found that IS NOT in the excludeAsInitialTarget list.
     */
    public void dispatchKaan(Nebula nebula, Player[] excludeAsInitialTarget) throws ApplicationError {
    	Player initialTarget = null;
    	Player[] players = MotEngine.instance.getPlayersNotNull().getPlayers();
    	if (players != null) {
    		for (int i = 0; i < players.length && initialTarget == null; i++) {
				if (!Comp.in(players[i], excludeAsInitialTarget)) {
					initialTarget = players[i];
				}
			}
    	}
		Craft craft = CraftFactory.create(KAAN_NAME, this, this, null, false, false);
        craft.setLocationInSpace(nebula.getLocation(), null);
        CraftTaskMaraudCraft task = general.createMaraudCraftTaskFor(craft, initialTarget);
        task.assignTaskToFleet();
        MotEngine.instance.broadcast(new MessageEF(L.KAAN_ATTACK_MSG));
		if (Logger.isDebug()) {
            Logger.logDebug("MonsterPlayer.dispatchCaan() attack "+initialTarget+" fleet="+craft.getNavigator());
        }
    }
    
    /**
     * Selects a player for the ancient players to attack. The player selected is;
     * - 50% The player with the highest power rating
     * - 50% A random player.
     * @return A random player
     */
    private Player selectVictim() throws ApplicationError {
        Player [] players = MotEngine.instance.getPlayersNotNull().getPlayers();
        if (players == null || players.length == 0) return null;
        if (Utils.random(2) == 0) {
            return MotEngine.instance.getPlayers().getLargestPlayer();
        } else {
            return players[Utils.random(players.length)];
        }
    }
    
    private static final int LOWEST_KAAN_ATTACK_DATE = 3125;
    private static final int KAAN_ATTACK_DATE_INTERVAL_RANGE = 50;
    private static final int KAAN_ATTACK_DATE_INTERVAL_MIN = 25;
    
    private static final int LOWEST_ATTACK_DATE_EASY_LEVEL = 3150;
    private static final int LOWEST_ATTACK_DATE_HARD_LEVEL = 3100;
    private static final int ATTACK_DATE_INTERVAL_RANGE = 25;
    private static final int ATTACK_DATE_INTERVAL_MIN = 15;

	private static final String KAAN_NAME = "kaan";
    
	private static int getLowestAttackDate() {
		if (MotEngine.instance.isAHardLevel()) {
			return LOWEST_ATTACK_DATE_HARD_LEVEL;
		} else {
			return LOWEST_ATTACK_DATE_EASY_LEVEL;
		}
	}
	
    /**
     * Schedules the attack date based upon the current date.
     */
    private void setNextAttackDate() {
        int date = MotEngine.instance.getDate();
        int lowestAttackDate = getLowestAttackDate();
        if (date < lowestAttackDate) {
            date = lowestAttackDate;
        }
        nextAttackDate = date + lowestAttackDate + Utils.random(ATTACK_DATE_INTERVAL_RANGE);
		if (Logger.isDebug()) {
            Logger.logDebug("MonsterPlayer.setNextAttackDate() date = "+nextAttackDate);
        }
    }

    public void __setNextKaanAttackDate() {  setNextKaanAttackDate();  }
    
    private void setNextKaanAttackDate() {
        int date = MotEngine.instance.getDate();
        if (date < LOWEST_KAAN_ATTACK_DATE) {
            date = LOWEST_KAAN_ATTACK_DATE;
        }
        nextKaanAttackDate = date + KAAN_ATTACK_DATE_INTERVAL_MIN + Utils.random(KAAN_ATTACK_DATE_INTERVAL_RANGE);
		if (Logger.isDebug()) {
            Logger.logDebug("MonsterPlayer.setNextKaanAttackDate() date = "+nextKaanAttackDate);
        }
    }

    public void setNextAttackDate(int nextAttackDate) {
        this.nextAttackDate = nextAttackDate;
    }


    public int getNextAttackDate() {
        return nextAttackDate;
    }

    /**
     * Called when any player in the game has lost a colony (via bombard or invade).
     * @param victim The player who has lost the colony
     * @param colony The planet that the colony that has been lost
     * @param agressor The player that destroyed the colony
     * @throws ApplicationError 
     */
	public void notifyMonsterColonyLost(Player victim, Planet colony, Player agressor) throws ApplicationError {
		if (victim != null && victim.getAlignment() != null && !MotEngine.instance.isLevelEasy()) {
			if (MotEngine.instance.getDate() - victim.getLastDateAncientsDefended() > MIN_INTERVAL_BETWEEN_DEFEND_FLEETS) {
	            AncientPlayer ap = getAncientPlayer(victim.getAlignment());
	            if (ap == null || ap.getHome() == null) {
	                // Assume the ancient has been destroyed.
	                return;
	            }
				int qtyAncientGuradingCraft = getQtyCraftsAt(ap.getHome());
				if (qtyAncientGuradingCraft == 0) {
	                // The ancient has been destroyed.
	                return;
				}
				victim.setLastDateAncientsDefended(MotEngine.instance.getDate());
	            general.createMaraudTaskFor(constructWrathFleet(ap), agressor, 1);
	            MotEngine.instance.broadcast(new MessageEF(L.ANCIENTS_ATTACK_MSG_LOSE_PLANET), agressor);
			}
		}
	}

	public int getNextKaanAttackDate() {
		return nextKaanAttackDate;
	}

	public void setNextKaanAttackDate(int nextKaanAttackDate) {
		this.nextKaanAttackDate = nextKaanAttackDate;
	}

	public int getQtyKaans() throws ApplicationError {
		Craft[] crafts = getCrafts();
		if (crafts == null || crafts.length == 0) return 0;
		int qty = 0;
		for (int i = 0; i < crafts.length; i++) {
			if (crafts[i].getName().startsWith(KAAN_NAME)) {
				qty++;
			}
		}
		return qty;
	}





}