/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.util.ArrayList;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.bean.NameList;
import mot.common.bean.NamedObject;
import mot.common.bean.Owner;
import mot.common.bean.PlayerAssets;
import mot.common.bean.PlayerCharacterBasic;
import mot.common.bean.PlayerColor;
import mot.common.bean.SalvageList;
import mot.common.bean.stats.Graph;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.WeaponDesign;
import mot.common.lang.L;
import mot.common.leaders.GeneralDetails;
import mot.common.leaders.GovenorDetails;
import mot.common.morals.MoralChoice;
import mot.common.technology.Biotechnologies;
import mot.common.technology.Biotechnology;
import mot.common.technology.Computer;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.Engines;
import mot.common.technology.Factories;
import mot.common.technology.Factory;
import mot.common.technology.GalacticLore;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Hull;
import mot.common.technology.Hulls;
import mot.common.technology.Humanities;
import mot.common.technology.Humanity;
import mot.common.technology.Laboratories;
import mot.common.technology.Laboratory;
import mot.common.technology.MiscTechnologies;
import mot.common.technology.MiscTechnology;
import mot.common.technology.Module;
import mot.common.technology.Modules;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyContainer;
import mot.common.technology.TechnologyList;
import mot.common.technology.TechnologyOwner;
import mot.common.technology.TechnologyTree;
import mot.common.technology.Theories;
import mot.common.technology.Theory;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.Battles;
import mot.engine.battles.MovingAutamaton;
import mot.engine.battles.WreckageCraft;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.OwnedObject;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectOrbital;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftOrbitFactory;
import mot.engine.crafts.Crafts;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.Navigator;
import mot.engine.event.FundsGoneEF;
import mot.engine.event.FundsLowEF;
import mot.engine.event.MessageEF;
import mot.engine.event.PlayerLoseEF;
import mot.engine.event.PlayerSurrenderEF;
import mot.engine.event.StartTurnEF;
import mot.engine.event.TooManySpiesEF;
import mot.engine.leaders.EngineGeneralLeader;
import mot.engine.leaders.EngineGovenorLeader;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.Project;
import mot.engine.planets.ProjectScanner;
import mot.engine.planets.ProjectShield;
import mot.engine.planets.ProjectWeapon;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRoute;
import mot.engine.players.mayor.CraftDesignerFighter;
import mot.engine.players.morals.MoralChoiceListenerList;
import mot.engine.players.morals.MoralPolicyAdvisor;
import mot.engine.players.morals.MoralPolicyAdvisorBasic;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.relations.Trader;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportManager;


/**
 * This is a player in the game.
 */
public abstract class Player extends NamedObject implements Trader, Owner, TechnologyOwner, TechnologyContainer {

	/**
	 * List of crafts owned by this player. Dynamically generated from Crafts.
	 */
	private transient Craft [] crafts;

	/**
	 * List of solar systems owned by this player. Dynamically generated from Planets.
	 */
	private transient SolarSystem [] solarSystems;

	/**
	 * List of stellar objects owned by this player. Dynamically generated from Planets.
	 */
	private transient StellarObject [] stellarObjects;

	/**
	 * List of planets owned by this player. Dynamically generated from Planets.
	 */
	private transient Planet [] planets;

	/**
	 * Factories/production facilities discovered by this player.
	 */
	private Factories factories;

	/**
	 * Research facilities discovered by this player.
	 */
	private Laboratories laboratories;

	/**
	 * The BioTechnology discovered by the player - Terreforming etc.
	 */
	private Biotechnologies biotechnologies;

	/**
	 * The hull types discovered by this player.
	 */
	private Hulls hulls;

	/**
	 * The computers mounted discovered by this player.
	 */
	private Computers computers;

	/**
	 * The weapons mounted discovered by this player.
	 */
	private Weapons weapons;

	/**
	 * The shields mounted discovered by this player.
	 */
	private Shields shields;

	/**
	 * The scanners mounted discovered by this player.
	 */
	private Scanners scanners;

	/**
	 * The transporters etc. discovered by this player.
	 */
	private Specials specials;

	/**
	 * The transporters etc. discovered by this player.
	 */
	private Engines engines;

    /**
     * The modules discovered by this player.
     */
    private Modules modules;

    /**
     * The modules discovered by this player.
     */
    private Humanities humanities;
    
	/**
	 * Theory subjects researched by this player.
	 */
	private Theories theories;

	/**
	 * Miscellaneous technologies researched by this player.
	 */
	private MiscTechnologies miscTechnologies;
	
	/**
	 * The bonuses this player inherently has.
	 */
	private PlayerCharacter character;

	/**
	 * The research project this user is undertaking at this point in time.
	 */
	private ResearchProject researchProject;

	/**
	 * The name of the last discovery made by this player
	 */
	private ResearchScriptOption lastDiscovery;
	
	/**
	 * This is the list of technologies available for research.
	 * This is not saved and is evaluated lazily and recalculated whenever this players technologies are modified. 
	 */
	private Technology [] technologiesAvailableForResearch;
	protected ResearchOutdatedTechnologies outdatedTechnologiesAvailableForResearch;
	
	/**
	 * True if this players turn has pressed the end turn button. 
	 * It means this player is not allowed to do anything until the start of the next turn.
	 * NB When turnEnded is true for all user players the turn ends and battles are executed.
	 */
	private boolean turnEnded;

	/**
	 * The money this player has in the bank.
	 */
	 private int funds;

	/**
	 * The boralium this player has in the bank.
	 */
	 private int boralium;

	/**
	 * This is the name of the ancient race this player is aligned to.
	 */
	 private String alignment;
	 
	 /**
	  * The last date that this player has lost a colony and had the ancients send a fleet 
	  * to the attackers to defend this player. 
	  */
	 private int lastDateAncientsDefended;

	/**
	 * The fleets owner by this player.
	 */
	private transient Fleet [] fleetArray;
	private ArrayList fleets = new ArrayList();
    
    /**
     * When this players objects are displayed this is the colour that this player 
     * will use.
     */
    private PlayerColor color;
    
    /**
     * This is the manager for all this players transports.
     */
    private TransportManager transportManager;
    
    /**
     * This maintains statistical information in the form of a graph for details the
     * empires total population, fleet size, research etc over the period of the game.
     */
    private Graph graph;
    
    /**
     * This is a designer for fighter crafts - shared among all players!
     */
    private static CraftDesignerFighter fighterDesigner;
    
    /**
     * Each player may have a single governor that will improve the performance of this
     * players planets. @see mot.common.leaders.Leader for details of how leaders work.
     */
    private EngineGovenorLeader govenorLeader;

    /**
     * Each player may have a single general that will improve the performance of this
     * players crafts. @see mot.common.leaders.Leader for details of how leaders work.
     */
    private EngineGeneralLeader generalLeader;

    /**
     * This is statistical information kept on the success of this player.
     * The quantity of battles this player has fought since the start of the game.
     */
    private int qtyBattles;
    
    /**
     * This is statistical information kept on the success of this player.
     * The quantity of battles this player has won since the start of the game.
     */
    private int qtyBattlesWon;
    
    /**
     * This is the probability that the Special HoloEmiter will work against this player.
     * A new Player in the game as a value of 1.0 (definitely will work), but every time
     * this trick is used the chanceHoloCraftWork is reduced.
     */
    private float chanceHoloCraftWork;
    
    private transient PlayerResourceCache resourceCache;

    /**
     * This maintains details of all the crafts this player has built. It is used to
     * determine the build quality of new crafts that are built.
     */
	private CraftBuildExperience craftBuildExperience;

	/**
	 * Through the game players get the option of making moral decisions. 
	 * @see MoralChoice for details. This is the list of moral choices by this player!
	 * The values in hear can effect this players weapon damage as well as relations. 
	 */
    private MoralChoiceListenerList moralChoices;
    
    /**
     * This is an AI module that recommends the adoption of Moral Policies for the player.
     */
    private MoralPolicyAdvisor moralPolicyAdvisor;
    
    /**
     * The maintenance cost (in money) per turn per spy.
     */
    private static final int MAINTENAINCE_COST_PER_SPY = 2;
    
    /**
     * This is the list of salvage this player has collected. This salvage can be used during 
     * craft construction to reduce the construction cost and if you have a salvaged item you
     * have not researched you can still use it on a craft! It is collected during battle
     * @see WreckageCraft for details.
     */
    private SalvageList salvage;
    
	/**
	 * Constructors.
	 */
	public Player() {
	}

	public Player(String name, PlayerCharacter character, PlayerColor color, String alignment) {
		super(name);

		this.character = character;
        this.color = color;
        this.alignment = alignment;
        turnEnded = false;
        
        moralPolicyAdvisor = new MoralPolicyAdvisorBasic();
    }

    /**
     * @return a list of all  crafts owned by this player.
     */
	public Craft [] getCrafts() throws ApplicationError {
		if (crafts == null) {
			ArrayList list = new ArrayList();
            Crafts cl = MotEngine.instance.getCrafts();
            if (cl == null) {
                return null;
            }
			Craft [] all = cl.getCrafts();
            if (all == null) {
                return null;
            }
			for (int i=0; i<all.length; i++) {
				if (all[i].getOwner() == this) {
					list.add(all[i]);
				}
			}
			crafts = (Craft []) list.toArray(new Craft[list.size()]);
		}
        return crafts;
	}

	public Craft [] getCraftsNotInScheduledBattle() throws ApplicationError {
		ArrayList list = new ArrayList();
        Crafts cl = MotEngine.instance.getCrafts();
        if (cl == null) {
            return null;
        }
		Craft [] all = cl.getCrafts();
        if (all == null) {
            return null;
        }
		for (int i=0; i<all.length; i++) {
			if (all[i].getOwner() == this) {
				if (!Battles.contains(all[i])) {
					list.add(all[i]);
				}
			}
		}
		return (Craft []) list.toArray(new Craft[list.size()]);
	}

    /**
     * @return a list of all crafts and fleets owned by this player (contains fleets and all crafts in the fleet).
     */
	public MoveableOwnedObject [] getCraftAndFleetObjects() throws ApplicationError {
		ArrayList list = new ArrayList();
        Crafts cl = MotEngine.instance.getCrafts();
        if (cl == null) {
            return null;
        }
		Craft [] all = cl.getCrafts();
        if (all == null) {
            return null;
        }
		for (int i=0; i<all.length; i++) {
			if (all[i].getOwner() == this) {
				list.add(all[i]);
				Fleet fleet = all[i].getFleet();
				if (fleet != null && !list.contains(fleet)) {
					list.add(fleet);
				}
			}
		}
		return (MoveableOwnedObject []) list.toArray(new MoveableOwnedObject[list.size()]);
	}


    /**
     * @return a list of crafts owned by this player and enroute to this solar system
     * or already at this solar system. This includes crafts that are members of a fleet.
     */
    public Craft [] getCraftsWithDestination( StellarObject so ) throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts == null) return null;
        ArrayList list = null;
        for (int i=0; i<crafts.length; i++) {
            Navigator nav = crafts[i].getNavigator();
            if (crafts[i].getLocation().equals(so.getLocation())) {
                if (list == null) list = new ArrayList();
                list.add(crafts[i]);
            } else if (nav != null && nav.getDestination().atSameSO(so)) {
                if (list == null) list = new ArrayList();
                list.add(crafts[i]);
            }
        }
        if (list == null || list.size() == 0) {
            return null;
        } else {
            return (Craft []) list.toArray(new Craft[list.size()]);
        }
    }

    /**
     * @return a list of crafts owned by this player and located at this stellar object.
     */
    public Craft [] getCraftsAt(StellarObject so, boolean includeInFleet, boolean includeWithNavigator) throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts == null) return null;
        ArrayList list = null;
        for (int i=0; i<crafts.length; i++) {
            Orbit orbit = crafts[i].getOrbit();
			if (orbit != null && orbit.atSameSO(so)) {
				if (crafts[i].getFleet() == null || includeInFleet) {
					if (crafts[i].getNavigator() == null || includeWithNavigator) {
		                if (list == null) list = new ArrayList();
		                list.add(crafts[i]);
					}
				}
            }
        }
        if (list == null || list.size() == 0) {
            return null;
        } else {
            return (Craft []) list.toArray(new Craft[list.size()]);
        }
    }

    /**
     * @return a list of crafts owned by this player and located at this stellar object.
     */
    public Craft [] getCraftsAt(Orbit orbit, boolean includeInFleet, boolean includeWithNavigator) throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts == null) return null;
        ArrayList list = null;
        for (int i=0; i<crafts.length; i++) {
            Orbit o = crafts[i].getOrbit();
			if (o != null && o.equals(orbit)) {
				if (crafts[i].getFleet() == null || includeInFleet) {
					if (crafts[i].getNavigator() == null || includeWithNavigator) {
		                if (list == null) list = new ArrayList();
		                list.add(crafts[i]);
					}
				}
            }
        }
        if (list == null || list.size() == 0) {
            return null;
        } else {
            return (Craft []) list.toArray(new Craft[list.size()]);
        }
    }

    public int getQtyMoosAt( StellarObject so ) throws ApplicationError {
        return getQtyCraftsAt(so) + getQtyTransportsAt(so);
    }


    /**
     * @return the quantity of crafts owned by this player and located at this stellar object.
     */
    public int getQtyCraftsAt( StellarObject so ) throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts == null) return 0;
        int qty = 0;
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getLocation().equals(so.getLocation())) {
                qty++;
            }
        }
        return qty;
    }

    public int getQtyTransportsAt( StellarObject so ) throws ApplicationError {
        Transport[] transports = getTransportManagerNotNull().getTransportsInUse();
        if (transports == null) return 0;
        int qty = 0;
        for (int i=0; i<transports.length; i++) {
            Orbit orbit = transports[i].getOrbit();
			if (orbit != null && orbit.getStellarObject() == so) {
                qty++;
            }
        }
        return qty;
    }

    /**
     * A military craft is defined as a craft that has weapons.
     * @return the quantity of military crafts owned by this player and located at this stellar object.
     */
    public int getQtyMilitaryCraftsAt( StellarObject so ) throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts == null) return 0;
        int qty = 0;
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getLocation().equals(so.getLocation())) {
                if (crafts[i].isMilitary()) {
                    qty++;
                }
            }
        }
        return qty;
    }

    /**
     * @return the first colony craft in the list of crafts owned by this player.
     */
	public Craft getColonyCraft() throws ApplicationError {
        Crafts cl = MotEngine.instance.getCrafts();
        if (cl == null) {
            return null;
        }
        Craft [] all = cl.getCrafts();
        if (all != null) {
	        for (int i=0; i<all.length; i++) {
	            if (all[i].getOwner() == this && all[i].canColonise()) {
	                return all[i];
	            }
	        }
        }
        return null;
	}

    /**
     * @return the quantity colony craft in the list of crafts owned by this player.
     */
	public int getQtyColonyCrafts() throws ApplicationError {
		int qty = 0;
		Craft crafts[] = getCrafts();
        if (crafts == null) {
            return 0;
        }
		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].canColonise()) qty++;
		}
		return qty++;
	}

	public int getQtyCraftsContaining(Weapon w) throws ApplicationError {
		int qty = 0;
		Craft crafts[] = getCrafts();
        if (crafts == null) {
            return 0;
        }
		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].containsWeapon(w)) qty++;
		}
		return qty++;
	}
	
	public int getQtyCraftsContainingWeaponGroup(String group) throws ApplicationError {
		int qty = 0;
		Craft crafts[] = getCrafts();
        if (crafts == null) {
            return 0;
        }
		for (int i=0; i<crafts.length; i++) {
			if (crafts[i].containsWeaponGroup(group)) qty++;
		}
		return qty++;
	}
	
//	public int getQtyDamageBotCrafts() throws ApplicationError {
//		int qty = 0;
//		Craft crafts[] = getCrafts();
//        if (crafts == null) {
//            return 0;
//        }
//		for (int i=0; i<crafts.length; i++) {
//			if (crafts[i].canInvade()) qty++;
//		}
//		return qty++;
//	}
	
    /**
     * @return the quantity military craft in the list of crafts owned by this player.
     */
	public int getQtyMilitartyCrafts() throws ApplicationError {
		int qty = 0;
		Craft crafts[] = getCrafts();
        if (crafts == null) {
            return 0;
        }
		for (int i=0; i<crafts.length; i++) {
			if (!crafts[i].canColonise()) qty++;
		}
		return qty;
	}

    /**
     * @return the total damage of all military crafts owned by this player.
     */
	public int getMilitaryCraftPower() throws ApplicationError {
		int qty = 0;
		Craft crafts[] = getCrafts();
        if (crafts == null) {
            return 0;
        }
		for (int i=0; i<crafts.length; i++) {
			if (!crafts[i].canColonise() && crafts[i].getWeaponBays() != null) {
                qty += crafts[i].getWeaponBays().getDamageSum(true);
            }
		}
		return qty;
	}


	public int getMilitaryCraftAverageDamage() throws ApplicationError {
		int qty = getQtyMilitartyCrafts();
		if (qty == 0) return 0;
		int total = getMilitaryCraftPower();
		return total / qty;
	}
    

    /**
     * @return the total quantity crafts in the list of crafts owned by this player.
     */
	public int getQtyCrafts() throws ApplicationError {
        Craft [] list = getCrafts();
        if (list == null) {
            return 0;
        } else {
            return list.length;
        }
	}

    /**
     * @return the quantity of crafts owned by this player and enroute to this solar system
     * or already at this solar system. This includes crafts that are members of a fleet.
     */
    public int getQtyCraftsWithDestination( StellarObject so ) throws ApplicationError {
        Craft [] crafts = getCrafts();
        if (crafts == null) return 0;
        int qty = 0;
        for (int i=0; i<crafts.length; i++) {
            Navigator nav = crafts[i].getNavigator();
            if (crafts[i].getLocation().equals(so.getLocation())) {
                qty++;
            } else if (nav != null && nav.getDestination().atSameSO(so)) {
                qty++;
            }
        }
        return qty;
    }

    /**
     * @return the quantity of troops owned by this player and enroute to this solar system 
     * or already at this planet.
     */
    public int getQtyTroopsWithDestination( StellarObject orbit ) throws ApplicationError {
    	return transportManager.getQtyTroopsWithDestination( orbit );
    }

    /**
     * Removes this player from the game.
     * It removed all this players crafts, colonies, spies as well as removing this 
     * player from the relations list. This method actually removes this player and 
     * all its associated things from the game.
     * Broadcasts an PlayerLoseEvent to all players.
     * @throws mot.common.util.ApplicationError
     */
	public void destroy() throws ApplicationError {
        destroy(true);
    }
    
    /**
     * Same as method above.
     */
	public void destroy(boolean sendEvent) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Player.destroy() player="+getName());
        }
        Craft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                crafts[i].destroy();
            }
        }
        Planet [] planets = getPlanets();
        if (planets != null) {
            for (int i=0; i<planets.length; i++) {
                planets[i].removeColony(true);
            }
        }
        
        MotEngine.instance.getSpiesNotNull().removeSpies(this);
        MotEngine.instance.getRelationsNotNull().remove(this);
        MotEngine.instance.getPlayersNotNull().remove(this);
        MotEngine.instance.getStellarObjectsNotNull().removeVisitors(this);
        MotEngine.instance.getStellarObjectsNotNull().removeStellarObjectOrbitals(this);
        MotEngine.instance.getSpiesNotNull().removeAllContaining(this);
        
        Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players != null) {
            for (int i=0; i<players.length; i++) {
                players[i].getPlayerCharacter().removeRelationshipsTo(this);
            }
        }
        
        MotEngine.instance.getDeadPlayersNotNull().add(this);
        
        if (sendEvent) {
            PlayerLoseEF event = new PlayerLoseEF(this);
            MotEngine.instance.broadcast(event);            
        }
    }

    /**
     * Removes the owned object from the list of owned objects for this player. This can 
     * be used to remove a craft or a planet. It does not actually create the object in
     * the game - it only flushes the cache of objects owned by this player so it goes
     * back to crafts or planets to recalculate its list of owned objects
     * @param o The owned object to remove.
     * @throws mot.common.util.ApplicationError
     */
	public void _remove(OwnedObject o) throws ApplicationError {
		if (o instanceof Craft) {
            _remove((Craft)o);
		} else if (o instanceof Planet) {
            _remove((Planet)o);
        } else {
            throw new ApplicationError("Player.remove() Invalid object type");
        }
	}

	public void _remove(Craft craft) {
		crafts = null;
	}

	public void _remove(Planet planet) {
		planets = null;
	}

    /**
     * Adds the owned object to the list of owned objects for this player. This can 
     * be used to add a craft or a planet. It does not actually create the object in
     * the game - it only flushes the cache of objects owned by this player so it goes
     * back to crafts or planets to recalculate its list of owned objects.
     * @param o The owned object to remove.
     * @throws mot.common.util.ApplicationError
     */
	public void add(OwnedObject o) throws ApplicationError {
		if (o instanceof Craft) {
            add((Craft)o);
		} else if (o instanceof Planet) {
            add((Planet)o);
        } else {
            throw new ApplicationError("Player.add() Invalid object type");
        }
	}

	public void add(Craft craft) {
		crafts = null;
	}

	public void add(Planet planet) {
		planets = null;
	}

	public void setFleets(ArrayList v) { fleets = v; }
	public ArrayList getFleets() { return fleets; }

    /**
     * Returns the list of fleet owner by this player.
     */
	public Fleet [] getFleet() {
		if (fleetArray == null && fleets != null && fleets.size() > 0) {
			fleetArray = (Fleet []) fleets.toArray(new Fleet[fleets.size()]);
		}
		return fleetArray;
	}

    /**
     * Returns the fleet owner by this player with the name passed in or null.
     */
	public Fleet getFleet(String name) {
		Fleet [] fleets = getFleet();
        if (fleets == null) {
            return null;
        }
		for (int i=0; i<fleets.length; i++) {
			Fleet fleet = fleets[i];
			if (fleet.sameName(name)) {
				return fleet;
			}
		}
        return null;
	}
    
    /**
     * Returns a list of fleet in orbit around the solar system passed in or null.
     */
    public Fleet [] getFleetAt(Orbit orbit, boolean includeWithNavigator) throws ApplicationError {
    	if (orbit == null) return null;
		Fleet [] fleets = getFleet();
        if (fleets == null) {
            return null;
        }
        ArrayList al = null;
		for (int i=0; i<fleets.length; i++) {
			Fleet fleet = fleets[i];
			if (orbit.equals(fleet.getOrbit())) {
				if (fleet.getNavigator() == null || includeWithNavigator) {
	                if (al == null) al = new ArrayList();
					al.add(fleet);
				}
			}
		}
        if (al == null) {
            return null;
        } else {
            return (Fleet []) al.toArray(new Fleet[al.size()]);
        }
    }

    /**
     * Returns a list of fleet in orbit around the StellarObject orbit and with destination 
     * dest or null if none exist.
     */
    public Fleet [] getFleetAtWithDest(StellarObject orbit, StellarObject dest) throws ApplicationError {
		Fleet [] fleets = getFleet();
        if (fleets == null) {
            return null;
        }
        ArrayList al = null;
		for (int i=0; i<fleets.length; i++) {
			Fleet fleet = fleets[i];
            Navigator nav = fleet.getNavigator();
			if (fleet.getOrbitStellarObject() == orbit && nav != null && nav.getDestination().atSameSO(dest)) {
                if (al == null) al = new ArrayList();
				al.add(fleet);
			}
		}
        if (al == null) {
            return null;
        } else {
            return (Fleet []) al.toArray(new Fleet[al.size()]);
        }
    }

    /**
     * Adds a new fleet to the list of fleets owned by this player.
     * @param fleet The fleet to add
     */
	public void setFleet(Fleet fleet) {
		fleetArray = null;
		fleets.add(fleet);
	}

    /**
     * Removes a fleet from the list of fleets owned by this player.
     * @param fleet The fleet to remove
     */
	public void removeFleet(Fleet fleet) {
		fleetArray = null;
		fleets.remove(fleet);
	}

    /**
     * @return The list of planets owned by this player
     */
	public Planet [] getPlanets() throws ApplicationError {
		if (planets == null && MotEngine.instance != null && MotEngine.instance.getPlanets() != null) {
			ArrayList list = new ArrayList();
			Planet [] all = MotEngine.instance.getPlanets().getPlanets();
			for (int i=0; i<all.length; i++) {
				if (all[i].getOwner() == this) {
					list.add(all[i]);
				}
			}
			planets = (Planet []) list.toArray(new Planet[list.size()]);
		}
        return planets;
	}

    /**
     * For details of when a planet is visible by a player @see SolarSystem.isVisable(Player).
     * This is a list of planets is an visible solar system, NOT, a list of viewable planets!
     * @return The list of planets visible by this player by this player.
     */
	public Planet [] getVisiblePlanets() throws ApplicationError {
		if (MotEngine.instance.getPlanets() != null) {
			ArrayList list = new ArrayList();
			Planet [] all = MotEngine.instance.getPlanets().getPlanets();
			for (int i=0; i<all.length; i++) {
				if (all[i].getSolarSystem().isVisible(this)) {
					list.add(all[i]);
				}
			}
			return (Planet []) list.toArray(new Planet[list.size()]);
		} else {
            return null;
        }
	}

    /**
     * Gets the list of planets in this solar system whose owner is at war with the
     * player passed in. 
     */
	public Planet [] getEnemyPlanetsAt(SolarSystem ss) throws ApplicationError {
        Planet [] all = ss.getPlanets();
        if (all == null) return null;
        Relations rels = MotEngine.instance.getRelations();
        ArrayList list = null;
        for (int i=0; i<all.length; i++) {
            if (all[i].getOwner() != null && rels.isWar(this, all[i].getOwner())) {
                if (list == null) list = new ArrayList();
                list.add(all[i]);
            }
        }
        if (list == null) {
            return null;
        } else {
            return (Planet []) list.toArray(new Planet[list.size()]);
        }
	}

    /**
     * Gets the list of planets owned by this player. This method returns a new ArrayList
     * every time it is called.
     * @return An ArrayList containing the planets for this player.
     */
	public ArrayList getPlanetsAsList() throws ApplicationError {
        ArrayList list = new ArrayList();
        Planet [] all = MotEngine.instance.getPlanets().getPlanets();
        for (int i=0; i<all.length; i++) {
            if (all[i].getOwner() == this) {
                list.add(all[i]);
            }
        }
        return list;
	}

    /**
     * @return The quantity of planets owned by this player.
     */
	public int getQtyPlanets() throws ApplicationError {
        Planet [] planets = getPlanets();
        if (planets == null) {
            return 0;
        } else {
            return planets.length;
        }
    }
    
    public SolarSystem getCapitolSolarSystem() throws ApplicationError {
    	Planet capitol = getCapitol();
    	if (capitol != null) {
    		return capitol.getSolarSystem();
    	} else {
    		return null;
    	}
    }
    
    /**
     * @return The Home World of this player. The Home World is defined as the 
     * planet This player has had as a colony for the longest.
     */
    public Planet getCapitol() throws ApplicationError {
        Planet [] planets = getPlanets();
        if (planets == null) {
            return null;
        } else {
            Planet best = null;
            for (int i=0; i<planets.length; i++) {
            	if (!planets[i].isMiningColony()) {
	                if (best == null || best.getDateColonised() > planets[i].getDateColonised()) {
	                    best = planets[i];
	                }
            	}
            }
            return best;
        }
    }

    public Planet getPlanetWithHighestProduction() throws ApplicationError {
        Planet [] planets = getPlanets();
        if (planets == null) {
            return null;
        } else {
            Planet best = null;
            int bestProduction = 0;
            for (int i=0; i<planets.length; i++) {
            	if (!planets[i].isMiningColony()) {
	                int p = planets[i].getProductionOutput();
					if (best == null || bestProduction <  p) {
	                    best = planets[i];
	                    bestProduction = p;
	                }
            	}
            }
            return best;
        }
    }

    /**
     * @return The quantity of planets owned by this player that have this project. 
     * @param project The project that all matching planets must have
     */
    public int getQtyPlanetsWithProject(Class project) throws ApplicationError {
        Planet [] planets = getPlanets();
        if (planets == null) {
            return 0;
        } else {
            int qty = 0;
            for (int i=0; i<planets.length; i++) {
                Project p = planets[i].getProject();
                if (p != null && p.getClass() == project) {
                    qty++;
                }
            }
            return qty;
        }
    }

    /**
     * @return The quantity of planets owned by this player that have this project. 
     * @param project The project that all matching planets must have
     */
    public int getQtyPlanetsWithPopulationBiggerThan(int minPopulation) throws ApplicationError {
        Planet [] planets = getPlanets();
        if (planets == null) {
            return 0;
        } else {
            int qty = 0;
            for (int i=0; i<planets.length; i++) {
                double pop = planets[i].getPopulation();
                if (pop > minPopulation) {
                    qty++;
                }
            }
            return qty;
        }
    }

    /**
     * @return The list of stellar objects owned by this player.
     */
	public StellarObject [] getStellarObjects() throws ApplicationError {
		if (stellarObjects == null) {
            StellarObject [] sos = MotEngine.getInstance().getStellarObjects().getStellarObjects();
            if (sos == null) return null;
			ArrayList list = new ArrayList();
			for (int i=0; i<sos.length; i++) {
				if (sos[i].isOwnedBy(this)) {
                    list.add(sos[i]);
                }
			}
			stellarObjects = (StellarObject []) list.toArray(new StellarObject [list.size()]);
		}
		return stellarObjects;
	}

	public BlackHole getWormHole() throws ApplicationError {
		StellarObject[] soa = getStellarObjects();
		if (soa != null) {
			for (int i = 0; i < soa.length; i++) {
				if (soa[i] instanceof BlackHole) {
					BlackHole bh = (BlackHole) soa[i];
					if (bh.isSpaceJumpAbility()) {
						return bh;
					}
				}
			}
		}
		return null;
	}

    /**
     * When a player gains or loses a planet, space station, outpost etc then flushCache() 
     * needs to be called to make sure the player recalculates its luist of owned objects.
     */
	public void flushCache() {
		planets = null;
		solarSystems = null;
        stellarObjects = null;
        resourceCache = null;
	}

    /**
     * @return the closest stellar object to this location owned by this player.
     */
	public StellarObject getClosestStellarObject(Location loc) throws ApplicationError {
		StellarObject [] sos = getStellarObjects();
		if (sos == null) return null;
		StellarObject closest = null;
		for (int i=0; i<sos.length; i++) {
			if ( (closest == null || sos[i].getDistanceTo(loc) < closest.getDistanceTo(loc)) ) {
				closest = sos[i];
			}
		}
		return closest;
	}

    /**
     * @return the distance to the closest stellar object owned by this player to the stellar object passed in.
     */
	public int getDistanceToClosestStellarObject(StellarObject other) throws ApplicationError {
		if (other == null) return Integer.MAX_VALUE;
		Location loc = other.getLocation();
		StellarObject [] sos = getStellarObjects();
		if (sos == null) return Integer.MAX_VALUE;
		int closestDist = Integer.MAX_VALUE;
		for (int i=0; i<sos.length; i++) {
			int dist = sos[i].getDistanceTo(loc);
			if ( (dist < closestDist) ) {
				closestDist = dist;			
			}
		}
		return closestDist;
	}

    /**
     * @return the closest craft owned by this player to location loc.
     */
	public Craft getClosestCraft(Location loc) throws ApplicationError {
		Craft [] crafts = getCrafts();
        if (crafts == null) return null;
		Craft closestCraft = null;
		for (int i=0; i<crafts.length; i++) {
            if (closestCraft == null || closestCraft.getDistanceTo(loc) > crafts[i].getDistanceTo(loc)) {
                closestCraft = crafts[i];
            }
		}
        return closestCraft;
	}

    /**
     * A player can travel to a stellar object if they have research engines whose range
     * is greater than the distance between it and the players closest stellar object.
     * @return True if this player can travel to the stellar object so.
     */
	public boolean canTravelTo(StellarObject so) throws ApplicationError {
        Engines engines = getEngines();
        if (engines == null) return false;
        int maxTravelDistance = engines.getMaxTravelDistance();
		StellarObject closest = getClosestStellarObject(so.getLocation());
        if (closest == null) return false;
		int distance = so.getLocation().getDistanceTo(closest.getLocation());
		return (distance <= maxTravelDistance);
	}

    /**
     * @return true if and only if this player can colonise planet.
     * To be colonisable the planet must not have an owner and this player
     * must have the ability to colonise planets of this quality. Regardless
     * of technology everybody can colonise planets whose terraform quality is
     * greater or equal to 0.8.
     */
	public boolean canColonise(Planet planet) throws ApplicationError {
        if (planet.getOwner() != null) return false;
        SolarSystem ss = planet.getSolarSystem();
		if (ss.getOwners() != null && !ss.isOwnedBy(this)) return false;
        if (planet.getTeraforming() >= 0.8) return true;
        Specials specials = getSpecials();
        if (specials == null) return false;
		Double x = specials.getCanColoniseRating();
        if (x == null) return false;
		double canColoniseRating = x.doubleValue();
        return (planet.getTeraforming() >= canColoniseRating);
	}

	public boolean isEnemy(Player player) throws ApplicationError {
        return player != null && MotEngine.instance.getRelationsNotNull().isWar( this, player );
	}

    /**
     * The planet returned may not be in range of this players engines - So that
     * player may not actually be able to colonise it. The player does have the
     * technology to colonise this planet though.
     * @return The closest planet that this player has the ability to colonise.
     * @throws mot.common.util.ApplicationError
     */
    public Planet getClosestPlanetCanColonise() throws ApplicationError {
        Planet [] planets = MotEngine.getInstance().getPlanets().getPlanets();
        int dist = 0;
        Planet best = null;
        for (int i=0; i<planets.length; i++) {
            if (canColonise(planets[i])) {
                Location loc = planets[i].getStellarObject().getLocation();
                StellarObject myClosest = getClosestStellarObject(loc);
                if (myClosest != null) {
                    int d = myClosest.getDistanceTo(loc);
                    if (best == null || dist > d) {
                        //Craft [] guards = MotEngine.instance.getMonsterPlayer().getCraftsAt(planets[i].getStellarObject());
                        //if (canBeGuarded || guards == null) {
	                        best = planets[i];
	                        dist = d;
                        //}
                    }
                }
            }
        }
        return best;
    }

    public Planet getClosestDefendedPlanetWithBiggestPopulation(StellarObject target) throws ApplicationError {
    	if (target == null) return null;
        Planet [] planets = getPlanets();
        int dist = 0;
        Planet best = null;
        for (int i=0; i<planets.length; i++) {
            if (planets[i].getStellarObject() != target && planets[i].isDefended()) {
                int d = target.getDistanceTo(planets[i].getStellarObject());
                if (best == null) {
                    best = planets[i];
                    dist = d;
                } else if (dist > d) {
                    best = planets[i];
                    dist = d;
                } else if (best.getStellarObject() == planets[i].getStellarObject() && best.getPopulation() > planets[i].getPopulation()) {
                    best = planets[i];
                    dist = d;
                }
            }
        }
        return best;
    }

    public Planet getClosestPlanetWithBiggestPopulation(StellarObject target) throws ApplicationError {
    	if (target == null) return null;
        Planet [] planets = getPlanets();
        int dist = 0;
        Planet best = null;
        for (int i=0; i<planets.length; i++) {
            if (planets[i].getStellarObject() != target) {
                int d = target.getDistanceTo(planets[i].getStellarObject());
                if (best == null) {
                    best = planets[i];
                    dist = d;
                } else if (dist > d) {
                    best = planets[i];
                    dist = d;
                } else if (best.getStellarObject() == planets[i].getStellarObject() && best.getPopulation() > planets[i].getPopulation()) {
                    best = planets[i];
                    dist = d;
                }
            }
        }
        return best;
    }

	public Player [] getEnemyPlayers() throws ApplicationError {
		return MotEngine.instance.getRelationsNotNull().getPlayersAtWarWith(this);
	}
	
    /**
     * @return a list of players this player is at war with who have a craft or planet located 
     * at this solar system.
     */
	public Player [] getEnemyPlayersAt(StellarObject so) throws ApplicationError {
		ArrayList list = null;
        Player [] owners = so.getOwners();
        if (owners != null) {
    		for (int i=0; i<owners.length; i++) {
                if ( isEnemy(owners[i]) ) {
                    if (list == null) {
                        list = new ArrayList();
                        list.add( owners[i] );
                    } else if (!list.contains(owners[i])) {
                        list.add( owners[i] );
                    }
                }
            }
        }
        if (MotEngine.instance.getCrafts() != null) {
            Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
            for (int i=0; crafts != null && i<crafts.length; i++) {
                if ( crafts[i].getOrbitStellarObject() == so && isEnemy(crafts[i].getOwner()) ) {
                    Player owner = crafts[i].getOwner();
                    if (list == null) {
                        list = new ArrayList();
                        list.add( owner );
                    } else if (!list.contains(owner)) {
                        list.add(owner);
                    }
                }
            }
        }
        if (list == null) {
            return null;
        } else {
    		return (Player [])list.toArray(new Player [list.size()]);
        }
	}

    /**
     * @return A list of craft owned by enemy players (of this player) located 
     * at this solar object. 
     */
	public Craft [] getEnemyCraftAt(StellarObject so, boolean includeCraftWithOrders) throws ApplicationError {
        if (MotEngine.instance.getCrafts() == null || so == null) {
            return null;
        }
		Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
		if (crafts == null) {
			return null;
		}
		ArrayList list = null;
		for (int i=0; i<crafts.length; i++) {
			if ( crafts[i].getOrbitStellarObject() == so && isEnemy(crafts[i].getOwner()) ) {
                if (crafts[i].getNavigator() == null || includeCraftWithOrders) {
                    if (list == null) list = new ArrayList();
                    list.add( crafts[i] );
                }
			}
		}
        if (list == null) {
            return null;
        } else {
    		return (Craft [])list.toArray(new Craft [list.size()]);
        }
	}

    /**
     * @return a list of craft owned by players this player is at war with located 
     * at this solar system or whose current destination is this solar system
     */
	public Craft [] getEnemyCraftMovingTo(StellarObject so) throws ApplicationError {
		Craft [] crafts = MotEngine.instance.getCrafts().getCrafts();
		ArrayList list = null;
		for (int i=0; i<crafts.length; i++) {
			if ( crafts[i].isOnrouteTo(so) || crafts[i].getOrbitStellarObject() == so ) {
                if ( isEnemy(crafts[i].getOwner()) ) {
                    if (list == null) list = new ArrayList();
                    list.add( crafts[i] );
                }
            }
		}
        if (list == null) {
            return null;
        } else {
    		return (Craft [])list.toArray(new Craft [list.size()]);
        }
	}

    /**
     * @return The quantity of crafts owned by players this player is at war with located 
     * at this solar system.
     */
	public int getQtyEnemyCraftAt(StellarObject so) throws ApplicationError {
        Crafts craftList = MotEngine.instance.getCrafts();
        if (craftList == null) return 0;
		Craft [] crafts = craftList.getCrafts();
        if (crafts == null) return 0;
		int qty = 0;
		for (int i=0; i<crafts.length; i++) {
			if ( crafts[i].getOrbitStellarObject() == so ) {
                if ( isEnemy(crafts[i].getOwner()) ) {
                    qty++;
                }
            }
		}
        return qty;
	}

    /**
     * @return The quantity of crafts owned by players this player is at war with located 
     * at this solar system or whose current destination is this solar system
     */
	public int getQtyEnemyCraftMovingTo(StellarObject so) throws ApplicationError {
        Crafts gameCrafts = MotEngine.instance.getCrafts();
        if (gameCrafts == null) return 0;
		Craft [] crafts = gameCrafts.getCrafts();
        if (crafts == null) return 0;
		int qty = 0;
		for (int i=0; i<crafts.length; i++) {
			if ( crafts[i].isOnrouteTo(so) || crafts[i].getOrbitStellarObject() == so ) {
                if ( isEnemy(crafts[i].getOwner()) ) {
                    qty++;
                }
            }
		}
        return qty;
	}

	/**
	 * @return The quantity of enemy craft blockading my planets or heading to my planets.
	 * @throws ApplicationError
	 */
	public int getQtyEnemyCraftAttacking() throws ApplicationError {
        Crafts gameCrafts = MotEngine.instance.getCrafts();
        if (gameCrafts == null) return 0;
		Craft [] crafts = gameCrafts.getCrafts();
        if (crafts == null) return 0;
		int qty = 0;
		for (int i=0; i<crafts.length; i++) {
            if ( isEnemy(crafts[i].getOwner()) ) {
    			if ( crafts[i].getOrbit() != null && crafts[i].getOrbit().isOwnedBy(this) ) {
    				qty++;
    			} else if ( crafts[i].getNavigator() != null && crafts[i].getNavigator().getDestination().isOwnedBy(this) ) {
    				qty++;
    			}
            }
		}
        return qty;
	}

	/**
	 * @return the strength at which this player can see objects at this location.
	 * This is defined as the strength of the closest scanner - the distance from 
     * the scanner.
	 */
	public int getScannerStrengthAt(Location loc) throws ApplicationError {
		int scanStrength = 0;
        StellarObject [] sos = getStellarObjects();
        if (sos != null) {
            for (int i=0; i<sos.length; i++) {
                int v = sos[i].getScannerStrengthAt(loc, this);
                if (v > scanStrength) scanStrength = v;
            }
        }
        Craft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                int v = crafts[i].getScannerStrengthAt(loc);
                if (v > scanStrength) scanStrength = v;
            }
        }
		return scanStrength;
	}

	public Player [] getPlayersAtWar() throws ApplicationError {
		return 	MotEngine.instance.getRelations().getPlayersAtWarWith(this);
	}

	public void setFactories(Factories v) { factories = v; }
	public void setLaboratories(Laboratories v) { laboratories = v;  }
	public void setPlayerCharacter(PlayerCharacter v) { character = v;  }
	public void setResearchProject(ResearchProject v) { researchProject = v; }
	public void setHulls(Hulls v) { hulls = v; }
	public void setComputers(Computers v) { computers = v; }
	public void setModules(Modules v) { modules = v; }
	public void setHumanities(Humanities v) { humanities = v; }
	public void setWeapons(Weapons v) { weapons = v; }
	public void setShields(Shields v) { shields = v; }
	public void setScanners(Scanners v) { scanners = v; }
	public void setSpecials(Specials v) { specials = v; }
	public void setEngines(Engines v) { engines = v; }
	public void setBiotechnologies(Biotechnologies v) { biotechnologies = v; }
	public void setTheories(Theories v) { theories = v; }
	public void setMiscTechnologies(MiscTechnologies v) { miscTechnologies = v; }
	public void setLastDiscovery(ResearchScriptOption v) { lastDiscovery = v; }
	public void setFunds(int v) { funds = v; }
	public void setBoralium(int v) { boralium = v; }
    public void setColor(String v) throws ApplicationError { setColor(PlayerColor.getColorForName(v)); }
    public void setColor(PlayerColor v) { color = v; }
    public void setTransportManager(TransportManager v) { transportManager = v; }
    
	public Factories getFactories() { return factories; }
	public Laboratories getLaboratories() { return laboratories; }
	public PlayerCharacter getPlayerCharacter() { return character; }
	public PlayerCharacterBasic getPlayerCharacterBasic() { return character == null ? null : character.getBasicCharacter(); }
	public TechnologyList getMissedTechnologies() throws ApplicationError { return character == null ? null : character.getBasicCharacter().getMissedTechnologiesAsList(); }
	public Hulls getHulls() { return hulls; }
	public Modules getModules() { return modules; }
	public Humanities getHumanities() { return humanities; }
	public Computers getComputers() { return computers; }
	public Weapons getWeapons() { return weapons; }
	public Shields getShields() { return shields; }
	public Scanners getScanners() { return scanners; }
	public Specials getSpecials() { return specials; }
	public Engines getEngines() { return engines; }
	public Biotechnologies getBiotechnologies() { return biotechnologies; }
	public Theories getTheories() { return theories; }
	public MiscTechnologies getMiscTechnologies() { return miscTechnologies; }
	public ResearchScriptOption getLastDiscovery() { return lastDiscovery; } 
	public int getFunds() { return funds; }
	public int getBoralium() { return boralium; }
    public PlayerColor getColor() { return color; }
    public boolean getTurnEnded() { return turnEnded; }
    public TransportManager getTransportManager() { return transportManager; }
    public TransportManager getTransportManagerNotNull() { if (transportManager == null) transportManager =  new TransportManager(); return transportManager; }
    
    public ResearchOutdatedTechnologies getOutdatedTechnologiesAvailableForResearch() throws ApplicationError {
    	if (outdatedTechnologiesAvailableForResearch == null) outdatedTechnologiesAvailableForResearch = new ResearchOutdatedTechnologies(this);
    	return outdatedTechnologiesAvailableForResearch; 
    }

    public void setTurnEnded(boolean v) {
    	if (Logger.isDebug()) {
    		Logger.logDebug("Player.setTurnEnded() "+getName()+" v="+v);
    	}
    	turnEnded = v; 
    }


    public PlayerResourceCache getResourceCache() throws ApplicationError {
        if (resourceCache == null) {
            resourceCache = new PlayerResourceCache(this);
        }
        return resourceCache;
    }

    public void addTechnologies(TechnologyList techs) throws ApplicationError {
        if (techs != null) {
            addTechnologies(techs.getAllAsArray());
        }
    }
    
    public void addTechnologies(Technology [] techs) throws ApplicationError {
        if (techs != null) {
            for (int i=0; i<techs.length; i++) {
                addTechnology(techs[i]);
            }
        }
    }
    
    public void addTech(Technology ... techs) throws ApplicationError {
        if (techs != null) {
            for (int i=0; i<techs.length; i++) {
                addTechnology(techs[i]);
            }
        }
    }
    
    /**
     * Adds this technology to the list of techs this player has already discovered.
     */
    public void addTechnology(Technology tech) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("Player.addTechnology() player="+getName()+" tech="+tech.getFullName());
        }
		if (tech instanceof Factory) {
			addFactory((Factory)tech);
		} else if (tech instanceof Laboratory) {
			addLaboratory((Laboratory)tech);
		} else if (tech instanceof Biotechnology) {
			addBiotechnology((Biotechnology)tech);
		} else if (tech instanceof Hull) {
			addHull((Hull)tech);
		} else if (tech instanceof Computer) {
			addComputer((Computer)tech);
		} else if (tech instanceof Weapon) {
			addWeapon((Weapon)tech);
		} else if (tech instanceof Shield) {
			addShield((Shield)tech);
		} else if (tech instanceof Scanner) {
			addScanner((Scanner)tech);
		} else if (tech instanceof Special) {
			addSpecial((Special)tech);
		} else if (tech instanceof Engine) {
			addEngine((Engine)tech);
		} else if (tech instanceof Module) {
			addModule((Module)tech);
		} else if (tech instanceof Humanity) {
			addHumanity((Humanity)tech);
		} else if (tech instanceof Theory) {
			addTheory((Theory)tech);
		} else if (tech instanceof MiscTechnology) {
			addMiscTechnology((MiscTechnology)tech);
		} else {
			throw new ApplicationError("Player.finishResearchProject() Invalid technology type "+tech.getClass().getName());
		}
		if (researchProject != null && researchProject.getTechnology() == tech) {
			researchProject = null;
			notifyNoResearchProject(true);
		}
    }
    
	public void addFactory(Factory v) throws ApplicationError {
		if (factories == null) factories = new Factories();
        if (v == null) throw new ApplicationError("Player.addFactory() Cannot add null");
        if (factories.contains(v)) return;
		factories.add(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addLaboratory(Laboratory v) throws ApplicationError {
		if (laboratories == null) laboratories = new Laboratories();
        if (v == null) throw new ApplicationError("Player.addLaboratory() Cannot add null");
        if (laboratories.contains(v)) return;
		laboratories.add(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addBiotechnology(Biotechnology v) throws ApplicationError {
		if (biotechnologies == null) biotechnologies = new Biotechnologies();
        if (v == null) throw new ApplicationError("Player.addBiotechnology() Cannot add null");
        if (biotechnologies.contains(v)) return;
		biotechnologies.add(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addHull(Hull v) throws ApplicationError {
		if (hulls == null) hulls = new Hulls();
        if (v == null) throw new ApplicationError("Player.addHull() Cannot add null");
        if (hulls.contains(v)) return;
		hulls.add(v);
		if (v.isBattleStation()) {
			Planet[] pa = getPlanets();
			if (pa != null) {
				for (int i = 0; i < pa.length; i++) {
					pa[i].upgradeOrbital(v);
				}
			}
		}
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addComputer(Computer v) throws ApplicationError {
		if (computers == null) computers = new Computers();
        if (v == null) throw new ApplicationError("Player.addComputer() Cannot add null");
        if (computers.contains(v)) return;
		computers.add(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addWeapon(Weapon v) throws ApplicationError {
		if (weapons == null) weapons = new Weapons();
        if (v == null) throw new ApplicationError("Player.addWeapon() Cannot add null");
        if (weapons.contains(v)) return;
		weapons.add(v);
		Planet[] pa = getPlanets();
		if (pa != null) {
			for (int i = 0; i < pa.length; i++) {
				pa[i].upgradeWeapons(v);
			}
		}
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addShield(Shield v) throws ApplicationError {
		if (shields == null) shields = new Shields();
        if (v == null) throw new ApplicationError("Player.addShield() Cannot add null");
        if (shields.contains(v)) return;
		shields.add(v);
		Planet[] pa = getPlanets();
		if (pa != null) {
			for (int i = 0; i < pa.length; i++) {
				pa[i].upgradeShield(v);
			}
		}
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addScanner(Scanner v) throws ApplicationError {
		if (scanners == null) scanners = new Scanners();
        if (v == null) throw new ApplicationError("Player.addScanner() Cannot add null");
        if (scanners.contains(v)) return;
		scanners.add(v);
		Planet[] pa = getPlanets();
		if (pa != null) {
			for (int i = 0; i < pa.length; i++) {
				pa[i].upgradeScanner(v);
			}
		}
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addSpecial(Special v) throws ApplicationError {
		if (specials == null) specials = new Specials();
        if (v == null) throw new ApplicationError("Player.addSpecial() Cannot add null");
        if (specials.contains(v)) return;
		specials.add(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addEngine(Engine v) throws ApplicationError {
		if (engines == null) engines = new Engines();
        if (v == null) throw new ApplicationError("Player.addEngine() Cannot add null");
        if (engines.contains(v)) return;
		engines.add(v);
        getTransportManagerNotNull().updateEngines(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}
    
	public void addModule(Module v) throws ApplicationError {
		if (modules == null) modules = new Modules();
        if (v == null) throw new ApplicationError("Player.addModule() Cannot add null");
        if (modules.contains(v)) return;
		modules.add(v);
		// Modules deployed to space station are upgraded automatically when discovered.
		Planet[] planets = getPlanets();
		for (int i = 0; planets != null && i < planets.length; i++) {
			Orbital orbital = planets[i].getOrbital();
			if (orbital != null) {
				if (planets[i].isMiningColony()) {
					if (v.isOnSpaceStation()) {
						orbital.addModule(v);
					}
				} else {
					if (v.isOnBattleStation()) {
						orbital.addModule(v);
					}
				}
			}
		}
		StellarObject[] sos = getStellarObjects();
		for (int i = 0; i < sos.length; i++) {
			StellarObjectOrbital orbital = sos[i].getOrbital();
			if (orbital != null) {
				Hull hull = orbital.getHull();
				if (hull != null) {
					if (hull.isSpaceStation() && v.isOnSpaceStation()) {
						orbital.getOrbital().addModule(v);
					} else if (hull.isOutpost() && v.isOnOutpost()) {
						orbital.getOrbital().addModule(v);
					}
				}
			}
		}
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}
    
    
	public void addHumanity(Humanity v) throws ApplicationError {
		if (humanities == null) humanities = new Humanities();
        if (v == null) throw new ApplicationError("Player.addHumanity() Cannot add null");
        if (humanities.contains(v)) return;
        humanities.add(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}
	
	public void addTheory(Theory v) throws ApplicationError {
		if (theories == null) theories = new Theories();
        if (v == null) throw new ApplicationError("Player.addTheory() Cannot add null");
        if (theories.contains(v)) return;
		theories.add(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
	}

	public void addMiscTechnology(MiscTechnology v) throws ApplicationError {
		if (miscTechnologies == null) miscTechnologies = new MiscTechnologies();
        if (v == null) throw new ApplicationError("Player.addMiscTechnology() Cannot add null");
        if (miscTechnologies.contains(v)) return;
        miscTechnologies.add(v);
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
		resourceCache = null;
		if (v instanceof GalacticLore) {
			StellarObject[] soa = MotEngine.instance.getStellarObjectsNotNull().getStellarObjects();
			if (soa != null) {
				for (int i = 0; i < soa.length; i++) {
					soa[i].addVisitor(this, false);
				}
			}
		}
	}

	
    /**
     * This method is slow as there is no caching of the list generated.
     * @return Returns a list containing all technology owned by this player.
     */
    public TechnologyList getTechnologies() {
        TechnologyList techList = new GeneralTechnologyList();
        if (factories != null) techList.add(factories);
        if (laboratories != null) techList.add(laboratories);
        if (biotechnologies != null) techList.add(biotechnologies);
        if (hulls != null) techList.add(hulls);
        if (computers != null) techList.add(computers);
        if (weapons != null) techList.add(weapons);
        if (shields != null) techList.add(shields);
        if (scanners != null) techList.add(scanners);
        if (specials != null) techList.add(specials);
        if (engines != null) techList.add(engines);
        if (modules != null) techList.add(modules);
        if (humanities != null) techList.add(humanities);
        if (theories != null) techList.add(theories);
        if (miscTechnologies != null) techList.add(miscTechnologies);
        return techList;
    }

    /**
     * removes all technologies this player has discovered.
     */
    public void removeAllTechnologies() {
        if (factories != null) factories.removeAll();
        if (laboratories != null) laboratories.removeAll();
        if (biotechnologies != null) biotechnologies.removeAll();
        if (hulls != null) hulls.removeAll();
        if (computers != null) computers.removeAll();
        if (weapons != null) weapons.removeAll();
        if (shields != null) shields.removeAll();
        if (scanners != null) scanners.removeAll();
        if (specials != null) specials.removeAll();
        if (engines != null) engines.removeAll();
        if (modules != null) modules.removeAll();
        if (humanities != null) humanities.removeAll();
        if (theories != null) theories.removeAll();
        if (miscTechnologies != null) miscTechnologies.removeAll();
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
    }
    
    public void removeTechnology(Technology tech) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("Player.removeTechnology() player="+getName()+" tech="+tech.getFullName());
        }
		if (tech instanceof Factory) {
			if (factories != null) factories.remove(tech);
		} else if (tech instanceof Laboratory) {
			if (laboratories != null) laboratories.remove(tech);
		} else if (tech instanceof Biotechnology) {
			if (biotechnologies != null) biotechnologies.remove(tech);
		} else if (tech instanceof Hull) {
			if (hulls != null) hulls.remove(tech);
		} else if (tech instanceof Computer) {
			if (computers != null) computers.remove(tech);
		} else if (tech instanceof Weapon) {
			if (weapons != null) weapons.remove(tech);
		} else if (tech instanceof Shield) {
			if (shields != null) shields.remove(tech);
		} else if (tech instanceof Scanner) {
			if (scanners != null) scanners.remove(tech);
		} else if (tech instanceof Special) {
			if (specials != null) specials.remove(tech);
		} else if (tech instanceof Engine) {
			if (engines != null) engines.remove(tech);
		} else if (tech instanceof Module) {
			if (modules != null) modules.remove(tech);
		} else if (tech instanceof Humanity) {
			if (humanities != null) humanities.remove(tech);
		} else if (tech instanceof Theory) {
			if (theories != null) theories.remove(tech);
		} else if (tech instanceof MiscTechnology) {
			if (miscTechnologies != null) miscTechnologies.remove(tech);
		} else {
			throw new ApplicationError("Player.finishResearchProject() Invalid technology type "+tech.getClass().getName());
		}
		technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
    }
    


    /**
     * A technology list or null of all tech that this player has and the other player does not have.
     * @param other The player who is a candidate for stealing the tech
     * @return A technology list or null of all tech that this player has and the other player does not have.
     * @throws mot.common.util.ApplicationError
     */
    public TechnologyList getStealableTechnologies(Player other) throws ApplicationError {
        TechnologyList techs = getTechnologies();
        techs = new GeneralTechnologyList(techs); // Clone the list!!!!
        techs.remove(techs.getEspionageForbiddenTechnologies());
        techs.remove(other.getTechnologies());
        return techs;
    }
    
    /**
     * The player other passed in will steel a technology owned by this player if any exist.
     * This broadcasts a StealTechnology event top both this player and the receiver. The technology is also
     * added to the receiver.
     * @param other The player steeling technology.
     * @return A technology discovered by this player and not by the player other.
     */
    public void steelTechnology(Player other, Technology suggestedTechnology) throws ApplicationError {
        // We first get a list of all technologies discovered by this player but not
        // discovered by the other player.
        TechnologyList techs = getStealableTechnologies(other);
        if (techs == null || techs.getQuantity() == 0) {
            if (Logger.isDebug()) {
                Logger.logDebug("Player.steelTechnology() Player "+other.getName()+" NOTHING TO STEEL");
            }
            return;
        }
        
        // Now choose a random technology from techs
        Technology tech = null;
        if (suggestedTechnology != null && techs.contains(suggestedTechnology)) {
        	tech = suggestedTechnology;
        }
        if (tech == null) {
        	tech = techs.getRandom();
        }
    
        // Now steal this technology
        other.addTechnology(tech);

        if (tech == null) {
            if (Logger.isDebug()) {
                Logger.logDebug("Player.steelTechnology() Player "+other.getName()+" NOTHING TO STEEL");
            }
        } else {
            if (Logger.isDebug()) {
                Logger.logDebug("Player.steelTechnology() Player "+other.getName()+" STEEL "+tech.getFullName());
            }
        }

        notifySpying(other, tech);
        other.notifySpySuccess(this, tech);
    }
    
    public void add(MoralChoice mc) throws ApplicationError {
    	if (mc == null) return;
    	if (moralChoices == null) moralChoices = new MoralChoiceListenerList();
    	moralChoices.add(this, mc);
    }
    
	public void addFunds(int amount) {
		funds += amount;
        if (Logger.isDebug()) {
            Logger.logDebug("Player.addFunds() player="+getName()+" amount = "+amount+" => "+funds);
        }
	}

	public boolean hasFunds(long amount) {
		return (funds >= amount);
	}
    
	public void addBoralium(int amount) {
		boralium += amount;
        if (Logger.isDebug()) {
            Logger.logDebug("Player.addBoralium() player="+getName()+" amount = "+amount+" => "+boralium);
        }
	}

	public boolean hasBoralium(int amount) {
		return (boralium >= amount);
	}
	
	/**
	 * The bonus to the maximum populations planets can support. This is includes 
	 * the bonuses from the players leaders and technologies. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOwnerReproductionBonus() throws ApplicationError {
        double bonus = 0.0;
        if (govenorLeader != null && govenorLeader.hasReproductionBonus()) {
            bonus += govenorLeader.getReproductionBonus();
        }
        if (biotechnologies != null) {
            bonus += biotechnologies.getReproductionBonus();
        }
        return bonus;
	}

	/**
	 * The bonus to the maximum populations planets can support. This is includes 
	 * the bonuses from the players character. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOriginReproductionBonus() throws ApplicationError {
        return character.getReproductionBonus();
	}

	public double getMaxTerraforming() {
        if (biotechnologies == null) {
            return 1.0;
        } else {
            return biotechnologies.getTerraformRating();
        }
	}

	/**
	 * The bonus to the maximum populations planets can support. This is includes 
	 * the bonuses from the players character. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOriginMaxPopulationBonus() throws ApplicationError {
        return character.getMaxPopulationBonus(); 
	}

	/**
	 * The bonus to the maximum populations planets can support. This is includes 
	 * the bonuses from the players leaders. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOwnerMaxPopulationBonus() throws ApplicationError {
        if (govenorLeader != null && govenorLeader.hasMaxPopulationBonus()) {
            return govenorLeader.getMaxPopulationBonus();  
        } else {
            return 0.0; 
        }
	}

	/**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus to quantity of factories a planet can build. This is the 
	 * bonuses from player character and leaders. 
	 */
	public double getFactoryPerPopulationBonus() throws ApplicationError { 
        if (govenorLeader != null && govenorLeader.hasFactoryPerPopulationBonus()) {
            return character.getFactoryPerPopulationBonus() + govenorLeader.getFactoryPerPopulationBonus();  
        } else {
            return character.getFactoryPerPopulationBonus(); 
        }
    }

	/**
	 * The bonus to planetary production based upon the origin. This is the bonuses from
	 * player character. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOriginProductionBonus(boolean includeLevelBonus) throws ApplicationError {
        return character.getProductionBonus(includeLevelBonus); 
	}

	/**
	 * The bonus to planetary production based upon the owner. This is the bonuses from
	 * leaders and moral choices. 
	 * A value of 0.1 implies a 10% bonus
	 */
	public double getOwnerProductionBonus() throws ApplicationError {
        double r = 0.0; 
        if (govenorLeader != null && govenorLeader.hasProductionBonus()) {
            r += govenorLeader.getProductionBonus();  
        }
        if (moralChoices != null) {
        	r += moralChoices.getProductionBonus();
        }
        return r;
	}
	
    public boolean isResourceful() { 
        return character.isResourceful(); 
    }

    public ResourcefulProcessor getResourcefulProcessor() { 
        return character.getResourcefulProcessor(); 
    }

    public boolean isImune(Weapon weapon) { 
        return character.isImune(weapon); 
    }

	/**
	 * The bonus to players research. This is includes the bonuses from 
	 * the players character and leaders. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getResearchBonus(boolean includeLevelBonus) throws ApplicationError {
        if (govenorLeader != null && govenorLeader.hasResearchBonus()) {
            return character.getResearchBonus(includeLevelBonus) + govenorLeader.getResearchBonus();  
        } else {
            return character.getResearchBonus(includeLevelBonus); 
        }
	}

	/**
	 * The bonus to a planets ecology. This is includes the bonuses from 
	 * the players leaders. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOwnerEcologyBonus() throws ApplicationError {
        if (govenorLeader != null && govenorLeader.hasEcologyBonus()) {
            return govenorLeader.getEcologyBonus();  
        } else {
            return 0.0; 
        }
	}

	/**
	 * The bonus to a planets ecology. This is includes the bonuses from 
	 * the players character. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOriginEcologyBonus() throws ApplicationError {
        return character.getEcologyBonus(); 
	}

	/**
	 * The bonus to a planets ecology. This is includes the bonuses from 
	 * the players leaders. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOwnerDevelopmentBonus() throws ApplicationError {
        if (govenorLeader != null && govenorLeader.hasDevelopmentBonus()) {
            return govenorLeader.getDevelopmentBonus();  
        } else {
            return 0.0; 
        }
	}

	/**
	 * The bonus to a planets ecology. This is includes the bonuses from 
	 * the players character. 
	 * A value of 0.1 implies an increase of 10%
	 */
	public double getOriginDevelopmentBonus(boolean includeLevelBonus) throws ApplicationError {
        return character.getDevelopmentBonus(includeLevelBonus); 
	}


	/**
	 * The bonus to a planets ecology. This is includes the bonuses from 
	 * the players character and leaders. 
	 * A value of 0.1 implies an increase of 10%
	 */
    public double getSpyBonus() throws ApplicationError { 
        if (govenorLeader != null && govenorLeader.hasSpyBonus()) {
            return character.getSpyBonus() + govenorLeader.getSpyBonus();  
        } else {
            return character.getSpyBonus(); 
        }
    }

    public int getMaxPlayerTradeRoutes() throws ApplicationError {
        if (govenorLeader != null && govenorLeader.hasMaxTradeRoutes()) {
            return character.getMaxPlayerTradeRoutes() + govenorLeader.getMaxTradeRoutes();  
        } else {
            return character.getMaxPlayerTradeRoutes();
        }
    }

	/**
	 * The bonus to a planets ecology. This is includes the bonuses from 
	 * the players character and leaders. 
	 * A value of 0.1 implies an increase of 10%
	 */
    public double getTradingBonus() throws ApplicationError { 
        if (govenorLeader != null && govenorLeader.hasMaxTradeRoutes()) {
            return character.getTradingBonus() + govenorLeader.getTradingBonus();  
        } else {
            return character.getTradingBonus();
        }
    }

	/**
	 * The maximum quantity of trade routes out of each solar system can be supported by this player. 
	 * A value of 1 implies player may have one trade route per solar system
	 */
    public int getMaxSolarSystemTradeRouteDistance() throws ApplicationError { 
    	//int max = getPlayerCharacter().getMaxSolarSystemTradeRoutes();
        if (humanities != null) {
            return humanities.getMaxSolarSystemTradeRouteDistance();  
        } else {
            return 0;
        }
    }

	/**
	 * The maximum quantity of trade routes out of each solar system can be supported by this player. 
	 * A value of 1 implies player may have one trade route per solar system
	 */
    public int getMaxSolarSystemTradeRoutes() throws ApplicationError { 
    	int max = getPlayerCharacter().getMaxSolarSystemTradeRoutes();
        if (humanities != null) {
            return max + humanities.getMaxSolarSystemTradeRoutes();  
        } else {
            return max;
        }
    }

    /**
	 * A value of 0.1 implies an increase of 10%
     * @return The bonus to a players influence from his technology, character and 
     */
    public double getInfluenceBonus() {
    	double bonus = getPlayerCharacter().getEloquenceBonus();
    	if (factories != null) {
    		bonus += factories.getPropaganda();
    	}
    	if (moralChoices != null) {
    		bonus += moralChoices.getInfluenceBonus();
    	}
    	return bonus;
    }


    /**
     * @return The psi rating of this player from its character
     */
    public int getOriginPsiRating() throws ApplicationError {
		return character.getPsiRating();
    }

    /**
     * Should be a value between 0 and 1 where a value of 0 implies no change,
     * and a value of 1 implies certainty.
     * @return The chance that this player will discover any technology it receives as salvage.
     */
	public double getChanceToDiscoverSalvagedTechnology() {
    	double probability = getPlayerCharacter().getChanceToDiscoverSalvagedTech();
    	if (laboratories != null) {
    		probability += laboratories.getChanceToDiscoverSalvagedTechBonus();
    	}
    	if (probability > 1) probability = 1.0;
    	if (probability < 0) probability = 0.0;
    	return probability;
	}
    
    /**
     * @return The psi rating of this player from its leaders, technologies
     * and resources
     */
    public int getOwnerPsiRating() throws ApplicationError {
        int rating = 0;
        if (govenorLeader != null && govenorLeader.hasPsiRating()) {
            rating = govenorLeader.getPsiRating();  
        }
		if (biotechnologies != null) {
			rating += biotechnologies.getPsiRating();
		}
		return rating + getResourceCache().getPsiBonus();
    }

    /**
	 * A value of 0.1 implies an increase of 10%
     * @return The ground combat ability this player possess from his character and leaders
     */
    public double getAssaultBonus() throws ApplicationError { 
        if (generalLeader != null && generalLeader.hasAssaultBonus()) {
            return character.getAssaultBonus() + generalLeader.getAssaultBonus();  
        } else {
            return character.getAssaultBonus(); 
        }
    }

    public String getAssaultBonusDescription() throws ApplicationError {
    	double dam = 0.0;
    	if (getWeapons() != null && getWeapons().getBestGroundWeaponDamage() != null) {
        	dam = getWeapons().getBestGroundWeaponDamage().getDamage();
    	}
    	double playerBonus = character.getAssaultBonus();
    	double leaderBonus = 0.0;
        if (generalLeader != null && generalLeader.hasAssaultBonus()) {
        	leaderBonus = generalLeader.getAssaultBonus();  
        }
        return L.ASSAULT+" = "+Formatter.fraction(dam)+
        		" + "+L.PLAYER_BONUS+" "+Formatter.percentLabelNoSign(playerBonus) +
        		" + "+L.LEADER_BONUS+" "+Formatter.percentLabelNoSign(leaderBonus) ;
    }

    public int getOriginCloakLevel() throws ApplicationError { 
        return character.getCloakLevel(); 
    }

    public int getOwnerCloakLevel() throws ApplicationError {
        int level = 0;
        if (generalLeader != null && generalLeader.hasCloakLevel()) {
            level = generalLeader.getCloakLevel();  
        }
        return level + getResourceCache().getCloakBonus();
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for craft shift movement on the battle field from the
	 * players character
     */
    public double getOriginSpeedBonus() throws ApplicationError {
        return character.getSpeedBonus();
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for craft shift movement on the battle field from the
	 * players leaders and resources
     */
    public double getOwnerSpeedBonus() throws ApplicationError {
        double leaderBonus = 0.0; 
        if (generalLeader != null && generalLeader.hasSpeedBonus()) {
            leaderBonus = generalLeader.getSpeedBonus();  
        }
        return leaderBonus + getResourceCache().getSpeedBonus();
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for craft shift movement on the battle field from the
	 * players character
     */
    public double getOriginBattleSpeedBonus() throws ApplicationError {
    	if (character == null) {
    		return 0.0;
    	} else {
    		return character.getBattleSpeedBonus(); 
    	}
    }
    
    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for craft shift movement on the battle field from the
	 * players leaders and resources
     */
    public double getOwnerBattleSpeedBonus() throws ApplicationError { 
        double leaderBonus = 0.0; 
        if (generalLeader != null && generalLeader.hasBattleSpeedBonus()) {
            leaderBonus = generalLeader.getBattleSpeedBonus();  
        }
        return leaderBonus + getResourceCache().getSpeedBonus();
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for craft turning movement on the battle field from the
	 * players character
     */
    public double getOriginBattleTurningBonus() throws ApplicationError { 
    	if (character == null) {
    		return 0.0;
    	} else {
    		return character.getBattleTurningBonus();
    	}
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for craft turning movement on the battle field from the
	 * players leaders and resources
     */
    public double getOwnerBattleTurningBonus() throws ApplicationError { 
        double leaderBonus = 0.0; 
        if (generalLeader != null && generalLeader.hasBattleTurningBonus()) {
            leaderBonus = generalLeader.getBattleTurningBonus();  
        }
        return leaderBonus + getResourceCache().getSpeedBonus();
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for craft turning movement on the battle field from the
	 * players character
     */
    public double getOriginInitiativeBonus() throws ApplicationError { 
    	if (character == null) {
    		return 0.0;
    	} else {
    		return character.getInitiativeBonus();
    	}
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for craft turning movement on the battle field from the
	 * players leaders and resources
     */
    public double getOwnerInitiativeBonus() throws ApplicationError { 
        if (generalLeader != null && generalLeader.hasInitiativeBonus()) {
            return generalLeader.getInitiativeBonus();  
        } else {
            return 0.0; 
        }
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for a crafts weapons damage from the players leaders 
	 * and moral choices
     */
    public double getOwnerDamageBonus() throws ApplicationError {
        double r = 0.0; 
        if (generalLeader != null && generalLeader.hasDamageBonus()) {
            r += generalLeader.getDamageBonus();  
        }
		if (moralChoices != null) {
			r += moralChoices.getDamageBonus();
        }
        return r + getResourceCache().getDamageBonus();
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for the amount of damage a craft can sustain before being 
	 * destroyed from the players character
     */
    public double getOriginMaxDamageBonus() throws ApplicationError { 
    	if (character == null) {
    		return 0.0;
    	} else {
    		return character.getMaxDamageBonus();
    	}
    }

    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for the amount of maximum charge a energy shields can hold during a battle.
     */
    public double getOwnerShieldPowerBonus() throws ApplicationError {
        double r = 0.0; 
        if (generalLeader != null && generalLeader.hasShieldPowerBonus()) {
            r += generalLeader.getShieldPowerBonus();  
        }
        return r + getResourceCache().getShieldBonus();
    }


    /**
	 * A value of 0.1 implies an increase of 10%
	 * @return The bonus for the amount of damage a craft can sustain before being 
	 * destroyed from the players leaders and moral choices
     */
    public double getOwnerMaxDamageBonus() throws ApplicationError { 
        double leaderBonus = 0.0; 
        if (generalLeader != null && generalLeader.hasMaxDamageBonus()) {
            leaderBonus = generalLeader.getMaxDamageBonus();  
        }
        return leaderBonus + getResourceCache().getMaxDamageBonus();
    }


	public ResearchProject getResearchProject() {
		return researchProject;
	}
    
    public boolean hasResearchProject() {
        return researchProject != null && researchProject.getTechnology() != null;
    }

	public void addResearch(int amount) throws ApplicationError {
		if (researchProject == null) {
			notifyNoResearchProject(false);
		}
		if (researchProject == null) {
			addFunds((int)(amount*0.25));
		} else {
            int remainingWork = researchProject.addResearch(amount);
            if (remainingWork > 0 && remainingWork < amount) {
                addResearch((int)(remainingWork));
            }
        }
	}

	public void finishResearchProject(ResearchProject researchProject, boolean success, boolean fromSalvage) throws ApplicationError {
		// Add this technology to this players discoveries
		Technology tech = researchProject.getTechnology();
		if (Logger.isDebug()) {
            Logger.logDebug("Player.finishResearchProject() tech = "+tech.getName()+" success="+success);
        }

		this.researchProject = null;
		
		if (success) {
			lastDiscovery = new ResearchScriptOption(tech);
	        addTechnology(tech);
	
			// Start a new project
			notifyTechnologyDiscovered(tech, fromSalvage);
		} else {
			MotEngine.instance.broadcast(new MessageEF(L.DISCOVER_FROM_SALVAGE_+tech.getName()+L._FAILED), this);
			notifyNoResearchProject(true);
		}
	}

    /**
     * @param techs The technologies that this player may have discovered.
     * @return Returns true if and only if this player has discovered all the technologies in techs.
     */
    public boolean hasDiscovered(TechnologyList techs) throws ApplicationError {
        if (techs == null) {
            return true;
        } else {
            Technology [] techArray = techs.getAllAsArray();
            if (techArray == null) {
                return true;
            } else {
                return hasDiscovered(techArray);
            }
        }
    }
    
    /**
     * @param techs The technologies that this player may have discovered.
     * @return Returns true if and only if this player has discovered all the technologies in techs.
     */
    public boolean hasDiscovered(Technology [] techs) throws ApplicationError {
        if (techs == null) return true;
        for (int i=0; i<techs.length; i++) {
            if (!hasDiscovered(techs[i])) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * @param techs The technologies that this player may have discovered.
     * @return Returns true if and only if this player has discovered one or more the technologies in techs.
     */
    public boolean hasDiscoveredAny(TechnologyList techs) throws ApplicationError {
        if (techs == null) {
            return false;
        } else {
            Technology [] techArray = techs.getAllAsArray();
            if (techArray == null) {
                return false;
            } else {
                return hasDiscoveredAny(techArray);
            }
        }
    }
    
    /**
     * @param techs The technologies that this player may have discovered.
     * @return Returns true if and only if this player has discovered one or more the technologies in techs.
     */
    public boolean hasDiscoveredAny(Technology [] techs) throws ApplicationError {
        if (techs == null) return false;
        for (int i=0; i<techs.length; i++) {
            if (hasDiscovered(techs[i])) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * @param techType The tech type that this player may have discovered.
     * @param techName The tech name that this player may have discovered.
     * @return Returns true if and only if this player has discovered technology tech.
     */
    public boolean hasDiscovered(String techType, String techName) throws ApplicationError {
        Technology t = Technologies.getTechnology(techType, techName);
        return hasDiscovered(t);
    }

    /**
     * @param tech The tech that this player may have discovered.
     * @return Returns true if and only if this player has discovered technology tech.
     */
    public boolean hasDiscovered(Technology tech) throws ApplicationError {
        if (tech == null) return true;
		if (tech instanceof Factory) {
            if (factories == null) return false;
			return factories.contains(tech);
		} else if (tech instanceof Laboratory) {
            if (laboratories == null) return false;
            return laboratories.contains(tech);
		} else if (tech instanceof Biotechnology) {
            if (biotechnologies == null) return false;
            return biotechnologies.contains(tech);
		} else if (tech instanceof Hull) {
            if (hulls == null) return false;
            return hulls.contains(tech);
		} else if (tech instanceof Computer) {
            if (computers == null) return false;
            return computers.contains(tech);
		} else if (tech instanceof Weapon) {
            if (weapons == null) return false;
            return weapons.contains(tech);
		} else if (tech instanceof Shield) {
            if (shields == null) return false;
            return shields.contains(tech);
		} else if (tech instanceof Scanner) {
            if (scanners == null) return false;
            return scanners.contains(tech);
		} else if (tech instanceof Special) {
            if (specials == null) return false;
            return specials.contains(tech);
		} else if (tech instanceof Engine) {
            if (engines == null) return false;
            return engines.contains(tech);
		} else if (tech instanceof Theory) {
            if (theories == null) return false;
            return theories.contains(tech);
		} else if (tech instanceof Module) {
            if (modules == null) return false;
            return modules.contains(tech);
		} else if (tech instanceof Humanity) {
            if (humanities == null) return false;
            return humanities.contains(tech);
		} else if (tech instanceof MiscTechnology) {
            if (miscTechnologies == null) return false;
            return miscTechnologies.contains(tech);
		} else {
			throw new ApplicationError("Player.hasDiscovered() Invalid technology type "+tech.getClass().getName());
		}
    }
    
    /**
     * @return The craft design for fighters to be used by this player
     */
    public CraftDesign getFighterDesign() throws ApplicationError {
        if (fighterDesigner == null) {
            fighterDesigner = new CraftDesignerFighter();
        }
        return fighterDesigner.designCraft(this);
    }
    
    public CraftDesign getSuicideFighterDesign() throws ApplicationError {
        if (fighterDesigner == null) {
            fighterDesigner = new CraftDesignerFighter();
        }
        return fighterDesigner.designSuicideCraft(this);
    }
    
    /**
     * For each technology that has been researched already it locates technology that is available for
     * research from that discovery, but, has not yet been discovered.
     * This is a complex operation, see TechnologyTree.java for details.
     */
    public Technology [] getTechnologyAvailableForResearch() throws ApplicationError {
    	if (technologiesAvailableForResearch == null) {
    		technologiesAvailableForResearch = TechnologyTree.getInstance().getTechnologyAvailableForResearch(this);
    	}
    	return technologiesAvailableForResearch;
    }
    
    public void flushTechnologyAvailableForResearch() {
    	technologiesAvailableForResearch = null;
		outdatedTechnologiesAvailableForResearch = null;
    }

    /**
     * @return All the technologies that this player could research now (dependencies have been
     * Satisfied - see technology tree for details) but they are on the missed list.
     */
    public Technology [] getMissedTechnology() throws ApplicationError {
        return TechnologyTree.getInstance().getVisibleMissedTechnology(this);
    }

    /**
     * Notify this user that one of its planets does not have a project. This is called at the 
     * end of every turn for each planet that does not have a project. The reason may be null,
     * but otherwise it is a user oriented message explaining why this planet has no project.
     * It is intended to be a description of the project the planet has just completed - This
     * may be used by the interface to display a project completed message!
     * @param planet The planet that does not have a project.
     * @param reason An optional reason explaining why it does not have a project.
     */
	public abstract void notifyNoProject(Planet planet, String reason) throws ApplicationError, ValidationError;

	public abstract void notifyNoResearchProject(boolean updateTechsToResearch) throws ApplicationError;
	public abstract void notifyEcologyFinished(Planet planet) throws ApplicationError, ValidationError;
	public abstract void notifyFactoriesFinished(Planet planet) throws ApplicationError, ValidationError;

	public void notifyColonyLost(Planet colony, Player aggressor, boolean genocide) throws ApplicationError {
		if (MotEngine.instance != null) {			
			MotEngine.instance.getMonsterPlayer().notifyMonsterColonyLost(this, colony, aggressor);
		}
	}

	public void notifyTechnologyDiscovered(Technology tech, boolean fromSalvage) throws ApplicationError {
	}
	
	public void notifyPlanetColonised(Planet planet) throws ApplicationError {
        if (govenorLeader != null) {
            govenorLeader.notifyColonisePlanet(this);
        }
    }
    
	public void notifyBattleEnded(Battle battle, boolean amWinner) throws ApplicationError {
        qtyBattles++;
        if (amWinner) qtyBattlesWon++;
        if (generalLeader != null) {
            generalLeader.notifyBattleEnded(amWinner, this);
        }
    }

	/**
	 * Called at the end of a turn when a trade route is found to be invalid (usually sector between 
	 * planets is controlled by another player)
	 * @param tradeRoute The trade route that has already been deleted that is invalid
	 * @throws ApplicationError
	 */
	public abstract void notifyTradeRouteInvalid(TradeRoute tradeRoute) throws ApplicationError;

    public abstract Planet getPlanetToAttack(Planet [] planets) throws ApplicationError;
    
    public Planet getPlanetToAttack(SolarSystem ss) throws ApplicationError {
        if (ss == null) {
            return null;
        } else {
            return getPlanetToAttack(ss.getPlanets());
        }
    }

    /**
     * Called when a Space Station has been built to find out which planet the owner would
     * like to deploy the station to.
     */
    public abstract Planet getPlanetToDeploySpaceStation(Transport transport, Orbital station) throws ApplicationError;

    /**
     * Called when a Space Station has been built to find out which planet the owner would
     * like to deploy the station to.
     */
    public abstract StellarObject getStellarObjectToDeployOutpost(Transport transport, Orbital station) throws ApplicationError;
    
    /**
     * Called when a Module has been built to find out which space station the owner would
     * like to add the module to.
     */
    public abstract Planet getStationToAddModule(Transport transport, Planet origin, Module module) throws ApplicationError;
    
	/**
	 * Called when a craft has been built or is idle to get its next task.
	 */
	public abstract void setNextCraftOrders(Craft craft) throws ApplicationError;

	/**
	 * Called when a fleet has been built or is idle to get its next task.
	 */
	public abstract void setNextFleetOrders(Fleet fleet) throws ApplicationError;

	/**
	 * Called when a craft in battle needs the next orders.
	 * NOTE - You should not use this directly. Use BattleCraft.getNextOrders()
	 * because this does not take the controller of a battle craft into account.
	 */
	public abstract void setNextBattleOrders(BattleCraft craft) throws ApplicationError;

	/**
     * Called when another player has been caught spying to notify the victim.
     * @param player The player caught spying
     * @param tech The technology stolen by the player
     */
	public abstract void notifySpying(Player player, Technology tech) throws ApplicationError ;

	/**
     * Called when this player has discovered a technology through a successful spy role from player.
     * @param player The player whom you have spied from
     * @param tech The technology stolen from player
     */
	public abstract void notifySpySuccess(Player player, Technology tech) throws ApplicationError;

	/**
     * Called when another player has been caught sabotaging installations to notify the victim.
     * @param planet The planet that has been sabotaged
     * @param player The player caught sabotaging
     */
	public abstract void notifySabotage(Planet planet, Player player, int productionSabotaged, int factoriesSabotaged, int populationSabotaged) throws ApplicationError ;

	/**
	 * Called when a planet in battle needs the next orders.
	 */
	public abstract void setNextBattleOrders(BattlePlanet planet) throws ApplicationError;
    
	/**
	 * Called when this player is being offered a leader.
	 */
	public abstract void offerGeneralLeaderTo(GeneralDetails leader) throws ApplicationError;
    
	/**
	 * Called when this player is being offered a leader.
	 */
	public abstract void offerGovenorLeaderTo(GovenorDetails leader) throws ApplicationError;
    
	/**
	 * Get a rating of the power for this player.
	 * Currently uses the total population for this player.
	 */
	public long getPowerRating() throws ApplicationError {
		return getTotalPopulation();
	}

    /**
     * @return The quantity of spies this race has built.
     * Maintenance costs are due on every spy this player builds.
     */
    public abstract int getQtyBuiltSpies() throws ApplicationError;
    
    /**
     * @return The quantity of spies this race controls.
     * This is the the quantity of spies built by this player plus the quantity of spies
     * given to this player from its influence (see Computers technology for details).
     */
    public int getQtySpies() throws ApplicationError {
        int influence = (computers == null) ? 0 : computers.getInfluence();
        return getQtyBuiltSpies() + influence;
    }
    
    /**
     * NOTE - The value returned comes from a cached value in Relations. For details
     * of how it is actually calculated at the end of each turn see _hasDiplomaticRelationsWith().
     * @return True iff this player has formed diplomatic relations with the player
     * passed in.
     */
    public boolean hasDiplomaticRelationsWith(Player player) throws ApplicationError {
    	return MotEngine.instance.getRelationsNotNull().hasDiplomaticRelations(this, player);
    }
    
    /**
     * Used to determine if a user player can communicate with another player using 
     * the diplomacy screen. Likewise no player can form trade relations unless
     * diplomacy has been established. 
     * 
     * Diplomatic relations are defined between two players when their combined scanner
     * strength is greater than the distance between the two players closest stellar 
     * objects This player has scan strength 100 and the other player has scan 
     * strength 150 then they can form diplomatic relations upto 250 apart.
     * 
     * @param player The player this player may have diplomatic relations with.
     * @return True if and only if these two players have diplomatic relations.
     */
    public boolean _hasDiplomaticRelationsWith(Player player) throws ApplicationError {
        StellarObject [] sos = getStellarObjects();
        if (sos == null || sos.length == 0) {
            // I have no solar systems - I must have just lost my last colony, at the end 
            // of this turn I will be removed!
            return false;
        }
        int minDist = Integer.MAX_VALUE;
        for (int i=0; i<sos.length; i++) {
            Location loc = sos[i].getLocation();
            StellarObject so = player.getClosestStellarObject(loc);
            if (so == null) return false;
            int dist = loc.getDistanceTo(so.getLocation());
            if (dist < minDist) {
                minDist = dist;
            }
        }
        int maxDist = 0;
        if (scanners != null) {
            maxDist = scanners.getStrength();
        }
        Scanners scannersOther = player.getScanners();
		if (scannersOther != null) {
            maxDist += scannersOther.getStrength();
        }
        return (maxDist >= minDist);
    }

    /**
     * @see hasDiplomaticRelationsWith(Player) for details
     * @return The has of all players this player has diplomatic relations with.
     */
    public Player [] getPlayerWithDiplomaticRelations() throws ApplicationError {
        Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
    	ArrayList r = null;
    	for (int i=0; i<players.length; i++) {
    		if (hasDiplomaticRelationsWith(players[i])) {
    			if (r == null) r = new ArrayList();
    			r.add(players[i]);
    		}
    	}
    	if (r == null) {
    		return null;
    	} else {
    		return (Player []) r.toArray(new Player[r.size()]);
    	}
    }
    
    /**
     * Inform players in the game that a war has started between other players
     * @param victim The player who the aggressor has declared war on
     * @param agressor The player who has started the war
     */
	public void notificationOfWar(Player agressor, Player victim) throws ApplicationError { }
	
    /**
     * Creates a WAR treaty between this player and the player passed in. War treaties do not
     * need accepting from the other party so this will not get a response.
     * @param player The player who has declared war (the aggressor)
     * @throws mot.common.util.ApplicationError
     */
	public void startWar(Player player, String reason, boolean becauseOfAlliance) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Player.declareWar() player "+player.getName()+" declares war on "+getName());
        }
		MotEngine.instance.getRelations().setWar(player, this, !becauseOfAlliance);
		if (!becauseOfAlliance) {
	        Player [] players = MotEngine.instance.getPlayers().getPlayers();
	        for (int i=0; i<players.length; i++) {
	        	if (players[i] != this && players[i] != player) {
	        		players[i].notificationOfWar(player, this);
	        	}
	        }
		}
	}

	public boolean isWar(Player other) throws ApplicationError {
		Relation rel = getRelationshipWith(other);
		if (rel == null) return false;
		return rel.isWar();
	}
	
	/**
	 * Called when the player passed in has submitted a diplomatic request to this player.
	 * A Diplomatic request can be a declaration of war, demand for assets, swap assets, peace treaty etc....
     * @param Player The player making the request
     * @param proposal The assets offered, wanted and the treaties being proposed by player
	 */
    public void diplomaticRequest(Player player, DiplomaticProposal proposal, String message) throws ApplicationError {
    	if (proposal.isWarWithNoDemand()) {
   			startWar(player, message, false);
    		proposal.setExecuted(true);
	    	MotEngine.instance.getRelations().getRelationshipNotNull(this, player).addDiplomaticHistoryEvent(proposal, player);
    	} else if (proposal.isFreeGift() && !isComputerPlayer()) {
    		swapAssets(player, proposal.getOffered(), null);
    		proposal.setExecuted(true);
	    	MotEngine.instance.getRelations().getRelationshipNotNull(this, player).addDiplomaticHistoryEvent(proposal, player);
    	}
    	diplomaticRequestNotification(player, proposal, message);
    }

    protected abstract void diplomaticRequestNotification(Player player, DiplomaticProposal proposal, String message) throws ApplicationError;

    /**
     * Called containing the response to a diplomatic request.
     * @param player The player that made the diplomatic request to this player.
     * @param proposal The assets offered, wanted and the treaties being proposed by player
     * @param gift A textual response (e.g. a thank you message)
     */
    public void diplomaticResponse(Player player, DiplomaticProposal proposal, boolean accepted, String message) throws ApplicationError {
    	if (proposal.isWarWithDemand()) {
    		if (!accepted) {
    			if (isWar(player)) {
    				Logger.logWarning("Player.diplomaticResponse() Player "+this+" already at war with "+player+" - "+proposal+" was ignored"); 
    			} else {
    				startWar(player, message, false);
    			}
    		} else {
    			PlayerAssets offered = proposal.getOffered();
    			PlayerAssets wanted = proposal.getWanted();
    			swapAssets(player, offered, wanted);    			
    		}
        	MotEngine.instance.getRelations().getRelationshipNotNull(this, player).addDiplomaticHistoryEvent(proposal, this);
    	} else {
    		if (accepted && !proposal.isExecuted()) {
    			if (proposal.isPeace()) {
    				MotEngine.instance.getRelations().setPeace(this, player);
    			} else if (proposal.isNonAggression()) {
    				MotEngine.instance.getRelations().setNonAgression(this, player);
    			} else if (proposal.isAlliance()) {
    				MotEngine.instance.getRelations().setAliance(this, player);
    			} else if (proposal.isResearch()) {
    				MotEngine.instance.getRelations().setResearch(this, player);
    			} else if (proposal.isTrade()) {
    				MotEngine.instance.getRelations().setTrade(this, player);
    			}
    			PlayerAssets offered = proposal.getOffered();
    			PlayerAssets wanted = proposal.getWanted();
    			if (offered != null || wanted != null) {
    				player.swapAssets(this, offered, wanted);
    			}
    	    	MotEngine.instance.getRelations().getRelationshipNotNull(this, player).addDiplomaticHistoryEvent(proposal, this);
    		}
    	}
        diplomaticResponseNotification(player, proposal, accepted, message);
    }
    
    protected abstract void diplomaticResponseNotification(Player player, DiplomaticProposal proposal, boolean accepted, String message) throws ApplicationError;
    

	/**
     * Performs a transfer of assets between players (gift, demand or swap). This player is
     * will get the offered assets while the player passed in will be giving the offered assets.
     * @throws mot.common.util.ApplicationError
     * @param wanted The assets to be given by this player to the player in parameter player.
     * @param offered The assets to be given by the player in parameter player to this player.
     * @param player The player that has made the swap, gift or demand for assets.
     */
    public void swapAssets(Player player, PlayerAssets offered, PlayerAssets wanted) throws ApplicationError {
        if (offered != null) {
            if (offered.getTechnologies() != null) {
                addTechnologies(offered.getTechnologList());
            }
            if (offered.getMoney() > 0) { 
                addFunds(offered.getMoney());
                player.addFunds(-offered.getMoney());
            }
            String planetName = offered.getPlanet();
			if (planetName != null) {
				Planet planet = MotEngine.instance.getPlanets().getPlanet(planetName);
				planet.changeOwner(this);
            }
	        NameList craftNames = offered.getCraftNames();
	        if (craftNames != null && craftNames.getQuantity() > 0) {
	        	String[] cna = craftNames.getNamesAsArray();
	        	for (int i = 0; i < cna.length; i++) {
	        		Craft craft = MotEngine.instance.getCrafts().getCraft(cna[i]);
	        		craft.changeOwner(this);
	        	}
	        }
        }
        
        if (wanted != null) {
            if (wanted.getTechnologies() != null) {
                player.addTechnologies(wanted.getTechnologList());
            }
            if (wanted.getMoney() > 0) {
                player.addFunds(wanted.getMoney());
                addFunds(-wanted.getMoney());
            }
            String planetName = wanted.getPlanet();
			if (planetName != null) {
				Planet planet = MotEngine.instance.getPlanets().getPlanet(planetName);
				planet.changeOwner(player);
            }
	        NameList craftNames = wanted.getCraftNames();
	        if (craftNames != null && craftNames.getQuantity() > 0) {
	        	String[] cna = craftNames.getNamesAsArray();
	        	for (int i = 0; i < cna.length; i++) {
	        		Craft craft = MotEngine.instance.getCrafts().getCraft(cna[i]);
	        		craft.changeOwner(player);
	        	}
	        }
        }
    }
    
    public void hasAssets(PlayerAssets assets) throws ValidationError, ApplicationError {
        int money = assets.getMoney();
		if (money > 0) {
            if (getFunds() < money || money < 0) {
                //throw new ValidationError("Player "+player.getName()+" does not have enough funds");
                throw new ValidationError(L.PLAYER_+getName()+L.DOES_NOT_AGREE);
            }
        }
        String planetName = assets.getPlanet();
		Planet planet = planetName == null ? null : MotEngine.instance.getPlanets().getPlanet(planetName);
		if (planet != null) {
            if (!planet.isOwnedBy(this)) {
                //throw new ValidationError("Player "+player.getName()+" does not own planet "+assets.getPlanet().getName());
                throw new ValidationError(L.PLAYER_+getName()+L.DOES_NOT_AGREE);
            }
            if (planet.isMiningColony()) {
                throw new ValidationError(L.INVALID_ASSET_TRANSFER);
            }
        }
        TechnologyList technologies = assets.getTechnologList();
		if (technologies != null) {
            Technology [] techs = technologies.getAllAsArray();
            if (techs != null) {
                for (int i=0; i<techs.length; i++) {
                    if (!hasDiscovered(techs[i])) {
                        //throw new ValidationError("Player "+player.getName()+" does not have technology "+techs[i].getName());
                        throw new ValidationError(L.PLAYER_+getName()+L.DOES_NOT_AGREE);
                    }
                }
            }
        }
        if (planet != null) {
        	if (planet.getOwner() == null || planet.getOwner() != this) {
        		throw new ValidationError(L.THIS_DOES_NOT_BELONG_TO_YOU);
        	}
        	if (planet.getPopulation() == 0.0 && planet.getOrbital() != null) {
        		throw new ValidationError("Cannot give a mining colony away (give the planet that owns it)");
        	}
        }
        NameList craftNames = assets.getCraftNames();
        if (craftNames != null && craftNames.getQuantity() > 0) {
        	String[] cna = craftNames.getNamesAsArray();
        	for (int i = 0; i < cna.length; i++) {
        		Craft craft = MotEngine.instance.getCrafts().getCraft(cna[i]);
        		if (craft == null) {
        			throw new ValidationError("Craft "+cna[i]+" does not exist");
        		}
        		if (!craft.isOwnedBy(this)) {
        			throw new ValidationError("Craft "+cna[i]+" is not owned by "+getName());
        		}
			}
        }
    }
    
    /**
     * This is called by the Moral Policy Advisor to recommend the adoption of a moral choice
     * to the player. NOTE - Just because a moral choice is recommended DOES NOT mean it is a good
     * idea. Computer players need to asses the decision (probably just refer to the character!).
     * @param mc The moral choice being proposed
     */
    public abstract void recommendMoralChoice(MoralChoice mc) throws ApplicationError;

    /**
     * Transfer qty colonists from planet p1 to planet p2. See Transport for details of speed
     * and how the transfers occur.
     * @param from Planet from which to take the colonists
     * @param to Planet to which to bring the colonists
     * @param qty The quantity of colonists to move
     */
    public void transferPopulation(Planet from, Planet to, int qty) {
        // create Trandport Class with attribute TransportTask  which has implementation
        // TransportTaskMoveColonists and can be extended to TransPortTaskTransferMaterials etc
        // for space stations - free transports are those that do not have a task.
    }
    
    /**
     * @param includeTrade Whether to include trade income in this list.
     * @return The total income earnt by this player at the end of this turn.
     */
    public int getTotalIncome(boolean includeTrade) throws ApplicationError {
        Planet [] planets = getPlanets();
        if (planets == null) return 0;
        int total = 0;
        for (int i=0; i<planets.length; i++) {
            total += planets[i].getMoneyOutput();
        }
        if (includeTrade) {
            total += getTradeIncome();
        }
        return total;
    }
    
	/**
	 * @returns The total maintenaince cost for all items owned by this player.
     * This is the sum of the upkeep cost for 
     *  - Each planet's weapons, shields and scanners.
     *  - For each craft
     *  - For the players spies
	 */
	public int getTotalMaintenanceCost() throws ApplicationError {
        return getCraftlMaintenanceCost() + getPlanetsMaintenanceCost() + getSpiesMaintenanceCost() + getSpyRecruitCost();
    }

	/**
	 * @returns The total maintenaince for all crafts owned by this player.
	 */
	public int getCraftlMaintenanceCost() throws ApplicationError {
        int cost = 0;
        Craft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                cost += crafts[i].getMaintenanceCost();
            }
        }
        return cost;
    }

	/**
	 * @returns The total maintenaince for all planets defenses by this player.
	 */
	public int getPlanetsMaintenanceCost() throws ApplicationError {
        int cost = 0;
        Planet [] planets = getPlanets();
        if (planets != null) {
            for (int i=0; i<planets.length; i++) {
                cost += planets[i].getMaintenanceCost();
            }
        }
        return cost;
    }
    
    /**
     * @return The amount of monet allocated to recruiting spies pur turn.
     * NOTE - Only Human players currently have to recruite spies so this is over ridden
     * in UserPlayer - but for all other players this is 0. 
     */
    public int getSpyRecruitCost() {
        return 0;
    }
    
	/**
	 * @returns The total maintenaince for all spies owned by this player.
	 */
	public int getSpiesMaintenanceCost() throws ApplicationError {
        return getQtyBuiltSpies() * MAINTENAINCE_COST_PER_SPY;
    }
    
    /**
     * @return The total income earnt by this player at the end of this turn from trading.
     */
    public int getTradeIncome() throws ApplicationError {
        return MotEngine.instance.getRelationsNotNull().getTradeIncome(this);
    }

    /**
     * This is the total income of this player minus the total maintenaince 
     * costs for this player.
     * @return The amount of money this player will earn close this turn.
     */
    public int getNetIncome() throws ApplicationError {
        return getTotalIncome(true) - getTotalMaintenanceCost(); 
    }
    
    /**
     * @return The total reseaech work performed each turn by this player on research.
     * NOTE - This may not be the actual work performed at the end of the turn as settings change 
     * and work produced depends on many settings including the percentage output designated to 
     * research on each planet.
     */
    public int getTotalResearch() throws ApplicationError {
        Planet [] planets = getPlanets();
        if (planets == null) return 0;
        int total = 0;
        for (int i=0; i<planets.length; i++) {
            total += planets[i].getResearchOutput();
        }
        return total;
    }
    
    /**
     * @return The total population for this player.
     */
    public int getTotalPopulation() throws ApplicationError {
        Planet [] planets = getPlanets();
        if (planets == null) return 0;
        double total = 0;
        for (int i=0; i<planets.length; i++) {
            total += planets[i].getPopulation();
        }
        return (int) total;
    }
    

    /**
     * @return The total production controlled by this player.
     */
    public int getTotalProduction() throws ApplicationError {
		int qty = 0;
		Planet planets [] = getPlanets();
		if (planets == null) return 0;
		for (int i=0; i<planets.length; i++) {
			qty += (int) planets[i].getProductionOutput();
		}

		return qty;
    }
    
    /**
     * Marks the turn as ended for this player. Also assigns all unused spies to the spy role of defend.
     * @throws mot.common.util.ValidationError
     * @throws mot.common.util.ApplicationError
     */
	public void endTurn() throws ValidationError, ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Player.endTurn() player="+getName());
        }
        
        if (hasLostGame()) {
        	if (!MotEngine.instance.getDeadPlayersNotNull().contains(this)) {
        		// This player has already been added to the list of dead players (ComputerPlayer.endTurn() calls strategist)
        		destroy();
        	}
            return;
        }
        
		int boraliumFromTrade = MotEngine.instance.getRelationsNotNull().getBoraliumPerTurnWith(this);
		if (boraliumFromTrade > 0) addBoralium(boraliumFromTrade);

        int qtySpies = getQtySpies();
        int used = MotEngine.instance.getSpiesNotNull().getQtySpies(this);
        if (qtySpies > used) {
            MotEngine.instance.getSpies().addDefendingSpies(this, qtySpies-used);
        } else if (used > qtySpies) {
            MotEngine.instance.getSpies().tooManySpies(this, qtySpies);
            TooManySpiesEF event = new TooManySpiesEF(this);
            MotEngine.instance.broadcast(event, this);
        }
        if (transportManager != null) {
            transportManager.endTurn();
        }
        
        int totalExpenses = getTotalMaintenanceCost();
        addFunds(-totalExpenses);
        int netIncome = getNetIncome();
        if (funds < 4*(-netIncome)) {
            MotEngine.instance.broadcast(new FundsLowEF(), this);
        }
        // NetIncome will be added during this turn!
        if (funds < -netIncome*2) {
            sellRandomObject();
        }
        
        if (govenorLeader != null) {
            govenorLeader.notifyEndTurn(this);
        }
        if (moralPolicyAdvisor != null) {
        	moralPolicyAdvisor.endTurn(this);
        }
        if (researchProject != null) {
        	researchProject.endTurn();
        }
        
        StatisticFactory.generate(this);
        
		//turnEnded = true;
	}

    /**
     * Randomly selects this players craft, planetary weapon, shield or scanner and sells 
     * it adding the raised funds to the treasury for this player.
     * @throws mot.common.util.ApplicationError
     */
    private void sellRandomObject() throws ApplicationError {
        Planet [] planets = getPlanets();
        if (Utils.random(0.5) && planets.length > 0) {
            int i = Utils.random(planets.length);
            if (planets[i].getScanners() != null && planets[i].getScanners().getQuantity() > 0) {
                Scanner scanner = planets[i].getScanners().getScannerAtIndex(0);
                int cost = (int) new ProjectScanner(new ScannerDesign(scanner)).getValue();
                if (Logger.isDebug()) {
                    Logger.logDebug("Player.sellRandomObject() Sell scanner "+scanner.getName()+" for value "+cost);
                }
                addFunds(cost);
                planets[i].getScanners().remove(scanner);
                MotEngine.instance.broadcast(new FundsGoneEF(scanner, planets[i], cost), this);
                return;
            }
            if (planets[i].getWeapons() != null && planets[i].getWeapons().getQuantity() > 0) {
                Weapon weapon = planets[i].getWeapons().getWeaponAtIndex(0);
                int cost = (int) new ProjectWeapon(new WeaponDesign(weapon)).getValue();
                if (Logger.isDebug()) {
                    Logger.logDebug("Player.sellRandomObject() Sell weapon "+weapon.getName()+" for value "+cost);
                }
                addFunds(cost);
                planets[i].getWeapons().remove(weapon);
                MotEngine.instance.broadcast(new FundsGoneEF(weapon, planets[i], cost), this);
                return;
            }
            if (planets[i].getShields() != null && planets[i].getShields().getQuantity() > 0) {
                Shield shield = planets[i].getShields().getShieldAtIndex(0);
                int cost = (int) new ProjectShield(new ShieldDesign(shield)).getValue();
                if (Logger.isDebug()) {
                    Logger.logDebug("Player.sellRandomObject() Sell shield "+shield.getName()+" for value "+cost);
                }
                addFunds(cost);
                planets[i].getShields().remove(shield);
                MotEngine.instance.broadcast(new FundsGoneEF(shield, planets[i], cost), this);
                return;
            }
        }
        
        if (getQtyCrafts() > 0) {
            Craft [] crafts = getCraftsNotInScheduledBattle();
            if (crafts != null && crafts.length > 0) {
	            int i = Utils.random(crafts.length);
	            int cost = (int) crafts[i].getValue();
	            if (Logger.isDebug()) {
	                Logger.logDebug("Player.sellRandomObject() Sell craft "+crafts[i].getName()+" for value "+cost);
	            }
	            addFunds(cost);
	            crafts[i].destroy();
	            MotEngine.instance.broadcast(new FundsGoneEF(crafts[i], null, cost), this);
            }
            return;
        }
    }
    
    public int sell(TechnologyList salvageTechs) {
    	Technology[] techs = salvageTechs.getAllAsArray();
    	int total = 0;
    	for (int i = 0; i < techs.length; i++) {
			total += sell(techs[i]);
		}
    	return total;
    }
    
    public int sell(Technology salvage) {
		int value = (int) (salvage.getCostToBuild() / Technology.COST_TO_SELL_FACTOR);
		addFunds(value);
    	return value;
    }
    
    /**
     * All planets a crafts that belong to this player are transfered to player other, then
     * this player is removed from the game.
     * @param other The player to transfer this players objects to.
     */
    public void surrendetTo(Player other) throws ApplicationError {
        PlayerSurrenderEF event = new PlayerSurrenderEF(this, other);
        MotEngine.instance.broadcast(event);
        Battles battles = MotEngine.instance.getBattlesNotNull();
        battles.removePlayerFromQueuedBattles(this);
        Fleet[] fleets = getFleet();
        if (fleets != null) {
        	for (int i = 0; i < fleets.length; i++) {
				fleets[i].destroy();
			}
        }
        Craft [] crafts = getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                crafts[i].surrender(other);
            }
        }
        Planet [] colonies = getPlanets();
        if (colonies != null) {
            for (int i=0; i<colonies.length; i++) {
                colonies[i].changeOwner(other);
            }
        }
        TechnologyList tl = getTechnologies();
        if (tl != null && tl.getQuantity() > 0) {
        	Technology[] ta = tl.getAllAsArray();
        	for (int i = 0; i < ta.length; i++) {
				if (!other.hasDiscovered(ta[i])) {
					other.addTechnology(ta[i]);
				}
			}
        }
        destroy(false);
    }
    
    /**
     * Check the lose game criteria to see if this player has met them.
     * @return True if and only if this player has lost this game and needs to be destroyed.
     */
    public boolean hasLostGame() throws ApplicationError {
        Planet [] planets = getPlanets();
        return (planets == null || planets.length == 0);
    }
    
    /**
     * @return The factory that MOOs should use the generate planetary orbits in a solar system
     * when one is not specified. A null value means that this player does not generate planetary 
     * orbits.
     */
    public CraftOrbitFactory getCraftOrbitFactory() {
    	return null;
    }
    
    /**
     * Marks the turn as started for this player.
     * @throws mot.common.util.ValidationError
     * @throws mot.common.util.ApplicationError
     */
	public void startTurn() throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("Player.startTurn() player="+getName());
        }
		if (turnEnded) {
			turnEnded = false;
			if (!isComputerPlayer()) {
				MotEngine.instance.broadcast(new StartTurnEF(), this);
			}
		}
	}

	/**
	 * @return True if this player is an automated player of some sort. Automated
	 * players are the monster player and all the computer players playing the game.
	 */
	public abstract boolean isComputerPlayer();

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		if (factories != null) {
			factories.referenceToXML(writer);
		}
		if (laboratories != null) {
			laboratories.referenceToXML(writer);
		}
		if (biotechnologies != null) {
			biotechnologies.referenceToXML(writer);
		}
		if (hulls != null) {
			hulls.referenceToXML(writer);
		}
		if (computers != null) {
			computers.referenceToXML(writer);
		}
		if (weapons != null) {
			weapons.referenceToXML(writer);
		}
		if (shields != null) {
			shields.referenceToXML(writer);
		}
		if (scanners != null) {
			scanners.referenceToXML(writer);
		}
		if (specials != null) {
			specials.referenceToXML(writer);
		}
		if (engines != null) {
			engines.referenceToXML(writer);
		}
        if (modules != null) {
            modules.referenceToXML(writer);
        }
        if (humanities != null) {
        	humanities.referenceToXML(writer);
        }
		if (theories != null) {
			theories.referenceToXML(writer);
		}
		if (miscTechnologies != null) {
			miscTechnologies.referenceToXML(writer);
		}
		if (character != null) {
			character.toXML(writer);
		}
		if (researchProject != null) {
			researchProject.toXML(writer);
		}
		getFleet();
		if (fleetArray != null) {
			for (int i=0; i<fleetArray.length; i++) {
				fleetArray[i].toXML(writer);
			}
		}
        if (transportManager != null) {
            transportManager.toXML(writer);
        }
        if (graph != null) {
            graph.toXML(writer);
        }
        
		if (lastDiscovery != null) lastDiscovery.toXML(writer, "last-discovery");
        
        if (color != null) {
            writer.addElement("color", color.getName());
        }
        writer.addElement("funds", funds);
        writer.addElement("boralium", boralium);
        writer.addElement("alignment", alignment);
        writer.addElement("last-date-ancients-defended", lastDateAncientsDefended);
        writer.addElement("qty-battles", qtyBattles);
        writer.addElement("qty-battles-won", qtyBattlesWon);
        writer.addElement("chance-holo-craft-work", chanceHoloCraftWork);
        if (govenorLeader != null) {
            govenorLeader.toXML(writer);
        }
        if (generalLeader != null) {
            generalLeader.toXML(writer);
        }
        if (moralChoices != null) {
        	moralChoices.toXML(writer, "moral-choices");
        }
        if (salvage != null) {
        	salvage.referenceToXML(writer, "salvage");
        }
        if (craftBuildExperience != null) {
        	craftBuildExperience.toXML(writer);
        }
	}


    public void setGraph(Graph graph) {
        this.graph = graph;
    }


    public Graph getGraph() {
        return graph;
    }


    public void setAlignment(String alignment) {
        this.alignment = alignment;
    }


    public String getAlignment() {
        return alignment;
    }


    public void setGovenorLeader(EngineGovenorLeader govenorLeader) {
        this.govenorLeader = govenorLeader;
    }


    public EngineGovenorLeader getGovenorLeader() {
        return govenorLeader;
    }


    public void setGeneralLeader(EngineGeneralLeader generalLeader) {
        this.generalLeader = generalLeader;
    }


    public EngineGeneralLeader getGeneralLeader() {
        return generalLeader;
    }
    
    public boolean canHaveNewLeader() {
        return getGeneralLeader() == null || getGovenorLeader() == null;
    }


    public void setQtyBattles(int qtyBattles) {
        this.qtyBattles = qtyBattles;
    }


    public int getQtyBattles() {
        return qtyBattles;
    }


    public void setQtyBattlesWon(int qtyBattlesWon) {
        this.qtyBattlesWon = qtyBattlesWon;
    }


    public int getQtyBattlesWon() {
        return qtyBattlesWon;
    }


    public void setChanceHoloCraftWork(float chanceHoloCraftWork) {
        this.chanceHoloCraftWork = chanceHoloCraftWork;
    }


    public float getChanceHoloCraftWork() {
        return chanceHoloCraftWork;
    }

    private static final float CHANGE_CHANGE_HOLO_CRAFT_WORKED = 0.75f;

    public void notifyHoloCraftTrickWorked() {
        chanceHoloCraftWork = chanceHoloCraftWork * CHANGE_CHANGE_HOLO_CRAFT_WORKED;
    }

	public CraftBuildExperience getCraftBuildExperience() {
		return craftBuildExperience;
	}

	public CraftBuildExperience getCraftBuildExperienceNotNull() {
		if (craftBuildExperience == null) craftBuildExperience = new CraftBuildExperience();
		return craftBuildExperience;
	}

	public void setCraftBuildExperience(CraftBuildExperience craftBuildExperience) {
		this.craftBuildExperience = craftBuildExperience;
	}

	public MoralChoiceListenerList getMoralChoices() {
		return moralChoices;
	}

	public String [] getMoralChoiceNameList() {
		if (moralChoices != null && moralChoices.getList() != null) {
			return moralChoices.getList().getAllNames();
		} else {
			return null;
		}
	}

	public void setMoralChoices(MoralChoiceListenerList moralChoices) {
		this.moralChoices = moralChoices;
	}

	public MoralPolicyAdvisor getMoralPolicyAdvisor() {
		return moralPolicyAdvisor;
	}

	public MoralPolicyAdvisor getMoralPolicyAdvisorNotNull() {
		if (moralPolicyAdvisor == null) moralPolicyAdvisor = new MoralPolicyAdvisorBasic();
		return moralPolicyAdvisor;
	}

	public void setMoralPolicyAdvisor(MoralPolicyAdvisor moralPolicyAdvisor) {
		this.moralPolicyAdvisor = moralPolicyAdvisor;
	}
	
	public void addSalvage(SalvageList salvageList) throws ApplicationError {
		if (salvageList != null && salvageList.getQuantity() > 0) {
			Technology[] sa = salvageList.getAllAsArray();
			addSalvage(sa);
		}
	}
	
	public void addSalvage(Technology [] sa) throws ApplicationError {
		if (sa != null && sa.length > 0) {
			for (int i = 0; i < sa.length; i++) {
				addSalvage(sa[i]);
			}
			if (!isComputerPlayer()) {
				MessageEF messageEvent = new MessageEF("Salvaged - "+Comp.toString(sa, ", ", 3, "..."));
				messageEvent.addStateUpdate(EngineVoFactory.getPlayer(this));
				MotEngine.instance.broadcast(messageEvent, this);
			}
		}
	}
	
	public void addSalvage(Technology technology) throws ApplicationError {
		if (Logger.isDebug()) {
			Logger.logDebug("Player "+getName()+" salvage technology "+technology);
		}
		if (salvage == null) {
			salvage = new SalvageList();
		}
		salvage.add(technology);
	}


	public void setSalvage(SalvageList salvage) {	this.salvage = salvage; }
	public SalvageList getSalvage() {	return salvage; 	}

	private static final String [] excludeWeaponsFromFireAllAtCraft = {Weapon.GROUP_SPACIAL_DESTORTION,Weapon.GROUP_DROID_MINE,Weapon.GROUP_DETONATOR,Weapon.GROUP_SALVAGE_BEAM,Weapon.GROUP_RETARGET};
	public String[] getWeaponsToExcludeFromFireAllAtCraft() throws ApplicationError {
		return excludeWeaponsFromFireAllAtCraft;
	}

	private static final String [] excludeWeaponsFromFireAllMovingAutomaton = {Weapon.GROUP_SPACIAL_DESTORTION,Weapon.GROUP_DROID_MINE,Weapon.GROUP_DETONATOR,Weapon.GROUP_SALVAGE_BEAM};
	public String[] getWeaponsToExcludeFromFireAllAtMovingAutomaton() throws ApplicationError {
		return excludeWeaponsFromFireAllMovingAutomaton;
	}

	public String[] getWeaponsToExcludeFromFireAll(BattleObject target) throws ApplicationError {
		if (target instanceof MovingAutamaton) {
			return excludeWeaponsFromFireAllMovingAutomaton;
		} else {
			return excludeWeaponsFromFireAllAtCraft;
		}
	}
	
	public Relation getRelationshipWith(Player other) throws ApplicationError {
		Relations rels = MotEngine.getInstance().getRelations();
		Relation r = rels.getRelationship(this, other);
		return r;
	}

	public int getLastDateAncientsDefended() {
		return lastDateAncientsDefended;
	}

	public void setLastDateAncientsDefended(int lastDateAncientsDefended) {
		this.lastDateAncientsDefended = lastDateAncientsDefended;
	}

	public boolean hasExtendedSpaceJump() {
		return miscTechnologies != null && miscTechnologies.getExtendedSpaceJump() != null;
	}

    
}