/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import mot.common.bean.GameConstants;
import mot.common.bean.Immunity;
import mot.common.bean.ImmunityBio;
import mot.common.bean.ImmunityGravity;
import mot.common.bean.ImmunityMindControl;
import mot.common.bean.ImmunityPsi;
import mot.common.bean.ImmunityReplicaBeam;
import mot.common.bean.ImmunityTroopTransport;
import mot.common.bean.NamedObject;
import mot.common.bean.PlayerAiSettings;
import mot.common.bean.PlayerCharacterBasic;
import mot.common.lang.L;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOption;
import mot.common.setup.SetupOptions;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValuePairList;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.NameValuePair;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.relations.Relation;

/**
 * The bonuses this player's characteristics provides to its owner.
 */
public class PlayerCharacter extends NamedObject {

	// These values can be found in relics.xml
	private static final String BIO_CONFORMITY = "Bio Conformity";
	private static final String BIO_GENETIC_TRANSFORMATION = "Bio Genetic Transformation";
	private static final String ELECTROLITIC_AUGUMENTATION = "Electrolitic Augumentation";
	private static final String CRYSTALINE_AUGUMENTATED_METALS = "Crystaline Augumentated Metals";

	// These values can be found in ancient_alignment_gifts.xml
	private static final String PSI_EMHANCEMENT = "Psi Enhancment";
	private static final String CLOAKING_SPACE_INVERTER = "Cloaking Space Inverter";

	private PlayerCharacterBasic basicCharacter;

	/**
	 * A value between 0 and 1 that is the probability if this player attacking an
	 * enemy's planet opposed to blockading.
	 */
	private double blockadeOrAttackProbability;

	/**
	 * A value between 0 and 1 that is the probability  this player will demand assets from
     * a player it has decided to go to war with instead of going to war directly.
	 */
	private double demandProbability;

	/**
	 * A value between 0 and 1 that is the probability of accepting peace when offered
	 * (given the relationship criteria have been met).
	 */
	private double acceptPeaceProbability;

	/**
	 * A value between 0 and 1 that is the probability of accepting an alliance when offered
	 * by a player whom I like (given the relationship criteria have been met).
	 */
	private double acceptAlianceProbabilityWhenLike;

	/**
	 * A value between 0 and 1 that is the probability of accepting an alliance when offered
	 * by a player whom I am fond (given the relationship criteria have been met).
	 */
	private double acceptAlianceProbabilityWhenFond;
    
	/**
	 * A value between 0 and 1 that is the probability of accepting a swap (tech etc) when offered
	 * by a player whom I like (or more) towards and its is a BAD Swap (other players asking for more than giving).
	 */
	private double acceptBadSwapProbabilityWhenLike = 0.5;
    
	/**
	 * A value between 0 and 1 that is the probability of accepting a swap (tech etc) when offered
	 * by a player whom I like (or more) towards and its is a Good Swap (other players giving for more than taking).
	 */
	private double acceptGoodSwapProbabilityWhenLike = 1.0;
    
	/**
	 * A value between 0 and 1 that is the probability of accepting a swap (tech etc) when offered
	 * by a player whom I am neutral (or more) towards.
     * NOTE - In StratagistBasic the player must offer as much as he asks for before given this chance!
	 */
	private double acceptGoodSwapProbabilityWhenNeutral = 0.5;
	private double acceptBadSwapProbabilityWhenNeutral = 0.0;

	/**
	 * A value between 0 and 1 that is the probability of accepting a swap (tech etc) when offered
	 * by a player whom I am less than neutral towards (dislike).
     * NOTE - In StratagistBasic the player must offer more than he asks for before given this chance!
	 */
	private double acceptGoodSwapProbabilityWhenDislike = 0.25;
	private double acceptBadSwapProbabilityWhenDislike = 0.0;

	/**
	 * A percentage change for this player in relationship ratings. For examaple, a value of
	 * 0.8 would make the player an introvert reducing all relationship ratings, while a value
	 * of 1.2 would make the player an extrovert increasing all relationship ratings
	 */
	private double introversionRating;

    /**
     * The maximum qty turns this player character will wait before sending out a fleet.
     */
    private int maxTimeBetweenFleets;

    /**
     * When a computer player is a peace with everyone this determines the size of his
     * military buildup before going to war.
     * A computer player will go to war on the player who is most hated (has the lowest
     * relationship rating - see Relations) once the computer player has more craft than
     * qtyCraftPerPlanetToDeclareWar * qtyPlanetsOwnedByPlayer OR the condition below.
     */
    private int qtyCraftPerPlanetToDeclareWar;

    /**
     * When a computer player is a peace with everyone this determines the size of his
     * military buildup before going to war.
     * A computer player will go to war on the player who is most hated (has the lowest
     * relationship rating - see Relations) once the computer player has more craft than
     * qtyCraftToDeclareWar OR the condition above.
     */
    private int qtyCraftToDeclareWar;

	/**
	 * Get the relationship between this player character and the other player
	 * character. Player characters have a base like or dislike towards each other.
	 * See Relationships class documentation for values.
	 */
	private Relation relationships [];

    /**
     * A list of special technology names this player would like on its crafts.
     * In the property file it is a comma separated list of mutually exclusive 
     * techs. This should be a list of technology group names and for each group
     * name used it will choose the latest technology in that group.
     */
    private String [] craftDeasignSpecials;

    /**
     * This is a list of special weapons this player would want on its crafts.
     */
    private String [] craftsSpecialWeapons;

    /**
     * This is true if this player would like to have its crafts designed using psi shields and
     * false otherwise.
     */
    private boolean craftsWantPsiShields;

    /**
     * This is true if this player would like to design military crafts that are particular to its character 
     */
    private boolean craftsPlayerPersonalityOn;
    
    /**
     * This is a map from diplomatic statement name (defined in DiplomaticStatements) to the sentence 
     * used by this player to make that statement.
     * Diplomatic statements are sentences used to communicate things like:
     *  - I declare war on you
     *  - Will you agree to peace with me?
     * NOTE - THIS IS NOT SAVED IN A SAVE GAME, It is always loaded from options when first requested!
     */
    private Map diplomaticStatements;
    
    /**
     * This is a list of names of acceptable moral choices for this player
     * A Computer player will only accept the suggestion of a MoralChoice if it is in this list
     */
    private String [] acceptableMoralChoices;
    
    /**
     * Can be values
     * 	0 Normal
     * 	1 Isolationist
     * 	2 Expansionist
     * 
     * A player that is isolationist tries to avoid declaring war and directs more effort to research
     * An isolationist player will cease isolationist policies once towards the end of a game.
     * 
     * A player that is expansionist will declare war sooner than normal.  
     */
    private int foreignPolicy;
    
    public PlayerCharacter() {
	}

	public void setRelationships(Relation [] v) { relationships = v; }
    
	public void setBlockadeOrAttackProbability(double v) { blockadeOrAttackProbability = v; }
	public void setAcceptPeaceProbability(double v) { acceptPeaceProbability = v; }
	public void setAcceptAlianceProbabilityWhenLike(double v) { acceptAlianceProbabilityWhenLike = v; }
	public void setAcceptAlianceProbabilityWhenFond(double v) { acceptAlianceProbabilityWhenFond = v; }
	public void setIntroversionRating(double v) { introversionRating = v; }
    public void setMaxTimeBetweenFleets(int v) { maxTimeBetweenFleets = v; }
	public void setQtyCraftPerPlanetToDeclareWar(int v) { qtyCraftPerPlanetToDeclareWar = v; }
	public void setQtyCraftToDeclareWar(int v) { qtyCraftToDeclareWar = v; }
	public void setAcceptBadSwapProbabilityWhenLike(double v) {acceptBadSwapProbabilityWhenLike = v; }
	public void setAcceptGoodSwapProbabilityWhenLike(double v) {acceptGoodSwapProbabilityWhenLike = v; }
	public void setAcceptBadSwapProbabilityWhenNeutral(double v) {acceptBadSwapProbabilityWhenNeutral = v; }
	public void setAcceptGoodSwapProbabilityWhenNeutral(double v) {acceptGoodSwapProbabilityWhenNeutral = v; }
	public void setAcceptBadSwapProbabilityWhenDislike(double v) {acceptBadSwapProbabilityWhenDislike = v; }
	public void setAcceptGoodSwapProbabilityWhenDislike(double v) {acceptGoodSwapProbabilityWhenDislike = v; }

	public double getBlockadeOrAttackProbability() { return blockadeOrAttackProbability; }
	public double getAcceptPeaceProbability() { return acceptPeaceProbability; }
	public double getAcceptAlianceProbabilityWhenLike() { return acceptAlianceProbabilityWhenLike; }
	public double getAcceptAlianceProbabilityWhenFond() { return acceptAlianceProbabilityWhenFond; }
	public double getAcceptBadSwapProbabilityWhenLike() { return acceptBadSwapProbabilityWhenLike;	}
	public double getAcceptGoodSwapProbabilityWhenLike() { return acceptGoodSwapProbabilityWhenLike;	}
    public double getAcceptGoodSwapProbabilityWhenNeutral() { return  acceptGoodSwapProbabilityWhenNeutral; }
    public double getAcceptBadSwapProbabilityWhenNeutral() { return  acceptBadSwapProbabilityWhenNeutral; }
    public double getAcceptGoodSwapProbabilityWhenDislike() { return  acceptGoodSwapProbabilityWhenDislike; }
    public double getAcceptBadSwapProbabilityWhenDislike() { return  acceptBadSwapProbabilityWhenDislike; }
	public double getIntroversionRating() { return introversionRating; }
    public int getMaxTimeBetweenFleets() { return maxTimeBetweenFleets; }
	public int getQtyCraftPerPlanetToDeclareWar() { return qtyCraftPerPlanetToDeclareWar; }
	public int getQtyCraftToDeclareWar() { return qtyCraftToDeclareWar; }

	// Backwards Compatibility v1.22
    public double getAcceptSwapTechProbabilityWhenNeutral() { return  acceptGoodSwapProbabilityWhenNeutral; }
    public double getAcceptSwapTechProbabilityWhenDislike() { return  acceptGoodSwapProbabilityWhenDislike; }

	public Relation [] getRelationships() { return relationships; }
    
    private ResourcefulProcessor resourcefulProcessor;
    public ResourcefulProcessor getResourcefulProcessor() {
        if (resourcefulProcessor == null) {
            resourcefulProcessor = new ResourcefulProcessorBasic();
        }
        return resourcefulProcessor;
    }

    /**
     * Use by XMLDocumentHandler to load saved games - It add this relation to the list of relations.
     * @param r The relation to add.
     */
    public void setRelation(Relation r) {
        ArrayList list = new ArrayList();
        if (relationships != null) {
            for (int i=0; i<relationships.length; i++) {
                list.add(relationships[i]);
            }
        }
        list.add(r);
        relationships = (Relation []) list.toArray(new Relation [list.size()]);
    }


    /**
     * Removes all relationships to in this player character containing the player.
     * @throws mot.common.util.ApplicationError
     * @param player The player for whom all relations should be deleted.
     */
	public void removeRelationshipsTo(Player player) throws ApplicationError {
        ArrayList list = new ArrayList();
        if (relationships != null) {
            for (int i=0; i<relationships.length; i++) {
                if (!relationships[i].contains(player)) {
                    list.add(relationships[i]);
                }
            }
            relationships = (Relation []) list.toArray(new Relation [list.size()]);
        }
	}
    
	public Relation getRelationshipTo(Player player) throws ApplicationError {
        if (relationships != null) {
			for (int i=0; i<relationships.length; i++) {
	            Relation rel = relationships[i];
				if (rel.getPlayer1() == player) {
					return rel;
				}
				if (rel.getPlayer2() == player) {
					return rel;
				}
			}
        }
		return null;
	}
    
	public int getRelationshipRatingTo(Player player) throws ApplicationError {
        Relation rel = getRelationshipTo(player);
        if (rel != null) {
            return rel.getRelationshipValue();
        } else {
            return 0;
        }
	}
    
    public String [] getCraftDeasignSpecials() {
        return craftDeasignSpecials;
    }
    
    private static final String SPECIALS_DELIMITER = ",";
    private double spyDefendPercent;
    private double spyAttackMostHatedPercent;
    private double spyAttackLargestPercent;
    public void setCraftDesignSpecials(String v) {
        StringTokenizer st = new StringTokenizer(v, SPECIALS_DELIMITER , false);
        craftDeasignSpecials = new String[st.countTokens()];
        int i = 0;
        while (st.hasMoreElements()) {
            String token = (String) st.nextElement();
            craftDeasignSpecials[i++] = token;
        }
    }

	/**
     * Constructs a players character from the setup options for this player's race.
     * Note some of the player statistics change for different levels.
     * 
     * The relationships must be defined before constructing a player character because
     * it uses the settings in relationships as default settings for this player.
     * 
     * @param myName The name of this new player
     * @param options The setup options to configure this player
     * @param level The level at which this game is being played
     * @param playerNames The list of players in this game.
     * @throws mot.common.util.ApplicationError
     */
	public PlayerCharacter(String myName, RaceOptions options, Double level, PlayerAiSettings playerAiSettings, String [] playerNames) throws ApplicationError {
        super(myName);

        setBasicCharacter(new PlayerCharacterBasic(myName, options, level, playerNames));
        
		introversionRating = options.getDoubleForName("introversion-rating");
        
        // Get relationships from MotEngine.
		ArrayList list = new ArrayList();
		for (int i = 0; playerNames != null && i < playerNames.length; i++) {
            if (Comp.same(myName, playerNames[i])) {
                continue;
            }
            Relation r = MotEngine.getInstance().getRelationsNotNull().get(myName, playerNames[i]);
            if (r == null) {
                Logger.logWarning("PlayerCharacter.init() no relationship between "+myName+" and "+playerNames[i]);
            } else {
                list.add( new Relation(r) );
            }
		}
		relationships = (Relation [] ) list.toArray( new Relation[list.size()] );

		demandProbability = options.getDoubleForName("demand-probability");
		blockadeOrAttackProbability = options.getDoubleForName("blockade-or-attack-probability");
		acceptPeaceProbability = options.getDoubleForName("accept-peace-propability");
		acceptAlianceProbabilityWhenLike = options.getDoubleForName("accept-alliance-propability-when-like");
		acceptAlianceProbabilityWhenFond = options.getDoubleForName("accept-alliance-propability-when-fond");
        maxTimeBetweenFleets = options.getIntForName("max-time-between-fleets");

		spyDefendPercent = options.getDoubleForName("spy-defend-percent");
		spyAttackMostHatedPercent = options.getDoubleForName("spy-attack-most-hated-percent");
		spyAttackLargestPercent = options.getDoubleForName("spy-attack-largest-percent");
        
        String v = options.getStringForName("craft-design-specials");
        if (v != null) {
            setCraftDesignSpecials(v);
        }

        craftsSpecialWeapons = options.getSplitStringForName("crafts-special-weapons", ",");
        craftsWantPsiShields = options.getBoolForName("crafts-want-psi-shields");
        acceptableMoralChoices = options.getSplitStringForName("acceptable-moral-choices", ",");

        if (playerAiSettings != null && playerAiSettings.getForeignPolicy() < 0) { // isolationist
        	foreignPolicy = 1;
            qtyCraftPerPlanetToDeclareWar = options.getIntForName("qty-craft-per-planet-to-declare-war")+1;
            qtyCraftToDeclareWar = options.getIntForName("qty-craft-to-declare-war")+1;
        } else if (playerAiSettings != null && playerAiSettings.getForeignPolicy() > 0) { // expansionist
        	foreignPolicy = 2;
			qtyCraftPerPlanetToDeclareWar = options.getIntForName("qty-craft-per-planet-to-declare-war")-1;
			qtyCraftToDeclareWar = options.getIntForName("qty-craft-to-declare-war")-2;
        } else { // normal
        	foreignPolicy = 0;
            qtyCraftPerPlanetToDeclareWar = options.getIntForName("qty-craft-per-planet-to-declare-war");
            qtyCraftToDeclareWar = options.getIntForName("qty-craft-to-declare-war");
        }
    
        if (playerAiSettings != null && playerAiSettings.getDiplomaticTrade() < 0) { // cautious
        	acceptBadSwapProbabilityWhenLike = 0.0;
        	acceptGoodSwapProbabilityWhenLike = options.getDoubleForName("accept-swap-propability-when-like");
        	acceptBadSwapProbabilityWhenNeutral = 0.0; 
        	acceptGoodSwapProbabilityWhenNeutral = options.getDoubleForName("accept-swap-propability-when-neutral"); 
        	acceptBadSwapProbabilityWhenDislike = 0.0; 
        	acceptGoodSwapProbabilityWhenDislike = 0.0; 
        } else if (playerAiSettings != null && playerAiSettings.getDiplomaticTrade() > 0) { // eager
        	acceptBadSwapProbabilityWhenLike = 1.0;
        	acceptGoodSwapProbabilityWhenLike = 1.0;
        	acceptBadSwapProbabilityWhenNeutral = 1.0; 
        	acceptGoodSwapProbabilityWhenNeutral = 1.0; 
        	acceptBadSwapProbabilityWhenDislike = options.getDoubleForName("accept-swap-propability-when-dislike"); 
        	acceptGoodSwapProbabilityWhenDislike = 1.0; 
        } else { // normal        	
        	acceptBadSwapProbabilityWhenLike = options.getDoubleForName("accept-swap-propability-when-like");
        	acceptGoodSwapProbabilityWhenLike = 1.0;
        	acceptBadSwapProbabilityWhenNeutral = options.getDoubleForName("accept-swap-propability-when-neutral"); 
        	acceptGoodSwapProbabilityWhenNeutral = 1.0; 
        	acceptBadSwapProbabilityWhenDislike = 0.0; 
        	acceptGoodSwapProbabilityWhenDislike = options.getDoubleForName("accept-swap-propability-when-dislike"); 
        }

        if (playerAiSettings != null && playerAiSettings.getPlayerPersonality() > 0) {        	
        	craftsPlayerPersonalityOn  = true;
        } else if (playerAiSettings != null && playerAiSettings.getPlayerPersonality() < 0) {        	
        	craftsPlayerPersonalityOn  = false;
        } else {
        	craftsPlayerPersonalityOn  = level != null && level > 1.0;
        }


    }

	public PlayerCharacter(PlayerCharacterBasic pc) {
		super(pc);
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		basicCharacter.toXML(writer);
		writer.addElement("blockade-or-attack-probability", blockadeOrAttackProbability);
		writer.addElement("accept-peace-probability", acceptPeaceProbability);
		writer.addElement("accept-aliance-probability-when-like", acceptAlianceProbabilityWhenLike);
		writer.addElement("accept-aliance-probability-when-fond", acceptAlianceProbabilityWhenFond);
		writer.addElement("accept-bad-swap-probability-when-like", acceptBadSwapProbabilityWhenLike);
		writer.addElement("accept-good-swap-probability-when-like", acceptGoodSwapProbabilityWhenLike);
		writer.addElement("accept-bad-swap-probability-when-neutral", acceptBadSwapProbabilityWhenNeutral);
		writer.addElement("accept-good-swap-probability-when-neutral", acceptGoodSwapProbabilityWhenNeutral);
		writer.addElement("accept-bad-swap-probability-when-dislike", acceptBadSwapProbabilityWhenDislike);
		writer.addElement("accept-good-swap-probability-when-dislike", acceptGoodSwapProbabilityWhenDislike);
		writer.addElement("introversion-rating", introversionRating);
		writer.addElement("max-time-between-fleets", maxTimeBetweenFleets);
        writer.addElement("qty-craft-per-planet-to-declare-war", qtyCraftPerPlanetToDeclareWar);
        writer.addElement("qty-craft-to-declare-war", qtyCraftToDeclareWar);
		writer.addElement("craft-design-specials", Comp.toString(craftDeasignSpecials, ","));
		writer.addElement("spy-defend-percent", spyDefendPercent);
		writer.addElement("spy-attack-most-hated-percent", spyAttackMostHatedPercent);
		writer.addElement("spy-attack-largest-percent", spyAttackLargestPercent);
        if (craftsSpecialWeapons != null) {
			for (int i=0; i<craftsSpecialWeapons.length; i++) {
				writer.addElement("crafts-special-weapons", craftsSpecialWeapons[i]);
			}
        }
		writer.addElement("crafts-want-psi-shields", craftsWantPsiShields);
		writer.addElement("crafts-player-personality-on", craftsPlayerPersonalityOn);
		
		if (relationships != null) {
			for (int i=0; i<relationships.length; i++) {
                if (relationships[i].getPlayer1() != null && relationships[i].getPlayer2() != null) {
                    relationships[i].toXML(writer);
                }
			}
		}
		writer.addElement("acceptable-moral-choices", Comp.toString(acceptableMoralChoices, ","));
		writer.addElement("foreign-policy", foreignPolicy);
	}

	
	protected String getElementName() { return "player-character"; }

    public double getSpyDefendPercent() {
        return spyDefendPercent;
    }

    public void setSpyDefendPercent(double spyDefendPercent) {
        this.spyDefendPercent = spyDefendPercent;
    }

    public double getSpyAttackMostHatedPercent() {
        return spyAttackMostHatedPercent;
    }

    public void setSpyAttackMostHatedPercent(double spyAttackMostHatedPercent) {
        this.spyAttackMostHatedPercent = spyAttackMostHatedPercent;
    }

    public double getSpyAttackLargestPercent() {
        return spyAttackLargestPercent;
    }

    public void setSpyAttackLargestPercent(double spyAttackLargestPercent) {
        this.spyAttackLargestPercent = spyAttackLargestPercent;
    }

    public void setCraftsSpecialWeapons(String[] craftsSpecialWeapons) {
        this.craftsSpecialWeapons = craftsSpecialWeapons;
    }

    public void setCraftsSpecialWeapons(String v) {
        if (craftsSpecialWeapons == null) {
            craftsSpecialWeapons = new String[1];
            craftsSpecialWeapons[0] = v;
        } else {
            String [] newArray = new String[craftsSpecialWeapons.length+1];
            Comp.copy(newArray, craftsSpecialWeapons);
            newArray[craftsSpecialWeapons.length] = v;
            craftsSpecialWeapons = newArray;
        }
    }


    public String[] getCraftsSpecialWeapons() {
        return craftsSpecialWeapons;
    }

    
    public void setCraftsWantPsiShields(boolean v) {
        this.craftsWantPsiShields = v;
    }


    public boolean getCraftsWantPsiShields() {
        return craftsWantPsiShields;
    }

    public void setCraftsPlayerPersonalityOn(boolean v) {
        this.craftsPlayerPersonalityOn = v;
    }


    public boolean getCraftsPlayerPersonalityOn() {
        return craftsPlayerPersonalityOn;
    }


    public void setDemandProbability(double demandProbability) {
        this.demandProbability = demandProbability;
    }


    public double getDemandProbability() {
        return demandProbability;
    }
    
    private static final String STATEMENT_OPTION_NAME = "DIPLOMATIC_STATEMENTS";
    
    public String getDiplomaticStatement(String statementName) throws ApplicationError {
        if (diplomaticStatements == null) {
            diplomaticStatements = new HashMap();
            RaceOptions options = OptionsFactory.getRaceOptions(getName());
            SetupOption option = options.getForName(STATEMENT_OPTION_NAME);
            if (option == null) return null;
            BasicNameValuePairList statementList = option.getParameters();
            if (statementList == null) return null;
            NameValuePair [] statements = statementList.getAsArray();
            if (statements == null) return null;
            for (int i=0; i<statements.length; i++) {
                diplomaticStatements.put(statements[i].getName(), statements[i].getValue());
            }
        }
        return (String) diplomaticStatements.get(statementName);
    }

    public void setDiplomaticStatementsToNull() {
    	diplomaticStatements = new HashMap();
    }

	public String[] getAcceptableMoralChoices() {
		return acceptableMoralChoices;
	}

	public void setAcceptableMoralChoices(String[] acceptableMoralChoices) {
		this.acceptableMoralChoices = acceptableMoralChoices;
	}

	public void setAcceptableMoralChoices(String acceptableMoralChoices) {
		this.acceptableMoralChoices = acceptableMoralChoices.split(",");
	}

	// Backwards compatibility v1.22
	public void setAcceptSwapTechProbabilityWhenNeutral(double v) {
		this.acceptGoodSwapProbabilityWhenNeutral = v;
	}

	// Backwards compatibility v1.22
	public void setAcceptSwapTechProbabilityWhenDislike(double v) {
		this.acceptGoodSwapProbabilityWhenDislike = v;
	}

	public PlayerCharacterBasic getBasicCharacter() {
		return basicCharacter;
	}

	public void setBasicCharacter(PlayerCharacterBasic basicCharacter) {
		this.basicCharacter = basicCharacter;
	}

	
	//
	// GETTERS AND SETTERS FOR BASIC CHARACTER
	//
	
	public void setProductionBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setProductionBonus(v); }
	public void setResearchBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setResearchBonus(v); }
	public void setDevelopmentBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setDevelopmentBonus(v); }
	public void setReproductionBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setReproductionBonus(v); }
	public void setFactoryPerPopulationBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setFactoryPerPopulationBonus(v); }
	public void setEcologyBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setEcologyBonus(v); }
    public void setMaxPopulationBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setMaxPopulationBonus(v); }

	public void setPlayerEcologyPercent(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setPlayerEcologyPercent(v); }
	public void setPlayerDevelopmentPercent(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setPlayerDevelopmentPercent(v); }
	public void setPlayerResearchPercent(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setPlayerResearchPercent(v); }
	public void setPlayerProductionPercent(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setPlayerProductionPercent(v); }
	public void setPlayerMoneyPercent(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setPlayerMoneyPercent(v); }

	public void setPsiRating(int v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setPsiRating(v); }
	public void setMissedTechnologies(TechnologyList v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setMissedTechnologies(v); }
	public void setMissedTechnologies(GeneralTechnologyList v ) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setMissedTechnologies(v); }
    
	public void setBattleTurningBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setBattleTurningBonus(v); }
	public void setBattleSpeedBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setBattleSpeedBonus(v); }
	public void setInitiativeBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setInitiativeBonus(v); }
	public void setSpeedBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setSpeedBonus(v); }
	public void setMaxDamageBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setMaxDamageBonus(v); }
	public void setCloakLevel(int v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setCloakLevel(v); }
    public void setTradingBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setTradingBonus(v); }
    public void setAssaultBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setAssaultBonus(v); }
    public void setSpyBonus(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setSpyBonus(v); }

    public void setResourceful(boolean v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setResourceful(v); }
	public void setChanceToDiscoverSalvagedTech(double v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setChanceToDiscoverSalvagedTech(v); 	}

    public void setImmunity(Immunity v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setImmunity(v); }
    public void setImmunityBio(ImmunityBio  v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setImmunityBio(v); }
    public void setImmunityPsi(ImmunityPsi  v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setImmunityPsi(v); }
    public void setImmunityGravity(ImmunityGravity  v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setImmunityGravity(v); }
    public void setImmunityMindControl(ImmunityMindControl  v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setImmunityMindControl(v); }
    public void setImmunityReplicaBeam(ImmunityReplicaBeam  v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setImmunityReplicaBeam(v); }
    public void setImmunityTroopTransport(ImmunityTroopTransport v) { if (basicCharacter == null) basicCharacter = new PlayerCharacterBasic(); basicCharacter.setImmunityTroopTransport(v); }

	public double getProductionBonus(boolean includeLevel) { return includeLevel ? basicCharacter.getProductionTotalBonus() : basicCharacter.getProductionBonus(); }
	public double getResearchBonus(boolean includeLevel) { return includeLevel ? basicCharacter.getResearchTotalBonus() : basicCharacter.getResearchBonus(); }
	public double getDevelopmentBonus(boolean includeLevel) { return includeLevel ? basicCharacter.getDevelopmentTotalBonus() : basicCharacter.getDevelopmentBonus(); }
	
	public double getReproductionBonus() { return basicCharacter.getReproductionBonus(); }
	public double getFactoryPerPopulationBonus() { return basicCharacter.getFactoryPerPopulationBonus(); }
	public double getEcologyBonus() { return basicCharacter.getEcologyBonus(); }
    public double getMaxPopulationBonus() { return basicCharacter.getMaxPopulationBonus(); }

	public double getPlayerEcologyPercent() { return basicCharacter.getPlayerEcologyPercent(); }
	public double getPlayerDevelopmentPercent() { return basicCharacter.getPlayerDevelopmentPercent(); }
	public double getPlayerResearchPercent() { return basicCharacter.getPlayerResearchPercent(); }
	public double getPlayerProductionPercent() { return basicCharacter.getPlayerProductionPercent(); }
	public double getPlayerIsolationistResearchPercent() { return basicCharacter.getPlayerIsolationistResearchPercent(); }
	public double getPlayerIsolationistProductionPercent() { return basicCharacter.getPlayerIsolationistProductionPercent(); }
	public double getPlayerMoneyPercent() { return basicCharacter.getPlayerMoneyPercent(); }

	public int getPsiRating() { return basicCharacter.getPsiRating(); }
	public TechnologyList getMissedTechnologies() throws ApplicationError { return basicCharacter.getMissedTechnologiesAsList(); }

    public double getBattleTurningBonus() { return basicCharacter.getBattleTurningBonus(); }
    public double getBattleSpeedBonus() { return basicCharacter.getBattleSpeedBonus(); }
	public double getInitiativeBonus() { return basicCharacter.getInitiativeBonus(); }
    public double getSpeedBonus() { return basicCharacter.getSpeedBonus(); }
    public double getMaxDamageBonus() { return basicCharacter.getMaxDamageBonus(); }
    public int getCloakLevel() { return basicCharacter.getCloakLevel(); }
    public double getTradingBonus() { return basicCharacter.getTradingBonus(); }
    public double getAssaultBonus() { return basicCharacter.getAssaultBonus(); }
    public double getSpyBonus() { return basicCharacter.getSpyBonus(); }

    public boolean getIsResourceful() { return basicCharacter.getIsResourceful(); }
    public boolean isResourceful() { return basicCharacter.isResourceful(); }
    
    public boolean isImune(Weapon weapon) { return basicCharacter.isImune(weapon); }
	public double getChanceToDiscoverSalvagedTech() { return basicCharacter.getChanceToDiscoverSalvagedTech(); }
	public int getCharisma() { return basicCharacter.getCharisma(); }
	public double getEloquenceBonus() { return basicCharacter.getEloquenceBonus(); }
	public Immunity [] getImmunities() { return basicCharacter.getImmunities(); }
	public int getMaxPlayerTradeRoutes() { return basicCharacter.getMaxPlayerTradeAgreements(); }
	public int getMaxSolarSystemTradeRoutes() { return basicCharacter.getMaxSolarSystemTradeRoutes(); }

	public void increasesBonusesFromOptions(SetupOption option) throws ApplicationError {
		basicCharacter.increasesBonusesFromOptions(option);
	}
	
	public void increasesBonusesFromOptions(SetupOptions options) throws ApplicationError {
		basicCharacter.increasesBonusesFromOptions(options);
	}

	public void setCharisma(int v) throws ApplicationError { basicCharacter.setCharisma(v); }
	
	// This is for backwards compatibility in saved files - can be deleted in the future
	public void setMaxTradeRoutes(int v) throws ApplicationError { basicCharacter.setMaxPlayerTradeAgreements(v); }

	public void setMaxPlayerTradeRoutes(int v) throws ApplicationError { basicCharacter.setMaxPlayerTradeAgreements(v); }
	public void setMaxPlanetTradeRoutes(int v) throws ApplicationError { basicCharacter.setMaxSolarSystemTradeRoutes(v); }
	public void setEloquenceBonus(double v) throws ApplicationError { basicCharacter.setEloquenceBonus(v); }

	public boolean designesInvaderCraft() {
		// Bears
		return craftsPlayerPersonalityOn && basicCharacter.getAssaultBonus() > 0.5;
	}

	public boolean designesPsiCraft() {
		// Ghosts
		return craftsPlayerPersonalityOn && basicCharacter.getPsiRating() > 10;
	}

	public boolean designesPlaguerCraft() {
		// Shifters
		return craftsPlayerPersonalityOn && basicCharacter.getMaxPopulationBonus() > 0.1;
	}

	public boolean designesDamageBotCraft() {
		// Machines
		return craftsPlayerPersonalityOn && basicCharacter.getDevelopmentBonus() > 0.1;
	}

	public boolean designesDroidMinerCraft() {
		// Birds
		return craftsPlayerPersonalityOn && basicCharacter.getBattleSpeedBonus() >= 0.2;
	}

	public boolean isIsolationist() {
		return foreignPolicy == 1;
	}

	public boolean isExpansionist() {
		return foreignPolicy == 2;
	}

	// Backwards Compatibility version 1.22
	public void setIsolationist(boolean isolationist) {
		this.foreignPolicy = isolationist ? 1 : 0;
	}

	public int getForeignPolicy() {
		return foreignPolicy;
	}

	public void setForeignPolicy(int foreignPolicy) {
		this.foreignPolicy = foreignPolicy;
	}
	
	public String getLevelDescription() {
		if (basicCharacter.getLevel() <= GameConstants.LEVEL_EASY_BONUS) {
			return L.LEVEL_1;
		} else if (basicCharacter.getLevel() <= GameConstants.LEVEL_NORMAL_BONUS) {
			return L.LEVEL_1;
		} else if (basicCharacter.getLevel() <= GameConstants.LEVEL_HARD_BONUS) {
			return L.LEVEL_2;
		} else if (basicCharacter.getLevel() <= GameConstants.LEVEL_REAL_HARD_BONUS) {
			return L.LEVEL_3;			
		}
		return "Unknown";
	}
	
	public String getForeignPolicyDescription() {
		if (foreignPolicy == 1) {
			return L.FOREIGN_POLICY_ISOLATIONIST;
		} else if (foreignPolicy == 2) {
			return L.FOREIGN_POLICY_EXPANSIONIST;
		} else {
			return L.FOREIGN_POLICY_NORMAL;			
		}
	}
	
	public boolean isLevelEasy() {
		return basicCharacter.getLevel() == GameConstants.LEVEL_EASY_BONUS;
	}

	public boolean isLevelNormal() {
		return basicCharacter.getLevel() == GameConstants.LEVEL_NORMAL_BONUS;
	}

	public boolean isLevelHard() {
		return basicCharacter.getLevel() == GameConstants.LEVEL_HARD_BONUS;
	}

	public boolean isLevelVeryHard() {
		return basicCharacter.getLevel() == GameConstants.LEVEL_REAL_HARD_BONUS;
	}

}