/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;
import java.util.ArrayList;

import mot.common.bean.PlayerResources;
import mot.common.resources.Resource;
import mot.common.resources.Resources;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.relations.TradeAgreementTreaty;

/**
 * This is used to hold the current resources held by a player. It provides cached
 * value for all the bonuses that resources may offer.
 * This class should be flushed or recreated every time the owning player loses or 
 * gains a new planet.
 */
public class PlayerResourceCache extends PlayerResources {

    public PlayerResourceCache(Player player) throws ApplicationError {
    	Planet[] planets = player.getPlanets();
        if (planets == null) return;
        for (int i=0; i<planets.length; i++) {
            if (planets[i].getResource() != null) {
                Resource resource = planets[i].getResource();
                addResource(resource);
            }
        }
        BlackHole[] blackHoles = MotEngine.instance.getStellarObjects().getBlackHoles();
        if (blackHoles != null) {
        	for (int i = 0; i < blackHoles.length; i++) {
				if (blackHoles[i].isOwnedBy(player)) {
					researchBonus += blackHoles[i].getResearchBonus();
					if (blackHoles[i].isSpaceJumpAbility() && player.hasExtendedSpaceJump()) {
						setInfinateRangeAbility(true);
					}
				}
			}
        }
        TradeAgreementTreaty[] tradeAgreements = MotEngine.instance.getRelationsNotNull().getTraderAgreementsWith(player);
        if (tradeAgreements != null) {
        	for (int i = 0; i < tradeAgreements.length; i++) {
        		Resource resource = tradeAgreements[i].getResourceRecieved(player);
        		if (resource != null) {
        			addResource(resource);
        		}
			}
        }
    }

	private void addResource(Resource resource) {
		if (resource == null) return;
		psiBonus += resource.getPsiBonus();
		cloakBonus += resource.getCloakBonus();
		speedBonus += resource.getSpeedBonus();
		damageBonus += resource.getDamageBonus();
		maxDamageBonus += resource.getMaxDamageBonus();
		shieldBonus += resource.getShieldBonus();

	}

	public Resource getRandomResourceExcluding(PlayerResourceCache excludeResources) {
		ArrayList candidates = new ArrayList();
		if (contains(Resources.antiMatter)) {
			if (excludeResources == null || !excludeResources.contains(Resources.antiMatter)) candidates.add(Resources.antiMatter);
		}
		if (contains(Resources.cloakPalm)) {
			if (excludeResources == null || !excludeResources.contains(Resources.cloakPalm)) candidates.add(Resources.cloakPalm);
		}
		if (contains(Resources.psiFungus)) {
			if (excludeResources == null || !excludeResources.contains(Resources.psiFungus)) candidates.add(Resources.psiFungus);
		}
		if (contains(Resources.tritaniumOre)) {
			if (excludeResources == null || !excludeResources.contains(Resources.tritaniumOre)) candidates.add(Resources.tritaniumOre);
		}
		if (contains(Resources.warpCrystals)) {
			if (excludeResources == null || !excludeResources.contains(Resources.warpCrystals)) candidates.add(Resources.warpCrystals);
		}
		if (contains(Resources.energySpores)) {
			if (excludeResources == null || !excludeResources.contains(Resources.energySpores)) candidates.add(Resources.energySpores);
		}
		Resource r = (Resource) Utils.random(candidates);
		return r;
	}

	public boolean contains(Resource r) {
		if (getDamageBonus() > 0.0 && Comp.same(r, Resources.antiMatter)) return true;
		if (getCloakBonus() > 0 && Comp.same(r, Resources.cloakPalm)) return true;
		if (getPsiBonus() > 0 && Comp.same(r, Resources.psiFungus)) return true;
		if (getMaxDamageBonus() > 0.0 && Comp.same(r, Resources.tritaniumOre)) return true;
		if (getSpeedBonus() > 0.0 && Comp.same(r, Resources.warpCrystals)) return true;
		if (getShieldBonus() > 0.0 && Comp.same(r, Resources.energySpores)) return true;

		return false;
	}
}