/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.ValidationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.crafts.Fleet;
import mot.engine.event.EndTurnEF;
import mot.engine.transports.Transport;

/**
 * A list of players. These list of players can be serialized to XML and
 * de-serialised from XML for saving.
 */
public class Players extends XMLBean {

    private ArrayList list;

    public ArrayList getList() { return list; }
    public void setList(ArrayList v) { list = v; }

    public Players() {
	}

    private transient Player [] players;
    public Player [] getPlayers() {
        if (list == null) return null;
		if (players == null) {
			players = (Player []) list.toArray( new Player[list.size()]);
		}
		return players;
	}
    
    public int getQuantity() {
        if (list == null) {
            return 0;
        } else {
            return list.size();
        }
    }

    public void setUserPlayer(UserPlayer player) {
        add(player);
    }
    
    public void setComputerPlayer(ComputerPlayer player) {
        add(player);
    }
    
    public void add(Player player) {
		players = null;
        if (list == null) list = new ArrayList();
		list.add(player);
	}

    public void remove(Player player) {
        if (list == null) return;
		list.remove(player);
        players = null;
	}

    public UserPlayer [] getUserPlayers() {
		Player [] players = getPlayers();
        if (players == null) {
            return null;
        }
        ArrayList results = new ArrayList();
		for (int i=0; i<players.length; i++) {
			if (players[i] instanceof UserPlayer) {
				results.add(players[i]);
			}
		}
        return (UserPlayer []) results.toArray(new UserPlayer[results.size()]);
    }
    
    public EndTurnEF [] createEndTurnEventForEachUserPlayer() throws ApplicationError {
    	UserPlayer[] upa = getUserPlayers();
    	if (upa == null) return null;
    	EndTurnEF [] r = new EndTurnEF [upa.length];
    	for (int i = 0; i < upa.length; i++) {
    		EndTurnEF eventContainer = new EndTurnEF(upa[i]);
            MotEngine.instance.getProxyManager().scheduleBroadcast(eventContainer, upa[i]);
            r[i] = eventContainer;
		}
    	return r;
    }

    public ComputerPlayer [] getComputerPlayers() {
		Player [] players = getPlayers();
        if (players == null) {
            return null;
        }
        ArrayList results = new ArrayList();
		for (int i=0; i<players.length; i++) {
			if (players[i] instanceof ComputerPlayer) {
				results.add(players[i]);
			}
		}
        return (ComputerPlayer []) results.toArray(new ComputerPlayer[results.size()]);
    }

    /**
     * @return A array containing the names of all player in this game.
     */
    public String [] getPlayerNames() {
        Player [] players = getPlayers();
        if (players == null || players.length == 0) return null;
        String [] r = new String[players.length];
        for (int i=0; i<players.length; i++) {
            r[i] = players[i].getName();
        }
		return r;
	}
    
    /**
     * @return A array containing the names of all user player in this game.
     */
    public String [] getUserPlayerNames() {
        Player [] players = getUserPlayers();
        if (players == null || players.length == 0) return null;
        String [] r = new String[players.length];
        for (int i=0; i<players.length; i++) {
            r[i] = players[i].getName();
        }
		return r;
	}
    
    /**
     * @param player The player to locate in this list.
     * @return True if the player passed in exists in this list.
     */
    public boolean contains(Player player) {
		Player [] players = getPlayers();
        if (players == null) {
            return false;
        }
		for (int i=0; i<players.length; i++) {
			if (players[i] == player) {
				return true;
			}
		}
		return false;
    }
    
    public UserPlayer getUserPlayer(String name) throws ApplicationError {
    	return (UserPlayer) getPlayer(name);
    }
    
    /**
     * This searches with list of players for a player with this name.
     * @param name The name of the player to find.
     * @return The player with the same name passed in or null
     * @throws ApplicationError 
     */
    public Player getPlayer(String name) throws ApplicationError {
		Player [] players = getPlayers();
        if (players == null || name == null) {
            return null;
        }
		for (int i=0; i<players.length; i++) {
			if (players[i].sameName(name)) {
				return players[i];
			}
		}
		MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
		if (mp.sameName(name)) return mp;
		return null;
	}

    public Player getPlayerDeadOrAlive(String name) throws ApplicationError {
    	if (name  == null) return null;
        Players players = MotEngine.instance.getPlayers();
        if (players == null) {
            return null;
        }
        Player player = players.getPlayer(name);
        if (player == null) {
            Players deadPlayers = MotEngine.instance.getDeadPlayers();
            if (deadPlayers != null) {
            	player = deadPlayers.getPlayer(name);
            }
            if (player == null) {
                return null;
            }
        }
        return player;
    }

    public Player [] getDeadOrAlivePlayers() {
        Player [] pa1 = getPlayers();
        Players dp = MotEngine.instance.getDeadPlayers();
        if (dp == null || dp.getQuantity() == 0) {
        	return pa1;
        } else {
        	Player [] pa2 = dp.getPlayers();
        	ArrayList<Player> r = new ArrayList<Player>();
        	for (int i = 0; i < pa1.length; i++) {
				r.add(pa1[i]);
			}
        	for (int i = 0; i < pa2.length; i++) {
				r.add(pa2[i]);
			}
        	return r.toArray(new Player[r.size()]);
        }
	}

    /**
     * This searches with list of players for a fleet with this name.
     * @param name The name of the fleet to find.
     * @return The fleet with the same name passed in or null is not such fleet exists
     * @throws ApplicationError 
     */
    public Fleet getFleet(String name) throws ApplicationError {
		Player [] players = getPlayers();
        if (players == null || name == null) {
            return null;
        }
		for (int i=0; i<players.length; i++) {
            Fleet fleet = players[i].getFleet(name);
			if (fleet != null) {
				return fleet;
			}
		}
		MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
		if (mp != null) {
			Fleet fleet = mp.getFleet(name);
			if (fleet != null) {
				return fleet;
			}
		}
		return null;
	}

	public boolean allUserPlayersMoved() {
		Player [] players = getPlayers();
        if (players == null) return true;
        for (int i=0; i<players.length; i++) {
			Player player = players[i];
			if ( (player instanceof UserPlayer) && (!player.getTurnEnded()) ) {
				return false;
			}
		}
		return true;
	}

    /**
     * Performs an end turn operation for all players.
     */
	public void endTurn() throws ValidationError, ApplicationError {
		Player [] players = getPlayers();
        if (players == null) return;
		for (int i=0; i<players.length; i++) {
            players[i].endTurn();
		}
	}
    
    /**
     * Performs an end turn operation for all players fleets.
     * This is performed separately to a players end turn as we want to move fleets
     * after crafts so crafts can join a fleet before it departs.
     */
    public void endFleetsTurn() throws ApplicationError {
		Player [] players = getPlayers();
        if (players == null) return;
		for (int i=0; i<players.length; i++) {
            Fleet [] fleets = players[i].getFleet();
            for (int j=0; fleets != null && j<fleets.length; j++) {
                fleets[j].endTurn();
            }
		}
        Player mp = MotEngine.instance.getMonsterPlayer();
        if (mp != null) {
            Fleet [] fleets = mp.getFleet();
            for (int j=0; fleets != null && j<fleets.length; j++) {
                fleets[j].endTurn();
            }
        }
    }

	public void startTurn() throws ApplicationError {
		Player [] players = getPlayers();
        if (players == null) return;
		for (int i=0; i<players.length; i++) {
			players[i].startTurn();
		}
	}
    
    /**
     * @return Returns the player in this list of players with the largest power 
     * rating. See Player.getPowerRating() for details.
     */
    public Player getLargestPlayer() throws ApplicationError {
        Player largest = null;
        long rating = 0;
		Player [] players = getPlayers();
        if (players == null) return null;
		for (int i=0; i<players.length; i++) {
            long p = players[i].getPowerRating();
			if (largest == null || p > rating) {
                largest = players[i];
                rating = p;
            }
		}
        return largest;
    }

    /**
     * Searched for any players who meat the end game criteria. If any player is 
     * found that player is destroyed.
     * @throws mot.common.util.ApplicationError
     */
    public void destroyFinishedPlayers() throws ApplicationError {
		Player [] players = getPlayers();
        if (players == null) return;
		for (int i=0; i<players.length; i++) {
            if (players[i].hasLostGame()) {
                players[i].destroy();
                return;
            }
        }
    }
    
    /**
     * @return The transport with name or null. 
     */
    public Transport getTransport(String name) {
        Player [] players = getPlayers(); 
		if (players != null) {
			for (int i=0; i<players.length; i++) {
                if (players[i].getTransportManagerNotNull() != null) {
                    Transport t = players[i].getTransportManager().getTransport(name);
                    if (t != null) return t;
                }
			}
		}
        return null;
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        Player [] players = getPlayers(); 
		if (players != null) {
			for (int i=0; i<players.length; i++) {
				if (players[i] != null) {
					players[i].toXML(writer);
				}
			}
		}
	}

	protected String getElementName() { return "players"; }
	
	/**
	 * @return True iff this list of players contains one or more players that are at war.
	 * @throws ApplicationError 
	 */
	public boolean hasEnemies() throws ApplicationError {
        Player [] players = getPlayers(); 
		if (players != null) {
			for (int i=0; i<players.length; i++) {
				if (players[i] != null && hasEnemy(players[i])) {
					return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * @return True iff this list of players contains one or more players that are at war with the player passed in.
	 * @throws ApplicationError 
	 */
	public boolean hasEnemy(Player player) throws ApplicationError {
        Player [] players = getPlayers(); 
		if (players != null) {
			for (int i=0; i<players.length; i++) {
				if (players[i] != null && players[i] != player && player.isEnemy(players[i])) {
					return true;
				}
			}
		}
		return false;
	}
	
	public Player [] getPlayersWithAlignment(String ancientName) {
		Player [] players = getPlayers();
        if (players == null) {
            return null;
        }
        ArrayList results = new ArrayList();
		for (int i=0; i<players.length; i++) {
			if (Comp.same(players[i].getAlignment(), ancientName)) {
				results.add(players[i]);
			}
		}
        return (Player []) results.toArray(new Player[results.size()]);
	}
}