/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.TechnologyTree;
import mot.common.technology.TechnologyTreeNode;
import mot.common.technology.Theories;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;

/**
 * 
 * Manages technologies that this player has NOT discovered but are out of date.
 * These are technologies the player has chosen to ignore.
 * To speed up the game these technologies can be discovered over time by themselves.
 * 
 * @author Paul Davis
 *
 */
public class ResearchOutdatedTechnologies {

	private Player owner;
	
	private TechnologyList outdatedTechnologies = new GeneralTechnologyList();
	
	public ResearchOutdatedTechnologies(Player owner) throws ApplicationError {
		this.owner = owner;
		
		TechnologyTree techTree = TechnologyTree.getInstance();
		outdatedTechnologies.removeAll();
		
		Technology[] available = owner.getTechnologyAvailableForResearch();
		Theories theories = owner.getTheories();
		if (available != null && theories != null && theories.getQuantity() > 0) {
			TechnologyTreeNode mostAdvancedTheoryDiscovered = techTree.getNode(theories.getMostExpensiveToResearch());
			TechnologyTreeNode [] oldTheoryDiscoveredNode = techTree.getParentNodesAsArray(mostAdvancedTheoryDiscovered);
			for (int i = 0; i < available.length; i++) {				
				TechnologyTreeNode availableToResearchNode = techTree.getNode(available[i]);
				Technology[] availableToResearchParents = availableToResearchNode.getParentsAsArray();
				if (isDependentOnParentIn(availableToResearchParents, oldTheoryDiscoveredNode)) {
					outdatedTechnologies.add(available[i]);
				}
			}
		}
	}

	private boolean isDependentOnParentIn(Technology[] availableToResearchParents, TechnologyTreeNode[] oldTheoryDiscoveredNode) throws ApplicationError {
		if (oldTheoryDiscoveredNode == null) return false;
		TechnologyTree techTree = TechnologyTree.getInstance();
		for (int i = 0; i < oldTheoryDiscoveredNode.length; i++) {
			if (Comp.in(oldTheoryDiscoveredNode[i].getNode(), availableToResearchParents)) {
				return true;
			}
			TechnologyTreeNode [] olderTheoryDiscoveredNode = techTree.getParentNodesAsArray(oldTheoryDiscoveredNode[i]);
			if (olderTheoryDiscoveredNode != null && olderTheoryDiscoveredNode.length > 0) {
				if (isDependentOnParentIn(availableToResearchParents, olderTheoryDiscoveredNode)) {
					return true;
				}
			}
		}
		return false;
	}

	public Technology selectRandomTechnology() {
		return outdatedTechnologies.getRandom();
	}

	public Technology[] getTechnologies() {
		return outdatedTechnologies.getAllAsArray();
	}
	
	
}
