/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import mot.common.bean.GalaxyDefinitions;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyReference;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * This is a Project owned by a single player to research a single technology.
 * When the technology is discovered or this player is killed this research project
 * will be deleted.
 */
public class ResearchProject extends XMLBean {

	/**
	 * The technology being researched.
	 */
	private Technology technology;

	/**
	 * The project owner.
	 */
	private PlayerReference playerReference;

	/**
	 * The work done so far in discovering the technology.
	 */
	private int work;

	/**
	 * True iff this research is on a salvage item this is the chance per turn that this
	 * technology will be discovered.
	 */
	private double fromSalvageProbabilityPerTurn;
	
    public ResearchProject() {
	}

    public ResearchProject(Technology technology, Player owner, boolean fromSalvage) throws ApplicationError {
		this(technology, new PlayerReference(owner), fromSalvage);
	}

    public ResearchProject(Technology technology, PlayerReference owner, boolean fromSalvage) throws ApplicationError {
		this.technology = technology;
		this.playerReference = owner;
		this.fromSalvageProbabilityPerTurn = 0.0;
        this.work = 0;
        
        if (fromSalvage) {
        	double cost = getCostToResearchFromSalvage();
        	double researchPerTurn = owner.getPlayer().getTotalResearch();
        	double chanceToDiscoverSalvagedTechnology = owner.getPlayer().getChanceToDiscoverSalvagedTechnology();
        	double qtyTurns = cost / researchPerTurn; 
        	fromSalvageProbabilityPerTurn = chanceToDiscoverSalvagedTechnology/qtyTurns; 
        }
	}

    /**
     * Adds this amount of work to this research project.
     * If this completes the project then finishResearchProject is called on the player
     * object and any remaining research effort is returned from this method.
     */
	public int addResearch(int amount) throws ApplicationError {
		if (Logger.isDebug()) {
            Logger.logDebug("ResearchProject.addResearch() amount="+amount+" to "+getProjectDescription());
        }
		if (technology == null) {
			return amount;
		}
		if (fromSalvageProbabilityPerTurn > 0.0) {
			work += amount;
			return 0;
		} else {
			int cost = getCostToResearch();
			int remainingWork = 0;
			if ((work + amount) > cost) {
				remainingWork = (work + amount) - cost;
				work = cost;
				playerReference.getPlayer().finishResearchProject(this, true, false);
			} else {
				work += amount;
			}
			return remainingWork;
		}
	}

//    public int getWorkRequiredToCompleteProject() throws ApplicationError {
//    	if (fromSalvageProbability > 0.0) {
//			return getCostToResearch();
//    	} else {
//			return technology.getCostToResearch();
//    	}
//    }

	public int getCostToResearchFromSalvage() {
		return technology.getCostToResearch()*GalaxyDefinitions.FROM_SALVAGE_COST_FACTOR;
	}
	
	public int getCostToResearch() {
		if (fromSalvageProbabilityPerTurn > 0.0) {
			return technology.getCostToResearch()*GalaxyDefinitions.FROM_SALVAGE_COST_FACTOR;
		} else {
			return technology.getCostToResearch();
		}
	}

    public void endTurn() throws ApplicationError {
    	if (fromSalvageProbabilityPerTurn > 0.0) {
    		Player player = playerReference.getPlayer();
			if (Utils.random(fromSalvageProbabilityPerTurn)) {
    			player.finishResearchProject(this, true, true);
    		} else {
    			int cost = getCostToResearch();
    			if (work > cost) {
    				double chanceToDiscoverSalvaged = player.getChanceToDiscoverSalvagedTechnology();
    		    	if (chanceToDiscoverSalvaged > 0.98) {
    	    			player.finishResearchProject(this, true, true);
    		    	} else {
    		    		player.finishResearchProject(this, false, true);
    		    	}
    			}
    		}
    	}
    }
    
    public String getProjectDescription() {
    	if (technology != null) {
    		int cost = getCostToResearch();
    		if (fromSalvageProbabilityPerTurn > 0.0) {
    			return "Researching a "+technology.getTechType()+" called "+technology.getName()+" from salvage [ "+work+" of "+technology.getCostToResearch()*GalaxyDefinitions.FROM_SALVAGE_COST_FACTOR+" ]";
    		} else {
    			return "Researching a "+technology.getTechType()+" called "+technology.getName()+" [ "+work+" of "+technology.getCostToResearch()+" ]";
    		}
    	} else {
    		return "Researching nothing";
    	}
    }

	public Player getOwner() throws ApplicationError {	return playerReference.getPlayer(); }
	public PlayerReference getPlayerReference() { return playerReference; }
	public Technology getTechnology() {	return technology; }
	public int getWork() { return work; }

	public void setOwner(Player v) { playerReference = new PlayerReference(v, v.getName()); }
	public void setPlayerReference(PlayerReference v) {	playerReference = v; }
	public void getTechnology(Technology v) { technology = v; }
	public void setWork(int v) { work = v; }

	protected void encode(XMLWriter writer) throws ApplicationError {
        if (technology != null) {
            new TechnologyReference(technology).toXML(writer);
        }
		if (playerReference != null) {
			playerReference.toXML(writer);
		}
		writer.addElement("work", work);
		writer.addElement("from-salvage-probability", fromSalvageProbabilityPerTurn);
	}

	protected String getElementName() { return "research-project"; }
    
    /*
     * When a Research project is Serialised to XML it does not serialise the technology its self but
     * a reference to the technology. This is used to deserialise the projects technology.
     */
    public void setTechnologyReference(TechnologyReference v) throws ApplicationError {
        technology = v.getTechnology();
    }

	public double getFromSalvageProbability() {
		return fromSalvageProbabilityPerTurn;
	}

	public void setFromSalvageProbability(double v) {
		this.fromSalvageProbabilityPerTurn = v;
	}

	@Override
	public String toString() {
		return "ResearchProject [technology=" + technology
				+ ", player=" + playerReference + ", work=" + work
				+ ", fromSalvageProbability=" + fromSalvageProbabilityPerTurn + "]";
	}


}