/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;

/**
 * Contains race setup option information.
 */
public class ResearchScriptOptions extends XMLBean {

	private ArrayList options;

	public ArrayList getOptions() { return options; }
	public void setOptions(ArrayList v) { options = v; array = null; }
	public void setResearchScriptOption(ResearchScriptOption v) { if (options == null) options = new ArrayList(); options.add(v); array = null; }
    public void removeResearchScriptOption(ResearchScriptOption v) { if (options == null) return; options.remove(v); array = null; }
    
	transient ResearchScriptOption [] array = null;
	public ResearchScriptOption [] getResearchScriptOptionsAsArray() {
		if (array == null && options != null && options.size() > 0) {
			array = (ResearchScriptOption []) options.toArray(new ResearchScriptOption [options.size()]);
		}
		return array;
	}
    
    public int size() { if (options == null) return 0; return options.size(); }
    public ResearchScriptOption get(int index) { if (options == null) return null; return (ResearchScriptOption) options.get(index); }

    public int indexOf(ResearchScriptOption v) { 
        if (options == null) return -1; 
        ResearchScriptOption [] rso = getResearchScriptOptionsAsArray();
        for (int i=0; i<rso.length; i++) {
            if (v.same(rso[i])) {
                return i;
            }
        }
        return -1; 
    }

    /**
     * Add the list of options passed in to the current list of options.
     * @param so The list of options to add
     * @param override If true options specified in so will replace those in options otherwise they will not.
     */
    public void add(ResearchScriptOptions so, boolean override) throws ApplicationError {
        if (so == null || options == null) {
            return;
        }
        ResearchScriptOption [] a = so.getResearchScriptOptionsAsArray();
        for (int i=0; i<a.length; i++) {
            boolean found = exists(a[i].getName());
            if (found && override) {
                ResearchScriptOption old = getForName(a[i].getName());
                removeResearchScriptOption(old);
                setResearchScriptOption(a[i]);
            } else if (!found) {
                setResearchScriptOption(a[i]); 
            }
        }
    }

    /**
     * If an option with name does not exist it creates a new option with name name
     * and value value, otherwise it updates the option name with value value.
     */
    public void setResearchScriptOption(String name, String type) throws ApplicationError {
        ResearchScriptOption so = getForName(name);
        if (so == null) {
            setResearchScriptOption(new ResearchScriptOption(name, type));
        } else {
            so.setType(type);
        }
    }

	public boolean exists(String name) throws ApplicationError {
		ResearchScriptOption so = getForName(name);
		return (so != null);
	}

	public ResearchScriptOption getForName(String name) throws ApplicationError {
		ResearchScriptOption [] so = getResearchScriptOptionsAsArray();
		if (so != null) {
			for (int i=0; i<so.length; i++) {
				if ( Comp.same(so[i].getName(), name) ) {
					return so[i];
				}
			}
		}
		return null;
	}

	protected String getElementName() { return "research-script-options"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
		for (int i=0; i<options.size(); i++) {
			ResearchScriptOption r = (ResearchScriptOption) options.get(i);
            if (r != null) {
                r.toXML(writer);
            }
		}
	}
}