/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;
import mot.common.event.BattleObjectNotificationType;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.battles.HoloCraft;
import mot.engine.event.BattleObjectNotificationEF;
import mot.engine.technology.ShieldInstance;

public class ResourcefulProcessorBasic implements ResourcefulProcessor {
	
	private static final double PERCENT_SHIELD_RECHARGE = 0.3;
	private static final double PERCENT_DAMAGE_REPAIRED = 0.1;
	
    public ResourcefulProcessorBasic() {
    }

    public void execute(BattleObject bo) throws ApplicationError {
    	if (bo instanceof HoloCraft) return;
    	
    	if (bo.getOwner().isResourceful()) {
    		ShieldInstance [] shields = bo.getShields(); 
    		if (shields != null) {
    			for (int i=0; i<shields.length; i++) {
    				if (shields[i].getShield().getDamageBlocked() > 0.0) {
    					double recharge = shields[i].getShield().getRechargeRate();
    					double power = shields[i].getPower();
    					if (recharge >= 0 && power <= recharge && Utils.random(0.25)) {
    						rechargeShields(bo, shields[i]);
    					}
    				}
    			}
    		}
    	}

        if (bo instanceof BattleCraft) {
    		BattleCraft bc = (BattleCraft) bo;
    		double resourcefulRepairAmount = 0.0;
    		double autoRepairAmount = 0.0;
            if (bc.getOwner().isResourceful() && bc.getDamageAsPercent() >= 0.75) {
                if (Utils.random(0.25)) {
                	resourcefulRepairAmount = bc.getCraft().getMaxDamage()*PERCENT_DAMAGE_REPAIRED;
                }
            }
    		if (bc.getCraft().getDamage() > 0.0 && bc.getSpecials() != null) {
    			Double autoRepair = bc.getSpecials().getRepairEffect();
    			if (autoRepair != null) {
    				autoRepairAmount = bc.getCraft().getMaxDamage()*autoRepair; 
    			}
    		}
    		if (resourcefulRepairAmount > 0 || autoRepairAmount > 0) {
    			repairDamage(bc, resourcefulRepairAmount, autoRepairAmount);
    		}
    	}

    }
    
    
    private void rechargeShields(BattleObject bo, ShieldInstance shield) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("ResourcefulProcessorBasic.rechargeShields() craft="+bo);
        }
        CraftStateFactory originalState = bo.getCraftState();
        int power = (int)(shield.getMaxRecharge() * PERCENT_SHIELD_RECHARGE);
        shield.addPower(power);
        String msg = L.RESOURCEFUL_SHIELD+power;
        Battle battle = bo.getBattle();
        BattleObjectNotificationEF event = new BattleObjectNotificationEF(bo, msg, BattleObjectNotificationType.repair, originalState, bo.getCraftState());
        MotEngine.instance.multicast(event, battle.getStartingUserPlayers());
    }

    
    private void repairDamage(BattleCraft bc, double resourcefulRepairAmount, double autoRepairAmount) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("ResourcefulProcessorBasic.repairDamage() craft="+bc+" resourcefulRepairAmount="+resourcefulRepairAmount+" autoRepairAmount="+autoRepairAmount);
        }
		String msg = L.AUTOMATED_REPAIR+"+"+Formatter.whole(resourcefulRepairAmount+autoRepairAmount);
		CraftStateFactory originalState = bc.getCraftState();
		bc.getCraft().repair(resourcefulRepairAmount+autoRepairAmount);
		BattleObjectNotificationEF event = new BattleObjectNotificationEF(bc, msg, BattleObjectNotificationType.repair, originalState, bc.getCraftState());
		Battle battle = bc.getBattle();
		MotEngine.instance.multicast(event, battle.getStartingUserPlayers());
    }
}