/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import mot.common.bean.NamedObject;
import mot.common.util.ApplicationError;
import mot.engine.MotEngine;
import mot.engine.relations.Relations;

/**
 * A Scientist decides what technology to research for a computer player. It attempts to prioritise
 * the importance of each research project and determine the next technology to research when a computer 
 * player is required to select its next research project.
 */
public abstract class Scientist extends NamedObject {

	private Player owner;

	public Scientist() {
	}

	public Scientist(Player owner) {
		super(owner);
		this.owner = owner;
	}

	public abstract ResearchProject getNextResearchProject(ResearchScriptOption lastDiscovery, Player owner) throws ApplicationError;
    
    /**
     * Called to inform the scientist that this player needs the ability to travel this distance.
     * It might even consifer fast tracking a better engine design!
     * @param dist The distance needed.
     */
    public abstract void notifyNeedEngineForDistance(Player owner, int dist) throws ApplicationError;

	/**
	 * Called every turn to give the scientist an opportunity to request research treaties.
	 * @throws ApplicationError 
	 */
	public abstract void assignResearchTreaty() throws ApplicationError;

    /**
     * Two player can form a trade route if
     *  - They are not the same player
     *  - Have established diplomatic relations
     *  - Are not at war
     *  - Neither player already has a research treaty
     * @param p1 Player wishing to form a trade route
     * @param p2 Player wishing to form a trade route
     * @param traders The current list of players trading with p1
     * @return True if and only if player p1 and p2 can form a trade routes at this point in time
     */
    public static boolean canFormResearchTreaty(Player p1, Player p2) throws ApplicationError {
        if (p1 == p2) {
            return false;
        }
        
        if (!p1.hasDiplomaticRelationsWith(p2)) {
            return false;
        }
        
        Relations rels = MotEngine.instance.getRelations();
        if (rels.isWar(p1, p2)) {
            return false;
        }
        
        if (rels.getQtyResearchersWith(p1) > 0 || rels.getQtyResearchersWith(p2) > 0) {
        	return false;
        }
        
        return true;
    }
	
	public Player getOwner() {
		return owner;
	}
	
  
}