/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.util.ArrayList;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.DiplomaticStatements;
import mot.common.bean.SalvageList;
import mot.common.technology.Engine;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;

/**
 * A Scientist decides what technology to research for a computer player. It attempts to prioritise
 * the importance of each research project and determine the next technology to research when a computer 
 * player is required to select its next research project.
 * 
 * This Scientist calculates all technology available for research by this player. It then chooses the
 * cheapest technology available for research from those technologies.
 */
public class ScientistBasic extends Scientist {

	public static final int CALCULATE_INTERVAL = 21;
	public static final int CALCULATE_START = 50;
	
	/**
	 * The date of the last time this player calculates its priorities.
	 */
	protected int dateOfLastCalculation;

	public int getDateOfLastCalculation() { return dateOfLastCalculation; }
	public void setDateOfLastCalculation(int v) { dateOfLastCalculation = v; }

	public ScientistBasic() {
	}

	public ScientistBasic(Player p) {
		super(p);
	}

	public ResearchProject getNextResearchProject(ResearchScriptOption lastDiscovery, Player owner) throws ApplicationError {
		SalvageList salvage = owner.getSalvage();
		if (salvage != null) {
			Technology[] salvageTechs = salvage.getAllAsArray();
			if (salvageTechs != null) {
				for (int i = 0; i < salvageTechs.length; i++) {
					if (owner.hasDiscovered(salvageTechs[i])) {
						owner.sell(salvageTechs[i]);
					} else {
						salvage.remove(salvageTechs[i]);
						return new ResearchProject(salvageTechs[i], owner, true);
					}
				}
			}
		}
		
        Technology[] availableArray = owner.getTechnologyAvailableForResearch();
        TechnologyList available = new GeneralTechnologyList(availableArray);
        TechnologyList selected = available.getCheapestToResearch(0, null);
        
        if (selected == null || selected.getQuantity() == 0) {
            if (Logger.isDebug()) {
                Logger.logDebug("ScientistBasic.getNextResearchProject() No more techs");
            }
			return null;
        }
        
        Technology selectedTech = selected.getTechnologyAtIndex(0);
        return new ResearchProject(selectedTech, owner, false);
	}

    /**
     * It cheats - just adds this tech to the player.
     * This prevents computer players from being unable to colonise planets for a long 
     * time!
     * @param dist The distance needed.
     */
    public void notifyNeedEngineForDistance(Player owner, int dist) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("ScientistBasic.notifyNeedEngineForDistance() Player "+owner.getName()+" needs engine for distance "+dist);
        }
        Technology[] availableArray = owner.getTechnologyAvailableForResearch();
        if (availableArray != null) {
            for (int i=0; i<availableArray.length; i++) {
                if (availableArray[i] instanceof Engine) {
                    Engine engine =(Engine)availableArray[(i)];
                    owner.addEngine(engine);
                }
            }
        }
    }

	public void assignResearchTreaty() throws ApplicationError {
		int date = MotEngine.getInstance().getDate();
		if ( (date - dateOfLastCalculation) < CALCULATE_INTERVAL) {
			return;
		}
		if (MotEngine.instance.getLevel() <= 1.0 && date < MotEngine.START_DATE+CALCULATE_START) {
			// Lets not bombard beginners with treaty requests.
			return;
		}
        dateOfLastCalculation = date;

        Player owner = getOwner();
        int maxTradeRoutes = owner.getMaxPlayerTradeRoutes();
        Player [] researchers = MotEngine.instance.getRelations().getResearchersWith(owner);
        if (researchers == null || researchers.length < maxTradeRoutes) {
            try {
                int qtyResearchers = (researchers == null) ? 0 : researchers.length;
                if (qtyResearchers == 0) {
                    createResearchTreaty();
                }
            } catch (ValidationError e) {
                // Ignore - there is nobody we can create a trade route with!
            }
        }
	}
	
    private void createResearchTreaty() throws ApplicationError, ValidationError {
        Player owner = getOwner();
        Player [] researchers = MotEngine.instance.getRelations().getResearchersWith(owner);
        Player [] all = MotEngine.instance.getPlayers().getPlayers();        
        
        // First create a list of all players that are eligible for us to form a
        // research relationship with.
        ArrayList candidates = null;
        for (int i=0; i<all.length; i++) {
            if (Scientist.canFormResearchTreaty(owner, all[i])) {
                if (candidates == null) candidates = new ArrayList();
                candidates.add(all[i]);
            }
        }
        
        // Now form a research relationship with a player from the candidates.
        if (candidates == null || candidates.size() == 0) {
            Logger.logDebug("ScientistBasic.createResearchTreaty() No eligable players to create trade route with");
            throw new ValidationError("There are no eligable races for us to create a trade route with");
        }
        
        Player player = (Player) Utils.random(candidates);
        
        if (Logger.isDebug()) {
            Logger.logDebug("ScientistBasic.createResearchTreaty() request research treaty between "+owner.getName()+" and "+player.getName());
        }
        
        PlayerCharacter pc = owner.getPlayerCharacter();
        String msg = pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_RESEARCH);
        player.diplomaticRequest(getOwner(), DiplomaticProposal.getResearch(), msg);
    }

	
	protected String getElementName() { return "scientist-basic"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
	}

}