/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;

import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLDocumentHandler;
import mot.common.xml.XMLWriter;
import mot.engine.players.strategists.StrategistResponse;

/**
 * A Scientist decides what technology to research for a computer player. It attempts to prioritise
 * the importance of each research project and determine the next techonology to research when a computer 
 * player is required to select its next research project.
 * 
 * This Scientist uses a research script - A Research script is a an ordered list of all technologies in 
 * the system. To determine the next research project it selects lowest technology in the list that has 
 * not been discovered.
 * 
 * It searches for a file called research_script_RACE_NAME.xml, if it cannot find 
 * that it searches for a file called research_script_general.xml, if it cannot find 
 * that it generates a default list which ir all technologies ordered in the cost 
 * to research.
 */
public class ScientistScript extends Scientist {

	/**
	 * Same a techs but it is the default list. A player may have a race specific research tree
	 * or use the general one.
	 */
	private static ResearchScriptOptions techsDefault;

    /**
     * Generates a list of technologies to be discovered in order of cheapest to research to most 
     * expensive. This is used to generate a research script when no XML research script exists.
     */
	private static synchronized ResearchScriptOptions getDefault() throws ApplicationError {
		if (techsDefault == null) {
            techsDefault = new ResearchScriptOptions();
            int currentCost = 0;
            Technology[] techs = mot.common.technology.Technologies.getCheapestToResearch(currentCost);
            while (techs != null) {
                for (int i=0; i<techs.length; i++) {
                    techsDefault.setResearchScriptOption(techs[i].getName(), techs[i].getTechType());
                }
                if (techs.length > 0) {
                    currentCost = techs[0].getCostToResearch();
                }
                techs = mot.common.technology.Technologies.getCheapestToResearch(++currentCost);
            }
		}
		return techsDefault;
	}

	/**
	 * The list of technoliges in discovery order this script dictates the player should
	 * discover. Its type is ArrayList of ResearchScriptOption.
	 */
	private ResearchScriptOptions techs;

	public ScientistScript() {
	}

	public ScientistScript(Player p) {
		super(p);
	}

	public ResearchProject getNextResearchProject(ResearchScriptOption lastDiscovery, Player owner) throws ApplicationError {
		if (techs == null) {
			String fileName = "xml/research_script_"+owner.getName()+".xml";
			URI uri = null;
			try {
				uri = new URI(fileName);
			} catch (URISyntaxException e) {
				Logger.logError("", e);
			}
			if (uri != null) {
				Object object = XMLDocumentHandler.parseResource(fileName);
				techs = (ResearchScriptOptions) object;
			} else {
				fileName = "xml/research_script_general.xml";
				try {
					uri = new URI(fileName);
				} catch (URISyntaxException e) {
					Logger.logError("", e);
				}
				if (uri != null && new File(uri).exists()) {
                    Object object = XMLDocumentHandler.parseResource(fileName);
                    techs = (ResearchScriptOptions) object;
                } else {
                    techs = getDefault();
                }
			}
		}
        
        int index = -1;
        if (lastDiscovery != null) {
            index = techs.indexOf(lastDiscovery);
            if (index < 0) {
                throw new ApplicationError("ScientistScript.getNextResearchProject() Unable to find last discovery "+lastDiscovery);
            }
        }
		int nextIndex = index + 1;
		if (nextIndex >= techs.size()) {
            if (Logger.isDebug()) {
                Logger.logDebug("ScientistScript.getNextResearchProject() No more techs");
            }
			return null;
		}

        // If for some reason the player already has this tech - do not research it!
		ResearchScriptOption nextDiscovery = techs.get(nextIndex);
        while ( owner.hasDiscovered(nextDiscovery.getType(), nextDiscovery.getName()) ) {
            if (Logger.isDebug()) {
                Logger.logDebug("ScientistScript.getNextResearchProject() Skip research project="+nextDiscovery.getType()+" "+nextDiscovery.getName()+" becuase player "+owner.getName()+" has already discovered it");
            }
            nextIndex++;
            if (nextIndex >= techs.size()) {
                if (Logger.isDebug()) {
                    Logger.logDebug("ScientistScript.getNextResearchProject() No more techs!!");
                }
                return null;
            }
    		nextDiscovery = techs.get(nextIndex);
        }

		if (Logger.isDebug()) {
			Logger.logDebug("ScientistScript.getNextResearchProject() Next research project="+nextDiscovery.toString());
		}

        Technology tech = mot.common.technology.Technologies.getTechnology(nextDiscovery.getType(), nextDiscovery.getName());
        return new ResearchProject(tech, new PlayerReference(owner), false );
	}

    public void notifyNeedEngineForDistance(Player owner, int dist) throws ApplicationError {
        
    }


	protected String getElementName() { return "scientist-basic"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
	}

	public StrategistResponse requestResearchTreaty(Player player) {
		return new StrategistResponse(false, "");
	}

	@Override
	public void assignResearchTreaty() {
		// TODO Auto-generated method stub
		
	}

}