/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import mot.common.bean.stats.Graph;
import mot.common.bean.stats.Statistic;
import mot.common.util.ApplicationError;
import mot.engine.MotEngine;

/**
 * This is used to create graphical information about the ascent / decline of an empire.
 * Each player at the end of every turn should call generate() at the end of every turn,
 * this factory will then generate statistical information which can be viewed as a graph.
 */
public class StatisticFactory  {

    private static final int INTERVAL = 25;
    
    /**
     * @returns the graph object for this player. If the player does not have a graph it 
     * creates the graph and returns it.
     */
    private static Graph getGraph(Player player) throws ApplicationError {
        Graph graph = player.getGraph();
        if (graph == null) {
            graph = new Graph();
            player.setGraph(graph);
        }
        return graph;
    }

    /**
     * Generates a new statistics for population fleet and research each INTERVAL turns.
     * @param player The player for whome the statistics are generated.
     * @throws mot.common.util.ApplicationError
     */
    public static void generate(Player player) throws ApplicationError {
        int date = MotEngine.instance.getDate();
        Graph graph = getGraph(player);
        long lastDate = graph.getLastDate();
        if (lastDate <= 0) {
            Statistic stat = graph.getLast(mot.client.deligates.GraphDelegate.POPULATION);
            if (stat != null) {
                lastDate = stat.getDate();
            } else {
                lastDate = date;
                _generate(player);
                return;
            }
        }
        
        if (date >= (lastDate + INTERVAL)) {
            _generate(player);
        }
    }
    
    /**
     * Generates a new statistics for population fleet and research.
     * @param player The player for whome the statistics are generated.
     * @throws mot.common.util.ApplicationError
     */
    public static void _generate(Player player) throws ApplicationError {
        Graph graph = getGraph(player);
        graph.setLastDate(MotEngine.instance.getDate());
        int date = MotEngine.instance.getDate();
        graph.add(mot.client.deligates.GraphDelegate.POPULATION, date, player.getTotalPopulation());
        graph.add(mot.client.deligates.GraphDelegate.FLEET, date, player.getQtyCrafts());
        graph.add(mot.client.deligates.GraphDelegate.RESEARCH, date, player.getTotalResearch());
    }

}