/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.PlayerColor;
import mot.common.bean.Settings;
import mot.common.bean.stats.GameScoreData;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.designs.Orbital;
import mot.common.leaders.GeneralDetails;
import mot.common.leaders.GovenorDetails;
import mot.common.morals.MoralChoice;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Module;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.vo.PlayerTechnologyVO;
import mot.common.vo.ResearchProjectUpdateVO;
import mot.common.vo.TechnologyAvailableForResearchVO;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattlePlanet;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.event.ColonisePlanetEF;
import mot.engine.event.DiplomaticRequestEF;
import mot.engine.event.DiplomaticResponseEF;
import mot.engine.event.MessageEF;
import mot.engine.event.NoProjectEF;
import mot.engine.event.NoResearchProjectEF;
import mot.engine.event.OfferLeaderEF;
import mot.engine.event.OfferMoralChoiceEF;
import mot.engine.event.PlanetColonisedEF;
import mot.engine.event.SabotageEF;
import mot.engine.event.TechnologyDiscoveredEF;
import mot.engine.event.TechnologyStolenEF;
import mot.engine.event.TradeRouteInvalidEF;
import mot.engine.planets.Planet;
import mot.engine.planets.TradeRoute;
import mot.engine.players.commanders.Commander;
import mot.engine.players.commanders.CommanderBasic;
import mot.engine.spies.SpyCenter;
import mot.engine.transports.Transport;

public class UserPlayer extends Player {

    /**
     * The list of craft designs this player currently has in use.
     */
    private CraftDesigns craftDesignes;
    
    /**
     * This manages the recruiting of new spies and the total quantity of spies 
     * recruited by this player so far.
     */
    private SpyCenter spyCenter;
    
    /**
     * This is the amount of money should be added to the total spy effort of the spy 
     * centre each turn.
     * NOTE - The effort is therefore equal to the amount of money allocated.
     */
    private int spyRecruitEffortPerTurn;
    
    /**
     * This is the location all new military craft should move to when created.
     * Allows a player to set a location to congregate a fleet to attack from.
     */
    private Orbit fleetWayPoint;
    
    /**
     * A flag to prevent a player having multiple No Research Project notifications sent 
     * a single turn. Once a No Research project notification is sent this flag is set to 
     * true which prevents future No Research Project events from being sent. At the end 
     * of every turn it is set to false. 
     */
    private boolean noResearchProjectNotified = false;
    
    /**
     * This is a leader offered to a player.
     * The engine remembers the last leader offered to a player so that when an AcceptLeaderCmd
     * is submitted to the engine by the UI, it can be validated against the offer of a leader.
     */
    private String offeredLeader;
    
    /**
     * Information recorded during a game to produce a score.
     */
    private GameScoreData gameScoreData;
    
    public CraftDesigns getCraftDesigns() { return craftDesignes; }
    public void setCraftDesigns(CraftDesigns v) { craftDesignes = v; }

    public CraftDesign [] getCraftDesignsAsArray() {
    	if (craftDesignes == null) {
    		return null;
    	} else {    		
    		return craftDesignes.getCraftDesigns(); 
    	}
    }

    public SpyCenter getSpyCenter() { 
        if (spyCenter == null) spyCenter = new SpyCenter();
        return spyCenter; 
    }
    public int getQtyBuiltSpies() { return getSpyCenter().getQtyBuiltSpies(); }
    
	public UserPlayer() {
	}

	public UserPlayer(String name, PlayerCharacter character, PlayerColor color, String alignment) {
        super(name, character, color, alignment);
	}

	public void endTurn() throws ValidationError, ApplicationError  {
		if (Logger.isDebug()) {
            Logger.logDebug("UserPlayer.endTurn()");
        }
        noResearchProjectNotified = false;
        super.endTurn();
        if (spyRecruitEffortPerTurn > 0) {
            getSpyCenter().addEffort(spyRecruitEffortPerTurn);
        }
        if (spyCenter != null) {
            spyCenter.endTurn(this);
        }
        checkOutdatedTechnologyDiscovery();
	}
	
	public void checkOutdatedTechnologyDiscovery() throws ApplicationError {
		if (Utils.random(0.1)) {
        	if (outdatedTechnologiesAvailableForResearch == null) {
        		outdatedTechnologiesAvailableForResearch = new ResearchOutdatedTechnologies(this);
        	}
        	Technology tech = outdatedTechnologiesAvailableForResearch.selectRandomTechnology();
        	if (tech != null) {
        		addTechnology(tech);
        		MessageEF event = new MessageEF("Outdated Technology "+tech.getName()+" has been discovered");
        		MotEngine.instance.broadcast( event, this );
        	}
        }
	}

	public void notifyNoProject(Planet planet, String reason) throws ApplicationError {
        if ( !planet.ownerWantsNoProject ) {
    		MotEngine.instance.broadcast( new NoProjectEF(planet, reason), this );
        }
	}

	public void notifyEcologyFinished(Planet planet) throws ApplicationError {
	}

	public void notifyFactoriesFinished(Planet planet) throws ApplicationError {
	}

	public void notifyTechnologyDiscovered(Technology tech, boolean fromSalvage) throws ApplicationError {
		// The player will create a new project using NoResearchProjectEvent
        // But we sent this one just in case he wants to display a you just discovered window.
		super.notifyTechnologyDiscovered(tech, fromSalvage);
		TechnologyDiscoveredEF event = new TechnologyDiscoveredEF(tech, fromSalvage);
		event.addStateUpdate(new ResearchProjectUpdateVO(getName(), EngineVoFactory.getResearchProject(getResearchProject())));
		event.addStateUpdate(new PlayerTechnologyVO(tech.getFullName(), getName()));
		event.addStateUpdate(new TechnologyAvailableForResearchVO(new GeneralTechnologyList(getTechnologyAvailableForResearch())));
		MotEngine.instance.broadcast( event, this );
	}

	public void notifyNoResearchProject(boolean updateTechsToResearch) throws ApplicationError {
        if (!noResearchProjectNotified) {
        	noResearchProjectNotified = true;            
            NoResearchProjectEF event = new NoResearchProjectEF(this);
            if (updateTechsToResearch) {
        		event.addStateUpdate(new TechnologyAvailableForResearchVO(new GeneralTechnologyList(getTechnologyAvailableForResearch())));
            }
			MotEngine.instance.broadcast( event, this );
        }
	}

	public void notifyPlanetColonised(Planet planet) throws ApplicationError {
		MotEngine.instance.broadcast( new PlanetColonisedEF(planet) );
        super.notifyPlanetColonised(planet);
    }

	public void notifyTradeRouteInvalid(TradeRoute tradeRoute) throws ApplicationError {
        TradeRouteInvalidEF event = new TradeRouteInvalidEF(tradeRoute.getSolarSystem1Name(), tradeRoute.getSolarSystem2Name());
		MotEngine.instance.broadcast( event, this );	
	}

	/**
     * Called when another player has been caught spying to notify the victim.
     * @param player The player caught spying
     * @param tech The technology stolen by the player
     */
	public void notifySpying(Player player, Technology tech) throws ApplicationError {
        TechnologyStolenEF event = new TechnologyStolenEF(tech, player, this);
		MotEngine.instance.broadcast( event, this );
    }

	/**
     * Called when this player has discovered a technology through a successful spy role from player.
     * @param player The player whom you have spied from
     * @param tech The technology stolen from player
     */
	public void notifySpySuccess(Player player, Technology tech) throws ApplicationError {
        TechnologyStolenEF event = new TechnologyStolenEF(tech, this, player);
		event.addStateUpdate(new TechnologyAvailableForResearchVO(new GeneralTechnologyList(getTechnologyAvailableForResearch())));
		event.addStateUpdate(new PlayerTechnologyVO(tech.getFullName(), getName()));
		MotEngine.instance.broadcast( event, this );
    }

	/**
     * Called when another player has been caught sabotaging installations to notify the victim.
     * @param planet The planet that has been sabotaged
     * @param player The player caught sabotaging
     */
	public void notifySabotage(Planet planet, Player player, int productionSabotaged, int factoriesSabotaged, int populationSabotaged)  throws ApplicationError {
        SabotageEF event = new SabotageEF(planet, player);
         if (productionSabotaged > 0) {
            event.setQtyProductionSabotaged(productionSabotaged);
         }
         if (populationSabotaged > 0) {
            event.setQtyPopulationSabotaged(populationSabotaged);
         }
         if (factoriesSabotaged > 0) {
            event.setQtyFactoriesSabotaged(factoriesSabotaged);
         }
		MotEngine.instance.broadcast( event, this );
    }

    public void diplomaticRequestNotification(Player player, DiplomaticProposal proposal, String message) throws ApplicationError {
        DiplomaticRequestEF ef = new DiplomaticRequestEF(player, proposal);
    	ef.addStateUpdate(EngineVoFactory.getPlayerAssetsUpdate(proposal, this, player));
		MotEngine.instance.broadcast(ef, this);
    }

    public void diplomaticResponseNotification(Player player, DiplomaticProposal proposal, boolean accepted, String message) throws ApplicationError {
        DiplomaticResponseEF ef = new DiplomaticResponseEF(player, proposal, accepted, message);
		ef.addStateUpdate(EngineVoFactory.getPlayerAssetsUpdate(proposal, this, player));
		MotEngine.instance.broadcast(ef, this);
    }

	/**
	 * Called when this player is being offered a leader.
	 */
	public void offerGeneralLeaderTo(GeneralDetails leader) throws ApplicationError {
        offeredLeader = leader.getName();
 		MotEngine.instance.broadcast( new OfferLeaderEF(leader), this );
    }
    
	/**
	 * Called when this player is being offered a leader.
	 */
	public void offerGovenorLeaderTo(GovenorDetails leader) throws ApplicationError {
        offeredLeader = leader.getName();
 		MotEngine.instance.broadcast( new OfferLeaderEF(leader), this );
    }

    
	/**
	 * Send a message to the user to ask if he or she would like to adopt this moral choice
	 * If the User accepts he should respond using the AcceptMoralChoiceCmd().
	 */
	public void recommendMoralChoice(MoralChoice mc) throws ApplicationError {
 		MotEngine.instance.broadcast( new OfferMoralChoiceEF(mc.getName()), this );
	}
	
	/**
	 * Called when another a craft has arrived at an enemies solar system. If returns
     * which planet to attack in the solar system.
	 */
    public Planet getPlanetToAttack(Planet [] planets) throws ApplicationError {
        for (int i=0; i<planets.length; i++) {
            if (MotEngine.getInstance().getRelations().isWar(planets[i].getOwner(), this)) {
                return planets[i];
            }
        }
        return null;
    }

    /**
     * Called when a Space Station has been built to find out which planet the owner would
     * like to deploy the station to.
     */
    public Planet getPlanetToDeploySpaceStation(Transport transport, Orbital station) throws ApplicationError {
        if (transport.getTask() != null && !transport.getTask().isUserInformedOfExistence()) {
            MessageEF event = new MessageEF("Your space station is loaded into transport "+transport.getName()+" and awaiting orders");
            MotEngine.instance.broadcast( event, this );
            transport.getTask().setUserInformedOfExistence(true);
        }
        return null;
    }
    
    /**
     * Called when a Space Station has been built to find out which planet the owner would
     * like to deploy the station to.
     */
    public StellarObject getStellarObjectToDeployOutpost(Transport transport, Orbital station) throws ApplicationError {
        if (transport.getTask() != null && !transport.getTask().isUserInformedOfExistence()) {
            MessageEF event = new MessageEF("Your outpost is loaded into transport "+transport.getName()+" and awaiting orders");
            MotEngine.instance.broadcast( event, this );
            transport.getTask().setUserInformedOfExistence(true);
        }
        return null;
    }
    
    /**
     * Called when a Module has been built to find out which space station the owner would
     * like to add the module to.
     */
    public Planet getStationToAddModule(Transport transport, Planet origin, Module module) throws ApplicationError {
        if (transport.getTask() != null && !transport.getTask().isUserInformedOfExistence()) {
            MessageEF event = new MessageEF("Your module is loaded into transport "+transport.getName()+" and awaiting orders");
            MotEngine.instance.broadcast( event, this );
            transport.getTask().setUserInformedOfExistence(true);
        }
        return null;
    }

	public boolean isComputerPlayer() {
		return false;
	}

	public void setNextCraftOrders(Craft craft) throws ApplicationError {
        if (craft.canColonise()) {
            Planet bestPlanet = craft.getBestPlanetCanColonise();
            if (bestPlanet != null) {
                int qtyEnemies = getQtyEnemyCraftAt(bestPlanet.getStellarObject());
                if (qtyEnemies == 0) {
                    MotEngine.instance.broadcast( new ColonisePlanetEF(craft), craft.getOwner() );
                    return;
                }
            }
        }
    }

	public void setNextFleetOrders(Fleet fleet) throws ApplicationError {
    }

	private Commander commander;
    
	public void setNextBattleOrders(BattleCraft craft) throws ApplicationError {
        //This is only called when auto is on
        if (commander == null) commander = new CommanderBasic(this);
		commander.assignTaskToCraft(craft);
    }

	public void setNextBattleOrders(BattlePlanet planet) throws ApplicationError {
        //This is only called when auto is on
        if (commander == null) commander = new CommanderBasic(this);
		commander.assignTaskToPlanet(planet);
    }

    /**
     * returns the last craft this player designed or the last craft in the default 
     * design list fo this player.
     */
    public CraftDesign getLastCraftDesigned() {
        if (craftDesignes == null) {
            return null;
        }
        CraftDesign [] l = craftDesignes.getCraftDesigns();
        if (l != null) {
            return l[l.length-1];
        } else {
            return null;
        }
    }
    public String getLastCraftDesignedName() {
    	CraftDesign lcd = getLastCraftDesigned();
    	if (lcd != null) {
    		return lcd.getName();
    	} else {
    		return null;
    	}
    }
    
	public boolean isViewableBy(Player p) throws ApplicationError {
		SpyCenter sc = getSpyCenter();
		if (sc != null) {
			String [] playerSpies = sc.getPlayerSpiesAsArray();
			return Comp.in(p.getName(), playerSpies);
		}
		return false;
	}

    /**
     * This increases the development percent to the player characters defined percentage 
     * on all planets owned by this player. This is used when when A new factory is discovered 
     * and a user players requests to allocate extra resources to development.
     */
    public void increaseDevelopment() throws ApplicationError, ValidationError {
        Planet [] planets = getPlanets();
        if (planets == null) return;
        for (int i=0; i < planets.length; i++) {
            planets[i].setPercentsAuto(false, false, false);
        }
    }
    
    /**
     * This increases the ecology percent by 10% on all planets owned by this player.
     * This is used when when A new biotech is discovered and a user players requests to
     * allocate extra resources to ecology.
     */
    public void increaseEcology() throws ApplicationError, ValidationError {
        Planet [] planets = getPlanets();
        if (planets == null) return;
        for (int i=0; i < planets.length; i++) {
            planets[i].setPercentsAuto(false, false, false);
        }
    }

	public String[] getWeaponsToExcludeFromFireAll() throws ApplicationError {
		return Settings.getInstance().getWeaponsToExcludeFromFireAll();
	}
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
        if (craftDesignes != null) {
            craftDesignes.toXML(writer);
        }
        if (spyCenter != null) {
            spyCenter.toXML(writer);
        }
        writer.addElement("spy-recruit-effort-per-turn", spyRecruitEffortPerTurn);
        if (fleetWayPoint != null) {
        	fleetWayPoint.toXML(writer, "fleet-way-point");
        }
        if (gameScoreData != null) {
        	gameScoreData.toXML(writer);
        }
	}

	protected String getElementName() { return "user-player"; }


    public void setSpyRecruitEffortPerTurn(int spyRecruitEffortPerTurn) {
        this.spyRecruitEffortPerTurn = spyRecruitEffortPerTurn;
    }


    public int getSpyRecruitEffortPerTurn() {
        return spyRecruitEffortPerTurn;
    }

    public int getSpyRecruitCost() {
        return spyRecruitEffortPerTurn;
    }

    public void setSpyCenter(SpyCenter spyCenter) {
        this.spyCenter = spyCenter;
    }

    public String getTransportName() {
        return "Transport";
    }

    public void setOfferedLeader(String offeredLeader) {
        this.offeredLeader = offeredLeader;
    }

    public String getOfferedLeader() {
        return offeredLeader;
    }
    
	public Orbit getFleetWayPoint() {
		return fleetWayPoint;
	}
	
	public void setFleetWayPoint(Orbit fleetWayPoint) {
		this.fleetWayPoint = fleetWayPoint;
	}
	
	public GameScoreData getGameScoreData() {
		return gameScoreData;
	}
	
	public GameScoreData getGameScoreDataNotNull() {
		if (gameScoreData == null) {
			gameScoreData = new GameScoreData();
		}
		return gameScoreData;
	}
	
	public void setGameScoreData(GameScoreData gameScoreData) {
		this.gameScoreData = gameScoreData;
	}

}