/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.commanders;

import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.Settings;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.ExclusionZone;
import mot.engine.battles.ExclusionZoneList;
import mot.engine.battles.incumbants.IncumbentEdge;
import mot.engine.battles.incumbants.Incumbents;
import mot.engine.battles.incumbants.PerimeterAlarm;
import mot.engine.common.ServerImageObject;
import mot.engine.planets.Planet;
import mot.engine.players.Player;
import mot.engine.technology.WeaponInstance;
import mot.engine.technology.WeaponInstanceDetonator;
import mot.engine.technology.WeaponInstanceSpacialDistortion;

/**
 * A commander manages a players crafts while in battle. It selects the next target to attack, and 
 * gives a craft orders to move to a particular target and attack it.
 * 
 * This is a very basic implementation of a commander - it selects the closest enemy craft for each
 * commanded craft and tells the craft to attack it.
 */
public class CommanderBasic extends Commander {
    private static final int MIN_DIST_TO_MOVE = 5;
    private static final int MIN_TURN_TO_MAKE = 5;

	public CommanderBasic() {
    }

    public CommanderBasic(Player owner) {
        super(owner);
    }

	/**
	 * This assigns tasks to a computer controlled craft in a battle situation. Currently a
	 * craft simply attacks the closest enemy craft in the battle.
	 */
	public void assignTaskToCraft(BattleCraft craft) throws ApplicationError {
        boolean finished = false;
        
        try {
	        if (craft.getCraft().containsWeapons() || craft.hasFightersToLaunch()) {
	            finished = assignTaskToFighter(craft);
	        }
        } catch (Exception e) {
        	Logger.logError("Craft Command Failed", e);
        }

        if (craft.getCraft() != null && craft.getCraft().getHull().isStation()) {
        	// BattleStations can not retreat.
        	finished = false;
        } if (craft.getCraft() == null) {
        	// Fighters can not retreat.
        	finished = false;
        }
        

        
        // Methods return true if there is nothing they can attack!
        Player owner = getOwner();
		Player craftOwner = craft.getOwner();
		if (finished && owner == craftOwner) {
            craft.retreat();
        } else {
        	craft.endTurn();
        }
    }
    
	private boolean assignTaskToFighter(BattleCraft craft) throws ApplicationError {
		Player owner = getOwner();
		Battle battle = craft.getBattle();
		
		craft.setExclusionZone(null);
		craft.notifyStartCommand();

		BattleObject enemyObject = null;
		if (craft.hasTroopTransports()) {
			BattleCraft candidate = battle.getClosestEnemyCraftWithCrewDamage(owner, craft.getLocation(), craft);
			if (candidate != null) {
				if (craft.getSpeed() > craft.getLocation().getDistanceTo(candidate.getLocation())) {
					enemyObject = candidate;
				}
			}
		}

		ExclusionZoneList exclusionZones = battle.getExclusionZones(owner);

		boolean isSuicideFighter = craft.isSuicideFighter();
		boolean isShieldBurstFighter = craft.hasShieldBurst();
		
		if (isSuicideFighter || isShieldBurstFighter) {
			int range = WeaponInstanceDetonator.getExclusionZone(craft);
			enemyObject = battle.getMostPowerfulEnemyCraftAwayFromMyCraftInRange(owner, craft, true, range, exclusionZones);
			if (enemyObject == null) {
				enemyObject = battle.getClosestEnemyCraftAwayFromMyCraft(owner, craft, true, range, exclusionZones);
				if (enemyObject == null) {
					enemyObject = battle.getClosestEnemyCraftAwayFromMyCraft(owner, craft, true, range, null);
				}
			}
		}
		
		BattlePlanet enemyPlanet = battle.getPlanet() != null && wantToAttack(battle.getPlanet()) ? battle.getPlanet() : null;
		
		if (enemyObject == null) {
			enemyObject = battle.getClosestEnemyCraft(owner, craft, true, exclusionZones);
			if (enemyObject == null) {
				enemyObject = battle.getClosestEnemyCraft(owner, craft, true, null);
			}
			if (enemyPlanet != null) {
				if (enemyObject == null) {
					enemyObject = enemyPlanet;
				} else {
					int cd = enemyObject.getLocation().getDistanceTo(craft.getLocation());
					int pd = enemyPlanet.getLocation().getDistanceTo(craft.getLocation());
					if (pd < cd) {
						enemyObject = enemyPlanet;
					}
				}
			}
		}
		
		ExclusionZone exclusionZone = null;
		if (enemyObject != null) {
			if (isSuicideFighter) {
    			int range = WeaponInstanceDetonator.getExclusionZone(craft);
    			exclusionZone = new ExclusionZone(enemyObject.getLocation(), range);
    		} else if (isShieldBurstFighter) {
    			WeaponInstance w = craft.getWeaponShieldBurst();
    			if (w != null) {
    				int range = w.getRange();
    				exclusionZone = new ExclusionZone(enemyObject.getLocation(), range);
    			}
    		}
		}
		craft.setExclusionZone(exclusionZone);
		
        if (enemyObject != null) {
        	persistentMoveTo(craft, enemyObject);
            craft.setMoveRemaining(0);
    
            //Logger.logDebug("Command "+craft+" "+craft.hasFightersToLaunch());
            if (craft.hasFightersToLaunch()) {
            	craft.launchFighters();
            }
            
            if (craft.inWeaponRange(enemyObject.getLocation())) {
                if (craft.hasWeaponsToFire()) {
                    fireWeapons(craft, enemyObject);
                }
            }
            // Must discharge weapons otherwise my turn has not ended!
            // To change this change BattleCraft.hasMoved().
            if (craft.hasFiredTimeWarpThisCommand()) {
            	persistentMoveTo(craft, enemyObject);
                craft.setMoveRemaining(0);
                fireWeapons(craft, enemyObject);
            }
            
            craft.dischargeWeapons();
            
            return false;
        }

        return true;
	}

	private void persistentMoveTo(BattleCraft craft, BattleObject enemy) throws ApplicationError {
		Battle battle = craft.getBattle();
		Location src = craft.getLocation();
		Location dest = enemy.getLocation();
		Incumbents incumbents = battle.getIncumbents();
		IncumbentEdge mine = incumbents == null ? null : incumbents.getFirstActiveEncumbent(craft, src, dest);
		if (mine == null) {
			_persistentMoveTo(craft, enemy);
		} else if (mine.getIncumbant() instanceof PerimeterAlarm && ((PerimeterAlarm)mine.getIncumbant()).isVisible(craft.getOwner())) {
			// I will explode one my own mines - do not move
		} else {
			_persistentMoveTo(craft, enemy);
		}
	}
	
	private void _persistentMoveTo(BattleCraft craft, BattleObject enemy) throws ApplicationError {
		if (enemy instanceof BattleCraft) {
			persistentMoveToCraft(craft, (BattleCraft) enemy);
		} else if (enemy instanceof BattlePlanet) {
			persistentMoveToPlanet(craft, (BattlePlanet) enemy);
		} else {
			Location location = enemy.getLocation();
			craft.moveTo(new LocationAndDirection(location, craft.getLocation().getDirectionFor(location)));
		}
	}
	
	/**
	 * Keep moving craft forwards until no progress is made.
	 * We do this because a craft may stop because of a mine explosion etc!
	 */
	private void persistentMoveToPlanet(BattleCraft craft, BattlePlanet enemyPlanet) throws ApplicationError {
		double prevMoveRemaining = -1.0;
		boolean keepMoving = true;
		double moveRemaining = craft.getMoveRemaining();
		while (moveRemaining > 0 && keepMoving && moveRemaining != prevMoveRemaining) {
			keepMoving = craft.moveTo(enemyPlanet);
			prevMoveRemaining = moveRemaining;
			moveRemaining = craft.getMoveRemaining();
		}
	}

	/**
	 * Keep moving craft forwards until no progress is made.
	 * We do this because a craft may stop because of a mine explosion etc!
	 */
	private void persistentMoveToCraft(BattleCraft craft, BattleCraft enemyCraft) throws ApplicationError {
		craft.fireMissileDefenceWeapons(null);
		double prevMoveRemaining = -1.0;
		boolean keepMoving = true;
		double moveRemaining = craft.getMoveRemaining();
		int maxMoves = 4;
		while (maxMoves > 0 && moveRemaining > 0 && keepMoving && moveRemaining != prevMoveRemaining && !craft.isDestroyed()) {
			int distance = craft.getLocation().getDistanceTo(enemyCraft.getLocation());
			int direction = craft.getLocation().getDirectionFor(enemyCraft.getLocation());
			int dirDelta = Location.getDirectionDeltaPos(craft.getDirection(), direction);
			if (distance < MIN_DIST_TO_MOVE && dirDelta < MIN_TURN_TO_MAKE) {
				return;
			}
			keepMoving = craft.moveTo(enemyCraft);
			prevMoveRemaining = moveRemaining;
			moveRemaining = craft.getMoveRemaining();
			maxMoves--;
		}
	}

	/**
	 * This assigns tasks to a computer controlled planets in a battle situation. Currently a
	 * craft simply attacks the closest enemy craft in the battle.
	 */
	public void assignTaskToPlanet(BattlePlanet planet) throws ApplicationError {
		Battle battle = planet.getBattle();
        
		planet.notifyStartCommand();
		
        BattleCraft enemyCraft = battle.getClosestEnemyCraft(getOwner(), planet.getLocation(), true);
        if (enemyCraft == null) {
            return;
        }

        //Logger.logDebug("Command "+planet+" "+planet.hasFightersToLaunch());
        if (planet.hasFightersToLaunch()) {
        	planet.launchFighters();
        }
        if (planet.inWeaponRange(enemyCraft.getLocation())) {
            if (planet.hasWeaponsToFire()) {
                // Fire weapons by grouping the same one together.
                fireWeapons(planet, enemyCraft);
            }
        }
        planet.dischargeWeapons();
    }

	private static final int MIN_DIST_TO_FIRE_SPACIAL_DISTORTION = 150;
	
	private void fireWeapons(BattleObject source, BattleObject enemyCraft) throws ApplicationError {
		boolean fireAll = true;
		WeaponInstance[] weapons = source.getWeapons();
		if (weapons == null || weapons.length == 0) return;
		
		if (source instanceof BattleCraft && ((BattleCraft)source).isSuicideFighter()) {
			int range = WeaponInstanceDetonator.getExclusionZone(source);
			int dist = source.getDistanceTo(enemyCraft.getLocation());
			if (dist < range / 2) {
				WeaponInstance wdm = source.getWeaponDroidMine();
				WeaponInstance wd = source.getWeaponDetonator();
				if (wdm != null) {
					BattleObject target = enemyCraft;
//					if (target instanceof BattleCraft && wd.getDamage() > ((BattleCraft)target).getDamageRequiredToDestroy() * 1.25) {
//						// Select a different target for the droid mine 
//					}
					source.fireWeapon(target, wdm, wdm.getQty());
				}
				source.fireWeapon(enemyCraft, wd, 1);
				source.dischargeWeapons();
				return;
			}
		}
		
		String [] excludeGroups = Settings.getInstance().getWeaponsToExcludeFromFireAll();

		for (int i = 0; i < weapons.length; i++) {
			Weapon weapon = weapons[i].getWeapon();
			if (weapon.isSpacialDistortion()) {
				Location loc = enemyCraft.getLocation();
				Location targetLoc = new Location(loc);
				int dist = enemyCraft.getWidth()/2+WeaponInstanceSpacialDistortion.DEFAULT_WIDTH/2+10;
				targetLoc.shift(source.getLocation(), dist);
				Battle battle = source.getBattle();
				Player owner = source.getOwner();
				BattleCraft myClosestCraft = battle.getMyClosestCraft(owner, targetLoc, false, null);
				int distanceToMyClosestCraft = Integer.MAX_VALUE;
				if (myClosestCraft != null) {
					distanceToMyClosestCraft = targetLoc.getDistanceTo(myClosestCraft.getLocation());
				}
				if (distanceToMyClosestCraft > MIN_DIST_TO_FIRE_SPACIAL_DISTORTION) {
					source.fireWeapon(targetLoc, weapons[i], weapons[i].getQty());
				} else {
					weapons[i].discharge();
				}
			} else if (weapon.onlyTargetsMovingAutomitons()) {
				Battle battle = source.getBattle();
				Location loc = source.getLocation();
				Automaton target = battle.getClosestEnemyAutomiton(getOwner(), loc, true, true);
				if (target != null) {
					source.fireWeapon(target, weapons[i], weapons[i].getQty());
				} else {
					weapons[i].discharge();
				}
			} else if (excludeGroups != null && Comp.in(weapon.getGroup(), excludeGroups)) {
				// Lets not fire these weapons, they are considered too dangerous
				weapons[i].discharge();
			} else {
				if (weapons[i].willHaveAnEffectOn(enemyCraft)) {
	                if (enemyCraft instanceof BattleCraft) {
	                    BattleCraft targetBC = (BattleCraft) enemyCraft;
	                    source.fireWeapons(targetBC, weapons[i], weapons[i].getQty());
	                } else {
						source.fireWeapons(enemyCraft, weapons[i], weapons[i].getQty());
	                }
				}
			}
		}
		
		if (!source.hasFiredTimeWarpThisCommand()) {
			findAlternateTargetForUnFiredWeapons(source, enemyCraft);
			source.dischargeWeapons();
		}
	}


	public static void findAlternateTargetForUnFiredWeapons(BattleObject source, ServerImageObject target) throws ApplicationError {
		WeaponInstance[] weapons = source.getWeapons();
		if (weapons == null || weapons.length == 0) return;
		for (int i = 0; i < weapons.length; i++) {
			if (weapons[i].hasWeaponsToFire()) {
				Weapon w = weapons[i].getWeapon();
				if (w.isShieldDrain() || w.isImmobiliser() || w.isSleep() || w.isMindGroup() || w.isDamageBot() || w.isPlague() || w.isBio()) {
					Battle battle = source.getBattle();
					BattleObject newTarget = battle.getClosestEnemyOperatingCraftWithinFiringAngleForWeapon(source.getController(), source, weapons[i]);
					if (newTarget != null) {
						source.fireWeapon(newTarget, weapons[i], weapons[i].getQty());
					}
				}
			}
		}
	}

    /**
     * Returns true if this commander would like to attack this planet. Even if a general chooses
     * a particular planet for conquering the fleet/crafts may have attack the previous turn and
     * may not want to attack again but wait for troops. Alternatively this general might want to 
     * bombard planets to destroy the colony. The commander chooses because it must know when to
     * retreat its fighters during a battle.
     * This must be a deterministic algorithm (must not contain a random determinent).
     */
    public boolean wantToAttack(Planet planet) throws ApplicationError {
        if (planet != null && planet.getOwner() != null && planet.getOwner().isEnemy(getOwner())) {
            return planet.isDefended();
        } else {
            return false;
        }
    }

    public boolean wantToAttack(BattlePlanet planet) throws ApplicationError {
        if (planet != null && planet.getOwner() != null && planet.getOwner().isEnemy(getOwner())) {
            return planet.getPlanet().isDefended() || planet.hasFightersToLaunch();
        } else {
            return false;
        }
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
	}

	protected String getElementName() { return "commander-basic"; }
}