/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import java.util.ArrayList;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.OwnedObject;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.crafts.FleetReference;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.StellarObjects;
import mot.engine.players.Player;

/**
 * This class is used by the General to keep track of tasks it needs to assign its 
 * crafts to and track task tasks to success or failure.
 * 
 * First a task is created, crafts will then be assigned to the task until such a time
 * as readyForAssignment() returns true. Then each time the fleet built asks for its next
 * orders assignTaskToFleet() will be called.
 * The complete() method is called every turn and when it returns true the task is 
 * destroyed and any craft it contained re-distributed among other tasks.
 */
public abstract class CraftTask extends OwnedObject {

    /**
     * This is the target of this task. For example for invasion it is the place to be 
     * invaded, for defence it is the place to defend.
     */
    private Orbit target;
    
    /**
     * Most tasks involve a fleet of crafts being built to meet the objectives. This is
     * the fleet of crafts being built.
     */
    private FleetReference fleet;
    
    /**
     * This is the date that this Craft Task / Priority was created.
     */
    private int createdDate;

    public CraftTask() {
    }

    public CraftTask(Planet target, Player owner) throws ApplicationError {
    	this(new Orbit(target), owner);
    }
    
    public CraftTask(StellarObject target, Player owner) throws ApplicationError {
    	this(new Orbit(target), owner);
    }
    
    public CraftTask(Orbit target, Player owner) throws ApplicationError {
        super(MotEngine.instance.getNameGenerator().getNextName("task_") , owner);
        this.target = target;
        this.fleet = null;
        this.createdDate = MotEngine.instance.getDate();
    }
    
    public abstract String getTaskName();

    public String getDescription() throws ApplicationError { return getName() + " : "+ getTaskName() + " " + getTarget()+" fleet="+((getFleet()!=null)?getFleet().getName():"None")+" created="+createdDate; }

    /**
     * Adds this craft to the fleet of crafts the player is building to meets this tasks objectives.
     * @param craft The craft to add to this task.
     * @throws mot.common.util.ApplicationError
     */
    public void add(Craft craft) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTask.add() craft="+craft.getDescription()+" assigned to task="+getDescription());
        }
        if (fleet == null || fleet.getFleet() == null) {
            Fleet f = FleetFactory.create(craft);
            if (f != null) {
                f.setTask(this);
                fleet = new FleetReference(f);
            }
        } else {
            Fleet f = fleet.getFleet();
            f.setTask(this);
            if (f.getOrbit() != null) {
                if (f.getOrbit().atSameSO(craft.getOrbit())) {
                    f.add(craft);
                } else {
                	routeTo(craft, f.getOrbit());
                }
            } else {
                Logger.logWarning("GeneralCraftPriotity.add() craft="+craft.getName()+" cannot join fleet "+fleet.getName()+" becuase it is at warp");
            }
        }
    }

    /**
     * Adds these crafts to the fleet of crafts the player is building to meets this tasks objectives.
     * @param object The craft to add to this task.
     * @throws mot.common.util.ApplicationError
     */
    public void add(Craft [] crafts) throws ApplicationError {
        if (crafts == null) return;
        for (int i=0; i<crafts.length; i++) {
        	if (crafts[i] != null) {        		
        		if (requires(crafts[i])) {
        			Fleet f = crafts[i].getFleet();
        			if (f != null) {
        				f.remove(crafts[i]);
        			}
        			add(crafts[i]);
        		}
        	}
        }
    }
    
    /**
     * Removes all the crafts from the task other passed in and adds them to this task.
     * Note - It does not take crafts / a fleet from another task if the orbit is null.
     * @param The task from which to take the crafts.
     * @throws mot.common.util.ApplicationError
     */
    public void addCraftsFrom(CraftTask other) throws ApplicationError {
        if (other == null) {
            return;
        }
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTask.addCraftsFrom() Crafts from task "+other.getDescription()+" to task "+getDescription());
        }
        if (other.getFleet() != null) {
            Fleet f = other.getFleet();
            if (f != null && f.getOrbit() != null) {
                add(f.getCrafts());
                if (f.getSize() == 0) {
                    other.deleteFleet();
                }
            }
        } else {
            Player owner = other.getOwner();
            if (owner != null) {
                Craft [] crafts = owner.getCrafts();
                if (crafts != null) {
                    for (int i=0; i<crafts.length; i++) {
                        if (crafts[i].getTask() == other && crafts[i].getOrbit() != null) {
                            if (requires(crafts[i])) {
                                add(crafts[i]);
                            }
                        }
                    }
                }
            }
        }
    }

    public boolean higherPriority(CraftTask best, Location loc) {
        int p1 = getPriority();
        int p2 = best.getPriority();
        return (p1 > p2) || 
               (p1 == p2 && getTarget() != null && 
                best.getTarget() != null && 
                getTargetStellarObject().getDistanceTo(loc) < best.getTargetStellarObject().getDistanceTo(loc));        
    }
    
    /**
     * @return The priority of this task - The higher the number the more likely it is 
     * to be performed.
     */
    public abstract int getPriority();

	/**
	 * The General will only assign orders to the fleet if its task is ready for dispatch.
     * This should only return true if tasks minimum requirements have been met so the player 
     * can attempt the task e.g. Invade, Colonise, Attack etc.
     * 
     * For example - When building a fleet to attack - We may say we want to collect 10 crafts
     * in the fleet - The task will not be ready for dispatch until 10 crafts have been assigned 
     * to the fleet.
     * 
     * @return True if the task has enough resources for the player to attempt to carry our the task..
	 */
    public abstract boolean readyForAssignment() throws ApplicationError;
    
    /**
     * This assigns orders to the fleet belonging to this task. This should only be called in
     * response to the fleet asking for orders.
     * @throws mot.common.util.ApplicationError
     */
    public abstract void assignTaskToFleet() throws ApplicationError;

    /**
     * The method readyForAssignment() has returned false. Call the assignPreReadyTask in case the task
     * would like to command crafts to do something before the task is ready.
     */
	public void assignPreReadyTask()  throws ApplicationError { }


	/**
	 * This is called at the end of every turn.
     * This gives this task the opportunity to asses its status and if the player really 
     * needs the task any more. It the task is not required it is removed and the next time 
     * its fleet asks for orders its task will no longer exist - Thus the craft will be broken
     * up and its crafts re-assigned to different tasks.
     * 
     * For example - A task to in invade a planet is complete when the planet no longer is owned 
     * by an enemy or the fleet size is 0. The task cen then be deleted.
     * 
     * @return True if the task has been completed. If so this task should be removed from
     * the list of tasks.
	 */
    public abstract boolean complete() throws ApplicationError;
    
    /**
     * @param craft The craft this task may or may not need.
     * @return True if this task required this craft otherwise false. 
     * @throws mot.common.util.ApplicationError
     */
    public abstract boolean requires(Craft craft) throws ApplicationError;
    
    /**
     * Even if a task is complete - we cannot delete it if it has a fleet that is not in orbit.
     * @return True is this task is in a state such that it (or its fleet) can be deleted.
     */
    public boolean canDelete() throws ApplicationError {
        Fleet f = getFleet();
        return (f == null || f.getSize() == 0 || f.getOrbit() != null);
    }

    /**
     * Removes all crafts in this task and sets the task for each craft to null.
     * @throws mot.common.util.ApplicationError
     */
    public void removeCraftsFromTask() throws ApplicationError {
        Craft [] c = getOwner().getCrafts();
        if (c != null) {
            for (int i=0; i<c.length; i++) {
                if (c[i].getTask() == this) {
                    c[i].setTask(null);
                }
            }
        }
    }

    /**
     * Get all crafts assigned to this task. This includes crafts that are in the tasks fleet as 
     * well as crafts not included in the fleet.
     */
    public Craft [] getAllCrafts() throws ApplicationError {
    	ArrayList<Craft> r = null;  
        Craft [] c = getOwner().getCrafts();
        if (c != null) {
            for (int i=0; i<c.length; i++) {
                if (c[i].getTask() == this) {
                    if (r == null) r = new ArrayList<Craft>();
                    r.add(c[i]);
                }
            }
        }
        if (r == null) {
        	return null;
        } else {
        	return r.toArray(new Craft[r.size()]);
        }
    }

    public int getQtyMillitary() throws ApplicationError {
        Craft [] c = getOwner().getCrafts();
        int qty = 0;
        if (c != null) {
            for (int i=0; i<c.length; i++) {
                if (c[i].getTask() == this && c[i].isMilitary()) {
                    qty++;
                }
            }
        }
        return qty;
	}

    public int getQtyMillitaryInFleet() throws ApplicationError {
    	if (fleet == null) return 0;
        Craft [] c = fleet.getFleet().getCrafts();
        int qty = 0;
        if (c != null) {
            for (int i=0; i<c.length; i++) {
                if (c[i].isMilitary()) {
                    qty++;
                }
            }
        }
        return qty;
	}

    public int getQtyColonisers() throws ApplicationError {
        Craft [] c = getOwner().getCrafts();
        int qty = 0;
        if (c != null) {
            for (int i=0; i<c.length; i++) {
                if (c[i].getTask() == this && c[i].canColonise()) {
                    qty++;
                }
            }
        }
        return qty;
	}

    public int getQtyColonisersInFleet() throws ApplicationError {
    	if (fleet == null) return 0;
        Craft [] c = fleet.getFleet().getCrafts();
        int qty = 0;
        if (c != null) {
            for (int i=0; i<c.length; i++) {
                if (c[i].canColonise()) {
                    qty++;
                }
            }
        }
        return qty;
	}

    public void removeCraft(Craft craft) throws ApplicationError {
    	if (fleet != null && fleet.contains(craft)) {
    		fleet.getFleet().remove(craft);
    		craft.setTask(null);
    	}
    }
    
    /**
     * @return True iff this task allows crafts assigned to it to be relocated to another higher priority task.
     */
	public boolean canDonateCraft() {
		return true;
	}
    
    /**
     * This should be called periodically by the General to allow tasks to take a general
     * assessment of their position. Most tasks do not use this, however if a task wished to
     * it can override this method and perform a long a complex assessment which will only occur
     * once every n turns.
     * This default implementation does not perform any task.
     */
    public void assesPosition() throws ApplicationError {
    }
    
    /**
     * This is called by the battle framework when a craft has fought in a battle and survived.
     * This may be over s to perform functionality such as bombard when a craft has successfully
     * won a battle. This default implementation does not perform any function.
     * @throws mot.common.util.ApplicationError
     */
    public void informBattleFinished(Battle battle) throws ApplicationError {
    }

    public Orbit getTarget() {
        return target;
    }

    public StellarObject getTargetStellarObject() {
    	if (target == null) {
    		return null;
    	} else {
    		return target.getStellarObject();
    	}
    }

    public Planet getTargetPlanet() {
    	if (target == null) {
    		return null;
    	} else {
    		return target.getPlanet();
    	}
    }

    public Fleet getFleet() throws ApplicationError {
        if (fleet == null) {
            return null;
        } else {
            return fleet.getFleet();
        }
    }
    
    public void deleteFleet() throws ApplicationError {
        Fleet f = getFleet();
        if (f != null) {
            f.destroy();
            fleet = null;
        }
    }

    public int getFleetSize() throws ApplicationError {
        if (fleet == null) {
            return 0;
        } else {
            return fleet.getFleet().getSize();
        }
    }
    
    public boolean contains(Craft c) throws ApplicationError {
        if (fleet == null) {
            return false;
        } else {
            return fleet.contains(c);
        }
    }
    
    /**
     * This method is an alternative to MoveableOwnedObject.moveTo(). It moves the craft to the
     * destination orbit, using moveTo(), but if there is a galactic incumbent in the path it
     * attempts to route the craft around it. 
     * @param moo The craft/Fleet to move.
     * @param destination The place to move the craft / fleet to.
     * @throws ApplicationError 
     */
    protected void routeTo(MoveableOwnedObject moo, Orbit destination) throws ApplicationError {
    	StellarObjects sos = MotEngine.getInstance().getStellarObjects();
    	Location srcLoc = moo.getLocation();
    	Location destLoc = destination.getStellarObject().getLocation();
		if (sos.hasGalacticIncumbentsBetween(srcLoc, destLoc, BlackHole.class)) {
			Player owner = moo.getOwner();
			StellarObject[] steps = sos.getAllWithinRange(owner, moo.getMaxTravelDistance());
			int bestDistance = Integer.MAX_VALUE;
			StellarObject best = null;
			if (steps != null) {
				for (int i = 0; i < steps.length; i++) {
					Location loc = steps[i].getLocation();
					int dist = srcLoc.getDistanceTo(loc) + loc.getDistanceTo(destLoc);
					if (best == null || dist < bestDistance) {
						if (!sos.hasGalacticIncumbentsBetween(srcLoc, loc, BlackHole.class) &&
							!sos.hasGalacticIncumbentsBetween(loc, destLoc, BlackHole.class)) {
							best = steps[i];
							bestDistance = dist;
						}
					}
				}
			}
			if (best == null) {
				if (Logger.isDebug()) {
					Logger.logDebug(""+getTaskName()+" cannot find route from "+srcLoc+" to "+destLoc);
				}
				return;
			} else {
				moo.moveTo(new Orbit(best));
			}
		} else {
			moo.moveTo(destination);
		}
    }

	public void setFleet(Fleet v) throws ApplicationError { fleet = new FleetReference(v); }
    public void setTarget(Orbit target) { this.target = target; }
	public void setFleetReference(FleetReference v) { fleet = v; }

	protected void encode(XMLWriter writer) throws ApplicationError {
        super.encode(writer);
        if (target != null) {
            target.toXML(writer, "target");
        }
		if (fleet != null && fleet.getFleet() != null) {
			fleet.toXML(writer);
		}
        writer.addElement("created-date", createdDate); 
    }

    public int getCreatedDate() {
        return createdDate;
    }

    public void setCreatedDate(int createdDate) {
        this.createdDate = createdDate;
    }

	public String toString() {
		return getName()+" ("+getTaskName()+")";
	}

}

