/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.PlanetReference;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.commanders.Commander;

/**
 * This task is used to organise crafts being used to attack an enemy player.
 */
public class CraftTaskAttack extends CraftTask {

    public int getPriority() { return 10; }
    public String getTaskName() { return "Attack"; }

	/**
	 * The qty crafts in the fleet we would like to construct for this player.
     * We may not actually build this many crafts in time (see dateOfFleet for details).
	 */
	private int qtyCraftsOfFleet;
    
    /**
     * This is true if the fleet has been sent to attack the enemy - and it too late for new craft to join.
     */
    private boolean assigned;
    
    /**
     * An attack craft may choose to attack an enemy via another location (if the enemy player has a worm hole)
     * This is to hide the actual attack location and avoid too many enemy ships protecting the target.
     */
    private PlanetReference attackViaPlanet;
    
    public CraftTaskAttack() {
    }
    
    public CraftTaskAttack(Planet target, Player owner, int qtyCraftsOfFleet, Planet route) throws ApplicationError {
        super(target, owner);
        this.qtyCraftsOfFleet = qtyCraftsOfFleet;
        this.assigned = false;
        if (route != null) this.attackViaPlanet = new PlanetReference(route);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskAttack.init() New task="+getDescription());
        }
    }

    public CraftTaskAttack(StellarObject target, Player owner, int qtyCraftsOfFleet, Planet route) throws ApplicationError {
        super(target, owner);
        this.qtyCraftsOfFleet = qtyCraftsOfFleet;
        this.assigned = false;
        if (route != null) this.attackViaPlanet = new PlanetReference(route);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskAttack.init() New task="+getDescription());
        }
    }

	public int getQtyCraftsOfFleet() { return qtyCraftsOfFleet; }

	public void setQtyCraftsOfFleet(int v) { qtyCraftsOfFleet = v; }

    /**
     * The add craft method is overridden because we want to check if there is another fleet
     * with task attack in this solar system - If there is we assume it has just been dispatched
     * and add this craft to it as well.
     * @param craft The craft to add to an attack craft
     * @throws mot.common.util.ApplicationError
     */
    public void add(Craft craft) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskAttack.add() craft="+craft.getName()+" priority="+getPriority()+" send to attack="+getTarget());
        }
        
        Fleet fleet = getFleet();
        if (fleet == null) {
            // Just check if a previous fleet I built is at this location with the correct destination.
            // Two crafts may be added in a single turn to this task, but the first one met the objectives 
            // and caused it to be completed.
            // A bit of a waist for this craft to wait for the next one though!
            Fleet [] otherFleet = getOwner().getFleetAtWithDest(craft.getOrbitStellarObject(), getTargetStellarObject());
            if (otherFleet != null && otherFleet.length > 0) {
                CraftTask otherTask = otherFleet[0].getTask();
                if (otherTask != null && otherTask instanceof CraftTaskAttack) {
                    otherFleet[0].add(craft);
                    return;
                }
            }
            StellarObject waypoint = getWaypoint();
            if (waypoint != null && craft.getOrbitStellarObject() != waypoint) {
            	routeTo(craft, waypoint.getOrbit());
            	return;
            }
        } else {
            if (fleet.getSize() == 0) {
                deleteFleet();
            }
        }
        super.add(craft);
    }
    
	/**
     * The qty crafts each fleet build increases by.
     */
    public static final int FLEET_SIZE_INCREMENT = 2;

    /**
     * This task is ready for dispatch if any of the following conditions are true
     *  - The size of the fleet is > qtyCraftsOfFleet
     *  - The time since the task was built is > maxTimeBetweenFleets
     *  Once a task has been dispatched, assigned is set to true so it is not recalled.
     * @return True if the fleet is ready for dispatch.
     * @throws mot.common.util.ApplicationError
     */
    public boolean readyForAssignment() throws ApplicationError {
        if (assigned) {
            return true;
        }
        Player owner = getOwner();
        Fleet fleet = getFleet();
        int maxTimeBetweenFleets = owner.getPlayerCharacter().getMaxTimeBetweenFleets();
        int dateOfFleet = getCreatedDate();
        int date = MotEngine.instance.getDate();
        if (fleet != null && (date - dateOfFleet) > maxTimeBetweenFleets) {
            if (Logger.isDebug()) {
                Logger.logDebug("CraftTaskAttack.readyForAssignment() fleet="+fleet.getName()+" dispatched to player because time expired ");
            }
            this.assigned = true;
            return true;
        } else if (fleet != null && fleet.getSize() >= qtyCraftsOfFleet) {
            if (Logger.isDebug()) {
                Logger.logDebug("CraftTaskAttack.readyForAssignment() fleet="+fleet.getName()+" dispatched to player becuase size>"+qtyCraftsOfFleet);
            }
            if (owner instanceof ComputerPlayer) {
                ComputerPlayer cp = (ComputerPlayer)owner;
                //int qty = cp.getGeneral().getQtyCraftsInAttackFleet();
                cp.getGeneral().setQtyCraftsInAttackFleet(qtyCraftsOfFleet + FLEET_SIZE_INCREMENT);
            }
            this.assigned = true;
            return true;
        }
        return false;
    }

    /**
     * This moves the fleet to the enemy solar system and then attacks or blockades the target.
     */
    public void assignTaskToFleet() throws ApplicationError {
        Fleet fleet = getFleet();
		if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskAttack.assignTaskToFleet() fleet="+fleet+" assign "+getDescription() );
        }

        if (fleet == null || complete()) {
            return;
        }

		Orbit target = getTarget();
        // Do not assign tasks to the fleet being built.
        if (!fleet.getOrbit().atSameSO(target)) {
        	routeTo(fleet, target);
        } else {
        	if (target.getPlanet() == null) {
        		// We are attacking an outpost or space station at a black hole
           		attackStellarObject(fleet, target.getStellarObject());
        	} else {
        		// We are attacking a planet
                double p = getOwner().getPlayerCharacter().getBlockadeOrAttackProbability();
                if (Utils.random(p)) {
            		if (target.getStellarObject() != fleet.getOrbitStellarObject()) {
            			routeTo(fleet, target);
            		} else {
            			fleet.blockade(target.getStellarObject());
            		}
                } else {
               		attackPlanet(fleet, target.getPlanet());
                }
        	}
        }
    }
    
    private void attackStellarObject(Fleet fleet, StellarObject stellarObject) throws ApplicationError {
		if (stellarObject != fleet.getActualOrbit()) {
			routeTo(fleet, stellarObject.getOrbit());
		} else {
            Craft [] enemyCraft = getOwner().getEnemyCraftAt(stellarObject, false);
            if ((enemyCraft != null && enemyCraft.length > 0) || (stellarObject.getOrbital() != null && stellarObject.hasEnemyOwner(getOwner()))) {
            	fleet.attack(stellarObject, enemyCraft);
            }
		}
	}
    
	public void attackPlanet(Fleet fleet, Planet planet) throws ApplicationError {
    	if (planet == null) {
            SolarSystem ss = (SolarSystem) getTarget().getStellarObject();
            if (!(ss instanceof SolarSystem)) {
                Logger.logWarning("CraftTaskAttack.attackPlanet() Only handles Solar Systems");
                return;
            }
            planet = getOwner().getPlanetToAttack(ss.getPlanets());
        } else {
        	if (planet.getOwner() == null || !planet.getOwner().isEnemy(getOwner())) {
                SolarSystem ss = planet.getSolarSystem();
        		planet = getOwner().getPlanetToAttack(ss.getPlanets());
        	}
        }
        if (planet != null) {
    		if (planet.getSolarSystem() != fleet.getActualOrbit()) {
    			routeTo(fleet, planet.getOrbit());
    		} else {
    			Commander commander = ((ComputerPlayer)getOwner()).getCommander();
    			if (commander.wantToAttack(planet)) {
    				fleet.attack( planet );
    			} else if (!planet.hasDefenses()) {
    				StellarObject orbit = getTargetStellarObject();
    				int qtyTroopsComming = fleet.getOwner().getQtyTroopsWithDestination(orbit);
    				if (qtyTroopsComming == 0) {
    					fleet.bombard(planet);
    				}
    			}
    		}
        }
    }

    protected void routeTo(MoveableOwnedObject moo, Orbit destination) throws ApplicationError {
    	if (attackViaPlanet != null) {
    		Planet avp = attackViaPlanet.getPlanet();
			attackViaPlanet = null;
			if (moo.getOrbitStellarObject() != avp.getSolarSystem()) {
    			super.routeTo(moo, new Orbit(avp));
    		}
    	} else {
    		super.routeTo(moo, destination);
    	}
    }
    
    /**
     * This task is complete if :
     *  - The target has no enemies
     *  - The task is more than 25 turns old
     *  - The fleet is empty and we are at an enemy stellar object (this task was defeated).
     */
    public boolean complete() throws ApplicationError {
        StellarObject so = getTargetStellarObject();
        if (so == null) {
            Logger.logWarning("GeneralCraftPriotityAttack.complete() NO TARGET");
            return true;
        }
        long currentDate = MotEngine.instance.getDate();
        boolean hasEnemyOwner = so.hasEnemyOwner(getOwner()); 
        if (!hasEnemyOwner) {
            return true;
        }
        int createdDate = getCreatedDate();
		Fleet fleet = getFleet();
		if (currentDate > (createdDate + 35)) {
			if (fleet == null || fleet.getOrbit() != null) {
				return true;
			}
        }
        if (fleet != null && fleet.getSize() == 0 && fleet.getOrbit() != null) {
            return fleet.getActualOrbit().hasEnemyOwner(getOwner());
        }
        return false;
    }

    public boolean requires(Craft craft) throws ApplicationError {
        return craft.containsWeapons();
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("qtyCraftsOfFleet", qtyCraftsOfFleet);
		writer.addElement("assigned", assigned);
        if (attackViaPlanet != null)attackViaPlanet.toXML(writer, "attackViaPlanet");
	}

	public Planet getRoute() {
        if (attackViaPlanet != null) { 
        	return attackViaPlanet.getPlanet();
        } else {
        	return null;
        }
	}
	
	public PlanetReference getAttackViaPlanet() {
		return attackViaPlanet;
	}
	
	public void setAttackViaPlanet(PlanetReference route) {
		this.attackViaPlanet = route;
	}
	
	protected String getElementName() { return "craft-task-attack"; }


    public void setAssigned(boolean assigned) {
        this.assigned = assigned;
    }


    public boolean isAssigned() {
        return assigned;
    }
    
    /**
     * The location I would like to collect crafts at. 
     */
    private StellarObject getWaypoint() throws ApplicationError {
    	BlackHole wormHole = getOwner().getWormHole();
    	return wormHole;
	}


}