/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * This task is to bombard a particular planet until its colony is destroyed.
 */
public class CraftTaskBombard extends CraftTask {

    /**
     * The maximum age of this task.
     */
    public static final int MAX_AGE = 5;

    public int getPriority() { return 40; }
    
    public String getTaskName() { return "Bombard"; }

    public CraftTaskBombard() {
    }
    
    public CraftTaskBombard(Player owner, Planet targetPlanet, CraftTaskAttack feeder) throws ApplicationError {
        super(targetPlanet, owner);        
        addCraftsFrom(feeder);
        
    }

    /** 
     * This task is always ready for assignment as this task is created with the crafts 
     * it needs.
     */
    public boolean readyForAssignment() throws ApplicationError {
        return true;
    }

    /**
     * This moves the fleet to the destination solar system. If at the destination solar 
     * system it selects a planet which has defences and is highly populated and pillages it.
     * If It is at the destination solar system and there is no suitable planet to pillage it 
     * chooses another destination solar system.
     */
    public void assignTaskToFleet() throws ApplicationError {
        Fleet fleet = getFleet();
		if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskBombard.assignTaskToFleet() fleet="+fleet.getName()+" assign "+getDescription() );
        }

		Planet targetPlanet = getTargetPlanet();
        if (targetPlanet == null || fleet == null || complete()) {
            return;
        }

		Planet target = targetPlanet;
        if (fleet.getOrbitStellarObject() != target.getSolarSystem()) {
            fleet.moveTo(target.getOrbit());
        } else {
        	Player owner = targetPlanet.getOwner();
        	if (owner == null) {
        		return;
        	}
        	if (target.isDefended()) {
        		fleet.attack(target);
        	} else if (!target.hasDefenses()) {        		
        		fleet.bombard(target);
        	}
        }
    }

    /** 
     * This task has been completed if there are no crafts attacking us any more.
     */
    public boolean complete() throws ApplicationError {
        int date = MotEngine.instance.getDate();    	
        if (date > getCreatedDate()+MAX_AGE) {
        	return true;
        }
        
		Planet targetPlanet = getTargetPlanet();
        if (targetPlanet == null || targetPlanet.getOwner() == null) {
        	return true;
        }
        
        if (!targetPlanet.getOwner().isEnemy(getOwner())) {
        	return true;
        }
        
        return false;
    }
    
    public boolean requires(Craft craft) throws ApplicationError {
        return craft.containsWeapons();
    }

	protected String getElementName() { return "craft-task-bombard"; }






}