/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.Planet;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

/**
 * This task is used to navigate a craft to colonise a planet. Only a coloniser is sent to colonise
 * the planet so there must not be any any crafts at the destination.
 * 
 * Craft Required - One colony craft.
 * Complete - When target contains a colony.
 */
public class CraftTaskColonise extends CraftTask {

    public int getPriority() { return 80; }
    public String getTaskName() { return "Colonise"; }

    public String getDescription() throws ApplicationError { return super.getDescription()+" planet="+getTarget(); }

    public CraftTaskColonise() {
    }
    
    public CraftTaskColonise(Planet planet, Player owner) throws ApplicationError {
        super(planet, owner);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskColonise.init() New task "+getDescription());
        }
    }

    /**
     * The add craft method is overridden because we do not want to build a fleet 
     * for this target. We just send any craft we have to protect our stuff.
     * @param craft
     * @throws mot.common.util.ApplicationError
     */
    public void add(Craft craft) throws ApplicationError {
        Planet planet = getTargetPlanet();
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskColonise.add() craft="+craft.getName()+" colonise "+planet.getName());
        }
		if (planet != null) {
            Craft [] enemies = craft.getOwner().getEnemyCraftAt(planet.getStellarObject(), false);
            if (enemies != null) {
                // This planet has enemy craft protecting it - we need to notify the general that we
                // need an invasion force. This colony craft will be added to the invasion fleet so
                // it will not ask for orders every turn!
                Player owner = getOwner();
                if (owner instanceof ComputerPlayer) {
                    ((ComputerPlayer) owner).getGeneral().notifyNeedFleetToColonise(planet, craft);
                }
            } else if (!craft.canColonise(planet)) {
            	// The planet condition has been reduced since this
            	if (Logger.isDebug()) Logger.logDebug("CraftTaskColonise.add() colony craft "+craft.getName()+" cannot colonise target planet "+planet.getName());
            	removeColonyCraftFromFleet();
            	craft.setTask(null);
            } else {
                if (craft.getTask() != this) {
                    craft.setTask(this);
                }
        		if (planet.getSolarSystem() != craft.getOrbitStellarObject()) {
        			routeTo(craft, planet.getOrbit());
        		} else {
        			craft.colonise(planet);
        		}
            }
		}
    }
    
    /** 
     * As this task does not use a fleet this is not used!
     * Must be false - otherwise this task will not be reused by General.getHighestPriorityFor().
     */
    public boolean readyForAssignment() throws ApplicationError {
        return false;
    }

    /**
     * As this task does not use a fleet this is not used!
     */
    public void assignTaskToFleet() throws ApplicationError {
    }
    
    private void removeColonyCraftFromFleet() throws ApplicationError {
        Fleet fleet = getFleet();
        if (fleet == null) {
            return;
        }
        Craft [] crafts = fleet.getCrafts();        
        if (crafts == null) {
            return;
        }
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].canColonise()) {
                fleet.remove(crafts[i]);
            }
        }
    }

     /** 
      * This task is complete if the target planet has an owner.
      */
    public boolean complete() throws ApplicationError {
        return (getTargetPlanet().getOwner() != null);
    }
    

    public boolean requires(Craft craft) throws ApplicationError {
        return craft.canColonise(getTargetPlanet());
    }

	protected String getElementName() { return "craft-task-colonise"; }


}