/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.battles.Battle;
import mot.engine.battles.Battles;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftBase;
import mot.engine.crafts.Fleet;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * This task is used to organise crafts being used to defend ourselves from an enemy attack.
 * Enemy crafts are at or destined for one of our planets and this task is directing crafts
 * to goto the planet and defend it.
 * 
 * Craft Required - Millitary craft
 * Complete - When target does not have any enemy craft enroute or located at it.
 */
public class CraftTaskDefend extends CraftTask {

	/**
	 * The quantity of battles lost defending.
	 */
	private int qtyBattlesLost = 0;
	
    // TODO different planets could take presidence for protection!
    public int getPriority() { return 100; }
    public String getTaskName() { return "Defend"; }

    public CraftTaskDefend() {
    }
    
    public CraftTaskDefend(StellarObject so, Player owner) throws ApplicationError {
        super(new Orbit(so), owner);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskDefend.init() New task "+getDescription());
        }
    }

    /**
     * The add craft method is overridden because we do not want to build a fleet 
     * for this target. We just send any craft we have to protect our stuff.
     * @param craft
     * @throws mot.common.util.ApplicationError
     */
    public void add(Craft craft) throws ApplicationError {
    	StellarObject target = getTargetStellarObject();
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskDefend.add() craft="+craft.getName()+" send to defend "+target);
        }
        if (qtyBattlesLost < 2) {
        	rushCraftToDefendAlone(craft, target);
        } else {
        	// Add craft to fleet that will defend the target
        	Fleet fleet = getFleet();
        	if (fleet == null) {
        		Player owner = getOwner();
        		Planet buildFleetLocation = owner.getClosestDefendedPlanetWithBiggestPopulation(target);
        		if (buildFleetLocation == null) buildFleetLocation = owner.getClosestPlanetWithBiggestPopulation(target);
        		if (buildFleetLocation != null) {
	        		if (craft.getActualOrbit() == buildFleetLocation.getStellarObject()) {
	                    super.add(craft);
	        		} else {
	        			routeTo(craft, new Orbit(buildFleetLocation));
	        		}
        		} else {
        			rushCraftToDefendAlone(craft, target);
        		}
        	} else {
     			super.add(craft);
        	}
        }
    }
	private void rushCraftToDefendAlone(Craft craft, StellarObject target) throws ApplicationError {
		// Send craft to target to defend immediately
		if (craft.getTask() != this) {
			craft.setTask(this);
		}
		if ( craft.getLocation().equals(target.getLocation()) ) {
			attack(craft, target);
		} else {
			routeTo(craft, target.getOrbit());
		}
	}
    
	private void attack(CraftBase craft, StellarObject target) throws ApplicationError {
		// If my enemy has already attacked me craft will already be in a battle and 
		// does not need to attack! If craft attacks it may just bring more enemy craft 
		// into the battle
		if (Battles.contains(craft)) {
			return;
		}
		Player owner = getOwner();
		Craft [] enemyCraft = owner.getEnemyCraftAt(target, false);
		if (enemyCraft != null && enemyCraft.length > 0) {
			Battle battle = enemyCraft[0].getBattle();
			if (battle != null && battle.isStarted()) {
				// Battle has already started - maybe I am a craft that has just been created as part of a crisis during the battle when a colony is destroyed
				return;
			}
			craft.attack(target, enemyCraft);
		}
	}

    public boolean readyForAssignment() throws ApplicationError {
        Fleet fleet = getFleet();
    	if (fleet == null) {
	    	// Must be false - otherwise this task will not be reused by General.getHighestPriorityFor().
	    	return false;
    	} else {
    		Orbit target = getTarget();
    		Player owner = getOwner();
    		int qtyEnemy = owner.getQtyEnemyCraftAt(target.getStellarObject());
    		int qtyMine = fleet.getQtyCrafts();
    		return qtyMine >= qtyEnemy;
    	}
    }

    public void assignTaskToFleet() throws ApplicationError {
        Fleet fleet = getFleet();
		if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskDefend.assignTaskToFleet() fleet="+fleet+" assign "+getDescription() );
        }

        if (fleet == null || complete()) {
            return;
        }

		Orbit target = getTarget();
        // Do not assign tasks to the fleet being built.
        if (!fleet.getOrbit().atSameSO(target)) {
        	routeTo(fleet, target);
        } else {
        	attack(fleet, target.getStellarObject());
        }
    }


    /** 
     * This task has been completed if there are no crafts attacking us any more or
     * we are no longer owner of that stellar object.
     */
    public boolean complete() throws ApplicationError {
        Player owner = getOwner();
        StellarObject target = getTargetStellarObject();
        if (!target.isOwnedBy(owner)) {
        	return true;
        }
        int qtyEnemy = owner.getQtyEnemyCraftMovingTo(target);
        return (qtyEnemy == 0);
    }
    
    private static final int DEFEND_FACTOR = 2;

    /**
     * This task only needs crafts with weapons and when it has does not have enough crafts to
     * defend against the attackers.
     * @param craft The craft available for a task that is asking if this task needs it.
     * @return  True if this task is required by this task.
     * @throws mot.common.util.ApplicationError
     */
    public boolean requires(Craft craft) throws ApplicationError {
        if (!craft.containsWeapons()) {
            return false;
        }
        Player owner = getOwner();
        StellarObject target = getTargetStellarObject();
        int qtyMine = owner.getQtyCraftsWithDestination(target);
        int qtyEnemy = owner.getQtyEnemyCraftMovingTo(target);
        return qtyMine <= (qtyEnemy * DEFEND_FACTOR);
    }

    /**
     * If this task has more crafts defending than what is currently required to defend
     * against the attackers, then this method will remove un required crafts from the task.
     */
    public void assesPosition() throws ApplicationError {
        Player owner = getOwner();
        StellarObject target = getTargetStellarObject();
        int qtyMine = owner.getQtyCraftsWithDestination(target);
        int qtyEnemy = owner.getQtyEnemyCraftMovingTo(target);
        Craft [] crafts = owner.getCrafts();
        int qtyRequired = qtyEnemy * DEFEND_FACTOR;
        if (qtyMine > qtyRequired && crafts.length > qtyMine) {
            int qtyRemove = qtyMine - qtyRequired;
            for (int i=qtyRequired; i<crafts.length; i++) {
                if (crafts[i].getTask() == this) {
                    crafts[i].setTask(null);
                }
            }
        }
    }
    
    public void informBattleFinished(Battle battle) throws ApplicationError {
    	Player owner = getOwner();
    	Orbit target = getTarget();
    	if (battle.getActualOrbit() == target.getStellarObject() && !battle.isWinner(owner)) qtyBattlesLost++;
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("qty-battles-lost", qtyBattlesLost);
	}

	protected String getElementName() { return "craft-task-defend"; }
	
	public int getQtyBattlesLost() {
		return qtyBattlesLost;
	}
	
	public void setQtyBattlesLost(int qtyBattlesLost) {
		this.qtyBattlesLost = qtyBattlesLost;
	}

}