/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.StellarObjects;
import mot.engine.players.Player;

/**
 * This task is used to organise crafts that have no weapons, troops and cannot colonise.
 * They are used as explorers - If a craft is assigned to the explorer task it moves to random
 * solar systems owned by the player that have not been visited.
 * If there are no solar system in range that have not been visited then the craft is added
 * to an Idle task.
 */
public class CraftTaskExplore extends CraftTask {

    // TODO different planets could take presidence for protection!
    public int getPriority() { return 5; }
    public String getTaskName() { return "Explore"; }
    public String getDescription() throws ApplicationError { return super.getDescription()+" range="+range; }
    
    /**
     * The range from our closest stellar object that explorers should attempt to 
     * explore.
     */
    private int range;

    public CraftTaskExplore() {
    }
    
    public CraftTaskExplore(Player owner, int range) throws ApplicationError {
        super((Orbit)null, owner);
        this.range = range;
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskExplore.init() New task "+getDescription());
        }
    }

    /**
     * Crafts in this task do not use a fleet. Crafts not contained in a fleet are added to
     * the task every time they need their next orders.
     * @param craft The craft to add to this task.
     * @throws mot.common.util.ApplicationError
     */
    public void add(Craft craft) throws ApplicationError {
        if (craft.getTask() != this) {
            craft.setTask(this);
        }
        StellarObject so = getClosestObjectNotVistedVisitTo(craft.getLocation());
        if (so == null) {
            if (Logger.isDebug()) {
                Logger.logDebug("CraftTaskExplore.add() craft="+craft.getName()+" NOTHING TO EXPLORE ");
            }
        } else {
            if (Logger.isDebug()) {
                Logger.logDebug("CraftTaskExplore.add() craft="+craft.getName()+" send to explore "+so.getName());
            }
            removeFromVisitList(so);
        	routeTo(craft,so.getOrbit());
        }
    }
    
    /** 
     * As this task does not use a fleet this is not used!
     * Must be false - otherwise this task will not be reused by General.getHighestPriorityFor().
     */
    public boolean readyForAssignment() throws ApplicationError {
        Fleet f = getFleet();
        return (f != null && f.getSize() > 0);
    }

    /**
     * As this task does not use a fleet this is not used!
     */
    public void assignTaskToFleet() throws ApplicationError {
    }


    /** 
     * This task has been completed if there are no crafts attacking us any more.
     */
    public boolean complete() throws ApplicationError {
        ArrayList list = getListNeedingAVisit();
        return (list == null || list.size() == 0);
    }

    public boolean requires(Craft craft) throws ApplicationError {
        Fleet f = getFleet();
        return (f == null || f.getSize() == 0) && (!craft.canColonise());
    }
    
    private StellarObject getClosestObjectNotVistedVisitTo(Location loc) throws ApplicationError {
        ArrayList list = getListNeedingAVisit();
        StellarObject closest = null;
        int closestDist = Integer.MAX_VALUE;
        Iterator i = list.iterator();
        while (i.hasNext()) {
            StellarObject so = (StellarObject) i.next();
            int dist = loc.getDistanceTo(so.getLocation());
            if (closest == null || dist < closestDist) {
                closest = so;
                closestDist = dist;
            }
        }
        return closest;
    }
    
    /**
     * Remove this so from the list of objects that need a visit.
     * @param so The stellar object to remove.
     */
    private void removeFromVisitList(StellarObject so) {
        if (listNeedingAVisit != null) {
            listNeedingAVisit.remove(so);
        }
    }
    
    /**
     * This is the list of stellar objects that this task needs to send a craft to visit.
     * This does not need saving as it is dynamically gerenated.
     */
    private transient ArrayList listNeedingAVisit;
    
    /**
     * @return The list of stellar objects that need visiting.
     * @throws mot.common.util.ApplicationError
     */
    private ArrayList getListNeedingAVisit() throws ApplicationError {
        if (listNeedingAVisit == null) {
            StellarObjects sos = MotEngine.instance.getStellarObjects();
            StellarObject [] mySos = getOwner().getStellarObjects();
            if (sos != null && mySos != null) {
                listNeedingAVisit = sos.getNotVistedWithinRange(getOwner(), mySos, range, true);
            }
            if (listNeedingAVisit == null) {
                listNeedingAVisit = new ArrayList();
            }
        }
        return listNeedingAVisit;
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("range", range);
	}

	protected String getElementName() { return "craft-task-explore"; }


    public void setRange(int range) {
        this.range = range;
    }


    public int getRange() {
        return range;
    }

}