/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.players.Player;

/**
 * This task is used to assign a craft when it has nothing useful to do. This could be used for military
 * crafts when we are not at war and colony crafts when there is no planet we can colonise.
 *
 * Craft Required - ALL.
 * Complete - When is qtyIdleTurns turn are up.
 */
public class CraftTaskIdle extends CraftTask {

    public int getPriority() { return 0; }
    public String getTaskName() { return "Idle"; }
    public String getDescription() throws ApplicationError { return super.getDescription()+" idle="+qtyIdleTurns; }
    
    /**
     * This is the quantity of turns this task is to remain.
     */
    private int qtyIdleTurns;

    public CraftTaskIdle() {
    }

    public CraftTaskIdle(Player owner, int qtyIdleTurns) throws ApplicationError {
        super((Orbit)null, owner);
        this.qtyIdleTurns = qtyIdleTurns;
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskIdle.init() New task "+getDescription());
        }
    }

    /**
     * Crafts in this task do nothing - they do not join a fleet and they are given no commands.
     * @param craft The craft to add to this task.
     * @throws mot.common.util.ApplicationError
     */
    public void add(Craft craft) throws ApplicationError {
        if (craft.getTask() != this) {
            craft.setTask(this);
        }
        // Nothing - I am an idle task after all!
    }
    
    /**
     * As this task does not use a fleet this is not used!
     * Must be false - otherwise this task will not be reused by General.getHighestPriorityFor().
     */
    public boolean readyForAssignment() throws ApplicationError {
        return false;
    }

    /**
     * As this task does not use a fleet this is not used!
     */
    public void assignTaskToFleet() throws ApplicationError {
    }

    /**
     * The task is complete if the planet we want to invade satisfies any of the following
     * conditions:
     *  - Has no owner or the owner is me
     *  - It has an owner and we are not at war with its owner
     *  - The planet has weapons.
     */
    public boolean complete() throws ApplicationError {
        int dateOfFleet = getCreatedDate();
        int date = MotEngine.instance.getDate();
        return (date >= (dateOfFleet + qtyIdleTurns));
    }

    public boolean requires(Craft craft) throws ApplicationError {
        return true;
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
		writer.addElement("qty-idle-turns", qtyIdleTurns);
	}

	protected String getElementName() { return "craft-task-idle"; }


    public void setQtyIdleTurns(int qtyIdleTurns) {
        this.qtyIdleTurns = qtyIdleTurns;
    }


    public int getQtyIdleTurns() {
        return qtyIdleTurns;
    }

}