/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.battles.Battle;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.event.PlanetBombardedEF;
import mot.engine.planets.Planet;
import mot.engine.players.Player;
import mot.engine.players.PlayerReference;
import mot.engine.players.UserPlayer;

/**
 * This task is used to organise crafts to pillage and generally cause destruction one 
 * one or more other players. The crafts assigned to this task, move from solar system 
 * to solar system destroying all planet defences and pillaging the planets. After pillaging 
 * a planet in a solar system, the task moves the fleet of to another solar system.
 * 
 * NOTE - When this task completes it destroys all the crafts associated with it.
 * 
 * Craft Required - Military craft
 * Complete - 25 Turns of when all the crafts assigned to it have been destroyed or there are
 * no eligible planets to attack..
 */
public class CraftTaskMaraud extends CraftTask {

    /**
     * The maximum age of this task.
     */
    public static final int MAX_AGE = 35;

    /**
     * This is the player this task has been assigned to maraud.
     */
    private PlayerReference targetPlayer;
    
    /**
     * This is the maximum quantity of planets this task will pillage (maraud).
     * Once qtyPlanetsPillaged is equal to this the task is complete.
     */
    private int maxPlanetsPillaged;

    /**
     * This is the quantity of planets this task has pillaged (marauded).
     */
    private int qtyPlanetsPillaged;

    public int getPriority() { return 25; }
    
    public String getTaskName() { return "Maraud"; }

    public CraftTaskMaraud() {
    }
    
    public CraftTaskMaraud(Player owner, Player targetPlayer, Fleet fleet, int maxPlanetsPillaged) throws ApplicationError {
        super((Orbit)null, owner);
        this.targetPlayer = new PlayerReference(targetPlayer);
        this.maxPlanetsPillaged = maxPlanetsPillaged;
        this.qtyPlanetsPillaged= 0;
        setFleet(fleet);
        Planet targetPlanet = getBestPlanetToAttack(targetPlayer.getPlanets());
        if (targetPlanet != null) {
        	setTarget(new Orbit(targetPlanet));
        }
        fleet.setTask(this);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskMaraud.init() New task "+getDescription());
        }
    }

    /** 
     * This task is always ready for assignment as this task is created with the crafts 
     * it needs.
     */
    public boolean readyForAssignment() throws ApplicationError {
        return true;
    }

    /**
     * This moves the fleet to the destination solar system. If at the destination solar 
     * system it selects a planet which has defences and is highly populated and pillages it.
     * If It is at the destination solar system and there is no suitable planet to pillage it 
     * chooses another destination solar system.
     */
    public void assignTaskToFleet() throws ApplicationError {
        Fleet fleet = getFleet();
		if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskMaraud.assignTaskToFleet() fleet="+fleet.getName()+" assign "+getDescription() );
        }

        if (fleet == null || complete()) {
            return;
        }

		Planet target = getTargetPlanet();
		
        if (fleet.getOrbitStellarObject() != target.getSolarSystem()) {
            fleet.moveTo(target.getOrbit());
        } else {
            fleet.attack(target);
        }
    }

    private Planet getBestPlanetToAttack(Planet [] planets) throws ApplicationError {
        Planet best = null;
        for (int i=0; i<planets.length;i++) {
            if (targetPlayer == null || targetPlayer.getPlayer() == null) return null;
            if (planets[i].getOwner() == targetPlayer.getPlayer() && canPillage(planets[i])) {
                if (best == null) {
                    best = planets[i];
                } else if (best.getPopulation() < planets[i].getPopulation()) {
                    best = planets[i];
                }
            }
        }
        return best;
    }
    
    private boolean canPillage(Planet planet) throws ApplicationError {
        return  planet.hasDefenses() && planet.getPopulation() > (planet.getMaxPopulation()/2);
    }

    /** 
     * This task has been completed if there are no crafts attacking us any more.
     */
    public boolean complete() throws ApplicationError {
        if (qtyPlanetsPillaged >= maxPlanetsPillaged) {
            destroyCraftAndFleet();
            return true;
        }
		Planet target = getTargetPlanet();
        if (targetPlayer == null || targetPlayer.getPlayer() == null || target == null || target.getOwner() == null) {
            destroyCraftAndFleet();
            return true;
        }
        int date = MotEngine.instance.getDate();
        Fleet fleet = getFleet();
        Craft [] crafts = (fleet == null) ? null : fleet.getCrafts();
        boolean isComplete = targetPlayer.getPlayer().getQtyPlanets() == 0 || (date > getCreatedDate()+MAX_AGE) || crafts == null || crafts.length == 0;
        if (isComplete && crafts != null) {
            destroyCraftAndFleet();
        }
        return isComplete;
    }
    
    private void destroyCraftAndFleet() throws ApplicationError {
        Fleet fleet = getFleet();
		if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskMaraud.destroyCraftAndFleet() fleet="+fleet+" Task Complete - Destroy fleet and crafts " );
        }
        if (fleet != null) {
            Craft [] crafts = fleet.getCrafts();
            if (crafts != null) {
                for (int i=0; i<crafts.length; i++) {
                    crafts[i].destroy();
                }
            }
        }
    }

    public boolean requires(Craft craft) throws ApplicationError {
        return craft.containsWeapons();
    }

    /**
     * If the battle contained a planet it halves the population and quantity of factories.
     */
    public void informBattleFinished(Battle battle) throws ApplicationError {
        if (battle.getPlanet() != null && targetPlayer != null && targetPlayer.getPlayer() != null) {
            Planet planet = battle.getPlanet().getPlanet();
            double qtyFactories = planet.getQtyFactories();
            double pop = planet.getPopulation();
            double maxPop = planet.getMaxPopulation();
            Player planetOwner = planet.getOwner();
            if (qtyFactories > 0 || pop > maxPop/10) {
                planet.setPopulation(pop/2);
                planet.setQtyFactories(qtyFactories/2);
                double bomPop = pop - planet.getPopulation();
                double bomFact = qtyFactories - planet.getQtyFactories();
                PlanetBombardedEF event = new PlanetBombardedEF(planet, bomPop, bomFact);
                if (planetOwner instanceof UserPlayer) {
                	event.addStateUpdate(new PlanetVOAdapter(planet, planetOwner));
                }
				MotEngine.instance.broadcast(event, planetOwner);
                qtyPlanetsPillaged++;
            }
            Planet best = getBestPlanetToAttack(targetPlayer.getPlayer().getPlanets());
            if (best != null) {
                setTarget(new Orbit(best));
            } else {
                setTarget(null);
            }
        }
    }

	protected String getElementName() { return "craft-task-maraud"; }

    protected void encode(XMLWriter writer) throws ApplicationError {
        super.encode(writer);
        if (targetPlayer != null && targetPlayer.getPlayer() != null) {
            targetPlayer.toXML(writer, "target-player");
        }
        writer.addElement("maxPlanetsPillaged", maxPlanetsPillaged);
        writer.addElement("qtyPlanetsPillaged", qtyPlanetsPillaged);
    }


    public void setTargetPlayer(PlayerReference targetPlayer) {
        this.targetPlayer = targetPlayer;
    }


    public PlayerReference getTargetPlayer() {
        return targetPlayer;
    }


    public void setMaxPlanetsPillaged(int maxPlanetsPillaged) {
        this.maxPlanetsPillaged = maxPlanetsPillaged;
    }


    public int getMaxPlanetsPillaged() {
        return maxPlanetsPillaged;
    }


    public void setQtyPlanetsPillaged(int qtyPlanetsPillaged) {
        this.qtyPlanetsPillaged = qtyPlanetsPillaged;
    }


    public int getQtyPlanetsPillaged() {
        return qtyPlanetsPillaged;
    }

}