/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import java.util.ArrayList;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.players.Player;

/**
 * This task is used to organise crafts to travel around the galaxy attacking all craft /
 * fleet on the galaxy map that belong to players (anybody but monster player). This task
 * is only complete when the craft it commands is destroyed. 
 * 
 * This task is used by Kaan
 * 
 * Craft Required - Military craft.
 * Complete - when its only craft is destroyed.
 */
public class CraftTaskMaraudCraft extends CraftTask {
	
	/**
	 * Quantity of turns to wait before selecting a new target for attack. 
	 */
	private static final int WAIT_TIME = 10;
	
	/**
	 * The date this task's fleet arrived at the current solar system.
	 */
	private Integer  arrivedDate = null;

    public int getPriority() { return 0; }
    
    public String getTaskName() { return "MaraudCraft"; }

    public CraftTaskMaraudCraft() {
    }
    
    public CraftTaskMaraudCraft(Player owner, Craft craft, Player preferredPlayer) throws ApplicationError {
        super(getNextTarget(preferredPlayer, owner), owner);
        Fleet fleet = FleetFactory.create(craft);
        setFleet(fleet);
        fleet.setTask(this);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskMaraudCraft.init() New task "+getDescription());
        }
    }

    /** 
     * This task is always ready for assignment as this task is created with the crafts 
     * it needs.
     */
    public boolean readyForAssignment() throws ApplicationError {
        return true;
    }

    /**
     * This moves the fleet to the destination solar system. If at the destination solar 
     * system it selects a planet which has defences and is highly populated and pillages it.
     * If It is at the destination solar system and there is no suitable planet to pillage it 
     * chooses another destination solar system.
     */
    public void assignTaskToFleet() throws ApplicationError {
        Fleet fleet = getFleet();
		if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskMaraudCraft.assignTaskToFleet() fleet="+fleet+" assign "+getDescription() );
        }

        if (fleet == null || complete()) {
            return;
        }

        StellarObject target = getTargetStellarObject();
        if (target != null && fleet.getOrbitStellarObject() != target) {
            fleet.moveTo(target.getOrbit());
        } else {
        	Player owner = getOwner();
        	Craft[] enemies = null;
        	if (target != null) {
        		enemies = owner.getEnemyCraftAt(target, false);
        	}
        	if (enemies != null && enemies.length > 0) {
        		fleet.attack(target, enemies);
        		arrivedDate = -1;
        	} else {
        		if (fleet.isBlockading()) {
            		if (arrivedDate == null) {
            			arrivedDate = MotEngine.instance.getDate();
            			return;
            		} else if (MotEngine.instance.getDate() < (arrivedDate + WAIT_TIME)) {
            			// wait at this location for a few turns
            			return;
            		}
        		}
    			setArrivedDate(null);
    			Orbit nextTarget = getNextTarget(null, owner);
    			if (nextTarget != null && nextTarget.getStellarObject() != target) {
    				setTarget(nextTarget);
    				fleet.moveTo(nextTarget);
    			} else {
    				setTarget(null);
    			}
        	}
        }
    }

    private static Orbit getNextTarget(Player preferredPlayer, Player owner) throws ApplicationError {
    	if (MotEngine.instance == null) return null;
    	// If we have a preferred player - choose Stellar Object belongs to preferredPlayer that has the most  military craft
    	StellarObject [] sos = MotEngine.instance.getStellarObjectsNotNull().getStellarObjects();
    	if (sos == null) return null;
    	if (preferredPlayer != null) {
    		int bestQty = 0;
    		StellarObject best = null;
    		for (int i=0; i<sos.length; i++) {
    			if ( sos[i].isOwnedBy(preferredPlayer) ) {
    				int qty = preferredPlayer.getQtyMilitaryCraftsAt(sos[i]);
    				if (qty > bestQty) {
    					bestQty = qty;
    					best = sos[i];
    				}
    			}
    		}
    		if (best != null) {
    			return new Orbit(best);
    		}
    	}
    	// choose a random stellar object owned by a player (not monster) that has military craft
		ArrayList candidates = new ArrayList();
		for (int i=0; i<sos.length; i++) {
			int qty = owner.getQtyEnemyCraftAt(sos[i]);
			if (qty > 0) {
				candidates.add(sos[i]);
			}
		}
		if (candidates.size() == 0) {
            return null;
        }
        int i = Utils.random(candidates.size());
        return new Orbit((StellarObject) candidates.get(i));
    }
    
    /** 
     * This task has been completed if its fleet is destroyed.
     */
    public boolean complete() throws ApplicationError {
    	Fleet fleet = getFleet();
        return (fleet == null) || (fleet.getSize() == 0) ;
    }
    
    public boolean requires(Craft craft) throws ApplicationError {
        return false;
    }

	public boolean canDonateCraft() {
		return false;
	}
    
	protected void encode(XMLWriter writer) throws ApplicationError {
        super.encode(writer);
        if (getArrivedDate() != null) {
        	writer.addElement("arrived-date", getArrivedDate()); 
        }
    }
	
	protected String getElementName() { return "craft-task-maraud-craft"; }

	public Integer getArrivedDate() {
		return arrivedDate;
	}

	public void setArrivedDate(Integer arrivedDate) {
		this.arrivedDate = arrivedDate;
	}

}