/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.battles.Battle;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.players.Player;

/**
 * This task is used to organise crafts to protect a solar system. The system not 
 * under attack but crafts assigned to this task are stationed in orbit around the 
 * solar system to protect it. If any arrives in the system they will attack the 
 * enemies.
 * 
 * Craft Required - Military craft
 * Complete - When there are no crafts assigned to it.
 *            (This is used by monsters - when they are defeated - they are gone).
 */
public class CraftTaskProtect extends CraftTask {

    // TODO different planets could take residence for protection!
    public int getPriority() { return 25; }
    public String getTaskName() { return "Protect"; }

    public CraftTaskProtect() {
    }
    
    public CraftTaskProtect(StellarObject orbit, Player owner, Craft craft) throws ApplicationError {
        this(orbit, owner);
        add(craft);
    }
    
    public CraftTaskProtect(StellarObject orbit, Player owner) throws ApplicationError {
        super(new Orbit(orbit), owner);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskProtect.init() New task "+getDescription());
        }
    }

    /**
     * The add craft method is overridden because we do not want to build a fleet 
     * for this target. We just send any craft we have to protect our stuff.
     * @param craft
     * @throws mot.common.util.ApplicationError
     */
    public void add(Craft craft) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskProtect.add() craft="+craft.getName()+" send to protect "+getTarget());
        }
        if (craft.getTask() != this) {
            craft.setTask(this);
        }
        StellarObject so = getTargetStellarObject();
        if ( craft.getLocation().equals(so.getLocation()) ) {
            Craft [] enemyCraft = getOwner().getEnemyCraftAt(so, false);
            if (enemyCraft != null && enemyCraft.length > 0) {
                craft.attack(so, enemyCraft);
            }
        } else {
        	routeTo(craft, so.getOrbit());
        }
    }

    /**
     * Must not send crafts one at a time to defend a planet losing each one at a time.
     * 
     */
    public void informBattleFinished(Battle battle) throws ApplicationError {
    	// TODO
    }


    /** 
     * As this task does not use a fleet this is not used!
     * Must be false - otherwise this task will not be reused by General.getHighestPriorityFor().
     */
    public boolean readyForAssignment() throws ApplicationError {
        return false;
    }

    /**
     * As this task does not use a fleet this is not used!
     */
    public void assignTaskToFleet() throws ApplicationError {
    }


    /** 
     * This task has been completed if the crafts assigned to it have been destroyed.
     */
    public boolean complete() throws ApplicationError {
        return getOwner().getQtyCraftsAt(getTargetStellarObject()) == 0;
    }

    public boolean requires(Craft craft) throws ApplicationError {
        return craft.containsWeapons();
    }

	protected String getElementName() { return "craft-task-protect"; }

}