/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;

/**
 * This task is used to organise crafts being used to colonise another planet that is being 
 * defended by an enemies craft - Note the enemy is often a monster.
 * 
 * Craft Required - Military craft and one colony craft.
 * Complete - When target contains a colony of mine or there are no enemy craft at the target.
 */
public class CraftTaskSteelColony extends CraftTask {

    public int getPriority() { return 70; }
    public String getTaskName() { return "Colonise Guarded Planet"; }
    public String getDescription() throws ApplicationError { return super.getDescription()+" planet="+getTarget(); }

    public CraftTaskSteelColony() {
    }
    
    public CraftTaskSteelColony(Planet planet, Player owner) throws ApplicationError {
        super(planet, owner);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskSteelColony.init() New task "+getDescription());
        }
    }

    /** 
     * This task has been ready for dispatch if I have more weapons in my fleet than my enemies have 
     * defending the planet in their crafts or my enemies have gone!
     */
    public boolean readyForAssignment() throws ApplicationError {
        Player owner = getOwner();
        Fleet fleet = getFleet();
        if (fleet == null || fleet.getSize() == 0) {
            return false;
        }
        Craft [] enemies = owner.getEnemyCraftAt(getTargetStellarObject(), false);
        if (enemies == null || enemies.length == 0) {
            return true;
        }
        long qtyEnemies = getTotalWeaponsDamage(enemies);
        long qtyMine = getTotalWeaponsDamage(fleet.getCrafts());
        return (qtyMine > qtyEnemies);
    }
    
    /**
     * This moves the fleet to the enemy solar system and then attacks the target. When there is no enemy
     * it attempt to colonise the planet.
     */
    public void assignTaskToFleet() throws ApplicationError {
        Fleet fleet = getFleet();
		if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskSteelColony.assignTaskToFleet() fleet="+fleet.getName()+" assign "+getDescription() );
        }

        if (fleet == null || fleet.getSize() == 0) {
            return;
        }

        Planet planet = getTargetPlanet();
        // Do not assign tasks to the fleet being built.
        if (fleet.getOrbitStellarObject() != planet.getStellarObject()) {
        	routeTo(fleet, new Orbit(planet));
        } else {
        	SolarSystem ss = planet.getSolarSystem();
            Craft []  enemies = fleet.getOwner().getEnemyCraftAt(ss, false);
            if (enemies != null && enemies.length > 0) {
                fleet.attack(ss, enemies);
            } else {
                Craft colonyCraft = removeColonyCraftFromFleet();
                if (colonyCraft != null && planet != null) {
                    planet.colonise(colonyCraft);
                } else {
                    Logger.logWarning("CraftTaskSteelColony.dispatch() Steel Colony has no colony craft in fleet");
                }
            }
        }
    }
    
    private Craft removeColonyCraftFromFleet() throws ApplicationError {
        Fleet fleet = getFleet();
        if (fleet == null) {
            return null;
        }
        Craft [] crafts = fleet.getCrafts();        
        if (crafts == null) {
            return null;
        }
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].canColonise()) {
                fleet.remove(crafts[i]);
                return crafts[i];
            }
        }
        return null;
    }
    
    private static final int MAX_AGE = 50;

     /** 
     * This task is complete if there are no enemies at the target planet OR the
     * target planet belongs to me.
     */
    public boolean complete() throws ApplicationError {
        Player owner = getOwner();
        Planet planet = getTargetPlanet();
        Craft [] enemies = owner.getEnemyCraftAt(planet.getStellarObject(), false);
        if (enemies == null || enemies.length == 0) {
            Fleet fleet = getFleet();
            return !(planet.getOwner() == null && fleet != null && fleet.canColonise(planet));
        }
        int date = MotEngine.instance.getDate();
        long createdDate = getCreatedDate();
        return (planet.getOwner() != null) || (date > createdDate+MAX_AGE);
    }
    
    public static int getTotalWeaponsDamage(Craft [] crafts) {
        if (crafts == null) return 0;
        int total = 0;
        for (int i=0; i<crafts.length; i++) {
            if (crafts[i].getWeaponBays() != null) {
                total += crafts[i].getWeaponBays().getDamageSum(false);
            }
        }
        return total;
    }

    public boolean requires(Craft craft) throws ApplicationError {
        return craft.containsWeapons();
    }

	protected String getElementName() { return "craft-task-steel-colony"; }


}