/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.bean.DiplomaticStatements;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * This task is used to organise crafts being used to take a planet from another player that has no defenses
 * and we would like to take.
 * 
 * Craft Required - Military craft and one colony craft.
 * Complete - Crafts taken by another task or planet is taken or planet is defended by more than one missile.
 */
public class CraftTaskSurpriseAttackStealColony extends CraftTask {

    public int getPriority() { return 60; }
    public String getTaskName() { return "Suprise Attack on Planet"; }
    public String getDescription() throws ApplicationError { return super.getDescription()+" planet="+getTarget(); }

    public CraftTaskSurpriseAttackStealColony() {
    }
    
    public CraftTaskSurpriseAttackStealColony(Planet planet, Player owner) throws ApplicationError {
        super(planet, owner);
        if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskSurpriseAttackStealColony.init() New task "+getDescription());
        }
    }

    /** 
     * This task has been ready for dispatch if I have more weapons in my fleet than my enemies have 
     * defending the planet in their crafts or my enemies have gone!
     */
    public boolean readyForAssignment() throws ApplicationError {
    	Fleet fleet = getFleet();
    	Planet target = getTargetPlanet();
    	
    	return getQtyColonisersInFleet() >= 1 && getQtyMillitaryInFleet() >= 2;
    }
    
    /**
     * Declare war and attack the planet. This task is only ready for assignment when all crafts are in orbit at the destination planet.
     */
    public void assignTaskToFleet() throws ApplicationError {
        Fleet fleet = getFleet();
        Planet target = getTargetPlanet();
        Player owner = getOwner();
        
		if (Logger.isDebug()) {
            Logger.logDebug("CraftTaskSurpriseAttackStealColony.assignTaskToFleet() fleet="+fleet.getName()+" assign "+getDescription() );
        }

        if (target == null || fleet == null || fleet.getSize() == 0) {
            return;
        }
        
        if (!fleet.isOrbiting(target)) {
        	fleet.moveTo(target.getOrbit());
        	return;
        }
        
        Player targetOwner = target.getOwner();
        
        if (targetOwner == null) {
        	// We have destroyed the old colony - we can now colonise the planet
        	Craft coloniser = fleet.getColoniser();
        	if (coloniser != null) coloniser.colonise(target);
        } else if (targetOwner != owner) {
        	if (!targetOwner.isWar(owner)) {
        		targetOwner.startWar(owner, owner.getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DECLARE_WAR_SUPRISE_ATTACK), false);        		
        	}
        	if (target.hasDefenses()) {
        		fleet.attack( target );
        	} else {
        		fleet.bombard(target);
        	}
        	if (target.getOwner() == null) {
        		Craft coloniser = fleet.getColoniser();
            	if (coloniser != null) coloniser.colonise(target);
        	}
        }
        
    }
        
    private static final int MAX_AGE = 10;

     /** 
     * This task is complete if there are enemies at the target planet OR the
     * target planet is not well defended.
     */
    public boolean complete() throws ApplicationError {
        Player owner = getOwner();
        Planet planet = getTargetPlanet();
        int qtyEnemies = owner.getQtyEnemyCraftAt(planet.getStellarObject());
        if (qtyEnemies > 0) {
        	// Not a surprise attack if enemies are present
            return true;
        }
        
        int date = MotEngine.instance.getDate();
        long createdDate = getCreatedDate();
        if (date > createdDate+MAX_AGE){
            return true;
        }
        
        if (planet.getQtyWeapons() >= 2) {
        	// planet is now well defended
            return true;
        }
        
        if (planet.getOwner() == owner) {
        	// I have successfully taken the planet
            return true;
        }
        
        if (getQtyColonisers() == 0) {
        	// Our coloniser has been destroyed - we failed!
            return true;
        }
        
        if (getQtyMillitary() == 0) {
        	// Our millitary craft have been destroyed - we failed!
            return true;
        }
        
        return false;
    }
    

	public boolean requires(Craft craft) throws ApplicationError {
		Fleet fleet = getFleet();
		if (craft.canColonise()) {
			return fleet == null || fleet.getQtyColinserCraft() == 0;
		} else {
			return fleet == null || fleet.getQtyMilitaryCraft() < 2;
		}
    }

	private boolean containsColonyCraft() throws ApplicationError {
		Craft[] crafts = getAllCrafts();
		if (crafts != null) {
			for (int i = 0; i < crafts.length; i++) {
				if (crafts[i].canColonise()) return true;
			}
		}
		return false;
	}
	
	protected String getElementName() { return "craft-task-surprise-attack-steal-colony"; }


}