/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import java.util.ArrayList;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.common.OwnedObject;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.players.Player;
import mot.engine.transports.Transport;

/**
 * Generals manage the priorities for craft belonging to a computer player. It examines the
 * players position and assigns priorities to different tasks that need performing, then
 * crafts can be added to tasks and reduce their priority.
 *
 * Generals do not manage crafts when they are in battle - this is done by Commanders.
 * 
 * Task Name			Priority
 * Task Idle			0
 * Task Explore			5
 * Task Attack			10
 * Task Maraud			25
 * Task Protect			25
 * Task Bombard			40
 * Task Surprise Attack	60
 * Task Steal Colony	70
 * Task Colonise		80
 * Task Defend			100
 * 		
 */
public abstract class General extends OwnedObject {

	/**
	 * Contains a list of priorities built at the end of every turn.
     * This is dynamically generated from taskList;
	 */
	private transient CraftTask[] taskArray = null;
    
    /**
     * This is the list of tasks defined for this player. Each task is a requirement for
     * crafts to be built to fulfil.
     */
    private ArrayList taskList = null;

	/**
	 * The qty crafts in the fleet we would like to construct for this player for attacking 
     * enemy players. We may not actually build this many crafts in time (see dateOfFleet for 
     * details).
	 */
	private int qtyCraftsInAttackFleet = 2;
    
    public void _add(CraftTask task) throws ApplicationError {
        if (taskList == null) taskList = new ArrayList();
        taskArray = null;
        taskList.add(task);
    }

    public CraftTask add(CraftTask task) throws ApplicationError {
        if (taskList == null) taskList = new ArrayList();
        checkForTransfers(task);
        taskArray = null;
        taskList.add(task);
        return task;
    }

    public void remove(CraftTask task) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("General.remove() Remove task "+task.getDescription());
        }
        taskArray = null;
        if (taskList == null) return;
        task.removeCraftsFromTask();
        taskList.remove(task);
        task.deleteFleet();
    }

    public CraftTask[] getTasks() {
        if (taskArray == null && taskList != null && taskList.size() > 0) {
           taskArray = (CraftTask[])taskList.toArray(new CraftTask[(taskList.size())]);
        }
        return taskArray;
    }


    /**
     * 
     * @throws mot.common.util.ApplicationError
     */
	protected General() {
	}

	protected General(Player owner) {
		super(owner);
	}

	/**
	 * This searches for the highest priority task for this craft and assigns the craft 
     * to it. If there are no tasks requiring this craft and the craft is a scout it
     * destroys the craft. If there is no task for the craft and the craft is not a scout
     * no action is taken.
     * If the craft is assigned to a task then the task is tested for completeness and if
     * complete removed.
     * @param craft The craft to assign orders to.
	 */
	public void assignTaskToCraft(Craft craft) throws ApplicationError {
        if (Logger.isDebug()) {
			Logger.logDebug("General.assignTaskToCraft() craft="+craft.getName());
        }
		if (craft.getFleet() != null) {
			Logger.logWarning("General.assignTaskToCraft() Cannot assign task to craft in fleet craft="+craft.getName());
			Logger.logStackTrace();
			return;
		}
        
		// Assign craft to highest priority
		CraftTask task = craft.getTask();
        if (task == null) {
            task = _createTaskForCraft(craft);
        }

        if (task != null) {
            if (task.complete()) {
                remove(task);
                task = _createTaskForCraft(craft);
            } else if (task instanceof CraftTaskIdle) {
            	CraftTask newTask = _createTaskForCraft(craft);
            	if (newTask != null && !(newTask instanceof CraftTaskIdle)) {
            		task = newTask;
            	}
            }
        }
        
        if (task == null) {
            Logger.logWarning("General.assignTaskToCraft() No task asigned to craft "+craft.getName()+" for owner "+getOwner().getName());
            return;
        }

        // If task is new - it will call checkForUrgenTransfers() 
        // so craft may have already been added to task
        if (task.contains(craft)) return;


		// assign this craft to this task.
        if (task.getFleet() != null) {
            if (task.getFleet().getOrbit() != null) {
                if (craft.getOrbit() == task.getFleet().getOrbit()) {
                    task.add(craft);
                } else {
                	task.routeTo(craft, task.getFleet().getOrbit());
                }
            } else {
                Logger.logWarning("General.assignTaskToCraft() Craft "+craft.getName()+" cannot join fleet "+task.getFleet().getName()+" because it is at warp");
            }
        } else {
            task.add(craft);
        }
	}

    private CraftTask _createTaskForCraft(Craft craft) throws ApplicationError {
        CraftTask newTask = createTaskFor(craft);
        CraftTask bestAvail = getHighestPriorityFor(craft);
        if (newTask != null && (bestAvail == null || bestAvail.getPriority() < newTask.getPriority())) {
            return newTask;
        } else {
            return bestAvail;
        }
    }

    /**
     * @param fleet The fleet to assign orders to.
     * @throws mot.common.util.ValidationError
     * @throws mot.common.util.ApplicationError
     */
	public void assignTaskToFleet(Fleet fleet) throws ApplicationError {
        // Check that the task this fleet was assigned to has not been completed
        CraftTask myTask = fleet.getTask();
        if (Logger.isDebug()) {
			Logger.logDebug("General.assignTaskToFleet() craft="+fleet.getName()+" task="+((myTask==null)?"None":myTask.getDescription()));
        }
        if (myTask == null) {
            // Each craft will individually ask for orders
            fleet.destroy();
            return;
        }

        if (myTask.complete()) {
            remove(myTask);
            return;
        }

		// First check to that there are not any more urgent tasks waiting
		CraftTask urgentTask = getHighestPriority();
        if (urgentTask != null && urgentTask.getPriority() > myTask.getPriority() && myTask.canDonateCraft()) {
            urgentTask.addCraftsFrom(myTask);
        }

        if (myTask.readyForAssignment()) {
            myTask.assignTaskToFleet();
        } else {
        	myTask.assignPreReadyTask();
        }

	}

    /**
     * This is called every time a new task is added to the list of computer tasks.
     * If a task is more urgent than another task with crafts assigned to it this method moves crafts 
     * from other tasks to this more urgent task.
     * @param task The new task that has been added to the list of tasks.
     */
    private void checkForTransfers(CraftTask task) throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks == null) return;
        for (int i=0; i<tasks.length; i++) {
            if (task.getPriority() > tasks[i].getPriority() && tasks[i].canDonateCraft()) {
                task.addCraftsFrom(tasks[i]);
            }
        }
    }

    /**
     * Called when a colonise task could not colonise a planet because enemy crafts are located 
     * at the destination. This method creates a Steel colony task.
     * @param planet The planet that troop transports are required to invade.
     */
    public void notifyNeedFleetToColonise(Planet planet, Craft craft) throws ApplicationError {
        CraftTask task = new CraftTaskSteelColony(planet, getOwner());
        add( task );
        task.add(craft);
    }

	/**
	 * Called when another a craft has arrived at an enemies solar system. It must return
     * which planet to attack in the solar system.
	 */
    public abstract Planet getPlanetToAttack(Planet [] planets) throws ApplicationError;

    /**
     * Called when a Space Station has been built to find out which planet the owner would
     * like to deploy the station to.
     */
	public abstract Planet getBestPlanetCanDeployMiningStation(Transport transport) throws ApplicationError;

    /**
     * @return Return true if the general would consider mining this planet. 
     */
	public abstract boolean canDeployMiningStation(Planet p, boolean rightNow) throws ApplicationError;
	public abstract boolean canDeployMiningStation(BlackHole blackHole) throws ApplicationError;
	
    /**
     * Called when an outpost has been built to find out which planet the owner would
     * like to deploy it to.
     */
	public abstract StellarObject getBestStellarObjectToDeployOutpost() throws ApplicationError;

    /**
     * @return True if this general would like a planet to build an outpost to extend the range
     * of its crafts.
     */
    public abstract boolean needOutpost() throws ApplicationError;
	public abstract void clearOutpostNeeded();
    
    /**
     * This method creates a task suitable for the craft passed in (if any exist).
     * @param craft The craft to create a suitable task for.
     * @return A new Task or null.
     */
    public abstract CraftTask createTaskFor(Craft craft) throws ApplicationError;
    
    /**
     * This method asses the players position and creates tasks for any urgent tasks. It should
     * look for conditions like enemy craft enroute to my planet and create a defend task.
     * This should be called at the end of every turn.
     */
    protected abstract void calculatePriorities() throws ApplicationError;
   

    /**
     * After a successful attack on a planet, a CP attempts to move colonists to the target planet.
     * If the CP cannot move colonists, then it will call this to convert the attack to a bombard.
     * @param planet an enemy planet the player wishes to bombard. 
     * @throws ApplicationError 
     */
	public abstract void convertAttackToBombardAt(Planet planet) throws ApplicationError;
	

	/**
	 * The strategist has decided it would like to go to war in a surprise attack on the planet target
	 * to colonise it using the colonyCraft
	 * @param colonyCraft The craft to colonise target
	 * @param target The target is a planet not owner by this player and not currently an enemy of this
	 * player that is not well defended - This player wants to take it by force in a surprise attack.
	 * @throws ApplicationError 
	 */
	public void createSurpriseAttackStealColony(Craft colonyCraft, Planet target) throws ApplicationError {
		Player owner = getOwner();
		if (owner.getQtyMilitartyCrafts() >= 2) {
			if (colonyCraft.getTask() != null) colonyCraft.getTask().removeCraft(colonyCraft);
			CraftTaskSurpriseAttackStealColony task = new CraftTaskSurpriseAttackStealColony(target, owner);
			add(task);
			task.add(colonyCraft);
		}
	}
	
    public void endTurn() throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("General.endTurn() for "+getOwner().getName());
        }
        CraftTask[] tasks = getTasks();
        if (tasks != null) {
            for (int i=0; i<tasks.length; i++) {
                if ( tasks[i].complete() && tasks[i].canDelete() ) {
                    remove(tasks[i]);
                } else if (tasks[i].getFleet() != null && tasks[i].getFleet().getSize() == 0) {
                    tasks[i].deleteFleet();
                }
            }
        }
        calculatePriorities();
    }

    /**
     * Calls assesPosition() on each task.
     * @throws mot.common.util.ApplicationError
     */
    public void assesTaskPositions() throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks != null) {
            for (int i=0; i<tasks.length; i++) {
                tasks[i].assesPosition();
            }
        }
    }

	/**
	 * Returns the highest priority task that is closest to this location and not already 
     * been dispatched.
	 * It always chooses the highest priority, but if two tasks are or equals priority
	 * it will select the closest.
     * It only returns tasks that have not already been dispatched.
	 */
	public CraftTask getHighestPriorityFor(Craft craft) throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks == null) {
            return null;
        }

        Location loc = craft.getLocation();
		CraftTask best = null;
		for (int i=0; i<tasks.length; i++) {
            if (!tasks[i].readyForAssignment()) {
                if (tasks[i].requires(craft)) {
                    if ( best == null ||  tasks[i].higherPriority(best, loc) ) {
                        best = tasks[i];
                    }
                }
            }
		}
		return best;
	}

	/**
	 * Returns the highest priority task regardless of any other condition.
	 */
	public CraftTask getHighestPriority() throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks == null) {
            return null;
        }

		CraftTask best = null;
		for (int i=0; i<tasks.length; i++) {
            if ( best == null ||  tasks[i].getPriority() > best.getPriority() ) {
                best = tasks[i];
            }
		}
		return best;
	}

	/**
	 * Returns the task that contains this name or null if no task exist containing this 
     * name.
	 */
	public CraftTask getTaskForName(String name) throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks == null) {
            return null;
        }

		for (int i=0; i<tasks.length; i++) {
            if ( Comp.same(tasks[i].getName(), name) ) {
                return tasks[i];
            }
		}
		return null;
	}

	/**
	 * Returns the task that contains this fleet or null if no task exist containing this 
     * fleet.
	 */
	public CraftTask getTaskForFleet(Fleet fleet) throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks == null) {
            return null;
        }

		for (int i=0; i<tasks.length; i++) {
            if ( tasks[i].getFleet() == fleet ) {
                return tasks[i];
            }
		}
		return null;
	}

	/**
	 * Returns the task first task of the type passed in.
	 */
	public CraftTask getTaskType(Class theClass) throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks == null) {
            return null;
        }

		for (int i=0; i<tasks.length; i++) {
            if ( tasks[i].getClass() == theClass ) {
                return tasks[i];
            }
		}
		return null;
	}

	/**
	 * Returns the first task of the type passed in whose target is so
	 */
	public CraftTask getTaskType(Class theClass, StellarObject so) throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks == null) {
            return null;
        }

		for (int i=0; i<tasks.length; i++) {
            if ( tasks[i].getClass() == theClass && tasks[i].getTargetStellarObject() == so) {
                return tasks[i];
            }
		}
		return null;
	}

    public String getDescription() throws ApplicationError {
        CraftTask[] tasks = getTasks();
        if (tasks == null) {
            return "None";
        }

        StringBuffer sb = new StringBuffer();
		for (int i=0; i<tasks.length; i++) {
            sb.append("(");
            sb.append(tasks[i].getDescription());
            sb.append(") ");
        }
        return sb.toString();
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
        CraftTask[] tasks = getTasks();
        if (tasks != null) {
            for (int i=0; i<tasks.length; i++) {
                tasks[i].toXML(writer);    
            }
        }
		writer.addElement("qtyCraftsInAttackFleet", qtyCraftsInAttackFleet);
	}


    public void setQtyCraftsInAttackFleet(int qtyCraftsInAttackFleet) {
        this.qtyCraftsInAttackFleet = qtyCraftsInAttackFleet;
    }


    public int getQtyCraftsInAttackFleet() {
        return qtyCraftsInAttackFleet;
    }

    public void setCraftTaskExplore(CraftTaskExplore task) throws ApplicationError { _add(task); }
    public void setCraftTaskAttack(CraftTaskAttack task) throws ApplicationError { _add(task); }
    public void setCraftTaskColonise(CraftTaskColonise task) throws ApplicationError { _add(task); }
    public void setCraftTaskDefend(CraftTaskDefend task) throws ApplicationError { _add(task); }
    public void setCraftTaskIdle(CraftTaskIdle task) throws ApplicationError { _add(task); }
    public void setCraftTaskProtect(CraftTaskProtect task) throws ApplicationError { _add(task); }
    public void setCraftTaskSteelColony(CraftTaskSteelColony task) throws ApplicationError { _add(task); }
    public void setCraftTaskMaraud(CraftTaskMaraud task) throws ApplicationError { _add(task); }
    public void setCraftTaskMaraudCraft(CraftTaskMaraudCraft task) throws ApplicationError { _add(task); }
    public void setCraftTaskBombard(CraftTaskBombard task) throws ApplicationError { _add(task); }
    public void setCraftTaskSurpriseAttackStealColony(CraftTaskSurpriseAttackStealColony task) throws ApplicationError { _add(task); }
    
	public void removeAllTasks() throws ApplicationError {
		CraftTask[] tasks = getTasks();
		if (tasks != null) {
			for (int i = 0; i < tasks.length; i++) {				
				if (tasks[i] != null) {
                    remove(tasks[i]);
				}
			}
		}
	}


}