/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.bean.Location;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.relations.Relations;
import mot.engine.transports.Transport;

/**
 * Generals manage the priorities for craft belonging to a computer player. It examines the
 * players position and assigns priorities to different tasks that need performing, then
 * crafts can be added to tasks and reduce their priority.
 *
 * Generals do not manage crafts when they are in battle - this is done by Commanders.
 */
public class GeneralBasic extends General {

	public GeneralBasic() {
	}

	public GeneralBasic(Player owner) {
		super(owner);
	}
    
    private static final int INTERVAL = 10;
    private static transient int count = 0;
    
    /**
     * This implementation looks for the following situations and creates tasks to handle 
     * them.
     *  - Enemy fleets destined or at one of our planets.
     */
    protected void calculatePriorities() throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("GeneralBasic.calculatePriorities() for "+getOwner().getName());
        }
        
        createMilitaryTask(false, null);
        
        if (count <= 0) {
            count = INTERVAL;
            assesTaskPositions();
        } else {
            count--;
        }
    }
    
    /**
     * This implementation :
     *  - Creates an attack task if we are at war with any player and do not already 
     *    have an attack task which is not ready for dispatch.
     *  - Creates a defend task for each player we are at war with and has sent crafts
     *    to one of our solar systems.
     *  - Creates a idle task if one does not already exist.
     * @param craft The craft to create a suitable task for.
     * @return This method creates a task suitable for the craft passed in (if any exist).
     */
    public CraftTask createTaskFor(Craft craft) throws ApplicationError {
        Player owner = getOwner();
		if (Logger.isDebug()) {
            Logger.logDebug("GeneralBasic.createTaskFor() for "+craft.getName());
        }
        if (craft.canColonise()) {
            return createColonyTaskFor(craft);
        } else if (craft.containsWeapons()) {
            return createMilitaryTask(true, craft);
        } else if (craft.containsMarines()) {
            return createIdleTask(true);
        } else {
            return createExplorerTask();
        }
    }
    
    private CraftTask createMilitaryTask(boolean createIdle, Craft craft) throws ApplicationError {
        Player owner = getOwner();
        
        // For each player we are at war with, check to see if they have a fleet or crafts
        // heading for my planets (or at my planets)
        StellarObject [] sos = getOwner().getStellarObjects();
        for (int j=0; j<sos.length; j++) {
            int qtyEnemy = owner.getQtyEnemyCraftMovingTo(sos[j]);
            if (qtyEnemy > 0) {
                CraftTask task = getTaskType(CraftTaskDefend.class, sos[j]);
                if (task == null) {
                    task = add( new CraftTaskDefend(sos[j], owner) );
                    return task;
                } else if (craft != null && task.requires(craft)) {
                    return task;
                }
            }
        }
        
        if (craft == null) {
            return null;
        }
            
		Relations relations = MotEngine.instance.getRelations();
		Player [] enemies = relations.getPlayersAtWarWith(owner);
		if (enemies != null) {
            if (enemies.length > 0) {
                CraftTask existingAttackTask = getTaskType(CraftTaskAttack.class);
				if (existingAttackTask == null) {
	                // Of all the players I am at war with, choose a random planet of a
	                // random player to attack.
	                int pw = (enemies.length == 1) ? 0 : Utils.random(enemies.length);
	                Player enemy = enemies[pw];
	                BlackHole blackHole = MotEngine.instance.getStellarObjects().getWormHoleWithinRangeOf(enemy, owner);
	                if (blackHole == null) blackHole = MotEngine.instance.getStellarObjects().getBlackHoleWithinRangeOf(enemy, owner);
	                if (blackHole != null && Utils.random(0.5)) {
		        		CraftTask task = getTaskType(CraftTaskAttack.class, blackHole);
		        		if (task == null) {
		        			Planet route = getAttackRouteFor(enemy, owner, blackHole);
		        			task = add(new CraftTaskAttack(blackHole, owner, getQtyCraftsInAttackFleet(), route));
		        		}
		        		return task;
	                }
	                Planet planet = MotEngine.instance.getStellarObjects().getRandomPlanetWithinRangeOf(enemy, owner);
		        	if (planet != null) {
		        		CraftTask task = getTaskType(CraftTaskAttack.class, planet.getStellarObject());
		        		if (task == null) {
		        			Planet route = getAttackRouteFor(enemy, owner, planet.getSolarSystem());
		        			task = add(new CraftTaskAttack(planet, owner, getQtyCraftsInAttackFleet(), route));
		        		}
		        		return task;
		        	}
                } else {
                	return existingAttackTask;
                }
            }
		}

        return createIdleTask(createIdle);
    }


	private Planet getAttackRouteFor(Player enemy, Player owner, StellarObject target) throws ApplicationError {
		if (MotEngine.instance.isAHardLevel() && Utils.random(0.5)) {
			Planet r = MotEngine.instance.getStellarObjects().getClosestStellarObjectWithinRangeOf(enemy, owner, target);
			return r;
		}
		return null;
	}

	private CraftTask createColonyTaskFor(Craft craft) throws ApplicationError {
        Player owner = getOwner();
		Planet best = getBestPlanetCanColonise(craft);
		if (best != null) {
            Craft [] enemies = craft.getOwner().getEnemyCraftAt(best.getStellarObject(), false);
            if (enemies != null) {
                // This planet has enemy craft protecting it - we need to notify the general that we
                // need an invasion force. This colony craft will be added to the invasion fleet so
                // it will not ask for orders every turn!
                CraftTask task = getTaskType(CraftTaskSteelColony.class, best.getStellarObject());
                if (task == null) {
                    task = add( new CraftTaskSteelColony(best, getOwner()) );
                }
                return task;
            } else {
                CraftTask task = getTaskType(CraftTaskColonise.class, best.getStellarObject());
                if (task == null) {
                    task = add( new CraftTaskColonise(best, getOwner()) );
                }
                return task;
            }
		} else {
            Planet p = owner.getClosestPlanetCanColonise();
            if (p != null && !craft.canMoveTo(p.getSolarSystem())) {
                Location loc = p.getSolarSystem().getLocation();
                StellarObject so = owner.getClosestStellarObject(loc);
                if (so != null && owner instanceof ComputerPlayer) {
                    ((ComputerPlayer)owner).getScientist().notifyNeedEngineForDistance(owner, so.getDistanceTo(loc));
                }
                // We may be able to build an outpost that would enable us to colonise this planet
                // Search for another planet what is closer to this planet AND would extend our range.
                if (outpostCache == null || outpostCache.invalidOutpostTarget()) {
                    StellarObject stellarObject = getBestStellarObjectForOutpost(p);
                    if (stellarObject != null) outpostCache = new OutpostStellarObjectCache(stellarObject);
                }
            }
        }

        return createIdleTask(true);
	}
    
    private OutpostStellarObjectCache outpostCache;
	public StellarObject getBestStellarObjectToDeployOutpost() throws ApplicationError {
        if (outpostCache == null || outpostCache.invalidOutpostTarget()) {
            return getBestStellarObjectForOutpost();
        } else {
            return outpostCache.stellarObject;
        }
    }

    private static final float DEFEND_FACTOR = 2.5f;

	/**
     * The best planet is the planet with the highest planet quality score.
     * For details on planet score see Planet.
     * Defence score is the total weapons damage of the enemies in orbit around the solar system -
     * the total weapons damage available to this player.
     * This method ignores planets if the defence score is deemed so high that we cannot build enough ships to attack.
     * @return The best Planet this ship can colonise.
     */
	public Planet getBestPlanetCanColonise(Craft craft) throws ApplicationError {
        Planet [] planets = MotEngine.getInstance().getPlanets().getPlanets();
        Player owner = getOwner();
        
        Planet best = null;
        int bestScore = 0;
        int myDamage = -1;
        for (int i=0; i<planets.length; i++) {
            SolarSystem ss = planets[i].getSolarSystem();
    		if (craft.canColonise(planets[i]) && craft.canMoveTo(ss) && !ss.isPartOfTutorial()) {
                if (!ss.hasOwner() || ss.isOwnedBy(owner)) {
                    int score = planets[i].getQualityScore();
                    if (score > bestScore && planets[i].getOwner() == null) {
                        Craft [] enemies = owner.getEnemyCraftAt(ss, false);
                        if (enemies != null && enemies.length > 0) {
                            int enemyDamage = CraftTaskSteelColony.getTotalWeaponsDamage(enemies);
                            if (myDamage == -1) {
                                myDamage = CraftTaskSteelColony.getTotalWeaponsDamage(owner.getCrafts());
                            }
                            int qtyCraft = owner.getQtyCrafts();
                            if (myDamage < enemyDamage) {
                                score = -1;                                
                            }
                        }
                        if (score > bestScore) {
                            bestScore = score;
                            best = planets[i]; 
                        }
                        //Logger.logDebug("Planet "+planets[i].getName()+" score = "+score);
                    }
                }
            }
        }
        return best;
	}

    /**
     * @return The planet that would be best suited to build an outpost on to extend this
     * players range to colonise the closest planet.
     */
    public StellarObject getBestStellarObjectForOutpost() throws ApplicationError {
        return getBestStellarObjectForOutpost(getOwner().getClosestPlanetCanColonise());
    }
    
    /**
     * Return a planet that would make planet p in range if an outpost was build on it.
     * @return The stellar object that is farthest away from an existing colony but in
     * range of an existing colony and the planet p passed in.
     */
    public StellarObject getBestStellarObjectForOutpost(Planet p) throws ApplicationError {
        if (p == null) return getAnyStellarObjectForOutpost();
        StellarObject [] sos = MotEngine.getInstance().getStellarObjects().getStellarObjects();
        Player owner = getOwner();
        if (owner.getEngines() == null) return null;
        int maxTravelDistance = owner.getEngines().getMaxTravelDistance();
        SolarSystem target = p.getSolarSystem();

        StellarObject best = null;
        int bestDistance = 0;

        for (int i=0; i<sos.length; i++) {
            StellarObject so = sos[i];
            if (so.hasOwner()) continue;
            if (MotEngine.instance.getMonsterPlayer().getCraftsAt(so, true, true) != null) continue;
    		StellarObject closest = owner.getClosestStellarObject(so.getLocation());
            int distance1 = so.getLocation().getDistanceTo(closest.getLocation());
            int distance2 = so.getLocation().getDistanceTo(target.getLocation());
            if (distance1 <= maxTravelDistance && distance2 <= maxTravelDistance) {
                if (distance1 > bestDistance) {
                    bestDistance = distance1;
                    best = so;
                }
            }
        }
        
        if (best == null) {
            return getAnyStellarObjectForOutpost();
        } else {
            return best;
        }
    }
    
    /**
     * This should only be used if getBestStellarObjectForOutpost() has returned null.
     * @return A planet this player could build an outpost on (if one exists). 
     */
    public StellarObject getAnyStellarObjectForOutpost() throws ApplicationError {
        StellarObject [] sos = MotEngine.getInstance().getStellarObjects().getStellarObjects();
        Player owner = getOwner();
        if (owner.getEngines() == null) return null;
        int maxTravelDistance = owner.getEngines().getMaxTravelDistance();

        StellarObject best = null;
        int bestDistance = 0;

        for (int i=0; i<sos.length; i++) {
            StellarObject so = sos[i];
            if (so.hasOwner()) continue;
    		StellarObject closest = owner.getClosestStellarObject(so.getLocation());
            int distance1 = so.getLocation().getDistanceTo(closest.getLocation());
            if (distance1 <= maxTravelDistance) {
                if (distance1 > bestDistance) {
                    bestDistance = distance1;
                    best = so;
                }
            }
        }
        
        return best;
    }

    /**
     * @return True if the general cannot colonise any planets in range, but, can colonise
     * a planet out of range that would be in range if an outpost was deployed.
     */
    public boolean needOutpost() throws ApplicationError {
        return outpostCache != null && outpostCache.valid();
    }
    
	public void clearOutpostNeeded() {
		outpostCache = null;
	}

    public static final double MAX_TERRAFORM_TO_MINE = 0.6;

	/**
     * The best planet is the planet with the highest planet quality score.
     * For details on planet score see Planet.
     * @return The best Planet this transport can deploy a space station at.
     */
	public Planet getBestPlanetCanDeployMiningStation(Transport transport) throws ApplicationError {
        Planet [] planets = MotEngine.getInstance().getPlanets().getPlanets();
        Player owner = getOwner();
        
        Planet best = null;
        int bestScore = 0;
        long myDamage = -1;
        for (int i=0; i<planets.length; i++) {
            Planet p = planets[i];
            if (canDeployMiningStation(p, true)) {
                if (transport.canMoveTo(p.getSolarSystem())) {
                    int score = p.getMiningScore();
                    if (best == null || score > bestScore) {
                        bestScore = score;
                        best = p; 
                    }
                }
            }
        }
        return best;
	}

    /**
     * The general does not consider mining planets that are 
     *      owned - because it can't
     *      Tarraform level > MAX_TERRAFORM_TO_MINE - because they would make better colonies
     * @return Return true if the general would consider mining this planet. 
     */
	public boolean canDeployMiningStation(Planet p, boolean rightNow) throws ApplicationError {
        SolarSystem ss = p.getSolarSystem();
        Player planetOwner = p.getOwner();
        Player owner = getOwner();
        if (p.getTeraforming() < MAX_TERRAFORM_TO_MINE && planetOwner == null) {
            if (!ss.hasOwner() || ss.isOwnedBy(owner)) {
            	if (rightNow) {
	        		if (owner.getQtyEnemyCraftAt(ss) == 0) {
	        			return true;
	        		}
            	} else {
            		MonsterPlayer monsterPlayer = MotEngine.instance.getMonsterPlayer();
            		if (monsterPlayer.getQtyCraftsAt(p.getStellarObject()) == 0) {
            			return true;
            		}
            	}
            }
        }
        return false;
    }

	public boolean canDeployMiningStation(BlackHole blackHole) throws ApplicationError {
        if (blackHole.getOwners() == null) {
        	Player owner = getOwner();
            if (owner.getQtyEnemyCraftAt(blackHole) == 0) {
                return true;
            }
        }
        return false;
    }
	
    private CraftTask createIdleTask(boolean create) throws ApplicationError {
        Player owner = getOwner();
        CraftTask idleTask = getTaskType(CraftTaskIdle.class);
        if (create && idleTask == null) {
            idleTask = new CraftTaskIdle(owner, 9);
            add( idleTask );
        }
        return idleTask;
    }

    private CraftTask createExplorerTask() throws ApplicationError {
        int range = 0;
        if (getOwner().getEngines() != null) {
            range = getOwner().getEngines().getMaxTravelDistance();
        }
        CraftTask task = new CraftTaskExplore(getOwner(), range);
        if (!task.complete()) {
            return add(task);
        } else {
            return createIdleTask(true);
        }
    }

	/**
	 * Called when another craft has arrived at an enemies solar system. It must return
     * which planet to attack in the solar system.
     * It returns the planet with the highest population.
	 */
    public Planet getPlanetToAttack(Planet [] planets) throws ApplicationError {
        Planet best = null;
        double pop = 0;
        int i;
        for (i=0; i<planets.length; i++) {
            if (best == null || planets[i].getPopulation() > pop) {
                best = planets[i];
                pop = planets[i].getPopulation();
            }
        }
		if (Logger.isDebug()) {
            Logger.logDebug("GeneralBasic.getPlanetToAttack() player="+getOwner().getName()+" => "+((best!=null)?best.getName():"None"));
        }
        return best;
    }
    

	protected String getElementName() { return "general-basic"; }

	@Override
	public void convertAttackToBombardAt(Planet planet) throws ApplicationError {
		CraftTaskAttack attackTask = (CraftTaskAttack) getTaskType(CraftTaskAttack.class, planet.getStellarObject());
		if (attackTask != null) {
			CraftTaskBombard newTask = new CraftTaskBombard(getOwner(), planet, attackTask);
			add(newTask);
		}
	}
}


class OutpostStellarObjectCache {
    public OutpostStellarObjectCache(StellarObject stellarObject) {
        this.stellarObject = stellarObject;
        this.date = MotEngine.instance.getDate();
    }
    
    public StellarObject stellarObject;
    public int date;
    
    private static final int MAX_AGE = 25;
    
    public boolean isOutOfDate() {
        int age = (int) (MotEngine.instance.getDate() - date);
        return age > MAX_AGE;
    }

    public boolean invalidOutpostTarget() throws ApplicationError {
        return isOutOfDate() || stellarObject == null || stellarObject.hasOwner();
    }

    public boolean valid() throws ApplicationError {
        return stellarObject != null && !stellarObject.hasOwner();
    }
}