/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.generals;

import mot.common.util.ApplicationError;
import mot.common.xml.XMLWriter;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.players.Player;
import mot.engine.transports.Transport;

/**
 * Generals manage the priorities for craft belonging to a computer player. It examines the
 * players position and assigns priorities to different tasks that need performing, then
 * crafts can be added to tasks and reduce their priority.
 *
 * Generals do not manage crafts when they are in battle - this is done by Commanders.
 */
public class GeneralMonster extends General {

	public GeneralMonster() {
	}

	public GeneralMonster(Player owner) {
		super(owner);
	}

    /**
     * This method creates a task suitable for the craft passed in (if any exist).
     * @param craft The craft to create a suitable task for.
     * @return A new Task or null.
     */
    public CraftTask createTaskFor(Craft craft) throws ApplicationError {
        StellarObject so = craft.getOrbitStellarObject();
        CraftTaskProtect task = (CraftTaskProtect) getTaskType(CraftTaskProtect.class, so);;
        if (task != null) {
        	task.add(craft);
        	return task;
        } else {        	
        	task = new CraftTaskProtect(so, getOwner(), craft);
        	return add(task);
        }
	}

    /**
     * Creates a new craft task of type maraud.
     * @param fleet The fleet to be controller by this task.
     * @param target The player to be pillaged (marauded) by this task.
     * @return The new maraud task controlling the fleet.
     * @throws mot.common.util.ApplicationError
     */
    public CraftTaskMaraud createMaraudTaskFor(Fleet fleet, Player target, int maxQtyPlanetsPillaged) throws ApplicationError {
        CraftTaskMaraud task = new CraftTaskMaraud(getOwner(), target, fleet, maxQtyPlanetsPillaged);
		return (CraftTaskMaraud) add(task);
	}

    /**
     * Creates a new craft task of type maraud.
     * @param fleet The fleet to be controller by this task.
     * @param moveToDest The player to be pillaged (marauded) by this task.
     * @return The new maraud task controlling the fleet.
     * @throws mot.common.util.ApplicationError
     */
    public CraftTaskMaraudCraft createMaraudCraftTaskFor(Craft craft, Player initialTarget) throws ApplicationError {
    	CraftTaskMaraudCraft task = new CraftTaskMaraudCraft(getOwner(), craft, initialTarget);
		return (CraftTaskMaraudCraft) add(task);
	}

	/**
	 * Called when another a craft has arrived at an enemies solar system. It must return
     * which planet to attack in the solar system.
     * It returns the planet with the highest population.
	 */
    public Planet getPlanetToAttack(Planet [] planets) {
        return null;
    }

    /**
     * This implementation looks for the following situations and creates tasks to handle 
     * them.
     *  - Enemy fleets destined or at one of our planets.
     */
    protected void calculatePriorities() throws ApplicationError {
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
        super.encode(writer);
	}

	protected String getElementName() { return "general-monster"; }

    public Planet getBestPlanetCanDeployMiningStation(Transport transport) throws ApplicationError {
        return null;
    }

	public boolean canDeployMiningStation(Planet p, boolean rightNow) throws ApplicationError {
        return false;
    }

	public StellarObject getBestStellarObjectToDeployOutpost() throws ApplicationError {
        return null;
    }

    public boolean needOutpost() throws ApplicationError {
        return false;
    }

	public void clearOutpostNeeded() {		
	}

	public void convertAttackToBombardAt(Planet planet) throws ApplicationError {
	}

	public boolean canDeployMiningStation(BlackHole blackHole) throws ApplicationError {
		return false;
	}
}