/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.mayor;

import mot.common.designs.CraftDesign;
import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.players.Player;

/**
 * Use a colony craft designer to create craft designs to colonise a planet from 
 * the technology a player has discovered.
 */
public class CraftDesignerColony extends CraftDesigner {
    public CraftDesignerColony() {
    }

    /**
     * Uses the technology discovered by player p to design a new craft.
     * It creates the smallest craft hull is can use to fit the best colony module
     * with the best engines.
     * @param p Player for which it will design a new craft. All technology on 
     * the craft will restructed to techs researched by this player.
     * @return A new Craft Design.
     * @throws ApplicationError 
     */
    public CraftDesign designCraft(Player p) throws ApplicationError {
        Special best = p.getSpecials().getBestCanColonise();
        if (best == null) {
            Logger.logWarning("CraftDesignerColony.designCraft() player "+p.getName()+" does not have any colony module specials");
            return null;
        }
        
        Engine engine = p.getEngines().getLongestRangeEngine();
        Hull hull = p.getHulls().getSmallestCraftHullWithSpaceFor(engine, best);
        if (hull == null) {
            Logger.logWarning("CraftDesignerColony.designCraft() player "+p.getName()+" cannot find hull to fit special "+best.getName());
            return null;
        }
        
        CraftDesign cd = new CraftDesign(getNewName("Coloniser")+"_");
        cd.setHull(hull);
        cd.setEngine(engine);
        cd.setSpecial(best);
        
        int remainingSpace = cd.getRemainingSpace();
        
        if (remainingSpace > 0 && p.getShields() != null) {
            Shield s = p.getShields().getBestShield();
            if (s != null && s.getSpace() < remainingSpace) {
                cd.setShields(new Shields(s));
            }
        }
        
        return cd;
    }
}