/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.mayor;

import mot.common.designs.CraftDesign;
import mot.common.designs.WeaponBaysDesign;
import mot.common.technology.Hull;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.engine.players.Player;

/**
 * Use a military craft designer to create craft designs to invade an enemy craft during battle using a troop transport.
 */
public class CraftDesignerDamageBot extends CraftDesignerSpecial {

    private static final int QTY_TURNS_TO_BUILD_HULL = 20;

    public CraftDesignerDamageBot() {
    }

    public static boolean canBuild(Player p) {
    	Weapons playerWeapons = p.getWeapons();
		return playerWeapons.containsGroup(Weapon.GROUP_DAMAGE_BOT) && playerWeapons.containsGroup(Weapon.GROUP_SHIELD_DRAIN);
	}
        
    /**
     * Uses the technology discovered by player p to design a new craft.
     * @param p Player for which it will design a new craft. All technology on 
     * the craft will restricted to techs researched by this player.
     * @return A new Craft Design.
     */
    public CraftDesign designCraft(Player p) throws ApplicationError {
    	
		boolean canBuild = canBuild(p);
		if (!canBuild) {
			return null;
		}
		
        CraftDesign cd = new CraftDesign(getNewName("BotFighter")+"_");
               
        Hull hull = getBigHull(p);
        cd.setHull(hull);
        
        addFastestEngine(p, cd);
        addMostPowerfulComputer(p, cd);
        addMostPowerfulScanner(p, cd);
        addSpacePodSpecials(p, cd);


        // Add minimum equipment to make this craft viable
    	Weapons playerWeapons = p.getWeapons();
        Weapon techDamageBot = playerWeapons.getMaxDamageWeaponWithGroup(Weapon.GROUP_DAMAGE_BOT);
        Weapon techShieldDrain = playerWeapons.getMaxDamageWeaponWithGroup(Weapon.GROUP_SHIELD_DRAIN);
        
        Shield cloak = null;
        Shields shields = p.getShields();
        if (shields != null) {
        	cloak = shields.getBestCloakShield();
        }

		if (techDamageBot == null || techShieldDrain == null) {
			return null;
		}
		
        WeaponBaysDesign weaponBays = new WeaponBaysDesign();
        cd.setWeaponBays(weaponBays);
        
        weaponBays.addWeapon(techShieldDrain, 0);
        weaponBays.addWeapon(techDamageBot, 0);
        
        int remainingSpace = cd.getRemainingSpace();
        
        if (cloak != null && cloak.getSpace() <= remainingSpace) {
        	cd.add(cloak);
        	remainingSpace = cd.getRemainingSpace();
        }
        
        while (remainingSpace > techDamageBot.getSpace()) {
            weaponBays.addWeapon(techDamageBot, 0);
            remainingSpace = cd.getRemainingSpace();
            if (remainingSpace > techDamageBot.getSpace()) {            	
            	weaponBays.addWeapon(techDamageBot, 0);
                remainingSpace = cd.getRemainingSpace();
            }
            if (remainingSpace > techShieldDrain.getSpace()) {            	
            	weaponBays.addWeapon(techShieldDrain, 0);
                remainingSpace = cd.getRemainingSpace();
            }
        }

        return cd;
    }

	public int getQtyCraftOfThisType(Player player) throws ApplicationError {
		return player.getQtyCraftsContainingWeaponGroup(Weapon.GROUP_DAMAGE_BOT);
		//return player.getQtyDamageBotCrafts();
	}



}