/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.mayor;

import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.planets.Planet;
import mot.engine.players.Player;
import mot.engine.players.PlayerCharacter;
import mot.engine.players.Script;

/**
 * Uses the default or player specific list of Craft Designs to create craft designs for
 * Computer Players. (Currently only a default list of crafts designs is implemented,
 * see craft_designes.xml).
 * The craft designs must be in order of desirability, least desirable to most desirable. 
 * It searches the list of craft designs and returns the most desirable craft this player
 * can build.
 * When  generating a craft design all engines, shields and scanners are removed from the 
 * design and replaces with the best available. Note - This means a craft design is really
 * just a configuration of weapons, specials and hull.
 * 
 * NOTE - should place type in craft design for use by selection - e.g. coloniser, military, 
 * command etc
 */
public class CraftDesignerFactory extends Script {

	/**
	 * This is the most desirable colony craft this player can build.
	 */
	private transient CraftDesign colonyCraftDesign;

	/**
	 * This is the most desirable military craft this player can build.
	 */
	private transient CraftDesign militaryCraftDesign;

	/**
	 * This is the most desirable special craft this player can build.
	 * What each player trues to build:
	 * Bears = Invader (designed to suck shield power and board marines onto enemy craft using troop transport)
	 * Ghosts = Psi (designed to make crew of enemy craft surrender)
	 * Shifters = Plaguer (designed to kill crew of enemy craft)
	 * Machines = Damage Bots (destroy crafts using robots)
	 * Birds = Droid Miner (use speed to send small crafts with droid mines to attack)
	 * Humans = No Special Crafts
	 */
	private transient CraftDesign specialCraftDesign;
	private transient CraftDesignerSpecial specialCraftDesigner;
	
	/**
	 * This is the last year this priority was calculated.
	 */
	private long yearCalculated;

	/**
	 * This is the quantity of years priorities may exist without being re-calculated.
     * This provides a kind of caching mechanism so lost of work is not required every
     * time a new CraftDesign is requested.
	 */
	public static final int CALCULATE_INTERVAL = 20;

	public CraftDesignerFactory() {
	}

	public CraftDesignerFactory(String name) {
		super(name);
	}

    public void flushCraftDesigns() {
        colonyCraftDesign = null;
        militaryCraftDesign = null;
    }

    public void flushColonyCraftDesigns() {
        colonyCraftDesign = null;
    }

	public CraftDesign [] getCraftDesigns() throws ApplicationError {
        return CraftDesigns.getInstance().getCraftDesigns();
	}

	/**
	 * Moves maxIndexPlayerCanBuild to the most advanced craft this player can build.
	 */
	private void advanceMilitaryCraftDesign(Player player) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("CraftDesignerFactory.advanceMilitaryCraftDesign() for player "+player.getName());
        }
        colonyCraftDesign = new CraftDesignerColony().designCraft(player);
        militaryCraftDesign = new CraftDesignerMilitary().designCraft(player, CraftDesignerMilitary.SIZE_MEDIUM, 0.2, 0.0);
        
        PlayerCharacter pc = player.getPlayerCharacter();
		if (pc.designesInvaderCraft()) {
			specialCraftDesigner = new CraftDesignerInvader();
			specialCraftDesign = specialCraftDesigner.designCraft(player);
		} else if (pc.designesPsiCraft()) {
			specialCraftDesigner = new CraftDesignerPsi();
			specialCraftDesign = specialCraftDesigner.designCraft(player);
		} else if (pc.designesPlaguerCraft()) {
			specialCraftDesigner = new CraftDesignerPlague();
			specialCraftDesign = specialCraftDesigner.designCraft(player);
		} else if (pc.designesDamageBotCraft()) {
			specialCraftDesigner = new CraftDesignerDamageBot();
			specialCraftDesign = specialCraftDesigner.designCraft(player);
		} else if (pc.designesDroidMinerCraft()) {
			specialCraftDesigner = new CraftDesignerDroidMine();
			specialCraftDesign = specialCraftDesigner.designCraft(player);
		}
        
        yearCalculated = MotEngine.getInstance().getDate();
        
        if (Logger.isDebug()) {
            if (colonyCraftDesign != null) {
                colonyCraftDesign.log(player.getName());
            } else {
                Logger.logDebug("CraftDesignerFactory.advanceMilitaryCraftDesign() could not design colony craft for player "+player.getName());
            }
            if (militaryCraftDesign != null) {
                militaryCraftDesign.log(player.getName());
            } else {
                Logger.logDebug("CraftDesignerFactory.advanceMilitaryCraftDesign() could not design military craft for player "+player.getName());
            }
        }
	}

	/**
	 * Returns the best colony craft that this player can build or NULL if no such ship exists.
	 */
	public CraftDesign getNextColonyCraftToBuild(Player player) throws ApplicationError {
        return getNextColonyCraftToBuild(null, player);
    }
    
	/**
	 * Returns a colony craft that this player can build to colonise
	 * this planet or NULL if no such ship exists.
	 */
	public CraftDesign getNextColonyCraftToBuild(Planet planet, Player player) throws ApplicationError {
		if (colonyCraftDesign == null || MotEngine.getInstance().getDate() - yearCalculated >= CALCULATE_INTERVAL) {
			advanceMilitaryCraftDesign(player);
		}

        if (planet == null || colonyCraftDesign.canColonise(planet.getTeraforming())) {
            return colonyCraftDesign;
        } else {
            return null;
        }
	}

	public static final int INVADER_RATIO_WITH_ASSAULT_BONUS = 3;
	public static final int INVADER_RATIO = 6;
	
	/**
	 * Returns the most advanced military craft that this player can build.
	 */
	public CraftDesign getNextMilitaryCraftToBuild(Player player) throws ApplicationError {
        int qtyCrafts = player.getQtyCrafts();
        int qtyColony = player.getQtyColonyCrafts();
        int qtyMilitary = qtyCrafts - qtyColony;

        if (militaryCraftDesign == null || MotEngine.getInstance().getDate() - yearCalculated >= CALCULATE_INTERVAL) {
        	advanceMilitaryCraftDesign(player);
        }
        
        if (specialCraftDesigner != null && specialCraftDesign != null) {
			double specialCraftRatio = specialCraftDesigner.getRatio(player);
	        if (specialCraftRatio > 0.0) {
	        	int qtySpecial = specialCraftDesigner.getQtyCraftOfThisType(player);
	        	if (qtySpecial < Comp.floor(qtyMilitary * specialCraftRatio) ) {
	        		return specialCraftDesign;
	        	}
	        }
        }
        	
       	return militaryCraftDesign;
	}

	public CraftDesignerSpecial getSpecialCraftDesigner() {
		return specialCraftDesigner;
	}

	public CraftDesignerSpecial getAndUpdateSpecialCraftDesigner(Player player) throws ApplicationError {
		advanceMilitaryCraftDesign(player);
		return specialCraftDesigner;
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
	}

	protected String getElementName() { return "craft-script"; }

   
}