/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.mayor;

import mot.common.designs.CraftDesign;
import mot.common.designs.WeaponBaysDesign;
import mot.common.technology.Computer;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.players.Player;

/**
 * Use a military craft designer to create craft designs to defend, explore and attack
 * opponents.
 */
public class CraftDesignerFighter extends CraftDesigner {
    public CraftDesignerFighter() {
    }

    /**
     * It will choose the best hull of group Fighter discovered by this player.
     * @param p Player for which it will design a new craft. All technology on
     * the craft will restricted to techs researched by this player.
     * @return A new Craft Design.
     */
    public CraftDesign designCraft(Player p) throws ApplicationError {
        Weapon weapon = p.getWeapons().getMaxDamageAttackWeaponWithInfinateShots();
        return _designCraft(p, weapon, 2);
    }
    
    /**
     * It will choose the best hull of group Fighter discovered by this player.
     * @param p Player for which it will design a new craft. All technology on
     * the craft will restricted to techs researched by this player.
     * @return A new Craft Design.
     */
    public CraftDesign designSuicideCraft(Player p) throws ApplicationError {
        Weapon weapon = (Weapon) p.getWeapons().getTechnologyForGroup(Weapon.GROUP_DETONATOR);
        return _designCraft(p, weapon, 1);
    }
    
    private CraftDesign _designCraft(Player p, Weapon weapon, int qty) throws ApplicationError {

        if (p.getHulls() == null || p.getHulls().getQuantity() == 0) {
            Logger.logWarning("CraftDesignerFighter.designCraft() player "+p.getName()+" has not got any hull");
            return null;
        }
        
        Hull hull = p.getHulls().getBestFighterHull();

        if (hull == null) {
            Logger.logWarning("CraftDesignerFighter.designCraft() *WARNING* player "+p.getName()+" cannot find hull - USING BEST HULL");
            return null;
        }

        // Create a new craft design
        CraftDesign cd = new CraftDesign("BattleFighter");
        cd.setHull(hull);

        // Always use our best engines (what ever they are).
        // but for military choose the fastest.
        if (p.getEngines() == null) {
            Logger.logWarning("MilitaryCraftDesign.designCraft() Player "+p.getName()+" does not have any engines - cannot build craft design");
            return null;
        }
        
        Engine engine = p.getEngines().getFastestEngine();
        cd.setEngine(engine);

        // Always choose our best computer
        if (p.getComputers() != null) {
            Computer computer = p.getComputers().getMostPowerfulComputer();
            cd.setComputers(new Computers(computer));
        }

        // Now finally lets add our weapons!
        Weapons weapons = p.getWeapons();
        if (weapons == null || weapons.getQuantity() == 0) {
            Logger.logWarning("MilitaryCraftDesign.designCraft()  player  "+p.getName()+" does not have any weapons");
            return null;
        }

        WeaponBaysDesign weaponBays = new WeaponBaysDesign();
        cd.setWeaponBays(weaponBays);

        while (qty > 0) {
        	weaponBays.addWeapon(weapon, 0);
        	qty--;
        }
        
        return cd;
    }

    
}