/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.mayor;

import mot.common.designs.CraftDesign;
import mot.common.designs.WeaponBaysDesign;
import mot.common.technology.Computer;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * Use a military craft designer to create craft designs to defend, explore and attack 
 * opponents.
 */
public class CraftDesignerMilitary extends CraftDesigner {
    public CraftDesignerMilitary() {
    }
    
    private static final int QTY_TURNS_XLARGE = 100;
    private static final int QTY_TURNS_LARGE = 50;
    private static final int QTY_TURNS_MEDIUM = 25;
    private static final int QTY_TURNS_SMALL = 10;

    public static final int SIZE_LARGE = 0;
    public static final int SIZE_MEDIUM = 1;
    public static final int SIZE_SMALL = 2;
    public static final int SIZE_XLARGE = 3;

    private static final double MIN_WEAPON_RATION = 0.25;
    
    public CraftDesign designCraft(Player p) throws ApplicationError {
        return designCraft(p, SIZE_MEDIUM, 0.0, 0.0);
    }
    
    /**
     * Uses the technology discovered by player p to design a new craft.
     * It will use the hull that can be built within QTY_TURNS_<size> on the
     * home planet of player p.
     * @param p Player for which it will design a new craft. All technology on 
     * the craft will restricted to techs researched by this player.
     * @param size The size of the craft to be built. Should have value SIZE_LARGE,
     * SIZE_MEDIUM or SIZE_SMALL.
     * @param percentMissiles The percentage of the space to be allocated for weapons 
     * that should be use for missiles (or the weapon with the most damage but limited
     * shots).
     * @param percentSpecialWeapons The percentage of the space to be allocated for weapons 
     * that should be use for special weapons. Special weapons are defined as those weapons
     * with a group in the special weapon list list from the player character.
     * @return A new Craft Design.
     */
    public CraftDesign designCraft(Player p, int size, double percentMissiles, double percentSpecialWeapons) throws ApplicationError {
    
        // First determine the best HULL to use based upon how
        // long it would take to build the hull at the players home
        // planet.
        int qtyTurnsToBuildHull = QTY_TURNS_MEDIUM;
        switch (size) {
            case SIZE_LARGE :
                qtyTurnsToBuildHull = QTY_TURNS_LARGE;
                break;
            case SIZE_MEDIUM :
                qtyTurnsToBuildHull = QTY_TURNS_MEDIUM;
                break;
            case SIZE_SMALL :
                qtyTurnsToBuildHull = QTY_TURNS_SMALL;
                break;
            case SIZE_XLARGE :
                qtyTurnsToBuildHull = QTY_TURNS_XLARGE;
                break;
            default:
                Logger.logWarning("CraftDesignerMilitary.designCraft() Invalid size "+size+" using default");
        }
    
        Planet planet = p.getPlanetWithHighestProduction();
        long maxHullCost;
        if (planet == null) {
            Logger.logWarning("CraftDesignerMilitary.designCraft() Player "+p.getName()+" has no planets - I will use default production");
            maxHullCost = 30 * qtyTurnsToBuildHull;
        } else {
            maxHullCost = planet.getProductionOutput(true) * qtyTurnsToBuildHull;
        }
        
        if (p.getHulls() == null || p.getHulls().getQuantity() == 0) {
            Logger.logWarning("CraftDesignerMilitary.designCraft() player "+p.getName()+" has not got any hull");
            return null;
        }
        Hull[] hulls = p.getHulls().getCraftHulls();

        // Find the hull with the highest cost but costs less to 
        // build than maxHullCost.
        Hull hull = null;
        int hullCost = 0;
        for (int i=0; i<hulls.length; i++) {
            int cost = hulls[i].getCostToBuild();
            if (cost <= maxHullCost && cost > hullCost) {
                hullCost = cost;
                hull = hulls[i];
            }
        }

        if (hull == null) {
            Logger.logWarning("CraftDesignerMilitary.designCraft() *WARNING* player "+p.getName()+" cannot find hull with desired built cost - USING BEST HULL");
            hull = p.getHulls().getBiggestCraftHull();
        }
        
        return designCraft(p, hull, size, percentMissiles, percentSpecialWeapons);
    }
    
    /**
     * Uses the technology discovered by player p to design a new craft.
     * It will use the hull that can be built within QTY_TURNS_<size> on the
     * home planet of player p.
     * @param p Player for which it will design a new craft. All technology on 
     * the craft will restricted to techs researched by this player.
     * @param hull No value implies use size to determine the hull. If a hull is specified then this hull
     * must be used.
     * @param percentMissiles The percentage of the space to be allocated for weapons 
     * that should be use for missiles (or the weapon with the most damage but limited
     * shots).
     * @param percentSpecialWeapons The percentage of the space to be allocated for weapons 
     * that should be use for special weapons. Special weapons are defined as those weapons
     * with a group in the special weapon list list from the player character.
     * @return A new Craft Design.
     */
    public CraftDesign designCraft(Player p, Hull hull, int size, double percentMissiles, double percentSpecialWeapons) throws ApplicationError {
        // Create a new craft design
        CraftDesign cd = new CraftDesign(getNewName("Military")+"_");
        cd.setHull(hull);
        
        // Always use our best engines (what ever they are).
        // but for military choose the fastest.
        if (p.getEngines() == null) {
            Logger.logWarning("MilitaryCraftDesign.designCraft() Player "+p.getName()+" does not have any engines - cannot build craft design");
            return null;            
        }
        Engine engine = p.getEngines().getFastestEngine();
        cd.setEngine(engine);
        
        // Always choose our best computer
        if (p.getComputers() != null) {
            Computer computer = p.getComputers().getMostPowerfulComputer();
            cd.setComputers(new Computers(computer));
        }

        Special special = p.getSpecials().getBestSpacePod();
        Specials craftSpecials = new Specials();
        if (special != null) {
            craftSpecials.add(special);
            cd.setSpecials(craftSpecials);
        }

        int totalSpace = hull.getSpace();
        int remainingSpace = cd.getRemainingSpace();
        int weaponSpace = (int) (totalSpace*MIN_WEAPON_RATION);

        // Always choose our shields with the highest protection * power.
        // But we will reserve atleast MIN_WEAPON_RATION of space for weapons!
        if (p.getShields() != null) {
            Shield shield = p.getShields().getBestDefenseShield();
            if (shield != null) {
                int newRemainingSpace = remainingSpace - shield.getSpace();
                if (newRemainingSpace > weaponSpace) {
                    cd.setShield(shield);
                    remainingSpace = cd.getRemainingSpace();
                }
            }
        }

        // The player character decides if this player wants Psi Shields
        if (p.getShields() != null && p.getPlayerCharacter().getCraftsWantPsiShields()) {
            Shield shield = p.getShields().getBestPsiShield();
            double craftPsiDefense = 0.0;
            if (cd.getShields() != null) craftPsiDefense = cd.getShieldTechs().getPsiRating();
            if (shield != null && shield.getPsiRating() > craftPsiDefense) {
                int newRemainingSpace = remainingSpace - shield.getSpace();
                if (newRemainingSpace > weaponSpace) {
                	cd.setShield(shield);
                    remainingSpace = cd.getRemainingSpace();
                }
            }
        }

        // Always choose our best scanner
        // But we will reserve atleast MIN_WEAPON_RATION of space for weapons!
        if (p.getScanners() != null && size != SIZE_SMALL) {
            Scanner scanner = p.getScanners().getBestScanner();
            if (scanner != null) {
                int newRemainingSpace = remainingSpace - scanner.getSpace();
                if (newRemainingSpace > weaponSpace) {
                    cd.setScanners(new Scanners(scanner));
                    remainingSpace = cd.getRemainingSpace();
                }
            }
        }

        // Now choose the specials to place on the craft.
        // Use the players options select specials for this design.
        // But we will reserve atleast MIN_WEAPON_RATION of space for weapons!
        if (p.getSpecials() != null) {
            String [] specialNames = p.getPlayerCharacter().getCraftDeasignSpecials();
            for (int i=0; specialNames != null && i < specialNames.length; i++) {
                special = p.getSpecials().getSpecial(specialNames[i]);
                if (special == null) {
                    // Has not been discovered yet!
                    //Logger.logWarning("MilitaryCraftDesign.designCraft() Unable to find special "+specialNames[i]+" player options are probably invalid for player "+p.getName());
                } else if (craftSpecials.containsGroup(special.getGroup())) {
                    // Can't have it (must be a space pod and we always use the best anyway)
                } else {
                    int newRemainingSpace = remainingSpace - special.getSpace();
                    if (newRemainingSpace > weaponSpace) {
                        craftSpecials.add(special);
                        cd.setSpecials(craftSpecials);
                        remainingSpace = newRemainingSpace;
                    }
                }
            }
        }
        
        // NOW ADD OTHER SPECIALS IN PRIORITY ORDER TO THE CRAFT BUT MAKE SURE WE
        // PRESERVE THE MIN WEAPON SPACE
        
        special = p.getSpecials().getBestDefenderModification();
        if (special != null && special.getDefenderModification().doubleValue() > craftSpecials.getDefenderModification()) {
            // It better defender modification than we already have
            if (!craftSpecials.containsGroup(special.getGroup())) {
                if ((remainingSpace - special.getSpace()) > weaponSpace) {
                    craftSpecials.add(special);
                    cd.setSpecials(craftSpecials);
                    remainingSpace = cd.getRemainingSpace();
                }
            }
        }
        special = p.getSpecials().getBestSpeedBonus();
        if (special != null && special.getSpeedBonus().doubleValue() > craftSpecials.getSpeedBonus()) {
            // It better speed bonus than we already have
            if (!craftSpecials.containsGroup(special.getGroup())) {
                if ((remainingSpace - special.getSpace()) > weaponSpace) {
                    craftSpecials.add(special);
                    cd.setSpecials(craftSpecials);
                    remainingSpace = cd.getRemainingSpace();
                }
            }
        }
        special = p.getSpecials().getBestAttackerModification();
        if (special != null && special.getAttackerModification().doubleValue() > craftSpecials.getAttackerModification()) {
            // It better speed bonus than we already have
            if (!craftSpecials.containsGroup(special.getGroup())) {
                if ((remainingSpace - special.getSpace()) > weaponSpace) {
                    craftSpecials.add(special);
                    cd.setSpecials(craftSpecials);
                    remainingSpace = cd.getRemainingSpace();
                }
            }
        }
        special = p.getSpecials().getBestMaxDamageBonus();
        if (special != null && special.getMaxDamageBonus().doubleValue() > craftSpecials.getMaxDamageBonus()) {
            // It better speed bonus than we already have
            if (!craftSpecials.containsGroup(special.getGroup())) {
                if ((remainingSpace - special.getSpace()) > weaponSpace) {
                    craftSpecials.add(special);
                    cd.setSpecials(craftSpecials);
                    remainingSpace = cd.getRemainingSpace();
                }
            }
        }
        
        // Now finally lets add our weapons!
        Weapons weapons = p.getWeapons();
        if (weapons == null || weapons.getQuantity() == 0) {
            Logger.logWarning("MilitaryCraftDesign.designCraft()  player  "+p.getName()+" does not have any weapons");
            return null;
        }
        
        
        WeaponBaysDesign weaponBays = new WeaponBaysDesign();
        cd.setWeaponBays(weaponBays);
        
        // Lets use the special weapons This player wants
        // We will select the most advanced technology discovered from the list of special 
        // weapons this player would like to use.
        String [] specialWeapons = p.getPlayerCharacter().getCraftsSpecialWeapons();
        if (specialWeapons != null) {
            Weapon best = null;
            int score = 0;
            for (int i=0; i<specialWeapons.length; i++) {
                Weapon w = (Weapon)p.getWeapons().getMostAdvancedWithGroup(specialWeapons[i]);
                if (w != null && w.getCostToResearch() > score) {
                    score = w.getCostToResearch();
                    best = w;
                }
            }
            if (best != null) {
                weaponSpace = best.getSpace();
                int qtySpecialWeapons = (int)((percentSpecialWeapons * remainingSpace) / weaponSpace);
                for (int i=0; i<qtySpecialWeapons; i++) {
                	weaponBays.addWeapon(best, 0);
                }
                remainingSpace = cd.getRemainingSpace();
            }
        }

        Weapon weapon = p.getWeapons().getMaxDamageAttackWeapon();
        weaponSpace = weapon.getSpace();
        int qtyMissiles = (int)((percentMissiles * remainingSpace) / weaponSpace);
        for (int i=0; i<qtyMissiles; i++) {
        	weaponBays.addWeapon(weapon, 0);
        }
        remainingSpace = cd.getRemainingSpace();
        
        weapon = p.getWeapons().getMaxDamageAttackWeaponWithInfinateShots();
        if (weapon == null) {
            weapon = p.getWeapons().getMaxDamageAttackWeapon();
        }
        weaponSpace = weapon.getSpace();
        int qtyWeapons = remainingSpace / weaponSpace;
        for (int i=0; i<qtyWeapons; i++) {
        	weaponBays.addWeapon(weapon, 0);
        }
        
        return cd;
    }
    
}