/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.mayor;

import mot.common.designs.CraftDesign;
import mot.common.technology.Computer;
import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * Use a military craft designer to create craft designs to invade an enemy craft during battle using a troop transport.
 */
public abstract class CraftDesignerSpecial extends CraftDesigner {

    public CraftDesignerSpecial() {
    }

	public double getRatio(Player player) {
		return 0.25;
	}

	public abstract int getQtyCraftOfThisType(Player player) throws ApplicationError;

	
    private static final int QTY_TURNS_TO_BUILD_HULL = 20;

    protected Hull getBigHull(Player p) throws ApplicationError {
		Planet planet = p.getCapitol();
        long maxHullCost;
        if (planet == null) {
            Logger.logWarning("CraftDesignerSpecial.designCraft() Player "+p.getName()+" has no planets - I will use default production");
            maxHullCost = 30 * QTY_TURNS_TO_BUILD_HULL;
        } else {
            maxHullCost = planet.getProductionOutput(true) * QTY_TURNS_TO_BUILD_HULL;
        }
        
        if (p.getHulls() == null || p.getHulls().getQuantity() == 0) {
            Logger.logWarning("CraftDesignerSpecial.designCraft() player "+p.getName()+" has not got any hull");
            return null;
        }
        
        Hull[] hulls = p.getHulls().getCraftHulls();

        // Find the hull with the highest cost but costs less to 
        // build than maxHullCost.
        Hull hull = null;
        int hullCost = 0;
        for (int i=0; i<hulls.length; i++) {
            int cost = hulls[i].getCostToBuild();
            if (cost <= maxHullCost && cost > hullCost) {
                hullCost = cost;
                hull = hulls[i];
            }
        }

        if (hull == null) {
            Logger.logWarning("CraftDesignerSpecial.designCraft() *WARNING* player "+p.getName()+" cannot find hull with desired built cost - USING BEST HULL");
            hull = p.getHulls().getBiggestCraftHull();
        }
		return hull;
	}

    protected Hull getFastestHull(Player p, int minSpace) throws ApplicationError {
        Hull[] hulls = p.getHulls().getCraftHulls();

        Hull hull = null;
        double hullSpeedCost = 0;
        for (int i=0; i<hulls.length; i++) {
            int space = hulls[i].getSpace();
            if (space >= minSpace) {
            	double speedCost = hulls[i].getBattleMovementCost();
                if (hull == null || speedCost < hullSpeedCost) {
	            	hullSpeedCost = speedCost;
	                hull = hulls[i];
                }
            }
        }

        if (hull == null) {
            Logger.logWarning("CraftDesignerSpecial.designCraft() *WARNING* player "+p.getName()+" cannot find hull with desired built cost - USING BEST HULL");
            hull = p.getHulls().getFastestCraftHull();
        }
		return hull;
	}

	protected void addFastestEngine(Player p, CraftDesign cd) {
		// Always use our best engines (what ever they are).
        // but for military choose the fastest.
        if (p.getEngines() == null) {
            Logger.logWarning("CraftDesignerSpecial.designCraft() Player "+p.getName()+" does not have any engines - cannot build craft design");
            return;            
        }
        Engine engine = p.getEngines().getFastestEngine();
        cd.setEngine(engine);
	}

	protected Engine getFastestEngine(Player p) {
		// Always use our best engines (what ever they are).
        // but for military choose the fastest.
        if (p.getEngines() == null) {
            Logger.logWarning("CraftDesignerSpecial.designCraft() Player "+p.getName()+" does not have any engines - cannot build craft design");
            return null;            
        }
        Engine engine = p.getEngines().getFastestEngine();
        return engine;
	}


	protected void addSpacePodSpecials(Player p, CraftDesign cd) throws ApplicationError {
		Specials playerSpecials = p.getSpecials();
		if (playerSpecials == null) return;
        Special spacePodSpecial = playerSpecials.getBestSpacePod();
        if (spacePodSpecial != null) {
            cd.add(spacePodSpecial);
        }
	}

	protected void addHoloEmiter(Player p, CraftDesign cd) throws ApplicationError {
		Specials playerSpecials = p.getSpecials();
		if (playerSpecials == null) return;
        Special holoEmiterSpecial = playerSpecials.getBestHoloEmiter();
        if (holoEmiterSpecial != null) {
            cd.add(holoEmiterSpecial);
        }
	}

	protected void addMostPowerfulComputer(Player p, CraftDesign cd) throws ApplicationError {
		if (p.getComputers() != null) {
            Computer computer = p.getComputers().getMostPowerfulComputer();
            cd.setComputer(computer);
        }
	}
	
	protected void addMostPowerfulScanner(Player p, CraftDesign cd) throws ApplicationError {
		if (p.getComputers() != null) {
			Scanner scanner = p.getScanners().getBestScanner();
            cd.setScanner(scanner);
        }
	}
	
	protected void addShieldsWithHighestPower(Player p, CraftDesign cd) throws ApplicationError {
		// Always choose our shields with the highest protection * power.
        // But we will reserve atleast MIN_WEAPON_RATION of space for weapons!
        if (p.getShields() != null) {
            Shield shield = p.getShields().getBestDefenseShield();
            if (shield != null) {
                cd.setShields(new Shields(shield));
            }
        }
	}


}