/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.mayor;

import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.common.OwnedObject;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

/**
 * Mayors manage the priorities for Planets belonging to a computer player. It examines the
 * players position and determines what this player requires its planets to build.
 * Mayors manage the projects on planets and the resource percentages - Thus is manages
 *  - A Planets defence (shields, weapons, scanners)
 *  - Craft built by the planet (colony or military)
 */
public abstract class Mayor extends OwnedObject {
	
	private static final int CALCULATE_PRIORITIES_INTERVAL = 9;
	private int dateOfLastCalculation;

	protected Mayor() {
	}

	protected Mayor(ComputerPlayer owner) {
		super(owner);
	}

	/**
	 * Returns the highest priority task that is closest to this craft.
	 * It always chooses the highest priority, but if two tasks are or equals priority
	 * it will select the closest.
	 */
	public void endTurn() throws ApplicationError, ValidationError {
		int date = MotEngine.getInstance().getDate();
		if ( (date - dateOfLastCalculation) >= CALCULATE_PRIORITIES_INTERVAL) {
			setupPlanetaryTradeRoutes();
		}
       dateOfLastCalculation = date;
    }


	/**
	 * This generates craft designs for this player.
	 */
	private transient CraftDesignerFactory craftDesignFactory;

	/**
	 * Returns the next craft to build. This may be a colony ship or a war ship.
	 * @throws ValidationError 
	 */
	public abstract void notifyNoProject(Planet planet) throws ApplicationError, ValidationError;

	public CraftDesignerFactory getCraftDesignerFactory() {
		if (craftDesignFactory == null) {
			craftDesignFactory = new CraftDesignerFactory(getName());
		}
		return craftDesignFactory;
	}

    /**
     * Called when troops (troop transports) are required to invade a planet.
     * @param planet The planet that troop transports are required to invade.
     */
    public abstract void notifyNeedTroops(Planet planet) throws ApplicationError;
	public abstract void notifyColonyLost(Planet colony, Player agressor) throws ApplicationError;

	/**
	 * Requests that the mayor re-calculate the percentages for the planet passed in.
	 */
	public abstract void setPercents(Planet planet) throws ApplicationError, ValidationError;

    /**
     * @throws mot.common.util.ApplicationError
     * @return The closest planet that contains a resource and that the general would 
     * build a mine on.
     */
    public abstract Planet getClosestResourceWouldMine(boolean considerPlanetsWithoutAResource, boolean rightNow) throws ApplicationError;
    public abstract BlackHole getClosestBlackHoleWouldUse() throws ApplicationError;

    /**
     * Create trade routes between the planets controlled by this mayor.
     * @throws ApplicationError 
     */
    public abstract void setupPlanetaryTradeRoutes() throws ApplicationError;
    
	/**
	 * For Test Harnesses only!
	 */
	public abstract boolean buildOutpost(Planet builder) throws ApplicationError;
	public abstract boolean buildSpaceStation(Planet builder) throws ApplicationError;

}