/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.mayor;
import java.util.ArrayList;

import mot.common.bean.Location;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.WeaponDesign;
import mot.common.technology.Hull;
import mot.common.technology.Hulls;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Specials;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.adapter.CraftVOAdapter;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.Project;
import mot.engine.planets.ProjectBattleStation;
import mot.engine.planets.ProjectCraft;
import mot.engine.planets.ProjectCraftUpgrade;
import mot.engine.planets.ProjectOutpost;
import mot.engine.planets.ProjectScanner;
import mot.engine.planets.ProjectShield;
import mot.engine.planets.ProjectSpaceStation;
import mot.engine.planets.ProjectWeapon;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRouteFactory;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.generals.CraftTaskIdle;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskInvade;

/**
 * Mayors manage the priorities for Planets belonging to a computer player. It examines the
 * players position and determines what this player requires its planets to build.
 * Mayors manage the projects on planets and the resource percentages - Thus is manages
 *  - A Planets defence (shields, weapons, scanners)
 *  - Craft built by the planet (colony or military)
 */
public class MayorBasic extends Mayor {
	public static final int QTY_TROOPS_PER_TRANSPORT = 25;

	protected MayorBasic() {
	}

	public MayorBasic(ComputerPlayer owner) {
		super(owner);
	}

	/**
	 * Returns the next craft to build. This may be a colony ship or a war ship.
	 * @throws ValidationError 
	 */
	public void notifyNoProject(Planet planet) throws ApplicationError, ValidationError {
        ComputerPlayer owner = (ComputerPlayer) getOwner();
        int qtyCrafts = owner.getQtyCrafts();
        int qtyColony = owner.getQtyColonyCrafts();
        int qtyMilitary = qtyCrafts - qtyColony;
        
		if (Logger.isDebug()) {
            Logger.logDebug("MayorBasic.notifyNoProject() planet="+planet.getName()+"   qtyColony="+qtyColony+" qtyMilitary="+qtyMilitary);
        }
        

        // AI says we always try to have a colony craft available for colonisation and we try to keep this 
        // colony craft up to date with our technology.
        if (qtyColony > 0) {
            Craft colonyCraft = owner.getColonyCraft();
            if (colonyCraft.getTask() == null || colonyCraft.getTask() instanceof CraftTaskIdle) {
                CraftDesign design = getCraftDesignerFactory().getNextColonyCraftToBuild(owner);
                if ( design != null && hasBetterColonyTech(design.getSpecialTechs(), colonyCraft.getSpecials()) ) {
                    // This colony craft is out of date - delete it
                	colonyCraft.transferTo(planet.getOrbit());
                	if (design.getHullTech() != colonyCraft.getHull()) {
                		design = new CraftDesign(design);
                		design.setHull(colonyCraft.getHull());
                	}
                	planet.setProject(new ProjectCraftUpgrade(new CraftUpgradeDesign(design, new CraftVOAdapter(colonyCraft, owner)), planet));
                	return;
                }
            }
        } else {
	        // We will just build the best colony craft we can to have it always on hand to colonise.
			CraftDesign design = getCraftDesignerFactory().getNextColonyCraftToBuild(owner);
	        if (design != null) {
	            // check that no other planet is building this craft design for this owner.
	        	boolean otherPlanetBuildingColonyCraft = false;
	            Planet [] planets = owner.getPlanets();
	            for (int i=0; i<planets.length; i++) {
	                if (planets[i] == planet) continue;
	                Project pp = planets[i].getProject();
	                if (pp != null && pp instanceof ProjectCraft) {
	                    CraftDesign cd = ((ProjectCraft)pp).getCraftDesign();
	                    if (cd.canColonise() && isBetterColonyDesign(design, cd)) {
	                        CraftDesign ncd = getCraftDesignerFactory().getNextMilitaryCraftToBuild(owner);
	                        planets[i].changeProjectTo(new ProjectCraft(ncd, planets[i], true));
	                    	return;
	                    } else if (cd.canColonise()) {
	                    	otherPlanetBuildingColonyCraft = true;
	                        break;
	                    }
	                } else if (pp != null && pp instanceof ProjectCraftUpgrade) {
	                	CraftUpgradeDesign cd = ((ProjectCraftUpgrade)pp).getCraftDesign();
	                	if (cd.getDesign().canColonise()) {
	                		otherPlanetBuildingColonyCraft = true;
	                        break;
	                    }
	                }
	            }
	            if (!otherPlanetBuildingColonyCraft) {
	            	planet.setProject(new ProjectCraft(design, planet, true));
	            	return;
	            }
	        }
        }
        
        // This AI says for every military craft a player has, it will try to build a weapon in each of
        // its planets. (so I have 10 crafts => I shall have 10 weapons on each planet).
        Weapons weapons = planet.getWeapons();
        int qtyWeapons = (weapons == null) ? 0 : weapons.getQuantity();
        if (qtyWeapons < 2 || qtyWeapons < 2* qtyMilitary) {
            if (owner.getWeapons() != null && owner.getWeapons().getQuantity() > 0) {
                Weapon weapon = owner.getWeapons().getMaxDamageWeaponWithGroup(Weapon.GROUP_MISSILE);
                if (weapon != null) {
                	if (qtyWeapons + 1 <= weapon.getMaxQtyOnPlanet()) {
                		planet.setProject(new ProjectWeapon(new WeaponDesign(weapon)));
                		return;                		
                	}
                }
            }
        }        
        
        // The AI says if the general needs an outpost to colonise a new planet and no other
        // planet is building an outpost, then we will build an outpost.
        if (owner.getGeneral().needOutpost()) {
            int qty = owner.getQtyPlanetsWithProject(ProjectOutpost.class);
            if (qty == 0) {
            	int qtyTrans = owner.getTransportManagerNotNull().getQtyTransportsWithOutpost();
            	if (qtyTrans == 0) {
	                if (buildOutpost(planet)) {
	                	owner.getGeneral().clearOutpostNeeded();
	                    return;
	                }
            	}
            }
        }

        // The AI says one I should have 2 weapons I should have 2 military crafts to defend my planets
        if (owner.getPlayerCharacter().getQtyCraftToDeclareWar() < 3) {
        	Logger.logWarning("MayorBasic.notifyNoProject() - Will build many military crafts before developing planets");
        }
        
        if (qtyMilitary < 2) {
            CraftDesign design = getCraftDesignerFactory().getNextMilitaryCraftToBuild(owner);
            if (Logger.isDebug()) {
                Logger.logDebug("MayorBasic.notifyNoProject() Build craft design "+design.getName());
            }

    		planet.setProject(new ProjectCraft(design, planet, true));
    		return;
        }
        
        // The AI says if I should have more than 2 military craft then I shall build shields, scanners  
        // if I have not already. 
        Shields shields = planet.getShields();
        int qtyShields = (shields == null) ? 0 : shields.getQuantity();
        if (qtyShields == 0) {
            if (owner.getShields() != null && owner.getShields().getQuantity() > 0) {
                Shield shield = owner.getShields().getBestDefenseShield();
				planet.setProject(new ProjectShield(new ShieldDesign(shield)));
                return;
            }
        }
        
        // The AI says I shall build a scanner if I can 
        Scanners scanners = planet.getScanners();
        int qtyScanners = (scanners == null) ? 0 : scanners.getQuantity();
        if (qtyScanners == 0) {
            if (owner.getScanners() != null && owner.getScanners().getQuantity() > 0) {
                Scanner scanner = owner.getScanners().getBestScanner();
				planet.setProject(new ProjectScanner(new ScannerDesign(scanner)));
                return;
            }
        }
              
        // The AI says I shall build a battle station if I can 
        if (planet.getOrbital() == null) {
            if (owner.getHulls() != null) {
                Hull battleStation = owner.getHulls().getBiggestBattleStationHull();
                if (battleStation != null) {
                    planet.setProject(new ProjectBattleStation(new BattleStationDesign(battleStation)));
                    return;
                }
            }
        }
        
        // The AI says if we have a planet with a special resource then we will build 
        // a mining station to mine that planet.
    	if (buildSpaceStation(planet)) {
    		return;
    	}
        
        CraftDesign design = getCraftDesignerFactory().getNextMilitaryCraftToBuild(owner);
        if (Logger.isDebug()) {
            Logger.logDebug("MayorBasic.notifyNoProject() Build craft design "+design.getName());
        }

		planet.setProject(new ProjectCraft(design, planet, true));
	}

	public boolean buildOutpost(Planet builder) throws ApplicationError {
		ComputerPlayer owner = (ComputerPlayer) getOwner();
        if (owner.getHulls() != null) {
            Hull hull = owner.getHulls().getBiggestOutpostHull();
            if (hull != null) {
        		StellarObject target = owner.getGeneral().getBestStellarObjectToDeployOutpost();
        		builder.setProject(new ProjectOutpost(new OutpostDesign(hull), target));
                return true;
            }
        }
		return false;
	}
	
	public boolean buildSpaceStation(Planet builder) throws ApplicationError {
		Player owner = getOwner();
		Hulls hulls = owner.getHulls();
		if (hulls == null) {
			return false;
		}
    	Hull hull = owner.getHulls().getBiggestSpaceStationHull();
    	if (hull != null) {
    		BlackHole blackHole = getClosestBlackHoleWouldUse();
    		if (blackHole != null && !hasOrIsBuildingSpaceStationFor(blackHole)) {
            	builder.setProject(new ProjectSpaceStation(new SpaceStationDesign(hull), new Orbit(blackHole)));
                return true;
            }
            Planet planetWithResource = getClosestResourceWouldMine(true, false);
            if (planetWithResource != null && !hasOrIsBuildingSpaceStationFor(planetWithResource.getStellarObject())) {
               	builder.setProject(new ProjectSpaceStation(new SpaceStationDesign(hull), new Orbit(planetWithResource)));
                return true;
            }
    	}
		return false;
	}
	
    private boolean hasOrIsBuildingSpaceStationFor(StellarObject so) throws ApplicationError {
    	Player owner = getOwner();
		int qtyTransports = owner.getTransportManagerNotNull().getQtySpaceStationsWithDestination(so);
    	if (qtyTransports > 0) {
    		return true;
    	}
    	Planet[] planets = owner.getPlanets();
    	if (planets != null) {
    		for (int i = 0; i < planets.length; i++) {
				Project project = planets[i].getProject();
				if (project != null && project instanceof ProjectSpaceStation) {
					ProjectSpaceStation pss = (ProjectSpaceStation) project;
					Orbit target = pss.getTarget();
					if (target != null && target.getStellarObject() == so) {
						return true;
					}
				}
			}
    	}
		return false;
	}

	private boolean isBetterColonyDesign(CraftDesign cd1, CraftDesign cd2) throws ApplicationError {
        return hasBetterColonyTech(cd1.getSpecialTechs(), cd2.getSpecialTechs());
    }

    private boolean hasBetterColonyTech(Specials s1, Specials s2) {
        Double s1Rating = null;
        Double s2Rating = null;
        if (s1 != null) {
            s1Rating = s1.getCanColoniseRating();
        }
        if (s2 != null) {
            s2Rating = s2.getCanColoniseRating();
        }
        return (Comp.bigger(s2Rating, s1Rating));
    }

	/**
	 * Returns the best planet this player can colonise. If no planet can be colonised
	 * then null is returned.
     * Returns the best planet (defined by the planets quality score) within range.
	 */
	public Planet getBestPlanetToColonise() throws ApplicationError {
		Planet best = null;
		long bestRating = 0;
		Planet planets [] = MotEngine.instance.getPlanets().getPlanets();

		for (int i=0; i<planets.length; i++) {
            SolarSystem solarSystem = planets[i].getSolarSystem();
            if (getOwner().canTravelTo(solarSystem)) {
                if (!solarSystem.hasOwner() || solarSystem.isOwnedBy(getOwner())) {
                    if (getOwner().canColonise(planets[i]) && planets[i].getQualityScore() > bestRating) {
                        bestRating = planets[i].getQualityScore();
                        best = planets[i];
                    }
                }
            }
		}
		return best;
	}

    /**
     * Called when troops (troop transports) are required to invade a planet.
     * This method cheats - it simple creates new troop transports in the home world and sends them 
     * to the planet (if there are not any troop transports already on-route to the solar system).
     * @param target The planet that troop transports are required to invade.
     */
    public void notifyNeedTroops(Planet target) throws ApplicationError {
    	Player owner = getOwner();
        int qty = owner.getQtyTroopsWithDestination( target.getStellarObject() );
        if (Logger.isInfo()) {
            Logger.logDebug("MayorBasic.notifyNeedTroops() player="+owner.getName()+" desired to invade Planet="+target.getName()+" already has "+qty+" troops enroute");
        }
        
        if (owner instanceof ComputerPlayer && qty > 0) {
        	// Do not send troops if some are already on the way.
        	return;
        }
        
        Planet source = getClosestPlanetToGetMarines(target);
        if (source == null) {
        	return;
        }
        
        Transport transport = owner.getTransportManagerNotNull().getFreeTransport();
        if (transport == null) {
        	if (owner instanceof ComputerPlayer) {
    	        transport = new Transport(owner);
    	        owner.getTransportManagerNotNull().setTransport(transport);
        	} else {
        		return;
        	}
        }
        
        new TransportTaskInvade(transport, source, target, QTY_TROOPS_PER_TRANSPORT);
    	
    }

    /**
     * The best planet that should be used to fetch troops from is defined as
     *  - The closest that has defences of at least double QTY_TROOPS_PER_TRANSPORT
     * @throws mot.common.util.ApplicationError
     * @return The planet that this player will source troops from to invalid the planet passed in as a parameter.
     */
    private Planet getClosestPlanetToGetMarines(Planet target) throws ApplicationError {
    	ComputerPlayer owner = (ComputerPlayer) getOwner();
        Planet [] planets = owner.getPlanets();
        if (planets == null || target == null) {
        	return null;
        }
        Location targetLoc = target.getStellarObject().getLocation();
        int bestDist = Integer.MAX_VALUE;
        Planet best = null;
        for (int i=0; i<planets.length; i++) {
            Planet p = planets[i];
            if (p.getQtyGroundDefense() > QTY_TROOPS_PER_TRANSPORT * 2) {
                Location loc = p.getStellarObject().getLocation();                
                int d = targetLoc.getDistanceTo(loc);
                if (d < bestDist) {
                    best = p;
                    bestDist = d;
                }
            }
        }
        return best;
    }
    
    /**
     * For details on what a resource is @see mot.engine.planets.Resource
     * This will only return planets that this player's General would consider building a 
     * mine on!
     * @throws mot.common.util.ApplicationError
     * @return The closest planet that contains a resource and that the general would 
     * build a mine on.
     */
    public Planet getClosestResourceWouldMine(boolean considerPlanetsWithoutAResource, boolean rightNow) throws ApplicationError {
        Planet [] planets = MotEngine.getInstance().getPlanets().getPlanets();
        int distWithResource = 0;
        Planet bestWithResource = null;
        int distNoResource = 0;
        Planet bestNoResource = null;
        for (int i=0; i<planets.length; i++) {
            Planet p = planets[i];
            ComputerPlayer owner = (ComputerPlayer) getOwner();
            if ((considerPlanetsWithoutAResource || p.getResource() != null) &&  owner.getGeneral().canDeployMiningStation(p, rightNow)) {
                Location loc = p.getStellarObject().getLocation();
                StellarObject myClosest = getOwner().getClosestStellarObject(loc);
                if (myClosest != null) {
                    int d = myClosest.getDistanceTo(loc);
                    if (p.getResource() != null) {
                        if (bestWithResource == null || distWithResource > d) {
                        	bestWithResource = planets[i];
                            distWithResource = d;
                        }
                    } else {
                        if (bestNoResource == null || distNoResource > d) {
                        	bestNoResource = planets[i];
                            distNoResource = d;
                        }
                    }
                }
            }
        }
        if (bestWithResource != null) {
        	return bestWithResource;
        } else {        	
        	return bestNoResource;
        }
    }

    /**
     * For details on what a resource is @see mot.engine.planets.Resource
     * @throws mot.common.util.ApplicationError
     * @return The closest black hole that the general would build a mine on.
     */
    public BlackHole getClosestBlackHoleWouldUse() throws ApplicationError {
        BlackHole[] blackHoles = MotEngine.getInstance().getStellarObjects().getBlackHoles();
        int dist = 0;
        BlackHole best = null;
        for (int i=0; i<blackHoles.length; i++) {
        	BlackHole blackHole = blackHoles[i];
            ComputerPlayer owner = (ComputerPlayer) getOwner();
            if (owner.getGeneral().canDeployMiningStation(blackHole)) {
                Location loc = blackHole.getLocation();
                StellarObject myClosest = getOwner().getClosestStellarObject(loc);
                if (myClosest != null) {
                    int d = myClosest.getDistanceTo(loc);
                    if (best == null || dist > d) {
                        best = blackHole;
                        dist = d;
                    }
                }
            }
        }
        return best;
    }

	public void notifyColonyLost(Planet colony, Player agressor) throws ApplicationError {
		Player owner = getOwner();
		Planet homePlanet = owner.getCapitol();
        if (homePlanet == null) {
        	return;
        }
		CraftDesign fd = getCraftDesignerFactory().getNextMilitaryCraftToBuild(owner);
		int qty = colony.getSize()/40;
        if (Logger.isDebug()) {
            Logger.logDebug("MayorBasic.notifyColonyLost() Gifted "+fd.getName()+"x"+qty);
        }
        for (int i=0;i<qty;i++) {
        	CraftFactory.getCraftInstance(fd, owner, null, new Orbit(homePlanet), true, false);
        }
	}

    
	/**
	 * Requests that the mayor re-calculate the percentages for the planet passed in.
     * Uses the Planets default settings - calls setPercentsAuto().
	 */
	public void setPercents(Planet planet) throws ApplicationError, ValidationError {
		ComputerPlayer owner = (ComputerPlayer) getOwner();
		if (owner.isIsolationist() && !MotEngine.instance.getRelationsNotNull().isAtWarWithAPlayer(owner)) {
			boolean isBuildingMillitaryCraft = false;
			if (planet.getProject() != null && planet.getProject() instanceof ProjectCraft) {
				ProjectCraft p = (ProjectCraft) planet.getProject();
				isBuildingMillitaryCraft = p.getCraftDesign().isMilitary();
			}
			// Spend a long time building military craft, focus heavily on research
			planet.setPercentsAuto(true,true, isBuildingMillitaryCraft);
		} else {
			planet.setPercentsAuto(true,true, false);
		}
    }

	@Override
	public void setupPlanetaryTradeRoutes() throws ApplicationError {
		ComputerPlayer owner = (ComputerPlayer) getOwner();
		Planet[] planets = owner.getPlanets();
		if (planets == null) return;
		ArrayList<SolarSystemIndustryWork> candidatesList = new ArrayList();
		for (int i = 0; i < planets.length; i++) {
			SolarSystem ss = planets[i].getSolarSystem();
			if (TradeRouteFactory.canCreateTradeRoute(ss, null)) {
				candidatesList.add(new SolarSystemIndustryWork(ss, ss.getMaxIndustryWork(owner)));
			}
		}
		SolarSystemIndustryWork [] candidates = candidatesList.toArray(new SolarSystemIndustryWork [candidatesList.size()]);
		sortByIndustryWork(candidates);
		for (int i = 0; i < candidates.length; i++) {
			SolarSystem best = getBestSolarSystemToTradeWith(candidates[i].ss);
			if (best != null) {
				if (Logger.isDebug()) {
					Logger.logDebug("MayorBasic.setupPlanetaryTradeRoutes() Create Trade Route "+candidates[i].ss+"=>"+best); 
				}
				TradeRouteFactory.create(candidates[i].ss, best);
				// Only create one per turn
				return;
			}
		}
	}

	private SolarSystem getBestSolarSystemToTradeWith(SolarSystem solarSystem) throws ApplicationError {
		ComputerPlayer owner = (ComputerPlayer) getOwner();
		Planet[] planets = owner.getPlanets();
		if (planets == null) return null;
		SolarSystem best = null;
		int bestScore = 0;
		for (int i = 0; i < planets.length; i++) {
			SolarSystem ss = planets[i].getSolarSystem();
			if (ss != solarSystem) {
				if (TradeRouteFactory.canCreateTradeRoute(solarSystem, ss)) {
					int work = ss.getMaxIndustryWork(owner);
					if (work > bestScore) {
						bestScore = work;
						best = ss;
					}
				}				
			}
		}
		return best;
	}

	private class SolarSystemIndustryWork {
		public SolarSystemIndustryWork(SolarSystem ss, int industryWork) {
			this.ss = ss;
			this.industryWork = industryWork;
		}
		public String toString() {	return "" + ss;		}
		public SolarSystem ss;
		public int industryWork;
	}
	
	private void sortByIndustryWork(SolarSystemIndustryWork [] l) throws ApplicationError {
		if (l == null || l.length < 2) return;
		ComputerPlayer owner = (ComputerPlayer) getOwner();
		SolarSystem capitol = owner.getCapitolSolarSystem();
		for (int i = 0; i < l.length-1; i++) {
			for (int j = 0; j < l.length - 1 - i; j++) {
				if (l[j].industryWork < l[j+1].industryWork || l[j+1].ss == capitol) {
					SolarSystemIndustryWork t = l[j];
					l[j] = l[j+1];
					l[j+1] = t;
				}
			}			
		}
	}


	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
	}

	protected String getElementName() { return "mayor-basic"; }

}