/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.strategists;

import mot.common.bean.DiplomaticProposal;
import mot.common.morals.MoralChoice;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.ValidationError;
import mot.common.vo.StrategistEventVO;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.common.OwnedObject;
import mot.engine.planets.Planet;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

/**
 * Manages the player choice to make peace, declare war and generally manage
 * the players relationship on the board.
 * Currently it simple uses game setting on how much this player likes other
 * players by default then modifies them with a difficulty bonus that can make
 * the computer players hate or like the human player more or less and then
 * players tend to like any other player that is beginning to dominate the game.
 * 
 * This classes also maintains a list of the last HISTORY_SIZE treaties proposed 
 * by this strategist and the last HISTORY_SIZE rejected. These may be used by
 * implementations to avoid asking the same request over again or respond differently 
 * if the same player asks for the same treaty twice in a row.
 */
public abstract class Stratagist extends OwnedObject {

    /**
     * A list of diplomatic events that influence this players relations of response 
     * to treaty requests.
     */
    private StrategistEventManager eventManager;    

    public StrategistEventManager getEventManager() { return eventManager; }
    public StrategistEventManager getEventManagerNotNull() { if (eventManager == null) eventManager = new StrategistEventManager(); return eventManager; }
    public void setEventManager(StrategistEventManager eventManager) { this.eventManager = eventManager; }
    
    protected StratagistPlayerRating createPlayerRating(Player player, int rating) {
        return new StratagistPlayerRating(player, rating);
    }

	private static final int CALCULATE_PRIORITIES_INTERVAL = 10;

	/**
	 * The date of the last time this player calculates its priorities.
	 */
	protected int dateOfLastCalculation;

	public int getDateOfLastCalculation() { return dateOfLastCalculation; }
	public void setDateOfLastCalculation(int v) { dateOfLastCalculation = v; }

	public Stratagist() {
	}

	public Stratagist(ComputerPlayer owner) throws ApplicationError {
		super(owner);
		dateOfLastCalculation = 0;
	}

    public abstract void flushCache();
    
	/**
	 * Returns the highest priority task that is closest to this craft.
	 * It always chooses the highest priority, but if two tasks are or equals priority
	 * it will select the closest.
	 */
	public void endTurn() throws ApplicationError, ValidationError {
		int date = MotEngine.getInstance().getDate();
		if ( (date - dateOfLastCalculation) <= CALCULATE_PRIORITIES_INTERVAL) {
	        _endTurn();
			return;
		}
		if (MotEngine.instance.getLevel() < 1.0 && date < MotEngine.START_DATE+8) {
			// Lets not bombard beginners with treaty requests.
	        _endTurn();
			return;
		}
        dateOfLastCalculation = date;
        calculatePriorities();
        _endTurn();
    }

	protected abstract void _endTurn() throws ApplicationError, ValidationError;

	/**
	 * Calculate the priorities for this player. Performs the task @see calculatePriorities
     * for details. However - This method is only called once the CALCULATE_PRIORITIES_INTERVAL
     * has expired.
	 */
	public abstract void calculatePriorities() throws ApplicationError, ValidationError;

	/**
	 * Other player in game has made a diplomatic requested (peace, trade treaty, swap assets etc) 
	 * @param player The player making the proposal
	 * @param proposal The Proposal being made
	 * @return If this plays will accept the request
	 */
	public abstract StrategistResponse diplomaticRequest(Player player, DiplomaticProposal proposal) throws ApplicationError;
	
	/**
	 * This player has made a diplomatic requested (peace, trade treaty, swap assets etc) and this 
	 * notifies this player of the other players response. 
	 * @param player The player making the proposal
	 * @param proposal The Proposal being made
	 * @return If this plays will accept the request
	 */
	public abstract void notifyDiplomaticResponse(Player player, DiplomaticProposal proposal, boolean accepted) throws ApplicationError;


    /**
     * Informs the strategist of the war declaration so it can modify its relationships. 
     * @throws mot.common.util.ApplicationError
     * @param victim The player who the aggressor has declared war on
     * @param agressor The player who has started the war
     */
	public abstract void notificationOfWar(Player agressor, Player victim) throws ApplicationError;

	/**
     * Called when another player has been caught spying - Depending on the implementation of 
     * the stratagist it will have a negative impact on the relationship!
     * @param player The player caught spying
     */
	public abstract void notifySpying(Player player, Technology tech) throws ApplicationError ;

	/**
     * Called when another player has been caught sabotaging installations - Depending on the 
     * implementation of the stratagist it will have a negative impact on the relationship!
     * @param player The player caught sabotaging
     */
	public abstract void notifySabotage(Player player) throws ApplicationError ;

    /**
     * Called when a battle has ended that contained this player - Allows the AI to attempt to
     * make peace because we are losing or reject peace because we are winning.
     * @param event The end battle event.
     */
    public abstract void notiftBattleEnd(Battle battle) throws ApplicationError;

    /**
     * Called when any player makes a moral choice. 
     * @param player The player that has just made a moral decision
     * @param mc The moral decision that the player has just made
     */
	public void notifyOfMoralChoice(Player player, MoralChoice mc) throws ApplicationError {
		if (mc.getRelationsBonus() != null) {
			getEventManagerNotNull().add(new StrategistEventMoralDecision(player, mc));
		}
	}
    
    /**
     * Called when this player has lost a planet to the player agressor - Than may be an invasion
     * or a bombardment so the colony may still exist but owned by another player or the planet may
     * no longer have any population.
     * @param colony The planet that has been attacked.
     * @param agressor The player who has attacked our colony.
     * @throws mot.common.util.ApplicationError
     */
	public abstract void notifyColonyLost(Planet colony, Player agressor, boolean genocide) throws ApplicationError;

    /**
     * @return Returns the player that this player hates the most.
     */
    public abstract StratagistPlayerRating getMostHatedPlayer() throws ApplicationError;

    /**
     * @return Returns the player that this player likes or loves the most.
     */
    public abstract StratagistPlayerRating getMostLovedPlayer() throws ApplicationError;

    /**
     * @return Returns the love for this player that this stratagist recommends.
     */
	public abstract int getRating(Player player, boolean flushCache) throws ApplicationError;

    /**
     * @return Descriptions of all the events/actions this player has done that player 
     * other has remembered (and effect the relations).
     */
    public StrategistEventVO [] getStrategistEvents(Player other) throws ApplicationError {
    	Player owner = getOwner();
        return getEventManagerNotNull().getStrategistEvents(owner, other, true);
    }
//
//    /**
//     * @return A list of textual descriptions of all events this player remembers
//     * that effect the relationship with other players.
//     */
//    public DiplomaticEvent [] getDiplomaticEvents() throws ApplicationError {
//        return getEventManagerNotNull().getDiplomaticEvents();
//    }


    /**
     * Strategists maintain a list of significant events that effect relationships.
     * Therefore they need serializing as part of the players state.
     */
	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
        if (eventManager != null) {
            eventManager.toXML(writer, "event-manager");
        }
    }

	public abstract void notifyBreakNonAggression(Player player);
	public abstract void notifyBreakAlliance(Player player);
	public abstract void notifyUseOfBiologicalWeapons(Player player) throws ApplicationError;
	public abstract void notifyRelationsEstablished(Player player, boolean starting) throws ApplicationError;
	
	/**
	 * This player has been created but has not yet moved (just joined the game).
	 */
	public abstract void notifyEmpireEstablished() throws ApplicationError;

	/**
	 * Called when another player has lost a colony from bombardment.
	 */
	public abstract void notifyOfGenocide(Planet planet, Player agressor, Player victim) throws ApplicationError;

	/**
	 * Called when another player has lost a colony from invasion.
	 */
	public abstract void notifyOfInvasion(Planet planet, Player agressor, Player victim) throws ApplicationError;
	
	/**
	 * @return Do any of the tasks reference this planet. If yes this planet cannot be deleted by the editor and 
	 * the editor will throw an error.
	 */
	public boolean referencesPlanet(Planet planet) throws ApplicationError, ValidationError {
		StratagistEvent[] events = getEventManagerNotNull().getEventsAsArray();
		if (events != null) {
			for (int i = 0; i < events.length; i++) {
				if (events[i] instanceof StrategistEventColonyLost) {
					if (planet.sameName(((StrategistEventColonyLost)events[i]).getPlanetName())) {
						return true;
					}
				}
			}
		}
		return false;
	}

		
	
    
}


