/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.strategists;

import java.util.HashMap;

import mot.common.bean.DiplomaticProposal;
import mot.common.bean.DiplomaticStatements;
import mot.common.bean.NameList;
import mot.common.bean.PlayerAssets;
import mot.common.bean.TreatyType;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattlePlanet;
import mot.engine.crafts.Craft;
import mot.engine.planets.OrbitalReference;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.PlayerCharacter;
import mot.engine.players.ResearchProject;
import mot.engine.players.generals.CraftTask;
import mot.engine.players.generals.CraftTaskIdle;
import mot.engine.relations.AllianceTreaty;
import mot.engine.relations.NonAgressionTreaty;
import mot.engine.relations.PeaceTreaty;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.relations.WarTreaty;

/**
 * Manages the player choice to make peace, declare war and generally manage
 * the players relationship on the board.
 * Currently it simply uses game setting on how much this player likes other
 * players by default then modifies them with a difficulty bonus that can make
 * the computer players hate or like the human player more or less and then
 * players tend to like any other player that is beginning to dominate the game.
 * 
 * TODO - Make sure player declare war on is in range (maybe neighbours get it first).
 */
public class StratagistBasic extends Stratagist {

    private static final int MIN_AGE_OF_WAR_TO_ACCEPT_PEACE = 15;
    
	private static final int SPYING_EFFECT = 11;
    private static final int SABOTAGE_EFFECT = 21;
    private static final int BATTLE_EFFECT = 10;

    private static final int GIFT_TECHNOLOGY_EFFECT_SMALL = 5;
    private static final int GIFT_TECHNOLOGY_EFFECT_LARGE = 10;
    private static final int GIFT_MONEY_EFFECT_PER_1000 = 3;
    private static final int GIFT_CRAFT_EFFECT_PER_1000 = 3;
    private static final int GIFT_PLANET_VSMALL_EFFECT = 1;
    private static final int GIFT_PLANET_SMALL_EFFECT = 5;
    private static final int GIFT_PLANET_MEDIUM_EFFECT = 10;
    private static final int GIFT_PLANET_LARGE_EFFECT = 15;

    private static final int DEMAND_EFFECT_ACCEPTED = 5;
    private static final int DEMAND_EFFECT_REJECTED = 0;

    private static final int MIN_MILITARY_CRAFTS_TO_FACE_ANCIENTS = 8;
    private static final int FEAR_OF_ANCIENTS_BONUS = 20;
    
    public static Player __TEST_REQUEST_ALLIANCE__ = null;
    public static Player __TEST_REQUEST_TRADE__ = null;
    
    public Player surrenderToAtEndTurn = null;
    
	public StratagistBasic() {
	}

	public StratagistBasic(ComputerPlayer owner) throws ApplicationError {
		super(owner);
	}

    public StratagistPlayerRating getMostHatedPlayer() throws ApplicationError {
		Player mostHated = null;
		int mostHatedRating = 0;
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		for (int i=0; i<players.length; i++) {
			Player owner = getOwner();
			if (players[i] != owner && players[i].hasDiplomaticRelationsWith(owner)) {
				int rating = getRating(players[i]);
				if ( mostHated == null || mostHatedRating > rating) {
					mostHated = players[i];
					mostHatedRating = rating;
				}
			}
		}
        return createPlayerRating(mostHated, mostHatedRating);
    }
    
    public StratagistPlayerRating getMostLovedPlayer() throws ApplicationError {
		Player mostLoved = null;
		int mostLovedRating = 0;
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		for (int i=0; i<players.length; i++) {
			Player owner = getOwner();
			if (players[i] != owner && players[i].hasDiplomaticRelationsWith(owner)) {
				int rating = getRating(players[i]);
				if ( mostLoved == null || mostLovedRating < rating) {
					mostLoved = players[i];
					mostLovedRating = rating;
				}
			}
		}
        return createPlayerRating(mostLoved, mostLovedRating);
    }
    
	/**
	 * Returns the highest priority task that is closest to this craft.
	 * It always chooses the highest priority, but if two tasks are or equals priority
	 * it will select the closest.
	 */
	public void calculatePriorities() throws ApplicationError, ValidationError {
        ratings = null;

		if (Logger.isDebug()) {
            Logger.logDebug("StratagistBasic.calculatePriorities() for player "+getOwner().getName());
        }
		
		
		ComputerPlayer owner = (ComputerPlayer) getOwner();
		if (__TEST_REQUEST_ALLIANCE__ != null) {
			__TEST_REQUEST_ALLIANCE__.diplomaticRequest(owner, DiplomaticProposal.getAlliance(), "TEST ALLIANCE REQUEST");
			__TEST_REQUEST_ALLIANCE__ = null;
        }
		if (__TEST_REQUEST_TRADE__ != null) {
			__TEST_REQUEST_TRADE__.diplomaticRequest(owner, DiplomaticProposal.getTrade(), "TEST TRADE REQUEST");
			__TEST_REQUEST_TRADE__ = null;
        }

        StratagistPlayerRating mostHated = getMostHatedPlayer();
        StratagistPlayerRating mostLoved = getMostLovedPlayer();

		// Now request peace for all players we are at war with, except the most hated
        PlayerCharacter pc = owner.getPlayerCharacter();
		Player [] playersAtWar = owner.getPlayersAtWar();
        if (playersAtWar != null) {
            for (int i=0; i<playersAtWar.length; i++) {
                if (mostHated.player != null && !mostHated.player.equals(playersAtWar[i])) {
                	if (owner.hasDiplomaticRelationsWith(mostHated.player)) {
                		String msg = pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_NOT_MOST_HATED);
                		playersAtWar[i].diplomaticRequest(owner, DiplomaticProposal.getPeace(), msg);
                	}
                }
            }
        }
        
		if (owner.getPlayerCharacter().isIsolationist()) {
			if (owner.getHulls() != null && owner.getHulls().hasDiscoveredStation()) {
				owner.setIsolationist(false);
			} else {
				owner.setIsolationist(true);
			}
		}

		int qtyMilitartyCrafts = owner.getQtyMilitartyCrafts();
		Craft colonyCraft = owner.getColonyCraft();

		if (playersAtWar == null && qtyMilitartyCrafts > 1 && colonyCraft != null && MotEngine.instance.isAHardLevel() && !owner.isIsolationist()) {
			CraftTask colonyCraftTask = colonyCraft.getTask();
			if (colonyCraftTask == null || colonyCraftTask.getClass() == CraftTaskIdle.class) {				
				Planet target = findPlanetToStealBySurpriseAttackColonisableBy(colonyCraft);
				int averageCraftDamage = owner.getMilitaryCraftAverageDamage();
				if (target != null && BattlePlanet.estimateQtyPopulationKilledByDamage(averageCraftDamage*2) > target.getPopulation()) {
					owner.getGeneral().createSurpriseAttackStealColony(colonyCraft, target);
				}
			}
		}
		
		// If we are not at war with anybody, declare war on the most hated player.
        // providing we have at least N military craft per planet and we are not at 
        // war with anybody else.
		if (mostHated.player != null && playersAtWar == null) {
			boolean wantToDeclareWar = false;
			if (owner.isIsolationist()) {
				// Will not declare war
			} else {
				int qtyPlanetsWithPopulation100 = owner.getQtyPlanetsWithPopulationBiggerThan(100);
				if (qtyPlanetsWithPopulation100 < 1) qtyPlanetsWithPopulation100 = 1;
				int qtyMilitartyCraftPerPlanet = qtyMilitartyCrafts / qtyPlanetsWithPopulation100;
				wantToDeclareWar = qtyMilitartyCraftPerPlanet >= pc.getQtyCraftPerPlanetToDeclareWar() && qtyMilitartyCrafts >=  pc.getQtyCraftToDeclareWar(); 
			}
            if (wantToDeclareWar) {
                double demandProbability = pc.getDemandProbability();
                if (Utils.random(demandProbability)) {
                	PlayerAssets assets = getAssetsToDemand(mostHated.player);
                    if (assets != null) {
                        String text = pc.getDiplomaticStatement(DiplomaticStatements.DEMAND_REQUEST);
                        mostHated.player.diplomaticRequest(owner, new DiplomaticProposal(null,assets), text);
                    } else {
                        String text = pc.getDiplomaticStatement(DiplomaticStatements.DECLARE_WAR);
                        mostHated.player.diplomaticRequest(owner, DiplomaticProposal.getWar(), text);
                    }
                } else {
                    String text = pc.getDiplomaticStatement(DiplomaticStatements.DECLARE_WAR);
                    mostHated.player.diplomaticRequest(owner, DiplomaticProposal.getWar(), text);
                }
            }
		}

		if (mostHated.player != null && mostLoved.player != null && !mostLoved.player.equals(mostHated.player)) {
            Relations rels = MotEngine.instance.getRelationsNotNull();
			if (mostLoved.rating > Relations.LIKE && !rels.isAliance(owner, mostLoved.player)) {
            	String msg = pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_ALLIANCE);
            	mostLoved.player.diplomaticRequest(owner, DiplomaticProposal.getAlliance(), msg);
			}
		}
	}

	private Planet findPlanetToStealBySurpriseAttackColonisableBy(Craft colonyCraft) throws ApplicationError {
		Player owner = getOwner();
		SolarSystem[] solarSystems = MotEngine.instance.getStellarObjectsNotNull().getSolarSystems();
		if (solarSystems == null) return null;
		for (int i = 0; i < solarSystems.length; i++) {
			if (colonyCraft.canMoveTo(solarSystems[i])) {				
				Planet[] colonies = solarSystems[i].getColonies(false);
				if (colonies != null && colonies.length == 1 && !solarSystems[i].containsMiningColony()) {
					Planet p = colonies[0];
					Player o = p.getOwner();
					if (!owner.isWar(o) && colonyCraft.canColonise(p) && p.getQtyWeapons() < 2) {
						return p;
					}
				}
			}
		}
		return null;
	}

	public StrategistResponse diplomaticRequest(Player player, DiplomaticProposal proposal) throws ApplicationError {
		if (proposal.getQtyTreaties() > 1) {
			return new StrategistResponse(false, "We will not consider a proposal with so many treaties right now");
		}
		if (proposal.isWar()) {
			Player owner = getOwner();
			notificationOfWar(player, owner);
	        PlayerCharacter pc = owner.getPlayerCharacter();
	        int rating = getRating(player);
	        String message;
	        if (rating >= Relations.NEUTRAL) {
	        	message = pc.getDiplomaticStatement(DiplomaticStatements.ACCEPT_WAR_DECLARATION_LIKE);
	        } else {
	        	message = pc.getDiplomaticStatement(DiplomaticStatements.ACCEPT_WAR_DECLARATION_DISLIKE);
	        }
			return new StrategistResponse(false, message);
		} else if (proposal.isPeace()) {
			return requestPeace(player, proposal);
		} else if (proposal.isNonAggression()) {
			return requestNonAgression(player, proposal);
		} else if (proposal.isAlliance()) {
			return requestAliance(player, proposal);
		} else if (proposal.isTrade()) {
			ComputerPlayer owner = (ComputerPlayer) getOwner();
			return requestTradeRoute(player, proposal);
		} else if (proposal.isResearch()) {
			ComputerPlayer owner = (ComputerPlayer) getOwner();
			return requestResearchTreaty(player, proposal);
		} else {
			PlayerAssets wanted = proposal.getWanted();
			PlayerAssets offered = proposal.getOffered();
			if (wanted != null && offered != null) {
				return requestSwapAssets(player, offered, wanted);
			} else if (wanted != null) {
				return demandAssets(player, wanted);
			} else if (offered != null) {
				return notifyGift(player, offered);
			} else {
				return new StrategistResponse(false, "This proposal has no substance");
			}
		}
	}
	
	public void notifyDiplomaticResponse(Player player, DiplomaticProposal proposal, boolean accepted) throws ApplicationError {
    	if (!accepted) {
            if (proposal.isDemand()) {
    	        String text = getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DECLARE_WAR_AFTER_REJECT_DEMAND);
    	        player.diplomaticRequest(getOwner(), DiplomaticProposal.getWar(), text);
            } else if (!proposal.isWar()){
                getEventManagerNotNull().add(new StrategistEventTreatyRejected(player, null));
            }
    	}
	}

	/**
	 * Called when another player has requested peace. Returns true iff the request is
	 * accepted and the relationship status is changed.
	 */
	public StrategistResponse requestPeace(Player player, DiplomaticProposal proposal) throws ApplicationError {
        Player owner = getOwner();
        PlayerCharacter pc = owner.getPlayerCharacter();
        int netAssetScore = getNetAssetScore(proposal);
        if (getEventManagerNotNull().hasEventTypeWithAssetScoreAbove(StrategistEventRequestPeaceTreaty.class, netAssetScore)) {
            getEventManager().add(new StrategistEventAnnoy(player));
            return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_REJECT_REPEATED_REQUEST));
        }
        Relation rel = owner.getRelationshipWith(player);
        if (rel == null)  return null;
        WarTreaty warTreaty = (WarTreaty) rel.getTreaty(WarTreaty.class);
        if (warTreaty == null) return null;
        if (warTreaty.getDateStarted() + MIN_AGE_OF_WAR_TO_ACCEPT_PEACE > MotEngine.instance.getDate()) {
            return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_REJECT_JUST_DECLARED_WAR));
        }
        getEventManagerNotNull().add(new StrategistEventRequestPeaceTreaty(player, netAssetScore));
		double probabilityOfPeace = pc.getAcceptPeaceProbability();
		Player [] playersAtWar = owner.getPlayersAtWar();
        int qtyWars = (playersAtWar == null) ? 0 : playersAtWar.length;
        int rating = getRating(player, proposal);
        if (rating > Relations.NEUTRAL && qtyWars >= 2) {
			MotEngine.instance.getRelations().setPeace(owner, player);
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_ACCEPT_TOO_MANY_WARS));
		} else if (rating > Relations.FOND && qtyWars >= 1) {
			MotEngine.instance.getRelations().setPeace(owner, player);
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_ACCEPT_LIKE_AND_TWO_WARS));
		} else if (rating > Relations.DISLIKE && Utils.random(probabilityOfPeace)) {
			MotEngine.instance.getRelations().setPeace(owner, player);
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_ACCEPT_NEUTRAL));
		} else if (rating > Relations.WOW) {
			MotEngine.instance.getRelations().setPeace(owner, player);
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_ACCEPT_NEUTRAL));
        }
		if (rating > Relations.NEUTRAL) {
			return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_REJECT_LIKE));
		} else if (rating < Relations.DISLIKE) {
			return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_REJECT_HATE));
		} else {
			return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_PEACE_REJECT_DISLIKE));
		}
	}
	/**
	 * Called when another player has requested an alliance. Returns true iff the request
	 * is accepted and the relationship status is changed.
	 */
	public StrategistResponse requestAliance(Player player, DiplomaticProposal proposal) throws ApplicationError {
        Player owner = getOwner();
        PlayerCharacter pc = owner.getPlayerCharacter();
        int netAssetScore = getNetAssetScore(proposal);
        if (getEventManagerNotNull().hasEventTypeWithAssetScoreAbove(StrategistEventRequestAllianceTreaty.class, netAssetScore)) {
            getEventManager().add(new StrategistEventAnnoy(player));
            return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_ALLIANCE_REJECT_REPEATED_REQUEST));
        }
        getEventManagerNotNull().add(new StrategistEventRequestAllianceTreaty(player, netAssetScore));
        int rating = getRating(player, proposal);
        if (rating > Relations.WOW) {
    		MotEngine.instance.getRelations().setAliance(owner, player);
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_ALLIANCE_ACCEPT));
        }
		if (rating > Relations.LIKE) {
			double probabilityOfAliance = pc.getAcceptAlianceProbabilityWhenLike();
			if (Utils.random(probabilityOfAliance)) {
				MotEngine.instance.getRelations().setAliance(owner, player);
                return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_ALLIANCE_ACCEPT));
			}
		} else if (rating > Relations.FOND) {
			double probabilityOfAliance = pc.getAcceptAlianceProbabilityWhenFond();
			if (Utils.random(probabilityOfAliance)) {
				MotEngine.instance.getRelations().setAliance(owner, player);
                return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_ALLIANCE_ACCEPT));
			}
		}
        return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_ALLIANCE_REJECT));
	}

	/**
	 * Called when another player has requested a non aggression pact. Returns true iff the request
	 * is accepted and the relationship status is changed.
	 */
	public StrategistResponse requestNonAgression(Player player, DiplomaticProposal proposal) throws ApplicationError {
        Player owner = getOwner();
        PlayerCharacter pc = owner.getPlayerCharacter();
        int netAssetScore = getNetAssetScore(proposal);
        if (getEventManagerNotNull().hasEventTypeWithAssetScoreAbove(StrategistEventRequestNonAgressionTreaty.class, netAssetScore)) {
            getEventManager().add(new StrategistEventAnnoy(player));
            return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_NONAGRESSION_REJECT_REPEATED_REQUEST));
        }
        getEventManagerNotNull().add(new StrategistEventRequestNonAgressionTreaty(player, netAssetScore));
        int rating = getRating(player, proposal);
        if (rating > Relations.WOW) {
            MotEngine.instance.getRelations().setNonAgression(owner, player);
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_NONAGRESSION_ACCEPT));
        }
		Player [] playersAtWar = owner.getPlayersAtWar();
        if (playersAtWar != null) {
            if (playersAtWar.length >= 2 || rating > Relations.FOND) {
                MotEngine.instance.getRelations().setNonAgression(owner, player);
                return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_NONAGRESSION_ACCEPT_TOO_MANY_WARS));
            }
            if (playersAtWar.length >= 1 || rating > Relations.LIKE) {
                if (Utils.random(0.5)) {
                    MotEngine.instance.getRelations().setNonAgression(owner, player);
                    return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_NONAGRESSION_ACCEPT));
                }
            }
        }
        if (rating > Relations.NEUTRAL) {
	        return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_NONAGRESSION_REJECT_LIKE));
        } else {
	        return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_NONAGRESSION_REJECT_DISLIKE));
        }
    }

    /**
     * Called when another player (passed in) has requested a trade treaty with this player.
     * This method returns true and creates the trade treaty if the offer is accepted otherwise 
     * it returns false and no action is taken.
     * @param player Player requesting a trade treaty.
     * @return True if and only if the trade agreement has been setup.
     * @throws mot.common.util.ApplicationError
     * @throws mot.common.util.ValidationError
     */
	public StrategistResponse requestTradeRoute(Player player, DiplomaticProposal proposal) throws ApplicationError {
        Player owner = getOwner();
        PlayerCharacter pc = owner.getPlayerCharacter();
        int netAssetScore = getNetAssetScore(proposal);
        StrategistEventManager em = getEventManagerNotNull();
        if (owner instanceof ComputerPlayer) {
            if (em.hasEventTypeWithAssetScoreAbove(StrategistEventRequestTradeTreaty.class, netAssetScore)) {
                em.add(new StrategistEventAnnoy(player));
                return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_TRADE_REJECT_REPEATED_REQUEST));
            }
        }

        StrategistResponse r = _requestTradeRoute(player); 
        if (!r.isAccepted()) {
            if (em != null) {
                em.add(new StrategistEventRequestTradeTreaty(player, netAssetScore));
            }
        }
        return r;
    }

	public StrategistResponse _requestTradeRoute(Player player) throws ApplicationError {
        Relation rel = MotEngine.instance.getRelations().getRelationshipNotNull(getOwner(), player);
        if (player instanceof ComputerPlayer && rel.getValue() > Relations.HATE) {
            return new StrategistResponse(true, null);
        }
        
        Player owner = getOwner();
        PlayerCharacter pc = owner.getPlayerCharacter();
        
        int rating;
        if (owner instanceof ComputerPlayer) {
            ComputerPlayer cp = (ComputerPlayer) owner;
            rating = cp.getStratagist().getRating(player, false);
        } else {
            Logger.logWarning("BankerBasic._requestTradeRoute(Player) Should never get here!");
            rating = rel.getRelationshipValue();
        }

        if (Logger.isDebug()) {
            Logger.logDebug("BankerBasic._requestTradeRoute() request trade route between "+owner.getName()+" and "+player.getName()+" rating="+rating);
        }
        
        if (rating > Relations.FOND) {
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_TRADE_ACCEPT_LIKE));
        }
        if (rating < Relations.DISTRUST) {
            return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_TRADE_REJECT_DISLIKE));
        }
        int rand = Utils.random(2);
        if (rand == 0) {
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_TRADE_ACCEPT));
        } else {
            return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_TRADE_REJECT));
        }
    }

	public StrategistResponse requestResearchTreaty(Player player, DiplomaticProposal proposal) throws ApplicationError {
        Player owner = getOwner();
        PlayerCharacter pc = owner.getPlayerCharacter();
        StrategistEventManager em = getEventManagerNotNull();
        int netAssetScore = getNetAssetScore(proposal);
        if (owner instanceof ComputerPlayer) {
            if (em.hasEventTypeWithAssetScoreAbove(StrategistEventRequestResearchTreaty.class, netAssetScore)) {
                em.add(new StrategistEventAnnoy(player));
                return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_RESEARCH_REJECT_REPEATED_REQUEST));
            }
        }

        StrategistResponse r = _requestResearchTreaty(player); 
        if (!r.isAccepted()) {
            if (em != null) {
                em.add(new StrategistEventRequestResearchTreaty(player, netAssetScore));
            }
        }
        return r;
	}


	private StrategistResponse _requestResearchTreaty(Player player) throws ApplicationError {
        Relation rel = MotEngine.instance.getRelations().getRelationshipNotNull(getOwner(), player);
        if (player instanceof ComputerPlayer && rel.getValue() > Relations.HATE) {
            return new StrategistResponse(true, null);
        }
        
        Player owner = getOwner();
        PlayerCharacter pc = owner.getPlayerCharacter();
        
        int rating;
        if (owner instanceof ComputerPlayer) {
            ComputerPlayer cp = (ComputerPlayer) owner;
            rating = cp.getStratagist().getRating(player, false);
        } else {
            Logger.logWarning("ScientistBasic._requestResearchTreaty(Player) Should never get here!");
            rating = rel.getRelationshipValue();
        }

        if (Logger.isDebug()) {
            Logger.logDebug("ScientistBasic._requestResearchTreaty() request research treaty between "+owner.getName()+" and "+player.getName()+" rating="+rating);
        }
        
        if (rating > Relations.FOND) {
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_RESEARCH_ACCEPT_LIKE));
        }
        if (rating < Relations.DISTRUST) {
            return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_RESEARCH_REJECT_DISLIKE));
        }
        int rand = Utils.random(2);
        if (rand == 0) {
            return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_RESEARCH_ACCEPT));
        } else {
            return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.REQUEST_RESEARCH_REJECT));
        }
	}


    /**
     * Informs the stratagist of a war declaration so it can modify its relationships. 
     * @throws mot.common.util.ApplicationError
     * @param victim The player who the aggressor has declared war on
     * @param aggressor The aggressor who has declared the war
     */
	public void notificationOfWar(Player agressor, Player victim) throws ApplicationError {
        Player owner = getOwner();
        if (agressor == owner) {
            // nothing - It is me declaring war!
        } else if (victim == owner) {
            getEventManagerNotNull().add(new StrategistEventDeclareWar(agressor));
        } else {
            if (owner.isWar(victim)) {
                getEventManagerNotNull().add(new StrategistEventDeclareWarOnEnemy(agressor, victim));
            }
        }
    }

    private static final float BAD_SWAP = 0.85f; 
    private static final float GOOD_SWAP = 1.2f; 
    private static final float EXCELENT_SWAP = 1.5f; 
    private static final float MADLY_GOOD_SWAP = 2.0f; 

	public StrategistResponse requestSwapAssets(Player player, PlayerAssets offered, PlayerAssets wanted) throws ApplicationError {
        Relation rel = MotEngine.getInstance().getRelations().getRelationshipNotNull(getOwner(), player);
        Player owner = getOwner();
        PlayerCharacter pc = owner.getPlayerCharacter();
        if (!rel.isWar()) {
            int rating = getRating(player);
            int offeredScore = getAssetScore(offered, true);
            int wantedScore = getAssetScore(wanted, false);
            StrategistEventRejectSwapAssets lastEvent = (StrategistEventRejectSwapAssets) getEventManagerNotNull().getLastDiplomaticEvent(player, StrategistEventRejectSwapAssets.class);
            if (lastEvent != null && lastEvent.getWantedAssetScore() == wantedScore && lastEvent.getOfferedAssetScore() == offeredScore) {
                getEventManagerNotNull().add(new StrategistEventAnnoy(player));
    	        String statement = pc.getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_REJECT_REPEATED);
    	        return new StrategistResponse(false, statement);
            }
            if (rating >= Relations.LIKE) {
                if (offeredScore >= wantedScore) {
                	if (Utils.random(pc.getAcceptGoodSwapProbabilityWhenLike())) {
                		return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_ACCEPT_LIKE_GOOD_SWAP));
                	}
                } else if (offeredScore >= ((int)wantedScore*BAD_SWAP)) {
                	if (Utils.random(pc.getAcceptBadSwapProbabilityWhenLike())) {
                		return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_ACCEPT_LIKE_OK_SWAP));
                	}
                }
            } else if (rating >= Relations.NEUTRAL) {
                if (offeredScore >= ((int)wantedScore*GOOD_SWAP)) {
                	if (Utils.random(pc.getAcceptGoodSwapProbabilityWhenNeutral())) {
                		return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_ACCEPT_NEUTRAL_GOOD_SWAP));
                	}
                } else if (offeredScore >= wantedScore*BAD_SWAP) {
                    if (Utils.random(pc.getAcceptBadSwapProbabilityWhenNeutral())) {
                        return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_ACCEPT_NEUTRAL_OK_SWAP));
                    }
                }
            } else {
                if (offeredScore >= ((int)wantedScore*EXCELENT_SWAP)) {
                	if (Utils.random(pc.getAcceptGoodSwapProbabilityWhenDislike())) {
                		return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_ACCEPT_DISLIKE_EXCELENT_SWAP));
                	}
                } else if (offeredScore >= ((int)wantedScore)) {
                    if (Utils.random(pc.getAcceptBadSwapProbabilityWhenDislike())) {
                        return new StrategistResponse(true, pc.getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_ACCEPT_DISLIKE_EXCELENT_SWAP));
                    }
                }
            }
            getEventManagerNotNull().add(new StrategistEventRejectSwapAssets(player, wantedScore, offeredScore));
	        String statement = getRejectSwapStatement(rel, offered, offeredScore >= ((int)wantedScore*GOOD_SWAP));
	        return new StrategistResponse(false, statement);
        } else {
	        return new StrategistResponse(false, pc.getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_REJECT_WAR));
        }
    }

    private String getRejectSwapStatement(Relation rel, PlayerAssets offered, boolean good) throws ApplicationError {
    	TechnologyList technologList = offered.getTechnologList();
		if (technologList != null && getOwner().hasDiscovered(technologList)) {
            return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_REJECT_OFFERD_OLD_TECHS);
    	}
    	if (good) {
    		if (rel.getValue() <=  Relations.NEUTRAL) {
                return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_REJECT_DISLIKE_GOOD_SWAP);
    		} else {
                return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_REJECT_LIKE_GOOD_SWAP);
    		}
    	} else {
            return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.SWAP_RESPONSE_REJECT_BAD_SWAP);
    	}
    }
    
    public StrategistResponse demandAssets(Player player, PlayerAssets wanted) throws ApplicationError {
        StrategistEventManager eventManager = getEventManagerNotNull();
        if (eventManager.hasEventType(StrategistEventDemandAssets.class)) {
        	eventManager.add(new StrategistEventAnnoy(player));
            String statement = getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DEMAND_RESPONSE_REPEATED_DEMAND);
            return new StrategistResponse(false, statement);
        }
        
        Player owner = getOwner();
        Relation rel = MotEngine.getInstance().getRelations().getRelationshipNotNull(owner, player);
        int wantedScore = getAssetScore(wanted, false);
        ResearchProject rp = owner.getResearchProject();
        int researchLevel = (rp == null) ? 1000 : rp.getTechnology().getCostToResearch();
        
        boolean accept = false;
        if (rel.getValue() >= Relations.LIKE && wantedScore <= researchLevel) {
            accept = true;
        } else if (rel.getValue() >= Relations.LOVE && wantedScore <= researchLevel*3) {
            accept = true;
        }
        
        if (wanted.getPlanet() != null) accept = false;
        
        if (!accept) {
            int otherPower = player.getMilitaryCraftPower();
            int myPower = owner.getMilitaryCraftPower();
            if (owner.getWeapons() != null) {
                Weapon bestWeapon = owner.getWeapons().getMaxDamageAttackWeaponWithInfinateShots();
                if (bestWeapon != null) myPower += bestWeapon.getDamage() * owner.getQtyPlanets() * 2;
            }
            if (myPower < otherPower/2 && wantedScore < researchLevel*2) {
                accept = true;
            } else if (myPower < otherPower/4 && wantedScore < researchLevel*4) {
                accept = true;
            }
        }
        
        if (wanted.getPlanet() != null) {
        	Planet planet = MotEngine.instance.getPlanets().getPlanet(wanted.getPlanet());
        	if (planet != null && planet.getSize() > 40)  accept = false;
        }
        
        if (accept) {
            eventManager.add(new StrategistEventDemandAssets(player, wantedScore, true, wanted.getDescription()));
            rel.setValue((int)(rel.getValue() - DEMAND_EFFECT_ACCEPTED));        
        } else {
            eventManager.add(new StrategistEventDemandAssets(player, wantedScore, false, wanted.getDescription()));
            rel.setValue((int)(rel.getValue() - DEMAND_EFFECT_REJECTED));        
        }
        
        return new StrategistResponse(accept, getDemandStatement(rel, accept,  wantedScore));
    }

    private String getDemandStatement(Relation rel, boolean accepted, int value) throws ApplicationError {
    	if (accepted) {
    		if (rel.getValue() <=  Relations.NEUTRAL) {
                return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DEMAND_RESPONSE_DISLIKE_ACCEPTED);
    		} else {
                return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DEMAND_RESPONSE_LIKE_ACCEPTED);
    		}
    	} else {
    		if (rel.getValue() <=  Relations.NEUTRAL) {
                return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DEMAND_RESPONSE_DISLIKE_REJECTED);
    		} else {
                return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DEMAND_RESPONSE_LIKE_REJECTED);
    		}
    	}
    }
	
    public void notifyDemandAssetsRejected(Player player, PlayerAssets wanted) throws ApplicationError {
        String text = getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DECLARE_WAR_AFTER_REJECT_DEMAND);
        player.diplomaticRequest(getOwner(), DiplomaticProposal.getWar(), text);
    }

    /**
     * @return 0.0 => same value. 0.5 => wants twice as much given. 2.0 => giving twice as much as want
     */
    private double getAssetScoreRatio(DiplomaticProposal proposal, double minLevel) throws ApplicationError {
        double offeredScore = getAssetScore(proposal.getOffered(), true) / minLevel;
        double wantedScore = getAssetScore(proposal.getWanted(), false) / minLevel;
        if (wantedScore == 0.0) return offeredScore;
    	return offeredScore / wantedScore;
    }

    public double getAssetScoreRatio(DiplomaticProposal proposal) throws ApplicationError {
    	Technology mostAdvanced = getOwner().getHulls().getMostExpensiveToResearch();
		int minLevel = mostAdvanced.getCostToResearch(); 
		return getAssetScoreRatio(proposal, minLevel);
    }

    public int getAssetScore(PlayerAssets assets, boolean offered) throws ApplicationError {
        if (assets == null) return 0;
        int score = 0;
        Player owner = getOwner();
        if (offered) {
        	TechnologyList technologies = assets.getTechnologList();
        	if (technologies != null && technologies.getQuantity() > 0) {
        		TechnologyList missedTechs = owner.getPlayerCharacter().getMissedTechnologies();
        		Technology[] techs = technologies.getAllAsArray();
        		for (int i = 0; i < techs.length; i++) {
        			if (owner.hasDiscovered(techs[i])) {
        				// No change in score
        			} else if (missedTechs != null && missedTechs.contains(techs[i])) {
        				score += technologies.getCostToResearch()*2;
        			} else {					
        				score += technologies.getCostToResearch();
        			}
        		}
        	}
        } else {
        	TechnologyList technologies = assets.getTechnologList();
        	if (technologies != null && technologies.getQuantity() > 0) {
        		score += technologies.getCostToResearch();
        	}
        }
        if (assets.getMoney() > 0) {
            score += assets.getMoney() / 2;
        }
        String planetName = assets.getPlanet();
		if (planetName != null) {
			Planet planet = MotEngine.instance.getPlanets().getPlanet(planetName);
            int pq = planet.getQualityScore();
            ResearchProject rpOwner = owner.getResearchProject();
            int researchLevel = (rpOwner == null) ? 1000 : rpOwner.getTechnology().getCostToResearch();
            if (pq >= 100) {
                score += researchLevel * 16;
            } else if (pq >= 80) {
                score += researchLevel * 12;
            } else if (pq >= 50) {
                score += researchLevel * 3;
            } else {
                score += researchLevel;
            }
            OrbitalReference miningStation = planet.getMiningStation();
			if (miningStation != null && miningStation.getPlanet() != null) {
				pq = miningStation.getPlanet().getQualityScore();
	            if (pq >= 100) {
	                score += researchLevel * 10;
	            } else {
	                score += researchLevel * 5;
	            }
			}
        }
        NameList craftNames = assets.getCraftNames();
        if (craftNames != null && craftNames.getQuantity() > 0) {
        	int myQty = owner.getQtyMilitartyCrafts();
        	int qtyAttacking = owner.getQtyEnemyCraftAttacking();
        	String[] cna = craftNames.getNamesAsArray();
        	for (int i = 0; i < cna.length; i++) {
        		Craft craft = MotEngine.instance.getCrafts().getCraft(cna[i]);
        		if (craft != null) {
        			int value = craft.getValue();
        			if (myQty == 0 || myQty < qtyAttacking) {
        				score += value;
        			} else {
        				score += value / 2;
        			}
        		}
        	}
        }
        return score;
    }

	private int getNetAssetScore(DiplomaticProposal proposal) throws ApplicationError {
		PlayerAssets offered = proposal.getOffered();
		PlayerAssets wanted = proposal.getWanted();
		if (offered != null && wanted != null) {
			int os = getAssetScore(offered, true);
			int ws = getAssetScore(wanted, false);
			return os - ws;
		} else if (offered != null) {
			int os = getAssetScore(offered, true);
			return os;
		} else if (wanted != null) {
			int ws = getAssetScore(wanted, false);
			return -ws;
		} else {
			return 0;
		}
	}


	/**
     * Called when another player has been caught spying - Reduces the relationship 
     * by SPYING_EFFECT!
     * @param player The player caught spying
     */
	public void notifySpying(Player player, Technology tech) throws ApplicationError {
        ratings = null;
        ComputerPlayer owner = (ComputerPlayer) getOwner();
		Relation rel = MotEngine.getInstance().getRelations().getRelationship(owner, player);
        if (rel != null) {
            rel.setValue(rel.getValue() - SPYING_EFFECT);
        } else {
            Logger.logWarning("StrategistBasic.notifySpying() No relatiohship between "+owner.getName()+" and "+player.getName());
        }
        getEventManagerNotNull().add(new StrategistEventCaughtSpying(player, tech));
        if (owner.getFunds() > 1000) {
        	if (MotEngine.instance.isAHardLevel()) {
        		owner.addQtyAditionalSpies(2);
        	} else if (MotEngine.instance.isLevelNormal()) {
            	owner.addQtyAditionalSpies(1);
        	}
        }
    }

	public void notifyBreakNonAggression(Player player) {
        getEventManagerNotNull().add(new StrategistEventBrokeNonAggression(player, TreatyType.NON_AGGRESSION));
	}

	public void notifyBreakAlliance(Player player) {
        getEventManagerNotNull().add(new StrategistEventBrokeNonAggression(player, TreatyType.ALLIANCE));
	}

	public void notifyUseOfBiologicalWeapons(Player player) throws ApplicationError {
        StrategistEventManager em = getEventManagerNotNull();
        if (em.getLastDiplomaticEvent(player, StrategistEventUsedBiologicalWeapons.class) == null) {
        	em.add(new StrategistEventUsedBiologicalWeapons(player));
        }
	}

	public void notifyRelationsEstablished(Player player, boolean starting) throws ApplicationError {
        StrategistEventManager em = getEventManagerNotNull();
        if (em.getLastDiplomaticEvent(player, StrategistEventRelationsEstablished.class) == null) {
        	em.add(new StrategistEventRelationsEstablished(player, starting));
        }
	}

	public void notifyEmpireEstablished() throws ApplicationError {
        StrategistEventManager em = getEventManagerNotNull();
       	em.add(new StrategistEventEmpireEstablished(getOwner()));
	}

	/**
     * Called when another player has been caught sabotaging installations - Reduces 
     * the relationship by SABOTAGE_EFFECT and declares war if we are not already at 
     * war.
     * @param player The player caught sabotaging
     */
	public void notifySabotage(Player player) throws ApplicationError {
        ratings = null;
        Player owner = getOwner();
        Relation rel = MotEngine.getInstance().getRelations().getRelationship(owner, player);
        if (rel != null) {
            rel.setValue(rel.getValue() - SABOTAGE_EFFECT);
            if (!MotEngine.getInstance().getRelations().isWar(owner, player)) {
                String text = owner.getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.DECLARE_WAR_AFTER_SABOTAGE);
                player.diplomaticRequest(getOwner(), DiplomaticProposal.getWar(), text);
            }
        } else {
            Logger.logWarning("StrategistBasic.notifySpying() No relatiohship between "+getOwner().getName()+" and "+player.getName());
        }
        getEventManagerNotNull().add(new StrategistEventCaughtSabotaging(player));
    }
    
    /**
     * Called when Player player gives a monetary gift of amount to this player.
     * This method credits this player with the amount and modifies the relationship.
     * NOTE - The offered technologies list MUST NOT included any technologies already discovered!
     * @param Player The player giving the gift
     * @param amount The amount of money given
     */
    public StrategistResponse notifyGift(Player player, PlayerAssets offered) throws ApplicationError {
        ratings = null;
        Player owner = getOwner();
        Relation rel = MotEngine.getInstance().getRelations().getRelationshipNotNull(owner, player);
        int value = 0;
        if (Logger.isDebug()) {
            Logger.logDebug("StratagistBasic.notifyGift() player "+getOwner().getName()+" given "+offered.toString());
        }
        int amount = offered.getMoney();
        if (amount > 0) {
        	value += (amount/1000)*GIFT_MONEY_EFFECT_PER_1000;
        }
        TechnologyList techList = offered.getTechnologList();
        if (techList != null && techList.getQuantity() > 0) {
            Technology [] techs = techList.getAllAsArray();
            ResearchProject rp = owner.getResearchProject();
            int myBestTechResearchCost = owner.getTechnologies().getHighestResearchValueBelow();
            for (int i=0; i<techs.length; i++) {
            	value += getGiftEffect(player, techs[i], myBestTechResearchCost, rel);
            }
        }
        String planetName = offered.getPlanet();
        if (planetName != null) {
			Planet planet = MotEngine.instance.getPlanets().getPlanet(planetName);
            int score = planet.getQualityScore();
            if (score >= 100) {
            	value += GIFT_PLANET_LARGE_EFFECT;
            } else if (score >= 80) {
            	value += GIFT_PLANET_MEDIUM_EFFECT;
            } else if (score > 50) {
            	value += GIFT_PLANET_SMALL_EFFECT;
            } else {
            	value += GIFT_PLANET_VSMALL_EFFECT;
            }
        }
        NameList craftNames = offered.getCraftNames();
        if (craftNames != null && craftNames.getQuantity() > 0) {
        	int myQty = owner.getQtyMilitartyCrafts();
        	int qtyAttacking = owner.getQtyEnemyCraftAttacking();
        	String[] cna = craftNames.getNamesAsArray();
        	for (int i = 0; i < cna.length; i++) {
        		Craft craft = MotEngine.instance.getCrafts().getCraft(cna[i]);
        		if (craft != null) {
        			int v = craft.getValue();
        			if (myQty == 0 || myQty < qtyAttacking) {
        				value += (v/1000.0)*GIFT_CRAFT_EFFECT_PER_1000*2;
        			} else {
        				value += (v/1000.0)*GIFT_CRAFT_EFFECT_PER_1000;
        			}
        			if (value > 20) value = 5;
        			else if (value > 15) value = 4;
        			else if (value > 10) value = 3;
        			else if (value > 5) value = 2;
        		}
        	}
        }
        String assetDescription = offered.getDescription();
       	boolean annoying = addGiftRelationEffect(player, rel, value, assetDescription);
        return new StrategistResponse(true, getGiftStatement(rel, value, annoying));
    }

    /**
     * Called when Player player gives a technology gift of tech to this player.
     * This method add this tech to this players list  and modifies the relationship.
     * @param Player The player giving the gift
     * @param tech the technology given
     */
    private int getGiftEffect(Player player, Technology tech, int myBestTechResearchCost, Relation rel) throws ApplicationError {
    	if (getOwner().hasDiscovered(tech)) {
    		return 0;
    	}
        if (tech.getCostToResearch() > myBestTechResearchCost) {
            return GIFT_TECHNOLOGY_EFFECT_LARGE;
        } else {
        	return GIFT_TECHNOLOGY_EFFECT_SMALL;
        }
    }
    
    private boolean addGiftRelationEffect(Player player, Relation rel, int value, String assetDescription) throws ApplicationError {
        rel.setValue((int)(rel.getValue() + value));
        boolean annoyingMe = false;
        if (value == 0) {
        	StratagistEvent lastEvent = getEventManagerNotNull().getLastDiplomaticEvent(player, StrategistEventGiveAssets.class);
        	if (lastEvent != null) {
        		StrategistEventGiveAssets sega = (StrategistEventGiveAssets) lastEvent;
                // Giving technology I have already discovered should not annoy me
                // and the player does not know that I have discovered.
        		// annoyingMe = (sega.getRelationEffect() == 0);
        	}
        }
        getEventManagerNotNull().add(new StrategistEventGiveAssets(player, value, assetDescription));
        if (annoyingMe) {
            getEventManagerNotNull().add(new StrategistEventAnnoy(player));
        }
        return annoyingMe;
    }
    
    private String getGiftStatement(Relation rel, int value, boolean annoying) throws ApplicationError {
    	if (value <= 0) {
    		if (annoying) {
	            return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.GIFT_THANKYOU_NO_REPEATED_VALUE);
    		} else {
	            return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.GIFT_THANKYOU_NO_VALUE);
    		}
    	} else if (value < GIFT_TECHNOLOGY_EFFECT_LARGE) {
    		if (rel.getValue() <=  Relations.DISTRUST) {
                return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.GIFT_THANKYOU_SMALL_HATE);
    		} else {
                return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.GIFT_THANKYOU_SMALL);
    		}
    	} else {
            return getOwner().getPlayerCharacter().getDiplomaticStatement(DiplomaticStatements.GIFT_THANKYOU_LARGE);
    	}
    }
    
    /**
     * Called when a battle has ended that contained this player - Allows the AI to attempt to
     * make peace because we are losing or reject peace because we are winning.
     * @param event The end battle event.
     */
    public void notiftBattleEnd(Battle battle) throws ApplicationError {
        Relations rels = MotEngine.getInstance().getRelations();
        Player [] players = battle.getStartingPlayers();
        if (players == null) return;
        boolean winner = battle.isWinner(getOwner());
        for (int i=0; i<players.length; i++) {
            if (players[i] != getOwner()) {
                if (rels.isWar(getOwner(), players[i])) {
                    Relation rel = rels.getRelationship(getOwner(), players[i]);
                    if (rel != null) {
                        if (winner) {
                            if (rel.getValue() > Relations.HATE) {
                                rel.setValue((int)(rel.getValue() - BATTLE_EFFECT));
                            }
                        } else {
                            if (rel.getValue() < Relations.NEUTRAL) {
                                rel.setValue((int)(rel.getValue() + BATTLE_EFFECT/2));
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Called when this player has lost a planet to the player aggressor - Than may be an invasion
     * or a bombardment so the colony may still exist but owned by another player or the planet may
     * no longer have any population.
     * 
     * If the aggressor is more than twice as powerful as this player then this player looks for
     * another player to surrender to.
     * 
     * @param colony The planet that has been attacked.
     * @param aggressor The player who has attacked our colony.
     * @param genocide True if it was due to bombardment.
     * @throws mot.common.util.ApplicationError
     */
	public void notifyColonyLost(Planet colony, Player aggressor, boolean genocide) throws ApplicationError {
        if (aggressor instanceof MonsterPlayer) {
            return;
        }
    
        getEventManagerNotNull().add(new StrategistEventColonyLost(aggressor, colony.getName(), genocide));

        Player owner = getOwner();
        long myPower = owner.getPowerRating(); 
        long enemyPower = aggressor.getPowerRating();
        
        if (enemyPower > myPower*2 && MotEngine.instance.isAHardLevel()) {
        	surrenderToAtEndTurn = getBestPlayerToSurrenderTo(aggressor);
        	if (surrenderToAtEndTurn != null) {
        		long surrenderToRating = surrenderToAtEndTurn.getPowerRating();
        		if (myPower > surrenderToRating*1.5) {
        			// Do not surrender to smaller power because that helps my enemy
        			surrenderToAtEndTurn = null;
        		} else {
        			if (Logger.isDebug()) {
        				Logger.logDebug("StrategistBasic.notifyColonyLost() Player "+owner.getName()+" has lost colony to "+aggressor.getName()+" so will surrender to "+surrenderToAtEndTurn.getName());
        			}
        		}
        	}
        }
    }
    
	/**
	 * Called when another player has lost a colony from bombardment.
	 */
	public void notifyOfGenocide(Planet planet, Player aggressor, Player victim) throws ApplicationError {
		Player owner = getOwner();
		if (owner != null && !owner.isEnemy(victim)) {
			getEventManagerNotNull().add(new StrategistEventGenocide(aggressor, planet.getName()));
		}
	}
	
	/**
	 * Called when another player has lost a colony from invasion.
	 */
	public void notifyOfInvasion(Planet planet, Player aggressor, Player victim) throws ApplicationError {
		Player owner = getOwner();
		String victimName = victim == null ? null : victim.getName();
        getEventManagerNotNull().add(new StrategistEventInvasion(owner, aggressor, planet.getName(), victimName));
	}

    /**
     * @return The player this player would surrender to if any.
     */
    private Player getBestPlayerToSurrenderTo(Player exclude) throws ApplicationError {
        Player best = null;
        long bestScore = Long.MAX_VALUE;
        Player owner = getOwner();
        long myPower = owner.getPowerRating(); 
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        Relations rels = MotEngine.instance.getRelationsNotNull();
		for (int i=0; i<players.length; i++) {
            Player p = players[i];
            if (p != exclude && p != owner) {
            	long score = p.getPowerRating();
                if (score < bestScore) {
                    bestScore = score;
                    best = p;
                }
            }
        }
        return best;
    }
    
	protected void _endTurn() throws ApplicationError, ValidationError {
		if (surrenderToAtEndTurn != null) {
            Player owner = getOwner();
        	if (Logger.isDebug()) {
        		Logger.logDebug("StrategistBasic._endTurn() Player "+owner.getName()+" surrender to "+surrenderToAtEndTurn.getName());
        	}
        	owner.surrendetTo(surrenderToAtEndTurn);
        	surrenderToAtEndTurn = null;
		}
	}


    /**
     * This is used to cache rating calculations so the same calculation is never made twice.
     * This is flushed periodically though so new ratings can be calculated.
     */
    private transient HashMap ratings;
    
    public void flushCache() { ratings = null;  }

	/**
	 * Returns the power rating for the player based upon;
	 * 1 - The relationship between these two players (defined in Relations)
	 * 2 - Power (tend to like players of about the same size and hate players who are bigger).
	 * 3 - Any resources this player covets belonging to a player.
     * 4 - If we have peace, alliance on non agression treaties then we rate this player higher.
     * NOTE - Trade relationships also improve the relationship but this is done by the Banker.
	 */
	public int getRating(Player player, boolean flushCache) throws ApplicationError {
        if (flushCache) {
            ratings = null;
        }
        return getRating(player);
    }

	private int getRating(Player player, DiplomaticProposal proposal) throws ApplicationError {
        int rating = getRating(player);
        double assetScoreRatio = getAssetScoreRatio(proposal);
        if (assetScoreRatio > 3.0) rating += 20;
        if (assetScoreRatio > 2.0) rating += 10;
        if (assetScoreRatio > 1.5) rating += 5;
        return rating;
	}
	
    private static final int RATING_CHANGE_ALLIANCE = 15;
    private static final int RATING_CHANGE_PEACE = 10;
    private static final int RATING_CHANGE_NON_AGRESSION = 5;
    private static final int RATING_CHANGE_WAR = 20;
    
    /**
     * Implementation of getRating()
     */
	private int getRating(Player player) throws ApplicationError {
        if (ratings == null) ratings = new HashMap();
        Integer ratingInteger = (Integer) ratings.get(player.getName());
        if (ratingInteger != null) {
            return ratingInteger.intValue();
        }
    
		Relation relation = MotEngine.getInstance().getRelationsNotNull().getRelationshipNotNull(getOwner(), player);
		int rating = relation.getRelationshipValue();
        
        // For debugging purposes to allow a player to always accept.
        if (rating > Relations.WOW) return rating;

		// Use power rating difference of players as a percent to modify relationship.
        // If a player is twice as big as me I reduce his rating by 0.12
		long powerOther = player.getPowerRating();
		long powerMine = getOwner().getPowerRating();
        if (powerMine >= 100) {
            double percentDifference = ((double)powerOther) / ((double)powerMine);
            if (percentDifference >= 1.5) {
                // other player is more powerful than me
                rating -= (int) ( ((double)rating) * Comp.min(1.0, percentDifference-1.5)/4.0 );
            } else if (percentDifference <= 0.75) {
                // other player is less powerful than me
                rating -= (int) ( ((double)rating) * (0.75-percentDifference)/4.0 );
            }
        }

        Planet mostCovetted = getMostCovettedPlanet();
        if (mostCovetted != null && mostCovetted.getOwner() != null) {
            Player grudge = mostCovetted.getOwner();
            if (grudge == player) {
                rating -= 9;
            }
        }
        
        // It is very bad to incur the wrath of the ancients so I will not declare war on the
        // same alignment unless I am powerful.
        String ownerAlign = getOwner().getAlignment();
        if ( ownerAlign != null && Comp.same(ownerAlign, player.getAlignment()) ) {
            int qtyMilitaryCrafts = getOwner().getQtyMilitartyCrafts();
            if (qtyMilitaryCrafts < MIN_MILITARY_CRAFTS_TO_FACE_ANCIENTS) {
                rating += FEAR_OF_ANCIENTS_BONUS;
            }
        }
        
        StrategistEventManager sem = getEventManager();
        if (sem != null) {
            rating += sem.getRelationshipEffect(player);
        }
        
        if (relation.containsTreaty(NonAgressionTreaty.class)) {
            rating += RATING_CHANGE_NON_AGRESSION;
        }
        if (relation.containsTreaty(PeaceTreaty.class)) {
            rating += RATING_CHANGE_PEACE;
        }
        if (relation.containsTreaty(AllianceTreaty.class)) {
            rating += RATING_CHANGE_ALLIANCE;
        }
        
        if (rating < 0) rating = 0;
        if (rating > 100) rating = 100;

//		if (Logger.isDebug()) {
//            Logger.logDebug("Stratagist.getRating() Relationship rating between players "+getOwner().getName()+" and "+player.getName()+" is "+rating);
//        }

        // Add this rating to our cache
        ratings.put(player.getName(), new Integer(rating));
        
		return rating;
	}
    
    /**
     * This method returns a planet with an owner (that is not this player) that this
     * player would most like to own or null if there is no such planet.
     * A Planet is rated by its quality score (see planet for details) and their distance from
     * the home world. Better quality scores closer to my home world are more desirable.
     * @return The planet this strategist would most like to take from another player.
     */
    public Planet getMostCovettedPlanet() throws ApplicationError {
        Planet best = null;
        int bestScore = 0;
        Planet [] planets = MotEngine.instance.getPlanets().getPlanets();
        if (planets == null) {
            return null;
        }
        Player owner = getOwner();
        Planet myHome = owner.getCapitol();
        for (int i=0; i<planets.length; i++) {
            if (planets[i].getOwner() == null || planets[i].getOwner() == owner) {
                continue;
            }
            if (owner.canTravelTo(planets[i].getStellarObject())) {
                int score = planets[i].getQualityScore();
                if (myHome != null) {
                    int dist = myHome.getStellarObject().getDistanceTo(planets[i].getStellarObject().getLocation());
                    score -= dist / 20;
                }
                if (best == null || score > bestScore) {
                    bestScore = score;
                    best = planets[i];
                }
            }
        }
        return best;
    }
    
    private static final int MIN_FUNDS_TO_DEMAND = 100;
    
    /**
     * @return Assets that this strategist would recommend demanding from player passed in, 
     * or null if no recommendation is to be made. 
     */
    private PlayerAssets getAssetsToDemand(Player player) throws ApplicationError {
        PlayerAssets assets = null;
        Player owner = getOwner();
        TechnologyList techs = player.getStealableTechnologies(owner);
        if (techs != null && techs.getQuantity() > 0) {
            int index = Utils.random(techs.getQuantity());
            Technology tech = techs.getTechnologyAtIndex(index);
            assets = new PlayerAssets(tech);
        }
        if (player.getFunds() > owner.getFunds() && player.getFunds() > MIN_FUNDS_TO_DEMAND) {
            if (assets == null) assets = new PlayerAssets();
            assets.setMoney((int)(player.getFunds() - owner.getFunds()));
        }
        return assets;
    }

    public PlayerAssets _getAssetsToDemand(Player player) throws ApplicationError {
        return getAssetsToDemand(player);
    }

	protected String getElementName() { return "stratagist-basic"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
		super.encode(writer);
        writer.addElement("date-of-last-calculation", dateOfLastCalculation);
	}

}


 