/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.strategists;

import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.players.Player;
import mot.engine.players.PlayerReference;
import mot.engine.relations.Relation;

/**
 * This is a base class for all types of events that effect a players relationship.
 * @see StrategistEventManager for details.
 */
public abstract class StratagistEvent extends XMLBean {
    
	protected static final int NEVER_EXPIRE = 99999999; 
	  
    /**
     * This is the date this event occurred - Note it is a decimal as several things can occur in a single year and we must maintain order
	 */
	private double date;
	  
    /**
     * This is the date this event will expire at. Events that have expired can be 
     * removed because they no longer effect a players rating.
     */
    private int expiryDate;
    
    /**
     * This is the player who owns the strategist - Will always be a ComputerPlayer
     */
    private PlayerReference ownerReference;

    /**
     * This is the player involved in this diplomatic event.
     * For a treaty request it would be the player that requested the treaty.
     */
    private PlayerReference playerReference;

    protected StratagistEvent() { }
    
    protected StratagistEvent(Player owner, Player player, int expiryDate) {
		if (owner != null) {
            this.ownerReference = new PlayerReference(owner);
        }
		if (player != null) {
            this.playerReference = new PlayerReference(player);
        }
		this.date = Relation.getNextDiplomaticHistoryEventDate();
        this.expiryDate = expiryDate;
    }
    
    protected StratagistEvent(Player player, int expiryDate) {
		if (player != null) {
            this.playerReference = new PlayerReference(player);
        }
		this.date = Relation.getNextDiplomaticHistoryEventDate();
        this.expiryDate = expiryDate;
    }
    
    public boolean hasExpired() { return expiryDate <= MotEngine.instance.getDate(); }
    
    /**
     * @return The value which this event will effect the relationship to the player
     * parameter. If the value of positive this represents a improvement in the 
     * relationship while a negative value indicates a worsening of the relationship.
     */
    public int getRelationshipEffect(Player player) throws ApplicationError {
        if (playerReference != null && playerReference.getPlayer() == player) {
            return getRelationshipEffect();
        } else {
            return 0;
        }
    }

    /**
     * This is a default implementation that returns 0 (no effect). Sub classes that
     * modify the relationship should override this method.
     * @return The value which this event will effect the relationship to the player
     * parameter. If the value of positive this represents a improvement in the 
     * relationship while a negative value indicates a worsening of the relationship.
     * @throws ApplicationError 
     */
    protected int getRelationshipEffect() throws ApplicationError { return 0; }
    
    public void setExpiryDate(int expiryDate) {
        this.expiryDate = expiryDate;
    }

    public int getExpiryDate() {
        return expiryDate;
    }

    protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("date", date);
        writer.addElement("expiry-date", expiryDate);
        if (playerReference != null) {
            playerReference.toXML(writer);
        }
        if (ownerReference != null) {
        	ownerReference.toXML(writer, "owner-reference");
        }
    }

    public void setPlayerReference(PlayerReference player) {
        this.playerReference = player;
    }

    public PlayerReference getPlayerReference() {
        return playerReference;
    }

    public Player getPlayer() throws ApplicationError {
        if (playerReference != null) {
            return playerReference.getPlayer();
        } else {
            return null;
        }
    }
    
    public String getPlayerName() {
        if (playerReference != null) {
            return playerReference.getName();
        } else {
            return null;
        }
    }

    public void setOwnerReference(PlayerReference owner) {
        this.ownerReference = owner;
    }

    public PlayerReference getOwnerReference() {
        return ownerReference;
    }

    public Player getOwner() throws ApplicationError {
        if (ownerReference != null) {
            return ownerReference.getPlayer();
        } else {
            return null;
        }
    }
    
    public String getOwnerName() {
        if (ownerReference != null) {
            return ownerReference.getName();
        } else {
            return null;
        }
    }
    
    /**
     * @return A Textual description of this event suitable for displaying to a user. 
     * @throws ApplicationError 
     */
    public String getDescription() throws ApplicationError {
        return null;
    }

    protected String getElementName() {
        return "stratagist-event";
    }

	public double getDate() {
		return date;
	}

	public void setDate(double date) {
		this.date = date;
	}
}
