/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.players.strategists;
import java.util.ArrayList;
import java.util.Iterator;

import mot.common.event.DiplomaticEvent;
import mot.common.util.ApplicationError;
import mot.common.vo.StrategistEventVO;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.players.Player;

/**
 * Strategists are required to remember significant events from the past so that intelligent
 * decisions can be made later on in the game. Some examples are; If a player requests a treaty
 * this should be remembered because if I reject it and they ask again immediately this is annoying
 * me and may be construde as them being desperate! Also if a player has spied on me or invaded one
 * of my planets these events should effect the long term relationship.
 * 
 * This class manages a list of events for a stratagist and provides methods for quering the event
 * list.
 */
public class StrategistEventManager  extends XMLBean {

    /**
     * This is the list of events that this manager is maintaining.
     * This is a list of StratagistEvent objects.
     */
    private ArrayList events = new ArrayList();
    
    /**
     * This is a cache of events stored in the ArrayList of events.
     * This is re-generated every time the list of events are modified. It is used to
     * provide type safety and more importantly efficiency. The list of events is not 
     * modified very often but is iterated over every turn - thus using this cached
     * array means we do not need to construct an iterator.
     */
    private transient StratagistEvent[] eventArray = null;

    public StrategistEventManager() {
    }
    
    public StratagistEvent[] getEventsAsArray() {
        if (events.size() == 0) {
            return null;
        } else {
            return (StratagistEvent[])events.toArray(new StratagistEvent[(events.size())]);
        }
    }   
    
    public void setStratagistEvent(StratagistEvent event) {
        add(event);
    }
    
    public void setStratagistEvent(StrategistEventAssetScore event) {
        add(event);
    }
    
    
    public void add(StratagistEvent event) {
        events.add(event);
        eventArray = null;
    }
    
    public void clearEvents() {
        events.clear();
        eventArray = null;
    }
    
    /**
     * @return The value which these events will effect the relationship to the player
     * parameter. If the value of positive this represents a improvement in the relationship
     * while a negative value indicates a worsening of the relationship.
     */
    public int getRelationshipEffect(Player player) throws ApplicationError {
        StratagistEvent[] events = getEventsAsArray();
        if (events == null) return 0;
        int r = 0;
        for (int i=0; i<events.length; i++) {
        	if (!events[i].hasExpired()) {
        		r += events[i].getRelationshipEffect(player);
        	}
        }
        return r;
    }
    
    /**
     * @return True if this list of events contains one or more events of type eventClass.
     */
    public boolean hasEventType(Class eventClass) throws ApplicationError {
        StratagistEvent[] events = getEventsAsArray();
        if (events == null) return false;
        for (int i=0; i<events.length; i++) {
            if(!events[i].hasExpired() && events[i].getClass() == eventClass) {
                return true;
            }
        }
        return false;
    }
    
    public StratagistEvent getEventType(Class eventClass) throws ApplicationError {
        StratagistEvent[] events = getEventsAsArray();
        if (events == null) return null;
        for (int i=0; i<events.length; i++) {
            if(!events[i].hasExpired() && events[i].getClass() == eventClass) {
                return events[i];
            }
        }
        return null;
    }
    
    /**
     * @return True if this list of events contains one or more events of type eventClass.
     */
    public boolean hasEventTypeWithAssetScoreAbove(Class eventClass, int assetScore) throws ApplicationError {
        StratagistEvent[] events = getEventsAsArray();
        if (events == null) return false;
        for (int i=0; i<events.length; i++) {
            if(!events[i].hasExpired() && events[i].getClass() == eventClass) {
            	if (events[i] instanceof StrategistEventAssetScore) {
            		StrategistEventAssetScore seas = (StrategistEventAssetScore) events[i];
            		if (seas.getNetAssetScore() >= assetScore) {
            			return true;            			
            		}
            	}
            }
        }
        return false;
    }
    
    /**
     * @return A list of textual descriptions of all events that player other has caused
     * that effect the relationship with this player.
     */
    public StrategistEventVO [] getStrategistEvents(Player player, Player other, boolean includeExpiredEvents) throws ApplicationError {
        StratagistEvent[] events = getEventsAsArray();
        if (events == null) return null;
        ArrayList r = null;
        for (int i=0; i<events.length; i++) {
        	if (includeExpiredEvents || !events[i].hasExpired()) {        		
        		if (events[i].getPlayer() == null || events[i].getPlayer() == other) {
        			String description = events[i].getDescription();
        			if (description != null) {
        				if (r == null) r = new ArrayList();
        				r.add(new StrategistEventVO(events[i].getDate(), player.getName(), description));
        			}
        		}
        	}
        }
        if (r == null) {
            return null;
        } else {
            return (StrategistEventVO[]) r.toArray(new StrategistEventVO[r.size()]);
        }
    }

    /**
     * @return The last event that player other has caused that effect the relationship 
     * with this player of the type passed in.
     */
    public StratagistEvent getLastDiplomaticEvent(Player other, Class eventClass) throws ApplicationError {
        StratagistEvent[] events = getEventsAsArray();
        if (events == null) return null;
        ArrayList r = null;
        for (int i=events.length-1; i >= 0; i--) {
            Player player = events[i].getPlayer();
			if (!events[i].hasExpired() && player != null && player == other && events[i].getClass() == eventClass) {
            	return events[i];
            }
        }
        return null;
    }

    /**
     * @return A list of textual descriptions of all events this player remembers
     * that effect the relationship with other players.
     */
    public DiplomaticEvent [] getDiplomaticEvents(boolean includeExpiredEvents) throws ApplicationError {
        StratagistEvent[] events = getEventsAsArray();
        if (events == null) return null;
        ArrayList r = null;
        for (int i=0; i<events.length; i++) {
        	if (includeExpiredEvents || !events[i].hasExpired()) {        		
        		String description = events[i].getDescription();
        		if (events[i].getPlayer() != null && description != null) {
        			if (r == null) r = new ArrayList();
        			r.add(new DiplomaticEvent(events[i].getPlayer().getName(), description));
        		}
        	}
        }
        if (r == null) {
            return null;
        } else {
            return (DiplomaticEvent[]) r.toArray(new DiplomaticEvent[r.size()]);
        }
    }

    protected String getElementName() {
        return "strategist-event-manager";
    }

    protected void encode(XMLWriter writer) throws ApplicationError {
        if (events.size() > 0) {
            Iterator i = events.iterator();
            while (i.hasNext()) {
                StratagistEvent event = (StratagistEvent)i.next();
                event.toXML(writer);
            }
        }
    }
    
    
}

