/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.prizes;

import mot.common.setup.RelicsFactory;
import mot.common.setup.SetupOption;
import mot.common.setup.SetupOptions;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.common.StellarObject;
import mot.engine.setup.PlayerFactory;

/**
 * Used to generate random prizes at a solar system.
 */
public class PrizeFactory {

    /**
     * This is a list of technologies that are available as prizes.
     */
    private static GeneralTechnologyList prizeTechnologies = null;
    
    private static synchronized TechnologyList getPrizeTechnologies() throws ApplicationError {
        if (prizeTechnologies == null) {
            prizeTechnologies = new GeneralTechnologyList();
            Technology[] startingTechnologies = PlayerFactory.getStartingTechnologies(null);
            GeneralTechnologyList stl = new GeneralTechnologyList(startingTechnologies);
            prizeTechnologies.add(Technologies.getEngines().getAllExcluding(stl, 100, 750));
            prizeTechnologies.add(Technologies.getWeapons().getAllExcluding(stl, 100, 1200));
            prizeTechnologies.add(Technologies.getScanners().getAllExcluding(stl, 100, 1000));
            prizeTechnologies.add(Technologies.getFactories().getAllExcluding(stl, 100, 1000));
            prizeTechnologies.add(Technologies.getHulls().getAllExcluding(stl, 100, 1000));
            prizeTechnologies.add(Technologies.getComputers().getAllExcluding(stl, 100, 1000));
            prizeTechnologies.add(Technologies.getBiotechnologies().getAllExcluding(stl, 100, 1000));
        }
        return prizeTechnologies;
    }
    
    public static void createRandomPrize(StellarObject so) throws ApplicationError {
        Prize prize = createRandomPrize();
        if (prize != null && so != null) {
            so.setPrize(prize);
        }
    }
    
    public static Prize createRandomPrize() throws ApplicationError {
    	return _createRandomPrize(Utils.random(4));
    }
        
    private static Prize _createRandomPrize(int type) throws ApplicationError {
        switch(type) {
            case 0:
                return createMoneyPrize();
            case 1:
            	return createTechnologyPrize();
            case 2:
                return createCraftPrize();
            case 3:
            	return createRelicPrize();
            default:
                return new MoneyPrize(200);
        }
    }

    public static CraftPrize createCraftPrize() {
		return new CraftPrize();
	}

    public static Prize createRelicPrize() throws ApplicationError {
		SetupOptions relics =  RelicsFactory.getRelics();
		int j = Utils.random(relics.getQuantity());
		SetupOption relic = relics.getAtIndex(j);
		if (relic != null) {
			return new RelicPrize(relic);
		}
	    Logger.logError("No relics!!", new ApplicationError());
		return createCraftPrize();
	}

    public static Prize createTechnologyPrize() throws ApplicationError {
		TechnologyList techs = getPrizeTechnologies();
		if (techs == null || techs.getQuantity() == 0) {
		    Logger.logError("No technologies", new ApplicationError());
		    return createCraftPrize();
		}
		Technology tech = techs.getRandom();
		if (tech != null) {					
			return new TechnologyPrize(tech);
		}
	    Logger.logError("No technologies!!", new ApplicationError());
		return createCraftPrize();
	}

    public static MoneyPrize createMoneyPrize() {
		return new MoneyPrize(250+Utils.random(4)*250);
	}
    
    public static void createMoneyPrize(StellarObject so) throws ApplicationError {
    	MoneyPrize prize = createMoneyPrize();
        if (prize != null && so != null) {
        	so.setPrize(prize);
        }
    }

    public static void createBoraliumPrize(StellarObject so) throws ApplicationError {
    	BoraliumPrize prize = createBoraliumPrize();
        if (prize != null && so != null) {
        	so.setPrize(prize);
        }
    }

    public static BoraliumPrize createBoraliumPrize() {
		return new BoraliumPrize(200+Utils.random(3)*5);
	}

}