/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.relations;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.bean.DiplomaticHistory;
import mot.common.bean.DiplomaticProposal;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.event.MessageEF;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.PlayerReference;
import mot.engine.players.UserPlayer;


public class Relation extends XMLBean {

	private PlayerReference player1;
	private PlayerReference player2;
    
    /**
     * The basic relationship value between player1 and player2. This value is the default
     * relationship value as defined in the player character with negative effects such as spying
     * and sabotage added as well as positive effects such as gifts. This value is aged in such 
     * a way that it tends to the default values defined in the player character,
     */
	private int value;
    
    /**
     * This is a delta to the relationship value defined by the trading effect. A good trade route 
     * between these two player will cause a positive trade effect and this improve the relationship 
     * value.
     */
    private int tradeEffect;
    private int researchEffect;
    
    /**
     * A list of treaties in place between these two races.
     * This is a list of Treaty Objects (it may also be empty or null).
     */
    private transient Treaty [] treatiesArray = null;
    private ArrayList treaties = new ArrayList();
    
    /**
     * True if these two players have diplomatic relations and false otherwise. NULL implies
     * the value has not been calculated yet. This value should be recalculated each turn and
     * if the value changes events to be sent to user players informing them of the change! 
     * @see Player._hasDiplimaticRelations() for details.
     */
    private Boolean hasDiplimaticRelations;
    
    /**
     * All treaties made and broken during this relationship.
     */
    private DiplomaticHistory diplomaticHistory;

	public Relation() {
	}

	public Relation(Player  player1, Player player2, int value) {
		this.player1 = new PlayerReference(player1);
		this.player2 = new PlayerReference(player2);
		this.value = value;
	}

	public Relation(String  player1, String player2, int value) {
		this.player1 = new PlayerReference(player1);
		this.player2 = new PlayerReference(player2);
		this.value = value;
	}

	public Relation(Relation other) {
		this.player1 = new PlayerReference(other.getPlayerReference1().getName());
		this.player2 = new PlayerReference(other.getPlayerReference2().getName());
		this.value = other.getValue();
        this.tradeEffect = other.getTradeEffect();
        this.researchEffect = other.getResearchEffect();
        ArrayList treaties = other.getTreaties();
        if (treaties != null) {
            Iterator i = treaties.iterator();
            while (i.hasNext()) {
                Treaty t = (Treaty) i.next();
                _addTreaty(t);
            }
        }
        this.diplomaticHistory = new DiplomaticHistory(other.getDiplomaticHistory());
	}

	public PlayerReference getPlayerReference1() { return player1; }
	public PlayerReference getPlayerReference2() { return player2; }

	public Player getPlayer1() throws ApplicationError { return player1.getPlayer(); }
	public Player getPlayer2() throws ApplicationError { return player2.getPlayer(); }
	public String getPlayerName1() { return player1 == null ? null : player1.getName(); }
	public String getPlayerName2() { return player2 == null ? null : player2.getName(); }

	public int getValue() { return value; }
	public int getRelationshipValue() { return value + tradeEffect + researchEffect; }
	public ArrayList getTreaties() { return treaties; }

	public void setPlayer1(Player v) { player1 = new PlayerReference(v); }
	public void setPlayer2(Player v) { player2 = new PlayerReference(v); }
	public void setValue(int v) { value = v; }
	public void setTreaties(ArrayList v) { treaties = v; treatiesArray = null;}

	public void setPlayer1(PlayerReference v) { player1 = v; }
	public void setPlayer2(PlayerReference v) { player2 = v; }
    
    public boolean contains(Player p) throws ApplicationError { return (player1.getPlayer() == p || player2.getPlayer() == p); }
    
    public boolean hasDiplomaticRelations() throws ApplicationError {
    	if (hasDiplimaticRelations == null) {
    		boolean r = getPlayer1()._hasDiplomaticRelationsWith(getPlayer2());
    		hasDiplimaticRelations = r ? Boolean.TRUE : Boolean.FALSE;
    	}
    	return hasDiplimaticRelations.booleanValue();
    }
    
    public void updateHasDiplomaticRelations() throws ApplicationError  {
    	boolean r = getPlayer1()._hasDiplomaticRelationsWith(getPlayer2());
    	if (hasDiplimaticRelations == null) {
    		hasDiplimaticRelations = r ? Boolean.TRUE : Boolean.FALSE;
    	} else {
    		if (hasDiplimaticRelations.booleanValue() != r) {
    			if (!getPlayer1().isComputerPlayer()) {
    				sendStartOrEndDiplomacyMessage(r, getPlayer2(), getPlayer1());
    			}
    			if (!getPlayer2().isComputerPlayer()) {
    				sendStartOrEndDiplomacyMessage(r, getPlayer1(), getPlayer2());
    			}
    			hasDiplimaticRelations = r ? Boolean.TRUE : Boolean.FALSE;
    		}
    	}
    }
    
    private void sendStartOrEndDiplomacyMessage(boolean start, Player from, Player to) throws ApplicationError {
		String msg = start ?
				from.getPlayerCharacter().getDiplomaticStatement("FORM_DIPLOMATIC_RELATIONS"):
				from.getPlayerCharacter().getDiplomaticStatement("LOSE_DIPLOMATIC_RELATIONS");
		MessageEF event = new MessageEF("[ "+from.getName()+" ] " + msg);
        MotEngine.instance.broadcast(event, to); 
        if (from instanceof ComputerPlayer && to instanceof UserPlayer) {
        	((ComputerPlayer)from).getStratagist().notifyRelationsEstablished(to, start);
        }
    }
    
    public Treaty [] getTreatiesAsArray() {
        if (treaties == null || treaties.size() == 0) {
            return null;
        }
        treatiesArray = new Treaty[treaties.size()];
        for (int i=0; i<treaties.size();i++) {
            treatiesArray[i] = ((Treaty) treaties.get(i));
        }
        return treatiesArray;
    }
    
    public Treaty getTreaty(Class clazz) {
        if (treaties == null || treaties.size() == 0) {
            return null;
        }
        for (int i=0; i<treaties.size();i++) {
            Treaty t = ((Treaty) treaties.get(i));
            if (t.getClass() == clazz) {
                return t;
            }
        }
        return null;
    }
    
    /**
     * This is used by XMLDocumentHandler to load saved games.
     * Longcut to addTreaty(int).
     */
    public void setTreaty(Treaty treaty) throws NumberFormatException {
        addTreaty(treaty);
    }
    
    /**
     * This is used by XMLDocumentHandler to load saved games.
     * Longcut to addTreaty(int).
     */
    public void setAllianceTreaty(AllianceTreaty treaty) throws NumberFormatException {
        addTreaty(treaty);
    }
    
    /**
     * This is used by XMLDocumentHandler to load saved games.
     * Longcut to addTreaty(int).
     */
    public void setNonAgressionTreaty(NonAgressionTreaty treaty) throws NumberFormatException {
        addTreaty(treaty);
    }
    
    /**
     * This is used by XMLDocumentHandler to load saved games.
     * Longcut to addTreaty(int).
     */
    public void setPeaceTreaty(PeaceTreaty treaty) throws NumberFormatException {
        addTreaty(treaty);
    }
    
    // For backwards compatibility - Delete in the future - NOTE these treaties are lost!
    // REMOVE FROM  mot.common.xml.BackwardsCompatibility
    public void setTradeTreaty(TradeAgreementTreaty treaty) { }
    public void setResearchTreaty(ResearchAgreementTreaty treaty) { }
    
    /**
     * This is used by XMLDocumentHandler to load saved games.
     * Longcut to addTreaty(int).
     */
    public void setTradeAgreementTreaty(TradeAgreementTreaty treaty) throws NumberFormatException {
        addTreaty(treaty);
    }
    
    /**
     * This is used by XMLDocumentHandler to load saved games.
     * Longcut to addTreaty(int).
     */
    public void setResearchAgreementTreaty(ResearchAgreementTreaty treaty) throws NumberFormatException {
        addTreaty(treaty);
    }
    
    /**
     * This is used by XMLDocumentHandler to load saved games.
     * Longcut to addTreaty(int).
     */
    public void setWarTreaty(WarTreaty treaty) throws NumberFormatException {
        addTreaty(treaty);
    }
    
    /**
     * Add a new treaty to the relationship between these two players.
     */
    public void addTreaty(Treaty treaty) {
        if (Logger.isDebug()) {
            Logger.logDebug("Relation.addTreaty() Create treaty "+treaty.getClass().getName()+" between "+player1.getName()+" and "+player2.getName());
        }
        _addTreaty(treaty);
    }
    
    private void _addTreaty(Treaty treaty) {
        treaties.add( treaty );
        treatiesArray = null;
    }

    /**
     * Removes a treaty from the relationship between these two players.
     */
    public void removeTreaty(Class treaty) {
        if (treaties == null) {
            return;
        }
        Iterator i = treaties.iterator();
        while (i.hasNext()) {
            Treaty t = (Treaty) i.next();
            if (t.getClass() == treaty) {
                if (Logger.isDebug()) {
                    Logger.logDebug("Relation.removeTreaty() Remove treaty "+treaty.getName()+" between "+player1.getName()+" and "+player2.getName());
                }
                i.remove();
            }
        }
        treatiesArray = null;
    }

    /**
     * Appends a new item to the history or diplomatic treaties made in this relationship.
     */
	public void addDiplomaticHistoryEvent(double date, DiplomaticProposal proposal, String playerOfferMaker) {
		if (diplomaticHistory == null) diplomaticHistory = new DiplomaticHistory();
		diplomaticHistory.addDiplomaticHistoryEvent(date, proposal, playerOfferMaker);
	}

	private static double nextDate = -1;
	
	public static double getNextDiplomaticHistoryEventDate() {
		double date = MotEngine.instance.getDate();
		if (nextDate > date && nextDate < date + 1) {
			date = nextDate;
			nextDate = nextDate + 0.01;
			return date;
		}
		nextDate = date + 0.01;
		return date;
	}

	public void addDiplomaticHistoryEvent(String description) {
		if (diplomaticHistory == null) diplomaticHistory = new DiplomaticHistory();
		double date = getNextDiplomaticHistoryEventDate();
		diplomaticHistory.addDiplomaticHistoryEvent(date, description);
	}

	public void addDiplomaticHistoryEvent(DiplomaticProposal proposal, Player playerOfferMaker) {
		if (proposal == null || playerOfferMaker == null) return;
		if (diplomaticHistory == null) diplomaticHistory = new DiplomaticHistory();
		double date = getNextDiplomaticHistoryEventDate();
		diplomaticHistory.addDiplomaticHistoryEvent(date, proposal, playerOfferMaker.getName());
	}

    /**
     * Removes a treaty from the relationship between these two players.
     */
    public void removeTreaty(Treaty treaty) {
        if (treaties == null) {
            return;
        }
        Iterator i = treaties.iterator();
        while (i.hasNext()) {
            Treaty t = (Treaty) i.next();
            if (t.getClass() == treaty.getClass()) {
                if (Logger.isDebug()) {
                    Logger.logDebug("Relation.removeTreaty() Remove treaty "+treaty.getClass().getName()+" between "+player1.getName()+" and "+player2.getName());
                }
                i.remove();
            }
        }
        treatiesArray = null;
    }

    /**
     * @param treaty The treaty that exist in this relation
     * @return Returns true if anyt only if a treaty exists of this type in this relation.
     */
    public boolean containsTreaty(Treaty treaty) {
        if (treaties == null) {
            return false;
        }
        Iterator i = treaties.iterator();
        while (i.hasNext()) {
            Treaty t = (Treaty) i.next();
            if (t.getClass() == treaty.getClass()) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * @param treaty The treaty that exist in this relation
     * @return Returns true if anyt only if a treaty exists of this type in this relation.
     */
    public boolean containsTreaty(Class treaty) {
        if (treaties == null) {
            return false;
        }
        Iterator i = treaties.iterator();
        while (i.hasNext()) {
            Treaty t = (Treaty) i.next();
            if (t.getClass() == treaty) {
                return true;
            }
        }
        return false;
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
		writer.addElement("value", value);
		writer.addElement("trade-effect", tradeEffect);
		writer.addElement("research-effect", researchEffect);
		player1.toXML(writer, "player1");
		player2.toXML(writer, "player2");
        if (treaties != null) {
            Iterator i = treaties.iterator();
            while (i.hasNext()) {
                Treaty t = (Treaty) i.next();
                t.toXML(writer);
            }
        }
        if (diplomaticHistory != null) {
        	diplomaticHistory.toXML(writer);
        }
	}

	protected String getElementName() { return "relation"; }

    public int getTradeEffect() {
        return tradeEffect;
    }

    public void setTradeEffect(int tradeEffect) {
        this.tradeEffect = tradeEffect;
    }

    public boolean canHaveWarTreaty() { return !containsTreaty(WarTreaty.class); }
    public boolean canHaveTradeTreaty() { return !containsTreaty(WarTreaty.class) && !containsTreaty(TradeAgreementTreaty.class); }
    public boolean canHaveResearchTreaty() { return !containsTreaty(WarTreaty.class) && !containsTreaty(ResearchAgreementTreaty.class); }
    public boolean canHavePeaceTreaty() { return containsTreaty(WarTreaty.class) ; }
    public boolean canHaveNonAgressionTreaty() { return !containsTreaty(WarTreaty.class) && !containsTreaty(AllianceTreaty.class) && !containsTreaty(NonAgressionTreaty.class) ; }
    public boolean canHaveAllianceTreaty() { return !containsTreaty(WarTreaty.class) && !containsTreaty(AllianceTreaty.class) ; }

	public boolean isWar() throws ApplicationError { return containsTreaty(WarTreaty.class); }
	public boolean isNonAgression() throws ApplicationError { return containsTreaty(NonAgressionTreaty.class); }
	public boolean isPeace() throws ApplicationError { return containsTreaty(PeaceTreaty.class); }
	public boolean isAliance() throws ApplicationError { return containsTreaty(AllianceTreaty.class); }
	
	public TradeAgreementTreaty getTradeAgreementTreaty() throws ApplicationError { return (TradeAgreementTreaty) getTreaty(TradeAgreementTreaty.class); }
	public ResearchAgreementTreaty getResearchAgreementTreaty() throws ApplicationError { return (ResearchAgreementTreaty) getTreaty(ResearchAgreementTreaty.class); }
	
	public String toString() { return player1.getName() + "x" + player2.getName() + " = " + value; }

	public int getResearchEffect() {
		return researchEffect;
	}

	public void setResearchEffect(int researchEffect) {
		this.researchEffect = researchEffect;
	}

	public DiplomaticHistory getDiplomaticHistory() {
		return diplomaticHistory;
	}

	public void setDiplomaticHistory(DiplomaticHistory diplomaticHistory) {
		this.diplomaticHistory = diplomaticHistory;
	}

}