/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.relations;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.bean.DiplomaticProposal;
import mot.common.lang.L;
import mot.common.resources.Resource;
import mot.common.resources.ResourceBoralium;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValueList;
import mot.common.util.BasicNameValuePairList;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.NameValuePair;
import mot.common.util.NameValuePairList;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.event.MessageEF;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * The list of treaties that this player has with other players.
 * Types of treaty include, War, Non Aggression, Peace, Trade, Research etc.
 * 
 * The level wow is not reachable by normal means but the test harnesses set this
 * level manually to make sure the computer player accepts all requests.
 */
public class Relations extends XMLBean {

	public static final int WOW		    = 5000;
 	public static final int UNITED		= 100;
 	public static final int LOVE		= 90;
 	public static final int LIKE		= 70;
 	public static final int FOND		= 60;
 	public static final int NEUTRAL		= 50;
 	public static final int WARY		= 40;
 	public static final int DISTRUST	= 30;
 	public static final int DISLIKE		= 20;
 	public static final int HATE		= 10;
 	public static final int LOATH		= 0;

    public static final int DEFAULT_RELATIONSHIP_VALUE = NEUTRAL;
	public static final int AGE_RELATIONS_INTERVAL = 1;

	private ArrayList relations = new ArrayList();

	public Relations() {
	}
    
    public Relation [] getRelations() {
        if (relations == null || relations.size() == 0) {
            return null;
        } else {
            return (Relation []) relations.toArray(new Relation [relations.size()]);
        }
    }
    
    public Relation [] getRelations(Player p) throws ApplicationError {
        if (relations == null || relations.size() == 0) {
            return null;
        } else {
        	ArrayList<Relation> r = new ArrayList();
    		for (int i=0; i<relations.size(); i++) {
    			Relation rel = (Relation) relations.get(i);
    			if ( rel.contains(p) && rel.hasDiplomaticRelations() ) {
    				r.add(rel);
    			}
    		}
    		if (r.size() == 0) {
    			return null;
    		} else {
    			return (Relation []) r.toArray(new Relation [r.size()]);
    		}
        }
    }
    
	public boolean isAtWarWithAPlayer(Player p1) throws ApplicationError {
		for (int i=0; i<relations.size(); i++) {
			Relation r = (Relation) relations.get(i);
			if ( r.getPlayer1() == p1  && r.isWar()) {
				return true;
			}
		}
		return false;
	}


	public boolean containsWar(Player p1, Player [] list) throws ApplicationError {
        if (list == null) {
            return false;
        }
        for (int i=0; i<list.length; i++) {
            if (isWar(p1, list[i])) {
                return true;
            }
        }
        return false;
    }
    
	public boolean isWar(Player p1, Player p2) throws ApplicationError {
		return exists(p1, p2, WarTreaty.class);
	}

	public boolean isNonAgression(Player p1, Player p2) throws ApplicationError {
		return exists(p1, p2, NonAgressionTreaty.class);
	}

	public boolean isPeace(Player p1, Player p2) throws ApplicationError {
		return !exists(p1, p2, WarTreaty.class);
	}

	public boolean isAliance(Player p1, Player p2) throws ApplicationError {
		return exists(p1, p2, AllianceTreaty.class);
	}

	public boolean isTrade(Player p1, Player p2) throws ApplicationError {
		return exists(p1, p2, TradeAgreementTreaty.class);
	}

	public boolean isResearch(Player p1, Player p2) throws ApplicationError {
		return exists(p1, p2, ResearchAgreementTreaty.class);
	}

	private boolean exists(Player p1, Player p2, Class type) throws ApplicationError {
        MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
        if (p1 == mp || p2 == mp) {
            if (p1 != p2 && type == WarTreaty.class) {
                return true;
            } else {
                return false;
            }
        } else {
            Relation r = get(p1, p2);
            if (r != null) {
                return r.containsTreaty(type);
            } else {
                return false;
            }
        }
	}

	public NonAgressionTreaty getNonAggressionTreaty(Player p1, Player p2) throws ApplicationError {
        MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
        if (p1 == mp || p2 == mp) {
        	return null;
        } else {
            Relation r = get(p1, p2);
            if (r != null) {
                return (NonAgressionTreaty) r.getTreaty(NonAgressionTreaty.class);
            } else {
                return null;
            }
        }
	}

	public AllianceTreaty getAllianceTreaty(Player p1, Player p2) throws ApplicationError {
        MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
        if (p1 == mp || p2 == mp) {
        	return null;
        } else {
            Relation r = get(p1, p2);
            if (r != null) {
                return (AllianceTreaty) r.getTreaty(AllianceTreaty.class);
            } else {
                return null;
            }
        }
	}

    public boolean canHaveTradeTreaty(Relation r) {
        if (r == null) return true;
        return !r.containsTreaty(WarTreaty.class);
    }

    public boolean canHaveResearchTreaty(Relation r) {
        if (r == null) return true;
        return !r.containsTreaty(WarTreaty.class) ;
    }

    /**
     * Creates a new war treaty between p1 and p2. It removes any non aggression, peace,
     * alliance, trade or research treaties between p1 and p2. Any players allied to p1
     * also have war declared on p2, likewise any players allied to p2 have war declared 
     * on p1.
     * 
     * It also calls notifyWarDelcared on the monster player so ancients are informed of
     * the declaration.
     * 
     * @param p1 The player who has declared war (the aggressor)
     * @param p2 The player who has been declared war on (the victim)
     * @throws mot.common.util.ApplicationError
     */
	public void setWar(Player p1, Player p2) throws ApplicationError {
        setWar(p1, p2, true);
    }
    
    /**
     * Declaring a war on a player who has alliances also causes war to be declared on the alliance
     * players. However this recursion should not cause multiple fleets to be sent from the ancients.
     * This method allows an inform monster boolean to prevent the monster player dispatching a second 
     * fleet.
     * @param p1 The player who has declared war (the aggressor)
     * @param p2 The player who has been declared war on (the victim)
     */
	public void setWar(Player p1, Player p2, boolean notifyMonster) throws ApplicationError {
        Relation r = getRelationshipNotNull(p1, p2);
        r.removeTreaty(NonAgressionTreaty.class);
        r.removeTreaty(PeaceTreaty.class);
        r.removeTreaty(AllianceTreaty.class);
        r.removeTreaty(TradeAgreementTreaty.class);
        r.removeTreaty(ResearchAgreementTreaty.class);
		r.addTreaty(new WarTreaty(MotEngine.instance.getDate()));

		// for all players I have an alliance with, set them to status war.
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != p1) && (p != p2) ) {
				if (isAliance(p, p2)) {
                    if (!isWar(p, p1)) {
                        setWar(p1, p, false);
                    }
				}
			}
		}
        
        if (notifyMonster) {
            MotEngine.instance.getMonsterPlayer().notifyWarDeclared(p1, p2);
        }
	}

	public void setNonAgression(Player p1, Player p2) throws ApplicationError {
        Relation r = getRelationshipNotNull(p1, p2);
        r.removeTreaty(WarTreaty.class);
        r.removeTreaty(PeaceTreaty.class);
        r.removeTreaty(AllianceTreaty.class);
		r.addTreaty(new NonAgressionTreaty(MotEngine.instance.getDate()));
	}

    /**
     * Player p1 has requested a peace treaty with player p2 and p2 has accepted.
     * This method creates removes a war treaty between these two players.
     */
	public void setPeace(Player p1, Player p2) throws ApplicationError {
        Relation r = getRelationshipNotNull(p1, p2);
        r.removeTreaty(WarTreaty.class);
        r.removeTreaty(AllianceTreaty.class);
        r.removeTreaty(NonAgressionTreaty.class);
		r.addTreaty(new PeaceTreaty());
	}

    /**
     * Player p1 has requested an alliance with player p2 and p2 has accepted.
     * This method creates an alliance treaty between these two players.
     * After this, player p1 will be at war with all players p2 is at war with,
     * and player p2 will be at war with all players p1 is at war with.
     */
	public void setAliance(Player p1, Player p2) throws ApplicationError {
        Relation r = getRelationshipNotNull(p1, p2);
        r.removeTreaty(WarTreaty.class);
        r.removeTreaty(PeaceTreaty.class);
        r.removeTreaty(NonAgressionTreaty.class);
		r.addTreaty(new AllianceTreaty(MotEngine.instance.getDate()));

		// for all players I have a war with, set them to status war.
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != p1) && (p != p2) ) {
				if (isWar(p1, p)) {
					String text = L.WE_HAVE_SIGNED_AN_ALIANCE_WITH+p1.getName()+L.PREPARE_YOURSELF_FOR_WAR;
	                p.diplomaticRequest(p2, DiplomaticProposal.getWar(), text);
				}
				if (isWar(p2, p)) {
					if (p instanceof UserPlayer) {
						MotEngine.instance.broadcast(new MessageEF(L.WAR_HAS_BEEN_DECLARED_WITH+p.getName()+L.DUE_TO_OUR_OBLIGATION_UNDER_THE_ALIANCE_WITH+p1.getName()+"]"), p);
					}
                    setWar(p1, p, false);
				}
			}
		}
	}

	public void setTrade(Player p1, Player p2) throws ApplicationError {
        Relation r = getRelationshipNotNull(p1, p2);
        if (!canHaveTradeTreaty(r)) {
            Logger.logWarning("Relations.setTrade() No treaty between p1="+p1.getName()+" and p2="+p2.getName());
        }
		r.addTreaty(new TradeAgreementTreaty(p1,p2));
	}

	public void setResearch(Player p1, Player p2) throws ApplicationError {
        Relation r = getRelationshipNotNull(p1, p2);
        if (!canHaveResearchTreaty(r)) {
            Logger.logWarning("Relations.setResearch() No treaty between p1="+p1.getName()+" and p2="+p2.getName());
        }
		r.addTreaty(new ResearchAgreementTreaty(p1,p2));
	}

	public int getRelation(Player p1, Player p2) throws ApplicationError {
        Relation r = getRelationshipNotNull(p1, p2);
        return r.getRelationshipValue();
    }
    
	public Relation getRelationshipNotNull(Player p1, Player p2) throws ApplicationError {
        MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
        if (p1 == mp || p2 == mp) {
            Logger.logError("Relations.getRelationshipNotNull() relations do not exist for monster", new Exception(""));
            return new Relation(p1, p2, HATE);
        }
        Relation r = get(p1, p2);
        if (r == null) {
            Logger.logWarning("Relations.getRelationNotNull() No relation between p1="+p1.getName()+" and p2="+p2.getName()+" - SET TO NEUTRAL");
    		setRelation(p1, p2, NEUTRAL);
            return get(p1, p2);
        }
        return r;
    }
    
	public Relation getRelationship(Player p1, Player p2) throws ApplicationError {
        return get(p1, p2);
    }
    
    /**
     * Returns the relationship object between player named p1 and player named p2. The players 
     * must exist (the player references must be valid).
     * @param p1 The name of a player
     * @param p2 The name of a player
     * @return The Relationship between player1 and player2.
     */
	private Relation get(Player p1, Player p2) throws ApplicationError {
		for (int i=0; i<relations.size(); i++) {
			Relation r = (Relation) relations.get(i);
			if ( r.getPlayer1().equals(p1) && r.getPlayer2().equals(p2) ) {
				return r;
			}
			if ( r.getPlayer1().equals(p2) && r.getPlayer2().equals(p1) ) {
				return r;
			}
		}
		return null;
	}

    /**
     * Returns the relationship object between player named p1 and player named p2. This does 
     * not require the players to exist. It uses the names in the player references without finding
     * the player from the player reference.
     * @param p1 The name of a player
     * @param p2 The name of a player
     * @return The Relationship between player1 and player2.
     */
	public Relation get(String p1, String p2) {
		for (int i=0; i<relations.size(); i++) {
			Relation r = (Relation) relations.get(i);
			if ( Comp.same(r.getPlayerReference1().getName(), p1) && 
                 Comp.same(r.getPlayerReference2().getName(), p2) ) {
				return r;
			}
			if ( Comp.same(r.getPlayerReference1().getName(), p2) && 
                 Comp.same(r.getPlayerReference2().getName(), p1) ) {
				return r;
			}
		}
		return null;
	}

	public void setRelation(Player p1, Player p2, int rel) throws ApplicationError {
        set(p1, p2, rel);
    }
    
	public void set(Player p1, Player p2, int rel) throws ApplicationError {
		Relation r = get(p1, p2);
		if (r == null) {
			r = new Relation(p1, p2, rel);
            relations.add(r);
		} else {
			r.setValue(rel);
		}
	}

	public void set(String p1, String p2, int rel) {
		Relation r = get(p1, p2);
		if (r == null) {
			r = new Relation(p1, p2, rel);
            relations.add(r);
		} else {
			r.setValue(rel);
		}
	}
    
    public boolean hasDiplomaticRelations(Player p1, Player p2) throws ApplicationError {
        Relation r = getRelationshipNotNull(p1, p2);
        return r.hasDiplomaticRelations();
    }
    
    public void updateHasDiplomaticRelations() throws ApplicationError {
        Relation [] rels = getRelations();
        if (rels == null) return;
        for (int i=0; i<rels.length; i++) {
            rels[i].updateHasDiplomaticRelations();
        }
    }


    /**
     * All relations containing player p are removed. This would be used when a 
     * player is being removed from the game.
     * @param p The player to remove all relationships for.
     */
    public void remove(Player p) throws ApplicationError {
        Relation [] rels = getRelations();
        if (rels == null) return;
        for (int i=0; i<rels.length; i++) {
            if (rels[i].getPlayer1() == p || rels[i].getPlayer2() == p) {
                relations.remove(rels[i]);
            }
        }
    }


    /**
     * This is used by XMLDocumentHandler to load saved games.
     */
    public void setRelation(Relation r) {
        relations.add(r);
    }
    

    /**
     * @param player The player we wish to find enemies of
     * @return A list of all players at war with player
     */
	public Player [] getPlayersAtWarWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isWar(p, player) ) {
                if (list == null) list = new ArrayList();
                list.add(p);
            }
		}
        if (list == null) {
            return null;
        } else {
            return (Player []) list.toArray(new Player [list.size()]);
        }
	}
    
    /**
     * @param player The player we wish to find enemies of
     * @return A list of all player names at war with player
     */
	public String [] getPlayersNamesAtWarWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isWar(p, player) ) {
                if (list == null) list = new ArrayList();
                list.add(p.getName());
            }
		}
        if (list == null) {
            return null;
        } else {
            return (String []) list.toArray(new String [list.size()]);
        }
	}
    
    /**
     * @param player The player we wish to find enemies of
     * @return A list of all player names at war with player
     */
	public String [] getPlayersNamesHaveAllianceTreatyWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isAliance(p, player) ) {
                if (list == null) list = new ArrayList();
                list.add(p.getName());
            }
		}
        if (list == null) {
            return null;
        } else {
            return (String []) list.toArray(new String [list.size()]);
        }
	}
    
    /**
     * @param player The player we wish to find player at peace with
     * @return A list of all player names at peace with player
     */
	public String [] getPlayersNamesHavePeaceTreatyWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && exists(p, player, PeaceTreaty.class) ) {
                if (list == null) list = new ArrayList();
                list.add(p.getName());
            }
		}
        if (list == null) {
            return null;
        } else {
            return (String []) list.toArray(new String [list.size()]);
        }
	}
    
    /**
     * @param player The player we wish to find enemies of
     * @return A list of all player names at war with player
     */
	public String [] getPlayersNamesHaveNonAgressionTreatyWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isNonAgression(p, player) ) {
                if (list == null) list = new ArrayList();
                list.add(p.getName());
            }
		}
        if (list == null) {
            return null;
        } else {
            return (String []) list.toArray(new String [list.size()]);
        }
	}
    
    /**
     * @param player The player we wish to find enemies of
     * @return A list of all player names at war with player
     */
	public String [] getPlayersNamesHaveResearchTreatyWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isResearch(p, player) ) {
                if (list == null) list = new ArrayList();
                list.add(p.getName());
            }
		}
        if (list == null) {
            return null;
        } else {
            return (String []) list.toArray(new String [list.size()]);
        }
	}
    
    /**
     * @param player The player we wish to find relationship values for
     * @return A list of all players with a relationship value.
     * @throws ApplicationError 
     */
	public NameValuePairList getRelationshipValues(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		NameValuePairList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( p != player ) {
                Relation r = player.getRelationshipWith(p);
                if (r != null) {
	                if (list == null) list = new NameValuePairList();
	                list.add(new NameValuePair(p.getName(), ""+r.getValue()));
                }
            }
		}
		return list;
	}
	
	
    /**
     * @param player The player we wish to find trading partners for
     * @return A list of all players trading with player.
     */
	public Player [] getTradersWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isTrade(p, player) ) {
                if (list == null) list = new ArrayList();
                list.add(p);
            }
		}
        if (list == null) {
            return null;
        } else {
            return (Player []) list.toArray(new Player [list.size()]);
        }
	}
	
    /**
     * @param player The player we wish to find trading partners for
     * @return A list of all trade agreements with player.
     */
	public TradeAgreementTreaty [] getTraderAgreementsWith(Player player) throws ApplicationError {
		Relation[] rels = getRelations();
		if (rels == null) return null;
		ArrayList list = null;
		for (int i = 0; i < rels.length; i++) {
			if (rels[i].contains(player)) {
				TradeAgreementTreaty t = rels[i].getTradeAgreementTreaty();
				if (t instanceof TradeAgreementTreaty) {
					if (list == null) list = new ArrayList();
					list.add(t);
				}
			}
		}
        if (list == null) {
            return null;
        } else {
            return (TradeAgreementTreaty []) list.toArray(new TradeAgreementTreaty [list.size()]);
        }
	}

	/**
	 * @return The qty boralium per turn from trade agreements player earns (receives)
	 * @throws ApplicationError
	 */
	public int getBoraliumPerTurnWith(Player player) throws ApplicationError {
		Relation[] rels = getRelations();
		if (rels == null) return 0;
		int qty = 0;
		for (int i = 0; i < rels.length; i++) {
			if (rels[i].contains(player)) {
				TradeAgreementTreaty t = rels[i].getTradeAgreementTreaty();
				if (t instanceof TradeAgreementTreaty) {
					TradeAgreementTreaty tg = (TradeAgreementTreaty) t;
					Resource resource = tg.getResourceRecieved(player);
					if (resource instanceof ResourceBoralium) {
						//ResourceBoralium br = (ResourceBoralium) resource;
						qty++;
					}
				}
			}
		}
        return qty;
	}
	

    /**
     * @param player The player we wish to find researcher partners for
     * @return A list of all players with a research treaty with player.
     */
	public Player [] getResearchersWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
        if (players == null) return null;
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isResearch(p, player) ) {
                if (list == null) list = new ArrayList();
                list.add(p);
            }
		}
        if (list == null) {
            return null;
        } else {
            return (Player []) list.toArray(new Player [list.size()]);
        }
	}
	
    /**
     * @param player The player we wish to find trading partners for
     * @return A list of all trade agreements with player.
     */
	public ResearchAgreementTreaty [] getResearchAgreementTreatiesWith(Player player) throws ApplicationError {
		Relation[] rels = getRelations();
		if (rels == null) return null;
		ArrayList list = null;
		for (int i = 0; i < rels.length; i++) {
			if (rels[i].contains(player)) {
				ResearchAgreementTreaty t = rels[i].getResearchAgreementTreaty();
				if (t instanceof ResearchAgreementTreaty) {
					if (list == null) list = new ArrayList();
					list.add(rels[i]);
				}
			}
		}
        if (list == null) {
            return null;
        } else {
            return (ResearchAgreementTreaty []) list.toArray(new ResearchAgreementTreaty [list.size()]);
        }
	}
	

	public boolean hasMaximumPlayerTradeRoutes(Player player) throws ApplicationError {
        int maxQtyTradeRoutes = player.getMaxPlayerTradeRoutes();
        Player [] traders = getTradersWith(player);
        return (traders != null && traders.length >= maxQtyTradeRoutes);
	}
    
    /**
     * @param player The player we wish to find trading partners for
     * @return A list of all players trading with player.
     */
	public int getQtyTradersWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		int qty = 0;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isTrade(p, player) ) {
                qty++;
            }
		}
        return qty;
	}
   
	public boolean canTradeWith(Player player1, Player player2) throws ApplicationError {
		return player1 == player2 || isTrade(player1, player2);
	}
	
	public boolean hasMaximumResearchers(Player player) throws ApplicationError {
        int maxQtyTradeRoutes = player.getMaxPlayerTradeRoutes();
        int qtyResearchers = getQtyResearchersWith(player);
        return (qtyResearchers >= maxQtyTradeRoutes);
	}
    
    /**
     * @param player The player we wish to find trading partners for
     * @return A list of all players trading with player.
     */
	public int getQtyResearchersWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		int qty = 0;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isResearch(p, player) ) {
                qty++;
            }
		}
        return qty;
	}
    
    /**
     * @param player The player we wish to find alliance partners for
     * @return A list of all players who have an alliance with player.
     */
	public Player [] getAlliancesWith(Player player) throws ApplicationError {
		Player [] players = MotEngine.instance.getPlayers().getPlayers();
		ArrayList list = null;
		for (int i=0; i<players.length; i++) {
			Player p = players[i];
			if ( (p != player) && isAliance(p, player) ) {
                if (list == null) list = new ArrayList();
                list.add(p);
            }
		}
        if (list == null) {
            return null;
        } else {
            return (Player []) list.toArray(new Player [list.size()]);
        }
	}
    
    /**
     * This method performs all end turn related actions to relationships. This should 
     * be called at the end of every turn before the start of the next turn. It functions 
     * include;
     *  - Age Relations (over time relations tend to those defined in the player character).
     *  - Process Trade Routes.
     */
    public void endTurn() throws ApplicationError {
        ageRelations();
        processTradeRoutes();
        processResearchTreaties();
        updateHasDiplomaticRelations();
    }
    
	/**
	 * The date of the last time this player calculates its priorities.
	 */
	protected int dateOfLastAgeRelation;

	public int getDateOfLastAgeRelation() { return dateOfLastAgeRelation; }
	public void setDateOfLastAgeRelation(int v) { dateOfLastAgeRelation = v; }

    /**
     * A players relations tend to revert to those defined in the player character. So if race X
     * naturally dislikes race Y then giving gifts or stealing tech may improve or deprove a 
     * relationship for a while - but it will gradually get back to its predefined state.
     * 
     * This method changes the player relationship to be one closer the the default relationship
     * settings defined in the player characters.
     */
    public void ageRelations() throws ApplicationError {
		if ( (MotEngine.getInstance().getDate() - dateOfLastAgeRelation) < AGE_RELATIONS_INTERVAL) {
			return;
		}
        dateOfLastAgeRelation = MotEngine.getInstance().getDate();

        Iterator i = relations.iterator();
		while (i.hasNext()) {
			Relation r = (Relation) i.next();
            ageRelation(r);
		}
    }

    /**
     * A players relations tend to revert to those defined in the player character. So if race X
     * naturally dislikes race Y then giving gifts or stealing tech may improve or deprove a 
     * relationship for a while - but it will gradually get back to its predefined state.
     * 
     * For details of how computer players age their relationships @see mot.engine.players.Strategist.
     */
    public void ageRelation(Relation rel) throws ApplicationError {
        int current = rel.getValue();
        Player p1 = rel.getPlayer1();
        Player p2 = rel.getPlayer2();
        int defined = p1.getPlayerCharacter().getRelationshipRatingTo(p2);
        int diff = current - defined;
        if (Logger.isDebug()) {
            Logger.logDebug("Relations.ageRelations() player1="+p1.getName()+" player2="+p2.getName()+" difference="+diff);
        }
        if (diff > 0) {
            rel.setValue(current-1);
        } else if (diff < 0) {
            rel.setValue(current+1);
        }
    }
    
    /**
     * Calculates the amount of money made or lost on each trade route for a turn and adds
     * that amount to the participants of the trade route. This process all trade routes and
     * should be called at the end of every turn (by using Relations.endTurn()).
     */
    public void processTradeRoutes() throws ApplicationError {
        Iterator i = relations.iterator();
		while (i.hasNext()) {
			Relation r = (Relation) i.next();
            Treaty [] treaties = r.getTreatiesAsArray();
            if (treaties != null) {
                for (int j=0; j<treaties.length; j++) {
                    if (treaties[j] instanceof TradeAgreementTreaty) {
                        TradeAgreementTreaty tt = (TradeAgreementTreaty) treaties[j];
                        tt.endTurn();
                    }
                }
            }
		}
    }

    public void processResearchTreaties() throws ApplicationError {
        Iterator i = relations.iterator();
		while (i.hasNext()) {
			Relation r = (Relation) i.next();
            Treaty [] treaties = r.getTreatiesAsArray();
            if (treaties != null) {
                for (int j=0; j<treaties.length; j++) {
                    if (treaties[j] instanceof ResearchAgreementTreaty) {
                    	ResearchAgreementTreaty rt = (ResearchAgreementTreaty) treaties[j];
                        rt.endTurn();
                    }
                }
            }
		}
    }
    /**
     * @return The total income earnt by this player at the end of this turn from trading.
     */
    public int getTradeIncome(Player player) throws ApplicationError {
        Iterator i = relations.iterator();
        int total = 0;
		while (i.hasNext()) {
			Relation r = (Relation) i.next();
            Treaty [] treaties = r.getTreatiesAsArray();
            if (treaties != null && r.contains(player)) {
                for (int j=0; j<treaties.length; j++) {
                    if (treaties[j] instanceof TradeAgreementTreaty) {
                        TradeAgreementTreaty tt = (TradeAgreementTreaty) treaties[j];
                        total += tt.getEarnings(player);
                    }
                }
            }
		}
        return total;
    }

    /**
     * @return A list of name value pairs where each name is the name of the player with whom this
     * player has a trade route and the value is the current income generated by this trade route.
     */
    public BasicNameValueList getTradeIncomeDetails(Player player) throws ApplicationError {
        Iterator i = relations.iterator();
        BasicNameValuePairList nvpl = null;
		while (i.hasNext()) {
			Relation r = (Relation) i.next();
            Treaty [] treaties = r.getTreatiesAsArray();
            if (treaties != null && r.contains(player)) {
                for (int j=0; j<treaties.length; j++) {
                    if (treaties[j] instanceof TradeAgreementTreaty) {
                        if (nvpl == null) nvpl = new BasicNameValuePairList();
                        TradeAgreementTreaty tt = (TradeAgreementTreaty) treaties[j];
                        Player otherPlayer = (player == r.getPlayer1()) ? r.getPlayer2() : r.getPlayer1(); 
                        nvpl.add(otherPlayer.getName(), ""+tt.getEarnings(player));
                    }
                }
            }
		}
        return nvpl;
    }
    
	protected String getElementName() { return "relations"; }

	protected void encode(XMLWriter writer) throws ApplicationError {
        Iterator i = relations.iterator();
		while (i.hasNext()) {
			Relation r = (Relation) i.next();
            if (r != null) {
                r.toXML(writer);
            }
		}
	}
}