/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.relations;

import mot.common.lang.L;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.TechnologyTree;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.players.Player;
import mot.engine.players.PlayerReference;
import mot.engine.players.PlayerResourceCache;

public class ResearchAgreementTreaty extends Treaty {

    /**
     * The date this trade route was started.
     */
    private int dateStarted;
	
    /**
     * Technologies from missed technology list of player researcher1 that will not be missed as a result of this research agreement.
     */
    private TechnologyList missedTechnologies1;
	
    /**
     * Technologies from missed technology list of player researcher2 that will not be missed as a result of this research agreement.
     */
    private TechnologyList missedTechnologies2;

    private PlayerReference researcher1;
    private PlayerReference researcher2;
  
	public ResearchAgreementTreaty() { 
	}

	public ResearchAgreementTreaty(Player researcher1, Player researcher2) throws ApplicationError {
		this.dateStarted = MotEngine.instance.getDate();
    	this.researcher1 = new PlayerReference(researcher1);
    	this.researcher2 = new PlayerReference(researcher2);
    	
        TechnologyList candidates = getCandidatesTechnologies(researcher1, researcher2);
		if (candidates != null && candidates.getQuantity() > 0) {
			Technology t1 = candidates.getRandom();
			//candidates.remove(t1);
			missedTechnologies1 = new GeneralTechnologyList(t1);
			if (Logger.isDebug()) {
				Logger.logDebug("ResearchAgreementTreaty.ResearchAgreementTreaty() - "+researcher1+" Removed Missed Techs "+missedTechnologies1); 
			}
			researcher1.getPlayerCharacterBasic().getMissedTechnologies().remove(missedTechnologies1.getFullNamesList());
			researcher1.getPlayerCharacterBasic().flushMissedTechnologiesCache();
			researcher1.flushTechnologyAvailableForResearch();
		}
    	
        candidates = getCandidatesTechnologies(researcher2, researcher1);
		if (candidates != null && candidates.getQuantity() > 0) {
			Technology t1 = candidates.getRandom();
			//candidates.remove(t1);
			missedTechnologies2 = new GeneralTechnologyList(t1);
			if (Logger.isDebug()) {
				Logger.logDebug("ResearchAgreementTreaty.ResearchAgreementTreaty() - "+researcher2+" Removed Missed Techs "+missedTechnologies2); 
			}
			researcher2.getPlayerCharacterBasic().getMissedTechnologies().remove(missedTechnologies2.getFullNamesList());
			researcher2.getPlayerCharacterBasic().flushMissedTechnologiesCache();
			researcher2.flushTechnologyAvailableForResearch();
		}
    }

	private TechnologyList getCandidatesTechnologies(Player player, Player playerOther) throws ApplicationError {
        TechnologyList mt = TechnologyTree.getInstance().getVisibleMissedTechnologyAsList(player);
        if (mt == null) mt = player.getMissedTechnologies();
    	if (mt != null) {
    		TechnologyList candidates = mt.getAllExcluding(playerOther.getMissedTechnologies());
    		removeDiscoveredTechnologies(candidates, player);
    		if (candidates == null || candidates.getQuantity() == 0) {
    			mt = player.getMissedTechnologies();
        		candidates = mt.getAllExcluding(playerOther.getMissedTechnologies());
        		removeDiscoveredTechnologies(candidates, player);
    		}
    		return candidates;
    	} else {
    		return null;
    	}
	}
	
	private void removeDiscoveredTechnologies(TechnologyList tl, Player player) throws ApplicationError {
		if (tl == null) return;
		Technology[] ta = tl.getAllAsArray();
		for (int i = 0; i < ta.length; i++) {
			if (player.hasDiscovered(ta[i])) tl.remove(ta[i]);
		}
	}
	
    public void setResearcher1(PlayerReference v) { researcher1 = v; }
    public void setResearcher2(PlayerReference v) { researcher2 = v; }

    public Player getResearcher1() throws ApplicationError { return researcher1 == null ? null : researcher1.getPlayer(); }
    public Player getResearcher2() throws ApplicationError { return researcher2 == null ? null : researcher2.getPlayer(); }

    public void endTurn() throws ApplicationError {
		if (researcher1 == null || researcher2 == null) return;
		
		Player p1 = researcher1.getPlayer();
		Player p2 = researcher2.getPlayer();

		int researchGenerated = getResearchGenerated();
		
        if (Logger.isDebug()) {
            Logger.logDebug("ResearchAgreementTreaty.endTurn() player1="+p1.getName()+" player2="+p2.getName()+" researchGenerated="+researchGenerated);
        }
        
        p1.addResearch(researchGenerated);
        p2.addResearch(researchGenerated);
        
        // if we have earned money from this agreement let is have a positive effect 
        // on our relationship with the player.
        int age = MotEngine.instance.getDate() - dateStarted;
        if (age >= 100) {
            MotEngine.instance.getRelations().getRelationship(p1, p2).setResearchEffect(20);
        } else if (age >= 50) {
            MotEngine.instance.getRelations().getRelationship(p1, p2).setResearchEffect(10);
        } else if (age >= 10) {
            MotEngine.instance.getRelations().getRelationship(p1, p2).setResearchEffect(5);
        } else if (age > 0) {
            MotEngine.instance.getRelations().getRelationship(p1, p2).setResearchEffect(2);
        }
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("date-started", dateStarted);
        if (researcher1 != null) researcher1.toXML(writer, "researcher1");
        if (researcher2 != null) researcher2.toXML(writer, "researcher2");
        if (missedTechnologies1 != null) missedTechnologies1.referenceToXML(writer, "missed-technologies-1");
        if (missedTechnologies2 != null) missedTechnologies2.referenceToXML(writer, "missed-technologies-2");
	}

	protected String getElementName() { return "research-agreement-treaty"; }

    public String getName() {
        return L.TREATY_RESEARCH;
    }

    public String getDescription(Player p1, Player p2) throws ApplicationError {
        return "Research Treaty established "+dateStarted+" generating "+getResearchGenerated();
    }

	public int getDateStarted() {
		return dateStarted;
	}

	public void setDateStarted(int dateStarted) {
		this.dateStarted = dateStarted;
	}

	public void setMissedTechnologies1(TechnologyList v) {
		this.missedTechnologies1 = v;
	}

	public void setMissedTechnologies2(TechnologyList v) {
		this.missedTechnologies2 = v;
	}

	public TechnologyList getMissedTechnologies1() {
		return missedTechnologies1;
	}

	public Technology [] getMissedTechnologies1AsArray() {
		if (missedTechnologies1 == null) {
			return null;
		} else {
			return missedTechnologies1.getAllAsArray();
		}
	}

	public TechnologyList getMissedTechnologies2() {
		return missedTechnologies2;
	}

	public Technology [] getMissedTechnologies2AsArray() {
		if (missedTechnologies2 == null) {
			return null;
		} else {
			return missedTechnologies2.getAllAsArray();
		}
	}

	public int getResearchGenerated() throws ApplicationError {
		Player p1 = researcher1.getPlayer();
		PlayerResourceCache trader1Resources = p1.getResourceCache();
		Player p2 = researcher2.getPlayer();
		PlayerResourceCache trader2Resources = p2.getResourceCache();
		int p1Research = p1.getTotalResearch();
		int p2Research = p2.getTotalResearch();
		int researchGenerated = (p1Research + p2Research) / 20;
		return researchGenerated;
	}

	/**
	 * @return Technologies on the missed list of player that will be available for research during this research agreement.
	 */
	public TechnologyList getVoidedMissedTechnologies(Player player) {
		if (Comp.same(player, researcher1)) {
			return missedTechnologies1;
		} else if (Comp.same(player, researcher2)) {
			return missedTechnologies2;
		} else {
			return null;
		}
	}

	public String getResearcher1Name() {
		if (researcher1 == null) return null;
		return researcher1.getName();
	}

	public String getResearcher2Name() {
		if (researcher2 == null) return null;
		return researcher2.getName();
	}

}