/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.relations;

import mot.common.lang.L;
import mot.common.resources.Resource;
import mot.common.resources.Resources;
import mot.common.technology.Computers;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.event.MessageEF;
import mot.engine.players.Player;
import mot.engine.players.PlayerReference;
import mot.engine.players.PlayerResourceCache;

public class TradeAgreementTreaty extends Treaty {

    public static int TURNS_BEFORE_PROFIT = 10;
    public static int COST_TO_BUILD_ROUTE = -10;

    private PlayerReference trader1;
    private PlayerReference trader2;
    
    /**
     * Special resource received by payer1 (owner by player2) due to this trade agreement
     */
    private Resource resource1;
    
    /**
     * Special resource received by payer2 (owner by player1) due to this trade agreement
     */
    private Resource resource2;
    
    /**
     * The date this trade route was started.
     */
    private int dateStarted;
    
    public int getDateStarted() { return dateStarted; }
    public void setDateStarted(int v) { dateStarted = v; }

    public void setTrader1(PlayerReference v) { trader1 = v; }
    public void setTrader2(PlayerReference v) { trader2 = v; }
    
    public Player getTrader1() throws ApplicationError { return trader1 == null ? null : trader1.getPlayer(); }
    public Player getTrader2() throws ApplicationError { return trader2 == null ? null : trader2.getPlayer(); }

	public Resource getResource1() { return resource1; }
	public void setResource1(Resource v) { this.resource1 = v; }
	public void setResource1(String v) { this.resource1 = Resources.getResource(v); }

	public Resource getResource2() { return resource2; }
	public void setResource2(Resource v) { this.resource2 = v; }
	public void setResource2(String v) { this.resource2 = Resources.getResource(v); }

    public TradeAgreementTreaty() {
    }
    
    public TradeAgreementTreaty(Player trader1, Player trader2) throws ApplicationError {
		this.dateStarted = MotEngine.instance.getDate();
    	this.trader1 = new PlayerReference(trader1);
    	this.trader2 = new PlayerReference(trader2);
    	
    	updateResources(false);    	
    	
        if (Logger.isDebug()) {
            Logger.logDebug("TradeAgreementTreaty.init() player1="+trader1.getName()+" player2="+trader2.getName());
        }
    }
    
	private void updateResources(boolean sendUpdateNotifications) throws ApplicationError {
		if (trader1 == null || trader2 == null) return;
		
		Player player1 = trader1.getPlayer();
		PlayerResourceCache trader1Resources = player1.getResourceCache();
		Player player2 = trader2.getPlayer();
		PlayerResourceCache trader2Resources = player2.getResourceCache();
		
		if (resource1 != null && (trader2Resources == null || !trader2Resources.contains(resource1))) {
			Logger.logDebug("TradeAgreementTreaty.updateResources() player "+player1+" lost resource "+resource1);
			if (sendUpdateNotifications) MotEngine.instance.broadcast(new MessageEF("Trade Agreement with "+player2.getName()+" lost resource "+resource1.getName()), player1);
			MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(player1, player2).addDiplomaticHistoryEvent(player1.getName()+" lost resource "+resource1.getName());
			resource1 = null;
		}
		if (resource2 != null && (trader1Resources == null || !trader1Resources.contains(resource2))) {
			Logger.logDebug("TradeAgreementTreaty.updateResources() player "+player2+" lost resource "+resource2);
			if (sendUpdateNotifications) MotEngine.instance.broadcast(new MessageEF("Trade Agreement with "+player1.getName()+" lost resource "+resource2.getName()), player2);
			MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(player1, player2).addDiplomaticHistoryEvent(player2.getName()+" lost resource "+resource2.getName());
			resource2 = null;
		}
		
		if (resource1 == null && trader2Resources != null) {
			resource1 = trader2Resources.getRandomResourceExcluding(trader1Resources);
			if (resource1 != null) {
				Logger.logDebug("TradeAgreementTreaty.updateResources() player "+player1+" gained resource "+resource1);
				if (sendUpdateNotifications) MotEngine.instance.broadcast(new MessageEF("Trade Agreement with "+player2.getName()+" gained resource "+resource1.getName()), player1);
				MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(player1, player2).addDiplomaticHistoryEvent(player1.getName()+" gained resource "+resource1.getName());
			}
		}
		
		if (resource2 == null && trader1Resources != null) {
			resource2 = trader1Resources.getRandomResourceExcluding(trader2Resources);
			if (resource2 != null) {
				Logger.logDebug("TradeAgreementTreaty.updateResources() player "+player2+" gained resource "+resource2);
				if (sendUpdateNotifications) MotEngine.instance.broadcast(new MessageEF("Trade Agreement with "+player1.getName()+" gained resource "+resource2.getName()), player2);
				MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(player1, player2).addDiplomaticHistoryEvent(player2.getName()+" gained resource "+resource2.getName());
			}
		}
	}
    
    /**
     * @return The amount earn this turn from this trade route by this player.
     */
    public int getEarnings(Player player) throws ApplicationError {
    	if (trader1 == null || trader2 == null) {
    		Logger.logWarning("TradeAgreementTreaty.getEarnings() Has null trader");
    		return 0;
    	}
    	Player p1 = trader1.getPlayer();
    	Player p2 = trader2.getPlayer();
    	if (p1 == null || p1 == null) {
    		Logger.logWarning("TradeAgreementTreaty.getEarnings() Has null player (broken player reference)");
    		return 0;
    	}
    	int earnings = COST_TO_BUILD_ROUTE;
    	int timeOperating = MotEngine.instance.getDate() - dateStarted;
        if (timeOperating >= TURNS_BEFORE_PROFIT) {
        	int pop1 = p1.getTotalPopulation();
        	int pop2 = p2.getTotalPopulation();
            earnings = (pop1+pop2) / 20;
        }
        double bonus = 1.0;
        int additional = 0;
        if (player != null) {
            bonus += player.getTradingBonus();
            Computers computers = player.getComputers();
            additional = (computers == null) ? 0 : computers.getCommerce();
        }
        if (earnings < 0) {
            return (int) (earnings + (-earnings * (bonus-1.0))) + additional;
        } else {
            return (int) (earnings * bonus) + additional;
        }
    }
    
    /**
     * @return The resource received by player because of this trade agreement. May be Null.
     */
	public Resource getResourceRecieved(Player player) {
		if (Comp.same(player, trader1)) {
			return resource1;
		} else if (Comp.same(player, trader2)) {
			return resource2;
		} else {
			return null;
		}
	}

	public String getResourceRecievedName(Player player) {
		Resource r = getResourceRecieved(player);
		if (r == null) {
			return null;
		} else {
			return r.getName();
		}
	}
	
    /**
     * @return The resource given away by player because of this trade agreement. May be Null.
     */
	public Resource getResourceGiven(Player player) {
		if (Comp.same(player, trader1)) {
			return resource2;
		} else if (Comp.same(player, trader2)) {
			return resource1;
		} else {
			return null;
		}
	}

	public String getResourceGivenName(Player player) {
		Resource r = getResourceGiven(player);
		if (r == null) {
			return null;
		} else {
			return r.getName();
		}
	}

    /**
     * Add revenue to both traders as profits from a trade route. Note this may be negative!
     * Adding trade revenue effects the relationship between these two players.
     * @param p1 
     * @param p1 
     */
    public void endTurn() throws ApplicationError {
    	if (trader1 == null || trader2 == null) {
    		Logger.logWarning("TradeAgreementTreaty.endTurn() Has null trader");
    		return;
    	}
    	
    	updateResources(true);
    	
    	Player p1 = trader1.getPlayer();
    	Player p2 = trader2.getPlayer();
    	if (p1 == null || p1 == null) {
    		Logger.logWarning("TradeAgreementTreaty.endTurn() Has null player (broken player reference)");
    		return;
    	}

    	int earnings1 = getEarnings(p1);
    	int earnings2 = getEarnings(p2);
        if (Logger.isDebug()) {
            Logger.logDebug("TradeTreaty.endTurn() player1="+p1.getName()+" player2="+p2.getName()+" player1 earnings="+earnings1+" player2 earnings="+earnings2);
        }
        p1.addFunds(earnings1);
        p2.addFunds(earnings2);
        
        // if we have earned money from this agreement let is have a positive effect 
        // on our relationship with the player.
        int age = MotEngine.instance.getDate() - dateStarted;
        if (age >= 100) {
            MotEngine.instance.getRelations().getRelationship(p1, p2).setTradeEffect(20);
        } else if (age >= 50) {
            MotEngine.instance.getRelations().getRelationship(p1, p2).setTradeEffect(10);
        } else if (age >= 10) {
            MotEngine.instance.getRelations().getRelationship(p1, p2).setTradeEffect(5);
        } else if (age > 0) {
            MotEngine.instance.getRelations().getRelationship(p1, p2).setTradeEffect(2);
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("date-started", dateStarted);
        if (trader1 != null) trader1.toXML(writer, "trader1");
        if (trader2 != null) trader2.toXML(writer, "trader2");
        if (resource1 != null) {
        	writer.addElement("resource1", resource1.getName());
        }
        if (resource2 != null) {
        	writer.addElement("resource2", resource2.getName());
        }
	}

	protected String getElementName() { return "trade-agreement-treaty"; }

    public String getName() {
        return L.TREATY_TRADE;
    }
    
    public String getDescription(Player p1, Player p2) throws ApplicationError {
        return "Trade Treaty established "+dateStarted+" earning "+getEarnings(null);
    }
	
}