/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.setup;

import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.planets.Planet;

/**
 * This is a factory used to generate monsters and the planets they guard. Use the static create
 * methods to create a random monster or a specific size of monster. This factory performs the
 * following;
 *  Makes the planet desirable (e.g. size 120, Terraform 1.0).
 *  Create a new craft owned by the monster player that will protect this planet.
 */
public abstract class MonsterFactory  {

    public static Class [] types = { MonsterFactorySmall.class, 
                                     MonsterFactorySmall.class, 
                                     MonsterFactoryMedium.class };

    /**
     * Make this planet a very desirable planet and place a monster guarding it.
     * The type of monster and quality of the planet is randomly chosen from the
     * types available.
     * @param planet The planet to station the monster around.
     */
    public static void create(Planet planet) throws ApplicationError {
        create(planet, types[Utils.random(types.length)]);
    }
    
    /**
     * Make this planet a very desirable planet and place a monster guarding it.
     * The type of monster and quality of the planet is randomly chosen from the
     * types available.
     * @param planet The planet to station the monster around.
     */
    public static void create(Planet planet, Class monsterClass) throws ApplicationError {
        MonsterFactory factory = create( monsterClass );
        factory.doCreate(planet);
    }
    
    /**
     * Make this planet a very desirable planet and place a monster guarding it.
     * @param craft The planet to station the monster around.
     */
    private static MonsterFactory create(Class type) throws ApplicationError {
        try {
            return (MonsterFactory) type.newInstance();
        } catch (InstantiationException e) {
            Logger.logError("Error :", e);
            throw new ApplicationError("Unable to create Monster due to error "+e);
        } catch (IllegalAccessException e) {
            Logger.logError("Error :", e);
            throw new ApplicationError("Unable to create Monster due to error "+e);
        }
    }
    
    protected MonsterFactory() {
    }

    /**
     * This method actually creates the monster and converts the planet into a desirable 
     * planet guarded by the monster.
     * @param planet The planet to be guarded by the monster.
     */
    protected abstract void doCreate(Planet planet) throws ApplicationError;
}