/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.setup;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.bean.Circle;
import mot.common.bean.GalaxyDefinitions;
import mot.common.bean.GameConstants;
import mot.common.bean.Location;
import mot.common.bean.LocationList;
import mot.common.bean.NameList;
import mot.common.bean.Nebula;
import mot.common.bean.NewGameData;
import mot.common.bean.NewGameDataComputerPlayerDetails;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.bean.PlayerAiSettings;
import mot.common.bean.PlayerColor;
import mot.common.bean.PlayerColourList;
import mot.common.bean.RaceGraphics;
import mot.common.bean.RaceGraphicsList;
import mot.common.bean.Rectangle;
import mot.common.bean.Size;
import mot.common.bean.StellarObjectGraphics;
import mot.common.factory.GraphicsFactory;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.NameGenerator;
import mot.common.util.PropertyManager;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.xml.XMLDocumentHandler;
import mot.engine.MotEngine;
import mot.engine.common.MoveableOwnedObjects;
import mot.engine.common.StellarObject;
import mot.engine.crafts.CraftFactory;
import mot.engine.planets.BlackHole;
import mot.engine.planets.BlackHoleFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.PlanetFactory;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.StellarObjects;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.prizes.PrizeFactory;
import mot.engine.relations.Relations;

public class MotFactory {

    public static final int MIN_DIST_TO_EDGE = 35;

	private static final double SS_PER_PLANET_RATIO = 0.80;
    
    private static final String ALIGNEMNT_OPTION = "ALIGNMENT_OP";

	/**
	 * Create a new universe containing qtyPlayers players and qtyPlanets planets.
	 * The planets are distributed among a random number of star systems (between
	 * qtyPlanets and qtyPlanets*SS_PER_PLANET_RATIO.
     * 
     *  For each user player desired you must provide a list of race options configuring that race.
     *  These are passed in using userOptions parameter. We can thus determine the quantity of user 
     *  players by the size of this array.
     *  
     *  The level is the level of difficulty the game is being played at. A value 1.0 means average. 
     *  However as this value increases then computer players start to get production bonuses, research 
     *  bonuses and tend to like user players less. A level of 1.5 would give all computer player a 50%
     *  production bonus and like the human players 12.5% less.
	 * @param hasTutorial 
	 */
	public static MotEngine create(NewGameData gameData, NewGameUserPlayerData [] userData) throws ApplicationError, ValidationError {
		boolean hasTutorial = gameData.isHasTutorial();
		NewGameDataComputerPlayerDetails[] computerPlayers = gameData.getComputerPlayers();
		int qtyPlanets = gameData.getQtyPlanets();
		int qtyNebula = gameData.getQtyNebula();
		Size galaxySize = gameData.getGalaxySize();
		Double level = gameData.getLevel();
		boolean hasWormHole = gameData.isHasWormHole();
		boolean hasSpaceKaan = gameData.isHasSpaceKaan();
        return create(hasTutorial, hasWormHole, hasSpaceKaan, level, computerPlayers, qtyPlanets, qtyNebula, galaxySize, userData, true);
    }
    
	public static MotEngine create(boolean hasTutorial, boolean hasWormHole, boolean hasSpaceKaan, Double level, NewGameDataComputerPlayerDetails[] computerPlayers, int qtyPlanets, int qtyNebula, Size galaxySize, NewGameUserPlayerData [] userData, boolean addRandomRelationAdjustment) throws ApplicationError, ValidationError {
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.create() Create new game setup options ="+Comp.toStringOnAll(userData));
        }
        
		// Create a new Game instance
		MotEngine engine = MotEngine.newInstance((userData.length == 1) ? userData[0].getRaceName() : null);

        int qtyUserPlayers = userData.length; 
        int qtyComputerPlayers = computerPlayers == null ? 0 : computerPlayers.length;
        
        if (computerPlayers != null) {
        	selectRandomRacesForComputerPlayers(computerPlayers, userData);
        }

		// Create races for new game
        int minSS = qtyPlanets;
        int maxSS = (int)(qtyPlanets*SS_PER_PLANET_RATIO);
        if (minSS > maxSS) {
            int t = minSS;
            minSS = maxSS;
            maxSS = t;
        }
		double qtySolarSystems = minSS + Utils.random(maxSS - minSS);
        if (qtySolarSystems < 2) qtySolarSystems = 2;
		int qtyPlayers = qtyUserPlayers + qtyComputerPlayers;
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.create() create players qty="+qtyPlayers);
        }

        RaceGraphics [] ancientRaces = GraphicsFactory.getRaceGraphicsList().getGraphicsType(RaceGraphics.TYPE_ANCIENT);
        if (ancientRaces == null) {
            throw new ApplicationError("MotFactory.create() No ancient races");
        }
        
        String [] raceNames = new String[qtyPlayers];
        String [] alignments = new String[qtyPlayers];
        String lastAlignment = null; 
		for (int i=0; i<qtyUserPlayers; i++) {
            raceNames[i] = userData[i].getRaceName();
            alignments[i] = userData[i].getAlignment();
            lastAlignment = alignments[i]; 
        }
        int j = 0;
        if (lastAlignment != null) {
            j = getIndexOf(ancientRaces, lastAlignment);
        }
		for (int i=0; i<qtyComputerPlayers; i++) {
            raceNames[qtyUserPlayers+i] = computerPlayers[i].getRaceName();
            alignments[qtyUserPlayers+i] = ancientRaces[(++j)%ancientRaces.length].getName();
        }
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.create() Create races "+Comp.toString(raceNames));
        }
        
        RaceOptions [] userOptions = new RaceOptions[userData.length];
        for (int i = 0; i < userOptions.length; i++) {
        	userOptions[i] = new RaceOptions(userData[i].getRaceName(), userData[i].getUserOptions());
		}

		// Setup the relations
        setupRelations(userOptions, raceNames, level, alignments, addRandomRelationAdjustment);
        
        if (qtyPlayers > 8) {
            throw new ValidationError("You cannot have more than 8 players");
        }
        UserPlayerFactory upf = new UserPlayerFactory();
        ComputerPlayerFactory cpf = new ComputerPlayerFactory();
		for (int i=0; i<qtyUserPlayers; i++) {
			upf.createNewPlayer(userOptions[i], userData[i].getRaceName(), level, userData[i].getPlayerColour(), userData[i].getAlignment(), null, raceNames);
		}
		PlayerColourList usedColours = new PlayerColourList(userData);
		for (int i=0; i<qtyComputerPlayers; i++) {
			String name = computerPlayers[i].getRaceName();
			Double playerLevel = computerPlayers[i].getLevel();
			RaceOptions raceOptions = OptionsFactory.getRaceOptions(name);
			PlayerAiSettings playerAiSettings = computerPlayers[i].getPlayerAiSettings();
			cpf.createNewPlayer(raceOptions, name, playerLevel, PlayerColor.getAvailableColor(usedColours), alignments[i], playerAiSettings, raceNames);
		}
        
        Player [] playersArray = MotEngine.instance.getPlayers().getPlayers();

		// Load default names
		NameList names = (NameList) XMLDocumentHandler.parseResource("xml/solar_system_names.xml");
        
		StellarObjects solarSystems = new StellarObjects();
        solarSystems.setSize(galaxySize);
        
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.create() create solar systems qty="+qtySolarSystems);
        }
        
		Rectangle solarSystemBounds = new Rectangle(MIN_DIST_TO_EDGE, MIN_DIST_TO_EDGE, galaxySize.getX()-2*MIN_DIST_TO_EDGE, galaxySize.getY()-2*MIN_DIST_TO_EDGE);
		
		SolarSystem tutorialHomeSS = null;
		SolarSystem tutorialFirstSS = null;
		SolarSystem tutorialExploreSS = null;
		SolarSystem tutorialGuardedSS = null;
		
		for (int i=0; i<qtySolarSystems; i++) {
			if (hasTutorial && i == 0) {
				// The tutorial instructs the player to colonise a planet called First1 that is close to their Home Planet
				// We must create this planet artificially!
				tutorialHomeSS = new SolarSystem(GameConstants.TUTORIAL_HOME_NAME, 150, 150);
				tutorialHomeSS.setStarType( StellarObjectGraphics.getRandomStarType() );
				if (Logger.isDebug()) {
					Logger.logDebug("MotFactory.create() tutorial solar system "+tutorialHomeSS.getName()+" at "+tutorialHomeSS.getLocation().getX()+","+tutorialHomeSS.getLocation().getY());
				}
				solarSystems.add( tutorialHomeSS );
				tutorialFirstSS = new SolarSystem(GameConstants.TUTORIAL_FIRST_COLONY_NAME, 150, 75);
				tutorialFirstSS.setStarType( StellarObjectGraphics.getRandomStarType() );
				if (Logger.isDebug()) {
					Logger.logDebug("MotFactory.create() tutorial solar system "+tutorialFirstSS.getName()+" at "+tutorialFirstSS.getLocation().getX()+","+tutorialFirstSS.getLocation().getY());
				}
				solarSystems.add( tutorialFirstSS );
				tutorialExploreSS = new SolarSystem(GameConstants.TUTORIAL_FIRST_EXPLORE_NAME, 75, 150);
				tutorialExploreSS.setStarType( StellarObjectGraphics.getRandomStarType() );
				if (Logger.isDebug()) {
					Logger.logDebug("MotFactory.create() tutorial solar system "+tutorialExploreSS.getName()+" at "+tutorialExploreSS.getLocation().getX()+","+tutorialExploreSS.getLocation().getY());
				}
				solarSystems.add( tutorialExploreSS );
				tutorialGuardedSS = new SolarSystem(GameConstants.TUTORIAL_FIRST_GUIRDED_NAME, 240, 150);
				tutorialGuardedSS.setStarType( StellarObjectGraphics.getRandomStarType() );
				if (Logger.isDebug()) {
					Logger.logDebug("MotFactory.create() tutorial solar system "+tutorialGuardedSS.getName()+" at "+tutorialGuardedSS.getLocation().getX()+","+tutorialGuardedSS.getLocation().getY());
				}
				solarSystems.add( tutorialGuardedSS );
				i = 3;
			} else {
				String name = selectRandomSolarSystemName(names, solarSystems);
				Location best = createNewLocationForStallarObject(galaxySize, solarSystems, solarSystemBounds, name);
				SolarSystem ss = new SolarSystem(name, best.getX(), best.getY());
				ss.setStarType( StellarObjectGraphics.getRandomStarType() );
				if (Logger.isDebug()) {
					Logger.logDebug("MotFactory.create() create solar system "+ss.getName()+" at "+ss.getLocation().getX()+","+ss.getLocation().getY());
				}
				solarSystems.add( ss );
			}
		}
		engine.setStellarObjects(solarSystems);

        // Build a list of candidates solar systems to which we can add planets
        // Currently they are all candidates!
        ArrayList candidateSolarSystemList = new ArrayList();
   		Iterator cssli = solarSystems.getList().iterator();
        while (cssli.hasNext()) {
            SolarSystem ss = (SolarSystem) cssli.next();
            if (ss != tutorialHomeSS && ss != tutorialFirstSS && ss != tutorialExploreSS && ss != tutorialGuardedSS) {
	            CandidateSolarSystem css = new CandidateSolarSystem(ss, 0);
	            candidateSolarSystemList.add(css);
            }
        }

        // Distribute the planets created randomly among the solar systems.
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.create() create planets qty="+qtyPlanets);
        }
        
        ArrayList homePlanetIgnoreList = new ArrayList();
        ArrayList newPlanetIgnoreList = new ArrayList();
        
        if (hasTutorial) {
            homePlanetIgnoreList.add(tutorialFirstSS);
            homePlanetIgnoreList.add(tutorialExploreSS);
            homePlanetIgnoreList.add(tutorialGuardedSS);
            newPlanetIgnoreList.add(tutorialHomeSS);
            newPlanetIgnoreList.add(tutorialFirstSS);
            newPlanetIgnoreList.add(tutorialExploreSS);
            newPlanetIgnoreList.add(tutorialGuardedSS);
        }
        
        int firstIndex = 1;
        LocationList homesLocationList = new LocationList();
        int minDistanceBetweenPlayers = (int) (galaxySize.getX() / 2.25);
        Planet tutorialGuardedPlanet = null;
		for (int i=0; i<qtyPlanets; i++) {
            CandidateSolarSystem css = null;
            if (i < qtyPlayers) {
            	if (hasTutorial && i == 0) {
            		css = new CandidateSolarSystem(tutorialHomeSS, 1);
            	} else {
            		css = selectRandomSolarSystem(candidateSolarSystemList, homePlanetIgnoreList);
            		int count = 10;
            		int dist = homesLocationList.getClosestDistanceTo(css.solarSystem.getLocation());
            		// We ensure that players do not start close too close to other players
            		while (count > 0 && dist < minDistanceBetweenPlayers) {
            			css = selectRandomSolarSystem(candidateSolarSystemList, homePlanetIgnoreList);
            			dist = homesLocationList.getClosestDistanceTo(css.solarSystem.getLocation());
            			count--;
            		}
            	}
				homesLocationList.add(css.solarSystem.getLocation());
            } else {
                css = selectRandomSolarSystem(candidateSolarSystemList, newPlanetIgnoreList);
            }
			SolarSystem ss = css.solarSystem;
			if (i < qtyPlayers) {
				// This planet is a home world
				PlanetFactory.createHomePlanet(playersArray[i], ss, level);
				if (level <= 1.5 && playersArray[i] instanceof UserPlayer) {
					PlanetFactory.createBonusPlanet(ss);
				}
				if (hasTutorial && i == 0) {
					// The tutorial instructs the player to colonise a planet called First1 that is close to their Home Planet
					// We must create this planet artificially!
					tutorialFirstSS.addVisitor(playersArray[i], false);
					tutorialExploreSS.addVisitor(playersArray[i], false);
					PlanetFactory.createTutorialFirstColony(tutorialFirstSS);
					tutorialGuardedPlanet = PlanetFactory.createTutorialGuardedPlanet(tutorialGuardedSS);
				}
                homePlanetIgnoreList.add(css.solarSystem);
			} else {
				// This planet is not owned
				PlanetFactory.createRandomPlanet(ss);
			}
		}

		// Each race starts with an explorer and a coloniser
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.create() create coloniser and scout craft for each player qtyPlayers="+playersArray.length);
        }
		for (int i=0; i<playersArray.length; i++) {
			Player player = playersArray[i];
            // NOTE - We must not give orders because not all the players have been created
            // and the monsters do not yet exist!
            CraftFactory.create(player, player, "Coloniser", false, false);
            CraftFactory.create(player, player, "Scout", false, false);
		}

		engine.setDate(MotEngine.START_DATE);
		engine.setLevel(level.doubleValue());
		engine.setSpaceKaanEnabled(hasSpaceKaan);
        
        createMonsters(newPlanetIgnoreList, tutorialGuardedPlanet);
        createPrizes(newPlanetIgnoreList, tutorialExploreSS);
        createAncients(newPlanetIgnoreList);
        createResources();
        for (int i = 0; i < qtyNebula; i++) {			
        	createNebulae(solarSystems, galaxySize);
		}
        createBlackHoles(solarSystems, galaxySize, solarSystemBounds);
        if (hasWormHole) createWormHoles(solarSystems, galaxySize, solarSystemBounds);
        createGalacticImcumbents(galaxySize);

        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.create() Finished setup");
            Logger.flushLog();
        }
		return engine;
	}

	private static Location createNewLocationForStallarObject(Size galaxySize, StellarObjects stellarObjects, Rectangle solarSystemBounds, String name) throws ApplicationError {
		int x = Utils.random(galaxySize.getX()-MIN_DIST_TO_EDGE*2)+MIN_DIST_TO_EDGE;
		int y = Utils.random(galaxySize.getY()-MIN_DIST_TO_EDGE*2)+MIN_DIST_TO_EDGE;
		int attempt = 0;
		Location best = new Location(x,y);
		int bestDist = 0;
		int MAX_ATTEMPTS = 50;
		while (tooCloseToExistingSolarSystem(best.getX(), best.getY(), stellarObjects) && attempt < MAX_ATTEMPTS) {
		    x = Utils.random(galaxySize.getX()-MIN_DIST_TO_EDGE*2)+MIN_DIST_TO_EDGE;
		    y = Utils.random(galaxySize.getY()-MIN_DIST_TO_EDGE*2)+MIN_DIST_TO_EDGE;
			Location loc = findNewLocForSS(x, y, stellarObjects, solarSystemBounds);
			int newDist = stellarObjects.getClosestStellarObject(loc).getDistanceTo(loc);
			if ((bestDist == 0 || newDist > bestDist) && solarSystemBounds.contains(loc.getX(), loc.getY())) {
				bestDist = newDist;
				best = loc;
			}
		    attempt++;
		}
		if (bestDist == 0) {
			int dist = -1;
			if (stellarObjects != null) {
				StellarObject closest = stellarObjects.getClosestStellarObject(best);
				if (closest != null) {
					dist = closest.getDistanceTo(best);
				}
			}
			Logger.logWarning("Stellar Object is too close "+name+" at x="+x+" y="+y+" dist="+dist);
		}
		return best;
	}
	
	private static Location findNewLocForSS(int x, int y, StellarObjects ss, Rectangle ssBounds) throws ApplicationError {
		Location loc = new Location(x,y);
		StellarObject closestStellarObject = ss.getClosestStellarObject(loc);
		int dist = loc.getDistanceTo(closestStellarObject.getLocation());
		int direction = loc.getDirectionFor(closestStellarObject.getLocation());
		direction = (direction + 180) % 360;
		loc.shiftInDirection(direction, GalaxyDefinitions.MIN_DISTANCE_BETWEEN_SOLAR_SYSTEMS - dist + 2);
		if (!tooCloseToExistingSolarSystem(loc.getX(), loc.getY(), ss) && ssBounds.contains(loc)) {
			return loc;
		}
		Location best = loc;
		int bestDist = ss.getClosestStellarObject(loc).getDistanceTo(loc);
		// try moving north
		loc = findNewLocForSS(x, y, 0, -5, ss);
		if (!tooCloseToExistingSolarSystem(loc.getX(), loc.getY(), ss) && ssBounds.contains(loc)) {
			return loc;
		}
		if (ss.getClosestStellarObject(loc).getDistanceTo(loc) > bestDist && ssBounds.contains(loc)) {
			best = loc;
			bestDist = ss.getClosestStellarObject(loc).getDistanceTo(loc);
		}
		// try moving south
		loc = findNewLocForSS(x, y, 0, 5, ss);
		if (!tooCloseToExistingSolarSystem(loc.getX(), loc.getY(), ss) && ssBounds.contains(loc)) {
			return loc;
		}
		if (ss.getClosestStellarObject(loc).getDistanceTo(loc) > bestDist && ssBounds.contains(loc)) {
			best = loc;
			bestDist = ss.getClosestStellarObject(loc).getDistanceTo(loc);
		}
		// try moving east
		loc = findNewLocForSS(x, y, -5, 0, ss);
		if (!tooCloseToExistingSolarSystem(loc.getX(), loc.getY(), ss) && ssBounds.contains(loc)) {
			return loc;
		}
		if (ss.getClosestStellarObject(loc).getDistanceTo(loc) > bestDist && ssBounds.contains(loc)) {
			best = loc;
			bestDist = ss.getClosestStellarObject(loc).getDistanceTo(loc);
		}
		// try moving west
		loc = findNewLocForSS(x, y, 5, 0, ss);
		if (!tooCloseToExistingSolarSystem(loc.getX(), loc.getY(), ss) && ssBounds.contains(loc)) {
			return loc;
		}
		if (ss.getClosestStellarObject(loc).getDistanceTo(loc) > bestDist && ssBounds.contains(loc)) {
			best = loc;
			bestDist = ss.getClosestStellarObject(loc).getDistanceTo(loc);
		}
		return best;
	}

	private static Location findNewLocForSS(int x, int y, int dx, int dy, StellarObjects ss) {
		Location loc = new Location(x+dx,y+dy);
		int MaxQty = 20;
		int qty = 2;
		while (tooCloseToExistingSolarSystem(loc.getX()+dx*qty, loc.getY()+dy*qty, ss) && qty < MaxQty) {
			qty++;
		}
		return loc;
	}

	private static int getIndexOf(RaceGraphics [] ancientRaces, String ancientRace) {
        if (ancientRaces == null) return 0;
        for (int i=0; i<ancientRaces.length;i++) {
            if (Comp.same(ancientRaces[i].getName(), ancientRace)) {
                return i;
            }
        }
        return 0;
    }

	/**
     * The graphics XML file contains a list of all graphical information for all races. It is the
     * only place that has a list of all race names defined in the system.
     * @param qtyPlayers The quantity of players to create
     * @param exclude A list of player names that must not be selected
     * @return RaceGraphics item for each race selected
     * @throws mot.common.util.ApplicationError
     */
	private static void selectRandomRacesForComputerPlayers(NewGameDataComputerPlayerDetails[] computerPlayers, NewGameUserPlayerData [] userData) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.getRandomListOfRaces() Qty Races "+computerPlayers.length);
        }
        
        ArrayList<String> exclude = new ArrayList<String>();
        
        for (int i = 0; i < userData.length; i++) {
			exclude.add(userData[i].getRaceName());
		}
        
        int qtyRandomPlayers = 0;
        int qtySpecifiedPlayers = 0;
        for (int i = 0; i < computerPlayers.length; i++) {
			if (computerPlayers[i].getRaceName() != null) {
				exclude.add(computerPlayers[i].getRaceName());
				qtySpecifiedPlayers++;
			} else {
				qtyRandomPlayers++;
			}
		}
        
        // Create a list of candidate races
	    RaceGraphicsList graphicsList = GraphicsFactory.getRaceGraphicsList();
        ArrayList candidates = new ArrayList();
        Iterator it = graphicsList.getGraphics().iterator();
        while (it.hasNext()) {
            RaceGraphics race = (RaceGraphics) it.next();
            if (Comp.same(race.getType(), RaceGraphics.TYPE_PLAYER) && !Comp.in(race.getName(), exclude)) {
                candidates.add(race);
            }
        }
        
		if (qtyRandomPlayers > candidates.size()) {
			throw new ApplicationError("MotFactory.getRandomListOfRaces() qtyPlayers="+qtyRandomPlayers+" but only "+candidates.size()+" races defined" );
		}
		
	    ArrayList list = new ArrayList();
		for (int i = 0; i < computerPlayers.length; i++) {
			if (computerPlayers[i].getRaceName() == null) {
				RaceGraphics rg = (RaceGraphics) Utils.random(candidates);
				computerPlayers[i].setRaceName(rg.getName());
				candidates.remove(rg);
				if (Logger.isDebug()) {
                    Logger.logDebug("MotFactory.getRandomListOfRaces() Select race "+rg.getName());
                }
			}
		}
	}

	private static RaceGraphics [] getRaceGraphics(NewGameDataComputerPlayerDetails[] computerPlayers) throws ApplicationError {
	    RaceGraphicsList graphicsList = GraphicsFactory.getRaceGraphicsList();
		ArrayList<RaceGraphics> list = new ArrayList<RaceGraphics>();
		for (int i = 0; i < computerPlayers.length; i++) {
			RaceGraphics rg = graphicsList.getGraphicsFor(computerPlayers[i].getRaceName());
			list.add(rg);
		}
		return (RaceGraphics []) list.toArray(new RaceGraphics[list.size()]);
	}
	
    private final static double RANDOM_ADJUSTMENT = 0.2;
    private final static int ALIGNMENT_ADJUSTMENT_SAME = +10;
    private final static int ALIGNMENT_ADJUSTMENT_DIFFERNT = -10;
    private final static int ALIGNMENT_ADJUSTMENT_INDEPENDENT = 0;


	public static void setupRelations(RaceOptions [] userOptions, String [] playerNames, Double level, String [] alignments) throws ApplicationError {
        setupRelations(userOptions, playerNames, level, alignments, true);
    }
    
	/**
     * Creates and sets with the correct values the MotEngine relations object. For each race in
     * playerNames a relation is created for every other race. UserPlayers must exist in the userOptions
     * array and these options are used for each user race. For computer players, the default race options
     * are used for a player with that name.
     * @param userOptions A list of options (1 for every user player).
     * @param playerNames The list of all users in the game (user and computer).
     * @param level The level this game is being played at - will determine relationship bonuses for 
     * computer players.
     * @param alignments an array of the alignments for the list of players
     * @throws mot.common.util.ApplicationError
     */
	public static void setupRelations(RaceOptions [] userOptions, String [] playerNames, Double level, String [] alignments, boolean addRandomAdjustment) throws ApplicationError {
		// now setup the default relations for this races
		Relations relations = MotEngine.instance.getRelations();
		if (relations == null) {
			relations = new Relations();
			MotEngine.getInstance().setRelations(relations);
		}
        
        for (int i=0; i<playerNames.length; i++) {
            String name1 = playerNames[i];
            String alignment1 = alignments == null ? null : alignments[i];
            RaceOptions ro1 = getRaceOptions(name1, userOptions);
            for (int j=0; j<playerNames.length; j++) {
                String name2 = playerNames[j];
                String alignment2 = alignments == null ? null : alignments[j];
                if (relations.get(name1, name2) == null && !Comp.same(name1, name2)) { 
                    RaceOptions ro2 = getRaceOptions(name2, userOptions);
                    int charisma1 = ro1.getIntForName("charisma");
                    int charisma2 = ro2.getIntForName("charisma");
                    int relationship = Relations.DEFAULT_RELATIONSHIP_VALUE + charisma1 + charisma2;
                    if (level != null && level.doubleValue() > 0) {
                        int penalty = 0;
                        double ld = level.doubleValue();
                        if (ld >= 2.9) penalty = 20;
                        else if (ld >= 1.9) penalty = 10;
                        if (isUserPlayer(name1, userOptions)) {
                            relationship -= penalty;
                        }
                        if (isUserPlayer(name2, userOptions)) {
                            relationship -= penalty;
                        }
                		double introversionRating1 = ro1.getDoubleForName("introversion-rating");
                		double introversionRating2 = ro2.getDoubleForName("introversion-rating");
                        relationship = (int)(relationship * introversionRating1 * introversionRating2);
                    }
                    if (alignment1 == null || !Comp.same(alignment1, alignment2)) {
                        if (alignment1 != null && alignment2 != null) {
                            relationship += ALIGNMENT_ADJUSTMENT_DIFFERNT;
                        } else {
                            relationship += ALIGNMENT_ADJUSTMENT_INDEPENDENT;
                        }
                    } else {
                        relationship += ALIGNMENT_ADJUSTMENT_SAME;
                    }
                    if (addRandomAdjustment) {
                        relationship += Utils.randomDouble(RANDOM_ADJUSTMENT*2) - RANDOM_ADJUSTMENT;
                    }
                    if (Logger.isDebug()) {
                        Logger.logDebug("MotFactory.setupRelations() set relation for "+name1+","+name2+" to "+relationship+" [charisma1="+charisma1+" charisma2="+charisma2+" level="+level+"]" );
                    }
                    relations.set(name1, name2, relationship);
                }
            }
        }
	}

    /**
     * Returns the correct RaceOptions for the player with name passed in.
     * @param name The name of the player we want to race option for
     * @param raceOptions The list of options for user players
     * @return The correct RaceOptions for the player with name passed in
     * @throws mot.common.util.ApplicationError
     */
    public static RaceOptions getRaceOptions(String name, RaceOptions [] raceOptions) throws ApplicationError {
        if (raceOptions != null) {
            for (int i=0; i<raceOptions.length; i++) {
                if (Comp.same(name, raceOptions[i].getName())) {
                    return raceOptions[i];
                }
            }
        }
        return OptionsFactory.getRaceOptions(name);
    }
    
    /**
     * Search the race options looking for a player named name. If found it returns true otherwise 
     * it returns false.
     * @param name The name of the player
     * @param userOptions The list of options for user players
     * @return There if and only if the player name passed in is a user player.
     * @throws mot.common.util.ApplicationError
     */
    public static boolean isUserPlayer(String name, RaceOptions [] userOptions) throws ApplicationError {
        for (int i=0; i<userOptions.length; i++) {
            if (Comp.same(name, userOptions[i].getName())) {
                return true;
            }
        }
        return false;
    }
    
	/**
	 * Returns a random solar system that still has space to add another planet.
	 */
	private static CandidateSolarSystem selectRandomSolarSystem(ArrayList candidateSolarSystemList, ArrayList ignoreList) throws ApplicationError {
		if (candidateSolarSystemList == null || candidateSolarSystemList.size() == 0) {
			throw new ApplicationError("MotMactory.selectRandomSolarSystem() No solar systems exist");
		}

		CandidateSolarSystem css = (CandidateSolarSystem) Utils.random(candidateSolarSystemList);

        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.selectRandomSolarSystem() Select "+css.solarSystem.getName());
        }
        
        if (ignoreList != null) {
            if (candidateSolarSystemList.size() - ignoreList.size() < 1) {
                throw new ApplicationError("MotFactory.selectRandomSolarSystem() Not enough solar systems for home worlds");
            }
            // Must not be another player using this solar system
            while (ignoreList.contains(css.solarSystem)) {
                if (Logger.isDebug()) {
                    Logger.logDebug("MotFactory.selectRandomSolarSystem() RESELECT "+css.solarSystem.getName()+" previouse was used as a home planet by another race");
                }
                css = (CandidateSolarSystem) Utils.random(candidateSolarSystemList);        
            }
        }
        
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.selectRandomSolarSystem() Select "+css.solarSystem.getName());
        }
        
        css.qtyPlanets++;
		if (css.qtyPlanets >= GalaxyDefinitions.MAX_PLANETS_PER_SOLAR_SYSTEM) {
            candidateSolarSystemList.remove(css);
        }

        return css;
	}

	/**
	 * Returns a random solar system name that is not used in another another solar system.
	 */
	private static String selectRandomSolarSystemName(NameList names, StellarObjects solarSystems) throws ApplicationError {
		ArrayList exists = names.getNames();
		ArrayList existing = solarSystems.getList();

		if (exists == null || exists.size() == 0) {
			throw new ApplicationError("MotMactory.selectRandomSolarSystemName() No solar system names defined");
		}

		ArrayList candidates = new ArrayList();
		Iterator i1 = exists.iterator();
		while (i1.hasNext()) {
			String name = (String) i1.next();
            boolean found = false;
            if (existing != null) {
                Iterator i2 = existing.iterator();
                while (i2.hasNext() && !found) {
                    SolarSystem ss = (SolarSystem) i2.next();
                    if ( Comp.same(ss.getName(), name) ) {
                        found = true;
                    }
                }
            }
			if (!found) {
				candidates.add(name);
			}
		}

		if (candidates == null || candidates.size() == 0) {
			throw new ApplicationError("MotMactory.selectRandomSolarSystemName() All solar systems names have been used");
		}

		String name = (String) Utils.random(candidates);
        if (Logger.isDebug()) {
            Logger.logDebug("MotFactory.selectRandomSolarSystemName() Select "+name);
        }
        return name;
	}

	private static boolean tooCloseToExistingSolarSystem(int x, int y, StellarObjects solarSystems) {
		StellarObject [] solarSystemsArray = solarSystems.getStellarObjects();
        if (solarSystemsArray == null) {
            return false;
        }
		for (int i=0; i<solarSystemsArray.length; i++) {
			Location loc = solarSystemsArray[i].getLocation();
			int distanceTo = loc.getDistanceTo(x, y);
			if (distanceTo < GalaxyDefinitions.MIN_DISTANCE_BETWEEN_SOLAR_SYSTEMS) {
				return true;
			}
		}

		return false;
	}


    public static void createMonsters(ArrayList ignoreList, Planet tutorialGuardedPlanet) throws ApplicationError {
        Planet [] planets = MotEngine.instance.getPlanets().getPlanets();
        int total = 0;
        int MAX_MONSTERS = PropertyManager.getMandatoryAsInt("setup.max.monsters");
        float PERCENT = PropertyManager.getMandatoryAsFloat("setup.percent.monsters");
    	if (tutorialGuardedPlanet != null) {
			if (ignoreList != null) ignoreList.add(tutorialGuardedPlanet);
	        MonsterPlayer player = MotEngine.instance.getMonsterPlayer();
			CraftFactory.create("monster_small", player, player, tutorialGuardedPlanet.getOrbit(), false, false);
			//MonsterFactory.create(tutorialGuardedPlanet, MonsterFactorySmall.class);
			total++;
    	}
        for (int i=0; i<planets.length && total < MAX_MONSTERS; i++) {
            if ( !planets[i].getSolarSystem().hasOwner() && (ignoreList == null || !ignoreList.contains(planets[i].getSolarSystem())) ) {
                if (Utils.random(PERCENT)) {
                	if (ignoreList != null) ignoreList.add(planets[i].getSolarSystem());
                    MonsterFactory.create(planets[i]);
                    total++;
                }
            }
        }
    }
    
    public static void createAncients(ArrayList ignoreList) throws ApplicationError {
        Planet [] planets = MotEngine.instance.getPlanets().getPlanets();
	    RaceGraphicsList graphicsList = GraphicsFactory.getRaceGraphicsList();
        RaceGraphics [] ancients = graphicsList.getGraphicsType(RaceGraphics.TYPE_ANCIENT);
        for (int i = 0; i < ancients.length; i++) {
        	Planet planet = MotEngine.instance.getPlanets().getRandomUnGuardedEmptyPlanet(ignoreList);
        	if (planet != null) {
        		AncientFactory.createAncient(planet, ancients[i]);
        	} else {
        		Logger.logWarning("No planet to create an ancient");
        	}
		}
//        int total = 0;
//        float PERCENT = PropertyManager.getMandatoryAsFloat("setup.percent.ancients");
//        for (int i=0; i<planets.length && total < ancients.length; i++) {
//            if ( !planets[i].getSolarSystem().hasOwner() && MotEngine.instance.getCrafts().getQtyAt(planets[i].getSolarSystem()) == 0 ) {
//                if (PERCENT > 0 && Utils.random(PERCENT)) {
//                    AncientFactory.createAncient(planets[i], ancients[total]);
//                    total++;
//                }
//            }
//        }
    }
    
    public static void createPrizes(ArrayList ignoreList, SolarSystem tutorialExploreSS) throws ApplicationError {
        SolarSystem [] sos = MotEngine.instance.getStellarObjects().getSolarSystems();
        int total = 0;
        if (tutorialExploreSS != null) {
            PrizeFactory.createRandomPrize(tutorialExploreSS);            
        }
        float PERCENT = PropertyManager.getMandatoryAsFloat("setup.percent.prizes");
        for (int i=0; i<sos.length; i++) {
            if (!sos[i].hasOwner() && (ignoreList == null || !ignoreList.contains(sos[i])) && sos[i] != tutorialExploreSS ) {
                if (Utils.random(PERCENT)) {
                    PrizeFactory.createRandomPrize(sos[i]);
                }
            }
        }
        
        int qtyPlayers = MotEngine.instance.getPlayersNotNull().getQuantity();
        int qtyBoralium = qtyPlayers/2+1;
		for (int i = 0; i < qtyBoralium; i++) {
			StellarObject so = MotEngine.instance.getStellarObjects().getRandomStellarObject(false);
			PrizeFactory.createBoraliumPrize(so);
		}
    }
    
    public static void createResources() throws ApplicationError {
        MoveableOwnedObjects moos = MotEngine.instance.getMoveableOwnedObjects();
        float PERCENT = PropertyManager.getMandatoryAsFloat("setup.percent.resources");
        float MAX_TERRAFORM = PropertyManager.getMandatoryAsFloat("setup.max.terraform.for.resource");
        float MAX_PER_PLAYER = PropertyManager.getMandatoryAsFloat("setup.max.resources.per.player");
        int qtyPlayers = MotEngine.instance.getPlayers().getQuantity();
		float max = qtyPlayers * MAX_PER_PLAYER;
		int qtyPlanets = MotEngine.instance.getPlanets().getQtyPlanets();
		int qty = Comp.ceil(Comp.min(qtyPlanets*PERCENT, max));

		Planet[] pa = MotEngine.instance.getPlanets().getRandomPlanets(qty, MAX_TERRAFORM, true);
		for (int i = 0; i < pa.length; i++) {
            ResourceFactory.createResource(pa[i]);
		}
    }
    
    private static void createGalacticImcumbents(Size galaxySize) throws ApplicationError {
    	
    }

    private static final int MIN_NEBULA_SIZE = 150;
    private static final int NEBULA_SIZE_RANGE = 5;
    private static final int NEBULA_MIN_DIST_TO_EDGE = 100;
    private static final Size BLACK_HOLE_SIZE = new Size(30,30);
    
    
    public static void createNebulae(StellarObjects sos, Size galaxySize) throws ApplicationError {
		int x = Utils.random(galaxySize.getX()-NEBULA_MIN_DIST_TO_EDGE*2)+NEBULA_MIN_DIST_TO_EDGE;
		int y = Utils.random(galaxySize.getY()-NEBULA_MIN_DIST_TO_EDGE*2)+NEBULA_MIN_DIST_TO_EDGE;
		int count = 3;
		while (count > 0 && !isValidNedulaLocation(sos, x,y)) {
			x = Utils.random(galaxySize.getX()-NEBULA_MIN_DIST_TO_EDGE*2)+NEBULA_MIN_DIST_TO_EDGE;
			y = Utils.random(galaxySize.getY()-NEBULA_MIN_DIST_TO_EDGE*2)+NEBULA_MIN_DIST_TO_EDGE;
			count--;
		}
		if (count > 0) {
			createNebula(new Location(x, y), MIN_NEBULA_SIZE + Utils.random(NEBULA_SIZE_RANGE)*20);
		}
    }

	/**
	 * Create a new nebula on a random edge of the galaxy map heading in a random direction
	 * that represents a direction directly towards the opposite side of the galaxy +/- 90
	 * degrees. It has already been added to the list of stellar objects.
	 * @return A new Nebula
	 * @throws ApplicationError 
	 */
    public static Nebula createNebula(Location centerLoc, int diameter) throws ApplicationError {
		MotEngine engine = MotEngine.getInstance();
		NameGenerator ng = engine.getNameGenerator();
		String name = ng.getNextName("Nebula");
		Nebula r = new Nebula(name, centerLoc.getX(), centerLoc.getY(), diameter);
		engine.getStellarObjects().setNebula(r);
		return r;
	}
	
    private static final int NEBULA_GAP = 25;
    /**
     * Nebula cannot be located over a home planet or over another nebula
     */
	private static boolean isValidNedulaLocation(StellarObjects sos, int x, int y) throws ApplicationError {
		if (sos == null) return true;
		StellarObject[] soa = sos.getStellarObjects();
		if (soa == null) return true;
		Circle area = new Circle(new Location(x,y), MIN_NEBULA_SIZE+NEBULA_GAP);
		for (int i = 0; i < soa.length; i++) {
			if (soa[i].getOwners() != null) {
				if (area.contains(soa[i].getLocation())) {
					return false;
				}
			}
		}
		if (sos.getGalaxyDetails() != null) {
			Nebula[] nebulae = sos.getGalaxyDetails().getNebulaeAsArray();
			if (nebulae != null) {
				for (int i = 0; i < nebulae.length; i++) {
					if (nebulae[i].intersects(area)) {
						return false;
					}
				}
			}
		}
		return true;
	}
	
    private static void createBlackHoles(StellarObjects stellarObjects, Size galaxySize, Rectangle solarSystemBounds) throws ApplicationError {
		Location best = createNewLocationForStallarObject(galaxySize, stellarObjects, solarSystemBounds, "Black Hole");
		int count = 0;
		while (MotEngine.instance.getStellarObjects().getNebula(best) != null) {
			best = createNewLocationForStallarObject(galaxySize, stellarObjects, solarSystemBounds, "Black Hole");
			count++;
			if (count > 5) {
				return;
			}
		}
		BlackHole blackHole = BlackHoleFactory.createBlackHole(best);
	}
    
    private static void createWormHoles(StellarObjects stellarObjects, Size galaxySize, Rectangle solarSystemBounds) throws ApplicationError {
    	Location best = createNewLocationForStallarObject(galaxySize, stellarObjects, solarSystemBounds, "Worm Hole");
    	int count = 0;
    	while (MotEngine.instance.getStellarObjects().getNebula(best) != null) {
    		best = createNewLocationForStallarObject(galaxySize, stellarObjects, solarSystemBounds, "Worm Hole");
    		count++;
    		if (count > 5) {
    			return;
    		}
    	}
    	BlackHole blackHole = BlackHoleFactory.createWormHole(best);
    }
    
}


class CandidateSolarSystem {
    public CandidateSolarSystem(SolarSystem solarSystem, int qtyPlanets) {
        this.solarSystem = solarSystem;
        this.qtyPlanets = qtyPlanets;
    }
    public SolarSystem solarSystem;
    public int qtyPlanets;
};
