/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.setup;

import mot.common.bean.PlayerAiSettings;
import mot.common.bean.PlayerColor;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOption;
import mot.common.setup.SetupOptions;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Humanity;
import mot.common.technology.Kamikaze;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValuePairList;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.players.Player;
import mot.engine.players.Players;
import mot.engine.players.UserPlayer;

public abstract class PlayerFactory  {

    /**
     * Creates a new Player with the parameters passed in. The player type is unknown and
     * dependent on the implementation of this interface.
     * @param name The name of the new Player
     * @param character The Player character for the new Player
     * @param color The colour this new player will use.
     * @param alignment The alignment with an ancient race or null if this player does not have any.
     * @return The new Player.
     */
    public abstract Player create(RaceOptions raceOptions, String name, Double level, PlayerColor color, String alignment, PlayerAiSettings playerAiSettings, String [] playerNames) throws ApplicationError;

	/**
	 * Constructs a new Player object with correct starting technology, and preferences.
	 * The new player does not have any relations, crafts or planets.
	 */
	public Player createNewPlayer(RaceOptions raceOptions, String name, Double level, PlayerColor color, String alignment, PlayerAiSettings playerAiSettings, String [] playerNames) throws ApplicationError {
		Player player = create(raceOptions, name, level, color, alignment, playerAiSettings, playerNames);

		// now setup starting technologies
		Technology [] techs = getStartingTechnologies(raceOptions, player);

		for (int i = 0; i < techs.length; i++) {
			player.addTechnology(techs[i]);			
		}
		
		boolean isDedicated = raceOptions.getBoolForName("is-dedicated");
		if (isDedicated) {
			Kamikaze kamikazi = Technologies.getMiscKamikazeTechnology();
			if (kamikazi != null) {
				player.addTechnology(kamikazi);
			}
		}
        
		SetupOption ao = raceOptions.getForName("alignment");
		if (ao == null) {
			Logger.logWarning("PlayerFactory.getNewPlayer() missing alignment for player "+player.getName());
		} else {
			if (Utils.random(0.5)) {
				player.setAlignment(ao.getValue());
			}
		}
		
		int initialFunds = 100;
		if ((player instanceof UserPlayer) && level != null) {
			if (level.doubleValue() < 1.0) {
				initialFunds = 1000;
			} else if (level.doubleValue() < 1.5) {
				initialFunds = 500;
			}
		}
		
        player.setFunds(initialFunds);
        player.setChanceHoloCraftWork(1.0f);
        
        add(player);

		return player;
	}

	public static Technology [] getStartingTechnologies(Player player) throws ApplicationError {
		SetupOptions generalOptions = OptionsFactory.getGeneralOptions();
		if (player == null) {
            return getStartingTechnologies(generalOptions, null, player);
		} else {
			RaceOptions raceOptions = OptionsFactory.getRaceOptions(player.getName());
			return getStartingTechnologies(generalOptions, raceOptions.getRaceOptions(), player);
		}
	}

	private static Technology [] getStartingTechnologies(RaceOptions raceOptions, Player player) throws ApplicationError {
		SetupOptions generalOptions = OptionsFactory.getGeneralOptions();
		return getStartingTechnologies(generalOptions, raceOptions.getRaceOptions(), player);
	}
	
	private static Technology [] getStartingTechnologies(SetupOptions generalOptions, SetupOptions raceOptions, Player player) throws ApplicationError {
		TechnologyList startingTechs = new GeneralTechnologyList();
		if (raceOptions != null) {
			SetupOption so = raceOptions.getForName("starting-techs");
			if (so != null) {
				BasicNameValuePairList techs = so.getParameters();
				if (techs != null) {
					Technology [] r = techs.getParameterNamesAsTechs();
					if (r != null) {
						startingTechs.add(r);
					}
				}
			}
			if (raceOptions.getBoolForName("start-with-human-trade-route")) {
				Humanity humanTradeRoute = Technologies.getHumanTradeRoute();
				if (humanTradeRoute == null) {
					Logger.logWarning("PlayerFactory.getStartingTechnologies() No Human Trade Route");
				} else {					
					startingTechs.add(humanTradeRoute);
				}
			}
		}
		
		SetupOption so = generalOptions.getForName("starting-techs");
		if (so != null) {
			BasicNameValuePairList techs = so.getParameters();
			if (techs != null) {
				Technology [] r = techs.getParameterNamesAsTechs();
				if (r != null) {
					startingTechs.add(r);
				}
			}
		}
		
		return startingTechs.getAllAsArray();
	}

    private static void add(Player p) {
        MotEngine engine = MotEngine.getInstance();
        Players players = engine.getPlayers();
        if (players == null) {
            players = new Players();
            engine.setPlayers(players);
        }
        players.add(p);
    }
}