/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.setup;
import java.util.Collection;
import java.util.HashMap;

import mot.common.bean.NewGameUserPlayerData;
import mot.common.util.Comp;
import mot.common.util.Logger;

/**
 * This is a temporary store for players connecting to the server before the game 
 * has been started. It contains a list if PlayerRegistryItem.
 * 
 * When a game is starting up, first multiple players design their races. After each 
 * player has designed his/her race then a single player starts the game. This registry 
 * stores all the details of players who have registered but before the game has started.
 */
public class PlayerRegistry  {

    /**
     * This is a list of RaceOptions objects for each user player who is going to 
     * be participant in the game.
     */
    private static HashMap<String, NewGameUserPlayerData> items;

    public PlayerRegistry() {
    }

    public static void update(String clientId, NewGameUserPlayerData userData) {
        if (items == null) items = new HashMap();
        NewGameUserPlayerData v = items.get(clientId);
        if (v != null) {
        	items.remove(clientId);
        }
        if (Logger.isDebug()) {
        	Logger.logDebug("PlayerRegistry.update() clientId="+clientId+" userData="+userData); 
        }
        items.put(clientId, userData);
    }

    public static NewGameUserPlayerData getPlayerByClientId(String clientId) {
        if (items == null) return null;
        return items.get(clientId);
    }

	public static NewGameUserPlayerData getPlayerByName(String raceName) {
        if (items == null) return null;
		for (NewGameUserPlayerData v : items.values()) {
			if (Comp.same(raceName, v.getRaceName())) {
				return v;
			}
		}
		return null;
	}

	public static NewGameUserPlayerData getPlayerByColour(String colourName) {
        if (items == null) return null;
		for (NewGameUserPlayerData v : items.values()) {
			if (Comp.same(colourName, v.getColour())) {
				return v;
			}
		}
		return null;
	}


	public static NewGameUserPlayerData getPlayerByName(String raceName, String exludeClientId) {
        if (items == null) return null;
		for (String id : items.keySet()) {
			if (!Comp.same(exludeClientId, id)) {
				NewGameUserPlayerData v = items.get(id);
				if (Comp.same(raceName, v.getRaceName())) {
					return v;
				}
			}
		}
		return null;
	}

	public static NewGameUserPlayerData getPlayerByColour(String colour, String exludeClientId) {
        if (items == null) return null;
		for (String id : items.keySet()) {
			if (!Comp.same(exludeClientId, id)) {
				NewGameUserPlayerData v = items.get(id);
				if (Comp.same(colour, v.getColour())) {
					return v;
				}
			}
		}
		return null;
	}

    public static NewGameUserPlayerData [] getUserPlayerDetails() {
        if (items == null) return null;
        Collection<NewGameUserPlayerData> values = items.values();
        return (NewGameUserPlayerData []) values.toArray(new NewGameUserPlayerData[items.size()]);
    }


    public static void clear() {
        items = null;
    }

}