/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.spies;

import java.util.ArrayList;
import java.util.Iterator;

import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.BasicNameValuePairList;
import mot.common.util.Logger;
import mot.common.util.NameValuePair;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.players.Player;

/**
 * This object tracks all spies in the game as well as the tasks they have 
 * been assigned to perform.
 * 
 * All spies assigned to attack another player must use a spy role. Spies used to
 * defend against enemy spies do not need an explicit spy role. At the end fo every 
 * turn players assign unused spies to the spy role of defend.
 */
public class Spies extends  XMLBean {

    /**
     * This is a list of all spying roles currently active in the game (for all
     * players against all targets). It is a list of SpyRole objects.
     */
    private ArrayList roles = new ArrayList();
    
    /**
     * Returns the spy role for spies owned by player owner and spying on player target.
     * If no such role exists then null will be returned. To get the spies allocated by
     * a player for defence pass the owner as the target as well.
     * @param owner The owner of the spy role.
     * @param target The target of the spy role.
     * @return Returns the spy role for spies owned by player owner and spying on player target.
     */
    public SpyRole [] getSpies(Player owner, Player target) {
        Iterator i = roles.iterator();
        ArrayList list = null;
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == owner && role.getTarget() == target) {
                if (list == null) list = new ArrayList();
                list.add(role);
            }
        }
        if (list == null) {
            return null;
        } else {
            return (SpyRole []) list.toArray(new SpyRole [list.size()]);
        }
    }

    public SpyRoleEspionage getEspionageSpies(Player owner, Player target) {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == owner && role.getTarget() == target) {
                if (role instanceof SpyRoleEspionage) {
                	return (SpyRoleEspionage) role;
                }
            }
        }
        return null;
    }


    /**
     * @return The quantity of spies race player has assigned to perform sabotage
     * on the other player.
     */
    public int getQtySpiesDefending(Player player) {
        SpyRole role = getDefendingSpies(player);
        if (role == null) {
            return 0;
        } else {
            return role.getQtySpies();
        }
    }
    
    /**
     * @return The quantity of spies race player has assigned to perform espionage
     * on the other player.
     */
    public int getQtySpiesEspionage(Player player, Player other) {
        int qty = 0;
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player && role.getTarget() == other) {
                if (role instanceof SpyRoleEspionage) {
                    qty += role.getQtySpies();
                }
            }
        }
        return qty;
    }
    
    /**
     * @return The quantity of spies race player has assigned to perform sabotage
     * on the other player.
     */
    public int getQtySpiesSabotage(Player player, Player other) {
        int qty = 0;
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player && role.getTarget() == other) {
                if (role instanceof SpyRoleSabotage) {
                    qty += role.getQtySpies();
                }
            }
        }
        return qty;
    }
    

    /**
     * The quantity of spies player has assigned to perform espionage on the all other players.
     * @return A Name Value Pair list where the name is a name of a player and the vakue is the quantity 
     * of spies performing espionage on that player.
     */
    public BasicNameValuePairList getQtySpiesEspionage(Player player) {
        int qty = 0;
        Iterator i = roles.iterator();
        BasicNameValuePairList list = null;
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player) {
                if (role instanceof SpyRoleEspionage) {
                    if (list == null) list = new BasicNameValuePairList();
                    list.add(role.getTarget().getName(), role.getQtySpies());
                }
            }
        }
        return list;
    }
    
    /**
     * The quantity of spies player has assigned to perform espionage on the all other players.
     * @return A Name Value Pair list where the name is a name of a player and the vakue is the quantity 
     * of spies performing espionage on that player.
     */
    public BasicNameValuePairList getQtySpiesSabotage(Player player) {
        int qty = 0;
        Iterator i = roles.iterator();
        BasicNameValuePairList list = null;
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player) {
                if (role instanceof SpyRoleSabotage) {
                    if (list == null) list = new BasicNameValuePairList();
                    list.add(role.getTarget().getName(), role.getQtySpies());
                }
            }
        }
        return list;
    }

    public BasicNameValuePairList getSpyRolesEspionageSuccessChance(Player player) throws ApplicationError {
        int qty = 0;
        Iterator i = roles.iterator();
        BasicNameValuePairList list = null;
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player) {
                if (role instanceof SpyRoleEspionage) {
                	SpyRoleEspionage roleEspionage = (SpyRoleEspionage) role;
                	int chance = roleEspionage.getChanceEspionageSuccess();
                	if (list == null) list = new BasicNameValuePairList();
                	list.add(new NameValuePair(roleEspionage.getTarget().getName(), ""+chance));
                }
            }
        }
        return list;
    }

    public BasicNameValuePairList getSpyRolesSabotageSuccessChance(Player player) throws ApplicationError {
        int qty = 0;
        Iterator i = roles.iterator();
        BasicNameValuePairList list = null;
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player) {
                if (role instanceof SpyRoleSabotage) {
                	SpyRoleSabotage roleSabotage = (SpyRoleSabotage) role;
                	int chance = roleSabotage.getChanceSabotageSuccess();
                	if (list == null) list = new BasicNameValuePairList();
                	list.add(new NameValuePair(roleSabotage.getTarget().getName(), ""+chance));
                }
            }
        }
        return list;
    }
    
    public BasicNameValuePairList getSpyRolesEspionageTargets(Player player) throws ApplicationError {
        int qty = 0;
        Iterator i = roles.iterator();
        BasicNameValuePairList list = null;
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player) {
                if (role instanceof SpyRoleEspionage) {
                	SpyRoleEspionage roleEspionage = (SpyRoleEspionage) role;
                	Technology technology = roleEspionage.getTechnology();
                	if (technology != null) {
	                	if (list == null) list = new BasicNameValuePairList();
	                	list.add(new NameValuePair(roleEspionage.getTarget().getName(), technology.getFullName()));
                	}
                }
            }
        }
        return list;
    }


    /**
     * Returns the spy role for spies owned by player owner and assigned to defend the player against
     * enemy spies.
     * @param owner The owner of the spy role.
     * @return Returns the quantity of spies owned by player owner and assigned to defend.
     */
    public int getQtyDefendingSpies(Player owner) {
        SpyRole sr = getDefendingSpies(owner);
        if (sr == null) {
            return 0;
        } else {
            return sr.getQtySpies();
        }
    }
    
    /**
     * Returns the spy role for spies owned by player owner and assigned to defend the player against
     * enemy spies.
     * @param owner The owner of the spy role.
     * @return Returns the spy role for spies owned by player owner and assigned to defend.
     */
    public SpyRole getDefendingSpies(Player owner) {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == owner && role.getTarget() == owner) {
                return role;
            }
        }
        return null;
    }
    
    /**
     * Adds qty spies to defend player owner. It will add these spies to an existing 
     * defend role if one exists otherwise it will create a new defend role.
     */
    public void addDefendingSpies(Player owner, int qty) {
        SpyRole defenders = getDefendingSpies(owner);
        if (defenders == null) {
            addSpyRole(new SpyRoleDefend(owner, qty));
        } else {
            defenders.addQtySpies(qty);
        }
    }

    /**
     * Returns all the spy roles for spies owned by player
     * @param owner The owner of the spy role.
     * @return Returns the spy role for spies owned by player.
     */
    public SpyRole [] getSpies(Player owner) {
        ArrayList list = null;
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == owner) {
                if (list == null) list = new ArrayList();
                list.add(role);
            }
        }
        if (list == null) {
            return null;
        } else {
            return (SpyRole []) list.toArray(new SpyRole [list.size()]);
        }
    }

    /**
     * Returns the quantity of spies being used by this player
     * @param owner The owner of the spy role.
     * @return Returns the quantity of spies being used by this player
     */
    public int getQtySpies(Player owner) {
        int qty = 0;
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == owner) {
                qty += role.getQtySpies();
            }
        }
        return qty;
    }

    /**
     * Removes all spy roles belonging to player owner.
     * @param owner The owner of the spy role.
     */
    public void removeSpies(Player owner) {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == owner) {
                i.remove();
            }
        }
    }

    /**
     * Removes all spy roles belonging to player owner ans all spy roles containing player.
     * @param owner The owner of the spy role.
     */
    public void removeAllContaining(Player player) {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player) {
                i.remove();
            }
            if (role.getTarget() == player) {
                i.remove();
            }
        }
    }

    /**
     * Removes all spy roles belonging to player owner and assigned to attack or defend the other player.
     * @param owner The owner of the spy role.
     */
    public void removeSpies(Player owner, Player other) {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == owner && role.getTarget() == other) {
                i.remove();
            }
        }
    }

    /**
     * Removes the spy role sr from the list of spy roles. 
     * @param sr The spy role to remove.
     */
    public void removeSpyRole(SpyRole sr) {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role == sr) {
                i.remove();
            }
        }
    }
    
    /**
     * Reduces the quantity of spies owned by player to qtySpies. This removes spies from
     * roles in the following order until the required qty spies is reached;
     *  - Sabotage
     *  - Espionage
     *  - Defend
     * @param player The player who needs the quantity of spies reduced
     * @param maxQtySpies The amount to reduce the quantity of spies to.
     */
    public void tooManySpies(Player player, int maxQtySpies) {
        int qty = getQtySpies(player);
        if (qty <= maxQtySpies) {
            return;
        }
        int removeQty = qty - maxQtySpies;
        removeQty = _tooManySpies(player, SpyRoleSabotage.class, removeQty);
        if (removeQty == 0) return;
        removeQty = _tooManySpies(player, SpyRoleEspionage.class, removeQty);
        if (removeQty == 0) return;
        removeQty = _tooManySpies(player, SpyRoleDefend.class, removeQty);
        if (removeQty == 0) return;
        Logger.logWarning("Spies.tooManySpies() MUST HAVE UNKNWN SPY ROLES");
    }

    private int _tooManySpies(Player player, Class type, int removeQty) {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.getOwner() == player && role.getClass() == type) {
                int qty = role.getQtySpies();
                if (Logger.isDebug()) {
                    Logger.logDebug("Spies._tooManySpies() Remove "+removeQty+" spies from "+type.getName()+" with "+qty+" spies");
                }
                if (removeQty >= qty) {
                    i.remove();
                    if (removeQty == qty) {
                        return 0;
                    } else {
                        removeQty -= qty;
                    }
                } else {
                    role.setQtySpies(qty - removeQty);
                    return 0;
                }
            }
        }
        return removeQty;
    }

  	/**
	 * Performs an endTurn() operation for all spy roles in the game. @see SpyRole
     * for details.
	 */
	public void endTurn() throws ApplicationError {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            role.endTurn();
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            role.toXML(writer);
        }
	}

	protected String getElementName() { return "spies"; }

    public ArrayList getRoles() {
        return roles;
    }

    public void setRoles(ArrayList roles) {
        this.roles = roles;
    }
    
    public void addSpyRole(SpyRole v) {
        Iterator i = roles.iterator();
        while (i.hasNext()) {
            SpyRole role = (SpyRole) i.next();
            if (role.equals(v)) {
                role.addQtySpies(v.getQtySpies());
                return;
            }
        }
        roles.add(v);
    }
    
    public void setSpyRoleDefend(SpyRoleDefend v) {
        addSpyRole(v);
    }

    public void setSpyRoleEspionage(SpyRoleEspionage v) {
        addSpyRole(v);
    }

    public void setSpyRoleSabotage(SpyRoleSabotage v) {
        addSpyRole(v);
    }

}