/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.spies;

import mot.common.bean.NameList;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.event.NewSpyEF;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * The spy centre recruits new spies and keeps track of the total quantity of spies
 * it has generated so far. To recruit spies call addEffort() and then at the end 
 * of every turn call endTurn(). The chance of recruiting a new spies is dependent 
 * on the total effort added to recruitment so far.
 * 
 * Total effort is the sum of all effort added since the last spy was recruited. Each 
 * turn a random number between 0 and MAX_SPY_RECRUITE_EFFORT + qty spies*100. If this number is bigger 
 * than the totalEffort then a new spy is recruited.
 * 
 * This - If total effort is 100 and MAX_SPY_RECRUITE_EFFORT is 1000 then the player has
 * a 10% chance of recruiting a spy.
 */
public class SpyCenter extends XMLBean {

    /**
     * The total effort assigned to recruiting spies so far.
     */
    private int totalEffort;
    
    /**
     * This is the quantity of spies this player has built. User players must build
     * spies by allocating funds to recruit - see SpyFactory for details.
     */
    private int qtyBuiltSpies;
    
    /**
     * This is a list of player names that this spy centre has allocated a spies to.
     * When a player allocates spies to a another player then details of the target
     * players activities becomes visible.
     */
    private NameList playerSpies;
    
    private static final int MAX_SPY_RECRUITE_EFFORT = 2000;

    public SpyCenter() {
    }

    public void setQtyBuiltSpies(int qtyBuiltSpies) { this.qtyBuiltSpies = qtyBuiltSpies; }
    public int getQtyBuiltSpies() { return qtyBuiltSpies; }
    
    public void addEffort(int v) {
        totalEffort += v;
    }
    
    /**
     * Perform task that a required at the end of a turn. This is when this object uses a 
     * random number generator to see if it has generated a new spy.
     * @param player The player to whom this SpyCenter belongs.
     */
    public void endTurn(Player player) throws ApplicationError {
        if (totalEffort <= 0) return;
		int cieling = calculateChanceCieling();
        int r = Utils.random(cieling);
        if (totalEffort >= r) {
            qtyBuiltSpies++;
            if (Logger.isDebug()) {
                Logger.logDebug("SpyCenter.endTurn() TotalEffort="+totalEffort+" NEW SPY RECRUITED ("+qtyBuiltSpies+")");
            }
            totalEffort = 0;
            MotEngine.instance.getSpies().addDefendingSpies(player, 1);
            MotEngine.instance.broadcast(new NewSpyEF(qtyBuiltSpies), player);
        }
    }

	private int calculateChanceCieling() {
		return MAX_SPY_RECRUITE_EFFORT+qtyBuiltSpies*100;
	}

	public double getChanceToRecuite(UserPlayer player) {
		int spyRecruitEffortPerTurn = player.getSpyRecruitEffortPerTurn();
		double cieling = calculateChanceCieling();
		double bottom = totalEffort+spyRecruitEffortPerTurn;
		return bottom / cieling;
	}
    

    /**
     * @param other The player who the owner of this spy centre may be able to view.
     * @return True if the owner of this spy centre can view the details of the player other.
     */
    public boolean canViewDetailsOf(Player other) {
        if (playerSpies == null || other == null ) return false;
        return playerSpies.contains(other.getName());
    }
    
    /**
     * Adds player other to the list of players that the owner of this spy centre can view
     * the details of.
     * @param other The player to add to the player spy list.
     */
    public void addPlayerSpy(Player other) {
        if (playerSpies == null) playerSpies = new NameList();
        playerSpies.add(other.getName());
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("qty-built-spies", qtyBuiltSpies);
        writer.addElement("total-effort", totalEffort);
        if (playerSpies != null) {
            playerSpies.toXML(writer, "player-spies");
        }
	}
    

	protected String getElementName() { return "spy-center"; }


    public void setTotalEffort(int totalEffort) {
        this.totalEffort = totalEffort;
    }


    public int getTotalEffort() {
        return totalEffort;
    }


    public void setPlayerSpies(NameList playerSpies) {
        this.playerSpies = playerSpies;
    }


    public NameList getPlayerSpies() {
        return playerSpies;
    }
    
    public String [] getPlayerSpiesAsArray() {
        if (playerSpies == null) {
            return null;
        } else {
            return playerSpies.getNamesAsArray();
        }
    }

}