/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.spies;

import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.planets.Planet;
import mot.engine.planets.Planets;
import mot.engine.players.Player;
import mot.engine.players.PlayerReference;

/**
 * This represents a number of spies owned by the player and assigned to a task.
 *
 * Each turn a spy has a chance of CHANCE_OF_SUCCESS_ESPIONAGE or CHANCE_OF_SUCCESS_SABOTAGE
 * DIVIDED BY (spying advantage/10 + qty more spies than defending) of a successful mission.
 */
public abstract class SpyRole extends  XMLBean {

    /**
     * This is the player that ownes this visited object.
     */
    private Player owner;

    /**
     * This is the Player these spies are spying on.
     */
    private Player target;

    /**
     * This is the quantity of spies active in this object.
     * If the player has a spy active then the player can view the solar system window
     * details for planets not belonging to the player.
     */
    private int qtySpies;

    public SpyRole() {
    }

    public SpyRole(Player owner, Player target, int qtySpies) {
        this.owner = owner;
        this.target = target;
        this.qtySpies = qtySpies;

        if (Logger.isDebug()) {
            Logger.logDebug("SpyRole.init() Player "+owner.getName()+" Target "+target.getName()+" qtySpies="+qtySpies+" task="+getTaskDescription());
        }
    }

    public Player getOwner() {
        return owner;
    }

    public void setOwner(Player player) {
        this.owner = player;
    }

    public void setOwner(PlayerReference p) throws ApplicationError {
        owner = p.getPlayer();
    }

    public Player getTarget() {
        return target;
    }

    public void setTarget(Player player) {
        this.target = player;
    }

    public void setTarget(PlayerReference p) throws ApplicationError {
        target = p.getPlayer();
    }

    public int getQtySpies() {
        return qtySpies;
    }

    public void setQtySpies(int qtySpies) {
        this.qtySpies = qtySpies;
    }

    public int addQtySpies(int qtySpies) {
        this.qtySpies += qtySpies;
        if (this.qtySpies < 0) {
            int r = this.qtySpies;
            this.qtySpies = 0;
            return r;
        } else {
            return 0;
        }
    }
    
    public boolean equals(SpyRole o) {
        return o != null &&
               owner == o.getOwner() &&
               target == o.getTarget() &&
               getClass() == o.getClass();
    }
    
    public abstract String getTaskDescription();

    // With a 10 spy advantage that is one success every 33 turns.
    private static final int CHANCE_OF_SUCCESS_ESPIONAGE = 333;
    private static final int CHANCE_OF_SUCCESS_SABOTAGE = 333;

  	/**
	 * Performs any operation these objects need to do at the end of every turn.
     * Currently this only includes moving all the spies. Each turn each spy active
     * in a solar system has a change of performing a successful operation.
	 */
	public void endTurn() throws ApplicationError {
        Planets planets = MotEngine.instance.getPlanets();
        double espionageEffect = (planets != null) ? planets.getTotalEspionageEffect(target) : 0;
        
        if (this instanceof SpyRoleEspionage) {
        	SpyRoleEspionage srs = (SpyRoleEspionage)this;
        	int chance = getChanceEspionageSuccess();
        	if (chance == 0) return;
            int r = Utils.random(chance);
            if (r != 0 && espionageEffect > 0) r = Utils.random(espionageEffect) ? 0 : 1;  
            if (r == 0) {
            	Technology targetTechnology = srs.getTechnology();
            	srs.setTechnology((Technology)null);
                target.steelTechnology(owner, targetTechnology);
            }
        } else if (this instanceof SpyRoleSabotage) {
        	int chance = getChanceSabotageSuccess();
        	if (chance == 0) return;
            int r = Utils.random(chance);
            if (r != 0 && espionageEffect > 0) r = Utils.random(espionageEffect) ? 0 : 1;  
            if (r == 0) {
                Planet [] upa = target.getPlanets();
                if (upa != null && upa.length > 0) {
                    Planet planet = (Planet) Utils.random(upa);
                    planet.sabotage(owner);
                }
            }
        }

    }
	
	public int getChanceEspionageSuccess() throws ApplicationError {
        if (qtySpies <= 0) {
            // nothing to do as we have no spies
            return 0;
        }

        if (target == null) {
            // nothing to do as this solar system is not owned
            qtySpies = 0;
            return 0;
        }

        if (this instanceof SpyRoleDefend) {
            return 0;
        }

        SpyRole defending = MotEngine.instance.getSpies().getDefendingSpies(target);

        int SpyLevel1 = 0;
        if (owner.getBiotechnologies() != null) {
            SpyLevel1 += owner.getBiotechnologies().getSpyRating();
        }
        int SpyLevel2 = 0;
        if (target.getBiotechnologies() != null) {
            SpyLevel2 += target.getBiotechnologies().getSpyRating();
        }
        SpyLevel1 *= 1.0 + owner.getSpyBonus();
        SpyLevel2 *= 1.0 + target.getSpyBonus();
        int qtySpies2 = (defending == null) ? 0 : defending.getQtySpies();
        int spyAdv = (SpyLevel1 - SpyLevel2);
        int qtyAdv = qtySpies - qtySpies2;

        if (this instanceof SpyRoleEspionage) {
        	SpyRoleEspionage srs = (SpyRoleEspionage) this;
            int chance = CHANCE_OF_SUCCESS_ESPIONAGE / (qtySpies + Comp.max(spyAdv + qtyAdv, 1));
            Technology targetTechnology = srs.getTechnology();
            if (targetTechnology != null) {
            	chance = chance * 4;
            }
            if (Logger.isDebug()) {
                Logger.logDebug("SpyRoleEspionage.endTurn() Player "+owner.getName()+" on "+target.getName()+" has spyAdv="+spyAdv+" qtyAdv="+qtyAdv+" [chance is 1 in "+chance+"]");
            }
            return chance;
        } else {
        	return 0;
        }
	}

	public int getChanceSabotageSuccess() throws ApplicationError {
        if (qtySpies <= 0) {
            // nothing to do as we have no spies
            return 0;
        }

        if (target == null) {
            // nothing to do as this solar system is not owned
            qtySpies = 0;
            return 0;
        }

        if (this instanceof SpyRoleDefend) {
            return 0;
        }

        SpyRole defending = MotEngine.instance.getSpies().getDefendingSpies(target);

        int SpyLevel1 = 0;
        if (owner.getBiotechnologies() != null) {
            SpyLevel1 += owner.getBiotechnologies().getSpyRating();
        }
        int SpyLevel2 = 0;
        if (target.getBiotechnologies() != null) {
            SpyLevel2 += target.getBiotechnologies().getSpyRating();
        }
        SpyLevel1 *= 1.0 + owner.getSpyBonus();
        SpyLevel2 *= 1.0 + target.getSpyBonus();
        int qtySpies2 = (defending == null) ? 0 : defending.getQtySpies();
        int spyAdv = (SpyLevel1 - SpyLevel2);
        int qtyAdv = qtySpies - qtySpies2;

    	if (this instanceof SpyRoleSabotage) {
    		int chance = CHANCE_OF_SUCCESS_SABOTAGE / (qtySpies + Comp.max(spyAdv + qtyAdv, 1));
            if (Logger.isDebug()) {
                Logger.logDebug("SpyRoleSabotage.endTurn() Player "+owner.getName()+" on "+target.getName()+" has spyAdv="+spyAdv+" qtyAdv="+qtyAdv+" [chance is 1 in "+chance+"]");
            }
            return chance;
        } else {
        	return 0;
        }
	}

	protected void encode(XMLWriter writer) throws ApplicationError {
        new PlayerReference(owner).toXML(writer, "owner");
        new PlayerReference(target).toXML(writer, "target");
        writer.addElement("qty-spies", qtySpies);
	}


}