/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.technology.Shield;
import mot.common.technology.ShieldSpan;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.players.Player;
 
/**
 * This class is a wrapper around shield used during the course of a battle.
 * Each battle craft has a shield instance being an instance of the shield
 * being used by this battle craft during the battle.
 *
 * We need an instance of a shield as the shield power may fluctuate during
 * during the battle.
 *
 * Not we cannot simply put the power attribute in shield as the shield object
 * is shared among all craft using the same shield type.
 */
public class ShieldInstance  {

	/**
	 * The object that owned these shields
	 */
	private BattleObject owner;
	
    /**
     * This is the shield object upon which this instance is based.
     */
    private Shield shield;

	/**
	 * How much damage the shields can take before they fail. So a shield with no recharge,
	 * and power of 50, would take 5 hits of damage 10 to fail.
	 */
	private double power;


    public ShieldInstance(BattleObject owner, Shield shield) throws ApplicationError {
        this.owner = owner;
        this.shield = shield;
        power = getMaxShieldCharge();
    }

    public Shield getShield() {
        return shield;
    }

    public void setShield(Shield shield) {
        this.shield = shield;
    }
    
	public double getMaxShieldCharge() throws ApplicationError {
		double bonus = owner.getOwner().getOwnerShieldPowerBonus();
		double v = shield.getPower() * (1.0 + bonus);
		return v;
    }


	public double getPower() {
        return power;
    }

	public void setPower(double v) {
        power = v;
    }

	public void addPower(double v) throws ApplicationError {
        power += v;
        double max = getMaxShieldCharge();
		if (power > max) {
            power = max;
        }
    }

	public double getMaxRecharge() throws ApplicationError {
        double max = getMaxShieldCharge();
        return max - power;
    }

    /**
     * recharges all shields on the craft to their maximum levels.
     * @throws ApplicationError 
     */
	public void rechargeFull() throws ApplicationError {
        power = getMaxShieldCharge();
	}

	public void discharge() {
		power = 0.0;
	}

	/**
	 * @return True iff these shields are full charged (at their maximum power)
	 * @throws ApplicationError 
	 */
	public boolean fullyCharged() throws ApplicationError {
        double max = getMaxShieldCharge();
		return power >= max;
	}

	/**
	 * The shields are recharged. This should be called at the end of each battle
	 * turn.
	 * @return Un-used recharge capacity of these shields
	 * @throws ApplicationError 
	 */
	public double recharge() throws ApplicationError {
		double originalPower = power;
		double rechargeRate = shield.getRechargeRate();
		addPower(rechargeRate);
        double unusedPower = rechargeRate - (power - originalPower);
        return unusedPower;
	}

    /**
	 * Distributes the unused recharge capacity of these shields to the closest friendly craft 
	 * in the battle. @see Special for details (ShieldSpan attribute). 
	 * @param bc The craft containing these shields
	 * @param ss The shield spanning ability this craft possessed
	 * @param unusedPower The amount of recharge to distribute to the nearest friendly craft.
	 * @throws ApplicationError 
	 */
	public static void executeShieldSpan(BattleObject bc, ShieldSpan ss, double unusedPower) throws ApplicationError {
		ShieldInstance targetShield = getTarget(bc, ss.getRange());
		if (targetShield == null) {
			return;
		}
		BattleObject targetCraft = targetShield.getOwner();
		Weapon weapon = (Weapon) Technologies.getWeapons().getFirstInGroupMandatory(Weapon.GROUP_SHIELD_DRAIN);
		if (targetCraft != null) {
			CraftStateFactory oldTargetState = targetCraft.getCraftState();
			double charge = unusedPower*ss.getSpanPercent();
			targetShield.addPower(charge);
			CraftStateFactory newTargetState = targetCraft.getCraftState();
            String shieldCharge = Formatter.roundedToTwoDP(charge);
			BattleWeaponFireEF event = new BattleWeaponFireEF(bc, targetCraft, weapon, 0.0, charge, null, shieldCharge, false, null, oldTargetState, null, newTargetState);
            MotEngine.instance.multicast( event, bc.getBattle().getStartingUserPlayers());
		}
	}
	
	/**
	 * @return the closest craft to craft bc that is owned by the same player as bc,
	 * that is not bc and requires atleast one shield to recharged.
	 */
	private static ShieldInstance getTarget(BattleObject bc, int maxDist) throws ApplicationError {
		Player owner = bc.getOwner();
		BattleCraft [] crafts = bc.getBattle().getCrafts();

		Location craftLocation = bc.getLocation();
		ShieldInstance closestShield = null;
		int closestDist = 0;

		for (int i=0; i<crafts.length; i++) {
			if (crafts[i] != bc && crafts[i].isOwnedBy(owner)) {
				int dist = crafts[i].getLocation().getDistanceTo(craftLocation);
				if (dist < maxDist) {					
					if (closestShield == null || dist < closestDist) {
						ShieldInstance s = crafts[i].getShieldsThatRequireRecharging();
						if (s != null) {
							closestShield = s;
							closestDist = dist;
						}
					}
				}
			}
		}

		return closestShield;

	}	

	/**
     * The source craft source has fired on the target craft (these shields belong to the target
     * craft) inflicting damage amount of damage. This method returns the new damage inflicted
     * after the shields have blocked what they can. This method also reduces the shields charge
     * accordingly.
     * This involves reducing the damage qty times (once for each weapon fire).
     * When more than one shield is being used to block the attack, all shields with power reduce
     * the damage but the power lost is distributed evenly over the quantity of shields.
     * @param damage The total damage inflicted after firing the weapon qty times.
     * @param qtyWeapons The quantity of times this weapon was fired to produce the damage
     * @param qtyShields The quantity of shields are defending against this attack
     * @return The actual damage inflicted.
     */
	public double blockAttack(double damage, int qtyWeapons, int qtyShields) {
		// reduce power of shields
        double originalPower = power;
		power = power - (damage*shield.getDischargeRate())/qtyShields;
		if (power < 0) power = 0;

		// reduce damage done to craft if shields still operating
        double actualDamageBlocked = Comp.min(originalPower, shield.getDamageBlocked()*qtyWeapons);
        if (actualDamageBlocked > damage) {
            return 0;
        } else {
            return damage - actualDamageBlocked;
        }
	}

	public BattleObject getOwner() {
		return owner;
	}

}