/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.LocatedArea;
import mot.common.bean.Location;
import mot.common.bean.SimpleLocatedArea;
import mot.common.bean.Size;
import mot.common.lang.L;
import mot.common.technology.Shield;
import mot.common.technology.Specials;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.technology.WeaponBundle;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleMine;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattleStation;
import mot.engine.battles.PlayerBattleBonuses;
import mot.engine.battles.incumbants.Incumbents;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.players.Player;
import mot.engine.players.Players;

 /**
  * This is an instance of a weapon on a craft. Note that the Weapon object only contains
  * state common to all instances of that technology type on all crafts / planets.
  * Weapons have state during a battle on a craft by craft basis (such as hasFired).
  * Therefore we need an instance of a class around Weapon to keep this extra state.
  * 
  * Prototypes are often faulty by nature. In this game the	first few crafts built of 
  * a particular type suffer from reliability problems.	The first craft built of any 
  * given type is the least reliable and as a player builds more craft of a given type 
  * they become more reliable. When a craft starts a battle build quality determines 
  * the starting condition of the craft. 
  * 
  * Built quality may be improved using better production techniques. These technologies
  * can be researched through the game. @see mot.common.technology.Factory for details.
  * 
  * Most weapons are fired at a BattleObject. Implementations of a weapon instance must
  * implement the fire method containing a target of type BattleObject to define attacking
  * a battle object. However, some weapons can also (or instead) be fired at a location 
  * (e.g. SpacialDistortion). These must also implement the fire method that contains a 
  * target of type Location. 
  */
public abstract class WeaponInstance extends WeaponBundle {


	/**
     * This is the quantity of weapons that have been fired of this type
     * on this craft for thus turn. When qtyFired == qty then all the weapons
     * have been fired in this weapon instance.
     */
    protected int  qtyFired;

    /**
     * If this weapon installation is a heavy installation.
     * Heavy installations have twice the range and double the damage.
     */
    protected boolean heavy;

    /**
     * This is the quantity of shots that have been fired by this weapon in this battle.
     * i.e. We have 2 x missile 6 where max shots is 2 => qtyShots is a value between 0 and 4 
     */
    protected int qtyShots;

    /**
     * This is true of there is no limit to the number of times this weapon can be fired.
     */
    protected boolean infiniteShots;
    
    /**
     * The details of the bay in the hull that these weapons are contained in. Each bay 
     * on a hull has different characteristics, @see Hull for details.
     */
    protected WeaponBayDetails weaponBaydetails;
    
    /**
     * If the weapon state has changed since last time the user client states have been updated. 
     */
    protected boolean dirty = false;

    protected WeaponInstance(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBaydetails) {
        this.weapon = (Weapon)tb.getTechnology();
        this.qty = tb.getQty();
        this.heavy = heavy;
        this.qtyFired = 0;
        this.qtyShots = 0;
        this.infiniteShots = infiniteShots;
        this.weaponBaydetails = weaponBaydetails;
        this.technology = this.weapon;
    }

    protected WeaponInstance(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBaydetails) {
        this.weapon = weapon;
        this.qty = qty;
        this.heavy = heavy;
        this.qtyFired = 0;
        this.qtyShots = 0;
        this.infiniteShots = infiniteShots;
        this.weaponBaydetails = weaponBaydetails;
        this.technology = this.weapon;
    }

    public void recharge() {
        if (qtyFired > 0) {
            Integer maxQtyShots = weapon.getQtyShots();
            if (infiniteShots || maxQtyShots == null || (maxQtyShots.intValue()-1)*qty >= qtyShots) {
        		dirty = true;		
                qtyFired = 0;
            }
        }
    }
    
    public boolean canRecharge() {
        Integer maxQtyShots = weapon.getQtyShots();
        return (infiniteShots || maxQtyShots == null || (maxQtyShots.intValue()-1)*qty >= qtyShots);
    }

    /**
     * Discharge all weapons (make all weapons fired in this turn).
     * i.e. set qty weapons fired to qty weapons in this weapon instance.
     */
    public void discharge() {
		dirty = true;
        qtyFired = qty;
    }

    public void discharge(int qtyToDischarge) {
		dirty = true;		
        qtyFired += qtyToDischarge;
        if (qtyFired > qty) qtyFired = qty;
    }

    public boolean hasWeaponsToFire() {
        return qtyFired < qty;
    }

    public int getQtyWeaponsToFire() {
        return qty - qtyFired;
    }

    public int getRange() {
        if (heavy) {
            return weapon.getRange() * 2;
        } else {
            return weapon.getRange();
        }
    }

    public double getDamage() {
    	double d = weapon.getDamage();
    	if (d <= 0.0) {
    		d = getDefaultDamage();
    	}
        if (heavy) {
			return d * 2;
        } else {
            return d;
        }
    }

    private static double defaultDamage = 0.0;
    private static int defaultDamageDateCalculated = 0;
	private static double getDefaultDamage() {
		if (defaultDamageDateCalculated < MotEngine.instance.getDate()) {
			defaultDamageDateCalculated = MotEngine.instance.getDate();
			defaultDamage = 1.0;
			Players players = MotEngine.instance.getPlayers();
			if (players != null && players.getQuantity() > 0) {
				Player[] pa = players.getPlayers();
				for (int i = 0; i < pa.length; i++) {
					if (pa[i].getWeapons() != null) {
						Weapon weapon = pa[i].getWeapons().getMaxDamageBeamWeapon();
						if (weapon != null && weapon.getDamage() > defaultDamage) {
							defaultDamage = weapon.getDamage();							
						}
					}
				}
			}
		}
		return defaultDamage;
	}

    public double getDegredation() {
        if (heavy) {
            return weapon.getDegredation() / 2;
        } else {
            return weapon.getDegredation();
        }
    }

	/**
	 * Fires qtyToFire weapons of this type located on source at target.
     * The implementation may vary from weapon to weapon.
	 */
	public abstract void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError;

	/**
     * The implementation may vary from weapon to weapon.
     * This does not do anything for most weapons - but any weapon that can be fired at a 
     * location should define this.
	 */
	public abstract void fire(BattleObject source, Location target, int qtyToFire) throws ApplicationError;

    /**
     * Calculates the damage this weapon would inflict on a target if it were at location loc.
     * This method returns
     *   Damage - modification due to degradation 
     *          + modification due to variance
     *   		+ modification due to condition of source
     *   		+ owner damage bonus
     * @param loc The location an enemy is at upon which this will fire.
     * @param specialsEffectRange If this is true the specials on this object with range bonuses will be used.
     * @return The raw damage this weapon would produce over range between this object and the location loc.
     */
    protected double getDamage(BattleObject source, Location loc, boolean withOwnerBonus) throws ApplicationError {
        Specials sourceSpecials = source.getSpecials();

        double damage = getDamage(source, withOwnerBonus);
        double degredation = getDegredation();
        double range = getRange();
        double rangeBonus = 1.0;
        
        boolean specialsEffectRange = getSpecialsEffectRange();
        if (specialsEffectRange && sourceSpecials != null) {
            rangeBonus += sourceSpecials.getRangeModification();
        }
        
		// check we are in range
		int distance = source.getLocation().getDistanceTo( loc );
		if (distance > range*rangeBonus) return 0.0;

		// Calculate the damage that is generated by this weapon for this firing, This is based on
		// a random value and the comparative computers levels and manoeuvrability of each craft.
        damage = damage - damage * ((degredation * distance) / mot.common.technology.Weapon.DEGREDATION_DISTANCE_FACTOR);
        
        return damage;
    }
    
    /**
     * Calculates the damage this weapon would inflict on a target if it were at location loc.
     * This method returns
     *   Damage - modification due to degradation 
     *          + modification due to variance
     *   		+ modification due to condition of source
     *   		+ owner damage bonus
     *          + owner fleet damage bonuses 
     * @param loc The location an enemy is at upon which this will fire.
     * @param specialsEffectRange If this is true the specials on this object with range bonuses will be used.
     * @return The raw damage this weapon would produce over range between this object and the location loc.
     */
    protected double getDamage(BattleObject source, BattleObject target) throws ApplicationError {
    	double damage = getDamage(source, target.getLocation(), true);
    	
    	double damageBonus = 1.0;
    	
        Battle battle = source.getBattle();
        PlayerBattleBonuses attackerBonuses = battle.getPlayerBonuses(source.getOwner());
        if (attackerBonuses != null) {
        	if (attackerBonuses.getFleetAttackerModification() != null) {
        		damageBonus += attackerBonuses.getFleetAttackerModification();
        	}
        }
        
        PlayerBattleBonuses defenderBonuses = battle.getPlayerBonuses(target.getOwner());
        if (defenderBonuses != null) {
        	if (defenderBonuses.getFleetDefenderModification() != null) {
        		damageBonus -= defenderBonuses.getFleetDefenderModification();
        	}
        }
        
        damage = damage * damageBonus;
        
    	return damage;
    }
    
    public boolean getSpecialsEffectRange() { return true; }
    
    /**
     * @param source The source craft this weapon is mounted on (firing from)
     * @param loc The target of this weapon (usually the location of a craft)
     * @return True if this weapon can fire at location target at this point in time.
     */
    public boolean inRange(BattleObject source, Location loc) {
        Specials sourceSpecials = source.getSpecials();
        double range = getRange();
        double rangeBonus = 1.0;
        
        boolean specialsEffectRange = getSpecialsEffectRange();
        if (specialsEffectRange && sourceSpecials != null) {
            rangeBonus += sourceSpecials.getRangeModification();
        }

		int distance = source.getLocation().getDistanceTo( loc );
		return (distance <= range*rangeBonus);
    }

    
    /**
     * Calculates the damage this weapon would inflict on a target (with no range modification).
     * This method returns
     *   Damage + modification due to variance
     *   		+ modification due to morale
     *   		+ modification due to condition of source craft
     *   		+ owner damage bonus
     * If morale is -1.0 then damage is reduced by the variance while if damage is +1.0 then damage
     * is increased by the variance.
     * If condition is less than 1.0 the damage is reduced by the condition of the craft down to 0.5.
     * So if the condition is 0.7 then the damage will be 70% of normal value. However damage will 
     * never be reduce by more than 50%.
     * @return The raw damage this weapon would produce over range between this object and the location loc.
     */
    protected double getDamage(BattleObject source, boolean withOwnerBonus) throws ApplicationError {
        double damage = getDamage();
        double variance = weapon.getVariance();

        if (variance > 0.0) {
        	double effectiveVariance = variance * source.getMorale();
        	damage += damage * effectiveVariance;
        }
// I think it should be more like this!!!!
//        if (variance > 0.0) {
//        	damage += damage * variance * Utils.random;
//        }
//        double morale = source.getMorale();
//        if (morale > 0.0) {
//        	damage += damage*0.1*morale;
//        } else if (morale < 0.0) { 
//        	damage += damage*0.1*morale;
//        }

        
        double condition = source.getCondition();
        if (condition < 1.0) {
        	damage = damage - damage * (0.5 * (1.0 - condition));
        }
        
        if (withOwnerBonus) {
            double damageBonus = source.getOwner().getOwnerDamageBonus();
            if (damageBonus > 0.0) {
                damage = damage * (1.0 + damageBonus);
            }
        }
        
        return damage;
    }

    protected int getDamageAsInt(BattleObject source, boolean withOwnerBonus) throws ApplicationError {
    	return (int) getDamage(source, withOwnerBonus);
    }
    
	/**
     * Calculates the damage to the craft and shields based upon the specials, weapons, shields etc of both crafts.
     * It returns the damage inflicted by the source craft on the target craft.
     * 
	 * Calculate the damage that is generated by this weapon for this firing, This is based on
	 * the variance and the manoeuvrability and comparative computers levels of each craft.
	 *
	 * Call the specials, there may be special devices that modify the damage produced or
	 * inflicted.
     * 
     * @param source The craft firing this weapon.
     * @param target The craft having this weapon fired at it.
     * @param manvAdvantage The manoeuvrability advantage the source craft has over the target.
     * @param qtyToFire The quantity of weapons in the weapon instance to fire. Thus should be less than or equal 
     * to qty - qtyFired.
     * @return The details of the damage inflicted on the target craft.
     * @throws mot.common.util.ApplicationError
     */
	public WeaponFireResult _fire(BattleObject source, BattleObject target, double manvAdvantage, int qtyToFire, boolean angleEffect) throws ApplicationError {
        Specials sourceSpecials = source.getSpecials();
        Specials targetSpecials = target.getSpecials();
        ShieldInstance [] sourceShields = source.getShields();
        ShieldInstance [] targetShields = target.getShields();
    
        // Initialise the damage done to 0.
        double damage = getDamage(source, target);
        if (damage <= 0.0) return null;

        // The weapon has now fired
        qtyToFire = updateQtyFired(qtyToFire, true);

		// Call the specials.
        if (sourceSpecials != null) {
            damage *= (1.0 + sourceSpecials.getAttackerModification());
        }
        if (targetSpecials != null) {
            damage -= damage*targetSpecials.getDefenderModification();
        }

        // If one craft is twice an manoeuvrable than another, manvAdvantage = 2 => damage is up 100%
        damage = damage * manvAdvantage;

        // This fires all the weapons!
        damage = damage * qtyToFire;

		// Call the shields to reduce the damage (the shields may lose some charge as well).
        double damageBlockedByShields = blockedDamageByShields(targetShields, damage, qty);
        damage -= damageBlockedByShields;
        
        if (angleEffect && target instanceof BattleCraft && !(target instanceof BattleStation)) {
        	int angleOfIncident = target.getLocation().getDirectionFor(source.getLocation());
        	int targetDir = target.getDirection();
        	int a = angleOfIncident - targetDir;
        	if (a < 0) a += 360;
        	if (a > 135 && a < 225) {
        		// Attack is from behind
        		damage = damage*2;
        	} else if ((a >= 225 && a < 315) || (a > 45 && a <= 135)) {
        		// Attack is from the side
        		damage = damage*1.5;
        	}
        }

        if (Logger.isDebug()) {
            Logger.logDebug("Weapon.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" manvAdv="+manvAdvantage+" damage="+damage+" blocked damage="+damageBlockedByShields);
        }

		// The damage we have is the damage to the craft.
        WeaponFireResult result = new WeaponFireResult(damage, damageBlockedByShields);
        return result;
    }
    
    public static double blockedDamageByShields(ShieldInstance [] targetShields, double damage, int qty) {
        double damageBlockedByShields = 0.0;
        if (targetShields != null) {
            double maxDamage = damage;
            for (int i=0; i<targetShields.length; i++) {
                damage = targetShields[i].blockAttack(damage, qty, targetShields.length);
            }
            damageBlockedByShields += maxDamage - damage;
        }
        return damageBlockedByShields;
    }
    
    protected int updateQtyFired(int qtyToFire, boolean updateShots) {
        if (qtyToFire > (qty-qtyFired)) qtyToFire = qty-qtyFired;
        qtyFired += qtyToFire;
        if (updateShots) qtyShots += qtyToFire;
        if (qtyToFire != 0) dirty = true;
        return qtyToFire;
    }
    
    /**
     * Handles any incumbents that effect weapons fire from the craft source to the craft
     * target. A weapon incubence is any incumbance on which the effectsWeapons() returns
     * true. @see Incumbent for details. If a weapon fire incumberence is found this method
     * discharges this weapon and all sends a WeaponFireEvent to all players.
     * Note - If a craft is located inside an incumbent, but close enough to the edge for its
     * image to extend outside the incumbent, then this craft is not considered to be incumbered. 
     * @param source The craft firing a weapon
     * @param target The craft being field at
     * @return True if this has found an incumbent and false otherwise
     */
    public boolean handleEncumbants(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		Incumbents incumbants = source.getBattle().getIncumbents();
    	Location incumbantLoc = incumbants.getFirstWeaponIncumbentEdge(source, target);
    	if (incumbantLoc != null) {
    		if (Logger.isDebug()) {
    			Logger.logDebug("WeaponInstance.handleIncumbants() Stop weapon fire @ "+incumbantLoc); 
    		}
    		discharge(qtyToFire);
    		qtyShots += qtyToFire;
            BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, L.INCUMBERED, null, false, source.getCraftState(), null, null, null);
            event.setTargetLocation(incumbantLoc);
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
			return true;
    	} else {
			return false;
    	}
    }

    /**
     * This does the same as the method above but for weapons fires that are not directed 
     * at another craft (e.g. Spacial Distortion and Time Warp).
     */
    public boolean handleEncumbants(BattleObject source, Location target, int qtyToFire) throws ApplicationError {
    	LocatedArea targetArea = new SimpleLocatedArea(target, new Size(1,1));
		Incumbents incumbants = source.getBattle().getIncumbents();
    	Location incumbantLoc = incumbants.getFirstWeaponIncumbentEdge(source, targetArea);
    	if (incumbantLoc != null) {
    		if (Logger.isDebug()) {
    			Logger.logDebug("WeaponInstance.handleIncumbants() Stop weapon fire @ "+incumbantLoc); 
    		}
    		discharge(qtyToFire);
    		qtyShots += qtyToFire;
            BattleWeaponFireEF event = new BattleWeaponFireEF(source, null, getWeapon(), 0.0, 0.0, L.INCUMBERED, null, false, source.getCraftState(), null, null, null);
            event.setTargetLocation(incumbantLoc);
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
			return true;
    	} else {
			return false;
    	}
    }
    
    /**
     * If a weapon does not work if shields are UP this method will handle the weapon fire.
     */
    public boolean handleShields(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
        ShieldInstance targetShields = target.getShieldsWithHighestPower(Shield.GROUP_SHIELD);
    	if (targetShields != null && targetShields.getPower() > 0.0) {
    		discharge(qtyToFire);
    		qtyShots += qtyToFire;
    		String message = weapon.getName() + " " + L.SHIELD_BLOCKED;
            BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, message, false, source.getCraftState(), target.getCraftState(), null, null);
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
			return true;
    	} else {
			return false;
    	}
    }
    
    public boolean isImune(BattleObject bo) throws ApplicationError {
        Weapon wpn = getWeapon();
		boolean playerImunity = bo.getOwner().isImune(wpn);
    	boolean craftImunity = bo.getSpecials() != null && bo.getSpecials().isImune(wpn.getGroup());
    	boolean imuneToAttack = (bo instanceof BattleMine) && !wpn.isPulse();   
    	return playerImunity || craftImunity || imuneToAttack;
    }
    
    public String getDefaultFailMessage() {
    	return weapon.getName() + " " + L.FAIL; 
    }

    public boolean isHeavy() {
        return heavy;
    }

    public void setHeavy(boolean heavy) {
        this.heavy = heavy;
    }

    public void setInfiniteShots(boolean infiniteShots) {
        this.infiniteShots = infiniteShots;
    }


    public boolean getInfiniteShots() {
        return infiniteShots;
    }


    public void setQtyShots(int qtyShots) {
        this.qtyShots = qtyShots;
    }


    public int getQtyShots() {
        return qtyShots;
    }

    public int getQtyFired() {
        return qtyFired;
    }

	public WeaponBayDetails getWeaponBayDetails() {
		return weaponBaydetails;
	}

	public void setWeaponBaydetails(WeaponBayDetails weaponBaydetails) {
		this.weaponBaydetails = weaponBaydetails;
	}

	/**
	 * There are some weapons that cannot miss - like detonator!
	 */
	public boolean canMiss() {
		return true;
	}

	/**
	 * There are some weapons that can only fire if the angle between the source and the target is 
	 * within those allowed specified within the WeaponBayDetails. Others can fire at any angle.
	 * E.g. 
	 * Beam Weapons - can only fire if the angle permits.
	 * Detonator, Missiles - can fire at any angle. 
	 */
	public boolean usesWeaponBayFireAngle() {
		return true;
	}

    /**
     * Called when the user issues an command to this craft before any actions are carried out.
     * This is used by time warp to avoid discharging weapons after a command to fire all and discharge!
     * @return Return true if the command is to be ignored.
     */
	public boolean notifyStartCommand() {
        return false;
	}

	/**
	 * @return True if this weapon that can fire at a location and has charge.
	 */
	public abstract boolean canAttackLocationAndHasCharge();

	
	/**
	 * @return TRUE if any weapon states requires an updating on each user client (tests dirty flag)
	 */
	public boolean clientsNeedStateUpdate() {
		return dirty;
	}

	/**
	 * Called to mark all client weapon states as updated (not dirty)
	 */
	public void clientsWeaponStateUpdated() {
		dirty = false;
	}

	/**
	 * Examples of weapon fires that have no effect
	 *  - Shield Drain at a craft with no shields or shield power
	 *  - Mind Control at a craft already under control.
	 * @return True iff this weapon when fired at the target will have an effect.
	 * @throws ApplicationError 
	 */
	public boolean willHaveAnEffectOn(BattleObject target) throws ApplicationError {
		return true;
	}

}