/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.engine.MotEngine;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.battles.WeaponDamage;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.players.Player;

/**
 * @author Paul Davis
 *
 */
public class WeaponInstanceBeam extends WeaponInstance {

	public WeaponInstanceBeam(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, heavy, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceBeam(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }

	/**
     * Fires this beam weapon located on the source craft to fire at the target craft. This inflicts
     * damage to the target crafts hull and shields. If damage is caused by this firing then
     * a WeaponFireEvent is sent to the all players proxies.
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
		Player[] startingPlayers = source.getBattle().getStartingUserPlayers();
		
	    if (isImune(target)) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, L.IMMUNE, false, null, null, null, null);
			MotEngine.instance.multicast( event, startingPlayers );
	    	discharge();
	    	return;
	    }

	    double manvAdvantage = source.getManouverabilityAdvantage(target);
	    CraftStateFactory originalState = target.getCraftState();
        
        WeaponFireResult result = _fire(source, target, manvAdvantage, qtyToFire, true);
        
        BattleWeaponFireEF event;
        if (result == null) {
            event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, L.MISS, false, null, null, null, null);
            MotEngine.instance.multicast(event, startingPlayers);
        } else {
        	String damageDescription = Formatter.getDamageDescription(result.getDamage(), result.getShieldBlockedDamage());
            event = new BattleWeaponFireEF(source, target, getWeapon(), result.getDamage(), result.getShieldBlockedDamage(), null, damageDescription, false, source.getCraftState(), originalState, null, null);
            MotEngine.instance.getProxyManager().scheduleMulticast(event, startingPlayers);
            WeaponDamage damage = null;
            if (result.getDamage() > 0.0) {
            	damage = target.damage(result.getDamage(), source.getOwner(), originalState, null);
            }
            event.setNewTargetState(target.getCraftState());
            if (damage != null) {
            	event.setTargetDescription(damage.getDescription());
            }
            if (target.isDestroyed()) event.setDestroyed(true);
            MotEngine.instance.getProxyManager().sendScheduledEvent(event);
        }
        
    }
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

}