/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.event.EventContainerEF;
import mot.engine.players.Player;

public class WeaponInstanceBio extends WeaponInstance {

    public WeaponInstanceBio(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, heavy, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceBio(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }

	/**
     * Fires this bio weapon located on the source craft at the target. The target may be
     * a planet or a craft.
     * The damage of this weapon is the quantity of crew on a craft or population on a 
     * planet killed.
     * 
     * The weapon cannot penetrate shields. 
     * 
     * After the weapon has fired if the target 
     *  - Has crew then the target craft simple has less crew
     *  - Has no crew and all boarding craft are killed the craft is abandoned
     *  - Has no crew and there are still boarding crew then the craft has surrendered
     *
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
		if (handleShields(source, target, qtyToFire)) {
			return;
		}
        
        boolean imune = isImune(target);
        
        CraftStateFactory originalTargetState = target.getCraftState();

        EventContainerEF weaponFireEventContainer = new EventContainerEF();
        MotEngine.instance.getProxyManager().scheduleMulticast(weaponFireEventContainer, source.getBattle().getStartingUserPlayers());

        int peopleKilled = 0;
        int remainingCrew = 0;
        
        qtyToFire = updateQtyFired(qtyToFire, true);
        
        if (!imune) {
            peopleKilled = (int) getDamage(source, target);
            peopleKilled = peopleKilled * qtyToFire;
            remainingCrew = killCrew(target, peopleKilled, source.getOwner());
        }

        if (Logger.isDebug()) {
            Logger.logDebug("Bio.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" peopleKilled="+peopleKilled);
        }

        if (remainingCrew == NO_EFFECT) {
        	return;
        }

        String description = imune ? weapon.getName() + " " + L.IMMUNE : L.KILLED+peopleKilled+" ("+remainingCrew+")";
        BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, description, false, source.getCraftState(), originalTargetState, null, target.getCraftState());
        event.setTargetDescription(description);
        weaponFireEventContainer.setEvent(event);
        MotEngine.instance.getProxyManager().sendScheduledEvent(weaponFireEventContainer);
        
    }
	
	public static final int NO_EFFECT = -1;
	
	public static int killCrew(BattleObject target, int peopleKilled, Player agressor) throws ApplicationError {
		int remainingCrew = 0;
        if (target instanceof BattleCraft) {
            BattleCraft targetCraft = (BattleCraft) target;
            remainingCrew = targetCraft.getCrew()-peopleKilled;
            if (remainingCrew < 0) remainingCrew = 0;
            targetCraft.setCrew(remainingCrew);
            if (targetCraft.getCrew() == 0) {
                targetCraft.setMoveRemaining(0);
                targetCraft.dischargeWeapons();
            }
        } else if (target instanceof BattlePlanet) {
            BattlePlanet targetPlanet = (BattlePlanet) target;
            targetPlanet.killPopulation(peopleKilled, agressor);
            remainingCrew = (int) targetPlanet.getPlanet().getPopulation();
        } else {
            return NO_EFFECT;
        }
        return remainingCrew;
	}
	
	public boolean willHaveAnEffectOn(BattleObject target) {
		return target != null && !target.hasShieldPower();
	}
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }
	
}