/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.technology.Engine;
import mot.common.technology.Engines;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleMine;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.battles.WeaponDamage;
import mot.engine.event.BattleDamageEF;
import mot.engine.event.BattleDamageRingEF;

public class WeaponInstanceDetonator extends WeaponInstance {

    public WeaponInstanceDetonator(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, heavy, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceDetonator(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }

	/**
     * Destroys this craft by causing a warp core overload. This causes an explosion proportional
     * to the warp speed of the craft. The explosion destroys this craft but also damages or destroys
     * all crafts close to this craft. The damage is reduced the further away from this craft.
     * 
     * NOTE - This simply destroys the craft. destroying a craft with a detonator causes the warp core
     * to overload - see BattleObject.destroy() and explode() for details.
     * 
     * @param source The craft firing this weapon
     * @param target Not Used.
     * @param qtyToFire Not Used.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		        
        if (source.isDestroyed()) {
            return;
        }

        if (Logger.isDebug()) {
            Logger.logDebug("Detonator.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName());
        }
        
        source.destroy(null, source.getCraftState(), null);
    }

    /**
     * This causes the detonator on craft source to explode causing damage to all crafts close to this craft.
     * The damage inflicted is returned by getWarpCoreOverloadDamage() on the engine of the source object. However 
     * this is reduced by the distance from this craft.
     * @param source
     */
    public static void explode(BattleObject source) throws ApplicationError {
        if (source == null) {
            return;
        }
        Battle battle = source.getBattle();
        CraftStateFactory craftState = source.getCraftState();
        
        double damage = getWarpCoreOverloadDamage(source);
        
        if (Logger.isDebug()) {
            Logger.logDebug("Detonator.explode() "+source.getName()+" explodes with damage "+damage);
        }

        BattleDamageRingEF event = new BattleDamageRingEF(source, craftState, source.getLocation(), (int)damage, null);
        MotEngine.instance.getProxyManager().scheduleMulticast(event, battle.getStartingUserPlayers());

        BattleCraft [] crafts = battle.getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
                tryToDamage(source, crafts[i], damage, event, (int)damage, true);
            }
        }
        
        Automaton [] autos = battle.getAutomatons();
        if (autos != null) {
            for (int i=0; i<autos.length; i++) {
            	if (!(autos[i] instanceof BattleMine)) {
            		tryToDamage(source, autos[i], damage, event, (int)damage, true);
            	}
            }
        }
        
        MotEngine.instance.getProxyManager().sendScheduledEvent(event);
    }
    
    public static double getWarpCoreOverloadDamage(BattleObject source) throws ApplicationError {
    	double damage = 0.0;
    	
        if (source instanceof BattleCraft) {
            BattleCraft bc = (BattleCraft)source;
            if (bc.getCraft() != null && bc.getCraft().getEngine() != null) {
                damage = bc.getCraft().getEngine().getWarpCoreOverloadDamage();
            }
        }
        
        if (damage == 0.0) {
            damage = 100.0;
            Engines engines = source.getOwner().getEngines();
            if (engines != null) {
                Engine engine = engines.getFastestEngine();
                if (engine != null) {
                    damage = engine.getWarpCoreOverloadDamage();
                }
            }
        }
        
        return damage;
    }

	public static int getExclusionZone(BattleObject source) throws ApplicationError {
		return (int) getWarpCoreOverloadDamage(source);
	}
	

    public static void tryToDamage(BattleObject source, BattleObject target, double damage, BattleDamageRingEF event, int maxDistance, boolean reduceDamageByDistance) throws ApplicationError {
        int dist = source.getDistanceTo(target.getLocation());
        if (maxDistance > dist && source != target && !target.isDestroyed()) {
            double dam = reduceDamageByDistance ? damage - dist : damage; 
            CraftStateFactory oldState = target.getCraftState();
            double damageBlocked = blockedDamageByShields(target.getShields(), damage, 1);
            dam -= damageBlocked;
            if (Logger.isDebug()) {
                Logger.logDebug("WeaponDetoratorOrShieltBurst.damage() "+source.getName()+" Damages "+target.getName()+" dam="+dam+" blocked="+damageBlocked+" dist="+dist);
            }
            WeaponDamage wd = target.damage(dam, source.getOwner(), oldState, event);
            CraftStateFactory newState = target.getCraftState();
            boolean destroyed = target.isDestroyed();
            String damageDescription = (wd == null) ? Formatter.getDamageDescription(damage, damageBlocked) : wd.getDescription();
			BattleDamageEF damageEvent = new BattleDamageEF(target, damageDescription, damage, damageBlocked, destroyed, oldState, newState);
            event.link(damageEvent);
        }
    }
    
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

	public boolean canMiss() { return false; }
	public boolean usesWeaponBayFireAngle() { return false;	}

}