/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.technology.Technologies;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.DroidMine;
import mot.engine.event.BattleDroidMineLaunchEF;

public class WeaponInstanceDroidMine extends WeaponInstance {

    public WeaponInstanceDroidMine(TechnologyBundle tb, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, false, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceDroidMine(Weapon weapon, int qty, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, false, infiniteShots, weaponBayDetails);
    }

	/**
	 * This method fires a droid mine from the source craft at the target craft. The quantity 
     * of shots this weapon has used is increased by one and the weapon is discharged.
     * If the target is out of range or is an Automaton then no action is taken.
     * 
     * Once the droid mine has been launched a DroidMineLaunchEvent is sent to all players proxies.
     * 
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire Ignored. All missiles are always fired.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		// check we are in range
		int distance = source.getLocation().getDistanceTo( target.getLocation() );
		if (distance > getRange()) {
            return;
        }
        if (target instanceof Automaton) {
            return;
        }
        
        setQtyShots(getQtyShots()+1);
        discharge(qtyToFire);
        
        Weapons weapons = Technologies.getWeapons();
        if (weapons == null) {
        	Logger.logWarning("WeaponInstanceDroidMine() No weapons");
        	return;
        }
		Weapon detonator = (Weapon) weapons.getMostAdvancedWithGroup(Weapon.GROUP_DETONATOR);
		if (detonator == null) {
        	Logger.logWarning("WeaponInstanceDroidMine() No Detonator");
			return;
		}
		Battle battle = source.getBattle();
        
		while (qtyToFire > 0) {
			WeaponInstanceDetonator weapon = new WeaponInstanceDetonator(detonator, 1, false, false, getWeaponBayDetails());
	        DroidMine droidMine = null;
	        if (target instanceof BattleCraft) {
	        	droidMine = new DroidMine(source, target, weapon, target);
	            target.add(droidMine);
	        } else {
	        	droidMine = new DroidMine(source, target, weapon, null);
	        }
	        battle.add(droidMine);
	        MotEngine.instance.multicast( new BattleDroidMineLaunchEF(droidMine), battle.getStartingUserPlayers());
	        qtyToFire--;
		}
    }
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

	public boolean canMiss() { return false; }
	public boolean usesWeaponBayFireAngle() { return false;	}

}