/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattleStation;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleWeaponFireEF;

public class WeaponInstanceImmobiliser extends WeaponInstance {


	public WeaponInstanceImmobiliser(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, heavy, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceImmobiliser(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }

	/**
     * Fires this immobiliser weapon located on the source craft at the target craft.
     * This causes the impulse speed of the target craft to be reduced by the damage of this craft.
     * Also, once a craft has some immobilisation it cannot enter warp speed so cannot retreat from battle.
     * 
     * If the weapon is out of range or the targets shields have power to no action is taken.
     *
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
//		ShieldInstance targetShields = target.getShieldsWithHighestPower(Shield.GROUP_SHIELD);
//
//		if (targetShields != null && targetShields.getPower() > 0) {
//            return;
//		}
        
        qtyToFire = updateQtyFired(qtyToFire, true);

	    if (isImune(target) || !(target instanceof BattleCraft) || target instanceof BattleStation) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, weapon.getName() + " " + L.IMMUNE, false, source.getCraftState(), target.getCraftState(), null, null);
	    	MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
	    	return;
	    }

	    double damage = getDamage(source, target);
        if (damage <= 0.0) return;

        CraftStateFactory originalTargetState = target.getCraftState();

        BattleCraft targetCraft = (BattleCraft)target;
        int originalMovementDamage = targetCraft.getMovementDamage();
        damage = damage*qtyToFire;
        targetCraft.setMovementDamage(targetCraft.getMovementDamage()+(int)damage);

        if (Logger.isDebug()) {
            Logger.logDebug("Immobiliser.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" damage="+damage);
        }

        int actualDamage = targetCraft.getMovementDamage() - originalMovementDamage;
        String targetDescription = L.ENGINE_DAMAGE+actualDamage;
        BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, targetDescription, false, source.getCraftState(), originalTargetState, null, target.getCraftState());
        MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
    }
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

	public boolean willHaveAnEffectOn(BattleObject target) throws ApplicationError {
		if (target == null || !(target instanceof BattleCraft) || (target instanceof BattleStation)) {
			return false;
		}
		BattleCraft targetCraft = (BattleCraft)target;
		return targetCraft.getSpeed() > 0;
	}


}