/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleCraftController;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.players.Player;

public abstract class WeaponInstanceMindControlBase extends WeaponInstance {

    public WeaponInstanceMindControlBase(TechnologyBundle tb, WeaponBayDetails weaponBayDetails) {
        super(tb, false, false, weaponBayDetails);
    }

    public WeaponInstanceMindControlBase(Weapon weapon, int qty, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, false, false, weaponBayDetails);
    }
    
    protected abstract BattleCraftController createController(Player owner, BattleCraft targetCraft, int qtyTurns);

	/**
     * Fires this mined control weapon that causes the crew on the enemy ship to lose
     * control. The target craft will attack the closest craft to them regardless of 
     * owner on their next turn.
     * 
     * The damage of this weapon is the chance to succeed where a value of 1 is certainty.
     * Note the damage is reduced by psi shields.
     * 
     * The weapon may have a limited number of shots so the quantity of shots is increased
     * every time this weapon fires successful (target goes berserk). The owner does not 
     * lose a shot for failed attempts! 
     * 
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire Does not have any affect.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
        qtyToFire = updateQtyFired(qtyToFire, true);
        
		if (!(target instanceof BattleCraft)) {
            BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, getFailMessage(), false, source.getCraftState(), target.getCraftState(), null, null);
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
			return;
		}
		
		BattleCraft targetCraft = (BattleCraft) target;
		
		if (targetCraft.getControllerDetails() != null) {
            BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, getFailMessage(), false, source.getCraftState(), target.getCraftState(), null, null);
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
			return;
		}
		
        boolean imune = isImune(target);
        
        CraftStateFactory originalTargetState = target.getCraftState();

        double damage = 0;
        boolean success = false;
        if (!imune) {
            damage = getDamage(source, target);
            damage = targetCraft.psiBlockShields(damage);
    		int qtyTurns = (int)Math.floor(damage);
    		if(Utils.random(Comp.min(1.0, damage))) {
    			qtyTurns++;
    		}
    		success = (qtyTurns > 0);
    		if (success) {
    			Player owner = source.getOwner();
				targetCraft.setControllerDetails(createController(owner, targetCraft, qtyTurns));
    		}
        }

        if (Logger.isDebug()) {
            Logger.logDebug(weapon.getName()+".fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" success="+success+" dam="+damage);
        }

        String description = imune ? weapon.getName() + " " + L.IMMUNE : (success ? getSuccessMessage() : getFailMessage());
        BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, description, false, source.getCraftState(), originalTargetState, null, target.getCraftState());
        MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
    }
	
	public abstract String getSuccessMessage();
	public abstract String getFailMessage();
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	
	public boolean willHaveAnEffectOn(BattleObject target) throws ApplicationError {
		if (target == null || !(target instanceof BattleCraft)) {
			return false;
		}
		BattleCraft targetCraft = (BattleCraft)target;
		return targetCraft.getControllerDetails() == null;
	}

}