/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.battles.triggers.CraftTriggerBase;
import mot.engine.battles.triggers.Plague;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.event.EventContainerEF;

/**
 * This is a weapon that creates an end turn trigger that kills a number 
 * of crew at the end of each turn. For details of Triggers see the Trigger
 * interface in the battle package.
 */
public class WeaponInstancePlague extends WeaponInstance {
	

	public WeaponInstancePlague(Weapon weapon, int qty, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, false, false, weaponBayDetails);
    }

	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
		if (handleShields(source, target, qtyToFire)) {
			return;
		}
                
        EventContainerEF weaponFireEventContainer = new EventContainerEF();
        MotEngine.instance.getProxyManager().scheduleMulticast(weaponFireEventContainer, source.getBattle().getStartingUserPlayers());

        qtyToFire = updateQtyFired(qtyToFire, true);
        
		Battle battle = source.getBattle();
		Plague trigger = (Plague) battle.getTriggersNotNull().getBattleObjectTriggerOn(target, Plague.class);
		int dam = getDamageAsInt(source, false) * qtyToFire;
        boolean imune = isImune(target);
        if (!imune) {
			if (trigger != null) {
				trigger.setQtyKilledPerTurn(trigger.getQtyKilledPerTurn() + dam);
			} else {
				trigger = new Plague(source.getOwner(), battle, target, CraftTriggerBase.INFINATE_LIFE, dam);
				battle.getTriggersNotNull().add(trigger);
			}
        }

        if (trigger != null) {
        	target.getDecorators().update();
        }

        String description = imune ? weapon.getName() + " " + L.IMMUNE : L.PLAGUE+dam;
        CraftStateFactory targetState = target.getCraftState();
		BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, description, false, source.getCraftState(), targetState, null, null);
        weaponFireEventContainer.setEvent(event);
        MotEngine.instance.getProxyManager().sendScheduledEvent(weaponFireEventContainer);
	}

	public boolean willHaveAnEffectOn(BattleObject target) {
		return target != null && !target.hasShieldPower();
	}
	

	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

}
