/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleWeaponFireEF;

public class WeaponInstancePsi extends WeaponInstance {


	public WeaponInstancePsi(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, heavy, infiniteShots, weaponBayDetails);
    }

    public WeaponInstancePsi(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }
    
    private static final double HEAL_FACTOR = 1.0;

	/**
     * Fires this psi attack weapon located on the source craft at the target craft.
     * This causes the target craft to accumulate psi damage. The psi damage effects
     * the crafts ability to fire its weapons - see BattleCraft.fireWeapons() for details.
     *
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
				
		boolean imune = isImune(target);
        if (!(target instanceof BattleCraft)) {
        	imune = true;
        }
        
        CraftStateFactory originalTargetState = target.getCraftState();

        qtyToFire = updateQtyFired(qtyToFire, true);
        
        double damage = 0.0;
        if (!imune) {
            damage = getDamage(source, target);
            if (damage <= 0.0) return;
    
            BattleCraft targetCraft = (BattleCraft)target;
            damage = damage*qtyToFire;

            if (target.getOwner() == source.getOwner()) {
                damage = -damage*HEAL_FACTOR; 
            } else {
       	        damage = targetCraft.psiBlockShields(damage);
            }

    		targetCraft.psiDamage(damage);
        }

        if (Logger.isDebug()) {
            Logger.logDebug("Psi.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" damage="+damage);
        }

        String dam = null;
        if (imune) {
        	dam = weapon.getName() + " " + L.IMMUNE;
        } else if (damage < 0.0) {
        	dam = L.PSI_HEAL+Formatter.fractionWithSign(-damage);
        } else {
        	dam = L.PSI_DAMAGE+Formatter.fractionWithSign(damage);
        }
        BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, dam, false, source.getCraftState(), originalTargetState, null, target.getCraftState());
        MotEngine.instance.multicast( event,  source.getBattle().getStartingUserPlayers() );
    }

	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

}