/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import java.util.ArrayList;

import mot.common.bean.Location;
import mot.common.technology.Specials;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattleStation;
import mot.engine.battles.CraftStateFactory;
import mot.engine.battles.WeaponDamage;
import mot.engine.event.BattleDamageEF;
import mot.engine.event.BattlePulseEF;
import mot.engine.players.Player;

public class WeaponInstancePulse extends WeaponInstance {

    public WeaponInstancePulse(TechnologyBundle tb, WeaponBayDetails weaponBayDetails) {
        super(tb, false, true, weaponBayDetails);
    }

    public WeaponInstancePulse(Weapon weapon, int qty, WeaponBayDetails weaponBayDetails) {
    	super(weapon, qty, false, true, weaponBayDetails);
    }

	/**
     * Fires a pulse of radial energy out from the craft and damage all crafts in its arc of fire.
     * 
     * @param source The craft firing this weapon
     * @param target The target this weapon is fired at.
     * @param qtyToFire The quantity of weapons to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
        if (source.isDestroyed()) {
            return;
        }

        if (Logger.isDebug()) {
            Logger.logDebug("WeaponInstancePulse.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName());
        }
        
        Location srcLocation = source.getLocation();
        int weaponRange = weapon.getRange();
        int weaponAngle = weapon.getParam1AsInt();
        if (weaponAngle == 0) Logger.logWarning("WeaponInstancePulse.fire() No Angle");
        
        ArrayList<BattleObject> damagedObjects = new ArrayList(); 
        Battle battle = source.getBattle();
        BattleCraft [] crafts = battle.getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	if (crafts[i] != source) {
            		Location targetLocation = crafts[i].getLocation();
					if (srcLocation.getDistanceTo(targetLocation) <= weaponRange) {
            			int targetDirection = srcLocation.getDirectionFor(targetLocation);
            			int srcDirection = source.getDirection();
            			int change = targetDirection - srcDirection;
            			if (change < 0) change = -change;
            			if (change <= weaponAngle) {
            				if (!isImune(crafts[i])) {
            					damagedObjects.add(crafts[i]);
            				}
            			}
            		}
            	}
            }
        }
        
        Automaton[] automatons = battle.getAutomatons();
        if (automatons != null) {
        	for (int i = 0; i < automatons.length; i++) {
        		Location targetLocation = automatons[i].getLocation();
				if (srcLocation.getDistanceTo(targetLocation) <= weaponRange) {
        			int targetDirection = srcLocation.getDirectionFor(targetLocation);
        			int srcDirection = source.getDirection();
        			int change = targetDirection - srcDirection;
        			if (change < 0) change = -change;
        			if (change <= weaponAngle) {
       					damagedObjects.add(automatons[i]);
        			}
        		}
			}
        }
        
        CraftStateFactory craftState = source.getCraftState();
		Player[] startingPlayers = source.getBattle().getStartingUserPlayers();
		BattlePulseEF event = new BattlePulseEF(source, craftState, weapon);
		
        if (damagedObjects.size() == 0) {
            MotEngine.instance.multicast(event, startingPlayers);
            return;
        }
        
        MotEngine.instance.getProxyManager().scheduleMulticast(event, startingPlayers);
        
        qtyToFire = updateQtyFired(qtyToFire, true);
        for (BattleObject damagedObject : damagedObjects) {
        	CraftStateFactory oldState = damagedObject.getCraftState();
            WeaponFireResult result = _fire(source, damagedObject, qtyToFire);
			if (result.getTotalDamage() > 0.0) {
				double damage = result.getDamage();
				if (Logger.isDebug()) {
					Logger.logDebug("WeaponInstancePulse.damage() "+source.getName()+" Damages "+damagedObject.getName()+" dam="+damage);
				}
    			int qtyLinkedEvents = event.getQtyLinkedEvents();
				WeaponDamage wd = damagedObject.damage(damage, source.getOwner(), damagedObject.getCraftState(), event);
                CraftStateFactory newState = damagedObject.getCraftState();
                boolean destroyed = damagedObject.isDestroyed();
                double damageBlocked = result.getShieldBlockedDamage();
				String damageDescription = (wd == null) ? Formatter.getDamageDescription(damage, damageBlocked) : wd.getDescription();
    			BattleDamageEF damageEvent = new BattleDamageEF(damagedObject, damageDescription, damage, damageBlocked, destroyed, oldState, newState);
                event.link(damageEvent, qtyLinkedEvents);
            }
        }
        
        MotEngine.instance.getProxyManager().sendScheduledEvent(event);

    }
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

	public boolean canMiss() { return false; }
	public boolean usesWeaponBayFireAngle() { return false;	}
	
	
	private WeaponFireResult _fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
        Specials sourceSpecials = source.getSpecials();
        Specials targetSpecials = target.getSpecials();
        ShieldInstance [] sourceShields = source.getShields();
        ShieldInstance [] targetShields = target.getShields();
	    double manvAdvantage = source.getManouverabilityAdvantage(target);
    
        // Initialise the damage done to 0.
        double damage = getDamage(source, target);
        if (damage <= 0.0) return null;

		// Call the specials.
        if (sourceSpecials != null) {
            damage *= (1.0 + sourceSpecials.getAttackerModification());
        }
        if (targetSpecials != null) {
            damage -= damage*targetSpecials.getDefenderModification();
        }

        // If one craft is twice an manoeuvrable than another, manvAdvantage = 2 => damage is up 100%
        damage = damage * manvAdvantage;

        // This fires all the weapons!
        damage = damage * qtyToFire;

		// Call the shields to reduce the damage (the shields may lose some charge as well).
        double damageBlockedByShields = blockedDamageByShields(targetShields, damage, qty);
        damage -= damageBlockedByShields;
        
        if (target instanceof BattleCraft && !(target instanceof BattleStation)) {
        	int angleOfIncident = target.getLocation().getDirectionFor(source.getLocation());
        	int targetDir = target.getDirection();
        	int a = angleOfIncident - targetDir;
        	if (a < 0) a += 360;
        	if (a > 135 && a < 225) {
        		// Attack is from behind
        		damage = damage*2;
        	} else if ((a >= 225 && a < 315) || (a > 45 && a <= 135)) {
        		// Attack is from the side
        		damage = damage*1.5;
        	}
        }

        if (Logger.isDebug()) {
            Logger.logDebug("WeaponInstancePulse.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" manvAdv="+manvAdvantage+" damage="+damage+" blocked damage="+damageBlockedByShields);
        }

		// The damage we have is the damage to the craft.
        WeaponFireResult result = new WeaponFireResult(damage, damageBlockedByShields);
        return result;
    }
    

}