/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattleStation;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleCraftNewEF;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.players.Player;

public class WeaponInstanceReplicaBeam extends WeaponInstance {

	private static final int BORALIUM_COST = 25;

	public WeaponInstanceReplicaBeam(TechnologyBundle tb, WeaponBayDetails weaponBayDetails) {
        super(tb, false, false, weaponBayDetails);
        setQty(1);
    }

    public WeaponInstanceReplicaBeam(Weapon weapon, WeaponBayDetails weaponBayDetails) {
        super(weapon, 1, false, false, weaponBayDetails);
    }

	/**
	 * This method fires this replica beam from the source craft at the target craft. This weapon uses
	 * a boralium crystal EVERY time it is fired. If the player does not have boralium this does nothing.
	 * Otherwise it produces a cop of the target craft owned by this weapons owner.
     * 
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire Ignored. All missiles are always fired.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		// check we are in range
		int distance = source.getLocation().getDistanceTo( target.getLocation() );
		if (distance > getRange()) {
            return;
        }

        discharge();
        
        if (Logger.isDebug()) {
            Logger.logDebug("ReplicaBeam.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName());
        }

        CraftStateFactory srcState = source.getCraftState();
        CraftStateFactory targetState = target.getCraftState();
        
        if ((target instanceof BattleCraft) && !(target instanceof BattleStation)) {
        	BattleCraft targetCraft = (BattleCraft) target;
        	Player owner = source.getOwner();
			if (owner.getBoralium() < BORALIUM_COST) {
				MotEngine.instance.multicast( new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, L.REPLICA_BEAM_NO_BORALIUM, null, false, srcState, targetState, srcState, targetState), source.getBattle().getStartingUserPlayers() );
			} else if (isImune(target)) {
				MotEngine.instance.multicast( new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, weapon.getName() + " " + L.IMMUNE, false, srcState, targetState, srcState, targetState), source.getBattle().getStartingUserPlayers() );
        	} else {
        		owner.addBoralium(-BORALIUM_COST);
            	BattleCraft newCraft = targetCraft.duplicate(owner, targetCraft.getLocation(), source.getLocation());
            	CraftStateFactory newCraftState = newCraft.getCraftState(); 
            	BattleCraftNewEF event = new BattleCraftNewEF(newCraft, true);
            	BattleWeaponFireEF wfe1 = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, L.BORALIUM_USED+owner.getBoralium(), null, false, srcState, targetState, srcState, targetState);
            	wfe1.setReverseAnimation(true);
				event.link(wfe1);
                BattleWeaponFireEF wfe2 = new BattleWeaponFireEF(source, newCraft, getWeapon(), 0.0, 0.0, null, "", false, srcState, newCraftState, srcState, newCraftState);
                wfe2.setAnimationConfigNameSuffix(L.PUT);
				event.link(wfe2);
				MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
        	}
        } else {
			MotEngine.instance.multicast( new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, L.REPLICA_BEAM_NO_EFFECT, false, srcState, targetState, srcState, targetState), source.getBattle().getStartingUserPlayers() );
        }
    }
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }


}