/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.battles.MovingAutamaton;
import mot.engine.event.BattleWeaponFireEF;

public class WeaponInstanceRetarget extends WeaponInstance {


	public WeaponInstanceRetarget(TechnologyBundle tb, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, false, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceRetarget(Weapon weapon, int qty, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, false, infiniteShots, weaponBayDetails);
    }

	/**
     * Fires the retarget weapon located on the source craft to fire at the target automaton.
     * This weapon does not inflict any damage, but causes automatons that target an individual
     * craft to target on another craft.
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
		if (!(target instanceof MovingAutamaton)) {
	        BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, getWeapon().getName()+" "+L.IMMUNE, false, source.getCraftState(), target.getCraftState(), null, null);
	        MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
			return;
		}
		
		MovingAutamaton targetAutomiton = (MovingAutamaton) target;
		
		CraftStateFactory originalTargetState = targetAutomiton.getCraftState();
        qtyToFire = updateQtyFired(qtyToFire, true);
		double damage = getDamage(source, false) * qtyToFire;
        
    	BattleCraft newTarget = null;
    	
        if (weapon.getParam1AsInt() > 0) {
        	// Beam is Missile Retarget
    		boolean success = Utils.random(damage);
            if (success) {
            	Battle battle = source.getBattle();
            	Location location = target.getLocation();
        		BattleCraft bc = battle.getClosestEnemyCraft(source.getOwner(), location, false);
        		if (bc != null) {
        			newTarget = bc;
        			targetAutomiton.retargetTo(bc);
        		}
    		}
    		if (Logger.isDebug()) {
    			if (newTarget != null) {
    	            Logger.logDebug("Retarget.fire() "+source.getName()+" fires "+weapon.getName()+" at "+target.getName()+" retarget to "+newTarget);
    			} else {
    	            Logger.logDebug("Retarget.fire() "+source.getName()+" fires "+weapon.getName()+" at "+target.getName()+" fails to retarget");
    			}
            }

            String description = (newTarget != null) ? L.SUCCESS : getDefaultFailMessage();
            BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, description, false, source.getCraftState(), originalTargetState, null, target.getCraftState());
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
        } else {
        	int qtyMissiles = targetAutomiton.getQty();
        	// Beam is Missile Destroy
        	targetAutomiton.damage(damage, source.getOwner(), originalTargetState, null);
        	int newQtyMissiles = targetAutomiton.getQty();
        	int qtyDestroyed = qtyMissiles - newQtyMissiles;
        	if (Logger.isDebug()) {
       			Logger.logDebug("Retarget.fire() "+source.getName()+" fires "+weapon.getName()+" at "+target.getName()+" dam "+damage+" qtyDestroyed "+qtyDestroyed);
        	}
        	String damageDescription = Formatter.getDamageDescription(damage, 0.0);
       		damageDescription += " ("+qtyDestroyed+" "+L.DESTROYED+")";
        	BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, damageDescription, target.isDestroyed(), source.getCraftState(), originalTargetState, null, target.getCraftState());
        	MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
        }
        

    }
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

	public boolean canMiss() { return false; }
}