/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.bean.SalvageList;
import mot.common.bean.WreckageDetails;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.BattleObject;
import mot.engine.battles.Wreckage;
import mot.engine.battles.WreckageCraft;
import mot.engine.battles.Wreckages;
import mot.engine.event.BattleWeaponFireEF;

/**
 * This is a weapon that can be fired at the wreckage of another craft and salvage weapons, shields
 * and all other components that make up a ship. Note that the quantity of salvagable components in
 * wreckage can vary a lot depending on the technology and owner abilities of that craft.  
 * 
 * It has no effect when fired at anything other than debris.
 */
public class WeaponInstanceSalvageBeam extends WeaponInstance {
	
	private static final String NO_EFFECT = "Salvage - No Effect";

	public WeaponInstanceSalvageBeam(Weapon weapon) {
        super(weapon, 1, false, false, null);
    }
	
	public WeaponInstanceSalvageBeam(Weapon weapon, int qty, boolean infiniteShots, WeaponBayDetails weaponBaydetails) {
		super(weapon, qty, false, infiniteShots, weaponBaydetails);
	}

	private int qtySalvageCollected = 0;
	
    public void recharge() {
        if (qtyFired > 0) {
            Integer maxQtyShots = weapon.getQtyShots();
            if (infiniteShots || maxQtyShots == null || maxQtyShots.intValue() * qty > qtySalvageCollected) {
                qtyFired = 0;
            }
        }
    }
    
    public boolean canRecharge() {
        Integer maxQtyShots = weapon.getQtyShots();
        return (infiniteShots || maxQtyShots == null || maxQtyShots.intValue() * qty > qtySalvageCollected);
    }
    
    public boolean hasWeaponsToFire() {
        return qtyFired < qty;
    }
    
    public int getQtySalvageCanCollect() {
        Integer maxQtyShots = weapon.getQtyShots();
        if (maxQtyShots != null) {
        	return maxQtyShots.intValue() * qty - qtySalvageCollected;
        } else {
        	return Integer.MAX_VALUE;
        }
    }

	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
//        discharge();
//        WeaponFireEF event = new WeaponFireEF(source, target, getWeapon(), 0.0, 0.0, false, null, null, null, null);
//        event.setTargetDescription(NO_EFFECT);
//        event.setTargetLocation(target.getLocation());
//        MotEngine.instance.broadcast( event );
	}
	
	private void sendWeaponFireEvent(BattleObject source, Location target, String message, WreckageDetails wd) throws ApplicationError {
        BattleWeaponFireEF event = new BattleWeaponFireEF(source, null, getWeapon(), 0.0, 0.0, null, null, false, source.getCraftState(), null, null, null);
        event.setTargetDescription(message);
        event.setTargetLocation(target);
        event.setReverseAnimation(true);
        event.setData(wd);
        MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
	}
	
	public void fire(BattleObject source, Location target, int qtyToFire) throws ApplicationError { 
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
        if (Logger.isDebug()) {
            Logger.logDebug("SalvageBeam.fire() "+source.getName()+" fires "+weapon.getName()+" at "+target);
        }
		
        qtyToFire = updateQtyFired(1, true);
        
        double damage = getDamage();
        Wreckages wreckages = source.getBattle().getWreckages();
        Wreckage wreckage = wreckages.getWreckageAt(target);
        if (!(wreckage instanceof WreckageCraft)) {
        	sendWeaponFireEvent(source, target, NO_EFFECT, null);
        	return;
        }
        WreckageCraft craftWreckage = (WreckageCraft) wreckage;
        
        SalvageList salvage = craftWreckage.getSalvage();
		wreckages.remove(wreckage);
        if (salvage != null) {
    		int qtyToCollect = getQtySalvageCanCollect();
        	salvage = salvage.getSalvage(qtyToCollect);
        	source.addSalvage(salvage);
        	qtySalvageCollected += salvage.getQuantity();
        }
        
        WreckageDetails wd = new WreckageDetails(wreckage.getRectangle(), craftWreckage.getBattleCraft().getName());
    	sendWeaponFireEvent(source, target, "Salvage Transported", wd);
	}

	public boolean canAttackLocationAndHasCharge() { return hasWeaponsToFire(); }

}
