/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.technology.Shield;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleMine;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleDamageRingEF;

public class WeaponInstanceShieldBurst extends WeaponInstance {

    public WeaponInstanceShieldBurst(TechnologyBundle tb, WeaponBayDetails weaponBayDetails) {
        super(tb, false, true, weaponBayDetails);
    }

    public WeaponInstanceShieldBurst(Weapon weapon, WeaponBayDetails weaponBayDetails) {
    	super(weapon, 1, false, true, weaponBayDetails);
    }

	/**
     * Causes the shields to burst out from the craft and damage all crafts around it.
     * 
     * @param source The craft firing this weapon
     * @param target Not Used.
     * @param qtyToFire Not Used.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
        if (source.isDestroyed()) {
            return;
        }

        qtyToFire = updateQtyFired(1, true);
        
        if (Logger.isDebug()) {
            Logger.logDebug("WeaponInstanceShieldBurst.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName());
        }
        
        ShieldInstance shield = source.getShieldsWithHighestPower(Shield.GROUP_SHIELD);
        if (shield == null) {
        	return;
        }
        if ( shield.getPower() < 1 ) {
        	return;
        }
        
        double shieldPowerPercent = shield.getPower() / shield.getMaxShieldCharge();
        double damage = weapon.getDamage();
        double actualDamage = damage/2 + (damage/2)*shieldPowerPercent;
        
        Battle battle = source.getBattle();
        CraftStateFactory craftState = source.getCraftState();

        BattleDamageRingEF event = new BattleDamageRingEF(source, craftState, source.getLocationClone(), (int)damage, weapon);
        MotEngine.instance.getProxyManager().scheduleMulticast(event, battle.getStartingUserPlayers());

        BattleCraft [] crafts = battle.getCrafts();
        if (crafts != null) {
            for (int i=0; i<crafts.length; i++) {
            	if (crafts[i] != source) {
            		WeaponInstanceDetonator.tryToDamage(source, crafts[i], damage, event, weapon.getRange(), false);
            	}
            }
        }
        
        Automaton [] autos = battle.getAutomatons();
        if (autos != null) {
            for (int i=0; i<autos.length; i++) {
            	if (!(autos[i] instanceof BattleMine)) {
            		WeaponInstanceDetonator.tryToDamage(source, autos[i], damage, event, weapon.getRange(), false);
            	}
            }
        }
        
        MotEngine.instance.getProxyManager().sendScheduledEvent(event);
    }
	
	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

	public boolean canMiss() { return false; }
	public boolean usesWeaponBayFireAngle() { return false;	}
	
}