/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.Shield;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Formatter;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.players.Player;

public class WeaponInstanceShieldDrain extends WeaponInstance {


	public WeaponInstanceShieldDrain(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, heavy, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceShieldDrain(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }

	/**
     * Fires this shield drain weapon located on the source craft to fire at the target craft. 
     * This causes power to be drained from the target craft's shield and added to the source crafts
     * shields.
     * 
     * If the weapon is out of range or there is no shield power to drain no action is taken otherwise 
     * the weapon is discharged.
     * 
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject attacker, BattleObject target, int qtyToFire) throws ApplicationError {
		if (handleEncumbants(attacker, target, qtyToFire)) {
			return;
		}
		
        qtyToFire = updateQtyFired(qtyToFire, true);

	    if (isImune(target)) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(attacker, target, getWeapon(), 0.0, 0.0, null, weapon.getName() + " " + L.IMMUNE, false, attacker.getCraftState(), target.getCraftState(), null, null);
	    	MotEngine.instance.multicast( event, attacker.getBattle().getStartingUserPlayers() );
	    	return;
	    }

	    ShieldInstance sourceShields = attacker.getShieldsWithLowestPower(Shield.GROUP_SHIELD);
        ShieldInstance targetShields = target.getShieldsWithHighestPower(Shield.GROUP_SHIELD);
        
        double maxShieldCharge = 0.0;
        if (sourceShields != null) {
            maxShieldCharge = sourceShields.getMaxShieldCharge() - sourceShields.getPower();
        }
        BattleObject source = attacker;
        if (maxShieldCharge < 1.0) {
            BattleCraft craft = getCraftWithLowestPower(source.getBattle(), source.getOwner(), source.getLocation(), getRange());
            if (craft != null && craft != source) {
                source = craft;
                sourceShields = craft.getShieldsWithLowestPower(Shield.GROUP_SHIELD);
                if (sourceShields != null) {
                    maxShieldCharge = sourceShields.getMaxShieldCharge() - sourceShields.getPower();
                }
            }
        }

		if (targetShields == null) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(attacker, target, getWeapon(), 0.0, 0.0, null, weapon.getName() + " " + L.NO_EFFECT, false, attacker.getCraftState(), target.getCraftState(), null, null);
	    	MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
			return;
		}
        
        double damage = getDamage(attacker, target);
        if (damage <= 0.0) return;

        CraftStateFactory originalTargetState = target.getCraftState();
        CraftStateFactory originalSourceState = source.getCraftState();

        double maxPower = damage*qtyToFire;
        maxPower = Comp.min(maxPower, targetShields.getPower());
        double power = Comp.min(maxPower, maxShieldCharge);
        
        if (sourceShields != null) {
	        double originalSource = sourceShields.getPower();
	        sourceShields.setPower(originalSource + power);
        }
        
        double originalTarget = targetShields.getPower();
        targetShields.setPower(originalTarget - maxPower);
        
        String powerStrTarget = "-"+Formatter.fraction(maxPower);
        String powerStrSrc = "+"+Formatter.fraction(power);

        if (Logger.isDebug()) {
            Logger.logDebug("ShieldDrain.fire() "+attacker.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" drain power="+powerStrTarget+" power "+powerStrSrc+" given to "+attacker.getName());
        }

        if (source != attacker) {
        	CraftStateFactory attackerState = attacker.getCraftState();
            BattleWeaponFireEF event = new BattleWeaponFireEF(attacker, target, getWeapon(), 0.0, power, null, powerStrTarget, false, attackerState, originalTargetState, null, target.getCraftState());
            event.setReverseAnimation(true);
            BattleWeaponFireEF linkedEvent = new BattleWeaponFireEF(attacker, source, getWeapon(), 0.0, power, null, powerStrSrc, false, attackerState, originalSourceState, null, source.getCraftState());
            event.link(linkedEvent);
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
        } else {
            BattleWeaponFireEF event = new BattleWeaponFireEF(attacker, target, getWeapon(), 0.0, power, powerStrSrc, powerStrTarget, false, originalSourceState, originalTargetState, source.getCraftState(), target.getCraftState());
            event.setReverseAnimation(true);
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
        }
        
    }
    
    private static BattleCraft getCraftWithLowestPower(Battle battle, Player owner, Location loc, long range) throws ApplicationError {
        BattleCraft [] crafts = battle.getCrafts();
        if (crafts == null) return null;
        double power = 0.0;
        int bestIndex = -1;
        for (int i=0; i<crafts.length; i++) {
            int dist = crafts[i].getLocation().getDistanceTo(loc);
            if (crafts[i].getOwner() == owner && dist <= range) {
                ShieldInstance shields = crafts[i].getShieldsWithLowestPower(Shield.GROUP_SHIELD);
                if (shields != null) {
                    double p = shields.getMaxRecharge();
                    if (p > power) {
                        bestIndex = i;
                        power = p;
                    }
                }
            }
        }
        if (bestIndex >= 0) {
            return crafts[bestIndex];
        } else {
            return null;
        }
    }

	public boolean willHaveAnEffectOn(BattleObject target) {
		return target != null && target.hasShieldPower();
	}

	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }

	public boolean canMiss() { return false; }
	public boolean usesWeaponBayFireAngle() { return false;	}
	
}