/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Area;
import mot.common.bean.Location;
import mot.common.bean.Rectangle;
import mot.common.bean.SpaceDistortionDetails;
import mot.common.lang.L;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.incumbants.IncumbentFactory;
import mot.engine.battles.incumbants.Incumbents;
import mot.engine.battles.incumbants.SpaceDistortion;
import mot.engine.common.ServerImageObject;
import mot.engine.event.BattleWeaponFireEF;
import mot.engine.players.Player;

/**
 * This is a weapon that creates a spatial distortion on the battle field (an incumbent
 * see Incumbent for details). This weapon can only be placed onto a planet and can only be 
 * fired once (recharge has no effect).
 * 
 * The spatial distortion is created in front of the craft target. The distortion is a 
 * rectangle with width 50 and height of the weapon damage. The distortion cannot overlap
 * an asteroid field.
 *
 */
public class WeaponInstanceSpacialDistortion extends WeaponInstance {
	
	public static final int DEFAULT_WIDTH = 50;
	public static final int MIN_DIST_TO_PLANET = 150;
	public static final int SPACE_AROUND_REQUIRED = 10;

    public WeaponInstanceSpacialDistortion(Weapon weapon, int qty, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, false, false, weaponBayDetails);
    }
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		int width = DEFAULT_WIDTH;
		int height = (int)getWeapon().getDamage();
		int x = target.getLocation().getX() - target.getSpaceX();
		int y = target.getLocation().getY();
		fire(source, new Location(x,y), qtyToFire);
	}
	
	public void fire(BattleObject source, Location target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		int width = DEFAULT_WIDTH;
		int height = (int)getWeapon().getDamage();
		int x = target.getX();
		int y = target.getY();
		Rectangle sdArea = new Rectangle(x-width/2, y-height/2, width, height);
		Area sdAreaRequired = new Area(x-width/2-SPACE_AROUND_REQUIRED, y-height/2-SPACE_AROUND_REQUIRED, x+width/2+SPACE_AROUND_REQUIRED, y+height/2+SPACE_AROUND_REQUIRED);
		
		Battle battle = source.getBattle();
		Incumbents incumbents = battle.getIncumbents();
		Player owner = source.getOwner();
		ServerImageObject[] areaOccupiers = battle.getAreaOccupiers(null, sdAreaRequired);
		BattlePlanet planet = battle.getPlanet();
		String sourceDescription = null;
		String targetDescription = null;
		if (areaOccupiers != null || (planet != null && planet.getLocation().getDistanceTo(target) < MIN_DIST_TO_PLANET)) {
			sourceDescription = L.FAIL_SPACE_INCUMBERED_BY_SPACE_OBJECTS;
		}
		if (!incumbents.isEmpty(sdArea)) {
			targetDescription = L.FAIL_SPACE_INCUMBERED_BY_SPACE_OBJECTS;
		}

		SpaceDistortionDetails sdd = null;
		if (sourceDescription == null && targetDescription == null) {
			SpaceDistortion sd = IncumbentFactory.createSpacialDistortion(battle, owner, sdArea);
			sdd = sd.getSpaceDistortionDetails();
	        if (Logger.isDebug()) {
	            Logger.logDebug("SpacialDistortion.fire() "+source.getName()+" creates Incumbent at x="+x+" y="+y+" width="+width+" height="+height);
	        }
			discharge(1);
		} else {
	        if (Logger.isDebug()) {
	            Logger.logDebug("SpacialDistortion.fire() "+source.getName()+" fails at x="+x+" y="+y+" width="+width+" height="+height);
	        }
		}

        BattleWeaponFireEF event = new BattleWeaponFireEF(source, null, getWeapon(), 0.0, 0.0, sourceDescription, targetDescription, false, source.getCraftState(), null, null, null);
        event.setTargetLocation(new Location(x, y));
        event.setData(sdd);
        MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
	}

	public boolean canAttackLocationAndHasCharge() { return hasWeaponsToFire(); }
	
}
