/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattleStation;
import mot.engine.battles.CraftStateFactory;
import mot.engine.battles.HoloCraft;
import mot.engine.event.BattleCraftTeleportEF;
import mot.engine.event.BattleWeaponFireEF;

public class WeaponInstanceTeleporter extends WeaponInstance {

    public WeaponInstanceTeleporter(TechnologyBundle tb, WeaponBayDetails weaponBayDetails) {
        super(tb, false, false, weaponBayDetails);
    }

    public WeaponInstanceTeleporter(Weapon weapon, WeaponBayDetails weaponBayDetails) {
        super(weapon, 1, false, false, weaponBayDetails);
    }

	/**
     * Fires this Teleport weapon located on the source craft at the target craft.
     * This causes the target craft to be moved to a random location on the galaxy.
     *
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
        boolean imune = isImune(target) || !(target instanceof BattleCraft) || (target instanceof BattleStation);

        Battle battle = source.getBattle();
        Location dest = null;
        
        qtyToFire = updateQtyFired(qtyToFire, true);
        
        if (!imune) {
        	dest = battle.getRandomFreeLocationFor(target);
        	double damage = getDamage();
        	boolean success = Utils.random(damage);
    		if (!success) {
    			dest = null;
    		}
        }
		
        if (Logger.isDebug()) {
            Logger.logDebug("Teleporter.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" dest="+dest);
        }
        
        String damageDescription = null;
        if (imune) damageDescription = weapon.getName() + " " + L.IMMUNE;
        else if (dest == null) damageDescription = getDefaultFailMessage();
        BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, damageDescription, false, source.getCraftState(), target.getCraftState(), null, null);
        MotEngine.instance.getProxyManager().scheduleMulticast(event, source.getBattle().getStartingUserPlayers());

        if (target instanceof BattleCraft && dest != null) {
        	BattleCraft t = (BattleCraft) target;
        	CraftStateFactory originalState = t.getCraftState();
			t.getLocation().shiftTo(dest);
			CraftStateFactory newState = t.getCraftState();
    		BattleCraftTeleportEF event2 = new BattleCraftTeleportEF(t, originalState, newState, 0.0);
            MotEngine.instance.getProxyManager().multicast(event2, source.getBattle().getStartingUserPlayers());
            t.activateIncumbentsAt(dest);
        }
        
        event.setNewTargetState(target.getCraftState());
        MotEngine.instance.getProxyManager().sendScheduledEvent(event);

    }

	public boolean willHaveAnEffectOn(BattleObject target) throws ApplicationError {
		return target != null && target instanceof BattleCraft && !(target instanceof BattleStation) && !(target instanceof HoloCraft);
	}

	public void fire(BattleObject source, Location target, int qtyToFire) {	}
	public boolean canAttackLocationAndHasCharge() { return false; }

}