/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.technology.Weapon;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.Wreckage;
import mot.engine.battles.WreckageCraft;
import mot.engine.battles.Wreckages;
import mot.engine.event.BattleWeaponFireEF;

/**
 * This is a weapon that can be fired at the wreckage of another craft and will resurect 
 * the target craft provided that it was destroyed less than 'damage' turns ago. The 
 * resurected craft only lasts for the duration of this battle (it does not add a new craft
 * to the player, it just adds a battle craft to the battle).  
 * 
 * It has no effect when fired at anything other than debris.
 * 
 * This weapon uses Param1 config to store a number between 0 and 100 that is the chance to
 * change the owner of the resurected craft if the owner is different that the owner of this weapon.
 * A value of zero means it will never change the owner while a value of 100 means it will always 
 * change the owner. If this weapon fails to change the owner to the same owner as this weapon
 * then the weapon fire fails.
 */
public class WeaponInstanceTimeWarpBeam extends WeaponInstance {
	
    private static final String FAILED = " - Failed";
	private static final String NO_EFFECT = " - No Effect";

	public WeaponInstanceTimeWarpBeam(Weapon weapon) {
        super(weapon, 1, false, false, null);
    }

	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
//        discharge();
//        WeaponFireEF event = new WeaponFireEF(source, target, getWeapon(), 0.0, 0.0, false, null, null, null, null);
//        event.setTargetDescription(NO_EFFECT);
//        event.setTargetLocation(target.getLocation());
//        MotEngine.instance.broadcast( event );
	}
	
	private void sendNoEffectWeaponFireEvent(BattleObject source, Location target, String message) throws ApplicationError {
        qtyFired++;
        qtyShots++;
        BattleWeaponFireEF event = new BattleWeaponFireEF(source, null, getWeapon(), 0.0, 0.0, null, null, false, source.getCraftState(), null, null, null);
        event.setTargetDescription(message);
        event.setTargetLocation(target);
        MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
	}
	
	public void fire(BattleObject source, Location target, int qtyToFire) throws ApplicationError { 
		dirty = true;
		
		if (handleEncumbants(source, target, 1)) {
			return;
		}
		
        if (Logger.isDebug()) {
            Logger.logDebug("TimeWarpBeam.fire() "+source.getName()+" fires "+weapon.getName()+" at "+target);
        }
		
        double damage = getDamage();
        Wreckages wreckages = source.getBattle().getWreckages();
        Wreckage wreckage = wreckages.getWreckageAt(target);
        if (!(wreckage instanceof WreckageCraft)) {
        	sendNoEffectWeaponFireEvent(source, target, weapon.getName()+NO_EFFECT);
        	return;
        }
        WreckageCraft craftWreckage = (WreckageCraft) wreckage;
        int turnsSinceDestroyed = source.getBattle().getTurn() - wreckage.getTurnCreated();
        if (turnsSinceDestroyed > damage) {
        	sendNoEffectWeaponFireEvent(source, target, weapon.getName()+NO_EFFECT);
        	return;
        }
        
        BattleCraft targetCraft = craftWreckage.getBattleCraft();
        int chanceToChangeOwner = weapon.getParam1AsInt();
        
        if (targetCraft.getOwner() != source.getOwner()) {
        	if (chanceToChangeOwner == 0) {
            	sendNoEffectWeaponFireEvent(source, target, weapon.getName()+NO_EFFECT);
            	return;
        	}
        	if (chanceToChangeOwner < Utils.random(100)) {
            	sendNoEffectWeaponFireEvent(source, target, weapon.getName()+FAILED);
            	return;
        	}
        }
        
		targetCraft.resurect(source.getOwner());
		targetCraft.dischargeWeapons();

        qtyFired++;
        qtyShots++;
        BattleWeaponFireEF event = new BattleWeaponFireEF(source, null, getWeapon(), 0.0, 0.0, null, null, false, source.getCraftState(), targetCraft.getCraftState(), null, null);
        event.setTargetLocation(target);
        MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
	}

	public boolean canAttackLocationAndHasCharge() { return hasWeaponsToFire(); }
}
