/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.lang.L;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.crafts.Craft;
import mot.engine.event.BattleWeaponFireEF;

public class WeaponInstanceTransport extends WeaponInstance {

    public WeaponInstanceTransport(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, heavy, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceTransport(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }

	/**
     * Fires this transport weapon located on the source craft at the target craft.
     * For every point of damage there is a 1% chance the transport will work. When the transport 
     * works a random piece of equipment is moved from the target craft to the source craft. This
     * transporter can steal;
     *  - Weapons
     *  - Shields
     *  - Specials
     *  
     *  If the source craft's owner has not discovered the stolen technology then the item is throw
     *  away.
     *
     * If the weapon is out of range or the targets shields have power to no action is taken.
     *
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
		if (handleShields(source, target, qtyToFire)) {
			return;
		}
        
        qtyToFire = updateQtyFired(qtyToFire, true);

	    if (!(target instanceof BattleCraft) || !(source instanceof BattleCraft)) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, weapon.getName() + " " + L.IMMUNE, false, source.getCraftState(), null, null, null);
	    	MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
	    	return;
	    }

        double damage = getDamage(source, target);
        if (damage <= 0.0) return;

        CraftStateFactory originalTargetState = target.getCraftState();
        CraftStateFactory originalSourceState = null;

        damage = damage*qtyToFire;

        if (Logger.isDebug()) {
            Logger.logDebug("Transport.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" damage="+damage);
        }

        String targetDescription = getDefaultFailMessage();
        String srcDescription = null;
        BattleCraft targetCraft = (BattleCraft)target;
        if (Utils.random(100) < damage) {
            Technology tech = selectRandomTechnology(targetCraft.getCraft());
            if (tech != null) {
                targetCraft.remove(tech);
                BattleCraft sourceCraft = (BattleCraft) source;
               	sourceCraft.addSalvage(tech);
                originalSourceState = source.getCraftState();
                srcDescription = "+"+tech.getName(); 
                targetDescription = "-"+tech.getName(); 
            }
        }

        BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, srcDescription, targetDescription, false, source.getCraftState(), originalTargetState, null, originalTargetState);
        event.setReverseAnimation(true);
        MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
    }

    private Technology selectRandomTechnology(Craft c) {
        GeneralTechnologyList tl = new GeneralTechnologyList();
        if (c.getWeaponBays() != null) {
            tl.add(c.getWeaponBays().getAllWeapons());
        }
        tl.add(c.getShields());
        tl.add(c.getSpecials());
        int qty = tl.getQuantity();
        return tl.getTechnologyAtIndex(Utils.random(qty));
    }

	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }
}