/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.technology;

import mot.common.bean.Location;
import mot.common.bean.TroopBattleResult;
import mot.common.lang.L;
import mot.common.technology.Shield;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponBayDetails;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleObject;
import mot.engine.battles.CraftStateFactory;
import mot.engine.battles.HoloCraft;
import mot.engine.battles.TroopBattle;
import mot.engine.event.BattleWeaponFireEF;

public class WeaponInstanceTroopTransport extends WeaponInstance {

	public WeaponInstanceTroopTransport(TechnologyBundle tb, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(tb, heavy, infiniteShots, weaponBayDetails);
    }

    public WeaponInstanceTroopTransport(Weapon weapon, int qty, boolean heavy, boolean infiniteShots, WeaponBayDetails weaponBayDetails) {
        super(weapon, qty, heavy, infiniteShots, weaponBayDetails);
    }

    private static final double HEAL_FACTOR = 1.5;

	/**
     * Fires this troop transport weapon located on the source craft at the target craft.
     * The damage of this weapon is the quantity of troops transported from this craft to 
     * the enemy craft. We call this the boarding crew.
     * 
     * The weapon cannot penetrate shields. 
     * 
     * The boarding crew is subtracted from source crafts crew. The crew of the target fight 
     * the boarding crew. until there are none left of one crew (or both). 
     *
     * If after the crew fight the target craft 
     *  - Has crew then the target craft simple has less crew
     *  - Has no crew and all boarding craft are killed the craft is abandoned
     *  - Has no crew and there are still boarding crew then the craft has surrendered
     *
     * @param source The craft firing this weapon
     * @param target The craft this weapon is being fired at.
     * @param qtyToFire The quantity of weapons in this weapon instance to fire.
     * @throws mot.common.util.ApplicationError
	 */
	public void fire(BattleObject source, BattleObject target, int qtyToFire) throws ApplicationError {
		dirty = true;
		
		if (handleEncumbants(source, target, qtyToFire)) {
			return;
		}
		
        ShieldInstance targetShields = target.getShieldsWithHighestPower(Shield.GROUP_SHIELD);

        boolean isHeal = (target.getOwner() == source.getOwner());

		if (!isHeal) {
			if (handleShields(source, target, qtyToFire)) {
				return;
			}
        }
        
        qtyToFire = updateQtyFired(qtyToFire, true);
        
        boolean imune = isImune(target);
        if (imune) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, weapon.getName() + " " + L.IMMUNE, false, source.getCraftState(), null, null, null);
	    	MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
	    	return;
        }
        boolean isHoloCraft = target instanceof HoloCraft;
        if (isHoloCraft) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, L.LOST, true, source.getCraftState(), null, null, null);
	    	MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
	    	target.destroy(source.getOwner(), target.getCraftState(), null);
	    	return;
        }
        
        if (!(target instanceof BattleCraft)) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, weapon.getName() + " " + L.LOST, false, source.getCraftState(), null, null, null);
	    	MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
            return;
        }
        if (!(source instanceof BattleCraft)) {
	    	BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, null, weapon.getName() + " " + L.LOST, false, source.getCraftState(), null, null, null);
	    	MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
            return;
        }
        
        BattleCraft targetCraft = (BattleCraft) target;
        BattleCraft sourceCraft = (BattleCraft) source;

        CraftStateFactory originalTargetState = targetCraft.getCraftState();
        CraftStateFactory originalSourceState = sourceCraft.getCraftState();

        int maxTransportTroops = (int) getDamage();
		if (sourceCraft.getCrew() < 1) {
            return;
        }
		int boardingCrew = Comp.min(maxTransportTroops*qtyToFire, sourceCraft.getCrew()-1);
        if (boardingCrew <= 0) {
            return;
        }

        if (isHeal) {
            targetCraft.setCrew(targetCraft.getCrew()+boardingCrew);
            sourceCraft.setCrew(sourceCraft.getCrew()-boardingCrew);
            String sourceDescription = L.CREW+" -"+boardingCrew+" ("+sourceCraft.getCrew()+")";
            String targetDescription = L.CREW+" +"+boardingCrew+" ("+targetCraft.getCrew()+")";
            BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, sourceDescription, targetDescription, false, originalSourceState, originalTargetState, source.getCraftState(), target.getCraftState());
            MotEngine.instance.multicast( event, source.getBattle().getStartingUserPlayers() );
        } else {
            int defendingCrew = targetCraft.getCrew();
    
            TroopBattle tb = new TroopBattle(source.getOwner(), target.getOwner(), boardingCrew, defendingCrew, -2.0);
            TroopBattleResult tbr = tb.execute();
            int remainingDefenders = tb.getRemainingDefenders(); 
            int remainingAttackers = tb.getRemainingAttackers();
    
            if (Logger.isDebug()) {
                Logger.logDebug("TroopTransport.fire() "+source.getName()+" fires "+weapon.getName()+"X"+qtyToFire+" at "+target.getName()+" boardingCrew="+boardingCrew+" remainingDefenders="+remainingDefenders+" remainingAttackers="+remainingAttackers);
            }
    
            sourceCraft.setCrew(sourceCraft.getCrew()-boardingCrew);

            String sourceDescription = L.CREW+" -"+boardingCrew;
            String targetDescription;
            if (remainingDefenders > 0) {
                targetDescription = getDefaultFailMessage();
                targetCraft.setCrew(remainingDefenders);
            } else if (remainingDefenders <= 0 && remainingAttackers > 0) {
                targetDescription = L.SUCCESS;
                targetCraft.setCrew(remainingAttackers);
            } else {
                targetDescription = L.INOPERABLE_NO_CREW;
                targetCraft.setCrew(0);
            }
            
            if (targetCraft.getCrew() == 0) {
                targetCraft.setMoveRemaining(0);
            }
    
            BattleWeaponFireEF event = new BattleWeaponFireEF(source, target, getWeapon(), 0.0, 0.0, sourceDescription, targetDescription, false, originalSourceState, originalTargetState, source.getCraftState(), null);
            event.setData(tbr);
            MotEngine.instance.getProxyManager().scheduleMulticast(event, source.getBattle().getStartingUserPlayers());
    
            if (remainingAttackers > 0) {
                targetCraft.surrender(source.getOwner(), false);
            }
            
            event.setNewTargetState(target.getCraftState());
            MotEngine.instance.getProxyManager().sendScheduledEvent(event);
        }
    }

	public void fire(BattleObject source, Location target, int qtyToFire) { }
	public boolean canAttackLocationAndHasCharge() { return false; }
	
	public boolean canMiss() { return false; }
	public boolean usesWeaponBayFireAngle() { return false;	}
	
}