/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.transports;

import mot.common.bean.Size;
import mot.common.designs.TransportDesign;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.MoveableOwnedObject;
import mot.engine.common.MoveableOwnedObjectReference;
import mot.engine.common.Orbit;
import mot.engine.crafts.Navigator;
import mot.engine.players.Player;

/**
 * This is an instance of a transport built by a player and assigned to
 * a task (or not assigned if task is null). A transport is in use if it has been 
 * assigned a task otherwise it is idle. Only transports that are in use are visible 
 * on the galaxy map the other transports are in an available pool.
 * 
 * Each player has an instance of Transport manager that contains all the transports 
 * for that player. 
 * 
 * Transports are used to mine planets, transfer colonists, deploy modules etc.
 */
public class Transport extends MoveableOwnedObject {
    /**
     * The task assigned to this transport or null if no task has 
     * been assigned.
     */
    private TransportTask task;
    
    /**
     * This is the type of transport this object is.
     * CURRENTLY THERE IS ONLY ONE DESIGN - THEREFORE THIS IS STATIC - THIS MAY CHANGE AT SOME POINT!
     */
    private static TransportDesign design;
    
    public Transport() {
    }

    public Transport(Player owner) {
        super(MotEngine.instance.getNameGenerator().getNextName("Transport"), owner, owner.getEngines().getLongestRangeEngine(), null);
    }

    /**
     * Moves this transport - The task performed depends on the task this transport 
     * has been assigned to. This method does not do anything its self except call
     * move() on the task.
     * This should be called at the end of every turn.
     * @throws mot.common.util.ApplicationError
     */
    public void endTurn() throws ApplicationError {
        Navigator navigator = getNavigator();
        if (navigator != null) {
            navigator.move();
        } else if (task != null) {
            task.idle(this);
        }
    }

    /**
     * Called by the navigator when this MOO has arrived at its destination. This will
     * usually complete the transports task!
     * @param stellarObject The object this MOO has arrived in.
     */
	public void arrivedAt(Orbit orbit) throws ApplicationError {
        super.arrivedAt(orbit);
        if (task != null) {
            task.arrivedAt(orbit, this);
        }
    }
    
    /**
     * Removes this transport from the game. Removes this transport from the list 
     * of transports owned by this transports owner.
     */
    public void destroy() throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("Transport.destroy() name="+getName());
        }
        TransportManager tm = getOwner().getTransportManager();
        tm.removeTransport(this);
        setTask(null);
    }
    
    /**
     * A transport craft is visible when it has orders and is not displayed when 
     * it is idle.
     * @return True iff this transport should be displayed on the galaxy map. 
     */
    public boolean displayOnGalaxyMap() {
        return task != null;
    }
    
	protected void encode(XMLWriter writer) throws ApplicationError {
        super.encode(writer);
        if (task != null) {
            task.toXML(writer);
        }
	}

	protected String getElementName() { return "transport"; }

	/**
	 * @return true if this craft can be seen by this player at this point in time.
	 */
	public boolean isVisible(Player player) throws ApplicationError {
		return (player == getOwner()) || (getVisibility(player) > 0) || (getOrbit() != null && getOrbit().isOwnedBy(player));
	}
    
	/**
	 * @return the visibility of this craft by player at this point in time.
     * The visibility of a craft is determined by the closest scanner strength 
     * (on a planet or a craft) and the shields cloak value of this craft. The
     * player cloak bonus also effects a crafts visibility.
	 */
	public int getVisibility(Player player) throws ApplicationError {
		if (player == null) return 100;
		int playerSight = player.getScannerStrengthAt(getLocation());
		int cloakLevel = getOwner().getOriginCloakLevel() + getOwner().getOwnerCloakLevel();
		return (playerSight - cloakLevel);
	}

    /**
     * @return The visibility of this transport 
     */
    public int getVisibility() throws ApplicationError {
		int playerSight = getOwner().getScannerStrengthAt(getLocation());
        return playerSight;
    }

    /**
     * @return The space available on this transport.
     */
    public int getSpaceAvailable() {
    	if (task != null && task instanceof TransportTaskInvade) {
    		TransportTaskInvade tti = (TransportTaskInvade) task;
    		return TransportDesign.getSpace() - tti.getQtyAttackers();
    	} else if (task != null && task instanceof TransportTaskMoveColonists) {
    		TransportTaskMoveColonists ttmc = (TransportTaskMoveColonists) task;
    		return TransportDesign.getSpace() - ttmc.getQty();
    	} else {
            return TransportDesign.getSpace(); 
    	}
    }

    public TransportDesign getDesign() {
        if (design == null) {
            design = new TransportDesign();
        }
        return design;
    }

    public TransportTask getTask() {
        return task;
    }

    public void setTask(TransportTask task) {
        this.task = task;
    }
    
    public void setTransportTaskInvade(TransportTaskInvade task) {
        this.task = task;
    }

    public void setTransportTaskMoveColonists(TransportTaskMoveColonists task) {
        this.task = task;
    }

    public void setTransportTaskDeployStation(TransportTaskDeployStation task) {
        this.task = task;
    }

    public void setTransportTaskCreateTradeRoute(TransportTaskCreateTradeRoute task) {
        this.task = task;
    }

    public MoveableOwnedObjectReference createNewReference() throws ApplicationError {
        return new TransportReference(this);
    }
    

    private static Size size = new Size(50,50);
    public Size getImageSize() throws ApplicationError {
        return size;
    }

}