/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.transports;

import java.util.ArrayList;

import mot.common.technology.Engine;
import mot.common.technology.Hull;
import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.common.xml.XMLWriter;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Navigator;
import mot.engine.planets.Planet;
import mot.engine.players.Player;

/**
 * A TransportManager is like a Commander except moves transports.
 * A transport is a concept in the game is like a craft and are used to transport
 * colonists from one planet to another etc, however, unlike a craft they are not
 * assigned locations and do not move around the galaxy. A transport is assigned
 * a task and it is up to the task implementation how its actually uses a transport.
 *
 * Free transports are those with no Task Defined for them.
 *
 * Note A transport manager only manages transports for a single player. Each player
 * must have its own transport manager.
 */
public class TransportManager extends XMLBean {

    /**
     * This is the list of transports owned by this player.
     * A transport is in use if it has a task, otherwise it is idle.
     */
    private ArrayList transports;
    
    /**
     * This is a dynamically generated array of transports.
     */
    private transient Transport [] transportsArray;
    
    public Transport [] getTransportsAsArray() {
        if (transportsArray == null && transports != null && transports.size() > 0) {
            transportsArray = (Transport []) transports.toArray(new Transport[transports.size()]);
        }
        return transportsArray;
    }

    public ArrayList getTransports() {
        return transports;
    }

    public void setTransports(ArrayList transports) {
        this.transports = transports;
    }

    public void setTransport(Transport transport) {
        if (transports == null) transports = new ArrayList();
        transports.add(transport);
        transportsArray = null;
    }
    
    /**
     * Moves all transport managed by this transport manager.
     * This should be called at the end of every turn.
     * @throws mot.common.util.ApplicationError
     */
    public void endTurn() throws ApplicationError {
        Transport [] ta = getTransportsAsArray();
        if (ta != null) {
            for (int i=0; i<ta.length; i++) {
                ta[i].endTurn();
            }
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        Transport [] ta = getTransportsAsArray();
        if (ta != null) {
            for (int i=0; i<ta.length; i++) {
                ta[i].toXML(writer);
            }
        }
	}

	protected String getElementName() { return "transport-manager"; }

    /**
     * @return The transport with name or null. 
     */
    public Transport getTransport(String name) {
        Transport [] ta = getTransportsAsArray();
        if (ta != null) {
            for (int i=0; i<ta.length; i++) {
                if (ta[i].sameName(name)) {
                    return ta[i];
                }
            }
        }
        return null;
    }
    
    /**
     * Removes transport from the list of transports in the manager.
     * @param transport The transport to remove
     */
    public void removeTransport(Transport transport) {
        transports.remove(transport);
        transportsArray = null;
    }

    private boolean isLoadingColonistsAt(Transport transport, Planet origin, boolean troops) throws ApplicationError {
    	if (troops) {
            if (transport.getTask() != null && transport.getTask() instanceof TransportTaskInvade) {
                Orbit orbit = transport.getOrbit();
    			if (orbit != null && orbit.getPlanet() == origin && transport.getNavigator() == null) {
    				return true;
                }
            }
    	} else {
            if (transport.getTask() != null && transport.getTask() instanceof TransportTaskMoveColonists) {
                Orbit orbit = transport.getOrbit();
    			if (orbit != null && orbit.getPlanet() == origin && transport.getNavigator() == null) {
    				return true;
                }
            }
    	}
        return false;
    }

    /**
     * @param planet The planet colonists are enroute to
     * @return The total colonists on route to this planet from all other systems.
     * @throws ApplicationError 
     */
    public Transport getMoveColonistsTransportWithSpareCapacityAt(Planet planet, boolean troops) throws ApplicationError {
        Transport [] transports = getTransportsAsArray();
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (isLoadingColonistsAt(transports[i], planet, troops)) {
                    int transportCapacity = transports[i].getSpaceAvailable();
					if (transportCapacity > 0) {
                        return transports[i];
                    }
                }
            }
        }
        return null;
    }
    
    private boolean isMovingColonistsTo(Transport transport, Planet destination) throws ApplicationError {
        if (transport.getTask() != null && transport.getTask() instanceof TransportTaskMoveColonists) {
            Navigator navigator = transport.getNavigator();
			if (navigator != null && navigator.getDestination() != null && navigator.getDestination().getPlanet() == destination) {
				return true;
            }
        }
        return false;
    }

    private boolean isMovingTroopsTo(Transport transport, StellarObject orbit) throws ApplicationError {
        if (transport.getTask() != null && transport.getTask() instanceof TransportTaskInvade) {
            Navigator navigator = transport.getNavigator();
			if (navigator != null && navigator.getDestination() != null && navigator.getDestination().getStellarObject() == orbit) {
				return true;
            }
        }
        return false;
    }

    private boolean isMovingTroopsTo(Transport transport, Planet planet) throws ApplicationError {
        if (transport.getTask() != null && transport.getTask() instanceof TransportTaskInvade) {
            Navigator navigator = transport.getNavigator();
			if (navigator != null && navigator.getDestination() != null && navigator.getDestination().getPlanet() == planet) {
				return true;
            }
        }
        return false;
    }

    /**
     * @param planet The planet colonists are enroute to
     * @return The total colonists on route to this planet from all other systems.
     * @throws ApplicationError 
     */
    public int getQtyColonistsOnrouteTo(Planet planet) throws ApplicationError {
        Transport [] transports = getTransportsAsArray();
        int qty = 0;
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (isMovingColonistsTo(transports[i], planet)) {
                	TransportTaskMoveColonists ttmc = (TransportTaskMoveColonists) transports[i].getTask();
                    qty += ttmc.getQty();
                }
            }
        }
        return qty;
    }
    
    /**
     * @return the quantity of troops owned by this player and enroute to this solar system 
     * or already at this solar system.
     */
    public int getQtyTroopsWithDestination(StellarObject orbit) throws ApplicationError {
        Transport [] transports = getTransportsAsArray();
        int qty = 0;
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (isMovingTroopsTo(transports[i], orbit)) {
                	TransportTaskInvade ttmt = (TransportTaskInvade) transports[i].getTask();
                    qty += ttmt.getQtyAttackers();
                }
            }
        }
        return qty;
    }

    public int getQtyTroopsWithDestination(Planet planet) throws ApplicationError {
        Transport [] transports = getTransportsAsArray();
        int qty = 0;
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (isMovingTroopsTo(transports[i], planet)) {
                	TransportTaskInvade ttmt = (TransportTaskInvade) transports[i].getTask();
                    qty += ttmt.getQtyAttackers();
                }
            }
        }
        return qty;
    }


    public int getQtySpaceStationsWithDestination(StellarObject so) throws ApplicationError {
        Transport [] transports = getTransportsAsArray();
        int qty = 0;
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (transports[i].getTask() instanceof TransportTaskDeployStation) {
                	TransportTaskDeployStation ttds = (TransportTaskDeployStation) transports[i].getTask();
                    if (ttds.getDestination() != null) {
                    	StellarObject target = ttds.getDestination().getStellarObject();
                    	if (target == so) {
                    		qty++;
                    	}
                    }
                }
            }
        }
        return qty;
    }


    /**
     * @param planet The planet colonists are enroute to
     * @return The total colonists on route to this planet from all other systems.
     */
    public int getColonistsEtaOnrouteTo(Planet planet) throws ApplicationError {
        Transport [] transports = getTransportsAsArray();
        int eta = 0;
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (isMovingColonistsTo(transports[i], planet)) {
                    Navigator navigator = transports[i].getNavigator();
                    int newETA = navigator.getETA(); 
                    if (newETA > eta) {
                        eta = newETA;
                    }
                }
            }
        }
        return eta;
    }
    
    /**
     * @return A transport that has no task or null if all transports are in use.
     */
    public Transport [] getTransportsInUse() {
        Transport [] transports = getTransportsAsArray();
        ArrayList<Transport> r = null;
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (transports[i].getTask() != null) {
                	if (r == null) r = new ArrayList();
                    r.add(transports[i]);
                }
            }
        }
        if (r != null) {
        	return (Transport[]) r.toArray(new Transport[r.size()]);
        } else {
        	return null;
        }
    }
    
    /**
     * @return A transport that has no task or null if all transports are in use.
     */
    public Transport getFreeTransport() {
        Transport [] transports = getTransportsAsArray();
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (transports[i].getTask() == null) {
                    return transports[i];
                }
            }
        }
        return null;
    }

    public Transport getFreeTransportOrCreateTransport(Player owner) {
    	Transport t = getFreeTransport();
    	if (t == null) {
    		t = new Transport(owner);
			setTransport(t);
    	}
    	return t;
    }
    
    /**
     * @return The quantity of transports.
     */
    public int getQtyTransports() {
        if (transports != null) {
            return transports.size();
        } else {
            return 0;
        }
    }

    /**
     * @return The quantity of transports that have no task.
     */
    public int getQtyFreeTransport() {
        Transport [] transports = getTransportsAsArray();
        int qty =0;        
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (transports[i].getTask() == null) {
                    qty++;
                }
            }
        }
        return qty;
    }
    
    /**
     * The user has discovered a new engine - update our transports to use this new engine.
     */
    public void updateEngines(Engine v) {
        Transport [] transports = getTransportsAsArray();
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (transports[i].getEngine().getMaxTravelDistance() < v.getMaxTravelDistance()) {
                    transports[i].setEngine(v);
                }
            }
        }
    }

	public int getQtyTransportsWithOutpost() throws ApplicationError {
        Transport [] transports = getTransportsAsArray();
        int qty = 0;
        if (transports != null) {
            for (int i=0; i<transports.length; i++) {
                if (transports[i].getTask() != null && transports[i].getTask() instanceof TransportTaskDeployStation) {
                	TransportTaskDeployStation tt = (TransportTaskDeployStation) transports[i].getTask() ;
                	if (tt.getOrbital() != null && tt.getOrbital().getHullTech() != null) {
                		Hull hull = tt.getOrbital().getHullTech();
                		if (hull.isOutpost()) {
                			qty++;
                		}
                	}
                }
            }
        }
		return qty;
	}
}