/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.transports;

import mot.common.command_results.StateUpdateCommandResult;
import mot.common.util.ApplicationError;
import mot.common.xml.XMLBean;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.planets.Planet;

/**
 * Base class for all tasks a transport can be assigned to.
 */
public abstract class TransportTask extends XMLBean  {

    /**
     * This is false when a new transport task is created. However, if the owner of this
     * task is sent a message to inform him of the transports existence on the galaxy map 
     * carrying this cargo, then this should be set to true. This will allows us to stop
     * sending the same message more than once.
     */
    private boolean userInformedOfExistence = false;

    public TransportTask() {
    }

    /**
     * This method is called when this transport arrives at its destination.
     * @param orbit The stellar object / planet this craft has arrived at.
     */
	public abstract void arrivedAt(Orbit orbit, Transport transport) throws ApplicationError;

    /**
     * This method is called when A Transport has not got a navigator but does has a task.
     * This means that a transport has been assigned a task but is not going anywhere - it is
     * idle. This transport would then represent a waisted resource!
     * NOTE - Than can occur when a Space Station or outpost is built and there is so planet or
     * stellar object to deploy it to.
     * 
     * This implementation does not do anything - This method should be overridden by tasks that
     * want to take some action.
     * 
     * @throws mot.common.util.ApplicationError
     * @param transport This is the transport that is idle
     */
    public void idle(Transport transport) throws ApplicationError {
    	if (transport == null) return;
    	if (transport.getOrbit() != null && transport.getOwner().isComputerPlayer()) {
    		arrivedAt(transport.getOrbit(), transport);
    	}
    }

    /**
     * @return True if this task requires a destination planet to be deployed.
     */
	public abstract boolean canDeployTo(StellarObject so) throws ApplicationError;

    /**
     * Deploy this transports pay load at the destination specified.
     * @param transport The transport with this task that is to be deployed
     * @param destination If the destination of this task. Value must be null if 
     * requirePlanetToDeploy() returns false. Value must not be null if  requirePlanetToDeploy() 
     * returns true
     */
	public abstract StateUpdateCommandResult deploy(Transport transport, StellarObject destStellarObject, Planet destPlanet) throws ApplicationError;

    public abstract String getDescription() throws ApplicationError ;

    public boolean isUserInformedOfExistence() {
        return userInformedOfExistence;
    }

    public void setUserInformedOfExistence(boolean userInformedOfExistence) {
        this.userInformedOfExistence = userInformedOfExistence;
    }


}