/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.transports;

import mot.common.bean.MessageType;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.RemoveMooVO;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.event.MessageEF;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.SolarSystemReference;
import mot.engine.planets.TradeRouteFactory;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

/**
 * This is a task that may be assigned to a transport ship - It moves a space station
 * from the planet that build it to the planet it will be in orbit around.
 *
 * When this task is complete it sets the owner of the destination planet to the owner of
 * the transport and sets a link from the owning planet to the space station.
 */
public class TransportTaskCreateTradeRoute extends TransportTask {

    /**
     * This is the destination planet for the trade route.
     */
    private SolarSystemReference destination;

    /**
     * This is the planet that build the trade route and will become the origin of 
     * the trade route once it is created.
     */
    private SolarSystemReference origin;
    
    public TransportTaskCreateTradeRoute() {
    }

    public TransportTaskCreateTradeRoute(Transport transport, SolarSystem origin, SolarSystem target) throws ApplicationError {
        this.origin = new SolarSystemReference(origin);
        this.destination = target == null ? null : new SolarSystemReference(target);

        transport.transferTo(origin.getOrbit());
        transport.setTask(this);

        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskCreateTradeRoute.init() at "+origin+" dest="+destination);
        }
    }

    public String getDescription() throws ApplicationError {
    	if (destination == null) {
    		if (origin != null) {
    			return L.CREATE_A_TRADE_ROUTE+L._FOR_ + origin.getName();
    		} else {
    			return L.CREATE_A_TRADE_ROUTE;
    		}
    	} else {
    		if (origin != null) {
    			return L.CREATE_A_TRADE_ROUTE + L._FOR_ + origin.getName() + L._TO_ + destination;
    		} else {
        		return L.CREATE_A_TRADE_ROUTE + L._TO_ + destination;
    		}
    	}
    }

    /**
     * Removes this task from the transport and deploys the Orbital at the stellar object.
     * This method is called when this transport arrives at its destination.
     */
	public void arrivedAt(Orbit orbit, Transport transport) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskCreateTradeRoute.arrivedAt() At "+orbit);
        }

        if (orbit == null) {
        	Logger.logWarning("TransportTaskCreateTradeRoute.arivedAt() Transport "+transport+" Has null orbit!!");
        	if (destination != null) {
        		transport.moveTo(new Orbit(destination.getSolarSystem()));
        	}
        	return;
        }
        
        StellarObject so = orbit.getStellarObject();
        if (so == null) {
        	Logger.logWarning("TransportTaskCreateTradeRoute.arivedAt() Transport "+transport+" orbit has null SS");
        	return;
        }
        if (!(so instanceof SolarSystem)) {
        	Logger.logWarning("TransportTaskCreateTradeRoute.arivedAt() Transport "+transport+" orbit is not a SS");
        	return;
        }
        SolarSystem ss= (SolarSystem) so;
        
		Player owner = transport.getOwner();
		if (owner.getQtyMilitaryCraftsAt(ss) == 0 && owner.getQtyEnemyCraftAt(ss) > 0) {
	        transport.setTask(null);
			if (!owner.isComputerPlayer()) {
        		MessageEF event = new MessageEF(L.TRANSPORT_LOST_AT_+orbit.getShortDescription(), MessageType.flashing, null);
        		MotEngine.instance.broadcast( event, owner );        	
			}
			return;
		}

		if (!ss.isOwnedBy(owner)) {
			if (owner instanceof ComputerPlayer) {
		        transport.setTask(null);
			}
			return;
        }

		if (TradeRouteFactory.canCreateTradeRoute(origin.getSolarSystem(), ss)) {
			transport.setTask(null);

			TradeRouteFactory.create(origin.getSolarSystem(), ss);

			MessageEF msg = new MessageEF("Transport established trade route between "+origin.getName()+" and "+ss.getName(), MessageType.plain, null);
			MotEngine.getInstance().broadcast(msg, owner);
		}
    }

	public boolean canDeployTo(StellarObject so) throws ApplicationError {
        return true;
    }

	public StateUpdateCommandResult deploy(Transport transport, StellarObject destStellarObject, Planet destPlanet) throws ApplicationError {
		TradeRouteFactory.create(origin.getSolarSystem(), (SolarSystem)destStellarObject);
		StateUpdateCommandResult cr = new StateUpdateCommandResult(new RemoveMooVO(transport.getName()));
		if (destStellarObject != null && destStellarObject instanceof SolarSystem) {
			SolarSystem destination = (SolarSystem) destStellarObject;
			Player owner = transport.getOwner();
			cr.addStateUpdate(EngineVoFactory.getTradeRoutes(MotEngine.instance.getTradeRoutes(), owner));
			cr.addStateUpdate(EngineVoFactory.getPlanets(destination.getPlanets(), owner));
			cr.addStateUpdate(EngineVoFactory.getStellarObject(destination, owner));
			cr.addStateUpdate(EngineVoFactory.getPlanets(origin.getSolarSystem().getPlanets(), owner));
			cr.addStateUpdate(EngineVoFactory.getStellarObject(origin.getSolarSystem(), owner));
		}
    	cr.addStateUpdate(EngineVoFactory.getPlayer(transport.getOwner()));
		return cr;
   }
	
	protected void encode(XMLWriter writer) throws ApplicationError {
        if (origin != null) {
            origin.toXML(writer, "origin");
        }
        if (destination != null) {
        	destination.toXML(writer, "destination");
        }
	}

	protected String getElementName() { return "transport-task-create-trade-route"; }

    public void setOrigin(SolarSystemReference origin) {
        this.origin = origin;
    }


    public SolarSystemReference getOrigin() {
        return origin;
    }

	public String getOriginName() {
		if (origin != null) {
			return origin.getName();
		} else {
			return null;
		}
	}

    public void setDestination(SolarSystemReference destination) {
        this.destination = destination;
    }

    public void setDestination(SolarSystem destination) {
        this.destination = new SolarSystemReference(destination);
    }

    public SolarSystemReference getDestination() {
        return destination;
    }


}