/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.transports;

import mot.common.command_results.StateUpdateCommandResult;
import mot.common.designs.Orbital;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.RemoveMooVO;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.adapter.TransportVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.common.StellarObjectReference;
import mot.engine.event.MessageEF;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.PlanetReference;
import mot.engine.planets.SolarSystemReference;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

/**
 * This is a task that may be assigned to a transport ship - It moves a space station
 * from the planet that build it to the planet it will be in orbit around.
 *
 * When this task is complete it sets the owner of the destination planet to the owner of
 * the transport and sets a link from the owning planet to the space station.
 */
public class TransportTaskDeployStation extends TransportTask {


	/**
     * The station to deploy.
     */
    private Orbital orbital;

    /**
     * This is the destination planet for the space station.
     */
    private Orbit destination;

    /**
     * This is the planet that build the space station and will become the owner of 
     * the space station once it is deployed.
     */
    private PlanetReference origin;
    
    /**
     * The quantity of time idle has been called on this task.
     */
    private transient int idleCount = 0;

    public TransportTaskDeployStation() {
    }

    public TransportTaskDeployStation(Transport transport, Planet origin, Orbit target, Orbital orbital) throws ApplicationError {
        this(transport, origin, target, orbital, true);
    }
    
    public TransportTaskDeployStation(Transport transport, Planet origin, Orbit target, Orbital orbital, boolean dispatchTransport) throws ApplicationError {
        this.origin = new PlanetReference(origin);
        this.orbital = orbital;
        this.destination = target;

        transport.transferTo(origin.getOrbit());
        transport.setTask(this);

        if (destination != null) {
        	Player owner = transport.getOwner();
        	if (destination.getStellarObject().hasOwner() && !destination.getStellarObject().isOwnedBy(owner)) {
        		destination = null;
        		changeDestination(transport);
        	} else if (destination.getPlanet() != null && destination.getPlanet().getOwner() != null) {
    			destination = null;
    			changeDestination(transport);
        	}
        }

        if (dispatchTransport) {
        	dispatchToDestination(transport);
        }
        
        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskDeployStation.init() at "+origin.getName()+" dest="+destination);
        }
    }

    /**
     * Another player has taken the planet or stellar object we wanted to claim before we could
     * reach it. We will attempt to find another target and if none exists we will delete the
     * transport for computer players.
     * @throws mot.common.util.ApplicationError
     * @param transport
     */
    private void changeDestination(Transport transport) throws ApplicationError {
        if (destination == null && transport.getOwner().isComputerPlayer()) {
        	ComputerPlayer owner = (ComputerPlayer) transport.getOwner();
        	BlackHole blackHole = owner.getMayor().getClosestBlackHoleWouldUse();
        	if (blackHole != null) {
        		destination = new Orbit(blackHole);
        	} else {        		
        		Planet planet = owner.getMayor().getClosestResourceWouldMine(true, true);
        		if (planet != null) {
        			destination = new Orbit(planet);
        		} else {
        			transport.setTask(null);
        		}
        	}
        }
    }
    
    private void dispatchToDestination(Transport transport) throws ApplicationError {
        if (orbital.getHullTech().isOutpost()) {
        	if (destination == null) {
        		StellarObject so = transport.getOwner().getStellarObjectToDeployOutpost(transport, orbital);
				destination = (so == null) ? null : new Orbit(so);
        	}
            if (destination != null) {
                transport.moveTo(destination);
            }
        } else {
        	if (destination == null) {
        		Planet planet = transport.getOwner().getPlanetToDeploySpaceStation(transport, orbital);
        		destination = (planet == null) ? null : new Orbit(planet);
        	}
            if (destination != null) {
                transport.moveTo(destination);
            }
        }
    }

    public String getDescription() throws ApplicationError {
    	if (destination == null) {
    		if (origin != null) {
    			return L.DEPLOY_STATION+orbital.getHullTech().getName() + L._FOR_ + origin.getName();
    		} else {
    			return L.DEPLOY_STATION+orbital.getHullTech().getName();
    		}
    	} else {
    		if (origin != null) {
    			return L.DEPLOY_STATION+orbital.getHullTech().getName() + L._FOR_ + origin.getName() + L._TO_ + destination;
    		} else {
        		return L.DEPLOY_STATION+orbital.getHullTech().getName() + L._TO_ + destination;
    		}
    	}
    }

    /**
     * Removes this task from the transport and deploys the Orbital at the stellar object.
     * This method is called when this transport arrives at its destination.
     */
	public void arrivedAt(Orbit orbit, Transport transport) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskDeployStation.arrivedAt() At "+orbit);
        }

        StellarObject stellarObject = orbit.getStellarObject();
        
        Player owner = transport.getOwner();
        int qtyEnemyCraft = transport.getOwner().getQtyEnemyCraftAt(stellarObject);
        if (qtyEnemyCraft > 0) {
			int qtyMyMilitiaryCraft = owner.getQtyMilitaryCraftsAt(stellarObject);
            if (qtyMyMilitiaryCraft == 0) {
                MessageEF event = new MessageEF(L.YOUR_ENEMY_HAS_DESTROYED_A_TRANSPORT_AS_IT_ARRIVED_AT+stellarObject.getName());
                MotEngine.instance.broadcast(event, transport.getOwner());
                transport.destroy();
            }
            return;
        }
        
        if (destination == null) {
        	// Must be a user player - leave deployment to the user!
        	return;
        } else {
	        if (destination.getStellarObject() != stellarObject) {
	        	transport.moveTo(destination);
	        	return;
	        }
	        if (destination.getStellarObject() == null) {
	        	destination = null;
                changeDestination(transport);
	        	return;
            }
	        if (destination.getStellarObject().hasOwner() && !destination.getStellarObject().isOwnedBy(owner)) {
	        	destination = null;
                changeDestination(transport);
	        	return;
            }
	        if (orbital.getHullTech().isOutpost()) {
	        	if (destination.getStellarObject().hasOwner()) {
	                changeDestination(transport);
	                return;
	            } 
	        } else {
	            if (destination.getPlanet() != null && destination.getPlanet().getOwner() != null) {
	            	destination = null;
	                changeDestination(transport);
	                return;
	            }
	        }
	        deploy(transport, destination.getStellarObject(), destination.getPlanet());
        }
    }

	public boolean canDeployTo(StellarObject so) throws ApplicationError {
        return  ((orbital.getHullTech().isOutpost()  && (so.getOrbital() == null)) || (so instanceof BlackHole && !orbital.getHullTech().isOutpost()));
    }

	private void _deploy(Transport transport, StellarObject destStellarObject, Planet destPlanet) throws ApplicationError {
        if (destination != null && destination.getStellarObject() != destStellarObject && destStellarObject != transport.getOrbit().getStellarObject()) {
        	setDestination(new Orbit(destStellarObject, destPlanet));
            transport.moveTo(destination);
        } else if (destStellarObject != transport.getOrbit().getStellarObject()) {
            transport.moveTo(destination);
        } else {
	        if (orbital.getHullTech().isOutpost()) {
	            transport.setTask(null);
	            destStellarObject.deploy(orbital, transport.getOwner());
                orbital.addOutpostModules(transport.getOwner());
	        } else {
	            transport.setTask(null);
	            if (destPlanet == null) {
	                transport.setTask(null);
	                destStellarObject.deploy(orbital, transport.getOwner());
            		orbital.addCoreModule();
	                orbital.addSpaceStationModules(transport.getOwner());
	            } else {
	            	if (origin.getPlanet().getMiningStation() != null) {
	            		MotEngine.instance.broadcast(new MessageEF("Origin planet already has a mining station"), transport.getOwner());
	            	} else {
	            		destPlanet.deploy(origin.getPlanet(), orbital);
	            		orbital.addCoreModule();
	                    orbital.addSpaceStationModules(transport.getOwner());
	            	}
	            }
	        }
        }
	}
	
	public StateUpdateCommandResult deploy(Transport transport, StellarObject destStellarObject, Planet destPlanet) throws ApplicationError {
		
		_deploy(transport, destStellarObject, destPlanet);
		
		if (transport.getTask() == null) {
			StateUpdateCommandResult cr = new StateUpdateCommandResult(new RemoveMooVO(transport.getName()));
			if (destStellarObject != null) {
				cr.addStateUpdate(EngineVoFactory.getStellarObject(destStellarObject, transport.getOwner()));
			}
			Player owner = transport.getOwner();
			cr.addStateUpdate(EngineVoFactory.getPlanets(owner.getPlanets(), owner));
        	cr.addStateUpdate(EngineVoFactory.getPlayer(owner));
			return cr;
		} else {
			return new StateUpdateCommandResult(new TransportVOAdapter(transport));
		}
    }
	
	public boolean isOutpost() throws ApplicationError {
		if (orbital != null && orbital.getHull() != null) {
			return orbital.getHullTech().isOutpost();
		} else {
			return false;
		}
	}

    /**
     * This is the maximum quantity of times idle will be called before the task is deleted. 
     */
    private static final int MAX_IDLE = 5;

    /**
     * Attempt to find a destination for the task and stop it being idle.
     * Deleted the task (and its payload) once it has been called MAX_IDLE.
     */
    public void idle(Transport transport) throws ApplicationError {
        idleCount++;
        Player owner = transport.getOwner();
		if (idleCount == MAX_IDLE && owner != null && owner.isComputerPlayer()) {
            transport.setTask(null);
        } else {
        	dispatchToDestination(transport);
        }
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        orbital.toXML(writer, "orbital");
        if (origin != null) {
            origin.toXML(writer, "origin");
        }
        if (destination != null) {
        	destination.toXML(writer, "destination");
        }
	}

	protected String getElementName() { return "transport-task-deploy-station"; }


    public void setOrbital(Orbital orbital) {
        this.orbital = orbital;
    }


    public Orbital getOrbital() {
        return orbital;
    }

	public String getOrbitalHullName() {
		if (orbital != null) {
			return orbital.getHull();
		} else {
			return null;
		}
	}

    public void setOrigin(PlanetReference origin) {
        this.origin = origin;
    }

    public void setOrigin(Planet origin) {
        this.origin = new PlanetReference(origin);
    }

    public PlanetReference getOrigin() {
        return origin;
    }

	public String getOriginName() {
		if (origin != null) {
			return origin.getName();
		} else {
			return null;
		}
	}

    public void setDestination(Orbit destination) {
        this.destination = destination;
    }
    
    public void setDestination(StellarObjectReference destination) {
        this.destination = new Orbit(destination.getStellarObject());
    }

    public void setDestination(SolarSystemReference destination) {
        this.destination = new Orbit(destination.getSolarSystem());
    }

    public Orbit getDestination() {
        return destination;
    }


    public void setIdleCount(int idleCount) {
        this.idleCount = idleCount;
    }


    public int getIdleCount() {
        return idleCount;
    }


}