/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.transports;
import mot.common.bean.MessageType;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.RemoveMooVO;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.adapter.SolarSystemVOAdapter;
import mot.engine.adapter.TransportVOAdapter;
import mot.engine.battles.TroopBattle;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.event.MessageEF;
import mot.engine.event.PlanetConqueredEF;
import mot.engine.event.PlanetInvadeEF;
import mot.engine.planets.CrisisEmergency;
import mot.engine.planets.OrbitalReference;
import mot.engine.planets.Planet;
import mot.engine.planets.PlanetReference;
import mot.engine.planets.TradeRoute;
import mot.engine.planets.TradeRoutes;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.PlayerReference;
import mot.engine.players.UserPlayer;

/**
 * This is a task that may be assigned to a transport ship - It moves colonists 
 * from one troops from its owner's planet to an enemies planet ready for an invasion.
 * 
 * When this task is constructed it removes the qty colonists from the source planet
 * and places them into the transport. It calculates the time it takes (using the best 
 * engine the player has available) to move them from the source to the destination. 
 * After that many turn this task deleted its self and adds the colonists to the destination
 * planet.
 */
public class TransportTaskInvade extends TransportTask {


	/**
     * This is the quantity of colonists being transfered.
     */
    private int qtyAttackers;
    
    /**
     * This is the destination planet for the colonists.
     */
    private PlanetReference destination;
    
    /**
     * This is the owner of this transport and task.
     */
    private PlayerReference owner;
    
    public TransportTaskInvade() {
    }

    /**
     * Creates a Transport task to move qty colonists from source to destination.
     * Note that this removes the colonists from source and only places them into the
     * destination on completion of this task.
     * @param source The planet from which to take the colonists
     * @param destination The planet to which to place the colonists
     * @param qty The quantity of colonists to move
     * @throws mot.common.util.ValidationError
     */
    public TransportTaskInvade(Transport transport, Planet source, Planet destination, int qty) throws ApplicationError {
    	if (destination != null) {
    		this.destination = new PlanetReference(destination);
    	}
        this.qtyAttackers = 0;
        this.owner = new PlayerReference(source.getOwner());
        
        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskInvade.init() Move "+qty+" troops from "+source.getName()+" to "+destination);
        }
        
        transport.transferTo(source.getOrbit());
        
        addTroops(null, source, qty);
        
        transport.setTask(this);
    	if (destination != null) {
    		transport.moveTo(destination.getOrbit());
    	}
    	
    }

    public void addTroops(Transport transport, Planet source, int qtyToAdd) throws ApplicationError {
        int qtyGroundDefences = source.getQtyGroundDefense();
        if (qtyToAdd > qtyGroundDefences) {
        	qtyToAdd = qtyGroundDefences;
        }
        source.setQtyGroundDefense(qtyGroundDefences - qtyToAdd);
        qtyAttackers += qtyToAdd;
        if (transport != null) {
			MessageEF msg = new MessageEF(transport.getName()+L.NOW_HAS+qtyAttackers+L.TROOPS, MessageType.plain, null);
			MotEngine.getInstance().broadcast(msg, transport.getOwner());
        }
    }

    public String getDescription() throws ApplicationError {
        String d = L.MOVE_+qtyAttackers+L._TROOPS_;
        if (destination != null) {
            d += L._TO_+destination.getName();
        }
        return d;
    }

    /**
     * Removes this task from the transport and adds the population to the destination planet.
     * This method is called when this transport arrives at its destination.
     */
	public void arrivedAt(Orbit orbit, Transport transport) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskInvade.move() Move "+qtyAttackers+" troops to "+destination);
        }

        if (destination == null) {
        	return;
        }
        
        Planet dest = destination.getPlanet();
		if (orbit == null) {
        	Logger.logWarning("Transport.arivedAt() Transport "+transport+" Has null orbit!!");
        	if (destination != null) {
        		orbit = new Orbit(dest);
        	}
        }

        if (dest.getStellarObject() != orbit.getStellarObject()) {
        	transport.moveTo(dest.getOrbit());
        	return;
        }    
        
        int qtyEnemyCraft = owner.getPlayer().getQtyEnemyCraftAt(orbit.getStellarObject());

        if ( qtyEnemyCraft > 0 || dest.hasDefenses() ) {
        	int qtyMyCraft = owner.getPlayer().getQtyMilitaryCraftsAt(orbit.getStellarObject());
        	if (qtyMyCraft == 0) {        		
        		transport.destroy();
        		if (!owner.getPlayer().isComputerPlayer()) {        		
        			MessageEF event = new MessageEF(L.TRANSPORT_LOST_AT_+destination.getName(), MessageType.flashing, null);
        			MotEngine.instance.broadcast( event, owner.getPlayer() );        	
        		}
        	}
        	return;
        }
        
        if (owner.getPlayer().isComputerPlayer()) {
	        if (dest.getOwner() == null || !MotEngine.getInstance().getRelations().isWar(dest.getOwner(), getOwner())) {
	        	transport.setTask(null);
	        	return;
	        }
	        deploy(transport, dest.getStellarObject(), dest);
        }
        
	}

	public boolean canDeployTo(StellarObject so) throws ApplicationError {
        return true;
    }

	/**
     * This transport attempts to invade this planet.
	 * @return 
	 */
	public StateUpdateCommandResult deploy(Transport transport, StellarObject destStellarObject, Planet destPlanet) throws ApplicationError {
		transport.setTask(null);
		
		if (destPlanet == null || transport.getOrbit() == null) {
			return null;
		}
		
		Orbit orbit = transport.getOrbit();
		if (destPlanet.getStellarObject() != orbit.getStellarObject()) {
	        destination = new PlanetReference(destPlanet);
	        transport.moveTo(destPlanet.getOrbit());
	        return new StateUpdateCommandResult(new TransportVOAdapter(transport));
		}
		
		TradeRoutes tradeRoutes = MotEngine.instance.getTradeRoutes();
		Planet[] effectedPlanets = tradeRoutes.getAllPlanetsEffectedByTradeRoutesIn(destStellarObject);
		TradeRoute[] effectedTradeRoutes = MotEngine.instance.getTradeRoutes().getTradeRoutes(destStellarObject);

        int qtyDefenders = destPlanet.getQtyGroundDefense();

        TroopBattle tb = new TroopBattle(getOwner(), destPlanet.getOwner(), qtyAttackers, qtyDefenders);
        tb.execute();
        int remainingDefenders = tb.getRemainingDefenders(); 
        int remainingAttackers = tb.getRemainingAttackers();
        double attackerAdvantage = tb.getAttackerAdvantage();
        destPlanet.setQtyGroundDefense(remainingDefenders);
        
        if (tb.getRemainingDefenders() > 0) {
        	destPlanet.setCrisis(new CrisisEmergency(MotEngine.instance.getDate()));
        } else {
        	destPlanet.setCrisis(null);        	
        }
        
        if (effectedTradeRoutes != null) {
        	for (int i = 0; i < effectedTradeRoutes.length; i++) {
        		tradeRoutes.remove(effectedTradeRoutes[i]);
			}
        }
        
        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskInvade.deploy() attacks "+destPlanet.getName()+" attackerAdvantage="+attackerAdvantage+" remainingAttackers="+remainingAttackers+" remainingDefenders="+remainingDefenders);
        }
        
        PlanetInvadeEF event1 = new PlanetInvadeEF(transport, destPlanet, qtyAttackers, qtyDefenders, attackerAdvantage, remainingAttackers, remainingDefenders);
        if (owner.getPlayer() instanceof UserPlayer) {
	    	event1.addStateUpdate(new RemoveMooVO(transport.getName()));
			if (effectedPlanets != null) {
				event1.addStateUpdate(EngineVoFactory.getPlanets(effectedPlanets, owner.getPlayer()) );
			}
			if (effectedTradeRoutes != null) {
				event1.addStateUpdate(EngineVoFactory.getInvalidTradeRoutes(effectedTradeRoutes) );
			}
        }
        MotEngine.instance.broadcast( event1, owner.getPlayer() );
        
        PlanetInvadeEF event2 = new PlanetInvadeEF(transport, destPlanet, qtyAttackers, qtyDefenders, attackerAdvantage, remainingAttackers, remainingDefenders);
        if (destPlanet.getOwner() instanceof UserPlayer) {
	    	event2.addStateUpdate(new RemoveMooVO(transport.getName()));
			if (effectedPlanets != null) {
				event2.addStateUpdate(EngineVoFactory.getPlanets(effectedPlanets, destPlanet.getOwner()) );
			}
			if (effectedTradeRoutes != null) {
				event2.addStateUpdate(EngineVoFactory.getInvalidTradeRoutes(effectedTradeRoutes) );
			}
        }
        MotEngine.instance.broadcast( event2, destPlanet.getOwner() );
        
        if (remainingDefenders == 0 && remainingAttackers > 0) {
            OrbitalReference miningStation = destPlanet.getMiningStation();
            Player newOwner = owner.getPlayer();
            Player oldOwner = destPlanet.getOwner();
            destPlanet.changeOwner(newOwner);
            destPlanet.setProject(null);
            MotEngine.instance.getInfluence().update();
            if (oldOwner != null) {
                oldOwner.notifyColonyLost(destPlanet, newOwner, false);
                Player[] players = MotEngine.instance.getPlayers().getPlayers();
                for (int i = 0; i < players.length; i++) {
					if (players[i] instanceof ComputerPlayer && players[i] != oldOwner && players[i] != newOwner) {
						((ComputerPlayer)players[i]).getStratagist().notifyOfInvasion(destPlanet, newOwner, oldOwner);
					}
				}
            }
            PlanetConqueredEF event3 = new PlanetConqueredEF(transport, destPlanet, oldOwner);
            if (newOwner instanceof UserPlayer) {
            	event3.addStateUpdate(new SolarSystemVOAdapter(destPlanet.getSolarSystem(), newOwner));
            	event3.addStateUpdate(new PlanetVOAdapter(destPlanet, newOwner));
            	if (miningStation != null && miningStation.getPlanet() != null) {
                	event3.addStateUpdate(new SolarSystemVOAdapter(miningStation.getPlanet().getSolarSystem(), newOwner));
                	event3.addStateUpdate(new PlanetVOAdapter(miningStation.getPlanet(), newOwner));
            	}
            	event3.addStateUpdate(EngineVoFactory.getPlayer(newOwner));
            }
            MotEngine.instance.broadcast( event3, newOwner );
            
            PlanetConqueredEF event4 = new PlanetConqueredEF(transport, destPlanet, oldOwner);
            if (oldOwner instanceof UserPlayer) {
            	event4.addStateUpdate(new SolarSystemVOAdapter(destPlanet.getSolarSystem(), oldOwner));
            	event4.addStateUpdate(new PlanetVOAdapter(destPlanet, oldOwner));
            	if (miningStation != null && miningStation.getPlanet() != null) {
                	event4.addStateUpdate(new SolarSystemVOAdapter(miningStation.getPlanet().getSolarSystem(), oldOwner));
                	event4.addStateUpdate(new PlanetVOAdapter(miningStation.getPlanet(), oldOwner));
            	}
            	event4.addStateUpdate(EngineVoFactory.getPlayer(oldOwner));
            }
            MotEngine.instance.broadcast( event4, oldOwner );
        }
        return null;
    }

	
	protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("qty-attackers", qtyAttackers);
        if (destination != null) {
            destination.toXML(writer, "destination");
        }
        if (getOwner() != null) {
            writer.addElement("owner", getOwner().getName());
        }
	}

	protected String getElementName() { return "transport-task-invade"; }

    public int getQtyAttackers() {
        return qtyAttackers;
    }

    public void setQtyAttackers(int qty) {
        this.qtyAttackers = qty;
    }

    public PlanetReference getDestination() {
        return destination;
    }

    public void setDestination(Planet destination) {
        this.destination = new PlanetReference(destination);
    }

    public void setDestination(PlanetReference destination) {
        this.destination = destination;
    }

	public Player getOwner() throws ApplicationError {
		return owner.getPlayer();
	}

	public void setOwner(String owner) {
		this.owner = new PlayerReference(owner);
	}
}