/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.engine.transports;
import mot.common.bean.MessageType;
import mot.common.command_results.StateUpdateCommandResult;
import mot.common.lang.L;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.common.vo.RemoveMooVO;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.adapter.PlanetVOAdapter;
import mot.engine.adapter.TransportVOAdapter;
import mot.engine.common.EngineVoFactory;
import mot.engine.common.Orbit;
import mot.engine.common.StellarObject;
import mot.engine.event.MessageEF;
import mot.engine.planets.Planet;
import mot.engine.planets.PlanetReference;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

/**
 * This is a task that may be assigned to a transport ship - It moves colonists 
 * from one planet to another.
 * 
 * When this task is constructed it removes the qty colonists from the source planet
 * and places them into the transport. It calculates the time it takes (using the best 
 * engine the player has available) to move them from the source to the destination. 
 * After that many turn this task deleted its self and adds the colonists to the destination
 * planet.
 */
public class TransportTaskMoveColonists extends TransportTask {


	/**
     * This is the quantity of colonists being transfered.
     */
    private int qty;
    
    /**
     * This is the destination planet for the colonists.
     */
    private PlanetReference destination;
    
    public TransportTaskMoveColonists() {
    }

    /**
     * Creates a Transport task to move qty colonists from source to destination.
     * Note that this removes the colonists from source and only places them into the
     * destination on completion of this task.
     * @param source The planet from which to take the colonists
     * @param destination The planet to which to place the colonists
     * @param qty The quantity of colonists to move
     * @throws mot.common.util.ValidationError
     */
    public TransportTaskMoveColonists(Transport transport, Planet source, Planet destination, int qty) throws ApplicationError, ValidationError{
    	if (destination != null) {
    		this.destination = new PlanetReference(destination);
    	}
        this.qty = 0;
        
        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskMoveColonists.init() Move "+qty+" colonists from "+source.getName()+L._TO_+destination);
        }
        
        transport.transferTo(source.getOrbit());
        
        addColonists(null, source, qty);
        
        transport.setTask(this);
    	if (destination != null) {
    		transport.moveTo(destination.getOrbit());
    	}
    	
    }
    
    public void addColonists(Transport transport, Planet source, int qtyToAdd) throws ApplicationError {
        double pop = source.getPopulation();
        pop -= qtyToAdd;
        source.setPopulation(pop);
        qty += qtyToAdd;
        if (transport != null) {
			MessageEF msg = new MessageEF(transport.getName()+L.NOW_HAS+qty+L._COLONISTS, MessageType.plain, null);
			MotEngine.getInstance().broadcast(msg, transport.getOwner());
        }
    }

    public String getDescription() throws ApplicationError {
        String d = L.MOVE_+qty+L._COLONISTS_;
        if (destination != null) {
            d += L._TO_+destination.getName();
        }
        return d;
    }

    /**
     * Removes this task from the transport and adds the population to the destination planet.
     * This method is called when this transport arrives at its destination.
     */
	public void arrivedAt(Orbit orbit, Transport transport) throws ApplicationError {
        if (Logger.isDebug()) {
            Logger.logDebug("TransportTaskMoveColonists.move() Move "+qty+" colonists to "+destination);
        }

        if (orbit == null) {
        	Logger.logWarning("Transport.arivedAt() Transport "+transport+" Has null orbit!!");
        	if (destination != null) {
        		transport.moveTo(new Orbit(destination.getPlanet()));
        	}
        	return;
        }
        
        Planet dest = null;
        StellarObject ss = null;
        if (orbit.getPlanet() != null) {
        	dest = orbit.getPlanet();
       		ss = dest.getStellarObject();
        } else if (destination != null) {
        	dest = destination.getPlanet();
        	ss = dest.getSolarSystem();
        } else {
        	ss = orbit.getStellarObject();
        }
        
		Player owner = transport.getOwner();
		if (owner.getQtyMilitaryCraftsAt(ss) == 0 && owner.getQtyEnemyCraftAt(ss) > 0) {
	        transport.setTask(null);
			if (!owner.isComputerPlayer()) {
        		MessageEF event = new MessageEF(L.TRANSPORT_LOST_AT_+dest != null ? dest.getName() : ss.getName(), MessageType.flashing, null);
        		MotEngine.instance.broadcast( event, owner );        	
			}
			return;
		}

		if (dest == null || dest.getOwner() != owner || dest.isMiningColony() || dest.getPopulation() + qty > dest.getMaxPopulation()) {
			if (owner instanceof ComputerPlayer) {
		        transport.setTask(null);
			}
			return;
        }

        transport.setTask(null);

        double pop = dest.getPopulation();
        pop += qty;
        dest.setPopulation(pop);

		MessageEF msg = new MessageEF("Transport delivered "+qty+" colonists to "+dest.getName(), MessageType.plain, null);
		MotEngine.getInstance().broadcast(msg, owner);
	}

	public boolean canDeployTo(StellarObject so) throws ApplicationError {
        return true;
    }

	public StateUpdateCommandResult deploy(Transport transport, StellarObject destStellarObject, Planet destPlanet) throws ApplicationError {
		if (destPlanet != null) {
	        destination = new PlanetReference(destPlanet);
	        transport.moveTo(destPlanet.getOrbit());
		}
		if (transport.getTask() == null) {
			StateUpdateCommandResult cr = new StateUpdateCommandResult(new RemoveMooVO(transport.getName()));
			if (destPlanet != null) {
				cr.addStateUpdate(new PlanetVOAdapter(destPlanet, transport.getOwner()));
			}
        	cr.addStateUpdate(EngineVoFactory.getPlayer(transport.getOwner()));
			return cr;
		} else {
			return new StateUpdateCommandResult(new TransportVOAdapter(transport));
		}
    }

	protected void encode(XMLWriter writer) throws ApplicationError {
        writer.addElement("qty", qty);
        if (destination != null) {
            destination.toXML(writer, "destination");
        }
	}

	protected String getElementName() { return "transport-task-move-colonists"; }

    public int getQty() {
        return qty;
    }

    public void setQty(int qty) {
        this.qty = qty;
    }

    public PlanetReference getDestination() {
        return destination;
    }

    public void setDestination(Planet destination) {
        this.destination = new PlanetReference(destination);
    }

    public void setDestination(PlanetReference destination) {
        this.destination = destination;
    }
}