/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import junit.runner.Version;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.RaceGraphicsList;
import mot.common.designs.CraftDesigns;
import mot.common.factory.GraphicsFactory;
import mot.common.setup.SetupOptions;
import mot.common.technology.Shields;
import mot.common.technology.Weapons;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.Bombardment;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.generals.CraftTaskMaraud;
import mot.engine.players.generals.CraftTaskProtect;
import mot.engine.setup.AncientFactory;

/**
 * Tests that a craft moves between two solar system correctly.
 * Test the General objects.
 */
public class AncientsTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(AncientsTest.class);
        return suite;
    }

    public AncientsTest(String name) {
        super(name);
    }
    
    private SetupOptions options;
    
    private ComputerPlayer player;
    private ComputerPlayer enemyPlayer;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;

    private MonsterPlayer mp;
    
    protected void setUp() throws Exception {
        MotEngine.newInstance();
        setupConfigs();
        
        assertEquals("3.8.1", Version.id());
        
        String [] playerNames = { "TestPlayer1", "TestPlayer2" };
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player = newPlayer(playerNames[0], blue, options, playerNames);
        enemyPlayer = newPlayer(playerNames[1], green, options, playerNames);
        
        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, null, 100, 0.9);
        
        ss3 = newSolarSystem("SS3", 200, 200);
        planet3 = newPlanet("SS3-A", ss3, null, 100, 0.9);

        ss4 = newSolarSystem("SS4", 300, 100);
        planet4 = newPlanet("SS4-A", ss4, enemyPlayer);

        assertEquals(40, CraftDesigns.getInstance().get("small_WaterMan").getEngineTech().getWarpSpeed());

        mp = MotEngine.instance.getMonsterPlayer();
	    RaceGraphicsList graphicsList = GraphicsFactory.getRaceGraphicsList();
        AncientFactory.createAncient(planet2, graphicsList.getGraphicsFor(ANCIENT_WATER_MAN));
        AncientFactory.createAncient(planet3, graphicsList.getGraphicsFor(ANCIENT_AIR_MAN));       
        
    }

    /**
     * Test ancient send out marauding fleet to attack players in the galaxy.
     */
    public void testMaraudNoSuitablePlanet() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testMaraudNoSuitablePlanet() *************************");
        
        mp.setNextAttackDate(3160);
        MotEngine.instance.setDate(3160);
        
        assertNull(mp.getFleet());
        
        mp.endTurn();

        assertEquals("Created fleet a crafts", 1, mp.getFleet().length);
        Fleet ancientFleet = mp.getFleet()[0];
		assertEquals("Created fleet of 1 crafts", 1, ancientFleet.getSize());
        assertEquals("Fleet has maraud task", CraftTaskMaraud.class, ancientFleet.getCrafts()[0].getTask().getClass());
        assertNull("Fleet will maraud planet1", ((CraftTaskMaraud)ancientFleet.getCrafts()[0].getTask()).getTarget());
        
        mp.endTurn();
        
        assertNull("Created fleet a crafts", mp.getFleet());
    }

    /**
     * Test ancient send out marauding fleet to attack players in the galaxy.
     */
    public void testMaraudNoAlignment() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testMaraud() *************************");

        mp.endTurn();

        mp.setNextAttackDate(3250);
        MotEngine.instance.setDate(3250);

        planet1.setPopulation(100);
        planet1.setQtyFactories(2);
        planet1.setWeapons(new Weapons(tech_massDriver1));
        
        assertNull(mp.getFleet());
        assertEquals("No alignments", (String)null, player.getAlignment());
        assertEquals("No alignments", (String)null, enemyPlayer.getAlignment());
        
        Utils.setNextRandomInt(0); // Used in select victim
        Utils.setNextRandomInt(0); // Make sure we select ancient WaterMan to attack
        mp.endTurn();

        assertEquals("Created fleet a crafts", 1, mp.getFleet().length);
        Fleet ancientFleet = mp.getFleet()[0];
		assertEquals("Created fleet of 1 crafts", 2, ancientFleet.getSize());
        assertEquals("Fleet has maraud task", "small_WaterMan1", ancientFleet.getCrafts()[0].getName());
        assertEquals("Fleet has maraud task", tech_fusion40, ancientFleet.getCrafts()[0].getEngine());
        assertEquals("Fleet has maraud task", CraftTaskMaraud.class, ancientFleet.getCrafts()[0].getTask().getClass());
        assertEquals("Fleet will maraud planet1", planet1, ((CraftTaskMaraud)ancientFleet.getCrafts()[0].getTask()).getTarget());
        
        mp.endTurn();
        
        assertEquals(3, ancientFleet.getNavigator().getETA());
        
        ancientFleet.endTurn();
        ancientFleet.endTurn();
        ancientFleet.endTurn();
        
        Battle battle = MotEngine.instance.getBattles().getBattles()[0];
        battle.start();
        battle.automate();

        assertEquals("Planet1 Marauded", 50.0, planet1.getPopulation(), 1);
        assertEquals("Planet1 Marauded", 0.5, planet1.getQtyFactories(), 1);

        mp.endTurn();

        assertNull(mp.getFleet());
    }

    /**
     * Test ancient send out marauding fleet to attack players in the galaxy.
     */
    public void testMaraudWithAlignment() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testMaraudWithAlignment() *************************");

        mp.endTurn();

        mp.setNextAttackDate(3250);
        MotEngine.instance.setDate(3250);

        planet1.setPopulation(100);
        planet1.setQtyFactories(2);
        planet1.setWeapons(new Weapons(tech_massDriver1));
        
        assertNull(mp.getFleet());
        assertNull(player.getAlignment());
        assertNull(enemyPlayer.getAlignment());
        
        player.setAlignment(ANCIENT_AIR_MAN); // Will always be WaterMan because player aligned to AitMar
        Utils.setNextRandomInt(0); // Used in select victim
        mp.endTurn();

        assertEquals("Created fleet a crafts", 1, mp.getFleet().length);
        Fleet ancientFleet = mp.getFleet()[0];
		assertEquals("Created fleet of 1 crafts", 2, ancientFleet.getSize());
        assertEquals("Fleet has maraud task", "small_WaterMan1", ancientFleet.getCrafts()[0].getName());
        assertEquals("Fleet has maraud task", tech_fusion40, ancientFleet.getCrafts()[0].getEngine());
        assertEquals("Fleet has maraud task", CraftTaskMaraud.class, ancientFleet.getCrafts()[0].getTask().getClass());
        assertEquals("Fleet will maraud planet1", planet1, ((CraftTaskMaraud)ancientFleet.getCrafts()[0].getTask()).getTarget());
        
        mp.endTurn();
        
        assertEquals(3, ancientFleet.getNavigator().getETA());
        
        ancientFleet.endTurn();
        ancientFleet.endTurn();
        ancientFleet.endTurn();
        
        Battle battle = MotEngine.instance.getBattles().getBattles()[0];
        battle.start();
        battle.automate();

        assertEquals("Planet1 Marauded", 50.0, planet1.getPopulation(), 1);
        assertEquals("Planet1 Marauded", 0.5, planet1.getQtyFactories(), 1);

        mp.endTurn();

        assertNull(mp.getFleet());
    }

    public void testMaraudTargetLost() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testMaraudTargetLost() *************************");

        mp.endTurn();

        mp.setNextAttackDate(3250);
        MotEngine.instance.setDate(3250);

        planet1.setPopulation(100);
        planet1.setQtyFactories(2);
        planet1.setWeapons(new Weapons(tech_massDriver1));
        
        planet4.setPopulation(100);
        planet4.setQtyFactories(2);
        planet4.setWeapons(new Weapons(tech_massDriver1));
        
        assertNull(mp.getFleet());
        assertNull(player.getAlignment());
        assertNull(enemyPlayer.getAlignment());
        
        Utils.setNextRandomInt(1); // Used in select victim by random instead of largest
        Utils.setNextRandomInt(1); // Used in select victim
        Utils.setNextRandomInt(1); // Make sure we select ancient AirMan to attack
        mp.endTurn();

        assertEquals("Created fleet a crafts", 1, mp.getFleet().length);
        Fleet ancientFleet = mp.getFleet()[0];
		assertEquals("Created fleet of 1 crafts", 2, ancientFleet.getSize());
        assertEquals("Fleet has maraud task", "small_AirMan1", ancientFleet.getCrafts()[0].getName());
        assertEquals("Fleet has maraud task", tech_fusion40, ancientFleet.getCrafts()[0].getEngine());
        assertEquals("Fleet has maraud task", CraftTaskMaraud.class, ancientFleet.getCrafts()[0].getTask().getClass());
        assertEquals("Fleet will maraud planet4", planet4, ((CraftTaskMaraud)ancientFleet.getCrafts()[0].getTask()).getTarget());
        
        mp.endTurn();
        
        assertEquals(4, ancientFleet.getNavigator().getETA());
        
        ancientFleet.endTurn();
        ancientFleet.endTurn();
        ancientFleet.endTurn();
        
        //
        // Colony is destroyed when crafts are on route
        //
        
        planet4.removeColony(true);
        
        MotEngine.instance.endTurn(player);

        assertNull(mp.getFleet());
    }


    /**
     * Test that when a player who is aligned with an ancient - the ancients defends the player if it loses a colony.
     */
    public void testDefendAlignedPlayer() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testDefendAlignedPlayer() *************************");

        SolarSystem enemySS1 = newSolarSystem("SSE1", 100, 140);
        Planet enemyPlanet = newPlanet("SSE1-A", enemySS1, enemyPlayer);
        enemyPlanet.setWeapons(new Weapons(tech_massDriver1));

        SolarSystem enemySS2 = newSolarSystem("SSE2", 100, 190);
        Planet enemyPlanet2 = newPlanet("SSE2-A", enemySS2, enemyPlayer);
        enemyPlanet2.setWeapons(new Weapons(tech_massDriver1));

        enemyPlayer.setAlignment(ANCIENT_WATER_MAN);
        enemyPlayer.setLastDateAncientsDefended(3000);
        MotEngine.instance.setDate(3200);

        Craft craft1 = newCraft(powerfulDesign, player, enemySS2);
        Craft craft2 = newCraft(powerfulDesign, player, enemySS2);
        Craft craft3 = newCraft(powerfulDesign, player, enemySS2);
        new Bombardment(enemyPlanet2, player.getCrafts()).run();
        
        //
        // Test colony has been destroyed AND Defend Fleet is created
        //
        
        assertEquals(0.0, enemyPlanet2.getPopulation());
        
        assertEquals("Created fleet a crafts", 1, mp.getFleet().length);
        Fleet ancientFleet = mp.getFleet()[0];
		assertEquals("Created fleet of 2 crafts", 3, ancientFleet.getSize());
        assertEquals("Fleet has maraud task", CraftTaskMaraud.class, ancientFleet.getCrafts()[0].getTask().getClass());
        assertEquals("Fleet has maraud task", CraftTaskMaraud.class, ancientFleet.getCrafts()[1].getTask().getClass());
        assertNull("Maraud task did not find an eligable planet to pillage", ((CraftTaskMaraud)ancientFleet.getCrafts()[0].getTask()).getTarget());
        assertEquals("Maraud task is assigned to enemyPlayer", player, ((CraftTaskMaraud)ancientFleet.getCrafts()[1].getTask()).getTargetPlayer().getPlayer());
        
        mp.endTurn();
    }
    
    /**
     * Test that when a player who is aligned with an ancient - the ancients defends the player if it loses a colony.
     */
    public void testDefendAlignedPlayerFail() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testDefendAlignedPlayerFail() *************************");

        SolarSystem enemySS1 = newSolarSystem("SSE1", 100, 140);
        Planet enemyPlanet = newPlanet("SSE1-A", enemySS1, enemyPlayer);
        enemyPlanet.setWeapons(new Weapons(tech_massDriver1));

        SolarSystem enemySS2 = newSolarSystem("SSE2", 100, 190);
        Planet enemyPlanet2 = newPlanet("SSE2-A", enemySS2, enemyPlayer);
        enemyPlanet2.setWeapons(new Weapons(tech_massDriver1));

        enemyPlayer.setAlignment(ANCIENT_WATER_MAN);
        enemyPlayer.setLastDateAncientsDefended(3100);
        MotEngine.instance.setDate(3149);

        Craft craft1 = newCraft(powerfulDesign, player, enemySS2);
        Craft craft2 = newCraft(powerfulDesign, player, enemySS2);
        Craft craft3 = newCraft(powerfulDesign, player, enemySS2);
        new Bombardment(enemyPlanet2, player.getCrafts()).run();
        
        //
        // Test colony has been destroyed AND Defend Fleet is created
        //
        
        assertEquals(0.0, enemyPlanet2.getPopulation());
        
        assertNull(mp.getFleet());
    }
    
    /**
     * Test that when a player declares war on another player with the same alignment the ancients attack the aggressor.
     */
    public void testAttackOnWarDeclaration() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testAttackOnWarDeclaration() *************************");
 
        String [] players = { "TestPlayer2" };
        ComputerPlayer enemyPlayer = newPlayer(players[0], green, options, players);
        SolarSystem enemySS = newSolarSystem("SSE1", 100, 140);
        Planet enemyPlanet = newPlanet("SSE1-A", enemySS, enemyPlayer);
        enemyPlanet.setWeapons(new Weapons());
        enemyPlanet.getWeapons().add(tech_massDriver1);
        enemyPlanet.setShields(new Shields(tech_energy10));

        SolarSystem enemySS2 = newSolarSystem("SSE2", 100, 190);
        Planet enemyPlanet2 = newPlanet("SSE2-A", enemySS2, enemyPlayer);
        enemyPlanet2.setWeapons(new Weapons(tech_massDriver1));

        SolarSystem enemySS3 = newSolarSystem("SSE3", 60, 120);
        Planet enemyPlanet3 = newPlanet("SSE3-A", enemySS3, enemyPlayer);
        enemyPlanet3.setPopulation(10.0);
        enemyPlanet3.setSize(40);
        enemyPlanet3.setTeraforming(1.0);
        enemyPlanet3.setWeapons(new Weapons(tech_massDriver1));

        MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
        player.setAlignment(ANCIENT_WATER_MAN);
        enemyPlayer.setAlignment(ANCIENT_WATER_MAN);

        assertEquals("Ancient player has its Mother Craft", 8, mp.getCrafts().length);
        
        mp.endTurn();
        assertEquals("Mother Craft has protect task", CraftTaskProtect.class, mp.getCrafts()[0].getTask().getClass());
        
        // enemyPlayer declares war on player of same alignment but there are not any planets with pop > half full
        // so ancients have no eligible targets - Fleet of two crafts is created with task Maraud but will complete.

        assertNull(mp.getFleet());
        
        player.diplomaticRequest(enemyPlayer, DiplomaticProposal.getWar(), null);

        assertNull(mp.getFleet());
        
        mp.endTurn();

        assertNull(mp.getFleet());
        assertEquals("Maraud task destroyed", 2, mp.getGeneral().getTasks().length);
        assertEquals("Maraud task destroyed", "SS2", ((CraftTaskProtect)mp.getGeneral().getTasks()[0]).getTarget().getStellarObject().getName());
        assertEquals("Maraud task destroyed", "SS3", ((CraftTaskProtect)mp.getGeneral().getTasks()[1]).getTarget().getStellarObject().getName());
        
        enemyPlanet.setPopulation(100);
        enemyPlanet3.setPopulation(25);
        MotEngine.instance.setDate(3160);

        // enemyPlayer declares war on player of same alignment - Enemy player has enemyPlanet and enemyPlanet3 
        // with > 50% pop and defences so ancients maraud these planets.

        player.diplomaticRequest(enemyPlayer, DiplomaticProposal.getWar(), null);

        assertEquals("Created fleet a crafts", 1, mp.getFleet().length);
        Fleet ancientFleet = mp.getFleet()[0];
		assertEquals("Created fleet of 2 crafts", 2, ancientFleet.getSize());
        assertEquals("Fleet has maraud task", CraftTaskMaraud.class, ancientFleet.getCrafts()[0].getTask().getClass());
        assertEquals("Fleet has maraud task", CraftTaskMaraud.class, ancientFleet.getCrafts()[1].getTask().getClass());
        assertEquals("Maraud task found an eligable planet to pillage", enemyPlanet, ((CraftTaskMaraud)ancientFleet.getCrafts()[0].getTask()).getTarget().getPlanet());
        assertEquals("Maraud task is assigned to enemyPlayer", enemyPlayer, ((CraftTaskMaraud)ancientFleet.getCrafts()[0].getTask()).getTargetPlayer().getPlayer());

        Craft ancientCraft1 = ancientFleet.getCrafts()[0];
        Craft ancientCraft2 = ancientFleet.getCrafts()[1];
        mp.endTurn();
        ancientFleet.endTurn();
        
        assertEquals("Enemy Fleet enroute to enemySS", enemySS, ancientFleet.getNavigator().getDestination().getStellarObject());
        
        Craft enemyCraft = newCraft(fighterDesign, enemyPlayer, enemySS);
        ancientFleet.arrivedAt(enemySS.getOrbit());
        
        assertEquals("Battle scheduled", 1, MotEngine.instance.getBattles().getBattles().length);
        assertEquals("Battle contains correct crafts", true, MotEngine.instance.getBattles().getBattles()[0].contains(enemyCraft));
        assertEquals("Battle contains correct crafts", true, MotEngine.instance.getBattles().getBattles()[0].contains(ancientCraft1));
        assertEquals("Battle contains correct crafts", true, MotEngine.instance.getBattles().getBattles()[0].contains(ancientCraft2));
        assertEquals("Battle contains correct crafts", enemyPlanet, MotEngine.instance.getBattles().getBattles()[0].getPlanet().getPlanet());
        
        // Fight the battle at enemyPlanet and check that planet enemyPlanet has been pillaged.

        enemyPlanet.setQtyFactories(10.0);        
        MotEngine.instance.endTurn(player);
        
        assertEquals("Enemy craft has been destroyed", 0, enemyPlayer.getCrafts().length);
        assertEquals("EnemyPlanet defenses have been destroyed", 0, enemyPlanet.getWeapons().getWeapons().length);
        assertEquals("EnemyPlanet defenses have been destroyed", null, enemyPlanet.getOrbital());
        assertEquals("EnemyPlanet has lost half its population", 100.0, enemyPlanet.getMaxPopulation(), 1.0);
        assertEquals("EnemyPlanet has lost half its population", 50.0, enemyPlanet.getPopulation(), 1.0);
        assertEquals("EnemyPlanet has all its factories", 5, enemyPlanet.getQtyFactories(), 1.0);
        
        // Test that once enemyPlanet has been pillaged, the fleet will then head to enemyPlanet3 to
        // pillage that planet.

        mp.endTurn();
        ancientFleet.endTurn();

        assertEquals("Battle complete", null, MotEngine.instance.getBattles().getBattles());
        assertEquals("Enemy Fleet enroute to enemySS3", enemyPlanet3, ancientFleet.getNavigator().getDestination());
        
        ancientFleet.arrivedAt(enemySS3.getOrbit());

        assertEquals("Battle scheduled", 1, MotEngine.instance.getBattles().getBattles().length);
        assertEquals("Battle contains correct crafts", true, MotEngine.instance.getBattles().getBattles()[0].contains(ancientCraft1));
        assertEquals("Battle contains correct crafts", true, MotEngine.instance.getBattles().getBattles()[0].contains(ancientCraft2));
        assertEquals("Battle contains correct crafts", enemyPlanet3, MotEngine.instance.getBattles().getBattles()[0].getPlanet().getPlanet());

        // Fight the battle at enemyPlanet3 and check that planet enemyPlanet has been pillaged.

        enemyPlanet3.setQtyFactories(40.0);        
        ancientCraft1.setDamage(0);
        ancientCraft2.setDamage(0);
        MotEngine.instance.endTurn(player);
        
        assertEquals("EnemyPlanet3 defenses have been destroyed", 0, enemyPlanet3.getWeapons().getWeapons().length);
        assertEquals("EnemyPlanet3 defenses have been destroyed", null, enemyPlanet3.getOrbital());
        assertEquals("EnemyPlanet3 has lost half its population", 40.0, enemyPlanet3.getMaxPopulation(), 1.0);
        assertEquals("EnemyPlanet3 has lost half its population", 12.0, enemyPlanet3.getPopulation(), 1.0);
        assertEquals("EnemyPlanet3 has all its factories", 20, enemyPlanet3.getQtyFactories(), 1.0);
        
        // Maraud task does not have any other targets to maraud so it gets removed along with its crafts.
        
        mp.endTurn();

        assertNull(mp.getFleet());
        assertEquals("Maraud task destroyed", 2, mp.getGeneral().getTasks().length);
        assertEquals("Maraud task destroyed", "SS2", ((CraftTaskProtect)mp.getGeneral().getTasks()[0]).getTarget().getStellarObject().getName());
        assertEquals("Maraud task destroyed", "SS3", ((CraftTaskProtect)mp.getGeneral().getTasks()[1]).getTarget().getStellarObject().getName());
        
    }
      
    protected void tearDown() {
    	Logger.flushLog();
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
    
}