/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import java.io.File;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.client.animation.AnimationContainer;
import mot.client.animation.BattleAnimation;
import mot.client.animation.BattleAnimationList;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.ComputerPlayerDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.event.MotEventConsumer;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.MotServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.animation.BotAttackAnimation;
import mot.client.swing.animation.CraftShiftAnimation;
import mot.client.swing.animation.CraftTurnAnimation;
import mot.client.swing.animation.DamageAnimation;
import mot.client.swing.animation.DamageAnimationFactory;
import mot.client.swing.animation.DelayAnimation;
import mot.client.swing.animation.ExplodeAnimation;
import mot.client.swing.animation.FearAnimation;
import mot.client.swing.animation.FlashAnimation;
import mot.client.swing.animation.HologramDestroyedAnimation;
import mot.client.swing.animation.LinkedAnimation;
import mot.client.swing.animation.LinkedDamageAnimation;
import mot.client.swing.animation.MessageAnimation;
import mot.client.swing.animation.MissTargetAnimation;
import mot.client.swing.animation.MissileLaunchAnimation;
import mot.client.swing.animation.MoveMissileLinkedAnimation;
import mot.client.swing.animation.PlagueStrikeAnimation;
import mot.client.swing.animation.PulseAnimation;
import mot.client.swing.animation.ShieldBurstAnimation;
import mot.client.swing.animation.SleepStrikeAnimation;
import mot.client.swing.animation.TroopBattleAnimation;
import mot.client.swing.animation.WarpCoreOverloadAnimation;
import mot.client.swing.animation.WeaponFireAnimation;
import mot.client.swing.animation.WeaponFireAnimationFactory;
import mot.client.swing.event.SwingMotEventConsumer;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.TestBattleWindowConfig;
import mot.common.bean.CraftGraphics;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.ObjectMovement;
import mot.common.bean.ObjectShift;
import mot.common.bean.ObjectTurn;
import mot.common.bean.PlayerColor;
import mot.common.bean.RaceGraphics;
import mot.common.bean.RaceGraphicsList;
import mot.common.bean.TroopBattleResult;
import mot.common.designs.CraftDesign;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.event.BattleDamageEvent;
import mot.common.event.BattleDamageRingEvent;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.BattleDroidMineLaunchEvent;
import mot.common.event.BattleEndEvent;
import mot.common.event.BattleFearEvent;
import mot.common.event.BattleMissileLaunchEvent;
import mot.common.event.BattleMissileMoveEvent;
import mot.common.event.BattleNewTurnEvent;
import mot.common.event.BattleObjectNotificationEvent;
import mot.common.event.BattleObjectNotificationType;
import mot.common.event.BattleObjectTriggerEvent;
import mot.common.event.BattlePulseEvent;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.BattleWeaponMissEvent;
import mot.common.event.EndTurnEvent;
import mot.common.event.LinkedEventBase;
import mot.common.event.MotEvent;
import mot.common.event.NewBattleEvent;
import mot.common.event.NoProjectEvent;
import mot.common.event.NoResearchProjectEvent;
import mot.common.factory.GraphicsFactory;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.technology.Hull;
import mot.common.technology.Technologies;
import mot.common.technology.Weapon;
import mot.common.technology.WeaponFirePoint;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleMine;
import mot.engine.battles.BattleMineFactory;
import mot.engine.battles.HoloCraftFactory;
import mot.engine.battles.Missiles;
import mot.engine.battles.incumbants.IncumbentFactory;
import mot.engine.battles.triggers.CraftTriggerBase;
import mot.engine.battles.triggers.DamageBot;
import mot.engine.battles.triggers.Plague;
import mot.engine.battles.triggers.Sleep;
import mot.engine.common.CompletedServerCallbackSyncroniser;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.technology.WeaponInstanceDroidMine;
import mot.engine.technology.WeaponInstanceMissile;

/**
 * This test harness tests that players can use spies to steal technology
 * sabotage planets and hide. It also tests that spies stationed at home
 * find an kill enemy spies.
 */

public class AnimationTest extends TestBase implements MotEventConsumer {
	
    private static final String REMOTE_SERVER_URL = "http://localhost:7999/main";

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(mot.test.AnimationTest.class);
        return suite;
    }

    public AnimationTest(String name) {
        super(name);
    }
    
    private UserPlayer p1;
    private Player p2;
    private SolarSystem ss1, ss2;
    private Planet h1, h2;
    private Craft c1, c2, c3;

    private MotServerProxy serverProxy;
    private ServerProxy playerTwoServerProxy;
    
    private UserPlayerDelegate player1;
    private ComputerPlayerDelegate player2;
	private StellarObjectDelegate solarSystem1,solarSystem2;    
	private PlanetDelegate planet1,planet2;
	private CraftDelegate craft1, craft2, craft3;
	
	private TestClientProxy p1Proxy, p2Proxy;
	
	private SwingMotEventConsumer processor = new SwingMotEventConsumer(null);
	private CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();

	private static Hull tech_flyer30WithOneOffset;
	private static Hull tech_flyer30WithTwoOffsets;
	
	private static void createHulls() throws ApplicationError {
		tech_flyer30WithOneOffset = (Hull) tech_flyer30.cloneTechnology();
		tech_flyer30WithOneOffset.setName(tech_flyer30WithOneOffset.getName()+"WithOffsets1");
		Technologies.getHulls().add(tech_flyer30WithOneOffset);
		
		tech_flyer30WithTwoOffsets = (Hull) tech_flyer30.cloneTechnology();
		tech_flyer30WithTwoOffsets.setName(tech_flyer30WithTwoOffsets.getName()+"WithOffsets2");
		Technologies.getHulls().add(tech_flyer30WithTwoOffsets);
		
		RaceGraphicsList graphicsList = GraphicsFactory.getRaceGraphicsList();
        RaceGraphics rg = graphicsList.getGraphicsFor("Humans");
        CraftGraphics flyerGraphics = rg.getHullGraphics(tech_flyer30.getName());
        
        CraftGraphics cg1 = new CraftGraphics(flyerGraphics);
        cg1.setName(cg1.getName()+"WithOffsets1");
		cg1.setWeaponFireOffset(new WeaponFirePoint(-5,0));
        rg.setCraftGraphics(cg1);

        CraftGraphics cg2 = new CraftGraphics(flyerGraphics);
        cg2.setName(cg2.getName()+"WithOffsets2");
        cg2.setWeaponFireOffset(new WeaponFirePoint(-5,-5));
        cg2.setWeaponFireOffset(new WeaponFirePoint(-5,+5));
        cg2.setDishOffset(new WeaponFirePoint(-6,-0));
        rg.setCraftGraphics(cg2);

        rg = graphicsList.getGraphicsFor("Bears");
        flyerGraphics = rg.getHullGraphics(tech_flyer30.getName());
        
        cg1 = new CraftGraphics(flyerGraphics);
        cg1.setName(cg1.getName()+"WithOffsets1");
		cg1.setWeaponFireOffset(new WeaponFirePoint(-5,0));
        rg.setCraftGraphics(cg1);

        cg2 = new CraftGraphics(flyerGraphics);
        cg2.setName(cg2.getName()+"WithOffsets2");
        cg2.setWeaponFireOffset(new WeaponFirePoint(-5,-5));
        cg2.setWeaponFireOffset(new WeaponFirePoint(-5,+5));
        cg2.setDishOffset(new WeaponFirePoint(-6,-0));
        rg.setCraftGraphics(cg2);
	}
	
	protected void setUp() throws ApplicationError, ValidationError {
        MotEngine.newInstance();
        DeligateCacheManager.flush();
        setupConfigs();
        ProxyStore.removeInstance();
        MainWindow.__createTestMainWindow(new TestBattleWindowConfig(this));
       	new ProxyStore(new MotServerProxy(true, MainWindow.getExestingInstance(), this));
        playerTwoServerProxy = null;        
        
        if (tech_flyer30WithOneOffset == null) {
        	createHulls();
        }
    }
    
    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }
    
    public void createMap(CraftDesign craftType1, CraftDesign craftType2, boolean withAsteroids) throws Exception {
    	createMap(craftType1, craftType2, null, withAsteroids);
    }
    
    public void createMap(CraftDesign craftType1, CraftDesign craftType2, CraftDesign craftType3, boolean withAsteroids) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        RaceOptions options = OptionsFactory.getRaceOptions(playerNames[0]);
        p1 = newUserPlayer(playerNames[0], blue, null, options);
        p2 = newPlayer(playerNames[1], green, options, playerNames);

        MainWindow.getExestingInstance().__setWindowOwner(playerNames[0]);
        
        ss1 = newSolarSystem("SS1", 100, 100);
        ss2 = newSolarSystem("SS2", 150, 100);
        h1 = newPlanet("SS1-A", ss1, p1);
        h2 = newPlanet("SS2-A", ss2, p2);

        if (withAsteroids) h1.setHasAsteroids(true);
        
        serverProxy = (MotServerProxy) ProxyStore.getInstance();
        
        MotEngine.instance.getProxyManager().removeAll();
        CompletedServerCallbackSyncroniser callback = new CompletedServerCallbackSyncroniser();
        serverProxy.register(playerNames[0], true, callback, null);
        callback.waitTillCompleteAndProcessCommandResponse(serverProxy);

        player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer(playerNames[0]);
        player2 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer(playerNames[1]);
        
        p1Proxy = new TestClientProxy(p1);
        p2Proxy = new TestClientProxy(p2);

        solarSystem1 = DeligateCacheManager.getStellarObject("SS1");
        solarSystem2 = DeligateCacheManager.getStellarObject("SS2");
		planet1 = DeligateCacheManager.getPlanet("SS1-A");
		planet2 = DeligateCacheManager.getPlanet("SS2-A");
		
        MotEngine.getInstance().getRelations().setWar(p1, p2);

		if (craftType1 != null) {
			c1 = newCraft(craftType1, p1, ss1);
		}
		if (craftType2 != null) {
	        c2 = newCraft(craftType2, p2, ss1);
		}
		if (craftType3 != null) {
	        c3 = newCraft(craftType3, p1, ss1);
		}

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
		
		if (craftType1 != null) {
	        craft1 = DeligateCacheManager.getCraft(c1.getName());
		}
		if (craftType2 != null) {
	        craft2 = DeligateCacheManager.getCraft(c2.getName());
		}
		if (craftType3 != null) {
	        craft3 = DeligateCacheManager.getCraft(c3.getName());
		}
		
		MainWindow.getInstance()._setWindowOwner(player1);
    }
    
    private void processEndingThread() throws Exception {
    	Thread.sleep(100);
        while (serverProxy.isBusy()) {
        	Thread.yield();
        	Thread.sleep(100);
        	Thread.yield();
        }
    }
    
    public void testBayBeamWeaponFireAnimation() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testBayWeaponFireAnimation() *************************");
        CraftDesign attackerDesign = new CraftDesign("TestFighter", tech_flyer30WithTwoOffsets, tech_fusion25, tech_electronic25, tech_energy10, tech_massDriver1);        
        createMap(attackerDesign,fighterDesign, false);

		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), crafts[0].getName());
        assertEquals(c2.getName(), crafts[1].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

        
        //
        // CRAFT1 FIRES AT CRAFT2 WITH NO WEAPON FIRE OFFSETS
        //
        
        clearEvents();
        serverProxy.battleAttackCmd(battle, crafts[0], crafts[1], false, false, false, null, null, 0, callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        callbackSync.waitTillComplete();
        
		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_massDriver1.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), wfe.getSource());
		assertEquals(craft2.getName(), wfe.getTarget());
		assertEquals(new Location(500,500), wfe.getSourceLocation(null));
		assertEquals(new Location(600,500), wfe.getTargetLocation(null));
		assertEquals(new Location(500,500), wfe.getOldSourceState().getLocation());
		assertEquals(new Location(500,500), wfe.getNewSourceState().getLocation());
		assertEquals(90, 					wfe.getOldSourceState().getDirection());
		assertEquals(90, 					wfe.getNewSourceState().getDirection());
		assertEquals(0.0, 					wfe.getOldSourceState().getDamagePercent());
		assertEquals(0.0, 					wfe.getNewSourceState().getDamagePercent());
		assertEquals(new Location(600,500), wfe.getOldTargetState().getLocation());
		assertEquals(new Location(600,500), wfe.getNewTargetState().getLocation());
		assertEquals(0.0, 					wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.05, 					wfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(blue,					wfe.getOldSourceState().getPlayerColor());
		assertEquals(blue,					wfe.getNewSourceState().getPlayerColor());
		assertEquals(green,					wfe.getOldTargetState().getPlayerColor());
		assertEquals(green,					wfe.getNewTargetState().getPlayerColor());
		assertEquals((String)null,			wfe.getOldTargetState().getCloakedToPlayerNames());
		assertEquals((String)null,			wfe.getNewTargetState().getCloakedToPlayerNames());
		assertEquals(0.0, 					wfe.getOldTargetState().getPsiDamagePercent());
		assertEquals(0.0,					wfe.getNewTargetState().getPsiDamagePercent());
		assertEquals(1.0, 					wfe.getOldTargetState().getShieldPercent());
		assertEquals(0.8,					wfe.getNewTargetState().getShieldPercent());
		assertEquals(1.0, 					wfe.getOldTargetState().getCrewPercent());
		assertEquals(1.0,					wfe.getNewTargetState().getCrewPercent());
		assertEquals(null, 					wfe.getOldTargetState().getDecorators());
		assertEquals(null,					wfe.getNewTargetState().getDecorators());
		assertEquals(0.0, 					wfe.getOldTargetState().getMovementDamagePercent());
		assertEquals(0.0,					wfe.getNewTargetState().getMovementDamagePercent());
		assertEquals(false,					wfe.isReverseAnimation());
		assertEquals((String)null,			wfe.getSourceDescription());
		assertEquals("1.0",				wfe.getTargetDescription());
		MotEvent[] links = wfe.getLinkedEvents();
		assertEquals(null,					links);
		
		//
		// Process the event
		//
		
		ac.getAnimations().stopAll();
		processor.process(wfe, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(2, baa.length);
		
		WeaponFireAnimation animation = (WeaponFireAnimation) baa[0];
		assertEquals(2, 			animation.getSource().length);
		assertEquals(505,505, 		animation.getSource()[0]);
		assertEquals(505,495, 		animation.getSource()[1]);
		assertEquals(600,500, 		animation.getDestination());
		assertEquals(c1.getName(), 	animation.getAgressor().getName());
		assertEquals(c2.getName(), 	animation.getTarget().getName());
		assertEquals(null, 			animation.getLinkedAnimations());

		DamageAnimation da = (DamageAnimation) baa[1];
		assertEquals("1.0", 	da.getMessage());
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals(600,500, 		da.getOldState().getLocation());
		assertEquals(600,500, 		da.getNewState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.001);
		assertEquals(0.05, 			da.getNewState().getDamagePercent(), 0.001);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals((String)null,	da.getOldState().getCloakedToPlayerNames());
		assertEquals((String)null,	da.getNewState().getCloakedToPlayerNames());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(0.8,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		
    }     
    
    public void testGiratorDishBeamWeaponFireAnimation() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testGiratorDishBeamWeaponFireAnimation() *************************");
        CraftDesign attackerDesign = new CraftDesign("TestGirator", tech_flyer30WithTwoOffsets, tech_fusion25, tech_girator);
        createMap(attackerDesign,defenderDesign, fighterDesign, false);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.craftAttackCmd(craft3, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), crafts[0].getName());
        assertEquals(c2.getName(), crafts[1].getName());
        assertEquals(c3.getName(), crafts[2].getName());
        
        assertTrue(tech_girator.usesDish());
        
        //
        // CRAFT1 FIRES AT CRAFT2 WITH NO WEAPON FIRE OFFSETS
        //
        
		CraftGraphics cg = GraphicsFactory.getRaceGraphicsList().getHullGraphics(c1.getOwner().getName(), c1.getHull().getName());
		assertEquals(-6, 0, cg.getDishOffset());

        clearEvents();
        serverProxy.battleAttackCmd(battle, crafts[0], crafts[1], true, true, true, new LocationAndDirection(crafts[1].getLocation()), tech_girator, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        callbackSync.waitTillComplete();
        
		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_girator.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), wfe.getSource());
		assertEquals(craft2.getName(), wfe.getTarget());
		assertEquals(new Location(572,500), wfe.getSourceLocation(null));
		assertEquals(new Location(600,500), wfe.getTargetLocation(null));
		assertEquals(new Location(600,500), wfe.getOldTargetState().getLocation());
		assertEquals(new Location(600,500), wfe.getNewTargetState().getLocation());
		assertEquals(0.0, wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.625, wfe.getNewTargetState().getDamagePercent(), 0.001);
		
		WeaponFireAnimation animation = (WeaponFireAnimation) WeaponFireAnimationFactory.create(wfe, null);
		assertEquals(new Location(578,500), animation.getSource()[0]);
		assertEquals(new Location(600,500), animation.getDestination());
		
    }     

    public void testWeaponFireMissAnimation() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testWeaponFireMissAnimation() *************************");
        CraftDesign cloakedFighter = new CraftDesign("CloakedFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_cloak150, tech_massDriver1);
        createMap(fighterDesign, cloakedFighter, false);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 FIRES SHIELD DRAIN AT CRAFT2 - CRAFT2 IS CLOAKED
        //
        
        clearEvents();
        Utils.clearAndSetNextRandomProbability(0.1);
        bca[0].fireWeapons(bca[1], null, -1);
        
        BattleWeaponMissEvent mte = (BattleWeaponMissEvent) p1Proxy.getEvent(0);
		assertEquals(tech_massDriver1,			mte.getWeapon());
		assertEquals(craft1.getName(), 			mte.getSource());
		assertEquals(craft2.getName(), 			mte.getTarget());
		assertEquals(500,500, 					mte.getSourceLocation());
		assertEquals(600,500, 					mte.getTargetLocation());
		
		ac.getAnimations().stopAll();
		processor.process(mte, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(1, baa.length);
		
		MissTargetAnimation mta = (MissTargetAnimation) baa[0];
		assertEquals(507,500, 		mta.getSource()[0]);
		assertEquals(611,489, 		mta.getDestination());
		
    }     

    public void testFearAnimation() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testFearAnimation() *************************");
        createMap(fighterDesign, fighterDesign, false);
        p2.getPlayerCharacter().setPsiRating(25);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 FIRES MASS DRIVER AT CRAFT2 - CRAFT2 HAS HIGH PSI AND THIS CRAFT WILL FEAR
        //
        
        clearEvents();
        Utils.clearAndSetNextRandomInt(0);
        bca[0].fireWeapons(bca[1], null, -1);
        serverProxy._processQueuedEvents();
        
        BattleFearEvent fe = (BattleFearEvent) p1Proxy.getEvent(0);
		assertEquals(craft1.getName(), 		fe.getSource());
		assertEquals(craft2.getName(), 		fe.getTarget());
		
		ac.getAnimations().stopAll();
		processor.process(fe, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(1, baa.length);
		
		FearAnimation fa = (FearAnimation) baa[0];
		assertEquals(500,500, 			fa.getImage().getLocation());
		assertEquals(craft1.getName(), 	fa.getTarget().getName());
		
    }     



    
    public void testAutomatonDestruction() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testAutomatonDestruction() *************************");
        createMap(fighterDesign,defenderDesign, false);
        
        MotEngine.getInstance().getRelations().setWar(p1, p2);

		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), crafts[0].getName());
        assertEquals(c2.getName(), crafts[1].getName());
        
        WeaponInstanceDroidMine droidMineInstance = new WeaponInstanceDroidMine(tech_droidMine, 1, true, null);
        WeaponInstanceMissile missileInstance = new WeaponInstanceMissile(tech_missile6, 1, true, true);
        
        //
        // TEST CREATE DROID MINE
        //
        
        droidMineInstance.fire(bca[0], bca[1], 1);
        serverProxy._processQueuedEvents();
        
        BattleDroidMineLaunchEvent dmle = (BattleDroidMineLaunchEvent) p1Proxy.getEvent(BattleDroidMineLaunchEvent.class);
		assertEquals(500, 500, dmle.getDroidMine().getState().getLocation());
		assertEquals(500, 500, DeligateCacheManager.getBattleAutomaton(dmle.getBattle(), dmle.getDroidMine().getName()).getLocation());
		assertEquals(bca[1].getName(), dmle.getDroidMine().getTarget());
		
        //
        // TEST CREATE MISSILE
        //
        
		missileInstance.fire(bca[1], bca[0], 1);
        serverProxy._processQueuedEvents();
        
        BattleMissileLaunchEvent mle = (BattleMissileLaunchEvent) p1Proxy.getEvent(BattleMissileLaunchEvent.class);
		assertEquals(new Location(600, 500), mle.getMissile().getState().getLocation());
		assertEquals(new Location(600, 500), DeligateCacheManager.getBattleAutomaton(dmle.getBattle(), mle.getMissile().getName()).getLocation());
		assertEquals(bca[0].getName(), mle.getMissile().getTarget());
		
    }
    
    public void testShieldDrainAndCloakedCraft() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testShieldDrainAndCloakedCraft() *************************");
        CraftDesign cloakedFighter = new CraftDesign("CloakedFighter", tech_flyer30WithOneOffset, tech_fusion25, tech_electronic25, tech_energy10, tech_cloak150, tech_shieldDrain);
        createMap(cloakedFighter,fighterDesign, false);
        
        CraftDesign mindFighter = new CraftDesign("MindFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_mindControl);
		Craft c3 = newCraft(mindFighter, p1, ss1);
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        CraftDelegate craft3 = DeligateCacheManager.getCraft("MindFighter1");
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.craftAttackCmd(craft3, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[2].setLocation(new Location(550, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);
        bca[2].setDirection(90);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        assertEquals(c3.getName(), bca[2].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 FIRES SHIELD DRAIN AT CRAFT2 & CRAFT1 IS CLOAKED
        //
        
		CraftGraphics cg = GraphicsFactory.getRaceGraphicsList().getHullGraphics(c1.getOwner().getName(), c1.getHull().getName());
		assertEquals(1, cg.getWeaponFireOffset().length);

        clearEvents();
        bca[0].fireAllWeapons(bca[1]);
        
		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_shieldDrain.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), wfe.getSource());
		assertEquals(craft2.getName(), wfe.getTarget());
		assertEquals(500,500, 				wfe.getSourceLocation(null));
		assertEquals(600,500, 				wfe.getTargetLocation(null));
		assertEquals(500,500, 				wfe.getOldSourceState().getLocation());
		assertEquals(500,500, 				wfe.getNewSourceState().getLocation());
		assertEquals(90, 					wfe.getOldSourceState().getDirection());
		assertEquals(90, 					wfe.getNewSourceState().getDirection());
		assertEquals(0.0, 					wfe.getOldSourceState().getDamagePercent());
		assertEquals(0.0, 					wfe.getNewSourceState().getDamagePercent());
		assertEquals(600,500, 				wfe.getOldTargetState().getLocation());
		assertEquals(600,500, 				wfe.getNewTargetState().getLocation());
		assertEquals(0.0, 					wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.0, 					wfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(blue,					wfe.getOldSourceState().getPlayerColor());
		assertEquals(blue,					wfe.getNewSourceState().getPlayerColor());
		assertEquals(green,					wfe.getOldTargetState().getPlayerColor());
		assertEquals(green,					wfe.getNewTargetState().getPlayerColor());
		assertEquals((String)null,			wfe.getOldTargetState().getCloakedToPlayerNames());
		assertEquals((String)null,			wfe.getNewTargetState().getCloakedToPlayerNames());
		assertEquals("Bears",				wfe.getOldSourceState().getCloakedToPlayerNames()[0]);
		assertEquals("Bears",				wfe.getNewSourceState().getCloakedToPlayerNames()[0]);
		assertEquals(0.0, 					wfe.getOldTargetState().getPsiDamagePercent());
		assertEquals(0.0,					wfe.getNewTargetState().getPsiDamagePercent());
		assertEquals(1.0, 					wfe.getOldTargetState().getShieldPercent());
		assertEquals(0.5,					wfe.getNewTargetState().getShieldPercent());
		assertEquals(1.0, 					wfe.getOldTargetState().getCrewPercent());
		assertEquals(1.0,					wfe.getNewTargetState().getCrewPercent());
		assertEquals(null, 					wfe.getOldTargetState().getDecorators());
		assertEquals(null,					wfe.getNewTargetState().getDecorators());
		assertEquals(0.0, 					wfe.getOldTargetState().getMovementDamagePercent());
		assertEquals(0.0,					wfe.getNewTargetState().getMovementDamagePercent());
		assertEquals(true,					wfe.isReverseAnimation());
		assertEquals("+0",					wfe.getSourceDescription());
		assertEquals("-5.0",				wfe.getTargetDescription());
		MotEvent[] links = wfe.getLinkedEvents();
		assertEquals(null,					links);
		
		ac.getAnimations().stopAll();
		processor.process(wfe, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(3, baa.length);
		
		WeaponFireAnimation animation = (WeaponFireAnimation) baa[0];
		assertEquals(new Location(596,500), animation.getSource()[0]);
		assertEquals(new Location(500,500), animation.getDestination());
		assertEquals(c1.getName(), animation.getAgressor().getName());
		assertEquals(c2.getName(), animation.getTarget().getName());
		assertEquals(null, animation.getLinkedAnimations());

		DamageAnimation da = (DamageAnimation) baa[1];
		assertEquals("-5.0", 		da.getMessage());
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals(600,500, 		da.getOldState().getLocation());
		assertEquals(600,500, 		da.getNewState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.001);
		assertEquals(0.0, 			da.getNewState().getDamagePercent(), 0.001);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals((String)null,	da.getOldState().getCloakedToPlayerNames());
		assertEquals((String)null,	da.getNewState().getCloakedToPlayerNames());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(0.5,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		

		da = (DamageAnimation) baa[2];
		assertEquals("+0", 			da.getMessage());
		assertEquals(c1.getName(), 	da.getTarget().getName());
		assertEquals(500,500, 		da.getOldState().getLocation());
		assertEquals(500,500, 		da.getNewState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.001);
		assertEquals(0.0, 			da.getNewState().getDamagePercent(), 0.001);
		assertEquals(blue,			da.getOldState().getPlayerColor());
		assertEquals(blue,			da.getNewState().getPlayerColor());
		assertEquals("Bears",		da.getOldState().getCloakedToPlayerNames()[0]);
		assertEquals("Bears",		da.getNewState().getCloakedToPlayerNames()[0]);
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(1.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
        //
        // CRAFT1 FIRES SHIELD DRAIN AT CRAFT2 - CRAFT1 IS CLOAKED AND CRAFT3 CAN HAVE SHIELDS RECHARGED
        //
        
        clearEvents();
        bca[0].rechargeWeapons();
        bca[1].getShields()[0].setPower(10);
        bca[2].getShields()[0].setPower(0);
        bca[0].fireAllWeapons(bca[1]);
        
		wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_shieldDrain.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), 		wfe.getSource());
		assertEquals(craft2.getName(), 		wfe.getTarget());
		assertEquals(500,500, 				wfe.getSourceLocation(null));
		assertEquals(600,500, 				wfe.getTargetLocation(null));
		assertEquals(500,500, 				wfe.getOldSourceState().getLocation());
		assertEquals(500,500, 				wfe.getNewSourceState().getLocation());
		assertEquals(90, 					wfe.getOldSourceState().getDirection());
		assertEquals(90, 					wfe.getNewSourceState().getDirection());
		assertEquals(0.0, 					wfe.getOldSourceState().getDamagePercent());
		assertEquals(0.0, 					wfe.getNewSourceState().getDamagePercent());
		assertEquals(600,500, 				wfe.getOldTargetState().getLocation());
		assertEquals(600,500, 				wfe.getNewTargetState().getLocation());
		assertEquals(0.0, 					wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.0, 					wfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(blue,					wfe.getOldSourceState().getPlayerColor());
		assertEquals(blue,					wfe.getNewSourceState().getPlayerColor());
		assertEquals(green,					wfe.getOldTargetState().getPlayerColor());
		assertEquals(green,					wfe.getNewTargetState().getPlayerColor());
		assertEquals((String)null,			wfe.getOldTargetState().getCloakedToPlayerNames());
		assertEquals((String)null,			wfe.getNewTargetState().getCloakedToPlayerNames());
		assertEquals("Bears",				wfe.getOldSourceState().getCloakedToPlayerNames()[0]);
		assertEquals("Bears",				wfe.getNewSourceState().getCloakedToPlayerNames()[0]);
		assertEquals(0.0, 					wfe.getOldTargetState().getPsiDamagePercent());
		assertEquals(0.0,					wfe.getNewTargetState().getPsiDamagePercent());
		assertEquals(1.0, 					wfe.getOldTargetState().getShieldPercent());
		assertEquals(0.5,					wfe.getNewTargetState().getShieldPercent());
		assertEquals(1.0, 					wfe.getOldTargetState().getCrewPercent());
		assertEquals(1.0,					wfe.getNewTargetState().getCrewPercent());
		assertEquals(null, 					wfe.getOldTargetState().getDecorators());
		assertEquals(null,					wfe.getNewTargetState().getDecorators());
		assertEquals(0.0, 					wfe.getOldTargetState().getMovementDamagePercent());
		assertEquals(0.0,					wfe.getNewTargetState().getMovementDamagePercent());
		assertEquals(true,					wfe.isReverseAnimation());
		assertNull(							wfe.getSourceDescription());
		assertEquals("-5.0",				wfe.getTargetDescription());
		links = wfe.getLinkedEvents();
		BattleWeaponFireEvent lwfe = (BattleWeaponFireEvent) links[0];
		assertEquals(tech_shieldDrain.getName(), lwfe.getWeapon());
		assertEquals(craft1.getName(), 		lwfe.getSource());
		assertEquals(500,500, 				lwfe.getSourceLocation(null));
		assertEquals(500,500, 				lwfe.getOldSourceState().getLocation());
		assertEquals(500,500,				lwfe.getNewSourceState().getLocation());
		assertEquals(90, 					lwfe.getOldSourceState().getDirection());
		assertEquals(90, 					lwfe.getNewSourceState().getDirection());
		assertEquals(0.0, 					lwfe.getOldSourceState().getDamagePercent());
		assertEquals(0.0, 					lwfe.getNewSourceState().getDamagePercent());
		assertEquals(blue,					lwfe.getOldSourceState().getPlayerColor());
		assertEquals(blue,					lwfe.getNewSourceState().getPlayerColor());
		assertEquals("Bears",				lwfe.getOldSourceState().getCloakedToPlayerNames()[0]);
		assertEquals("Bears",				lwfe.getNewSourceState().getCloakedToPlayerNames()[0]);
		assertEquals(craft3.getName(), 		lwfe.getTarget());
		assertEquals(550,500, 				lwfe.getTargetLocation(null));
		assertEquals(550,500, 				lwfe.getOldTargetState().getLocation());
		assertEquals(550,500, 				lwfe.getNewTargetState().getLocation());
		assertEquals(0.0, 					lwfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.0, 					lwfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(blue,					lwfe.getOldTargetState().getPlayerColor());
		assertEquals(blue,					lwfe.getNewTargetState().getPlayerColor());
		assertEquals((String)null,			lwfe.getOldTargetState().getCloakedToPlayerNames());
		assertEquals((String)null,			lwfe.getNewTargetState().getCloakedToPlayerNames());
		assertEquals(0.0, 					lwfe.getOldTargetState().getPsiDamagePercent());
		assertEquals(0.0,					lwfe.getNewTargetState().getPsiDamagePercent());
		assertEquals(0.0, 					lwfe.getOldTargetState().getShieldPercent());
		assertEquals(0.5,					lwfe.getNewTargetState().getShieldPercent());
		assertEquals(1.0, 					lwfe.getOldTargetState().getCrewPercent());
		assertEquals(1.0,					lwfe.getNewTargetState().getCrewPercent());
		assertEquals(null, 					lwfe.getOldTargetState().getDecorators());
		assertEquals(null,					lwfe.getNewTargetState().getDecorators());
		assertEquals(0.0, 					lwfe.getOldTargetState().getMovementDamagePercent());
		assertEquals(0.0,					lwfe.getNewTargetState().getMovementDamagePercent());
		assertEquals(false,					lwfe.isReverseAnimation());
		assertNull(							lwfe.getSourceDescription());
		assertEquals("+5.0",				lwfe.getTargetDescription());
		
		ac.getAnimations().stopAll();
		processor.process(wfe, null);
		baa = ac.getAnimations().getAllAnimations();
		assertEquals(4, baa.length);
		
		animation = (WeaponFireAnimation) baa[0];
		assertEquals(new Location(596,500), animation.getSource()[0]);
		assertEquals(new Location(500,500), animation.getDestination());
		assertEquals(c1.getName(), animation.getAgressor().getName());
		assertEquals(c2.getName(), animation.getTarget().getName());
		assertEquals(null, animation.getLinkedAnimations());
		
		da = (DamageAnimation) baa[1];
		assertEquals("-5.0", 		da.getMessage());
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals(600,500, 		da.getOldState().getLocation());
		assertEquals(600,500, 		da.getNewState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.001);
		assertEquals(0.0, 			da.getNewState().getDamagePercent(), 0.001);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals((String)null,	da.getOldState().getCloakedToPlayerNames());
		assertEquals((String)null,	da.getNewState().getCloakedToPlayerNames());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(0.5,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		
		//DelayAnimation delay = (DelayAnimation) baa[2];
		
		animation = (WeaponFireAnimation) baa[2];
		assertEquals(new Location(505,500), animation.getSource()[0]);
		assertEquals(new Location(550,500), animation.getDestination());
		assertEquals(c1.getName(), animation.getAgressor().getName());
		assertEquals(c3.getName(), animation.getTarget().getName());
		assertEquals(null, animation.getLinkedAnimations());

		da = (DamageAnimation) baa[3];
		assertEquals("+5.0", 		da.getMessage());
		assertEquals(c3.getName(), 	da.getTarget().getName());
		assertEquals(550,500, 		da.getOldState().getLocation());
		assertEquals(550,500, 		da.getNewState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.001);
		assertEquals(0.0, 			da.getNewState().getDamagePercent(), 0.001);
		assertEquals(blue,			da.getOldState().getPlayerColor());
		assertEquals(blue,			da.getNewState().getPlayerColor());
		assertEquals((String)null,	da.getOldState().getCloakedToPlayerNames());
		assertEquals((String)null,	da.getNewState().getCloakedToPlayerNames());
		assertEquals(0.0, 			da.getOldState().getShieldPercent());
		assertEquals(0.5,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		
    }     

    public void testWeapons_Immobiliser_Psi_Bio_Plague_Animation() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testImmobiliserVirusAnimation() *************************");
        CraftDesign attackerDesign = new CraftDesign("TestAttacker", tech_flyer30WithTwoOffsets, tech_fusion25, tech_immobiliser, tech_psiBlast, tech_prioVirus, tech_plague);
        createMap(attackerDesign,coloniserDesign, false);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(550, 500));
        bca[1].setLocation(new Location(600, 500));

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), crafts[0].getName());
        assertEquals(c2.getName(), crafts[1].getName());
        
        //
        // CRAFT1 FIRES IMMOBILISER AT CRAFT2
        //
        
        clearEvents();
        serverProxy.battleAttackCmd(battle, crafts[0], crafts[1], false, false, false, null, tech_immobiliser, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        callbackSync.waitTillComplete();
        
		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_immobiliser.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), wfe.getSource());
		assertEquals(craft2.getName(), wfe.getTarget());
		assertEquals(new Location(550,500), wfe.getSourceLocation(null));
		assertEquals(new Location(600,500), wfe.getTargetLocation(null));
		assertEquals(new Location(600,500), wfe.getOldTargetState().getLocation());
		assertEquals(new Location(600,500), wfe.getNewTargetState().getLocation());
		assertEquals(0.0, wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.0, wfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(0.0, wfe.getOldTargetState().getMovementDamagePercent(), 0.001);
		assertEquals(0.31, wfe.getNewTargetState().getMovementDamagePercent(), 0.1);
		assertNull(wfe.getSourceDescription());
		assertEquals("Engine Damage +25", wfe.getTargetDescription());
		
		BattleAnimationList al = new BattleAnimationList();
		
		WeaponFireAnimationFactory.create(wfe, null, al);
		DamageAnimationFactory.create(wfe, null, al);
		
		BattleAnimation[] aa = al.getAllAnimations();
		assertEquals(2, aa.length);
		
		WeaponFireAnimation wfa = (WeaponFireAnimation) aa[0];
		assertEquals(1, wfa.getSource().length);
		assertEquals(new Location(556,500), wfa.getSource()[0]);
		assertEquals(new Location(600,500), wfa.getDestination());
		
		DamageAnimation da = (DamageAnimation) aa[1];
		assertEquals("Engine Damage +25", da.getMessage());
		assertEquals(crafts[1].getName(), da.getTarget().getName());
		assertEquals(600,500, da.getOldState().getLocation());
		assertEquals(600,500, da.getNewState().getLocation());
		assertEquals(0.0, da.getOldState().getMovementDamagePercent(), 0.001);
		assertEquals(0.31, da.getNewState().getMovementDamagePercent(), 0.1);

    
        //
        // CRAFT1 FIRES PSI BLAST AT CRAFT2
        //
        
        clearEvents();
        serverProxy.battleAttackCmd(battle, crafts[0], crafts[1], false, false, false, null, tech_psiBlast, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        callbackSync.waitTillComplete();
        
		wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_psiBlast.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), wfe.getSource());
		assertEquals(craft2.getName(), wfe.getTarget());
		assertEquals(new Location(550,500), wfe.getSourceLocation(null));
		assertEquals(new Location(600,500), wfe.getTargetLocation(null));
		assertEquals(new Location(600,500), wfe.getOldTargetState().getLocation());
		assertEquals(new Location(600,500), wfe.getNewTargetState().getLocation());
		assertEquals(0.0, wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.0, wfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(0.3125, wfe.getOldTargetState().getMovementDamagePercent(), 0.001);
		assertEquals(0.3125, wfe.getNewTargetState().getMovementDamagePercent(), 0.001);
		assertEquals(0.0, wfe.getOldTargetState().getPsiDamagePercent(), 0.001);
		assertEquals(0.02, wfe.getNewTargetState().getPsiDamagePercent(), 0.001);
		assertNull(wfe.getSourceDescription());
		assertEquals("Psi Damage +1.0", wfe.getTargetDescription());
		
		al = new BattleAnimationList();
		
		WeaponFireAnimationFactory.create(wfe, null, al);
		DamageAnimationFactory.create(wfe, null, al);
		
		aa = al.getAllAnimations();
		assertEquals(2, aa.length);
		
		wfa = (WeaponFireAnimation) aa[0];
		assertEquals(2, wfa.getSource().length);
		assertEquals(new Location(555,505), wfa.getSource()[0]);
		assertEquals(new Location(555,495), wfa.getSource()[1]);
		assertEquals(new Location(600,500), wfa.getDestination());
		
		da = (DamageAnimation) aa[1];
		assertEquals("Psi Damage +1.0", da.getMessage());
		assertEquals(crafts[1].getName(), da.getTarget().getName());
		assertEquals(600,500, da.getOldState().getLocation());
		assertEquals(600,500, da.getNewState().getLocation());
		assertEquals(0.0, da.getOldState().getPsiDamagePercent(), 0.001);
		assertEquals(0.02, da.getNewState().getPsiDamagePercent(), 0.001);

        //
        // CRAFT1 FIRES BIO WEAPON AT CRAFT2
        //
        
        clearEvents();
        serverProxy.battleAttackCmd(battle, crafts[0], crafts[1], false, false, false, null, tech_prioVirus, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        callbackSync.waitTillComplete();
        
		wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_prioVirus.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), wfe.getSource());
		assertEquals(craft2.getName(), wfe.getTarget());
		assertEquals(new Location(550,500), wfe.getSourceLocation(null));
		assertEquals(new Location(600,500), wfe.getTargetLocation(null));
		assertEquals(new Location(600,500), wfe.getOldTargetState().getLocation());
		assertEquals(new Location(600,500), wfe.getNewTargetState().getLocation());
		assertEquals(0.0, wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.0, wfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(0.31, wfe.getOldTargetState().getMovementDamagePercent(), 0.1);
		assertEquals(0.31, wfe.getNewTargetState().getMovementDamagePercent(), 0.1);
		assertEquals(1.0, wfe.getOldTargetState().getCrewPercent(), 0.1);
		assertEquals(0.5, wfe.getNewTargetState().getCrewPercent(), 0.1);
		assertNull(wfe.getSourceDescription());
		assertEquals("killed 5 (5)", wfe.getTargetDescription());
		
		al = new BattleAnimationList();
		
		WeaponFireAnimationFactory.create(wfe, null, al);
		DamageAnimationFactory.create(wfe, null, al);
		
		aa = al.getAllAnimations();
		assertEquals(2, aa.length);
		
		wfa = (WeaponFireAnimation) aa[0];
		assertEquals(1, wfa.getSource().length);
		assertEquals(new Location(556,500), wfa.getSource()[0]);
		assertEquals(new Location(600,500), wfa.getDestination());
		
		da = (DamageAnimation) aa[1];
		assertEquals("killed 5 (5)", da.getMessage());
		assertEquals(crafts[1].getName(), da.getTarget().getName());
		assertEquals(600,500, da.getOldState().getLocation());
		assertEquals(600,500, da.getNewState().getLocation());
		assertEquals(0.0,  da.getOldState().getDamagePercent(), 0.001);
		assertEquals(0.0,  da.getNewState().getDamagePercent(), 0.001);
		assertEquals(0.31, da.getOldState().getMovementDamagePercent(), 0.1);
		assertEquals(0.31, da.getNewState().getMovementDamagePercent(), 0.1);
		assertEquals(1.0,  da.getOldState().getCrewPercent(), 0.1);
		assertEquals(0.5,  da.getNewState().getCrewPercent(), 0.1);

        //
        // CRAFT1 FIRES PLAGUE AT CRAFT2
        //
        
        clearEvents();
        serverProxy.battleAttackCmd(battle, crafts[0], crafts[1], false, false, false, null, tech_plague, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        callbackSync.waitTillComplete();
        
		wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_plague.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), wfe.getSource());
		assertEquals(craft2.getName(), wfe.getTarget());
		assertEquals(new Location(550,500), wfe.getSourceLocation(null));
		assertEquals(new Location(600,500), wfe.getTargetLocation(null));
		assertEquals(new Location(600,500), wfe.getOldTargetState().getLocation());
		assertEquals(new Location(600,500), wfe.getNewTargetState().getLocation());
		assertEquals(0.0, wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(0.0, wfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(0.31, wfe.getOldTargetState().getMovementDamagePercent(), 0.1);
		assertEquals(0.31, wfe.getNewTargetState().getMovementDamagePercent(), 0.1);
		assertEquals(0.5, wfe.getOldTargetState().getCrewPercent(), 0.1);
		assertEquals(0.5, wfe.getNewTargetState().getCrewPercent(), 0.1);
		assertNull(wfe.getSourceDescription());
		assertEquals("Plague +5", wfe.getTargetDescription());
		
		al = new BattleAnimationList();
		
		WeaponFireAnimationFactory.create(wfe, null, al);
		DamageAnimationFactory.create(wfe, null, al);
		
		aa = al.getAllAnimations();
		assertEquals(2, aa.length);
		
		wfa = (WeaponFireAnimation) aa[0];
		assertEquals(1, wfa.getSource().length);
		assertEquals(new Location(556,500), wfa.getSource()[0]);
		assertEquals(new Location(600,500), wfa.getDestination());
		
		da = (DamageAnimation) aa[1];
		assertEquals("Plague +5", da.getMessage());
		assertEquals(crafts[1].getName(), da.getTarget().getName());
		assertEquals(600,500, da.getOldState().getLocation());
		assertEquals(600,500, da.getNewState().getLocation());
		assertEquals(0.0,  da.getOldState().getDamagePercent(), 0.001);
		assertEquals(0.0,  da.getNewState().getDamagePercent(), 0.001);
		assertEquals(0.31, da.getOldState().getMovementDamagePercent(), 0.1);
		assertEquals(0.31, da.getNewState().getMovementDamagePercent(), 0.1);
		assertEquals(0.5,  da.getOldState().getCrewPercent(), 0.1);
		assertEquals(0.5,  da.getNewState().getCrewPercent(), 0.1);

    }     
    
    public void testCraftDestroyed() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testCraftDestroyed() *************************");
        CraftDesign fusionFighter = new CraftDesign("FusionFighter", tech_flyer30WithTwoOffsets, tech_fusion25, tech_electronic25, tech_energy10, tech_fusionBeam4);
        createMap(fusionFighter,fighterDesign, false);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 FIRES FUSION BEAM AT CRAFT2 AND DESTROYES IT
        //
		
        clearEvents();
        bca[1].getShields()[0].setPower(0.0);
        bca[1].getCraft().setDamage(19);
        bca[0].fireWeapons(bca[1], null, -1);
        
		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(0);
		assertEquals(tech_fusionBeam4.getName(),	wfe.getWeapon());
		assertEquals(craft1.getName(), 		wfe.getSource());
		assertEquals(craft2.getName(), 		wfe.getTarget());
		assertEquals(500,500, 				wfe.getSourceLocation(null));
		assertEquals(600,500, 				wfe.getTargetLocation(null));
		assertEquals(500,500, 				wfe.getOldSourceState().getLocation());
		assertEquals(90, 					wfe.getOldSourceState().getDirection());
		assertEquals(0.0, 					wfe.getOldSourceState().getDamagePercent(), 0.01);
		assertEquals(1.0, 					wfe.getOldSourceState().getShieldPercent());
		assertEquals(600,500, 				wfe.getOldTargetState().getLocation());
		assertEquals(270, 					wfe.getOldTargetState().getDirection());
		assertEquals(0.95, 					wfe.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0, 					wfe.getOldTargetState().getShieldPercent());
		assertEquals(500,500, 				wfe.getNewSourceState().getLocation());
		assertEquals(90, 					wfe.getNewSourceState().getDirection());
		assertEquals(0.0, 					wfe.getNewSourceState().getDamagePercent());
		assertEquals(1.0, 					wfe.getNewSourceState().getShieldPercent());
		assertEquals(600,500, 				wfe.getNewTargetState().getLocation());
		assertEquals(270, 					wfe.getNewTargetState().getDirection());
		assertEquals(1.0, 					wfe.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					wfe.getNewTargetState().getShieldPercent());
		assertNull(							wfe.getSourceDescription());
		assertEquals("3.8",					wfe.getTargetDescription());
		MotEvent[] links = wfe.getLinkedEvents();
		assertEquals(null,					links);
		assertEquals(null,					wfe.getData());
		
		BattleDestroyedEvent de = (BattleDestroyedEvent) p1Proxy.getEvent(1);
		assertEquals(craft2.getName(), 		de.getTarget());
		assertEquals(600,500, 				de.getLocation(null));
		assertEquals(600,500, 				de.getLocation(null));
		assertEquals(600,500, 				de.getOldState().getLocation());
		assertEquals(270, 					de.getOldState().getDirection());
		assertEquals(0.95, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(0.0,					de.getOldState().getShieldPercent());

		ac.getAnimations().stopAll();
		processor.process(wfe, null);
		processor.process(de, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(3, baa.length);
		
		WeaponFireAnimation animation = (WeaponFireAnimation) baa[0];
		assertEquals(new Location(505,505), animation.getSource()[0]);
		assertEquals(new Location(505,495), animation.getSource()[1]);
		assertEquals(new Location(600,500), animation.getDestination());
		assertEquals(c1.getName(), animation.getAgressor().getName());
		assertEquals(c2.getName(), animation.getTarget().getName());
		assertEquals(null, animation.getLinkedAnimations());
		
		DamageAnimation da = (DamageAnimation) baa[1];
		assertEquals("3.8", 		da.getMessage());
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals(600,500, 		da.getOldState().getLocation());
		assertEquals(0.95, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(0.0, 			da.getOldState().getShieldPercent());
		assertEquals(600,500, 		da.getNewState().getLocation());
		assertEquals(1.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		ExplodeAnimation ea = (ExplodeAnimation) baa[2];
		assertEquals(600,500, 		ea.getImage().getLocation());
		
    }     
    

    public void testTroopBattle() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testTroopBattle() *************************");
        CraftDesign troopFighter = new CraftDesign("TroopFighter", tech_flyer30WithOneOffset, tech_fusion25, tech_electronic25, tech_energy10, tech_troopTransport, tech_troopTransport, tech_troopModule20);
        createMap(troopFighter,fighterDesign, false);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 FIRES TROOP TRANSPORT AT CRAFT2
        //
		
        clearEvents();
        bca[1].getShields()[0].setPower(0.0);
        Utils.clearAndSetNextRandomDouble(0.6);
        bca[0].fireAllWeapons(bca[1]);
        
		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(BattleWeaponFireEvent.class);
		assertEquals(tech_troopTransport.getName(), 	wfe.getWeapon());
		assertEquals(craft1.getName(), 		wfe.getSource());
		assertEquals(craft2.getName(), 		wfe.getTarget());
		assertEquals(500,500, 				wfe.getSourceLocation(null));
		assertEquals(600,500, 				wfe.getTargetLocation(null));
		assertEquals(500,500, 				wfe.getOldSourceState().getLocation());
		assertEquals(90, 					wfe.getOldSourceState().getDirection());
		assertEquals(0.0, 					wfe.getOldSourceState().getDamagePercent());
		assertEquals(blue,					wfe.getOldSourceState().getPlayerColor());
		assertEquals(1.0, 					wfe.getOldSourceState().getShieldPercent());
		assertEquals(1.0, 					wfe.getOldSourceState().getCrewPercent());
		assertEquals(600,500, 				wfe.getOldTargetState().getLocation());
		assertEquals(0.0, 					wfe.getOldTargetState().getDamagePercent(), 0.001);
		assertEquals(green,					wfe.getOldTargetState().getPlayerColor());
		assertEquals(0.0, 					wfe.getOldTargetState().getShieldPercent());
		assertEquals(1.0, 					wfe.getOldTargetState().getCrewPercent());
		assertEquals(null, 					wfe.getOldTargetState().getDecorators());
		assertEquals(500,500, 				wfe.getNewSourceState().getLocation());
		assertEquals(90, 					wfe.getNewSourceState().getDirection());
		assertEquals(0.0, 					wfe.getNewSourceState().getDamagePercent());
		assertEquals(blue,					wfe.getNewSourceState().getPlayerColor());
		assertEquals(1.0, 					wfe.getNewSourceState().getShieldPercent());
		assertEquals(1.0, 					wfe.getNewSourceState().getCrewPercent());
		assertEquals(600,500, 				wfe.getNewTargetState().getLocation());
		assertEquals(0.0, 					wfe.getNewTargetState().getDamagePercent(), 0.001);
		assertEquals(blue,					wfe.getNewTargetState().getPlayerColor());
		assertEquals(0.0,					wfe.getNewTargetState().getShieldPercent());
		assertEquals(0.2,					wfe.getNewTargetState().getCrewPercent());
		assertEquals(null,					wfe.getNewTargetState().getDecorators());
		assertEquals(false,					wfe.isReverseAnimation());
		assertEquals("Crew -20",			wfe.getSourceDescription());
		assertEquals("Success",				wfe.getTargetDescription());
		MotEvent[] links = wfe.getLinkedEvents();
		assertEquals(null,					links);
		
		TroopBattleResult data = (TroopBattleResult) wfe.getData();
		assertEquals(20,		data.getQtyAttackers());
		assertEquals(10,		data.getQtyDefenders());
		assertEquals(2,			data.getRemainingAttackers());
		assertEquals(0,			data.getRemainingDefenders());
		
		ac.getAnimations().stopAll();
		processor.process(wfe, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(4, baa.length);
		
		WeaponFireAnimation animation = (WeaponFireAnimation) baa[0];
		assertEquals(new Location(505,500), animation.getSource()[0]);
		assertEquals(new Location(600,500), animation.getDestination());
		assertEquals(c1.getName(), animation.getAgressor().getName());
		assertEquals(c2.getName(), animation.getTarget().getName());
		assertEquals(null, animation.getLinkedAnimations());
		
		TroopBattleAnimation tb = (TroopBattleAnimation) baa[1];
		assertEquals(data, 		tb.getTbr());
		
		DamageAnimation da = (DamageAnimation) baa[2];
		assertEquals("Success", 	da.getMessage());
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals(600,500, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(0.0, 			da.getOldState().getShieldPercent());
		assertEquals(600,500, 		da.getNewState().getLocation());
		assertEquals(0.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		

		da = (DamageAnimation) baa[3];
		assertEquals("Crew -20", 	da.getMessage());
		assertEquals(c1.getName(), 	da.getTarget().getName());
		assertEquals(500,500, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(500,500, 		da.getNewState().getLocation());
		assertEquals(0.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getNewState().getPlayerColor());
		assertEquals(1.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
    }     

    public void testMineExplosion() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testMineExplosion() *************************");
        createMap(fighterDesign,fighterDesign, false);
        
		Craft c3 = newCraft(fighterDesign, p1, ss1);
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter3");
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.craftAttackCmd(craft3, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        Battle activeBattle = MotEngine.instance.getBattles().getActiveBattle();
		BattleCraft [] bca = activeBattle.getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(610, 500));
        bca[2].setLocation(new Location(300, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        
        BattleMineFactory.create(p1, activeBattle, 50, 1);
        BattleMine mine = (BattleMine) activeBattle.getAutomatons()[0];
        
        mine.setLocation(new Location(550,500));
        MotEngine.instance.getBattles().getActiveBattle().sendStateUpdateEvents(p1, false, true, false);
        serverProxy._processQueuedEvents();
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 TRIGGERS A MINE FIELD THAT DESTROYES IT AND DAMAGES CRAFT2
        //
		
        clearEvents();
        mine.getAlarm().activate(bca[0]);
        serverProxy._processQueuedEvents();
        
        BattleDamageRingEvent dre = (BattleDamageRingEvent) p1Proxy.getEvent(0);
		assertEquals(mine.getName(),		dre.getSource());
		assertEquals(75, 					dre.getDamage());
		MotEvent[] le = dre.getLinkedEvents();
		assertEquals(3, 					le.length);
		
		BattleDestroyedEvent de = (BattleDestroyedEvent) le[0];
		assertEquals(craft1.getName(), 		de.getTarget());
		assertEquals(500,500, 				de.getLocation(null));
		assertEquals(500,500, 				de.getOldState().getLocation());
		assertEquals(90, 					de.getOldState().getDirection());
		assertEquals(0.0, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(1.0,					de.getOldState().getShieldPercent());

		BattleDamageEvent dae = (BattleDamageEvent) le[1];
		assertEquals(craft1.getName(), 		dae.getTarget());
		assertEquals(500,500, 				dae.getOldTargetState().getLocation());
		assertEquals(90, 					dae.getOldTargetState().getDirection());
		assertEquals(0.0, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(500,500, 				dae.getNewTargetState().getLocation());
		assertEquals(90, 					dae.getNewTargetState().getDirection());
		assertEquals(1.0, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent());
		assertEquals(true,					dae.getDestroyed());

		dae = (BattleDamageEvent) le[2];
		assertEquals(craft2.getName(), 		dae.getTarget());
		assertEquals(610,500, 				dae.getOldTargetState().getLocation());
		assertEquals(270, 					dae.getOldTargetState().getDirection());
		assertEquals(0.0, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(610,500, 				dae.getNewTargetState().getLocation());
		assertEquals(270, 					dae.getNewTargetState().getDirection());
		assertEquals(0.7, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent());
		assertEquals(false,					dae.getDestroyed());

		ac.getAnimations().stopAll();
		processor.process(dre, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(7, baa.length);
		
		DelayAnimation delay1 = (DelayAnimation) baa[0];
		
		FlashAnimation fa = (FlashAnimation) baa[1];
		assertEquals(mine.getName(), 	fa.getTarget().getName());
		assertEquals(550,500, 			fa.getTarget().getLocation());

		WarpCoreOverloadAnimation wco = (WarpCoreOverloadAnimation) baa[2];
		assertEquals(mine.getName(), 			wco.getSource().getName());
		assertEquals(550,500, 					wco.getSource().getLocation());
		
		ExplodeAnimation ea = (ExplodeAnimation) baa[3];
		assertEquals(500,500, 		ea.getImage().getLocation());
		
		DamageAnimation da = (DamageAnimation) baa[4];
		assertEquals(c1.getName(), 	da.getTarget().getName());
		assertEquals("75.0",	 	da.getMessage());
		assertEquals(500,500, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(500,500, 		da.getNewState().getLocation());
		assertEquals(1.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		da = (DamageAnimation) baa[5];
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals("75.0",		da.getMessage());
		assertEquals(610,500, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(610,500, 		da.getNewState().getLocation());
		assertEquals(0.7, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		DelayAnimation delay2 = (DelayAnimation) baa[6];
		
    }     
    

    public void testShieldBurst() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testShieldBurst() *************************");
        CraftDesign burstFighter = new CraftDesign("BurstFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_shieldBurst);
        createMap(burstFighter,fighterDesign, false);
        
		Craft c3 = newCraft(fighterDesign, p2, ss1);
		Craft c4 = newCraft(fighterDesign, p2, ss1);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter2");
        CraftDelegate craft4 = DeligateCacheManager.getCraft("TestFighter3");
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        Battle activeBattle = MotEngine.instance.getBattles().getActiveBattle();
		BattleCraft [] bca = activeBattle.getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(530, 490));
        bca[2].setLocation(new Location(530, 515));
        bca[3].setLocation(new Location(500, 470));
        bca[0].setDirection(90);
        bca[1].setDirection(270);
        bca[2].setDirection(270);
        bca[3].setDirection(180);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        assertEquals(c3.getName(), bca[2].getName());
        assertEquals(c4.getName(), bca[3].getName());
        
        bca[3].getCraft().setDamage(bca[3].getCraft().getMaxDamage()-2);
        bca[3].setShieldPower(0, 1);
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 TRIGGERS A MINE FIELD THAT DESTROYES IT AND DAMAGES CRAFT2
        //
		
        clearEvents();
        bca[0].fireWeapons(bca[1], null, -1);
        
        BattleDamageRingEvent dre = (BattleDamageRingEvent) p1Proxy.getEvent(0);
		assertEquals(craft1.getName(),		dre.getSource());
		assertEquals(10, 					dre.getDamage());
		MotEvent[] le = dre.getLinkedEvents();
		assertEquals(4, 					le.length);
		
		BattleDamageEvent dae = (BattleDamageEvent) le[0];
		assertEquals(craft2.getName(), 		dae.getTarget());
		assertEquals(530,490, 				dae.getOldTargetState().getLocation());
		assertEquals(270, 					dae.getOldTargetState().getDirection());
		assertEquals(0.0, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(530,490, 				dae.getNewTargetState().getLocation());
		assertEquals(270, 					dae.getNewTargetState().getDirection());
		assertEquals(0.45, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent());
		assertEquals(false,					dae.getDestroyed());

		dae = (BattleDamageEvent) le[1];
		assertEquals(craft3.getName(), 		dae.getTarget());
		assertEquals(530,515, 				dae.getOldTargetState().getLocation());
		assertEquals(270, 					dae.getOldTargetState().getDirection());
		assertEquals(0.0, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(530,515, 				dae.getNewTargetState().getLocation());
		assertEquals(270, 					dae.getNewTargetState().getDirection());
		assertEquals(0.45, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent());
		assertEquals(false,					dae.getDestroyed());

		BattleDestroyedEvent de = (BattleDestroyedEvent) le[2];
		assertEquals(craft4.getName(), 		de.getTarget());
		assertEquals(500,470, 				de.getOldState().getLocation());
		assertEquals(180, 					de.getOldState().getDirection());
		assertEquals(0.9, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(0.1,					de.getOldState().getShieldPercent());
		
		dae = (BattleDamageEvent) le[3];
		assertEquals(craft4.getName(), 		dae.getTarget());
		assertEquals(500,470, 				dae.getOldTargetState().getLocation());
		assertEquals(180, 					dae.getOldTargetState().getDirection());
		assertEquals(0.9, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(0.1,					dae.getOldTargetState().getShieldPercent());
		assertEquals(500,470, 				dae.getNewTargetState().getLocation());
		assertEquals(180, 					dae.getNewTargetState().getDirection());
		assertEquals(1.0, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent());
		assertEquals(true,					dae.getDestroyed());

		ac.getAnimations().stopAll();
		processor.process(dre, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(7, baa.length);

		DelayAnimation delay1 = (DelayAnimation) baa[0];
		
		ShieldBurstAnimation sba = (ShieldBurstAnimation) baa[1];		
		assertEquals(craft1.getName(),		sba.getSource().getName());
		assertEquals(null,					sba.getLinkedAnimations());
		
		DamageAnimation da = (DamageAnimation) baa[2];
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals("10.0", 		da.getMessage());
		assertEquals(530,490, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(530,490, 		da.getNewState().getLocation());
		assertEquals(0.45, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		da = (DamageAnimation) baa[3];
		assertEquals(c3.getName(), 	da.getTarget().getName());
		assertEquals("10.0", 		da.getMessage());
		assertEquals(530,515, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(530,515, 		da.getNewState().getLocation());
		assertEquals(0.45, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		ExplodeAnimation ea = (ExplodeAnimation) baa[4];
		assertEquals(500,470, 		ea.getImage().getLocation());
		
		da = (DamageAnimation) baa[5];
		assertEquals(c4.getName(), 	da.getTarget().getName());
		assertEquals("10.0", 		da.getMessage());
		assertEquals(500,470, 		da.getOldState().getLocation());
		assertEquals(0.9, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(0.1, 			da.getOldState().getShieldPercent());
		assertEquals(500,470, 		da.getNewState().getLocation());
		assertEquals(1.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		DelayAnimation delay2 = (DelayAnimation) baa[6];
		
    }     
    
    public void testPulseWeapon() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testPulseWeapon() *************************");
        CraftDesign pulseFighterDesign = new CraftDesign("PulseFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_pulse);
        createMap(pulseFighterDesign,fighterDesign, false);
        
		Craft c3 = newCraft(fighterDesign, p2, ss1);
		Craft c4 = newCraft(fighterDesign, p2, ss1);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter2");
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        Battle activeBattle = MotEngine.instance.getBattles().getActiveBattle();
		BattleCraft [] bca = activeBattle.getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(550, 460));
        bca[2].setLocation(new Location(550, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);
        bca[2].setDirection(90);
        
        bca[2].getCraft().setDamage(bca[3].getCraft().getMaxDamage()-2);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        assertEquals(c3.getName(), bca[2].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 Fires Pulse weapon damaging craft c2 and destroying craft c3 
        //
		
        clearEvents();
        bca[0].fireWeapons(bca[1], null, -1);
        
        BattlePulseEvent pe = (BattlePulseEvent) p1Proxy.getEvent(0);
		assertEquals(craft1.getName(),		pe.getSource());
		assertEquals(500,500, 				pe.getOldState().getLocation());
		assertEquals(0.0, 					pe.getOldState().getDamagePercent());
		assertEquals(500,500, 				pe.getNewState().getLocation());
		assertEquals(0.0, 					pe.getNewState().getDamagePercent());
		MotEvent[] le = pe.getLinkedEvents();
		assertEquals(3, 					le.length);
		
		BattleDamageEvent dae = (BattleDamageEvent) le[0];
		assertEquals(craft2.getName(), 		dae.getTarget());
		assertEquals(550,460, 				dae.getOldTargetState().getLocation());
		assertEquals(270, 					dae.getOldTargetState().getDirection());
		assertEquals(0.0, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(550,460, 				dae.getNewTargetState().getLocation());
		assertEquals(270, 					dae.getNewTargetState().getDirection());
		assertEquals(0.45, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent());
		assertEquals(false,					dae.getDestroyed());

		dae = (BattleDamageEvent) le[1];
		assertEquals(craft3.getName(), 		dae.getTarget());
		assertEquals(550,500, 				dae.getOldTargetState().getLocation());
		assertEquals(90, 					dae.getOldTargetState().getDirection());
		assertEquals(0.9, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(550,500, 				dae.getNewTargetState().getLocation());
		assertEquals(90, 					dae.getNewTargetState().getDirection());
		assertEquals(1.0, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent());
		assertEquals(true,					dae.getDestroyed());

		BattleDestroyedEvent de = (BattleDestroyedEvent) le[2];
		assertEquals(craft3.getName(), 		de.getTarget());
		assertEquals(550,500, 				de.getOldState().getLocation());
		assertEquals(90, 					de.getOldState().getDirection());
		assertEquals(0.9, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(0.0,					de.getOldState().getShieldPercent());
		
		ac.getAnimations().stopAll();
		processor.process(pe, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(6, baa.length);

		DelayAnimation delay1 = (DelayAnimation) baa[0];
		
		PulseAnimation sba = (PulseAnimation) baa[1];		
		assertEquals(craft1.getName(),		sba.getSource().getName());
		assertEquals(null,					sba.getLinkedAnimations());
		
		DamageAnimation da = (DamageAnimation) baa[2];
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals("9.0", 		da.getMessage());
		assertEquals(550,460, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(550,460, 		da.getNewState().getLocation());
		assertEquals(0.45, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		da = (DamageAnimation) baa[3];
		assertEquals(c3.getName(), 	da.getTarget().getName());
		assertEquals("22.8", 		da.getMessage());
		assertEquals(550,500, 		da.getOldState().getLocation());
		assertEquals(0.9, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(550,500, 		da.getNewState().getLocation());
		assertEquals(1.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		ExplodeAnimation ea = (ExplodeAnimation) baa[4];
		assertEquals(550,500, 		ea.getImage().getLocation());
		
		DelayAnimation delay2 = (DelayAnimation) baa[5];
		
    }     

    public void testPulseWeaponDestroysMine() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testPulseWeaponDestroysMine() *************************");
        CraftDesign pulseFighterDesign = new CraftDesign("PulseFighter", tech_flyer60, tech_fusion25, tech_electronic25, tech_energy10, tech_pulse);
        createMap(pulseFighterDesign,fighterDesign, false);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        Battle activeBattle = MotEngine.instance.getBattles().getActiveBattle();
		BattleCraft [] bca = activeBattle.getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(90);
        
        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        
        BattleMineFactory.create(p1, activeBattle, 30, new Location(550,500));
        BattleMine mine = (BattleMine) activeBattle.getAutomatons()[0];
        
        MotEngine.instance.getBattles().getActiveBattle().sendStateUpdateEvents(p1, false, true, false);
        serverProxy._processQueuedEvents();
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		assertNotNull(DeligateCacheManager.getBattleObject(activeBattle.getName(), mine.getName()));
		
		//
        // CRAFT1 Fires Pulse weapon damaging craft c2 and destroying mine 
        //
		
        clearEvents();
        bca[0].fireWeapons(bca[1], null, -1);
        
        BattlePulseEvent pe = (BattlePulseEvent) p1Proxy.getEvent(0);
		assertEquals(craft1.getName(),		pe.getSource());
		assertEquals(500,500, 				pe.getOldState().getLocation());
		assertEquals(0.0, 					pe.getOldState().getDamagePercent());
		assertEquals(500,500, 				pe.getNewState().getLocation());
		assertEquals(0.0, 					pe.getNewState().getDamagePercent());
		MotEvent[] le = pe.getLinkedEvents();
		assertEquals(2, 					le.length);
		
		BattleDamageEvent dae = (BattleDamageEvent) le[0];
		assertEquals(craft2.getName(), 		dae.getTarget());
		assertEquals(600,500, 				dae.getOldTargetState().getLocation());
		assertEquals(90, 					dae.getOldTargetState().getDirection());
		assertEquals(0.0, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(600,500, 				dae.getNewTargetState().getLocation());
		assertEquals(90, 					dae.getNewTargetState().getDirection());
		assertEquals(0.72, 					dae.getNewTargetState().getDamagePercent(), 0.01); // not right but damage by mine done before final state of Pulse
		assertEquals(0.17,					dae.getNewTargetState().getShieldPercent(), 0.01);
		assertEquals(false,					dae.getDestroyed());

		dae = (BattleDamageEvent) le[1];
		assertEquals(mine.getName(), 		dae.getTarget());
		assertEquals(550,500, 				dae.getOldTargetState().getLocation());
		assertEquals(0, 					dae.getOldTargetState().getDirection());
		assertEquals(0.0, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(550,500, 				dae.getNewTargetState().getLocation());
		assertEquals(0, 					dae.getNewTargetState().getDirection());
		assertEquals(0.0, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent());
		assertEquals(true,					dae.getDestroyed());

		BattleDamageRingEvent dre = (BattleDamageRingEvent) p1Proxy.getEvent(1);
		assertEquals(mine.getName(),		dre.getSource());
		assertEquals(75, 					dre.getDamage());
		le = dre.getLinkedEvents();
		assertEquals(3, 					le.length);
		
		dae = (BattleDamageEvent) le[0];
		assertEquals(craft1.getName(), 		dae.getTarget());
		assertEquals(500,500, 				dae.getOldTargetState().getLocation());
		assertEquals(90, 					dae.getOldTargetState().getDirection());
		assertEquals(0.0, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0,					dae.getOldTargetState().getShieldPercent());
		assertEquals(500,500, 				dae.getNewTargetState().getLocation());
		assertEquals(90, 					dae.getNewTargetState().getDirection());
		assertEquals(0.6, 					dae.getNewTargetState().getDamagePercent(), 0.01); // not right but damage by mine done before new state of Pulse Weapon
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent(), 0.01);
		assertEquals(false,					dae.getDestroyed());
		
		BattleDestroyedEvent de = (BattleDestroyedEvent) le[1];
		assertEquals(craft2.getName(), 		de.getTarget());
		assertEquals(600,500, 				de.getOldState().getLocation());
		assertEquals(90, 					de.getOldState().getDirection());
		assertEquals(0.72, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(0.17,					de.getOldState().getShieldPercent(), 0.01);

		dae = (BattleDamageEvent) le[2];
		assertEquals(craft2.getName(), 		dae.getTarget());
		assertEquals(600,500, 				dae.getOldTargetState().getLocation());
		assertEquals(90, 					dae.getOldTargetState().getDirection());
		assertEquals(0.72, 					dae.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(0.17,					dae.getOldTargetState().getShieldPercent(), 0.01);
		assertEquals(600,500, 				dae.getNewTargetState().getLocation());
		assertEquals(90, 					dae.getNewTargetState().getDirection());
		assertEquals(1.0, 					dae.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					dae.getNewTargetState().getShieldPercent(), 0.01);
		assertEquals(true,					dae.getDestroyed());

		ac.getAnimations().stopAll();
		processor.process(pe, null);
		processor.process(dre, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(12, baa.length);

		DelayAnimation delay1 = (DelayAnimation) baa[0];
		
		PulseAnimation sba = (PulseAnimation) baa[1];		
		assertEquals(craft1.getName(),		sba.getSource().getName());
		assertEquals(null,					sba.getLinkedAnimations());
		
		DamageAnimation da = (DamageAnimation) baa[2];
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals("14.6", 		da.getMessage());
		assertEquals(600,500, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(600,500, 		da.getNewState().getLocation());
		assertEquals(0.72, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.17,			da.getNewState().getShieldPercent(), 0.01);
		assertEquals(null, 			da.getLinkedAnimations());
		
		da = (DamageAnimation) baa[3];
		assertEquals(mine.getName(),da.getTarget().getName());
		assertEquals("10.0", 		da.getMessage());
		assertEquals(550,500, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(550,500, 		da.getNewState().getLocation());
		assertEquals(0.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(null, 			da.getLinkedAnimations());
		
		DelayAnimation delay2 = (DelayAnimation) baa[4];
		DelayAnimation delay3 = (DelayAnimation) baa[5];
		FlashAnimation fa = (FlashAnimation) baa[6];
		WarpCoreOverloadAnimation wcoa = (WarpCoreOverloadAnimation) baa[7];
		
		da = (DamageAnimation) baa[8];
		assertEquals(c1.getName(), 	da.getTarget().getName());
		assertEquals("75.0", 		da.getMessage());
		assertEquals(500,500, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(500,500, 		da.getNewState().getLocation());
		assertEquals(0.6, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent(), 0.01);
		assertEquals(null, 			da.getLinkedAnimations());
		
		ExplodeAnimation ea = (ExplodeAnimation) baa[9];
		assertEquals(600,500, 		ea.getImage().getLocation());

		da = (DamageAnimation) baa[10];
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals("75.0", 		da.getMessage());
		assertEquals(600,500, 		da.getOldState().getLocation());
		assertEquals(0.72, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(0.17, 			da.getOldState().getShieldPercent(), 0.01);
		assertEquals(600,500, 		da.getNewState().getLocation());
		assertEquals(1.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent(), 0.01);
		assertEquals(null, 			da.getLinkedAnimations());
		
		DelayAnimation delay4 = (DelayAnimation) baa[11];
		
    }     
    

    public void testCraftMoveWithDamage() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testCraftMoveWithDamage() *************************");
        CraftDesign fastFighter = new CraftDesign("FastFighter", tech_flyer30, tech_fusion40, tech_electronic25, tech_energy10, tech_massDriver1);
        createMap(fastFighter,fighterDesign, false);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        bca[0].damage(5, null, null, null);
        bca[0].setMovementDamage(40);
        bca[0].setPsiDamage(5);
        bca[0].setShieldPower(0, 5);
        bca[0].setCrew(5);

        		
        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        
        MainWindow.getInstance().toBattleWindow(battle);
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 MOVES FROM 500,500 TO 550,550 AND TURNS 45 degrees
        //
        
        clearEvents();
        bca[0]._moveTo(new Location(525,525));
        
		BattleCraftMoveEvent cme = (BattleCraftMoveEvent) p1Proxy.getEvent(BattleCraftMoveEvent.class);
		ObjectMovement[] ms = cme.getMovements();
		assertEquals(2, 		ms.length);
		assertEquals(135, 			((ObjectTurn)ms[0]).getTo());
		assertEquals(0.22,	 		((ObjectTurn)ms[0]).getMovePercent(), 0.01);
		assertEquals(500,500, 		((ObjectTurn)ms[0]).getState().getLocation());
		assertEquals(90, 			((ObjectTurn)ms[0]).getState().getDirection());
		assertEquals(0.25, 			((ObjectTurn)ms[0]).getState().getDamagePercent(), 0.001);
		assertEquals(blue,			((ObjectTurn)ms[0]).getState().getPlayerColor());
		assertEquals((String)null,	((ObjectTurn)ms[0]).getState().getCloakedToPlayerNames());
		assertEquals(0.5, 			((ObjectTurn)ms[0]).getState().getShieldPercent());
		assertEquals(0.33,			((ObjectTurn)ms[0]).getState().getMovementDamagePercent(), 0.01);
		assertEquals(0.1, 			((ObjectTurn)ms[0]).getState().getPsiDamagePercent());
		assertEquals(0.5, 			((ObjectTurn)ms[0]).getState().getCrewPercent());
		assertEquals(500,500, 		((ObjectShift)ms[1]).getOrigin());
		assertEquals(525,525, 		((ObjectShift)ms[1]).getDestination());
		assertEquals(null,	 		((ObjectShift)ms[1]).getDamage());
		assertEquals(0.46,			((ObjectShift)ms[1]).getMovePercent(), 0.01);
		assertEquals(500,500, 		((ObjectShift)ms[1]).getState().getLocation());
		assertEquals(135, 			((ObjectShift)ms[1]).getState().getDirection());
		assertEquals(0.25, 			((ObjectShift)ms[1]).getState().getDamagePercent(), 0.001);
		assertEquals(blue,			((ObjectShift)ms[1]).getState().getPlayerColor());
		assertEquals((String)null,	((ObjectShift)ms[1]).getState().getCloakedToPlayerNames());
		assertEquals(0.5, 			((ObjectShift)ms[1]).getState().getShieldPercent());
		assertEquals(0.33,			((ObjectShift)ms[1]).getState().getMovementDamagePercent(), 0.01);
		assertEquals(0.1, 			((ObjectShift)ms[1]).getState().getPsiDamagePercent());
		assertEquals(0.5, 			((ObjectShift)ms[1]).getState().getCrewPercent());
		MotEvent[] links = cme.getLinkedEvents();
		assertEquals(null,		links);
		
		ac.getAnimations().stopAll();
		processor.process(cme, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		
		assertEquals(2, baa.length);
		CraftTurnAnimation a1 = (CraftTurnAnimation) baa[0];
		
		assertEquals(90.0,			a1.getFromDirection());
		assertEquals(135.0,			a1.getToDirection());
		assertEquals(90.0,			a1.getDirection());
		assertEquals(null,			a1.getLinkedAnimations());
		assertEquals(500,500,		a1.getState().getLocation());
		assertEquals(90,			a1.getState().getDirection());
		assertEquals(0.25, 			a1.getState().getDamagePercent(), 0.001);
		assertEquals(blue,			a1.getState().getPlayerColor());
		assertEquals((String)null,	a1.getState().getCloakedToPlayerNames());
		assertEquals(0.5, 			a1.getState().getShieldPercent());
		
		CraftShiftAnimation a2 = (CraftShiftAnimation) baa[1];
		
		assertEquals(500,500,		a2.getSource());
		assertEquals(525,525,		a2.getDestination());
		assertEquals(null,			a2.getLinkedAnimations());
		assertEquals(500,500,		a2.getState().getLocation());
		assertEquals(135,			a2.getState().getDirection());
		assertEquals(0.25, 			a2.getState().getDamagePercent(), 0.001);
		assertEquals(blue,			a2.getState().getPlayerColor());
		assertEquals((String)null,	a2.getState().getCloakedToPlayerNames());
		assertEquals(0.5, 			a2.getState().getShieldPercent());
		
    }
    
    
    public void testCraftMoveThroughAsteroidBelt() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testCraftMoveThroughAsteroidBelt() *************************");
        CraftDesign fastFighter = new CraftDesign("FastFighter", tech_flyer30, tech_fusion40, tech_electronic25, tech_energy10, tech_massDriver1);
        createMap(fastFighter,fighterDesign, false);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        callbackSync.waitTillComplete();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(400, 500));
        bca[1].setLocation(new Location(900, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        // asteroid field is at 450,150 -> 550,850
        IncumbentFactory.createAsteroidField(MotEngine.instance.getBattles().getActiveBattle());
        		
        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        
        MainWindow.getInstance().toBattleWindow(battle);
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 MOVES FROM 400,500 TO 600,500 - HITS ASTEROID FIELD AND GETS DAMAGED!
        //
        
        clearEvents();
        bca[0].setLocation(new Location(650,500));
        bca[0]._moveTo(new Location(850,500));
        
		BattleCraftMoveEvent cme = (BattleCraftMoveEvent) p1Proxy.getEvent(BattleCraftMoveEvent.class);
		ObjectMovement[] ms = cme.getMovements();
		assertEquals(3, 		ms.length);
		assertEquals(650,500, 	((ObjectShift)ms[0]).getOrigin());
		assertEquals(700,500, 	((ObjectShift)ms[0]).getDestination());
		assertEquals(null,	 	((ObjectShift)ms[0]).getDamage());
		assertEquals(0.41,		((ObjectShift)ms[0]).getMovePercent(), 0.01);
		assertEquals(650,500, 	((ObjectShift)ms[0]).getState().getLocation());
		assertEquals(90, 		((ObjectShift)ms[0]).getState().getDirection());
		assertEquals(0.0, 		((ObjectShift)ms[0]).getState().getDamagePercent(), 0.01);
		assertEquals(700,500, 	((ObjectShift)ms[1]).getOrigin());
		assertEquals(734,500, 	((ObjectShift)ms[1]).getDestination());
		assertEquals("4.0",		((ObjectShift)ms[1]).getDamage().getTargetDescription());
		assertEquals(0.56,		((ObjectShift)ms[1]).getMovePercent(), 0.01);
		assertEquals(700,500, 	((ObjectShift)ms[1]).getState().getLocation());
		assertEquals(90, 		((ObjectShift)ms[1]).getState().getDirection());
		assertEquals(0.0, 		((ObjectShift)ms[1]).getState().getDamagePercent(), 0.01);
		MotEvent[] links = cme.getLinkedEvents();
		assertEquals(null,		links);
		
		ac.getAnimations().stopAll();
		processor.process(cme, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		
		assertEquals(3, baa.length);
		CraftShiftAnimation a1 = (CraftShiftAnimation) baa[0];
		
		assertEquals(650,500,	a1.getSource());
		assertEquals(700,500,	a1.getDestination());
		assertEquals(650,500,	a1.getState().getLocation());
		assertEquals(90,		a1.getState().getDirection());
		assertEquals(0.0, 		a1.getState().getDamagePercent(), 0.01);
		assertEquals(1.0, 		a1.getState().getShieldPercent());
		assertEquals(null,		a1.getLinkedAnimations());
		
		CraftShiftAnimation a2 = (CraftShiftAnimation) baa[1];
		
		assertEquals(700,500,	a2.getSource());
		assertEquals(734,500,	a2.getDestination());
		assertEquals(700,500,	a2.getState().getLocation());
		assertEquals(90,		a2.getState().getDirection());
		assertEquals(0.2, 		a2.getState().getDamagePercent(), 0.01);
		assertEquals(0.5, 		a2.getState().getShieldPercent());
		LinkedAnimation[] la = a2.getLinkedAnimations();
		assertEquals(1,			la.length);
		assertEquals("4.0",		((LinkedDamageAnimation)la[0]).getMessage());
		
    }
    
    public void testHologramDestroyed() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testHologramDestroyed() *************************");
        CraftDesign holoDesign = new CraftDesign("HoloFighter", tech_flyer30WithOneOffset, tech_fusion25, tech_electronic25, tech_energy10, tech_massDriver2, tech_holoEmiter);
        CraftDesign fighterDesign = new CraftDesign("Fighter", tech_flyer30WithOneOffset, tech_fusion25, tech_electronic25, tech_energy10, tech_massDriver1);
        createMap(holoDesign,fighterDesign, false);
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        //processEndingThread();
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleCraft [] bca = b.getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        HoloCraftFactory.create(b, bca[0], true);
        bca = b.getCrafts(); 
        
        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        assertEquals("HoloFighter111", bca[2].getName());
        assertEquals(c1.getName(), crafts[0].getName());
        assertEquals(c2.getName(), crafts[1].getName());
        assertEquals("HoloFighter111", crafts[2].getName());
        
        assertEquals(1, crafts[2].getState().getDecorators().getDecorators().length);
        assertEquals("HologramDecorator", crafts[2].getState().getDecorators().getDecorators()[0].getName());
        
        BattleCraftDelegate craft3 = crafts[2];
        bca[2].setLocation(new Location(550, 500));
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT1 FIRES FUSION BEAM AT CRAFT2 AND DESTROYES IT
        //
		
        clearEvents();
        bca[1].fireWeapons(bca[2], null, -1);
        
		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(0);
		assertEquals(tech_massDriver1.getName(), wfe.getWeapon());
		assertEquals(craft2.getName(), 		wfe.getSource());
		assertEquals(craft3.getName(), 		wfe.getTarget());
		assertEquals(600,500, 				wfe.getSourceLocation(null));
		assertEquals(550,500, 				wfe.getTargetLocation(null));
		assertEquals(600,500, 				wfe.getOldSourceState().getLocation());
		assertEquals(270, 					wfe.getOldSourceState().getDirection());
		assertEquals(0.0, 					wfe.getOldSourceState().getDamagePercent(), 0.01);
		assertEquals(1.0, 					wfe.getOldSourceState().getShieldPercent());
		assertEquals(550,500, 				wfe.getOldTargetState().getLocation());
		assertEquals(90, 					wfe.getOldTargetState().getDirection());
		assertEquals(0.0, 					wfe.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(1.0, 					wfe.getOldTargetState().getShieldPercent());
		assertEquals(600,500, 				wfe.getNewSourceState().getLocation());
		assertEquals(270, 					wfe.getNewSourceState().getDirection());
		assertEquals(0.0, 					wfe.getNewSourceState().getDamagePercent());
		assertEquals(1.0, 					wfe.getNewSourceState().getShieldPercent());
		assertEquals(550,500, 				wfe.getNewTargetState().getLocation());
		assertEquals(90, 					wfe.getNewTargetState().getDirection());
		assertEquals(1.0, 					wfe.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					wfe.getNewTargetState().getShieldPercent());
		assertNull(							wfe.getSourceDescription());
		assertEquals("1.3",					wfe.getTargetDescription());
		MotEvent[] links = wfe.getLinkedEvents();
		assertEquals(null,					links);
		assertEquals(null,					wfe.getData());
		
		BattleDestroyedEvent de = (BattleDestroyedEvent) p1Proxy.getEvent(1);
		assertEquals(craft3.getName(), 		de.getTarget());
		assertEquals(550,500, 				de.getLocation(null));
		assertEquals(550,500, 				de.getOldState().getLocation());
		assertEquals(90, 					de.getOldState().getDirection());
		assertEquals(0.0, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(1.0,					de.getOldState().getShieldPercent());

		ac.getAnimations().stopAll();
		processor.process(wfe, null);
		processor.process(de, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(3, baa.length);
		
		WeaponFireAnimation animation = (WeaponFireAnimation) baa[0];
		assertEquals(new Location(595,500), animation.getSource()[0]);
		assertEquals(new Location(550,500), animation.getDestination());
		assertEquals(craft2.getName(), animation.getAgressor().getName());
		assertEquals(craft3.getName(), animation.getTarget().getName());
		assertEquals(null, animation.getLinkedAnimations());
		
		DamageAnimation da = (DamageAnimation) baa[1];
		assertEquals("1.3", 		da.getMessage());
		assertEquals(craft3.getName(),da.getTarget().getName());
		assertEquals(550,500, 		da.getOldState().getLocation());
		assertEquals(0.0, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getOldState().getPlayerColor());
		assertEquals(1.0, 			da.getOldState().getShieldPercent());
		assertEquals(550,500, 		da.getNewState().getLocation());
		assertEquals(1.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(blue,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		HologramDestroyedAnimation ea = (HologramDestroyedAnimation) baa[2];
		assertEquals(550,500, 		ea.getArea().getCenterLocation());
		
    }     
    
    public void testBattleEndTurnMessagesAndTriggerEvents() throws Exception {
        Logger.logDebug("*******************************  WarTest.testBattleEndTurnMessagesAndTriggerEvents() *************************");
        createMap(null, null, false);

        p1.getPlayerCharacter().setResourceful(true);

        CraftDesign repairer = new CraftDesign("TestRepairer", tech_flyer30, tech_fusion25, tech_auto_repair_unit, tech_massDriver1);

        Craft c1 = newCraft(fighterDesign, p1, ss2);
        Craft c2 = newCraft(repairer, p1, ss2);
        Craft c3 = newCraft(fighterDesign, p2, ss2);
        Craft c4 = newCraft(fighterDesign, p2, ss2);
        Craft c5 = newCraft(fighterDesign, p2, ss2);
        Craft c6 = newCraft(repairer, p2, ss2);
        Craft c7 = newCraft(fighterDesign, p2, ss2);
        Craft c8 = newCraft(fighterDesign, p2, ss2);
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestRepairer1");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter2");
        CraftDelegate craft4 = DeligateCacheManager.getCraft("TestFighter3");
        CraftDelegate craft5 = DeligateCacheManager.getCraft("TestFighter4");
        CraftDelegate craft6 = DeligateCacheManager.getCraft("TestRepairer2");
        CraftDelegate craft7 = DeligateCacheManager.getCraft("TestFighter5");
        CraftDelegate craft8 = DeligateCacheManager.getCraft("TestFighter6");
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        serverProxy.craftAttackCmd(craft2, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();

        MotEngine.instance.endTurn(p1);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleCraft [] bca = b.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c3.getName(), bca[1].getName());
        assertEquals(c4.getName(), bca[2].getName());
        assertEquals(c5.getName(), bca[3].getName());
        assertEquals(c6.getName(), bca[4].getName());
        assertEquals(c7.getName(), bca[5].getName());
        assertEquals(c8.getName(), bca[6].getName());
        assertEquals(c2.getName(), bca[7].getName());

        // craft2, craft6 have automated repair message
        bca[7].getCraft().setDamage(5);
        bca[4].getCraft().setDamage(5);

        // craft1, craft2 belong to resourceful player (get repaired AND shield recharged).
        bca[0].getCraft().setDamage(10);
        bca[7].getCraft().setDamage(10);
        bca[0].getShields()[0].setPower(0);
        
        // craft1, craft2, craft7, craft8 has Plague
		b.getTriggersNotNull().add(new Plague(p2, b, bca[0] , CraftTriggerBase.INFINATE_LIFE, 5));
		b.getTriggersNotNull().add(new Plague(p2, b, bca[7] , CraftTriggerBase.INFINATE_LIFE, 5));
		b.getTriggersNotNull().add(new Plague(p1, b, bca[5] , CraftTriggerBase.INFINATE_LIFE, 5));
		b.getTriggersNotNull().add(new Plague(p1, b, bca[6] , CraftTriggerBase.INFINATE_LIFE, 5));

		// craft3 craft7 has sleep
		b.getTriggersNotNull().add(new Sleep(p1, b, bca[1] , 1));
		b.getTriggersNotNull().add(new Sleep(p1, b, bca[5] , 1));
        
		// craft1, craft2 craft5, craft6 has DamageBot
		b.getTriggersNotNull().add(new DamageBot(p2, b, bca[0] ,1, 5));
		b.getTriggersNotNull().add(new DamageBot(p2, b, bca[7] ,1, 5));
		b.getTriggersNotNull().add(new DamageBot(p1, b, bca[3] ,1, 5));
		b.getTriggersNotNull().add(new DamageBot(p1, b, bca[4] ,1, 5));
        
        // make sure resourcefulness does heal and charge shields
        Utils.clearAndSetNextRandomProbability(0.1);
        Utils.setNextRandomProbability(0.1);
        Utils.setNextRandomProbability(0.1);
        Utils.setNextRandomProbability(0.1);
        Utils.setNextRandomProbability(0.1);
        Utils.setNextRandomProbability(0.1);
        Utils.setNextRandomProbability(0.1);
        Utils.setNextRandomProbability(0.1);
        Utils.setNextRandomProbability(0.1);
        
        serverProxy._processQueuedEvents();
        
        EndTurnEvent ste = (EndTurnEvent) p1Proxy.getEvent(0); 
        
        NoResearchProjectEvent nre = (NoResearchProjectEvent) p1Proxy.getEvent(1); 
        assertNotNull(nre);
        
        NoProjectEvent npe = (NoProjectEvent) p1Proxy.getEvent(2); 
        assertEquals(h1.getName(), npe.getPlanet());
        
        NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(3); 
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
		assertEquals(ss2.getName(), battle.getOrbit().getStellarObject());
        
		p1Proxy.clear();
		
		bca[0].endTurn();
		bca[1].endTurn();
		bca[2].endTurn();
		bca[3].endTurn();
		bca[4].endTurn();
		bca[5].endTurn();
		bca[6].endTurn();
		bca[7].endTurn();
		b.endTurn();
		
        BattleObjectTriggerEvent te1 = (BattleObjectTriggerEvent) p1Proxy.getEvent(0); 
        assertEquals(craft1.getName(), te1.getBattleObject());
        assertEquals(Weapon.GROUP_DAMAGE_BOT, te1.getType());
        
        BattleObjectTriggerEvent te2 = (BattleObjectTriggerEvent) p1Proxy.getEvent(1); 
        assertEquals(craft2.getName(), te2.getBattleObject());
        assertEquals(Weapon.GROUP_DAMAGE_BOT, te2.getType());
        
        BattleObjectTriggerEvent te3 = (BattleObjectTriggerEvent) p1Proxy.getEvent(2); 
        assertEquals(craft5.getName(), te3.getBattleObject());
        assertEquals(Weapon.GROUP_DAMAGE_BOT, te3.getType());
        
        BattleObjectTriggerEvent te4 = (BattleObjectTriggerEvent) p1Proxy.getEvent(3); 
        assertEquals(craft6.getName(), te4.getBattleObject());
        assertEquals(Weapon.GROUP_DAMAGE_BOT, te4.getType());
        
        BattleObjectTriggerEvent te5 = (BattleObjectTriggerEvent) p1Proxy.getEvent(4); 
        assertEquals(craft1.getName(), te5.getBattleObject());
        assertEquals(Weapon.GROUP_PLAGUE, te5.getType());
        
        BattleObjectTriggerEvent te6 = (BattleObjectTriggerEvent) p1Proxy.getEvent(5); 
        assertEquals(craft2.getName(), te6.getBattleObject());
        assertEquals(Weapon.GROUP_PLAGUE, te6.getType());
        
        BattleObjectTriggerEvent te7 = (BattleObjectTriggerEvent) p1Proxy.getEvent(6); 
        assertEquals(craft7.getName(), te7.getBattleObject());
        assertEquals(Weapon.GROUP_PLAGUE, te7.getType());
        
        BattleObjectTriggerEvent te8 = (BattleObjectTriggerEvent) p1Proxy.getEvent(7); 
        assertEquals(craft8.getName(), te8.getBattleObject());
        assertEquals(Weapon.GROUP_PLAGUE, te8.getType());
        
        BattleObjectTriggerEvent te9 = (BattleObjectTriggerEvent) p1Proxy.getEvent(8); 
        assertEquals(craft3.getName(), te9.getBattleObject());
        assertEquals(Weapon.GROUP_SLEEP, te9.getType());
        
        BattleObjectTriggerEvent te10 = (BattleObjectTriggerEvent) p1Proxy.getEvent(9); 
        assertEquals(craft7.getName(), te10.getBattleObject());
        assertEquals(Weapon.GROUP_SLEEP, te10.getType());
        
        BattleObjectNotificationEvent te11 = (BattleObjectNotificationEvent) p1Proxy.getEvent(10); 
        assertEquals("Plague Spread", te11.getMessage());
        assertEquals(craft5.getName(), te11.getTarget());
        assertEquals(BattleObjectNotificationType.plague, te11.getType());
        
        BattleObjectNotificationEvent te12 = (BattleObjectNotificationEvent) p1Proxy.getEvent(11); 
        assertEquals("Plague Spread", te12.getMessage());
        assertEquals(craft6.getName(), te12.getTarget());
        assertEquals(BattleObjectNotificationType.plague, te12.getType());
        
        BattleObjectNotificationEvent te13 = (BattleObjectNotificationEvent) p1Proxy.getEvent(12); 
        assertEquals("Resourceful : Shield + 2", te13.getMessage());
        assertEquals(craft1.getName(), te13.getTarget());
        assertEquals(BattleObjectNotificationType.repair, te13.getType());
        
        BattleObjectNotificationEvent te14 = (BattleObjectNotificationEvent) p1Proxy.getEvent(13); 
        assertEquals("Automated Repair +2", te14.getMessage());
        assertEquals(craft1.getName(), te14.getTarget());
        assertEquals(BattleObjectNotificationType.repair, te14.getType());
        
        BattleObjectNotificationEvent te15 = (BattleObjectNotificationEvent) p1Proxy.getEvent(14); 
        assertEquals("Automated Repair +2", te15.getMessage());
        assertEquals(craft6.getName(), te15.getTarget());
        assertEquals(BattleObjectNotificationType.repair, te15.getType());
        
        BattleObjectNotificationEvent te16 = (BattleObjectNotificationEvent) p1Proxy.getEvent(15); 
        assertEquals("Automated Repair +4", te16.getMessage());
        assertEquals(craft2.getName(), te16.getTarget());
        assertEquals(BattleObjectNotificationType.repair, te16.getType());
        
        BattleNewTurnEvent te17 = (BattleNewTurnEvent) p1Proxy.getEvent(16); 
        
        AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();
        ac.getAnimations().stopAll();
        
		processor.process(te1, null);
		processor.process(te2, null);
		processor.process(te3, null);
		processor.process(te4, null);
		processor.process(te5, null);
		processor.process(te6, null);
		processor.process(te7, null);
		processor.process(te8, null);
		processor.process(te9, null);
		processor.process(te10, null);
		processor.process(te11, null);
		processor.process(te12, null);
		processor.process(te13, null);
		processor.process(te14, null);
		processor.process(te15, null);
		processor.process(te16, null);
		
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(31, baa.length);

        assertEquals(te1, ((BotAttackAnimation)baa[0]).getEvent());
        assertEquals(te1, ((DamageAnimation)baa[1]).getEvent());
        assertEquals(te2, ((BotAttackAnimation)baa[2]).getEvent());
        assertEquals(te2, ((DamageAnimation)baa[3]).getEvent());
        assertEquals(te3, ((BotAttackAnimation)baa[4]).getEvent());
        assertEquals(te3, ((DamageAnimation)baa[5]).getEvent());
        assertEquals(te4, ((BotAttackAnimation)baa[6]).getEvent());
        assertEquals(te4, ((DamageAnimation)baa[7]).getEvent());
        
        assertEquals(1, ((DelayAnimation)baa[8]).getLife());
        
        assertEquals(te5, ((PlagueStrikeAnimation)baa[9]).getEvent());
        assertEquals(te5, ((DamageAnimation)baa[10]).getEvent());
        assertEquals(te6, ((PlagueStrikeAnimation)baa[11]).getEvent());
        assertEquals(te6, ((DamageAnimation)baa[12]).getEvent());
        assertEquals(te7, ((PlagueStrikeAnimation)baa[13]).getEvent());
        assertEquals(te7, ((DamageAnimation)baa[14]).getEvent());
        assertEquals(te8, ((PlagueStrikeAnimation)baa[15]).getEvent());
        assertEquals(te8, ((DamageAnimation)baa[16]).getEvent());
        
        assertEquals(1, ((DelayAnimation)baa[17]).getLife());
        
        assertEquals(te9, ((SleepStrikeAnimation)baa[18]).getEvent());
        assertEquals(te9, ((DamageAnimation)baa[19]).getEvent());
        assertEquals(te10, ((SleepStrikeAnimation)baa[20]).getEvent());
        assertEquals(te10, ((DamageAnimation)baa[21]).getEvent());
        
        assertEquals(1, ((DelayAnimation)baa[22]).getLife());
        
        assertEquals(te11, ((MessageAnimation)baa[23]).getEvent());
        assertEquals(te12, ((MessageAnimation)baa[24]).getEvent());
        
        assertEquals(1, ((DelayAnimation)baa[25]).getLife());
        
        assertEquals(te13, ((MessageAnimation)baa[26]).getEvent());
        
        assertEquals(1, ((DelayAnimation)baa[27]).getLife());
        
        assertEquals(te14, ((MessageAnimation)baa[28]).getEvent());
        assertEquals(te15, ((MessageAnimation)baa[29]).getEvent());
        assertEquals(te16, ((MessageAnimation)baa[30]).getEvent());
    }
    
    public void testCraftDestroyedDuringTurnByMissile() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testCraftDestroyedDuringTurnByMissile() *************************");
        CraftDesign missileDesign = new CraftDesign("MissileFighter", tech_flyer30, tech_fusion25, tech_missile6);
        createMap(missileDesign,missileDesign, false);
        
        Craft c3 = newCraft(fighterDesign, p2, ss1);
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
		clearEvents();
		serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
        callbackSync.waitTillComplete();
		//processEndingThread();
		        
		NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(700, 500));
        bca[1].setDirection(0);
        bca[2].setLocation(new Location(800, 500));
        bca[2].setDirection(270);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        assertEquals(c3.getName(), bca[2].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // COMPUTER PLAYERS CRAFT MOVES WHEN MISSILE IS ABOUT TO STRIKE
        //
		
        clearEvents();

        bca[0].fireAllWeapons(bca[1]);
        bca[0].dischargeWeapons();
        bca[0].setMoveRemaining(0.0);
        bca[1].getCraft().setDamage(bca[0].getCraft().getMaxDamage()-1);
        bca[2].dischargeWeapons();
        bca[2].setMoveRemaining(0.0);
        
        Missiles missiles = (Missiles) MotEngine.instance.getBattles().getActiveBattle().getAutomatons()[0];
        missiles.setLocation(new Location(690, 500));
        
        bca[1].moveTo(bca[0]);

        serverProxy._processQueuedEvents();

//        serverProxy.battleGetNextObject(battle, callbackSync, null);
//        serverProxy._processCommandResponse();
//        serverProxy._processQueuedEvents();
//        String next = ((NameCR) callbackSync.waitTillComplete()).getName();

        BattleMissileLaunchEvent mle = (BattleMissileLaunchEvent) p1Proxy.getEvent(0);

        BattleCraftMoveEvent cme = (BattleCraftMoveEvent) p1Proxy.getEvent(1);
		ObjectMovement[] ms = cme.getMovements();
		MotEvent[] links = cme.getLinkedEvents();
		assertEquals(1, 			ms.length);
		assertEquals(355, 			((ObjectTurn)ms[0]).getTo());
		assertEquals(1,				links.length);
		assertEquals(690,500,		((BattleMissileMoveEvent)links[0]).getFrom());
		assertEquals(700,500,		((BattleMissileMoveEvent)links[0]).getTo());
		
		
		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) p1Proxy.getEvent(2);
		assertEquals(tech_missile6.getName(), 		wfe.getWeapon());
		assertEquals(missiles.getName(), 	wfe.getSource());
		assertEquals(craft2.getName(), 		wfe.getTarget());
		assertEquals(700,500, 				wfe.getSourceLocation(null));
		assertEquals(700,500, 				wfe.getTargetLocation(null));
		assertEquals(700,500, 				wfe.getOldSourceState().getLocation());
		assertEquals(90, 					wfe.getOldSourceState().getDirection());
		assertEquals(0.0, 					wfe.getOldSourceState().getDamagePercent(), 0.01);
		assertEquals(0.0, 					wfe.getOldSourceState().getShieldPercent());
		assertEquals(700,500, 				wfe.getOldTargetState().getLocation());
		assertEquals(355, 					wfe.getOldTargetState().getDirection());
		assertEquals(0.95, 					wfe.getOldTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0, 					wfe.getOldTargetState().getShieldPercent());
		assertEquals(700,500, 				wfe.getNewSourceState().getLocation());
		assertEquals(90, 					wfe.getNewSourceState().getDirection());
		assertEquals(0.0, 					wfe.getNewSourceState().getDamagePercent());
		assertEquals(0.0, 					wfe.getNewSourceState().getShieldPercent());
		assertEquals(700,500, 				wfe.getNewTargetState().getLocation());
		assertEquals(355, 					wfe.getNewTargetState().getDirection());
		assertEquals(1.0, 					wfe.getNewTargetState().getDamagePercent(), 0.01);
		assertEquals(0.0,					wfe.getNewTargetState().getShieldPercent());
		assertNull(							wfe.getSourceDescription());
		assertEquals("6.0",					wfe.getTargetDescription());
		
		BattleDestroyedEvent de = (BattleDestroyedEvent) p1Proxy.getEvent(3);
		assertEquals(craft2.getName(), 		de.getTarget());
		assertEquals(700,500, 				de.getLocation(null));
		assertEquals(700,500, 				de.getOldState().getLocation());
		assertEquals(355, 					de.getOldState().getDirection());
		assertEquals(0.95, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(0.0,					de.getOldState().getShieldPercent());

		ac.getAnimations().stopAll();
		processor.process(mle, null);
		processor.process(cme, null);
		processor.process(wfe, null);
		processor.process(de, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(4, baa.length);
		
		MissileLaunchAnimation a0 = (MissileLaunchAnimation) baa[0];
		
		CraftTurnAnimation a1 = (CraftTurnAnimation) baa[1];
		assertEquals(0.0,		a1.getFromDirection());
		assertEquals(355.0,		a1.getToDirection());
		LinkedAnimation[] linkedAnimations = a1.getLinkedAnimations();
		assertEquals(1, linkedAnimations.length);
		assertEquals(690,500, 	((MoveMissileLinkedAnimation)linkedAnimations[0]).getSource());
		assertEquals(700,500, 	((MoveMissileLinkedAnimation)linkedAnimations[0]).getDestination());
		
		DamageAnimation da = (DamageAnimation) baa[2];
		assertEquals("6.0", 		da.getMessage());
		assertEquals(c2.getName(), 	da.getTarget().getName());
		assertEquals(700,500, 		da.getOldState().getLocation());
		assertEquals(0.95, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getOldState().getPlayerColor());
		assertEquals(0.0, 			da.getOldState().getShieldPercent());
		assertEquals(700,500, 		da.getNewState().getLocation());
		assertEquals(1.0, 			da.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da.getNewState().getPlayerColor());
		assertEquals(0.0,			da.getNewState().getShieldPercent());
		assertEquals(null, 			da.getLinkedAnimations());
		
		ExplodeAnimation ea = (ExplodeAnimation) baa[3];
		assertEquals(700,500, 		ea.getImage().getLocation());
		
    }     

    public void testCraftDestroyedDuringTurnByAsteroids() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testCraftDestroyedDuringTurnByAsteroids() *************************");
        CraftDesign missileDesign = new CraftDesign("MissileFighter", tech_flyer30, tech_fusion25, tech_missile6);
        createMap(missileDesign,missileDesign, true);
        
        Craft c3 = newCraft(fighterDesign, p2, ss1);
        
		clearEvents();
		serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
		//processEndingThread();
		        
		NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        assertEquals(c3.getName(), bca[2].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // CRAFT MOVES THROUGH ASTEROID FIELD AND IS DESTROYED
        //
		
        clearEvents();

        bca[1].setLocation(new Location(775, 500));
        bca[1].setDirection(270);
        bca[1].getCraft().setDamage(bca[0].getCraft().getMaxDamage()-1);
        bca[1].moveTo(new LocationAndDirection(650, 500, 270));
        
        BattleCraftMoveEvent cme = (BattleCraftMoveEvent) p1Proxy.getEvent(0);
		ObjectMovement[] ms = cme.getMovements();
		MotEvent[] links = cme.getLinkedEvents();
		assertEquals(1, 			ms.length);
		assertEquals(775,500, 		((ObjectShift)ms[0]).getOrigin());
		assertEquals(750,500, 		((ObjectShift)ms[0]).getDestination());
		assertEquals(1.0, 			((ObjectShift)ms[0]).getSpeedModification());
		
		BattleDamageEvent ae = (BattleDamageEvent) p1Proxy.getEvent(1);
		links = ae.getLinkedEvents();
		assertEquals(craft2.getName(), 		ae.getTarget());
		assertEquals(750,500, 				ae.getOldTargetState().getLocation());
		assertEquals(false, 				ae.getOldTargetState().isDestroyed());
		assertEquals(750,500, 				ae.getNewTargetState().getLocation());
		assertEquals(true, 					ae.getNewTargetState().isDestroyed());
		assertEquals("5.0", 				ae.getTargetDescription());
		
		BattleDestroyedEvent de = (BattleDestroyedEvent) p1Proxy.getEvent(2);
		links = de.getLinkedEvents();
		assertEquals(craft2.getName(), 		de.getTarget());
		assertEquals(750,500, 				de.getLocation(null));
		assertEquals(750,500, 				de.getOldState().getLocation());
		assertEquals(270, 					de.getOldState().getDirection());
		assertEquals(0.95, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(0.0,					de.getOldState().getShieldPercent());
		assertEquals(null,					links);

		assertEquals(3, p1Proxy.getQtyEvent());
		
		ac.getAnimations().stopAll();
		processor.process(cme, null);
		processor.process(de, null);
		processor.process(ae, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(3, baa.length);
		
		CraftShiftAnimation a1 = (CraftShiftAnimation) baa[0];
		assertEquals(775,500,		a1.getSource());
		assertEquals(750,500,		a1.getDestination());
		assertEquals(0.95,			a1.getState().getDamagePercent());
		
		ExplodeAnimation a2 = (ExplodeAnimation) baa[1];
		assertEquals(750,500, 		a2.getImage().getLocation());
		
		DamageAnimation da = (DamageAnimation) baa[2];
		assertEquals("5.0", 		da.getMessage());
		assertEquals(750,500, 		da.getOldState().getLocation());
		assertEquals(0.95, 			da.getOldState().getDamagePercent(), 0.01);
		assertEquals(750,500, 		da.getNewState().getLocation());
		assertEquals(1.0, 			da.getNewState().getDamagePercent(), 0.01);
		
    }     

    public void testDetonator() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testDetonator() *************************");
        CraftDesign detonatorDesign = new CraftDesign("DetonatorFighter", tech_flyer30, tech_fusion25, tech_detonator);
        createMap(fighterDesign,detonatorDesign, false);
        
        Craft c3 = newCraft(fighterDesign, p2, ss1);
		clearEvents();
		serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        serverProxy._processQueuedEvents();
		//processEndingThread();
		        
		NewBattleEvent nbe = (NewBattleEvent) p1Proxy.getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(550, 500));
        bca[1].setDirection(270);
        bca[2].setLocation(new Location(610, 500));
        bca[2].setDirection(270);

        BattleCraftDelegate [] crafts = battle.getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c2.getName(), bca[1].getName());
        assertEquals(c3.getName(), bca[2].getName());
        
		AnimationContainer ac = MainWindow.getInstance().getAnimationContainer();

		//
        // COMPUTER PLAYERS CRAFT MOVES WHEN MISSILE IS ABOUT TO STRIKE
        //
		
        clearEvents();

        bca[0].getCraft().setDamage(bca[0].getCraft().getMaxDamage()-1);
        bca[1].fireWeapon(bca[0], bca[1].getWeapon(tech_detonator), 1);
        serverProxy._processQueuedEvents();
        
		BattleDestroyedEvent de = (BattleDestroyedEvent) p1Proxy.getEvent(0);
		assertEquals(craft2.getName(), 		de.getTarget());
		assertEquals(550,500, 				de.getLocation(null));
		assertEquals(550,500, 				de.getOldState().getLocation());
		assertEquals(270, 					de.getOldState().getDirection());
		assertEquals(0.0, 					de.getOldState().getDamagePercent(), 0.01);
		assertEquals(0.0,					de.getOldState().getShieldPercent());

		BattleDamageRingEvent dre = (BattleDamageRingEvent) p1Proxy.getEvent(1);
		assertEquals(550,500, 			dre.getLocation());
		assertEquals(bca[1].getName(), 	dre.getSource());
		assertNull(		 				dre.getWeapon());
		MotEvent[] links = dre.getLinkedEvents();
		assertEquals(3, 			links.length);
		assertEquals(500,500,		((BattleDestroyedEvent)links[0]).getLocation(null));
		assertEquals(bca[0],		((BattleDestroyedEvent)links[0]).getTarget());
		assertEquals(500,500,		((BattleDamageEvent)links[1]).getOldTargetState().getLocation());
		assertEquals(bca[0],		((BattleDamageEvent)links[1]).getTarget());
		assertEquals(610,500,		((BattleDamageEvent)links[2]).getOldTargetState().getLocation());
		assertEquals(bca[2],		((BattleDamageEvent)links[2]).getTarget());
				
		BattleEndEvent ebe = (BattleEndEvent) p1Proxy.getEvent(2);
		
		ac.getAnimations().stopAll();
		processor.process(de, null);
		processor.process(dre, null);
		BattleAnimation[] baa = ac.getAnimations().getAllAnimations();
		assertEquals(6, baa.length);
		
		ExplodeAnimation ea1 = (ExplodeAnimation) baa[0];
		assertEquals(550,500, 		ea1.getImage().getLocation());

		WarpCoreOverloadAnimation wcol = (WarpCoreOverloadAnimation) baa[1];
		assertEquals(550,500, 		wcol.getSource().getLocation());

		ExplodeAnimation ea2 = (ExplodeAnimation) baa[2];
		assertEquals(500,500, 		ea2.getImage().getLocation());

		DamageAnimation da1 = (DamageAnimation) baa[3];
		assertEquals("75.0", 		da1.getMessage());
		assertEquals(c1.getName(), 	da1.getTarget().getName());
		assertEquals(500,500, 		da1.getOldState().getLocation());
		assertEquals(0.95, 			da1.getOldState().getDamagePercent(), 0.01);
		assertEquals(blue,			da1.getOldState().getPlayerColor());
		assertEquals(1.0, 			da1.getOldState().getShieldPercent());
		assertEquals(500,500, 		da1.getNewState().getLocation());
		assertEquals(1.0, 			da1.getNewState().getDamagePercent(), 0.01);
		assertEquals(blue,			da1.getNewState().getPlayerColor());
		assertEquals(0.0,			da1.getNewState().getShieldPercent());

		DamageAnimation da2 = (DamageAnimation) baa[4];
		assertEquals("75.0", 		da2.getMessage());
		assertEquals(c3.getName(), 	da2.getTarget().getName());
		assertEquals(610,500, 		da2.getOldState().getLocation());
		assertEquals(0.0, 			da2.getOldState().getDamagePercent(), 0.01);
		assertEquals(green,			da2.getOldState().getPlayerColor());
		assertEquals(1.0, 			da2.getOldState().getShieldPercent());
		assertEquals(610,500, 		da2.getNewState().getLocation());
		assertEquals(0.7, 			da2.getNewState().getDamagePercent(), 0.01);
		assertEquals(green,			da2.getNewState().getPlayerColor());
		assertEquals(0.0,			da2.getNewState().getShieldPercent());
		
		DelayAnimation da = (DelayAnimation) baa[5];
		
    }     
    
    public void testFilesExist() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testFilesExist() *************************");

        String race = "Humans";
        RaceGraphics rg = GraphicsFactory.getRaceGraphicsList().getGraphicsFor(race);
        CraftGraphics [] cg = rg.getCraftGraphicsAsArray();
        for (int i = 0; i < cg.length; i++) {
        	if (cg[i].getName().contains("AirMan") || cg[i].getName().contains("WaterMan") || cg[i].getName().contains("kaan") || cg[i].getName().contains("Sentinel")) {
                String image = "resources/"+CraftGraphics.CRAFT_IMAGE_FOLDER+cg[i].getPictureNormal() + cg[i].getFileExtension();
    			assertTrue("Image "+image+" does not exist", new File(image).exists());
        	} else {
            	PlayerColor[] pcs = PlayerColor.gePlayerColors();
            	for (int j = 0; j < pcs.length; j++) {
                    String image = "resources/"+CraftGraphics.CRAFT_IMAGE_FOLDER+cg[i].getPictureNormal() + "_" + pcs[j] + cg[i].getFileExtension();
        			assertTrue("Image "+image+" does not exist", new File(image).exists());
        			if (cg[i].getPictureCloaked() != null) {
    	                String cloakedImage = "resources/"+CraftGraphics.CRAFT_IMAGE_FOLDER+cg[i].getPictureCloaked() + "_" + pcs[j] + cg[i].getFileExtension();
    	    			assertTrue("Image "+cloakedImage+" does not exist", new File(cloakedImage).exists());
        			}
    			}
        	}
		}
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

	@Override
	public void process(MotEvent e, LinkedEventBase parent) throws ApplicationError, ValidationError {
        Logger.logDebug("DeligateTest.process() EVENT "+e.toString());
        eventList.add(e);
	}
	
    protected void clearEvents() {
        Logger.logDebug("DeligateTest.notify() Clear Events ");
        eventList.clear();
        p1Proxy.clear();
        p2Proxy.clear();
    }

}
