/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.event.MotEvent;
import mot.common.proxy.ClientProxy;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Banker;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.relations.PeaceTreaty;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.relations.TradeAgreementTreaty;
import mot.engine.setup.MotFactory;

/**
 * Tests
 *  - Diplomatic relations are formed correctly (when scanners can meet see Player).
 *  - Relationships move toward values defined in player character.
 *  - Computer players form trade routes with eligible players.
 *  - At the end of every turn the correct revenue is generated for each participant of a trade route
 *  - Trading effects the relationship
 *  - Gifts effects relationships
 */
public class BankerAndTradeTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        TestSuite suite = new TestSuite(mot.test.BankerAndTradeTest.class);
        return suite;
    }

    public BankerAndTradeTest(String name) {
        super(name);
    }

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private ComputerPlayer proxyOwner;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;

    protected void setUp() throws Exception {
        MotEngine.newInstance();
        setupConfigs();
        
        Logger.setInMemory(false);
        Logger.setLogLevelLoaded();
    	Logger.setDebug();
    }

    public void createMap2() throws Exception {
        String [] races = {"Bears", "Birds"};
        MotFactory.setupRelations(null, races, null, null, false);

        player1 = newPlayer(races[0], blue, TestFactory.getBearsOptions(), races);
        player2 = newPlayer(races[1], green, TestFactory.getBirdsOptions(), races);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 301, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 200, 100);
        planet3 = newPlanet("SS3-A", ss3, null, 80, 1.0);

        proxyOwner = player1;
        MotEngine.getInstance().getProxyManager().add(this);
        MotEngine.getInstance().setDate(MotEngine.START_DATE+Banker.CALCULATE_START+1);
    }

    /**
     * Tests that Banker requests trade routes and that they are created when accepted.
     */
    public void testCreateDeleteTradeRoutes() throws Exception {
        Logger.logDebug("*******************************  BankerAndRelationsTest.testCreateDeleteTradeRoutes() *************************");
        createMap2();

        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationshipNotNull(player1, player2);

        assertFalse("No diplomatic relations - so cannot trade", Banker.canFormTradeRoute(player1, player2));

        player1.getBanker().assignTradeRoutes();

        assertNull("No diplomatic relations - so cannot trade", rel.getTreatiesAsArray());

        SolarSystem ss3 = newSolarSystem("SS3", 200, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, player1);
        player1.getBanker().assignTradeRoutes();
        MotEngine.instance.getRelations().updateHasDiplomaticRelations();

        assertTrue("Has diplomatic relations - so can trade", Banker.canFormTradeRoute(player1, player2));
        assertNull("Only form trade routes every Banker.CALCULATE_INTERVAL", rel.getTreatiesAsArray());

        MotEngine.instance.setDate(MotEngine.instance.getDate()+Banker.CALCULATE_INTERVAL+1);
        player1.getBanker().assignTradeRoutes();
        TradeAgreementTreaty tradeTreaty = (TradeAgreementTreaty) rel.getTreatiesAsArray()[0];

        assertEquals("Trade route formed this year", MotEngine.instance.getDate(), tradeTreaty.getDateStarted());

        // TRADE ROUTES ARE DELETED WHEN WE GO TO WAR AND CANNOT BE FORMED UNLESS WAR IS REMOVED

        player1.diplomaticRequest(player2, DiplomaticProposal.getWar(), null);

        assertTrue("Trade route deleted because of war", rels.isWar(player1, player2));
        assertFalse("Trade route deleted because of war", rels.isTrade(player1, player2));

        player1.getBanker().assignTradeRoutes();

        assertTrue("Trade route not created because of war", rels.isWar(player1, player2));
        assertFalse("Trade route not created because of war", rels.isTrade(player1, player2));

        rels.setPeace(player1, player2);
        MotEngine.instance.setDate(MotEngine.instance.getDate()+Banker.CALCULATE_INTERVAL);
        player1.getBanker().assignTradeRoutes();
        PeaceTreaty peaceTreaty = (PeaceTreaty) rel.getTreatiesAsArray()[0];
        tradeTreaty = (TradeAgreementTreaty) rel.getTreatiesAsArray()[1];

        assertFalse("Trade route created", rels.isWar(player1, player2));
        assertTrue("Trade route created", rels.isTrade(player1, player2));
        assertEquals("Trade route formed this year", MotEngine.instance.getDate(), tradeTreaty.getDateStarted());
    }

    /**
     * Test trade routes generate correct revenue and relationship benefits.
     */
    public void testTrade() throws Exception {
        Logger.logDebug("*******************************  BankerAndRelationsTest.testTrade() *************************");
        createMap2();

        planet1.setPopulation(10);
        planet2.setPopulation(10);
        player1.setFunds(0);
        player2.setFunds(0);

        ss2.setLocation(new Location(200,100));

        assertTrue("Can trade", Banker.canFormTradeRoute(player1, player2));

        player1.getBanker().assignTradeRoutes();
        Relation rel = MotEngine.instance.getRelations().getRelationship(player1, player2);

        assertTrue("Have formed Trade route", rel.containsTreaty(TradeAgreementTreaty.class));
        assertTrue("Have formed Trade route", rel.containsTreaty(new TradeAgreementTreaty(player1, player2)));
        assertEquals("No trade performed yet", 0, player1.getFunds());
        assertEquals("No trade performed yet", 0, rel.getTradeEffect());

        MotEngine.instance.getRelations().endTurn();
        TradeAgreementTreaty tradeTreaty = (TradeAgreementTreaty) rel.getTreatiesAsArray()[0];

        assertEquals("Trade route formed this year", 3011, tradeTreaty.getDateStarted());
        assertEquals("Trade costs money to setup", TradeAgreementTreaty.COST_TO_BUILD_ROUTE, player1.getFunds());
        assertEquals("Trade costs money to setup", TradeAgreementTreaty.COST_TO_BUILD_ROUTE, player2.getFunds());
        assertEquals("No Trade performed until route is TURNS_BEFORE_PROFIT turns old", 0, rel.getTradeEffect());
        assertEquals("No Trade performed until route is TURNS_BEFORE_PROFIT turns old", 50, rel.getRelationshipValue());

        player1.setFunds(0);
        player2.setFunds(0);
        MotEngine.instance.setDate(3011+TradeAgreementTreaty.TURNS_BEFORE_PROFIT);
        MotEngine.instance.getRelations().endTurn();

        assertEquals("Has correct population", 10, player1.getTotalPopulation());
        assertEquals("Has correct population", 10, player2.getTotalPopulation());
        assertEquals("Trade performed", 1, player1.getFunds());
        assertEquals("Trade performed", 1, player2.getFunds());
        assertEquals("Trade performed", 5, rel.getTradeEffect());
        assertEquals("Trade performed", 55, rel.getRelationshipValue());

        player1.setFunds(0);
        player2.setFunds(0);
        planet1.setPopulation(100);
        planet2.setPopulation(100);
        MotEngine.instance.getRelations().endTurn();

        assertEquals("Trade performed", 10, player1.getFunds());
        assertEquals("Trade performed", 10, player2.getFunds());
        assertEquals("Trade performed", 5, rel.getTradeEffect());
        assertEquals("Trade performed", 55, rel.getRelationshipValue());

        planet1.setPopulation(500);
        planet2.setPopulation(500);
        TradeAgreementTreaty tt = (TradeAgreementTreaty) rel.getTreaty(TradeAgreementTreaty.class);
        tt.setDateStarted(tt.getDateStarted()-50);
        MotEngine.instance.getRelations().endTurn();

        assertEquals("Trade performed", 60, player1.getFunds());
        assertEquals("Trade performed", 60, player2.getFunds());
        assertEquals("Trade performed", 10, rel.getTradeEffect());
        assertEquals("Trade performed", 60, rel.getRelationshipValue());

        // TEST PLAYER BONUSES
        player1.setFunds(0);
        player2.setFunds(0);
        player1.getPlayerCharacter().setTradingBonus(1.0);
        MotEngine.instance.getRelations().endTurn();

        assertEquals("Trade performed", 100, player1.getFunds());
        assertEquals("Trade performed", 50, player2.getFunds());
        assertEquals("Trade performed", 10, rel.getTradeEffect());
        assertEquals("Trade performed", 60, rel.getRelationshipValue());

        // TEST TECHNOLOGY BONUSES
        player1.setFunds(0);
        player2.setFunds(0);
        player1.addTechnology(tech_galacticExchangeI);
        MotEngine.instance.getRelations().endTurn();

        assertEquals("Trade performed", 105, player1.getFunds());
        assertEquals("Trade performed", 50, player2.getFunds());
        assertEquals("Trade performed", 10, rel.getTradeEffect());
        assertEquals("Trade performed", 60, rel.getRelationshipValue());

    }

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }

	public void notify(MotEvent e) throws ApplicationError {
    }

	public Player getOwner() {
        return proxyOwner;
    }

    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
}