/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import java.util.ArrayList;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.ImmunityMindControl;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.ObjectShift;
import mot.common.bean.ObjectTurn;
import mot.common.bean.Settings;
import mot.common.command_results.BooleanCR;
import mot.common.command_results.CommandResult;
import mot.common.designs.CraftDesign;
import mot.common.event.BattleCraftLeaveEvent;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.event.BattleCraftSurrenderEvent;
import mot.common.event.BattleCraftTeleportEvent;
import mot.common.event.BattleDamageEvent;
import mot.common.event.BattleDamageRingEvent;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.BattleDroidMineLaunchEvent;
import mot.common.event.BattleEndEvent;
import mot.common.event.BattleMissileLaunchEvent;
import mot.common.event.BattleObjectNotificationEvent;
import mot.common.event.BattleObjectTriggerEvent;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.BattleWeaponMissEvent;
import mot.common.event.MotEvent;
import mot.common.event.RoutedEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleCraftControllerMindControl;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.Missiles;
import mot.engine.command.BattleAttackCmd;
import mot.engine.command.BattleAttackLocationCmd;
import mot.engine.command.BattleCraftMoveToCmd;
import mot.engine.crafts.Craft;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.technology.WeaponInstanceRetarget;

public class BattleCommandTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(BattleCommandTest.class);
        //TestSuite suite = new TestSuite();
        //suite.addTest(new WarTest("testBattleStation"));
        return suite;
    }

    public BattleCommandTest(String name) {
        super(name);
    	try {
			Settings.getInstance();
		} catch (ApplicationError e) {
			e.printStackTrace();
		}
    }

    private SolarSystem ss1;
    private SolarSystem ss2;

    private UserPlayer userPlayer;
    private ComputerPlayer player2;

    private Planet planet1;
    private Planet planet2;
    
    private Battle battle;
    
    private Craft craft1;
    private Craft craft2;
    private Craft craft3;
    private Craft craft4;
    
    private BattleCraft bc1;
    private BattleCraft bc2;
    private BattleCraft bc3;
    private BattleCraft bc4;
    
    private BattlePlanet bp;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
    }
    
    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }

    public void createUserPlayerMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        userPlayer = newUserPlayer(playerNames[0], blue, null, options);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, userPlayer);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(userPlayer, player2);
    }

    private Battle newBattle(SolarSystem ss) {
        return MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss.getOrbit(), true);
    }

    public void createMapAndCraft(CraftDesign design1, CraftDesign design2, CraftDesign design3, CraftDesign design4, Weapons planetWeapons, Shield planetShield, SetupOptions optionsRace) throws Exception {
    	createUserPlayerMap(optionsRace);

        craft1 = newCraft(design1, userPlayer, ss1);
        craft2 = newCraft(design2, player2, ss1);
        if (design3 != null) craft3 = newCraft(design3, userPlayer, ss1);
        if (design4 != null) craft4 = newCraft(design4, player2, ss1);
        
        if (planetWeapons != null) {
        	planet1.setWeapons(planetWeapons);        	
        }
        if (planetShield != null) {
        	planet1.setShields(new Shields(planetShield));
        }

        battle = newBattle(ss1);
        battle.join(craft1, true);
        battle.join(craft2, false);
        if (craft3 != null) battle.join(craft3, true);
        if (craft4 != null) battle.join(craft4, false);
        if (planetWeapons != null) {
        	battle.join(planet1);
        	bp = battle.getPlanet();
        }
        battle.setup();
        
        bc1 = battle.getCraft(craft1.getName());
        bc2 = battle.getCraft(craft2.getName());
        if (craft3 != null) bc3 = battle.getCraft(craft3.getName());
        if (craft4 != null) bc4 = battle.getCraft(craft4.getName());
    }

    public void testCraftFireAllWeaponsAndDischarge() throws Exception {
        Logger.logDebug("*******************************  BattleCommandTest.testCraftFireAllWeaponsAndDischarge() *************************");
        CraftDesign tfd = new CraftDesign("AllFighter", tech_flyer30, tech_fusion25, 
        		tech_massDriver1, tech_massDriver1, 
        		tech_missile6, tech_missile6,
        		tech_fusionBeam4, tech_fusionBeam4,
        		tech_immobiliser, tech_immobiliser,
        		tech_shieldDrain, tech_shieldDrain,
        		tech_girator, tech_girator,
        		tech_psiBlast, tech_psiBlast,
        		tech_equipmentTransport, tech_equipmentTransport,
        		tech_detonator, tech_detonator,
        		tech_prioVirus, tech_prioVirus,
        		tech_troopTransport, tech_troopTransport,
        		tech_spacialDistortion, tech_spacialDistortion,
        		tech_plague, tech_plague,
        		tech_timeWarpGenerator, tech_timeWarpGenerator,
        		tech_damageBots, tech_damageBots,
        		tech_mindControl, tech_mindControl,
        		tech_berserker, tech_berserker,
        		tech_cowardice, tech_cowardice,
        		tech_droidMine, tech_droidMine,
        		tech_retargetBeam, tech_retargetBeam,
        		tech_timeWarpBeam, tech_timeWarpBeam,
        		tech_salvageBeam, tech_salvageBeam,
        		tech_replicaBeam, tech_replicaBeam,
        		tech_shieldBurst, tech_shieldBurst,
        		tech_teleporter, tech_teleporter,
        		tech_sleep, tech_sleep
        		);
        CraftDesign fighter = new CraftDesign("BigFighter", 
				tech_flyer100,
				tech_fusion25,
				tech_energy20,
				tech_armour200,
				tech_troopModule20,
				tech_missile6);
        createMapAndCraft(tfd, fighter, fighter, fighterDesign, null, null, null);

        // When MindControl weapons are fired, if first succeeds, then rest will not fire
        player2.getPlayerCharacter().setImmunity(new ImmunityMindControl());
        
        bc1.setLocation(new Location(400, 500));
        bc1.setDirection(90);
        bc2.setLocation(new Location(500, 500));
        bc2.setDirection(270);
        bc3.setLocation(new Location(450, 450));
        bc3.setDirection(270);
        
        
        bc2.setMoveRemaining(0.0);
        bc2.fireWeapon(bc3, bc2.getWeapon(tech_missile6), 1);
        Missiles missile = (Missiles) battle.getAutomatons()[0];
        missile.setMoveRemaining(0.0);
        bc4.setMoveRemaining(0.0);
        bc4.dischargeWeapons();
        
        // 
        // TARGET = CRAFT therefore
        // All weapons fired and discharged
        //
        
        bc2.getShields()[0].setPower(8);
        CommandResult r = new BattleAttackCmd(userPlayer, battle, bc1, bc2, true, true, true, null, null, -1, false).run();
        
        assertEquals(467,500, bc1.getLocation());
        assertEquals(1.0, bc1.getMoveRemaining());
        
        int i = 0;
        
        assertEquals(tech_retargetBeam, weaponFireEvents.get(i++).getWeaponTechnology());
        if (weaponFireEvents.get(i).getWeaponTechnology() == tech_retargetBeam) i++;
        if (weaponFireEvents.get(i).getWeaponTechnology() == tech_retargetBeam) i++;
        
        assertEquals(tech_immobiliser, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_immobiliser).getQtyShots());
        
        assertEquals(tech_shieldDrain, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_shieldDrain).getQtyShots());
        
        assertEquals(tech_psiBlast, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_psiBlast).getQtyShots());
        
        //assertEquals(tech_shieldBurst, damageRingEvent.getWeapon());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyShots());
        
        assertEquals(tech_prioVirus, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_prioVirus).getQtyShots());
        
        assertEquals(tech_troopTransport, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_troopTransport).getQtyShots());
        
        assertEquals(tech_massDriver1, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_massDriver1).getQtyShots());
        
        assertEquals(tech_fusionBeam4, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_fusionBeam4).getQtyShots());
        
        assertEquals(tech_girator, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_girator).getQtyShots());
        
        assertEquals(tech_equipmentTransport, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_equipmentTransport).getQtyShots());
        
        assertEquals(tech_plague, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_plague).getQtyShots());
        
        assertEquals(tech_damageBots, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_damageBots).getQtyShots());
        
        assertEquals(tech_mindControl, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_mindControl).getQtyShots());
        
        assertEquals(tech_berserker, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_berserker).getQtyShots());
        
        assertEquals(tech_cowardice, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_cowardice).getQtyShots());
        
        assertEquals(null, bc1.getWeapon(tech_spacialDistortion));
        
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_detonator).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_timeWarpBeam).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_salvageBeam).hasWeaponsToFire());
        
        assertEquals(1, bc1.getWeapon(tech_timeWarpGenerator).getQtyShots());
        assertEquals(false, bc1.getWeapon(tech_timeWarpGenerator).hasWeaponsToFire());

        // 
        // TIME WARP FIRED GIVING CRAFT A NEW TURN
        //
        
        weaponFireEvents.clear();
        bc1.setLocation(new Location(400, 500));
        bc1.setDirection(90);
        bc2.setLocation(new Location(500, 500));
        bc2.setDirection(270);
        bc2.getCraft().setDamage(0.0);
        bc2.setCrew(100);
        if (bc2.getShields() == null) bc2.addShieldInstance(tech_energy10);
        bc2.getShields()[0].setPower(8);
        
        r = new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, new LocationAndDirection(450,500,90), null, -1, false).run();
        
        assertEquals(450,500, bc1.getLocation());
        assertEquals(0.0, bc1.getMoveRemaining());
        
        i = 0;
        
        assertEquals(tech_immobiliser, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_immobiliser).getQtyShots());
        
        assertEquals(tech_shieldDrain, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_shieldDrain).getQtyShots());
        
        assertEquals(tech_psiBlast, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_psiBlast).getQtyShots());
        
        //assertEquals(tech_shieldBurst, damageRingEvent.getWeapon());
        assertEquals(2, bc1.getWeapon(tech_shieldBurst).getQtyShots());
        
        assertEquals(tech_prioVirus, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_prioVirus).getQtyShots());
        
        // only has one shot
        //assertEquals(tech_troopTransport, weaponFireEvents.get(i++).getWeapon());
        assertEquals(2, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_troopTransport).getQtyShots());
        
        assertEquals(tech_massDriver1, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_massDriver1).getQtyShots());
        
        assertEquals(tech_fusionBeam4, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_fusionBeam4).getQtyShots());
        
        assertEquals(tech_girator, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_girator).getQtyShots());
        
        assertEquals(tech_equipmentTransport, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_equipmentTransport).getQtyShots());
        
        assertEquals(tech_plague, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_plague).getQtyShots());
        
        assertEquals(tech_damageBots, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_damageBots).getQtyShots());
        
        assertEquals(tech_mindControl, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_mindControl).getQtyShots());
        
        assertEquals(tech_berserker, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_berserker).getQtyShots());
        
        assertEquals(tech_cowardice, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_cowardice).getQtyShots());
        
        assertEquals(null, bc1.getWeapon(tech_spacialDistortion));
        
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyShots());
        assertEquals(false, bc1.getWeapon(tech_detonator).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyShots());
        assertEquals(false, bc1.getWeapon(tech_timeWarpBeam).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyShots());
        assertEquals(false, bc1.getWeapon(tech_salvageBeam).hasWeaponsToFire());
        
        assertEquals(1, bc1.getWeapon(tech_timeWarpGenerator).getQtyShots());
        assertEquals(false, bc1.getWeapon(tech_timeWarpGenerator).hasWeaponsToFire());
        
        // 
        // TARGET = Automaton therefore
        // Only automaton weapons fired and discharged - DOES NO DISCHARGE ALL
        //
        
        weaponFireEvents.clear();
        bc1.rechargeWeapons();
        bc1.getWeapon(tech_timeWarpGenerator).discharge();
        bc1.setLocation(new Location(430, 500));
        bc1.setDirection(90);
        missile.retargetTo(bc1);
        missile.getWeapon(tech_missile6).setQty(4);
        
        new BattleAttackCmd(userPlayer, battle, bc1, missile, false, true, true, null, null, -1, false).run();
        
        assertEquals(430,500, bc1.getLocation());
        assertEquals(90, bc1.getDirection());
        assertEquals(0.0, bc1.getMoveRemaining());
        
        i = 0;
        
        assertEquals(tech_retargetBeam, weaponFireEvents.get(i++).getWeaponTechnology());
        if (weaponFireEvents.size() > i && weaponFireEvents.get(i).getWeaponTechnology() == tech_retargetBeam) i++;
        if (weaponFireEvents.size() > i && weaponFireEvents.get(i).getWeaponTechnology() == tech_retargetBeam) i++;
        
        assertEquals(0, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_immobiliser).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_immobiliser).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_shieldDrain).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_shieldDrain).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_psiBlast).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_psiBlast).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(2, bc1.getWeapon(tech_shieldBurst).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_shieldBurst).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_prioVirus).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_prioVirus).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_troopTransport).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_troopTransport).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_massDriver1).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_massDriver1).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_fusionBeam4).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_fusionBeam4).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_girator).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_girator).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_equipmentTransport).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_equipmentTransport).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_plague).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_plague).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_damageBots).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_damageBots).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_mindControl).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_mindControl).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_berserker).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_berserker).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_cowardice).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_cowardice).hasWeaponsToFire());
        
        assertEquals(null, bc1.getWeapon(tech_spacialDistortion));
        
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_detonator).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_timeWarpBeam).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_salvageBeam).hasWeaponsToFire());
        
        assertEquals(1, bc1.getWeapon(tech_timeWarpGenerator).getQtyShots());
        assertEquals(false, bc1.getWeapon(tech_timeWarpGenerator).hasWeaponsToFire());
        
        // 
        // TARGET = Location therefore
        // Only location weapons fired and discharged - DOES NO DISCHARGE ALL
        //
        
        weaponFireEvents.clear();
        bc1.rechargeWeapons();
        bc1.getWeapon(tech_timeWarpGenerator).discharge();
        bc1.setLocation(new Location(400, 500));
        bc1.setDirection(180);
        bc1.setMoveRemaining(1.0);
        missile.retargetTo(bc1);
        missile.getWeapon(tech_missile6).setQty(4);
        
        new BattleAttackLocationCmd(userPlayer, battle, bc1, new Location(400,550), false, null, -1).run();
        
        assertEquals(400,500, bc1.getLocation());
        assertEquals(180, bc1.getDirection());
        assertEquals(1.0, bc1.getMoveRemaining());
        
        i = 0;
        
        assertEquals(0, bc1.getWeapon(tech_retargetBeam).getQtyFired());
        assertEquals(true, bc1.getWeapon(tech_retargetBeam).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_immobiliser).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_immobiliser).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_shieldDrain).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_shieldDrain).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_psiBlast).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_psiBlast).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(2, bc1.getWeapon(tech_shieldBurst).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_shieldBurst).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_prioVirus).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_prioVirus).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_troopTransport).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_troopTransport).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_massDriver1).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_massDriver1).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_fusionBeam4).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_fusionBeam4).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_girator).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_girator).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_equipmentTransport).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_equipmentTransport).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_plague).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_plague).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_damageBots).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_damageBots).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_mindControl).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_mindControl).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_berserker).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_berserker).hasWeaponsToFire());
        
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(4, bc1.getWeapon(tech_cowardice).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_cowardice).hasWeaponsToFire());
        
        assertEquals(null, bc1.getWeapon(tech_spacialDistortion));
        
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyShots());
        assertEquals(true, bc1.getWeapon(tech_detonator).hasWeaponsToFire());
        
        assertEquals(tech_timeWarpBeam, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyShots());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(false, bc1.getWeapon(tech_timeWarpBeam).hasWeaponsToFire());
        
        assertEquals(tech_salvageBeam, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyShots());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(true, bc1.getWeapon(tech_salvageBeam).hasWeaponsToFire());
        
        assertEquals(1, bc1.getWeapon(tech_timeWarpGenerator).getQtyShots());
        assertEquals(false, bc1.getWeapon(tech_timeWarpGenerator).hasWeaponsToFire());
        
    }

    public void testIndividualWeaponFire() throws Exception {
        Logger.logDebug("*******************************  BattleCommandTest.testIndividualWeaponFireWithoutMove() *************************");
        CraftDesign tfd = new CraftDesign("AllFighter", tech_flyer30, tech_fusion25, 
        		tech_massDriver1, tech_massDriver1, tech_massDriver1, 
        		tech_missile6, tech_missile6, tech_missile6,
        		tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4,
        		tech_immobiliser, tech_immobiliser, tech_immobiliser,
        		tech_shieldDrain, tech_shieldDrain, tech_shieldDrain,
        		tech_girator, tech_girator, tech_girator,
        		tech_psiBlast, tech_psiBlast, tech_psiBlast,
        		tech_equipmentTransport, tech_equipmentTransport, tech_equipmentTransport,
        		tech_detonator, tech_detonator, tech_detonator,
        		tech_prioVirus, tech_prioVirus, tech_prioVirus,
        		tech_troopTransport, tech_troopTransport, tech_troopTransport,
        		tech_spacialDistortion, tech_spacialDistortion, tech_spacialDistortion,
        		tech_plague, tech_plague, tech_plague,
        		tech_timeWarpGenerator, tech_timeWarpGenerator, tech_timeWarpGenerator,
        		tech_damageBots, tech_damageBots, tech_damageBots,
        		tech_mindControl, tech_mindControl, tech_mindControl,
        		tech_berserker, tech_berserker, tech_berserker,
        		tech_cowardice, tech_cowardice, tech_cowardice,
        		tech_droidMine, tech_droidMine, tech_droidMine,
        		tech_retargetBeam, tech_retargetBeam, tech_retargetBeam,
        		tech_timeWarpBeam, tech_timeWarpBeam, tech_timeWarpBeam,
        		tech_salvageBeam, tech_salvageBeam, tech_salvageBeam,
        		tech_replicaBeam, tech_replicaBeam, tech_replicaBeam,
        		tech_shieldBurst, tech_shieldBurst, tech_shieldBurst,
        		tech_teleporter, tech_teleporter, tech_teleporter,
        		tech_sleep, tech_sleep, tech_sleep
        		);
        CraftDesign fighter = new CraftDesign("BigFighter", 
				tech_flyer100,
				tech_fusion25,
				tech_energy20,
				tech_missile6);
        createMapAndCraft(tfd, fighter, fighter, fighterDesign, null, null, null);

        bc1.setLocation(new Location(450, 500));
        bc1.setDirection(90);
        bc2.setLocation(new Location(500, 500));
        bc2.setDirection(270);
        bc3.setLocation(new Location(450, 500));
        bc3.setDirection(270);
        
        
        bc2.setMoveRemaining(0.0);
        bc2.fireWeapon(bc3, bc2.getWeapon(tech_missile6), 1);
        Missiles missile = (Missiles) battle.getAutomatons()[0];
        bc4.setMoveRemaining(0.0);
        bc4.dischargeWeapons();
        
        // 
        // TARGET = CRAFT therefore
        // All weapons fired and discharged
        //
        
        missileLaunchEvent = null;
        weaponFireEvents.clear();
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, false, null, tech_missile6, 1, false).run();
        
        assertEquals(0, bc1.getWeapon(tech_retargetBeam).getQtyFired());
        assertEquals(true, bc1.getWeapon(tech_retargetBeam).hasWeaponsToFire());
        
        assertEquals(450,500, missileLaunchEvent.getMissile().getState().getLocation());
        assertEquals(1, bc1.getWeapon(tech_missile6).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_immobiliser, 1, false).run();
        
        assertEquals(tech_immobiliser, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_shieldDrain, 1, false).run();
        
        assertEquals(tech_shieldDrain, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_psiBlast, 1, false).run();
        
        assertEquals(tech_psiBlast, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_shieldBurst, 1, false).run();
        
        //assertEquals(tech_shieldBurst, weaponFireEvents.get(0).getWeapon());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_prioVirus, 1, false).run();
        
        assertEquals(tech_prioVirus, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_troopTransport, 1, false).run();
        
        assertEquals(tech_troopTransport, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_fusionBeam4, 1, false).run();
        
        assertEquals(tech_fusionBeam4, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_massDriver1, 1, false).run();
        
        assertEquals(tech_massDriver1, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_equipmentTransport, 1, false).run();
        
        assertEquals(tech_equipmentTransport, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_girator, 1, false).run();
        
        assertEquals(tech_girator, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_plague, 1, false).run();
        
        assertEquals(tech_plague, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_damageBots, 1, false).run();
        
        assertEquals(tech_damageBots, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());


        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_mindControl, 1, false).run();
        
        assertEquals(tech_mindControl, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_berserker, 1, false).run();
        
        assertEquals(tech_berserker, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_cowardice, 1, false).run();
        
        assertEquals(tech_cowardice, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackLocationCmd(userPlayer, battle, bc1, new Location(450,500), false, tech_salvageBeam, 1).run();
        
        assertEquals(tech_salvageBeam, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackLocationCmd(userPlayer, battle, bc1, new Location(450,500), false, tech_timeWarpBeam, 1).run();
        
        assertEquals(tech_timeWarpBeam, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());
        
        
        
        
        
        
        //
        // NOW FIR ALL REMAINING WEAPONS
        //
        
        
        
        
        
        
        bc2.getCraft().setDamage(0.0);
        missileLaunchEvent = null;
        weaponFireEvents.clear();
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_missile6, -1, false).run();
        
        assertEquals(450,500, missileLaunchEvent.getMissile().getState().getLocation());
        assertEquals(3, bc1.getWeapon(tech_missile6).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_immobiliser, -1, false).run();
        
        assertEquals(tech_immobiliser, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_shieldDrain, -1, false).run();
        
        assertEquals(tech_shieldDrain, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_psiBlast, -1, false).run();
        
        assertEquals(tech_psiBlast, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_shieldBurst, -1, false).run();
        
        //assertEquals(tech_shieldBurst, weaponFireEvents.get(0).getWeapon());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_prioVirus, -1, false).run();
        
        assertEquals(tech_prioVirus, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_troopTransport, -1, false).run();
        
        assertEquals(tech_troopTransport, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_fusionBeam4, -1, false).run();
        
        assertEquals(tech_fusionBeam4, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        bc2.getCraft().setDamage(0.0);
        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_massDriver1, -1, false).run();
        
        assertEquals(tech_massDriver1, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        bc2.getCraft().setDamage(0.0);
        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_equipmentTransport, -1, false).run();
        
        assertEquals(tech_equipmentTransport, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_girator, -1, false).run();
        
        assertEquals(tech_girator, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_plague, -1, false).run();
        
        assertEquals(tech_plague, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_damageBots, -1, false).run();
        
        assertEquals(tech_damageBots, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());


        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_mindControl, -1, false).run();
        
        assertEquals(tech_mindControl, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_berserker, -1, false).run();
        
        assertEquals(tech_berserker, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, false, true, true, null, tech_cowardice, -1, false).run();
        
        assertEquals(tech_cowardice, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

        weaponFireEvents.clear();        
        new BattleAttackLocationCmd(userPlayer, battle, bc1, new Location(450,500), false, tech_salvageBeam, -1).run();
        
        assertEquals(tech_salvageBeam, weaponFireEvents.get(0).getWeaponTechnology());
        assertEquals(3, bc1.getWeapon(tech_immobiliser).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_psiBlast).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_shieldBurst).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_prioVirus).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_troopTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_fusionBeam4).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_equipmentTransport).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_girator).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_plague).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_damageBots).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_mindControl).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_berserker).getQtyFired());
        assertEquals(3, bc1.getWeapon(tech_cowardice).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_detonator).getQtyFired());
        assertEquals(2, bc1.getWeapon(tech_salvageBeam).getQtyFired());
        assertEquals(1, bc1.getWeapon(tech_timeWarpBeam).getQtyFired());
        assertEquals(0, bc1.getWeapon(tech_timeWarpGenerator).getQtyFired());

    }

    
    public void testPlanetFireAllWeapons() throws Exception {
        Logger.logDebug("*******************************  BattleCommandTest.testPlanetFireAllWeapons() *************************");
        CraftDesign tfd = new CraftDesign("AllFighter", tech_flyer100, tech_fusion25, tech_energy20, tech_armour200, 
        		tech_massDriver1, 
        		tech_missile6,
        		tech_fusionBeam4,
        		tech_immobiliser,
        		tech_shieldDrain,
        		tech_girator,
        		tech_psiBlast,
        		tech_equipmentTransport,
        		tech_detonator,
        		tech_prioVirus,
        		tech_troopTransport,
        		tech_spacialDistortion,
        		tech_plague,
        		tech_timeWarpGenerator,
        		tech_damageBots,
        		tech_mindControl,
        		tech_berserker,
        		tech_cowardice,
        		tech_droidMine,
        		tech_retargetBeam,
        		tech_timeWarpBeam,
        		tech_salvageBeam,
        		tech_replicaBeam,
        		tech_shieldBurst,
        		tech_teleporter,
        		tech_sleep);
        Weapons planetWeapons = new Weapons(tech_massDriver1, 
        		tech_missile6,
        		tech_fusionBeam4,
        		tech_immobiliser,
        		tech_shieldDrain,
        		tech_girator,
        		tech_psiBlast,
        		tech_equipmentTransport,
        		tech_detonator,
        		tech_prioVirus,
        		tech_troopTransport,
        		tech_spacialDistortion,
        		tech_plague,
        		tech_damageBots,
        		tech_mindControl,
        		tech_berserker,
        		tech_cowardice,
        		tech_droidMine,
        		tech_retargetBeam,
        		tech_timeWarpBeam,
        		tech_salvageBeam,
        		tech_replicaBeam,
        		tech_shieldBurst,
        		tech_teleporter,
        		tech_sleep);
        
        createMapAndCraft(fighterDesign, tfd, null, null, planetWeapons, tech_energy10, null);

        // When MindControl weapons are fired, if first succeeds, then rest will not fire
        userPlayer.getPlayerCharacter().setImmunity(new ImmunityMindControl());
        player2.getPlayerCharacter().setImmunity(new ImmunityMindControl());
        
        bc1.setLocation(new Location(100, 600));
        bc1.setDirection(90);
        bc2.setLocation(new Location(150, 500));
        bc2.setDirection(270);
        bc1.setMoveRemaining(0.0);
        bc1.dischargeWeapons();
        bp.getShields()[0].setPower(4.0);
        
        CommandResult r = new BattleAttackCmd(player2, battle, bc2, bp, false, false, false, null, null, -1, false).run();
        
        assertEquals(150,500, bc2.getLocation());
        assertEquals(1.0, bc2.getMoveRemaining());
        assertEquals(true, bc2.hasWeaponsToFire());
        
        Missiles missiles = (Missiles) battle.getAutomatons()[0];
        assertEquals(bp, missiles.getTarget());
        bp.getShields()[0].setPower(1.0);
        
        int i = 0;

        // Immobiliser did not fire at a planet
        assertEquals(true, bc2.getWeapon(tech_immobiliser).hasWeaponsToFire());
        assertEquals(0, bc2.getWeapon(tech_immobiliser).getQtyShots());
        
        assertEquals(tech_shieldDrain, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_shieldDrain).getQtyShots());
        
        assertEquals(tech_psiBlast, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_psiBlast).getQtyShots());
        
        //assertEquals(tech_shieldBurst, damageRingEvent.getWeapon());
        assertEquals(1, bc2.getWeapon(tech_shieldBurst).getQtyShots());
        
        assertEquals(tech_prioVirus, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_prioVirus).getQtyShots());
        
        assertEquals(tech_troopTransport, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_troopTransport).getQtyShots());
        
        assertEquals(tech_massDriver1, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_massDriver1).getQtyShots()); 
        
        assertEquals(tech_fusionBeam4, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_fusionBeam4).getQtyShots()); 
        
        assertEquals(tech_girator, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_girator).getQtyShots());
        
        assertEquals(tech_equipmentTransport, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_equipmentTransport).getQtyShots());
        
        assertEquals(tech_plague, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_plague).getQtyShots());
        
        assertEquals(tech_damageBots, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_damageBots).getQtyShots());
        
        // Mind Control did not fire at a planet
        assertEquals(true, bc2.getWeapon(tech_mindControl).hasWeaponsToFire());
        assertEquals(0, bc2.getWeapon(tech_mindControl).getQtyShots());
        
        // Mind Control did not fire at a planet
        assertEquals(true, bc2.getWeapon(tech_berserker).hasWeaponsToFire());
        assertEquals(0, bc2.getWeapon(tech_berserker).getQtyShots());
        
        // Mind Control did not fire at a planet
        assertEquals(true, bc2.getWeapon(tech_cowardice).hasWeaponsToFire());
        assertEquals(0, bc2.getWeapon(tech_cowardice).getQtyShots());
        
        assertEquals(tech_replicaBeam, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(0, bc2.getWeapon(tech_replicaBeam).getQtyShots());
        
        assertEquals(tech_sleep, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bc2.getWeapon(tech_sleep).getQtyShots());
        
        assertEquals(null, bc2.getWeapon(tech_spacialDistortion));
        
        assertEquals(0, bc2.getWeapon(tech_detonator).getQtyShots());
        assertEquals(true, bc2.getWeapon(tech_detonator).hasWeaponsToFire());
        
        assertEquals(0, bc2.getWeapon(tech_timeWarpBeam).getQtyShots());
        assertEquals(true, bc2.getWeapon(tech_timeWarpBeam).hasWeaponsToFire());
        
        assertEquals(0, bc2.getWeapon(tech_salvageBeam).getQtyShots());
        assertEquals(true, bc2.getWeapon(tech_salvageBeam).hasWeaponsToFire());
        
        //assertEquals(tech_teleporter, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(0, bc2.getWeapon(tech_teleporter).getQtyShots());
        
        assertEquals(1, bc2.getWeapon(tech_timeWarpGenerator).getQtyShots());
        assertEquals(false, bc2.getWeapon(tech_timeWarpGenerator).hasWeaponsToFire());

        bc2.setMoveRemaining(0.0);
        bc2.dischargeWeapons();
        bc2.getShields()[0].setPower(8);
        missiles.destroy(null, null, null); // These are the missiles fired from the craft
        weaponFireEvents.clear();
        r = new BattleAttackCmd(userPlayer, battle, bp, bc2, true, false, false, null, null, -1, false).run();
        
        assertEquals(100,500, bp.getLocation());
        assertEquals(true, bp.hasWeaponsToFire());
        
        i = 0;
        
        assertEquals(tech_immobiliser, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_immobiliser).getQtyShots());
        
        assertEquals(tech_shieldDrain, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_shieldDrain).getQtyShots());
        
        assertEquals(tech_psiBlast, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_psiBlast).getQtyShots());
        
        //assertEquals(tech_shieldBurst, damageRingEvent.getWeapon());
        assertEquals(1, bp.getWeapon(tech_shieldBurst).getQtyShots());
        
        assertEquals(tech_prioVirus, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_prioVirus).getQtyShots());
        
        assertEquals(tech_troopTransport, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_troopTransport).getQtyShots());
        
        assertEquals(tech_massDriver1, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bp.getWeapon(tech_massDriver1).getQtyShots()); // Planet gets Qty Weapons Bonus
        
        assertEquals(tech_fusionBeam4, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(2, bp.getWeapon(tech_fusionBeam4).getQtyShots()); // Planet gets Qty Weapons Bonus
        
        assertEquals(tech_girator, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_girator).getQtyShots());
        
        assertEquals(tech_equipmentTransport, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_equipmentTransport).getQtyShots());
        
        assertEquals(tech_plague, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_plague).getQtyShots());
        
        assertEquals(tech_damageBots, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_damageBots).getQtyShots());
        
        assertEquals(tech_mindControl, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_mindControl).getQtyShots());
        
        assertEquals(tech_berserker, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_berserker).getQtyShots());
        
        assertEquals(tech_cowardice, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_cowardice).getQtyShots());
        
        assertEquals(tech_replicaBeam, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(0, bp.getWeapon(tech_replicaBeam).getQtyShots());
        
        assertEquals(tech_sleep, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_sleep).getQtyShots());
        
        assertEquals(tech_teleporter, weaponFireEvents.get(i++).getWeaponTechnology());
        assertEquals(1, bp.getWeapon(tech_teleporter).getQtyShots());
        
        //assertEquals(tech_spacialDistortion, weaponFireEvents.get(i++).getWeapon());
        assertEquals(0, bp.getWeapon(tech_spacialDistortion).getQtyShots());
        
        assertEquals(0, bp.getWeapon(tech_detonator).getQtyShots());
        assertEquals(true, bp.getWeapon(tech_detonator).hasWeaponsToFire());
        
        assertEquals(0, bp.getWeapon(tech_timeWarpBeam).getQtyShots());
        assertEquals(true, bp.getWeapon(tech_timeWarpBeam).hasWeaponsToFire());
        
        assertEquals(0, bp.getWeapon(tech_salvageBeam).getQtyShots());
        assertEquals(true, bp.getWeapon(tech_salvageBeam).hasWeaponsToFire());
        
        
    }
    
    public void testStopOnMissileAttack() throws Exception {
        Logger.logDebug("*******************************  BattleCommandTest.testStopOnMissileAttack() *************************");
        CraftDesign mfd = new CraftDesign("MissileFighter", tech_flyer30, tech_fusion25, tech_missile6 );
        createMapAndCraft(fighterDesign, mfd, null, null, null, null, null);

        bc1.setLocation(new Location(400, 500));
        bc1.setDirection(90);
        bc2.setLocation(new Location(500, 500));
        bc2.setDirection(270);       
        
        bc2.setMoveRemaining(0.0);
        bc2.fireWeapon(bc1, bc2.getWeapon(tech_missile6), 1);
        Missiles missile = (Missiles) battle.getAutomatons()[0];
        
        // 
        // TARGET = CRAFT therefore
        // All weapons fired and discharged
        //
        
        weaponFireEvents.clear();
        CommandResult r = new BattleAttackCmd(userPlayer, battle, bc1, bc2, true, true, true, null, null, 1, false).run();
        
        assertEquals("Stopped on missile attack", true, ((BooleanCR)r).isResult());

        assertEmptyArray(bc1.getMoveLinks());
        assertEmptyArray(battle.getAutomatons());
        assertEquals(5.0, bc1.getDamage());
    }
    
    
    
    /**
     * Tests that crafts can move a percentage of their total move and remaining move is correct.
     * Tests that crafts cannot move more than a full move in one turn.
     */
    public void testCraftMovementToSameLocation() throws Exception {
        Logger.logDebug("*******************************  BattleCommandTest.testCraftMovementToSameLocation() *************************");
        CraftDesign tfd = new CraftDesign("TeleportFighter", tech_flyer30, tech_teleport);
        createMapAndCraft(tfd, fighterDesign, null, null, null, null, null);

        ObjectTurn turn;
        ObjectShift shift;

        bc1.setLocation(new Location(100, 500));
        bc1.setDirection(0);
        bc2.setLocation(new Location(500, 500));
        bc2.setDirection(0);

        // 
        // Move craft to same location but different location
        //
        lastCraftTeleportEvent = null;
        new BattleCraftMoveToCmd(userPlayer, battle, bc1, new LocationAndDirection(100, 500, 270), null, false, false, false).run();

        assertEquals(0, lastCraftTeleportEvent.getOldState().getDirection());
        assertEquals(270, lastCraftTeleportEvent.getNewState().getDirection());
        
        assertEquals(270, bc1.getDirection());
        assertEquals(100,500, bc1.getLocation());
        
        // 
        // Move craft to same location but different location
        //
        lastCraftMovementEvent = null;
        new BattleCraftMoveToCmd(player2, battle, bc2, new LocationAndDirection(500, 500, 90), null, false, false, false).run();

        assertEquals(1, lastCraftMovementEvent.getMovements().length);
        turn = (ObjectTurn) lastCraftMovementEvent.getMovements()[0];
        assertEquals(90, turn.getTo());
        
        assertEquals(90, bc2.getDirection());
        assertEquals(500,500, bc2.getLocation());
    }

    public void testMovementMissileDefence() throws Exception {
        Logger.logDebug("*******************************  BattleCommandTest.testMovementMissileDefence() *************************");
        CraftDesign missileDesign = new CraftDesign("MF1", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_missile6);
        CraftDesign retargetDesign = new CraftDesign("RT1", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_retargetBeam);
        createMapAndCraft(retargetDesign, missileDesign, null, missileDesign, null, null, null);

        bc1.setLocation(new Location(300, 500));
        bc1.setDirection(90);
        bc2.setLocation(new Location(500, 500));
        bc2.setDirection(270);
        bc4.setLocation(new Location(550, 500));
        bc4.setDirection(270);
        
        bc4.setMoveRemaining(0.0);
        bc4.dischargeWeapons();
        
        //
        // Craft2 FIRE MISSILE at craft1
        //
        
        assertTrue(bc2.inWeaponRange(bc1.getLocation()));
        bc2.fireAllWeapons(bc1);
        bc2.setMoveRemaining(0.0);
        
        Automaton [] automitons = battle.getAutomatons();
        assertEquals(1, automitons.length);
        Missiles missile = (Missiles) automitons[0];
        assertEquals(bc2, missile.getOrigin());
        assertEquals(bc1, missile.getTarget());
        assertEquals(player2, missile.getOwner());
        
        //
        // TARGET ATTACKS MISSILE BEFORE MOVING
        //

        missile.setLocation(new Location(400,500));
        weaponFireEvent = null;
        Utils.clearAndSetNextRandomProbability(0.9);
        Utils.setNextRandomProbability(0.4);
        new BattleCraftMoveToCmd(userPlayer, battle, bc1, new LocationAndDirection(450, 500, 90), null, true, true, false).run();
        
        assertEquals(tech_retargetBeam, weaponFireEvent.getWeaponTechnology());
        assertEquals(missile.getName(), weaponFireEvent.getTarget());
        assertEquals(bc2, missile.getOrigin());
        assertEquals(bc2, missile.getTarget());
        
        ((WeaponInstanceRetarget)bc1.getWeapons()[0]).recharge();
        weaponFireEvent = null;
        Utils.clearAndSetNextRandomProbability(0.9);
        Utils.setNextRandomProbability(0.4);
        bc1.rechargeWeapons();
        new BattleAttackCmd(userPlayer, battle, bc1, bc2, true, true, true, null, null, 0, false).run();
        
        assertEquals(bc2, missile.getOrigin());
        assertEquals(bc2, missile.getTarget());
        
        //
        // Test with three missiles and the closest missile is mine, the second closest has been retargetted to the enemy
        // so it should select the farthest missile to retarget.
        //
        
        bc1.setLocation(new Location(400,500));
        bc1.setDirection(90);
        bc2.setLocation(new Location(500,500));
        bc2.setDirection(270);
        bc4.setLocation(new Location(475,500));
        bc4.setDirection(270);

        missile.setLocation(new Location(425,500));

        bc4.rechargeWeapons();
        bc4.fireAllWeapons(bc2);
        bc2.rechargeWeapons();
        bc2.fireAllWeapons(bc1);

        automitons = battle.getAutomatons();
        assertEquals(3, automitons.length);
        Missiles missile1 = (Missiles) automitons[0];
        Missiles missile2 = (Missiles) automitons[1];
        Missiles missile3 = (Missiles) automitons[2];
        
        assertEquals(bc2, missile1.getOrigin());
        assertEquals(bc2, missile1.getTarget());
        assertEquals(player2, missile1.getOwner());
        assertEquals(425,500, missile1.getLocation());
        
        assertEquals(bc4, missile2.getOrigin());
        assertEquals(bc2, missile2.getTarget());
        assertEquals(player2, missile2.getOwner());
        assertEquals(475,500, missile2.getLocation());
        
        assertEquals(bc2, missile3.getOrigin());
        assertEquals(bc1, missile3.getTarget());
        assertEquals(player2, missile3.getOwner());
        assertEquals(500,500, missile3.getLocation());
        
        ((WeaponInstanceRetarget)bc1.getWeapons()[0]).recharge();
        weaponFireEvent = null;
        Utils.clearAndSetNextRandomProbability(0.4);
        Utils.setNextRandomProbability(0.4);
        bc1.setMoveRemaining(1.0);
        new BattleCraftMoveToCmd(userPlayer, battle, bc1, new LocationAndDirection(495, 500, 270), null, true, true, false).run();
        
        assertEquals(tech_retargetBeam, weaponFireEvent.getWeaponTechnology());
        assertEquals(missile3.getName(), weaponFireEvent.getTarget());
        assertEquals(bc2, missile3.getOrigin());
        assertEquals(bc2, missile3.getTarget());
    }

    
    

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	private ArrayList<MotEvent> events = new ArrayList();
	private ArrayList<BattleWeaponFireEvent> weaponFireEvents = new ArrayList();
    private MotEvent lastEvent = null;
    private BattleCraftMoveEvent lastCraftMovementEvent;
    private BattleCraftTeleportEvent lastCraftTeleportEvent;
    private BattleDamageEvent lastDamageEvent;
    private BattleObjectTriggerEvent lastTriggerEvent;
    private BattleEndEvent endBattleEvent;
    private RoutedEvent routedEvent;
    private BattleWeaponFireEvent weaponFireEvent;
    private BattleObjectNotificationEvent notificationEvent;
    private BattleWeaponMissEvent missTargetEvent;
    private BattleCraftSurrenderEvent craftSurrenderEvent;
    private BattleMissileLaunchEvent missileLaunchEvent;
    private BattleDroidMineLaunchEvent droidMineLaunchEvent;
    private BattleDamageRingEvent damageRingEvent;
    
	public void notify(MotEvent e) throws ApplicationError {
		lastEvent = e;
		events.add(e);
        if (e instanceof BattleCraftLeaveEvent) {
            BattleCraftLeaveEvent event = (BattleCraftLeaveEvent)e;
            //BattleCraftDelegate craftDeligate = event.getCraft();
            //BattleCraft craft = craftDeligate.getBattleCraft();
        } else if (e instanceof BattleDestroyedEvent) {
            BattleDestroyedEvent event = (BattleDestroyedEvent)e;
        } else if (e instanceof BattleWeaponFireEvent) {
            weaponFireEvent = (BattleWeaponFireEvent) e;
            weaponFireEvents.add(weaponFireEvent);
        } else if (e instanceof BattleDamageEvent) {
            lastDamageEvent = (BattleDamageEvent) e;
        } else if (e instanceof BattleWeaponMissEvent) {
        	missTargetEvent = (BattleWeaponMissEvent) e;
        } else if (e instanceof BattleCraftSurrenderEvent) {
        	craftSurrenderEvent = (BattleCraftSurrenderEvent) e;
        } else if (e instanceof BattleMissileLaunchEvent) {
        	missileLaunchEvent = (BattleMissileLaunchEvent) e;
        } else if (e instanceof BattleDamageRingEvent) {
        	damageRingEvent = (BattleDamageRingEvent) e;
        } else if (e instanceof BattleDroidMineLaunchEvent) {
        	droidMineLaunchEvent = (BattleDroidMineLaunchEvent) e;
        } else if (e instanceof BattleCraftMoveEvent) {
            lastCraftMovementEvent = (BattleCraftMoveEvent)e;
        } else if (e instanceof BattleObjectNotificationEvent) {
        	notificationEvent = (BattleObjectNotificationEvent)e;
        } else if (e instanceof BattleCraftTeleportEvent) {
        	lastCraftTeleportEvent = (BattleCraftTeleportEvent)e;
        } else if ( e instanceof BattleObjectTriggerEvent) {
        	lastTriggerEvent = (BattleObjectTriggerEvent)e;
        } else if ( e instanceof BattleEndEvent) {
        	endBattleEvent = (BattleEndEvent)e;
        	Logger.logDebug("EndBattleEvent received in Battle Test "+Comp.toString(endBattleEvent.getCraftDestroyd()));
        } else if ( e instanceof RoutedEvent) {
        	routedEvent = (RoutedEvent)e;
        	Logger.logDebug("RoutedEvent received in Battle Test "+Comp.toString(routedEvent.getCraftDestroyed()));
        } 
    }

    public void testCraftFireAllWeaponsAndDischargeWithWeaponsWithNoEffect() throws Exception {
        Logger.logDebug("*******************************  BattleCommandTest.testCraftFireAllWeaponsAndDischargeWithWeaponsWithNoEffect() *************************");
        CraftDesign tfd = new CraftDesign("AllFighter", tech_flyer30, tech_fusion25, tech_energy10,
        		tech_massDriver1, 
        		tech_immobiliser,
        		tech_shieldDrain,
        		tech_mindControl
        		);
        createMapAndCraft(tfd, fighterDesign, fighterDesign, fighterDesign, null, null, null);

        bc1.setLocation(new Location(400, 500));
        bc1.setDirection(90);
        bc2.setLocation(new Location(450, 500));
        bc2.setDirection(270);
        bc3.setLocation(new Location(450, 450));
        bc3.setDirection(90);
        bc4.setLocation(new Location(500, 500));
        bc4.setDirection(270);
        
        bc2.setMoveRemaining(0.0);
        bc2.dischargeWeapons();
        bc4.setMoveRemaining(0.0);
        bc4.dischargeWeapons();
        
        // 
        // bc1 fire all weapons at bc2 with discharge = true so weapons with effect are used on bc4
        //
        
        bc2.getShields()[0].setPower(0);
        bc2.setControllerDetails(new BattleCraftControllerMindControl(userPlayer, bc2, 1));
        bc2.setMovementDamage(bc2.getSpeed());
        
        CommandResult r = new BattleAttackCmd(userPlayer, battle, bc1, bc2, true, true, true, null, null, -1, false).run();
        
        assertEquals(425,500, bc1.getLocation());
        assertEquals(0.0, bc1.getMoveRemaining());
        
        int i = 0;
        
        BattleWeaponFireEvent we = weaponFireEvents.get(i++);
		assertEquals(tech_massDriver1, we.getWeaponTechnology());
		assertEquals(bc1.getName(), we.getSource());
		assertEquals(bc2.getName(), we.getTarget());
        assertEquals(1, bc1.getWeapon(tech_massDriver1).getQtyShots());
        
        we = weaponFireEvents.get(i++);
		assertEquals(tech_immobiliser, we.getWeaponTechnology());
		assertEquals(bc1.getName(), we.getSource());
		assertEquals(bc4.getName(), we.getTarget());
        assertEquals(1, bc1.getWeapon(tech_immobiliser).getQtyShots());
        
        we = weaponFireEvents.get(i++);
		assertEquals(tech_shieldDrain, we.getWeaponTechnology());
		assertEquals(bc1.getName(), we.getSource());
		assertEquals(bc4.getName(), we.getTarget());
        assertEquals(1, bc1.getWeapon(tech_shieldDrain).getQtyShots());
        
        we = weaponFireEvents.get(i++);
		assertEquals(tech_mindControl, we.getWeaponTechnology());
		assertEquals(bc1.getName(), we.getSource());
		assertEquals(bc4.getName(), we.getTarget());
        assertEquals(1, bc1.getWeapon(tech_mindControl).getQtyShots());
        
    }

	public Player getOwner() {
        return userPlayer;
    }


    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}
