/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.IntersectionLocations;
import mot.common.bean.Location;
import mot.common.bean.ObjectMovement;
import mot.common.bean.ObjectShift;
import mot.common.bean.Settings;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.event.BattleDamageRingEvent;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.MotEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleMine;
import mot.engine.battles.BattleMineFactory;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.BattleStation;
import mot.engine.battles.incumbants.AsteroidField;
import mot.engine.battles.incumbants.IncumbentFactory;
import mot.engine.battles.incumbants.PerimeterAlarm;
import mot.engine.battles.incumbants.SpaceDistortion;
import mot.engine.crafts.Craft;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

public class BattleIncumbentTest  extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(BattleIncumbentTest.class);
        return suite;
    }

    public BattleIncumbentTest(String name) {
        super(name);
    	try {
			Settings.getInstance();
		} catch (ApplicationError e) {
			e.printStackTrace();
		}
    }

    private SolarSystem ss1;
    private SolarSystem ss2;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private Planet planet1;
    private Planet planet2;

    private Battle battle;
    
    private Craft craft1;
    private Craft craft2;
    
    private BattleCraft [] bca;
    private BattlePlanet bp;
    
    private Weapon spacialDistortion;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
    }

    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }

    public void createMap() throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(player1, player2);
    }

    public void createMapAndTwoCraft(CraftDesign design1, CraftDesign design2, boolean withAsteroids, boolean withMines, Weapons planetWeapons) throws Exception {
        createMap();
        
        if (planetWeapons != null) planet2.setWeapons(new Weapons(planetWeapons));
        if (withAsteroids) planet2.setHasAsteroids(true);
        if (withMines) player2.addMiscTechnology(tech_mineField);
        
        craft1 = newCraft(design1, player1, ss2);
        craft2 = newCraft(design2, player2, ss2);

        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss2.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.join(planet2);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);
        
        bca = battle.getCrafts();
        bp = battle.getPlanet();
    }

    /**
     * Test the base classes IncumbentRectangle
     * Checks that the calculations for the edges of the rectangles are calculated properly
     */
    public void testIncumbentRectangle() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testIncumbentRectangle() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, true, false, null);

        AsteroidField af = (AsteroidField) battle.getIncumbents().getIncumbants()[0];  
        
        assertEquals(700, af.getRectangle().getLeft());
        assertEquals(150, af.getRectangle().getTop());
        assertEquals(50, af.getRectangle().getWidth());
        assertEquals(700, af.getRectangle().getHeight());
        
        IntersectionLocations r;
        
        
        //
        // Test line passing right through rectangle
        //
        r = af.getIntersections(new Location(650,400), new Location(800,400));
        assertEquals(700,400, r.getFirstEdge());
        assertEquals(750,400, r.getSecondEdge());
        
        //
        // Test line starting in rectangle and moving out
        //
        
        r = af.getIntersections(new Location(730,400), new Location(800,400));
        assertEquals(750,400, r.getFirstEdge());
        assertEquals(null, r.getSecondEdge());
        
        assertEquals(true, af.inside(new Location(750,400)));
        assertEquals(false, af.inside(new Location(900,400)));
    }
    
    /**
     * Test the base classes IncumbentRectangle
     * Checks that the calculations for the edges of the rectangles are calculated properly
     */
    public void testIncumbentCircle() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testIncumbentCircle() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, false, false, null);
        IncumbentFactory.createPerimeterAlarm(null, battle, new Location(500,500), 10);

        PerimeterAlarm alarm = (PerimeterAlarm) battle.getIncumbents().getIncumbants()[0];  
        
        assertEquals(500,500, alarm.getCenter());
        assertEquals(10, alarm.getRadius());
        
        Location r;
        
        //
        // Test line passing right through center
        //
        
        r = alarm.getClosestPointToCenter(new Location(400,400), new Location(600,600));
        assertEquals(500,500, r);
        
        //
        // Test line passing past center
        //
        
        r = alarm.getClosestPointToCenter(new Location(500,490), new Location(510,500));
        assertEquals(505,495, r);
        
        //
        // Test line does not head near incumbent
        //
        
        r = alarm.getClosestPointToCenter(new Location(100,100), new Location(200,100));
        assertEquals(200,100, r);
    }
    
    /**
     * Tests that a crafts movement is broken down into parts.
     * While travelling through an incumbent a separate shift animation is used with a
     * different speed. This means the users get to see the effect of an incumbent on movement.
     */
    public void testCraftDestroyedEnteringAsteroid() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testCraftDestroyedEnteringAsteroid() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, true, false, null);
        
        //
        // Test Craft Movement through the field (slows movement and damages craft)
        //

        bca[0].setLocation(new Location(650, 500));
        bca[0].setDirection(90);
        
        assertEquals(100, bca[0].getSpeed());
        assertEquals(200, bca[0].getTurningSpeed());
        assertEquals(10, bca[0].getShields()[0].getPower(), 0.001);
        
        assertEquals(1, battle.getIncumbents().getIncumbants().length);

        AsteroidField af = (AsteroidField) battle.getIncumbents().getIncumbants()[0];  
        
        assertEquals(700, af.getRectangle().getLeft());
        assertEquals(150, af.getRectangle().getTop());
        assertEquals(50, af.getRectangle().getWidth());
        assertEquals(700, af.getRectangle().getHeight());
        
        //
        // Test craft 0 gets damaged as it enters the asteroid field and slows down as it moves through the asteroid field.
        //
        
        bca[0]._moveTo(new Location(900,500));
        
        BattleCraftMoveEvent event = (BattleCraftMoveEvent) lastEvent;        
        ObjectMovement[] movements = event.getMovements();
        
        assertEquals(2, movements.length);
        assertEquals(650,500, ((ObjectShift)movements[0]).getOrigin());
        assertEquals(700,500, ((ObjectShift)movements[0]).getDestination());
        assertEquals(700,500, ((ObjectShift)movements[1]).getOrigin());
        assertEquals(725,500, ((ObjectShift)movements[1]).getDestination());
        assertEquals(1, ((ObjectShift)movements[0]).getSpeedModification(), 0.001);
        assertEquals(4, ((ObjectShift)movements[1]).getDamage().getDamage(), 0.001);
        assertEquals(1, ((ObjectShift)movements[1]).getDamage().getShieldBlockedDamage(), 0.001);
        assertEquals(0.5, ((ObjectShift)movements[1]).getSpeedModification(), 0.001);

        assertEquals(4, craft1.getDamage(), 0.001);
        assertEquals(5, bca[0].getShields()[0].getPower(), 0.001);

        //
        // Test craft 0 leaves the asteroid field with no extra damage but is slowed down as it moves through the asteroid field.
        //
        
        bca[0].setMoveRemaining(1.0);
        bca[0]._moveTo(new Location(850,500));

        event = (BattleCraftMoveEvent) lastEvent;        
        movements = event.getMovements();
        
        assertEquals(2, movements.length);
        assertEquals(725,500, ((ObjectShift)movements[0]).getOrigin());
        assertEquals(750,500, ((ObjectShift)movements[0]).getDestination());
        assertEquals(750,500, ((ObjectShift)movements[1]).getOrigin());
        assertEquals(794,500, ((ObjectShift)movements[1]).getDestination());
        assertEquals(4, ((ObjectShift)movements[0]).getDamage().getDamage(), 0.001);
        assertEquals(1, ((ObjectShift)movements[0]).getDamage().getShieldBlockedDamage(), 0.001);
        
        assertEquals(8, craft1.getDamage(), 0.001);
        assertEquals(0, bca[0].getShields()[0].getPower(), 0.001);

        //
        // Test Weapons firing through the field (does not effect weapons)
        //

        bca[0].setMoveRemaining(1.0);
        bca[0].setLocation(new Location(690, 500));
        bca[0].setDirection(90);
        bca[0].getCraft().setDamage(0);

        bca[1].setLocation(new Location(760, 500));
        bca[1].setDirection(270);
        
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(1, craft2.getDamage(), 0.001);
        assertEquals(8, bca[1].getShields()[0].getPower(), 0.001);

        
        //
        // Test craft 0 can move right through asteroid field and out the other side
        //
        
        af.getRectangle().setLeft(700);
        af.getRectangle().setWidth(10);
        
        bca[0].setMoveRemaining(1.0);
        bca[0].setLocation(new Location(690, 400));
        bca[0].setDirection(90);
        bca[0].getCraft().setDamage(0);
        bca[0].getShields()[0].rechargeFull();

        bca[0]._moveTo(new Location(800,400));

        event = (BattleCraftMoveEvent) lastEvent;        
        movements = event.getMovements();
        
        assertEquals(3, movements.length);
        assertEquals(690,400, ((ObjectShift)movements[0]).getOrigin());
        assertEquals(700,400, ((ObjectShift)movements[0]).getDestination());
        assertEquals(1.0, 	  ((ObjectShift)movements[0]).getSpeedModification(), 0.001);
        assertEquals(700,400, ((ObjectShift)movements[1]).getOrigin());
        assertEquals(710,400, ((ObjectShift)movements[1]).getDestination());
        assertEquals(4.0, 	  ((ObjectShift)movements[1]).getDamage().getDamage(), 0.001);
        assertEquals(1.0, 	  ((ObjectShift)movements[1]).getDamage().getShieldBlockedDamage(), 0.001);
        assertEquals(0.5, 	  ((ObjectShift)movements[1]).getSpeedModification(), 0.001);
        assertEquals(710,400, ((ObjectShift)movements[2]).getOrigin());
        assertEquals(780,400, ((ObjectShift)movements[2]).getDestination());
        assertEquals(1.0, 	  ((ObjectShift)movements[2]).getSpeedModification(), 0.001);
        assertEquals(null, 	  ((ObjectShift)movements[2]).getDamage());

        assertEquals(4, craft1.getDamage(), 0.001);
        assertEquals(5, bca[0].getShields()[0].getPower(), 0.001);

        
        //
        // Test craft 0 can be destroyed when it hits the asteroid field.
        //
        
        bca[0].setMoveRemaining(1.0);
        bca[0].setLocation(new Location(690, 500));
        bca[0].setDirection(90);
        bca[0].getCraft().setDamage(bca[0].getCraft().getMaxDamage()-1);
        lastMoveEvent = null;
        
        bca[0]._moveTo(new Location(800,500));

        movements = lastMoveEvent.getMovements();
        
        assertEquals(1, movements.length);
        assertEquals(new Location(690,500), ((ObjectShift)movements[0]).getOrigin());
        assertEquals(new Location(700,500), ((ObjectShift)movements[0]).getDestination());
        assertEquals(1, 	((ObjectShift)movements[0]).getSpeedModification(), 0.001);
        assertEquals(null,	((ObjectShift)movements[0]).getDamage());

        assertEquals(true, craft1.isDestroyed());

    }
    
    /**
     * Tests that a crafts movement is broken down into parts.
     * While travelling through an incumbent a separate shift animation is used with a
     * different speed. This means the users get to see the effect of an incumbent on movement.
     */
    public void testCraftDestroyedLeavingAsteroid() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testCraftDestroyedLeavingAsteroid() *************************");
        CraftDesign fd = new CraftDesign("TestFighter", tech_flyer30, tech_fusion400, tech_massDriver1);
        createMapAndTwoCraft(fd, fd, true, false, null);
        
        //
        // Test Craft Movement through the field (slows movement and damages craft)
        //

        AsteroidField af = (AsteroidField) battle.getIncumbents().getIncumbants()[0];  
        
        assertEquals(700, af.getRectangle().getLeft());
        assertEquals(150, af.getRectangle().getTop());
        assertEquals(50, af.getRectangle().getWidth());
        assertEquals(700, af.getRectangle().getHeight());
        
        //
        // Test craft gets damaged moving into asteroid field
        //
        
        lastMoveEvent = null;
        bca[0].setLocation(new Location(760, 300));
        bca[0].setDirection(225);
        bca[0]._moveTo(new Location(740,320));
        
        BattleCraftMoveEvent event = (BattleCraftMoveEvent) lastMoveEvent;        
        ObjectMovement[] movements = event.getMovements();
        
        assertEquals(2, movements.length);
        assertEquals(760,300, ((ObjectShift)movements[0]).getOrigin());
        assertEquals(750,310, ((ObjectShift)movements[0]).getDestination());
        assertEquals(750,310, ((ObjectShift)movements[1]).getOrigin());
        assertEquals(740,320, ((ObjectShift)movements[1]).getDestination());
        assertEquals(5.0, ((ObjectShift)movements[1]).getDamage().getDamage(), 0.001);
        assertEquals(0.5, ((ObjectShift)movements[1]).getSpeedModification(), 0.001);

        assertEquals(5.0, craft1.getDamage(), 0.001);

        //
        // Test craft gets damaged when moves inside asteroid field
        //
        
        lastMoveEvent = null;
        bca[0]._moveTo(new Location(720,340));
        
        event = (BattleCraftMoveEvent) lastMoveEvent;        
        movements = event.getMovements();
        
        assertEquals(1, movements.length);
        assertEquals(740,320, ((ObjectShift)movements[0]).getOrigin());
        assertEquals(720,340, ((ObjectShift)movements[0]).getDestination());
        assertEquals(5.0, 	  ((ObjectShift)movements[0]).getDamage().getDamage(), 0.001);

        assertEquals(10.0, craft1.getDamage(), 0.001);

        //
        // Test craft gets damaged when leaves asteroid field
        //
        
        lastMoveEvent = null;
        bca[0]._moveTo(new Location(690,370));
        
        event = (BattleCraftMoveEvent) lastMoveEvent;        
        movements = event.getMovements();
        
        assertEquals(2, movements.length);
        assertEquals(720,340, ((ObjectShift)movements[0]).getOrigin());
        assertEquals(700,360, ((ObjectShift)movements[0]).getDestination());
        assertEquals(5.0, 	  ((ObjectShift)movements[0]).getDamage().getDamage(), 0.001);
        assertEquals(0.5, 	  ((ObjectShift)movements[0]).getSpeedModification(), 0.001);
        assertEquals(700,360, ((ObjectShift)movements[1]).getOrigin());
        assertEquals(690,370, ((ObjectShift)movements[1]).getDestination());

        assertEquals(15.0, craft1.getDamage(), 0.001);
    }
    
    /**
     * Tests that a crafts movement is broken down into parts.
     * While travelling through an incumbent a separate shift animation is used with a
     * different speed. This means the users get to see the effect of an incumbent on movement.
     */
    public void testFinalMoveLocationsAfterIncumbentSlowsMovement() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testFinalMoveLocationsAfterIncumbentSlowsMovement() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, true, false, null);
        
        //
        // Test Craft Movement through the field (slows movement and damages craft)
        //

        bca[0].setLocation(new Location(730, 500));
        bca[0].setDirection(270);
        
        assertEquals(100, bca[0].getSpeed());
        assertEquals(200, bca[0].getTurningSpeed());
        assertEquals(10, bca[0].getShields()[0].getPower(), 0.001);
        
        assertEquals(1, battle.getIncumbents().getIncumbants().length);

        AsteroidField af = (AsteroidField) battle.getIncumbents().getIncumbants()[0];  
        af.getRectangle().setWidth(25);
        
        assertEquals(700, af.getRectangle().getLeft());
        assertEquals(150, af.getRectangle().getTop());
        assertEquals(25, af.getRectangle().getWidth());
        assertEquals(700, af.getRectangle().getHeight());
        
        //
        // Test that a craft cannot cannot move on top of another craft
        //
        
        bca[1].setLocation(new Location(655, 500));
        bca[0]._moveTo(new Location(655,500));
        
        BattleCraftMoveEvent event = (BattleCraftMoveEvent) lastEvent;        
        ObjectMovement[] movements = event.getMovements();
        
        assertEquals(3, movements.length);
        assertEquals(new Location(730,500), ((ObjectShift)movements[0]).getOrigin());
        assertEquals(new Location(725,500), ((ObjectShift)movements[0]).getDestination());
        assertEquals(new Location(725,500), ((ObjectShift)movements[1]).getOrigin());
        assertEquals(new Location(700,500), ((ObjectShift)movements[1]).getDestination());
        assertEquals(new Location(700,500), ((ObjectShift)movements[2]).getOrigin());
        assertEquals(new Location(680,500), ((ObjectShift)movements[2]).getDestination());

        af.getRectangle().setWidth(50);
    }
    
    /**
     * Tests that a crafts movement is broken down into parts.
     * While travelling through an incumbent a separate shift animation is used with a
     * different speed. This means the users get to see the effect of an incumbent on movement.
     */
    public void testSpaceDistortion() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testSpaceDistortion() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, false, false, new Weapons(tech_spacialDistortion));
        
        //
        // Fire the weapon Spacial Distruption on the planet at craft1
        // Check that incumbant spacial distortion is created
        //

        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(270);

        assertEquals(388, bca[0].getArea().getLeft());
        assertEquals(412, bca[0].getArea().getRight());

        bp.fireWeapon(new Location(312,500), bp.getWeapons()[0], 1);
        
        assertEquals(1, battle.getIncumbents().getIncumbants().length);

        SpaceDistortion sd = (SpaceDistortion) battle.getIncumbents().getIncumbants()[0];
        
        assertEquals(287, sd.getRectangle().getLeft());
        assertEquals(450, sd.getRectangle().getTop());
        assertEquals(50, sd.getRectangle().getWidth());
        assertEquals(100, sd.getRectangle().getHeight());
        
        
        //
        // Test movement of a craft through the spacial distortion
        //
        
        assertEquals(100, bca[0].getSpeed());
        assertEquals(200, bca[0].getTurningSpeed());
        assertEquals(10, bca[0].getShields()[0].getPower(), 0.001);
        
        int start = sd.getRectangle().getLeft()+sd.getRectangle().getWidth()+1;
        bca[0].setLocation(new Location(start, 500));
        bca[0]._moveTo(new Location(100,500));
        
        BattleCraftMoveEvent event = (BattleCraftMoveEvent) lastEvent;        
        ObjectMovement[] movements = event.getMovements();
        
        assertEquals(2, movements.length);
        assertEquals(new Location(start,500), ((ObjectShift)movements[0]).getOrigin());
        assertEquals(new Location(start-1,500), ((ObjectShift)movements[0]).getDestination());
        assertEquals(new Location(start-1,500), ((ObjectShift)movements[1]).getOrigin());
        assertEquals(new Location(start-1-49,500), ((ObjectShift)movements[1]).getDestination());
        assertEquals(1, ((ObjectShift)movements[0]).getSpeedModification(), 0.001);
        assertEquals(0.5, ((ObjectShift)movements[1]).getSpeedModification(), 0.001);
        assertNull(((ObjectShift)movements[1]).getDamage());

        
        //
        // Check image width
        //
        
        assertEquals(18, bca[0].getWidth());
        assertEquals(18, bca[1].getWidth());
        
        //
        // Test Weapons firing through the field - BLOCKED
        //

        bca[0].setMoveRemaining(1.0);
        bca[0].setLocation(new Location(350, 500));
        bca[0].setDirection(270);

        bca[1].setLocation(new Location(250, 500));
        bca[1].setDirection(90);
        bca[1].getCraft().setDamage(0);
        bca[1].getShields()[0].rechargeFull();
        
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(0, craft2.getDamage(), 0.001);
        assertEquals(10, bca[1].getShields()[0].getPower(), 0.001);
        
        BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) lastEvent;
        
        assertEquals(new Location(337,500), wfe.getTargetLocation(null));
        
        //
        // Test Weapons firing from within the field to outside the field - BLOCKED
        //

        bca[0].setMoveRemaining(1.0);
        bca[0].rechargeWeapons();
        bca[0].setLocation(new Location(300, 500));
        bca[0].setDirection(270);

        bca[1].setLocation(new Location(250, 500));
        bca[1].setDirection(90);
        bca[1].getCraft().setDamage(0);
        bca[1].getShields()[0].rechargeFull();
        
        bca[0].fireAllWeapons(bca[1]);

        assertTrue(sd.inside(bca[0].getLocation()));
        assertEquals(0, craft2.getDamage(), 0.001);
        assertEquals(10, bca[1].getShields()[0].getPower(), 0.001);
        
        wfe = (BattleWeaponFireEvent) lastEvent;
        assertEquals(new Location(287,500), wfe.getTargetLocation(null));
        
        
        //
        // Test Weapons firing from within the field to within the field - BLOCKED
        //

        bca[0].setMoveRemaining(1.0);
        bca[0].rechargeWeapons();
        bca[0].setLocation(new Location(320, 500));
        bca[0].setDirection(270);

        bca[1].setLocation(new Location(300, 500));
        bca[1].setDirection(90);
        bca[1].getCraft().setDamage(0);
        bca[1].getShields()[0].rechargeFull();
        
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(0, craft2.getDamage(), 0.001);
        assertEquals(10, bca[1].getShields()[0].getPower(), 0.001);
        
        wfe = (BattleWeaponFireEvent) lastEvent;
        assertEquals(new Location(320,500), wfe.getTargetLocation(null));
        
        //
        // Test Weapons firing from within the field close enough to the edge for the craft's
        // image to stick outside the field - NOT BLOCKED
        //

        bca[0].setMoveRemaining(1.0);
        bca[0].rechargeWeapons();
        bca[0].setLocation(new Location(280, 500));
        bca[0].setDirection(270);

        bca[1].setLocation(new Location(250, 500));
        bca[1].setDirection(90);
        bca[1].getCraft().setDamage(0);
        bca[1].getShields()[0].rechargeFull();
        
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(1.0, craft2.getDamage(), 0.001);
        assertEquals(8, bca[1].getShields()[0].getPower(), 0.001);

        
        //
        // Test Weapons firing from within the field close enough to the edge for the craft's
        // image to stick outside the field - NOT BLOCKED
        //

        bca[0].setMoveRemaining(1.0);
        bca[0].rechargeWeapons();
        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(270);

        bca[1].setLocation(new Location(385, 500));
        bca[1].setDirection(90);
        bca[1].getCraft().setDamage(0);
        bca[1].getShields()[0].rechargeFull();
        
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(1.0, craft2.getDamage(), 0.001);
        assertEquals(8, bca[1].getShields()[0].getPower(), 0.001);

        
        //
        // Test Weapons firing from outside the field to outside the field - NOT BLOCKED
        //

        bca[0].setMoveRemaining(1.0);
        bca[0].rechargeWeapons();
        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(270);

        bca[1].setLocation(new Location(390, 500));
        bca[1].setDirection(90);
        bca[1].getCraft().setDamage(0);
        bca[1].getShields()[0].rechargeFull();
        
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(1, craft2.getDamage(), 0.001);
        assertEquals(8, bca[1].getShields()[0].getPower(), 0.001);
       
    }

    /**
     * Tests that a crafts movement is broken down into parts.
     * While travelling through an incumbent a separate shift animation is used with a
     * different speed. This means the users get to see the effect of an incumbent on movement.
     */
    public void testSpaceDistortionCantFireCloseToCraft() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testSpaceDistortionCantFireCloseToCraft() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, false, false, new Weapons(tech_spacialDistortion));
        
        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(270);

        assertEquals(388, bca[0].getArea().getLeft());
        assertEquals(412, bca[0].getArea().getRight());

        // Fire weapon too close to craft bca[0]
        bp.fireWeapon(new Location(354,500), bp.getWeapons()[0], 1);
        
        assertEquals(10, bp.getWeapons()[0].getWeapon().getQtyShots().intValue());
        assertEquals(null, battle.getIncumbents().getIncumbants());

        bp.rechargeWeapons();
        // Fire weapon as close to craft bca[0]is is allowed
        bp.fireWeapon(new Location(352,500), bp.getWeapons()[0], 1);
        
        assertEquals(1, battle.getIncumbents().getIncumbants().length);
        
        // 
        // And cant fire close to planet either
        //

        bp.rechargeWeapons();
        bp.fireWeapon(new Location(249,500), bp.getWeapons()[0], 1);
        
        assertEquals(100, bp.getLocation().getX());
        assertEquals(100, bp.getImageRadius());
        
        assertEquals(1, battle.getIncumbents().getIncumbants().length);
        
        bp.rechargeWeapons();
        bp.fireWeapon(new Location(600,500), bp.getWeapons()[0], 1);
        
        assertEquals(100, bp.getLocation().getX());
        assertEquals(100, bp.getImageRadius());
        
        assertEquals(2, battle.getIncumbents().getIncumbants().length);
        
        // 
        // Craft firing at planet can destroy space distortion
        //
    }
    
    /**
     * Tests that a crafts movement is broken down into parts.
     * While travelling through an incumbent a separate shift animation is used with a
     * different speed. This means the users get to see the effect of an incumbent on movement.
     */
    public void testSpaceDistortionDestroyedOnPlanet() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testSpaceDistortionDestroyedOnPlanet() *************************");
        CraftDesign cd = new CraftDesign("Attacker", tech_flyer30, tech_fusion40, tech_ancientWeapon, tech_ancientWeapon, tech_ancientWeapon, tech_ancientWeapon, tech_ancientWeapon, tech_ancientWeapon);
        createMapAndTwoCraft(cd, fighterDesign, false, false, new Weapons(tech_missile6, tech_massDriver1, tech_spacialDistortion));
        
        bca[0].setLocation(new Location(175, 500));
        bca[0].setDirection(270);

        bca[1].destroy(null, null, null);
        
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bp);
        
        assertEquals(1, bp.getWeapons().length);
        
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bp);
        
        assertNull(bp.getWeapons());
        assertEquals(0, planet2.getWeapons().getQuantity());
        assertEquals(true, battle.hasEnded());
        
    }
    
    /**
     * Tests that a crafts movement is broken down into parts.
     * While travelling through an incumbent a separate shift animation is used with a
     * different speed. This means the users get to see the effect of an incumbent on movement.
     */
    public void testMines() throws Exception {
        Logger.logDebug("*******************************  BattleIncumbentTest.testMines() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, false, true, null);
        
        //
        // Check that a mines have been placed in the correct position 
        //
        
        Automaton [] mines = battle.getAutomatons();
        assertEquals(3, mines.length);
        assertEquals(25, ((BattleMine)mines[0]).getAlarm().getRadius());
        assertEquals(25, ((BattleMine)mines[1]).getAlarm().getRadius());
        assertEquals(25, ((BattleMine)mines[2]).getAlarm().getRadius());
        
        // We place mine two close to mine one to test one mine does not damage another
        BattleMine mine1 = (BattleMine)mines[0];
        BattleMine mine2 = (BattleMine)mines[1];
        BattleMine mine3 = (BattleMine)mines[2];
        mine1.setLocation(new Location(500, 500));
        mine2.setLocation(new Location(500, 490));
        mine3.setLocation(new Location(500, 700));
        
        //
        // Check that a mines does not explode when any craft is not close enough
        //

        bca[0].setLocation(new Location(550, 530));
        bca[0].setDirection(270);
        
        bca[0]._moveTo(new Location(450,530));
        
        assertEquals(new Location(450,530), bca[0].getLocation());
        assertEquals(0, bca[0].getDamage(), 0.001);
        
        assertFalse(mines[0].isDestroyed());
        assertFalse(mines[1].isDestroyed());
        assertFalse(mines[2].isDestroyed());
        
        assertNull(warpCoreOverloadEvent);
        
        //
        // Check that a mines explodes when any craft is close enough and damage 
        // all craft in the area 
        //

        bca[0].setLocation(new Location(550, 500));
        bca[0].setDirection(270);
        bca[0].setMoveRemaining(1.0);
        bca[1].setLocation(new Location(440, 500));
        bca[1].setDirection(90);

        bca[0]._moveTo(new Location(450,500));
        
        assertEquals(new Location(500,500), bca[0].getLocation());
        assertEquals(74.0, bca[0].getDamage(), 0.001);        
        assertEquals(true, bca[0].isDestroyed());        
        assertEquals(14.0, bca[1].getDamage(), 0.001);        
        assertEquals(false, bca[1].isDestroyed());        
        
        assertTrue(mines[0].isDestroyed());
        assertFalse(mines[1].isDestroyed());
        assertFalse(mines[2].isDestroyed());
        
        assertEquals(2, battle.getAutomatons().length);
        
        assertEquals(new Location(500, 500), warpCoreOverloadEvent.getLocation());
        
    }
    
    public void testMinesCreatedOnPlanet() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlanet() *************************");
        createMap();
        
        planet2.setWeapons(new Weapons(tech_missile6));
        planet2.setOrbital(new Orbital(tech_battleStationI, null, null));
        player2.addMiscTechnology(tech_mineField);
        
        craft1 = newCraft(fighterDesign, player1, ss2);
        craft2 = newCraft(fighterDesign, player2, ss2);

        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss2.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.join(planet2);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);
        
        bca = battle.getCrafts();
        bp = battle.getPlanet();
        BattleStation battleStation = battle.getBattleStation();

        assertEquals(3, battle.getQtyMines());
		assertEquals(tech_battleStationI, battleStation.getCraft().getHull());
    }

    public void testMinesCreatedOnWormHole() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlanet() *************************");
        createMap();
        
        BlackHole blackHole = newBlackHole("WormHole", 50, 50, 0, false);
        createSpaceStation(tech_battleStationI, blackHole, player1);
        player1.addMiscTechnology(tech_mineField);
        
        craft1 = newCraft(fighterDesign, player2, blackHole);
        craft1.attack(blackHole, null);
        

        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(blackHole.getOrbit(), false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);
        
        BattleStation battleStation = battle.getBattleStation();

        assertEquals(3, battle.getQtyMines());
		assertEquals(tech_battleStationI, battleStation.getCraft().getHull());
    }

    
    public void testAvoidsOwnMines() throws Exception {
        Logger.logDebug("*******************************  WarTest.testAvoidsOwnMines() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, false, false, null);
        
        BattleCraft bc1 = battle.getCraft(craft1.getName());
        BattleCraft bc2 = battle.getCraft(craft2.getName());
        
        bc1.setLocation(new Location(475,500));
        bc1.setDirection(270);
        bc2.setLocation(new Location(425,500));
        bc2.setDirection(90);
        BattleMineFactory.create(player1, battle, 10, new Location(450,500));

        clearEvents();
        player1.getCommander().assignTaskToCraft(bc1);
        
        // Craft did not move because it can see the mine
        // Craft did fire weapons
        
        assertEquals(475,500, bc1.getLocation());
        assertEquals(bc1.getName(), ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getSource());
        assertEquals(bc2.getName(), ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getTarget());

        clearEvents();
        player2.getCommander().assignTaskToCraft(bc2);
        
        // Craft did move because it can't see the mine
        // Mine did detonate
        // Craft did fire weapons
        
        assertEquals(450,500, bc2.getLocation());
        assertEquals("Mine1", ((BattleDamageRingEvent)getEvent(BattleDamageRingEvent.class)).getSource());
        
    }
    

    private BattleDamageRingEvent warpCoreOverloadEvent;
    
	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	private BattleCraftMoveEvent lastMoveEvent = null;
	private MotEvent lastEvent = null;
	
	public void notify(MotEvent e) throws ApplicationError {
		lastEvent = e;
		eventList.add(e);
		if (e instanceof BattleDamageRingEvent) {
			warpCoreOverloadEvent = (BattleDamageRingEvent) e;
		}
		if (e instanceof BattleCraftMoveEvent) {
			lastMoveEvent = (BattleCraftMoveEvent) e;
		}
	}

	public Player getOwner() {
        return player1;
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
}
