/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;
import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.designs.Orbital;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleStation;
import mot.engine.battles.BattleStationFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;

/**
 * Changes - Inter Block Gap is now 3
 * Mass Driver has damage 1
 */

public class BattleStationFactoryTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(BattleStationFactoryTest.class);
        return suite;
    }

    public BattleStationFactoryTest(String name) {
        super(name);
    }

    private ComputerPlayer player1;
    private SolarSystem ss1;
    private Planet planet1;

    protected void setUp() {
        try {
            MotEngine.newInstance();
            setupConfigs();
            String [] playerNames = { "TestPlayer1" };
            SetupOptions options = mot.common.setup.OptionsFactory.getGeneralOptions();
            player1 = newPlayer(playerNames[0], blue, options, playerNames);
            ss1 = newSolarSystem("SS1", 100, 100);
            planet1 = newPlanet("SS1-A", ss1, player1);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }

    public void testCreate() throws Exception {
        Logger.logDebug("*******************************  BattleStationFactoryTest.testCreate() *************************");

        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
        
        Battle battle = new Battle(ss1.getOrbit());
        //new ProjectBattleStation(new BattleStationDesign(tech_battleStationI)).build(planet1);
        //Orbital orbital = planet1.getOrbital();
        Orbital orbital = createBattleStation(tech_battleStationI, planet1);
        BattleStation station = BattleStationFactory.createBattleStation(battle, planet1);
        
        assertEquals("BattleStation has shield", null, station.getShields());
        assertEquals("BattleStation has weapons", 1, station.getWeapons().length);
        assertEquals("BattleStation has weapons", tech_massDriver1, station.getWeapons()[0].getWeapon());
        assertEquals("BattleStation has weapons", 10, station.getWeapons()[0].getQty());
        assertEquals("BattleStation has weapons", true, station.getWeapons()[0].isHeavy());

        // Add shields to the players techs and test that station now has shields.
        
        player1.addTechnology(tech_energy10);
        station = BattleStationFactory.createBattleStation(battle, planet1);
        
        assertEquals("BattleStation has shield", 1, station.getShields().length);
        assertEquals("BattleStation has shield", tech_energy10, station.getShields()[0].getShield());
        assertEquals("BattleStation has weapons", 1, station.getWeapons().length);
        assertEquals("BattleStation has weapons", tech_massDriver1, station.getWeapons()[0].getWeapon());
        assertEquals("BattleStation has weapons", 7, station.getWeapons()[0].getQty());
        assertEquals("BattleStation has weapons", true, station.getWeapons()[0].isHeavy());

        // Improve the players technology to check that better techs are used when available
        
        player1.addTechnology(tech_energy20);
        player1.addTechnology(tech_fusionBeam4);

        station = BattleStationFactory.createBattleStation(battle, planet1);
        
        assertEquals("BattleStation has shield", 1, station.getShields().length);
        assertEquals("BattleStation has shield", tech_energy20, station.getShields()[0].getShield());
        assertEquals("BattleStation has weapons", 1, station.getWeapons().length);
        assertEquals("BattleStation has weapons", tech_fusionBeam4, station.getWeapons()[0].getWeapon());
        assertEquals("BattleStation has weapons", 5, station.getWeapons()[0].getQty());
        assertEquals("BattleStation has weapons", true, station.getWeapons()[0].isHeavy());

        player1.addTechnology(tech_psiI);

        station = BattleStationFactory.createBattleStation(battle, planet1);
        
        assertEquals("BattleStation has shield", 2, station.getShields().length);
        assertEquals("BattleStation has shield", tech_energy20, station.getShields()[0].getShield());
        assertEquals("BattleStation has shield", tech_psiI, station.getShields()[1].getShield());
        assertEquals("BattleStation has weapons", 1, station.getWeapons().length);
        assertEquals("BattleStation has weapons", tech_fusionBeam4, station.getWeapons()[0].getWeapon());
        assertEquals("BattleStation has weapons", 5, station.getWeapons()[0].getQty());
        assertEquals("BattleStation has weapons", true, station.getWeapons()[0].isHeavy());
    }
    
    public void testCreateWithMissiles() throws Exception {
        Logger.logDebug("*******************************  BattleStationFactoryTest.testCreateWithMissiles() *************************");

        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
        
        Battle battle = new Battle(ss1.getOrbit());
        //new ProjectBattleStation(new BattleStationDesign(tech_battleStationI)).build(planet1);
        //Orbital orbital = new Orbital(tech_battleStationI, null, null);
        //planet1.setOrbital(orbital);
        Orbital orbital = createBattleStation(tech_battleStationI, planet1);
        
        planet1.getOwner().addTechnology(tech_stationMissileBase);
        
        BattleStation station = BattleStationFactory.createBattleStation(battle, planet1);
        
        assertEquals("BattleStation has shield", null, station.getShields());
        assertEquals("BattleStation has weapons", 2, station.getWeapons().length);
        assertEquals("BattleStation has weapons", tech_massDriver1, station.getWeapons()[0].getWeapon());
        assertEquals("BattleStation has weapons", 10, station.getWeapons()[0].getQty());
        assertEquals("BattleStation has weapons", true, station.getWeapons()[0].isHeavy());
        assertEquals("BattleStation has weapons", tech_missile6, station.getWeapons()[1].getWeapon());
        assertEquals("BattleStation has weapons", 2, station.getWeapons()[1].getQty());

    }
    

    protected void tearDown() {

    }


    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}