/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Location;
import mot.common.designs.CraftDesign;
import mot.common.event.BattleCraftLeaveEvent;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.event.BattleCraftSurrenderEvent;
import mot.common.event.BattleDamageEvent;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.BattleFearEvent;
import mot.common.event.BattleMissileLaunchEvent;
import mot.common.event.BattleNewTurnEvent;
import mot.common.event.BattleObjectTriggerEvent;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.BattleWeaponMissEvent;
import mot.common.event.MotEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattlePlanet;
import mot.engine.crafts.Craft;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.technology.ShieldInstance;

/**
 * This tests the battle using a statistical approach.
 * It re-tests some the the code tested in Battle Test, but rather than setting the next
 * random numbers to determine future random events this test runs an event many times
 * and expects the results to fall within 99% center of the bell curve.
 */
public class BattleStatisticalTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(BattleStatisticalTest.class);
        return suite;
    }

    public BattleStatisticalTest(String name) {
        super(name);
    }

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;

    private ComputerPlayer player1;
    private ComputerPlayer player2;
    private ComputerPlayer player3;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    
    private Battle battle;
    
    private Craft craft1;
    private Craft craft2;
    private BattleCraft [] bca;
    private BattlePlanet bp;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
    }

    private int battleTurn = -1;

    
    public void createMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(player1, player2);
    }

    private Battle newBattle(SolarSystem ss) {
        return MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss.getOrbit(), true);
    }

    public void createMapAndTwoCraft(CraftDesign design1, CraftDesign design2, SetupOptions optionsRace) throws Exception {
        createMap(optionsRace);

        craft1 = newCraft(design1, player1, ss2);
        craft2 = newCraft(design2, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        
        bca = battle.getCrafts();
        int dist = bca[0].getLocation().getDistanceTo(bca[1].getLocation());
        assertEquals("Distance should be 800", 800, dist);
        assertEquals("Craft1 located on left", bca[0].getLocation().getX(), 100);
        assertEquals("Craft1 located in center left", bca[0].getLocation().getY(), 500);
        assertEquals("Craft2 located on right", bca[1].getLocation().getX(), 900);
        assertEquals("Craft2 located in center left", bca[1].getLocation().getY(), 500);

        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());

        assertEquals("craft1 is pointing to craft2", 90, bca[0].getDirection());
        assertEquals("craft2 is pointing to craft1", 270, bca[1].getDirection());
    }

    /**
     * Tests that the weapon inflict the right damage and the shields block damage until their power 
     * is exhausted.
     */
    public void testPsi() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPsi() *************************");
        createMapAndTwoCraft(fighterDesign, defenderDesign, null);
        
        bca[0].setLocation(new Location(200,500));
        bca[1].setLocation(new Location(300,500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);
        
        // Add psionic ability to player1 and see that crafts do start missing turns!
        // Psi advantage is 10 so 10 percent defect.
        {
            qtyFearEvents = 0;
            ShieldInstance[] shields = bca[1].getShields();
            player2.addTechnology(tech_psi10);
            assertEquals("Psi Rating is correct", 0, bca[0].getPsiLevel());
            assertEquals("Psi Rating is correct", 10, bca[1].getPsiLevel());
            int qtyMissed = 0;
            int qtyNotMissed = 0;
            for (int i=0; i<100;i++) {
                fired = false;
                craft2.setDamage(0);
                shields[0].setPower(10);
                bca[0].rechargeWeapons();
                bca[0].setMoveRemaining(1);
                bca[1].setMoveRemaining(1);
                bca[0].fireAllWeapons(bca[1]);
                bca[0].setMoveRemaining(0);
                BattleCraft bc = (BattleCraft) battle.getNextToMove(false);
                if (fired) {
                    qtyNotMissed++; 
                    fired = false;
                } else {
                    qtyMissed++;
                }
            }
            assertTrue("About 10% of defenders moves should have been lost ("+qtyMissed+")", qtyMissed >= 3 && qtyMissed <= 22);
            assertEquals("Received surrender events ", qtyMissed, qtyFearEvents);
            player2.getBiotechnologies().remove(tech_psi10);
        }
        
        // Add psi techs and spi bonuses to player2 and see that crafts start surrendering!
        // Player2 has a psi advantage of 40.
        {
            qtySurrenderEvents = 0;
            ShieldInstance[] shields = bca[1].getShields();
            player1.addTechnology(tech_psi10);
            player1.addTechnology(tech_psi20);
            player1.getPlayerCharacter().setPsiRating(20);
            assertEquals("Psi Rating is correct", 40, bca[0].getPsiLevel());
            assertEquals("Psi Rating is correct", 0, bca[1].getPsiLevel());
            int qtySurrendered = 0;
            for (int i=0; i<100;i++) {
                fired = false;
                craft2.setDamage(0);
                shields[0].setPower(10);
                bca[0].rechargeWeapons();
                bca[0].setMoveRemaining(1);
                bca[1].setMoveRemaining(1);
                bca[0].fireAllWeapons(bca[1]);
                if (bca[1].getOwner() != player2) {
                    assertEquals("changed ownership", 2, player1.getCrafts().length);
                    assertEquals("changed ownership", 0, player2.getCrafts().length);
                    bca[1].getCraft().changeOwner(player2);
                    qtySurrendered++;
                }
            }
            assertTrue("About 10% of attackers moves should have surrendered ("+qtySurrendered+")", qtySurrendered >= 3 && qtySurrendered <= 22);
            assertEquals("Received surrender events ", qtySurrendered, qtySurrenderEvents);
            player1.getBiotechnologies().remove(tech_psi10);
            player1.getBiotechnologies().remove(tech_psi20);
            player1.getPlayerCharacter().setPsiRating(0);
            bca[1].getCraft().setOriginReference(null);
        }
    }
        
    /**
     * Tests that the weapon inflict the right damage and the shields block damage until their power 
     * is exhausted.
     */
    public void testCloak() throws Exception {
        Logger.logDebug("*******************************  WarTest.testCloak() *************************");
        createMapAndTwoCraft(fighterDesign, defenderDesign, null);

        bca[0].setLocation(new Location(200,500));
        bca[1].setLocation(new Location(300,500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);
        
        
        // Add cloak ability to player2 and check craft2 becomes visible in the correct distance.
        // When craft2 has a cloak it is visible if : scan strength - distance > cloak
        {
            Location originalLoc = bca[1].getLocation();
            Location l = bca[0].getLocation();
            
            player2.getPlayerCharacter().setCloakLevel(100);
            bca[1].setLocation(new Location(l.getX()+50, l.getY()));
            assertEquals("Correct distance apart", 50, bca[0].getLocation().getDistanceTo(bca[1].getLocation()));
            assertEquals("Correct scan strength", 0, battle.getScannerStrengthAt(player1, bca[1].getLocation()));
            assertEquals("Correct scan strength", 100, bca[1].getCloakLevel());
            assertEquals("Has correct visibility", false, bca[1].isVisible(player1));
            
            bca[0].getCraft().setScanners(new Scanners(tech_photon100));
            assertEquals("Correct scan strength", 50, battle.getScannerStrengthAt(player1, bca[1].getLocation()));
            assertEquals("Has correct visibility", false, bca[1].isVisible(player1));
            
            bca[0].getCraft().setScanners(new Scanners(tech_photon200));
            assertEquals("Correct scan strength", 150, battle.getScannerStrengthAt(player1, bca[1].getLocation()));
            assertEquals("Has correct visibility", true, bca[1].isVisible(player1));
            
            player2.getPlayerCharacter().setCloakLevel(0);
            bca[0].getCraft().setScanners(null);
            bca[1].setLocation(originalLoc);
        }
        
        // Add cloak ability to player2 and see crafts and check that crafts within range of a scanner
        // are still visible and those ouside the range of a scanner are not visible. Test that when a
        // craft is not visible weapons miss sometimes.
        {
            assertEquals("craft2 is not cloaked", true, bca[1].isVisible(player1));
            assertEquals("Player has no cloak", 0, player2.getPlayerCharacter().getCloakLevel());
            player2.getPlayerCharacter().setCloakLevel(100);
            assertEquals("craft2 is cloaked", false, bca[1].isVisible(player1));
            ShieldInstance[] shields = bca[1].getShields();
            int qtyMissed = 0;
            int qtyNotMissed = 0;
            qtyMissedEvents = 0;
            for (int i=0; i<100;i++) {
                fired = false;
                craft2.setDamage(0);
                shields[0].setPower(10);
                bca[0].rechargeWeapons();
                bca[0].setMoveRemaining(1);
                bca[1].setMoveRemaining(1);
                bca[0].fireAllWeapons(bca[1]);
                bca[0].setMoveRemaining(0);
                BattleCraft bc = (BattleCraft) battle.getNextToMove(false);
                if (fired) {
                    qtyNotMissed++; 
                    fired = false;
                } else {
                    qtyMissed++;
                }
            }
            assertTrue("About 60% of weapons did not fire ("+qtyMissed+")", qtyMissed >= 30 && qtyMissed <= 80);
            assertEquals("Received all missed events", qtyMissedEvents, qtyMissed);
            player2.getPlayerCharacter().setCloakLevel(0);
        }
        
    }

    
    /**
     * Tests crafts attacking a planet and planet attacking crafts.
     */
    public void testPlanetAttack() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlanetAttack() *************************");
        createMap(null);

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player1, ss2);
        Craft craft3 = newCraft(fighterDesign, player1, ss2);

        planet2.setShields(new Shields(tech_energy10));
        planet2.setWeapons(new Weapons(tech_massDriver1));

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, true);
        battle.join(planet2);
        battle.setup();

        bp = battle.getPlanet();
        assertEquals("Planet located on left", bp.getLocation().getX(), 100);
        assertEquals("Planet located in center left", bp.getLocation().getY(), 500);
        
        bca = battle.getCrafts();
        bca[0].setLocation(new Location(490, 450));
        bca[1].setLocation(new Location(490, 500));
        bca[2].setLocation(new Location(490, 550));
        
        battleTurn = 1;

        // Add cloak ability to player2 and see crafts and check that crafts within range of a scanner
        // are still visible and those ouside the range of a scanner are not visible. Test that when a
        // craft is not visible weapons miss sometimes.
        {
            assertEquals("craft2 is not cloaked", true, bca[2].isVisible(player2));
            assertEquals("Player has no cloak", 0, player1.getPlayerCharacter().getCloakLevel());
            player1.getPlayerCharacter().setCloakLevel(100);
            assertEquals("craft2 is cloaked", false, bca[2].isVisible(player2));
            ShieldInstance[] shields = bca[2].getShields();
            int qtyMissed = 0;
            int qtyNotMissed = 0;
            qtyMissedEvents = 0;
            for (int i=0; i<100;i++) {
                fired = false;
                craft3.setDamage(0);
                shields[0].setPower(10);
                bp.rechargeWeapons();
                bp.fireAllWeapons(bca[2]);
                if (fired) {
                    qtyNotMissed++; 
                    fired = false;
                } else {
                    qtyMissed++;
                }
            }
            assertEquals("Received all missed events", qtyMissed, qtyMissedEvents);
            assertTrue("About 75% of weapons did not fire ("+qtyMissed+")", qtyMissed >= 45 && qtyMissed <= 90);
            player1.getPlayerCharacter().setCloakLevel(0);
            bp.rechargeWeapons();
        }
    }


    
    protected void tearDown() {

    }

    private boolean fired = false;
    private int qtyFired = 0;
    private int qtyMissedEvents = 0;
    private int qtySurrenderEvents = 0;
    private int qtyFearEvents = 0;
    private int qtyMissilesLaunched = 0;
    private int qtyCraftDestroyed = 0;
    private BattleCraft lastCraftDestroyed;
    private BattleCraftMoveEvent lastCraftMovementEvent;
    private BattleDamageEvent lastDamageEvent;
    private BattleObjectTriggerEvent lastTriggerEvent;
    
	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
        if ( e instanceof BattleNewTurnEvent) {
            battleTurn++;
        } else if (e instanceof BattleCraftLeaveEvent) {
            BattleCraftLeaveEvent event = (BattleCraftLeaveEvent)e;
            //LocalBattleCraftDeligate craftDeligate = (LocalBattleCraftDeligate)event.getCraft();
            //BattleCraft craft = craftDeligate.getBattleCraft();
        } else if (e instanceof BattleDestroyedEvent) {
            BattleDestroyedEvent event = (BattleDestroyedEvent)e;
            lastCraftDestroyed = MotEngine.instance.getBattles().getActiveBattle().getCraft(event.getTarget());
            qtyCraftDestroyed++;
        } else if (e instanceof BattleWeaponFireEvent) {
            fired = true;
            qtyFired++;
        } else if (e instanceof BattleDamageEvent) {
            lastDamageEvent = (BattleDamageEvent) e;
        } else if (e instanceof BattleWeaponMissEvent) {
            qtyMissedEvents++;
        } else if (e instanceof BattleCraftSurrenderEvent) {
            qtySurrenderEvents++;
        } else if (e instanceof BattleFearEvent) {
            qtyFearEvents++;
        } else if (e instanceof BattleMissileLaunchEvent) {
            qtyMissilesLaunched++;
        } else if (e instanceof BattleCraftMoveEvent) {
            lastCraftMovementEvent = (BattleCraftMoveEvent)e;
        } else if ( e instanceof BattleObjectTriggerEvent) {
        	lastTriggerEvent = (BattleObjectTriggerEvent)e;
        }
    }


	public Player getOwner() {
        return player1;
    }


    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}