/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.WeaponBayDesign;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.WeaponBundle;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.adapter.CraftVOAdapter;
import mot.engine.command.AddCraftDesign;
import mot.engine.command.SetProjectCmd;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.Project;
import mot.engine.planets.ProjectCraft;
import mot.engine.planets.ProjectCraftUpgrade;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.players.mayor.CraftDesignerDamageBot;
import mot.engine.players.mayor.CraftDesignerDroidMine;
import mot.engine.players.mayor.CraftDesignerFactory;
import mot.engine.players.mayor.CraftDesignerInvader;
import mot.engine.players.mayor.CraftDesignerMilitary;
import mot.engine.players.mayor.CraftDesignerPlague;
import mot.engine.players.mayor.CraftDesignerPsi;
import mot.engine.setup.MonsterFactory;
import mot.engine.setup.MonsterFactoryMedium;
import mot.engine.setup.MonsterFactorySmall;

/**
 * Test Building Troop Transports, Their moving on the galaxy map and operating in
 * battle.
 */
public class CraftDesignTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(CraftDesignTest.class);
        return suite;
    }

    public CraftDesignTest(String name) {
        super(name);
    }

    private SolarSystem ss1,ss2;

    private Player player1, player2;

    private Planet planet1, planet2;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        Logger.setInMemory(false);
        Logger.setDebug();
    }

    public void createMapWithComputerPlayer(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);
        planet1.setPopulation(100.0);
        planet1.setQtyFactories(50.0);
    }

    public void createMapWithUserPlayer(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newUserPlayer(playerNames[0], blue, null, options);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);
        planet1.setPopulation(100.0);
        planet1.setQtyFactories(50.0);
    }

    public void createMapWithUserPlayerAndComputerPlayer(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newUserPlayer(playerNames[0], blue, null, options);
        player2 = newUserPlayer(playerNames[1], green, null, options);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);
        planet1.setPopulation(100.0);
        planet1.setQtyFactories(50.0);

        ss2 = newSolarSystem("SS2", 100, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);
        planet2.setPopulation(100.0);
        planet2.setQtyFactories(50.0);
    }

    /**
     * Tests that colony crafts are created correctly utilising all the space, no duplicate 
     * techs from different groups.
     * 
     * Test that craft designs are only updated every CraftDesignerFactory.CALCULATE_INTERVAL turns.
     */
    public void testCraftDesignFactoryForColonyCrafts() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftDesignFactoryForColonyCrafts() *************************");
        createMapWithComputerPlayer(null);
        ComputerPlayer p = (ComputerPlayer) player1;
        
        CraftDesignerFactory cdf = p.getMayor().getCraftDesignerFactory();
        
        CraftDesign cd = cdf.getNextColonyCraftToBuild(null, p);
        assertTrue("Has best colony module", cd.getSpecials().contains(TestTechs.COLONY_MODULE8));
        assertEquals("Has best engine", cd.getEngine(), TestTechs.FUSION25);
        assertTrue("Not overfilled", cd.getRemainingSpace()>=0);

        player1.addTechnology(tech_colonyModule7);
        player1.addTechnology(tech_fusion25x);
        player1.addTechnology(tech_flyer100);
        
        assertEquals("Have correct values", 100, tech_flyer100.getSpace());
        assertEquals("Have correct values", 40, tech_colonyModule7.getSpace());
        
        cd = cdf.getNextColonyCraftToBuild(null, p);
        assertTrue("Design has not been updated", cd.getSpecials().contains(TestTechs.COLONY_MODULE8));
        MotEngine.instance.setDate(MotEngine.instance.getDate()+CraftDesignerFactory.CALCULATE_INTERVAL+1);

        cd = cdf.getNextColonyCraftToBuild(null, p);
        assertTrue("Has best colony module", cd.getSpecials().contains(TestTechs.COLONY_MODULE7));
        assertEquals("Has best engine", cd.getEngine(), TestTechs.FUSION25x);
        assertEquals("Has smallest hull that is big enough", cd.getHull(), TestTechs.FLYER60);
        assertTrue("Not overfilled", cd.getRemainingSpace()>=0);

        // MAKE SURE IT ALWAYS CHOOSES THE ENGINE WITH THE LARGEST RANGE!
        int originalMaxTravelDist = tech_fusion25.getMaxTravelDistance();
        tech_fusion25.setMaxTravelDistance(originalMaxTravelDist+300);
        cdf.flushCraftDesigns();
        cd = cdf.getNextColonyCraftToBuild(null, p);
        assertEquals("Has basic colony module", cd.getEngine(), TestTechs.FUSION25);
        tech_fusion25.setMaxTravelDistance(originalMaxTravelDist);
    }
    
    /**
     * Tests that military crafts are created correctly utilising all the space, no duplicate 
     * techs from different groups.
     */
    public void testCraftDesignFactoryForMilitaryCrafts() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftDesignFactoryForMilitaryCrafts() *************************");
        createMapWithComputerPlayer(null);
        ComputerPlayer p = (ComputerPlayer) player1;
        CraftDesignerMilitary cdm = new CraftDesignerMilitary();

        player1.addTechnology(tech_flyer100);
        
        CraftDesign cd = cdm.designCraft(p, CraftDesignerMilitary.SIZE_SMALL, 0.0, 0.0);
        cd.log("Test");
        assertEquals("Has correct hull", TestTechs.FLYER30, cd.getHull());
        assertEquals("Has correct engine", TestTechs.FUSION25, cd.getEngine());
        assertEquals("Has correct weapon", 5, cd.getWeaponBays().getQtyWeapons());
        assertTrue("Has correct weapon", cd.getWeaponBays().contains(TestTechs.MASS_DRIVER1));
        assertNull("Has correct computer", cd.getComputers());
        assertNull("Has correct scanner", cd.getScanners());
        assertNull("Has correct special", cd.getSpecials());
        assertNull("Has correct shield", cd.getShields());
        assertTrue("Craft Design is not overfilled", cd.getRemainingSpace()>=0);
        
        planet1.setProject(new ProjectCraft(cd, planet1, false));
        assertEquals("Hull should be built in less than 10 turns", 14, planet1.getTurnsToCompleteProject());

        cd = cdm.designCraft(p, CraftDesignerMilitary.SIZE_MEDIUM, 0.0, 0.0);
        cd.log("Test");
        assertEquals("Has correct hull", TestTechs.FLYER60, cd.getHull());
        assertEquals("Has correct engine", TestTechs.FUSION25, cd.getEngine());
        assertEquals("Has correct scanner", TestTechs.PHOTON100, cd.getScannerTechs().getAllAsArray(true)[0].getName());
        assertTrue("Has correct weapon", cd.getWeaponBays().contains(TestTechs.MASS_DRIVER1));
        assertEquals("Has correct weapon", 10, cd.getWeaponBays().getQtyWeapons());
        assertNull("Has correct computer", cd.getComputers());
        assertNull("Has correct special", cd.getSpecials());
        assertNull("Has correct shield", cd.getShields());
        assertTrue("Craft Design is not overfilled", cd.getRemainingSpace()>=0);

        planet1.setProject(new ProjectCraft(cd, planet1, false));
        assertTrue("Hull should be built in less than 25 turns", planet1.getTurnsToCompleteProject() < 35);

        player1.addTechnology(tech_photon100);
        player1.addTechnology(tech_energy10);
        player1.addTechnology(tech_shieldModulatorI);
        player1.addTechnology(tech_electronic25);

        cd = cdm.designCraft(p, CraftDesignerMilitary.SIZE_MEDIUM, 0.0, 0.0);
        cd.log("Test");
        assertEquals("Has correct hull", TestTechs.FLYER60, cd.getHull());
        assertEquals("Has correct engine", TestTechs.FUSION25, cd.getEngine());
        assertEquals("Has correct computer", TestTechs.ELECTRONIC25, cd.getComputerTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct scanner", TestTechs.PHOTON100, cd.getScannerTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct special", TestTechs.SHIELD_MODULATORI, cd.getSpecialTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct shield", TestTechs.ENERGY10, cd.getShieldTechs().getAllAsArray(true)[0].getName());
        assertTrue("Has correct weapon", cd.getWeaponBays().contains(TestTechs.MASS_DRIVER1));
        assertEquals("Has correct weapon", 5, cd.getWeaponBays().getQtyWeapons());
        assertTrue("Craft Design is not overfilled", cd.getRemainingSpace()>=0);

        player1.addTechnology(tech_massDriver2);
        player1.addTechnology(tech_fusion25x);
        player1.addTechnology(tech_electronic40);
        player1.addTechnology(tech_photon150);
        player1.addTechnology(tech_energy10);
        player1.addTechnology(tech_spacePods10);
        player1.addTechnology(tech_armour25);
        player1.addTechnology(tech_eneryAmplifierI);
        player1.addTechnology(tech_augumentedEnginesI);
        player1.addTechnology(tech_tacticalStationI);
        player1.addTechnology(tech_shieldModulatorII);

        cd = cdm.designCraft(p, CraftDesignerMilitary.SIZE_MEDIUM, 0.0, 0.0);
        cd.log("Test");
        assertEquals("Has correct hull", TestTechs.FLYER60, cd.getHull());
        assertEquals("Has incorrect engine", TestTechs.FUSION25x, cd.getEngine());
        assertEquals("Has correct computer", TestTechs.ELECTRONIC40, cd.getComputerTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct scanner", TestTechs.PHOTON150, cd.getScannerTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct special", 2, cd.getSpecialTechs().getAllAsArray(true).length);
        assertEquals("Has correct special", TestTechs.SPACE_PODS10, cd.getSpecialTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct special", TestTechs.SHIELD_MODULATORII, cd.getSpecialTechs().getAllAsArray(true)[1].getName());
        assertEquals("Has correct shield", TestTechs.ENERGY10, cd.getShieldTechs().getAllAsArray(true)[0].getName());
        assertTrue("Has correct weapon", cd.getWeaponBays().contains(TestTechs.MASS_DRIVER2));
        assertEquals("Has correct weapon", 5, cd.getWeaponBays().getQtyWeapons());
        assertTrue("Craft Design is overfilled", cd.getRemainingSpace()>=0);

        cd = cdm.designCraft(p, CraftDesignerMilitary.SIZE_LARGE, 0.2, 0.2);
        cd.log("Test");
        assertEquals("Has correct hull", TestTechs.FLYER100, cd.getHull());
        assertEquals("Has incorrect engine", TestTechs.FUSION25x, cd.getEngine());
        assertEquals("Has correct computer", TestTechs.ELECTRONIC40, cd.getComputerTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct scanner", TestTechs.PHOTON150, cd.getScannerTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct special", 4, cd.getSpecialTechs().getAllAsArray(true).length);
        assertEquals("Has correct special", TestTechs.SPACE_PODS10, cd.getSpecialTechs().getAllAsArray(true)[0].getName());
        assertEquals("Has correct special", TestTechs.SHIELD_MODULATORII, cd.getSpecialTechs().getAllAsArray(true)[1].getName());
        assertEquals("Has correct special", TestTechs.AUGUMENTED_ENGINEI, cd.getSpecialTechs().getAllAsArray(true)[2].getName());
        assertEquals("Has correct special", TestTechs.ENERGY_AMPLIFIERI, cd.getSpecialTechs().getAllAsArray(true)[3].getName());
        assertEquals("Has correct shield", TestTechs.ENERGY10, cd.getShieldTechs().getAllAsArray(true)[0].getName());
        WeaponBayDesign[] bays = cd.getWeaponBays().getBaysAsArray();
		assertEquals("Has correct weapon", 1, bays.length);
		assertEquals("Has correct weapon", 8, bays[0].getWeaponsList().getWeapons().length);
        assertTrue("Has correct weapon", cd.getWeaponBays().contains(TestTechs.MASS_DRIVER2));
        TechnologyBundle[] tb = cd.getWeaponBays().getAllWeaponsAsList().getDistinct();
        assertEquals("Has correct weapon", TestTechs.MISSILE6, tb[0].getTechnology().getName());
        assertEquals("Has correct weapon", 1, tb[0].getQty());
        assertEquals("Has correct weapon", TestTechs.MASS_DRIVER2, tb[1].getTechnology().getName());
        assertEquals("Has correct weapon", 7, tb[1].getQty());
        assertTrue("Craft Design is overfilled", cd.getRemainingSpace()>=0);

        p.addTechnology(tech_shieldDrain);
        p.addTechnology(tech_immobiliser);
        p.addTechnology(tech_psiI);
        cd = cdm.designCraft(p, CraftDesignerMilitary.SIZE_LARGE, 0.2, 0.2);
        cd.log("Test");
        assertEquals("Has incorrect engine", tech_fusion25x, cd.getEngine());
        assertEquals("Has correct computer", tech_electronic40, cd.getComputerTechs().getAllAsArray(true)[0]);
        assertEquals("Has correct scanner", tech_photon150, cd.getScannerTechs().getAllAsArray(true)[0]);
        assertEquals("Has correct special", 4, cd.getSpecialTechs().getAllAsArray(true).length);
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs().getAllAsArray(true)[0]);
        assertEquals("Has correct special", tech_shieldModulatorII, cd.getSpecialTechs().getAllAsArray(true)[1]);
        assertEquals("Has correct special", tech_augumentedEnginesI, cd.getSpecialTechs().getAllAsArray(true)[2]);
        assertEquals("Has correct special", tech_eneryAmplifierI, cd.getSpecialTechs().getAllAsArray(true)[3]);
        assertEquals("Has correct shield", tech_energy10, cd.getShieldTechs().getAllAsArray(true)[0]);
        assertEquals("Has correct shield", tech_psiI, cd.getShieldTechs().getAllAsArray(true)[1]);
        assertTrue("Has correct weapon", cd.getWeaponBays().contains(tech_massDriver2));
        tb = cd.getWeaponBays().getAllWeaponsAsList().getDistinct();
        assertEquals("Has correct weapon", tech_shieldDrain, tb[0].getTechnology());
        assertEquals("Has correct weapon", 1, tb[0].getQty());
        assertEquals("Has correct weapon", tech_massDriver2, tb[1].getTechnology());
        assertEquals("Has correct weapon", 5, tb[1].getQty());
        assertTrue("Craft Design is overfilled", cd.getRemainingSpace()>=0);

    }

    /**
     * Tests that military crafts are created correctly utilising all the space, no duplicate 
     * techs from different groups.
     */
    public void testCraftDesignFactoryForInvaderCrafts() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftDesignFactoryForInvaderCrafts() *************************");
        createMapWithComputerPlayer(null);
        ComputerPlayer p = (ComputerPlayer) player1;
        CraftDesignerInvader cdi = new CraftDesignerInvader();

        player1.addTechnology(tech_flyer100);
        
        CraftDesign cd = cdi.designCraft(p);

        // Cannot Design Invader unless we have discovered shield drain AND troop transport
        assertEquals("Cannot build Invader", null, cd);
        
        p.addWeapon(tech_shieldDrain);
        cd = cdi.designCraft(p);
        
        // Cannot Design Invader unless we have discovered shield drain AND troop transport
        assertEquals("Cannot build Invader", null, cd);
        
        p.addWeapon(tech_troopTransport);
        cd = cdi.designCraft(p);
        
        WeaponBundle[] wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer60, cd.getHull());
        assertEquals("Has correct engine", TestTechs.FUSION25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 1, wa[0].getQty());
        assertEquals("Has correct weapon", tech_troopTransport, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());

        p.addTechnology(tech_flyer100);
        planet1.setQtyFactories(100);
        planet1.setPopulation(200);
        Logger.logDebug("Planet Production = "+planet1.getProductionOutput()+" hull cost="+tech_flyer100.getCostToBuild());
        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_troopTransport, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());
        
        p.addTechnology(tech_troopModule20);
        p.addTechnology(tech_spacePods10);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_troopTransport, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", 2, cd.getSpecialTechs().getSpecials().length);
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs().getSpecials()[0]);
        assertEquals("Has correct special", tech_troopModule20, cd.getSpecialTechs().getSpecials()[1]);
        assertEquals("Has correct shield", null, cd.getShields());
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());

        p.addTechnology(tech_energy10);
        p.addTechnology(tech_electronic25);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_troopTransport, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", tech_electronic25, cd.getComputerTechs().getAllAsArray()[0]);
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", 1, cd.getSpecialTechs().getSpecials().length);
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs().getSpecials()[0]);
        //assertEquals("Has correct special", tech_troopModule20, cd.getSpecialTechs().getSpecials()[1]);
        assertEquals("Has correct shield", tech_energy10, cd.getShieldTechs().getAllAsArray()[0]);
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());

        p.addTechnology(tech_prioVirus);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 3, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_troopTransport, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct weapon", tech_prioVirus, wa[2].getTechnology());
        assertEquals("Has correct weapon", 1, wa[2].getQty());
        assertEquals("Has correct computer", tech_electronic25, cd.getComputerTechs().getAllAsArray()[0]);
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", 1, cd.getSpecialTechs().getSpecials().length);
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs().getSpecials()[0]);
        assertEquals("Has correct shield", tech_energy10, cd.getShieldTechs().getAllAsArray()[0]);

    }

    /**
     * Tests that military crafts are created correctly utilising all the space, no duplicate 
     * techs from different groups.
     */
    public void testCraftDesignFactoryForDamageBotCrafts() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftDesignFactoryForDamageBotCrafts() *************************");
        createMapWithComputerPlayer(null);
        ComputerPlayer p = (ComputerPlayer) player1;
        CraftDesignerDamageBot cdi = new CraftDesignerDamageBot();

        player1.addTechnology(tech_flyer100);
        
        CraftDesign cd = cdi.designCraft(p);

        // Cannot Design Invader unless we have discovered shield drain AND damage bot
        assertEquals("Cannot build", null, cd);
        
        p.addTechnology(tech_flyer100);
        p.addWeapon(tech_shieldDrain);
        p.addWeapon(tech_damageBots);
        cd = cdi.designCraft(p);
        
        WeaponBundle[] wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer60, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 1, wa[0].getQty());
        assertEquals("Has correct weapon", tech_damageBots, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());

        planet1.setQtyFactories(100);
        planet1.setPopulation(200);
        Logger.logDebug("Planet Production = "+planet1.getProductionOutput()+" hull cost="+tech_flyer100.getCostToBuild());
        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_damageBots, wa[1].getTechnology());
        assertEquals("Has correct weapon", 2, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());
        
        p.addTechnology(tech_spacePods10);
        p.addTechnology(tech_energy10);
        p.addTechnology(tech_electronic25);
        p.addTechnology(tech_electronic40);
        p.addTechnology(tech_photon100);
        p.addTechnology(tech_photon150);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_damageBots, wa[1].getTechnology());
        assertEquals("Has correct weapon", 2, wa[1].getQty());
        assertEquals("Has correct computer", tech_electronic40, cd.getComputerTechs().getAllAsArray()[0]);
        assertEquals("Has correct scanner", tech_photon150, cd.getScannerTechs());
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs());
        assertEquals("Has correct shield", null, cd.getShieldTechs());
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());
        
        player1.addTechnology(tech_cloak150);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct engine", tech_cloak150, cd.getShieldTechs().getShields()[0]);
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_damageBots, wa[1].getTechnology());
        assertEquals("Has correct weapon", 2, wa[1].getQty());
        assertEquals("Has correct computer", tech_electronic40, cd.getComputerTechs().getAllAsArray()[0]);
        assertEquals("Has correct scanner", tech_photon150, cd.getScannerTechs());
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs());
        //assertEquals("Has correct shield", 2, cd.getRemainingSpace());
    }

    public void testCraftDesignFactoryForPlagueCrafts() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftDesignFactoryForPlagueCrafts() *************************");
        createMapWithComputerPlayer(null);
        ComputerPlayer p = (ComputerPlayer) player1;
        CraftDesignerPlague cdi = new CraftDesignerPlague();

        player1.addTechnology(tech_flyer100);
        
        CraftDesign cd = cdi.designCraft(p);

        // Cannot Design Invader unless we have discovered shield drain AND damage bot
        assertEquals("Cannot build", null, cd);
        
        p.addTechnology(tech_flyer100);
        p.addWeapon(tech_shieldDrain);
        p.addWeapon(tech_plague);
        cd = cdi.designCraft(p);
        
        WeaponBundle[] wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer60, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_plague, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());

        planet1.setQtyFactories(100);
        planet1.setPopulation(200);
        Logger.logDebug("Planet Production = "+planet1.getProductionOutput()+" hull cost="+tech_flyer100.getCostToBuild());
        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_plague, wa[1].getTechnology());
        assertEquals("Has correct weapon", 2, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());
        
        p.addTechnology(tech_spacePods10);
        p.addTechnology(tech_energy10);
        p.addTechnology(tech_electronic25);
        p.addTechnology(tech_electronic40);
        p.addTechnology(tech_photon100);
        p.addTechnology(tech_photon150);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_shieldDrain, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct weapon", tech_plague, wa[1].getTechnology());
        assertEquals("Has correct weapon", 2, wa[1].getQty());
        assertEquals("Has correct computer", tech_electronic40, cd.getComputerTechs().getAllAsArray()[0]);
        assertEquals("Has correct scanner", tech_photon150, cd.getScannerTechs());
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs());
        assertEquals("Has correct shield", tech_energy10, cd.getShieldTechs().getAllAsArray()[0]);
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());

    }

    public void testCraftDesignFactoryForPsiCrafts() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftDesignFactoryForPsiCrafts() *************************");
        createMapWithComputerPlayer(null);
        ComputerPlayer p = (ComputerPlayer) player1;
        CraftDesignerPsi cdi = new CraftDesignerPsi();

        player1.addTechnology(tech_flyer100);
        
        CraftDesign cd = cdi.designCraft(p);

        // Cannot Design Invader unless we have discovered shield drain AND damage bot
        assertEquals("Cannot build", null, cd);
        
        p.addTechnology(tech_flyer100);
        p.addWeapon(tech_psiBlast);
        cd = cdi.designCraft(p);
        
        WeaponBundle[] wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer60, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 1, wa.length);
        assertEquals("Has correct weapon", tech_psiBlast, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());

        planet1.setQtyFactories(100);
        planet1.setPopulation(200);
        Logger.logDebug("Planet Production = "+planet1.getProductionOutput()+" hull cost="+tech_flyer100.getCostToBuild());
        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 1, wa.length);
        assertEquals("Has correct weapon", tech_psiBlast, wa[0].getTechnology());
        assertEquals("Has correct weapon", 3, wa[0].getQty());
         assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());
        
        p.addTechnology(tech_spacePods10);
        p.addTechnology(tech_energy10);
        p.addTechnology(tech_electronic25);
        p.addTechnology(tech_electronic40);
        p.addTechnology(tech_photon100);
        p.addTechnology(tech_photon150);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer100, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 1, wa.length);
        assertEquals("Has correct weapon", tech_psiBlast, wa[0].getTechnology());
        assertEquals("Has correct weapon", 2, wa[0].getQty());
         assertEquals("Has correct computer", tech_electronic40, cd.getComputerTechs().getAllAsArray()[0]);
        assertEquals("Has correct scanner", tech_photon150, cd.getScannerTechs());
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs());
        assertEquals("Has correct shield", tech_energy10, cd.getShieldTechs().getAllAsArray()[0]);
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());
        p.addTechnology(tech_photon150);

        p.addTechnology(tech_computerisedFactoryII);
        p.addTechnology(tech_flyer200);
        p.addTechnology(tech_cowardice);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer200, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_psiBlast, wa[0].getTechnology());
        assertEquals("Has correct weapon", 5, wa[0].getQty());
        assertEquals("Has correct weapon", tech_cowardice, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
         assertEquals("Has correct computer", tech_electronic40, cd.getComputerTechs().getAllAsArray()[0]);
        assertEquals("Has correct scanner", tech_photon150, cd.getScannerTechs());
        assertEquals("Has correct special", tech_spacePods10, cd.getSpecialTechs());
        assertEquals("Has correct shield", tech_energy10, cd.getShieldTechs().getAllAsArray()[0]);
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());

    }

    public void testCraftDesignFactoryForDroidMineCraft() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftDesignFactoryForDroidMineCraft() *************************");
        createMapWithComputerPlayer(null);
        ComputerPlayer p = (ComputerPlayer) player1;
        CraftDesignerDroidMine cdi = new CraftDesignerDroidMine();

        player1.addTechnology(tech_flyer100);
        
        CraftDesign cd = cdi.designCraft(p);

        // Cannot Design Invader unless we have discovered shield drain AND damage bot
        assertEquals("Cannot build", null, cd);
        
        p.addTechnology(tech_flyer100);
        p.addTechnology(tech_flyer30);
        p.addWeapon(tech_droidMine);
        p.addWeapon(tech_detonator);
        cd = cdi.designCraft(p);
        
        WeaponBundle[] wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer60, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_droidMine, wa[0].getTechnology());
        assertEquals("Has correct weapon", 1, wa[0].getQty());
        assertEquals("Has correct weapon", tech_detonator, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());

        planet1.setQtyFactories(100);
        planet1.setPopulation(200);
        Logger.logDebug("Planet Production = "+planet1.getProductionOutput()+" hull cost="+tech_flyer100.getCostToBuild());
        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer60, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_droidMine, wa[0].getTechnology());
        assertEquals("Has correct weapon", 1, wa[0].getQty());
        assertEquals("Has correct weapon", tech_detonator, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", null, cd.getScanners());
        assertEquals("Has correct special", null, cd.getSpecials());
        assertEquals("Has correct shield", null, cd.getShields());
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());
        
        p.addTechnology(tech_spacePods10);
        p.addTechnology(tech_energy10);
        p.addTechnology(tech_electronic25);
        p.addTechnology(tech_electronic40);
        p.addTechnology(tech_photon100);
        p.addTechnology(tech_photon150);
        p.addTechnology(tech_holoEmiter);

        cd = cdi.designCraft(p);
        
        wa = cd.getDistinctWeapons();
        cd.log("Test");
        assertEquals("Has correct hull", tech_flyer60, cd.getHull());
        assertEquals("Has correct engine", tech_fusion25, cd.getEngine());
        assertEquals("Has correct weapon", 2, wa.length);
        assertEquals("Has correct weapon", tech_droidMine, wa[0].getTechnology());
        assertEquals("Has correct weapon", 1, wa[0].getQty());
        assertEquals("Has correct weapon", tech_detonator, wa[1].getTechnology());
        assertEquals("Has correct weapon", 1, wa[1].getQty());
        assertEquals("Has correct computer", null, cd.getComputers());
        assertEquals("Has correct scanner", tech_photon150, cd.getScannerTechs());
        assertEquals("Has correct special", tech_holoEmiter, cd.getSpecialTechs());
        assertEquals("Has correct shield", null, cd.getShields());
        //assertEquals("Has correct shield", 5, cd.getRemainingSpace());

    }



    /**
     * Test that the monster .
     */
    public void testCraftFactoryForMonsters() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftFactoryForMonsters() *************************");

        MonsterPlayer player = MotEngine.instance.getMonsterPlayer();
        
        SolarSystem ss1 = newSolarSystem("SS1", 100, 100);
        Planet planet1 = newPlanet("SS1-A", ss1, null, 10, 0.1);
        MonsterFactory.create(planet1, MonsterFactorySmall.class);

        assertEquals(110, planet1.getSize());
        assertEquals(1.0, planet1.getTeraforming(), 0.0);
        Craft [] crafts = player.getCraftsAt(ss1, true, true);
        assertEquals(1, crafts.length);

        // Design is in TestBase!
        assertEquals(tech_flyer30, crafts[0].getHull());
        assertEquals(tech_fusion25, crafts[0].getEngine());
        assertEquals(tech_electronic40, crafts[0].getComputers().getAllAsArray()[0]);
        assertEquals(tech_energy20, crafts[0].getShields().getAllAsArray()[0]);
        assertEquals(1, crafts[0].getWeaponBays().getAllWeapons().length);
        assertEquals(tech_fusionBeam4, crafts[0].getWeaponBays().getAllWeapons()[0]);

        SolarSystem ss2 = newSolarSystem("SS2", 200, 200);
        Planet planet2 = newPlanet("SS2-A", ss2, null, 10, 0.1);
        MonsterFactory.create(planet2, MonsterFactoryMedium.class);
        crafts = player.getCraftsAt(ss2, true, true);

        // Design is in TestBase!
        assertEquals(tech_flyer60, crafts[0].getHull());
        assertEquals(tech_fusion40, crafts[0].getEngine());
        assertEquals(tech_electronic40, crafts[0].getComputers().getAllAsArray()[0]);
        assertEquals(tech_energy20, crafts[0].getShields().getAllAsArray()[0]);
        assertEquals(4, crafts[0].getWeaponBays().getAllWeapons().length);
        assertEquals(tech_fusionBeam4, crafts[0].getWeaponBays().getAllWeapons()[0]);
    }

    /**
     * Test the command to create new crafts for user players works.
     */
    public void testAddCraftDesignCommand() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testAddCraftDesignCommand() *************************");
        createMapWithUserPlayer(null);
        UserPlayer p = (UserPlayer) player1;
        CraftDesign cd = new CraftDesign("Test Design");
        
        //
        // Design with no hull is invalid
        //
        
        try {
			new AddCraftDesign(cd, p).run();
			fail("No Hull");
		} catch (ValidationError e) {
			assertEquals("Invalid Craft Design - Has no hull", e.getMessage());
		}
        cd.setHull(tech_flyer30);

        //
        // Design with no engine is invalid
        //

        try {
			new AddCraftDesign(cd, p).run();
			fail("No Engine");
		} catch (ValidationError e) {
			assertEquals("Invalid Craft Design - Has no engine", e.getMessage());
		}
        cd.setEngine(tech_fusion25);
        
        //
        // Only allowed a single time warp beam
        //

        cd.addWeapon(tech_timeWarpBeam);
        cd.addWeapon(tech_timeWarpBeam);
        try {
			new AddCraftDesign(cd, p).run();
			fail("Not allowed two time warp weapons");
		} catch (ValidationError e) {
			assertEquals("Invalid Craft Design - You can only install 1 Time Warp Beam per craft", e.getMessage());
		}
		
		cd.remove(tech_timeWarpBeam);

    }
    
    /**
     * Tests that we can build a craft on a planet
     * NB This is also done in PlanetTest and DeligateTest 
     */
    public void testSetProjectCommand() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testSetProjectCommand() *************************");
        createMapWithUserPlayer(null);
        UserPlayer p = (UserPlayer) player1;
        CraftDesign cd = new CraftDesign("TestSetProjectCraft", tech_flyer100, tech_fusion25);
        
        //
        // Test cannot use techs I have not discovered
        //
        
        assertEquals(false, player1.hasDiscovered(tech_flyer100));
        assertEquals(true, player1.hasDiscovered(tech_fusion25));
        
        try {
        	new SetProjectCmd(planet1, cd, 1, player1, false, false).run();
        	fail("Have not discovered flyer100");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered hull Flyer100 or have enough in salvage", e.getMessage());
		}
        
        //
        // Test cannot use techs I have not discovered - multiple shields
        //
        
        player1.addTechnology(tech_flyer100);
        player1.addTechnology(tech_energy10);
        assertEquals(false, player1.hasDiscovered(tech_energy20));
        assertEquals(true, player1.hasDiscovered(tech_energy10));

        cd.add(tech_energy10);
        cd.add(tech_energy20);
        
        try {
        	new SetProjectCmd(planet1, cd, 1, player1, false, false).run();
        	fail("Have not discovered Energy20");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered shield Energy20 or have enough in salvage", e.getMessage());
		}
        
        //
        // Test cannot use techs I have not discovered - multiple weapons
        //
        
        player1.addTechnology(tech_energy20);
        assertEquals(false, player1.hasDiscovered(tech_fusionBeam4));
        assertEquals(false, player1.hasDiscovered(tech_detonator));
        assertEquals(true, player1.hasDiscovered(tech_massDriver1));

        cd.add(tech_fusionBeam4);
        cd.add(tech_massDriver1);
        cd.add(tech_detonator);
        
        try {
        	new SetProjectCmd(planet1, cd, 1, player1, false, false).run();
        	fail("Have not discovered FusionBeam4");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered weapon FusionBeam4 or have enough in salvage", e.getMessage());
		}

        player1.addTechnology(tech_fusionBeam4);
        try {
        	new SetProjectCmd(planet1, cd, 1, player1, false, false).run();
        	fail("Have not discovered Detonator");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered weapon Detonator or have enough in salvage", e.getMessage());
		}
        
        player1.addTechnology(tech_detonator);
        
        //
        // Test cannot use techs I have not discovered - specials
        //
        
        assertEquals(false, player1.hasDiscovered(tech_shieldModulatorI));
        cd.add(tech_shieldModulatorI);
        
        try {
        	new SetProjectCmd(planet1, cd, 1, player1, false, false).run();
        	fail("Have not discovered ShieldModulatorI");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered special ShieldModulatorI or have enough in salvage", e.getMessage());
		}

        player1.addTechnology(tech_shieldModulatorI);

        //
        // Test cannot use techs I have not discovered - computers
        //
        
        assertEquals(false, player1.hasDiscovered(tech_electronic40));
        cd.add(tech_electronic40);
        
        try {
        	new SetProjectCmd(planet1, cd, 1, player1, false, false).run();
        	fail("Have not discovered Electronic40");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered computer Electronic40 or have enough in salvage", e.getMessage());
		}

        player1.addTechnology(tech_electronic40);

        //
        // Test cannot use techs I have not discovered - scanners
        //
        
        assertEquals(false, player1.hasDiscovered(tech_photon150));
        cd.add(tech_photon150);
        
        try {
        	new SetProjectCmd(planet1, cd, 1, player1, false, false).run();
        	fail("Have not discovered Photon150");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered scanner Photon150 or have enough in salvage", e.getMessage());
		}

        player1.addTechnology(tech_photon150);
        
        //
        // Test Command does work with all techs
        //
        
    	new SetProjectCmd(planet1, cd, 1, player1, false, false).run();
    	
    	ProjectCraft project = (ProjectCraft) planet1.getProject();
		assertEquals("TestSetProjectCraft", project.getName());
        
    }

    /**
     * Tests that qwe can build a craft on a planet
     * NB This is also done in PlanetTest and DeligateTest 
     */
    public void testSetProjectCommandWithSalvage() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testSetProjectCommandWithSalvage() *************************");
        createMapWithUserPlayer(null);
        UserPlayer p = (UserPlayer) player1;
        CraftDesign cd = new CraftDesign("TestSetProjectCraft", tech_flyer100, tech_fusion25x);
        
        //
        // Test cannot use techs I have not discovered unless have salvage
        //
        
        assertEquals(false, player1.hasDiscovered(tech_flyer100));
        assertEquals(false, player1.hasDiscovered(tech_fusion25x));
        
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        	fail("Have not discovered flyer100");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered hull Flyer100 or have enough in salvage", e.getMessage());
		}
        
        player1.addSalvage(tech_flyer100);
        
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        	fail("Have not discovered fusion25x");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered engine Fusion25x or have enough in salvage", e.getMessage());
		}

        player1.addSalvage(tech_fusion25x);

        //
        // Test cannot use techs I have not discovered unless have salvage - multiple shields
        //
        
        
        player1.addTechnology(tech_energy10);
        assertEquals(false, player1.hasDiscovered(tech_energy20));
        assertEquals(true, player1.hasDiscovered(tech_energy10));

        cd.add(tech_energy10);
        cd.add(tech_energy20);
        
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        	fail("Have not discovered Energy20");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered shield Energy20 or have enough in salvage", e.getMessage());
		}
        
        player1.addSalvage(tech_energy20);

        //
        // Test cannot use techs I have not discovered unless have salvage - multiple weapons
        //
        
        assertEquals(false, player1.hasDiscovered(tech_fusionBeam4));
        assertEquals(false, player1.hasDiscovered(tech_detonator));
        assertEquals(true, player1.hasDiscovered(tech_massDriver1));

        cd.add(tech_fusionBeam4);
        cd.add(tech_massDriver1);
        cd.add(tech_detonator);
        
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        	fail("Have not discovered FusionBeam4");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered weapon FusionBeam4 or have enough in salvage", e.getMessage());
		}

        player1.addSalvage(tech_fusionBeam4);
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        	fail("Have not discovered Detonator");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered weapon Detonator or have enough in salvage", e.getMessage());
		}
        
        player1.addSalvage(tech_detonator);
        
        //
        // Test cannot use techs I have not discovered unless have salvage - specials
        //
        
        assertEquals(false, player1.hasDiscovered(tech_shieldModulatorI));
        cd.add(tech_shieldModulatorI);
        
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        	fail("Have not discovered ShieldModulatorI");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered special ShieldModulatorI or have enough in salvage", e.getMessage());
		}

        player1.addSalvage(tech_shieldModulatorI);

        //
        // Test cannot use techs I have not discovered unless have salvage - computers
        //
        
        assertEquals(false, player1.hasDiscovered(tech_electronic40));
        cd.add(tech_electronic40);
        
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        	fail("Have not discovered Electronic40");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered computer Electronic40 or have enough in salvage", e.getMessage());
		}

        player1.addSalvage(tech_electronic40);

        //
        // Test cannot use techs I have not discovered unless have salvage - scanners
        //
        
        assertEquals(false, player1.hasDiscovered(tech_photon150));
        cd.add(tech_photon150);
        
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        	fail("Have not discovered Photon150");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered scanner Photon150 or have enough in salvage", e.getMessage());
		}

        player1.addSalvage(tech_photon150);

        //
        // Test quantity to build does work - Try to build 2 crafts
        //
        
        player1.addShield(tech_energy20);
        player1.addEngine(tech_fusion25x);
        player1.addWeapon(tech_fusionBeam4);
        player1.addWeapon(tech_detonator);
        player1.addScanner(tech_photon150);
        player1.addSpecial(tech_shieldModulatorI);
        player1.addComputer(tech_electronic40);
        player1.getSalvage().removeAll();
        player1.getHulls().remove(tech_flyer100);
        player1.getSalvage().add(tech_flyer100);
        planet1.setProduction(0);
        try {
        	new SetProjectCmd(planet1, cd, 1, player1, false, true).run();
        	fail("Have not discovered Hull flyer100");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered hull Flyer100 or have enough in salvage", e.getMessage());
		}
        try {
        	new SetProjectCmd(planet1, cd, 0, player1, false, false).run();
        	fail("Have not discovered Hull flyer100");
        } catch (ValidationError e) {
        	assertEquals("You have not discovered hull Flyer100 or have enough in salvage", e.getMessage());
		}

        //
        // Test Command does work and uses salvage item
        //
        
    	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
    	
    	ProjectCraft project = (ProjectCraft) planet1.getProject();
		assertEquals("TestSetProjectCraft", project.getName());
		assertEquals(2445, cd.getCostToBuildOnPlanet());
		assertEquals(1045, project.getCostToBuildOnPlanet());
		assertEquals(0, player1.getSalvage().getQuantity());

        //
        // Test Command does work and uses salvage item
        //

        cd = new CraftDesign("TD", tech_flyer100, tech_fusion25, tech_electronic25, tech_armour25, tech_fusionBeam4, tech_photon100);
        player1.getSalvage().add(tech_flyer100, 2);
        player1.getSalvage().add(tech_fusion25, 2);
        player1.getSalvage().add(tech_electronic25, 2);
        player1.getSalvage().add(tech_armour25, 2);
        player1.getSalvage().add(tech_fusionBeam4, 2);
        player1.getSalvage().add(tech_photon100, 2);

    	new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
    	
    	project = (ProjectCraft) planet1.getProject();
		assertEquals("TD", project.getName());
		assertEquals(1930, cd.getCostToBuildOnPlanet());
		assertEquals(0, project.getCostToBuildOnPlanet());
		assertEquals(1, player1.getSalvage().getQuantity(tech_flyer100));
		assertEquals(1, player1.getSalvage().getQuantity(tech_fusion25));
		assertEquals(1, player1.getSalvage().getQuantity(tech_electronic25));
		assertEquals(1, player1.getSalvage().getQuantity(tech_armour25));
		assertEquals(1, player1.getSalvage().getQuantity(tech_fusionBeam4));
		assertEquals(1, player1.getSalvage().getQuantity(tech_photon100));

    }

    /**
     * Tests that we can build a craft on a planet
     * NB This is also done in PlanetTest and DeligateTest 
     */
    public void testSalvageReducesCost() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testSalvageReducesCost() *************************");
        createMapWithUserPlayer(null);
        UserPlayer p = (UserPlayer) player1;
        CraftDesign cd = new CraftDesign("TestSetProjectCraft", tech_flyer100, tech_fusion25);
        
        //
        // Test basics
        //
        
        player1.addTechnology(tech_flyer100);
        
        new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        assertEquals(1700, planet1.getProject().getCostToBuildOnPlanet());
        
        player1.addSalvage(tech_flyer100);
        
        Project originalProject = planet1.getProject();
        new SetProjectCmd(planet1, cd, 1, player1, false, true).run();
        assertEquals(300, planet1.getProject().getCostToBuildOnPlanet());
        assertEquals(0, player1.getSalvage().getQuantity());
        assertEquals(1700, originalProject.getCostToBuildOnPlanet());
        originalProject = planet1.getProject();

        player1.addSalvage(tech_fusion25);

        // Salvage flyer100 has been used so only now have engine funsion25 for free
        // New projects being created do not effect the cost of old projects
        new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        assertEquals(1400, planet1.getProject().getCostToBuildOnPlanet());
        assertEquals(300, originalProject.getCostToBuildOnPlanet());
        assertEquals(0, player1.getSalvage().getQuantity());

        //
        // Test Shields.... 
        // 
       
        player1.addSalvage(tech_flyer100);
        player1.addSalvage(tech_fusion25);
        player1.addSalvage(tech_energy20);
        player1.addTechnology(tech_energy10);
        player1.addTechnology(tech_energy20);
        cd.add(tech_energy10);
        cd.add(tech_energy20);
        
        new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        assertEquals(150, planet1.getProject().getCostToBuildOnPlanet());
        assertEquals(0, player1.getSalvage().getQuantity());

        new SetProjectCmd(planet1, cd, 1, player1, false, true).run();
        assertEquals(2050, planet1.getProject().getCostToBuildOnPlanet());

        //
        // Test Weapons.... 
        // 
        
        player1.addSalvage(tech_flyer100);
        player1.addSalvage(tech_fusion25);
        player1.addSalvage(tech_energy10);
        player1.addSalvage(tech_energy20);
        player1.addSalvage(tech_fusionBeam4);
        player1.addTechnology(tech_fusionBeam4);
        player1.addTechnology(tech_massDriver1);
        cd.add(tech_fusionBeam4);
        cd.add(tech_massDriver1);
        
        new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        assertEquals(20, planet1.getProject().getCostToBuildOnPlanet());
        assertEquals(0, player1.getSalvage().getQuantity());
        

        new SetProjectCmd(planet1, cd, 0, player1, false, true).run();
        assertEquals(2100, planet1.getProject().getCostToBuildOnPlanet());
     }


    /**
     * Tests that we can build a craft on a planet
     * NB This is also done in PlanetTest and DeligateTest 
     */
    public void testCraftUpgradeCosts() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftUpgradeCosts() *************************");
        createMapWithUserPlayerAndComputerPlayer(null);
        UserPlayer p = (UserPlayer) player1;
        
        player1.addTechnology(tech_flyer60);
        player1.addTechnology(tech_fusion25x);
        player1.addTechnology(tech_electronic25);
        player1.addTechnology(tech_electronic40);
        player1.addTechnology(tech_massDriver1);
        player1.addTechnology(tech_massDriver2);
        player1.addTechnology(tech_photon100);
        player1.addTechnology(tech_photon150);
        
        CraftDesign cd1 = new CraftDesign("TCA", tech_flyer60, tech_fusion25, tech_electronic25, tech_massDriver1, tech_photon100);
        Craft craft = CraftFactory.getCraftInstance(cd1, player1, player1, planet1.getOrbit(), false, false);
        
        //
        // Test upgrade a single item - the engine costs are right
        //
        
        CraftDesign cd2 = new CraftDesign("TCB", tech_flyer60, tech_fusion25x, tech_electronic25, tech_massDriver1, tech_photon100);
        CraftUpgradeDesign cud = new CraftUpgradeDesign(cd2, new CraftVOAdapter(craft, player1));
        
        assertEquals(200, cud.getCostToBuildOnPlanet());
        
		new SetProjectCmd(planet1, cud, 0, player1, false, true).run();
		ProjectCraftUpgrade cup = (ProjectCraftUpgrade) planet1.getProject();

        assertEquals(200, cup.getCostToBuild());
        
    }
    
    /**
     * Tests that we can build a craft on a planet
     * NB This is also done in PlanetTest and DeligateTest 
     */
    public void testCraftUpgrade() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftUpgrade() *************************");
        createMapWithUserPlayerAndComputerPlayer(null);
        UserPlayer p = (UserPlayer) player1;
        
        CraftDesign cd1 = new CraftDesign("TCA", tech_flyer100, tech_fusion25, tech_electronic25, tech_armour25, tech_massDriver1, tech_photon100);
        Craft craft = CraftFactory.getCraftInstance(cd1, player1, player1, planet1.getOrbit(), false, false);
        
        //
        // Test upgrade a single craft
        //
        
        CraftDesign cd2 = new CraftDesign("TCB", tech_flyer100, tech_fusion25x, tech_electronic40, tech_armour200, tech_massDriver2, tech_photon150);
        CraftUpgradeDesign cud = new CraftUpgradeDesign(cd2);
        player1.addTechnology(tech_flyer100);
        player1.addTechnology(tech_fusion25x);
        player1.addTechnology(tech_electronic40);
        player1.addTechnology(tech_armour200);
        player1.addTechnology(tech_massDriver2);
        player1.addTechnology(tech_photon150);
        
        new SetProjectCmd(planet1, new CraftUpgradeDesign(cd2, new CraftVOAdapter(craft, player1)), 0, player1, false, true).run();
        
        assertEquals(600, planet1.getProject().getCostToBuildOnPlanet());
        assertEquals(cd2, ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getDesign());
        assertEquals(1, ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts().length);
        assertEquals("TCA1", ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts()[0].getName());
        assertNull(player1.getCrafts());
        assertNull(MotEngine.instance.getCrafts().getCrafts());

        planet1.addProduction(planet1.getProject().getCostToBuildOnPlanet());
        craft = player1.getCrafts()[0];
        
        assertNull(planet1.getProject());
        assertEquals(1, player1.getCrafts().length);
        assertEquals(tech_flyer100, craft.getHull());
        assertEquals(tech_fusion25x, craft.getEngine());
        assertEquals(tech_electronic40, craft.getComputers());
        assertEquals(tech_armour200, craft.getSpecials());
        assertEquals(tech_massDriver2, craft.getWeaponBays().getAllWeaponsAsList());
        assertEquals(tech_photon150, craft.getScanners());
        assertEquals(craft, MotEngine.instance.getCrafts().getCrafts());

        //
        // Upgrade crafts in a fleet
        //
        
        player1.getCrafts()[0].destroy();
        Craft craft1 = CraftFactory.getCraftInstance(cd1, player1, player1, planet1.getOrbit(), false, false);
        Craft craft2 = CraftFactory.getCraftInstance(cd1, player1, player2, planet1.getOrbit(), false, false);
        Fleet fleet = FleetFactory.create(craft1, craft2);
        
        new SetProjectCmd(planet1, new CraftUpgradeDesign(cd2, new CraftVOAdapter(craft1, player1), new CraftVOAdapter(craft2, player1)), 0, player1, false, true).run();
        
        assertEquals(1200, planet1.getProject().getCostToBuildOnPlanet());
        assertEquals(cd2, ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getDesign());
        assertEquals(2, ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts().length);
        assertEquals("TCA2", ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts()[0].getName());
        assertEquals("TCA3", ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts()[1].getName());
        assertNull(player1.getCrafts());
        assertNull(MotEngine.instance.getCrafts().getCrafts());

        planet1.addProduction(planet1.getProject().getCostToBuildOnPlanet());
        
        craft1 = player1.getCrafts()[0];
        craft2 = player1.getCrafts()[1];
        
        assertNull(planet1.getProject());
        assertEquals(2, player1.getCrafts().length);
        assertEquals("TCB2", player1.getCrafts()[0].getName());
        assertEquals(tech_flyer100, craft1.getHull());
        assertEquals(tech_fusion25x, craft1.getEngine());
        assertEquals(tech_electronic40, craft1.getComputers());
        assertEquals(tech_armour200, craft1.getSpecials());
        assertEquals(tech_massDriver2, craft1.getWeaponBays().getAllWeaponsAsList());
        assertEquals(tech_photon150, craft1.getScanners());
        assertEquals("TCB3", craft2.getName());
        assertEquals(tech_flyer100, craft2.getHull());
        assertEquals(tech_fusion25x, craft2.getEngine());
        assertEquals(tech_electronic40, craft2.getComputers());
        assertEquals(tech_armour200, craft2.getSpecials());
        assertEquals(tech_massDriver2, craft2.getWeaponBays().getAllWeaponsAsList());
        assertEquals(tech_photon150, craft2.getScanners());
        assertEquals(craft1, craft2, MotEngine.instance.getCrafts().getCrafts());
        assertNull(player1.getFleet());
        assertNull(craft1.getFleet());
        assertNull(craft2.getFleet());
        assertEquals(player1, craft1.getOwner());
        assertEquals(player1, craft2.getOwner());
        assertEquals(player1, craft1.getOrigin());
        assertEquals(player2, craft2.getOrigin());

        //
        // Upgrade craft that does not need upgrading
        //
        
        Craft craft3 = CraftFactory.getCraftInstance(cd1, player1, player2, planet1.getOrbit(), false, false);
        
        new SetProjectCmd(planet1, new CraftUpgradeDesign(cd2, new CraftVOAdapter(craft2, player1), new CraftVOAdapter(craft3, player1)), 0, player1, false, true).run();
        
        assertEquals(600, planet1.getProject().getCostToBuildOnPlanet());
        assertEquals(cd2, ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getDesign());
        assertEquals(2, ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts().length);
        assertEquals("TCB3", ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts()[0].getName());
        assertEquals("TCA4", ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts()[1].getName());
        assertEquals(1, player1.getCrafts().length);
        assertEquals("TCB2", player1.getCrafts()[0].getName());
        assertEquals(1, MotEngine.instance.getCrafts().getCrafts().length);
        assertEquals("TCB2", MotEngine.instance.getCrafts().getCrafts()[0].getName());

        planet1.addProduction(planet1.getProject().getCostToBuildOnPlanet());
        
        craft1 = player1.getCrafts()[0];
        craft2 = player1.getCrafts()[1];
        craft3 = player1.getCrafts()[2];
        
        assertNull(planet1.getProject());
        assertEquals(3, player1.getCrafts().length);
        assertEquals("TCB2", craft1.getName());
        assertEquals(tech_flyer100, craft1.getHull());
        assertEquals(tech_fusion25x, craft1.getEngine());
        assertEquals(tech_electronic40, craft1.getComputers());
        assertEquals(tech_armour200, craft1.getSpecials());
        assertEquals(tech_massDriver2, craft1.getWeaponBays().getAllWeaponsAsList());
        assertEquals(tech_photon150, craft1.getScanners());
        assertEquals("TCB4", craft2.getName());
        assertEquals(tech_flyer100, craft2.getHull());
        assertEquals(tech_fusion25x, craft2.getEngine());
        assertEquals(tech_electronic40, craft2.getComputers());
        assertEquals(tech_armour200, craft2.getSpecials());
        assertEquals(tech_massDriver2, craft2.getWeaponBays().getAllWeaponsAsList());
        assertEquals(tech_photon150, craft2.getScanners());
        assertEquals("TCB5", craft3.getName());
        assertEquals(tech_flyer100, craft3.getHull());
        assertEquals(tech_fusion25x, craft3.getEngine());
        assertEquals(tech_electronic40, craft3.getComputers());
        assertEquals(tech_armour200, craft3.getSpecials());
        assertEquals(tech_massDriver2, craft3.getWeaponBays().getAllWeaponsAsList());
        assertEquals(tech_photon150, craft3.getScanners());
        assertEquals(craft1, craft2, craft3, MotEngine.instance.getCrafts().getCrafts());
    }
    

    /**
     * Tests that we can build a craft on a planet
     * NB This is also done in PlanetTest and DeligateTest 
     */
    public void testCraftUpgradeWithSalvage() throws Exception {
        Logger.logDebug("*******************************  CraftDesignTest.testCraftUpgradeWithSalvage() *************************");
        createMapWithUserPlayerAndComputerPlayer(null);
        UserPlayer p = (UserPlayer) player1;
        
        CraftDesign cd1 = new CraftDesign("TCA", tech_flyer100, tech_fusion25, tech_electronic25, tech_armour25, tech_massDriver1, tech_photon100);
        Craft craft = CraftFactory.getCraftInstance(cd1, player1, player1, planet1.getOrbit(), false, false);
        
        //
        // Test upgrade a single craft
        //
        
        CraftDesign cd2 = new CraftDesign("TCB", tech_flyer100, tech_fusion25x, tech_electronic40, tech_armour200, tech_massDriver2, tech_photon150);
        CraftUpgradeDesign cud = new CraftUpgradeDesign(cd2);
        player1.addSalvage(tech_flyer100);
        player1.addSalvage(tech_electronic40);
        player1.addSalvage(tech_armour200);
        player1.addSalvage(tech_massDriver2);
        player1.addSalvage(tech_photon150);
        player1.addSalvage(tech_fusion25x);
        
        new SetProjectCmd(planet1, new CraftUpgradeDesign(cd2, new CraftVOAdapter(craft, player1)), 0, player1, false, true).run();
        
        assertEquals(0, planet1.getProject().getCostToBuildOnPlanet());
        assertEquals(cd2, ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getDesign());
        assertEquals(1, ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts().length);
        assertEquals("TCA1", ((ProjectCraftUpgrade)planet1.getProject()).getCraftDesign().getCrafts()[0].getName());
        assertNull(player1.getCrafts());
        assertNull(MotEngine.instance.getCrafts().getCrafts());

        planet1.addProduction(1);
        craft = player1.getCrafts()[0];
        
        assertNull(planet1.getProject());
        assertEquals(1, player1.getCrafts().length);
        assertEquals(tech_flyer100, craft.getHull());
        assertEquals(tech_fusion25x, craft.getEngine());
        assertEquals(tech_electronic40, craft.getComputers());
        assertEquals(tech_armour200, craft.getSpecials());
        assertEquals(tech_massDriver2, craft.getWeaponBays().getAllWeaponsAsList());
        assertEquals(tech_photon150, craft.getScanners());
        assertEquals(craft, MotEngine.instance.getCrafts().getCrafts());

    }
    

    protected void tearDown() {
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}