/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.event.MotEvent;
import mot.common.event.PlanetBombardedEvent;
import mot.common.event.PlanetCrisisEvent;
import mot.common.lang.L;
import mot.common.proxy.ClientProxy;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Modules;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Specials;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.command.MooDeployCmd;
import mot.engine.command.MooMoveCmd;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.BlackHole;
import mot.engine.planets.BlackHoleFactory;
import mot.engine.planets.CrisisEmergency;
import mot.engine.planets.CrisisPlague;
import mot.engine.planets.CrisisPsi;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.players.strategists.StrategistEventUsedBiologicalWeapons;

/**
 * Tests that a craft moves between two solar system correctly.
 * Test the General objects.
 */
public class CraftTest extends TestBase implements ClientProxy {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(mot.test.CraftTest.class);
        return suite;
    }

    public CraftTest(String name) {
        super(name);
    }

    private SetupOptions options;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;
    private SolarSystem ss5;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;
    private Planet planet5;


    protected void setUp() throws Exception {
        MotEngine.newInstance();
        setupConfigs();
        Logger.setDebug();
        MotEngine.getInstance().getProxyManager().add(this);

        clearEvents();
        
        String [] playerNames = { "TestPlayer1", "TestPlayer2" };
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 150, 100);

        ss3 = newSolarSystem("SS3", 199, 100);
        planet3 = newPlanet("SS3-A", ss3, null, 100, 0.7);

        ss4 = newSolarSystem("SS4", 248, 100);

        ss5 = newSolarSystem("SS5", 347, 100);
        planet5 = newPlanet("SS5-A", ss4, null, 110, 1.0);
    }
    

    /**
     * Test that crafts move at the right speed between stellar objects on the galaxy map
     * and that speed is effected by player and craft special bonuses.
     * Tests that crafts arriving at a stellar object have a planetary orbit generated 
     * for them (FOR COMPUTER PLAYERS)
     */
    public void testMovementToStellarObject1() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testMovementToStellarObject1() *************************");
        
        Craft craft = newCraft(fighterDesign, player1, ss1);
        craft.moveTo(ss3.getOrbit());
        
        assertNotNull("Craft has a navigator", craft.getNavigator());
        assertEquals("Craft is still located at ss1", ss1, craft.getOrbit().getStellarObject());
        assertEquals("Craft is still located at ss1", null, craft.getOrbit().getPlanet());
        assertEquals("Craft navigator will move it to ss2", ss3, craft.getNavigator().getDestination().getStellarObject());
        assertEquals("Craft navigator will move it to ss2", null, craft.getNavigator().getDestination().getPlanet());
        assertTrue("Craft still located at ss1", ss1.getLocation().equals(craft.getLocation()));
        assertEquals("Engine speed is correct", 25, craft.getEngine().getWarpSpeed());
        assertEquals("Engine speed is correct", 4, craft.getNavigator().getETA());
        
        craft.endTurn();

        assertNull("Craft has left ss1", craft.getOrbit());
        assertTrue("Craft location is correct", craft.getLocation().equals(125, 100));
        
        // Add AugumentedEnginesI to increase speed - increase speed by 0.1 => speed = 27.5
        craft.setSpecials(new Specials(tech_augumentedEnginesI));

        craft.endTurn();

        assertTrue("Craft location is correct", craft.getLocation().equals(152, 100));
        
        craft.endTurn();
        craft.endTurn();

        assertNull("Craft has no navigator", craft.getNavigator());
        assertEquals("Craft is located at ss3", ss3, craft.getOrbit().getStellarObject());
        assertEquals("Craft planet orbit generated for it", planet3, craft.getOrbit().getPlanet());
        assertTrue("Craft still located at ss3", ss3.getLocation().equals(craft.getLocation()));
    }
    
    /**
     * Tests that crafts arriving at a stellar object with a planet orbit do not have a
     * planetary orbit generated for them (FOR HUMAN PLAYERS)
     */
    public void testMovementToStellarObject2() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testMovementToStellarObject2() *************************");
        
        String userPlayerName = "TestUserPLaner";
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions ro = new RaceOptions(userPlayerName, null, options);
        UserPlayer userPlayer = newUserPlayer(userPlayerName, blue, null, ro);
        Craft craft = newCraft(fighterDesign, userPlayer, ss1);
        craft.moveTo(ss3.getOrbit());
        
        assertNotNull("Craft has a navigator", craft.getNavigator());
        assertEquals("Craft is still located at ss1", ss1, craft.getOrbit().getStellarObject());
        assertEquals("Craft is still located at ss1", null, craft.getOrbit().getPlanet());
        assertEquals("Craft navigator will move it to ss2", ss3, craft.getNavigator().getDestination().getStellarObject());
        assertEquals("Craft navigator will move it to ss2", null, craft.getNavigator().getDestination().getPlanet());
        
        craft.endTurn();
        craft.endTurn();
        craft.endTurn();
        craft.endTurn();

        assertNull("Craft has no navigator", craft.getNavigator());
        assertEquals("Craft is located at ss3", ss3, craft.getOrbit().getStellarObject());
        assertEquals("Craft planet orbit not generated", null, craft.getOrbit().getPlanet());
        assertTrue("Craft still located at ss3", ss3.getLocation().equals(craft.getLocation()));
    }
    
    public void testCraftRepair() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testCraftRepair() *************************");

        Craft craft = newCraft(fighterDesign, player1, ss1);
        
        // Test that crafts repair themselves at the end of every turn.
        craft.setDamage(5);
        craft.endTurn();
        assertEquals("Total hit points is correct", 20, craft.getMaxDamage());
        assertEquals("Craft has repaired its self", 0.0, craft.getDamage(), 0.001);

        // Test that crafts repair themselves at the end of every turn.
        craft.setDamage(10);
        craft.endTurn();
        assertEquals("Total hit points is correct", 20, craft.getMaxDamage());
        assertEquals("Craft has repaired its self", 4.0, craft.getDamage(), 0.001);

        craft.endTurn();
        craft.endTurn();
        assertEquals("Craft has repaired its self", .0, craft.getDamage(), 0.001);
    }
    
    /**
     * Test that fleets move at the right speed between stellar objects on the galaxy map
     * and that speed is effected by player and craft special bonuses.
     * Tests that fleets arriving at a stellar object without a planet orbit being specified
     * have a planetary orbit generated for them.
     */
    public void testMovementToPlanet() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testMovementToPlanet() *************************");
        
        Craft craft = newCraft(fighterDesign, player1, planet1);
        craft.moveTo(planet3.getOrbit());
        
        assertNotNull("Craft has a navigator", craft.getNavigator());
        assertEquals("Craft is still located at ss1", ss1, craft.getOrbit().getStellarObject());
        assertEquals("Craft is still located at ss1", planet1, craft.getOrbit().getPlanet());
        assertEquals("Craft navigator will move it to ss3", ss3, craft.getNavigator().getDestination().getStellarObject());
        assertEquals("Craft navigator will move it to ss3", planet3, craft.getNavigator().getDestination().getPlanet());
        assertTrue("Craft still located at ss1", ss1.getLocation().equals(craft.getLocation()));
        assertEquals("Engine speed is correct", 25, craft.getEngine().getWarpSpeed());
        assertEquals("Engine speed is correct", 4, craft.getNavigator().getETA());
        
        craft.endTurn();

        assertNull("Craft has left ss1", craft.getOrbit());
        assertTrue("Craft location is correct", craft.getLocation().equals(125, 100));
        
        // Add AugumentedEnginesI to increase speed - increase speed by 0.1 => speed = 27.5
        craft.setSpecials(new Specials(tech_augumentedEnginesI));

        craft.endTurn();

        assertTrue("Craft location is correct", craft.getLocation().equals(152, 100));
        
        craft.endTurn();
        craft.endTurn();

        assertNull("Craft has no navigator", craft.getNavigator());
        assertEquals("Craft is located at ss3", ss3, craft.getOrbit().getStellarObject());
        assertEquals("Craft is located at ss3", planet3, craft.getOrbit().getPlanet());
        assertTrue("Craft still located at ss3", ss3.getLocation().equals(craft.getLocation()));
        
        // Test that crafts repair themselves at the end of every turn.
        craft.setDamage(5);
        craft.endTurn();
        assertEquals("Total hit points is correct", 20, craft.getMaxDamage());
        assertEquals("Craft has repaired its self", 3.0, craft.getDamage(), 0.001);

        craft.endTurn();
        craft.endTurn();
        assertEquals("Craft has repaired its self", .0, craft.getDamage(), 0.001);
    }
    
    /**
     * Test that fleets move at the right speed between stellar objects on teh galaxy map
     * and that speed is effected by player and craft special bonuses.
     */
    public void testFleetMovementToStellarObject() throws Exception {
        Logger.logDebug("*******************************  CraftTest.testFleetMovementToStellarObject() *************************");

        player1.setFunds(1000);
        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player1, ss1);
        Craft craft3 = newCraft(fighterDesign, player1, ss1);
        craft1.setEngine(tech_fusion40);
        craft3.setEngine(tech_fusion40);
        Fleet fleet = new Fleet(player1, ss1.getOrbit());
        fleet.add(craft1);
        fleet.add(craft1);
        fleet.add(craft2);
        fleet.moveTo(ss3.getOrbit());
        
        assertNotNull("Craft has a navigator", fleet.getNavigator());
        assertEquals("Craft is still located at ss1", ss1, fleet.getOrbit().getStellarObject());
        assertEquals("Craft is still located at ss1", null, fleet.getOrbit().getPlanet());
        assertEquals("Craft navigator will move it to ss3", ss3, fleet.getNavigator().getDestination().getStellarObject());
        assertEquals("Craft navigator will move it to ss3", null, fleet.getNavigator().getDestination().getPlanet());
        assertTrue("Craft still located at ss1", ss1.getLocation().equals(fleet.getLocation()));
        assertEquals("Engine speed is correct", 25, fleet.getEngine().getWarpSpeed());
        
        fleet.endTurn();

        assertNull("Craft has left ss1", fleet.getOrbit());
        assertTrue("Craft location is correct", fleet.getLocation().equals(125, 100));
        
        // Add AugumentedEnginesI to increase speed - increase speed by 0.1 => speed = 27.5
        craft2.setSpecials(new Specials(tech_augumentedEnginesI));

        fleet.endTurn();

        assertTrue("Craft location is correct", fleet.getLocation().equals(152, 100));

        // Now fleets slowest engine in Fusion40 => warp speed = 40*1.1=44.
        craft2.setEngine(tech_fusion40);
        fleet.setEngine(fleet.getFleetEngine());
        
        fleet.endTurn();

        assertTrue("Craft location is correct ", fleet.getLocation().equals(196, 100));

        fleet.endTurn();

        assertNull("Craft has no navigator", fleet.getNavigator());
        assertEquals("Craft is located at ss3", ss3, fleet.getOrbit().getStellarObject());
        assertEquals("Craft is located at ss3", planet3, fleet.getOrbit().getPlanet());
        assertTrue("Craft still located at ss3", ss3.getLocation().equals(fleet.getLocation()));
    }

    /**
     * Test that fleets move at the right speed between stellar objects on teh galaxy map
     * and that speed is effected by player and craft special bonuses.
     */
    public void testFleetMovementToPlanet() throws Exception {
        Logger.logDebug("*******************************  CraftTest.testFleetMovementToPlanet() *************************");

        player1.setFunds(1000);
        Craft craft1 = newCraft(fighterDesign, player1, planet1);
        Craft craft2 = newCraft(fighterDesign, player1, planet1);
        Craft craft3 = newCraft(fighterDesign, player1, planet1);
        craft1.setEngine(tech_fusion40);
        craft3.setEngine(tech_fusion40);
        Fleet fleet = new Fleet(player1, planet1.getOrbit());
        fleet.add(craft1);
        fleet.add(craft1);
        fleet.add(craft2);
        fleet.moveTo(planet3.getOrbit());
        
        assertNotNull("Craft has a navigator", fleet.getNavigator());
        assertEquals("Craft is still located at ss1", ss1, fleet.getOrbit().getStellarObject());
        assertEquals("Craft is still located at ss1", planet1, fleet.getOrbit().getPlanet());
        assertEquals("Craft navigator will move it to ss3", ss3, fleet.getNavigator().getDestination().getStellarObject());
        assertEquals("Craft navigator will move it to ss3", planet3, fleet.getNavigator().getDestination().getPlanet());
        assertTrue("Craft still located at ss1", ss1.getLocation().equals(fleet.getLocation()));
        assertEquals("Engine speed is correct", 25, fleet.getEngine().getWarpSpeed());
        
        fleet.endTurn();

        assertNull("Craft has left ss1", fleet.getOrbit());
        assertTrue("Craft location is correct", fleet.getLocation().equals(125, 100));
        
        // Add AugumentedEnginesI to increase speed - increase speed by 0.1 => speed = 27.5
        craft2.setSpecials(new Specials(tech_augumentedEnginesI));

        fleet.endTurn();

        assertTrue("Craft location is correct", fleet.getLocation().equals(152, 100));

        // Now fleets slowest engine in Fusion40 => warp speed = 40*1.1=44.
        craft2.setEngine(tech_fusion40);
        fleet.setEngine(fleet.getFleetEngine());
        
        fleet.endTurn();

        assertTrue("Craft location is correct ", fleet.getLocation().equals(196, 100));

        fleet.endTurn();

        assertNull("Craft has no navigator", fleet.getNavigator());
        assertEquals("Craft is located at ss3", ss3, fleet.getOrbit().getStellarObject());
        assertEquals("Craft is located at ss3", planet3, fleet.getOrbit().getPlanet());
        assertTrue("Craft still located at ss3", ss3.getLocation().equals(fleet.getLocation()));
    }

    /**
     * Tests that crafts are visible when in scanning range of a planet or outpost.
     * Tests that cloaking reduces a crafts visibility.
     */
    public void testVisibility() throws Exception {
        Logger.logDebug("*******************************  CraftTest.testVisibility() *************************");
        
        Craft enemy = newCraft(fighterDesign, player2, ss1);
        
        assertEquals(true, enemy.isVisible(player1));
        
        enemy.transferTo(ss3.getOrbit());

        assertEquals(false, enemy.isVisible(player1));
        
        planet1.setScanners(new Scanners(tech_photon100));

        assertEquals(true, enemy.isVisible(player1));

        enemy.transferTo(ss4.getOrbit());

        assertEquals(false, enemy.isVisible(player1));

        Craft mine = newCraft(fighterDesign, player1, ss2);
        mine.setScanners(new Scanners(tech_photon100));
        
        assertEquals(true, enemy.isVisible(player1));

        enemy.transferTo(ss5.getOrbit());

        assertEquals(false, enemy.isVisible(player1));
        
        Orbital outpost = new Orbital(tech_outpost, tech_photon100, null);
        ss4.deploy(outpost, player1);

        assertEquals(true, enemy.isVisible(player1));
        
        enemy.setShields(new Shields(tech_cloak150));

        assertEquals(false, enemy.isVisible(player1));
    }

    /**
     * Tests that crafts located in an enemy solar system blockade.
     * Tests that blockades are removed when a crft is destroyed OR when they move away.
     */
    public void testRange() throws Exception {
        Logger.logDebug("*******************************  ExploreTest.testRange() *************************");

		CraftDesign explorerDesign = new CraftDesign("TestExplorer", 
				tech_flyer30,
				tech_fusion25);
        
        SolarSystem ss6 = newSolarSystem("SS6", 397, 100);
        Craft craft1 = newCraft(explorerDesign, player1, ss1);

        assertTrue(player1.canTravelTo(ss2));
        assertTrue(craft1.canTravelTo(ss2));
        assertTrue(player1.canTravelTo(ss3));
        assertTrue(craft1.canTravelTo(ss3));
        assertTrue(player1.canTravelTo(ss4));
        assertTrue(craft1.canTravelTo(ss4));
        
        assertFalse(player1.canTravelTo(ss5));
        assertFalse(craft1.canTravelTo(ss5));
        assertFalse(player1.canTravelTo(ss6));
        assertFalse(craft1.canTravelTo(ss6));

        //
        // Test building an outpost extends our range
        //

        Orbital outpost = new Orbital(tech_outpost, null, null);
        ss3.deploy(outpost, player1);

        assertTrue(player1.canTravelTo(ss5));
        assertTrue(craft1.canTravelTo(ss5));
        
        assertFalse(player1.canTravelTo(ss6));
        assertFalse(craft1.canTravelTo(ss6));
        
        //
        // Test better engines extend our range
        //

        player1.addEngine(tech_fusion25x);
        craft1.setEngine(tech_fusion25x);

        assertTrue(player1.canTravelTo(ss6));
        assertTrue(craft1.canTravelTo(ss6));
        
    }

    /**
     * Tests that crafts located in an enemy solar system blockade.
     * Tests that blockades are removed when a crft is destroyed OR when they move away.
     */
    public void testBlockade() throws Exception {
        Logger.logDebug("*******************************  ExploreTest.testBlockade() *************************");

        //
        // Test owners own craft do not blockade planet
        //

        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        assertFalse(planet1.isBlockaded());

        //
        // Test friendly players craft do not blockade planet
        //

        Craft craft2 = newCraft(fighterDesign, player2, ss1);
        planet1.flushBlockadeCache();
        assertFalse(planet1.isBlockaded());

        //
        // Test enemy craft do blockade planet
        // Test Blockade cache works
        //

        player2.diplomaticRequest(player1, DiplomaticProposal.getWar(), null);
        assertFalse(planet1.isBlockaded());
        
        //
        // Test Crafts.endTurn() flush blockade cache
        //

        MotEngine.instance.getCrafts().endTurn();
        assertTrue(planet1.isBlockaded());
        
    }

    /**
     * 
     */
    public void testBlackHole() throws Exception {
        Logger.logDebug("*******************************  CraftTest.testBlackHole() *************************");

        TestClientProxy proxy = new TestClientProxy(player1);
        
        BlackHole blackHole = BlackHoleFactory.createBlackHole(new Location(100,150));
        
        //
        // Test crafts can move to a black hole and leave again
        //
        
        Craft craft = newCraft(fighterDesign, player1, ss1);
        craft.moveTo(blackHole.getOrbit());
        
        assertEquals(ss1, craft.getOrbit().getStellarObject());
        assertEquals(blackHole, craft.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft.getNavigator().getDestination().getPlanet());
        assertEquals(25, craft.getEngine().getWarpSpeed());
        assertEquals(2, craft.getNavigator().getETA());
        
        craft.endTurn();
        craft.endTurn();

        assertEquals(null, craft.getNavigator());
        assertEquals(blackHole, craft.getOrbit().getStellarObject());
        assertEquals(null, craft.getOrbit().getPlanet());
        assertEquals(blackHole.getLocation(), craft.getLocation());

        craft.moveTo(ss1.getOrbit());
        
        assertEquals(ss1, craft.getNavigator().getDestination().getStellarObject());
        assertEquals(blackHole, craft.getOrbit().getStellarObject());
        assertEquals(null, craft.getOrbit().getPlanet());
        assertEquals(blackHole.getLocation(), craft.getLocation());
        
        craft.endTurn();
        
        assertEquals(ss1, craft.getNavigator().getDestination().getStellarObject());
        assertEquals((Orbit)null, craft.getOrbit());

    }
    
    /**
     * Test that from a work hole a player owns a craft can move to any location in one turn.
     */
    public void testWormHole() throws Exception {
        Logger.logDebug("*******************************  CraftTest.testWormHole() *************************");

        TestClientProxy proxy = new TestClientProxy(player1);
        
        BlackHole blackHole = BlackHoleFactory.createWormHole(new Location(100,150));
        
        //
        // Test crafts can move to a black hole and leave again
        //
        
        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player1, blackHole);
        Craft craft3 = newCraft(fighterDesign, player1, blackHole);
        Craft craft4 = newCraft(fighterDesign, player1, ss1);
        Craft craft5 = newCraft(fighterDesign, player1, blackHole);
        
        craft1.moveTo(ss2.getOrbit());
        craft2.moveTo(ss2.getOrbit());
        
        assertEquals(ss1, craft1.getOrbit().getStellarObject());
        assertEquals(ss2, craft1.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft1.getNavigator().getDestination().getPlanet());
        assertEquals(25, craft1.getEngine().getWarpSpeed());
        assertEquals(2, craft1.getNavigator().getETA());
        
        assertEquals(blackHole, craft2.getOrbit().getStellarObject());
        assertEquals(ss2, craft2.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft2.getNavigator().getDestination().getPlanet());
        assertEquals(25, craft2.getEngine().getWarpSpeed());
        assertEquals(3, craft2.getNavigator().getETA());
        
        //new MooMoveCmd(moo, destination, planet, generateOrbit, owner)
        
        craft1.endTurn();
        craft1.endTurn();
        craft2.endTurn();
        craft2.endTurn();
        craft2.endTurn();

        assertEquals(null, craft1.getNavigator());
        assertEquals(ss2, craft1.getOrbit().getStellarObject());
        assertEquals(null, craft1.getOrbit().getPlanet());
        assertEquals(ss2.getLocation(), craft1.getLocation());

        assertEquals(null, craft2.getNavigator());
        assertEquals(ss2, craft2.getOrbit().getStellarObject());
        assertEquals(null, craft2.getOrbit().getPlanet());
        assertEquals(ss2.getLocation(), craft2.getLocation());

        craft1.transferTo(ss1.getOrbit());
        craft2.transferTo(blackHole.getOrbit());
        newOutpost(planet1, blackHole);
        
        assertEquals(false, player1.getResourceCache().isInfinateRangeAbility());
                
        craft1.moveTo(ss2.getOrbit());
        craft2.moveTo(ss2.getOrbit());
        new MooMoveCmd(craft3, ss2, null, false, player1).run();
        
        assertEquals(ss1, craft1.getOrbit().getStellarObject());
        assertEquals(ss2, craft1.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft1.getNavigator().getDestination().getPlanet());
        assertEquals(2, craft1.getNavigator().getETA());
        
        assertEquals(blackHole, craft2.getOrbit().getStellarObject());
        assertEquals(ss2, craft2.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft2.getNavigator().getDestination().getPlanet());
        assertEquals(1, craft2.getNavigator().getETA());
        
        assertEquals(blackHole, craft3.getOrbit().getStellarObject());
        assertEquals(ss2, craft3.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft3.getNavigator().getDestination().getPlanet());
        assertEquals(1, craft3.getNavigator().getETA());

        assertEquals(3000, ss2.getLastVisitedDate(player1));

        MotEngine.instance.setDate(3005);
        
        craft1.endTurn();
        craft2.endTurn();
        craft3.endTurn();

        assertNull(craft1.getOrbit());
        assertEquals(ss2, craft1.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft1.getNavigator().getDestination().getPlanet());
        assertEquals(125,100, craft1.getLocation());
        assertEquals(1, craft1.getNavigator().getETA());

        assertEquals(null, craft2.getNavigator());
        assertEquals(ss2, craft2.getOrbit().getStellarObject());
        assertEquals(null, craft2.getOrbit().getPlanet());
        assertEquals(ss2.getLocation(), craft2.getLocation());

        assertEquals(null, craft3.getNavigator());
        assertEquals(ss2, craft3.getOrbit().getStellarObject());
        assertEquals(null, craft3.getOrbit().getPlanet());
        assertEquals(ss2.getLocation(), craft3.getLocation());
        
        assertEquals(-1, ss5.getLastVisitedDate(player1));
        assertEquals(3005, ss2.getLastVisitedDate(player1));

         //
        // Test moving out of range using Space Jump Ability
        //

        player1.addTechnology(tech_extendedSpaceJump);
        
        assertEquals(true, player1.getResourceCache().isInfinateRangeAbility());

        assertEquals(150, craft4.getMaxTravelDistance());
        assertTrue(craft4.canMoveTo(ss5));
        assertEquals(150, craft5.getMaxTravelDistance());
        assertTrue(craft5.canMoveTo(ss5));

        craft4.moveTo(ss5.getOrbit());
        craft5.moveTo(ss5.getOrbit());
        
        assertEquals(ss1, craft4.getOrbit().getStellarObject());
        assertEquals(ss5, craft4.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft4.getNavigator().getDestination().getPlanet());
        assertEquals(10, craft4.getNavigator().getETA());
        
        assertEquals(blackHole, craft5.getOrbit().getStellarObject());
        assertEquals(ss5, craft5.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft5.getNavigator().getDestination().getPlanet());
        assertEquals(1, craft5.getNavigator().getETA());

        craft4.endTurn();
        craft5.endTurn();

        assertNull(craft4.getOrbit());
        assertEquals(ss5, craft4.getNavigator().getDestination().getStellarObject());
        assertEquals(null, craft4.getNavigator().getDestination().getPlanet());
        assertEquals(125,100, craft4.getLocation());
        assertEquals(9, craft4.getNavigator().getETA());

        assertEquals(null, craft5.getNavigator());
        assertEquals(ss5, craft5.getOrbit().getStellarObject());
        assertEquals(null, craft5.getOrbit().getPlanet());
        assertEquals(ss5.getLocation(), craft5.getLocation());

        assertEquals(3005, ss5.getLastVisitedDate(player1));
        assertEquals(3005, ss2.getLastVisitedDate(player1));

    }
    
    
    public void testBombardment() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testBombardment() *************************");

        SolarSystem sse = newSolarSystem("SSE", 50, 50);
        Planet planete = newPlanet("SSE-A", sse, player2);
        Planet mine = newPlanet("SSE-B", sse, null, 40, 0.2);
        mine.deploy(planete, new Orbital(tech_spaceStation, null, new Modules(tech_habitatI)));

        MotEngine.getInstance().getRelations().setWar(player1, player2);
        
        CraftDesign cd = new CraftDesign("Bomber", tech_flyer60, tech_fusion25, tech_electronic25, tech_energy10, tech_ancientWeapon);
        Craft c1 = newCraft(cd, player1, sse);

        planete.setSize(100);
        planete.setPopulation(50);
        planete.setQtyFactories(10);
        planete.setTeraforming(1.0);
        planete.setQtyGroundDefense(10);

        clearEvents();
        c1.bombard(planete);

        assertEquals(player2, planete.getOrigin());
        assertEquals(player2, planete.getOwner());
        assertEquals(100, planete.getSize());
        assertEquals(105.0, planete.getMaxPopulation(), 1.0);
        assertEquals(25.0, planete.getPopulation(), 1.0);
        assertEquals(0.0, planete.getQtyFactories(), 0.1);
        assertEquals(1.0, planete.getTeraforming(), 0.01);
        assertEquals(0, planete.getQtyGroundDefense());
        assertEquals(5, planete.getMiningStationPopulation());
        assertEquals(mine, planete.getMiningStation().getPlanet());
        assertEquals(3000, ((CrisisEmergency)planete.getCrisis()).getDateStarted());

        assertEquals(player2, mine.getOwner());
        assertEquals(tech_spaceStation, mine.getOrbital().getHullTech());

        PlanetBombardedEvent pbe = (PlanetBombardedEvent) getEvent(PlanetBombardedEvent.class);
        
        assertNotNull(pbe);
        assertEquals("SSE-A", pbe.getTarget());
        assertEquals(10.0, pbe.getFactoriesDestroyed(), 0.1);
        assertEquals(25.0, pbe.getPopulationDestroyed(), 0.1);

        clearEvents();
        c1.setHasMoved(false);
        c1.bombard(planete);

        assertEquals(null, planete.getOrigin());
        assertEquals(null, planete.getOwner());
        assertEquals(100, planete.getSize());
        assertEquals(0.0, planete.getPopulation(), 1.0);
        assertEquals(0.0, planete.getQtyFactories(), 0.1);
        assertEquals(1.0, planete.getTeraforming(), 0.01);
        assertEquals(0, planete.getQtyGroundDefense());
        assertEquals(0, planete.getMiningStationPopulation());
        assertEquals(null, planete.getMiningStation());

        assertEquals(null, mine.getOwner());
        assertEquals(null, mine.getOrbital());

        pbe = (PlanetBombardedEvent) getEvent(PlanetBombardedEvent.class);
        
        assertNotNull(pbe);
        assertEquals("SSE-A", pbe.getTarget());
        assertEquals(0.0, pbe.getFactoriesDestroyed(), 0.1);
        assertEquals(25.0, pbe.getPopulationDestroyed(), 0.1);

    }

    public void testDeploymentOfPlagueCrisis() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testDeploymentOfPlagueCrisis() *************************");

        SolarSystem sse = newSolarSystem("SSE", 50, 50);
        Planet planete = newPlanet("SSE-A", sse, player2);

        MotEngine.instance.getRelations().setWar(player1, player2);
        MotEngine.instance.setDate(3010);
        player1.getStratagist().getEventManager().clearEvents();
        player2.getStratagist().getEventManager().clearEvents();
        
        CraftDesign cd = new CraftDesign("Paguer", tech_flyer60, tech_fusion25, tech_electronic25, tech_plague);
        Craft c1 = newCraft(fighterDesign, player1, sse);
        Craft c2 = newCraft(cd, player1, sse);

        try {
        	new MooDeployCmd(c1, sse, planete, player1).run();
        	fail();
        } catch (ValidationError e) {
        	assertEquals(L.THIS_DOES_NOT_HAVE_ANYTHING_TO_DEPLOY, e.getMessage());
        }

        clearEvents();
        new MooDeployCmd(c2, sse, planete, player1).run();

        PlanetCrisisEvent event = (PlanetCrisisEvent) getEvent(PlanetCrisisEvent.class);
        assertEquals(3010, ((CrisisPlague)planete.getCrisis()).getDateStarted());
        assertEquals("SSE-A", event.getTarget());
        assertEquals(1, player2.getStratagist().getEventManager().getEventsAsArray().length);
        assertEquals(player1, ((StrategistEventUsedBiologicalWeapons)player2.getStratagist().getEventManager().getEventsAsArray()[0]).getPlayer());
        assertNull(player1.getStratagist().getEventManagerNotNull().getEventsAsArray());

        MotEngine.instance.setDate(3011);
        planete.setCrisis(null);
        Fleet fleet = FleetFactory.create(c1, c2);
        clearEvents();
        new MooDeployCmd(fleet, sse, planete, player1).run();

        event = (PlanetCrisisEvent) getEvent(PlanetCrisisEvent.class);
        assertEquals(3011, ((CrisisPlague)planete.getCrisis()).getDateStarted());
        assertEquals("SSE-A", event.getTarget());
        assertEquals(1, player2.getStratagist().getEventManager().getEventsAsArray().length);
        assertEquals(player1, ((StrategistEventUsedBiologicalWeapons)player2.getStratagist().getEventManager().getEventsAsArray()[0]).getPlayer());

        c1.setHasMoved(false);
        try {
        	new MooDeployCmd(c1, sse, planete, player1).run();
        	fail();
        } catch (ValidationError e) {
        	assertEquals(L.INVALID_COMMAND_CANNOT_DEPLOY_TO_A_PLANET_IN_CRISIS, e.getMessage());
        }

    }

    public void testDeploymentOfPsiCrisis() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testDeploymentOfPsiCrisis() *************************");

        SolarSystem sse = newSolarSystem("SSE", 50, 50);
        Planet planete = newPlanet("SSE-A", sse, player2);

        MotEngine.instance.getRelations().setWar(player1, player2);
        MotEngine.instance.setDate(3010);
        player1.getStratagist().getEventManager().clearEvents();
        player2.getStratagist().getEventManager().clearEvents();

        CraftDesign cd = new CraftDesign("Psier", tech_flyer60, tech_fusion25, tech_electronic25, tech_psiBlast);
        Craft c1 = newCraft(fighterDesign, player1, sse);
        Craft c2 = newCraft(cd, player1, sse);

        try {
        	new MooDeployCmd(c1, sse, planete, player1).run();
        	fail();
        } catch (ValidationError e) {
        	// success
        }

        clearEvents();
        new MooDeployCmd(c2, sse, planete, player1).run();

        PlanetCrisisEvent event = (PlanetCrisisEvent) getEvent(PlanetCrisisEvent.class);
        assertEquals(3010, ((CrisisPsi)planete.getCrisis()).getDateStarted());
        assertEquals("SSE-A", event.getTarget());
        assertEquals(player1, ((StrategistEventUsedBiologicalWeapons)player2.getStratagist().getEventManager().getEventsAsArray()[0]).getPlayer());

    }

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
		eventList.add(e);
    }

	public Player getOwner() {
        return player1;
    }

    protected void tearDown() {
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}