/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import java.util.ArrayList;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.client.deligates.ComputerPlayerDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.event.MotEventConsumer;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.MotServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.TestBattleWindowConfig;
import mot.common.event.LinkedEventBase;
import mot.common.event.MotEvent;
import mot.common.setup.RaceOptions;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * This test harness tests that players can use spies to steal technology
 * sabotage planets and hide. It also tests that spies stationed at home
 * find an kill enemy spies.
 */

public class DeligateBattleMPTest extends TestBase {

    private static final String REMOTE_SERVER_URL = "http://localhost:7999/main";

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(mot.test.DeligateBattleMPTest.class);
        return suite;
    }

    public DeligateBattleMPTest(String name) {
        super(name);
    }
    
    private UserPlayer p1, p2;
    private Player p3;
    private SolarSystem ss1, ss2, ss3, ss4;
    private Planet h1, h2, h3, h4, h3b;

    private MotEventConsumer eventConsumer1, eventConsumer2; 
    private MotServerProxy serverProxy1, serverProxy2;
    private MainWindow mainWindow1, mainWindow2;
    
    private UserPlayerDelegate player1, player2;
    private ComputerPlayerDelegate player3;
    
	private CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();
    
    protected void setUp() throws ApplicationError, ValidationError {
        MotEngine.newInstance();
        DeligateCacheManager.flush();
        setupConfigs();
        mainWindow1 = MainWindow.__createTestMainWindow(new TestBattleWindowConfig(this));
        mainWindow2 = MainWindow.__createTestMainWindow(new TestBattleWindowConfig(this));
        eventConsumer1 = new TestMotEventConsumer(); 
        eventConsumer2 = new TestMotEventConsumer(); 
       	MotServerProxy serverProxy1 = new MotServerProxy(true, mainWindow1, eventConsumer1);
       	MotServerProxy serverProxy2 = new MotServerProxy(true, mainWindow2, eventConsumer2);
		new ProxyStore(serverProxy1);
       	// Events come through server connection
        //MotEngine.getInstance().getProxyManager().add(this);
    }
    
    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }
    
    public void createMap() throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        RaceOptions options = mot.common.setup.OptionsFactory.getRaceOptions(playerNames[0]);
        p1 = newUserPlayer(playerNames[0], blue, null, options);
        p2 = newUserPlayer(playerNames[1], green, null, options);

        ss1 = newSolarSystem("SS1", 100, 100);
        h1 = newPlanet("SS1-A", ss1, p1);

        ss2 = newSolarSystem("SS2", 200, 100);
        h2 = newPlanet("SS2-A", ss2, p2);
        
        ss3 = newSolarSystem("SS3", 150, 100);
        h3 = newPlanet("SS3-A", ss3, null, 100, 0.9);
        
        serverProxy1.register(playerNames[0], true, callbackSync, null);
        serverProxy1._processCommandResponse();
        
        serverProxy2.register(playerNames[0], true, callbackSync, null);
        serverProxy2._processCommandResponse();
        
        player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Humans");
        player2 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Bears");
        
        assertEquals("Player Human is correct", "Humans", player1.getName());
        assertEquals("Player Bears is correct", "Bears", player2.getName());

        PlayerDelegate [] players = DeligateCacheManager.getPlayers();
        assertEquals("Have two players", 2, players.length);

    }



    public void testNextCraftToMove() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testNextCraftToMove() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h1.setWeapons(new Weapons(tech_missile6, tech_missile6));
        Craft c1 = newCraft(fighterDesign, p2, ss1);
        Craft c2 = newCraft(fighterDesign, p1, ss1);

        serverProxy1.updateDelegates(null);
        serverProxy1._processCommandResponse();
        
        StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
        PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter2");
    }
    
     
	
	class TestMotEventConsumer implements MotEventConsumer {

	    private ArrayList events = new ArrayList();

		@Override
		public void process(MotEvent e, LinkedEventBase parent) throws ApplicationError, ValidationError {
	        Logger.logDebug("DeligateTest.notify() EVENT "+e.toString());
	        events.add(e);
		}
		
		public void clearEvents() {
			Logger.logDebug("DeligateTest.notify() Clear Events ");
			events.clear();
		}
		
		public MotEvent getEvent(Class c) {
			for (int i=events.size()-1; i>=0; i--) {
				MotEvent e = (MotEvent) events.get(i);
				if (e.getClass() == c) {
					return e;
				}
			}
			return null;
		}
		
		public MotEvent getEventFromBack(Class c, int index) {
			for (int i=events.size()-1; i>=0; i--) {
				MotEvent e = (MotEvent) events.get(i);
				if (e.getClass() == c) {
					if (index == 0) {
						return e;
					} else {
						index--;
					}
				}
			}
			return null;
		}
		
		public MotEvent getEventFromFront(Class c, int index) {
			for (int i=0; i<events.size(); i++) {
				MotEvent e = (MotEvent) events.get(i);
				if (e.getClass() == c) {
					if (index == 0) {
						return e;
					} else {
						index--;
					}
				}
			}
			return null;
		}
		
		public int getQtyEvent(Class c) {
			int qty = 0;
			for (int i=events.size()-1; i>=0; i--) {
				MotEvent e = (MotEvent) events.get(i);
				if (e.getClass() == c) {
					qty++;
				}
			}
			return qty;
		}
	}


    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}