/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.client.deligates.BattleAutomatonDelegate;
import mot.client.deligates.BattleCraftDelegate;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.BattleDroidMineDelegate;
import mot.client.deligates.BattleMineDelegate;
import mot.client.deligates.BattlePlanetDelegate;
import mot.client.deligates.ComputerPlayerDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.ImageObjectDelegate;
import mot.client.deligates.MissilesDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.deligates.mot.MotBattleCraftDelegate;
import mot.client.deligates.mot.MotBlackHoleDelegate;
import mot.client.deligates.mot.MotPlanetDelegate;
import mot.client.deligates.mot.MotSolarSystemDelegate;
import mot.client.event.MotEventConsumer;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.MotServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.TestBattleWindowConfig;
import mot.common.bean.AnimatedBattleAreaDetails;
import mot.common.bean.AsteroidFieldDetails;
import mot.common.bean.Decorator;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.ObjectShift;
import mot.common.bean.ObjectTurn;
import mot.common.bean.SpaceDistortionDetails;
import mot.common.bean.WreckageDetails;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.event.BattleCraftLeaveEvent;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.event.BattleCraftTeleportEvent;
import mot.common.event.BattleDroidMineLaunchEvent;
import mot.common.event.BattleDroidMineMoveEvent;
import mot.common.event.BattleEndEvent;
import mot.common.event.BattleMissileLaunchEvent;
import mot.common.event.BattleMissileMoveEvent;
import mot.common.event.BattleNewTurnEvent;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.LinkedEventBase;
import mot.common.event.MotEvent;
import mot.common.event.NewBattleEvent;
import mot.common.event.RoutedEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.RaceOptions;
import mot.common.technology.Biotechnology;
import mot.common.technology.Modules;
import mot.common.technology.Shields;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.vo.BattleStateVO;
import mot.common.vo.PlanetVO;
import mot.engine.MotEngine;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleCraftControllerBerserker;
import mot.engine.battles.BattleCraftControllerCowardice;
import mot.engine.battles.BattleCraftControllerMindControl;
import mot.engine.battles.BattleMine;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.DroidMine;
import mot.engine.battles.Missiles;
import mot.engine.battles.triggers.CraftTriggerBase;
import mot.engine.battles.triggers.DamageBot;
import mot.engine.battles.triggers.Plague;
import mot.engine.common.CompletedServerCallbackSyncroniser;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.event.BattleCraftNewEF;
import mot.engine.event.BattleCraftUpdateEF;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 * This test harness tests that players can use spies to steal technology
 * sabotage planets and hide. It also tests that spies stationed at home
 * find an kill enemy spies.
 */

public class DeligateBattleTest extends TestBase implements MotEventConsumer, ClientProxy {

    private static final String REMOTE_SERVER_URL = "http://localhost:7999/main";

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(mot.test.DeligateBattleTest.class);
        return suite;
    }

    public DeligateBattleTest(String name) {
        super(name);
    }
    
    private UserPlayer p1;
    private Player p2, p3;
    private SolarSystem ss1, ss2, ss3, ss4;
    private Planet h1, h2, h3, h4, h3b;

    private MotServerProxy serverProxy;
    private UserPlayerDelegate player1, player4;
    private ComputerPlayerDelegate player2, player3;
    
	private CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();
    
    protected void setUp() throws ApplicationError, ValidationError {
        MotEngine.newInstance();
        DeligateCacheManager.flush();
        setupConfigs();
        MainWindow.__createTestMainWindow(new TestBattleWindowConfig(this));
       	new ProxyStore(new MotServerProxy(true, MainWindow.getExestingInstance(), this));
       	// Events come through server connection
        //MotEngine.getInstance().getProxyManager().add(this);
    }
    
    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }
    
    public void createMap() throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        RaceOptions options = mot.common.setup.OptionsFactory.getRaceOptions(playerNames[0]);
        p1 = newUserPlayer(playerNames[0], blue, null, options);
        p2 = newPlayer(playerNames[1], green, options, playerNames);

        MainWindow.getExestingInstance().__setWindowOwner(playerNames[0]);
        
        ss1 = newSolarSystem("SS1", 100, 100);
        h1 = newPlanet("SS1-A", ss1, p1);

        ss2 = newSolarSystem("SS2", 200, 100);
        h2 = newPlanet("SS2-A", ss2, p2);
        
        ss3 = newSolarSystem("SS3", 150, 100);
        h3 = newPlanet("SS3-A", ss3, null, 100, 0.9);
        
        // Events for p1 will come through the server connection (MotEventHandler interface)
        // while events for p2 will come directly from the server (ClientProxy interface).
        owner = p2;

        serverProxy = (MotServerProxy) ProxyStore.getInstance();
        CompletedServerCallbackSyncroniser callback = new CompletedServerCallbackSyncroniser();
        serverProxy.register(playerNames[0], true, callback, null);
        callback.waitTillCompleteAndProcessCommandResponse(serverProxy);
        
        player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Humans");
        player2 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Bears");
        
        assertEquals("Player Human is correct", "Humans", player1.getName());
        assertEquals("Player Bears is correct", "Bears", player2.getName());

        PlayerDelegate [] players = DeligateCacheManager.getPlayers();
        assertEquals("Have two players", 2, players.length);

    }



    /**
     * Test battle planet can attack computer crafts;
     *  - Test BattleDelegate, BattleCraftDelegate, BattlePlanetDelegate
     *  - Test End Turn Command on planet.
     *  - Test computer crafts surrenders.
     *  - Test MissileLaunchEvent and MissileDelegate
     */
    public void testComputerCraftAttackUserPlanet() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testComputerCraftAttackUserPlanet() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h1.setWeapons(new Weapons(tech_missile6, tech_missile6));
        Craft c1 = newCraft(fighterDesign, p2, ss1);
        Craft c2 = newCraft(fighterDesign, p1, ss1);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
        PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter2");
        
        try {
            serverProxy.craftAttackCmd(craft1, planet1, null, new TestErrorHandler());
            serverProxy._processCommandResponse();
            fail("This does not belong to player 1");
        } catch (ValidationError e) { }

        //
        // Test Computer Player craft attacking User Player planet produces a NewBattleEvent
        //
        
        c1.attack(h1);

        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        callbackSync.waitTillComplete();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        assertEquals(b.getName(), battle.getName());
        assertEquals(b.getTurn(), battle.getTurn());
        
        BattleCraft bc1 = b.getCrafts()[0];
        BattlePlanet bp = b.getPlanet();

        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toBattleWindow(battle);

        //
        // Test battle delegate works.
        //
        
        assertEquals(2, battle.getCrafts().length);
        BattleCraftDelegate battleCraft1 = battle.getCrafts()[0];
        BattleCraftDelegate battleCraft2 = battle.getCrafts()[1];
        assertEquals(craft1.getName(), battleCraft1.getName());
        assertTrue(battle.containsStartingPlayer(player1));
        assertEquals("SS1", battle.getOrbit().getStellarObject());
        
        ImageObjectDelegate iod = battle.getDelegateCache().getObjectAt(bc1.getLocation(), false);
        assertEquals(battleCraft1.getName(), iod.getName());
        
        //
        // Test battle craft delegate works.
        //
        
        assertEquals(bc1.getLocation(), battleCraft1.getLocation());
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(c1.getName(), battleCraft1.getCraft().getName());
        assertEquals(battle.getName(), battleCraft1.getBattleName());
        assertEquals(bc1.getWeapons()[0].getQty(), battleCraft1.getWeapons()[0].getQty());
        assertEquals(bc1.getWeapons()[0].getWeapon(), battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(bc1.getWeapons()[0].getQtyFired(), battleCraft1.getWeapons()[0].getQtyFired());

        //
        // Test battle planet delegate work.
        //
        
        BattlePlanetDelegate battlePlanet = battle.getPlanet(); 
        assertEquals("SS1-A", battlePlanet.getName());
        assertEquals(bp.getLocation(), battlePlanet.getLocation());
        assertEquals(tech_missile6, battlePlanet.getWeapons()[0].getWeapon());
        assertEquals(2, battlePlanet.getWeapons()[0].getQty());
        assertEquals(0, battlePlanet.getWeapons()[0].getQtyFired());
        
        //
        // Test Computer Player moving craft1 has state updated in client delegate
        // Test missile also moves and has client state updated
        // Test cloak changes visibility and image used to display craft changes
        //

        assertTrue(battleCraft1.isVisible());
        assertEquals("images/crafts/hull_flyer_green.png", battleCraft1.getImage());

        bc1.setMoveRemaining(1.0);
        bc1.rechargeShields();
        bc1.setLocation(new Location(900,500));
        bc1.setDirection(270);
        c1.setShields(new Shields(tech_cloak150));
        bc1.setShieldPower(0, bc1.getShields()[0].getPower()*0.9);
        bc1.getCraft().setDamage(bc1.getCraft().getMaxDamage()*0.2);
        bc1.setMovementDamage((int)(bc1.getSpeed(false)*0.7));
        bc1.setPsiDamage(Biotechnology.PSI_MAX_RETREAT_RATING*0.6);
        bc1.setCrew((int)(bc1.getCraft().getHull().getCrew()*0.8));
        
        clearEvents();
        bc1.moveTo(new LocationAndDirection(880,500,270));
        bc1.endTurn();
        serverProxy._processQueuedEvents();
        
        assertTrue(battleCraft1.isSelectableBy(player2));
        assertFalse(battleCraft1.isSelectableBy(player1));
        assertTrue(battleCraft1.isOwnedBy(player2));
        assertFalse(battleCraft1.isOwnedBy(player1));

        assertEquals(880,500, battleCraft1.getLocation());
        assertEquals(270, battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(0.9, battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(0.2, battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(0.7, battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(0.6, battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(0.8, battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(c1.getName(), battleCraft1.getCraft().getName());
        assertEquals(battle.getName(), battleCraft1.getBattleName());
        assertEquals(1, battleCraft1.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(1, battleCraft1.getWeapons()[0].getQtyFired());
        
        assertFalse(battleCraft1.isVisible());
        assertEquals("images/crafts/hull_flyer_cloaked_green.png", battleCraft1.getImage());
        
        BattleCraftMoveEvent cme = (BattleCraftMoveEvent) getEvent(BattleCraftMoveEvent.class);
        assertEquals(bc1.getName(), cme.getName());
        
        assertEquals(battleCraft2, battle.getNextObjectToMove());

        //
        // End BattleCraft2 Turn discharges weapons and updates the weapon state
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleEndTurnCmd - BattleCraft2 dischares weapons");
        Logger.logDebug("");
        
        assertEquals(1, battleCraft2.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft2.getWeapons()[0].getWeapon());
        assertEquals(0, battleCraft2.getWeapons()[0].getQtyFired());
        
        serverProxy.battleEndTurnCmd(battle, battleCraft2, true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();

        assertEquals(1, battleCraft2.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft2.getWeapons()[0].getWeapon());
        assertEquals(1, battleCraft2.getWeapons()[0].getQtyFired());

        assertEquals(battlePlanet, battle.getNextObjectToMove());
        
        assertEquals(tech_missile6, battlePlanet.getWeapons()[0].getWeapon());
        assertEquals(0, battlePlanet.getWeapons()[0].getQtyFired());
        assertEquals(2, battlePlanet.getWeapons()[0].getQty());
        
        
        
        //
        // Planet fires missiles a computer craft - creates new Missile Delegate
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleAttackCmd - Planet fires missiles a computer craft");
        Logger.logDebug("");
        
        c1.setShields(null);
        bc1.setLocation(new Location(250,500));
        bc1.setDirection(270);
        
        serverProxy.battleAttackCmd(battle, battlePlanet, battleCraft1, false, false, false, null, tech_missile6, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        
        BattleMissileLaunchEvent mle = (BattleMissileLaunchEvent) getEvent(BattleMissileLaunchEvent.class);
        assertNotNull(mle);
        
        Missiles m = (Missiles) b.getAutomatons()[0];
        BattleAutomatonDelegate [] automitons = battle.getAutomatons();
        MissilesDelegate missile = (MissilesDelegate) automitons[0];
        
        assertEquals(1, automitons.length);
        assertEquals(1, battle.getAutomatons().length);
        assertEquals(m.getName(), missile.getName());
        assertEquals(m.getLocation(), missile.getLocation());
        assertEquals(m.getDirection(), missile.getDirection());
        
        assertEquals(tech_missile6, battlePlanet.getWeapons()[0].getWeapon());
        assertEquals(2, battlePlanet.getWeapons()[0].getQty());
        assertEquals(1, battlePlanet.getWeapons()[0].getQtyFired());
        
        iod = battle.getDelegateCache().getObjectAt(bp.getLocation(), true);
        assertEquals(battlePlanet.getName(), iod.getName());

        iod = battle.getDelegateCache().getObjectAt(bp.getLocation(), false);
        assertEquals(missile.getName(), iod.getName());

        
        //
        // End Planet Turn discharges weapons and updates the weapon state - Starts New Turn
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleEndTurnCmd - Planet dischares weapons");
        Logger.logDebug("");
        
        clearEvents();
        serverProxy.battleEndTurnCmd(battle, battlePlanet, true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();
        callbackSync.waitTillComplete();
        serverProxy._processQueuedEvents();

        BattleNewTurnEvent nte = (BattleNewTurnEvent) getEvent(BattleNewTurnEvent.class);
        assertEquals(2, ((BattleStateVO)nte.getType(BattleStateVO.class)).getTurn());
        
        assertEquals(battleCraft2, battle.getNextObjectToMove());

        assertEquals(tech_missile6, battlePlanet.getWeapons()[0].getWeapon());
        assertEquals(2, battlePlanet.getWeapons()[0].getQty());
        assertEquals(0, battlePlanet.getWeapons()[0].getQtyFired());

        assertEquals(tech_massDriver1, battleCraft2.getWeapons()[0].getWeapon());
        assertEquals(1, battleCraft2.getWeapons()[0].getQty());
        assertEquals(0, battleCraft2.getWeapons()[0].getQtyFired());

        BattleMissileMoveEvent mme = (BattleMissileMoveEvent) getEvent(BattleMissileMoveEvent.class);
        assertEquals(m.getName(), mme.getMissile());

        assertEquals(bc1.getLocation(), battleCraft1.getLocation());
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getMoveRemaining(), battleCraft1.getMoveRemainingAsPercent());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(c1.getName(), battleCraft1.getCraft().getName());
        assertEquals(battle.getName(), battleCraft1.getBattleName());
        assertEquals(1, battleCraft1.getWeapons().length);
        assertEquals(0, battleCraft1.getWeapons()[0].getQtyFired());

        assertEquals(bp.getLocation(), battlePlanet.getLocation());
        assertEquals(bp.getDirection(), battlePlanet.getDirection());
        assertEquals(bp.getShieldAsPercent(), battlePlanet.getState().getShieldPercent(), 0.01);
        assertEquals(bp.getDamageAsPercent(), battlePlanet.getState().getDamagePercent(), 0.01);
        assertEquals(bp.getMovementDamageAsPercent(), battlePlanet.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bp.getPsiDamageAsPercent(), battlePlanet.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bp.getCrewAsPercent(), battlePlanet.getState().getCrewPercent(), 0.0);
        assertEquals(bp.getDirection(), battlePlanet.getDirection());
        assertEquals(battle.getName(), battlePlanet.getBattleName());
        assertEquals(1, battlePlanet.getWeapons().length);
        assertEquals(0, battlePlanet.getWeapons()[0].getQtyFired());

        assertEquals(m.getName(), missile.getName());
        assertEquals(m.getLocation(), missile.getLocation());

        assertEquals(tech_missile6, battlePlanet.getWeapons()[0].getWeapon());
        assertEquals(2, battlePlanet.getWeapons()[0].getQty());
        assertEquals(0, battlePlanet.getWeapons()[0].getQtyFired());

        assertNull(battle.getDelegateCache().getAutomatons());
        missile = (MissilesDelegate) battle.getDelegateCache().getBattleAutomaton(missile.getName());
        assertEquals(true, missile.getState().isDestroyed());
        
        //
        // Test that if craft1 surrenders - its image changes
        //
        
        bc1.surrender(p1, true);
        serverProxy._processQueuedEvents();
        
        assertEquals("images/crafts/hull_flyer_blue.png", battleCraft1.getImage());
    }
    
    /**
     * Test battle planet can attack computer crafts;
     */
    public void testComputerCraftDamageUserPlanet() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testComputerCraftDamageUserPlanet() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h1.setWeapons(new Weapons(tech_missile6, tech_missile6));
        Craft c1 = newCraft(new CraftDesign("TestFighter", tech_flyer100, tech_fusion40, tech_energy20, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4), p2, ss1);
        Craft c2 = newCraft(fighterDesign, p1, ss1);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
        PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter2");
        
        c1.attack(h1);

        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft bc1 = b.getCrafts()[0];
        BattleCraft bc2 = b.getCrafts()[1];
        BattlePlanet bp = b.getPlanet();

        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toBattleWindow(battle);

        BattleCraftDelegate[] bca = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[2], "TestFighter1", "TestFighter2");
        BattleCraftDelegate battleCraft1 = battle.getCrafts()[0];
        BattleCraftDelegate battleCraft2 = battle.getCrafts()[1];
        BattlePlanetDelegate battlePlanet = battle.getPlanet();
        
        //
        // Computer craft attacks user planet and damages it
        //

        Logger.logDebug("");
        Logger.logDebug("Computer craft attacks user planet and damages it");
        Logger.logDebug("");

        assertEquals(tech_missile6, battlePlanet.getWeapons()[0].getWeapon());
        assertEquals(0, battlePlanet.getWeapons()[0].getQtyFired());
        assertEquals(2, battlePlanet.getWeapons()[0].getQty());

        bc2.setLocation(new Location(900,500));
        bc1.setLocation(new Location(200,500));
        bc1.setDirection(270);
        bc1.setMoveRemaining(1.0);
        bc1.rechargeWeapons();
        
        serverProxy.battleEndTurnCmd(battle, battleCraft2, true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();

        BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) getEvent(BattleWeaponFireEvent.class);
        assertEquals("Weaponsx1.0 destroyed", wfe.getTargetDescription());
        
        assertEquals(tech_fusionBeam4, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(10, battleCraft1.getWeapons()[0].getQty());
        assertEquals(10, battleCraft1.getWeapons()[0].getQtyFired());

        assertEquals(tech_massDriver1, battleCraft2.getWeapons()[0].getWeapon());
        assertEquals(1, battleCraft2.getWeapons()[0].getQty());
        assertEquals(1, battleCraft2.getWeapons()[0].getQtyFired());

        assertEquals(battlePlanet, battle.getNextObjectToMove());
        
        assertEquals(tech_missile6, battlePlanet.getWeapons()[0].getWeapon());
        assertEquals(0, battlePlanet.getWeapons()[0].getQtyFired());
        assertEquals(1, battlePlanet.getWeapons()[0].getQty());
        
        //
        // Computer craft attacks user planet and damages it (NEW TURN)
        //

        Logger.logDebug("");
        Logger.logDebug("Computer craft attacks user planet and destroys it");
        Logger.logDebug("");

        bc1.setMoveRemaining(1.0);
        bc1.rechargeWeapons();
        
        clearEvents();
        serverProxy.battleEndTurnCmd(battle, battlePlanet, true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();

        BattleNewTurnEvent nte = (BattleNewTurnEvent) getEvent(BattleNewTurnEvent.class);
        assertNotNull(nte);
        
        wfe = (BattleWeaponFireEvent) getEvent(BattleWeaponFireEvent.class);
        assertEquals("32.6 [Population 0.26 killed ]", wfe.getTargetDescription());

        assertEquals(tech_fusionBeam4, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(10, battleCraft1.getWeapons()[0].getQty());
        assertEquals(0, battleCraft1.getWeapons()[0].getQtyFired());

        assertEquals(tech_massDriver1, battleCraft2.getWeapons()[0].getWeapon());
        assertEquals(1, battleCraft2.getWeapons()[0].getQty());
        assertEquals(0, battleCraft2.getWeapons()[0].getQtyFired());

        assertEquals(battleCraft2, battle.getNextObjectToMove());
        
        assertEquals(null, battlePlanet.getWeapons());
        
    }
    
    /**
     * 
     */
    public void testWinningBattleEndsBlockade() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testWinningBattleEndsBlockade() *************************");
        createMap();

        Planet h1b = newPlanet("SS1-B", ss1, p1);

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h1.setWeapons(new Weapons(tech_missile6, tech_missile6));
        Craft c1 = newCraft(fighterDesign, p2, h1);
        c1.moveTo(new Orbit(h2));
        h1.flushBlockadeCache();

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
        PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS1-B");
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");

        assertEquals(tech_missile6, tech_missile6, planet1.getWeapons());
        assertEquals(true, h1.isBlockaded());
        assertEquals(true, h1b.isBlockaded());
        assertEquals(true, planet1.isBlockaded());
        assertEquals(true, planet2.isBlockaded());

        //
        // Test Computer Player craft attacking User Player planet produces a NewBattleEvent
        //
        
        c1.attack(h1);

        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        callbackSync.waitTillComplete();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        serverProxy.battleAutomateCmd(battle, callbackSync, null);
        serverProxy._processCommandResponse();
        callbackSync.waitTillComplete();
        
        assertTrue(battle.hasEnded());
        
        BattleEndEvent ebe = (BattleEndEvent) getEvent(BattleEndEvent.class);
        assertEquals(false, ((PlanetVO)ebe.getState(planet1.getName())).isBlockaded());
        assertEquals(false, ((PlanetVO)ebe.getState(planet2.getName())).isBlockaded());
        
        assertEquals(true, battle.isWinner(player1));
        assertEquals(c1.getName(), ebe.getCraftDestroyd());
        assertEquals(false, planet1.isBlockaded());
        assertEquals(false, planet2.isBlockaded());
    }
    

    /**
     * Tests specific bug where destroying planet weapon should end the battle
     * When there is next craft to move is null - battle should end
     */
    public void testComputerCraftDamageUserPlanet2() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testComputerCraftDamageUserPlanet2() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h1.setWeapons(new Weapons(tech_missile6));
        h1.setShields(new Shields(tech_energy10));
        CraftDesign cd = new CraftDesign("FusionFighter", tech_flyer100, tech_fusion40, tech_energy20, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4);
		Craft c1 = newCraft(cd, p2, ss1);
		Craft c2 = newCraft(cd, p2, ss1);
        Craft c3 = newCraft(fighterDesign, p1, ss1);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        c1.attack(h1);
        c2.attack(h1);

        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft[] crafts = (BattleCraft[]) order(b.getCrafts(), new BattleCraft[3], "FusionFighter1", "FusionFighter2", "TestFighter1");
		BattleCraft bc1 = crafts[0];
        BattleCraft bc2 = crafts[1];
        BattleCraft bc3 = crafts[2];
        BattlePlanet bp = b.getPlanet();

        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toBattleWindow(battle);

        BattleCraftDelegate[] bca = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[3], "FusionFighter1", "FusionFighter2", "TestFighter1");
        BattleCraftDelegate battleCraft1 = battle.getCrafts()[0];
        BattleCraftDelegate battleCraft2 = battle.getCrafts()[1];
        BattleCraftDelegate battleCraft3 = battle.getCrafts()[2];
        BattlePlanetDelegate battlePlanet = battle.getPlanet();
        
        //
        // BattleCraft TestFighter2 is not operating and should not stop battle ending
        //
        bc3.dischargeWeapons();
        bc3.setMoveRemaining(0.0);
        bc3.setCrew(0);
        bc3.setLocation(new Location(100,100));
        assertEquals(false, bc3.isOperating());
        
        b.sendStateUpdateEvents(p1, true, false, false);
        serverProxy._processQueuedEvents();

        //
        // Computer craft attacks user planet and damages it
        //

        clearEvents();
        
        Logger.logDebug("");
        Logger.logDebug("Computer craft attacks user planet and destroys it");
        Logger.logDebug("");

        assertEquals(battlePlanet, battle.getNextObjectToMove());
        
        assertEquals(tech_missile6, battlePlanet.getWeapons()[0].getWeapon());
        assertEquals(0, battlePlanet.getWeapons()[0].getQtyFired());
        assertEquals(1, battlePlanet.getWeapons()[0].getQty());

        bc1.setLocation(new Location(200,500));
        bc1.setDirection(270);
        bc1.setMoveRemaining(1.0);
        bc1.rechargeWeapons();
        
        serverProxy.battleEndTurnCmd(battle, battlePlanet, true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();

        BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) getEvent(BattleWeaponFireEvent.class);
        assertEquals("Shieldsx0 destroyed", wfe.getTargetDescription());
        
        BattleEndEvent bee = (BattleEndEvent) getEvent(BattleEndEvent.class);
        assertEquals(b.getName(), bee.getBattle());

        assertEquals(tech_fusionBeam4, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(6, battleCraft1.getWeapons()[0].getQty());
        assertEquals(6, battleCraft1.getWeapons()[0].getQtyFired());

        assertNull(battle.getNextObjectToMove());
        assertNull(battlePlanet.getWeapons());
        
        assertTrue(b.hasEnded());
        
        assertEquals(0, h1.getWeapons().getQuantity());
        assertEquals(1, h1.getShields().getQuantity());

    }
    



    /**
     * Test user craft can attack computer planets and crafts;
     */
    public void testUserCraftAttackComputerPlanetAndComputerCraft() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testUserCraftAttackComputerPlanetAndComputerCraft() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h2.setWeapons(new Weapons(tech_massDriver1));

        StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

        Craft c1 = newCraft(new CraftDesign("TestAttacker1", tech_flyer100, tech_fusion400, tech_energy20, tech_missile6, tech_antiMissileBeam, tech_massDriver1, tech_massDriver1, tech_tacticalStationI), p1, ss2);
        Craft c2 = newCraft(new CraftDesign("TestFighter2", tech_flyer30, tech_fusion25, tech_massDriver1, tech_missile6), p2, ss2);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestAttacker11");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter21");

        assertFalse(craft1.hasMoved());

        serverProxy.craftAttackCmd(craft1, planet2, null, null);
        serverProxy._processCommandResponse();
        
        assertTrue(c1.hasMoved());
        assertTrue(craft1.hasMoved());
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        assertEquals(b.getName(), battle.getName());
        assertEquals(b.getTurn(), battle.getTurn());
        
        BattleCraft bc1 = b.getCrafts()[0];
        BattleCraft bc2 = b.getCrafts()[1];
        BattlePlanet bp = b.getPlanet();

        BattleCraftDelegate[] bca = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[2], "TestAttacker11", "TestFighter21");
        BattleCraftDelegate battleCraft1 = bca[0];
        BattleCraftDelegate battleCraft2 = bca[1];
        BattlePlanetDelegate battlePlanet = battle.getPlanet();

        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toBattleWindow(battle);

        bc1.setLocation(new Location(900,500));
        bc1.setDirection(270);
        bc2.setLocation(new Location(200,500));
        bc2.setDirection(90);
        
        //
        // Test battleGetNextObject works (Attacker will move first).
        //
        
        assertEquals(333, bc1.getSpeed());
        assertEquals(100, bc2.getSpeed());
        
        clearEvents();
//        serverProxy.battleGetNextObject(battle, callbackSync, null);
//        serverProxy._processCommandResponse();
//        NameCR next = (NameCR) callbackSync.waitTillComplete();
        assertEquals(battleCraft1, battle.getNextObjectToMove());
        
        //
        // Test battleCraftMoveToCmd works - Can move craft with no target target
        //

        Logger.logDebug("");
        Logger.logDebug("battleCraftMoveToCmd works - Can move craft with no target targe");
        Logger.logDebug("");
        
        bc1.setMoveRemaining(1.0);
        bc1.rechargeShields();
        bc1.setShieldPower(0, bc1.getShields()[0].getPower()*0.9);
        bc1.getCraft().setDamage(bc1.getCraft().getMaxDamage()*0.2);
        bc1.setMovementDamage((int)(bc1.getSpeed(false)*0.7));
        bc1.setPsiDamage(1);
        bc1.setCrew((int)(bc1.getCraft().getHull().getCrew()*0.8));

        serverProxy.battleCraftMoveToCmd(battle, battleCraft1, new LocationAndDirection(850, 450, 270), null, false, false, callbackSync, null);
        serverProxy._processCommandResponse();
        
        BattleCraftMoveEvent cme = (BattleCraftMoveEvent) getEvent(BattleCraftMoveEvent.class);
        assertEquals(315, ((ObjectTurn)cme.getMovements()[0]).getTo());
        assertEquals(0.13, ((ObjectTurn)cme.getMovements()[0]).getMovePercent(), 0.1);
        assertEquals(900,500, ((ObjectShift)cme.getMovements()[1]).getOrigin());
        assertEquals(850, 450, ((ObjectShift)cme.getMovements()[1]).getDestination());
        assertEquals(0.75, ((ObjectShift)cme.getMovements()[1]).getMovePercent(), 0.1);
        assertEquals(272, ((ObjectTurn)cme.getMovements()[2]).getTo());
        assertEquals(0.13, ((ObjectTurn)cme.getMovements()[2]).getMovePercent(), 0.1);

        assertEquals(850, 450, battleCraft1.getLocation()); // These numbers do not matter, just check they are in the right direction
        assertEquals(272, battleCraft1.getDirection());
        assertEquals(0.0, battleCraft1.getMoveRemainingAsPercent());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(battle.getName(), battleCraft1.getBattleName());
        
        //
        // Test battleCraftMoveToCmd works - Craft moves and fires missile defence
        //

        Logger.logDebug("");
        Logger.logDebug("battleCraftMoveToCmd works - Craft moves and fires missile defence");
        Logger.logDebug("");
        
        bc1.setMoveRemaining(1.0);
        bc1.setLocation(new Location(400,500));
        bc1.setDirection(270);
        bc1.getCraft().setDamage(0.0);
        
        bc2.fireWeapon(bc1, bc2.getWeapon(tech_missile6), 1);
        serverProxy._processQueuedEvents();
        
        Missiles m = (Missiles) b.getAutomatons()[0];
        MissilesDelegate missile = (MissilesDelegate) battle.getAutomatons()[0];

        assertEquals(bc1, m.getTarget());

        clearEvents();
        serverProxy.battleCraftMoveToCmd(battle, battleCraft1, new LocationAndDirection(200, 500, 270), battleCraft2, true, true, callbackSync, null);
        serverProxy._processCommandResponse();

        assertEquals(true, m.isDestroyed());
        assertEquals(m.getName(), ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getTarget());
        assertEquals(bc1.getName(), ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getSource());
        assertEquals(tech_antiMissileBeam, ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getWeaponTechnology());

        assertEquals(300, 500, battleCraft1.getLocation()); // These numbers do not matter, just check they are in the right direction
        assertEquals(270, battleCraft1.getDirection());
        assertEquals(0.0, battleCraft1.getMoveRemainingAsPercent(), 0.1);
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());

        //
        // Test battleCraftMoveToCmd works - Craft stops on missile attack
        //

        Logger.logDebug("");
        Logger.logDebug("battleCraftMoveToCmd works - Craft stops on missile attack");
        Logger.logDebug("");
        
        bc1.setMoveRemaining(1.0);
        bc1.setLocation(new Location(400,500));
        bc1.setDirection(270);
        bc1.getCraft().setDamage(0.0);
        
        bc2.rechargeWeapons();
        bc2.fireWeapon(bc1, bc2.getWeapon(tech_missile6), 1);
        serverProxy._processQueuedEvents();
        
        m = (Missiles) b.getAutomatons()[0];
        missile = (MissilesDelegate) battle.getAutomatons()[0];

        assertEquals(bc1, m.getTarget());

        clearEvents();
        serverProxy.battleCraftMoveToCmd(battle, battleCraft1, new LocationAndDirection(200, 500, 270), battleCraft2, true, true, callbackSync, null);
        serverProxy._processCommandResponse();

        assertEquals(343, 500, battleCraft1.getLocation()); // These numbers do not matter, just check they are in the right direction
        assertEquals(270, battleCraft1.getDirection());
        assertEquals(0.43, battleCraft1.getMoveRemainingAsPercent(), 0.1);
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        
        BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) getEvent(BattleWeaponFireEvent.class);
        assertEquals(m.getName(), wfe.getSource());
        assertEquals(bc1.getName(), wfe.getTarget());
        assertEquals(true, m.isDestroyed());
        assertNull(battle.getAutomatons());

        //
        // Test battleCraftMoveToCmd works - Can move craft to target craft
        //

        Logger.logDebug("");
        Logger.logDebug("battleCraftMoveToCmd works - Can move craft to target craft");
        Logger.logDebug("");
        
        bc1.setMoveRemaining(1.0);
        bc1.setLocation(new Location(300,475));
        bc1.setDirection(270);
        bc1.getCraft().setDamage(0.0);
        
        serverProxy.battleCraftMoveToCmd(battle, battleCraft1, new LocationAndDirection(205, 495, 270), battleCraft2, true, true, callbackSync, null);
        serverProxy._processCommandResponse();

        assertEquals(233, 467, battleCraft1.getLocation()); // These numbers do not matter, just check they are in the right direction
        assertEquals(225, battleCraft1.getDirection());
        assertEquals(0.17, battleCraft1.getMoveRemainingAsPercent(), 0.1);
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());

        //
        // Test that my craft may have moved but has not finished its turn because it has weapons to fire.
        //

//        serverProxy.battleGetNextObject(battle, callbackSync, null);
//        serverProxy._processCommandResponse();
//        next = (NameCR) callbackSync.waitTillComplete();
        assertEquals(battleCraft1, battle.getNextObjectToMove());
        
        //
        // Test battleAttackCmd at a craft (No movement, single weapon)
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleAttackCmd at a craft (No movement, single weapon)");
        Logger.logDebug("");
        
        assertEquals(2, battleCraft1.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[0].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[1].getQty());
        assertEquals(tech_missile6, battleCraft1.getWeapons()[1].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[1].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[2].getQty());
        assertEquals(tech_antiMissileBeam, battleCraft1.getWeapons()[2].getWeapon());
        assertEquals(1, battleCraft1.getWeapons()[2].getQtyFired());
        
        bc1.rechargeWeapons();
        serverProxy.battleAttackCmd(battle, battleCraft1, battleCraft2, false, false, false, null, tech_missile6, 1, callbackSync, null);
        serverProxy._processCommandResponse();

        m = (Missiles) b.getAutomatons()[0];
        missile = (MissilesDelegate) battle.getAutomatons()[0];
        assertEquals(bc2, m.getTarget());
        assertEquals(battleCraft2.getName(), missile.getTarget());
        m._performMove(1.0, null);

        serverProxy._processQueuedEvents();

        assertEquals(bc1.getLocation(), battleCraft1.getLocation());
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());

        assertEquals(bc2.getLocation(), battleCraft2.getLocation());
        assertEquals(bc2.getDirection(), battleCraft2.getDirection());
        assertEquals(bc2.getHeight(), battleCraft2.getImageSize().getY());
        assertEquals(bc2.getWidth(), battleCraft2.getImageSize().getX());
        assertEquals(bc2.getShieldAsPercent(), battleCraft2.getState().getShieldPercent(), 0.01);
        assertEquals(bc2.getDamageAsPercent(), battleCraft2.getState().getDamagePercent(), 0.01);
        assertEquals(bc2.getMovementDamageAsPercent(), battleCraft2.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc2.getPsiDamageAsPercent(), battleCraft2.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc2.getCrewAsPercent(), battleCraft2.getState().getCrewPercent(), 0.0);
        assertEquals(bc2.getDirection(), battleCraft2.getDirection());

        assertEquals(2, battleCraft1.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[0].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[1].getQty());
        assertEquals(tech_missile6, battleCraft1.getWeapons()[1].getWeapon());
        assertEquals(1, battleCraft1.getWeapons()[1].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[2].getQty());
        assertEquals(tech_antiMissileBeam, battleCraft1.getWeapons()[2].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[2].getQtyFired());

        //
        // Test that my craft may have moved but has not finished its turn because it has weapons to fire.
        //

//        serverProxy.battleGetNextObject(battle, callbackSync, null);
//        serverProxy._processCommandResponse();
//        next = (NameCR) callbackSync.waitTillComplete();
        assertEquals(battleCraft1, battle.getNextObjectToMove());
        
        //
        // Test battleAttackCmd at a craft (With movement, with missile defence, all weapons of a single type)
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleAttackCmd at a craft (With movement, with missile defence, all weapons of a single type)");
        Logger.logDebug("");

        bc2.getCraft().setDamage(0);
        bc1.setLocation(new Location(300,500));
        bc1.setDirection(270);
        bc1.rechargeWeapons();
        bc1.setMoveRemaining(1.0);

        serverProxy.battleAttackCmd(battle, battleCraft1, battleCraft2, true, true, false, null, tech_massDriver1, -1, callbackSync, null);
        serverProxy._processCommandResponse();

        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());

        assertEquals(2, battleCraft1.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(2, battleCraft1.getWeapons()[0].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[1].getQty());
        assertEquals(tech_missile6, battleCraft1.getWeapons()[1].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[1].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[2].getQty());
        assertEquals(tech_antiMissileBeam, battleCraft1.getWeapons()[2].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[2].getQtyFired());

        assertEquals(2, battle.getCrafts().length);        
        assertEquals(battleCraft1, battle.getCrafts()[0]);
        assertEquals(battleCraft2, battle.getCrafts()[1]);
       
        //
        // Test battleAttackCmd at a craft (With movement, all weapons)
        // Target craft (bc2) destroyed
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleAttackCmd at a craft (With movement, all weapons, target destroyed)");
        Logger.logDebug("");

        bc2.getCraft().setDamage(bc2.getCraft().getMaxDamage()-1);
        bc1.setLocation(new Location(300,500));
        bc1.setDirection(270);
        bc1.rechargeWeapons();

        serverProxy.battleAttackCmd(battle, battleCraft1, battleCraft2, true, true, false, null, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();

        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());

        assertEquals(2, battleCraft1.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[0].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[1].getQty());
        assertEquals(tech_missile6, battleCraft1.getWeapons()[1].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[1].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[2].getQty());
        assertEquals(tech_antiMissileBeam, battleCraft1.getWeapons()[2].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[2].getQtyFired());

        assertEquals(1, battle.getCrafts().length);        
        assertEquals(battleCraft1, battle.getCrafts()[0]);
        
        order(battle.getCrafts(), new BattleCraftDelegate[1], "TestAttacker11");

        //
        // Trying to attack with no weapons to fire causes an error
        //
        
        bc1.dischargeWeapons();
        
        try {
        	serverProxy.battleAttackCmd(battle, battleCraft1, battlePlanet, false, false, false, null, null, -1, callbackSync, new TestErrorHandler());
            serverProxy._processCommandResponse();
        	fail("Should not get here");
        } catch (ValidationError ex) {
        	// All Good
        }
        
        //
        // Test battleAttackCmd at a Planet (No movement, all weapons, No Discharge)
        // Test Planet Destroyed and battle ends 
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleAttackCmd at a Planet (No movement, all weapons, No Discharge)");
        Logger.logDebug("");

        bc1.rechargeWeapons();
        bc1.setLocation(new Location(200,500));
        bc1.setDirection(270);

        serverProxy.battleAttackCmd(battle, battleCraft1, battlePlanet, false, false, false, null, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();
    }
    
    /**
     * Test Automate Battle Commands;
     */
    public void testAutomateBattleCommands() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testAutomateBattleCommands() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h2.setWeapons(new Weapons(tech_massDriver1));

        StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

        Craft c1 = newCraft(new CraftDesign("TestAttacker1", tech_flyer100, tech_fusion400, tech_energy20, tech_antiMissileBeam, tech_massDriver1, tech_massDriver1, tech_tacticalStationI), p1, ss2);
        Craft c2 = newCraft(new CraftDesign("TestFighter2", tech_flyer30, tech_fusion25, tech_massDriver1), p2, ss2);
        Craft c3 = newCraft(new CraftDesign("TestAttacker2", tech_flyer100, tech_fusion400, tech_energy20, tech_antiMissileBeam, tech_massDriver1, tech_massDriver1), p1, ss2);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestAttacker11");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter21");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestAttacker21");

        serverProxy.craftAttackCmd(craft1, planet2, null, null);
        serverProxy._processCommandResponse();
        serverProxy.craftAttackCmd(craft3, planet2, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        assertEquals(b.getName(), battle.getName());
        assertEquals(b.getTurn(), battle.getTurn());
        
        BattleCraft bc1 = b.getCrafts()[0];
        BattleCraft bc2 = b.getCrafts()[1];
        BattleCraft bc3 = b.getCrafts()[2];
        BattlePlanet bp = b.getPlanet();

        BattleCraftDelegate[] bca = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[3], "TestAttacker11", "TestFighter21", "TestAttacker21");
        BattleCraftDelegate battleCraft1 = bca[0];
        BattleCraftDelegate battleCraft2 = bca[1];
        BattleCraftDelegate battleCraft3 = bca[2];
        BattlePlanetDelegate battlePlanet = battle.getPlanet();

        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toBattleWindow(battle);

        bc1.setLocation(new Location(600,500));
        bc1.setDirection(270);
        bc2.setLocation(new Location(200,500));
        bc2.setDirection(90);
        
        assertEquals(333, bc1.getSpeed());
        assertEquals(100, bc2.getSpeed());
        assertEquals(333, bc3.getSpeed());

        //
        // Test auto move computer player's craft causes validation error
        //
        
        clearEvents();
        try {
            serverProxy.battleGetNextOrdersCmd(battle, battleCraft2, new TestErrorHandler());
            serverProxy._processCommandResponse();
        	fail("Cannot auto move another player's craft");
        } catch (ValidationError ex) { 
        	// success 
        }
        BattleCraftMoveEvent cme = (BattleCraftMoveEvent) getEvent(BattleCraftMoveEvent.class);
        assertNull(cme);
        
        //
        // Test auto move my craft works
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleGetNextOrdersCmd- auto move my craft works");
        Logger.logDebug("");
        
        clearEvents();
        serverProxy.battleGetNextOrdersCmd(battle, battleCraft1, null);
        serverProxy._processCommandResponse();
        
        cme = (BattleCraftMoveEvent) getEvent(BattleCraftMoveEvent.class);
        assertEquals(new Location(600,500), ((ObjectShift)cme.getMovements()[0]).getOrigin());
        assertEquals(new Location(267,500), ((ObjectShift)cme.getMovements()[0]).getDestination());
        assertEquals(1.0, ((ObjectShift)cme.getMovements()[0]).getMovePercent(), 0.001);
        
        BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) getEvent(BattleWeaponFireEvent.class);
        assertEquals(battleCraft1.getName(), wfe.getSource());
        assertEquals(battleCraft2.getName(), wfe.getTarget());

        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getMoveRemaining(), battleCraft1.getMoveRemainingAsPercent(), 0.01);
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        
        assertEquals(bc2.getDirection(), battleCraft2.getDirection());
        assertEquals(bc2.getHeight(), battleCraft2.getImageSize().getY());
        assertEquals(bc2.getWidth(), battleCraft2.getImageSize().getX());
        assertEquals(bc2.getMoveRemaining(), battleCraft2.getMoveRemainingAsPercent(), 0.01);
        assertEquals(bc2.getShieldAsPercent(), battleCraft2.getState().getShieldPercent(), 0.01);
        assertEquals(bc2.getDamageAsPercent(), battleCraft2.getState().getDamagePercent(), 0.01);
        assertEquals(bc2.getMovementDamageAsPercent(), battleCraft2.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc2.getPsiDamageAsPercent(), battleCraft2.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc2.getCrewAsPercent(), battleCraft2.getState().getCrewPercent(), 0.0);
        assertEquals(bc2.getDirection(), battleCraft2.getDirection());
        
        //
        // Next Object to move is users craft
        //
        
        assertEquals(battleCraft3, battle.getNextObjectToMove());
        
        //
        // Test that Automate battle works - Complete the battle
        //
        
        serverProxy.battleAutomateCmd(battle, null, null);
        serverProxy._processCommandResponse();
        assertNull(MotEngine.instance.getBattles().getActiveBattle());

        BattleEndEvent ebe = (BattleEndEvent) getEvent(BattleEndEvent.class);
        assertNotNull(ebe);
        assertEquals(1, ebe.getCraftDestroyd().length);
        assertEquals("TestFighter21", ebe.getCraftDestroyd()[0]);
        
        assertEquals(b.getName(), battle.getName());
        assertEquals(true, battle.hasEnded());
        assertEquals(true, battle.isWinner(player1));
        assertEquals(false, battle.isWinner(player2));
        assertEquals(p1.getName(), battle.getAggressors().getNamesAsArray()[0]);
        
        NamedObjectDelegate[] moos = order(DeligateCacheManager.getMoos(), new MoveableOwnedObjectDelegate[2], "TestAttacker11", "TestAttacker21"); 
    }

    /**
     * Test Automate Craft Through Battle;
     */
    public void testUserCraftLoseToMonster() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testUserCraftLoseToMonster() *************************");
        createMap();

        MonsterPlayer monsterPlayer = MotEngine.instance.getMonsterPlayer();
        Craft craft = CraftFactory.create("monster", monsterPlayer, monsterPlayer, h3.getOrbit(), false, false);

        StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");
        PlanetDelegate planet3 = DeligateCacheManager.getPlanet("SS3-A");

        Craft c1 = newCraft(new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_massDriver1), p1, ss3);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");

        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft bc1 = b.getCrafts()[0];
        BattleCraft bc2 = b.getCrafts()[1];

        BattleCraftDelegate[] bca = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[2], "monster1", "TestFighter1");
        BattleCraftDelegate battleCraft1 = bca[0];
        BattleCraftDelegate battleCraft2 = bca[1];

        //
        // Test auto move my craft works
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleGetNextOrdersCmd- auto move my craft works");
        Logger.logDebug("");
        
        while(!battle.hasEnded()) {
            assertEquals(battleCraft2, battle.getNextObjectToMove());
	        serverProxy.battleGetNextOrdersCmd(battle, battleCraft2, null);
	        serverProxy._processCommandResponse();
        }
        
        //
        // Battle has Completed
        //
        
        BattleEndEvent ebe = (BattleEndEvent) getEvent(BattleEndEvent.class);
        assertNotNull(ebe);
        assertEquals(1, ebe.getCraftDestroyd().length);
        assertEquals("TestFighter1", ebe.getCraftDestroyd()[0]);
        
        assertEquals(b.getName(), battle.getName());
        assertEquals(true, battle.hasEnded());
        assertEquals(false, battle.isWinner(player1));
        assertEquals("Monster", battle.getAggressors().getNamesAsArray()[0]);
        
        assertNull(DeligateCacheManager.getCrafts(player1));
        assertNull(DeligateCacheManager.getMoos());
        //NamedObjectDelegate[] moos = order(DeligateCacheManager.getMoos(), new MoveableOwnedObjectDelegate[1], "monster1");
        
        assertEquals(false, ssd3.isVisible());
    }


    /**
     * Test Launch Fighters
     */
    public void testLaunchFightersCommands() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testLaunchFightersCommands() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h2.setWeapons(new Weapons(tech_massDriver1));

        StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

        Craft c1 = newCraft(new CraftDesign("TestAttacker1", tech_flyer100, tech_fusion400, tech_energy20, tech_fighterBays), p1, ss2);
        Craft c2 = newCraft(fighterDesign, p1, ss2);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestAttacker11");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter1");

        serverProxy.craftAttackCmd(craft1, planet2, null, null);
        serverProxy._processCommandResponse();
        serverProxy.craftAttackCmd(craft2, planet2, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        assertEquals(b.getName(), battle.getName());
        assertEquals(b.getTurn(), battle.getTurn());
        
        BattleCraft bc1 = b.getCrafts()[0];
        BattleCraft bc2 = b.getCrafts()[1];
        BattlePlanet bp = b.getPlanet();

        BattleCraftDelegate[] bca = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[2], "TestAttacker11", "TestFighter1");
        BattleCraftDelegate battleCraft1 = bca[0];
        BattleCraftDelegate battleCraft2 = bca[1];
        BattlePlanetDelegate battlePlanet = battle.getPlanet();

        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toBattleWindow(battle);

        bc1.setLocation(new Location(600,500));
        bc1.setDirection(270);
        
        //
        // Test auto move my craft works
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleCraftLaunchFightersCmd- auto move my craft works");
        Logger.logDebug("");

        assertEquals(true, battleCraft1.hasFightersToLaunch());

        clearEvents();
        serverProxy.battleCraftLaunchFightersCmd(battle, battleCraft1, null, null);
        serverProxy._processCommandResponse();
        
        BattleCraft bc3 = b.getCrafts()[2];
        BattleCraft bc4 = b.getCrafts()[3];
        
        bca = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[4], "TestAttacker11", "TestFighter1", "BattleFighter1", "BattleFighter2");
        BattleCraftDelegate battleCraft3 = bca[2];
        BattleCraftDelegate battleCraft4 = bca[3];
        
        assertEquals(false, battleCraft1.hasFightersToLaunch());
        
        assertEquals(bc3.getDirection(), battleCraft3.getDirection());
        assertEquals(bc3.getHeight(), battleCraft3.getImageSize().getY());
        assertEquals(bc3.getWidth(), battleCraft3.getImageSize().getX());
        assertEquals(bc3.getMoveRemaining(), battleCraft3.getMoveRemainingAsPercent(), 0.01);
        assertEquals(bc3.getShieldAsPercent(), battleCraft3.getState().getShieldPercent(), 0.01);
        assertEquals(bc3.getDamageAsPercent(), battleCraft3.getState().getDamagePercent(), 0.01);
        assertEquals(bc3.getMovementDamageAsPercent(), battleCraft3.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc3.getPsiDamageAsPercent(), battleCraft3.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc3.getCrewAsPercent(), battleCraft3.getState().getCrewPercent(), 0.0);
        assertEquals(bc3.getDirection(), battleCraft3.getDirection());
        
        assertEquals(bc4.getDirection(), battleCraft4.getDirection());
        assertEquals(bc4.getHeight(), battleCraft4.getImageSize().getY());
        assertEquals(bc4.getWidth(), battleCraft4.getImageSize().getX());
        assertEquals(bc4.getMoveRemaining(), battleCraft4.getMoveRemainingAsPercent(), 0.01);
        assertEquals(bc4.getShieldAsPercent(), battleCraft4.getState().getShieldPercent(), 0.01);
        assertEquals(bc4.getDamageAsPercent(), battleCraft4.getState().getDamagePercent(), 0.01);
        assertEquals(bc4.getMovementDamageAsPercent(), battleCraft4.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc4.getPsiDamageAsPercent(), battleCraft4.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc4.getCrewAsPercent(), battleCraft4.getState().getCrewPercent(), 0.0);
        assertEquals(bc4.getDirection(), battleCraft4.getDirection());
        
        // Fighters are temporary crafts
        assertNull(DeligateCacheManager.getCraft("BattleFighter1"));
        assertNull(DeligateCacheManager.getCraft("BattleFighter2"));
    }


    
    /**
     * Test battle craft and the following battle commands;
     *  - Craft Retreat Command
     *  - Craft Teleport Command (move craft when engine is teleport)
     *  - Craft End Turn
     *  - Battle Ends when no enemies exist
     */
    public void testOtherCraftCommands() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testOtherCraftCommands() *************************");
        createMap();
        
        MotEngine.getInstance().getRelations().setWar(p1, p2);

        StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

        CraftDesign cd = new CraftDesign("XF", tech_flyer30, tech_teleport, tech_electronic25, tech_energy10, tech_massDriver1, tech_massDriver1, tech_massDriver2);
        Craft c1 = newCraft(cd, p1, ss2);
        Craft c2 = newCraft(fighterDesign, p2, ss2);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("XF1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter1");

        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        assertEquals(b.getName(), battle.getName());
        assertEquals(b.getTurn(), battle.getTurn());
        
        BattleCraft bc1 = b.getCrafts()[0];
        BattleCraft bc2 = b.getCrafts()[1];
        BattlePlanet bp = b.getPlanet();

        BattleCraftDelegate[] bca = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[2], "XF1", "TestFighter1");
        BattleCraftDelegate battleCraft1 = bca[0];
        BattleCraftDelegate battleCraft2 = bca[1];
        BattlePlanetDelegate battlePlanet = battle.getPlanet();

        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toBattleWindow(battle);

        bc1.setLocation(new Location(600,500));
        bc1.setDirection(275);
        bc2.setLocation(new Location(200,500));
        bc2.setDirection(90);
        
        //
        // TEST CRAFT TELEPORT
        //
        
        clearEvents();
        serverProxy.battleCraftMoveToCmd(battle, battleCraft1, new LocationAndDirection(500,500), null, false, false, callbackSync, null);
        serverProxy._processCommandResponse();

        BattleCraftTeleportEvent cte = (BattleCraftTeleportEvent) getEvent(BattleCraftTeleportEvent.class);
        assertNotNull(cte);
        assertEquals(bc1.getCraft().getName(), cte.getCraft());
        assertEquals(600,500, cte.getOldState().getLocation());
        assertEquals(500,500, cte.getNewState().getLocation());
        assertEquals(275, cte.getOldState().getDirection());
        assertEquals(270, cte.getNewState().getDirection());
        
        assertEquals(270, battleCraft1.getDirection());
        assertEquals(500,500, battleCraft1.getLocation());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getMoveRemaining(), battleCraft1.getMoveRemainingAsPercent(), 0.01);
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        
        //
        // TEST END TURN WITH SPECIFIC WEAPONS TO DISCHARGE (DISCHARGE ALL IS TESTED ELSE WHERE) - STARTS NEW TURN
        //
        
        Logger.logDebug("");
        Logger.logDebug("battleEndTurnCmd - end turn of craft (all weapons and movement)");
        Logger.logDebug("");

        bc1.setMoveRemaining(1.0);
        bc1.rechargeWeapons();
        clearEvents();
        
		ProxyStore.getInstance().battleEndTurnCmd(battle, battleCraft1, true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();
		
        BattleNewTurnEvent nte = (BattleNewTurnEvent) getEvent(BattleNewTurnEvent.class);
        assertEquals(2, ((BattleStateVO)nte.getType(BattleStateVO.class)).getTurn());
        
		assertEquals(1.0, bc1.getMoveRemaining(), 0.01);
		assertEquals(true, bc1.hasWeaponsToFire());
		
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());

        assertEquals(2, battleCraft1.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[0].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[1].getQty());
        assertEquals(tech_massDriver2, battleCraft1.getWeapons()[1].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[1].getQtyFired());

        bc1.setMoveRemaining(1.0);
        bc1.rechargeWeapons();
        clearEvents();
        
        Logger.logDebug("");
        Logger.logDebug("battleEndTurnCmd - end turn of craft (single weapon and not movement)");
        Logger.logDebug("");

		ProxyStore.getInstance().battleEndTurnCmd(battle, battleCraft1, false, true, tech_massDriver1, 1, callbackSync, null);
        serverProxy._processCommandResponse();

		assertEquals(1.0, bc1.getMoveRemaining(), 0.01);
		assertEquals(2, bc1.getWeapons()[0].getQty());
		assertEquals(1, bc1.getWeapons()[0].getQtyFired());
		assertEquals(1, bc1.getWeapons()[1].getQty());
		assertEquals(0, bc1.getWeapons()[1].getQtyFired());

        assertEquals(bc1.getDirection(), battleCraft1.getDirection());
        assertEquals(bc1.getHeight(), battleCraft1.getImageSize().getY());
        assertEquals(bc1.getWidth(), battleCraft1.getImageSize().getX());
        assertEquals(bc1.getShieldAsPercent(), battleCraft1.getState().getShieldPercent(), 0.01);
        assertEquals(bc1.getDamageAsPercent(), battleCraft1.getState().getDamagePercent(), 0.01);
        assertEquals(bc1.getMovementDamageAsPercent(), battleCraft1.getState().getMovementDamagePercent(), 0.0);
        assertEquals(bc1.getPsiDamageAsPercent(), battleCraft1.getState().getPsiDamagePercent(), 0.0);
        assertEquals(bc1.getCrewAsPercent(), battleCraft1.getState().getCrewPercent(), 0.0);
        assertEquals(bc1.getDirection(), battleCraft1.getDirection());

        assertEquals(2, battleCraft1.getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, battleCraft1.getWeapons()[0].getWeapon());
        assertEquals(1, battleCraft1.getWeapons()[0].getQtyFired());
        assertEquals(1, battleCraft1.getWeapons()[1].getQty());
        assertEquals(tech_massDriver2, battleCraft1.getWeapons()[1].getWeapon());
        assertEquals(0, battleCraft1.getWeapons()[1].getQtyFired());

        //
        // TEST BATTLE ENDS AND SEND END BATTLE EVENT
        //

        Logger.logDebug("");
        Logger.logDebug("battleRetreatCmd");
        Logger.logDebug("");
        
        bc1.setMoveRemaining(1.0);
        
        clearEvents();
        serverProxy.battleRetreatCmd(battle, battleCraft1, null, null);
        serverProxy._processCommandResponse();
        
        BattleCraftLeaveEvent clbe = (BattleCraftLeaveEvent) getEvent(BattleCraftLeaveEvent.class);
        assertNotNull(clbe);
        
        assertTrue(battle.hasEnded());
        
        BattleEndEvent ebe = (BattleEndEvent) getEvent(BattleEndEvent.class);
        assertNotNull(nbe);
        
    }     
    
    /**
     * Test that Mind Control weapons work.
     *  - Control can be passed from a computer player to a Human Player.
     *  - Control is passed back after user has controlled a computer players craft
     */
    public void testBattleCraftControllers() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testBattleCraftControllers() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);

        StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");
        PlanetDelegate planet3 = DeligateCacheManager.getPlanet("SS3-A");

        CraftDesign cd = new CraftDesign("MindFighter", tech_flyer30, tech_teleport, tech_electronic25, tech_energy10, tech_mindControl);
        Craft c1 = newCraft(cd, p1, ss3);
        Craft c2 = newCraft(fighterDesign, p2, ss3);
        Craft c3 = newCraft(fighterDesign, p2, ss3);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("MindFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter2");
        
        assertEquals("Correct Damage", 0.5, tech_mindControl.getDamage(), 0.01);
        assertEquals("Correct Damage", 3, tech_mindControl.getQtyShots().intValue());

		clearEvents();
		
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(700, 500));
        bca[2].setLocation(new Location(750, 500));

        BattleCraftDelegate[] crafts = (BattleCraftDelegate[]) order(battle.getCrafts(), new BattleCraftDelegate[3], "MindFighter1", "TestFighter1", "TestFighter2");
        
        //
        // CRAFT1 FIRES MIND CONTROL AT CRAFT2
        //

        clearEvents();
        Utils.setNextRandomProbability(0.4);
        serverProxy.battleAttackCmd(battle, crafts[0], crafts[1], false, false, false, null, tech_mindControl, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        
        BattleCraftControllerMindControl controller = (BattleCraftControllerMindControl) bca[1].getControllerDetails();
		assertEquals(1, controller.getQtyTurns());

		BattleWeaponFireEvent wfe = (BattleWeaponFireEvent) getEvent(BattleWeaponFireEvent.class);
		assertEquals("MindControlDecorator", crafts[1].getState().getDecorators().getDecorators()[0].getName());
		assertEquals(tech_mindControl.getName(), wfe.getWeapon());
		assertEquals(craft1.getName(), wfe.getSource());
		assertEquals(craft2.getName(), wfe.getTarget());

        //
        // CRAFT2 UNDER CONTROL OF PLAYER1 - 
		// PLAYER1 CAN MAKE CRAFT2 FIRES AT CRAFT3
		// NOTE - CRAFT1 HAS HIGHEST PRIORITY BUT IS CONTROLLED BY PLAYER1 SAME AS CRAFT2 
		// SO CRAFT2 CAN MOVE
        //

        Logger.logDebug("");
        Logger.logDebug("battleAttackCmd - Command craft under mind controlled (attack)");
        Logger.logDebug("");

        clearEvents();
        
        bca[1].setDirection(90);
        bca[1].setLocation(new Location(700, 500));
        serverProxy.battleAttackCmd(battle, crafts[1], crafts[2], false, false, false, null, null, 0, callbackSync, null);
        serverProxy._processCommandResponse();
		wfe = (BattleWeaponFireEvent) getEvent(BattleWeaponFireEvent.class);
		
		assertEquals("MindControlDecorator", crafts[1].getState().getDecorators().getDecorators()[0].getName());
		assertEquals(tech_massDriver1.getName(), wfe.getWeapon());
		assertEquals(craft2.getName(), wfe.getSource());
		assertEquals(craft3.getName(), wfe.getTarget());

        //
        // PLAYER1 CAN MOVE CRAFT2
        //

        Logger.logDebug("");
        Logger.logDebug("battleAttackCmd - Command craft under mind controlled (move)");
        Logger.logDebug("");

        clearEvents();
        
        serverProxy.battleCraftMoveToCmd(battle, crafts[1], new LocationAndDirection(710,500), null, false, false, callbackSync, null);
        serverProxy._processCommandResponse();
        
		assertEquals("MindControlDecorator", crafts[1].getState().getDecorators().getDecorators()[0].getName());
		
        BattleCraftMoveEvent cme = (BattleCraftMoveEvent) getEvent(BattleCraftMoveEvent.class);
        assertEquals(new Location(700,500), ((ObjectShift)cme.getMovements()[0]).getOrigin());
        assertEquals(new Location(710,500), ((ObjectShift)cme.getMovements()[0]).getDestination());
        
        //
        // CONTROL IS RETURNED TO PLAYER2 ONCE CRAFT HAS FINISHED MOVING
        //
        
        serverProxy.battleEndTurnCmd(battle, crafts[1], true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();

		assertNull(crafts[1].getState().getDecorators().getDecorators());
		assertNull(bca[1].getControllerDetails());
    }     

    
    /**
     * Test Mines and spatial distortions - Mines not visible to enemy player.
     */
    public void testIncumbentsMinesNotVisible() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testIncumbentsMinesNotVisible() *************************");
        createMap();
        
        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h2.setHasAsteroids(true);
        h2.setWeapons(new Weapons(tech_spacialDistortion));
        p2.addMiscTechnology(tech_mineField);

        StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

        Craft c1 = newCraft(fighterDesign, p1, ss2);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        
        serverProxy.craftAttackCmd(craft1, planet2, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        Battle _battle = MotEngine.instance.getBattles().getActiveBattle();
		Automaton [] mines = _battle.getAutomatons();
        BattleMine mine1 = (BattleMine)mines[0];
        mine1.setLocation(new Location(450, 500));
        
        BattleCraft [] bca = _battle.getCrafts(); 
        BattleCraftDelegate bc1 = battle.getCrafts()[0];

        //
        // Test that asteroids exist
        //
        
        AsteroidFieldDetails af = (AsteroidFieldDetails) battle.getAnnimatedBattleAreas()[0];
        assertEquals(700, af.getLeft());
        assertEquals(150, af.getTop());
        assertEquals(50, af.getWidth());
        assertEquals(700, af.getHeight());
        
        //
        // Test player1 cannot see the mines
        //
        
        assertEquals(null, battle.getAutomatons());

        //
        // Test spatial distortion is visible on the battle field 
        //

        bca[0].setMoveRemaining(0.0);
        bca[0].dischargeWeapons();
        
        BattlePlanetDelegate planet = battle.getPlanet();
        BattlePlanet _planet = _battle.getPlanet();
        _planet.fireWeapon(new Location(300,500), _planet.getWeapon(tech_spacialDistortion), 1);
        
        serverProxy._processQueuedEvents();
        
        AnimatedBattleAreaDetails [] sd = battle.getAnnimatedBattleAreas();
        assertTrue(sd[1] instanceof SpaceDistortionDetails);
        assertEquals(275, sd[1].getLeft());
        assertEquals(450, sd[1].getTop());
        assertEquals(50, sd[1].getWidth());
        assertEquals(100, sd[1].getHeight());
        
        //
        // Test Craft Debris 
        //

        
        bca[0].destroy(null, null, null);
        serverProxy._processQueuedEvents();
        
        sd = battle.getAnnimatedBattleAreas();
        assertTrue(sd[2] instanceof WreckageDetails);
        assertEquals(888, sd[2].getLeft());
        assertEquals(488, sd[2].getTop());
        assertEquals(20, sd[2].getWidth());
        assertEquals(20, sd[2].getHeight());
        
        assertNull(battle.getCrafts());
    }
    
    /**
     * Test Mines and spatial distortions - Mines visible to player that owns them.
     */
    public void testIncumbentsMinesVisible() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testIncumbentsMinesVisible() *************************");
        createMap();
        
        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h1.setHasAsteroids(true);
        h1.setWeapons(new Weapons(tech_spacialDistortion, tech_missile6));
        p1.addMiscTechnology(tech_mineField);

        StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS1-A");

        CraftDesign cd = new CraftDesign("SalvageFighter", tech_flyer30, tech_fusion25, tech_salvageBeam);
        Craft c1 = newCraft(fighterDesign, p2, ss1);
        Craft c2 = newCraft(cd, p2, ss1);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("SalvageFighter1");
        
        c1.attack(h1);
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        Battle _battle = MotEngine.instance.getBattles().getActiveBattle();
		Automaton [] mines = _battle.getAutomatons();
        BattleMine mine1 = (BattleMine)mines[0];
        
        BattleCraft [] _bca = _battle.getCrafts(); 
        MotBattleCraftDelegate [] bca = (MotBattleCraftDelegate[]) order(battle.getCrafts(), new MotBattleCraftDelegate[2], "TestFighter1", "SalvageFighter1");
        // If mine placed too near craft causes this to fail
        
        //
        // Test that asteroids exist
        //
        
        AsteroidFieldDetails af = (AsteroidFieldDetails) battle.getAnnimatedBattleAreas()[0];
        assertEquals(700, af.getLeft());
        assertEquals(150, af.getTop());
        assertEquals(50, af.getWidth());
        assertEquals(700, af.getHeight());
        
        //
        // Test that player1 can see the mines
        //
        
        assertEquals(3, battle.getAutomatons().length);
        BattleMineDelegate bmd1 = (BattleMineDelegate)battle.getAutomatons()[0];
        BattleMineDelegate bmd2 = (BattleMineDelegate)battle.getAutomatons()[1];
        BattleMineDelegate bmd3 = (BattleMineDelegate)battle.getAutomatons()[2];
        
        Automaton[] aa = _battle.getAutomatons();
		assertEquals(aa[0].getName(), bmd1.getName());
        assertEquals(aa[0].getLocation(), bmd1.getLocation());
       	
        //
        // Test spatial distortion is visible on the battle field 
        //

        _bca[0].setMoveRemaining(0.0);
        _bca[0].dischargeWeapons();
        _bca[1].setMoveRemaining(0.0);
        _bca[1].dischargeWeapons();
        
        BattlePlanetDelegate planet = battle.getPlanet();
        serverProxy.battleAttackLocationCmd(battle, planet, new Location(300,500), tech_spacialDistortion, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        
        AnimatedBattleAreaDetails [] sd = battle.getAnnimatedBattleAreas();
        assertEquals(2, sd.length);
        assertTrue(sd[0] instanceof AsteroidFieldDetails);
        assertTrue(sd[1] instanceof SpaceDistortionDetails);
        assertEquals(275, sd[1].getLeft());
        assertEquals(450, sd[1].getTop());
        assertEquals(50, sd[1].getWidth());
        assertEquals(100, sd[1].getHeight());
        
        //
        // Test Craft produces debris 
        //

        _bca[0].destroy(null, null, null);
        serverProxy._processQueuedEvents();
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        sd = battle.getAnnimatedBattleAreas();
        assertEquals(3, sd.length);
        assertTrue(sd[0] instanceof AsteroidFieldDetails);
        assertTrue(sd[1] instanceof SpaceDistortionDetails);
        assertTrue(sd[2] instanceof WreckageDetails);
        assertEquals(788, sd[2].getLeft());
        assertEquals(488, sd[2].getTop());
        assertEquals(20, sd[2].getWidth());
        assertEquals(20, sd[2].getHeight());
        
        //
        // Test debris can be removed from the battle 
        //
        
        _bca[1].rechargeWeapons();
        _bca[1].setLocation(new Location(950,500));
        _bca[1].fireWeapon(_battle.getAnnimatedBattleAreas()[2].getCenterLocation(), _bca[1].getWeapon(tech_salvageBeam), 1);

        serverProxy._processQueuedEvents();
        
        sd = battle.getAnnimatedBattleAreas();
        assertEquals(2, sd.length);
        assertTrue(sd[0] instanceof AsteroidFieldDetails);
        assertTrue(sd[1] instanceof SpaceDistortionDetails);
        
    }
    
    /**
     * Test hologram decorator is visible to owner but not to enemies. 
     */
    public void testDecoratorsVisible() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testDecoratorsVisible() *************************");
        createMap();
        
        MotEngine.getInstance().getRelations().setWar(p1, p2);

        StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

        CraftDesign cd1 = new CraftDesign("HoloFighter", tech_flyer30, tech_fusion25, tech_massDriver1, tech_holoEmiter);
        CraftDesign cd3 = new CraftDesign("SuicideFighter", tech_flyer30, tech_fusion25, tech_detonator);
        CraftDesign cd4 = new CraftDesign("Commander", tech_flyer30, tech_fusion25, tech_commandModule);
        Craft c1 = newCraft(cd1, p1, ss2);
        Craft c2 = newCraft(fighterDesign, p2, ss2);
        Craft c3 = newCraft(cd3, p1, ss2);
        Craft c4 = newCraft(cd4, p1, ss2);
        Craft c5 = newCraft(fighterDesign, p1, ss2);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("HoloFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("SuicideFighter1");
        CraftDelegate craft4 = DeligateCacheManager.getCraft("Commander1");
        CraftDelegate craft5 = DeligateCacheManager.getCraft("TestFighter2");
        
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.craftAttackCmd(craft3, null, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.craftAttackCmd(craft4, null, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.craftAttackCmd(craft5, null, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        Battle _battle = MotEngine.instance.getBattles().getActiveBattle();
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] _bca = _battle.getCrafts();
        MotBattleCraftDelegate [] bca = (MotBattleCraftDelegate[]) order(battle.getCrafts(), new MotBattleCraftDelegate[6], "HoloFighter1", "TestFighter1", "HoloFighter111", "SuicideFighter1", "Commander1", "TestFighter2");
        BattleCraftDelegate bc1 = bca[0];
        BattleCraftDelegate bc2 = bca[1];
        BattleCraftDelegate bc3 = bca[2];
        BattleCraftDelegate bc4 = bca[3];
        BattleCraftDelegate bc5 = bca[4];
        BattleCraftDelegate bc6 = bca[5];
        
        assertEquals("TestFighter2", _bca[4].getName());
        _bca[4].setCrew(0);
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[4], _bca[4].getCraftState()));
        serverProxy._processQueuedEvents();
        
        assertEquals(1, bc3.getWeapons().length);
        assertEquals("FakeWeapon", bc3.getWeapons()[0].getWeapon().getName());
        
        //
        // check that player two cannot see the hologram, Command Centre decorator
        //
        
        Decorator [] decorators = bc3.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("HologramDecorator", decorators[0].getName());
        
        decorators = bc4.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("SuicideDecorator", decorators[0].getName());
        
        assertEquals("Commander1", bc5.getCraft().getName());
        decorators = bc5.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("CommandCenterDecorator", decorators[0].getName());
        
        //
        // check disabled craft have decorator visible
        //
        
        assertEquals("TestFighter2", bc6.getCraft().getName());
        decorators = bc6.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("DisabledDecorator", decorators[0].getName());
        
        //
        // Test Mind Control Decorator 
        //
        
        _bca[1].setControllerDetails(new BattleCraftControllerMindControl(p2, _bca[1], 1));
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[1], _bca[1].getCraftState()));
        serverProxy._processQueuedEvents();

        decorators = bc2.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("MindControlDecorator", decorators[0].getName());
        
        //
        // Test Berserker Decorator
        //
        
        _bca[1].setControllerDetails(new BattleCraftControllerBerserker(_bca[1], 1));
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[1], _bca[1].getCraftState()));
        serverProxy._processQueuedEvents();

        decorators = bc2.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("BerserkerDecorator", decorators[0].getName());
        
        //
        // Test Cowardice Decorator
        //
        
        _bca[1].setControllerDetails(new BattleCraftControllerCowardice(_bca[1], 1));
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[1], _bca[1].getCraftState()));
        serverProxy._processQueuedEvents();

        decorators = bc2.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("CowardiceDecorator", decorators[0].getName());
        
        //
        // Test DamageBot Decorator
        //
        
        _bca[1].setControllerDetails(null);
		_battle.getTriggersNotNull().add(new DamageBot(_bca[1].getOwner(), _battle, _bca[1], CraftTriggerBase.INFINATE_LIFE, 6));
		_battle.getTriggersNotNull().add(new Plague(_bca[1].getOwner(), _battle, _bca[1], CraftTriggerBase.INFINATE_LIFE, 7));
		_bca[1].getDecorators().update();
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[1], _bca[1].getCraftState()));
        serverProxy._processQueuedEvents();

        decorators = bc2.getState().getDecorators().getDecorators();
        
        assertEquals(2, decorators.length);
        assertEquals("DamageBotDecorator", decorators[0].getName());
        assertEquals("PlagueDecorator", decorators[1].getName());
    }
    
    /**
     * Test hologram decorator is visible to owner but not to enemies. 
     */
    public void testDecoratorsNotVisible() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testDecoratorsNotVisible() *************************");
        createMap();
        
        MotEngine.getInstance().getRelations().setWar(p1, p2);

        StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
        PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

        CraftDesign cd1 = new CraftDesign("HoloFighter", tech_flyer30, tech_fusion25, tech_massDriver1, tech_holoEmiter);
        CraftDesign cd3 = new CraftDesign("SuicideFighter", tech_flyer30, tech_fusion25, tech_detonator);
        CraftDesign cd4 = new CraftDesign("Commander", tech_flyer30, tech_fusion25, tech_commandModule);
        Craft c1 = newCraft(fighterDesign, p1, ss2);
        Craft c2 = newCraft(cd1, p2, ss2);
        Craft c3 = newCraft(cd3, p2, ss2);
        Craft c4 = newCraft(cd4, p2, ss2);
        Craft c5 = newCraft(fighterDesign, p2, ss2);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("HoloFighter1");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("SuicideFighter1");
        CraftDelegate craft4 = DeligateCacheManager.getCraft("Commander1");
        CraftDelegate craft5 = DeligateCacheManager.getCraft("TestFighter2");

        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
 

        Battle _battle = MotEngine.instance.getBattles().getActiveBattle();
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] _bca = _battle.getCrafts();
        MotBattleCraftDelegate [] bca = (MotBattleCraftDelegate[]) order(battle.getCrafts(), new MotBattleCraftDelegate[6], "HoloFighter1", "TestFighter1", "HoloFighter111", "SuicideFighter1", "Commander1", "TestFighter2");
        BattleCraftDelegate bc1 = bca[0];
        BattleCraftDelegate bc2 = bca[1];
        BattleCraftDelegate bc3 = bca[2];
        BattleCraftDelegate bc4 = bca[3];        
        BattleCraftDelegate bc5 = bca[4];
        BattleCraftDelegate bc6 = bca[5];
        
        assertEquals("TestFighter2", _bca[4].getName());
        _bca[4].setCrew(0);
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[4], _bca[4].getCraftState()));
        serverProxy._processQueuedEvents();

        assertEquals(1, bc3.getWeapons().length);
        assertEquals("FakeWeapon", bc3.getWeapons()[0].getWeapon().getName());
        
        //
        // check that player two cannot see the hologram, command centre decorator
        //
        
        assertEquals(null, bc3.getState().getDecorators());
        assertEquals(null, bc4.getState().getDecorators());
        assertEquals(null, bc5.getState().getDecorators());

        //
        // check disabled craft have decorator visible
        //
        
        assertEquals("TestFighter2", bc6.getCraft().getName());
        Decorator [] decorators = bc6.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("DisabledDecorator", decorators[0].getName());
        
        //
        // Test Mind Control Decorator
        //
        
        assertEquals("TestFighter1", _bca[0].getName());
        assertEquals("TestFighter1", bc2.getName());
        
        _bca[0].setControllerDetails(new BattleCraftControllerMindControl(p2, _bca[0], 1));
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[0], _bca[0].getCraftState()));
        serverProxy._processQueuedEvents();

        decorators = bc2.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("MindControlDecorator", decorators[0].getName());
        
        
        //
        // Test Berserker Decorator
        //
        
        _bca[0].setControllerDetails(new BattleCraftControllerBerserker(_bca[0], 1));
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[0], _bca[0].getCraftState()));
        serverProxy._processQueuedEvents();

        decorators = bc2.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("BerserkerDecorator", decorators[0].getName());
        
        //
        // Test Cowardice Decorator
        //
        
        _bca[0].setControllerDetails(new BattleCraftControllerCowardice(_bca[0], 1));
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[0], _bca[0].getCraftState()));
        serverProxy._processQueuedEvents();

        decorators = bc2.getState().getDecorators().getDecorators();
        
        assertEquals(1, decorators.length);
        assertEquals("CowardiceDecorator", decorators[0].getName());
        
        //
        // Test DamageBot Decorator
        //
        
		_battle.getTriggersNotNull().add(new DamageBot(_bca[0].getOwner(), _battle, _bca[0], CraftTriggerBase.INFINATE_LIFE, 6));
		_battle.getTriggersNotNull().add(new Plague(_bca[0].getOwner(), _battle, _bca[0], CraftTriggerBase.INFINATE_LIFE, 7));
		_bca[0].setCrew(0);
        MotEngine.instance.broadcast(new BattleCraftUpdateEF(_bca[0], _bca[0].getCraftState()));
        serverProxy._processQueuedEvents();

        decorators = bc2.getState().getDecorators().getDecorators();
        
        assertEquals(4, decorators.length);
        assertEquals("DisabledDecorator", decorators[0].getName());
        assertEquals("CowardiceDecorator", decorators[1].getName());
        assertEquals("DamageBotDecorator", decorators[2].getName());
        assertEquals("PlagueDecorator", decorators[3].getName());
    }
    
    /**
     * Test that Droid Mines work
     *  - When weapon is fired creates automatons.
     *  - These automatons can be destroyed by weapons and their delegates contain the right info
     */
    public void testBattleDroidMine() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testBattleDroidMine() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);

        StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");
        PlanetDelegate planet3 = DeligateCacheManager.getPlanet("SS3-A");

        CraftDesign cd = new CraftDesign("AutomatonFighter", tech_flyer30, tech_fusion25, tech_massDriver1, tech_droidMine, tech_tacticalStationI);
        Craft c1 = newCraft(cd, p1, ss3);
        Craft c2 = newCraft(fighterDesign, p2, ss3);
        
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        CraftDelegate craft1 = DeligateCacheManager.getCraft("AutomatonFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter1");
        
		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] _bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        _bca[0].setLocation(new Location(500, 500));
        _bca[0].setDirection(90);
        _bca[1].setLocation(new Location(700, 500));
        _bca[1].setDirection(270);
        
        MotBattleCraftDelegate [] bca = (MotBattleCraftDelegate[]) order(battle.getCrafts(), new MotBattleCraftDelegate[2], "AutomatonFighter1",  "TestFighter1");
        BattleCraftDelegate bc1 = bca[0];
        BattleCraftDelegate bc2 = bca[1];

        //
        // TEST DROID MINES DO NOT FIRE UNLESS EXPLICITLY NAMED
        //

        clearEvents();
        Utils.setNextRandomProbability(0.4);
        _bca[0].rechargeWeapons();
        serverProxy.battleAttackCmd(battle, bc1, bc2, false, false, false, null, null, 0, callbackSync, null);
        serverProxy._processCommandResponse();
        
        assertNull(battle.getAutomatons());

        //
        // CRAFT1 FIRES DROID MINE AT CRAFT2 - CREATES DROID MINE AUTOMITON WITH TARGET CRAFT2
        //

        clearEvents();
        Utils.setNextRandomProbability(0.4);
        _bca[0].rechargeWeapons();
        serverProxy.battleAttackCmd(battle, bc1, bc2, false, false, false, null, tech_droidMine, 1, callbackSync, null);
        serverProxy._processCommandResponse();
        
        DroidMine _droidMine = (DroidMine) MotEngine.instance.getBattles().getActiveBattle().getAutomatons()[0];
        BattleDroidMineDelegate droidMine = (BattleDroidMineDelegate) battle.getAutomatons()[0];
        BattleDroidMineLaunchEvent dmle = (BattleDroidMineLaunchEvent) getEvent(BattleDroidMineLaunchEvent.class);
        
        assertEquals(craft2.getName(), droidMine.getTarget());
        assertEquals(craft2.getName(), dmle.getDroidMine().getTarget());

        assertEquals(_droidMine.getOrigin().getName(), droidMine.getOrigin());
        assertEquals(_droidMine.getTarget().getName(), droidMine.getTarget());
        assertEquals(_droidMine.getLocation(), droidMine.getState().getLocation());
        assertEquals(_droidMine.getDirection(), droidMine.getState().getDirection());
        
        //
        // DROID MINE MOVES TOWARDS CRAFT2
        //

        clearEvents();
        serverProxy.battleEndTurnCmd(battle, bc1, true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();

//        serverProxy.battleGetNextObject(battle, callbackSync, null);
//        serverProxy._processCommandResponse();
//        serverProxy._processQueuedEvents();
//        callbackSync.waitTillComplete();
        
        BattleCraftMoveEvent cme = (BattleCraftMoveEvent) getEvent(BattleCraftMoveEvent.class);
        BattleDroidMineMoveEvent dmme = (BattleDroidMineMoveEvent) cme.getLinkedEvents()[0];
        assertEquals(craft2.getName(), ((BattleDroidMineDelegate)DeligateCacheManager.getBattleAutomaton(dmme.getBattle(), dmme.getDroidMine())).getTarget());
        
        assertEquals(_droidMine.getOrigin().getName(), droidMine.getOrigin());
        assertEquals(_droidMine.getTarget().getName(), droidMine.getTarget());
        assertEquals(_droidMine.getLocation(), droidMine.getState().getLocation());
        assertEquals(_droidMine.getDirection(), droidMine.getState().getDirection());
        assertEquals(false, droidMine.getState().isDestroyed());
        
        //
        // DROID MINE MOVES TOWARDS CRAFT2 AND DETONATES - DAMAGES CRAFT2 AND IS GONE BUT STILL AVAILABLE IN CACHE BY NAME
        //

        clearEvents();
        serverProxy.battleEndTurnCmd(battle, bc1, true, true, null, -1, callbackSync, null);
        serverProxy._processCommandResponse();

//        serverProxy.battleGetNextObject(battle, callbackSync, null);
//        serverProxy._processCommandResponse();
//        serverProxy._processQueuedEvents();
//        callbackSync.waitTillComplete();
        
        cme = (BattleCraftMoveEvent) getEvent(BattleCraftMoveEvent.class);
        dmme = (BattleDroidMineMoveEvent) cme.getLinkedEvents()[0];
        BattleAutomatonDelegate t = DeligateCacheManager.getBattleAutomaton(dmme.getBattle(), dmme.getDroidMine());
		assertEquals(craft2.getName(), ((BattleDroidMineDelegate)t).getTarget());
        
        assertEquals(_droidMine.getOrigin().getName(), droidMine.getOrigin());
        assertEquals(_droidMine.getTarget().getName(), droidMine.getTarget());
        assertEquals(_droidMine.getLocation(), droidMine.getState().getLocation());
        assertEquals(_droidMine.getDirection(), droidMine.getState().getDirection());
        assertEquals(true, droidMine.getState().isDestroyed());
        
        assertNull(DeligateCacheManager.getBattle(dmme.getBattle()).getAutomatons());
    }
    
    /**
     * Tests that when a craft is retreated from battle and its owner loses the battle - this craft is destroyed.
     */
    public void testBattleRoute() throws Exception {
        Logger.logDebug("*******************************  WarTest.testRoute() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);

        StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");
        PlanetDelegate planet3 = DeligateCacheManager.getPlanet("SS3-A");

        Craft c1 = newCraft(fighterDesign, p1, ss2);
        Craft c2 = newCraft(fighterDesign, p1, ss2);
        Craft c3 = newCraft(fighterDesign, p1, ss2);

        Craft c4 = newCraft(fighterDesign, p2, ss2);
        Craft c5 = newCraft(fighterDesign, p2, ss2);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter2");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter3");

		clearEvents();
        serverProxy.craftAttackCmd(craft1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy.craftAttackCmd(craft2, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy.craftAttackCmd(craft3, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 
        assertEquals(c1.getName(), bca[0].getName());
        assertEquals(c4.getName(), bca[1].getName());
        assertEquals(c5.getName(), bca[2].getName());
        assertEquals(c2.getName(), bca[3].getName());
        assertEquals(c3.getName(), bca[4].getName());

        bca[0].retreat();
        MotEngine.instance.getProxyManager().broadcastQueue();
        serverProxy._processQueuedEvents();

        assertEquals(false, battle.hasEnded());
        
        BattleCraftDelegate[] visibleMOOs = battle.getCrafts();
        assertEquals(4, visibleMOOs.length);
        assertEquals(c4.getName(), visibleMOOs[0].getName());
        assertEquals(c5.getName(), visibleMOOs[1].getName());
        assertEquals(c2.getName(), visibleMOOs[2].getName());
        assertEquals(c3.getName(), visibleMOOs[3].getName());
        
        MotEngine.instance.getBattles().getActiveBattle().endTurn();
        eventList.clear();
        
        bca[3].retreat();
        bca[4].retreat();
        MotEngine.instance.getProxyManager().broadcastQueue();
        serverProxy._processQueuedEvents();

        assertEquals(true, battle.hasEnded());
        
        visibleMOOs = battle.getCrafts();
        assertEquals(2, visibleMOOs.length);
        assertEquals(c4.getName(), visibleMOOs[0].getName());
        assertEquals(c5.getName(), visibleMOOs[1].getName());
        
        RoutedEvent re = (RoutedEvent) serverProxy.getClientEventProcessor().fetchFirstEventType(RoutedEvent.class);
        
        assertEquals(2, re.getCraftDestroyed().length);
        assertEquals(c2.getName(), re.getCraftDestroyed()[0]);
        assertEquals(c3.getName(), re.getCraftDestroyed()[1]);
    }

    /**
     * 
     */
    public void testDestroyOutpostUsingFleet() throws Exception {
        Logger.logDebug("*******************************  WarTest.testDestroyOutpostUsingFleet() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);

        StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");

        ss3.deploy(new Orbital(tech_outpost, tech_photon100, null), p2);
        
        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer60, tech_fusion400, tech_fusionBeam4, tech_fusionBeam4, tech_armour200);
        Craft c1 = newCraft(cd, p1, ss3);
        Craft c2 = newCraft(cd, p1, ss3);
        Craft c3 = newCraft(cd, p1, ss3);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter2");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter3");

        assertEquals(tech_outpost, ssd3.getStellarObjectOrbital().getOrbital().getHullTech());
        assertEquals(player2, ssd3.getStellarObjectOrbital().getOwner());

        serverProxy.joinFleetCmd(craft1, null, null);
        serverProxy._processCommandResponse();
        serverProxy.joinFleetCmd(craft2, null, null);
        serverProxy._processCommandResponse();
        serverProxy.joinFleetCmd(craft3, null, null);
        serverProxy._processCommandResponse();

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        FleetDelegate fleet1 = (FleetDelegate) DeligateCacheManager.getMoo("Fleet1");

        assertEquals(3, fleet1.getCrafts().length);
        CraftDelegate[] fleetCrafts = fleet1.getCrafts();
		assertEquals(craft1, craft2, craft3, fleetCrafts);
        assertEquals(fleet1, craft1.getFleetDelegate());
        assertEquals(fleet1, craft2.getFleetDelegate());
        assertEquals(fleet1, craft3.getFleetDelegate());

		clearEvents();
        serverProxy.craftAttackCmd(fleet1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 

        BattleCraftDelegate[] visibleMOOs = battle.getCrafts();
        assertEquals(4, visibleMOOs.length);
        assertEquals("BattleStation1", visibleMOOs[0].getName());
        assertEquals(c1.getName(), visibleMOOs[1].getName());
        assertEquals(c2.getName(), visibleMOOs[2].getName());
        assertEquals(c3.getName(), visibleMOOs[3].getName());
        
        serverProxy.battleAutomateCmd(battle, null, null);
        serverProxy._processCommandResponse();

        assertEquals(true, battle.hasEnded());
        BattleEndEvent endEvent = (BattleEndEvent) getEvent(BattleEndEvent.class);
        
        assertEquals(null, ss3.getOrbital());
        assertNull(null, endEvent.getCraftDestroyd()); // Battle Stations do not produce wreckage
        assertEquals(null, ssd3.getStellarObjectOrbital());
        
        CraftDelegate[] crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(player1), new CraftDelegate[3], "TestFighter1", "TestFighter2", "TestFighter3");
        
    }

    /**
     * 
     */
    public void testDestroySpaceStationAtBlackHoleUsingFleet() throws Exception {
        Logger.logDebug("*******************************  WarTest.testDestroySpaceStationAtBlackHoleUsingFleet() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);

        BlackHole bh = newBlackHole("BlackHole1", 200, 200, 0.1f, false);
        bh.deploy(new Orbital(tech_spaceStation, null, null), p2);
        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        MotBlackHoleDelegate blackHole = (MotBlackHoleDelegate) DeligateCacheManager.getStellarObject("BlackHole1");
        assertEquals(tech_spaceStation, blackHole.getStellarObjectOrbital().getOrbital().getHullTech());
        assertEquals(player2, blackHole.getStellarObjectOrbital().getOwner());

        
        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer200, tech_fusion400, tech_electronic40, tech_missile15, tech_missile15, tech_missile15, tech_missile15, tech_missile15, tech_missile15, tech_missile15, tech_missile15, tech_missile15, tech_missile15, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_armour200, tech_energy20, tech_shieldModulatorII);
        Craft c1 = newCraft(cd, p1, bh);
        Craft c2 = newCraft(cd, p1, bh);
        Craft c3 = newCraft(cd, p1, bh);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter2");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter3");

        serverProxy.joinFleetCmd(craft1, null, null);
        serverProxy._processCommandResponse();
        serverProxy.joinFleetCmd(craft2, null, null);
        serverProxy._processCommandResponse();
        serverProxy.joinFleetCmd(craft3, null, null);
        serverProxy._processCommandResponse();

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        FleetDelegate fleet1 = (FleetDelegate) DeligateCacheManager.getMoo("Fleet1");

        assertEquals(3, fleet1.getCrafts().length);
        assertEquals(craft1, craft2, craft3, fleet1.getCrafts());
        assertEquals(fleet1, craft1.getFleetDelegate());
        assertEquals(fleet1, craft2.getFleetDelegate());
        assertEquals(fleet1, craft3.getFleetDelegate());

		clearEvents();
        serverProxy.craftAttackCmd(fleet1, null, null, null);
        serverProxy._processCommandResponse();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 

        BattleCraftDelegate[] visibleMOOs = battle.getCrafts();
        assertEquals(4, visibleMOOs.length);
        assertEquals("BattleStation1", visibleMOOs[0].getName());
        assertEquals(c1.getName(), visibleMOOs[1].getName());
        assertEquals(c2.getName(), visibleMOOs[2].getName());
        assertEquals(c3.getName(), visibleMOOs[3].getName());
        
        serverProxy.battleGetNextObject(battle, null, null);
        serverProxy._processCommandResponse();
        while (battle.getNextObjectToMove() != null) {
	        serverProxy.battleGetNextOrdersCmd(battle, battle.getNextObjectToMove(), null);
	        serverProxy._processCommandResponse();
	        serverProxy.battleGetNextObject(battle, null, null);
	        serverProxy._processCommandResponse();
        }

        assertEquals(true, battle.hasEnded());
        BattleEndEvent endEvent = (BattleEndEvent) getEvent(BattleEndEvent.class);
        
        assertEquals(null, bh.getOrbital());
        assertNull(endEvent.getCraftDestroyd()); // battle stations do not produce wreckage
        assertEquals(null, blackHole.getStellarObjectOrbital());
        
        CraftDelegate[] crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(player1), new CraftDelegate[3], "TestFighter1", "TestFighter2", "TestFighter3");
        
    }


    /**
     * 
     */
    public void testDestroyMiningColony() throws Exception {
        Logger.logDebug("*******************************  WarTest.testDestroyMiningColony() *************************");
        createMap();

        SolarSystem ssm = newSolarSystem("SSM", 50, 50);
        Planet m = newPlanet("SSM-A", ssm, null, 40, 0.2);
        m.setProductionBonus(1.0);
        m.setMoneyBonus(20);
        m.setResearchBonus(1.0);
        m.deploy(h2, new Orbital(tech_spaceStation, null, new Modules(tech_habitatI, tech_oreExtractor, tech_researchModule)));
        
        MotEngine.getInstance().getRelations().setWar(p1, p2);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        MotSolarSystemDelegate ssdm = (MotSolarSystemDelegate) DeligateCacheManager.getStellarObject("SSM");
        MotSolarSystemDelegate ssd2 = (MotSolarSystemDelegate) DeligateCacheManager.getStellarObject("SS2");
        MotPlanetDelegate mine = (MotPlanetDelegate) DeligateCacheManager.getPlanet("SSM-A");
        MotPlanetDelegate planet2 = (MotPlanetDelegate) DeligateCacheManager.getPlanet("SS2-A");
        
        assertEquals(tech_spaceStation, mine.getOrbital().getHullTech());
        assertEquals(player2, mine.getOwner());
        
        assertEquals(mine, planet2.getMiningColonyDelegate());
        assertEquals(10, planet2.getMiningPopulation());
        assertEquals(40, planet2.getMiningProduction());
        assertEquals(60, planet2.getMiningResearch());
        assertEquals(20, planet2.getMiningMoney());
        
        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer200, tech_fusion400, tech_electronic40, tech_ancientWeapon, tech_ancientWeapon, tech_ancientWeapon, tech_ancientWeapon, tech_armour200, tech_energy20, tech_shieldModulatorII);
        Craft c1 = newCraft(cd, p1, m);
        Craft c2 = newCraft(cd, p1, m);
        Craft c3 = newCraft(cd, p1, m);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();

        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter2");
        CraftDelegate craft3 = DeligateCacheManager.getCraft("TestFighter3");

        serverProxy.craftAttackCmd(craft1, mine, null, null);
        serverProxy._processCommandResponse();
        serverProxy.craftAttackCmd(craft2, mine, null, null);
        serverProxy._processCommandResponse();
        serverProxy.craftAttackCmd(craft3, mine, null, null);
        serverProxy._processCommandResponse();

		clearEvents();
        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft [] bca = MotEngine.instance.getBattles().getActiveBattle().getCrafts(); 

        BattleCraftDelegate[] visibleCrafts = battle.getCrafts();
        assertEquals(4, visibleCrafts.length);
        assertEquals("BattleStation1", visibleCrafts[0].getName());
        assertEquals(c1.getName(), visibleCrafts[1].getName());
        assertEquals(c2.getName(), visibleCrafts[2].getName());
        assertEquals(c3.getName(), visibleCrafts[3].getName());
        
        serverProxy.battleGetNextObject(battle, null, null);
        serverProxy._processCommandResponse();
        while (!battle.hasEnded()) {
	        serverProxy.battleGetNextOrdersCmd(battle, battle.getNextObjectToMove(), null);
	        serverProxy._processCommandResponse();
	        serverProxy.battleGetNextObject(battle, null, null);
	        serverProxy._processCommandResponse();
        }

        assertEquals(true, battle.hasEnded());
        BattleEndEvent endEvent = (BattleEndEvent) getEvent(BattleEndEvent.class);
        
        assertEquals(null, ssm.getOwners());
        assertEquals(null, ssdm.getOwnerDelegates());
        assertEquals(null, mine.getOrigin());
        assertEquals(null, mine.getOwner());
        assertEquals(null, mine.getOrbital());
        
        assertEquals(p2.getName(), ssd2.getOwners()[0]);
        assertEquals(player2, planet2.getOwner());        
        assertNull(planet2.getMiningColonyDelegate());
        assertEquals(0, planet2.getMiningPopulation());
        assertEquals(0, planet2.getMiningProduction());
        assertEquals(0, planet2.getMiningResearch());
        assertEquals(0, planet2.getMiningMoney());
        
    }


    /**
     */
    public void testCraftMoveToActualLocation() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testCraftMoveToActualLocation() *************************");
        createMap();

        MotEngine.getInstance().getRelations().setWar(p1, p2);
        h1.setWeapons(new Weapons(tech_massDriver2));
        Craft c1 = newCraft(fighterDesign, p1, ss2);
        Craft c2 = newCraft(fighterDesign, p2, ss2);

        serverProxy.updateDelegates(null);
        serverProxy._processCommandResponse();
        
        StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS2");
        PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS2-A");
        CraftDelegate craft1 = DeligateCacheManager.getCraft("TestFighter1");
        CraftDelegate craft2 = DeligateCacheManager.getCraft("TestFighter2");
        
        //
        // Test Computer Player craft attacking User Player planet produces a NewBattleEvent
        //
        
        c1.attack(h2);

        serverProxy.endTurnCmd(callbackSync, null);
        serverProxy._processCommandResponse();
        
        NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
        assertNotNull(nbe);
        
        Battle b = MotEngine.instance.getBattles().getActiveBattle();
        BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());
        
        BattleCraft bc1 = b.getCrafts()[0];
        BattleCraft bc2 = b.getCrafts()[1];
        BattlePlanet bp = b.getPlanet();

        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toBattleWindow(battle);
        
        // P = Planet, C = Other Craft, F = From , T = To
        //
        //    PC T <----- F
        //
        //
        
        bp.setLocation(new Location(100,500));
        bc2.setLocation(new Location(165,500));
        bc1.setLocation(new Location(300,500));
        bc1.setDirection(270);

        b.sendStateUpdateEvents(p1, true, true, true);
        serverProxy._processQueuedEvents();
        
        BattleCraftDelegate[] battleCrafts = (BattleCraftDelegate[])order(battle.getCrafts(), new BattleCraftDelegate[2], "TestFighter1", "TestFighter2");
        BattleCraftDelegate battleCraft1 = battleCrafts[0];
        BattleCraftDelegate battleCraft2 = battleCrafts[1];

        ServerProxy proxy = ProxyStore.getInstance();
        
		LocationAndDirection t = proxy.battleFetchCraftMoveActualLocation(battle, battleCraft1, new LocationAndDirection(150, 500, 270), battleCraft2);
        assertEquals(200,500,270, t);
        assertEquals(300,500, battleCraft1.getLocation());
        
        // P = Planet, C = Other Craft, F = From , T = To
        //
        //     T
        //    PCT    F
        //
        //
        
        bc1.moveTo(new LocationAndDirection(200, 500, 270));
        serverProxy._processQueuedEvents();
        
        assertEquals(200,500, battle.getDelegateCache().getBattleSandbox().getBattleCraft("TestFighter1").getLocation());
        assertEquals(0.0, battle.getDelegateCache().getBattleSandbox().getBattleCraft("TestFighter1").getMovementDamageAsPercent());

        bc1.setMoveRemaining(1.0);
        MotEngine.instance.getBattles().getActiveBattle().sendStateUpdateEvents(p1, true, true, true);
        serverProxy._processQueuedEvents();

        assertEquals(1.0, battle.getDelegateCache().getBattleCraft("TestFighter1").getMoveRemainingAsPercent());
        assertEquals(1.0, battle.getDelegateCache().getBattleSandbox().getBattleCraft("TestFighter1").getMoveRemaining());
        
		t = proxy.battleFetchCraftMoveActualLocation(battle, battleCraft1, new LocationAndDirection(170, 500, 270), battleCraft2);
        assertEquals(190,500,270, t);
        
		t = proxy.battleFetchCraftMoveActualLocation(battle, battleCraft1, new LocationAndDirection(165, 495, 180), battleCraft2);
        assertEquals(166,475,226, t);
        
        assertEquals(200,500, battleCraft1.getLocation());
        
        // Check speed is updated from client and includes battle bonuses and damage
        //
        //    P    T <-------- F
        //
        //
        
        CraftDesign cd = new CraftDesign("TestCommandShip",  tech_flyer30, tech_fusion25, tech_massDriver1, tech_commandModule);
		BattleCraft _commandCraft = b.newCraft(p1, cd, new Location(900,900));
        bc1.setMoveRemaining(1.0);
        bc1.setLocation(new Location(600,500));
        bc1.setDirection(270);
        bc2.setMoveRemaining(0.0);
        bc2.dischargeWeapons();
        bc2.setLocation(new Location(600,450));
        bc2.setDirection(180);
		MotEngine.instance.broadcast(new BattleCraftNewEF(_commandCraft, false));

        MotEngine.instance.getBattles().getActiveBattle().sendStateUpdateEvents(p1, true, true, true);
        serverProxy._processQueuedEvents();
        BattleCraftDelegate commandCraft = battle.getDelegateCache().getBattleCraft("TestCommandShip1");
        
        LocationAndDirection moveToDest = new LocationAndDirection(200, 500, 270);
        LocationAndDirection a;
        
        //assertEquals(110, battleCraft1.getSpeed());
        assertEquals(bc1.getSpeed(), battleCraft1.getSpeed());
        assertEquals(bc1.getTurningSpeed(), battleCraft1.getTurningSpeed());
		t = proxy.battleFetchCraftMoveActualLocation(battle, battleCraft1, moveToDest, null);
		a = bc1.getCraftMoveActualLocation(moveToDest, null, null);
		assertEquals(a, t);
		
		_commandCraft.setMoveRemaining(1.0);
		_commandCraft.rechargeWeapons();
		serverProxy.battleRetreatCmd(battle, commandCraft, null, null);
        serverProxy._processCommandResponse();
        
        //assertEquals(100, battleCraft1.getSpeed());
        assertEquals(bc1.getSpeed(), battleCraft1.getSpeed());
        assertEquals(bc1.getTurningSpeed(), battleCraft1.getTurningSpeed());
		t = proxy.battleFetchCraftMoveActualLocation(battle, battleCraft1, moveToDest, null);
		a = bc1.getCraftMoveActualLocation(moveToDest, null, null);
		assertEquals(a, t);
        
        bc2.rechargeWeapons();
        bc1.getCraft().setDamage(bc1.getCraft().getMaxDamage()/2);
		bc2.attack(bc1, null, -1);
		serverProxy.battleCraftMoveToCmd(battle, battleCraft1, new LocationAndDirection(590, 500, 270), null, false, false, null, null);
        serverProxy._processCommandResponse();
        
        assertEquals(0.9, battleCraft1.getMoveRemainingAsPercent(), 0.1);
        assertEquals(0.59, battleCraft1.getState().getDamagePercent(), 0.1);
        //assertEquals(100, battleCraft1.getSpeed());
        assertEquals(bc1.getSpeed(), battleCraft1.getSpeed());
        assertEquals(bc1.getTurningSpeed(), battleCraft1.getTurningSpeed());
		t = proxy.battleFetchCraftMoveActualLocation(battle, battleCraft1, moveToDest, null);
		a = bc1.getCraftMoveActualLocation(moveToDest, null, null);
		assertEquals(a, t);
        
    }
    


    
    
    
    private MotEvent lastEvent;
    
    private BattleDelegate battle;
    
    /**
     * This process events coming directly from the server.
     */
	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(p1));
    }
    
    /**
     * This process events coming directly from the server.
     */
	public void notify(MotEvent e) throws ApplicationError {
        Logger.logDebug("DeligateTest.notify() EVENT "+e.toString());
        lastEvent = e;
        eventList.add(e);
    }
    
    /**
     * This process events coming from the server connection.
     */
	public void process(MotEvent e, LinkedEventBase parent) {
        Logger.logDebug("DeligateTest.process() EVENT "+e.toString());
        lastEvent = e;
        eventList.add(e);
    }
    
    protected void clearEvents() {
        Logger.logDebug("DeligateTest.notify() Clear Events ");
        eventList.clear();
        lastEvent = null;
    }

    private Player owner;    
	public Player getOwner() {
        return owner;
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}