/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.ComputerPlayerDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.event.MotEventConsumer;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.LocalServerProxy;
import mot.client.proxy.MotServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.TestGalaxyWindowConfig;
import mot.common.event.LinkedEventBase;
import mot.common.event.MotEvent;
import mot.common.event.StateContainer;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.common.CompletedServerCallbackSyncroniser;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;

/**
 *
 */

public class GalaxyEditorTest extends TestBase implements MotEventConsumer, ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(mot.test.GalaxyEditorTest.class);
        return suite;
    }

    public GalaxyEditorTest(String name) {
        super(name);
    }
    
    private UserPlayer p1,p4;
    private Player p2, p3;
    private SolarSystem ss1, ss2, ss3, ss4;
    private Planet h1, h2, h3, h4, h3b;

    private MotServerProxy serverProxy;
    private UserPlayerDelegate player1;
    private ComputerPlayerDelegate player2, player3;

	private CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();
    
    protected void setUp() throws ApplicationError, ValidationError {
        Logger.setDebug();
        MotEngine.newInstance();
        DeligateCacheManager.flush();
        setupConfigs();
        MainWindow.__createTestMainWindow(new TestGalaxyWindowConfig());
       	new ProxyStore(new MotServerProxy(true, MainWindow.getExestingInstance(), this));
        MotEngine.getInstance().getProxyManager().add(this);
    }
    
    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }
    
    public void createMap() throws Exception {
    	DeligateCacheManager.flush();
    	
    	setupOneTierTechTree();
    	
        String [] playerNames = { "Humans", "Bears" };
        RaceOptions options = OptionsFactory.getRaceOptions(playerNames[0]);
        p1 = newUserPlayer(playerNames[0], blue, null, options);
        p2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        h1 = newPlanet("SS1-A", ss1, p1);

        ss2 = newSolarSystem("SS2", 200, 100);
        h2 = newPlanet("SS2-A", ss2, p2);
        
        ss3 = newSolarSystem("SS3", 150, 100);
        h3 = newPlanet("SS3-A", ss3, null, 100, 0.9);
        
        ss4 = newSolarSystem("SS4", 50, 100);
        
        owner = p2;

        serverProxy = (MotServerProxy) ProxyStore.getInstance();
        CompletedServerCallbackSyncroniser callback = new CompletedServerCallbackSyncroniser();
        serverProxy.register("Humans", true, callback, null);
        callback.waitTillCompleteAndProcessCommandResponse(serverProxy);
        
        player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Humans");
        player2 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Bears");
        
        MainWindow.getExestingInstance()._setWindowOwner(player1);
        MainWindow.getExestingInstance().toGalaxyWindow();
        
        assertEquals("Player Human is correct", "Humans", player1.getName());
        assertEquals("Player Bears is correct", "Bears", player2.getName());

        PlayerDelegate [] players = (PlayerDelegate[]) order(DeligateCacheManager.getPlayers(), new PlayerDelegate [2], "Humans", "Bears");
    }

    /**
     * Test create new game works and that the following factions work in a game with
     * two players (one computer and one human);
     */
    public void testPlanetUpdate() throws Exception {
        Logger.logDebug("*******************************  DeligateTest.testPlanetUpdate() *************************");
        createMap();
        
        serverProxy = (MotServerProxy) ProxyStore.getInstance();
        
		LocalServerProxy localDelegate = serverProxy.getLocalServerProxy();
		
		assertNotNull(localDelegate.getPlanet("SS1-A"));
		assertNotNull(localDelegate.getSolarSystem("SS1"));
		assertNotNull(localDelegate.getStellarObject("SS1"));
		assertNotNull(localDelegate.getPlayer("Humans"));

    }
    
    private MotEvent lastEvent;
    
    private BattleDelegate battle;
    
    /**
     * This process events coming directly from the server.
     */
	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(p1));
    }
    
    /**
     * This process events coming directly from the server.
     */
	public void notify(MotEvent e) throws ApplicationError {
        Logger.logDebug("DeligateTest.notify() EVENT "+e.toString());
		if (e instanceof StateContainer) {
			DeligateCacheManager.update((StateContainer)e);
		}
        lastEvent = e;
        eventList.add(e);
    }
    
    /**
     * This process events coming from the server connection.
     */
	public void process(MotEvent e, LinkedEventBase parent) {
        Logger.logDebug("DeligateTest.process() EVENT "+e.toString());
        lastEvent = e;
        eventList.add(e);
    }
    
    protected void clearEvents() {
        Logger.logDebug("DeligateTest.notify() Clear Events ");
        eventList.clear();
        lastEvent = null;
    }

    private Player owner;    
	public Player getOwner() {
        return owner;
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
        if (!ProxyStore.hasInstance()) {
        	try {
				ProxyStore.getInstance().disconnect();
			} catch (ApplicationError e) {
				Logger.logError("", e);
			}
        }
    }

}