/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.GalaxyDetails;
import mot.common.bean.Location;
import mot.common.bean.Nebula;
import mot.common.bean.RaceGraphics;
import mot.common.bean.RaceGraphicsList;
import mot.common.bean.Size;
import mot.common.factory.GraphicsFactory;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Scanners;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.command.RenameCmd;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.planets.BlackHole;
import mot.engine.planets.BlackHoleFactory;
import mot.engine.planets.GalacticIncumbent;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.StellarObjects;
import mot.engine.players.Player;
import mot.engine.setup.AncientFactory;
import mot.engine.setup.MotFactory;

public class GalaxyTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        Logger.setFull();
        TestSuite suite = new TestSuite(GalaxyTest.class);
        return suite;
    }
    
    public GalaxyTest(String name) {
        super(name);
    }
    
    private Player player;
    
    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    
    private Planet planet1a;
    private Planet planet3a;
    private Planet planet3b;
    
    private void createMap(SetupOptions optionsRace) {
        try {          
            String [] playerNames = new String [] { "Humans" };
            SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
            RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
            player = newPlayer(playerNames[0], blue, options, playerNames);
            
            ss1 = newSolarSystem("SS1", 100, 100);
            ss2 = newSolarSystem("SS2", 150, 150);
            ss3 = newSolarSystem("SS3", 150, 100);
    
            planet1a = newPlanet("SS1-A", ss1, player);
            planet3a = newPlanet("SS3-A", ss3, null, 100, 0.8);
            planet3b = newPlanet("SS3-B", ss3, null, 100, 0.8);
       } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }
    
    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        
        Logger.setInMemory(false);
        Logger.setDebug();
    }
    
    /**
     * Basics of planets and orbits.
     */
    public void testBasics() throws Exception {
        Logger.logDebug("*******************************  GalaxyTest.testBasics() *************************");
        
        createMap(null);
        
        MotEngine engine = MotEngine.getInstance();
		assertEquals(3, engine.getStellarObjects().getQty());
        assertEquals(3, engine.getStellarObjects().getStellarObjects().length);
        assertEquals(ss1, engine.getStellarObjects().getStellarObjects()[0]);
        assertEquals(ss2, engine.getStellarObjects().getStellarObjects()[1]);
        assertEquals(ss3, engine.getStellarObjects().getStellarObjects()[2]);

        assertEquals(3, engine.getStellarObjects().getSolarSystems().length);
        assertEquals(ss1, engine.getStellarObjects().getSolarSystems()[0]);
        assertEquals(ss2, engine.getStellarObjects().getSolarSystems()[1]);
        assertEquals(ss3, engine.getStellarObjects().getSolarSystems()[2]);
        
        SolarSystem ss4 = newSolarSystem("SS4", 300, 100);

        assertEquals(4, engine.getStellarObjects().getQty());
        assertEquals(4, engine.getStellarObjects().getStellarObjects().length);
        assertEquals(ss1, engine.getStellarObjects().getStellarObjects()[0]);
        assertEquals(ss2, engine.getStellarObjects().getStellarObjects()[1]);
        assertEquals(ss3, engine.getStellarObjects().getStellarObjects()[2]);
        assertEquals(ss4, engine.getStellarObjects().getStellarObjects()[3]);

        assertEquals(4, engine.getStellarObjects().getSolarSystems().length);
        assertEquals(ss1, engine.getStellarObjects().getSolarSystems()[0]);
        assertEquals(ss2, engine.getStellarObjects().getSolarSystems()[1]);
        assertEquals(ss3, engine.getStellarObjects().getStellarObjects()[2]);
        assertEquals(ss4, engine.getStellarObjects().getSolarSystems()[3]);

        assertEquals(ss1, engine.getStellarObjects().getStellarObject("SS1"));
        assertEquals(ss4, engine.getStellarObjects().getStellarObject("SS4"));
        assertEquals(null, engine.getStellarObjects().getStellarObject("xx"));

        assertEquals(ss1, engine.getStellarObjects().getStellarObjectAt(100, 100));
        assertEquals(null, engine.getStellarObjects().getStellarObjectAt(10, 100));

        assertEquals(ss1, engine.getStellarObjects().getClosestStellarObject(new Location(110,110)));
        assertEquals(ss3, engine.getStellarObjects().getClosestStellarObject(new Location(140,110)));
        
        assertEquals(3, engine.getPlanets().getPlanets().length);
        assertEquals(planet1a, engine.getPlanets().getPlanets()[0]);
        assertEquals(planet3a, engine.getPlanets().getPlanets()[1]);
        assertEquals(planet3b, engine.getPlanets().getPlanets()[2]);
        
        assertEquals(planet1a, engine.getStellarObjects().getSolarSystems()[0].getPlanets()[0]);
        assertEquals(planet3a, engine.getStellarObjects().getSolarSystems()[2].getPlanets()[0]);
        assertEquals(planet3b, engine.getStellarObjects().getSolarSystems()[2].getPlanets()[1]);
        
        Planet planet3c = newPlanet("SS3-C", ss3, null, 100, 0.8);

        assertEquals(4, engine.getPlanets().getPlanets().length);
        assertEquals(planet1a, engine.getPlanets().getPlanets()[0]);
        assertEquals(planet3a, engine.getPlanets().getPlanets()[1]);
        assertEquals(planet3b, engine.getPlanets().getPlanets()[2]);
        assertEquals(planet3c, engine.getPlanets().getPlanets()[3]);
        
        assertEquals(planet3a, ss3.getPlanets()[0]);
        assertEquals(planet3b, ss3.getPlanets()[1]);
        assertEquals(planet3c, ss3.getPlanets()[2]);

        assertEquals(planet1a, engine.getPlanets().getPlanet("SS1-A"));
        assertEquals(planet3c, engine.getPlanets().getPlanet("SS3-C"));

        assertEquals(1, engine.getPlanets().getPlanetsOwnedBy(player).length);
        assertEquals(planet1a, engine.getPlanets().getPlanetsOwnedBy(player)[0]);

        assertEquals(1, engine.getPlanets().getColonies().length);
        assertEquals(planet1a, engine.getPlanets().getColonies()[0]);
    }
    
    /**
     * Basics of planets and orbits.
     */
    public void testChangeName() throws Exception {
        Logger.logDebug("*******************************  GalaxyTest.testChangeName() *************************");
        
        createMap(null);
        
        MotEngine engine = MotEngine.getInstance();
        
        planet1a.changeNameTo("Ireland");

        assertEquals(planet1a, engine.getPlanets().getPlanet("Ireland"));
        assertEquals(planet1a, engine.getPlanets().getPlanet("SS1-A"));
        
        ss3.changeNameTo("Best");
        
        assertEquals(ss3, engine.getStellarObjects().getStellarObject("Best"));
        assertEquals(ss3, engine.getStellarObjects().getStellarObject("SS3"));
        assertEquals(planet3a, engine.getPlanets().getPlanet("SS3-A"));
        assertEquals(null, engine.getPlanets().getPlanet("Best-A"));

        Craft craft = CraftFactory.getCraftInstance(coloniserDesign, player, player, ss3.getOrbit(), false, false);
        planet3a.colonise(craft);
        
        assertEquals(2, engine.getPlanets().getColonies().length);
        assertEquals(planet1a, engine.getPlanets().getColonies()[0]);
        assertEquals(planet3a, engine.getPlanets().getColonies()[1]);
        
        assertEquals(2, engine.getPlanets().getColonies().length);
        assertEquals(planet1a, player.getPlanets()[0]);
        assertEquals(planet3a, player.getPlanets()[1]);
        
        new RenameCmd(ss3, "Better", player).run();
        
        assertEquals(ss3, engine.getStellarObjects().getStellarObject("Better"));
        assertEquals(ss3, engine.getStellarObjects().getStellarObject("Best"));
        assertEquals(ss3, engine.getStellarObjects().getStellarObject("SS3"));
        assertEquals(planet3a, engine.getPlanets().getPlanet("SS3-A"));
        assertEquals(planet3a, engine.getPlanets().getPlanet("Better-A"));
    }
    
    public void testChangeName2() throws Exception {
        Logger.logDebug("*******************************  GalaxyTest.testChangeName2() *************************");
        
        createMap(null);
        
        MotEngine engine = MotEngine.getInstance();
        new RenameCmd(ss1, "Start", player).run();
        
        assertEquals(ss1, engine.getStellarObjects().getStellarObject("Start"));
        assertEquals(ss1, engine.getStellarObjects().getStellarObject("SS1"));
        assertEquals(planet1a, engine.getPlanets().getPlanet("SS1-A"));
        assertEquals(planet1a, engine.getPlanets().getPlanet("Start-A"));
    }

    /**
     * Test factory creates them and that they move across the galaxy.
     */
    public void testNebula() throws Exception {
        Logger.logDebug("*******************************  GalaxyTest.testNebula() *************************");
        createMap(null);
        
        MotEngine engine = MotEngine.getInstance();
        StellarObjects sos = engine.getStellarObjects();
		assertEquals(new Size(500,500), sos.getGalaxyDetails().getSize());
        assertEquals(new Size(800,800), Nebula._getNedulaGraphics().getSize());

        Nebula nebula = MotFactory.createNebula(new Location(140,100), 25);
        
        assertEquals(nebula, sos.getNebula("Nebula1"));
        assertEquals(1, sos.getNebulae().length);
        assertEquals(nebula, sos.getNebulae()[0]);
        
        assertEquals(false, ss1.isInNebula());
        assertEquals(false, ss2.isInNebula());
        assertEquals(true, ss3.isInNebula());
    }
    
    /**
     * Test factory creates them and that they move across the galaxy.
     */
    public void testBlackHole() throws Exception {
        Logger.logDebug("*******************************  GalaxyTest.testBlackHole() *************************");
        createMap(null);

        GalaxyDetails gd = MotEngine.getInstance().getStellarObjectsNotNull().getGalaxyDetails();

        MotEngine engine = MotEngine.getInstance();
		assertEquals(3, engine.getStellarObjects().getQty());
        assertEquals(3, engine.getStellarObjects().getStellarObjects().length);
        assertEquals(ss1, engine.getStellarObjects().getStellarObjects()[0]);
        assertEquals(ss2, engine.getStellarObjects().getStellarObjects()[1]);
        assertEquals(ss3, engine.getStellarObjects().getStellarObjects()[2]);

        assertEquals(3, engine.getStellarObjects().getSolarSystems().length);
        assertEquals(ss1, engine.getStellarObjects().getSolarSystems()[0]);
        assertEquals(ss2, engine.getStellarObjects().getSolarSystems()[1]);
        assertEquals(ss3, engine.getStellarObjects().getSolarSystems()[2]);
        
        BlackHole blackHole1 = BlackHoleFactory.createBlackHole(new Location(100,150));
        
        assertEquals(4, engine.getStellarObjects().getQty());
        assertEquals(4, engine.getStellarObjects().getStellarObjects().length);
        assertEquals(ss1, engine.getStellarObjects().getStellarObjects()[0]);
        assertEquals(ss2, engine.getStellarObjects().getStellarObjects()[1]);
        assertEquals(ss3, engine.getStellarObjects().getStellarObjects()[2]);
        assertEquals(blackHole1, engine.getStellarObjects().getStellarObjects()[3]);

        assertEquals(3, engine.getStellarObjects().getSolarSystems().length);
        assertEquals(ss1, engine.getStellarObjects().getSolarSystems()[0]);
        assertEquals(ss2, engine.getStellarObjects().getSolarSystems()[1]);
        assertEquals(ss3, engine.getStellarObjects().getSolarSystems()[2]);
        
        assertEquals(1, engine.getStellarObjects().getBlackHoles().length);
        assertEquals(blackHole1, engine.getStellarObjects().getBlackHoles()[0]);
        
    }
    
    /**
     * 
     */
    public void testGalacticLore() throws Exception {
        Logger.logDebug("*******************************  GalaxyTest.testGalacticLore() *************************");
        createMap(null);

        GalaxyDetails gd = MotEngine.getInstance().getStellarObjectsNotNull().getGalaxyDetails();
        planet1a.setScanners(new Scanners(tech_photon200));

        SolarSystem ss4 = newSolarSystem("SS4", 500, 100);
        Planet planet4 = newPlanet("SS4-A", ss4, null, 100, 0.8);

        SolarSystem ss5 = newSolarSystem("SS5", 500, 200);
        Planet planet5 = newPlanet("SS5-A", ss5, null, 140, 1.0);
        
	    RaceGraphicsList graphicsList = GraphicsFactory.getRaceGraphicsList();
        RaceGraphics [] ancients = graphicsList.getGraphicsType(RaceGraphics.TYPE_ANCIENT);
        AncientFactory.createAncient(planet5, ancients[0]);
        
        assertEquals(true, ss3.isVisible(player));
        assertEquals(false, planet3a.isViewableBy(player));
        assertEquals(false, ss4.isVisible(player));
        assertEquals(false, planet4.isViewableBy(player));
        assertEquals(false, ss5.isVisible(player));
        assertEquals(false, planet5.isViewableBy(player));

        assertEquals(1, ss5.getPrizes().length);
        
        player.addTechnology(Technologies.getMiscTechnologies().getGalacticLoreTechnology());
       
        assertEquals(true, ss3.isVisible(player));
        assertEquals(false, planet3a.isViewableBy(player));
        assertEquals(true, ss4.isVisible(player));
        assertEquals(false, planet4.isViewableBy(player));
        assertEquals(true, ss5.isVisible(player));
        assertEquals(false, planet5.isViewableBy(player));

        assertEquals(1, ss5.getPrizes().length);
        

    }
    
    public void testNoIncumbents() throws Exception {
        Logger.logDebug("*******************************  GalaxyTest.testNoIncumbents() *************************");
        createMap(null);
        
        MotEngine engine = MotEngine.getInstance();
        StellarObjects sos = engine.getStellarObjects();
        GalaxyDetails gd = engine.getStellarObjects().getGalaxyDetails();
        gd.setSize(new Size(900,900));
        engine.getStellarObjects().setBlackHole(new BlackHole("BlackHole1", 600,600, 0.1f, false));
        engine.getStellarObjects().setNebula(new Nebula("Nebula1", 100,100, 101));
        
        assertNull(sos.getGalacticIncumbentsBetween(new Location(100,100), new Location(125,100)));
        
        BlackHole blackHole2 = new BlackHole("BlackHole2", 115,100, 0.1f, false);
		engine.getStellarObjects().setBlackHole(blackHole2);
        
        GalacticIncumbent[] gis = sos.getGalacticIncumbentsBetween(new Location(100,100), new Location(125,100));
        
        //assertEquals(1, gis.length);
        //assertEquals(blackHole2, gis[0]);
    }
    
    protected void tearDown() {
        
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

    
}