/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.GameConstants;
import mot.common.bean.Location;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Hull;
import mot.common.technology.Shields;
import mot.common.technology.Technologies;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.crafts.Fleet;
import mot.engine.planets.BlackHole;
import mot.engine.planets.BlackHoleFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.UserPlayer;
import mot.engine.players.generals.CraftTask;
import mot.engine.players.generals.CraftTaskAttack;
import mot.engine.players.generals.CraftTaskBombard;
import mot.engine.players.generals.CraftTaskColonise;
import mot.engine.players.generals.CraftTaskDefend;
import mot.engine.players.generals.CraftTaskIdle;
import mot.engine.players.generals.CraftTaskProtect;
import mot.engine.players.generals.CraftTaskSteelColony;
import mot.engine.players.generals.General;
import mot.engine.players.generals.GeneralBasic;
import mot.engine.prizes.CraftPrize;
import mot.engine.prizes.MoneyPrize;
import mot.engine.prizes.TechnologyPrize;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskInvade;

/**
 * Tests that a craft moves between two solar system correctly.
 * Test the General objects.
 */
public class GeneralTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(GeneralTest.class);
        return suite;
    }

    public GeneralTest(String name) {
        super(name);
    }
    
    private SetupOptions options;
    
    private ComputerPlayer player;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;

    
    protected void setUp() {
    }
    
    private ComputerPlayer enemyPlayer;
    private SolarSystem enemySS1;
    private Planet enemyPlanet1;
    private Planet enemyPlanet2;

	private BlackHole blackHole;
	private BlackHole wormHole;

    protected void createMap() throws ApplicationError, ValidationError {
    	Utils.clearNextRandomInt();
    	MotEngine.newInstance();
        Logger.setDebug();
        setupConfigs();
        
        String [] playerNames = { "TestPlayer1" };
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player = newPlayer(playerNames[0], blue, options, playerNames);
        
        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, null, 100, 0.9);

        ss3 = newSolarSystem("SS3", 150, 100);
        planet3 = newPlanet("SS3-A", ss3, null, 100, 0.7);

        ss4 = newSolarSystem("SS4", 500, 100);
        planet4 = newPlanet("SS4-A", ss4, null, 110, 1.0);
    }

    private void addEnemy(int x, int y) throws ApplicationError, ValidationError {
        String [] players = { "TestPlayer2" };
        enemyPlayer = newPlayer(players[0], green, options, players);
        enemySS1 = newSolarSystem("SSE", x, y);
        enemyPlanet1 = newPlanet("SSE-A", enemySS1, enemyPlayer);
        enemyPlanet1.setWeapons(new Weapons());
        enemyPlanet1.getWeapons().add(tech_massDriver1);
        enemyPlanet1.setShields(new Shields(tech_energy10));
    }

    private void addEnemyMultiple() throws ApplicationError, ValidationError {
        String [] players = { "TestPlayer2" };
        enemyPlayer = newPlayer(players[0], green, options, players);
        enemySS1 = newSolarSystem("SSE", 100, 140);
        enemyPlanet1 = newPlanet("SSE-A", enemySS1, enemyPlayer);
        enemyPlanet1.setWeapons(new Weapons());
        enemyPlanet1.getWeapons().add(tech_massDriver1);
        enemyPlanet1.setShields(new Shields(tech_energy10));
        enemyPlanet2 = newPlanet("SSE-B", enemySS1, enemyPlayer, 110, 0.9);
        blackHole = BlackHoleFactory.createBlackHole(new Location(100,200));
        blackHole.deploy(new Orbital(tech_spaceStation, null, null), enemyPlayer);
    }

    private void addEnemyMultipleWithWormHole() throws ApplicationError, ValidationError {
        String [] players = { "TestPlayer2" };
        enemyPlayer = newPlayer(players[0], green, options, players);
        enemySS1 = newSolarSystem("SSE", 100, 140);
        enemyPlanet1 = newPlanet("SSE-A", enemySS1, enemyPlayer);
        enemyPlanet1.setWeapons(new Weapons());
        enemyPlanet1.getWeapons().add(tech_massDriver1);
        enemyPlanet1.setShields(new Shields(tech_energy10));
        blackHole = BlackHoleFactory.createBlackHole(new Location(100,200));
        blackHole.deploy(new Orbital(tech_spaceStation, null, null), enemyPlayer);
        wormHole = BlackHoleFactory.createWormHole(new Location(100,225));
        wormHole.deploy(new Orbital(tech_spaceStation, null, null), enemyPlayer);
    }
    
    /**
     * Test that crafts move at the right speed between stellar objects on the galaxy map
     * and that speed is effected by player and craft special bonuses.
     * Tests that prizes are released when a craft enters a solar system.
     */
    public void testExplore() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testExplore() *************************");
        createMap();
        
        SolarSystem ss5 = newSolarSystem("SS5", 210, 100);
        ss4.setLocation(new Location(300, 100));
        
		CraftDesign explorerDesign = new CraftDesign("TestExplorer", tech_flyer30, tech_fusion25);
        Craft craft1 = newCraft(explorerDesign, player, ss1);
        Craft craft2 = CraftFactory.create("monster_small", MotEngine.instance.getMonsterPlayer(), MotEngine.instance.getMonsterPlayer(), planet2.getOrbit(), true, false);
        ss5.setPrize(new CraftPrize());
        ss5.setPrize(new TechnologyPrize(tech_energy10));
        ss5.setPrize(new MoneyPrize(1000));

        General general = player.getGeneral();

        //
        // ON THIS MAP SS2 will not be explored as it has a craft in orbit
        // ON THIS MAP SS4 will not be explored as it is out of range
        // SHOULD EXPLORE SS3 THEN SS5 - AFTER THAT EXPLORE TASK SHOULD COMPLETE.
        //

        assertFalse(player.canTravelTo(ss4));
        assertFalse(craft1.canTravelTo(ss4));

        MotEngine.instance.endTurn(player);
        player.setFunds(1000);
        //MotEngine.instance.logSummary();

        assertEquals("craft1 has not visited ss3", -1, ss3.getLastVisitedDate(player));
        assertEquals("Have correct tasks", mot.engine.players.generals.CraftTaskExplore.class, craft1.getTask().getClass());
        assertNull("craft1 going to explore ss3", craft1.getOrbit());
        assertEquals("craft1 going to explore ss3", ss3, craft1.getNavigator().getDestination().getStellarObject());

        general.endTurn();
        craft1.arrivedAt(ss3.getOrbit());
        craft1.endTurn();
        MotEngine.instance.logSummary();

        assertEquals("craft1 has visited ss3", MotEngine.instance.getDate(), ss3.getLastVisitedDate(player));
        assertEquals("craft1 has not visited ss5", -1, ss5.getLastVisitedDate(player));
        assertEquals("Have correct tasks", mot.engine.players.generals.CraftTaskExplore.class, craft1.getTask().getClass());
        assertNull("craft1 going to explore ss5", craft1.getOrbit());
        assertEquals("craft1 going to explore ss5", ss5, craft1.getNavigator().getDestination().getStellarObject());

        craft1.arrivedAt(ss5.getOrbit());
        craft1.endTurn();
        MotEngine.instance.logSummary();

        // Check does not explore further as ss4 is out of range
        assertEquals("craft1 has visited ss5", MotEngine.instance.getDate(), ss5.getLastVisitedDate(player));
        assertEquals("Have correct tasks", CraftTaskIdle.class, craft1.getTask().getClass());
        assertEquals("craft1 has nothing to explore", ss5, craft1.getOrbit().getStellarObject());
        assertEquals("craft1 has nothing to explore", null, craft1.getNavigator());
        
        // Have received prizes
        assertEquals("Have received funds prize", 2000, player.getFunds());
        assertEquals("Have received tech prize", true, player.hasDiscovered(tech_energy10));
        assertEquals("Have received craft prize", 2, player.getQtyCrafts());
                
        // Test that craft range is extended using an outpost
        Hull outpostHull = Technologies.getHull(TestTechs.OUTPOST);
        Orbital outpost = new Orbital(outpostHull, null, null);
        ss3.deploy(outpost, player);
        
        assertTrue(player.canTravelTo(ss4));
        assertTrue(craft1.canTravelTo(ss4));
        
        MotEngine.instance.setDate(MotEngine.instance.getDate()+100);
        //craft1.getTask().removeCraftsFromTask();
        general.endTurn();
        player.setNextCraftOrders(craft1);
        craft1.endTurn();
        
        assertEquals("Have correct tasks", mot.engine.players.generals.CraftTaskExplore.class, craft1.getTask().getClass());
        assertNull("craft1 going to explore ss4", craft1.getOrbit());
        assertEquals("craft1 going to explore ss4", ss4, craft1.getNavigator().getDestination().getStellarObject());
    }

    /**
     * Tests that the general asses the most desirable planet to colonise correctly.
     */
    public void testColoniseBest() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testColoniseBest() *************************");
        createMap();

        GeneralBasic general = (GeneralBasic)player.getGeneral();
        
		CraftDesign monsterSmallDesign = new CraftDesign("monster_small", 
				tech_flyer60,
				tech_fusion25,
				tech_electronic40,
				tech_energy20,
				tech_massDriver1,tech_massDriver1,tech_massDriver1,tech_massDriver1,
				tech_massDriver1,tech_massDriver1,tech_massDriver1,tech_massDriver1);
		CraftDesign monsterDesign = new CraftDesign("monster", 
				tech_flyer100,
				tech_fusion40,
				tech_electronic40,
				tech_energy20,
				tech_fusionBeam4,tech_fusionBeam4,tech_fusionBeam4,tech_fusionBeam4,
				tech_fusionBeam4,tech_fusionBeam4,tech_fusionBeam4,tech_fusionBeam4);
		CraftDesign defender2Design = new CraftDesign("TestDefender2", 
				tech_flyer60,
				tech_fusion25,
				tech_electronic25,
				tech_energy10,
				tech_massDriver1,tech_massDriver1,tech_massDriver1,tech_massDriver1,
				tech_massDriver1,tech_massDriver1,tech_massDriver1,tech_massDriver1);

		newCraft(powerfulDesign, player, ss1);
		Craft craft = newCraft(coloniserDesign, player, ss1);
		Craft craft2 = newCraft(monsterSmallDesign, MotEngine.instance.getMonsterPlayer(), ss2);
        planet2.setTeraforming(0.7);

        assertEquals("No planet with terraform I can colonise", null, general.getBestPlanetCanColonise(craft));
		
        planet2.setTeraforming(0.9);

        assertEquals(planet2, general.getBestPlanetCanColonise(craft));
        
        ss2.setLocation(new Location(300,100));
        
        assertEquals("No planet with terraform I can colonise in range", null, general.getBestPlanetCanColonise(craft));

        ss2.setLocation(new Location(200,100));
        
        assertEquals("Has correct colonisation rating", 0.8, craft.getSpecials().getCanColoniseRating().doubleValue(), 0.001);
        assertEquals("Has correct best planet", planet2, general.getBestPlanetCanColonise(craft));

        planet3.setTeraforming(1.0);
        assertEquals("Has correct best planet", planet3, general.getBestPlanetCanColonise(craft));
    }
    
    /**
     * Tests that the general moves colony crafts to best available planets for colonisation.
     * But ignores planets out of range!
     */
    public void testColoniser1() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testColoniser1() *************************");
        createMap();

        General general = player.getGeneral();
        Craft craft = newCraft(coloniserDesign, player, ss1);
        player.setFunds(1000);
        
        player.setNextCraftOrders(craft);
        craft.endTurn();
        MotEngine.instance.logSummary();
        
        Craft colonyCraft = player.getColonyCraft();
        assertEquals("Players only colony is mine", craft, colonyCraft);
        assertEquals("Colony craft heading to best planet", ss2, craft.getNavigator().getDestination().getStellarObject());
        assertEquals("Colony craft has correct task", CraftTaskColonise.class, colonyCraft.getTask().getClass());

        int speed = colonyCraft.getEngine().getWarpSpeed();
        assertEquals("Colony craft has correct speed", 25, speed);
        for (int i=0; i < 4; i++) {
            craft.endTurn();
        }

        colonyCraft = player.getColonyCraft();
        assertTrue("Player does not have a colony craft", colonyCraft == null);
        assertEquals("Player has a new colony", planet2.getOwner(), player);
        assertEquals("Player has a new solar system", ss2.getOwners()[0], player);
        assertEquals("SS2-A has been added to players planets", "SS2-A", player.getPlanets()[1].getName());
    }

    /**
     * Tests that coloniser can only colonise a planet when they have the correct technology to do so
     * and they are in range.
     */
    public void testColoniser2() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testColoniser2() *************************");
        createMap();

        General general = player.getGeneral();
        Craft craft = newCraft(coloniserDesign, player, ss1);
        player.setFunds(1000);
        
        // Move solar system 2 out of range - No planets are not in range that can be colonised by the player
        ss2.setLocation(new Location(500,500));
        
        player.setNextCraftOrders(craft);
        craft.endTurn();
        
        Craft colonyCraft = player.getColonyCraft();
        assertEquals("Players only colony is mine", craft, colonyCraft);
        assertEquals("Planer cannot colonise planet3", null, craft.getNavigator());
        assertEquals("Colony craft has correct task", CraftTaskIdle.class, craft.getTask().getClass());

        // Player now has the ability to colonise planet3
        colonyCraft.getSpecials().remove(tech_colonyModule8);
        colonyCraft.getSpecials().add(tech_colonyModule7);

        //NOTHING HAPPENS UNTIL IDLE TASK TIMES OUT
        player.setNextCraftOrders(craft);
        craft.endTurn();
        
        assertEquals("Players only colony is mine", craft, colonyCraft);
        assertEquals("Colony craft heading to best planet", ss3, craft.getNavigator().getDestination().getStellarObject());
        assertEquals("Colony craft has correct task", CraftTaskColonise.class, craft.getTask().getClass());
        
        int speed = colonyCraft.getEngine().getWarpSpeed();
        assertEquals("Colony craft has correct speed", 25, speed);
        craft.endTurn();

        colonyCraft = player.getColonyCraft();
        assertTrue("Player does not have a colony craft", colonyCraft == null);
        assertEquals("Player has a new colony", planet3.getOwner(), player);
        assertEquals("Player has a new solar system", ss3.getOwners()[0], player);
        assertEquals("SS2-A has been added to players planets", "SS3-A", player.getPlanets()[1].getName());
    }

    /**
     * Tests that coloniser can only colonise a planet when they have the correct technology to do so
     * and they are in range.
     */
    public void testColoniserWhenPlanetConditionChanged() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testColoniserWhenPlanetConditionChanged() *************************");
        createMap();

        General general = player.getGeneral();
        Craft craft = newCraft(coloniserDesign, player, ss1);
        player.setFunds(1000);

        
        player.setNextCraftOrders(craft);
        craft.endTurn();
        
        // Make it so play can colonise both planets
        Craft colonyCraft = player.getColonyCraft();        
        colonyCraft.getSpecials().add(tech_colonyModule7);

        assertEquals("Players only colony is mine", craft, colonyCraft);
        assertEquals("Colony craft heading to best planet", planet2, craft.getNavigator().getDestination().getPlanet());
        assertEquals("Colony craft heading to best planet", ss2, craft.getNavigator().getDestination().getStellarObject());
        assertEquals("Colony craft has correct task", CraftTaskColonise.class, craft.getTask().getClass());
        
        int speed = colonyCraft.getEngine().getWarpSpeed();
        assertEquals("Colony craft has correct speed", 25, speed);
        assertEquals("Colony will take 4 turn to arrive", 3, craft.getNavigator().getETA());
        craft.endTurn();
        
        // Planet's condition changes during journey
        planet2.setTeraforming(0.5);
        
        craft.endTurn();
        
        assertEquals("Colony craft heading to best planet", planet2, craft.getNavigator().getDestination().getPlanet());
        assertEquals("Colony craft heading to best planet", ss2, craft.getNavigator().getDestination().getStellarObject());
        assertEquals("Colony craft has correct task", CraftTaskColonise.class, craft.getTask().getClass());
        
        craft.endTurn();
        craft.getOwner().endTurn();

        colonyCraft = player.getColonyCraft();        
        assertEquals("Player could not colonise Planet2", craft, colonyCraft);
        assertEquals("Colony craft heading to best planet", planet3, craft.getNavigator().getDestination().getPlanet());
        assertEquals("Colony craft heading to best planet", ss3, craft.getNavigator().getDestination().getStellarObject());
        assertEquals("Colony craft has correct task", CraftTaskColonise.class, craft.getTask().getClass());
        assertEquals("Colony will take 1 turns to arrive", 2, craft.getNavigator().getETA());
        
        // Planet's condition changes during journey
        planet3.setTeraforming(0.5);
        craft.endTurn();
        craft.endTurn();
        
        colonyCraft = player.getColonyCraft();
        assertEquals("Player could not colonise Planet3", craft, colonyCraft);
        assertEquals("Player has a new colony", null, planet3.getOwner());
        assertEquals("Colony craft has correct task", null, craft.getTask());
    }


    
    /**
     * Test that colonisation of a solar system with an enemy in orbit requires a fleet with military crafts.
     * Tests that monsters do not move and attack crafts that arrive.
     */
    public void testColoniser3() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testColoniser3() *************************");
        createMap();

        General general = player.getGeneral();
        player.setFunds(1000);
		newCraft(powerfulDesign, player, ss1);
        Craft colonyCraft1 = newCraft(coloniserDesign, player, ss1);
        Craft monsterCraft = CraftFactory.create("monster", MotEngine.instance.getMonsterPlayer(), MotEngine.instance.getMonsterPlayer(), ss2.getOrbit(), true, false);

        // Test that monster is too powerful so player does not form any task to attempt to 
        // take the planet.
        player.setNextCraftOrders(colonyCraft1);
        colonyCraft1.endTurn();

        assertEquals(1, general.getTasks().length);
        CraftTaskSteelColony task1 = (CraftTaskSteelColony) general.getTasks()[0];
        
        assertEquals("Players only colony is mine", colonyCraft1, player.getColonyCraft());
        assertEquals("Colony craft has correct task", task1, colonyCraft1.getTask());
        assertEquals("Colony craft has correct task", task1.getFleet(), colonyCraft1.getFleet());
        assertEquals("Colony craft has correct task", true, task1.contains(colonyCraft1));
        assertEquals("Not dispatched becuase the monster is too powerful", null, colonyCraft1.getNavigator());

        //
		// test that as military craft become available they are added to the task of stealing
        // the colony.
        //
        
        colonyCraft1.getTask().removeCraftsFromTask();
        Craft fighterCraft1 = newCraft(defenderDesign, player, ss1);
        player.setNextCraftOrders(colonyCraft1);
        player.setNextCraftOrders(fighterCraft1);
        
        assertEquals("craft has correct task", task1, colonyCraft1.getTask());
        assertEquals("craft has correct task", task1, fighterCraft1.getTask());
        assertEquals(null, colonyCraft1.getNavigator());
        assertEquals(null, fighterCraft1.getNavigator());
        assertEquals(task1.getFleet(), colonyCraft1.getFleet());
        assertEquals(task1.getFleet(), fighterCraft1.getFleet());
        assertEquals(task1.getFleet(), player.getFleet()[0]);
        assertEquals(null, task1.getFleet().getNavigator());

        // Test that monster does not move.
        MotEngine.instance.getMonsterPlayer().setNextCraftOrders(monsterCraft);
        monsterCraft.endTurn();
        assertEquals("Monster protecting planet2", null, monsterCraft.getNavigator());
        assertEquals("Monster protecting planet2", ss2, monsterCraft.getOrbit().getStellarObject());
        assertEquals("Monster craft has correct task", CraftTaskProtect.class, monsterCraft.getTask().getClass());

        //
        // Test that when military crafts are created they are added to the colonisation task.
        // But they must first move to the correct location.
        //
        
        Craft fighterCraft2 = newCraft(powerfulDesign, player, ss3);

        player.setNextFleetOrders(task1.getFleet());
        player.setNextCraftOrders(fighterCraft2);
        task1.getFleet().endTurn();

        assertEquals("craft has correct task", task1, colonyCraft1.getTask());
        assertEquals("craft has correct task", task1, fighterCraft1.getTask());
        assertEquals("craft has correct task", null, fighterCraft2.getTask());
        
        assertEquals("Craft4 moving to join fleet", null, fighterCraft2.getFleet());
        assertEquals("Craft4 moving to join fleet", ss1, fighterCraft2.getNavigator().getDestination().getStellarObject());
        assertEquals("Task not yet ready for launch", false, general.getTasks()[0].readyForAssignment());

        assertEquals("Colony craft has correct speed", 25, colonyCraft1.getEngine().getWarpSpeed());
        fighterCraft2.endTurn();
        fighterCraft2.endTurn();

        MotEngine.instance.logSummary();

        //
        // Test fleet does launch when fighter craft2 arrives.
        //
        
        assertEquals("craft has correct task", task1, colonyCraft1.getTask());
        assertEquals("craft has correct task", task1, fighterCraft1.getTask());
        assertEquals("craft has correct task", task1, fighterCraft2.getTask());
        
        Fleet fleet = task1.getFleet();
        assertEquals("Craft4 joined fleet", task1.getFleet(), fighterCraft2.getFleet());
        assertEquals("Task not yet ready for launch", true, fighterCraft2.getTask().readyForAssignment());
        player.setNextFleetOrders(fleet);

        // Test when release conditions are meat then the fleet leaves ss1 bound for ss2 and colonises planet2.
        assertTrue("Fleet contains correct craft", fleet.contains(colonyCraft1));
        assertTrue("Fleet contains correct craft", fleet.contains(fighterCraft1));
        assertTrue("Fleet contains correct craft", fleet.contains(fighterCraft2));
        assertEquals("Fleet heading for ss2", ss2, fleet.getNavigator().getDestination().getStellarObject());

        fleet.endTurn();
        fleet.endTurn();
        fleet.endTurn();
        fleet.endTurn();

        // Fleet has arrived at ss2        
        assertEquals("Fleet arrives at ss2", null, fleet.getNavigator());
        assertEquals("Fleet arrives at ss2", ss2, fleet.getOrbit().getStellarObject());

        // Test that monster attacks all enemies.
        MotEngine.instance.getMonsterPlayer().endTurn();
        monsterCraft.endTurn();
        MotEngine.instance.endTurn(player);

        assertTrue("Monster destroyed", monsterCraft.getDamage() >= monsterCraft.getHull().getMaxDamage());

        player.setNextFleetOrders(fleet);
        player.setNextFleetOrders(fleet);

        assertEquals("Task has completed", null, general.getTaskType(CraftTaskSteelColony.class));
        assertEquals("Player has a new colony", player, planet2.getOwner());

    }

    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testAttackSingleEnemyTarget() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testAttackSingleEnemyTarget() *************************");
        createMap();
        addEnemy(100, 140);
        
        // Give Player an extra planet and some Crafts
        Craft fighter1 = newCraft(fighterDesign, player, ss1);
        Craft fighter2 = newCraft(fighterDesign, player, ss2);
        
        // Give funds so things are not sold
        player.setFunds(1000);
        enemyPlayer.setFunds(1000);
        
        // Make population so small they take ages to build anything
        planet1.setPopulation(0.1);
        planet1.setQtyFactories(0.0);
        enemyPlanet1.setPopulation(0.1);
        enemyPlanet1.setQtyFactories(0.0);
        
        player.getPlayerCharacter().setMaxTimeBetweenFleets(5);
        General general = player.getGeneral();
        ((GeneralBasic)general).setQtyCraftsInAttackFleet(2);
        Location ss1Loc = ss1.getLocation();
        Location ss3Loc = ss3.getLocation();
        
        //
        // TEST WHEN PLAYER IS NOT AT WAR GENERAL DOES NOT DO ANYTHING!
        //

        MotEngine.instance.endTurn(player);
        
        assertEquals("Have no task to perform", 1, general.getTasks().length);
        assertEquals("Have no task to perform", CraftTaskIdle.class, general.getTasks()[0].getClass());
        assertEquals("Fighter1 has not moved", ss1, fighter1.getOrbit().getStellarObject());
        assertEquals("Fighter1 has not moved", null, fighter1.getNavigator());
        assertEquals("Fighter2 has not moved", ss2, fighter2.getOrbit().getStellarObject());
        assertEquals("Fighter2 has not moved", null, fighter2.getNavigator());
        assertEquals("Fighter1 is not part of a fleet", null, fighter1.getFleet());
        assertEquals("Fighter2 is not part of a fleet", null, fighter2.getFleet());
        
        //
        // TEST WHEN WAR IS DECLARED THEN WE BUILD A FLEET OF QtyCraftsOfLastFleet (2) size to attack.
        // TEST CRAFTS ARE MOVED FROM OTHER STELLAR OBJECTS TO JOIN FLEET
        //

        general.remove(general.getTasks()[0]);
        fighter1.setTask(null);
        fighter2.setTask(null);
        fighter2.setOrbit(ss2.getOrbit());
        fighter2.setNavigator(null);
        fighter2.setLocation(new Location(ss2.getLocation()));
        
        MotEngine.instance.getRelations().setWar(player, enemyPlayer);
        MotEngine.instance.endTurn(player);
        Fleet fleet = player.getFleet()[0];
        
        assertEquals("Have attack task to perform", CraftTaskAttack.class, fleet.getTask().getClass());
        assertEquals("Fighter1 is in a fleet and has not moved", ss1, fighter1.getOrbit().getStellarObject());
        assertEquals("Fighter1 is in a fleet and has not moved", null, fighter1.getNavigator());
        assertEquals("Fighter2 is not part of a fleet", null, fighter2.getFleet());
        assertEquals("Fighter2 enroute to ss1 to join fleet", ss1, fighter2.getNavigator().getDestination().getStellarObject());
        
        // move fighter2 to ss1
        fighter2.endTurn();
        fighter2.endTurn();
        fighter2.endTurn();
        
        assertEquals("Fighter2 is part of a fleet with fighter1", fleet, fighter2.getFleet());
        assertEquals("Fighter2 is in a fleet and has not moved", ss1, fighter1.getOrbit().getStellarObject());
        assertEquals("Fighter2 is in a fleet and has not moved", null, fighter1.getNavigator());

        MotEngine.instance.endTurn(player);
        
        assertEquals("Fighter1 is part of a fleet enrouts to enemySS", enemySS1, fighter1.getNavigator().getDestination().getStellarObject());
        assertEquals("Fighter2 is part of a fleet enrouts to enemySS", enemySS1, fighter2.getNavigator().getDestination().getStellarObject());

        //
        // TEST BATTLE OCCURS AND THE FIGHTER ARE COMPLETELY OUT GUNNED
        //

        // General will create Bombard or invade tasks when it is informed of a battle finishing.
        MotEngine.instance.getProxyManager().removeAll();
        MotEngine.instance.endTurn(player);
        
        Craft [] crafts = player.getCrafts();
        assertTrue("Fighter1 and Fighter2 destroyed", crafts == null || crafts.length == 0);
        assertTrue("Fleet1 is destroyed", player.getFleet() == null || player.getFleet().length == 0);
        assertTrue("Battle has ended", MotEngine.instance.getBattles() == null || MotEngine.instance.getBattles().getQtyBattles() == 0);

        //
        // TEST NEXT FEET BUILT HAS 2 + FLEET_SIZE_INCREMENT CRAFTS (4).
        // TEST CRAFTS ARE MOVED TO DEFEND AGAINST ATTACK
        //
        
        Craft defender1 = newCraft(defenderDesign, player, ss1);
        Craft defender2 = newCraft(defenderDesign, player, ss1);
        Craft defender3 = newCraft(defenderDesign, player, ss1);
        Craft enemyFighter = newCraft(fighterDesign, enemyPlayer, ss3);
        
        MotEngine.instance.endTurn(player);
        
        //
        // FLEET IS NOT RELEASED BECUASE IT IS NOT YET SIZE 4 - BUT WILL BE RELEASED IF BUILD TIME EXPIRES
        //
        
        assertEquals("Have attack task to perform", CraftTaskAttack.class, defender1.getTask().getClass());
        assertEquals("Have attack task to perform", CraftTaskAttack.class, defender2.getTask().getClass());
        assertEquals("Have attack task to perform", CraftTaskAttack.class, defender3.getTask().getClass());

        assertEquals("Defender1 is in a fleet and has not moved", ss1, defender1.getFleet().getOrbit().getStellarObject());
        assertNull("Defender1 is in a fleet and has not moved", defender1.getFleet().getNavigator());
        assertEquals("Defender2 is in a fleet and has not moved", ss1, defender2.getFleet().getOrbit().getStellarObject());
        assertNull("Defender2 is in a fleet and has not moved", defender2.getFleet().getNavigator());
        assertEquals("Defender3 is in a fleet and has not moved", ss1, defender3.getFleet().getOrbit().getStellarObject());
        assertNull("Defender3 is in a fleet and has not moved", defender3.getFleet().getNavigator());
        
        //
        // CHECK DEFEND TASK IS REMOVED WHEN NOT UNDER ATTACK
        //
        
        enemyPlayer.getGeneral().endTurn();
        
        assertEquals("Have attack task to perform", null, enemyPlayer.getGeneral().getTaskType(CraftTaskDefend.class));

        //
        // Check fleet is released
        //
        
        enemyFighter.arrivedAt(ss2.getOrbit());
        MotEngine.instance.logSummary();
        MotEngine.instance.setDate(MotEngine.instance.getDate()+10);
        MotEngine.instance.endTurn(player);       
        fleet = player.getFleet()[0];
        
        assertEquals("Fleet2 is released to player1", fleet, defender1.getFleet());
        assertEquals("Fleet2 is released to player1", fleet, defender2.getFleet());
        assertEquals("Fleet2 is released to player1", fleet, defender3.getFleet());
        assertEquals("Fleet is enroute to enemySS", enemySS1, fleet.getNavigator().getDestination().getStellarObject());

        //
        // Let enemy react to my attack
        //

        enemyPlayer.getGeneral().endTurn();
        enemyFighter.endTurn();
        
        assertEquals("Have attack task to perform", CraftTaskAttack.class, defender1.getTask().getClass());
        assertEquals("Have attack task to perform", CraftTaskDefend.class, enemyFighter.getTask().getClass());
        assertNull("EnemyFighter is moving to defend", enemyFighter.getOrbit());
        assertEquals("EnemyFighter is moving to defend", enemySS1, enemyFighter.getNavigator().getDestination().getStellarObject());
    }

    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testAttackAfterWarDeclared() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testAttackAfterWarDeclared() *************************");
        createMap();
        addEnemy(100, 140);
        
        // Give Player an extra planet and some Crafts
        Craft fighter1 = newCraft(fighterDesign, player, ss1);
        Craft fighter2 = newCraft(fighterDesign, player, enemySS1);
        
        // Give funds so things are not sold
        player.setFunds(1000);
        enemyPlayer.setFunds(1000);
        
        // Make population so small they take ages to build anything
        planet1.setPopulation(0.1);
        planet1.setQtyFactories(0.0);
        enemyPlanet1.setPopulation(0.1);
        enemyPlanet1.setQtyFactories(0.0);
        
        player.getPlayerCharacter().setMaxTimeBetweenFleets(5);
        General general = player.getGeneral();
        ((GeneralBasic)general).setQtyCraftsInAttackFleet(2);
        
        //
        // TEST EXISTING CRAFTS IN AN IDLE TASK ARE ADDED TO AN ATTACK TASK
        //

        MotEngine.instance.__endTurn_CraftsFleetsBattles();
        
        assertEquals(CraftTaskIdle.class, fighter1.getTask().getClass());
        assertEquals(CraftTaskIdle.class, fighter2.getTask().getClass());
        
        MotEngine.instance.getRelations().setWar(player, enemyPlayer);
        MotEngine.instance.__endTurn_CraftsFleetsBattles();

        assertEquals(enemyPlanet1, ((CraftTaskAttack)fighter1.getTask()).getTargetPlanet());
        assertEquals(fighter1, ((CraftTaskAttack)fighter1.getTask()).getFleet().getCrafts());
        assertEquals(ss1, fighter2.getNavigator().getDestination().getStellarObject());
        assertEquals(1, fighter2.getNavigator().getETA());

        fighter2.endTurn();
        
        assertEquals(fighter1, fighter2, ((CraftTaskAttack)fighter1.getTask()).getFleet().getCrafts());
        
    }
    
    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testAttackBlackHole() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testAttackBlackHole() *************************");
        createMap();
        addEnemyMultiple();
        
        Craft fighter1 = newCraft(fighterDesign, player, ss1);
        Craft fighter2 = newCraft(fighterDesign, player, ss1);
        
        // Give funds so things are not sold
        player.setFunds(1000);
        enemyPlayer.setFunds(1000);
        
        // Make population so small they take ages to build anything
        planet1.setPopulation(0.1);
        planet1.setQtyFactories(0.0);
        enemyPlanet1.setPopulation(0.1);
        enemyPlanet1.setQtyFactories(0.0);
        
        player.getPlayerCharacter().setMaxTimeBetweenFleets(5);
        General general = player.getGeneral();
        ((GeneralBasic)general).setQtyCraftsInAttackFleet(2);
        Location ss1Loc = ss1.getLocation();
        Location ss3Loc = ss3.getLocation();
        
        //
        // TEST WHEN WAR IS DECLARED THEN WE BUILD A FLEET OF QtyCraftsOfLastFleet (2) size to attack.
        // TEST CRAFTS ARE MOVED FROM OTHER STELLAR OBJECTS TO JOIN FLEET
        //

        MotEngine.instance.getRelations().setWar(player, enemyPlayer);
        Utils.setNextRandomInt(0,1); // select enemy 0 to attack
        Utils.setNextRandomProbability(0.2,0.5); // select target blackHole to attack        
        player.getGeneral().assignTaskToCraft(fighter1);
        player.getGeneral().assignTaskToCraft(fighter2);
        Fleet fleet = player.getFleet()[0];
        
        assertEquals("Have attack task to perform", CraftTaskAttack.class, fleet.getTask().getClass());
        assertEquals("Fighter1 is part of a fleet", fleet, fighter1.getFleet());
        assertEquals("Fighter2 is part of a fleet", fleet, fighter2.getFleet());
        assertEquals("Fighter2 is in a fleet and has not moved", null, fleet.getNavigator());
        assertEquals(blackHole, fleet.getTask().getTarget().getStellarObject());
        assertEquals(true, fleet.getTask().readyForAssignment());

        player.setNextFleetOrders(fleet);
        
        assertEquals("Fleet enrout", blackHole, fleet.getNavigator().getDestination().getStellarObject());
        assertEquals("Fighter1 is part of a fleet enrouts to enemySS", blackHole, fighter1.getNavigator().getDestination().getStellarObject());
        assertEquals("Fighter2 is part of a fleet enrouts to enemySS", blackHole, fighter2.getNavigator().getDestination().getStellarObject());

        //
        // TEST BATTLE OCCURS WHEN FIGHTERS ATTACK SPACE STATION
        //

        fleet.endTurn();
        fleet.endTurn();
        fleet.endTurn();
        fleet.endTurn();
        
        Battle battle = MotEngine.instance.getBattles().getBattles()[0];
        BattleCraft[] crafts = battle.getCrafts();
		assertEquals(3, crafts.length);
		assertEquals(tech_spaceStation, crafts[0].getCraft().getHull());
		assertEquals(fighter1, crafts[1].getCraft());
		assertEquals(fighter2, crafts[2].getCraft());

    }

    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testAttackWormHoleWithAttackVia() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testAttackWormHoleWithAttackVia() *************************");
        createMap();
        addEnemyMultipleWithWormHole();
        
        // Hard Game implies Computer Player will use attack via to disguise its target
        MotEngine.instance.setLevel(2.0);
        
        Craft fighter1 = newCraft(fighterDesign, player, ss1);
        Craft fighter2 = newCraft(fighterDesign, player, ss1);
        
        // Give funds so things are not sold
        player.setFunds(1000);
        enemyPlayer.setFunds(1000);
        
        // Make population so small they take ages to build anything
        planet1.setPopulation(0.1);
        planet1.setQtyFactories(0.0);
        enemyPlanet1.setPopulation(0.1);
        enemyPlanet1.setQtyFactories(0.0);
        
        player.getPlayerCharacter().setMaxTimeBetweenFleets(5);
        General general = player.getGeneral();
        ((GeneralBasic)general).setQtyCraftsInAttackFleet(3);
        Location ss1Loc = ss1.getLocation();
        Location ss3Loc = ss3.getLocation();
        
        //
        // TEST WHEN WAR IS DECLARED THEN WE BUILD A FLEET OF QtyCraftsOfLastFleet (3) size to attack.
        // TEST CRAFTS ARE MOVED FROM OTHER STELLAR OBJECTS TO JOIN FLEET
        //

        MotEngine.instance.getRelations().setWar(player, enemyPlayer);
        Utils.setNextRandomInt(0,1); // select enemy 0 to attack
        Utils.setNextRandomProbability(0.2,0.5); // select target blackHole to attack        
        Utils.setNextRandomProbability(0.2,0.5); // select choose to use attack via planet        
        player.getGeneral().assignTaskToCraft(fighter1);
        player.getGeneral().assignTaskToCraft(fighter2);
        Fleet fleet = player.getFleet()[0];
        
        assertEquals(fighter1, fighter2, fleet.getCrafts());
        assertEquals(null, fleet.getNavigator());
        assertEquals(wormHole, ((CraftTaskAttack)fleet.getTask()).getTarget().getStellarObject());
        assertEquals(false, fleet.getTask().readyForAssignment());

        Craft fighter3 = newCraft(fighterDesign, player, ss1);
        player.getGeneral().assignTaskToCraft(fighter3);

        assertEquals(fighter1, fighter2, fighter3, fleet.getCrafts());
        assertEquals(enemyPlanet1, ((CraftTaskAttack)fleet.getTask()).getAttackViaPlanet());
        assertEquals(wormHole, ((CraftTaskAttack)fleet.getTask()).getTarget().getStellarObject());
        assertEquals(true, fleet.getTask().readyForAssignment());
        
        player.setNextFleetOrders(fleet);

        assertEquals(enemyPlanet1, fleet.getNavigator().getDestination().getPlanet());
        assertEquals(2, fleet.getNavigator().getETA());

        fleet.endTurn();
        fleet.endTurn();

        //
        // TEST BATTLE OCCURS WHEN FIGHTERS ATTACK SPACE STATION
        //

        assertEquals(wormHole, fleet.getNavigator().getDestination().getStellarObject());
        assertEquals(4, fleet.getNavigator().getETA());

        fleet.endTurn();
        fleet.endTurn();
        fleet.endTurn();
        fleet.endTurn();
        
        Battle battle = MotEngine.instance.getBattles().getBattles()[0];
        BattleCraft[] crafts = battle.getCrafts();
		assertEquals(4, crafts.length);
		assertEquals(tech_spaceStation, crafts[0].getCraft().getHull());
		assertEquals(fighter1, crafts[1].getCraft());
		assertEquals(fighter2, crafts[2].getCraft());
		assertEquals(fighter3, crafts[3].getCraft());

    }

	/**
	 * Test Attack on enemy planet with enemy craft AND craft from another enemy present.
	 */
	public void testAttackPlanetWithMultipleEnemies() throws Exception {
	    Logger.logDebug("*******************************  WarTest.testAttackPlanetWithMultipleEnemies() *************************");
	    
    	Utils.clearNextRandomInt();
    	MotEngine.newInstance();
        Logger.setDebug();
        setupConfigs();

        String [] playerNames = { "Humans", "Bears" , "Birds" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
        UserPlayer userPlayer1 = newUserPlayer(playerNames[0], blue, null, options);
        ComputerPlayer player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);
        ComputerPlayer player3 = newPlayer(playerNames[2], yellow, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, userPlayer1);
        blackHole = newBlackHole("BlackHole", 50, 50, 0.1f, false);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 100, 200);
        planet3 = newPlanet("SS3-A", ss3, player3);

        MotEngine.getInstance().getRelations().setWar(userPlayer1, player2);
        MotEngine.getInstance().getRelations().setWar(player3, player2);
        clearEvents();
	            
        newSpaceStation(userPlayer1, blackHole);
	    Craft craft1 = newCraft(fighterDesign, userPlayer1, blackHole);
	    Craft craft2 = newCraft(fighterDesign, player2, blackHole);
	    Craft craft3 = newCraft(fighterDesign, player3, blackHole);
	
        General general = player2.getGeneral();
        ((GeneralBasic)general).setQtyCraftsInAttackFleet(1);
        general.add(new CraftTaskAttack(blackHole, player2, 1, null));
	    player2.setNextCraftOrders(craft2);
	    
        assertEquals(craft2, ((CraftTaskAttack)craft2.getTask()).getFleet().getCrafts());

        ((CraftTaskAttack)craft2.getTask()).assignTaskToFleet();
        
        Battle battle = MotEngine.instance.getBattles().getBattles()[0];
	    BattleCraft[] bca = (BattleCraft[]) order(battle.getCrafts(), new BattleCraft[4], "BattleStation1", "TestFighter1", "TestFighter2", "TestFighter3");
	
	    assertEquals("BattleStation1", "TestFighter1", "TestFighter2", "TestFighter3", bca);
    
	}


    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testDefend() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testDefend() *************************");
        createMap();
        addEnemy(100, 140);

        SolarSystem enemySS2 = newSolarSystem("SSE2", 100, 120);
        Planet enemyPlanet2 = newPlanet("SSE2-A", enemySS2, enemyPlayer);

        // Give Player an extra planet and some Crafts
        Craft fighter1 = newCraft(powerfulDesign, player, ss1);
        Craft fighter2 = newCraft(powerfulDesign, player, ss1);
        
        Craft enemyFighter1 = newCraft(fighterDesign, enemyPlayer, enemySS1);
        
        // Give funds so things are not sold
        player.setFunds(1000);
        enemyPlayer.setFunds(1000);
        
        player.getPlayerCharacter().setMaxTimeBetweenFleets(5);
        General general = player.getGeneral();
        ((GeneralBasic)general).setQtyCraftsInAttackFleet(2);
        
        //
        // TEST WHEN WAR IS DECLARED THEN WE BUILD A FLEET OF QtyCraftsOfLastFleet (2) size to attack.
        // TEST CRAFTS ARE MOVED FROM OTHER STELLAR OBJECTS TO JOIN FLEET
        //

        MotEngine.instance.getRelations().setWar(player, enemyPlayer);

        // Computer will not BOMBARD after winning battle if troops are on the way.
        Transport transport = player.getTransportManagerNotNull().getFreeTransportOrCreateTransport(player);
        planet1.setQtyGroundDefense(100);
        transport.setTask(new TransportTaskInvade(transport, planet1, enemyPlanet2, 10));
        transport.transferTo(new Orbit(planet4));
        transport.moveTo(new Orbit(enemyPlanet2));
        transport.getNavigator().move();
        transport.setLocation(new Location(1000,1000));
      

        player.getPlayerCharacter().setBlockadeOrAttackProbability(0.501);
        Utils.setNextRandomInt(1, 2); // select enemyPlanet2 as target
        Utils.setNextRandomProbability(0.1, 0.501); // Attack when arrive at enemySS2
        MotEngine.instance.__endTurn_CraftsFleetsBattles();
        
        Fleet fleet = player.getFleet()[0];
        assertEquals(enemyPlanet2, ((CraftTaskAttack)fleet.getTask()).getTargetPlanet());
        assertEquals(enemySS2, fighter1.getOrbit().getStellarObject());
        assertEquals(null, fighter1.getNavigator());
        assertEquals(enemySS2, fighter2.getOrbit().getStellarObject());
        assertEquals(null, fighter2.getNavigator());
        assertEquals(enemySS2, enemyFighter1.getOrbit().getStellarObject());
        assertEquals(null, enemyFighter1.getNavigator());
        assertEquals(null, ((CraftTaskDefend)enemyFighter1.getTask()).getFleet());
        
        MotEngine.instance.__endTurn_CraftsFleetsBattles();
        
        assertEquals(true, enemyFighter1.isDestroyed());
        assertEquals(enemyPlanet2, ((CraftTaskAttack)fleet.getTask()).getTargetPlanet());
        assertEquals(enemySS2, ((CraftTaskDefend)enemyPlayer.getGeneral().getTasks()[0]).getTargetStellarObject());
        assertEquals(1, ((CraftTaskDefend)enemyPlayer.getGeneral().getTasks()[0]).getQtyBattlesLost());

        //
        // TEST CRAFT ASSIGNED TO DEFEND NOW JOIN A FLEET AND ATTACK TO DEFEND AS A GROUP	
        //
        
        ((CraftTaskDefend)enemyPlayer.getGeneral().getTasks()[0]).setQtyBattlesLost(2);
        
        Craft enemyFighter2 = newCraft(powerfulDesign, enemyPlayer, enemySS1);
        
        MotEngine.instance.__endTurn_CraftsFleetsBattles();

        Fleet enemyFleet = enemyPlayer.getFleet()[0];
        assertEquals(enemyFighter2, enemyFleet.getCrafts());
        
        assertEquals(enemyPlanet2, ((CraftTaskAttack)fleet.getTask()).getTargetPlanet());
        assertEquals(enemySS2, ((CraftTaskDefend)enemyFleet.getTask()).getTargetStellarObject());
        assertEquals(false, ((CraftTaskDefend)enemyFleet.getTask()).complete());
        
        assertEquals(enemyPlanet2, fighter1.getOrbit().getPlanet());
        assertEquals(enemyPlanet2, fighter2.getOrbit().getPlanet());
        assertEquals(enemySS1, enemyFighter2.getOrbit().getStellarObject());

        Craft enemyFighter3 = newCraft(powerfulDesign, enemyPlayer, enemySS1);
        Craft enemyFighter4 = newCraft(powerfulDesign, enemyPlayer, enemySS1);

        MotEngine.instance.__endTurn_CraftsFleetsBattles();

        assertEquals(enemyFighter2, enemyFighter3, enemyFighter4, enemyFleet.getCrafts());
        
        assertEquals(enemyPlanet2, ((CraftTaskAttack)fleet.getTask()).getTargetPlanet());
        assertEquals(enemyPlanet2, ((CraftTaskDefend)enemyFleet.getTask()).getTargetPlanet());
        assertEquals(true, ((CraftTaskDefend)enemyFleet.getTask()).complete());
        
        assertEquals(true, fighter1.isDestroyed());
        assertEquals(true, fighter1.isDestroyed());
        assertEquals(enemyPlanet2, enemyFighter2.getOrbit().getPlanet());
        assertEquals(enemyPlanet2, enemyFighter3.getOrbit().getPlanet());
        assertEquals(enemyPlanet2, enemyFighter4.getOrbit().getPlanet());


     }
    
    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testWayPoint() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testWayPoint() *************************");
        createMap();
        addEnemy(200,200);
        
        BlackHole wormHole = BlackHoleFactory.createWormHole(new Location(100,200));
        wormHole.deploy(new Orbital(tech_spaceStation, null, null), player);

        Craft fighter1 = newCraft(fighterDesign, player, ss1);
        
        // Give funds so things are not sold
        player.setFunds(1000);
        enemyPlayer.setFunds(1000);
        
        // Make population so small they take ages to build anything
        planet1.setPopulation(0.1);
        planet1.setQtyFactories(0.0);
        enemyPlanet1.setPopulation(0.1);
        enemyPlanet1.setQtyFactories(0.0);
        
        player.getPlayerCharacter().setMaxTimeBetweenFleets(5);
        General general = player.getGeneral();
        ((GeneralBasic)general).setQtyCraftsInAttackFleet(2);

        MotEngine.instance.getRelations().setWar(player, enemyPlayer);
        player.getGeneral().assignTaskToCraft(fighter1);
        
        //
        // Test fleet is built at worm hole
        //
        
        MotEngine.instance.getRelations().setWar(player, enemyPlayer);
        player.getGeneral().assignTaskToCraft(fighter1);
        
        assertNull(fighter1.getFleet());
        assertNull(fighter1.getTask());
        assertEquals(wormHole, fighter1.getNavigator().getDestination().getStellarObject());
        assertEquals(4, fighter1.getNavigator().getETA());
        
        fighter1.endTurn();
        fighter1.endTurn();
        fighter1.endTurn();
        fighter1.endTurn();
        
        assertEquals(fighter1, fighter1.getFleet().getCrafts());
        assertEquals(wormHole, fighter1.getFleet().getOrbit().getStellarObject());
        assertEquals(CraftTaskAttack.class, fighter1.getTask().getClass());
    }

    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testBombard() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testBombard() *************************");
        createMap();
        addEnemy(100, 140);
        MotEngine engine = MotEngine.getInstance();
        
        // Give Player an extra planet and some Crafts
        Craft fighter1 = newCraft(fighterDesign, player, enemyPlanet1);
        Craft fighter2 = newCraft(fighterDesign, player, enemyPlanet1);
        
        // Give funds so things are not sold
        player.setFunds(1000);
        enemyPlayer.setFunds(1000);
        engine.getRelationsNotNull().setWar(player, enemyPlayer);
        
        planet1.setPopulation(0.1);
        planet1.setQtyFactories(0.1);
        enemyPlanet1.setPopulation(50);
        enemyPlanet1.setQtyFactories(50);
        
        CraftTaskAttack attackTask = new CraftTaskAttack(enemyPlanet1, player, 2, null);
        General general = player.getGeneral();
		general.add(attackTask);
        general.assignTaskToCraft(fighter1);
        general.assignTaskToCraft(fighter2);
        
        assertEquals("Crafts are in attack task", 2, attackTask.getFleetSize());
        assertEquals("Crafts are in attack task", fighter1, attackTask.getFleet().getCrafts()[0]);
        assertEquals("Crafts are in attack task", fighter2, attackTask.getFleet().getCrafts()[1]);
        
        //
        // TEST WHEN BATTLE ENDS THESE CRAFTS ARE ASSIGNED TO BOMBARD THE PLANET
        //

        enemyPlanet1.getWeapons().removeAll();
        enemyPlanet1.getShields().removeAll();
        Battle battle = startBattle(fighter1, fighter2, enemyPlanet1);
		battle.endBattle();
		MotEngine.instance.getProxyManager().broadcastQueue();
        CraftTask bombardTask = general.getTaskType(CraftTaskBombard.class);
        
        MotEngine.instance.logSummary();
        player.endTurn();
        
        assertEquals(fighter1, fighter2, bombardTask.getFleet().getCrafts());
        assertEquals(true, enemyPlanet1.getPopulation() < 50);
        assertEquals(true, enemyPlanet1.getQtyFactories() < 50);

        //
        // TEST WHEN TASK BOMBARD HAS A PLANET WITH DEFENCES - IT ATTACKS
        //
        
        enemyPlanet1.setPopulation(50);
        enemyPlanet1.setQtyFactories(50);

        enemyPlanet1.getWeapons().add(tech_massDriver1);
        enemyPlanet1.setShields(new Shields(tech_energy10));

        assertEquals(0, engine.getBattles().getQtyBattles());
        
        general.assignTaskToFleet(bombardTask.getFleet());
        
        assertEquals(1, engine.getBattles().getQtyBattles());
        battle = engine.getBattles().getBattles()[0];
		assertEquals(enemyPlanet1, battle.getPlanet().getPlanet());
        assertEquals(2, battle.getCrafts().length);
        assertEquals(fighter1, battle.getCrafts()[0].getCraft());
        assertEquals(fighter2, battle.getCrafts()[1].getCraft());
        
        //
        // TEST ONCE COLONY IS DESTROYED, BOMBARD TASK IS COMPLETE
        //
        
        enemyPlanet1.removeColony(true);

        assertEquals(true, bombardTask.complete());

        general.assignTaskToFleet(bombardTask.getFleet());
        
        assertNull(general.getTaskType(CraftTaskBombard.class));
        assertEquals(null, fighter1.getTask());
        assertEquals(null, fighter2.getTask());
        
    }

   
    /**
     * Test that crafts are transfered between tasks when a task of a higher priority arrives. This
     *  - Creates an Attack task with one military and an idle task with one colony crafts.
     *  - We then move ss2 in range so it has higher priority SteelColony (it takes crafts from idle and attack)
     *  - Before SteelColony task is ready for dispatch, It moves a planet close to colonise task steel the coloniser.
     *  - Before SteelColony task is ready for dispatch, an attack fleet sets sail for ss1, which steels the military crafts. 
     */
    public void testCraftTaskTransfers() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testCraftTaskTransfers() *************************");
        createMap();

        String [] players = { "TestPlayer2" };
        ComputerPlayer enemyPlayer = newPlayer(players[0], yellow, options, players);
        SolarSystem enemySS = newSolarSystem("SSE", 100, 200);
        Planet enemyPlanet = newPlanet("SSE-A", enemySS, enemyPlayer);

		Craft coloniserCraft1 = newCraft(coloniserDesign, player, ss1);
        Craft monsterCraft = CraftFactory.create("monster_small", MotEngine.instance.getMonsterPlayer(), MotEngine.instance.getMonsterPlayer(), planet2.getOrbit(), true, false);
        Craft fighterCraft1 = newCraft(defenderDesign, player, ss1);

        MotEngine.getInstance().getRelationsNotNull().setWar(player, enemyPlayer);
        player.setFunds(1000);
        enemyPlayer.setFunds(1000);
        General general = player.getGeneral();

        // move ss2 out of range so it cannot be colonised!
        ss2.setLocation(new Location(600,600));
        
        //
        // No task created for coloniser because no planet can be colonised
        // Attack task created for fighter because we have an enemy 
        //
        
        general.endTurn();
        player.setNextCraftOrders(coloniserCraft1);
        coloniserCraft1.endTurn();
        player.setNextCraftOrders(fighterCraft1);
        fighterCraft1.endTurn();
        MotEngine.instance.logSummary();
        
        assertEquals("Have correct tasks", 2, general.getTasks().length);
        CraftTaskIdle task3 = (CraftTaskIdle) general.getTasks()[0];
        CraftTaskAttack task4 = (CraftTaskAttack) general.getTasks()[1];
        
        assertEquals("Player cannot colonise anything", null, coloniserCraft1.getNavigator());
        assertEquals("Player cannot colonise anything", task3, coloniserCraft1.getTask());
        assertEquals("Player building attack fleet", null, fighterCraft1.getNavigator());
        assertEquals("Player building attack fleet", task4, fighterCraft1.getTask());
        assertEquals("Player building attack fleet", enemyPlanet, task4.getTargetPlanet());
        assertEquals("Player building attack fleet", true, task4.getFleet().contains(fighterCraft1));
        
        // 
        // TEST IDLE TASK TIMED OUT AND PLANET @ ss2 NOW READY FOR COLONISATION
        // This results in the creation of a new task - steel colony that will take 
        // the coloniser and the fighter!
        //
        ss2.setLocation(new Location(200,100));

        MotEngine.instance.setDate(MotEngine.instance.getDate()+10);
        general.endTurn();
        player.setNextCraftOrders(coloniserCraft1);
        coloniserCraft1.endTurn();
        player.setNextCraftOrders(fighterCraft1);
        fighterCraft1.endTurn();
        MotEngine.instance.logSummary();
        
        assertEquals(task4, general.getTasks()[0]);
        CraftTaskSteelColony task5 = (CraftTaskSteelColony) general.getTasks()[1];
        
        assertEquals("Have correct tasks", 2, general.getTasks().length);
        assertEquals("Have correct tasks", task5, coloniserCraft1.getTask());
        assertEquals("Have correct tasks", task5, fighterCraft1.getTask());
        
        assertEquals("craft1 colonise ss2", null, coloniserCraft1.getNavigator());
        assertEquals("craft1 colonise ss2", true, task5.contains(coloniserCraft1));
        assertEquals("craft2 attack ss2", null, fighterCraft1.getNavigator());
        assertEquals("craft2 attack ss2", true, task5.getFleet().contains(fighterCraft1));
        assertEquals("craft2 not attack sse", false, task3.contains(fighterCraft1));
        
        //
        // Make planet 3 colonisable  - It is more desirable than planet 2 so we 
        // create new task Colonise and the coloniser is assigned to it.
        //
        
        planet3.setTeraforming(1.0);
        task5.getFleet().remove(coloniserCraft1);
        
        assertEquals(null, coloniserCraft1.getTask());
        
        general.createTaskFor(coloniserCraft1);
        
        player.setNextCraftOrders(coloniserCraft1);
        player.setNextCraftOrders(fighterCraft1);
        MotEngine.instance.logSummary();
        
        assertEquals(3, general.getTasks().length);
        assertEquals(task4, general.getTasks()[0]);
        assertEquals(task5, general.getTasks()[1]);
        CraftTaskColonise task6 = (CraftTaskColonise) general.getTasks()[2];
        
        assertEquals("Have correct tasks", task6, coloniserCraft1.getTask());
        assertEquals("Have correct tasks", task5, fighterCraft1.getTask());        
        assertEquals("craft3 colonise ss2", true, task5.contains(fighterCraft1));
        assertEquals("craft1 colonise ss3", ss3, coloniserCraft1.getNavigator().getDestination().getStellarObject());
        assertEquals("craft3 colonise ss2", null, fighterCraft1.getNavigator());
        
        //
        // Enemies now attack ss1, so fighter must defend ss1
        //
        
		planet1.setWeapons(new Weapons(tech_missile6, 4));
        Craft enemyCraft = newCraft(fighterDesign, enemyPlayer, enemySS);
        enemyCraft.moveTo(ss1.getOrbit());
        enemyCraft.endTurn();
        general.endTurn();
        MotEngine.instance.logSummary();

        assertEquals(4, general.getTasks().length);
        assertEquals(task4, general.getTasks()[0]);
        assertEquals(task5, general.getTasks()[1]);
        assertEquals(task6, general.getTasks()[2]);
        CraftTaskDefend task7 = (CraftTaskDefend) general.getTasks()[3];

        assertEquals("Have correct tasks", task6, coloniserCraft1.getTask());
        assertEquals("Have correct tasks", task7, fighterCraft1.getTask());
        assertEquals("Player defend ss1", null, fighterCraft1.getNavigator());
        assertEquals("Player defend ss1", ss1, fighterCraft1.getOrbit().getStellarObject());
        assertEquals("Player defend ss1", null, fighterCraft1.getFleet());
        
        //
        // ENEMY ATTACK FINISHES - fighter re assigned to steal colony task
        //
        
        enemyCraft.arrivedAt(enemySS.getOrbit());
        general.endTurn();
        player.setNextCraftOrders(coloniserCraft1);
        player.setNextCraftOrders(fighterCraft1);
        MotEngine.instance.logSummary();
        
        assertEquals(3, general.getTasks().length);
        assertEquals(task4, general.getTasks()[0]);
        assertEquals(task5, general.getTasks()[1]);
        assertEquals(task6, general.getTasks()[2]);

        assertEquals("Have correct tasks", task6, coloniserCraft1.getTask());
        assertEquals("Have correct tasks", task5, fighterCraft1.getTask());
        assertEquals("colonise ss3", planet3, coloniserCraft1.getNavigator().getDestination());
        assertEquals("colonise ss2", true, task5.contains(fighterCraft1));
        assertEquals("colonise ss2", null, fighterCraft1.getNavigator());

        //
        // CRAFT1 NOW COLONISES ss3 and we build a new coloniser and new fighter. These
        // should be added to steal colony task but not Released yet because does not 
        // contain enough fire power.
        //

        coloniserCraft1.arrivedAt(ss3.getOrbit());
        Craft coloniserCraft2 = newCraft(coloniserDesign, player, ss1);
        Craft fighterCraft2 = newCraft(defenderDesign, player, ss1);
        general.endTurn();
        player.setNextCraftOrders(fighterCraft1);
        player.setNextCraftOrders(coloniserCraft2);
        player.setNextCraftOrders(fighterCraft2);
        MotEngine.instance.logSummary();
        
        //CraftTaskIdle task9 = (CraftTaskIdle) general.getTasks()[3];

        assertEquals(2, general.getTasks().length);
        assertEquals(task4, general.getTasks()[0]);
        assertEquals(task5, general.getTasks()[1]);
        //assertEquals(task9, general.getTasks()[2]);

        assertEquals("Have correct tasks", task5, fighterCraft1.getTask());
        assertEquals("Have correct tasks", task5, coloniserCraft2.getTask());
        assertEquals("Have correct tasks", task4, fighterCraft2.getTask());
        assertEquals("colonise ss2", true, task5.contains(fighterCraft1));
        assertEquals("colonise ss2", true, task5.contains(coloniserCraft2));
        assertEquals("colonise ss2", true, task4.contains(fighterCraft2));
        assertEquals("colonise ss2", null, fighterCraft1.getNavigator());
        assertEquals("colonise ss2", null, coloniserCraft2.getNavigator());
        assertEquals("colonise ss2", null, fighterCraft2.getNavigator());

        //
        // ENEMY PLAYER GETS COLONY ON SS2 - Steal Colony task deleted
        // so fighter craft get assigned to attack out enemy and the coloniser 
        // has nothing to do.
        //

        monsterCraft.destroy();
        Craft craft6 = newCraft(coloniserDesign, enemyPlayer, ss2);
        craft6.colonise(planet2);
        general.endTurn();
        player.setNextCraftOrders(fighterCraft1);
        player.setNextCraftOrders(coloniserCraft2);
        player.setNextCraftOrders(fighterCraft2);
        MotEngine.instance.logSummary();

        assertEquals(2, general.getTasks().length);
        assertEquals(task4, general.getTasks()[0]);
        //assertEquals(task9, general.getTasks()[1]);

        assertEquals("Have correct tasks", task4, fighterCraft1.getTask());
        assertEquals("Have correct tasks", task4, fighterCraft2.getTask());
        //assertEquals("Have correct tasks", task9, coloniserCraft2.getTask());
        assertEquals("craft3 attack enemy", null, fighterCraft1.getNavigator());
        assertEquals("craft3 attack enemy", null, fighterCraft2.getNavigator());
        assertEquals("craft3 attack enemy", null, coloniserCraft2.getNavigator());
    }

    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testAvoidTutorial() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testAvoidTutorial() *************************");
        createMap();
        
        MotEngine.newInstance();
        Logger.setDebug();
        setupConfigs();
        String [] playerNames = { "TestPlayer1" };
        
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player = newPlayer(playerNames[0], blue, options, playerNames);
        
        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player);

        ss2 = newSolarSystem(GameConstants.TUTORIAL_FIRST_COLONY_NAME, 200, 100);
        planet2 = newPlanet(GameConstants.TUTORIAL_FIRST_COLONY_NAME+"-A", ss2, null, 100, 1.0);

        ss3 = newSolarSystem("SS3", 100, 200);
        planet3 = newPlanet("SS3-A", ss3, null, 100, 0.8);

        ss4 = newSolarSystem("SS4", 50, 100);
        planet4 = newPlanet("SS4-A", ss4, null, 110, 0.7);
        
        //
        // Test DO NOT colonise a planet used in the tutorial system
        //
        
        Craft colonyCraft = newCraft(coloniserDesign, player, ss1);
        player.setFunds(1000);
        
        player.setNextCraftOrders(colonyCraft);
        colonyCraft.endTurn();
        MotEngine.instance.logSummary();
        
        assertEquals("Colony craft heading to best planet", ss3, colonyCraft.getNavigator().getDestination().getStellarObject());
        assertEquals("Colony craft has correct task", planet3, ((CraftTaskColonise)colonyCraft.getTask()).getTarget().getPlanet());

    }
    
    /**
     * Tests that The General builds a fleet at a location then uses the fleet to attack its enemy.
     */
    public void testAvoidIncumbents() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testAvoidIncumbents() *************************");
        createMap();
        
        ss1.setLocation(new Location(100,100));
        ss2.setLocation(new Location(200,100));
        ss3.setLocation(new Location(150,200));
        
//        BlackHole blackHole = BlackHoleFactory.createBlackHole(new Location(150, 65));
//        StellarObjects sos = MotEngine.getInstance().getStellarObjects();
//        
//        assertEquals(new Area(110,25,190,105), blackHole.getArea());
//		assertTrue(sos.hasGalacticIncumbentsBetween(ss1.getLocation(), ss2.getLocation(), BlackHole.class));
//		assertFalse(sos.hasGalacticIncumbentsBetween(ss1.getLocation(), ss3.getLocation(), BlackHole.class));
//		assertFalse(sos.hasGalacticIncumbentsBetween(ss3.getLocation(), ss2.getLocation(), BlackHole.class));
//
//		Craft craft1 = newCraft(coloniserDesign, player, ss1);
//		CraftTaskColonise task1 = new CraftTaskColonise(planet2, player);
//		task1.add(craft1);
//		
//		assertEquals(ss3, craft1.getNavigator().getDestination().getStellarObject());
//
//		addEnemy();
//        MotEngine.instance.getRelations().setWar(player, enemyPlayer);
//		enemySS.setLocation(new Location(200,100));
//		ss2.setLocation(new Location(150,10));
//		Craft craft2 = newCraft(fighterDesign, player, ss1);
//		CraftTaskAttack task2 = new CraftTaskAttack(enemyPlanet, player, 1);
//		task2.add(craft2);
//		task2.assignTaskToFleet();
//		
//		// Should not go via ss2 because it will pass through the black hole!
//		assertEquals(ss3, craft2.getNavigator().getDestination().getStellarObject());
    }
    
    protected void tearDown() {
    	Logger.flushLog();
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
    
}