/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Area;
import mot.common.bean.Geometry;
import mot.common.bean.IntersectionLocations;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.Rectangle;
import mot.common.setup.OptionsFactory;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.MovementCostCalculator;
import mot.engine.common.ServerImageObject;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;

public class GeometryTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(GeometryTest.class);
        return suite;
    }

    public GeometryTest(String name) {
        super(name);
    }

    protected void setUp() {
    }
    
    private Player player1,player2;
    private SolarSystem ss1;
    private Planet planet1;
    private Craft craft1,craft2;
    private BattleCraft [] bca;
    
    private void createBattle() throws ApplicationError, ValidationError {
        MotEngine.newInstance();
        SetupOptions options = OptionsFactory.getGeneralOptions();;
        String [] playerNames = { "Humans", "Bears", "Ghosts" };
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        MotEngine.getInstance().getRelations().setWar(player1, player2);
        
        craft1 = newCraft(fighterDesign, player1, ss1);
        craft2 = newCraft(fighterDesign, player2, ss1);

        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        
        bca = battle.getCrafts();    
    }
    
    public void testDistance() throws Exception {
//    	double d1,d2;
//    	System.out.println(d1=Location.getDistanceBetween(518, 177, 531, 102));
//    	System.out.println(d2=Location.getDistanceBetween(518, 177, 322, 144));
//    	double r1,r2;
//    	System.out.println(r1=d1/20.1);
//    	System.out.println(r2=d2/52.8);
//    	double a1,a2,a3,a4;
//    	System.out.println(a1=Location.getDistanceBetween(69,34, 80,34)/3.77);
//    	System.out.println(a2=Location.getDistanceBetween(80,34, 142,211)/3.77);
//    	System.out.println(a2=Location.getDistanceBetween(142,211, 33,249)/3.77);
//    	System.out.println(a2=Location.getDistanceBetween(33,249, 80,34)/3.77);
    	assertEquals(76, new Location(518,177).getDistanceTo(531, 102), 0.1);
    }
    
    public void testRectangle() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testRectangle() *************************");
        Rectangle r1 = new Rectangle(10,10,5,5);
		assertTrue(r1.contains(10,10));
        assertTrue(r1.contains(10,15));
        assertTrue(r1.contains(12,12));
        assertTrue(r1.contains(15,15));
        assertFalse(r1.contains(9,11));
        assertFalse(r1.contains(11,9));
        assertFalse(r1.contains(16,15));
        assertFalse(r1.contains(15,16));

		assertTrue(r1.intersects(new Rectangle(8,8,5,5)));
		assertTrue(r1.intersects(new Rectangle(14,14,5,5)));
		assertTrue(r1.intersects(new Rectangle(14,8,5,5)));
		assertTrue(r1.intersects(new Rectangle(8,14,5,5)));
		assertTrue(r1.intersects(new Rectangle(4,4,15,15)));
		assertTrue(r1.intersects(new Rectangle(11,8,15,15)));
		assertTrue(r1.intersects(new Rectangle(11,15,15,15)));
		assertTrue(r1.intersects(new Rectangle(15,4,15,15)));
		assertFalse(r1.intersects(new Rectangle(4,4,5,5)));
		assertFalse(r1.intersects(new Rectangle(16,16,5,5)));
		assertFalse(r1.intersects(new Rectangle(4,10,5,5)));
		assertFalse(r1.intersects(new Rectangle(16,10,5,5)));
		assertFalse(r1.intersects(new Rectangle(10,4,5,5)));
		assertFalse(r1.intersects(new Rectangle(10,16,5,5)));
    }    
    

    public void testArea() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testArea() *************************");
        Area r1 = new Area(10,10,15,15);
		assertTrue(r1.contains(10,10));
        assertTrue(r1.contains(10,15));
        assertTrue(r1.contains(12,12));
        assertTrue(r1.contains(15,15));
        assertFalse(r1.contains(9,11));
        assertFalse(r1.contains(11,9));
        assertFalse(r1.contains(16,15));
        assertFalse(r1.contains(15,16));

		assertTrue(r1.intersects(new Area(8,8,13,13)));
		assertTrue(r1.intersects(new Area(14,14,19,19)));
		assertTrue(r1.intersects(new Area(14,8,19,13)));
		assertTrue(r1.intersects(new Area(8,14,13,19)));
		assertTrue(r1.intersects(new Area(4,4,19,19)));
		assertTrue(r1.intersects(new Area(11,8,26,23)));
		assertTrue(r1.intersects(new Area(11,15,26,30)));
		assertTrue(r1.intersects(new Area(15,4,30,19)));
		assertFalse(r1.intersects(new Area(4,4,9,9)));
		assertFalse(r1.intersects(new Area(16,16,21,21)));
		assertFalse(r1.intersects(new Area(4,10,9,15)));
		assertFalse(r1.intersects(new Area(16,10,21,15)));
		assertFalse(r1.intersects(new Area(10,4,15,9)));
		assertFalse(r1.intersects(new Area(10,16,15,21)));
		
		assertEquals(1414, new Area(0,0,1000,1000).getDiameter());
		
		Area a1 = new Area(new Location(100,100), 20,20);
		
		assertEquals(new Location(89,100), a1.getClosestEdgeTo(new Location(50,100)));
		assertEquals(new Location(89,89), a1.getClosestEdgeTo(new Location(50,50)));
		assertEquals(new Location(100,111), a1.getClosestEdgeTo(new Location(100,150)));
		assertEquals(new Location(111,111), a1.getClosestEdgeTo(new Location(150,150)));
		assertEquals(new Location(89,111), a1.getClosestEdgeTo(new Location(75,150)));

		assertEquals(new Location(90,100), a1.getFirstIntersection(new Location(50,100), new Location(200,100)));
		assertEquals(new Location(110,100), a1.getFirstIntersection(new Location(200,100), new Location(50,100)));
		assertEquals(new Location(90,90), a1.getFirstIntersection(new Location(50,50), new Location(200,200)));
		assertEquals(new Location(110,100), a1.getFirstIntersection(new Location(200,100), new Location(50,100)));
		assertEquals(new Location(110,110), a1.getFirstIntersection(new Location(150,150), new Location(50,50)));
		assertEquals(new Location(100,90), a1.getFirstIntersection(new Location(100,50), new Location(100,150)));
		assertEquals(new Location(100,110), a1.getFirstIntersection(new Location(100,150), new Location(100,50)));

		assertEquals(new Location(90,90), a1.getFirstIntersection(new Location(50,50), new Location(100,100)));
		assertEquals(new Location(100,90), a1.getFirstIntersection(new Location(100,100), new Location(100,50)));
		assertEquals(null, a1.getFirstIntersection(new Location(95,95), new Location(105,105)));

		assertEquals(new Location(93,90), a1.getFirstIntersection(new Location(88,80), new Location(110,120)));
		assertEquals(new Location(90,107), a1.getFirstIntersection(new Location(85,110), new Location(120,95)));
    }    
    
    public void testDirection() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testDirection() *************************");
        Location loc1 = new Location(100,100);

        Location loc2 = new Location(100,0);
        Location loc3 = new Location(200,100);
        Location loc4 = new Location(100,200);
        Location loc5 = new Location(0,100);
        
        Location loc6 = new Location(150,50);
        Location loc7 = new Location(150,150);
        Location loc8 = new Location(50,150);
        Location loc9 = new Location(50,50);
        
        assertEquals("Angle is not correct", 0, loc1.getDirectionFor(loc2));
        assertEquals("Angle is not correct", 90, loc1.getDirectionFor(loc3));
        assertEquals("Angle is not correct", 180, loc1.getDirectionFor(loc4));
        assertEquals("Angle is not correct", 270, loc1.getDirectionFor(loc5));

        assertEquals("Angle is not correct", 45, loc1.getDirectionFor(loc6));
        assertEquals("Angle is not correct", 135, loc1.getDirectionFor(loc7));
        assertEquals("Angle is not correct", 225, loc1.getDirectionFor(loc8));
        assertEquals("Angle is not correct", 315, loc1.getDirectionFor(loc9));
    }

    public void testShiftInDirection() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testShiftInDirection() *************************");
        
        Location loc = new Location(100,100);
        loc.shiftInDirection(0, 10);
        assertEquals(new Location(100,90), loc);
        
        loc = new Location(100,100);
        loc.shiftInDirection(90, 10);
        assertEquals(new Location(110,100), loc);
        
        loc = new Location(100,100);
        loc.shiftInDirection(180, 10);
        assertEquals(new Location(100,110), loc);
        
        loc = new Location(100,100);
        loc.shiftInDirection(270, 10);
        assertEquals(new Location(90,100), loc);
        
        loc = new Location(100,100);
        loc.shiftInDirection(45, 10);
        assertEquals(new Location(107,93), loc);
        
        loc = new Location(100,100);
        loc.shiftInDirection(135, 10);
        assertEquals(new Location(107,107), loc);
        
        loc = new Location(100,100);
        loc.shiftInDirection(225, 10);
        assertEquals(new Location(93,107), loc);
        
        loc = new Location(100,100);
        loc.shiftInDirection(315, 10);
        assertEquals(new Location(93,93), loc);
    }

    public void testRotation() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testRotation() *************************");
        Location loc = new Location(90,100);
        loc.rotateAround(90, 100, 100);
        assertEquals(loc, new Location(100,90));

        loc = new Location(90,100);
        loc.rotateAround(45, 100, 100);
        assertEquals(loc, new Location(93,93));

        loc = new Location(90,100);
        loc.rotateAround(135, 100, 100);
        assertEquals(loc, new Location(107,93));
    }
    
    public void testLineIntersection() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testShiftInDirection() *************************");
        
        Location line1Start = new Location(10,0);
        Location line1End = new Location(10,10);
        Location line2Start = new Location(5,5);
        Location line2End = new Location(15,5);
        
        Location r = Geometry.getIntersection(line1Start, line1End, line2Start, line2End);
        
        assertEquals(10, r.getX());
        assertEquals(5, r.getY());

        line1Start = new Location(0,0);
        line1End = new Location(100,100);
        line2Start = new Location(0,50);
        line2End = new Location(100,50);
        
        r = Geometry.getIntersection(line1Start, line1End, line2Start, line2End);
        
        assertEquals(50, r.getX());
        assertEquals(50, r.getY());

        line1Start = new Location(0,0);
        line1End = new Location(10,10);
        line2Start = new Location(0,20);
        line2End = new Location(20,20);
        
       	r = Geometry.getIntersection(line1Start, line1End, line2Start, line2End);
        
        assertEquals(null, r);

        line1Start = new Location(0,0);
        line1End = new Location(100,0);
        line2Start = new Location(0,50);
        line2End = new Location(100,50);
        
       	r = Geometry.getIntersection(line1Start, line1End, line2Start, line2End);
        
        assertEquals(null, r);
    }

    public void testGetIntersections() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testGetIntersections() *************************");
        Rectangle rectangle = new Rectangle(50, 50, 50, 50);
        IntersectionLocations r;
        
        r = Geometry.getIntersections(rectangle, new Location(28,84), new Location(81,39));
        assertEquals(new Location(50,65), r.getFirstEdge());
        assertEquals(new Location(68,50), r.getSecondEdge());

        r = Geometry.getIntersections(rectangle, new Location(81,39), new Location(28,84));
        assertEquals(new Location(50,65), r.getSecondEdge());
        assertEquals(new Location(68,50), r.getFirstEdge());
        
        r = Geometry.getIntersections(rectangle, new Location(92,101), new Location(121,63));
        assertEquals(new Location(92,100), r.getFirstEdge());
        assertEquals(new Location(100,90), r.getSecondEdge());
        
        r = Geometry.getIntersections(rectangle, new Location(121,63), new Location(92,101));
        assertEquals(new Location(92,100), r.getSecondEdge());
        assertEquals(new Location(100,90), r.getFirstEdge());
        
        r = Geometry.getIntersections(rectangle, new Location(90,67), new Location(90,39));
        assertEquals(new Location(90,50), r.getFirstEdge());
        assertEquals(null, r.getSecondEdge());    
        
        r = Geometry.getIntersections(rectangle, new Location(90,67), new Location(90,101));
        assertEquals(new Location(90,100), r.getFirstEdge());
        assertEquals(null, r.getSecondEdge());
        
        r = Geometry.getIntersections(rectangle, new Location(86,79), new Location(113,109));
        assertEquals(new Location(100,94), r.getFirstEdge());
        assertEquals(null, r.getSecondEdge());
        
        r = Geometry.getIntersections(rectangle, new Location(33,60), new Location(56,53));
        assertEquals(new Location(50,54), r.getFirstEdge());
        assertEquals(null, r.getSecondEdge());

    }
    
    public void testClosestPointToALine() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testShiftInDirection() *************************");
        
        Location line1Start = new Location(10,0);
        Location line1End = new Location(10,10);
        Location point = new Location(5,5);
        
        Location r = Geometry.getClosestPoint(line1Start, line1End, point);
        
        assertEquals(10, r.getX());
        assertEquals(5, r.getY());

        line1Start = new Location(0,0);
        line1End = new Location(10,10);
        point = new Location(0,5);
        
        r = Geometry.getClosestPoint(line1Start, line1End, point);
        
        assertEquals(2, r.getX());
        assertEquals(2, r.getY());

        line1Start = new Location(5,5);
        line1End = new Location(10,10);
        point = new Location(0,1);
        
        r = Geometry.getClosestPoint(line1Start, line1End, point);
        
        assertEquals(5, r.getX());
        assertEquals(5, r.getY());

        line1Start = new Location(5,5);
        line1End = new Location(10,10);
        point = new Location(20,20);
        
        r = Geometry.getClosestPoint(line1Start, line1End, point);
        
        assertEquals(10, r.getX());
        assertEquals(10, r.getY());
    
        line1Start = new Location(0,0);
        line1End = new Location(10,10);
        point = new Location(5,5);
        
        r = Geometry.getClosestPoint(line1Start, line1End, point);
        
        assertEquals(5, r.getX());
        assertEquals(5, r.getY());
    }

    public void testMovementCostCalculator() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testMovementCostCalculator() *************************");
        
        MovementCostCalculator mcc = new MovementCostCalculator(400, 200);
        
        assertEquals(150, mcc.getMovementRemainingAfterTurn(100));
        assertEquals(50, mcc.getMovementRemainingAfterTurn(300));
        
        assertEquals(true, mcc.canMoveFromTo(new LocationAndDirection(100,100, 90), new Location(200,100)));
        assertEquals(true, mcc.canMoveFromTo(new LocationAndDirection(100,100, 90), new Location(300,100)));
        assertEquals(false, mcc.canMoveFromTo(new LocationAndDirection(100,100, 90), new Location(310,100)));
        assertEquals(true, mcc.canMoveFromTo(new LocationAndDirection(100,100, 40), new Location(275,100)));
        assertEquals(false, mcc.canMoveFromTo(new LocationAndDirection(100,100, 40), new Location(276,100)));

        createBattle();
        bca[0].setLocation(new Location(500,500));
        bca[0].setDirection(90);

        assertEquals(200, mcc.getMovementRemainingAfterTurn(bca[0], new Location(700,500)));
        assertEquals(155, mcc.getMovementRemainingAfterTurn(bca[0], new Location(500,600)));
        assertEquals(110, mcc.getMovementRemainingAfterTurn(bca[0], new Location(400,500)));
        assertEquals(132, mcc.getMovementRemainingAfterTurn(bca[0], new Location(400,400)));
        assertEquals(110, mcc.getMovementRemainingAfterTurn(bca[0], new Location(400,500)));

        assertEquals(600,500, mcc.getClosestLocationTo(bca[0], new Location(600,500)));
        assertEquals(700,500, mcc.getClosestLocationTo(bca[0], new Location(700,500)));
        
        bca[0].setDirection(40);
        assertEquals(675,500, mcc.getClosestLocationTo(bca[0], new Location(700,500)));
        bca[0].setDirection(140);
        assertEquals(675,500, mcc.getClosestLocationTo(bca[0], new Location(700,500)));
        bca[0].setDirection(350);
        assertEquals(650,500, mcc.getClosestLocationTo(bca[0], new Location(700,500)));
        bca[0].setDirection(190);
        assertEquals(650,500, mcc.getClosestLocationTo(bca[0], new Location(700,500)));

        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(90);
        assertEquals(25, mcc.getScoreToTarget(bca[0], new Location(475,500), new Location(500,500)));
        
        bca[0].setLocation(new Location(300,500));
        bca[0].setDirection(90);
        assertEquals(25, mcc.getScoreToTarget(bca[0], new Location(500,500), new Location(525,500)));
        
        bca[0].setLocation(new Location(290,500));
        bca[0].setDirection(90);
        assertEquals(Integer.MAX_VALUE, mcc.getScoreToTarget(bca[0], new Location(500,500), new Location(525,500)));
        
        bca[0].setLocation(new Location(310,500));
        bca[0].setDirection(90);
        assertEquals(59, mcc.getScoreToTarget(bca[0], new Location(500,525), new Location(525,500)));
    }
    
    public void testServerImageObject() throws Exception {
        Logger.logDebug("*******************************  GeometryTest.testServerImageObject() *************************");
        
        createBattle();
        ServerImageObject io = bca[0];
        
        io.setLocation(new Location(500,500));
        
        assertEquals(18,18, io.getSize());
        assertEquals(12, io.getSpaceX());
        assertEquals(12, io.getSpaceY());
        
        assertEquals(12, io.getDistanceToEdgeFrom(500,500, 0));
        assertEquals(12, io.getDistanceToEdgeFrom(500,500, 90));
        assertEquals(12, io.getDistanceToEdgeFrom(500,500, 180));
        assertEquals(12, io.getDistanceToEdgeFrom(500,500, 270));
        
        assertEquals(17, io.getDistanceToEdgeFrom(505,505, 0));
        assertEquals(7, io.getDistanceToEdgeFrom(505,505, 90));
        assertEquals(7, io.getDistanceToEdgeFrom(505,505, 180));
        assertEquals(17, io.getDistanceToEdgeFrom(505,505, 270));
        
        assertEquals(7, io.getDistanceToEdgeFrom(495,495, 0));
        assertEquals(17, io.getDistanceToEdgeFrom(495,495, 90));
        assertEquals(17, io.getDistanceToEdgeFrom(495,495, 180));
        assertEquals(7, io.getDistanceToEdgeFrom(495,495, 270));
        
        assertEquals(12, io.getDistanceToEdgeFrom(495,500, 0));
        assertEquals(17, io.getDistanceToEdgeFrom(495,500, 90));
        assertEquals(12, io.getDistanceToEdgeFrom(495,500, 180));
        assertEquals(7, io.getDistanceToEdgeFrom(495,500, 270));
        
        assertEquals(17, io.getDistanceToEdgeFrom(500,505, 0));
        assertEquals(12, io.getDistanceToEdgeFrom(500,505, 90));
        assertEquals(7, io.getDistanceToEdgeFrom(500,505, 180));
        assertEquals(12, io.getDistanceToEdgeFrom(500,505, 270));
        
        assertEquals(29, io.getOverlapInDirection(new Location(500,505), bca[1], 0));
        assertEquals(24, io.getOverlapInDirection(new Location(500,505), bca[1], 90));
        assertEquals(19, io.getOverlapInDirection(new Location(500,505), bca[1], 180));
        assertEquals(24, io.getOverlapInDirection(new Location(500,505), bca[1], 270));
        
        bca[1].setLocation(new Location(495,500));
        assertEquals(24, io.getOverlapInDirection(bca[1].getLocation(), bca[1], 0));
        assertEquals(29, io.getOverlapInDirection(bca[1].getLocation(), bca[1], 90));
        assertEquals(24, io.getOverlapInDirection(bca[1].getLocation(), bca[1], 180));
        assertEquals(19, io.getOverlapInDirection(bca[1].getLocation(), bca[1], 270));
        
    }
    
    
    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}