/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.setup.SetupOptions;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.setup.MotFactory;

/**
 * Tests Prizes Work - When a craft arrives at a planet for the first time
 * it activates a prize for its owner. Future visitors do not get the prize.
 */
public class GiftTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(GiftTest.class);
        return suite;
    }

    public GiftTest(String name) {
        super(name);
    }

    private SetupOptions options;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;

    protected void setUp() throws Exception {
        MotEngine.newInstance();
        setupConfigs();
        
        String [] playerNames = { "TestPlayer1", "TestPlayer2" };
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player1 = newPlayer(playerNames[0], blue, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 120, 100);
        planet2 = newPlanet("SS2-A", ss2, null);
        ss3 = newSolarSystem("SS3", 120, 100);
        planet3 = newPlanet("SS3-A", ss3, null);
    }
    

    public void testRound1GiftAirman() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testRound1GiftAirman() *************************");
        setUp();
        
        MotEngine.instance.setDate(3075);
        player1.setAlignment(ANCIENT_AIR_MAN);
        MotFactory.createAncients(null);
        
        assertEquals(0, player1.getPlayerCharacter().getPsiRating());
        
        MotEngine.getInstance().getMonsterPlayer().endTurn();
        
        assertEquals(5, player1.getPlayerCharacter().getPsiRating());
    }

    public void testRound1GiftWaterMan() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testRound1GiftWaterMan() *************************");
        setUp();
        
        MotEngine.instance.setDate(3075);
        player1.setAlignment(ANCIENT_WATER_MAN);
        MotFactory.createAncients(null);
        
        assertEquals(0, player1.getPlayerCharacter().getCloakLevel());
        
        MotEngine.getInstance().getMonsterPlayer().endTurn();
        
        assertEquals(50, player1.getPlayerCharacter().getCloakLevel());
    }

    public void testRound2GiftAirman() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testRound2GiftAirman() *************************");
        setUp();
        
        MotEngine.instance.setDate(3250);
        player1.setAlignment(ANCIENT_WATER_MAN);
        MotFactory.createAncients(null);
        
        assertEquals(false, player1.hasDiscovered(tech_stationMissileBase));
        
        MotEngine.getInstance().getMonsterPlayer().endTurn();
        
        assertEquals(true, player1.hasDiscovered(tech_stationMissileBase));
    }

    public void testRound2GiftWaterMan() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testRound2GiftWaterMan() *************************");
        setUp();
        
        MotEngine.instance.setDate(3250);
        player1.setAlignment(ANCIENT_AIR_MAN);
        MotFactory.createAncients(null);
        
        assertEquals(false, player1.hasDiscovered(tech_dimensionChamber));
        
        MotEngine.getInstance().getMonsterPlayer().endTurn();
        
        assertEquals(true, player1.hasDiscovered(tech_dimensionChamber));
    }


    protected void tearDown() {
    	Logger.flushLog();
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}