/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Location;
import mot.common.bean.SectorOwnerGrid;
import mot.common.bean.Size;
import mot.common.designs.Orbital;
import mot.common.morals.MoralChoices;
import mot.common.resources.ResourceBoralium;
import mot.common.setup.SetupOptions;
import mot.common.technology.Factories;
import mot.common.technology.Technologies;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.common.Outpost;
import mot.engine.planets.InfluenceGrid;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;

/**
 * Tests that a craft moves between two solar system correctly.
 * Test the General objects.
 */
public class InfluenceTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(mot.test.InfluenceTest.class);
        return suite;
    }

    public InfluenceTest(String name) {
        super(name);
    }

    private SetupOptions options;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;
    private SolarSystem ss5;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;
    private Planet planet5;


    /**
     * The galaxy has four sectors and planets are laid out
     * as follows;
     * 
     *      0          100      200
     * 	0   ---------------------
     *      |         |         |
     *      |  p1 p5  |   p2    |
     *      |         |         |
     * 	100 ---------------------
     *      |         |         |
     *      |  p3     |   p4    |
     *      |         |         |
     * 	200 ---------------------
     * 
     */
    protected void setUp() throws Exception {
        MotEngine.newInstance();
        setupConfigs();
        
        String [] playerNames = { "TestPlayer1", "TestPlayer2" };
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 50, 50);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 150, 50);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 50, 150);
        planet3 = newPlanet("SS3-A", ss3, null, 100, 1.0);

        ss4 = newSolarSystem("SS4", 150, 150);
        planet4 = newPlanet("SS4-A", ss4, null, 100, 1.0);

        ss5 = newSolarSystem("SS5", 99, 50);
        planet5 = newPlanet("SS5-A", ss5, null, 110, 1.0);
        
        MotEngine.instance.getStellarObjects().setSize(new Size(200,200));
    }
    

    /**
     * Tests that a players influence is calculated correctly and that each sector is 
     * owned by the correct player.
     */
    public void testSectorOwnership() throws Exception {
        Logger.logDebug("*******************************  InfluenceTest.testSectorOwnership() *************************");
        
        planet1.setInfluence(20);
        planet2.setInfluence(20);
        InfluenceGrid grid = MotEngine.instance.getInfluence();
        
        assertEquals(2, grid.getQtyCols());
        assertEquals(2, grid.getQtyRows());
        assertEquals(player1, grid.getOwnerBySector(0,0));
        assertEquals(player2, grid.getOwnerBySector(0,1));
        assertEquals(null, grid.getOwnerBySector(1,0));
        assertEquals(null, grid.getOwnerBySector(1,1));

        assertEquals(player1, grid.getOwnerByLocation(0,0));
        assertEquals(player1, grid.getOwnerByLocation(99,99));
        assertEquals(player2, grid.getOwnerByLocation(100,0));
        assertEquals(null, grid.getOwnerByLocation(100,100));

        assertEquals(20, grid.getInfluenceBySector(0, 0, player1));
        assertEquals(0, grid.getInfluenceBySector(0, 1, player1));
        assertEquals(0, grid.getInfluenceBySector(1, 0, player1));
        assertEquals(0, grid.getInfluenceBySector(1, 1, player1));

        planet1.setInfluence(108);
        grid.update();

        assertEquals(108, grid.getInfluenceBySector(0, 0, player1));
        assertEquals(8, grid.getInfluenceBySector(0, 1, player1));
        assertEquals(8, grid.getInfluenceBySector(1, 0, player1));
        assertEquals(8, grid.getInfluenceBySector(1, 1, player1));
        
        assertEquals(0, grid.getInfluenceBySector(0, 0, player2));
        assertEquals(20, grid.getInfluenceBySector(0, 1, player2));
        assertEquals(0, grid.getInfluenceBySector(1, 0, player2));
        assertEquals(0, grid.getInfluenceBySector(1, 1, player2));

        assertEquals(player1, grid.getOwnerBySector(0,0));
        assertEquals(player2, grid.getOwnerBySector(0,1));
        assertEquals(player1, grid.getOwnerBySector(1,0));
        assertEquals(player1, grid.getOwnerBySector(1,1));
        
        SectorOwnerGrid sog = grid.getSectorOwnerGrid();
        assertEquals(player1.getName(), sog.getSectorOwner(0, 0).getPlayer());
        assertEquals(player2.getName(), sog.getSectorOwner(0, 1).getPlayer());
        assertEquals(player1.getName(), sog.getSectorOwner(1, 0).getPlayer());
        assertEquals(player1.getName(), sog.getSectorOwner(1, 1).getPlayer());

        planet2.setInfluence(5);
        grid.update();

        assertEquals(player1, grid.getOwnerBySector(0,0));
        assertEquals(player1, grid.getOwnerBySector(1,0));
        assertEquals(player1, grid.getOwnerBySector(0,1));
        assertEquals(player1, grid.getOwnerBySector(1,1));

        assertEquals(108, grid.getInfluenceByLocation(0,0, player1));
        assertEquals(108, grid.getInfluenceByLocation(99,99, player1));
        assertEquals(8, grid.getInfluenceByLocation(100,0, player1));
        assertEquals(8, grid.getInfluenceByLocation(100,100, player1));

        assertEquals(0, grid.getInfluenceByLocation(0,0, player2));
        assertEquals(0, grid.getInfluenceByLocation(99,99, player2));
        assertEquals(5, grid.getInfluenceByLocation(100,0, player2));
        assertEquals(0, grid.getInfluenceByLocation(100,100, player2));

    }
 
    /**
     * Tests that a players influence is calculated correctly and that each sector is 
     * owned by the correct player - USES TWO PLANETS IN A SINGAL SECTOR.
     */
    public void testSectorOwnershipWith3ColoniesInSector() throws Exception {
        Logger.logDebug("*******************************  InfluenceTest.testSectorOwnershipWith3ColoniesInSector() *************************");
        
        Planet planet1b = newPlanet("SS1-B", ss1, player1);
        
        SolarSystem ss1c = newSolarSystem("SSX", 90, 90);
        Planet planet1c = newPlanet("SSX-A", ss1c, player1);

        planet1.setInfluence(110);
        planet2.setInfluence(110);

        planet1b.setInfluence(90);
        planet1c.setInfluence(30);

        InfluenceGrid grid = MotEngine.instance.getInfluence();
        grid.update();

        assertEquals(230, grid.getInfluenceBySector(0,0, player1));
        assertEquals(130, grid.getInfluenceBySector(1,0, player1));
        assertEquals(130, grid.getInfluenceBySector(0,1, player1));
        assertEquals(130, grid.getInfluenceBySector(1,1, player1));

        assertEquals(10, grid.getInfluenceBySector(0,0, player2));
        assertEquals(10, grid.getInfluenceBySector(1,0, player2));
        assertEquals(110, grid.getInfluenceBySector(0,1, player2));
        assertEquals(10, grid.getInfluenceBySector(1,1, player2));

        assertEquals(player1, grid.getOwnerBySector(0,0));
        assertEquals(player1, grid.getOwnerBySector(1,0));
        assertEquals(player1, grid.getOwnerBySector(0,1));
        assertEquals(player1, grid.getOwnerBySector(1,1));

        ss2.setLocation(new Location(40,40));
        
        planet1.setInfluence(90);
        planet2.setInfluence(210);

        planet1b.setInfluence(110);
        planet1c.setInfluence(30);

        grid.update();

        assertEquals(230, grid.getInfluenceBySector(0,0, player1));
        assertEquals(130, grid.getInfluenceBySector(1,0, player1));
        assertEquals(130, grid.getInfluenceBySector(0,1, player1));
        assertEquals(130, grid.getInfluenceBySector(1,1, player1));

        assertEquals(210, grid.getInfluenceBySector(0,0, player2));
        assertEquals(110, grid.getInfluenceBySector(1,0, player2));
        assertEquals(110, grid.getInfluenceBySector(0,1, player2));
        assertEquals(110, grid.getInfluenceBySector(1,1, player2));

        assertEquals(player1, grid.getOwnerBySector(0,0));
        assertEquals(player1, grid.getOwnerBySector(1,0));
        assertEquals(player1, grid.getOwnerBySector(0,1));
        assertEquals(player1, grid.getOwnerBySector(1,1));
        
    }
    
    /**
     * Tests that a players influence is calculated correctly and that each sector is 
     * owned by the correct player - USES TWO PLANETS IN A SINGAL SECTOR.
     */
    public void testSectorOwnershipWithColonyOutpostAndSpaceStationInSameSector() throws Exception {
        Logger.logDebug("*******************************  InfluenceTest.testSectorOwnershipWithColonyOutpostAndSpaceStationInSameSector() *************************");
        
        Planet planet1b = newPlanet("SS1-B", ss1, null);
        
        Orbital mine = new Orbital(tech_spaceStation, tech_photon100, null);
        planet1b.deploy(planet1, mine);
        
        SolarSystem ssx = newSolarSystem("SSX", 90, 90);
        Orbital outpostOrbital = new Orbital(tech_outpost, tech_photon100, null);
        ssx.setOutpost(new Outpost(outpostOrbital, player1));
        
        planet1.setInfluence(200);
        planet2.setInfluence(90);

        InfluenceGrid grid = MotEngine.instance.getInfluence();
        grid.update();

        // 200 + 10 (SSA-A mining station) + 10 (SSA-B mining colony) + 5 (SSX outpost) 
        assertEquals(225, grid.getInfluenceBySector(0,0, player1));
        assertEquals(125, grid.getInfluenceBySector(1,0, player1));
        assertEquals(125, grid.getInfluenceBySector(0,1, player1));
        assertEquals(125, grid.getInfluenceBySector(1,1, player1));

        assertEquals(0, grid.getInfluenceBySector(0,0, player2));
        assertEquals(0, grid.getInfluenceBySector(1,0, player2));
        assertEquals(90, grid.getInfluenceBySector(0,1, player2));
        assertEquals(0, grid.getInfluenceBySector(1,1, player2));

        assertEquals(player1, grid.getOwnerBySector(0,0));
        assertEquals(player1, grid.getOwnerBySector(1,0));
        assertEquals(player1, grid.getOwnerBySector(0,1));
        assertEquals(player1, grid.getOwnerBySector(1,1));
        
        //
        // Test mining stations grow influence on the mined planet
        //
        
        for (int i = 0; i < 40; i++) {
        	planet1b.endTurn();
        }
        
        grid.update();

        // 200 + 10 (SSA-A mining station) + 10+10 (SSA-B mining colony) + 5 (SSX outpost) 
        assertEquals(235, grid.getInfluenceBySector(0,0, player1));
    }
    
    /**
     * Tests that a players influence is calculated correctly and that each sector is 
     * owned by the correct player - USES TWO PLANETS IN A SINGAL SECTOR.
     *      0          100      200
     * 	0   ---------------------
     *      |         |         |
     *      |  p1 p5  |   p2    |
     *      |         |         |
     * 	100 ---------------------
     *      |         |         |
     *      |  p3     |   p4    |
     *      |         |         |
     * 	200 ---------------------
     */
    public void testInfluenceWithColonyAndSpaceStationInDifferentSector() throws Exception {
        Logger.logDebug("*******************************  InfluenceTest.testInfluenceWithColonyAndSpaceStationInDifferentSector() *************************");
        
        planet1.setInfluence(99);
        planet2.setInfluence(5);

        InfluenceGrid grid = MotEngine.instance.getInfluence();
        grid.update();

        assertEquals(99, grid.getInfluenceBySector(0,0, player1));
        assertEquals(0, grid.getInfluenceBySector(1,0, player1));
        assertEquals(0, grid.getInfluenceBySector(0,1, player1));
        assertEquals(0, grid.getInfluenceBySector(1,1, player1));

        assertEquals(0, grid.getInfluenceBySector(0,0, player2));
        assertEquals(0, grid.getInfluenceBySector(1,0, player2));
        assertEquals(5, grid.getInfluenceBySector(0,1, player2));
        assertEquals(0, grid.getInfluenceBySector(1,1, player2));

        Orbital outpost = new Orbital(tech_outpost, null, null);
        ss5.deploy(outpost, player1);

        grid.update();

        assertEquals(104, grid.getInfluenceBySector(0,0, player1));
        assertEquals(4, grid.getInfluenceBySector(1,0, player1));
        assertEquals(4, grid.getInfluenceBySector(0,1, player1));
        assertEquals(4, grid.getInfluenceBySector(1,1, player1));

        assertEquals(0, grid.getInfluenceBySector(0,0, player2));
        assertEquals(0, grid.getInfluenceBySector(1,0, player2));
        assertEquals(5, grid.getInfluenceBySector(0,1, player2));
        assertEquals(0, grid.getInfluenceBySector(1,1, player2));

        planet3.setResource(new ResourceBoralium());
        Orbital orbital = new Orbital(tech_spaceStation, null, null);
        planet3.deploy(planet1, orbital);

        grid.update();

        assertEquals(114, grid.getInfluenceBySector(0,0, player1)); // 20 from mining colony + 14 from neighbour
        assertEquals(34, grid.getInfluenceBySector(1,0, player1)); // 20 from mining colony + 14 from neighbour 
        assertEquals(14, grid.getInfluenceBySector(0,1, player1));
        assertEquals(14, grid.getInfluenceBySector(1,1, player1));

        //
        // Test mining stations grow influence on the mined planet
        // at double rate because of resource
        //
        
        for (int i = 0; i < 40; i++) {
        	planet3.endTurn();
        }
        
        grid.update();

        // 14 + 20 (starting influence) + 20 (influence growth) 
        assertEquals(54, grid.getInfluenceBySector(1,0, player1));
    }
    
    /**
     * Tests that influence generated from one planet affects planets owned
     * by other players planets.
     */
    public void testPlanetInfluence() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testPlanetInfluence() *************************");

        //
        // TEST THAT A PLANET DOES ACCUMULATE INFLUENCE
        //
        
        planet1.setQtyFactories(0.0);
        planet2.setQtyFactories(0.0);
        planet1.setPopulation(100);
        planet2.setPopulation(100);
        
        assertEquals(1.0, planet1.getInfluence(), 0.001);
        assertEquals(0.5, planet1.getInfluenceGrowth(), 0.001);

        planet1.endTurn();
        player1.setFactories(new Factories(Technologies.getFactory(TestTechs.FACT_GALACTIC_NEWS)));
        player1.getPlayerCharacter().setEloquenceBonus(0.1);
        
        assertEquals(1.5, planet1.getInfluenceAsDouble(), 0.001);
        assertEquals(0.60, planet1.getInfluenceGrowth(), 0.001);

        planet1.endTurn();

        assertEquals(2.10, planet1.getInfluenceAsDouble(), 0.001);
        assertEquals(0.60, planet1.getInfluenceGrowth(), 0.001);

        player1.add(MoralChoices.getMoralChoice(TEST_MORAL_CHOICE));
        
        assertEquals(0.70, planet1.getInfluenceGrowth(), 0.001);
        
        //
        // TEST THAT A PLANET IS DOMINATED WHEN IN A SECTOR OWNED BY ANOTHER PLAYER
        // **** Move ss2 into same sector as ss1
        //
        
        assertEquals(100, planet1.getIndustryWork(), 0.001);

        ss2.setLocation(new Location(60,60));
        planet1.setInfluence(300);
        planet2.setInfluence(10);
        
        assertEquals(100, planet1.getIndustryWork(), 0.001);
        assertEquals(100, planet2.getIndustryWork(), 0.001);
        
        InfluenceGrid grid = MotEngine.instance.getInfluence();
        grid.update();

        assertEquals(player1, grid.getOwnerByStellarObject(ss1));
        assertEquals(false, planet1.isDominated());
        assertEquals(true, planet2.isDominated());
        assertEquals(100, planet1.getIndustryWork(), 0.001);
        assertEquals(75, planet2.getIndustryWork(), 0.001);

        planet1.setInfluence(10);
        planet2.setInfluence(30);

        grid.update();

        assertEquals(player2, grid.getOwnerByStellarObject(ss1));
        assertEquals(true, planet1.isDominated());
        assertEquals(false, planet2.isDominated());
        assertEquals(75, planet1.getIndustryWork(), 0.001);
        assertEquals(100, planet2.getIndustryWork(), 0.001);
    }
    
    /**
     * Tests that a players influence is calculated correctly and that each sector is 
     * owned by the correct player - USES TWO PLANETS IN A SINGAL SECTOR.
     *      200        300      400    500
     * 	300 ------------------------------
     *      |         |         |        |
     *      |         |         |        |
     *      |         |         |        |
     * 	400 ------------------------------
     *      |         |         |        |
     *      |         |    p1   |        |  row 4
     *      |         |         |        |
     * 	500 ------------------------------
     *                    col 3
     */
    public void testLargeGalaxyEdge() throws Exception {
        Logger.logDebug("*******************************  InfluenceTest.testLargeGalaxyEdge() *************************");
        
        MotEngine.instance.getStellarObjects().setSize(new Size(800,500));
        ss1.setLocation(new Location(350,450));
        
        InfluenceGrid grid = MotEngine.instance.getInfluence();
        assertEquals(grid.getInfluenceBySector(4,3, player1), grid.getInfluenceByLocation(350, 450, player1));
        
        planet1.setInfluence(99);

        grid.update();

        assertEquals(0, grid.getInfluenceBySector(3,2, player1));
        assertEquals(0, grid.getInfluenceBySector(3,3, player1));
        assertEquals(0, grid.getInfluenceBySector(3,4, player1));
        assertEquals(0, grid.getInfluenceBySector(4,2, player1));
        assertEquals(99, grid.getInfluenceBySector(4,3, player1));
        assertEquals(0, grid.getInfluenceBySector(4,4, player1));

        planet1.setInfluence(105);
        grid.update();

        assertEquals(5, grid.getInfluenceBySector(3,2, player1));
        assertEquals(5, grid.getInfluenceBySector(3,3, player1));
        assertEquals(5, grid.getInfluenceBySector(3,4, player1));
        assertEquals(5, grid.getInfluenceBySector(4,2, player1));
        assertEquals(105, grid.getInfluenceBySector(4,3, player1));
        assertEquals(5, grid.getInfluenceBySector(4,4, player1));

    }
    
    protected void tearDown() {
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}