/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Location;
import mot.common.bean.Nebula;
import mot.common.bean.Settings;
import mot.common.designs.CraftDesign;
import mot.common.setup.SetupOptions;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.generals.CraftTaskMaraudCraft;
import mot.engine.players.generals.CraftTaskProtect;
import mot.engine.setup.MonsterFactory;
import mot.engine.setup.MonsterFactorySmall;
import mot.engine.setup.MotFactory;

/**
 * Tests that a craft moves between two solar system correctly.
 * Test the General objects.
 */
public class KaanTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(KaanTest.class);
        return suite;
    }

    public KaanTest(String name) {
        super(name);
    	try {
			Settings.getInstance();
		} catch (ApplicationError e) {
			e.printStackTrace();
		}
    }
    
    private SetupOptions options;
    
    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;
    
    private SolarSystem ssm;
    private Planet planetm;

    private Nebula nebula;
    
    private MonsterPlayer mp;
    
    protected void setUp() throws Exception {
        MotEngine.newInstance();
        setupConfigs();
        
        MotEngine.instance.setSpaceKaanEnabled(true);
        
        String [] playerNames = { "TestPlayer1", "TestPlayer2" };
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);
        
        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);
        
        ss3 = newSolarSystem("SS3", 100, 200);
        planet3 = newPlanet("SS3-A", ss3, null, 100, 1.0);
        
        ss4 = newSolarSystem("SS4", 200, 200);
        planet4 = newPlanet("SS4-A", ss4, player1);

        nebula = MotFactory.createNebula(new Location(125,100), 100);
        
        ssm = newSolarSystem("SSM", 400, 400);
        planetm = newPlanet("SSM-1", ssm, null, 110, 1.0);
        MonsterFactory.create(planetm, MonsterFactorySmall.class);
        
        mp = MotEngine.instance.getMonsterPlayer();
    }

    /**
     * Test Basic Kaan functionality.
     */
    public void testEndBattleInNebulaReleasesKaan() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testEndBattleInNebulaReleasesKaan() *************************");

        //
        // Test Kaan is released when battle finished in a nebula and target is attacker
        //
        
        MotEngine.instance.setDate(3101);
        
        planet1.setWeapons(new Weapons(tech_missile6));
        planet2.setWeapons(new Weapons(tech_missile6));
        
        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player2, ss1);
        Craft craft3 = newCraft(fighterDesign, player2, ss2);
        
        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.start();
        battle.getCrafts()[1].destroy(null, null, null);
        //battle.endBattle();
        
		assertEquals(1, mp.getQtyKaans());
        assertEquals(1, mp.getFleet().length);
        Fleet monsterFleet = mp.getFleet()[0];
		assertEquals(1, monsterFleet.getSize());
        assertEquals(CraftTaskMaraudCraft.class, monsterFleet.getCrafts()[0].getTask().getClass());
        assertEquals(ss2, ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getTarget());
        assertEquals(1, ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getFleet().getCrafts().length);
        
        //
        // Test Kaan is correct (VALUES from craft_designs.xml)
        //
        
        Craft kaan = ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getFleet().getCrafts()[0];
        
        //
        // Test Kaan travels to target and attacks craft but NOT the planet!
        //
        
        assertEquals(ss2, ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getTarget());
        assertEquals(nebula.getLocation(), kaan.getLocation());
        assertEquals((Orbit)null, kaan.getNavigator().getOrigin());
        assertEquals(ss2, kaan.getNavigator().getDestination());
        assertEquals(2, kaan.getNavigator().getETA());
        
        monsterFleet.endTurn();
        monsterFleet.endTurn();
        
        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss2.getOrbit(), false);
        assertEquals(kaan, battle.getCrafts()[0].getCraft());
        assertEquals(craft3, battle.getCrafts()[1].getCraft());
        assertEquals(null, battle.getPlanet());
        
        battle.start();
        battle.automate();
        
        assertEquals(true, craft3.isDestroyed());
        assertEquals(false, kaan.isDestroyed());
        
        //
        // Test Kaan acquires new target (only craft left in galaxy)
        //
        
        craft1.setOrbit(new Orbit(ss4));
        mp.endTurn();
        monsterFleet.endTurn();
        
        assertEquals(ss4, ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getTarget());
        assertEquals(ss2, kaan.getNavigator().getOrigin());
        assertEquals(ss4, kaan.getNavigator().getDestination());
        assertEquals(2, kaan.getNavigator().getETA());
        
        monsterFleet.endTurn();
        monsterFleet.endTurn();
        
        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss4.getOrbit(), false);
        assertEquals(kaan, battle.getCrafts()[0].getCraft());
        assertEquals(craft1, battle.getCrafts()[1].getCraft());
        assertEquals(null, battle.getPlanet());

        battle.start();
        battle.automate();
        
        assertEquals(true, craft1.isDestroyed());
        assertEquals(false, kaan.isDestroyed());
        
        //
        // Test Kaan does nothing as there are no craft in the galaxy now
        //
        
        mp.endTurn();
        
        assertNull(((CraftTaskMaraudCraft)kaan.getTask()).getTarget());
        assertEquals(null, kaan.getNavigator());
        
        //
        // Test Kaan ignores crafts at warp
        //
        
        kaan.getWeaponBays().getWeaponBay(0).setWeapon(tech_massDriver1);

        CraftDesign pf = new CraftDesign("PF", tech_flyer100, tech_fusion40, tech_armour25, tech_energy20, tech_electronic40, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4);
        Craft craft4 = newCraft(pf, player1, ss1);
        Craft craft5 = newCraft(pf, player1, ss1);
        Fleet fleet = FleetFactory.create(craft4);
        fleet.add(craft5);
        
        fleet.moveTo(new Orbit(ss3));
        fleet.endTurn();
        
        assertEquals(2, craft4.getNavigator().getETA());
        
        mp.endTurn();
        monsterFleet.endTurn();
        
        assertNull(((CraftTaskMaraudCraft)kaan.getTask()).getTarget());
        assertEquals(null, kaan.getNavigator());
        
        //
        // Test Kaan does attack craft in a SS with no owner
        // Test Kaan destruction
        //
        
        fleet.endTurn();
        fleet.endTurn();
        MotEngine.instance.setDate(MotEngine.instance.getDate()+11);
        
        assertEquals(ss3, fleet.getOrbitStellarObject());
        
        mp.endTurn();
        monsterFleet.endTurn();
        
        assertEquals(ss3, ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getTarget());
        assertEquals(ss4, kaan.getNavigator().getOrigin());
        assertEquals(ss3, kaan.getNavigator().getDestination());
        assertEquals(2, kaan.getNavigator().getETA());
        
        monsterFleet.endTurn();
        monsterFleet.endTurn();
                
        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss3.getOrbit(), false);
        assertEquals(kaan, battle.getCrafts()[0].getCraft());
        assertEquals(craft4, battle.getCrafts()[1].getCraft());
        assertEquals(craft5, battle.getCrafts()[2].getCraft());
        assertEquals(null, battle.getPlanet());

        battle.start();
        battle.automate();
        mp.endTurn();
        
        assertEquals(true, kaan.isDestroyed());
        assertEquals(CraftTaskProtect.class, mp.getGeneral().getTasks()[0].getClass());
        
        mp.endTurn();
        
        assertEquals(CraftTaskProtect.class, mp.getGeneral().getTasks()[0].getClass());
    }


    /**
     * Test Basic Kaan functionality.
     */
    public void testSpaceKaanDisabled() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testSpaceKaanDisabled() *************************");

        //
        // Test Kaan is released when battle finished in a nebula and target is attacker
        //
        
        MotEngine.instance.setDate(3101);
        MotEngine.instance.setSpaceKaanEnabled(false);
        
        planet1.setWeapons(new Weapons(tech_missile6));
        planet2.setWeapons(new Weapons(tech_missile6));
        
        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player2, ss1);
        Craft craft3 = newCraft(fighterDesign, player2, ss2);
        
        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.start();
        battle.getCrafts()[1].destroy(null, null, null);
        //battle.endBattle();
        
		assertEquals(0, mp.getQtyKaans());
        assertNull(mp.getFleet());
    }
    
    /**
     * Test Basic Kaan functionality.
     */
    public void testKaanWaitsAtTargetWhenNotChallanged() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testKaanWaitsAtTargetWhenNotChallanged() *************************");
        
        Craft craft1 = newCraft(fighterDesign, player1, planet1);
        mp.dispatchKaan(nebula, null);
        
        assertEquals(1, mp.getFleet().length);
        Fleet monsterFleet = mp.getFleet()[0];
		assertEquals(1, monsterFleet.getSize());

		Fleet kaanFleet = ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getFleet();
        assertEquals(ss1, ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getTarget());
        assertEquals(ss1, kaanFleet.getNavigator().getDestination());
        
        //
        // craft1 runs away - as Kaan has not enemy when it arrived it blockades for 10 turns.
        //
        
        craft1.arrivedAt(new Orbit(planet2));
        kaanFleet.arrivedAt(new Orbit(planet1));
        
        mp.endTurn();
        mp.endTurn();
        
        assertEquals(ss1, ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getTarget());
        assertEquals(planet1, kaanFleet.getOrbit());
        assertEquals(null, kaanFleet.getNavigator());
        assertEquals(true, planet1.isBlockaded());
        
        MotEngine.instance.setDate(MotEngine.instance.getDate()+11);
        mp.endTurn();
     
        assertEquals(ss2, ((CraftTaskMaraudCraft)monsterFleet.getCrafts()[0].getTask()).getTarget());
        assertEquals(planet1, kaanFleet.getOrbit());
        assertEquals(ss2, kaanFleet.getNavigator().getDestination());
        
    }
    
    /**
     * Test Basic Kaan functionality.
     */
    public void testEndBattleInNebulaDoesNotAlwaysReleaseKaan() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testEndBattleInNebulaReleasesKaan() *************************");

        //
        // Test Kaan is not released when battle finished at start of game
        //

        MotEngine.instance.setDate(3050);

        Craft crafta = newCraft(fighterDesign, player1, ss1);
        Craft craftb = newCraft(fighterDesign, player2, ss1);
        
        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(crafta, true);
        battle.join(craftb, false);
        battle.setup();
        battle.getCrafts()[1].destroy(null, null, null);
        
		assertEquals(0, mp.getQtyKaans());
        assertNull(mp.getFleet());
        
        MotEngine.instance.setDate(3101);
        
        //
        // Test Kaan is released when battle finished in a nebula and target is attacker
        //
        
        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player2, ss1);
        Craft craft3 = newCraft(fighterDesign, player2, ss1);
        Craft craft4 = newCraft(fighterDesign, player2, ss1);
        
        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.start();
        battle.getCrafts()[1].destroy(null, null, null);
        
		assertEquals(1, mp.getQtyKaans());
        assertEquals(1, mp.getFleet().length);
		assertEquals(1, mp.getFleet()[0].getSize());
        assertEquals(CraftTaskMaraudCraft.class, mp.getFleet()[0].getCrafts()[0].getTask().getClass());
        assertEquals(1, mp.getGeneral().getTasks().length);        
        assertEquals(1, ((CraftTaskMaraudCraft)mp.getGeneral().getTasks()[0]).getFleet().getCrafts().length);
        
        Craft kaan = ((CraftTaskMaraudCraft)mp.getFleet()[0].getCrafts()[0].getTask()).getFleet().getCrafts()[0];
        
        //
        // Test Kaan is NOT released if once has been released less than 10 years ago
        //
        
        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft3, false);
        battle.start();
        battle.getCrafts()[1].destroy(null, null, null);
        
		assertEquals(1, mp.getQtyKaans());
        assertEquals(1, mp.getFleet().length);
		assertEquals(1, mp.getFleet()[0].getSize());
        assertEquals(1, ((CraftTaskMaraudCraft)mp.getFleet()[0].getCrafts()[0].getTask()).getFleet().getCrafts().length);
        assertEquals(1, mp.getGeneral().getTasks().length);        
        assertEquals(1, ((CraftTaskMaraudCraft)mp.getGeneral().getTasks()[0]).getFleet().getCrafts().length);
        assertEquals(1, mp.getQtyKaans());
        
        //
        // Test Kaan is released once previous release date is more than 26 years ago
        //
        
        MotEngine.instance.setDate(MotEngine.instance.getDate()+51);
        
        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft4, false);
        battle.start();
        battle.getCrafts()[1].destroy(null, null, null);
        
		assertEquals(2, mp.getQtyKaans());
        assertEquals(2, mp.getFleet().length);
		assertEquals(1, mp.getFleet()[0].getSize());
		assertEquals(1, mp.getFleet()[1].getSize());
        assertEquals(1, ((CraftTaskMaraudCraft)mp.getFleet()[0].getCrafts()[0].getTask()).getFleet().getCrafts().length);
        assertEquals(1, ((CraftTaskMaraudCraft)mp.getFleet()[1].getCrafts()[0].getTask()).getFleet().getCrafts().length);
        assertEquals(2, mp.getGeneral().getTasks().length);        
        assertEquals(1, ((CraftTaskMaraudCraft)mp.getGeneral().getTasks()[0]).getFleet().getCrafts().length);
        assertEquals(1, ((CraftTaskMaraudCraft)mp.getGeneral().getTasks()[1]).getFleet().getCrafts().length);
        assertEquals(2, mp.getQtyKaans());
        
        //
        // Test Kaan is not released if there are already two of them
        //
        
        MotEngine.instance.setDate(MotEngine.instance.getDate()+22);
        
        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft4, false);
        battle.start();
        battle.getCrafts()[1].destroy(null, null, null);
        
        assertEquals(2, mp.getFleet().length);
        assertEquals(2, mp.getQtyKaans());
        
    }
    
    public void testKaanReleasedPeriodically() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testKaanReleasedPeriodically() *************************");

        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        
        
        mp.setNextKaanAttackDate(3201);
        MotEngine.instance.setDate(3200);
        mp.endTurn();
        
        assertNull(mp.getFleet());
		assertEquals(1, mp.getCrafts().length);
        assertEquals("monster_small1", mp.getCrafts()[0].getName());
        assertEquals(1, mp.getGeneral().getTasks().length);
        assertEquals(CraftTaskProtect.class, mp.getGeneral().getTasks()[0].getClass());
        
        MotEngine.instance.setDate(3201);
        mp.endTurn();
        
		assertEquals(2, mp.getCrafts().length);
		assertEquals("kaan1", mp.getCrafts()[1].getName());
        assertEquals(CraftTaskMaraudCraft.class, mp.getGeneral().getTasks()[1].getClass());
        assertEquals(CraftTaskMaraudCraft.class, mp.getCrafts()[1].getTask().getClass());
        assertEquals(ss1, ((CraftTaskMaraudCraft)mp.getCrafts()[1].getTask()).getTarget());
        assertEquals(1, ((CraftTaskMaraudCraft)mp.getCrafts()[1].getTask()).getFleet().getCrafts().length);
        
        mp.setNextKaanAttackDate(3250);
        mp.getCrafts()[1].destroy();
        mp.endTurn();
        
        assertNull(mp.getFleet());
		assertEquals(1, mp.getCrafts().length);
        assertEquals("monster_small1", mp.getCrafts()[0].getName());
        assertEquals(1, mp.getGeneral().getTasks().length);
        assertEquals(CraftTaskProtect.class, mp.getGeneral().getTasks()[0].getClass());

        MotEngine.instance.setDate(3250);
        Utils.setNextRandomInt(25, 50);
        mp.__setNextKaanAttackDate();
        
        assertEquals(3300, mp.getNextKaanAttackDate());
    }
    
    
    protected void tearDown() {
    	Logger.flushLog();
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
    
}