/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Location;
import mot.common.event.MotEvent;
import mot.common.event.OfferLeaderEvent;
import mot.common.leaders.GeneralDetails;
import mot.common.leaders.GovenorDetails;
import mot.common.proxy.ClientProxy;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Scanners;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.crafts.Craft;
import mot.engine.event.MotEventFactory;
import mot.engine.leaders.LeaderFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;

/**
 * Tests the Money functionality. This tests;
 *  - Unused production produces money
 *  - Crafts and planetary shields, weapons, scanners have a maintainence cost
 *  - When a player runs out of money items are sold.
 */
public class LeaderTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(LeaderTest.class);
        return suite;
    }

    public LeaderTest(String name) {
        super(name);
    }

    private SetupOptions options;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;

    private Planet planet1;
    private Planet planet2;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
    }

    public void createMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(player1, player2);

    }

    public void testCreate() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testCreate() *************************");
        createMap(null);

        assertEquals(null, player1.getGeneralLeader());
        assertEquals(null, player1.getGovenorLeader());

        //
        // Test that players are offered leaders and that computer players accept them automatically. 
        //
        
        MotEngine.instance.setDate(2002);
        player1.setQtyBattles(1);
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);

        LeaderFactory.endTurn();
        
        assertEquals("TestGeneral", player1.getGeneralLeader().getName());
        assertEquals(null, player1.getGovenorLeader());

        //
        // Test that players are only offered leader types that they do not have. 
        //
        
        player1.setQtyBattles(1);
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);

        LeaderFactory.endTurn();
        
        assertEquals("TestGeneral", player1.getGeneralLeader().getName());
        assertEquals("TestGovenor", player1.getGovenorLeader().getName());

        //
        // Test that players are not offered leaders when they already have leaders. 
        //
        
        player1.setQtyBattles(1);
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);

        LeaderFactory.endTurn();
        
        assertEquals("TestGeneral", player1.getGeneralLeader().getName());
        assertEquals("TestGovenor", player1.getGovenorLeader().getName());

        //
        // Test that players only get offered leaders on years divisable by 11 
        //

        player1.setGeneralLeader(null);
        player1.setGovenorLeader(null);
        
        MotEngine.instance.setDate(2000);
        player1.setQtyBattles(1);
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);

        LeaderFactory.endTurn();
        
        assertNull(player1.getGeneralLeader());
        assertNull(player1.getGovenorLeader());

        //
        // Test that players only get offered leaders if they have won a battle 
        //

        player1.setGeneralLeader(null);
        player1.setGovenorLeader(null);
        
        MotEngine.instance.setDate(2002);
        player1.setQtyBattles(0);
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);

        LeaderFactory.endTurn();
        
        assertNull(player1.getGeneralLeader());
        assertNull(player1.getGovenorLeader());
        
    }

    public void testGovenorAbilities() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testGovenorAbilities() *************************");
        createMap(null);
        
        planet1.setTeraforming(0.95);
        planet1.setPopulation(50);
        planet1.setQtyFactories(25);
        planet1.setPercentsAuto(true, true, false);
        MotEngine.getInstance().getRelations().setTrade(player1, player2);
        
        assertEquals(95.0, planet1.getMaxPopulation(), 0.001);
        assertEquals(19.0, planet1.getProductionOutput(), 0.001);
        assertEquals(12.0, planet1.getResearchOutput(), 0.001);
        assertEquals(0.925, planet1.getPopulationGrowth(), 0.001);
        assertEquals(0.0015, planet1.getTeraformingGrowth(), 0.00001);
        assertEquals(47.5, planet1.getMaxFactories(), 0.001);
        assertEquals(0.1875, planet1.getFactoriesGrowth(), 0.001);

        assertEquals(1, player1.getMaxPlayerTradeRoutes());
        assertEquals(-10, player1.getTradeIncome());
        assertEquals(0, player1.getOwnerPsiRating());
        assertEquals(0.0, player1.getSpyBonus(), 0.001);

        GovenorDetails govenorDetails = (GovenorDetails) mot.engine.leaders.LeaderFactory.getLeader("TestGovenor");
        mot.engine.leaders.LeaderFactory.addGovenorLeaderTo(player1, govenorDetails);

        assertEquals(104.5, planet1.getMaxPopulation(), 0.001);
        assertEquals(23.0, planet1.getProductionOutput(), 0.001);
        assertEquals(16.0, planet1.getResearchOutput(), 0.001);
        assertEquals(1.4, planet1.getPopulationGrowth(), 0.001);
        assertEquals(0.0055, planet1.getTeraformingGrowth(), 0.0001);
        assertEquals(99.275, planet1.getMaxFactories(), 0.001);
        assertEquals(0.28125, planet1.getFactoriesGrowth(), 0.001);

        assertEquals(3, player1.getMaxPlayerTradeRoutes());
        assertEquals(-3, player1.getTradeIncome());
        assertEquals(10, player1.getOwnerPsiRating());
        assertEquals(0.5, player1.getSpyBonus(), 0.001);

        //
        // NOW TEST ABILITIES WHEN LEADER GAINS A LEVEL
        //
 
        player1.getGovenorLeader().setExperience(100);
        
        assertEquals(2, player1.getGovenorLeader().getLevel());

        assertEquals(114.0, planet1.getMaxPopulation(), 0.001);
        assertEquals(27.0, planet1.getProductionOutput(), 0.001);
        assertEquals(20.0, planet1.getResearchOutput(), 0.001);
        assertEquals(1.8, planet1.getPopulationGrowth(), 0.001);
        assertEquals(0.0095, planet1.getTeraformingGrowth(), 0.0001);
        assertEquals(159.6, planet1.getMaxFactories(), 0.001);
        assertEquals(0.375, planet1.getFactoriesGrowth(), 0.001);

        assertEquals(5, player1.getMaxPlayerTradeRoutes());
        assertEquals(2, player1.getTradeIncome());
        assertEquals(20, player1.getOwnerPsiRating());
        assertEquals(1.0, player1.getSpyBonus(), 0.001);

    }

    public void testGeneralAbilities() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testGeneralAbilities() *************************");
        createMap(null);
        
        planet2.setScanners(new Scanners(Technologies.getScanner(TestTechs.PHOTON100)));

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player2, ss2);

        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss2.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        
        BattleCraft [] bca = battle.getCrafts();
        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());

        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(500, 500));
        bca[1].setDirection(270);

        assertEquals(0.0, player1.getAssaultBonus(), 0.001);
        assertEquals(25, craft1.getWarpSpeed());
        assertEquals(20, craft1.getMaxDamage());
        assertEquals(100, craft1.getVisibility(player2));
        assertEquals(100, bca[0].getSpeed(), 0.001);
        assertEquals(200, bca[0].getTurningSpeed(), 0.001);
        assertEquals(180, bca[0].getInitiative());
        assertEquals(0, bca[0].getCloakLevel());
        
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(1.0, bca[1].getDamage(), 0.001);
        craft2.setDamage(0.0);
        bca[0].rechargeWeapons();
        
        GeneralDetails generalDetails = (GeneralDetails) mot.engine.leaders.LeaderFactory.getLeader("TestGeneral");
        mot.engine.leaders.LeaderFactory.addGeneralLeaderTo(player1, generalDetails);

        assertEquals(0.1, player1.getAssaultBonus(), 0.001);
        assertEquals(40, craft1.getWarpSpeed());
        assertEquals(30, craft1.getMaxDamage());
        assertEquals(-50, craft1.getVisibility(player2));
        assertEquals(120, bca[0].getSpeed(), 0.001);
        assertEquals(200, bca[0].getTurningSpeed(), 0.001);
        assertEquals(264, bca[0].getInitiative());
        assertEquals(150, bca[0].getCloakLevel());
        
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(1.748, bca[1].getDamage(), 0.001);
        craft2.setDamage(0.0);
        bca[0].rechargeWeapons();
 
        //
        // NOW TEST ABILITIES WHEN LEADER GAINS A LEVEL
        //
 
        player1.getGeneralLeader().setExperience(100);
        
        assertEquals(2, player1.getGeneralLeader().getLevel());
       
        assertEquals(0.2, player1.getAssaultBonus(), 0.001);
        assertEquals(55, craft1.getWarpSpeed());
        assertEquals(40, craft1.getMaxDamage());
        assertEquals(-200, craft1.getVisibility(player2));
        assertEquals(140, bca[0].getSpeed(), 0.001);
        assertEquals(200, bca[0].getTurningSpeed(), 0.001);
        assertEquals(356, bca[0].getInitiative());
        assertEquals(300, bca[0].getCloakLevel());
        
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(2.5657, bca[1].getDamage(), 0.001);
    }

    public void testGovenorExperience() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testGovenorExperience() *************************");
        createMap(null);

        GovenorDetails govenorDetails = (GovenorDetails) mot.engine.leaders.LeaderFactory.getLeader("TestGovenor");
        mot.engine.leaders.LeaderFactory.addGovenorLeaderTo(player1, govenorDetails);
        
        assertEquals(0, player1.getGovenorLeader().getExperience());
        assertEquals(1, player1.getGovenorLeader().getLevel());
        
        MotEngine.instance.endTurn(player1);

        assertEquals(1, player1.getGovenorLeader().getExperience());
        
        MotEngine.getInstance().getRelations().setTrade(player1, player2);
        MotEngine.instance.endTurn(player1);

        assertEquals(5, player1.getGovenorLeader().getExperience());
        
        SolarSystem ss3 = newSolarSystem("SS3", 100, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 50, 0.8);
        Craft coloniser = newCraft(coloniserDesign, player1, ss3);
        coloniser.colonise(planet3);

        assertEquals(55, player1.getGovenorLeader().getExperience());

    }
        
    public void testGeneralExperience() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testGeneralExperience() *************************");
        createMap(null);

        GeneralDetails generalDetails = (GeneralDetails) mot.engine.leaders.LeaderFactory.getLeader("TestGeneral");
        mot.engine.leaders.LeaderFactory.addGeneralLeaderTo(player1, generalDetails);

        assertEquals(0, player1.getGeneralLeader().getExperience());
        assertEquals(1, player1.getGeneralLeader().getLevel());
        
        //
        // Test general gains correct experience when winning a battle
        //

        Craft craft1 = newCraft(defenderDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player2, ss2);

        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss2.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        
        BattleCraft [] bca = battle.getCrafts();
        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());

        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(500, 500));
        bca[1].setDirection(270);
        
        battle.automate();
        
        assertFalse(craft1.isDestroyed());
        assertTrue(craft2.isDestroyed());

        assertEquals(50, player1.getGeneralLeader().getExperience());

        //
        // Test general gains correct experience when loses a battle
        //

        Craft craft3 = newCraft(powerfulDesign, player2, ss2);

        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss2.getOrbit(), true);
        battle.join(craft3, true);
        battle.join(craft1, false);
        battle.setup();
        
        bca = battle.getCrafts();

        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(500, 525));
        bca[1].setDirection(270);
        
        battle.automate();
        
        //assertTrue(craft1.isDestroyed()); can't do this becuase the level has changed thich increases max damage!
        assertFalse(craft3.isDestroyed());

        assertEquals(150, player1.getGeneralLeader().getExperience());
    }
    


    private OfferLeaderEvent event = null;

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }

	public void notify(MotEvent e) throws ApplicationError {
        if (e instanceof OfferLeaderEvent) event = (OfferLeaderEvent)e;
   }

	public Player getOwner() {
        return player1;
    }

    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}