/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Location;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.WeaponBaysDesign;
import mot.common.resources.ResourceTritanOre;
import mot.common.resources.ResourceWarpCrystal;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Module;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Weapons;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.adapter.CraftVOAdapter;
import mot.engine.crafts.Craft;
import mot.engine.planets.BlackHole;
import mot.engine.planets.BlackHoleFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.Project;
import mot.engine.planets.ProjectCraft;
import mot.engine.planets.ProjectCraftUpgrade;
import mot.engine.planets.ProjectOutpost;
import mot.engine.planets.ProjectSpaceStation;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.ResearchProject;
import mot.engine.players.mayor.CraftDesignerFactory;
import mot.engine.players.mayor.Mayor;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskDeployStation;

/**
 * Tests the Mayor functionality - what is built on a planet. This tests;
 *  - When we have no colony crafts and no colony craft projects it build a colony craft
 *  - When we have no military crafts but we do have a colony craft it builds a military craft
 *  - When we have a colony craft being built, it builds military crafts
 *  - If we have n military crafts and >= 1 colony craft (actual on in production) it builds n weapons.
 *  - When an out of date colony craft is on production or exists it builds the latest colony craft
 */
public class MayorTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        Logger.setFull();
        TestSuite suite = new TestSuite(MayorTest.class);
        return suite;
    }
    
    public MayorTest(String name) {
        super(name);
    }
    
    private SetupOptions options;
    
    private ComputerPlayer player1;
    
    private SolarSystem ss1;

    private Planet planet1;

    
    protected void setUp() {
        try {
            MotEngine.newInstance();
            setupConfigs();
            
            Logger.setInMemory(false);
            Logger.setFull();
            
            String [] playerNames = { "TestPlayer1" };
            options = OptionsFactory.getGeneralOptions();
            player1 = newPlayer(playerNames[0], blue, options, playerNames);
            
            ss1 = newSolarSystem("SS1", 100, 100);
            planet1 = newPlanet("SS1-A", ss1, player1);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }
    
    /**
     * Test that the a player colonises available planets and that it always keeps a coloniser handy to
     * colonise any planet that may come available.
     */
    public void testBuildPiorities() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testBuildPiorities() *************************");
        planet1.setPopulation(100.0);
        planet1.setQtyFactories(100.0);

        player1.addShield(tech_energy10);
        player1.addScanner(tech_photon100);
        player1.addHull(tech_battleStationI);
        
        //
        // TEST BUILD COLONY CRAFT FIRST
        //
        planet1.endTurn();
        
        Project project = planet1.getProject();
        assertEquals("Mayor always build a colony craft first", "Coloniser1_", project.getName());

        CraftDesign cd = ((ProjectCraft)project).getCraftDesign();
        WeaponBaysDesign weaponBays = cd.getWeaponBays();
        assertTrue("Planet is not building a millitary craft", cd.canColonise());
        assertTrue("Colony craft has no weapons", weaponBays == null);
                
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals("Coloniser1_1" , player1.getCrafts());

        //
        // TEST BUILD TWO WEAPONS PER MILITARY CRAFT AND WEAPONS BUILT BEFORE CRAFT
        //

        planet1.endTurn();
        
        project = planet1.getProject();
        assertEquals(TestTechs.MISSILE6, project.getName());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_missile6, planet1.getWeapons());

        planet1.endTurn();
        
        project = planet1.getProject();
        assertEquals(TestTechs.MISSILE6, project.getName());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_missile6, tech_missile6, planet1.getWeapons());


        //
        // TEST BUILD MILITARY CRAFT AFTER 2 MISSILES AND COLONY CRAFT
        //
        
        planet1.endTurn();

        project = planet1.getProject();
        assertEquals("Military1_", project.getName());
        
        cd = ((ProjectCraft)project).getCraftDesign();
        assertFalse(cd.canColonise());
        assertTrue(cd.isMilitary());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals("Coloniser1_1", "Military1_1", player1.getCrafts());
        Craft militaryCraft1 = player1.getCrafts()[1];

        //
        // TEST BUILD MILITARY CRAFT AFTER 2 MISSILES AND COLONY CRAFT
        //

        planet1.endTurn();

        project = planet1.getProject();
        assertEquals("Military1_", project.getName());
        
        cd = ((ProjectCraft)project).getCraftDesign();
        assertFalse(cd.canColonise());
        assertTrue(cd.isMilitary());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals("Coloniser1_1", "Military1_1", "Military1_2", player1.getCrafts());
        Craft militaryCraft2 = player1.getCrafts()[2];

        //
        // TEST BUILD TWO WEAPONS PER MILITARY CRAFT 
        //

        planet1.endTurn();
        
        project = planet1.getProject();
        assertEquals(TestTechs.MISSILE6, project.getName());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_missile6, tech_missile6, tech_missile6, planet1.getWeapons());

        project = planet1.getProject();
        assertEquals(TestTechs.MISSILE6, project.getName());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_missile6, tech_missile6, tech_missile6, tech_missile6, planet1.getWeapons());

        //
        // TEST BUILD ONE SHIELD ONCE WE HAVE 2 MILITARY CRAFT
        //
        
        planet1.endTurn();

        project = planet1.getProject();
        assertEquals("Energy10", project.getName());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_energy10, planet1.getShields());

        //
        // TEST BUILD ONE SCANNER ONCE WE HAVE 2 MILITARY CRAFT AND SHIELDS
        //
        
        planet1.endTurn();

        project = planet1.getProject();
        assertEquals("Photon100", project.getName());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_photon100, planet1.getScanners());

        //
        // TEST BUILD ONE BATTLE STATION ONCE WE HAVE 2 MILITARY CRAFT + SCANNER, SHIELD, WEAPONS
        //
        
        planet1.endTurn();
        
        project = planet1.getProject();
        assertEquals("BattleStationI", project.getName());
        
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_battleStationI, planet1.getOrbital().getHullTech());

        //
        // TEST BUILD MILITARY CRAFT ONCE ALL UP TO DATE
        //
        
        planet1.endTurn();

        project = planet1.getProject();
        assertEquals("Military1_", project.getName());
        
        cd = ((ProjectCraft)project).getCraftDesign();
        assertFalse(cd.canColonise());
        assertTrue(cd.isMilitary());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals("Coloniser1_1", "Military1_1", "Military1_2", "Military1_3", player1.getCrafts());
        Craft militaryCraft3 = player1.getCrafts()[3];

    }

    
    /**
     * 
     */
    public void testDoesNotBuildColoniserWhenAlreadyBuildingOne() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testDoesNotBuildColoniserWhenAlreadyBuildingOne() *************************");
        planet1.setPopulation(100.0);
        planet1.setQtyFactories(100.0);

        //
        // TEST BUILD COLONY CRAFT FIRST
        //
        
        planet1.endTurn();
        
        assertEquals("Coloniser1_", planet1.getProject().getName());
        assertTrue( ((ProjectCraft)planet1.getProject()).getCraftDesign().canColonise());
        
        CraftDesign coloniserDesign = ((ProjectCraft)planet1.getProject()).getCraftDesign();
        
        //
        // Other planet will not build Coloniser if one is being built
        //
        
        SolarSystem ss2 = newSolarSystem("SS2", 100, 100);
        Planet planet2 = newPlanet("SS2-A", ss1, player1);

        planet2.endTurn();

        // Does not matter what this is but must not be a coloniser
        assertEquals("Missile6", planet2.getProject().getName());

        
        planet1.setProject(new ProjectCraftUpgrade(new CraftUpgradeDesign(coloniserDesign, new CraftVOAdapter(newCraft(coloniserDesign, player1, planet1), player1)), planet1));
        planet2.setProject(null);

        planet2.endTurn();

        // Does not matter what this is but must not be a coloniser
        assertEquals("Missile6", planet2.getProject().getName());
        
        planet1.addProduction(planet1.getProject().getCostToBuildOnPlanet()+1);
        planet2.setProject(null);

        planet2.endTurn();

        // Does not matter what this is but must not be a coloniser
        assertEquals("Missile6", planet2.getProject().getName());

        player1.getCrafts()[0].destroy();
        planet2.setProject(null);

        planet2.endTurn();

        // Now this must be a coloniser
        assertEquals("Coloniser1_", planet2.getProject().getName());

    }
    

    /**
     * Tests that the Mayor Upgraded colony crafts when they become obsolete.
     * This test lets the planets actually build the objects.
     */
    public void testUpgradeColoniser() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testUpgradeColoniser() *************************");
        planet1.setPopulation(100.0);
        planet1.setQtyFactories(100.0);
    
        MotEngine.getInstance().endTurn(player1);

        Project project = planet1.getProject();
        assertEquals("Coloniser1_", project.getName());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        
        Craft colonyCraft = player1.getColonyCraft();
        assertTrue(colonyCraft != null);
        
        project = planet1.getProject();
        assertEquals("Missile6", project.getName());  

        //
        // TEST THAT WILL BUILD A NEW COLONISER WHEN CURRENT ONE IS OUT OF DATE
        //
        player1.addSpecial(tech_colonyModule7);
        planet1.setProject(null);
        MotEngine.getInstance().setDate(MotEngine.getInstance().getDate()+21);
        
        MotEngine.getInstance().endTurn(player1);

        project = planet1.getProject();
        assertEquals("Coloniser2_", ((CraftUpgradeDesign)project.getProjectDesign()).getDesign().getName());
        assertEquals(tech_colonyModule7, ((CraftUpgradeDesign)project.getProjectDesign()).getDesign().getSpecialTechs());
        
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        colonyCraft = player1.getColonyCraft();
        assertTrue(colonyCraft.canColonise());
        assertEquals(tech_colonyModule7, colonyCraft.getSpecials());
    }

    /**
     * Tests that the Mayor builds better military crafts as new technology becomes available.
     */
    public void testUpgradeMilitary() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testUpgradeMilitary() *************************");
        // Add weapons so Mayor does not build weapons
        planet1.setWeapons(new Weapons());
        //planet1.getWeapons().add(Technologies.getWeapon("MassDriver1"));
        planet1.setPopulation(100.0);
        planet1.setWeapons(new Weapons(tech_missile6, tech_missile6));
        planet1.setShields(new Shields(tech_energy10));
        planet1.setScanners(new Scanners(tech_photon100));
        planet1.setQtyFactories(100.0);
        player1.addResearch(-10000);
        player1.setFunds(999999999);
    
        MotEngine.getInstance().endTurn(player1);

        assertEquals("Planet is building a colony craft", "Coloniser1_", planet1.getProject().getName());

        Project project = planet1.getProject();
        while (project == planet1.getProject()) {
            MotEngine.getInstance().endTurn(player1);
            MotEngine.instance.setDate(3000); // Do not design more craft
        }

        assertEquals("Coloniser1_1", player1.getCrafts());
        assertTrue(player1.getColonyCraft().canColonise());
        
        //
        // TEST BUILD MILITARY CRAFT
        //
        assertEquals("Military1_", planet1.getProject().getName());  

        // after building 1 craft it will want to build a weapon!
        project = planet1.getProject();
        while (project == planet1.getProject()) {
            MotEngine.getInstance().endTurn(player1);
            MotEngine.instance.setDate(3000); // Do not design more craft
        }
        
        assertEquals(1, player1.getQtyMilitartyCrafts());
        assertEquals("Coloniser1_1", "Military1_1", player1.getCrafts());
        assertEquals("Military1_", planet1.getProject().getName());  

        //
        // TEST MILITARY CRAFT DESIGN IS UPDATED
        //
        player1.addWeapon(tech_massDriver2);
        player1.addShield(tech_energy20);
        planet1.setProject(null);

        MotEngine.getInstance().endTurn(player1);
        assertEquals("Designs only updated every 20 years", "Military1_", planet1.getProject().getName());  

        MotEngine.instance.setDate(3010+CraftDesignerFactory.CALCULATE_INTERVAL); // Ready to design another craft
        planet1.setProject(null);

        MotEngine.getInstance().endTurn(player1);
        assertEquals("Military2_", planet1.getProject().getName());  
        
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        
        Craft[] crafts = player1.getCrafts();
		assertEquals("Coloniser1_1", "Military1_1", "Military2_1", crafts);
        assertFalse(crafts[1].canColonise());
        assertEquals(6, crafts[2].getWeaponBays().getAllWeapons(tech_massDriver2).length); // exact number of weapons does not matter
        assertEquals(tech_energy20, crafts[2].getShields());

    }

    /**
     * Tests that the Mayor builds better military crafts as new technology becomes available.
     */
    public void testCraftDesignProportions() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testCraftDesignProportions() *************************");
        
        String raceName = "Birds";
        String [] playerNames = { "Birds", "TestPlayer1" };
		RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
		ComputerPlayer p = newPlayer(raceName, blue, 1.5, null, humansOptions, playerNames);
		
		//
		// Birds will create Droid Mine Crafts
		//
		
		p.getPlayerCharacter().setCraftsPlayerPersonalityOn(true);
		p.getPlayerCharacter().setBattleSpeedBonus(2.0);
		p.addTechnology(tech_droidMine);
		p.addTechnology(tech_detonator);

		CraftDesignerFactory cdf = p.getMayor().getCraftDesignerFactory();
		CraftDesign cd = cdf.getNextMilitaryCraftToBuild(p);
		
		assertEquals("Military1_", cd.getName());
		assertEquals(true, cd.getWeaponTechs().contains(tech_massDriver1));
		
		assertEquals(0.25, cdf.getSpecialCraftDesigner().getRatio(p),0.001);

		newCraft(cd, p, ss1);
		
		cd = cdf.getNextMilitaryCraftToBuild(p);
		
		assertEquals("Military1_", cd.getName());
		assertEquals(true, cd.getWeaponTechs().contains(tech_massDriver1));
		
		newCraft(cd, p, ss1);
		newCraft(cd, p, ss1);
		newCraft(cd, p, ss1);
		
		cd = cdf.getNextMilitaryCraftToBuild(p);
		
		assertEquals("DroidFighter1_", cd.getName());
		assertEquals(true, cd.getWeaponTechs().contains(tech_droidMine));

		newCraft(cd, p, ss1);

		cd = cdf.getNextMilitaryCraftToBuild(p);
		
		assertEquals("Military1_", cd.getName());
		assertEquals(true, cd.getWeaponTechs().contains(tech_massDriver1));
    }

    /**
     * Tests that the Mayor focuses on research when not at war and has isolationist personality.
     */
    public void testIsolationism() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testIsolationism() *************************");
        
        String [] playerNames = { "TestPlayer2" };
        ComputerPlayer player2 = newPlayer(playerNames[0], blue, options, playerNames);
        
        SolarSystem ss2 = newSolarSystem("SS2", 150, 100);
        Planet planet2 = newPlanet("SS2-A", ss2, player2);

		newCraft(coloniserDesign, player1, planet1);
		
		planet1.setShields(new Shields(tech_energy10));
		planet1.setWeapons(new Weapons(tech_missile6, 8));
		planet1.setScanners(new Scanners(tech_photon100));
		createBattleStation(tech_battleStationI, planet1);
		
		//
		// have well defended planet and coloniser, next craft I build will be military
		//
		
		player1.setIsolationist(true);
		Mayor mayor = player1.getMayor();
		
		mayor.notifyNoProject(planet1);
		mayor.setPercents(planet1);
		
		assertEquals("Military1_", ((ProjectCraft)planet1.getProject()).getName());
		
		// Does not matter that these are - but research must be much higher (80% to 20% remaining output)
		assertEquals(0.134, planet1.getPercentProduction(),0.01); 
		assertEquals(0.536, planet1.getPercentResearch(),0.01);

        MotEngine.instance.getRelationsNotNull().setWar(player1, player2);
		mayor.setPercents(planet1);
        
		assertEquals(0.335, planet1.getPercentProduction(),0.01); 
		assertEquals(0.335, planet1.getPercentResearch(),0.01);

    }
    

    protected void tearDown() {
        
    }

    /**
     * Tests that if a general has requested an outpost AND no other planet is building one, 
     * then the mayor will build one.
     * 
     *  NOTE - SOME OF THIS TESTING IS MIRRORED IN TransportTest
     *  
     */
    public void testBuildOutpost() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testBuildOutpost() *************************");
        
        // Outposts are built when
        // - We have a colony craft
        // - no planets installation require building or upgrading.
        // - The general wants an outpost
        
        Craft colonyCraft1 = newCraft(coloniserDesign, player1, ss1);
        player1.getHulls().add(tech_outpost);
        planet1.setWeapons(new Weapons(tech_missile6, tech_missile6, tech_missile6, tech_missile6));
        
        // Will not build outpost because general has not requested one!
        
        player1.getMayor().notifyNoProject(planet1);
        
        assertEquals(false, player1.getGeneral().needOutpost());

        // Now make general request an outpost

        assertEquals(150, player1.getEngines().getLongestRangeEngine().getMaxTravelDistance());
        SolarSystem ss2 = newSolarSystem("SS2", 200, 100);
        SolarSystem ss3 = newSolarSystem("SS3", 300, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 80, 0.8);

        planet1.setProject(null);        
        player1.getGeneral().assignTaskToCraft(colonyCraft1);
        
        assertEquals(true, player1.getGeneral().needOutpost());
        
        player1.getMayor().notifyNoProject(planet1);
        ProjectOutpost project = (ProjectOutpost)planet1.getProject();
        
        assertEquals(false, player1.getGeneral().needOutpost());
		assertEquals(TestTechs.OUTPOST, project.getOutpost().getName());
        assertEquals(ss2, project.getTarget());
        
        //
        // Test that I will not build another outpost if we are building one or have one
        //

        Craft colonyCraft2 = newCraft(coloniserDesign, player1, ss1);
        player1.getGeneral().assignTaskToCraft(colonyCraft2);
        assertEquals(true, player1.getGeneral().needOutpost());
        
        Planet planet2 = newPlanet("SS1-B", ss1, player1);
        planet2.setWeapons(new Weapons(tech_missile6, tech_missile6, tech_missile6, tech_missile6));
        player1.getMayor().notifyNoProject(planet2);
        assertFalse(ProjectOutpost.class == planet2.getProject().getClass());
        
        project.build(planet1);
        planet1.setProject(null);        
        Transport transport = player1.getTransportManager().getTransportsAsArray()[0];

        assertEquals(tech_outpost, ((TransportTaskDeployStation)transport.getTask()).getOrbital().getHullTech());
        assertEquals(true, player1.getGeneral().needOutpost());
        
        planet2.setProject(null);        
        player1.getMayor().notifyNoProject(planet2);
        assertFalse(ProjectOutpost.class == planet2.getProject().getClass());
        
        //
        // But if transport is destroyed then we will build another
        //

        assertEquals(true, player1.getGeneral().needOutpost());

        transport.setTask(null);
        planet2.setProject(null);        
        player1.getMayor().notifyNoProject(planet2);
        
        assertEquals("Outpost", planet2.getProject().getName());
        assertEquals(false, player1.getGeneral().needOutpost());
    
    }
    
    /**
     * Tests that the mayor will build a mining station if there is a planet special resources
     * within range AND no other planet is building one.
     */
    public void testBuildMiningStationForPlanet() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testBuildMiningStationForPlanet() *************************");

        Planet planet2 = newPlanet("SS1-B", ss1, null, 110, 0.1);
        planet2.setProductionBonus(3.0);

        Planet planet3 = newPlanet("SS1-C", ss1, null, 110, 0.1);
        planet3.setResource(new ResourceWarpCrystal());
        
        Craft colonyCraft = newCraft(coloniserDesign, player1, ss1);
        Craft fighterCraft1 = newCraft(fighterDesign, player1, ss1);
        Craft fighterCraft2 = newCraft(fighterDesign, player1, ss1);
        player1.getHulls().add(tech_spaceStation);
        
        planet1.setWeapons(new Weapons(tech_missile6, tech_missile6, tech_missile6, tech_missile6));
        planet1.setScanners(new Scanners(tech_photon100));

        // Mining stations are built when
        // - We have a colony craft
        // - no planets installation require building or upgrading.
        // - There are resources to mine
        // General should mine resource warp crystals because that are far more valuable than production bonuses

        // Now should build space station
        
        player1.getMayor().notifyNoProject(planet1);

        assertEquals(tech_spaceStation, ((ProjectSpaceStation)planet1.getProject()).getSpaceStation());
        assertEquals(planet3, ((ProjectSpaceStation)planet1.getProject()).getTarget().getPlanet());
    }

    /**
     * Tests that the mayor will build a mining station if there is a black hole
     * within range AND no other planet is building one.
     */
    public void testBuildMiningStationForBlackHole() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testBuildMiningStationForBlackHole() *************************");

        Planet planet2 = newPlanet("SS1-B", ss1, null, 110, 0.1);
        planet2.setProductionBonus(3.0);
        
        BlackHole blackHole = BlackHoleFactory.createBlackHole(new Location(125,150));

        Planet planet3 = newPlanet("SS1-C", ss1, null, 110, 0.1);
        planet3.setResource(new ResourceWarpCrystal());
        
        Craft colonyCraft = newCraft(coloniserDesign, player1, ss1);
        player1.getHulls().add(tech_spaceStation);

        planet1.setWeapons(new Weapons(tech_missile6, tech_missile6, tech_missile6, tech_missile6));
        planet1.setScanners(new Scanners(tech_photon100));
        Craft fighterCraft1 = newCraft(fighterDesign, player1, ss1);
        Craft fighterCraft2 = newCraft(fighterDesign, player1, ss1);

        // Mining stations are built when
        // - We have a colony craft
        // - no planets installation require building or upgrading.
        // - There are resources to mine

        // Now should build space station
        
        player1.getMayor().notifyNoProject(planet1);

        assertEquals(tech_spaceStation, ((ProjectSpaceStation)planet1.getProject()).getSpaceStation());
        assertEquals(blackHole, ((ProjectSpaceStation)planet1.getProject()).getTarget().getStellarObject());
    }

    /**
     * Tests modules are added to mining stations
     *  Also Tests
     *  - Deploying a mining station to a planet in the same solar system
     */
    public void testBuildModules() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testBuildModules() *************************");
        Planet planet2 = newPlanet("SS1-B", ss1, null, 60, 0.1);
        planet2.setResource(new ResourceTritanOre());
        
        Craft colonyCraft = newCraft(coloniserDesign, player1, ss1);
        player1.getHulls().add(tech_spaceStation);
        
        planet1.setWeapons(new Weapons(tech_missile6, tech_missile6, tech_missile6, tech_missile6));
        planet1.setScanners(new Scanners(tech_photon100));
        Craft fighterCraft1 = newCraft(fighterDesign, player1, ss1);
        Craft fighterCraft2 = newCraft(fighterDesign, player1, ss1);

        // Mining stations are built when
        // - We have a colony craft
        // - no planets installation require building or upgrading.
        // - There are resources to mine
        // General should mine resource warp crystals because that are far more valuable than production bonuses

        // Now should build space station
        
        player1.getMayor().notifyNoProject(planet1);

        assertEquals(tech_spaceStation, ((ProjectSpaceStation)planet1.getProject()).getSpaceStation());
        
        //
        // Build this project and deploy the station
        //
        
        player1.addModule(tech_oreExtractor);
        planet1.getProject().build(planet1);
        //player1.getTransportManager().endTurn();
        player1.getMayor().notifyNoProject(planet1);     
        
        assertEquals(planet2, planet1.getMiningStation().getPlanet());
        Module[] ma = planet2.getOrbital().getModuleTechs().getModules();
		assertEquals(tech_core_habitat, tech_oreExtractor,  ma);

		//
        // Modules are created automatically when researched 
        //
		
		player1.finishResearchProject(new ResearchProject(tech_habitatI, player1, false), true, false);

        ma = planet2.getOrbital().getModuleTechs().getModules();
		assertEquals(tech_core_habitat, tech_oreExtractor, tech_habitatI,  ma);
    }
    

    /**
     * Tests mayor will build a trade route between solar systems.
     */
    public void testBuildTradeRoute() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testBuildTradeRoute() *************************");
        
        planet1.setDateColonised(90);
        planet1.setPopulation(100);
        planet1.setQtyFactories(25);
        
        SolarSystem ss2 = newSolarSystem("SS2", 140, 100);
        Planet planet2 = newPlanet("SS2-A", ss2, player1, 90, 1.0, 10, 5);
        
        SolarSystem ss3 = newSolarSystem("SS3", 180, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 1.0);
        
        SolarSystem ss4 = newSolarSystem("SS4", 290, 100);
        Planet planet4 = newPlanet("SS4-A", ss4, player1, 130, 1.0, 20, 10);

        
        assertEquals(planet1, player1.getCapitol());
        assertEquals(ss1, player1.getCapitolSolarSystem());
        
        //
        // Test does not build trade routes unless we have the knowledge
        //
        
        player1.getMayor().setupPlanetaryTradeRoutes();

        assertNull(MotEngine.instance.getTradeRoutes().getTradeRoutes());
        
        player1.addTechnology(tech_trade_route);
        player1.getMayor().setupPlanetaryTradeRoutes();
        
        assertEquals(ss1, ss2, 3000, MotEngine.instance.getTradeRoutes());

        //
        // Test does build best trade routes and builds one at a time
        //
        
        planet3.colonise(newCraft(coloniserDesign, player1, ss3));
        
        player1.getMayor().setupPlanetaryTradeRoutes();

        assertEquals(ss1, ss2, 3000, ss1, ss3, 3000, MotEngine.instance.getTradeRoutes());

        player1.getMayor().setupPlanetaryTradeRoutes();

        assertEquals(ss1, ss2, 3000, ss1, ss3, 3000,  ss3, ss2, 3000, MotEngine.instance.getTradeRoutes());
        
        //
        // Does not create trade routes when they are out of range
        //
        
        player1.getMayor().setupPlanetaryTradeRoutes();

        assertEquals(ss1, ss2, 3000, ss1, ss3, 3000,  ss3, ss2, 3000, MotEngine.instance.getTradeRoutes());
        
        player1.addTechnology(tech_trade_routeII);
        player1.getMayor().setupPlanetaryTradeRoutes();

        assertEquals(ss1, ss2, 3000, ss1, ss3, 3000,  ss3, ss2, 3000,  ss1, ss4, 3000, MotEngine.instance.getTradeRoutes());
    
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}