/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.common.designs.ShieldDesign;
import mot.common.designs.WeaponDesign;
import mot.common.event.FundsGoneEvent;
import mot.common.event.FundsLowEvent;
import mot.common.event.MotEvent;
import mot.common.event.StateContainer;
import mot.common.proxy.ClientProxy;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.crafts.Craft;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.ProjectFactory;
import mot.engine.planets.ProjectShield;
import mot.engine.planets.ProjectWeapon;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.ResearchProject;

/**
 * Tests the Money functionality. This tests;
 *  - Unused production produces money
 *  - Crafts and planetary shields, weapons, scanners have a maintenance cost
 *  - When a player runs out of money items are sold.
 */
public class MoneyTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(MoneyTest.class);
        return suite;
    }

    public MoneyTest(String name) {
        super(name);
    }

    private SetupOptions options;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;

    private Planet planet1;
    private Planet planet2;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
        Logger.setDebug();
    }

    public void createMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(player1, player2);
        
    }

    private void setState(Planet planet) throws ApplicationError {
        planet.setAutoDevelopment(false);
        planet.setAutoEcology(false);
        planet.setPopulation(100.0);
        planet.setQtyFactories(0.0);
        planet.setPercentProduction(0.2);
        planet.setPercentResearch(0.2);
        planet.setPercentEcology(0.2);
        planet.setPercentDevelopment(0.2);
        assertEquals(100.0, planet1.getIndustryWork(), 0.0);
    }

    /**
     * Test Money.
     */
    public void testMoney() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testMoney() *************************");
        createMap(null);
        
        MotEngine.instance.setLevel(2.0);

        setState(planet1);
        player1.calculateQtySpies();
        player2.calculateQtySpies();
        
        // Total income is 40% of production output because we have no project
        // We have pop 100 which means this computer player has one spy = maint cost = 2;
        assertEquals(0.2, planet1.getUnusedPercent(), 0.01);
        assertEquals(18, player1.getTotalIncome(true));
        assertEquals(2, player1.getTotalMaintenanceCost());
        assertEquals(16, player1.getNetIncome());

        planet1.setProject(ProjectFactory.getProject(fighterDesign, planet1, true));

        // Total income is 20% of production output
        assertEquals(15, player1.getTotalIncome(true));
        assertEquals(2, player1.getTotalMaintenanceCost());
        assertEquals(13, player1.getNetIncome());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        planet1.setProject(new ProjectWeapon(new WeaponDesign(tech_massDriver1)));

        // Have produced craft with maintenance cost of 4 
        // Total income is 20% of production output
        assertEquals(1, player1.getQtyCrafts());
        assertEquals(4, player1.getCrafts()[0].getMaintenanceCost());
        assertEquals(15, player1.getTotalIncome(true));
        assertEquals(6, player1.getTotalMaintenanceCost());
        assertEquals(9, player1.getNetIncome());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        planet1.setProject(new ProjectShield(new ShieldDesign(tech_energy10)));

        // Have produced craft with maintenance cost of 4, 
        // Weapon with maintenance cost 4
        // Total income is 20% of production output
        assertEquals(15, player1.getTotalIncome(true));
        assertEquals(10, player1.getTotalMaintenanceCost());
        assertEquals(5, player1.getNetIncome());
        
        // Add a battle station then remove it just to check that the maintenance 
        // costs do change correctly.
        //new ProjectBattleStation(tech_battleStationI).build(planet1);
        createBattleStation(tech_battleStationI, planet1);
        assertEquals(14, player1.getTotalMaintenanceCost());
        planet1.setOrbital(null);

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        planet1.setProject(new ProjectShield(new ShieldDesign(tech_energy10)));

        // Have produced craft with maintenance cost of 4, 
        // Weapon with maintenance cost 4, 
        // Shield with maintenance cost of 3
        // Total income is 20% of production output
        assertEquals(15, player1.getTotalIncome(true));
        assertEquals(13, player1.getTotalMaintenanceCost());
        assertEquals(2, player1.getNetIncome());
        
        // Test that low funds event is sent when we do not have much money!
        // Income +16 => funds are now 6 which is < 2*maint cost.
        // Low funds event causes computer player to be credited with maint cost * 5 => 65
        // Funds are -10 + 7 + 65 = 54;
        player1.setResearchProject(new ResearchProject(tech_battleStationII, player1, false));
        eventList.clear();
        player1.setFunds(-10);
        planet1.endTurn();
        player1.endTurn();
        setState(planet1);
        
        assertEquals(15, player1.getTotalIncome(true));
        assertEquals(13, player1.getTotalMaintenanceCost());
        assertEquals(2, player1.getNetIncome());
        
        FundsLowEvent event1 = (FundsLowEvent) eventList.get(0);
        assertNotNull(event1);
        assertEquals(57, player1.getFunds());
        
        Craft craft = player1.getCrafts()[0];
        assertEquals(427, craft.getValue());
        assertEquals(137, new ProjectWeapon(new WeaponDesign(tech_massDriver1)).getValue());

        // Test that low funds event is sent when we do not have much money!
        // Income +16 => funds are now -44 which is < 0.
        // Low funds event causes computer player to be credited with maint cost * 5 => 55
        // Funds are -100 + 9 + 55 = -36 => Will sell something
        eventList.clear();
        player1.setFunds(-100);
        planet1.endTurn();
        player1.endTurn();
        setState(planet1);
        
        event1 = (FundsLowEvent) eventList.get(0);
        FundsGoneEvent event2 = (FundsGoneEvent) eventList.get(1);
        assertTrue(Comp.same(event2.getObject(), craft.getName()) || Comp.same(event2.getObject(), tech_massDriver1.getName()));
        if (Comp.same(event2.getObject(), craft.getName())) {
            assertEquals(394, player1.getFunds());
            assertEquals("Craft is gone", 0, player1.getQtyCrafts());
        } else {
            assertEquals(104, player1.getFunds());
            assertEquals("Weapon in gone", 0, planet1.getWeapons().getQuantity());
        }
        
    }

    /**
     * Test Money.
     */
    public void testSellAssets() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testSellAssets() *************************");
        createMap(null);

        MotEngine.instance.setLevel(2.0);

        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player1, ss1);
        planet1.setWeapons(new Weapons(tech_massDriver1, tech_massDriver1));
        
        // Sell a planet weapon
        Utils.setNextRandomProbability(0.2);
        Utils.setNextRandomProbability(0.2);
        
        setState(planet1);
        player1.setFunds(-1000);
        eventList.clear();
        player1.endTurn();
        
        FundsLowEvent event1 = (FundsLowEvent) eventList.get(0);
        FundsGoneEvent event2 = (FundsGoneEvent) eventList.get(1);
        
        assertEquals(1, planet1.getWeapons().getQuantity());
        assertEquals(2, player1.getQtyCrafts());
        assertEquals(-791, player1.getFunds());

        // Sell a craft
        Utils.clearAndSetNextRandomProbability(0.6);
        Utils.setNextRandomInt(0);
        
        setState(planet1);
        player1.setFunds(-1000);
        eventList.clear();
        player1.endTurn();
        
        event1 = (FundsLowEvent) eventList.get(0);
        event2 = (FundsGoneEvent) eventList.get(1);
        
        assertEquals(1, planet1.getWeapons().getQuantity());
        assertEquals(craft2, player1.getCrafts());
        assertEquals(-501, player1.getFunds());


        // Sell a craft travelling
        if (craft2.getFleet() != null) craft2.getFleet().remove(craft2);
        craft2.moveTo(ss2.getOrbit());
        craft2.endTurn();
        
        assertNull(craft2.getOrbit());
        assertNotNull(craft2.getNavigator());
        
        Utils.setNextRandomProbability(0.6);
        Utils.setNextRandomInt(0);

        setState(planet1);
        player1.setFunds(-1000);
        eventList.clear();
        player1.endTurn();
        
        event1 = (FundsLowEvent) eventList.get(0);
        event2 = (FundsGoneEvent) eventList.get(1);
        
        assertEquals(1, planet1.getWeapons().getQuantity());
        assertEquals(0, player1.getQtyCrafts());
        assertEquals(-517, player1.getFunds());

    }
	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
		Logger.logDebug("Process Event "+e);
		if (e instanceof StateContainer) {
			DeligateCacheManager.update((StateContainer)e);
		}
		eventList.add(e);
    }

	public Player getOwner() {
        return player1;
    }
    
    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}