/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Size;
import mot.common.event.MessageEvent;
import mot.common.event.MotEvent;
import mot.common.event.OfferMoralChoiceEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.command.AcceptMoralChoiceCmd;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.players.morals.MoralPolicyAdvisorBasic;

/**
 * Tests that a craft moves between two solar system correctly.
 * Test the General objects.
 */
public class MoralChoiceTest extends TestBase implements ClientProxy {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(mot.test.MoralChoiceTest.class);
        return suite;
    }

    public MoralChoiceTest(String name) {
        super(name);
    }

    private SetupOptions options;
    
    private UserPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;

    private Planet planet1;
    private Planet planet2;
    
    private Player proxyOwner;


    protected void setUp() throws Exception {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
        
        String [] playerNames = { "TestPlayer1", "TestPlayer2" };
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player1 = newUserPlayer(playerNames[0], blue, TestFactory.getHumansOptions());
        player2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 50, 50);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 150, 50);
        planet2 = newPlanet("SS2-A", ss2, player2);
            
        MotEngine.instance.getStellarObjects().setSize(new Size(200,200));
        
        proxyOwner = player1;
    }
    

    /**
     * Tests that the moral choices a player makes do effect the player's 
     *  - Production
     *  - Influence
     *  - Damage
     */
    public void testBonuses() throws Exception {
        Logger.logDebug("*******************************  MoralChoiceTest.testBonuses() *************************");
        
        assertEquals(0.0, player1.getOwnerProductionBonus(), 0.001);
        assertEquals(0.0, player1.getInfluenceBonus(), 0.001);
        assertEquals(0.0, player1.getOwnerDamageBonus(), 0.001);
        assertEquals(null, player1.getMoralChoices());
        
        player1.add(mc_testMoralChoice);

        assertEquals(0.1, player1.getOwnerProductionBonus(), 0.001);
        assertEquals(0.2, player1.getInfluenceBonus(), 0.001);
        assertEquals(0.3, player1.getOwnerDamageBonus(), 0.001);
        assertEquals(-20, player1.getMoralChoices().getRelationsBonus());
        
        player1.add(mc_testMoralChoice);

        assertEquals(0.2, player1.getOwnerProductionBonus(), 0.001);
        assertEquals(0.4, player1.getInfluenceBonus(), 0.001);
        assertEquals(0.6, player1.getOwnerDamageBonus(), 0.001);
        assertEquals(-40, player1.getMoralChoices().getRelationsBonus());
    }
 
    /**
     * Tests that the moral choices are proposed by the BasicMoralPolicyAdvisor
     */
    public void testMoralPolicyAdvisorUserPlayer() throws Exception {
        Logger.logDebug("*******************************  MoralChoiceTest.testMoralPolicyAdvisorUserPlayer() *************************");
        
        MoralPolicyAdvisorBasic mpa = (MoralPolicyAdvisorBasic) player1.getMoralPolicyAdvisor();
        mpa.setTurnToAdvisePolicy(2);
        player1.endTurn();
        
        assertEquals(null, lastEvent);
        assertEquals(1, mpa.getTurnToAdvisePolicy());

        Utils.setNextRandomInt(0);
        player1.endTurn();
        
        assertEquals(TEST_MORAL_CHOICE, ((OfferMoralChoiceEvent)lastEvent).getName());
        
        new AcceptMoralChoiceCmd(player1, TEST_MORAL_CHOICE).run();

        assertEquals(1, player1.getMoralChoices().getList().getAllAsArray().length);
        assertEquals(TEST_MORAL_CHOICE, player1.getMoralChoices().getList().getAllAsArray()[0].getName());

        try {
        	new AcceptMoralChoiceCmd(player1, "InvalidMoralChoice").run();
        	fail("Moral Policy is not the last recommended");
        } catch (ValidationError ex) {
            assertEquals("Moral Policy InvalidMoralChoice was not offered to you", ex.getMessage());        	
        }
        
        mpa.setLastPolicyRecomended("InvalidMoralChoice");

        try {
        	new AcceptMoralChoiceCmd(player1, "InvalidMoralChoice").run();
        	fail("Moral policy does not exist");
        } catch (ValidationError ex) { 
            assertEquals("Moral Policy InvalidMoralChoice does not exist", ex.getMessage());        	
        }
    }
 
    /**
     * Tests that the moral choices are proposed by the BasicMoralPolicyAdvisor
     */
    public void testMoralPolicyAdvisorComputerPlayer() throws Exception {
        Logger.logDebug("*******************************  MoralChoiceTest.testMoralPolicyAdvisorComputerPlayer() *************************");
        proxyOwner = player2;
        
        MoralPolicyAdvisorBasic mpa = (MoralPolicyAdvisorBasic) player2.getMoralPolicyAdvisor();
        mpa.setTurnToAdvisePolicy(2);
        player2.endTurn();
        
        assertEquals(null, lastEvent);
        assertEquals(1, mpa.getTurnToAdvisePolicy());

        Utils.setNextRandomInt(0);
        player2.endTurn();

        assertEquals(1, player2.getMoralChoices().getList().getAllAsArray().length);
        assertEquals("TestMoralChoice", player2.getMoralChoices().getList().getAllAsArray()[0].getName());
        assertEquals("Player TestPlayer2 has adopted moral policy : TestMoralChoice", ((MessageEvent)lastEvent).getMessage());
    }
 
    protected void tearDown() {
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

	public Player getOwner() {
		return proxyOwner;
	}

	private MotEvent lastEvent;
	
	public void notify(MotEvent e) throws ApplicationError {
		lastEvent = e;
	}

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
	}

}