/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import java.util.ArrayList;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.NameIntPair;
import mot.common.command_results.CommandResult;
import mot.common.designs.AsteroidDesign;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.ConstructPlanetDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.TransportDesign;
import mot.common.designs.WeaponDesign;
import mot.common.event.MotEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Modules;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Technology;
import mot.common.technology.WeaponBundle;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.command.SetPlanetAutoPercentsCmd;
import mot.engine.command.SetPlanetPercentsCmd;
import mot.engine.command.SetProjectCmd;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.event.MotEventFactory;
import mot.engine.event.TradeRouteInvalidEF;
import mot.engine.planets.CrisisFactory;
import mot.engine.planets.CrisisPlague;
import mot.engine.planets.Planet;
import mot.engine.planets.Project;
import mot.engine.planets.ProjectBattleStation;
import mot.engine.planets.ProjectCraft;
import mot.engine.planets.ProjectScanner;
import mot.engine.planets.ProjectShield;
import mot.engine.planets.ProjectSpaceStation;
import mot.engine.planets.ProjectWeapon;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRoute;
import mot.engine.planets.TradeRouteFactory;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.ResearchProject;
import mot.engine.players.UserPlayer;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskDeployStation;
import mot.engine.transports.TransportTaskInvade;

public class PlanetTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        Logger.setFull();
        TestSuite suite = new TestSuite(PlanetTest.class);
        return suite;
    }
    
    public PlanetTest(String name) {
        super(name);
    }
    
    private ComputerPlayer player;
    private UserPlayer userPlayer;
    private Player optionalPlayer;
    private SolarSystem ss1;
    private Planet planet1;
    private Planet planet2;
    private SolarSystem ssu;
    private Planet planetu;
    
    private void createMap(SetupOptions optionsRace, boolean withOptionalPlayer) {
        try {
            String [] playerNames;
            if (withOptionalPlayer) {
                playerNames = new String [] { "Humans", "Birds", "Bears" }; 
            } else {
                playerNames = new String [] { "Humans", "Birds" };
            }
            SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
            RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
            player = newPlayer(playerNames[0], blue, options, playerNames);
            userPlayer = newUserPlayer(playerNames[1], blue, null, options);
            if (withOptionalPlayer) {
                optionalPlayer = newPlayer(playerNames[1], blue, optionsGeneral, playerNames); 
            }
            
            ss1 = newSolarSystem("SS1", 100, 100);
    
            planet1 = newPlanet("SS1-A", ss1, player);
            planet2 = newPlanet("SS1-B", ss1, null, 100, 0.8);

            ssu = newSolarSystem("SSU", 200, 200);            
            planetu = newPlanet("SSU-A", ssu, userPlayer);

            planet1.setProject(new ProjectCraft(coloniserDesign, planet1, true));
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }
    
    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
    }
    
    /**
     * Tests that a basic planet functionality;
     *  - population produces the correct production, ecology, research.
     *  - population grows / shrinks depending on terraform level.
     */
    public void testPlanet() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testPlanet() *************************");
        createMap(null, false);

        planet1.setSize(100);
        planet1.setTeraforming(1.0);
        planet1.setQtyFactories(75.0);
        
        assertEquals("Players best factory has correct production", 1, player.getFactories().getProduction());
        assertEquals("Players best factory has correct pollution", 0.5, player.getFactories().getMinEcologyWork(), 0.0);
        assertEquals("Max Population is correct", 100.0, planet1.getMaxPopulation(), 0.0);
        
        new ProjectBattleStation(new BattleStationDesign(tech_battleStationI)).build(planet1);
        assertEquals("Max Population is correct", 105.0, planet1.getMaxPopulation(), 0.0);
        planet1.setOrbital(null);

        planet1.setPopulation(10.0);
        assertEquals("Population growth is correct", 0.4, planet1.getPopulationGrowth(), 0.0);
        planet1.setPopulation(90.0);
        assertEquals("Population growth is correct", 0.4, planet1.getPopulationGrowth(), 0.0);
        planet1.setPopulation(50.0);
        assertEquals("Population growth is correct", 1.0, planet1.getPopulationGrowth(), 0.0);
        planet1.setPopulation(100.0);
        assertEquals("Population growth is correct", 0.0, planet1.getPopulationGrowth(), 0.0);
        assertFalse("Population growth is correct", planet1.canHaveMorePopulation());
        planet1.setPopulation(110.0);
        assertEquals("Population growth is correct", -1.0, planet1.getPopulationGrowth(), 0.0);
        assertFalse("Population growth is correct", planet1.canHaveMorePopulation());
        
        planet1.setCrisis(new CrisisPlague(MotEngine.instance.getDate()));
        
        planet1.setPopulation(10.0);
        assertEquals("Population growth is correct", -0.01, planet1.getPopulationGrowth(), 0.1);
        planet1.setPopulation(90.0);
        assertEquals("Population growth is correct", -4.1, planet1.getPopulationGrowth(), 0.1);
        planet1.setPopulation(50.0);
        assertEquals("Population growth is correct", -1.5, planet1.getPopulationGrowth(), 0.0);
        planet1.setPopulation(100.0);
        assertEquals("Population growth is correct", -5.0, planet1.getPopulationGrowth(), 0.0);
        assertFalse("Population growth is correct", planet1.canHaveMorePopulation());
        
        planet1.setCrisis(null);

        assertEquals("Maximum factories planet can use is correct", 50.0, planet1.getQtyFactoriesInUse(), 0.0);
        assertFalse("Maximum factories is correct", planet1.canHaveMoreFactories());
        
        // Work done is pop + qty factories in use * 1
        assertEquals("Planet is producing correct industry output", 150.0, planet1.getIndustryWork(), 0.0);
        assertEquals("Planet is producing correct pollution", 35.0, planet1.getMinimumEcologyOutput(), 0.0);
        assertEquals("Planet is producing correct pollution", 0.233, planet1.getMinimumEcologyPercent(), 0.001);
        assertEquals("Planet is producing correct ecology work", 37.5, planet1.getEcologyOutput(), 0.0);
        
        planet1.setAutoEcology(true);
        assertEquals("Set Auto percent works for ecology", 0.233, planet1.getMinimumEcologyPercent(), 0.001);

        planet1.setPercentEcology(0.5);
        planet1.setTeraforming(1.0);
        assertEquals("Planet has correct terraform growth", 0.0, planet1.getTeraformingGrowth(), 0.001);
        player.addBiotechnology(tech_terraforming10);
        assertEquals("Planet has correct terraform growth", 0.016, planet1.getTeraformingGrowth(), 0.001);
        planet1.setTeraforming(0.9);
        assertEquals("Planet has correct terraform growth", 0.008, planet1.getTeraformingGrowth(), 0.001);
        planet1.setTeraforming(0.7);
        assertEquals("Planet has correct terraform growth", 0.0057, planet1.getTeraformingGrowth(), 0.0001);
        planet1.setTeraforming(1.1);
        planet1.setPercentEcology(0.0);
        assertEquals("Planet has correct terraform growth", -0.014, planet1.getTeraformingGrowth(), 0.001);
        assertFalse("Maximum Terraform is correct", planet1.canHaveMoreTerraforming());
        planet1.setPercentEcology(0.25);
        
        // Research done is pop + qty labs in use * 1
        assertEquals("Planet is producing correct research output", 100.0, planet1.getResearchWork(true), 0.0);
        player.addLaboratory(tech_ComputerisedLabI);
        assertEquals("Planet is producing correct research output", 150.0, planet1.getResearchWork(true), 0.0);
        assertEquals("Planet is producing correct research output", 37, planet1.getResearchOutput());

        assertEquals("Players best factory has correct cost to build", 40, planet1.getOwner().getFactories().getCostToBuild());
        assertEquals("Planet is producing correct development output", 37.5, planet1.getDevelopmentOutput(), 0.0);
        assertEquals("Planet is producing correct quantity of factories", 0.9375, planet1.getFactoriesGrowth(), 0.001);
        
        assertEquals("Planet is producing correct production output", 37, planet1.getProductionOutput());
        
        // TEST PLANET BONUSES WORK
        planet1.setProductionBonus(1.0);
        planet1.setResearchBonus(1.0);

        assertEquals("Planet is producing correct production output", 75, planet1.getProductionOutput());
        assertEquals("Planet is producing correct research output", 75, planet1.getResearchOutput());

        planet1.setTeraforming(1.0);
        planet1.setPopulation(10.0);
        assertEquals("Population growth is correct", 0.4, planet1.getPopulationGrowth(), 0.0);
        planet1.setReproductionBonus(1.0);
        assertEquals("Population growth is correct", 0.8, planet1.getPopulationGrowth(), 0.0);
        planet1.setPopulation(90.0);
        assertEquals("Population growth is correct", 0.8, planet1.getPopulationGrowth(), 0.0);
        planet1.setPopulation(50.0);
        assertEquals("Population growth is correct", 2.0, planet1.getPopulationGrowth(), 0.0);
		planet1.getOwner().addBiotechnology(tech_spice25);
        assertEquals("Population growth is correct", 2.25, planet1.getPopulationGrowth(), 0.0);
        planet1.setPopulation(100.0);
        assertFalse("Population growth is correct", planet1.canHaveMorePopulation());
    }

    public void testCrisisPlague() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testCrisisPlague() *************************");
        createMap(null, false);

        planet1.setSize(100);
        planet1.setTeraforming(1.0);
        planet1.setQtyFactories(75.0);
        planet1.setPopulation(100.0);
        
        CrisisFactory.createPlagueCrisis(planet1);
        
        assertEquals("Population growth is correct", -5.0, planet1.getPopulationGrowth(), 0.0);
        
        planet1.endTurn();

        assertEquals("Population growth is correct", -4.44, planet1.getPopulationGrowth(), 0.1);
        assertEquals("Population growth is correct", 95.0, planet1.getPopulation(), 0.1);

        MotEngine.instance.setDate(MotEngine.instance.getDate()+11);
        planet1.endTurn();
        
        assertNull(planet1.getCrisis());

    }

    public void testCrisisPsi() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testCrisisPsi() *************************");
        createMap(null, false);

        planet1.setSize(100);
        planet1.setTeraforming(1.0);
        planet1.setQtyFactories(75.0);
        planet1.setPopulation(100.0);
        
        CrisisFactory.createPsiCrisis(planet1);
        
        assertEquals("Espionage Effected", 0.15, MotEngine.instance.getPlanets().getTotalEspionageEffect(player), 0.0);
        
        planet1.endTurn();

        assertEquals("Espionage Effected", 0.15, MotEngine.instance.getPlanets().getTotalEspionageEffect(player), 0.0);

        MotEngine.instance.setDate(MotEngine.instance.getDate()+11);
        planet1.endTurn();
        
        assertNull(planet1.getCrisis());

    }

    public void testCrisisEmergency() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testCrisisEmergency() *************************");
        createMap(null, false);

        planet1.setSize(100);
        planet1.setTeraforming(1.0);
        planet1.setQtyFactories(75.0);
        planet1.setPopulation(100.0);
        
        assertEquals(150, planet1.getIndustryWork(), 0.1);
        assertEquals(37, planet1.getProductionOutput(), 0.1);

        CrisisFactory.createEmergencyCrisis(planet1);
        
        assertEquals(150, planet1.getIndustryWork(), 0.1);
        assertEquals(75, planet1.getProductionOutput(), 0.1);

        MotEngine.instance.setDate(MotEngine.instance.getDate()+11);
        planet1.endTurn();
        
        assertNull(planet1.getCrisis());

    }

    
    public void testPercentsAuto() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testPercentsAuto() *************************");
        createMap(null, false);

        assertEquals(0.1, player.getPlayerCharacter().getPlayerDevelopmentPercent(), 0.1);
        assertEquals(0.1, player.getPlayerCharacter().getPlayerEcologyPercent(), 0.1);
        assertEquals(0.5, player.getPlayerCharacter().getPlayerProductionPercent(), 0.1);
        assertEquals(0.5, player.getPlayerCharacter().getPlayerResearchPercent(), 0.1);
        assertEquals(0.2, player.getPlayerCharacter().getPlayerIsolationistProductionPercent(), 0.1);
        assertEquals(0.8, player.getPlayerCharacter().getPlayerIsolationistResearchPercent(), 0.1);

        assertTrue(player.hasDiscovered(tech_basicComputerisedFactory));
        assertEquals(0.5, tech_basicComputerisedFactory.getQtyPerPopulation());

        planet1.setSize(100);
        planet1.setPopulation(100);
        planet1.setTeraforming(0.9);
        planet1.setQtyFactories(1.0);
        planet1.setPopulation(100.0);

        planet1.setPercentsAuto(true, true, false);

        assertTrue(planet1.canHaveMoreFactories());
        assertTrue(planet1.canHaveMoreTerraforming());

        assertEquals(0.1, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.2, planet1.getPercentEcology(), 0.01);
        assertEquals(0.32, planet1.getPercentProduction(), 0.01);
        assertEquals(0.32, planet1.getPercentResearch(), 0.01);

        planet1.setPercentDevelopment(1.0);
        planet1.setPercentEcology(1.0);

        assertEquals(10.5, planet1.getMinimumEcologyOutput(), 0.1);
        assertEquals(101, planet1.getEcologyOutput(), 0.1);
        assertEquals(0.018, planet1.getTeraformingGrowth(), 0.001);
        assertEquals(2.52, planet1.getFactoriesGrowth(), 0.1);
        
        planet1.setPercentsAuto(true, true, false);

        assertTrue(planet1.canHaveMoreFactories());
        assertTrue(planet1.canHaveMoreTerraforming());

        assertEquals(0.1, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.20, planet1.getPercentEcology(), 0.01);
        assertEquals(0.32, planet1.getPercentProduction(), 0.01);
        assertEquals(0.32, planet1.getPercentResearch(), 0.01);

        assertEquals(10.5, planet1.getMinimumEcologyOutput(), 0.1);
        assertEquals(20.6, planet1.getEcologyOutput(), 0.1);
        assertEquals(0.002, planet1.getTeraformingGrowth(), 0.001);
        assertEquals(0.25, planet1.getFactoriesGrowth(), 0.1);
        
        planet1.setQtyFactories(50);
        planet1.setTeraforming(1.0);
        
        assertEquals(50.0, planet1.getMaxFactories());

        planet1.setPercentsAuto(true, true, false);

        assertFalse(planet1.canHaveMoreFactories());
        assertFalse(planet1.canHaveMoreTerraforming());

        assertEquals(35.0, planet1.getMinimumEcologyOutput(), 0.1);
        assertEquals(35.0, planet1.getEcologyOutput(), 0.1);
        assertEquals(0.0, planet1.getTeraformingGrowth(), 0.1);
        assertEquals(0.0, planet1.getFactoriesGrowth(), 0.1);
        
        assertEquals(0.0, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.23, planet1.getPercentEcology(), 0.01);
        assertEquals(0.35, planet1.getPercentProduction(), 0.01);
        assertEquals(0.35, planet1.getPercentResearch(), 0.01);
        
        assertEquals(0.25, tech_computerisedFactoryII.getQtyPerPopulation());
        player.addTechnology(tech_computerisedFactoryII);
        
        assertEquals(75.0, planet1.getMaxFactories());

        planet1.setPercentsAuto(true, true, false);

        assertTrue(planet1.canHaveMoreFactories());
        assertFalse(planet1.canHaveMoreTerraforming());

        assertEquals(35.0, planet1.getMinimumEcologyOutput(), 0.1);
        assertEquals(35.0, planet1.getEcologyOutput(), 0.1);
        assertEquals(0.0, planet1.getTeraformingGrowth(), 0.1);
        assertEquals(0.5, planet1.getFactoriesGrowth(), 0.1);
        
        assertEquals(0.1, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.17, planet1.getPercentEcology(), 0.01);
        assertEquals(0.33, planet1.getPercentProduction(), 0.01);
        assertEquals(0.33, planet1.getPercentResearch(), 0.01);

        planet1.setQtyFactories(75);
        planet1.setPercentsAuto(true, true, false);

        assertFalse(planet1.canHaveMoreFactories());
        assertFalse(planet1.canHaveMoreTerraforming());

        assertEquals(47.5, planet1.getMinimumEcologyOutput(), 0.1);
        assertEquals(47.5, planet1.getEcologyOutput(), 0.1);
        assertEquals(0.0, planet1.getTeraformingGrowth(), 0.1);
        assertEquals(0.0, planet1.getFactoriesGrowth(), 0.1);
        
        assertEquals(0.0, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.19, planet1.getPercentEcology(), 0.01);
        assertEquals(0.38, planet1.getPercentProduction(), 0.01);
        assertEquals(0.38, planet1.getPercentResearch(), 0.01);

        planet1.setPercentsAuto(true, true, true);
        
        assertEquals(0.0, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.19, planet1.getPercentEcology(), 0.01);
        assertEquals(0.15, planet1.getPercentProduction(), 0.01);
        assertEquals(0.61, planet1.getPercentResearch(), 0.01);

    }
    
    public void testVaryPercents() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testPercentsAuto() *************************");
        createMap(null, false);

        planet1.setAutoEcology(false);
        planet1.setAutoDevelopment(false);
        planet1.setPercentDevelopment(0.2);
        planet1.setPercentEcology(0.2);
        planet1.setPercentProduction(0.25);
        planet1.setPercentResearch(0.25);
        
        assertEquals(0.1, planet1.getUnusedPercent(), 0.01);
        
        new SetPlanetAutoPercentsCmd(planet1, null, null, true, false, player).run();
        
        planet1.varyProductionResearchPercents();        
        
        assertEquals(0.1, planet1.getTargetSurplusPercent(), 0.01);
        assertEquals(0.2, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.2, planet1.getPercentEcology(), 0.01);
        assertEquals(0.25, planet1.getPercentProduction(), 0.01);
        assertEquals(0.25, planet1.getPercentResearch(), 0.01);
        assertEquals(0.1, planet1.getUnusedPercent(), 0.01);

        planet1.setPercentResearch(0.4);        
        planet1.varyProductionResearchPercents();        
        
        assertEquals(0.1, planet1.getTargetSurplusPercent(), 0.01);
        assertEquals(0.2, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.2, planet1.getPercentEcology(), 0.01);
        assertEquals(0.1, planet1.getPercentProduction(), 0.01);
        assertEquals(0.4, planet1.getPercentResearch(), 0.01);
        assertEquals(0.1, planet1.getUnusedPercent(), 0.01);

        new SetPlanetAutoPercentsCmd(planet1, null, null, false, true, player).run();
        
        planet1.varyProductionResearchPercents();        
        
        assertEquals(0.1, planet1.getTargetSurplusPercent(), 0.01);
        assertEquals(0.2, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.2, planet1.getPercentEcology(), 0.01);
        assertEquals(0.1, planet1.getPercentProduction(), 0.01);
        assertEquals(0.4, planet1.getPercentResearch(), 0.01);
        assertEquals(0.1, planet1.getUnusedPercent(), 0.01);

        planet1.setPercentProduction(0.25);
        planet1.varyProductionResearchPercents();        
        
        assertEquals(0.1, planet1.getTargetSurplusPercent(), 0.01);
        assertEquals(0.2, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.2, planet1.getPercentEcology(), 0.01);
        assertEquals(0.25, planet1.getPercentProduction(), 0.01);
        assertEquals(0.25, planet1.getPercentResearch(), 0.01);
        assertEquals(0.1, planet1.getUnusedPercent(), 0.01);

        new SetPlanetPercentsCmd(planet1, 0.2, 0.2, 0.2, 0.2, player).run();
        
        assertNull(planet1.getTargetSurplusPercent());
        assertEquals(false, planet1.getVaryProduction());
        assertEquals(false, planet1.getVaryResearch());
        assertEquals(0.2, planet1.getPercentDevelopment(), 0.01);
        assertEquals(0.2, planet1.getPercentEcology(), 0.01);
        assertEquals(0.2, planet1.getPercentProduction(), 0.01);
        assertEquals(0.2, planet1.getPercentResearch(), 0.01);
        assertEquals(0.2, planet1.getUnusedPercent(), 0.01);


    }
    
    public void testMoneyOutput() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testMoneyOutput() *************************");
        createMap(null, false);

        assertEquals(tech_basicComputerisedFactory, player.getFactories());
        
        planet1.setSize(100);
        planet1.setTeraforming(1.0);
        planet1.setPopulation(100.0);
        planet1.setProductionBonus(0.0);
        planet1.setQtyFactories(50.0);
        
        //
        // Money output is unused percent
        //

        planet1.setPercentDevelopment(0.2);
        planet1.setPercentEcology(0.2);
        planet1.setPercentProduction(0.2);
        planet1.setPercentResearch(0.36);
        
        assertEquals(0.05, planet1.getUnusedPercent(), 0.01);
        assertEquals(150.0, planet1.getIndustryWork(), 0.01);
        assertEquals(0, planet1.getMoneyWasted());
        assertEquals(6, planet1.getMoneyOutput());
             
        planet1.setPercentDevelopment(0.2);
        planet1.setPercentEcology(0.2);
        planet1.setPercentProduction(0.2);
        planet1.setPercentResearch(0.26);
        
        assertEquals(0.14, planet1.getUnusedPercent(), 0.01);
        assertEquals(150.0, planet1.getIndustryWork(), 0.01);
        assertEquals(4, planet1.getMoneyWasted());
        assertEquals(17, planet1.getMoneyOutput());
        
	    planet1.setPercentDevelopment(0.2);
	    planet1.setPercentEcology(0.2);
	    planet1.setPercentProduction(0.2);
	    planet1.setPercentResearch(0.2);
	    
	    assertEquals(0.2, planet1.getUnusedPercent(), 0.01);
	    assertEquals(150.0, planet1.getIndustryWork(), 0.01);
	    assertEquals(8, planet1.getMoneyWasted());
	    assertEquals(22, planet1.getMoneyOutput());
       
        //
        // Money output is unused percent + production percent
        //
        
        planet1.setProject(null);        
        
        assertEquals(0.2, planet1.getUnusedPercent(), 0.01);
        assertEquals(150.0, planet1.getIndustryWork(), 0.01);
        assertEquals(33, planet1.getMoneyWasted());
        assertEquals(27, planet1.getMoneyOutput());
        
        //
        // A rich planet also produces more money
        //
        
        planet1.setProductionBonus(2.0);
        
        assertEquals(0.2, planet1.getUnusedPercent(), 0.01);
        assertEquals(150.0, planet1.getIndustryWork(), 0.01);
        assertEquals(101, planet1.getMoneyWasted());
        assertEquals(79, planet1.getMoneyOutput());
        
        planet1.setProject(new ProjectCraft(defenderDesign, planet1, true));

        assertEquals(0.2, planet1.getUnusedPercent(), 0.01);
        assertEquals(150.0, planet1.getIndustryWork(), 0.01);
        assertEquals(26, planet1.getMoneyWasted());
        assertEquals(64, planet1.getMoneyOutput());
        
        planet1.setPercentDevelopment(0.1);
        planet1.setPercentEcology(0.1);
        planet1.setPercentProduction(0.25);
        planet1.setPercentResearch(0.1);
    
        assertEquals(0.45, planet1.getUnusedPercent(), 0.01);
        assertEquals(150.0, planet1.getIndustryWork(), 0.01);
        assertEquals(121, planet1.getMoneyWasted());
        assertEquals(82, planet1.getMoneyOutput());
        
        planet1.setPercentDevelopment(0.0);
        planet1.setPercentEcology(0.28);
        planet1.setPercentProduction(0.05);
        planet1.setPercentResearch(0.38);
        
        planet1.setProject(null);
        planet1.setProductionBonus(0.0);
    
        assertEquals(0.29, planet1.getUnusedPercent(), 0.01);
        assertEquals(150.0, planet1.getIndustryWork(), 0.01);
        assertEquals(25, planet1.getMoneyWasted());//28
        assertEquals(25, planet1.getMoneyOutput());
        
    }
    
    
    public void testMoneyWastedSpecialTest() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testMoneyOutput() *************************");
        createMap(null, false);

        assertEquals(tech_basicComputerisedFactory, player.getFactories());
        
        planet1.setSize(120);
        planet1.setTeraforming(1.3);
        planet1.setPopulation(235.0);
        planet1.setProductionBonus(3.0);
        planet1.setQtyFactories(120.0);
       
        assertTrue(planet1.getProject() != null);
        
        //
        // Money output is unused percent
        //

        planet1.setPercentProduction(0.5);
        planet1.setPercentEcology(0.3);
        planet1.setPercentDevelopment(0.0);
        planet1.setPercentResearch(0.15);
        
        assertEquals(0.05, planet1.getUnusedPercent(), 0.01);
        assertEquals(705.0, planet1.getProductionOutput(true), 0.01);
        assertEquals(71, planet1.getMoneyOutput());
        assertEquals(0, planet1.getMoneyWasted());
        
        planet1.setProject(null);
        
        assertEquals(0.05, planet1.getUnusedPercent(), 0.01);
        assertEquals(705.0, planet1.getProductionOutput(true), 0.01);
        assertEquals(705, planet1.getProductionOutput());
        assertEquals(269, planet1.getMoneyOutput());
        assertEquals(507, planet1.getMoneyWasted());
        
   }
    
    /**
     * This tests that the player is getting the correct bonuses from the planet's population character.
     * Tests that when the owner of a planet changes the origin player still provides the correct bonuses.
     * Tests with maximum factories
     */
    public void testPlayerBonuses() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testPlayerBonuses() *************************");
        SetupOptions optionsRace = new SetupOptions();
        optionsRace.setOption("production-bonus", "0.25", 0);
        optionsRace.setOption("research-bonus", "0.25", 0);
        optionsRace.setOption("development-bonus", "0.25", 0);
        optionsRace.setOption("reproduction-bonus", "0.25", 0);
        optionsRace.setOption("ecology-bonus", "0.25", 0);
        optionsRace.setOption("trading-bonus", "0.25", 0);
        optionsRace.setOption("charisma-bonus", "0.25", 0);
        optionsRace.setOption("factory-per-population-bonus", "0.25", 0);
        optionsRace.setOption("max-population-bonus", "0.25", 0);
        optionsRace.setOption("psi-rating", "20", 0);
        optionsRace.setOption("speed-bonus", "1.25", 0);
        optionsRace.setOption("home-planet-size", "120", 0);
        optionsRace.setOption("home-planet-population", "50", 0);
        optionsRace.setOption("home-planet-teraforming", "1.1", 0);
        optionsRace.setOption("home-planet-qty-factories", "10", 0);
        optionsRace.setOption("home-planet-production-bonus", "1.5", 0);
        optionsRace.setOption("home-planet-research-bonus", "2.0", 0);
        optionsRace.setOption("home-planet-reproduction-bonus", "2.5", 0);
        
        createMap(optionsRace, true);

        assertEquals("Home Planet is correct size", 120, planet1.getSize());
        assertEquals("Home Planet has correct pop", 50.0, planet1.getPopulation(), 0.001);
        assertEquals("Home Planet has correct terraform", 1.1, planet1.getTeraforming(), 0.001);
        assertEquals("Home Planet has correct qty factories", 10.0, planet1.getQtyFactories(), 0.001);
        assertEquals("Home Planet has correct production bonus", 1.5, planet1.getProductionBonus(), 0.001);
        assertEquals("Home Planet has correct research bonus", 2.0, planet1.getResearchBonus(), 0.001);
        assertEquals("Home Planet has correct reproduction bonus", 2.5, planet1.getReproductionBonus(), 0.001);
        
        planet1.setSize(100);
        planet1.setTeraforming(1.0);
        planet1.setQtyFactories(100.0);
        planet1.setPopulation(10.0);
        planet1.setReproductionBonus(0.0);
        planet1.setResearchBonus(0.0);
        planet1.setProductionBonus(0.0);
        
        assertEquals(player, planet1.getOwner());
        assertEquals(null, planet1.getOriginReference());

        assertEquals("Players best factory has correct production", 1, player.getFactories().getProduction());
        assertEquals("Players best factory has correct pollution", 0.5, player.getFactories().getMinEcologyWork(), 0.0);
        assertEquals("Max Population is correct", 125.0, planet1.getMaxPopulation(), 0.0);

        assertEquals("Population growth is correct", 0.5, planet1.getPopulationGrowth(), 0.001);
        planet1.setPopulation(90.0);
        assertEquals("Population growth is correct", 0.968, planet1.getPopulationGrowth(), 0.001);
        planet1.setPopulation(125.0);
        assertFalse("Population growth is correct", planet1.canHaveMorePopulation());
        
        assertEquals("Maximum factories planet can use is correct", 78.125, planet1.getQtyFactoriesInUse(), 0.001);
        assertFalse("Maximum factories is correct", planet1.canHaveMoreFactories());
        
        // Work done is pop + qty factories in use * 1
        assertEquals("Planet is producing correct industry output", 203.125, planet1.getIndustryWork(), 0.001);
        assertEquals("Planet is producing correct pollution", 51.562, planet1.getMinimumEcologyOutput(), 0.001);
        assertEquals("Planet is producing correct pollution", 0.203, planet1.getMinimumEcologyPercent(), 0.001);
        assertEquals("Planet is producing correct ecology work", 63.476, planet1.getEcologyOutput(), 0.001);
        
        // Research done is pop + qty factories in use * 1
        assertEquals("Planet is producing correct research output", 125.0, planet1.getResearchWork(true), 0.001);
        planet1.getOwner().addLaboratory(tech_ComputerisedLabI);
        assertEquals("Planet is producing correct research output", 203.125, planet1.getResearchWork(true), 0.001);

        assertEquals("Planet is producing correct production output", 63, planet1.getProductionOutput());
        assertEquals("Planet is producing correct production output", 63, planet1.getResearchOutput());

        planet1.setPopulation(10.0);

        // I will now change the owner to a player that has no bonuses, and the bonuses
        // should not change
        planet1.changeOwner(optionalPlayer);
        
        assertEquals(optionalPlayer, planet1.getOwner());
        assertEquals(player, planet1.getOrigin());

        assertEquals("Players best factory has correct production", 1, player.getFactories().getProduction());
        assertEquals("Players best factory has correct pollution", 0.5, player.getFactories().getMinEcologyWork(), 0.0);
        assertEquals("Max Population is correct", 125.0, planet1.getMaxPopulation(), 0.0);

        assertEquals("Population growth is correct", 0.5, planet1.getPopulationGrowth(), 0.001);
        planet1.setPopulation(90.0);
        assertEquals("Population growth is correct", 0.96875, planet1.getPopulationGrowth(), 0.001);
        planet1.setPopulation(125.0);
        assertFalse("Population growth is correct", planet1.canHaveMorePopulation());
        
        assertEquals("Maximum factories planet can use is correct", 78.125, planet1.getQtyFactoriesInUse(), 0.001);
        assertFalse("Maximum factories is correct", planet1.canHaveMoreFactories());
        
        // Work done is pop + qty factories in use * 1
        assertEquals("Planet is producing correct industry output", 203.125, planet1.getIndustryWork(), 0.001);
        assertEquals("Planet is producing correct pollution", 51.562, planet1.getMinimumEcologyOutput(), 0.001);
        assertEquals("Planet is producing correct pollution", 0.203, planet1.getMinimumEcologyPercent(), 0.001);
        assertEquals("Planet is producing correct ecology work", 63.476, planet1.getEcologyOutput(), 0.001);
        
        // Research done is pop + qty factories in use * 1
        assertEquals("Planet is producing correct research output", 125.0, planet1.getResearchWork(true), 0.001);
        planet1.getOwner().addLaboratory(tech_ComputerisedLabI);
        assertEquals("Planet is producing correct research output", 203.125, planet1.getResearchWork(true), 0.001);

        assertEquals("Planet is producing correct production output", 63, planet1.getProductionOutput());
        assertEquals("Planet is producing correct production output", 63, planet1.getResearchOutput());

        planet1.setPopulation(10.0);

        // I will now check the planets output is effected by the owners technology (not
        // the origins technology).

        assertEquals("Max Population is correct", 125.0, planet1.getHighestMaxPopulation(), 0.0);
        assertEquals("Planet is producing correct production output", 5, planet1.getProductionOutput());
        assertEquals("Planet is producing correct production output", 5, planet1.getResearchOutput());
        assertEquals("Planet is producing correct production output", 5, planet1.getDevelopmentOutput(), 0.1);
        assertEquals("Planet is producing correct production output", 5, planet1.getEcologyOutput(), 0.1);
        
        optionalPlayer.addTechnology(tech_basicComputerisedFactory);
        optionalPlayer.addTechnology(tech_computerisedFactoryII);
        optionalPlayer.addTechnology(tech_ComputerisedLabI);
        optionalPlayer.addTechnology(tech_ComputerisedLabII);
        optionalPlayer.addTechnology(tech_terraforming10);

        assertEquals("Players best factory has correct production", 2, planet1.getOwner().getFactories().getProduction());
        assertEquals("Players best laboratory has correct production", 2, planet1.getOwner().getLaboratories().getProduction());

        assertEquals("Max Population is correct", 137.5, planet1.getHighestMaxPopulation(), 0.0);
        assertEquals("Planet is producing correct production output", 8, planet1.getProductionOutput());
        assertEquals("Planet is producing correct production output", 8, planet1.getResearchOutput());
        assertEquals("Planet is producing correct production output", 8.9, planet1.getDevelopmentOutput(), 0.1);
        assertEquals("Planet is producing correct production output", 8.9, planet1.getEcologyOutput(), 0.1);
        
    }

    /**
     * This tests that the planet gets the correct bonuses when it has a mining station.
     * Tests with no factories
     */
    public void testMiningStation() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testMiningStation() *************************");
        createMap(null, false);

        planet1.setDateColonised(3000);
        planet1.setSize(100);
        planet1.setTeraforming(1.0);
        planet1.setQtyFactories(0);
        planet1.setPopulation(100.0);
        planet1.setReproductionBonus(0.0);
        planet1.setResearchBonus(0.0);
        planet1.setProductionBonus(0.0);
        
        assertEquals(planet1, player.getCapitol());
        
        assertEquals(25, planet1.getProductionOutput());
        assertEquals(25, planet1.getResearchOutput());
        assertEquals(25, (int)planet1.getDevelopmentOutput());
        assertEquals(25, (int)planet1.getEcologyOutput());

        // Test that a space station deploys successfully but does not provide any
        // bonuses without modules
        Orbital orbital = new Orbital(tech_spaceStation, null, null);
        planet2.deploy(planet1, orbital);

        assertEquals(planet1, planet2, player.getPlanets());
        assertEquals(player, planet2.getOwner());
        assertEquals(planet2, planet1.getMiningStation().getPlanet());
        assertEquals(orbital, planet1.getMiningStation().getOrbital());
        assertEquals(planet1, player.getCapitol());
        
        assertEquals(25, planet1.getProductionOutput());
        assertEquals(25, planet1.getResearchOutput());
        assertEquals(25, (int)planet1.getDevelopmentOutput());
        assertEquals(25, (int)planet1.getEcologyOutput());
        assertEquals(100, (int)planet1.getMaxPopulation());

        // Test endTurn() on mining colony does not do anything
        player.setResearchProject(new ResearchProject(tech_armour25, player, false));
        assertEquals(0, player.getResearchProject().getWork());
        assertEquals(0.0, planet2.getPopulation(), 0.0);
        planet2.endTurn();
        assertEquals(0, player.getResearchProject().getWork());
        assertEquals(0.0, planet2.getPopulation(), 0.0);
        
        // Test deploying modules provides bonuses
        orbital.addModule(tech_oreExtractor);
        orbital.addModule(tech_researchModule);
        orbital.addModule(tech_habitatI);

        assertEquals(45, planet1.getProductionOutput());
        assertEquals(55, planet1.getResearchOutput());
        assertEquals(25, (int)planet1.getDevelopmentOutput());
        assertEquals(25, (int)planet1.getEcologyOutput());
        assertEquals(105, (int)planet1.getMaxPopulation());
        
        // Test artifacted and mineral rich planets boost production bonuses
        planet2.setResearchBonus(2.0);
        planet2.setProductionBonus(1.0);

        assertEquals(65, planet1.getProductionOutput());
        assertEquals(115, planet1.getResearchOutput());
        assertEquals(25, (int)planet1.getDevelopmentOutput());
        assertEquals(25, (int)planet1.getEcologyOutput());
        assertEquals(110, (int)planet1.getMaxPopulation());
        
        // Test endTurn works
        planet1.setProject(new ProjectCraft(defenderDesign, planet1, true));
        planet1.endTurn();
        assertEquals(115, player.getResearchProject().getWork());
        assertEquals(65, planet1.getProduction());
        
        // Test mining stations are automatically upgraded when new modules already on station are discovered.
        player.addTechnology(tech_habitatII);

        Technology[] modules = planet1.getMiningStation().getOrbital().getModuleTechs().getAllAsArray();
		assertEquals(4, modules.length);
		assertEquals(tech_oreExtractor, modules[0]);
		assertEquals(tech_researchModule, modules[1]);
		assertEquals(tech_habitatI, modules[2]);
		assertEquals(tech_habitatII, modules[3]);

        // Test losing the mining station (e.g. in a battle)
        planet2.destroyOrbital();

        assertEquals(null, planet1.getMiningStation());
        assertEquals(null, planet2.getOrbital());
        assertEquals(null, planet2.getOwner());

        assertEquals(25, planet1.getProductionOutput());
        assertEquals(25, planet1.getResearchOutput());
        

    }
    
    /**
     * Tests that a planet builds a project in the correct quantity of time.
     */
    public void testProjectCompletion() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProjectCompletion() *************************");
        createMap(null, true);

        // TEST BUILD CRAFT

        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_massDriver1);
        
        Project project = new ProjectCraft(cd, planet1, true); 
        planet1.setProject(project);
        planet1.addProduction((int)project.getCostToBuildOnPlanet()-1);
        
        assertNull(player.getCrafts());
        planet1.addProduction(1);        
        assertEquals(1, player.getCrafts().length);
        Craft craft = player.getCrafts()[0];
        
        assertTrue(craft.getName().startsWith("TestFighter"));
        assertEquals(player, craft.getOwner());
        assertEquals(player, craft.getOrigin());
        assertEquals(0, planet1.getProduction());
        
        assertEquals(tech_flyer30, craft.getHull());
        assertEquals(tech_fusion25, craft.getEngine());
        assertEquals(tech_electronic25, craft.getComputers().getComputerAtIndex(0));
        assertNull(craft.getScanners());
        assertEquals(tech_energy10, craft.getShields().getShields()[0]);
        assertEquals(1, craft.getWeaponBays().getBaysAsArray().length);
        assertEquals(1, craft.getWeaponBays().getBaysAsArray()[0].getWeapons().getQuantity());
        assertEquals(tech_massDriver1, craft.getWeaponBays().getBaysAsArray()[0].getWeapons().getWeaponAtIndex(0));
        assertEquals(0.7, craft.getBuildQuality());
        
        NameIntPair cde = player.getCraftBuildExperience().getDesignExperience(tech_flyer30.getName());
        assertEquals(1, cde.getValue());

        // BUILD ANOTHER CRAFT AND CHECK DESGIN EXPERINCE HAS GONE UP
        // DISCOVER QUALITY_CONTROL AND ALRAEDY BUILD ONE OF THIS TYPE

        player.addFactory(tech_qualityControl);
        project = new ProjectCraft(cd, planet1, true); 
        planet1.setProject(project);
        planet1.addProduction((int)project.getCostToBuildOnPlanet());
        
        assertEquals(2, player.getCrafts().length);
        craft = player.getCrafts()[1];
        
        assertTrue(craft.getName().startsWith("TestFighter"));
        assertEquals(player, craft.getOwner());
        assertEquals(0, planet1.getProduction());
        assertEquals(0.85, craft.getBuildQuality());
        
        cde = player.getCraftBuildExperience().getDesignExperience(tech_flyer30.getName());
        assertEquals(2, cde.getValue());
        
        // TEST BUILD CRAFT WITH ORIGIN PLAYER DIFFERENT TO OWNER
        // CRAFT ALSO HAS WEAPON BAY DETAILS AND MORE STUFF

        cd = new CraftDesign("TestPowerful", 
				tech_flyer100,
				tech_fusion40,
				tech_electronic40,
				tech_photon200,
				tech_energy20,
				tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4,
				tech_missile6, tech_missile6,
				tech_shieldDrain,
				tech_shieldModulatorII, tech_augumentedEnginesI, tech_eneryAmplifierI, tech_armour25);
        
		planet1.setOrigin(optionalPlayer);
        project = new ProjectCraft(cd, planet1, true); 
        planet1.setProject(project);
        planet1.addProduction((int)project.getCostToBuildOnPlanet());
        craft = player.getCrafts()[2];
        
        assertEquals(3, player.getCrafts().length);
		assertTrue(craft.getName().startsWith("TestPowerful"));
        assertEquals(player, craft.getOwner());
        assertEquals(optionalPlayer, craft.getOrigin());
        assertEquals(0, planet1.getProduction());
        
        assertEquals(tech_flyer100, craft.getHull());
        assertEquals(tech_fusion40, craft.getEngine());
        assertEquals(tech_electronic40, craft.getComputers().getComputerAtIndex(0));
        assertEquals(tech_photon200, craft.getScanners().getScanners()[0]);
        assertEquals(tech_energy20, craft.getShields().getShields()[0]);
        WeaponBundle[] weapons = craft.getWeaponBays().getDistinctWeapons();
		assertEquals(3, weapons.length);
        assertEquals(tech_fusionBeam4, weapons[0].getWeapon());
        assertEquals(4, weapons[0].getQty());
        assertEquals(tech_missile6, weapons[1].getWeapon());
        assertEquals(2, weapons[1].getQty());
        assertEquals(tech_shieldDrain, weapons[2].getWeapon());
        assertEquals(1, weapons[2].getQty());
        Technology [] specials = craft.getSpecials().getAllAsArray();
        assertEquals(tech_shieldModulatorII, specials[0]);
        assertEquals(tech_augumentedEnginesI, specials[1]);
        assertEquals(tech_eneryAmplifierI, specials[2]);
        assertEquals(tech_armour25, specials[3]);

        assertEquals(0.75, craft.getBuildQuality());
        
        cde = player.getCraftBuildExperience().getDesignExperience(tech_flyer100.getName());
        assertEquals(1, cde.getValue());
        
        planet1.setOrigin(null);
        
        // TEST BUILD CRAFT WITH WAYPOINT SET
        userPlayer.setFleetWayPoint(new Orbit(planet1));

        assertEquals(0, userPlayer.getCrafts().length);
        
        project = new ProjectCraft(defenderDesign, planetu, true); 
        planetu.setProject(project);
        planetu.addProduction((int)project.getCostToBuildOnPlanet());
        
        assertEquals(1, userPlayer.getCrafts().length);
        craft = userPlayer.getCrafts()[0];
        assertTrue(craft.getName().startsWith("TestDefender"));
        assertEquals(planetu, craft.getOrbit().getPlanet());
        assertEquals(planetu, craft.getNavigator().getOrigin().getPlanet());
        assertEquals(planet1, craft.getNavigator().getDestination().getPlanet());

        craft.endTurn();
        
        assertEquals((Orbit)null, craft.getOrbit());
        assertEquals(null, craft.getTask());
        assertEquals(planetu, craft.getNavigator().getOrigin().getPlanet());
        assertEquals(planet1, craft.getNavigator().getDestination().getPlanet());
        
        // TEST BUILD SCANNER

        new SetProjectCmd(planet1, new ScannerDesign(tech_photon100), 0, player, false, false).run();
        project = planet1.getProject();
        planet1.addProduction((int)project.getCostToBuildOnPlanet()-1);
        
        assertNull(planet1.getScanners());
        
        planet1.endTurn();

        assertEquals(1, planet1.getScanners().getAllAsArray().length);
        assertEquals(tech_photon100, planet1.getScanners().getAllAsArray()[0]);


        planet1.setProduction(0);
        // TEST BUILD SHIELD

        new SetProjectCmd(planet1, new ShieldDesign(tech_energy10), 0, player, false, false).run();
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(1, planet1.getShields().getAllAsArray().length);
        assertEquals(tech_energy10, planet1.getShields().getAllAsArray()[0]);
        assertEquals(0, planet1.getProduction());

        // TEST BUILD WEAPON

        new SetProjectCmd(planet1, new WeaponDesign(tech_missile6), 0, player, false, false).run();
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(1, planet1.getWeapons().getAllAsArray().length);
        assertEquals(tech_missile6, planet1.getWeapons().getAllAsArray()[0]);
        assertEquals(0, planet1.getProduction());

        // TEST BUILD BATTLE STATION

        new SetProjectCmd(planet1, new BattleStationDesign(tech_battleStationI), 0, player, false, false).run();
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_battleStationI, planet1.getOrbital().getHullTech());
        assertEquals(tech_core_habitat, planet1.getOrbital().getModuleTechs());
        assertEquals(0, planet1.getProduction());

        // TEST BUILD BATTLE STATION WHEN HAVE LOTS OD MODULES DISCOVERED

        planet1.setOrbital(null);
        player.addModule(tech_habitatI);
        player.addModule(tech_oreExtractor);
        player.addModule(tech_sentinelsModule);
        new SetProjectCmd(planet1, new BattleStationDesign(tech_battleStationI), 0, player, false, false).run();
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        assertEquals(tech_battleStationI, planet1.getOrbital().getHullTech());
        assertEquals(tech_core_habitat, tech_habitatI, tech_sentinelsModule, planet1.getOrbital().getModuleTechs());
        assertEquals(0, planet1.getProduction());
        
        player.getModules().removeAll();

        // TEST BUILD TRANSPORT

        new SetProjectCmd(planet1, new TransportDesign(), 0, player, false, false).run();
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        
        assertEquals(1, player.getTransportManager().getQtyFreeTransport());
        assertEquals(0, planet1.getProduction());

        
        // TEST BUILD OUTPOST

        new SetProjectCmd(planet1, new OutpostDesign(tech_outpost), 0, player, false, false).run();
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        
        assertTrue(player.getTransportManager().getTransportsAsArray()[0].getTask() instanceof TransportTaskDeployStation);
        TransportTaskDeployStation tds = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(tech_outpost, tds.getOrbital().getHullTech()); 
        player.getTransportManager().getTransportsAsArray()[0].setTask(null);
        

        // TEST BUILD MINING STATION

        planet2.setTeraforming(0.5);
        ProjectSpaceStation project2 = new ProjectSpaceStation(new SpaceStationDesign(tech_spaceStation), new Orbit(planet2)); 
        planet1.setProject(project2);
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        Transport transportCraft = player.getTransportManager().getTransportsAsArray()[0];
        assertEquals(null, transportCraft.getTask());
        assertEquals(tech_spaceStation, planet2.getOrbital().getHullTech());
        assertEquals(tech_core_habitat, planet2.getOrbital().getModuleTechs()); 

        new SetProjectCmd(planet1, new SpaceStationDesign(tech_spaceStation), 0, player, false, false).run();
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());

        transportCraft = player.getTransportManager().getTransportsAsArray()[0];
        assertEquals(planet1, ((TransportTaskDeployStation)transportCraft.getTask()).getOrigin().getPlanet());
        assertEquals(tech_spaceStation, ((TransportTaskDeployStation)transportCraft.getTask()).getOrbital().getHullTech());

        // TEST BUILD Asteroids

        new SetProjectCmd(planet1, new AsteroidDesign(tech_asteroidFieldConstruction), 0, player, false, false).run();
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        
        assertEquals(true, planet1.getHasAsteroids()); 
        
 
    }

    /**
     * Tests that a planet builds a project in the correct quantity of time.
     */
    public void testUpgradeTechnologies() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testUpgradeTechnologies() *************************");
        createMap(null, false);

        planet1.setProject(null);
        Planet planet3 = newPlanet("SS1-C", ss1, player);
        Planet planet4 = newPlanet("SS1-D", ss1, player);

        // TEST UPGRADE SCANNER

        player.addScanner(tech_photon100);
        
        assertEquals((Scanners)null, planet1.getScanners());
        assertEquals((Scanners)null, planet2.getScanners());
        assertEquals((Scanners)null, planet3.getScanners());
        assertEquals((Scanners)null, planet4.getScanners());
       
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals((Project)null, planet4.getProject());
       
        planet3.setScanners(new Scanners(tech_photon100));
        planet4.setProject(new ProjectScanner(new ScannerDesign(tech_photon100)));
        
        assertEquals((Scanners)null, planet1.getScanners());
        assertEquals((Scanners)null, planet2.getScanners());
        assertEquals(tech_photon100, planet3.getScanners());
        assertEquals((Scanners)null, planet4.getScanners());
        
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals(tech_photon100, ((ProjectScanner)planet4.getProject()).getScanner());

        player.addScanner(tech_photon150);

        assertEquals((Scanners)null, planet1.getScanners());
        assertEquals((Scanners)null, planet2.getScanners());
        assertEquals(tech_photon150, planet3.getScanners());
        assertEquals((Scanners)null, planet4.getScanners());
        
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals(tech_photon150, ((ProjectScanner)planet4.getProject()).getScanner());

        // TEST UPGRADE SHIELD

        planet4.setProject(null);
        player.addShield(tech_energy10);
        
        assertEquals((Shields)null, planet1.getShields());
        assertEquals((Shields)null, planet2.getShields());
        assertEquals((Shields)null, planet3.getShields());
        assertEquals((Shields)null, planet4.getShields());
       
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals((Project)null, planet4.getProject());
       
        planet3.setShields(new Shields(tech_energy10));
        planet4.setProject(new ProjectShield(new ShieldDesign(tech_energy10)));
        
        assertEquals((Shields)null, planet1.getShields());
        assertEquals((Shields)null, planet2.getShields());
        assertEquals(tech_energy10, planet3.getShields());
        assertEquals((Shields)null, planet4.getShields());
        
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals(tech_energy10, ((ProjectShield)planet4.getProject()).getShield());

        player.addShield(tech_energy20);

        assertEquals((Shields)null, planet1.getShields());
        assertEquals((Shields)null, planet2.getShields());
        assertEquals(tech_energy20, planet3.getShields());
        assertEquals((Shields)null, planet4.getShields());
        
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals(tech_energy20, ((ProjectShield)planet4.getProject()).getShield());

        // TEST UPGRADE WEAPON

        planet4.setProject(null);
        player.addWeapon(tech_missile6);
        
        assertEquals((Weapons)null, planet1.getWeapons());
        assertEquals((Weapons)null, planet2.getWeapons());
        assertEquals((Weapons)null, planet3.getWeapons());
        assertEquals((Weapons)null, planet4.getWeapons());
       
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals((Project)null, planet4.getProject());
       
        planet3.setWeapons(new Weapons(tech_missile6, 2));
        planet4.setProject(new ProjectWeapon(new WeaponDesign(tech_missile6)));
        
        assertEquals((Weapons)null, planet1.getWeapons());
        assertEquals((Weapons)null, planet2.getWeapons());
        assertEquals(tech_missile6, tech_missile6, planet3.getWeapons());
        assertEquals((Weapons)null, planet4.getWeapons());
        
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals(tech_missile6, ((ProjectWeapon)planet4.getProject()).getWeapon());

        player.addWeapon(tech_missile15);

        assertEquals((Weapons)null, planet1.getWeapons());
        assertEquals((Weapons)null, planet2.getWeapons());
        assertEquals(tech_missile15, tech_missile15, planet3.getWeapons());
        assertEquals((Weapons)null, planet4.getWeapons());
        
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals(tech_missile15, ((ProjectWeapon)planet4.getProject()).getWeapon());


        // TEST UPGRADE BATTLE STATION

        Planet planetMine = newPlanet("SS1-M", ss1, null, 60, 0.4);
        planetMine.setProductionBonus(2.0);
        newSpaceStation(planet1, planetMine);

        planet4.setProject(null);
        player.addHull(tech_battleStationI);
        
        assertEquals((Orbital)null, planet1.getOrbital());
        assertEquals((Orbital)null, planet2.getOrbital());
        assertEquals((Orbital)null, planet3.getOrbital());
        assertEquals((Orbital)null, planet4.getOrbital());
       
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals((Project)null, planet4.getProject());
       
        new ProjectBattleStation(new BattleStationDesign(tech_battleStationI)).build(planet3);
        planet4.setProject(new ProjectBattleStation(new BattleStationDesign(tech_battleStationI)));
        
        assertEquals((Orbital)null, planet1.getOrbital());
        assertEquals((Orbital)null, planet2.getOrbital());
        assertEquals(tech_battleStationI, planet3.getOrbital().getHull());
        assertEquals(tech_core_habitat, planet3.getOrbital().getModuleTechs());
        assertEquals((Orbital)null, planet4.getOrbital());
        assertEquals(tech_spaceStation, planetMine.getOrbital().getHull());
        assertEquals((Modules)null, planetMine.getOrbital().getModuleTechs());
        
        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals(tech_battleStationI, ((ProjectBattleStation)planet4.getProject()).getBattleStation());

        player.addHull(tech_battleStationII);

        assertEquals((Orbital)null, planet1.getOrbital());
        assertEquals((Orbital)null, planet2.getOrbital());
        assertEquals(tech_battleStationII, planet3.getOrbital().getHull());
        assertEquals(tech_core_habitat, planet3.getOrbital().getModuleTechs());
        assertEquals((Orbital)null, planet4.getOrbital());
        assertEquals(tech_battleStationII, ((ProjectBattleStation)planet4.getProject()).getBattleStation());
        assertEquals(tech_spaceStation, planetMine.getOrbital().getHull());
        assertEquals((Modules)null, planetMine.getOrbital().getModuleTechs());

        assertEquals((Project)null, planet1.getProject());
        assertEquals((Project)null, planet2.getProject());
        assertEquals((Project)null, planet3.getProject());
        assertEquals(tech_battleStationII, ((ProjectBattleStation)planet4.getProject()).getBattleStation());
        
        
        Planet planetGive = newPlanet("SSU-B", ssu, userPlayer);

        planetGive.setWeapons(new Weapons(tech_missile6));
        planetGive.setShields(new Shields(tech_energy10));
        planetGive.setScanners(new Scanners(tech_photon100));
        planetGive.setOrbital(new Orbital(tech_battleStationI, null, new Modules(tech_core_habitat)));
        planetGive.setProject(new ProjectWeapon(new WeaponDesign(tech_missile6)));

        player.addModule(tech_habitatI);
        player.diplomaticRequest(userPlayer, DiplomaticProposal.gift(planetGive.getName()), null);
        //userPlayer.giveTo(player, new PlayerAssets(planetGive.getName()));
        
        assertEquals(tech_missile15, planetGive.getWeapons());
        assertEquals(tech_photon150, planetGive.getScanners());
        assertEquals(tech_energy20, planetGive.getShields());
        assertEquals(tech_battleStationII, planetGive.getOrbital().getHull());
        assertEquals(tech_core_habitat, tech_habitatI, planetGive.getOrbital().getModuleTechs());
        assertEquals(tech_missile15, ((ProjectWeapon)planetGive.getProject()).getWeapon());
        
        assertEquals(tech_core_habitat, tech_habitatI, planet3.getOrbital().getModuleTechs());
        assertEquals(tech_habitatI, planetMine.getOrbital().getModuleTechs());

        
        Planet planetInvade = newPlanet("SSU-C", ssu, userPlayer);
        planetInvade.setWeapons(new Weapons(tech_missile6));
        planetInvade.setShields(new Shields(tech_energy10));
        planetInvade.setScanners(new Scanners(tech_photon100));
        planetInvade.setOrbital(new Orbital(tech_battleStationI, null, null));
        planetInvade.setProject(new ProjectWeapon(new WeaponDesign(tech_missile6)));
        
        player.addModule(tech_habitatII);
        planetGive.setQtyGroundDefense(50);
        Transport transport = new Transport(player);
        player.getTransportManagerNotNull().setTransport(transport);
        new TransportTaskInvade(transport, planetGive, planetInvade, 20).deploy(transport, null, planetInvade);

        assertEquals(tech_missile15, planetInvade.getWeapons());
        assertEquals(tech_photon150, planetInvade.getScanners());
        assertEquals(tech_energy20, planetInvade.getShields());
        assertEquals(tech_battleStationII, planetInvade.getOrbital().getHull());
        assertEquals(tech_habitatI, tech_habitatII, planetInvade.getOrbital().getModuleTechs());
        assertEquals(null, planetInvade.getProject());
        
        assertEquals(tech_core_habitat, tech_habitatI, tech_habitatII, planet3.getOrbital().getModuleTechs());
        assertEquals(tech_habitatI, tech_habitatII, planetMine.getOrbital().getModuleTechs());

        
        player.addModule(tech_oreExtractor);

        assertEquals((Orbital)null, planet1.getOrbital());
        assertEquals((Orbital)null, planet2.getOrbital());
        assertEquals(tech_battleStationII, planet3.getOrbital().getHull());
        assertEquals(tech_core_habitat, tech_habitatI, tech_habitatII, planet3.getOrbital().getModuleTechs());
        assertEquals((Orbital)null, planet4.getOrbital());
        assertEquals(tech_battleStationII, planetGive.getOrbital().getHull());
        assertEquals(tech_core_habitat, tech_habitatI, tech_habitatII, planetGive.getOrbital().getModuleTechs());
        assertEquals(tech_battleStationII, planetInvade.getOrbital().getHull());
        assertEquals(tech_habitatI, tech_habitatII, planetInvade.getOrbital().getModuleTechs());
        assertEquals(tech_spaceStation, planetMine.getOrbital().getHull());
        assertEquals(tech_habitatI, tech_habitatII, tech_oreExtractor, planetMine.getOrbital().getModuleTechs());

        player.addModule(tech_researchModule);

        assertEquals((Orbital)null, planet1.getOrbital());
        assertEquals((Orbital)null, planet2.getOrbital());
        assertEquals(tech_battleStationII, planet3.getOrbital().getHull());
        assertEquals(tech_core_habitat, tech_habitatI, tech_habitatII, planet3.getOrbital().getModuleTechs());
        assertEquals((Orbital)null, planet4.getOrbital());
        assertEquals(tech_battleStationII, planetGive.getOrbital().getHull());
        assertEquals(tech_core_habitat, tech_habitatI, tech_habitatII, planetGive.getOrbital().getModuleTechs());
        assertEquals(tech_battleStationII, planetInvade.getOrbital().getHull());
        assertEquals(tech_habitatI, tech_habitatII, planetInvade.getOrbital().getModuleTechs());
        assertEquals(tech_spaceStation, planetMine.getOrbital().getHull());
        assertEquals(tech_habitatI, tech_habitatII, tech_oreExtractor, tech_researchModule, planetMine.getOrbital().getModuleTechs());

    }
    

    public void testConstructPlanetProject() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testConstructPlanetProject() *************************");
        createMap(null, false);
        Planet planet3 = newPlanet("SS1-C", ss1, null, 100, 0.1);

        //
        // Get validation error if we do not have enough boralium
        //
        
        ConstructPlanetDesign project = new ConstructPlanetDesign(tech_planetConstruction, ss1.getName());
        try {
        	CommandResult cr = new SetProjectCmd(planet1, project, 0, player, false, false).run();
        	fail("No boralium");
        } catch (ValidationError e) {
        	assertEquals("You do not have enough boralium crystals (100 required)", e.getMessage());
        }
        
        
        //
        // Test we can create the project
        //
        
        player.setBoralium(200);
    	new SetProjectCmd(planet1, project, 0, player, false, false).run();

    	assertEquals(3, ss1.getQtyPlanets());
    	assertEquals(0, player.getBoralium());
    	assertEquals(9000, planet1.getProject().getCostToBuildOnPlanet());
    	assertEquals("Basic Planet Construction", planet1.getProject().getName());
    	
        //
        // Test we can complete the project and the planet is created
        //
        
    	planet1.addProduction(9000);

    	assertEquals(4, ss1.getQtyPlanets());
    	assertEquals(0, player.getBoralium());
    	assertEquals("SS1-D", ss1.getPlanets()[3].getName());
    	assertEquals(110, ss1.getPlanets()[3].getSize());
    	assertEquals(0.7, ss1.getPlanets()[3].getTeraforming());
    	assertEquals(null, ss1.getPlanets()[3].getOwner());
    	assertEquals(0, ss1.getPlanets()[3].getPopulation(), 0.01);
    	
    }
    
    /**
     * This tests a planets and its mining colony are destroyed correctly.
     */
    public void testDestroy() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testDestroy() *************************");
        createMap(null, false);
        
        newSpaceStation(planet1, planet2);
        planet1.removeColony(true);
        
        assertEquals(null, planet2.getOrbital());
        assertEquals(null, planet1.getOwner());
        assertEquals(0.0, planet1.getPopulation(), 0.0);
        
        //
        // Test trade routes destroyed
        //
        
        MotEngine.instance.getProxyManager().add(new ClientProxy() {
			public void notify(MotEventFactory e) throws ApplicationError {	eventList.add(e);	}
			public void notify(MotEvent e) throws ApplicationError {	}
			public Player getOwner() {	return userPlayer;	}
		});
        
        SolarSystem ss2 = newSolarSystem("SS2", 500, 500);
        Planet tp2 = newPlanet("SS2-A", ss2, userPlayer, 100, 1.0, 110, 60);
        
        SolarSystem ss3 = newSolarSystem("SS3", 550, 500);
        Planet tp3 = newPlanet("SS3-A", ss3, userPlayer, 100, 1.0, 110, 60);
        
        SolarSystem ss4 = newSolarSystem("SS4", 600, 500);
        Planet tp4 = newPlanet("SS4-A", ss4, userPlayer, 100, 1.0, 110, 60);
        
        TradeRouteFactory.create(ss2, ss3);
        TradeRouteFactory.create(ss3, ss4);

        assertEquals(ss2, ss3, 3000, ss3, ss4, 3000, MotEngine.instance.getTradeRoutes());

        eventList.clear();
        tp2.removeColony(true);
        
        assertEquals(ss3, ss4, 3000, MotEngine.instance.getTradeRoutes());
        
        eventList.clear();
        tp4.changeOwner(player);
        
        assertEquals(0, MotEngine.instance.getTradeRoutes().getQtyTradeRoutes());
        
    }

    /**
     * This tests a planets and its mining colony are destroyed correctly.
     */
    public void testNatives() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testNatives() *************************");
        createMap(null, false);
        
        planet2.setHasNatives(true);
        planet2.setQtyNatives(30);
        
        Craft coloniser = newCraft(coloniserDesign, player, ss1);
        planet2.colonise(coloniser);
        
        assertEquals(true, planet2.hasNatives());
        assertEquals(0, planet2.getQtyNatives());
        assertEquals(30.0, planet2.getPopulation());
        assertEquals(15.0, planet2.getQtyFactories());
    }


    /**
     * Tests that moral choices made can effect production on a planet
     */
    public void testMoralChoice() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testMoralChoice() *************************");
        createMap(null, false);

        planet1.setQtyFactories(20);
        planet1.setPopulation(20.0);
        planet1.setProductionBonus(0.0);
        planet1.setPercentProduction(1.0);
        
        assertEquals(30.0, planet1.getProductionOutput(), 0.001);
        
        planet1.getOwner().add(mc_testMoralChoice);
        
        assertEquals(33, planet1.getProductionOutput(), 0.001);
    }

    /**
     * NOTE - PLANET PROJECT BUILD CRAFT IS DONE IN CraftDesignTest 
     */
    public void testSetProjectCommandForScannerShieldWeapon() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testSetProjectCommandForScannerShieldWeapon() *************************");
        createMap(null, false);

        Planet scannerPlanet = newPlanet("SS1-SCANNER", ss1, player);
        Planet shieldPlanet = newPlanet("SS1-SHIELD", ss1, player);
        Planet weaponPlanet = newPlanet("SS1-WEAPON", ss1, player);
        Planet battleStationPlanet = newPlanet("SS1-BATTLE_STATION", ss1, player);
            
        //
        // test build projects do work
        //
        
        new SetProjectCmd(scannerPlanet, new ScannerDesign(tech_photon100), 0, player, false, false).run();
        new SetProjectCmd(shieldPlanet, new ShieldDesign(tech_energy10), 0, player, false, false).run();
        new SetProjectCmd(weaponPlanet, new WeaponDesign(tech_massDriver1), 0, player, false, false).run();
        new SetProjectCmd(battleStationPlanet, new BattleStationDesign(tech_battleStationI), 0, player, false, false).run();
        
        scannerPlanet.addProduction((int)scannerPlanet.getProject().getCostToBuildOnPlanet());
        shieldPlanet.addProduction((int)shieldPlanet.getProject().getCostToBuildOnPlanet());
        weaponPlanet.addProduction((int)weaponPlanet.getProject().getCostToBuildOnPlanet());
        battleStationPlanet.addProduction((int)battleStationPlanet.getProject().getCostToBuildOnPlanet());
        
        assertEquals(1, scannerPlanet.getScanners().getScanners().length);
        assertEquals(1, shieldPlanet.getShields().getShields().length);
        assertEquals(1, weaponPlanet.getWeapons().getWeapons().length);
        
        assertEquals(tech_photon100, scannerPlanet.getScanners().getScanners()[0]);
        assertEquals(tech_energy10, shieldPlanet.getShields().getShields()[0]);
        assertEquals(tech_massDriver1, weaponPlanet.getWeapons().getWeapons()[0]);        
        assertEquals(tech_battleStationI, battleStationPlanet.getOrbital().getHullTech());
        
        //
        // test build projects do not work
        //
        
        try {        	
        	new SetProjectCmd(scannerPlanet, new ScannerDesign(tech_photon100), 0, player, false, false).run();
        	fail();
        } catch(ValidationError e) { }
        try {        	
            new SetProjectCmd(shieldPlanet, new ShieldDesign(tech_energy10), 0, player, false, false).run();
        	fail();
        } catch(ValidationError e) { }
        try {        	
        	new SetProjectCmd(battleStationPlanet, new BattleStationDesign(tech_battleStationI), 0, player, false, false).run();
        	fail();
        } catch(ValidationError e) { }
        
    }

    /**
     * Tests that moral choices made can effect production on a planet
     */
    public void testSetProjectOnAllIdleCommand() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testSetProjectOnIdleCommand() *************************");
        createMap(null, false);

        SolarSystem ss = newSolarSystem("SS", 50, 50);
        Planet pp1 = newPlanet("SS1-A", ss, player);
        Planet pp2 = newPlanet("SS1-B", ss, player);
        Planet pp3 = newPlanet("SS1-C", ss, player);
        Planet pp4 = newPlanet("SS1-D", ss, player);
        
        new SetProjectCmd(pp1, new ScannerDesign(tech_photon100), 0, player, true, false).run();
        pp1.addProduction((int)pp1.getProject().getCostToBuildOnPlanet());
        pp2.addProduction((int)pp2.getProject().getCostToBuildOnPlanet());
        pp3.addProduction((int)pp3.getProject().getCostToBuildOnPlanet());
        pp4.addProduction((int)pp4.getProject().getCostToBuildOnPlanet());
        pp1.setProject(null);
        pp2.setProject(null);
        pp3.setProject(null);
        pp4.setProject(null);
        new SetProjectCmd(pp2, new ShieldDesign(tech_energy10), 0, player, true, false).run();
        pp1.addProduction((int)pp1.getProject().getCostToBuildOnPlanet());
        pp2.addProduction((int)pp2.getProject().getCostToBuildOnPlanet());
        pp3.addProduction((int)pp3.getProject().getCostToBuildOnPlanet());
        pp4.addProduction((int)pp4.getProject().getCostToBuildOnPlanet());
        pp1.setProject(null);
        pp2.setProject(null);
        pp3.setProject(null);
        pp4.setProject(null);
        new SetProjectCmd(pp3, new WeaponDesign(tech_massDriver1), 0, player, true, false).run();
        pp1.addProduction((int)pp1.getProject().getCostToBuildOnPlanet());
        pp2.addProduction((int)pp2.getProject().getCostToBuildOnPlanet());
        pp3.addProduction((int)pp3.getProject().getCostToBuildOnPlanet());
        pp4.addProduction((int)pp4.getProject().getCostToBuildOnPlanet());
        pp1.setProject(null);
        pp2.setProject(null);
        pp3.setProject(null);
        pp4.setProject(null);
        new SetProjectCmd(pp4, new BattleStationDesign(tech_battleStationI), 0, player, true, false).run();
        pp1.addProduction((int)pp1.getProject().getCostToBuildOnPlanet());
        pp2.addProduction((int)pp2.getProject().getCostToBuildOnPlanet());
        pp3.addProduction((int)pp3.getProject().getCostToBuildOnPlanet());
        pp4.addProduction((int)pp4.getProject().getCostToBuildOnPlanet());
        
        
        assertEquals(tech_photon100, pp1.getScanners().getScanners()[0]);
        assertEquals(tech_photon100, pp2.getScanners().getScanners()[0]);
        assertEquals(tech_photon100, pp3.getScanners().getScanners()[0]);
        assertEquals(tech_photon100, pp4.getScanners().getScanners()[0]);
        assertEquals(tech_energy10, pp1.getShields().getShields()[0]);
        assertEquals(tech_energy10, pp2.getShields().getShields()[0]);
        assertEquals(tech_energy10, pp2.getShields().getShields()[0]);
        assertEquals(tech_energy10, pp4.getShields().getShields()[0]);
        assertEquals(tech_massDriver1, pp1.getWeapons().getWeapons()[0]);
        assertEquals(tech_massDriver1, pp2.getWeapons().getWeapons()[0]);
        assertEquals(tech_massDriver1, pp3.getWeapons().getWeapons()[0]);
        assertEquals(tech_massDriver1, pp4.getWeapons().getWeapons()[0]);
        assertEquals(tech_battleStationI, pp1.getOrbital().getHullTech());
        assertEquals(tech_battleStationI, pp2.getOrbital().getHullTech());
        assertEquals(tech_battleStationI, pp3.getOrbital().getHullTech());
        assertEquals(tech_battleStationI, pp4.getOrbital().getHullTech());
    }
    
    /**
     * Tests that moral choices made can effect production on a planet
     */
    public void testSetProjectOnAllIdleInvalidProjectsCommand() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testSetProjectOnIdleCommand() *************************");
        createMap(null, false);

        SolarSystem ss = newSolarSystem("SS", 50, 50);
        Planet pp1 = newPlanet("SS1-A", ss, player);
        Planet pp2 = newPlanet("SS1-B", ss, player);
        Planet pp3 = newPlanet("SS1-C", ss, player);
        Planet pp4 = newPlanet("SS1-D", ss, player);
        Planet pp5 = newPlanet("SS1-E", ss, player);
        Planet pp6 = newPlanet("SS1-E", ss, player);
        
        // Converts build mass driver 2 into upgrade to mass driver 2. 
        pp1.setScanners(new Scanners(tech_photon100));
        pp1.setShields(new Shields(tech_energy10));
        pp1.setWeapons(new Weapons(tech_massDriver1));
        pp1.setOrbital(new Orbital(tech_battleStationI, null, null));
        
        // Does not set project on planets with a project
        pp2.setProject(new ProjectScanner(new ScannerDesign(tech_photon100)));
        pp3.setProject(new ProjectShield(new ShieldDesign(tech_energy10)));
        pp4.setProject(new ProjectWeapon(new WeaponDesign(tech_massDriver1)));
        pp5.setProject(new ProjectBattleStation(new BattleStationDesign(tech_battleStationI)));
        
        // Does not set project on mining colony
        planet2.deploy(planet1, new Orbital(tech_spaceStation, null, null));
        

        //
        // Test Build Projects
        //
        
        planet1.setProject(null);
        new SetProjectCmd(planet1, new ScannerDesign(tech_photon200), 0, player, true, false).run();
        
        assertEquals(tech_photon200, ((ProjectScanner)planet1.getProject()).getScanner());
        assertEquals(null, planet2.getProject());
        assertEquals(null, pp1.getProject());
        assertEquals(tech_photon100, ((ProjectScanner)pp2.getProject()).getScanner());
        assertEquals(tech_energy10, ((ProjectShield)pp3.getProject()).getShield());
        assertEquals(tech_massDriver1, ((ProjectWeapon)pp4.getProject()).getWeapon());
        assertEquals(tech_battleStationI, ((ProjectBattleStation)pp5.getProject()).getBattleStation());
        assertEquals(tech_photon200, ((ProjectScanner)pp6.getProject()).getScanner());
        
        planet1.setProject(null);
        pp1.setProject(null);
        pp6.setProject(null);
        new SetProjectCmd(planet1, new ShieldDesign(tech_energy20), 0, player, true, false).run();
        
        assertEquals(tech_energy20, ((ProjectShield)planet1.getProject()).getShield());
        assertEquals(null, planet2.getProject());
        assertEquals(null, pp1.getProject());
        assertEquals(tech_photon100, ((ProjectScanner)pp2.getProject()).getScanner());
        assertEquals(tech_energy10, ((ProjectShield)pp3.getProject()).getShield());
        assertEquals(tech_massDriver1, ((ProjectWeapon)pp4.getProject()).getWeapon());
        assertEquals(tech_battleStationI, ((ProjectBattleStation)pp5.getProject()).getBattleStation());
        assertEquals(tech_energy20, ((ProjectShield)pp6.getProject()).getShield());
        
        planet1.setProject(null);
        pp1.setProject(null);
        pp6.setProject(null);
        new SetProjectCmd(planet1, new WeaponDesign(tech_massDriver2), 0, player, true, false).run();
        
        assertEquals(tech_massDriver2, ((ProjectWeapon)planet1.getProject()).getWeapon());
        assertEquals(null, planet2.getProject());
        assertEquals(null, pp1.getProject());
        assertEquals(tech_photon100, ((ProjectScanner)pp2.getProject()).getScanner());
        assertEquals(tech_energy10, ((ProjectShield)pp3.getProject()).getShield());
        assertEquals(tech_massDriver1, ((ProjectWeapon)pp4.getProject()).getWeapon());
        assertEquals(tech_battleStationI, ((ProjectBattleStation)pp5.getProject()).getBattleStation());
        assertEquals(tech_massDriver2, ((ProjectWeapon)pp6.getProject()).getWeapon());
        
        planet1.setProject(null);
        pp1.setProject(null);
        pp6.setProject(null);
        new SetProjectCmd(planet1, new BattleStationDesign(tech_battleStationII), 0, player, true, false).run();
        
        assertEquals(tech_battleStationII, ((ProjectBattleStation)planet1.getProject()).getBattleStation());
        assertEquals(null, planet2.getProject());
        assertEquals(null, pp1.getProject());
        assertEquals(tech_photon100, ((ProjectScanner)pp2.getProject()).getScanner());
        assertEquals(tech_energy10, ((ProjectShield)pp3.getProject()).getShield());
        assertEquals(tech_massDriver1, ((ProjectWeapon)pp4.getProject()).getWeapon());
        assertEquals(tech_battleStationI, ((ProjectBattleStation)pp5.getProject()).getBattleStation());
        assertEquals(tech_battleStationII, ((ProjectBattleStation)pp6.getProject()).getBattleStation());
        
        planet1.addProduction(3000);
        
        assertEquals(84, planet1.getTurnsToCompleteProject());
        assertEquals(334, pp6.getTurnsToCompleteProject());
        
    }

    private ArrayList<MotEventFactory> eventList = new ArrayList();
    
    /**
     * Tests that moral choices made can effect production on a planet
     */
    public void testTradeRoutes() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testTradeRoutes() *************************");
        createMap(null, false);
        
        SolarSystem tss1 = newSolarSystem("TS1", 99, 400);
        Planet tp1 = newPlanet("TS1-A", tss1, userPlayer, 110, 1.0, 110, 60);
        
        SolarSystem tss2 = newSolarSystem("TS2", 199, 400);
        Planet tp2 = newPlanet("TS2-A", tss2, player, 110, 1.0, 110, 60);
        
        SolarSystem tss3 = newSolarSystem("TS3", 299, 400);
        Planet tp3 = newPlanet("TS3-A", tss3, userPlayer, 110, 1.0, 110, 60);
        
        SolarSystem tss4 = newSolarSystem("TS4", 399, 400);
        Planet tp4 = newPlanet("TS4-A", tss4, userPlayer, 110, 1.0, 110, 60);
        
        MotEngine.instance.getProxyManager().add(new ClientProxy() {
			public void notify(MotEventFactory e) throws ApplicationError {	eventList.add(e);	}
			public void notify(MotEvent e) throws ApplicationError {	}
			public Player getOwner() {	return userPlayer;	}
		});
        
        tp1.setInfluence(110);
        tp2.setInfluence(1);
        tp3.setInfluence(110);
        tp4.setInfluence(20);

        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(100, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(200, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(300, 400));

        assertEquals(165.0, tp1.getIndustryWork(), 0.5);
        assertEquals(165.0, tp3.getIndustryWork(), 0.5);
        assertEquals(165.0, tp1.getCoreIndustryWork());
        assertEquals(165.0, tp2.getCoreIndustryWork());
        assertEquals(165.0, tp3.getCoreIndustryWork());
        
        TradeRouteFactory.create(tss1, tss3);

        assertEquals(173.0, tp1.getIndustryWork(), 0.5);
        assertEquals(173.0, tp3.getIndustryWork(), 0.5);
        assertEquals(165.0, tp1.getCoreIndustryWork());
        assertEquals(165.0, tp2.getCoreIndustryWork());
        assertEquals(165.0, tp3.getCoreIndustryWork());
                
        //
        // Test trade route cannot pass through sector not owned
        //
        
        eventList.clear();
        tp2.setInfluence(50);
        MotEngine.instance.getInfluence().update();
        
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(99, 400));
        assertEquals(player, MotEngine.instance.getInfluence().getOwnerByLocation(199, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(299, 400));
        
        MotEngine.instance.getTradeRoutes().endTurn();

        TradeRouteInvalidEF invalidEvent = (TradeRouteInvalidEF) eventList.get(0);
        
        assertEquals(tss1, invalidEvent.getSolarSystem1());
        assertEquals(tss3, invalidEvent.getSolarSystem2());

        assertEquals(165.0, tp1.getIndustryWork(), 0.5);
        assertEquals(165.0, tp3.getIndustryWork(), 0.5);
        assertEquals(165.0, tp1.getCoreIndustryWork());
        assertEquals(165.0, tp2.getCoreIndustryWork());
        assertEquals(165.0, tp3.getCoreIndustryWork());
        
        //
        // Test Blockade destroys trade route
        //
        
		tp2.setInfluence(1);
		MotEngine.instance.getInfluence().update();
		TradeRouteFactory.create(tss1, tss3);
		
		assertEquals(173.0, tp1.getIndustryWork(), 0.5);
		assertEquals(173.0, tp3.getIndustryWork(), 0.5);
		assertEquals(165.0, tp1.getCoreIndustryWork());
		assertEquals(165.0, tp2.getCoreIndustryWork());
		assertEquals(165.0, tp3.getCoreIndustryWork());

        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(99, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(199, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(299, 400));
        
        assertEquals(0, MotEngine.instance.getTradeRoutes().getTradeRoute(tss1, tss3).getBlockadeTurns());

        MotEngine.instance.getRelationsNotNull().setWar(player, userPlayer);
        Craft blockader = newCraft(fighterDesign, player, tp1);
        tp1.flushBlockadeCache();
        assertTrue(tp1.isBlockaded());
        
        eventList.clear();
        MotEngine.instance.getTradeRoutes().endTurn();
        
        assertEquals(1, MotEngine.instance.getTradeRoutes().getTradeRoute(tss1, tss3).getBlockadeTurns());
        
        MotEngine.instance.getTradeRoutes().endTurn();
        
        assertEquals(2, MotEngine.instance.getTradeRoutes().getTradeRoute(tss1, tss3).getBlockadeTurns());

        MotEngine.instance.getTradeRoutes().endTurn();

        invalidEvent = (TradeRouteInvalidEF) eventList.get(0);
        
        assertEquals(tss1, invalidEvent.getSolarSystem1());
        assertEquals(tss3, invalidEvent.getSolarSystem2());

        assertEquals(165.0, tp1.getIndustryWork(), 0.5);
        assertEquals(165.0, tp3.getIndustryWork(), 0.5);
        assertEquals(165.0, tp1.getCoreIndustryWork());
        assertEquals(165.0, tp2.getCoreIndustryWork());
        assertEquals(165.0, tp3.getCoreIndustryWork());
        
        blockader.destroy();
        tp1.flushBlockadeCache();
        assertFalse(tp1.isBlockaded());
   
        TradeRouteFactory.create(tss1, tss3);

        assertEquals(173.0, tp1.getIndustryWork(), 0.5);
        assertEquals(173.0, tp3.getIndustryWork(), 0.5);
        assertEquals(165.0, tp1.getCoreIndustryWork());
        assertEquals(165.0, tp2.getCoreIndustryWork());
        assertEquals(165.0, tp3.getCoreIndustryWork());

        //
        // Test planet changes owner in a solar system that has a trade route
        //
        
        eventList.clear();
        tp3.changeOwner(player);
        MotEngine.instance.getInfluence().update();
        MotEngine.instance.getTradeRoutes().endTurn();

        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(99, 400));
        assertEquals(player, MotEngine.instance.getInfluence().getOwnerByLocation(199, 400));
        assertEquals(player, MotEngine.instance.getInfluence().getOwnerByLocation(299, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(399, 400));
        
        assertEquals(165.0, tp1.getIndustryWork(), 0.5);
        assertEquals(165.0, tp2.getIndustryWork(), 0.5);
        assertEquals(165.0, tp3.getIndustryWork(), 0.5);
        assertEquals(165.0, tp4.getIndustryWork(), 0.5);
        assertEquals(165.0, tp1.getCoreIndustryWork());
        assertEquals(165.0, tp2.getCoreIndustryWork());
        assertEquals(165.0, tp3.getCoreIndustryWork());
        assertEquals(165.0, tp4.getCoreIndustryWork());
                 
        //
        // Test two trade routes at a solar system gives double bonus
        //

        tp3.changeOwner(userPlayer);
        MotEngine.instance.getInfluence().update();

        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(99, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(199, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(299, 400));
        assertEquals(userPlayer, MotEngine.instance.getInfluence().getOwnerByLocation(399, 400));

        TradeRoute tr1 = TradeRouteFactory.create(tss1, tss3);
        TradeRoute tr2 = TradeRouteFactory.create(tss3, tss4);
        
        assertEquals(173.0, tp1.getIndustryWork(), 0.5);
        assertEquals(181.5, tp3.getIndustryWork(), 0.5);
        assertEquals(173.0, tp4.getIndustryWork(), 0.5);
        assertEquals(165.0, tp1.getCoreIndustryWork());
        assertEquals(165.0, tp2.getCoreIndustryWork());
        assertEquals(165.0, tp3.getCoreIndustryWork());
        assertEquals(165.0, tp4.getCoreIndustryWork());
        
        assertEquals("TS1-A", "8", "TS3-A", "8", tr1.getIndustryOutput(userPlayer));
        assertEquals("TS3-A", "8", "TS4-A", "8", tr2.getIndustryOutput(userPlayer));
		        
		//
		// Test Mining planets contribute to GDP but do not get trade route bonuses
		//
		
        Planet tp1m = newPlanet("TS1-A", tss1, null, 40, 0.4, 0, 0);
        newSpaceStation(tp1, tp1m);

		assertEquals(tp1m, tp1.getMiningStation().getPlanet());

		assertEquals(173.0, tp1.getIndustryWork(), 0.5);
		assertEquals(181.5, tp3.getIndustryWork(), 0.5);
		assertEquals(173.0, tp4.getIndustryWork(), 0.5);
		assertEquals(165.0, tp1.getCoreIndustryWork());
		assertEquals(165.0, tp2.getCoreIndustryWork());
		assertEquals(165.0, tp3.getCoreIndustryWork());
		assertEquals(165.0, tp4.getCoreIndustryWork());
		
		assertEquals("TS1-A", "8", "TS3-A", "8", tr1.getIndustryOutput(userPlayer));
		assertEquals("TS3-A", "8", "TS4-A", "8", tr2.getIndustryOutput(userPlayer));

    }
    
    
    protected void tearDown() {
        
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

    
}