/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.Planets;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Player;

public class PlanetsTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(PlanetsTest.class);
        return suite;
    }
    
    public PlanetsTest(String name) {
        super(name);
    }
    
    private Player player;
    private Planet planet1, planet2, planet3, planet4, planet5;
	private SolarSystem ss1, ss2;
    
    private void createPlanets() {
        try {
        	String [] playerNames = new String [] { "Humans" };
        	SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
            RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
            player = newPlayer(playerNames[0], blue, options, playerNames);
                
            ss1 = newSolarSystem("SS1", 100, 100);
            ss2 = newSolarSystem("SS2", 200, 100);
    
            planet1 = newPlanet("SS1-A", ss1, null, 100, 1.0);
            planet2 = newPlanet("SS1-B", ss1, player);
            planet3 = newPlanet("SS2-A", ss2, null, 100, 0.8);
            planet4 = newPlanet("SS2-B", ss2, null, 100, 0.8);
            planet5 = newPlanet("SS2-C", ss2, player);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }
    
    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
    }
    
    public void testPlanetList() throws Exception {
        Logger.logDebug("*******************************  PlanetsTest.testPlanetList() *************************");
        createPlanets();

        Planets planets = MotEngine.instance.getPlanets();
        Planet[] pa = planets.getPlanets();
        
		assertEquals(5, planets.getList().size());
		assertEquals(planet1, pa[0]);
		assertEquals(planet2, pa[1]);
		assertEquals(planet3, pa[2]);
		assertEquals(planet4, pa[3]);
		assertEquals(planet5, pa[4]);
		
		assertEquals(planet1, planets.getPlanet("SS1-A"));
		assertEquals(planet4, planets.getPlanet("SS2-B"));
		assertEquals(null, planets.getPlanet("SS2-D"));
		
		Planet planet6 = newPlanet("SS2-D", ss2, null, 20, 0.1);
		
		assertEquals(6, planets.getList().size());
		assertEquals(planet6, planets.getPlanet("SS2-D"));
		
		Planet[] colonies = planets.getColonies();
		assertEquals(2, colonies.length);
		assertEquals(planet2, colonies[0]);
		assertEquals(planet5, colonies[1]);
		
		planet2.removeColony(true);
		
		colonies = planets.getColonies();
		assertEquals(1, colonies.length);
		assertEquals(planet5, colonies[0]);
		
        Craft coloniser = newCraft(coloniserDesign, player, ss1);
		planet1.colonise(coloniser);
		
		colonies = planets.getColonies();
		assertEquals(2, colonies.length);
		assertEquals(planet1, colonies[0]);
		assertEquals(planet5, colonies[1]);
    }

    public void testRandomPlanetSelection() throws Exception {
        Logger.logDebug("*******************************  PlanetsTest.testRandomPlanetSelection() *************************");
        createPlanets();
        
        Planets planets = MotEngine.instance.getPlanets();

        Utils.setNextRandomInt(0);
        Planet[] pa = planets.getRandomPlanets(2, 0.8, false);
		assertEquals(2, pa.length);
		assertEquals(planet3, pa[0]);
		assertEquals(planet4, pa[1]);


        Utils.setNextRandomInt(0);
        pa = planets.getRandomPlanets(8, 0.8, false);
		assertEquals(2, pa.length);
		assertEquals(planet3, pa[0]);
		assertEquals(planet4, pa[1]);
		
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);
        pa = planets.getRandomPlanets(8, 1.0, true);
		assertEquals(5, pa.length);
		assertEquals(planet1, pa[0]);
		assertEquals(planet2, pa[1]);
		assertEquals(planet3, pa[2]);
		assertEquals(planet4, pa[3]);
		assertEquals(planet5, pa[4]);
		
    }
    
    protected void tearDown() {
        
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

    
}