/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.common.bean.Location;
import mot.common.bean.Settings;
import mot.common.designs.Orbital;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.MotEvent;
import mot.common.event.StateContainer;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleStation;
import mot.engine.battles.HoloCraft;
import mot.engine.battles.HoloCraftFactory;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.ProjectCraft;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.players.generals.CraftTaskAttack;

/**
 * Tests the Money functionality. This tests;
 *  - Unused production produces money
 *  - Crafts and planetary shields, weapons, scanners have a maintenance cost
 *  - When a player runs out of money items are sold.
 */
public class PlayerTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(PlayerTest.class);
        return suite;
    }

    public PlayerTest(String name) {
        super(name);
    	try {
			Settings.getInstance();
		} catch (ApplicationError e) {
			e.printStackTrace();
		}
    }

    private SetupOptions options;

    private Player player1;
    private ComputerPlayer player2;
    private ComputerPlayer player3;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
    }

    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }
    
    
    public void createMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears", "Machines" };
        SetupOptions optionsGeneral = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);
        player3 = newPlayer(playerNames[2], red, optionsGeneral, playerNames);
        
        MotEngine.instance.setDate(3000);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);
        planet1.getStellarObject().addVisitor(player1, false);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);
        planet2.getStellarObject().addVisitor(player2, false);

        ss3 = newSolarSystem("SS3", 300, 100);
        planet3 = newPlanet("SS3-A", ss3, player3);
        planet3.getStellarObject().addVisitor(player3, false);
        

    }

    /**
     * Test .
     */
    public void testSurrender() throws Exception {
        Logger.logDebug("*******************************  MayorTest.testSurrender() *************************");
        createMap(null);

        Craft craft1 = newCraft(fighterDesign, player2, planet2);
        Craft craft2 = newCraft(fighterDesign, player2, planet2);
        Craft craft3 = newCraft(fighterDesign, player2, planet2);
        Craft craft4 = newCraft(fighterDesign, player2, planet2);
        
        CraftTaskAttack task = new CraftTaskAttack(planet3, player2, 5, null);
		player2.getGeneral().add(task);
		task.add(craft3);

		player2.addTechnology(tech_plague);
		
        assertEquals(planet2, craft3.getFleet().getOrbit().getPlanet());
        assertEquals(null, craft3.getFleet().getNavigator());
        
        Fleet fleet = new Fleet(player2, new Orbit(planet2));
        fleet.add(craft4);

        craft2.moveTo(new Orbit(planet3));
        craft2.endTurn();
        fleet.moveTo(new Orbit(planet3));
        fleet.endTurn();
        
        player2.surrendetTo(player3);
        
        assertEquals(0, player2.getQtyCrafts());
        assertEquals(0, player2.getCrafts().length);
        
        assertEquals(4, player3.getQtyCrafts());
        assertEquals(4, player3.getCrafts().length);

        assertEquals(craft1, player3.getCrafts()[0]);
        assertEquals(null, player3.getCrafts()[0].getFleet());
        assertEquals(planet2, player3.getCrafts()[0].getOrbit().getPlanet());
        assertEquals(null, player3.getCrafts()[0].getNavigator());

        assertEquals(craft2, player3.getCrafts()[1]);
        assertEquals(null, player3.getCrafts()[1].getFleet());
        assertEquals((Orbit)null, player3.getCrafts()[1].getOrbit());
        assertEquals(planet3, player3.getCrafts()[1].getNavigator().getDestination().getPlanet());

        assertEquals(craft3, player3.getCrafts()[2]);
        assertEquals(null, player3.getCrafts()[2].getFleet());
        assertEquals(planet2, player3.getCrafts()[2].getOrbit().getPlanet());
        assertEquals(null, player3.getCrafts()[2].getNavigator());

        assertEquals(craft4, player3.getCrafts()[3]);
        assertEquals(null, player3.getCrafts()[3].getFleet());
        assertEquals((Orbit)null, player3.getCrafts()[3].getOrbit());
        assertEquals(planet3, player3.getCrafts()[3].getNavigator().getDestination().getPlanet());
        
        assertEquals(0, player2.getPlanets().length);
        assertNull(player2.getFleet());
        assertEquals(0, player2.getCrafts().length);
        assertNull(player2.getGeneral().getTasks());

        assertEquals(planet2, player3.getPlanets()[0]);
        assertEquals(planet3, player3.getPlanets()[1]);

        assertEquals(-1, ss1.getLastVisitedDate(player3));
        assertEquals(3000, ss2.getLastVisitedDate(player3));
        assertEquals(3000, ss3.getLastVisitedDate(player3));
        
        assertTrue(player3.hasDiscovered(tech_plague));
    }

    /**
     * Tests 
     */
    public void testGameScoreData_CraftDestroyed() throws Exception {
        Logger.logDebug("*******************************  WarTest.testGameScoreData_CraftDestroyed() *************************");
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options1 = new RaceOptions(playerNames[0], null, optionsGeneral);
        UserPlayer userPlayer = newUserPlayer(playerNames[0], green, null, options1);
        player1 = userPlayer;
        RaceOptions options2 = new RaceOptions(playerNames[1], null, optionsGeneral);
        player2 = newPlayer(playerNames[1], blue, options2, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, userPlayer);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);
        planet2.setOrbital(new Orbital(tech_battleStationI, null, null));

        MotEngine.getInstance().getRelations().setWar(player2, userPlayer);
        
        Craft craft1 = newCraft(fighterDesign, userPlayer, planet2);
        Craft craft2 = newCraft(fighterDesign, player2, planet2);
        Craft craft3 = newCraft(fighterDesign, player2, planet2);

        Orbit orbit = planet2.getOrbit();
		Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(orbit, true);
        battle.join(planet2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.join(craft3, false);
        battle.setup();
        MotEngine.getInstance().getBattles().start(false);

        BattleCraft [] bca = battle.getCrafts();
        BattleStation bs = (BattleStation) bca[0];
        BattleCraft bc1 = bca[1];
        BattleCraft bc2 = bca[2];
        BattleCraft bc3 = bca[3];
		assertEquals("craft1 is bca[0]", craft1, bc1.getCraft());
        assertEquals("craft2 is bca[1]", craft2, bc2.getCraft());
        assertEquals("craft3 is bca[2]", craft3, bc3.getCraft());
        
        HoloCraft holoCraft1 = HoloCraftFactory._create(battle, bc2, false);        
        holoCraft1.setLocation(new Location(200, 700));
        holoCraft1.setDirection(270);
        MotEngine.instance._sendStateUpdateEvents(player1, true, true, true, true, false, false);
        battle.sendStateUpdateEvents(player1, true, true, true);

        assertEquals(craft2, holoCraft1.getParent());

        BattleCraft[] fa = bc2._launchFighters(battle, player2, orbit, player2.getFighterDesign(), 1, null, 0);
        BattleCraft fighter = fa[0];
        
        assertEquals(true, fighter.isFighter());
        
        bc2.setLocation(new Location(200, 300));
        bc3.setLocation(new Location(200, 400));
        bc2.setDirection(270);
        bc3.setDirection(270);
        fighter.setLocation(new Location(200, 600));
        fighter.setDirection(270);

        bc2.getCraft().setDamage(craft2.getMaxDamage()-0.1);
        bc3.getCraft().setDamage(craft3.getMaxDamage()-0.1);
        bs.getCraft().setDamage(bs.getCraft().getMaxDamage()-0.1);
        fighter.getCraft().setDamage(fighter.getCraft().getMaxDamage()-0.1);
        
        //
        // Hologram destroyed does not produce a score
        //
        assertEquals(5, battle.getQtyCraft(player2));

        eventList.clear();
        bc1.setLocation(new Location(250, 700));
        bc1.setDirection(270);
        bc1.rechargeWeapons();
        bc1.fireAllWeapons(holoCraft1);
        
        assertEquals(4, battle.getQtyCraft(player2));
        assertEquals(true, holoCraft1.isDestroyed());
        assertEquals(holoCraft1, ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getTarget());
        assertEquals(holoCraft1, ((BattleDestroyedEvent)getEvent(BattleDestroyedEvent.class)).getTarget());
        assertEquals(0, userPlayer.getGameScoreDataNotNull().getCraftDestroyedList().size());
        
        //
        // Destroying a craft produces a score
        //
        eventList.clear();
        bc1.setLocation(new Location(250, 300));
        bc1.setDirection(270);
        bc1.rechargeWeapons();
        bc1.fireAllWeapons(bc2);
        
        assertEquals(3, battle.getQtyCraft(player2));
        assertEquals(true, bc2.isDestroyed());
        assertEquals(bc2, ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getTarget());
        assertEquals(bc2, ((BattleDestroyedEvent)getEvent(BattleDestroyedEvent.class)).getTarget());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftDestroyedList().size());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftDestroyed(tech_flyer30).getQty());
        
        //
        // Destroying a second craft produces a score
        //
        
        eventList.clear();
        bc1.setLocation(new Location(250, 400));
        bc1.setDirection(270);
        bc1.rechargeWeapons();
        bc1.fireAllWeapons(bc3);
        
        assertEquals(2, battle.getQtyCraft(player2));
        assertEquals(true, holoCraft1.isDestroyed());
        assertEquals(bc3, ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getTarget());
        assertEquals(bc3, ((BattleDestroyedEvent)getEvent(BattleDestroyedEvent.class)).getTarget());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftDestroyedList().size());
        assertEquals(2, userPlayer.getGameScoreDataNotNull().getCraftDestroyed(tech_flyer30).getQty());
        
        //
        // Fighter destroyed does not produce a score
        //

        eventList.clear();
        bc1.setLocation(new Location(250, 600));
        bc1.setDirection(270);
        bc1.rechargeWeapons();
        bc1.fireAllWeapons(fighter);
        
        assertEquals(1, battle.getQtyCraft(player2));
        assertEquals(true, fighter.isDestroyed());
        assertEquals(fighter, ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getTarget());
        assertEquals(fighter, ((BattleDestroyedEvent)getEvent(BattleDestroyedEvent.class)).getTarget());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftDestroyedList().size());
        assertEquals(2, userPlayer.getGameScoreDataNotNull().getCraftDestroyed(tech_flyer30).getQty());
        
        //
        // BattleStation destroyed does not produce a score
        //

        eventList.clear();
        bc1.setLocation(new Location(250, 500));
        bc1.setDirection(270);
        bc1.rechargeWeapons();
        bc1.fireAllWeapons(bs);
        
        assertEquals(0, battle.getQtyCraft(player2));
        assertEquals(true, fighter.isDestroyed());
        assertEquals(bs, ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getTarget());
        assertEquals(bs, ((BattleDestroyedEvent)getEvent(BattleDestroyedEvent.class)).getTarget());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftDestroyedList().size());
        assertEquals(2, userPlayer.getGameScoreDataNotNull().getCraftDestroyed(tech_flyer30).getQty());
        

    }

    /**
     * Tests 
     */
    public void testGameScoreData_CraftInvaded() throws Exception {
        Logger.logDebug("*******************************  WarTest.testGameScoreData_CraftInvaded() *************************");
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options1 = new RaceOptions(playerNames[0], null, optionsGeneral);
        UserPlayer userPlayer = newUserPlayer(playerNames[0], green, null, options1);
        RaceOptions options2 = new RaceOptions(playerNames[1], null, optionsGeneral);
        player2 = newPlayer(playerNames[1], blue, options2, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, userPlayer);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(player2, userPlayer);
        
        Craft craft1 = newCraft(fighterDesign, userPlayer, planet2);
        Craft craft2 = newCraft(fighterDesign, player2, planet2);
        Craft craft3 = newCraft(fighterDesign, player2, planet2);

        Orbit orbit = planet2.getOrbit();
		Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(orbit, true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.join(craft3, false);
        battle.setup();

        BattleCraft [] bca = battle.getCrafts();
        BattleCraft bc1 = bca[0];
        BattleCraft bc2 = bca[1];
        BattleCraft bc3 = bca[2];
		assertEquals("craft1 is bca[0]", craft1, bc1.getCraft());
        assertEquals("craft2 is bca[1]", craft2, bc2.getCraft());
        assertEquals("craft3 is bca[2]", craft3, bc3.getCraft());
        
        bc2.setLocation(new Location(200, 300));
        bc3.setLocation(new Location(200, 400));
        bc2.setDirection(270);
        bc3.setDirection(270);

        bc2.endTurn();
        bc3.endTurn();
        
        //
        // Inoperable crafts at end of battle surrender to winner and get a score
        //
        
        bc2.setCrew(0);
        assertEquals(false, bc2.isOperating());
        
        //
        // Crafts that surrender during a battle get a score
        //
        eventList.clear();
        bc3.surrender(userPlayer, false);
        bc1.endTurn();
        battle.getNextToMove(true);
        
        assertEquals(true, battle.hasEnded());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftInvadedList().size());
        assertEquals(2, userPlayer.getGameScoreDataNotNull().getCraftInvaded(tech_flyer30).getQty());
        
    }

    /**
     * Tests 
     */
    public void testGameScoreData_CraftBuilt() throws Exception {
        Logger.logDebug("*******************************  WarTest.testGameScoreData_CraftBuilt() *************************");
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options1 = new RaceOptions(playerNames[0], null, optionsGeneral);
        UserPlayer userPlayer = newUserPlayer(playerNames[0], green, null, options1);
        RaceOptions options2 = new RaceOptions(playerNames[1], null, optionsGeneral);
        player2 = newPlayer(playerNames[1], blue, options2, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, userPlayer);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ProjectCraft project1 = new ProjectCraft(fighterDesign, planet1, false);
		planet1.setProject(project1);
		planet1.addProduction(project1.getCostToBuild()+1);
        
        assertEquals(1, userPlayer.getQtyCrafts());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftBuiltList().size());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftBuilt(tech_flyer30).getQty());

        ProjectCraft project2 = new ProjectCraft(fighterDesign, planet1, false);
		planet1.setProject(project2);
		planet1.addProduction(project2.getCostToBuild()+1);
        
        assertEquals(2, userPlayer.getQtyCrafts());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftBuiltList().size());
        assertEquals(2, userPlayer.getGameScoreDataNotNull().getCraftBuilt(tech_flyer30).getQty());

        ProjectCraft project3 = new ProjectCraft(powerfulDesign, planet1, false);
		planet1.setProject(project3);
		planet1.addProduction(project3.getCostToBuild()+1);
        
        assertEquals(3, userPlayer.getQtyCrafts());
        assertEquals(2, userPlayer.getGameScoreDataNotNull().getCraftBuiltList().size());
        assertEquals(2, userPlayer.getGameScoreDataNotNull().getCraftBuilt(tech_flyer30).getQty());
        assertEquals(1, userPlayer.getGameScoreDataNotNull().getCraftBuilt(tech_flyer100).getQty());
        
    }

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
		Logger.logDebug("Process Event "+e);
		if (e instanceof StateContainer) {
			DeligateCacheManager.update((StateContainer)e);
		}
		eventList.add(e);
    }

	public Player getOwner() {
        return player1;
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}