/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.designs.CraftDesign;
import mot.common.setup.SetupOptions;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.Battles;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.prizes.MoneyPrize;
import mot.engine.prizes.PrizeFactory;
import mot.engine.prizes.TechnologyPrize;
import mot.engine.setup.MonsterFactorySmall;

/**
 * Tests Prizes Work - When a craft arrives at a planet for the first time
 * it activates a prize for its owner. Future visitors do not get the prize.
 */
public class PrizeTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(PrizeTest.class);
        return suite;
    }

    public PrizeTest(String name) {
        super(name);
    }

    private SetupOptions options;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;

    private Craft craft;
    
    protected void setUp() throws Exception {
    	setUp(null);
    }
    
    protected void setUp(CraftDesign cd) throws Exception {
        Logger.setDebug();
        MotEngine.newInstance();
        setupConfigs();
        
        if (cd == null) cd = powerfulDesign;
        
        String [] playerNames = { "TestPlayer1", "TestPlayer2" };
        options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 120, 100);

        ss3 = newSolarSystem("SS3", 80, 100);
        planet3 = newPlanet("SS3-A", ss3, null, 100, 0.7);

        craft = newCraft(cd, player1, ss1);
    }
    

    /**
     * Test that Money prize works
     */
    public void testMoneyPrize() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testMoneyPrize() *************************");
        setUp();
        
        player1.setFunds(100);
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(1);
        PrizeFactory.createRandomPrize(ss2);
        craft.moveTo(ss2.getOrbit());
        craft.endTurn();
        
        assertEquals(ss2, craft.getOrbit().getStellarObject());
        assertEquals(600, player1.getFunds());
    }

    
    /**
     * Test that Money prize works
     */
    public void testProtectedPrizeSuccess() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testProtectedPrizeSuccess() *************************");
        setUp();
        
        Utils.setNextRandomInt(0);
        MonsterFactorySmall.create(planet3);
        player1.setFunds(100);
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(2);
        PrizeFactory.createRandomPrize(ss3);
        craft.moveTo(new Orbit(planet3));
        craft.endTurn();
        
        assertEquals(1, ss3.getPrizes().length);
        assertEquals(750, ((MoneyPrize)ss3.getPrizes()[0]).getAmount());
        
        assertEquals(1, MotEngine.getInstance().getMonsterPlayer().getQtyCrafts());
        assertEquals(ss3, craft.getOrbit().getStellarObject());
        assertEquals(100, player1.getFunds());
        
        MotEngine.getInstance().getMonsterPlayer().endTurn();
        Battles battles = MotEngine.getInstance().getBattles();
        Battle battle = battles.getBattles()[0];
        battles.start(true);
        
        assertEquals(0, MotEngine.getInstance().getMonsterPlayer().getQtyCrafts());
        assertEquals(true, battle.isWinner(player1));
        assertEquals(850, player1.getFunds());
    }

    /**
     * Test that Money prize works
     */
    public void testProtectedPrizeFail() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testProtectedPrizeFail() *************************");
        setUp(coloniserDesign);
        
        Utils.setNextRandomInt(0);
        MonsterFactorySmall.create(planet3);
        player1.setFunds(100);
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(2);
        PrizeFactory.createRandomPrize(ss3);
        craft.moveTo(new Orbit(planet3));
        craft.endTurn();
        
        assertEquals(-1, ss3.getLastVisitedDate(player1));
        assertEquals(1, ss3.getPrizes().length);
        assertEquals(750, ((MoneyPrize)ss3.getPrizes()[0]).getAmount());
        
        assertEquals(1, MotEngine.getInstance().getMonsterPlayer().getQtyCrafts());
        assertEquals(ss3, craft.getOrbit().getStellarObject());
        assertEquals(100, player1.getFunds());
        
        MotEngine.getInstance().getMonsterPlayer().endTurn();
        Battles battles = MotEngine.getInstance().getBattles();
        Battle battle = battles.getBattles()[0];
        battles.start(true);
        
        assertEquals(1, MotEngine.getInstance().getMonsterPlayer().getQtyCrafts());
        assertEquals(ss3, MotEngine.getInstance().getMonsterPlayer().getCrafts()[0].getOrbit().getStellarObject());
        assertEquals(false, battle.isWinner(player1));
        assertEquals(100, player1.getFunds());
        
        //
        // Check prize was not released but player1 has now visited this solar system.
        //

        assertEquals(1, ss3.getPrizes().length);
        assertEquals(750, ((MoneyPrize)ss3.getPrizes()[0]).getAmount());
        
        assertEquals(-1, ss3.getLastVisitedDate(player1));
    }

    /**
     * Test that Tech prize works
     */
    public void testTechnologyPrize() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testTechnologyPrize() *************************");
        setUp();
        
        assertEquals(false, player1.hasDiscovered(tech_fusion40));
        Utils.clearAndSetNextRandomInt(1); // create technology prize
        Utils.setNextRandomInt(0); // use technology at index 0
        PrizeFactory.createRandomPrize(ss2);
        assertEquals(tech_fusion40, ((TechnologyPrize)ss2.getPrizes()[0]).getTech());
        craft.moveTo(ss2.getOrbit());
        craft.endTurn();
        
        assertEquals(ss2, craft.getOrbit().getStellarObject());
        assertEquals(true, player1.hasDiscovered(tech_fusion40));
    }

    /**
     * Test that Boralium prize works
     */
    public void testBoraliumPrize() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testBoraliumPrize() *************************");
        setUp();
        
        Utils.setNextRandomInt(1);
        PrizeFactory.createBoraliumPrize(ss2);
        craft.moveTo(ss2.getOrbit());
        craft.endTurn();
        
        assertEquals(ss2, craft.getOrbit().getStellarObject());
        assertEquals(null, ss2.getPrizes());
        assertEquals(205, player1.getBoralium());
    }


    /**
     * Test that Tech prize works
     */
    public void testCraftPrize() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testCraftPrize() *************************");
        setUp();
        
        assertEquals(1, player1.getQtyCrafts());
        
        Utils.setNextRandomInt(2);
        PrizeFactory.createRandomPrize(ss2);
        craft.moveTo(ss2.getOrbit());
        craft.endTurn();
        
        assertEquals(ss2, craft.getOrbit().getStellarObject());
        assertEquals(2, player1.getQtyCrafts());
        assertEquals("Prize1", player1.getCrafts()[1].getName());
    }

    public void testRelic_CrystalineAugumentatedMetalsPrize() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testRelic_CrystalineAugumentatedMetalsPrize() *************************");
        setUp();
        
        assertEquals(0.0, player1.getPlayerCharacter().getMaxDamageBonus(), 0.001);
        
        Utils.setNextRandomInt(3);
        Utils.setNextRandomInt(0);
        PrizeFactory.createRandomPrize(ss2);
        craft.moveTo(ss2.getOrbit());
        craft.endTurn();
        
        assertEquals(ss2, craft.getOrbit().getStellarObject());
        assertEquals(0.1, player1.getPlayerCharacter().getMaxDamageBonus(), 0.001);
    }

    public void testRelic_ElectroliticAugumentationePrize() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testRelic_ElectroliticAugumentationePrize() *************************");
        setUp();
        
        assertEquals(0.0, player1.getPlayerCharacter().getSpeedBonus(), 0.001);
        assertEquals(0.0, player1.getPlayerCharacter().getBattleSpeedBonus(), 0.001);
        
        Utils.setNextRandomInt(3);
        Utils.setNextRandomInt(1);
        PrizeFactory.createRandomPrize(ss2);
        craft.moveTo(ss2.getOrbit());
        craft.endTurn();
        
        assertEquals(ss2, craft.getOrbit().getStellarObject());
        assertEquals(0.1, player1.getPlayerCharacter().getSpeedBonus(), 0.001);
        assertEquals(0.1, player1.getPlayerCharacter().getBattleSpeedBonus(), 0.001);
    }

    public void testRelic_BioGeneticTransformationPrize() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testRelic_BioGeneticTransformationPrize() *************************");
        setUp();
        
        assertEquals(0.0, player1.getPlayerCharacter().getMaxPopulationBonus(), 0.001);
        
        Utils.setNextRandomInt(3);
        Utils.setNextRandomInt(2);
        PrizeFactory.createRandomPrize(ss2);
        craft.moveTo(ss2.getOrbit());
        craft.endTurn();
        
        assertEquals(ss2, craft.getOrbit().getStellarObject());
        assertEquals(0.05, player1.getPlayerCharacter().getMaxPopulationBonus(), 0.001);
    }

    public void testRelic_BioConformityPrize() throws Exception {
        Logger.logDebug("*******************************  GeneralTest.testRelic_BioConformityPrize() *************************");
        setUp();
        
        assertEquals(0.0, player1.getPlayerCharacter().getSpyBonus(), 0.001);
        
        Utils.setNextRandomInt(3);
        Utils.setNextRandomInt(3);
        PrizeFactory.createRandomPrize(ss2);
        craft.moveTo(ss2.getOrbit());
        craft.endTurn();
        
        assertEquals(ss2, craft.getOrbit().getStellarObject());
        assertEquals(0.1, player1.getPlayerCharacter().getSpyBonus(), 0.001);
    }

    
    
    protected void tearDown() {
    	Logger.flushLog();
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}