/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.bean.NameList;
import mot.common.bean.PlayerAssets;
import mot.common.designs.Orbital;
import mot.common.event.DiplomaticRequestEvent;
import mot.common.event.DiplomaticResponseEvent;
import mot.common.event.MessageEvent;
import mot.common.event.MotEvent;
import mot.common.morals.MoralChoices;
import mot.common.proxy.ClientProxy;
import mot.common.resources.ResourceCloakPalm;
import mot.common.resources.Resources;
import mot.common.setup.OptionsFactory;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Technology;
import mot.common.technology.TestTechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.command.DiplomaticRequestCmd;
import mot.engine.command.DiplomaticResponseCmd;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.OrbitalReference;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.ResearchProject;
import mot.engine.players.strategists.StratagistBasic;
import mot.engine.players.strategists.StrategistEventBrokeNonAggression;
import mot.engine.players.strategists.StrategistEventDeclareWar;
import mot.engine.players.strategists.StrategistEventEmpireEstablished;
import mot.engine.players.strategists.StrategistEventGiveAssets;
import mot.engine.players.strategists.StrategistEventManager;
import mot.engine.players.strategists.StrategistEventMoralDecision;
import mot.engine.players.strategists.StrategistEventRelationsEstablished;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.relations.ResearchAgreementTreaty;
import mot.engine.relations.TradeAgreementTreaty;
import mot.engine.setup.MotFactory;

/**
 * Tests 
 *  - Diplomatic relations are formed correctly (when scanners can meet see Player).
 *  - Relationships move toward values defined in player character.
 *  - Computer players form trade routes with eligible players.
 *  - At the end of every turn the correct revenue is generated for each participant of a trade route
 *  - Trading effects the relationship
 *  - Gifts effects relationships
 */
public class RelationsTest extends TestBase implements ClientProxy {

    public static Test suite() {
    	Logger.clearContents();
        TestSuite suite = new TestSuite(mot.test.RelationsTest.class);
        return suite;
    }

    public RelationsTest(String name) {
        super(name);
    }

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;

    private Player player1;
    private ComputerPlayer player2;
    private ComputerPlayer player3;

    private Player proxyOwner;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    
    protected void setUp() throws Exception {
    	Logger.setLogLevelLoaded();
        Logger.setInMemory(false);
        Logger.setDebug();
        MotEngine.newInstance();
        setupConfigs();
    }

    public void createMap2() throws Exception {
    	createMap2(false);
    }
    
    public void createMap2(boolean player1IsUser) throws Exception {
        String [] races = {"Bears", "Birds"};
        MotFactory.setupRelations(null, races, null, null, false);
        
        if (player1IsUser) {
        	player1 = newUserPlayer(races[0], blue, TestFactory.getBearsOptions());
        } else {
        	player1 = newPlayer(races[0], blue, TestFactory.getBearsOptions(), races);
        }
        player2 = newPlayer(races[1], green, TestFactory.getBirdsOptions(), races);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 301, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 200, 100);
        planet3 = newPlanet("SS3-A", ss3, null, 80, 1.0);
        
//        if (player1 instanceof ComputerPlayer) {
//        	proxyOwner = (ComputerPlayer) player1;
//        }
    	proxyOwner = player1;
        MotEngine.getInstance().getProxyManager().add(this);
    }

    public void createMap3() throws Exception {
        String [] races = {"Bears", "Birds", "Humans"};
        MotFactory.setupRelations(null, races, null, null, false);
        
        player1 = newPlayer(races[0], blue, OptionsFactory.getRaceOptions("Bears"), races);
        player2 = newPlayer(races[1], green, OptionsFactory.getRaceOptions("Birds"), races);
        player3 = newPlayer(races[2], green, OptionsFactory.getRaceOptions("Humans"), races);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 301, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 200, 100);
        planet3 = newPlanet("SS3-A", ss3, player3);
    }

    /**
     * Tests that diplomatic relations are only formed when player can communicate.
     * Tests that relationship values tend to the defaults defined in the player character.
     */
    public void testRelations() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testRelations() *************************");
        createMap2(true);
        
        StrategistEventManager semP2 = ((ComputerPlayer)player2).getStratagist().getEventManagerNotNull();

        assertFalse("Too far away for players to have diplomatic relations", player1.hasDiplomaticRelationsWith(player2));
        assertFalse("Too far away for players to have diplomatic relations", player2.hasDiplomaticRelationsWith(player1));
        assertEquals("Has correct events", 3000, (int)((StrategistEventEmpireEstablished)semP2.getEventsAsArray()[0]).getDate());

        MotEngine.instance.setDate(3010);
        
        SolarSystem ss4 = newSolarSystem("SS4", 210, 120);
        Planet planet4 = newPlanet("SS4-A", ss4, player1);
        MotEngine.instance.getRelationsNotNull().updateHasDiplomaticRelations();
        assertTrue("Players have diplomatic relations", player1.hasDiplomaticRelationsWith(player2));
        assertTrue("Players have diplomatic relations", player2.hasDiplomaticRelationsWith(player1));

        assertEquals("Has correct events", 3000.0, ((StrategistEventEmpireEstablished)semP2.getEventsAsArray()[0]).getDate(), 0.99);
        assertEquals("Has correct events", 3010.0, ((StrategistEventRelationsEstablished)semP2.getEventsAsArray()[1]).getDate(), 0.99);
        assertEquals("Has correct events", true, ((StrategistEventRelationsEstablished)semP2.getEventsAsArray()[1]).isStarting());

        
        assertEquals("Has correct relationship", 50, MotEngine.instance.getRelations().getRelation(player1, player2));
        Relation rel = MotEngine.instance.getRelations().getRelationship(player1, player2);
        rel.setValue(60);
        
        // FINALLY TEST THAT RELATIONSHIPS MOVE BACK TO VALUES IN PLAYER CHARACTER
        MotEngine.instance.setDate(MotEngine.instance.getDate()+Relations.AGE_RELATIONS_INTERVAL);
        MotEngine.instance.getRelations().ageRelations();
        assertEquals("Relationships age", 59, MotEngine.instance.getRelations().getRelation(player1, player2));
        MotEngine.instance.getRelations().ageRelations();
        assertEquals("Relationships age only age every AGE_RELATIONS_INTERVAL turns", 59, MotEngine.instance.getRelations().getRelation(player1, player2));
        MotEngine.instance.setDate(MotEngine.instance.getDate()+Relations.AGE_RELATIONS_INTERVAL);
        MotEngine.instance.getRelations().ageRelations();
        assertEquals("Relationships age only age every AGE_RELATIONS_INTERVAL turns", 58, MotEngine.instance.getRelations().getRelation(player1, player2));
        
        MotEngine.instance.setDate(3020);
        planet4.removeColony(true);
      
        MotEngine.instance.getRelationsNotNull().updateHasDiplomaticRelations();
        assertFalse("Players have diplomatic relations", player1.hasDiplomaticRelationsWith(player2));
        assertFalse("Players have diplomatic relations", player2.hasDiplomaticRelationsWith(player1));

        assertEquals("Has correct events", 3000.0, ((StrategistEventEmpireEstablished)semP2.getEventsAsArray()[0]).getDate(), 0.99);
        assertEquals("Has correct events", 3010.0, ((StrategistEventRelationsEstablished)semP2.getEventsAsArray()[1]).getDate(), 0.99);
        assertEquals("Has correct events", true, ((StrategistEventRelationsEstablished)semP2.getEventsAsArray()[1]).isStarting());
        assertEquals("Has correct events", 3020.0, ((StrategistEventRelationsEstablished)semP2.getEventsAsArray()[2]).getDate(), 0.99);
        assertEquals("Has correct events", false, ((StrategistEventRelationsEstablished)semP2.getEventsAsArray()[2]).isStarting());

    }

    /**
     * Tests the interaction between treaties - 
     *  - Declaring removes Peace, Trade, Research, Non Aggression and alliance  treaties.
     *  - Declaring war in a player with an alliance also declares war on other alliance members.
     */
    public void testTreaties() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testRelations() *************************");
        createMap3();
        
        Relations rels = MotEngine.instance.getRelations();
        
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveWarTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveResearchTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveTradeTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveNonAgressionTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveAllianceTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHavePeaceTreaty());

        assertNull(rels.getRelationshipNotNull(player1, player2).getResearchAgreementTreaty());
        assertNull(rels.getRelationshipNotNull(player1, player2).getTradeAgreementTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isWar());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isNonAgression());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isAliance());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isPeace());
        
        assertEquals(false, rels.isWar(player1, player2));
        assertEquals(false, rels.isResearch(player1, player2));
        assertEquals(false, rels.isTrade(player1, player2));
        assertEquals(false, rels.isNonAgression(player1, player2));
        assertEquals(false, rels.isAliance(player1, player2));
        assertEquals(true, rels.isPeace(player1, player2));

        rels.setTrade(player1, player2);

        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveWarTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveResearchTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveTradeTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveNonAgressionTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveAllianceTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHavePeaceTreaty());

        assertNull(rels.getRelationshipNotNull(player1, player2).getResearchAgreementTreaty());
        assertEquals(3000, rels.getRelationshipNotNull(player1, player2).getTradeAgreementTreaty().getDateStarted());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isWar());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isNonAgression());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isAliance());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isPeace());
        
        rels.setNonAgression(player1, player2);

        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveWarTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveResearchTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveTradeTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveNonAgressionTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveAllianceTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHavePeaceTreaty());

        assertNull(rels.getRelationshipNotNull(player1, player2).getResearchAgreementTreaty());
        assertEquals(3000, rels.getRelationshipNotNull(player1, player2).getTradeAgreementTreaty().getDateStarted());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isWar());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).isNonAgression());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isAliance());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isPeace());
        
        rels.setAliance(player1, player2);

        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveWarTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveResearchTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveTradeTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveNonAgressionTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveAllianceTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHavePeaceTreaty());

        assertNull(rels.getRelationshipNotNull(player1, player2).getResearchAgreementTreaty());
        assertEquals(3000, rels.getRelationshipNotNull(player1, player2).getTradeAgreementTreaty().getDateStarted());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isWar());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isNonAgression());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).isAliance());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isPeace());
        
        assertEquals(false, rels.isWar(player1, player2));
        assertEquals(false, rels.isResearch(player1, player2));
        assertEquals(true, rels.isTrade(player1, player2));
        assertEquals(false, rels.isNonAgression(player1, player2));
        assertEquals(true, rels.isAliance(player1, player2));
        assertEquals(true, rels.isPeace(player1, player2));
        
        rels.setWar(player1, player2);

        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveWarTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveResearchTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveTradeTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveNonAgressionTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHaveAllianceTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHavePeaceTreaty());

        assertNull(rels.getRelationshipNotNull(player1, player2).getResearchAgreementTreaty());
        assertNull(rels.getRelationshipNotNull(player1, player2).getTradeAgreementTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).isWar());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isNonAgression());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isAliance());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isPeace());
        
        rels.setPeace(player1, player2);

        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveWarTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveResearchTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveTradeTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveNonAgressionTreaty());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).canHaveAllianceTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).canHavePeaceTreaty());

        assertNull(rels.getRelationshipNotNull(player1, player2).getResearchAgreementTreaty());
        assertNull(rels.getRelationshipNotNull(player1, player2).getTradeAgreementTreaty());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isWar());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isNonAgression());
        assertEquals(false, rels.getRelationshipNotNull(player1, player2).isAliance());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).isPeace());

        rels.setAliance(player3, player2);
        rels.setWar(player1, player2);

        assertEquals(true, rels.getRelationshipNotNull(player1, player2).isWar());
        assertEquals(true, rels.getRelationshipNotNull(player1, player3).isWar());

        assertEquals(true, rels.isWar(player1, player2));
        assertEquals(true, rels.isWar(player1, player3));
        
        //rels.setPeace(player1, player2);

        rels.setAliance(player3, player1);
        rels.setAliance(player3, player2);
        rels.setWar(player1, player2);

        assertEquals(true, rels.getRelationshipNotNull(player3, player1).isWar());
        assertEquals(false, rels.getRelationshipNotNull(player3, player2).isWar());
        assertEquals(true, rels.getRelationshipNotNull(player1, player2).isWar());

        assertEquals(true, rels.isWar(player3, player1));
        assertEquals(false, rels.isWar(player3, player2));
        assertEquals(true, rels.isWar(player1, player2));
    }    

    /**
     * Tests that ending a battle effects the relationship between two players.
     */
    public void testBattleWin() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testBattleWin() *************************");
        createMap2(true);

        ss2.setLocation(new Location(150,100));
        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(defenderDesign, player2, ss2);

        //((ComputerPlayer)player1).getGeneral().setQtyCraftsInAttackFleet(2);
        player1.diplomaticRequest(player2, DiplomaticProposal.getWar(), null);
        //player1.declareWar(player2, null, false);
        MotEngine.instance.getRelations().setRelation(player1, player2, 50);
        craft1.attack(ss2, player1.getEnemyCraftAt(ss2, true));

        // When a computer player wins a battle the relationship gets worse (player wants to beat enemy).
        // Create a mock battle and end battle Event such that player1 won the battle.
        // Player2 wins => relationship -10
        MotEngine.instance.endTurn(player1);
        MotEngine.instance.getBattles().getActiveBattle().automate();

        assertEquals(39, MotEngine.instance.getRelations().getRelation(player1, player2));
    }
    
    /**
     * Tests that ending a battle effects the relationship between two players.
     */
    public void testBattleLose() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testBattleLose() *************************");
        createMap2();

        ss2.setLocation(new Location(150,100));
        Craft craft1 = newCraft(defenderDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player2, ss2);

        ((ComputerPlayer)player1).getGeneral().setQtyCraftsInAttackFleet(2);
        player1.diplomaticRequest(player2, DiplomaticProposal.getWar(), null);
        //player1.declareWar(player2, null, false);
        MotEngine.instance.getRelations().setRelation(player1, player2, 5);

        // When a computer player wins a battle the relationship gets worse (player wants to beat enemy).
        // Create a mock battle and end battle Event such that player1 won the battle.
        // Player1 wins => relationship -10 & Player2 loses => relationship +5 (because both player 1 and 2 are computer players)
        // Relation moves 1 towards normal
        MotEngine.instance.endTurn(player1);

        assertEquals(1, MotEngine.instance.getRelations().getRelation(player1, player2));
    }
    
    /**
     * Tests that gifts effect relations correctly
     */
    public void testGifts() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testGifts() *************************");
        createMap2();
        
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        StrategistEventManager sem = ((ComputerPlayer)player1).getStratagist().getEventManagerNotNull();
    
        rel.setValue(50);
        player1.setFunds(0);
        player2.setFunds(0);
        
        //
        // TEST CAN ONLY GIVE MONEY THAT I HAVE
        //

        try {
        	new DiplomaticRequestCmd(player2, player1, DiplomaticProposal.gift(1000), null).run();
            //new GiveAssetsCmd(player2, player1, new PlayerAssets((Technology)null, 1000, null)).run();
            fail("I do not have any money - How can I give 1000");
        } catch (ValidationError e) { }

        assertEquals("Default relationship exists", 50, rel.getRelationshipValue());

        //
        // TEST GIVE MONEY TRANSFERS MONEY AND EFFECTS RELATIONS
        //

        rel.setValue(50);
        player2.setFunds(10000);
        player1.diplomaticRequest(player2, DiplomaticProposal.gift(100), null);
        //player2.giveTo(player1, new PlayerAssets(100));
        
        assertEquals("Default relationship exists", 50, rel.getRelationshipValue());
        assertEquals(0, ((StrategistEventGiveAssets)sem.getEventType(StrategistEventGiveAssets.class)).getRelationEffect());        
        assertEquals("Correct money exists", 100, player1.getFunds());
        
        sem.clearEvents();
        player1.diplomaticRequest(player2, DiplomaticProposal.gift(1000), null);
        //player2.giveTo(player1, new PlayerAssets(1000));
        
        assertEquals("Default relationship exists", 53, rel.getRelationshipValue());
        assertEquals(3, ((StrategistEventGiveAssets)sem.getEventType(StrategistEventGiveAssets.class)).getRelationEffect());        
        assertEquals("Correct money exists", 1100, player1.getFunds());
        assertEquals("Correct money exists", 8900, player2.getFunds());
        

        //
        // TEST CAN ONLY GIVE TECHNOLOGY THAT I HAVE AND THAT RECEPIENT DOES NOT HAVE
        //

        rel.setValue(50);
        sem.clearEvents();
        Technology tech = tech_girator;
        try {
        	new DiplomaticRequestCmd(player2, player1, DiplomaticProposal.gift(tech), null).run();
            //new GiveAssetsCmd(player2, player1, new PlayerAssets(tech, 0, null)).run();
            fail("I do not have this tech myself");
        } catch (ValidationError e) { }
        
        player1.addTechnology(tech);
        player2.addTechnology(tech);

        try {
        	new DiplomaticRequestCmd(player2, player1, DiplomaticProposal.gift(tech), null).run();
            //new GiveAssetsCmd(player2, player1, new PlayerAssets(tech, 0, null)).run();
            fail("Player1 already has this technology");
        } catch (ValidationError e) { }

        assertEquals("relationship has not changed", 50, rel.getRelationshipValue());
        
        //
        // TEST GIVE TECHNOLOGY ADDS TECHNOLOGY AND EFFECTS RELATIONS
        //

        rel.setValue(50);
        sem.clearEvents();
        tech = tech_galacticExchangeI;
        assertFalse("Player does not have tech", player1.hasDiscovered(tech));
        player1.diplomaticRequest(player2, DiplomaticProposal.gift(tech), null);
        //player2.giveTo(player1, new PlayerAssets(tech));
        
        assertEquals("Relationship is correct", 55, rel.getRelationshipValue());
        assertEquals(5, ((StrategistEventGiveAssets)sem.getEventType(StrategistEventGiveAssets.class)).getRelationEffect());        
        assertEquals("Correct money exists", 1100, player1.getFunds());
        assertTrue("Player now has new tech", player1.hasDiscovered(tech));


        //
        // TEST CAN ONLY GIVE PLANET  THAT I HAVE
        //

        try {
        	new DiplomaticRequestCmd(player2, player1, DiplomaticProposal.gift(planet3.getName()), null).run();
            //new GiveAssetsCmd(player2, player1, new PlayerAssets((Technology)null, 0, planet3.getName())).run();
            fail("I do not own this planet");
        } catch (ValidationError e) { }


        //
        // TEST GIVE PLANET TRANSFERS PLANET AND EFFECTS RELATIONS
        //

        rel.setValue(50);
        sem.clearEvents();
        planet3.setOwner(player2);
        player1.diplomaticRequest(player2, DiplomaticProposal.gift(planet3.getName()), null);
        //player2.giveTo(player1, new PlayerAssets(planet3.getName()));
        
        assertEquals("Relationship is correct", 60, rel.getRelationshipValue());
        assertEquals(10, ((StrategistEventGiveAssets)sem.getEventType(StrategistEventGiveAssets.class)).getRelationEffect());        
        assertEquals("Planet has changed hands", player1, planet3.getOwner());

        //
        // TEST GIVE CRAFT TRANSFERS PLANET AND EFFECTS RELATIONS
        //

        rel.setValue(50);
        sem.clearEvents();
        Craft craft1 = newCraft(fighterDesign, player2, planet2);
        player1.diplomaticRequest(player2, DiplomaticProposal.gift(new NameList(craft1.getName())), null);
        //player2.giveTo(player1, new PlayerAssets(new NameList(craft1.getName())));
        
        assertEquals(2, ((StrategistEventGiveAssets)sem.getEventType(StrategistEventGiveAssets.class)).getRelationEffect());        
        assertEquals("Relationship is correct", 52, rel.getRelationshipValue());
        assertEquals("Planet has changed hands", player1, craft1.getOwner());
        

    }

    /**
     * Tests that gifts effect relations correctly
     */
    public void testGivePlanetWithMiningStation() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testGivePlanetWithMiningStation() *************************");
        createMap2();
        player1.addScanner(tech_photon200);
        player2.addScanner(tech_photon200);
        
        MotEngine.instance.setDate(3010);
        
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        StrategistEventManager sem = ((ComputerPlayer)player1).getStratagist().getEventManagerNotNull();
        sem.clearEvents();
    
        rel.setValue(50);
        
        //
        // Make planet3 a mining colony owned by planet2
        //
        
        planet3.setResource(new ResourceCloakPalm());
        planet3.setOrbital(new Orbital(tech_spaceStation, null, null));
        planet2.setMiningStation(new OrbitalReference(planet3));
        
        assertEquals(-1, ss2.getLastVisitedDate(player1));
        assertEquals(-1, ss3.getLastVisitedDate(player1));
        
        assertEquals(26000, ((StratagistBasic)player2.getStratagist()).getAssetScore(new PlayerAssets(planet2.getName()), true));
        
        new DiplomaticRequestCmd(player2, player1, DiplomaticProposal.gift(planet2.getName()), null).run();
        //player2.giveTo(player1, new PlayerAssets(planet2.getName()));
        
        assertEquals(65, rel.getRelationshipValue());
        assertEquals(15, ((StrategistEventGiveAssets)sem.getEventsAsArray()[0]).getRelationEffect());        
        assertEquals(player1, planet2.getOwner());
        assertEquals(player1, planet3.getOwner());
        
        assertEquals(3010, ss2.getLastVisitedDate(player1));
        assertEquals(3010, ss3.getLastVisitedDate(player1));
        
    }

    /**
     * Tests that gifts effect relations correctly
     */
    public void testGiveCraftInFleetAndBattle() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testGiveCraftInFleetAndBattle() *************************");
        createMap2();
        
        MotEngine.instance.setDate(3010);
        
        Relations rels = MotEngine.instance.getRelations();
        rels.setWar(player1, player2);
        Relation rel = rels.getRelationship(player1, player2);
        StrategistEventManager sem = ((ComputerPlayer)player2).getStratagist().getEventManagerNotNull();
        sem.clearEvents();
        
        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player1, ss2);
        Craft craft3 = newCraft(fighterDesign, player2, ss2);
        Fleet fleet1 = FleetFactory.create(craft1);
        fleet1.add(craft2);
        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss2.getOrbit(), true);
        battle.join(fleet1, true);
        battle.join(craft3, false);
        
        rel.setValue(50);
        
        player2.diplomaticRequest(player1, DiplomaticProposal.gift(new NameList(craft1.getName())), null);
        
        assertEquals("Craft has changed owner", player2, craft1.getOwner());
        assertEquals("Craft has changed owner", player1, craft2.getOwner());
        assertEquals("Craft has changed owner", player2, craft3.getOwner());
        assertEquals("Craft has changed owner", null, craft1.getFleet());
        assertEquals("Craft has changed owner", fleet1, craft2.getFleet());
        assertEquals("Craft has changed owner", 1, fleet1.getCrafts().length);
        assertEquals("Craft has changed owner", craft2, fleet1.getCrafts()[0]);

        assertEquals("Battle loses given craft", false, battle.contains(craft1));
        assertEquals("Battle loses given craft", true, battle.contains(craft2));
        assertEquals("Battle loses given craft", true, battle.contains(craft3));
        assertEquals("Battle loses given craft", 1, MotEngine.instance.getBattles().getQtyBattles());

        assertEquals(2, ((StrategistEventGiveAssets)sem.getEventsAsArray()[0]).getRelationEffect());        
        assertEquals("Relationship is correct", 52, rel.getRelationshipValue());

        rel.setValue(50);
        sem.clearEvents();
        
        player2.diplomaticRequest(player1, DiplomaticProposal.gift(new NameList(craft2.getName())), null);
        //player1.giveTo(player2, new PlayerAssets(new NameList(craft2.getName())));
        
        assertEquals("Craft has changed owner", player2, craft1.getOwner());
        assertEquals("Craft has changed owner", player2, craft2.getOwner());
        assertEquals("Craft has changed owner", player2, craft3.getOwner());
        assertEquals("Craft has changed owner", null, craft2.getFleet());
        assertEquals("Craft has changed owner", null, craft2.getFleet());
        assertNull( player1.getFleet());

        assertEquals("Battle loses given craft", false, battle.contains(craft1));
        assertEquals("Battle loses given craft", false, battle.contains(craft2));
        assertEquals("Battle loses given craft", true, battle.hasEnded());
        assertEquals("Battle loses given craft", 0, MotEngine.instance.getBattles().getQtyBattles());

        assertEquals(1, ((StrategistEventGiveAssets)sem.getEventsAsArray()[0]).getRelationEffect());        
        assertEquals("Relationship is correct", 51, rel.getRelationshipValue());

    }
    
    /**
     * Tests that players can swap technologies
     *  * That cannot swap technology you do not have or the recipient already has
     *  * That Stratagist will not accept a bad deal
     */
    public void testSwapTechnology() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testSwapTechnology() *************************");
        
        createMap2(true);
        player2.getPlayerCharacter().setMissedTechnologies(new GeneralTechnologyList(tech_battleStationI));
        player1.addScanner(tech_photon150);
        
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        player1.setFunds(0);
    
        assertEquals("Default relationship exists", 50, rel.getRelationshipValue());
        
        Technology offeredTech = tech_girator;
        Technology wantedTech = tech_galacticExchangeI;
        
        //
        // TEST CAN ONLY SWAP MONEY, TECHS AND PLANETS THAT I HAVE
        //

        PlayerAssets offered = new PlayerAssets(offeredTech, 0, null);
        PlayerAssets wanted = new PlayerAssets(wantedTech, 0, null);
        player2.addTechnology(wantedTech);

        try {
        	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
            //new SwapCmd(player1, player2, offered, wanted).run();
            fail("Giver does not have this offered technology");
        } catch (ValidationError e) { }

        offered = new PlayerAssets((Technology)null, 10000, null);

        try {
        	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
            //new SwapCmd(player1, player2, offered, wanted).run();
            fail("Giver does not have this offered technology");
        } catch (ValidationError e) { }

        offered = new PlayerAssets((Technology)null, 100, planet2.getName());

        try {
        	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
            //new SwapCmd(player1, player2, offered, wanted).run();
            fail("Giver does not have this offered technology");
        } catch (ValidationError e) { }

        player2.getComputers().remove(wantedTech);
        player1.addTechnology(offeredTech);

        try {
        	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
            //new SwapCmd(player1, player2, offered, wanted).run();
            fail("Receiver does not have this wanted technology");
        } catch (ValidationError e) { }

        //
        // Test Offering technology a player already has is scored as worth 0
        //

        offered = new PlayerAssets(offeredTech, 0, null);
        wanted = new PlayerAssets(wantedTech, 0, null);
        
        player2.addTechnology(wantedTech);
        player2.addTechnology(offeredTech);
        lastEvent = null;
        
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
        //new SwapCmd(player1, player2, offered, wanted).run();

        assertEquals(0, ((StratagistBasic)player2.getStratagist()).getAssetScore(offered, true));
        assertEquals(250, ((StratagistBasic)player2.getStratagist()).getAssetScore(wanted, false));
        assertEquals("rejected", ((DiplomaticResponseEvent)lastEvent).getResponseVerb());

        player2.getWeapons().remove(offeredTech);
        player1.addTechnology(wantedTech);
        
        try {
        	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
            //new SwapCmd(player1, player2, offered, wanted).run();
            fail("Giver already has wanted technology");
        } catch (ValidationError e) { }
        
        //
        // Test that the swap command works
        //
        rel.setValue(Relations.NEUTRAL);
        lastEvent = null;
        player1.getComputers().remove(wantedTech);

        assertEquals(2500, ((StratagistBasic)player2.getStratagist()).getAssetScore(offered, true));
        assertEquals(250, ((StratagistBasic)player2.getStratagist()).getAssetScore(wanted, false));

    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
        //new SwapCmd(player1, player2, offered, wanted).run();
        
    	assertEquals("accepted", ((DiplomaticResponseEvent)lastEvent).getResponseVerb());
        assertTrue(player1.hasDiscovered(offeredTech));
        assertTrue(player1.hasDiscovered(wantedTech));
        assertTrue(player2.hasDiscovered(offeredTech));
        assertTrue(player2.hasDiscovered(wantedTech));

        //
        // Test techs score higher if in missing list
        //

        assertEquals(900, tech_battleStationI.getCostToResearch());
        assertEquals(1800, ((StratagistBasic)player2.getStratagist()).getAssetScore(new PlayerAssets(tech_battleStationI, 0, null), true));
        assertEquals(900, ((StratagistBasic)player2.getStratagist()).getAssetScore(new PlayerAssets(tech_battleStationI, 0, null), false));

        assertEquals(0, ((StratagistBasic)player2.getStratagist()).getAssetScore(new PlayerAssets(tech_massDriver1, 0, null), true));

        //
        // TEST STRATAGIST ONLY ACCEPTS OFFERS THAT ARE GOOD
        // (>=LOVE) ACCEPTS A DEAL IF RESEARCH COST OFFERED > RESEARCH COST WANTED 
        //                        OR RESEARCH COST OFFERED > RESEARCH COST WANTED * 0.85 (random chance)
        // (>=NEUTRAL) ACCEPTS A DEAL IF RESEARCH COST OFFERED > RESEARCH COST WANTED * 1.2
        //                        OR RESEARCH COST OFFERED > RESEARCH COST WANTED (random chance)
        // (OTHERWISE) ACCEPTS A DEAL IF RESEARCH COST OFFERED > RESEARCH COST WANTED * 1.5 (ramdom chance)
        //
        
        rel.setValue(Relations.NEUTRAL);
        
        offeredTech = tech_fusion25x;
        wantedTech = tech_detonator;
        assertEquals(25, offeredTech.getCostToResearch());
        assertEquals(700, wantedTech.getCostToResearch());
        player2.addTechnology(wantedTech);
        player1.addTechnology(offeredTech);
        assertEquals(25, ((StratagistBasic)player2.getStratagist()).getAssetScore(new PlayerAssets(offeredTech, 0, null), true));
        assertEquals(700, ((StratagistBasic)player2.getStratagist()).getAssetScore(new PlayerAssets(wantedTech, 0, null), false));
        
        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, false);

        offeredTech = tech_energy20;
        assertEquals(600, offeredTech.getCostToResearch());
        player1.addTechnology(offeredTech);
        
        assertEquals(600, ((StratagistBasic)player2.getStratagist()).getAssetScore(new PlayerAssets(offeredTech, 0, null), true));
        assertEquals(700, ((StratagistBasic)player2.getStratagist()).getAssetScore(new PlayerAssets(wantedTech, 0, null), false));

        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, false, 0.7, 0.5);

        // (>=LOVE) ACCEPTS A DEAL IF RESEARCH COST OFFERED > RESEARCH COST WANTED 
        //                        OR RESEARCH COST OFFERED > RESEARCH COST WANTED * 0.85 (random chance)
        
        rel.setValue(Relations.LOVE);
        player2.getStratagist().flushCache();

        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, true);
        
        // (>=NEUTRAL) ACCEPTS A DEAL IF RESEARCH COST OFFERED > RESEARCH COST WANTED * 1.2
        //                        OR RESEARCH COST OFFERED > RESEARCH COST WANTED (random chance)

        rel.setValue(Relations.NEUTRAL);
        player2.getStratagist().flushCache();
        offeredTech = tech_eneryAmplifierI;
        wantedTech = tech_photon200;
        assertEquals(900, offeredTech.getCostToResearch());
        assertEquals(1000, wantedTech.getCostToResearch());
        
        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, false, 0.7, 0.5);

        offeredTech = tech_galaxyNetI;
        assertEquals(1100, offeredTech.getCostToResearch());
        player1.addTechnology(offeredTech);
        
        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, false, 1.0, 0.5);
        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, true, 0.0, 0.5);

        // (<=DISLIKE) ACCEPTS A DEAL IF RESEARCH COST OFFERED > RESEARCH COST WANTED * 1.5 (ramdom chance)

        rel.setValue(Relations.HATE);
        player2.getStratagist().flushCache();

        offeredTech = tech_immobiliser;
        wantedTech = tech_fusionBeam4;
        assertEquals(1100, offeredTech.getCostToResearch());
        assertEquals(1100, wantedTech.getCostToResearch());
        
        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, false);

        offeredTech = tech_shieldDrain;
        assertEquals(2100, offeredTech.getCostToResearch());

        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, true, 0.05, 0.1);

    }
    
    /**
     * Tests that players can swap Craft
     */
    public void testSwapCraft() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testSwapTechnology() *************************");
        
        createMap2(true);
                
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        
        assertFalse(player1.hasDiplomaticRelationsWith(player2));
        
        player1.addScanner(tech_photon200);
        rels.endTurn();

        assertTrue(player1.hasDiplomaticRelationsWith(player2));

        rel.setValue(80);
        
        
        //
        // Offer craft for my missing technology
        //
        
        player1.getPlayerCharacter().setMissedTechnologies(new GeneralTechnologyList(tech_battleStationI));
        player2.addTechnology(tech_battleStationI);
        Craft craft1 = newCraft(powerfulDesign, player1, ss1);
        Craft craft2 = newCraft(powerfulDesign, player1, ss1);
        Craft craft3 = newCraft(powerfulDesign, player1, ss1);
        Fleet fleet1 = FleetFactory.create(craft1);
        fleet1.add(craft2);

        PlayerAssets offered = new PlayerAssets(new NameList(craft1.getName(), craft2.getName(), craft3.getName()));
        PlayerAssets wanted = new PlayerAssets(tech_battleStationI);

        clearEvents();
        
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
        //new SwapCmd(player1, player2, offered, wanted).run();

        assertEquals(2988, ((StratagistBasic)player2.getStratagist()).getAssetScore(offered, true));
        assertEquals(900, ((StratagistBasic)player2.getStratagist()).getAssetScore(wanted, false));
        assertEquals("accepted", ((DiplomaticResponseEvent)lastEvent).getResponseVerb());

        assertEquals("Craft has changed owner", player2, craft1.getOwner());
        assertEquals("Craft has changed owner", player2, craft2.getOwner());
        assertEquals("Craft has changed owner", player2, craft3.getOwner());
        assertEquals("Craft has changed owner", null, craft2.getFleet());
        assertEquals("Craft has changed owner", null, craft2.getFleet());
        assertEquals("Craft has changed owner", null, craft3.getFleet());
        assertNull(player1.getFleet());
        assertNull(player2.getFleet());

    }
    
    private void playerOneOfferSwapToPlayerTwo(Technology offeredTech, Technology wantedTech, boolean shouldAccept) throws Exception {
        playerOneOfferSwapToPlayerTwo(offeredTech, wantedTech, shouldAccept, null, null);
    }
    
    private void playerOneOfferSwapToPlayerTwo(Technology offeredTech, Technology wantedTech, boolean shouldAccept, Double nrd, Double nrdCheck) throws Exception {
        if (shouldAccept) {
            assertFalse(player1.hasDiscovered(wantedTech));
            assertFalse(player2.hasDiscovered(offeredTech));
        }
        
        player2.addTechnology(wantedTech);
        player1.addTechnology(offeredTech);
        PlayerAssets offered = new PlayerAssets(offeredTech);
        PlayerAssets wanted = new PlayerAssets(wantedTech);
        
        player2.getStratagist().getEventManagerNotNull().clearEvents();
        
        if (nrd != null) {
	        Utils.clearNextRandomProbability();
	        Utils.setNextRandomProbability(nrd, nrdCheck);
        }
        player2.diplomaticRequest(player1, new DiplomaticProposal(offered, wanted), null);
        //player2.requestSwapAssets(player1, offered, wanted);
        
        if (shouldAccept) {
            assertTrue(((DiplomaticResponseEvent)lastEvent).isAccept());
            assertTrue(player1.hasDiscovered(offeredTech));
            assertTrue(player1.hasDiscovered(wantedTech));
            assertTrue(player2.hasDiscovered(offeredTech));
            assertTrue(player2.hasDiscovered(wantedTech));
        } else {        
            assertFalse(((DiplomaticResponseEvent)lastEvent).isAccept());
        }
    }
    
    /**
     * Tests that players can swap technologies
     *  * That cannot swap technology you do not have or the recipient already has
     *  * That Strategist will not accept a bad deal
     */
    public void testSwapMultiple() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testSwapMultiple() *************************");

        createMap2(true);
        
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        player1.setFunds(0);
   
        assertEquals("Default relationship exists", 50, rel.getRelationshipValue());
        
        Technology tech1 = tech_fusionBeam4;
        Technology tech2 = tech_immobiliser;
        
        player1.addTechnology(tech1);
        player2.addTechnology(tech2);
        
        planet3.setOwner(player1);
        planet3.setPopulation(10);
        
        //
        // TEST CAN ONLY SWAP MONEY, TECHS AND PLANETS THAT I HAVE
        //

        PlayerAssets offered = new PlayerAssets(tech1, 0, planet3.getName());
        PlayerAssets wanted = new PlayerAssets(tech2, 1000, null);

        try {
        	new DiplomaticRequestCmd(player2, player1, new DiplomaticProposal(offered, wanted), null).run();
            //new SwapCmd(player2, player1, offered, wanted).run();
            fail("Player1 does not have enough money");
        } catch (ValidationError e) { }

        player1.setFunds(0);
        player2.setFunds(2000);

        try {
        	new DiplomaticRequestCmd(player2, player1, new DiplomaticProposal(offered, wanted), null).run();
            //new SwapCmd(player2, player1, offered, wanted).run();
            fail("Player2 does not own planet3");
        } catch (ValidationError e) { }

        Utils.clearAndSetNextRandomProbability(0.0);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
        //new SwapCmd(player1, player2, offered, wanted).run();
        
        assertTrue(((DiplomaticResponseEvent)lastEvent).isAccept());
        assertTrue(player1.hasDiscovered(tech1));
        assertTrue(player1.hasDiscovered(tech2));
        assertTrue(player2.hasDiscovered(tech1));
        assertTrue(player2.hasDiscovered(tech2));
        
        assertEquals(1000, player1.getFunds());
        assertEquals(1000, player2.getFunds());

        assertEquals(player2, planet3.getOwner());

        offered = new PlayerAssets(planet3.getName());
        wanted = new PlayerAssets(500);

        try {
        	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(offered, wanted), null).run();
            //new SwapCmd(player1, player2, offered, wanted).run();
            fail("Player1 does not own planet3");
        } catch (ValidationError e) { }
    }

    /**
     * Tests that players can make a demand for assets
     */
    public void testDemand() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testDemand() *************************");

        createMap2(true);
        player1.addScanner(tech_photon150);
        
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        player1.setFunds(55000);
        player2.setFunds(55000);
        player2.getStratagist().getEventManagerNotNull().clearEvents();
   
        assertEquals("Default relationship exists", 50, rel.getRelationshipValue());
        
        // CANNOT DEMAND A TECH THAT THE OTHER PLAYER DOES NOT HAVE
        try {
            PlayerAssets wanted = new PlayerAssets(tech_fusionBeam4, 0, null);
        	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
            //new DemandAssetsCmd(player1, player2, wanted).run();
            fail("Player2 does not have tech1");
        } catch (ValidationError e) { }

        player2.addTechnology(tech_fusionBeam4);
        player2.addTechnology(tech_immobiliser);
        player2.addTechnology(tech_shieldDrain);
        player2.addTechnology(tech_massDriver2);
        player1.addTechnology(tech_fusionBeam4);
        
        // CANNOT DEMAND A TECH THAT I HAVE PLAYER DOES NOT HAVE
        try {
            PlayerAssets wanted = new PlayerAssets(tech_fusionBeam4, 0, null);
        	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
            //new DemandAssetsCmd(player1, player2, wanted).run();
            fail("Player1 already has tech1");
        } catch (ValidationError e) { }

        player1.getWeapons().remove(tech_fusionBeam4);        

        // TEST REJECTS DEMANDS THAT I HAVE NO REASON TO ACCEPT
        lastEvent = null;
        rel.setValue(Relations.NEUTRAL);
        PlayerAssets wanted = new PlayerAssets(tech_fusionBeam4, 0, null);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
        //new DemandAssetsCmd(player1, player2, wanted).run();
        
        assertEquals(false, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(Relations.NEUTRAL, rel.getValue());
        assertEquals(-5, player2.getStratagist().getEventManager().getRelationshipEffect(player1));
        assertEquals("<html>Demand for assets (-5)<br/>Wanted: Technology FusionBeam4</html>", player2.getStratagist().getStrategistEvents(player1)[0].getMessage());

        // TEST ACCEPTS DEMANDS WHEN LIKE THE DEMANDER AND DEMAND IS SMALL (LESS THAN CURRENT RESEARCH PROJECT) 
        lastEvent = null;
        player2.getStratagist().getEventManager().clearEvents();
        player2.setResearchProject(new ResearchProject(tech_immobiliser, player2, false));
        rel.setValue(Relations.LIKE);
        wanted = new PlayerAssets(tech_fusionBeam4, 0, null);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
        //new DemandAssetsCmd(player1, player2, wanted).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(true, player1.hasDiscovered(tech_fusionBeam4));
        assertEquals(Relations.LIKE-5, rel.getValue());
        assertEquals(-5, player2.getStratagist().getEventManager().getRelationshipEffect(player1));

        // TEST REJECTS DEMANDS WHEN HAVE ALREADY CONCEDED TO AN DEMAND 
        lastEvent = null;
        rel.setValue(Relations.LIKE);
        wanted = new PlayerAssets(tech_massDriver2, 0, null);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
        //new DemandAssetsCmd(player1, player2, wanted).run();
        
        assertEquals(false, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(false, player1.hasDiscovered(tech_massDriver2));

        // TEST THAT IT WOULD HAVE ACCEPTED THE OFFER OBOVE IF IT HAD NOT JUST CONCEDED TO A DEMAND 
        lastEvent = null;
        player2.getStratagist().getEventManager().clearEvents();
        rel.setValue(Relations.LIKE);
        wanted = new PlayerAssets(tech_massDriver2, 0, null);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
        //new DemandAssetsCmd(player1, player2, wanted).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(true, player1.hasDiscovered(tech_massDriver2));

        // TEST THAT WOULD NOT ACCEPT DEMANDS THAT ARE TOO BIG 
        lastEvent = null;
        player2.getStratagist().getEventManager().clearEvents();
        player1.getWeapons().remove(tech_massDriver2);
        rel.setValue(Relations.LIKE);
        wanted = new PlayerAssets(tech_shieldDrain, 0, null);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
        //new DemandAssetsCmd(player1, player2, wanted).run();
        
        assertEquals(false, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(false, player1.hasDiscovered(tech_massDriver2));

        // TEST THAT WOULD ACCEPT BIG DEMANDS IF REALLY LOVE THE OTHER PLAYER 
        lastEvent = null;
        player2.getStratagist().getEventManager().clearEvents();
        rel.setValue(Relations.LOVE);
        wanted = new PlayerAssets(tech_shieldDrain, 0, null);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
        //new DemandAssetsCmd(player1, player2, wanted).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(true, player1.hasDiscovered(tech_shieldDrain));

        // TEST THAT WOULD ACCEPT DEMANDS IF OTHER PLAYER IS MUCH MORE POWERFUL THAN I 
        Craft craft1 = newCraft(defenderDesign, player1, ss1);
        Craft craft2 = newCraft(defenderDesign, player1, ss1);
        Craft craft3 = newCraft(defenderDesign, player1, ss1);
        lastEvent = null;
        player1.getWeapons().remove(tech_immobiliser);
        player2.getStratagist().getEventManager().clearEvents();
        rel.setValue(Relations.NEUTRAL);
        wanted = new PlayerAssets(tech_immobiliser, 0, null);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
        //new DemandAssetsCmd(player1, player2, wanted).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(true, player1.hasDiscovered(tech_immobiliser));

        // TEST THAT WOULD NOT ACCEPT DEMAND IF ITS TOO HIGH EVEN IF OTHER PLAYER IS POWERFUL
        lastEvent = null;
        player1.getWeapons().remove(tech_immobiliser);
        player2.getStratagist().getEventManager().clearEvents();
        rel.setValue(Relations.NEUTRAL);
        wanted = new PlayerAssets(tech_immobiliser, 50000, null);
    	new DiplomaticRequestCmd(player1, player2, new DiplomaticProposal(null, wanted), null).run();
        //new DemandAssetsCmd(player1, player2, wanted).run();
        
        assertEquals(false, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(false, player1.hasDiscovered(tech_immobiliser));
    }
    /**
     * Tests that players can respond to a demand for assets
     */
    public void testDemandResponses() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testDemandResponses() *************************");

        createMap2(true);
        player1.addScanner(tech_photon150);
        
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        player1.setFunds(55000);
        player2.setFunds(55000);
   
        assertEquals("Default relationship exists", 50, rel.getRelationshipValue());
        
        PlayerAssets assets = ((StratagistBasic)player2.getStratagist())._getAssetsToDemand(player1);
        player1.diplomaticRequest(player2, new DiplomaticProposal(null, assets), null);
		//player1.demandAssets(player2, assets);

        assertEquals(tech_photon150, ((DiplomaticRequestEvent)lastEvent).getProposal().getWanted().getTechnologList());
        
        lastEvent = null;
    	new DiplomaticResponseCmd(player1, player2, new DiplomaticProposal(null, assets), true, null).run();
        //new DemandAssetsResponseCmd(player1, player2, assets, true).run();
        
        assertTrue(player2.hasDiscovered(tech_photon150));
        assertNull(lastEvent);

        player1.addScanner(tech_photon200);
        assets = ((StratagistBasic)player2.getStratagist())._getAssetsToDemand(player1);
        player1.diplomaticRequest(player2, new DiplomaticProposal(null, assets), null);
		//player1.demandAssets(player2, assets);

        assertEquals(tech_photon200, ((DiplomaticRequestEvent)lastEvent).getProposal().getWanted().getTechnologList());
        
        lastEvent = null;
    	new DiplomaticResponseCmd(player1, player2, new DiplomaticProposal(null, assets), false, null).run();
        //new DemandAssetsResponseCmd(player1, player2, null, false).run();
        
        assertFalse(player2.hasDiscovered(tech_photon200));

        assertEquals(0, player2.getStratagist().getEventManager().getRelationshipEffect(player1));
    }
    
    /**
     * 
     */
    public void testComputerPlayersDeclareWarWhenDemandRejested() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testComputerPlayersDeclareWarWhenDemandRejested() *************************");
        createMap2(false);
        player2.getStratagist().getEventManagerNotNull().clearEvents();

        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        player1.setFunds(55000);
        player2.setFunds(55000);

        PlayerAssets wanted = new PlayerAssets(tech_fusionBeam4, 0, null);
    	player2.diplomaticRequest(player1, new DiplomaticProposal(null, wanted), null);
        
        assertEquals(Relations.NEUTRAL, rel.getValue());
        assertEquals(-10, player2.getStratagist().getEventManager().getRelationshipEffect(player1));
        assertEquals("<html>Demand for assets (-5)<br/>Wanted: Technology FusionBeam4</html>", player2.getStratagist().getStrategistEvents(player1)[0].getMessage());
        assertEquals("Declaration of war (-5)", player2.getStratagist().getStrategistEvents(player1)[1].getMessage());
    }
    
    /**
     * Tests that players form diplomatic relations once they are in range with one another
     */
    public void testDiplomaticRelations() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testDiplomaticRelations() *************************");

        createMap2();
        
        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        
        //
        // Relations are formed between these players if distance is less than 200
        //
        
        assertEquals(100, player1.getScanners().getBestScanner().getStrength());
        assertEquals(100, player2.getScanners().getBestScanner().getStrength());
        
        assertEquals(false, player1.hasDiplomaticRelationsWith(player2));
        
        //
        // Test that players only form diplomatic relations at the end of a turn
        //
        
        player1.addScanner(tech_photon100);
        
        assertEquals(false, player1.hasDiplomaticRelationsWith(player2));
        
        //
        // Test diplomatic relations are formed
        //

        player1.addScanner(tech_photon150);
        MotEngine.instance.getRelations().endTurn();
        
        assertEquals(true, player1.hasDiplomaticRelationsWith(player2));
        
    }

    /**
     * Tests moral choices do effect relations.
     */
    public void testMoralChoice() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testBattleWin() *************************");
        createMap2();

        Relations relations = MotEngine.instance.getRelations();
        player2.getStratagist().getEventManager().clearEvents();
        
		assertEquals(50, relations.getRelation(player1, player2));
        assertEquals(50, player2.getStratagist().getRating(player1, true));
        
        player1.add(MoralChoices.getMoralChoice(TEST_MORAL_CHOICE));

		assertEquals(50, relations.getRelation(player1, player2));
        assertEquals(30, player2.getStratagist().getRating(player1, true));
        
        StrategistEventManager eventManager = player2.getStratagist().getEventManager();
        
        StrategistEventMoralDecision semd = (StrategistEventMoralDecision) eventManager.getEventsAsArray()[0];
		assertEquals(player1, semd.getPlayer());
    }

    /**
     * Tests moral choices do effect relations.
     */
    public void testBreakTreaties() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testBreakTreaties() *************************");
        createMap2(true);
        player1.addScanner(tech_photon150);

        Relations relations = MotEngine.instance.getRelations();
        player2.getStratagist().getEventManager().clearEvents();
        
		assertEquals(50, relations.getRelation(player1, player2));
        assertEquals(50, player2.getStratagist().getRating(player1, true));

        relations.setNonAgression(player1, player2);
    	new DiplomaticRequestCmd(player1, player2, DiplomaticProposal.getWar(), null).run();
        //new RequestTreatyWarCmd(player1, player2, null).run();
        
        assertTrue(relations.isWar(player1, player2));
        assertEquals(2, player2.getStratagist().getEventManager().getEventsAsArray().length);
        assertEquals(StrategistEventBrokeNonAggression.class, player2.getStratagist().getEventManager().getEventsAsArray()[0].getClass());
        assertEquals(StrategistEventDeclareWar.class, player2.getStratagist().getEventManager().getEventsAsArray()[1].getClass());
        
        relations.setPeace(player1, player2);
        relations.setAliance(player1, player2);
        
        assertTrue(relations.isAliance(player1, player2));
        assertFalse(relations.isWar(player1, player2));
        assertEquals(2, player2.getStratagist().getEventManager().getEventsAsArray().length);
        assertEquals(StrategistEventBrokeNonAggression.class, player2.getStratagist().getEventManager().getEventsAsArray()[0].getClass());
        assertEquals(StrategistEventDeclareWar.class, player2.getStratagist().getEventManager().getEventsAsArray()[1].getClass());

    	new DiplomaticRequestCmd(player1, player2, DiplomaticProposal.getWar(), null).run();
        //new RequestTreatyWarCmd(player1, player2, null).run();

        assertFalse(relations.isAliance(player1, player2));
        assertTrue(relations.isWar(player1, player2));
        assertEquals(4, player2.getStratagist().getEventManager().getEventsAsArray().length);
        assertEquals(StrategistEventBrokeNonAggression.class, player2.getStratagist().getEventManager().getEventsAsArray()[0].getClass());
        assertEquals(StrategistEventDeclareWar.class, player2.getStratagist().getEventManager().getEventsAsArray()[1].getClass());
        assertEquals(StrategistEventBrokeNonAggression.class, player2.getStratagist().getEventManager().getEventsAsArray()[2].getClass());
        assertEquals(StrategistEventDeclareWar.class, player2.getStratagist().getEventManager().getEventsAsArray()[3].getClass());

        
    }
    
    public void testTradeTreaty() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testTradeTreaty() *************************");
        
        createMap2(true);
        planet1.setResource(Resources.antiMatter);
        planet2.setResource(Resources.cloakPalm);

        player1.addTech(tech_photon150); // Now have diplomatic relations

        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        
        rel.setValue( Relations.WOW );
        player1.setFunds(0);
        player2.setFunds(0);

        new DiplomaticRequestCmd(player1, player2, DiplomaticProposal.getTrade(), null).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(player1, rel.getTradeAgreementTreaty().getTrader1());
        assertEquals(player2, rel.getTradeAgreementTreaty().getTrader2());
        assertEquals(Resources.cloakPalm, rel.getTradeAgreementTreaty().getResource1());
        assertEquals(Resources.antiMatter, rel.getTradeAgreementTreaty().getResource2());
        
        //
        // Test Trade Treaty when no resources to swap (both players have the same resources).
        //

        Planet planet1b = newPlanet("SS1-B", ss1, player1);
        Planet planet2b = newPlanet("SS2-B", ss2, player2);
        planet1b.setResource(Resources.cloakPalm);
        planet2b.setResource(Resources.antiMatter);
        player1.flushCache();
        player2.flushCache();
        rel.removeTreaty(TradeAgreementTreaty.class);
        
        new DiplomaticRequestCmd(player1, player2, DiplomaticProposal.getTrade(), null).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(player1, rel.getTradeAgreementTreaty().getTrader1());
        assertEquals(player2, rel.getTradeAgreementTreaty().getTrader2());
        assertNull(rel.getTradeAgreementTreaty().getResource1());
        assertNull(rel.getTradeAgreementTreaty().getResource2());

    }
    
    
    public void testLoseResourceInTradeTreaty() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testLoseResourceInTradeTreaty() *************************");
        
        createMap2(true);
        Planet planet1b = newPlanet("SS1-B", ss1, player1);
        Planet planet2b = newPlanet("SS2-B", ss2, player2);
        planet1b.setResource(Resources.antiMatter);
        planet2b.setResource(Resources.cloakPalm);

        player1.addTech(tech_photon150); // Now have diplomatic relations

        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        
        rel.setValue( Relations.WOW );
        player1.setFunds(0);
        player2.setFunds(0);

        new DiplomaticRequestCmd(player1, player2, DiplomaticProposal.getTrade(), null).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(player1, rel.getTradeAgreementTreaty().getTrader1());
        assertEquals(player2, rel.getTradeAgreementTreaty().getTrader2());
        assertEquals(Resources.cloakPalm, rel.getTradeAgreementTreaty().getResource1());
        assertEquals(Resources.antiMatter, rel.getTradeAgreementTreaty().getResource2());
        
        //
        // Player 2 loses the colony containing the resource - so the trade treaty loses the resource as well.
        //

        planet2b.removeColony(true);
        player1.flushCache();
        player2.flushCache();
        clearEvents();
        rels.endTurn();
        
        assertEquals(player1, rel.getTradeAgreementTreaty().getTrader1());
        assertEquals(player2, rel.getTradeAgreementTreaty().getTrader2());
        assertNull(rel.getTradeAgreementTreaty().getResource1());
        assertEquals(Resources.antiMatter, rel.getTradeAgreementTreaty().getResource2());

        assertEquals("Trade Agreement with Birds lost resource Cloak Palm", ((MessageEvent)getEvent(MessageEvent.class)).getMessage());

    }
    

    public void testResearchTreaty() throws Exception {
        Logger.logDebug("*******************************  RelationsTest.testResearchTreaty() *************************");
        
        createMap2(true);
        setupOneTierTechTree();
        
        player1.addTech(tech_massDriver2, tech_fusionBeam4, tech_fusion25, tech_fusion25x, tech_trade_route, tech_mineField, tech_photon150, tech_armour25, tech_troopModule20, tech_ComputerisedLabI);
        player1.getPlayerCharacter().setMissedTechnologies(new TestTechnologyList(tech_ComputerisedLabI, tech_computerisedFactoryII, tech_energy10));
        assertEquals(tech_electronic25, tech_flyer100, player1.getTechnologyAvailableForResearch());
        
        player2.addTech(tech_massDriver2, tech_fusionBeam4, tech_fusion25, tech_fusion25x, tech_trade_route, tech_flyer100, tech_photon150, tech_armour25, tech_troopModule20, tech_ComputerisedLabI);
        player2.getPlayerCharacter().setMissedTechnologies(new TestTechnologyList(tech_ComputerisedLabI, tech_computerisedFactoryII, tech_electronic25));
        assertEquals(tech_energy10, tech_mineField, player2.getTechnologyAvailableForResearch());

        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationship(player1, player2);
        
        rel.setValue( Relations.WOW );
        player1.setFunds(0);
        player2.setFunds(0);

        Utils.clearNextRandomInt();
        
        new DiplomaticRequestCmd(player1, player2, DiplomaticProposal.getResearch(), null).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(player1, rel.getResearchAgreementTreaty().getResearcher1());
        assertEquals(player2, rel.getResearchAgreementTreaty().getResearcher2());
        assertEquals(tech_energy10,  rel.getResearchAgreementTreaty().getMissedTechnologies1());
        assertEquals(tech_electronic25, rel.getResearchAgreementTreaty().getMissedTechnologies2());
        assertEquals(tech_ComputerisedLabI, tech_computerisedFactoryII, player1.getPlayerCharacter().getMissedTechnologies());
        assertEquals(tech_ComputerisedLabI, tech_computerisedFactoryII, player2.getPlayerCharacter().getMissedTechnologies());
        assertEquals(tech_electronic25, tech_flyer100, tech_energy10, player1.getTechnologyAvailableForResearch());
        assertEquals(tech_electronic25, tech_energy10, tech_mineField, player2.getTechnologyAvailableForResearch());

        //
        // Test Research treaty when no missed techs available for research (now chooses any missed tech)
        //
        
        player1.addTechnology(tech_energy10);
        player2.addTechnology(tech_electronic25);
        player1.getPlayerCharacterBasic().addMissedTechnology(tech_timeWarpBeam);
        player2.getPlayerCharacterBasic().addMissedTechnology(tech_salvageBeam);
        
        rel.removeTreaty(ResearchAgreementTreaty.class);
        new DiplomaticRequestCmd(player1, player2, DiplomaticProposal.getResearch(), null).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(player1, rel.getResearchAgreementTreaty().getResearcher1());
        assertEquals(player2, rel.getResearchAgreementTreaty().getResearcher2());
        assertEquals(tech_timeWarpBeam,  rel.getResearchAgreementTreaty().getMissedTechnologies1());
        assertEquals(tech_salvageBeam, rel.getResearchAgreementTreaty().getMissedTechnologies2());
        assertEquals(tech_ComputerisedLabI, tech_computerisedFactoryII, player1.getPlayerCharacter().getMissedTechnologies());
        assertEquals(tech_ComputerisedLabI, tech_computerisedFactoryII, player2.getPlayerCharacter().getMissedTechnologies());
      

        //
        // Test Research treaty when no missed techs not discovered (nothing to choose - treaty has no techs)
        //
        
        rel.removeTreaty(ResearchAgreementTreaty.class);
        new DiplomaticRequestCmd(player1, player2, DiplomaticProposal.getResearch(), null).run();
        
        assertEquals(true, ((DiplomaticResponseEvent)lastEvent).isAccept());
        assertEquals(player1, rel.getResearchAgreementTreaty().getResearcher1());
        assertEquals(player2, rel.getResearchAgreementTreaty().getResearcher2());
        assertNull(rel.getResearchAgreementTreaty().getMissedTechnologies1());
        assertNull(rel.getResearchAgreementTreaty().getMissedTechnologies2());

    }
        
    Boolean treatyResponse = null;
    
    MotEvent lastEvent;

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
        lastEvent = e;
        if ( e instanceof DiplomaticResponseEvent) {
        	DiplomaticResponseEvent event = (DiplomaticResponseEvent)e;
            treatyResponse = new Boolean(event.isAccept());
        }
        eventList.add(e);
    }
    
	public Player getOwner() {
        return proxyOwner;
    }
    
    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
}