/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.event.MotEvent;
import mot.common.event.TechnologyDiscoveredEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.command.SetResearchProjectCmd;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.Banker;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.ResearchProject;
import mot.engine.players.Scientist;
import mot.engine.players.ScientistBasic;
import mot.engine.players.UserPlayer;
import mot.engine.relations.PeaceTreaty;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.relations.ResearchAgreementTreaty;
import mot.engine.relations.WarTreaty;
import mot.engine.setup.MotFactory;

public class ResearchTest extends TestBase implements ClientProxy {


    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        Logger.setDebug();
        TestSuite suite = new TestSuite(ResearchTest.class);
        return suite;
    }

    public ResearchTest(String name) {
        super(name);
    }

    protected void setUp() {
        try {
            MotEngine.newInstance();
            setupConfigs();
            setupOneTierTechTree(); // 
            Logger.setInMemory(false);
            Logger.setDebug();
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }

    private UserPlayer userPlayer;
    private Player player1;
    private Player player2;
	private Planet planet1;
	private Planet planet2;
	private Planet planet3;
	private SolarSystem ss1;
	private SolarSystem ss2;
	private SolarSystem ss3;
	
	private Player proxyOwner;
	private MotEvent lastEvent;

    private void setupGame() throws Exception {
        SetupOptions options = OptionsFactory.getGeneralOptions();
        String [] playerNames = {"TestPlayer1", "TestPlayer2"};
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);
    }
    
    private void setupGame(String playerName1, String playerName2) throws Exception {
        SetupOptions options = OptionsFactory.getGeneralOptions();
        String [] playerNames = {playerName1, playerName2};
        player1 = newPlayer(playerNames[0], blue, OptionsFactory.getRaceOptions(playerName1), playerNames);
        player2 = newPlayer(playerNames[1], green, OptionsFactory.getRaceOptions(playerName2), playerNames);
        
        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 301, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 200, 100);
        planet3 = newPlanet("SS3-A", ss3, null, 80, 1.0);

        proxyOwner = player1;
        MotEngine.getInstance().getProxyManager().add(this);
        MotEngine.getInstance().setDate(MotEngine.START_DATE+Banker.CALCULATE_START+1);
    }
    
    private void setupUserPlayerGame(String playerName1, String playerName2) throws Exception {
        SetupOptions options = OptionsFactory.getGeneralOptions();
        String [] playerNames = {playerName1, playerName2};
        player1 = userPlayer = newUserPlayer(playerName1, blue, null, OptionsFactory.getRaceOptions(playerName1));
        player2 = newPlayer(playerNames[1], green, OptionsFactory.getRaceOptions(playerName2), playerNames);
        
        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 301, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 200, 100);
        planet3 = newPlanet("SS3-A", ss3, null, 80, 1.0);

        proxyOwner = player1;
        MotEngine.getInstance().getProxyManager().add(this);
        MotEngine.getInstance().setDate(MotEngine.START_DATE+Banker.CALCULATE_START+1);
    }
    
    private void setupUserPlayerGame(SetupOptions optionsRace) throws Exception {
    	String [] playerNames = {"TestPlayer1", "TestPlayer2"};
    	SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
    	RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        userPlayer = newUserPlayer(playerNames[0], blue, null, options);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);
    }

    public void createMap() throws Exception {
        String [] races = {"Bears", "Birds"};
        MotFactory.setupRelations(null, races, null, null, false);

        player1 = newPlayer(races[0], blue, TestFactory.getBearsOptions(), races);
        player2 = newPlayer(races[1], green, TestFactory.getBirdsOptions(), races);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 301, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 200, 100);
        planet3 = newPlanet("SS3-A", ss3, null, 80, 1.0);

        proxyOwner = player1;
        MotEngine.getInstance().getProxyManager().add(this);
        MotEngine.getInstance().setDate(MotEngine.START_DATE+Banker.CALCULATE_START+1);
    }

    
    /**
     * Tests that a player can research new technology until no technology is left
     * and that the order of tech research is correct.
     */
    public void testDiscovery() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testDiscovery() *************************");
        setupGame();
        
		assertEquals(6, tech_fusion25.getCostToResearch());
		assertEquals(7, tech_mineField.getCostToResearch());
		assertEquals(9, tech_electronic25.getCostToResearch());
		assertEquals(25, tech_fusion25x.getCostToResearch());
		
		
		//
        // Test this player sets up a research project when research is performed
        // Tech being researched should be cheapest becuase computer always chooses the cheapest
        // available.
        //
        
		player1.getPlayerCharacter().getMissedTechnologies().removeAll();
		player1.getHulls().removeAll();
		player1.getEngines().removeAll();
		player1.getSpecials().removeAll();
		player1.setComputers(null);
		
        player1.addResearch(1);
        ResearchProject rp = player1.getResearchProject();

        assertTrue(player1.hasResearchProject());
		assertFalse(player1.hasDiscovered(rp.getTechnology()));
		assertEquals(tech_fusion25, rp.getTechnology());

		//
		// Test that when enough effort is put in the tech is discovered and a new
		// project is started
		//
		
        assertFalse(player1.hasDiscovered(tech_fusion25));

        player1.addResearch(7);
        
        assertTrue(player1.hasDiscovered(tech_fusion25));
        assertTrue(player1.hasResearchProject());
        
        rp = player1.getResearchProject();
        
		assertFalse(player1.hasDiscovered(rp.getTechnology()));
		assertEquals(tech_mineField, rp.getTechnology());
		
		//
		// Test adding lots of effort can flow over to next project
		//
		
        assertFalse(player1.hasDiscovered(tech_mineField));
        assertFalse(player1.hasDiscovered(tech_electronic25));
        
        player1.addResearch(18);
		
        assertTrue(player1.hasDiscovered(tech_fusion25));
        assertTrue(player1.hasDiscovered(tech_mineField));
        assertTrue(player1.hasDiscovered(tech_electronic25));
    }

    public void testResearchPriority() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testResearchPriority() *************************");
        setupGame("Humans", "Birds");
        setupOneTierTechTree();

        //player1.getMissedTechnologies().removeAll();
        
        //
        // These are all techs in tech tree
        //
		assertEquals(6, tech_fusion25.getCostToResearch());
		assertEquals(7, tech_mineField.getCostToResearch());
		assertEquals(9, tech_electronic25.getCostToResearch());
		assertEquals(10, tech_photon100.getCostToResearch());
		assertEquals(25, tech_fusion25x.getCostToResearch());
		assertEquals(50, tech_energy10.getCostToResearch());
		assertEquals(50, tech_ComputerisedLabI.getCostToResearch());
		assertEquals(100, tech_troopModule20.getCostToResearch());
		assertEquals(250, tech_flyer100.getCostToResearch());
		assertEquals(250, tech_massDriver2.getCostToResearch());
		assertEquals(500, tech_computerisedFactoryII.getCostToResearch());
		assertEquals(600, tech_armour25.getCostToResearch());
		assertEquals(1100, tech_fusionBeam4.getCostToResearch());

        assertTrue(player1.hasDiscovered(tech_fusion25));
        assertTrue(player1.hasDiscovered(tech_photon100));
        assertTrue(player1.hasDiscovered(tech_basicComputerisedFactory));

		ComputerPlayer cp = (ComputerPlayer) player1;
		
		ResearchProject rp = cp.getScientist().getNextResearchProject(null, cp);
		assertEquals(tech_mineField, rp.getTechnology());
		assertEquals(0.0, rp.getFromSalvageProbability());

		cp.addTechnology(tech_mineField);

		rp = cp.getScientist().getNextResearchProject(null, cp);
		assertEquals(tech_electronic25, rp.getTechnology());
		assertEquals(0.0, rp.getFromSalvageProbability());

		cp.addSalvage(tech_fusionBeam4);
		
		rp = cp.getScientist().getNextResearchProject(null, cp);
		assertEquals(tech_fusionBeam4, rp.getTechnology());
		assertEquals(0.003, rp.getFromSalvageProbability(), 0.001);

		cp.addTechnology(tech_fusionBeam4);
		
		rp = cp.getScientist().getNextResearchProject(null, cp);
		assertEquals(tech_electronic25, rp.getTechnology());
		assertEquals(0.0, rp.getFromSalvageProbability());

    }


    /**
     * Tests that research a planet produces goes to the players research project AND
     * new techs become available to a player once that required work has been done.
     */
    public void testPlanetResearch() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testPlanetResearch() *************************");
        setupGame();
        ComputerPlayer p = (ComputerPlayer) player2;

        assertTrue(p.getSpecials().contains(tech_colonyModule8));
        assertTrue(p.getEngines().contains(tech_fusion25));

        // Test that a planet ads its research values to the players research project.
        SolarSystem ss = newSolarSystem("SS1", 100, 100);
        Planet planet = newPlanet("SS1-A", ss, player2);
        planet.setPopulation(100.0);

        player2.setResearchProject(new ResearchProject(tech_massDriver2, player2, false));

        assertEquals("No research Done", 0, p.getResearchProject().getWork());
        planet.endTurn();
        assertTrue("Research Done", p.getResearchProject().getWork() > 0);

    }

    /**
     * Tests that research a planet produces goes to the players research project AND
     * new techs become available to a player once that required work has been done.
     */
    public void testCanDiscoverAllTypesOfTech() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testCanDiscoverAllTypesOfTech() *************************");
        setupGame();
        ComputerPlayer p = (ComputerPlayer) player2;

        assertEquals("Has Correct Research Cost", 250, tech_massDriver2.getCostToResearch());

        player2.setResearchProject(new ResearchProject(tech_massDriver2, player2, false));
        player2.addResearch(200);
        
        assertEquals("Correct research Done", 200, p.getResearchProject().getWork());
        assertFalse("Have technology", p.getWeapons().contains(tech_massDriver2));
        
        player2.addResearch(53);
        
        assertEquals("Correct research Done", 3, p.getResearchProject().getWork());
        assertTrue("Have technology", p.hasDiscovered(tech_massDriver2));
        assertTrue("Have technology", p.getWeapons().contains(tech_massDriver2.getName()));

        player2.addTechnology(tech_fusion25x);
        assertTrue("Have technology", p.hasDiscovered(tech_fusion25x));

        player2.addTechnology(tech_electronic40);
        assertTrue("Have technology", p.hasDiscovered(tech_electronic40));

        player2.addTechnology(tech_flyer100);
        assertTrue("Have technology", p.hasDiscovered(tech_flyer100));

        player2.addTechnology(tech_colonyModule7);
        assertTrue("Have technology", p.hasDiscovered(tech_colonyModule7));

        player2.addTechnology(tech_shieldModulatorI);
        assertTrue("Have technology", p.hasDiscovered(tech_shieldModulatorI));

        player2.addTechnology(tech_photon150);
        assertTrue("Have technology", p.hasDiscovered(tech_photon150));

        player2.addTechnology(tech_energy10);
        assertTrue("Have technology", p.hasDiscovered(tech_energy10));
       
    }

    /**
     * Tests that new technologies discovered are used by the computer player to upgrade
     * their planets.
     */
    public void testAutomaticUpgradeOfScanners() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testAutomaticUpgradeOfScanners() *************************");
        setupGame();
        ComputerPlayer p = (ComputerPlayer) player2;
        SolarSystem ss1 = newSolarSystem("SS1", 100, 100);
        Planet planet = newPlanet("SS1-A", ss1, p);

        assertNull(planet.getScanners());
        ResearchProject rp = new ResearchProject(tech_photon150, p, false);
        p.setResearchProject(rp);
        rp.addResearch(101);
        assertEquals(1, planet.getScanners().getQuantity());
        assertEquals(true, planet.getScanners().contains(tech_photon150));

        assertEquals(false, planet.getScanners().contains(tech_photon200));
        rp = new ResearchProject(tech_photon200, p, false);
        p.setResearchProject(rp);
        rp.addResearch(1001);
        assertEquals(1, planet.getScanners().getQuantity());
        assertEquals(true, planet.getScanners().contains(tech_photon200));

    }

    /**
     * Tests that new technologies discovered are used by the computer player to upgrade
     * their planets.
     */
    public void testDiscoverTechsFromSalvage() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testDiscoverTechsFromSalvage() *************************");
        setupUserPlayerGame("Humans", "Bears");
        
        // Humans are resourceful so they also discover techs
        assertEquals(1.0, player1.getPlayerCharacter().getChanceToDiscoverSalvagedTech());
        assertEquals(0.5, player2.getPlayerCharacter().getChanceToDiscoverSalvagedTech());
        
        assertFalse(player1.hasDiscovered(tech_photon200));
        assertFalse(player1.hasDiscovered(tech_shieldDrain));
        
        try {
        	new SetResearchProjectCmd(tech_photon200, player1).run();
        	fail();
        } catch (ValidationError e) { }
        
        //
        // Test successful discover tech from salvage
        //
        
        player1.addSalvage(tech_photon200);
        player1.addSalvage(tech_shieldDrain);
        
       	new SetResearchProjectCmd(tech_photon200, player1).run();
        
       	assertFalse(player1.hasDiscovered(tech_photon200));
        assertFalse(player1.hasDiscovered(tech_shieldDrain));
        assertFalse(player1.getSalvage().contains(tech_photon200));
        assertTrue(player1.getSalvage().contains(tech_shieldDrain));
        
        assertEquals(10, player1.getTotalResearch());
        assertEquals(3000, player1.getResearchProject().getCostToResearch());
        assertEquals(0.003, player1.getResearchProject().getFromSalvageProbability(), 0.001);

        Utils.clearNextRandomProbability();
        Utils.setNextRandomProbability(0.51);

        player1.getResearchProject().addResearch(0);
        player1.getResearchProject().endTurn();

        Utils.clearNextRandomProbability();
        Utils.setNextRandomProbability(0.001);

        player1.getResearchProject().addResearch(0);
        player1.getResearchProject().endTurn();

        assertTrue(player1.hasDiscovered(tech_photon200));
        assertFalse(player1.hasDiscovered(tech_shieldDrain));
        
        assertEquals(TechnologyDiscoveredEvent.class, lastEvent.getClass());
        
        // Salvage Analyser must be in missing list of players that do not need it
        
        // Createive players have no missed techs
        //assertTrue(player1.getMissedTechnologies().contains(tech_SalvageAnalyser));

        //
        // Test un-successful discover tech from salvage
        //
        
        lastEvent = null;
        
       	new SetResearchProjectCmd(tech_shieldDrain, player1).run();
        
        assertTrue(player1.hasDiscovered(tech_photon200));
        assertFalse(player1.hasDiscovered(tech_shieldDrain));
        assertFalse(player1.getSalvage().contains(tech_photon200));
        assertFalse(player1.getSalvage().contains(tech_shieldDrain));
        
        Utils.clearNextRandomProbability();
        Utils.setNextRandomProbability(0.1);

        player1.getResearchProject().endTurn();
        player1.getResearchProject().addResearch(0);

        assertTrue(player1.hasDiscovered(tech_photon200));
        assertFalse(player1.hasDiscovered(tech_shieldDrain));
              
        assertNull(lastEvent);

        //
        // Discovering techs increases change to discover salvage
        //

        assertEquals(0.5, player2.getChanceToDiscoverSalvagedTechnology());
        
        player2.addTechnology(tech_SalvageAnalyser);
        
        assertEquals(1.0, player2.getChanceToDiscoverSalvagedTechnology());


    }
    
    
    /**
     * Tests that Banker requests trade routes and that they are created when accepted.
     */
    public void testCreateDeleteResearchTreaties() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testCreateDeleteResearchTreaties() *************************");
        createMap();

        Relations rels = MotEngine.instance.getRelations();
        Relation rel = rels.getRelationshipNotNull(player1, player2);

        assertFalse("No diplomatic relations - so cannot trade", Scientist.canFormResearchTreaty(player1, player2));

        ((ComputerPlayer)player1).getScientist().assignResearchTreaty();

        assertNull("No diplomatic relations - so cannot trade", rel.getTreatiesAsArray());

        Planet planet4 = newPlanet("SS3-A", ss3, player1);
        
        ((ComputerPlayer)player1).getScientist().assignResearchTreaty();
        MotEngine.instance.getRelations().updateHasDiplomaticRelations();

        assertTrue("Has diplomatic relations - so can trade", Scientist.canFormResearchTreaty(player1, player2));
        assertNull("Only form trade routes every ScientistBasic.CALCULATE_INTERVAL", rel.getTreatiesAsArray());

        MotEngine.instance.setDate(MotEngine.instance.getDate()+ScientistBasic.CALCULATE_START+1);
        ((ComputerPlayer)player1).getScientist().assignResearchTreaty();
        ResearchAgreementTreaty researchTreaty = (ResearchAgreementTreaty) rel.getTreatiesAsArray()[0];

        assertEquals("Trade route formed this year", MotEngine.instance.getDate(), researchTreaty.getDateStarted());

        // RESEARCH TREATIES ARE DELETED WHEN WE GO TO WAR AND CANNOT BE FORMED UNLESS WAR IS REMOVED

        player1.diplomaticRequest(player2, DiplomaticProposal.getWar(), null);

        assertTrue("Trade route deleted because of war", rels.isWar(player1, player2));
        assertFalse("Trade route deleted because of war", rels.isResearch(player1, player2));
        assertEquals("No Research Treaty Exists", null, rel.getTreaty(ResearchAgreementTreaty.class));
        assertEquals("No Research Treaty Exists", 1, rel.getTreatiesAsArray().length);
        assertEquals("No Research Treaty Exists", WarTreaty.class, rel.getTreatiesAsArray()[0].getClass());

        ((ComputerPlayer)player1).getScientist().assignResearchTreaty();

        assertTrue("Trade route not created because of war", rels.isWar(player1, player2));
        assertFalse("Trade route not created because of war", rels.isResearch(player1, player2));
        assertEquals("No Research Treaty Exists", null, rel.getTreaty(ResearchAgreementTreaty.class));
        assertEquals("No Research Treaty Exists", 1, rel.getTreatiesAsArray().length);
        assertEquals("No Research Treaty Exists", WarTreaty.class, rel.getTreatiesAsArray()[0].getClass());

        rels.setPeace(player1, player2);
        MotEngine.instance.setDate(MotEngine.instance.getDate()+ScientistBasic.CALCULATE_INTERVAL);
        ((ComputerPlayer)player1).getScientist().assignResearchTreaty();
        PeaceTreaty peaceTreaty = (PeaceTreaty) rel.getTreatiesAsArray()[0];
        researchTreaty = (ResearchAgreementTreaty) rel.getTreatiesAsArray()[1];

        assertFalse("Research Treaty created", rels.isWar(player1, player2));
        assertTrue("Research Treaty created", rels.isResearch(player1, player2));
        assertEquals("Research Treaty formed this year", MotEngine.instance.getDate(), researchTreaty.getDateStarted());
    }

    /**
     * Test trade routes generate correct revenue and relationship benefits.
     */
    public void testResearchTreaty() throws Exception {
        Logger.logDebug("*******************************  BankerAndRelationsTest.testResearchTreaty() *************************");
        createMap();

        planet1.setPopulation(100);
        planet2.setPopulation(100);
        player1.setFunds(0);
        player2.setFunds(0);

        ss2.setLocation(new Location(250,100));
        Relation rel = MotEngine.instance.getRelations().getRelationship(player1, player2);
        
        assertEquals(50, rel.getValue());
        assertTrue("Can trade", Scientist.canFormResearchTreaty(player1, player2));

        MotEngine.instance.setDate(4000);
        ((ComputerPlayer)player1).getScientist().assignResearchTreaty();
        ResearchAgreementTreaty researchTreaty = (ResearchAgreementTreaty) rel.getTreatiesAsArray()[0];
        researchTreaty.endTurn();

        assertEquals("Research Treaty Formed this year", 1, rel.getTreatiesAsArray().length);
        assertEquals("Research Treaty Formed this year", 4000, researchTreaty.getDateStarted());
        assertEquals("Research Treaty affects relations", 0, rel.getResearchEffect());
        assertEquals("Research Treaty affects relations", 50, rel.getRelationshipValue());
        
        MotEngine.instance.setDate(4001);
        researchTreaty.endTurn();

        assertEquals("Research Treaty Formed this year", 4000, researchTreaty.getDateStarted());
        assertEquals("Research Treaty affects relations", 2, rel.getResearchEffect());
        assertEquals("Research Treaty affects relations", 52, rel.getRelationshipValue());
        
        player1.setResearchProject(new ResearchProject(tech_armour25, player1, false));
        MotEngine.instance.getRelations().endTurn();
        
        assertEquals("Research Treaty generates research", tech_armour25, player1.getResearchProject().getTechnology());
        assertEquals("Research Treaty generates research", 2, player1.getResearchProject().getWork());

        MotEngine.instance.setDate(4051);
        researchTreaty.endTurn();

        assertEquals("Research Treaty Formed this year", 4000, researchTreaty.getDateStarted());
        assertEquals("Research Treaty affects relations", 10, rel.getResearchEffect());
        assertEquals("Research Treaty affects relations", 60, rel.getRelationshipValue());
    }
    
	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }

	public void notify(MotEvent e) throws ApplicationError {
		lastEvent = e;
    }

	public Player getOwner() {
        return proxyOwner;
    }

    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}