/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import java.io.File;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.client.swing.config.AnimationConfigFactory;
import mot.client.swing.config.ImageConfigFactory;
import mot.common.bean.CraftGraphics;
import mot.common.bean.GalaxyGraphicsList;
import mot.common.bean.NewGameDataComputerPlayerDetails;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.bean.RaceGraphicsList;
import mot.common.bean.Size;
import mot.common.designs.CraftDesigns;
import mot.common.factory.GraphicsFactory;
import mot.common.gifts.AncientGifts;
import mot.common.gifts.AncientGiftsFactory;
import mot.common.leaders.GeneralDetailsList;
import mot.common.leaders.GovenorDetailsList;
import mot.common.morals.MoralChoiceList;
import mot.common.morals.MoralChoices;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.RelicsFactory;
import mot.common.setup.SetupOptions;
import mot.common.technology.Biotechnologies;
import mot.common.technology.Biotechnology;
import mot.common.technology.Computer;
import mot.common.technology.Computers;
import mot.common.technology.Engine;
import mot.common.technology.Engines;
import mot.common.technology.Factories;
import mot.common.technology.Factory;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Hull;
import mot.common.technology.Hulls;
import mot.common.technology.Humanities;
import mot.common.technology.Humanity;
import mot.common.technology.Laboratories;
import mot.common.technology.Laboratory;
import mot.common.technology.MineField;
import mot.common.technology.MiscTechnologies;
import mot.common.technology.MiscTechnology;
import mot.common.technology.Module;
import mot.common.technology.Modules;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.technology.TechnologyTree;
import mot.common.technology.TechnologyTreeNode;
import mot.common.technology.Theories;
import mot.common.technology.Theory;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.xml.XMLWriter;
import mot.engine.MotEngine;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.ResearchProject;
import mot.engine.players.UserPlayer;
import mot.engine.players.mayor.CraftDesignerDamageBot;
import mot.engine.players.mayor.CraftDesignerDroidMine;
import mot.engine.players.mayor.CraftDesignerInvader;
import mot.engine.players.mayor.CraftDesignerPlague;
import mot.engine.players.mayor.CraftDesignerPsi;
import mot.engine.players.mayor.CraftDesignerSpecial;
import mot.engine.setup.MotFactory;

public class ResearchTestOnLiveXML extends TestBase {

    //
    // Technology test does not use the test techs - It uses the real techs from the
    // XML files - This is to test the XML files are correct and to test the tech module
    // as it is in the game.
    //

    public static final String MASS_DRIVER2 = "Mass Driver 2";
    public static final String ENERGY10 = "Energy 10";
    public static final String FUSION25x = "Fusion 25x";
    public static final String COLONY_MODULE7 = "Colony Module 7";
    public static final String COMPUTERISEDI = "Computerised I";
    public static final String RESEARCHLABI = "Research Laboratory I";
    public static final String TERRAFORMINGI = "Terraforming I";
    public static final String ELECTRONIC50 = "Electronic 50";
    public static final String ELECTRONIC80 = "Electronic 80";
	private static final String COLONY_MODULE8 = "Colony Module 8";
	private static final String FUSION25 = "Fusion 25";
	private static final String FLYER100 = "Flyer 100";
	private static final String SHIELD_MODULATORI = "Shield Modulator I";
	private static final String PHOTON150 = "Photon 150";
	private static final String PHOTON200 = "Photon 200";
	private static final String ENERGY20 = "Energy 20";
	private static final String TRADE_ROUTE = "Trade Route";

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(ResearchTestOnLiveXML.class);
        return suite;
    }

    public ResearchTestOnLiveXML(String name) {
        super(name);
    }

    private Player player1;
    private Player player2;

   private String [] playerNames = { "TestPlayer1", "TestPlayer2" };

    protected void setUp() {
        try {
        	Technologies.removeUpdates();
            MotEngine.newInstance();
            Technologies.flush();
            TechnologyTree.flush();
            GeneralDetailsList.flush();
            GovenorDetailsList.flush();
            OptionsFactory.flush();
            GraphicsFactory.flush();
            SetupOptions options = mot.common.setup.OptionsFactory.getGeneralOptions();
            player1 = newPlayer(playerNames[0], blue, options, playerNames);
            player2 = newPlayer(playerNames[1], green, options, playerNames);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }

    public void testAllTechnologiesAreInTechTree() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testAllTechnologiesAreInTechTree() *************************");
        
        Technology[] allTechsInTechTree = TechnologyTree.getInstance().getTechnologies();
        TechnologyList allTechs = Technologies.getTechnologies();
		if (!allTechs.contains(allTechsInTechTree)) {
        	TechnologyList diff = allTechs.getAllExcluding(new GeneralTechnologyList(allTechsInTechTree));
        	Logger.logDebug("DIFFERENCE = "+diff);
        	fail("MISSING TECHS");
        }
    }
    
    /**
     * Tests that a player can research new technology until no technology is left
     * and that the order of tech research is correct.
     */
    public void testDiscovery() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testDiscovery() *************************");
        for (int i=0; i<100; i++) {
            ResearchProject prp = player1.getResearchProject();
            player1.addResearch(51);

            ResearchProject rp = player1.getResearchProject();
            if (!player1.hasResearchProject()) {
                Logger.logDebug("NOTHING MORE TO RESEARCH");
                assertTrue("Must have researched somehting", i > 0);
                return;
            }
            assertNotNull("Have a research Project", rp);
            log(player1, i);

            assertFalse("Should not research something already discovered", player1.hasDiscovered(rp.getTechnology()) );

            // This is for user players??
            TechnologyTreeNode node = mot.common.technology.TechnologyTree.getInstance().getNode(rp.getTechnology());
            assertNotNull("Tech "+rp.getTechnology().getName()+" must be in tech tree XML file", node);
            if (!player1.hasDiscovered(node.getParentsAsArray())) {
                Logger.logDebug("Player has not discovered all techs "+rp.getTechnology().getName()+" depends on : "+Comp.toStringNamedItems(node.getParents()));
            }
            assertTrue("Player has not discovered all techs "+rp.getTechnology().getName()+" depends on : "+Comp.toStringNamedItems(node.getParents()), player1.hasDiscovered(node.getParentsAsArray()) );

            if (prp != null) {
                if (rp.getTechnology().getCostToResearch() < prp.getTechnology().getCostToResearch() && !(prp.getTechnology() instanceof Theory) ) {
                    fail("Should by default research cheapest thing to research rp cost="+rp.getTechnology().toString()+" prpCost="+prp.getTechnology().toString());
                }
            }
        }
    }

    /**
     * Tests that research a planet produces goes to the players research project AND
     * new techs become available to a player once that required work has been done.
     */
    public void testAbilities() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testAbilities() *************************");
        ComputerPlayer p = (ComputerPlayer) player2;

        assertTrue("Have technology", p.getSpecials().contains(COLONY_MODULE8));
        assertTrue("Have technology", p.getEngines().contains(FUSION25));

        // Test that a planet ads its research values to the players research project.
        SolarSystem ss = newSolarSystem("SS1", 100, 100);
        Planet planet = newPlanet("SS1-A", ss, player2);
        planet.setPopulation(100.0);

        Technology massDriver2 = mot.common.technology.Technologies.getWeapon(MASS_DRIVER2);
        player2.setResearchProject(new ResearchProject(massDriver2, player2, false));

        assertEquals("No research Done", 0, p.getResearchProject().getWork());
        planet.endTurn();
        assertTrue("Research Done", p.getResearchProject().getWork() > 0);

        p.getResearchProject().setWork(0);

        assertEquals("Has Correct Research Cost", 250, massDriver2.getCostToResearch());

        player2.addResearch(200);
        assertEquals("Correct research Done", 200, p.getResearchProject().getWork());
        assertFalse("Have technology", p.getWeapons().contains(massDriver2));
        player2.addResearch(60);
        assertEquals("Correct research Done", 10, p.getResearchProject().getWork());
        assertTrue("Have technology", p.getWeapons().contains(massDriver2));
        assertTrue("Have technology", p.getWeapons().contains(MASS_DRIVER2));

        player2.addTechnology(mot.common.technology.Technologies.getEngine(FUSION25x));
        assertTrue("Have technology", p.getEngines().contains(FUSION25x));

        player2.addTechnology(mot.common.technology.Technologies.getComputer(ELECTRONIC80));
        assertTrue("Have technology", p.getComputers().contains(ELECTRONIC80));

        player2.addTechnology(mot.common.technology.Technologies.getHull(FLYER100));
        assertTrue("Have technology", p.getHulls().contains(FLYER100));

        player2.addTechnology(mot.common.technology.Technologies.getSpecial(COLONY_MODULE7));
        assertTrue("Have technology", p.getSpecials().contains(COLONY_MODULE7));

        player2.addTechnology(mot.common.technology.Technologies.getSpecial(SHIELD_MODULATORI));
        assertTrue("Have technology", p.getSpecials().contains(SHIELD_MODULATORI));

        player2.addTechnology(mot.common.technology.Technologies.getScanner(PHOTON150));
        assertTrue("Have technology", p.getScanners().contains(PHOTON150));

        player2.addTechnology(mot.common.technology.Technologies.getShield(ENERGY20));
        assertTrue("Have technology", p.getShields().contains(ENERGY20));

        player2.addTechnology(mot.common.technology.Technologies.getHumanity(TRADE_ROUTE));
        assertTrue("Have technology", p.getHumanities().contains(TRADE_ROUTE));
    }

    /**
     * Tests that new technologies discovered are used by the computer player to upgrade
     * their planets.
     */
    public void testUpgrading() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testUpgrading() *************************");
        ComputerPlayer p = (ComputerPlayer) player2;
        SolarSystem ss1 = newSolarSystem("SS1", 100, 100);
        Planet planet = newPlanet("SS1-A", ss1, p);

        assertEquals("Have technology", null, planet.getScanners());
        ResearchProject rp = new ResearchProject(mot.common.technology.Technologies.getScanner(PHOTON150), p, false);
        p.setResearchProject(rp);
        rp.addResearch(100000);
        assertEquals("Have technology", 1, planet.getScanners().getQuantity());
        assertEquals("Have technology", true, planet.getScanners().contains(PHOTON150));

        assertEquals("Have technology", false, planet.getScanners().contains(PHOTON200));
        rp = new ResearchProject(mot.common.technology.Technologies.getScanner(PHOTON200), p, false);
        p.setResearchProject(rp);
        rp.addResearch(100000);
        assertEquals("Have technology", 1, planet.getScanners().getQuantity());
        assertEquals("Have technology", true, planet.getScanners().contains(PHOTON200));
    }

    /**
     * Tests that technologies are loaded properly using DOM parser (pre-loaded as used by applet).
     */
    public void IGNOREtestLoadTechsUsingDOMParser() throws Exception {

		//
		// Load Techs from using DOM parser and check its the same as those loaded by SAX
		//
		
    	Biotechnologies biotechnologies = Technologies.getBiotechnologies();
    	Computers computers = Technologies.getComputers();
    	Engines engines = Technologies.getEngines();
    	Factories factories = Technologies.getFactories();
    	Hulls hulls = Technologies.getHulls();
    	Laboratories laboratories = Technologies.getLaboratories();
    	Scanners scanners = Technologies.getScanners();
    	Shields shields = Technologies.getShields();
    	Weapons weapons = Technologies.getWeapons();
    	Theories theories = Technologies.getTheories();
    	Modules modules = Technologies.getModules();
    	Humanities humanities = Technologies.getHumanities();
    	Specials specials = Technologies.getSpecials();
    	MiscTechnologies miscTechnologies = Technologies.getMiscTechnologies(); 
    	TechnologyTree techTree = TechnologyTree.getInstance();
    	
		Technologies.flush();
		TechnologyTree.flush();
		
    	Technologies.preLoad();
    	TechnologyTree.preLoad();

    	assertSameContent(biotechnologies, Technologies.getBiotechnologies());
    	assertSameContent(computers, Technologies.getComputers());
    	assertSameContent(engines, Technologies.getEngines());
    	assertSameContent(factories, Technologies.getFactories());
    	assertSameContent(hulls, Technologies.getHulls());
    	assertSameContent(laboratories, Technologies.getLaboratories());
    	assertSameContent(scanners, Technologies.getScanners());
    	assertSameContent(shields, Technologies.getShields());
    	assertSameContent(weapons, Technologies.getWeapons());
    	assertSameContent(theories, Technologies.getTheories());
    	assertSameContent(modules, Technologies.getModules());
    	assertSameContent(humanities, Technologies.getHumanities());
    	assertSameContent(specials, Technologies.getSpecials());
    	assertSameContent(miscTechnologies, Technologies.getMiscTechnologies());
    	assertSameContent(miscTechnologies, Technologies.getMiscTechnologies());
    	assertSameContent(techTree, TechnologyTree.getInstance());
    	
    	assertDifferentyInstances(biotechnologies, Technologies.getBiotechnologies());
    	assertDifferentyInstances(computers, Technologies.getComputers());
    	assertDifferentyInstances(engines, Technologies.getEngines());
    	assertDifferentyInstances(factories, Technologies.getFactories());
    	assertDifferentyInstances(hulls, Technologies.getHulls());
    	assertDifferentyInstances(laboratories, Technologies.getLaboratories());
    	assertDifferentyInstances(scanners, Technologies.getScanners());
    	assertDifferentyInstances(shields, Technologies.getShields());
    	assertDifferentyInstances(weapons, Technologies.getWeapons());
    	assertDifferentyInstances(theories, Technologies.getTheories());
    	assertDifferentyInstances(modules, Technologies.getModules());
    	assertDifferentyInstances(humanities, Technologies.getHumanities());
    	assertDifferentyInstances(specials, Technologies.getSpecials());
    	assertDifferentyInstances(miscTechnologies, Technologies.getMiscTechnologies());
    	
		//
		// Test Other XML Files can be loaded using DOM Parser
		//
    	
    	MoralChoices.flush();
    	CraftDesigns.flush();
		
    	AncientGifts ancientGifts = AncientGiftsFactory.getAncientGifts();
    	GalaxyGraphicsList galaxyGraphicsList = GraphicsFactory.getGalaxyGraphicsList();
    	RaceGraphicsList raceGraphicsList = GraphicsFactory.getRaceGraphicsList();
    	ImageConfigFactory imageConfigs = ImageConfigFactory.getInstance();
    	GeneralDetailsList generals = GeneralDetailsList.getInstance();
    	GovenorDetailsList govenors = GovenorDetailsList.getInstance();
    	MoralChoiceList moralChoices = MoralChoices.getMoralChoices();
    	SetupOptions relics = RelicsFactory.getRelics();
    	AnimationConfigFactory animationConfigs = AnimationConfigFactory.getInstance();
    	CraftDesigns craftDesigns = CraftDesigns.getInstance();
    	
    	AncientGiftsFactory.preLoad();
    	GraphicsFactory.preLoadGalaxyGraphics();
    	GraphicsFactory.preLoadRaceGraphics();
    	ImageConfigFactory.preLoad();
    	GeneralDetailsList.preLoad();
    	GovenorDetailsList.preLoad();
    	MoralChoices.preLoad();
    	RelicsFactory.preLoad();
    	AnimationConfigFactory.preLoad();
    	CraftDesigns.preLoad();

    	assertEquals(ancientGifts, AncientGiftsFactory.getAncientGifts());
    	assertEquals(galaxyGraphicsList, GraphicsFactory.getGalaxyGraphicsList());
    	assertEquals(raceGraphicsList, GraphicsFactory.getRaceGraphicsList());
    	assertEquals(imageConfigs, ImageConfigFactory.getInstance());
    	assertEquals(generals, GeneralDetailsList.getInstance());
    	assertEquals(govenors, GovenorDetailsList.getInstance());
    	assertEquals(moralChoices, MoralChoices.getMoralChoices());
    	assertEquals(relics, RelicsFactory.getRelics());
    	assertEquals(animationConfigs, AnimationConfigFactory.getInstance());
    	assertEquals(craftDesigns, CraftDesigns.getInstance());

    }

    public void testPlayerPersonalityEffectsCraftDesign() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testPlayerPersonalityEffectsCraftDesign() *************************");

        TechnologyList allTechs = Technologies.getTechnologies();
        
        {
	        String raceName = "Humans";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.5, null, humansOptions, playerNames);	        
			SolarSystem ss = newSolarSystem("HumanHome", 100, 100);
			Planet planet = newPlanet("HumanHome-A", ss, player1);
			p.addTechnologies(allTechs);
	        planet.setQtyFactories(100);
	        planet.setPopulation(200);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getSpecialCraftDesigner();
			
			assertNull(cdr);
        }
        
        {
	        String raceName = "Bears";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.5, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertEquals(CraftDesignerInvader.class, cdr.getClass());
        }
        
        {
	        String raceName = "Ghosts";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.5, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertEquals(CraftDesignerPsi.class, cdr.getClass());
        }
        
        {
	        String raceName = "Shifters";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.5, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertEquals(CraftDesignerPlague.class, cdr.getClass());
        }
        
        {
	        String raceName = "Machines";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.5, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertEquals(CraftDesignerDamageBot.class, cdr.getClass());
        }
        
        {
	        String raceName = "Birds";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.5, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertEquals(CraftDesignerDroidMine.class, cdr.getClass());
        }
        
    }
  

    public void testPlayerPersonalityOffOnEasyGames() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testPlayerPersonalityEffectsCraftDesign() *************************");

        TechnologyList allTechs = Technologies.getTechnologies();
        
        {
	        String raceName = "Humans";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.0, null, humansOptions, playerNames);	        
			SolarSystem ss = newSolarSystem("HumanHome", 100, 100);
			Planet planet = newPlanet("HumanHome-A", ss, player1);
			p.addTechnologies(allTechs);
	        planet.setQtyFactories(100);
	        planet.setPopulation(200);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getSpecialCraftDesigner();
			
			assertNull(cdr);
        }
        
        {
	        String raceName = "Bears";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.0, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertNull(cdr);
        }
        
        {
	        String raceName = "Ghosts";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.0, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertNull(cdr);
        }
        
        {
	        String raceName = "Shifters";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.0, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertNull(cdr);
        }
        
        {
	        String raceName = "Machines";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.0, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertNull(cdr);
        }
        
        {
	        String raceName = "Birds";
			RaceOptions humansOptions = OptionsFactory.getRaceOptions(raceName);
			ComputerPlayer p = newPlayer(raceName, blue, 1.0, null, humansOptions, playerNames);	        
			p.addTechnologies(allTechs);

			CraftDesignerSpecial cdr = p.getMayor().getCraftDesignerFactory().getAndUpdateSpecialCraftDesigner(p);
			
			assertNull(cdr);
        }
        
    }
    

    /**
     * Tests that new technologies discovered are used by the computer player to upgrade
     * their planets.
     */
    public void testTechsExistThatAreNamesInTheGame() throws Exception {
        assertNotNull( Technologies.getMiscTechnology("Basic Planet Construction") );
        assertNotNull( Technologies.getMiscTechnology("Advanced Planet Construction") );
        assertNotNull( Technologies.getMiscTechnology("Extended Space Jump") );
        assertNotNull( Technologies.getHumanity("Human Trade Route") );
    }


    /**
     * Tests that new technologies discovered are used by the computer player to upgrade
     * their planets.
     */
    public void testStartingTechnologies() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testStartingTechnologies() *************************");
        OptionsFactory.flush();

        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[1];
        userDetails[0] = new NewGameUserPlayerData("Humans", OptionsFactory.getSetupOptions("Humans"), playerColourBlue, null);

        NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[1];
        cpd[0] = new NewGameDataComputerPlayerDetails("Birds", 1.0, null);
        MotEngine engine = MotFactory.create(false, false, false, new Double(1.0), cpd, 5, 0, new Size(500, 500), userDetails, false);

        Planet [] planets = engine.getPlanets().getPlanets();
        Player [] players = engine.getPlayers().getPlayers();
        Player p1 = engine.getPlayers().getPlayers()[0];
        Player p2 = engine.getPlayers().getPlayers()[1];
        
        assertEquals("Has correct quantity of players", 2, players.length);
        assertTrue(p1 instanceof UserPlayer);
        assertTrue(p2 instanceof ComputerPlayer);
        
        assertEquals("Humans", p1.getName());
        assertTrue(p1.hasDiscovered(Technologies.getHumanity("Human Trade Route")));
        assertTrue(p1.hasDiscovered(Technologies.getEngine("Fusion 25")));
        assertTrue(p1.hasDiscovered(Technologies.getHull("Flyer 30")));
        assertTrue(p1.hasDiscovered(Technologies.getHull("Flyer 30")));
        assertTrue(p1.hasDiscovered(Technologies.getHull("Fighter I")));
        assertTrue(p1.hasDiscovered(Technologies.getWeapon("Mass Driver 1")));
        assertTrue(p1.hasDiscovered(Technologies.getWeapon("Missile 6")));
        assertTrue(p1.hasDiscovered(Technologies.getScanner("Photon 100")));
        assertTrue(p1.hasDiscovered(Technologies.getFactory("Basic Computerised Factory")));
        assertTrue(p1.hasDiscovered(Technologies.getSpecial("Colony Module 8")));
        
        assertEquals("Birds", p2.getName());
        assertFalse(p2.hasDiscovered(Technologies.getHumanity("Human Trade Route")));
        assertTrue(p2.hasDiscovered(Technologies.getEngine("Fusion 25")));
        assertTrue(p2.hasDiscovered(Technologies.getHull("Flyer 30")));
        assertTrue(p2.hasDiscovered(Technologies.getHull("Flyer 30")));
        assertTrue(p2.hasDiscovered(Technologies.getHull("Fighter I")));
        assertTrue(p2.hasDiscovered(Technologies.getWeapon("Mass Driver 1")));
        assertTrue(p2.hasDiscovered(Technologies.getWeapon("Missile 6")));
        assertTrue(p2.hasDiscovered(Technologies.getScanner("Photon 100")));
        assertTrue(p2.hasDiscovered(Technologies.getFactory("Basic Computerised Factory")));
        assertTrue(p2.hasDiscovered(Technologies.getSpecial("Colony Module 8")));
        
    }
    
    /**
     * Tests that technologies are loaded from updates folder when it exists and can be saved to updates.
     */
    public void testSaveAndLoadTechs() throws Exception {
    	
		//
		// ADD NEW CONTENT NO IN NORMAL TECHS
		//
		
    	Technologies.removeUpdates();
    	TechnologyTree.removeUpdates();
    	
		Weapons weapons = Technologies.getWeapons();
		Biotechnologies biotechnologies = Technologies.getBiotechnologies();
		Computers computers = Technologies.getComputers();
		Engines engines = Technologies.getEngines();
		Factories factories = Technologies.getFactories();
		Hulls hulls = Technologies.getHulls();
		Laboratories laboratories = Technologies.getLaboratories();
		MiscTechnologies miscTechnologies = Technologies.getMiscTechnologies();
		Modules modules = Technologies.getModules();
		Humanities humanities = Technologies.getHumanities();
		Scanners scanners = Technologies.getScanners();
		Shields shields = Technologies.getShields();
		Specials specials = Technologies.getSpecials();
		Theories theories = Technologies.getTheories();
		TechnologyTree techTree = TechnologyTree.getInstance();
		
		assertFalse(weapons.contains("NEWWEAPON"));
		assertFalse(biotechnologies.contains("NEWBIO"));
		assertFalse(computers.contains("NEWCOMP"));
		assertFalse(engines.contains("NEWENGINE"));
		assertFalse(factories.contains("NEWFACTORY"));
		assertFalse(hulls.contains("NEWHULL"));
		assertFalse(laboratories.contains("NEWLAB"));
		assertFalse(miscTechnologies.contains("NEWMINEFIELD"));
		assertFalse(modules.contains("NEWMODULE"));
		assertFalse(humanities.contains("NEWHUMANITY"));
		assertFalse(scanners.contains("NEWSCANNER"));
		assertFalse(shields.contains("NEWSHIELD"));
		assertFalse(specials.contains("NEWSPECIAL"));
		assertFalse(theories.contains("NEWTHEORY"));
		assertFalse(techTree.contains("theory , NEWTHEORY"));
		
		Weapon nw = (Weapon) weapons.getAllAsArray()[0].cloneTechnology();
		nw.setName("NEWWEAPON");
		weapons.add(nw);
		
		Biotechnology nb = (Biotechnology) biotechnologies.getAllAsArray()[0].cloneTechnology();
		nb.setName("NEWBIO");
		biotechnologies.add(nb);
		
		Computer nc = (Computer) computers.getAllAsArray()[0].cloneTechnology();
		nc.setName("NEWCOMP");
		computers.add(nc);
		
		Engine ne = (Engine) engines.getAllAsArray()[0].cloneTechnology();
		ne.setName("NEWENGINE");
		engines.add(ne);
		
		Factory nf = (Factory) factories.getAllAsArray()[0].cloneTechnology();
		nf.setName("NEWFACTORY");
		factories.add(nf);
		
		Hull nh = (Hull) hulls.getAllAsArray()[0].cloneTechnology();
		nh.setName("NEWHULL");
		hulls.add(nh);
		
		Laboratory nl = (Laboratory) laboratories.getAllAsArray()[0].cloneTechnology();
		nl.setName("NEWLAB");
		laboratories.add(nl);
		
		MiscTechnology nmf = new MineField();
		nmf.setName("NEWMINEFIELD");
		miscTechnologies.add(nmf);
		
		Module nmo = (Module) modules.getAllAsArray()[0].cloneTechnology();
		nmo.setName("NEWMODULE");
		modules.add(nmo);
		
		Humanity nhu = (Humanity) humanities.getAllAsArray()[0].cloneTechnology();
		nhu.setName("NEWHUMANITY");
		humanities.add(nhu);
		
		Scanner ns = (Scanner) scanners.getAllAsArray()[0].cloneTechnology();
		ns.setName("NEWSCANNER");
		scanners.add(ns);
		
		Shield nsh = (Shield) shields.getAllAsArray()[0].cloneTechnology();
		nsh.setName("NEWSHIELD");
		shields.add(nsh);
		
		Special nsp = (Special) specials.getAllAsArray()[0].cloneTechnology();
		nsp.setName("NEWSPECIAL");
		specials.add(nsp);
		
		Theory nt = (Theory) theories.getAllAsArray()[0].cloneTechnology();
		nt.setName("NEWTHEORY");
		theories.add(nt);
		
		TechnologyTreeNode nn = (TechnologyTreeNode) techTree.getNodesAsArray()[0].clone();
		nn.setNode(nt);
		techTree.setNode(nn);
		
		//
		// CLONE TECHNOLOGIES SO WE HAVE A COPY OF THE ORIGINALS
		//
		
		weapons = new Weapons(weapons);
		biotechnologies = new Biotechnologies(biotechnologies);
		computers = new Computers(computers);
		engines = new Engines(engines);
		factories = new Factories(factories);
		hulls = new Hulls(hulls);
		laboratories = new Laboratories(laboratories);
		miscTechnologies = new MiscTechnologies(miscTechnologies);
		modules = new Modules(modules);
		humanities = new Humanities(humanities);
		scanners = new Scanners(scanners);
		shields = new Shields(shields);
		specials = new Specials(specials);
		theories = new Theories(theories);
		techTree = new TechnologyTree(techTree);
		
		
		//
		// SAVE AND RELOAD
		//
		
		XMLWriter writer = new XMLWriter(null, null, null, true);
		Technologies.getWeapons().toXML(writer);
		writer.save(new File("updates/tech_weapons.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getBiotechnologies().toXML(writer);
		writer.save(new File("updates/tech_bios.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getComputers().toXML(writer);
		writer.save(new File("updates/tech_computers.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getEngines().toXML(writer);
		writer.save(new File("updates/tech_engines.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getFactories().toXML(writer);
		writer.save(new File("updates/tech_factories.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getHulls().toXML(writer);
		writer.save(new File("updates/tech_hulls.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getLaboratories().toXML(writer);
		writer.save(new File("updates/tech_labs.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getMiscTechnologies().toXML(writer);
		writer.save(new File("updates/tech_misc.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getModules().toXML(writer);
		writer.save(new File("updates/tech_modules.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getHumanities().toXML(writer);
		writer.save(new File("updates/tech_humanities.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getScanners().toXML(writer);
		writer.save(new File("updates/tech_scanners.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getShields().toXML(writer);
		writer.save(new File("updates/tech_shields.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getSpecials().toXML(writer);
		writer.save(new File("updates/tech_specials.xml"));
		writer = new XMLWriter(null, null, null, true);
		Technologies.getTheories().toXML(writer);
		writer.save(new File("updates/tech_theories.xml"));
		writer = new XMLWriter(null, null, null, true);
		TechnologyTree.getInstance().toXML(writer);
		writer.save(new File("updates/tech_tree.xml"));
		
		Technologies.flush();
		TechnologyTree.flush();
		
		//
		// CHECK NEW CONTENT EXISTS HENCE IT IS LOADED FROM UPDATES FOLDER
		// CHECK TECHS HAVE SAME CONTENT BUT ARE NEW INSTANCES (HAVE BEEN RELOADED)
		//
		
		assertTrue(weapons.contains("NEWWEAPON"));
		assertTrue(biotechnologies.contains("NEWBIO"));
		assertTrue(computers.contains("NEWCOMP"));
		assertTrue(engines.contains("NEWENGINE"));
		assertTrue(factories.contains("NEWFACTORY"));
		assertTrue(hulls.contains("NEWHULL"));
		assertTrue(laboratories.contains("NEWLAB"));
		assertTrue(miscTechnologies.contains("NEWMINEFIELD"));
		assertTrue(modules.contains("NEWMODULE"));
		assertTrue(humanities.contains("NEWHUMANITY"));
		assertTrue(scanners.contains("NEWSCANNER"));
		assertTrue(shields.contains("NEWSHIELD"));
		assertTrue(specials.contains("NEWSPECIAL"));
		assertTrue(theories.contains("NEWTHEORY"));
		assertTrue(techTree.contains("theory , NEWTHEORY"));
		
		assertSameContent(weapons, Technologies.getWeapons());
		assertSameContent(biotechnologies, Technologies.getBiotechnologies());
		assertSameContent(computers, Technologies.getComputers());
		assertSameContent(engines, Technologies.getEngines());
		assertSameContent(factories, Technologies.getFactories());
		assertSameContent(hulls, Technologies.getHulls());
		assertSameContent(laboratories, Technologies.getLaboratories());
		assertSameContent(miscTechnologies, Technologies.getMiscTechnologies());
		assertSameContent(modules, Technologies.getModules());
		assertSameContent(humanities, Technologies.getHumanities());
		assertSameContent(scanners, Technologies.getScanners());
		assertSameContent(shields, Technologies.getShields());
		assertSameContent(specials, Technologies.getSpecials());
		assertSameContent(theories, Technologies.getTheories());
		assertSameContent(techTree, TechnologyTree.getInstance());
		
		//
		// REMOVE UPDATES FOLDER AND RELOAD TECHS
		// CHECK NEW CONTENT IS NOW MISSING
		//
		
    	Technologies.removeUpdates();
    	TechnologyTree.removeUpdates();
		Technologies.flush();
		TechnologyTree.flush();
    	
		weapons = Technologies.getWeapons();
		biotechnologies = Technologies.getBiotechnologies();
		computers = Technologies.getComputers();
		engines = Technologies.getEngines();
		factories = Technologies.getFactories();
		hulls = Technologies.getHulls();
		laboratories = Technologies.getLaboratories();
		miscTechnologies = Technologies.getMiscTechnologies();
		modules = Technologies.getModules();
		humanities = Technologies.getHumanities();
		scanners = Technologies.getScanners();
		shields = Technologies.getShields();
		specials = Technologies.getSpecials();
		theories = Technologies.getTheories();
		techTree = TechnologyTree.getInstance();
		
		assertFalse(weapons.contains("NEWWEAPON"));
		assertFalse(biotechnologies.contains("NEWBIO"));
		assertFalse(computers.contains("NEWCOMP"));
		assertFalse(engines.contains("NEWENGINE"));
		assertFalse(factories.contains("NEWFACTORY"));
		assertFalse(hulls.contains("NEWHULL"));
		assertFalse(laboratories.contains("NEWLAB"));
		assertFalse(miscTechnologies.contains("NEWMINEFIELD"));
		assertFalse(modules.contains("NEWMODULE"));
		assertFalse(humanities.contains("NEWHUMANITY"));
		assertFalse(scanners.contains("NEWSCANNER"));
		assertFalse(shields.contains("NEWSHIELD"));
		assertFalse(specials.contains("NEWSPECIAL"));
		assertFalse(theories.contains("NEWTHEORY"));
		assertFalse(techTree.contains("theory , NEWTHEORY"));
    }

    public void testDefaultCraftGraphicsExist() throws Exception {
    	CraftGraphics cg = GraphicsFactory.getRaceGraphicsList().getDefaultHullGraphics();
    	assertNotNull(cg);
    }

	public void log(Player player, int i) throws Exception {
        StringBuffer sb = new StringBuffer();
        if (player.getFactories() != null) sb.append("\nFactories ="+(player.getFactories().getNames(", ")));
        if (player.getLaboratories() != null) sb.append("\nLaboratories ="+(player.getLaboratories().getNames(", ")));
        if (player.getHulls() != null) sb.append("\nHulls ="+(player.getHulls().getNames(", ")));
        if (player.getComputers() != null) sb.append("\nComputers ="+(player.getComputers().getNames(", ")));
        if (player.getWeapons() != null) sb.append("\nWeapons ="+(player.getWeapons().getNames(", ")));
        if (player.getShields() != null) sb.append("\nShields ="+(player.getShields().getNames(", ")));
        if (player.getScanners() != null) sb.append("\nScanners ="+(player.getScanners().getNames(", ")));
        if (player.getSpecials() != null) sb.append("\nSpecials ="+(player.getSpecials().getNames(", ")));
        if (player.getEngines() != null) sb.append("\nEngines ="+(player.getEngines().getNames(", ")));
        if (player.getBiotechnologies() != null) sb.append("\nBiotechnologies ="+(player.getBiotechnologies().getNames(", ")));
        String description = "No Project";
        if (player.getResearchProject() != null) {
            description = player.getResearchProject().getProjectDescription();
        }
        Logger.logDebug("TURN "+i+" "+description+"\nResearched Technologies:"+sb.toString());
    }

    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}