/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Location;
import mot.common.event.MotEvent;
import mot.common.proxy.ClientProxy;
import mot.common.resources.ResourceAntiMatter;
import mot.common.resources.ResourceBoralium;
import mot.common.resources.ResourceCloakPalm;
import mot.common.resources.ResourceEnergySpores;
import mot.common.resources.ResourcePsiFungus;
import mot.common.resources.ResourceTritanOre;
import mot.common.resources.ResourceWarpCrystal;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Scanners;
import mot.common.technology.Technologies;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.crafts.Craft;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.setup.ResourceFactory;

/**
 * Tests the Money functionality. This tests;
 *  - Unused production produces money
 *  - Crafts and planetary shields, weapons, scanners have a maintainence cost
 *  - When a player runs out of money items are sold.
 */
public class ResourceTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(ResourceTest.class);
        return suite;
    }

    public ResourceTest(String name) {
        super(name);
    }

    private SetupOptions options;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private SolarSystem ss1;
    private SolarSystem ss2;

    private Planet planet1;
    private Planet planet2;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
    }

    public void createMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        craft1 = newCraft(fighterDesign, player1, ss2);
        craft2 = newCraft(fighterDesign, player2, ss2);

    }
    
    private Battle battle;
    
    private Craft craft1;
    private Craft craft2;

    private BattleCraft [] bca;

    private void createWarAndBattleOnMap() throws Exception {
        MotEngine.getInstance().getRelations().setWar(player1, player2);
        
        battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss2.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();

        bca = battle.getCrafts();
        
        assertEquals(craft1, bca[0].getCraft());
        assertEquals(craft2, bca[1].getCraft());
        
        bca[0].setLocation(new Location(100, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(150, 500));
        bca[1].setDirection(270);
    }

    public void testBonuses() throws Exception {
        Logger.logDebug("*******************************  ResourceTest.testBonuses() *************************");
        createMap(null);
        createWarAndBattleOnMap();

        assertEquals(25, craft1.getWarpSpeed());
        assertEquals(100, bca[0].getSpeed());
        assertEquals(20, craft1.getMaxDamage());
        
        planet1.setResource(new ResourceWarpCrystal());
        player1.flushCache();
        
        assertEquals(31, craft1.getWarpSpeed());
        assertEquals(125, bca[0].getSpeed());
        
        planet1.setResource(new ResourceTritanOre());
        player1.flushCache();
        
        assertEquals(30, craft1.getMaxDamage());
        
        planet1.setResource(new ResourceAntiMatter());
        player1.flushCache();
        
        assertEquals(0.25, player1.getOwnerDamageBonus(), 0.0);
        bca[1].getShields()[0].setPower(0.0);
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(2.5, bca[1].getDamage(), 0.0);
        
        planet1.setResource(new ResourcePsiFungus());
        player1.flushCache();

        assertEquals(15, player1.getOwnerPsiRating());

        planet1.setResource(new ResourceCloakPalm());
        craft2.setScanners(new Scanners(Technologies.getScanner(TestTechs.PHOTON200)));
        player1.flushCache();
        
        assertEquals(150, player1.getOwnerCloakLevel());
        assertEquals(true, bca[0].isVisible(player2));
        bca[0].setLocation(new Location(90, 500));
        assertEquals(false, bca[0].isVisible(player2));
        
        planet1.setResource(new ResourceEnergySpores());
        player1.flushCache();
        
        assertEquals(0.5, player1.getOwnerShieldPowerBonus(), 0.0);
        bca[0].getShields()[0].rechargeFull();
        assertEquals(10, bca[0].getShields()[0].getShield().getPower(), 0.0);
        assertEquals(15, bca[0].getShields()[0].getPower(), 0.0);
        

    }

    public void testResourceFactory() throws Exception {
        Logger.logDebug("*******************************  testResourceFactory() *************************");
        createMap(null);
        createWarAndBattleOnMap();

        Utils.setNextRandomInt(0);
        ResourceFactory.createResource(planet1);
        assertEquals(true, planet1.getResource() instanceof ResourceAntiMatter);

        Utils.setNextRandomInt(1);
        ResourceFactory.createResource(planet1);
        assertEquals(true, planet1.getResource() instanceof ResourceCloakPalm);

        Utils.setNextRandomInt(2);
        ResourceFactory.createResource(planet1);
        assertEquals(true, planet1.getResource() instanceof ResourcePsiFungus);

        Utils.setNextRandomInt(3);
        ResourceFactory.createResource(planet1);
        assertEquals(true, planet1.getResource() instanceof ResourceTritanOre);

        Utils.setNextRandomInt(4);
        ResourceFactory.createResource(planet1);
        assertEquals(true, planet1.getResource() instanceof ResourceWarpCrystal);

        Utils.setNextRandomInt(5);
        ResourceFactory.createResource(planet1);
        assertEquals(true, planet1.getResource() instanceof ResourceBoralium);

        Utils.setNextRandomInt(6);
        ResourceFactory.createResource(planet1);
        assertEquals(true, planet1.getResource() instanceof ResourceEnergySpores);
        
    }
    

    public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
    }

	public Player getOwner() {
        return player1;
    }

    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}