/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.SalvageList;
import mot.common.designs.CraftDesign;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Technology;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.SalvageFactory;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;

/**
 * Changes - Inter Block Gap is now 3
 * Mass Driver has damage 1
 */

public class SalvageFactoryTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(SalvageFactoryTest.class);
        return suite;
    }

    public SalvageFactoryTest(String name) {
        super(name);
    }

    private SolarSystem ss1;
    private SolarSystem ss2;

    private ComputerPlayer player1;
    private ComputerPlayer player2;

    private Planet planet1;
    private Planet planet2;
    
    private BattleCraft [] bca;
	private Craft craft1;
	private Craft craft2;
    
    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
    }

    public void createMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(player1, player2);
    }

    public void createMap(SetupOptions optionsRace, CraftDesign cd) throws Exception {
        createMap(optionsRace);
        
        craft1 = newCraft(cd, player1, ss1);
        craft2 = newCraft(fighterDesign, player2, ss1);
        
        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);;
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        bca = battle.getCrafts();
        
        assertEquals(craft1, bca[0].getCraft());
        assertEquals(craft2, bca[1].getCraft());
    	
    }

    /**
     * Tests that the salvage weapon can extract technology from wreckage on the battle field 
     */
    public void testWithFewTechs() throws Exception {
        Logger.logDebug("*******************************  SalvageFactoryTest.testWithFewTechs() *************************");
		CraftDesign design = new CraftDesign("TestSalvage", tech_flyer30, tech_fusion25, tech_energy10, tech_massDriver1, tech_massDriver1);
        createMap(null, design);

		//
		// Test when random numbers select all available techs
		//
		
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(1); // qty tech in salvage
        Utils.setNextRandomInt(0);
        
        SalvageList salvage = SalvageFactory.create(bca[0]);
        Technology[] salvageArray = salvage.getAllAsArray();
        
		assertEquals(1, salvageArray.length);
		assertEquals(tech_flyer30, salvageArray[0]);
        
		//
		// Test when random numbers select none available techs
		//
		
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(0); // qty tech in salvage

        salvage = SalvageFactory.create(bca[0]);
        salvageArray = salvage.getAllAsArray();

		assertNull(salvageArray);

		//
		// Test when random numbers select two available techs
		//
		
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(2); // qty tech in salvage
        Utils.setNextRandomInt(2);
        Utils.setNextRandomInt(3);

        salvage = SalvageFactory.create(bca[0]);
        salvageArray = salvage.getAllAsArray();

		assertEquals(tech_energy10, tech_massDriver1, salvageArray);
    }

    /**
     * Tests that the incinerator causes no salvage to be created
     */
    public void testIncineratorRemovesSalvage() throws Exception {
        Logger.logDebug("*******************************  SalvageFactoryTest.testModifiedSalvagePercent1() *************************");
		CraftDesign design = new CraftDesign("TestSalvage", 
				tech_flyer100,
				tech_fusion40,
				tech_incinerator);
        createMap(null, design);

        
        SalvageList salvage = SalvageFactory.create(bca[0]);
        Technology[] salvageArray = salvage.getAllAsArray();
        
        assertNull(salvageArray);
    }
    
    /**
     * Tests that the salvage effect of a weapon effects the quantity of salvage produced 
     */
    public void testWeaponSalvageEffect() throws Exception {
        Logger.logDebug("*******************************  SalvageFactoryTest.testModifiedSalvagePercent2() *************************");
		CraftDesign design = new CraftDesign("TestSalvage", 
				tech_flyer100,
				tech_fusion40,
				tech_detonator);
        createMap(null, design);

    }


    /**
     * Tests at the end of a battle salvage is collected and can cause research to be completed. 
     */
    public void testEndBattle() throws Exception {
        Logger.logDebug("*******************************  SalvageFactoryTest.testModifiedSalvagePercent2() *************************");
		CraftDesign design = new CraftDesign("TestSalvage", 
				tech_flyer100,
				tech_fusion40,
				tech_energy10,
				tech_girator,
				tech_fusionBeam4);
        createMap(null, design);
        
        Battle battle = MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss1.getOrbit(), true);
        battle.join(craft1, true);
        battle.join(craft2, true);

        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(2); // qty tech in salvage
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(3);
        
        SalvageList salvage = SalvageFactory.create(bca[0]);
        Technology[] salvageArray = salvage.getAllAsArray();
        
        assertEquals(tech_flyer100, tech_fusionBeam4, salvageArray);
              
    }


    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}