/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import java.io.File;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.GalaxyDetails;
import mot.common.bean.ImmunityBio;
import mot.common.bean.ImmunityGravity;
import mot.common.bean.ImmunityMindControl;
import mot.common.bean.ImmunityPsi;
import mot.common.bean.ImmunityReplicaBeam;
import mot.common.bean.Location;
import mot.common.bean.Nebula;
import mot.common.bean.SalvageList;
import mot.common.bean.Size;
import mot.common.bean.stats.GameScoreData;
import mot.common.bean.stats.Graph;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftUpgradeDesign;
import mot.common.designs.Orbital;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.WeaponDesign;
import mot.common.leaders.GeneralDetails;
import mot.common.leaders.GeneralDetailsList;
import mot.common.leaders.GovenorDetails;
import mot.common.leaders.GovenorDetailsList;
import mot.common.morals.MoralChoices;
import mot.common.resources.ResourceAntiMatter;
import mot.common.resources.ResourceBoralium;
import mot.common.resources.ResourceCloakPalm;
import mot.common.resources.ResourceEnergySpores;
import mot.common.resources.ResourcePsiFungus;
import mot.common.resources.ResourceTritanOre;
import mot.common.resources.ResourceWarpCrystal;
import mot.common.resources.Resources;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.AsteroidFieldConstruction;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Modules;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Technologies;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.vo.CraftVO;
import mot.engine.MotEngine;
import mot.engine.common.Orbit;
import mot.engine.common.Outpost;
import mot.engine.common.SpaceStation;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.planets.BlackHole;
import mot.engine.planets.BlackHoleFactory;
import mot.engine.planets.CrisisEmergency;
import mot.engine.planets.CrisisFactory;
import mot.engine.planets.CrisisPlague;
import mot.engine.planets.CrisisPsi;
import mot.engine.planets.OrbitalReference;
import mot.engine.planets.Planet;
import mot.engine.planets.ProjectAsteroids;
import mot.engine.planets.ProjectBattleStation;
import mot.engine.planets.ProjectConstructPlanet;
import mot.engine.planets.ProjectCraft;
import mot.engine.planets.ProjectCraftUpgrade;
import mot.engine.planets.ProjectOutpost;
import mot.engine.planets.ProjectScanner;
import mot.engine.planets.ProjectShield;
import mot.engine.planets.ProjectSpaceStation;
import mot.engine.planets.ProjectWeapon;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRouteFactory;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.UserPlayer;
import mot.engine.players.generals.CraftTask;
import mot.engine.players.generals.CraftTaskAttack;
import mot.engine.players.generals.CraftTaskBombard;
import mot.engine.players.generals.CraftTaskColonise;
import mot.engine.players.generals.CraftTaskDefend;
import mot.engine.players.generals.CraftTaskExplore;
import mot.engine.players.generals.CraftTaskMaraud;
import mot.engine.players.generals.CraftTaskMaraudCraft;
import mot.engine.players.generals.CraftTaskProtect;
import mot.engine.players.generals.CraftTaskSurpriseAttackStealColony;
import mot.engine.players.generals.General;
import mot.engine.players.generals.GeneralBasic;
import mot.engine.players.strategists.StrategistEventAnnoy;
import mot.engine.players.strategists.StrategistEventBrokeNonAggression;
import mot.engine.players.strategists.StrategistEventCaughtSabotaging;
import mot.engine.players.strategists.StrategistEventCaughtSpying;
import mot.engine.players.strategists.StrategistEventColonyLost;
import mot.engine.players.strategists.StrategistEventDeclareWar;
import mot.engine.players.strategists.StrategistEventDeclareWarOnEnemy;
import mot.engine.players.strategists.StrategistEventDemandAssets;
import mot.engine.players.strategists.StrategistEventEmpireEstablished;
import mot.engine.players.strategists.StrategistEventGenocide;
import mot.engine.players.strategists.StrategistEventGiveAssets;
import mot.engine.players.strategists.StrategistEventInvasion;
import mot.engine.players.strategists.StrategistEventManager;
import mot.engine.players.strategists.StrategistEventMoralDecision;
import mot.engine.players.strategists.StrategistEventRejectSwapAssets;
import mot.engine.players.strategists.StrategistEventRelationsEstablished;
import mot.engine.players.strategists.StrategistEventRequestAllianceTreaty;
import mot.engine.players.strategists.StrategistEventRequestNonAgressionTreaty;
import mot.engine.players.strategists.StrategistEventRequestPeaceTreaty;
import mot.engine.players.strategists.StrategistEventRequestResearchTreaty;
import mot.engine.players.strategists.StrategistEventRequestTradeTreaty;
import mot.engine.players.strategists.StrategistEventTreatyRejected;
import mot.engine.players.strategists.StrategistEventUsedBiologicalWeapons;
import mot.engine.prizes.BoraliumPrize;
import mot.engine.prizes.CraftPrize;
import mot.engine.prizes.MoneyPrize;
import mot.engine.prizes.TechnologyPrize;
import mot.engine.relations.AllianceTreaty;
import mot.engine.relations.NonAgressionTreaty;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.relations.ResearchAgreementTreaty;
import mot.engine.relations.TradeAgreementTreaty;
import mot.engine.spies.Spies;
import mot.engine.spies.SpyRoleDefend;
import mot.engine.spies.SpyRoleEspionage;
import mot.engine.spies.SpyRoleSabotage;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskCreateTradeRoute;
import mot.engine.transports.TransportTaskDeployStation;
import mot.engine.transports.TransportTaskMoveColonists;

/**
 * Tests that a craft moves between two solar system correctly.
 * Test the General objects.
 */
public class SaveLoadTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(SaveLoadTest.class);
        return suite;
    }

    public SaveLoadTest(String name) {
        super(name);
    }

    private SetupOptions options;

    private ComputerPlayer player1;
    private ComputerPlayer player2;
    private ComputerPlayer player3;
    private ComputerPlayer player4;
    private UserPlayer userPlayer;
    private ComputerPlayer deadPlayer;

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;
    private SolarSystem ss5;
    private SolarSystem ss6;
    private SolarSystem ss7;
    private SolarSystem ss8;
    private SolarSystem ss9;
    private SolarSystem ss10;

    private Planet planet1;
    private Planet planet1mine;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;
    private Planet planet6;
    private Planet planet7a;
    private Planet planet7b;
    private Planet planet7c;
    private Planet planet7d;
    private Planet planet7e;
    private Planet planet7f;
    private Planet planet7g;
    private Planet planet7h;
    private Planet planet7i;
    private Planet planet10;

    private BlackHole blackHole;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        Logger.setDebug();

        try {
            String [] playerNames = { "TestPlayer1", "TestPlayer2", "TestPlayer3", "TestPlayer4", "DeadPlayer" };
            options = mot.common.setup.OptionsFactory.getGeneralOptions();
            player1 = newPlayer(playerNames[0], blue, options, playerNames);
            player2 = newPlayer(playerNames[1], green, options, playerNames);
            player3 = newPlayer(playerNames[2], red, options, playerNames);
            player4 = newPlayer(playerNames[3], red, options, playerNames);
            deadPlayer = newPlayer(playerNames[4], yellow, options, playerNames);
            
            RaceOptions raceOptions = OptionsFactory.getRaceOptions("Humans");
            userPlayer = newUserPlayer("TestUserPlayer", blue, null, raceOptions);
            deadPlayer.surrendetTo(userPlayer);

            ss1 = newSolarSystem("SS1", 100, 100);
            planet1 = newPlanet("SS1-A", ss1, player1);
            planet1mine = newPlanet("SS1-MINE", ss1, null, 50, 0.1);
            ss1.addVisitor(player1, false);

            ss2 = newSolarSystem("SS2", 150, 100);
            planet2 = newPlanet("SS2-A", ss2, player2);

            ss3 = newSolarSystem("SS3", 200, 100);
            planet3 = newPlanet("SS3-A", ss3, null, 100, 0.8);

            ss4 = newSolarSystem("SS4", 250, 100);
            planet4 = newPlanet("SS4-A", ss4, null, 110, 1.0);

            ss5 = newSolarSystem("SS5", 100, 200);
            
            ss6 = newSolarSystem("SS6", 100, 250);
            planet6 = newPlanet("SS6-A", ss6, player3);

            ss7 = newSolarSystem("SS7", 100, 300);
            planet7a = newPlanet("SS7-A", ss7, null, 80, 0.5);
            planet7b = newPlanet("SS7-B", ss7, null, 80, 0.5);
            planet7c = newPlanet("SS7-C", ss7, null, 80, 0.5);
            planet7d = newPlanet("SS7-D", ss7, null, 80, 0.5);
            planet7e = newPlanet("SS7-E", ss7, null, 80, 0.5);
            planet7f = newPlanet("SS7-F", ss7, null, 80, 0.5);
            planet7g = newPlanet("SS7-G", ss7, null, 80, 0.5);
            planet7h = newPlanet("SS7-H", ss7, null, 80, 0.5);
            planet7i = newPlanet("SS7-I", ss7, null, 80, 0.5);
            
            ss2.setPrize(new MoneyPrize(250));
            ss3.setPrize(new TechnologyPrize(tech_massDriver1));
            ss4.setPrize(new CraftPrize());
            ss5.setPrize(new BoraliumPrize(7));
            
            ss8 = newSolarSystem("SS8", 200, 300);
            ss9 = newSolarSystem("SS9", 200, 400);
            
            blackHole = BlackHoleFactory.createBlackHole(new Location(500,500));  
            
            ss10 = newSolarSystem("SS10", 650, 100);
            planet10 = newPlanet("SS10-A", ss10, player1);
            Craft blockader = newCraft(bigFighterDesign, player3, planet10);

        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }

    private Craft craft1;
    private Craft craft2;

    private Fleet fleet1;
    private Craft craft3;

    private Fleet fleet2;
    private Craft craft4;
    private Craft craft5;

    private Craft craft6;

    private Craft attackCraft1;
    private Craft attackCraft2;
    private Craft bombardCraft1;
    
    private Craft surpriseAttackCraft1;
    private Craft surpriseAttackCraft2;
    private Craft surpriseAttackColoniser;
    
    private Craft defendCraft1;

    private Craft exploreCraft;
    private Craft coloniseCraft;
    
    private Craft maraudCraft1;
    private Craft maraudCraft2;
    private Craft maraudCraft3;
    
    private Craft protectCraft1;

    /**
     * Creates a game in a fixed state then saves and loads the game a tests that the
     * game is still in the same state.
     */
    public void testSaveLoad() throws Exception {
        Logger.logDebug("*******************************  SaveLoadTest.testSaveLoad() *************************");
        GeneralBasic general = (GeneralBasic)player1.getGeneral();
        
        // Test player details are saved correctly
        player1.setFunds(700);
        player1.setBoralium(71);
        player1.setLastDateAncientsDefended(3010);
        player1.addEngine(tech_fusion25x);
        player1.addWeapon(tech_massDriver2);
        player1.addShield(tech_energy10);
        player1.addSpecial(tech_armour25);
        player1.addScanner(tech_photon150);
        player1.addBiotechnology(tech_terraforming10);
        player1.addTheory(tech_researchLabI);
        player1.addFactory(tech_basicComputerisedFactory);
        player1.addLaboratory(tech_ComputerisedLabI);
        player1.addModule(tech_habitatI);
        player1.addHumanity(tech_trade_route);
        player1.addMiscTechnology(tech_planetConstruction);
        player1.getPlayerCharacter().setBattleSpeedBonus(1.1);
        player1.getPlayerCharacter().setBattleTurningBonus(1.2);
        player1.getPlayerCharacter().setMaxDamageBonus(1.05);
        player1.getPlayerCharacter().setSpeedBonus(1.3);
        player1.getPlayerCharacter().getBasicCharacter().setProductionBonus(1.5);
        player1.getPlayerCharacter().getBasicCharacter().setProductionLevelBonus(6.0);
        player1.getPlayerCharacter().getBasicCharacter().setResearchBonus(1.6);
        player1.getPlayerCharacter().getBasicCharacter().setResearchLevelBonus(6.0);
        player1.getPlayerCharacter().getBasicCharacter().setDevelopmentBonus(1.6);
        player1.getPlayerCharacter().getBasicCharacter().setDevelopmentLevelBonus(6.0);
        player1.getPlayerCharacter().setReproductionBonus(1.7);
        player1.getPlayerCharacter().setAssaultBonus(1.25);
        player1.getPlayerCharacter().setInitiativeBonus(1.8);
        player1.getPlayerCharacter().setSpyDefendPercent(0.5);
        player1.getPlayerCharacter().setSpyBonus(0.9);
        player1.getPlayerCharacter().setTradingBonus(1.9);
        player1.getPlayerCharacter().setEcologyBonus(0.7);
        player1.getPlayerCharacter().setFactoryPerPopulationBonus(2.0);
        player1.getPlayerCharacter().setPsiRating(20);
        player1.getPlayerCharacter().setMaxPlayerTradeRoutes(6);
        player1.getPlayerCharacter().setMaxPlanetTradeRoutes(7);
        player1.getPlayerCharacter().setCharisma(5);
        player1.getPlayerCharacter().setCloakLevel(10);
        player1.getPlayerCharacter().setRelation(new Relation(player1, player2, 55));
        player1.getPlayerCharacter().setCloakLevel(10);
        player1.getPlayerCharacter().setFactoryPerPopulationBonus(2.0);
        player1.getPlayerCharacter().setBlockadeOrAttackProbability(0.4);
        player1.getPlayerCharacter().setAcceptAlianceProbabilityWhenFond(0.1);
        player1.getPlayerCharacter().setAcceptAlianceProbabilityWhenLike(0.2);
        player1.getPlayerCharacter().setAcceptPeaceProbability(0.3);
        player1.getPlayerCharacter().setQtyCraftPerPlanetToDeclareWar(4);
        player1.getPlayerCharacter().setMaxTimeBetweenFleets(30);
        player1.getPlayerCharacter().setCraftsSpecialWeapons((String[])null);
        player1.getPlayerCharacter().setCraftsSpecialWeapons("XXX");
        player1.getPlayerCharacter().setCraftsSpecialWeapons("YYY");
        player1.getPlayerCharacter().setCraftsWantPsiShields(true);
        player1.getPlayerCharacter().setResourceful(true);
        player1.getPlayerCharacter().setImmunity(null);
        player1.setQtyBattles(25);
        player1.setQtyBattlesWon(26);
        player1.setChanceHoloCraftWork(0.55f);
        GeneralTechnologyList missed = new GeneralTechnologyList();
        missed.add(tech_ComputerisedLabI);
        missed.add(tech_basicComputerisedFactory);
        player1.getPlayerCharacter().setMissedTechnologies(missed);
        player1.setSalvage(new SalvageList(tech_fusionBeam4));
        player1.getCraftBuildExperienceNotNull().addDesignExperience(tech_flyer60.getName());
        
        GeneralDetails generalDetails = GeneralDetailsList.getInstance().getGeneralDetails("TestGeneral");
        mot.engine.leaders.LeaderFactory.addGeneralLeaderTo(player1, generalDetails);
        GovenorDetails govenorDetails = GovenorDetailsList.getInstance().getGovenorDetails("TestGovenor");
        mot.engine.leaders.LeaderFactory.addGovenorLeaderTo(player1, govenorDetails);
        
        StrategistEventManager sem = player3.getStratagist().getEventManagerNotNull();
        sem.add(new StrategistEventCaughtSpying(player2, tech_antiMissileBeam));
        sem.add(new StrategistEventCaughtSabotaging(player2));
        sem.add(new StrategistEventDeclareWar(player2));
        sem.add(new StrategistEventDeclareWarOnEnemy(player2, player1));
        sem.add(new StrategistEventColonyLost(player2, "PlanetName", true));
        sem.add(new StrategistEventRequestAllianceTreaty(player2, 10));
        sem.add(new StrategistEventRequestNonAgressionTreaty(player2, 11));
        sem.add(new StrategistEventRequestPeaceTreaty(player2, 12));
        sem.add(new StrategistEventRequestResearchTreaty(player2, 13));
        sem.add(new StrategistEventRequestTradeTreaty(player2, 14));
        sem.add(new StrategistEventDemandAssets(player2, 11, false, "XX"));
        sem.add(new StrategistEventAnnoy(player2));
        sem.add(new StrategistEventGiveAssets(player2, 88, "Money 500"));
        sem.add(new StrategistEventRejectSwapAssets(player2, 10, 20));
        sem.add(new StrategistEventTreatyRejected(player2, "XXX"));
        sem.add(new StrategistEventBrokeNonAggression(player2, "Peace"));
        sem.add(new StrategistEventUsedBiologicalWeapons(player2));
        sem.add(new StrategistEventInvasion(player3, player2, "PlanetName", "PlayerVictim"));
        sem.add(new StrategistEventGenocide(player2, "PlanetName"));
        player1.add(MoralChoices.getMoralChoice(TEST_MORAL_CHOICE));
        player3.getStratagist().notifyRelationsEstablished(player1, true);
        player1.setQtyAdditionalSpies(21);
        player1.setQtyBaseSpies(23);

        StrategistEventManager sem2 = player4.getStratagist().getEventManagerNotNull();
        sem2.add(new StrategistEventCaughtSpying(deadPlayer, tech_armour25));
        sem2.add(new StrategistEventCaughtSabotaging(deadPlayer));
        sem2.add(new StrategistEventDeclareWar(deadPlayer));
        sem2.add(new StrategistEventDeclareWarOnEnemy(player4, deadPlayer));
        sem2.add(new StrategistEventColonyLost(deadPlayer, "PlanetName", true));
        sem2.add(new StrategistEventRequestAllianceTreaty(deadPlayer, 1));
        sem2.add(new StrategistEventRequestNonAgressionTreaty(deadPlayer, 2));
        sem2.add(new StrategistEventRequestPeaceTreaty(deadPlayer, 3));
        sem2.add(new StrategistEventRequestResearchTreaty(deadPlayer, 4));
        sem2.add(new StrategistEventRequestTradeTreaty(deadPlayer, 5));
        sem2.add(new StrategistEventDemandAssets(deadPlayer, 11, false, "XXX"));
        sem2.add(new StrategistEventAnnoy(deadPlayer));
        sem2.add(new StrategistEventGiveAssets(deadPlayer, 88, "Money 400"));
        sem2.add(new StrategistEventRejectSwapAssets(deadPlayer, 10, 20));
        sem2.add(new StrategistEventTreatyRejected(deadPlayer, "XXX"));
        sem2.add(new StrategistEventBrokeNonAggression(deadPlayer, "Non Agression"));

        userPlayer.setSpyRecruitEffortPerTurn(25);
        userPlayer.getSpyCenter().setQtyBuiltSpies(50);
        userPlayer.getSpyCenter().setTotalEffort(250);
        userPlayer.setFleetWayPoint(new Orbit(planet2));
        userPlayer.addSalvage(tech_photon100);
        userPlayer.addSalvage(tech_armour200);
        userPlayer.addSalvage(tech_flyer100);
        userPlayer.addSalvage(tech_shieldBurst);
        userPlayer.addSalvage(tech_teleport);
        
        GameScoreData gsd = new GameScoreData();
        gsd.addCraftBuilt(tech_flyer30, 6);
        gsd.addCraftBuilt(tech_flyer60, 1);
		userPlayer.setGameScoreData(gsd);
        
        player2.getPlayerCharacter().setResourceful(false);
        player2.getPlayerCharacter().setImmunity(new ImmunityBio());
        player2.getPlayerCharacter().setImmunity(new ImmunityMindControl());
        player2.getPlayerCharacter().setImmunity(new ImmunityGravity());
        player2.getPlayerCharacter().setImmunity(new ImmunityPsi());
        player2.getPlayerCharacter().setImmunity(new ImmunityReplicaBeam());

        // Test Relations and treaties
        MotEngine engine = MotEngine.instance;
		Relations rels = engine.getRelations();
        Relation r = rels.getRelationshipNotNull(player1, player2);
        r.setValue(65);
        TradeAgreementTreaty tt = new TradeAgreementTreaty(player1, player2);
        tt.setDateStarted(2999);
        tt.setResource1(Resources.antiMatter);
        tt.setResource2(Resources.cloakPalm);
        r.addTreaty(tt);
        ResearchAgreementTreaty rt = new ResearchAgreementTreaty(player1, player2);
        rt.setDateStarted(2998);
        rt.setMissedTechnologies1(new GeneralTechnologyList(tech_teleport));
        rt.setMissedTechnologies2(new GeneralTechnologyList(tech_armour200));
        r.addTreaty(rt);
        NonAgressionTreaty nat = new NonAgressionTreaty();
        nat.setDateStarted(2997);
        r.addTreaty(nat);
        rels.setAliance(player1, player4);
        ((AllianceTreaty)rels.getRelationshipNotNull(player1, player4).getTreatiesAsArray()[0]).setDateStarted(2996);
        rels.setWar(player1, player3);
        
        GalaxyDetails gd = engine.getStellarObjects().getGalaxyDetails();
        gd.setSize(new Size(900,900));
        engine.getStellarObjects().getGalaxyDetails().setNebulae(new Nebula("Nebula1", 600,602, 102));
        
        engine.setDate(3000);
        engine.setSpaceKaanEnabled(true);
        engine.setLevel(1.5);

        // Create Craft in Orbit
        craft1 = newCraft(fighterDesign, player1, ss1);

        // Create Craft at warp between ss1 and ss7
        craft2 = newCraft(fighterDesign, player1, ss1);
        craft2.moveTo(ss7.getOrbit());
        craft2.endTurn();
        craft2.setHasMoved(true);

        // Create fleet containing craft3 in orbit of ss2
        craft3 = newCraft(fighterDesign, player2, ss2);
        fleet1 = FleetFactory.create(craft3);
        
        // Create fleet containing craft3 and craft4 at warp to ss3
        craft4 = newCraft(fighterDesign, player2, ss2);
        craft5 = newCraft(fighterDesign, player2, ss2);
        fleet2 = FleetFactory.create(craft4);
        fleet2.add(craft5);
        fleet2.moveTo(ss3.getOrbit());
        fleet2.endTurn();
        fleet2.setHasMoved(true);

        // Create in orbit around planet with command to move to a planet
        craft6 = newCraft(fighterDesign, player1, planet1);
        craft6.moveTo(planet2.getOrbit());

        // Test the explore task is saved correctly
		CraftDesign explorerDesign = new CraftDesign("TestExplorer", tech_flyer30, tech_fusion25);
        CraftTask exploreTask = new CraftTaskExplore(player1, 200);
        exploreCraft = newCraft(explorerDesign, player1, ss1);
        general.add(exploreTask);
        exploreTask.add(exploreCraft);
        general.assignTaskToCraft(exploreCraft);
        exploreCraft.endTurn();
        
        // Test Colonise Task
        CraftTask coloniseTask = new CraftTaskColonise(planet3, player1);
        coloniseCraft = newCraft(coloniserDesign, player1, ss1);
        general.add(coloniseTask);
        coloniseTask.add(coloniseCraft);
        coloniseCraft.endTurn();
        
        // Test Attack Task
        CraftTask attackTask = new CraftTaskAttack(planet6, player1, 2, planet7a);
        attackCraft1 =  newCraft(fighterDesign, player1, ss1);
        attackCraft2 =  newCraft(fighterDesign, player1, ss1);
        general._add(attackTask);
        attackTask.add(attackCraft1);
        attackTask.add(attackCraft2);
        //general.assignTaskToFleet(attackCraft1.getFleet());
        //attackCraft1.getFleet().endTurn();
   
        CraftTask surpriseAttack = new CraftTaskSurpriseAttackStealColony(planet6, player1);
        surpriseAttackColoniser =  newCraft(coloniserDesign, player1, ss1);
        surpriseAttackCraft1 =  newCraft(fighterDesignNoShield, player1, ss1);
        surpriseAttackCraft2 =  newCraft(fighterDesignNoShield, player1, ss1);
        general._add(surpriseAttack);
        surpriseAttack.add(surpriseAttackColoniser);
        surpriseAttack.add(surpriseAttackCraft1);
        surpriseAttack.add(surpriseAttackCraft2);

        // Test Bombard Task
        CraftTask bombardTask = new CraftTaskBombard(player1, planet7a, null);
        bombardCraft1 =  newCraft(fighterDesign, player1, ss1);
        bombardTask.add(bombardCraft1);
        general._add(bombardTask);
    
        // Test Defend Task
        CraftTaskDefend defendTask = new CraftTaskDefend(ss6, player3);
        defendCraft1 = newCraft(fighterDesign, player3, ss6);
        player3.getGeneral().add(defendTask);
        defendTask.add(defendCraft1);
        defendTask.setQtyBattlesLost(6);
        defendCraft1.endTurn();
        
        // Test basic planet characteristics
        planet1.setSize(110);
        planet1.setTeraforming(0.98);
        planet1.setPopulation(25); // 5 colonists moved onto transport!
        planet1.setMoneyBonus(5);
        planet1.setReproductionBonus(1.1);
        planet1.setProductionBonus(1.2);
        planet1.setResearchBonus(1.3);
        planet1.setQtyFactories(15);
        planet1.setQtyGroundDefense(9);
        planet1.setHasAsteroids(true);
        planet1.setInfluence(99.9);
        planet1.setHasNatives(true);
        planet1.setQtyNatives(15);
        planet2.setOrigin(player1);
        planet2.setHasNatives(false);
        
        // Test that planet weapons, shields and scanners are saved correctly
        AsteroidFieldConstruction asteroidField = (AsteroidFieldConstruction) Technologies.getMiscTechnology(TestTechs.MISC_ASTEROIDS);
        Weapons weapons = new Weapons();
        weapons.add(tech_massDriver1);
        weapons.add(tech_massDriver1);
        planet1.setWeapons(weapons);
        planet1.setShields(new Shields(tech_energy10));
        planet1.setScanners(new Scanners(tech_photon100));
        
        // Test planet 1 with mining station on planet 1 mine
        Orbital mine = new Orbital(tech_spaceStation, tech_photon100, null);
        mine.addModule(tech_habitatI);
        mine.addModule(tech_habitatII);
        mine.addModule(tech_oreExtractor);
        mine.addModule(tech_researchModule);
        planet1mine.setOrbital(mine);
        planet1.setMiningStation(new OrbitalReference(planet1mine));
        
        // Test that planet build craft and build weapon is saved correctly
        planet1.setProject(new ProjectCraft(fighterDesign, planet1, false));
        planet1.setProduction(50);
        planet1.setRepeatProject(5);
        
        planet2.setProject(new ProjectWeapon(new WeaponDesign(tech_massDriver1)));
        planet3.setWeapons(new Weapons(tech_massDriver1));
        //planet3.setProject(new ProjectWeaponUpgrade(new WeaponUpgradeDesign(tech_massDriver1, tech_fusionBeam4, planet3.getWeapons())));
        planet4.setProject(new ProjectShield(new ShieldDesign(tech_energy10)));
        planet6.setShields(new Shields(tech_energy10));
        //planet6.setProject(new ProjectShieldUpgrade(new ShieldUpgradeDesign(tech_energy10, tech_energy20, planet6.getShields())));
        planet7a.setProject(new ProjectBattleStation(new BattleStationDesign(tech_battleStationI)));
        //planet7b.setProject(new ProjectBattleStationUpgrade(new BattleStationUpgradeDesign(tech_battleStationI, tech_battleStationII)));
        planet7c.setProject(new ProjectScanner(new ScannerDesign(tech_photon100)));
        planet7d.setScanners(new Scanners(tech_photon100));
        //planet7d.setProject(new ProjectScannerUpgrade(new ScannerUpgradeDesign(tech_photon100, tech_photon150, planet7d.getScanners())));
        planet7e.setProject(new ProjectOutpost(new OutpostDesign(tech_outpost), ss2));
        planet7f.setProject(new ProjectSpaceStation(new SpaceStationDesign(tech_spaceStation), new Orbit(planet2)));
        planet7g.setProject(new ProjectAsteroids(asteroidField));
        planet7h.setProject(new ProjectConstructPlanet(tech_planetConstruction, "SS1"));
        CraftVO upgradeCraftVO = new CraftVO("OldTestFighter", player1.getName(), null, null, null, 0, 0, false, null, null, new CraftDesign("OldTestFighter",  tech_flyer30,tech_fusion25, tech_massDriver1), 5, 0, 0, false, false);
        planet7i.setProject(new ProjectCraftUpgrade(new CraftUpgradeDesign(fighterDesign, upgradeCraftVO), planet1));
        
        planet1.setOrbital(new Orbital(tech_battleStationI, null, new Modules(tech_habitatI)));
        
        TradeRouteFactory.create(ss4, ss6);
        
        // Test outpost
        Orbital outpostOrbital = new Orbital(tech_outpost, tech_photon150, null);
        ss1.setOutpost(new Outpost(outpostOrbital, player1));
        
        // Test Maraud Task
        maraudCraft1 = newCraft(fighterDesign, player3, ss6);
        maraudCraft2 = newCraft(fighterDesign, player3, ss6);
        Fleet fleet = FleetFactory.create(maraudCraft1);
        fleet.add(maraudCraft2);        
        CraftTask maraudTask = new CraftTaskMaraud(player3, player1, fleet, 1);
        maraudTask.setTarget(new Orbit(planet1));
        maraudTask.setCreatedDate(5050);
        player3.getGeneral().add(maraudTask);

        MonsterPlayer monsterPlayer = MotEngine.getInstance().getMonsterPlayer();
        General monsterGeneral = monsterPlayer.getGeneral();

        maraudCraft3 = newCraft(fighterDesign, monsterPlayer, ss8);
        CraftTaskMaraudCraft maraudCraftTask = new CraftTaskMaraudCraft(monsterPlayer, maraudCraft3, null);
        maraudCraftTask.setTarget(new Orbit(ss6));
        maraudCraftTask.setArrivedDate(2500);
		monsterGeneral.add(maraudCraftTask);
        
        protectCraft1 = newCraft(fighterDesign, monsterPlayer, ss9);
        monsterGeneral._add(new CraftTaskProtect(ss9, monsterPlayer, protectCraft1));
        
        // Test the player statistics
        Graph graph = new Graph();
        player1.setGraph(graph);
        graph.add("TESTLINE1", 3000, 10);
        graph.add("TESTLINE1", 3010, 20);
        graph.add("TESTLINE2", 2, 1);
        
        // Test resources
        planet1mine.setResource(new ResourceAntiMatter());
        planet1.setResource(new ResourceWarpCrystal());
        planet2.setResource(new ResourceCloakPalm());
        planet3.setResource(new ResourceTritanOre());
        planet4.setResource(new ResourcePsiFungus());
        planet7a.setResource(new ResourceBoralium());
        planet7b.setResource(new ResourceEnergySpores());
        
        // Test transport enroute to target
        player1.getTransportManagerNotNull().setTransport(new Transport(player1));
        player1.getTransportManager().setTransport(new Transport(player1));
        player1.getTransportManager().setTransport(new Transport(player1));
        player1.getTransportManager().setTransport(new Transport(player1));
        player1.getTransportManager().setTransport(new Transport(player1));
        player1.getTransportManager().setTransport(new Transport(player1));
        
        // Test Crisis
        CrisisFactory.createPlagueCrisis(planet7a); 
        CrisisFactory.createPsiCrisis(planet7b); 
        CrisisFactory.createEmergencyCrisis(planet7c); 
        
        //new ProjectOutpost(tech_outpost, ss2).build(planet1, false);
        createOutpost(tech_outpost, ss9, planet1);
        Transport outpostTransport = player1.getTransportManager().getTransportsAsArray()[0];
        TransportTaskDeployStation outpostTask = (TransportTaskDeployStation)outpostTransport.getTask();
        outpostTransport.moveTo(ss9.getOrbit());
        outpostTransport.endTurn(); 

        new ProjectSpaceStation(new SpaceStationDesign(tech_spaceStation), new Orbit(planet4)).build(planet1, false);
        Transport stationTransport = player1.getTransportManager().getTransportsAsArray()[1];
        TransportTaskDeployStation stationTask = (TransportTaskDeployStation)stationTransport.getTask();
        stationTransport.moveTo(planet4.getOrbit());
        
        // Test transports
        double tmpPlanet1Pop = planet1.getPopulation();        
        Transport populationTransport = player1.getTransportManager().getFreeTransport();
        new TransportTaskMoveColonists(populationTransport, planet1, planet2, 5);
        planet1.setPopulation(tmpPlanet1Pop);
 
        Transport tradeRouteTransport = player1.getTransportManager().getFreeTransport();
        new TransportTaskCreateTradeRoute(tradeRouteTransport, ss1, ss2);
 
        // test black hole
        blackHole.setOrbital(new SpaceStation(new Orbital(tech_spaceStation, tech_photon100, new Modules(tech_researchModule)), player1));
        blackHole.setResearchBonus(0.1f);
        blackHole.setSpaceJumpAbility(true);
        
        Spies spies = MotEngine.instance.getSpiesNotNull();
        spies.setSpyRoleDefend(new SpyRoleDefend(player1, 1));
        spies.setSpyRoleEspionage(new SpyRoleEspionage(player1, player2, 2));
        spies.setSpyRoleSabotage(new SpyRoleSabotage(player1, player2, 3));
        spies.getEspionageSpies(player1, player2).setTechnology(tech_SalvageAnalyser);
        
        engine.logSummary();
        
        _testState();
        
        File file = new File("test.mot");
        
        engine.save(file);
        MotEngine.load(file);

        engine = MotEngine.instance;
        
        player1 = (ComputerPlayer) engine.getPlayers().getPlayer("TestPlayer1");
        player2 = (ComputerPlayer) engine.getPlayers().getPlayer("TestPlayer2");
        player3 = (ComputerPlayer) engine.getPlayers().getPlayer("TestPlayer3");
        userPlayer = (UserPlayer) engine.getPlayers().getPlayer("TestUserPlayer");
        
        ss1 = engine.getStellarObjects().getSolarSystem("SS1");
        ss2 = engine.getStellarObjects().getSolarSystem("SS2");
        ss3 = engine.getStellarObjects().getSolarSystem("SS3");
        ss4 = engine.getStellarObjects().getSolarSystem("SS4");
        ss5 = engine.getStellarObjects().getSolarSystem("SS5");
        ss6 = engine.getStellarObjects().getSolarSystem("SS6");
        ss7 = engine.getStellarObjects().getSolarSystem("SS7");
        ss8 = engine.getStellarObjects().getSolarSystem("SS8");
        ss9 = engine.getStellarObjects().getSolarSystem("SS9");
        blackHole = engine.getStellarObjects().getBlackHole("Black Hole1");
        
        planet1 = engine.getPlanets().getPlanet("SS1-A");
        planet1mine = engine.getPlanets().getPlanet("SS1-MINE");
        planet2 = engine.getPlanets().getPlanet("SS2-A");
        planet3 = engine.getPlanets().getPlanet("SS3-A");
        planet4 = engine.getPlanets().getPlanet("SS4-A");
        planet6 = engine.getPlanets().getPlanet("SS6-A");
        planet7a = engine.getPlanets().getPlanet("SS7-A");
        planet7b = engine.getPlanets().getPlanet("SS7-B");
        planet7c = engine.getPlanets().getPlanet("SS7-C");
        planet7d = engine.getPlanets().getPlanet("SS7-D");
        planet7e = engine.getPlanets().getPlanet("SS7-E");
        planet7f = engine.getPlanets().getPlanet("SS7-F");
        planet7g = engine.getPlanets().getPlanet("SS7-G");
        planet7i = engine.getPlanets().getPlanet("SS7-I");
                
        craft1 = engine.getCrafts().getCraft("TestFighter1");
        craft2 = engine.getCrafts().getCraft("TestFighter2");
        craft3 = engine.getCrafts().getCraft("TestFighter3");
        craft4 = engine.getCrafts().getCraft("TestFighter4");
        craft5 = engine.getCrafts().getCraft("TestFighter5");
        craft6 = engine.getCrafts().getCraft("TestFighter6");
        attackCraft1 = engine.getCrafts().getCraft("TestFighter7");
        attackCraft2 = engine.getCrafts().getCraft("TestFighter8");
        surpriseAttackColoniser = engine.getCrafts().getCraft("Coloniser2");
        surpriseAttackCraft1 = engine.getCrafts().getCraft("TestFighterNoShield1");
        surpriseAttackCraft2 = engine.getCrafts().getCraft("TestFighterNoShield2");
        attackCraft2 = engine.getCrafts().getCraft("TestFighter8");
        bombardCraft1 = engine.getCrafts().getCraft("TestFighter9");
        defendCraft1 = engine.getCrafts().getCraft("TestFighter10");
        maraudCraft1 = engine.getCrafts().getCraft("TestFighter11");
        maraudCraft2 = engine.getCrafts().getCraft("TestFighter12");
        maraudCraft3 = engine.getCrafts().getCraft("TestFighter13");
        protectCraft1 = engine.getCrafts().getCraft("TestFighter14");
        
        fleet1 = player2.getFleet("Fleet1");
        fleet2 = player2.getFleet("Fleet2");

        exploreCraft = engine.getCrafts().getCraft("TestExplorer1");
        coloniseCraft = engine.getCrafts().getCraft("Coloniser1");
        
        _testState();
      
    }
    
    private void _testState() throws Exception {
        MotEngine engine = MotEngine.instance;

        assertEquals("player1 is correct", 700, player1.getFunds());
        assertEquals("player1 is correct", 71, player1.getBoralium());
        assertEquals("player1 is correct", 3010, player1.getLastDateAncientsDefended());
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_fusion25));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_fusion25x));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_massDriver1));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_massDriver2));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_energy10));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_armour25));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_photon150));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_terraforming10));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_researchLabI));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_basicComputerisedFactory));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_ComputerisedLabI));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_habitatI));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_trade_route));
        assertEquals("player1 is correct", true, player1.hasDiscovered(tech_planetConstruction));
        assertEquals("player1 is correct", 1.1, player1.getPlayerCharacter().getBattleSpeedBonus(), 0.0);
        assertEquals("player1 is correct", 1.2, player1.getPlayerCharacter().getBattleTurningBonus(), 0.0);
        assertEquals("player1 is correct", 1.05, player1.getPlayerCharacter().getMaxDamageBonus(), 0.0);
        assertEquals("player1 is correct", 1.3, player1.getPlayerCharacter().getSpeedBonus(), 0.0);
        assertEquals("player1 is correct", 1.5, player1.getPlayerCharacter().getProductionBonus(false), 0.0);
        assertEquals("player1 is correct", 7.5, player1.getPlayerCharacter().getProductionBonus(true), 0.0);
        assertEquals("player1 is correct", 1.6, player1.getPlayerCharacter().getResearchBonus(false), 0.0);
        assertEquals("player1 is correct", 7.6, player1.getPlayerCharacter().getResearchBonus(true), 0.0);
        assertEquals("player1 is correct", 1.6, player1.getPlayerCharacter().getDevelopmentBonus(false), 0.0);
        assertEquals("player1 is correct", 7.6, player1.getPlayerCharacter().getDevelopmentBonus(true), 0.0);
        assertEquals("player1 is correct", 1.7, player1.getPlayerCharacter().getReproductionBonus(), 0.0);
        assertEquals("player1 is correct", 1.25, player1.getPlayerCharacter().getAssaultBonus(), 0.0);
        assertEquals("player1 is correct", 1.8, player1.getPlayerCharacter().getInitiativeBonus(), 0.0);
        assertEquals("player1 is correct", 0.5, player1.getPlayerCharacter().getSpyDefendPercent(), 0.0);
        assertEquals("player1 is correct", 0.9, player1.getPlayerCharacter().getSpyBonus(), 0.0);
        assertEquals("player1 is correct", 1.9, player1.getPlayerCharacter().getTradingBonus(), 0.0);
        assertEquals("player1 is correct", 0.7, player1.getPlayerCharacter().getEcologyBonus(), 0.0);
        assertEquals("player1 is correct", 2.0, player1.getPlayerCharacter().getFactoryPerPopulationBonus(), 0.0);
        assertEquals("player1 is correct", 0.4, player1.getPlayerCharacter().getBlockadeOrAttackProbability(), 0.0);
        assertEquals("player1 is correct", 0.1, player1.getPlayerCharacter().getAcceptAlianceProbabilityWhenFond(), 0.0);
        assertEquals("player1 is correct", 0.2, player1.getPlayerCharacter().getAcceptAlianceProbabilityWhenLike(), 0.0);
        assertEquals("player1 is correct", 0.3, player1.getPlayerCharacter().getAcceptPeaceProbability(), 0.0);
        assertEquals("player1 is correct", 4, player1.getPlayerCharacter().getQtyCraftPerPlanetToDeclareWar());
        assertEquals("player1 is correct", 30, player1.getPlayerCharacter().getMaxTimeBetweenFleets());
        assertEquals("player1 is correct", 20, player1.getPlayerCharacter().getPsiRating());
        assertEquals("player1 is correct", 6, player1.getPlayerCharacter().getMaxPlayerTradeRoutes());
        assertEquals("player1 is correct", 7, player1.getPlayerCharacter().getMaxSolarSystemTradeRoutes());
        assertEquals("player1 is correct", 5, player1.getPlayerCharacter().getCharisma());
        assertEquals("player1 is correct", 10, player1.getPlayerCharacter().getCloakLevel());
        assertEquals("player1 is correct", 55, player1.getPlayerCharacter().getRelationshipRatingTo(player2));
        assertEquals("player1 is correct", 2, player1.getPlayerCharacter().getMissedTechnologies().getQuantity());
        assertEquals("player1 is correct", true, player1.getPlayerCharacter().getMissedTechnologies().contains(Technologies.getFactory(TestTechs.FACT_COMPUTERISEDI)));
        assertEquals("player1 is correct", "XXX", player1.getPlayerCharacter().getCraftsSpecialWeapons()[0]);
        assertEquals("player1 is correct", "YYY", player1.getPlayerCharacter().getCraftsSpecialWeapons()[1]);
        assertEquals("player1 is correct", true, player1.getPlayerCharacter().getCraftsWantPsiShields());
        assertEquals("player1 is correct", true, player1.getPlayerCharacter().getIsResourceful());
        assertEquals("player1 is correct", null, player1.getPlayerCharacter().getImmunities());

        assertEquals("player1 is correct", 25, player1.getQtyBattles());
        assertEquals("player1 is correct", 26, player1.getQtyBattlesWon());
        assertEquals("player1 is correct", 0.55f, player1.getChanceHoloCraftWork(), 0.0);

        GeneralDetails generalDetails = GeneralDetailsList.getInstance().getGeneralDetails("TestGeneral");
        GovenorDetails govenorDetails = GovenorDetailsList.getInstance().getGovenorDetails("TestGovenor");
        assertEquals("player1 leaders are correct", generalDetails.getName(), player1.getGeneralLeader().getName());
        assertEquals("player1 leaders are correct", generalDetails, player1.getGeneralLeader().getDetails());
        assertEquals("player1 leaders are correct", govenorDetails.getName(), player1.getGovenorLeader().getName());
        assertEquals("player1 leaders are correct", govenorDetails, player1.getGovenorLeader().getDetails());
        
        assertEquals("user player is correct", 25, userPlayer.getSpyRecruitEffortPerTurn());
        assertEquals("user player is correct", 50, userPlayer.getSpyCenter().getQtyBuiltSpies());
        assertEquals("user player is correct", 250, userPlayer.getSpyCenter().getTotalEffort());
        assertEquals("user player is correct", planet2, userPlayer.getFleetWayPoint().getPlanet());
        assertEquals(tech_photon100, tech_armour200, tech_flyer100, tech_shieldBurst, tech_teleport, userPlayer.getSalvage().getAllAsArray());

        GameScoreData gsd = userPlayer.getGameScoreData();
        assertEquals("user player is correct", 6, gsd.getCraftBuilt(tech_flyer30).getQty());
        assertEquals("user player is correct", 1, gsd.getCraftBuilt(tech_flyer60).getQty());

        assertEquals("player1 is correct", false, player2.getPlayerCharacter().getIsResourceful());
        assertEquals("player1 is correct", 5, player2.getPlayerCharacter().getImmunities().length);
        assertEquals("player1 is correct", ImmunityBio.class, player2.getPlayerCharacter().getImmunities()[0].getClass());
        assertEquals("player1 is correct", ImmunityMindControl.class, player2.getPlayerCharacter().getImmunities()[1].getClass());
        assertEquals("player1 is correct", ImmunityGravity.class, player2.getPlayerCharacter().getImmunities()[2].getClass());
        assertEquals("player1 is correct", ImmunityPsi.class, player2.getPlayerCharacter().getImmunities()[3].getClass());
        assertEquals("player1 is correct", ImmunityReplicaBeam.class, player2.getPlayerCharacter().getImmunities()[4].getClass());

        assertEquals("player1 is correct", true, player1.getSalvage().contains(tech_fusionBeam4));
        assertEquals("player1 is correct", null, player1.getCraftBuildExperience().getDesignExperience(tech_flyer30.getName()));
        assertEquals("player1 is correct", 1, player1.getCraftBuildExperience().getDesignExperience(tech_flyer60.getName()).getValue());

		assertEquals("Have dead player", 1, engine.getDeadPlayers().getPlayers().length);
        assertEquals("Have dead player", "DeadPlayer", engine.getDeadPlayers().getPlayers()[0].getName());
        
        StrategistEventManager sem = player3.getStratagist().getEventManagerNotNull();
        assertEquals("Has correct events", 22, sem.getEventsAsArray().length);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[1] instanceof StrategistEventCaughtSpying);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[2] instanceof StrategistEventCaughtSabotaging);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[3] instanceof StrategistEventDeclareWar);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[4] instanceof StrategistEventDeclareWarOnEnemy);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[5] instanceof StrategistEventColonyLost);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[6] instanceof StrategistEventRequestAllianceTreaty);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[7] instanceof StrategistEventRequestNonAgressionTreaty);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[8] instanceof StrategistEventRequestPeaceTreaty);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[9] instanceof StrategistEventRequestResearchTreaty);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[10] instanceof StrategistEventRequestTradeTreaty);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[11] instanceof StrategistEventDemandAssets);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[12] instanceof StrategistEventAnnoy);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[13] instanceof StrategistEventGiveAssets);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[14] instanceof StrategistEventRejectSwapAssets);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[15] instanceof StrategistEventTreatyRejected);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[16] instanceof StrategistEventBrokeNonAggression);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[17] instanceof StrategistEventUsedBiologicalWeapons);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[18] instanceof StrategistEventInvasion);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[19] instanceof StrategistEventGenocide);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[20] instanceof StrategistEventMoralDecision);
        assertEquals("Has correct events", true, sem.getEventsAsArray()[21] instanceof StrategistEventRelationsEstablished);
       
        assertEquals("Has correct events", player2, ((StrategistEventDeclareWarOnEnemy)sem.getEventsAsArray()[4]).getPlayer());
        assertEquals("Has correct events", player1, ((StrategistEventDeclareWarOnEnemy)sem.getEventsAsArray()[4]).getCommonEnemy().getPlayer());
        assertEquals("Has correct events", "PlanetName", ((StrategistEventColonyLost)sem.getEventsAsArray()[5]).getPlanetName());
        assertEquals("Has correct events", true, ((StrategistEventColonyLost)sem.getEventsAsArray()[5]).isGenocide());
        assertEquals("Has correct events", 12, ((StrategistEventRequestPeaceTreaty)sem.getEventsAsArray()[8]).getNetAssetScore());
        assertEquals("Has correct events", 13, ((StrategistEventRequestResearchTreaty)sem.getEventsAsArray()[9]).getNetAssetScore());
        assertEquals("Has correct events", 14, ((StrategistEventRequestTradeTreaty)sem.getEventsAsArray()[10]).getNetAssetScore());
        assertEquals("Has correct events", 11, ((StrategistEventDemandAssets)sem.getEventsAsArray()[11]).getAssetScore());
        assertEquals("Has correct events", 88, ((StrategistEventGiveAssets)sem.getEventsAsArray()[13]).getRelationEffect());
        assertEquals("Has correct events", 10, ((StrategistEventRejectSwapAssets)sem.getEventsAsArray()[14]).getWantedAssetScore());
        assertEquals("Has correct events", 20, ((StrategistEventRejectSwapAssets)sem.getEventsAsArray()[14]).getOfferedAssetScore());
        assertEquals("Has correct events", 20, ((StrategistEventRejectSwapAssets)sem.getEventsAsArray()[14]).getOfferedAssetScore());
        assertEquals("Has correct events", "PlanetName", ((StrategistEventInvasion)sem.getEventsAsArray()[18]).getPlanetName());
        assertEquals("Has correct events", "PlanetName", ((StrategistEventGenocide)sem.getEventsAsArray()[19]).getPlanetName());
        assertEquals("Has correct events", player1, ((StrategistEventMoralDecision)sem.getEventsAsArray()[20]).getPlayer());
        assertEquals("Has correct events", MoralChoices.getMoralChoice(TEST_MORAL_CHOICE), ((StrategistEventMoralDecision)sem.getEventsAsArray()[20]).getMoralChoice());
        assertEquals("Has correct events", player1, ((StrategistEventRelationsEstablished)sem.getEventsAsArray()[21]).getPlayer());
        assertEquals("Has correct events", true, ((StrategistEventRelationsEstablished)sem.getEventsAsArray()[21]).isStarting());

        assertEquals("Have correct Moral Choice", 1, player1.getMoralChoices().getList().getAllAsArray().length);
        assertEquals("Have correct Moral Choice", MoralChoices.getMoralChoice(TEST_MORAL_CHOICE), player1.getMoralChoices().getList().getAllAsArray()[0]);
        assertEquals(23, player1.getQtyBaseSpies());
        assertEquals(21, player1.getQtyAdditionalSpies());
        
        StrategistEventManager sem2 = player4.getStratagist().getEventManagerNotNull();
        assertEquals("Has correct events", 18, sem2.getEventsAsArray().length);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[1] instanceof StrategistEventMoralDecision);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[2] instanceof StrategistEventCaughtSpying);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[3] instanceof StrategistEventCaughtSabotaging);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[4] instanceof StrategistEventDeclareWar);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[5] instanceof StrategistEventDeclareWarOnEnemy);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[6] instanceof StrategistEventColonyLost);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[7] instanceof StrategistEventRequestAllianceTreaty);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[8] instanceof StrategistEventRequestNonAgressionTreaty);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[9] instanceof StrategistEventRequestPeaceTreaty);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[10] instanceof StrategistEventRequestResearchTreaty);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[11] instanceof StrategistEventRequestTradeTreaty);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[12] instanceof StrategistEventDemandAssets);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[13] instanceof StrategistEventAnnoy);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[14] instanceof StrategistEventGiveAssets);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[15] instanceof StrategistEventRejectSwapAssets);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[16] instanceof StrategistEventTreatyRejected);
        assertEquals("Has correct events", true, sem2.getEventsAsArray()[17] instanceof StrategistEventBrokeNonAggression);
        
        assertEquals("Has correct events", player4, ((StrategistEventDeclareWarOnEnemy)sem2.getEventsAsArray()[5]).getPlayer());
        assertEquals("Has correct events", deadPlayer, ((StrategistEventDeclareWarOnEnemy)sem2.getEventsAsArray()[5]).getCommonEnemy().getPlayer());
        assertEquals("Has correct events", 11, ((StrategistEventDemandAssets)sem2.getEventsAsArray()[12]).getAssetScore());
        assertEquals("Has correct events", 88, ((StrategistEventGiveAssets)sem2.getEventsAsArray()[14]).getRelationEffect());
        assertEquals("Has correct events", 10, ((StrategistEventRejectSwapAssets)sem2.getEventsAsArray()[15]).getWantedAssetScore());
        assertEquals("Has correct events", 20, ((StrategistEventRejectSwapAssets)sem2.getEventsAsArray()[15]).getOfferedAssetScore());

        Relations rels = engine.getRelations();
        Relation r = rels.getRelationship(player1, player2);
        TradeAgreementTreaty tt = (TradeAgreementTreaty) r.getTreatiesAsArray()[0];
        ResearchAgreementTreaty rt = (ResearchAgreementTreaty) r.getTreatiesAsArray()[1];
        NonAgressionTreaty nat = (NonAgressionTreaty) r.getTreatiesAsArray()[2];
        assertEquals("Test relations and treaties", 65, r.getValue());
        assertEquals("Test relations and treaties", 2999, tt.getDateStarted());
        assertEquals(player1, tt.getTrader1());
        assertEquals(player2, tt.getTrader2());
        assertEquals(Resources.antiMatter, tt.getResource1());
        assertEquals(Resources.cloakPalm, tt.getResource2());
        assertEquals("Test relations and treaties", 2998, rt.getDateStarted());
        assertEquals(player1, rt.getResearcher1());
        assertEquals(player2, rt.getResearcher2());
        assertEquals(tech_teleport, rt.getMissedTechnologies1());
        assertEquals(tech_armour200, rt.getMissedTechnologies2());

        assertEquals("Test relations and treaties", 2997, nat.getDateStarted());
        assertEquals("Test relations and treaties", true, rels.isNonAgression(player1, player2));
        assertEquals("Test relations and treaties", false, rels.isAliance(player1, player2));
        assertEquals("Test relations and treaties", true, rels.isWar(player1, player3));
        assertEquals("Test relations and treaties", false, rels.isNonAgression(player1, player4));
        assertEquals("Test relations and treaties", true, rels.isAliance(player1, player4));
        r = rels.getRelationship(player1, player4);
        AllianceTreaty at = (AllianceTreaty) r.getTreatiesAsArray()[0];
        assertEquals("Test relations and treaties", 2996, at.getDateStarted());

        GalaxyDetails galaxyDetails = engine.getStellarObjects().getGalaxyDetails();
        Nebula[] nebulae = galaxyDetails.getNebulaeAsArray();
		assertEquals(900,900, galaxyDetails.getSize());
		assertEquals(1, nebulae.length);
		assertEquals(600,602, nebulae[0].getLocation());
		assertEquals(102, nebulae[0].getDiameter());
		
 		assertEquals(3000, engine.getDate());
		assertTrue(engine.isSpaceKaanEnabled());
		assertEquals(1.5, engine.getLevel());

        StellarObject[] sos = engine.getStellarObjects().getStellarObjects();
		assertEquals(11, sos.length);
		assertEquals("SS1", sos[0].getName());
		assertEquals(100,100, sos[0].getLocation());
		assertEquals("SS2", sos[1].getName());
		assertEquals(150,100, sos[1].getLocation());
		assertEquals("SS3", sos[2].getName());
		assertEquals(200,100, sos[2].getLocation());
		assertEquals("SS4", sos[3].getName());
		assertEquals(250,100, sos[3].getLocation());
		assertEquals("SS5", sos[4].getName());
		assertEquals(100,200, sos[4].getLocation());
		assertEquals("SS6", sos[5].getName());
		assertEquals(100,250, sos[5].getLocation());
		assertEquals("SS7", sos[6].getName());
		assertEquals(100,300, sos[6].getLocation());
		assertEquals("SS8", sos[7].getName());
		assertEquals(200,300, sos[7].getLocation());
		assertEquals("SS9", sos[8].getName());
		assertEquals(200,400, sos[8].getLocation());
		assertEquals("Black Hole1", sos[9].getName());
		assertEquals(500,500, sos[9].getLocation());
		assertEquals(0.1f, ((BlackHole)sos[9]).getResearchBonus());
		assertEquals(true, ((BlackHole)sos[9]).isSpaceJumpAbility());
		assertEquals("SS10", sos[10].getName());
		assertEquals(650,100, sos[10].getLocation());

        
        
        assertEquals("craft1 is correct", "TestFighter1", craft1.getName());
        assertEquals("craft1 is correct", ss1, craft1.getOrbit().getStellarObject());
        assertEquals("craft1 is correct", null, craft1.getNavigator());
        assertEquals("craft1 is correct", player1, craft1.getOwner());
        assertEquals("craft1 is correct", false, craft1.hasMoved());

        assertEquals("craft2 is correct", "TestFighter2", craft2.getName());
        assertNull(  "craft2 is correct",craft2.getOrbit());
        assertEquals("craft2 is correct", ss7, craft2.getNavigator().getDestination().getStellarObject());
        assertEquals("craft2 is correct", null, craft2.getNavigator().getDestination().getPlanet());
        assertEquals("craft2 is correct", player1, craft2.getOwner());
        assertEquals("craft2 is correct", true, craft2.hasMoved());

        assertEquals("fleet1 is correct", "Fleet1", fleet1.getName());
        assertEquals("fleet1 is correct", "TestFighter3", craft3.getName());
        assertEquals("fleet1 is correct", fleet1, craft3.getFleet());
        assertEquals("fleet1 is correct", ss2, craft3.getOrbit().getStellarObject());
        assertEquals("fleet1 is correct", null, craft3.getNavigator());
        assertEquals("fleet1 is correct", player2, craft3.getOwner());

        assertEquals("fleet2 is correct", "Fleet2", fleet2.getName());
        assertEquals("fleet2 is correct", "TestFighter4", craft4.getName());
        assertEquals("fleet2 is correct", "TestFighter5", craft5.getName());
        assertEquals("fleet2 is correct", fleet2, craft4.getFleet());
        assertEquals("fleet2 is correct", fleet2, craft5.getFleet());
        assertNull("fleet2 is correct", fleet2.getOrbit());
        assertEquals("fleet2 is correct", ss3, fleet2.getNavigator().getDestination().getStellarObject());
        assertEquals("fleet2 is correct", player2, craft3.getOwner());
        assertEquals("fleet2 is correct", true, fleet2.hasMoved());

        assertEquals("craft6 is correct", "TestFighter6", craft6.getName());
        assertEquals("craft6 is correct", ss1, craft6.getOrbit().getStellarObject());
        assertEquals("craft6 is correct", planet1, craft6.getOrbit().getPlanet());
        assertEquals("craft6 is correct", ss2, craft6.getNavigator().getDestination().getStellarObject());
        assertEquals("craft6 is correct", planet2, craft6.getNavigator().getDestination().getPlanet());
        assertEquals("craft6 is correct", player1, craft6.getOwner());

        assertEquals("Tasks are correct", 5, player1.getGeneral().getTasks().length);
        assertEquals("Tasks are correct", 2, player3.getGeneral().getTasks().length);

        assertEquals("Explore Craft is correct", "TestExplorer1", exploreCraft.getName());
        assertEquals("Explore Craft is correct", mot.engine.players.generals.CraftTaskExplore.class, exploreCraft.getTask().getClass());
        assertEquals("Explore Craft is correct", player1, exploreCraft.getTask().getOwner());
        assertNull("Explore Craft is correct", exploreCraft.getTask().getTarget());        
        assertEquals("Explore Craft is correct", null, exploreCraft.getTask().getFleet());
        assertEquals("Explore Craft is correct", ss5, exploreCraft.getNavigator().getDestination().getStellarObject());

        assertEquals("Colonise Craft is correct", "Coloniser1", coloniseCraft.getName());
        assertEquals("Colonise Craft is correct", mot.engine.players.generals.CraftTaskColonise.class, coloniseCraft.getTask().getClass());
        assertEquals("Colonise Craft is correct", player1, coloniseCraft.getTask().getOwner());        
        assertEquals("Colonise Craft is correct", planet3, coloniseCraft.getTask().getTarget());        
        assertEquals("Colonise Craft is correct", null, coloniseCraft.getTask().getFleet());
        assertEquals("Colonise Craft is correct", ss3, coloniseCraft.getNavigator().getDestination().getStellarObject());

        assertEquals("Attack Craft is correct", "TestFighter7", attackCraft1.getName());
        assertEquals("Attack Craft is correct", "TestFighter8", attackCraft2.getName());
        assertEquals("Attack Craft is correct", CraftTaskAttack.class, attackCraft1.getTask().getClass());
        assertEquals("Attack Craft is correct", CraftTaskAttack.class, attackCraft2.getTask().getClass());
        assertEquals("Attack Craft is correct", planet7a, ((CraftTaskAttack)attackCraft1.getTask()).getRoute());
        assertEquals("Attack Craft is correct", player1, attackCraft1.getTask().getOwner());        
        assertEquals("Attack Craft is correct", player1, attackCraft2.getTask().getOwner());        
        assertEquals("Attack Craft is correct", attackCraft1, attackCraft1.getTask().getFleet().getCrafts()[0]);
        assertEquals("Attack Craft is correct", attackCraft2, attackCraft2.getTask().getFleet().getCrafts()[1]);
        assertEquals("Attack Craft is correct", planet6, attackCraft1.getTask().getTarget());
        assertEquals("Attack Craft is correct", planet6, attackCraft2.getTask().getTarget());
        assertEquals("Attack Craft is correct", null, attackCraft1.getNavigator());
        assertEquals("Attack Craft is correct", null, attackCraft2.getNavigator());
        assertEquals("Attack Craft is correct", ss1, attackCraft1.getOrbit());
        assertEquals("Attack Craft is correct", ss1, attackCraft2.getOrbit());
     
        assertEquals("Surprise Attack Craft is correct", "Coloniser2", surpriseAttackColoniser.getName());
        assertEquals("Surprise Attack Craft is correct", "TestFighterNoShield1", surpriseAttackCraft1.getName());
        assertEquals("Surprise Attack Craft is correct", "TestFighterNoShield2", surpriseAttackCraft2.getName());
        assertEquals("Surprise Attack Craft is correct", planet6, ((CraftTaskSurpriseAttackStealColony)surpriseAttackColoniser.getTask()).getTarget());
        assertEquals("Surprise Attack Craft is correct", planet6, ((CraftTaskSurpriseAttackStealColony)surpriseAttackCraft1.getTask()).getTarget());
        assertEquals("Surprise Attack Craft is correct", planet6, ((CraftTaskSurpriseAttackStealColony)surpriseAttackCraft2.getTask()).getTarget());
        assertEquals("Surprise Attack Craft is correct", player1, surpriseAttackColoniser.getTask().getOwner());        
        assertEquals("Surprise Attack Craft is correct", player1, surpriseAttackCraft1.getTask().getOwner());        
        assertEquals("Surprise Attack Craft is correct", player1, surpriseAttackCraft2.getTask().getOwner());        
        assertEquals("Surprise Attack Craft is correct", ss1, surpriseAttackColoniser.getOrbit());
        assertEquals("Surprise Attack Craft is correct", ss1, surpriseAttackCraft1.getOrbit());
        assertEquals("Surprise Attack Craft is correct", ss1, surpriseAttackCraft2.getOrbit());

        assertEquals("Bombard Craft is correct", "TestFighter9", bombardCraft1.getName());
        assertEquals("Bombard Craft is correct", mot.engine.players.generals.CraftTaskBombard.class, bombardCraft1.getTask().getClass());
        assertEquals("Bombard Craft is correct", player1, bombardCraft1.getTask().getOwner());        
        assertEquals("Bombard Craft is correct", ss7, bombardCraft1.getTask().getTarget().getStellarObject());
        assertEquals("Bombard Craft is correct", planet7a, bombardCraft1.getTask().getTarget().getPlanet());
        assertEquals("Bombard Craft is correct", null, bombardCraft1.getNavigator());
        assertEquals("Bombard Craft is correct", ss1, bombardCraft1.getOrbit().getStellarObject());

        assertEquals("Defend Craft is correct", "TestFighter10", defendCraft1.getName());
        assertEquals("Defend Craft is correct", mot.engine.players.generals.CraftTaskDefend.class, defendCraft1.getTask().getClass());
        assertEquals("Defend Craft is correct", player3, defendCraft1.getTask().getOwner());        
        assertEquals("Defend Craft is correct", ss6, defendCraft1.getTask().getTarget());
        assertEquals("Defend Craft is correct", null, defendCraft1.getNavigator());
        assertEquals("Defend Craft is correct", ss6, defendCraft1.getOrbit().getStellarObject());
        assertEquals("Defend Craft is correct", 6, ((CraftTaskDefend)defendCraft1.getTask()).getQtyBattlesLost());

        assertEquals("Maraud is correct", "TestFighter11", maraudCraft1.getName());
        assertEquals("Maraud is correct", "TestFighter12", maraudCraft2.getName());
        assertEquals("Maraud is correct", maraudCraft1.getFleet(), maraudCraft2.getFleet());
        assertEquals("Maraud is correct", maraudCraft1.getTask(), maraudCraft2.getTask());
        assertEquals("Maraud is correct", mot.engine.players.generals.CraftTaskMaraud.class, maraudCraft1.getTask().getClass());
        assertEquals("Maraud is correct", mot.engine.players.generals.CraftTaskMaraud.class, maraudCraft1.getTask().getClass());
        assertEquals("Maraud is correct", ss6, maraudCraft1.getFleet().getOrbit().getStellarObject());
        assertEquals("Maraud is correct", player3, maraudCraft1.getFleet().getOwner());
        assertEquals("Maraud is correct", planet1, maraudCraft1.getTask().getTarget());
        assertEquals("Maraud is correct", 5050, maraudCraft1.getTask().getCreatedDate());
        assertEquals("Maraud is correct", player1, ((CraftTaskMaraud)maraudCraft1.getTask()).getTargetPlayer().getPlayer());

        MonsterPlayer monsterPlayer = MotEngine.getInstance().getMonsterPlayer();
        
        assertEquals("Maraud Craft is correct", "TestFighter13", maraudCraft3.getName());
        assertEquals("Maraud Craft is correct", mot.engine.players.generals.CraftTaskMaraudCraft.class, maraudCraft3.getTask().getClass());
        assertEquals("Maraud Craft is correct", ss8, maraudCraft3.getFleet().getOrbit().getStellarObject());
        assertEquals("Maraud Craft is correct", monsterPlayer, maraudCraft3.getFleet().getOwner());
        assertEquals("Maraud Craft is correct", ss6, maraudCraft3.getTask().getTarget());
        assertEquals("Maraud Craft is correct", ss6, ((CraftTaskMaraudCraft)maraudCraft3.getTask()).getTargetStellarObject());
        assertEquals("Maraud Craft is correct", 2500, ((CraftTaskMaraudCraft)maraudCraft3.getTask()).getArrivedDate().intValue());

        assertEquals("Protect Craft is correct", "TestFighter14", protectCraft1.getName());
        assertEquals("Protect Craft is correct", mot.engine.players.generals.CraftTaskProtect.class, protectCraft1.getTask().getClass());
        assertEquals("Protect Craft is correct", ss9, protectCraft1.getOrbit().getStellarObject());
        assertEquals("Protect Craft is correct", monsterPlayer, protectCraft1.getOwner());
        assertEquals("Protect Craft is correct", ss9, protectCraft1.getTask().getTarget());
        assertEquals("Protect Craft is correct", ss9, ((CraftTaskProtect)protectCraft1.getTask()).getTargetStellarObject());

        assertEquals("Test money prize", 250, ((MoneyPrize)ss2.getPrizes()[0]).getAmount());
        assertEquals("Test tech prize", tech_massDriver1, ((TechnologyPrize)ss3.getPrizes()[0]).getTech());
        assertEquals("Test craft prize", CraftPrize.class, ((CraftPrize)ss4.getPrizes()[0]).getClass());
        assertEquals("Test boralium prize", BoraliumPrize.class, ((BoraliumPrize)ss5.getPrizes()[0]).getClass());
        assertEquals("Test boralium prize", 7, ((BoraliumPrize)ss5.getPrizes()[0]).getAmount());

        assertEquals("Test planet", "SS1", planet1.getSolarSystem().getName());
        assertEquals("Test planet", 100, planet1.getSolarSystem().getLocation().getX());
        assertEquals("Test planet", 100, planet1.getSolarSystem().getLocation().getY());
        assertEquals("Test planet", 110, planet1.getSize());
        assertEquals("Test planet", 0.98, planet1.getTeraforming(), 0.0);
        assertEquals("Test planet", 25, planet1.getPopulation(), 0.0);
        assertEquals("Test planet", 5, planet1.getMoneyBonus());
        assertEquals("Test planet", 1.1, planet1.getReproductionBonus(), 0.0);
        assertEquals("Test planet", 1.2, planet1.getProductionBonus(), 0.0);
        assertEquals("Test planet", 1.3, planet1.getResearchBonus(), 0.0);
        assertEquals("Test planet", 15, planet1.getQtyFactories(), 0.0);
        assertEquals("Test planet", 9, planet1.getQtyGroundDefense());
        assertEquals("Test planet", true, planet1.getHasAsteroids());
        assertEquals("Test planet", 99.9, planet1.getInfluenceAsDouble(), 0.001);
        assertEquals("Test planet", true, planet1.hasNatives());
        assertEquals("Test planet", 15, planet1.getQtyNatives());
        assertEquals("Test planet", player1, planet1.getOwner());
        assertEquals("Test planet", player1, planet1.getOrigin());
        assertEquals("Test planet", null, planet1.getOriginReference());
        assertEquals("Test planet", player2, planet2.getOwner());
        assertEquals("Test planet", player1, planet2.getOrigin());
        assertEquals("Test planet", false, planet2.hasNatives());

        assertEquals("Test planet installations", tech_massDriver1, tech_massDriver1, planet1.getWeapons());
        assertEquals("Test planet installations", tech_energy10, planet1.getShields());
        assertEquals("Test planet installations", tech_photon100, planet1.getScanners());
        assertEquals("Test planet installations", tech_battleStationI, planet1.getOrbital().getHullTech());
        assertEquals("Test planet installations", tech_habitatI, planet1.getOrbital().getModuleTechs());

        // Test outpost
        Orbital outpostOrbital = ss1.getOrbital().getOrbital();
        assertEquals("Test Outpost", player1, ss1.getOrbital().getOwner());
        assertEquals("Test Outpost", tech_outpost, outpostOrbital.getHullTech());
        assertEquals("Test Outpost", tech_photon150, outpostOrbital.getScannerTech());
        assertEquals("Test Outpost", null, outpostOrbital.getModuleTechs());

        // Test Mining Station
        assertEquals("Test planet Space Station", planet1mine, planet1.getMiningStation().getPlanet());
        assertEquals("Test planet Space Station", planet1.getMiningStation().getOrbital(), planet1mine.getOrbital());
        Orbital mine = planet1.getMiningStation().getOrbital();
        assertEquals("Test planet Space Station", tech_habitatI, tech_habitatII, tech_oreExtractor, tech_researchModule, mine.getModuleTechs());
        assertEquals("Test planet Space Station", tech_photon100, mine.getScannerTech());
        assertEquals("Test planet Space Station", tech_spaceStation, mine.getHullTech());

        assertEquals("Test planet project", "TestFighter", planet1.getProject().getName());
        assertEquals("Test planet project", 50, planet1.getProduction());
        assertEquals("Test planet project", 5, planet1.getRepeatProject());

        assertEquals("Test planet project", TestTechs.MASS_DRIVER1, planet2.getProject().getName());
        //assertEquals("Test planet project", "Upgrade "+TestTechs.MASS_DRIVER1+" to "+TestTechs.FUSION_BEAM4, planet3.getProject().getName());
        assertEquals("Test planet project", TestTechs.ENERGY10, planet4.getProject().getName());
        //assertEquals("Test planet project", "Upgrade "+TestTechs.ENERGY10+" to "+TestTechs.ENERGY20, planet6.getProject().getName());
        assertEquals("Test planet project", TestTechs.BATTLE_STATIONI, planet7a.getProject().getName());
        //assertEquals("Test planet project", "Upgrade "+TestTechs.BATTLE_STATIONI+" to "+TestTechs.BATTLE_STATIONII, planet7b.getProject().getName());
        
        assertEquals("Test planet project", tech_massDriver1, ((ProjectWeapon)planet2.getProject()).getWeapon());
        //assertEquals("Test planet project", tech_massDriver1, ((ProjectWeaponUpgrade)planet3.getProject()).getOldWeapon());
        //assertEquals("Test planet project", tech_fusionBeam4, ((ProjectWeaponUpgrade)planet3.getProject()).getNewWeapon());
        assertEquals("Test planet project", tech_energy10, ((ProjectShield)planet4.getProject()).getShield());
        //assertEquals("Test planet project", tech_energy10, ((ProjectShieldUpgrade)planet6.getProject()).getOldShield());
        //assertEquals("Test planet project", tech_energy20, ((ProjectShieldUpgrade)planet6.getProject()).getNewShield());
        assertEquals("Test planet project", tech_battleStationI, ((ProjectBattleStation)planet7a.getProject()).getBattleStation());
        //assertEquals("Test planet project", tech_battleStationI, ((ProjectBattleStationUpgrade)planet7b.getProject()).getOldBattleStation());
        //assertEquals("Test planet project", tech_battleStationII, ((ProjectBattleStationUpgrade)planet7b.getProject()).getNewBattleStation());
        assertEquals("Test planet project", tech_photon100, ((ProjectScanner)planet7c.getProject()).getScanner());
        //assertEquals("Test planet project", tech_photon100, ((ProjectScannerUpgrade)planet7d.getProject()).getOldScanner());
        //assertEquals("Test planet project", tech_photon150, ((ProjectScannerUpgrade)planet7d.getProject()).getNewScanner());
        assertEquals("Test planet project", tech_outpost, ((ProjectOutpost)planet7e.getProject()).getOutpost());
        assertEquals("Test planet project", ss2, ((ProjectOutpost)planet7e.getProject()).getTarget());
        assertEquals("Test planet project", tech_spaceStation, ((ProjectSpaceStation)planet7f.getProject()).getSpaceStation());
        assertEquals("Test planet project", planet2, ((ProjectSpaceStation)planet7f.getProject()).getTarget());
        assertEquals("Test planet project", 1500, ((ProjectAsteroids)planet7g.getProject()).getCostToBuildOnPlanet());
        assertEquals("Test planet project", "SS1", ((ProjectConstructPlanet)planet7h.getProject()).getConstructPlanetDesign().getSolarSystem());
        assertEquals("Test planet project", 1, ((ProjectCraftUpgrade)planet7i.getProject()).getCraftDesign().getQtyCrafts());
        assertEquals("Test planet project", tech_flyer30, ((ProjectCraftUpgrade)planet7i.getProject()).getCraftDesign().getDesign().getHullTech());

        assertEquals(ss4, ss6, 3000, MotEngine.instance.getTradeRoutes().getTradeRoutes()[0]);

       Graph graph = player1.getGraph();
        assertEquals("Has correct Graph", 2, graph.getGraphLines().length);
        assertEquals("Has correct Graph", "TESTLINE1", graph.getGraphLines()[0].getName());
        assertEquals("Has correct Graph", 2, graph.getGraphLines()[0].getStatsData().length);
        assertEquals("Has correct Graph", 3000, graph.getGraphLines()[0].getStatsData()[0].getDate());
        assertEquals("Has correct Graph", 10, graph.getGraphLines()[0].getStatsData()[0].getValue());
        assertEquals("Has correct Graph", 3010, graph.getGraphLines()[0].getStatsData()[1].getDate());
        assertEquals("Has correct Graph", 20, graph.getGraphLines()[0].getStatsData()[1].getValue());
        assertEquals("Has correct Graph", "TESTLINE2", graph.getGraphLines()[1].getName());
        assertEquals("Has correct Graph", 1, graph.getGraphLines()[1].getStatsData().length);
        assertEquals("Has correct Graph", 2, graph.getGraphLines()[1].getStatsData()[0].getDate());
        assertEquals("Has correct Graph", 1, graph.getGraphLines()[1].getStatsData()[0].getValue());

        assertEquals("Test Resources", mot.common.resources.ResourceAntiMatter.class, planet1mine.getResource().getClass());
        assertEquals("Test Resources", mot.common.resources.ResourceWarpCrystal.class, planet1.getResource().getClass());
        assertEquals("Test Resources", mot.common.resources.ResourceCloakPalm.class, planet2.getResource().getClass());
        assertEquals("Test Resources", mot.common.resources.ResourceTritanOre.class, planet3.getResource().getClass());
        assertEquals("Test Resources", mot.common.resources.ResourcePsiFungus.class, planet4.getResource().getClass());
        assertEquals("Test Resources", mot.common.resources.ResourceBoralium.class, planet7a.getResource().getClass());
        assertEquals("Test Resources", mot.common.resources.ResourceEnergySpores.class, planet7b.getResource().getClass());
        
        assertEquals("Test Transports", 6, player1.getTransportManager().getQtyTransports());
        assertEquals("Test Transports", 2, player1.getTransportManager().getQtyFreeTransport());

        Transport outpostTransport = player1.getTransportManager().getTransportsAsArray()[0];
        Transport stationTransport = player1.getTransportManager().getTransportsAsArray()[1];
        Transport populationTransport = player1.getTransportManager().getTransportsAsArray()[2];

        TransportTaskDeployStation outpostTask = (TransportTaskDeployStation)outpostTransport.getTask();
        assertEquals("Test transports", "SS9", outpostTransport.getNavigator().getDestination().getStellarObject().getName());
        assertEquals("Test transports", null, outpostTask.getDestination().getPlanet());
        assertEquals("Test transports", ss9, outpostTask.getDestination().getStellarObject());
        assertEquals("Test transports", planet1, outpostTask.getOrigin().getPlanet());
        assertEquals("Test transports", 108, outpostTransport.getLocation().getX());
        assertEquals("Test transports", 124, outpostTransport.getLocation().getY());
        assertNull(  "Test transports", outpostTransport.getOrbit());
        assertEquals("Test transports", 12, outpostTransport.getNavigator().getETA());
        
        TransportTaskDeployStation stationTask = (TransportTaskDeployStation)stationTransport.getTask();
        assertEquals("Test transports", "SS4", stationTransport.getNavigator().getDestination().getStellarObject().getName());
        assertEquals("Test transports", planet4, stationTask.getDestination().getPlanet());
        assertEquals("Test transports", ss4, stationTask.getDestination().getStellarObject());
        assertEquals("Test transports", planet1, stationTask.getOrigin().getPlanet());
        assertEquals("Test transports", ss1.getLocation(), stationTransport.getLocation());
        assertEquals("Test transports", ss1, stationTransport.getOrbit().getStellarObject());
        assertEquals("Test transports", 6, stationTransport.getNavigator().getETA());

        TransportTaskMoveColonists popTask = (TransportTaskMoveColonists)populationTransport.getTask();
        assertEquals("Test transports", ss2, populationTransport.getNavigator().getDestination().getStellarObject());
        assertEquals("Test transports", planet2, popTask.getDestination().getPlanet());
        assertEquals("Test transports", 5, popTask.getQty());
        assertEquals("Test transports", 5, player1.getTransportManager().getQtyColonistsOnrouteTo(planet2));

        Transport tradeRouteTransport = player1.getTransportManager().getTransportsAsArray()[3];
        TransportTaskCreateTradeRoute tradeRouteTask = (TransportTaskCreateTradeRoute)tradeRouteTransport.getTask();
        assertEquals("Test transports", ss1, tradeRouteTask.getOriginName());
        assertEquals("Test transports", ss2, tradeRouteTask.getDestination().getName());

        assertEquals("Test Black Hole", tech_spaceStation,   ((SpaceStation)blackHole.getOrbital()).getOrbital().getHullTech());
        assertEquals("Test Black Hole", 1,                   ((SpaceStation)blackHole.getOrbital()).getOrbital().getModuleTechs().getModules().length);
        assertEquals("Test Black Hole", tech_researchModule, ((SpaceStation)blackHole.getOrbital()).getOrbital().getModuleTechs().getModules()[0]);
        assertEquals("Test Black Hole", player1.getName(), 	 ((SpaceStation)blackHole.getOrbital()).getOwner().getName());
        assertEquals("Test Black Hole", tech_photon100,      ((SpaceStation)blackHole.getOrbital()).getOrbital().getScannerTech());
        assertEquals("Test Black Hole", 100,                 ((SpaceStation)blackHole.getOrbital()).getScanStrength());
        
        assertEquals("Test Crisis", 3000,                 ((CrisisPlague)planet7a.getCrisis()).getDateStarted());
        assertEquals("Test Crisis", 3000,                 ((CrisisPsi)planet7b.getCrisis()).getDateStarted());
        assertEquals("Test Crisis", 3000,                 ((CrisisEmergency)planet7c.getCrisis()).getDateStarted());

        Spies spies = MotEngine.instance.getSpiesNotNull();

        assertEquals(1,                 	spies.getQtyDefendingSpies(player1));
        assertEquals(2,                 	spies.getEspionageSpies(player1, player2).getQtySpies());
        assertEquals(tech_SalvageAnalyser,  spies.getEspionageSpies(player1, player2).getTechnology());
        assertEquals(3,                 	spies.getQtySpiesSabotage(player1, player2));

        assertTrue(planet10.isBlockaded());

    }

    protected void tearDown() {
    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}