/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.Location;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.bean.Size;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.setup.OptionsFactory;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.planets.Planet;
import mot.engine.planets.StellarObjects;
import mot.engine.players.Player;
import mot.engine.setup.MotFactory;

/**
 * Tests MotFactory and Playing levels.
 */

public class SetupStatisticalTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(SetupStatisticalTest.class);
        return suite;
    }

    public SetupStatisticalTest(String name) {
        super(name);
    }
    
    protected void setUp() {
    	MotEngine.newInstance();
        Utils.clearNextRandomInt();
        Utils.clearNextRandomProbability();
        Utils.clearNextRandomDouble();
    }

    public void testProportions() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testProportions() *************************");
        setupConfigs();
        
        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[2];
        userDetails[0] = new NewGameUserPlayerData("Humans", OptionsFactory.getSetupOptions("Humans"), playerColourBlue, ANCIENT_WATER_MAN);
        userDetails[1] = new NewGameUserPlayerData("Bears", OptionsFactory.getSetupOptions("Bears"), playerColourBlue, ANCIENT_AIR_MAN);
        
		CraftDesigns cds = CraftDesigns.getInstance();
		cds.addOrUpdate(new CraftDesign("large_WaterMan", tech_flyer60, tech_fusion40, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_electronic40, tech_energy20));
		cds.addOrUpdate(new CraftDesign("large_AirMan", tech_flyer60, tech_fusion40, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_electronic40, tech_energy20));

        MotEngine engine = MotFactory.create(false, false, false, new Double(1.0), null, 100, 0, new Size(1000, 1000), userDetails, false);
        
        Player [] players = engine.getPlayers().getPlayers();
        Planet [] planets = engine.getPlanets().getPlanets();
        StellarObjects sos = engine.getStellarObjects();

        assertEquals("Has correct quantity of players", 2, players.length);
        assertEquals("Player has correct allignment", ANCIENT_WATER_MAN, players[0].getAlignment());
        assertEquals("Player has correct allignment", ANCIENT_AIR_MAN, players[1].getAlignment());
        
        assertEquals("Galaxy is the correct size", 100, planets.length);
        assertInRange("Galaxy is the correct size", 80, 100, sos.getQty());
        assertEquals("Galaxy is the correct size", 1000, sos.getSize().getX());
        assertEquals("Galaxy is the correct size", 1000, sos.getSize().getY());

        // CHECK ALL PLANETS ARE SPACED APART CORRECTLY AND NOT ON THE EDGES OF THE MAP
        for (int i=0; i<planets.length; i++) {
            Location loc = planets[i].getSolarSystem().getLocation();
            StellarObject closest = MotEngine.getInstance().getStellarObjects().getClosestStellarObject(loc);
            if (loc.getDistanceTo(closest.getLocation()) >= 40) {
                fail("Stellar objects are not too close ");
            }
            assertTrue("Stellar objects are not too close to the edge", loc.getX() >= 25 && loc.getX() <= 975);
            assertTrue("Stellar objects are not too close to the edge", loc.getY() >= 25 && loc.getY() <= 975);
        }

        // CHECK WE HAVE THE CORRECT NUMBER OF MONSTERS AND ANCIENTS : 2 ancients + 100*0.1 monsters = 12
        int qtyMonsters = 0;
        for (int i=0; i<planets.length; i++) {
            Craft [] ca = MotEngine.instance.getCrafts().getAt(planets[i].getStellarObject().getLocation());
            if (ca != null && ca[0].getOwner() == MotEngine.instance.getMonsterPlayer()) {
                qtyMonsters++;
            }
        }
        assertInRange("Correct qty of guarded planets "+qtyMonsters, 3, 32, qtyMonsters);

        // CHECK WE HAVE THE CORRECT NUMBER OF RESOURCES : 100 * 0.1 = 10
        int qtyResources = 0;
        for (int i=0; i<planets.length; i++) {
            if (planets[i].getResource() != null) {
                qtyResources++;
                assertTrue("Resources only exist on bad planets", planets[i].getTeraforming() <= 0.2);
            }
        }
        assertInRange("Correct qty of planets with resources "+qtyResources, 2, 18, qtyResources);
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}