/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import java.util.ArrayList;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.ImmunityGravity;
import mot.common.bean.ImmunityPsi;
import mot.common.bean.Location;
import mot.common.bean.Nebula;
import mot.common.bean.NewGameDataComputerPlayerDetails;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.bean.PlayerAiSettings;
import mot.common.bean.Size;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.factory.GraphicsFactory;
import mot.common.resources.ResourceCloakPalm;
import mot.common.resources.ResourceTritanOre;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.common.StellarObject;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Crafts;
import mot.engine.planets.BlackHole;
import mot.engine.planets.Planet;
import mot.engine.planets.Planets;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.StellarObjects;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.PlayerCharacter;
import mot.engine.players.UserPlayer;
import mot.engine.prizes.BoraliumPrize;
import mot.engine.prizes.MoneyPrize;
import mot.engine.prizes.Prize;
import mot.engine.prizes.RelicPrize;
import mot.engine.prizes.TechnologyPrize;
import mot.engine.setup.MotFactory;

/**
 * Tests MotFactory and Playing levels.
 */

public class SetupTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(SetupTest.class);
        return suite;
    }

    public SetupTest(String name) {
        super(name);
    }
    
    protected void setUp() {
    	MotEngine.newInstance();
        Utils.clearNextRandomInt();
        Utils.clearNextRandomProbability();
        Utils.clearNextRandomDouble();
    }
    
    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }


    public void testLevel1() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testLevel1() *************************");
        setupConfigs();

        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[1];
        userDetails[0] = new NewGameUserPlayerData("Humans", OptionsFactory.getSetupOptions("Humans"), playerColourBlue, null);

        // Always select same race BEARS.
        Utils.clearAndSetNextRandomInt(0);
        NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[2];
        cpd[0] = new NewGameDataComputerPlayerDetails("Bears", 1.0, null);
        cpd[1] = new NewGameDataComputerPlayerDetails("Ghosts", 1.0, null);
        MotEngine engine = MotFactory.create(false, false, false, new Double(1.0), cpd, 9, 0, new Size(500, 500), userDetails, false);

        Planet [] planets = engine.getPlanets().getPlanets();
        Player [] players = engine.getPlayers().getPlayers();
        Player p1 = engine.getPlayers().getPlayers()[0];
        Player p2 = engine.getPlayers().getPlayers()[1];
        Player p3 = engine.getPlayers().getPlayers()[2];
        
        
        assertEquals("Has correct quantity of planets", 10, planets.length);
        assertEquals("Has correct quantity of players", 3, players.length);
        assertTrue(p1 instanceof UserPlayer);
        assertTrue(p2 instanceof ComputerPlayer);
        assertTrue(p3 instanceof ComputerPlayer);
        
        // By default home worlds are size 100, terraform 40.0 population 10, factories 0.
        assertEquals("Home planet belongs to human player", "Humans", p1.getName());
        assertEquals("Home planet is correct size", 100, planets[0].getSize());
        assertEquals("Home planet is correct terraform", 1.0, planets[0].getTeraforming(), 0.01);
        assertEquals("Home planet is correct population", 40.0, planets[0].getPopulation(), 0.01);
        assertEquals("Home planet is correct qty factories", 10.0, planets[0].getQtyFactories(), 0.01);

        int p1Charisma = p1.getPlayerCharacter().getCharisma();
        int p2Charisma = p2.getPlayerCharacter().getCharisma();
        assertEquals("Player has correct charisma", 20, p1Charisma);
        assertEquals("Player has correct charisma", 0, p2Charisma);
        
        // Player1 has Human bonuses
        assertEquals("Human Player has correct relation", 70+p2Charisma, engine.getRelations().getRelation(p1, p2), 5.0);
        assertEquals("Human Player has alignment", (String)null, p1.getAlignment());
        assertEquals("Human Player is resourceful", true, p1.isResourceful());
        assertEquals("Human Player has no immuity",null, p1.getPlayerCharacter().getImmunities());

        // Player2 has no bonuses - No other players have charisma +10, but some have charisma -10!
        assertTrue("Other Player has correct relation", engine.getRelations().getRelation(p2, p3) <= 50);
        
        assertEquals("player1 is correct", "Girator", p1.getPlayerCharacter().getCraftsSpecialWeapons()[0]);
        assertEquals("player1 is correct", "Transport", p1.getPlayerCharacter().getCraftsSpecialWeapons()[1]);
        assertEquals("player1 is correct", "ShieldDrain", p1.getPlayerCharacter().getCraftsSpecialWeapons()[2]);
        assertEquals("player1 is correct", "Immobiliser", p1.getPlayerCharacter().getCraftsSpecialWeapons()[3]);
        assertEquals("player1 is correct", "Psi", p1.getPlayerCharacter().getCraftsSpecialWeapons()[4]);
        assertEquals("player1 is correct", true, p1.getPlayerCharacter().getCraftsWantPsiShields());
        assertEquals("Resourceful players always discover techs", 1.0, p1.getPlayerCharacter().getChanceToDiscoverSalvagedTech());
    }

    public void testLevelEasyWithTutorial() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testLevelEasyWithTutorial() *************************");
        setupConfigs();

        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[1];
        userDetails[0] = new NewGameUserPlayerData("Humans", OptionsFactory.getSetupOptions("Humans"), playerColourBlue, null);

        // Always select same race BEARS.
        Utils.clearAndSetNextRandomInt(0);
        NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[1];
        cpd[0] = new NewGameDataComputerPlayerDetails("Bears", 0.75, null);
        MotEngine engine = MotFactory.create(true, false, false, new Double(0.75), cpd, 15, 0, new Size(500, 500), userDetails, false);

        SolarSystem [] solarSystems = engine.getStellarObjects().getSolarSystems();
        Planet [] planets = engine.getPlanets().getPlanets();
        Player [] players = engine.getPlayers().getPlayers();
        Player p1 = engine.getPlayers().getPlayers()[0];
        Player p2 = engine.getPlayers().getPlayers()[1];
        Crafts crafts = engine.getCrafts();
        
        
        assertEquals("Has correct quantity of planets", 19, planets.length);
        assertEquals("Has correct quantity of players", 2, players.length);
        assertTrue(p1 instanceof UserPlayer);
        assertTrue(p2 instanceof ComputerPlayer);
        
        assertEquals("Home planet belongs to human player", "Humans", p1.getName());
        assertEquals("Home planet belongs to human player", "Humans", solarSystems[0].getOwners()[0].getName());
        assertEquals("Home Solar System exists", "Home", solarSystems[0].getName());
        assertEquals("Home Solar System  has planets", 2, solarSystems[0].getQtyPlanets());
        assertEquals(150, 150, solarSystems[0].getLocation());
        
        // By default home worlds are size 100, terraform 40.0 population 10, factories 0.
        assertEquals("Home planet is correct size", "Home-A", planets[0].getName());
        assertEquals("Home planet is in home SS", solarSystems[0], planets[0].getSolarSystem());
        assertEquals("Home planet is correct size", 100, planets[0].getSize());
        assertEquals("Home planet is correct terraform", 1.0, planets[0].getTeraforming(), 0.01);
        assertEquals("Home planet is correct population", 40.0, planets[0].getPopulation(), 0.01);
        assertEquals("Home planet is correct qty factories", 10.0, planets[0].getQtyFactories(), 0.01);
        
        // Easy levels have bonus planet in user players home ss size 100, terraform 90.0 population 0, rich +100.
        assertEquals("Bonus planet is correct size", "Home-B", planets[1].getName());
        assertEquals("Bonus planet is in home SS", solarSystems[0], planets[1].getSolarSystem());
        assertEquals("Bonus planet is correct size", 100, planets[1].getSize());
        assertEquals("Bonus planet is correct terraform", 0.9, planets[1].getTeraforming(), 0.01);
        assertEquals("Bonus planet is correct population", 0.0, planets[1].getPopulation(), 0.01);
        assertEquals("Bonus planet is correct qty factories", 2.0, planets[1].getProductionBonus(), 0.01);

        assertEquals("First Colony belongs to nobody", null,  solarSystems[1].getOwners());
        assertEquals("First Colony System exists", "First", solarSystems[1].getName());
        assertEquals("First Colony has planets", 1, solarSystems[1].getQtyPlanets());
        assertEquals(150, 75, solarSystems[1].getLocation());
        
        assertEquals("First Colony is exists", "First-A", planets[2].getName());
        assertEquals("First Colony is correct size", 100, planets[2].getSize());
        assertEquals("First Colony is correct terraform", 1.0, planets[2].getTeraforming(), 0.01);
        assertEquals("First Colony is correct population", 0.0, planets[2].getPopulation(), 0.01);
        assertEquals("First Colony is correct qty factories", 0.0, planets[2].getProductionBonus(), 0.01);
        assertEquals("First Colony is correct qty factories", 1.0, planets[2].getReproductionBonus(), 0.01);

        assertEquals("Explore Solar System belongs to nobody", null,  solarSystems[2].getOwners());
        assertEquals("Explore Solar System System exists", "ExploreMe", solarSystems[2].getName());
        assertEquals("Explore Solar System has no planets", 0, solarSystems[2].getQtyPlanets());
        assertEquals(75, 150, solarSystems[2].getLocation());

        assertEquals("Guarded Solar System belongs to nobody", null,  solarSystems[3].getOwners());
        assertEquals("Guarded Solar System System exists", "Guarded", solarSystems[3].getName());
        assertEquals("Guarded Solar System has 2 or more planets", true, solarSystems[3].getQtyPlanets() >= 2);
        assertEquals(240, 150, solarSystems[3].getLocation());

        assertEquals("Guarded Solar System has a monster in Orbit", "Monster", crafts.getAt(solarSystems[3])[0].getOwner().getName());

        assertEquals("Guarded Planet 1 is correct", "Guarded-A", planets[3].getName());
        assertEquals("Guarded Planet 1 is correct SS", solarSystems[3], planets[3].getSolarSystem());
        assertEquals("Guarded Planet 1 is correct size", 110, planets[3].getSize());
        assertEquals("Guarded Planet 1 is correct terraform", 1.0, planets[3].getTeraforming(), 0.01);
        assertEquals("Guarded Planet 1 is correct population", 0.0, planets[3].getPopulation(), 0.01);
        assertEquals("Guarded Planet 1 is correct qty factories", 2.0, planets[3].getProductionBonus(), 0.01);
        assertEquals("Guarded Planet 1 is correct qty factories", 0.0, planets[3].getReproductionBonus(), 0.01);

        assertEquals("Guarded Planet 2 is correct", "Guarded-B", planets[4].getName());
        assertEquals("Guarded Planet 1 is correct SS", solarSystems[3], planets[4].getSolarSystem());
        assertEquals("Guarded Planet 2 is correct size", 90, planets[4].getSize());
        assertEquals("Guarded Planet 2 is correct terraform", 0.7, planets[4].getTeraforming(), 0.01);
        assertEquals("Guarded Planet 2 is correct population", 0.0, planets[4].getPopulation(), 0.01);
        assertEquals("Guarded Planet 2 is correct qty factories", 0.0, planets[4].getProductionBonus(), 0.01);
        assertEquals("Guarded Planet 2 is correct qty factories", 0.0, planets[4].getReproductionBonus(), 0.01);
        assertEquals("Guarded Planet 2 is correct qty factories", 1.0, planets[4].getResearchBonus(), 0.01);

        
//        Planet firstPlanet = MotEngine.instance.getPlanets().getPlanet("First1-A");
//        assertEquals("First Colony Planet for Tutorial Exists", 100, firstPlanet.getSize());
//        assertEquals("First Colony Planet for Tutorial Exists", 1.0, firstPlanet.getProductionBonus());
//        assertEquals("First Colony Planet for Tutorial Exists", true, planets[2].getSolarSystem().getLocation().getDistanceTo(planets[0].getSolarSystem().getLocation()) < 150);
        
        // Player2 negative bonuses - because level is easy!
        assertEquals(0.0, p1.getPlayerCharacter().getProductionBonus(true));
        assertEquals(0.0, p1.getPlayerCharacter().getResearchBonus(true));
        assertEquals(0.0, p1.getPlayerCharacter().getDevelopmentBonus(true));
        
        assertEquals(0.0, p2.getPlayerCharacter().getProductionBonus(false));
        assertEquals(0.0, p2.getPlayerCharacter().getResearchBonus(false));
        assertEquals(0.0, p2.getPlayerCharacter().getDevelopmentBonus(false));
        assertEquals(-0.25, p2.getPlayerCharacter().getProductionBonus(true));
        assertEquals(-0.25, p2.getPlayerCharacter().getResearchBonus(true));
        assertEquals(-0.25, p2.getPlayerCharacter().getDevelopmentBonus(true));
        
        assertEquals(p1, planets[0].getOwner());
        assertEquals(p2, planets[5].getOwner());
                
    }
    

    public void testLevel2() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testLevel2() *************************");
        setupConfigs();

        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[1];
        userDetails[0] = new NewGameUserPlayerData("Humans", OptionsFactory.getSetupOptions("Humans"), playerColourBlue, null);
        
        // Check that the order of the races in race graphics list is Humans, Bears, Machines
        // Otherwise we will not be getting the correct 'random' players!
        assertEquals("Humans", GraphicsFactory.getRaceGraphicsList().getRaceGraphicsAsArray()[0].getName());
        assertEquals("Bears", GraphicsFactory.getRaceGraphicsList().getRaceGraphicsAsArray()[1].getName());
        assertEquals("Machines", GraphicsFactory.getRaceGraphicsList().getRaceGraphicsAsArray()[2].getName());

        NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[2];
        cpd[0] = new NewGameDataComputerPlayerDetails("Bears", 1.5, new PlayerAiSettings(1, 1, 1));
        cpd[1] = new NewGameDataComputerPlayerDetails("Machines", 1.5, new PlayerAiSettings(-1, -1, -1));
        MotEngine engine = MotFactory.create(false, false, true, new Double(1.5), cpd, 9, 0, new Size(500, 500), userDetails, false);
        
        Planet [] planets = engine.getPlanets().getPlanets();
        Player [] players = engine.getPlayers().getPlayers();
        Player p1 = engine.getPlayers().getPlayers()[0];
        Player p2 = engine.getPlayers().getPlayers()[1];
        Player p3 = engine.getPlayers().getPlayers()[2];
        
        assertEquals("Has correct quantity of planets", 10, planets.length);
        assertEquals("Has correct quantity of players", 3, players.length);
        assertTrue(p1 instanceof UserPlayer);
        assertTrue(p2 instanceof ComputerPlayer);
        assertTrue(p3 instanceof ComputerPlayer);
        
        assertEquals(true, MotEngine.instance.isSpaceKaanEnabled());

        // Level has effected relationships
        assertEquals("Human Player has correct relation", 70, engine.getRelations().getRelation(p1, p2), 5.0);
        assertEquals("Human Player has alignment", (String)null, p1.getAlignment());
        
        // Level increases production, research and development for all planets by
        // (level-1)/2 => bonuses of +1.5!
        assertEquals("Computer players have received bonuses", 0.0, p2.getPlayerCharacter().getProductionBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 0.0, p2.getPlayerCharacter().getResearchBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 0.0, p2.getPlayerCharacter().getDevelopmentBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 0.5, p2.getPlayerCharacter().getProductionBonus(true), 0.01 );
        assertEquals("Computer players have received bonuses", 0.5, p2.getPlayerCharacter().getResearchBonus(true), 0.01 );
        assertEquals("Computer players have received bonuses", 0.5, p2.getPlayerCharacter().getDevelopmentBonus(true), 0.01 );
        assertEquals("Computer players have received bonuses", false, p2.getPlayerCharacter().isIsolationist());
        
        // Machines have development bonus of +25% and level bonus of 50%
        assertEquals("Computer players have received bonuses", 0.0, p3.getPlayerCharacter().getProductionBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 0.0, p3.getPlayerCharacter().getResearchBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 0.25, p3.getPlayerCharacter().getDevelopmentBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 0.5, p3.getPlayerCharacter().getProductionBonus(true), 0.01 );
        assertEquals("Computer players have received bonuses", 0.5, p3.getPlayerCharacter().getResearchBonus(true), 0.01 );
        assertEquals("Computer players have received bonuses", true, p3.getPlayerCharacter().isIsolationist());
        
		assertEquals(2, p2.getPlayerCharacter().getQtyCraftPerPlanetToDeclareWar());
		assertEquals(1, p2.getPlayerCharacter().getQtyCraftToDeclareWar());
		assertEquals(false, p2.getPlayerCharacter().isIsolationist());
		assertEquals(1.0, p2.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenLike(), 0.001);
		assertEquals(1.0, p2.getPlayerCharacter().getAcceptBadSwapProbabilityWhenLike(), 0.001);
		assertEquals(1.0, p2.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(1.0, p2.getPlayerCharacter().getAcceptBadSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(1.0, p2.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenDislike(), 0.001);
		assertEquals(0.1, p2.getPlayerCharacter().getAcceptBadSwapProbabilityWhenDislike(), 0.001);

		assertEquals(4, p3.getPlayerCharacter().getQtyCraftPerPlanetToDeclareWar());
		assertEquals(4, p3.getPlayerCharacter().getQtyCraftToDeclareWar());
		assertEquals(true, p3.getPlayerCharacter().isIsolationist());
		assertEquals(1.0, p3.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenLike(), 0.001);
		assertEquals(0.0, p3.getPlayerCharacter().getAcceptBadSwapProbabilityWhenLike(), 0.001);
		assertEquals(0.5, p3.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(0.0, p3.getPlayerCharacter().getAcceptBadSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(0.0, p3.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenDislike(), 0.001);
		assertEquals(0.0, p3.getPlayerCharacter().getAcceptBadSwapProbabilityWhenDislike(), 0.001);

    }

    public void testLevel3WithWormHole() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testLevel3WithWormHole() *************************");
        setupConfigs();

        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[1];
        userDetails[0] = new NewGameUserPlayerData("Ghosts", OptionsFactory.getSetupOptions("Ghosts"), playerColourGreen, null);
        
        NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[1];
        cpd[0] = new NewGameDataComputerPlayerDetails("Bears", 2.0, null);
        MotEngine engine = MotFactory.create(false, true, false, new Double(2.0), cpd, 8, 0, new Size(500, 500), userDetails, false);
        
        Planet [] planets = engine.getPlanets().getPlanets();
        Player [] players = engine.getPlayers().getPlayers();
        Player p1 = engine.getPlayers().getPlayers()[0];
        Player p2 = engine.getPlayers().getPlayers()[1];
        
        assertEquals("Has correct quantity of planets", 8, planets.length);
        assertEquals("Has correct quantity of players", 2, players.length);
        assertTrue(p1 instanceof UserPlayer);
        assertTrue(p2 instanceof ComputerPlayer);
        
        assertEquals(false, MotEngine.instance.isSpaceKaanEnabled());

        // Level has effected relationships
        assertEquals("Human Player has correct relation", 30, engine.getRelations().getRelation(p1, p2), 5.0);
        assertEquals("Human Player has alignment", (String)null, p1.getAlignment());
        
        // Level increases production, research and development for all planets by
        // (level-1)/2 => bonuses of +1.5!
        assertEquals("Computer players have received bonuses", 0.0, p2.getPlayerCharacter().getProductionBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 0.0, p2.getPlayerCharacter().getResearchBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 0.0, p2.getPlayerCharacter().getDevelopmentBonus(false), 0.01 );
        assertEquals("Computer players have received bonuses", 1.0, p2.getPlayerCharacter().getProductionBonus(true), 0.01 );
        assertEquals("Computer players have received bonuses", 1.0, p2.getPlayerCharacter().getResearchBonus(true), 0.01 );
        assertEquals("Computer players have received bonuses", 1.0, p2.getPlayerCharacter().getDevelopmentBonus(true), 0.01 );
        
        BlackHole blackHole = MotEngine.instance.getStellarObjects().getBlackHoles()[0];
        BlackHole wormHole = MotEngine.instance.getStellarObjects().getBlackHoles()[1];
        
		assertEquals(0.1f, blackHole.getResearchBonus());
		assertEquals(false, blackHole.isSpaceJumpAbility());
		assertEquals(0.0f, wormHole.getResearchBonus());
		assertEquals(true, wormHole.isSpaceJumpAbility());
				
    }

    public void testBirds() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testBirds() *************************");
        setupConfigs();

        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[1];
        userDetails[0] = new NewGameUserPlayerData("Birds", OptionsFactory.getSetupOptions("Birds"), playerColourBlue, null);

        NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[1];
        cpd[0] = new NewGameDataComputerPlayerDetails("Humans", 1.0, null);
        MotEngine engine = MotFactory.create(false, false, false, new Double(1.0), cpd, 5, 0, new Size(500, 500), userDetails, false);

        Planet [] planets = engine.getPlanets().getPlanets();
        Player [] players = engine.getPlayers().getPlayers();
        Player p1 = engine.getPlayers().getPlayers()[0];
        Player p2 = engine.getPlayers().getPlayers()[1];
        
        
        assertEquals("Has correct quantity of planets", 6, planets.length);
        assertEquals("Has correct quantity of players", 2, players.length);
        assertTrue(p1 instanceof UserPlayer);
        assertTrue(p2 instanceof ComputerPlayer);
        
        // By default home worlds are size 100, terraform 40.0 population 10, factories 0.
        assertEquals("Home planet belongs to human player", "Birds", p1.getName());
        assertEquals("Home planet is correct size", 100, planets[0].getSize());
        assertEquals("Home planet is correct terraform", 1.0, planets[0].getTeraforming(), 0.01);
        assertEquals("Home planet is correct population", 40.0, planets[0].getPopulation(), 0.01);
        assertEquals("Home planet is correct qty factories", 10.0, planets[0].getQtyFactories(), 0.01);

        // Player1 has Human bonuses
        int p2Charisma = p2.getPlayerCharacter().getCharisma();
        assertEquals("Player has correct relation", 50+p2Charisma, engine.getRelations().getRelation(p1, p2), 5.0);
        assertEquals("Player has alignment", (String)null, p1.getAlignment());
        assertEquals("Player is resourceful", false, p1.isResourceful());
        assertEquals("Player has immuity", ImmunityPsi.class, p1.getPlayerCharacter().getImmunities()[0].getClass());
        assertEquals("Player has bonuses", 0.25, p1.getOriginBattleSpeedBonus(), 0.0);
        assertEquals("Player has bonuses", 0.25, p1.getOriginBattleTurningBonus(), 0.0);
        assertEquals("Player has bonuses", 0.5, p1.getOriginSpeedBonus(), 0.0);
        assertEquals("Player has bonuses", 0.1, p1.getPlayerCharacter().getEloquenceBonus(), 0.0);
    }
    
    public void testBears() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testBears() *************************");
        setupConfigs();

        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[1];
        userDetails[0] = new NewGameUserPlayerData("Bears", OptionsFactory.getSetupOptions("Bears"), playerColourBlue, null);
        NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[1];
        cpd[0] = new NewGameDataComputerPlayerDetails("Ghosts", 1.0, null);
        MotEngine engine = MotFactory.create(false, false, false, new Double(1.0), cpd, 14, 0, new Size(500, 500), userDetails, false);

        Planet [] planets = engine.getPlanets().getPlanets();
        Player [] players = engine.getPlayers().getPlayers();
        Player p1 = engine.getPlayers().getPlayers()[0];
        Player p2 = engine.getPlayers().getPlayers()[1];
        
        
        assertEquals("Has correct quantity of planets", 15, planets.length);
        assertEquals("Has correct quantity of players", 2, players.length);
        assertTrue(p1 instanceof UserPlayer);
        assertTrue(p2 instanceof ComputerPlayer);
        
        // By default home worlds are size 100, terraform 40.0 population 10, factories 0.
        assertEquals("Home planet belongs to human player", "Bears", p1.getName());
        assertEquals("Home planet is correct size", 100, planets[0].getSize());
        assertEquals("Home planet is correct terraform", 1.0, planets[0].getTeraforming(), 0.01);
        assertEquals("Home planet is correct population", 40.0, planets[0].getPopulation(), 0.01);
        assertEquals("Home planet is correct qty factories", 10.0, planets[0].getQtyFactories(), 0.01);

        // Player1 has Human bonuses
        assertEquals("Player is resourceful", false, p1.isResourceful());
        assertEquals("Player has immuity", ImmunityGravity.class, p1.getPlayerCharacter().getImmunities()[0].getClass());
        assertEquals("Player has bonuses", 0.25, p1.getOriginMaxDamageBonus(), 0.0);
        assertEquals("Player has bonuses", 0.25, p1.getFactoryPerPopulationBonus(), 0.0);
        assertEquals("Player has bonuses", 1.0, p1.getAssaultBonus(), 0.0);
    }
    
    public void testPlayerDeaflts() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testPlayerDeaflts() *************************");
        SetupOptions go = OptionsFactory.getGeneralOptions();
        RaceOptions ro = new RaceOptions("Test", go, go);
        PlayerCharacter pc = new PlayerCharacter("Test", ro, (double) 1, null, null);
        
        assertEquals(40, pc.getMaxTimeBetweenFleets());
        assertEquals(3, pc.getQtyCraftPerPlanetToDeclareWar());
        assertEquals(0.0, pc.getDemandProbability());
        assertEquals(0.0, pc.getBlockadeOrAttackProbability());
        assertEquals(0.25, pc.getAcceptPeaceProbability());
        assertEquals(0.5, pc.getAcceptAlianceProbabilityWhenLike());
        assertEquals(0.25, pc.getAcceptAlianceProbabilityWhenFond());
        assertEquals(1.0, pc.getAcceptGoodSwapProbabilityWhenLike());
        assertEquals(1.0, pc.getAcceptBadSwapProbabilityWhenLike());
        assertEquals(1.0, pc.getAcceptGoodSwapProbabilityWhenNeutral());
        assertEquals(0.5, pc.getAcceptBadSwapProbabilityWhenNeutral());
        assertEquals(0.1, pc.getAcceptGoodSwapProbabilityWhenDislike());
        assertEquals(0.0, pc.getAcceptBadSwapProbabilityWhenDislike());
        assertEquals(0.5, pc.getSpyDefendPercent());
        assertEquals("Girator,Transport,ShieldDrain,Immobiliser,Psi", Comp.toString(pc.getCraftsSpecialWeapons(),","));
        assertEquals(true, pc.getCraftsWantPsiShields());
        assertEquals(0.5, pc.getChanceToDiscoverSalvagedTech());
        
    }
    
    public void testPlayerAlignmentAndGalaxySize() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testPlayerAlignmentAndGalaxySize() *************************");
        setupConfigs();
        
        NewGameUserPlayerData[] userDetails = new NewGameUserPlayerData[2];
        userDetails[0] = new NewGameUserPlayerData("Humans", OptionsFactory.getSetupOptions("Humans"), playerColourGreen, ANCIENT_WATER_MAN);
        userDetails[1] = new NewGameUserPlayerData("Bears", OptionsFactory.getSetupOptions("Bears"), playerColourRed, ANCIENT_AIR_MAN);

		CraftDesigns cds = CraftDesigns.getInstance();
		cds.addOrUpdate(new CraftDesign("large_WaterMan", tech_flyer60, tech_fusion40, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_electronic40, tech_energy20));
		cds.addOrUpdate(new CraftDesign("large_AirMan", tech_flyer60, tech_fusion40, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_electronic40, tech_energy20));

        MotEngine engine = MotFactory.create(false, false, false, new Double(2.0), null, 100, 0, new Size(1000, 1000), userDetails, false);
        
        Player [] players = engine.getPlayers().getPlayers();
        Planet [] planets = engine.getPlanets().getPlanets();
        StellarObjects sos = engine.getStellarObjects();

        assertEquals("Has correct quantity of players", 2, players.length);
        assertEquals("Player has correct allignment", ANCIENT_WATER_MAN, players[0].getAlignment());
        assertEquals("Player has correct allignment", ANCIENT_AIR_MAN, players[1].getAlignment());
        assertEquals("Player has correct colour", playerColourGreen, players[0].getColor());
        assertEquals("Player has correct colour", playerColourRed, players[1].getColor());
        
        assertEquals("Galaxy is the correct size", 100, planets.length);
        assertInRange("Galaxy is the correct size", 80, 100, sos.getQty());
        assertEquals("Galaxy is the correct size", 1000, sos.getSize().getX());
        assertEquals("Galaxy is the correct size", 1000, sos.getSize().getY());

        // CHECK ALL PLANETS ARE SPACED APART CORRECTLY AND NOT ON THE EDGES OF THE MAP
        for (int i=0; i<planets.length; i++) {
            Location loc = planets[i].getSolarSystem().getLocation();
            StellarObject closest = MotEngine.getInstance().getStellarObjects().getClosestStellarObject(loc);
            if (loc.getDistanceTo(closest.getLocation()) >= 40) {
                fail("Stellar objects are not too close ");
            }
            assertTrue("Stellar objects are not too close to the edge "+loc+" at "+i, loc.getX() >= 25 && loc.getX() <= 975);
            assertTrue("Stellar objects are not too close to the edge "+loc+" at "+i, loc.getY() >= 25 && loc.getY() <= 975);
        }

    }

    public void testMonsters() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testMonsters() *************************");
        setupConfigs();
        MotEngine.instance.getMonsterPlayer();
        
    	String [] playerNames = new String [] { "Humans", "Bears" };
    	SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
        Player player1 = newPlayer(playerNames[0], blue, options, playerNames);
        Player player2 = newPlayer(playerNames[1], blue, options, playerNames);
        
        
        Planet planet1 = newPlanetAndSS("SS1", 100, 100, 100, 0.2);
        Planet planet2 = newPlanetAndSS("SS2", 200, 200, 100, 0.2);
        Planet planet3 = newPlanetAndSS("SS3", 300, 300, 100, 0.2);
        
        Utils.setNextRandomProbability(0.5); // Planet SS1-A no monster 
        Utils.setNextRandomProbability(0.0); // Planet SS2-A has monster 
        Utils.setNextRandomInt(0); // Small monster
        Utils.setNextRandomProbability(0.0); // Planet SS3-A has monster 
        Utils.setNextRandomInt(2); // Medium monster
        
        MotFactory.createMonsters(new ArrayList(), null);
        Craft[] crafts = MotEngine.instance.getCrafts().getCrafts();
        
        assertEquals(2, crafts.length);
        assertEquals("monster_small1", crafts[0].getName());
        assertEquals("SS2", crafts[0].getOrbit().getStellarObject().getName());
        assertEquals("monster1", crafts[1].getName());
        assertEquals("SS3", crafts[1].getOrbit().getStellarObject().getName());

        Planets planets = MotEngine.instance.getPlanets();
		assertEquals(110, planets.getPlanet("SS2-A").getSize());
		assertEquals(1.0, planets.getPlanet("SS2-A").getTeraforming());
        assertEquals(110, planets.getPlanet("SS3-A").getSize());
        assertEquals(1.0, planets.getPlanet("SS3-A").getTeraforming());
        
    }


    public void testAncients() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testAncients() *************************");
        setupConfigs();
        
    	String [] playerNames = new String [] { "Humans", "Bears" };
    	SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
        Player player1 = newPlayer(playerNames[0], blue, options, playerNames);
        Player player2 = newPlayer(playerNames[1], blue, options, playerNames);

        
        Planet planet1 = newPlanetAndSS("SS1", 100, 100, 100, 0.2);
        Planet planet2 = newPlanetAndSS("SS2", 200, 200, 100, 0.2);
        Planet planet3 = newPlanetAndSS("SS3", 300, 300, 100, 0.2);
        
        Utils.setNextRandomInt(0);
        Utils.setNextRandomInt(0);
        MotFactory.createAncients(null);
        
        Craft[] crafts = MotEngine.instance.getCrafts().getCrafts();
        
        assertEquals(8, crafts.length);
        assertEquals("large_WaterMan1", crafts[0].getName());
        assertEquals("SS1", crafts[0].getOrbit().getStellarObject().getName());
        assertEquals("medium_WaterMan1", crafts[1].getName());
        assertEquals("SS1", crafts[1].getOrbit().getStellarObject().getName());
        Planets planets = MotEngine.instance.getPlanets();
		assertEquals(150, planets.getPlanet("SS1-A").getSize());
		assertEquals(1.0, planets.getPlanet("SS1-A").getTeraforming());
		assertEquals(3.0, planets.getPlanet("SS1-A").getProductionBonus());
		assertEquals(2.0, planets.getPlanet("SS1-A").getResearchBonus());
        
		Prize[] prizes = planet1.getSolarSystem().getPrizes();
		assertEquals("Ancients give a Prize", 1, prizes.length);
    }
    
    public void testCreateResources() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testBears() *************************");
        
    	String [] playerNames = new String [] { "Humans", "Bears" };
    	SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
        Player player1 = newPlayer(playerNames[0], blue, options, playerNames);
        Player player2 = newPlayer(playerNames[1], blue, options, playerNames);

        SolarSystem ss1 = newSolarSystem("SS1", 100, 100);
        SolarSystem ss2 = newSolarSystem("SS2", 200, 100);

        Planet planet1 = newPlanet("SS1-A", ss1, null, 100, 1.0);
        Planet planet2 = newPlanet("SS1-B", ss1, null, 100, 0.1);
        Planet planet3 = newPlanet("SS2-A", ss2, null, 100, 0.2);
        Planet planet4 = newPlanet("SS2-B", ss2, null, 100, 0.8);
        Planet planet5 = newPlanet("SS2-C", ss2, null, 100, 0.2);
        Planet planet6 = newPlanet("SS2-D", ss2, null, 100, 0.2);
        
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(1); // Select SS2-A
        Utils.setNextRandomInt(0); // Select SS1-B
        Utils.setNextRandomInt(3); // ResourceTritanOre
        Utils.setNextRandomInt(1); // ResourceCloakPalm
        Utils.setNextRandomInt(0); // Select SS1-A has Boralium
        Utils.setNextRandomInt(4); // Select SS2-D has Boralium
        MotFactory.createResources();

        assertEquals(true, planet3.getResource() instanceof ResourceTritanOre);
        assertEquals(true, planet2.getResource() instanceof ResourceCloakPalm);
    }
    
    public void testCreateNebula() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testCreateNebula() *************************");
        
    	String [] playerNames = new String [] { "Humans", "Bears" };
    	SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
        Player player1 = newPlayer(playerNames[0], blue, options, playerNames);
        Player player2 = newPlayer(playerNames[1], red, options, playerNames);
        
        
        Planet planet1 = newPlanetAndSS("SS1", 100, 200, player1);
        Planet planet2 = newPlanetAndSS("SS2", 400, 200, player2);
        Planet planet3 = newPlanetAndSS("SS3", 200, 200, 100, 0.2);
        
        StellarObjects sos = MotEngine.instance.getStellarObjects();
        
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(50, 300); // First location is 150, 200 which is rejected because planet exists at this location
        Utils.setNextRandomInt(100, 300);
        Utils.setNextRandomInt(350, 300); // Second location is 450, 200 which is rejected because planet exists at this location
        Utils.setNextRandomInt(100, 300);
        Utils.setNextRandomInt(150, 300); // Third location is 250, 200 which is GOOD
        Utils.setNextRandomInt(100, 300);
        Utils.setNextRandomInt(0, 5);    // Diameter is 150
		MotFactory.createNebulae(sos, sos.getGalaxyDetails().getSize());

        Nebula[] nebulae = MotEngine.instance.getStellarObjects().getGalaxyDetails().getNebulaeAsArray();
		assertEquals(1, nebulae.length);
        assertEquals(250,200, nebulae[0].getLocation());
        assertEquals(150, nebulae[0].getDiameter());
        
        //
        // Test after four attempts a nebula is not created
        // Test will not create nebula where one already exists
        //
        
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(160, 300); // First location is 260, 200 which is rejected because nebula already exists in that area
        Utils.setNextRandomInt(100, 300);
        Utils.setNextRandomInt(350, 300); // Second location is 450, 200 which is rejected because planet exists at this location
        Utils.setNextRandomInt(100, 300);
        Utils.setNextRandomInt(350, 300); // Third location is 450, 200 which is rejected because planet exists at this location
        Utils.setNextRandomInt(150, 300);
        Utils.setNextRandomInt(350, 300); // Fourth location is 450, 200 which is rejected because planet exists at this location
        Utils.setNextRandomInt(100, 300);
		MotFactory.createNebulae(sos, sos.getGalaxyDetails().getSize());

        nebulae = MotEngine.instance.getStellarObjects().getGalaxyDetails().getNebulaeAsArray();
		assertEquals(1, nebulae.length);
        assertEquals(250,200, nebulae[0].getLocation());
        assertEquals(150, nebulae[0].getDiameter());
        
        //
        // Test can have more than one nebula
        //
        
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(150, 300); // First location is 250, 400 which is GOOD
        Utils.setNextRandomInt(300, 300);
        Utils.setNextRandomInt(1, 5);
		MotFactory.createNebulae(sos, sos.getGalaxyDetails().getSize());

        nebulae = MotEngine.instance.getStellarObjects().getGalaxyDetails().getNebulaeAsArray();
		assertEquals(2, nebulae.length);
        assertEquals(250,200, nebulae[0].getLocation());
        assertEquals(150, nebulae[0].getDiameter());
        assertEquals(250,400, nebulae[1].getLocation());
        assertEquals(170, nebulae[1].getDiameter());
        
    }
    
    public void testPrizes() throws Exception {
        Logger.logDebug("*******************************  SetupTest.testPrizes() *************************");
        setupConfigs();
        
        Planet planet1 = newPlanetAndSS("SS1", 100, 100, 100, 0.2);
        Planet planet2 = newPlanetAndSS("SS2", 200, 200, 100, 0.2);
        Planet planet3 = newPlanetAndSS("SS3", 300, 300, 100, 0.2);
        Planet planet4 = newPlanetAndSS("SS4", 300, 300, 100, 0.2);
        
        Utils.setNextRandomProbability(0.0); // Planet SS1 has a Prize 
        Utils.setNextRandomInt(0); 			 // Money Prize
        Utils.setNextRandomInt(1); 			 // Have 500 credits in MoneyPrize
        Utils.setNextRandomProbability(0.0); // Planet SS2 mhas a Prize 
        Utils.setNextRandomInt(1); 			 // Technology Prize
        Utils.setNextRandomInt(5);			 // Tech is sixth in list
        Utils.setNextRandomProbability(0.5); // Planet SS3 has no Prize 
        Utils.setNextRandomProbability(0.0); // Planet SS3 has a Prize 
        Utils.setNextRandomInt(3); 			 // Relic Prize
        Utils.setNextRandomProbability(0.0); // Planet SS4 has a Prize 
        Utils.setNextRandomInt(2); 			 // Craft Prize
        Utils.setNextRandomInt(0); 			 // Select SS1 for Boralium Prize
        Utils.setNextRandomInt(1); 			 // SS1 Has 100 Boralium Crystals
        MotFactory.createPrizes(null, null);
        
		assertEquals(2, planet1.getSolarSystem().getPrizes().length);
		assertEquals(500, ((MoneyPrize)planet1.getSolarSystem().getPrizes()[0]).getAmount());
		assertEquals(205, ((BoraliumPrize)planet1.getSolarSystem().getPrizes()[1]).getAmount());
		assertEquals(true, ((TechnologyPrize)planet2.getSolarSystem().getPrizes()[0]).getTech() != null);
		assertEquals(true, ((RelicPrize)planet4.getSolarSystem().getPrizes()[0]).getRelic() != null);
        
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}