/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.event.MotEvent;
import mot.common.event.SabotageEvent;
import mot.common.event.TechnologyStolenEvent;
import mot.common.event.TooManySpiesEvent;
import mot.common.lang.L;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.technology.Biotechnology;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.spies.SpyRole;

/**
 * This test harness tests that players can use spies to steal technology
 * sabotage planets and hide. It also tests that spies stationed at home
 * find an kill enemy spies.
 */

public class SpyStatisticalTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(mot.test.SpyStatisticalTest.class);
        return suite;
    }

    public SpyStatisticalTest(String name) {
        super(name);
    }

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;

    private ComputerPlayer player1;
    private ComputerPlayer player2;
    private ComputerPlayer player3;
    private ComputerPlayer player4;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;

    
    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
    }
    
    public void createMap() throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        RaceOptions options = OptionsFactory.getRaceOptions("Humans");
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);
        
        owner = player2;
    }

    public void createMapWith4Players() throws Exception {
        MotEngine.newInstance();

        String [] playerNames = { "Humans", "Bears", "Machines", "Birds" };
        RaceOptions options = OptionsFactory.getRaceOptions("Humans");
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);
        player3 = newPlayer(playerNames[2], green, options, playerNames);
        player4 = newPlayer(playerNames[3], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 300, 100);
        planet3 = newPlanet("SS3-A", ss3, player3);

        ss4 = newSolarSystem("SS4", 400, 100);
        planet4 = newPlanet("SS4-A", ss4, player4);

        owner = player2;
    }

    /**
     * Tests that a computer player has a quantity of spies based upon it total population / 100 + influence.
     * Tests that a player assigns its spies in accordance with the percentages defined in its personality.
     * Tests that a spies can steal an enemy player.
     * Tests that spies can be used to defend against enemy spies.
     * Tests espionage only as no war is declared!
     * 
     * In These Tests there is only one other player and the other layer is not the largest!
     */
    public void testWithTwoPlayers() throws Exception {
        Logger.logDebug("*******************************  SpyTest.testWithTwoPlayers() *************************");
        createMap();
        MotEngine.getInstance().getProxyManager().add(this);

        planet1.setPopulation(10);
        planet2.setPopulation(10);
        
        assertEquals("Has correct Qty Spies", 0, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 0, player2.getQtySpies());
        
        planet1.setSize(800);
        planet1.setPopulation(800.0);
        planet2.setSize(400);
        planet2.setPopulation(400.0);
        
        assertEquals("Has correct Qty Spies", 15, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 7, player2.getQtySpies());
        
        player1.addTechnology(tech_galaxyNetI);
        player2.addTechnology(tech_galaxyNetI);
        
        assertEquals("Has correct Qty Spies", 17, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 9, player2.getQtySpies());
        
        player1.getIntelligence().assignSpies();
        
        assertEquals("Have correct spy percents", 0.5, player1.getPlayerCharacter().getSpyDefendPercent(), 0.0);
        assertEquals("Have correct spy percents", 0.25, player1.getPlayerCharacter().getSpyAttackLargestPercent(), 0.0);
        assertEquals("Have correct spy percents", 0.25, player1.getPlayerCharacter().getSpyAttackMostHatedPercent(), 0.0);
        
        SpyRole sr = MotEngine.instance.getSpies().getDefendingSpies(player1);
        assertEquals("Player1 Spies are defending", 9, sr.getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_DEFEND, sr.getTaskDescription());
        assertEquals("Player1 Spies are defending", player1, sr.getTarget());

        SpyRole [] srs = MotEngine.instance.getSpies().getSpies(player1, player2);
        assertEquals("Player1 Spies are attacking", 8, srs[0].getQtySpies());
        assertEquals("Player1 Spies are attacking", L.SPY_ROLE_ESPIONAGE, srs[0].getTaskDescription());
        assertEquals("Player1 Spies are attacking", player2, srs[0].getTarget());

        assertNull("Player2 has no spies",   MotEngine.instance.getSpies().getSpies(player2, player1) );   
        
        // NOW TEST THAT SPYING WORKS - CAN STEAL TECH AND SABOTAGE!
        int qtyTechs = player1.getTechnologies().getQuantity();
        
        // Player1 has Spy Qty Adv 8 and Spy Adv of 0 => Success of 46 in 1000
        for (int i=0; i<1000; i++) {
            MotEngine.instance.getSpies().endTurn();
        }
        int newQtyTechs = player1.getTechnologies().getQuantity();
        int qtyTechStolen1 = newQtyTechs - qtyTechs;
        qtyTechs = newQtyTechs;
        assertEquals("No Technology to steel", 0, qtyTechStolen1);

        for (int i=0; i<100;i++) {
            player2.addTechnology(createTestWeapon(i));
        }
        
        // Player1 has Spy Qty Adv 8 and Spy Adv of 0 => Success of 36 in 1000
        for (int i=0; i<1000; i++) {
            MotEngine.instance.getSpies().endTurn();
        }
        newQtyTechs = player1.getTechnologies().getQuantity();
        qtyTechStolen1 = newQtyTechs - qtyTechs;
        qtyTechs = newQtyTechs;
        assertTrue("Successful spies is "+qtyTechStolen1, qtyTechStolen1 >=30 && qtyTechStolen1 <= 90);
        assertEquals("Successful spies", qtyTechStolen1, this.qtyTechStolen1);
        this.qtyTechStolen1 = 0;

        tech_bioCloak1.setSpyRating(5);
        player1.addTechnology(tech_bioCloak1);
        
        // Player1 has Spy Qty Adv 8 and Spy Adv of 5 => Success of 63 in 1000.
        for (int i=0; i<1000; i++) {
            MotEngine.instance.getSpies().endTurn();
        }
        assertTrue("Successful spies is "+this.qtyTechStolen1, this.qtyTechStolen1 >= 39 && this.qtyTechStolen1 <= 105);
        this.qtyTechStolen1 = 0;
        
        // TEST THAT DEFENDING AGAINST SPIES WORKS
        player2.getIntelligence().assignSpies();

        sr = MotEngine.instance.getSpies().getDefendingSpies(player2);
        assertEquals("Player2 Spies are defending", 5, sr.getQtySpies());
        assertEquals("Player2 Spies are defending", L.SPY_ROLE_DEFEND, sr.getTaskDescription());
        assertEquals("Player2 Spies are defending", player2, sr.getTarget());
        assertEquals("Player1 Spies are espionage", 8, MotEngine.instance.getSpies().getQtySpiesEspionage(player1, player2));
        
        player2.addTechnology(tech_bioCloak1);
        
        // Player1 has Spy Qty Adv 4 and Spy Adv of 0 => Success of 36 in 1000.
        for (int i=0; i<1000; i++) {
            MotEngine.instance.getSpies().endTurn();
        }
        assertTrue("Successful spies is "+this.qtyTechStolen1, this.qtyTechStolen1 >= 18 && this.qtyTechStolen1 <= 73);
        this.qtyTechStolen1 = 0;
    }

    /**
     * Tests that a player has a quantity of spies based upon it total population / 100 + influence.
     * Tests that a player assigns its spies in accordance with the percentages defined in its personality.
     * Tests that a spies can sabotage an enemy player.
     */
    public void testWithFourPlayers() throws Exception {
        Logger.logDebug("*******************************  SpyTest.testWithFourPlayers() *************************");
        createMapWith4Players();
        MotEngine.getInstance().getProxyManager().add(this);

        planet1.setSize(800);
        planet1.setPopulation(800.0);
        planet2.setSize(400);
        planet2.setPopulation(400.0);
        planet3.setSize(1000);
        planet3.setPopulation(1000.0);
        planet4.setSize(100);
        planet4.setPopulation(100.0);
        
        assertEquals("Has correct Qty Spies", 15, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 7, player2.getQtySpies());
        assertEquals("Has correct Qty Spies", 19, player3.getQtySpies());
        assertEquals("Has correct Qty Spies", 1, player4.getQtySpies());

        player1.getIntelligence().assignSpies();
        
        assertEquals("Have correct spy percents", 0.5, player1.getPlayerCharacter().getSpyDefendPercent(), 0.0);
        assertEquals("Have correct spy percents", 0.25, player1.getPlayerCharacter().getSpyAttackLargestPercent(), 0.0);
        assertEquals("Have correct spy percents", 0.25, player1.getPlayerCharacter().getSpyAttackMostHatedPercent(), 0.0);
        
        SpyRole sr = MotEngine.instance.getSpies().getDefendingSpies(player1);
        assertEquals("Player1 Spies are defending", 8, sr.getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_DEFEND, sr.getTaskDescription());
        assertEquals("Player1 Spies are defending", player1, sr.getTarget());

        SpyRole [] srs = MotEngine.instance.getSpies().getSpies(player1, player2);
        assertEquals("Player1 Spies are attacking", 4, srs[0].getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_ESPIONAGE, srs[0].getTaskDescription());
        assertEquals("Player1 Spies are attacking", player2, srs[0].getTarget());

        srs = MotEngine.instance.getSpies().getSpies(player1, player3);
        assertEquals("Player1 Spies are attacking", 3, srs[0].getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_ESPIONAGE, srs[0].getTaskDescription());
        assertEquals("Player1 Spies are attacking", player3, srs[0].getTarget());
        
        MotEngine.getInstance().getRelations().setWar(player1, player2);
        player1.getIntelligence().setDateOfLastCalculation(0);
        player1.getIntelligence().assignSpies();
        
        srs = MotEngine.instance.getSpies().getSpies(player1, player2);
        assertEquals("Player1 Spies are attacking", 4, srs[0].getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_ESPIONAGE, srs[0].getTaskDescription());
        assertEquals("Player1 Spies are attacking", player2, srs[0].getTarget());
        assertEquals("Player1 Spies are attacking", 3, srs[1].getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE__SABOTAGE, srs[1].getTaskDescription());
        assertEquals("Player1 Spies are attacking", player2, srs[1].getTarget());
        Biotechnology bioCloak = (Biotechnology) tech_bioCloak1.cloneTechnology();
        bioCloak.setSpyRating(10);
        player1.addTechnology(bioCloak);
        
        // Player1 has Spy Qty Adv 4 and Spy Adv of 10 => Success of 18 in 1000 for espoinage
        // and sabotage.
        for (int i=0; i<1000; i++) {
            MotEngine.instance.getSpies().endTurn();
        }
        assertEquals("No Technology to steel", 0, qtyTechStolen1);
        assertTrue("Successful spies is "+qtySabotage1, qtySabotage1 >= 30 && qtySabotage1 <= 78);
        qtyTechStolen1 = 0;
    }
    
    int qtyTechStolen1 = 0;
    int qtySabotage1 = 0;

    int qtyTechStolen2 = 0;
    int qtySabotage2 = 0;
    
    int tooManySpies = 0;

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
        Logger.logDebug("EVENT "+e.toString());
        if ( e instanceof TechnologyStolenEvent) {
            TechnologyStolenEvent event = (TechnologyStolenEvent)e;
            Player receiver = getPlayer(event.getReceiver());
            if (receiver == player1) {
                qtyTechStolen1++;
            } else {
                qtyTechStolen2++;
            }
        } else if ( e instanceof SabotageEvent) {
            SabotageEvent event = (SabotageEvent)e;
            Player player = getPlayer(event.getPlayer());
            if (player == player1) {
                qtySabotage1++;
            } else {
                qtySabotage2++;
            }
        } else if ( e instanceof TooManySpiesEvent) {
            tooManySpies++;
        }
    }

    private Player owner;    
	public Player getOwner() {
        return owner;
    }

    
    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
}