/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.PlayerColor;
import mot.common.event.MotEvent;
import mot.common.event.SabotageEvent;
import mot.common.event.TechnologyStolenEvent;
import mot.common.event.TooManySpiesEvent;
import mot.common.lang.L;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.command.SetRecruiteSpyEffortCmd;
import mot.engine.command.SpyRoleAddEspionageCmd;
import mot.engine.command.SpyRoleAddSabotageCmd;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.CrisisFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.setup.UserPlayerFactory;
import mot.engine.spies.Spies;
import mot.engine.spies.SpyCenter;
import mot.engine.spies.SpyRole;
import mot.engine.spies.SpyRoleEspionage;
import mot.engine.spies.SpyRoleSabotage;

/**
 * This test harness tests that players can use spies to steal technology
 * sabotage planets and hide. It also tests that spies stationed at home
 * find an kill enemy spies.
 */

public class SpyTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        Logger.setFull();
        TestSuite suite = new TestSuite(mot.test.SpyTest.class);
        return suite;
    }

    public SpyTest(String name) {
        super(name);
    }

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;

    private ComputerPlayer player1;
    private ComputerPlayer player2;
    private ComputerPlayer player3;
    private ComputerPlayer player4;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;

    
    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        Logger.setInMemory(false);
        Logger.setDebug();
    }
    
    public void createMap() throws Exception {
        Logger.setDebug();
        String [] playerNames = { "Humans", "Bears" };
        RaceOptions options = OptionsFactory.getRaceOptions("Humans");
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);
        
        owner = player2;
        
        player1.removeAllTechnologies();
        player2.removeAllTechnologies();
        
        player1.addTechnology(tech_photon100);
        player2.addTechnology(tech_photon100);
    }

    public void createMapWith4Players() throws Exception {
        MotEngine.newInstance();

        String [] playerNames = { "Humans", "Bears", "Machines", "Birds" };
        RaceOptions options = OptionsFactory.getRaceOptions("Humans");
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);
        player3 = newPlayer(playerNames[2], green, options, playerNames);
        player4 = newPlayer(playerNames[3], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 300, 100);
        planet3 = newPlanet("SS3-A", ss3, player3);

        ss4 = newSolarSystem("SS4", 400, 100);
        planet4 = newPlanet("SS4-A", ss4, player4);

        owner = player2;
    }
    
    /**
     * Tests that a computer player has a quantity of spies based upon it total population / 100 + influence.
     * Tests that a player assigns its spies in accordance with the percentages defined in its personality.
     * Tests that a spies can steal an enemy player.
     * Tests that spies can be used to defend against enemy spies.
     * Tests espionage only as no war is declared!
     * 
     * In These Tests there is only one other player and the other layer is not the largest!
     */
    public void testEspionage() throws Exception {
        Logger.logDebug("*******************************  SpyTest.testEspionage() *************************");
        createMap();
        MotEngine.getInstance().getProxyManager().add(this);

        planet1.setPopulation(10);
        planet2.setPopulation(10);
        player1.endTurn();
        player2.endTurn();
                
        assertEquals("Has correct Qty Spies", 0, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 0, player2.getQtySpies());
        
        planet1.setSize(800);
        planet1.setPopulation(800.0);
        planet2.setSize(400);
        planet2.setPopulation(400.0);
        player1.endTurn();
        player2.endTurn();
        
        assertEquals("Has correct Qty Spies", 15, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 7, player2.getQtySpies());
        
        //
        // TEST SPIES ASSIGNED TO DEFEND WHEN NO CONTACT EXISTS BETWEEN PLAYERS
        //
        
        player1.removeAllTechnologies();
        player2.removeAllTechnologies();        
        player1.getIntelligence().assignSpies();
        
        SpyRole sr = MotEngine.instance.getSpies().getDefendingSpies(player1);
        assertTrue(MotEngine.instance.getRelationsNotNull().hasDiplomaticRelations(player1, player2));
        assertEquals("Player1 Spies are defending", 15, sr.getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_DEFEND, sr.getTaskDescription());
        assertEquals("Player1 Spies are defending", player1, sr.getTarget());

        //
        // TEST GALAXY NET INCREASES QTY SPIES AND SPIES ARE ASSIGNED TO TASKS BY CHARACTER PERCENTAGES
        //

        player1.addTechnology(tech_photon100);
        player2.addTechnology(tech_photon100);
        player1.addTechnology(tech_galaxyNetI);
        player2.addTechnology(tech_galaxyNetI);
        MotEngine.instance.getRelationsNotNull().updateHasDiplomaticRelations();
        
        assertTrue(MotEngine.instance.getRelationsNotNull().hasDiplomaticRelations(player1, player2));
        assertEquals("Has correct Qty Spies", 17, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 9, player2.getQtySpies());
        
        player1.getIntelligence().setDateOfLastCalculation(0);
        player1.getStratagist().setDateOfLastCalculation(0);
        player1.getIntelligence().assignSpies();
        
        assertEquals("Have correct spy percents", 0.5, player1.getPlayerCharacter().getSpyDefendPercent(), 0.0);
        assertEquals("Have correct spy percents", 0.25, player1.getPlayerCharacter().getSpyAttackLargestPercent(), 0.0);
        assertEquals("Have correct spy percents", 0.25, player1.getPlayerCharacter().getSpyAttackMostHatedPercent(), 0.0);
        
        sr = MotEngine.instance.getSpies().getDefendingSpies(player1);
        assertEquals("Player1 Spies are defending", 9, sr.getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_DEFEND, sr.getTaskDescription());
        assertEquals("Player1 Spies are defending", player1, sr.getTarget());

        SpyRole [] srs = MotEngine.instance.getSpies().getSpies(player1, player2);
        assertEquals("Player1 Spies are attacking", 8, srs[0].getQtySpies());
        assertEquals("Player1 Spies are attacking", L.SPY_ROLE_ESPIONAGE, srs[0].getTaskDescription());
        assertEquals("Player1 Spies are attacking", player2, srs[0].getTarget());

        sr = MotEngine.instance.getSpies().getDefendingSpies(player2);
        assertEquals("Player1 Spies are defending", 7, sr.getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_DEFEND, sr.getTaskDescription());
        assertEquals("Player1 Spies are defending", player2, sr.getTarget());
        
        //
        // TEST ESPIONAGE CAN STEAL TECH
        //
        
        player2.addTechnology(tech_armour25);
        player2.setFunds(5000);
        // Player1 has Spy Qty Adv 1 and Spy Adv of 0 => Random between 0 and 62 must be 0 for success
        Utils.clearAndSetNextRandomInt(1, 37);
        MotEngine.instance.getSpies().endTurn();
        
        assertEquals("Nothing stolen", 2, player1.getTechnologies().getQuantity());
        
        lastEvent = null;
        // Player1 has Spy Qty Adv 1 and Spy Adv of 0 => Random between 0 and 62 must be 0 for success
        Utils.clearAndSetNextRandomInt(0, 37);
        MotEngine.instance.getSpies().endTurn();
        
        assertEquals("Stolen a tech", tech_armour25, ((TechnologyStolenEvent)lastEvent).getTechnology());
        assertEquals(tech_galaxyNetI, tech_photon100, tech_armour25, player1.getTechnologies());
        assertEquals(1, player2.getQtyAdditionalSpies());
        assertEquals(10, player2.getQtySpies());

        //
        // TEST ESPIONAGE WHEN NO TECH TO STEAL
        //
        
        // Player1 has Spy Qty Adv 1 and Spy Adv of 0 => Random between 0 and 62 must be 0 for success
        Utils.clearAndSetNextRandomInt(0, 37);
        MotEngine.instance.getSpies().endTurn();
        
        assertEquals("No Technology to steel", 3, player1.getTechnologies().getQuantity());

        player2.addTechnology(tech_battleStationI);
        player2.addTechnology(tech_energy10);
        player2.addTechnology(tech_energy20);
        player2.addTechnology(tech_fusion25x);
        player2.addTechnology(tech_flyer100);
        
        // Player1 has Spy Qty Adv 1 and Spy Adv of 0 => Random between 0 and 62 must be 0 for success
        Utils.clearAndSetNextRandomInt(0, 37);
        // Player 2 has 5 techs to steel of which we will choose index 2
        Utils.setNextRandomInt(2, 5);
        MotEngine.instance.getSpies().endTurn();

        assertEquals("Stolen a tech", 4, player1.getTechnologies().getQuantity());
        assertEquals("Stolen a tech", tech_galaxyNetI, player1.getTechnologies().getAllAsArray()[0]);
        assertEquals("Stolen a tech", tech_energy10, player1.getTechnologies().getAllAsArray()[1]);
        assertEquals("Stolen a tech", tech_photon100, player1.getTechnologies().getAllAsArray()[2]);
        assertEquals("Stolen a tech", tech_armour25, player1.getTechnologies().getAllAsArray()[3]);
        
        //
        // TEST ESPIONAGE  - BIO Cloak improves spying ability
        //
        
        player1.addTechnology(tech_bioCloak1);
        // Player1 has Spy Qty Adv 1 and Spy Adv of 1 => Random between 0 and 58 must be 0 for success
        Utils.clearAndSetNextRandomInt(5, 33);
        MotEngine.instance.getSpies().endTurn();

        
        //
        // TEST ESPIONAGE  - Defending spies makes espionage harder
        //
        
        player2.getIntelligence().assignSpies();
        sr = MotEngine.instance.getSpies().getDefendingSpies(player2);
        
        assertEquals("Player1 Spies are defending", 7, sr.getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_DEFEND, sr.getTaskDescription());
        assertEquals("Player1 Spies are defending", player2, sr.getTarget());
        
        // Player1 has Spy Qty Adv 3 and Spy Adv of 1 => Random between 0 and 83 must be 0 for success
        Utils.clearAndSetNextRandomInt(5, 33);
        MotEngine.instance.getSpies().endTurn();

        assertEquals(tech_battleStationI, tech_flyer100, tech_energy20, tech_fusion25x, player2.getStealableTechnologies(player1));

        //
        // Test Theories are NOT stealable
        //
        
        player2.addTechnology(tech_armour200);
        player2.addTechnology(tech_researchLabI);
        
        assertEquals(tech_battleStationI, tech_flyer100, tech_energy20, tech_armour200, tech_fusion25x, player2.getStealableTechnologies(player1));

    }

    /**
     * Tests that a player distributes its spies to different players in the game.
     */
    public void testEspionageMultiplePlayers() throws Exception {
        Logger.logDebug("*******************************  SpyTest.testEspionageMultiplePlayers() *************************");
        createMapWith4Players();
        MotEngine.getInstance().getProxyManager().add(this);

        planet1.setSize(800);
        planet1.setPopulation(800.0);
        planet2.setSize(400);
        planet2.setPopulation(400.0);
        planet3.setSize(1000);
        planet3.setPopulation(1000.0);
        planet4.setSize(100);
        planet4.setPopulation(100.0);
        
        player1.calculateQtySpies();
        player2.calculateQtySpies();
        player3.calculateQtySpies();
        player4.calculateQtySpies();
        
        assertEquals("Has correct Qty Spies", 15, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 7, player2.getQtySpies());
        assertEquals("Has correct Qty Spies", 19, player3.getQtySpies());
        assertEquals("Has correct Qty Spies", 1, player4.getQtySpies());
        assertEquals(player2, player1.getStratagist().getMostHatedPlayer().player);

        player1.getIntelligence().assignSpies();
        
        assertEquals("Have correct spy percents", 0.5, player1.getPlayerCharacter().getSpyDefendPercent(), 0.0);
        assertEquals("Have correct spy percents", 0.25, player1.getPlayerCharacter().getSpyAttackLargestPercent(), 0.0);
        assertEquals("Have correct spy percents", 0.25, player1.getPlayerCharacter().getSpyAttackMostHatedPercent(), 0.0);
        
        SpyRole sr = MotEngine.instance.getSpies().getDefendingSpies(player1);
        assertEquals("Player1 Spies are defending", 8, sr.getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_DEFEND, sr.getTaskDescription());
        assertEquals("Player1 Spies are defending", player1, sr.getTarget());

        SpyRole [] srs = MotEngine.instance.getSpies().getSpies(player1, player2);
        assertEquals("Player1 Spies espionage most hated player", 4, srs[0].getQtySpies());
        assertEquals("Player1 Spies espionage most hated player", L.SPY_ROLE_ESPIONAGE, srs[0].getTaskDescription());
        assertEquals("Player1 Spies espionage most hated player", player2, srs[0].getTarget());

        srs = MotEngine.instance.getSpies().getSpies(player1, player3);
        assertEquals("Player1 Spies espionage biggest player", 3, srs[0].getQtySpies());
        assertEquals("Player1 Spies espionage biggest player", L.SPY_ROLE_ESPIONAGE, srs[0].getTaskDescription());
        assertEquals("Player1 Spies espionage biggest player", player3, srs[0].getTarget());
    }
    
    /**
     * Tests that a player assigns spies to sabotage when at war AND that sabotage does work.
     */
    public void testSabotage() throws Exception {
        Logger.logDebug("*******************************  SpyTest.testSabotage() *************************");
        createMap();
        MotEngine.getInstance().getProxyManager().add(this);

        planet1.setSize(800);
        planet1.setPopulation(800.0);
        planet2.setSize(400);
        planet2.setPopulation(400.0);
        
        player1.calculateQtySpies();
        player2.calculateQtySpies();
        
        assertEquals("Has correct Qty Spies", 15, player1.getQtySpies());
        assertEquals("Has correct Qty Spies", 7, player2.getQtySpies());

        //
        // TEST SPIES ASSIGNED TO SABOTAGE WHEN AT WAR
        //
        
        MotEngine.getInstance().getRelations().setWar(player1, player2);
        player1.getIntelligence().setDateOfLastCalculation(0);
        player1.getIntelligence().assignSpies();
        
        SpyRole [] srs = MotEngine.instance.getSpies().getSpies(player1, player2);
        assertEquals("Player1 Spies are attacking", 4, srs[0].getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE_ESPIONAGE, srs[0].getTaskDescription());
        assertEquals("Player1 Spies are attacking", player2, srs[0].getTarget());
        assertEquals("Player1 Spies are attacking", 3, srs[1].getQtySpies());
        assertEquals("Player1 Spies are defending", L.SPY_ROLE__SABOTAGE, srs[1].getTaskDescription());
        assertEquals("Player1 Spies are attacking", player2, srs[1].getTarget());
        
        //
        // TEST SABOTAGE WORKS
        //
        
        // Espionage fails
        Utils.clearAndSetNextRandomInt(1);
        // Player1 has to roll a 0 in a random between 0 adn 166 to sabotage
        Utils.setNextRandomInt(0, 55);
        // Select planet at index 0
        Utils.setNextRandomInt(0, 1);
        // Select sabotage population
        Utils.setNextRandomInt(1, 3);
        // Select sabotage 5 + 3 population
        Utils.setNextRandomInt(3, 10);
        
        MotEngine.instance.getSpies().endTurn();

        assertEquals(planet2.getName(), ((SabotageEvent)lastEvent).getPlanet());
        assertEquals(0, ((SabotageEvent)lastEvent).getQtyFactoriesSabotaged());
        assertEquals(0, ((SabotageEvent)lastEvent).getQtyProductionSabotaged());
        assertEquals(8, ((SabotageEvent)lastEvent).getQtyPopulationSabotaged());
        assertEquals(392, planet2.getPopulation(), 0.01);
    }

    public void testPsiCrisis() throws Exception {
        Logger.logDebug("*******************************  SpyTest.testPsiCrisis() *************************");
        createMap();
        MotEngine.getInstance().getProxyManager().add(this);

        player1.addTechnology(tech_photon100);
        player2.addTechnology(tech_photon100);
        assertTrue(MotEngine.instance.getRelationsNotNull().hasDiplomaticRelations(player1, player2));

        //
        // TEST ESPIONAGE MORE LIKELY TO SUCCEED
        //
        
        Spies spies = MotEngine.getInstance().getSpiesNotNull();
        spies.removeSpies(player1);
        spies.addSpyRole(new SpyRoleEspionage(player1, player2, 1));
        player2.addTechnology(tech_armour25);
        
        // Player1 has Spy Qty Adv 1 and Spy Adv of 0 => Random between 0 and 166 must be 0 for success
        Utils.clearAndSetNextRandomInt(1, 166); // espionage fails
        lastEvent = null;
        MotEngine.instance.getSpies().endTurn();
        
        assertNull(lastEvent);
        assertEquals(false, player1.hasDiscovered(tech_armour25));
        
        // Player1 has Spy Qty Adv 1 and Spy Adv of 0 => Random between 0 and 166 must be 0 for success
        Utils.clearAndSetNextRandomInt(1, 166); // espionage fails
        Utils.clearAndSetNextRandomProbability(0.1, 0.15); // espionage succeeds (using crisis)
        lastEvent = null;
        CrisisFactory.createPsiCrisis(planet2);
        spies.endTurn();
        
        assertEquals(true, player1.hasDiscovered(tech_armour25));
        assertEquals(tech_armour25, ((TechnologyStolenEvent)lastEvent).getTechnology());
        
        planet2.setCrisis(null);

        //
        // TEST SABOTAGE MORE LIKELY TO SUCCEED
        //
        
        spies.removeSpies(player1);
        spies.addSpyRole(new SpyRoleSabotage(player1, player2, 1));
        planet2.setQtyFactories(10.0);

        // Player1 has Spy Qty Adv 1 and Spy Adv of 0 => Random between 0 and 166 must be 0 for success
        Utils.clearAndSetNextRandomInt(1, 166); // sabotage fails
        lastEvent = null;
        spies.endTurn();
        
        assertNull(lastEvent);
        assertEquals(10.0, planet2.getQtyFactories());
        
        // Player1 has Spy Qty Adv 1 and Spy Adv of 0 => Random between 0 and 166 must be 0 for success
        Utils.clearAndSetNextRandomInt(1, 166); // sabotage fails
        Utils.clearAndSetNextRandomProbability(0.1, 0.15); // sabotage succeeds (using crisis)
        Utils.setNextRandomInt(0, 1); // sabotage first planet in list
        Utils.setNextRandomInt(2, 3); // sabotage factories
        Utils.setNextRandomInt(2, 10); // sabotage 7 factories
        lastEvent = null;
        CrisisFactory.createPsiCrisis(planet2);
        MotEngine.instance.getSpies().endTurn();
        
        assertEquals(3.0, planet2.getQtyFactories());
        assertEquals(7, ((SabotageEvent)lastEvent).getQtyFactoriesSabotaged());
        
    }
    public void testUserPlayerSpyRecruite() throws Exception {
        Logger.logDebug("*******************************  SpyTest.testWithFourPlayers() *************************");
        RaceOptions options = OptionsFactory.getRaceOptions("Humans");
        UserPlayer player = (UserPlayer) new UserPlayerFactory().create(options, "TestUser", null, PlayerColor.getColorForName("blue"), null, null, null);
        SpyCenter sc = player.getSpyCenter();
        
        assertEquals("No Effort", 0, player.getSpyRecruitEffortPerTurn());
        assertEquals("No Effort", 0, sc.getTotalEffort());
        assertEquals("No Spies", 0, sc.getQtyBuiltSpies());

        player.endTurn();
        
        assertEquals("No Effort", 0, player.getSpyRecruitEffortPerTurn());
        assertEquals("No Effort", 0, sc.getTotalEffort());
        assertEquals("No Spies", 0, sc.getQtyBuiltSpies());
        
        player.setFunds(1000);
        new SetRecruiteSpyEffortCmd(player, 10).run();
        assertEquals("Correct Effort", 10, player.getSpyRecruitEffortPerTurn());

        Utils.setNextRandomInt(1000);
        player.endTurn();
        
        assertEquals("Correct Effort", 10, sc.getTotalEffort());
        assertEquals("No Spies", 0, sc.getQtyBuiltSpies());

        sc.addEffort(90);        
        Utils.setNextRandomInt(1000);
        sc.endTurn(player);

        assertEquals("Correct Effort", 100, sc.getTotalEffort());
        assertEquals("No Spies", 0, sc.getQtyBuiltSpies());

        Utils.setNextRandomInt(50);
        sc.endTurn(player);

        assertEquals("Correct Effort", 0, sc.getTotalEffort());
        assertEquals("Has Spy", 1, sc.getQtyBuiltSpies());

        assertEquals("Player has correct Spies", 1, player.getQtyBuiltSpies());
        assertEquals("Player has correct Spies", 1, player.getQtySpies());
        
        player.addTechnology(tech_galaxyNetI);
        
        assertEquals("Player has correct Spies", 1, player.getQtyBuiltSpies());
        assertEquals("Player has correct Spies", 3, player.getQtySpies());
        
    }

    public void testCommands() throws Exception {
        Logger.logDebug("*******************************  SpyTest.testCommands() *************************");
        RaceOptions options = OptionsFactory.getRaceOptions("Humans");
        UserPlayer player = (UserPlayer) new UserPlayerFactory().create(options, "TestUser", null, PlayerColor.getColorForName("blue"), null, null, null);
        Player target = newPlayer("Humans", blue, options, null);
        SpyCenter sc = player.getSpyCenter();
        Spies spies = MotEngine.instance.getSpiesNotNull();
        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player);
        owner = player;
        MotEngine.getInstance().getProxyManager().add(this);

        sc.setQtyBuiltSpies(1);
        
        assertEquals("Has correct spy roles", null, spies.getDefendingSpies(player));
        
        player.endTurn();

        assertEquals("Has correct spy roles", 1, spies.getDefendingSpies(player).getQtySpies());

        sc.setQtyBuiltSpies(5);
        player.endTurn();

        assertEquals("Has correct spy roles", 5, spies.getDefendingSpies(player).getQtySpies());

        sc.setQtyBuiltSpies(7);
        spies.addDefendingSpies(player, 2);
        new SpyRoleAddSabotageCmd(player, target, 1).run();
        new SpyRoleAddEspionageCmd(player, target, 1).run();
        
        assertEquals("Has correct spy roles", 5, spies.getDefendingSpies(player).getQtySpies());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target).length);
        assertEquals("Has correct spy roles", SpyRoleSabotage.class, spies.getSpies(player, target)[0].getClass());
        assertEquals("Has correct spy roles", 1, spies.getSpies(player, target)[0].getQtySpies());
        assertEquals("Has correct spy roles", SpyRoleEspionage.class, spies.getSpies(player, target)[1].getClass());
        assertEquals("Has correct spy roles", 1, spies.getSpies(player, target)[1].getQtySpies());

        sc.setQtyBuiltSpies(9);
        spies.addDefendingSpies(player, 2);
        new SpyRoleAddEspionageCmd(player, target, 1).run();
        new SpyRoleAddSabotageCmd(player, target, 1).run();
        
        assertEquals("Has correct spy roles", 5, spies.getDefendingSpies(player).getQtySpies());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target).length);
        assertEquals("Has correct spy roles", SpyRoleSabotage.class, spies.getSpies(player, target)[0].getClass());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target)[0].getQtySpies());
        assertEquals("Has correct spy roles", SpyRoleEspionage.class, spies.getSpies(player, target)[1].getClass());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target)[1].getQtySpies());

        sc.setQtyBuiltSpies(8);
        player.endTurn();

        assertEquals("Got Event", 1, tooManySpies);

        assertEquals("Has correct spy roles", 5, spies.getDefendingSpies(player).getQtySpies());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target).length);
        assertEquals("Has correct spy roles", SpyRoleSabotage.class, spies.getSpies(player, target)[0].getClass());
        assertEquals("Has correct spy roles", 1, spies.getSpies(player, target)[0].getQtySpies());
        assertEquals("Has correct spy roles", SpyRoleEspionage.class, spies.getSpies(player, target)[1].getClass());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target)[1].getQtySpies());

        sc.setQtyBuiltSpies(6);
        player.endTurn();

        assertEquals("Got Event", 2, tooManySpies);

        assertEquals("Has correct spy roles", 5, spies.getDefendingSpies(player).getQtySpies());
        assertEquals("Has correct spy roles", 1, spies.getSpies(player, target).length);
        assertEquals("Has correct spy roles", SpyRoleEspionage.class, spies.getSpies(player, target)[0].getClass());
        assertEquals("Has correct spy roles", 1, spies.getSpies(player, target)[0].getQtySpies());

        sc.setQtyBuiltSpies(9);
        spies.addDefendingSpies(player, 3);
        new SpyRoleAddSabotageCmd(player, target, 2).run();
        new SpyRoleAddEspionageCmd(player, target, 2).run();

        assertEquals("Has correct spy roles", 4, spies.getDefendingSpies(player).getQtySpies());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target).length);
        assertEquals("Has correct spy roles", SpyRoleEspionage.class, spies.getSpies(player, target)[0].getClass());
        assertEquals("Has correct spy roles", 3, spies.getSpies(player, target)[0].getQtySpies());
        assertEquals("Has correct spy roles", SpyRoleSabotage.class, spies.getSpies(player, target)[1].getClass());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target)[1].getQtySpies());

        new SpyRoleAddSabotageCmd(player, target, -1).run();
        new SpyRoleAddEspionageCmd(player, target, -1).run();

        assertEquals("Has correct spy roles", 6, spies.getDefendingSpies(player).getQtySpies());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target).length);
        assertEquals("Has correct spy roles", SpyRoleEspionage.class, spies.getSpies(player, target)[0].getClass());
        assertEquals("Has correct spy roles", 2, spies.getSpies(player, target)[0].getQtySpies());
        assertEquals("Has correct spy roles", SpyRoleSabotage.class, spies.getSpies(player, target)[1].getClass());
        assertEquals("Has correct spy roles", 1, spies.getSpies(player, target)[1].getQtySpies());

        new SpyRoleAddSabotageCmd(player, target, -1).run();
        new SpyRoleAddEspionageCmd(player, target, -2).run();

        assertEquals("Has correct spy roles", 9, spies.getDefendingSpies(player).getQtySpies());
        assertEquals("Has correct spy roles", null, spies.getSpies(player, target));

    }
        
    int qtyTechStolen1 = 0;
    int qtySabotage1 = 0;

    int qtyTechStolen2 = 0;
    int qtySabotage2 = 0;
    
    int tooManySpies = 0;

    private MotEvent lastEvent;
    
	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
		lastEvent = e;
        Logger.logDebug("EVENT "+e.toString());
        if ( e instanceof TechnologyStolenEvent) {
            TechnologyStolenEvent event = (TechnologyStolenEvent)e;
            Player receiver = getPlayer(event.getReceiver());
            if (receiver == player1) {
                qtyTechStolen1++;
            } else {
                qtyTechStolen2++;
            }
        } else if ( e instanceof SabotageEvent) {
            SabotageEvent event = (SabotageEvent)e;
            Player player = getPlayer(event.getPlayer());
            if (player == player1) {
                qtySabotage1++;
            } else {
                qtySabotage2++;
            }
        } else if ( e instanceof TooManySpiesEvent) {
            tooManySpies++;
        }
    }

    private Player owner;    
	public Player getOwner() {
        return owner;
    }

    
    protected void tearDown() {

    }

    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
}