/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.GameConstants;
import mot.common.designs.CraftDesign;
import mot.common.designs.WeaponDesign;
import mot.common.event.BattleNewTurnEvent;
import mot.common.event.DestroyColonyEvent;
import mot.common.event.MotEvent;
import mot.common.event.PlanetBombardedEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.OptionsFactory;
import mot.common.setup.SetupOptions;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.vo.StrategistEventVO;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.ProjectWeapon;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.UserPlayer;
import mot.engine.players.generals.CraftTaskIdle;
import mot.engine.players.generals.CraftTaskSurpriseAttackStealColony;
import mot.engine.players.strategists.Stratagist;
import mot.engine.players.strategists.StratagistBasic;
import mot.engine.players.strategists.StrategistEventAnnoy;
import mot.engine.players.strategists.StrategistEventCaughtSabotaging;
import mot.engine.players.strategists.StrategistEventCaughtSpying;
import mot.engine.players.strategists.StrategistEventDeclareWar;
import mot.engine.players.strategists.StrategistEventDeclareWarOnEnemy;
import mot.engine.players.strategists.StrategistEventDemandAssets;
import mot.engine.players.strategists.StrategistEventEmpireEstablished;
import mot.engine.players.strategists.StrategistEventManager;
import mot.engine.players.strategists.StrategistEventRejectSwapAssets;
import mot.engine.players.strategists.StrategistEventRequestAllianceTreaty;
import mot.engine.players.strategists.StrategistEventRequestNonAgressionTreaty;
import mot.engine.players.strategists.StrategistEventRequestPeaceTreaty;
import mot.engine.players.strategists.StrategistEventRequestTradeTreaty;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.setup.MotFactory;
import mot.engine.transports.Transport;

/**
 * Tests that computer players make the correct choices between peace and war.
 */

public class StratagistTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        Logger.setLogLevelLoaded();
        Logger.setDebug();
        TestSuite suite = new TestSuite(StratagistTest.class);
        return suite;
    }

    public StratagistTest(String name) {
        super(name);
    }

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;
    private SolarSystem ss4;
    private SolarSystem ss5;
    private SolarSystem ss6;
    private SolarSystem ss7;

    private ComputerPlayer player1;
    private ComputerPlayer player2;
    private ComputerPlayer player3;
    private ComputerPlayer player4;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    private Planet planet4;
    private Planet planet5;
    private Planet planet6;
    private Planet planet7;
    
    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        Logger.setInMemory(false);
        Logger.setLogLevelLoaded();
        Logger.setDebug();
    }

    public void createMap2() throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions options = mot.common.setup.OptionsFactory.getGeneralOptions();
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, options, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);
        
        player1.setFunds(1000);
        player2.setFunds(1000);
    }


    public void createMap3() throws Exception {
        String [] races = {"Humans", "Bears", "Birds"};
        MotFactory.setupRelations(null, races, null, null, false);
        
        player1 = newPlayer(races[0], blue, OptionsFactory.getRaceOptions("Humans"), races);
        player2 = newPlayer(races[1], blue, OptionsFactory.getRaceOptions("Bears"), races);
        player3 = newPlayer(races[2], blue, OptionsFactory.getRaceOptions("Birds"), races);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 199, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 301, 100);
        planet3 = newPlanet("SS3-A", ss3, player3);

        player1.setFunds(1000);
        player2.setFunds(1000);
        player3.setFunds(1000);
    }


    public void createMap4() throws Exception {
        String [] races = {"Humans", "Bears", "Birds", "Ghosts"};
        MotFactory.setupRelations(null, races, null, null, false);
        
        player1 = newPlayer(races[0], blue, OptionsFactory.getRaceOptions("Humans"), races);
        player2 = newPlayer(races[1], blue, OptionsFactory.getRaceOptions("Bears"), races);
        player3 = newPlayer(races[2], blue, OptionsFactory.getRaceOptions("Birds"), races);
        player4 = newPlayer(races[3], blue, OptionsFactory.getRaceOptions("Ghosts"), races);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 199, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 301, 100);
        planet3 = newPlanet("SS3-A", ss3, player3);

        ss4 = newSolarSystem("SS4", 400, 100);
        planet4 = newPlanet("SS4-A", ss4, player4);
        
        player1.setFunds(1000);
        player2.setFunds(1000);
        player3.setFunds(1000);
        player4.setFunds(1000);
    }

    /**
     * Tests that computer players declare war when they are not at war with any other player AND
     * they have enough military crafts.
     */
    public void testWarDeclaration() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testWarDeclaration() *************************");
        createMap2();

        // Form fleets of three crafts!
        player1.getGeneral().setQtyCraftsInAttackFleet(3);
        player1.getPlayerCharacter().setDemandProbability(0.0);

        //
        // TEST STRATEGIST DOES NOT DECLARE WAR UNTIL IT HAS ENOUGH MILITARY CRAFTS
        //
        
        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player1, ss1);

        player1.getStratagist().endTurn();
        assertFalse(MotEngine.getInstance().getRelations().isWar(player1, player2));
        //craft1.getTask().removeCraftsFromTask();

        //
        // TEST WHEN STRATAGIST HAS ENOUGH MILITARY CRAFT AND NOT AT WAR IT DECLARES 
        // WAR ON MOST HATED PLAYER
        //
        
        Craft craft3 = newCraft(fighterDesign, player1, ss1);
        Craft craft4 = newCraft(fighterDesign, player2, ss2);

        player1.getStratagist().setDateOfLastCalculation(0);
        player1.getStratagist().endTurn();
        assertTrue(MotEngine.getInstance().getRelations().isWar(player1, player2));

    }

    /**
     * Tests that computer players declare war when they are not at war with any other player AND
     * they have enough military crafts.
     */
    public void testWarDeclarationForIsolationist() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testWarDeclarationForIsolationist() *************************");
        createMap2();

        // Form fleets of three crafts!
        player1.getGeneral().setQtyCraftsInAttackFleet(2);
        player1.getPlayerCharacter().setDemandProbability(0.0);
        player1.getPlayerCharacter().setIsolationist(true);

        //
        // TEST STRATEGIST DOES NOT DECLARE WAR UNTIL AFTER SPACE STATIONS ARE DISCOVERED
        //
        
        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player1, ss1);
        Craft craft3 = newCraft(fighterDesign, player1, ss1);

        player1.getStratagist().endTurn();
        
        assertTrue(player1.isIsolationist());
        assertFalse(MotEngine.getInstance().getRelations().isWar(player1, player2));

        player1.addTechnology(tech_spaceStation);
        player1.getStratagist().setDateOfLastCalculation(0);
        player1.getStratagist().endTurn();
        
        assertFalse(player1.isIsolationist());
        assertTrue(MotEngine.getInstance().getRelations().isWar(player1, player2));

    }

    /**
     * Tests that computer players sometimes make demands instead of declaring war directly.
     * If other player concedes to demand then computer players do not declare war.
     *  - NOTE The details of a demand are tested in RelationsTest
     */
    public void testMakeDemand() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testMakeDemand() *************************");
        createMap2();
        proxyOwner = player2;

        // Form fleets of three crafts!
        player1.getGeneral().setQtyCraftsInAttackFleet(2);
        player1.getPlayerCharacter().setQtyCraftToDeclareWar(2);
        player1.getPlayerCharacter().setQtyCraftPerPlanetToDeclareWar(2);
        player1.getPlayerCharacter().setDemandProbability(1.0);
        player2.addTechnology(tech_fusionBeam4);

        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player1, ss1);
        Craft craft3 = newCraft(fighterDesign, player2, ss2);
        
        //
        // PLAYER1 DEMANDS ASSETS FROM PLAYER2 AND PLAYER2 REJECTS SO PLAYER1 DECLARES WAR
        //

        player1.getStratagist().endTurn();
        assertTrue(player2.getStratagist().getEventManager().getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertTrue(player2.getStratagist().getEventManager().getEventsAsArray()[1] instanceof StrategistEventDemandAssets);
        assertEquals(player1, ((StrategistEventDemandAssets)player2.getStratagist().getEventManager().getEventsAsArray()[1]).getPlayer());
        assertTrue(MotEngine.getInstance().getRelations().isWar(player1, player2));
        

    }

    /**
     * Tests that computer players calculate relations correctly, gifts improve relations, spying
     * and sabotaging damage relations, tend to like other players of the same size, dislike other player
     * who has a planet I covet and that relations age to become player character defaults - Has a 
     * map of four players.
     */
    public void testOpponentRating() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testOpponentRating() *************************");
        createMap4();
        
        player1.addScanner(tech_photon400);
        player2.addScanner(tech_photon400);
        player3.addScanner(tech_photon400);
        player4.addScanner(tech_photon400);
        //MotEngine.instance.getRelations().updateHasDiplomaticRelations();
        
        assertEquals(true, player1.hasDiplomaticRelationsWith(player2));
        assertEquals(true, player1.hasDiplomaticRelationsWith(player3));
        assertEquals(true, player1.hasDiplomaticRelationsWith(player4));
        assertEquals(true, player2.hasDiplomaticRelationsWith(player1));
        assertEquals(true, player2.hasDiplomaticRelationsWith(player3));
        assertEquals(true, player2.hasDiplomaticRelationsWith(player4));
        assertEquals(true, player3.hasDiplomaticRelationsWith(player1));
        assertEquals(true, player3.hasDiplomaticRelationsWith(player2));
        assertEquals(true, player3.hasDiplomaticRelationsWith(player4));
        assertEquals(true, player4.hasDiplomaticRelationsWith(player1));
        assertEquals(true, player4.hasDiplomaticRelationsWith(player2));
        assertEquals(true, player4.hasDiplomaticRelationsWith(player3));

        assertEquals("Has correct relationship", 70, MotEngine.instance.getRelations().getRelation(player1, player2));
        assertEquals("Has correct relationship", 70, MotEngine.instance.getRelations().getRelation(player1, player3));
        assertEquals("Has correct relationship", 60, MotEngine.instance.getRelations().getRelation(player1, player4));
        
        assertEquals("Has correct relationship", 50, MotEngine.instance.getRelations().getRelation(player2, player3));
        assertEquals("Has correct relationship", 40, MotEngine.instance.getRelations().getRelation(player2, player4));

        assertEquals("Has correct relationship", 40, MotEngine.instance.getRelations().getRelation(player3, player4));
        
        // Player1 covets player2 planet!
        assertEquals("Have correct rating", 61, player1.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 70, player1.getStratagist().getRating(player3, true));
        assertEquals("Have correct rating", 60, player1.getStratagist().getRating(player4, true));
        assertEquals("Have correct Most Hated player", player4, player1.getStratagist().getMostHatedPlayer().player);

        // player2 covets player1 planet
        assertEquals("Have correct rating", 61, player2.getStratagist().getRating(player1, true));
        assertEquals("Have correct rating", 50, player2.getStratagist().getRating(player3, true));
        assertEquals("Have correct rating", 40, player2.getStratagist().getRating(player4, true));
        assertEquals("Have correct Most Hated player", player4, player2.getStratagist().getMostHatedPlayer().player);

        // player3 covets player4 planet and player4 is uncharasmatic
        assertEquals("Have correct rating", 70, player3.getStratagist().getRating(player1, true));
        assertEquals("Have correct rating", 50, player3.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 31, player3.getStratagist().getRating(player4, true));
        assertEquals("Have correct Most Hated player", player4, player3.getStratagist().getMostHatedPlayer().player);
        
        // player4 covets player3 planet
        assertEquals("Have correct rating", 60, player4.getStratagist().getRating(player1, true));
        assertEquals("Have correct rating", 40, player4.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 31, player4.getStratagist().getRating(player3, true));
        assertEquals("Have correct Most Hated player", player3, player4.getStratagist().getMostHatedPlayer().player);
        
        // MAKE PLAYER3 HAVE REALLY GOOD PLANET SO ALL COVVET IT
        planet3.setSize(200);
        planet3.setProductionBonus(3.0);

        // Player1 would covets player3 planet but it is out of range!
        assertEquals("Have correct rating", 61, player1.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 70, player1.getStratagist().getRating(player3, true));
        assertEquals("Have correct rating", 60, player1.getStratagist().getRating(player4, true));
        assertEquals("Have correct Most Hated player", player4, player1.getStratagist().getMostHatedPlayer().player);

        //player1.getEngines().getFastestEngine().setMaxTravelDistance(250);
        player1.getEngines().setEngine(tech_fusion40);

        // Player1 covets player3 planet!
        assertEquals("Have correct rating", 70, player1.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 61, player1.getStratagist().getRating(player3, true));
        assertEquals("Have correct rating", 60, player1.getStratagist().getRating(player4, true));
        assertEquals("Have correct Most Hated player", player4, player1.getStratagist().getMostHatedPlayer().player);

        // player2 covets player3 planet
        assertEquals("Have correct rating", 70, player2.getStratagist().getRating(player1, true));
        assertEquals("Have correct rating", 41, player2.getStratagist().getRating(player3, true));
        assertEquals("Have correct rating", 40, player2.getStratagist().getRating(player4, true));
        assertEquals("Have correct Most Hated player", player4, player2.getStratagist().getMostHatedPlayer().player);

        // MAKE PLAYER3 HAVE HUGE POP SO ALL DISLIKE HIM MORE
        // If size >= 50% bigger RATING -= (Percent Diff / 8).
        planet3.setSize(100);
        planet3.setProductionBonus(0.0);
        planet1.setPopulation(100.0);
        planet2.setPopulation(100.0);
        planet3.setPopulation(200.0);
        planet4.setPopulation(100.0);

        assertEquals("Correct Power", 100, player1.getPowerRating());
        assertEquals("Correct Power", 100, player2.getPowerRating());
        assertEquals("Correct Power", 200, player3.getPowerRating());
        assertEquals("Correct Power", 100, player4.getPowerRating());

        // Test power effects relationships
        // Player1 dislikes that player3 twice as big (also covets planet 3)
        MotEngine.instance.getRelations().setRelation(player1, player2, 50);
        MotEngine.instance.getRelations().setRelation(player1, player3, 50);
        MotEngine.instance.getRelations().setRelation(player1, player4, 50);
        planet3.setSize(200);
        assertEquals(planet3, ((StratagistBasic)player1.getStratagist()).getMostCovettedPlanet());
        assertEquals(player4, player1.getStratagist().getMostHatedPlayer().player);
        assertEquals("Have correct rating", 50, player1.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 35, player1.getStratagist().getRating(player3, true));
        assertEquals("Have correct rating", 50, player1.getStratagist().getRating(player4, true));

        // Player1 Steal tech from player2
        MotEngine.instance.getRelations().setRelation(player1, player2, 50);
        player2.addTechnology(createTestWeapon(1));
        player2.steelTechnology(player1, null);
        StrategistEventManager semP2 = player2.getStratagist().getEventManagerNotNull();
        assertEquals("Has correct events", 2, semP2.getEventsAsArray().length);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[1] instanceof StrategistEventCaughtSpying);
        assertEquals("Events effects ralationship", -5, semP2.getRelationshipEffect(player1));
        assertEquals("Has correct relationship", 39, MotEngine.instance.getRelations().getRelation(player1, player2));
        assertEquals("Have correct rating", 39, player1.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 34, player2.getStratagist().getRating(player1, true));

        // Player2 sabotage project production for player1
        MotEngine.instance.getRelations().setRelation(player1, player2, 50);
        planet2.setProduction(10);
        planet2.setProject(new ProjectWeapon(new WeaponDesign(tech_massDriver1)));
        Utils.setNextRandomInt(0);
        planet2.sabotage(player1);
        StrategistEventManager semP1 = player1.getStratagist().getEventManagerNotNull();
        assertEquals("Has correct events", 3, semP2.getEventsAsArray().length);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[1] instanceof StrategistEventCaughtSpying);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[2] instanceof StrategistEventCaughtSabotaging);
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[1] instanceof StrategistEventDeclareWar);
        assertEquals("Events effects ralationship", -5, semP1.getRelationshipEffect(player2));
        assertEquals("Events effects ralationship", -10, semP2.getRelationshipEffect(player1));
        assertEquals("Production has been destroyed", 0, planet1.getProduction());
        assertEquals("Has correct relationship", 29, MotEngine.instance.getRelations().getRelation(player1, player2));
        assertEquals("Have correct rating", 24, player1.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 19, player2.getStratagist().getRating(player1, true));
        
        // Player2 sabotage factories for player1
        MotEngine.instance.getRelations().setRelation(player1, player2, 50);
        planet2.setQtyFactories(5.0);
        Utils.setNextRandomInt(3);
        planet2.sabotage(player1);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[1] instanceof StrategistEventCaughtSpying);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[2] instanceof StrategistEventCaughtSabotaging);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[3] instanceof StrategistEventCaughtSabotaging);
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[1] instanceof StrategistEventDeclareWar);
        assertEquals("Events effects ralationship", -5, semP1.getRelationshipEffect(player2));
        assertEquals("Events effects ralationship", -15, semP2.getRelationshipEffect(player1));
        assertEquals("Factories have been destroyed", 0.0, planet2.getQtyFactories(), 0.0);
        assertEquals("Has correct relationship", 29, MotEngine.instance.getRelations().getRelation(player1, player2));
        assertEquals("Have correct rating", 24, player1.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 14, player2.getStratagist().getRating(player1, true));
        
        // Population is sabotaged - Rating is different due to power difference!
        MotEngine.instance.getRelations().setRelation(player1, player2, 50);
        Utils.setNextRandomInt(1);
        planet2.sabotage(player1);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[1] instanceof StrategistEventCaughtSpying);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[2] instanceof StrategistEventCaughtSabotaging);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[3] instanceof StrategistEventCaughtSabotaging);
        assertEquals("Has correct events", true, semP2.getEventsAsArray()[4] instanceof StrategistEventCaughtSabotaging);
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[1] instanceof StrategistEventDeclareWar);
        assertEquals("Events effects ralationship", -5, semP1.getRelationshipEffect(player2));
        assertEquals("Events effects ralationship", -20, semP2.getRelationshipEffect(player1));
        assertTrue("Power has dropped becuase population have been destroyed", player2.getPowerRating() <= 95 && player2.getPowerRating() >= 85);
        assertEquals("Has correct relationship", 29, MotEngine.instance.getRelations().getRelation(player1, player2));
        assertEquals("Have correct rating", 24, player1.getStratagist().getRating(player2, true));
        assertEquals("Have correct rating", 9, player2.getStratagist().getRating(player1, true));
        
        // Test StrategistEventDeclareWarOnEnemy event.
        // Create a third player at war with player 1 and test that when player 2 declares war
        // on the new player relations improve with player 1. 
        semP1.clearEvents();
        MotEngine.instance.getRelations().setRelation(player1, player2, 59);
        MotEngine.instance.getRelations().setRelation(player1, player3, 50);
        MotEngine.instance.getRelations().setWar(player1, player3);

        assertEquals("Have correct rating", 35, player1.getStratagist().getRating(player3, true));

        player3.diplomaticRequest(player2, DiplomaticProposal.getWar(), null);
        
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[0] instanceof StrategistEventDeclareWarOnEnemy);
        assertEquals("Events effects ralationship", 5, semP1.getRelationshipEffect(player2));
        assertEquals("Have correct rating", 64, player1.getStratagist().getRating(player2, true));
    }

    /**
     * Tests that computer players that have Strategist events are removed once they have expired.
     */
    public void testEventExpiry() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testEventExpiry() *************************");
        createMap4();

        MotEngine.instance.setDate(100);
        StrategistEventManager semP1 = player1.getStratagist().getEventManagerNotNull();
        semP1.add(new StrategistEventCaughtSpying(player2, tech_armour25));
        
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertEquals("Has correct events", true, semP1.getEventsAsArray()[1] instanceof StrategistEventCaughtSpying);

        assertEquals("Has Not Expired", false, semP1.getEventsAsArray()[1].hasExpired());
        
        MotEngine.instance.setDate(201);

        assertEquals("Has Expired", true, semP1.getEventsAsArray()[1].hasExpired());

        player1.endTurn();
        
        assertEquals("Removed", null, semP1.getDiplomaticEvents(false));
        assertEquals("<html>Act of spying (effects expired)<br/>Bears stole special , Armour25</html>", semP1.getDiplomaticEvents(true)[0].getMessage());
    }

    /**
     * Tests that computer players get reject a treaty if they have recently been asked for the same treaty.
     * Tests that computer players get annoyed if they are asked for the same treaty over and over again.
     */
    public void testAnnoy() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testAnnoy() *************************");
        createMap4();
        StrategistEventManager semP1 = player1.getStratagist().getEventManagerNotNull();
        Relation rel = MotEngine.instance.getRelations().getRelationship(player1, player2);
        rel.setValue(0);

        //        
        // TEST REQUEST FOR ALLIANCE TREATRY IS REMEMBERED
        //
        
        player1.diplomaticRequest(player2, DiplomaticProposal.getAlliance(), null);
        //player1.requestAliance(player2, null);
        
        assertFalse(rel.isAliance());
        assertEquals(2, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertTrue(semP1.getEventsAsArray()[1] instanceof StrategistEventRequestAllianceTreaty);

        player1.diplomaticRequest(player2, DiplomaticProposal.getAlliance(), null);
        //player1.requestAliance(player2, null);

        assertFalse(rel.isAliance());
        assertEquals(3, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventEmpireEstablished);
        assertTrue(semP1.getEventsAsArray()[1] instanceof StrategistEventRequestAllianceTreaty);
        assertTrue(semP1.getEventsAsArray()[2] instanceof StrategistEventAnnoy);
        
        //        
        // TEST REQUEST FOR NON AGRESSION TREATRY IS REMEMBERED
        //
        
        semP1.clearEvents();
        player1.diplomaticRequest(player2, DiplomaticProposal.getNonAggression(), null);
        //player1.requestNonAgression(player2, null);
        
        assertFalse(rel.isNonAgression());
        assertEquals(1, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRequestNonAgressionTreaty);
        
        player1.diplomaticRequest(player2, DiplomaticProposal.getNonAggression(), null);
        //player1.requestNonAgression(player2, null);

        assertFalse(rel.isNonAgression());
        assertEquals(2, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRequestNonAgressionTreaty);
        assertTrue(semP1.getEventsAsArray()[1] instanceof StrategistEventAnnoy);
        
        //        
        // TEST REQUEST FOR RESEARCH TREATRY IS REMEMBERED - TODO
        //
        
        semP1.clearEvents();
        player1.diplomaticRequest(player2, DiplomaticProposal.getResearch(), null);
        //player1.requestResearchTreaty(player2, null);
        
        assertNull(rel.getResearchAgreementTreaty());
        //assertEquals(1, semP1.getEventsAsArray().length);
        //assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRequestResearchTreaty);
        
        player1.diplomaticRequest(player2, DiplomaticProposal.getResearch(), null);
        //player1.requestResearchTreaty(player2, null);

        assertNull(rel.getResearchAgreementTreaty());
        //assertEquals(2, semP1.getEventsAsArray().length);
        //assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRequestResearchTreaty);
        //assertTrue(semP1.getEventsAsArray()[1] instanceof StrategistEventAnnoy);
        
        //        
        // TEST REQUEST FOR TRADE TREATRY IS REMEMBERED
        //
        
        semP1.clearEvents();
        player1.diplomaticRequest(player2, DiplomaticProposal.getTrade(), null);
        //player1.requestTradeRoute(player2, null);
        
        assertNull(rel.getTradeAgreementTreaty());
        assertEquals(1, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRequestTradeTreaty);
        
        player1.diplomaticRequest(player2, DiplomaticProposal.getTrade(), null);
        //player1.requestTradeRoute(player2, null);

        assertNull(rel.getTradeAgreementTreaty());
        assertEquals(2, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRequestTradeTreaty);
        assertTrue(semP1.getEventsAsArray()[1] instanceof StrategistEventAnnoy);
        
        //        
        // TEST REQUEST FOR PEACE TREATRY IS REMEMBERED
        //
        
        player1.diplomaticRequest(player2, DiplomaticProposal.getWar(), null);
        //player1.declareWar(player2, null, false);
        semP1.clearEvents();
        
        player1.diplomaticRequest(player2, DiplomaticProposal.getPeace(), null);
        //player1.requestPeace(player2, null);
        assertFalse(rel.isPeace());
        
        MotEngine.instance.setDate(MotEngine.instance.getDate()+50);
        player1.diplomaticRequest(player2, DiplomaticProposal.getPeace(), null);
        //player1.requestPeace(player2, null);
        
        assertFalse(rel.isPeace());
        assertEquals(1, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRequestPeaceTreaty);
        
        player1.diplomaticRequest(player2, DiplomaticProposal.getPeace(), null);
        //player1.requestPeace(player2, null);

        assertFalse(rel.isPeace());
        assertEquals(2, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRequestPeaceTreaty);
        assertTrue(semP1.getEventsAsArray()[1] instanceof StrategistEventAnnoy);
        
        //        
        // TEST REQUEST FOR SWAP IS REMEMBERED
        //

        MotEngine.instance.getRelations().setPeace(player1, player2);
        player1.setFunds(100000);
        player2.setFunds(100000);
        semP1.clearEvents();
        
        assertEquals(null, semP1.getEventsAsArray());
        
        player1.diplomaticRequest(player2, DiplomaticProposal.swap(100, 10000), null);
        //player1.requestSwapAssets(player2, new PlayerAssets(100), new PlayerAssets(10000));

        assertEquals(1, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRejectSwapAssets);
        
        player1.diplomaticRequest(player2, DiplomaticProposal.swap(100, 10000), null);
        //player1.requestSwapAssets(player2, new PlayerAssets(100), new PlayerAssets(10000));

        assertEquals(2, semP1.getEventsAsArray().length);
        assertTrue(semP1.getEventsAsArray()[0] instanceof StrategistEventRejectSwapAssets);
        assertTrue(semP1.getEventsAsArray()[1] instanceof StrategistEventAnnoy);
    }

    /**
     * Tests that computer players surrender to other players when they lose a colony to another player who 
     * is twice the size as them.
     */
    public void testSurrenderOnHardLevels() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testSurrenderOnHardLevels() *************************");
        createMap4();
        
        MotEngine.instance.setLevel(GameConstants.LEVEL_HARD_BONUS);
        Relations rels = MotEngine.instance.getRelationsNotNull();
        rels.setWar(player1, player2);
        
        // CREATE CRAFTS TO BOMBARD player2
        Craft craft1 = newCraft(defenderDesign, player1, ss2);
        Fleet fleet = FleetFactory.create(craft1);
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        
        // CREATE PLANET, MINING COLONY AND CRAFT TO TRANSFER TO ANOTHER PLAYER WHEN player2 SURRENDERS
        Craft craft6 = newCraft(fighterDesign, player2, ss3);
        SolarSystem ss5 = newSolarSystem("SS5", 10, 10);
        Planet planet5 = newPlanet("SS5-A", ss5, player2);
        planet5.setPopulation(5);
        Planet planet6 = newPlanet("SS5-B", ss1, null, 100, 0.2);
        newSpaceStation(planet5, planet6);
        
        assertEquals("Player 2 exists", 4, MotEngine.instance.getPlayers().getQuantity());
        assertEquals("Planet 2 has owner", player2, planet2.getOwner());
        assertEquals("Planet 5 has owner", player2, planet5.getOwner());
        assertEquals("Craft 6 has owner", player2, craft6.getOwner());

        // PLAYER1 BOMBARD PLAYER2 (planet2) - Causing player2 to surrender to player3
        fleet.bombard(planet2);
        player2.getStratagist().endTurn();
        
        assertEquals("Player 2 gone", 3, MotEngine.instance.getPlayers().getQuantity());
        assertEquals("Player 2 gone", 1, MotEngine.instance.getDeadPlayers().getQuantity());
        assertEquals("Player 2 gone", player2, MotEngine.instance.getDeadPlayers().getPlayers()[0]);
        assertEquals("Planet 2 has no owner", null, planet2.getOwner());
        assertEquals("Planet 5 has changed owner", player3, planet5.getOwner());
        assertEquals("Planet 6 has changed owner", player3, planet6.getOwner());
        assertEquals("Craft 6 has changed owner", player3, craft6.getOwner());
        
        assertEquals(true, planet6.isMiningColony());
        assertEquals(planet6, planet5.getMiningStation().getPlanet());
    }

    /**
     * Tests that computer players surrender to other players when they lose a colony to another player who 
     * is twice the size as them.
     */
    public void testSurrenderOnEasyLevels() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testSurrender() *************************");
        createMap4();
        
        Relations rels = MotEngine.instance.getRelationsNotNull();
        rels.setWar(player1, player2);
        
        // CREATE CRAFTS TO BOMBARD player2
        Craft craft1 = newCraft(defenderDesign, player1, ss2);
        Fleet fleet = FleetFactory.create(craft1);
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        fleet.add(newCraft(defenderDesign, player1, ss2));
        
        // CREATE PLANET, MINING COLONY AND CRAFT TO TRANSFER TO ANOTHER PLAYER WHEN player2 SURRENDERS
        Craft craft6 = newCraft(fighterDesign, player2, ss3);
        SolarSystem ss5 = newSolarSystem("SS5", 10, 10);
        Planet planet5 = newPlanet("SS5-A", ss5, player2);
        planet5.setPopulation(5);
        Planet planet6 = newPlanet("SS5-B", ss1, null, 100, 0.2);
        newSpaceStation(planet5, planet6);
        
        assertEquals("Player 2 exists", 4, MotEngine.instance.getPlayers().getQuantity());
        assertEquals("Planet 2 has owner", player2, planet2.getOwner());
        assertEquals("Planet 5 has owner", player2, planet5.getOwner());
        assertEquals("Craft 6 has owner", player2, craft6.getOwner());

        // PLAYER1 BOMBARD PLAYER2 (planet2) - Causing player2 to surrender to player3
        fleet.bombard(planet2);
        player2.getStratagist().endTurn();
        
        assertEquals("Player NOT gone", 4, MotEngine.instance.getPlayers().getQuantity());
        assertNull("Player NOT gone", MotEngine.instance.getDeadPlayers());
        
    }


    /**
     * Tests that computer players surrender to other players when they lose a colony to another player who 
     * is twice the size as them.
     */
    public void testSurrenderWhenBattleScheduled() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testSurrenderWhenBattleScheduled() *************************");
        createMap4();
        
        MotEngine.instance.setLevel(GameConstants.LEVEL_HARD_BONUS);
        Relations rels = MotEngine.instance.getRelationsNotNull();
        rels.setWar(player1, player2);
        
        // CREATE CRAFTS TO BOMBARD player2
        CraftDesign cd = new CraftDesign("TestDefender", tech_flyer60, tech_fusion40, tech_electronic40, tech_energy20, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4, tech_fusionBeam4);
        Craft craft1 = newCraft(cd, player1, ss2);
        Fleet fleet1 = FleetFactory.create(craft1);
        fleet1.add(newCraft(cd, player1, ss2));
        fleet1.add(newCraft(cd, player1, ss2));
        fleet1.add(newCraft(cd, player1, ss2));
        
        // CREATE PLANET AND FLEET TO TRANSFER TO ANOTHER PLAYER WHEN player2 SURRENDERS
        SolarSystem ss5 = newSolarSystem("SS5", 10, 10);
        Planet planet5 = newPlanet("SS5-A", ss5, player2);
        planet5.setPopulation(5);
        Craft craft6 = newCraft(fighterDesign, player2, ss5);
        Craft craft7 = newCraft(fighterDesign, player2, ss5);
        Fleet fleet2 = FleetFactory.create(craft6);
		fleet2.add(craft7);
        
        assertEquals("Player 2 exists", 4, MotEngine.instance.getPlayers().getQuantity());
        assertEquals("Planet 2 has owner", player2, planet2.getOwner());
        assertEquals("Planet 5 has owner", player2, planet5.getOwner());
        assertEquals("Fleet has owner", player2, fleet2.getCrafts()[0].getOwner());
        assertEquals("Fleet has owner", player2, fleet2.getCrafts()[1].getOwner());
        
        // CREATE BATTLE WITH player2 INSIDE
        Craft craft8 = newCraft(cd, player1, ss5);
        battle = newBattle(ss5);
        battle.join(fleet2, true);
        battle.join(craft8, true);


        // PLAYER1 BOMBARD PLAYER2 (planet2) - Causing player2 to surrender to player3
        fleet1.bombard(planet2);
        player2.getStratagist().endTurn();
        
        assertEquals("Player 2 gone", 3, MotEngine.instance.getPlayers().getQuantity());
        assertEquals("Player 2 gone", 1, MotEngine.instance.getDeadPlayers().getQuantity());
        assertEquals("Player 2 gone", player2, MotEngine.instance.getDeadPlayers().getPlayers()[0]);
        assertEquals("Planet 2 has no owner", null, planet2.getOwner());
        assertEquals("Planet 5 has changed owner", player3, planet5.getOwner());
        assertEquals("Fleet has changed owner", player3, craft6.getOwner());
        assertEquals("Fleet has changed owner", player3, craft7.getOwner());
        assertEquals("Fleet has changed owner", null, craft6.getFleet());
        assertEquals("Fleet has changed owner", null, craft7.getFleet());

        assertEquals("Battle gone", 0, MotEngine.instance.getBattles().getQtyBattles());

    }

    /**
     * Test computer players create Surprise Attack Task.
     */
    public void testSurpriseAttack() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testSurpriseAttack() *************************");
        
        setUp();
        
        MotEngine.instance.setLevel(GameConstants.LEVEL_HARD_BONUS);
        
	    String [] playerNames = { "Humans", "Bears" };
	    SetupOptions options = mot.common.setup.OptionsFactory.getGeneralOptions();
	    UserPlayer userPlayer = newUserPlayer(playerNames[0], blue, options);
	    player1 = newPlayer(playerNames[1], green, options, playerNames);
	
	    ss1 = newSolarSystem("SS1", 100, 100);
	    planet1 = newPlanet("SS1-A", ss1, userPlayer);
	    planet1.setWeapons(new Weapons(tech_missile6, tech_missile6));
	
	    ss2 = newSolarSystem("SS2", 150, 100);
	    planet2 = newPlanet("SS2-A", ss2, userPlayer);
	
	    ss3 = newSolarSystem("SS3", 200, 100);
	    planet3 = newPlanet("SS3-A", ss3, player1);
	    
	    userPlayer.setFunds(1000);
	    player1.setFunds(1000);
	    
    	proxyOwner = player1;
        MotEngine.getInstance().getProxyManager().add(this);

	    //
	    // Will not create surprise attack if cannot take planet easily (population too big to kill in one turn)
	    //

	    Craft coloniser = newCraft(coloniserDesign, player1, ss3);
	    Craft craft1 = newCraft(fighterDesign, player1, ss3);
	    Craft craft2 = newCraft(fighterDesign, player1, ss3);

	    player1.setNextCraftOrders(craft1);
	    player1.setNextCraftOrders(craft2);

	    player1.getStratagist().calculatePriorities();
	    
	    //MotEngine.instance.logSummary();

	    CraftTaskIdle task1 = (CraftTaskIdle)player1.getGeneral().getTasks()[0];
	    assertNull(coloniser.getTask());
	    assertEquals(task1, craft1.getTask());
	    assertEquals(task1, craft2.getTask());

	    //
	    // Create Task Surprise Attack
	    //

	    planet2.setPopulation(1.0);
	    player1.getStratagist().calculatePriorities();
	    
	    CraftTaskSurpriseAttackStealColony task2 = (CraftTaskSurpriseAttackStealColony)player1.getGeneral().getTasks()[1];
		Fleet fleet = task2.getFleet();

	    assertEquals(planet2, task2.getTarget().getPlanet());
	    assertEquals(craft1, craft2, coloniser, fleet.getCrafts());

	    eventList.clear();
	    player1.getGeneral().assignTaskToFleet(fleet);
	    endTurnUntilArrive(fleet, planet2);
	    
	    //MotEngine.instance.logSummary();
	    
	    assertEquals(planet2.getName(), ((DestroyColonyEvent)eventList.get(0)).getPlanet());
	    assertEquals(planet2.getName(), ((PlanetBombardedEvent)eventList.get(1)).getTarget());
	    assertEquals(player1, planet2.getOwner());
	    
	}

    public void testSurpriseAttackWithDispersedCraft() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testSurpriseAttackWithDispersedCraft() *************************");
        
        setUp();
        
        MotEngine.instance.setLevel(GameConstants.LEVEL_HARD_BONUS);
        
	    String [] playerNames = { "Humans", "Bears" };
	    SetupOptions options = mot.common.setup.OptionsFactory.getGeneralOptions();
	    UserPlayer userPlayer = newUserPlayer(playerNames[0], blue, options);
	    player1 = newPlayer(playerNames[1], green, options, playerNames);
	
	    ss1 = newSolarSystem("SS1", 100, 100);
	    planet1 = newPlanet("SS1-A", ss1, userPlayer);
	    planet1.setWeapons(new Weapons(tech_missile6, tech_missile6));
	
	    ss2 = newSolarSystem("SS2", 150, 100);
	    planet2 = newPlanet("SS2-A", ss2, userPlayer);
	    planet2.setPopulation(1.0);
	
	    ss3 = newSolarSystem("SS3", 200, 100);
	    planet3 = newPlanet("SS3-A", ss3, player1);

	    SolarSystem ss4 = newSolarSystem("SS3", 200, 90);
	    SolarSystem ss5 = newSolarSystem("SS3", 200, 110);
	    
	    userPlayer.setFunds(1000);
	    player1.setFunds(1000);
	    
    	proxyOwner = player1;
        MotEngine.getInstance().getProxyManager().add(this);

	    //
	    // Create Task Surprise Attack
	    //

	    Craft coloniser1 = newCraft(coloniserDesign, player1, ss3);
	    Craft craft1 = newCraft(fighterDesign, player1, ss3);
	    Craft craft2 = newCraft(fighterDesign, player1, ss4);

	    player1.setNextCraftOrders(craft1);
	    player1.setNextCraftOrders(craft2);
	    
	    assertEquals(CraftTaskIdle.class, craft1.getTask().getClass());
	    assertEquals(CraftTaskIdle.class, craft2.getTask().getClass());
	    
	    player1.getStratagist().calculatePriorities();

	    player1.setNextCraftOrders(craft1);
	    player1.setNextCraftOrders(craft2);
	    
	    //MotEngine.instance.logSummary();
	    
	    CraftTaskIdle taskIdle = (CraftTaskIdle)player1.getGeneral().getTasks()[0];
	    CraftTaskSurpriseAttackStealColony taskSA = (CraftTaskSurpriseAttackStealColony)player1.getGeneral().getTasks()[1];
		Fleet fleet = taskSA.getFleet();

	    assertEquals(taskSA, craft1.getTask());
	    assertEquals(taskIdle, craft2.getTask());
	    assertEquals(ss3, craft2.getNavigator().getDestination().getStellarObject());

	    endTurnUntilArrive(craft2, ss3);

	    assertEquals(planet2, taskSA.getTarget().getPlanet());
	    assertNotNull(fleet.getColoniser());
	    assertEquals(2, fleet.getQtyMilitaryCraft());

	    eventList.clear();
	    player1.getGeneral().assignTaskToFleet(fleet);
	    endTurnUntilArrive(fleet, planet2);
	    	    
	    //MotEngine.instance.logSummary();
	    
	    assertEquals(planet2.getName(), ((DestroyColonyEvent)eventList.get(0)).getPlanet());
	    assertEquals(planet2.getName(), ((PlanetBombardedEvent)eventList.get(1)).getTarget());
	    assertEquals(player1, planet2.getOwner());

	}

    public void testStrategistEvents() throws Exception {
        Logger.logDebug("*******************************  StratagistTest.testStrategistEvents() *************************");
        createMap4();
        player1.getStratagist().getEventManagerNotNull().clearEvents();
        player2.getStratagist().getEventManagerNotNull().clearEvents();
        player3.getStratagist().getEventManagerNotNull().clearEvents();
        player4.getStratagist().getEventManagerNotNull().clearEvents();

        assertEquals(70, MotEngine.instance.getRelations().getRelation(player2, player1));

        MotEngine.instance.getRelationsNotNull().setWar(player1, player2);
        MotEngine.instance.getRelationsNotNull().setWar(player2, player4);
        player1.addWeapon(tech_ancientWeapon);
        Stratagist stratagist1 = player1.getStratagist();
        Stratagist stratagist2 = player2.getStratagist();
        Stratagist stratagist3 = player3.getStratagist();
        Stratagist stratagist4 = player4.getStratagist();
        
        //
        // Test ColonyLost - invaded
        //

        ss5 = newSolarSystem("SS5", 150, 100);
        planet5 = newPlanet("SS5-A", ss5, player2);
        Transport transport1 = newTransportWithTroopsLoaded(planet1, planet5, 100);
        transport1.transferTo(new Orbit(planet5));
        
        transport1.getTask().deploy(transport1, ss4, planet5);
		StrategistEventVO[] events1 = stratagist1.getStrategistEvents(player2);
		StrategistEventVO[] events2 = stratagist2.getStrategistEvents(player1);
		StrategistEventVO[] events3a = stratagist3.getStrategistEvents(player1);
		StrategistEventVO[] events3b = stratagist3.getStrategistEvents(player2);
		StrategistEventVO[] events4a = stratagist4.getStrategistEvents(player1);
		StrategistEventVO[] events4b = stratagist4.getStrategistEvents(player2);
 
		assertEquals(player1, planet5.getOwner());
		assertNull(events1);
		assertEquals("Invasion of my colony on SS5-A (-10)", events2[0].getMessage());
		assertEquals("Expansionist Behaviour - Your invasion of SS5-A (-2)", events3a[0].getMessage());
		assertNull(events3b);
		assertEquals("Invasion of my enemy's planet SS5-A (+2)", events4a[0].getMessage());
		assertNull(events4b);
        
        //
        // Test ColonyLost - Genocide
        //

        ss6 = newSolarSystem("SS6", 150, 125);
        planet6 = newPlanet("SS6-A", ss6, player2);
        Craft bomber1 = newCraft(bomberDesign, player1, planet6);
        
        bomber1.bombard(planet6);
		events1 = stratagist1.getStrategistEvents(player2);
		events2 = stratagist2.getStrategistEvents(player1);
		events3a = stratagist3.getStrategistEvents(player1);
		events3b = stratagist3.getStrategistEvents(player2);
		events4a = stratagist4.getStrategistEvents(player1);
		events4b = stratagist4.getStrategistEvents(player2);
        
        assertFalse(planet6.hasOwner());
        
		assertNull(events1);
		assertEquals("Invasion of my colony on SS5-A (-10)", events2[0].getMessage());
		assertEquals("Destruction of my colony on SS6-A Genocide (-15)", events2[1].getMessage());
		assertEquals("Expansionist Behaviour - Your invasion of SS5-A (-2)", events3a[0].getMessage());
		assertEquals("Genocide - Killing all life on SS6-A (-5)", events3a[1].getMessage());
		assertNull(events3b);
		assertEquals("Invasion of my enemy's planet SS5-A (+2)", events4a[0].getMessage());
		assertNull(events4b);
 
        assertEquals(0, player1.getStratagist().getEventManagerNotNull().getRelationshipEffect(player2));
        assertEquals(-25, player2.getStratagist().getEventManagerNotNull().getRelationshipEffect(player1));
        assertEquals(-7, player3.getStratagist().getEventManagerNotNull().getRelationshipEffect(player1));
        assertEquals(2, player4.getStratagist().getEventManagerNotNull().getRelationshipEffect(player1));

        assertEquals(70, MotEngine.instance.getRelations().getRelation(player2, player1));

    }
    
	private Battle newBattle(SolarSystem ss) {
        return MotEngine.getInstance().getBattlesNotNull().getBattleAt(ss.getOrbit(), true);
    }

    protected void tearDown() {

    }

    private Player proxyOwner;
    private int battleTurn = -1;
    private Battle battle;
    private MotEvent lastEvent;

	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
		eventList.add(e);
        lastEvent = e;
        if ( e instanceof BattleNewTurnEvent) {
            BattleNewTurnEvent event = (BattleNewTurnEvent)e;
            Battle battle = MotEngine.instance.getBattles().getBattle( event.getBattle() );
            assertEquals("The Battle has started : We are on the correct turn", battleTurn++, battle.getTurn());
        }
    }

	public Player getOwner() {
        return proxyOwner;
    }


    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}