/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.technology.TechnologyBundle;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.engine.MotEngine;

public class TechnologyTest extends TestBase {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(TechnologyTest.class);
        return suite;
    }

    public TechnologyTest(String name) {
        super(name);
    }

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
    }

    public void testTechnologyList() throws Exception {
        Logger.logDebug("*******************************  TechnologyTest.testTechnologyList() *************************");
        
        Weapons list = new Weapons();

        list.add(tech_massDriver1);
        
        assertEquals(1, list.getQuantity());
        assertEquals(tech_massDriver1, list.getAllAsArray()[0]);
        
        list.add(tech_fusionBeam4);
        
        assertEquals(2, list.getQuantity());
        assertEquals(tech_massDriver1, list.getAllAsArray()[0]);
        assertEquals(tech_fusionBeam4, list.getAllAsArray()[1]);
        
        list.add(tech_missile6);
        list.add(tech_missile6);
        
        assertEquals(4, list.getQuantity());
        assertEquals(tech_massDriver1, list.getAllAsArray()[0]);
        assertEquals(tech_fusionBeam4, list.getAllAsArray()[1]);
        assertEquals(tech_missile6, list.getAllAsArray()[2]);
        assertEquals(tech_missile6, list.getAllAsArray()[3]);
        
        list.sortByName();

        assertEquals(4, list.getQuantity());
        assertEquals(tech_fusionBeam4, list.getAllAsArray()[0]);
        assertEquals(tech_massDriver1, list.getAllAsArray()[1]);
        assertEquals(tech_missile6, list.getAllAsArray()[2]);
        assertEquals(tech_missile6, list.getAllAsArray()[3]);
    }

    public void testCraftAndPlanetWeapons() throws Exception {
        Logger.logDebug("*******************************  TechnologyTest.testCraftAndPlanetWeapons() *************************");
        
        Weapons list = new Weapons();

        assertNull(list.getCraftWeapons(null));
        assertNull(list.getPlanetWeapons(null));
        
        list.add(tech_massDriver1);
        list.add(tech_missile6);
        
        assertEquals(2 , list.getCraftWeapons(null).length);
        assertEquals(tech_massDriver1 , list.getCraftWeapons(null)[0]);
        assertEquals(tech_missile6 , list.getCraftWeapons(null)[1]);
        
        assertEquals(1, list.getPlanetWeapons(null).length);
        assertEquals(tech_missile6 , list.getPlanetWeapons(null)[0]);
        
        Weapons existing = new Weapons(tech_massDriver1, tech_massDriver1, tech_missile6, tech_missile6);
        assertEquals(new Weapons(tech_massDriver1, tech_missile6).getWeapons() , list.getCraftWeapons(existing));
        assertEquals(new Weapons(tech_missile6).getWeapons() , list.getPlanetWeapons(existing));
        
        list.add(tech_spacialDistortion);
        list.add(tech_missile15);
        
        assertEquals(3 , list.getCraftWeapons(null).length);
        assertEquals(tech_massDriver1 , list.getCraftWeapons(null)[0]);
        assertEquals(tech_missile6 , list.getCraftWeapons(null)[1]);
        assertEquals(tech_missile15 , list.getCraftWeapons(null)[2]);
        
        assertEquals(2, list.getPlanetWeapons(null).length);
        assertEquals(tech_missile15 , list.getPlanetWeapons(null)[0]);
        assertEquals(tech_spacialDistortion , list.getPlanetWeapons(null)[1]);
        
        assertEquals(new Weapons(tech_massDriver1, tech_missile6, tech_missile15).getWeapons() , list.getCraftWeapons(existing));
        assertEquals(new Weapons(tech_missile15, tech_spacialDistortion).getWeapons() , list.getPlanetWeapons(existing));
        
        existing.add(tech_spacialDistortion);
        assertEquals(new Weapons(tech_massDriver1, tech_missile6, tech_missile15).getWeapons() , list.getCraftWeapons(existing));
        assertEquals(new Weapons(tech_missile15).getWeapons() , list.getPlanetWeapons(existing));
    }


    public void testGetDistinct() throws Exception {
        Logger.logDebug("*******************************  TechnologyTest.testGetDistinct() *************************");
        
        Weapons wl = new Weapons();
        wl.add(tech_massDriver1);
        wl.add(tech_massDriver1);
        wl.add(tech_massDriver2);
        
        TechnologyBundle[] d1 = wl.getDistinct();
        
        assertEquals(2, d1.length);
        assertEquals(tech_massDriver1, d1[0].getTechnology());
        assertEquals(2, d1[0].getQty());
        assertEquals(tech_massDriver2, d1[1].getTechnology());
        assertEquals(1, d1[1].getQty());
        
        
        wl = new Weapons();
        wl.add(tech_massDriver1);
        wl.add(tech_massDriver2);
        wl.add(tech_fusionBeam4);
        wl.add(tech_massDriver1);
        wl.add(tech_massDriver2);
        wl.add(tech_massDriver1);
        
        TechnologyBundle[] d2 = wl.getDistinct();
        
        assertEquals(3, d2.length);
        assertEquals(tech_massDriver1, d2[0].getTechnology());
        assertEquals(3, d2[0].getQty());
        assertEquals(tech_massDriver2, d2[1].getTechnology());
        assertEquals(2, d2[1].getQty());
        assertEquals(tech_fusionBeam4, d2[2].getTechnology());
        assertEquals(1, d2[2].getQty());
        
        TechnologyBundle[] d3 = wl.getDistinct((Weapons)null);
        
        assertEquals(3, d3.length);
        assertEquals(tech_massDriver1, d3[0].getTechnology());
        assertEquals(3, d3[0].getQty());
        assertEquals(tech_massDriver2, d3[1].getTechnology());
        assertEquals(2, d3[1].getQty());
        assertEquals(tech_fusionBeam4, d3[2].getTechnology());
        assertEquals(1, d3[2].getQty());

        {
	        TechnologyBundle[] d4 = wl.getDistinct(new Weapons(tech_massDriver1));
	        
	        assertEquals(3, d4.length);
	        assertEquals(tech_massDriver1, d4[0].getTechnology());
	        assertEquals(2, d4[0].getQty());
	        assertEquals(tech_massDriver2, d4[1].getTechnology());
	        assertEquals(2, d4[1].getQty());
	        assertEquals(tech_fusionBeam4, d4[2].getTechnology());
	        assertEquals(1, d4[2].getQty());
        }

        {
	        TechnologyBundle[] d4 = wl.getDistinct(new Weapons(tech_massDriver1, tech_massDriver1, tech_fusionBeam4, tech_fusionBeam4));
	        
	        assertEquals(2, d4.length);
	        assertEquals(tech_massDriver1, d4[0].getTechnology());
	        assertEquals(1, d4[0].getQty());
	        assertEquals(tech_massDriver2, d4[1].getTechnology());
	        assertEquals(2, d4[1].getQty());
        }

    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}