/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.setup.OptionsFactory;
import mot.common.setup.SetupOptions;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.MinCostRestriction;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyTree;
import mot.common.technology.TechnologyTreeConditionPercentage;
import mot.common.technology.TechnologyTreeNode;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.UserPlayer;

/**
 * Tests technology functionality works.
 */
public class TechnologyTreeTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(TechnologyTreeTest.class);
        return suite;
    }

    public TechnologyTreeTest(String name) {
        super(name);
    }
    
    private ComputerPlayer player;
    private UserPlayer userPlayer;

    private SetupOptions optionsGeneral;
    
    private SolarSystem ss1;
    private Planet planet1;
    
    private TechnologyTree tree = new TechnologyTree();

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        try {
            String [] playerNames = { "TestPlayer1" };
            optionsGeneral = OptionsFactory.getGeneralOptions();
            player = newPlayer(playerNames[0], blue, optionsGeneral, playerNames);
            ss1 = newSolarSystem("SS1", 100, 100);
            planet1 = newPlanet("SS1-A", ss1, player);
            
            player.setEngines(null);
            player.setHulls(null);
            player.setWeapons(null);
            player.setComputers(null);
            player.setScanners(null);
            player.setShields(null);
            player.setTheories(null);
            player.setLaboratories(null);
            player.setFactories(null);
            player.setBiotechnologies(null);
            player.setMiscTechnologies(null);
            player.setSpecials(null);
            
            player.getPlayerCharacter().setMissedTechnologies(null);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }

    protected void setUpUserPlayer() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        try {
            String [] playerNames = { "TestPlayer1" };
            optionsGeneral = OptionsFactory.getGeneralOptions();
            userPlayer = newUserPlayer(playerNames[0], blue, optionsGeneral);
            ss1 = newSolarSystem("SS1", 100, 100);
            planet1 = newPlanet("SS1-A", ss1, userPlayer);
            
            userPlayer.setEngines(null);
            userPlayer.setHulls(null);
            userPlayer.setWeapons(null);
            userPlayer.setComputers(null);
            userPlayer.setScanners(null);
            userPlayer.setShields(null);
            userPlayer.setTheories(null);
            userPlayer.setLaboratories(null);
            userPlayer.setFactories(null);
            userPlayer.setBiotechnologies(null);
            userPlayer.setMiscTechnologies(null);
            userPlayer.setSpecials(null);
            
            userPlayer.getPlayerCharacter().setMissedTechnologies(null);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }

    /**
     * Tests that the correct technologies are available for research.
     */
    public void testAvailableListFlatTree() throws Exception {
        Logger.logDebug("*******************************  TechnologyTreeTest.testAvailableListFlatTree() *************************");

        player.getPlayerCharacter().setMissedTechnologies(new GeneralTechnologyList());
        assertNull(player.getPlayerCharacter().getMissedTechnologies());
        setupOneTierTechTree();

        Technology[] available = player.getTechnologyAvailableForResearch();

        assertEquals(14, available.length);
        assertEquals(tech_massDriver2, 		available[0]);
        assertEquals(tech_fusionBeam4, 		available[1]);
        assertEquals(tech_fusion25, 		available[2]);
        assertEquals(tech_fusion25x, 		available[3]);
        assertEquals(tech_armour25, 		available[4]);
        assertEquals(tech_troopModule20, 	available[5]);
        assertEquals(tech_photon100, 		available[6]);
        assertEquals(tech_electronic25, 	available[7]);
        assertEquals(tech_flyer100, 		available[8]);
        assertEquals(tech_energy10, 		available[9]);
        assertEquals(tech_computerisedFactoryII, available[10]);
        assertEquals(tech_ComputerisedLabI, available[11]);
        assertEquals(tech_mineField, 		available[12]);
		assertEquals(tech_trade_route, 		available[13]);

        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_energy10);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_massDriver2);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_fusion25x);
        player.flushTechnologyAvailableForResearch();
        available = player.getTechnologyAvailableForResearch();

        assertEquals(11, available.length);
        assertEquals(tech_fusionBeam4, 		available[0]);
        assertEquals(tech_fusion25, 		available[1]);
        assertEquals(tech_armour25, 		available[2]);
        assertEquals(tech_troopModule20, 	available[3]);
        assertEquals(tech_photon100, 		available[4]);
        assertEquals(tech_electronic25,		available[5]);
        assertEquals(tech_flyer100, 		available[6]);
        assertEquals(tech_computerisedFactoryII, available[7]);
        assertEquals(tech_ComputerisedLabI, available[8]);
        assertEquals(tech_mineField, 		available[9]);
		assertEquals(tech_trade_route, 		available[10]);

        //
        // Test technology already discovered is not available for research
        //

        player.addTechnology(tech_computerisedFactoryII);
        player.addTechnology(tech_troopModule20);
        player.addTechnology(tech_ComputerisedLabI);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(8, available.length);
        assertEquals(tech_fusionBeam4, 		available[0]);
        assertEquals(tech_fusion25, 		available[1]);
        assertEquals(tech_armour25, 		available[2]);
        assertEquals(tech_photon100, 		available[3]);
        assertEquals(tech_electronic25,		available[4]);
        assertEquals(tech_flyer100, 		available[5]);
        assertEquals(tech_mineField, 		available[6]);
		assertEquals(tech_trade_route, 		available[7]);

    }

    /**
     * Tests that the correct technologies are available for research.
     */
    public void testAvailableListOneTierTree() throws Exception {
        Logger.logDebug("*******************************  TechnologyTreeTest.testAvailableListFlatTree() *************************");

        setupTwoTierTechTree();
        Technology[] available = player.getTechnologyAvailableForResearch();

        //
        // Test with a tech tree only those whose pre-requisites met are available
        //
        
        assertEquals(3, available.length);
        assertEquals(tech_massDriver1, 	available[0]);
        assertEquals(tech_flyer30, 		available[1]);
        assertEquals(tech_flyer60, 		available[2]);

        //
        // Test technology already discovered is not available for research
        //

        player.addTechnology(tech_flyer30);
        player.addTechnology(tech_researchLabI);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(16, available.length);
        assertEquals(tech_massDriver1, 		available[0]);
        assertEquals(tech_flyer60, 			available[1]);
        assertEquals(tech_massDriver2, 		available[2]);
        assertEquals(tech_fusionBeam4, 		available[3]);
        assertEquals(tech_fusion25, 		available[4]);
        assertEquals(tech_fusion25x, 		available[5]);
        assertEquals(tech_armour25, 		available[6]);
        assertEquals(tech_troopModule20, 	available[7]);
        assertEquals(tech_photon100, 		available[8]);
        assertEquals(tech_electronic25, 	available[9]);
        assertEquals(tech_flyer100, 		available[10]);
        assertEquals(tech_energy10, 		available[11]);
        assertEquals(tech_computerisedFactoryII, available[12]);
        assertEquals(tech_ComputerisedLabI, available[13]);
        assertEquals(tech_mineField, 		available[14]);
		assertEquals(tech_trade_route, 		available[15]);

        player.addTechnology(tech_flyer60);
        player.addTechnology(tech_massDriver1);
        available = player.getTechnologyAvailableForResearch();
        
        assertEquals(14, available.length);
        assertEquals(tech_massDriver2, 		available[0]);
        assertEquals(tech_fusionBeam4, 		available[1]);
        assertEquals(tech_fusion25, 		available[2]);
        assertEquals(tech_fusion25x, 		available[3]);
        assertEquals(tech_armour25, 		available[4]);
        assertEquals(tech_troopModule20, 	available[5]);
        assertEquals(tech_photon100, 		available[6]);
        assertEquals(tech_electronic25, 	available[7]);
        assertEquals(tech_flyer100, 		available[8]);
        assertEquals(tech_energy10, 		available[9]);
        assertEquals(tech_computerisedFactoryII, available[10]);
        assertEquals(tech_ComputerisedLabI, available[11]);
        assertEquals(tech_mineField, 		available[12]);
		assertEquals(tech_trade_route, 		available[13]);

        //
        // Test that a having a technology on the missing list cannot block 
        // research up the tree.
        //
        
        addTierToTechTree();
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_fusionBeam4);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_fusion25x);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_armour25);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_troopModule20);
        player.addTechnology(tech_electronic25);
        player.addTechnology(tech_flyer100);
        player.addTechnology(tech_mineField);
        player.addTechnology(tech_computerisedFactoryII);
        player.addTechnology(tech_ComputerisedLabI);
    
        available = player.getTechnologyAvailableForResearch();

        assertEquals(7, available.length);
        assertEquals(tech_massDriver2, 		available[0]);
        assertEquals(tech_fusion25, 		available[1]);
        assertEquals(tech_photon100, 		available[2]);
        assertEquals(tech_energy10, 		available[3]);
        assertEquals(tech_trade_route, 		available[4]);
        assertEquals(tech_girator, 			available[5]);
        assertEquals(tech_mindControl, 		available[6]);
    }
    
    /**
     * Tests that the correct technologies are available for research.
     */
    public void testConditionPercent() throws Exception {
        Logger.logDebug("*******************************  TechnologyTreeTest.testConditionPercent() *************************");

        setupThreeTierTechTree();
        TechnologyTreeNode theoryNode = TechnologyTree.getInstance().getNode(tech_researchLabII);
        theoryNode.setCondition(new TechnologyTreeConditionPercentage(tech_researchLabI, 0.5));

        
        //
        // Test no condition tech tree
        //
        
        Technology[] available = player.getTechnologyAvailableForResearch();
        assertEquals(3, available.length);
        assertEquals(tech_massDriver1, 	available[0]);
        assertEquals(tech_flyer30, 		available[1]);
        assertEquals(tech_flyer60, 		available[2]);
        
        player.addTechnology(tech_massDriver1);
        player.addTechnology(tech_flyer60);
        player.addTechnology(tech_researchLabI);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(5, available.length);
        assertEquals(tech_flyer30, 			available[0]);
        assertEquals(tech_massDriver2, 		available[1]);
        assertEquals(tech_fusionBeam4, 		available[2]);
        assertEquals(tech_fusion25, 		available[3]);
        assertEquals(tech_fusion25x, 		available[4]);
        
        player.addTechnology(tech_flyer30);
        player.addTechnology(tech_flyer60);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(false, 	theoryNode.getCondition().isSatisfied(player, null));        
        assertEquals(4, available.length);
        assertEquals(tech_massDriver2, 		available[0]);
        assertEquals(tech_fusionBeam4, 		available[1]);
        assertEquals(tech_fusion25, 		available[2]);
        assertEquals(tech_fusion25x, 		available[3]);
        
        player.addTechnology(tech_massDriver2);
        player.addTechnology(tech_fusionBeam4);
        player.addTechnology(tech_fusion25);
        available = player.getTechnologyAvailableForResearch();

        // this sometimes fails - reason unknown
        assertEquals(true, 	theoryNode.getCondition().isSatisfied(player, null));
        if (available.length != 2) {
        	Logger.logDebug("BUG - "+Comp.toString(available));
        	Logger.logDebug("BUG - "+Comp.toString(player.getMissedTechnology()));
        	Logger.logDebug("BUG - "+Comp.toString(TechnologyTree.getInstance()));
        }
        assertEquals(2, available.length);
        assertEquals(tech_fusion25x, 		available[0]);
        assertEquals(tech_researchLabII, 	available[1]);
        
    }
    /**
     * Tests that new technologies discovered are used by the computer player to upgrade
     * their planets.
     */
    public void testAutoDiscoverOutdatedTechnologies() throws Exception {
        Logger.logDebug("*******************************  ResearchTest.testAutoDiscoverOutdatedTechnologies() *************************");
        setUpUserPlayer();
        
        setupFourTierTechTree_As_Per_Tech_Tree();
        
        Technology[] technologyAvailableForResearch = userPlayer.getTechnologyAvailableForResearch();
        Technology[] outdatedTechnologyAvailableForResearch = userPlayer.getOutdatedTechnologiesAvailableForResearch().getTechnologies();
		assertEquals(tech_massDriver1, tech_flyer30, tech_flyer60, technologyAvailableForResearch);
		assertNull(outdatedTechnologyAvailableForResearch);
		
		userPlayer.addTechnology(tech_massDriver1);
		userPlayer.addTechnology(tech_flyer30);
		userPlayer.addTechnology(tech_flyer60);
		
        technologyAvailableForResearch = userPlayer.getTechnologyAvailableForResearch();
        outdatedTechnologyAvailableForResearch = userPlayer.getOutdatedTechnologiesAvailableForResearch().getTechnologies();
		assertEquals(tech_researchLabI, technologyAvailableForResearch);
		assertNull(outdatedTechnologyAvailableForResearch);

		userPlayer.addTechnology(tech_researchLabI);

        technologyAvailableForResearch = userPlayer.getTechnologyAvailableForResearch();
        outdatedTechnologyAvailableForResearch = userPlayer.getOutdatedTechnologiesAvailableForResearch().getTechnologies();
		assertEquals(tech_missile6, tech_massDriver2, tech_fusionBeam4, tech_fusion25, tech_photon100, tech_electronic25, technologyAvailableForResearch);
		assertNull(outdatedTechnologyAvailableForResearch);

		userPlayer.addTechnology(tech_missile6);
		userPlayer.addTechnology(tech_massDriver2);
		userPlayer.addTechnology(tech_fusionBeam4);
		userPlayer.addTechnology(tech_fusion25);
		userPlayer.addTechnology(tech_photon100);
		// missing tech_electronic25 for this level
		
        technologyAvailableForResearch = userPlayer.getTechnologyAvailableForResearch();
        outdatedTechnologyAvailableForResearch = userPlayer.getOutdatedTechnologiesAvailableForResearch().getTechnologies();
		assertEquals(tech_electronic25, tech_researchLabII, technologyAvailableForResearch);
		assertNull(outdatedTechnologyAvailableForResearch);

		userPlayer.addTechnology(tech_researchLabII);
		
        technologyAvailableForResearch = userPlayer.getTechnologyAvailableForResearch();
        outdatedTechnologyAvailableForResearch = userPlayer.getOutdatedTechnologiesAvailableForResearch().getTechnologies();
		assertEquals(tech_electronic25, tech_missile15, tech_fusion25x, tech_armour25, tech_photon150, tech_electronic40, tech_troopModule20,  technologyAvailableForResearch);
		assertEquals(tech_electronic25, outdatedTechnologyAvailableForResearch);

		userPlayer.addTechnology(tech_missile15);
		userPlayer.addTechnology(tech_fusion25x);
		userPlayer.addTechnology(tech_armour25);
		userPlayer.addTechnology(tech_photon150);
		userPlayer.addTechnology(tech_electronic40);
		// missing tech_electronic25 for last level
		// missing tech_troopModule20 for this level

        technologyAvailableForResearch = userPlayer.getTechnologyAvailableForResearch();
        outdatedTechnologyAvailableForResearch = userPlayer.getOutdatedTechnologiesAvailableForResearch().getTechnologies();
		assertEquals(tech_electronic25, tech_troopModule20, tech_researchLabIII, technologyAvailableForResearch);
		assertEquals(tech_electronic25, outdatedTechnologyAvailableForResearch);

		userPlayer.addTechnology(tech_researchLabIII);
		
        technologyAvailableForResearch = userPlayer.getTechnologyAvailableForResearch();
        outdatedTechnologyAvailableForResearch = userPlayer.getOutdatedTechnologiesAvailableForResearch().getTechnologies();
		assertEquals(tech_electronic25, tech_troopModule20, tech_fusion40, tech_armour200, tech_photon400,  technologyAvailableForResearch);
		assertEquals(tech_electronic25, tech_troopModule20, outdatedTechnologyAvailableForResearch);
		
		Utils.setNextRandomProbability(0.01);
		Utils.setNextRandomInt(0);
		userPlayer.checkOutdatedTechnologyDiscovery();
		
		assertTrue(userPlayer.hasDiscovered(tech_electronic25));
		
        technologyAvailableForResearch = userPlayer.getTechnologyAvailableForResearch();
        outdatedTechnologyAvailableForResearch = userPlayer.getOutdatedTechnologiesAvailableForResearch().getTechnologies();
		assertEquals(tech_troopModule20, tech_fusion40, tech_armour200, tech_photon400,  technologyAvailableForResearch);
		assertEquals(tech_troopModule20, outdatedTechnologyAvailableForResearch);
		
    }
    

    /**
     * Tests that the correct technologies are available for research.
     */
    public void testConditionPercentWithExcludeList() throws Exception {
        Logger.logDebug("*******************************  TechnologyTreeTest.testConditionPercentWithExcludeList() *************************");

        setupThreeTierTechTree();
        TechnologyTreeNode theoryNode = TechnologyTree.getInstance().getNode(tech_researchLabII);
        theoryNode.setCondition(new TechnologyTreeConditionPercentage(tech_researchLabI, 0.5));

        
        //
        // Test no condition tech tree
        //
        
        player.addTechnology(tech_massDriver1);
        player.addTechnology(tech_flyer30);
        player.addTechnology(tech_flyer60);
        player.addTechnology(tech_researchLabI);
        Technology[] available = player.getTechnologyAvailableForResearch();

        assertEquals(false, 	theoryNode.getCondition().isSatisfied(player, null));        
        assertEquals(4, available.length);
        assertEquals(tech_massDriver2, 		available[0]);
        assertEquals(tech_fusionBeam4, 		available[1]);
        assertEquals(tech_fusion25, 		available[2]);
        assertEquals(tech_fusion25x, 		available[3]);
        
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_massDriver2);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_fusionBeam4);
        player.addTechnology(tech_fusion25);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(true, 	theoryNode.getCondition().isSatisfied(player, null));        
        assertEquals(2, available.length);
        assertEquals(tech_fusion25x, 		available[0]);
        assertEquals(tech_researchLabII, 	available[1]);
        
    }
    
    /**
     * Tests that the correct technologies are available for research.
     */
    public void testRestrictionMinResearchCost() throws Exception {
        Logger.logDebug("*******************************  TechnologyTreeTest.testRestrictionMinResearchCost() *************************");

        setupThreeTierTechTree();

        MinCostRestriction restriction1 = new MinCostRestriction();
        restriction1.setMinCost(50);
		tech_researchLabI.setMinResearchCostRestriction(restriction1);
        
        MinCostRestriction restriction2 = new MinCostRestriction();
        restriction2.setMinCost(500);
		tech_researchLabII.setMinResearchCostRestriction(restriction2);
		
        //
        // 
        //
        
        Technology[] available = player.getTechnologyAvailableForResearch();
        assertEquals(3, available.length);
        assertEquals(tech_massDriver1, 	available[0]);
        assertEquals(tech_flyer30, 		available[1]);
        assertEquals(tech_flyer60, 		available[2]);
        
        player.addTechnology(tech_massDriver1);
        player.addTechnology(tech_flyer30);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(1, available.length);
        assertEquals(tech_flyer60, 		available[0]);
        
        player.addTechnology(tech_flyer60);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(1, available.length);
        assertEquals(tech_researchLabI, 	available[0]);
        
        player.addTechnology(tech_researchLabI);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(4, available.length);
        assertEquals(tech_massDriver2, 		available[0]);
        assertEquals(tech_fusionBeam4, 		available[1]);
        assertEquals(tech_fusion25, 		available[2]);
        assertEquals(tech_fusion25x, 		available[3]);
    }
    
    
    /**
     * Tests that the correct technologies are available for research.
     */
    public void testRestrictionMinResearchCostWithExcludeList() throws Exception {
        Logger.logDebug("*******************************  TechnologyTreeTest.testRestrictionMinResearchCostWithExcludeList() *************************");

        setupThreeTierTechTree();

        MinCostRestriction restriction1 = new MinCostRestriction();
        restriction1.setMinCost(50);
		tech_researchLabI.setMinResearchCostRestriction(restriction1);
        
        MinCostRestriction restriction2 = new MinCostRestriction();
        restriction2.setMinCost(500);
		tech_researchLabII.setMinResearchCostRestriction(restriction2);
		
        //
        // 
        //
        
        Technology[] available = player.getTechnologyAvailableForResearch();
        assertEquals(3, available.length);
        assertEquals(tech_massDriver1, 	available[0]);
        assertEquals(tech_flyer30, 		available[1]);
        assertEquals(tech_flyer60, 		available[2]);
        
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_massDriver1);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_flyer30);
        
        player.flushTechnologyAvailableForResearch();
        available = player.getTechnologyAvailableForResearch();

        assertEquals(1, available.length);
        assertEquals(tech_flyer60, 		available[0]);
        
        player.addTechnology(tech_flyer60);
        available = player.getTechnologyAvailableForResearch();

        assertEquals(1, available.length);
        assertEquals(tech_researchLabI, 	available[0]);
        
    }
    
   
    /**
     * Tests Technology Tree Works only missed technologies that would have been available
     * for resdearch are returned by player (as current missed tecyhnologies) 
     */
    public void testMissedTechnologies() throws Exception {
        Logger.logDebug("*******************************  TechnologyTreeTest.testMissedTechnologies() *************************");
        
        setupOneTierTechTree();

        //
        // Test missed list is empty
        //
        
        Technology[] missedTechs = player.getMissedTechnology();
       
        assertNull(missedTechs);
        
        //
        // Test missed list returns everything when all techs are available for research
        //
        
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_massDriver2);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_fusion25);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_energy10);

		missedTechs = player.getMissedTechnology();
		assertEquals(3, missedTechs.length);
		assertEquals(tech_massDriver2, missedTechs[0]);
		assertEquals(tech_fusion25, missedTechs[1]);
		assertEquals(tech_energy10, missedTechs[2]);
        
        
        //
        // Test missed list does not return techs if they have been discovered
        //
        
        player.addTechnology(tech_massDriver2);
		missedTechs = player.getMissedTechnology();

		assertEquals(2, missedTechs.length);
		assertEquals(tech_fusion25, missedTechs[0]);
		assertEquals(tech_energy10, missedTechs[1]);
        
        
        //
        // Test missed list does not return techs if they are not available for research yet
        //
        
        addTierToTechTree();
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_girator);
        player.getPlayerCharacter().getBasicCharacter().addMissedTechnology(tech_mindControl);
		missedTechs = player.getMissedTechnology();

		assertEquals(2, missedTechs.length);
		assertEquals(tech_fusion25, missedTechs[0]);
		assertEquals(tech_energy10, missedTechs[1]);
		
		// Check that the new tied would be in the missed list if dependent tech was discovered
        
        player.addTechnology(tech_fusionBeam4);
		missedTechs = player.getMissedTechnology();
        
		assertEquals(4, missedTechs.length);
		assertEquals(tech_fusion25, missedTechs[0]);
		assertEquals(tech_energy10, missedTechs[1]);
		assertEquals(tech_girator, missedTechs[2]);
		assertEquals(tech_mindControl, missedTechs[3]);
		
    }
  

    protected void tearDown() {
        
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

    
}