/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.designs.Orbital;
import mot.common.designs.OutpostDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.TransportDesign;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Module;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.command.MooDeployCmd;
import mot.engine.command.MooMoveCmd;
import mot.engine.command.TransportLoadCmd;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.planets.BlackHole;
import mot.engine.planets.BlackHoleFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.ProjectOutpost;
import mot.engine.planets.ProjectSpaceStation;
import mot.engine.planets.ProjectTransport;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.UserPlayer;
import mot.engine.setup.ResourceFactory;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskDeployStation;
import mot.engine.transports.TransportTaskInvade;
import mot.engine.transports.TransportTaskMoveColonists;

/**
 * Tests that planets can build transports and that transports can be used to move colonists around.
 */
public class TransportTest extends TestBase {
    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(TransportTest.class);
        return suite;
    }

    public TransportTest(String name) {
        super(name);
    }
    
    private ComputerPlayer player;
    private ComputerPlayer playerOther;
    private UserPlayer userPlayer;

    private SolarSystem ss1;
    private SolarSystem ss2;

    private Planet planet1;
    private Planet planet2;

    private SetupOptions optionsGeneral;

    protected void setUp() throws ApplicationError {
    	Logger.setLogLevelLoaded();
    	Logger.setInMemory(false);
    	Logger.setDebug();
    }
    
    protected void createComputerPlayerMap() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        try {
            String [] playerNames = { "TestPlayer1" };
            optionsGeneral = OptionsFactory.getGeneralOptions();
            player = newPlayer(playerNames[0], blue, optionsGeneral, playerNames);
            player.addTechnology(tech_outpost);
            player.addTechnology(tech_spaceStation);
            
            ss1 = newSolarSystem("SS1", 100, 100);
            planet1 = newPlanet("SS1-A", ss1, player);

            ss2 = newSolarSystem("SS2", 200, 100);
            planet2 = newPlanet("SS2-A", ss2, null, 100, 0.9);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }

    protected void createTwoComputerPlayerMap() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        try {
            String [] playerNames = { "TestPlayer1","TestPlayer2" };
            optionsGeneral = OptionsFactory.getGeneralOptions();
            player = newPlayer(playerNames[0], blue, optionsGeneral, playerNames);
            playerOther = newPlayer(playerNames[1], blue, optionsGeneral, playerNames);
            
            player.addTechnology(tech_outpost);
            player.addTechnology(tech_spaceStation);
            
            ss1 = newSolarSystem("SS1", 100, 100);
            planet1 = newPlanet("SS1-A", ss1, player);

            ss2 = newSolarSystem("SS2", 200, 100);
            planet2 = newPlanet("SS2-A", ss2, playerOther);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }

    protected void createUserPlayerMap() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        try {
            String [] playerNames = { "TestPlayer1" };
            optionsGeneral = OptionsFactory.getGeneralOptions();
            RaceOptions options = new RaceOptions(playerNames[0], null, optionsGeneral);
            userPlayer = newUserPlayer(playerNames[0], blue, null, options);
            userPlayer.addTechnology(tech_outpost);
            userPlayer.addTechnology(tech_spaceStation);
            
            ss1 = newSolarSystem("SS1", 100, 100);
            planet1 = newPlanet("SS1-A", ss1, userPlayer);

            ss2 = newSolarSystem("SS2", 200, 100);
            planet2 = newPlanet("SS2-A", ss2, null, 100, 0.9);
        } catch (Exception e) {
            Logger.logError("Failed ",e);
        }
    }
    
    //
    // NOTE THAT DEPLOYING STATION AND MODULE IN THE SAME SYSTEM AS THE BUILDER IS TESTED IN MayorTest
    //
    
    
    /**
     * Tests
     *  - ProjectTransport builds new transports that are available for use when created
     *  - Tests task TransportTaskMoveColonists moves colonists between planets correctly
     */
    public void testCreateAndMoveColonists() throws Exception {
        Logger.logDebug("*******************************  CraftTest.testCreateAndMoveColonists() *************************");
        createComputerPlayerMap();
        
        assertEquals("Correct qty transports", 0, player.getTransportManagerNotNull().getQtyTransports());
        assertNull("No Transports available", player.getTransportManagerNotNull().getFreeTransport());
        
        ProjectTransport project = new ProjectTransport();
        planet1.setProject(project);
        while (planet1.getProject() == project) {
            planet1.endTurn();
        }
        
        assertEquals("Correct qty transports", 1, player.getTransportManagerNotNull().getQtyTransports());
        assertNotNull("A Transport available", player.getTransportManagerNotNull().getFreeTransport());
        
        planet1.setPopulation(10.0);
        planet2.setPopulation(10.0);
        planet2.setOwner(player);
        
        Transport transport = player.getTransportManagerNotNull().getFreeTransport();
        new TransportTaskMoveColonists(transport, planet1, planet2, 5);
        //player.getTransportManagerNotNull().endTurn();
        int eta = transport.getNavigator().getETA();
        
        assertEquals("Planet has correct population", 5.0, planet1.getPopulation(), 0.001);
        assertEquals("Planet has correct population", 10.0, planet2.getPopulation(), 0.001);
        assertEquals("Engine speed is correct", 25, player.getEngines().getFastestEngine().getWarpSpeed());
        assertEquals("Have correct ETA", 4, eta);
        
        while (eta > 0) {
            player.getTransportManagerNotNull().endTurn();
            eta--;
        }
        
        assertEquals("Planet has correct population", 5.0, planet1.getPopulation(), 0.001);
        assertEquals("Planet has correct population", 15.0, planet2.getPopulation(), 0.001);
        assertEquals("Correct qty transports", 1, player.getTransportManagerNotNull().getQtyTransports());
        assertNotNull("A Transport available", player.getTransportManagerNotNull().getFreeTransport());
        assertNull("A Transport available", transport.getTask());
        
        //
        // Test sending a transport to a planet with Max Population
        //
        
        int maxPopulation = (int) planet2.getMaxPopulation();
        
        planet1.setPopulation(10.0);
        planet2.setPopulation(maxPopulation);
        
        new TransportTaskMoveColonists(transport, planet1, planet2, 5);
        
        assertEquals("Planet has correct population", 5.0, planet1.getPopulation(), 0.001);
        assertEquals("Planet has correct population", maxPopulation, planet2.getPopulation(), 0.001);
        
        eta = transport.getNavigator().getETA();
        while (eta > 0) {
            player.getTransportManagerNotNull().endTurn();
            eta--;
        }
        
        assertEquals("Planet has correct population", 5.0, planet1.getPopulation(), 0.001);
        assertEquals("Planet has correct population", maxPopulation, planet2.getPopulation(), 0.001);
        

    }

    /**
     * Tests
     *  - ProjectTransport builds new transports that are available for use when created
     *  - Tests task TransportTaskMoveColonists moves colonists between planets correctly
     */
    public void testCreateAndMoveColonistsCmd() throws Exception {
        Logger.logDebug("*******************************  CraftTest.testCreateAndMoveColonistsCmd() *************************");
        createComputerPlayerMap();
        
        UserPlayer userPlayer = TestBase.newUserPlayer("TestPlayer2", green, null, new RaceOptions("TestPlayer2", optionsGeneral, optionsGeneral));
        planet1.setOwner(userPlayer);
        planet1.setPopulation(20.0);
        SolarSystem ss3 = newSolarSystem("SS3", 140, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, userPlayer);
        planet3.setPopulation(20.0);
        ss2.setLocation(new Location(110, 100));
        
        //
        // Test transport at half capacity, immediate destination
        //
        
        userPlayer.getTransportManagerNotNull().setTransport(new Transport(userPlayer));
      	new TransportLoadCmd(planet1.getOrbit(), planet3.getOrbit(), 5, userPlayer, 0).run();
      	      	
      	Transport[] transports = userPlayer.getTransportManager().getTransportsAsArray();
      	assertEquals("Transport1", transports);

        assertEquals(0, userPlayer.getTransportManager().getQtyFreeTransport());
        assertEquals(planet3, transports[0].getNavigator().getDestination().getPlanet());
        assertEquals(20, planet3.getPopulation(), 0.001);
        assertEquals(15, planet1.getPopulation(), 0.001);
        assertEquals(5, userPlayer.getTransportManager().getQtyColonistsOnrouteTo(planet3), 0.001);
        
        transports[0].endTurn();
        assertEquals(0, userPlayer.getTransportManager().getQtyFreeTransport());
        transports[0].endTurn();
        
        assertEquals(1, userPlayer.getTransportManager().getQtyFreeTransport());
        assertEquals(null, transports[0].getTask());
        assertEquals(25, planet3.getPopulation(), 0.001);
        assertEquals(15, planet1.getPopulation(), 0.001);
        
        //
        // Test transport at 1.5 time capacity, immediate destination
        // Use multiple shuttles to transport people into transport
        //
        
        planet1.setPopulation(50.0);
        planet3.setPopulation(50.0);
        userPlayer.getTransportManager().setTransport(new Transport(userPlayer));
        // upload 5 colonists => use 0.5 of transport capacity 
      	new TransportLoadCmd(planet1.getOrbit(), null, 5, userPlayer, 0).run();
      	
      	transports = userPlayer.getTransportManager().getTransportsAsArray();
      	assertEquals("Transport1", "Transport2", transports);
     	
        assertEquals(1, userPlayer.getTransportManager().getQtyFreeTransport());
        assertEquals(5, ((TransportTaskMoveColonists)transports[0].getTask()).getQty());
        assertEquals(null, transports[0].getNavigator());
        assertEquals(50, planet3.getPopulation(), 0.001);
        assertEquals(45, planet1.getPopulation(), 0.001);
        
        // upload 25 colonists => use 1.5 of transport capacity 
      	new TransportLoadCmd(planet1.getOrbit(), null, 25, userPlayer, 0).run();
      	
      	transports = userPlayer.getTransportManager().getTransportsAsArray();
      	assertEquals("Transport1", "Transport2", transports);
      	
        assertEquals(0, userPlayer.getTransportManager().getQtyFreeTransport());
        assertEquals(25, ((TransportTaskMoveColonists)transports[0].getTask()).getQty());
        assertEquals(5, ((TransportTaskMoveColonists)transports[1].getTask()).getQty());
        assertEquals(null, transports[0].getNavigator());
        assertEquals(50, planet3.getPopulation(), 0.001);
        assertEquals(20, planet1.getPopulation(), 0.001);

        // Test they do not do anything until moved (assigned a destination)
        transports[0].endTurn();
        transports[1].endTurn();
        
        assertEquals(0, userPlayer.getTransportManager().getQtyFreeTransport());
        assertEquals(50.0, planet3.getPopulation(), 0.001);
        assertEquals(20.0, planet1.getPopulation(), 0.001);
        
        // Test they do not do anything if moved to an empty planet
        transports[0].moveTo(new Orbit(planet3));
        transports[1].moveTo(new Orbit(planet3));
        transports[0].endTurn();
        transports[1].endTurn();
        
        assertEquals(0, userPlayer.getTransportManager().getQtyFreeTransport());
        assertEquals(50, planet3.getPopulation(), 0.001);
        assertEquals(20, planet1.getPopulation(), 0.001);
        
        // Test they drop colonists when moved to a planet owned by the owner
        transports[0].moveTo(new Orbit(planet3));
        transports[1].moveTo(new Orbit(planet3));
        transports[0].endTurn();
        transports[1].endTurn();
        
        assertEquals(2, userPlayer.getTransportManager().getQtyFreeTransport());
        assertEquals(null, transports[0].getTask());
        assertEquals(null, transports[1].getTask());
        assertEquals(80, planet3.getPopulation(), 0.001);
        assertEquals(20, planet1.getPopulation(), 0.001);
    }

    //
    // Test sending a transport to a planet with Max Population
    //
    
    public void testMoveColonistsToFullPlanetCmd() throws Exception {
        Logger.logDebug("*******************************  CraftTest.testMoveColonistsToFullPlanetCmd() *************************");
        createComputerPlayerMap();
        
        UserPlayer userPlayer = TestBase.newUserPlayer("TestPlayer2", green, null, new RaceOptions("TestPlayer2", optionsGeneral, optionsGeneral));
        planet1.setOwner(userPlayer);
        planet1.setPopulation(20.0);
        SolarSystem ss3 = newSolarSystem("SS3", 140, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, userPlayer);
        planet3.setPopulation(20.0);
        ss2.setLocation(new Location(110, 100));
        
	    int maxPopulation = (int) planet2.getMaxPopulation();
	    
	    planet1.setPopulation(10.0);
	    planet2.setPopulation(maxPopulation);
	    
        userPlayer.getTransportManager().setTransport(new Transport(userPlayer));
        Transport transport = userPlayer.getTransportManager().getTransportsAsArray()[0];
	    new TransportTaskMoveColonists(transport, planet1, planet2, 5);
	    
	    assertEquals("Planet has correct population", 5.0, planet1.getPopulation(), 0.001);
	    assertEquals("Planet has correct population", maxPopulation, planet2.getPopulation(), 0.001);
	    
	    int eta = transport.getNavigator().getETA();
	    while (eta > 0) {
	        player.getTransportManagerNotNull().endTurn();
	        eta--;
	    }
	    
	    assertEquals("Planet has correct population", 5.0, planet1.getPopulation(), 0.001);
	    assertEquals("Planet has correct population", maxPopulation, planet2.getPopulation(), 0.001);
	    assertEquals("Transport not deployed", 5, ((TransportTaskMoveColonists)transport.getTask()).getQty());
    }

    /**
     * Tests 
     *  - A planet builds a Transport carrying an outpost
     *  - AI moves the transport to a planet and deploys the outpost
     *  - Uses Solar Systems with planets
     */
    public void testProject_Outpost1_ProjectContainsTarget() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_Outpost1_ProjectContainsTarget() *************************");
        createComputerPlayerMap();

        new ProjectTransport().build(planet1);
        planet2.setTeraforming(0.2);
        
        SolarSystem ss3 = newSolarSystem("SS3", 150, 100);
        SolarSystem ss4 = newSolarSystem("SS4", 300, 100);
        Planet planet4 = newPlanet("SS4-A", ss3, null, 110, 1.0);

        assertEquals(1, player.getTransportManager().getQtyTransports());
        assertEquals(1, player.getTransportManager().getQtyFreeTransport());
        
        // Specify ss3 but ss2 would be a better and what the AI would choose. 
        new ProjectOutpost(new OutpostDesign(tech_outpost), ss3).build(planet1); 
        
        // Computer players cheat creating themselves a transport when they need one
        assertEquals(1, player.getTransportManager().getQtyTransports());
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());

        assertEquals(0, player.getTransportManager().getQtyFreeTransport());
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(ss3,  task.getDestination());
        assertEquals(planet1,  task.getOrigin().getPlanet());
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(null,  task);
        assertEquals(true,  ss3.hasOwner());
        assertEquals(player,  ss3.getOwners()[0]);
        assertEquals((Planet)null,  ss3.getColonies(true));
        assertEquals(100,  ss3.getOrbital().getScanStrength());        
    }
    
    public void testProject_Outpost1() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_Outpost1() *************************");
        createComputerPlayerMap();

        planet2.setTeraforming(0.2);
        
        SolarSystem ss3 = newSolarSystem("SS3", 300, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 1.0);
        player.getMayor().buildOutpost(planet1);

        assertEquals(0, player.getTransportManager().getQtyTransports());
        assertEquals(false, ((ProjectOutpost)planet1.getProject()).isIncludeTransport());

        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        
        assertEquals(0, player.getTransportManager().getQtyTransports());
        assertEquals(true, ((ProjectOutpost)planet1.getProject()).isIncludeTransport());

        planet1.addProduction((int)TransportDesign.getCostToBuild());

        assertEquals(1, player.getTransportManager().getQtyTransports());
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());
        
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(ss2,  task.getDestination());
        assertEquals(planet1,  task.getOrigin().getPlanet());
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(null,  task);
        assertEquals(true,  ss2.hasOwner());
        assertEquals(player,  ss2.getOwners()[0]);
        assertEquals((Planet)null,  ss2.getColonies(true));
        assertEquals(100,  ss2.getOrbital().getScanStrength());        
    }

    /**
     * Tests 
     *  - A planet builds a Transport carrying an outpost
     *  - AI moves the transport to a planet and deploys the outpost
     *  - Uses Solar Systems without planets
     *  - Uses multiple solar system that match criteria and some that do not
     *  - AI creates transports
     */
    public void testProject_Outpost2() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_Outpost2() *************************");
        createComputerPlayerMap();

        planet2.setTeraforming(0.2);

        // The best match should be SS5 because it is 150 from ss1 and 50 from ss3
        assertEquals(150, player.getEngines().getLongestRangeEngine().getMaxTravelDistance());
        SolarSystem ss5 = newSolarSystem("SS5", 250, 100);
        SolarSystem ss6 = newSolarSystem("SS6", 50, 100);
        SolarSystem ss7 = newSolarSystem("SS7", 200, 200);
        SolarSystem ss8 = newSolarSystem("SS8", 275, 100);
        
        SolarSystem ss3 = newSolarSystem("SS3", 300, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 1.0);

        player.getMayor().buildOutpost(planet1);
        planet1.getProject().build(planet1);
        
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());
        assertEquals(1, player.getTransportManager().getQtyTransports());
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(ss5,  task.getDestination());
        assertEquals(planet1,  task.getOrigin());
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(null,  task);
        assertEquals(true,  ss5.hasOwner());
        assertEquals(player,  ss5.getOwners()[0]);
        assertEquals((Planet)null,  ss5.getColonies(true));
        assertEquals(100,  ss5.getOrbital().getScanStrength());        
    }

    /**
     * Tests 
     *    - A planet builds a Transport carrying an space station
     *    - AI moves the transport to a planet and deploys the space station
     *  AND
     *    - A planet builds a Transport carrying an space station module
     *    - AI moves the transport to a the space station to deploy it
     */
    public void testProject_SpaceStationWithTransport() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_SpaceStationWithTransport() *************************");
        createComputerPlayerMap();

        // Planet3 is a better option for mining - but have specified planet2
        SolarSystem ss3 = newSolarSystem("SS3", 100, 200);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 0.1);
        planet3.setProductionBonus(2.0);
        player.getTransportManager().setTransport(new Transport(player));

        assertEquals(1, player.getTransportManager().getQtyTransports());
        assertEquals(1, player.getTransportManager().getQtyFreeTransport());
        
        new ProjectSpaceStation(new SpaceStationDesign(tech_spaceStation), new Orbit(planet2)).build(planet1); 
        
        // Computer players cheat creating themselves a transport when they need one
        assertEquals(1, player.getTransportManager().getQtyTransports());
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());
        
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(planet2,  task.getDestination());
        assertEquals(planet1,  task.getOrigin());
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(null,  task);
        assertEquals(true,  ss2.hasOwner());
        assertEquals(player,  ss2.getOwners()[0]);
        assertEquals(planet2,  ss2.getColonies(true)[0]);
        assertEquals("SpaceStation",  ss2.getColonies(true)[0].getOrbital().getName());
        assertEquals(planet2,  planet1.getMiningStation().getPlanet());

		assertEquals(tech_core_habitat, planet2.getOrbital().getModuleTechs());
    }

    public void testProject_SpaceStationWithDestinationTakenDuringBuild() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_SpaceStationWithTransportDestinationTaken() *************************");
        createTwoComputerPlayerMap();

        SolarSystem ss3 = newSolarSystem("SS3", 100, 200);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 0.1);
        planet3.setProductionBonus(2.0);

        SolarSystem ss4 = newSolarSystem("SS4", 100, 200);
        Planet planet4 = newPlanet("SS4-A", ss4, null, 110, 0.1);
        planet4.setProductionBonus(1.0);

        player.getTransportManager().setTransport(new Transport(player));
        assertEquals(1, player.getTransportManager().getQtyTransports());
        assertEquals(1, player.getTransportManager().getQtyFreeTransport());
        
        ProjectSpaceStation project = new ProjectSpaceStation(new SpaceStationDesign(tech_spaceStation), new Orbit(planet3));
        planet1.setProject(project);

        assertEquals(planet3,  project.getTarget().getPlanet());

        //
        // Another player deploys a station in this orbit while I am building a Space Station
        //
        
        createSpaceStation(tech_spaceStation, planet3, planet2);

        assertEquals(planet3,  project.getTarget().getPlanet());

		project.build(planet1); 

        assertEquals(1, player.getTransportManager().getQtyTransports());
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());
        
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(planet4,  task.getDestination());
        assertEquals(planet1,  task.getOrigin());

        
    }


    /**
     * Tests 
     *    - A planet builds a Transport carrying an space station
     *    - AI moves the transport to a planet and deploys the space station
     *    - Projects are upgraded to include transports if no transport is available
     *  AND
     *    - A planet builds a Transport carrying an space station module
     *    - AI moves the transport to a the space station to deploy it
     */
    public void testProject_SpaceStationWithNoTransport() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_SpaceStationWithNoTransport() *************************");
        createComputerPlayerMap();

        // Planet 2 should not be mined as we want this as a colony because it is habitable
        // Planet 3 should be mined
        SolarSystem ss3 = newSolarSystem("SS3", 100, 200);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 0.1);
        planet3.setProductionBonus(2.0);
        ResourceFactory.createResource(planet3);
        
        // Define modules that should be on space station
        player.addModule(tech_habitatI);
        player.addModule(tech_habitatII);
        player.addModule(tech_oreExtractor);
        player.addModule(tech_researchModule);
        
        assertEquals(0, player.getTransportManager().getQtyTransports());

        player.getMayor().buildSpaceStation(planet1);
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        
        // We have no transport and no Space Station
        assertEquals(0, player.getTransportManager().getQtyTransports());
        assertEquals(true, ((ProjectSpaceStation)planet1.getProject()).isIncludeTransport());

        planet1.addProduction((int) TransportDesign.getCostToBuild());
        
        // Project is upgraded to include a Transport which has now been created
        assertEquals(1, player.getTransportManager().getQtyTransports());
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(planet3,  task.getDestination());
        assertEquals(planet1,  task.getOrigin());
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(null,  task);
        assertEquals(true,  ss3.hasOwner());
        assertEquals(player,  ss3.getOwners()[0]);
        assertEquals(planet3,  ss3.getColonies(true)[0]);
        assertEquals("SpaceStation",  ss3.getColonies(true)[0].getOrbital().getName());

        // Test modules have been added to space station
        Module[] ma = planet3.getOrbital().getModuleTechs().getModules();
		assertEquals(5,  ma.length);
		assertEquals(tech_core_habitat,  ma[0]);
		assertEquals(tech_habitatI,  ma[1]);
		assertEquals(tech_habitatII,  ma[2]);
		assertEquals(tech_oreExtractor,  ma[3]);
		assertEquals(tech_researchModule,  ma[4]);
    }

    /**
     * Tests 
     *    - A planet builds a Transport carrying an space station
     *    - AI moves the transport to a planet but planet now has another players Space Station
     *    - AI attempts to redeploy the station on another planet
     *    - AI does not mine mineral rich planets if they could be colonised
     */
    public void testProject_SpaceStationWithDestinationTakenDuringTransport() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_SpaceStationDeployTargetTaken() *************************");
        createComputerPlayerMap();

        // make planet2 very mineral rich to test that planets with colonisable terraform
        // level are not mined
        planet2.setProductionBonus(3.0);

        newEnemyPlayer();
    
        // Planet 2 should not be mined as we want this as a colony because it is habitable
        // Planet 3 should be mined
        SolarSystem ss3 = newSolarSystem("SS3", 100, 160);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 0.1);
        planet3.setProductionBonus(2.0);
        
        // We want a transport available
        player.getTransportManagerNotNull().setTransport(new Transport(player));
        player.getMayor().buildSpaceStation(planet1);
        planet1.getProject().build(planet1);
        
        // Computer player has not needed to create a transport
        assertEquals(1, player.getTransportManager().getQtyTransports());
        
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(planet3,  task.getDestination());
        assertEquals(planet1,  task.getOrigin().getPlanet());
        
        
        player.getTransportManager().endTurn();
        
        // Another player places a space station on the planet
        // Make Planet2 non colonisable so the computer player will place a mining station on it.
        newSpaceStation(planetEnemy, planet3);
        planet2.setTeraforming(0.2);
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals("SS2-A",  task.getDestination().getPlanet().getName());
        assertEquals("SS2",  task.getDestination().getStellarObject().getName());
        assertEquals("SS1-A",  task.getOrigin().getName());
        
    }

    /**
     * Tests 
     *    - A planet builds a Transport carrying an space station
     *    - AI moves the transport to a black hole and deploys the space station
     *    - Projects are upgraded to include transports if no transport is available
     *  AND
     *    - A planet builds a Transport carrying an space station module
     *    - AI moves the transport to a the space station to deploy it
     */
    public void testProject_SpaceStationAtBlackHole() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_SpaceStationAtBlackHole() *************************");
        createComputerPlayerMap();

        BlackHole blackHole = BlackHoleFactory.createBlackHole(new Location(100,200));
        
        // Define modules that should be on Space Station
        player.addModule(tech_habitatI);
        player.addModule(tech_oreExtractor);
        player.addModule(tech_researchModule);
        
        // Planet 2 should not be mined because black holes are used first
        // Planet 3 should be mined
        planet2.setProductionBonus(2.0);
        ResourceFactory.createResource(planet2);
        
        player.getTransportManager().setTransport(new Transport(player));
        assertEquals(1, player.getTransportManager().getQtyTransports());

        player.getMayor().buildSpaceStation(planet1);
        planet1.addProduction((int)planet1.getProject().getCostToBuildOnPlanet());
        
        // We have a transport and no Space Station Project
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(blackHole,  task.getDestination());
        assertEquals(planet1,  task.getOrigin());
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        assertEquals(null,  player.getTransportManager().getTransportsAsArray()[0].getTask());
        assertEquals(true,  blackHole.hasOwner());
        assertEquals(player,  blackHole.getOwners()[0]);
        assertEquals(player,  blackHole.getOrbital().getOwner());
        assertEquals(tech_spaceStation,  blackHole.getOrbital().getOrbital().getHullTech());
        assertEquals(0.1,  player.getResourceCache().getResearchBonus(), 0.01);
        
        // Test modules have been added to space station
        Module[] ma = blackHole.getOrbital().getOrbital().getModuleTechs().getModules();
		assertEquals(4,  ma.length);
		assertEquals(tech_core_habitat,  ma[0]);
		assertEquals(tech_habitatI,  ma[1]);
		assertEquals(tech_oreExtractor,  ma[2]);
		assertEquals(tech_researchModule,  ma[3]);

    }

    /**
     * Tests 
     *  - A planet builds a Transport carrying an outpost
     *  - AI moves the transport to a planet and deploys the outpost
     */
    public void testIdleTransport() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testIdleTransport() *************************");
        createComputerPlayerMap();

        // BUILD AN OUTPOST AND WITH NO SS SUITABLE FOR IT TO BE DEPLOYED TO. 
    
        ss2.setLocation(new Location(500,100));
        player.getMayor().buildOutpost(planet1);
        planet1.getProject().build(planet1);
        
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        player.getTransportManager().endTurn();
        assertNull(task.getDestination());
        assertEquals(1, task.getIdleCount());
        assertEquals("SS1-A",  task.getOrigin().getName());
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());

        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();

        // TASK HAS NOT BEEN DELETED BECAUSE IT REACHED MAX IDLE
        assertEquals(5, task.getIdleCount());
        assertEquals(1, player.getTransportManager().getQtyFreeTransport());
        assertEquals(null, player.getTransportManager().getTransportsAsArray()[0].getTask());

        // BUILD AN OUTPOST AND WITH NO SS SUITABLE FOR IT TO BE DEPLOYED TO.
        // BUT THEN AFTER IT BEING IDLE - MAKE A SS2 SUITABLE AND CHECK IT THEN 
        // DEPLOYES TO SS2

        player.getMayor().buildOutpost(planet1);
        planet1.getProject().build(planet1);

        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertNull(task.getDestination());
        assertEquals("SS1-A",  task.getOrigin().getName());

        player.getTransportManager().endTurn();

        assertEquals(1, task.getIdleCount());

        // NOW MAKE PLANET2 SOMETHING THE GENERAL WOULD CONSIDER PLACING A STATION ON        
        ss2.setLocation(new Location(200,100));

        player.getTransportManager().endTurn();

        assertEquals("SS2",  task.getDestination().getStellarObject().getName());

        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(null,  task);
        assertEquals(true,  ss2.hasOwner());
        assertEquals(player,  ss2.getOwners()[0]);
        assertEquals((Planet)null,  ss2.getColonies(true));
        assertEquals(100,  ss2.getOrbital().getScanStrength());
    }

    private ComputerPlayer enemy;
    private Planet planetEnemy;
    private void newEnemyPlayer() throws Exception {
        String [] playerNames = { "TestPlayer1" };
        enemy = newPlayer("Bears", blue, OptionsFactory.getGeneralOptions(), playerNames);
        SolarSystem sse = newSolarSystem("SSe", 400, 100);
        planetEnemy = newPlanet("SSe-A", sse, enemy);
        enemy.diplomaticRequest(player, DiplomaticProposal.getWar(), null);
    }

    /**
     * Tests 
     *  - Transport is destroyed when it arrives at a stellar object with enemy crafts and no protection.
     *  SHOULD NOT NEED TO TEST THE SAME FOR SPACE STATION
     */
    public void testOutpostWithEnemyCraft1() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testOutpostWithEnemyCraft1() *************************");
        createComputerPlayerMap();
        newEnemyPlayer();
        Craft enemyCraft = newCraft(defenderDesign, enemy, ss2);

        SolarSystem ss3 = newSolarSystem("SS3", 300, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 0.8);
        planet2.setTeraforming(0.7);
        
        //
        // Player cannot colonise any planet in range, but, could colonise planet3
        // if an outpost was built in ss2 making it in range.
        //
        
        assertEquals(true, player.canColonise(planet3));
        assertEquals(false, player.canColonise(planet2));
        assertEquals(false, player.canTravelTo(ss3));

        player.getMayor().buildOutpost(planet1);
        planet1.getProject().build(planet1);
        
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());
        assertEquals(1, player.getTransportManager().getQtyTransports());
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(ss2,  task.getDestination());
        assertEquals(planet1,  task.getOrigin());
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        assertEquals(0, player.getTransportManager().getQtyTransports());
        assertEquals(false,  ss3.hasOwner());
        assertEquals(null,  ss3.getOrbital());        
    }
    
    /**
     * Tests 
     *  - Transport is idle destroyed when it arrives at a stellar object with enemy crafts and its own military craft.
     *  - Test that when enemy craft is gone the outpost is finally deployed (after IDLE for 5 turns).
     *  SHOULD NOT NEED TO TEST THE SAME FOR SPACE STATION
     */
    public void testOutpostWithEnemyCraft2() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testOutpostWithEnemyCraft2() *************************");
        createComputerPlayerMap();
        newEnemyPlayer();
        Craft enemyCraft = newCraft(defenderDesign, enemy, ss2);
        Craft friendlyCraft = newCraft(defenderDesign, player, ss2);

        SolarSystem ss3 = newSolarSystem("SS3", 300, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 110, 0.8);
        planet2.setTeraforming(0.7);
        
        //
        // Player cannot colonise any planet in range, but, could colonise planet3
        // if an outpost was built in ss2 making it in range.
        //
        
        assertEquals(true, player.canColonise(planet3));
        assertEquals(false, player.canColonise(planet2));
        assertEquals(false, player.canTravelTo(ss3));

        player.getMayor().buildOutpost(planet1);
        planet1.getProject().build(planet1);
        
        assertEquals(0, player.getTransportManager().getQtyFreeTransport());
        assertEquals(1, player.getTransportManager().getQtyTransports());
        TransportTaskDeployStation task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(null,  task.getDestination().getPlanet());
        assertEquals("SS2",  task.getDestination().getStellarObject().getName());
        assertEquals("SS1-A",  task.getOrigin().getName());
        
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        player.getTransportManager().endTurn();
        
        assertEquals(1, player.getTransportManager().getQtyTransports());
        task = (TransportTaskDeployStation) player.getTransportManager().getTransportsAsArray()[0].getTask();
        assertEquals(planet2,  task.getDestination());
        assertEquals(false,  ss3.hasOwner());
        assertEquals(null,  ss3.getOrbital());
        
        enemyCraft.destroy();
        player.getTransportManager().endTurn();

        assertEquals(1, player.getTransportManager().getQtyFreeTransport());
        assertEquals(true,  ss2.hasOwner());
        assertEquals(player,  ss2.getOwners()[0]);
        assertEquals(100,  ss2.getOrbital().getScanStrength());
    }

    /**
     * Tests 
     *  - A planet builds a Transport carrying an outpost
     *  - AI moves the transport to a planet and deploys the outpost
     *  - Uses Solar Systems with planets
     */
    public void testUpdateEngine() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testUpdateEngine() *************************");
        createComputerPlayerMap();

        new ProjectTransport().build(planet1);
        assertEquals(tech_fusion25, player.getTransportManager().getTransportsAsArray()[0].getEngine());
        
        player.addEngine(tech_fusion40);

        assertEquals(tech_fusion40, player.getTransportManager().getTransportsAsArray()[0].getEngine());
    }
    
    public void testDeployCommand_SpaceStation() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testProject_SpaceStation() *************************");
        createComputerPlayerMap();

        BlackHole blackHole = BlackHoleFactory.createBlackHole(new Location(100,200));
        Planet miningColony = newPlanet("SS1-M", ss1, null, 100, 0.2);
        miningColony.setProductionBonus(2.0);
        ResourceFactory.createResource(miningColony);

        Transport transport = new Transport(player);
		player.getTransportManager().setTransport(transport);
        assertEquals(1, player.getTransportManager().getQtyTransports());
        
        //
        // Test deploy transport at black hole
        //
        
        Orbital orbital = new Orbital(tech_spaceStation, null, null);
        new TransportTaskDeployStation(transport, (Planet) planet1, null, orbital, false);
        
        transport.arrivedAt(new Orbit(blackHole));
        new MooDeployCmd(transport, blackHole, null, player).run();
        
        assertEquals(true,  blackHole.hasOwner());
        assertEquals(player,  blackHole.getOwners()[0]);
        assertEquals(player,  blackHole.getOrbital().getOwner());
        
        //
        // Test deploy transport at mining colony
        //
        
        orbital = new Orbital(tech_spaceStation, null, null);
        new TransportTaskDeployStation(transport, (Planet) planet1, null, orbital, false);
        
        transport.arrivedAt(new Orbit(miningColony));
        new MooDeployCmd(transport, ss1, miningColony, player).run();
        
        assertEquals(player,  miningColony.getOwner());
        assertEquals(miningColony,  planet1.getMiningStation().getPlanet());
        assertFalse(player.hasDiscovered(tech_core_habitat));
        assertEquals(tech_core_habitat, miningColony.getOrbital().getModuleTechs());
        
        //
        // Test deploy transport at mining colony
        //
        
        miningColony.removeColony(true);
        player.addModule(tech_habitatI);
        player.addModule(tech_oreExtractor);
        player.addModule(tech_sentinelsModule);

        orbital = new Orbital(tech_spaceStation, null, null);
        new TransportTaskDeployStation(transport, (Planet) planet1, null, orbital, false);
        
        transport.arrivedAt(new Orbit(miningColony));
        new MooDeployCmd(transport, ss1, miningColony, player).run();
        
        assertEquals(player,  miningColony.getOwner());
        assertEquals(miningColony,  planet1.getMiningStation().getPlanet());
        assertEquals(tech_core_habitat, tech_habitatI, tech_oreExtractor, tech_sentinelsModule, miningColony.getOrbital().getModuleTechs());

    }
    
    public void testDeployCommand_Outpost() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testDeployCommand_Outpost() *************************");
        createComputerPlayerMap();

        BlackHole blackHole = BlackHoleFactory.createBlackHole(new Location(100,200));
        Planet miningColony = newPlanet("SS1-M", ss1, null, 100, 0.2);
        miningColony.setProductionBonus(2.0);
        ResourceFactory.createResource(miningColony);

        Transport transport = new Transport(player);
		player.getTransportManager().setTransport(transport);
        assertEquals(1, player.getTransportManager().getQtyTransports());
        
        //
        // Test deploy transport at black hole
        //
        
        Orbital orbital = new Orbital(tech_outpost, null, null);
        new TransportTaskDeployStation(transport, (Planet) planet1, null, orbital, false);
        
        transport.arrivedAt(new Orbit(blackHole));
        new MooDeployCmd(transport, blackHole, null, player).run();
        
        assertEquals(true,  blackHole.hasOwner());
        assertEquals(player,  blackHole.getOwners()[0]);
        assertEquals(player,  blackHole.getOrbital().getOwner());
        
        //
        // Test deploy transport at mining colony
        //
        
        orbital = new Orbital(tech_outpost, null, null);
        new TransportTaskDeployStation(transport, (Planet) planet1, null, orbital, false);
        
        transport.arrivedAt(new Orbit(miningColony));
        new MooDeployCmd(transport, ss1, null, player).run();
        
        assertEquals(player,  ss1.getOrbital().getOwner());

        //
        // Special case - can deploy space station over outpost at a black hole
        //
        
        orbital = new Orbital(tech_spaceStation, null, null);
        new TransportTaskDeployStation(transport, (Planet) planet1, null, orbital, false);
        
        transport.arrivedAt(new Orbit(blackHole));
        new MooDeployCmd(transport, blackHole, null, player).run();
        
        assertEquals(tech_spaceStation,  blackHole.getOrbital().getOrbital().getHullTech());
    }

    public void testDeployToMiningColony() throws Exception {
        Logger.logDebug("*******************************  PlanetTest.testDeployToMiningColony() *************************");
        createUserPlayerMap();

        Transport transport = new Transport(userPlayer);
        userPlayer.getTransportManagerNotNull().setTransport(transport);
		transport.setTask(new TransportTaskDeployStation(transport, planet1, null, new Orbital(tech_spaceStation, null, null)));
		
		transport.setOrbit(ss2, planet2);
		new MooDeployCmd(transport, ss2, planet2, userPlayer).run();
		
		assertEquals(tech_spaceStation, planet2.getOrbital().getHullTech());
		assertEquals(planet1, planet2.getMiner());
        
        //
        // Test deploy Space Station at a Mining Colony (planet2)
        //

        Transport spaceStationTransport = new Transport(userPlayer);
        userPlayer.getTransportManagerNotNull().setTransport(spaceStationTransport);
		spaceStationTransport.setTask(new TransportTaskDeployStation(spaceStationTransport, planet1, null, new Orbital(tech_spaceStation, null, null)));

		try {
			spaceStationTransport.setOrbit(ss2, planet2);
			new MooDeployCmd(spaceStationTransport, ss2, planet2, userPlayer).run();
			fail();
		} catch (ValidationError e) {
			// Success
		}
        
        //
        // Test deploy Troops at a Mining Colony (planet2)
        //

        Transport troopTransport = new Transport(userPlayer);
        userPlayer.getTransportManagerNotNull().setTransport(troopTransport);
		troopTransport.setTask(new TransportTaskInvade(troopTransport, planet1, null, 10));

		try {
			troopTransport.setOrbit(ss2, planet2);
			new MooDeployCmd(troopTransport, ss2, planet2, userPlayer).run();
			fail();
		} catch (ValidationError e) {
			// Success
		}

        //
        // Test deploy Colonists at a Mining Colony (planet2)
        //

        Transport populationTransport = new Transport(userPlayer);
        userPlayer.getTransportManagerNotNull().setTransport(populationTransport);
		populationTransport.setTask(new TransportTaskMoveColonists(populationTransport, planet1, null, 10));

		try {
			populationTransport.setOrbit(ss2, planet2);
			new MooDeployCmd(populationTransport, ss2, planet2, userPlayer).run();
			fail();
		} catch (ValidationError e) {
			// Success
		}

		try {
			populationTransport.setOrbit(ss2, null);
			new MooMoveCmd(populationTransport, ss2, planet2, false, userPlayer).run();
			//fail();
		} catch (ValidationError e) {
			// Success
		}
		
		assertEquals(TransportTaskMoveColonists.class, populationTransport.getTask().getClass());
    }
    

    protected void tearDown() {
        
    }
    
    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

    
}