/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.event.MotEvent;
import mot.common.event.PlanetInvadeEvent;
import mot.common.proxy.ClientProxy;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.util.ApplicationError;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.engine.MotEngine;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattlePlanet;
import mot.engine.crafts.Craft;
import mot.engine.event.BattleEndEF;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.CrisisEmergency;
import mot.engine.planets.Planet;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.mayor.MayorBasic;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskInvade;

/**
 * Test Building Troop Transports, Their moving on the galaxy map and operating in
 * battle.
 */
public class TroopTransportTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        Logger.setFull();
        TestSuite suite = new TestSuite(TroopTransportTest.class);
        return suite;
    }

    public TroopTransportTest(String name) {
        super(name);
    }

    private SolarSystem ss1;
    private SolarSystem ss2;

    private Player player1;
    private Player player2;

    private Planet planet1;
    private Planet planet2;
    
    private Battle battle;
    
    private Craft craft1;
    private BattleCraft [] bca;
    private BattlePlanet bp;
    
    private PlanetInvadeEvent invadeEvent = null;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        Logger.setDebug();
    }

    private int battleTurn = -1;
    private boolean fighter1LeftBattle = false;
    private boolean fighter2LeftBattle = false;
    private boolean fighter3LeftBattle = false;
    private boolean fighter4Destroyed = false;

    
    public void createMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(player1, player2);
        MotEngine.getInstance().getProxyManager().add(this);
    }

    /**
     * Tests that craft with ground assault modules can attack a planet when (only) its 
     * weapons have been depleted. Tests that planets can be taken in combat and that the 
     * ownership changes.
     * 
     * Tests that ground-combat setup option gives an advantage.
     */
    public void testInvadeSuccess() throws Exception {
        Logger.logDebug("*******************************  TroopTransport.testInvadeSuccess() *************************");
        createMap(null);

        player1.addTechnology(tech_troopModule20);
        player1.addTechnology(tech_fusion25x);
        
        // TEST WHEN A BATTLE FINISHES AND IS WON THEN TROOP TRANSPORTS ARE CREATED 
        // (IF NONE IDLE ALREADY EXIST) TO THE PLANET TO INVADE

        craft1 = newCraft(fighterDesign, player1, ss2);
        Battle testBattle = new Battle(ss2.getOrbit());
        testBattle.join(planet2);
        testBattle.join(craft1, true);
        testBattle.setup();
        planet1.setPopulation(100);
        planet1.setQtyGroundDefense(100);
        Utils.setNextRandomInt(0); // make sure we only build one troop transport
        MotEngine.instance.broadcast(new BattleEndEF(testBattle, null));
        MotEngine.instance.logSummary();
        
        Transport[] transports = player1.getTransportManager().getTransportsAsArray();
        Transport transport = transports[0];
        
        assertEquals("TroopTransport created", 1, transports.length);
        TransportTaskInvade task = (TransportTaskInvade) transport.getTask();
        assertEquals("TroopTransport created", ss1, transport.getOrbit().getStellarObject());
        assertEquals("TroopTransport going to ss2", ss2, transport.getNavigator().getDestination().getStellarObject());
        assertEquals("TroopTransport going to ss2", MayorBasic.QTY_TROOPS_PER_TRANSPORT, task.getQtyAttackers());
        

        //player1.endTurn();
        transport.endTurn();

        assertEquals("TroopTransport going to ss2", ss2, transport.getNavigator().getDestination().getStellarObject());

        transport.endTurn();
        
        // TEST WHEN A BATTLE FINISHES AND IS WON THEN TROOP TRANSPORTS ARE NOT 
        // CREATED IF ALREADY EXIST ENROUTE TO TARGET

        MotEngine.instance.broadcast(new BattleEndEF(testBattle, null));
        assertEquals("No new Troop Transport created", 1, player1.getTransportManager().getTransportsAsArray().length);
        assertEquals("No new Troop Transport created", MayorBasic.QTY_TROOPS_PER_TRANSPORT, player1.getTransportManager().getQtyTroopsWithDestination(planet2));
        assertTrue(planet2.isBlockaded());

        // TEST THAT WHEN A TROOP TRANSPORT ARRIVES IT INVADES

        invadeEvent = null;
        planet2.setQtyGroundDefense(5);
        Utils.setNextRandomDouble(0.5);
        transport.endTurn();
        transport.endTurn();
        transport.endTurn();

        assertEquals("Ground Assault worked", 0, planet2.getQtyGroundDefense());
        assertEquals("Ground Assault worked", player1, planet2.getOwner());
        assertEquals("Ground Assault worked", player2, planet2.getOrigin());
        assertEquals("Ground Assault worked", null, planet2.getCrisis());

        assertEquals("Craft2 is TroopTransport", null, transport.getTask());
        assertEquals("Ground Assault worked", MayorBasic.QTY_TROOPS_PER_TRANSPORT, invadeEvent.getQtyAttackers());
        assertEquals("Ground Assault worked", 5, invadeEvent.getQtyDefenders());
        assertEquals("Ground Assault worked", 2, invadeEvent.getAttackerAdvantage(), 0.0);
        assertEquals("Ground Assault worked", MayorBasic.QTY_TROOPS_PER_TRANSPORT-2, invadeEvent.getRemainingAttackers());
        assertEquals("Ground Assault worked", 0, invadeEvent.getRemainingDefenders());

        assertFalse(planet2.isBlockaded());

        // 
        // TEST THAT TROOPS ARE NOT TAKEN FROM SMALL PLANETS
        //

        
    }

    /**
     * Tests that craft with ground assault modules can attack a planet when (only) its 
     * weapons have been depleted. Tests that planets can be taken in combat and that the 
     * ownership changes.
     * 
     * Tests that ground-combat setup option gives an advantage.
     */
    public void testInvadeFailure() throws Exception {
        Logger.logDebug("*******************************  TroopTransport.testInvadeFailure() *************************");
        createMap(null);

        player1.addTechnology(tech_fusion25x);
        
        planet1.setQtyGroundDefense(100);
        planet2.setQtyGroundDefense(100);

        Transport transport = new Transport(player1);
        player1.getTransportManager().setTransport(transport);
        new TransportTaskInvade(transport, planet1, planet2, 5);

        invadeEvent = null;
        
        Utils.setNextRandomDouble(0.5);
        transport.endTurn();
        transport.endTurn();
        transport.endTurn();
        transport.endTurn();

        assertEquals("Ground Assault failed", 90, planet2.getQtyGroundDefense());
        assertEquals("Ground Assault failed", player2, planet2.getOwner());
        assertEquals("Ground Assault failed", player2, planet2.getOrigin());
        assertEquals("Ground Assault failed", 3000, ((CrisisEmergency)planet2.getCrisis()).getDateStarted());

        assertEquals("TroopTransport has no task", null, transport.getTask());
        assertEquals("Ground Assault failed", 5, invadeEvent.getQtyAttackers());
        assertEquals("Ground Assault failed", 100, invadeEvent.getQtyDefenders());
        assertEquals("Ground Assault failed", 2, invadeEvent.getAttackerAdvantage(), 0.0);
        assertEquals("Ground Assault failed", 0, invadeEvent.getRemainingAttackers());
        assertEquals("Ground Assault failed", 90, invadeEvent.getRemainingDefenders());
        
    }

    /**
     * Tests that craft with ground assault modules can attack a planet when (only) its 
     * weapons have been depleted. Tests that planets can be taken in combat and that the 
     * ownership changes.
     * 
     * Tests that ground-combat setup option gives an advantage.
     */
    public void testArrivalAtPlanetWithEneniesInOrbit() throws Exception {
        Logger.logDebug("*******************************  TroopTransport.testArrivalAtPlanetWithEneniesInOrbit() *************************");
        createMap(null);

        player1.addTechnology(tech_troopModule20);
        player1.addTechnology(tech_fusion25x);

        planet1.setPopulation(100);
        planet1.setQtyGroundDefense(100);
        Utils.setNextRandomInt(0); // make sure we only build one troop transport
        ((ComputerPlayer)player1).getMayor().notifyNeedTroops(planet2);

        Transport[] transports = player1.getTransportManager().getTransportsAsArray();
        Transport transport = transports[0];
        
        assertEquals("TroopTransport created", 1, transports.length);
        TransportTaskInvade task = (TransportTaskInvade) transport.getTask();
        assertEquals("TroopTransport created", ss1, transport.getOrbit().getStellarObject());
        assertEquals("TroopTransport going to ss2", ss2, transport.getNavigator().getDestination().getStellarObject());
        assertEquals("TroopTransport going to ss2", MayorBasic.QTY_TROOPS_PER_TRANSPORT, task.getQtyAttackers());

        //player1.endTurn();
        transport.endTurn();

        assertEquals("TroopTransport going to ss2", ss2, transport.getNavigator().getDestination().getStellarObject());

        
        //
        // TEST THAT WHEN A TROOP TRANSPORT ARRIVES IT INVADES AND ENEMY CRAFT ARE IN ORBIT IT WAITS
        // TRANSPORTS ARE NOT DESTROYED IF MY MILITARY CRAFT ARE THERE AS WELL.
        //
        
        craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player2, ss2);

        invadeEvent = null;
        planet2.setQtyGroundDefense(5);
        Utils.setNextRandomDouble(0.5);
        transport.endTurn();
        transport.endTurn();
        transport.endTurn();
        transport.endTurn();

        assertNull(invadeEvent);
        transports = player1.getTransportManager().getTransportsAsArray();
        assertEquals("TroopTransport created", 1, transports.length);
        assertEquals("Craft2 is TroopTransport", TransportTaskInvade.class, transport.getTask().getClass());

        // 
        // TEST WHEN FRIENDLY MILITARY CRAFT ARE REMOVED THE TRANSPORT IS DESTROYED
        // TRANSPORTS ARE DESTROYED IF ENEMY MILITARY CRAFT ARE IN ORBIT.
        //

        craft1.arrivedAt(ss1.getOrbit());
        transport.endTurn();
        
        assertNull(player1.getTransportManager().getTransportsAsArray());
        
    }    
    protected void tearDown() {

    }


	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
        if (e instanceof PlanetInvadeEvent) {
        	invadeEvent = (PlanetInvadeEvent)e;
        }
    }


	public Player getOwner() {
        return player1;
    }


    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }
}