/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import java.util.ArrayList;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.common.bean.ImageObjectSettings;
import mot.common.bean.ImmunityBio;
import mot.common.bean.ImmunityGravity;
import mot.common.bean.ImmunityMoveCraft;
import mot.common.bean.ImmunityPsi;
import mot.common.bean.Location;
import mot.common.bean.LocationAndDirection;
import mot.common.bean.ObjectMovement;
import mot.common.bean.ObjectShift;
import mot.common.bean.ObjectTurn;
import mot.common.bean.Rectangle;
import mot.common.bean.SalvageList;
import mot.common.bean.Settings;
import mot.common.designs.CraftDesign;
import mot.common.designs.Orbital;
import mot.common.event.BattleCraftLeaveEvent;
import mot.common.event.BattleCraftMoveEvent;
import mot.common.event.BattleCraftNewEvent;
import mot.common.event.BattleCraftSurrenderEvent;
import mot.common.event.BattleCraftTeleportEvent;
import mot.common.event.BattleDamageEvent;
import mot.common.event.BattleDamageRingEvent;
import mot.common.event.BattleDestroyedEvent;
import mot.common.event.BattleDroidMineLaunchEvent;
import mot.common.event.BattleDroidMineMoveEvent;
import mot.common.event.BattleEndEvent;
import mot.common.event.BattleFearEvent;
import mot.common.event.BattleMissileLaunchEvent;
import mot.common.event.BattleMissileMoveEvent;
import mot.common.event.BattleObjectNotificationEvent;
import mot.common.event.BattleObjectTriggerEvent;
import mot.common.event.BattleWeaponFireEvent;
import mot.common.event.BattleWeaponMissEvent;
import mot.common.event.MotEvent;
import mot.common.event.NewBattleEvent;
import mot.common.event.RoutedEvent;
import mot.common.event.StartTurnEvent;
import mot.common.leaders.GeneralDetailsList;
import mot.common.proxy.ClientProxy;
import mot.common.resources.ResourceAntiMatter;
import mot.common.resources.ResourceCloakPalm;
import mot.common.resources.ResourcePsiFungus;
import mot.common.resources.ResourceTritanOre;
import mot.common.resources.ResourceWarpCrystal;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOptions;
import mot.common.technology.Biotechnology;
import mot.common.technology.Hull;
import mot.common.technology.Scanner;
import mot.common.technology.Scanners;
import mot.common.technology.Shield;
import mot.common.technology.Shields;
import mot.common.technology.Special;
import mot.common.technology.Specials;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.Weapon;
import mot.common.technology.Weapons;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.engine.MotEngine;
import mot.engine.battles.Automaton;
import mot.engine.battles.Battle;
import mot.engine.battles.BattleCraft;
import mot.engine.battles.BattleCraftControllerBase;
import mot.engine.battles.BattleCraftControllerBerserker;
import mot.engine.battles.BattleCraftControllerCowardice;
import mot.engine.battles.BattleCraftControllerMindControl;
import mot.engine.battles.BattleObject;
import mot.engine.battles.BattlePlanet;
import mot.engine.battles.BattleStation;
import mot.engine.battles.Battles;
import mot.engine.battles.DroidMine;
import mot.engine.battles.Missiles;
import mot.engine.battles.MovementCostCalculator;
import mot.engine.battles.WreckageCraft;
import mot.engine.battles.incumbants.IncumbentFactory;
import mot.engine.battles.triggers.DamageBot;
import mot.engine.battles.triggers.Plague;
import mot.engine.battles.triggers.Sleep;
import mot.engine.command.BattleAttackCmd;
import mot.engine.command.BattleCraftLaunchFightersCmd;
import mot.engine.command.BattleCraftMoveToCmd;
import mot.engine.command.BattleCraftWaitCmd;
import mot.engine.command.BattleEndTurnCmd;
import mot.engine.command.CraftAttackCmd;
import mot.engine.command.EndTurnCmd;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftDesignAdapter;
import mot.engine.crafts.Crafts;
import mot.engine.event.MotEventFactory;
import mot.engine.leaders.EngineGeneralLeader;
import mot.engine.planets.Planet;
import mot.engine.planets.Planets;
import mot.engine.planets.SolarSystem;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.Player;
import mot.engine.players.Players;
import mot.engine.players.UserPlayer;
import mot.engine.players.generals.CraftTask;
import mot.engine.players.generals.CraftTaskAttack;
import mot.engine.players.generals.GeneralBasic;
import mot.engine.relations.Relations;
import mot.engine.setup.MotFactory;
import mot.engine.technology.ShieldInstance;
import mot.engine.technology.WeaponInstance;
import mot.engine.technology.WeaponInstanceBeam;
import mot.engine.technology.WeaponInstanceBio;
import mot.engine.technology.WeaponInstanceDamageBot;
import mot.engine.technology.WeaponInstanceGirator;
import mot.engine.technology.WeaponInstancePlague;
import mot.engine.technology.WeaponInstancePsi;
import mot.engine.technology.WeaponInstanceSalvageBeam;
import mot.engine.technology.WeaponInstanceSleep;
import mot.engine.technology.WeaponInstanceTeleporter;
import mot.engine.technology.WeaponInstanceTimeWarp;
import mot.engine.technology.WeaponInstanceTroopTransport;

/**
 * Changes - Inter Block Gap is now 3
 * Mass Driver has damage 1
 */

public class BattleTest extends TestBase implements ClientProxy {

    public static Test suite() {
        Logger.clearContents();
        Logger.setInMemory(false);
        TestSuite suite = new TestSuite(BattleTest.class);
        //TestSuite suite = new TestSuite();
        //suite.addTest(new WarTest("testBattleStation"));
        return suite;
    }

    public BattleTest(String name) {
        super(name);
    	try {
			Settings.getInstance();
		} catch (ApplicationError e) {
			e.printStackTrace();
		}
    }

    private SolarSystem ss1;
    private SolarSystem ss2;
    private SolarSystem ss3;

    private UserPlayer userPlayer1;
    private UserPlayer userPlayer2;
    private ComputerPlayer player1;
    private ComputerPlayer player2;
    private ComputerPlayer player3;

    private Planet planet1;
    private Planet planet2;
    private Planet planet3;
    
    private Battle battle;
    
    private Craft craft1;
    private Craft craft2;
    private Craft craft3;
    private Craft craft4;
    private Craft [] enemyCraft;
    private BattleCraft [] bca;
    private BattlePlanet bp;
    private BattleStation bs;

    protected void setUp() throws ApplicationError {
        MotEngine.newInstance();
        setupConfigs();
        MotEngine.getInstance().getProxyManager().add(this);
    }
    
    protected void tearDown() throws Exception {
    	super.tearDown();
    	Logger.flushLog();
    }
    
    public void createMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(player1, player2);
        clearEvents();
    }

    public void createUserPlayerMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        userPlayer1 = newUserPlayer(playerNames[0], blue, null, options);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, userPlayer1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        MotEngine.getInstance().getRelations().setWar(userPlayer1, player2);
        clearEvents();
    }
    public void createTwoUserPlayerMap(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears" };
        SetupOptions optionsGeneral = OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        userPlayer1 = newUserPlayer(playerNames[0], blue, null, options);
        userPlayer2 = newUserPlayer(playerNames[1], green, null, options);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, userPlayer1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, userPlayer2);

        MotEngine.getInstance().getRelationsNotNull().setWar(userPlayer1, userPlayer2);
        clearEvents();
    }

    public void createMapAndTwoCraft(CraftDesign design1, CraftDesign design2, SetupOptions optionsRace) throws Exception {

        createMap(optionsRace);

        craft1 = newCraft(design1, player1, ss2);
        craft2 = newCraft(design2, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);
        
        bca = battle.getCrafts();
        int dist = bca[0].getLocation().getDistanceTo(bca[1].getLocation());
        assertEquals("Distance should be 800", 800, dist);
        assertEquals("Craft1 located on left", bca[0].getLocation().getX(), 100);
        assertEquals("Craft1 located in center left", bca[0].getLocation().getY(), 500);
        assertEquals("Craft2 located on right", bca[1].getLocation().getX(), 900);
        assertEquals("Craft2 located in center left", bca[1].getLocation().getY(), 500);

        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());

        assertEquals("craft1 is pointing to craft2", 90, bca[0].getDirection());
        assertEquals("craft2 is pointing to craft1", 270, bca[1].getDirection());

    	if (optionsRace == null) {
            checkDesignAssumptions(design1, bca[0]);
            checkDesignAssumptions(design2, bca[1]);
        }
    }

    public void createMapAndTwoCraft(CraftDesign design1, int xloc1, CraftDesign design2, int xloc2, SetupOptions optionsRace) throws Exception {
        createMap(optionsRace);

        craft1 = newCraft(design1, player1, ss2);
        craft2 = newCraft(design2, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bca = battle.getCrafts();
        bca[0].setLocation(new Location(xloc1, 500));
        bca[1].setLocation(new Location(xloc2, 500));
        bca[0].setDirection(90);
        bca[1].setDirection(270);

        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());
    }

    
    public void createMapAndThreeCraft(CraftDesign design1, CraftDesign design2, CraftDesign design3, SetupOptions optionsRace) throws Exception {
        createMap(optionsRace);

        craft1 = newCraft(design1, player1, ss2);
        craft2 = newCraft(design2, player2, ss2);
        craft3 = newCraft(design3, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.join(craft3, false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bca = battle.getCrafts();
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(700, 500));
        bca[2].setLocation(new Location(700, 600));
        bca[0].setDirection(90);
        bca[1].setDirection(270);
        bca[2].setDirection(270);

        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());
        assertEquals("craft3 is bca[2]", craft3, bca[2].getCraft());
    }
    
    public void createMapAndCrafts(CraftDesign design1, SetupOptions optionsRace, CraftDesign... enemyDesigns) throws Exception {
        createMap(optionsRace);

        craft1 = newCraft(design1, player1, ss2);
        ArrayList<Craft> cl = new ArrayList();
        int i = 0;
        for (CraftDesign cd : enemyDesigns) {
        	cl.add(newCraft(enemyDesigns[i++], player2, ss2));
        }
        enemyCraft = (Craft[]) cl.toArray(new Craft[cl.size()]);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        for (Craft c : enemyCraft) {        	
        	battle.join(c, false);
        }
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bca = battle.getCrafts();
    }
    
    public void createMapAndThreeCraft(Weapons planetWeapons, Hull planetOrbital, boolean planetWithFighters, boolean planetWithSentinals, CraftDesign design1, CraftDesign design2, CraftDesign design3, SetupOptions optionsRace) throws Exception {
        createMap(optionsRace);

        planet1.setOrbital(new Orbital(planetOrbital, null, null));
        if (planetWithFighters)  { player1.addSpecial(tech_fighterBays); planet1.setWeapons(new Weapons(tech_missile6)); planet1.setShields(new Shields(tech_energy10)); planet1.setScanners(new Scanners(tech_photon100)); } 
        if (planetWithSentinals) player1.addModule(tech_sentinelsModule);
        
        craft1 = newCraft(design1, player1, ss1);
        craft2 = newCraft(design2, player2, ss1);
        craft3 = newCraft(design3, player2, ss1);

        battle = newBattle(ss1);
        battle.join(planet1);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.join(craft3, false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bp = battle.getPlanet();
        
        bca = battle.getCrafts();
        bs = (BattleStation) bca[0];

        assertEquals("craft1 is bca[0]", craft1, bca[1].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[2].getCraft());
        assertEquals("craft3 is bca[2]", craft3, bca[3].getCraft());
        
        bca[1].setLocation(new Location(500, 500));
        bca[2].setLocation(new Location(700, 500));
        bca[3].setLocation(new Location(700, 600));
        bca[1].setDirection(90);
        bca[2].setDirection(270);
        bca[3].setDirection(270);
    }

    
    public void createMapWithThreePlayers(SetupOptions optionsRace) throws Exception {
        String [] playerNames = { "Humans", "Bears", "Ghosts" };
        SetupOptions optionsGeneral = mot.common.setup.OptionsFactory.getGeneralOptions();
        RaceOptions options = new RaceOptions(playerNames[0], optionsRace, optionsGeneral);
        player1 = newPlayer(playerNames[0], blue, options, playerNames);
        player2 = newPlayer(playerNames[1], green, optionsGeneral, playerNames);
        player3 = newPlayer(playerNames[2], blue, optionsGeneral, playerNames);

        ss1 = newSolarSystem("SS1", 100, 100);
        planet1 = newPlanet("SS1-A", ss1, player1);

        ss2 = newSolarSystem("SS2", 200, 100);
        planet2 = newPlanet("SS2-A", ss2, player2);

        ss3 = newSolarSystem("SS3", 300, 300);
        planet3 = newPlanet("SS3-A", ss3, player3);

        MotEngine.getInstance().getRelations().setWar(player1, player2);
        MotEngine.getInstance().getRelations().setWar(player1, player3);
        MotEngine.getInstance().getRelations().setWar(player2, player3);
        clearEvents();
    }
    
    public void checkDesignAssumptions(CraftDesign design, BattleCraft bc) throws ApplicationError {
        if (Comp.same("TestFighter", design)) {
            // Craft is design Fighter, so has engine Fusion and hull Defender.
            // Speed = (engine impulse speed 100) - (hull cost factor 1.2 => 20% slower) = 80
            assertEquals("Fighter correct speed", 100, bc.getSpeed());
            assertEquals("Fighter correct turning speed", 200, bc.getTurningSpeed());
        } else if (Comp.same("TestDefender", design)) {
            // Craft is design Defender, so has engine Fusion and hull Defender.
            // Turning = (engine turning speed 200) / (hull cost factor 1.25) = 200.
            // Speed = (engine impulse speed 100) / (hull cost factor 1.25) = 100.
            assertEquals("Defender correct speed", 80, bc.getSpeed());
            assertEquals("Defender correct turning speed", 160, bc.getTurningSpeed());
        } else if (Comp.same("SuicideFighter", design)) {
        } else if (Comp.same("BioFighter", design)) {
        } else if (Comp.same("HoloFighter", design)) {
        } else {
        }
    }

    protected Battle newBattle(SolarSystem ss) throws ApplicationError {
        Battles battles = MotEngine.getInstance().getBattlesNotNull();
		Battle _battle = battles.getBattleAt(ss.getOrbit(), true);
        battles.start(false);
		return _battle;
    }
    

    /**
     * Tests that crafts cannot move on top of each other.
     */
    public void testCraftMovementSimple() throws Exception {
        Logger.logDebug("*******************************  WarTest.testCraftMovementSimple() *************************");
        createMapAndTwoCraft(fighterDesign, defenderDesign, null);

        assertEquals("Layout Manager has put them down correctly", 100, bca[0].getLocation().getX());
        assertEquals("Layout Manager has put them down correctly", 500, bca[0].getLocation().getY());
        assertEquals("Layout Manager has put them down correctly", 900, bca[1].getLocation().getX());
        assertEquals("Layout Manager has put them down correctly", 500, bca[1].getLocation().getY());

        int dist = bca[0].getLocation().getDistanceTo(bca[1].getLocation());
        assertEquals("Distance should be 800", 800, dist);

        // Test craft1 moves toward craft2
        {
        	//
        	//    T                     C
        	//
        	//   x=100                 x=900
        	//
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Attacking craft is first to move", bca[0], craft);
            fired = false;
            craft.getNextOrders();
            dist = craft.getLocation().getDistanceTo(bca[1].getLocation());
            assertEquals("Craft1 has not moved correct distance", 700, dist);
            assertEquals("Craft1 is too far away to fire", 0.0, bca[1].getCraft().getDamage(), 0.0);
            assertEquals("Craft1 is too far away to fire", false, fired);
            assertTrue("Craft1 pointing towards craft2", craft.getDirection() == 90);
        }

        // Test craft2 moves toward craft1
        {
        	//
        	//    C                     T
        	//
        	//   x=100                 x=800
        	//
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Defending craft is second to move", bca[1], craft);
            craft.getNextOrders();
            dist = craft.getLocation().getDistanceTo(bca[0].getLocation());
            assertEquals("Craft2 has not moved correct distance", 620, dist);
            assertEquals("Craft2 is too far away to fire", 0.0, bca[0].getCraft().getDamage(), 0.0);
            assertEquals("Craft2 is too far away to fire", false, fired);
            assertTrue("Craft2 pointing towards craft1", craft.getDirection() == 270);
        }

        // Test direction change
        {
        	//
        	//   c0          c1           
        	//
        	//   x=300       x=600                     
        	//
            bca[0].setLocation(new Location(300,300));
            bca[0].setDirection(0);
    
            bca[1].setLocation(new Location(600,300));
            bca[1].setDirection(0);

            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft1 moves first", bca[0], craft);
            assertEquals("craft1 pointing to craft2", 90, craft.getDirection());
            
            craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft2 moves second", bca[1], craft);
            assertEquals("craft2 pointing to craft1", 270, craft.getDirection());
        }

        // Test direction change
        {
        	//
        	//   c0                     
        	//
        	//           c1                     
        	//
            bca[0].setLocation(new Location(300,300));
            bca[0].setDirection(45);
    
            bca[1].setLocation(new Location(600,600));
            bca[1].setDirection(200);

            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft1 moves first", bca[0], craft);
            assertEquals("craft1 pointing to craft2", 135, craft.getDirection());
            
            craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft2 moves second", bca[1], craft);
            assertEquals("craft2 pointing to craft1", 315, craft.getDirection());
        }

        // Test direction change
        {
            bca[0].setLocation(new Location(300,300));
            bca[0].setDirection(350);
    
            bca[1].setLocation(new Location(500,100));
            bca[1].setDirection(270);

            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft1 moves first", bca[0], craft);
            assertEquals("craft1 pointing to craft2", 45, craft.getDirection());
            
            craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft2 moves second", bca[1], craft);
            assertEquals("craft2 pointing to craft1", 225, craft.getDirection());
        }

        // Test movement remaining after turning cost
        {
        	craft1.setDamage(0);
        	craft2.setDamage(0);
        	
            bca[0].setLocation(new Location(300,300));
            bca[0].setDirection(0);
    
            bca[1].setLocation(new Location(600,300));
            bca[1].setDirection(0);

            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft1 moves first", bca[0], craft);
            assertEquals("craft1 pointing to craft2", 90, craft.getDirection());
            // turned 90 of 200 => 45% => moves 55% of 100 = 55.
            assertEquals("craft1 has moved", 245, bca[0].getLocation().getDistanceTo(bca[1].getLocation()));
        }

        // Test no movement remaining large turning cost
        {
            bca[0].setLocation(new Location(300,600));
            bca[0].setDirection(0);
            bca[0].setMoveRemaining(0);
    
            bca[1].setLocation(new Location(300,300));
            bca[1].setDirection(0);

            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft2 moved", bca[1], craft);
            // Has a maximum turning speed of 160 (but needs 180).
            assertEquals("craft2 not pointing to craft1", 160, craft.getDirection());
            assertTrue("craft2 has not moved", craft.getLocation().equals(300,300));
        }

        // Test no movement remaining after turning cost
        {
            bca[0].setLocation(new Location(500,850));
            bca[0].setDirection(0);
            bca[0].setMoveRemaining(0);
    
            bca[1].setLocation(new Location(300,300));
            bca[1].setDirection(0);
            bca[1].setMoveRemaining(1);

            // we want an angle of 160 to test when turning cost leaves 0% left
            assertEquals("Angle between 300,300 and 500,849 is 160", 160, bca[1].getLocation().getDirectionFor(bca[0].getLocation()));

            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            craft.getNextOrders();
            assertEquals("craft2 moved", bca[1], craft);
            assertEquals("craft2 pointing to craft1", 160, craft.getDirection());
            assertTrue("craft2 has not moved", craft.getLocation().equals(300,300));
        }

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               D
        //               T
        //              
        //               F
        {
            bca[0].setLocation(new Location(200,300));
            bca[0].setDirection(90);
            bca[0].setMoveRemaining(1);
    
            bca[1].setLocation(new Location(300,300));
            bca[1].setDirection(0);

            bca[0].moveTo(bca[1]);
            
            assertEquals("craft1 pointing to craft2", 90, bca[0].getDirection());
            // craft1 uses hull Flyer which has size 15, craft 2 uses hull Defender which has size 25.
            // So the minimum distance between the craft is 28 (including inter craft gap of 4);
            assertEquals("craft1 has not changed size", 18, craft1.getCraftGraphics().getSize().getX());
            assertEquals("craft2 has not changed size", 25, craft2.getCraftGraphics().getSize().getX());
            assertEquals("Inter craft gap has not changed", 3, ImageObjectSettings.INTER_CRAFT_GAP);
            assertEquals("craft1 has moved to edge of craft2", 272, bca[0].getLocation().getX());
            assertEquals("craft1 has moved to edge of craft2", 300, bca[0].getLocation().getY());
        }

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //               
        //       DF   Craft cannot move any closer       
        //               
        {
            bca[0].setLocation(new Location(200,300));
            bca[0].setDirection(270);
            bca[0].setMoveRemaining(1);
    
            bca[1].setLocation(new Location(173,300));
            bca[1].setDirection(90);

            bca[0].moveTo(bca[1]);
            
            assertEquals(200,300, bca[0].getLocation());
            assertEquals(270, bca[0].getDirection());
        }

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //               
        //       DF   Craft cannot move any closer       
        //               
        {
            bca[0].setLocation(new Location(200,300));
            bca[0].setDirection(270);
            bca[0].setMoveRemaining(1);
    
            bca[1].setLocation(new Location(173,300));
            bca[1].setDirection(90);

            bca[0]._moveTo(bca[1].getLocation());
            
            assertEquals(200,300, bca[0].getLocation());
            assertEquals(270, bca[0].getDirection());
        }

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //               
        //       DF   Craft cannot move any closer (crafts a 1 space apart)       
        //               
        {
            bca[0].setLocation(new Location(200,300));
            bca[0].setDirection(270);
            bca[0].setMoveRemaining(1);
    
            bca[1].setLocation(new Location(172,300));
            bca[1].setDirection(90);

            bca[0]._moveTo(bca[1].getLocation());
            
            assertEquals(200,300, bca[0].getLocation());
            assertEquals(270, bca[0].getDirection());
        }

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //               
        //       DF   Craft are already overlapped (this should never happen in a real game)       
        //               
        {
            bca[0].setLocation(new Location(200,300));
            bca[0].setDirection(90);
            bca[0].setMoveRemaining(1);
    
            bca[1].setLocation(new Location(174,300));
            bca[1].setDirection(90);

            bca[0]._moveTo(bca[1].getLocation());
            
            assertEquals(202,300, bca[0].getLocation());
            assertEquals(270, bca[0].getDirection());
        }


        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //         F    TOX
        //               
        {
            bca[0].setLocation(new Location(200,300));
            bca[0].setDirection(90);
            bca[0].setMoveRemaining(1);
    
            bca[1].setLocation(new Location(300,300));
            bca[1].setDirection(0);

            bca[0].moveTo(new LocationAndDirection(325, 300), 1.0, null, null, false);
            
            // craft1 uses hull Flyer which has size 15, craft 2 uses hull Defender which has size 25.
            // So the minimum distance between the craft is 28 (including inter craft gap);
            assertEquals("craft1 has moved to edge of craft2", 272, 300, bca[0].getLocation());
            assertEquals("craft1 pointing to craft2", 90, bca[0].getDirection());
        }

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               T
        //               O
        //              
        //               F
        {
            bca[0].setLocation(new Location(275,275));
            bca[0].setDirection(135);
            bca[0].setMoveRemaining(1);
    
            bca[1].setLocation(new Location(300,300));
            bca[1].setDirection(0);

            bca[0]._moveTo(new Location(330, 330));
            
            assertEquals("craft1 pointing to craft2", 135, bca[0].getDirection());
            // craft1 uses hull Flyer which has size 15, craft 2 uses hull Defender which has size 25.
            // So the minimum distance between the craft is 28 (including inter craft gap);
            assertEquals("craft1 has moved to edge of craft2", 330, bca[0].getLocation().getX());
            assertEquals("craft1 has moved to edge of craft2", 330, bca[0].getLocation().getY());
        }
    }
    
    /**
     * Tests that crafts can move a percentage of their total move and remaining move is correct.
     * Tests that crafts cannot move more than a full move in one turn.
     */
    public void testCraftMovementPercent() throws Exception {
        Logger.logDebug("*******************************  WarTest.testCraftMovementPercent() *************************");
        createMapAndTwoCraft(fighterDesign, 100, fighterDesign, 500, null);

        ObjectTurn turn;
        ObjectShift shift;

        // Test craft only moves half of its turn. Impulse speed is 100 => moves 50.
        bca[0]._moveTo(bca[1].getLocation(), 0.5);

        assertEquals(1, lastCraftMovementEvent.getMovements().length);
        shift = (ObjectShift) lastCraftMovementEvent.getMovements()[0];
        assertEquals("Craft has moved correctly", 150, bca[0].getLocation().getX());
        assertEquals("Craft has moved correctly", 500, bca[0].getLocation().getY());
        assertEquals("Craft has moved correctly", 0.5, shift.getMovePercent(), 0.001);
        

        // Test craft moves remaining half of its turn.
        bca[0]._moveTo(bca[1].getLocation());

        assertEquals(1, lastCraftMovementEvent.getMovements().length);
        shift = (ObjectShift) lastCraftMovementEvent.getMovements()[0];
        assertEquals("Craft has moved correctly", 90, bca[0].getDirection());
        assertEquals("Craft has moved correctly", 200, bca[0].getLocation().getX());
        assertEquals("Craft has moved correctly", 500, bca[0].getLocation().getY());
        assertEquals("Craft has moved correctly", 0.5, shift.getMovePercent(), 0.001);

        bca[0].setLocation(new Location(100, 500));
        bca[0].setDirection(40);
        bca[0].startTurn();
        
        // Test craft only moves half of its turn. Turn speed = 200 => turn 50 = 25%
        // Impulse speed is 100 => move 25 = 25%.
        bca[0]._moveTo(bca[1].getLocation(), 0.5);

        assertEquals(2, lastCraftMovementEvent.getMovements().length);
        turn = (ObjectTurn) lastCraftMovementEvent.getMovements()[0];
        shift = (ObjectShift) lastCraftMovementEvent.getMovements()[1];
        assertEquals("Craft has moved correctly", 90, bca[0].getDirection());
        assertEquals("Craft has moved correctly", 125, bca[0].getLocation().getX());
        assertEquals("Craft has moved correctly", 500, bca[0].getLocation().getY());
        assertEquals("Craft has moved correctly", 0.25, shift.getMovePercent(), 0.001);
        assertEquals("Craft has moved correctly", 0.25, turn.getMovePercent(), 0.001);

        bca[0].setLocation(new Location(100, 500));
        bca[0].setDirection(190);
        bca[0].startTurn();
        
        // Test craft moves full turn. Turn speed = 200 => turn 100 = 50%
        // Impulse speed is 100 => move 50 = 50%.
        bca[0]._moveTo(bca[1].getLocation(), 1.0);

        assertEquals(2, lastCraftMovementEvent.getMovements().length);
        turn = (ObjectTurn) lastCraftMovementEvent.getMovements()[0];
        shift = (ObjectShift) lastCraftMovementEvent.getMovements()[1];
        assertEquals("Craft has moved correctly", 90, bca[0].getDirection());
        assertEquals("Craft has moved correctly", 150, bca[0].getLocation().getX());
        assertEquals("Craft has moved correctly", 500, bca[0].getLocation().getY());
        assertEquals("Craft has moved correctly", 0.5, shift.getMovePercent(), 0.001);
        assertEquals("Craft has moved correctly", 0.5, turn.getMovePercent(), 0.001);

        bca[0].setLocation(new Location(100, 500));
        bca[0].setDirection(271);
        bca[0].startTurn();
        
        // Test craft only moves half of its turn. Turn speed = 200 => turn 100 = 50%
        bca[0]._moveTo(bca[1].getLocation(), 0.5);

        assertEquals(1, lastCraftMovementEvent.getMovements().length);
        turn = (ObjectTurn) lastCraftMovementEvent.getMovements()[0];
        assertEquals("Craft has moved correctly", 11, bca[0].getDirection());
        assertEquals("Craft has moved correctly", 100, bca[0].getLocation().getX());
        assertEquals("Craft has moved correctly", 500, bca[0].getLocation().getY());
        assertEquals("Craft has moved correctly", 0.5, turn.getMovePercent(), 0.001);

        
        // Test craft only moves half of its turn. Turn speed = 200 => turn 79 = 39.5% & Impulse speed 100 => 0.105% = 10.5
        // Test that only 0.5 of a move is performed despite 1.0 being requested
        bca[0]._moveTo(bca[1].getLocation(), 1.0);

        assertEquals(2, lastCraftMovementEvent.getMovements().length);
        turn = (ObjectTurn) lastCraftMovementEvent.getMovements()[0];
        shift = (ObjectShift) lastCraftMovementEvent.getMovements()[1];
        assertEquals("Craft has moved correctly", 90, bca[0].getDirection());
        assertEquals("Craft has moved correctly", 110, bca[0].getLocation().getX());
        assertEquals("Craft has moved correctly", 500, bca[0].getLocation().getY());
        assertEquals("Craft has moved correctly", 0.105, shift.getMovePercent(), 0.01);
        assertEquals("Craft has moved correctly", 0.395, turn.getMovePercent(), 0.01);
    }

    public void testGetClosestFreeSpaceTo() throws Exception {
        Logger.logDebug("*******************************  WarTest.testGetClosestFreeSpaceTo() *************************");
        createMapAndThreeCraft(fighterDesign, fighterDesign, fighterDesign, null);
        bca[0].getCraft().setEngine(tech_fusion400);
        
        assertEquals(18, bca[0].getCraft().getImageSize().getX());
        assertEquals(18, bca[0].getCraft().getImageSize().getY());
        assertEquals(500, bca[0].getSpeed());
        assertEquals(900, bca[0].getTurningSpeed());
        
        MovementCostCalculator mcc500 = new MovementCostCalculator(900, 500);

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //               
        //  F        OD      
        //           T  
        //               
        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(475,500));
        
        Location targetLoc = bca[1].getLocation();
        Location closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc500, targetLoc, bca[1]);

        assertEquals(475, 525, closestFreeSpaceTo);

        bca[0].moveTo(bca[1]);
        
        assertEquals(475,525, bca[0].getLocation());
        assertEquals(45, bca[0].getDirection());
        
        assertEquals(3, lastCraftMovementEvent.getMovements().length);
        ObjectTurn turn1 = (ObjectTurn) lastCraftMovementEvent.getMovements()[0];
        ObjectShift shift = (ObjectShift) lastCraftMovementEvent.getMovements()[1];
        ObjectTurn turn2 = (ObjectTurn) lastCraftMovementEvent.getMovements()[2];
        
        assertEquals("Craft has moved correctly", 108, turn1.getTo(), 0.001);
        assertEquals("Craft has moved correctly", 0.02, turn1.getMovePercent(), 0.001);
        assertEquals("Craft has moved correctly", 400, shift.getOrigin().getX(), 0.001);
        assertEquals("Craft has moved correctly", 500, shift.getOrigin().getY(), 0.001);
        assertEquals("Craft has moved correctly", 475, shift.getDestination().getX(), 0.001);
        assertEquals("Craft has moved correctly", 525, shift.getDestination().getY(), 0.001);
        assertEquals("Craft has moved correctly", 0.158, shift.getMovePercent(), 0.001); // Distance = 79 so 79 / 500 = 0.158

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //           T   
        //  F        OD      
        //           O   
        //
        BattleCraft nc1 = battle.newCraft(player1, fighterDesign, new Location(475,525));
        bca = battle.getCrafts();
        assertEquals(4, bca.length);
        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(475,500));
        bca[3].setLocation(new Location(475,525));
        
        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc500, targetLoc, bca[1]);

        assertEquals(475, 475, closestFreeSpaceTo);

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //           O   
        //  F       TOD      
        //           O   
        //
        BattleCraft nc2 = battle.newCraft(player1, fighterDesign, new Location(475,475));
        bca = battle.getCrafts();
        assertEquals(5, bca.length);
        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(475,500));
        bca[3].setLocation(new Location(475,525));
        bca[4].setLocation(new Location(475,475));
        
        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc500, targetLoc, bca[1]);

        assertEquals(450, 500, closestFreeSpaceTo);

        closestFreeSpaceTo = battle.getClosestFreeLocationInLine(new Location(500,500), bca[0]);

        assertEquals(450, 500, closestFreeSpaceTo);

        // Test crafts craft move beside if they do not have the speed to move over a craft in front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //     
        //     			   F
        // 				  O
        //               DT
        //
        bca[0].setLocation(new Location(500,500));
        bca[0].setDirection(270);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(450,550));
        bca[2].setLocation(new Location(475,525));
        bca[3].setLocation(new Location(10,10));
        bca[4].setLocation(new Location(10,10));

        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc500, bca[1].getLocation(), bca[1]);

        assertEquals(475,550, closestFreeSpaceTo);
        
        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //           OO
        //           OO  
        //  FF       OODD
        //  FF       OODD   
        //          TTOO
        //          TTOO 
        //
        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(475,500));
        bca[3].setLocation(new Location(492,525));
        bca[4].setLocation(new Location(475,475));
        
        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc500, targetLoc, bca[1]);

        assertEquals(467, 525, closestFreeSpaceTo);
        
        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //          TTOO
        //          TTOO  
        //  FF       OODD
        //  FF       OODD   
        //           OO
        //           OO 
        //
        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(475,500));
        bca[3].setLocation(new Location(475,525));
        bca[4].setLocation(new Location(492,475));
        
        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc500, targetLoc, bca[1]);

        assertEquals(467, 475, closestFreeSpaceTo);
        
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //           O   
        //  F        OD      
        //           OT 	Target is location that overlaps D so needs shifting to closest spot with no craft   
        //

        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(475,500));
        bca[3].setLocation(new Location(475,475));
        bca[4].setLocation(new Location(475,475));
        
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(500, 515), mcc500, targetLoc, bca[1]);

        assertEquals(500,525, closestFreeSpaceTo);

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //        
        //       D        
        //        T     
        //               
        //           F   
        //
        bca[0].setLocation(new Location(600,600));
        bca[0].setDirection(250); // Note this craft would score higher above D because less turning required
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));

        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc500, targetLoc, bca[1]);

        assertEquals(525, 525, closestFreeSpaceTo);

        closestFreeSpaceTo = battle.getClosestFreeLocationInLine(new Location(500,500), bca[0]);

        assertEquals(525, 525, closestFreeSpaceTo);

        // Test crafts can move over crafts in-front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //        
        //          
        //  F O T     D      
        //        
        //
        //
        bca[0].setLocation(new Location(600,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(900,500));
        bca[2].setLocation(new Location(700,500));

        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, new MovementCostCalculator(400, 200), targetLoc, bca[1]);

        assertEquals(800, 500, closestFreeSpaceTo);


        // Test crafts craft move beside if they do not have the speed to move over a craft in front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //        
        //          
        //  F  TO     D      
        //        
        //
        //
        bca[0].setLocation(new Location(600,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(900,500));
        bca[2].setLocation(new Location(700,500));

        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, new MovementCostCalculator(200, 100), targetLoc, bca[1]);

        assertEquals(675, 500, closestFreeSpaceTo);


        // Test crafts craft move beside if they do not have the speed to move over a craft in front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //        
        //          
        //  F   O     D      
        //      T 
        //
        //
        bca[0].setLocation(new Location(600,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(900,500));
        bca[2].setLocation(new Location(700,500));

        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, new MovementCostCalculator(220, 110), targetLoc, bca[1]);

        assertEquals(700, 525, closestFreeSpaceTo);

        
        // Test crafts craft move beside if they do not have the speed to move over a craft beside it
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //        
        //          TF
        //     D    O       
        //       
        //
        //
        bca[0].setLocation(new Location(600,475));
        bca[0].setDirection(270);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(575,500));

        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, new MovementCostCalculator(60, 30), targetLoc, bca[1]);

        assertEquals(571, 475, closestFreeSpaceTo);

        
        // Test crafts craft move beside if they do not have the speed to move over a craft in front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //        
        //    D          
        //    TO     
        //     
        //     F 
        bca = battle.getCrafts();
        assertEquals(5, bca.length);
        bca[0].setLocation(new Location(500,500));
        bca[0].setDirection(350);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(475,420));
        bca[2].setLocation(new Location(500,460));
        bca[3].setLocation(new Location(10,10));
        bca[4].setLocation(new Location(10,10));

        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, new MovementCostCalculator(200, 100), targetLoc, bca[1]);

        assertEquals(475, 445, closestFreeSpaceTo);

        
        // Test crafts craft move beside if they do not have the speed to move over a craft in front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //        
        // DD  OO   OO
        // DD  OO   OOFF    
        //     TT     FF
        //     TT     
        //
        bca[0].setLocation(new Location(515,520));
        bca[0].setDirection(270);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(410,500));
        bca[2].setLocation(new Location(450,500));
        bca[3].setLocation(new Location(490,500));

        targetLoc = bca[1].getLocation();
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, new MovementCostCalculator(200, 100), targetLoc, bca[1]);

        assertEquals(435, 525, closestFreeSpaceTo);

        
    }

    public void testGetClosestFreeSpaceTo_MoveAroundTarget() throws Exception {
        Logger.logDebug("*******************************  WarTest.testGetClosestFreeSpaceTo_MoveAroundTarget() *************************");
        createMapAndThreeCraft(fighterDesign, fighterDesign, fighterDesign, null);
        bca[0].getCraft().setEngine(tech_fusion400);
        
        assertEquals(18, bca[0].getCraft().getImageSize().getX());
        assertEquals(18, bca[0].getCraft().getImageSize().getY());
        assertEquals(500, bca[0].getSpeed());
        assertEquals(900, bca[0].getTurningSpeed());
        
        MovementCostCalculator mcc500 = new MovementCostCalculator(900, 500);

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //			F=T When attacking do not make micro movements (involving lots of turns)           
        //
        //           FD      
        //               
        //               
        bca[0].setLocation(new Location(475,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(10,10));
        
        Location closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(490,500), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(475, 500, closestFreeSpaceTo);

        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(492,498), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(475, 500, closestFreeSpaceTo);

        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(492,502), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(475, 500, closestFreeSpaceTo);

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //            T  
        //           FD      
        //               
        //               
        bca[0].setLocation(new Location(475,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(10,10));
        
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(500,490), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(500, 475, closestFreeSpaceTo);

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //              
        //           TD     F 
        //               
        //               
        bca[0].setLocation(new Location(550,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(10,10));
        
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(495,500), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(475, 500, closestFreeSpaceTo);


        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //              
        //          D     F 
        //          T    
        //               
        bca[0].setLocation(new Location(550,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(10,10));
        
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(500,505), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(500, 525, closestFreeSpaceTo);

        // Test crafts do not overlap on the screen
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //               
        //            F  
        //                 
        //            O    
        //           D
        //
        bca[0].setLocation(new Location(525,460));
        bca[0].setDirection(180);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(500,500));
        bca[2].setLocation(new Location(525,490));
        
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(500,490), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(500, 475, closestFreeSpaceTo);
        
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(490,480), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(490, 475, closestFreeSpaceTo);
        
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(505,495), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(500, 475, closestFreeSpaceTo);
        
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(new Location(505,500), mcc500, bca[1].getLocation(), bca[1]);
        assertEquals(525, 515, closestFreeSpaceTo);
    }
    
    public void testCraftMovementComplex() throws Exception {
        Logger.logDebug("*******************************  WarTest.testCraftMovementComplex() *************************");
        createMapAndThreeCraft(fighterDesign, fighterDesign, fighterDesign, null);
        bca[0].getCraft().setEngine(tech_fusion25);
                
        assertEquals(18, bca[0].getCraft().getImageSize().getX());
        assertEquals(18, bca[0].getCraft().getImageSize().getY());
        assertEquals(100, bca[0].getSpeed());
        assertEquals(200, bca[0].getTurningSpeed());
        
        MovementCostCalculator mcc500 = new MovementCostCalculator(900, 500);
        MovementCostCalculator mcc100 = new MovementCostCalculator(200, 100);
        

        // Test crafts craft move beside if they do not have the speed to move over a craft in front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //     
        //     OO
        // FF  OODD
        // FF  TTDD    
        //     TT     
        //          
        //
        bca[0].setLocation(new Location(500,500));
        bca[0].setDirection(90);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(550,500));
        bca[2].setLocation(new Location(525,488));

        Location targetLoc = bca[1].getLocation();
        Location closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc100, targetLoc, bca[1]);

        assertEquals(525,513, closestFreeSpaceTo);
        
        //
        // ADDITIONAL EXTRA - Check the craft does move here and rotate to target on next move 
        //
        lastCraftMovementEvent = null;
        bca[0].moveTo(bca[1]); 
        
        ObjectMovement[] movements = lastCraftMovementEvent.getMovements();

        assertEquals(3, movements.length);
        ObjectTurn turn1 = (ObjectTurn) movements[0];
        ObjectShift shift = (ObjectShift) movements[1];
        ObjectTurn turn2 = (ObjectTurn) movements[2];
        assertEquals(117, turn1.getTo());
        assertEquals(525, 513, shift.getDestination());
        assertEquals(62, turn2.getTo());

        // Test crafts craft move beside if they do not have the speed to move over a craft in front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //     
        //     			   F
        // 				  O
        //              D T
        //
        bca[0].setLocation(new Location(500,500));
        bca[0].setDirection(270);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(450,550));
        bca[2].setLocation(new Location(475,525));

        targetLoc = bca[0].getClosestLocationTo(bca[1]);
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc500, bca[1].getLocation(), bca[1]);

        assertEquals(475,550, closestFreeSpaceTo);
        
        lastCraftMovementEvent = null;
        bca[0].moveTo(bca[1]); 
        
        movements = lastCraftMovementEvent.getMovements();

        assertEquals(3, movements.length);
        turn1 = (ObjectTurn) movements[0];
        shift = (ObjectShift) movements[1];
        turn2 = (ObjectTurn) movements[2];
        assertEquals("Craft has moved correctly", 206, turn1.getTo());
        assertEquals(475, 550, shift.getDestination());
        assertEquals("Craft has moved correctly", 229, turn2.getTo());

        // Test crafts craft move beside if they do not have the speed to move over a craft in front
        // D = Destination Craft Sent To Occupied, X = Destination Craft Sent To Empty, T = Move To, F = Move From, O = Other Craft
        //     
        //     			   F
        // 				T O
        //              D 
        //
        bca[0].setLocation(new Location(495,500));
        bca[0].setDirection(225);
        bca[0].setMoveRemaining(1);
        bca[1].setLocation(new Location(450,550));
        bca[2].setLocation(new Location(475,525));

        targetLoc = bca[0].getClosestLocationTo(bca[1]);
        closestFreeSpaceTo = bca[0].getClosestFreeSpaceTo(targetLoc, mcc100, bca[1].getLocation(), bca[1]);

        assertEquals(450,525, closestFreeSpaceTo);
        
        lastCraftMovementEvent = null;
        bca[0].moveTo(bca[1]); 
        
         movements = lastCraftMovementEvent.getMovements();

        assertEquals(3, movements.length);
        turn1 = (ObjectTurn) movements[0];
        shift = (ObjectShift) movements[1];
        turn2 = (ObjectTurn) movements[2];
        assertEquals(450, 525, shift.getDestination());
        assertEquals("Craft has moved correctly", 240, turn1.getTo());
        assertEquals("Craft has moved correctly", 180, turn2.getTo());

    }

    
    /**
     * Tests that crafts cannot move on top of each other.
     * Tests that crafts can move a percentage of their total move and remaining move is correct.
     * Tests that crafts cannot move more than a full move in one turn.
     */
    public void testTeleport() throws Exception {
        Logger.logDebug("*******************************  WarTest.testTeleport() *************************");
        CraftDesign teleportDesign = new CraftDesign("TeleportFighter", tech_flyer30, tech_teleport, tech_massDriver1);
        createMapAndTwoCraft(teleportDesign, 400, fighterDesign, 600, null);

        assertEquals("MassDriver1", bca[0].getWeapons()[0].getWeapon().getName());
        assertEquals(2.0, bca[0].getWeapons()[0].getDamage(), 0.0);
        assertEquals(1, bca[0].getWeapons()[0].getQty());

        //
        // Test craft can move / teleport to new locations and a single 
        // teleport uses the whole movement turn
        // Cannot move ontop of craft2
        //
        
        lastCraftMovementEvent = null;
        lastCraftTeleportEvent = null;
        bca[0]._moveTo(bca[1].getLocation(), 0.5);
        
        assertEquals(575,500, bca[0].getLocation());
        assertEquals(90, bca[0].getDirection());

        assertEquals(0.0, bca[0].getMoveRemaining(), 0.001);
        assertEquals(null, lastCraftMovementEvent);
        assertEquals(1.0, lastCraftTeleportEvent.getPercentMove(), 0.001);
        assertEquals(400,500, lastCraftTeleportEvent.getOldState().getLocation());
        assertEquals(575,500, lastCraftTeleportEvent.getNewState().getLocation());
        assertEquals(90, lastCraftTeleportEvent.getOldState().getDirection());
        assertEquals(90, lastCraftTeleportEvent.getNewState().getDirection());
        
        //
        // Can move a long way and turning does not cost anything!
        //
        
        assertEquals(500, tech_teleport.getImpulseSpeed());
        
        lastCraftMovementEvent = null;
        lastCraftTeleportEvent = null;
        bca[0].setLocation(new Location(600,500));
        bca[0].setDirection(90);
        
        bca[0].setMoveRemaining(1.0);
        bca[0]._moveTo(new Location(100,500), 1.0);
        
        assertEquals(100,500, bca[0].getLocation());
        assertEquals(270, bca[0].getDirection());

        assertEquals(null, lastCraftMovementEvent);
        assertEquals(1.0, lastCraftTeleportEvent.getPercentMove(), 0.001);
        assertEquals(600,500, lastCraftTeleportEvent.getOldState().getLocation());
        assertEquals(100,500, lastCraftTeleportEvent.getNewState().getLocation());
        assertEquals(90, lastCraftTeleportEvent.getOldState().getDirection());
        assertEquals(270, lastCraftTeleportEvent.getNewState().getDirection());
        
        //
        // Can move across spatial distortions and asteroid fields 
        // with no affects
        //
        
        lastCraftMovementEvent = null;
        lastCraftTeleportEvent = null;
        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(0);
        
        IncumbentFactory.createAsteroidField(battle);
        assertTrue(battle.getIncumbents().getIncumbants()[0].inside(new Location(750,500)));
        IncumbentFactory.createSpacialDistortion(battle, player2, new Rectangle(600,400,50,200));
        
        bca[0].setMoveRemaining(1.0);
        bca[0]._moveTo(new Location(900,500), 1.0);
        
        assertEquals(new Location(900,500), bca[0].getLocation());
        assertEquals(90, bca[0].getDirection());
        assertEquals(0.0, bca[0].getDamage(), 0.001);

        
        //
        // When move into an asteroid field craft does receive damage
        //
        
        lastCraftMovementEvent = null;
        lastCraftTeleportEvent = null;
        bca[0].setLocation(new Location(400,500));
        bca[0].setDirection(180);
        
        assertEquals(0.0, bca[0].getDamage(), 0.001);

        bca[0].setMoveRemaining(1.0);
        bca[0]._moveTo(new Location(750,500), 1.0);
        
        assertEquals(new Location(750,500), bca[0].getLocation());
        assertEquals(90, bca[0].getDirection());
        assertEquals(5.0, bca[0].getDamage(), 0.001);
    }

    public void testTeleportWithMissileStrike() throws Exception {
        Logger.logDebug("*******************************  WarTest.testTeleportWithMissileStrike() *************************");
        CraftDesign teleportDesign = new CraftDesign("TeleportFighter", tech_flyer60, tech_teleport, tech_massDriver1);
        CraftDesign missileDesign = new CraftDesign("MissileFighter", tech_flyer60, tech_fusion25, tech_missile6);
        createMapAndTwoCraft(missileDesign, 400, teleportDesign, 700, null);

        bca[0].fireAllWeapons(bca[1]);
        bca[0].endTurn();
        Missiles missile = (Missiles) battle.getAutomatons()[0];
        
        assertEquals(400,500, missile.getLocation());
        assertEquals(250, missile.getSpeed());
        assertEquals(bca[1], missile.getTarget());
        
        eventList.clear();
        lastCraftMovementEvent = null;
        lastCraftTeleportEvent = null;

        bca[1].moveTo(bca[0]);
        bca[1].endTurn();
        
        Object[] events = eventList.toArray();
        assertEquals(700,500, ((BattleCraftTeleportEvent)events[0]).getOldState().getLocation());
        assertEquals(431,500, ((BattleCraftTeleportEvent)events[0]).getNewState().getLocation());
        assertEquals(400,500, ((BattleMissileMoveEvent)events[1]).getFrom());
        assertEquals(431,500, ((BattleMissileMoveEvent)events[1]).getTo());
        assertEquals("Missile1", ((BattleWeaponFireEvent)events[2]).getSource());
        assertEquals(431,500, ((BattleWeaponFireEvent)events[2]).getNewTargetState().getLocation());

    }
    

    /**
     * Tests crafts containing fighters can launch the fighters and that computer players can control
     * the fighters to attack enemies.
     */
    public void testMovementWithExclusionZones() throws Exception {
        Logger.logDebug("*******************************  WarTest.testMovementWithExclusionZones() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, null);
        
        //
        // Test with single enemy target craft (must attack with craft in exclusion zone).
        //
		
        CraftDesign suicideDesign = new CraftDesign("SuicideFighter", tech_flyer30, tech_fusion25, tech_detonator, tech_droidMine);
        Craft craft3 = newCraft(suicideDesign, player1, ss2);
        battle.join(craft3, false);

        CraftDesign shieldBurstDesign = new CraftDesign("BurstFighter", tech_flyer30, tech_fusion25, tech_shieldBurst);
        Craft craft4 = newCraft(shieldBurstDesign, player1, ss2);
        battle.join(craft4, false);

        bca = battle.getCrafts();
        
        bca[0].setLocation(new Location(100, 500));
        bca[1].setLocation(new Location(900, 500)); // enemy
        bca[2].setLocation(new Location(100, 470));
        bca[3].setLocation(new Location(100, 530));

        bca[1].endTurn();
        
        // Moves distance 100 from 100,470 -> 900,500 
        bca[2].setMoveRemaining(1.0);
        bca[2].getNextOrders();
        
        assertEquals(154,472, bca[2].getLocation());
        assertEquals(900,500,75, bca[2].getExclusionZone());

        // Moves distance 100 from 100,530 -> 900,500 
        bca[3].setMoveRemaining(1.0);
        bca[3].getNextOrders();

        assertEquals(156,528, bca[3].getLocation());
        assertEquals(900,500,80, bca[3].getExclusionZone());
        
        // Moves distance 100 from 100,500 -> 900,500 
        bca[0].setMoveRemaining(1.0);
        bca[0].getNextOrders();

        assertEquals(200,500, bca[0].getLocation());
        assertEquals(null, bca[0].getExclusionZone());

        
        //
        // Test with three enemy targets (must attack with craft in exclusion zone).
        // Test too far away to fire weapons
        //
		
        Craft craft5 = newCraft(fighterDesign, player2, ss2);
        battle.join(craft5, false);
		
        Craft craft6 = newCraft(fighterDesign, player2, ss2);
        battle.join(craft6, false);

        bca = battle.getCrafts();
        bca[4].endTurn();
        bca[5].endTurn();

        bca[0].setLocation(new Location(100, 500));
        bca[1].setLocation(new Location(900, 500)); // enemy
        bca[2].setLocation(new Location(100, 470));
        bca[3].setLocation(new Location(100, 530));
        bca[4].setLocation(new Location(900, 200)); // enemy
        bca[5].setLocation(new Location(900, 800)); // enemy
        bca[2].setExclusionZone(null);
        bca[3].setExclusionZone(null);

        // Moves distance 100 from 100,470 -> 900,500 
        bca[2].setMoveRemaining(1.0);
        bca[2].getNextOrders();
        
        assertEquals(200,474, bca[2].getLocation());
        assertEquals(900,500,75, bca[2].getExclusionZone());
        assertEquals(bca[2].getName(), lastCraftMovementEvent.getName());
        assertEquals(null, weaponFireEvent);

        // Moves distance 100 from 100,530 -> 900,800 
        bca[3].setMoveRemaining(1.0);
        bca[3].getNextOrders();

        assertEquals(184,558, bca[3].getLocation());
        assertEquals(900,800,80, bca[3].getExclusionZone());
        assertEquals(bca[3].getName(), lastCraftMovementEvent.getName());
        assertEquals(null, weaponFireEvent);

        // Moves distance 100 from 100,500 -> 900,200 
        bca[0].setMoveRemaining(1.0);
        bca[0].getNextOrders();

        assertEquals(175,471, bca[0].getLocation());
        assertEquals(bca[0].getName(), lastCraftMovementEvent.getName());
        assertEquals(null, bca[0].getExclusionZone());
        assertEquals(null, weaponFireEvent);

        //
        // Test when only single craft outside exclusion zone a LONG WAY AWAY
        //

        // Moves distance 100 from 100,800 -> 900,200 
        bca[0].setLocation(new Location(100, 800));
        bca[0].setMoveRemaining(1.0);
        bca[0].getNextOrders();

        assertEquals(174,745, bca[0].getLocation());
        assertEquals(null, bca[0].getExclusionZone());

        //
        // Test with three enemy targets (Do not attack enemy that already has friendly craft close to it).
        // Test too far away to fire weapons
        //
		
        bca[0].setLocationDirection(new Location(550, 500), 90);
        bca[1].setLocationDirection(new Location(600, 500), 270); // enemy
        bca[2].setLocationDirection(new Location(100, 470), 90);
        bca[3].setLocationDirection(new Location(100, 530), 90);
        bca[4].setLocationDirection(new Location(900, 200), 270); // enemy
        bca[5].setLocationDirection(new Location(900, 800), 270); // enemy
        bca[0].setExclusionZone(null);
        bca[2].setExclusionZone(null);
        bca[3].setExclusionZone(null);

        // Moves distance 100 from 100,470 -> 900,200 
        bca[2].rechargeWeapons();
        bca[2].setMoveRemaining(1.0);
        bca[2].getNextOrders();
        
        assertEquals(185,441, bca[2].getLocation());
        assertEquals(900,200,75, bca[2].getExclusionZone());
        assertEquals(bca[2].getName(), lastCraftMovementEvent.getName());
        assertEquals(null, weaponFireEvent);

        // Moves distance 100 from 100,530 -> 900,800 
        bca[3].rechargeWeapons();
        bca[3].setMoveRemaining(1.0);
        bca[3].getNextOrders();

        assertEquals(186,559, bca[3].getLocation());
        assertEquals(900,800,80, bca[3].getExclusionZone());
        assertEquals(bca[3].getName(), lastCraftMovementEvent.getName());
        assertEquals(null, weaponFireEvent);

        //
        // Suicide Craft do not attack enemy that already has friendly craft close to it
        // Test do not fire detonator unless very close
        //
		
        bca[0].setLocationDirection(new Location(100, 500), 90);
        bca[1].setLocationDirection(new Location(600, 500), 270); // enemy
        bca[2].setLocationDirection(new Location(400, 500), 90);
        bca[3].setLocationDirection(new Location(100, 530), 90);
        bca[4].setLocationDirection(new Location(900, 200), 270); // enemy
        bca[5].setLocationDirection(new Location(900, 800), 270); // enemy
        bca[0].setExclusionZone(null);
        bca[2].setExclusionZone(null);
        bca[3].setExclusionZone(null);
        
        assertEquals(true, bca[2].isSuicideFighter());

        // Moves distance 100 from 100,470 -> 900,200 
        bca[2].rechargeWeapons();
        bca[2].setMoveRemaining(1.0);
        bca[2].getNextOrders();
        
        assertEquals(500,500, bca[2].getLocation());
        assertEquals(600,500,75, bca[2].getExclusionZone());
        assertEquals(bca[2].getName(), lastCraftMovementEvent.getName());
        assertEquals(null, weaponFireEvent);
        assertEquals(false, bca[3].isDestroyed());


        //
        // Suicide Craft do not fire Detonator unless very close
        //
		
        bca[0].setLocationDirection(new Location(100, 500), 90);
        bca[1].setLocationDirection(new Location(600, 500), 270); // enemy
        bca[2].setLocationDirection(new Location(400, 500), 90);
        bca[3].setLocationDirection(new Location(100, 530), 90);
        bca[4].setLocationDirection(new Location(900, 200), 270); // enemy
        bca[5].setLocationDirection(new Location(900, 800), 270); // enemy
        bca[0].setExclusionZone(null);
        bca[2].setExclusionZone(null);
        bca[3].setExclusionZone(null);

        // Moves distance 100 from 100,470 -> 600, 500 
        lastCraftMovementEvent = null;
        damageRingEvent1 = null;
        bca[2].rechargeWeapons();
        bca[2].setMoveRemaining(1.0);
        bca[2].getNextOrders();
        
        assertEquals(500,500, bca[2].getLocation());
        assertEquals(600,500,75, bca[2].getExclusionZone());
        assertEquals(bca[2].getName(), lastCraftMovementEvent.getName());
        assertEquals(null, damageRingEvent1);



        //
        // Suicide Craft does fire Detonator when very close
        //
		
        // Moves distance 100 from 500,500 -> 600, 500
        lastCraftMovementEvent = null;
        bca[2].rechargeWeapons();
        bca[2].setMoveRemaining(1.0);
        bca[2].getNextOrders();
        
        assertEquals(575,500, bca[2].getLocation());
        assertEquals(600,500,75, bca[2].getExclusionZone());
        assertEquals(bca[2].getName(), lastCraftMovementEvent.getName());
        assertEquals("DroidMine1", damageRingEvent1.getSource());
        assertEquals(bca[2].getName(), damageRingEvent2.getSource());
        assertEquals(bca[2].getName(), droidMineLaunchEvent.getDroidMine().getOrigin());
        


    }

    /**
     * Tests crafts containing fighters can launch the fighters and that computer players can control
     * the fighters to attack enemies.
     */
    public void testFastMovementWithExclusionZones() throws Exception {
        Logger.logDebug("*******************************  WarTest.testFastMovementWithExclusionZones() *************************");
        
        createMapAndTwoCraft(fastFighterDesign, fighterDesign, null);
        setupFlyer200Graphics();
        
        CraftDesign suicideDesign = new CraftDesign("SuicideFighter", tech_flyer30, tech_fusion400, tech_detonator, tech_droidMine);
        Craft craft3 = newCraft(suicideDesign, player1, ss2);
        battle.join(craft3, false);

        CraftDesign shieldBurstDesign = new CraftDesign("BurstFighter", tech_flyer30, tech_fusion400, tech_energy20, tech_shieldBurst);
        Craft craft4 = newCraft(shieldBurstDesign, player1, ss2);
        battle.join(craft4, false);
		
        Craft craft5 = newCraft(bigFighterDesign, player2, ss2);
        battle.join(craft5, false);
		
        Craft craft6 = newCraft(bigFighterDesign, player2, ss2);
        battle.join(craft6, false);

        
       //
       // Test Suicide craft attacks larger enemy even if its further away (provided its in range)
       //

        bca = battle.getCrafts();
        bca[1].endTurn();
        bca[4].endTurn();
        bca[5].endTurn();

        bca[0].setLocation(new Location(100, 500));
        bca[1].setLocation(new Location(200, 500)); // small enemy
        bca[2].setLocation(new Location(100, 500));
        bca[3].setLocation(new Location(100, 500));
        bca[4].setLocation(new Location(300, 300)); // enemy
        bca[5].setLocation(new Location(300, 800)); // enemy
        bca[0].setExclusionZone(null);
        bca[2].setExclusionZone(null);
        bca[3].setExclusionZone(null);

        // Moves distance 400 from 100,500 -> 300,300 
        bca[3].setMoveRemaining(1.0);
        bca[3].getNextOrders();
        MotEngine.instance.getProxyManager().broadcastQueue();
        
        assertEquals(267,333, bca[3].getLocation());
        assertEquals(300,300,80, bca[3].getExclusionZone());
        assertEquals(bca[3].getName(), lastCraftMovementEvent.getName());
        assertEquals(bca[3].getName(), damageRingEvent1.getSource());
        assertEquals(267,333, damageRingEvent1.getLocation());

        // Moves distance 400 from 100,500 -> 300,800 
        bca[2].setMoveRemaining(1.0);
        bca[2].getNextOrders();
        MotEngine.instance.getProxyManager().broadcastQueue();
        
        assertEquals(276,767, bca[2].getLocation());
        assertEquals(300,800,275, bca[2].getExclusionZone());
        assertEquals(bca[2].getName(), lastCraftMovementEvent.getName());
        assertEquals(276,767, damageRingEvent2.getLocation());
        assertEquals(bca[2].getName(), damageRingEvent2.getSource());
        assertEquals(276,767, droidMineLaunchEvent.getDroidMine().getState().getLocation());
        assertEquals(droidMineLaunchEvent.getDroidMine().getName(), damageRingEvent1.getSource());
        assertEquals(276,767, damageRingEvent1.getLocation());
    }


    public void testSpecialsThatEffectMovement() throws Exception {
        Logger.logDebug("*******************************  WarTest.testSpecialsThatEffectMovement() *************************");
        createMapAndTwoCraft(fighterDesign, defenderDesign, null);


        // Test movement specials
        {
            craft1.setSpecials(new Specials(tech_augumentedEnginesI));
            assertEquals("Fighter has speed 100", 110, bca[0].getSpeed());
            assertEquals("Fighter has turning speed 100", 220, bca[0].getTurningSpeed());
            
            bca[0].setLocation(new Location(100,100));
            bca[0].setDirection(70);
            bca[0].setMoveRemaining(1);
            bca[1].setLocation(new Location(300,300));
            bca[1].setDirection(0);
    
            bca[0]._moveTo(new Location(500, 100));
            
            assertEquals("craft1 pointing to 500, 100", 90, bca[0].getDirection());
            assertEquals("craft1 has moved toward 500, 100", 200, bca[0].getLocation().getX());
            assertEquals("craft1 has moved toward 500, 100", 100, bca[0].getLocation().getY());

            craft1.setSpecials(null);
        }

        // Test condition effects movement
        {
            assertEquals("Fighter has speed 100", 20, craft1.getMaxDamage());
            craft1.setDamage(10);
            assertEquals("Fighter has speed 100", 87, bca[0].getSpeed());
            assertEquals("Fighter has turning speed 100", 175, bca[0].getTurningSpeed());
            
            bca[0].setLocation(new Location(100,100));
            bca[0].setDirection(70);
            bca[0].setMoveRemaining(1);
    
            bca[0]._moveTo(new Location(500, 100));
            
            assertEquals("craft1 pointing to 500, 100", 90, bca[0].getDirection());
            assertEquals("craft1 has moved toward 500, 100", 177, bca[0].getLocation().getX());
            assertEquals("craft1 has moved toward 500, 100", 100, bca[0].getLocation().getY());

            craft1.setDamage(0);
        }
        
    }

    /**
     * Tests that getNextCraft() does return craft of highest initiative AND that the wait list can delay a craft's use.
     */
    public void testNextCraftToMove() throws Exception {
        Logger.logDebug("*******************************  WarTest.testNextCraftToMove() *************************");
        createUserPlayerMap(null);

        Craft craft1 = newCraft(fighterDesign, userPlayer1, ss1);
        Craft craft2 = newCraft(defenderDesign, player2, ss1);
        
        planet1.setWeapons(new Weapons(tech_missile6));
        planet1.setOrbital(new Orbital(tech_battleStationI, null, null));

        battle = newBattle(ss2);
        battle.join(craft1, false);
        battle.join(craft2, true);
        battle.join(planet1);
        battle.setup();

        bp = battle.getPlanet();
        bca = battle.getCrafts();
        BattleCraft bc1 = bca[1];
        BattleCraft bc2 = bca[2];
        BattleStation bs = (BattleStation) bca[0];
        
        assertEquals(craft1, bc1.getCraft());
        assertEquals(craft2, bc2.getCraft());
        
        assertEquals(175, bc1.getInitiative());
        assertEquals(150, bc2.getInitiative());
        assertEquals(bc1, battle.getNextToMove(true));
        
        battle.addToWaitList(bc1);
        
        // do it twice because GUI calls this multiple times AND state changes
        assertEquals(bc2, battle.getNextToMove(false));
        assertEquals(bc2, battle.getNextToMove(false));
        
        // do it twice because GUI calls this multiple times AND state changes
        assertEquals(bs, battle.getNextToMove(true));
        assertEquals(bs, battle.getNextToMove(true));
        assertEquals(true, bc2.hasMoved());
        
        bs.endTurn();
        
        // do it twice because GUI calls this multiple times AND state changes
        assertEquals(bc1, battle.getNextToMove(true));
        assertEquals(bc1, battle.getNextToMove(true)); 
        
        bc1.endTurn();

        // do it twice because GUI calls this multiple times AND state changes
        assertEquals(bp, battle.getNextToMove(true));
        assertEquals(bp, battle.getNextToMove(true));
        
        bp.dischargeWeapons();
        
        //
        // Test destroyed craft are removed from wait list
        //
        
        bc1.getCraft().setDamage(bc1.getCraft().getMaxDamage()-1);
        bc1.setLocation(new Location(500, 500));
        bc1.setDirection(90);
        bc2.setLocation(new Location(550, 500));
        bc2.setDirection(270);
        
        bc2.rechargeWeapons();
        bc2.fireAllWeapons(bc1);
        
        assertTrue(bc1.isDestroyed());
        assertEquals(2, battle.getCrafts().length);
        assertEquals(bs, battle.getCrafts()[0]);
        assertEquals(bc2, battle.getCrafts()[1]);
        assertEquals((BattleCraft)null, battle.getWaitingCraft());

        // do it twice because GUI calls this multiple times AND state changes
        assertEquals(bs, battle.getNextToMove(true));
        assertEquals(bs, battle.getNextToMove(true));
    }
    
    /**
     * Tests that A planet with no weapons but has fighters can still join a battle and be active in it.
     */
    public void testNextCraftToMoveWithPlanetFighters() throws Exception {
        Logger.logDebug("*******************************  WarTest.testNextCraftToMoveWithPlanetFighters() *************************");
                
        createTwoUserPlayerMap(null);
        userPlayer1.addSpecial(tech_fighterBays);
        craft1 = newCraft(fighterDesign, userPlayer2, ss1);

        planet1.setShields(new Shields(tech_energy10));
        planet1.setScanners(new Scanners(tech_photon150));
        planet1.setWeapons(new Weapons(tech_missile6));
        planet1.setOrbital(new Orbital(tech_battleStationI, null, null));

        craft1.attack(planet1);
        
        battle = MotEngine.instance.getBattles().getBattles()[0];
        battle.setup();

        bp = battle.getPlanet();        
        bca = (BattleCraft[]) order(battle.getCrafts(), new BattleCraft[2], "BattleStation1", "TestFighter1");
        
        bca[1].setLocation(new Location(300, 500));
        bca[1].setDirection(270);
        
        new BattleCraftWaitCmd(userPlayer2, battle, bca[1]).run();

        assertEquals(bca[1], battle.getWaitingCraft());
        assertEquals(bca[0], battle.getNextToMove(true));

        //
        // Can attack when higher priority craft is waiting
        //
        
        clearEvents();
        new BattleAttackCmd(userPlayer1, battle, bca[0], bca[1], true, true, true, null, null, -1, false).run();

        assertEquals(bca[0].getName(), ((BattleWeaponFireEvent)eventList.get(0)).getSource());
        assertEquals(bca[1].getName(), ((BattleWeaponFireEvent)eventList.get(0)).getTarget());
        bca[1].getCraft().setDamage(0.0);
        
        assertEquals(bca[1], battle.getNextToMove(true));
        
        clearEvents();
        new BattleAttackCmd(userPlayer2, battle, bca[1], bca[0], true, true, true, null, null, -1, false).run();

        assertEquals(bca[1].getName(), ((BattleCraftMoveEvent)eventList.get(0)).getName());
        assertEquals(bca[1].getName(), ((BattleWeaponFireEvent)eventList.get(1)).getSource());
        assertEquals(bca[0].getName(), ((BattleWeaponFireEvent)eventList.get(1)).getTarget());
        assertNull(battle.getWaitingCraft());
        bca[0].getCraft().setDamage(0.0);
        
        assertEquals(bp, battle.getNextToMove(true));
        
        //
        // Test planet can end its turn and launch fighters next turn
        //

        new BattleEndTurnCmd(battle, bp, userPlayer1, true, true, null, -1).run();

        assertEquals(2, battle.getTurn());
        assertEquals(bca[1], battle._getNextToMove());
        
        clearEvents();
        assertEquals(bca[1], battle.getNextToMove(true));

        new BattleEndTurnCmd(battle, bca[1], userPlayer2, true, true, null, -1).run();

        assertEquals(bca[0], battle.getNextToMove(true));

        new BattleEndTurnCmd(battle, bca[0], userPlayer1, true, true, null, -1).run();

        assertEquals(bp, battle.getNextToMove(true));
        
        clearEvents();
        new BattleAttackCmd(userPlayer1, battle, bp, bca[1], true, true, true, null, null, -1, false).run();

        assertEquals(bca[1].getName(), ((BattleMissileLaunchEvent)eventList.get(0)).getMissile().getTarget());

        clearEvents();
        new BattleCraftLaunchFightersCmd(userPlayer1, battle, bp).run();

        bca = (BattleCraft[]) order(battle.getCrafts(), new BattleCraft[6], "BattleStation1", "TestFighter1", "BattleFighter1", "BattleFighter2", "BattleFighter3", "BattleFighter4");
    }
    
    /**
     * Tests that the weapon inflict the right damage and the shields block damage until their power 
     * is exhausted.
     */
    public void testAttack() throws Exception {
        Logger.logDebug("*******************************  WarTest.testAttack() *************************");
        createMapAndTwoCraft(fighterDesign, defenderDesign, null);
        
        int dist = bca[0].getLocation().getDistanceTo(bca[1].getLocation());
        assertEquals("Distance should be 800", 800, dist);

        assertTrue("Craft1 has weapons to fire", bca[0].hasWeaponsToFire());
        assertFalse("Craft1 is too far away to fire", bca[0].inWeaponRange(bca[1].getLocation()));

        // place craft within weapon range
        bca[0].setLocation(new Location(100,100));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(251,100));
        bca[1].setDirection(270);

        assertTrue("Craft1 is in weapons range", bca[0].inWeaponRange(bca[1].getLocation()));
        assertEquals("Craft2 has no damage", 0.0, bca[1].getCraft().getDamage(), 0.0);
        assertEquals("Craft2 has shields at full power", 10.0, bca[1].getShieldPower(), 0.0);
        
        assertEquals(tech_massDriver1, bca[0].getWeapons()[0].getWeapon());
        assertEquals(2.0, bca[0].getWeapons()[0].getDamage(), 0.0);
        assertEquals(1, bca[0].getWeapons()[0].getQty());

        // Test basic beam weapon damage is correct
        {
            bca[0].fireAllWeapons(bca[1]);
            ShieldInstance[] shields = bca[1].getShields();
            Weapon[] weapons = craft1.getWeaponBays().getAllWeapons();
    
            assertTrue("Craft1 is within weapons range and has fired weapons", !bca[0].hasWeaponsToFire());
            // Craft 1 has MassDriver1 which has variance 0, degradation 0, damage 2.
            // Craft 1 has manvAdv of 1.2069 => damage = 2.4138
            // Craft 2 has Energy10 shield which blocks 1 => damage = 1.4138
            assertEquals("Craft1 inflicted correct damage on craft2", 1.4138, craft2.getDamage(), 0.001);
            assertEquals("Craft2 shields have lost power", 7.5862, shields[0].getPower(), 0.001);
        }
        
        // Check variance works - that damage does change according to the
        // variance.
        {
            Weapon[] weapons = craft1.getWeaponBays().getAllWeapons();
            weapons[0].setVariance(0.2);
            ShieldInstance[] shields = bca[1].getShields();
            
            ((BattleCraft)bca[0]).setMorale(0.0);
            craft2.setDamage(0);
            shields[0].setPower(10);
            bca[0].rechargeWeapons();
            bca[0].fireAllWeapons(bca[1]);
            assertEquals("Craft1 does correct damage to craft2", 1.4138, craft2.getDamage(), 0.001);
            
            ((BattleCraft)bca[0]).setMorale(-1.0);
            craft2.setDamage(0);
            shields[0].setPower(10);
            bca[0].rechargeWeapons();
            bca[0].fireAllWeapons(bca[1]);
            assertEquals("Craft1 does correct damage to craft2", 0.9310, craft2.getDamage(), 0.001);
            
            ((BattleCraft)bca[0]).setMorale(1.0);
            craft2.setDamage(0);
            shields[0].setPower(10);
            bca[0].rechargeWeapons();
            bca[0].fireAllWeapons(bca[1]);
            assertEquals("Craft1 does correct damage to craft2", 1.8965, craft2.getDamage(), 0.001);
            
            weapons[0].setVariance(0.0);
        }

        // Check degradation works for MassDriver1 set degradation to 0.5
        // Damage 2 - Damage 2 * (dist 151 * degradation 0.5) / DEGREDATION_DISTANCE_FACTOR 100 = 0.49
        // Craft 1 has manvAdv of 1.2069 => damage = 0.5914
        {
            ShieldInstance[] shields = bca[1].getShields();
            Weapon[] weapons = craft1.getWeaponBays().getAllWeapons();
            weapons[0].setDegredation(0.5);
            craft2.setDamage(0);
            shields[0].setPower(0);
            bca[0].rechargeWeapons();
            bca[0].fireAllWeapons(bca[1]);
            assertEquals("Craft1 not inflicted correct damage on craft2", 0.5914, craft2.getDamage(), 0.001);
            weapons[0].setDegredation(0);
        }
        
        // check that condition of a craft does reduce the damage it inflicts on other crafts
        // Max Dam = 20, Dam = 10 => condition = 0.5
        // Craft 1 has manvAdv of 1.0758 => damage = 2 * (condition 0.5) * (manv adv 1.0758) = 1.2069
        {
            ShieldInstance[] shields = bca[1].getShields();
            Weapon[] weapons = craft1.getWeaponBays().getAllWeapons();
            craft1.setDamage(10);
            craft2.setDamage(0);
            shields[0].setPower(0);
            bca[0].rechargeWeapons();
            bca[0].fireAllWeapons(bca[1]);
            assertEquals("Craft1 inflicted correct damage on craft2", 1.6, craft2.getDamage(), 0.1);
            craft1.setDamage(0);
            craft2.setDamage(0);
        }
            
        // Check that shields do fail after repeated attack
        // Energy10 has power 10, reduced by dam 2.4138 * discharge rate 1 per attack.
        // Implies it takes about 5 turns to lose shields.
        {
            ShieldInstance[] shields = bca[1].getShields();
            Weapon[] weapons = craft1.getWeaponBays().getAllWeapons();
            shields[0].setPower(10);
            for (int i=0; i<6;i++) {
                craft2.setDamage(0);
                bca[0].rechargeWeapons();
                bca[0].fireAllWeapons(bca[1]);
            }
            assertEquals("Craft2 shields have not colapsed", 0, shields[0].getPower(), 0);
            assertEquals("Craft1 inflicted correct damage on craft2", 2.4138, craft2.getDamage(), 0.001);
        }
        
    }
    
    /**
     * Tests crafts cannot attack another object unless it is pointing at it (when hull weapon
     * bay restricts firing angle).
     */
    public void testAttackWithDefaultDamage() throws Exception {
        Logger.logDebug("*******************************  WarTest.testDefaultWeaponDamage() *************************");
        createMap(null);

        CraftDesign ddf = new CraftDesign("DDF", tech_flyer30, tech_fusion25, tech_electronic25, tech_weaponWithDefaultDamage);
        Craft craft1 = newCraft(ddf, player1, ss2);
        Craft craft2 = newCraft(coloniserDesign, player2, ss2);
        
        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.setup();

        bca = battle.getCrafts();
        bca[0].setLocation(new Location(500, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(600, 500));
        
        //
        // Check that damage is calculated when players have NO weapons
        //
        
        Player[] players = MotEngine.instance.getPlayers().getPlayers();
        players[0].getWeapons().removeAll();
        players[1].getWeapons().removeAll();
        
        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(1.0, bca[0].getWeapons()[0].getDamage(), 1.0);
        assertEquals(1.3, bca[1].getCraft().getDamage(), 1.5);
        
        //
        // Check that default damage is cached and only changes when date increases
        //
        
        players[0].getWeapons().add(tech_massDriver2);
        
        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(1.0, bca[0].getWeapons()[0].getDamage(), 1.0);
        assertEquals(1.3, bca[1].getCraft().getDamage(), 1.5);
        
        //
        // Check that ANY Player having better weapon increases the damage of the default weapon
        //
        
        MotEngine.instance.setDate(MotEngine.instance.getDate()+1);
        
        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(2.0, bca[0].getWeapons()[0].getDamage());
        assertEquals(2.6, bca[1].getCraft().getDamage(), 0.1);
        
        //
        // Check that ANY Player having better weapon increases the damage of the default weapon
        //
        
        players[1].getWeapons().add(tech_fusionBeam4);
        MotEngine.instance.setDate(MotEngine.instance.getDate()+1);
        
        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(4.0, bca[0].getWeapons()[0].getDamage());
        assertEquals(5.25, bca[1].getCraft().getDamage(), 0.1);
    }
    

    /**
     * Tests that the weapon inflict the right damage and the shields block damage until their power 
     * is exhausted.
     */
    public void testManouverabilityAdvantage() throws Exception {
        Logger.logDebug("*******************************  WarTest.testManouverabilityAdvantage() *************************");
        createMapAndTwoCraft(fighterDesign, fighterDesign, null);
        
        assertEquals(1.0, bca[0].getManouverabilityAdvantage(bca[1]), 0.0001);
        
        // Craft 0 manv = (100 + 200)/2 + 40;
        // Craft 1 manv = (100 + 200)/2 + 25;
        bca[0].getCraft().getComputers().upgradeTo(Technologies.getComputer(TestTechs.ELECTRONIC40));
        assertEquals(1.0857, bca[0].getManouverabilityAdvantage(bca[1]), 0.001);
        
        // Craft 0 manv = (100*1.1 + 200*1.1)/2 + 40;
        // Craft 1 manv = (100 + 200)/2 + 25;
        bca[0].getCraft().setSpecials(new Specials(Technologies.getSpecial(TestTechs.AUGUMENTED_ENGINEI)));
        assertEquals(1.1714, bca[0].getManouverabilityAdvantage(bca[1]), 0.001);
        
        // Craft 0 manv = (120 + 250)/2 + 40;
        // Craft 1 manv = (100 + 200)/2 + 25;
        bca[0].getCraft().setSpecials(new Specials());
        bca[0].getCraft().setEngine(Technologies.getEngine(TestTechs.FUSION40));
        assertEquals(1.2857, bca[0].getManouverabilityAdvantage(bca[1]), 0.001);
        
        // Craft 0 manv = (120 + 250)/2 + 40 = 225;
        // Craft 1 manv = (100/1.25 + 200/1.25)/2 + 25 = 145;
        bca[1].getCraft().setHull(Technologies.getHull(TestTechs.FLYER60));
        assertEquals(1.5517, bca[0].getManouverabilityAdvantage(bca[1]), 0.001);
        
    }
    
    /**
     * Tests Craft Destruction and the creation of wreckage
     */
    public void testDestroyCraft() throws Exception {
        Logger.logDebug("*******************************  WarTest.testDestroyCraft() *************************");
        createMapAndThreeCraft(fighterDesign, fighterDesign, fighterDesign, null);

        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[2].setLocation(new Location(550, 450));

        bca[1].getCraft().setDamage(craft2.getMaxDamage()-0.1);
        bca[2].getCraft().setDamage(craft3.getMaxDamage()-0.1);
        
        //
        // Test when craft1 fires at craft2 it is destroyed and wreckage is created.
        //

        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(true, craft2.isDestroyed());
        assertEquals(1, craft2.getOwner().getQtyCrafts());
        assertEquals(1, battle.getWreckages().getWrecks().length);
        WreckageCraft destroyedCraft = (WreckageCraft) battle.getWreckages().getWrecks()[0];
        assertEquals(bca[1], destroyedCraft.getBattleCraft());

        //
        // Test when craft1 fires at craft3 it is destroyed and more wreckage is created.
        //

        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[2]);
        
        assertEquals(true, craft3.isDestroyed());
        assertEquals(0, craft3.getOwner().getQtyCrafts());
        assertEquals(2, battle.getWreckages().getWrecks().length);
        destroyedCraft = (WreckageCraft) battle.getWreckages().getWrecks()[1];
        assertEquals(bca[2], destroyedCraft.getBattleCraft());

    }

    /**
     * Tests crafts attacking a Other crafts when there are multiple weapons, shields and computers
     * on the crafts.
     */
    public void testMultiples() throws Exception {
        Logger.logDebug("*******************************  WarTest.testMultiples() *************************");
        createMap(null);

		CraftDesign multipleDesign = new CraftDesign("TestMultiple", 
				tech_flyer60,
				tech_fusion25,
				tech_electronic25, tech_electronic40,
				tech_energy10, tech_energy20,
				tech_massDriver2, tech_massDriver2, tech_massDriver2, tech_fusionBeam4,
				tech_shieldModulatorI, tech_eneryAmplifierI, tech_armour25);

		Craft craft1 = newCraft(multipleDesign, player1, ss2);
        Craft craft2 = newCraft(defenderDesign, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bca = battle.getCrafts();
        bca[0].setLocation(new Location(100, 500));
        bca[1].setLocation(new Location(500, 500));
        
        {
            WeaponInstance[] wi = bca[0].getWeapons();
            assertEquals("Has correct weapons", tech_massDriver2, wi[0].getWeapon());
            assertEquals("Has correct weapons", tech_fusionBeam4, wi[1].getWeapon());
            assertEquals("Has correct qty weapons", 3, wi[0].getQty());
            assertEquals("Has correct qty weapons", 1, wi[1].getQty());
            
            // Has ShieldModulatorI and EnergyAmplifierI which effect range, damage etc.
            //assertEquals("Has correct damage", 2.2, wi[1].getDamage(), 0.0);
            //assertEquals("Has correct damage", 4.4, wi[0].getWeapon().getDamage(), 0.0);
            //assertEquals("Has correct range", 330, wi[1].getWeapon().getRange());
            //assertEquals("Has correct range", 220, wi[0].getWeapon().getRange());
            
            // Has Armour25
            assertEquals("Has correct hit points", 50, bca[0].getCraft().getMaxDamage());
            
            // Has Computers Electronic25 and Electronic40
            assertEquals("Has correct manouverability", 185, bca[0].getManoeuvrability());
            assertEquals("Has correct manouverability", 145, bca[1].getManoeuvrability());
            assertEquals("craft1 has manv adv over craft2", 1.27586, ((double)bca[0].getManoeuvrability())/((double)bca[1].getManoeuvrability()), 0.0001);
        }
        
        // Test craft1 attacks craft2 3XMassDriver2 in range but not 1XFusionBean4.
        {
            qtyFired = 0;
            bca[0].getNextOrders();
            assertTrue("Craft1 moved towards craft2", Comp.equal(320, bca[0].getLocation().getDistanceTo(bca[1].getLocation()), 10));
            assertTrue("Craft1 is in weapons range", bca[0].inWeaponRange(bca[1].getLocation()));
            assertFalse("MassDriver2 has fired", bca[0].getWeapons()[1].hasWeaponsToFire());
            assertEquals("MassDriver2 has fired but not FusionBeam4", 1, qtyFired);
            // Craft1 has 3XMassDriver2 = damage 6 EnergyAmplifier increases damage to 6.6.
            // Craft1 has manv adv of 1.27586 => damage = 8.4207
            // Craft1 has shield Energy10 = block 1X3 => damage = 5.4207 and power lost is 8.4207
            ShieldInstance[] shields = bca[1].getShields();
            assertEquals("Craft1 inflicted correct damage on craft2", 5.4207, craft2.getDamage(), 0.01);
            assertEquals("Craft2 shields have lost power", 1.5793, shields[0].getPower(), 0.01);
        }

        // Test craft2 attacks craft1 3XMassDriver1 when in range
        // Test that power loss on shields is spread evenly.
        {
            qtyFired = 0;
            craft2.setDamage(0.0);
            bca[1].getNextOrders();
            assertTrue("Craft2 moved towards craft1", Comp.equal(240, bca[1].getLocation().getDistanceTo(bca[0].getLocation()), 10));
            assertFalse("Craft1 is out of weapons range", bca[1].inWeaponRange(bca[0].getLocation()));
            assertEquals("MassDriver1 has not fired", 0, qtyFired);
            
            bca[1].startTurn();
            bca[1].getNextOrders();
            assertTrue("Craft2 moved towards craft1", Comp.equal(160, bca[1].getLocation().getDistanceTo(bca[0].getLocation()), 10));
            assertTrue("Craft1 is in weapons range", bca[1].inWeaponRange(bca[0].getLocation()));
            assertFalse("MassDriver1 has fired", bca[1].getWeapons()[0].hasWeaponsToFire());
            // Craft1 has 3XMassDriver1 => damage = 6.
            // Craft1 has ShieldModulatorI 0.1 => damage = 5.4
            // Craft1 has manv adv of 0.7838 => damage = 4.2325
            // Craft2 has shield Energy10 block 1X3 => damage = 1.2325 and power lost is 2.116 (half becuase we have 2 shields)
            // Craft2 has shield Energy20 block 2X3 => damage = 0.0 and power lost is 0.616 (half becuase we have 2 shields)
            ShieldInstance[] shields = bca[0].getShields();
            assertEquals("Craft1 inflicted correct damage on craft2", 0.0, craft1.getDamage(), 0.01);
            assertEquals("Craft1 Energy20 shields have not lost power", 19.384, shields[1].getPower(), 0.01);
        }
    }
    
    /**
     * Tests crafts attacking a planet and planet attacking crafts.
     */
    public void testPlanetAttack() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlanetAttack() *************************");
        createMap(null);

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player1, ss2);
        Craft craft3 = newCraft(fighterDesign, player1, ss2);

        planet2.setShields(new Shields(tech_energy10));
        planet2.setWeapons(new Weapons(tech_massDriver1));

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, true);
        battle.join(planet2);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bp = battle.getPlanet();
        assertEquals("Planet located on left", bp.getLocation().getX(), 100);
        assertEquals("Planet located in center left", bp.getLocation().getY(), 500);
        
        bca = battle.getCrafts();
        bca[0].setLocation(new Location(490, 450));
        bca[1].setLocation(new Location(490, 500));
        bca[2].setLocation(new Location(490, 550));
        
        assertEquals("Planet does not have missile guidance unit", null, bp.getSpecials());
        
        // Test craft1 moves toward planet and is not in weapons range
        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("craft1 is first to move", bca[0], craft);
            craft.getNextOrders();
            assertTrue("Craft moved towards planet", Comp.equal(290, craft.getLocation().getDistanceTo(bp.getLocation()), 9));
            assertTrue("Craft is out of weapons range", !craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is out of weapons range", !fired);
        }

        // Test craft2 moves toward planet and is not in weapons range
        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Attacking craft2 is next to move", bca[1], craft);
            craft.getNextOrders();
            assertTrue("Craft is out of weapons range", !craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is out of weapons range", !fired);
        }

        // Test craft3 moves toward planet and is not in weapons range
        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Attacking craft3 is next to move", bca[2], craft);
            craft.getNextOrders();
            assertTrue("Craft is out of weapons range", !craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is out of weapons range", !fired);
        }

        assertEquals("Planet weapon does twice as much damage", 4.0, bp.getWeapons()[0].getDamage(), 0.01);
        assertEquals("Planet weapon is actually 4 weapons", 2, bp.getWeapons()[0].getQty());
        //bp.getWeapons()[0].setQty(2);

        // Test planet moves at end of turn and is in weapons range
        {
            BattleObject nextToMove = battle.getNextToMove(false);
            assertEquals("Planet moves next", bp, nextToMove);
            assertEquals("Turn has not ended", 1, battle.getTurn());
            assertTrue("Planet is in weapons range", bp.inWeaponRange(bca[1].getLocation()));
            fired = false;
            bp.getNextOrders();
            assertTrue("Planet has fired weapons", fired);
            // Planet has MassDriver1 which has variance 0, degredation 0, damage 4, range 200.
            // MassDriver on a planet is range is 400 and damage is 4.
            // Craft 2 has Energy10 shield which blocks 1 => Damage = 3
            // Shield power after attack is 6
            // This all happened two times - once for each weapon.
            assertEquals("Planet inflicted correct damage on craft2", 6, craft2.getDamage(), 0.001);
            assertEquals("Craft2 shields have lost power", 2, bca[1].getShieldPower(), 0.001);
        }

        // Add cloak ability to player2 and see crafts and check that crafts within range of a scanner
        // are still visible and those outside the range of a scanner are not visible. Test that when a
        // craft is not visible weapons miss sometimes.
        {
            assertEquals("craft2 is not cloaked", true, bca[2].isVisible(player2));
            assertEquals("Player has no cloak", 0, player1.getPlayerCharacter().getCloakLevel());
            player1.getPlayerCharacter().setCloakLevel(100);
            assertEquals("craft2 is cloaked", false, bca[2].isVisible(player2));
            ShieldInstance[] shields = bca[2].getShields();

            craft3.setDamage(0);
            shields[0].setPower(10);
            Utils.setNextRandomProbability(0.7); // 0.5 chance to miss
            bp.rechargeWeapons();
            bp.fireAllWeapons(bca[2]);
            assertEquals("Hit", 6, bca[2].getDamage(), 0.001);
            
            craft3.setDamage(0);
            shields[0].setPower(10);
            Utils.setNextRandomProbability(0.4); // 0.5 chance to miss
            bp.rechargeWeapons();
            bp.fireAllWeapons(bca[2]);
            assertEquals("Miss", 0, bca[2].getDamage(), 0.001);
            
            player1.getPlayerCharacter().setCloakLevel(0);
            bp.rechargeWeapons();
        }
        
        assertEquals("Have not started a new turn", 1, battle.getTurn());
        bp.dischargeWeapons();
        battle.getNextToMove(false);
        assertEquals("Have started a new turn", 2, battle.getTurn());
        assertEquals("Planet's shields have recharged", 3, bca[1].getShieldPower(), 0.001);
        
        // Test craft1 moves toward planet and is in weapons range
        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("craft1 is first to move", bca[0], craft);
            craft.getNextOrders();
            assertTrue("Craft moved towards planet", Comp.equal(190, craft.getLocation().getDistanceTo(bp.getLocation()), 9));
            assertTrue("Craft is out of weapons range", craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is out of weapons range", fired);
            assertEquals("Craft inflicted correct damage on Planet", 1.0, bp.getDamage(), 0.001);
            assertEquals("Planet shields have lost power", 8.0, bp.getShieldPower(), 0.001);
        }

        // Test craft2 moves toward planet and is in weapons range
        {
            fired = false;
            craft2.setDamage(0.0);
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Attacking craft2 is next to move", bca[1], craft);
            craft.getNextOrders();
            assertTrue("Craft is out of weapons range", craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is out of weapons range", fired);
            assertEquals("Craft inflicted correct damage on Planet", 2.0, bp.getDamage(), 0.001);
            assertEquals("Planet shields have lost power", 6.0, bp.getShieldPower(), 0.0010);
        }

        // Test craft3 moves toward planet and is in weapons range
        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Attacking craft3 is next to move", bca[2], craft);
            craft.getNextOrders();
            assertTrue("Craft is out of weapons range", craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is out of weapons range", fired);
            assertEquals("Craft inflicted correct damage on Planet", 3.0, bp.getDamage(), 0.001);
            assertEquals("Planet shields have lost power", 4.0, bp.getShieldPower(), 0.001);
        }
        
        // Test planet its the planets move again
        {
            BattleObject nextToMove = battle.getNextToMove(false);
            assertEquals("Planet moves next", bp, nextToMove);
            assertEquals("Turn has not ended", 2, battle.getTurn());
            bp.dischargeWeapons();
        }
        
        // SET THE DAMAGE ON THE PLANET SO NEXT WEAPON FIRE SHOULD DESTROY THE WEAPON!
        assertEquals("Weapon in planet has correct build cost", 20.0, (double)(bp.getWeapons()[0].getWeapon().getCostToBuild()), 0.0);
        bp.setDamage(19.5);
        bp.setShieldPower(0, 0.5);

        // Test craft1 moves toward planet and is in weapons range
        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Turn has ended", 3, battle.getTurn());
            assertEquals("craft1 is first to move", bca[0], craft);
            craft.getNextOrders();
            assertTrue("Craft is in weapons range", craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is in weapons range", fired);
            assertEquals("Craft inflicted correct damage on Planet", 0.5, bp.getDamage(), 0.001);
            assertEquals("Planet shields have lost power", 0.0, bp.getShieldPower(), 0.0);
            assertTrue("Planet has had its weapon destroyed", bp.getWeapons() == null || bp.getWeapons().length == 0);
            assertTrue("Planet has had its weapon destroyed", bp.getPlanet().getWeapons() == null || bp.getPlanet().getWeapons().getQuantity() == 0);
        }

        // SET THE DAMAGE ON THE PLANET SO NEXT WEAPON FIRE SHOULD DESTROY THE SHIELDS!
        // REMAINING DAMAGE WILL WIPE OUT POPULATION - So Damage is 0.0
        assertEquals("Shield in planet has correct build cost", 150.0, (double)(bp.getShields()[0].getShield().getCostToBuild()), 0.0);
        bp.setDamage(149.5);
        bp.getPlanet().setPopulation(10.0);
        bp.getPlanet().setQtyFactories(10.0);

        // Test craft2 moves toward planet and is in weapons range
        {
            fired = false;
            craft2.setDamage(0.0);
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Attacking craft2 is next to move", bca[1], craft);
            craft.fireAllWeapons(bp);
            craft.endTurn();
            assertTrue("Craft is out of weapons range", craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is out of weapons range", fired);
            assertEquals("Craft inflicted correct damage on Planet", 0.0, bp.getDamage(), 0.0);
            assertEquals("Planet shields have no power", 0.0, bp.getShieldPower(), 0.0);
            assertTrue("Planet has had its weapon destroyed", bp.getShields() == null || bp.getShields().length == 0);
            assertTrue("Planet has had its weapon destroyed", bp.getPlanet().getShields() == null || bp.getPlanet().getShields().getQuantity() == 0);
            assertTrue("Planet has lost population", bp.getPlanet().getPopulation() < 10.0);
            assertTrue("Planet has lost factories", bp.getPlanet().getQtyFactories() < 10.0);
        }

        // CHECK THE PLANET POPULATION IS EXPECTED VALUE SO WE CAN TEST POPULATION AND FACTORIES HAVE BEEN LOST.
        bp.setDamage(0.0);
        bp.getPlanet().setPopulation(10.0);
        bp.getPlanet().setQtyGroundDefense(10);
        bp.getPlanet().setQtyFactories(10.0);

        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Attacking craft2 is next to move", bca[2], craft);
            craft.moveTo(bp);
            craft.fireAllWeapons(bp);
            assertTrue("Craft is in weapons range", craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is in weapons range", fired);
            // POPULATION_DESTROYED_PER_DAMAGE = 0.05
            // FACTORIES_DESTROYED_PER_DAMAGE = 0.05
            assertEquals("Planet has lost population", 9.9, bp.getPlanet().getPopulation(), 0.03);
            assertEquals("Planet has lost marines", 9, bp.getPlanet().getQtyGroundDefense());
            assertEquals("Planet has lost factories", 9.9, bp.getPlanet().getQtyFactories(), 0.03);
        }

        // TEST COLONY IS DESTROYED WHEN POPULATION HAS BEEN KILLED
        bp.setDamage(0.0);
        bp.getPlanet().setPopulation(0.001);
        bca[2].startTurn();

        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Attacking craft2 is next to move", bca[2], craft);
            craft.moveTo(bp);
            craft.fireAllWeapons(bp);
            assertTrue("Craft is in weapons range", craft.inWeaponRange(bp.getLocation()));
            assertTrue("Craft is in weapons range", fired);
            // POPULATION_DESTROYED_PER_DAMAGE = 0.05
            // FACTORIES_DESTROYED_PER_DAMAGE = 0.05
            assertEquals("Planet has no population", 0.0, bp.getPlanet().getPopulation(), 0.0);
            assertFalse("Planet is not a colony", bp.isColony());
            assertTrue("Planet has no owner", bp.getPlanet().getOwner() == null);
            assertNull("Craft has no emeny to fight", battle.getClosestEnemyCraft(craft.getOwner(), null, true));
        }

        // EACH CRAFT WILL NOW DETECT THE BATTLE HAS ENDED AND LEAVE
        bca[0].startTurn();
        bca[0].getNextOrders();
        bca[1].startTurn();
        bca[1].getNextOrders();
        bca[2].startTurn();
        bca[2].getNextOrders();
        MotEngine.instance.getProxyManager().broadcastQueue();
            
        assertEquals("TestFighter1", ((BattleCraftLeaveEvent)getEventFromFront(BattleCraftLeaveEvent.class, 0)).getCraft());
        assertEquals("TestFighter2", ((BattleCraftLeaveEvent)getEventFromFront(BattleCraftLeaveEvent.class, 1)).getCraft());
        assertEquals("TestFighter3", ((BattleCraftLeaveEvent)getEventFromFront(BattleCraftLeaveEvent.class, 2)).getCraft());
    }

    /**
     * Tests crafts attacking a planet and planet attacking crafts.
     */
    public void testPlanet() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlanet() *************************");
        createMap(null);

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        planet2.setWeapons(new Weapons(tech_massDriver1));
        planet2.getOwner().addTechnology(tech_missile_guidance_unit);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(planet2);
        battle.setup();

        bp = battle.getPlanet();
        
        assertEquals("Planet has missile guidance unit", true, bp.getSpecials().hasMissileRetarget());
        assertEquals("Planet has missile guidance unit", tech_missile_guidance_unit, bp.getSpecials().getMostAdvancedWithGroup(Special.GROUP_MISSILE_GUIDANCE));
        
        assertEquals(0, battle.getQtyMines());
    }
    
    /**
     * Tests crafts cannot attack another object unless it is pointing at it (when hull weapon
     * bay restricts firing angle).
     */
    public void testWeaponBayAngleFiring() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponBayAngleFiring() *************************");
        createMap(null);

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player2, ss2);
        
        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.setup();

        bca = battle.getCrafts();
        bca[0].setLocation(new Location(500, 500));
        bca[0].setDirection(270);
        bca[1].setLocation(new Location(600, 500));
        
        // Check battle craft have been created containing weapon bay details
        assertEquals(1, bca[0].getWeapons().length);
        assertEquals(1, bca[0].getWeapons().length);
        assertEquals("TestWeaponBayDetails", bca[0].getWeapons()[0].getWeaponBayDetails().getName());
        assertEquals(45, bca[0].getWeapons()[0].getWeaponBayDetails().getMaxFireAngle());
        
        //
        // Cannot fire weapons because craft is pointing in wrong direction
        //
        assertFalse(bca[0].isTargetAngleWithinWeaponFiringAngle(bca[0].getWeapons()[0], bca[1]));
        lastEvent = null;
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(0.0, bca[1].getDamage(), 0.0);
//        assertEquals(null, ((MissTargetEvent)lastEvent).getWeapon());
//        assertEquals(bca[0].getName(), ((MissTargetEvent)lastEvent).getSource().getName());
//        assertEquals(bca[1].getName(), ((MissTargetEvent)lastEvent).getTarget().getName());

        bca[0].setDirection(44);
        assertFalse(bca[0].isTargetAngleWithinWeaponFiringAngle(bca[0].getWeapons()[0], bca[1]));
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(0.0, bca[1].getDamage(), 0.0);

        bca[0].setDirection(136);
        assertFalse(bca[0].isTargetAngleWithinWeaponFiringAngle(bca[0].getWeapons()[0], bca[1]));
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(0.0, bca[1].getDamage(), 0.0);
        
        //
        // Can fire weapon when pointing in the right direction
        //

        bca[0].setDirection(90);
        bca[1].getCraft().setDamage(0.0);
        assertTrue(bca[0].isTargetAngleWithinWeaponFiringAngle(bca[0].getWeapons()[0], bca[1]));
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(1, bca[1].getDamage(), 0.0);

        bca[0].setDirection(46);
        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0.0);
        assertTrue(bca[0].isTargetAngleWithinWeaponFiringAngle(bca[0].getWeapons()[0], bca[1]));
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(1, bca[1].getDamage(), 0.0);

        bca[0].setDirection(134);
        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0.0);
        assertTrue(bca[0].isTargetAngleWithinWeaponFiringAngle(bca[0].getWeapons()[0], bca[1]));
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(1, bca[1].getDamage(), 0.0);
    }

    /**
     * Tests crafts receive more damage when struck by a beam weapon from behind.
     */
    public void testBeamWeaponAngleEffectsDamage() throws Exception {
        Logger.logDebug("*******************************  WarTest.testBeamWeaponAngleEffectsDamage() *************************");
        createMap(null);

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player2, ss2);
        
        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.setup();

        bca = battle.getCrafts();
        bca[0].setLocation(new Location(500, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(550, 500));
        bca[1].setDirection(270);
        
        bca[1].setShieldPower(0, 0);
        
        //
        // Test weapon damage head on
        //
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], null, -1);
        assertEquals(2.0, bca[1].getDamage(), 0.0);
        
        //
        // Test weapon from behind
        //
        bca[1].setDirection(90);
        bca[1].getCraft().setDamage(0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], null, -1);
        assertEquals(4.0, bca[1].getDamage(), 0.0);
        
        
        //
        // Test weapon from left side
        //
        bca[1].setDirection(0);
        bca[1].getCraft().setDamage(0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], null, -1);
        assertEquals(3.0, bca[1].getDamage(), 0.0);
        
        
        //
        // Test weapon from right side
        //
        bca[1].setDirection(180);
        bca[1].getCraft().setDamage(0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], null, -1);
        assertEquals(3.0, bca[1].getDamage(), 0.0);
        
        
        
        
        bca[0].setLocation(new Location(400, 400));
        bca[0].setDirection(180);
        bca[1].setLocation(new Location(400, 450));
        bca[1].setDirection(0);
        
        //
        // Test weapon damage head on
        //
        bca[1].getCraft().setDamage(0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], null, -1);
        assertEquals(2.0, bca[1].getDamage(), 0.0);
        
        //
        // Test weapon from behind
        //
        bca[1].setDirection(180);
        bca[1].getCraft().setDamage(0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], null, -1);
        assertEquals(4.0, bca[1].getDamage(), 0.0);
        
        
        //
        // Test weapon from left side
        //
        bca[1].setDirection(90);
        bca[1].getCraft().setDamage(0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], null, -1);
        assertEquals(3.0, bca[1].getDamage(), 0.0);
        
        
        //
        // Test weapon from right side
        //
        bca[1].setDirection(270);
        bca[1].getCraft().setDamage(0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], null, -1);
        assertEquals(3.0, bca[1].getDamage(), 0.0);
        
    }
    
    /**
     * Tests that crafts do receive fear events from psi AND can surrender to psi attack.
     */
    public void testPsi() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPsi() *************************");
        createMapAndTwoCraft(fighterDesign, 200, defenderDesign, 300, null);
        
        Biotechnology psi10 = Technologies.getBiotechnology(TestTechs.PSI10);
        Biotechnology psi20 = Technologies.getBiotechnology(TestTechs.PSI20);
        
        player2.addTechnology(psi10);
        
        assertEquals("Psi Rating is correct", 0, bca[0].getPsiLevel());
        assertEquals("Psi Rating is correct", 10, bca[1].getPsiLevel());

        // Add psionic ability to player1 and see that crafts do start missing turns!
        // Psi advantage is 10 so 10 percent miss turn.
        {
            qtyFearEvents = 0;
            ShieldInstance[] shields = bca[1].getShields();
            craft2.setDamage(0);
            shields[0].setPower(10);
            bca[0].rechargeWeapons();
            Utils.setNextRandomInt(9); // value less than 10 => miss turn
            bca[0].fireAllWeapons(bca[1]);
            assertEquals("Received fear event", 1, qtyFearEvents);
            assertEquals("No damge", 0, bca[1].getDamage(), 0.001);
            bca[0].rechargeWeapons();
            Utils.setNextRandomInt(11); // value less than 10 => miss turn
            bca[0].fireAllWeapons(bca[1]);
            assertEquals("Did not receive fear event", 1, qtyFearEvents);
            assertEquals("Damge done", 1.4139, bca[1].getDamage(), 0.001);
        }
        
        player2.getBiotechnologies().remove(psi10);
        player1.addTechnology(psi10);
        player1.addTechnology(psi20);
        player1.getPlayerCharacter().setPsiRating(20);
        
        assertEquals("Psi Rating is correct", 40, bca[0].getPsiLevel());
        assertEquals("Psi Rating is correct", 0, bca[1].getPsiLevel());

        // Add psi techs and spi bonuses to player2 and see that crafts start surrendering!
        // Player2 has a psi advantage of 40.
        {
            qtySurrenderEvents = 0;
            ShieldInstance[] shields = bca[1].getShields();
            craft2.setDamage(0);
            shields[0].setPower(10);
            bca[0].rechargeWeapons();
            Utils.setNextRandomInt(4); // value less than 5 => surrender
            bca[0].fireAllWeapons(bca[1]);
            assertEquals("Received surrender event", 1, qtySurrenderEvents);
            assertEquals("changed ownership", 2, player1.getCrafts().length);
            assertEquals("changed ownership", 0, player2.getCrafts().length);
            bca[1].getCraft().changeOwner(player2);
            bca[1].getCraft().setOriginReference(null);
        }
        
        player1.getBiotechnologies().remove(psi10);
        player1.getBiotechnologies().remove(psi20);
        player1.getPlayerCharacter().setPsiRating(0);
    }
        
    /**
     * Tests that the weapon inflict the right damage and the shields block damage until their power 
     * is exhausted.
     */
    public void testCloak() throws Exception {
        Logger.logDebug("*******************************  WarTest.testCloak() *************************");
        createMapAndTwoCraft(fighterDesign, defenderDesign, null);

        // Add cloak ability to player2 and check craft2 becomes visible in the correct distance.
        // When craft2 has a cloak it is visible if : scan strength - distance > cloak
        {
            Location originalLoc = bca[1].getLocation();
            Location l = bca[0].getLocation();
            
            player2.getPlayerCharacter().setCloakLevel(100);
            bca[1].setLocation(new Location(l.getX()+50, l.getY()));
            assertEquals("Correct distance apart", 50, bca[0].getLocation().getDistanceTo(bca[1].getLocation()));
            assertEquals("Correct scan strength", 0, battle.getScannerStrengthAt(player1, bca[1].getLocation()));
            assertEquals("Correct scan strength", 100, bca[1].getCloakLevel());
            assertEquals("Has correct visibility", false, bca[1].isVisible(player1));
            
            Scanner photon100 = Technologies.getScanner(TestTechs.PHOTON100);
            bca[0].getCraft().setScanners(new Scanners(photon100));
            assertEquals("Correct scan strength", 50, battle.getScannerStrengthAt(player1, bca[1].getLocation()));
            assertEquals("Has correct visibility", false, bca[1].isVisible(player1));
            
            Scanner photon200 = Technologies.getScanner(TestTechs.PHOTON200);
            bca[0].getCraft().setScanners(new Scanners(photon200));
            assertEquals("Correct scan strength", 150, battle.getScannerStrengthAt(player1, bca[1].getLocation()));
            assertEquals("Has correct visibility", true, bca[1].isVisible(player1));
            
            player2.getPlayerCharacter().setCloakLevel(0);
            bca[0].getCraft().setScanners(null);
        }
        
        // Add cloak ability to player2 and see crafts and check that crafts within range of a scanner
        // are still visible and those outside the range of a scanner are not visible. Test that when a
        // craft is not visible weapons miss sometimes.
        {
            assertEquals("craft2 is not cloaked", true, bca[1].isVisible(player1));
            assertEquals("Player has no cloak", 0, player2.getPlayerCharacter().getCloakLevel());
            player2.getPlayerCharacter().setCloakLevel(100);
            assertEquals("craft2 is cloaked", false, bca[1].isVisible(player1));
            ShieldInstance[] shields = bca[1].getShields();
            qtyMissedEvents = 0;
            bca[1].setLocation(new Location(200,500));
            
            craft2.setDamage(0);
            shields[0].setPower(10);
            bca[0].rechargeWeapons();
            Utils.setNextRandomProbability(0.4); // Value < 0.5 => miss target
            bca[0].fireAllWeapons(bca[1]);
            
            assertEquals("Weapons missed target", 1, qtyMissedEvents);
            player2.getPlayerCharacter().setCloakLevel(0);
        }
        
    }

    /**
     * Tests that the condition of a craft effects its performance in battle.
     */
    public void testCondition() throws Exception {
        Logger.logDebug("*******************************  WarTest.testCondition() *************************");
        createMapAndTwoCraft(fighterDesign, 300, fighterDesign, 450, null);

        bca[0].getCraft().setBuildQuality(0.7);
        assertEquals(1.0, bca[1].getCraft().getBuildQuality());
        
        assertEquals(92, bca[0].getSpeed());
        assertEquals(185, bca[0].getTurningSpeed());
        assertEquals(163, bca[0].getManoeuvrability());
        assertEquals(100, bca[1].getSpeed());
        assertEquals(200, bca[1].getTurningSpeed());
        assertEquals(175, bca[1].getManoeuvrability());
        
        bca[0].moveTo(bca[1]);
        
        assertEquals(392,500, bca[0].getLocation());
        
        bca[1].getShields()[0].setPower(0.0);
        WeaponInstance weapon = bca[1].getWeapon(tech_massDriver1);
		bca[0].fireWeapon(bca[1], weapon, 1);
        
        assertEquals(1.58, bca[1].getDamage(),0.1);
    }

                
    public void testMissiles() throws Exception {
        Logger.logDebug("*******************************  WarTest.testMissiles() *************************");
        createMap(null); 

        CraftDesign craftDesign = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_massDriver1, tech_missile6);
        Craft craft1 = newCraft(craftDesign, player1, ss2);

        planet2.setWeapons(new Weapons(tech_massDriver1, tech_missile6, tech_missile6));
        planet2.setOrbital(new Orbital(tech_battleStationI, null, null));

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(planet2);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bp = battle.getPlanet();
        bp.setLocation(new Location(100, 500));
        
        bca = battle.getCrafts();
        BattleCraft bc = bca[1];
        BattleStation bs = (BattleStation) bca[0];
        
        bc.setLocation(new Location(600, 500));
        
        // Test craft1 cannot fire its missiles at the planet or battle station
        {
            qtyMissilesLaunched = 0;
            bc.fireAllWeapons(bp);
            assertTrue("Craft is out of weapons range", !bc.inWeaponRange(bp.getLocation()));
            assertEquals("Craft is out of weapons range", 0, qtyMissilesLaunched);
            bc.fireAllWeapons(bs);
            assertTrue("Craft is out of weapons range", !bc.inWeaponRange(bs.getLocation()));
            assertEquals("Craft is out of weapons range", 0, qtyMissilesLaunched);
        }

        bc.setLocation(new Location(550, 500));
        
        // Test the planet can fire its missiles at the craft and its missile take two moves to strike.
        // Test its mass driver is out of range
        {
            qtyMissilesLaunched = 0;
            qtyFired = 0;
            bp.fireAllWeapons(bc);
            assertTrue("Craft is in weapons range", bp.inWeaponRange(bc.getLocation()));
            assertEquals("Craft is in weapons range", 0, qtyFired);
            assertEquals("Craft is in weapons range", 1, qtyMissilesLaunched);
            Automaton [] missiles = battle.getAutomatons();
            assertEquals("Battle now has a missile", 1, missiles.length);
            Missiles missile = (Missiles) missiles[0];
            assertEquals("Missiles are correct", bc, missile.getTarget());
            assertEquals("Missiles are correct", 250, missile.getSpeed());
            assertEquals("Missiles are correct", bp, missile.getOrigin());
            assertEquals("Missiles are correct", player2, missile.getOwner());
            assertEquals("Missiles are correct", 2, missile.getWeapons()[0].getQty());
            assertEquals("Missiles are correct", tech_missile6, missile.getWeapons()[0].getWeapon());
            assertEquals("Missiles are correct", 450, missile.getLocation().getDistanceTo(missile.getTarget().getLocation()));
            bc.moveLinkedObjects(1.0, null, null);
            missiles = battle.getAutomatons();
            assertEquals("Missile has not struck target", 1, missiles.length);
            assertEquals("Craft 1 has not been attacked", 0.0, craft1.getDamage(), 0.001);
            missile.startTurn();
            bc.moveLinkedObjects(1.0, null, null);
            missiles = battle.getAutomatons();
            assertEquals("Craft 1 has been attacked", null, missiles);
            assertEquals("Craft 1 has been attacked", 10.0, craft1.getDamage(), 0.001);
            assertEquals("Craft 1 has been attacked", 0.0, bc.getShieldPower(), 0.001);
        }

        bc.setLocation(new Location(325, 500));
        craft1.setDamage(0);
        bc.setShieldPower(0, 10.0);
        bp.startTurn();
        
        // Test the planet fires its missiles at the craft and its missile take one move to strike.
        // The mass driver and the other missile are not fired.
        {
            qtyMissilesLaunched = 0;
            qtyFired = 0;
            bp.fireWeapons(bc, bp.getWeapons()[1], 2);
            assertTrue("Craft is in weapons range", bp.inWeaponRange(bc.getLocation()));
            assertEquals("Craft is in weapons range", 0, qtyFired);
            assertEquals("Craft is in weapons range", 1, qtyMissilesLaunched);
            Automaton [] missiles = battle.getAutomatons();
            assertEquals("Battle now has a missile", 1, missiles.length);
            Missiles missile = (Missiles) missiles[0];
            assertEquals("Missiles are correct", bc, missile.getTarget());
            assertEquals("Missiles are correct", 250, missile.getSpeed());
            assertEquals("Missiles are correct", bp, missile.getOrigin());
            assertEquals("Missiles are correct", player2, missile.getOwner());
            assertEquals("Missiles are correct", 2, missile.getWeapons()[0].getQty());
            assertEquals("Missiles are correct", tech_missile6, missile.getWeapons()[0].getWeapon());
            assertEquals("Missiles are correct", 225, missile.getLocation().getDistanceTo(missile.getTarget().getLocation()));
            bc.moveLinkedObjects(1.0, null, null);
            missiles = battle.getAutomatons();
            assertEquals("Craft 1 has been attacked", null, missiles);
            assertEquals("Craft 1 has been attacked", 10.0, craft1.getDamage(), 0.001);
            assertEquals("Craft 1 has been attacked", 0.0, bc.getShieldPower(), 0.001);
        }

        bc.setLocation(new Location(300, 500));

        // Test the craft can fire its missiles at the planet but not its mass driver.
        {
            qtyMissilesLaunched = 0;
            qtyFired = 0;
            bc.fireWeapons(bp, bc.getWeapons()[1], 1);
            assertTrue("Craft in of weapons range", bc.inWeaponRange(bp.getLocation()));
            assertEquals("Craft in of weapons range", 1, qtyMissilesLaunched);
            assertEquals("Craft is in weapons range", 0, qtyFired);
            Automaton [] missiles = battle.getAutomatons();
            assertEquals("Battle now has a missile", 1, missiles.length);
            Missiles missile = (Missiles) missiles[0];
            assertEquals("Missiles are correct", bp, missile.getTarget());
            assertEquals("Missiles are correct", bc, missile.getOrigin());
            assertEquals("Missiles are correct", player1, missile.getOwner());
            assertEquals("Missiles are correct", 1, missile.getWeapons()[0].getQty());
            assertEquals("Missiles are correct", tech_missile6, missile.getWeapons()[0].getWeapon());
            assertEquals("Missiles are correct", 200, missile.getLocation().getDistanceTo(missile.getTarget().getLocation()));
            battle.endTurn();
            missiles = battle.getAutomatons();
            assertEquals("Planet has been attacked", null, missiles);
            assertEquals("Planet has been attacked", 6.0, bp.getDamage(), 0.001);
        }

        // Test the craft can fire its missiles at the battle station.
        {
            qtyMissilesLaunched = 0;
            qtyFired = 0;
            bc.fireWeapons(bs, bc.getWeapons()[1], 1);
            assertTrue("Craft in of weapons range", bc.inWeaponRange(bp.getLocation()));
            assertEquals("Craft in of weapons range", 1, qtyMissilesLaunched);
            assertEquals("Craft is in weapons range", 0, qtyFired);
            Automaton [] missiles = battle.getAutomatons();
            assertEquals("Battle now has a missile", 1, missiles.length);
            Missiles missile = (Missiles) missiles[0];
            assertEquals("Missiles are correct", bs, missile.getTarget());
            assertEquals("Missiles are correct", bc, missile.getOrigin());
            assertEquals("Missiles are correct", player1, missile.getOwner());
            assertEquals("Missiles are correct", 1, missile.getWeapons()[0].getQty());
            assertEquals("Missiles are correct", tech_missile6, missile.getWeapons()[0].getWeapon());
            assertEquals("Missiles are correct", 100, missile.getLocation().getDistanceTo(missile.getTarget().getLocation()));
            battle.endTurn();
            missiles = battle.getAutomatons();
            assertEquals("Station has been attacked", null, missiles);
            assertEquals("Station has been attacked", 6.0, bs.getDamage(), 0.001);
        }

        // Test that a craft can fire a limited quantity of missiles but a planet has no limit to the 
        // quantity of missiles it can fire.
        {
            assertEquals("Missile has correct qty shots", 5, tech_missile6.getQtyShots().intValue());
            qtyMissilesLaunched = 0;
            qtyFired = 0;
            assertEquals("Missile has correct record or shots fired", 2, bc.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 0, qtyMissilesLaunched);
            bc.fireAllWeapons(bp);
            assertEquals("Missile has correct record or shots fired", 3, bc.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 1, qtyMissilesLaunched);
            bc.fireAllWeapons(bp);
            assertEquals("Missile has correct record or shots fired", 3, bc.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 1, qtyMissilesLaunched);
            bc.rechargeWeapons();
            bc.fireAllWeapons(bp);
            assertEquals("Missile has correct record or shots fired", 4, bc.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 2, qtyMissilesLaunched);
            bc.rechargeWeapons();
            bc.fireAllWeapons(bp);
            assertEquals("Missile has correct record or shots fired", 5, bc.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 3, qtyMissilesLaunched);
            bc.rechargeWeapons();
            bc.fireAllWeapons(bp);
            assertEquals("Missile has correct record or shots fired", 5, bc.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 3, qtyMissilesLaunched);
            bc.rechargeWeapons();
            bc.fireAllWeapons(bp);
            assertEquals("Missile has correct record or shots fired", 5, bc.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 3, qtyMissilesLaunched);
            bp.rechargeWeapons();
            bp.fireAllWeapons(bc);
            assertEquals("Missile has correct record or shots fired", 6, bp.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 4, qtyMissilesLaunched);
            bp.getWeapons()[1].setQtyShots(8);
            bp.rechargeWeapons();
            bc.getCraft().setDamage(0.0);
            bp.fireAllWeapons(bc);
            assertEquals("Missile has correct record or shots fired", 10, bp.getWeapons()[1].getQtyShots());
            assertEquals("Missile has correct record or shots fired", 5, qtyMissilesLaunched);
        }

    }

    public void testCraftStopsOnMissileStrike() throws Exception {
        Logger.logDebug("*******************************  WarTest.testCraftStopsOnMissileStrike() *************************");
        createMap(null); 

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        planet2.setWeapons(new Weapons(tech_missile6));

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(planet2);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bp = battle.getPlanet();
        bp.setLocation(new Location(100, 500));
        
        bca = battle.getCrafts();
        BattleCraft bc = bca[0];
        bc.setLocation(new Location(800, 500));
        
        bp.fireAllWeapons(bc);
        
        Automaton [] missiles = battle.getAutomatons();
        assertEquals("Battle now has a missile", 1, missiles.length);
        Missiles missile = (Missiles) missiles[0];
        
        assertEquals(100, bca[0].getSpeed());
        assertEquals(bc, missile.getTarget());
        assertEquals(250, missile.getSpeed());
        assertEquals(bp, missile.getOrigin());
        
        //
        // TEST CALCULATE PERCENT MOVE BEFORE MISSILE STRIKES PROPERLY
        //
        
        missile.setLocation(new Location(100,500));
        assertEquals(2.0, missile.getPercentMoveBeforeStrike());
        
        missile.setLocation(new Location(450,500));
        assertEquals(1.0, missile.getPercentMoveBeforeStrike());

        missile.setLocation(new Location(625,500));
        assertEquals(0.5, missile.getPercentMoveBeforeStrike());

        missile.setLocation(new Location(800,750));
        assertEquals(0.9, missile.getPercentMoveBeforeStrike(), 0.1);

        missile.setLocation(new Location(800,250));
        assertEquals(0.9, missile.getPercentMoveBeforeStrike(), 0.1);

        bc.setLocation(new Location(500, 500));
        missile.setLocation(new Location(800,500));
        assertEquals(2.0, missile.getPercentMoveBeforeStrike(), 0.1);

        bc.setLocation(new Location(500, 500));
        missile.setLocation(new Location(600,500));
        assertEquals(0.66, missile.getPercentMoveBeforeStrike(), 0.1);

        //
        // TEST CRAFT STOPS AND MISSILE STRIKES EVEN THOUGH CRAFT HAS MOVE REMAINING
        //
        
        missile.setLocation(new Location(675,500));
        missile.startTurn();
        
        bc.setDirection(90);
        bc.setLocation(new Location(500, 500));
        bc._setMoveRemaining(1.0);
        assertEquals(0.5, missile.getPercentMoveBeforeStrike(), 0.1);
        
        bc._moveTo(new Location(550,500));
        
        assertEquals(550,500, bca[0].getLocation());
        assertEquals(5.0, bca[0].getDamage());
        assertEquals(0.5, bca[0].getMoveRemaining());
        assertEquals(550,500, missile.getLocation());
        assertEquals(true, missile.isDestroyed());

    }
    
    /**
     * Tests that missiles can be destroyed.
     */
    public void testMissileDestruction() throws Exception {
        Logger.logDebug("*******************************  WarTest.testMissileDestruction() *************************");
        CraftDesign missileDesign = new CraftDesign("MissileFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_missile6, tech_missile6);
        createMapAndTwoCraft(missileDesign, 400, fighterDesign, 600, null);
        
        bca[0].fireAllWeapons(bca[1]);
        
        Missiles missiles = (Missiles)battle.getAutomatons()[0];

        assertEquals(0.0, missiles.getDamage());
        assertEquals(2, missiles.getWeapons()[0].getQty());
        
        bca[1].fireAllWeapons(missiles);

        assertEquals(2.0, missiles.getDamage());
        assertEquals(2, missiles.getWeapons()[0].getQty());
        assertEquals(16.0, missiles.getDamageToDestroy());

        missiles.setDamage(15);
        bca[1].rechargeWeapons();
        bca[1].fireAllWeapons(missiles);
        
        assertEquals(1.0, missiles.getDamage());
        assertEquals(1, missiles.getWeapons().length);
        assertEquals(1, missiles.getWeapons()[0].getQty());
        
        //
        // test missiles are retargeted when target is destroyed and have retarget special
        //
        
        Specials specials = new Specials();
        bca[0].getCraft().setSpecials(specials);
        specials.add(tech_missile_guidance_unit);
        BattleCraft newCraft = battle.newCraft(player2, fighterDesign, new Location(500,500));

        assertEquals("Can retarget missiles", true, bca[0].getSpecials().hasMissileRetarget());
        assertEquals("Missiles have correct target", bca[1], missiles.getTarget());
        
        bca[1].destroy(null, null, null);
        
        assertEquals("Missiles changed target", newCraft, missiles.getTarget());

        specials.remove(tech_missile_guidance_unit);
        BattleCraft newCraft2 = battle.newCraft(player2, fighterDesign, new Location(400,500));
        newCraft.destroy(null, null, null);

        assertEquals("Can not retarget missiles", false, bca[0].getSpecials().hasMissileRetarget());
        assertEquals("Missiles have been destroyed", null, battle.getAutomatons());
        assertEquals("Missiles have been destroyed", null, battle.getAutomatons());
    }
    
    /**
     * Tests the creation and firing / being damaged of a battle station with no shields or scanners.
     * Tests that a battle station is destroyed correctly
     */
    public void testBattleStation() throws Exception {
        Logger.logDebug("*******************************  WarTest.testBattleStation() *************************");
        createMap(null);

        Hull hull = (Hull) tech_battleStationI.cloneTechnology();
        hull.setSpace(30);
        player2.setScanners(new Scanners());
        
        planet2.setWeapons(new Weapons(tech_massDriver1));
        createBattleStation(hull, planet2);
        Craft craft1 = newCraft(defenderDesign, player1, ss2);
        
        battle = newBattle(ss2);
        battle.join(planet2);
        battle.join(craft1, true);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bp = battle.getPlanet();
        bca = battle.getCrafts();
        assertEquals("Planet located on left", 100, 500, bp.getLocation());
        assertEquals("All have joined battle", hull, bca[0].getCraft().getHull().getName());
        assertEquals("All have joined battle", 11, bca[0].getCraft().getWeaponBays().getAllWeapons().length);
        assertEquals("All have joined battle", craft1, bca[1].getCraft());

        // Move enemy craft - craft moves towards battle station.
        assertEquals("Correct craft moves next", bca[1], battle.getNextToMove(false));
        bca[1].setLocation(new Location(900, 500));
        bca[1].setDirection(270);
        bca[1].getNextOrders();
        assertEquals("Craft moved", 820, 500, bca[1].getLocation());
        assertEquals("Craft is in weapons range", 0, qtyFired);

        // Move my battle station - does nothing because enemy is out of range.
        assertEquals("Correct craft moves next", bca[0], battle.getNextToMove(false));
        bca[0].getNextOrders();
        assertEquals("Craft moved", 200, 500, bca[0].getLocation());
        assertEquals("Craft is in weapons range", 0, qtyFired);
        
        // Test Planet has moves next - but is out of range
        assertEquals("Correct craft moves next", bp, battle.getNextToMove(false));
        bp.getNextOrders();

        // Move enemy craft - craft moves towards battle station and fires at it.
        bca[1].setLocation(new Location(200, 550));
        bca[1].setDirection(90);
        assertEquals("Correct craft moves next", bca[1], battle.getNextToMove(false));
        weaponFireEvent = null;
        bca[1].getNextOrders();
        assertEquals("Craft is in weapons range", bca[0].getName(), weaponFireEvent.getTarget());
        assertEquals("Craft is in weapons range", bca[1].getName(), weaponFireEvent.getSource());
        assertEquals("Craft is in weapons range", 2.0, bca[1].getManouverabilityAdvantage(bca[0]), 0.01);
        qtyFired = 0;
        bca[0].getCraft().setDamage(0.0);
        
        // Move my battle station - fires at craft1.
        assertEquals("Correct craft moves next", bca[0], battle.getNextToMove(false));
        weaponFireEvent = null;
        bca[0].getNextOrders();
        assertEquals("Craft is in weapons range", bca[1].getName(), weaponFireEvent.getTarget());
        assertEquals("Craft is in weapons range", bca[0].getName(), weaponFireEvent.getSource());
        assertEquals("Craft is in weapons range", 0.5, bca[0].getManouverabilityAdvantage(bca[1]), 0.01);
        assertEquals("Craft is in weapons range", 12.0, bca[1].getCraft().getDamage(), 0.01);
        assertEquals("Craft is in weapons range", 0.0, bca[1].getShields()[0].getPower(), 0.01);
        qtyFired = 0;
        bca[1].setLocation(new Location(200, 550));
        bca[1].getCraft().setDamage(0.0);
        bca[1].getShields()[0].setPower(10);
        
        // Test Planet has moved - Note Damage done is 8 not 14 like battle station!
        assertEquals("Correct craft moves next", bp, battle.getNextToMove(false));
        bp.getNextOrders();
        assertEquals("Craft is in weapons range", bca[1].getName(), weaponFireEvent.getTarget());
        assertEquals("Craft is in weapons range", bp.getName(), weaponFireEvent.getSource());
        assertEquals("Craft is in weapons range", 9.0, bca[1].getCraft().getDamage(), 0.01);
        assertEquals("Craft is in weapons range", 2.0, bca[1].getShields()[0].getPower(), 0.01);
        
        // Test enemy craft destroying my battle station
        bca[0].getCraft().setDamage(bca[0].getCraft().getMaxDamage()-2);
        bca[1].rechargeWeapons();
        bca[1].getNextOrders();
        assertEquals("Battle Station destroyed", bca[0].getName(), lastCraftDestroyed);
    }

    /**
     * Tests the creation and firing / being damaged of a battle station with shields and scanners.
     * Tests that crafts can attack a space station and that when the space station is destroyed 
     * the mining station of the owning planet is removed.
     * Tests the CraftAttackPlanet command
     */
    public void testSpaceStation() throws Exception {
        Logger.logDebug("*******************************  WarTest.testSpaceStation() *************************");
        createMap(null);

        SolarSystem ss3 = newSolarSystem("SS3", 250, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 80, 0.8);

        player2.addShield(tech_energy10);
        
        newSpaceStation(planet2, planet3);
        Craft craft1 = newCraft(powerfulDesign, player1, ss3);
        Craft craft2 = newCraft(powerfulDesign, player1, ss3);
        Craft craft3 = newCraft(powerfulDesign, player1, ss3);
        Craft craft4 = newCraft(powerfulDesign, player1, ss3);

        assertEquals(true, planet3.isOwnedBy(player2));
        assertEquals(planet3, planet2.getMiningStation().getPlanet());
        
        new CraftAttackCmd(craft1, planet3, player1).run();
        new CraftAttackCmd(craft2, planet3, player1).run();
        new CraftAttackCmd(craft3, planet3, player1).run();
        new CraftAttackCmd(craft4, planet3, player1).run();
        battle = MotEngine.instance.getBattles().getBattleAt(ss3.getOrbit());
        battle.setup();

        bca = battle.getCrafts();

        assertEquals(5, bca.length);
        assertEquals(null, battle.getPlanet());
        assertEquals(tech_spaceStation, bca[0].getCraft().getHull());
        assertEquals(tech_massDriver1, 21, bca[0].getWeapons());
        assertEquals(tech_energy10, bca[0].getShields());
        assertEquals(tech_photon100, bca[0].getCraft().getScanners());
        
        //
        // Space station should be destroyed because 4 power full craft are more powerful.
        // Test space station is destroyed and that planet no longer belongs to player2
        //
        
        battle.automate();
        
        assertEquals(false, planet3.isOwnedBy(player2));
        assertEquals(null, planet2.getMiningStation());
    }

    public void testSpaceStationWithDefenceModules1() throws Exception {
        Logger.logDebug("*******************************  WarTest.testSpaceStationWithDefenceModules1() *************************");
        createMap(null);

        SolarSystem ss3 = newSolarSystem("SS3", 250, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 80, 0.8);

        player2.addComputer(tech_electronic25);
        player2.addComputer(tech_electronic40);
        player2.addShield(tech_energy10);
        player2.addShield(tech_energy20);
        player2.addScanner(tech_photon100);
        player2.addScanner(tech_photon150);
        player2.addWeapon(tech_fusionBeam4);
        player2.addWeapon(tech_antiMissileBeam);
        player2.addWeapon(tech_retargetBeam);
        
        Orbital spaceStation = newSpaceStation(planet2, planet3, tech_fightersModule, tech_sentinelsModule);
        Craft craft1 = newCraft(fighterDesign, player1, ss3);

        assertEquals(true, planet3.isOwnedBy(player2));
        assertEquals(planet3, planet2.getMiningStation().getPlanet());
        
        new CraftAttackCmd(craft1, planet3, player1).run();
        battle = MotEngine.instance.getBattles().getBattleAt(ss3.getOrbit());
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bca = battle.getCrafts();

        assertEquals(4, bca.length);
        
        assertEquals(null, battle.getPlanet());
        
        assertEquals(player2, bca[0].getOwner());
        assertEquals(200,500, bca[0].getLocation());
        assertEquals(tech_spaceStation, bca[0].getCraft().getHull());
        assertEquals(tech_stationFighterBays, tech_stationFighterBays, bca[0].getCraft().getSpecials());
        assertEquals(tech_fusionBeam4, 14, bca[0].getWeapons());
        assertEquals(true, bca[0].getWeapons()[0].isHeavy());
        assertEquals(tech_energy20, bca[0].getShields());
        assertEquals(tech_photon150, bca[0].getCraft().getScanners());
        assertEquals(tech_electronic40, bca[0].getCraft().getComputers());

        assertEquals(player1, bca[1].getOwner());
        assertEquals(tech_flyer30, bca[1].getCraft().getHull());
        assertEquals(900,500, bca[1].getLocation());

        assertEquals(player2, bca[2].getOwner());
        assertEquals(500,333, bca[2].getLocation());
        assertEquals(90, bca[2].getDirection());
        assertEquals(tech_sentinelHull, bca[2].getCraft().getHull());
        assertEquals(tech_fusionBeam4, 1, tech_antiMissileBeam, 2, bca[2].getWeapons());
        assertEquals(tech_energy20, bca[2].getShields());
        assertEquals(tech_photon150, bca[2].getCraft().getScanners());
        assertEquals(tech_electronic40, bca[2].getCraft().getComputers());

        assertEquals(player2, bca[3].getOwner());
        assertEquals(500,666, bca[3].getLocation());
        assertEquals(90, bca[3].getDirection());
        assertEquals(tech_sentinelHull, bca[3].getCraft().getHull());
        assertEquals(0, bca[3].getCraft().getEngine().getImpulseSpeed());
        assertEquals(180, bca[3].getCraft().getEngine().getTurningSpeed());
        assertEquals(tech_fusionBeam4, 1, tech_antiMissileBeam, 2, bca[3].getWeapons());
        assertEquals(tech_energy20, bca[3].getShields());
        assertEquals(tech_photon150, bca[3].getCraft().getScanners());
        assertEquals(tech_electronic40, bca[3].getCraft().getComputers());

        //
        // Sentinel can not move but can turn to enemy craft and attack 
        //
        
        bca[1].setLocation(new Location(500, 500));
        bca[1].endTurn();
        bca[2].getNextOrders();
        
        assertEquals(500,333, bca[2].getLocation());
        assertEquals(180, bca[2].getDirection());
        assertEquals(false, bca[2].hasWeaponsToFire());
        assertEquals(true, bca[2].hasMoved());
        assertEquals(bca[2].getName(), lastCraftMovementEvent.getName());
        assertEquals(1, lastCraftMovementEvent.getMovements().length);
        assertEquals(180, ((ObjectTurn)lastCraftMovementEvent.getMovements()[0]).getTo());
        assertEquals(bca[2].getName(), weaponFireEvent.getSource());
        assertEquals(bca[1].getName(), weaponFireEvent.getTarget());

        //
        // Can be be instructed using a command 
        //

        new BattleAttackCmd(player2, battle, bca[3], bca[1], true, true, true, null, null, -1, true).run();
        
        assertEquals(500,666, bca[3].getLocation());
        assertEquals(0, bca[3].getDirection());
        assertEquals(false, bca[3].hasWeaponsToFire());
        assertEquals(true, bca[3].hasMoved());
        assertEquals(1, lastCraftMovementEvent.getMovements().length);
        assertEquals(0, ((ObjectTurn)lastCraftMovementEvent.getMovements()[0]).getTo());
        assertEquals(bca[3].getName(), weaponFireEvent.getSource());
        assertEquals(bca[1].getName(), weaponFireEvent.getTarget());

    }
    

    public void testSpaceStationWithDefenceModules2() throws Exception {
        Logger.logDebug("*******************************  WarTest.testSpaceStationWithDefenceModules2() *************************");
        createMap(null);

        SolarSystem ss3 = newSolarSystem("SS3", 250, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 80, 0.8);

        Orbital spaceStation = newSpaceStation(planet2, planet3, tech_fightersModule, tech_sentinelsModule);
        Craft craft1 = newCraft(fighterDesign, player1, ss3);

        assertEquals(true, planet3.isOwnedBy(player2));
        assertEquals(planet3, planet2.getMiningStation().getPlanet());
        
        new CraftAttackCmd(craft1, planet3, player1).run();
        battle = MotEngine.instance.getBattles().getBattleAt(ss3.getOrbit());
        battle.setup();

        bca = battle.getCrafts();

        assertEquals(4, bca.length);
        
        assertEquals(null, battle.getPlanet());
        
        assertEquals(player2, bca[0].getOwner());
        assertEquals(200,500, bca[0].getLocation());
        assertEquals(tech_spaceStation, bca[0].getCraft().getHull());
        assertEquals(tech_stationFighterBays, tech_stationFighterBays, bca[0].getCraft().getSpecials());
        assertEquals(tech_massDriver1, 24, bca[0].getWeapons());
        assertEquals(true, bca[0].getWeapons()[0].isHeavy());
        assertEquals(tech_photon100, bca[0].getCraft().getScanners());
        assertNull(bca[0].getShields());
        assertNull(bca[0].getCraft().getComputers());

        assertEquals(player1, bca[1].getOwner());
        assertEquals(tech_flyer30, bca[1].getCraft().getHull());
        assertEquals(900,500, bca[1].getLocation());

        assertEquals(player2, bca[2].getOwner());
        assertEquals(500,333, bca[2].getLocation());
        assertEquals(tech_sentinelHull, bca[2].getCraft().getHull());
        assertEquals(tech_massDriver1, 2, bca[2].getWeapons());
        assertEquals(tech_photon100, bca[2].getCraft().getScanners());
        assertNull(bca[2].getShields());
        assertNull(bca[2].getCraft().getComputers());

        assertEquals(player2, bca[3].getOwner());
        assertEquals(500,666, bca[3].getLocation());
        assertEquals(tech_sentinelHull, bca[3].getCraft().getHull());
        assertEquals(tech_massDriver1, 2, bca[3].getWeapons());
        assertEquals(tech_photon100, bca[3].getCraft().getScanners());
        assertNull(bca[3].getShields());
        assertNull(bca[3].getCraft().getComputers());
        
        //
        // Space station should win because it has four fighters, sentinels and powerful weapons.
        //
        
        battle.automate();
        
        assertEquals(true, planet3.isOwnedBy(player2));
        assertEquals(spaceStation, planet2.getMiningStation().getOrbital());
    }
    

    
    /**
     * Tests the creation and firing / being damaged of a battle station with shields and scanners.
     * Tests that crafts can attack a space station and that when the space station is destroyed 
     * the mining station of the owning planet is removed.
     * Tests the CraftAttackStellarObject command
     */
    public void testOutpost() throws Exception {
        Logger.logDebug("*******************************  WarTest.testOutpost() *************************");
        createMap(null);

        SolarSystem ss3 = newSolarSystem("SS3", 250, 100);
        Planet planet3 = newPlanet("SS3-A", ss3, null, 80, 0.8);

        player2.addShield(tech_energy10);
        player2.addComputer(tech_electronic25);
        
        newOutpost(planet2, ss3);
        Craft craft1 = newCraft(powerfulDesign, player1, ss3);

        assertEquals(true, ss3.isOwnedBy(player2));
        assertEquals(tech_outpost, ss3.getOrbital().getOrbital().getHullTech());
        
        new CraftAttackCmd(craft1, null, player1).run();
        battle = MotEngine.instance.getBattles().getBattleAt(ss3.getOrbit());
        battle.setup();

        bca = battle.getCrafts();

        assertEquals(2, bca.length);
        assertEquals(null, battle.getPlanet());
        assertEquals(tech_outpost, bca[0].getCraft().getHull());
        assertEquals(2, bca[0].getWeapons()[0].getQty());
        assertEquals(tech_massDriver1, bca[0].getWeapons()[0].getWeapon());
        assertEquals(tech_photon100, bca[0].getCraft().getScanners());
        assertNull(bca[0].getShields());
        assertNull(bca[0].getCraft().getComputers());
        
        //
        // Outpost should be destroyed because a craft TestPowerful is more powerful.
        // Test outpost is destroyed and that solar system no longer belongs to player2
        //
        
        battle.automate();
        
        assertEquals(false, ss3.isOwnedBy(player2));
        assertEquals(null, ss3.getOrbital());
    }
    

    /**
     * Tests one craft attacking another craft that is located in the same solar system BUT is in orbit
     * around a different planet.
     */
    public void testAttackFromDifferentPlanet() throws Exception {
        Logger.logDebug("*******************************  WarTest.testAttackFromDifferentPlanet() *************************");
        createMap(null);
        Planet planet1b = newPlanet("SS1-B", ss1, null);

        Craft craft1 = newCraft(fighterDesign, player1, planet1);
        Craft craft2 = newCraft(fighterDesign, player2, planet1b);
        createBattleStation(tech_battleStationI, planet1);
        planet1.setWeapons(new Weapons(tech_missile6));
        
        craft2.attack(ss1, toArray(craft1));
        
        battle = MotEngine.getInstance().getBattles().getBattles()[0];
        
        assertEquals("Battle is at SS1-A", planet1, battle.getOrbit().getPlanet());
        assertEquals("Planet is in the battle", planet1, battle.getPlanet().getPlanet());
        BattleCraft[] crafts = battle.getCrafts();
		assertEquals("BattleStation is in the battle", "BattleStation1", crafts[0].getName());
		assertEquals("Craft1 is in the battle", craft1.getName(), crafts[2].getName());
		assertEquals("Craft2 is in the battle", craft2.getName(), crafts[1].getName());
    }
    

	/**
     * This tests that the player is getting the correct bonuses from its character.
     * This test should be run after PlanetTest because planet test test that the levels
     * are correct in the absence of any bonuses.
     */
    public void testPlayerBonuses() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlayerBonusesAndCloak() *************************");
        SetupOptions optionsRace = new SetupOptions();
        optionsRace.setOption("psi-rating", "20", 0);
        optionsRace.setOption("battle-turning-bonus", "0.25", 0);
        optionsRace.setOption("battle-speed-bonus", "0.25", 0);
        optionsRace.setOption("initiative-bonus", "0.25", 0);
        optionsRace.setOption("cloak-level", "200", 0);
        optionsRace.setOption("max-damage-bonus", "0.25", 0);
        optionsRace.setOption("psi-rating", "20", 0);
        
        createMapAndTwoCraft(fighterDesign, defenderDesign, optionsRace);

        Craft craft3 = newCraft(defenderDesign, player1, ss2);
        battle.join(craft3, true);
        bca = battle.getCrafts();

        bca[0].setLocation(new Location(100, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(150, 500));
        bca[1].setDirection(270);
        bca[2].setLocation(new Location(75, 500));
        bca[2].setDirection(90);
        
        // manv = (speed + turning speed) / 2 + computer power;
        // initiative = manv * bonuses + Attacker Initiative of 5.

        // TEST FIGHTER FOR PLAYER1 HAS GOT ANY PLAYER BONUSES
        assertEquals("Fighter correct speed", 125, bca[0].getSpeed());
        assertEquals("Fighter correct turning speed", 250, bca[0].getTurningSpeed());
        assertEquals("Fighter correct manouverability", 212, bca[0].getManoeuvrability());
        assertEquals("Fighter correct initiative", 270, bca[0].getInitiative());
        assertEquals("Fighter correct visibility", 200, bca[0].getCloakLevel());

        // TEST DEFENDER FOR PLAYER2 HAS NOT GOT ANY PLAYER BONUSES
        assertEquals("Defender correct speed", 80, bca[1].getSpeed());
        assertEquals("Defender correct turning speed", 160, bca[1].getTurningSpeed());
        assertEquals("Defender correct manouverability", 145, bca[1].getManoeuvrability());
        assertEquals("Defender correct initiative", 145, bca[1].getInitiative());
        assertEquals("Defender correct visibility", 0, bca[1].getCloakLevel());

        // TEST DEFENDER FOR PLAYER1 HAS GOT PLAYER BONUSES
        assertEquals("Defender correct speed", 100, bca[2].getSpeed());
        assertEquals("Defender correct turning speed", 200, bca[2].getTurningSpeed());
        assertEquals("Defender correct manouverability", 175, bca[2].getManoeuvrability());
        assertEquals("Defender correct initiative", 223, bca[2].getInitiative());
        assertEquals("Defender correct visibility", 200, bca[2].getCloakLevel());
        assertEquals("Defender correct max damage", 50, bca[2].getCraft().getMaxDamage());
        assertEquals("Defender correct psi rating", 20, bca[2].getPsiLevel());

        // ADD LEADER AND RESOURCE BONUSES TO PLAYER1
        player1.setGeneralLeader(new EngineGeneralLeader(GeneralDetailsList.getInstance().getGenerals()[0]));
        planet1.setResource(new ResourceWarpCrystal());
        player1.flushCache();

        // TEST DEFENDER FOR PLAYER1 HAS LEADER AND RESOURCE BONUSES
        assertEquals("Defender correct speed", 136, bca[2].getSpeed());
        assertEquals("Defender correct turning speed", 240, bca[2].getTurningSpeed());
        assertEquals("Defender correct manouverability", 213, bca[2].getManoeuvrability());
        assertEquals("Defender correct initiative", 356, bca[2].getInitiative());
        planet1.setResource(new ResourceCloakPalm()); 
        player1.flushCache();
        assertEquals("Defender correct visibility", 500, bca[2].getCloakLevel());
        planet1.setResource(new ResourceTritanOre());
        player1.flushCache();
        assertEquals("Defender correct max damage", 90, bca[2].getCraft().getMaxDamage());
        planet1.setResource(new ResourceAntiMatter());
        player1.flushCache();
        bca[1].getShields()[0].setPower(0.0);
        bca[2].fireAllWeapons(bca[1]);
        assertEquals("Defender correct weapon damage", 11.73, bca[1].getCraft().getDamage(), 0.1);
        planet1.setResource(new ResourcePsiFungus());
        player1.flushCache();
        assertEquals("Defender correct psi rating", 35, bca[2].getPsiLevel());

        assertEquals(player1, bca[2].getOwner());
        assertEquals(player1, bca[2].getOriginPlayer());
        
        // TEST DEFENDER STILL HAS ORIGIN BONUSES BUT NOT OWNER BONUSES AFTER IT SURRENDERS TO player1
        bca[2].surrender(player2, true);

        assertEquals(player2, bca[2].getOwner());
        assertEquals(player1, bca[2].getOriginPlayer());

        assertEquals("Defender correct speed", 100, bca[2].getSpeed());
        assertEquals("Defender correct turning speed", 200, bca[2].getTurningSpeed());
        assertEquals("Defender correct manouverability", 175, bca[2].getManoeuvrability());
        assertEquals("Defender correct initiative", 223, bca[2].getInitiative());
        assertEquals("Defender correct visibility", 200, bca[2].getCloakLevel());
        assertEquals("Defender correct max damage", 50, bca[2].getCraft().getMaxDamage());
        bca[0].getShields()[0].setPower(0.0);
        bca[2].rechargeWeapons();
        Utils.setNextRandomInt(99); // Make sure fear event does not happen (due to psi)
        Utils.setNextRandomProbability(1.0); // Make sure miss event does not happen (due to cloak)
        bca[0].setDirection(270); // Make sure target is facing attacker
        bca[2].fireAllWeapons(bca[0]);
        assertEquals("Defender correct weapon damage", 4.7, bca[0].getCraft().getDamage(), 0.1);
        assertEquals("Defender correct psi rating", 20, bca[2].getPsiLevel());
        
        // Remove Psi Bonus because it may cause craft to surrender and cause following tests to fail.
        
        planet1.setResource(null);
        player1.flushCache();

        // ADD DAMAGE BONUS OF 0.3 FROM A MORAL CHOICE  

        bca[0].setDirection(90); // Make sure attacker is facing target
        bca[1].getShields()[0].setPower(0.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(3.59, bca[1].getCraft().getDamage(), 0.1);

        player1.add(mc_testMoralChoice);
        
        bca[1].getShields()[0].setPower(0.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(4.42, bca[1].getCraft().getDamage(), 0.1);

    }

    /**
     * Tests that crafts in battle with fleet wide bonuses do effect all crafts owned by the same player.
     */
    public void testFleetBonuses() throws Exception {
        Logger.logDebug("*******************************  WarTest.testFleetBonuses() *************************");
        createMap(null);

        CraftDesign motherDesign = new CraftDesign("Mother",  tech_flyer100, tech_fusion25, tech_electronic25, tech_commandModule, tech_massDriver1);
        CraftDesign fighterDesign = new CraftDesign("Fighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_massDriver1);
        
        Craft craft1 = newCraft(motherDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player1, ss2);
        Craft craft3 = newCraft(fighterDesign, player2, ss2);
        
        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bca = battle.getCrafts();
        bca[0].setLocation(new Location(100, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(390, 500));
        bca[1].setDirection(275);
        bca[2].setLocation(new Location(550, 500));
        bca[2].setDirection(270);
        
        
        //
        // Test Turning Bonus does work
        //
        lastCraftMovementEvent = null;
        bca[1].moveTo(bca[2]);
        
        assertEquals(0.2, battle.getPlayerBonuses(player1).getFleetBattleTurningBonus(), 0.0);
        assertEquals(240, bca[1].getTurningSpeed(), 0.0);
        assertEquals(90, bca[1].getDirection());
        assertEquals(419,500, bca[1].getLocation());
        
        // This bit does not need checking, can delete it
        assertEquals(2, lastCraftMovementEvent.getMovements().length);
        assertEquals(0.73, ((ObjectTurn)lastCraftMovementEvent.getMovements()[0]).getMovePercent(), 0.1);
        assertEquals(90, ((ObjectTurn)lastCraftMovementEvent.getMovements()[0]).getTo());
        assertEquals(0.27, ((ObjectShift)lastCraftMovementEvent.getMovements()[1]).getMovePercent(), 0.1);
        assertEquals(390,500, ((ObjectShift)lastCraftMovementEvent.getMovements()[1]).getOrigin());
        assertEquals(419,500, ((ObjectShift)lastCraftMovementEvent.getMovements()[1]).getDestination());
        
        //
        // Test Movement Bonus does work
        //
        bca[1].setLocation(new Location(390, 500));
        bca[1].setDirection(90);
        bca[1].setMoveRemaining(1.0);
        bca[1].moveTo(bca[2]);
        
        assertEquals(0.1, battle.getPlayerBonuses(player1).getFleetBattleSpeedBonus(), 0.0);
        assertEquals(110, bca[1].getSpeed(), 0.0);
        assertEquals(500,500, bca[1].getLocation());
                
        //
        // Test Attacker Bonus does work
        //
        bca[1].rechargeWeapons();
        bca[1].fireWeapons(bca[2], null, -1);
        
        assertEquals(0.5, battle.getPlayerBonuses(player1).getFleetAttackerModification(), 0.0);
        assertEquals(3.4, bca[2].getDamage(), 0.1);
        
        //
        // Test Initiative Bonus does work
        //
        
        assertEquals(0.3, battle.getPlayerBonuses(player1).getFleetInitiativeBonus(), 0.0);
        assertEquals(200, bca[1].getManoeuvrability());
        assertEquals(265, bca[1].getInitiative());
        
        //
        // Test Defender Bonus does work
        //
        bca[2].rechargeWeapons();
        bca[2].fireWeapons(bca[1], null, -1);
        
        assertEquals(0.75, battle.getPlayerBonuses(player1).getFleetDefenderModification(), 0.0);
        assertEquals(0.35, bca[1].getDamage(), 0.1);
        
        //
        // Test bonuses also apply to the mother craft
        //
        
        assertEquals(160, bca[0].getTurningSpeed(), 0.0);
        assertEquals(73, bca[0].getSpeed(), 0.0);
        
        bca[2].getCraft().setDamage(0.0);
        bca[0].setLocation(new Location(500,500));
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[2], null, -1);
        
        assertEquals(2.4, bca[2].getDamage(), 0.1);
        
        //
        // Test bonuses DO NOT apply to the enemy craft
        //
        
        assertEquals(null, battle.getPlayerBonuses(player2).getFleetAttackerModification());
        assertEquals(null, battle.getPlayerBonuses(player2).getFleetDefenderModification());
        assertEquals(null, battle.getPlayerBonuses(player2).getFleetBattleSpeedBonus());
        assertEquals(null, battle.getPlayerBonuses(player2).getFleetBattleTurningBonus());
        assertEquals(null, battle.getPlayerBonuses(player2).getFleetInitiativeBonus());
        assertEquals(null, battle.getPlayerBonuses(player2).getFleetRangeModification());
        
        bca[2].getCraft().setDamage(0.0);
        
        assertEquals(200, bca[2].getTurningSpeed(), 0.0);
        assertEquals(100, bca[2].getSpeed(), 0.0);
        
        bca[1].setLocation(new Location(500, 500));
        bca[1].setDirection(90);
        bca[2].setLocation(new Location(550, 500));
        
        bca[2].setDirection(270);
        bca[1].getCraft().setDamage(0.0);
        bca[2].rechargeWeapons();
        bca[2].fireWeapons(bca[1], null, -1);
        
        assertEquals(0.43, bca[1].getDamage(), 0.1);
        
        //
        // Test bonuses are lost when mother craft is destroyed
        //
        
        bca[0].destroy(null, null, null);
        
        bca[1].getCraft().setDamage(0.0);
        bca[1].setLocation(new Location(390, 500));
        bca[1].setDirection(275);
        bca[1].setMoveRemaining(1.0);
        bca[1].moveTo(bca[2]);
        
        assertEquals(null, battle.getPlayerBonuses(player1).getFleetBattleTurningBonus());
        assertEquals(200, bca[1].getTurningSpeed(), 0.0);
        assertEquals(90, bca[1].getDirection());
        assertEquals(402,500, bca[1].getLocation());
        
        bca[1].setLocation(new Location(390, 500));
        bca[1].setMoveRemaining(1.0);
        bca[1].moveTo(bca[2]);
        
        assertEquals(null, battle.getPlayerBonuses(player1).getFleetBattleSpeedBonus());
        assertEquals(100, bca[1].getSpeed(), 0.0);
        assertEquals(490,500, bca[1].getLocation());
                
        bca[1].getCraft().setDamage(0.0);
        bca[2].getCraft().setDamage(0.0);
        bca[1].rechargeWeapons();
        bca[1].fireWeapons(bca[2], null, -1);
        
        assertEquals(null, battle.getPlayerBonuses(player1).getFleetAttackerModification());
        assertEquals(2.0, bca[2].getDamage(), 0.0);
        
        assertEquals(null, battle.getPlayerBonuses(player1).getFleetInitiativeBonus());
        assertEquals(175, bca[1].getManoeuvrability());
        assertEquals(180, bca[1].getInitiative());
        
        //
        // Test Defender Bonus does work
        //
        bca[1].getCraft().setDamage(0.0);
        bca[2].getCraft().setDamage(0.0);
        bca[2].rechargeWeapons();
        bca[2].fireWeapons(bca[1], null, -1);
        
        assertEquals(null, battle.getPlayerBonuses(player1).getFleetDefenderModification());
        assertEquals(2.0, bca[1].getDamage(), 0.0);
    }
    
    /**
     * This tests that a war between two computer players happens automatically.
     */
    public void testComputerWar() throws Exception {
        Logger.logDebug("*******************************  WarTest.testComputerWar() *************************");
        createMap(null);
        craft1 = newCraft(fighterDesign, player1, ss2);
        craft2 = newCraft(defenderDesign, player2, ss2);
        
        MotEngine.instance.endTurn(player1);
        
        assertEquals("Player2 won the battle", 0, player1.getQtyCrafts());
        assertEquals("Player2 won the battle", 1, player1.getQtyBattles());
        assertEquals("Player2 won the battle", 1, player2.getQtyBattles());
        assertEquals("Player2 won the battle", 0, player1.getQtyBattlesWon());
        assertEquals("Player2 won the battle", 1, player2.getQtyBattlesWon());
    }

    
    /**
     * This tests that a war when a player loses its last planet it gets removed from the game.
     * Tests battle state is correct at end of a battle
     *      - Has correct winner
     *      - Has correct crafts and starting crafts
     */
    public void testBattleLose() throws Exception {
        Logger.logDebug("*******************************  WarTest.testBattleLose() *************************");
        createMap(null);

        endBattleEvent = null;
        routedEvent = null;

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player1, ss2);
        Craft craft3 = newCraft(fighterDesign, player1, ss2);
        Craft craft4 = newCraft(fighterDesign, player2, ss2);
        Craft craft5 = newCraft(fighterDesign, player2, ss2);

        Craft craft6 = newCraft(fighterDesign, player1, ss2);
        Craft craft7 = newCraft(fighterDesign, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, true);
        battle.join(craft4, false);
        battle.join(craft5, false);
        battle.setup();
        
        assertFalse(battle.hasEnded());
        assertTrue(battle.contains(player1));
        assertEquals(3, battle.getQtyCraft(player1));
        assertEquals(2, battle.getQtyCraft(player2));
        assertEquals(2, battle.getQtyEnemyCraft(player1));
        assertEquals(3, battle.getQtyEnemyCraft(player2));
        
        BattleCraft [] bca = battle.getCrafts();
        assertEquals(5, bca.length);
        assertEquals(craft1, bca[0].getCraft());
        assertEquals(craft2, bca[1].getCraft());
        assertEquals(craft3, bca[2].getCraft());
        assertEquals(craft4, bca[3].getCraft());
        assertEquals(craft5, bca[4].getCraft());
        
        bca[0].destroy(null, null, null);
        
        assertFalse(battle.hasEnded());
        assertEquals(2, battle.getQtyCraft(player1));
        assertEquals(4, battle.getQtyCraft());
        
        bca[1].retreat();
        
        assertFalse(battle.hasEnded());
        assertEquals(1, battle.getQtyCraft(player1));
        assertEquals(3, battle.getQtyCraft());
        
        bca[4].destroy(null, null, null);
        
        assertFalse(battle.hasEnded());
        assertEquals(1, battle.getQtyCraft(player2));
        assertEquals(2, battle.getQtyCraft());

        // This will call Battle.endBattle()
        bca[2].destroy(null, null, null);
        MotEngine.getInstance().getProxyManager().broadcastQueue();
        
        assertTrue(battle.hasEnded());
        assertEquals(0, battle.getQtyCraft(player1));
        assertEquals(1, battle.getQtyCraft(player2));
        assertEquals(false, battle.isWinner(player1));
        assertEquals(true, battle.isWinner(player2));
        
        assertNotNull(endBattleEvent);
        assertNotNull(routedEvent);

        assertEquals(5, battle.getStartingCrafts().length);
        assertEquals(craft1, battle.getStartingCrafts()[0]);
        assertEquals(craft2, battle.getStartingCrafts()[1]);
        assertEquals(craft3, battle.getStartingCrafts()[2]);
        assertEquals(craft4, battle.getStartingCrafts()[3]);
        assertEquals(craft5, battle.getStartingCrafts()[4]);
        
        assertEquals(1, battle.getRetreatedCraft().length);
        assertEquals(craft2, battle.getRetreatedCraft()[0].getCraft());
        
        assertNull(battle.getCrafts(player1));
        assertEquals(1, battle.getCrafts(player2).length);

        assertEquals(false, battle.isWinner(player1));
        assertEquals(true, battle.isWinner(player2));

        // Because Humans lost retreating craft is destroyed
        assertTrue("craft1 was destroyed even though it retreated", craft1.isDestroyed());
        Craft [] plCrafts = player1.getCraftsAt(ss2, true, true);
        assertEquals(1, plCrafts.length);
        assertEquals(craft6, plCrafts[0]);

        Craft [] p2Crafts = player2.getCraftsAt(ss2, true, true);
        assertEquals("Crafts have not routed", 2, p2Crafts.length);
        
        // End Battle event contains named of this players crafts that were destroyed
        assertEquals(3, endBattleEvent.getCraftDestroyd().length);
        assertEquals("TestFighter1", endBattleEvent.getCraftDestroyd()[0]);
        assertEquals("TestFighter5", endBattleEvent.getCraftDestroyd()[1]);
        assertEquals("TestFighter3", endBattleEvent.getCraftDestroyd()[2]);

        // Route event contains named of this players crafts that were routed
        assertEquals(1, routedEvent.getCraftDestroyed().length);
        assertEquals("TestFighter2", routedEvent.getCraftDestroyed()[0]);
}

    /**
     * Tests that when a craft is retreated from battle and its owner loses the battle - this craft is destroyed.
     */
    public void testRoute() throws Exception {
        Logger.logDebug("*******************************  WarTest.testRoute() *************************");
        createMap(null);

        Craft craft1 = newCraft(fighterDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player1, ss2);
        Craft craft3 = newCraft(fighterDesign, player2, ss2);

        Craft craft4 = newCraft(fighterDesign, player1, ss2);
        Craft craft5 = newCraft(fighterDesign, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, false);
        battle.setup();
        
        BattleCraft [] bca = battle.getCrafts();
        bca[0].retreat();

        assertEquals(false, battle.hasEnded());
        
        battle.endTurn();
        
        bca[1].retreat();
        
        assertEquals(true, battle.hasEnded());
        assertEquals(false, craft1.isDestroyed());
        assertEquals(true, craft2.isDestroyed());
        assertEquals(false, craft3.isDestroyed());
        assertEquals(false, craft4.isDestroyed());
        assertEquals(false, craft5.isDestroyed());
        assertEquals(2, player1.getCrafts().length);
        assertEquals(craft1, player1.getCrafts()[0]);
        assertEquals(craft4, player1.getCrafts()[1]);
    }
    
    
    /**
     * This tests that a war when a player loses its last planet it gets removed from the game.
     */
    public void testPlayerLose() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlayerLose() *************************");
        createMapWithThreePlayers(null);
        Craft craft1 = newCraft(fighterDesign, player1, ss1);
        Craft craft2 = newCraft(fighterDesign, player2, ss2);
        Craft craft3 = newCraft(fighterDesign, player3, ss3);
        
        planet3.removeColony(true);
        
        assertEquals("Players removed at end of turn", 3, MotEngine.instance.getPlayers().getQuantity());
        
        MotEngine.instance.endTurn(player1);
        
        Players players = MotEngine.instance.getPlayers();
        Crafts crafts = MotEngine.instance.getCrafts();
        Planets planets = MotEngine.instance.getPlanets();
        Relations rels = MotEngine.instance.getRelations();
        Player [] pl = players.getPlayers();

        assertEquals("Players removed at end of turn", 2, players.getQuantity());
        assertEquals("Players removed at end of turn", 2, crafts.getCrafts().length);
        assertEquals("Players removed at end of turn", 3, planets.getPlanets().length);
        assertEquals("Players removed at end of turn", 0.0, planets.getPlanets()[2].getPopulation(), 0.0);
        assertEquals("Players removed at end of turn", null, planets.getPlanets()[2].getOwner());

        assertEquals("Player 1 still exists", player1, pl[0]);
        assertEquals("Player 1 still exists", craft1, pl[0].getCrafts()[0]);
        assertEquals("Player 1 still exists", planet1, pl[0].getPlanets()[0]);

        assertEquals("Player 2 still exists", player2, pl[1]);
        assertEquals("Player 2 still exists", craft2, pl[1].getCrafts()[0]);
        assertEquals("Player 2 still exists", planet2, pl[1].getPlanets()[0]);
    }
    
    public void testBombard() throws Exception {
        Logger.logDebug("*******************************  WarTest.testBombard() *************************");
        createMap(null);
        GeneralBasic general = (GeneralBasic)player1.getGeneral();

        planet2.setPopulation(10.0);
        planet2.setQtyFactories(10.0);
        
        CraftTask attackTask = new CraftTaskAttack(planet2, player1, 2, null);
        Craft attackCraft1 =  newCraft(fighterDesign, player1, ss2);
        Craft attackCraft2 =  newCraft(fighterDesign, player1, ss2);
        general.add(attackTask);
        attackTask.add(attackCraft1);
        attackTask.add(attackCraft2);
        general.assignTaskToFleet(attackCraft1.getFleet());
        
        assertEquals("planet2 has been bombarded", 8.0, planet2.getPopulation(), 0.0);
        assertEquals("planet2 has been bombarded", 8.0, planet2.getQtyFactories(), 0.0);
                
    }

    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponShieldDrain() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponShieldDrain() *************************");
        CraftDesign cd = new CraftDesign("TestShieldDrain", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_shieldDrain);
        createMapAndTwoCraft(cd, 500, fighterDesign, 600, null);

        assertEquals("Drain correct power", 5.0, tech_shieldDrain.getDamage(), 0.01);
        
        assertEquals("Have correct shield power", 10.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 10.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST DOES NOT DRAIN POWER WHEN SHIELDS ARE FULLY CHARGED
        //
        
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 10.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 5.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST DOES DRAIN POWER WHEN SHIELDS ARE NOT FULLY CHARGED BUT ONLY TO FULLY CHARGED LEVEL
        //
        
        bca[0].setShieldPower(0, 8.0);
        bca[0].getCraft().setDamage(0.0);
        bca[1].setShieldPower(0, 10.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 10.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 5.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST DOES DRAIN MAXIMUM POWER WHEN SHIELDS ARE NOT FULLY CHARGED
        //
        
        bca[0].setShieldPower(0, 0.0);
        bca[0].getCraft().setDamage(0.0);
        bca[1].setShieldPower(0, 10.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 5.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 5.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST DOES NOT DRAIN POWER WHEN SHIELDS THERE IS NO POWER TO DRAIN
        //
        
        bca[0].setShieldPower(0, 0.0);
        bca[0].getCraft().setDamage(0.0);
        bca[1].setShieldPower(0, 0.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 0.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 0.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST DOES DRAIN AS MUCH POWER AS IS AVAILABLE
        //
        
        bca[0].setShieldPower(0, 0.0);
        bca[0].getCraft().setDamage(0.0);
        bca[1].setShieldPower(0, 2.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 2.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 0.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST DOES DRAIN AS MUCH POWER AS IS AVAILABLE
        //
        
        bca[0].setShieldPower(0, 0.0);
        bca[0].getCraft().setDamage(0.0);
        bca[1].setShieldPower(0, 2.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 2.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 0.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST DOES DRAIN POWER AND TRANSFER IT TO ANOTHER CRAFT IF ITS SHIELDS ARE FULLY CHARGED
        //

        Craft craft = newCraft(fighterDesign, player1, ss2);
        BattleCraft bc = battle.join(craft, true);
        bc.setLocation(new Location(550, 500));
        
        bca[0].setShieldPower(0, 9.5);
        bca[0].getCraft().setDamage(0.0);
        bca[1].setShieldPower(0, 10.0);
        bca[1].getCraft().setDamage(0.0);
        bc.setShieldPower(0, 6.0);
        bc.getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 9.5, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 5.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);

        assertEquals("Have correct shield power", 10.0, bc.getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bc.getDamage(), 0.01);
        
        //
        // TEST DOES NOT DRAIN POWER AND TRANSFER IT TO ANOTHER CRAFT IF THE OTHER CRAFT IS OUT OF RANGE
        //

        bc.setLocation(new Location(700, 500));
        
        bca[0].setShieldPower(0, 9.5);
        bca[0].getCraft().setDamage(0.0);
        bca[1].setShieldPower(0, 10.0);
        bca[1].getCraft().setDamage(0.0);
        bc.setShieldPower(0, 6.0);
        bc.getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 10.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 5.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);

        assertEquals("Have correct shield power", 6.0, bc.getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bc.getDamage(), 0.01);
        
        //
        // TEST SHIELD DRAIN WORKS WHEN WE HAVE MULTIPLE OF THEM AND A MASS DRIVER 2
        //

        Weapons weapons1 = craft1.getWeaponBays().getBaysAsArray()[0].getWeapons();
        weapons1.add(tech_shieldDrain);
        weapons1.add(tech_massDriver2);
        bca[0].setWeapons(weapons1, false, true, null);
        
        bca[0].setShieldPower(0, 0.0);
        bca[0].getCraft().setDamage(0.0);
        bca[1].setShieldPower(0, 10.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 10.0, bca[0].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[0].getDamage(), 0.01);

        assertEquals("Have correct shield power", 0.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 2.0, bca[1].getDamage(), 0.01);
        
    	//
    	// Test firing all weapons including at a craft with no shields has no effect
    	//
		bca[0].getShields()[0].setPower(tech_energy10.getPower()/2);
		bca[1].getShields()[0].setPower(0.0);
		bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(1, bca[0].getWeapon(tech_massDriver2).getQtyFired());
        assertEquals(0, bca[0].getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals("2.0", weaponFireEvent.getTargetDescription());

        bca[0].fireWeapons(bca[1], bca[0].getWeapon(tech_shieldDrain), 1);
        
        assertEquals(1, bca[0].getWeapon(tech_massDriver2).getQtyFired());
        assertEquals(1, bca[0].getWeapon(tech_shieldDrain).getQtyFired());
        assertEquals("-0", weaponFireEvent.getTargetDescription());
        assertEquals("+0", weaponFireEvent.getSourceDescription());

        
    }

    /**
     * Tests crafts containing fighters can launch the fighters and that computer players can control
     * the fighters to attack enemies.
     */
    public void testFighterCraft() throws Exception {
        Logger.logDebug("*******************************  WarTest.testFighterCraft() *************************");
        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_massDriver1, tech_fighterBays);
        createMapAndTwoCraft(cd, 500, fighterDesign, 600, null);
        
        BattleCraft [] fighters = bca[0].launchFighters();
        
        craft2.setShields(null);
        bca[1].setShields(null, true);
        
        assertEquals("Have launched fighters correctly", 2, fighters.length);
        assertEquals("Have launched fighters correctly", 3, battle.getQtyCraft(player1));
        
        assertEquals("Have launched fighters correctly", 536.0, fighters[0].getLocation().getX(), 5.0);
        assertEquals("Have launched fighters correctly", 500.0, fighters[0].getLocation().getY(), 5.0);
        assertEquals("Have launched fighters correctly", 90, fighters[0].getDirection());
        
        assertEquals("Have launched fighters correctly", 536.0, fighters[1].getLocation().getX(), 5.0);
        assertEquals("Have launched fighters correctly", 521.0, fighters[1].getLocation().getY(), 5.0);
        assertEquals("Have launched fighters correctly", 90, fighters[1].getDirection());
        
        assertEquals("Fighter craft has correct design", tech_fighterI, fighters[0].getCraft().getHull());
        assertEquals("Fighter craft has correct design", tech_fusion25, fighters[0].getCraft().getEngine());
        assertEquals("Fighter craft has correct design", null, fighters[0].getCraft().getComputers());
        assertEquals("Fighter craft has correct design", null, fighters[0].getCraft().getShields());
        assertEquals("Fighter craft has correct design", 2, fighters[0].getCraft().getWeaponBays().getQtyWeapons());
        assertEquals("Fighter craft has correct design", tech_massDriver1, fighters[0].getCraft().getWeaponBays().getAllWeapons()[0]);
        assertEquals("Fighter craft has correct design", tech_massDriver1, fighters[0].getCraft().getWeaponBays().getAllWeapons()[1]);
        
        // Test Fighter have the highest initiative so move first and fighter will attack craft2.
        {
            fired = false;
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Next craft to move is correct", fighters[0], craft);
            craft.getNextOrders();
            assertEquals(new Location(577, 500), craft.getLocation());
            assertEquals("Craft attacked", true, fired);
            assertEquals("Craft attacked", 4.5, craft2.getDamage(), 1.0);
        }

        // Test Fighter have the highest initiative so move first and fighter will attack craft2.
        {
            fired = false;
            craft2.setDamage(0.0);
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Next craft to move is correct", fighters[1], craft);
            craft.getNextOrders();
            assertEquals(new Location(577, 521), craft.getLocation());
            assertEquals("Craft attacked", true, fired);
            assertEquals("Craft attacked", 4.5, craft2.getDamage(), 1.0);
        }
        
        bca[0].setMoveRemaining(0.0);
        bca[0].dischargeWeapons();

        // Test craft2 will attack the closest fighter.
        {
            fired = false;
            craft2.setDamage(0.0);
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Next craft to move is correct", bca[1], craft);
            craft.getNextOrders();
            assertEquals("Craft attacked", true, fired);
            assertEquals("Craft attacked", 1.75, fighters[0].getDamage(), 0.5);
        }
        
        bca[1].rechargeWeapons();
        fighters[0].getCraft().setDamage(fighters[0].getCraft().getMaxDamage()-0.5);

        // Test craft2 will attack the closest fighter.
        {
            fired = false;
            qtyCraftDestroyed = 0;
            lastCraftDestroyed = null;
            craft2.setDamage(0.0);
            BattleCraft craft = (BattleCraft) battle.getNextToMove(false);
            assertEquals("Next craft to move is correct", bca[1], craft);
            craft.getNextOrders();
            assertEquals("Craft attacked", true, fired);
            assertEquals("Craft attacked", 1, qtyCraftDestroyed);
            assertEquals("Craft attacked", fighters[0].getName(), lastCraftDestroyed);
            assertEquals("Craft attacked", 2, battle.getQtyCraft(player1));
            assertEquals(null, battle.getWreckages().getWrecks()); // Fighters do not cause wreckage

        }

    }

    /**
     * Tests crafts containing fighters can launch the fighters and that computer players can control
     * the fighters to attack enemies.
     */
    public void testFighterCraftOnPlanet() throws Exception {
        Logger.logDebug("*******************************  WarTest.testFighterCraftOnPlanet() *************************");
        createUserPlayerMap(null);

        Craft craft1 = newCraft(fighterDesign, player2, ss1);
        
        planet1.setWeapons(new Weapons(tech_missile6));
        planet1.setShields(new Shields(tech_energy10));
        planet1.setScanners(new Scanners(tech_photon100));

        userPlayer1.addSpecial(tech_fighterBays);
        userPlayer1.addWeapon(tech_detonator);
        userPlayer1.addMiscTechnology(tech_kamikaze);
        
        battle = newBattle(ss2);
        battle.join(craft1, false);
        battle.join(planet1);
        battle.setup();

        bp = battle.getPlanet();
        bca = battle.getCrafts();

        assertEquals(1, bca.length);
        assertEquals(craft1, bca[0].getCraft());
        
        bca[0].endTurn();
        bp.launchFighters();
        
        bca = battle.getCrafts();
        assertEquals(7, bca.length);
        assertEquals(craft1, bca[0].getCraft());
        assertTrue(bca[1].isFighter());
        assertTrue(bca[2].isFighter());
        assertTrue(bca[3].isFighter());
        assertTrue(bca[4].isFighter());
        assertEquals(tech_massDriver1, 2, bca[1].getWeapons());
        assertEquals(tech_massDriver1, 2, bca[2].getWeapons());
        assertEquals(tech_massDriver1, 2, bca[3].getWeapons());
        assertEquals(tech_massDriver1, 2, bca[4].getWeapons());
        assertTrue(bca[5].isFighter());
        assertTrue(bca[6].isFighter());
        assertEquals(tech_detonator, 1, bca[5].getWeapons());
        assertEquals(tech_detonator, 1, bca[6].getWeapons());
        
        assertEquals("SuicideDecorator", bca[5].getDecorators().getVisibleDecorators(userPlayer1).getDecorators());
        assertEquals("SuicideDecorator", bca[6].getDecorators().getVisibleDecorators(userPlayer1).getDecorators());
        assertEquals(null, bca[5].getDecorators().getVisibleDecorators(player2));
        assertEquals(null, bca[6].getDecorators().getVisibleDecorators(player2));

    }
    
    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponGirator() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponGirator() *************************");
        CraftDesign cd = new CraftDesign("GiratorFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_girator);
        createMapAndTwoCraft(cd, 500, fighterDesign, 700, null);

        assertEquals("Correct Damage", 25.0, tech_girator.getDamage(), 0.01);
        
        assertEquals("Have correct shield power", 10.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST GIRATOR DOES NOT FIRE WHEN OUT OF RANGE
        //
        
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 10.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 0.0, bca[1].getDamage(), 0.01);
        
        //
        // TEST GIRATOR INFLICTS CORRECT DAMAGE
        //
        
        bca[1].setLocation(new Location(600, 500));
        bca[1].setShieldPower(0, 10.0);
        bca[1].getCraft().setDamage(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have correct shield power", 10.0, bca[1].getShieldPower(), 0.01);
        assertEquals("Have correct damage", 25.0, bca[1].getDamage(), 0.01);
    }
        
    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponImmobiliser() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponImmobiliser() *************************");
        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_immobiliser);
        createMapAndTwoCraft(cd, 500, fighterDesign, 700, null);

        bca[1].getShields()[0].setPower(0.0);
        
        assertEquals("Correct Damage", 25.0, tech_immobiliser.getDamage(), 0.01);
        assertEquals("Have Turning Speed", 200, bca[1].getTurningSpeed());
        assertEquals("Have Impulse Speed", 100, bca[1].getSpeed());
        
        //
        // TEST IMMOBILISER DOES NOT FIRE WHEN OUT OF RANGE
        //
        
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have Turning Speed", 200, bca[1].getTurningSpeed());
        assertEquals("Have Impulse Speed", 100, bca[1].getSpeed());
        
        //
        // TEST GIRATOR INFLICTS CORRECT DAMAGE
        //
        
        bca[1].setLocation(new Location(600, 500));
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have Turning Speed", 175, bca[1].getTurningSpeed());
        assertEquals("Have Impulse Speed", 75, bca[1].getSpeed());

        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Have Turning Speed", 150, bca[1].getTurningSpeed());
        assertEquals("Have Impulse Speed", 50, bca[1].getSpeed());

        //
        // TEST IMMOBILISER DOES FIRE WHEN SHIELDS ARE RAISED
        //
        
        bca[1].getShields()[0].setPower(1.0);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);

        assertEquals("Have Turning Speed", 125, bca[1].getTurningSpeed());
        assertEquals("Have Impulse Speed", 25, bca[1].getSpeed());
        
    	//
    	// Test firing all weapons including at a craft with no movement does not fire weapon
    	//
        
        weaponFireEvent = null;
		bca[0].rechargeWeapons();
		bca[1].setMovementDamage(bca[1].getSpeed(false));
        bca[0].fireAllWeapons(bca[1]);
        
        assertNull(weaponFireEvent);
        assertEquals(0, bca[0].getWeapon(tech_immobiliser).getQtyFired());

        bca[0].fireWeapons(bca[1], bca[0].getWeapon(tech_immobiliser), 1);
        
        assertEquals(1, bca[0].getWeapon(tech_immobiliser).getQtyFired());
        assertEquals("Engine Damage +0", weaponFireEvent.getTargetDescription());

    }
        

    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponPsi() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponImmobiliser() *************************");
        createMapAndTwoCraft(fighterDesign, 500, fighterDesign, 700, null);

        bca[1].getShields()[0].setPower(0.0);
        
        assertEquals("Correct Damage", 1.0, tech_psiBlast.getDamage(), 0.01);
		bca[0]._replaceAllWeaponsWith(tech_psiBlast, false, true);
        
        assertEquals("Has correct psi damage", 0.0, bca[1].getPsiDamage(), 0.01);
        
        //
        // TEST PSI BLAST DOES NOT FIRE WHEN OUT OF RANGE
        //
        
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Has correct psi damage", 0.0, bca[1].getPsiDamage(), 0.01);
        
        //
        // TEST PSI BLAST INFLICTS CORRECT DAMAGE
        //
        
        bca[1].setLocation(new Location(600, 500));
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Has correct psi damage", 1.0, bca[1].getPsiDamage(), 0.01);

        //
        // TEST PSI BLAST IS REDUCED BY PSI SHIELDS
        //
        
        craft2.getShields().getShieldAtIndex(0).setPsiRating(0.75);
        craft2.getShields().flushCache();
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);

        assertEquals("Has correct psi damage", 1.25, bca[1].getPsiDamage(), 0.01);

        bca[1].getShields()[0].getShield().setPsiRating(0.0);
    }
        
    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponBerserker() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponBerserker() *************************");
        createMapAndThreeCraft(fighterDesign, fighterDesign, fighterDesign, null);

        assertEquals("Correct Damage", 0.5, tech_berserker.getDamage(), 0.01);
        assertEquals("Correct Qty Shots", 3, tech_berserker.getQtyShots().intValue());
		bca[0]._replaceAllWeaponsWith(tech_berserker, false, true);
        
        //
        // TEST BERSERKER DOES NOT ALWAYS WORK (works 50% of time)
        //
        
		Utils.setNextRandomProbability(0.8);
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(player2, bca[1].getController());
        
        assertEquals("Did not work", null, bca[1].getControllerDetails());
        assertEquals("Did not work", player2, bca[1].getController());

        //
        // TEST BERSERKER WORKS SOMETIMES (50%)
        //
        
		Utils.setNextRandomProbability(0.4);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);

        assertEquals("Berserker worked", true, bca[1].getControllerDetails() instanceof BattleCraftControllerBerserker);
        assertEquals(null, bca[1].getController());
        assertEquals(2, bca[0].getWeapons()[0].getQtyShots());

        //
        // Test when craft2 moves it attacks the closest craft (craft3) - goes berserk
        //
        
        bca[2].setDirection(0);
        bca[0].endTurn();
        bca[2].endTurn();
        bca[1].getNextOrders();
        
        assertEquals(null, bca[1].getControllerDetails());
        assertEquals(player2, bca[1].getController());
        assertEquals(180, bca[1].getDirection());
        assertEquals(700,555, bca[1].getLocation());
        assertEquals(1, bca[1].getWeapons()[0].getQtyShots());
        assertEquals(1.0, bca[2].getDamage(), 0.0);
        
        //
        // TEST PSI SHIELDS REDUCE CHANCE MIND CONTROL WORKS 
        //
        
        Shields shields = craft2.getShields();
        shields.removeAll();
        shields.add(Technologies.getShield(TestTechs.PSI_I));
        bca[1].setShields(shields, true);
        
		Utils.setNextRandomProbability(0.4);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Did not work", null, bca[1].getControllerDetails());
        assertEquals("Did not work", player2, bca[1].getController());
        assertEquals(3, bca[0].getWeapons()[0].getQtyShots());
        
        //
        // TEST ONLY GET THREE SHOTS 
        //
        
        
        assertFalse(bca[0].getWeapons()[0].canRecharge());
        
        bca[0].dischargeWeapons();
        bca[0].rechargeWeapons();
        
        assertFalse(bca[0].getWeapons()[0].hasWeaponsToFire());
        
        
    }
        
    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponMindControl() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponMindControl() *************************");
        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_mindControl);        
        createMapAndThreeCraft(cd, fighterDesign, fighterDesign, null);

        assertEquals("Correct Damage", 0.5, tech_mindControl.getDamage(), 0.01);
        assertEquals("Correct Damage", 3, tech_mindControl.getQtyShots().intValue());
        
        //
        // TEST MIND CONTROL DOES NOT ALWAYS WORK (works 50% of time)
        //
        
		Utils.setNextRandomProbability(0.8);
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Did not work", null, bca[1].getControllerDetails());
        assertEquals("Did not work", player2, bca[1].getController());

        //
        // TEST MIND CONTROL WORKS SOMETIMES (50%)
        //
        
		Utils.setNextRandomProbability(0.4);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);

        assertEquals("Berserker worked", true, bca[1].getControllerDetails() instanceof BattleCraftControllerMindControl);
        assertEquals(player1, bca[1].getController());
        assertEquals(2, bca[0].getWeapons()[0].getQtyShots());

        //
        // Test when craft2 moves it is controlled by player1 so attacks the craft3
        //
        
        bca[2].setDirection(0);
        bca[0].endTurn();
        bca[2].endTurn();
        bca[1].getNextOrders();
        
        assertEquals(null, bca[1].getControllerDetails());
        assertEquals(player2, bca[1].getController());
        assertEquals(180, bca[1].getDirection());
        assertEquals(new Location(700,555), bca[1].getLocation());
        assertEquals(1, bca[1].getWeapons()[0].getQtyShots());
        assertEquals(1.0, bca[2].getDamage(), 0.0);
        
        //
        // TEST PSI SHIELDS REDUCE CHANCE MIND CONTROL WORKS 
        //
        
        Shields shields = craft2.getShields();
        shields.removeAll();
        shields.add(tech_psiI);
        bca[1].setShields(shields, true);
        
		Utils.setNextRandomProbability(0.4);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Did not work", null, bca[1].getControllerDetails());
        assertEquals("Did not work", player2, bca[1].getController());
        assertEquals(3, bca[0].getWeapons()[0].getQtyShots());
        
        //
        // TEST ONLY GET THREE SHOTS 
        //
        
        
        assertFalse(bca[0].getWeapons()[0].canRecharge());
        
        bca[0].dischargeWeapons();
        bca[0].rechargeWeapons();
        
        assertFalse(bca[0].getWeapons()[0].hasWeaponsToFire());
    }
        
    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponCowardice() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponCowardice() *************************");
        createMapAndThreeCraft(fighterDesign, fighterDesign, fighterDesign, null);

        assertEquals("Correct Damage", 0.5, tech_cowardice.getDamage(), 0.01);
        assertEquals("Correct Qty Shots", 3, tech_cowardice.getQtyShots().intValue());
		bca[0]._replaceAllWeaponsWith(tech_cowardice, false, true);
        
        //
        // TEST COWARDICE DOES NOT ALWAYS WORK (works 50% of time)
        //
        
		Utils.setNextRandomProbability(0.8);
        bca[0].fireAllWeapons(bca[1]);
        assertEquals(player2, bca[1].getController());
        
        assertEquals("Did not work", null, bca[1].getControllerDetails());
        assertEquals("Did not work", player2, bca[1].getController());

        //
        // TEST COWARDICE WORKS SOMETIMES (50%)
        //
        
		Utils.setNextRandomProbability(0.4);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(true, bca[1].getControllerDetails() instanceof BattleCraftControllerCowardice);
        assertEquals(null, bca[1].getController());
        assertEquals(2, bca[0].getWeapons()[0].getQtyShots());

        //
        // Test when craft2 moves is uses the cowardice controller - runs away
        //
        
        bca[0].endTurn();
        bca[2].endTurn();
        bca[1].getNextOrders();
        
        assertEquals(null, bca[1].getControllerDetails());
        assertEquals(player2, bca[1].getController());
        assertEquals(90, bca[1].getDirection());
        assertEquals(new Location(709,500), bca[1].getLocation());
        
        //
        // TEST PSI SHIELDS REDUCE CHANCE MIND CONTROL WORKS 
        //
        
        Shields shields = craft2.getShields();
        shields.removeAll();
        shields.add(tech_psiI);
        bca[1].setShields(shields, true);
        
		Utils.setNextRandomProbability(0.4);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Did not work", null, bca[1].getControllerDetails());
        assertEquals("Did not work", player2, bca[1].getController());
        assertEquals(3, bca[0].getWeapons()[0].getQtyShots());
        
        //
        // TEST ONLY GET THREE SHOTS 
        //
        
        
        assertFalse(bca[0].getWeapons()[0].canRecharge());
        
        bca[0].dischargeWeapons();
        bca[0].rechargeWeapons();
        
        assertFalse(bca[0].getWeapons()[0].hasWeaponsToFire());
    }

    public void testCorardiceFireAllThatWillHaveNoEffect() throws Exception {
		CraftDesign tf1 = new CraftDesign("TF1",tech_flyer30, tech_fusion25, tech_energy10, tech_massDriver1, tech_cowardice);
		CraftDesign tf2 = new CraftDesign("TF1",tech_flyer30, tech_fusion25, tech_energy10, tech_massDriver1);
		createMapAndTwoCraft(tf1, 500, tf2, 600, null);

    	//
    	// Test firing all weapons including Mind Controller at a craft that is already under mind control
    	//
        
		bca[1].setControllerDetails(new BattleCraftControllerCowardice(bca[1], 1));
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(1, bca[0].getWeapon(tech_massDriver1).getQtyFired());
        assertEquals(0, bca[0].getWeapon(tech_cowardice).getQtyFired());
        assertEquals("1.0", weaponFireEvent.getTargetDescription());

        bca[0].fireWeapons(bca[1], bca[0].getWeapon(tech_cowardice), 1);
        
        assertEquals(1, bca[0].getWeapon(tech_cowardice).getQtyFired());
        assertEquals("Cowardice - Failed", weaponFireEvent.getTargetDescription());
    }
    

    private Weapon getBigCoward() {
        Weapon v = new Weapon();
        v.setName("BigCowardice");
        v.setDamage(1.5);
        v.setDegredation(0.0);
        v.setQtyShots(new Integer(2));
        v.setRange(500);
        v.setVariance(0.0);
        v.setSpace(25);
        v.setCostToBuild(80);
        v.setCostToResearch(14000);
        v.setMaintenanceCost(null);
        v.setGroup("Cowardice");
        return v;
    }

    
    /**
     * Test a mind weapon (such as Mind Control/Cowardice/Berserker) that lasts for more 
     * than one turn.
     */
    public void testPowerfulMindWeapon() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPowerfulMindWeapon() *************************");
        createMapAndTwoCraft(fighterDesign, 400, fighterDesign, 600, null);

        Weapon cowardice = getBigCoward();
		bca[0]._replaceAllWeaponsWith(cowardice, false, true);
        
        //
        // TEST COWARDICE WORKS SOMETIMES (50%)
        //
		Logger.logDebug("");
		Logger.logDebug("Fire Cowardice");
		Logger.logDebug("");
        
		Utils.setNextRandomProbability(0.4);
        bca[0].fireAllWeapons(bca[1]);

        BattleCraftControllerBase controller =  (BattleCraftControllerBase) bca[1].getControllerDetails(); 
        assertEquals(2, controller.getQtyTurns());
        assertEquals(0, controller.getTurn());
        assertEquals(null, bca[1].getController());
        assertEquals(600,500, bca[1].getLocation());
        assertEquals(270, bca[1].getDirection());

        //
        // CRAFT2 IS CONTROLLED BY COWARDICE AND RUNS AWAY (STILL ONE TURN LEFT)
        //
        
		Logger.logDebug("");
		Logger.logDebug("Cowardice turn 1");
		Logger.logDebug("");
		
        bca[0].endTurn();
        bca[1].getNextOrders();
        
        controller =  (BattleCraftControllerBase) bca[1].getControllerDetails(); 
        assertEquals(1, controller.getTurn());
        assertEquals(null, bca[1].getController());
        assertEquals(609,500, bca[1].getLocation());
        assertEquals(90, bca[1].getDirection());
        
        //
        // CRAFT2 IS CONTROLLED BY COWARDICE AND RUNS AWAY AGAIN BUT CONTROL RETURNS TO OWNER
        //
        
		Logger.logDebug("");
		Logger.logDebug("Cowardice turn 2");
		Logger.logDebug("");
		
        //battle.getNextToMove(true);
        bca[0].endTurn();
        bca[1].setMoveRemaining(1.0);
        bca[1].getNextOrders();
        
        assertEquals(null, bca[1].getControllerDetails());
        assertEquals(player2, bca[1].getController());
        assertEquals(709,500, bca[1].getLocation());
        assertEquals(90, bca[1].getDirection());
        
    }

    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponMindControlWithNoEnemy() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponMindControlWithNoEnemy() *************************");
        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_mindControl);        
        createMapAndTwoCraft(cd, 400, fighterDesign, 600, null);

        assertEquals("Correct Damage", 0.5, tech_mindControl.getDamage(), 0.01);
        assertEquals("Correct Damage", 3, tech_mindControl.getQtyShots().intValue());
        
        //
        // TEST MIND CONTROL WORKS SOMETIMES (50%)
        //
        
		Utils.setNextRandomProbability(0.4);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(true, bca[1].getControllerDetails() instanceof BattleCraftControllerMindControl);
        assertEquals(player1, bca[1].getController());
        assertEquals(1, bca[0].getWeapons()[0].getQtyShots());

        //
        // Test when craft2 moves it has no enemy so ends turn
        //
        
        assertEquals(1.0, bca[1].getMoveRemaining());
        assertEquals(true, bca[1].hasWeaponsToFire());
        assertEquals(0.0, bca[0].getDamage(), 0.0);
        assertEquals(0.0, bca[1].getDamage(), 0.0);
        
        new BattleAttackCmd(player1, battle, bca[1], null, false, false, false, null, null, -1, false).run();
        
        assertEquals(null, bca[1].getControllerDetails());
        assertEquals(player2, bca[1].getController());
        assertEquals(0.0, bca[1].getMoveRemaining());
        assertEquals(false, bca[1].hasWeaponsToFire());
        assertEquals(0.0, bca[0].getDamage(), 0.0);
        assertEquals(0.0, bca[1].getDamage(), 0.0);
    }

    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponTransport() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponTransport() *************************");
		CraftDesign etDesign = new CraftDesign("TestET", tech_flyer60, tech_fusion25, tech_electronic25, tech_equipmentTransport);
        createMapAndTwoCraft(etDesign, 500, fighterDesign, 700, null);

        assertEquals("Correct Damage", 5.0, tech_equipmentTransport.getDamage(), 0.01);
        
        craft2.setShields(null);
        bca[1].setShields(null, true);
        
        testCraftState(bca[1], tech_massDriver1, 1, null, 0, null, 0, null, 0);
        
        //
        // TEST TRANSPORT DOES NOT FIRE WHEN OUT OF RANGE
        //
        
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);
        
        assertNull(bca[0].getSalvage());
        testCraftState(bca[1], tech_massDriver1, 1, null, 0, null, 0, null, 0);
        
        //
        // TEST TRANSPORT CAN STEEL WEAPONS
        //
        
        bca[1].setLocation(new Location(600, 500));
        bca[1].getCraft().setDamage(0);
        bca[0].rechargeWeapons();
        Utils.clearNextRandomInt();
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(new Technology[]{tech_massDriver1}, bca[0].getSalvage());
        testCraftState(bca[1], null, 0, null, 0, null, 0, null, 0);

        //
        // TEST CAN STEAL MORE THAN ONE THING AT A TIME
        //

        weapons = craft1.getWeaponBays().getBaysAsArray()[0].getWeapons();
        weapons.removeAll();
        weapons.add(tech_equipmentTransport);
        bca[0].setWeapons(weapons, false, true, null);
        bca[0].setSalvage(null);

        weapons = craft2.getWeaponBays().getBaysAsArray()[0].getWeapons();
        weapons.removeAll();
        weapons.add(tech_massDriver1);
        weapons.add(tech_massDriver1);
        bca[1].setWeapons(weapons, false, true, null);

        craft2.setShields(new Shields(tech_energy10));
        bca[1].setShields(craft2.getShields(), true);

        craft2.setSpecials(new Specials(tech_armour25));

        testCraftState(bca[1], tech_massDriver1, 2, null, 0, tech_energy10, 1, tech_armour25, 1);

        //
        // TEST TRANSPORT CANNOT STEAL ANYTHING WHILE THE SHIELDS ARE UP
        //

        bca[0].rechargeWeapons();
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);

        assertNull(bca[0].getSalvage());
        testCraftState(bca[1], tech_massDriver1, 2, null, 0, tech_energy10, 1, tech_armour25, 1);

        
        //
        // TEST TRANSPORT CAN STEAL ALL TECHNOLOGIES
        //

        bca[1].getShields()[0].setPower(0.0);
        
        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);

        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);

        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);

        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);

        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(new Technology[]{tech_massDriver1, tech_massDriver1, tech_energy10, tech_armour25}, bca[0].getSalvage());
        testCraftState(bca[1], null, 0, null, 0, null, 0, null, 0);

        //
        // TEST TRANSPORT ONLY STEALS ONE WEAPON FROM WEAPON INSTANCE
        //

        weapons = craft2.getWeaponBays().getBaysAsArray()[0].getWeapons();
        weapons.removeAll();
        weapons.add(tech_massDriver1);
        weapons.add(tech_massDriver1);
        bca[1].setWeapons(weapons, false, true, null);

        bca[0].setSalvage(null);
        bca[0].rechargeWeapons();
        bca[1].getCraft().setDamage(0);
        Utils.setNextRandomInt(2);
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(new Technology[]{tech_massDriver1}, bca[0].getSalvage());
        testCraftState(bca[1], tech_massDriver1, 1, null, 0, null, 0, null, 0);
    }

    private static void testCraftState(BattleCraft  bc, Weapon weapon1, int qtyWeapon1, Weapon weapon2, int qtyWeapon2, Shield shield, int qtyShields, Special special, int qtySpecials) throws ApplicationError {
        Craft craft = bc.getCraft();
        int qtyInstances = 0;
        int totalWeapons = 0;
        if (qtyWeapon1 > 0) {
            qtyInstances++;
            totalWeapons += qtyWeapon1;
        }
        if (qtyWeapon2 > 0) {
            qtyInstances++;
            totalWeapons += qtyWeapon2;
        }
        
        if (totalWeapons == 0) {
            assertEquals("craft has correct weapons", true, craft.getWeaponBays() == null || craft.getWeaponBays().getQtyWeapons() == 0);
        } else {
            assertEquals("craft has correct weapons", totalWeapons, craft.getWeaponBays().getAllWeapons().length);
        }
        
        if (qtyInstances == 0) {
            assertEquals("craft has correct weapons", null, bc.getWeapons());
        } else {
            assertEquals("craft has correct weapons", qtyInstances, bc.getWeapons().length);
        }
        
        if (qtyWeapon1 > 0) {
            assertEquals("craft has correct weapons", qtyWeapon1, craft.getWeaponBays().getAllWeapons(weapon1).length);
            assertEquals("craft has correct weapons", weapon1, bc.getWeapons()[0].getWeapon());
            assertEquals("craft has correct weapons", qtyWeapon1, bc.getWeapons()[0].getQty());
        } else {
            assertEquals("craft has correct weapons", 0, craft.getWeaponBays().getAllWeapons(weapon1).length);
            assertEquals("craft has correct weapons", false, craft.getWeaponBays().contains(weapon1));
            assertEquals("craft has correct weapons", true, bc.getWeapons() == null || bc.getWeapons().length == 0);
        }
        
        if (qtyWeapon2 > 0) {
            assertEquals("craft has correct weapons", qtyWeapon2, craft.getWeaponBays().getAllWeapons(weapon2).length);
            assertEquals("craft has correct weapons", weapon2, bc.getWeapons()[1].getWeapon());
            assertEquals("craft has correct weapons", qtyWeapon2, bc.getWeapons()[1].getQty());
        } else {
            assertEquals("craft has correct weapons", 0, craft.getWeaponBays().getAllWeapons(weapon2).length);
            assertEquals("craft has correct weapons", false, craft.getWeaponBays().contains(weapon2));
            assertEquals("craft has correct weapons", true, bc.getWeapons() == null || bc.getWeapons().length <= 1);
        }
        
        if (qtyShields > 0) {
            assertEquals("craft has correct shields", qtyShields, craft.getShields().getAllAsArray().length);
            assertEquals("craft has correct shields", shield, craft.getShields().getAllAsArray()[0]);
            assertEquals("craft has correct shields", qtyShields, bc.getShields().length);
            assertEquals("craft has correct shields", shield, bc.getShields()[0].getShield());
        } else {
            assertEquals("craft has correct shields", true, craft.getShields() == null || craft.getShields().getQuantity() == 0);
            assertEquals("craft has correct shields", true, bc.getShields() == null);
        }
        
        if (qtySpecials > 0) {
            assertEquals("craft has correct specials", qtySpecials, craft.getSpecials().getAllAsArray().length);
            assertEquals("craft has correct specials", special, craft.getSpecials().getAllAsArray()[0]);
        } else {
            assertEquals("craft has correct specials", true, craft.getSpecials() == null || craft.getSpecials().getQuantity() == 0);
        }
        
    }

    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponDetonator() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponDetonator() *************************");
		CraftDesign suicideDesign = new CraftDesign("SuicideFighter", 
				tech_flyer30,
				tech_fusion25,
				tech_electronic25,
				tech_energy10,
				tech_detonator);
        createMapAndTwoCraft(suicideDesign, 600, fighterDesign, 700, null);

        bca[1].getShields()[0].setPower(0.0);


        //
        // TEST CRAFT DOES NOT DETONATE USING fireAllWeapons()
        //

        bca[0].fireAllWeapons(bca[1]);
        assertEquals("Has fired weapons", true, bca[0].isSuicideFighter());
        assertEquals("Has fired weapons", true, bca[0].hasWeaponsToFire());
        assertEquals("Has fired weapons", false, bca[0].isDestroyed());

        //
        // TEST CRAFT DETONATES
        //

        assertEquals("Has correct weapons", tech_detonator, bca[0].getWeapons()[0].getWeapon());
        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], bca[0].getWeapons()[0], 1);
        assertEquals("Has fired weapons", true, bca[0].isDestroyed());
        assertEquals("bca[1] not damaged becuase it is out of range", 0.0, bca[1].getDamage(), 0.001);
        
        //
        // TEST WHEN DETONATOR IS USED CORRECT DAMAGE IS PERFORMED
        //
        
        bca[0] = battle.join(newCraft(suicideDesign, player1, ss2), true);
        bca[0].setLocation(new Location(650, 500));
        BattleCraft newCraft = battle.join(newCraft(fighterDesign, player1, ss2), true);
        newCraft.setLocation(new Location(650, 480));

        bca[0].rechargeWeapons();
        bca[0].fireWeapons(bca[1], bca[0].getWeapons()[0], 1);
        assertEquals("Has correct damage", 25.0, bca[1].getDamage(), 0.01);
        assertEquals("Has correct damage", 54.0, newCraft.getDamage(), 0.01);
    }
        
    public void testDroidMine() throws Exception {
        Logger.logDebug("*******************************  WarTest.testDroidMine() *************************");
		CraftDesign cd = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_droidMine);
        createMapAndThreeCraft(cd, fighterDesign, fighterDesign, null);

        bca[0].setLocation(new Location(200, 500));
        bca[1].setLocation(new Location(800, 500));
        
        //
        // CANNOT FIRE DROID MINE AT TARGET OUT OF RANGE.
        //
        
        qtyDroidMinesLaunched = 0;
        assertTrue("Craft is out of weapons range", !bca[0].inWeaponRange(bca[1].getLocation()));
        try {
        	new BattleAttackCmd(player1, battle, bca[0], bca[1], false, false, false, null, tech_droidMine, 1, false).run();
        	fail("Out of Range");
        } catch (ValidationError ex) { }
        assertEquals("Craft is out of weapons range", 0, qtyDroidMinesLaunched);

        bca[0].setLocation(new Location(400, 500));
        
        //
        // CAN FIRE DROID MINE AT TARGET IN RANGE. 
        //

        qtyDroidMinesLaunched = 0;
        qtyFired = 0;
        new BattleAttackCmd(player1, battle, bca[0], bca[1], false, false, false, null, tech_droidMine, 1, false).run();
        
        assertTrue("Craft is in weapons range", bca[0].inWeaponRange(bca[1].getLocation()));
        assertEquals("Craft is in weapons range", 0, qtyFired);
        assertEquals("Craft is in weapons range", 1, qtyDroidMinesLaunched);
        Automaton [] automitons = battle.getAutomatons();
        assertEquals("Battle now has a missile", 1, automitons.length);
        DroidMine droidMine = (DroidMine) automitons[0];
        assertEquals("DroidMine are correct", bca[1], droidMine.getTarget());
        assertEquals("DroidMine are correct", 50, droidMine.getSpeed());
        assertEquals("DroidMine are correct", bca[0], droidMine.getOrigin());
        assertEquals("DroidMine are correct", player1, droidMine.getOwner());
        assertEquals("DroidMine are correct", 1, droidMine.getWeapons()[0].getQty());
        assertEquals("DroidMine are correct", tech_detonator, droidMine.getWeapons()[0].getWeapon());
        assertEquals("DroidMine are correct", 400, droidMine.getLocation().getDistanceTo(droidMine.getTarget().getLocation()));
        
        //
        // DROID MINE MOVES TOWARDS TARGET. 
        //

        bca[1].moveLinkedObjects(1.0, null, null);
        automitons = battle.getAutomatons();
        droidMine = (DroidMine) automitons[0];
        assertEquals("Missile has not struck target", 1, automitons.length);
        assertEquals("Craft2 has not been attacked", 0.0, craft2.getDamage(), 0.001);
        assertEquals("DroidMine has towards Craft2", 350, droidMine.getLocation().getDistanceTo(droidMine.getTarget().getLocation()));
        
        //
        // DROID MINE ATTACKS TARGET WHEN IN RANGE. 
        //

        bca[1].setLocation(new Location(460, 500));
        assertEquals("DroidMine has towards Craft2", 10, droidMine.getLocation().getDistanceTo(droidMine.getTarget().getLocation()));

        droidMine.startTurn();
        bca[1].moveLinkedObjects(1.0, null, null);
        assertEquals("Craft2 has been attacked", 76.0, craft2.getDamage(), 2.0);
        assertEquals("Craft2 has been attacked", 0.0, bca[1].getShieldPower(), 0.001);
        automitons = battle.getAutomatons();
        assertEquals("DroidMine has been destroyed", null, automitons);
        
        //
        // DROID MINE EXPLODES DAMAGE ALL CRAFT WHEN DESTROYED 
        //
        
        bca[0].setLocation(new Location(200, 500));
        bca[2].setLocation(new Location(300, 500));
        craft1.setDamage(0.0);
        craft3.setDamage(0.0);
        bca[0].setShieldPower(0, 0);
        bca[2].setShieldPower(0, 0);

        qtyDroidMinesLaunched = 0;
        qtyFired = 0;
        bca[0].rechargeWeapons();
        new BattleAttackCmd(player1, battle, bca[0], bca[2], false, false, false, null, tech_droidMine, 1, false).run();
        automitons = battle.getAutomatons();
        automitons[0].setLocation(new Location(250,500));
        droidMine = (DroidMine) automitons[0];
        droidMine.damage(droidMine.getDamageToDestroy()-0.1, player2, null, null);
        bca[2].fireAllWeapons(droidMine);

        assertEquals("Craft1 has been attacked", 25.0, craft1.getDamage(), 0.001);
        assertEquals("Craft3 has been attacked", 25.0, craft3.getDamage(), 0.001);
        automitons = battle.getAutomatons();
        assertEquals("DroidMine has been destroyed", null, automitons);
    }

    public void testDoubleDroidMine() throws Exception {
        Logger.logDebug("*******************************  WarTest.testDoubleDroidMine() *************************");
		CraftDesign cd1 = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_armour200, tech_missile6, tech_droidMine, tech_droidMine);
		CraftDesign cd2 = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_armour200, tech_missile6);
        createMapAndThreeCraft(cd1, cd2, fighterDesign, null);

        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(500, 500));
        bca[2].setLocation(new Location(450, 450));
        bca[2].getCraft().setDamage(bca[2].getCraft().getMaxDamage()-1);
        
        bca[1].fireAllWeapons(bca[0]);
        Automaton [] automitons = battle.getAutomatons();
        assertEquals("Battle now has a missile", 1, automitons.length);
        Missiles missile1 = (Missiles) automitons[0];
        missile1.setLocation(new Location(425,500));
        //
        // CAN FIRE DROID MINE AT TARGET IN RANGE. 
        //

        new BattleAttackCmd(player1, battle, bca[0], bca[1], false, false, false, null, tech_droidMine, 2, false).run();
        
        automitons = battle.getAutomatons();
        assertEquals("Battle now has a missile", 3, automitons.length);
        DroidMine droidMine1 = (DroidMine) automitons[1];
        assertEquals("DroidMine1", droidMine1.getName());
        assertEquals("DroidMine are correct", bca[1], droidMine1.getTarget());
        assertEquals("DroidMine are correct", 50, droidMine1.getSpeed());
        assertEquals("DroidMine are correct", bca[0], droidMine1.getOrigin());
        assertEquals("DroidMine are correct", player1, droidMine1.getOwner());
        assertEquals(400, 500, droidMine1.getLocation());
        
        DroidMine droidMine2 = (DroidMine) automitons[2];
        assertEquals("DroidMine2", droidMine2.getName());
        assertEquals("DroidMine are correct", bca[1], droidMine2.getTarget());
        assertEquals("DroidMine are correct", 50, droidMine2.getSpeed());
        assertEquals("DroidMine are correct", bca[0], droidMine2.getOrigin());
        assertEquals("DroidMine are correct", player1, droidMine2.getOwner());
        assertEquals(400, 500, droidMine2.getLocation());
        
        BattleDroidMineLaunchEvent dmle1 = (BattleDroidMineLaunchEvent) getEventFromFront(BattleDroidMineLaunchEvent.class, 0);
        assertEquals("DroidMine1", dmle1.getDroidMine().getName());
        assertEquals(400,500, dmle1.getDroidMine().getState().getLocation());
        assertEquals(bca[0], dmle1.getDroidMine().getOrigin());
        assertEquals(bca[1], dmle1.getDroidMine().getTarget());
        
        BattleDroidMineLaunchEvent dmle2 = (BattleDroidMineLaunchEvent) getEventFromFront(BattleDroidMineLaunchEvent.class, 1);
        assertEquals("DroidMine2", dmle2.getDroidMine().getName());
        assertEquals(400,500, dmle2.getDroidMine().getState().getLocation());
        assertEquals(bca[0], dmle2.getDroidMine().getOrigin());
        assertEquals(bca[1], dmle2.getDroidMine().getTarget());
       
        //
        // DROID MINE ATTACKS TARGET WHEN IN RANGE. 
        //

        clearEvents();
        bca[1].moveTo(new LocationAndDirection(450, 500, 90), 1.0);
        
        assertEquals("Craft1 has been attacked", 99.0, craft1.getDamage(), 0.1);
        assertEquals("Craft1 has been attacked", 0.0, bca[0].getShieldPower(), 0.001);
        
        assertEquals("Craft2 has been attacked", 101.0, craft2.getDamage(), 2.0);
        assertEquals("Craft2 has been attacked", 0.0, bca[1].getShieldPower(), 0.001);
        
        assertEquals("Craft2 has not been attacked", 43.0, craft3.getDamage(), 0.1);
        assertEquals("Craft2 has not been attacked", 0.0, bca[2].getShieldPower(), 0.001);
        
        automitons = battle.getAutomatons();
        assertEquals("DroidMine has been destroyed", null, automitons);
        
        BattleCraftMoveEvent  bcme = (BattleCraftMoveEvent) getEventFromFront(BattleCraftMoveEvent.class, 0);
        
        BattleDroidMineMoveEvent  dmme1 = (BattleDroidMineMoveEvent) bcme.getLinkedEvents()[0];
		assertEquals("DroidMine1", dmme1.getDroidMine());
        assertEquals(400, 500, dmme1.getFrom());
        assertEquals(450, 500, dmme1.getTo());
        
//        BattleDroidMineMoveEvent  dmme2 = (BattleDroidMineMoveEvent) bcme.getLinkedEvents()[1];
//		assertEquals("DroidMine2", dmme2.getDroidMine());
//        assertEquals(400, 500, dmme2.getFrom());
//        assertEquals(450, 500, dmme2.getTo());
        
		BattleDamageRingEvent dre1 = (BattleDamageRingEvent) getEventFromFront(BattleDamageRingEvent.class, 0);
		assertEquals("DroidMine1", dre1.getSource());
		assertEquals(450,500, dre1.getLocation());
		assertEquals("TestFighter1", ((BattleDamageEvent)dre1.getLinkedEvents()[0]).getTarget());
		assertEquals(400,500, ((BattleDamageEvent)dre1.getLinkedEvents()[0]).getOldTargetState().getLocation());
		assertEquals(400,500, ((BattleDamageEvent)dre1.getLinkedEvents()[0]).getNewTargetState().getLocation());
		assertEquals("TestFighter2", ((BattleDamageEvent)dre1.getLinkedEvents()[1]).getTarget());
		assertEquals(450,500, ((BattleDamageEvent)dre1.getLinkedEvents()[1]).getOldTargetState().getLocation());
		assertEquals(450,500, ((BattleDamageEvent)dre1.getLinkedEvents()[1]).getNewTargetState().getLocation());
		assertEquals("TestFighter3", ((BattleDestroyedEvent)dre1.getLinkedEvents()[2]).getTarget());
		assertEquals(450,450, ((BattleDestroyedEvent)dre1.getLinkedEvents()[2]).getOldState().getLocation());
		assertEquals("TestFighter3", ((BattleDamageEvent)dre1.getLinkedEvents()[3]).getTarget());
		assertEquals(450,450, ((BattleDamageEvent)dre1.getLinkedEvents()[3]).getOldTargetState().getLocation());
		assertEquals(450,450, ((BattleDamageEvent)dre1.getLinkedEvents()[3]).getNewTargetState().getLocation());
		assertEquals("Missile1", ((BattleDestroyedEvent)dre1.getLinkedEvents()[4]).getTarget());
		assertEquals(425,500, ((BattleDestroyedEvent)dre1.getLinkedEvents()[4]).getOldState().getLocation());
		assertEquals("Missile1", ((BattleDamageEvent)dre1.getLinkedEvents()[5]).getTarget());
		assertEquals(425,500, ((BattleDamageEvent)dre1.getLinkedEvents()[5]).getOldTargetState().getLocation());
		assertEquals(425,500, ((BattleDamageEvent)dre1.getLinkedEvents()[5]).getNewTargetState().getLocation());
		assertEquals("DroidMine2", ((BattleDamageEvent)dre1.getLinkedEvents()[6]).getTarget());
		assertEquals(400,500, ((BattleDamageEvent)dre1.getLinkedEvents()[6]).getOldTargetState().getLocation());
		assertEquals(400,500, ((BattleDamageEvent)dre1.getLinkedEvents()[6]).getNewTargetState().getLocation());
		  
		BattleDamageRingEvent dre2 = (BattleDamageRingEvent) getEventFromFront(BattleDamageRingEvent.class, 1);
		assertEquals("DroidMine2", dre2.getSource());
		assertEquals(400,500, dre2.getLocation());
		assertEquals("TestFighter1", ((BattleDamageEvent)dre2.getLinkedEvents()[0]).getTarget());
		assertEquals(400,500, ((BattleDamageEvent)dre2.getLinkedEvents()[0]).getOldTargetState().getLocation());
		assertEquals(400,500, ((BattleDamageEvent)dre2.getLinkedEvents()[0]).getNewTargetState().getLocation());
		assertEquals("TestFighter2", ((BattleDamageEvent)dre2.getLinkedEvents()[1]).getTarget());
		assertEquals(450,500, ((BattleDamageEvent)dre2.getLinkedEvents()[1]).getOldTargetState().getLocation());
		assertEquals(450,500, ((BattleDamageEvent)dre2.getLinkedEvents()[1]).getNewTargetState().getLocation());
      
    }

    /**
     * Tests weapons on crafts that are not beam weapons or missiles.
     */
    public void testWeaponPulse() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponDetonator() *************************");
		CraftDesign pulseDesign = new CraftDesign("PulseFighter", 
				tech_flyer60,
				tech_fusion25,
				tech_electronic25,
				tech_pulse);
		CraftDesign fd = new CraftDesign("TestFighter", 
				tech_flyer30,
				tech_fusion25,
				tech_electronic25,
				tech_energy10,
				tech_massDriver1,
				tech_missile6);
        createMapAndCrafts(pulseDesign, null, fd, fd, fd, fd);

        assertEquals(craft1, bca[0].getCraft());
        assertEquals(enemyCraft[0], bca[1].getCraft());
        assertEquals(enemyCraft[1], bca[2].getCraft());
        assertEquals(enemyCraft[2], bca[3].getCraft());
        assertEquals(enemyCraft[3], bca[4].getCraft());

        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(550, 500)); // Damaged
        bca[2].setLocation(new Location(550, 460)); // Damaged
        bca[3].setLocation(new Location(550, 540)); // Damaged from behind
        bca[4].setLocation(new Location(650, 500)); // Undamaged out of range
        bca[0].setDirection(90);
        bca[1].setDirection(270);
        bca[2].setDirection(270);
        bca[3].setDirection(90);
        bca[4].setDirection(270);

        //
        // FIRE PULSE WEAPON
        //

        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(false, bca[0].hasWeaponsToFire());
        assertEquals(false, bca[0].isDestroyed());
        assertEquals(0.0, bca[0].getDamage());
        assertEquals(0.0, bca[0].getShieldPower());

        assertEquals(false, bca[1].isDestroyed());
        assertEquals(7.28, bca[1].getDamage(),0.1);
        assertEquals(1.71, bca[1].getShieldPower(),0.1);

        assertEquals(false, bca[2].isDestroyed());
        assertEquals(7.28, bca[2].getDamage(),0.1);
        assertEquals(1.71, bca[2].getShieldPower(),0.1);

        assertEquals(false, bca[3].isDestroyed());
        assertEquals(14.56, bca[3].getDamage(),0.1);
        assertEquals(1.71, bca[3].getShieldPower(),0.1);

        assertEquals(false, bca[4].isDestroyed());
        assertEquals(0.0, bca[4].getDamage());
        assertEquals(10.0, bca[4].getShieldPower());


        //
        // FIRE PULSE WEAPON AGAIN - with one craft DESTROYED
        // Missile also destroyed
        //
        
        Logger.logDebug("FIRE MISSILE");
        bca[2].fireWeapons(bca[0], bca[2].getWeapon(tech_missile6), -1);

        assertEquals(550, 460, ((Missiles)battle.getAutomatons()[0]).getLocation());
        ((Missiles)battle.getAutomatons()[0]).setDamage(Missiles.DAMAGE_TO_DESTROY_A_MISSILE-1);
        
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);

        assertEquals(false, bca[0].hasWeaponsToFire());
        assertEquals(false, bca[0].isDestroyed());
        assertEquals(0.0, bca[0].getDamage());
        assertEquals(0.0, bca[0].getShieldPower());

        assertEquals(false, bca[1].isDestroyed());
        assertEquals(15.38, bca[1].getDamage(),0.1);
        assertEquals(0.0, bca[1].getShieldPower(),0.1);

        assertEquals(false, bca[2].isDestroyed());
        assertEquals(15.38, bca[2].getDamage(),0.1);
        assertEquals(0.0, bca[2].getShieldPower(),0.1);

        assertEquals(true, bca[3].isDestroyed());
        assertEquals(32.29, bca[3].getDamage(),0.1);
        assertEquals(0.0, bca[3].getShieldPower(),0.1);

        assertEquals(false, bca[4].isDestroyed());
        assertEquals(0.0, bca[4].getDamage());
        assertEquals(10.0, bca[4].getShieldPower());

        assertNull(battle.getAutomatons());

    }
        

	/**
     * Tests Bio weapons on crafts.
     */
    public void testWeaponBio() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponBio() *************************");
		CraftDesign bioDesign = new CraftDesign("BioFighter", 
				tech_flyer30,
				tech_fusion25,
				tech_electronic25,
				tech_energy10,
				tech_prioVirus, 
				tech_troopModule20);
		createMapAndTwoCraft(bioDesign, 500, fighterDesign, 600, null);

        WeaponInstanceBio weaponInstance = (WeaponInstanceBio) bca[0].getWeapons()[0];
        
        //
        // Make sure crafts have desired crew
        //
        assertEquals(30, bca[0].getCrew());
        assertEquals(10, bca[1].getCrew());

        //
        // Test has no effect when target has shields up
        //

        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(10, bca[1].getCrew());

        //
        // Test that weapon kills 5 people when shields are down
        //

        bca[1].getShields()[0].setPower(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[1], weaponInstance, 1);

        assertEquals(5, bca[1].getCrew());

        //
        // Test crafts do not move when they have no crew
        // Test craft does not surrender just because it has no crew
        //

        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[1], weaponInstance, 1);

        assertEquals(0, bca[1].getCrew());
        assertTrue(bca[1].hasMoved());
        
        bca[1].startTurn();
        assertTrue(bca[1].hasMoved());
        assertEquals(player2, craft2.getOwner());
        
        //
        // Test battle has ended when all enemy crafts are disabled
        // Test that crafts with no crew surrender to the victor player
        //
        
        assertEquals(true, battle.hasEnded());
        
        battle.endTurn();
        
        assertEquals(true, battle.isWinner(player1));
        assertEquals(false, battle.isWinner(player2));
        assertEquals(0, MotEngine.instance.getBattles().getQtyBattles());
        assertEquals(player1, craft2.getOwner());
    }

        
    /**
     * Tests Troop Transport weapons on crafts.
     * Tests that a craft can be conquered and changes sides.
     */
    public void testWeaponTroopTransport() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponTroopTransport() *************************");
		CraftDesign bioDesign = new CraftDesign("BioFighter", 
				tech_flyer30,
				tech_fusion25,
				tech_electronic25,
				tech_energy10,
				tech_troopTransport,
				tech_troopModule20);
		createMapAndTwoCraft(bioDesign, fighterDesign, null);

        Craft craft3 = newCraft(fighterDesign, player2, ss2);
        battle.join(craft3, false);

        bca = battle.getCrafts();
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[2].setLocation(new Location(630, 500));
        WeaponInstanceTroopTransport weaponInstance = (WeaponInstanceTroopTransport) bca[0].getWeapons()[0];
        
        //
        // Make sure crafts have desired crew
        //
        assertEquals(30, bca[0].getCrew());
        assertEquals(10, bca[1].getCrew());

        //
        // Test has no effect when target has shields up
        //

        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(10, bca[1].getCrew());

        //
        // Test that weapon kills 5 people when shields are down
        //

        Utils.setNextRandomDouble(1.0);
        bca[1].getShields()[0].setPower(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[1], weaponInstance, 1);

        assertEquals(20, bca[0].getCrew());
        assertEquals(3, bca[1].getCrew());
        assertEquals(player2, bca[1].getOwner());

        //
        // Test crafts surrender once conquered
        //
        
        clearEvents();

        Utils.setNextRandomDouble(1.0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[1], weaponInstance, 1);

        assertEquals(10, bca[0].getCrew());
        assertEquals(6, bca[1].getCrew());
        assertEquals(player1, bca[1].getOwner());
        
        assertEquals(tech_troopTransport, ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getWeaponTechnology());
        assertEquals("Bears", ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getOldTargetState().getOwner());
        assertEquals("Humans", ((BattleWeaponFireEvent)getEvent(BattleWeaponFireEvent.class)).getNewTargetState().getOwner());
        
        //
        // Give player 1 better ground assault weapons AND an attack bonus
        // Attacker Bonus is 2.5 meaning only 3 attackers required to kill 10 defenders - target is conquered!
        //
        
        player1.addWeapon(tech_fusionBeam4);
        player1.getPlayerCharacter().setAssaultBonus(2.0);

        Utils.setNextRandomDouble(1.0);
        bca[2].getShields()[0].setPower(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[2], weaponInstance, 1);

        assertEquals(1, bca[0].getCrew());
        assertEquals(6, bca[1].getCrew());
        assertEquals(player1, bca[1].getOwner());
        assertEquals(bca[1], battle.getCraft(bca[1].getName()));
    }
        
    /**
     * Tests Plague weapons on crafts.
     */
    public void testWeaponPlague() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponPlague() *************************");
        createMap(null);

		CraftDesign bioDesign = new CraftDesign("BioFighter", 
				tech_flyer30,
				tech_fusion25,
				tech_electronic25,
				tech_energy10,
				tech_plague,
				tech_troopModule20);

		Craft craft1 = newCraft(bioDesign, player1, ss2);
        Craft craft2 = newCraft(fighterDesign, player2, ss2);
        Craft craft3 = newCraft(fighterDesign, player2, ss2);
        Craft craft4 = newCraft(fighterDesign, player2, ss2);
        Craft craft5 = newCraft(fighterDesign, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.join(craft3, false);
        battle.join(craft4, false);
        battle.join(craft5, false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bca = battle.getCrafts();
        setLocationAndDirection(bca[0], new Location(500, 500), 90, craft1);
        setLocationAndDirection(bca[1], new Location(600, 500), 270, craft2);
        setLocationAndDirection(bca[2], new Location(700, 400), 270, craft3);
        setLocationAndDirection(bca[3], new Location(700, 500), 270, craft4);
        setLocationAndDirection(bca[4], new Location(700, 550), 270, craft5);
        WeaponInstancePlague weaponInstance = (WeaponInstancePlague) bca[0].getWeapons()[0];
        
        //
        // Make sure crafts have desired crew
        //
        assertEquals(30, bca[0].getCrew());
        assertEquals(10, bca[1].getCrew());
        assertEquals(10, bca[4].getCrew());

        //
        // Test has no effect when target has shields up
        //

        bca[0].fireWeapon(bca[1], weaponInstance, 1);
        
        assertEquals(10, bca[1].getCrew());
        assertEquals(null, battle.getTriggers().getTriggers());

        //
        // Test that weapon kills 0 people when shields are down
        // But creates a trigger to kill people at the end of the turn
        //

        bca[1].getShields()[0].setPower(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[1], weaponInstance, 1);
        Plague trigger = (Plague) battle.getTriggers().getTriggers()[0];

        assertEquals(10, bca[1].getCrew());
        assertEquals(5, trigger.getQtyKilledPerTurn());
        assertEquals(bca[1], trigger.getBattleObject());
        assertEquals(player1, trigger.getOwner());
        
        //
        // Test at the end of a battle turn people are killed AND plague spreads
        //
        
        lastTriggerEvent = null;
        notificationEvent = null;
        
        battle.endTurn();
        
        assertEquals(5, bca[1].getCrew());
        assertEquals(bca[1], lastTriggerEvent.getBattleObject());
        assertEquals(1.0, lastTriggerEvent.getOriginalState().getCrewPercent(), 0.001);
        assertEquals(0.5, lastTriggerEvent.getNewState().getCrewPercent(), 0.001);

        assertEquals(bca[3], notificationEvent.getTarget());
        assertEquals("Plague Spread", notificationEvent.getMessage());
        assertEquals(10, bca[3].getCrew());

        //
        // Test Plague does not spread because closest craft already has the plague
        //
        
        lastTriggerEvent = null;
        notificationEvent = null;
        
        battle.endTurn();
        
        assertEquals(0, bca[1].getCrew());
        assertEquals(5, bca[3].getCrew());
        assertEquals(bca[3], lastTriggerEvent.getBattleObject());

        assertEquals(bca[4], notificationEvent.getTarget());
        assertEquals("Plague Spread", notificationEvent.getMessage());
        assertEquals(10, bca[4].getCrew());
    }
        

	/**
     * Tests Damage Bot weapons on crafts.
     */
    public void testDamageBots() throws Exception {
        Logger.logDebug("*******************************  WarTest.testDamageBots() *************************");
        CraftDesign cd = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_damageBots);
        createMapAndTwoCraft(cd ,500 , fighterDesign, 600, null);

        assertEquals("Correct Damage", 5.0, tech_damageBots.getDamage(), 0.01);
        WeaponInstanceDamageBot weaponInstance = (WeaponInstanceDamageBot) bca[0].getWeapons()[0];
        
        //
        // Test has no effect when target has shields up
        //

        bca[0].fireWeapon(bca[1], weaponInstance, 1);
        
        assertEquals(0.0, bca[1].getDamage(), 0.001);
        assertEquals(true, battle.getTriggers() == null || battle.getTriggers().getTriggers() == null);

        //
        // Test that weapon deploys bots when shields are down
        // But creates a trigger to kill people at the end of the turn
        //

        bca[1].getShields()[0].setPower(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[1], weaponInstance, 1);
        DamageBot trigger = (DamageBot) battle.getTriggers().getTriggers()[0];

        assertEquals(0.0, bca[1].getDamage(), 0.001);
        assertEquals(5, trigger.getDamage());
        assertEquals(bca[1], trigger.getBattleObject());
        assertEquals(player1, trigger.getOwner());
        
        //
        // Test at the end of a battle turn people are killed
        //
        
        battle.endTurn();
        
        assertEquals(5.0, bca[1].getDamage(), 0.001);
        assertEquals(bca[1].getName(), lastTriggerEvent.getBattleObject());
        assertEquals(0.0, lastTriggerEvent.getOriginalState().getDamagePercent(), 0.001);
        assertEquals(0.25, lastTriggerEvent.getNewState().getDamagePercent(), 0.001);

    }
        
    /**
     * Tests Time Warp weapons on crafts.
     */
    public void testTimeWarp() throws Exception {
        Logger.logDebug("*******************************  WarTest.testTimeWarp() *************************");
        CraftDesign timeWarpFighter = new CraftDesign("TimeFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_massDriver1, tech_timeWarpGenerator);
        createMapAndTwoCraft(timeWarpFighter, timeWarpFighter, null);

        WeaponInstanceBeam wiMassDriver = (WeaponInstanceBeam) bca[0].getWeapons()[0];
        WeaponInstanceTimeWarp wiTimeWarp = (WeaponInstanceTimeWarp) bca[0].getWeapons()[1];

        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(90);
        bca[1].setLocation(new Location(600, 500));
        
        //
        // Test when time warp weapon fires movement is restored and weapons recharged
        //

        bca[0].moveTo(bca[1]);
        bca[0].fireWeapon(bca[1], wiMassDriver, 1);
        wiTimeWarp.discharge();

        assertEquals(new Location(500,500), bca[0].getLocation());
        assertEquals(0.0, bca[0].getMoveRemaining(), 0.001);
        assertEquals(false, bca[0].hasWeaponsToFire());
        assertEquals(true, bca[0].hasMoved());
        assertEquals(1.0, bca[1].getDamage(), 0.001);
        assertEquals(8.0, bca[1].getShieldPower(), 0.001);

        wiTimeWarp.recharge();
        bca[0].fireWeapon(bca[1], wiTimeWarp, 1);
        
        assertEquals(new Location(500,500), bca[0].getLocation());
        assertEquals(1.0, bca[0].getMoveRemaining(), 0.001);
        assertEquals(true, bca[0].hasWeaponsToFire());
        assertEquals(false, bca[0].hasMoved());

        bca[0].fireWeapon(bca[1], wiMassDriver, 1);

        assertEquals(2.035, bca[1].getDamage(), 0.001);
        assertEquals(5.965, bca[1].getShieldPower(), 0.001);

        //
        // Test Mass Driver has fired twice
        //

        bca[1].getCraft().setDamage(0);
        bca[1].rechargeShieldsFull();
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals(2.034, bca[1].getDamage(), 0.001);
        assertEquals(5.965, bca[1].getShieldPower(), 0.001);
        
        // Test User Commands fire time warp beam at end of turn AND allow craft to move again
        // Tests command does not discharge weapons at end of turn
        
        bca[0].setLocation(new Location(400, 500));
        bca[0].setDirection(90);
        bca[0].rechargeWeapons();
        bca[0].setMoveRemaining(1.0);

        bca[1].setLocation(new Location(600, 500));
        bca[1].dischargeShields();
        bca[1].getCraft().setDamage(0);
        
        new BattleCraftMoveToCmd(player1, battle, bca[0], new LocationAndDirection(600,500,90), null, false, false, false).run();
        new BattleAttackCmd(player1, battle, bca[0], bca[1], true, true, true, null, null, 0, false).run();
        
        assertEquals(500, 500, bca[0].getLocation());
        assertEquals(2.0, bca[1].getDamage());
        assertEquals(1.0, bca[0].getMoveRemaining());
        assertEquals(true, bca[0].hasWeaponsToFire());

        // Test User Commands fire time warp beam at end of turn AND allow craft to move again
        // Tests commander does not discharge weapons at end of turn
        
//        bca[0].setLocation(new Location(400, 500));
//        bca[0].setDirection(90);
//        bca[0].dischargeShields();
//        bca[0].getCraft().setDamage(0);
//
//        bca[1].setLocation(new Location(600, 500));
//        bca[1].setDirection(270);
//        bca[1].rechargeWeapons();
//        bca[1].setMoveRemaining(1.0);
//        
//        player2.getCommander().assignTaskToCraft(bca[1]);
//        
//        assertEquals(503, 500, bca[1].getLocation());
//        assertEquals(1.7, bca[0].getDamage(), 0.1);
//        assertEquals(1.0, bca[1].getMoveRemaining());
//        assertEquals(true, bca[1].hasWeaponsToFire());
    }
        
    /**
     * Tests that when a player is immune to a type of attack then those weapons have 
     * no effect on that players crafts.
     */
    public void testImmunity() throws Exception {
        Logger.logDebug("*******************************  WarTest.testImmunity() *************************");
		CraftDesign bioDesign = new CraftDesign("BioFighter", 
				tech_flyer30,
				tech_fusion25,
				tech_electronic25,
				tech_energy10,
				tech_prioVirus, tech_troopTransport, tech_plague, tech_psiBlast, tech_girator);
        createMapAndTwoCraft(bioDesign, 500, fighterDesign, 600, null);

        bca = battle.getCrafts();
        
        assertEquals(10, bca[1].getCrew());
        assertEquals(true, bca[0].getWeapons()[0] instanceof WeaponInstancePsi);
        assertEquals(true, bca[0].getWeapons()[1] instanceof WeaponInstanceBio);
        assertEquals(true, bca[0].getWeapons()[3] instanceof WeaponInstancePlague);
        assertEquals(true, bca[0].getWeapons()[4] instanceof WeaponInstanceGirator);

        bca[1].getShields()[0].setPower(0.0);

        //
        // Test that Bio Weapons has no effect on target craft
        //

        player2.getPlayerCharacter().setImmunity(new ImmunityBio());
        bca[0].fireWeapons(bca[1], bca[0].getWeapons()[1], 1);
        assertEquals(10, bca[1].getCrew());
        assertEquals("PrioVirus Immune", weaponFireEvent.getTargetDescription());
        bca[0].fireWeapons(bca[1], bca[0].getWeapons()[3], 1);
        assertNull(battle.getTriggers().getTriggers());
        assertEquals("Plague Immune", weaponFireEvent.getTargetDescription());


        //
        // Test that Psi Weapons has no effect on target craft
        //

        player2.getPlayerCharacter().setImmunity(new ImmunityPsi());
        bca[0].fireWeapons(bca[1], bca[0].getWeapons()[0], 1);
        assertEquals(0.0, bca[1].getPsiDamage(), 0.0);
        assertEquals("PsiBlast Immune", weaponFireEvent.getTargetDescription());


        //
        // Test that Gravity Weapons has no effect on target craft
        //

        player2.getPlayerCharacter().setImmunity(new ImmunityGravity());
        bca[0].fireWeapons(bca[1], bca[0].getWeapons()[4], 1);
        assertEquals(0.0, bca[1].getDamage(), 0.0);
        assertEquals("Girator Immune", weaponFireEvent.getTargetDescription());
    }
    
    public void testWeaponRetargetBeam() throws Exception {
        Logger.logDebug("*******************************  WarTest.testRetargetBeam() *************************");
        CraftDesign missileDesign = new CraftDesign("MF1", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_missile6);
        CraftDesign retargetDesign = new CraftDesign("RF1", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_retargetBeam);
        createMapAndTwoCraft(missileDesign, 200, retargetDesign, 500, null);

        //
        // Craft1 FIRE MISSILE at craft2 - Craft2 will retarget this missile! 
        //
        
        assertTrue(bca[0].inWeaponRange(bca[1].getLocation()));
        bca[0].fireAllWeapons(bca[1]);
        
        Automaton [] automitons = battle.getAutomatons();
        assertEquals(1, automitons.length);
        Missiles missile = (Missiles) automitons[0];
        assertEquals(bca[0], missile.getOrigin());
        assertEquals(bca[1], missile.getTarget());
        assertEquals(250, missile.getSpeed());
        assertEquals(player1, missile.getOwner());
        assertEquals(1, missile.getWeapons()[0].getQty());
        assertEquals(tech_missile6, missile.getWeapons()[0].getWeapon());
        assertEquals(300, missile.getLocation().getDistanceTo(missile.getTarget().getLocation()));

        //
        // RETARGET FIRES - OUT OF RANGE 
        //

        bca[1].fireAllWeapons(missile);
        
        assertEquals(bca[0], missile.getOrigin());
        assertEquals(bca[1], missile.getTarget());
        
        
        //
        // CRAFT2 FIRES RETARGET BEAM AT MISSILE - RETARGET TO CRAFT1
        //

        missile.setLocation(new Location(450,500));
        
        Utils.clearAndSetNextRandomProbability(0.1);
        bca[1].rechargeWeapons();
        bca[1].fireAllWeapons(missile);
        
        automitons = battle.getAutomatons();
        missile = (Missiles) automitons[0];
        assertEquals(bca[0], missile.getOrigin());
        assertEquals(bca[0], missile.getTarget());
        assertEquals(270, missile.getDirection());
        assertEquals(450,500, missile.getLocation());
        assertEquals(false, bca[1].hasWeaponsToFire());
        
        
        //
        // Check movement of missile is now linked to craft1
        //
        
        missile.setMoveRemaining(1.0);
        bca[0].setDirection(90);
        bca[0]._setMoveRemaining(1.0);
        bca[0]._moveTo(new Location(210,500));
        
        assertEquals(210,500, missile.getLocation());
        assertEquals(5.0, bca[0].getDamage());
        assertEquals(true, missile.isDestroyed());
        
        //
        // Check retarget beam is in exclude list for weapons fires at a craft
        //
        
        assertEquals(false, bca[1].getWeapon(tech_retargetBeam).hasWeaponsToFire());
        
        bca[1].rechargeWeapons();
        bca[1].fireAllWeapons(bca[0]);
        
        assertEquals(true, bca[1].getWeapon(tech_retargetBeam).hasWeaponsToFire());
        
    }

    public void testWeaponAntiMissileBeam() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponAntiMissileBeam() *************************");
        CraftDesign missileDesign = new CraftDesign("MF1", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_missile6);
        CraftDesign retargetDesign = new CraftDesign("AMF1", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_antiMissileBeam);
        createMapAndTwoCraft(missileDesign, 200, retargetDesign, 500, null);

        //
        // Craft1 FIRE MISSILE at craft2 - Craft2 will fire missile beam at this missile! 
        //
        
        assertTrue(bca[0].inWeaponRange(bca[1].getLocation()));
        bca[0].fireAllWeapons(bca[1]);
        
        Automaton [] automitons = battle.getAutomatons();
        assertEquals(1, automitons.length);
        Missiles missile = (Missiles) automitons[0];
        assertEquals(bca[0], missile.getOrigin());
        assertEquals(bca[1], missile.getTarget());
        assertEquals(250, missile.getSpeed());
        assertEquals(player1, missile.getOwner());
        assertEquals(1, missile.getWeapons()[0].getQty());
        assertEquals(tech_missile6, missile.getWeapons()[0].getWeapon());
        assertEquals(300, missile.getLocation().getDistanceTo(missile.getTarget().getLocation()));

        //
        // RETARGET FIRES - OUT OF RANGE 
        //

        bca[1].fireAllWeapons(missile);
        
        assertEquals(false, missile.isDestroyed());
        assertEquals(0.0, missile.getDamage());
        assertEquals(bca[0], missile.getOrigin());
        assertEquals(bca[1], missile.getTarget());
        
        //
        // CRAFT2 FIRES MISSILE BEAM AT MISSILE
        //

        missile.setLocation(new Location(450,500));
        weaponFireEvent = null;
        
        bca[1].rechargeWeapons();
        bca[1].fireAllWeapons(missile);
        
        assertEquals(null, battle.getAutomatons());
        assertEmptyArray(bca[1].getMoveLinks());
        assertEquals(true, weaponFireEvent.getDestroyed());
        assertEquals("25.0 (1 destroyed)", weaponFireEvent.getTargetDescription());
        
        //
        // Check retarget beam is in exclude list for weapons fires at a craft
        //

        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        missile.setLocation(new Location(450,500));
        
        assertEquals(false, bca[1].getWeapon(tech_antiMissileBeam).hasWeaponsToFire());
        
        bca[1].rechargeWeapons();
        bca[1].fireAllWeapons(bca[0]);
        
        assertEquals(true, bca[1].getWeapon(tech_antiMissileBeam).hasWeaponsToFire());
        
    }


    /**
     * Tests that when a player is resourceful then his enginuity is used to recharge 
     * shields and repair damage from time to time.
     */
    public void testResourceful() throws Exception {
        Logger.logDebug("*******************************  WarTest.testResourceful() *************************");
        createMapAndTwoCraft(fighterDesign, 500, fighterDesign, 600, null);

        bca = battle.getCrafts();
        player1.getPlayerCharacter().setResourceful(true);
        
        double damage = craft1.getMaxDamage()*0.95;
        craft1.setDamage(damage);
        
        Utils.clearAndSetNextRandomProbability(0.1);        
        bca[0].startTurn();
        
        double repairedDamage = craft1.getMaxDamage()*0.1;
        assertEquals(damage-repairedDamage, craft1.getDamage(), 0.0);
        
        //
        // Test shields recharge by recharge rate (1) + 0.3 * Max Recharge (9) = 3
        //
        
        bca[0].getShields()[0].setPower(0.0);
        Utils.setNextRandomProbability(0.1);        
        bca[0].startTurn();

        assertEquals(3.0, bca[0].getShields()[0].getPower(), 0.0);
    }


    /**
     * Tests that
     *  - Holo crafts deploy at the start of a battle like normal craft.
     *  - Holo Crafts are controlled like normal crafts by computers players
     *  - Holo crafts are targeted like normal crafts by computer players
     *  - Chance that a player will fall for a holo craft again reduces in the next battle
     */
    public void testHoloCrafts() throws Exception {
        Logger.logDebug("*******************************  WarTest.testHoloCrafts() *************************");
		CraftDesign holoDesign =new CraftDesign("HoloFighter", 
				tech_flyer30,
				tech_fusion25,
				tech_electronic25,
				tech_energy10,
				tech_holoEmiter,tech_holoEmiter);
        createMapAndTwoCraft(holoDesign, fighterDesign, null);

        bca = battle.getCrafts();
        
        assertEquals(4, bca.length);
        assertEquals(tech_fakeWeapon, bca[2].getWeapons()[0].getWeapon());
        assertEquals(tech_fakeShield, bca[2].getShields()[0].getShield());
        assertEquals(tech_fakeWeapon, bca[3].getWeapons()[0].getWeapon());
        assertEquals(tech_fakeShield, bca[3].getShields()[0].getShield());
        
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[2].setLocation(new Location(550, 500));
        bca[3].setLocation(new Location(450, 500));
        
        //
        // Test that computer players attack and destroy holo crafts
        //

        lastCraftDestroyed = null;
        bca[1].getNextOrders();
        
        assertEquals(false, bca[0].isDestroyed());
        assertEquals(false, bca[1].isDestroyed());
        assertEquals(true, bca[2].isDestroyed());
        assertEquals(false, bca[3].isDestroyed());
        assertEquals(bca[2].getCraft().getName(), lastCraftDestroyed);
        assertEquals(null, battle.getWreckages().getWrecks()); // Holo craft do not cuase wreckage
        
        //
        // Test that computer players move holo crafts
        //

        bca[3].getNextOrders();
        
        assertEquals(550, bca[3].getLocation().getX());
        assertEquals(false, bca[3].hasWeaponsToFire());
        
        //
        // Test that when the parent craft of a hologram craft is destroyed then so 
        // is the holo craft
        //
        
        lastCraftDestroyed = null;
        bca[3].setLocation(new Location(450, 500));
        bca[0].getCraft().setDamage(bca[0].getCraft().getMaxDamage()-0.1);
        bca[0].getShields()[0].setPower(0.0);
        bca[1].rechargeWeapons();
        bca[1].getNextOrders();
        
        assertEquals(true, bca[0].isDestroyed());
        assertEquals(false, bca[1].isDestroyed());
        assertEquals(true, bca[2].isDestroyed());
        assertEquals(true, bca[3].isDestroyed());
        assertEquals(bca[3].getName(), lastCraftDestroyed);
        
        battle.automate();
        
        //
        // Test that player 2 is now less likely to fall for the holo craft trick
        // Test that a battle is created without holo craft
        //
        
        assertEquals(0.75, player2.getChanceHoloCraftWork(), 0.0);
        
        craft1 = newCraft(holoDesign, player1, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        Utils.setNextRandomProbability(0.9);
        battle.setup();
        bca = battle.getCrafts();
        
        assertEquals(2, bca.length);
        assertEquals(0.75, player2.getChanceHoloCraftWork(), 0.0);
    }

    
    /**
     * Tests that morale on a craft does change when enemy / friendly crafts are destroyed.
     */
    public void testMorale() throws Exception {
        Logger.logDebug("*******************************  WarTest.testMorale() *************************");
        createMap(null);

        Craft craft3;

        //
        // Test around a planet with a owner containing Planetary News System
        // Planet has morale +1 and enemy crafts haver morale -1
        //

        player2.addTechnology(tech_planetaryNewsService);
        
        craft1 = newCraft(fighterDesign, player1, ss2);
        craft2 = newCraft(fighterDesign, player1, ss2);
        craft3 = newCraft(fighterDesign, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, false);
        battle.join(planet2);
        battle.setup();
        
        bp = battle.getPlanet();
        bca = battle.getCrafts();
        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());
        assertEquals("craft3 is bca[2]", craft3, bca[2].getCraft());

        assertEquals(1.0, bp.getMorale(), 0.001);
        assertEquals(-1.0, bca[0].getMorale(), 0.001);
        assertEquals(-1.0, bca[1].getMorale(), 0.001);
        assertEquals(1.0, bca[2].getMorale(), 0.001);

        //
        // Test around a planet with a owner containing Planetary News System AND 
        // a craft has a fleet news service
        // Planet has morale 0 and enemy crafts haver morale 0
        //
        
        craft1.setSpecials(new Specials(tech_FleetNewsService));

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, false);
        battle.join(planet2);
        battle.setup();
        
        bp = battle.getPlanet();
        bca = battle.getCrafts();
        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());
        assertEquals("craft3 is bca[2]", craft3, bca[2].getCraft());

        assertEquals(0.0, bp.getMorale(), 0.001);
        assertEquals(0.0, bca[0].getMorale(), 0.001);
        assertEquals(0.0, bca[1].getMorale(), 0.001);
        assertEquals(0.0, bca[2].getMorale(), 0.001);

        //
        // Test around a planet with a craft that has a fleet news service
        // Planet has morale -1 and enemy crafts haver morale +1
        //
        
        player2.getMiscTechnologies().remove(tech_planetaryNewsService);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, false);
        battle.join(planet2);
        battle.setup();
        
        bp = battle.getPlanet();
        bca = battle.getCrafts();
        assertEquals("craft1 is bca[0]", craft1, bca[0].getCraft());
        assertEquals("craft2 is bca[1]", craft2, bca[1].getCraft());
        assertEquals("craft3 is bca[2]", craft3, bca[2].getCraft());

        assertEquals(-1.0, bp.getMorale(), 0.001);
        assertEquals(1.0, bca[0].getMorale(), 0.001);
        assertEquals(1.0, bca[1].getMorale(), 0.001);
        assertEquals(-1.0, bca[2].getMorale(), 0.001);

    }
    
    /**
     * Tests that the time warp weapon can resurrect a craft after it has been destroyed 
     */
    public void testWeaponTimeWarpBeam() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponTimeWarpBeam() *************************");
        CraftDesign timeWarpFighter = new CraftDesign("TimeFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_timeWarpBeam);
        createMapAndThreeCraft(timeWarpFighter, timeWarpFighter, fighterDesign, null);

        assertEquals(3.0, tech_timeWarpBeam.getDamage(), 0.01);
        
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[2].setLocation(new Location(550, 450));
        battle.setTurn(10);

        bca[2].destroy(null, null, null);
        WreckageCraft wreckage = battle.getWreckages().getWreckage(bca[2]);
        
        assertEquals(true, craft3.isDestroyed());
        assertEquals(true, bca[2].isDestroyed());
        assertEquals(bca[2], wreckage.getBattleCraft());
        assertEquals(true, bca[2].getArea().getMotRectangle().equals(wreckage.getRectangle()));
        assertEquals(10, wreckage.getTurnCreated());
        
        //
        // Test when craft2 fires Time warp beam at the wreckage of craft3 it is
        // resurected.....
        //

        WeaponInstance wi = bca[1].getWeapon(tech_timeWarpBeam);
        bca[1].fireWeapon(wreckage.getRectangle().getCenterLocation(), wi, 1);
        
        assertEquals(null, battle.getWreckages().getWrecks());
        assertEquals(false, craft3.isDestroyed());
        assertEquals(false, bca[2].isDestroyed());
        assertEquals(player2, bca[2].getOwner());
        assertEquals("The craft has been added to the battle", true, battle.getCraftList().contains(bca[2]));
        assertEquals("The craft has not been added to the player", 1, bca[2].getOwner().getQtyCrafts());
        assertEquals("Craft is repaired", 0, craft3.getDamage(), 0.01);
        assertEquals("Shields are recharged", 10, bca[2].getShieldPower(), 0.01);

        //
        // Test when craft2 fires Time warp beam at the wreckage of craft3 it is
        // not resurected because it was destroyed more then three turns ago
        //

        bca[2].destroy(null, null, null);
        battle.setTurn(14);
        bca[1].rechargeWeapons();
        wi = bca[1].getWeapon(tech_timeWarpBeam);
        bca[1].fireWeapon(wreckage.getRectangle().getCenterLocation(), wi, 1);
        
        assertEquals(1, battle.getWreckages().getWrecks().length);
        assertEquals(true, craft3.isDestroyed());
        assertEquals(true, bca[2].isDestroyed());
        assertEquals(false, battle.getCraftList().contains(bca[2]));

        //
        // Test when craft1 fires Time warp beam at the wreckage of an enemy craft 
        // player1 controls the resurected craft instead of player2
        //

        battle.getWreckages().getWrecks()[0].setTurnCreated(12);
        bca[0].rechargeWeapons();
        wi = bca[0].getWeapon(tech_timeWarpBeam);
        bca[0].fireWeapon(wreckage.getRectangle().getCenterLocation(), wi, 1);
        
        assertEquals(null, battle.getWreckages().getWrecks());
        assertEquals(false, craft3.isDestroyed());
        assertEquals(false, bca[2].isDestroyed());
        assertEquals(player1, bca[2].getOwner());
        assertEquals("The craft has been added to the battle", true, battle.getCraftList().contains(bca[2]));
        assertEquals("The craft has not been added to the player", 1, bca[2].getOwner().getQtyCrafts());
        
    }

    
    /**
     * Tests that the salvage weapon can extract technology from wreckage on the battle field 
     * Tests when a battle has finished then the salvage is given to the winning layer.... 
     */
    public void testWeaponSalvageBeam() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponSalvageBeam() *************************");
        CraftDesign salvager = new CraftDesign("Salvager",  tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_salvageBeam, tech_salvageBeam);
        createMapAndThreeCraft(fighterDesign, salvager, fighterDesign, null);

        Craft notUsedCraft1 = newCraft(fighterDesign, player1, ss2);
        Craft notUsedCraft2 = newCraft(fighterDesign, player1, ss2);
        battle.join(notUsedCraft1, true);
        battle.join(notUsedCraft2, true);
        bca = battle.getCrafts();

        assertEquals("TestFighter1", bca[0].getCraft().getName());
        assertEquals("Salvager1", bca[1].getCraft().getName());
        assertEquals("TestFighter2", bca[2].getCraft().getName());
        assertEquals("TestFighter3", bca[3].getCraft().getName());
        assertEquals("TestFighter4", bca[4].getCraft().getName());
        
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[2].setLocation(new Location(550, 450));
        bca[3].setLocation(new Location(700, 550));
        bca[4].setLocation(new Location(700, 450));

        bca[0].destroy(null, null, null);
        bca[2].destroy(null, null, null);
        bca[4].destroy(null, null, null);
        WreckageCraft wreckage0 = battle.getWreckages().getWreckage(bca[0]);
        WreckageCraft wreckage2 = battle.getWreckages().getWreckage(bca[2]);
        WreckageCraft wreckage3 = battle.getWreckages().getWreckage(bca[4]);
        
        // The content of wreckage is tested in SalvageFactoryTest and uses random
        // number generation making it unpredictable so we will not guess the content
        // of the salvage here - we will create our own!
        
        //
        // Test Fetching salvage from a wreckage, removes salvage from wreckage and adds
        // it to the battle craft.
        //
        
        Technology [] t1 = { tech_massDriver1,
					         tech_energy10,
					         tech_fusion25,
					         tech_shieldModulatorI};
        wreckage2.setSalvage(new SalvageList(t1));
        WeaponInstanceSalvageBeam wi = (WeaponInstanceSalvageBeam) bca[1].getWeapon(tech_salvageBeam);
        assertEquals(6, wi.getQtySalvageCanCollect());
        bca[1].fireWeapon(wreckage2.getRectangle().getCenterLocation(), wi, 1);
        
        assertEquals(null, battle.getWreckages().getWreckage(bca[2]));
        assertEquals(true, bca[1].getSalvage().contains(t1));
        assertEquals(4, bca[1].getSalvage().getQuantity());
        
        //
        // Test we can add more salvage to existing salvage when we fire again
        // Test Limit of three salvage item per beam means we cannot get all four techs.
        //

        Technology [] t2 = { tech_electronic40,
					         tech_photon200,
					         tech_flyer30,
					         tech_girator};
		wreckage0.setSalvage(new SalvageList(t2));
        assertEquals(2, wi.getQtySalvageCanCollect());
		bca[1].rechargeWeapons();
		bca[1].fireWeapon(wreckage0.getRectangle().getCenterLocation(), wi, 1);
		
		assertEquals(null, battle.getWreckages().getWreckage(bca[0]));
		assertEquals(true, bca[1].getSalvage().contains(t1));
		assertEquals(true, bca[1].getSalvage().contains(tech_electronic40));
		assertEquals(true, bca[1].getSalvage().contains(tech_photon200));
		assertEquals(false, bca[1].getSalvage().contains(tech_flyer30));
		assertEquals(false, bca[1].getSalvage().contains(tech_girator));
		assertEquals(6, bca[1].getSalvage().getQuantity());
		
        //
        // Test salvage beam cannot be recharged
        //
        Technology [] t3 = { tech_armour25, tech_auto_repair_unit};
        wreckage3.setSalvage(new SalvageList(t3));
        
		bca[1].rechargeWeapons();

		assertEquals(false, wi.canRecharge());
		assertEquals(false, bca[0].hasWeaponsToFire());

		//
        // End the battle now and check salvage is given to player....
		// Because all salvage beams have been used up, remaining wreckage is not taken
		//
		
		battle.getCraft(notUsedCraft1.getName()).destroy(null, null, null);
        battle.endTurn();
        
        assertEquals(true, battle.isWinner(player2));
        assertEquals(false, battle.isWinner(player1));
        
        assertEquals(6, player2.getSalvage().getQuantity());
        assertEquals(true, player2.getSalvage().contains(t1));
		assertEquals(true, player2.getSalvage().contains(tech_electronic40));
		assertEquals(true, player2.getSalvage().contains(tech_photon200));
		assertEquals(false, player2.getSalvage().contains(tech_flyer30));
		assertEquals(false, player2.getSalvage().contains(tech_girator));
    }

    /**
     * Tests that the salvage weapon can extract technology from wreckage on the battle field 
     * Tests when a battle has finished then the salvage is given to the winning layer.... 
     */
    public void testWeaponSalvageBeamEndBattle() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponSalvageBeamEndBattle() *************************");
        CraftDesign salvager = new CraftDesign("Salvager",  tech_flyer30, tech_fusion25, tech_electronic25, tech_energy10, tech_salvageBeam);
        createMapAndThreeCraft(fighterDesign, salvager, fighterDesign, null);

        Craft notUsedCraft1 = newCraft(fighterDesign, player1, ss2);
        battle.join(notUsedCraft1, true);
        bca = battle.getCrafts();
        
        assertEquals("TestFighter1", bca[0].getCraft().getName());
        assertEquals("Salvager1", bca[1].getCraft().getName());
        assertEquals("TestFighter2", bca[2].getCraft().getName());
        assertEquals("TestFighter3", bca[3].getCraft().getName());
        
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        bca[2].setLocation(new Location(550, 450));
        bca[3].setLocation(new Location(700, 500));

        bca[0].destroy(null, null, null);
        bca[2].destroy(null, null, null);
        WreckageCraft wreckage0 = battle.getWreckages().getWreckage(bca[0]);
        WreckageCraft wreckage2 = battle.getWreckages().getWreckage(bca[2]);
        
        //
        // Test salvage beam cannot be recharged
        //
        
        Technology [] t0 = { tech_armour25, tech_auto_repair_unit };
        wreckage0.setSalvage(new SalvageList(t0));
        Technology [] t2 = { tech_photon200, tech_flyer30 };
        wreckage2.setSalvage(new SalvageList(t2));
        
		//
        // End the battle now and check salvage is given to player....
		//
		
        bca[3].destroy(null, null, null);
        
        assertEquals(true, battle.isWinner(player2));
        assertEquals(false, battle.isWinner(player1));

        assertNull(player1.getSalvage());

        assertEquals(3, player2.getSalvage().getQuantity());
        assertEquals(true, player2.getSalvage().contains(t0));
		assertEquals(true, player2.getSalvage().contains(tech_photon200));
		assertEquals(false, player2.getSalvage().contains(tech_flyer30));
    }

    /**
     * Tests that the salvage weapon can extract technology from wreckage on the battle field 
     * Tests when a battle has finished then the salvage is given to the winning layer.... 
     */
    public void testPlanetSalvageAtEndBattle() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlanetSalvageAtEndBattle() *************************");
        createUserPlayerMap(null);

        CraftDesign noWreckageDesign = new CraftDesign("TestNoSalvage",  tech_flyer30, tech_fusion25, tech_massDriver1, tech_incinerator);
        Craft craft1 = newCraft(fighterDesign, player2, ss1);
        Craft craft2 = newCraft(defenderDesign, player2, ss1);
        Craft craft3 = newCraft(noWreckageDesign, player2, ss1);
        
        planet1.setWeapons(new Weapons(tech_missile6));

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, true);
        battle.join(planet1);
        battle.setup();

        bp = battle.getPlanet();
        bca = battle.getCrafts();

        bca[0].destroy(null, null, null);
        bca[1].destroy(null, null, null);
        
        WreckageCraft wreckage0 = battle.getWreckages().getWreckage(bca[0]);
        WreckageCraft wreckage2 = battle.getWreckages().getWreckage(bca[1]);
        
        Technology [] t0 = { tech_armour25, tech_auto_repair_unit };
        wreckage0.setSalvage(new SalvageList(t0));
        Technology [] t2 = { tech_photon200};
        wreckage2.setSalvage(new SalvageList(t2));
        
		//
        // End the battle now and check salvage is given to player....
		//
		
        bca[2].destroy(null, null, null);
        
        assertEquals(true, battle.isWinner(userPlayer1));
        assertEquals(false, battle.isWinner(player2));

        assertNull(player2.getSalvage());
        assertEquals(tech_armour25, tech_auto_repair_unit, tech_photon200, userPlayer1.getSalvage());
    }


    /**
     * Tests that the replica weapon can create a new craft that is a photocopy of an enemy craft during combat. 
     */
    public void testWeaponReplicaBeam() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponReplicaBeam() *************************");
        createMapAndTwoCraft(new CraftDesign("TestReplica", tech_flyer30, tech_fusion25, tech_replicaBeam), 400, powerfulDesign, 600, null);

        //
        // Test firing replica when with no boralium
        //
		
        bca[0].fireAllWeapons(bca[1]);
        
        assertEquals("Replica Beam - No Boralium", ((BattleWeaponFireEvent)lastEvent).getSourceDescription());
        
        //
        // Test firing replica and that it works
        //
		
        player1.setBoralium(100);
        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);
        bca = battle.getCrafts();
        
        BattleCraftNewEvent cme = (BattleCraftNewEvent) lastEvent;
        assertEquals("TestPowerful2", cme.getBattleCraft().getName());
        assertEquals("Boralium Used - 75", ((BattleWeaponFireEvent)cme.getLinkedEvents()[0]).getSourceDescription());
		assertEquals(3, bca.length);
		assertEquals(true, new CraftDesignAdapter(bca[2].getCraft()).same(powerfulDesign));
		assertEquals(player1, bca[2].getOwner());
		assertEquals(player2, bca[2].getOriginPlayer());
		assertEquals(75, player1.getBoralium());
		
        //
        // Test Immunity to replica
        //
		
		bca[1].add(tech_imunity);

        bca[0].rechargeWeapons();
        bca[0].fireAllWeapons(bca[1]);

        assertEquals("Replica Beam Immune", ((BattleWeaponFireEvent)lastEvent).getTargetDescription());
    }

    /**
     * Tests that the replica weapon can create a new craft that is a photocopy of an enemy craft during combat. 
     */
    public void testWeaponShieldBurst() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponShieldBurst() *************************");
        createMapAndThreeCraft(new CraftDesign("TestShildBurst", tech_flyer30, tech_fusion25, tech_energy10, tech_shieldBurst), powerfulDesign, fighterDesign, null);
        Craft newCraft1 = newCraft(fighterDesign, player1, ss2);
        Craft newCraft2 = newCraft(fighterDesign, player2, ss2);
        battle.join(newCraft1, true);
        battle.join(newCraft2, true);
        
        bca = battle.getCrafts();
        
        assertEquals(5, bca.length);
        
        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(550, 500)); // In range owner = p2
        bca[2].setLocation(new Location(600, 600)); // Out of range owner = p2
        bca[3].setLocation(new Location(500, 450)); // In range owner = p1
        bca[4].setLocation(new Location(500, 550)); // In range owner = p2

        //
        // Test weapon fires and damage two craft within range but not the craft just outside its range
        //
		
        bca[0].fireAllWeapons(bca[1]);
        
		assertEquals(0.0, bca[0].getDamage());
		assertEquals(8.0, bca[1].getDamage());
		assertEquals(0.0, bca[2].getDamage());
		assertEquals(9.0, bca[3].getDamage());
		assertEquals(9.0, bca[4].getDamage());
		
        assertEquals(10, ((BattleDamageRingEvent)lastEvent).getDamage());
        assertEquals(bca[0].getName(), ((BattleDamageRingEvent)lastEvent).getSource());
        assertEquals(tech_shieldBurst, ((BattleDamageRingEvent)lastEvent).getWeaponTechnology());

        //
        // test it does not fire when shields are drained
        //
		
        bca[1].getCraft().setDamage(0);
        bca[2].getCraft().setDamage(0);
        bca[3].getCraft().setDamage(0);
        bca[4].getCraft().setDamage(0);
        bca[0].getShields()[0].setPower(0);
        bca[0].fireAllWeapons(bca[1]);
        
		assertEquals(0.0, bca[0].getDamage());
		assertEquals(0.0, bca[1].getDamage());
		assertEquals(0.0, bca[2].getDamage());
		assertEquals(0.0, bca[3].getDamage());
		assertEquals(0.0, bca[4].getDamage());
    }

    /**
     * Tests teleport weapon on crafts.
     */
    public void testWeaponTeleport() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponTeleport() *************************");
		CraftDesign design = new CraftDesign("TeleportFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_teleporter);
		
        createMap(null);

        planet2.setWeapons(new Weapons(tech_massDriver1));
        createBattleStation(tech_battleStationI, planet2);
        
        craft1 = newCraft(design, player1, ss2);
        craft2 = newCraft(fighterDesign, player2, ss2);
        
        battle = newBattle(ss2);
        battle.join(planet2);
        battle.join(craft1, true);
        battle.join(craft2, false);
        battle.setup();
        battle._addUserPlayerForTestHarness(player1);

        bp = battle.getPlanet();
        bca = battle.getCrafts();

        assertEquals("BattleStation1", bca[0].getName());
        assertEquals("TeleportFighter1", bca[1].getName());
        assertEquals("TestFighter1", bca[2].getName());

        bca[1].setLocation(new Location(500, 500));
        bca[1].setDirection(90);
        bca[2].setLocation(new Location(600, 500));
        bca[2].setDirection(270);
        WeaponInstanceTeleporter weaponInstance = (WeaponInstanceTeleporter) bca[1].getWeapons()[0];
        
        //
        // Test weapon can fail
        //

        weaponFireEvent = null;
        bca[1].rechargeWeapons();
        Utils.clearAndSetNextRandomProbability(0.7);
        bca[1].fireWeapon(bca[2], weaponInstance, 1);
        
        assertEquals(600,500, bca[2].getLocation());
        assertEquals("TeleportWeapon Fail", weaponFireEvent.getTargetDescription());
        
        //
        // Player can be immune
        //

        player2.getPlayerCharacter().setImmunity(new ImmunityMoveCraft());
        weaponFireEvent = null;
        bca[1].rechargeWeapons();
        Utils.clearAndSetNextRandomProbability(0.2);
        bca[1].fireWeapon(bca[2], weaponInstance, 1);
        
        assertEquals(600,500, bca[2].getLocation());
        assertEquals("TeleportWeapon Immune", weaponFireEvent.getTargetDescription());

        //
        // Test weapon move the crafts to a new location
        //

        player2.getPlayerCharacter().setImmunity(null);
        weaponFireEvent = null;
        lastCraftTeleportEvent = null;
        Utils.clearAndSetNextRandomProbability(0.2);
        Utils.clearAndSetNextRandomInt(800);
        Utils.setNextRandomInt(600);
        bca[1].rechargeWeapons();
        bca[1].fireWeapon(bca[2], weaponInstance, 1);
        
        assertEquals(818,618, bca[2].getLocation());
        assertEquals(600,500, weaponFireEvent.getOldTargetState().getLocation());
        assertEquals(818,618, weaponFireEvent.getNewTargetState().getLocation());
        assertEquals(600,500, lastCraftTeleportEvent.getOldState().getLocation());
        assertEquals(818,618, lastCraftTeleportEvent.getNewState().getLocation());
        
        //
        // Test weapon has not effect on a battle station
        //
        
        Utils.clearAndSetNextRandomProbability(0.2);
        bca[1].setLocation(new Location(250,500));
        bca[1].setDirection(270);
        bca[1].rechargeWeapons();
        bca[1].fireWeapon(bca[0], weaponInstance, 1);
        
        assertEquals(200,500, bca[0].getLocation());
        assertEquals("TeleportWeapon Immune", weaponFireEvent.getTargetDescription());

        //
        // Test weapon has not effect on a planet
        //
        
        Utils.clearAndSetNextRandomProbability(0.2);
        bca[1].rechargeWeapons();
        bca[1].fireWeapon(bp, weaponInstance, 1);
        
        assertEquals(100,500, bp.getLocation());
        assertEquals("TeleportWeapon Immune", weaponFireEvent.getTargetDescription());

    }
        
    /**
     * Tests sleep weapon on crafts.
     */
    public void testWeaponSleep() throws Exception {
        Logger.logDebug("*******************************  WarTest.testWeaponSleep() *************************");
		CraftDesign design = new CraftDesign("SleepFighter", tech_flyer30, tech_fusion25, tech_electronic25, tech_sleep);
        createMapAndTwoCraft(design, fighterDesign, null);

        bca[0].setLocation(new Location(500, 500));
        bca[1].setLocation(new Location(600, 500));
        WeaponInstanceSleep weaponInstance = (WeaponInstanceSleep) bca[0].getWeapons()[0];
        
        //
        // Test has no effect when target has shields up
        //

        weaponFireEvent = null;
        bca[0].fireWeapon(bca[1], weaponInstance, 1);
        
        assertEquals("SleepWeapon Shield Blocked", weaponFireEvent.getTargetDescription());
        assertEquals(null, battle.getTriggers().getTriggers());
        assertEquals(null, bca[1].getDecorators().getVisibleDecorators(player1));
        assertEquals(null, bca[1].getDecorators().getVisibleDecorators(player2));

        
        //
        // Test weapon can fail
        //

        bca[1].getShields()[0].setPower(0.0);
        weaponFireEvent = null;
        bca[0].setMorale(-1.0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[1], weaponInstance, 1);
        
        assertEquals("SleepWeapon Fail", weaponFireEvent.getTargetDescription());
        assertEquals(null, battle.getTriggers().getTriggers());
        assertEquals(null, bca[1].getDecorators().getVisibleDecorators(player1));
        assertEquals(null, bca[1].getDecorators().getVisibleDecorators(player2));

        //
        // Test that weapon places a trigger on the trigger causing it to miss its turn.
        //

        bca[1].getShields()[0].setPower(0.0);
        bca[0].setMorale(0.0);
        bca[0].rechargeWeapons();
        bca[0].fireWeapon(bca[1], weaponInstance, 1);
        Sleep trigger = (Sleep) battle.getTriggers().getTriggers()[0];

        assertNull(weaponFireEvent.getTargetDescription());
        assertEquals(1, trigger.getQtyTurnsToMiss());
        assertEquals(trigger, battle.getTriggersNotNull().getBattleObjectTriggerOn(bca[1], Sleep.class));
        assertEquals("SleepDecorator", bca[1].getDecorators().getVisibleDecorators(player1).getDecorators()[0].getName());
        assertEquals("SleepDecorator", bca[1].getDecorators().getVisibleDecorators(player2).getDecorators()[0].getName());
        assertEquals(true, trigger.isComplete());
        
        battle.endTurn();
        
        assertEquals(null, battle.getTriggers().getTriggers());
        assertEquals(null, battle.getTriggersNotNull().getBattleObjectTriggerOn(bca[1], Sleep.class));
        assertEquals(null, battle.getTriggersNotNull().getBattleObjectTriggerOn(bca[1], Sleep.class));


        //
        // Test if fired at a craft that has already moved this turn it misses next turn.
        //

        bca[1].getShields()[0].setPower(0.0);
        bca[1].dischargeWeapons();
        bca[1].setMoveRemaining(0.0);
        bca[0].rechargeWeapons();
        Utils.setNextRandomProbability(0.2);
        bca[0].fireWeapon(bca[1], weaponInstance, 1);
        trigger = (Sleep) battle.getTriggers().getTriggers()[0];

        assertNull(weaponFireEvent.getTargetDescription());
        assertEquals(1, trigger.getQtyTurnsToMiss());
        assertEquals(trigger, battle.getTriggersNotNull().getBattleObjectTriggerOn(bca[1], Sleep.class));
        assertEquals("SleepDecorator", bca[1].getDecorators().getVisibleDecorators(player1).getDecorators()[0].getName());
        assertEquals("SleepDecorator", bca[1].getDecorators().getVisibleDecorators(player2).getDecorators()[0].getName());
        assertEquals(false, trigger.isComplete());
        
        battle.endTurn();
        
        assertEquals(null, battle.getTriggers().getTriggers());
        assertEquals(null, battle.getTriggersNotNull().getBattleObjectTriggerOn(bca[1], Sleep.class));
        assertEquals(null, battle.getTriggersNotNull().getBattleObjectTriggerOn(bca[1], Sleep.class));

    }
        

    /**
     * Tests Crafts destroyed with an incinerator do not generate wreckage 
     */
    public void testIncineratorSpecialAndNoWreckageHulls() throws Exception {
        Logger.logDebug("*******************************  WarTest.testIncineratorSpecialAndNoWreckageHulls() *************************");
		CraftDesign incineratorDesign =new CraftDesign("IncineratorFighter", tech_flyer30, tech_fusion25, tech_incinerator, tech_holoEmiter);
        createMapAndThreeCraft(new Weapons(tech_missile6), tech_battleStationI, true, true, fighterDesign, fighterDesign, incineratorDesign, null);

        BattlePlanet planet = battle.getPlanet();
        BattleCraft battleStation = battle.getCraft("BattleStation1");
        BattleCraft battleCraft1 = battle.getCraft("TestFighter1");
        BattleCraft battleCraft2 = battle.getCraft("TestFighter2");
        BattleCraft battleCraft3 = battle.getCraft("IncineratorFighter1");
        BattleCraft holoFighter = battle.getCraft("IncineratorFighter111");
        BattleCraft sentinal1 = battle.getCraft("BattleSentinel1");
        
        assertEquals(true, battleCraft3.getCraft().getSpecials().containsGroup(Special.GROUP_INCINERATOR));
        
        battleCraft3.destroy(null, null, null);
        
        assertEquals(true, battleCraft3.isDestroyed());
        assertEquals(null, battle.getCraft("IncineratorFighter1"));
        assertEquals(null, battle.getWreckages().getWrecks());
        
        holoFighter.destroy(player1, null, null);
        
        assertEquals(true, holoFighter.isDestroyed());
        assertEquals(null, battle.getCraft("IncineratorFighter111"));
        assertEquals(null, battle.getWreckages().getWrecks());
        
        battleStation.destroy(player2, null, null);
        
        assertEquals(true, battleStation.isDestroyed());
        assertEquals(null, battle.getCraft("BattleStation1"));
        assertEquals(null, battle.getWreckages().getWrecks());
        
        sentinal1.destroy(player2, null, null);
        
        assertEquals(true, sentinal1.isDestroyed());
        assertEquals(null, battle.getCraft("BattleSentinel1"));
        assertEquals(null, battle.getWreckages().getWrecks());
    
        planet.launchFighters();
        BattleCraft fighter1 = battle.getCraft("BattleFighter1");
        
        fighter1.destroy(player2, null, null);
        
        assertEquals(true, battleCraft3.isDestroyed());
        assertEquals(null, battle.getCraft("Fighter1"));
        assertEquals(null, battle.getWreckages().getWrecks());
        
        battleCraft1.destroy(player2, null, null);
        
        assertEquals(true, battleCraft1.isDestroyed());
        assertEquals(null, battle.getCraft("TestFighter1"));
        assertEquals(1, battle.getWreckages().getWrecks().length);
        assertEquals("TestFighter1", ((WreckageCraft)battle.getWreckages().getWrecks()[0]).getBattleCraft().getName());

    }

    /**
     * Tests the special Shield Span capability.
     */
    public void testShieldSpanSpecial() throws Exception {
        Logger.logDebug("*******************************  WarTest.testShieldSpanSpecial() *************************");
		CraftDesign shieldSpanFighter = new CraftDesign("SpanningShieldFighter", 
				tech_flyer30, tech_fusion25, tech_energy10, tech_shieldSpan);
        createMapAndThreeCraft(fighterDesign, shieldSpanFighter, fighterDesign, null);

        bca = battle.getCrafts();
        ShieldInstance shields1 = bca[1].getShields()[0];
        ShieldInstance shields2 = bca[2].getShields()[0];
        
        //
        // Test recharge works
        //
        
        shields2.setPower(8);
        
		assertEquals(shields2, bca[2].getShieldsThatRequireRecharging());
        assertEquals(8, shields2.getPower(), 0.001);
        assertEquals(2, shields2.getMaxRecharge(), 0.001);
        
        lastEvent = null;
        bca[1].rechargeShields();
        
        assertEquals(8.5, shields2.getPower(), 0.001);
        assertEquals(1.5, shields2.getMaxRecharge(), 0.001);
        assertEquals(bca[1].getName(), ((BattleWeaponFireEvent)lastEvent).getSource());
        assertEquals(bca[2].getName(), ((BattleWeaponFireEvent)lastEvent).getTarget());
        assertEquals(Weapon.GROUP_SHIELD_DRAIN, ((BattleWeaponFireEvent)lastEvent).getWeaponTechnology().getGroup());
        assertEquals("+0.5", ((BattleWeaponFireEvent)lastEvent).getTargetDescription());

        //
        // Test that cannot over charge shields
        //
        
        bca[1].rechargeShields();
        bca[1].rechargeShields();
        bca[1].rechargeShields();
        bca[1].rechargeShields();
        
		assertEquals(null, bca[2].getShieldsThatRequireRecharging());
        assertEquals(10, shields2.getPower(), 0.001);
        assertEquals(0, shields2.getMaxRecharge(), 0.001);

        //
        // Test no recharge occurs when out of range
        //
        
        bca[2].setLocation(new Location(851,500));
        shields2.setPower(8);
        
        bca[1].rechargeShields();

        assertEquals(8, shields2.getPower(), 0.001);
        assertEquals(2, shields2.getMaxRecharge(), 0.001);
    }

    /**
     * Tests the special Shield Span capability.
     */
    public void testShieldSpanSpecialWithMultipleShields() throws Exception {
        Logger.logDebug("*******************************  WarTest.testShieldSpanSpecialWithMultipleShields() *************************");
		CraftDesign shieldSpanFighter = new CraftDesign("SpanningShieldFighter", 
				tech_flyer30, tech_fusion25, tech_energy10, tech_energy10, tech_shieldSpan);
        createMapAndThreeCraft(fighterDesign, shieldSpanFighter, shieldSpanFighter, null);

        bca = battle.getCrafts();
        ShieldInstance shields1a = bca[1].getShields()[0];
        ShieldInstance shields1b = bca[1].getShields()[1];
        ShieldInstance shields2a = bca[2].getShields()[0];
        ShieldInstance shields2b = bca[2].getShields()[1];
        
        //
        // Test recharge works
        //
        
        shields2b.setPower(8);
        
		assertEquals(shields2b, bca[2].getShieldsThatRequireRecharging());
        assertEquals(8, shields2b.getPower(), 0.001);
        assertEquals(2, shields2b.getMaxRecharge(), 0.001);
        
        lastEvent = null;
        bca[1].rechargeShields();
        
        assertEquals(9.0, shields2b.getPower(), 0.001);
        assertEquals(1.0, shields2b.getMaxRecharge(), 0.001);
        assertEquals(bca[1].getName(), ((BattleWeaponFireEvent)lastEvent).getSource());
        assertEquals(bca[2].getName(), ((BattleWeaponFireEvent)lastEvent).getTarget());
        assertEquals(Weapon.GROUP_SHIELD_DRAIN, ((BattleWeaponFireEvent)lastEvent).getWeaponTechnology().getGroup());
        // This makes sure there has been a single WFE for for spare capacity on both shields
        assertEquals("+1", ((BattleWeaponFireEvent)lastEvent).getTargetDescription());

    }

    /**
     * Tests Automated Repair Unit 
     */
    public void testAutomatedRepair() throws Exception {
        Logger.logDebug("*******************************  WarTest.testAutomatedRepair() *************************");
        createMapAndTwoCraft(fighterDesign, 400, fighterDesign, 600, null);

        //
        // Test automated repair unit heals craft damage At the end of each turn
        //
        
        Specials specials = new Specials();
        specials.add(tech_auto_repair_unit);
        bca[0].getCraft().setSpecials(specials);
        assertEquals(0.0, bca[0].getDamage());
        
        bca[0].damage(10, player1, null, null);
        bca[0].startTurn();
        
        assertEquals(20, bca[0].getCraft().getMaxDamage());
        assertEquals("Damage has been healed", 8.0, bca[0].getDamage());
        
        endBattleEvent = null;
        bca[1].destroy(null, null, null);
        battle.endBattle();
        MotEngine.getInstance().getProxyManager().broadcastQueue();
        
        assertEquals(bca[1].getName(), endBattleEvent.getCraftDestroyd()[0]);
                
        //
        // Test automated repair unit completely heals craft between battles
        //
        
        craft3 = newCraft(fighterDesign, player2, ss2);

        battle = newBattle(ss2);
        battle.join(craft1, true);
        battle.join(craft3, false);
        battle.setup();
        
        bca = battle.getCrafts();
        
        assertEquals(craft1.getName(), bca[0].getName());
        assertEquals(0.0, bca[0].getDamage());
        
    }

    /**
     * Tests that battles fought inside a nebula
     * 1 - Do not have shields
     * 2 - Have no cloaking
     */
    public void testNebula() throws Exception {
        Logger.logDebug("*******************************  WarTest.testNebula() *************************");
        SetupOptions optionsRace = new SetupOptions();
        optionsRace.setOption("cloak-level", "200", 0);
        
        createMap(optionsRace);
        
        MotFactory.createNebula(new Location(100,150), 150);

		CraftDesign tf1 = new CraftDesign("TF1",tech_flyer30, tech_fusion25, tech_energy10, tech_cloak150, tech_psiI, tech_massDriver1);
		CraftDesign tf2 = new CraftDesign("TF2",tech_flyer30, tech_fusion25, tech_massDriver1);
        craft1 = newCraft(tf1, player1, ss1);
        craft2 = newCraft(tf2, player1, ss1);
        craft3 = newCraft(tf1, player2, ss1);
        craft4 = newCraft(tf2, player2, ss1);

        Shields planetShields = new Shields(tech_energy10);
        planet1.setShields(planetShields);
        Weapons planetWeapons = new Weapons();
        planetWeapons.add(tech_missile6);
        planet1.setWeapons(planetWeapons);
        
        battle = newBattle(ss1);
        battle.join(planet1);
        battle.join(craft1, true);
        battle.join(craft2, true);
        battle.join(craft3, false);
        battle.join(craft4, false);
        battle.setup();

        bp = battle.getPlanet();        
        bca = battle.getCrafts();
        bca[0].setLocation(new Location(200, 480));
        bca[1].setLocation(new Location(200, 520));
        bca[2].setLocation(new Location(900, 480));
        bca[3].setLocation(new Location(900, 520));
        bca[0].setDirection(90);
        bca[1].setDirection(90);
        bca[2].setDirection(270);
        bca[3].setDirection(270);
        
        assertEquals(200, player1.getPlayerCharacter().getCloakLevel());
        assertEquals(0, bp.getCloakLevel());
        assertEquals(0, bca[0].getCloakLevel());
        assertEquals(0, bca[1].getCloakLevel());
        assertEquals(0, bca[2].getCloakLevel());
        assertEquals(0, bca[3].getCloakLevel());
        assertEquals(true, bp.isVisible(player1));
        assertEquals(true, bca[0].isVisible(player1));
        assertEquals(true, bca[1].isVisible(player1));
        assertEquals(true, bca[2].isVisible(player1));
        assertEquals(true, bca[3].isVisible(player1));
        assertEquals(true, bp.isVisible(player2));
        assertEquals(true, bca[0].isVisible(player2));
        assertEquals(true, bca[1].isVisible(player2));
        assertEquals(true, bca[2].isVisible(player2));
        assertEquals(true, bca[3].isVisible(player2));
        assertNull(bp.getShields());
        assertNull(bca[0].getShields());
        assertNull(bca[1].getShields());
        assertNull(bca[2].getShields());
        assertNull(bca[3].getShields());
        
        //
        // Craft can still attack and do damage
        //
        
        bca[2].setLocation(new Location(300, 480));
        bca[3].setLocation(new Location(300, 520));

        bca[0].fireAllWeapons(bca[2]);
        assertEquals(2.0, bca[2].getDamage(), 0.01);
        
        
    }


    /**
     * Tests that A planet with no weapons but has fighters can still join a battle and be active in it.
     */
    public void testPlanetWithNoWeapons() throws Exception {
        Logger.logDebug("*******************************  WarTest.testPlanetWithNoWeapons() *************************");
        createUserPlayerMap(null);
                
        userPlayer1.addSpecial(tech_fighterBays);
        craft1 = newCraft(fighterDesign, player2, ss1);

        planet1.setShields(new Shields(tech_energy10));
        planet1.setScanners(new Scanners(tech_photon150));

        craft1.attack(planet1);
        
        battle = MotEngine.instance.getBattles().getBattles()[0];
        battle.setup();

        bp = battle.getPlanet();        
        bca = (BattleCraft[]) order(battle.getCrafts(), new BattleCraft[1], "TestFighter1");
        
        bca[0].setLocation(new Location(300, 500));
        bca[0].setDirection(270);

        assertEquals(bca[0], battle._getNextToMove());
        
        clearEvents();
        assertEquals(bp, battle.getNextToMove(true));

        assertEquals(bca[0].getName(), ((BattleCraftMoveEvent)eventList.get(0)).getName());
        assertEquals(bca[0].getName(), ((BattleWeaponFireEvent)eventList.get(1)).getSource());
        assertEquals(bp.getName(), ((BattleWeaponFireEvent)eventList.get(1)).getTarget());

        clearEvents();
        //new BattleAttackCmd(userPlayer, battle, bp, bca[0], true, true, true, null, null, -1, false).run();
        new BattleCraftLaunchFightersCmd(userPlayer1, battle, bp).run();

        bca = (BattleCraft[]) order(battle.getCrafts(), new BattleCraft[5], "TestFighter1", "BattleFighter1", "BattleFighter2", "BattleFighter3", "BattleFighter4");
        assertEquals(player2, bca[0].getOwner());
        assertEquals(userPlayer1, bca[1].getOwner());
        assertEquals(userPlayer1, bca[2].getOwner());
        assertEquals(userPlayer1, bca[3].getOwner());
        assertEquals(userPlayer1, bca[4].getOwner());
        assertEquals(4, battle.getCrafts(userPlayer1).length);
        assertEquals(bca[1].getName(), ((BattleCraftNewEvent)eventList.get(0)).getBattleCraft().getName());
        assertEquals(bca[2].getName(), ((BattleCraftNewEvent)eventList.get(1)).getBattleCraft().getName());
        assertEquals(bca[3].getName(), ((BattleCraftNewEvent)eventList.get(2)).getBattleCraft().getName());
        assertEquals(bca[4].getName(), ((BattleCraftNewEvent)eventList.get(3)).getBattleCraft().getName());
        
        //
        // Test craft can destroy planet fighter bays then battle ends
        //

        assertEquals(bca[1], battle._getNextToMove());
        
        bca[1]._destroy();
        bca[2]._destroy();
        bca[3]._destroy();
        bca[4]._destroy();
        bca[0].startTurn();

        clearEvents();
        assertEquals(null, battle.getNextToMove(true));
        assertTrue(battle.hasEnded());
        
    }
    
  /**
   * Tests that A planet with no weapons but has fighters can still join a battle and be active in it.
   */
  public void testBattleCancel() throws Exception {
      Logger.logDebug("*******************************  WarTest.testBattleCancel() *************************");
      createUserPlayerMap(null);
              
      craft1 = newCraft(fighterDesign, userPlayer1, ss2);
      craft2 = newCraft(fighterDesign, player2, ss2);

      new CraftAttackCmd(craft1, null, userPlayer1).run();

      battle = MotEngine.instance.getBattles().getBattles()[0];
      
      bca = (BattleCraft[]) order(battle.getCrafts(), new BattleCraft[2], "TestFighter1", "TestFighter2");

      //
      // Attacked player deletes craft (could be forced sale etc) which cancel the battle
      //
      
      craft2.destroy();

      assertEquals(true, battle.hasEnded());
      assertNull(MotEngine.instance.getBattles().getBattles());

      //
      // End Turn does not get a NewBattleEvent
      //
      
      clearEvents();
      new EndTurnCmd(userPlayer1).run();
     
      assertNull(getEvent(NewBattleEvent.class));
      assertNotNull(getEvent(StartTurnEvent.class));
      
  }
  
    private MotEvent lastEvent = null;
    private boolean fired = false;
    private int qtyFired = 0;
    private int qtyMissedEvents = 0;
    private int qtySurrenderEvents = 0;
    private int qtyFearEvents = 0;
    private int qtyMissilesLaunched = 0;
    private int qtyDroidMinesLaunched = 0;
    private int qtyCraftDestroyed = 0;
    private String lastCraftDestroyed;
    private BattleCraftMoveEvent lastCraftMovementEvent;
    private BattleCraftTeleportEvent lastCraftTeleportEvent;
    private BattleDamageEvent lastDamageEvent;
    private BattleObjectTriggerEvent lastTriggerEvent;
    private BattleEndEvent endBattleEvent;
    private RoutedEvent routedEvent;
    private BattleWeaponFireEvent weaponFireEvent;
    private BattleObjectNotificationEvent notificationEvent;
    private BattleDamageRingEvent damageRingEvent1;
    private BattleDamageRingEvent damageRingEvent2;
    private BattleDroidMineLaunchEvent droidMineLaunchEvent;
    
	public void notify(MotEventFactory e) throws ApplicationError {
        notify(e.createEvent(getOwner()));
    }
    
	public void notify(MotEvent e) throws ApplicationError {
//		if (e instanceof StateContainer) {
//			DeligateCacheManager.update((StateContainer)e);
//		}
		eventList.add(e);
		lastEvent = e;
        if (e instanceof BattleDestroyedEvent) {
            BattleDestroyedEvent event = (BattleDestroyedEvent)e;
            lastCraftDestroyed = event.getTarget();
            qtyCraftDestroyed++;
        } else if (e instanceof BattleWeaponFireEvent) {
            fired = true;
            qtyFired++;
            weaponFireEvent = (BattleWeaponFireEvent) e;
        } else if (e instanceof BattleDamageEvent) {
            lastDamageEvent = (BattleDamageEvent) e;
        } else if (e instanceof BattleDamageRingEvent) {
        	damageRingEvent2 = damageRingEvent1;
        	damageRingEvent1 = (BattleDamageRingEvent) e;
        } else if (e instanceof BattleDroidMineLaunchEvent) {
        	droidMineLaunchEvent = (BattleDroidMineLaunchEvent) e;
            qtyDroidMinesLaunched++;
        } else if (e instanceof BattleWeaponMissEvent) {
            qtyMissedEvents++;
        } else if (e instanceof BattleCraftSurrenderEvent) {
            qtySurrenderEvents++;
        } else if (e instanceof BattleFearEvent) {
            qtyFearEvents++;
        } else if (e instanceof BattleMissileLaunchEvent) {
            qtyMissilesLaunched++;
        } else if (e instanceof BattleCraftMoveEvent) {
            lastCraftMovementEvent = (BattleCraftMoveEvent)e;
        } else if (e instanceof BattleObjectNotificationEvent) {
        	notificationEvent = (BattleObjectNotificationEvent)e;
        } else if (e instanceof BattleCraftTeleportEvent) {
        	lastCraftTeleportEvent = (BattleCraftTeleportEvent)e;
        } else if ( e instanceof BattleObjectTriggerEvent) {
        	lastTriggerEvent = (BattleObjectTriggerEvent)e;
        } else if ( e instanceof BattleEndEvent) {
        	endBattleEvent = (BattleEndEvent)e;
        	Logger.logDebug("EndBattleEvent received in Battle Test "+Comp.toString(endBattleEvent.getCraftDestroyd()));
        } else if ( e instanceof RoutedEvent) {
        	routedEvent = (RoutedEvent)e;
        	Logger.logDebug("RoutedEvent received in Battle Test "+Comp.toString(routedEvent.getCraftDestroyed()));
        } 
    }

    protected void clearEvents() {
        super.clearEvents();
        lastEvent = null;
    }

	public Player getOwner() {
		if (player1 == null) {
			return userPlayer1;
		} else {
			return player1;
		}
    }


    public static void main(String args[]) {
        Logger.clearContents();
        junit.textui.TestRunner.run(suite());
    }

}