/*
 * Copyright (c) 2013-2019 Paul Davis
 *  
 * This file is part of Master of Tactics.
 *
 * Master of Tactics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Master of Tactics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Master of Tactics.  If not, see <https://www.gnu.org/licenses/>.
 * 
 * @author Paul Davis
 */
package mot.test;

import junit.framework.Test;
import junit.framework.TestSuite;
import mot.client.bean.MoveableObjectLocation;
import mot.client.bean.MoveableObjectLocationFactory;
import mot.client.deligates.BattleDelegate;
import mot.client.deligates.BlackHoleDelegate;
import mot.client.deligates.ComputerPlayerDelegate;
import mot.client.deligates.CraftDelegate;
import mot.client.deligates.FleetDelegate;
import mot.client.deligates.MoveableOwnedObjectDelegate;
import mot.client.deligates.NamedObjectDelegate;
import mot.client.deligates.PlanetDelegate;
import mot.client.deligates.PlayerDelegate;
import mot.client.deligates.RelationDelegate;
import mot.client.deligates.ResearchProjectDelegate;
import mot.client.deligates.SolarSystemDelegate;
import mot.client.deligates.StellarObjectDelegate;
import mot.client.deligates.TradeRouteDelegate;
import mot.client.deligates.TransportDelegate;
import mot.client.deligates.UserPlayerDelegate;
import mot.client.deligates.cache.DeligateCacheManager;
import mot.client.deligates.mot.MotCraftDelegate;
import mot.client.deligates.mot.MotPlanetDelegate;
import mot.client.event.MotEventConsumer;
import mot.client.proxy.CompletedClientCallbackSyncroniser;
import mot.client.proxy.MotServerProxy;
import mot.client.proxy.ProxyStore;
import mot.client.proxy.ServerProxy;
import mot.client.swing.window.galaxy.actions.EndTurnButton;
import mot.client.swing.window.main.MainWindow;
import mot.client.swing.window.main.TestGalaxyWindowConfig;
import mot.common.bean.DiplomaticProposal;
import mot.common.bean.Location;
import mot.common.bean.NameList;
import mot.common.bean.Nebula;
import mot.common.bean.NewGameData;
import mot.common.bean.NewGameDataComputerPlayerDetails;
import mot.common.bean.NewGameUserPlayerData;
import mot.common.bean.PlayerAiSettings;
import mot.common.bean.PlayerAssets;
import mot.common.bean.Rectangle;
import mot.common.bean.SalvageList;
import mot.common.bean.SavedGameFile;
import mot.common.bean.SectorOwnerGrid;
import mot.common.bean.Settings;
import mot.common.bean.Size;
import mot.common.bean.stats.Graph;
import mot.common.bean.stats.GraphPlayerState;
import mot.common.bean.stats.GraphState;
import mot.common.command_results.CraftDesignCR;
import mot.common.command_results.FileNamesCR;
import mot.common.command_results.NameListCR;
import mot.common.command_results.TechnologyListCR;
import mot.common.designs.BattleStationDesign;
import mot.common.designs.CraftDesign;
import mot.common.designs.CraftDesigns;
import mot.common.designs.Orbital;
import mot.common.designs.OutpostDesign;
import mot.common.designs.ProjectDesign;
import mot.common.designs.ScannerDesign;
import mot.common.designs.ShieldDesign;
import mot.common.designs.SpaceStationDesign;
import mot.common.designs.TransportDesign;
import mot.common.designs.WeaponDesign;
import mot.common.event.BattleEndEvent;
import mot.common.event.ColonisePlanetEvent;
import mot.common.event.DiplomaticRequestEvent;
import mot.common.event.DiplomaticResponseEvent;
import mot.common.event.LinkedEventBase;
import mot.common.event.MessageEvent;
import mot.common.event.MotEvent;
import mot.common.event.NewBattleEvent;
import mot.common.event.NoProjectEvent;
import mot.common.event.NoResearchProjectEvent;
import mot.common.event.OfferLeaderEvent;
import mot.common.event.OfferMoralChoiceEvent;
import mot.common.event.PlanetBombardedEvent;
import mot.common.event.PlanetConqueredEvent;
import mot.common.event.PlanetInvadeEvent;
import mot.common.event.PrizeGiftEvent;
import mot.common.event.TechnologyDiscoveredEvent;
import mot.common.proxy.ClientProxy;
import mot.common.resources.ResourceBoralium;
import mot.common.resources.ResourcePsiFungus;
import mot.common.setup.OptionsFactory;
import mot.common.setup.RaceOptions;
import mot.common.setup.SetupOption;
import mot.common.setup.SetupOptions;
import mot.common.technology.GeneralTechnologyList;
import mot.common.technology.Modules;
import mot.common.technology.Scanners;
import mot.common.technology.Shields;
import mot.common.technology.Technologies;
import mot.common.technology.Technology;
import mot.common.technology.TechnologyList;
import mot.common.util.ApplicationError;
import mot.common.util.Comp;
import mot.common.util.Logger;
import mot.common.util.Utils;
import mot.common.util.ValidationError;
import mot.common.vo.OrbitVO;
import mot.common.vo.StrategistEventVO;
import mot.engine.MotEngine;
import mot.engine.command.CraftJoinFleetCmd;
import mot.engine.common.CompletedServerCallbackSyncroniser;
import mot.engine.common.Orbit;
import mot.engine.crafts.Craft;
import mot.engine.crafts.CraftFactory;
import mot.engine.crafts.Fleet;
import mot.engine.crafts.FleetFactory;
import mot.engine.event.MotEventFactory;
import mot.engine.planets.BlackHole;
import mot.engine.planets.BlackHoleFactory;
import mot.engine.planets.Planet;
import mot.engine.planets.ProjectCraft;
import mot.engine.planets.ProjectOutpost;
import mot.engine.planets.ProjectSpaceStation;
import mot.engine.planets.SolarSystem;
import mot.engine.planets.TradeRoute;
import mot.engine.planets.TradeRouteFactory;
import mot.engine.players.AncientPlayer;
import mot.engine.players.ComputerPlayer;
import mot.engine.players.MonsterPlayer;
import mot.engine.players.Player;
import mot.engine.players.ResearchProject;
import mot.engine.players.UserPlayer;
import mot.engine.players.morals.MoralPolicyAdvisorBasic;
import mot.engine.players.strategists.StratagistBasic;
import mot.engine.prizes.MoneyPrize;
import mot.engine.prizes.TechnologyPrize;
import mot.engine.relations.AllianceTreaty;
import mot.engine.relations.Relation;
import mot.engine.relations.Relations;
import mot.engine.relations.WarTreaty;
import mot.engine.setup.MotFactory;
import mot.engine.spies.Spies;
import mot.engine.transports.Transport;
import mot.engine.transports.TransportTaskCreateTradeRoute;
import mot.engine.transports.TransportTaskDeployStation;
import mot.engine.transports.TransportTaskInvade;
import mot.engine.transports.TransportTaskMoveColonists;

/**
 * This test harness tests that players can use spies to steal technology
 * sabotage planets and hide. It also tests that spies stationed at home find an
 * kill enemy spies.
 */

public class DeligateGalaxyTest extends TestBase implements MotEventConsumer, ClientProxy {

	private static final String REMOTE_SERVER_URL = "http://localhost:7999/main";

	public static Test suite() {
		Logger.clearContents();
		Logger.setInMemory(false);
		TestSuite suite = new TestSuite(mot.test.DeligateGalaxyTest.class);
		return suite;
	}

	public DeligateGalaxyTest(String name) {
		super(name);
	}

	private ComputerPlayer p2, p3;
	private UserPlayer p1, p4;
	private SolarSystem ss1, ss2, ss3, ss4, ss5;
	private Planet h1, h2, h3, h4, h3b;

	private MotServerProxy serverProxy;
	private ServerProxy playerTwoServerProxy;
	private UserPlayerDelegate player1, player4;
	private ComputerPlayerDelegate player2, player3;

	private CompletedClientCallbackSyncroniser callbackSync = new CompletedClientCallbackSyncroniser();

	protected void setUp() throws ApplicationError, ValidationError {
		Logger.setDebug();
		MotEngine.newInstance();
		DeligateCacheManager.flush();
		setupConfigs();
		MainWindow.__createTestMainWindow(new TestGalaxyWindowConfig());
		new ProxyStore(new MotServerProxy(true, MainWindow.getExestingInstance(), this));
		playerTwoServerProxy = null;
		MotEngine.getInstance().getProxyManager().add(this);
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		Logger.flushLog();
		if (ProxyStore.hasInstance())
			ProxyStore.getInstance().disconnect();
	}

	public void createMap() throws Exception {
		createMap(null);
	}

	public void createMap(SetupOption option) throws Exception {
		DeligateCacheManager.flush();

		setupOneTierTechTree();

		String[] playerNames = { "Humans", "Bears" };
		RaceOptions options = OptionsFactory.getRaceOptions(playerNames[0]);
		if (option != null) options.getRaceOptions().setOption(option);
		p1 = newUserPlayer(playerNames[0], blue, null, options);
		p2 = newPlayer(playerNames[1], green, options, playerNames);

		ss1 = newSolarSystem("SS1", 100, 100);
		h1 = newPlanet("SS1-A", ss1, p1);

		ss2 = newSolarSystem("SS2", 200, 100);
		h2 = newPlanet("SS2-A", ss2, p2);

		ss3 = newSolarSystem("SS3", 150, 100);
		h3 = newPlanet("SS3-A", ss3, null, 100, 0.9);

		ss4 = newSolarSystem("SS4", 50, 100);

		// Events for p1 will come through the server connection (MotEventHandler
		// interface)
		// while events for p2 will come directly from the server (ClientProxy
		// interface).
		owner = p2;

		serverProxy = (MotServerProxy) ProxyStore.getInstance();
		CompletedServerCallbackSyncroniser callback = new CompletedServerCallbackSyncroniser();
		serverProxy.register("Humans", true, callback, null);
		callback.waitTillCompleteAndProcessCommandResponse(serverProxy);

		player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Humans");
		player2 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Bears");

		MainWindow.getExestingInstance()._setWindowOwner(player1);
		MainWindow.getExestingInstance().toGalaxyWindow();

		assertEquals("Player Human is correct", "Humans", player1.getName());
		assertEquals("Player Bears is correct", "Bears", player2.getName());

		PlayerDelegate[] players = (PlayerDelegate[]) order(DeligateCacheManager.getPlayers(), new PlayerDelegate[2], "Humans", "Bears");
	}

	public void createMapWith3Players() throws Exception {
		DeligateCacheManager.flush();

		setupOneTierTechTree();

		String[] playerNames = { "Humans", "Bears", "Ghosts" };
		RaceOptions options = OptionsFactory.getRaceOptions(playerNames[0]);
		p1 = newUserPlayer(playerNames[0], blue, null, options);
		p2 = newPlayer(playerNames[1], green, options, playerNames);
		p3 = newPlayer(playerNames[2], red, options, playerNames);

		ss1 = newSolarSystem("SS1", 100, 100);
		h1 = newPlanet("SS1-A", ss1, p1);

		ss2 = newSolarSystem("SS2", 200, 100);
		h2 = newPlanet("SS2-A", ss2, p2);

		ss3 = newSolarSystem("SS3", 300, 100);
		h3 = newPlanet("SS3-A", ss3, p3);

		ss4 = newSolarSystem("SS4", 50, 100);
		h4 = newPlanet("SS4-A", ss3, null, 100, 0.9);

		// Events for p1 will come through the server connection (MotEventHandler
		// interface)
		// while events for p2 will come directly from the server (ClientProxy
		// interface).
		owner = p2;

		serverProxy = (MotServerProxy) ProxyStore.getInstance();
		CompletedServerCallbackSyncroniser callback = new CompletedServerCallbackSyncroniser();
		serverProxy.register("Humans", true, callback, null);
		callback.waitTillCompleteAndProcessCommandResponse(serverProxy);

		player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Humans");
		player2 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Bears");
		player3 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Ghosts");

		MainWindow.getExestingInstance()._setWindowOwner(player1);
		MainWindow.getExestingInstance().toGalaxyWindow();

		assertEquals("Player Human is correct", "Humans", player1.getName());
		assertEquals("Player Bears is correct", "Bears", player2.getName());
		assertEquals("Player Ghosts is correct", "Ghosts", player3.getName());

		PlayerDelegate[] players = (PlayerDelegate[]) order(DeligateCacheManager.getPlayers(), new PlayerDelegate[3],
				"Humans", "Bears", "Ghosts");
	}

	public void createMap2UserPlayers() throws Exception {
		String[] playerNames = { "Humans", "Bears" };
		RaceOptions options = mot.common.setup.OptionsFactory.getRaceOptions(playerNames[0]);
		p1 = newUserPlayer(playerNames[0], blue, null, options);
		p4 = newUserPlayer(playerNames[1], green, null, options);

		ss1 = newSolarSystem("SS1", 100, 100);
		h1 = newPlanet("SS1-A", ss1, p1);

		ss2 = newSolarSystem("SS2", 200, 100);
		h2 = newPlanet("SS2-A", ss2, p4);

		ss3 = newSolarSystem("SS3", 150, 100);
		h3 = newPlanet("SS3-A", ss3, null, 100, 0.9);

		// Events for p1 will come through the server connection (MotEventHandler
		// interface)
		// while events for p4 will come directly from the server (ClientProxy
		// interface).
		owner = p4;

		serverProxy = (MotServerProxy) ProxyStore.getInstance();
		CompletedServerCallbackSyncroniser callback = new CompletedServerCallbackSyncroniser();
		serverProxy.register("Humans", true, callback, null);
		callback.waitTillCompleteAndProcessCommandResponse(serverProxy);

		player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Humans");
		player4 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Bears");

		MainWindow.getExestingInstance()._setWindowOwner(player1);
		MainWindow.getExestingInstance().toGalaxyWindow();

		assertEquals("Player Human is correct", "Humans", player1.getName());
		assertEquals("Player Bears is correct", "Bears", player4.getName());

		PlayerDelegate[] players = DeligateCacheManager.getPlayers();
		assertEquals("Have two players", 2, players.length);
	}

	public void createMapWith4by2Sectors() throws Exception {
		MotEngine.getInstance().getStellarObjectsNotNull().setSize(new Size(400, 200));

		String[] playerNames = { "Humans", "Bears", "Birds" };
		RaceOptions options = OptionsFactory.getRaceOptions(playerNames[0]);
		p1 = newUserPlayer(playerNames[0], blue, null, options);
		p2 = newPlayer(playerNames[1], green, options, playerNames);
		p3 = newPlayer(playerNames[2], red, options, playerNames);

		ss1 = newSolarSystem("SS1", 50, 50);
		h1 = newPlanet("SS1-A", ss1, p1);

		ss2 = newSolarSystem("SS2", 250, 50);
		h2 = newPlanet("SS2-A", ss2, p2);

		ss3 = newSolarSystem("SS3", 150, 150);
		h3 = newPlanet("SS3-A", ss3, p3);
		h3b = newPlanet("SS3-B", ss3, p3);

		ss4 = newSolarSystem("SS4", 350, 150);
		h4 = newPlanet("SS4-A", ss4, p3);

		// Events for p1 will come through the server connection (MotEventHandler
		// interface)
		// while events for p2 will come directly from the server (ClientProxy
		// interface).
		owner = p2;

		serverProxy = (MotServerProxy) ProxyStore.getInstance();
		CompletedServerCallbackSyncroniser callback = new CompletedServerCallbackSyncroniser();
		serverProxy.register("Humans", true, callback, null);
		callback.waitTillCompleteAndProcessCommandResponse(serverProxy);

		player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Humans");
		player2 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Bears");
		player3 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Birds");

		MainWindow.getExestingInstance()._setWindowOwner(player1);
		MainWindow.getExestingInstance().toGalaxyWindow();

		assertEquals("Player Human is correct", "Humans", player1.getName());
		assertEquals("Player Bears is correct", "Bears", player2.getName());
		assertEquals("Player Birds is correct", "Birds", player3.getName());

		PlayerDelegate[] players = DeligateCacheManager.getPlayers();
		assertEquals("Have two players", 3, players.length);
	}

	/**
	 * Test create new game works and that the following factions work in a game
	 * with two players (one computer and one human);
	 */
	public void testNewGame() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testNewGame() *************************");
		serverProxy = (MotServerProxy) ProxyStore.getInstance();
		Double level = new Double(1.0);
		int qtyComputerPlayers = 1;
		int qtyPlanets = 5;
		int qtyNebula = 1;
		Size galaxySize = new Size(800, 800);
		String player1Name = "Humans";
		SetupOptions userOptions = OptionsFactory.getSetupOptions(player1Name);
		NewGameUserPlayerData userPlayerData = new NewGameUserPlayerData(player1Name, userOptions, playerColourBlue, ANCIENT_WATER_MAN);
		serverProxy.newGameUserPlayer(userPlayerData, null, null);
		serverProxy._processCommandResponse();
		NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[1];
		cpd[0] = new NewGameDataComputerPlayerDetails("Bears", level, null);
		NewGameData gameData = new NewGameData(cpd, level, false, qtyPlanets, qtyNebula, galaxySize, false, false);
		serverProxy.newGame(userPlayerData, gameData, null, null);
		serverProxy._processCommandResponse();
		player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer(player1Name);
		assertEquals("Player Human is correct", player1Name, player1.getName());

		PlayerDelegate[] players = DeligateCacheManager.getPlayers();
		assertEquals("Have two players", 2, players.length);
		player2 = (ComputerPlayerDelegate) (players[1] instanceof ComputerPlayerDelegate ? players[1] : players[0]);

		PlanetDelegate[] planets = DeligateCacheManager.getPlanets();
		assertEquals("Have 6 planets players", 6, planets.length);

		PlanetDelegate[] player1Planets = DeligateCacheManager.getPlanets(player1);
		assertEquals("Have 1 planets players", 1, player1Planets.length);
		SolarSystemDelegate player1SS = player1Planets[0].getSolarSystem();
		assertEquals("Solar System has correct owner", player1Name, player1SS.getOwners()[0]);
		assertEquals("Solar System has correct owner", true, player1SS.isOwnedBy(player1));

		PlanetDelegate[] player2Planets = DeligateCacheManager.getPlanets(player2);
		assertEquals("Have 1 planets players", 1, player2Planets.length);
		SolarSystemDelegate player2SS = player1Planets[0].getSolarSystem();
		assertEquals("Solar System has correct owner", player1Name, player2SS.getOwners()[0]);
		assertEquals("Solar System has correct owner", true, player2SS.isOwnedBy(player1));

		CraftDelegate[] player1Crafts = DeligateCacheManager.getCrafts(player1);
		assertEquals("Have 2 crafts", 2, player1Crafts.length);
		assertEquals("Have 2 crafts", player1.getName(), player1Crafts[0].getOwner());
		assertEquals("Have 2 crafts", player1SS.getName(), player1Crafts[0].getOrbit().getStellarObject());

		assertEquals(false, player1.isAutomated());
		assertEquals(true, player2.isAutomated());

		Planet homePlanet = MotEngine.getInstance().getPlayersNotNull().getPlayer(player1Name).getCapitol();
		assertEquals("Player has home planet", homePlanet.getName(), player1.getHomePlanet().getName());

		Nebula[] nebulae = DeligateCacheManager.getGalaxyDetails().getNebulaeAsArray();
		assertEquals("Have 1 Nebula players", 1, nebulae.length);

		CraftDesign[] cda = player1.getCraftDesigns().getCraftDesigns();
		assertEquals(2, cda.length);
		assertEquals("Coloniser", cda[0].getName());
		assertEquals("Scout", cda[1].getName());
		
		assertEquals(2, MotEngine.instance.getPlayers().getQuantity());

		ComputerPlayer cp = MotEngine.instance.getPlayers().getComputerPlayers()[0];
		
		assertEquals(0.0, cp.getOriginProductionBonus(true), 0.01);
		assertEquals(0.0, cp.getResearchBonus(true), 0.01);
		assertEquals(0.0, cp.getOriginDevelopmentBonus(true), 0.01);

		assertEquals(3, cp.getPlayerCharacter().getQtyCraftPerPlanetToDeclareWar());
		assertEquals(3, cp.getPlayerCharacter().getQtyCraftToDeclareWar());
		assertEquals(false, cp.getPlayerCharacter().isIsolationist());
		assertEquals(1.0, cp.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenLike(), 0.001);
		assertEquals(1.0, cp.getPlayerCharacter().getAcceptBadSwapProbabilityWhenLike(), 0.001);
		assertEquals(1.0, cp.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(0.5, cp.getPlayerCharacter().getAcceptBadSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(0.1, cp.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenDislike(), 0.001);
		assertEquals(0.0, cp.getPlayerCharacter().getAcceptBadSwapProbabilityWhenDislike(), 0.001);
	}

	public void testNewGame2ComputerPlayersWithAiSettings() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testNewGame2ComputerPlayersWithAiSettings() *************************");
		serverProxy = (MotServerProxy) ProxyStore.getInstance();
		Double level = new Double(2.0);
		int qtyComputerPlayers = 2;
		int qtyPlanets = 10;
		int qtyNebula = 0;
		Size galaxySize = new Size(1000, 1000);
		String player1Name = "Humans";
		SetupOptions userOptions = OptionsFactory.getSetupOptions(player1Name);
		NewGameUserPlayerData userPlayerData = new NewGameUserPlayerData(player1Name, userOptions, playerColourBlue, ANCIENT_WATER_MAN);
		serverProxy.newGameUserPlayer(userPlayerData, null, null);
		serverProxy._processCommandResponse();
		NewGameDataComputerPlayerDetails[] cpd = new NewGameDataComputerPlayerDetails[2];
		cpd[0] = new NewGameDataComputerPlayerDetails("Bears", level, new PlayerAiSettings(1, 1, 1));
		cpd[1] = new NewGameDataComputerPlayerDetails("Birds", level, new PlayerAiSettings(-1, -1, -1));
		NewGameData gameData = new NewGameData(cpd, level, false, qtyPlanets, qtyNebula, galaxySize, false, false);
		serverProxy.newGame(userPlayerData, gameData, null, null);
		serverProxy._processCommandResponse();
		player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer(player1Name);
		assertEquals("Player Human is correct", player1Name, player1.getName());

		PlayerDelegate[] players = DeligateCacheManager.getPlayers();
		assertEquals("Have two players", 3, players.length);
		player2 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Bears");
		player3 = (ComputerPlayerDelegate) DeligateCacheManager.getPlayer("Birds");

		PlanetDelegate[] planets = DeligateCacheManager.getPlanets();
		assertEquals("Have 6 planets players", 10, planets.length);

		PlanetDelegate[] player1Planets = DeligateCacheManager.getPlanets(player1);
		assertEquals("Have 1 planets players", 1, player1Planets.length);
		SolarSystemDelegate player1SS = player1Planets[0].getSolarSystem();
		assertEquals("Solar System has correct owner", player1Name, player1SS.getOwners()[0]);
		assertEquals("Solar System has correct owner", true, player1SS.isOwnedBy(player1));

		PlanetDelegate[] player2Planets = DeligateCacheManager.getPlanets(player2);
		assertEquals("Have 1 planets players", 1, player2Planets.length);
		SolarSystemDelegate player2SS = player2Planets[0].getSolarSystem();
		assertEquals("Solar System has correct owner", "Bears", player2SS.getOwners()[0]);
		assertEquals("Solar System has correct owner", true, player2SS.isOwnedBy(player2));

		PlanetDelegate[] player3Planets = DeligateCacheManager.getPlanets(player3);
		assertEquals("Have 1 planets players", 1, player3Planets.length);
		SolarSystemDelegate player3SS = player3Planets[0].getSolarSystem();
		assertEquals("Solar System has correct owner", "Birds", player3SS.getOwners()[0]);
		assertEquals("Solar System has correct owner", true, player3SS.isOwnedBy(player3));

		CraftDelegate[] player1Crafts = DeligateCacheManager.getCrafts(player1);
		assertEquals("Have 2 crafts", 2, player1Crafts.length);
		assertEquals("Have 2 crafts", player1.getName(), player1Crafts[0].getOwner());
		assertEquals("Have 2 crafts", player1SS.getName(), player1Crafts[0].getOrbit().getStellarObject());

		assertEquals(false, player1.isAutomated());
		assertEquals(true, player2.isAutomated());
		assertEquals(true, player3.isAutomated());

		Planet homePlanet = MotEngine.getInstance().getPlayersNotNull().getPlayer(player1Name).getCapitol();
		assertEquals("Player has home planet", homePlanet.getName(), player1.getHomePlanet().getName());

		Nebula[] nebulae = DeligateCacheManager.getGalaxyDetails().getNebulaeAsArray();
		assertNull(nebulae);

		CraftDesign[] cda = player1.getCraftDesigns().getCraftDesigns();
		assertEquals(2, cda.length);
		assertEquals("Coloniser", cda[0].getName());
		assertEquals("Scout", cda[1].getName());

		ComputerPlayer cp2 = (ComputerPlayer) MotEngine.instance.getPlayers().getPlayer("Bears");
		assertEquals(2, cp2.getPlayerCharacter().getQtyCraftPerPlanetToDeclareWar());
		assertEquals(1, cp2.getPlayerCharacter().getQtyCraftToDeclareWar());
		assertEquals(false, cp2.getPlayerCharacter().isIsolationist());
		assertEquals(1.0, cp2.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenLike(), 0.001);
		assertEquals(1.0, cp2.getPlayerCharacter().getAcceptBadSwapProbabilityWhenLike(), 0.001);
		assertEquals(1.0, cp2.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(1.0, cp2.getPlayerCharacter().getAcceptBadSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(1.0, cp2.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenDislike(), 0.001);
		assertEquals(0.1, cp2.getPlayerCharacter().getAcceptBadSwapProbabilityWhenDislike(), 0.001);

		assertEquals(1.0, cp2.getOriginProductionBonus(true), 0.01);
		assertEquals(1.0, cp2.getResearchBonus(true), 0.01);
		assertEquals(1.0, cp2.getOriginDevelopmentBonus(true), 0.01);

		ComputerPlayer cp3 = (ComputerPlayer) MotEngine.instance.getPlayers().getPlayer("Birds");
		assertEquals(4, cp3.getPlayerCharacter().getQtyCraftPerPlanetToDeclareWar());
		assertEquals(4, cp3.getPlayerCharacter().getQtyCraftToDeclareWar());
		assertEquals(true, cp3.getPlayerCharacter().isIsolationist());
		assertEquals(1.0, cp3.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenLike(), 0.001);
		assertEquals(0.0, cp3.getPlayerCharacter().getAcceptBadSwapProbabilityWhenLike(), 0.001);
		assertEquals(0.5, cp3.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(0.0, cp3.getPlayerCharacter().getAcceptBadSwapProbabilityWhenNeutral(), 0.001);
		assertEquals(0.0, cp3.getPlayerCharacter().getAcceptGoodSwapProbabilityWhenDislike(), 0.001);
		assertEquals(0.0, cp3.getPlayerCharacter().getAcceptBadSwapProbabilityWhenDislike(), 0.001);

		assertEquals(1.0, cp3.getOriginProductionBonus(true), 0.01);
		assertEquals(1.0, cp3.getResearchBonus(true), 0.01);
		assertEquals(1.0, cp3.getOriginDevelopmentBonus(true), 0.01);

	}


	/**
	 * Test Relations Commands - Tests that a user player can request each type of
	 * treaty
	 */
	public void testDiplomaticRequestsSend() throws Exception {
		Logger.logDebug( "*******************************  DeligateTest.testDiplomaticRequestsSend() *************************");
		createMap();
		
		//
		// Test no diplomatic relations
		//

		MotEngine.instance.getRelations().endTurn();
		
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertFalse(player1.hasDiplomaticRelationsWith(player2));

		// Form diplomatic relations between players
		p1.addScanner(tech_photon200);
		assertTrue(MotEngine.instance.getRelations().getRelationshipNotNull(p1, p2).hasDiplomaticRelations());

		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertTrue(player1.hasDiplomaticRelationsWith(player2));

		RelationDelegate rd = player1.getRelation(player2);
		assertEquals(false, rd.isAliance());
		assertEquals(false, rd.isPeace());
		assertEquals(false, rd.isNonAgression());
		assertEquals(false, rd.isWar());
		assertNull(rd.getResearchAgreement());
		assertNull(rd.getTradeAgreement());
		assertEquals(true, rd.canHaveAllianceTreaty());
		assertEquals(true, rd.canHaveNonAgressionTreaty());
		assertEquals(true, rd.canHaveTradeTreaty());
		assertEquals(true, rd.canHaveWarTreaty());
		assertEquals(false, rd.canHavePeaceTreaty());

		rd = player1.getRelation(player2);
		assertEquals(50, rd.getValue());
		
        assertNull(MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p1).getDiplomaticHistory());

		MotEngine.instance.getRelationsNotNull().getRelationship(p1, p2).setValue(Relations.WOW + 100);
		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.getWar(), null, null, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		rd = player1.getRelation(player2);
		assertEquals(false, rd.isAliance());
		assertEquals(false, rd.isPeace());
		assertEquals(false, rd.isNonAgression());
		assertEquals(true, rd.isWar());
		assertNull(rd.getResearchAgreement());
		assertNull(rd.getTradeAgreement());
		assertEquals(Relations.WOW + 100, rd.getValue());
		assertEquals(false, rd.canHaveAllianceTreaty());
		assertEquals(false, rd.canHaveNonAgressionTreaty());
		assertEquals(false, rd.canHaveTradeTreaty());
		assertEquals(false, rd.canHaveWarTreaty());
		assertEquals(true, rd.canHavePeaceTreaty());

		((WarTreaty) MotEngine.instance.getRelations().getRelationship(p1, p2).getTreatiesAsArray()[0]).setDateStarted(-100);
		
		assertEquals(3000, "Humans declared War on Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);

		MotEngine.instance.setDate(MotEngine.instance.getDate()+1);
		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.getPeace(), null, null, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		rd = player1.getRelation(player2);
		assertEquals(false, rd.isAliance());
		assertEquals(true, rd.isPeace());
		assertEquals(false, rd.isNonAgression());
		assertEquals(false, rd.isWar());
		assertNull(rd.getResearchAgreement());
		assertNull(rd.getTradeAgreement());
		assertEquals(true, rd.canHaveAllianceTreaty());
		assertEquals(true, rd.canHaveNonAgressionTreaty());
		assertEquals(true, rd.canHaveTradeTreaty());
		assertEquals(true, rd.canHaveWarTreaty());
		assertEquals(false, rd.canHavePeaceTreaty());
		
		assertEquals(3000, "Humans declared War on Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3001, "Bears agreed to a Peace Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);

		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.getNonAggression(), null, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		rd = player1.getRelation(player2);
		assertEquals(false, rd.isAliance());
		assertEquals(false, rd.isPeace());
		assertEquals(true, rd.isNonAgression());
		assertEquals(false, rd.isWar());
		assertNull(rd.getResearchAgreement());
		assertNull(rd.getTradeAgreement());
		
		assertEquals(3000, "Humans declared War on Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3001, "Bears agreed to a Peace Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);
		assertEquals(3001, "Bears agreed to a Non Aggression Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[2], p1, p2);

		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.getTrade(), null, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		rd = player1.getRelation(player2);
		assertEquals(false, rd.isAliance());
		assertEquals(false, rd.isPeace());
		assertEquals(true, rd.isNonAgression());
		assertEquals(false, rd.isWar());
		assertNull(rd.getResearchAgreement());
		assertEquals(2, rd.getTradeAgreement().getMoney());
		
		assertEquals(3000, "Humans declared War on Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3001, "Bears agreed to a Peace Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);
		assertEquals(3001, "Bears agreed to a Non Aggression Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[2], p1, p2);
		assertEquals(3001, "Bears agreed to a Trade Agreement Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[3], p1, p2);

		MotEngine.instance.setDate(MotEngine.instance.getDate()+1);
		
		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.getAlliance(), null, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		rd = player1.getRelation(player2);
		assertEquals(true, rd.isAliance());
		assertEquals(false, rd.isPeace());
		assertEquals(false, rd.isNonAgression());
		assertEquals(false, rd.isWar());
		assertNull(rd.getResearchAgreement());
		assertEquals(2, rd.getTradeAgreement().getMoney());

		assertTrue(player1.hasDiplomaticRelationsWith(player2));

		assertEquals(3000, "Humans declared War on Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3001, "Bears agreed to a Peace Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);
		assertEquals(3001, "Bears agreed to a Non Aggression Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[2], p1, p2);
		assertEquals(3001, "Bears agreed to a Trade Agreement Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[3], p1, p2);
		assertEquals(3002, "Bears agreed to an Alliance Treaty from Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[4], p1, p2);

		//
		// Test diplomatic statements
		//

		MotEngine.instance.getRelationsNotNull().getRelationship(p1, p2).removeTreaty(AllianceTreaty.class);
		MotEngine.instance.getRelationsNotNull().getRelationship(p1, p2).setValue(10);

		clearEvents();
		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.getResearch(), null, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		DiplomaticResponseEvent event = (DiplomaticResponseEvent) getEvent(DiplomaticResponseEvent.class);
		assertEquals("rejected", event.getResponseVerb());

		rd = player1.getRelation(player2);
		assertEquals(false, rd.isAliance());
		assertEquals(false, rd.isPeace());
		assertEquals(false, rd.isNonAgression());
		assertEquals(false, rd.isWar());
		assertNull(rd.getResearchAgreement());
		assertEquals(2, rd.getTradeAgreement().getMoney());
		assertEquals(10, rd.getValue());

		StrategistEventVO[] diplomaticEvents = player1.getStrategistEvents(player2);
		assertEquals(2, diplomaticEvents.length);
		assertEquals(3000, diplomaticEvents[0].getDate(), 0.5);
		assertEquals("<html>Empire Established<br/>  Level: Easy<br/>  Foreign Policy: Normal</html>", diplomaticEvents[0].getMessage());
		assertEquals("Declaration of war (-5)", diplomaticEvents[1].getMessage());

		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.getResearch(), null, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		diplomaticEvents = player1.getStrategistEvents(player2);
		assertEquals(3, diplomaticEvents.length);
		assertEquals("<html>Empire Established<br/>  Level: Easy<br/>  Foreign Policy: Normal</html>", diplomaticEvents[0].getMessage());
		assertEquals("Declaration of war (-5)", diplomaticEvents[1].getMessage());
		assertEquals("Annoying behaviour (-1)", diplomaticEvents[2].getMessage());

	}

	/**
	 * Test Relations Events - Tests that a user player can receive requests for
	 * each type of treaty
	 */
	public void testDiplomaticRequestsReceive() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testDiplomaticRequestsReceive() *************************");
		createMap();

		Relation rel = MotEngine.instance.getRelations().getRelationshipNotNull(p1, p2);
		owner = p1;

		//
		// TEST NON AGRESSION TREATY
		//

		lastEvent = null;
		p1.diplomaticRequest(p2, DiplomaticProposal.getNonAggression(), null);
		// p1.requestNonAgression(p2, null);
		assertEquals(player2.getName(), ((DiplomaticRequestEvent) lastEvent).getPlayer());
		assertEquals(true, ((DiplomaticRequestEvent) lastEvent).getProposal().isNonAggression());

		serverProxy.acceptDiplomaticRequest(player2, ((DiplomaticRequestEvent) lastEvent).getProposal(), null,callbackSync, null);
		// serverProxy.acceptNonAgressionTreaty(player2, callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(true, rel.isNonAgression());

		assertEquals(3000, "Humans agreed to a Non Aggression Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		

		//
		// TEST ALLIANCE TREATY
		//

		lastEvent = null;
		p1.diplomaticRequest(p2, DiplomaticProposal.getAlliance(), null);
		// p1.requestAliance(p2, null);
		assertEquals(player2.getName(), ((DiplomaticRequestEvent) lastEvent).getPlayer());
		assertEquals(true, ((DiplomaticRequestEvent) lastEvent).getProposal().isAlliance());

		serverProxy.acceptDiplomaticRequest(player2, ((DiplomaticRequestEvent) lastEvent).getProposal(), null,callbackSync, null);
		// serverProxy.acceptAllianceTreaty(player2, callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(true, rel.isAliance());

		assertEquals(3000, "Humans agreed to a Non Aggression Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3000, "Humans agreed to an Alliance Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);		

		//
		// TEST WAR TREATY
		//

		lastEvent = null;
		p1.diplomaticRequest(p2, DiplomaticProposal.getWar(), null);
		// p1.declareWar(p2, null, false);
		assertEquals(player2.getName(), ((DiplomaticRequestEvent) lastEvent).getPlayer());
		assertEquals(true, ((DiplomaticRequestEvent) lastEvent).getProposal().isWar());

		assertEquals(true, rel.isWar());

		assertEquals(3000, "Humans agreed to a Non Aggression Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3000, "Humans agreed to an Alliance Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);		
		assertEquals(3000, "Bears declared War on Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[2], p1, p2);		

		//
		// TEST PEACE TREATY
		//

		lastEvent = null;
		p1.diplomaticRequest(p2, DiplomaticProposal.getPeace(), null);
		// p1.requestPeace(p2, null);
		assertEquals(player2.getName(), ((DiplomaticRequestEvent) lastEvent).getPlayer());
		assertEquals(true, ((DiplomaticRequestEvent) lastEvent).getProposal().isPeace());

		serverProxy.acceptDiplomaticRequest(player2, ((DiplomaticRequestEvent) lastEvent).getProposal(), null,callbackSync, null);
		// serverProxy.acceptPeaceTreaty(player2, callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(true, rel.isPeace());

		assertEquals(3000, "Humans agreed to a Non Aggression Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3000, "Humans agreed to an Alliance Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);		
		assertEquals(3000, "Bears declared War on Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[2], p1, p2);		
		assertEquals(3000, "Humans agreed to a Peace Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[3], p1, p2);		

		//
		// TEST TRADE TREATY
		//

		lastEvent = null;
		p1.diplomaticRequest(p2, DiplomaticProposal.getTrade(), null);
		// p1.requestTradeRoute(p2, null);
		assertEquals(player2.getName(), ((DiplomaticRequestEvent) lastEvent).getPlayer());
		assertEquals(true, ((DiplomaticRequestEvent) lastEvent).getProposal().isTrade());

		serverProxy.acceptDiplomaticRequest(player2, ((DiplomaticRequestEvent) lastEvent).getProposal(), null,callbackSync, null);
		// serverProxy.acceptTradeTreaty(player2, callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(3000, rel.getTradeAgreementTreaty().getDateStarted());

		assertEquals(3000, "Humans agreed to a Non Aggression Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3000, "Humans agreed to an Alliance Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);		
		assertEquals(3000, "Bears declared War on Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[2], p1, p2);		
		assertEquals(3000, "Humans agreed to a Peace Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[3], p1, p2);		
		assertEquals(3000, "Humans agreed to a Trade Agreement Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[4], p1, p2);		


		//
		// TEST RESEARCH TREATY
		//

		lastEvent = null;
		p1.diplomaticRequest(p2, DiplomaticProposal.getResearch(), null);
		// p1.requestResearchTreaty(p2, null);
		assertEquals(player2.getName(), ((DiplomaticRequestEvent) lastEvent).getPlayer());
		assertEquals(true, ((DiplomaticRequestEvent) lastEvent).getProposal().isResearch());

		serverProxy.acceptDiplomaticRequest(player2, ((DiplomaticRequestEvent) lastEvent).getProposal(), null,callbackSync, null);
		// serverProxy.acceptResearchTreaty(player2, callbackSync, null);
		serverProxy._processCommandResponse();
		// assertEquals(true, rel.isResearch());

		assertEquals(3000, "Humans agreed to a Non Aggression Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[0], p1, p2);		
		assertEquals(3000, "Humans agreed to an Alliance Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[1], p1, p2);		
		assertEquals(3000, "Bears declared War on Humans", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[2], p1, p2);		
		assertEquals(3000, "Humans agreed to a Peace Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[3], p1, p2);		
		assertEquals(3000, "Humans agreed to a Trade Agreement Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[4], p1, p2);		
		assertEquals(3000, "Humans agreed to a Research Agreement Treaty from Bears", MotEngine.instance.getRelationsNotNull().getRelationshipNotNull(p1, p2).getDiplomaticHistory().getEvents()[5], p1, p2);		
	}

	/**
	 * Test Relations Commands
	 */
	public void testFetchPlayerTechnology() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testFetchPlayerTechnology() *************************");
		createMap();

		try {
			serverProxy.fetchTechnologies(player2, callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Should get a validation error");
		} catch (ValidationError e) {
			// Success
		}

		Spies spies = MotEngine.instance.getSpiesNotNull();
		spies.addDefendingSpies(p1, 10);
		p1.getSpyCenter().addPlayerSpy(p2);

		serverProxy.fetchTechnologies(player2, callbackSync, null);
		serverProxy._processCommandResponse();
		TechnologyList techs = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		assertEquals(p2.getTechnologies(), techs);

	}

	/**
	 * Test Galaxy Craft Commands
	 */
	public void testCanViewGalaxy() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testCanViewGalaxy() *************************");
		createMap();

		StellarObjectDelegate[] sods = (StellarObjectDelegate[]) Comp.sortByName(DeligateCacheManager.getStellarObjects());

		assertEquals(4, sods.length);
		assertEquals("SS1", sods[0].getName());
		assertEquals("SS2", sods[1].getName());
		assertEquals("SS3", sods[2].getName());
		assertEquals("SS4", sods[3].getName());

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");

		assertEquals("SS1", ssd1.getName());
		assertEquals(new Location(100, 100), ssd1.getLocation());
		assertEquals("Humans", ssd1.getOwners()[0]);

		assertEquals("SS1-A", pd1.getName());

		//
		// Test we can see Nebula
		//

		Nebula nebula = MotFactory.createNebula(new Location(100, 100), 50);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals(1, DeligateCacheManager.getGalaxyDetails().getNebulaeAsArray().length);
		assertEquals(nebula, DeligateCacheManager.getGalaxyDetails().getNebulaeAsArray()[0]);
		assertEquals(100, 100, DeligateCacheManager.getGalaxyDetails().getNebulaeAsArray()[0].getLocation());
		assertEquals(50, 50, DeligateCacheManager.getGalaxyDetails().getNebulaeAsArray()[0].getDiameter());

		//
		// Test we can see Black Holes
		//

		BlackHole blackHole = BlackHoleFactory.createBlackHole(new Location(100, 100));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		sods = (StellarObjectDelegate[]) Comp.sortByName(DeligateCacheManager.getStellarObjects());

		assertEquals(5, sods.length);
		assertEquals("Black Hole1", sods[0].getName());
		assertEquals("SS1", sods[1].getName());
		assertEquals("SS2", sods[2].getName());
		assertEquals("SS3", sods[3].getName());
		assertEquals("SS4", sods[4].getName());

		assertEquals(blackHole.getLocation(), sods[0].getLocation());

		//
		// test asteroids appear
		//

		assertFalse(pd1.hasAsteroids());

		h1.setHasAsteroids(true);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertTrue(pd1.hasAsteroids());
	}

	/**
	 * Test Galaxy Craft Commands
	 */
	public void testGalaxyCraftCommands() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testGalaxyCraftCommands() *************************");
		createMap();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");

		PlanetDelegate hd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate hd2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate hd3 = DeligateCacheManager.getPlanet("SS3-A");

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];
		PlayerDelegate pd2 = pds[1];

		//
		// Create a craft1 at ss1
		// Test proxy returns a single craft at correct location
		//

		Craft craft1 = newCraft(defenderDesign, p1, ss1);
		Craft craft2 = newCraft(defenderDesign, p1, ss1);
		String craft1Name = craft1.getName();
		String craft2Name = craft2.getName();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] crafts = DeligateCacheManager.getMoos();
		assertEquals(2, crafts.length);
		assertEquals(craft1Name, crafts[0].getName());
		assertEquals(craft2Name, crafts[1].getName());
		assertEquals(150, crafts[0].getRange());
		assertEquals(150, crafts[1].getRange());

		//
		// Move craft1 to SS2
		// Test moving a craft to a stellar object
		//

		serverProxy.mooMoveCmd(crafts[0], ssd2, null, false, callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = DeligateCacheManager.getCrafts(pd1);
		assertEquals("SS1", crafts[0].getOrbit().getStellarObject());
		assertNull(crafts[0].getOrbit().getPlanet());
		assertEquals("SS2", crafts[0].getNavigator().getDestination().getStellarObject());
		assertNull(crafts[0].getNavigator().getDestination().getPlanet());
		assertEquals(4, crafts[0].getNavigator().getEta());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		crafts = DeligateCacheManager.getCrafts(pd1);
		assertNull(crafts[0].getOrbit());
		assertEquals("SS2", crafts[0].getNavigator().getDestination().getStellarObject());
		assertNull(crafts[0].getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		crafts = DeligateCacheManager.getCrafts(pd1);
		assertEquals(2, crafts.length);
		assertEquals(craft1Name, crafts[0].getName());
		assertEquals(craft2Name, crafts[1].getName());

		assertNull(((CraftDelegate) crafts[0]).getFleet());
		assertEquals("SS2", crafts[0].getOrbit().getStellarObject());
		assertNull(crafts[0].getOrbit().getPlanet());
		assertNull(crafts[0].getNavigator());

		//
		// Move craft1 to SS3-A
		// Test moving a craft to a planet
		//

		serverProxy.mooMoveCmd(crafts[0], ssd3, hd3, false, callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = DeligateCacheManager.getCrafts(pd1);
		assertEquals("SS2", crafts[0].getOrbit().getStellarObject());
		assertNull(crafts[0].getOrbit().getPlanet());
		assertEquals("SS3", crafts[0].getNavigator().getDestination().getStellarObject());
		assertEquals("SS3", crafts[0].getNavigator().getDestination().getStellarObject());
		assertEquals("SS3-A", crafts[0].getNavigator().getDestination().getPlanet());
		assertEquals("SS3-A", crafts[0].getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		crafts = DeligateCacheManager.getCrafts(pd1);
		assertEquals(null, crafts[0].getOrbit());
		assertEquals("SS3", crafts[0].getNavigator().getDestination().getStellarObject());
		assertEquals("SS3-A", crafts[0].getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertNull(((CraftDelegate) crafts[0]).getFleet());
		assertEquals("SS3", crafts[0].getOrbit().getStellarObject());
		assertEquals("SS3-A", crafts[0].getOrbit().getPlanet());
		assertEquals(null, crafts[0].getNavigator());

		//
		// Move craft1 to SS1
		// Test moving a craft to a solar system and have engine auto select an planet
		// orbit
		//

		serverProxy.mooMoveCmd(crafts[0], ssd1, null, true, callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = DeligateCacheManager.getCrafts(pd1);
		assertEquals("SS3", crafts[0].getOrbit().getStellarObject());
		assertEquals("SS3-A", crafts[0].getOrbit().getPlanet());
		assertEquals("SS1", crafts[0].getNavigator().getDestination().getStellarObject());
		assertEquals("SS1-A", crafts[0].getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		crafts = DeligateCacheManager.getCrafts(pd1);
		assertEquals(null, crafts[0].getOrbit());
		assertEquals("SS1", crafts[0].getNavigator().getDestination().getStellarObject());
		assertEquals("SS1-A", crafts[0].getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertNull(((CraftDelegate) crafts[0]).getFleet());
		assertEquals("SS1", crafts[0].getOrbit().getStellarObject());
		assertEquals("SS1-A", crafts[0].getOrbit().getPlanet());
		assertEquals(null, crafts[0].getNavigator());
		assertEquals("SS1", crafts[1].getOrbit().getStellarObject());
		assertNull(crafts[1].getOrbit().getPlanet());
		assertNull(crafts[1].getNavigator());

		//
		// Test move craft within an SS
		//

		serverProxy.mooMoveCmd(crafts[1], ssd1, hd1, false, callbackSync, null);
		serverProxy._processCommandResponse();

		Rectangle r = new Rectangle(75, 75, 50, 50);
		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMOOsAt(r, false, hd1.getOrbit());
		assertEquals(2, moos.length);
		assertEquals(craft1Name, moos[0].getName());
		assertEquals(craft2Name, moos[1].getName());

		//
		// Test delete a craft
		//

		assertEquals(106, pd1.getFunds()); // does not matter what this value is, but must increase after sale

		serverProxy.mooDeleteCmd(((CraftDelegate) crafts[1]), callbackSync, null);
		serverProxy._processCommandResponse();

		moos = DeligateCacheManager.getMOOsAt(r, false, hd1.getOrbit());
		assertEquals(1, moos.length);
		assertEquals(craft1Name, moos[0].getName());
		assertEquals(906, pd1.getFunds());

	}

	/**
	 * Test that when a craft arrives in a solar system status is set to visited and
	 * prizes are released
	 */
	public void testVisited() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testVisited() *************************");
		createMap();

		p1.setFunds(500);

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate hd1 = DeligateCacheManager.getPlanet("SS1-A");

		SolarSystem _ssPlain = newSolarSystem("Plain", 100, 120);
		SolarSystem _ssPrize = newSolarSystem("Prize", 100, 140);
		SolarSystem _ssGuardedPrize = newSolarSystem("GuardedPrize", 100, 160);
		Planet _guardedPrizePlanet = newPlanet("GuardedPrize-A", _ssGuardedPrize, null, 100, 0.9);

		_ssPrize.setPrize(new MoneyPrize(1000));
		_ssGuardedPrize.setPrize(new MoneyPrize(1000));

		MonsterPlayer player = MotEngine.instance.getMonsterPlayer();
		Craft craft = CraftFactory.create("monster_small", player, player, _guardedPrizePlanet.getOrbit(), false,
				false);

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];

		CraftDesign cd1 = new CraftDesign("TestFighter", tech_flyer30, tech_fusion25, tech_massDriver1);
		Craft _craft1 = newCraft(fighterDesign, p1, ss1);
		CraftDesign cd2 = new CraftDesign("TestFighter", tech_flyer100, tech_fusion400, tech_electronic40,
				tech_energy20, tech_ancientWeapon, tech_ancientWeapon, tech_ancientWeapon, tech_ancientWeapon,
				tech_armour200);
		Craft _craft2 = newCraft(cd2, p1, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssPlain = DeligateCacheManager.getStellarObject(_ssPlain.getName());
		StellarObjectDelegate ssPrize = DeligateCacheManager.getStellarObject(_ssPrize.getName());
		StellarObjectDelegate ssGuardedPrize = DeligateCacheManager.getStellarObject(_ssGuardedPrize.getName());
		PlanetDelegate guardedPrizePlanet = DeligateCacheManager.getPlanet(_guardedPrizePlanet.getName());

		CraftDelegate craft1 = DeligateCacheManager.getCraft(_craft1.getName());
		CraftDelegate craft2 = DeligateCacheManager.getCraft(_craft2.getName());

		assertTrue(ssd1.isVisible());
		assertFalse(ssPlain.isVisible());
		assertFalse(ssPrize.isVisible());
		assertFalse(ssGuardedPrize.isVisible());

		//
		// When craft arrives it has visited the solar system
		//

		serverProxy.mooMoveCmd(craft1, ssPlain, null, false, null, null);
		serverProxy._processCommandResponse();

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(ssPlain.getName(), craft1.getOrbit().getStellarObject());

		assertTrue(ssd1.isVisible());
		assertTrue(ssPlain.isVisible());
		assertFalse(ssPrize.isVisible());
		assertFalse(ssGuardedPrize.isVisible());

		//
		// When craft arrives it gets a prize
		//

		assertEquals(500, player1.getFunds(), 100);
		p1.setFunds(500);

		serverProxy.mooMoveCmd(craft1, ssPrize, null, false, null, null);
		serverProxy._processCommandResponse();

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(ssPrize.getName(), craft1.getOrbit().getStellarObject());
		assertEquals(1500, player1.getFunds(), 100);

		assertTrue(ssd1.isVisible());
		assertTrue(ssPlain.isVisible());
		assertTrue(ssPrize.isVisible());
		assertFalse(ssGuardedPrize.isVisible());

		//
		// When prize NOT released but IS visited when lose battle at guarded planet
		//

		p1.setFunds(500);

		serverProxy.mooMoveCmd(craft1, ssGuardedPrize, null, false, null, null);
		serverProxy._processCommandResponse();

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();

		NewBattleEvent nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
		BattleDelegate battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());

		serverProxy.battleAutomateCmd(battle, null, null);
		serverProxy._processCommandResponse();

		assertEquals(true, battle.hasEnded());
		BattleEndEvent endEvent = (BattleEndEvent) getEvent(BattleEndEvent.class);

		assertEquals(craft1.getName(), endEvent.getCraftDestroyd()[0]);
		assertEquals(1, DeligateCacheManager.getCrafts(player1).length);
		assertEquals(craft2, DeligateCacheManager.getCrafts(player1)[0]);

		assertEquals(500, player1.getFunds(), 100);

		assertTrue(ssd1.isVisible());
		assertTrue(ssPlain.isVisible());
		assertTrue(ssPrize.isVisible());
		assertFalse(ssGuardedPrize.isVisible());

		assertEquals(1000, ((MoneyPrize) _ssGuardedPrize.getPrizes()[0]).getAmount());

		//
		// When prize released and visited when win battle at guarded planet
		//

		p1.setFunds(500);

		serverProxy.mooMoveCmd(craft2, ssGuardedPrize, null, false, null, null);
		serverProxy._processCommandResponse();

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();

		nbe = (NewBattleEvent) getEvent(NewBattleEvent.class);
		battle = DeligateCacheManager.getBattle(nbe.getBattle().getName());

		serverProxy.battleAutomateCmd(battle, null, null);
		serverProxy._processCommandResponse();

		assertEquals(true, battle.hasEnded());
		endEvent = (BattleEndEvent) getEvent(BattleEndEvent.class);

		assertEquals("monster_small1", endEvent.getCraftDestroyd()[0]);
		assertEquals(1, DeligateCacheManager.getCrafts(player1).length);
		assertEquals(craft2, DeligateCacheManager.getCrafts(player1)[0]);

		assertEquals(1500, player1.getFunds(), 100);

		assertTrue(ssd1.isVisible());
		assertTrue(ssPlain.isVisible());
		assertTrue(ssPrize.isVisible());
		assertTrue(ssGuardedPrize.isVisible());

		assertEquals(null, _ssGuardedPrize.getPrizes());

	}

	/**
	 * Test Galaxy Craft Commands
	 */
	public void testGalaxyCraftVisibility() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testGalaxyCraftVisibility() *************************");
		createMap();

		ss5 = newSolarSystem("SS5", 300, 100);

		//
		// Create a craft1 with cloak and craft2 with no cloak at ss1
		// Test a craft is visible to a player when in orbit around his planet
		//

		Craft craft1 = newCraft(defenderDesign, p2, ss1);
		craft1.getShields().add(tech_cloak150);
		Craft craft2 = newCraft(defenderDesign, p2, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] cda = DeligateCacheManager.getMoos();
		CraftDelegate cd1 = (CraftDelegate) cda[0];
		CraftDelegate cd2 = (CraftDelegate) cda[1];
		assertEquals(craft1.getName(), cd1.getName());
		assertEquals(craft2.getName(), cd2.getName());

		assertTrue(craft1.isVisible(p1));
		assertTrue(craft2.isVisible(p1));

		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos();
		assertEquals(craft1, craft2, moos);

		//
		// Test crafts are not displayed for a player if they are not visible
		// Can not see remote crafts if I do not have scanners
		//

		craft1.transferTo(new Orbit(ss2, null));
		craft2.transferTo(new Orbit(ss2, null));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertFalse(craft1.isVisible(p1));
		assertFalse(craft2.isVisible(p1));

		moos = DeligateCacheManager.getMoos();
		assertNull(moos);

		//
		// Test crafts are not displayed for a player if they are not visible
		// Can not see remote crafts if their cloak is better than my scanner
		//

		h1.setScanners(new Scanners(tech_photon150));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertFalse(craft1.isVisible(p1));
		assertTrue(craft2.isVisible(p1));

		moos = DeligateCacheManager.getMoos();
		assertEquals(craft2, moos);

		//
		// Test crafts are not displayed for a player if they are not visible
		// Cannot see craft in fleet if craft is cloaked
		//

		new CraftJoinFleetCmd(craft1, p2).run();
		new CraftJoinFleetCmd(craft2, p2).run();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		moos = DeligateCacheManager.getMoos();
		assertEquals(1, moos.length);
		assertEquals("Fleet1", moos[0].getName());
		// This is correct - but what the heck!!!
		// assertEquals(1, ((FleetDeligate)moos[0]).getCrafts().length);
		// assertEquals(craft2.getName(), ((FleetDeligate)moos[0]).getCrafts()[0]);

		//
		// Test on far away solar system (ss5 - out of scan range) enemy crafts visible
		// if my craft is in orbit with it (or transport)
		//

		(craft1.getFleet()).destroy();
		craft1.transferTo(new Orbit(ss5, null));
		craft2.transferTo(new Orbit(ss5, null));
		CraftDesign cd = new CraftDesign("NoScannerCraft", tech_flyer30, tech_fusion25);
		Craft craft3 = newCraft(cd, p1, ss5);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertTrue(craft1.isVisible(p1));
		assertTrue(craft2.isVisible(p1));
		assertTrue(craft3.isVisible(p1));

		moos = DeligateCacheManager.getMoos();
		assertEquals(craft3, craft1, craft2, moos);

		serverProxy.mooDeleteCmd(moos[0], null, null);
		serverProxy._processCommandResponse();

		assertEquals(0, p1.getCrafts().length);
		assertFalse(craft1.isVisible(p1));
		assertFalse(craft2.isVisible(p1));

		moos = DeligateCacheManager.getMoos();
		assertNull(moos);

		Transport transport = newTransportWithStationLoaded(h1, tech_spaceStation);
		transport.transferTo(new Orbit(ss5, null));

		assertTrue(craft1.isVisible(p1));
		assertTrue(craft2.isVisible(p1));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		moos = DeligateCacheManager.getMoos();
		assertEquals(transport, craft1, craft2, moos);

	}

	/**
	 * Test Galaxy Fleet Commands and craft locations with multiple crafts
	 */
	public void testGalaxyFleetCommands() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testGalaxyFleetCommands() *************************");
		createMap();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");

		PlanetDelegate hd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate hd2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate hd3 = DeligateCacheManager.getPlanet("SS3-A");

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];
		PlayerDelegate pd2 = pds[1];

		//
		// Create a craft1, craft2, craft3 at ss1
		// Test proxy returns three crafts at correct location
		//

		Craft craft1 = newCraft(fighterDesign, p1, ss1);
		Craft craft2 = newCraft(defenderDesign, p1, ss1);
		Craft craft3 = newCraft(defenderDesign, p1, ss3);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		String craft1Name = craft1.getName();
		String craft2Name = craft2.getName();
		String craft3Name = craft3.getName();

		CraftDelegate[] crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("SS1", crafts[0].getOrbit().getStellarObject());
		assertNull(crafts[0].getOrbit().getPlanet());
		assertNull(crafts[0].getNavigator());

		CraftDelegate[] cds = DeligateCacheManager.getCraftsAt(pd1, ssd1);
		assertEquals(craft1Name, craft2Name, cds);

		//
		// Craft2 joins a fleet
		// Test creating a new fleet in orbit around a stellar object
		//

		serverProxy.joinFleetCmd(crafts[0], callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertNull(crafts[1].getFleet());
		assertEquals(1, crafts[0].getFleetDelegate().getCrafts().length);

		// Check most visible craft is the image for the fleet
		assertEquals("images/crafts/hull_flyer_blue.png", crafts[0].getFleetDelegate().getImage());

		//
		// Craft3 joins a fleet
		// Test joining an existing fleet
		//

		serverProxy.joinFleetCmd(crafts[1], callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertEquals("Fleet1", crafts[1].getFleet());
		assertEquals(2, crafts[0].getFleetDelegate().getCrafts().length);

		// Check most visible craft is the image for the fleet
		assertEquals("images/crafts/hull_flyer_blue.png", crafts[0].getFleetDelegate().getImage());

		//
		// Move fleet (craft1 and craft2) to ss2
		// Test Moving a fleet to a stellar object
		//

		serverProxy.mooMoveCmd(DeligateCacheManager.getMoo(crafts[1].getFleet()), ssd2, null, false, callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertEquals("Fleet1", crafts[1].getFleet());
		assertEquals("SS1", crafts[1].getFleetDelegate().getOrbit().getStellarObject());
		assertNull(crafts[1].getFleetDelegate().getOrbit().getPlanet());
		assertEquals("SS2", crafts[1].getFleetDelegate().getNavigator().getDestination().getStellarObject());
		assertNull(crafts[1].getFleetDelegate().getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertEquals("Fleet1", crafts[1].getFleet());
		assertNull(crafts[1].getFleetDelegate().getOrbit());
		assertEquals("SS2", crafts[1].getFleetDelegate().getNavigator().getDestination().getStellarObject());
		assertNull(crafts[1].getFleetDelegate().getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertEquals("Fleet1", crafts[1].getFleet());
		assertEquals("SS2", crafts[1].getFleetDelegate().getOrbit().getStellarObject());
		assertNull(crafts[1].getFleetDelegate().getOrbit().getPlanet());
		assertNull(crafts[1].getFleetDelegate().getNavigator());

		//
		// Move fleet (craft1 and craft2) to planet3
		// Test Moving a fleet to a planet
		//

		serverProxy.mooMoveCmd(DeligateCacheManager.getMoo(crafts[1].getFleet()), ssd3, hd3, false, callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertEquals("Fleet1", crafts[1].getFleet());
		assertEquals("SS2", crafts[1].getFleetDelegate().getOrbit().getStellarObject());
		assertEquals("SS3", crafts[1].getFleetDelegate().getNavigator().getDestination().getStellarObject());
		assertEquals("SS3-A", crafts[1].getFleetDelegate().getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertEquals("Fleet1", crafts[1].getFleet());
		assertEquals(null, crafts[1].getFleetDelegate().getOrbit());
		assertEquals("SS3", crafts[1].getFleetDelegate().getNavigator().getDestination().getStellarObject());
		assertEquals("SS3-A", crafts[1].getFleetDelegate().getNavigator().getDestination().getPlanet());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertEquals("Fleet1", crafts[1].getFleet());
		assertEquals("SS3", crafts[1].getFleetDelegate().getOrbit().getStellarObject());
		assertEquals("SS3-A", crafts[1].getFleetDelegate().getOrbit().getPlanet());
		assertEquals("SS3", crafts[0].getOrbit().getStellarObject());
		assertEquals("SS3-A", crafts[0].getOrbit().getPlanet());
		assertEquals("SS3", crafts[1].getOrbit().getStellarObject());
		assertEquals("SS3-A", crafts[1].getOrbit().getPlanet());
		assertEquals(crafts[0].getFleet(), crafts[1].getFleet());
		assertEquals(null, crafts[0].getFleetDelegate().getNavigator());
		assertEquals(null, crafts[0].getNavigator());
		assertEquals(null, crafts[1].getNavigator());

		Rectangle r = new Rectangle(125, 75, 50, 50);
		OrbitVO odh3 = hd3.getOrbit();
		OrbitVO odss3 = ssd3.getOrbit();
		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMOOsAt(r, false, odh3);
		assertEquals(craft1Name, craft2Name, ((FleetDelegate) moos[0]).getCrafts());

		moos = DeligateCacheManager.getMOOsAt(r, false, odss3);
		assertEquals(1, moos.length);
		assertEquals(craft3Name, moos[0].getName());

		//
		// Craft1 joins a existing fleet
		// Test joining an existing fleet when fleet orbit a planet and craft orbits SS
		//

		serverProxy.mooMoveCmd(crafts[2], ssd3, hd3, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("SS3", crafts[2].getOrbit().getStellarObject());
		assertEquals("SS3-A", crafts[2].getOrbit().getPlanet());

		serverProxy.joinFleetCmd(crafts[2], callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name,
				craft2Name, craft3Name);
		assertEquals("Fleet1", crafts[0].getFleet());
		assertEquals("Fleet1", crafts[1].getFleet());
		assertEquals("SS3", crafts[2].getOrbit().getStellarObject());
		assertEquals("SS3-A", crafts[2].getOrbit().getPlanet());
		assertEquals(3, crafts[0].getFleetDelegate().getCrafts().length);

		//
		// Test removing a craft from a fleet
		//

		moos = DeligateCacheManager.getMOOsAt(r, false, odh3);
		assertEquals(1, moos.length);
		assertEquals(craft1Name, craft2Name, craft3Name, ((FleetDelegate) moos[0]).getCrafts());

		serverProxy.craftLeaveFleetCmd(crafts[0], callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(null, craft1.getFleet());

		moos = DeligateCacheManager.getMOOsAt(r, false, odh3);
		assertEquals(craft1Name, "Fleet1", moos);
		assertEquals(craft2Name, craft3Name, ((FleetDelegate) moos[1]).getCrafts());

		//
		// Test delete a craft that is in a fleet
		//

		serverProxy.mooDeleteCmd(crafts[1], callbackSync, null);
		serverProxy._processCommandResponse();

		moos = DeligateCacheManager.getMOOsAt(r, false, odh3);
		assertEquals(craft1Name, "Fleet1", moos);
		assertEquals(craft3Name, ((FleetDelegate) moos[1]).getCrafts());

		serverProxy.mooDeleteCmd(crafts[2], callbackSync, null);
		serverProxy._processCommandResponse();

		moos = DeligateCacheManager.getMOOsAt(r, false, odh3);
		assertEquals(1, moos.length);
		assertEquals(craft1Name, moos[0].getName());

		serverProxy.mooDeleteCmd(crafts[0], callbackSync, null);
		serverProxy._processCommandResponse();

		moos = DeligateCacheManager.getMOOsAt(r, false, odh3);
		assertNull(moos);
	}

	/**
	 * Test Galaxy Fleet Commands and craft locations with multiple crafts
	 */
	public void testFleetJoinFleetCommands() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testFleetJoinFleetCommands() *************************");
		createMap();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd4 = DeligateCacheManager.getStellarObject("SS4");
		PlanetDelegate hd1 = DeligateCacheManager.getPlanet("SS1-A");

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];
		PlayerDelegate pd2 = pds[1];

		//
		// Create a craft1, craft2, craft3 at ss1
		// Test proxy returns three crafts at correct location
		//

		Craft craft1 = newCraft(fighterDesign, p1, ss1);
		Craft craft2 = newCraft(fighterDesign, p1, ss1);
		Craft craft3 = newCraft(fighterDesign, p1, ss1);
		Craft craft4 = newCraft(fighterDesign, p1, ss1);
		Craft craft5 = newCraft(fighterDesign, p1, ss1);

		Transport _transport = new Transport(p1);
		p1.getTransportManager().setTransport(_transport);
		h1.setPopulation(100);
		new TransportTaskMoveColonists(_transport, h1, null, 10);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		serverProxy.mooMoveCmd(DeligateCacheManager.getMoo("Transport1"), ssd1, null, false, null, null);
		serverProxy._processCommandResponse();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		CraftDelegate[] crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[5], "TestFighter1", "TestFighter2", "TestFighter3", "TestFighter4", "TestFighter5");

		//
		// Single craft joins fleet (creates a new fleet)
		//

		serverProxy.joinFleetCmd(crafts[4], callbackSync, null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] moos = (MoveableOwnedObjectDelegate[]) order(DeligateCacheManager.getMoos(),
				new MoveableOwnedObjectDelegate[6], "TestFighter1", "TestFighter2", "TestFighter3", "TestFighter4",
				"Fleet1", "Transport1");

		assertNull(((MotCraftDelegate) moos[0]).getFleet());
		assertNull(((MotCraftDelegate) moos[1]).getFleet());
		assertNull(((MotCraftDelegate) moos[2]).getFleet());
		assertNull(((MotCraftDelegate) moos[3]).getFleet());
		order(((FleetDelegate) moos[4]).getCrafts(), new MotCraftDelegate[1], "TestFighter5");
		assertEquals("Transport1", ((TransportDelegate) moos[5]).getName());

		//
		// Transport can't Join Fleet
		//

		try {
			serverProxy.joinFleetCmd(moos[5], callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Should get a validation error");
		} catch (ValidationError e) {
			// Success
		}

		//
		// Join Fleet when fleet is selected joins all crafts at the same location
		//

		serverProxy.joinFleetCmd(moos[4], callbackSync, null);
		serverProxy._processCommandResponse();

		moos = (MoveableOwnedObjectDelegate[]) order(DeligateCacheManager.getMoos(), new MoveableOwnedObjectDelegate[2], "Fleet1", "Transport1");

		order(((FleetDelegate) moos[0]).getCrafts(), new CraftDelegate[5], "TestFighter1", "TestFighter2", "TestFighter3", "TestFighter4", "TestFighter5");

		//
		// Remove two craft from the fleet and move fleet to another solar system
		//

		serverProxy.craftLeaveFleetCmd(crafts[0], callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy.craftLeaveFleetCmd(crafts[1], callbackSync, null);
		serverProxy._processCommandResponse();

		moos = (MoveableOwnedObjectDelegate[]) order(DeligateCacheManager.getMoos(), new MoveableOwnedObjectDelegate[4], "Fleet1", "TestFighter1", "TestFighter2", "Transport1");

		serverProxy.mooMoveCmd(moos[0], ssd1, hd1, false, null, null);
		serverProxy._processCommandResponse();

		assertEquals("SS1", ((FleetDelegate) moos[0]).getOrbit().getStellarObject());
		assertEquals("SS1", ((MotCraftDelegate) moos[1]).getOrbit().getStellarObject());
		assertEquals("SS1", ((MotCraftDelegate) moos[2]).getOrbit().getStellarObject());
		assertEquals("SS1", ((TransportDelegate) moos[3]).getOrbit().getStellarObject());

		assertEquals("SS1-A", ((FleetDelegate) moos[0]).getOrbit().getPlanet());
		assertNull(((MotCraftDelegate) moos[1]).getOrbit().getPlanet());
		assertNull(((MotCraftDelegate) moos[2]).getOrbit().getPlanet());
		assertNull(((TransportDelegate) moos[3]).getOrbit().getPlanet());

		order(((FleetDelegate) moos[0]).getCrafts(), new CraftDelegate[3], "TestFighter3", "TestFighter4", "TestFighter5");

		//
		// Make a second fleet (join craft in same SS but different orbit)
		//

		serverProxy.joinFleetCmd(moos[1], callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy.joinFleetCmd(moos[2], callbackSync, null);
		serverProxy._processCommandResponse();

		moos = (MoveableOwnedObjectDelegate[]) order(DeligateCacheManager.getMoos(), new MoveableOwnedObjectDelegate[3], "Fleet1", "Transport1", "Fleet2");

		order(((FleetDelegate) moos[0]).getCrafts(), new MotCraftDelegate[3], "TestFighter3", "TestFighter4", "TestFighter5");
		assertEquals("Transport1", ((TransportDelegate) moos[1]).getName());
		order(((FleetDelegate) moos[2]).getCrafts(), new MotCraftDelegate[2], "TestFighter1", "TestFighter2");

		assertEquals("SS1", ((FleetDelegate) moos[0]).getOrbit().getStellarObject());
		assertEquals("SS1", ((TransportDelegate) moos[1]).getOrbit().getStellarObject());
		assertEquals("SS1", ((FleetDelegate) moos[2]).getOrbit().getStellarObject());

		assertEquals("SS1-A", ((FleetDelegate) moos[0]).getOrbit().getPlanet());
		assertNull(((TransportDelegate) moos[1]).getOrbit().getPlanet());
		assertNull(((FleetDelegate) moos[2]).getOrbit().getPlanet());

		//
		// Move fleets to same location
		//

		serverProxy.mooMoveCmd(moos[2], ssd1, hd1, false, null, null);
		serverProxy._processCommandResponse();

		moos = (MoveableOwnedObjectDelegate[]) order(DeligateCacheManager.getMoos(), new MoveableOwnedObjectDelegate[3], "Fleet1", "Transport1", "Fleet2");

		assertEquals("SS1", ((FleetDelegate) moos[0]).getOrbit().getStellarObject());
		assertEquals("SS1", ((TransportDelegate) moos[1]).getOrbit().getStellarObject());
		assertEquals("SS1", ((FleetDelegate) moos[2]).getOrbit().getStellarObject());

		assertEquals("SS1-A", ((FleetDelegate) moos[0]).getOrbit().getPlanet());
		assertNull(((TransportDelegate) moos[1]).getOrbit().getPlanet());
		assertEquals("SS1-A", ((FleetDelegate) moos[2]).getOrbit().getPlanet());

		order(((FleetDelegate) moos[0]).getCrafts(), new MotCraftDelegate[3], "TestFighter3", "TestFighter4", "TestFighter5");
		assertEquals("Transport1", ((TransportDelegate) moos[1]).getName());
		order(((FleetDelegate) moos[2]).getCrafts(), new MotCraftDelegate[2], "TestFighter1", "TestFighter2");

		//
		// Join Fleet to a Fleet
		//

		serverProxy.joinFleetCmd(moos[2], callbackSync, null);
		serverProxy._processCommandResponse();

		moos = (MoveableOwnedObjectDelegate[]) order(DeligateCacheManager.getMoos(), new MoveableOwnedObjectDelegate[2], "Fleet1", "Transport1");

		order(((FleetDelegate) moos[0]).getCrafts(), new CraftDelegate[5], "TestFighter1", "TestFighter2", "TestFighter3", "TestFighter4", "TestFighter5");

		//
		// Delete a fleet
		//

		serverProxy.mooDeleteCmd(moos[0], callbackSync, null);
		serverProxy._processCommandResponse();

		moos = (MoveableOwnedObjectDelegate[]) order(DeligateCacheManager.getMoos(), new MoveableOwnedObjectDelegate[6], "TestFighter1", "TestFighter2", "TestFighter3", "TestFighter4", "TestFighter5", "Transport1");

		assertNull(((MotCraftDelegate) moos[0]).getFleet());
		assertNull(((MotCraftDelegate) moos[1]).getFleet());
		assertNull(((MotCraftDelegate) moos[2]).getFleet());
		assertNull(((MotCraftDelegate) moos[3]).getFleet());
		assertNull(((MotCraftDelegate) moos[4]).getFleet());
		assertEquals("Transport1", ((TransportDelegate) moos[5]).getName());

		assertEquals("SS1", ((CraftDelegate) moos[0]).getOrbit().getStellarObject());
		assertEquals("SS1", ((CraftDelegate) moos[1]).getOrbit().getStellarObject());
		assertEquals("SS1", ((CraftDelegate) moos[2]).getOrbit().getStellarObject());
		assertEquals("SS1", ((CraftDelegate) moos[3]).getOrbit().getStellarObject());
		assertEquals("SS1", ((CraftDelegate) moos[4]).getOrbit().getStellarObject());
		assertEquals("SS1", ((TransportDelegate) moos[5]).getOrbit().getStellarObject());

		assertEquals("SS1-A", ((CraftDelegate) moos[0]).getOrbit().getPlanet());
		assertEquals("SS1-A", ((CraftDelegate) moos[1]).getOrbit().getPlanet());
		assertEquals("SS1-A", ((CraftDelegate) moos[2]).getOrbit().getPlanet());
		assertEquals("SS1-A", ((CraftDelegate) moos[3]).getOrbit().getPlanet());
		assertEquals("SS1-A", ((CraftDelegate) moos[4]).getOrbit().getPlanet());
		assertNull(((TransportDelegate) moos[5]).getOrbit().getPlanet());
	}


	/**
	 * Test Galaxy Fleet Commands and craft locations with multiple crafts
	 */
	public void testGalaxyDeleteFleetWithNavigator() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testGalaxyDeleteFleetWithNavigator() *************************");
		createMap();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];
		PlayerDelegate pd2 = pds[1];

		//
		// Create a craft1, craft2, craft3 at ss1
		// Test proxy returns three crafts at correct location
		//

		Craft craft1 = newCraft(fighterDesign, p1, ss1);
		Craft craft2 = newCraft(defenderDesign, p1, ss1);
		Craft craft3 = newCraft(defenderDesign, p1, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		String craft1Name = craft1.getName();
		String craft2Name = craft2.getName();
		String craft3Name = craft3.getName();

		CraftDelegate[] crafts = (CraftDelegate[]) order(DeligateCacheManager.getCrafts(pd1), new CraftDelegate[3], craft1Name, craft2Name, craft3Name);
		assertEquals("SS1", crafts[0].getOrbit().getStellarObject());
		assertNull(crafts[0].getOrbit().getPlanet());
		assertNull(crafts[0].getNavigator());

		assertEquals("SS1", crafts[1].getOrbit().getStellarObject());
		assertNull(crafts[1].getOrbit().getPlanet());
		assertNull(crafts[1].getNavigator());

		assertEquals("SS1", crafts[2].getOrbit().getStellarObject());
		assertNull(crafts[2].getOrbit().getPlanet());
		assertNull(crafts[2].getNavigator());

		//
		// Crafts join a fleet
		//

		serverProxy.joinFleetCmd(crafts[0], callbackSync, null);
		serverProxy._processCommandResponse();

		FleetDelegate fleet = crafts[0].getFleetDelegate();
		
		serverProxy.joinFleetCmd(fleet, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Fleet1", fleet.getName());
		assertEquals(crafts[0], crafts[1], crafts[2], fleet.getCrafts());

		//
		// Assign orders to fleet 
		//

		serverProxy.mooMoveCmd(DeligateCacheManager.getMoo(crafts[1].getFleet()), ssd2, null, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("SS2", fleet.getNavigator().getDestination().getStellarObject());
		assertEquals("SS2", crafts[0].getNavigator().getDestination().getStellarObject());
		assertEquals("SS2", crafts[1].getNavigator().getDestination().getStellarObject());
		assertEquals("SS2", crafts[2].getNavigator().getDestination().getStellarObject());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		//
		// Test delete a fleet that is in a fleet
		//

		serverProxy.mooDeleteCmd(fleet, callbackSync, null);
		serverProxy._processCommandResponse();

		assertNull(DeligateCacheManager.getFleets());
		assertNull(crafts[0].getFleetDelegate());
		assertNull(crafts[1].getFleetDelegate());
		assertNull(crafts[2].getFleetDelegate());

		assertEquals(craft1, craft1.getNavigator().getObject());
		assertEquals(craft2, craft2.getNavigator().getObject());
		assertEquals(craft3, craft3.getNavigator().getObject());

		assertTrue(craft1.getNavigator() !=  craft2.getNavigator());
		assertTrue(craft2.getNavigator() !=  craft3.getNavigator());
	}


	/**
	 * Test Galaxy Craft Commands
	 */
	public void testUpgradeEngineCommand() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testUpgradeEngineCommand() *************************");
		createMap();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");

		PlanetDelegate hd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate hd2 = DeligateCacheManager.getPlanet("SS2-A");

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];
		PlayerDelegate pd2 = pds[1];

		CraftDesign craftDesign = new CraftDesign("TF", tech_flyer30, tech_fusion25);
		Craft craft1 = newCraft(craftDesign, p1, ss1);
		Craft craft2 = newCraft(craftDesign, p1, ss1);
		Craft craft3 = newCraft(craftDesign, p1, ss1);
		Craft craft4 = newCraft(craftDesign, p2, ss1);
		String craft1Name = craft1.getName();
		String craft2Name = craft2.getName();
		String craft3Name = craft3.getName();
		String craft4Name = craft4.getName();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] crafts = (MoveableOwnedObjectDelegate[]) Comp
				.sortByName(DeligateCacheManager.getMoos());
		assertEquals(4, crafts.length);
		assertEquals(craft1Name, crafts[0].getName());
		assertEquals(craft2Name, crafts[1].getName());
		assertEquals(craft3Name, crafts[2].getName());
		assertEquals(craft4Name, crafts[3].getName());

		serverProxy.joinFleetCmd((CraftDelegate) crafts[0], callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy.joinFleetCmd((CraftDelegate) crafts[1], callbackSync, null);
		serverProxy._processCommandResponse();

		crafts = DeligateCacheManager.getMoos();

		assertEquals("Fleet1", craft3Name, craft4Name, crafts);

		FleetDelegate fleet = (FleetDelegate) crafts[0];
		CraftDelegate craftMine = (CraftDelegate) crafts[1];
		CraftDelegate craftEnemy = (CraftDelegate) crafts[2];

		assertEquals(craft1Name, craft2Name, fleet.getCrafts());

		assertEquals(tech_fusion25, fleet.getCrafts()[0].getCraftDesign().getEngine());
		assertEquals(tech_fusion25, fleet.getCrafts()[1].getCraftDesign().getEngine());
		assertEquals(tech_fusion25, craftMine.getCraftDesign().getEngine());
		assertEquals(tech_fusion25, craftEnemy.getCraftDesign().getEngine());
		assertEquals(150, fleet.getRange());
		assertEquals(150, craftMine.getRange());

		p1.setFunds(50000);

	}

	/**
	 * Test Galaxy Commands (Move Colonise planet command, Abandon planet command)
	 */
	public void testColoniseCommands() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testColoniseCommands() *************************");
		createMap();

		StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");
		assertEquals((Planet) null, ss3.getColonies(true));
		assertEquals(150, 100, ss3.getLocation());
		assertEquals(null, ss3.getOwners());

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];
		PlayerDelegate pd2 = pds[1];

		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet3 = DeligateCacheManager.getPlanet("SS3-A");

		Craft craft1 = newCraft(coloniserDesign, p1, ss1);
		String craft1Name = craft1.getName();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] crafts = DeligateCacheManager.getMoos();
		assertEquals(1, crafts.length);
		assertEquals(craft1Name, crafts[0].getName());
		CraftDelegate coloniser = (CraftDelegate) crafts[0];

		h3.setTeraforming(0.8);
		assertTrue(coloniser.getCraftDesign().canColonise(planet3));

		lastEvent = null;
		craft1.arrivedAt(ss3.getOrbit());
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertNotNull(lastEvent);
		assertTrue(lastEvent instanceof ColonisePlanetEvent);
		assertEquals(craft1.getName(), ((ColonisePlanetEvent) lastEvent).getCraft());
		assertEquals(ss3.getName(), DeligateCacheManager.getCraft(((ColonisePlanetEvent) lastEvent).getCraft())
				.getOrbit().getStellarObject());

		PlanetDelegate planetDeligate = DeligateCacheManager.getPlanet("SS3-A");
		serverProxy.craftColoniseCmd(coloniser, planetDeligate, callbackSync, null);
		serverProxy._processCommandResponse();

		ssd3 = DeligateCacheManager.getStellarObject("SS3");
		assertEquals(planetDeligate.getName(), ss3.getColonies(true)[0].getName());
		assertEquals(player1.getName(), ss3.getOwners()[0].getName());

		PlanetDelegate[] userPlanets = (PlanetDelegate[]) order(DeligateCacheManager.getPlanets(pd1), new PlanetDelegate[2], "SS1-A", "SS3-A");

		PlanetDelegate[] pda = ((SolarSystemDelegate) DeligateCacheManager.getStellarObject("SS3")).getPlanets();
		assertEquals(1, pda.length);
		assertEquals("SS3-A", pda[0].getName());

		pda = ((SolarSystemDelegate) DeligateCacheManager.getStellarObject("SS1")).getColonies();
		assertEquals(1, pda.length);
		assertEquals("SS1-A", pda[0].getName());

		serverProxy.abandonCmd(userPlanets[1], callbackSync, null);
		serverProxy._processCommandResponse();

		userPlanets = DeligateCacheManager.getPlanets(pd1);
		assertEquals(1, userPlanets.length);
		assertEquals("SS1-A", userPlanets[0].getName());
	}

	/**
	 * Test Galaxy Commands (Move Colonise planet command, Abandon planet command)
	 */
	public void testCantColoniseCommands() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testCantColoniseCommands() *************************");
		createMap();

		StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");
		assertEquals((Planet) null, ss3.getColonies(true));
		assertEquals(null, ss3.getOwners());

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];
		PlayerDelegate pd2 = pds[1];

		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet3 = DeligateCacheManager.getPlanet("SS3-A");

		Craft craft1 = newCraft(coloniserDesign, p1, ss1);
		String craft1Name = craft1.getName();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] crafts = DeligateCacheManager.getMoos();
		assertEquals(1, crafts.length);
		assertEquals(craft1Name, crafts[0].getName());
		CraftDelegate coloniser = (CraftDelegate) crafts[0];

		h3.setTeraforming(0.7);
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertFalse(coloniser.getCraftDesign().canColonise(planet3));

		PlanetDelegate planetDeligate = DeligateCacheManager.getPlanet("SS3-A");
		try {
			serverProxy.craftColoniseCmd(coloniser, planetDeligate, callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Cant colonise this planet");
		} catch (ValidationError e) {
			// Success
		}

		ssd3 = DeligateCacheManager.getStellarObject("SS3");
		assertEquals(false, ss3.hasOwner());
		assertEquals((Planet) null, ss3.getColonies(true));

	}

	/**
	 * Test Galaxy get MOOs and get Mols (used to display galaxy map).
	 */
	public void testViewGalaxyCommands() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testViewGalaxyCommands() *************************");
		createMap();

		StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");

		UserPlayerDelegate pd1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer("Humans");

		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

		assertTrue(planet1.isOwnedBy(pd1));
		assertFalse(planet2.isOwnedBy(pd1));

		Craft craft1 = newCraft(fighterDesign, p1, h1);
		Craft craft2 = newCraft(fighterDesign, p1, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] crafts = DeligateCacheManager.getMoos();
		assertEquals(2, crafts.length);
		assertEquals(craft1.getName(), crafts[0].getName());
		assertEquals(craft2.getName(), crafts[1].getName());

		//
		// Test we fetch MOOs and MOLs works for two crafts in orbit with no orders
		//

		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos();
		MoveableObjectLocation[] mols = MoveableObjectLocationFactory.sortMoosByLocation(moos, false, true);
		assertEquals(2, mols.length);
		assertEquals(1, mols[0].getMoos().length);
		assertEquals(1, mols[1].getMoos().length);
		assertEquals(craft2.getName(), mols[0].getMoos()[0].getName());
		assertEquals(craft1.getName(), mols[1].getMoos()[0].getName());

		moos = DeligateCacheManager.getMoos();
		mols = MoveableObjectLocationFactory.sortMoosByLocation(moos, false, false);
		assertEquals(1, mols.length);
		assertEquals(2, mols[0].getMoos().length);
		assertEquals(craft1.getName(), mols[0].getMoos()[0].getName());
		assertEquals(craft2.getName(), mols[0].getMoos()[1].getName());

		MoveableOwnedObjectDelegate[] moods = null;
		OrbitVO odp1 = planet1.getOrbit();
		OrbitVO odss1 = planet1.getSolarSystem().getOrbit();
		moods = DeligateCacheManager.getMOOsAt(new Rectangle(ss1.getLocation(), 20), false, odp1);
		assertEquals(1, moods.length);
		assertEquals(craft1.getName(), moods[0].getName());

		moods = DeligateCacheManager.getMOOsAt(new Rectangle(ss1.getLocation(), 20), false, odss1);
		assertEquals(1, moods.length);
		assertEquals(craft2.getName(), moods[0].getName());

		moods = DeligateCacheManager.getMOOsAt(new Rectangle(ss1.getLocation(), 20), true, null);
		assertNull(moods);

		moods = DeligateCacheManager.getMOOsAt(new Rectangle(ss1.getLocation(), 20), true, odp1);
		assertNull(moods);

		//
		// Test we fetch MOOs and MOLs works for one craft and one fleet in orbit with
		// no orders
		//

		serverProxy.joinFleetCmd(((CraftDelegate) crafts[0]), callbackSync, null);
		serverProxy._processCommandResponse();

		moods = DeligateCacheManager.getMOOsAt(new Rectangle(ss1.getLocation(), 20), false, odp1);
		assertEquals(1, moods.length);
		assertEquals(craft1.getFleet().getName(), moods[0].getName());

		moos = DeligateCacheManager.getMoos();
		mols = MoveableObjectLocationFactory.sortMoosByLocation(moos, false, false);

		assertEquals(1, mols.length);
		MoveableOwnedObjectDelegate[] molMoos = mols[0].getMoos();
		assertEquals(craft2.getName(), "Fleet1", molMoos);
		FleetDelegate fd = (FleetDelegate) molMoos[1];
		assertEquals(1, fd.getCrafts().length);
		assertEquals(craft1, fd.getCrafts());

		serverProxy.craftLeaveFleetCmd(((CraftDelegate) crafts[0]), callbackSync, null);
		serverProxy._processCommandResponse();

		moos = DeligateCacheManager.getMoos();
		mols = MoveableObjectLocationFactory.sortMoosByLocation(moos, false, false);
		assertEquals(1, mols.length);
		molMoos = mols[0].getMoos();
		assertEquals(craft1, craft2, molMoos);

		PlanetDelegate[] player1VisiblePlanets = pd1.getVisiblePlanets();
		assertEquals(1, player1VisiblePlanets.length);
		assertEquals(h1.getName(), player1VisiblePlanets[0].getName());

		//
		// Test we fetch MOOs and MOLs works for one craft in orbit with no orders and
		// another craft
		// in orbit with orders
		//

		serverProxy.mooMoveCmd(crafts[0], planet2.getSolarSystem(), planet2, false, callbackSync, null);
		serverProxy._processCommandResponse();

		moods = DeligateCacheManager.getMOOsAt(new Rectangle(ss1.getLocation(), 20), true, odp1);
		assertEquals(1, moods.length);
		assertEquals(craft1.getName(), moods[0].getName());

		moods = DeligateCacheManager.getMOOsAt(new Rectangle(ss1.getLocation(), 20), false, odss1);
		assertEquals(1, moods.length);
		assertEquals(craft2.getName(), moods[0].getName());

		moods = DeligateCacheManager.getMOOsAt(new Rectangle(ss1.getLocation(), 20), true, odp1);
		assertEquals(1, moods.length);
		assertEquals(craft1.getName(), moods[0].getName());
		assertEquals("SS2", moods[0].getNavigator().getDestination().getStellarObject());
		assertEquals("SS2-A", moods[0].getNavigator().getDestination().getPlanet());

		moos = DeligateCacheManager.getMoos();
		mols = MoveableObjectLocationFactory.sortMoosByLocation(moos, true, false);
		assertEquals(2, mols.length);
		assertEquals(1, mols[0].getMoos().length);
		assertEquals(craft2.getName(), mols[0].getMoos()[0].getName());
		assertEquals(1, mols[0].getMoos().length);
		assertEquals(craft1.getName(), mols[1].getMoos()[0].getName());

	}

	/**
	 * Test Galaxy Commands (Move Colonise planet command)
	 */
	public void testSetProjectCommands() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testSetProjectCommands() *************************");
		createMap();

		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

		assertEquals("SS1", planet1.getSolarSystem().getName());
		assertEquals(40.0, planet1.getPopulation(), 0.0);
		assertEquals(10.0, planet1.getQtyFactories(), 0.0);
		assertEquals(1.0, planet1.getTeraforming(), 0.0);
		assertEquals(0.0, planet1.getProductionBonus(), 0.0);
		assertEquals(0.0, planet1.getResearchBonus(), 0.0);
		assertEquals(0.0, planet1.getReproductionBonus(), 0.0);
		assertEquals(100, planet1.getSize());
		assertEquals(100.0, planet1.getMaxPopulation(), 0.0);
		assertEquals("Humans", planet1.getOwner().getName());
		assertEquals(null, planet1.getOrbital());
		assertEquals(0, planet1.getQtyGroundDefense());
		assertEquals(null, planet1.getProjectDesign());
		assertNull(planet1.getScanners());
		assertNull(planet1.getShields());
		assertNull(planet1.getWeapons());

		//
		// Test Creating a build weapon project
		//

		ProjectDesign project = new WeaponDesign(tech_missile6);
		try {
			// ServerProxy playerTwoServerProxy = getPlayerTwoServerProxy();
			serverProxy.setProjectCmd(planet2, project, 0, false, false, callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Only the owner of a planet should be able to set the project");
		} catch (ValidationError ex) {
			// Success - Only the owner of a planet can set the project!!!
			assertEquals(null, h2.getProject());
			assertEquals(null, planet2.getProjectDesign());
		}

		serverProxy.setProjectCmd(planet1, project, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(project.getName(), planet1.getProjectDesign().getName());
		assertEquals(h1.getTurnsToCompleteProject(), planet1.getTurnsToCompleteProject(planet1.getProjectDesign()));

		//
		// Test end turn builds the weapon and that the delegate sees newly built weapon
		// on the planet
		//

		clearEvents();
		h1.setProduction(project.getCostToBuildOnPlanet() - 1);
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		;
		serverProxy._processQueuedEvents();

		NoProjectEvent npe = (NoProjectEvent) getEvent(NoProjectEvent.class);
		assertEquals("SS1-A", ((NoProjectEvent) npe).getPlanet());
		assertEquals(1, planet1.getWeapons().getAllAsArray().length);
		assertEquals(tech_missile6, planet1.getWeapons().getAllAsArray()[0]);
		assertEquals(1, planet1.getQtyGroundDefense());

		//
		// Test Build Craft Project
		//

		serverProxy.designMilitaryCraft(null, null, callbackSync, null);
		serverProxy._processCommandResponse();
		CraftDesign mcd = ((CraftDesignCR) callbackSync.waitTillComplete()).getResult();

		serverProxy.setProjectCmd(planet1, mcd, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertTrue(h1.getProject() instanceof ProjectCraft);
		assertEquals(h1.getTurnsToCompleteProject(), planet1.getTurnsToCompleteProject(planet1.getProjectDesign()));

		//
		// Test purchase project builds Craft on the planet
		//

		p1.setFunds(900);
		assertEquals(853, planet1.getCostToPurchaseProject()); // Value does not matter but moneys after purchase must
																// be right

		serverProxy.purchaseProjectCmd(planet1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(47, player1.getFunds());

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		CraftDelegate[] crafts = DeligateCacheManager.getCrafts(player1);

		assertEquals(null, planet1.getProjectDesign());
		assertEquals(1, crafts.length);
		assertTrue(crafts[0].getCraftDesign().getName().startsWith(mcd.getName()));
		assertEquals(crafts[0].getCraftDesign().getHull(), mcd.getHull());
		assertEquals(crafts[0].getCraftDesign().getEngine(), mcd.getEngine());
		assertEquals(crafts[0].getCraftDesign().getComputers(), mcd.getComputers());
		assertEquals(crafts[0].getCraftDesign().getScanners(), mcd.getScanners());
		assertEquals(crafts[0].getCraftDesign().getSpecials(), mcd.getSpecials());
		assertEquals(crafts[0].getCraftDesign().getShields(), mcd.getShields());
		assertEquals(crafts[0].getCraftDesign().getWeaponBays().getAllWeaponsAsList(), mcd.getWeaponBays().getAllWeaponsAsList());

		//
		// Test Build scanner works
		//

		project = new ScannerDesign(tech_photon100);
		serverProxy.setProjectCmd(planet1, project, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(project.getName(), h1.getProject().getName());
		assertEquals(h1.getTurnsToCompleteProject(), planet1.getTurnsToCompleteProject(planet1.getProjectDesign()));

		//
		// Test Build shield works
		//

		project = new ShieldDesign(tech_energy10);
		serverProxy.setProjectCmd(planet1, project, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(project.getName(), h1.getProject().getName());
		assertEquals(h1.getTurnsToCompleteProject(), planet1.getTurnsToCompleteProject(planet1.getProjectDesign()));

		//
		// Test Build battle station works
		//

		Orbital battleStation = new Orbital(tech_battleStationI, null, null);
		project = new BattleStationDesign(tech_battleStationI);
		serverProxy.setProjectCmd(planet1, project, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(project.getName(), h1.getProject().getName());
		assertEquals(h1.getTurnsToCompleteProject(), planet1.getTurnsToCompleteProject(planet1.getProjectDesign()));

		//
		// Test Build Transport
		//

		project = new TransportDesign();
		serverProxy.setProjectCmd(planet1, project, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(project.getName(), h1.getProject().getName());
		assertEquals(h1.getTurnsToCompleteProject(), planet1.getTurnsToCompleteProject(planet1.getProjectDesign()));

		//
		// Test Build scanner on all Idle planets works
		//

		Planet h1b = newPlanet("SS1-B", ss1, p1);
		h1.setScanners(null);
		h1.setProject(null);
		project = new ScannerDesign(tech_photon100);
		serverProxy.setProjectCmd(planet1, project, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(project.getName(), h1.getProject().getName());
		assertEquals(null, h1b.getProject());

		h1.setScanners(null);
		h1.setProject(null);
		serverProxy.setProjectCmd(planet1, project, 0, true, false, callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(project.getName(), h1.getProject().getName());
		assertEquals(project.getName(), h1b.getProject().getName());

		//
		// Test Build Outpost
		//

		OutpostDesign outpostDesign = new OutpostDesign(Technologies.getHull(TestTechs.OUTPOST));
		serverProxy.setProjectCmd(planet1, outpostDesign, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(TestTechs.OUTPOST, ((ProjectOutpost) h1.getProject()).getOutpost().getName());

		//
		// Test Build Space Station
		//

		SpaceStationDesign spaceStatonDesign = new SpaceStationDesign(tech_spaceStation);
		serverProxy.setProjectCmd(planet1, spaceStatonDesign, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_spaceStation, ((ProjectSpaceStation) h1.getProject()).getSpaceStation());

		//
		// Test planetary percents
		//

		serverProxy.setPlanetPercentsCmd(planet1, 0.3, 0.3, 0.3, 0.3, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(0.3, planet1.getPercentEcology(), 0.001);
		assertEquals(0.3, planet1.getPercentDevelopment(), 0.001);
		assertEquals(0.3, planet1.getPercentProduction(), 0.001);
		assertEquals(0.3, planet1.getPercentResearch(), 0.001);

		serverProxy.setPlanetAutoPercentsCmd(planet1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertFalse(0.3 == planet1.getPercentEcology());
		assertFalse(0.3 == planet1.getPercentDevelopment());
		assertEquals(h1.getMinimumEcologyPercent(), planet1.getPercentEcology(), 0.001);
		assertEquals(h1.getPercentDevelopment(), planet1.getPercentDevelopment(), 0.001);

		serverProxy.setPlanetPercentsCmd(planet1, 1.0, 1.0, 0.3, 0.3, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(1.0, planet1.getPercentEcology(), 0.001);
		assertEquals(1.0, planet1.getPercentDevelopment(), 0.001);
		assertEquals(0.3, planet1.getPercentProduction(), 0.001);
		assertEquals(0.3, planet1.getPercentResearch(), 0.001);
		assertEquals(h1.getMoneyOutput(), planet1.getMoneyOutput(), 0.001);
		//assertEquals(-83, planet1.getMoneyOutput(), 0.001);

		serverProxy.setPlanetAutoPercentsCmd(planet1, null, null, true, true, callbackSync, null);
		serverProxy._processCommandResponse();

		assertFalse(1.0 == planet1.getPercentEcology());
		assertFalse(1.0 == planet1.getPercentDevelopment());

		assertEquals(h1.getMinimumEcologyPercent(), planet1.getPercentEcology(), 0.001);
		assertEquals(h1.getPercentDevelopment(), planet1.getPercentDevelopment(), 0.001);
		assertEquals(h1.getMoneyOutput(), planet1.getMoneyOutput(), 0.001);
		//assertEquals(10, planet1.getMoneyOutput(), 0.001);

		//
		// Test that the Planet delegate fetches the correct values
		//

		h1.setInfluence(2);
		h1.setHasNatives(true);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals(h1.getTurnsToCompleteProject(h1.getProject()), planet1.getTurnsToCompleteProject(h1.getProject().getProjectDesign()));
		assertEquals(h1.getProduction(), planet1.getProduction());
		assertEquals(h1.getProductionBonus(), planet1.getProductionBonus(), 0.0);
		assertEquals(h1.getDevelopmentOutput(), planet1.getDevelopmentOutput(), 0.0);
		assertEquals(h1.getFactoriesGrowth(), planet1.getFactoriesGrowth(), 0.0);
		assertEquals(h1.getEcologyOutput(), planet1.getEcologyOutput(), 0.0);
		assertEquals(h1.getMinimumEcologyPercent(), planet1.getMinimumEcologyPercent(), 0.0);
		assertEquals(h1.getMinimumEcologyOutput(), planet1.getMinimumEcologyOutput(), 0.0);
		assertEquals(h1.getTeraforming(), planet1.getTeraforming(), 0.0);
		assertEquals(h1.getTeraformingGrowth(), planet1.getTeraformingGrowth(), 0.0);
		assertEquals(h1.getMaxPopulation(), planet1.getMaxPopulation(), 0.0);
		assertEquals(h1.getMoneyBonus(), planet1.getMoneyBonus());
		assertEquals(h1.getMoneyOutput(), planet1.getMoneyOutput());
		assertEquals(h1.getMaintenanceCost(), planet1.getMaintenanceCost());
		assertEquals(h1.getSize(), planet1.getSize());
		assertEquals(h1.getResearchBonus(), planet1.getResearchBonus(), 0.0);
		assertEquals(h1.getReproductionBonus(), planet1.getReproductionBonus(), 0.0);
		assertEquals(h1.getResource(), planet1.getResource());
		assertEquals(h1.getInfluence(), planet1.getInfluence());
		assertEquals(h1.hasNatives(), planet1.hasNatives());

		assertFalse(planet1.isBlockaded());
		assertTrue(planet1.isOwnedBy(player1));
		assertFalse(planet1.isOwnedBy(player2));

		assertTrue(planet1.isViewable());
		assertFalse(planet2.isViewable());

		//
		// Test Resources
		//

		h1.setResource(new ResourcePsiFungus());
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(h1.getResource().getName(), planet1.getResource().getName());
	}

	/**
	 * Test Galaxy Commands (Move Colonise planet command)
	 */
	public void testSetProjectOnIdleAndRepeat() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testSetProjectOnIdleAndRepeat() *************************");
		createMap();

		Planet h1b = newPlanet("SS1-B", ss1, p1);
		Planet h1c = newPlanet("SS1-C", ss1, p1);
		Planet h1d = newPlanet("SS1-D", ss1, p1);
		Planet h1e = newPlanet("SS1-E", ss1, p2);

		ProjectCraft craftProject = new ProjectCraft(fighterDesign, h1c, false);
		h1c.setProject(craftProject);

		//
		// Test Creating a build weapon project on SS1-A and all idle planets
		//

		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		ProjectDesign shieldProject = new ShieldDesign(tech_energy10);
		serverProxy.setProjectCmd(planet1, shieldProject, 0, true, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(shieldProject.getName(), h1.getProject().getName());
		assertEquals(shieldProject.getName(), h1b.getProject().getName());
		assertEquals(craftProject.getName(), h1c.getProject().getName());
		assertEquals(shieldProject.getName(), h1d.getProject().getName());
		assertEquals(null, h1e.getProject());

		assertEquals(0, h1.getRepeatProject());
		assertEquals(0, h1b.getRepeatProject());
		assertEquals(0, h1c.getRepeatProject());
		assertEquals(0, h1d.getRepeatProject());
		assertEquals(0, h1e.getRepeatProject());

		assertEquals(shieldProject.getName(), planet1.getProjectDesign().getName());

		h1d.setProject(null);
		ProjectDesign missileProject = new WeaponDesign(tech_missile6);
		serverProxy.setProjectCmd(planet1, missileProject, 5, true, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(missileProject.getName(), h1.getProject().getName());
		assertEquals(shieldProject.getName(), h1b.getProject().getName());
		assertEquals(craftProject.getName(), h1c.getProject().getName());
		assertEquals(missileProject.getName(), h1d.getProject().getName());
		assertEquals(null, h1e.getProject());

		assertEquals(5, h1.getRepeatProject());
		assertEquals(0, h1b.getRepeatProject());
		assertEquals(0, h1c.getRepeatProject());
		assertEquals(5, h1d.getRepeatProject());
		assertEquals(0, h1e.getRepeatProject());

	}

	/**
	 * Test the commands to view and set the research project work.
	 */
	public void testSetResearchProject() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testSetResearchProject() *************************");
		createMap();
		setupOneTierTechTree();

		p1.getMissedTechnologies().removeAll();
		p1.flushTechnologyAvailableForResearch();

		//
		// Test that setResearchProjectCmd sets the research project correctly.
		//

		serverProxy.setResearchProjectCmd(tech_massDriver2, callbackSync, null);
		serverProxy._processCommandResponse();

		// This sometimes fails
		Logger.logDebug("Missed: " + p1.getMissedTechnologies());
		Logger.logDebug("" + Comp.toString(p1.getTechnologyAvailableForResearch()));
		Logger.logDebug("" + p1.getResearchProject());

		assertEquals(tech_massDriver2, p1.getResearchProject().getTechnology());
		ResearchProjectDelegate rpd = player1.getResearchProject();
		assertEquals(tech_massDriver2, rpd.getTechnology());

		//
		// Test that when a research project is completed correctly, we get a
		// NNoResearchProjectEvent.
		//

		lastEvent = null;
		p1.getResearchProject().addResearch(p1.getResearchProject().getTechnology().getCostToResearch() + 1);

		serverProxy._processQueuedEvents();

		assertNotNull(lastEvent);
		assertTrue(lastEvent instanceof TechnologyDiscoveredEvent);
		assertEquals(tech_massDriver2, ((TechnologyDiscoveredEvent) lastEvent).getTechnology());

		lastEvent = null;
		p1.addResearch(1);

		serverProxy._processQueuedEvents();

		assertTrue(lastEvent != null && lastEvent instanceof NoResearchProjectEvent);

		//
		// Test techs missed and available for research and techs already researched
		// is correct
		//

		p1.getPlayerCharacter().setMissedTechnologies(new GeneralTechnologyList(tech_armour25));
		p1.flushTechnologyAvailableForResearch();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		Technology[] available = DeligateCacheManager.getTechsAvailableForResearch();

		assertEquals(10, available.length);
		assertEquals(tech_fusionBeam4, available[0]);
		assertEquals(tech_fusion25x, available[1]);
		assertEquals(tech_troopModule20, available[2]);
		assertEquals(tech_electronic25, available[3]);
		assertEquals(tech_flyer100, available[4]);
		assertEquals(tech_energy10, available[5]);
		assertEquals(tech_computerisedFactoryII, available[6]);
		assertEquals(tech_ComputerisedLabI, available[7]);
		assertEquals(tech_mineField, available[8]);
		assertEquals(tech_trade_route, available[9]);

		assertTrue(p1.hasDiscovered(tech_massDriver2));
		assertFalse(p2.hasDiscovered(tech_massDriver2));

		TechnologyList missedTechs = player1.getMissedTechnologies();
		assertEquals(tech_armour25, missedTechs);

		//
		// Test make new discovery has client updated
		//

		p1.finishResearchProject(new ResearchProject(tech_massDriver2, p1, false), true, false);
		serverProxy._processQueuedEvents();

		available = DeligateCacheManager.getTechsAvailableForResearch();
		assertFalse(Comp.in(tech_massDriver2, available));

		TechnologyList techsDiscovered = player1.getTechnologies();
		assertTrue(techsDiscovered.contains(tech_massDriver2));

		//
		// Test steal technology has client updated
		//

		assertTrue(Comp.in(tech_mineField, available));
		assertFalse(techsDiscovered.contains(tech_mineField));

		p2.addTechnology(tech_mineField);
		p2.steelTechnology(p1, null);
		serverProxy._processQueuedEvents();

		available = DeligateCacheManager.getTechsAvailableForResearch();
		assertFalse(Comp.in(tech_mineField, available));

		techsDiscovered = player1.getTechnologies();
		assertTrue(techsDiscovered.contains(tech_mineField));

		//
		// Test technology prize has client updated
		//

		assertTrue(Comp.in(tech_ComputerisedLabI, available));
		assertFalse(techsDiscovered.contains(tech_ComputerisedLabI));

		TechnologyPrize prize = new TechnologyPrize(tech_ComputerisedLabI);
		ss4.setPrize(prize);
		prize.release(p1, ss4);
		serverProxy._processQueuedEvents();

		available = DeligateCacheManager.getTechsAvailableForResearch();
		assertFalse(Comp.in(tech_ComputerisedLabI, available));

		techsDiscovered = player1.getTechnologies();
		assertTrue(techsDiscovered.contains(tech_ComputerisedLabI));

		//
		// Test steal specific technology has client updated
		//

		assertTrue(Comp.in(tech_computerisedFactoryII, available));
		assertFalse(techsDiscovered.contains(tech_computerisedFactoryII));

		p2.addTechnology(tech_computerisedFactoryII);
		p2.addTechnology(tech_energy10);
		p2.steelTechnology(p1, tech_computerisedFactoryII);
		serverProxy._processQueuedEvents();

		available = DeligateCacheManager.getTechsAvailableForResearch();
		assertFalse(Comp.in(tech_computerisedFactoryII, available));

		techsDiscovered = player1.getTechnologies();
		assertTrue(techsDiscovered.contains(tech_computerisedFactoryII));

	}

	public void testTechnologyTradingCommands() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testTechnologyTradingCommands() *************************");
		createMapWith3Players();
		setupOneTierTechTree();

		p1.getMissedTechnologies().removeAll();
		p1.flushTechnologyAvailableForResearch();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		Technology[] available = DeligateCacheManager.getTechsAvailableForResearch();

		assertEquals(12, available.length);
		assertEquals(tech_massDriver2, available[0]);
		assertEquals(tech_fusionBeam4, available[1]);
		assertEquals(tech_fusion25x, available[2]);
		assertEquals(tech_armour25, available[3]);
		assertEquals(tech_troopModule20, available[4]);
		assertEquals(tech_electronic25, available[5]);
		assertEquals(tech_flyer100, available[6]);
		assertEquals(tech_energy10, available[7]);
		assertEquals(tech_computerisedFactoryII, available[8]);
		assertEquals(tech_ComputerisedLabI, available[9]);
		assertEquals(tech_mineField, available[10]);
		assertEquals(tech_trade_route, available[11]);

		//
		// Test that player2 can only see one of the technologies available to steal or
		// trade
		//

		p1.addTechnology(tech_massDriver2);
		p1.addTechnology(tech_flyer100);
		p1.addTechnology(tech_energy10);

		assertEquals(tech_flyer100, tech_massDriver2, tech_energy10, p1.getStealableTechnologies(p2));

		Utils.clearAndSetNextRandomInt(0);
		serverProxy.fetchStealableTechnology(player1, player2, null, callbackSync, null);
		serverProxy._processCommandResponse();
		TechnologyList techsToSteal = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		assertEquals(tech_flyer100, techsToSteal);
		assertEquals(tech_flyer100,MotEngine.instance.fetchHistory.get(player1.getName() + player2.getName()).technologies);
		assertEquals(3000, MotEngine.instance.fetchHistory.get(player1.getName() + player2.getName()).date);

		//
		// Test repeatedly asking for a technology to swap does not reveal more techs
		// available
		//

		try {
			serverProxy.fetchStealableTechnology(player1, player2, new GeneralTechnologyList(tech_flyer100),callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Oops - asked for more tech suggestions and was given them");
		} catch (ValidationError e) {
			// success
		}

		serverProxy.fetchStealableTechnology(player1, player2, null, callbackSync, null);
		serverProxy._processCommandResponse();
		techsToSteal = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		assertEquals(tech_flyer100, techsToSteal);
		assertEquals(tech_flyer100,MotEngine.instance.fetchHistory.get(player1.getName() + player2.getName()).technologies);
		assertEquals(3000, MotEngine.instance.fetchHistory.get(player1.getName() + player2.getName()).date);

		//
		// Test given different suggestion after two turns
		//

		MotEngine.instance.setDate(MotEngine.instance.getDate() + 3);

		Utils.clearAndSetNextRandomInt(1);
		serverProxy.fetchStealableTechnology(player1, player2, null, callbackSync, null);
		serverProxy._processCommandResponse();
		techsToSteal = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		assertEquals(tech_massDriver2, techsToSteal);

		//
		// Can offer techs to more then one player (is same turn)
		//

		p2.addTechnology(tech_armour25);
		p2.addTechnology(tech_armour200);

		assertEquals(tech_armour25, tech_armour200, p2.getStealableTechnologies(p1));

		MotEngine.instance.setDate(MotEngine.instance.getDate() + 3);

		Utils.clearAndSetNextRandomInt(0);
		serverProxy.fetchStealableTechnology(player2, player1, null, callbackSync, null);
		serverProxy._processCommandResponse();
		techsToSteal = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		assertEquals(tech_armour25, techsToSteal);

		Utils.clearAndSetNextRandomInt(1);
		serverProxy.fetchStealableTechnology(player2, player3, null, callbackSync, null);
		serverProxy._processCommandResponse();
		techsToSteal = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		assertEquals(tech_armour200, techsToSteal);

		//
		// Test can swap more than one technology but only up to 25% of available
		//

		p1.addTechnology(tech_fusion25x);
		p1.addTechnology(tech_ComputerisedLabI);
		assertEquals(tech_ComputerisedLabI, tech_flyer100, tech_massDriver2, tech_energy10, tech_fusion25x, p1.getStealableTechnologies(p2));

		Utils.clearAndSetNextRandomInts(0,0,0,0,0);
		serverProxy.fetchStealableTechnology(player1, player2, null, callbackSync, null);
		serverProxy._processCommandResponse();
		techsToSteal = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		// Will never be tech_massDriver2 as that is in the suggestion history
		assertEquals(tech_ComputerisedLabI, techsToSteal.getAllAsArray());
		assertEquals(2,MotEngine.instance.fetchHistory.get(player1.getName() + player2.getName()).technologies.getQuantity());

		//
		// Test discovering a technology removes it from list of techs to steal
		//
		
		p2.addTechnology(tech_ComputerisedLabI);
		
		Utils.clearAndSetNextRandomInts(0,0,0,0,0);
		serverProxy.fetchStealableTechnology(player1, player2, null, callbackSync, null);
		serverProxy._processCommandResponse();
		techsToSteal = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		assertEquals(tech_flyer100, techsToSteal);
		
		//
		// Test when spying on player can select technology from a list
		//

		assertFalse(player1.canViewDetails(player2));

		MotEngine.instance.getSpiesNotNull().addDefendingSpies(p1, 10);
		serverProxy.addSpyPlayer(player2, callbackSync, null);
		serverProxy._processCommandResponse();

		assertTrue(player1.canViewDetails(player2));
		assertEquals(7, player1.getQtySpiesDefending());

		serverProxy.fetchStealableTechnology(player1, player2, null, callbackSync, null);
		serverProxy._processCommandResponse();
		techsToSteal = ((TechnologyListCR) callbackSync.waitTillComplete()).getResultAsList();

		assertEquals(tech_flyer100, tech_massDriver2, tech_energy10, tech_fusion25x,techsToSteal);

	}

	/**
	 * Test the commands to view player spies and assign tasks.
	 */
	public void testSpyCommands() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testSpyCommands() *************************");
		createMap();

		p1.getSpyCenter().setQtyBuiltSpies(10);
		MotEngine.instance.getSpiesNotNull().addDefendingSpies(p1, 10);
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(10, player1.getQtySpiesDefending());
		assertEquals(0, player1.getQtySpiesEspionage(player2));
		assertEquals(0, player1.getQtySpiesSabotage(player2));

		serverProxy.fetchPlayerDetails(player1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(0, player1.getEspionageSuccessChance(player2.getName()));
		assertEquals(0, player1.getSabotageSuccessChance(player2.getName()));

		serverProxy.addSpyEspionage(player2, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(9, player1.getQtySpiesDefending());
		assertEquals(1, player1.getQtySpiesEspionage(player2));
		assertEquals(0, player1.getQtySpiesSabotage(player2));

		serverProxy.fetchPlayerDetails(player1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(166, player1.getEspionageSuccessChance(player2.getName()));
		assertEquals(0, player1.getSabotageSuccessChance(player2.getName()));

		serverProxy.removeSpyEspionage(player2, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(10, player1.getQtySpiesDefending());
		assertEquals(0, player1.getQtySpiesEspionage(player2));
		assertEquals(0, player1.getQtySpiesSabotage(player2));

		serverProxy.fetchPlayerDetails(player1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(0, player1.getEspionageSuccessChance(player2.getName()));
		assertEquals(0, player1.getSabotageSuccessChance(player2.getName()));

		serverProxy.addSpySabotage(player2, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(9, player1.getQtySpiesDefending());
		assertEquals(0, player1.getQtySpiesEspionage(player2));
		assertEquals(1, player1.getQtySpiesSabotage(player2));

		serverProxy.fetchPlayerDetails(player1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(0, player1.getEspionageSuccessChance(player2.getName()));
		assertEquals(166, player1.getSabotageSuccessChance(player2.getName()));

		serverProxy.removeSpySabotage(player2, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(10, player1.getQtySpiesDefending());
		assertEquals(0, player1.getQtySpiesEspionage(player2));
		assertEquals(0, player1.getQtySpiesSabotage(player2));

		assertFalse(player1.canViewDetails(player2));

		serverProxy.addSpyPlayer(player2, callbackSync, null);
		serverProxy._processCommandResponse();

		assertTrue(player1.canViewDetails(player2));

		p1.addFunds(99000);

		p1.getSpyCenter().addEffort(299);
		serverProxy.setRecruiteSpyEffortCmd(501, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(501, p1.getSpyRecruitEffortPerTurn());
		assertEquals(501, player1.getSpyRecruitCosts());
		assertEquals(0.266, player1.getSpyRecruitChance(), 0.01);
		assertEquals(0, player1.getEspionageSuccessChance(player2.getName()));
		assertEquals(0, player1.getSabotageSuccessChance(player2.getName()));

		serverProxy.addSpyEspionage(player2, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(6, player1.getQtySpiesDefending());
		assertEquals(1, player1.getQtySpiesEspionage(player2));
		assertEquals(0, player1.getQtySpiesSabotage(player2));

		assertEquals(null, player1.getEspionageTarget(player2.getName()));
		assertEquals(166, player1.getEspionageSuccessChance(player2.getName()));

		p2.addTechnology(tech_battleStationI);
		serverProxy.setEspionageTarget(player2, tech_battleStationI, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_battleStationI, player1.getEspionageTarget(player2.getName()));
		assertEquals(664, player1.getEspionageSuccessChance(player2.getName()));

	}

	/**
	 * Test the commands assign tasks to transports and that colonists do move from
	 * one colony to another.
	 */
	public void testTransportColonistsDifferentSolarSystem() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testTransportColonistsDifferentSolarSystem() *************************");
		createMap();
		Planet h4 = newPlanet("SS3-B", ss3, p1);
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate pd4 = DeligateCacheManager.getPlanet("SS3-B");

		assertEquals("SS1-A", pd1.getName());
		assertEquals("SS3-B", pd4.getName());

		h1.setPopulation(50.0);
		h4.setPopulation(10.0);

		assertEquals(0, player1.getQtyFreeTransport());

		Transport _transport = new Transport(p1);
		p1.getTransportManager().setTransport(_transport);
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(1, player1.getQtyFreeTransport());

		serverProxy.loadTransportCmd(pd1, null, 10, 0, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Transport1", ((NameListCR) callbackSync.waitTillComplete()));

		assertEquals(40.0, h1.getPopulation(), 0.0);
		assertEquals(10.0, h4.getPopulation(), 0.0);
		assertEquals(40.0, pd1.getPopulation(), 0.0);
		assertEquals(10.0, pd4.getPopulation(), 0.0);

		//
		// Test Transports are visible on galaxy map when given a command
		//

		TransportDelegate transport = (TransportDelegate) DeligateCacheManager.getMoos()[0];

		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());

		//
		//

		serverProxy.mooMoveCmd(transport, pd4.getSolarSystem(), pd4, false, null, null);
		serverProxy._processCommandResponse();

		assertEquals("SS1", transport.getOrbit().getStellarObject());
		assertEquals("SS1-A", transport.getOrbit().getPlanet());
		assertEquals("SS3", transport.getNavigator().getDestination().getStellarObject());
		assertEquals("SS3-B", transport.getNavigator().getDestination().getPlanet());

		assertEquals(10, pd4.getQtyColonistsOnrouteTo());
		assertEquals(2, pd4.getColonistsEtaOnrouteTo());

		double planet1PopulationGrowth = h1.getPopulationGrowth();
		double planet4PopulationGrowth = h4.getPopulationGrowth();

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		assertEquals(null, transport.getOrbit());
		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		assertNull(DeligateCacheManager.getMoos());
		assertEquals(0, pd4.getColonistsEtaOnrouteTo());

		assertEquals(40.0 + planet1PopulationGrowth * 2, h1.getPopulation(), 0.25);
		assertEquals(20.0 + planet4PopulationGrowth * 2, h4.getPopulation(), 0.25);
		assertEquals(40.0 + planet1PopulationGrowth * 2, pd1.getPopulation(), 0.25);
		assertEquals(20.0 + planet4PopulationGrowth * 2, pd4.getPopulation(), 0.25);

		//
		// Test transport colonists without AUTO deploy
		//

		h1.setPopulation(50.0);
		h4.setPopulation(10.0);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals(1, player1.getQtyFreeTransport());

		serverProxy.loadTransportCmd(pd1, null, 10, 0, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(40.0, h1.getPopulation(), 0.0);
		assertEquals(10.0, h4.getPopulation(), 0.0);
		assertEquals(40.0, pd1.getPopulation(), 0.0);
		assertEquals(10.0, pd4.getPopulation(), 0.0);

		//
		// Test Transports are visible on galaxy map when given a command
		//

		assertEquals(0, player1.getQtyFreeTransport());

		transport = (TransportDelegate) DeligateCacheManager.getMoos()[0];
		assertEquals("Transport1", transport.getName());

		//
		// Test moving Transports - MANUAL DEPLOY on arrival
		//

		serverProxy.mooMoveCmd(transport, pd4.getSolarSystem(), null, false, null, null);
		serverProxy._processCommandResponse();

		assertEquals("SS1", transport.getOrbit().getStellarObject());
		assertEquals("SS1-A", transport.getOrbit().getPlanet());
		assertEquals("SS3", transport.getNavigator().getDestination().getStellarObject());
		assertNull(transport.getNavigator().getDestination().getPlanet());

		planet1PopulationGrowth = h1.getPopulationGrowth();
		planet4PopulationGrowth = h4.getPopulationGrowth();

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		assertEquals(null, transport.getOrbit());
		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());

		//
		// Test manually deploying the transport
		//

		serverProxy.mooDeploy(transport, pd4.getSolarSystem(), pd4, null, null);
		serverProxy._processCommandResponse();

		assertNull(DeligateCacheManager.getMoos());

		assertEquals(40.0 + planet1PopulationGrowth * 2, h1.getPopulation(), 2.0);
		assertEquals(20.0 + planet4PopulationGrowth * 2, h4.getPopulation(), 2.0);
		assertEquals(40.0 + planet1PopulationGrowth * 2, pd1.getPopulation(), 2.0);
		assertEquals(20.0 + planet4PopulationGrowth * 2, pd4.getPopulation(), 2.0);

	}

	/**
	 * Test the commands assign tasks to transports and that colonists do move from
	 * one colony to another.
	 */
	public void testTransportColonistsSameSolarSystem() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testTransportColonistsSameSolarSystem() *************************");
		createMap();
		Planet h4 = newPlanet("SS1-B", ss1, p1);
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();
		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate pd4 = DeligateCacheManager.getPlanet("SS1-B");

		assertEquals("SS1-A", pd1.getName());
		assertEquals("SS1-B", pd4.getName());

		h1.setPopulation(50.0);
		h4.setPopulation(10.0);

		assertEquals(0, player1.getQtyFreeTransport());

		Transport _transport = new Transport(p1);
		p1.getTransportManager().setTransport(_transport);
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(1, player1.getQtyFreeTransport());

		//
		// Test load transport
		//

		serverProxy.loadTransportCmd(pd1, null, 10, 0, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Transport1", ((NameListCR) callbackSync.waitTillComplete()));

		assertEquals(40.0, h1.getPopulation(), 0.0);
		assertEquals(10.0, h4.getPopulation(), 0.0);
		assertEquals(40.0, pd1.getPopulation(), 0.0);
		assertEquals(10.0, pd4.getPopulation(), 0.0);

		//
		// Test Transports are visible on galaxy map when given a command
		//

		assertEquals(0, player1.getQtyFreeTransport());

		TransportDelegate transport = (TransportDelegate) DeligateCacheManager.getMoos()[0];
		assertEquals("Transport1", transport.getName());

		//
		// Test Transports Moving in the same solar system deploy immediately
		//

		serverProxy.mooMoveCmd(transport, pd4.getSolarSystem(), null, false, null, null);
		serverProxy._processCommandResponse();

		assertEquals(40.0, h1.getPopulation(), 0.0);
		assertEquals(10.0, h4.getPopulation(), 0.0);
		assertEquals(40.0, pd1.getPopulation(), 0.0);
		assertEquals(10.0, pd4.getPopulation(), 0.0);

		//
		// Test manually deploying the transport
		//

		serverProxy.mooDeploy(transport, pd4.getSolarSystem(), pd4, null, null);
		serverProxy._processCommandResponse();

		assertEquals(40.0, h1.getPopulation(), 0.0);
		assertEquals(20.0, h4.getPopulation(), 0.0);
		assertEquals(40.0, pd1.getPopulation(), 0.0);
		assertEquals(20.0, pd4.getPopulation(), 0.0);

		assertNull(DeligateCacheManager.getMoos());
		assertEquals(1, player1.getQtyFreeTransport());

		//
		// Test when load Transport with target in same solar system - it deploys on
		// load
		//

		h1.setPopulation(50.0);
		h4.setPopulation(10.0);

		serverProxy.loadTransportCmd(pd1, pd4, 10, 0, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(40.0, h1.getPopulation(), 0.0);
		assertEquals(20.0, h4.getPopulation(), 0.0);
		assertEquals(40.0, pd1.getPopulation(), 0.0);
		assertEquals(20.0, pd4.getPopulation(), 0.0);

		//
		// Test Transports are visible on galaxy map when given a command
		//

		Rectangle r = new Rectangle(75, 75, 50, 50);
		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMOOsAt(r, false, pd1.getOrbit());
		assertNull(moos);
		moos = DeligateCacheManager.getMoos();
		assertNull(moos);

		//
		// Check transport delivering it is visible
		//

		OutpostDesign outpostDesign = new OutpostDesign(tech_outpost);
		serverProxy.setProjectCmd(pd1, outpostDesign, 0, false, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_outpost, ((ProjectOutpost) h1.getProject()).getOutpost());
	}

	/**
	 * Test the commands assign tasks to transports and that colonists do move from
	 * one colony to another.
	 */
	public void testCreateTradeRoute() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testCreateTradeRoute() *************************");
		createMap();

		p1.addTechnology(tech_trade_route);

		SolarSystem ss9 = newSolarSystem("SS9", 125, 100);
		Planet h9 = newPlanet("SS9-A", ss9, p1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		SolarSystemDelegate ssd1 = (SolarSystemDelegate) DeligateCacheManager.getStellarObject("SS1");
		SolarSystemDelegate ssd9 = (SolarSystemDelegate) DeligateCacheManager.getStellarObject("SS9");
		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate pd9 = DeligateCacheManager.getPlanet("SS9-A");

		assertEquals("SS1-A", pd1.getName());
		assertEquals("SS9-A", pd9.getName());

		h1.setPopulation(50.0);
		h9.setPopulation(50.0);

		assertEquals(0, player1.getQtyFreeTransport());

		Transport _transport = new Transport(p1);
		p1.getTransportManager().setTransport(_transport);
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(1, player1.getQtyFreeTransport());

		//
		// Test load transport with Create Trade Route Task
		//

		serverProxy.loadTransportCmd(ssd1.getOrbit(), null, 0, 2, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Transport1", ((NameListCR) callbackSync.waitTillComplete()).getNames());

		assertEquals(ss1, ((TransportTaskCreateTradeRoute) _transport.getTask()).getOrigin());
		assertNull(((TransportTaskCreateTradeRoute) _transport.getTask()).getDestination());

		//
		// Test Transports Moving to solar system deploy on arrival
		//

		TransportDelegate transport = (TransportDelegate) DeligateCacheManager.getMoos()[0];
		assertEquals("Transport1", transport.getName());

		serverProxy.mooMoveCmd(transport, ssd9, null, false, null, null);
		serverProxy._processCommandResponse();

		assertEquals(ss9, transport.getNavigator().getDestination().getStellarObject());
		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());

		p1.getTransportManager().endTurn();
		// p1.getTransportManager().endTurn();

		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertNull(DeligateCacheManager.getMoos());
		assertEquals(1, player1.getQtyFreeTransport());

		TradeRoute tradeRoute = MotEngine.instance.getTradeRoutes().getTradeRoutes()[0];
		assertEquals(ss1, ss9, 3000, tradeRoute);

		TradeRouteDelegate[] trs = DeligateCacheManager.getTradeRoutes();
		assertEquals(ss1, ss9, 3000, trs);

	}

	/**
	 * Tests the user player delegate and commands that are specific to user players
	 * including; - Adding Removing Craft Designs - Get last craft designed -
	 * Setting player character percents - Player delegate (technologies). -
	 * Fetching / Selling Salvage
	 */
	public void testUserPlayer() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testUserPlayer() *************************");
		createMap();

		CraftDesigns cds = player1.getCraftDesigns();
		assertNull(cds);

//        CraftDesign fighterDesign = TestFactory.getCraftDesign("TestFighter");
//        CraftDesign multipleDesign = TestFactory.getCraftDesign("TestDefender");

		//
		// Test add and remove craft designs works
		//

		serverProxy.addCraftDesign(fighterDesign, callbackSync, null);
		serverProxy._processCommandResponse();
		cds = player1.getCraftDesigns();
		assertEquals(1, cds.getList().size());
		assertEquals(fighterDesign, cds.getCraftDesigns()[0]);
		assertEquals(fighterDesign, player1.getLastCraftDesigned());
		assertEquals(p1.getCraftBuildExperience().getBuildQuality(p1, fighterDesign.getName()),
				player1.getBuildQuality(fighterDesign.getName()));

		serverProxy.addCraftDesign(defenderDesign, callbackSync, null);
		serverProxy._processCommandResponse();
		cds = player1.getCraftDesigns();
		assertEquals(2, cds.getList().size());
		assertEquals(fighterDesign, cds.getCraftDesigns()[0]);
		assertEquals(defenderDesign, cds.getCraftDesigns()[1]);
		assertEquals(defenderDesign, player1.getLastCraftDesigned());
		assertEquals(p1.getCraftBuildExperience().getBuildQuality(p1, defenderDesign.getName()),
				player1.getBuildQuality(defenderDesign.getName()));

		serverProxy.removeCraftDesign(fighterDesign.getName(), callbackSync, null);
		serverProxy._processCommandResponse();
		cds = player1.getCraftDesigns();
		assertEquals(1, cds.getList().size());
		assertEquals(defenderDesign, cds.getCraftDesigns()[0]);
		assertEquals(defenderDesign, player1.getLastCraftDesigned());

		//
		// Test design crafts
		//

		serverProxy.designColonyCraft(callbackSync, null);
		serverProxy._processCommandResponse();
		CraftDesign lastDesign = ((CraftDesignCR) callbackSync.waitTillComplete()).getResult();
		assertTrue(lastDesign.canColonise());
		assertNull(lastDesign.getSpecialTechs().getBestQtyBerths());
		assertNull(lastDesign.getWeaponBays());

		serverProxy.designMilitaryCraft(null, null, callbackSync, null);
		serverProxy._processCommandResponse();
		lastDesign = ((CraftDesignCR) callbackSync.waitTillComplete()).getResult();
		assertFalse(lastDesign.canColonise());
		assertNull(lastDesign.getSpecials());
		assertEquals(4, lastDesign.getWeaponBays().getAllWeapons().length);

		serverProxy.designMilitaryCraft(tech_outpost.getName(), new OrbitVO(ss1.getName(), h1.getName()), callbackSync,
				null);
		serverProxy._processCommandResponse();
		lastDesign = ((CraftDesignCR) callbackSync.waitTillComplete()).getResult();
		assertFalse(lastDesign.canColonise());
		assertNull(lastDesign.getSpecials());
		assertEquals(2, lastDesign.getWeaponBays().getAllWeapons().length);

		//
		// Test player character percents
		//

		serverProxy.setPlayerCharacterPercents(new Double(0.11), new Double(0.12), new Double(0.13), new Double(0.14),
				new Double(0.15), new Double(0.8), new Double(0.9), callbackSync, null);
		serverProxy._processCommandResponse();

		// assertEquals(0.11, p1.getPlayerCharacter().getPlayerEcologyPercent(), 0.0);
		// assertEquals(0.12, p1.getPlayerCharacter().getPlayerDevelopmentPercent(),
		// 0.0);
		assertEquals(0.13, p1.getPlayerCharacter().getPlayerResearchPercent(), 0.0);
		assertEquals(0.14, p1.getPlayerCharacter().getPlayerProductionPercent(), 0.0);
		assertEquals(0.15, p1.getPlayerCharacter().getPlayerMoneyPercent(), 0.0);

		assertEquals(0.8, p1.getPlayerCharacter().getPlayerEcologyPercent(), 0.0);
		assertEquals(0.9, p1.getPlayerCharacter().getPlayerDevelopmentPercent(), 0.0);

		assertEquals(0.8, player1.getPlayerCharacter().getPlayerEcologyPercent(), 0.0);
		assertEquals(0.9, player1.getPlayerCharacter().getPlayerDevelopmentPercent(), 0.0);

		assertEquals(0.13, player1.getPlayerCharacter().getPlayerResearchPercent(), 0.0);
		assertEquals(0.14, player1.getPlayerCharacter().getPlayerProductionPercent(), 0.0);
		assertEquals(0.15, player1.getPlayerCharacter().getPlayerMoneyPercent(), 0.0);

		//
		// Test player delegates work and return correct technologies
		//

		assertEquals(p1.getBiotechnologies(), player1.getBiotechnologies());
		assertEquals(p1.getComputers(), player1.getComputers());
		assertEquals(p1.getEngines(), player1.getEngines());
		assertEquals(p1.getFactories(), player1.getFactories());
		assertEquals(p1.getHulls(), player1.getHulls());
		assertEquals(p1.getLaboratories(), player1.getLaboratories());
		assertEquals(p1.getScanners(), player1.getScanners());
		assertEquals(p1.getShields(), player1.getShields());
		assertEquals(p1.getSpecials(), player1.getSpecials());
		assertEquals(p1.getTheories(), player1.getTheories());
		assertEquals(p1.getWeapons(), player1.getWeapons());
		assertTrue(player1.getTechnologies().contains(p1.getTechnologies()));

		p1.addTechnology(tech_terraforming10);
		p1.addTechnology(tech_electronic40);
		p1.addTechnology(tech_fusion25x);
		p1.addTechnology(tech_battleStationI);
		p1.addTechnology(tech_ComputerisedLabI);
		p1.addTechnology(tech_augumentedEnginesI);
		p1.addTechnology(tech_photon200);
		p1.addTechnology(tech_energy20);
		// p1.addTechnology(Technologies.getTheory(TestTechs.TLAB_COMPUTERISEDI));
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(p1.getBiotechnologies(), player1.getBiotechnologies());
		assertEquals(p1.getComputers(), player1.getComputers());
		assertEquals(p1.getEngines(), player1.getEngines());
		assertEquals(p1.getFactories(), player1.getFactories());
		assertEquals(p1.getHulls(), player1.getHulls());
		assertEquals(p1.getLaboratories(), player1.getLaboratories());
		assertEquals(p1.getScanners(), player1.getScanners());
		assertEquals(p1.getShields(), player1.getShields());
		assertEquals(p1.getSpecials(), player1.getSpecials());
		assertEquals(p1.getTheories(), player1.getTheories());
		assertEquals(p1.getWeapons(), player1.getWeapons());

		//
		// Add salvage to player and check delegate fetches it
		//

		Technology[] salvage = { tech_armour25, tech_battleStationI, tech_fusionBeam4, tech_fusionBeam4 };
		p1.setSalvage(new SalvageList(salvage));
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(salvage, player1.getSalvage());

		//
		// Test sell salvage command
		//

		Technology[] sellSalvageFail = { tech_armour25, tech_battleStationI, tech_fusion40 };
		Technology[] sellSalvageSuccess = { tech_armour25, tech_battleStationI, tech_fusionBeam4 };
		p1.setFunds(500);
		try {
			serverProxy.sellSavlage(new GeneralTechnologyList(sellSalvageFail), callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Oops - I do not own all this salavage");
		} catch (ValidationError e) {
			// success
		}
		assertEquals(500, p1.getFunds());
		assertEquals(salvage, p1.getSalvage());

		serverProxy.sellSavlage(new GeneralTechnologyList(sellSalvageSuccess), callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(1933, p1.getFunds());
		assertEquals(new Technology[] { tech_fusionBeam4 }, p1.getSalvage());
	}

	/**
	 * Test the commands assign tasks to transports and that colonists do move from
	 * one colony to another.
	 */
	public void testLeaders() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testLeaders() *************************");
		createMap();

		Craft c1 = newCraft(fighterDesign, p1, ss2);

		mot.engine.leaders.LeaderFactory.offerGeneralLeaderTo(p1);
		serverProxy._processQueuedEvents();

		OfferLeaderEvent ole = (OfferLeaderEvent) lastEvent;
		assertEquals(null, player1.getGeneralLeader());
		assertEquals("TestGeneral", ole.getLeader());

		serverProxy.acceptLeader(ole.getLeader(), callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("TestGeneral", p1.getGeneralLeader().getName());
		assertEquals("TestGeneral", player1.getGeneralLeader().getName());
		assertEquals(1, player1.getGeneralLeader().getLevel());

//      assertEquals(c1.getMaxDamage(), DeligateCacheManager.getCraft(c1.getName()).get);

		p1.getGeneralLeader().setExperience(100);
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals("TestGeneral", player1.getGeneralLeader().getName());
		assertEquals(2, player1.getGeneralLeader().getLevel());

		mot.engine.leaders.LeaderFactory.offerGovenorLeaderTo(p1);
		serverProxy._processQueuedEvents();

		ole = (OfferLeaderEvent) lastEvent;
		assertEquals(null, player1.getGovenorLeader());
		assertEquals("TestGovenor", ole.getLeader());

		serverProxy.acceptLeader(ole.getLeader(), callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("TestGovenor", p1.getGovenorLeader().getName());
		assertEquals("TestGovenor", player1.getGovenorLeader().getName());
		assertEquals(1, player1.getGovenorLeader().getLevel());

		assertEquals(h1.getProduction(), DeligateCacheManager.getPlanet(h1.getName()).getProduction());
		assertEquals(h1.getMaxPopulation(), DeligateCacheManager.getPlanet(h1.getName()).getMaxPopulation());

		p1.getGovenorLeader().setExperience(100);
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals("TestGovenor", player1.getGovenorLeader().getName());
		assertEquals(2, player1.getGovenorLeader().getLevel());
	}

	/**
	 */
	public void testAncientTechnologyGift() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testAncientTechnologyGift() *************************");
		createMap();

		MonsterPlayer mp = MotEngine.instance.getMonsterPlayer();
		mp = MotEngine.instance.getMonsterPlayer();
		mp.addAncient(new AncientPlayer(ANCIENT_AIR_MAN, h3.getStellarObject()));

		p1.setAlignment(ANCIENT_AIR_MAN);
		MotEngine.instance.setDate(3074);
		clearEvents();

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		PrizeGiftEvent gift = (PrizeGiftEvent) getEvent(PrizeGiftEvent.class);

		assertEquals(
				"Ancient AirMan has given all players aligned to them Psychometric Genetic Manipulation that has increased your race's natural psi ability by 5",
				gift.getDescription());

		assertEquals(5, p1.getPlayerCharacter().getPsiRating());
		assertEquals(5, player1.getPlayerCharacter().getPsiRating());

		MotEngine.instance.setDate(3249);
		clearEvents();

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		gift = (PrizeGiftEvent) getEvent(PrizeGiftEvent.class);

		assertEquals("special , Dimension Chamber", gift.getTechFullName());

		Technology technology = Technologies.getTechnology(gift.getTechFullName());

		assertEquals(tech_dimensionChamber, technology);
		assertEquals(true, player1.getTechnologies().contains(tech_dimensionChamber));

	}

	/**
	 * Tests that the rename commands work for Planet and Solar System. Tests that
	 * delegates still work after an object has been renamed.
	 */
	public void testRenameStellarObject() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testRename() *************************");
		createMap();

		Craft c1 = newCraft(fighterDesign, p1, ss2);
		c1.moveTo(ss1.getOrbit());
		Craft c2 = newCraft(fighterDesign, p1, ss2);
		c2.moveTo(h1.getOrbit());
		Craft c3 = newCraft(fighterDesign, p1, ss1);
		Craft c4 = newCraft(fighterDesign, p1, ss1);
		c4.setOrbit(new Orbit(h1));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		SolarSystemDelegate ssd1 = (SolarSystemDelegate) DeligateCacheManager.getStellarObject("SS1");
		SolarSystemDelegate ssd2 = (SolarSystemDelegate) DeligateCacheManager.getStellarObject("SS2");
		SolarSystemDelegate ssd3 = (SolarSystemDelegate) DeligateCacheManager.getStellarObject("SS3");

		assertEquals("SS1", ssd1.getName());
		assertEquals("SS1-A", ssd1.getPlanets()[0].getName());
		assertEquals("SS2", ssd2.getName());
		assertEquals("SS2-A", ssd2.getPlanets()[0].getName());

		CraftDelegate[] crafts = DeligateCacheManager.getCrafts(player1);
		assertEquals(c1, c2, c3, c4, crafts);
		CraftDelegate craft1 = crafts[0];
		CraftDelegate craft2 = crafts[1];
		CraftDelegate craft3 = crafts[2];
		CraftDelegate craft4 = crafts[3];

		assertEquals("SS1", c1.getNavigator().getDestination().getStellarObject().getName());
		assertEquals("SS1-A", c2.getNavigator().getDestination().getPlanet().getName());

		assertEquals("SS1", craft1.getNavigator().getDestination().getStellarObject());
		assertEquals("SS1-A", craft2.getNavigator().getDestination().getPlanet());

		StellarObjectDelegate stellarObject = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate planet = DeligateCacheManager.getPlanet("SS1-A");

		serverProxy.renameStellarObject(ssd1, "SO1", callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("SO1", ss1.getName());
		assertEquals("SO1-A", ss1.getPlanets()[0].getName());

		assertEquals("SO1", stellarObject.getName());
		assertEquals("SO1", planet.getSolarSystemName());
		assertEquals("SO1", planet.getSolarSystem().getName());
		assertEquals("SO1", DeligateCacheManager.getStellarObject("SO1").getName());
		assertEquals("SO1-A", planet.getName());
		assertEquals("SO1-A", ((SolarSystemDelegate) stellarObject).getPlanets()[0].getName());
		assertEquals("SO1-A",
				((SolarSystemDelegate) DeligateCacheManager.getStellarObject("SO1")).getPlanets()[0].getName());

		assertEquals("SO1", craft1.getNavigator().getDestination().getStellarObject());
		assertNull(craft1.getNavigator().getDestination().getPlanet());
		assertEquals("SO1", craft2.getNavigator().getDestination().getStellarObject());
		assertEquals("SO1-A", craft2.getNavigator().getDestination().getPlanet());
		assertEquals("SO1", craft3.getOrbit().getStellarObject());
		assertNull(craft3.getOrbit().getPlanet());
		assertEquals("SO1", craft4.getOrbit().getStellarObject());
		assertEquals("SO1-A", craft4.getOrbit().getPlanet());

	}

	/**
	 * Tests that the rename commands work for Planet and Solar System. Tests that
	 * delegates still work after an object has been renamed.
	 */
	public void testRenameFleet() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testRenameFleet() *************************");
		createMap();

		Craft c1 = newCraft(fighterDesign, p1, ss2);
		Craft c2 = newCraft(fighterDesign, p1, ss2);
		new CraftJoinFleetCmd(c1, p1).run();
		new CraftJoinFleetCmd(c2, p1).run();
		Fleet f1 = c1.getFleet();
		f1.moveTo(h1.getOrbit());

		Craft c3 = newCraft(fighterDesign, p1, ss1);
		new CraftJoinFleetCmd(c3, p1).run();
		Fleet f2 = c3.getFleet();
		f2.setOrbit(new Orbit(h1));

		Craft c4 = newCraft(fighterDesign, p1, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		SolarSystemDelegate ssd1 = (SolarSystemDelegate) DeligateCacheManager.getStellarObject("SS1");

		MoveableOwnedObjectDelegate[] moos = (MoveableOwnedObjectDelegate[]) order(DeligateCacheManager.getMoos(),
				new MoveableOwnedObjectDelegate[2], "Fleet1", "Fleet2", "TestFighter4");
		FleetDelegate fleet1 = (FleetDelegate) moos[0];
		FleetDelegate fleet2 = (FleetDelegate) moos[1];
		CraftDelegate craft4 = DeligateCacheManager.getCrafts(player1)[2];

		serverProxy.renameMoo(fleet1, "AttackFleet", callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("AttackFleet", fleet1.getName());
		assertEquals("AttackFleet", fleet1.getCrafts()[0].getFleet());
		assertEquals("AttackFleet", fleet1.getCrafts()[1].getFleet());
		assertEquals(fleet1, fleet1.getCrafts()[0].getFleetDelegate());
		assertEquals(fleet1, fleet1.getCrafts()[1].getFleetDelegate());
		assertEquals("AttackFleet", DeligateCacheManager.getMoo("AttackFleet").getName());

	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testInvade() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testInvade() *************************");
		createMap();

		MotEngine.getInstance().getRelations().setWar(p1, p2);
		Craft craft1 = newCraft(fighterDesign, p1, h2);
		h2.flushBlockadeCache();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");

		assertFalse(planet2.isBlockaded());
		
		h1.setScanners(new Scanners(tech_photon150));
		h2.flushBlockadeCache();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();
		
		assertTrue(planet2.isBlockaded());

		Transport transport = new Transport(p1);
		p1.getTransportManager().setTransport(transport);

		h1.setQtyGroundDefense(100);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals(craft1, DeligateCacheManager.getMoos());
		assertEquals(1, player1.getQtyFreeTransport());

		serverProxy.loadTransportCmd(planet1, null, 10, 1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Transport1", ((NameListCR) callbackSync.waitTillComplete()));

		assertEquals(transport, craft1, DeligateCacheManager.getMoos());
		assertEquals(0, player1.getQtyFreeTransport());

		assertEquals("Troops are loaded onto Transport", 90, h1.getQtyGroundDefense());
		assertEquals("Troops are loaded onto Transport", 0, p1.getTransportManager().getQtyFreeTransport());
		TransportTaskInvade task = (TransportTaskInvade) p1.getTransportManager().getTransportsAsArray()[0].getTask();
		assertEquals("Troops are loaded onto Transport", 10, task.getQtyAttackers());
		// assertEquals("Troops are loaded onto Transport", h2, task.getDestination());

		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos();
		assertEquals("Transport is at SS1", "SS1", ((TransportDelegate) moos[0]).getOrbit().getStellarObject());
		assertEquals("Transport is at SS1", "SS1-A", ((TransportDelegate) moos[0]).getOrbit().getPlanet());
		assertEquals("Transport is at SS1", "Move 10 troops ", ((TransportDelegate) moos[0]).getTaskDescription());

		transport.setOrbit(new Orbit(h2));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals("Transport is at SS1", "SS2", moos[0].getOrbit().getStellarObject());
		assertEquals("Transport is at SS1", "SS2-A", moos[0].getOrbit().getPlanet());

		assertEquals(player2, planet2.getOwner());
		assertEquals(player2, ssd2.getOwnerDelegates()[0]);
		assertTrue(planet2.isBlockaded());

		TransportDelegate td = (TransportDelegate) moos[0];
		serverProxy.invadePlanet(td, planet2, null);
		serverProxy._processCommandResponse();

		PlanetInvadeEvent ipe = (PlanetInvadeEvent) getEvent(PlanetInvadeEvent.class);
		assertNotNull(ipe);
		assertEquals(planet2.getName(), ipe.getTarget());
		assertEquals(10, ipe.getQtyAttackers());
		assertEquals(0, ipe.getQtyDefenders());
		assertEquals(10, ipe.getRemainingAttackers());
		assertEquals(0, ipe.getRemainingDefenders());
		// assertEquals("Event is correct", "Transport1", ipe.get);
		assertEquals("Event is correct", "SS2-A", ipe.getTarget());

		PlanetConqueredEvent pqe = (PlanetConqueredEvent) getEvent(PlanetConqueredEvent.class);
		assertEquals(planet2.getName(), pqe.getTarget());

		assertFalse(planet2.isBlockaded());
		assertEquals(craft1, DeligateCacheManager.getMoos());
		assertEquals(1, player1.getQtyFreeTransport());

		assertEquals(player1, planet2.getOwner());
		assertEquals(player1, ssd2.getOwnerDelegates()[0]);
	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testInvadeWithBlockade() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testInvadeWithBlockade() *************************");
		createMap();

		h1.setScanners(new Scanners(tech_photon150));
		MotEngine.getInstance().getRelations().setWar(p1, p2);
		Craft fighter = newCraft(fighterDesign, p1, h2);
		h2.flushBlockadeCache();
		p1.addWeapon(tech_fusionBeam4);
		SolarSystem ss5 = newSolarSystem("SS5", 250, 100);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		StellarObjectDelegate ssd5 = DeligateCacheManager.getStellarObject("SS5");

		Transport transport = new Transport(p1);
		p1.getTransportManager().setTransport(transport);

		h1.setQtyGroundDefense(100);
		h2.setQtyGroundDefense(1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals(1, player1.getQtyFreeTransport());

		serverProxy.loadTransportCmd(planet1, null, 25, 1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(0, player1.getQtyFreeTransport());
		assertEquals("Troops are loaded onto Transport", 75, h1.getQtyGroundDefense());
		assertEquals("Troops are loaded onto Transport", 0, p1.getTransportManager().getQtyFreeTransport());
		TransportTaskInvade task = (TransportTaskInvade) p1.getTransportManager().getTransportsAsArray()[0].getTask();
		assertEquals("Troops are loaded onto Transport", 25, task.getQtyAttackers());
		// assertEquals("Troops are loaded onto Transport", h2, task.getDestination());

		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos();
		assertEquals("Transport1", "TestFighter1", moos);
		assertEquals("Troops are loaded onto Transport", transport.getName(), moos[0].getName());
		assertEquals("Transport is at SS1", "SS1", moos[0].getOrbit().getStellarObject());
		assertEquals("Transport is at SS1", "SS1-A", moos[0].getOrbit().getPlanet());

		transport.setOrbit(new Orbit(h2));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals("Transport is at SS1", "SS2", moos[0].getOrbit().getStellarObject());
		assertEquals("Transport is at SS1", "SS2-A", moos[0].getOrbit().getPlanet());

		assertEquals(player2, planet2.getOwner());
		assertEquals(player2, ssd2.getOwnerDelegates()[0]);
		assertEquals(true, planet2.isBlockaded());
		assertEquals(150, ssd5.getRange(player1));

		TransportDelegate td = (TransportDelegate) moos[0];
		serverProxy.invadePlanet(td, planet2, null);
		serverProxy._processCommandResponse();

		PlanetInvadeEvent ipe = (PlanetInvadeEvent) getEvent(PlanetInvadeEvent.class);
		assertNotNull(ipe);
		assertEquals(planet2.getName(), ipe.getTarget());
		assertEquals(25, ipe.getQtyAttackers());
		assertEquals(1, ipe.getQtyDefenders());
		assertEquals("Event is correct", "SS2-A", ipe.getTarget());
		assertTrue(ipe.getRemainingAttackers() > 0);

		assertEquals("TestFighter1", DeligateCacheManager.getMoos());
		assertEquals(1, player1.getQtyFreeTransport());

		assertEquals(player1, planet2.getOwner());
		assertEquals(player1, ssd2.getOwnerDelegates()[0]);
		assertEquals(false, planet2.isBlockaded());
		assertEquals(50, ssd5.getRange(player1));
	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testInvadeWithMiningColony() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testInvadeWithMiningColony() *************************");
		createMap();

		SolarSystem ssm = newSolarSystem("SSM", 50, 50);
		Planet m = newPlanet("SSM-A", ssm, null, 40, 0.2);
		// Space station for bombard has modules and planet bonuses
		m.deploy(h2, new Orbital(tech_spaceStation, null, null));

		MotEngine.getInstance().getRelations().setWar(p1, p2);

		Transport transport = new Transport(p1);
		p1.getTransportManager().setTransport(transport);

		h1.setQtyGroundDefense(100);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		StellarObjectDelegate ssdm = DeligateCacheManager.getStellarObject("SSM");
		PlanetDelegate mine = DeligateCacheManager.getPlanet("SSM-A");

		assertNull(DeligateCacheManager.getMoos());
		assertEquals(1, player1.getQtyFreeTransport());

		serverProxy.loadTransportCmd(planet1, null, 10, 1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());
		assertEquals(0, player1.getQtyFreeTransport());

		assertEquals("Troops are loaded onto Transport", 90, h1.getQtyGroundDefense());
		assertEquals("Troops are loaded onto Transport", 0, p1.getTransportManager().getQtyFreeTransport());
		TransportTaskInvade task = (TransportTaskInvade) p1.getTransportManager().getTransportsAsArray()[0].getTask();
		assertEquals("Troops are loaded onto Transport", 10, task.getQtyAttackers());
		// assertEquals("Troops are loaded onto Transport", h2, task.getDestination());

		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos();
		assertEquals("Troops are loaded onto Transport", 1, moos.length);
		assertEquals("Troops are loaded onto Transport", transport.getName(), moos[0].getName());
		assertEquals("Transport is at SS1", "SS1", moos[0].getOrbit().getStellarObject());
		assertEquals("Transport is at SS1", "SS1-A", moos[0].getOrbit().getPlanet());

		assertEquals(p2, ssm.getOwners()[0]);
		assertEquals(player2, ssdm.getOwnerDelegates()[0]);
		assertEquals(player2, mine.getOrigin());
		assertEquals(player2, mine.getOwner());
		assertEquals(tech_spaceStation, mine.getOrbital().getHullTech());
		assertEquals(mine, planet2.getMiningColonyDelegate());
		assertEquals(0, planet2.getMiningPopulation());
		assertEquals(0, planet2.getMiningProduction());
		assertEquals(0, planet2.getMiningResearch());
		assertEquals(0, planet2.getMiningMoney());

		transport.setOrbit(new Orbit(h2));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals("Transport is at SS1", "SS2", moos[0].getOrbit().getStellarObject());
		assertEquals("Transport is at SS1", "SS2-A", moos[0].getOrbit().getPlanet());

		assertEquals(player2, planet2.getOwner());
		assertEquals(player2, ssd2.getOwnerDelegates()[0]);

		TransportDelegate td = (TransportDelegate) moos[0];
		serverProxy.invadePlanet(td, planet2, null);
		serverProxy._processCommandResponse();

		PlanetInvadeEvent ipe = (PlanetInvadeEvent) getEvent(PlanetInvadeEvent.class);
		assertNotNull(ipe);
		assertEquals(planet2.getName(), ipe.getTarget());
		assertEquals(10, ipe.getQtyAttackers());
		assertEquals(0, ipe.getQtyDefenders());
		assertEquals(10, ipe.getRemainingAttackers());
		assertEquals(0, ipe.getRemainingDefenders());
		// assertEquals("Event is correct", "Transport1", ipe.get);
		assertEquals("Event is correct", "SS2-A", ipe.getTarget());

		assertNull(DeligateCacheManager.getMoos());
		assertEquals(1, player1.getQtyFreeTransport());

		assertEquals(player1, planet2.getOwner());
		assertEquals(player1, ssd2.getOwnerDelegates()[0]);

		assertEquals(p1, ssm.getOwners()[0]);
		assertEquals(player1, ssdm.getOwnerDelegates()[0]);
		assertEquals(player2, mine.getOrigin());
		assertEquals(player1, mine.getOwner());
		assertEquals(tech_spaceStation, mine.getOrbital().getHullTech());
		assertEquals(mine, planet2.getMiningColonyDelegate());
		assertEquals(0, planet2.getMiningPopulation());
		assertEquals(0, planet2.getMiningProduction());
		assertEquals(0, planet2.getMiningResearch());
		assertEquals(0, planet2.getMiningMoney());
	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testInvaded() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testInvaded() *************************");
		createMap();

		MotEngine.getInstance().getRelations().setWar(p1, p2);

		h1.setScanners(new Scanners(tech_photon200));
		h2.setQtyGroundDefense(100);
		p2.addEngine(tech_fusion400);
		p2.getMayor().notifyNeedTroops(h1);

		assertEquals(75, h2.getQtyGroundDefense());
		assertEquals(25, ((TransportTaskInvade) p2.getTransportManager().getTransportsAsArray()[0].getTask()).getQtyAttackers());

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		UserPlayerDelegate player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer(p1.getName());
		PlayerDelegate player2 = (PlayerDelegate) DeligateCacheManager.getPlayer(p2.getName());

		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());
		assertEquals("SS1-A", DeligateCacheManager.getMoos()[0].getNavigator().getDestination().getPlanet());
		assertEquals(1, DeligateCacheManager.getMoos()[0].getNavigator().getEta());
		assertEquals(player2, DeligateCacheManager.getMoos()[0].getOwnerDelegate());

		assertEquals(player1, planet1.getOwner());
		assertEquals(player1, ssd1.getOwnerDelegates()[0]);

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		assertEquals(player2, planet1.getOwner());
		assertEquals(player2, ssd1.getOwnerDelegates()[0]);
		assertNull(DeligateCacheManager.getMoos());

		PlanetInvadeEvent ipe = (PlanetInvadeEvent) getEvent(PlanetInvadeEvent.class);
		assertNotNull(ipe);
		assertEquals(planet1.getName(), ipe.getTarget());
		assertEquals(25, ipe.getQtyAttackers());
		assertEquals(0, ipe.getQtyDefenders());
		assertEquals(25, ipe.getRemainingAttackers());
		assertEquals(0, ipe.getRemainingDefenders());
		assertEquals("Event is correct", "SS1-A", ipe.getTarget());

	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testInvadedWithMiningColony() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testInvadedWithMiningColony() *************************");
		createMap();

		SolarSystem ssm = newSolarSystem("SSM", 50, 50);
		Planet m = newPlanet("SSM-A", ssm, null, 40, 0.2);
		m.deploy(h1, new Orbital(tech_spaceStation, null, null));

		MotEngine.getInstance().getRelations().setWar(p1, p2);

		h1.setScanners(new Scanners(tech_photon200));
		h2.setQtyGroundDefense(100);
		p2.addEngine(tech_fusion400);
		p2.getMayor().notifyNeedTroops(h1);

		assertEquals(75, h2.getQtyGroundDefense());
		assertEquals(25, ((TransportTaskInvade) p2.getTransportManager().getTransportsAsArray()[0].getTask()).getQtyAttackers());

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		StellarObjectDelegate ssdm = DeligateCacheManager.getStellarObject("SSM");
		PlanetDelegate mine = DeligateCacheManager.getPlanet("SSM-A");
		UserPlayerDelegate player1 = (UserPlayerDelegate) DeligateCacheManager.getPlayer(p1.getName());
		PlayerDelegate player2 = (PlayerDelegate) DeligateCacheManager.getPlayer(p2.getName());

		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());
		assertEquals("SS1-A", DeligateCacheManager.getMoos()[0].getNavigator().getDestination().getPlanet());
		assertEquals(1, DeligateCacheManager.getMoos()[0].getNavigator().getEta());
		assertEquals(player2, DeligateCacheManager.getMoos()[0].getOwnerDelegate());

		assertEquals(player1, planet1.getOwner());
		assertEquals(player1, ssd1.getOwnerDelegates()[0]);

		assertEquals(p1, ssm.getOwners()[0]);
		assertEquals(player1, ssdm.getOwnerDelegates()[0]);
		assertEquals(tech_spaceStation, mine.getOrbital().getHullTech());
		assertEquals(player1, mine.getOrigin());
		assertEquals(player1, mine.getOwner());
		assertEquals(mine, planet1.getMiningColonyDelegate());
		assertEquals(0, planet1.getMiningPopulation());
		assertEquals(0, planet1.getMiningProduction());
		assertEquals(0, planet1.getMiningResearch());
		assertEquals(0, planet1.getMiningMoney());

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		assertEquals(player2, planet1.getOwner());
		assertEquals(player2, ssd1.getOwnerDelegates()[0]);
		assertNull(DeligateCacheManager.getMoos());

		PlanetInvadeEvent ipe = (PlanetInvadeEvent) getEvent(PlanetInvadeEvent.class);
		assertNotNull(ipe);
		assertEquals(planet1.getName(), ipe.getTarget());
		assertEquals(25, ipe.getQtyAttackers());
		assertEquals(0, ipe.getQtyDefenders());
		assertEquals(25, ipe.getRemainingAttackers());
		assertEquals(0, ipe.getRemainingDefenders());
		assertEquals("Event is correct", "SS1-A", ipe.getTarget());

		assertEquals(p2, ssm.getOwners()[0]);
		assertEquals(player2, ssdm.getOwnerDelegates()[0]);
		assertEquals(tech_spaceStation, mine.getOrbital().getHullTech());
		assertEquals(player1, mine.getOrigin());
		assertEquals(player2, mine.getOwner());
		assertEquals(mine, planet1.getMiningColonyDelegate());
		assertEquals(0, planet1.getMiningPopulation());
		assertEquals(0, planet1.getMiningProduction());
		assertEquals(0, planet1.getMiningResearch());
		assertEquals(0, planet1.getMiningMoney());
	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testInvadedByUserPlayerWithTradeRoute() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testInvadedByUserPlayerWithTradeRoute() *************************");
		createMap();

		p2.addTechnology(tech_trade_route_bonus);

		MotEngine.getInstance().getRelations().setWar(p1, p2);

		SolarSystem sst = newSolarSystem("SST", 200, 50);
		Planet ht = newPlanet("SST-A", sst, p2, 110, 1.2, 110, 50);
		
		TradeRouteFactory.create(ss2, sst);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate ssdt = DeligateCacheManager.getStellarObject("SST");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate planett = DeligateCacheManager.getPlanet("SST-A");

		TradeRouteDelegate[] trl = DeligateCacheManager.getTradeRoutes();
		assertEquals(ss2, sst, 3000, trl);

		Transport transport = newTransportWithTroopsLoaded(h1, null, 100);
		transport.transferTo(sst.getOrbit());

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals(16, planet2.getProductionOutput());
		assertEquals(52, planett.getProductionOutput());

		TransportDelegate td = (TransportDelegate) DeligateCacheManager.getMoo(transport.getName());

		serverProxy.invadePlanet(td, planett, null);
		serverProxy._processCommandResponse();

		PlanetInvadeEvent ipe = (PlanetInvadeEvent) getEvent(PlanetInvadeEvent.class);
		assertEquals(100, ipe.getRemainingAttackers());
		assertEquals(0, ipe.getRemainingDefenders());
		
		assertNull(DeligateCacheManager.getTradeRoutes());
		
		assertEquals(12, planet2.getProductionOutput());
		assertEquals(40, planett.getProductionOutput());
	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testInvasionOffUserPlayerWithTradeRoute() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testInvasionOffUserPlayerWithTradeRoute() *************************");
		createMap();

		p1.addTechnology(tech_trade_route_bonus);

		MotEngine.getInstance().getRelations().setWar(p1, p2);

		SolarSystem sst = newSolarSystem("SST", 100, 50);
		Planet ht = newPlanet("SST-A", sst, p1, 110, 1.2, 110, 50);
		
		TradeRouteFactory.create(ss1, sst);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate ssdt = DeligateCacheManager.getStellarObject("SST");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate planett = DeligateCacheManager.getPlanet("SST-A");

		TradeRouteDelegate[] trl = DeligateCacheManager.getTradeRoutes();
		assertEquals(ss1, sst, 3000, trl);

		assertEquals(16, planet1.getProductionOutput());
		assertEquals(52, planett.getProductionOutput());

		Transport transport = newTransportWithTroopsLoaded(h2, ht, 100);
		transport.transferTo(sst.getOrbit());
		transport.arrivedAt(ht.getOrbit());

		serverProxy._processQueuedEvents();

		PlanetInvadeEvent ipe = (PlanetInvadeEvent) getEvent(PlanetInvadeEvent.class);
		assertEquals(100, ipe.getRemainingAttackers());
		assertEquals(0, ipe.getRemainingDefenders());
		
		assertNull(DeligateCacheManager.getTradeRoutes());
		
		assertEquals(12, planet1.getProductionOutput());
		assertEquals(40, planett.getProductionOutput());
	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testBombardByUserPlayerCommands() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testBombardCommands() *************************");
		createMap();

		MotEngine.getInstance().getRelations().setWar(p1, p2);
		CraftDesign cd = new CraftDesign("Bomber", tech_flyer60, tech_fusion25, tech_electronic25, tech_energy10, tech_ancientWeapon);
		Craft c1 = newCraft(cd, p1, ss2);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		CraftDelegate craft1 = DeligateCacheManager.getCrafts(player1)[0];

		serverProxy.craftBombardPlanetCmd(craft1, planet2, null, null);
		serverProxy._processCommandResponse();

		PlanetBombardedEvent pbe = (PlanetBombardedEvent) getEvent(PlanetBombardedEvent.class);
		assertNotNull(pbe);
		assertEquals(planet2.getName(), pbe.getTarget());
		assertTrue(pbe.getFactoriesDestroyed() > 0.0);
		assertTrue(pbe.getPopulationDestroyed() > 0.0);

		assertEquals(player2, ssd2.getOwnerDelegates()[0]);
		assertEquals(player2, planet2.getOwner());

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		serverProxy.craftBombardPlanetCmd(craft1, planet2, null, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertNull(ssd2.getOwners());
		assertNull(planet2.getOwner());

	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testBombardByUserPlayerWithMiningColony() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testBombardByUserPlayerWithMiningColony() *************************");
		createMap();

		SolarSystem ssm = newSolarSystem("SSM", 50, 50);
		Planet m = newPlanet("SSM-A", ssm, null, 40, 0.2);
		m.setProductionBonus(1.0);
		m.setMoneyBonus(20);
		m.setResearchBonus(1.0);
		m.deploy(h2, new Orbital(tech_spaceStation, null, new Modules(tech_habitatI, tech_oreExtractor, tech_researchModule)));

		MotEngine.getInstance().getRelations().setWar(p1, p2);
		CraftDesign cd = new CraftDesign("Bomber", tech_flyer60, tech_fusion25, tech_electronic25, tech_energy10, tech_ancientWeapon);
		Craft c1 = newCraft(cd, p1, ss2);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate ssdm = DeligateCacheManager.getStellarObject("SSM");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate mine = DeligateCacheManager.getPlanet("SSM-A");
		CraftDelegate craft1 = DeligateCacheManager.getCrafts(player1)[0];

		assertEquals(p2, ssm.getOwners()[0]);
		assertEquals(player2, ssdm.getOwnerDelegates()[0]);
		assertEquals(player2, mine.getOrigin());
		assertEquals(player2, mine.getOwner());
		assertEquals(tech_spaceStation, mine.getOrbital().getHullTech());
		assertEquals(mine, planet2.getMiningColonyDelegate());
		assertEquals(10, planet2.getMiningPopulation());
		assertEquals(40, planet2.getMiningProduction());
		assertEquals(60, planet2.getMiningResearch());
		assertEquals(20, planet2.getMiningMoney());

		serverProxy.craftBombardPlanetCmd(craft1, planet2, null, null);
		serverProxy._processCommandResponse();

		PlanetBombardedEvent pbe = (PlanetBombardedEvent) getEvent(PlanetBombardedEvent.class);
		assertNotNull(pbe);
		assertEquals(planet2.getName(), pbe.getTarget());
		assertTrue(pbe.getFactoriesDestroyed() > 0.0);
		assertTrue(pbe.getPopulationDestroyed() > 0.0);

		assertEquals(player2, ssd2.getOwnerDelegates()[0]);
		assertEquals(player2, planet2.getOwner());

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		serverProxy.craftBombardPlanetCmd(craft1, planet2, null, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertNull(ssd2.getOwners());
		assertNull(planet2.getOwner());

		assertEquals(null, ssm.getOwners());
		assertEquals(null, ssdm.getOwnerDelegates());
		assertEquals(null, mine.getOrigin());
		assertEquals(null, mine.getOwner());
		assertEquals(null, mine.getOrbital());
		assertNull(planet2.getMiningColonyDelegate());
		assertEquals(0, planet2.getMiningPopulation());
		assertEquals(0, planet2.getMiningProduction());
		assertEquals(0, planet2.getMiningResearch());
		assertEquals(0, planet2.getMiningMoney());

	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testBombardOffUserPlayer() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testBombardOffUserPlayer() *************************");
		createMap();

		MotEngine.getInstance().getRelations().setWar(p1, p2);
		CraftDesign cd = new CraftDesign("Bomber", tech_flyer60, tech_fusion25, tech_electronic25, tech_energy10, tech_ancientWeapon);
		Craft c1 = newCraft(cd, p2, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");

		c1.bombard(h1);
		serverProxy._processQueuedEvents();

		PlanetBombardedEvent pbe = (PlanetBombardedEvent) getEvent(PlanetBombardedEvent.class);
		assertNotNull(pbe);
		assertEquals("SS1-A", pbe.getTarget());
		assertTrue(pbe.getFactoriesDestroyed() > 0.0);
		assertTrue(pbe.getPopulationDestroyed() > 0.0);

		assertEquals(player1, ssd1.getOwnerDelegates()[0]);
		assertEquals(player1, planet1.getOwner());

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		c1.bombard(h1);
		serverProxy._processQueuedEvents();

		assertNull(ssd1.getOwners());
		assertNull(planet1.getOwner());

	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testBombardOffUserPlayerWithMiningColony() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testBombardOffUserPlayerWithMiningColony() *************************");
		createMap();

		SolarSystem ssm = newSolarSystem("SSM", 50, 50);
		Planet m = newPlanet("SSM-A", ssm, null, 40, 0.2);
		m.setProductionBonus(1.0);
		m.setMoneyBonus(20);
		m.setResearchBonus(1.0);
		m.deploy(h1, new Orbital(tech_spaceStation, null, new Modules(tech_habitatI, tech_oreExtractor, tech_researchModule)));

		MotEngine.getInstance().getRelations().setWar(p1, p2);
		CraftDesign cd = new CraftDesign("Bomber", tech_flyer60, tech_fusion25, tech_electronic25, tech_energy10, tech_ancientWeapon);
		Craft c1 = newCraft(cd, p2, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssdm = DeligateCacheManager.getStellarObject("SSM");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate mine = DeligateCacheManager.getPlanet("SSM-A");

		assertEquals(p1, ssm.getOwners()[0]);
		assertEquals(player1, ssdm.getOwnerDelegates()[0]);
		assertEquals(tech_spaceStation, mine.getOrbital().getHullTech());
		assertEquals(player1, mine.getOrigin());
		assertEquals(player1, mine.getOwner());
		assertEquals(mine, planet1.getMiningColonyDelegate());
		assertEquals(10, planet1.getMiningPopulation());
		assertEquals(40, planet1.getMiningProduction());
		assertEquals(60, planet1.getMiningResearch());
		assertEquals(20, planet1.getMiningMoney());

		c1.bombard(h1);
		serverProxy._processQueuedEvents();

		PlanetBombardedEvent pbe = (PlanetBombardedEvent) getEvent(PlanetBombardedEvent.class);
		assertNotNull(pbe);
		assertEquals("SS1-A", pbe.getTarget());
		assertTrue(pbe.getFactoriesDestroyed() > 0.0);
		assertTrue(pbe.getPopulationDestroyed() > 0.0);

		assertEquals(player1, ssd1.getOwnerDelegates()[0]);
		assertEquals(player1, planet1.getOwner());

		serverProxy.endTurnCmd(null, null);
		serverProxy._processCommandResponse();

		c1.bombard(h1);
		serverProxy._processQueuedEvents();

		assertNull(ssd1.getOwners());
		assertNull(planet1.getOwner());

		assertEquals(null, ssm.getOwners());
		assertEquals(null, ssdm.getOwnerDelegates());
		assertEquals(null, mine.getOrbital());
		assertEquals(null, mine.getOrigin());
		assertEquals(null, mine.getOwner());
		assertNull(planet1.getMiningColonyDelegate());
		assertEquals(0, planet1.getMiningPopulation());
		assertEquals(0, planet1.getMiningProduction());
		assertEquals(0, planet1.getMiningResearch());
		assertEquals(0, planet1.getMiningMoney());

	}


	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testBombardByUserPlayerWithTradeRoute() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testBombardByUserPlayerWithTradeRoute() *************************");
		createMap();
		
		p2.addTechnology(tech_trade_route_bonus);
		
		MotEngine.getInstance().getRelations().setWar(p1, p2);

		SolarSystem sst = newSolarSystem("SST", 200, 50);
		Planet ht = newPlanet("SST-A", sst, p2, 110, 1.2, 110, 50);
		
		TradeRouteFactory.create(ss2, sst);

		CraftDesign cd = new CraftDesign("Bomber", tech_flyer60, tech_fusion25, tech_electronic25, tech_energy10, tech_ancientWeapon);
		Craft c1 = newCraft(cd, p1, sst);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate ssdt = DeligateCacheManager.getStellarObject("SST");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate planett = DeligateCacheManager.getPlanet("SST-A");

		TradeRouteDelegate[] trl = DeligateCacheManager.getTradeRoutes();
		assertEquals(ss2, sst, 3000, trl);

		assertEquals(16, planet2.getProductionOutput());
		assertEquals(26, planett.getProductionOutput());

		CraftDelegate craft1 = DeligateCacheManager.getCrafts(player1)[0];

		serverProxy.craftBombardPlanetCmd(craft1, planett, null, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		
		PlanetBombardedEvent pbe = (PlanetBombardedEvent) getEvent(PlanetBombardedEvent.class);
		assertNotNull(pbe);
		assertEquals(planett.getName(), pbe.getTarget());
		assertTrue(planett.getPopulation() > 0.0);
		assertTrue(pbe.getFactoriesDestroyed() > 0.0);
		assertTrue(pbe.getPopulationDestroyed() > 0.0);

		assertNull(DeligateCacheManager.getTradeRoutes());

		assertEquals(12, planet2.getProductionOutput());
		assertEquals(27, planett.getProductionOutput()); // Crisis increases production
	}

	/**
	 * Test galaxy attack planet command and all the commands for crafts in a
	 * battle.
	 */
	public void testBombardOffUserPlayerWithTradeRoute() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testBombardOffUserPlayerWithTradeRoute() *************************");
		createMap();
		
		p1.addTechnology(tech_trade_route_bonus);
		
		MotEngine.getInstance().getRelations().setWar(p1, p2);

		SolarSystem sst = newSolarSystem("SST", 100, 50);
		Planet ht = newPlanet("SST-A", sst, p1, 110, 1.2, 110, 50);
		
		TradeRouteFactory.create(ss1, sst);

		CraftDesign cd = new CraftDesign("Bomber", tech_flyer60, tech_fusion25, tech_electronic25, tech_energy10, tech_ancientWeapon);
		Craft c1 = newCraft(cd, p2, sst);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate ssdt = DeligateCacheManager.getStellarObject("SST");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate planett = DeligateCacheManager.getPlanet("SST-A");

		TradeRouteDelegate[] trl = DeligateCacheManager.getTradeRoutes();
		assertEquals(ss1, sst, 3000, trl);

		assertEquals(16, planet1.getProductionOutput());
		assertEquals(26, planett.getProductionOutput());

		CraftDelegate craft1 = DeligateCacheManager.getCrafts(player2)[0];

		c1.bombard(ht);
		serverProxy._processQueuedEvents();
		
		PlanetBombardedEvent pbe = (PlanetBombardedEvent) getEvent(PlanetBombardedEvent.class);
		assertNotNull(pbe);
		assertEquals(planett.getName(), pbe.getTarget());
		assertTrue(planett.getPopulation() > 0.0);
		assertTrue(pbe.getFactoriesDestroyed() > 0.0);
		assertTrue(pbe.getPopulationDestroyed() > 0.0);

		assertNull(DeligateCacheManager.getTradeRoutes());

		assertEquals(12, planet1.getProductionOutput());
		assertEquals(27, planett.getProductionOutput()); // Crisis increases output
	}

	/**
	 * Tests the deployment of stations - Space Station - Space Station modules.
	 */
	public void testDeploySpaceStationAndAbandonMiningColony() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testDeployAndAbandonMiningColony() *************************");
		createMap();

		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate pd3 = DeligateCacheManager.getPlanet("SS3-A");
		StellarObjectDelegate sd3 = DeligateCacheManager.getStellarObject("SS3");
		p1.getTransportManagerNotNull().setTransport(new Transport(p1));

		new ProjectSpaceStation(new SpaceStationDesign(tech_spaceStation), null).build(h1, true);
		Transport transport = p1.getTransportManager().getTransportsAsArray()[0];
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMOOsAt(new Rectangle(0, 0, 1000, 1000), false, pd1.getOrbit());
		TransportDelegate td = (TransportDelegate) moos[0];

		assertEquals(null, pd3.getOrbital());
		assertEquals((Orbit) null, ((TransportTaskDeployStation) transport.getTask()).getDestination());
		assertEquals("SS1-A", moos[0].getOrbit().getPlanet());

		serverProxy.mooDeploy(td, null, pd3, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(h3, ((TransportTaskDeployStation) transport.getTask()).getDestination().getPlanet());
		assertEquals(ss3, transport.getNavigator().getDestination().getStellarObject());

		p1.getTransportManager().endTurn();
		p1.getTransportManager().endTurn();
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(null, transport.getTask());
		assertEquals(tech_spaceStation, pd3.getOrbital().getHullTech());
		assertEquals(tech_core_habitat, pd3.getOrbital().getModuleTechs());
		assertEquals(null, pd3.getOrbital().getScannerTech());
		assertEquals("SS3-A", pd1.getMiningColonyDelegate().getName());

		//
		// We have deployed a space station successfully
		// Now lets deploy a module to the space station
		//

		assertEquals(tech_core_habitat, pd3.getOrbital().getModuleTechs());

		p1.finishResearchProject(new ResearchProject(tech_researchModule, p1, false), true, false);

		assertEquals(2, pd3.getOrbital().getModuleTechs().getModules().length);
		assertEquals(tech_core_habitat, pd3.getOrbital().getModuleTechs().getModules()[0]);
		assertEquals(tech_researchModule, pd3.getOrbital().getModuleTechs().getModules()[1]);

		//
		// Test Planet delegate fetches correct values for mining station research,
		// production population, money
		//

		h3.setProductionBonus(1.0);
		h3.setResearchBonus(1.0);
		h3.setMoneyBonus(8);

		h3.getOrbital().addModule(tech_oreExtractor);
		h3.getOrbital().addModule(tech_habitatI);
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(8, pd1.getMiningMoney());
		assertEquals(40, pd1.getMiningProduction());
		assertEquals(60, pd1.getMiningResearch());
		assertEquals(20, pd1.getMiningPopulation());

		//
		// Test abandon space station works
		//

		serverProxy.abandonCmd(pd3, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(null, h3.getOrbital());
		assertEquals(null, h1.getMiningStation());
		assertEquals(null, h3.getOwner());
		assertEquals(p1, h1.getOwner());

		assertEquals(0, pd1.getMiningMoney());
		assertEquals(0, pd1.getMiningProduction());
		assertEquals(0, pd1.getMiningResearch());
		assertEquals(0, pd1.getMiningPopulation());
	}

	/**
	 * Tests the deployment of stations - Space Station - Space Station modules.
	 */
	public void testAbandonPlanetWithMiningColony() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testDeployAndAbandonSpaceStation() *************************");
		createMap();

		p1.addModule(tech_habitatI);
		p1.addModule(tech_habitatII);
		p1.addModule(tech_oreExtractor);
		p1.addModule(tech_researchModule);
		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate pd3 = DeligateCacheManager.getPlanet("SS3-A");
		StellarObjectDelegate sd3 = DeligateCacheManager.getStellarObject("SS3");

		Transport transport = new Transport(p1);
		p1.getTransportManagerNotNull().setTransport(transport);
		transport.setTask(new TransportTaskDeployStation(transport, h1, null, new Orbital(tech_spaceStation, null, null)));
		transport.setOrbit(ss3, h3);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos(player1);
		TransportDelegate td = (TransportDelegate) moos[0];

		serverProxy.mooDeploy(td, null, pd3, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(player1, pd1.getOwner());
		assertEquals("SS3-A", pd1.getMiningColonyDelegate().getName());
		assertEquals(player1, pd3.getOwner());
		assertEquals(player1.getName(), sd3.getOwners()[0]);
		assertEquals(tech_spaceStation, pd3.getOrbital().getHullTech());
		assertEquals(tech_core_habitat, pd3.getOrbital().getModuleTechs().getAllAsArray()[0]);
		assertEquals(tech_habitatI, pd3.getOrbital().getModuleTechs().getAllAsArray()[1]);
		assertEquals(tech_habitatII, pd3.getOrbital().getModuleTechs().getAllAsArray()[2]);
		assertEquals(tech_oreExtractor, pd3.getOrbital().getModuleTechs().getAllAsArray()[3]);
		assertEquals(tech_researchModule, pd3.getOrbital().getModuleTechs().getAllAsArray()[4]);
		assertEquals(null, pd3.getOrbital().getScannerTech());

		assertEquals(0, pd1.getMiningMoney());
		assertEquals(20, pd1.getMiningProduction());
		assertEquals(30, pd1.getMiningResearch());
		assertEquals(20, pd1.getMiningPopulation());

		//
		// Test abandon planet with mining station works
		//

		serverProxy.abandonCmd(pd1, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(null, h3.getOrbital());
		assertEquals(null, h3.getOwner());
		assertEquals(null, ss3.getOwners());
		assertEquals(null, h1.getOwner());
		assertEquals(null, ss1.getOwners());
		assertEquals(null, h1.getMiningStation());

		assertNull(pd1.getMiningColonyDelegate());
		assertEquals(null, pd1.getOwner());
		assertEquals(null, ss1.getOwners());
		assertEquals(null, pd3.getOwner());
		assertEquals(null, ss3.getOwners());
	}

	/**
	 * Tests the deployment of stations - Space Station - Space Station modules.
	 */
	public void testAbandonPlanetWithTradeRoute() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testAbandonPlanetWithTradeRoute() *************************");
		createMap();

		p1.addTechnology(tech_trade_route_bonus);
		
		SolarSystem sst1 = newSolarSystem("SST1", 100, 50);
		Planet ht1 = newPlanet("SST1-A", sst1, p1, 110, 1.2, 110, 50);
		
		SolarSystem sst2 = newSolarSystem("SST2", 50, 50);
		Planet ht2a = newPlanet("SST2-A", sst2, p1, 110, 1.2, 110, 50);
		Planet ht2b = newPlanet("SST2-B", sst2, p1, 110, 1.2, 110, 50);
		
		TradeRouteFactory.create(ss1, sst1);
		TradeRouteFactory.create(ss1, sst2);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SST1");
		StellarObjectDelegate ssdt = DeligateCacheManager.getStellarObject("SST2");
		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planett1a = DeligateCacheManager.getPlanet("SST1-A");
		PlanetDelegate planett2a = DeligateCacheManager.getPlanet("SST2-A");
		PlanetDelegate planett2b = DeligateCacheManager.getPlanet("SST2-B");
		
		assertEquals(20, h1.getProductionOutput());
		assertEquals(52, ht1.getProductionOutput());
		assertEquals(52, ht2a.getProductionOutput());
		assertEquals(52, ht2b.getProductionOutput());
		
		assertEquals(20, planet1.getProductionOutput());
		assertEquals(52, planett1a.getProductionOutput());
		assertEquals(52, planett2a.getProductionOutput());
		assertEquals(52, planett2b.getProductionOutput());

		TradeRouteDelegate[] trl = (TradeRouteDelegate[]) order(DeligateCacheManager.getTradeRoutes(), new TradeRouteDelegate[2], "SS1SST1", "SS1SST2");

		//
		// Test abandon planet that should destroy trading route
		//

		serverProxy.abandonCmd(planett1a, callbackSync, null);
		serverProxy._processCommandResponse();

		trl = DeligateCacheManager.getTradeRoutes();

		assertEquals(ss1, sst2, 3000, trl);
		assertEquals(16, planet1.getProductionOutput());
		assertEquals(0, planett1a.getProductionOutput());
		assertEquals(52, planett2a.getProductionOutput());
		assertEquals(52, planett2b.getProductionOutput());

		//
		// Test abandon planet 1
		//

		serverProxy.abandonCmd(planett2b, callbackSync, null);
		serverProxy._processCommandResponse();

		trl = DeligateCacheManager.getTradeRoutes();

		assertEquals(ss1, sst2, 3000, trl);
		assertEquals(16, planet1.getProductionOutput());
		assertEquals(0, planett1a.getProductionOutput());
		assertEquals(52, planett2a.getProductionOutput());
		assertEquals(0, planett2b.getProductionOutput());

		//
		// Test abandon planet 2
		//

		serverProxy.abandonCmd(planett2a, callbackSync, null);
		serverProxy._processCommandResponse();

		trl = DeligateCacheManager.getTradeRoutes();

		assertNull(trl);
		assertEquals(12, planet1.getProductionOutput());
		assertEquals(0, planett1a.getProductionOutput());
		assertEquals(0, planett2a.getProductionOutput());
		assertEquals(0, planett2b.getProductionOutput());


	}

	/**
	 * Tests the deployment of stations - Space Station - Space Station modules.
	 */
	public void testDeploySpaceStationAtBlackHole() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testDeployAndAbandonMiningColony() *************************");
		createMap();

		BlackHole blackHole = newBlackHole("BlackHole", 50, 50, 0.1f, false);

		p1.getTransportManagerNotNull().setTransport(new Transport(p1));
		new ProjectSpaceStation(new SpaceStationDesign(tech_spaceStation), null).build(h1, true);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");
		StellarObjectDelegate bh = DeligateCacheManager.getStellarObject("BlackHole");
		TransportDelegate transport = (TransportDelegate) DeligateCacheManager.getMoo("Transport1");

		serverProxy.mooMoveCmd(transport, bh, null, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Deploy station SpaceStation for SS1-A", transport.getTaskDescription());
		assertEquals("BlackHole", transport.getNavigator().getDestination().getStellarObject());

		p1.getTransportManager().endTurn();
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(null, transport.getOrbit());
		assertEquals("BlackHole", transport.getNavigator().getDestination().getStellarObject());
		assertEquals(2, transport.getNavigator().getEta());

		p1.getTransportManager().endTurn();
		p1.getTransportManager().endTurn();
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		//
		// Test Transport deploys and research output on home planet has been increased
		//

		assertEquals(10, pd1.getResearchOutput());

		serverProxy.mooDeploy(transport, bh, null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertNull(DeligateCacheManager.getMoo("Transport1"));
		assertTrue(bh.isOwnedBy(player1));
		assertEquals(11, pd1.getResearchOutput());
	}


	/**
	 * Tests the deployment of stations - Space Station - Space Station modules.
	 */
	public void testReBaseSpaceStation() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testReBaseSpaceStation() *************************");
		createMap();

		Planet h1b = newPlanet("SS1-B", ss1, p1, 100, 1.0, 100, 50);
		Planet h1c = newPlanet("SS1-C", ss1, 20, 0.2, 3.0);

		p1.addModule(tech_researchModule);
		p1.addModule(tech_oreExtractor);
		p1.addModule(tech_habitatI);
		
		newTransportWithStationLoaded(h1, tech_spaceStation);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ss1d = DeligateCacheManager.getStellarObject("SS1");
		PlanetDelegate pd1b = DeligateCacheManager.getPlanet("SS1-B");
		PlanetDelegate pd1c = DeligateCacheManager.getPlanet("SS1-C");
		TransportDelegate transport = (TransportDelegate) DeligateCacheManager.getMoo("Transport1");

		serverProxy.mooDeploy(transport, ss1d, pd1b, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Deploy station SpaceStation for SS1-B", transport.getTaskDescription());

//		p1.getTransportManager().endTurn();
//		serverProxy.updateDelegates(callbackSync);
//		serverProxy._processCommandResponse();

		//
		// Test Transport deploys and research output on home planet has been increased
		//

		assertEquals(100.0, h1b.getMaxPopulation(), 0.1);
		assertEquals(37, h1b.getProductionOutput());
		assertEquals(25, h1b.getResearchOutput());

		serverProxy.mooMoveCmd(transport, ss1d, pd1c, false, callbackSync, null);
		serverProxy._processCommandResponse();
		
		serverProxy.mooDeploy(transport, ss1d, pd1c, callbackSync, null);
		serverProxy._processCommandResponse();

		assertNull(DeligateCacheManager.getMoo("Transport1"));
		assertTrue(pd1c.isOwnedBy(player1));
		
		assertEquals(120.0, h1b.getMaxPopulation(), 0.1);
		assertEquals(117, h1b.getProductionOutput());
		assertEquals(55, h1b.getResearchOutput());

		assertEquals(24.0, pd1c.getMaxPopulation(), 0.1);
		assertEquals(80, pd1c.getProductionOutput());
		assertEquals(30, pd1c.getResearchOutput());
	}

	/**
	 * Tests the deployment of stations
	 */
	public void testDeployOutpost() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testDeployOutpost() *************************");
		createMap();

		p1.getTransportManagerNotNull().setTransport(new Transport(p1));
		// new ProjectOutpost(tech_outpost, null).build(h1, true);
		createOutpost(tech_outpost, null, h1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate pd3 = DeligateCacheManager.getPlanet("SS3-A");
		StellarObjectDelegate sd3 = DeligateCacheManager.getStellarObject("SS3");

		Transport transport = p1.getTransportManager().getTransportsAsArray()[0];
		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMOOsAt(new Rectangle(0, 0, 1000, 1000), false,
				pd1.getOrbit());
		TransportDelegate td = (TransportDelegate) moos[0];

		assertEquals((Orbit) null, ((TransportTaskDeployStation) transport.getTask()).getDestination());

		serverProxy.mooMoveCmd(td, sd3, null, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("SS3", td.getNavigator().getDestination().getStellarObject());
		assertNull(td.getNavigator().getDestination().getPlanet());
		assertEquals(2, td.getNavigator().getEta());

		p1.getTransportManager().endTurn();
		p1.getTransportManager().endTurn();

		serverProxy.mooDeploy(td, sd3, null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_outpost, ss3.getOrbital().getOrbital().getHullTech());
		assertEquals(tech_outpost, sd3.getStellarObjectOrbital().getOrbital().getHullTech());
		assertEquals(p1.getName(), sd3.getStellarObjectOrbital().getOwner().getName());
	}

	/**
	 * Tests commands on transport objects - Tests getVisibleMoosByLocation returns
	 * Transports - Tests User can give Transport commands - Tests Transport does
	 * not auto deploy when owned by a User Player - Tests Transport will not deploy
	 * to a planet owned by another player
	 */
	public void testMoveDeleteTransport() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testDeployOutpost() *************************");
		createMap();

		p1.getTransportManagerNotNull().setTransport(new Transport(p1));
		new ProjectOutpost(new OutpostDesign(tech_outpost), null).build(h1, false);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate pd3 = DeligateCacheManager.getPlanet("SS3-A");
		StellarObjectDelegate sd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate sd3 = DeligateCacheManager.getStellarObject("SS3");

		//
		// Create a Transport and test that it is visible on the galaxy map
		//

		Transport transport = p1.getTransportManager().getTransportsAsArray()[0];
		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos();
		TransportDelegate td = (TransportDelegate) moos[0];

		assertEquals(transport.getName(), td.getName());

		assertEquals((Orbit) null, ((TransportTaskDeployStation) transport.getTask()).getDestination());

		//
		// Test the transport can be moved using the normal moo move command
		//

		serverProxy.mooMoveCmd(td, sd2, null, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("SS2", transport.getNavigator().getDestination().getStellarObject().getName());
		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());

		p1.getTransportManager().endTurn();
		p1.getTransportManager().endTurn();
		p1.getTransportManager().endTurn();
		p1.getTransportManager().endTurn();
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals("SS2", td.getOrbit().getStellarObject());
		assertEquals("Transport1", DeligateCacheManager.getMoos()[0].getName());

		//
		// Test the transport will not deploy to a solar system not owned by player1
		//

		try {
			serverProxy.mooDeploy(td, sd2, null, callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("You cannot deploy to a stellar object owned by someone else");
		} catch (ValidationError e) {
		}

		assertEquals(tech_outpost, ((TransportTaskDeployStation) transport.getTask()).getOrbital().getHullTech());

		//
		// Test the transport can be deleted
		//

		serverProxy.mooDeleteCmd(td, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(null, transport.getTask());
		assertNull(DeligateCacheManager.getMoos());

	}

	/**
	 * Tests the deployment of stations - Outpost
	 */
	public void testDeleteTransport() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testDeployOutpost() *************************");
		createMap();

		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		StellarObjectDelegate solarSystem1 = DeligateCacheManager.getStellarObject("SS1");

		p1.getTransportManagerNotNull().setTransport(new Transport(p1));
		p1.getTransportManagerNotNull().setTransport(new Transport(p1));
		p1.getTransportManagerNotNull().setTransport(new Transport(p1));

//        new ProjectOutpost(tech_outpost, null).build(h1, true);  
		createOutpost(tech_outpost, null, h1);
		new ProjectSpaceStation(new SpaceStationDesign(tech_spaceStation), null).build(h1, true);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		h1.setPopulation(100);
		h1.setQtyGroundDefense(100);
		serverProxy.loadTransportCmd(planet1, null, 10, 1, null, null);
		serverProxy._processCommandResponse();

		TransportDelegate[] moos = (TransportDelegate[]) order(DeligateCacheManager.getMoos(player1),
				new TransportDelegate[3], "Transport1", "Transport2", "Transport3");

		assertEquals("Deploy station Outpost for SS1-A", moos[0].getTaskDescription());
		assertEquals("Deploy station SpaceStation for SS1-A", moos[1].getTaskDescription());
		assertEquals("Move 10 troops ", moos[2].getTaskDescription());

		serverProxy.mooDeleteCmd(moos[0], null, null);
		serverProxy._processCommandResponse();

		serverProxy.mooDeleteCmd(moos[1], null, null);
		serverProxy._processCommandResponse();

		serverProxy.mooDeleteCmd(moos[2], null, null);
		serverProxy._processCommandResponse();

		assertNull(DeligateCacheManager.getMoos(player1));
	}

	/**
	 * Test the commands to view player spies and assign tasks.
	 */
	public void testUserPlayerProperties() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testUserPlayerProperties() *************************");
		createMap();

		SolarSystem outOfRangeSS = newSolarSystem("OOR", 900, 900);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd = DeligateCacheManager.getStellarObject("SS3");
		PlanetDelegate pd = DeligateCacheManager.getPlanet("SS3-A");

		serverProxy.setFleetWayPoint(pd.getOrbit(), callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(h3, p1.getFleetWayPoint().getPlanet());
		assertEquals(ss3, p1.getFleetWayPoint().getStellarObject());
		assertEquals("SS3-A", player1.getFleetWayPoint().getPlanet());
		assertEquals("SS3", player1.getFleetWayPoint().getStellarObject());

		serverProxy.setFleetWayPoint(ssd.getOrbit(), callbackSync, null);
		serverProxy._processCommandResponse();
		assertEquals(null, p1.getFleetWayPoint().getPlanet());
		assertEquals(ss3, p1.getFleetWayPoint().getStellarObject());
		assertNull(player1.getFleetWayPoint().getPlanet());
		assertEquals("SS3", player1.getFleetWayPoint().getStellarObject());

		try {
			serverProxy.setFleetWayPoint(DeligateCacheManager.getStellarObject("OOR").getOrbit(), callbackSync,testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail();
		} catch (ValidationError e) {
			// Success
		}

		assertEquals(8, player1.getNetIncome()); // does not matter what this is, but much change

		serverProxy.setProjectCmd(DeligateCacheManager.getPlanet("SS1-A"), new TransportDesign(), 0, false, false,callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(false, h1.ownerWantsNoProject);
		assertEquals("Transport", DeligateCacheManager.getPlanet("SS1-A").getProjectDesign().getName());
		assertEquals(0, player1.getNetIncome()); // does not matter what this is, but much change

		serverProxy.setPlanetWantsNoProjectCmd(DeligateCacheManager.getPlanet("SS1-A"), true, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(true, h1.ownerWantsNoProject);
		assertNull(DeligateCacheManager.getPlanet("SS1-A").getProjectDesign());
		assertEquals(8, player1.getNetIncome()); // does not matter what this is, but much change

		serverProxy.setPlanetWantsNoProjectCmd(DeligateCacheManager.getPlanet("SS1-A"), false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(false, h1.ownerWantsNoProject);

		try {
			serverProxy.setPlanetWantsNoProjectCmd(DeligateCacheManager.getPlanet("SS2-A"), true, callbackSync,testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Does not belong to me");
		} catch (ValidationError e) {
			// Success
		}
	}

	/**
	 * Test the commands to view player spies and assign tasks.
	 */
	public void testUserGiveAssets() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testUserGiveAssets() *************************");
		createMap();

		p1.addFunds(99000);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");

		//
		// Giving money works
		//

		assertEquals(100, p2.getFunds());
		MotEngine.instance.getRelations().getRelationshipNotNull(p1, p2).setValue(50);
		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift(1000), null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(53, MotEngine.instance.getRelations().getRelation(p1, p2));
		assertEquals(1100, p2.getFunds());

		assertEquals(98100, player1.getFunds());

		//
		// Giving valuable tech works
		//

		p1.addWeapon(tech_fusionBeam4);
		assertEquals(false, p2.hasDiscovered(tech_fusionBeam4));
		MotEngine.instance.getRelations().getRelationship(p1, p2).setValue(50);
		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift(tech_fusionBeam4), null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(true, p2.hasDiscovered(tech_fusionBeam4));
		assertEquals(60, MotEngine.instance.getRelations().getRelation(p1, p2));
		assertEquals(60, player1.getRelation(player2).getValue());

		//
		// Giving valuable tech works
		//

		p1.addWeapon(tech_massDriver2);
		assertEquals(false, p2.hasDiscovered(tech_massDriver2));
		MotEngine.instance.getRelations().getRelationship(p1, p2).setValue(50);
		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift(tech_massDriver2), null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(55, MotEngine.instance.getRelations().getRelation(p1, p2));
		assertEquals(true, p2.hasDiscovered(tech_massDriver2));

		//
		// Giving tech already discovered does nothing
		//

		p1.addWeapon(tech_massDriver1);
		assertEquals(true, p2.hasDiscovered(tech_massDriver1));
		MotEngine.instance.getRelations().getRelationship(p1, p2).setValue(50);
		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift(tech_massDriver1), null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(50, MotEngine.instance.getRelations().getRelation(p1, p2));
		assertEquals(true, p2.hasDiscovered(tech_massDriver1));

		//
		// Can't give something I have not discovered
		//

		assertEquals(false, p1.hasDiscovered(tech_battleStationI));
		MotEngine.instance.getRelations().getRelationship(p1, p2).setValue(50);
		try {
			serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift(tech_battleStationI), null, callbackSync,testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("This should not WORK");
		} catch (ValidationError ex) {
		}

		assertEquals(50, MotEngine.instance.getRelations().getRelation(p1, p2));
		assertEquals(false, p2.hasDiscovered(tech_battleStationI));

		//
		// Giving planet works
		//

		Planet h2b = newPlanet("SS2-B", ss2, p1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		PlanetDelegate newPlanet = DeligateCacheManager.getPlanet("SS2-B");
		order(ssd2.getOwners(), player1.getName(), player2.getName());

		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift("SS2-B"), null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(p2.getName(), newPlanet.getOwnerName());
		order(ssd2.getOwners(), player2.getName());

		//
		// Giving mining station works
		//

		Planet _miner = newPlanet("Miner", ss2, p1);
		Planet _mine = newPlanet("Mine", ss2, null, 40, 0.2);
		_mine.deploy(_miner, new Orbital(tech_spaceStation, null, null));

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		PlanetDelegate miner = DeligateCacheManager.getPlanet("Miner");
		PlanetDelegate mine = DeligateCacheManager.getPlanet("Mine");

		try {
			serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift("Mine"), null, callbackSync, null);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail();
		} catch (ValidationError e) {
			// success
		}

		assertEquals(p1.getName(), mine.getOwnerName());

		//
		// Giving planet with mining station works
		//

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift("Miner"), null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(p2.getName(), mine.getOwnerName());
		assertEquals(p2.getName(), miner.getOwnerName());

		//
		// Giving craft works
		//

		Craft c1 = newCraft(fighterDesign, p1, ss2);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		CraftDelegate craft1 = DeligateCacheManager.getCraft(c1.getName());

		assertEquals(player1, craft1.getOwnerDelegate());
		assertEquals(player1, craft1.getOriginDelegate());

		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift(new NameList(craft1.getName())), null, callbackSync, null);
		serverProxy._processCommandResponse();

		// NOTE - craft is not visible so has vanished!
		assertNull(DeligateCacheManager.getCrafts(player1));
		assertNull(DeligateCacheManager.getCrafts(player2));

		//
		// Giving fleet works
		//

		Craft c2 = newCraft(fighterDesign, p1, ss1);
		Craft c3 = newCraft(fighterDesign, p1, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		CraftDelegate craft2 = DeligateCacheManager.getCraft(c2.getName());
		CraftDelegate craft3 = DeligateCacheManager.getCraft(c3.getName());

		assertEquals(player1, craft2.getOwnerDelegate());
		assertEquals(player1, craft3.getOriginDelegate());
		
		serverProxy.joinFleetCmd(craft2, callbackSync, null);
		serverProxy._processCommandResponse();
		
		serverProxy.joinFleetCmd(craft3, callbackSync, null);
		serverProxy._processCommandResponse();
		
		StellarObjectDelegate stellarObject1 = DeligateCacheManager.getStellarObject(ss1.getName());
		FleetDelegate fleet = DeligateCacheManager.getFleetsAt(player1, stellarObject1.getOrbit())[0];
		assertEquals(c2, c3, fleet.getCrafts());
		assertEquals(fleet, craft2.getFleetDelegate());
		assertEquals(fleet, craft3.getFleetDelegate());
		assertEquals("images/crafts/hull_flyer_blue.png", craft2.getImage());
		assertEquals("images/crafts/hull_flyer_blue.png", craft3.getImage());

		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.gift(new NameList(craft2.getName(), craft3.getName())), null, callbackSync, null);
		serverProxy._processCommandResponse();

		// Crafts have changed owner and are removed from fleet
		assertNull(DeligateCacheManager.getCrafts(player1));
		assertNull(DeligateCacheManager.getFleetsAt(player1, stellarObject1.getOrbit()));
		assertEquals(c2, c3, DeligateCacheManager.getCrafts(player2));
		assertNull(DeligateCacheManager.getCrafts(player2)[0].getFleetDelegate());
		assertNull(DeligateCacheManager.getCrafts(player2)[1].getFleetDelegate());
		assertEquals("images/crafts/hull_flyer_green.png", craft2.getImage());
		assertEquals("images/crafts/hull_flyer_green.png", craft3.getImage());
		
		assertNull(c2.getFleet());
		assertNull(c3.getFleet());
	}

	/**
	 * Test the commands to view player spies and assign tasks.
	 */
	public void testComputerGiveAssets() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testComputerGiveAssets() *************************");
		createMap(new SetupOption("qty-missed-techs", "0.0", 0));

		p2.addFunds(99000);
		assertEquals(100, p1.getFunds());

		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();
		callbackSync.waitTillComplete();

		//
		// Giving money works
		//

		p1.diplomaticRequest(p2, DiplomaticProposal.gift(1000), null);

		serverProxy._processQueuedEvents();

		assertEquals(1000, ((DiplomaticRequestEvent) lastEvent).getProposal().getOffered().getMoney());
		assertEquals(1100, p1.getFunds());
		assertEquals(1100, player1.getFunds());

		//
		// Giving tech works
		//

		assertNull(player1.getMissedTechnologies());

		Technology[] available = DeligateCacheManager.getTechsAvailableForResearch();
		TechnologyList techsDiscovered = player1.getTechnologies();

		assertTrue(Comp.in(tech_fusionBeam4, available));
		assertFalse(techsDiscovered.contains(tech_fusionBeam4));

		p2.addWeapon(tech_fusionBeam4);
		p1.diplomaticRequest(p2, DiplomaticProposal.gift(tech_fusionBeam4), null);

		serverProxy._processQueuedEvents();

		assertEquals(tech_fusionBeam4, ((DiplomaticRequestEvent) lastEvent).getProposal().getOffered().getTechnologList());

		available = DeligateCacheManager.getTechsAvailableForResearch();
		techsDiscovered = player1.getTechnologies();

		assertFalse(Comp.in(tech_fusionBeam4, available));
		assertTrue(techsDiscovered.contains(tech_fusionBeam4));

		//
		// Giving planet works
		//

		Planet h2b = newPlanet("SS2-B", ss2, p2);
		p1.diplomaticRequest(p2, DiplomaticProposal.gift(h2b.getName()), null);

		serverProxy._processQueuedEvents();

		PlanetDelegate[] planets = (PlanetDelegate[]) order(DeligateCacheManager.getPlanets(), new MotPlanetDelegate[4], "SS1-A", "SS2-A", "SS2-B", "SS3-A");

		assertEquals(p1.getName(), planets[0].getOwnerName());
		assertEquals(p2.getName(), planets[1].getOwnerName());
		assertEquals(p1.getName(), planets[2].getOwnerName());

	}

	/**
	 * Tests commands on swapping assets - Tests that user players may request asset
	 * swaps with computer players - Tests that computer players may request swaps
	 * with user players
	 */
	public void testSwapAssets() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testSwapAssets() *************************");
		createMap();
		owner = p1;

		MotEngine.instance.getRelations().getRelationshipNotNull(p1, p2).setValue(Relations.WOW);

		p1.addWeapon(tech_fusionBeam4);
		p2.addWeapon(tech_massDriver2);
		p1.addFunds(5000);

		Planet h1b = newPlanet("SS1-B", ss1, p1);
		Planet h2b = newPlanet("SS2-B", ss2, p2);

		Craft c1 = newCraft(fighterDesign, p1, ss2);
		Craft c2 = newCraft(fighterDesign, p1, ss2);
		Fleet f1 = FleetFactory.create(c1, c2);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		PlanetDelegate planet1a = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet1b = DeligateCacheManager.getPlanet("SS1-B");
		PlanetDelegate planet2a = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate planet2b = DeligateCacheManager.getPlanet("SS2-B");

		CraftDelegate craft1 = DeligateCacheManager.getCraft(c1.getName());
		CraftDelegate craft2 = DeligateCacheManager.getCraft(c2.getName());

		PlayerAssets offered = new PlayerAssets(tech_fusionBeam4, 1000, h1b.getName());
		PlayerAssets wanted = new PlayerAssets(tech_massDriver2, 0, h2b.getName());
		offered.addCraft(c1.getName());

		serverProxy.sendDiplomaticRequest(player2, new DiplomaticProposal(offered, wanted), null, callbackSync, null);
		serverProxy._processCommandResponse();
		callbackSync.waitTillComplete();
		serverProxy._processQueuedEvents();

		assertEquals("accepted", ((DiplomaticResponseEvent) lastEvent).getResponseVerb());
		assertEquals(tech_fusionBeam4,((DiplomaticResponseEvent) lastEvent).getProposal().getOffered().getTechnologies().getAllAsArray()[0]);
		assertEquals(tech_massDriver2,((DiplomaticResponseEvent) lastEvent).getProposal().getWanted().getTechnologies().getAllAsArray()[0]);
		assertEquals(1000, ((DiplomaticResponseEvent) lastEvent).getProposal().getOffered().getMoney());
		assertTrue(p1.hasDiscovered(tech_fusionBeam4));
		assertTrue(p1.hasDiscovered(tech_massDriver2));
		assertTrue(p2.hasDiscovered(tech_fusionBeam4));
		assertTrue(p2.hasDiscovered(tech_massDriver2));
		assertEquals(player2, craft1.getOwnerDelegate());
		assertEquals(player1, craft2.getOwnerDelegate());
		assertEquals((String) null, craft1.getFleet());
		assertEquals("Fleet1", craft2.getFleet());
		assertEquals(1, craft2.getFleetDelegate().getCrafts().length);
		assertEquals(craft2, craft2.getFleetDelegate().getCrafts()[0]);

		Technology[] available = DeligateCacheManager.getTechsAvailableForResearch();
		TechnologyList techsDiscovered = player1.getTechnologies();

		assertFalse(Comp.in(tech_fusionBeam4, available));
		assertFalse(Comp.in(tech_massDriver2, available));
		assertTrue(techsDiscovered.contains(tech_fusionBeam4));
		assertTrue(techsDiscovered.contains(tech_massDriver2));
		assertEquals(4100, player1.getFunds());
		assertEquals(p2.getName(), planet1b.getOwnerName());
		assertEquals(p1.getName(), planet1b.getOriginName());
		assertEquals(p1.getName(), planet2b.getOwnerName());
		assertEquals(p2.getName(), planet2b.getOriginName());

		p1.addWeapon(tech_girator);
		p2.addWeapon(tech_shieldDrain);

		offered = new PlayerAssets(tech_shieldDrain, 500, h1b.getName());
		wanted = new PlayerAssets(tech_girator, 0, h2b.getName());

		p1.diplomaticRequest(p2, new DiplomaticProposal(offered, wanted), null);

		DiplomaticProposal eventProposal = ((DiplomaticRequestEvent) lastEvent).getProposal();
		assertEquals(tech_shieldDrain,((DiplomaticRequestEvent) lastEvent).getProposal().getOffered().getTechnologies().getAllAsArray()[0]);
		assertEquals(tech_girator,((DiplomaticRequestEvent) lastEvent).getProposal().getWanted().getTechnologies().getAllAsArray()[0]);
		assertEquals(500, ((DiplomaticRequestEvent) lastEvent).getProposal().getOffered().getMoney());

		serverProxy.acceptDiplomaticRequest(player2, ((DiplomaticRequestEvent) lastEvent).getProposal(), null,callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertTrue(p1.hasDiscovered(tech_shieldDrain));
		assertTrue(p1.hasDiscovered(tech_girator));
		assertTrue(p2.hasDiscovered(tech_shieldDrain));
		assertTrue(p2.hasDiscovered(tech_girator));

		available = DeligateCacheManager.getTechsAvailableForResearch();
		techsDiscovered = player1.getTechnologies();

		assertFalse(Comp.in(tech_shieldDrain, available));
		assertFalse(Comp.in(tech_girator, available));
		assertTrue(techsDiscovered.contains(tech_shieldDrain));
		assertTrue(techsDiscovered.contains(tech_girator));
		assertEquals(4600, player1.getFunds());

		assertEquals(p1.getName(), planet1b.getOwnerName());
		assertEquals(p1.getName(), planet1b.getOriginName());
		assertEquals(p2.getName(), planet2b.getOwnerName());
		assertEquals(p2.getName(), planet2b.getOriginName());
		
		//
		// Test Planets updates with new techs
		//
		
		p1.addTechnology(tech_energy10);
		p1.addTechnology(tech_armour200);
		p1.addTechnology(tech_auto_repair_unit);
		
		p2.addTechnology(tech_energy10);
		p2.addTechnology(tech_energy20);

		h1.setShields(new Shields(tech_energy10));
		
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		offered = new PlayerAssets(tech_armour200, tech_auto_repair_unit);
		wanted = new PlayerAssets(tech_energy20);
		
		serverProxy.sendDiplomaticRequest(player2, new DiplomaticProposal(offered, wanted), null, callbackSync, null);
		serverProxy._processCommandResponse();
		callbackSync.waitTillComplete();
		serverProxy._processQueuedEvents();

		assertEquals("accepted", ((DiplomaticResponseEvent) lastEvent).getResponseVerb());
		assertEquals(tech_armour200, tech_auto_repair_unit,((DiplomaticResponseEvent) lastEvent).getProposal().getOffered().getTechnologies().getAllAsArray());
		assertEquals(tech_energy20,((DiplomaticResponseEvent) lastEvent).getProposal().getWanted().getTechnologies().getAllAsArray());

		assertTrue(player1.hasDiscovered(tech_energy20));
		assertEquals(tech_energy20, planet1a.getShields().getShields());

	}

	/**
	 * Tests commands on demanding assets - Tests that user players may demand
	 * assets from computer players - Tests that computer players may demand assets
	 * from user players
	 */
	public void testDemandAssets() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testDemandAssets() *************************");
		createMap();
		owner = p1;

		p2.addWeapon(tech_massDriver2);
		p2.setFunds(1000);
		p1.setFunds(1000);

		MotEngine.instance.getRelations().getRelationshipNotNull(p1, p2).setValue(Relations.WOW);

		Planet h1b = newPlanet("SS1-B", ss1, p2);
		h1b.setSize(20);
		h1b.setTeraforming(0.7);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		PlayerAssets wanted = new PlayerAssets(tech_massDriver2, 10, null);

		serverProxy.sendDiplomaticRequest(player2, DiplomaticProposal.demand(wanted), null, callbackSync, null);
		// serverProxy.demandAssets(player2, wanted, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("accepted", ((DiplomaticResponseEvent) lastEvent).getResponseVerb());
		assertEquals(tech_massDriver2,((DiplomaticResponseEvent) lastEvent).getProposal().getWanted().getTechnologies().getAllAsArray()[0]);
		assertEquals(10, ((DiplomaticResponseEvent) lastEvent).getProposal().getWanted().getMoney());
		assertTrue(p1.hasDiscovered(tech_massDriver2));
		assertEquals(1010, p1.getFunds());
		assertEquals(990, p2.getFunds());

		Technology[] available = DeligateCacheManager.getTechsAvailableForResearch();
		TechnologyList techsDiscovered = player1.getTechnologies();

		assertFalse(Comp.in(tech_massDriver2, available));
		assertTrue(techsDiscovered.contains(tech_massDriver2));
		assertEquals(1010, player1.getFunds());

		p1.addWeapon(tech_girator);
		wanted = new PlayerAssets(tech_girator, 10, null);
		p1.diplomaticRequest(p2, DiplomaticProposal.demand(wanted), null);
		// p1.demandAssets(p2, wanted);

		DiplomaticProposal eventProposal = ((DiplomaticRequestEvent) lastEvent).getProposal();
		assertEquals(tech_girator, eventProposal.getWanted().getTechnologies().getAllAsArray()[0]);
		assertEquals(10, eventProposal.getWanted().getMoney());

		serverProxy.acceptDiplomaticRequest(player2, eventProposal, null, callbackSync, null);
		// serverProxy.acceptDemandAssets(player2, wanted, callbackSync, null);
		serverProxy._processCommandResponse();

		assertTrue(p1.hasDiscovered(tech_girator));
		assertTrue(p2.hasDiscovered(tech_girator));
		assertEquals(1000, player1.getFunds());

	}

	/**
	 * Test the Technologies Available For Research is correct.
	 */
	public void testTechnologiesAvailableForResearch() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testTechnologiesAvailableForResearch() *************************");
		createMap(new SetupOption("qty-missed-techs", "0.0", 0));

		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();
		callbackSync.waitTillComplete();

		Technology[] available = DeligateCacheManager.getTechsAvailableForResearch();

		assertEquals(12, available.length);
		assertEquals(tech_massDriver2, available[0]);
		assertEquals(tech_fusionBeam4, available[1]);
		assertEquals(tech_fusion25x, available[2]);
		assertEquals(tech_armour25, available[3]);
		assertEquals(tech_troopModule20, available[4]);
		assertEquals(tech_electronic25, available[5]);
		assertEquals(tech_flyer100, available[6]);
		assertEquals(tech_energy10, available[7]);
		assertEquals(tech_computerisedFactoryII, available[8]);
		assertEquals(tech_ComputerisedLabI, available[9]);
		assertEquals(tech_mineField, available[10]);
		assertEquals(tech_trade_route, available[11]);

		//
		// Test discovering a technology through research
		//

		serverProxy.setResearchProjectCmd(tech_mineField, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_mineField, player1.getResearchProject().getTechnology());

		clearEvents();
		p1.addResearch(tech_mineField.getCostToResearch() + 1);
		serverProxy._processQueuedEvents();

		assertEquals(tech_mineField, ((TechnologyDiscoveredEvent) eventList.get(0)).getTechnology());
		assertEquals(NoResearchProjectEvent.class, eventList.get(1).getClass());

		assertTrue(player1.getTechnologies().contains(tech_mineField));
		assertNull(player1.getResearchProject());

		available = DeligateCacheManager.getTechsAvailableForResearch();
		assertEquals(11, available.length);
		assertEquals(tech_massDriver2, available[0]);
		assertEquals(tech_fusionBeam4, available[1]);
		assertEquals(tech_fusion25x, available[2]);
		assertEquals(tech_armour25, available[3]);
		assertEquals(tech_troopModule20, available[4]);
		assertEquals(tech_electronic25, available[5]);
		assertEquals(tech_flyer100, available[6]);
		assertEquals(tech_energy10, available[7]);
		assertEquals(tech_computerisedFactoryII, available[8]);
		assertEquals(tech_ComputerisedLabI, available[9]);
		assertEquals(tech_trade_route, available[10]);

		//
		// Test being given a technology
		//

		serverProxy.setResearchProjectCmd(tech_ComputerisedLabI, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_ComputerisedLabI, player1.getResearchProject().getTechnology());
		assertEquals(0.0, player1.getResearchProject().getFromSalvageProbability());

		p2.addTechnology(tech_ComputerisedLabI);
		clearEvents();
		p1.diplomaticRequest(p2, DiplomaticProposal.gift(tech_ComputerisedLabI), null);
		// p2.giveTo(p1, new PlayerAssets(tech_ComputerisedLabI));
		serverProxy._processQueuedEvents();

		DiplomaticProposal eventProposal = ((DiplomaticRequestEvent) eventList.get(0)).getProposal();
		assertEquals(tech_ComputerisedLabI, eventProposal.getOffered().getTechnologList().getTechnologyAtIndex(0));

		assertTrue(player1.getTechnologies().contains(tech_ComputerisedLabI));
		assertNull(player1.getResearchProject());

		available = DeligateCacheManager.getTechsAvailableForResearch();
		assertEquals(10, available.length);
		assertEquals(tech_massDriver2, available[0]);
		assertEquals(tech_fusionBeam4, available[1]);
		assertEquals(tech_fusion25x, available[2]);
		assertEquals(tech_armour25, available[3]);
		assertEquals(tech_troopModule20, available[4]);
		assertEquals(tech_electronic25, available[5]);
		assertEquals(tech_flyer100, available[6]);
		assertEquals(tech_energy10, available[7]);
		assertEquals(tech_computerisedFactoryII, available[8]);
		assertEquals(tech_trade_route, available[9]);

		//
		// Test swapping technology
		//

		serverProxy.setResearchProjectCmd(tech_fusion25x, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_fusion25x, player1.getResearchProject().getTechnology());

		p1.addTechnology(tech_fusionBeam4);
		p2.addTechnology(tech_fusion25x);
		MotEngine.instance.getRelations().getRelationshipNotNull(p1, p2).setValue(Relations.WOW);

		PlayerAssets offered = new PlayerAssets(tech_fusionBeam4);
		PlayerAssets wanted = new PlayerAssets(tech_fusion25x);

		clearEvents();
		serverProxy.sendDiplomaticRequest(player2, new DiplomaticProposal(offered, wanted), null, callbackSync, null);
		// serverProxy.requestSwapAssets(player2, offered, wanted, callbackSync, null);
		serverProxy._processCommandResponse();

		eventProposal = ((DiplomaticResponseEvent) eventList.get(0)).getProposal();
		assertEquals(offered, eventProposal.getOffered());
		assertEquals(wanted, eventProposal.getWanted());
		assertEquals("accepted", ((DiplomaticResponseEvent) eventList.get(0)).getResponseVerb());

		assertTrue(player1.getTechnologies().contains(tech_fusion25x));
		assertNull(player1.getResearchProject());

		available = DeligateCacheManager.getTechsAvailableForResearch();
		assertEquals(8, available.length);
		assertEquals(tech_massDriver2, available[0]);
		assertEquals(tech_armour25, available[1]);
		assertEquals(tech_troopModule20, available[2]);
		assertEquals(tech_electronic25, available[3]);
		assertEquals(tech_flyer100, available[4]);
		assertEquals(tech_energy10, available[5]);
		assertEquals(tech_computerisedFactoryII, available[6]);
		assertEquals(tech_trade_route, available[7]);

		//
		// Test salvage technology
		//

		p1.addSalvage(tech_energy20);
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		assertEquals(tech_energy20, player1.getSalvage());

		serverProxy.setResearchProjectCmd(tech_energy20, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_energy20, player1.getResearchProject().getTechnology());
		assertEquals(0.006, player1.getResearchProject().getFromSalvageProbability(), 0.001);
		assertNull(player1.getSalvage().getAllAsArray());

		//
		// Test discover Humanity
		//
		
		serverProxy.setResearchProjectCmd(tech_trade_route, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(tech_trade_route, player1.getResearchProject().getTechnology());

		clearEvents();
		p1.addResearch(tech_trade_route.getCostToResearch() + 1);
		serverProxy._processQueuedEvents();

		assertEquals(tech_trade_route, ((TechnologyDiscoveredEvent) eventList.get(0)).getTechnology());
		//assertEquals(NoResearchProjectEvent.class, eventList.get(1).getClass());

		assertTrue(player1.getTechnologies().contains(tech_trade_route));
		assertNull(player1.getResearchProject());

		available = DeligateCacheManager.getTechsAvailableForResearch();
		assertEquals(7, available.length);
		assertEquals(tech_massDriver2, available[0]);
		assertEquals(tech_armour25, available[1]);
		assertEquals(tech_troopModule20, available[2]);
		assertEquals(tech_electronic25, available[3]);
		assertEquals(tech_flyer100, available[4]);
		assertEquals(tech_energy10, available[5]);
		assertEquals(tech_computerisedFactoryII, available[6]);


	}

	/**
	 * Tests delegates return correct information about sector ownership and trade
	 * imbalance
	 */
	public void testInfluence() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testInfluence() *************************");
		createMap();
		h1.setInfluence(10);
		h2.setInfluence(101);
		MotEngine.instance.getInfluence().update();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		SectorOwnerGrid sog = DeligateCacheManager.getSectorOwnerGrid();
		assertEquals(null, sog.getSectorOwner(0, 0));
		assertEquals(player2.getName(), sog.getSectorOwner(0, 1).getPlayer());
		assertEquals(player2.getName(), sog.getSectorOwner(0, 2).getPlayer());
		assertEquals(player2.getName(), sog.getSectorOwner(0, 3).getPlayer());
		assertEquals(null, sog.getSectorOwner(0, 4));
		assertEquals(null, sog.getSectorOwner(1, 0));
		assertEquals(player1.getName(), sog.getSectorOwner(1, 1).getPlayer());
		assertEquals(player2.getName(), sog.getSectorOwner(1, 2).getPlayer());
		assertEquals(player2.getName(), sog.getSectorOwner(1, 3).getPlayer());
		assertEquals(null, sog.getSectorOwner(0, 4));

		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");

		assertEquals(false, pd1.isDominated());

		SolarSystem ssx = newSolarSystem("SSx", 50, 50);
		Planet hx = newPlanet("SSx-A", ssx, p2);
		hx.setInfluence(200);
		MotEngine.instance.getInfluence().update();
		serverProxy.updateDelegates(callbackSync);
		serverProxy._processCommandResponse();

		PlanetDelegate pdx = DeligateCacheManager.getPlanet("SSx-A");

		assertEquals(true, pd1.isDominated());
		assertEquals(false, pdx.isDominated());

		assertEquals(9, pd1.getProductionOutput());
		assertEquals(3, pd1.getLostProductionOutput());
		assertEquals(7, pd1.getResearchOutput());
		assertEquals(3, pd1.getLostResearchOutput());

	}

	/**
	 * Tests delegates return correct information about sector ownership when the
	 * universe has more columns than rows (galaxy not square!)
	 */
	public void testInfluenceInDifferentGalaxy() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testInfluenceInDifferentGalaxy() *************************");
		createMapWith4by2Sectors();
		h1.setInfluence(10);
		h2.setInfluence(105);
		h4.setInfluence(50);
		h3.setInfluence(0);

		MotEngine.instance.getInfluence().update();
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		SectorOwnerGrid sog = DeligateCacheManager.getSectorOwnerGrid();
		assertEquals(player1.getName(), sog.getSectorOwner(0, 0).getPlayer());
		assertEquals(player2.getName(), sog.getSectorOwner(0, 1).getPlayer());
		assertEquals(player2.getName(), sog.getSectorOwner(0, 2).getPlayer());
		assertEquals(player2.getName(), sog.getSectorOwner(0, 3).getPlayer());
		assertEquals(null, sog.getSectorOwner(1, 0));
		assertEquals(player2.getName(), sog.getSectorOwner(1, 1).getPlayer());
		assertEquals(player2.getName(), sog.getSectorOwner(1, 2).getPlayer());
		assertEquals(player3.getName(), sog.getSectorOwner(1, 3).getPlayer());
	}

	/**
	 * Test the commands assign tasks to transports and that colonists do move from
	 * one colony to another.
	 */
	public void testMoralChoices() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testMoralChoices() *************************");
		createMap();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals(null, player1.getMoralChoices());

		MoralPolicyAdvisorBasic mpa = (MoralPolicyAdvisorBasic) p1.getMoralPolicyAdvisorNotNull();
		mpa.createNewAdvice(p1);
		serverProxy._processQueuedEvents();

		OfferMoralChoiceEvent event = (OfferMoralChoiceEvent) lastEvent;
		assertEquals(TEST_MORAL_CHOICE, event.getName());
		assertEquals(null, player1.getMoralChoices());

		serverProxy.acceptMoralChoiceCmd(TEST_MORAL_CHOICE, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(TEST_MORAL_CHOICE, p1.getMoralChoices().getList().getAllAsArray()[0].getName());
		assertEquals(1, p1.getMoralChoices().getList().getAllAsArray().length);
		assertEquals(1, player1.getMoralChoices().getQuantity());
		assertEquals(TEST_MORAL_CHOICE, player1.getMoralChoices().getAllNames()[0]);

		try {
			serverProxy.acceptMoralChoiceCmd("BAD_CHOICE", callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("This should not WORK");
		} catch (ValidationError ex) {
		}
	}

	/**
	 * Tests calls to the delegates used for saving and loading game files
	 */
	public void testSaveLoad() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testSaveLoad() *************************");
		createMap();

		// SavedGameFile [] files = (SavedGameFile[]) new
		// FetchSavedGameFilesCmdHandler().process(null, p1);
		// assertEquals(2, files.length);
		// assertEquals("t.mot", files[0]);
		// assertEquals("tt.mot", files[1]);

		serverProxy.getSavedGameFiles(callbackSync, null);
		serverProxy._processCommandResponse();
		SavedGameFile[] files = ((FileNamesCR) callbackSync.waitTillComplete()).getResult();
		// assertEquals(2, files.length);
		// assertEquals("t.mot", files[0]);
		// assertEquals("tt.mot", files[1]);

	}

	/**
	 * Test that the images of planets are updated on end turn when terraform
	 * changes Test that the images for crafts are updated when they surrender
	 */
	public void testEndTurnFlushedState() throws Exception {
		Logger.logDebug(
				"*******************************  DeligateTest.testEndTurnFlushedState() *************************");
		createMap();

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd2 = DeligateCacheManager.getStellarObject("SS2");
		StellarObjectDelegate ssd3 = DeligateCacheManager.getStellarObject("SS3");

		PlanetDelegate hd1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate hd2 = DeligateCacheManager.getPlanet("SS2-A");
		PlanetDelegate hd3 = DeligateCacheManager.getPlanet("SS3-A");

		PlayerDelegate[] pds = DeligateCacheManager.getPlayers();
		PlayerDelegate pd1 = pds[0];
		PlayerDelegate pd2 = pds[1];

		//
		// Test Images of planets are updated on end turn when terraform changes
		//

		assertEquals("images/planets/planet_100_1.png", hd1.getImageForPlanet());

		h1.setTeraforming(1.11);
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();
		DeligateCacheManager.notifyEndTurn();

		assertEquals("images/planets/planet_110_1.png", hd1.getImageForPlanet());

		//
		// Test Images for crafts are updated when they surrender
		//

		Craft craft1 = newCraft(fighterDesign, p1, ss1);
		Craft craft2 = newCraft(fighterDesign, p2, ss1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		String craft1Name = craft1.getName();
		String craft2Name = craft2.getName();

		MoveableOwnedObjectDelegate[] crafts = DeligateCacheManager.getMoos();
		assertEquals(2, crafts.length);
		assertEquals(craft1Name, crafts[0].getName());
		assertEquals(craft2Name, crafts[1].getName());

		assertEquals("images/crafts/hull_flyer_blue.png", crafts[0].getImage());
		assertEquals("images/crafts/hull_flyer_green.png", crafts[1].getImage());

		craft2.surrender(p1);
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();
		DeligateCacheManager.notifyEndTurn();

		assertEquals("images/crafts/hull_flyer_blue.png", crafts[0].getImage());
		assertEquals("images/crafts/hull_flyer_blue.png", crafts[1].getImage());

		p1.getTransportManager().setTransport(new Transport(p1));

		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertEquals(MotEngine.instance.getDate(), DeligateCacheManager.getDate());
		assertEquals(p1.getFunds(), player1.getFunds());
		assertEquals(p1.getNetIncome(), player1.getNetIncome());
		assertEquals(1, player1.getQtyFreeTransport());

	}

	/**
	 * Test that the images of planets are updated on end turn when terraform
	 * changes Test that the images for crafts are updated when they surrender
	 */
	public void testMiningColony() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testMiningColony() *************************");
		createMap();

		SolarSystem miningSS = newSolarSystem("SSM", 50, 100);
		Planet miningColony = newPlanet("SSM-A", miningSS, null, 60, 0.2);
		miningColony.setResource(new ResourceBoralium());
		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		StellarObjectDelegate[] sos = DeligateCacheManager.getStellarObjects();

		SolarSystemDelegate solarSystem1 = (SolarSystemDelegate) Comp.getNameStartingWith("SS1", sos);
		SolarSystemDelegate solarSystemMining = (SolarSystemDelegate) Comp.getNameStartingWith("SSM", sos);
		PlanetDelegate planet1 = solarSystem1.getPlanets()[0];
		PlanetDelegate planetMining = solarSystemMining.getPlanets()[0];

		assertEquals(player1.getName(), solarSystem1.getOwners()[0]);
		assertNull(solarSystemMining.getOwners());
		assertEquals(player1, planet1.getOwner());
		assertNull(planetMining.getOwner());

		//
		// Build a Space Station
		//

		serverProxy.setProjectCmd(planet1, new SpaceStationDesign(tech_spaceStation), 0, false, false, callbackSync,
				null);
		serverProxy._processCommandResponse();

		assertEquals("SpaceStation", h1.getProject().getProjectDesign().getName());
		assertEquals(false, ((ProjectSpaceStation) h1.getProject()).isIncludeTransport());

		clearEvents();
		h1.addProduction(h1.getProject().getCostToBuildOnPlanet());

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertEquals(true, ((ProjectSpaceStation) h1.getProject()).isIncludeTransport());
		assertEquals(
				"Project SpaceStation has been upgraded to include transport on SS1-A because there are no free transports",
				((MessageEvent) getEvent(MessageEvent.class)).getMessage());

		h1.addProduction(h1.getProject().getCostToBuildOnPlanet());

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] moos = DeligateCacheManager.getMoos();

		TransportDelegate transport = (TransportDelegate) moos[0];

		assertEquals("SS1", transport.getOrbit().getStellarObject());
		assertEquals("SS1-A", transport.getOrbit().getPlanet());

		//
		// Move Space Station Transport to SSM
		//

		serverProxy.mooMoveCmd(transport, solarSystemMining, planetMining, false, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertEquals("SSM", transport.getOrbit().getStellarObject());
		assertEquals("SSM-A", transport.getOrbit().getPlanet());

		//
		// Deploy Space Station
		//

		serverProxy.mooDeploy(transport, solarSystemMining, planetMining, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals(null, h1.getProject());
		assertEquals(true, planet1.hasMiningColony());
		assertEquals("SSM-A", planet1.getMiningColonyDelegate().getName());

		//
		// Check Mining Station does not request a Project
		//

		clearEvents();
		serverProxy.setPlanetWantsNoProjectCmd(planet1, true, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy.setResearchProjectCmd(tech_colonyModule7, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertEquals(null, getEvent(NoProjectEvent.class));

		serverProxy.setPlanetWantsNoProjectCmd(planet1, false, callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertEquals("SS1-A", ((NoProjectEvent) getEvent(NoProjectEvent.class)).getPlanet());

		//
		// Test Can't set project on mining colony
		//

	}

	/**
	 * Test that the images of planets are updated on end turn when terraform
	 * changes Test that the images for crafts are updated when they surrender
	 */
	public void testComputerPlayerDestroyed() throws Exception {
		Logger.logDebug("*******************************  DeligateTest.testComputerPlayerDestroyed() *************************");
		createMapWith3Players();

		p3.destroy(true);
		serverProxy._processQueuedEvents();

		NamedObjectDelegate[] pa = order(DeligateCacheManager.getPlayers(), new PlayerDelegate[2], "Humans", "Bears");
		// assertEquals("Ghosts", DeligateCacheManager.getDeadPlayers());
		assertEquals(p3.getName(), DeligateCacheManager.getPlayer("Ghosts").getName());
		assertEquals(p3.getName(), DeligateCacheManager.getPlayer("Ghosts").getPlayerCharacter().getName());

		DeligateCacheManager.flush();
		MotEngine old = MotEngine.instance;

		serverProxy.save("test.mot", null);
		serverProxy._processCommandResponse();

		serverProxy.load("test.mot", "Humans", callbackSync, null);
		serverProxy._processCommandResponse();

		assertNotSame(old, MotEngine.instance);
		assertTrue(p3 != MotEngine.instance.getPlayers().getPlayer("Ghosts"));

		pa = order(DeligateCacheManager.getPlayers(), new PlayerDelegate[2], "Humans", "Bears");
		// assertEquals("Ghosts", DeligateCacheManager.getDeadPlayers());
		assertEquals(p3.getName(), DeligateCacheManager.getPlayer("Ghosts").getName());
		assertEquals(p3.getName(), DeligateCacheManager.getPlayer("Ghosts").getPlayerCharacter().getName());

	}

	/**
	 * Test that from a work hole a player owns a craft can move to any location in
	 * one turn.
	 */
	public void testWormHole() throws Exception {
		Logger.logDebug("*******************************  CraftTest.testWormHole() *************************");
		createMap();

		BlackHole blackHole = BlackHoleFactory.createWormHole(new Location(100, 150));
		ss5 = newSolarSystem("SS5", 347, 100);

		Craft craft1 = newCraft(fighterDesign, p1, ss1);
		Craft craft2 = newCraft(fighterDesign, p1, blackHole);
		String craft1Name = craft1.getName();
		String craft2Name = craft2.getName();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate[] crafts = DeligateCacheManager.getMoos();
		assertEquals(2, crafts.length);
		assertEquals(craft1Name, crafts[0].getName());
		assertEquals(craft2Name, crafts[1].getName());
		assertEquals(150, crafts[0].getRange());
		assertEquals(150, crafts[1].getRange());

		StellarObjectDelegate ssd1 = DeligateCacheManager.getStellarObject("SS1");
		StellarObjectDelegate ssd5 = DeligateCacheManager.getStellarObject("SS5");
		BlackHoleDelegate whd = (BlackHoleDelegate) DeligateCacheManager.getStellarObject("Worm Hole1");
		PlanetDelegate pd1 = DeligateCacheManager.getPlanet("SS1-A");

		//
		// Move craft1 to SS5
		// Test moving a craft to a stellar object out of range
		//

		try {
			serverProxy.mooMoveCmd(crafts[0], ssd5, null, false, callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Out of range");
		} catch (ValidationError e1) {
		}

		try {
			serverProxy.mooMoveCmd(crafts[1], ssd5, null, false, callbackSync, testErrorHandler);
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Out of range");
		} catch (ValidationError e2) {
		}

		//
		// Deploy space station at worm hole
		//

		p1.getTransportManager().setTransport(new Transport(p1));
		new ProjectOutpost(new OutpostDesign(tech_outpost), null).build(h1, true);
		Transport transport = p1.getTransportManager().getTransportsAsArray()[0];
		transport.transferTo(blackHole.getOrbit());
		p1.addTechnology(tech_extendedSpaceJump);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		MoveableOwnedObjectDelegate td = DeligateCacheManager.getMoo(transport.getName());

		serverProxy.mooDeploy(td, whd, null, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Humans", whd.getOwners());
		assertEquals("Humans", whd.getStellarObjectOrbital().getOwnerName());
		assertEquals(tech_outpost, whd.getStellarObjectOrbital().getOrbital().getHullTech());
		assertEquals(0.0f, whd.getResearchBonus());
		assertTrue(whd.hasSpaceJumpAbility());

		serverProxy.mooMoveCmd(crafts[0], ssd5, null, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("SS1", crafts[0].getOrbit().getStellarObject());
		assertEquals("SS5", crafts[0].getNavigator().getDestination().getStellarObject());
		assertEquals(10, crafts[0].getNavigator().getEta());

		serverProxy.mooMoveCmd(crafts[1], ssd5, null, false, callbackSync, null);
		serverProxy._processCommandResponse();

		assertEquals("Worm Hole1", crafts[1].getOrbit().getStellarObject());
		assertEquals("SS5", crafts[1].getNavigator().getDestination().getStellarObject());
		assertEquals(1, crafts[1].getNavigator().getEta());

	}

	/**
	 * Test that from a work hole a player owns a craft can move to any location in
	 * one turn.
	 */
	public void testTradeRoutes() throws Exception {
		Logger.logDebug("*******************************  CraftTest.testTradeRoutes() *************************");
		createMap();

		SolarSystem ss5 = newSolarSystem("SS5", 100, 200);
		Planet h5 = newPlanet("SS5-A", ss5, p1, 110, 1.2, 110, 50);

		SolarSystem ss6 = newSolarSystem("SS6", 100, 300);
		Planet h6 = newPlanet("SS6-A", ss6, p1);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		TradeRouteDelegate[] tradeRoutes = DeligateCacheManager.getTradeRoutes();

		assertNull(tradeRoutes);

		PlanetDelegate planet1 = DeligateCacheManager.getPlanet("SS1-A");
		PlanetDelegate planet5 = DeligateCacheManager.getPlanet("SS5-A");
		PlanetDelegate planet6 = DeligateCacheManager.getPlanet("SS6-A");

		assertEquals(8, planet1.getMoneyOutput());
		assertEquals(4, planet1.getMoneyWasted());
		assertEquals(12, planet1.getProductionOutput());
		assertEquals(10, planet1.getResearchOutput());
		assertEquals(26, planet5.getMoneyOutput());
		assertEquals(40, planet5.getProductionOutput());
		assertEquals(27, planet5.getResearchOutput());
		assertEquals(8, planet6.getMoneyOutput());
		assertEquals(12, planet6.getProductionOutput());
		assertEquals(10, planet6.getResearchOutput());

		TradeRouteFactory.create(ss1, ss5);
		TradeRouteFactory.create(ss5, ss6);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		assertEquals(9, planet1.getMoneyOutput());
		assertEquals(13, planet1.getProductionOutput());
		assertEquals(10, planet1.getResearchOutput());
		assertEquals(29, planet5.getMoneyOutput());
		assertEquals(44, planet5.getProductionOutput());
		assertEquals(30, planet5.getResearchOutput());
		assertEquals(9, planet6.getMoneyOutput());
		assertEquals(13, planet6.getProductionOutput());
		assertEquals(10, planet6.getResearchOutput());

		TradeRouteDelegate[] trl = (TradeRouteDelegate[]) order(DeligateCacheManager.getTradeRoutes(), new TradeRouteDelegate[2], "SS1SS5", "SS5SS6");

		assertEquals(0.5, trl[0].getIndustryOutput("SS1-A") * planet1.getPercentProduction(), 0.1);
		assertEquals(0.5, trl[0].getIndustryOutput("SS1-A") * planet1.getPercentResearch(), 0.1);
		assertEquals(0.0, trl[0].getIndustryOutput("SS1-A") * planet1.getPercentMoney(), 0.1);
		assertEquals(2.0, trl[0].getIndustryOutput("SS5-A") * planet1.getPercentProduction(), 0.1);
		assertEquals(2.0, trl[0].getIndustryOutput("SS5-A") * planet1.getPercentResearch(), 0.1);
		assertEquals(0.0, trl[0].getIndustryOutput("SS5-A") * planet1.getPercentMoney(), 0.1);

		assertEquals(2.0, trl[1].getIndustryOutput("SS5-A") * planet1.getPercentProduction(), 0.1);
		assertEquals(2.0, trl[1].getIndustryOutput("SS5-A") * planet1.getPercentResearch(), 0.1);
		assertEquals(0.0, trl[1].getIndustryOutput("SS5-A") * planet1.getPercentMoney(), 0.1);
		assertEquals(0.5, trl[1].getIndustryOutput("SS6-A") * planet1.getPercentProduction(), 0.1);
		assertEquals(0.5, trl[1].getIndustryOutput("SS6-A") * planet1.getPercentResearch(), 0.1);
		assertEquals(0.0, trl[1].getIndustryOutput("SS6-A") * planet1.getPercentMoney(), 0.1);
		
		serverProxy.abandonCmd(planet6, callbackSync, null);
		serverProxy._processCommandResponse();

		trl = (TradeRouteDelegate[]) order(DeligateCacheManager.getTradeRoutes(), new TradeRouteDelegate[1], "SS1SS5");
		
	}

	/**
	 * Test
	 */
	public void testGraph() throws Exception {
		Logger.logDebug("*******************************  CraftTest.testGraph() *************************");
		createMapWith3Players();

		Graph graphPlayer1 = new Graph();
		p1.setGraph(graphPlayer1);
		graphPlayer1.add("Population", 3000, 10);
		graphPlayer1.add("Population", 3010, 20);
		graphPlayer1.add("Research", 3000, 40);
		graphPlayer1.add("Research", 3010, 50);

		Graph graphPlayer2 = new Graph();
		p2.setGraph(graphPlayer2);
		graphPlayer2.add("Population", 3000, 100);
		graphPlayer2.add("Population", 3010, 200);
		graphPlayer2.add("Research", 3000, 400);
		graphPlayer2.add("Research", 3010, 500);

		Graph graphPlayer3 = new Graph();
		p3.setGraph(graphPlayer3);
		graphPlayer3.add("Population", 3000, 1);
		graphPlayer3.add("Research", 3000, 4);

		MotEngine.instance.setDate(3020);

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		((StratagistBasic) p3.getStratagist()).surrenderToAtEndTurn = p2;
		serverProxy.endTurnCmd(callbackSync, null);
		serverProxy._processCommandResponse();
		serverProxy._processQueuedEvents();

		assertEquals(2, DeligateCacheManager.getPlayers().length);
		assertEquals("Humans", DeligateCacheManager.getPlayers()[0].getName());
		assertEquals("Bears", DeligateCacheManager.getPlayers()[1].getName());

		serverProxy.getGraph(callbackSync, null);
		serverProxy._processCommandResponse();
		GraphState graph = (GraphState) callbackSync.waitTillComplete();

		GraphPlayerState[] graphPlayerState = graph.getGraphPlayerState();
		assertEquals(3, graphPlayerState.length);
		assertEquals("Humans", graphPlayerState[0].getPlayerName());
		assertEquals(2, graphPlayerState[0].getGraph().getGraphLines().length);
		assertEquals("Population", graphPlayerState[0].getGraph().getGraphLines()[0].getName());
		assertEquals(2, graphPlayerState[0].getGraph().getGraphLines()[0].getStatsData().length);
		assertEquals(3000, graphPlayerState[0].getGraph().getGraphLines()[0].getStatsData()[0].getDate());
		assertEquals(10, graphPlayerState[0].getGraph().getGraphLines()[0].getStatsData()[0].getValue());
		assertEquals("Bears", graphPlayerState[1].getPlayerName());
		assertEquals("Ghosts", graphPlayerState[2].getPlayerName());

		assertEquals("Humans", DeligateCacheManager.getPlayer(graphPlayerState[0].getPlayerName()).getName());
		assertEquals("Bears", DeligateCacheManager.getPlayer(graphPlayerState[1].getPlayerName()).getName());
		assertEquals("Ghosts", DeligateCacheManager.getPlayer(graphPlayerState[2].getPlayerName()).getName());

		PlayerDelegate adp = DeligateCacheManager.getPlayer(graphPlayerState[2].getPlayerName());
		assertEquals("red", adp.getColor().getName());
	}

	/**
	 * Test crafts can sleep and end turn button warns if no all crafts have been moved.
	 */
	public void testSleep() throws Exception {
		Logger.logDebug("*******************************  CraftTest.testSleep() *************************");
		createMap();

		Craft craft1 = newCraft(fighterDesign, p1, ss1);
		Fleet fleet1 = newFleet(p1, h2, fighterDesign, 2);
		Craft craft2 = fleet1.getCrafts()[0];
		Craft craft3 = fleet1.getCrafts()[1];
		String craft1Name = craft1.getName();
		String fleet1Name = fleet1.getName();
		String craft2Name = craft2.getName();
		String craft3Name = craft3.getName();

		serverProxy.updateDelegates(null);
		serverProxy._processCommandResponse();

		CraftDelegate cd1 = DeligateCacheManager.getCraft(craft1Name);
		FleetDelegate fd1 = (FleetDelegate) DeligateCacheManager.getMoo(fleet1Name);
		CraftDelegate cd2 = DeligateCacheManager.getCraft(craft2Name);
		CraftDelegate cd3 = DeligateCacheManager.getCraft(craft3Name);
		
		serverProxy.mooSleep(cd1, callbackSync, testErrorHandler);
		serverProxy._processCommandResponse();

		assertTrue(craft1.isSleeping());
		assertFalse(fleet1.isSleeping());
		assertFalse(craft2.isSleeping());
		assertFalse(craft3.isSleeping());
		
		assertTrue(cd1.isSleeping());
		assertFalse(fd1.isSleeping());
		assertFalse(cd2.isSleeping());
		assertFalse(cd3.isSleeping());
		
		assertEquals(3000, DeligateCacheManager.getDate());

		EndTurnButton endTurnButton = new EndTurnButton();
		Settings.getInstance().setMoveCraftWarning(true);
		
		try {
			endTurnButton._testDoAction();
			serverProxy._processCommandResponse();
			testErrorHandler._processCommandResponse();
			fail("Have Craft to Move");
		} catch (ValidationError e1) {	}
		
		//
		// Pressing the end turn button again allows us to end the turn with un-moved crafts
		//
		
		endTurnButton._testDoAction();
		serverProxy._processCommandResponse();

		assertTrue(craft1.isSleeping());
		assertFalse(fleet1.isSleeping());
		assertFalse(craft2.isSleeping());
		assertFalse(craft3.isSleeping());
		
		assertTrue(cd1.isSleeping());
		assertFalse(fd1.isSleeping());
		assertFalse(cd2.isSleeping());
		assertFalse(cd3.isSleeping());
		
		assertEquals(3001, DeligateCacheManager.getDate());
		
		serverProxy.mooSleep(fd1, callbackSync, testErrorHandler);
		serverProxy._processCommandResponse();

		assertTrue(craft1.isSleeping());
		assertTrue(fleet1.isSleeping());
		assertTrue(craft2.isSleeping());
		assertTrue(craft3.isSleeping());
		
		assertTrue(cd1.isSleeping());
		assertTrue(fd1.isSleeping());
		assertTrue(cd2.isSleeping());
		assertTrue(cd3.isSleeping());
		
		//
		// Pressing the end turn button now works first time because all crafts are sleeping
		//

		endTurnButton.setEndTurnInProgress(false);
		endTurnButton._testDoAction();
		serverProxy._processCommandResponse();

		assertTrue(craft1.isSleeping());
		assertTrue(fleet1.isSleeping());
		assertTrue(craft2.isSleeping());
		assertTrue(craft3.isSleeping());
		
		assertTrue(cd1.isSleeping());
		assertTrue(fd1.isSleeping());
		assertTrue(cd2.isSleeping());
		assertTrue(cd3.isSleeping());
		
		assertEquals(3002, DeligateCacheManager.getDate());
		
		serverProxy.mooSleep(null, callbackSync, testErrorHandler);
		serverProxy._processCommandResponse();

		assertFalse(craft1.isSleeping());
		assertFalse(fleet1.isSleeping());
		assertFalse(craft2.isSleeping());
		assertFalse(craft3.isSleeping());
		
		assertFalse(cd1.isSleeping());
		assertFalse(fd1.isSleeping());
		assertFalse(cd2.isSleeping());
		assertFalse(cd3.isSleeping());

	}


	/*
	 * THESE ARE ALL THE COMMANDS THAT NEED TESTING public void
	 * increaseEcologyCmd(PlayerDeligate player) throws ApplicationError,
	 * ValidationError; public void increaseDevelopmentCmd(PlayerDeligate player)
	 * throws ApplicationError, ValidationError; public void
	 * upgradeEngineCmd(CraftDeligate craft, PlayerDeligate owner) throws
	 * ApplicationError, ValidationError;
	 * 
	 * public void addSpyStellarObject(StellarObjectDeligate so, UserPlayerDeligate
	 * owner) throws ApplicationError, ValidationError;
	 * 
	 * public void newGameUserPlayer(SelectedRaceOptions userOptions, String
	 * alignment) throws ApplicationError, ValidationError; public void
	 * newGame(Double level, int qtyComputerPlayers, int qtyPlanets, Size
	 * galaxySize, SelectedRaceOptions userOptions, String alignment) throws
	 * ApplicationError, ValidationError; public void load(File file) throws
	 * ApplicationError, ValidationError; public void save(File file) throws
	 * ApplicationError, ValidationError; public void registerProxy(PlayerDeligate
	 * player, ClientStub stub) throws ApplicationError, ValidationError;
	 */

	private MotEvent lastEvent;

	private BattleDelegate battle;

	/**
	 * This process events coming directly from the server.
	 */
	public void notify(MotEventFactory e) throws ApplicationError {
		notify(e.createEvent(p1));
	}

	/**
	 * This process events coming directly from the server.
	 */
	public void notify(MotEvent e) throws ApplicationError {
		Logger.logDebug("DeligateTest.notify() EVENT " + e.toString());
		DeligateCacheManager.update(e);
		lastEvent = e;
		eventList.add(e);
	}

	/**
	 * This process events coming from the server connection.
	 */
	public void process(MotEvent e, LinkedEventBase parent) {
		Logger.logDebug("DeligateTest.process() EVENT " + e.toString());
		lastEvent = e;
		eventList.add(e);
	}

	protected void clearEvents() {
		Logger.logDebug("DeligateTest.notify() Clear Events ");
		eventList.clear();
		lastEvent = null;
	}

	private Player owner;

	public Player getOwner() {
		return owner;
	}

	public static void main(String args[]) {
		Logger.clearContents();
		junit.textui.TestRunner.run(suite());
		if (!ProxyStore.hasInstance()) {
			try {
				ProxyStore.getInstance().disconnect();
			} catch (ApplicationError e) {
				Logger.logError("", e);
			}
		}
	}

}